@@ -71,6 +71,46 @@ class TestBinder(testcase.IDLTestcase):
 
     # pylint: disable=too-many-public-methods
 
+    # Create a text wrap for common types.
+    common_types = textwrap.dedent("""
+    types:
+        object:
+            description: foo
+            cpp_type: foo
+            bson_serialization_type: object
+            serializer: foo
+            deserializer: foo
+
+        string:
+            description: foo
+            cpp_type: foo
+            bson_serialization_type: string
+            serializer: foo
+            deserializer: foo
+
+        any_type:
+            description: foo
+            cpp_type: foo
+            bson_serialization_type: any
+            serializer: foo
+            deserializer: foo
+
+        tenant_id:
+            bson_serialization_type: any
+            description: foo
+            cpp_type: foo
+            deserializer: foo
+            serializer: foo
+
+        database_name:
+            bson_serialization_type: string
+            description: foo
+            cpp_type: foo
+            serializer: foo
+            deserializer: foo
+
+    """)
+
     def test_empty(self):
         # type: () -> None
         """Test an empty document works."""
@@ -536,6 +576,11 @@ class TestBinder(testcase.IDLTestcase):
                 serializer: foo
                 deserializer: foo
                 default: foo
+            int:
+                description: foo
+                cpp_type: std::int32_t
+                bson_serialization_type: int
+                deserializer: mongo::BSONElement::_numberInt
         """)
 
         self.assert_bind(test_preamble + textwrap.dedent("""
@@ -547,6 +592,15 @@ class TestBinder(testcase.IDLTestcase):
                         foo: string
             """))
 
+        self.assert_bind(test_preamble + textwrap.dedent("""
+            structs:
+                foo:
+                    description: foo
+                    strict: true
+                    fields:
+                        foo: array<int>
+            """))
+
     def test_struct_negative(self):
         # type: () -> None
         """Negative struct tests."""
@@ -561,6 +615,11 @@ class TestBinder(testcase.IDLTestcase):
                 serializer: foo
                 deserializer: foo
                 default: foo
+            int:
+                description: foo
+                cpp_type: std::int32_t
+                bson_serialization_type: int
+                deserializer: mongo::BSONElement::_numberInt
         """)
 
         # Test array as name
@@ -574,6 +633,265 @@ class TestBinder(testcase.IDLTestcase):
                         foo: string
             """), idl.errors.ERROR_ID_ARRAY_NOT_VALID_TYPE)
 
+        self.assert_bind(test_preamble + textwrap.dedent("""
+            structs:
+                foo:
+                    description: foo
+                    strict: true
+                    fields:
+                        foo: array<int>
+            """))
+
+    def test_variant_positive(self):
+        # type: () -> None
+        """Positive variant test cases."""
+
+        # Setup some common types
+        test_preamble = textwrap.dedent("""
+        types:
+            string:
+                description: foo
+                cpp_type: foo
+                bson_serialization_type: string
+                serializer: foo
+                deserializer: foo
+                default: foo
+            int:
+                description: foo
+                cpp_type: std::int32_t
+                bson_serialization_type: int
+                deserializer: mongo::BSONElement::_numberInt
+            bindata_function:
+                bson_serialization_type: bindata
+                bindata_subtype: function
+                description: "A BSON bindata of function sub type"
+                cpp_type: "std::vector<std::uint8_t>"
+                deserializer: "mongo::BSONElement::_binDataVector"
+        """)
+
+        self.assert_bind(test_preamble + textwrap.dedent("""
+        structs:
+            foo:
+                description: foo
+                fields:
+                    my_variant_field:
+                        type:
+                            variant:
+                            - string
+                            - int
+            """))
+
+        self.assert_bind(test_preamble + textwrap.dedent("""
+        structs:
+            foo:
+                description: foo
+                fields:
+                    my_variant_field:
+                        type:
+                            variant:
+                            - string
+                            - bindata_function
+            """))
+
+        self.assert_bind(test_preamble + textwrap.dedent("""
+        structs:
+            foo:
+                description: foo
+                fields:
+                    my_variant_field:
+                        type:
+                            variant:
+                            - string
+                            - int
+                        default: 1
+            """))
+
+    def test_variant_negative(self):
+        # type: () -> None
+        """Negative variant test cases."""
+
+        # Setup some common types
+        test_preamble = textwrap.dedent("""
+        enums:
+            foo_enum:
+                description: foo
+                type: int
+                values:
+                    v1: 0
+                    v2: 1
+        types:
+            string:
+                description: foo
+                cpp_type: foo
+                bson_serialization_type: string
+                serializer: foo
+                deserializer: foo
+                default: foo
+            int:
+                description: foo
+                cpp_type: std::int32_t
+                bson_serialization_type: int
+                deserializer: mongo::BSONElement::_numberInt
+            safeInt:
+                bson_serialization_type:
+                - long
+                - int
+                - decimal
+                - double
+                description: foo
+                cpp_type: "std::int32_t"
+                deserializer: "mongo::BSONElement::safeNumberInt"
+        """)
+
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        structs:
+            foo:
+                description: foo
+                fields:
+                    my_variant_field:
+                        type:
+                            variant:
+                            - string
+            """), idl.errors.ERROR_ID_USELESS_VARIANT)
+
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        structs:
+            foo:
+                description: foo
+                fields:
+                    my_variant_field:
+                        type:
+                            variant:
+                            - string
+                            - int
+                            - not_defined
+            """), idl.errors.ERROR_ID_UNKNOWN_TYPE)
+
+        # Enums are banned in variants for now.
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        structs:
+            foo:
+                description: foo
+                fields:
+                    my_variant_field:
+                        type:
+                            variant:
+                            - string
+                            - foo_enum
+            """), idl.errors.ERROR_ID_NO_VARIANT_ENUM)
+
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        structs:
+            foo:
+                description: foo
+                fields:
+                    my_variant_field:
+                        type:
+                            variant:
+                            - string
+                            - string
+            """), idl.errors.ERROR_ID_VARIANT_DUPLICATE_TYPES)
+
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        structs:
+            foo:
+                description: foo
+                fields:
+                    my_variant_field:
+                        type:
+                            variant:
+                            - array<string>
+                            - array<string>
+            """), idl.errors.ERROR_ID_VARIANT_DUPLICATE_TYPES)
+
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        structs:
+            struct0:
+                description: foo
+            struct1:
+                description: foo
+            foo:
+                description: foo
+                fields:
+                    my_variant_field:
+                        type:
+                            variant:
+                            - array<struct0>
+                            - array<struct1>
+            """), idl.errors.ERROR_ID_VARIANT_DUPLICATE_TYPES)
+
+        # At most one array can have BSON serialization type NumberInt.
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        structs:
+            foo:
+                description: foo
+                fields:
+                    my_variant_field:
+                        type:
+                            variant:
+                            - array<int>
+                            - array<safeInt>
+            """), idl.errors.ERROR_ID_VARIANT_DUPLICATE_TYPES)
+
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        structs:
+            one_string:
+                description: foo
+                fields: {value: string}
+            foo:
+                description: foo
+                fields:
+                    my_variant_field:
+                        type:
+                            variant:
+                            - one_string
+                            - one_string
+                            - int
+            """), idl.errors.ERROR_ID_VARIANT_STRUCTS)
+
+        # At most one type can have BSON serialization type Object.
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        structs:
+            one_string:
+                description: foo
+                fields: {value: string}
+            one_int:
+                description: foo
+                fields: {value: int}
+            foo:
+                description: foo
+                fields:
+                    my_variant_field:
+                        type:
+                            variant:
+                            - one_string
+                            - one_int
+                            - int
+            """), idl.errors.ERROR_ID_VARIANT_STRUCTS)
+
+        # At most one type can have BSON serialization type NumberInt.
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        structs:
+            foo:
+                description: foo
+                fields:
+                    my_variant_field:
+                        type:
+                            variant:
+                            - safeInt
+                            - int
+      """), idl.errors.ERROR_ID_VARIANT_DUPLICATE_TYPES)
+
     def test_field_positive(self):
         # type: () -> None
         """Positive test cases for field."""
@@ -651,6 +969,36 @@ class TestBinder(testcase.IDLTestcase):
                         arrayOfString: arrayfake
             """))
 
+        # Test always_serialize with optional
+        self.assert_bind(test_preamble + textwrap.dedent("""
+            structs:
+                foo:
+                    description: foo
+                    strict: true
+                    fields:
+                        foo:
+                            type: string
+                            optional: true
+                            always_serialize: true
+            """))
+
+        # Test field of a struct type with default=true
+        self.assert_bind(test_preamble + textwrap.dedent("""
+            structs:
+                foo:
+                    description: foo
+                    fields:
+                        field1: string
+
+                bar:
+                    description: foo
+                    fields:
+                        field2:
+                            type: foo
+                            default: true
+
+            """))
+
     def test_field_negative(self):
         # type: () -> None
         """Negative field tests."""
@@ -673,7 +1021,7 @@ class TestBinder(testcase.IDLTestcase):
                 bindata_subtype: uuid
         """)
 
-        # Test field of a struct type with a default
+        # Test field of a struct type with a non-true default
         self.assert_bind_fail(
             test_preamble + textwrap.dedent("""
             structs:
@@ -689,7 +1037,7 @@ class TestBinder(testcase.IDLTestcase):
                             type: foo
                             default: foo
 
-            """), idl.errors.ERROR_ID_FIELD_MUST_BE_EMPTY_FOR_STRUCT)
+            """), idl.errors.ERROR_ID_DEFAULT_MUST_BE_TRUE_OR_EMPTY_FOR_STRUCT)
 
         # Test array as field name
         self.assert_bind_fail(
@@ -772,6 +1120,20 @@ class TestBinder(testcase.IDLTestcase):
                             optional: true
             """), idl.errors.ERROR_ID_ILLEGAL_FIELD_DEFAULT_AND_OPTIONAL)
 
+        # Test always_serialize without optional for the same field
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+            structs:
+                foo:
+                    description: foo
+                    strict: true
+                    fields:
+                        foo:
+                            type: string
+                            default: 42
+                            always_serialize: true
+            """), idl.errors.ERROR_ID_ILLEGAL_FIELD_ALWAYS_SERIALIZE_NOT_OPTIONAL)
+
         # Test duplicate comparison order
         self.assert_bind_fail(
             test_preamble + textwrap.dedent("""
@@ -1365,25 +1727,25 @@ class TestBinder(testcase.IDLTestcase):
         """Positive command tests."""
 
         # Setup some common types
-        test_preamble = textwrap.dedent("""
-        types:
-            string:
+        test_preamble = self.common_types + textwrap.dedent("""
+        structs:
+            reply:
                 description: foo
-                cpp_type: foo
-                bson_serialization_type: string
-                serializer: foo
-                deserializer: foo
-                default: foo
+                fields:
+                    foo: string
         """)
 
         self.assert_bind(test_preamble + textwrap.dedent("""
             commands:
                 foo:
                     description: foo
+                    command_name: foo
                     namespace: ignored
+                    api_version: ""
                     strict: true
                     fields:
                         foo1: string
+                    reply_type: reply
             """))
 
     def test_command_negative(self):
@@ -1391,30 +1753,24 @@ class TestBinder(testcase.IDLTestcase):
         """Negative command tests."""
 
         # Setup some common types
-        test_preamble = textwrap.dedent("""
-        types:
-            string:
-                description: foo
-                cpp_type: foo
-                bson_serialization_type: string
-                serializer: foo
-                deserializer: foo
-                default: foo
-        """)
-
+        test_preamble = self.common_types
         # Commands cannot be fields in other commands
         self.assert_bind_fail(
             test_preamble + textwrap.dedent("""
             commands:
                 foo:
                     description: foo
+                    command_name: foo
                     namespace: ignored
+                    api_version: ""
                     fields:
                         foo1: string
 
                 bar:
                     description: foo
+                    command_name: bar
                     namespace: ignored
+                    api_version: ""
                     fields:
                         foo: foo
             """), idl.errors.ERROR_ID_FIELD_NO_COMMAND)
@@ -1425,7 +1781,9 @@ class TestBinder(testcase.IDLTestcase):
             commands:
                 foo:
                     description: foo
+                    command_name: foo
                     namespace: ignored
+                    api_version: ""
                     fields:
                         foo1: string
 
@@ -1442,33 +1800,44 @@ class TestBinder(testcase.IDLTestcase):
             commands:
                 foo:
                     description: foo
+                    command_name: foo
                     namespace: ignored
+                    api_version: ""
                     fields:
                         foo: string
             """), idl.errors.ERROR_ID_COMMAND_DUPLICATES_FIELD)
 
+        # Reply type must be resolvable
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+            commands:
+                foo:
+                    description: foo
+                    command_name: foo
+                    namespace: ignored
+                    api_version: ""
+                    reply_type: not_defined
+            """), idl.errors.ERROR_ID_UNKNOWN_TYPE)
+
+        # Reply type must be a struct
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+            commands:
+                foo:
+                    description: foo
+                    command_name: foo
+                    namespace: ignored
+                    api_version: ""
+                    reply_type: string
+            """), idl.errors.ERROR_ID_INVALID_REPLY_TYPE)
+
     def test_command_doc_sequence_positive(self):
         # type: () -> None
         """Positive supports_doc_sequence tests."""
         # pylint: disable=invalid-name
 
         # Setup some common types
-        test_preamble = textwrap.dedent("""
-        types:
-            object:
-                description: foo
-                cpp_type: foo
-                bson_serialization_type: object
-                serializer: foo
-                deserializer: foo
-
-            string:
-                description: foo
-                cpp_type: foo
-                bson_serialization_type: string
-                serializer: foo
-                deserializer: foo
-
+        test_preamble = self.common_types + textwrap.dedent("""
         structs:
             foo_struct:
                 description: foo
@@ -1481,7 +1850,9 @@ class TestBinder(testcase.IDLTestcase):
             commands:
                 foo:
                     description: foo
+                    command_name: foo
                     namespace: ignored
+                    api_version: ""
                     fields:
                         foo1:
                             type: array<object>
@@ -1492,7 +1863,9 @@ class TestBinder(testcase.IDLTestcase):
             commands:
                 foo:
                     description: foo
+                    command_name: foo
                     namespace: ignored
+                    api_version: ""
                     fields:
                         foo1:
                             type: array<foo_struct>
@@ -1505,29 +1878,7 @@ class TestBinder(testcase.IDLTestcase):
         # pylint: disable=invalid-name
 
         # Setup some common types
-        test_preamble = textwrap.dedent("""
-        types:
-            object:
-                description: foo
-                cpp_type: foo
-                bson_serialization_type: object
-                serializer: foo
-                deserializer: foo
-
-            string:
-                description: foo
-                cpp_type: foo
-                bson_serialization_type: string
-                serializer: foo
-                deserializer: foo
-
-            any_type:
-                description: foo
-                cpp_type: foo
-                bson_serialization_type: any
-                serializer: foo
-                deserializer: foo
-        """)
+        test_preamble = self.common_types
 
         test_preamble2 = test_preamble + textwrap.dedent("""
         structs:
@@ -1556,7 +1907,9 @@ class TestBinder(testcase.IDLTestcase):
             commands:
                 foo:
                     description: foo
+                    command_name: foo
                     namespace: ignored
+                    api_version: ""
                     fields:
                         foo:
                             type: object
@@ -1569,7 +1922,9 @@ class TestBinder(testcase.IDLTestcase):
             commands:
                 foo:
                     description: foo
+                    command_name: foo
                     namespace: ignored
+                    api_version: ""
                     fields:
                         foo1:
                             type: array<string>
@@ -1582,7 +1937,9 @@ class TestBinder(testcase.IDLTestcase):
             commands:
                 foo:
                     description: foo
+                    command_name: foo
                     namespace: ignored
+                    api_version: ""
                     fields:
                         foo1:
                             type: array<string>
@@ -1592,23 +1949,18 @@ class TestBinder(testcase.IDLTestcase):
     def test_command_type_positive(self):
         # type: () -> None
         """Positive command custom type test cases."""
-        test_preamble = textwrap.dedent("""
-        types:
-            string:
-                description: foo
-                cpp_type: foo
-                bson_serialization_type: string
-                serializer: foo
-                deserializer: foo
-        """)
+        # Setup some common types
+        test_preamble = self.common_types
 
         # string
         self.assert_bind(test_preamble + textwrap.dedent("""
         commands:
             foo:
                 description: foo
+                command_name: foo
                 strict: true
                 namespace: type
+                api_version: ""
                 type: string
                 fields:
                     field1: string
@@ -1619,8 +1971,10 @@ class TestBinder(testcase.IDLTestcase):
         commands:
             foo:
                 description: foo
+                command_name: foo
                 strict: true
                 namespace: type
+                api_version: ""
                 type: array<string>
                 fields:
                     field1: string
@@ -1629,15 +1983,8 @@ class TestBinder(testcase.IDLTestcase):
     def test_command_type_negative(self):
         # type: () -> None
         """Negative command type test cases."""
-        test_preamble = textwrap.dedent("""
-        types:
-            string:
-                description: foo
-                cpp_type: foo
-                bson_serialization_type: string
-                serializer: foo
-                deserializer: foo
-        """)
+        # Setup some common types
+        test_preamble = self.common_types
 
         # supports_doc_sequence must be a bool
         self.assert_bind_fail(
@@ -1645,7 +1992,9 @@ class TestBinder(testcase.IDLTestcase):
         commands:
             foo:
                 description: foo
+                command_name: foo
                 namespace: type
+                api_version: ""
                 type: int
                 fields:
                     field1: string
@@ -1657,7 +2006,7 @@ class TestBinder(testcase.IDLTestcase):
 
         # server parameter with storage.
         # Also try valid set_at values.
-        for set_at in ["startup", "runtime", "[ startup, runtime ]"]:
+        for set_at in ["startup", "runtime", "[ startup, runtime ]", "cluster"]:
             self.assert_bind(
                 textwrap.dedent("""
             server_parameters:
@@ -1685,6 +2034,24 @@ class TestBinder(testcase.IDLTestcase):
                     callback: qux
             """))
 
+        # Cluster server parameter with storage.
+        self.assert_bind(
+            textwrap.dedent("""
+        server_parameters:
+            foo:
+                set_at: cluster
+                description: bar
+                cpp_varname: baz
+                cpp_vartype: bazStorage
+                on_update: buzz
+                validator:
+                    gt: 0
+                    gte: 1
+                    lte: 999
+                    lt: 1000
+                    callback: qux
+            """))
+
         # Bound setting with arbitrary expression default and validators.
         self.assert_bind(
             textwrap.dedent("""
@@ -1737,6 +2104,7 @@ class TestBinder(testcase.IDLTestcase):
                     data: bling
                     override_set: true
                     override_ctor: false
+                    override_validate: true
         """))
 
         self.assert_bind(
@@ -1752,6 +2120,21 @@ class TestBinder(testcase.IDLTestcase):
                 deprecated_name: bling
         """))
 
+        self.assert_bind(
+            textwrap.dedent("""
+        server_parameters:
+            foo:
+                set_at: cluster
+                description: bar
+                cpp_class:
+                    name: baz
+                    override_validate: true
+                condition: { expr: "true" }
+                redact: true
+                test_only: true
+                deprecated_name: bling
+        """))
+
         # Default without data.
         self.assert_bind(
             textwrap.dedent("""
@@ -1790,7 +2173,7 @@ class TestBinder(testcase.IDLTestcase):
 
     def test_config_option_positive(self):
         # type: () -> None
-        """Posative config option test cases."""
+        """Positive config option test cases."""
 
         # Every field.
         self.assert_bind(
@@ -1985,6 +2368,334 @@ class TestBinder(testcase.IDLTestcase):
                     source: cli
             """), idl.errors.ERROR_ID_MISSING_SHORT_NAME_WITH_SINGLE_NAME)
 
+    def test_feature_flag(self):
+        # type: () -> None
+        """Test feature flag checks around version."""
+
+        # feature flag can default to false without a version
+        self.assert_bind(
+            textwrap.dedent("""
+            feature_flags:
+                featureFlagToaster:
+                    description: "Make toast"
+                    cpp_varname: gToaster
+                    default: false
+            """))
+
+        # feature flag can default to true with a version
+        self.assert_bind(
+            textwrap.dedent("""
+            feature_flags:
+                featureFlagToaster:
+                    description: "Make toast"
+                    cpp_varname: gToaster
+                    default: true
+                    version: 123
+            """))
+
+        # true is only allowed with a version
+        self.assert_bind_fail(
+            textwrap.dedent("""
+            feature_flags:
+                featureFlagToaster:
+                    description: "Make toast"
+                    cpp_varname: gToaster
+                    default: true
+            """), idl.errors.ERROR_ID_FEATURE_FLAG_DEFAULT_TRUE_MISSING_VERSION)
+
+        # false is not allowed with a version
+        self.assert_bind_fail(
+            textwrap.dedent("""
+            feature_flags:
+                featureFlagToaster:
+                    description: "Make toast"
+                    cpp_varname: gToaster
+                    default: false
+                    version: 123
+            """), idl.errors.ERROR_ID_FEATURE_FLAG_DEFAULT_FALSE_HAS_VERSION)
+
+    def test_access_check(self):
+        # type: () -> None
+        """Test access check."""
+
+        test_preamble = self.common_types + textwrap.dedent("""
+        enums:
+            AccessCheck:
+                description: "test"
+                type: string
+                values:
+                    kIsAuthenticated :  "is_authenticated"
+                    kIsCoAuthorized :  "is_coauthorized"
+
+            ActionType:
+                description: "test"
+                type: string
+                values:
+                    addShard :  "addShard"
+                    serverStatus :  "serverStatus"
+
+            MatchType:
+                description: "test"
+                type: string
+                values:
+                    matchClusterResource :  "cluster"
+
+        structs:
+            reply:
+                description: foo
+                fields:
+                    foo: string
+        """)
+
+        # Test none
+        self.assert_bind(test_preamble + textwrap.dedent("""
+        commands:
+            test1:
+                description: foo
+                command_name: foo
+                api_version: ""
+                namespace: ignored
+                access_check:
+                    none: true
+                fields:
+                    foo: string
+                reply_type: reply
+            """))
+
+        # Test simple with access check
+        self.assert_bind(test_preamble + textwrap.dedent("""
+        commands:
+            test1:
+                description: foo
+                command_name: foo
+                api_version: ""
+                namespace: ignored
+                access_check:
+                    simple:
+                        check: is_authenticated
+                fields:
+                    foo: string
+                reply_type: reply
+            """))
+
+        # Test simple with privilege
+        self.assert_bind(test_preamble + textwrap.dedent("""
+        commands:
+            test1:
+                description: foo
+                command_name: foo
+                api_version: ""
+                namespace: ignored
+                access_check:
+                    simple:
+                        privilege:
+                            resource_pattern: cluster
+                            action_type: addShard
+                fields:
+                    foo: string
+                reply_type: reply
+            """))
+
+        self.assert_parse(
+            textwrap.dedent("""
+        commands:
+            foo:
+                description: foo
+                command_name: foo
+                api_version: 1
+                namespace: ignored
+                access_check:
+                    complex:
+                        - privilege:
+                            resource_pattern: cluster
+                            action_type: addShard
+                        - privilege:
+                            resource_pattern: cluster
+                            action_type: serverStatus
+                        - check: is_authenticated
+                fields:
+                    foo: bar
+                reply_type: foo_reply_struct
+            """))
+
+    def test_access_check_negative(self):
+        # type: () -> None
+        """Negative access check tests."""
+
+        test_preamble = self.common_types + textwrap.dedent("""
+        enums:
+            AccessCheck:
+                description: "test"
+                type: string
+                values:
+                    kIsAuthenticated :  "is_authenticated"
+                    kIsCoAuthorized :  "is_coauthorized"
+
+            ActionType:
+                description: "test"
+                type: string
+                values:
+                    addShard :  "addShard"
+                    serverStatus :  "serverStatus"
+
+            MatchType:
+                description: "test"
+                type: string
+                values:
+                    matchClusterResource :  "cluster"
+        structs:
+            reply:
+                description: foo
+                fields:
+                    foo: string
+        """)
+
+        # Test simple with bad access check
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        commands:
+            test1:
+                description: foo
+                command_name: foo
+                api_version: ""
+                namespace: ignored
+                access_check:
+                    simple:
+                        check: unknown
+                fields:
+                    foo: string
+                reply_type: reply
+            """), idl.errors.ERROR_ID_UNKOWN_ENUM_VALUE)
+
+        # Test simple with bad access check with privilege
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        commands:
+            test1:
+                description: foo
+                command_name: foo
+                api_version: ""
+                namespace: ignored
+                access_check:
+                    simple:
+                        privilege:
+                            resource_pattern: foo
+                            action_type: addShard
+                fields:
+                    foo: string
+                reply_type: reply
+            """), idl.errors.ERROR_ID_UNKOWN_ENUM_VALUE)
+
+        # Test simple with bad access check with privilege
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        commands:
+            test1:
+                description: foo
+                command_name: foo
+                api_version: ""
+                namespace: ignored
+                access_check:
+                    simple:
+                        privilege:
+                            resource_pattern: cluster
+                            action_type: foo
+                fields:
+                    foo: string
+                reply_type: reply
+            """), idl.errors.ERROR_ID_UNKOWN_ENUM_VALUE)
+
+        # Test simple with access check and privileges
+        self.assert_bind(test_preamble + textwrap.dedent("""
+        commands:
+            test1:
+                description: foo
+                command_name: foo
+                api_version: ""
+                namespace: ignored
+                access_check:
+                    simple:
+                        privilege:
+                            resource_pattern: cluster
+                            action_type: [addShard, serverStatus]
+                fields:
+                    foo: string
+                reply_type: reply
+            """))
+
+        # Test simple with privilege with duplicate action_type
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        commands:
+            test1:
+                description: foo
+                command_name: foo
+                api_version: ""
+                namespace: ignored
+                access_check:
+                    simple:
+                        privilege:
+                            resource_pattern: cluster
+                            action_type: [addShard, addShard]
+                fields:
+                    foo: string
+                reply_type: reply
+            """), idl.errors.ERROR_ID_DUPLICATE_ACTION_TYPE)
+
+        # complex with duplicate check
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        commands:
+            test1:
+                description: foo
+                command_name: foo
+                api_version: 1
+                namespace: ignored
+                access_check:
+                    complex:
+                        - check: is_authenticated
+                        - check: is_authenticated
+                fields:
+                    foo: string
+                reply_type: reply
+            """), idl.errors.ERROR_ID_DUPLICATE_ACCESS_CHECK)
+
+        # complex with duplicate priv
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        commands:
+            test1:
+                description: foo
+                command_name: foo
+                api_version: 1
+                namespace: ignored
+                access_check:
+                    complex:
+                        - privilege:
+                            resource_pattern: cluster
+                            action_type: addShard
+                        - privilege:
+                            resource_pattern: cluster
+                            action_type: [addShard, serverStatus]
+                fields:
+                    foo: string
+                reply_type: reply
+            """), idl.errors.ERROR_ID_DUPLICATE_ACCESS_CHECK)
+
+        # api_version != "" but not access_check
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+        commands:
+            test1:
+                description: foo
+                command_name: foo
+                api_version: 1
+                namespace: ignored
+                fields:
+                    foo: string
+                reply_type: reply
+            """), idl.errors.ERROR_ID_MISSING_ACCESS_CHECK)
+
 
 if __name__ == '__main__':
 
