@@ -62,7 +62,7 @@ class Spec(object):
     def is_nightly(self):
         """Return True if nightly."""
         return bool(re.search("-$", self.version())) or bool(
-            re.search(r"\d-\d+-g[0-9a-f]+$", self.version()))
+            re.search(r"-g[0-9a-f]+$", self.version()))
 
     def is_patch(self):
         """Return True if patch."""
@@ -70,7 +70,7 @@ class Spec(object):
 
     def is_rc(self):
         """Return True if rc."""
-        return bool(re.search(r"-rc\d+$", self.version()))
+        return bool(re.search(r"(-rc|-alpha)\d+$", self.version()))
 
     def is_pre_release(self):
         """Return True if pre-release."""
@@ -103,7 +103,10 @@ class Spec(object):
 
     def suffix(self):
         """Return suffix."""
-        return "-org" if int(self.ver.split(".")[1]) % 2 == 0 else "-org-unstable"
+        if int(self.ver.split(".")[0]) >= 5:
+            return "-org" if int(self.ver.split(".")[1]) == 0 else "-org-unstable"
+        else:
+            return "-org" if int(self.ver.split(".")[1]) % 2 == 0 else "-org-unstable"
 
     def prelease(self):
         """Return pre-release verison suffix."""
@@ -299,8 +302,6 @@ class Distro(object):
                 return "bionic"
             elif build_os == 'ubuntu2004':
                 return "focal"
-            elif build_os == 'ubuntu2204':
-                return "jammy"
             else:
                 raise Exception("unsupported build_os: %s" % build_os)
         elif self.dname == 'debian':
@@ -310,6 +311,8 @@ class Distro(object):
                 return 'stretch'
             elif build_os == 'debian10':
                 return 'buster'
+            elif build_os == 'debian11':
+                return 'bullseye'
             else:
                 raise Exception("unsupported build_os: %s" % build_os)
         else:
@@ -348,10 +351,9 @@ class Distro(object):
                 "ubuntu1604",
                 "ubuntu1804",
                 "ubuntu2004",
-                "ubuntu2204",
             ]
         elif self.dname == 'debian':
-            return ["debian81", "debian92", "debian10"]
+            return ["debian81", "debian92", "debian10", "debian11"]
         else:
             raise Exception("BUG: unsupported platform?")
 
@@ -505,10 +507,10 @@ def unpack_binaries_into(build_os, arch, spec, where):
             print("moving file: %s/%s" % (release_dir, releasefile))
             os.rename("%s/%s" % (release_dir, releasefile), releasefile)
         os.rmdir(release_dir)
-    except Exception as oexc:
+    except Exception:
         exc = sys.exc_info()[1]
         os.chdir(rootdir)
-        raise exc from oexc
+        raise exc
     os.chdir(rootdir)
 
 
@@ -593,6 +595,15 @@ def make_deb(distro, build_os, arch, spec, srcdir):
     #
     sysassert(["sh", "-c", "cp -v \"%sdebian/\"*.postinst \"%sdebian/\"" % (srcdir, sdir)])
 
+    with open(sdir + "debian/substvars", "w") as fh:
+        # Empty for now. This makes it easier to add substvars to packages
+        # later on if we need it.
+        fh.write("\n")
+
+    ensure_dir(sdir + "debian/source/format")
+    with open(sdir + "debian/source/format", "w") as fh:
+        fh.write("1.0\n")
+
     # Do the packaging.
     oldcwd = os.getcwd()
     try:
@@ -669,12 +680,12 @@ def move_repos_into_place(src, dst):  # pylint: disable=too-many-branches
         try:
             os.mkdir(dname)
             break
-        except OSError as oexc:
+        except OSError:
             exc = sys.exc_info()[1]
             if exc.errno == errno.EEXIST:
                 pass
             else:
-                raise exc from oexc
+                raise exc
         i = i + 1
 
     # Put the stuff in our new directory.
@@ -689,12 +700,12 @@ def move_repos_into_place(src, dst):  # pylint: disable=too-many-branches
         try:
             os.symlink(dname, tmpnam)
             break
-        except OSError as oexc:  # as exc: # Python >2.5
+        except OSError:  # as exc: # Python >2.5
             exc = sys.exc_info()[1]
             if exc.errno == errno.EEXIST:
                 pass
             else:
-                raise exc from oexc
+                raise exc
         i = i + 1
 
     # Make a symlink to the old directory; this symlink will be
@@ -707,12 +718,12 @@ def move_repos_into_place(src, dst):  # pylint: disable=too-many-branches
             try:
                 os.symlink(os.readlink(dst), oldnam)
                 break
-            except OSError as oexc:  # as exc: # Python >2.5
+            except OSError:  # as exc: # Python >2.5
                 exc = sys.exc_info()[1]
                 if exc.errno == errno.EEXIST:
                     pass
                 else:
-                    raise exc from oexc
+                    raise exc
 
     os.rename(tmpnam, dst)
     if oldnam:
@@ -779,14 +790,8 @@ def make_rpm(distro, build_os, arch, spec, srcdir):  # pylint: disable=too-many-
         ensure_dir("%s/%s/" % (topdir, subdir))
     distro_arch = distro.archname(arch)
 
-    # The version of rpm and rpm tools in RHEL 5.5 can't interpolate the
-    # %{dynamic_version} macro, so do it manually
-    with open(specfile, "r") as spec_source:
-        with open(topdir + "SPECS/" + os.path.basename(specfile), "w") as spec_dest:
-            for line in spec_source:
-                line = line.replace('%{dynamic_version}', spec.pversion(distro))
-                line = line.replace('%{dynamic_release}', spec.prelease())
-                spec_dest.write(line)
+    # Places the RPM Spec file where it's expected for the rpmbuild execution later.
+    shutil.copy(specfile, topdir + "SPECS")
 
     oldcwd = os.getcwd()
     os.chdir(sdir + "/../")
@@ -806,8 +811,6 @@ def make_rpm(distro, build_os, arch, spec, srcdir):  # pylint: disable=too-many-
         "-D",
         f"dist .{distro.release_dist(build_os)}",
         "-D",
-        "_use_internal_dependency_generator 0",
-        "-D",
         f"dynamic_version {spec.pversion(distro)}",
         "-D",
         f"dynamic_release {spec.prelease()}",
@@ -847,12 +850,12 @@ def ensure_dir(filename):
     dirpart = os.path.dirname(filename)
     try:
         os.makedirs(dirpart)
-    except OSError as oexc:  # as exc: # Python >2.5
+    except OSError:  # as exc: # Python >2.5
         exc = sys.exc_info()[1]
         if exc.errno == errno.EEXIST:
             pass
         else:
-            raise exc from oexc
+            raise exc
     return filename
 
 
