@@ -6,6 +6,8 @@ import itertools
 import os.path
 import time
 
+import buildscripts.resmokelib.setup_multiversion.config as multiversion_config
+
 # Subdirectory under the dbpath prefix that contains directories with data files of mongod's started
 # by resmoke.py.
 FIXTURE_SUBDIR = "resmoke"
@@ -37,7 +39,10 @@ DEFAULT_DBPATH_PREFIX = os.path.normpath("/data/db")
 
 # Default directory that we expect to contain binaries for multiversion testing. This directory is
 # added to the PATH when calling programs.make_process().
-DEFAULT_MULTIVERSION_DIR = os.path.normpath("/data/multiversion")
+DEFAULT_MULTIVERSION_DIRS = [os.path.normpath("/data/multiversion")]
+if os.path.isfile(multiversion_config.WINDOWS_BIN_PATHS_FILE):
+    with open(multiversion_config.WINDOWS_BIN_PATHS_FILE) as wbpf:
+        DEFAULT_MULTIVERSION_DIRS.extend(wbpf.read().split(os.pathsep))
 
 # Default location for the genny executable. Override this in the YAML suite configuration if
 # desired.
@@ -45,11 +50,12 @@ DEFAULT_GENNY_EXECUTABLE = os.path.normpath("genny/build/src/driver/genny")
 
 # Names below correspond to how they are specified via the command line or in the options YAML file.
 DEFAULTS = {
+    "auto_kill": "on",
     "always_use_log_files": False,
-    "archive_file": None,
     "archive_limit_mb": 5000,
     "archive_limit_tests": 10,
     "base_port": 20000,
+    "backup_on_restart_dir": None,
     "buildlogger_url": "https://logkeeper2.build.10gen.cc",
     "continue_on_failure": False,
     "dbpath_prefix": None,
@@ -58,48 +64,63 @@ DEFAULTS = {
     "exclude_with_any_tags": None,
     "flow_control": None,
     "flow_control_tickets": None,
+    "fuzz_mongod_configs": False,
+    "config_fuzz_seed": None,
     "genny_executable": None,
     "include_with_any_tags": None,
+    "include_with_all_tags": None,
     "install_dir": None,
     "jobs": 1,
-    "logger_file": None,
+    "logger_file": "console",
     "mongo_executable": None,
     "mongod_executable": None,
     "mongod_set_parameters": [],
     "mongos_executable": None,
     "mongos_set_parameters": [],
+    "mongocryptd_set_parameters": [],
+    "mrlog": None,
     "no_journal": False,
     "num_clients_per_fixture": 1,
     "perf_report_file": None,
+    "cedar_report_file": None,
     "repeat_suites": 1,
     "repeat_tests": 1,
     "repeat_tests_max": None,
     "repeat_tests_min": None,
     "repeat_tests_secs": None,
+    "replay_file": None,
+    "report_failure_status": "fail",
     "report_file": None,
+    "run_all_feature_flag_tests": False,
+    "run_no_feature_flag_tests": False,
+    "additional_feature_flags": None,
+    "additional_feature_flags_file": None,
     "seed": int(time.time() * 256),  # Taken from random.py code in Python 2.7.
     "service_executor": None,
     "shell_conn_string": None,
     "shell_port": None,
-    "shell_read_mode": None,
-    "shell_write_mode": None,
     "shuffle": None,
-    "spawn_using": None,
     "stagger_jobs": None,
-    "majority_read_concern": None,  # Default is set on the commandline.
-    "enable_enterprise_tests": None,
-    "storage_engine": None,
+    "majority_read_concern": "on",
+    "storage_engine": "wiredTiger",
     "storage_engine_cache_size_gb": None,
     "suite_files": "with_server",
-    "tag_file": None,
-    "test_files": None,
+    "tag_files": [],
+    "test_files": [],
     "transport_layer": None,
+    "user_friendly_output": None,
     "mixed_bin_versions": None,
+    "old_bin_version": None,
     "linear_chain": None,
     "num_replset_nodes": None,
     "num_shards": None,
+    "export_mongod_config": "off",
+
+    # Internal testing options.
+    "internal_params": [],
 
     # Evergreen options.
+    "evergreen_url": "evergreen.mongodb.com",
     "build_id": None,
     "distro_id": None,
     "execution_number": 0,
@@ -109,6 +130,7 @@ DEFAULTS = {
     "revision_order_id": None,
     "task_id": None,
     "task_name": None,
+    "task_doc": None,
     "variant_name": None,
     "version_id": None,
 
@@ -124,7 +146,16 @@ DEFAULTS = {
     "benchmark_repetitions": None,
 
     # Config Dir
-    "config_dir": "buildscripts/resmokeconfig"
+    "config_dir": "buildscripts/resmokeconfig",
+
+    # UndoDB options
+    "undo_recorder_path": None,
+
+    # Generate multiversion exclude tags options
+    "exclude_tags_file_path": "generated_resmoke_config/multiversion_exclude_tags.yml",
+
+    # Limit the number of tests to execute
+    "max_test_queue_size": None,
 }
 
 _SuiteOptions = collections.namedtuple("_SuiteOptions", [
@@ -137,6 +168,7 @@ _SuiteOptions = collections.namedtuple("_SuiteOptions", [
     "num_repeat_tests_max",
     "num_repeat_tests_min",
     "time_repeat_tests_secs",
+    "report_failure_status",
 ])
 
 
@@ -203,6 +235,7 @@ class SuiteOptions(_SuiteOptions):
                     REPEAT_TESTS_MAX,
                     REPEAT_TESTS_MIN,
                     REPEAT_TESTS_SECS,
+                    REPORT_FAILURE_STATUS,
                 ])))
 
         options = self._asdict()
@@ -216,22 +249,39 @@ class SuiteOptions(_SuiteOptions):
 SuiteOptions.ALL_INHERITED = SuiteOptions(  # type: ignore
     **dict(list(zip(SuiteOptions._fields, itertools.repeat(SuiteOptions.INHERIT)))))
 
+
+class MultiversionOptions(object):
+    """Represent the multiversion version choices."""
+
+    LAST_LTS = "last_lts"
+    LAST_CONTINUOUS = "last_continuous"
+
+    @classmethod
+    def all_options(cls):
+        """Return available version options for multiversion."""
+
+        return [cls.LAST_LTS, cls.LAST_CONTINUOUS]
+
+
 ##
 # Variables that are set by the user at the command line or with --options.
 ##
 
+# Allow resmoke permission to automatically kill existing rogue mongo processes.
+AUTO_KILL = "on"
+
 # Log to files located in the db path and don't clean dbpaths after tests.
 ALWAYS_USE_LOG_FILES = False
 
-# The name of the archive JSON file used to associate S3 archives to an Evergreen task.
-ARCHIVE_FILE = None
-
 # The limit size of all archive files for an Evergreen task.
 ARCHIVE_LIMIT_MB = None
 
 # The limit number of tests to archive for an Evergreen task.
 ARCHIVE_LIMIT_TESTS = None
 
+# Whether to back up data when restarting a process.
+BACKUP_ON_RESTART_DIR = None
+
 # The starting port number to use for mongod and mongos processes spawned by resmoke.py and the
 # mongo shell.
 BASE_PORT = None
@@ -250,6 +300,9 @@ DBTEST_EXECUTABLE = None
 # actually running them).
 DRY_RUN = None
 
+# URL to connect to the Evergreen service.
+EVERGREEN_URL = None
+
 # An identifier consisting of the project name, build variant name, commit hash, and the timestamp.
 # For patch builds, it also includes the patch version id.
 EVERGREEN_BUILD_ID = None
@@ -279,6 +332,9 @@ EVERGREEN_TASK_ID = None
 # The name of the Evergreen task that resmoke.py is being run for.
 EVERGREEN_TASK_NAME = None
 
+# The documentation that describes what Evergreen task does.
+EVERGREEN_TASK_DOC = None
+
 # The name of the Evergreen build variant that resmoke.py is being run on.
 EVERGREEN_VARIANT_NAME = None
 
@@ -295,6 +351,9 @@ EXCLUDED_TAG = "__TEMPORARILY_DISABLED__"
 # If true, then a test failure or error will cause resmoke.py to exit and not run any more tests.
 FAIL_FAST = None
 
+FUZZ_MONGOD_CONFIGS = False
+CONFIG_FUZZ_SEED = None
+
 # Executable file for genny, passed in as a command line arg.
 GENNY_EXECUTABLE = None
 
@@ -302,6 +361,13 @@ GENNY_EXECUTABLE = None
 # jstest portion of the suite(s).
 INCLUDE_WITH_ANY_TAGS = None
 
+# If set, only jstests that have all the tags will be run.
+INCLUDE_TAGS = None
+
+# Params that can be set to change internal resmoke behavior. Used to test resmoke and should
+# not be set by the user.
+INTERNAL_PARAMS = []
+
 # If set, then resmoke.py starts the specified number of Job instances to run tests.
 JOBS = None
 
@@ -311,6 +377,18 @@ LOGGER_FILE = None
 # Where to find the MONGO*_EXECUTABLE binaries
 INSTALL_DIR = None
 
+# Whether to run tests for feature flags.
+RUN_ALL_FEATURE_FLAG_TESTS = None
+
+# Whether to run the tests with enabled feature flags
+RUN_NO_FEATURE_FLAG_TESTS = None
+
+# the path to a file containing feature flags
+ADDITIONAL_FEATURE_FLAGS_FILE = None
+
+# List of enabled feature flags.
+ENABLED_FEATURE_FLAGS = []
+
 # The path to the mongo executable used by resmoke.py.
 MONGO_EXECUTABLE = None
 
@@ -326,6 +404,9 @@ MONGOS_EXECUTABLE = None
 # The --setParameter options passed to mongos.
 MONGOS_SET_PARAMETERS = []
 
+# The --setParameter options passed to mongocryptd.
+MONGOCRYPTD_SET_PARAMETERS = []
+
 # If true, then all mongod's started by resmoke.py and by the mongo shell will not have journaling
 # enabled.
 NO_JOURNAL = None
@@ -336,6 +417,9 @@ NUM_CLIENTS_PER_FIXTURE = None
 # Report file for the Evergreen performance plugin.
 PERF_REPORT_FILE = None
 
+# Report file for Cedar.
+CEDAR_REPORT_FILE = None
+
 # If set, then the RNG is seeded with the specified value. Otherwise uses a seed based on the time
 # this module was loaded.
 RANDOM_SEED = None
@@ -357,6 +441,9 @@ REPEAT_TESTS_MIN = None
 # If set, then each test is repeated the specified time (seconds) inside the suites.
 REPEAT_TESTS_SECS = None
 
+# Controls if the test failure status should be reported as failed or be silently ignored.
+REPORT_FAILURE_STATUS = None
+
 # If set, then resmoke.py will write out a report file with the status of each test that ran.
 REPORT_FILE = None
 
@@ -367,20 +454,10 @@ SERVICE_EXECUTOR = None
 # connection string instead.
 SHELL_CONN_STRING = None
 
-# If set, then mongo shells started by resmoke.py will use the specified read mode.
-SHELL_READ_MODE = None
-
-# If set, then mongo shells started by resmoke.py will use the specified write mode.
-SHELL_WRITE_MODE = None
-
 # If true, then the order the tests run in is randomized. Otherwise the tests will run in
 # alphabetical (case-insensitive) order.
 SHUFFLE = None
 
-# Possible values are python and jasper. If python, resmoke uses the python built-in subprocess
-# or subprocess32 module to spawn threads. If jasper, resmoke uses the jasper module.
-SPAWN_USING = None
-
 # If true, the launching of jobs is staggered in resmoke.py.
 STAGGER_JOBS = None
 
@@ -390,12 +467,21 @@ MAJORITY_READ_CONCERN = None
 # Specifies the binary versions of each node we should run for a replica set.
 MIXED_BIN_VERSIONS = None
 
+# Specifies the binary version of last-lts or last-continous when multiversion enabled
+MULTIVERSION_BIN_VERSION = None
+
 # Specifies the number of replica set members in a ReplicaSetFixture.
 NUM_REPLSET_NODES = None
 
+# Specifies the number of replica sets in a MultiReplicaSetFixture.
+NUM_REPLSETS = None
+
 # Specifies the number of shards in a ShardedClusterFixture.
 NUM_SHARDS = None
 
+# Specifies whether to export the history of mongod config options.
+EXPORT_MONGOD_CONFIG = None
+
 # If true, run ReplicaSetFixture with linear chaining.
 LINEAR_CHAIN = None
 
@@ -418,7 +504,7 @@ STORAGE_ENGINE_CACHE_SIZE = None
 SUITE_FILES = None
 
 # The tag file to use that associates tests with tags.
-TAG_FILE = None
+TAG_FILES = None
 
 # The test files to execute.
 TEST_FILES = None
@@ -444,10 +530,22 @@ BENCHMARK_LIST_TESTS = None
 BENCHMARK_MIN_TIME = None
 BENCHMARK_REPETITIONS = None
 
+# UndoDB options
+UNDO_RECORDER_PATH = None
+
+# # Generate multiversion exclude tags options
+EXCLUDE_TAGS_FILE_PATH = None
+
+# Limit the number of tests to execute
+MAX_TEST_QUEUE_SIZE = None
+
 ##
 # Internally used configuration options that aren't exposed to the user
 ##
 
+# The name of the archive JSON file used to associate S3 archives to an Evergreen task.
+ARCHIVE_FILE = "archive.json"
+
 # S3 Bucket to upload archive files.
 ARCHIVE_BUCKET = "mongodatafiles"
 
@@ -475,8 +573,23 @@ EXTERNAL_SUITE_SELECTORS = (DEFAULT_BENCHMARK_TEST_LIST, DEFAULT_UNIT_TEST_LIST,
 
 # Where to look for logging and suite configuration files
 CONFIG_DIR = None
-NAMED_SUITES = None
 LOGGER_DIR = None
 
 # Generated logging config for the current invocation.
-LOGGING_CONFIG = None
+LOGGING_CONFIG: dict = {}
+SHORTEN_LOGGER_NAME_CONFIG: dict = {}
+
+# Whether legacy multiversion code is used. This value is not used on the master version of
+# resmoke.py but is needed to ensure the v5.0 version of fixture classes (e.g. standalone.py)
+# that get loaded for multiversion tests can behave correctly on master and on v5.0; the latter
+# case runs 5.0 and 4.4 binaries and has this value set to True. Can be removed after 6.0.
+USE_LEGACY_MULTIVERSION = True
+
+# Expansions file location
+# in CI, the expansions file is located in the ${workdir}, one dir up
+# from src, the checkout directory
+EXPANSIONS_FILE = "../expansions.yml" if 'CI' in os.environ else None
+
+# Symbolizer secrets
+SYMBOLIZER_CLIENT_SECRET = None
+SYMBOLIZER_CLIENT_ID = None
