@@ -1,28 +1,34 @@
 """Configure the command line input for the resmoke 'run' subcommand."""
 
 import collections
+import configparser
+import datetime
 import os
 import os.path
+import distutils.spawn
 import sys
-import configparser
+import platform
 import random
 import glob
 import textwrap
 import shlex
 
-from typing import NamedTuple
-
-import datetime
 import pymongo.uri_parser
 
-from . import config as _config
-from . import utils
+from buildscripts.idl import gen_all_feature_flag_list
+from buildscripts.idl.lib import ALL_FEATURE_FLAG_FILE
+
+from buildscripts.resmokelib import config as _config
+from buildscripts.resmokelib import utils
+from buildscripts.resmokelib import mongod_fuzzer_configs
+from buildscripts.resmokelib.suitesconfig import SuiteFinder
 
 
 def validate_and_update_config(parser, args):
     """Validate inputs and update config module."""
     _validate_options(parser, args)
     _update_config_vars(args)
+    _update_symbolizer_secrets()
     _validate_config(parser)
     _set_logging_config()
 
@@ -41,6 +47,16 @@ def _validate_options(parser, args):
                      " test(s) under those suite configuration(s)".format(
                          args.executor_file, " ".join(args.test_files)))
 
+    # The "test_files" positional argument logically overlaps with `--replayFile`. Disallow using both.
+    if args.test_files and args.replay_file:
+        parser.error(
+            "Cannot use --replayFile with additional test files listed on the command line invocation."
+        )
+
+    if args.additional_feature_flags_file and not os.path.isfile(
+            args.additional_feature_flags_file):
+        parser.error("The specified additional feature flags file does not exist.")
+
     def get_set_param_errors(process_params):
         agg_set_params = collections.defaultdict(list)
         for set_param in process_params:
@@ -49,23 +65,30 @@ def _validate_options(parser, args):
 
         errors = []
         for key, values in agg_set_params.items():
-            if len(values) > 1:
-                for left, _ in enumerate(values):
-                    for right in range(left + 1, len(values)):
-                        if values[left] != values[right]:
-                            errors.append(
-                                f"setParameter has multiple distinct values. Key: {key} Values: {values}"
-                            )
+            if len(values) == 1:
+                continue
+
+            for left, _ in enumerate(values):
+                for right in range(left + 1, len(values)):
+                    if values[left] != values[right]:
+                        errors.append(
+                            f"setParameter has multiple distinct values. Key: {key} Values: {values}"
+                        )
+
         return errors
 
     config = vars(args)
     mongod_set_param_errors = get_set_param_errors(config.get('mongod_set_parameters') or [])
     mongos_set_param_errors = get_set_param_errors(config.get('mongos_set_parameters') or [])
+    mongocryptd_set_param_errors = get_set_param_errors(
+        config.get('mongocryptd_set_parameters') or [])
     error_msgs = {}
     if mongod_set_param_errors:
         error_msgs["mongodSetParameters"] = mongod_set_param_errors
     if mongos_set_param_errors:
         error_msgs["mongosSetParameters"] = mongos_set_param_errors
+    if mongocryptd_set_param_errors:
+        error_msgs["mongocryptdSetParameters"] = mongocryptd_set_param_errors
     if error_msgs:
         parser.error(str(error_msgs))
 
@@ -92,6 +115,23 @@ def _validate_config(parser):  # pylint: disable=too-many-branches
                 parser.error("Must specify binary versions as 'old' or 'new' in format"
                              " 'version1-version2'")
 
+    if _config.UNDO_RECORDER_PATH is not None:
+        if not sys.platform.startswith('linux') or platform.machine() not in [
+                "i386", "i686", "x86_64"
+        ]:
+            parser.error("--recordWith is only supported on x86 and x86_64 Linux distributions")
+            return
+
+        resolved_path = distutils.spawn.find_executable(_config.UNDO_RECORDER_PATH)
+        if resolved_path is None:
+            parser.error(
+                f"Cannot find the UndoDB live-record binary '{_config.UNDO_RECORDER_PATH}'. Check that it exists and is executable"
+            )
+            return
+
+        if not os.access(resolved_path, os.X_OK):
+            parser.error(f"Found '{resolved_path}', but it is not an executable file")
+
 
 def _find_resmoke_wrappers():
     # This is technically incorrect. PREFIX_BINDIR defaults to $PREFIX/bin, so
@@ -141,29 +181,84 @@ be invoked as either:
 - buildscripts/resmoke.py --installDir {shlex.quote(user_config['install_dir'])}""")
         raise RuntimeError(err)
 
+    def process_feature_flag_file(path):
+        with open(path) as fd:
+            return fd.read().split()
+
+    def setup_feature_flags():
+        _config.RUN_ALL_FEATURE_FLAG_TESTS = config.pop("run_all_feature_flag_tests")
+        _config.RUN_NO_FEATURE_FLAG_TESTS = config.pop("run_no_feature_flag_tests")
+        _config.ADDITIONAL_FEATURE_FLAGS_FILE = config.pop("additional_feature_flags_file")
+
+        if _config.RUN_ALL_FEATURE_FLAG_TESTS:
+            print("Generating: ", ALL_FEATURE_FLAG_FILE)
+            gen_all_feature_flag_list.gen_all_feature_flags_file()
+
+        all_ff = []
+        enabled_feature_flags = []
+        try:
+            all_ff = process_feature_flag_file(ALL_FEATURE_FLAG_FILE)
+        except FileNotFoundError:
+            # If we ask resmoke to run with all feature flags, the feature flags file
+            # needs to exist.
+            if _config.RUN_ALL_FEATURE_FLAG_TESTS or _config.RUN_NO_FEATURE_FLAG_TESTS:
+                raise
+
+        if _config.RUN_ALL_FEATURE_FLAG_TESTS:
+            enabled_feature_flags = all_ff[:]
+
+        if _config.ADDITIONAL_FEATURE_FLAGS_FILE:
+            enabled_feature_flags.extend(
+                process_feature_flag_file(_config.ADDITIONAL_FEATURE_FLAGS_FILE))
+
+        # Specify additional feature flags from the command line.
+        # Set running all feature flag tests to True if this options is specified.
+        additional_feature_flags = _tags_from_list(config.pop("additional_feature_flags"))
+        if additional_feature_flags is not None:
+            enabled_feature_flags.extend(additional_feature_flags)
+
+        return enabled_feature_flags, all_ff
+
+    _config.ENABLED_FEATURE_FLAGS, all_feature_flags = setup_feature_flags()
+    not_enabled_feature_flags = list(set(all_feature_flags) - set(_config.ENABLED_FEATURE_FLAGS))
+
+    _config.AUTO_KILL = config.pop("auto_kill")
     _config.ALWAYS_USE_LOG_FILES = config.pop("always_use_log_files")
-    _config.ARCHIVE_FILE = config.pop("archive_file")
     _config.BASE_PORT = int(config.pop("base_port"))
+    _config.BACKUP_ON_RESTART_DIR = config.pop("backup_on_restart_dir")
     _config.BUILDLOGGER_URL = config.pop("buildlogger_url")
     _config.DBPATH_PREFIX = _expand_user(config.pop("dbpath_prefix"))
     _config.DRY_RUN = config.pop("dry_run")
+
     # EXCLUDE_WITH_ANY_TAGS will always contain the implicitly defined EXCLUDED_TAG.
     _config.EXCLUDE_WITH_ANY_TAGS = [_config.EXCLUDED_TAG]
     _config.EXCLUDE_WITH_ANY_TAGS.extend(
         utils.default_if_none(_tags_from_list(config.pop("exclude_with_any_tags")), []))
+
+    if _config.RUN_NO_FEATURE_FLAG_TESTS:
+        # Don't run any feature flag tests.
+        _config.EXCLUDE_WITH_ANY_TAGS.extend(all_feature_flags)
+    else:
+        # Don't run tests with feature flags that are not enabled.
+        _config.EXCLUDE_WITH_ANY_TAGS.extend(not_enabled_feature_flags)
+
     _config.FAIL_FAST = not config.pop("continue_on_failure")
     _config.FLOW_CONTROL = config.pop("flow_control")
     _config.FLOW_CONTROL_TICKETS = config.pop("flow_control_tickets")
+
     _config.INCLUDE_WITH_ANY_TAGS = _tags_from_list(config.pop("include_with_any_tags"))
+    _config.INCLUDE_TAGS = _tags_from_list(config.pop("include_with_all_tags"))
+
     _config.GENNY_EXECUTABLE = _expand_user(config.pop("genny_executable"))
     _config.JOBS = config.pop("jobs")
     _config.LINEAR_CHAIN = config.pop("linear_chain") == "on"
     _config.MAJORITY_READ_CONCERN = config.pop("majority_read_concern") == "on"
-    _config.ENABLE_ENTERPRISE_TESTS = config.pop("enable_enterprise_tests")
     _config.MIXED_BIN_VERSIONS = config.pop("mixed_bin_versions")
     if _config.MIXED_BIN_VERSIONS is not None:
         _config.MIXED_BIN_VERSIONS = _config.MIXED_BIN_VERSIONS.split("-")
 
+    _config.MULTIVERSION_BIN_VERSION = config.pop("old_bin_version")
+
     _config.INSTALL_DIR = config.pop("install_dir")
     if values.command == "run" and _config.INSTALL_DIR is None:
         resmoke_wrappers = _find_resmoke_wrappers()
@@ -198,37 +293,60 @@ or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py
         return utils.dump_yaml(ret)
 
     _config.MONGOD_EXECUTABLE = _expand_user(config.pop("mongod_executable"))
-    _config.MONGOD_SET_PARAMETERS = _merge_set_params(config.pop("mongod_set_parameters"))
-    _config.MONGOS_EXECUTABLE = _expand_user(config.pop("mongos_executable"))
-    _config.MONGOS_SET_PARAMETERS = _merge_set_params(config.pop("mongos_set_parameters"))
 
+    mongod_set_parameters = config.pop("mongod_set_parameters")
+
+    _config.MONGOD_SET_PARAMETERS = _merge_set_params(mongod_set_parameters)
+    _config.FUZZ_MONGOD_CONFIGS = config.pop("fuzz_mongod_configs")
+    _config.CONFIG_FUZZ_SEED = config.pop("config_fuzz_seed")
+
+    if _config.FUZZ_MONGOD_CONFIGS:
+        if not _config.CONFIG_FUZZ_SEED:
+            _config.CONFIG_FUZZ_SEED = random.randrange(sys.maxsize)
+        else:
+            _config.CONFIG_FUZZ_SEED = int(_config.CONFIG_FUZZ_SEED)
+        _config.MONGOD_SET_PARAMETERS, _config.WT_ENGINE_CONFIG, _config.WT_COLL_CONFIG, \
+        _config.WT_INDEX_CONFIG = mongod_fuzzer_configs.fuzz_set_parameters(
+            _config.CONFIG_FUZZ_SEED, _config.MONGOD_SET_PARAMETERS)
+
+    _config.MONGOS_EXECUTABLE = _expand_user(config.pop("mongos_executable"))
+    mongos_set_parameters = config.pop("mongos_set_parameters")
+    _config.MONGOS_SET_PARAMETERS = _merge_set_params(mongos_set_parameters)
+
+    _config.MONGOCRYPTD_SET_PARAMETERS = _merge_set_params(config.pop("mongocryptd_set_parameters"))
+
+    _config.MRLOG = config.pop("mrlog")
     _config.NO_JOURNAL = config.pop("no_journal")
     _config.NUM_CLIENTS_PER_FIXTURE = config.pop("num_clients_per_fixture")
     _config.NUM_REPLSET_NODES = config.pop("num_replset_nodes")
     _config.NUM_SHARDS = config.pop("num_shards")
     _config.PERF_REPORT_FILE = config.pop("perf_report_file")
+    _config.CEDAR_REPORT_FILE = config.pop("cedar_report_file")
     _config.RANDOM_SEED = config.pop("seed")
     _config.REPEAT_SUITES = config.pop("repeat_suites")
     _config.REPEAT_TESTS = config.pop("repeat_tests")
     _config.REPEAT_TESTS_MAX = config.pop("repeat_tests_max")
     _config.REPEAT_TESTS_MIN = config.pop("repeat_tests_min")
     _config.REPEAT_TESTS_SECS = config.pop("repeat_tests_secs")
+    _config.REPORT_FAILURE_STATUS = config.pop("report_failure_status")
     _config.REPORT_FILE = config.pop("report_file")
     _config.SERVICE_EXECUTOR = config.pop("service_executor")
-    _config.SHELL_READ_MODE = config.pop("shell_read_mode")
-    _config.SHELL_WRITE_MODE = config.pop("shell_write_mode")
-    _config.SPAWN_USING = config.pop("spawn_using")
+    _config.EXPORT_MONGOD_CONFIG = config.pop("export_mongod_config")
     _config.STAGGER_JOBS = config.pop("stagger_jobs") == "on"
     _config.STORAGE_ENGINE = config.pop("storage_engine")
     _config.STORAGE_ENGINE_CACHE_SIZE = config.pop("storage_engine_cache_size_gb")
     _config.SUITE_FILES = config.pop("suite_files")
     if _config.SUITE_FILES is not None:
         _config.SUITE_FILES = _config.SUITE_FILES.split(",")
-    _config.TAG_FILE = config.pop("tag_file")
-    _config.TEST_FILES = config.pop("test_files")
+    _config.TAG_FILES = config.pop("tag_files")
     _config.TRANSPORT_LAYER = config.pop("transport_layer")
+    _config.USER_FRIENDLY_OUTPUT = config.pop("user_friendly_output")
+
+    # Internal testing options.
+    _config.INTERNAL_PARAMS = config.pop("internal_params")
 
     # Evergreen options.
+    _config.EVERGREEN_URL = config.pop("evergreen_url")
     _config.EVERGREEN_BUILD_ID = config.pop("build_id")
     _config.EVERGREEN_DISTRO_ID = config.pop("distro_id")
     _config.EVERGREEN_EXECUTION = config.pop("execution_number")
@@ -238,6 +356,7 @@ or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py
     _config.EVERGREEN_REVISION_ORDER_ID = config.pop("revision_order_id")
     _config.EVERGREEN_TASK_ID = config.pop("task_id")
     _config.EVERGREEN_TASK_NAME = config.pop("task_name")
+    _config.EVERGREEN_TASK_DOC = config.pop("task_doc")
     _config.EVERGREEN_VARIANT_NAME = config.pop("variant_name")
     _config.EVERGREEN_VERSION_ID = config.pop("version_id")
 
@@ -245,26 +364,23 @@ or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py
     if not _config.EVERGREEN_TASK_ID:
         _config.ARCHIVE_FILE = None
     else:
-        required_builder_names = set([
-            "ubuntu1804-debug-aubsan-lite",
-            "linux-64-debug",
-            "enterprise-windows-required",
-            "enterprise-rhel-62-64-bit",
-            "enterprise-ubuntu-dynamic-1604-clang",
-        ])
-        # Enable archival globally for all required mainline builders.
-        if (_config.EVERGREEN_VARIANT_NAME is not None
-                and _config.EVERGREEN_VARIANT_NAME in required_builder_names
-                and not _config.EVERGREEN_PATCH_BUILD):
+        # Enable archival globally for all mainline variants.
+        if _config.EVERGREEN_VARIANT_NAME is not None and not _config.EVERGREEN_PATCH_BUILD:
             _config.FORCE_ARCHIVE_ALL_DATA_FILES = True
 
     _config.ARCHIVE_LIMIT_MB = config.pop("archive_limit_mb")
     _config.ARCHIVE_LIMIT_TESTS = config.pop("archive_limit_tests")
 
-    # Wiredtiger options.
-    _config.WT_COLL_CONFIG = config.pop("wt_coll_config")
-    _config.WT_ENGINE_CONFIG = config.pop("wt_engine_config")
-    _config.WT_INDEX_CONFIG = config.pop("wt_index_config")
+    # Wiredtiger options. Prevent fuzzed wt configs from being overwritten unless user specifies it.
+    wt_engine_config = config.pop("wt_engine_config")
+    if wt_engine_config:
+        _config.WT_ENGINE_CONFIG = config.pop("wt_engine_config")
+    wt_coll_config = config.pop("wt_coll_config")
+    if wt_coll_config:
+        _config.WT_COLL_CONFIG = config.pop("wt_coll_config")
+    wt_index_config = config.pop("wt_index_config")
+    if wt_index_config:
+        _config.WT_INDEX_CONFIG = config.pop("wt_index_config")
 
     # Benchmark/Benchrun options.
     _config.BENCHMARK_FILTER = config.pop("benchmark_filter")
@@ -277,19 +393,40 @@ or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py
     # Config Dir options.
     _config.CONFIG_DIR = config.pop("config_dir")
 
-    # Populate the named suites by scanning config_dir/suites
-    named_suites = {}
+    # Configure evergreen task documentation
+    if _config.EVERGREEN_TASK_NAME:
+        task_name = utils.get_task_name_without_suffix(_config.EVERGREEN_TASK_NAME,
+                                                       _config.EVERGREEN_VARIANT_NAME)
+        evg_task_doc_file = os.path.join(_config.CONFIG_DIR, "evg_task_doc", "evg_task_doc.yml")
+        if os.path.exists(evg_task_doc_file):
+            evg_task_doc = utils.load_yaml_file(evg_task_doc_file)
+            if task_name in evg_task_doc:
+                _config.EVERGREEN_TASK_DOC = evg_task_doc[task_name]
 
-    suites_dir = os.path.join(_config.CONFIG_DIR, "suites")
-    root = os.path.abspath(suites_dir)
-    files = os.listdir(root)
-    for filename in files:
-        (short_name, ext) = os.path.splitext(filename)
-        if ext in (".yml", ".yaml"):
-            pathname = os.path.join(root, filename)
-            named_suites[short_name] = pathname
+    _config.UNDO_RECORDER_PATH = config.pop("undo_recorder_path")
 
-    _config.NAMED_SUITES = named_suites
+    _config.EXCLUDE_TAGS_FILE_PATH = config.pop("exclude_tags_file_path")
+
+    _config.MAX_TEST_QUEUE_SIZE = config.pop("max_test_queue_size")
+
+    def configure_tests(test_files, replay_file):
+        # `_validate_options` has asserted that at most one of `test_files` and `replay_file` contains input.
+
+        to_replay = None
+        # Treat `resmoke run @to_replay` as `resmoke run --replayFile to_replay`
+        if len(test_files) == 1 and test_files[0].startswith("@"):
+            to_replay = test_files[0][1:]
+        elif replay_file:
+            to_replay = replay_file
+
+        if to_replay:
+            # The replay file is expected to be one file per line, but cope with extra whitespace.
+            with open(to_replay) as fd:
+                _config.TEST_FILES = fd.read().split()
+        else:
+            _config.TEST_FILES = test_files
+
+    configure_tests(config.pop("test_files"), config.pop("replay_file"))
 
     _config.LOGGER_DIR = os.path.join(_config.CONFIG_DIR, "loggers")
 
@@ -319,7 +456,7 @@ or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py
     _config.LOGGER_FILE = config.pop("logger_file")
 
     if config:
-        raise ValueError(f"Unkown option(s): {list(config.keys())}s")
+        raise ValueError(f"Unknown option(s): {list(config.keys())}s")
 
 
 def _set_logging_config():
@@ -329,7 +466,9 @@ def _set_logging_config():
         # If the user provides a full valid path to a logging config
         # we don't need to search LOGGER_DIR for the file.
         if os.path.exists(pathname):
-            _config.LOGGING_CONFIG = utils.load_yaml_file(pathname).pop("logging")
+            logger_config = utils.load_yaml_file(pathname)
+            _config.LOGGING_CONFIG = logger_config.pop("logging")
+            _config.SHORTEN_LOGGER_NAME_CONFIG = logger_config.pop("shorten_logger_name")
             return
 
         root = os.path.abspath(_config.LOGGER_DIR)
@@ -340,12 +479,14 @@ def _set_logging_config():
                 config_file = os.path.join(root, filename)
                 if not os.path.isfile(config_file):
                     raise ValueError("Expected a logger YAML config, but got '%s'" % pathname)
-                _config.LOGGING_CONFIG = utils.load_yaml_file(config_file).pop("logging")
+                logger_config = utils.load_yaml_file(config_file)
+                _config.LOGGING_CONFIG = logger_config.pop("logging")
+                _config.SHORTEN_LOGGER_NAME_CONFIG = logger_config.pop("shorten_logger_name")
                 return
 
         raise ValueError("Unknown logger '%s'" % pathname)
-    except FileNotFoundError as exc:
-        raise IOError("Directory {} does not exist.".format(_config.LOGGER_DIR)) from exc
+    except FileNotFoundError:
+        raise IOError("Directory {} does not exist.".format(_config.LOGGER_DIR))
 
 
 def _expand_user(pathname):
@@ -367,3 +508,13 @@ def _tags_from_list(tags_list):
             tags.extend([t for t in tag.split(",") if t != ""])
         return tags
     return None
+
+
+def _update_symbolizer_secrets():
+    """Open `expansions.yml`, get values for symbolizer secrets and update their values inside config.py ."""
+    if not _config.EVERGREEN_TASK_ID:
+        # not running on Evergreen
+        return
+    yml_data = utils.load_yaml_file(_config.EXPANSIONS_FILE)
+    _config.SYMBOLIZER_CLIENT_SECRET = yml_data.get("symbolizer_client_secret")
+    _config.SYMBOLIZER_CLIENT_ID = yml_data.get("symbolizer_client_id")
