@@ -7,20 +7,17 @@ on whether they apply to C++ unit tests, dbtests, or JS tests.
 import collections
 import errno
 import fnmatch
-import math
 import os.path
 import random
 import subprocess
 import sys
 
-import buildscripts.ciconfig.tags as _tags
-from . import config
-from . import errors
-from . import utils
-from .utils import globstar
-from .utils import jscomment
-
-ENTERPRISE_TEST_DIR = os.path.normpath("src/mongo/db/modules/enterprise/jstests")
+import buildscripts.resmokelib.testing.tags as _tags
+from buildscripts.resmokelib import config
+from buildscripts.resmokelib import errors
+from buildscripts.resmokelib import utils
+from buildscripts.resmokelib.utils import globstar
+from buildscripts.resmokelib.utils import jscomment
 
 ########################
 #  Test file explorer  #
@@ -117,7 +114,7 @@ class TestFileExplorer(object):
         return program.returncode, stdout.decode("utf-8"), stderr.decode("utf-8")
 
     @staticmethod
-    def parse_tag_file(test_kind, tag_file=None, tagged_tests=None):
+    def parse_tag_files(test_kind, tag_files=None, tagged_tests=None):
         """Parse the tag file and return a dict of tagged tests.
 
         The resulting dict will have as a key the filename and the
@@ -125,17 +122,21 @@ class TestFileExplorer(object):
         """
         if tagged_tests is None:
             tagged_tests = collections.defaultdict(list)
-        if tag_file and os.path.exists(tag_file):
-            tags_conf = _tags.TagsConfig.from_file(tag_file)
-            tagged_roots = tags_conf.get_test_patterns(test_kind)
-            for tagged_root in tagged_roots:
-                # Multiple tests could be returned for a set of tags.
-                tests = globstar.iglob(tagged_root)
-                test_tags = tags_conf.get_tags(test_kind, tagged_root)
-                for test in tests:
-                    # A test could have a tag in more than one place, due to wildcards in the
-                    # selector.
-                    tagged_tests[test].extend(test_tags)
+
+        if tag_files is None:
+            tag_files = []
+        for tag_file in tag_files:
+            if tag_file and os.path.exists(tag_file):
+                tags_conf = _tags.TagsConfig.from_file(tag_file)
+                tagged_roots = tags_conf.get_test_patterns(test_kind)
+                for tagged_root in tagged_roots:
+                    # Multiple tests could be returned for a set of tags.
+                    tests = globstar.iglob(tagged_root)
+                    test_tags = tags_conf.get_tags(test_kind, tagged_root)
+                    for test in tests:
+                        # A test could have a tag in more than one place, due to wildcards in the
+                        # selector.
+                        tagged_tests[test].extend(test_tags)
         return tagged_tests
 
 
@@ -214,13 +215,6 @@ class _TestList(object):
                          format(path)))
                 self._filtered.discard(path)
 
-    def filter_enterprise_tests(self):
-        """Exclude tests that start with the enterprise module directory from the test list."""
-        self._filtered = {
-            test
-            for test in self._filtered if not os.path.normpath(test).startswith(ENTERPRISE_TEST_DIR)
-        }
-
     def match_tag_expression(self, tag_expression, get_tags):
         """Filter the test list to only include tests that match the tag expression.
 
@@ -253,7 +247,8 @@ class _TestList(object):
         excluded = []
         for test in self._roots:
             if test in self._filtered:
-                if test not in tests:
+                if config.TEST_FILES or test not in tests:
+                    # Allow duplicate tests if the tests were explicitly duplicated on the CLI invocation or replay file.
                     tests.append(test)
             elif test not in excluded:
                 excluded.append(test)
@@ -378,8 +373,13 @@ class _SelectorConfig(object):
         exclude_with_any_tags = self.__merge_lists(exclude_with_any_tags,
                                                    config.EXCLUDE_WITH_ANY_TAGS)
 
+        # This is functionally similar to `include_tags` but contains a list of tags rather
+        # than an expression.
+        include_with_all_tags = config.INCLUDE_TAGS
+
         self.tags_expression = self.__make_tags_expression(
-            include_tags, exclude_tags, include_with_any_tags, exclude_with_any_tags)
+            include_tags, exclude_tags, include_with_any_tags, exclude_with_any_tags,
+            include_with_all_tags)
 
     @staticmethod
     def __merge_lists(list_a, list_b):
@@ -393,10 +393,13 @@ class _SelectorConfig(object):
 
     @staticmethod
     def __make_tags_expression(include_tags, exclude_tags, include_with_any_tags,
-                               exclude_with_any_tags):
+                               exclude_with_any_tags, include_with_all_tags):
         expressions = []
         if include_tags:
             expressions.append(make_expression(include_tags))
+        if include_with_all_tags:
+            include_with_all_tags_expr = make_expression({"$allOf": include_with_all_tags})
+            expressions.append(include_with_all_tags_expr)
         elif exclude_tags:
             expressions.append(_NotExpression(make_expression(exclude_tags)))
         if include_with_any_tags:
@@ -419,6 +422,7 @@ class _Selector(object):
 
         Args:
             test_file_explorer: a TestFileExplorer instance.
+            tests_are_files: whether tests are files.
         """
         self._test_file_explorer = test_file_explorer
         self._tests_are_files = tests_are_files
@@ -449,9 +453,6 @@ class _Selector(object):
         # 5. Apply the include files last with force=True to take precedence over the tags.
         if self._tests_are_files and selector_config.include_files:
             test_list.include_files(selector_config.include_files, force=True)
-        # 6: Apply the enterprise tests filter
-        if self.get_enterprise_tests_status() == "off":
-            test_list.filter_enterprise_tests()
 
         return self.sort_tests(*test_list.get_tests())
 
@@ -467,11 +468,6 @@ class _Selector(object):
         """Retrieve the tags associated with the give test file."""
         return []
 
-    @staticmethod
-    def get_enterprise_tests_status() -> str:
-        """Get the status of enterprise tests from the configuration."""
-        return config.ENABLE_ENTERPRISE_TESTS
-
 
 class _JSTestSelectorConfig(_SelectorConfig):
     """_SelectorConfig subclass for JavaScript tests."""
@@ -491,11 +487,11 @@ class _JSTestSelector(_Selector):
 
     def __init__(self, test_file_explorer):
         _Selector.__init__(self, test_file_explorer)
-        self._tags = self._test_file_explorer.parse_tag_file("js_test", config.TAG_FILE)
+        self._tags = self._test_file_explorer.parse_tag_files("js_test", config.TAG_FILES)
 
     def select(self, selector_config):
-        self._tags = self._test_file_explorer.parse_tag_file("js_test", selector_config.tag_file,
-                                                             self._tags)
+        self._tags = self._test_file_explorer.parse_tag_files("js_test", [selector_config.tag_file],
+                                                              self._tags)
         return _Selector.select(self, selector_config)
 
     def get_tags(self, test_file):
@@ -726,6 +722,7 @@ _SELECTOR_REGISTRY = {
     "parallel_fsm_workload_test": (_MultiJSTestSelectorConfig, _MultiJSTestSelector),
     "json_schema_test": (_FileBasedSelectorConfig, _Selector),
     "js_test": (_JSTestSelectorConfig, _JSTestSelector),
+    "all_versions_js_test": (_JSTestSelectorConfig, _JSTestSelector),
     "mql_model_haskell_test": (_FileBasedSelectorConfig, _Selector),
     "mql_model_mongod_test": (_FileBasedSelectorConfig, _Selector),
     "multi_stmt_txn_passthrough": (_JSTestSelectorConfig, _JSTestSelector),
@@ -734,6 +731,7 @@ _SELECTOR_REGISTRY = {
     "genny_test": (_FileBasedSelectorConfig, _Selector),
     "gennylib_test": (_GennylibTestCaseSelectorConfig, _GennylibTestCaseSelector),
     "cpp_libfuzzer_test": (_CppTestSelectorConfig, _CppTestSelector),
+    "tla_plus_test": (_FileBasedSelectorConfig, _Selector),
 }
 
 
