@@ -1,77 +1,84 @@
 """Replica set fixture for executing JSTests against."""
 
 import os.path
+import random
 import time
 
-import bson.errors
+import bson
 import pymongo
 import pymongo.errors
 import pymongo.write_concern
 
-from . import interface
-from . import replicaset_utils
-from . import standalone
-from ... import config
-from ... import errors
-from ... import utils
-from ...multiversionconstants import LAST_STABLE_MONGOD_BINARY
+import buildscripts.resmokelib.testing.fixtures.interface as interface
 
 
-class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-instance-attributes
+def compare_timestamp(timestamp1, timestamp2):
+    """Compare the timestamp object ts part."""
+    if timestamp1.time == timestamp2.time:
+        if timestamp1.inc < timestamp2.inc:
+            return -1
+        elif timestamp1.inc > timestamp2.inc:
+            return 1
+        else:
+            return 0
+    elif timestamp1.time < timestamp2.time:
+        return -1
+    else:
+        return 1
+
+
+def compare_optime(optime1, optime2):
+    """Compare timestamp object t part."""
+    if optime1["t"] > optime2["t"]:
+        return 1
+    elif optime1["t"] < optime2["t"]:
+        return -1
+    else:
+        return compare_timestamp(optime1["ts"], optime2["ts"])
+
+
+class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-instance-attributes, too-many-public-methods
     """Fixture which provides JSTests with a replica set to run against."""
 
     def __init__(  # pylint: disable=too-many-arguments, too-many-locals
-            self, logger, job_num, mongod_options=None, dbpath_prefix=None, preserve_dbpath=False,
-            num_nodes=2, start_initial_sync_node=False, write_concern_majority_journal_default=None,
-            auth_options=None, replset_config_options=None, voting_secondaries=True,
-            all_nodes_electable=False, use_replica_set_connection_string=None, linear_chain=False,
-            mixed_bin_versions=None, default_read_concern=None, default_write_concern=None):
+            self, logger, job_num, fixturelib, mongod_executable=None, mongod_options=None,
+            dbpath_prefix=None, preserve_dbpath=False, num_nodes=2, start_initial_sync_node=False,
+            write_concern_majority_journal_default=None, auth_options=None,
+            replset_config_options=None, voting_secondaries=True, all_nodes_electable=False,
+            use_replica_set_connection_string=None, linear_chain=False, default_read_concern=None,
+            default_write_concern=None, shard_logging_prefix=None, replicaset_logging_prefix=None,
+            replset_name=None):
         """Initialize ReplicaSetFixture."""
 
-        interface.ReplFixture.__init__(self, logger, job_num, dbpath_prefix=dbpath_prefix)
+        interface.ReplFixture.__init__(self, logger, job_num, fixturelib,
+                                       dbpath_prefix=dbpath_prefix)
 
-        self.mongod_options = utils.default_if_none(mongod_options, {})
+        self.mongod_executable = mongod_executable
+        self.mongod_options = self.fixturelib.make_historic(
+            self.fixturelib.default_if_none(mongod_options, {}))
         self.preserve_dbpath = preserve_dbpath
         self.start_initial_sync_node = start_initial_sync_node
         self.write_concern_majority_journal_default = write_concern_majority_journal_default
         self.auth_options = auth_options
-        self.replset_config_options = utils.default_if_none(replset_config_options, {})
+        self.replset_config_options = self.fixturelib.make_historic(
+            self.fixturelib.default_if_none(replset_config_options, {}))
         self.voting_secondaries = voting_secondaries
         self.all_nodes_electable = all_nodes_electable
         self.use_replica_set_connection_string = use_replica_set_connection_string
         self.default_read_concern = default_read_concern
         self.default_write_concern = default_write_concern
-        self.mixed_bin_versions = utils.default_if_none(mixed_bin_versions,
-                                                        config.MIXED_BIN_VERSIONS)
+        self.shard_logging_prefix = shard_logging_prefix
+        self.replicaset_logging_prefix = replicaset_logging_prefix
+        self.num_nodes = num_nodes
+        self.replset_name = replset_name
+        # Used by the enhanced multiversion system to signify multiversion mode.
+        # None implies no multiversion run.
+        self.fcv = None
 
         # Use the values given from the command line if they exist for linear_chain and num_nodes.
-        linear_chain_option = utils.default_if_none(config.LINEAR_CHAIN, linear_chain)
+        linear_chain_option = self.fixturelib.default_if_none(self.config.LINEAR_CHAIN,
+                                                              linear_chain)
         self.linear_chain = linear_chain_option if linear_chain_option else linear_chain
-        num_replset_nodes = config.NUM_REPLSET_NODES
-        self.num_nodes = num_replset_nodes if num_replset_nodes else num_nodes
-
-        if self.mixed_bin_versions is not None:
-            mongod_executable = utils.default_if_none(config.MONGOD_EXECUTABLE,
-                                                      config.DEFAULT_MONGOD_EXECUTABLE)
-            latest_mongod = mongod_executable
-            # The last-stable binary is currently expected to live in '/data/multiversion', which is
-            # part of the PATH.
-            is_config_svr = "configsvr" in self.replset_config_options and self.replset_config_options[
-                "configsvr"]
-            if not is_config_svr:
-                self.mixed_bin_versions = [
-                    latest_mongod if (x == "new") else LAST_STABLE_MONGOD_BINARY
-                    for x in self.mixed_bin_versions
-                ]
-            else:
-                # Our documented recommended path for upgrading shards lets us assume that config
-                # server nodes will always be fully upgraded before shard nodes.
-                self.mixed_bin_versions = [latest_mongod, latest_mongod]
-            num_versions = len(self.mixed_bin_versions)
-            if num_versions != self.num_nodes and not is_config_svr:
-                msg = (("The number of binary versions specified: {} do not match the number of"\
-                        " nodes in the replica set: {}.")).format(num_versions, self.num_nodes)
-                raise errors.ServerFailure(msg)
 
         # By default, we only use a replica set connection string if all nodes are capable of being
         # elected primary.
@@ -79,12 +86,12 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
             self.use_replica_set_connection_string = self.all_nodes_electable
 
         if self.default_write_concern is True:
-            self.default_write_concern = {
+            self.default_write_concern = self.fixturelib.make_historic({
                 "w": "majority",
                 # Use a "signature" value that won't typically match a value assigned in normal use.
                 # This way the wtimeout set by this override is distinguishable in the server logs.
                 "wtimeout": 5 * 60 * 1000 + 321,  # 300321ms
-            }
+            })
 
         # Set the default oplogSize to 511MB.
         self.mongod_options.setdefault("oplogSize", 511)
@@ -95,46 +102,27 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         if "dbpath" in self.mongod_options:
             self._dbpath_prefix = self.mongod_options.pop("dbpath")
         else:
-            self._dbpath_prefix = os.path.join(self._dbpath_prefix, config.FIXTURE_SUBDIR)
+            self._dbpath_prefix = os.path.join(self._dbpath_prefix, self.config.FIXTURE_SUBDIR)
 
         self.nodes = []
-        self.replset_name = None
+        if "serverless" not in self.mongod_options:
+            self.replset_name = self.mongod_options.setdefault("replSet", "rs")
         self.initial_sync_node = None
         self.initial_sync_node_idx = -1
 
-    def setup(self):  # pylint: disable=too-many-branches,too-many-statements
+    def setup(self):  # pylint: disable=too-many-branches,too-many-statements,too-many-locals
         """Set up the replica set."""
-        self.replset_name = self.mongod_options.get("replSet", "rs")
-        if not self.nodes:
-            for i in range(self.num_nodes):
-                node = self._new_mongod(i, self.replset_name)
-                self.nodes.append(node)
 
+        # Version-agnostic options for mongod/s can be set here.
+        # Version-specific options should be set in get_version_specific_options_for_mongod()
+        # to avoid options for old versions being applied to new Replicaset fixtures.
         for i in range(self.num_nodes):
-            if self.linear_chain and i > 0:
-                self.nodes[i].mongod_options["set_parameters"][
-                    "failpoint.forceSyncSourceCandidate"] = {
-                        "mode": "alwaysOn",
-                        "data": {"hostAndPort": self.nodes[i - 1].get_internal_connection_string()}
-                    }
             self.nodes[i].setup()
 
-        if self.start_initial_sync_node:
-            if not self.initial_sync_node:
-                self.initial_sync_node_idx = len(self.nodes)
-                self.initial_sync_node = self._new_mongod(self.initial_sync_node_idx,
-                                                          self.replset_name)
+        if self.initial_sync_node:
             self.initial_sync_node.setup()
             self.initial_sync_node.await_ready()
 
-        if self.mixed_bin_versions:
-            for i in range(self.num_nodes):
-                if self.nodes[i].mongod_executable != self.mixed_bin_versions[i]:
-                    msg = (f"Executable of node{i}: {self.nodes[i].mongod_executable} does not "
-                           f"match the executable assigned by mixedBinVersions: "
-                           f"{self.mixed_bin_versions[i]}.")
-                    raise errors.ServerFailure(msg)
-
         # We need only to wait to connect to the first node of the replica set because we first
         # initiate it as a single node replica set.
         self.nodes[0].await_ready()
@@ -160,11 +148,11 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
 
         repl_config = {"_id": self.replset_name, "protocolVersion": 1}
         client = self.nodes[0].mongo_client()
+        interface.authenticate(client, self.auth_options)
 
-        self.auth(client, self.auth_options)
-
-        if client.local.system.replset.count():
+        if client.local.system.replset.count_documents(filter={}):
             # Skip initializing the replset if there is an existing configuration.
+            self.logger.info("Configuration exists. Skipping initializing the replset.")
             return
 
         if self.write_concern_majority_journal_default is not None:
@@ -172,9 +160,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
                 "writeConcernMajorityJournalDefault"] = self.write_concern_majority_journal_default
         else:
             server_status = client.admin.command({"serverStatus": 1})
-            cmd_line_opts = client.admin.command({"getCmdLineOpts": 1})
-            if not (server_status["storageEngine"]["persistent"] and cmd_line_opts["parsed"].get(
-                    "storage", {}).get("journal", {}).get("enabled", True)):
+            if not server_status["storageEngine"]["persistent"]:
                 repl_config["writeConcernMajorityJournalDefault"] = False
 
         if self.replset_config_options.get("configsvr", False):
@@ -192,26 +178,19 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         # contains more than 1 node), so the primary is elected more quickly.
         repl_config["members"] = [members[0]]
         self.logger.info("Issuing replSetInitiate command: %s", repl_config)
-        self._configure_repl_set(client, {"replSetInitiate": repl_config})
+        self._initiate_repl_set(client, repl_config)
         self._await_primary()
 
-        if self.mixed_bin_versions is not None:
-            if self.mixed_bin_versions[0] == "new":
-                fcv_response = client.admin.command(
-                    {"getParameter": 1, "featureCompatibilityVersion": 1})
-                fcv = fcv_response["featureCompatibilityVersion"]["version"]
-                if fcv != ReplicaSetFixture._LATEST_FCV:
-                    msg = (("Server returned FCV{} when we expected FCV{}.").format(
-                        fcv, ReplicaSetFixture._LATEST_FCV))
-                    raise errors.ServerFailure(msg)
-
+        if self.fcv is not None:
             # Initiating a replica set with a single node will use "latest" FCV. This will
-            # cause IncompatibleServerVersion errors if additional "last-stable" binary version
+            # cause IncompatibleServerVersion errors if additional "last-lts" binary version
             # nodes are subsequently added to the set, since such nodes cannot set their FCV to
-            # "latest". Therefore, we make sure the primary is "last-stable" FCV before adding in
+            # "latest". Therefore, we make sure the primary is "last-lts" FCV before adding in
             # nodes of different binary versions to the replica set.
-            client.admin.command(
-                {"setFeatureCompatibilityVersion": ReplicaSetFixture._LAST_STABLE_FCV})
+            client.admin.command({
+                "setFeatureCompatibilityVersion": self.fcv,
+                "fromConfigServer": True,
+            })
 
         if self.nodes[1:]:
             # Wait to connect to each of the secondaries before running the replSetReconfig
@@ -220,18 +199,11 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
                 node.await_ready()
             # Add in the members one at a time, since non force reconfigs can only add/remove a
             # single voting member at a time.
-            repl_config["version"] = client.admin.command(
-                {"replSetGetConfig": 1})['config']['version']
             for ind in range(2, len(members) + 1):
-                repl_config["version"] = repl_config["version"] + 1
-                repl_config["members"] = members[:ind]
-                self.logger.info("Issuing replSetReconfig command: %s", repl_config)
-                self._configure_repl_set(
-                    client, {
-                        "replSetReconfig": repl_config,
-                        "maxTimeMS": self.AWAIT_REPL_TIMEOUT_MINS * 60 * 1000
-                    })
+                self._add_node_to_repl_set(client, repl_config, ind, members)
+
         self._await_secondaries()
+        self._await_newly_added_removals()
 
     def pids(self):
         """:return: all pids owned by this fixture if any."""
@@ -242,14 +214,49 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
             self.logger.debug('No members running when gathering replicaset fixture pids.')
         return pids
 
-    def _configure_repl_set(self, client, cmd_obj):
-        # replSetInitiate and replSetReconfig commands can fail with a NodeNotFound error
+    def _add_node_to_repl_set(self, client, repl_config, member_index, members):
+        self.logger.info("Adding in node %d: %s", member_index, members[member_index - 1])
+        while True:
+            try:
+                # 'newlyAdded' removal reconfigs could bump the version.
+                # Get the current version to be safe.
+                curr_version = client.admin.command({"replSetGetConfig": 1})['config']['version']
+                repl_config["version"] = curr_version + 1
+                repl_config["members"] = members[:member_index]
+                self.logger.info("Issuing replSetReconfig command: %s", repl_config)
+                client.admin.command({
+                    "replSetReconfig": repl_config,
+                    "maxTimeMS": self.AWAIT_REPL_TIMEOUT_MINS * 60 * 1000
+                })
+                break
+            except pymongo.errors.OperationFailure as err:
+                # These error codes may be transient, and so we retry the reconfig with a
+                # (potentially) higher config version. We should not receive these codes
+                # indefinitely.
+                # pylint: disable=too-many-boolean-expressions
+                if (err.code != ReplicaSetFixture._NEW_REPLICA_SET_CONFIGURATION_INCOMPATIBLE
+                        and err.code != ReplicaSetFixture._CURRENT_CONFIG_NOT_COMMITTED_YET
+                        and err.code != ReplicaSetFixture._CONFIGURATION_IN_PROGRESS
+                        and err.code != ReplicaSetFixture._NODE_NOT_FOUND
+                        and err.code != ReplicaSetFixture._INTERRUPTED_DUE_TO_REPL_STATE_CHANGE
+                        and err.code != ReplicaSetFixture._INTERRUPTED_DUE_TO_STORAGE_CHANGE):
+                    msg = ("Operation failure while setting up the "
+                           "replica set fixture: {}").format(err)
+                    self.logger.error(msg)
+                    raise self.fixturelib.ServerFailure(msg)
+
+                msg = ("Retrying failed attempt to add new node to fixture: {}").format(err)
+                self.logger.error(msg)
+                time.sleep(0.1)  # Wait a little bit before trying again.
+
+    def _initiate_repl_set(self, client, repl_config):
+        # replSetInitiate (and replSetReconfig) commands can fail with a NodeNotFound error
         # if a heartbeat times out during the quorum check. We retry three times to reduce
         # the chance of failing this way.
         num_initiate_attempts = 3
         for attempt in range(1, num_initiate_attempts + 1):
             try:
-                client.admin.command(cmd_obj)
+                client.admin.command({"replSetInitiate": repl_config})
                 break
             except pymongo.errors.OperationFailure as err:
                 # Retry on NodeNotFound errors from the "replSetInitiate" command.
@@ -257,7 +264,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
                     msg = ("Operation failure while configuring the "
                            "replica set fixture: {}").format(err)
                     self.logger.error(msg)
-                    raise errors.ServerFailure(msg)
+                    raise self.fixturelib.ServerFailure(msg)
 
                 msg = "replSetInitiate failed attempt {0} of {1} with error: {2}".format(
                     attempt, num_initiate_attempts, err)
@@ -265,15 +272,15 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
                 if attempt == num_initiate_attempts:
                     msg = "Exceeded number of retries while configuring the replica set fixture"
                     self.logger.error(msg + ".")
-                    raise errors.ServerFailure(msg)
+                    raise self.fixturelib.ServerFailure(msg)
                 time.sleep(5)  # Wait a little bit before trying again.
 
     def await_last_op_committed(self, timeout_secs=None):
         """Wait for the last majority committed op to be visible."""
         primary_client = self.get_primary().mongo_client()
-        self.auth(primary_client, self.auth_options)
+        interface.authenticate(primary_client, self.auth_options)
 
-        primary_optime = replicaset_utils.get_last_optime(primary_client)
+        primary_optime = get_last_optime(primary_client, self.fixturelib)
         up_to_date_nodes = set()
 
         def check_rcmaj_optime(client, node):
@@ -324,22 +331,16 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
             while True:
                 self.logger.info("Waiting for secondary on port %d to become available.",
                                  secondary.port)
-                is_secondary = client.admin.command("isMaster")["secondary"]
-                if is_secondary:
-                    break
+                try:
+                    is_secondary = client.admin.command("isMaster")["secondary"]
+                    if is_secondary:
+                        break
+                except pymongo.errors.OperationFailure as err:
+                    if err.code != ReplicaSetFixture._INTERRUPTED_DUE_TO_STORAGE_CHANGE:
+                        raise
                 time.sleep(0.1)  # Wait a little bit before trying again.
             self.logger.info("Secondary on port %d is now available.", secondary.port)
 
-    @staticmethod
-    def auth(client, auth_options=None):
-        """Auth a client connection."""
-        if auth_options is not None:
-            auth_db = client[auth_options["authenticationDatabase"]]
-            auth_db.authenticate(auth_options["username"], password=auth_options["password"],
-                                 mechanism=auth_options["authenticationMechanism"])
-
-        return client
-
     def _await_stable_recovery_timestamp(self):
         """
         Awaits stable recovery timestamps on all nodes in the replica set.
@@ -357,7 +358,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         # Since this method is called at startup we expect the first node to be primary even when
         # self.all_nodes_electable is True.
         primary_client = self.nodes[0].mongo_client()
-        self.auth(primary_client, self.auth_options)
+        interface.authenticate(primary_client, self.auth_options)
 
         # All nodes must be in primary/secondary state prior to this point. Perform a majority
         # write to ensure there is a committed operation on the set. The commit point will
@@ -371,7 +372,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
             self.logger.info("Waiting for node on port %d to have a stable recovery timestamp.",
                              node.port)
             client = node.mongo_client(read_preference=pymongo.ReadPreference.SECONDARY)
-            self.auth(client, self.auth_options)
+            interface.authenticate(client, self.auth_options)
 
             client_admin = client["admin"]
 
@@ -396,6 +397,44 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
                     break
                 time.sleep(0.1)  # Wait a little bit before trying again.
 
+    def _should_await_newly_added_removals_longer(self, client):
+        """
+        Return whether the current replica set config has any 'newlyAdded' fields.
+
+        Return true if the current config is not committed.
+        """
+
+        get_config_res = client.admin.command(
+            {"replSetGetConfig": 1, "commitmentStatus": True, "$_internalIncludeNewlyAdded": True})
+        for member in get_config_res["config"]["members"]:
+            if "newlyAdded" in member:
+                self.logger.info(
+                    "Waiting longer for 'newlyAdded' removals, " +
+                    "member %d is still 'newlyAdded'", member["_id"])
+                return True
+        if not get_config_res["commitmentStatus"]:
+            self.logger.info("Waiting longer for 'newlyAdded' removals, " +
+                             "config is not yet committed")
+            return True
+
+        return False
+
+    def _await_newly_added_removals(self):
+        """
+        Wait for all 'newlyAdded' fields to be removed from the replica set config.
+
+        Additionally, wait for that config to be committed, and for the in-memory
+        and on-disk configs to match.
+        """
+
+        self.logger.info("Waiting to remove all 'newlyAdded' fields")
+        primary = self.get_primary()
+        client = primary.mongo_client()
+        interface.authenticate(client, self.auth_options)
+        while self._should_await_newly_added_removals_longer(client):
+            time.sleep(0.1)  # Wait a little bit before trying again.
+        self.logger.info("All 'newlyAdded' fields removed")
+
     def _setup_cwrwc_defaults(self):
         """Set up the cluster-wide read/write concern defaults."""
         if self.default_read_concern is None and self.default_write_concern is None:
@@ -429,7 +468,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
             self.logger.info("Successfully stopped all members of the replica set.")
         else:
             self.logger.error("Stopping the replica set fixture failed.")
-            raise errors.ServerFailure(teardown_handler.get_error_message())
+            raise self.fixturelib.ServerFailure(teardown_handler.get_error_message())
 
     def is_running(self):
         """Return True if all nodes in the replica set are running."""
@@ -477,11 +516,12 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
                 if (now - start) >= timeout_secs:
                     msg = "Timed out while {} for replica set '{}'.".format(msg, self.replset_name)
                     self.logger.error(msg)
-                    raise errors.ServerFailure(msg)
+                    raise self.fixturelib.ServerFailure(msg)
 
                 try:
                     if node.port not in clients:
-                        clients[node.port] = self.auth(node.mongo_client(), self.auth_options)
+                        clients[node.port] = interface.authenticate(node.mongo_client(),
+                                                                    self.auth_options)
 
                     if fn(clients[node.port], node):
                         return node
@@ -492,11 +532,164 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
                     # of isMaster requests.
                     continue
 
+    def stop_primary(self, primary, background_reconfig, kill):
+        """Stop the primary node method."""
+        # Check that the fixture is still running before stepping down or killing the primary.
+        # This ensures we still detect some cases in which the fixture has already crashed.
+        if not self.is_running():
+            raise self.fixturelib.ServerFailure("ReplicaSetFixture {} expected to be running in"
+                                                " ContinuousStepdown, but wasn't.".format(
+                                                    self.replset_name))
+
+        # If we're running with background reconfigs, it's possible to be in a scenario
+        # where we kill a necessary voting node (i.e. in a 5 node repl set), only 2 are
+        # voting. In this scenario, we want to avoid killing the primary because no
+        # secondary can step up.
+        if background_reconfig:
+            # stagger the kill thread so that it runs a little after the reconfig thread
+            time.sleep(1)
+            voting_members = self.get_voting_members()
+
+            self.logger.info("Current voting members: %s", voting_members)
+
+            if len(voting_members) <= 3:
+                # Do not kill or terminate the primary if we don't have enough voting nodes to
+                # elect a new primary.
+                return False
+
+        should_kill = kill and random.choice([True, False])
+        action = "Killing" if should_kill else "Terminating"
+        self.logger.info("%s the primary on port %d of replica set '%s'.", action, primary.port,
+                         self.replset_name)
+
+        # We send the mongod process the signal to exit but don't immediately wait for it to
+        # exit because clean shutdown may take a while and we want to restore write availability
+        # as quickly as possible.
+        teardown_mode = interface.TeardownMode.KILL if should_kill else interface.TeardownMode.TERMINATE
+        primary.mongod.stop(mode=teardown_mode)
+        return True
+
+    def change_version_and_restart_node(self, primary, auth_options):
+        """
+        Select Secondary for stepUp.
+
+        Ensure its version is different to that
+        of the old primary; change the version of the Secondary is needed.
+        """
+
+        def get_chosen_node_from_replsetstatus(status_member_infos):
+            max_optime = None
+            chosen_index = None
+            # We always select the secondary with highest optime to setup.
+            for member_info in status_member_infos:
+                if member_info.get("self", False):
+                    # Ignore self, which is the old primary and not eligible
+                    # to be re-elected in downgrade multiversion cluster.
+                    continue
+                optime_dict = member_info["optime"]
+                if max_optime is None:
+                    chosen_index = member_info["_id"]
+                    max_optime = optime_dict
+                else:
+                    if compare_optime(optime_dict, max_optime) > 0:
+                        chosen_index = member_info["_id"]
+                        max_optime = optime_dict
+
+                if chosen_index is None or max_optime is None:
+                    raise self.fixturelib.ServerFailure(
+                        "Failed to find a secondary eligible for "
+                        f"election; index: {chosen_index}, optime: {max_optime}")
+
+                return self.nodes[chosen_index]
+
+        primary_client = interface.authenticate(primary.mongo_client(), auth_options)
+        retry_time_secs = self.AWAIT_REPL_TIMEOUT_MINS * 60
+        retry_start_time = time.time()
+
+        while True:
+            member_infos = primary_client.admin.command({"replSetGetStatus": 1})["members"]
+            chosen_node = get_chosen_node_from_replsetstatus(member_infos)
+
+            if chosen_node.change_version_if_needed(primary):
+                self.logger.info(
+                    "Waiting for the chosen secondary on port %d of replica set '%s' to exit.",
+                    chosen_node.port, self.replset_name)
+
+                teardown_mode = interface.TeardownMode.TERMINATE
+                chosen_node.mongod.stop(mode=teardown_mode)
+                chosen_node.mongod.wait()
+
+                self.logger.info(
+                    "Attempting to restart the chosen secondary on port %d of replica set '%s'.",
+                    chosen_node.port, self.replset_name)
+
+                chosen_node.setup()
+                self.logger.info(interface.create_fixture_table(self))
+                chosen_node.await_ready()
+
+            if self.stepup_node(chosen_node, auth_options):
+                break
+
+            if time.time() - retry_start_time > retry_time_secs:
+                raise self.fixturelib.ServerFailure(
+                    "The old primary on port {} of replica set {} did not step up in"
+                    " {} seconds.".format(chosen_node.port, self.replset_name, retry_time_secs))
+
+        return chosen_node
+
+    def stepup_node(self, node, auth_options):
+        """Try to step up the given node; return whether the attempt was successful."""
+        try:
+            self.logger.info(
+                "Attempting to step up the chosen secondary on port %d of replica set '%s'.",
+                node.port, self.replset_name)
+            client = interface.authenticate(node.mongo_client(), auth_options)
+            client.admin.command("replSetStepUp")
+            return True
+        except pymongo.errors.OperationFailure:
+            # OperationFailure exceptions are expected when the election attempt fails due to
+            # not receiving enough votes. This can happen when the 'chosen' secondary's opTime
+            # is behind that of other secondaries. We handle this by attempting to elect a
+            # different secondary.
+            self.logger.info("Failed to step up the secondary on port %d of replica set '%s'.",
+                             node.port, self.replset_name)
+            return False
+        except pymongo.errors.AutoReconnect:
+            # It is possible for a replSetStepUp to fail with AutoReconnect if that node goes
+            # into Rollback (which causes it to close any open connections).
+            return False
+
+    def restart_node(self, chosen):
+        """Restart the new step up node."""
+        self.logger.info("Waiting for the old primary on port %d of replica set '%s' to exit.",
+                         chosen.port, self.replset_name)
+
+        chosen.mongod.wait()
+
+        self.logger.info("Attempting to restart the old primary on port %d of replica set '%s'.",
+                         chosen.port, self.replset_name)
+
+        # Restart the mongod on the old primary and wait until we can contact it again. Keep the
+        # original preserve_dbpath to restore after restarting the mongod.
+        original_preserve_dbpath = chosen.preserve_dbpath
+        chosen.preserve_dbpath = True
+        try:
+            chosen.setup()
+            self.logger.info(interface.create_fixture_table(self))
+            chosen.await_ready()
+        finally:
+            chosen.preserve_dbpath = original_preserve_dbpath
+
     def get_secondaries(self):
         """Return a list of secondaries from the replica set."""
         primary = self.get_primary()
         return [node for node in self.nodes if node.port != primary.port]
 
+    def get_secondary_indices(self):
+        """Return a list of secondary indices from the replica set."""
+        primary = self.get_primary()
+        return [index for index, node in enumerate(self.nodes) if node.port != primary.port]
+
     def get_voting_members(self):
         """Return the number of voting nodes in the replica set."""
         primary = self.get_primary()
@@ -511,21 +704,32 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         """Return initial sync node from the replica set."""
         return self.initial_sync_node
 
-    def _new_mongod(self, index, replset_name):
-        """Return a standalone.MongoDFixture configured to be used as replica-set member."""
-        mongod_executable = None if self.mixed_bin_versions is None else self.mixed_bin_versions[
-            index]
-        mongod_logger = self._get_logger_for_mongod(index)
+    def set_fcv(self, fcv):
+        """Set the fcv used by this fixtures."""
+        self.fcv = fcv
+
+    def install_mongod(self, mongod):
+        """Install a mongod node. Called by a builder."""
+        self.nodes.append(mongod)
+
+    def get_options_for_mongod(self, index):
+        """Return options that may be passed to a mongod."""
         mongod_options = self.mongod_options.copy()
-        mongod_options["replSet"] = replset_name
+
         mongod_options["dbpath"] = os.path.join(self._dbpath_prefix, "node{}".format(index))
-        mongod_options["set_parameters"] = mongod_options.get("set_parameters", {}).copy()
+        mongod_options["set_parameters"] = mongod_options.get("set_parameters",
+                                                              self.fixturelib.make_historic(
+                                                                  {})).copy()
 
-        return standalone.MongoDFixture(
-            mongod_logger, self.job_num, mongod_executable=mongod_executable,
-            mongod_options=mongod_options, preserve_dbpath=self.preserve_dbpath)
+        if self.linear_chain and index > 0:
+            self.mongod_options["set_parameters"][
+                "failpoint.forceSyncSourceCandidate"] = self.fixturelib.make_historic({
+                    "mode": "alwaysOn",
+                    "data": {"hostAndPort": self.nodes[index - 1].get_internal_connection_string()}
+                })
+        return mongod_options
 
-    def _get_logger_for_mongod(self, index):
+    def get_logger_for_mongod(self, index):
         """Return a new logging.Logger instance.
 
         The instance is used as the primary, secondary, or initial sync member of a replica-set.
@@ -541,23 +745,35 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
             suffix = str(index - 1) if self.num_nodes > 2 else ""
             node_name = "secondary{}".format(suffix)
 
-        return self.logger.new_fixture_node_logger(node_name)
+        if self.shard_logging_prefix is not None:
+            node_name = f"{self.shard_logging_prefix}:{node_name}"
+            return self.fixturelib.new_fixture_node_logger("ShardedClusterFixture", self.job_num,
+                                                           node_name)
+
+        if self.replicaset_logging_prefix is not None:
+            node_name = f"{self.replicaset_logging_prefix}:{node_name}"
+
+        return self.fixturelib.new_fixture_node_logger(self.__class__.__name__, self.job_num,
+                                                       node_name)
 
     def get_internal_connection_string(self):
         """Return the internal connection string."""
-        if self.replset_name is None:
-            raise ValueError("Must call setup() before calling get_internal_connection_string()")
-
         conn_strs = [node.get_internal_connection_string() for node in self.nodes]
         if self.initial_sync_node:
             conn_strs.append(self.initial_sync_node.get_internal_connection_string())
         return self.replset_name + "/" + ",".join(conn_strs)
 
+    def get_node_info(self):
+        """Return a list of dicts of NodeInfo objects."""
+        output = []
+        for node in self.nodes:
+            output += node.get_node_info()
+        if self.initial_sync_node:
+            output += self.initial_sync_node.get_node_info()
+        return output
+
     def get_driver_connection_url(self):
         """Return the driver connection URL."""
-        if self.replset_name is None:
-            raise ValueError("Must call setup() before calling get_driver_connection_url()")
-
         if self.use_replica_set_connection_string:
             # We use a replica set connection string when all nodes are electable because we
             # anticipate the client will want to gracefully handle any failovers.
@@ -569,3 +785,31 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
             # We return a direct connection to the expected pimary when only the first node is
             # electable because we want the client to error out if a stepdown occurs.
             return self.nodes[0].get_driver_connection_url()
+
+    def write_historic(self, obj):
+        """Convert the obj to a record to track history."""
+        self.fixturelib.make_historic(obj)
+
+
+def get_last_optime(client, fixturelib):
+    """Get the latest optime.
+
+    This function is derived from _getLastOpTime() in ReplSetTest.
+    """
+    repl_set_status = client.admin.command({"replSetGetStatus": 1})
+    conn_status = [m for m in repl_set_status["members"] if "self" in m][0]
+    optime = conn_status["optime"]
+
+    optime_is_empty = False
+
+    if isinstance(optime, bson.Timestamp):  # PV0
+        optime_is_empty = (optime == bson.Timestamp(0, 0))
+    else:  # PV1
+        optime_is_empty = (optime["ts"] == bson.Timestamp(0, 0) and optime["t"] == -1)
+
+    if optime_is_empty:
+        raise fixturelib.ServerFailure(
+            "Uninitialized opTime being reported by {addr[0]}:{addr[1]}: {repl_set_status}".format(
+                addr=client.address, repl_set_status=repl_set_status))
+
+    return optime
