@@ -8,15 +8,19 @@
 // handle explain output for sharded collections.
 // This test makes assumptions about how the explain output will be formatted, so cannot be run when
 // pipeline optimization is disabled.
-// @tags: [do_not_wrap_aggregations_in_facets, assumes_unsharded_collection,
-//         requires_pipeline_optimization, requires_profiling]
+// @tags: [
+//   assumes_unsharded_collection,
+//   do_not_wrap_aggregations_in_facets,
+//   requires_pipeline_optimization,
+//   requires_profiling,
+// ]
 (function() {
 "use strict";
 
-load("jstests/aggregation/extras/utils.js");  // For 'orderedArrayEq' and 'arrayEq'.
 load("jstests/concurrency/fsm_workload_helpers/server_types.js");  // For isWiredTiger.
 load("jstests/libs/analyze_plan.js");     // For 'aggPlanHasStage' and other explain helpers.
 load("jstests/libs/fixture_helpers.js");  // For 'isMongos' and 'isSharded'.
+load("jstests/libs/sbe_util.js");         // For checkSBEEnabled.
 
 const coll = db.optimize_away_pipeline;
 coll.drop();
@@ -71,8 +75,11 @@ function assertPipelineUsesAggregation({
 
     if (expectedResult) {
         const actualResult = coll.aggregate(pipeline, pipelineOptions).toArray();
-        assert(preserveResultOrder ? orderedArrayEq(actualResult, expectedResult)
-                                   : arrayEq(actualResult, expectedResult));
+        if (preserveResultOrder) {
+            assert.docEq(actualResult, expectedResult);
+        } else {
+            assert.sameMembers(actualResult, expectedResult);
+        }
     }
 
     return explainOutput;
@@ -112,8 +119,11 @@ function assertPipelineDoesNotUseAggregation({
 
     if (expectedResult) {
         const actualResult = coll.aggregate(pipeline, pipelineOptions).toArray();
-        assert(preserveResultOrder ? orderedArrayEq(actualResult, expectedResult)
-                                   : arrayEq(actualResult, expectedResult));
+        if (preserveResultOrder) {
+            assert.docEq(actualResult, expectedResult);
+        } else {
+            assert.sameMembers(actualResult, expectedResult);
+        }
     }
 
     return explainOutput;
@@ -124,7 +134,15 @@ function testGetMore({command = null, expectedResult = null} = {}) {
     const documents =
         new DBCommandCursor(db, assert.commandWorked(db.runCommand(command)), 1 /* batchsize */)
             .toArray();
-    assert(arrayEq(documents, expectedResult));
+    assert.sameMembers(documents, expectedResult);
+}
+
+const groupPushdownEnabled = checkSBEEnabled(db);
+
+// Calls 'assertPushdownEnabled' if groupPushdownEnabled is 'true'. Otherwise, it calls
+// 'assertPushdownDisabled'.
+function assertPipelineIfGroupPushdown(assertPushdownEnabled, assertPushdownDisabled) {
+    return groupPushdownEnabled ? assertPushdownEnabled() : assertPushdownDisabled();
 }
 
 let explainOutput;
@@ -176,6 +194,19 @@ assertPipelineDoesNotUseAggregation({
     expectedStages: ["IXSCAN"],
     expectedResult: [{x: 20}]
 });
+// However, when the $project is computed, pushing it down into the find() layer would sometimes
+// have the effect of reordering it before the $sort and $limit. This can cause a valid query to
+// throw an error, as in SERVER-54128.
+assertPipelineUsesAggregation({
+    pipeline: [
+        {$match: {x: {$gte: 20}}},
+        {$sort: {x: 1}},
+        {$limit: 1},
+        {$project: {x: {$substr: ["$y", 0, 1]}, _id: 0}}
+    ],
+    expectedStages: ["IXSCAN"],
+    expectedResult: [{x: ""}]
+});
 assert.commandWorked(coll.dropIndexes());
 
 assert.commandWorked(coll.insert({_id: 4, x: 40, a: {b: "ab1"}}));
@@ -210,14 +241,14 @@ assertPipelineDoesNotUseAggregation({
 });
 assert.commandWorked(coll.deleteOne({_id: 4}));
 
-// Pipelines which cannot be optimized away.
-
-// TODO SERVER-40909: $skip stage is not supported yet.
-assertPipelineUsesAggregation({
+assertPipelineDoesNotUseAggregation({
     pipeline: [{$match: {x: {$gte: 20}}}, {$skip: 1}],
-    expectedStages: ["COLLSCAN"],
+    expectedStages: ["COLLSCAN", "SKIP"],
     expectedResult: [{_id: 3, x: 30}]
 });
+
+// Pipelines which cannot be optimized away.
+
 // We cannot optimize away a pipeline if there are stages which have no equivalent in the
 // find command.
 assertPipelineUsesAggregation({
@@ -225,11 +256,22 @@ assertPipelineUsesAggregation({
     expectedStages: ["COLLSCAN"],
     expectedResult: [{count: 2}]
 });
-assertPipelineUsesAggregation({
-    pipeline: [{$match: {x: {$gte: 20}}}, {$group: {_id: "null", s: {$sum: "$x"}}}],
-    expectedStages: ["COLLSCAN"],
-    expectedResult: [{_id: "null", s: 50}]
-});
+
+assertPipelineIfGroupPushdown(
+    function() {
+        return assertPipelineDoesNotUseAggregation({
+            pipeline: [{$match: {x: {$gte: 20}}}, {$group: {_id: "null", s: {$sum: "$x"}}}],
+            expectedStages: ["COLLSCAN", "GROUP"],
+            expectedResult: [{_id: "null", s: 50}],
+        });
+    },
+    function() {
+        return assertPipelineUsesAggregation({
+            pipeline: [{$match: {x: {$gte: 20}}}, {$group: {_id: "null", s: {$sum: "$x"}}}],
+            expectedStages: ["COLLSCAN", "PROJECTION_SIMPLE"],
+            expectedResult: [{_id: "null", s: 50}],
+        });
+    });
 
 // Test that we can optimize away a pipeline with a $text search predicate.
 assert.commandWorked(coll.createIndex({y: "text"}));
@@ -239,7 +281,7 @@ assertPipelineDoesNotUseAggregation(
 assertPipelineDoesNotUseAggregation({
     pipeline:
         [{$match: {$text: {$search: "abc"}}}, {$sort: {sortField: 1}}, {$project: {a: 1, b: 1}}],
-    expectedStages: ["TEXT", "SORT", "PROJECTION_SIMPLE"],
+    expectedStages: ["TEXT_MATCH", "SORT", "PROJECTION_SIMPLE"],
     optimizedAwayStages: ["$match", "$sort", "$project"]
 });
 assert.commandWorked(coll.dropIndexes());
@@ -337,13 +379,23 @@ assertPipelineDoesNotUseAggregation({
     expectedResult: [{x: 30}, {x: 20}],
 });
 
-// For $sort, $limit, $group, the $sort and $limit can be pushed down, but $group cannot.
-assertPipelineUsesAggregation({
-    pipeline: [{$sort: {x: 1}}, {$limit: 2}, {$group: {_id: null, s: {$sum: "$x"}}}],
-    expectedStages: ["IXSCAN", "PROJECTION_COVERED", "LIMIT"],
-    expectedResult: [{_id: null, s: 30}],
-    optimizedAwayStages: ["$sort", "$limit"],
-});
+let pipeline = [{$sort: {x: 1}}, {$limit: 2}, {$group: {_id: null, s: {$sum: "$x"}}}];
+assertPipelineIfGroupPushdown(
+    function() {
+        return assertPipelineDoesNotUseAggregation({
+            pipeline: pipeline,
+            expectedStages: ["IXSCAN", "PROJECTION_COVERED", "LIMIT", "GROUP"],
+            expectedResult: [{_id: null, s: 30}],
+        });
+    },
+    function() {
+        return assertPipelineUsesAggregation({
+            pipeline: pipeline,
+            expectedStages: ["IXSCAN", "PROJECTION_COVERED", "LIMIT"],
+            expectedResult: [{_id: null, s: 30}],
+            optimizedAwayStages: ["$sort", "$limit"],
+        });
+    });
 
 // Test that $limit can be pushed down before a group, but it prohibits the DISTINCT_SCAN
 // optimization.
@@ -352,23 +404,44 @@ assertPipelineUsesAggregation({
     expectedStages: ["DISTINCT_SCAN", "PROJECTION_COVERED"],
     expectedResult: [{_id: 10}, {_id: 20}, {_id: 30}],
 });
-assertPipelineUsesAggregation({
-    pipeline: [{$limit: 2}, {$group: {_id: "$x"}}],
-    expectedStages: ["COLLSCAN", "LIMIT"],
-    optimizedAwayStages: ["$limit"],
-});
 assertPipelineUsesAggregation({
     pipeline: [{$sort: {x: 1}}, {$group: {_id: "$x"}}],
     expectedStages: ["DISTINCT_SCAN", "PROJECTION_COVERED"],
     expectedResult: [{_id: 10}, {_id: 20}, {_id: 30}],
     optimizedAwayStages: ["$sort"],
 });
-assertPipelineUsesAggregation({
-    pipeline: [{$sort: {x: 1}}, {$limit: 2}, {$group: {_id: "$x"}}],
-    expectedResult: [{_id: 10}, {_id: 20}],
-    expectedStages: ["IXSCAN", "LIMIT"],
-    optimizedAwayStages: ["$sort", "$limit"],
-});
+
+pipeline = [{$limit: 2}, {$group: {_id: "$x"}}];
+assertPipelineIfGroupPushdown(
+    function() {
+        return assertPipelineDoesNotUseAggregation({
+            pipeline: pipeline,
+            expectedStages: ["COLLSCAN", "LIMIT", "GROUP"],
+        });
+    },
+    function() {
+        return assertPipelineUsesAggregation({
+            pipeline: pipeline,
+            expectedStages: ["COLLSCAN", "LIMIT"],
+            optimizedAwayStages: ["$limit"],
+        });
+    });
+
+pipeline = [{$sort: {x: 1}}, {$limit: 2}, {$group: {_id: "$x"}}];
+assertPipelineIfGroupPushdown(
+    function() {
+        return assertPipelineDoesNotUseAggregation({
+            pipeline: pipeline,
+            expectedStages: ["IXSCAN", "PROJECTION_COVERED", "LIMIT", "GROUP"],
+        });
+    },
+    function() {
+        return assertPipelineUsesAggregation({
+            pipeline: pipeline,
+            expectedStages: ["IXSCAN", "PROJECTION_COVERED", "LIMIT"],
+            optimizedAwayStages: ["$sort", "$limit"],
+        });
+    });
 
 // $limit after a group has no effect on our ability to produce a DISTINCT_SCAN plan.
 assertPipelineUsesAggregation({
@@ -378,7 +451,7 @@ assertPipelineUsesAggregation({
 });
 
 // For $limit, $project, $limit, we can optimize away both $limit stages.
-let pipeline = [{$match: {x: {$gte: 0}}}, {$limit: 2}, {$project: {_id: 0, x: 1}}, {$limit: 1}];
+pipeline = [{$match: {x: {$gte: 0}}}, {$limit: 2}, {$project: {_id: 0, x: 1}}, {$limit: 1}];
 assertPipelineDoesNotUseAggregation({
     pipeline: pipeline,
     expectedStages: ["IXSCAN", "PROJECTION_COVERED", "LIMIT"],
@@ -389,10 +462,7 @@ let limitStage = getAggPlanStage(explain, "LIMIT");
 assert.neq(null, limitStage, explain);
 assert.eq(1, limitStage.limitAmount, explain);
 
-// We can optimize away interleaved $limit and $skip after a project. The $limits can be collapsed
-// into a single $limit:35 prior to the $skip stages. We currently do not push down $skip into the
-// PlanStage layer (see SERVER-40909), which prevents this pipeline from being entirely optimized
-// away.
+// We can optimize away interleaved $limit and $skip after a project.
 pipeline = [
     {$match: {x: {$gte: 0}}},
     {$project: {_id: 0, x: 1}},
@@ -401,18 +471,20 @@ pipeline = [
     {$skip: 10},
     {$limit: 7}
 ];
-assertPipelineUsesAggregation({
+assertPipelineDoesNotUseAggregation({
     pipeline: pipeline,
-    expectedStages: ["IXSCAN", "PROJECTION_COVERED", "LIMIT"],
-    optimizedAwayStages: ["$match", "$limit"],
+    expectedStages: ["IXSCAN", "PROJECTION_COVERED", "LIMIT", "SKIP"],
+    optimizedAwayStages: ["$match", "$limit", "$skip"],
 });
 explain = coll.explain().aggregate(pipeline);
+
+let skipStage = getAggPlanStage(explain, "SKIP");
+assert.neq(null, skipStage, explain);
+assert.eq(30, skipStage.skipAmount, explain);
+
 limitStage = getAggPlanStage(explain, "LIMIT");
 assert.neq(null, limitStage, explain);
-assert.eq(35, limitStage.limitAmount, explain);
-let skipStage = getAggPlanStage(explain, "$skip");
-assert.neq(null, skipStage, explain);
-assert.eq(30, skipStage.$skip, explain);
+assert.eq(5, limitStage.limitAmount, explain);
 
 assert.commandWorked(coll.dropIndexes());
 
@@ -458,14 +530,60 @@ assert.commandWorked(coll.dropIndexes());
 // Test that even if we don't have a projection stage at the front of the pipeline but there is a
 // finite dependency set, a projection representing this dependency set is pushed down.
 pipeline = [{$group: {_id: "$a", b: {$sum: "$b"}}}];
-assertPipelineUsesAggregation({
-    pipeline: pipeline,
-    expectedStages: ["COLLSCAN", "PROJECTION_SIMPLE"],
-});
-explain = coll.explain().aggregate(pipeline);
-let projStage = getAggPlanStage(explain, "PROJECTION_SIMPLE");
-assert.neq(null, projStage, explain);
-assert.eq({a: 1, b: 1, _id: 0}, projStage.transformBy, explain);
+assertPipelineIfGroupPushdown(
+    function() {
+        return assertPipelineDoesNotUseAggregation({
+            pipeline: pipeline,
+            expectedStages: ["COLLSCAN", "GROUP"],
+        });
+    },
+    function() {
+        return assertPipelineUsesAggregation({
+            pipeline: pipeline,
+            expectedStages: ["COLLSCAN", "PROJECTION_SIMPLE"],
+        });
+    });
+pipeline = [{$group: {_id: "$a", b: {$sum: "$b"}}}, {$group: {_id: "$c", x: {$sum: "$b"}}}];
+assertPipelineIfGroupPushdown(
+    function() {
+        const explain = coll.explain().aggregate(pipeline);
+        // Both $group must be pushed down.
+        assert.eq(getPlanStages(explain, "GROUP").length, 2);
+        // PROJECTION_SIMPLE must be optimized away.
+        assert.eq(getPlanStages(explain, "PROJECTION_SIMPLE").length, 0);
+        // At bottom, there must be a COLLSCAN.
+        assert.eq(getPlanStages(explain, "COLLSCAN").length, 1);
+    },
+    function() {
+        return assertPipelineUsesAggregation({
+            pipeline: pipeline,
+            expectedStages: ["COLLSCAN", "PROJECTION_SIMPLE"],
+        });
+    });
+
+function assertTransformByShape(expected, actual, message) {
+    assert.eq(Object.keys(expected).sort(), Object.keys(actual).sort(), message);
+    for (let key in expected) {
+        assert.eq(expected[key], actual[key]);
+    }
+}
+
+assertPipelineIfGroupPushdown(
+    // When $group pushdown is enabled, $group will be lowered and the PROJECTION_SIMPLE will be
+    // erased.
+    function() {
+        explain = coll.explain().aggregate(pipeline);
+        let projStage = getAggPlanStage(explain, "PROJECTION_SIMPLE");
+        assert.eq(null, projStage, explain);
+    },
+    // When $group pushdown is disabled, $group will not be lowered and the PROJECTION_SIMPLE will
+    // be preserved.
+    function() {
+        explain = coll.explain().aggregate(pipeline);
+        let projStage = getAggPlanStage(explain, "PROJECTION_SIMPLE");
+        assert.neq(null, projStage, explain);
+        assertTransformByShape({a: 1, b: 1, _id: 0}, projStage.transformBy, explain);
+    });
 
 // Similar as above, but with $addFields stage at the front of the pipeline.
 pipeline = [{$addFields: {z: "abc"}}, {$group: {_id: "$a", b: {$sum: "$b"}}}];
@@ -474,9 +592,79 @@ assertPipelineUsesAggregation({
     expectedStages: ["COLLSCAN", "PROJECTION_SIMPLE"],
 });
 explain = coll.explain().aggregate(pipeline);
-projStage = getAggPlanStage(explain, "PROJECTION_SIMPLE");
+let projStage = getAggPlanStage(explain, "PROJECTION_SIMPLE");
 assert.neq(null, projStage, explain);
-assert.eq({a: 1, b: 1, _id: 0}, projStage.transformBy, explain);
+assertTransformByShape({a: 1, b: 1, _id: 0}, projStage.transformBy, explain);
+
+// Asserts that, if group pushdown is enabled, we can remove a redundant projection stage before a
+// group.
+function assertProjectionCanBeRemovedBeforeGroup(pipeline, projectionType = "PROJECTION_SIMPLE") {
+    assertPipelineIfGroupPushdown(
+        // The projection and group should both be pushed down, and we expect to optimize away the
+        // projection after realizing that it will not affect the output of the group.
+        function() {
+            let explain = assertPipelineDoesNotUseAggregation(
+                {pipeline: pipeline, expectedStages: ["COLLSCAN", "GROUP"]});
+            assert(!planHasStage(db, explain, projectionType), explain);
+        },
+        // If group pushdown is not enabled we still expect the projection to be pushed down.
+        function() {
+            assertPipelineUsesAggregation({
+                pipeline: pipeline,
+                expectedStages: ["COLLSCAN", projectionType, "$group"],
+            });
+        });
+}
+
+// Asserts that a projection stage is not optimized out of a pipeline with a projection and a group
+// stage.
+function assertProjectionIsNotRemoved(pipeline, projectionType = "PROJECTION_SIMPLE") {
+    assertPipelineIfGroupPushdown(
+        // The projection and group should both be pushed down, and we expect NOT to optimize away
+        // the projection.
+        function() {
+            assertPipelineDoesNotUseAggregation(
+                {pipeline: pipeline, expectedStages: ["COLLSCAN", projectionType, "GROUP"]});
+        },
+        // If group pushdown is not enabled we still expect the projection to be pushed down.
+        function() {
+            assertPipelineUsesAggregation({
+                pipeline: pipeline,
+                expectedStages: ["COLLSCAN", projectionType, "$group"],
+            });
+        });
+}
+
+// Test that an inclusion projection is optimized away if it is redundant/unnecessary.
+assertProjectionCanBeRemovedBeforeGroup(
+    [{$project: {a: 1, b: 1}}, {$group: {_id: "$a", s: {$sum: "$b"}}}]);
+
+assertProjectionCanBeRemovedBeforeGroup(
+    [{$project: {'a.b': 1, 'b.c': 1}}, {$group: {_id: "$a.b", s: {$sum: "$b.c"}}}],
+    "PROJECTION_DEFAULT");
+
+// Test that an inclusion projection is NOT optimized away if it is NOT redundant. This one fails to
+// include a dependency of the $group and so will have an impact on the query results.
+assertProjectionIsNotRemoved([{$project: {a: 1}}, {$group: {_id: "$a", s: {$sum: "$b"}}}]);
+// Test similar cases with dotted paths.
+assertProjectionIsNotRemoved([{$project: {'a.b': 1}}, {$group: {_id: "$a.b", s: {$sum: "$b"}}}],
+                             "PROJECTION_DEFAULT");
+assertProjectionIsNotRemoved([{$project: {'a.b': 1}}, {$group: {_id: "$a.b", s: {$sum: "$a.c"}}}],
+                             "PROJECTION_DEFAULT");
+
+// TODO SERVER-67323 This one could be removed, but is left for future work.
+assertProjectionIsNotRemoved(
+    [{$project: {a: 1, b: 1}}, {$group: {_id: "$a.b", s: {$sum: "$b.c"}}}]);
+
+// Spinoff on the one above: Without supporting this kind of prefixing analysis, we can confuse
+// ourselves with our dependency analysis. If the $group depends on both "path" and "path.subpath"
+// then it will generate a $project on only "path" to express its dependency set. We then fail to
+// optimize that out.
+pipeline = [{$group: {_id: "$a.b", s: {$first: "$a"}}}];
+// TODO SERVER-XYZ Assert this can be optimized out.
+// assertProjectionCanBeRemovedBeforeGroup(pipeline, "PROJECTION_DEFAULT");
+// assertProjectionCanBeRemovedBeforeGroup(pipeline, "PROJECTION_SIMPLE");
+assertProjectionIsNotRemoved(pipeline);
 
 // We generate a projection stage from dependency analysis, even if the pipeline begins with an
 // exclusion projection.
@@ -488,7 +676,7 @@ assertPipelineUsesAggregation({
 explain = coll.explain().aggregate(pipeline);
 projStage = getAggPlanStage(explain, "PROJECTION_SIMPLE");
 assert.neq(null, projStage, explain);
-assert.eq({a: 1, b: 1, _id: 0}, projStage.transformBy, explain);
+assertTransformByShape({a: 1, b: 1, _id: 0}, projStage.transformBy, explain);
 
 // Similar as above, but with a field 'a' presented both in the finite dependency set, and in the
 // exclusion projection at the front of the pipeline.
@@ -500,7 +688,7 @@ assertPipelineUsesAggregation({
 explain = coll.explain().aggregate(pipeline);
 projStage = getAggPlanStage(explain, "PROJECTION_SIMPLE");
 assert.neq(null, projStage, explain);
-assert.eq({a: 1, b: 1, _id: 0}, projStage.transformBy, explain);
+assertTransformByShape({a: 1, b: 1, _id: 0}, projStage.transformBy, explain);
 
 // Test that an exclusion projection at the front of the pipeline is not pushed down, if there no
 // finite dependency set.
@@ -510,6 +698,32 @@ explain = coll.explain().aggregate(pipeline);
 assert(!planHasStage(db, explain, "PROJECTION_SIMPLE"), explain);
 assert(!planHasStage(db, explain, "PROJECTION_DEFAULT"), explain);
 
+// Test that a computed projection at the front of the pipeline is pushed down, even if there's no
+// finite dependency set.
+pipeline = [{$project: {x: {$add: ["$a", 1]}}}];
+assertPipelineDoesNotUseAggregation(
+    {pipeline: pipeline, expectedStages: ["COLLSCAN", "PROJECTION_DEFAULT"]});
+
+pipeline = [{$project: {a: {$add: ["$a", 1]}}}, {$group: {_id: "$a", s: {$sum: "$b"}}}];
+assertPipelineIfGroupPushdown(
+    // Test that a computed projection at the front of the pipeline is pushed down when there's a
+    // finite dependency set. Additionally, the group pushdown shouldn't erase the computed
+    // projection.
+    function() {
+        explain = coll.explain().aggregate(pipeline);
+        assertPipelineDoesNotUseAggregation(
+            {pipeline: pipeline, expectedStages: ["COLLSCAN", "PROJECTION_DEFAULT", "GROUP"]});
+    },
+    // Test that a computed projection at the front of the pipeline is pushed down when there's a
+    // finite dependency set.
+    function() {
+        explain = coll.explain().aggregate(pipeline);
+        assertPipelineUsesAggregation({
+            pipeline: pipeline,
+            expectedStages: ["COLLSCAN", "PROJECTION_DEFAULT", "$group"],
+        });
+    });
+
 // getMore cases.
 
 // Test getMore on a collection with an optimized away pipeline.
@@ -547,6 +761,8 @@ if (!FixtureHelpers.isSharded(coll)) {
 // pipeline. Cannot be run on mongos as profiling can be enabled only on mongod. Also profiling
 // is supported on WiredTiger only.
 if (!FixtureHelpers.isMongos(db) && isWiredTiger(db)) {
+    // Should turn off profiling before dropping system.profile collection.
+    db.setProfilingLevel(0);
     db.system.profile.drop();
     db.setProfilingLevel(2);
     testGetMore({
@@ -560,10 +776,9 @@ if (!FixtureHelpers.isMongos(db) && isWiredTiger(db)) {
     });
     db.setProfilingLevel(0);
     let profile = db.system.profile.find({}, {op: 1, ns: 1}).sort({ts: 1}).toArray();
-    assert(
-        arrayEq(profile,
-                [{op: "command", ns: coll.getFullName()}, {op: "getmore", ns: coll.getFullName()}]),
-        profile);
+    assert.sameMembers(
+        profile,
+        [{op: "command", ns: coll.getFullName()}, {op: "getmore", ns: coll.getFullName()}]);
     // Test getMore puts a correct namespace into profile data for a view with an optimized away
     // pipeline.
     if (!FixtureHelpers.isSharded(coll)) {
@@ -580,9 +795,9 @@ if (!FixtureHelpers.isMongos(db) && isWiredTiger(db)) {
         });
         db.setProfilingLevel(0);
         profile = db.system.profile.find({}, {op: 1, ns: 1}).sort({ts: 1}).toArray();
-        assert(arrayEq(
+        assert.sameMembers(
             profile,
-            [{op: "query", ns: view.getFullName()}, {op: "getmore", ns: view.getFullName()}]));
+            [{op: "query", ns: view.getFullName()}, {op: "getmore", ns: view.getFullName()}]);
     }
 }
 }());
