@@ -1,15 +1,26 @@
-// @tags: [
-//   assumes_superuser_permissions,
-//   requires_fastcount,
-//   requires_non_retryable_commands,
-//   # applyOps uses the oplog that require replication support
-//   requires_replication,
-// ]
+/*
+ * @tags: [
+ *   assumes_superuser_permissions,
+ *   requires_fastcount,
+ *   requires_non_retryable_commands,
+ *   # applyOps is not supported on mongos
+ *   assumes_against_mongod_not_mongos,
+ *   # applyOps uses the oplog that require replication support
+ *   requires_replication,
+ *   # Tenant migrations don't support applyOps.
+ *   tenant_migration_incompatible,
+ *   # This test will fail on a mixed 6.1/6.0 cluster because a 6.0 node can successfully apply
+ *   # a $v: 1 oplog entry, but a 6.1 node cannot successfully replicate it. This isn't an issue
+ *   # because applyOps is an internal command, so users aren't expected to manually insert $v: 1
+ *   # entries into the oplog.
+ *   requires_fcv_61,
+ * ]
+ */
 
 (function() {
 "use strict";
 
-load("jstests/libs/get_index_helpers.js");
+load("jstests/libs/index_catalog_helpers.js");
 
 var t = db.apply_ops1;
 t.drop();
@@ -239,11 +250,18 @@ function testCrudOperationOnNonExistentNamespace(optype, o, o2, expectedErrorCod
 // Insert and update operations on non-existent collections/databases should return
 // NamespaceNotFound.
 testCrudOperationOnNonExistentNamespace('i', {_id: 0}, {}, ErrorCodes.NamespaceNotFound);
-testCrudOperationOnNonExistentNamespace('u', {x: 0}, {_id: 0}, ErrorCodes.NamespaceNotFound);
+testCrudOperationOnNonExistentNamespace(
+    'u', {$v: 2, diff: {x: 0}}, {_id: 0}, ErrorCodes.NamespaceNotFound);
 
-// Delete operations on non-existent collections/databases should return OK for idempotency
-// reasons.
-testCrudOperationOnNonExistentNamespace('d', {_id: 0}, {});
+// TODO(SERVER-46221): These oplog entries are inserted as given.  After SERVER-21700 and with
+// steady-state oplog constraint enforcement on, they will result in secondary crashes.  We
+// will need to have applyOps apply operations as executed rather than as given for this to
+// work properly.
+if (false) {
+    // Delete operations on non-existent collections/databases should return OK for idempotency
+    // reasons.
+    testCrudOperationOnNonExistentNamespace('d', {_id: 0}, {});
+}
 
 assert.commandWorked(db.createCollection(t.getName()));
 var a = assert.commandWorked(
@@ -251,10 +269,13 @@ var a = assert.commandWorked(
 assert.eq(1, t.find().count(), "Valid insert failed");
 assert.eq(true, a.results[0], "Bad result value for valid insert");
 
-a = assert.commandWorked(
-    db.adminCommand({applyOps: [{"op": "i", "ns": t.getFullName(), "o": {_id: 5, x: 17}}]}));
-assert.eq(1, t.find().count(), "Duplicate insert failed");
-assert.eq(true, a.results[0], "Bad result value for duplicate insert");
+// TODO(SERVER-46221): Duplicate inserts result in invalid oplog entries, as above.
+if (false) {
+    a = assert.commandWorked(
+        db.adminCommand({applyOps: [{"op": "i", "ns": t.getFullName(), "o": {_id: 5, x: 17}}]}));
+    assert.eq(1, t.find().count(), "Duplicate insert failed");
+    assert.eq(true, a.results[0], "Bad result value for duplicate insert");
+}
 
 var o = {_id: 5, x: 17};
 assert.eq(o, t.findOne(), "Mismatching document inserted.");
@@ -265,8 +286,8 @@ assert.commandFailed(db.adminCommand({applyOps: [{op: 'i', ns: t.getFullName(),
 
 var res = assert.commandWorked(db.runCommand({
     applyOps: [
-        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$set: {x: 18}}},
-        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$set: {x: 19}}}
+        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$v: 2, diff: {u: {x: 18}}}},
+        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$v: 2, diff: {u: {x: 19}}}}
     ]
 }));
 
@@ -281,8 +302,8 @@ assert.eq(true, res.results[1], "Bad result value for valid update");
 // preCondition fully matches
 res = db.runCommand({
     applyOps: [
-        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$set: {x: 20}}},
-        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$set: {x: 21}}}
+        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$v: 2, diff: {u: {x: 20}}}},
+        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$v: 2, diff: {u: {x: 21}}}}
     ],
     preCondition: [{ns: t.getFullName(), q: {_id: 5}, res: {x: 19}}]
 });
@@ -291,7 +312,7 @@ res = db.runCommand({
 // with {allowAtomic: false}.
 assert.commandFailedWithCode(
     db.runCommand({
-        applyOps: [{op: 'u', ns: t.getFullName(), o2: {_id: 5}, o: {$set: {x: 22}}}],
+        applyOps: [{op: 'u', ns: t.getFullName(), o2: {_id: 5}, o: {$v: 2, diff: {u: {x: 22}}}}],
         preCondition: [{ns: t.getFullName(), q: {_id: 5}, res: {x: 21}}],
         allowAtomic: false,
     }),
@@ -318,8 +339,8 @@ assert.eq(true, res.results[1], "Bad result value for valid update");
 // preCondition doesn't match ns
 res = db.runCommand({
     applyOps: [
-        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$set: {x: 22}}},
-        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$set: {x: 23}}}
+        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$v: 2, diff: {u: {x: 22}}}},
+        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$v: 2, diff: {u: {x: 23}}}}
     ],
     preCondition: [{ns: "foo.otherName", q: {_id: 5}, res: {x: 21}}]
 });
@@ -329,8 +350,8 @@ assert.eq(o, t.findOne(), "preCondition didn't match, but ops were still applied
 // preCondition doesn't match query
 res = db.runCommand({
     applyOps: [
-        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$set: {x: 22}}},
-        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$set: {x: 23}}}
+        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$v: 2, diff: {u: {x: 22}}}},
+        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$v: 2, diff: {u: {x: 23}}}}
     ],
     preCondition: [{ns: t.getFullName(), q: {_id: 5}, res: {x: 19}}]
 });
@@ -339,8 +360,8 @@ assert.eq(o, t.findOne(), "preCondition didn't match, but ops were still applied
 
 res = db.runCommand({
     applyOps: [
-        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$set: {x: 22}}},
-        {op: "u", ns: t.getFullName(), o2: {_id: 6}, o: {$set: {x: 23}}}
+        {op: "u", ns: t.getFullName(), o2: {_id: 5}, o: {$v: 2, diff: {u: {x: 22}}}},
+        {op: "u", ns: t.getFullName(), o2: {_id: 6}, o: {$v: 2, diff: {u: {x: 23}}}}
     ]
 });
 
@@ -366,7 +387,7 @@ res = db.runCommand({
             op: "u",
             ns: t.getFullName(),
             o2: {_id: 8},
-            o: {$set: {x: 25}},
+            o: {$v: 2, diff: {u: {x: 25}}},
             lsid: lsid,
             txnNumber: NumberLong(1),
             stmtId: NumberInt(1)
@@ -386,15 +407,49 @@ assert.eq(true, res.results[0], "Valid insert with transaction number failed");
 assert.eq(true, res.results[1], "Valid update with transaction number failed");
 assert.eq(true, res.results[2], "Valid delete with transaction number failed");
 
-// When applying a "u" (update) op, we default to 'UpdateNode' update semantics, and $set
-// operations add new fields in lexicographic order.
+// Ops with multiple statement IDs are valid.
+res = db.runCommand({
+    applyOps: [
+        {
+            op: "i",
+            ns: t.getFullName(),
+            o: {_id: 7, x: 24},
+            lsid: lsid,
+            txnNumber: NumberLong(3),
+            stmtId: [NumberInt(0), NumberInt(1)]
+        },
+        {
+            op: "u",
+            ns: t.getFullName(),
+            o2: {_id: 8},
+            o: {$v: 2, diff: {u: {x: 25}}},
+            lsid: lsid,
+            txnNumber: NumberLong(3),
+            stmtId: [NumberInt(2), NumberInt(3)]
+        },
+        {
+            op: "d",
+            ns: t.getFullName(),
+            o: {_id: 7},
+            lsid: lsid,
+            txnNumber: NumberLong(4),
+            stmtId: [NumberInt(0), NumberInt(1)]
+        },
+    ]
+});
+
+assert.eq(true, res.results[0], "Valid insert with multiple statement IDs failed");
+assert.eq(true, res.results[1], "Valid update with multiple statement IDs failed");
+assert.eq(true, res.results[2], "Valid delete with multiple statement IDs failed");
+
+// When applying a "u" (update) op in the $v: 2 format, the
 res = assert.commandWorked(db.adminCommand({
     applyOps: [
-        {"op": "i", "ns": t.getFullName(), "o": {_id: 6}},
-        {"op": "u", "ns": t.getFullName(), "o2": {_id: 6}, "o": {$set: {z: 1, a: 2}}}
+        {"op": "i", "ns": t.getFullName(), "o": {_id: 9}},
+        {"op": "u", "ns": t.getFullName(), "o2": {_id: 9}, "o": {$v: 2, diff: {u: {z: 1, a: 2}}}},
     ]
 }));
-assert.eq(t.findOne({_id: 6}), {_id: 6, a: 2, z: 1});  // Note: 'a' and 'z' have been sorted.
+assert.eq(t.findOne({_id: 9}), {_id: 9, z: 1, a: 2});  // Note: 'a' and 'z' have been sorted.
 
 // 'ModifierInterface' semantics are not supported, so an update with {$v: 0} should fail.
 res = assert.commandFailed(db.adminCommand({
@@ -404,24 +459,110 @@ res = assert.commandFailed(db.adminCommand({
             "op": "u",
             "ns": t.getFullName(),
             "o2": {_id: 7},
-            "o": {$v: NumberLong(0), $set: {z: 1, a: 2}}
+            "o": {$v: NumberInt(0), $set: {z: 1, a: 2}}
         }
     ]
 }));
-assert.eq(res.code, 40682);
+assert.eq(res.code, 4772600);
 
-// When we explicitly specify {$v: 1}, we should get 'UpdateNode' update semantics, and $set
-// operations get performed in lexicographic order.
-res = assert.commandWorked(db.adminCommand({
+// When we explicitly specify {$v: 1} it should fail because this version is no longer supported.
+assert.commandFailedWithCode(db.adminCommand({
     applyOps: [
-        {"op": "i", "ns": t.getFullName(), "o": {_id: 8}},
+        {"op": "i", "ns": t.getFullName(), "o": {_id: 10}},
         {
             "op": "u",
             "ns": t.getFullName(),
-            "o2": {_id: 8},
-            "o": {$v: NumberLong(1), $set: {z: 1, a: 2}}
+            "o2": {_id: 10},
+            "o": {$v: NumberInt(1), $set: {z: 1, a: 2}}
+        }
+    ]
+}),
+                             4772600);
+
+// {$v: 2} entries encode diffs differently, and operations are applied in the order specified
+// rather than in lexicographic order.
+res = assert.commandWorked(db.adminCommand({
+    applyOps: [
+        {"op": "i", "ns": t.getFullName(), "o": {_id: 11, deleteField: 1}},
+        {
+            "op": "u",
+            "ns": t.getFullName(),
+            "o2": {_id: 11},
+            // The diff indicates that 'deleteField' will be removed and 'newField' will be added
+            // with value "foo".
+            "o": {$v: NumberInt(2), diff: {d: {deleteField: false}, i: {newField: "foo"}}}
         }
     ]
 }));
-assert.eq(t.findOne({_id: 8}), {_id: 8, a: 2, z: 1});  // Note: 'a' and 'z' have been sorted.
+assert.eq(t.findOne({_id: 11}), {_id: 11, newField: "foo"});
+
+// {$v: 3} does not exist yet, and we check that trying to use it throws an error.
+res = assert.commandFailed(db.adminCommand({
+    applyOps: [
+        {"op": "i", "ns": t.getFullName(), "o": {_id: 12}},
+        {
+            "op": "u",
+            "ns": t.getFullName(),
+            "o2": {_id: 12},
+            "o": {$v: NumberInt(3), diff: {d: {deleteField: false}}}
+        }
+    ]
+}));
+assert.eq(res.code, 4772600);
+
+var insert_op1 = {_id: 13, x: 'inserted apply ops1'};
+var insert_op2 = {_id: 14, x: 'inserted apply ops2'};
+assert.commandWorked(db.adminCommand({
+    "applyOps": [{
+        op: 'c',
+        ns: 'admin.$cmd',
+        o: {
+            "applyOps": [{
+                op: 'c',
+                ns: 'test.$cmd',
+                o: {
+                    "applyOps": [
+                        {op: 'i', ns: t.getFullName(), o: insert_op1},
+                        {op: 'i', ns: t.getFullName(), o: insert_op2}
+                    ]
+                }
+            }],
+        }
+    }]
+}),
+                     "Nested apply ops was NOT successful");
+assert.eq(t.findOne({_id: 13}), insert_op1);
+assert.eq(t.findOne({_id: 14}), insert_op2);
+
+assert.commandWorked(db.adminCommand({
+    "applyOps": [{
+        op: 'c',
+        ns: 'admin.$cmd',
+        o: {
+            "applyOps": [{
+                op: 'c',
+                ns: 'test.$cmd',
+                o: {
+                    "applyOps": [
+                        {
+                            op: 'u',
+                            ns: t.getFullName(),
+                            o2: {_id: 13},
+                            o: {$v: 2, diff: {u: {x: 'nested apply op update1'}}},
+                        },
+                        {
+                            op: 'u',
+                            ns: t.getFullName(),
+                            o2: {_id: 14},
+                            o: {$v: 2, diff: {u: {x: 'nested apply op update2'}}},
+                        }
+                    ]
+                }
+            }],
+        }
+    }]
+}),
+                     "Nested apply ops was NOT successful");
+assert.eq(t.findOne({_id: 13}), {_id: 13, x: 'nested apply op update1'});
+assert.eq(t.findOne({_id: 14}), {_id: 14, x: 'nested apply op update2'});
 })();
