@@ -1,13 +1,21 @@
-// @tags: [requires_fastcount, assumes_against_mongod_not_mongos]
+/**
+ * Tests that count() in shell takes query.
+ *
+ * @tags: [
+ *     requires_fastcount,
+ *     assumes_against_mongod_not_mongos,
+ * ]
+ */
 
 (function() {
 "use strict";
 
-const coll = db.jstests_count;
+const collNamePrefix = 'jstests_count_';
+let collCount = 0;
 
+let coll = db.getCollection(collNamePrefix + collCount++);
 coll.drop();
-assert.commandWorked(coll.insert({i: 1}));
-assert.commandWorked(coll.insert({i: 2}));
+assert.commandWorked(coll.insert([{_id: 1, i: 1}, {_id: 2, i: 2}]));
 assert.eq(1, coll.find({i: 1}).count());
 assert.eq(1, coll.count({i: 1}));
 assert.eq(2, coll.find().count());
@@ -15,15 +23,17 @@ assert.eq(2, coll.find(undefined).count());
 assert.eq(2, coll.find(null).count());
 assert.eq(2, coll.count());
 
+coll = db.getCollection(collNamePrefix + collCount++);
 coll.drop();
+assert.commandWorked(coll.createIndex({b: 1, a: 1}));
 assert.commandWorked(coll.insert({a: true, b: false}));
-assert.commandWorked(coll.ensureIndex({b: 1, a: 1}));
 assert.eq(1, coll.find({a: true, b: false}).count());
 assert.eq(1, coll.find({b: false, a: true}).count());
 
+coll = db.getCollection(collNamePrefix + collCount++);
 coll.drop();
+assert.commandWorked(coll.createIndex({b: 1, a: 1, c: 1}));
 assert.commandWorked(coll.insert({a: true, b: false}));
-assert.commandWorked(coll.ensureIndex({b: 1, a: 1, c: 1}));
 
 assert.eq(1, coll.find({a: true, b: false}).count());
 assert.eq(1, coll.find({b: false, a: true}).count());
