@@ -6,6 +6,9 @@
 //   assumes_unsharded_collection,
 //   assumes_write_concern_unchanged,
 //   requires_fastcount,
+//   # This test writes with {w: 0} which doesn't wait for the storage transaction writing the
+//   # document and the oplog entry to commit so the TenantMigrationConflict will not be caught.
+//   tenant_migration_incompatible,
 // ]
 
 (function() {
@@ -46,14 +49,8 @@ var crudAPISpecTests = function crudAPISpecTests() {
     }
 
     function checkResultObject(first, second) {
-        // Only assert on the "modifiedCount" property when write commands are enabled
-        if (db.getMongo().writeMode() === 'commands') {
-            assert.docEq(first, second);
-        } else {
-            var overrideModifiedCount = {modifiedCount: undefined};
-            assert.docEq(Object.merge(first, overrideModifiedCount),
-                         Object.merge(second, overrideModifiedCount));
-        }
+        // Only assert on the "modifiedCount" property
+        assert.docEq(first, second);
     }
 
     // Setup executors
@@ -670,10 +667,6 @@ var crudAPISpecTests = function crudAPISpecTests() {
     var cursor = coll.find({}).sort({a: 1}).tailable(false);
     assert.eq(2, (cursor._options & ~DBQuery.Option.slaveOk));
 
-    // Check modifiers
-    var cursor = coll.find({}).modifiers({$hint: 'a_1'});
-    assert.eq('a_1', cursor._query['$hint']);
-
     // allowPartialResults
     var cursor = coll.find({}).allowPartialResults();
     assert.eq(128, (cursor._options & ~DBQuery.Option.slaveOk));
@@ -700,7 +693,7 @@ var crudAPISpecTests = function crudAPISpecTests() {
 
     // Drop collection
     coll.drop();
-    coll.ensureIndex({a: 1}, {unique: true});
+    coll.createIndex({a: 1}, {unique: true});
 
     // Should throw duplicate key error
     assert.throws(function() {
