@@ -2,10 +2,12 @@
 //   assumes_superuser_permissions,
 //   creates_and_authenticates_user,
 //   requires_profiling,
+//   # This test depends on hardcoded database name equality.
+//   tenant_migration_incompatible,
 // ]
 // special db so that it can be run in parallel tests
 var stddb = db;
-var db = db.getSisterDB("profile3");
+var db = db.getSiblingDB("profile3");
 
 db.dropAllUsers();
 t = db.profile3;
@@ -21,6 +23,8 @@ profileCursor = function(query) {
 try {
     username = "jstests_profile3_user";
     db.createUser({user: username, pwd: "password", roles: jsTest.basicUserRoles});
+
+    db.logout();
     db.auth(username, "password");
 
     db.setProfilingLevel(0);
@@ -49,5 +53,6 @@ try {
 
 } finally {
     db.setProfilingLevel(0);
+    db.logout();
     db = stddb;
 }
