@@ -1,19 +1,20 @@
 // @tags: [
+//   assumes_unsharded_collection,
 //   assumes_superuser_permissions,
 //   does_not_support_stepdowns,
+//   requires_emptycapped,
 //   requires_fastcount,
-//   requires_fcv_44,
 //   requires_getmore,
 //   requires_non_retryable_commands,
 //   requires_non_retryable_writes,
 //   uses_map_reduce_with_temp_collections,
-//   requires_emptycapped,
+//   # Tenant migrations don't support applyOps.
+//   tenant_migration_incompatible,
+//   # Explain of a resolved view must be executed by mongos.
+//   directly_against_shardsvrs_incompatible,
+//   # This test has statements that do not support non-local read concern.
+//   does_not_support_causal_consistency,
 // ]
-//
-// Tagged as 'requires_fcv_44', since this test cannot run against versions less then 4.4. This is
-// because 'planCacheListPlans' and 'planCacheListQueryShapes' were deleted in 4.4, and thus not
-// tested here. But this test asserts that all commands are covered, so will fail against a version
-// of the server which implements these commands.
 
 /*
  * Declaratively-defined tests for views for all database commands. This file contains a map of test
@@ -70,45 +71,71 @@
 (function() {
 "use strict";
 
+load('jstests/sharding/libs/last_lts_mongod_commands.js');
+
 // Pre-written reasons for skipping a test.
 const isAnInternalCommand = "internal command";
 const isUnrelated = "is unrelated";
-const wasRemovedInBinaryVersion44 =
-    "must define test coverage for the multiversion passthrough suite";
 
 let viewsCommandTests = {
     _addShard: {skip: isAnInternalCommand},
-    _cloneCatalogData: {skip: wasRemovedInBinaryVersion44},
+    _cloneCatalogData: {skip: isAnInternalCommand},
     _cloneCollectionOptionsFromPrimaryShard: {skip: isAnInternalCommand},
+    _configsvrAbortReshardCollection: {skip: isAnInternalCommand},
     _configsvrAddShard: {skip: isAnInternalCommand},
     _configsvrAddShardToZone: {skip: isAnInternalCommand},
     _configsvrBalancerCollectionStatus: {skip: isAnInternalCommand},
     _configsvrBalancerStart: {skip: isAnInternalCommand},
     _configsvrBalancerStatus: {skip: isAnInternalCommand},
     _configsvrBalancerStop: {skip: isAnInternalCommand},
+    _configsvrCleanupReshardCollection: {skip: isAnInternalCommand},
+    _configsvrCollMod: {skip: isAnInternalCommand},
     _configsvrClearJumboFlag: {skip: isAnInternalCommand},
-    _configsvrCommitChunkMerge: {skip: isAnInternalCommand},
     _configsvrCommitChunksMerge: {skip: isAnInternalCommand},
     _configsvrCommitChunkMigration: {skip: isAnInternalCommand},
     _configsvrCommitChunkSplit: {skip: isAnInternalCommand},
-    _configsvrCommitMovePrimary: {skip: isAnInternalCommand},
-    _configsvrCreateCollection: {skip: isAnInternalCommand},
+    _configsvrCommitIndex: {skip: isAnInternalCommand},
+    _configsvrCommitMovePrimary:
+        {skip: isAnInternalCommand},  // Can be removed once 6.0 is last LTS
+    _configsvrCommitReshardCollection: {skip: isAnInternalCommand},
+    _configsvrConfigureAutoSplit: {
+        skip: isAnInternalCommand
+    },  // TODO SERVER-62374: remove this once 5.3 becomes last continuos release
+    _configsvrConfigureCollectionBalancing: {skip: isAnInternalCommand},
     _configsvrCreateDatabase: {skip: isAnInternalCommand},
-    _configsvrDropCollection: {skip: isAnInternalCommand},
-    _configsvrDropDatabase: {skip: isAnInternalCommand},
-    _configsvrEnableSharding: {skip: isAnInternalCommand},
+    _configsvrDropCollection:
+        {skip: isAnInternalCommand},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
+    _configsvrDropDatabase:
+        {skip: isAnInternalCommand},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
+    _configsvrDropIndexCatalogEntry: {skip: isAnInternalCommand},
+    _configsvrEnableSharding:
+        {skip: isAnInternalCommand},  // TODO (SERVER-58843): Remove once 6.0 becomes last LTS
     _configsvrEnsureChunkVersionIsGreaterThan: {skip: isAnInternalCommand},
-    _configsvrMoveChunk: {skip: isAnInternalCommand},
+    _configsvrMoveChunk: {skip: isAnInternalCommand},  // Can be removed once 6.0 is last LTS
     _configsvrMovePrimary: {skip: isAnInternalCommand},
+    _configsvrMoveRange: {skip: isAnInternalCommand},
     _configsvrRefineCollectionShardKey: {skip: isAnInternalCommand},
     _configsvrRenameCollection: {skip: isAnInternalCommand},
+    _configsvrRenameCollectionMetadata: {skip: isAnInternalCommand},
+    _configsvrRemoveChunks: {skip: isAnInternalCommand},
     _configsvrRemoveShard: {skip: isAnInternalCommand},
     _configsvrRemoveShardFromZone: {skip: isAnInternalCommand},
+    _configsvrRemoveTags: {skip: isAnInternalCommand},
     _configsvrRepairShardedCollectionChunksHistory: {skip: isAnInternalCommand},
-    _configsvrShardCollection: {skip: isAnInternalCommand},
+    _configsvrReshardCollection: {skip: isAnInternalCommand},
+    _configsvrRunRestore: {skip: isAnInternalCommand},
+    _configsvrSetAllowMigrations: {skip: isAnInternalCommand},
+    _configsvrSetClusterParameter: {skip: isAnInternalCommand},
+    _configsvrSetUserWriteBlockMode: {skip: isAnInternalCommand},
+    _configsvrShardCollection:
+        {skip: isAnInternalCommand},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
     _configsvrUpdateZoneKeyRange: {skip: isAnInternalCommand},
     _flushDatabaseCacheUpdates: {skip: isUnrelated},
+    _flushDatabaseCacheUpdatesWithWriteConcern: {skip: isUnrelated},
+    _flushReshardingStateChange: {skip: isUnrelated},
     _flushRoutingTableCacheUpdates: {skip: isUnrelated},
+    _flushRoutingTableCacheUpdatesWithWriteConcern: {skip: isUnrelated},
+    _getAuditConfigGeneration: {skip: isUnrelated},
     _getNextSessionMods: {skip: isAnInternalCommand},
     _getUserCacheGeneration: {skip: isAnInternalCommand},
     _hashBSONElement: {skip: isAnInternalCommand},
@@ -116,21 +143,61 @@ let viewsCommandTests = {
     _killOperations: {skip: isUnrelated},
     _mergeAuthzCollections: {skip: isAnInternalCommand},
     _migrateClone: {skip: isAnInternalCommand},
-    _movePrimary: {skip: wasRemovedInBinaryVersion44},
+    _movePrimary: {skip: isAnInternalCommand},
     _recvChunkAbort: {skip: isAnInternalCommand},
     _recvChunkCommit: {skip: isAnInternalCommand},
+    _recvChunkReleaseCritSec: {skip: isAnInternalCommand},
     _recvChunkStart: {skip: isAnInternalCommand},
     _recvChunkStatus: {skip: isAnInternalCommand},
+    _shardsvrAbortReshardCollection: {skip: isAnInternalCommand},
     _shardsvrCloneCatalogData: {skip: isAnInternalCommand},
+    _shardsvrCompactStructuredEncryptionData: {skip: isAnInternalCommand},
+    _shardsvrDropCollection: {skip: isAnInternalCommand},
+    _shardsvrDropCollectionIfUUIDNotMatching: {skip: isUnrelated},
+    _shardsvrDropCollectionParticipant: {skip: isAnInternalCommand},
+    _shardsvrDropIndexCatalogEntryParticipant: {skip: isAnInternalCommand},
+    _shardsvrDropIndexes: {skip: isAnInternalCommand},
+    _shardsvrCleanupReshardCollection: {skip: isAnInternalCommand},
+    _shardsvrRegisterIndex: {skip: isAnInternalCommand},
+    _shardsvrCommitIndexParticipant: {skip: isAnInternalCommand},
+    _shardsvrCommitReshardCollection: {skip: isAnInternalCommand},
+    _shardsvrCreateCollection: {skip: isAnInternalCommand},
+    _shardsvrCreateCollectionParticipant: {skip: isAnInternalCommand},
+    _shardsvrDropDatabase: {skip: isAnInternalCommand},
+    _shardsvrDropDatabaseParticipant: {skip: isAnInternalCommand},
+    _shardsvrGetStatsForBalancing: {skip: isAnInternalCommand},
+    _shardsvrJoinMigrations: {skip: isAnInternalCommand},
     _shardsvrMovePrimary: {skip: isAnInternalCommand},
+    _shardsvrMoveRange: {
+        command: {_shardsvrMoveRange: "test.view"},
+        skipStandalone: true,
+        isAdminCommand: true,
+        expectFailure: true,
+        expectedErrorCode: ErrorCodes.NamespaceNotSharded,
+    },
+    _shardsvrRefineCollectionShardKey: {skip: isAnInternalCommand},
     _shardsvrRenameCollection: {skip: isAnInternalCommand},
+    _shardsvrRenameCollectionParticipant: {skip: isAnInternalCommand},
+    _shardsvrRenameCollectionParticipantUnblock: {skip: isAnInternalCommand},
+    _shardsvrReshardCollection: {skip: isAnInternalCommand},
+    _shardsvrReshardingOperationTime: {skip: isAnInternalCommand},
     _shardsvrSetAllowMigrations: {skip: isAnInternalCommand},
-    _shardsvrShardCollection: {skip: isAnInternalCommand},
+    _shardsvrSetClusterParameter: {skip: isAnInternalCommand},
+    _shardsvrSetUserWriteBlockMode: {skip: isAnInternalCommand},
+    _shardsvrShardCollection:
+        {skip: isAnInternalCommand},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
+    _shardsvrCollMod: {skip: isAnInternalCommand},
+    _shardsvrCollModParticipant: {skip: isAnInternalCommand},
+    _shardsvrParticipantBlock: {skip: isAnInternalCommand},
+    _shardsvrUnregisterIndex: {skip: isAnInternalCommand},
     _transferMods: {skip: isAnInternalCommand},
+    _vectorClockPersist: {skip: isAnInternalCommand},
+    abortReshardCollection: {skip: isUnrelated},
     abortTransaction: {skip: isUnrelated},
     addShard: {skip: isUnrelated},
     addShardToZone: {skip: isUnrelated},
     aggregate: {command: {aggregate: "view", pipeline: [{$match: {}}], cursor: {}}},
+    analyze: {skip: isUnrelated},
     appendOplogNote: {skip: isUnrelated},
     applyOps: {
         command: {applyOps: [{op: "i", o: {_id: 1}, ns: "test.view"}]},
@@ -146,7 +213,6 @@ let viewsCommandTests = {
         },
         expectFailure: true,
     },
-    availableQueryOptions: {skip: isAnInternalCommand},
     balancerCollectionStatus: {
         command: {balancerCollectionStatus: "test.view"},
         setup: function(conn) {
@@ -169,6 +235,7 @@ let viewsCommandTests = {
     cleanupOrphaned: {
         skip: "Tested in views/views_sharded.js",
     },
+    cleanupReshardCollection: {skip: isUnrelated},
     clearJumboFlag: {
         command: {clearJumboFlag: "test.view"},
         skipStandalone: true,
@@ -177,16 +244,29 @@ let viewsCommandTests = {
         expectedErrorCode: ErrorCodes.NamespaceNotSharded,
     },
     clearLog: {skip: isUnrelated},
-    cloneCollection: {skip: wasRemovedInBinaryVersion44},
     cloneCollectionAsCapped: {
         command: {cloneCollectionAsCapped: "view", toCollection: "testcapped", size: 10240},
         expectFailure: true,
     },
+    clusterAbortTransaction: {skip: "already tested by 'abortTransaction' tests on mongos"},
+    clusterAggregate: {skip: "already tested by 'aggregate' tests on mongos"},
+    clusterCommitTransaction: {skip: "already tested by 'commitTransaction' tests on mongos"},
+    clusterDelete: {skip: "already tested by 'delete' tests on mongos"},
+    clusterFind: {skip: "already tested by 'find' tests on mongos"},
+    clusterGetMore: {skip: "already tested by 'getMore' tests on mongos"},
+    clusterInsert: {skip: "already tested by 'insert' tests on mongos"},
+    clusterUpdate: {skip: "already tested by 'update' tests on mongos"},
     collMod: {command: {collMod: "view", viewOn: "other", pipeline: []}},
     collStats: {skip: "Tested in views/views_coll_stats.js"},
+    commitReshardCollection: {skip: isUnrelated},
     commitTransaction: {skip: isUnrelated},
     compact: {command: {compact: "view", force: true}, expectFailure: true, skipSharded: true},
+    compactStructuredEncryptionData: {skip: isUnrelated},
     configureFailPoint: {skip: isUnrelated},
+    configureCollectionAutoSplitter: {
+        skip: isUnrelated
+    },  // TODO SERVER-62374: remove this once 5.3 becomes last continuos release
+    configureCollectionBalancing: {skip: isUnrelated},
     connPoolStats: {skip: isUnrelated},
     connPoolSync: {skip: isUnrelated},
     connectionStatus: {skip: isUnrelated},
@@ -245,13 +325,24 @@ let viewsCommandTests = {
     dbStats: {command: {dbStats: 1}},
     delete: {command: {delete: "view", deletes: [{q: {x: 1}, limit: 1}]}, expectFailure: true},
     distinct: {command: {distinct: "view", key: "_id"}},
+    donorAbortMigration: {skip: isUnrelated},
+    // TODO : remove overrides once possible SERVER-61845
+    donorAbortSplit: {skip: "has been removed from the server"},
+    donorForgetMigration: {skip: isUnrelated},
+    donorForgetSplit: {skip: "has been removed from the server"},
+    donorStartMigration: {skip: isUnrelated},
+    donorStartSplit: {skip: "has been removed from the server"},
+    donorWaitForMigrationToCommit: {skip: isUnrelated},
+    abortShardSplit: {skip: isUnrelated},
+    commitShardSplit: {skip: isUnrelated},
+    forgetShardSplit: {skip: isUnrelated},
     driverOIDTest: {skip: isUnrelated},
     drop: {command: {drop: "view"}},
     dropAllRolesFromDatabase: {skip: isUnrelated},
     dropAllUsersFromDatabase: {skip: isUnrelated},
     dropConnections: {skip: isUnrelated},
     dropDatabase: {command: {dropDatabase: 1}},
-    dropIndexes: {command: {dropIndexes: "view"}, expectFailure: true},
+    dropIndexes: {command: {dropIndexes: "view", index: "a_1"}, expectFailure: true},
     dropRole: {
         command: {dropRole: "testrole"},
         setup: function(conn) {
@@ -281,18 +372,14 @@ let viewsCommandTests = {
     flushRouterConfig: {skip: isUnrelated},
     fsync: {skip: isUnrelated},
     fsyncUnlock: {skip: isUnrelated},
+    getAuditConfig: {skip: isUnrelated},
     getDatabaseVersion: {skip: isUnrelated},
-    geoSearch: {
-        command: {
-            geoSearch: "view",
-            search: {},
-            near: [-50, 37],
-        },
-        expectFailure: true
-    },
+    getChangeStreamState: {skip: isUnrelated},
+    getClusterParameter: {skip: isUnrelated},
     getCmdLineOpts: {skip: isUnrelated},
     getDefaultRWConcern: {skip: isUnrelated},
     getDiagnosticData: {skip: isUnrelated},
+    getFreeMonitoringStatus: {skip: isUnrelated},
     getLastError: {skip: isUnrelated},
     getLog: {skip: isUnrelated},
     getMore: {
@@ -333,20 +420,9 @@ let viewsCommandTests = {
     getParameter: {skip: isUnrelated},
     getShardMap: {skip: isUnrelated},
     getShardVersion: {
-        command: function(conn) {
-            // getShardVersion was updated to be allowed on views in v4.4, but in v4.2 would return
-            // CommandNotSupportedOnView. Ignore a CommandNotSupportedOnView error code so that the
-            // test passes in the mixed-version replica sets passthrough. The v4.2 behavior is
-            // tested in the v4.2 branch.
-            try {
-                assert.commandWorked(conn.adminCommand({getShardVersion: "test.view"}));
-            } catch (e) {
-                if (e.code == ErrorCodes.CommandNotSupportedOnView) {
-                    return;
-                }
-                throw e;
-            }
-        },
+        command: {getShardVersion: "test.view"},
+        expectFailure: true,
+        expectedErrorCode: ErrorCodes.NoShardingEnabled,
         isAdminCommand: true,
         skipSharded: true,  // mongos is tested in views/views_sharded.js
     },
@@ -359,6 +435,8 @@ let viewsCommandTests = {
     hello: {skip: isUnrelated},
     hostInfo: {skip: isUnrelated},
     httpClientRequest: {skip: isAnInternalCommand},
+    exportCollection: {skip: isUnrelated},
+    importCollection: {skip: isUnrelated},
     insert: {command: {insert: "view", documents: [{x: 1}]}, expectFailure: true},
     internalRenameIfOptionsAndIndexesMatch: {skip: isAnInternalCommand},
     invalidateUserCache: {skip: isUnrelated},
@@ -403,6 +481,7 @@ let viewsCommandTests = {
     listCollections: {skip: "tested in views/views_creation.js"},
     listCommands: {skip: isUnrelated},
     listDatabases: {skip: isUnrelated},
+    listDatabasesForAllTenants: {skip: isUnrelated},
     listIndexes: {command: {listIndexes: "view"}, expectFailure: true},
     listShards: {skip: isUnrelated},
     lockInfo: {skip: isUnrelated},
@@ -416,7 +495,6 @@ let viewsCommandTests = {
             {mapReduce: "view", map: function() {}, reduce: function(key, vals) {}, out: "out"},
         expectFailure: true
     },
-    "mapreduce.shardedfinish": {skip: isAnInternalCommand},
     mergeChunks: {
         command: {mergeChunks: "test.view", bounds: [{x: 0}, {x: 10}]},
         skipStandalone: true,
@@ -425,15 +503,17 @@ let viewsCommandTests = {
         expectedErrorCode: ErrorCodes.NamespaceNotSharded,
     },
     moveChunk: {
-        command: {moveChunk: "test.view"},
+        command: {moveChunk: "test.view", find: {}, to: "a"},
         skipStandalone: true,
         isAdminCommand: true,
         expectFailure: true,
         expectedErrorCode: ErrorCodes.NamespaceNotSharded,
     },
     movePrimary: {skip: "Tested in sharding/movePrimary1.js"},
+    moveRange: {skip: isUnrelated},
     multicast: {skip: isUnrelated},
     netstat: {skip: isAnInternalCommand},
+    pinHistoryReplicated: {skip: isAnInternalCommand},
     ping: {command: {ping: 1}},
     planCacheClear: {command: {planCacheClear: "view"}, expectFailure: true},
     planCacheClearFilters: {command: {planCacheClearFilters: "view"}, expectFailure: true},
@@ -444,8 +524,15 @@ let viewsCommandTests = {
     refineCollectionShardKey: {skip: isUnrelated},
     refreshLogicalSessionCacheNow: {skip: isAnInternalCommand},
     reapLogicalSessionCacheNow: {skip: isAnInternalCommand},
+    recipientForgetMigration: {skip: isUnrelated},
+    recipientSyncData: {skip: isUnrelated},
+    recipientVoteImportedFiles: {skip: isAnInternalCommand},
     refreshSessions: {skip: isUnrelated},
-    reIndex: {command: {reIndex: "view"}, expectFailure: true},
+    reIndex: {
+        command: {reIndex: "view"},
+        expectFailure: true,
+        expectedErrorCode: [ErrorCodes.IllegalOperation, ErrorCodes.CommandNotSupportedOnView],
+    },
     removeShard: {skip: isUnrelated},
     removeShardFromZone: {skip: isUnrelated},
     renameCollection: [
@@ -463,8 +550,6 @@ let viewsCommandTests = {
             skipSharded: true,
         }
     ],
-
-    repairCursor: {skip: wasRemovedInBinaryVersion44},
     repairShardedCollectionChunksHistory: {
         command: {repairShardedCollectionChunksHistory: "test.view"},
         skipStandalone: true,
@@ -486,9 +571,19 @@ let viewsCommandTests = {
     replSetStepUp: {skip: isUnrelated},
     replSetSyncFrom: {skip: isUnrelated},
     replSetTest: {skip: isUnrelated},
+    replSetTestEgress: {skip: isUnrelated},
     replSetUpdatePosition: {skip: isUnrelated},
     replSetResizeOplog: {skip: isUnrelated},
-    resetError: {skip: isUnrelated},
+    reshardCollection: {
+        command: {reshardCollection: "test.view", key: {_id: 1}},
+        setup: function(conn) {
+            assert.commandWorked(conn.adminCommand({enableSharding: "test"}));
+        },
+        expectedErrorCode: ErrorCodes.NamespaceNotSharded,
+        skipStandalone: true,
+        expectFailure: true,
+        isAdminCommand: true,
+    },
     revokePrivilegesFromRole: {
         command: {
             revokePrivilegesFromRole: "testrole",
@@ -504,10 +599,6 @@ let viewsCommandTests = {
     },
     revokeRolesFromRole: {skip: isUnrelated},
     revokeRolesFromUser: {skip: isUnrelated},
-    rolesInfo: {skip: isUnrelated},
-    saslContinue: {skip: isUnrelated},
-    saslStart: {skip: isUnrelated},
-    serverStatus: {command: {serverStatus: 1}, skip: isUnrelated},
     setAllowMigrations: {
         command: {setAllowMigrations: "test.view", allowMigrations: false},
         setup: function(conn) {
@@ -518,13 +609,23 @@ let viewsCommandTests = {
         expectFailure: true,
         isAdminCommand: true
     },
+    rolesInfo: {skip: isUnrelated},
+    rotateCertificates: {skip: isUnrelated},
+    saslContinue: {skip: isUnrelated},
+    saslStart: {skip: isUnrelated},
+    sbe: {skip: isAnInternalCommand},
+    serverStatus: {command: {serverStatus: 1}, skip: isUnrelated},
     setIndexCommitQuorum: {skip: isUnrelated},
+    setAuditConfig: {skip: isUnrelated},
     setCommittedSnapshot: {skip: isAnInternalCommand},
     setDefaultRWConcern: {skip: isUnrelated},
     setFeatureCompatibilityVersion: {skip: isUnrelated},
-    setProfilingFilterGlobally: {skip: isUnrelated},
+    setFreeMonitoring: {skip: isUnrelated},
     setParameter: {skip: isUnrelated},
     setShardVersion: {skip: isUnrelated},
+    setChangeStreamState: {skip: isUnrelated},
+    setClusterParameter: {skip: isUnrelated},
+    setUserWriteBlockMode: {skip: isUnrelated},
     shardCollection: {
         command: {shardCollection: "test.view", key: {_id: 1}},
         setup: function(conn) {
@@ -534,7 +635,6 @@ let viewsCommandTests = {
         expectFailure: true,
         isAdminCommand: true,
     },
-    shardConnPoolStats: {skip: isUnrelated},
     shardingState: {skip: isUnrelated},
     shutdown: {skip: isUnrelated},
     sleep: {skip: isUnrelated},
@@ -553,11 +653,11 @@ let viewsCommandTests = {
             max: {x: 0},
             keyPattern: {x: 1},
             splitKeys: [{x: -2}, {x: -1}],
-            shardVersion: [Timestamp(1, 2), ObjectId()]
+            shardVersion: {t: Timestamp(1, 2), e: ObjectId(), v: Timestamp(1, 1)}
         },
         skipSharded: true,
         expectFailure: true,
-        expectedErrorCode: 193,
+        expectedErrorCode: ErrorCodes.NoShardingEnabled,
         isAdminCommand: true,
     },
     splitVector: {
@@ -572,9 +672,14 @@ let viewsCommandTests = {
     startRecordingTraffic: {skip: isUnrelated},
     startSession: {skip: isAnInternalCommand},
     stopRecordingTraffic: {skip: isUnrelated},
+    testDeprecation: {skip: isAnInternalCommand},
+    testDeprecationInVersion2: {skip: isAnInternalCommand},
+    testInternalTransactions: {skip: isAnInternalCommand},
+    testRemoval: {skip: isAnInternalCommand},
+    testReshardCloneCollection: {skip: isAnInternalCommand},
+    testVersion2: {skip: isAnInternalCommand},
+    testVersions1And2: {skip: isAnInternalCommand},
     top: {skip: "tested in views/views_stats.js"},
-    touch: {skip: wasRemovedInBinaryVersion44},
-    unsetSharding: {skip: isAnInternalCommand},
     update: {command: {update: "view", updates: [{q: {x: 1}, u: {x: 2}}]}, expectFailure: true},
     updateRole: {
         command: {
@@ -593,7 +698,10 @@ let viewsCommandTests = {
     updateZoneKeyRange: {skip: isUnrelated},
     usersInfo: {skip: isUnrelated},
     validate: {command: {validate: "view"}, expectFailure: true},
+    validateDBMetadata:
+        {command: {validateDBMetadata: 1, apiParameters: {version: "1", strict: true}}},
     waitForOngoingChunkSplits: {skip: isUnrelated},
+    voteCommitImportCollection: {skip: isUnrelated},
     voteCommitIndexBuild: {skip: isUnrelated},
     voteCommitTransaction: {skip: isUnrelated},
     voteAbortTransaction: {skip: isUnrelated},
@@ -602,6 +710,10 @@ let viewsCommandTests = {
     whatsmysni: {skip: isUnrelated}
 };
 
+commandsRemovedFromMongodSinceLastLTS.forEach(function(cmd) {
+    viewsCommandTests[cmd] = {skip: "must define test coverage for backwards compatibility"};
+});
+
 /**
  * Helper function for failing commands or writes that checks the result 'res' of either.
  * If 'code' is null we only check for failure, otherwise we confirm error code matches as
@@ -617,9 +729,9 @@ let assertCommandOrWriteFailed = function(res, code, msg) {
 };
 
 // Are we on a mongos?
-var isMaster = db.runCommand("ismaster");
-assert.commandWorked(isMaster);
-var isMongos = (isMaster.msg === "isdbgrid");
+var hello = db.runCommand("hello");
+assert.commandWorked(hello);
+var isMongos = (hello.msg === "isdbgrid");
 
 // Obtain a list of all commands.
 let res = db.runCommand({listCommands: 1});
