@@ -9,13 +9,12 @@
  * retries transactions that fail due to implicit collection creation.
  *
  * (2) When a network connection to the mongo shell is closed, attempting to call
- * Mongo.prototype.runCommand() and Mongo.prototype.runCommandWithMetadata() throws a JavaScript
- * exception. This override catches these exceptions (i.e. ones where isNetworkError() returns true)
- * and automatically re-sends the command request to the server, or propagates the error if the
- * command should already be using the shell's existing retryability logic. The goal of this
- * override is to implement retry logic such that the assertions within our existing JavaScript
- * tests still pass despite stepdowns of replica set primaries (optionally in sharded clusters)
- * happening in the background.
+ * Mongo.prototype.runCommand() throws a JavaScript exception. This override catches these
+ * exceptions (i.e. ones where isNetworkError() returns true) and automatically re-sends the
+ * command request to the server, or propagates the error if the command should already be using
+ * the shell's existing retryability logic. The goal of this override is to implement retry logic
+ * such that the assertions within our existing JavaScript tests still pass despite stepdowns of
+ * replica set primaries (optionally in sharded clusters) happening in the background.
  *
  * These two overrides are unified to simplify the retry logic.
  *
@@ -74,17 +73,14 @@ const kNonRetryableCommands = new Set([
     // Commands that take write concern and do not support txnNumbers.
     "_configsvrAddShard",
     "_configsvrAddShardToZone",
-    "_configsvrCommitChunkMerge",
     "_configsvrCommitChunksMerge",
     "_configsvrCommitChunkMigration",
     "_configsvrCommitChunkSplit",
     "_configsvrCreateDatabase",
-    "_configsvrEnableSharding",
     "_configsvrMoveChunk",
-    "_configsvrMovePrimary",
+    "_configsvrMoveRange",
     "_configsvrRemoveShard",
     "_configsvrRemoveShardFromZone",
-    "_configsvrShardCollection",
     "_configsvrUpdateZoneKeyRange",
     "_mergeAuthzCollections",
     "_recvChunkStart",
@@ -96,8 +92,6 @@ const kNonRetryableCommands = new Set([
     "convertToCapped",
     "create",
     "createIndexes",
-    "createRole",
-    "createUser",
     "deleteIndexes",
     "drop",
     "dropAllRolesFromDatabase",
@@ -108,14 +102,7 @@ const kNonRetryableCommands = new Set([
     "dropUser",
     "emptycapped",
     "godinsert",
-    "grantPrivilegesToRole",
-    "grantRolesToRole",
-    "grantRolesToUser",
-    "mapreduce.shardedfinish",
-    "renameCollection",
-    "revokePrivilegesFromRole",
-    "revokeRolesFromRole",
-    "revokeRolesFromUser",
+    "internalRenameIfOptionsAndIndexesMatch",
     "updateRole",
     "updateUser",
 ]);
@@ -126,6 +113,8 @@ const kNonRetryableCommands = new Set([
 const kAcceptableNonRetryableCommands = new Set([
     "create",
     "createIndexes",
+    "createRole",
+    "createUser",
     "deleteIndexes",
     "drop",
     "dropDatabase",  // Already ignores NamespaceNotFound errors, so not handled below.
@@ -146,9 +135,14 @@ function isRetryableReadCmdName(cmdName) {
 // given command type.
 function isAcceptableRetryFailedResponse(cmdName, res) {
     assert(!res.ok, res);
+    // These codes are uniquely returned from user_management_commands.cpp
+    const kErrorCodeRoleAlreadyExists = 51002;
+    const kErrorCodeUserAlreadyExists = 51003;
     return ((cmdName === "create" && res.code === ErrorCodes.NamespaceExists) ||
             (cmdName === "createIndexes" && res.code === ErrorCodes.IndexAlreadyExists) ||
             (cmdName === "drop" && res.code === ErrorCodes.NamespaceNotFound) ||
+            ((cmdName == "createUser") && (res.code === kErrorCodeUserAlreadyExists)) ||
+            ((cmdName == "createRole") && (res.code === kErrorCodeRoleAlreadyExists)) ||
             ((cmdName === "dropIndexes" || cmdName === "deleteIndexes") &&
              res.code === ErrorCodes.IndexNotFound));
 }
@@ -419,9 +413,10 @@ function appendReadAndWriteConcern(conn, dbName, cmdName, cmdObj) {
         }
     } else if (cmdName === "aggregate") {
         if (OverrideHelpers.isAggregationWithListLocalSessionsStage(cmdName, cmdObj) ||
-            OverrideHelpers.isAggregationWithChangeStreamStage(cmdName, cmdObj)) {
-            // The $listLocalSessions stage can only be used with readConcern={level: "local"},
-            // and the $changeStream stage can only be used with
+            OverrideHelpers.isAggregationWithChangeStreamStage(cmdName, cmdObj) ||
+            OverrideHelpers.isAggregationWithCurrentOpStage(cmdName, cmdObj)) {
+            // The $listLocalSessions and $currentOp stages can only be used with
+            // readConcern={level: "local"}, and the $changeStream stage can only be used with
             // readConcern={level: "majority"}.
             shouldForceReadConcern = false;
         }
@@ -740,25 +735,6 @@ function retryWithTxnOverride(res, conn, dbName, cmdName, cmdObj, lsid, logError
         assert.gt(ops.length, 0);
         abortTransaction(conn, lsid, txnOptions.txnNumber);
 
-        const fcv =
-            conn.getDB("admin").system.version.findOne({_id: 'featureCompatibilityVersion'});
-        if (fcv.version === "4.2" && !fcv.hasOwnProperty("targetVersion")) {
-            // With FCV 4.2, implicit collection creation via commands that insert are not
-            // permitted inside multi-document transactions.
-            // If the command inserted data and is not supported in a transaction, we assume it
-            // failed because the collection did not exist. We will create the collection and retry
-            // the entire transaction. We should not receive this error in this override for any
-            // other reason.
-            // Tests that expect collections to not exist will have to be skipped.
-            if (kCmdsThatInsert.has(cmdName) &&
-                includesErrorCode(res, ErrorCodes.OperationNotSupportedInTransaction)) {
-                const collName = cmdObj[cmdName];
-                createCollectionExplicitly(conn, dbName, collName, lsid);
-
-                return retryEntireTransaction(conn, lsid);
-            }
-        }
-
         // Transaction statements cannot be retried, but retryable codes are expected to succeed
         // on full transaction retry.
         if (configuredForNetworkRetry() && RetryableWritesUtil.isRetryableCode(res.code)) {
@@ -1125,7 +1101,7 @@ function runCommandOverride(conn, dbName, cmdName, cmdObj, clientFunction, makeF
         // Many tests run queries that are expected to fail. In this case, when we wrap CRUD ops
         // in transactions, the transaction including the failed query will not be able to
         // commit. This override expects transactions to be able to commit. Rather than
-        // blacklisting all tests containing queries that are expected to fail, we clear the ops
+        // denylisting all tests containing queries that are expected to fail, we clear the ops
         // list when we return an error to the test so we do not retry the failed query.
         if (configuredForTxnOverride() && !isNested() && hasError(res) && (ops.length > 0)) {
             logMsgFull("Clearing ops on failed command",
