@@ -115,7 +115,6 @@ if (typeof _threadInject != "undefined") {
     // Helper class for running tests in parallel.  It assembles a set of tests
     // and then calls assert.parallelests to run them.
     ParallelTester = function() {
-        assert.neq(db.getMongo().writeMode(), "legacy", "wrong shell write mode");
         this.params = new Array();
     };
 
@@ -164,12 +163,6 @@ if (typeof _threadInject != "undefined") {
             "run_program1.js",
             "bench_test1.js",
 
-            // These tests use the getLastError command, which is unsafe to use in this environment,
-            // since a previous test's cursors could be garbage collected in the middle of the next
-            // test, which would reset the last error associated with the shell's client.
-            "dropdb_race.js",
-            "bulk_legacy_enforce_gle.js",
-
             // These tests use getLog to examine the logs. Tests which do so shouldn't be run in
             // this suite because any test being run at the same time could conceivably spam the
             // logs so much that the line they are looking for has been rotated off the server's
@@ -195,11 +188,8 @@ if (typeof _threadInject != "undefined") {
             // Assumes that other tests are not creating cursors.
             "kill_cursors.js",
 
-            // This test takes an IX global lock on an async thread, then runs CRUD ops needing IX
-            // locks. If a concurrent JS test queues a global X lock after the async thread takes
-            // the IX lock, and before the CRUD ops get IX locks, then there's a deadlock: the async
-            // thread will not release the acquired IX lock until the CRUD ops have finished.
-            "background_validation.js",
+            // Assumes that other tests are not starting operations.
+            "currentop_shell.js",
 
             // These tests check global command counters.
             "find_and_modify_metrics.js",
@@ -210,36 +200,59 @@ if (typeof _threadInject != "undefined") {
             "views/views_all_commands.js",        // Drops test DB.
             "views/view_with_invalid_dbname.js",  // Puts invalid view definitions in system.views.
 
-            // This test works close to the BSON document limit for entries in the durable catalog,
-            // so running it in parallel with other tests will cause failures.
-            "long_collection_names.js",
-
             // This test causes collMod commands to hang, which interferes with other tests running
             // collMod.
             "crud_ops_do_not_throw_locktimeout.js",
 
             // Can fail if isMaster takes too long on a loaded machine.
             "dbadmin.js",
-        ]);
 
-        // The following tests cannot run when shell readMode is legacy.
-        if (db.getMongo().readMode() === "legacy") {
-            var requires_find_command = [
-                "apply_ops_system_dot_views.js",
-                "internal_apply_oplog_update.js",
-                "merge_sort_collation.js",
-                "update_pipeline_shell_helpers.js",
-                "update_with_pipeline.js",
-                "verify_update_mods.js",
-                "views/dbref_projection.js",
-                "views/views_aggregation.js",
-                "views/views_change.js",
-                "views/views_drop.js",
-                "views/views_find.js",
-                "wildcard_index_collation.js"
-            ];
-            Object.assign(skipTests, makeKeys(requires_find_command));
-        }
+            // Other tests will fail while the requireApiVersion server parameter is set.
+            "require_api_version.js",
+
+            // This sets the 'disablePipelineOptimization' failpoint, which causes other tests
+            // running in parallel to fail if they were expecting their pipelines to be optimized.
+            "type_bracket.js",
+
+            // This test updates global memory usage counters in the bucket catalog in a way that
+            // may affect other time-series tests running concurrently.
+            "timeseries/timeseries_idle_buckets.js",
+
+            // Assumes that other tests are not creating API version 1 incompatible data.
+            "validate_db_metadata_command.js",
+
+            // The tests in 'bench_test*.js' files use 'benchRun()'. The main purpose of
+            // 'benchRun()' is for performance testing and the 'benchRun()' implementation itself
+            // launches multiple threads internally, it's not necessary to keep 'bench_test*.js'
+            // within the parallel test job.
+            "bench_test1.js",
+            "bench_test2.js",
+
+            // These tests cause deletes and updates to hang, which may affect other tests running
+            // concurrently.
+            "timeseries/timeseries_delete_hint.js",
+            "timeseries/timeseries_update_hint.js",
+            "timeseries/timeseries_delete_concurrent.js",
+            "timeseries/timeseries_update_concurrent.js",
+
+            // These tests rely on no writes happening that would force oplog truncation.
+            "write_change_stream_pit_preimage_in_transaction.js",
+            "write_change_stream_pit_preimage.js",
+
+            // These tests convert a non-unique index to a unique one, which is not compatible
+            // when running against inMemory storage engine variants. Since this test only fails
+            // in the parallel tester, which does not respect test tags, we omit the tests
+            // instead of manually checking TestData values in the mongo shell for the Evergreen
+            // variant.
+            "collmod_convert_index_uniqueness.js",
+            "collmod_convert_to_unique_apply_ops.js",
+            "collmod_convert_to_unique_violations.js",
+            "collmod_convert_to_unique_violations_size_limit.js",
+
+            // TODO (SERVER-63228): Remove this exclusion once the feature flag is enabled by
+            // default.
+            "timeseries/timeseries_index_ttl_partial.js",
+        ]);
 
         // Get files, including files in subdirectories.
         var getFilesRecursive = function(dir) {
@@ -256,7 +269,11 @@ if (typeof _threadInject != "undefined") {
         };
 
         // Transactions are not supported on standalone nodes so we do not run them here.
-        let txnsTestFiles = getFilesRecursive("jstests/core/txns").map(f => ("txns/" + f.baseName));
+        // NOTE: We need to take substring of the full test path to ensure that 'jstests/core/' is
+        // not included.
+        const txnsTestFiles =
+            getFilesRecursive("jstests/core/txns/")
+                .map(fullPathToTest => fullPathToTest.name.substring("jstests/core/".length));
         Object.assign(skipTests, makeKeys(txnsTestFiles));
 
         var parallelFilesDir = "jstests/core";
@@ -296,12 +313,12 @@ if (typeof _threadInject != "undefined") {
             parallelFilesDir + "/profile_query_hash.js",
             parallelFilesDir + "/profile_sampling.js",
             parallelFilesDir + "/profile_update.js",
+            parallelFilesDir + "/cached_plan_trial_does_not_discard_work.js",
 
             // These tests rely on a deterministically refreshable logical session cache. If they
             // run in parallel, they could interfere with the cache and cause failures.
             parallelFilesDir + "/list_all_local_sessions.js",
             parallelFilesDir + "/list_all_sessions.js",
-            parallelFilesDir + "/list_local_sessions.js",
             parallelFilesDir + "/list_sessions.js",
         ];
         var serialTests = makeKeys(serialTestsArr);
@@ -419,7 +436,7 @@ if (typeof CountDownLatch !== 'undefined') {
         //       prototype are lost during the serialization to BSON that occurs
         //       when passing data to a child thread.
 
-        this.awaitLatch = function() {
+        this.await = function() {
             CountDownLatch._await(this._descriptor);
         };
         this.countDown = function() {
