@@ -50,7 +50,7 @@ var BackupRestoreTest = function(options) {
             let iteration = 0;
 
             var coll = db.getSiblingDB(dbName).getCollection(collectionName);
-            coll.ensureIndex({x: 1});
+            coll.createIndex({x: 1});
 
             var largeValue = new Array(1024).join('L');
 
@@ -109,7 +109,8 @@ var BackupRestoreTest = function(options) {
 
         // Returns the pid of the started mongo shell so the CRUD test client can be terminated
         // without waiting for its execution to finish.
-        return startMongoProgramNoConnect(MongoRunner.mongoShellPath,
+        let shellPath = MongoRunner.getMongoShellPath();
+        return startMongoProgramNoConnect(shellPath,
                                           '--eval',
                                           '(' + crudClientCmds + ')("' + dbName + '", "' +
                                               collectionName + '", ' + numNodes + ')',
@@ -123,8 +124,9 @@ var BackupRestoreTest = function(options) {
         // Launch FSM client
         const suite = 'concurrency_replication_for_backup_restore';
         const resmokeCmd = 'python buildscripts/resmoke.py run --shuffle --continueOnFailure' +
-            ' --repeat=99999 --mongo=' + MongoRunner.mongoShellPath +
-            ' --shellConnString=mongodb://' + host + ' --suites=' + suite;
+            ' --repeat=99999 --internalParam=is_inner_level --mongo=' +
+            MongoRunner.getMongoShellPath() + ' --shellConnString=mongodb://' + host +
+            ' --suites=' + suite;
 
         // Returns the pid of the FSM test client so it can be terminated without waiting for its
         // execution to finish.
@@ -308,7 +310,11 @@ var BackupRestoreTest = function(options) {
         if (!_isWindows()) {
             // The mongo shell calls TerminateProcess() on Windows rather than more gracefully
             // interrupting resmoke.py test execution.
-            assert.eq(130, exitCode, 'expected resmoke.py to exit due to being interrupted');
+
+            // resmoke.py may exit cleanly on SIGINT returning 130, or uncleanly in which case
+            // stopMongoProgramByPid returns -SIGINT.
+            assert(exitCode == 130 || exitCode == -kSIGINT,
+                   'expected resmoke.py to exit due to being interrupted');
         }
 
         // Make sure the databases are not in a drop-pending state. This can happen if we
