@@ -1,8 +1,10 @@
-/*
+/**
  * Tests query/command option $maxTimeMS.
-
+ *
  * Creates a sharded cluster.
- * @tags: [requires_sharding]
+ * @tags: [
+ *   requires_sharding,
+ * ]
  */
 (function() {
 "use strict";
@@ -42,11 +44,7 @@ function executeTest(db, isMongos) {
         error = assert.throws(function() {
             cursor.itcount();
         }, [], "expected query to abort due to time limit");
-        // TODO SERVER-32565: The error should always be MaxTimeMSExpired, but there are rare cases
-        // where interrupting javascript execution on the server with a stepdown or timeout results
-        // in an error code of InternalError or Interrupted instead, so we also accept those here.
-        assert.commandFailedWithCode(
-            error, [ErrorCodes.MaxTimeMSExpired, ErrorCodes.Interrupted, ErrorCodes.InternalError]);
+        assert.commandFailedWithCode(error, ErrorCodes.MaxTimeMSExpired);
     })();
 
     //
@@ -102,11 +100,7 @@ function executeTest(db, isMongos) {
             cursor.next();
             cursor.next();
         }, [], "expected batch 2 (getmore) to abort due to time limit");
-        // TODO SERVER-32565: The error should always be MaxTimeMSExpired, but there are rare cases
-        // where interrupting javascript execution on the server with a stepdown or timeout results
-        // in an error code of InternalError or Interrupted instead, so we also accept those here.
-        assert.commandFailedWithCode(
-            error, [ErrorCodes.MaxTimeMSExpired, ErrorCodes.Interrupted, ErrorCodes.InternalError]);
+        assert.commandFailedWithCode(error, ErrorCodes.MaxTimeMSExpired);
     })();
 
     //
@@ -168,11 +162,7 @@ function executeTest(db, isMongos) {
         error = assert.throws(function() {
             cursor.itcount();
         }, [], "expected find() to abort due to time limit");
-        // TODO SERVER-32565: The error should always be MaxTimeMSExpired, but there are rare cases
-        // where interrupting javascript execution on the server with a stepdown or timeout results
-        // in an error code of InternalError or Interrupted instead, so we also accept those here.
-        assert.commandFailedWithCode(
-            error, [ErrorCodes.MaxTimeMSExpired, ErrorCodes.Interrupted, ErrorCodes.InternalError]);
+        assert.commandFailedWithCode(error, ErrorCodes.MaxTimeMSExpired);
     })();
 
     //
@@ -330,20 +320,9 @@ function executeTest(db, isMongos) {
                                      ErrorCodes.BadValue);
         assert.commandFailedWithCode(db.runCommand({ping: 1, maxTimeMS: {}}), ErrorCodes.BadValue);
 
-        // Verify that the maxTimeMS command argument can be sent with $query-wrapped commands.
-        cursor = t.getDB().$cmd.find({ping: 1, maxTimeMS: 0}).limit(-1);
-        cursor._ensureSpecial();
-        assert.eq(1, cursor.next().ok);
-
         // Verify that the server rejects invalid command argument $maxTimeMS.
-        cursor = t.getDB().$cmd.find({ping: 1, $maxTimeMS: 0}).limit(-1);
-        cursor._ensureSpecial();
-        assert.commandFailed(cursor.next());
-
-        // Verify that the $maxTimeMS query option can't be sent with $query-wrapped commands.
-        cursor = t.getDB().$cmd.find({ping: 1}).limit(-1).maxTimeMS(0);
-        cursor._ensureSpecial();
-        assert.commandFailed(cursor.next());
+        assert.commandFailed(t.getDB().runCommand({ping: 1, $maxTimeMS: 0}),
+                             [ErrorCodes.InvalidOptions, 40415]);
     })();
 
     //
@@ -462,10 +441,10 @@ function executeTest(db, isMongos) {
     })();
 
     //
-    // Test that maxTimeMS is accepted by commands that have an option whitelist.
+    // Test that maxTimeMS is accepted by commands that have an option allowlist.
     //
-    (function testCommandsWithOptionWhitelist() {
-        const t = db.max_time_ms_option_whitelist;
+    (function testCommandsWithOptionAllowlist() {
+        const t = db.max_time_ms_option_allowlist;
         // The namespace must exist for collMod to work.
         assert.commandWorked(t.insert({x: 1}));
 
