@@ -25,34 +25,47 @@ const isPrimaryOnly = "primary only";
 const allCommands = {
     _addShard: {skip: isPrimaryOnly},
     _cloneCollectionOptionsFromPrimaryShard: {skip: isPrimaryOnly},
+    _configsvrAbortReshardCollection: {skip: isPrimaryOnly},
     _configsvrAddShard: {skip: isPrimaryOnly},
     _configsvrAddShardToZone: {skip: isPrimaryOnly},
     _configsvrBalancerCollectionStatus: {skip: isPrimaryOnly},
     _configsvrBalancerStart: {skip: isPrimaryOnly},
     _configsvrBalancerStatus: {skip: isPrimaryOnly},
     _configsvrBalancerStop: {skip: isPrimaryOnly},
+    _configsvrCleanupReshardCollection: {skip: isPrimaryOnly},
+    _configsvrCollMod: {skip: isAnInternalCommand},
     _configsvrClearJumboFlag: {skip: isPrimaryOnly},
-    _configsvrCommitChunkMerge: {skip: isPrimaryOnly},
     _configsvrCommitChunksMerge: {skip: isPrimaryOnly},
     _configsvrCommitChunkMigration: {skip: isPrimaryOnly},
     _configsvrCommitChunkSplit: {skip: isPrimaryOnly},
+    _configsvrCommitIndex: {skip: isPrimaryOnly},
     _configsvrCommitMovePrimary: {skip: isPrimaryOnly},
-    _configsvrCreateCollection: {skip: isPrimaryOnly},
+    _configsvrCommitReshardCollection: {skip: isPrimaryOnly},
+    _configsvrConfigureCollectionBalancing: {skip: isPrimaryOnly},
     _configsvrCreateDatabase: {skip: isPrimaryOnly},
-    _configsvrDropCollection: {skip: isPrimaryOnly},
-    _configsvrDropDatabase: {skip: isPrimaryOnly},
-    _configsvrEnableSharding: {skip: isPrimaryOnly},
+    _configsvrDropIndexCatalogEntry: {skip: isPrimaryOnly},
     _configsvrEnsureChunkVersionIsGreaterThan: {skip: isPrimaryOnly},
     _configsvrMoveChunk: {skip: isPrimaryOnly},
-    _configsvrMovePrimary: {skip: isPrimaryOnly},
+    _configsvrMoveRange: {skip: isPrimaryOnly},
     _configsvrRefineCollectionShardKey: {skip: isPrimaryOnly},
+    _configsvrRemoveChunks: {skip: isPrimaryOnly},
     _configsvrRemoveShard: {skip: isPrimaryOnly},
     _configsvrRemoveShardFromZone: {skip: isPrimaryOnly},
-    _configsvrRepairShardedCollectionChunksHistory: {skip: isAnInternalCommand},
-    _configsvrShardCollection: {skip: isPrimaryOnly},
+    _configsvrRemoveTags: {skip: isPrimaryOnly},
+    _configsvrRepairShardedCollectionChunksHistory: {skip: isPrimaryOnly},
+    _configsvrRenameCollectionMetadata: {skip: isPrimaryOnly},
+    _configsvrReshardCollection: {skip: isPrimaryOnly},
+    _configsvrRunRestore: {skip: isPrimaryOnly},
+    _configsvrSetAllowMigrations: {skip: isPrimaryOnly},
+    _configsvrSetClusterParameter: {skip: isPrimaryOnly},
+    _configsvrSetUserWriteBlockMode: {skip: isPrimaryOnly},
     _configsvrUpdateZoneKeyRange: {skip: isPrimaryOnly},
     _flushDatabaseCacheUpdates: {skip: isPrimaryOnly},
+    _flushDatabaseCacheUpdatesWithWriteConcern: {skip: isPrimaryOnly},
+    _flushReshardingStateChange: {skip: isPrimaryOnly},
     _flushRoutingTableCacheUpdates: {skip: isPrimaryOnly},
+    _flushRoutingTableCacheUpdatesWithWriteConcern: {skip: isPrimaryOnly},
+    _getAuditConfigGeneration: {skip: isNotAUserDataRead},
     _getNextSessionMods: {skip: isPrimaryOnly},
     _getUserCacheGeneration: {skip: isNotAUserDataRead},
     _hashBSONElement: {skip: isNotAUserDataRead},
@@ -62,39 +75,83 @@ const allCommands = {
     _migrateClone: {skip: isPrimaryOnly},
     _recvChunkAbort: {skip: isPrimaryOnly},
     _recvChunkCommit: {skip: isPrimaryOnly},
+    _recvChunkReleaseCritSec: {skip: isPrimaryOnly},
     _recvChunkStart: {skip: isPrimaryOnly},
     _recvChunkStatus: {skip: isPrimaryOnly},
+    _shardsvrAbortReshardCollection: {skip: isPrimaryOnly},
+    _shardsvrCleanupReshardCollection: {skip: isPrimaryOnly},
     _shardsvrCloneCatalogData: {skip: isPrimaryOnly},
+    _shardsvrCompactStructuredEncryptionData: {skip: isPrimaryOnly},
+    _shardsvrRegisterIndex: {skip: isPrimaryOnly},
+    _shardsvrCommitIndexParticipant: {skip: isPrimaryOnly},
+    _shardsvrCommitReshardCollection: {skip: isPrimaryOnly},
+    _shardsvrDropCollection: {skip: isPrimaryOnly},
+    _shardsvrCreateCollection: {skip: isPrimaryOnly},
+    _shardsvrDropCollectionIfUUIDNotMatching: {skip: isNotAUserDataRead},
+    _shardsvrDropCollectionParticipant: {skip: isPrimaryOnly},
+    _shardsvrDropIndexCatalogEntryParticipant: {skip: isPrimaryOnly},
+    _shardsvrDropIndexes: {skip: isAnInternalCommand},
+    _shardsvrCreateCollectionParticipant: {skip: isPrimaryOnly},
+    _shardsvrGetStatsForBalancing: {skip: isPrimaryOnly},
+    _shardsvrJoinMigrations: {skip: isAnInternalCommand},
     _shardsvrMovePrimary: {skip: isPrimaryOnly},
+    _shardsvrMoveRange: {skip: isPrimaryOnly},
+    _shardsvrRenameCollection: {skip: isPrimaryOnly},
+    _shardsvrRenameCollectionParticipant: {skip: isAnInternalCommand},
+    _shardsvrRenameCollectionParticipantUnblock: {skip: isAnInternalCommand},
+    _shardsvrDropDatabase: {skip: isPrimaryOnly},
+    _shardsvrDropDatabaseParticipant: {skip: isPrimaryOnly},
+    _shardsvrReshardCollection: {skip: isPrimaryOnly},
+    _shardsvrReshardingOperationTime: {skip: isPrimaryOnly},
+    _shardsvrRefineCollectionShardKey: {skip: isPrimaryOnly},
     _shardsvrSetAllowMigrations: {skip: isPrimaryOnly},
-    _shardsvrShardCollection: {skip: isPrimaryOnly},
+    _shardsvrSetClusterParameter: {skip: isAnInternalCommand},
+    _shardsvrSetUserWriteBlockMode: {skip: isPrimaryOnly},
+    _shardsvrUnregisterIndex: {skip: isPrimaryOnly},
+    _shardsvrCollMod: {skip: isPrimaryOnly},
+    _shardsvrCollModParticipant: {skip: isAnInternalCommand},
+    _shardsvrParticipantBlock: {skip: isAnInternalCommand},
     _transferMods: {skip: isPrimaryOnly},
+    _vectorClockPersist: {skip: isPrimaryOnly},
+    abortReshardCollection: {skip: isPrimaryOnly},
     abortTransaction: {skip: isPrimaryOnly},
     aggregate: {
         command: {aggregate: collName, pipeline: [{$match: {}}], cursor: {}},
         expectFailure: true,
         expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary,
     },
+    analyze: {skip: isPrimaryOnly},
     appendOplogNote: {skip: isPrimaryOnly},
     applyOps: {skip: isPrimaryOnly},
     authenticate: {skip: isNotAUserDataRead},
-    autoSplitVector: {skip: isNotAUserDataRead},
-    availableQueryOptions: {skip: isNotAUserDataRead},
+    autoSplitVector: {skip: isPrimaryOnly},
     buildInfo: {skip: isNotAUserDataRead},
     captrunc: {skip: isPrimaryOnly},
     checkShardingIndex: {skip: isPrimaryOnly},
     cleanupOrphaned: {skip: isPrimaryOnly},
+    cleanupReshardCollection: {skip: isPrimaryOnly},
     clearLog: {skip: isNotAUserDataRead},
     cloneCollectionAsCapped: {skip: isPrimaryOnly},
+    clusterAbortTransaction: {skip: "already tested by 'abortTransaction' tests on mongos"},
+    clusterAggregate: {skip: "already tested by 'aggregate' tests on mongos"},
+    clusterCommitTransaction: {skip: "already tested by 'commitTransaction' tests on mongos"},
+    clusterDelete: {skip: "already tested by 'delete' tests on mongos"},
+    clusterFind: {skip: "already tested by 'find' tests on mongos"},
+    clusterGetMore: {skip: "already tested by 'getMore' tests on mongos"},
+    clusterInsert: {skip: "already tested by 'insert' tests on mongos"},
+    clusterUpdate: {skip: "already tested by 'update' tests on mongos"},
     collMod: {skip: isPrimaryOnly},
     collStats: {
         command: {aggregate: collName, pipeline: [{$collStats: {count: {}}}], cursor: {}},
         expectFailure: true,
         expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary,
     },
+    commitReshardCollection: {skip: isPrimaryOnly},
     commitTransaction: {skip: isPrimaryOnly},
     compact: {skip: isNotAUserDataRead},
+    compactStructuredEncryptionData: {skip: isPrimaryOnly},
     configureFailPoint: {skip: isNotAUserDataRead},
+    configureCollectionBalancing: {skip: isPrimaryOnly},
     connPoolStats: {skip: isNotAUserDataRead},
     connPoolSync: {skip: isNotAUserDataRead},
     connectionStatus: {skip: isNotAUserDataRead},
@@ -131,6 +188,13 @@ const allCommands = {
         expectFailure: true,
         expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary,
     },
+    donorAbortMigration: {skip: isPrimaryOnly},
+    donorForgetMigration: {skip: isPrimaryOnly},
+    donorStartMigration: {skip: isPrimaryOnly},
+    donorWaitForMigrationToCommit: {skip: isPrimaryOnly},
+    abortShardSplit: {skip: isPrimaryOnly},
+    commitShardSplit: {skip: isPrimaryOnly},
+    forgetShardSplit: {skip: isPrimaryOnly},
     driverOIDTest: {skip: isNotAUserDataRead},
     drop: {skip: isPrimaryOnly},
     dropAllRolesFromDatabase: {skip: isPrimaryOnly},
@@ -159,19 +223,14 @@ const allCommands = {
     flushRouterConfig: {skip: isNotAUserDataRead},
     fsync: {skip: isNotAUserDataRead},
     fsyncUnlock: {skip: isNotAUserDataRead},
-    geoSearch: {
-        command: {
-            geoSearch: collName,
-            search: {},
-            near: [-42, 42],
-        },
-        expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary
-    },
+    getAuditConfig: {skip: isNotAUserDataRead},
+    getChangeStreamState: {skip: isNotAUserDataRead},
+    getClusterParameter: {skip: isNotAUserDataRead},
     getCmdLineOpts: {skip: isNotAUserDataRead},
     getDatabaseVersion: {skip: isNotAUserDataRead},
     getDefaultRWConcern: {skip: isNotAUserDataRead},
     getDiagnosticData: {skip: isNotAUserDataRead},
+    getFreeMonitoringStatus: {skip: isNotAUserDataRead},
     getLastError: {skip: isPrimaryOnly},
     getLog: {skip: isNotAUserDataRead},
     getMore: {
@@ -190,6 +249,8 @@ const allCommands = {
     hello: {skip: isNotAUserDataRead},
     hostInfo: {skip: isNotAUserDataRead},
     httpClientRequest: {skip: isNotAUserDataRead},
+    exportCollection: {skip: isNotAUserDataRead},
+    importCollection: {skip: isNotAUserDataRead},
     insert: {skip: isPrimaryOnly},
     internalRenameIfOptionsAndIndexesMatch: {skip: isAnInternalCommand},
     invalidateUserCache: {skip: isNotAUserDataRead},
@@ -211,6 +272,12 @@ const allCommands = {
         expectFailure: true,
         expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary
     },
+    listDatabasesForAllTenants: {
+        command: {listDatabasesForAllTenants: 1},
+        isAdminCommand: true,
+        expectFailure: true,
+        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary
+    },
     listIndexes: {
         command: {listIndexes: collName},
         expectFailure: true,
@@ -232,9 +299,10 @@ const allCommands = {
         expectFailure: true,
         expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary,
     },
-    "mapreduce.shardedfinish": {skip: isAnInternalCommand},
     mergeChunks: {skip: isPrimaryOnly},
     moveChunk: {skip: isPrimaryOnly},
+    moveRange: {skip: isPrimaryOnly},
+    pinHistoryReplicated: {skip: isAnInternalCommand},
     ping: {skip: isNotAUserDataRead},
     planCacheClear: {skip: isNotAUserDataRead},
     planCacheClearFilters: {skip: isNotAUserDataRead},
@@ -243,6 +311,9 @@ const allCommands = {
     prepareTransaction: {skip: isPrimaryOnly},
     profile: {skip: isPrimaryOnly},
     reapLogicalSessionCacheNow: {skip: isNotAUserDataRead},
+    recipientForgetMigration: {skip: isPrimaryOnly},
+    recipientSyncData: {skip: isPrimaryOnly},
+    recipientVoteImportedFiles: {skip: isNotAUserDataRead},
     refreshLogicalSessionCacheNow: {skip: isNotAUserDataRead},
     refreshSessions: {skip: isNotAUserDataRead},
     reIndex: {skip: isNotAUserDataRead},
@@ -262,24 +333,28 @@ const allCommands = {
     replSetStepUp: {skip: isNotAUserDataRead},
     replSetSyncFrom: {skip: isNotAUserDataRead},
     replSetTest: {skip: isNotAUserDataRead},
+    replSetTestEgress: {skip: isNotAUserDataRead},
     replSetUpdatePosition: {skip: isNotAUserDataRead},
     replSetResizeOplog: {skip: isNotAUserDataRead},
-    resetError: {skip: isNotAUserDataRead},
     revokePrivilegesFromRole: {skip: isPrimaryOnly},
     revokeRolesFromRole: {skip: isPrimaryOnly},
     revokeRolesFromUser: {skip: isPrimaryOnly},
     rolesInfo: {skip: isPrimaryOnly},
+    rotateCertificates: {skip: isNotAUserDataRead},
     saslContinue: {skip: isPrimaryOnly},
     saslStart: {skip: isPrimaryOnly},
     serverStatus: {skip: isNotAUserDataRead},
+    setAuditConfig: {skip: isNotAUserDataRead},
     setCommittedSnapshot: {skip: isNotAUserDataRead},
     setDefaultRWConcern: {skip: isPrimaryOnly},
     setIndexCommitQuorum: {skip: isPrimaryOnly},
     setFeatureCompatibilityVersion: {skip: isPrimaryOnly},
-    setProfilingFilterGlobally: {skip: isNotAUserDataRead},
+    setFreeMonitoring: {skip: isPrimaryOnly},
     setParameter: {skip: isNotAUserDataRead},
     setShardVersion: {skip: isNotAUserDataRead},
-    shardConnPoolStats: {skip: isNotAUserDataRead},
+    setChangeStreamState: {skip: isNotAUserDataRead},
+    setClusterParameter: {skip: isNotAUserDataRead},
+    setUserWriteBlockMode: {skip: isPrimaryOnly},
     shardingState: {skip: isNotAUserDataRead},
     shutdown: {skip: isNotAUserDataRead},
     sleep: {skip: isNotAUserDataRead},
@@ -289,13 +364,25 @@ const allCommands = {
     startRecordingTraffic: {skip: isNotAUserDataRead},
     startSession: {skip: isNotAUserDataRead},
     stopRecordingTraffic: {skip: isNotAUserDataRead},
+    testDeprecation: {skip: isNotAUserDataRead},
+    testDeprecationInVersion2: {skip: isNotAUserDataRead},
+    testInternalTransactions: {skip: isNotAUserDataRead},
+    testRemoval: {skip: isNotAUserDataRead},
+    testReshardCloneCollection: {skip: isNotAUserDataRead},
+    testVersions1And2: {skip: isNotAUserDataRead},
+    testVersion2: {skip: isNotAUserDataRead},
     top: {skip: isNotAUserDataRead},
-    unsetSharding: {skip: isNotAUserDataRead},
     update: {skip: isPrimaryOnly},
     updateRole: {skip: isPrimaryOnly},
     updateUser: {skip: isPrimaryOnly},
     usersInfo: {skip: isPrimaryOnly},
     validate: {skip: isNotAUserDataRead},
+    validateDBMetadata: {
+        command: {validateDBMetadata: 1, apiParameters: {version: "1", strict: true}},
+        expectFailure: true,
+        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary,
+    },
+    voteCommitImportCollection: {skip: isNotAUserDataRead},
     voteCommitIndexBuild: {skip: isNotAUserDataRead},
     waitForFailPoint: {skip: isNotAUserDataRead},
     waitForOngoingChunkSplits: {skip: isNotAUserDataRead},
@@ -355,4 +442,4 @@ AllCommandsTest.testAllCommands(secondary, allCommands, function(test) {
 // Turn off maintenance mode and stop the test.
 assert.commandWorked(secondary.adminCommand({replSetMaintenance: 0}));
 rst.stopSet();
-})();
\ No newline at end of file
+})();
