@@ -1,12 +1,11 @@
 /**
  * Specifies for each command whether it is expected to send a databaseVersion, and verifies that
  * the commands match the specification.
- * @tags: [requires_fcv_44]
  */
 (function() {
 'use strict';
 
-load('jstests/sharding/libs/last_stable_mongos_commands.js');
+load('jstests/sharding/libs/last_lts_mongos_commands.js');
 
 function getNewDbName(dbName) {
     if (!getNewDbName.counter) {
@@ -177,7 +176,11 @@ function testCommandAfterDropRecreateDatabase(testCase, st) {
     // Drop and recreate the database through the second mongos. Insert the entry for the new
     // database explicitly to ensure it is assigned the other shard as the primary shard.
     assert.commandWorked(st.s1.getDB(dbName).dropDatabase());
-    let currDbVersion = {uuid: UUID(), lastMod: NumberInt(1)};
+    let currDbVersion = {
+        uuid: UUID(),
+        timestamp: Timestamp(dbVersionBefore.timestamp.getTime() + 1, 0),
+        lastMod: NumberInt(1)
+    };
     assert.commandWorked(st.s1.getDB("config").getCollection("databases").insert({
         _id: dbName,
         partitioned: false,
@@ -239,10 +242,12 @@ function testCommandAfterDropRecreateDatabase(testCase, st) {
 }
 
 let testCases = {
+    _getAuditConfigGeneration: {skip: "not on a user database", conditional: true},
     _hashBSONElement: {skip: "executes locally on mongos (not sent to any remote node)"},
     _isSelf: {skip: "executes locally on mongos (not sent to any remote node)"},
     _killOperations: {skip: "executes locally on mongos (not sent to any remote node)"},
     _mergeAuthzCollections: {skip: "always targets the config server"},
+    abortReshardCollection: {skip: "always targets the config server"},
     abortTransaction: {skip: "unversioned and uses special targetting rules"},
     addShard: {skip: "not on a user database"},
     addShardToZone: {skip: "not on a user database"},
@@ -263,14 +268,17 @@ let testCases = {
             }
         }
     },
+    analyze: {
+        skip: "unimplemented. Serves only as a stub."
+    },  // TODO SERVER-68055: Extend test to work with analyze
     appendOplogNote: {skip: "unversioned and executes on all shards"},
     authenticate: {skip: "does not forward command to primary shard"},
-    availableQueryOptions: {skip: "executes locally on mongos (not sent to any remote node)"},
     balancerCollectionStatus: {skip: "does not forward command to primary shard"},
     balancerStart: {skip: "not on a user database"},
     balancerStatus: {skip: "not on a user database"},
     balancerStop: {skip: "not on a user database"},
     buildInfo: {skip: "executes locally on mongos (not sent to any remote node)"},
+    cleanupReshardCollection: {skip: "always targets the config server"},
     clearJumboFlag: {skip: "does not forward command to primary shard"},
     clearLog: {skip: "executes locally on mongos (not sent to any remote node)"},
     collMod: {
@@ -291,8 +299,11 @@ let testCases = {
             },
         }
     },
+    commitReshardCollection: {skip: "always targets the config server"},
     commitTransaction: {skip: "unversioned and uses special targetting rules"},
     compact: {skip: "not allowed through mongos"},
+    compactStructuredEncryptionData: {skip: "requires encrypted collections"},
+    configureCollectionBalancing: {skip: "always targets the config server"},
     configureFailPoint: {skip: "executes locally on mongos (not sent to any remote node)"},
     connPoolStats: {skip: "executes locally on mongos (not sent to any remote node)"},
     connPoolSync: {skip: "executes locally on mongos (not sent to any remote node)"},
@@ -306,6 +317,7 @@ let testCases = {
             },
         }
     },
+    coordinateCommitTransaction: {skip: "unimplemented. Serves only as a stub."},
     count: {
         run: {
             sendsDbVersion: true,
@@ -446,7 +458,8 @@ let testCases = {
     },
     flushRouterConfig: {skip: "executes locally on mongos (not sent to any remote node)"},
     fsync: {skip: "broadcast to all shards"},
-    fsyncUnlock: {skip: "broadcast to all shards"},
+    getAuditConfig: {skip: "not on a user database", conditional: true},
+    getClusterParameter: {skip: "always targets the config server"},
     getCmdLineOpts: {skip: "executes locally on mongos (not sent to any remote node)"},
     getDefaultRWConcern: {skip: "executes locally on mongos (not sent to any remote node)"},
     getDiagnosticData: {skip: "executes locally on mongos (not sent to any remote node)"},
@@ -539,6 +552,7 @@ let testCases = {
     mergeChunks: {skip: "does not forward command to primary shard"},
     moveChunk: {skip: "does not forward command to primary shard"},
     movePrimary: {skip: "reads primary shard from sharding catalog with readConcern: local"},
+    moveRange: {skip: "does not forward command to primary shard"},
     multicast: {skip: "does not forward command to primary shard"},
     netstat: {skip: "executes locally on mongos (not sent to any remote node)"},
     ping: {skip: "executes locally on mongos (not sent to any remote node)"},
@@ -602,15 +616,17 @@ let testCases = {
     },
     repairShardedCollectionChunksHistory: {skip: "always targets the config server"},
     replSetGetStatus: {skip: "not supported in mongos"},
-    resetError: {skip: "not on a user database"},
+    reshardCollection: {skip: "does not forward command to primary shard"},
     revokePrivilegesFromRole: {skip: "always targets the config server"},
     revokeRolesFromRole: {skip: "always targets the config server"},
     revokeRolesFromUser: {skip: "always targets the config server"},
     rolesInfo: {skip: "always targets the config server"},
+    rotateCertificates: {skip: "executes locally on mongos (not sent to any remote node)"},
     saslContinue: {skip: "not on a user database"},
     saslStart: {skip: "not on a user database"},
     serverStatus: {skip: "executes locally on mongos (not sent to any remote node)"},
     setAllowMigrations: {skip: "not on a user database"},
+    setAuditConfig: {skip: "not on a user database", conditional: true},
     setDefaultRWConcern: {skip: "always targets the config server"},
     setIndexCommitQuorum: {
         run: {
@@ -628,16 +644,24 @@ let testCases = {
         }
     },
     setFeatureCompatibilityVersion: {skip: "not on a user database"},
-    setProfilingFilterGlobally: {skip: "executes locally on mongos (not sent to any remote node)"},
+    setFreeMonitoring:
+        {skip: "explicitly fails for mongos, primary mongod only", conditional: true},
     setParameter: {skip: "executes locally on mongos (not sent to any remote node)"},
+    setClusterParameter: {skip: "always targets the config server"},
+    setUserWriteBlockMode: {skip: "executes locally on mongos (not sent to any remote node)"},
     shardCollection: {skip: "does not forward command to primary shard"},
-    shardConnPoolStats: {skip: "does not forward command to primary shard"},
     shutdown: {skip: "does not forward command to primary shard"},
     split: {skip: "does not forward command to primary shard"},
     splitVector: {skip: "does not forward command to primary shard"},
     startRecordingTraffic: {skip: "executes locally on mongos (not sent to any remote node)"},
     startSession: {skip: "executes locally on mongos (not sent to any remote node)"},
     stopRecordingTraffic: {skip: "executes locally on mongos (not sent to any remote node)"},
+    testDeprecation: {skip: "executes locally on mongos (not sent to any remote node)"},
+    testDeprecationInVersion2: {skip: "executes locally on mongos (not sent to any remote node)"},
+    testInternalTransactions: {skip: "executes locally on mongos (not sent to any remote node)"},
+    testRemoval: {skip: "executes locally on mongos (not sent to any remote node)"},
+    testVersion2: {skip: "executes locally on mongos (not sent to any remote node)"},
+    testVersions1And2: {skip: "executes locally on mongos (not sent to any remote node)"},
     update: {
         run: {
             sendsDbVersion: true,
@@ -673,12 +697,22 @@ let testCases = {
             },
         }
     },
+    validateDBMetadata: {
+        run: {
+            // validateDBMetadata is always broadcast to all shards.
+            sendsDbVersion: false,
+            explicitlyCreateCollection: true,
+            command: function(dbName, collName) {
+                return {validateDBMetadata: 1, apiParameters: {version: "1"}};
+            },
+        }
+    },
     waitForFailPoint: {skip: "executes locally on mongos (not sent to any remote node)"},
     whatsmyuri: {skip: "executes locally on mongos (not sent to any remote node)"},
 };
 
-commandsRemovedFromMongosIn44.forEach(function(cmd) {
-    testCases[cmd] = {skip: "must define test coverage for 4.2 backwards compatibility"};
+commandsRemovedFromMongosSinceLastLTS.forEach(function(cmd) {
+    testCases[cmd] = {skip: "must define test coverage for latest version backwards compatibility"};
 });
 
 const st = new ShardingTest({shards: 2, mongos: 2});
@@ -701,14 +735,15 @@ assert.commandWorked(listCommandsRes);
     // After iterating through all the existing commands, ensure there were no additional test cases
     // that did not correspond to any mongos command.
     for (let key of Object.keys(testCases)) {
-        // We have defined real test cases for commands added in 4.4 so that the test cases are
-        // exercised in the regular suites, but because these test cases can't run in the last
-        // stable suite, we skip processing them here to avoid failing the below assertion.
-        // We have defined "skip" test cases for commands removed in 4.4 so the test case is defined
-        // in last stable suites (in which these commands still exist on the mongos), but these test
-        // cases won't be run in regular suites, so we skip processing them below as well.
-        if (commandsAddedToMongosIn44.includes(key) ||
-            commandsRemovedFromMongosIn44.includes(key)) {
+        // We have defined real test cases for commands added since the last LTS version so that the
+        // test cases are exercised in the regular suites, but because these test cases can't run in
+        // the last stable suite, we skip processing them here to avoid failing the below assertion.
+        // We have defined "skip" test cases for commands removed since the last LTS version so the
+        // test case is defined in last stable suites (in which these commands still exist on the
+        // mongos), but these test cases won't be run in regular suites, so we skip processing them
+        // below as well.
+        if (commandsAddedToMongosSinceLastLTS.includes(key) ||
+            commandsRemovedFromMongosSinceLastLTS.includes(key)) {
             continue;
         }
         assert(testCases[key].validated || testCases[key].conditional,
