@@ -23,7 +23,6 @@
  *
  * @tags: [
  *   does_not_support_stepdowns,
- *   requires_fcv_44,
  *   requires_majority_read_concern,
  *   requires_profiling,
  *   uses_transactions,
@@ -34,6 +33,12 @@
 
 load('jstests/libs/profiler.js');
 load("jstests/libs/logv2_helpers.js");
+load('jstests/sharding/libs/last_lts_mongod_commands.js');
+
+// TODO SERVER-50144 Remove this and allow orphan checking.
+// This test calls removeShard which can leave docs in config.rangeDeletions in state "pending",
+// therefore preventing orphans from being cleaned up.
+TestData.skipCheckOrphans = true;
 
 let db = "test";
 let coll = "foo";
@@ -74,35 +79,60 @@ let validateTestCase = function(test) {
 let testCases = {
     _addShard: {skip: "internal command"},
     _cloneCollectionOptionsFromPrimaryShard: {skip: "internal command"},
+    _configsvrAbortReshardCollection: {skip: "internal command"},
     _configsvrAddShard: {skip: "internal command"},
     _configsvrAddShardToZone: {skip: "internal command"},
     _configsvrBalancerCollectionStatus: {skip: "internal command"},
     _configsvrBalancerStart: {skip: "internal command"},
     _configsvrBalancerStatus: {skip: "internal command"},
     _configsvrBalancerStop: {skip: "internal command"},
+    _configsvrCleanupReshardCollection: {skip: "internal command"},
+    _configsvrCollMod: {skip: "internal command"},
     _configsvrClearJumboFlag: {skip: "internal command"},
-    _configsvrCommitChunkMerge: {skip: "internal command"},
     _configsvrCommitChunksMerge: {skip: "internal command"},
     _configsvrCommitChunkMigration: {skip: "internal command"},
     _configsvrCommitChunkSplit: {skip: "internal command"},
-    _configsvrCommitMovePrimary: {skip: "internal command"},
-    _configsvrCreateCollection: {skip: "internal command"},
+    _configsvrCommitIndex: {skip: "internal command"},
+    _configsvrCommitMovePrimary: {skip: "internal command"},  // Can be removed once 6.0 is last LTS
+    _configsvrCommitReshardCollection: {skip: "internal command"},
+    _configsvrConfigureAutoSplit: {
+        skip: "internal command"
+    },  // TODO SERVER-62374: remove this once 5.3 becomes last continuos release
+    _configsvrConfigureCollectionBalancing: {skip: "internal command"},
     _configsvrCreateDatabase: {skip: "internal command"},
-    _configsvrDropCollection: {skip: "internal command"},
-    _configsvrDropDatabase: {skip: "internal command"},
-    _configsvrEnableSharding: {skip: "internal command"},
+    _configsvrDropCollection:
+        {skip: "internal command"},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
+    _configsvrDropDatabase:
+        {skip: "internal command"},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
+    _configsvrDropIndexCatalogEntry: {skip: "internal command"},
+    _configsvrEnableSharding:
+        {skip: "internal command"},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
     _configsvrEnsureChunkVersionIsGreaterThan: {skip: "internal command"},
     _configsvrMoveChunk: {skip: "internal command"},
-    _configsvrMovePrimary: {skip: "internal command"},
+    _configsvrMovePrimary: {skip: "internal command"},  // Can be removed once 6.0 is last LTS
+    _configsvrMoveRange: {skip: "internal command"},
     _configsvrRefineCollectionShardKey: {skip: "internal command"},
+    _configsvrRemoveChunks: {skip: "internal command"},
     _configsvrRemoveShard: {skip: "internal command"},
     _configsvrRemoveShardFromZone: {skip: "internal command"},
+    _configsvrRemoveTags: {skip: "internal command"},
     _configsvrRenameCollection: {skip: "internal command"},
+    _configsvrRenameCollectionMetadata: {skip: "internal command"},
     _configsvrRepairShardedCollectionChunksHistory: {skip: "internal command"},
-    _configsvrShardCollection: {skip: "internal command"},
+    _configsvrReshardCollection: {skip: "internal command"},
+    _configsvrRunRestore: {skip: "internal command"},
+    _configsvrSetAllowMigrations: {skip: "internal command"},
+    _configsvrSetClusterParameter: {skip: "internal command"},
+    _configsvrSetUserWriteBlockMode: {skip: "internal command"},
+    _configsvrShardCollection:
+        {skip: "internal command"},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
     _configsvrUpdateZoneKeyRange: {skip: "internal command"},
     _flushDatabaseCacheUpdates: {skip: "internal command"},
+    _flushDatabaseCacheUpdatesWithWriteConcern: {skip: "internal command"},
+    _flushReshardingStateChange: {skip: "internal command"},
     _flushRoutingTableCacheUpdates: {skip: "internal command"},
+    _flushRoutingTableCacheUpdatesWithWriteConcern: {skip: "internal command"},
+    _getAuditConfigGeneration: {skip: "does not accept read or write concern"},
     _getNextSessionMods: {skip: "internal command"},
     _getUserCacheGeneration: {skip: "internal command"},
     _hashBSONElement: {skip: "internal command"},
@@ -112,17 +142,54 @@ let testCases = {
     _migrateClone: {skip: "internal command"},
     _recvChunkAbort: {skip: "internal command"},
     _recvChunkCommit: {skip: "internal command"},
+    _recvChunkReleaseCritSec: {skip: "internal command"},
     _recvChunkStart: {skip: "internal command"},
     _recvChunkStatus: {skip: "internal command"},
+    _shardsvrAbortReshardCollection: {skip: "internal command"},
+    _shardsvrCleanupReshardCollection: {skip: "internal command"},
     _shardsvrCloneCatalogData: {skip: "internal command"},
+    _shardsvrRegisterIndex: {skip: "internal command"},
+    _shardsvrCommitIndexParticipant: {skip: "internal command"},
+    _shardsvrCommitReshardCollection: {skip: "internal command"},
+    _shardsvrCompactStructuredEncryptionData: {skip: "internal command"},
+    _shardsvrCreateCollection: {skip: "internal command"},
+    _shardsvrCreateCollectionParticipant: {skip: "internal command"},
+    _shardsvrDropCollection: {skip: "internal command"},
+    _shardsvrDropCollectionIfUUIDNotMatching: {skip: "internal command"},
+    _shardsvrDropCollectionParticipant: {skip: "internal command"},
+    _shardsvrUnregisterIndex: {skip: "internal command"},
+    _shardsvrDropIndexCatalogEntryParticipant: {skip: "internal command"},
+    _shardsvrDropIndexes: {skip: "internal command"},
+    _shardsvrDropDatabase: {skip: "internal command"},
+    _shardsvrDropDatabaseParticipant: {skip: "internal command"},
+    _shardsvrGetStatsForBalancing: {skip: "internal command"},
+    _shardsvrJoinMigrations: {skip: "internal command"},
     _shardsvrMovePrimary: {skip: "internal command"},
+    _shardsvrMoveRange: {
+        skip:
+            "does not accept read or write concern (accepts writeConcern, but only explicitly and when _secondaryThrottle is true)"
+    },
+    _shardsvrRefineCollectionShardKey: {skip: "internal command"},
     _shardsvrRenameCollection: {skip: "internal command"},
+    _shardsvrRenameCollectionParticipant: {skip: "internal command"},
+    _shardsvrRenameCollectionParticipantUnblock: {skip: "internal command"},
+    _shardsvrReshardCollection: {skip: "internal command"},
+    _shardsvrReshardingOperationTime: {skip: "internal command"},
     _shardsvrSetAllowMigrations: {skip: "internal command"},
-    _shardsvrShardCollection: {skip: "internal command"},
+    _shardsvrSetClusterParameter: {skip: "internal command"},
+    _shardsvrSetUserWriteBlockMode: {skip: "internal command"},
+    _shardsvrCollMod: {skip: "internal command"},
+    _shardsvrCollModParticipant: {skip: "internal command"},
+    _shardsvrParticipantBlock: {skip: "internal command"},
+    _shardsvrShardCollection:
+        {skip: "internal command"},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
     _transferMods: {skip: "internal command"},
+    _vectorClockPersist: {skip: "internal command"},
+    abortReshardCollection: {skip: "does not accept read or write concern"},
     abortTransaction: {
         setUp: function(conn) {
-            assert.commandWorked(conn.getDB(db).runCommand({create: coll, writeConcern: {w: 1}}));
+            assert.commandWorked(
+                conn.getDB(db).runCommand({create: coll, writeConcern: {w: 'majority'}}));
             // Ensure that the dbVersion is known.
             assert.commandWorked(conn.getCollection(nss).insert({x: 1}, {writeConcern: {w: 1}}));
             assert.eq(1,
@@ -155,6 +222,7 @@ let testCases = {
         checkReadConcern: true,
         checkWriteConcern: true,
     },
+    analyze: {skip: "TODO SERVER-67772"},
     appendOplogNote: {
         command: {appendOplogNote: 1, data: {foo: 1}},
         checkReadConcern: false,
@@ -166,7 +234,6 @@ let testCases = {
     applyOps: {skip: "internal command"},
     authenticate: {skip: "does not accept read or write concern"},
     autoSplitVector: {skip: "internal command"},
-    availableQueryOptions: {skip: "internal command"},
     balancerCollectionStatus: {skip: "does not accept read or write concern"},
     balancerStart: {skip: "does not accept read or write concern"},
     balancerStatus: {skip: "does not accept read or write concern"},
@@ -175,6 +242,7 @@ let testCases = {
     captrunc: {skip: "test command"},
     checkShardingIndex: {skip: "does not accept read or write concern"},
     cleanupOrphaned: {skip: "only on shard server"},
+    cleanupReshardCollection: {skip: "does not accept read or write concern"},
     clearJumboFlag: {skip: "does not accept read or write concern"},
     clearLog: {skip: "does not accept read or write concern"},
     clone: {skip: "deprecated"},
@@ -186,6 +254,14 @@ let testCases = {
         checkReadConcern: false,
         checkWriteConcern: true,
     },
+    clusterAbortTransaction: {skip: "already tested by 'abortTransaction' tests on mongos"},
+    clusterAggregate: {skip: "already tested by 'aggregate' tests on mongos"},
+    clusterCommitTransaction: {skip: "already tested by 'commitTransaction' tests on mongos"},
+    clusterDelete: {skip: "already tested by 'delete' tests on mongos"},
+    clusterFind: {skip: "already tested by 'find' tests on mongos"},
+    clusterGetMore: {skip: "already tested by 'getMore' tests on mongos"},
+    clusterInsert: {skip: "already tested by 'insert' tests on mongos"},
+    clusterUpdate: {skip: "already tested by 'update' tests on mongos"},
     collMod: {
         setUp: function(conn) {
             assert.commandWorked(conn.getDB(db).runCommand({create: coll, writeConcern: {w: 1}}));
@@ -195,9 +271,11 @@ let testCases = {
         checkWriteConcern: true,
     },
     collStats: {skip: "does not accept read or write concern"},
+    commitReshardCollection: {skip: "does not accept read or write concern"},
     commitTransaction: {
         setUp: function(conn) {
-            assert.commandWorked(conn.getDB(db).runCommand({create: coll, writeConcern: {w: 1}}));
+            assert.commandWorked(
+                conn.getDB(db).runCommand({create: coll, writeConcern: {w: 'majority'}}));
             // Ensure that the dbVersion is known.
             assert.commandWorked(conn.getCollection(nss).insert({x: 1}, {writeConcern: {w: 1}}));
             assert.eq(1,
@@ -221,6 +299,11 @@ let testCases = {
         useLogs: true,
     },
     compact: {skip: "does not accept read or write concern"},
+    compactStructuredEncryptionData: {skip: "does not accept read or write concern"},
+    configureCollectionAutoSplitter: {
+        skip: "does not accept read or write concern"
+    },  // TODO SERVER-62374: remove this once 5.3 becomes last continuos release
+    configureCollectionBalancing: {skip: "does not accept read or write concern"},
     configureFailPoint: {skip: "does not accept read or write concern"},
     connPoolStats: {skip: "does not accept read or write concern"},
     connPoolSync: {skip: "internal command"},
@@ -294,6 +377,13 @@ let testCases = {
         checkReadConcern: true,
         checkWriteConcern: false,
     },
+    donorAbortMigration: {skip: "does not accept read or write concern"},
+    donorForgetMigration: {skip: "does not accept read or write concern"},
+    donorStartMigration: {skip: "does not accept read or write concern"},
+    donorWaitForMigrationToCommit: {skip: "does not accept read or write concern"},
+    abortShardSplit: {skip: "internal command"},
+    commitShardSplit: {skip: "internal command"},
+    forgetShardSplit: {skip: "internal command"},
     driverOIDTest: {skip: "internal command"},
     drop: {
         setUp: function(conn) {
@@ -389,22 +479,14 @@ let testCases = {
     forceerror: {skip: "test command"},
     fsync: {skip: "does not accept read or write concern"},
     fsyncUnlock: {skip: "does not accept read or write concern"},
-    geoSearch: {
-        setUp: function(conn) {
-            assert.commandWorked(conn.getDB(db).runCommand({
-                createIndexes: coll,
-                indexes: [{key: {loc: "geoHaystack", foo: 1}, bucketSize: 1, name: "foo"}],
-                writeConcern: {w: 1}
-            }));
-        },
-        command: {geoSearch: coll, search: {}, near: [0, 0], maxDistance: 1},
-        checkReadConcern: true,
-        checkWriteConcern: false,
-    },
+    getAuditConfig: {skip: "does not accept read or write concern"},
+    getChangeStreamState: {skip: "does not accept read or write concern"},
+    getClusterParameter: {skip: "does not accept read or write concern"},
     getCmdLineOpts: {skip: "does not accept read or write concern"},
     getDatabaseVersion: {skip: "does not accept read or write concern"},
     getDefaultRWConcern: {skip: "does not accept read or write concern"},
     getDiagnosticData: {skip: "does not accept read or write concern"},
+    getFreeMonitoringStatus: {skip: "does not accept read or write concern"},
     getLastError: {skip: "does not accept read or write concern"},
     getLog: {skip: "does not accept read or write concern"},
     getMore: {skip: "does not accept read or write concern"},
@@ -457,6 +539,8 @@ let testCases = {
     hello: {skip: "does not accept read or write concern"},
     hostInfo: {skip: "does not accept read or write concern"},
     httpClientRequest: {skip: "does not accept read or write concern"},
+    exportCollection: {skip: "internal command"},
+    importCollection: {skip: "internal command"},
     insert: {
         setUp: function(conn) {
             assert.commandWorked(conn.getDB(db).runCommand({create: coll, writeConcern: {w: 1}}));
@@ -477,6 +561,7 @@ let testCases = {
     listCollections: {skip: "does not accept read or write concern"},
     listCommands: {skip: "does not accept read or write concern"},
     listDatabases: {skip: "does not accept read or write concern"},
+    listDatabasesForAllTenants: {skip: "does not accept read or write concern"},
     listIndexes: {skip: "does not accept read or write concern"},
     listShards: {skip: "does not accept read or write concern"},
     lockInfo: {skip: "does not accept read or write concern"},
@@ -486,15 +571,19 @@ let testCases = {
     logout: {skip: "does not accept read or write concern"},
     makeSnapshot: {skip: "does not accept read or write concern"},
     mapReduce: {skip: "does not accept read or write concern"},
-    "mapreduce.shardedfinish": {skip: "internal command"},
     mergeChunks: {skip: "does not accept read or write concern"},
     moveChunk: {
         skip:
             "does not accept read or write concern (accepts writeConcern, but only explicitly and when _secondaryThrottle is true)"
     },
     movePrimary: {skip: "does not accept read or write concern"},
+    moveRange: {
+        skip:
+            "does not accept read or write concern (accepts writeConcern, but only explicitly and when _secondaryThrottle is true)"
+    },
     multicast: {skip: "does not accept read or write concern"},
     netstat: {skip: "internal command"},
+    pinHistoryReplicated: {skip: "internal command"},
     ping: {skip: "does not accept read or write concern"},
     planCacheClear: {skip: "does not accept read or write concern"},
     planCacheClearFilters: {skip: "does not accept read or write concern"},
@@ -504,6 +593,9 @@ let testCases = {
     profile: {skip: "does not accept read or write concern"},
     reIndex: {skip: "does not accept read or write concern"},
     reapLogicalSessionCacheNow: {skip: "does not accept read or write concern"},
+    recipientForgetMigration: {skip: "does not accept read or write concern"},
+    recipientSyncData: {skip: "does not accept read or write concern"},
+    recipientVoteImportedFiles: {skip: "does not accept read or write concern"},
     refineCollectionShardKey: {skip: "does not accept read or write concern"},
     refreshLogicalSessionCacheNow: {skip: "does not accept read or write concern"},
     refreshSessions: {skip: "does not accept read or write concern"},
@@ -516,6 +608,7 @@ let testCases = {
         },
         command: {renameCollection: nss, to: nss + "2"},
         db: "admin",
+        target: "replset",
         checkReadConcern: false,
         checkWriteConcern: true,
     },
@@ -535,8 +628,9 @@ let testCases = {
     replSetStepUp: {skip: "does not accept read or write concern"},
     replSetSyncFrom: {skip: "does not accept read or write concern"},
     replSetTest: {skip: "does not accept read or write concern"},
+    replSetTestEgress: {skip: "does not accept read or write concern"},
     replSetUpdatePosition: {skip: "does not accept read or write concern"},
-    resetError: {skip: "does not accept read or write concern"},
+    reshardCollection: {skip: "does not accept read or write concern"},
     resync: {skip: "does not accept read or write concern"},
     revokePrivilegesFromRole: {
         setUp: function(conn) {
@@ -591,19 +685,24 @@ let testCases = {
         useLogs: true,
     },
     rolesInfo: {skip: "does not accept read or write concern"},
+    rotateCertificates: {skip: "does not accept read or write concern"},
     saslContinue: {skip: "does not accept read or write concern"},
     saslStart: {skip: "does not accept read or write concern"},
+    sbe: {skip: "internal command"},
     serverStatus: {skip: "does not accept read or write concern"},
     setAllowMigrations: {skip: "does not accept read or write concern"},
+    setAuditConfig: {skip: "does not accept read or write concern"},
     setCommittedSnapshot: {skip: "internal command"},
     setDefaultRWConcern: {skip: "special case (must run after all other commands)"},
     setFeatureCompatibilityVersion: {skip: "does not accept read or write concern"},
-    setProfilingFilterGlobally: {skip: "does not accept read or write concern"},
+    setFreeMonitoring: {skip: "does not accept read or write concern"},
     setIndexCommitQuorum: {skip: "does not accept read or write concern"},
     setParameter: {skip: "does not accept read or write concern"},
     setShardVersion: {skip: "internal command"},
+    setChangeStreamState: {skip: "does not accept read or write concern"},
+    setClusterParameter: {skip: "does not accept read or write concern"},
+    setUserWriteBlockMode: {skip: "does not accept read or write concern"},
     shardCollection: {skip: "does not accept read or write concern"},
-    shardConnPoolStats: {skip: "does not accept read or write concern"},
     shardingState: {skip: "does not accept read or write concern"},
     shutdown: {skip: "does not accept read or write concern"},
     sleep: {skip: "does not accept read or write concern"},
@@ -614,8 +713,14 @@ let testCases = {
     startRecordingTraffic: {skip: "does not accept read or write concern"},
     startSession: {skip: "does not accept read or write concern"},
     stopRecordingTraffic: {skip: "does not accept read or write concern"},
+    testDeprecation: {skip: "does not accept read or write concern"},
+    testDeprecationInVersion2: {skip: "does not accept read or write concern"},
+    testInternalTransactions: {skip: "internal command"},
+    testRemoval: {skip: "does not accept read or write concern"},
+    testReshardCloneCollection: {skip: "internal command"},
+    testVersions1And2: {skip: "does not accept read or write concern"},
+    testVersion2: {skip: "does not accept read or write concern"},
     top: {skip: "does not accept read or write concern"},
-    unsetSharding: {skip: "internal command"},
     update: {
         setUp: function(conn) {
             assert.commandWorked(conn.getCollection(nss).insert({x: 1}, {writeConcern: {w: 1}}));
@@ -652,6 +757,8 @@ let testCases = {
     updateZoneKeyRange: {skip: "does not accept read or write concern"},
     usersInfo: {skip: "does not accept read or write concern"},
     validate: {skip: "does not accept read or write concern"},
+    validateDBMetadata: {skip: "does not accept read or write concern"},
+    voteCommitImportCollection: {skip: "internal command"},
     voteCommitIndexBuild: {skip: "internal command"},
     waitForFailPoint: {skip: "does not accept read or write concern"},
     waitForOngoingChunkSplits: {skip: "does not accept read or write concern"},
@@ -659,6 +766,10 @@ let testCases = {
     whatsmyuri: {skip: "internal command"},
 };
 
+commandsRemovedFromMongodSinceLastLTS.forEach(function(cmd) {
+    testCases[cmd] = {skip: "must define test coverage for backwards compatibility"};
+});
+
 // Running setDefaultRWConcern in the middle of a scenario would define defaults when there
 // shouldn't be for subsequently-tested commands. Thus it is special-cased to be run at the end of
 // the scenario.
@@ -752,7 +863,11 @@ function createLogLineRegularExpressionForTestCase(test, cmdName, targetId, expl
 // }
 function createProfileFilterForTestCase(test, targetId, explicitRWC) {
     let expectedProvenance = explicitRWC ? "clientSupplied" : "customDefault";
-    let commandProfile = {"command.comment": targetId};
+    let commandProfile = {
+        "command.comment": targetId,
+        /* Filter out failed operations */
+        errCode: {$exists: false}
+    };
     if (test.checkReadConcern) {
         commandProfile = Object.extend(
             {"readConcern.level": "majority", "readConcern.provenance": expectedProvenance},
@@ -772,11 +887,6 @@ function createProfileFilterForTestCase(test, targetId, explicitRWC) {
 function runScenario(
     desc, conn, regularCheckConn, configSvrCheckConn, {explicitRWC, explicitProvenance = false}) {
     let runCommandTest = function(cmdName, test) {
-        // These commands were removed but break this test in multiversion
-        if (cmdName === "getFreeMonitoringStatus" || cmdName === "setFreeMonitoring") {
-            return;
-        }
-
         assert(test !== undefined,
                "coverage failure: must define a RWC defaults application test for " + cmdName);
 
@@ -818,7 +928,7 @@ function runScenario(
 
         // Do any test-specific setup.
         if (typeof (test.setUp) === "function") {
-            conn._runWithForcedReadMode("commands", test.setUp);
+            test.setUp(conn);
         }
 
         // Get the command from the test case.
@@ -938,14 +1048,13 @@ function runTests(conn, regularCheckConn, configSvrCheckConn) {
                 configSvrCheckConn,
                 {explicitRWC: true, explicitProvenance: true});
 
-    assert.commandWorked(conn.adminCommand(
-        {setDefaultRWConcern: 1, defaultReadConcern: {}, defaultWriteConcern: {}}));
-    runScenario("Scenario: RWC defaults unset, explicit RWC present, absent provenance",
+    assert.commandWorked(conn.adminCommand({setDefaultRWConcern: 1, defaultReadConcern: {}}));
+    runScenario("Scenario: Read concern defaults unset, explicit RWC present, absent provenance",
                 conn,
                 regularCheckConn,
                 configSvrCheckConn,
                 {explicitRWC: true, explicitProvenance: false});
-    runScenario("Scenario: RWC defaults unset, explicit RWC present, explicit provenance",
+    runScenario("Scenario: Read concern defaults unset, explicit RWC present, explicit provenance",
                 conn,
                 regularCheckConn,
                 configSvrCheckConn,
