@@ -1,10 +1,6 @@
 //
 // Basic tests for refineCollectionShardKey.
 //
-// Tag this test as 'requires_find_command' to prevent it from running in the legacy passthroughs
-// and as 'requires_document_locking' because it uses retryable writes.
-// @tags: [requires_find_command, requires_document_locking, requires_fcv_44]
-//
 
 (function() {
 'use strict';
@@ -12,6 +8,7 @@ load('jstests/libs/fail_point_util.js');
 load('jstests/libs/profiler.js');
 load('jstests/sharding/libs/shard_versioning_util.js');
 load('jstests/sharding/libs/sharded_transactions_helpers.js');
+load("jstests/sharding/libs/find_chunks_util.js");
 
 const st = new ShardingTest({
     mongos: 2,
@@ -184,12 +181,12 @@ function setupConfigChunksBeforeRefine() {
     assert.commandWorked(mongos.adminCommand({split: kNsName, middle: {a: 0, b: 0}}));
     assert.commandWorked(mongos.adminCommand({split: kNsName, middle: {a: 5, b: 5}}));
 
-    return mongos.getCollection(kConfigChunks).findOne({ns: kNsName}).lastmodEpoch;
+    return findChunksUtil.findOneChunkByNs(mongos.getDB('config'), kNsName).lastmodEpoch;
 }
 
 function validateConfigChunksAfterRefine(oldEpoch) {
     const chunkArr =
-        mongos.getCollection(kConfigChunks).find({ns: kNsName}).sort({min: 1}).toArray();
+        findChunksUtil.findChunksByNs(mongos.getDB('config'), kNsName).sort({min: 1}).toArray();
     assert.eq(3, chunkArr.length);
     assert.eq({a: MinKey, b: MinKey, c: MinKey, d: MinKey}, chunkArr[0].min);
     assert.eq({a: 0, b: 0, c: MinKey, d: MinKey}, chunkArr[0].max);
@@ -199,7 +196,7 @@ function validateConfigChunksAfterRefine(oldEpoch) {
     assert.eq({a: MaxKey, b: MaxKey, c: MaxKey, d: MaxKey}, chunkArr[2].max);
     assert.eq(chunkArr[0].lastmodEpoch, chunkArr[1].lastmodEpoch);
     assert.eq(chunkArr[1].lastmodEpoch, chunkArr[2].lastmodEpoch);
-    assert.neq(oldEpoch, chunkArr[0].lastmodEpoch);
+    assert(!oldEpoch && !chunkArr[0].lastmodEpoch || oldEpoch != chunkArr[0].lastmodEpoch);
 }
 
 function setupConfigTagsBeforeRefine() {
@@ -240,7 +237,8 @@ function validateUnrelatedCollAfterRefine(oldCollArr, oldChunkArr, oldTagsArr) {
     assert.eq(1, collArr.length);
     assert.sameMembers(oldCollArr, collArr);
 
-    const chunkArr = mongos.getCollection(kConfigChunks).find({ns: kUnrelatedName}).toArray();
+    const chunkArr =
+        findChunksUtil.findChunksByNs(mongos.getDB('config'), kUnrelatedName).toArray();
     assert.eq(3, chunkArr.length);
     assert.sameMembers(oldChunkArr, chunkArr);
 
@@ -251,6 +249,8 @@ function validateUnrelatedCollAfterRefine(oldCollArr, oldChunkArr, oldTagsArr) {
 
 jsTestLog('********** SIMPLE TESTS **********');
 
+var result;
+
 // Should fail because arguments 'refineCollectionShardKey' and 'key' are invalid types.
 assert.commandFailedWithCode(
     mongos.adminCommand({refineCollectionShardKey: {_id: 1}, key: {_id: 1, aKey: 1}}),
@@ -276,6 +276,11 @@ assert.commandFailedWithCode(
     mongos.adminCommand({refineCollectionShardKey: kNsName, key: {_id: 1, aKey: 1}}),
     ErrorCodes.NamespaceNotSharded);
 
+// Should fail because operation can't run on config server
+assert.commandFailedWithCode(
+    mongos.adminCommand({refineCollectionShardKey: "config.collections", key: {_id: 1, aKey: 1}}),
+    ErrorCodes.NoShardingEnabled);
+
 enableShardingAndShardColl({_id: 1});
 
 // Should fail because shard key is invalid (i.e. bad values).
@@ -331,26 +336,6 @@ dropAndRecreateColl({aKey: 1});
 hangAfterRefreshFailPoint.off();
 awaitShellToTriggerNamespaceNotSharded();
 
-assert.commandWorked(mongos.adminCommand({shardCollection: kNsName, key: {_id: 1}}));
-
-// Configure failpoint 'hangRefineCollectionShardKeyAfterRefresh' on staleMongos and run
-// refineCollectionShardKey against this mongos in a parallel thread.
-hangAfterRefreshFailPoint =
-    configureFailPoint(staleMongos, 'hangRefineCollectionShardKeyAfterRefresh');
-const awaitShellToTriggerStaleEpoch = startParallelShell(() => {
-    assert.commandFailedWithCode(
-        db.adminCommand({refineCollectionShardKey: 'db.foo', key: {_id: 1, aKey: 1}}),
-        ErrorCodes.StaleEpoch);
-}, staleMongos.port);
-hangAfterRefreshFailPoint.wait();
-
-// Drop and re-shard namespace 'db.foo' without staleMongos refreshing its metadata.
-dropAndReshardColl({_id: 1});
-
-// Should fail because staleMongos has a stale epoch.
-hangAfterRefreshFailPoint.off();
-awaitShellToTriggerStaleEpoch();
-
 assert.commandWorked(mongos.getDB(kDbName).dropDatabase());
 
 jsTestLog('********** SHARD KEY VALIDATION TESTS **********');
@@ -372,36 +357,46 @@ assert.commandFailedWithCode(
 dropAndReshardColl({_id: 1});
 assert.commandWorked(mongos.getCollection(kNsName).createIndex({_id: 1, aKey: 1}, {sparse: true}));
 
-assert.commandFailedWithCode(
-    mongos.adminCommand({refineCollectionShardKey: kNsName, key: {_id: 1, aKey: 1}}),
-    ErrorCodes.InvalidOptions);
+result = mongos.adminCommand({refineCollectionShardKey: kNsName, key: {_id: 1, aKey: 1}});
+assert.commandFailedWithCode(result, ErrorCodes.InvalidOptions);
+assert(result.errmsg.includes("Index key is sparse."));
+
+// Should fail because index has a non-simple collation.
+dropAndReshardColl({aKey: 1});
+assert.commandWorked(mongos.getCollection(kNsName).createIndex({aKey: 1, bKey: 1}, {
+    collation: {
+        locale: "en",
+    }
+}));
+result = mongos.adminCommand({refineCollectionShardKey: kNsName, key: {aKey: 1, bKey: 1}});
+assert.commandFailedWithCode(result, ErrorCodes.InvalidOptions);
+assert(result.errmsg.includes("Index has a non-simple collation."));
 
 // Should fail because only a partial index exists for new shard key {_id: 1, aKey: 1}.
 dropAndReshardColl({_id: 1});
 assert.commandWorked(mongos.getCollection(kNsName).createIndex(
     {_id: 1, aKey: 1}, {partialFilterExpression: {aKey: {$gt: 0}}}));
 
-assert.commandFailedWithCode(
-    mongos.adminCommand({refineCollectionShardKey: kNsName, key: {_id: 1, aKey: 1}}),
-    ErrorCodes.OperationFailed);
+result = mongos.adminCommand({refineCollectionShardKey: kNsName, key: {_id: 1, aKey: 1}});
+assert.commandFailedWithCode(result, ErrorCodes.InvalidOptions);
+assert(result.errmsg.includes("Index key is partial."));
 
 // Should fail because only a multikey index exists for new shard key {_id: 1, aKey: 1}.
 dropAndReshardColl({_id: 1});
 assert.commandWorked(mongos.getCollection(kNsName).createIndex({_id: 1, aKey: 1}));
 assert.commandWorked(mongos.getCollection(kNsName).insert({aKey: [1, 2, 3, 4, 5]}));
 
-assert.commandFailedWithCode(
-    mongos.adminCommand({refineCollectionShardKey: kNsName, key: {_id: 1, aKey: 1}}),
-    ErrorCodes.OperationFailed);
+result = mongos.adminCommand({refineCollectionShardKey: kNsName, key: {_id: 1, aKey: 1}});
+assert.commandFailedWithCode(result, ErrorCodes.InvalidOptions);
+assert(result.errmsg.includes("Index key is multikey."));
 
 // Should fail because current shard key {a: 1} is unique, new shard key is {a: 1, b: 1}, and an
 // index only exists on {a: 1, b: 1, c: 1}.
 dropAndReshardCollUnique({a: 1});
 assert.commandWorked(mongos.getCollection(kNsName).createIndex({a: 1, b: 1, c: 1}));
 
-assert.commandFailedWithCode(
-    mongos.adminCommand({refineCollectionShardKey: kNsName, key: {a: 1, b: 1}}),
-    ErrorCodes.InvalidOptions);
+mongos.adminCommand({refineCollectionShardKey: kNsName, key: {a: 1, b: 1}});
+assert.commandFailedWithCode(result, ErrorCodes.InvalidOptions);
 
 // Should work because current shard key {_id: 1} is not unique, new shard key is {_id: 1, aKey:
 // 1}, and an index exists on {_id: 1, aKey: 1, bKey: 1}.
@@ -477,27 +472,28 @@ assert.commandFailedWithCode(
     ErrorCodes.InvalidOptions);
 
 // Should fail because index key is sparse and index has non-simple collation.
-dropAndReshardColl({_id: 1, aKey: 1});
-assert.commandWorked(mongos.getCollection(kNsName).createIndex({_id: 1, aKey: 1, bKey: 1}, {
+dropAndReshardColl({_id: 1});
+assert.commandWorked(mongos.getCollection(kNsName).createIndex({_id: 1, aKey: 1}, {
     sparse: true,
     collation: {
         locale: "en",
     }
 }));
-let result =
-    mongos.adminCommand({refineCollectionShardKey: kNsName, key: {_id: 1, aKey: 1, bKey: 1}});
+result = mongos.adminCommand({refineCollectionShardKey: kNsName, key: {_id: 1, aKey: 1}});
 assert.commandFailedWithCode(result, ErrorCodes.InvalidOptions);
+assert(result.errmsg.includes("Index key is sparse.") &&
+       result.errmsg.includes("Index has a non-simple collation."));
 
 // Should fail because index key is multikey and is partial.
 dropAndReshardColl({_id: 1});
 assert.commandWorked(mongos.getCollection(kNsName).createIndex(
     {_id: 1, aKey: 1}, {name: "index_1_part", partialFilterExpression: {aKey: {$gt: 0}}}));
 assert.commandWorked(
-    mongos.getCollection(kNsName).createIndex({_id: 1, aKey: 1, bKey: 1}, {name: "index_2"}));
-assert.commandWorked(mongos.getCollection(kNsName).insert({bKey: [1, 2, 3, 4, 5]}));
+    mongos.getCollection(kNsName).createIndex({_id: 1, aKey: 1}, {name: "index_2"}));
+assert.commandWorked(mongos.getCollection(kNsName).insert({aKey: [1, 2, 3, 4, 5]}));
 
 result = mongos.adminCommand({refineCollectionShardKey: kNsName, key: {_id: 1, aKey: 1}});
-assert.commandFailedWithCode(result, [ErrorCodes.InvalidOptions, ErrorCodes.OperationFailed]);
+assert.commandFailedWithCode(result, ErrorCodes.InvalidOptions);
 assert(result.errmsg.includes("Index key is multikey.") &&
        result.errmsg.includes("Index key is partial."));
 
@@ -506,9 +502,9 @@ dropAndReshardColl({_id: 1});
 assert.commandWorked(mongos.getCollection(kNsName).createIndex(
     {_id: 1, aKey: 1}, {name: "index_1_part", partialFilterExpression: {aKey: {$gt: 0}}}));
 assert.commandWorked(mongos.getCollection(kNsName).createIndex(
-    {_id: 1, aKey: 1, bKey: 1}, {name: "index_2_sparse", sparse: true}));
+    {_id: 1, aKey: 1}, {name: "index_2_sparse", sparse: true}));
 result = mongos.adminCommand({refineCollectionShardKey: kNsName, key: {_id: 1, aKey: 1}});
-assert.commandFailedWithCode(result, [ErrorCodes.InvalidOptions, ErrorCodes.OperationFailed]);
+assert.commandFailedWithCode(result, ErrorCodes.InvalidOptions);
 assert(result.errmsg.includes("Index key is partial.") &&
        result.errmsg.includes("Index key is sparse."));
 
@@ -580,7 +576,6 @@ validateConfigCollectionsUnique(true);
 // Verify that enforceUniquenessCheck: false allows non-unique indexes.
 assert.commandWorked(mongos.getDB(kDbName).runCommand({drop: kCollName}));
 assert.commandWorked(mongos.getCollection(kNsName).createIndex({a: 1, b: 1}));
-assert.commandWorked(mongos.adminCommand({enableSharding: kDbName}));
 assert.commandWorked(mongos.adminCommand(
     {shardCollection: kNsName, key: {a: 1}, unique: true, enforceUniquenessCheck: false}));
 assert.commandWorked(mongos.adminCommand(
@@ -680,7 +675,7 @@ assert.commandWorked(mongos.adminCommand({
 }));
 
 const oldCollArr = mongos.getCollection(kConfigCollections).find({_id: kUnrelatedName}).toArray();
-const oldChunkArr = mongos.getCollection(kConfigChunks).find({ns: kUnrelatedName}).toArray();
+const oldChunkArr = findChunksUtil.findChunksByNs(mongos.getDB('config'), kUnrelatedName).toArray();
 const oldTagsArr = mongos.getCollection(kConfigTags).find({ns: kUnrelatedName}).toArray();
 assert.eq(1, oldCollArr.length);
 assert.eq(3, oldChunkArr.length);
@@ -779,38 +774,6 @@ if (!isStepdownSuite) {
     assert(minKeyShardDB.system.profile.drop());
 })();
 
-(() => {
-    //
-    // Verify refineCollectionShardKey can return a StaleConfig error without crashing the config
-    // server.
-    //
-
-    // Create a sharded collection with one chunk on shard0.
-    const dbName = "testReturnStaleConfig";
-    const ns = dbName + ".fooReturnStaleConfig";
-    assert.commandWorked(st.s.adminCommand({enableSharding: dbName}));
-    st.ensurePrimaryShard(dbName, st.shard0.shardName);
-    assert.commandWorked(st.s.adminCommand({shardCollection: ns, key: {x: 1}}));
-
-    // Move the last chunk away from shard0 without refreshing shard1 so it will be stale when its
-    // indexes are read during the next refine. Disable refreshes on shard1 so it will repeatedly
-    // return StaleConfig until refineCollectionShardKey runs out of retries.
-    ShardVersioningUtil.moveChunkNotRefreshRecipient(st.s, ns, st.shard0, st.shard1, {x: 1});
-
-    let disableRefreshesFailPoint =
-        configureFailPoint(st.rs1.getPrimary(), "skipShardFilteringMetadataRefresh");
-
-    assert.commandWorked(st.rs1.getPrimary().getCollection(ns).createIndex({x: 1, y: 1}));
-    assert.commandFailedWithCode(
-        st.s.adminCommand({refineCollectionShardKey: ns, key: {x: 1, y: 1}}),
-        ErrorCodes.StaleConfig);
-
-    disableRefreshesFailPoint.off();
-
-    // The refresh should succeed now.
-    assert.commandWorked(st.s.adminCommand({refineCollectionShardKey: ns, key: {x: 1, y: 1}}));
-})();
-
 // Assumes the given arrays are sorted by the max field.
 function compareMinAndMaxFields(shardedArr, refinedArr) {
     assert(shardedArr.length && refinedArr.length, tojson(shardedArr) + ", " + tojson(refinedArr));
@@ -829,9 +792,9 @@ function compareMinAndMaxFields(shardedArr, refinedArr) {
 function compareBoundaries(conn, shardedNs, refinedNs) {
     // Compare chunks.
     const shardedChunks =
-        conn.getDB("config").chunks.find({ns: shardedNs}).sort({max: 1}).toArray();
+        findChunksUtil.findChunksByNs(conn.getDB("config"), shardedNs).sort({max: 1}).toArray();
     const refinedChunks =
-        conn.getDB("config").chunks.find({ns: refinedNs}).sort({max: 1}).toArray();
+        findChunksUtil.findChunksByNs(conn.getDB("config"), refinedNs).sort({max: 1}).toArray();
     compareMinAndMaxFields(shardedChunks, refinedChunks);
 
     // Compare tags.
