@@ -16,20 +16,17 @@
  * - behavior: Must be "unshardedOnly", or "versioned". Determines what system profiler checks are
  * performed.
  *
- * Tagged as 'requires_fcv_44', since this test cannot run against versions less then 4.4. This is
- * because 'planCacheListPlans' and 'planCacheListQueryShapes' were deleted in 4.4, and thus not
- * tested here. But this test asserts that all commands are covered, so will fail against a version
- * of the server which implements these commands.
- *
  * @tags: [
- *   requires_fcv_44,
- * ]
+ *   # SERVER-56565 avoid CS stepdowns, since  an election may trigger a  refresh of stale metadata
+ *   #              that form part of the test setup.
+ *   does_not_support_stepdowns,
+ *  ]
  */
 (function() {
 "use strict";
 
 load('jstests/libs/profiler.js');
-load('jstests/sharding/libs/last_stable_mongos_commands.js');
+load('jstests/sharding/libs/last_lts_mongos_commands.js');
 
 let db = "test";
 let coll = "foo";
@@ -53,31 +50,50 @@ let testCases = {
     _configsvrBalancerStatus: {skip: "primary only"},
     _configsvrBalancerStop: {skip: "primary only"},
     _configsvrClearJumboFlag: {skip: "primary only"},
-    _configsvrCommitChunkMerge: {skip: "primary only"},
     _configsvrCommitChunksMerge: {skip: "primary only"},
     _configsvrCommitChunkMigration: {skip: "primary only"},
     _configsvrCommitChunkSplit: {skip: "primary only"},
-    _configsvrCommitMovePrimary: {skip: "primary only"},
-    _configsvrDropCollection: {skip: "primary only"},
-    _configsvrDropDatabase: {skip: "primary only"},
+    _configsvrCommitIndex: {skip: "primary only"},
+    _configsvrCommitMovePrimary:
+        {skip: "primary only"},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
+    _configsvrConfigureAutoSplit: {
+        skip: "primary only"
+    },  // TODO SERVER-62374: remove this once 5.3 becomes last continuos release
+    _configsvrConfigureCollectionBalancing: {skip: "primary only"},
+    _configsvrDropCollection:
+        {skip: "primary only"},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
+    _configsvrDropDatabase:
+        {skip: "primary only"},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
+    _configsvrDropIndexCatalogEntry: {skip: "primary only"},
     _configsvrMoveChunk: {skip: "primary only"},
-    _configsvrMovePrimary: {skip: "primary only"},
+    _configsvrMovePrimary:
+        {skip: "primary only"},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
+    _configsvrMoveRange: {skip: "primary only"},
     _configsvrRemoveShardFromZone: {skip: "primary only"},
-    _configsvrShardCollection: {skip: "primary only"},
+    _configsvrReshardCollection: {skip: "primary only"},
+    _configsvrShardCollection:
+        {skip: "primary only"},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
     _configsvrUpdateZoneKeyRange: {skip: "primary only"},
+    _flushReshardingStateChange: {skip: "does not return user data"},
     _flushRoutingTableCacheUpdates: {skip: "does not return user data"},
+    _flushRoutingTableCacheUpdatesWithWriteConcern: {skip: "does not return user data"},
+    _getAuditConfigGeneration: {skip: "does not return user data"},
     _getUserCacheGeneration: {skip: "does not return user data"},
     _hashBSONElement: {skip: "does not return user data"},
     _isSelf: {skip: "does not return user data"},
     _killOperations: {skip: "does not return user data"},
     _mergeAuthzCollections: {skip: "primary only"},
     _migrateClone: {skip: "primary only"},
+    _shardsvrCompactStructuredEncryptionData: {skip: "primary only"},
     _shardsvrMovePrimary: {skip: "primary only"},
+    _shardsvrMoveRange: {skip: "primary only"},
     _recvChunkAbort: {skip: "primary only"},
     _recvChunkCommit: {skip: "primary only"},
+    _recvChunkReleaseCritSec: {skip: "primary only"},
     _recvChunkStart: {skip: "primary only"},
     _recvChunkStatus: {skip: "primary only"},
     _transferMods: {skip: "primary only"},
+    abortReshardCollection: {skip: "primary only"},
     abortTransaction: {skip: "primary only"},
     addShard: {skip: "primary only"},
     addShardToZone: {skip: "primary only"},
@@ -92,11 +108,11 @@ let testCases = {
         },
         behavior: "versioned"
     },
+    analyze: {skip: "primary only"},
     appendOplogNote: {skip: "primary only"},
     applyOps: {skip: "primary only"},
     authSchemaUpgrade: {skip: "primary only"},
     authenticate: {skip: "does not return user data"},
-    availableQueryOptions: {skip: "does not return user data"},
     balancerCollectionStatus: {skip: "primary only"},
     balancerStart: {skip: "primary only"},
     balancerStatus: {skip: "primary only"},
@@ -105,19 +121,35 @@ let testCases = {
     captrunc: {skip: "primary only"},
     checkShardingIndex: {skip: "primary only"},
     cleanupOrphaned: {skip: "primary only"},
+    cleanupReshardCollection: {skip: "primary only"},
     clearJumboFlag: {skip: "primary only"},
     clearLog: {skip: "does not return user data"},
     clone: {skip: "primary only"},
     cloneCollectionAsCapped: {skip: "primary only"},
+    clusterAbortTransaction: {skip: "already tested by 'abortTransaction' tests on mongos"},
+    clusterAggregate: {skip: "already tested by 'aggregate' tests on mongos"},
+    clusterCommitTransaction: {skip: "already tested by 'commitTransaction' tests on mongos"},
+    clusterDelete: {skip: "already tested by 'delete' tests on mongos"},
+    clusterFind: {skip: "already tested by 'find' tests on mongos"},
+    clusterGetMore: {skip: "already tested by 'getMore' tests on mongos"},
+    clusterInsert: {skip: "already tested by 'insert' tests on mongos"},
+    clusterUpdate: {skip: "already tested by 'update' tests on mongos"},
     collMod: {skip: "primary only"},
     collStats: {skip: "does not return user data"},
+    commitReshardCollection: {skip: "primary only"},
     commitTransaction: {skip: "primary only"},
     compact: {skip: "does not return user data"},
+    compactStructuredEncryptionData: {skip: "does not return user data"},
+    configureCollectionAutoSplitter: {
+        skip: "does not return user data"
+    },  // TODO SERVER-62374: remove this once 5.3 becomes last continuos release
+    configureCollectionBalancing: {skip: "does not return user data"},
     configureFailPoint: {skip: "does not return user data"},
     connPoolStats: {skip: "does not return user data"},
     connPoolSync: {skip: "does not return user data"},
     connectionStatus: {skip: "does not return user data"},
     convertToCapped: {skip: "primary only"},
+    coordinateCommitTransaction: {skip: "unimplemented. Serves only as a stub."},
     count: {
         setUp: function(mongosConn) {
             assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
@@ -183,7 +215,9 @@ let testCases = {
     forceerror: {skip: "does not return user data"},
     fsync: {skip: "does not return user data"},
     fsyncUnlock: {skip: "does not return user data"},
-    geoSearch: {skip: "not supported in mongos"},
+    getAuditConfig: {skip: "does not return user data"},
+    getChangeStreamState: {skip: "does not return user data"},
+    getClusterParameter: {skip: "does not return user data"},
     getCmdLineOpts: {skip: "does not return user data"},
     getDefaultRWConcern: {skip: "does not return user data"},
     getDiagnosticData: {skip: "does not return user data"},
@@ -245,6 +279,7 @@ let testCases = {
     mergeChunks: {skip: "primary only"},
     moveChunk: {skip: "primary only"},
     movePrimary: {skip: "primary only"},
+    moveRange: {skip: "primary only"},
     multicast: {skip: "does not return user data"},
     netstat: {skip: "does not return user data"},
     ping: {skip: "does not return user data"},
@@ -278,25 +313,30 @@ let testCases = {
     replSetTest: {skip: "does not return user data"},
     replSetUpdatePosition: {skip: "does not return user data"},
     replSetResizeOplog: {skip: "does not return user data"},
-    resetError: {skip: "does not return user data"},
+    reshardCollection: {skip: "primary only"},
     resync: {skip: "primary only"},
     revokePrivilegesFromRole: {skip: "primary only"},
     revokeRolesFromRole: {skip: "primary only"},
     revokeRolesFromUser: {skip: "primary only"},
     rolesInfo: {skip: "primary only"},
+    rotateCertificates: {skip: "does not return user data"},
     saslContinue: {skip: "primary only"},
     saslStart: {skip: "primary only"},
+    sbe: {skip: "internal command"},
     serverStatus: {skip: "does not return user data"},
     setAllowMigrations: {skip: "primary only"},
+    setAuditConfig: {skip: "does not return user data"},
     setCommittedSnapshot: {skip: "does not return user data"},
     setDefaultRWConcern: {skip: "primary only"},
     setIndexCommitQuorum: {skip: "primary only"},
     setFeatureCompatibilityVersion: {skip: "primary only"},
-    setProfilingFilterGlobally: {skip: "does not return user data"},
+    setFreeMonitoring: {skip: "primary only"},
     setParameter: {skip: "does not return user data"},
     setShardVersion: {skip: "does not return user data"},
+    setChangeStreamState: {skip: "does not return user data"},
+    setClusterParameter: {skip: "does not return user data"},
+    setUserWriteBlockMode: {skip: "primary only"},
     shardCollection: {skip: "primary only"},
-    shardConnPoolStats: {skip: "does not return user data"},
     shardingState: {skip: "does not return user data"},
     shutdown: {skip: "does not return user data"},
     sleep: {skip: "does not return user data"},
@@ -307,21 +347,27 @@ let testCases = {
     startRecordingTraffic: {skip: "does not return user data"},
     startSession: {skip: "does not return user data"},
     stopRecordingTraffic: {skip: "does not return user data"},
+    testDeprecation: {skip: "does not return user data"},
+    testDeprecationInVersion2: {skip: "does not return user data"},
+    testInternalTransactions: {skip: "primary only"},
+    testRemoval: {skip: "does not return user data"},
+    testVersions1And2: {skip: "does not return user data"},
+    testVersion2: {skip: "does not return user data"},
     top: {skip: "does not return user data"},
-    unsetSharding: {skip: "does not return user data"},
     update: {skip: "primary only"},
     updateRole: {skip: "primary only"},
     updateUser: {skip: "primary only"},
     updateZoneKeyRange: {skip: "primary only"},
     usersInfo: {skip: "primary only"},
     validate: {skip: "does not return user data"},
+    validateDBMetadata: {skip: "does not return user data"},
     waitForFailPoint: {skip: "does not return user data"},
     waitForOngoingChunkSplits: {skip: "does not return user data"},
     whatsmyuri: {skip: "does not return user data"}
 };
 
-commandsRemovedFromMongosIn44.forEach(function(cmd) {
-    testCases[cmd] = {skip: "must define test coverage for 4.2 backwards compatibility"};
+commandsRemovedFromMongosSinceLastLTS.forEach(function(cmd) {
+    testCases[cmd] = {skip: "must define test coverage for backwards compatibility"};
 });
 
 let scenarios = {
@@ -515,6 +561,9 @@ let staleMongos = st.s1;
 
 let res = st.s.adminCommand({listCommands: 1});
 assert.commandWorked(res);
+// The default WC is majority and this test can't satisfy majority writes.
+assert.commandWorked(staleMongos.adminCommand(
+    {setDefaultRWConcern: 1, defaultWriteConcern: {w: 1}, writeConcern: {w: "majority"}}));
 
 let commands = Object.keys(res.commands);
 for (let command of commands) {
