@@ -1,13 +1,22 @@
 /**
  * Tests to validate the functionality of update command in the presence of a compound shard key.
- * @tags: [uses_transactions, uses_multi_shard_transaction, requires_fcv_44]
+ *
+ * @tags: [
+ *   uses_multi_shard_transaction,
+ *   uses_transactions,
+ * ]
  */
 (function() {
 'use strict';
 
+load("jstests/sharding/libs/sharded_transactions_helpers.js");
 load("jstests/sharding/libs/update_shard_key_helpers.js");
 
 const st = new ShardingTest({mongos: 1, shards: 3});
+
+const updateDocumentShardKeyUsingTransactionApiEnabled =
+    isUpdateDocumentShardKeyUsingTransactionApiEnabled(st.s);
+
 const kDbName = 'update_compound_sk';
 const ns = kDbName + '.coll';
 const session = st.s.startSession({retryWrites: true});
@@ -77,6 +86,22 @@ function assertUpdateWorkedWithNoMatchingDoc(query, update, isUpsert, inTransact
               st.s.getDB(kDbName).coll.find(update["$set"] ? update["$set"] : update).itcount());
 }
 
+/**
+ * Updates to a document's shard key that would change the document's owning shard will succeed if
+ * the updateDocumentShardKeyUsingTransactionApi feature flag is enabled.
+ */
+function assertWouldChangeOwningShardUpdateResult(res, expectedUpdatedDoc) {
+    const numDocsUpdated = st.s.getDB(kDbName).coll.find(expectedUpdatedDoc).itcount();
+
+    if (updateDocumentShardKeyUsingTransactionApiEnabled) {
+        assert.commandWorked(res);
+        assert.eq(1, numDocsUpdated);
+    } else {
+        assert.commandFailedWithCode(res, ErrorCodes.IllegalOperation);
+        assert.eq(0, numDocsUpdated);
+    }
+}
+
 //
 // Update Type Replacement-style.
 //
@@ -145,23 +170,29 @@ assert.commandFailedWithCode(
 //
 
 // Case when upsert needs to insert a new document and the new document should belong in a shard
-// other than the one targeted by the update. These upserts can only succeed in a
-// multi-statement transaction or with retryWrites: true.
+// other than the one targeted by the update. These upserts can only succeed when the
+// updateDocumentShardKeyUsingTransactionApi feature flag is enabled or if not enabled, when the
+// upsert is run in a multi-statement transaction or with retryWrites: true.
 const updateDoc = {
     x: 1110,
     y: 55,
     z: 3,
     replStyleUpdate: true
 };
-assert.commandFailedWithCode(
-    st.s.getDB(kDbName).coll.update({x: 4, y: 0, z: 0}, updateDoc, {upsert: true}),
-    ErrorCodes.IllegalOperation);
+const updateRes = st.s.getDB(kDbName).coll.update({x: 4, y: 0, z: 0}, updateDoc, {upsert: true});
+assertWouldChangeOwningShardUpdateResult(updateRes, updateDoc);
 
-// The above upsert works with transactions.
+const updateDocTxn = {
+    x: 1110,
+    y: 55,
+    z: 4,
+    replStyleUpdate: true
+};
 session.startTransaction();
-assertUpdateWorkedWithNoMatchingDoc({x: 4, y: 0, z: 0}, updateDoc, true, true);
+assertUpdateWorkedWithNoMatchingDoc(
+    {x: 4, y: 0, z: 0}, updateDocTxn, true /*isUpsert*/, true /*inTransaction*/);
 assert.commandWorked(session.commitTransaction_forTesting());
-assert.eq(1, st.s.getDB(kDbName).coll.find(updateDoc).itcount());
+assert.eq(1, sessionDB.coll.find(updateDocTxn).itcount());
 
 // Full shard key not specified in query.
 
@@ -249,20 +280,26 @@ assert.commandFailedWithCode(st.s.getDB(kDbName).coll.update(
 // Test upsert-specific behaviours.
 
 // Case when upsert needs to insert a new document and the new document should belong in a shard
-// other than the one targeted by the update. These upserts can only succeed in a
-// multi-statement transaction or with retryWrites: true.
-const update = {
-    "$set": {x: 2110, y: 55, z: 3, opStyle: true}
+// other than the one targeted by the update. These upserts can only succeed when the
+// updateDocumentShardKeyUsingTransactionApi feature flag is enabled or if not enabled, when the
+// upsert is run in a multi-statement transaction or with retryWrites: true.
+const upsertDoc = {
+    x: 2110,
+    y: 55,
+    z: 3,
+    opStyle: true
 };
-assert.commandFailedWithCode(
-    st.s.getDB(kDbName).coll.update({x: 4, y: 0, z: 0, opStyle: true}, update, {upsert: true}),
-    ErrorCodes.IllegalOperation);
+const upsertRes =
+    st.s.getDB(kDbName).coll.update({x: 4, y: 0, z: 0, opStyle: true}, upsertDoc, {upsert: true});
+assertWouldChangeOwningShardUpdateResult(upsertRes, upsertDoc);
 
-// The above upsert works with transactions.
+const upsertDocTxn = {
+    "$set": {x: 2110, y: 55, z: 4, opStyle: true}
+};
 session.startTransaction();
-assertUpdateWorkedWithNoMatchingDoc({x: 4, y: 0, z: 0, opStyle: true}, update, true, true);
+assertUpdateWorkedWithNoMatchingDoc({x: 4, y: 0, z: 0, opStyle: true}, upsertDocTxn, true, true);
 assert.commandWorked(session.commitTransaction_forTesting());
-assert.eq(1, st.s.getDB(kDbName).coll.find(update["$set"]).itcount());
+assert.eq(1, sessionDB.coll.find(upsertDocTxn["$set"]).itcount());
 
 // Full shard key not specified in query.
 
@@ -364,35 +401,44 @@ assert.commandFailedWithCode(
 // Test upsert-specific behaviours.
 
 // Case when upsert needs to insert a new document and the new document should belong in a shard
-// other than the one targeted by the update. These upserts can only succeed in a
-// multi-statement transaction or with retryWrites: true.
-assert.commandFailedWithCode(
-    st.s.getDB(kDbName).coll.update({x: 4, y: 0, z: 0},
-                                    [{
-                                        "$project": {
-                                            x: {$literal: 2111},
-                                            y: {$literal: 55},
-                                            z: {$literal: 3},
-                                            pipelineUpdate: {$literal: true}
-                                        }
-                                    }],
-                                    {upsert: true}),
-    ErrorCodes.IllegalOperation);
+// other than the one targeted by the update. These upserts can only succeed when run in a
+// multi-statement transaction or with retryWrites: true or if the
+// updateDocumentShardKeyUsingTransactionApi feature flag is enabled.
+const upsertProjectDoc = {
+    x: 2111,
+    y: 55,
+    z: 3
+};
+const upsertProjectRes = st.s.getDB(kDbName).coll.update({x: 4, y: 0, z: 0},
+                                                         [{
+                                                             "$project": {
+                                                                 x: {$literal: 2111},
+                                                                 y: {$literal: 55},
+                                                                 z: {$literal: 3},
+                                                                 pipelineUpdate: {$literal: true}
+                                                             }
+                                                         }],
+                                                         {upsert: true});
+assertWouldChangeOwningShardUpdateResult(upsertProjectRes, upsertProjectDoc);
 
-// The above upsert works with transactions.
+const upsertProjectTxnDoc = {
+    x: 2111,
+    y: 55,
+    z: 4
+};
 session.startTransaction();
 assertUpdateWorkedWithNoMatchingDoc({x: 4, y: 0, z: 0, pipelineUpdate: true},
                                     [{
                                         "$project": {
                                             x: {$literal: 2111},
                                             y: {$literal: 55},
-                                            z: {$literal: 3},
+                                            z: {$literal: 4},
                                             pipelineUpdate: {$literal: true}
                                         }
                                     }],
                                     true);
 assert.commandWorked(session.commitTransaction_forTesting());
-assert.eq(1, st.s.getDB(kDbName).coll.find({x: 2111, y: 55, z: 3, pipelineUpdate: true}).itcount());
+assert.eq(1, sessionDB.coll.find(upsertProjectTxnDoc).itcount());
 
 // Full shard key not specified in query.
 assert.commandFailedWithCode(st.s.getDB(kDbName).coll.update(
