@@ -1,8 +1,7 @@
 #!/usr/bin/env python3
 """
 This script (re)generates x509 certificates from a definition file.
-Invoke as `python3 jstests/ssl/x509/mkcert.py --config jstests/ssl/x509/certs.yml`
-and `python3 jstests/ssl/x509/mkcert.py --config src/mongo/db/modules/enterprise/jstests/libs/certs.yml`
+Invoke as `python3 jstests/ssl/x509/mkcert.py --config certs.yml`
 Optionally providing a cert ID to only regenerate a single cert.
 """
 import argparse
@@ -18,6 +17,8 @@ import OpenSSL
 import re
 import shutil
 
+import mkdigest
+
 # pylint: disable=protected-access
 OpenSSL._util.lib.OBJ_create(b'1.2.3.45', b'DummyOID45', b'Dummy OID 45')
 OpenSSL._util.lib.OBJ_create(b'1.2.3.56', b'DummyOID56', b'Dummy OID 56')
@@ -38,6 +39,9 @@ MUST_STAPLE_KEY = bytes(MUST_STAPLE_KEY_STR, "utf-8")
 MUST_STAPLE_VALUE_STR = 'DER:30:03:02:01:05' # ASN.1 value: SEQUENCE { INTEGER 0x05 (5 decimal) }
 MUST_STAPLE_VALUE = str(MUST_STAPLE_VALUE_STR).encode('utf-8')
 
+# <= 825 in order to abide by https://support.apple.com/en-us/HT210176. 
+MAX_VALIDITY_PERIOD_DAYS = 824
+
 def glbl(key, default=None):
     """Fetch a key from the global dict."""
     return CONFIG.get('global', {}).get(key, default)
@@ -139,8 +143,7 @@ def set_validity(x509, cert):
         # TODO: Parse human readable dates and/or datedeltas
         not_after = int(not_after)
     else:
-        # Default 20 years hence.
-        not_after = 20 * 365 * 24 * 60 * 60
+        not_after = not_before + MAX_VALIDITY_PERIOD_DAYS * 24 * 60 * 60
     x509.gmtime_adj_notAfter(not_after)
 
 def set_general_dict_extension(x509, exts, cert, name, typed_values):
@@ -508,7 +511,7 @@ def process_client_multivalue_rdn(cert):
     subject = '/CN=client+OU=KernelUser+O=MongoDB/L=New York City+ST=New York+C=US'
     subprocess.check_call(['openssl', 'req', '-new', '-nodes', '-multivalue-rdn', '-subj', subject, '-keyout', key, '-out', csr])
     subprocess.check_call(['openssl', 'rsa', '-in', key, '-out', rsa])
-    subprocess.check_call(['openssl', 'x509', '-in', csr, '-out', pem, '-req', '-CA', ca, '-CAkey', ca, '-days', '3650', '-sha256', '-set_serial', serial])
+    subprocess.check_call(['openssl', 'x509', '-in', csr, '-out', pem, '-req', '-CA', ca, '-CAkey', ca, '-days', str(MAX_VALIDITY_PERIOD_DAYS), '-sha256', '-set_serial', serial])
 
     open(dest, 'wt').write(get_header_comment(cert) + "\n" + open(pem, 'rt').read() + open(rsa, 'rt').read())
     os.remove(key)
@@ -568,7 +571,7 @@ def process_ecdsa_ca(cert):
     subject = '/C=US/ST=New York/L=New York City/O=MongoDB/OU=Kernel/CN=Kernel Test ESCDA CA/'
 
     reqargs = ['openssl', 'req', '-new', '-key', key, '-out', csr, '-subj', subject]
-    x509args = ['openssl', 'x509', '-in', csr, '-out', pem, '-req', '-signkey', key, '-days', '7300', '-sha256', '-set_serial', serial]
+    x509args = ['openssl', 'x509', '-in', csr, '-out', pem, '-req', '-signkey', key, '-days', str(MAX_VALIDITY_PERIOD_DAYS), '-sha256', '-set_serial', serial]
     ecparamargs = (['openssl', 'ecparam', '-name', 'prime256v1', '-genkey', '-out', key, '-noout']
                    if "ocsp" in cert.get('tags', [])
                    else ['openssl', 'ecparam', '-name', 'prime256v1', '-genkey', '-out', key])
@@ -612,7 +615,7 @@ def process_ecdsa_leaf(cert):
     subject = '/C=US/ST=New York/L=New York City/O=MongoDB/OU=' + ou + '/CN=' + mode
 
     reqargs = ['openssl', 'req', '-new', '-key', key, '-out', csr, '-subj', subject]
-    x509args = ['openssl', 'x509', '-in', csr, '-out', pem, '-req', '-CA', ca, '-CAkey', ca, '-days', '7300', '-sha256', '-set_serial', serial]
+    x509args = ['openssl', 'x509', '-in', csr, '-out', pem, '-req', '-CA', ca, '-CAkey', ca, '-days', str(MAX_VALIDITY_PERIOD_DAYS), '-sha256', '-set_serial', serial]
     if mode == 'server':
         reqargs = reqargs + ['-reqexts', 'v3_req']
         extfile = tempfile.mkstemp()[1]
@@ -790,6 +793,9 @@ def main():
     items = sort_items(items)
     for item in items:
         process_cert(item)
+        filename = make_filename(item)
+        mkdigest.make_digest(filename, 'cert', 'sha256')
+        mkdigest.make_digest(filename, 'cert', 'sha1')
 
 if __name__ == '__main__':
     main()
