@@ -1,20 +1,29 @@
-# Copyright 2019 MongoDB Inc.
+# Copyright 2020 MongoDB Inc.
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
 #
-# http://www.apache.org/licenses/LICENSE-2.0
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
+# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
+# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 
 # TODO: Handle chmod state
 
 from collections import defaultdict, namedtuple
+from typing import List
 
 import SCons
 from SCons.Tool import install
@@ -33,8 +42,10 @@ ROLE_DECLARATIONS = "AIB_ROLE_DECLARATIONS"
 SUFFIX_MAP = "AIB_SUFFIX_MAP"
 TASKS = "AIB_TASKS"
 
-
-SuffixMap = namedtuple("SuffixMap", ["directory", "default_role"],)
+SuffixMap = namedtuple(
+    "SuffixMap",
+    ["directory", "default_role"],
+)
 
 
 class RoleInfo:
@@ -89,24 +100,17 @@ def declare_roles(env, roles, base_role=None, meta_role=None):
     for role in roles:
         for d in role.dependencies:
             if d not in role_names:
-                raise Exception(
-                    "Role dependency '{}' does not name a declared role".format(d)
-                )
+                raise Exception("Role dependency '{}' does not name a declared role".format(d))
 
     if isinstance(base_role, str):
         if base_role not in role_names:
             raise Exception(
-                "A base_role argument was provided but it does not name a declared role"
-            )
+                "A base_role argument was provided but it does not name a declared role")
     elif isinstance(base_role, DeclaredRole):
         if base_role not in roles:
-            raise Exception(
-                "A base_role argument was provided but it is not a declared role"
-            )
+            raise Exception("A base_role argument was provided but it is not a declared role")
     elif base_role is not None:
-        raise Exception(
-            "The base_role argument must be a string name of a role or a role object"
-        )
+        raise Exception("The base_role argument must be a string name of a role or a role object")
     else:
         # Set it to something falsey
         base_role = str()
@@ -114,17 +118,12 @@ def declare_roles(env, roles, base_role=None, meta_role=None):
     if isinstance(meta_role, str):
         if meta_role not in role_names:
             raise Exception(
-                "A meta_role argument was provided but it does not name a declared role"
-            )
+                "A meta_role argument was provided but it does not name a declared role")
     elif isinstance(meta_role, DeclaredRole):
         if meta_role not in roles:
-            raise Exception(
-                "A meta_role argument was provided but it is not a declared role"
-            )
+            raise Exception("A meta_role argument was provided but it is not a declared role")
     elif meta_role is not None:
-        raise Exception(
-            "The meta_role argument must be a string name of a role or a role object"
-        )
+        raise Exception("The meta_role argument must be a string name of a role or a role object")
     else:
         # Set it to something falsy
         meta_role = str()
@@ -190,12 +189,7 @@ def get_alias_map_entry(env, component, role):
             r_entry.dependencies.add(base_c_entry)
 
         meta_role = env.get(META_ROLE)
-        if (
-            meta_role
-            and role != meta_role
-            and meta_component
-            and component != meta_component
-        ):
+        if (meta_role and role != meta_role and meta_component and component != meta_component):
             meta_r_entry = get_alias_map_entry(env, component, meta_role)
             meta_c_r_entry = get_alias_map_entry(env, meta_component, meta_role)
             meta_c_r_entry.dependencies.add(meta_r_entry)
@@ -218,8 +212,8 @@ def scan_for_transitive_install(node, env, _path):
     if component is None:
         return []
 
-    scanned = getattr(node.attributes, "AIB_SCANNED", [])
-    if scanned:
+    scanned = getattr(node.attributes, "AIB_SCANNED", None)
+    if scanned is not None:
         return scanned
 
     # Access directly by keys because we don't want to accidentally
@@ -250,22 +244,15 @@ def scan_for_transitive_install(node, env, _path):
         if component_base_entry.files:
             results.update(component_base_entry.files)
 
-    if (
-        base_role
-        and base_component
-        and component != base_component
-        and role != base_role
-    ):
+    if (base_role and base_component and component != base_component and role != base_role):
         base_base_entry = alias_map[base_component][base_role]
         if base_base_entry.files:
             results.update(base_base_entry.files)
 
-    installed_children = [
-        grandchild
-        for child in node.children()
-        for grandchild in child.children()
-        if grandchild.has_builder()
-    ]
+    installed_children = set(grandchild for child in node.children()
+                             for direct_children in child.children()
+                             for grandchild in direct_children.get_executor().get_all_targets()
+                             if direct_children.get_executor() and grandchild.has_builder())
 
     for child in installed_children:
         auto_installed_files = get_auto_installed_files(env, child)
@@ -314,11 +301,8 @@ def tag_components(env, target, **kwargs):
         raise Exception("AIB_COMPONENT must be a string and contain no whitespace.")
 
     if component is None:
-        raise Exception(
-            "AIB_COMPONENT must be provided; untagged targets: {}".format(
-                [t.path for t in target]
-            )
-        )
+        raise Exception("AIB_COMPONENT must be provided; untagged targets: {}".format(
+            [t.path for t in target]))
 
     if role is None:
         raise Exception("AIB_ROLE was not provided.")
@@ -334,11 +318,8 @@ def tag_components(env, target, **kwargs):
     # component or base component. These cause dependency cycles because
     # get_alias_map_entry will do that wiring for us then we will try to
     # map them back on themselves in our loop.
-    if (
-        component != env.get(BASE_COMPONENT)
-        and role != env.get(META_ROLE)
-        and component != env.get(META_COMPONENT)
-    ):
+    if (component != env.get(BASE_COMPONENT) and role != env.get(META_ROLE)
+            and component != env.get(META_COMPONENT)):
         for component in kwargs.get(REVERSE_COMPONENT_DEPENDENCIES, []):
             component_dep = get_alias_map_entry(env, component, role)
             component_dep.dependencies.add(entry)
@@ -376,9 +357,7 @@ def auto_install_pseudobuilder(env, target, source, **kwargs):
             auto_install_mapping = env[SUFFIX_MAP].get(suffix)
 
             if not auto_install_mapping:
-                raise Exception(
-                    "No target provided and no auto install mapping found for:", str(s)
-                )
+                raise Exception("No target provided and no auto install mapping found for:", str(s))
 
             target_for_source = auto_install_mapping.directory
 
@@ -439,14 +418,10 @@ def finalize_install_dependencies(env):
                 alias_name = generate_alias_name(env, component, role, task)
                 alias = env.Alias(alias_name, func(env, component, role))
                 if generate_dependent_aliases:
-                    dependent_aliases = env.Flatten(
-                        [
-                            env.Alias(
-                                generate_alias_name(env, d.component, d.role, task)
-                            )
-                            for d in info.dependencies
-                        ]
-                    )
+                    dependent_aliases = env.Flatten([
+                        env.Alias(generate_alias_name(env, d.component, d.role, task))
+                        for d in info.dependencies
+                    ])
                     env.Alias(alias, dependent_aliases)
 
 
@@ -489,11 +464,8 @@ def add_suffix_mapping(env, suffix, role=None):
     """Map suffix to role"""
     if isinstance(suffix, str):
         if role not in env[ROLE_DECLARATIONS]:
-            raise Exception(
-                "target {} is not a known role available roles are {}".format(
-                    role, env[ROLE_DECLARATIONS].keys()
-                )
-            )
+            raise Exception("target {} is not a known role available roles are {}".format(
+                role, env[ROLE_DECLARATIONS].keys()))
         env[SUFFIX_MAP][env.subst(suffix)] = role
 
     if not isinstance(suffix, dict):
@@ -502,11 +474,8 @@ def add_suffix_mapping(env, suffix, role=None):
     for _, mapping in suffix.items():
         role = mapping.default_role
         if role not in env[ROLE_DECLARATIONS]:
-            raise Exception(
-                "target {} is not a known role. Available roles are {}".format(
-                    target, env[ROLE_DECLARATIONS].keys()
-                )
-            )
+            raise Exception("target {} is not a known role. Available roles are {}".format(
+                target, env[ROLE_DECLARATIONS].keys()))
 
     env[SUFFIX_MAP].update({env.subst(key): value for key, value in suffix.items()})
 
@@ -527,16 +496,16 @@ def list_components(env, **kwargs):
         print("\t", key)
 
 
-def list_recursive(mapping, counter=0):
+def list_hierarchical_aib_recursive(mapping, counter=0):
     if counter == 0:
         print("  " * counter, mapping.id)
     counter += 1
     for dep in mapping.dependencies:
         print("  " * counter, dep.id)
-        list_recursive(dep, counter=counter)
+        list_hierarchical_aib_targets(dep, counter=counter)
 
 
-def list_targets(dag_mode=False):
+def list_hierarchical_aib_targets(dag_mode=False):
     def target_lister(env, **kwargs):
         if dag_mode:
             installed_files = set(env.FindInstalledFiles())
@@ -544,7 +513,44 @@ def list_targets(dag_mode=False):
                 scan_for_transitive_install(f, env, None)
 
         mapping = env[ALIAS_MAP][env[META_COMPONENT]][env[META_ROLE]]
-        list_recursive(mapping)
+        list_hierarchical_aib_recursive(mapping)
+
+    return target_lister
+
+
+def list_recursive(mapping) -> List[str]:
+    items = set()
+    items.add(mapping.id)
+    for dep in mapping.dependencies:
+        items |= list_recursive(dep)
+    return items
+
+
+def list_targets():
+    def target_lister(env, **kwargs):
+        mapping = env[ALIAS_MAP][env[META_COMPONENT]][env[META_ROLE]]
+        tasks = sorted(list(env[TASKS].keys()))
+        roles = sorted(list(env[ROLE_DECLARATIONS].keys()))
+        targets_with_role = list(list_recursive(mapping)) + [mapping.id]
+        targets: List[str] = []
+        for target_role in targets_with_role:
+            # Does this target_role end with one of our speicifed roles
+            matching_roles = list(filter(target_role.endswith, [f"-{role}" for role in roles]))
+            assert len(matching_roles) == 1
+
+            targets.append(target_role[:-len(matching_roles[0])])
+
+        # dedup and sort targets
+        targets = sorted(list(set(targets)))
+        print(
+            "The following are AIB targets. Note that runtime role is implied if not specified. For example, install-mongod"
+        )
+        tasks_str = ','.join(tasks)
+        print(f"TASK={{{tasks_str}}}")
+        roles_str = ','.join(roles)
+        print(f"ROLE={{{roles_str}}}")
+        for target in targets:
+            print(f"  TASK-{target}-ROLE")
 
     return target_lister
 
@@ -574,12 +580,13 @@ def generate(env):  # pylint: disable=too-many-statements
     env[SUFFIX_MAP] = {}
     env[ALIAS_MAP] = defaultdict(dict)
 
-    env[TASKS] = {
+    env.AppendUnique(AIB_TASKS={
         "install": auto_install_task,
-    }
+    })
 
     env.AddMethod(
-        scan_for_transitive_install_pseudobuilder, "GetTransitivelyInstalledFiles"
+        scan_for_transitive_install_pseudobuilder,
+        "GetTransitivelyInstalledFiles",
     )
     env.AddMethod(get_role_declaration, "GetRoleDeclaration")
     env.AddMethod(get_auto_installed_files, "GetAutoInstalledFiles")
@@ -597,11 +604,14 @@ def generate(env):  # pylint: disable=too-many-statements
     env.Alias("list-aib-components", [], [list_components])
     env.AlwaysBuild("list-aib-components")
 
-    env.Alias("list-aib-targets", [], [list_targets(dag_mode=False)])
-    env.AlwaysBuild("list-aib-targets")
+    env.Alias("list-hierarchical-aib-targets", [], [list_hierarchical_aib_targets(dag_mode=False)])
+    env.AlwaysBuild("list-hierarchical-aib-targets")
 
-    env.Alias("list-aib-dag", [], [list_targets(dag_mode=True)])
-    env.AlwaysBuild("list-aib-dag")
+    env.Alias("list-hierarchical-aib-dag", [], [list_hierarchical_aib_targets(dag_mode=True)])
+    env.AlwaysBuild("list-hierarchical-aib-dag")
+
+    env.Alias("list-targets", [], [list_targets()])
+    env.AlwaysBuild("list-targets")
 
     for builder in ["Program", "SharedLibrary", "LoadableModule", "StaticLibrary"]:
         builder = env["BUILDERS"][builder]
@@ -615,5 +625,6 @@ def generate(env):  # pylint: disable=too-many-statements
     assert base_install_builder.target_scanner is None
 
     base_install_builder.target_scanner = SCons.Scanner.Scanner(
-        function=scan_for_transitive_install, path_function=None
+        function=scan_for_transitive_install,
+        path_function=None,
     )
