@@ -1,16 +1,24 @@
-# Copyright 2019 MongoDB Inc.
+# Copyright 2020 MongoDB Inc.
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
 #
-# http://www.apache.org/licenses/LICENSE-2.0
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
+# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
+# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 """Generate build.ninja files from SCons aliases."""
 
 import sys
@@ -19,8 +27,10 @@ import importlib
 import io
 import shutil
 import shlex
+import tempfile
 import textwrap
 
+from collections import OrderedDict
 from glob import glob
 from os.path import join as joinpath
 from os.path import splitext
@@ -142,9 +152,7 @@ def alias_to_ninja_build(node):
     return {
         "outputs": get_outputs(node),
         "rule": "phony",
-        "implicit": [
-            get_path(src_file(n)) for n in node.children() if is_valid_dependent_node(n)
-        ],
+        "implicit": [get_path(src_file(n)) for n in node.children() if is_valid_dependent_node(n)],
     }
 
 
@@ -152,18 +160,22 @@ def get_order_only(node):
     """Return a list of order only dependencies for node."""
     if node.prerequisites is None:
         return []
-    return [get_path(src_file(prereq)) for prereq in node.prerequisites if is_valid_dependent_node(prereq)]
+    return [
+        get_path(src_file(prereq)) for prereq in node.prerequisites
+        if is_valid_dependent_node(prereq)
+    ]
 
 
 def get_dependencies(node, skip_sources=False):
     """Return a list of dependencies for node."""
     if skip_sources:
         return [
-            get_path(src_file(child))
-            for child in node.children()
+            get_path(src_file(child)) for child in node.children()
             if child not in node.sources and is_valid_dependent_node(child)
         ]
-    return [get_path(src_file(child)) for child in node.children() if is_valid_dependent_node(child)]
+    return [
+        get_path(src_file(child)) for child in node.children() if is_valid_dependent_node(child)
+    ]
 
 
 def get_inputs(node, skip_unknown_types=False):
@@ -192,10 +204,12 @@ def get_inputs(node, skip_unknown_types=False):
         else:
             if skip_unknown_types:
                 continue
-            raise Exception("Can't process {} node '{}' as an input for '{}'".format(
-                type(input_node),
-                str(input_node),
-                str(node)))
+            raise Exception(
+                "Can't process {} node '{}' as an input for '{}'".format(
+                    type(input_node),
+                    str(input_node),
+                    str(node),
+                ), )
 
     # convert node items into raw paths/aliases for ninja
     return [get_path(src_file(o)) for o in ninja_nodes]
@@ -216,6 +230,7 @@ def get_outputs(node):
 
     return outputs
 
+
 def generate_depfile(env, node, dependencies):
     """
     Ninja tool function for writing a depfile. The depfile should include
@@ -251,6 +266,7 @@ def generate_depfile(env, node, dependencies):
         with open(depfile, 'w') as f:
             f.write(depfile_contents)
 
+
 class SConsToNinjaTranslator:
     """Translates SCons Actions into Ninja build objects."""
 
@@ -347,25 +363,19 @@ class SConsToNinjaTranslator:
         if handler is not None:
             return handler(node.env if node.env else self.env, node)
 
-        raise Exception(
-            "Found unhandled function action {}, "
-            " generating scons command to build\n"
-            "Note: this is less efficient than Ninja,"
-            " you can write your own ninja build generator for"
-            " this function using NinjaRegisterFunctionHandler".format(name)
-        )
+        raise Exception("Found unhandled function action {}, "
+                        " generating scons command to build\n"
+                        "Note: this is less efficient than Ninja,"
+                        " you can write your own ninja build generator for"
+                        " this function using NinjaRegisterFunctionHandler".format(name))
 
     # pylint: disable=too-many-branches
     def handle_list_action(self, node, action):
         """TODO write this comment"""
         results = [
-            self.action_to_ninja_build(node, action=act)
-            for act in action.list
-            if act is not None
-        ]
-        results = [
-            result for result in results if result is not None and result["outputs"]
+            self.action_to_ninja_build(node, action=act) for act in action.list if act is not None
         ]
+        results = [result for result in results if result is not None and result["outputs"]]
         if not results:
             return None
 
@@ -404,12 +414,16 @@ class SConsToNinjaTranslator:
 
             # Make sure we didn't generate an empty cmdline
             if cmdline:
+
+                env = node.env if node.env else self.env
+                sources = [get_path(src_file(s)) for s in node.sources]
+
                 ninja_build = {
                     "outputs": all_outputs,
                     "rule": "CMD",
                     "variables": {
                         "cmd": cmdline,
-                        "env": get_command_env(node.env if node.env else self.env),
+                        "env": get_command_env(env, all_outputs, sources),
                     },
                     "implicit": dependencies,
                 }
@@ -470,20 +484,22 @@ class NinjaState:
                         if arg not in COMMAND_LINE_TARGETS
                     ]),
                 ),
-            "SCONS_INVOCATION_W_TARGETS": "{} {}".format(
-                sys.executable, " ".join([scons_escape(arg) for arg in sys.argv])
-            ),
+            "SCONS_INVOCATION_W_TARGETS":
+                "{} {}".format(
+                    sys.executable,
+                    " ".join([ninja_syntax.escape(scons_escape(arg)) for arg in sys.argv])),
             # This must be set to a global default per:
             # https://ninja-build.org/manual.html
             #
             # (The deps section)
-            "msvc_deps_prefix": "Note: including file:",
+            "msvc_deps_prefix":
+                "Note: including file:",
         }
 
         self.rules = {
             "CMD": {
                 "command": "cmd /c $env$cmd" if sys.platform == "win32" else "$env$cmd",
-                "description": "Building $out",
+                "description": "Built $out",
                 "pool": "local_pool",
             },
             # We add the deps processing variables to this below. We
@@ -493,13 +509,13 @@ class NinjaState:
             # command.
             "CC": {
                 "command": "$env$CC @$out.rsp",
-                "description": "Compiling $out",
+                "description": "Compiled $out",
                 "rspfile": "$out.rsp",
                 "rspfile_content": "$rspc",
             },
             "CXX": {
                 "command": "$env$CXX @$out.rsp",
-                "description": "Compiling $out",
+                "description": "Compiled $out",
                 "rspfile": "$out.rsp",
                 "rspfile_content": "$rspc",
             },
@@ -517,7 +533,7 @@ class NinjaState:
             },
             "LINK": {
                 "command": "$env$LINK @$out.rsp",
-                "description": "Linking $out",
+                "description": "Linked $out",
                 "rspfile": "$out.rsp",
                 "rspfile_content": "$rspc",
                 "pool": "local_pool",
@@ -531,20 +547,22 @@ class NinjaState:
             },
             "SYMLINK": {
                 "command": (
-                    "cmd /c mklink $out $in"
-                    if sys.platform == "win32"
-                    else "ln -s $in $out"
-                ),
-                "description": "Symlink $in -> $out",
+                    "cmd /c mklink $out $in" if sys.platform == "win32" else "ln -s $in $out"),
+                "description": "Symlinked $in -> $out",
+            },
+            "NOOP": {
+                "command": "$NOOP",
+                "description": "Checked $out",
+                "pool": "local_pool",
             },
             "INSTALL": {
                 "command": "$COPY $in $out",
-                "description": "Install $out",
+                "description": "Installed $out",
                 "pool": "install_pool",
             },
             "TEMPLATE": {
                 "command": "$SCONS_INVOCATION $out",
-                "description": "Rendering $out",
+                "description": "Rendered $out",
                 "pool": "scons_pool",
                 "restat": 1,
             },
@@ -572,9 +590,9 @@ class NinjaState:
             },
             "REGENERATE": {
                 "command": "$SCONS_INVOCATION_W_TARGETS",
-                "description": "Regenerating $out",
-                "generator": 1,
+                "description": "Regenerated $self",
                 "depfile": os.path.join(get_path(env['NINJA_BUILDDIR']), '$out.depfile'),
+                "generator": 1,
                 # Console pool restricts to 1 job running at a time,
                 # it additionally has some special handling about
                 # passing stdin, stdout, etc to process in this pool
@@ -587,10 +605,10 @@ class NinjaState:
                 "restat": 1,
             },
         }
-
+        num_jobs = self.env.get('NINJA_MAX_JOBS', self.env.GetOption("num_jobs"))
         self.pools = {
-            "local_pool": self.env.GetOption("num_jobs"),
-            "install_pool": self.env.GetOption("num_jobs") / 2,
+            "local_pool": num_jobs,
+            "install_pool": num_jobs / 2,
             "scons_pool": 1,
         }
 
@@ -659,7 +677,7 @@ class NinjaState:
         ninja.variable("builddir", get_path(self.env['NINJA_BUILDDIR']))
 
         for pool_name, size in self.pools.items():
-            ninja.pool(pool_name, size)
+            ninja.pool(pool_name, min(self.env.get('NINJA_MAX_JOBS', size), size))
 
         for var, val in self.variables.items():
             ninja.variable(var, val)
@@ -707,20 +725,17 @@ class NinjaState:
 
         if generated_sources_alias and generated_sources_build:
             generated_source_files = sorted(
-                [] if not generated_sources_build else generated_sources_build['implicit']
-            )
+                [] if not generated_sources_build else generated_sources_build['implicit'])
+
             def check_generated_source_deps(build):
-                return (
-                    build != generated_sources_build
-                    and set(build["outputs"]).isdisjoint(generated_source_files)
-                )
+                return (build != generated_sources_build
+                        and set(build["outputs"]).isdisjoint(generated_source_files))
         else:
             generated_sources_build = None
             generated_source_files = sorted({
                 output
                 # First find builds which have header files in their outputs.
-                for build in self.builds.values()
-                if self.has_generated_sources(build["outputs"])
+                for build in self.builds.values() if self.has_generated_sources(build["outputs"])
                 for output in build["outputs"]
                 # Collect only the header files from the builds with them
                 # in their output. We do this because is_generated_source
@@ -732,17 +747,17 @@ class NinjaState:
 
             if generated_source_files:
                 generated_sources_alias = "_ninja_generated_sources"
-                ninja.build(
+                ninja_sorted_build(
+                    ninja,
                     outputs=generated_sources_alias,
                     rule="phony",
-                    implicit=generated_source_files
+                    implicit=generated_source_files,
                 )
+
                 def check_generated_source_deps(build):
-                    return (
-                        not build["rule"] == "INSTALL"
-                        and set(build["outputs"]).isdisjoint(generated_source_files)
-                        and set(build.get("implicit", [])).isdisjoint(generated_source_files)
-                    )
+                    return (not build["rule"] == "INSTALL"
+                            and set(build["outputs"]).isdisjoint(generated_source_files)
+                            and set(build.get("implicit", [])).isdisjoint(generated_source_files))
 
         template_builders = []
 
@@ -798,10 +813,7 @@ class NinjaState:
             # sources and none of the direct implicit dependencies are
             # generated sources or else we will create a dependency
             # cycle.
-            if (
-                generated_source_files
-                and check_generated_source_deps(build)
-            ):
+            if (generated_source_files and check_generated_source_deps(build)):
 
                 # Make all non-generated source targets depend on
                 # _generated_sources. We use order_only for generated
@@ -854,8 +866,11 @@ class NinjaState:
                 )
 
                 if remaining_outputs:
-                    ninja.build(
-                        outputs=sorted(remaining_outputs), rule="phony", implicit=first_output,
+                    ninja_sorted_build(
+                        ninja,
+                        outputs=sorted(remaining_outputs),
+                        rule="phony",
+                        implicit=first_output,
                     )
 
                 build["outputs"] = first_output
@@ -867,13 +882,14 @@ class NinjaState:
             # be repurposed for anything, as long as you have a way to regenerate the depfile.
             # More specific info can be found here: https://ninja-build.org/manual.html#_depfile
             if rule is not None and rule.get('depfile') and build.get('deps_files'):
-                path = build['outputs'] if SCons.Util.is_List(build['outputs']) else [build['outputs']]
+                path = build['outputs'] if SCons.Util.is_List(
+                    build['outputs']) else [build['outputs']]
                 generate_depfile(self.env, path[0], build.pop('deps_files', []))
 
             if "inputs" in build:
                 build["inputs"].sort()
 
-            ninja.build(**build)
+            ninja_sorted_build(ninja, **build)
 
         for build, kwargs in self.builds.items():
             if kwargs['rule'] in [
@@ -906,11 +922,8 @@ class NinjaState:
                     new_val.append(cur_val)
                 template_builds[agg_key] = new_val
 
-            # Collect all other keys
-            template_builds.update(template_builder)
-
         if template_builds.get("outputs", []):
-            ninja.build(**template_builds)
+            ninja_sorted_build(ninja, **template_builds)
 
         # We have to glob the SCons files here to teach the ninja file
         # how to regenerate itself. We'll never see ourselves in the
@@ -927,24 +940,38 @@ class NinjaState:
         # list of build generation about. However, because the generate rule
         # is hardcoded here, we need to do this generate_depfile call manually.
         ninja_file_path = self.env.File(ninja_file).path
+        ninja_in_file_path = os.path.join(
+            get_path(self.env['NINJA_BUILDDIR']), os.path.basename(ninja_file)) + ".in"
         generate_depfile(
             self.env,
-            ninja_file_path,
-            self.env['NINJA_REGENERATE_DEPS']
+            ninja_in_file_path,
+            self.env['NINJA_REGENERATE_DEPS'],
         )
 
-        ninja.build(
-            ninja_file_path,
+        ninja_sorted_build(
+            ninja,
+            outputs=ninja_in_file_path,
             rule="REGENERATE",
+            variables={
+                "self": ninja_file_path,
+            },
+        )
+
+        # This sets up a dependency edge between build.ninja.in and build.ninja
+        # without actually taking any action to transform one into the other
+        # because we write both files ourselves later.
+        ninja_sorted_build(
+            ninja,
+            outputs=ninja_file_path,
+            rule="NOOP",
+            inputs=[ninja_in_file_path],
             implicit=[__file__],
         )
 
         # Look in SCons's list of DEFAULT_TARGETS, find the ones that
         # we generated a ninja build rule for.
         scons_default_targets = [
-            get_path(tgt)
-            for tgt in SCons.Script.DEFAULT_TARGETS
-            if get_path(tgt) in self.built
+            get_path(tgt) for tgt in SCons.Script.DEFAULT_TARGETS if get_path(tgt) in self.built
         ]
 
         # If we found an overlap between SCons's list of default
@@ -953,8 +980,10 @@ class NinjaState:
         if scons_default_targets:
             ninja.default(" ".join(scons_default_targets))
 
-        with open(ninja_file, "w") as build_ninja:
-            build_ninja.write(content.getvalue())
+        with tempfile.NamedTemporaryFile(delete=False, mode='w') as temp_ninja_file:
+            temp_ninja_file.write(content.getvalue())
+        shutil.move(temp_ninja_file.name, ninja_file)
+        shutil.copy2(ninja_file, ninja_in_file_path)
 
         self.__generated = True
 
@@ -1003,9 +1032,27 @@ def get_comstr(env, action, targets, sources):
     return action.genstring(targets, sources, env)
 
 
-def get_command_env(env):
+def ninja_recursive_sorted_dict(build):
+    sorted_dict = OrderedDict()
+    for key, val in sorted(build.items()):
+        if isinstance(val, dict):
+            sorted_dict[key] = ninja_recursive_sorted_dict(val)
+        elif isinstance(val, list) and key in ('inputs', 'outputs', 'implicit', 'order_only',
+                                               'implicit_outputs'):
+            sorted_dict[key] = sorted(val)
+        else:
+            sorted_dict[key] = val
+    return sorted_dict
+
+
+def ninja_sorted_build(ninja, **build):
+    sorted_dict = ninja_recursive_sorted_dict(build)
+    ninja.build(**sorted_dict)
+
+
+def get_command_env(env, target, source):
     """
-    Return a string that sets the enrivonment for any environment variables that
+    Return a string that sets the environment for any environment variables that
     differ between the OS environment and the SCons command ENV.
 
     It will be compatible with the default shell of the operating system.
@@ -1019,16 +1066,15 @@ def get_command_env(env):
     # os.environ or differ from it. We assume if it's a new or
     # differing key from the process environment then it's
     # important to pass down to commands in the Ninja file.
-    ENV = get_default_ENV(env)
+    ENV = env.get('SHELL_ENV_GENERATOR', get_default_ENV)(env, target, source)
     scons_specified_env = {
         key: value
-        for key, value in ENV.items()
-        if key not in os.environ or os.environ.get(key, None) != value
+        for key, value in ENV.items() if key not in os.environ or os.environ.get(key, None) != value
     }
 
     windows = env["PLATFORM"] == "win32"
     command_env = ""
-    for key, value in scons_specified_env.items():
+    for key, value in sorted(scons_specified_env.items()):
         # Ensure that the ENV values are all strings:
         if is_List(value):
             # If the value is a list, then we assume it is a
@@ -1052,15 +1098,19 @@ def get_command_env(env):
             # doesn't make builds on paths with spaces (Ninja and SCons issues)
             # nor expanding response file paths with spaces (Ninja issue) work.
             value = value.replace(r' ', r'$ ')
-            command_env += "{}='{}' ".format(key, value)
+            command_env += "export {}='{}';".format(key,
+                                                    env.subst(value, target=target, source=source))
 
     env["NINJA_ENV_VAR_CACHE"] = command_env
     return command_env
 
 
-def gen_get_response_file_command(env, rule, tool, tool_is_dynamic=False):
+def gen_get_response_file_command(env, rule, tool, tool_is_dynamic=False, custom_env=None):
     """Generate a response file command provider for rule name."""
 
+    if custom_env is None:
+        custom_env = {}
+
     # If win32 using the environment with a response file command will cause
     # ninja to fail to create the response file. Additionally since these rules
     # generally are not piping through cmd.exe /c any environment variables will
@@ -1077,15 +1127,11 @@ def gen_get_response_file_command(env, rule, tool, tool_is_dynamic=False):
             cmd_list, _, _ = action.process(targets, sources, env, executor=executor)
             cmd_list = [str(c).replace("$", "$$") for c in cmd_list[0]]
         else:
-            command = generate_command(
-                env, node, action, targets, sources, executor=executor
-            )
+            command = generate_command(env, node, action, targets, sources, executor=executor)
             cmd_list = shlex.split(command)
 
         if tool_is_dynamic:
-            tool_command = env.subst(
-                tool, target=targets, source=sources, executor=executor
-            )
+            tool_command = env.subst(tool, target=targets, source=sources, executor=executor)
         else:
             tool_command = tool
 
@@ -1093,11 +1139,8 @@ def gen_get_response_file_command(env, rule, tool, tool_is_dynamic=False):
             # Add 1 so we always keep the actual tool inside of cmd
             tool_idx = cmd_list.index(tool_command) + 1
         except ValueError:
-            raise Exception(
-                "Could not find tool {} in {} generated from {}".format(
-                    tool, cmd_list, get_comstr(env, action, targets, sources)
-                )
-            )
+            raise Exception("Could not find tool {} in {} generated from {}".format(
+                tool, cmd_list, get_comstr(env, action, targets, sources)))
 
         cmd, rsp_content = cmd_list[:tool_idx], cmd_list[tool_idx:]
         rsp_content = " ".join(rsp_content)
@@ -1105,7 +1148,15 @@ def gen_get_response_file_command(env, rule, tool, tool_is_dynamic=False):
         variables = {"rspc": rsp_content}
         variables[rule] = cmd
         if use_command_env:
-            variables["env"] = get_command_env(env)
+            variables["env"] = get_command_env(env, targets, sources)
+
+            for key, value in custom_env.items():
+                variables["env"] += env.subst(
+                    f"export {key}={value};",
+                    target=targets,
+                    source=sources,
+                    executor=executor,
+                ) + " "
         return rule, variables, [tool_command]
 
     return get_response_file_command
@@ -1137,11 +1188,17 @@ def generate_command(env, node, action, targets, sources, executor=None):
 
 
 def get_generic_shell_command(env, node, action, targets, sources, executor=None):
+
+    if env.get('NINJA_TEMPLATE'):
+        rule = 'TEMPLATE'
+    else:
+        rule = 'CMD'
+
     return (
-        "CMD",
+        rule,
         {
             "cmd": generate_command(env, node, action, targets, sources, executor=None),
-            "env": get_command_env(env),
+            "env": get_command_env(env, targets, sources),
         },
         # Since this function is a rule mapping provider, it must return a list of dependencies,
         # and usually this would be the path to a tool, such as a compiler, used for this rule.
@@ -1150,7 +1207,7 @@ def get_generic_shell_command(env, node, action, targets, sources, executor=None
         # generally this function will not be used soley and is more like a template to generate
         # the basics for a custom provider which may have more specific options for a provier
         # function for a custom NinjaRuleMapping.
-        []
+        [],
     )
 
 
@@ -1187,7 +1244,14 @@ def get_command(env, node, action):  # pylint: disable=too-many-branches
         return None
 
     provider = __NINJA_RULE_MAPPING.get(comstr, get_generic_shell_command)
-    rule, variables, provider_deps = provider(sub_env, node, action, tlist, slist, executor=executor)
+    rule, variables, provider_deps = provider(
+        sub_env,
+        node,
+        action,
+        tlist,
+        slist,
+        executor=executor,
+    )
 
     # Get the dependencies for all targets
     implicit = list({dep for tgt in tlist for dep in get_dependencies(tgt)})
@@ -1210,7 +1274,8 @@ def get_command(env, node, action):  # pylint: disable=too-many-branches
         # in some case the tool could be in the local directory and be suppled without the ext
         # such as in windows, so append the executable suffix and check.
         prog_suffix = sub_env.get('PROGSUFFIX', '')
-        provider_dep_ext = provider_dep if provider_dep.endswith(prog_suffix) else provider_dep + prog_suffix
+        provider_dep_ext = provider_dep if provider_dep.endswith(
+            prog_suffix) else provider_dep + prog_suffix
         if os.path.exists(provider_dep_ext):
             implicit.append(provider_dep_ext)
             continue
@@ -1218,7 +1283,8 @@ def get_command(env, node, action):  # pylint: disable=too-many-branches
         # Many commands will assume the binary is in the path, so
         # we accept this as a possible input from a given command.
 
-        provider_dep_abspath = sub_env.WhereIs(provider_dep) or sub_env.WhereIs(provider_dep, path=os.environ["PATH"])
+        provider_dep_abspath = sub_env.WhereIs(provider_dep) or sub_env.WhereIs(
+            provider_dep, path=os.environ["PATH"])
         if provider_dep_abspath:
             implicit.append(provider_dep_abspath)
             continue
@@ -1334,10 +1400,12 @@ def register_custom_pool(env, pool, size):
     """Allows the creation of custom Ninja pools"""
     env[NINJA_POOLS][pool] = size
 
+
 def set_build_node_callback(env, node, callback):
     if 'conftest' not in str(node):
         setattr(node.attributes, "ninja_build_callback", callback)
 
+
 def ninja_csig(original):
     """Return a dummy csig"""
 
@@ -1361,6 +1429,7 @@ def ninja_contents(original):
 
     return wrapper
 
+
 def CheckNinjaCompdbExpand(env, context):
     """ Configure check testing if ninja's compdb can expand response files"""
 
@@ -1371,18 +1440,20 @@ def CheckNinjaCompdbExpand(env, context):
         text=textwrap.dedent("""
             rule CMD_RSP
               command = $cmd @$out.rsp > fake_output.txt
-              description = Building $out
+              description = Built $out
               rspfile = $out.rsp
               rspfile_content = $rspc
             build fake_output.txt: CMD_RSP fake_input.txt
               cmd = echo
               pool = console
               rspc = "test"
-            """))
+            """),
+    )
     result = '@fake_output.txt.rsp' not in output
     context.Result(result)
     return result
 
+
 def ninja_stat(_self, path):
     """
     Eternally memoized stat call.
@@ -1455,6 +1526,13 @@ def ninja_always_serial(self, num, taskmaster):
     self.job = SCons.Job.Serial(taskmaster)
 
 
+def ninja_print_conf_log(s, target, source, env):
+    """Command line print only for conftest to generate a correct conf log."""
+    if target and "conftest" in str(target[0]):
+        action = SCons.Action._ActionAction()
+        action.print_cmd_line(s, target, source, env)
+
+
 class NinjaNoResponseFiles(SCons.Platform.TempFileMunge):
     """Overwrite the __call__ method of SCons' TempFileMunge to not delete."""
 
@@ -1501,9 +1579,13 @@ def generate(env):
     # exists upstream: https://github.com/SCons/scons/issues/3625
     def ninja_generate_deps(env):
         return sorted([env.File("#SConstruct").path] + glob("**/SConscript", recursive=True))
+
     env['_NINJA_REGENERATE_DEPS_FUNC'] = ninja_generate_deps
 
-    env['NINJA_REGENERATE_DEPS'] = env.get('NINJA_REGENERATE_DEPS', '${_NINJA_REGENERATE_DEPS_FUNC(__env__)}')
+    env['NINJA_REGENERATE_DEPS'] = env.get(
+        'NINJA_REGENERATE_DEPS',
+        '${_NINJA_REGENERATE_DEPS_FUNC(__env__)}',
+    )
 
     # This adds the required flags such that the generated compile
     # commands will create depfiles as appropriate in the Ninja file.
@@ -1561,12 +1643,8 @@ def generate(env):
         from SCons.Tool.mslink import compositeLinkAction
 
         if env["LINKCOM"] == compositeLinkAction:
-            env[
-                "LINKCOM"
-            ] = '${TEMPFILE("$LINK $LINKFLAGS /OUT:$TARGET.windows $_LIBDIRFLAGS $_LIBFLAGS $_PDB $SOURCES.windows", "$LINKCOMSTR")}'
-            env[
-                "SHLINKCOM"
-            ] = '${TEMPFILE("$SHLINK $SHLINKFLAGS $_SHLINK_TARGETS $_LIBDIRFLAGS $_LIBFLAGS $_PDB $_SHLINK_SOURCES", "$SHLINKCOMSTR")}'
+            env["LINKCOM"] = '${TEMPFILE("$LINK $LINKFLAGS /OUT:$TARGET.windows $_LIBDIRFLAGS $_LIBFLAGS $_PDB $SOURCES.windows", "$LINKCOMSTR")}'
+            env["SHLINKCOM"] = '${TEMPFILE("$SHLINK $SHLINKFLAGS $_SHLINK_TARGETS $_LIBDIRFLAGS $_LIBFLAGS $_PDB $_SHLINK_SOURCES", "$SHLINKCOMSTR")}'
 
     # Normally in SCons actions for the Program and *Library builders
     # will return "${*COM}" as their pre-subst'd command line. However
@@ -1664,12 +1742,8 @@ def generate(env):
     # slows down the build significantly and we don't need contents or
     # content signatures calculated when generating a ninja file since
     # we're not doing any SCons caching or building.
-    SCons.Executor.Executor.get_contents = ninja_contents(
-        SCons.Executor.Executor.get_contents
-    )
-    SCons.Node.Alias.Alias.get_contents = ninja_contents(
-        SCons.Node.Alias.Alias.get_contents
-    )
+    SCons.Executor.Executor.get_contents = ninja_contents(SCons.Executor.Executor.get_contents)
+    SCons.Node.Alias.Alias.get_contents = ninja_contents(SCons.Node.Alias.Alias.get_contents)
     SCons.Node.FS.File.get_contents = ninja_contents(SCons.Node.FS.File.get_contents)
     SCons.Node.FS.File.get_csig = ninja_csig(SCons.Node.FS.File.get_csig)
     SCons.Node.FS.Dir.get_csig = ninja_csig(SCons.Node.FS.Dir.get_csig)
@@ -1686,7 +1760,7 @@ def generate(env):
     SCons.Executor.Executor._get_unchanged_targets = SCons.Executor.Executor._get_targets
 
     # Replace false action messages with nothing.
-    env["PRINT_CMD_LINE_FUNC"] = ninja_noop
+    env["PRINT_CMD_LINE_FUNC"] = ninja_print_conf_log
 
     # This reduces unnecessary subst_list calls to add the compiler to
     # the implicit dependencies of targets. Since we encode full paths
@@ -1741,9 +1815,10 @@ def generate(env):
         try:
             emitter = builder.emitter
             if emitter is not None:
-                builder.emitter = SCons.Builder.ListEmitter(
-                    [emitter, ninja_file_depends_on_all]
-                )
+                builder.emitter = SCons.Builder.ListEmitter([
+                    emitter,
+                    ninja_file_depends_on_all,
+                ], )
             else:
                 builder.emitter = ninja_file_depends_on_all
         # Users can inject whatever they want into the BUILDERS
