@@ -27,12 +27,11 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+#include "mongo/bson/bsonelement.h"
 
 #include "mongo/db/jsobj.h"
 
 #include "mongo/base/data_range.h"
-#include "mongo/bson/bson_validate.h"
 #include "mongo/bson/bsonelement_comparator_interface.h"
 #include "mongo/bson/generator_extended_canonical_2_0_0.h"
 #include "mongo/bson/generator_extended_relaxed_2_0_0.h"
@@ -43,6 +42,9 @@
 #include "mongo/util/hex.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
+
 namespace mongo {
 
 namespace {
@@ -90,9 +92,8 @@ int compareObjects(const BSONObj& firstObj,
 void BSONObj::_assertInvalid(int maxSize) const {
     StringBuilder ss;
     int os = objsize();
-    ss << "BSONObj size: " << os << " (0x" << integerToHex(os) << ") is invalid. "
-       << "Size must be between 0 and " << BSONObjMaxInternalSize << "("
-       << (maxSize / (1024 * 1024)) << "MB)";
+    ss << "BSONObj size: " << os << " (0x" << unsignedHex(os) << ") is invalid. "
+       << "Size must be between 0 and " << maxSize << "(" << (maxSize / (1024 * 1024)) << "MB)";
     try {
         BSONElement e = firstElement();
         ss << " First element: " << e.toString();
@@ -138,25 +139,42 @@ BSONObj BSONObj::getOwned(const BSONObj& obj) {
     return obj.getOwned();
 }
 
-BSONObj BSONObj::redact() const {
+BSONObj BSONObj::redact(bool onlyEncryptedFields) const {
     _validateUnownedSize(objsize());
 
     // Helper to get an "internal function" to be able to do recursion
     struct redactor {
-        void operator()(BSONObjBuilder& builder, const BSONObj& obj, bool appendMask) {
+        void appendRedactedElem(BSONObjBuilder& builder, const BSONElement& e, bool appendMask) {
+            if (appendMask) {
+                builder.append(e.fieldNameStringData(), "###"_sd);
+            } else {
+                builder.appendNull(e.fieldNameStringData());
+            }
+        }
+
+        void operator()(BSONObjBuilder& builder,
+                        const BSONObj& obj,
+                        bool appendMask,
+                        bool onlyEncryptedFields) {
             for (BSONElement e : obj) {
                 if (e.type() == Object) {
                     BSONObjBuilder subBuilder = builder.subobjStart(e.fieldNameStringData());
-                    operator()(subBuilder, e.Obj(), appendMask);
+                    operator()(subBuilder, e.Obj(), appendMask, onlyEncryptedFields);
                     subBuilder.done();
                 } else if (e.type() == Array) {
                     BSONObjBuilder subBuilder = builder.subarrayStart(e.fieldNameStringData());
-                    operator()(subBuilder, e.Obj(), appendMask);
+                    operator()(subBuilder, e.Obj(), appendMask, onlyEncryptedFields);
                     subBuilder.done();
-                } else if (appendMask) {
-                    builder.append(e.fieldNameStringData(), "###"_sd);
                 } else {
-                    builder.appendNull(e.fieldNameStringData());
+                    if (onlyEncryptedFields) {
+                        if (e.type() == BinData && e.binDataType() == BinDataType::Encrypt) {
+                            appendRedactedElem(builder, e, appendMask);
+                        } else {
+                            builder.append(e);
+                        }
+                    } else {
+                        appendRedactedElem(builder, e, appendMask);
+                    }
                 }
             }
         }
@@ -164,7 +182,7 @@ BSONObj BSONObj::redact() const {
 
     try {
         BSONObjBuilder builder;
-        redactor()(builder, *this, /*appendMask=*/true);
+        redactor()(builder, *this, /*appendMask=*/true, onlyEncryptedFields);
         return builder.obj();
     } catch (const ExceptionFor<ErrorCodes::BSONObjectTooLarge>&) {
     }
@@ -174,7 +192,7 @@ BSONObj BSONObj::redact() const {
     // we use BSONType::jstNull, which ensures the redacted object will not be larger than the
     // original.
     BSONObjBuilder builder;
-    redactor()(builder, *this, /*appendMask=*/false);
+    redactor()(builder, *this, /*appendMask=*/false, onlyEncryptedFields);
     return builder.obj();
 }
 
@@ -281,9 +299,6 @@ BSONObj BSONObj::jsonStringBuffer(JsonStringFormat format,
     }
 }
 
-bool BSONObj::valid(BSONVersion version) const {
-    return validateBSON(objdata(), objsize(), version).isOK();
-}
 
 int BSONObj::woCompare(const BSONObj& r,
                        const Ordering& o,
@@ -420,42 +435,6 @@ BSONElement BSONObj::getFieldUsingIndexNames(StringData fieldName, const BSONObj
     return BSONElement();
 }
 
-/* note: addFields always adds _id even if not specified
-   returns n added not counting _id unless requested.
-*/
-int BSONObj::addFields(BSONObj& from, std::set<std::string>& fields) {
-    verify(isEmpty() && !isOwned()); /* partial implementation for now... */
-
-    BSONObjBuilder b;
-
-    int N = fields.size();
-    int n = 0;
-    BSONObjIterator i(from);
-    bool gotId = false;
-    while (i.moreWithEOO()) {
-        BSONElement e = i.next();
-        const char* fname = e.fieldName();
-        if (fields.count(fname)) {
-            b.append(e);
-            ++n;
-            gotId = gotId || strcmp(fname, "_id") == 0;
-            if (n == N && gotId)
-                break;
-        } else if (strcmp(fname, "_id") == 0) {
-            b.append(e);
-            gotId = true;
-            if (n == N && gotId)
-                break;
-        }
-    }
-
-    if (n) {
-        *this = b.obj();
-    }
-
-    return n;
-}
-
 bool BSONObj::couldBeArray() const {
     BSONObjIterator i(*this);
     int index = 0;
@@ -638,9 +617,9 @@ bool BSONObj::getBoolField(StringData name) const {
     return e.type() == Bool ? e.boolean() : false;
 }
 
-const char* BSONObj::getStringField(StringData name) const {
+StringData BSONObj::getStringField(StringData name) const {
     BSONElement e = getField(name);
-    return e.type() == String ? e.valuestr() : "";
+    return e.valueStringDataSafe();
 }
 
 bool BSONObj::getObjectID(BSONElement& e) const {
@@ -672,6 +651,29 @@ BSONObj BSONObj::addField(const BSONElement& field) const {
     return b.obj();
 }
 
+BSONObj BSONObj::addFields(const BSONObj& from,
+                           const boost::optional<StringDataSet>& fields) const {
+    BSONObjBuilder bob;
+    for (auto&& originalField : *this) {
+        auto commonField = from[originalField.fieldNameStringData()];
+        // If there is a common field, add the value from 'from' object.
+        if (commonField && (!fields || fields->count(originalField.fieldNameStringData()))) {
+            bob.append(commonField);
+        } else {
+            bob.append(originalField);
+        }
+    }
+
+    for (auto&& fromField : from) {
+        // Ignore the common fields as they are already added earlier.
+        if (!hasField(fromField.fieldNameStringData()) &&
+            (!fields || fields->count(fromField.fieldNameStringData()))) {
+            bob.append(fromField);
+        }
+    }
+    return bob.obj();
+}
+
 BSONObj BSONObj::removeField(StringData name) const {
     BSONObjBuilder b;
     BSONObjIterator i(*this);
@@ -684,6 +686,28 @@ BSONObj BSONObj::removeField(StringData name) const {
     return b.obj();
 }
 
+BSONObj BSONObj::removeFields(const std::set<std::string>& fields) const {
+    BSONObjBuilder bob;
+    for (auto&& field : *this) {
+        if (fields.count(field.fieldName())) {
+            continue;
+        }
+        bob.append(field);
+    }
+    return bob.obj();
+}
+
+BSONObj BSONObj::removeFields(const StringDataSet& fields) const {
+    BSONObjBuilder bob;
+    for (auto&& field : *this) {
+        if (fields.count(field.fieldNameStringData())) {
+            continue;
+        }
+        bob.append(field);
+    }
+    return bob.obj();
+}
+
 std::string BSONObj::hexDump() const {
     std::stringstream ss;
     const char* d = objdata();
@@ -807,7 +831,7 @@ StringBuilder& operator<<(StringBuilder& s, const BSONObj& o) {
 class BSONIteratorSorted::ElementFieldCmp {
 public:
     ElementFieldCmp(bool isArray);
-    bool operator()(const char* s1, const char* s2) const;
+    bool operator()(const Field& lhs, const Field& rhs) const;
 
 private:
     str::LexNumCmp _cmp;
@@ -815,18 +839,18 @@ private:
 
 BSONIteratorSorted::ElementFieldCmp::ElementFieldCmp(bool isArray) : _cmp(!isArray) {}
 
-bool BSONIteratorSorted::ElementFieldCmp::operator()(const char* s1, const char* s2) const {
-    // Skip the type byte and compare field names.
-    return _cmp(s1 + 1, s2 + 1);
+bool BSONIteratorSorted::ElementFieldCmp::operator()(const Field& lhs, const Field& rhs) const {
+    // Just compare field names.
+    return _cmp(lhs.fieldName, rhs.fieldName);
 }
 
 BSONIteratorSorted::BSONIteratorSorted(const BSONObj& o, const ElementFieldCmp& cmp)
-    : _nfields(o.nFields()), _fields(new const char*[_nfields]) {
+    : _nfields(o.nFields()), _fields(std::make_unique<Field[]>(_nfields)) {
     int x = 0;
     BSONObjIterator i(o);
     while (i.more()) {
-        _fields[x++] = i.next().rawdata();
-        verify(_fields[x - 1]);
+        auto elem = i.next();
+        _fields[x++] = {elem.fieldNameStringData(), elem.size()};
     }
     verify(x == _nfields);
     std::sort(_fields.get(), _fields.get() + _nfields, cmp);
@@ -839,4 +863,17 @@ BSONObjIteratorSorted::BSONObjIteratorSorted(const BSONObj& object)
 BSONArrayIteratorSorted::BSONArrayIteratorSorted(const BSONArray& array)
     : BSONIteratorSorted(array, ElementFieldCmp(true)) {}
 
+/**
+ * Types used to represent BSONObj and BSONArray memory in the Visual Studio debugger
+ */
+#if defined(_MSC_VER) && defined(_DEBUG)
+struct BSONObjData {
+    int32_t size;
+} bsonObjDataInstance;
+
+struct BSONArrayData {
+    int32_t size;
+} bsonObjArrayInstance;
+#endif  // defined(_MSC_VER) && defined(_DEBUG)
+
 }  // namespace mongo
