@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 
 #include "mongo/platform/basic.h"
 
@@ -35,8 +34,10 @@
 
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/bson/json.h"
 #include "mongo/bson/util/bson_extract.h"
+#include "mongo/client/internal_auth.h"
 #include "mongo/client/sasl_client_authenticate.h"
 #include "mongo/config.h"
 #include "mongo/db/auth/authorization_manager.h"
@@ -50,6 +51,9 @@
 #include "mongo/util/net/ssl_options.h"
 #include "mongo/util/password_digest.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
+
+
 namespace mongo {
 namespace auth {
 
@@ -110,7 +114,7 @@ StatusWith<OpMsgRequest> createX509AuthCmd(const BSONObj& params, StringData cli
     if (username != clientName.toString()) {
         StringBuilder message;
         message << "Username \"";
-        message << params[saslCommandUserFieldName].valuestr();
+        message << params[saslCommandUserFieldName].valueStringData();
         message << "\" does not match the provided client certificate user \"";
         message << clientName.toString() << "\"";
         return {ErrorCodes::AuthenticationFailed, message.str()};
@@ -138,7 +142,22 @@ Future<void> authX509(RunCommandHook runCommand, const BSONObj& params, StringDa
     return runCommand(authRequest.getValue()).ignoreValue();
 }
 
+class DefaultInternalAuthParametersProvider : public InternalAuthParametersProvider {
+public:
+    ~DefaultInternalAuthParametersProvider() = default;
+
+    BSONObj get(size_t index, StringData mechanism) final {
+        return getInternalAuthParams(index, mechanism);
+    }
+};
+
 }  // namespace
+
+std::shared_ptr<InternalAuthParametersProvider> createDefaultInternalAuthProvider() {
+    return std::make_shared<DefaultInternalAuthParametersProvider>();
+}
+
+
 //
 // General Auth
 //
@@ -187,67 +206,6 @@ Future<void> authenticateClient(const BSONObj& params,
 
 AuthMongoCRHandler authMongoCR = authMongoCRImpl;
 
-static auto internalAuthKeysMutex = MONGO_MAKE_LATCH();
-static bool internalAuthSet = false;
-static std::vector<std::string> internalAuthKeys;
-static BSONObj internalAuthParams;
-
-void setInternalAuthKeys(const std::vector<std::string>& keys) {
-    stdx::lock_guard<Latch> lk(internalAuthKeysMutex);
-
-    internalAuthKeys = keys;
-    fassert(50996, internalAuthKeys.size() > 0);
-    internalAuthSet = true;
-}
-
-void setInternalUserAuthParams(BSONObj obj) {
-    stdx::lock_guard<Latch> lk(internalAuthKeysMutex);
-    internalAuthParams = obj.getOwned();
-    internalAuthKeys.clear();
-    internalAuthSet = true;
-}
-
-bool hasMultipleInternalAuthKeys() {
-    stdx::lock_guard<Latch> lk(internalAuthKeysMutex);
-    return internalAuthSet && internalAuthKeys.size() > 1;
-}
-
-bool isInternalAuthSet() {
-    stdx::lock_guard<Latch> lk(internalAuthKeysMutex);
-    return internalAuthSet;
-}
-
-BSONObj getInternalAuthParams(size_t idx, const std::string& mechanism) {
-    stdx::lock_guard<Latch> lk(internalAuthKeysMutex);
-    if (!internalAuthSet) {
-        return BSONObj();
-    }
-
-    // If we've set a specific BSONObj as the internal auth pararms, return it if the index
-    // is zero (there are no alternate credentials if we've set a BSONObj explicitly).
-    if (!internalAuthParams.isEmpty()) {
-        return idx == 0 ? internalAuthParams : BSONObj();
-    }
-
-    // If the index is larger than the number of keys we know about then return an empty
-    // BSONObj.
-    if (idx + 1 > internalAuthKeys.size()) {
-        return BSONObj();
-    }
-
-    auto password = internalAuthKeys.at(idx);
-    if (mechanism == kMechanismScramSha1) {
-        password = mongo::createPasswordDigest(
-            internalSecurity.user->getName().getUser().toString(), password);
-    }
-
-    return BSON(saslCommandMechanismFieldName
-                << mechanism << saslCommandUserDBFieldName
-                << internalSecurity.user->getName().getDB() << saslCommandUserFieldName
-                << internalSecurity.user->getName().getUser() << saslCommandPasswordFieldName
-                << password << saslCommandDigestPasswordFieldName << false);
-}
-
 Future<std::string> negotiateSaslMechanism(RunCommandHook runCommand,
                                            const UserName& username,
                                            boost::optional<std::string> mechanismHint,
@@ -273,7 +231,7 @@ Future<std::string> negotiateSaslMechanism(RunCommandHook runCommand,
 
             auto obj = mechsArrayObj.Obj();
             std::vector<std::string> availableMechanisms;
-            for (const auto elem : obj) {
+            for (const auto& elem : obj) {
                 if (elem.type() != String) {
                     return Status{ErrorCodes::BadValue, "Expected array of SASL mechanism names"};
                 }
@@ -290,24 +248,28 @@ Future<std::string> negotiateSaslMechanism(RunCommandHook runCommand,
         });
 }
 
-Future<void> authenticateInternalClient(const std::string& clientSubjectName,
-                                        const HostAndPort& remote,
-                                        boost::optional<std::string> mechanismHint,
-                                        StepDownBehavior stepDownBehavior,
-                                        RunCommandHook runCommand) {
+Future<void> authenticateInternalClient(
+    const std::string& clientSubjectName,
+    const HostAndPort& remote,
+    boost::optional<std::string> mechanismHint,
+    StepDownBehavior stepDownBehavior,
+    RunCommandHook runCommand,
+    std::shared_ptr<InternalAuthParametersProvider> internalParamsProvider) {
+    auto systemUser = internalSecurity.getUser();
     return negotiateSaslMechanism(
-               runCommand, internalSecurity.user->getName(), mechanismHint, stepDownBehavior)
-        .then([runCommand, clientSubjectName, remote](std::string mechanism) -> Future<void> {
-            auto params = getInternalAuthParams(0, mechanism);
+               runCommand, (*systemUser)->getName(), mechanismHint, stepDownBehavior)
+        .then([runCommand, clientSubjectName, remote, internalParamsProvider](
+                  std::string mechanism) -> Future<void> {
+            auto params = internalParamsProvider->get(0, mechanism);
             if (params.isEmpty()) {
                 return Status(ErrorCodes::BadValue,
                               "Missing authentication parameters for internal user auth");
             }
             return authenticateClient(params, remote, clientSubjectName, runCommand)
                 .onError<ErrorCodes::AuthenticationFailed>(
-                    [runCommand, clientSubjectName, remote, mechanism](
+                    [runCommand, clientSubjectName, remote, mechanism, internalParamsProvider](
                         Status status) -> Future<void> {
-                        auto altCreds = getInternalAuthParams(1, mechanism);
+                        auto altCreds = internalParamsProvider->get(1, mechanism);
                         if (!altCreds.isEmpty()) {
                             return authenticateClient(
                                 altCreds, remote, clientSubjectName, runCommand);
@@ -320,11 +282,11 @@ Future<void> authenticateInternalClient(const std::string& clientSubjectName,
 BSONObj buildAuthParams(StringData dbname,
                         StringData username,
                         StringData passwordText,
-                        bool digestPassword) {
-    return BSON(saslCommandMechanismFieldName
-                << "SCRAM-SHA-1" << saslCommandUserDBFieldName << dbname << saslCommandUserFieldName
-                << username << saslCommandPasswordFieldName << passwordText
-                << saslCommandDigestPasswordFieldName << digestPassword);
+                        StringData mechanism) {
+
+    return BSON(saslCommandMechanismFieldName << mechanism << saslCommandUserDBFieldName << dbname
+                                              << saslCommandUserFieldName << username
+                                              << saslCommandPasswordFieldName << passwordText);
 }
 
 StringData getSaslCommandUserDBFieldName() {
@@ -416,7 +378,7 @@ SpeculativeAuthType speculateAuth(BSONObjBuilder* isMasterRequest,
         return SpeculativeAuthType::kNone;
     }
 
-    auto params = std::move(optParams.get());
+    auto params = std::move(optParams.value());
 
     auto ret = _speculateAuth(isMasterRequest,
                               mechanism,
@@ -456,16 +418,5 @@ SpeculativeAuthType speculateInternalAuth(
     return SpeculativeAuthType::kNone;
 }
 
-std::string getInternalAuthDB() {
-    stdx::lock_guard<Latch> lk(internalAuthKeysMutex);
-
-    if (!internalAuthParams.isEmpty()) {
-        return getBSONString(internalAuthParams, saslCommandUserDBFieldName);
-    }
-
-    auto isu = internalSecurity.user;
-    return isu ? isu->getName().getDB().toString() : "admin";
-}
-
 }  // namespace auth
 }  // namespace mongo
