@@ -31,7 +31,6 @@
  * Connect to a Mongo database as a database, from C++.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
 #include "mongo/platform/basic.h"
 
@@ -45,13 +44,16 @@
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/bson/util/builder.h"
 #include "mongo/client/authenticate.h"
+#include "mongo/client/client_api_version_parameters_gen.h"
 #include "mongo/client/constants.h"
 #include "mongo/client/dbclient_cursor.h"
 #include "mongo/config.h"
+#include "mongo/db/api_parameters_gen.h"
+#include "mongo/db/auth/validated_tenancy_scope.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/json.h"
 #include "mongo/db/namespace_string.h"
-#include "mongo/db/query/killcursors_request.h"
+#include "mongo/db/query/kill_cursors_gen.h"
 #include "mongo/db/wire_version.h"
 #include "mongo/executor/remote_command_request.h"
 #include "mongo/executor/remote_command_response.h"
@@ -68,7 +70,9 @@
 #include "mongo/util/debug_util.h"
 #include "mongo/util/net/ssl_manager.h"
 #include "mongo/util/net/ssl_options.h"
-#include "mongo/util/password_digest.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
+
 
 namespace mongo {
 
@@ -93,41 +97,10 @@ bool DBClientBase::isOk(const BSONObj& o) {
     return o["ok"].trueValue();
 }
 
-bool DBClientBase::isNotMasterErrorString(const BSONElement& e) {
-    return e.type() == String && str::contains(e.valuestr(), "not master");
-}
-
-
-enum QueryOptions DBClientBase::availableOptions() {
-    if (!_haveCachedAvailableOptions) {
-        _cachedAvailableOptions = _lookupAvailableOptions();
-        _haveCachedAvailableOptions = true;
-    }
-    return _cachedAvailableOptions;
-}
-
-enum QueryOptions DBClientBase::_lookupAvailableOptions() {
-    BSONObj ret;
-    if (runCommand("admin", BSON("availablequeryoptions" << 1), ret)) {
-        return QueryOptions(ret.getIntField("options"));
-    }
-    return QueryOptions(0);
-}
-
-rpc::ProtocolSet DBClientBase::getClientRPCProtocols() const {
-    return _clientRPCProtocols;
-}
-
-rpc::ProtocolSet DBClientBase::getServerRPCProtocols() const {
-    return _serverRPCProtocols;
-}
-
-void DBClientBase::setClientRPCProtocols(rpc::ProtocolSet protocols) {
-    _clientRPCProtocols = std::move(protocols);
-}
-
-void DBClientBase::_setServerRPCProtocols(rpc::ProtocolSet protocols) {
-    _serverRPCProtocols = std::move(protocols);
+bool DBClientBase::isNotPrimaryErrorString(const BSONElement& e) {
+    return e.type() == String &&
+        (str::contains(e.valueStringData(), "not primary") ||
+         str::contains(e.valueStringData(), "not master"));
 }
 
 void DBClientBase::setRequestMetadataWriter(rpc::RequestMetadataWriter writer) {
@@ -169,25 +142,67 @@ rpc::UniqueReply DBClientBase::parseCommandReplyMessage(const std::string& host,
     return rpc::UniqueReply(replyMsg, std::move(commandReply));
 }
 
+namespace {
+void appendMetadata(OperationContext* opCtx,
+                    const rpc::RequestMetadataWriter& metadataWriter,
+                    const ClientAPIVersionParameters& apiParameters,
+                    OpMsgRequest& request) {
+
+    if (!metadataWriter && !apiParameters.getVersion()) {
+        return;
+    }
+
+    BSONObjBuilder bob(std::move(request.body));
+    if (metadataWriter) {
+        uassertStatusOK(metadataWriter(opCtx, &bob));
+    }
+
+    if (apiParameters.getVersion()) {
+        bool hasVersion = false, hasStrict = false, hasDeprecationErrors = false;
+        auto i = bob.iterator();
+        while (i.more()) {
+            auto elem = i.next();
+            if (elem.fieldNameStringData() == APIParametersFromClient::kApiVersionFieldName) {
+                hasVersion = true;
+            } else if (elem.fieldNameStringData() == APIParametersFromClient::kApiStrictFieldName) {
+                hasStrict = true;
+            } else if (elem.fieldNameStringData() ==
+                       APIParametersFromClient::kApiDeprecationErrorsFieldName) {
+                hasDeprecationErrors = true;
+            }
+        }
+
+        if (!hasVersion) {
+            bob.append(APIParametersFromClient::kApiVersionFieldName, *apiParameters.getVersion());
+        }
+
+        // Include apiStrict/apiDeprecationErrors if they are not boost::none.
+        if (!hasStrict && apiParameters.getStrict()) {
+            bob.append(APIParametersFromClient::kApiStrictFieldName, *apiParameters.getStrict());
+        }
+
+        if (!hasDeprecationErrors && apiParameters.getDeprecationErrors()) {
+            bob.append(APIParametersFromClient::kApiDeprecationErrorsFieldName,
+                       *apiParameters.getDeprecationErrors());
+        }
+    }
+
+    request.body = bob.obj();
+
+    if (opCtx) {
+        if (auto validatedTenancyScope = auth::ValidatedTenancyScope::get(opCtx)) {
+            request.validatedTenancyScope = validatedTenancyScope;
+        }
+    }
+}
+}  // namespace
+
 DBClientBase* DBClientBase::runFireAndForgetCommand(OpMsgRequest request) {
-    // Make sure to reconnect if needed before building our request, since the request depends on
-    // the negotiated protocol which can change due to a reconnect.
+    // Make sure to reconnect if needed before building our request.
     checkConnection();
 
-    if (uassertStatusOK(rpc::negotiate(getClientRPCProtocols(), getServerRPCProtocols())) !=
-        rpc::Protocol::kOpMsg) {
-        // Other protocols don't support fire-and-forget. Downgrade to two-way command and throw
-        // away reply.
-        return runCommandWithTarget(request).second;
-    }
-
-    if (_metadataWriter) {
-        BSONObjBuilder metadataBob(std::move(request.body));
-        uassertStatusOK(
-            _metadataWriter((haveClient() ? cc().getOperationContext() : nullptr), &metadataBob));
-        request.body = metadataBob.obj();
-    }
-
+    auto opCtx = haveClient() ? cc().getOperationContext() : nullptr;
+    appendMetadata(opCtx, _metadataWriter, _apiParameters, request);
     auto requestMsg = request.serialize();
     OpMsg::setFlag(&requestMsg, OpMsg::kMoreToCome);
     say(requestMsg);
@@ -196,33 +211,26 @@ DBClientBase* DBClientBase::runFireAndForgetCommand(OpMsgRequest request) {
 
 std::pair<rpc::UniqueReply, DBClientBase*> DBClientBase::runCommandWithTarget(
     OpMsgRequest request) {
-    // Make sure to reconnect if needed before building our request, since the request depends on
-    // the negotiated protocol which can change due to a reconnect.
+    // Make sure to reconnect if needed before building our request.
     checkConnection();
 
     // call() oddly takes this by pointer, so we need to put it on the stack.
     auto host = getServerAddress();
 
     auto opCtx = haveClient() ? cc().getOperationContext() : nullptr;
-    if (_metadataWriter) {
-        BSONObjBuilder metadataBob(std::move(request.body));
-        uassertStatusOK(_metadataWriter(opCtx, &metadataBob));
-        request.body = metadataBob.obj();
-    }
-
-    auto requestMsg =
-        rpc::messageFromOpMsgRequest(getClientRPCProtocols(), getServerRPCProtocols(), request);
+    appendMetadata(opCtx, _metadataWriter, _apiParameters, request);
+    auto requestMsg = request.serialize();
 
     Message replyMsg;
 
-    // We always want to throw if there was a network error, we do it here
-    // instead of passing 'true' for the 'assertOk' parameter so we can construct a
-    // more helpful error message. Note that call() can itself throw a socket exception.
-    uassert(ErrorCodes::HostUnreachable,
-            str::stream() << "network error while attempting to run "
-                          << "command '" << request.getCommandName() << "' "
-                          << "on host '" << host << "' ",
-            call(requestMsg, replyMsg, false, &host));
+    try {
+        call(requestMsg, replyMsg, &host);
+    } catch (DBException& e) {
+        e.addContext(str::stream() << str::stream() << "network error while attempting to run "
+                                   << "command '" << request.getCommandName() << "' "
+                                   << "on host '" << host << "' ");
+        throw;
+    }
 
     auto commandReply = parseCommandReplyMessage(host, replyMsg);
 
@@ -273,51 +281,6 @@ bool DBClientBase::runCommand(const string& dbname, BSONObj cmd, BSONObj& info,
     return std::get<0>(res);
 }
 
-
-/* note - we build a bson obj here -- for something that is super common like getlasterror you
-          should have that object prebuilt as that would be faster.
-*/
-bool DBClientBase::simpleCommand(const string& dbname, BSONObj* info, const string& command) {
-    BSONObj o;
-    if (info == nullptr)
-        info = &o;
-    BSONObjBuilder b;
-    b.append(command, 1);
-    return runCommand(dbname, b.done(), *info);
-}
-
-bool DBClientBase::runPseudoCommand(StringData db,
-                                    StringData realCommandName,
-                                    StringData pseudoCommandCol,
-                                    const BSONObj& cmdArgs,
-                                    BSONObj& info,
-                                    int options) {
-    BSONObjBuilder bob;
-    bob.append(realCommandName, 1);
-    bob.appendElements(cmdArgs);
-    auto cmdObj = bob.done();
-
-    bool success = false;
-
-    if (!(success = runCommand(db.toString(), cmdObj, info, options))) {
-        auto status = getStatusFromCommandResult(info);
-        verify(!status.isOK());
-
-        if (status == ErrorCodes::CommandResultSchemaViolation) {
-            msgasserted(28624,
-                        str::stream() << "Received bad " << realCommandName
-                                      << " response from server: " << info);
-        } else if (status == ErrorCodes::CommandNotFound) {
-            NamespaceString pseudoCommandNss(db, pseudoCommandCol);
-            // if this throws we just let it escape as that's how runCommand works.
-            info = findOne(pseudoCommandNss.ns(), cmdArgs, nullptr, options);
-            return true;
-        }
-    }
-
-    return success;
-}
-
 long long DBClientBase::count(const NamespaceStringOrUUID nsOrUuid,
                               const BSONObj& query,
                               int options,
@@ -359,67 +322,6 @@ BSONObj DBClientBase::_countCmd(const NamespaceStringOrUUID nsOrUuid,
     return b.obj();
 }
 
-BSONObj DBClientBase::getLastErrorDetailed(bool fsync, bool j, int w, int wtimeout) {
-    return getLastErrorDetailed("admin", fsync, j, w, wtimeout);
-}
-
-BSONObj DBClientBase::getLastErrorDetailed(
-    const std::string& db, bool fsync, bool j, int w, int wtimeout) {
-    BSONObj info;
-    BSONObjBuilder b;
-    b.append("getlasterror", 1);
-
-    if (fsync)
-        b.append("fsync", 1);
-    if (j)
-        b.append("j", 1);
-
-    // only affects request when greater than one node
-    if (w >= 1)
-        b.append("w", w);
-    else if (w == -1)
-        b.append("w", "majority");
-
-    if (wtimeout > 0)
-        b.append("wtimeout", wtimeout);
-
-    runCommand(db, b.obj(), info);
-
-    return info;
-}
-
-string DBClientBase::getLastError(bool fsync, bool j, int w, int wtimeout) {
-    return getLastError("admin", fsync, j, w, wtimeout);
-}
-
-string DBClientBase::getLastError(const std::string& db, bool fsync, bool j, int w, int wtimeout) {
-    BSONObj info = getLastErrorDetailed(db, fsync, j, w, wtimeout);
-    return getLastErrorString(info);
-}
-
-string DBClientBase::getLastErrorString(const BSONObj& info) {
-    if (info["ok"].trueValue()) {
-        BSONElement e = info["err"];
-        if (e.eoo())
-            return "";
-        if (e.type() == Object)
-            return e.toString();
-        return e.str();
-    } else {
-        // command failure
-        BSONElement e = info["errmsg"];
-        if (e.eoo())
-            return "";
-        if (e.type() == Object)
-            return "getLastError command failed: " + e.toString();
-        return "getLastError command failed: " + e.str();
-    }
-}
-
-string DBClientBase::createPasswordDigest(const string& username, const string& clearTextPassword) {
-    return mongo::createPasswordDigest(username, clearTextPassword);
-}
-
 namespace {
 class ScopedMetadataWriterRemover {
     ScopedMetadataWriterRemover(const ScopedMetadataWriterRemover&) = delete;
@@ -461,18 +363,14 @@ void DBClientBase::_auth(const BSONObj& params) {
     // We will only have a client name if SSL is enabled
     std::string clientName = "";
 #ifdef MONGO_CONFIG_SSL
-    if (getSSLManager() != nullptr) {
-        clientName = getSSLManager()->getSSLConfiguration().clientSubjectName.toString();
+    auto sslConfiguration = getSSLConfiguration();
+    if (sslConfiguration) {
+        clientName = sslConfiguration->clientSubjectName.toString();
     }
 #endif
 
     HostAndPort remote(getServerAddress());
-    auth::authenticateClient(params, remote, clientName, _makeAuthRunCommandHook())
-        .onError([](Status status) {
-            // for some reason, DBClient transformed all errors into AuthenticationFailed errors
-            return Status(ErrorCodes::AuthenticationFailed, status.reason());
-        })
-        .get();
+    auth::authenticateClient(params, remote, clientName, _makeAuthRunCommandHook()).get();
 }
 
 Status DBClientBase::authenticateInternalUser(auth::StepDownBehavior stepDownBehavior) {
@@ -488,16 +386,19 @@ Status DBClientBase::authenticateInternalUser(auth::StepDownBehavior stepDownBeh
     // We will only have a client name if SSL is enabled
     std::string clientName = "";
 #ifdef MONGO_CONFIG_SSL
-    if (getSSLManager() != nullptr) {
-        clientName = getSSLManager()->getSSLConfiguration().clientSubjectName.toString();
+    auto sslConfiguration = getSSLConfiguration();
+    if (sslConfiguration) {
+        clientName = sslConfiguration->clientSubjectName.toString();
     }
 #endif
 
+    auto authProvider = auth::createDefaultInternalAuthProvider();
     auto status = auth::authenticateInternalClient(clientName,
                                                    HostAndPort(getServerAddress()),
                                                    boost::none,
                                                    stepDownBehavior,
-                                                   _makeAuthRunCommandHook())
+                                                   _makeAuthRunCommandHook(),
+                                                   authProvider)
                       .getNoThrow();
     if (status.isOK()) {
         return status;
@@ -521,11 +422,23 @@ void DBClientBase::auth(const BSONObj& params) {
 bool DBClientBase::auth(const string& dbname,
                         const string& username,
                         const string& password_text,
-                        string& errmsg,
-                        bool digestPassword) {
+                        string& errmsg) {
+
+    UserName user{username, dbname};
+
+    StatusWith<string> mechResult =
+        auth::negotiateSaslMechanism(_makeAuthRunCommandHook(),
+                                     user,
+                                     boost::none,
+                                     auth::StepDownBehavior::kKeepConnectionOpen)
+            .getNoThrow();
+
+    // To prevent unexpected behavior for existing clients, default to SCRAM-SHA-1 if the SASL
+    // negotiation does not succeeed for some reason.
+    StringData mech = mechResult.isOK() ? mechResult.getValue() : "SCRAM-SHA-1"_sd;
+
     try {
-        const auto authParams =
-            auth::buildAuthParams(dbname, username, password_text, digestPassword);
+        const auto authParams = auth::buildAuthParams(dbname, username, password_text, mech);
         auth(authParams);
         return true;
     } catch (const AssertionException& ex) {
@@ -540,18 +453,19 @@ void DBClientBase::logout(const string& dbname, BSONObj& info) {
     runCommand(dbname, BSON("logout" << 1), info);
 }
 
-bool DBClientBase::isMaster(bool& isMaster, BSONObj* info) {
+bool DBClientBase::isPrimary(bool& isPrimary, BSONObj* info) {
     BSONObjBuilder bob;
-    bob.append("ismaster", 1);
-    if (WireSpec::instance().isInternalClient) {
-        WireSpec::appendInternalClientWireVersion(WireSpec::instance().outgoing, &bob);
+    bob.append(_apiParameters.getVersion() ? "hello" : "ismaster", 1);
+    if (auto wireSpec = WireSpec::instance().get(); wireSpec->isInternalClient) {
+        WireSpec::appendInternalClientWireVersion(wireSpec->outgoing, &bob);
     }
 
     BSONObj o;
     if (info == nullptr)
         info = &o;
     bool ok = runCommand("admin", bob.obj(), *info);
-    isMaster = info->getField("ismaster").trueValue();
+    isPrimary =
+        info->getField(_apiParameters.getVersion() ? "isWritablePrimary" : "ismaster").trueValue();
     return ok;
 }
 
@@ -587,7 +501,7 @@ list<BSONObj> DBClientBase::getCollectionInfos(const string& db, const BSONObj&
     if (runCommand(db,
                    BSON("listCollections" << 1 << "filter" << filter << "cursor" << BSONObj()),
                    res,
-                   QueryOption_SlaveOk)) {
+                   QueryOption_SecondaryOk)) {
         BSONObj cursorObj = res["cursor"].Obj();
         BSONObj collections = cursorObj["firstBatch"].Obj();
         BSONObjIterator it(collections);
@@ -596,14 +510,22 @@ list<BSONObj> DBClientBase::getCollectionInfos(const string& db, const BSONObj&
             infos.push_back(e.Obj().getOwned());
         }
 
+        if (res.hasField(LogicalTime::kOperationTimeFieldName)) {
+            setOperationTime(LogicalTime::fromOperationTime(res).asTimestamp());
+        }
+
         const long long id = cursorObj["id"].Long();
 
         if (id != 0) {
             const std::string ns = cursorObj["ns"].String();
-            unique_ptr<DBClientCursor> cursor = getMore(ns, id, 0, 0);
+            unique_ptr<DBClientCursor> cursor = getMore(ns, id);
             while (cursor->more()) {
                 infos.push_back(cursor->nextSafe().getOwned());
             }
+
+            if (cursor->getOperationTime()) {
+                setOperationTime(*(cursor->getOperationTime()));
+            }
         }
 
         return infos;
@@ -633,7 +555,7 @@ vector<BSONObj> DBClientBase::getDatabaseInfos(const BSONObj& filter,
     BSONObj cmd = bob.done();
 
     BSONObj res;
-    if (runCommand("admin", cmd, res, QueryOption_SlaveOk)) {
+    if (runCommand("admin", cmd, res, QueryOption_SecondaryOk)) {
         BSONObj dbs = res["databases"].Obj();
         BSONObjIterator it(dbs);
         while (it.more()) {
@@ -641,6 +563,10 @@ vector<BSONObj> DBClientBase::getDatabaseInfos(const BSONObj& filter,
             infos.push_back(e.Obj().getOwned());
         }
 
+        if (res.hasField(LogicalTime::kOperationTimeFieldName)) {
+            setOperationTime(LogicalTime::fromOperationTime(res).asTimestamp());
+        }
+
         return infos;
     }
 
@@ -656,192 +582,61 @@ bool DBClientBase::exists(const string& ns) {
     return !results.empty();
 }
 
-/** query N objects from the database into an array.  makes sense mostly when you want a small
- * number of results.  if a huge number, use query() and iterate the cursor.
- */
-void DBClientBase::findN(vector<BSONObj>& out,
-                         const string& ns,
-                         Query query,
-                         int nToReturn,
-                         int nToSkip,
-                         const BSONObj* fieldsToReturn,
-                         int queryOptions,
-                         boost::optional<BSONObj> readConcernObj) {
-    out.reserve(nToReturn);
-
-    unique_ptr<DBClientCursor> c = this->query(NamespaceString(ns),
-                                               query,
-                                               nToReturn,
-                                               nToSkip,
-                                               fieldsToReturn,
-                                               queryOptions,
-                                               0 /* batchSize */,
-                                               readConcernObj);
-
-    // query() throws on network error so OK to uassert with numeric code here.
-    uassert(10276,
-            str::stream() << "DBClientBase::findN: transport error: " << getServerAddress()
-                          << " ns: " << ns << " query: " << query.toString(),
-            c.get());
-
-    if (c->hasResultFlag(ResultFlag_ShardConfigStale)) {
-        BSONObj error;
-        c->peekError(&error);
-        uasserted(StaleConfigInfo::parseFromCommandError(error), "findN stale config");
-    }
-
-    for (int i = 0; i < nToReturn; i++) {
-        if (!c->more())
-            break;
-        out.push_back(c->nextSafe());
-    }
-}
-
-BSONObj DBClientBase::findOne(const string& ns,
-                              const Query& query,
-                              const BSONObj* fieldsToReturn,
-                              int queryOptions,
-                              boost::optional<BSONObj> readConcernObj) {
-    vector<BSONObj> v;
-    findN(v, ns, query, 1, 0, fieldsToReturn, queryOptions, readConcernObj);
-    return v.empty() ? BSONObj() : v[0];
-}
-
-std::pair<BSONObj, NamespaceString> DBClientBase::findOneByUUID(
-    const std::string& db,
-    UUID uuid,
-    const BSONObj& filter,
-    boost::optional<BSONObj> readConcernObj) {
-    list<BSONObj> results;
-    BSONObj res;
-
-    BSONObjBuilder cmdBuilder;
-    uuid.appendToBuilder(&cmdBuilder, "find");
-    cmdBuilder.append("filter", filter);
-    cmdBuilder.append("limit", 1);
-    cmdBuilder.append("singleBatch", true);
-    if (readConcernObj) {
-        cmdBuilder.append(repl::ReadConcernArgs::kReadConcernFieldName, *readConcernObj);
-    }
-
-    BSONObj cmd = cmdBuilder.obj();
-
-    if (runCommand(db, cmd, res, QueryOption_SlaveOk)) {
-        BSONObj cursorObj = res.getObjectField("cursor");
-        BSONObj docs = cursorObj.getObjectField("firstBatch");
-        BSONObjIterator it(docs);
-        while (it.more()) {
-            BSONElement e = it.next();
-            results.push_back(e.Obj().getOwned());
-        }
-        invariant(results.size() <= 1);
-        NamespaceString resNss(cursorObj["ns"].valueStringData());
-        if (results.empty()) {
-            return {BSONObj(), resNss};
-        }
-        return {results.front(), resNss};
-    }
-
-    uassertStatusOKWithContext(getStatusFromCommandResult(res),
-                               str::stream() << "find command using UUID failed. Command: " << cmd);
-    MONGO_UNREACHABLE;
-}
-
 const uint64_t DBClientBase::INVALID_SOCK_CREATION_TIME = std::numeric_limits<uint64_t>::max();
 
-unique_ptr<DBClientCursor> DBClientBase::query(const NamespaceStringOrUUID& nsOrUuid,
-                                               Query query,
-                                               int nToReturn,
-                                               int nToSkip,
-                                               const BSONObj* fieldsToReturn,
-                                               int queryOptions,
-                                               int batchSize,
-                                               boost::optional<BSONObj> readConcernObj) {
-    unique_ptr<DBClientCursor> c(new DBClientCursor(this,
-                                                    nsOrUuid,
-                                                    query.obj,
-                                                    nToReturn,
-                                                    nToSkip,
-                                                    fieldsToReturn,
-                                                    queryOptions,
-                                                    batchSize,
-                                                    readConcernObj));
-    if (c->init())
-        return c;
+std::unique_ptr<DBClientCursor> DBClientBase::find(FindCommandRequest findRequest,
+                                                   const ReadPreferenceSetting& readPref,
+                                                   ExhaustMode exhaustMode) {
+    auto cursor = std::make_unique<DBClientCursor>(
+        this, std::move(findRequest), readPref, exhaustMode == ExhaustMode::kOn);
+    if (cursor->init()) {
+        return cursor;
+    }
     return nullptr;
 }
 
-unique_ptr<DBClientCursor> DBClientBase::getMore(const string& ns,
-                                                 long long cursorId,
-                                                 int nToReturn,
-                                                 int options) {
+void DBClientBase::find(FindCommandRequest findRequest,
+                        const ReadPreferenceSetting& readPref,
+                        ExhaustMode exhaustMode,
+                        std::function<void(const BSONObj&)> callback) {
+    auto cursor = this->find(std::move(findRequest), readPref, exhaustMode);
+    while (cursor->more()) {
+        callback(cursor->nextSafe());
+    }
+}
+
+BSONObj DBClientBase::findOne(FindCommandRequest findRequest,
+                              const ReadPreferenceSetting& readPref) {
+    tassert(5951200,
+            "caller cannot provide a limit when calling DBClientBase::findOne()",
+            !findRequest.getLimit());
+    findRequest.setLimit(1);
+    auto cursor = this->find(std::move(findRequest), readPref, ExhaustMode::kOff);
+
+    uassert(5951201, "DBClientBase::findOne() could not produce cursor", cursor);
+
+    return cursor->more() ? cursor->nextSafe() : BSONObj{};
+}
+
+BSONObj DBClientBase::findOne(const NamespaceStringOrUUID& nssOrUuid, BSONObj filter) {
+    FindCommandRequest findRequest{nssOrUuid};
+    findRequest.setFilter(std::move(filter));
+    return findOne(std::move(findRequest));
+}
+
+unique_ptr<DBClientCursor> DBClientBase::getMore(const string& ns, long long cursorId) {
     unique_ptr<DBClientCursor> c(
-        new DBClientCursor(this, NamespaceString(ns), cursorId, nToReturn, options));
+        new DBClientCursor(this, NamespaceString(ns), cursorId, false /*isExhaust*/));
     if (c->init())
         return c;
     return nullptr;
 }
 
-struct DBClientFunConvertor {
-    void operator()(DBClientCursorBatchIterator& i) {
-        while (i.moreInCurrentBatch()) {
-            _f(i.nextSafe());
-        }
-    }
-    std::function<void(const BSONObj&)> _f;
-};
-
-unsigned long long DBClientBase::query(std::function<void(const BSONObj&)> f,
-                                       const NamespaceStringOrUUID& nsOrUuid,
-                                       Query query,
-                                       const BSONObj* fieldsToReturn,
-                                       int queryOptions,
-                                       int batchSize,
-                                       boost::optional<BSONObj> readConcernObj) {
-    DBClientFunConvertor fun;
-    fun._f = f;
-    std::function<void(DBClientCursorBatchIterator&)> ptr(fun);
-    return this->query(
-        ptr, nsOrUuid, query, fieldsToReturn, queryOptions, batchSize, readConcernObj);
-}
-
-unsigned long long DBClientBase::query(std::function<void(DBClientCursorBatchIterator&)> f,
-                                       const NamespaceStringOrUUID& nsOrUuid,
-                                       Query query,
-                                       const BSONObj* fieldsToReturn,
-                                       int queryOptions,
-                                       int batchSize,
-                                       boost::optional<BSONObj> readConcernObj) {
-    // mask options
-    queryOptions &= (int)(QueryOption_NoCursorTimeout | QueryOption_SlaveOk);
-
-    unique_ptr<DBClientCursor> c(this->query(
-        nsOrUuid, query, 0, 0, fieldsToReturn, queryOptions, batchSize, readConcernObj));
-    // query() throws on network error so OK to uassert with numeric code here.
-    uassert(16090, "socket error for mapping query", c.get());
-
-    unsigned long long n = 0;
-
-    while (c->more()) {
-        DBClientCursorBatchIterator i(*c);
-        f(i);
-        n += i.n();
-    }
-    return n;
-}
-
-void DBClientBase::insert(const string& ns,
-                          BSONObj obj,
-                          int flags,
-                          boost::optional<BSONObj> writeConcernObj) {
-    insert(ns, std::vector<BSONObj>{obj}, flags, writeConcernObj);
-}
-
-void DBClientBase::insert(const string& ns,
-                          const vector<BSONObj>& v,
-                          int flags,
-                          boost::optional<BSONObj> writeConcernObj) {
-    bool ordered = !(flags & InsertOption_ContinueOnError);
+namespace {
+OpMsgRequest createInsertRequest(const string& ns,
+                                 const vector<BSONObj>& v,
+                                 bool ordered,
+                                 boost::optional<BSONObj> writeConcernObj) {
     auto nss = NamespaceString(ns);
     BSONObjBuilder cmdBuilder;
     cmdBuilder.append("insert", nss.coll());
@@ -852,33 +647,15 @@ void DBClientBase::insert(const string& ns,
     auto request = OpMsgRequest::fromDBAndBody(nss.db(), cmdBuilder.obj());
     request.sequences.push_back({"documents", v});
 
-    runFireAndForgetCommand(std::move(request));
+    return request;
 }
 
-void DBClientBase::remove(const string& ns,
-                          Query obj,
-                          int flags,
-                          boost::optional<BSONObj> writeConcernObj) {
-    int limit = (flags & RemoveOption_JustOne) ? 1 : 0;
-    auto nss = NamespaceString(ns);
-
-    BSONObjBuilder cmdBuilder;
-    cmdBuilder.append("delete", nss.coll());
-    if (writeConcernObj) {
-        cmdBuilder.append(WriteConcernOptions::kWriteConcernField, *writeConcernObj);
-    }
-    auto request = OpMsgRequest::fromDBAndBody(nss.db(), cmdBuilder.obj());
-    request.sequences.push_back({"deletes", {BSON("q" << obj.obj << "limit" << limit)}});
-
-    runFireAndForgetCommand(std::move(request));
-}
-
-void DBClientBase::update(const string& ns,
-                          Query query,
-                          BSONObj obj,
-                          bool upsert,
-                          bool multi,
-                          boost::optional<BSONObj> writeConcernObj) {
+OpMsgRequest createUpdateRequest(const string& ns,
+                                 const BSONObj& filter,
+                                 BSONObj updateSpec,
+                                 bool upsert,
+                                 bool multi,
+                                 boost::optional<BSONObj> writeConcernObj) {
     auto nss = NamespaceString(ns);
 
     BSONObjBuilder cmdBuilder;
@@ -889,27 +666,95 @@ void DBClientBase::update(const string& ns,
     auto request = OpMsgRequest::fromDBAndBody(nss.db(), cmdBuilder.obj());
     request.sequences.push_back(
         {"updates",
-         {BSON("q" << query.obj << "u" << obj << "upsert" << upsert << "multi" << multi)}});
+         {BSON("q" << filter << "u" << updateSpec << "upsert" << upsert << "multi" << multi)}});
 
+    return request;
+}
+
+OpMsgRequest createRemoveRequest(const string& ns,
+                                 const BSONObj& filter,
+                                 bool removeMany,
+                                 boost::optional<BSONObj> writeConcernObj) {
+    const int limit = removeMany ? 0 : 1;
+    auto nss = NamespaceString(ns);
+
+    BSONObjBuilder cmdBuilder;
+    cmdBuilder.append("delete", nss.coll());
+    if (writeConcernObj) {
+        cmdBuilder.append(WriteConcernOptions::kWriteConcernField, *writeConcernObj);
+    }
+    auto request = OpMsgRequest::fromDBAndBody(nss.db(), cmdBuilder.obj());
+    request.sequences.push_back({"deletes", {BSON("q" << filter << "limit" << limit)}});
+
+    return request;
+}
+}  // namespace
+
+BSONObj DBClientBase::insertAcknowledged(const string& ns,
+                                         const vector<BSONObj>& v,
+                                         bool ordered,
+                                         boost::optional<BSONObj> writeConcernObj) {
+    OpMsgRequest request = createInsertRequest(ns, v, ordered, writeConcernObj);
+    rpc::UniqueReply reply = runCommand(std::move(request));
+    return reply->getCommandReply();
+}
+
+void DBClientBase::insert(const string& ns,
+                          BSONObj obj,
+                          bool ordered,
+                          boost::optional<BSONObj> writeConcernObj) {
+    insert(ns, std::vector<BSONObj>{obj}, ordered, writeConcernObj);
+}
+
+void DBClientBase::insert(const string& ns,
+                          const vector<BSONObj>& v,
+                          bool ordered,
+                          boost::optional<BSONObj> writeConcernObj) {
+    auto request = createInsertRequest(ns, v, ordered, writeConcernObj);
     runFireAndForgetCommand(std::move(request));
 }
 
-void DBClientBase::update(const string& ns,
-                          Query query,
-                          BSONObj obj,
-                          int flags,
+BSONObj DBClientBase::removeAcknowledged(const string& ns,
+                                         const BSONObj& filter,
+                                         bool removeMany,
+                                         boost::optional<BSONObj> writeConcernObj) {
+    OpMsgRequest request = createRemoveRequest(ns, filter, removeMany, writeConcernObj);
+    rpc::UniqueReply reply = runCommand(std::move(request));
+    return reply->getCommandReply();
+}
+
+void DBClientBase::remove(const string& ns,
+                          const BSONObj& filter,
+                          bool removeMany,
                           boost::optional<BSONObj> writeConcernObj) {
-    update(ns,
-           std::move(query),
-           std::move(obj),
-           flags & UpdateOption_Upsert,
-           flags & UpdateOption_Multi,
-           writeConcernObj);
+    auto request = createRemoveRequest(ns, filter, removeMany, writeConcernObj);
+    runFireAndForgetCommand(std::move(request));
+}
+
+BSONObj DBClientBase::updateAcknowledged(const string& ns,
+                                         const BSONObj& filter,
+                                         BSONObj updateSpec,
+                                         bool upsert,
+                                         bool multi,
+                                         boost::optional<BSONObj> writeConcernObj) {
+    auto request = createUpdateRequest(ns, filter, updateSpec, upsert, multi, writeConcernObj);
+    rpc::UniqueReply reply = runCommand(std::move(request));
+    return reply->getCommandReply();
+}
+
+void DBClientBase::update(const string& ns,
+                          const BSONObj& filter,
+                          BSONObj updateSpec,
+                          bool upsert,
+                          bool multi,
+                          boost::optional<BSONObj> writeConcernObj) {
+    auto request = createUpdateRequest(ns, filter, updateSpec, upsert, multi, writeConcernObj);
+    runFireAndForgetCommand(std::move(request));
 }
 
 void DBClientBase::killCursor(const NamespaceString& ns, long long cursorId) {
-    runFireAndForgetCommand(
-        OpMsgRequest::fromDBAndBody(ns.db(), KillCursorsRequest(ns, {cursorId}).toBSON()));
+    runFireAndForgetCommand(OpMsgRequest::fromDBAndBody(
+        ns.db(), KillCursorsCommandRequest(ns, {cursorId}).toBSON(BSONObj{})));
 }
 
 namespace {
@@ -941,20 +786,6 @@ std::list<BSONObj> DBClientBase::getIndexSpecs(const NamespaceStringOrUUID& nsOr
     return _getIndexSpecs(nsOrUuid, makeListIndexesCommand(nsOrUuid, includeBuildUUIDs), options);
 }
 
-std::list<BSONObj> DBClientBase::getReadyIndexSpecs(const NamespaceStringOrUUID& nsOrUuid,
-                                                    int options) {
-    auto specsWithBuildUUIDs =
-        _getIndexSpecs(nsOrUuid, makeListIndexesCommand(nsOrUuid, true), options);
-    list<BSONObj> specs;
-    for (const auto& spec : specsWithBuildUUIDs) {
-        if (spec["buildUUID"]) {
-            continue;
-        }
-        specs.push_back(spec);
-    }
-    return specs;
-}
-
 std::list<BSONObj> DBClientBase::_getIndexSpecs(const NamespaceStringOrUUID& nsOrUuid,
                                                 const BSONObj& cmd,
                                                 int options) {
@@ -968,6 +799,10 @@ std::list<BSONObj> DBClientBase::_getIndexSpecs(const NamespaceStringOrUUID& nsO
             specs.push_back(i.next().Obj().getOwned());
         }
 
+        if (res.hasField(LogicalTime::kOperationTimeFieldName)) {
+            setOperationTime(LogicalTime::fromOperationTime(res).asTimestamp());
+        }
+
         const long long id = cursorObj["id"].Long();
 
         if (id != 0) {
@@ -975,10 +810,14 @@ std::list<BSONObj> DBClientBase::_getIndexSpecs(const NamespaceStringOrUUID& nsO
             if (nsOrUuid.nss()) {
                 invariant((*nsOrUuid.nss()).toString() == cursorNs);
             }
-            unique_ptr<DBClientCursor> cursor = getMore(cursorNs, id, 0, 0);
+            unique_ptr<DBClientCursor> cursor = getMore(cursorNs, id);
             while (cursor->more()) {
                 specs.push_back(cursor->nextSafe().getOwned());
             }
+
+            if (cursor->getOperationTime()) {
+                setOperationTime(*(cursor->getOperationTime()));
+            }
         }
 
         return specs;
@@ -1136,5 +975,12 @@ string nsGetCollection(const string& ns) {
     return ns.substr(pos + 1);
 }
 
+Timestamp DBClientBase::getOperationTime() {
+    return _lastOperationTime;
+}
+
+void DBClientBase::setOperationTime(Timestamp operationTime) {
+    _lastOperationTime = operationTime;
+}
 
 }  // namespace mongo
