@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/client/dbclient_connection.h"
 #include "mongo/client/dbclient_cursor.h"
@@ -36,6 +35,9 @@
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/assert_util.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
+
 namespace mongo {
 namespace {
 
@@ -49,31 +51,9 @@ namespace {
 class DBClientConnectionForTest : public DBClientConnection {
 public:
     DBClientConnectionForTest() {
-        _setServerRPCProtocols(rpc::supports::kAll);       // allow all protocol types by default.
         _serverAddress = HostAndPort("localhost", 27017);  // dummy server address.
     }
 
-    bool call(Message& toSend,
-              Message& response,
-              bool assertOk,
-              std::string* actualServer) override {
-
-        // Intercept request.
-        const auto reqId = nextMessageId();
-        toSend.header().setId(reqId);
-        toSend.header().setResponseToMsgId(0);
-        OpMsg::appendChecksum(&toSend);
-        _lastSent = toSend;
-
-        // Mock response.
-        response = _mockCallResponse;
-        response.header().setId(nextMessageId());
-        response.header().setResponseToMsgId(reqId);
-        OpMsg::appendChecksum(&response);
-
-        return true;
-    }
-
     Status recv(Message& m, int lastRequestId) override {
         m = _mockRecvResponse;
         return Status::OK();
@@ -84,10 +64,6 @@ public:
         LOGV2(20131, "Killing cursor in DBClientConnectionForTest");
     }
 
-    void setSupportedProtocols(rpc::ProtocolSet protocols) {
-        _setServerRPCProtocols(protocols);
-    }
-
     void setCallResponse(Message reply) {
         _mockCallResponse = reply;
     }
@@ -105,6 +81,22 @@ public:
     }
 
 private:
+    void _call(Message& toSend, Message& response, std::string* actualServer) override {
+
+        // Intercept request.
+        const auto reqId = nextMessageId();
+        toSend.header().setId(reqId);
+        toSend.header().setResponseToMsgId(0);
+        OpMsg::appendChecksum(&toSend);
+        _lastSent = toSend;
+
+        // Mock response.
+        response = _mockCallResponse;
+        response.header().setId(nextMessageId());
+        response.header().setResponseToMsgId(reqId);
+        OpMsg::appendChecksum(&response);
+    }
+
     Message _mockCallResponse;
     Message _mockRecvResponse;
     Message _lastSent;
@@ -153,7 +145,8 @@ TEST_F(DBClientCursorTest, DBClientCursorCallsMetaDataReaderOncePerBatch) {
     // Set up the DBClientCursor and a mock client connection.
     DBClientConnectionForTest conn;
     const NamespaceString nss("test", "coll");
-    DBClientCursor cursor(&conn, NamespaceStringOrUUID(nss), Query().obj, 0, 0, nullptr, 0, 0);
+    FindCommandRequest findCmd{nss};
+    DBClientCursor cursor(&conn, findCmd, ReadPreferenceSetting{}, false);
     cursor.setBatchSize(2);
 
     // Set up mock 'find' response.
@@ -199,8 +192,8 @@ TEST_F(DBClientCursorTest, DBClientCursorHandlesOpMsgExhaustCorrectly) {
     // Set up the DBClientCursor and a mock client connection.
     DBClientConnectionForTest conn;
     const NamespaceString nss("test", "coll");
-    DBClientCursor cursor(
-        &conn, NamespaceStringOrUUID(nss), Query().obj, 0, 0, nullptr, QueryOption_Exhaust, 0);
+    FindCommandRequest findCmd{nss};
+    DBClientCursor cursor(&conn, findCmd, ReadPreferenceSetting{}, true /*isExhaust*/);
     cursor.setBatchSize(0);
 
     // Set up mock 'find' response.
@@ -263,8 +256,8 @@ TEST_F(DBClientCursorTest, DBClientCursorResendsGetMoreIfMoreToComeFlagIsOmitted
     // Set up the DBClientCursor and a mock client connection.
     DBClientConnectionForTest conn;
     const NamespaceString nss("test", "coll");
-    DBClientCursor cursor(
-        &conn, NamespaceStringOrUUID(nss), Query().obj, 0, 0, nullptr, QueryOption_Exhaust, 0);
+    FindCommandRequest findCmd{nss};
+    DBClientCursor cursor(&conn, findCmd, ReadPreferenceSetting{}, true /*isExhaust*/);
     cursor.setBatchSize(0);
 
     // Set up mock 'find' response.
@@ -344,12 +337,11 @@ TEST_F(DBClientCursorTest, DBClientCursorResendsGetMoreIfMoreToComeFlagIsOmitted
 }
 
 TEST_F(DBClientCursorTest, DBClientCursorMoreThrowsExceptionOnNonOKResponse) {
-
     // Set up the DBClientCursor and a mock client connection.
     DBClientConnectionForTest conn;
     const NamespaceString nss("test", "coll");
-    DBClientCursor cursor(
-        &conn, NamespaceStringOrUUID(nss), Query().obj, 0, 0, nullptr, QueryOption_Exhaust, 0);
+    FindCommandRequest findCmd{nss};
+    DBClientCursor cursor(&conn, findCmd, ReadPreferenceSetting{}, true /*isExhaust*/);
     cursor.setBatchSize(0);
 
     // Set up mock 'find' response.
@@ -376,12 +368,11 @@ TEST_F(DBClientCursorTest, DBClientCursorMoreThrowsExceptionOnNonOKResponse) {
 }
 
 TEST_F(DBClientCursorTest, DBClientCursorMoreThrowsExceptionWhenMoreToComeFlagSetWithZeroCursorId) {
-
     // Set up the DBClientCursor and a mock client connection.
     DBClientConnectionForTest conn;
     const NamespaceString nss("test", "coll");
-    DBClientCursor cursor(
-        &conn, NamespaceStringOrUUID(nss), Query().obj, 0, 0, nullptr, QueryOption_Exhaust, 0);
+    FindCommandRequest findCmd{nss};
+    DBClientCursor cursor(&conn, findCmd, ReadPreferenceSetting{}, true /*isExhaust*/);
     cursor.setBatchSize(0);
 
     // Set up mock 'find' response.
@@ -409,67 +400,13 @@ TEST_F(DBClientCursorTest, DBClientCursorMoreThrowsExceptionWhenMoreToComeFlagSe
     ASSERT_THROWS_CODE(cursor.more(), DBException, 50935);
 }
 
-TEST_F(DBClientCursorTest, DBClientCursorIgnoresExhaustForOpQueryMessages) {
-    // Set up the DBClientCursor and a mock client connection. If we set the server RPC protocol to
-    // OpQuery, then when we assemble a command request in DBClientCursor, we will make a command
-    // style OpQuery request, as opposed to an OpMsg request. We want to make sure that for command
-    // style OpQuery requests, we ignore the exhaust option, and that cursor queries work normally.
-    DBClientConnectionForTest conn;
-    conn.setSupportedProtocols(rpc::supports::kOpQueryOnly);
-
-    const NamespaceString nss("test", "coll");
-    DBClientCursor cursor(
-        &conn, NamespaceStringOrUUID(nss), Query().obj, 0, 0, nullptr, QueryOption_Exhaust, 0);
-    cursor.setBatchSize(0);
-
-    // Set up mock 'find' response.
-    const long long cursorId = 42;
-    Message findResponseMsg = mockFindResponse(nss, cursorId, {});
-
-    conn.setCallResponse(findResponseMsg);
-    ASSERT(cursor.init());
-
-    // Verify that the initial 'find' request was sent.
-    auto m = conn.getLastSentMessage();
-    ASSERT(!m.empty());
-    QueryMessage queryMsg(m);
-    ASSERT_EQ(queryMsg.query.getStringField("find"), nss.coll());
-    ASSERT_EQ(queryMsg.query["batchSize"].number(), 0);
-    ASSERT_EQ(0, queryMsg.queryOptions);
-
-    // Create and set a non-exhaust getMore response.
-    cursor.setBatchSize(2);
-    auto getMoreResponseMsg = mockGetMoreResponse(nss, cursorId, {docObj(1), docObj(2)});
-    conn.setCallResponse(getMoreResponseMsg);
-
-    // Trigger another 'getMore' request.
-    conn.clearLastSentMessage();
-    ASSERT(cursor.more());
-
-    // Make sure the sent request has no exhaust query options set.
-    m = conn.getLastSentMessage();
-    ASSERT(!m.empty());
-    queryMsg = QueryMessage(m);
-    ASSERT_EQ(queryMsg.query["getMore"].number(), cursorId);
-    ASSERT_EQ(queryMsg.query["collection"].str(), nss.coll());
-    ASSERT_EQ(0, queryMsg.queryOptions);
-    ASSERT_BSONOBJ_EQ(docObj(1), cursor.next());
-    ASSERT_BSONOBJ_EQ(docObj(2), cursor.next());
-}
-
 TEST_F(DBClientCursorTest, DBClientCursorPassesReadOnceFlag) {
-
     // Set up the DBClientCursor and a mock client connection.
     DBClientConnectionForTest conn;
     const NamespaceString nss("test", "coll");
-    DBClientCursor cursor(&conn,
-                          NamespaceStringOrUUID(nss),
-                          QUERY("query" << BSONObj() << "$readOnce" << true).obj,
-                          0,
-                          0,
-                          nullptr,
-                          /*QueryOption*/ 0,
-                          0);
+    FindCommandRequest findCmd{nss};
+    findCmd.setReadOnce(true);
+    DBClientCursor cursor(&conn, findCmd, ReadPreferenceSetting{}, false);
     cursor.setBatchSize(0);
 
     // Set up mock 'find' response.
@@ -494,16 +431,10 @@ TEST_F(DBClientCursorTest, DBClientCursorPassesResumeFields) {
     // Set up the DBClientCursor and a mock client connection.
     DBClientConnectionForTest conn;
     const NamespaceString nss("test", "coll");
-    DBClientCursor cursor(&conn,
-                          NamespaceStringOrUUID(nss),
-                          QUERY("query" << BSONObj() << "$_requestResumeToken" << true
-                                        << "$_resumeAfter" << BSON("$recordId" << 5LL))
-                              .obj,
-                          0,
-                          0,
-                          nullptr,
-                          /*QueryOption*/ 0,
-                          0);
+    FindCommandRequest findCmd{nss};
+    findCmd.setRequestResumeToken(true);
+    findCmd.setResumeAfter(BSON("$recordId" << 5LL));
+    DBClientCursor cursor(&conn, findCmd, ReadPreferenceSetting{}, false);
     cursor.setBatchSize(0);
 
     // Set up mock 'find' response.
@@ -536,14 +467,9 @@ TEST_F(DBClientCursorTest, DBClientCursorTailable) {
     // Set up the DBClientCursor and a mock client connection.
     DBClientConnectionForTest conn;
     const NamespaceString nss("test", "coll");
-    DBClientCursor cursor(&conn,
-                          NamespaceStringOrUUID(nss),
-                          Query().obj,
-                          0,
-                          0,
-                          nullptr,
-                          QueryOption_CursorTailable,
-                          0);
+    FindCommandRequest findCmd{nss};
+    findCmd.setTailable(true);
+    DBClientCursor cursor(&conn, findCmd, ReadPreferenceSetting{}, false);
     cursor.setBatchSize(0);
 
     // Set up mock 'find' response.
@@ -635,14 +561,10 @@ TEST_F(DBClientCursorTest, DBClientCursorTailableAwaitData) {
     // Set up the DBClientCursor and a mock client connection.
     DBClientConnectionForTest conn;
     const NamespaceString nss("test", "coll");
-    DBClientCursor cursor(&conn,
-                          NamespaceStringOrUUID(nss),
-                          Query().obj,
-                          0,
-                          0,
-                          nullptr,
-                          QueryOption_CursorTailable | QueryOption_AwaitData,
-                          0);
+    FindCommandRequest findCmd{nss};
+    findCmd.setTailable(true);
+    findCmd.setAwaitData(true);
+    DBClientCursor cursor(&conn, findCmd, ReadPreferenceSetting{}, false);
     cursor.setBatchSize(0);
 
     // Set up mock 'find' response.
@@ -701,14 +623,10 @@ TEST_F(DBClientCursorTest, DBClientCursorTailableAwaitDataExhaust) {
     // Set up the DBClientCursor and a mock client connection.
     DBClientConnectionForTest conn;
     const NamespaceString nss("test", "coll");
-    DBClientCursor cursor(&conn,
-                          NamespaceStringOrUUID(nss),
-                          Query().obj,
-                          0,
-                          0,
-                          nullptr,
-                          QueryOption_CursorTailable | QueryOption_AwaitData | QueryOption_Exhaust,
-                          0);
+    FindCommandRequest findCmd{nss};
+    findCmd.setTailable(true);
+    findCmd.setAwaitData(true);
+    DBClientCursor cursor(&conn, findCmd, ReadPreferenceSetting{}, true /*isExhaust*/);
     cursor.setBatchSize(0);
 
     // Set up mock 'find' response.
@@ -841,8 +759,8 @@ TEST_F(DBClientCursorTest, DBClientCursorTailableAwaitDataExhaust) {
 
 TEST_F(DBClientCursorTest, DBClientCursorOplogQuery) {
     // This tests DBClientCursor supports oplog query with special fields in the command request.
-    // 1. Initial find command has "filter", "tailable", "awaitData", "oplogReplay", "maxTimeMS",
-    //    "batchSize", "term" and "readConcern" fields set.
+    // 1. Initial find command has "filter", "tailable", "awaitData", "maxTimeMS", "batchSize",
+    //    "term" and "readConcern" fields set.
     // 2. A subsequent getMore command sets awaitData timeout and lastKnownCommittedOpTime
     //    correctly.
 
@@ -853,18 +771,16 @@ TEST_F(DBClientCursorTest, DBClientCursorOplogQuery) {
     const BSONObj readConcernObj = BSON("afterClusterTime" << Timestamp(0, 1));
     const long long maxTimeMS = 5000LL;
     const long long term = 5;
-    const auto oplogQuery = QUERY("query" << filterObj << "readConcern" << readConcernObj
-                                          << "$maxTimeMS" << maxTimeMS << "term" << term);
 
-    DBClientCursor cursor(&conn,
-                          NamespaceStringOrUUID(nss),
-                          oplogQuery.obj,
-                          0,
-                          0,
-                          nullptr,
-                          QueryOption_CursorTailable | QueryOption_AwaitData |
-                              QueryOption_OplogReplay,
-                          0);
+    FindCommandRequest findCmd{nss};
+    findCmd.setFilter(filterObj);
+    findCmd.setReadConcern(readConcernObj);
+    findCmd.setMaxTimeMS(maxTimeMS);
+    findCmd.setTerm(term);
+    findCmd.setTailable(true);
+    findCmd.setAwaitData(true);
+
+    DBClientCursor cursor(&conn, findCmd, ReadPreferenceSetting{}, false);
     cursor.setBatchSize(0);
 
     // Set up mock 'find' response.
@@ -884,7 +800,6 @@ TEST_F(DBClientCursorTest, DBClientCursorOplogQuery) {
     ASSERT_BSONOBJ_EQ(msg.body["filter"].Obj(), filterObj);
     ASSERT_TRUE(msg.body.getBoolField("tailable")) << msg.body;
     ASSERT_TRUE(msg.body.getBoolField("awaitData")) << msg.body;
-    ASSERT_TRUE(msg.body.getBoolField("oplogReplay")) << msg.body;
     ASSERT_EQ(msg.body["maxTimeMS"].numberLong(), maxTimeMS) << msg.body;
     ASSERT_EQ(msg.body["batchSize"].number(), 0) << msg.body;
     ASSERT_EQ(msg.body["term"].numberLong(), term) << msg.body;
