@@ -45,12 +45,15 @@
 #include "mongo/client/dbclient_rs.h"
 #include "mongo/client/replica_set_monitor.h"
 #include "mongo/client/replica_set_monitor_protocol_test_util.h"
+#include "mongo/client/streamable_replica_set_monitor_for_testing.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/dbtests/mock/mock_conn_registry.h"
 #include "mongo/dbtests/mock/mock_replica_set.h"
+#include "mongo/idl/server_parameter_test_util.h"
 #include "mongo/stdx/unordered_set.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source_mock.h"
 
 namespace mongo {
 namespace {
@@ -62,11 +65,6 @@ using std::string;
 using std::unique_ptr;
 using std::vector;
 
-MONGO_INITIALIZER(DisableReplicaSetMonitorRefreshRetries)(InitializerContext*) {
-    ReplicaSetMonitor::disableRefreshRetries_forTest();
-    return Status::OK();
-}
-
 /**
  * Constructs a metadata object containing the passed server selection metadata.
  */
@@ -75,20 +73,30 @@ BSONObj makeMetadata(ReadPreference rp, TagSet tagSet) {
 }
 
 /**
- * Ensures a global ServiceContext exists and the ScanningReplicaSetMonitor is used for each test.
+ * Ensures a global ServiceContext exists.
  */
 class DBClientRSTest : public unittest::Test {
+public:
+    ClockSource* clock() {
+        return _clkSource.get();
+    }
+
+    sdam::MockTopologyManager* getTopologyManager() {
+        return _rsmMonitor.getTopologyManager();
+    }
+
 protected:
     void setUp() {
         auto serviceContext = ServiceContext::make();
         setGlobalServiceContext(std::move(serviceContext));
-
-        ReplicaSetMonitorProtocolTestUtil::setRSMProtocol(ReplicaSetMonitorProtocol::kScanning);
     }
 
-    void tearDown() {
-        ReplicaSetMonitorProtocolTestUtil::resetRSMProtocol();
-    }
+    std::shared_ptr<ClockSourceMock> _clkSource = std::make_shared<ClockSourceMock>();
+    StreamableReplicaSetMonitorForTesting _rsmMonitor;
+
+private:
+    RAIIServerParameterControllerForTest _findHostTimeout{"defaultFindReplicaSetHostTimeoutMS",
+                                                          100};
 };
 
 /**
@@ -101,6 +109,9 @@ protected:
         ReplicaSetMonitor::cleanup();
 
         _replSet.reset(new MockReplicaSet("test", 2));
+        _rsmMonitor.setup(_replSet->getURI());
+        getTopologyManager()->setTopologyDescription(_replSet->getTopologyDescription(clock()));
+
         ConnectionString::setConnectionHook(mongo::MockConnRegistry::get()->getConnStrHook());
     }
 
@@ -125,7 +136,6 @@ void assertOneOfNodesSelected(MockReplicaSet* replSet,
                               ReadPreference rp,
                               const std::vector<std::string> hostNames) {
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
-    ReplicaSetMonitor::get(replSet->getSetName())->runScanForMockReplicaSet();
     bool secondaryOk = (rp != ReadPreference::PrimaryOnly);
     auto tagSet = secondaryOk ? TagSet() : TagSet::primaryOnly();
     // We need the command to be a "SecOk command"
@@ -146,11 +156,10 @@ TEST_F(BasicRS, QueryPrimary) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::PrimaryOnly, BSONArray());
-
     // Note: IdentityNS contains the name of the server.
-    unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    auto cursor =
+        replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::PrimaryOnly});
     BSONObj doc = cursor->next();
     ASSERT_EQUALS(replSet->getPrimary(), doc[HostField.name()].str());
 }
@@ -163,11 +172,10 @@ TEST_F(BasicRS, QuerySecondaryOnly) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::SecondaryOnly, BSONArray());
-
     // Note: IdentityNS contains the name of the server.
-    unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    auto cursor =
+        replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::SecondaryOnly});
     BSONObj doc = cursor->next();
     ASSERT_EQUALS(replSet->getSecondaries().front(), doc[HostField.name()].str());
 }
@@ -181,14 +189,10 @@ TEST_F(BasicRS, QueryPrimaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    // Need up-to-date view, since either host is valid if view is stale.
-    ReplicaSetMonitor::get(replSet->getSetName())->runScanForMockReplicaSet();
-
-    Query query;
-    query.readPref(mongo::ReadPreference::PrimaryPreferred, BSONArray());
-
     // Note: IdentityNS contains the name of the server.
-    unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    auto cursor =
+        replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::PrimaryPreferred});
     BSONObj doc = cursor->next();
     ASSERT_EQUALS(replSet->getPrimary(), doc[HostField.name()].str());
 }
@@ -201,14 +205,10 @@ TEST_F(BasicRS, QuerySecondaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    // Need up-to-date view, since either host is valid if view is stale.
-    ReplicaSetMonitor::get(replSet->getSetName())->runScanForMockReplicaSet();
-
-    Query query;
-    query.readPref(mongo::ReadPreference::SecondaryPreferred, BSONArray());
-
     // Note: IdentityNS contains the name of the server.
-    unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    auto cursor = replConn.find(std::move(findCmd),
+                                ReadPreferenceSetting{ReadPreference::SecondaryPreferred});
     BSONObj doc = cursor->next();
     ASSERT_EQUALS(replSet->getSecondaries().front(), doc[HostField.name()].str());
 }
@@ -235,6 +235,9 @@ protected:
              ++iter) {
             _replSet->kill(iter->toString());
         }
+
+        _rsmMonitor.setup(_replSet->getURI());
+        getTopologyManager()->setTopologyDescription(_replSet->getTopologyDescription(clock()));
     }
 
     void tearDown() {
@@ -267,9 +270,10 @@ TEST_F(AllNodesDown, QueryPrimary) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::PrimaryOnly, BSONArray());
-    ASSERT_THROWS(replConn.query(NamespaceString(IdentityNS), query), AssertionException);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    ASSERT_THROWS(
+        replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::PrimaryOnly}),
+        AssertionException);
 }
 
 TEST_F(AllNodesDown, CommandPrimary) {
@@ -280,9 +284,10 @@ TEST_F(AllNodesDown, QuerySecondaryOnly) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::SecondaryOnly, BSONArray());
-    ASSERT_THROWS(replConn.query(NamespaceString(IdentityNS), query), AssertionException);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    ASSERT_THROWS(
+        replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::SecondaryOnly}),
+        AssertionException);
 }
 
 TEST_F(AllNodesDown, CommandSecondaryOnly) {
@@ -293,9 +298,10 @@ TEST_F(AllNodesDown, QueryPrimaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::PrimaryPreferred, BSONArray());
-    ASSERT_THROWS(replConn.query(NamespaceString(IdentityNS), query), AssertionException);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    ASSERT_THROWS(
+        replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::PrimaryPreferred}),
+        AssertionException);
 }
 
 TEST_F(AllNodesDown, CommandPrimaryPreferred) {
@@ -306,9 +312,10 @@ TEST_F(AllNodesDown, QuerySecondaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::SecondaryPreferred, BSONArray());
-    ASSERT_THROWS(replConn.query(NamespaceString(IdentityNS), query), AssertionException);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    ASSERT_THROWS(replConn.find(std::move(findCmd),
+                                ReadPreferenceSetting{ReadPreference::SecondaryPreferred}),
+                  AssertionException);
 }
 
 TEST_F(AllNodesDown, CommandSecondaryPreferred) {
@@ -319,9 +326,9 @@ TEST_F(AllNodesDown, QueryNearest) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::Nearest, BSONArray());
-    ASSERT_THROWS(replConn.query(NamespaceString(IdentityNS), query), AssertionException);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    ASSERT_THROWS(replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::Nearest}),
+                  AssertionException);
 }
 
 TEST_F(AllNodesDown, CommandNearest) {
@@ -340,6 +347,9 @@ protected:
         _replSet.reset(new MockReplicaSet("test", 2));
         ConnectionString::setConnectionHook(mongo::MockConnRegistry::get()->getConnStrHook());
         _replSet->kill(_replSet->getPrimary());
+
+        _rsmMonitor.setup(_replSet->getURI());
+        getTopologyManager()->setTopologyDescription(_replSet->getTopologyDescription(clock()));
     }
 
     void tearDown() {
@@ -362,9 +372,10 @@ TEST_F(PrimaryDown, QueryPrimary) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::PrimaryOnly, BSONArray());
-    ASSERT_THROWS(replConn.query(NamespaceString(IdentityNS), query), AssertionException);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    ASSERT_THROWS(
+        replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::PrimaryOnly}),
+        AssertionException);
 }
 
 TEST_F(PrimaryDown, CommandPrimary) {
@@ -375,11 +386,10 @@ TEST_F(PrimaryDown, QuerySecondaryOnly) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::SecondaryOnly, BSONArray());
-
     // Note: IdentityNS contains the name of the server.
-    unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    auto cursor =
+        replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::SecondaryOnly});
     BSONObj doc = cursor->next();
     ASSERT_EQUALS(replSet->getSecondaries().front(), doc[HostField.name()].str());
 }
@@ -393,11 +403,10 @@ TEST_F(PrimaryDown, QueryPrimaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::PrimaryPreferred, BSONArray());
-
     // Note: IdentityNS contains the name of the server.
-    unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    auto cursor =
+        replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::PrimaryPreferred});
     BSONObj doc = cursor->next();
     ASSERT_EQUALS(replSet->getSecondaries().front(), doc[HostField.name()].str());
 }
@@ -411,11 +420,10 @@ TEST_F(PrimaryDown, QuerySecondaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::SecondaryPreferred, BSONArray());
-
     // Note: IdentityNS contains the name of the server.
-    unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    auto cursor = replConn.find(std::move(findCmd),
+                                ReadPreferenceSetting{ReadPreference::SecondaryPreferred});
     BSONObj doc = cursor->next();
     ASSERT_EQUALS(replSet->getSecondaries().front(), doc[HostField.name()].str());
 }
@@ -429,9 +437,8 @@ TEST_F(PrimaryDown, Nearest) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::Nearest, BSONArray());
-    unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    auto cursor = replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::Nearest});
     BSONObj doc = cursor->next();
     ASSERT_EQUALS(replSet->getSecondaries().front(), doc[HostField.name()].str());
 }
@@ -449,6 +456,9 @@ protected:
         ConnectionString::setConnectionHook(mongo::MockConnRegistry::get()->getConnStrHook());
 
         _replSet->kill(_replSet->getSecondaries().front());
+
+        _rsmMonitor.setup(_replSet->getURI());
+        getTopologyManager()->setTopologyDescription(_replSet->getTopologyDescription(clock()));
     }
 
     void tearDown() {
@@ -471,11 +481,10 @@ TEST_F(SecondaryDown, QueryPrimary) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::PrimaryOnly, BSONArray());
-
     // Note: IdentityNS contains the name of the server.
-    unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    auto cursor =
+        replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::PrimaryOnly});
     BSONObj doc = cursor->next();
     ASSERT_EQUALS(replSet->getPrimary(), doc[HostField.name()].str());
 }
@@ -488,9 +497,10 @@ TEST_F(SecondaryDown, QuerySecondaryOnly) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::SecondaryOnly, BSONArray());
-    ASSERT_THROWS(replConn.query(NamespaceString(IdentityNS), query), AssertionException);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    ASSERT_THROWS(
+        replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::SecondaryOnly}),
+        AssertionException);
 }
 
 TEST_F(SecondaryDown, CommandSecondaryOnly) {
@@ -501,11 +511,10 @@ TEST_F(SecondaryDown, QueryPrimaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::PrimaryPreferred, BSONArray());
-
     // Note: IdentityNS contains the name of the server.
-    unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    auto cursor =
+        replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::PrimaryPreferred});
     BSONObj doc = cursor->next();
     ASSERT_EQUALS(replSet->getPrimary(), doc[HostField.name()].str());
 }
@@ -518,11 +527,9 @@ TEST_F(SecondaryDown, QuerySecondaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::SecondaryPreferred, BSONArray());
-
-    // Note: IdentityNS contains the name of the server.
-    unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    auto cursor = replConn.find(std::move(findCmd),
+                                ReadPreferenceSetting{ReadPreference::SecondaryPreferred});
     BSONObj doc = cursor->next();
     ASSERT_EQUALS(replSet->getPrimary(), doc[HostField.name()].str());
 }
@@ -535,11 +542,8 @@ TEST_F(SecondaryDown, QueryNearest) {
     MockReplicaSet* replSet = getReplSet();
     DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
-    Query query;
-    query.readPref(mongo::ReadPreference::Nearest, BSONArray());
-
-    // Note: IdentityNS contains the name of the server.
-    unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+    FindCommandRequest findCmd{NamespaceString{IdentityNS}};
+    auto cursor = replConn.find(std::move(findCmd), ReadPreferenceSetting{ReadPreference::Nearest});
     BSONObj doc = cursor->next();
     ASSERT_EQUALS(replSet->getPrimary(), doc[HostField.name()].str());
 }
@@ -557,9 +561,6 @@ protected:
     void setUp() {
         DBClientRSTest::setUp();
 
-        // Tests for pinning behavior require this.
-        ReplicaSetMonitor::useDeterministicHostSelection = true;
-
         // This shuts down the background RSMWatcher thread and prevents it from running. These
         // tests depend on controlling when the RSMs are updated.
         ReplicaSetMonitor::cleanup();
@@ -653,16 +654,16 @@ protected:
             }
 
             membersBuilder.done();
-            mongo::repl::ReplSetConfig newConfig;
-            fassert(28569, newConfig.initialize(newConfigBuilder.done()));
+            auto newConfig = mongo::repl::ReplSetConfig::parse(newConfigBuilder.done());
             fassert(28568, newConfig.validate());
             _replSet->setConfig(newConfig);
         }
+
+        _rsmMonitor.setup(_replSet->getURI());
+        getTopologyManager()->setTopologyDescription(_replSet->getTopologyDescription(clock()));
     }
 
     void tearDown() {
-        ReplicaSetMonitor::useDeterministicHostSelection = false;
-
         ConnectionString::setConnectionHook(_originalConnectionHook);
         ReplicaSetMonitor::cleanup();
         _replSet.reset();
@@ -689,19 +690,18 @@ TEST_F(TaggedFiveMemberRS, ConnShouldPinIfSameSettings) {
 
     string dest;
     {
-        Query query;
-        query.readPref(mongo::ReadPreference::PrimaryPreferred, BSONArray());
-
         // Note: IdentityNS contains the name of the server.
-        unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+        std::unique_ptr<DBClientCursor> cursor =
+            replConn.find(FindCommandRequest{NamespaceString{IdentityNS}},
+                          ReadPreferenceSetting{ReadPreference::PrimaryPreferred});
         BSONObj doc = cursor->next();
         dest = doc[HostField.name()].str();
     }
 
     {
-        Query query;
-        query.readPref(mongo::ReadPreference::PrimaryPreferred, BSONArray());
-        unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+        std::unique_ptr<DBClientCursor> cursor =
+            replConn.find(FindCommandRequest{NamespaceString{IdentityNS}},
+                          ReadPreferenceSetting{ReadPreference::PrimaryPreferred});
         BSONObj doc = cursor->next();
         const string newDest = doc[HostField.name()].str();
         ASSERT_EQUALS(dest, newDest);
@@ -717,11 +717,10 @@ TEST_F(TaggedFiveMemberRS, ConnShouldNotPinIfHostMarkedAsFailed) {
 
     string dest;
     {
-        Query query;
-        query.readPref(mongo::ReadPreference::PrimaryPreferred, BSONArray());
-
         // Note: IdentityNS contains the name of the server.
-        unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+        std::unique_ptr<DBClientCursor> cursor =
+            replConn.find(FindCommandRequest{NamespaceString{IdentityNS}},
+                          ReadPreferenceSetting{ReadPreference::PrimaryPreferred});
         BSONObj doc = cursor->next();
         dest = doc[HostField.name()].str();
     }
@@ -729,105 +728,34 @@ TEST_F(TaggedFiveMemberRS, ConnShouldNotPinIfHostMarkedAsFailed) {
     // This is the only difference from ConnShouldPinIfSameSettings which tests that we *do* pin
     // in if the host is still marked as up. Note that this only notifies the RSM, and does not
     // directly effect the DBClientRS.
-    ReplicaSetMonitor::get(replSet->getSetName())
-        ->failedHost(HostAndPort(dest), {ErrorCodes::InternalError, "Test error"});
+    getReplSet()->getNode(dest)->shutdown();
+    getTopologyManager()->setTopologyDescription(getReplSet()->getTopologyDescription(clock()));
 
     {
-        Query query;
-        query.readPref(mongo::ReadPreference::PrimaryPreferred, BSONArray());
-        unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
+        std::unique_ptr<DBClientCursor> cursor =
+            replConn.find(FindCommandRequest{NamespaceString{IdentityNS}},
+                          ReadPreferenceSetting{ReadPreference::PrimaryPreferred});
         BSONObj doc = cursor->next();
         const string newDest = doc[HostField.name()].str();
         ASSERT_NOT_EQUALS(dest, newDest);
     }
 }
 
-TEST_F(TaggedFiveMemberRS, ConnShouldNotPinIfDiffMode) {
+// Note: secondaryConn is dangerous and should be deprecated! Also see SERVER-7801.
+TEST_F(TaggedFiveMemberRS, SecondaryConnReturnsSecConn) {
     MockReplicaSet* replSet = getReplSet();
     vector<HostAndPort> seedList;
     seedList.push_back(HostAndPort(replSet->getPrimary()));
 
     DBClientReplicaSet replConn(replSet->getSetName(), seedList, StringData());
 
-    // Need up-to-date view to ensure there are multiple valid choices.
-    ReplicaSetMonitor::get(replSet->getSetName())->runScanForMockReplicaSet();
-
-    string dest;
-    {
-        Query query;
-        query.readPref(mongo::ReadPreference::SecondaryPreferred, BSONArray());
-
-        // Note: IdentityNS contains the name of the server.
-        unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
-        BSONObj doc = cursor->next();
-        dest = doc[HostField.name()].str();
-        ASSERT_NOT_EQUALS(dest, replSet->getPrimary());
-    }
-
-    {
-        Query query;
-        query.readPref(mongo::ReadPreference::SecondaryOnly, BSONArray());
-        unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
-        BSONObj doc = cursor->next();
-        const string newDest = doc[HostField.name()].str();
-        ASSERT_NOT_EQUALS(dest, newDest);
-    }
-}
-
-TEST_F(TaggedFiveMemberRS, ConnShouldNotPinIfDiffTag) {
-    MockReplicaSet* replSet = getReplSet();
-    vector<HostAndPort> seedList;
-    seedList.push_back(HostAndPort(replSet->getPrimary()));
-
-    DBClientReplicaSet replConn(replSet->getSetName(), seedList, StringData());
-
-    // Need up-to-date view to ensure there are multiple valid choices.
-    ReplicaSetMonitor::get(replSet->getSetName())->runScanForMockReplicaSet();
-
-    string dest;
-    {
-        Query query;
-        query.readPref(mongo::ReadPreference::SecondaryPreferred,
-                       BSON_ARRAY(BSON("dc"
-                                       << "sf")));
-
-        // Note: IdentityNS contains the name of the server.
-        unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
-        BSONObj doc = cursor->next();
-        dest = doc[HostField.name()].str();
-        ASSERT_NOT_EQUALS(dest, replSet->getPrimary());
-    }
-
-    {
-        Query query;
-        vector<pair<string, string>> tagSet;
-        query.readPref(mongo::ReadPreference::SecondaryPreferred, BSON_ARRAY(BSON("group" << 1)));
-        unique_ptr<DBClientCursor> cursor = replConn.query(NamespaceString(IdentityNS), query);
-        BSONObj doc = cursor->next();
-        const string newDest = doc[HostField.name()].str();
-        ASSERT_NOT_EQUALS(dest, newDest);
-    }
-}
-
-// Note: slaveConn is dangerous and should be deprecated! Also see SERVER-7801.
-TEST_F(TaggedFiveMemberRS, SlaveConnReturnsSecConn) {
-    MockReplicaSet* replSet = getReplSet();
-    vector<HostAndPort> seedList;
-    seedList.push_back(HostAndPort(replSet->getPrimary()));
-
-    DBClientReplicaSet replConn(replSet->getSetName(), seedList, StringData());
-
-    // Need up-to-date view since slaveConn() uses SecondaryPreferred, and this test assumes it
-    // knows about at least one secondary.
-    ReplicaSetMonitor::get(replSet->getSetName())->runScanForMockReplicaSet();
-
-    string dest;
-    mongo::DBClientConnection& secConn = replConn.slaveConn();
+    mongo::DBClientConnection& secConn = replConn.secondaryConn();
 
     // Note: IdentityNS contains the name of the server.
-    unique_ptr<DBClientCursor> cursor = secConn.query(NamespaceString(IdentityNS), Query());
+    std::unique_ptr<DBClientCursor> cursor =
+        secConn.find(FindCommandRequest{NamespaceString{IdentityNS}});
     BSONObj doc = cursor->next();
-    dest = doc[HostField.name()].str();
+    std::string dest = doc[HostField.name()].str();
     ASSERT_NOT_EQUALS(dest, replSet->getPrimary());
 }
 
