@@ -46,6 +46,8 @@
 #include "mongo/db/auth/sasl_command_constants.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/stdx/utility.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/ctype.h"
 #include "mongo/util/dns_name.h"
 #include "mongo/util/dns_query.h"
 #include "mongo/util/hex.h"
@@ -72,7 +74,7 @@ const std::vector<std::pair<std::string, std::string>> permittedTXTOptions = {{"
  */
 void mongo::uriEncode(std::ostream& ss, StringData toEncode, StringData passthrough) {
     for (const auto& c : toEncode) {
-        if ((c == '-') || (c == '_') || (c == '.') || (c == '~') || isalnum(c) ||
+        if ((c == '-') || (c == '_') || (c == '.') || (c == '~') || ctype::isAlnum(c) ||
             (passthrough.find(c) != std::string::npos)) {
             ss << c;
         } else {
@@ -85,24 +87,22 @@ void mongo::uriEncode(std::ostream& ss, StringData toEncode, StringData passthro
 mongo::StatusWith<std::string> mongo::uriDecode(StringData toDecode) {
     StringBuilder out;
     for (size_t i = 0; i < toDecode.size(); ++i) {
-        const char c = toDecode[i];
+        char c = toDecode[i];
         if (c == '%') {
-            if (i + 2 > toDecode.size()) {
+            if (i + 2 >= toDecode.size()) {
                 return Status(ErrorCodes::FailedToParse,
                               "Encountered partial escape sequence at end of string");
             }
-            auto swHex = fromHex(toDecode.substr(i + 1, 2));
-            if (swHex.isOK()) {
-                out << swHex.getValue();
-            } else {
+            try {
+                c = hexblob::decodePair(toDecode.substr(i + 1, 2));
+            } catch (const ExceptionFor<ErrorCodes::FailedToParse>&) {
                 return Status(ErrorCodes::Error(51040),
                               "The characters after the % do not form a hex value. Please escape "
                               "the % or pass a valid hex value. ");
             }
             i += 2;
-        } else {
-            out << c;
         }
+        out << c;
     }
     return out.str();
 }
@@ -421,14 +421,14 @@ MongoURI MongoURI::parseImpl(StringData url) {
         using std::end;
         std::transform(
             begin(srvEntries), end(srvEntries), back_inserter(servers), [&domain](auto&& srv) {
-                const dns::HostName target(srv.host);  // FQDN
+                const dns::HostName target(srv.first.host);  // FQDN
 
                 if (!domain.contains(target)) {
                     uasserted(ErrorCodes::FailedToParse,
                               str::stream() << "Hostname " << target << " is not within the domain "
                                             << domain);
                 }
-                return HostAndPort(target.noncanonicalName(), srv.port);
+                return HostAndPort(target.noncanonicalName(), srv.first.port);
             });
     }
 
@@ -456,12 +456,12 @@ MongoURI MongoURI::parseImpl(StringData url) {
     auto options =
         addTXTOptions(parseOptions(connectionOptions, url), canonicalHost, url, isSeedlist);
 
-    // If a replica set option was specified, store it in the 'setName' field.
+    // If a replica set option was specified, store it in the 'replicaSetName' field.
     auto optIter = options.find("replicaSet");
-    std::string setName;
+    std::string replicaSetName;
     if (optIter != end(options)) {
-        setName = optIter->second;
-        invariant(!setName.empty());
+        replicaSetName = optIter->second;
+        invariant(!replicaSetName.empty());
     }
 
     // If an appName option was specified, validate that is 128 bytes or less.
@@ -484,6 +484,20 @@ MongoURI MongoURI::parseImpl(StringData url) {
         }
     }
 
+    const auto helloOk = [&options]() -> boost::optional<bool> {
+        if (auto optIter = options.find("helloOk"); optIter != end(options)) {
+            if (auto value = optIter->second; value == "true") {
+                return true;
+            } else if (value == "false") {
+                return false;
+            } else {
+                uasserted(ErrorCodes::FailedToParse,
+                          "helloOk must be either \"true\" or \"false\"");
+            }
+        }
+        return boost::none;
+    }();
+
     transport::ConnectSSLMode sslMode = transport::kGlobalSSLMode;
     auto sslModeIter = std::find_if(options.begin(), options.end(), [](auto pred) {
         return pred.first == CaseInsensitiveString("ssl") ||
@@ -500,14 +514,16 @@ MongoURI MongoURI::parseImpl(StringData url) {
         }
     }
 
-    ConnectionString cs(
-        setName.empty() ? ConnectionString::MASTER : ConnectionString::SET, servers, setName);
+    auto cs = replicaSetName.empty()
+        ? ConnectionString::forStandalones(std::move(servers))
+        : ConnectionString::forReplicaSet(replicaSetName, std::move(servers));
     return MongoURI(std::move(cs),
                     username,
                     password,
                     database,
                     std::move(retryWrites),
                     sslMode,
+                    helloOk,
                     std::move(options));
 }
 
@@ -517,7 +533,7 @@ StatusWith<MongoURI> MongoURI::parse(StringData url) try {
     return exceptionToStatus();
 }
 
-const boost::optional<std::string> MongoURI::getAppName() const {
+boost::optional<std::string> MongoURI::getAppName() const {
     const auto optIter = _options.find("appName");
     if (optIter != end(_options)) {
         return optIter->second;
