@@ -26,7 +26,6 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include <fstream>
 #include <iostream>
@@ -36,21 +35,24 @@
 #include <boost/filesystem.hpp>
 #include <boost/filesystem/operations.hpp>
 #include <boost/format.hpp>
-#include <boost/optional/optional_io.hpp>
+#include <boost/optional/optional.hpp>
 
 #include "mongo/bson/json.h"
 #include "mongo/client/mongo_uri.h"
 #include "mongo/client/sdam/json_test_arg_parser.h"
 #include "mongo/client/sdam/sdam_configuration_parameters_gen.h"
 #include "mongo/client/sdam/topology_manager.h"
-#include "mongo/logger/logger.h"
 #include "mongo/logv2/log.h"
 #include "mongo/stdx/unordered_set.h"
 #include "mongo/util/clock_source_mock.h"
+#include "mongo/util/optional_util.h"
 #include "mongo/util/options_parser/environment.h"
 #include "mongo/util/options_parser/option_section.h"
 #include "mongo/util/options_parser/options_parser.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
+
 /**
  * This program runs the Server Discover and Monitoring JSON test files located in
  * the src/monogo/client/sdam/json_tests/sdam_tests sub-directory.
@@ -86,10 +88,10 @@ public:
             const auto bsonIsMaster = pair[1].Obj();
 
             if (bsonIsMaster.nFields() == 0) {
-                _isMasterResponses.push_back(IsMasterOutcome(address, BSONObj(), "network error"));
+                _isMasterResponses.push_back(HelloOutcome(address, BSONObj(), "network error"));
             } else {
                 _isMasterResponses.push_back(
-                    IsMasterOutcome(address, bsonIsMaster, duration_cast<IsMasterRTT>(kLatency)));
+                    HelloOutcome(address, bsonIsMaster, duration_cast<HelloRTT>(kLatency)));
             }
         }
         _topologyOutcome = phase["outcome"].Obj();
@@ -141,7 +143,8 @@ private:
     template <typename T, typename U>
     std::string errorMessageNotEqual(T expected, U actual) const {
         std::stringstream errorMessage;
-        errorMessage << "expected '" << actual << "' to equal '" << expected << "'";
+        errorMessage << "expected '" << optional_io::Extension{actual} << "' to equal '"
+                     << optional_io::Extension{expected} << "'";
         return errorMessage.str();
     }
 
@@ -419,7 +422,7 @@ private:
 
     MongoURI _testUri;
     int _phaseNum;
-    std::vector<IsMasterOutcome> _isMasterResponses;
+    std::vector<HelloOutcome> _isMasterResponses;
     BSONObj _topologyOutcome;
 };
 
@@ -455,7 +458,7 @@ public:
                                                 _replicaSetName);
 
         auto clockSource = std::make_unique<ClockSourceMock>();
-        TopologyManager topology(*config, clockSource.get());
+        TopologyManagerImpl topology(*config, clockSource.get());
 
         TestCaseResult result{{}, _testFilePath, _testName};
 
@@ -487,7 +490,7 @@ private:
             _jsonTest = fromjson(json.str());
         }
 
-        _testName = _jsonTest.getStringField("description");
+        _testName = _jsonTest.getStringField("description").toString();
         _testUri = uassertStatusOK(mongo::MongoURI::parse(_jsonTest["uri"].String()));
 
         _replicaSetName = _testUri.getOption("replicaSet");
@@ -570,7 +573,7 @@ public:
             LOGV2(20209, "### Failed Test Results ###");
         }
 
-        for (const auto result : results) {
+        for (const auto& result : results) {
             auto file = result.file;
             auto testName = result.name;
             auto phaseResults = result.phaseResults;
@@ -650,8 +653,9 @@ private:
 int main(int argc, char* argv[]) {
     ArgParser args(argc, argv);
 
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logv2::LogSeverity::Debug(args.Verbose()));
+    ::mongo::logv2::LogManager::global().getGlobalSettings().setMinimumLoggedSeverity(
+        MONGO_LOGV2_DEFAULT_COMPONENT, ::mongo::logv2::LogSeverity::Debug(args.Verbose()));
+
     args.LogParams();
 
     SdamJsonTestRunner testRunner(args.SourceDirectory(), args.TestFilters());
