@@ -26,7 +26,6 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include <fstream>
 #include <iostream>
@@ -36,7 +35,6 @@
 #include <boost/filesystem.hpp>
 #include <boost/filesystem/operations.hpp>
 #include <boost/format.hpp>
-#include <boost/optional/optional_io.hpp>
 
 #include "mongo/bson/json.h"
 #include "mongo/client/sdam/json_test_arg_parser.h"
@@ -44,14 +42,18 @@
 #include "mongo/client/sdam/server_description_builder.h"
 #include "mongo/client/sdam/server_selector.h"
 #include "mongo/client/sdam/topology_manager.h"
-#include "mongo/logger/logger.h"
 #include "mongo/logv2/log.h"
 #include "mongo/stdx/unordered_set.h"
 #include "mongo/util/clock_source_mock.h"
+#include "mongo/util/ctype.h"
+#include "mongo/util/optional_util.h"
 #include "mongo/util/options_parser/environment.h"
 #include "mongo/util/options_parser/option_section.h"
 #include "mongo/util/options_parser/options_parser.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
+
 /**
  * This program runs the Server Discover and Monitoring JSON test files located in
  * the src/monogo/client/sdam/json_tests/server_selection_tests sub-directory.
@@ -127,11 +129,11 @@ public:
             auto clockSource = std::make_unique<ClockSourceMock>();
             updatedServerDescription = std::make_shared<ServerDescription>(
                 ServerDescription(clockSource.get(),
-                                  IsMasterOutcome(HostAndPort("dummy"),
-                                                  BSON("ok" << 1 << "setname"
-                                                            << "replSet"
-                                                            << "ismaster" << true),
-                                                  IsMasterRTT(Milliseconds(_newRtt)))));
+                                  HelloOutcome(HostAndPort("dummy"),
+                                               BSON("ok" << 1 << "setname"
+                                                         << "replSet"
+                                                         << "ismaster" << true),
+                                               HelloRTT(Milliseconds(_newRtt)))));
         }
 
         TestCaseResult result{{}, _testFilePath};
@@ -162,7 +164,7 @@ private:
         // Only create the initial server description if the original avg rtt is not "NULL". If it
         // is, the test case is meant to mimic creating the first ServerDescription which we will do
         // above.
-        std::string origRttAsString = _jsonTest.getStringField("avg_rtt_ms");
+        std::string origRttAsString = _jsonTest.getStringField("avg_rtt_ms").toString();
         if (origRttAsString.compare("NULL") != 0) {
             auto serverDescription = ServerDescriptionBuilder()
                                          .withAddress(HostAndPort("dummy"))
@@ -187,7 +189,7 @@ private:
             return;
         }
 
-        auto newAvgRtt = duration_cast<Milliseconds>(newServerDescription->getRtt().get());
+        auto newAvgRtt = duration_cast<Milliseconds>(newServerDescription->getRtt().value());
         if (newAvgRtt.compare(duration_cast<Milliseconds>(Milliseconds(_newAvgRtt))) != 0) {
             std::stringstream errorMessage;
             errorMessage << "new average RTT is incorrect, got '" << newAvgRtt
@@ -286,8 +288,8 @@ private:
         // lowercased keywords. Also, change the key "tags_set" to "tags".
         // This can throw for test cases that have invalid read preferences.
         auto readPrefObj = _jsonTest.getObjectField("read_preference");
-        std::string mode = readPrefObj.getStringField("mode");
-        mode[0] = std::tolower(mode[0]);
+        std::string mode = readPrefObj.getStringField("mode").toString();
+        mode[0] = ctype::toLower(mode[0]);
         auto tagSetsObj = readPrefObj["tag_sets"];
         auto tags = tagSetsObj ? BSONArray(readPrefObj["tag_sets"].Obj()) : BSONArray();
 
@@ -363,8 +365,8 @@ private:
 
             auto tagsObj = server.getObjectField("tags");
             const auto keys = tagsObj.getFieldNames<std::set<std::string>>();
-            for (const auto key : keys) {
-                serverDescription.withTag(key, tagsObj.getStringField(key));
+            for (const auto& key : keys) {
+                serverDescription.withTag(key, tagsObj.getStringField(key).toString());
             }
 
             serverDescriptions.push_back(serverDescription.instance());
@@ -379,12 +381,13 @@ private:
         if (serverAddresses.size() > 0)
             seedList = serverAddresses;
 
-        auto config = SdamConfiguration(seedList,
-                                        initType,
-                                        Milliseconds{sdamHeartBeatFrequencyMs},
-                                        Milliseconds{sdamConnectTimeoutMs},
-                                        Milliseconds{sdamLocalThreshholdMs},
-                                        setName);
+        auto config =
+            SdamConfiguration(seedList,
+                              initType,
+                              Milliseconds{sdamHeartBeatFrequencyMs},
+                              Milliseconds{sdamConnectTimeoutMs},
+                              Milliseconds{serverGlobalParams.defaultLocalThresholdMillis},
+                              setName);
         _topologyDescription = std::make_shared<TopologyDescription>(config);
 
         std::vector<BSONElement> bsonLatencyWindow;
@@ -598,8 +601,8 @@ private:
 int main(int argc, char* argv[]) {
     ArgParser args(argc, argv);
 
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logv2::LogSeverity::Debug(args.Verbose()));
+    ::mongo::logv2::LogManager::global().getGlobalSettings().setMinimumLoggedSeverity(
+        MONGO_LOGV2_DEFAULT_COMPONENT, ::mongo::logv2::LogSeverity::Debug(args.Verbose()));
     args.LogParams();
 
     ServerSelectionJsonTestRunner testRunner(args.SourceDirectory(), args.TestFilters());
