@@ -27,29 +27,26 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/client/sdam/sdam_test_base.h"
 #include "mongo/client/sdam/topology_description.h"
 
-#include <boost/optional/optional_io.hpp>
-
 #include "mongo/client/sdam/server_description.h"
 #include "mongo/client/sdam/server_description_builder.h"
 #include "mongo/db/wire_version.h"
 #include "mongo/logv2/log.h"
 #include "mongo/unittest/death_test.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
+
 namespace mongo {
-template std::ostream& operator<<(std::ostream& os, const std::vector<HostAndPort>& s);
 
 bool operator==(const TopologyVersion& a, const TopologyVersion& b) {
     return a.getProcessId() == b.getProcessId() && a.getCounter() == b.getCounter();
 }
 
 namespace sdam {
-using mongo::operator<<;
-
 
 class TopologyDescriptionTestFixture : public SdamTestFixture {
 protected:
@@ -72,11 +69,14 @@ protected:
     static inline const std::vector<HostAndPort> kThreeServers{
         HostAndPort("foo:1234"), HostAndPort("bar:1234"), HostAndPort("baz:1234")};
 
-    static inline const auto kDefaultConfig = SdamConfiguration();
-    static inline const auto kSingleSeedConfig =
-        SdamConfiguration(kOneServer, TopologyType::kSingle);
-
     static inline const auto kNotUsedMs = Milliseconds{1000};
+
+    const SdamConfiguration kDefaultConfig;
+    const SdamConfiguration kSingleSeedConfig;
+
+    TopologyDescriptionTestFixture()
+        : kDefaultConfig(SdamConfiguration()),
+          kSingleSeedConfig(SdamConfiguration(kOneServer, TopologyType::kSingle)) {}
 };
 
 void TopologyDescriptionTestFixture::assertDefaultConfig(
@@ -106,7 +106,7 @@ TEST_F(TopologyDescriptionTestFixture, ShouldHaveCorrectDefaultValues) {
 //
 //    auto expectedAddresses = kTwoServersNormalCase;
 //
-//    auto serverAddresses = map<ServerDescriptionPtr, HostAndPort>(
+//    auto serverAddresses = map(
 //        topologyDescription.getServers(),
 //        [](const ServerDescriptionPtr& description) { return description->getAddress(); });
 //
@@ -118,10 +118,9 @@ TEST_F(TopologyDescriptionTestFixture, ShouldAllowTypeSingleWithASingleSeed) {
 
     ASSERT(TopologyType::kSingle == topologyDescription.getType());
 
-    auto servers = map<ServerDescriptionPtr, HostAndPort>(
-        topologyDescription.getServers(),
-        [](const ServerDescriptionPtr& desc) { return desc->getAddress(); });
-    ASSERT_EQUALS(kOneServer, servers);
+    auto servers =
+        map(topologyDescription.getServers(), [](const auto& desc) { return desc->getAddress(); });
+    ASSERT_EQUALS(adaptForAssert(kOneServer), adaptForAssert(servers));
 }
 
 TEST_F(TopologyDescriptionTestFixture, DoesNotAllowMultipleSeedsWithSingle) {
@@ -194,7 +193,7 @@ TEST_F(TopologyDescriptionTestFixture, ShouldNotAllowChangingTheHeartbeatFrequen
 
 TEST_F(TopologyDescriptionTestFixture,
        ShouldSetWireCompatibilityErrorForMinWireVersionWhenMinWireVersionIsGreater) {
-    const auto outgoingMaxWireVersion = WireSpec::instance().outgoing.maxWireVersion;
+    const auto outgoingMaxWireVersion = WireSpec::instance().get()->outgoing.maxWireVersion;
     const auto config = SdamConfiguration(kOneServer, TopologyType::kUnknown);
     const auto topologyDescription = std::make_shared<TopologyDescription>(config);
     const auto serverDescriptionMinVersion = ServerDescriptionBuilder()
@@ -211,7 +210,7 @@ TEST_F(TopologyDescriptionTestFixture,
 
 TEST_F(TopologyDescriptionTestFixture,
        ShouldSetWireCompatibilityErrorForMinWireVersionWhenMaxWireVersionIsLess) {
-    const auto outgoingMinWireVersion = WireSpec::instance().outgoing.minWireVersion;
+    const auto outgoingMinWireVersion = WireSpec::instance().get()->outgoing.minWireVersion;
     const auto config = SdamConfiguration(kOneServer, TopologyType::kUnknown);
     const auto topologyDescription = std::make_shared<TopologyDescription>(config);
     const auto serverDescriptionMaxVersion = ServerDescriptionBuilder()
@@ -227,7 +226,7 @@ TEST_F(TopologyDescriptionTestFixture,
 }
 
 TEST_F(TopologyDescriptionTestFixture, ShouldNotSetWireCompatibilityErrorWhenServerTypeIsUnknown) {
-    const auto outgoingMinWireVersion = WireSpec::instance().outgoing.minWireVersion;
+    const auto outgoingMinWireVersion = WireSpec::instance().get()->outgoing.minWireVersion;
     const auto config = SdamConfiguration(kOneServer, TopologyType::kUnknown);
     const auto topologyDescription = std::make_shared<TopologyDescription>(config);
     const auto serverDescriptionMaxVersion =
@@ -244,7 +243,7 @@ TEST_F(TopologyDescriptionTestFixture, ShouldSetLogicalSessionTimeoutToMinOfAllS
 
     const auto logicalSessionTimeouts = std::vector{300, 100, 200};
     auto timeoutIt = logicalSessionTimeouts.begin();
-    const auto serverDescriptionsWithTimeouts = map<ServerDescriptionPtr, ServerDescriptionPtr>(
+    const auto serverDescriptionsWithTimeouts = map(
         topologyDescription->getServers(), [&timeoutIt](const ServerDescriptionPtr& description) {
             auto newInstanceBuilder = ServerDescriptionBuilder()
                                           .withType(ServerType::kRSSecondary)
@@ -274,8 +273,8 @@ TEST_F(TopologyDescriptionTestFixture,
     const auto logicalSessionTimeouts = std::vector{300, 100, 200};
     auto timeoutIt = logicalSessionTimeouts.begin();
 
-    const auto serverDescriptionsWithTimeouts = map<ServerDescriptionPtr, ServerDescriptionPtr>(
-        topologyDescription->getServers(), [&](const ServerDescriptionPtr& description) {
+    const auto serverDescriptionsWithTimeouts =
+        map(topologyDescription->getServers(), [&](const ServerDescriptionPtr& description) {
             auto timeoutValue = (timeoutIt == logicalSessionTimeouts.begin())
                 ? boost::none
                 : boost::make_optional(*timeoutIt);
