@@ -26,11 +26,9 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
 #include "mongo/client/sdam/topology_state_machine.h"
 
-#include <functional>
 #include <ostream>
 
 #include "mongo/client/sdam/election_id_set_version_pair.h"
@@ -38,9 +36,12 @@
 #include "mongo/logv2/log.h"
 #include "mongo/util/fail_point.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
+
+
 namespace mongo::sdam {
 namespace {
-static constexpr auto kLogLevel = 2;
+static constexpr auto kLogLevel = 0;
 }  // namespace
 
 TopologyStateMachine::TopologyStateMachine(const SdamConfiguration& config) : _config(config) {
@@ -59,18 +60,17 @@ inline int idx(T enumType) {
  * https://github.com/mongodb/specifications/blob/master/source/server-discovery-and-monitoring/server-discovery-and-monitoring.rst#topologytype-table
  */
 void mongo::sdam::TopologyStateMachine::initTransitionTable() {
-    using namespace std::placeholders;
+    auto bindThis = [&](auto&& pmf) { return [=](auto&&... a) { (this->*pmf)(a...); }; };
 
     // init the table to No-ops
-    const TransitionAction NO_OP([](const TopologyDescription&, const ServerDescriptionPtr&) {});
     _stt.resize(allTopologyTypes().size() + 1);
     for (auto& row : _stt) {
-        row.resize(allServerTypes().size() + 1, NO_OP);
+        row.resize(allServerTypes().size() + 1, [](auto&&...) {});
     }
 
     // From TopologyType: Unknown
     _stt[idx(TopologyType::kUnknown)][idx(ServerType::kStandalone)] =
-        std::bind(&TopologyStateMachine::updateUnknownWithStandalone, this, _1, _2);
+        bindThis(&TopologyStateMachine::updateUnknownWithStandalone);
     _stt[idx(TopologyType::kUnknown)][idx(ServerType::kMongos)] =
         setTopologyTypeAction(TopologyType::kSharded);
     _stt[idx(TopologyType::kUnknown)][idx(ServerType::kRSPrimary)] =
@@ -80,8 +80,8 @@ void mongo::sdam::TopologyStateMachine::initTransitionTable() {
         const auto serverTypes = std::vector<ServerType>{
             ServerType::kRSSecondary, ServerType::kRSArbiter, ServerType::kRSOther};
         for (auto newServerType : serverTypes) {
-            _stt[idx(TopologyType::kUnknown)][idx(newServerType)] = std::bind(
-                &TopologyStateMachine::setTopologyTypeAndUpdateRSWithoutPrimary, this, _1, _2);
+            _stt[idx(TopologyType::kUnknown)][idx(newServerType)] =
+                bindThis(&TopologyStateMachine::setTopologyTypeAndUpdateRSWithoutPrimary);
         }
     }
 
@@ -95,7 +95,7 @@ void mongo::sdam::TopologyStateMachine::initTransitionTable() {
                                                          ServerType::kRSGhost};
         for (auto newServerType : serverTypes) {
             _stt[idx(TopologyType::kSharded)][idx(newServerType)] =
-                std::bind(&TopologyStateMachine::removeAndStopMonitoring, this, _1, _2);
+                bindThis(&TopologyStateMachine::removeAndStopMonitoring);
         }
     }
 
@@ -105,7 +105,7 @@ void mongo::sdam::TopologyStateMachine::initTransitionTable() {
             std::vector<ServerType>{ServerType::kStandalone, ServerType::kMongos};
         for (auto serverType : serverTypes) {
             _stt[idx(TopologyType::kReplicaSetNoPrimary)][idx(serverType)] =
-                std::bind(&TopologyStateMachine::removeAndStopMonitoring, this, _1, _2);
+                bindThis(&TopologyStateMachine::removeAndStopMonitoring);
         }
     }
 
@@ -117,7 +117,7 @@ void mongo::sdam::TopologyStateMachine::initTransitionTable() {
             ServerType::kRSSecondary, ServerType::kRSArbiter, ServerType::kRSOther};
         for (auto serverType : serverTypes) {
             _stt[idx(TopologyType::kReplicaSetNoPrimary)][idx(serverType)] =
-                std::bind(&TopologyStateMachine::updateRSWithoutPrimary, this, _1, _2);
+                bindThis(&TopologyStateMachine::updateRSWithoutPrimary);
         }
     }
 
@@ -127,7 +127,7 @@ void mongo::sdam::TopologyStateMachine::initTransitionTable() {
             std::vector<ServerType>{ServerType::kUnknown, ServerType::kRSGhost};
         for (auto serverType : serverTypes) {
             _stt[idx(TopologyType::kReplicaSetWithPrimary)][idx(serverType)] =
-                std::bind(&TopologyStateMachine::checkIfHasPrimary, this, _1, _2);
+                bindThis(&TopologyStateMachine::checkIfHasPrimary);
         }
     }
 
@@ -135,19 +135,19 @@ void mongo::sdam::TopologyStateMachine::initTransitionTable() {
         const auto serverTypes = std::vector<ServerType>{ServerType::kMongos};
         for (auto serverType : serverTypes) {
             _stt[idx(TopologyType::kReplicaSetWithPrimary)][idx(serverType)] =
-                std::bind(&TopologyStateMachine::removeAndCheckIfHasPrimary, this, _1, _2);
+                bindThis(&TopologyStateMachine::removeAndCheckIfHasPrimary);
         }
     }
 
     _stt[idx(TopologyType::kReplicaSetWithPrimary)][idx(ServerType::kRSPrimary)] =
-        std::bind(&TopologyStateMachine::updateRSFromPrimary, this, _1, _2);
+        bindThis(&TopologyStateMachine::updateRSFromPrimary);
 
     {
         const auto serverTypes = std::vector<ServerType>{
             ServerType::kRSSecondary, ServerType::kRSArbiter, ServerType::kRSOther};
         for (auto serverType : serverTypes) {
             _stt[idx(TopologyType::kReplicaSetWithPrimary)][idx(serverType)] =
-                std::bind(&TopologyStateMachine::updateRSWithPrimaryFromMember, this, _1, _2);
+                bindThis(&TopologyStateMachine::updateRSWithPrimaryFromMember);
         }
     }
 }
@@ -155,12 +155,15 @@ void mongo::sdam::TopologyStateMachine::initTransitionTable() {
 void TopologyStateMachine::onServerDescription(TopologyDescription& topologyDescription,
                                                const ServerDescriptionPtr& serverDescription) {
     if (!topologyDescription.containsServerAddress(serverDescription->getAddress())) {
-        LOGV2_DEBUG(20219,
-                    kLogLevel,
-                    "Ignoring isMaster reply from server that is not in the topology: "
-                    "{serverAddress}",
-                    "Ignoring isMaster reply from server that is not in the topology",
-                    "serverAddress"_attr = serverDescription->getAddress());
+        const auto& setName = topologyDescription.getSetName();
+        LOGV2_DEBUG(
+            20219,
+            kLogLevel,
+            "{replSetName}: Ignoring isMaster reply from server that is not in the topology: "
+            "{serverAddress}",
+            "Ignoring isMaster reply from server that is not in the topology",
+            "replicaSet"_attr = setName ? *setName : std::string(""),
+            "serverAddress"_attr = serverDescription->getAddress());
         return;
     }
 
@@ -386,7 +389,8 @@ void TopologyStateMachine::removeServerDescription(TopologyDescription& topology
                 kLogLevel,
                 "Server '{serverAddress}' was removed from the topology",
                 "Server was removed from the topology",
-                "serverAddress"_attr = serverAddress);
+                "serverAddress"_attr = serverAddress,
+                "topologyDescription"_attr = topologyDescription.toBSON());
 }
 
 void TopologyStateMachine::modifyTopologyType(TopologyDescription& topologyDescription,
