@@ -28,8 +28,6 @@
  */
 #include "mongo/client/sdam/topology_state_machine.h"
 
-#include <boost/optional/optional_io.hpp>
-
 #include "mongo/client/sdam/sdam_test_base.h"
 #include "mongo/client/sdam/server_description.h"
 #include "mongo/client/sdam/server_description_builder.h"
@@ -47,21 +45,22 @@ protected:
     static inline const auto kNotUsedMs = Milliseconds(100);
     static inline const auto kFiveHundredMs = Milliseconds(500);
 
-    static inline const auto kTwoSeedConfig =
-        SdamConfiguration(std::vector<HostAndPort>{kLocalServer, kLocalServer2},
-                          TopologyType::kUnknown,
-                          kFiveHundredMs);
-
-    static inline const auto kTwoSeedReplicaSetNoPrimaryConfig =
-        SdamConfiguration(std::vector<HostAndPort>{kLocalServer, kLocalServer2},
-                          TopologyType::kReplicaSetNoPrimary,
-                          kFiveHundredMs,
-                          kNotUsedMs,
-                          kNotUsedMs,
-                          std::string("setName"));
-
-    static inline const auto kSingleConfig =
-        SdamConfiguration(std::vector<HostAndPort>{kLocalServer}, TopologyType::kSingle);
+    const SdamConfiguration kTwoSeedConfig;
+    const SdamConfiguration kTwoSeedReplicaSetNoPrimaryConfig;
+    const SdamConfiguration kSingleConfig;
+    TopologyStateMachineTestFixture()
+        : kTwoSeedConfig(SdamConfiguration(std::vector<HostAndPort>{kLocalServer, kLocalServer2},
+                                           TopologyType::kUnknown,
+                                           kFiveHundredMs)),
+          kTwoSeedReplicaSetNoPrimaryConfig(
+              SdamConfiguration(std::vector<HostAndPort>{kLocalServer, kLocalServer2},
+                                TopologyType::kReplicaSetNoPrimary,
+                                kFiveHundredMs,
+                                kNotUsedMs,
+                                kNotUsedMs,
+                                std::string("setName"))),
+          kSingleConfig(
+              SdamConfiguration(std::vector<HostAndPort>{kLocalServer}, TopologyType::kSingle)) {}
 
     // Given we are in 'starting' state with initial config 'initialConfig'. We receive a
     // ServerDescription with type 'incoming', and expected the ending topology state to be
@@ -189,14 +188,12 @@ TEST_F(TopologyStateMachineTestFixture,
                                  .instance();
 
     ASSERT_EQUALS(static_cast<size_t>(2), topologyDescription->getServers().size());
-    auto serversBefore = map<ServerDescriptionPtr, HostAndPort>(topologyDescription->getServers(),
-                                                                getServerDescriptionAddress);
+    auto serversBefore = map(topologyDescription->getServers(), getServerDescriptionAddress);
 
     stateMachine.onServerDescription(*topologyDescription, serverDescription);
 
-    auto serversAfter = map<ServerDescriptionPtr, HostAndPort>(topologyDescription->getServers(),
-                                                               getServerDescriptionAddress);
-    ASSERT_EQUALS(serversBefore, serversAfter);
+    auto serversAfter = map(topologyDescription->getServers(), getServerDescriptionAddress);
+    ASSERT_EQUALS(adaptForAssert(serversBefore), adaptForAssert(serversAfter));
 }
 
 TEST_F(TopologyStateMachineTestFixture, ShouldRemoveServerDescriptionIfShardDoesntMatch) {
@@ -299,15 +296,14 @@ TEST_F(TopologyStateMachineTestFixture,
     ASSERT_EQUALS(topologyDescription->getType(), TopologyType::kReplicaSetWithPrimary);
     ASSERT_EQUALS(static_cast<size_t>(2), topologyDescription->getServers().size());
 
-    auto serversBefore = map<ServerDescriptionPtr, HostAndPort>(topologyDescription->getServers(),
-                                                                getServerDescriptionAddress);
+    auto serversBefore = map(topologyDescription->getServers(), getServerDescriptionAddress);
 
     stateMachine.onServerDescription(*topologyDescription, serverDescription);
-    ASSERT_EQUALS(topologyDescription->getType(), TopologyType::kReplicaSetWithPrimary);
+    ASSERT_EQUALS(adaptForAssert(topologyDescription->getType()),
+                  adaptForAssert(TopologyType::kReplicaSetWithPrimary));
 
-    auto serversAfter = map<ServerDescriptionPtr, HostAndPort>(topologyDescription->getServers(),
-                                                               getServerDescriptionAddress);
-    ASSERT_EQUALS(serversBefore, serversAfter);
+    auto serversAfter = map(topologyDescription->getServers(), getServerDescriptionAddress);
+    ASSERT_EQUALS(adaptForAssert(serversBefore), adaptForAssert(serversAfter));
 }
 
 TEST_F(TopologyStateMachineTestFixture,
