@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/platform/basic.h"
 
@@ -45,10 +44,13 @@
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/duration.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
+
 namespace mongo {
 namespace {
 
-const sdam::IsMasterRTT initialRTT = duration_cast<Milliseconds>(Milliseconds(100));
+const sdam::HelloRTT initialRTT = duration_cast<Milliseconds>(Milliseconds(100));
 using executor::NetworkInterfaceMock;
 using executor::RemoteCommandResponse;
 using executor::ThreadPoolExecutorTest;
