@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 #include "mongo/platform/basic.h"
 
 #include "mongo/client/streamable_replica_set_monitor.h"
@@ -40,6 +39,8 @@
 #include "mongo/client/connpool.h"
 #include "mongo/client/global_conn_pool.h"
 #include "mongo/client/read_preference.h"
+#include "mongo/client/replica_set_monitor_server_parameters_gen.h"
+#include "mongo/client/streamable_replica_set_monitor_discovery_time_processor.h"
 #include "mongo/client/streamable_replica_set_monitor_query_processor.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/bson_extract_optime.h"
@@ -54,6 +55,9 @@
 #include "mongo/util/string_map.h"
 #include "mongo/util/timer.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
+
+
 namespace mongo {
 using namespace mongo::sdam;
 
@@ -91,11 +95,11 @@ bool primaryOrSecondaryPredicate(const ServerDescriptionPtr& server) {
     return serverType == ServerType::kRSPrimary || serverType == ServerType::kRSSecondary;
 }
 
-std::string readPrefToStringWithMinOpTime(const ReadPreferenceSetting& readPref) {
+std::string readPrefToStringFull(const ReadPreferenceSetting& readPref) {
     BSONObjBuilder builder;
     readPref.toInnerBSON(&builder);
-    if (!readPref.minOpTime.isNull()) {
-        builder.append("minOpTime", readPref.minOpTime.toBSON());
+    if (!readPref.minClusterTime.isNull()) {
+        builder.append("minClusterTime", readPref.minClusterTime.toBSON());
     }
     return builder.obj().toString();
 }
@@ -118,6 +122,43 @@ double pingTimeMillis(const ServerDescriptionPtr& serverDescription) {
 }
 
 constexpr auto kZeroMs = Milliseconds(0);
+
+Status makeUnsatisfiedReadPrefError(const std::string& name,
+                                    const ReadPreferenceSetting& criteria) {
+    return Status(ErrorCodes::FailedToSatisfyReadPreference,
+                  str::stream() << "Could not find host matching read preference "
+                                << criteria.toString() << " for set " << name);
+}
+
+Status makeReplicaSetMonitorRemovedError(const std::string& name) {
+    return Status(ErrorCodes::ShutdownInProgress,
+                  str::stream() << "ReplicaSetMonitor for set " << name << " is removed");
+}
+
+bool hasMembershipChange(sdam::TopologyDescriptionPtr oldDescription,
+                         sdam::TopologyDescriptionPtr newDescription) {
+    if (oldDescription->getServers().size() != newDescription->getServers().size())
+        return true;
+
+    for (const auto& server : oldDescription->getServers()) {
+        const auto newServer = newDescription->findServerByAddress(server->getAddress());
+        if (!newServer)
+            return true;
+        const ServerDescription& s = *server;
+        const ServerDescription& ns = **newServer;
+        if (s != ns)
+            return true;
+    }
+
+    for (const auto& server : newDescription->getServers()) {
+        auto oldServer = oldDescription->findServerByAddress(server->getAddress());
+        if (!oldServer)
+            return true;
+    }
+
+    return false;
+}
+
 }  // namespace
 
 
@@ -147,13 +188,19 @@ constexpr auto kZeroMs = Milliseconds(0);
 StreamableReplicaSetMonitor::StreamableReplicaSetMonitor(
     const MongoURI& uri,
     std::shared_ptr<TaskExecutor> executor,
-    std::shared_ptr<executor::EgressTagCloser> connectionManager)
-    : _errorHandler(std::make_unique<SdamErrorHandler>(uri.getSetName())),
+    std::shared_ptr<executor::EgressTagCloser> connectionManager,
+    std::function<void()> cleanupCallback,
+    std::shared_ptr<ReplicaSetMonitorManagerStats> managerStats)
+    : ReplicaSetMonitor(cleanupCallback),
+      _errorHandler(std::make_unique<SdamErrorHandler>(uri.getSetName())),
       _queryProcessor(std::make_shared<StreamableReplicaSetMonitorQueryProcessor>()),
+      _primaryDiscoveryTimeProcessor(
+          std::make_shared<StreamableReplicaSetMonitorDiscoveryTimeProcessor>()),
       _uri(uri),
       _connectionManager(connectionManager),
       _executor(executor),
-      _random(PseudoRandom(SecureRandom().nextInt64())) {
+      _random(PseudoRandom(SecureRandom().nextInt64())),
+      _stats(std::make_shared<ReplicaSetMonitorStats>(managerStats)) {
     // Maintain order of original seed list
     std::vector<HostAndPort> seedsNoDups;
     std::set<HostAndPort> alreadyAdded;
@@ -174,12 +221,22 @@ StreamableReplicaSetMonitor::StreamableReplicaSetMonitor(
     _serverSelector = std::make_unique<SdamServerSelector>(_sdamConfig);
 }
 
+StreamableReplicaSetMonitor::~StreamableReplicaSetMonitor() {
+    // `drop` is idempotent and a duplicate call from ReplicaSetMonitorManager::removeMonitor() is
+    // safe.
+    drop();
+}
+
 ReplicaSetMonitorPtr StreamableReplicaSetMonitor::make(
     const MongoURI& uri,
     std::shared_ptr<TaskExecutor> executor,
-    std::shared_ptr<executor::EgressTagCloser> connectionManager) {
-    auto result = std::make_shared<StreamableReplicaSetMonitor>(uri, executor, connectionManager);
+    std::shared_ptr<executor::EgressTagCloser> connectionManager,
+    std::function<void()> cleanupCallback,
+    std::shared_ptr<ReplicaSetMonitorManagerStats> managerStats) {
+    auto result = std::make_shared<StreamableReplicaSetMonitor>(
+        uri, executor, connectionManager, cleanupCallback, managerStats);
     result->init();
+    invariant(managerStats);
     return result;
 }
 
@@ -192,27 +249,49 @@ void StreamableReplicaSetMonitor::init() {
                 "uri"_attr = _uri,
                 "config"_attr = _sdamConfig.toBson());
 
+    invariant(shared_from_this().use_count() > 1,
+              "StreamableReplicaSetMonitor::init() is invoked when there is no owner");
+
     _eventsPublisher = std::make_shared<sdam::TopologyEventsPublisher>(_executor);
-    _topologyManager = std::make_unique<TopologyManager>(
+    _topologyManager = std::make_unique<TopologyManagerImpl>(
         _sdamConfig, getGlobalServiceContext()->getPreciseClockSource(), _eventsPublisher);
 
-    _eventsPublisher->registerListener(shared_from_this());
+    _eventsPublisher->registerListener(weak_from_this());
 
     _pingMonitor = std::make_unique<ServerPingMonitor>(
         _uri, _eventsPublisher.get(), _sdamConfig.getHeartBeatFrequency(), _executor);
     _eventsPublisher->registerListener(_pingMonitor);
 
-    _isMasterMonitor = std::make_unique<ServerIsMasterMonitor>(
-        _uri, _sdamConfig, _eventsPublisher, _topologyManager->getTopologyDescription(), _executor);
-    _eventsPublisher->registerListener(_isMasterMonitor);
+    _serverDiscoveryMonitor =
+        std::make_unique<ServerDiscoveryMonitor>(_uri,
+                                                 _sdamConfig,
+                                                 _eventsPublisher,
+                                                 _topologyManager->getTopologyDescription(),
+                                                 _stats,
+                                                 _executor);
+    _eventsPublisher->registerListener(_serverDiscoveryMonitor);
 
     _eventsPublisher->registerListener(_queryProcessor);
 
+    _eventsPublisher->registerListener(_primaryDiscoveryTimeProcessor);
+
     _isDropped.store(false);
 
     ReplicaSetMonitorManager::get()->getNotifier().onFoundSet(getName());
 }
 
+void StreamableReplicaSetMonitor::initForTesting(sdam::TopologyManagerPtr topologyManager) {
+    stdx::lock_guard lock(_mutex);
+
+    _eventsPublisher = std::make_shared<sdam::TopologyEventsPublisher>(_executor);
+    _topologyManager = std::move(topologyManager);
+
+    _eventsPublisher->registerListener(weak_from_this());
+
+    _isDropped.store(false);
+    ReplicaSetMonitorManager::get()->getNotifier().onFoundSet(getName());
+}
+
 void StreamableReplicaSetMonitor::drop() {
     {
         stdx::lock_guard lock(_mutex);
@@ -229,8 +308,14 @@ void StreamableReplicaSetMonitor::drop() {
           "Closing Replica Set Monitor",
           "replicaSet"_attr = getName());
     _queryProcessor->shutdown();
-    _pingMonitor->shutdown();
-    _isMasterMonitor->shutdown();
+
+    if (_pingMonitor) {
+        _pingMonitor->shutdown();
+    }
+
+    if (_serverDiscoveryMonitor) {
+        _serverDiscoveryMonitor->shutdown();
+    }
 
     ReplicaSetMonitorManager::get()->getNotifier().onDroppedSet(getName());
     LOGV2(4333210,
@@ -240,8 +325,10 @@ void StreamableReplicaSetMonitor::drop() {
 }
 
 SemiFuture<HostAndPort> StreamableReplicaSetMonitor::getHostOrRefresh(
-    const ReadPreferenceSetting& criteria, Milliseconds maxWait) {
-    return getHostsOrRefresh(criteria, maxWait)
+    const ReadPreferenceSetting& criteria,
+    const std::vector<HostAndPort>& excludedHosts,
+    const CancellationToken& cancelToken) {
+    return getHostsOrRefresh(criteria, excludedHosts, cancelToken)
         .thenRunOn(_executor)
         .then([self = shared_from_this()](const std::vector<HostAndPort>& result) {
             invariant(result.size());
@@ -260,123 +347,136 @@ std::vector<HostAndPort> StreamableReplicaSetMonitor::_extractHosts(
 }
 
 SemiFuture<std::vector<HostAndPort>> StreamableReplicaSetMonitor::getHostsOrRefresh(
-    const ReadPreferenceSetting& criteria, Milliseconds maxWait) {
+    const ReadPreferenceSetting& criteria,
+    const std::vector<HostAndPort>& excludedHosts,
+    const CancellationToken& cancelToken) {
     // In the fast case (stable topology), we avoid mutex acquisition.
     if (_isDropped.load()) {
-        return _makeReplicaSetMonitorRemovedError();
+        return makeReplicaSetMonitorRemovedError(getName());
     }
 
     // start counting from the beginning of the operation
-    const auto deadline = _executor->now() + ((maxWait > kZeroMs) ? maxWait : kZeroMs);
+    const auto deadline =
+        _executor->now() + Milliseconds(gDefaultFindReplicaSetHostTimeoutMS.load());
 
     // try to satisfy query immediately
-    auto immediateResult = _getHosts(criteria);
+    auto immediateResult = _getHosts(criteria, excludedHosts);
     if (immediateResult) {
         return {*immediateResult};
     }
 
-    _isMasterMonitor->requestImmediateCheck();
+    if (_serverDiscoveryMonitor) {
+        _serverDiscoveryMonitor->requestImmediateCheck();
+    }
+
     LOGV2_DEBUG(4333212,
                 kLowerLogLevel,
                 "RSM {replicaSet} start async getHosts with {readPref}",
                 "RSM start async getHosts",
                 "replicaSet"_attr = getName(),
-                "readPref"_attr = readPrefToStringWithMinOpTime(criteria));
+                "readPref"_attr = readPrefToStringFull(criteria));
 
-    // fail fast on timeout
+    // Fail fast on timeout or cancellation.
     const Date_t& now = _executor->now();
-    if (deadline <= now) {
-        return _makeUnsatisfiedReadPrefError(criteria);
+    if (deadline <= now || cancelToken.isCanceled()) {
+        return makeUnsatisfiedReadPrefError(getName(), criteria);
     }
 
-    return _topologyManager->executeWithLock(
-        [this, criteria, deadline](const TopologyDescriptionPtr& topologyDescription)
-            -> SemiFuture<std::vector<HostAndPort>> {
-            stdx::lock_guard lk(_mutex);
+    return _topologyManager->executeWithLock([this, criteria, cancelToken, deadline, excludedHosts](
+                                                 const TopologyDescriptionPtr& topologyDescription)
+                                                 -> SemiFuture<std::vector<HostAndPort>> {
+        stdx::lock_guard lk(_mutex);
 
-            // We check if we are closed under the mutex here since someone could have called
-            // close() concurrently with the code above.
-            if (_isDropped.load()) {
-                return _makeReplicaSetMonitorRemovedError();
-            }
-            // try to satisfy the query again while holding both the StreamableRSM mutex and
-            // TopologyManager mutex to avoid missing any topology change that has occurred
-            // since the last check.
-            auto immediateResult = _getHosts(topologyDescription, criteria);
-            if (immediateResult) {
-                return {*immediateResult};
-            }
+        // We check if we are closed under the mutex here since someone could have called
+        // close() concurrently with the code above.
+        if (_isDropped.load()) {
+            return makeReplicaSetMonitorRemovedError(getName());
+        }
+        // try to satisfy the query again while holding both the StreamableRSM mutex and
+        // TopologyManager mutex to avoid missing any topology change that has occurred
+        // since the last check.
+        auto immediateResult = _getHosts(topologyDescription, criteria, excludedHosts);
+        if (immediateResult) {
+            return {*immediateResult};
+        }
 
-            return _enqueueOutstandingQuery(lk, criteria, deadline);
-        });
+        return _enqueueOutstandingQuery(lk, criteria, excludedHosts, cancelToken, deadline);
+    });
 }
 
 SemiFuture<std::vector<HostAndPort>> StreamableReplicaSetMonitor::_enqueueOutstandingQuery(
-    WithLock, const ReadPreferenceSetting& criteria, const Date_t& deadline) {
-    using HostAndPortList = std::vector<HostAndPort>;
-    Future<HostAndPortList> result;
+    WithLock,
+    const ReadPreferenceSetting& criteria,
+    const std::vector<HostAndPort>& excludedHosts,
+    const CancellationToken& cancelToken,
+    const Date_t& deadline) {
 
-    auto query = std::make_shared<HostQuery>();
+    auto query = std::make_shared<HostQuery>(_stats);
     query->criteria = criteria;
-    query->deadline = deadline;
+    query->excludedHosts = excludedHosts;
 
-    auto pf = makePromiseFuture<HostAndPortList>();
+    auto pf = makePromiseFuture<std::vector<HostAndPort>>();
     query->promise = std::move(pf.promise);
 
-    auto deadlineCb =
-        [this, query, self = shared_from_this()](const TaskExecutor::CallbackArgs& cbArgs) {
-            stdx::lock_guard lock(_mutex);
-            if (query->done) {
-                return;
+    // Make the deadline task cancelable for when the query is satisfied or when the input
+    // cancelToken is canceled.
+    query->deadlineCancelSource = CancellationSource(cancelToken);
+    query->start = _executor->now();
+
+    // Add the query to the list of outstanding queries.
+    auto queryIter = _outstandingQueries.insert(_outstandingQueries.end(), query);
+
+    // After a deadline or when the input cancellation token is canceled, cancel this query. If the
+    // query completes first, the deadlineCancelSource will be used to cancel this task.
+    _executor->sleepUntil(deadline, query->deadlineCancelSource.token())
+        .getAsync([this, query, queryIter, self = shared_from_this(), cancelToken](Status status) {
+            // If the deadline was reached or cancellation occurred on the input cancellation token,
+            // mark the query as canceled. Otherwise, the deadlineCancelSource must have been
+            // canceled due to the query completing successfully.
+            if (status.isOK() || cancelToken.isCanceled()) {
+                auto errorStatus = makeUnsatisfiedReadPrefError(self->getName(), query->criteria);
+                // Mark query as done, and if it wasn't already done, remove it from the list of
+                // outstanding queries.
+                if (query->tryCancel(errorStatus)) {
+                    LOGV2_INFO(4333208,
+                               "RSM {replicaSet} host selection timeout: {error}",
+                               "RSM host selection timeout",
+                               "replicaSet"_attr = self->getName(),
+                               "error"_attr = errorStatus.toString());
+
+                    stdx::lock_guard lk(_mutex);
+                    // Check that the RSM hasn't been dropped (and _outstandingQueries has not
+                    // been cleared) before erasing.
+                    if (!_isDropped.load()) {
+                        invariant(_outstandingQueries.size() > 0);
+                        _eraseQueryFromOutstandingQueries(lk, queryIter);
+                    }
+                }
             }
-
-            const auto cbStatus = cbArgs.status;
-            if (!cbStatus.isOK()) {
-                query->promise.setError(cbStatus);
-                query->done = true;
-                return;
-            }
-
-            const auto errorStatus = _makeUnsatisfiedReadPrefError(query->criteria);
-            query->promise.setError(errorStatus);
-            query->done = true;
-            LOGV2_INFO(4333208,
-                       "RSM {replicaSet} host selection timeout: {error}",
-                       "RSM host selection timeout",
-                       "replicaSet"_attr = getName(),
-                       "error"_attr = errorStatus.toString());
-        };
-    auto swDeadlineHandle = _executor->scheduleWorkAt(query->deadline, deadlineCb);
-
-    if (!swDeadlineHandle.isOK()) {
-        LOGV2_INFO(4333207,
-                   "RSM {replicaSet} error scheduling deadline handler: {error}",
-                   "RSM error scheduling deadline handler",
-                   "replicaSet"_attr = getName(),
-                   "error"_attr = swDeadlineHandle.getStatus());
-        return SemiFuture<HostAndPortList>::makeReady(swDeadlineHandle.getStatus());
-    }
-    query->deadlineHandle = swDeadlineHandle.getValue();
-    _outstandingQueries.push_back(query);
+        });
 
     return std::move(pf.future).semi();
 }
 
 boost::optional<std::vector<HostAndPort>> StreamableReplicaSetMonitor::_getHosts(
-    const TopologyDescriptionPtr& topology, const ReadPreferenceSetting& criteria) {
-    auto result = _serverSelector->selectServers(topology, criteria);
+    const TopologyDescriptionPtr& topology,
+    const ReadPreferenceSetting& criteria,
+    const std::vector<HostAndPort>& excludedHosts) {
+    auto result = _serverSelector->selectServers(topology, criteria, excludedHosts);
     if (!result)
         return boost::none;
     return _extractHosts(*result);
 }
 
 boost::optional<std::vector<HostAndPort>> StreamableReplicaSetMonitor::_getHosts(
-    const ReadPreferenceSetting& criteria) {
-    return _getHosts(_currentTopology(), criteria);
+    const ReadPreferenceSetting& criteria, const std::vector<HostAndPort>& excludedHosts) {
+    return _getHosts(_currentTopology(), criteria, excludedHosts);
 }
 
-HostAndPort StreamableReplicaSetMonitor::getMasterOrUassert() {
-    return getHostOrRefresh(kPrimaryOnlyReadPreference).get();
+HostAndPort StreamableReplicaSetMonitor::getPrimaryOrUassert() {
+    return ReplicaSetMonitorInterface::getHostOrRefresh(kPrimaryOnlyReadPreference,
+                                                        CancellationToken::uncancelable())
+        .get();
 }
 
 sdam::TopologyEventsPublisherPtr StreamableReplicaSetMonitor::getEventsPublisher() {
@@ -423,13 +523,13 @@ void StreamableReplicaSetMonitor::_doErrorActions(
         if (errorActions.dropConnections)
             _connectionManager->dropConnections(host);
 
-        if (errorActions.requestImmediateCheck)
-            _isMasterMonitor->requestImmediateCheck();
+        if (errorActions.requestImmediateCheck && _serverDiscoveryMonitor)
+            _serverDiscoveryMonitor->requestImmediateCheck();
     }
 
     // Call outside of the lock since this may generate a topology change event.
-    if (errorActions.isMasterOutcome)
-        _topologyManager->onServerDescription(*errorActions.isMasterOutcome);
+    if (errorActions.helloOutcome)
+        _topologyManager->onServerDescription(*errorActions.helloOutcome);
 }
 
 boost::optional<ServerDescriptionPtr> StreamableReplicaSetMonitor::_currentPrimary() const {
@@ -518,14 +618,14 @@ void StreamableReplicaSetMonitor::appendInfo(BSONObjBuilder& bsonObjBuilder, boo
     BSONArrayBuilder hosts(monitorInfo.subarrayStart("hosts"));
     for (const auto& serverDescription : topologyDescription->getServers()) {
         bool isUp = false;
-        bool isMaster = false;
+        bool isWritablePrimary = false;
         bool isSecondary = false;
         bool isHidden = false;
 
         switch (serverDescription->getType()) {
             case ServerType::kRSPrimary:
                 isUp = true;
-                isMaster = true;
+                isWritablePrimary = true;
                 break;
             case ServerType::kRSSecondary:
                 isUp = true;
@@ -550,7 +650,7 @@ void StreamableReplicaSetMonitor::appendInfo(BSONObjBuilder& bsonObjBuilder, boo
         BSONObjBuilder builder(hosts.subobjStart());
         builder.append("addr", serverDescription->getAddress().toString());
         builder.append("ok", isUp);
-        builder.append("ismaster", isMaster);  // intentionally not camelCase
+        builder.append("ismaster", isWritablePrimary);  // intentionally not camelCase
         builder.append("hidden", isHidden);
         builder.append("secondary", isSecondary);
         builder.append("pingTimeMillis", pingTimeMillis(serverDescription));
@@ -592,13 +692,13 @@ void StreamableReplicaSetMonitor::onTopologyDescriptionChangedEvent(
         return;
 
     // Notify external components if there are membership changes in the topology.
-    if (_hasMembershipChange(previousDescription, newDescription)) {
+    if (hasMembershipChange(previousDescription, newDescription)) {
         LOGV2(4333213,
               "RSM {replicaSet} Topology Change: {newTopologyDescription}",
               "RSM Topology Change",
               "replicaSet"_attr = getName(),
-              "newTopologyDescription"_attr = newDescription->toString(),
-              "previousTopologyDescription"_attr = previousDescription->toString());
+              "newTopologyDescription"_attr = newDescription->toBSON(),
+              "previousTopologyDescription"_attr = previousDescription->toBSON());
 
         auto maybePrimary = newDescription->getPrimary();
         if (maybePrimary) {
@@ -654,8 +754,8 @@ void StreamableReplicaSetMonitor::onTopologyDescriptionChangedEvent(
 
 void StreamableReplicaSetMonitor::onServerHeartbeatSucceededEvent(const HostAndPort& hostAndPort,
                                                                   const BSONObj reply) {
-    // After the inital handshake, isMasterResponses should not update the RTT with durationMs.
-    IsMasterOutcome outcome(hostAndPort, reply, boost::none);
+    // After the inital handshake, hello responses should not update the RTT with durationMs.
+    HelloOutcome outcome(hostAndPort, reply, boost::none);
     _topologyManager->onServerDescription(outcome);
 }
 
@@ -677,7 +777,7 @@ void StreamableReplicaSetMonitor::onServerHandshakeFailedEvent(const HostAndPort
     _failedHost(HostAndPort(address), status, reply, HandshakeStage::kPreHandshake, false);
 };
 
-void StreamableReplicaSetMonitor::onServerPingSucceededEvent(sdam::IsMasterRTT durationMS,
+void StreamableReplicaSetMonitor::onServerPingSucceededEvent(sdam::HelloRTT durationMS,
                                                              const HostAndPort& hostAndPort) {
     LOGV2_DEBUG(4668132,
                 kLowerLogLevel,
@@ -688,10 +788,10 @@ void StreamableReplicaSetMonitor::onServerPingSucceededEvent(sdam::IsMasterRTT d
     _topologyManager->onServerRTTUpdated(hostAndPort, durationMS);
 }
 
-void StreamableReplicaSetMonitor::onServerHandshakeCompleteEvent(sdam::IsMasterRTT durationMs,
+void StreamableReplicaSetMonitor::onServerHandshakeCompleteEvent(sdam::HelloRTT durationMs,
                                                                  const HostAndPort& hostAndPort,
                                                                  const BSONObj reply) {
-    IsMasterOutcome outcome(hostAndPort, reply, durationMs);
+    HelloOutcome outcome(hostAndPort, reply, durationMs);
     _topologyManager->onServerDescription(outcome);
 }
 
@@ -701,39 +801,15 @@ std::string StreamableReplicaSetMonitor::_logPrefix() {
 
 void StreamableReplicaSetMonitor::_failOutstandingWithStatus(WithLock, Status status) {
     for (const auto& query : _outstandingQueries) {
-        if (query->done)
-            continue;
-
-        query->done = true;
-        _executor->cancel(query->deadlineHandle);
-        query->promise.setError(status);
+        (void)query->tryCancel(status);
     }
     _outstandingQueries.clear();
 }
 
-bool StreamableReplicaSetMonitor::_hasMembershipChange(
-    sdam::TopologyDescriptionPtr oldDescription, sdam::TopologyDescriptionPtr newDescription) {
-
-    if (oldDescription->getServers().size() != newDescription->getServers().size())
-        return true;
-
-    for (const auto& server : oldDescription->getServers()) {
-        const auto newServer = newDescription->findServerByAddress(server->getAddress());
-        if (!newServer)
-            return true;
-        const ServerDescription& s = *server;
-        const ServerDescription& ns = **newServer;
-        if (s != ns)
-            return true;
-    }
-
-    for (const auto& server : newDescription->getServers()) {
-        auto oldServer = oldDescription->findServerByAddress(server->getAddress());
-        if (!oldServer)
-            return true;
-    }
-
-    return false;
+std::list<StreamableReplicaSetMonitor::HostQueryPtr>::iterator
+StreamableReplicaSetMonitor::_eraseQueryFromOutstandingQueries(
+    WithLock, std::list<HostQueryPtr>::iterator iter) {
+    return _outstandingQueries.erase(iter);
 }
 
 void StreamableReplicaSetMonitor::_processOutstanding(
@@ -745,53 +821,52 @@ void StreamableReplicaSetMonitor::_processOutstanding(
 
     stdx::lock_guard lock(_mutex);
 
-    bool shouldRemove;
     auto it = _outstandingQueries.begin();
+    bool hadUnresolvedQuery{false};
+
+    // Iterate through the outstanding queries and try to resolve them via calls to _getHosts. If we
+    // succeed in resolving a query, the query is removed from the list. If a query has already been
+    // canceled, or there are no results, it will be skipped. Cancellation logic elsewhere will
+    // handle removing the canceled queries from the list.
     while (it != _outstandingQueries.end()) {
         auto& query = *it;
-        shouldRemove = false;
 
-        if (query->done) {
-            shouldRemove = true;
-        } else {
-            auto result = _getHosts(topologyDescription, query->criteria);
+        // If query has not been canceled yet, try to satisfy it.
+        if (!query->hasBeenResolved()) {
+            auto result = _getHosts(topologyDescription, query->criteria, query->excludedHosts);
             if (result) {
-                _executor->cancel(query->deadlineHandle);
-                query->done = true;
-                query->promise.emplaceValue(std::move(*result));
-                const auto latency = _executor->now() - query->start;
-                LOGV2_DEBUG(433214,
-                            kLowerLogLevel,
-                            "RSM {replicaSet} finished async getHosts: {readPref} ({duration})",
-                            "RSM finished async getHosts",
-                            "replicaSet"_attr = getName(),
-                            "readPref"_attr = readPrefToStringWithMinOpTime(query->criteria),
-                            "duration"_attr = Milliseconds(latency));
-                shouldRemove = true;
+                if (query->tryResolveWithSuccess(std::move(*result))) {
+                    const auto latency = _executor->now() - query->start;
+                    LOGV2_DEBUG(433214,
+                                kLowerLogLevel,
+                                "RSM {replicaSet} finished async getHosts: {readPref} ({duration})",
+                                "RSM finished async getHosts",
+                                "replicaSet"_attr = getName(),
+                                "readPref"_attr = readPrefToStringFull(query->criteria),
+                                "duration"_attr = Milliseconds(latency));
+
+                    it = _eraseQueryFromOutstandingQueries(lock, it);
+                } else {
+                    // The query was canceled, so skip to the next entry without erasing it.
+                    ++it;
+                }
+            } else {
+                // Results were not available, so skip to the next entry without erasing it.
+                ++it;
+                hadUnresolvedQuery = true;
             }
+        } else {
+            // The query was canceled, so skip to the next entry without erasing it.
+            ++it;
         }
-
-        it = (shouldRemove) ? _outstandingQueries.erase(it) : ++it;
     }
 
-    if (_outstandingQueries.size()) {
-        // enable expedited mode
-        _isMasterMonitor->requestImmediateCheck();
+    // If there remain unresolved queries, enable expedited mode.
+    if (hadUnresolvedQuery && _serverDiscoveryMonitor) {
+        _serverDiscoveryMonitor->requestImmediateCheck();
     }
 }
 
-Status StreamableReplicaSetMonitor::_makeUnsatisfiedReadPrefError(
-    const ReadPreferenceSetting& criteria) const {
-    return Status(ErrorCodes::FailedToSatisfyReadPreference,
-                  str::stream() << "Could not find host matching read preference "
-                                << criteria.toString() << " for set " << getName());
-}
-
-Status StreamableReplicaSetMonitor::_makeReplicaSetMonitorRemovedError() const {
-    return Status(ErrorCodes::ShutdownInProgress,
-                  str::stream() << "ReplicaSetMonitor for set " << getName() << " is removed");
-}
-
 void StreamableReplicaSetMonitor::runScanForMockReplicaSet() {
     MONGO_UNREACHABLE;
 }
