@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 
 #include "mongo/platform/basic.h"
 
@@ -44,6 +43,7 @@
 #include "mongo/config.h"
 #include "mongo/crypto/mechanism_scram.h"
 #include "mongo/db/auth/address_restriction.h"
+#include "mongo/db/auth/auth_types_gen.h"
 #include "mongo/db/auth/authorization_manager_global_parameters_gen.h"
 #include "mongo/db/auth/authorization_manager_impl_parameters_gen.h"
 #include "mongo/db/auth/authorization_session_impl.h"
@@ -51,48 +51,81 @@
 #include "mongo/db/auth/sasl_options.h"
 #include "mongo/db/auth/user_document_parser.h"
 #include "mongo/db/auth/user_management_commands_parser.h"
+#include "mongo/db/curop.h"
 #include "mongo/db/global_settings.h"
 #include "mongo/db/mongod_options.h"
 #include "mongo/logv2/log.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/net/ssl_peer_info.h"
 #include "mongo/util/net/ssl_types.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
+
+
 namespace mongo {
 namespace {
 
+std::shared_ptr<UserHandle> createSystemUserHandle() {
+    auto user = std::make_shared<UserHandle>(User(UserName("__system", "local")));
+
+    ActionSet allActions;
+    allActions.addAllActions();
+    PrivilegeVector privileges;
+    auth::generateUniversalPrivileges(&privileges);
+    (*user)->addPrivileges(privileges);
+
+    if (internalSecurity.credentials) {
+        (*user)->setCredentials(internalSecurity.credentials.value());
+    }
+
+    return user;
+}
+
+class ClusterNetworkRestrictionManagerImpl : public ClusterNetworkRestrictionManager {
+public:
+    static void configureRestrictions(std::shared_ptr<UserHandle> user) {
+        const auto allowlistedClusterNetwork =
+            std::atomic_load(&mongodGlobalParams.allowlistedClusterNetwork);  // NOLINT
+        if (allowlistedClusterNetwork) {
+            auto restriction =
+                std::make_unique<ClientSourceRestriction>(*allowlistedClusterNetwork);
+            auto restrictionSet = std::make_unique<RestrictionSet<>>(std::move(restriction));
+            auto restrictionDocument =
+                std::make_unique<RestrictionDocument<>>(std::move(restrictionSet));
+
+            RestrictionDocuments clusterAllowList(std::move(restrictionDocument));
+            (*user)->setRestrictions(clusterAllowList);
+        }
+    }
+
+    void updateClusterNetworkRestrictions() override {
+        auto user = createSystemUserHandle();
+        configureRestrictions(user);
+        auto originalUser = internalSecurity.setUser(user);
+        (*originalUser)->invalidate();
+    }
+};
+
 MONGO_INITIALIZER_GENERAL(SetupInternalSecurityUser,
                           ("EndStartupOptionStorage"),
                           ("CreateAuthorizationManager"))
 (InitializerContext* const context) try {
-    UserHandle user(User(UserName("__system", "local")));
-
-    ActionSet allActions;
-    allActions.addAllActions();
-    PrivilegeVector privileges;
-    RoleGraph::generateUniversalPrivileges(&privileges);
-    user->addPrivileges(privileges);
-
-    if (mongodGlobalParams.whitelistedClusterNetwork) {
-        const auto& whitelist = *mongodGlobalParams.whitelistedClusterNetwork;
-
-        auto restriction = std::make_unique<ClientSourceRestriction>(whitelist);
-        auto restrictionSet = std::make_unique<RestrictionSet<>>(std::move(restriction));
-        auto restrictionDocument =
-            std::make_unique<RestrictionDocument<>>(std::move(restrictionSet));
-
-        RestrictionDocuments clusterWhiteList(std::move(restrictionDocument));
-
-        user->setRestrictions(std::move(clusterWhiteList));
-    }
-
-    internalSecurity.user = user;
-
-    return Status::OK();
+    auto user = createSystemUserHandle();
+    ClusterNetworkRestrictionManagerImpl::configureRestrictions(user);
+    internalSecurity.setUser(user);
 } catch (...) {
-    return exceptionToStatus();
+    uassertStatusOK(exceptionToStatus());
 }
 
+ServiceContext::ConstructorActionRegisterer setClusterNetworkRestrictionManager{
+    "SetClusterNetworkRestrictionManager", [](ServiceContext* service) {
+        std::unique_ptr<ClusterNetworkRestrictionManager> manager =
+            std::make_unique<ClusterNetworkRestrictionManagerImpl>();
+        ClusterNetworkRestrictionManager::set(service, std::move(manager));
+    }};
+
 class PinnedUserSetParameter {
 public:
     void append(BSONObjBuilder& b, const std::string& name) const {
@@ -107,7 +140,7 @@ public:
 
     Status set(const BSONElement& newValueElement) {
         if (newValueElement.type() == String) {
-            return setFromString(newValueElement.valuestrsafe());
+            return setFromString(newValueElement.str());
         } else if (newValueElement.type() == Array) {
             auto array = static_cast<BSONArray>(newValueElement.embeddedObject());
             std::vector<UserName> out;
@@ -173,7 +206,7 @@ public:
 private:
     Status _checkForSystemUser(const std::vector<UserName>& names) {
         if (std::any_of(names.begin(), names.end(), [&](const UserName& userName) {
-                return (userName == internalSecurity.user->getName());
+                return (userName == (*internalSecurity.getUser())->getName());
             })) {
             return {ErrorCodes::BadValue,
                     "Cannot set __system as a pinned user, it is always pinned"};
@@ -213,7 +246,7 @@ bool loggedCommandOperatesOnAuthzData(const NamespaceString& nss, const BSONObj&
         return true;
     } else if (cmdName == "renameCollection") {
         const NamespaceString fromNamespace(cmdObj.firstElement().valueStringDataSafe());
-        const NamespaceString toNamespace(cmdObj["to"].valueStringDataSafe());
+        const NamespaceString toNamespace(cmdObj.getStringField("to"));
 
         if (fromNamespace.isAdminDB() || toNamespace.isAdminDB()) {
             return isAuthzCollection(fromNamespace.coll()) || isAuthzCollection(toNamespace.coll());
@@ -229,17 +262,21 @@ bool loggedCommandOperatesOnAuthzData(const NamespaceString& nss, const BSONObj&
     }
 }
 
-bool appliesToAuthzData(const char* op, const NamespaceString& nss, const BSONObj& o) {
-    switch (*op) {
+bool appliesToAuthzData(StringData op, const NamespaceString& nss, const BSONObj& o) {
+    if (op.empty()) {
+        return true;
+    }
+
+    switch (op[0]) {
         case 'i':
         case 'u':
         case 'd':
-            if (op[1] != '\0')
+            if (op.size() != 1) {
                 return false;  // "db" op type
+            }
             return isAuthzNamespace(nss);
         case 'c':
             return loggedCommandOperatesOnAuthzData(nss, o);
-            break;
         case 'n':
             return false;
         default:
@@ -247,46 +284,6 @@ bool appliesToAuthzData(const char* op, const NamespaceString& nss, const BSONOb
     }
 }
 
-/**
- * Parses privDoc and fully initializes the user object (credentials, roles, and privileges) with
- * the information extracted from the privilege document.
- */
-Status initializeUserFromPrivilegeDocument(User* user, const BSONObj& privDoc) {
-    V2UserDocumentParser parser;
-    std::string userName = parser.extractUserNameFromUserDocument(privDoc);
-    if (userName != user->getName().getUser()) {
-        return Status(ErrorCodes::BadValue,
-                      str::stream() << "User name from privilege document \"" << userName
-                                    << "\" doesn't match name of provided User \""
-                                    << user->getName().getUser() << "\"");
-    }
-
-    user->setID(parser.extractUserIDFromUserDocument(privDoc));
-
-    Status status = parser.initializeUserCredentialsFromUserDocument(user, privDoc);
-    if (!status.isOK()) {
-        return status;
-    }
-    status = parser.initializeUserRolesFromUserDocument(privDoc, user);
-    if (!status.isOK()) {
-        return status;
-    }
-    status = parser.initializeUserIndirectRolesFromUserDocument(privDoc, user);
-    if (!status.isOK()) {
-        return status;
-    }
-    status = parser.initializeUserPrivilegesFromUserDocument(privDoc, user);
-    if (!status.isOK()) {
-        return status;
-    }
-    status = parser.initializeAuthenticationRestrictionsFromUserDocument(privDoc, user);
-    if (!status.isOK()) {
-        return status;
-    }
-
-    return Status::OK();
-}
-
 /**
  * Returns true if roles for this user were provided by the client, and can be obtained from
  * the connection.
@@ -319,6 +316,40 @@ std::unique_ptr<AuthorizationManager> authorizationManagerCreateImpl(
 auto authorizationManagerCreateRegistration =
     MONGO_WEAK_FUNCTION_REGISTRATION(AuthorizationManager::create, authorizationManagerCreateImpl);
 
+MONGO_FAIL_POINT_DEFINE(waitForUserCacheInvalidation);
+void handleWaitForUserCacheInvalidation(OperationContext* opCtx, const UserHandle& user) {
+    auto fp = waitForUserCacheInvalidation.scopedIf([&](const auto& bsonData) {
+        IDLParserContext ctx("waitForUserCacheInvalidation");
+        auto data = WaitForUserCacheInvalidationFailPoint::parse(ctx, bsonData);
+
+        const auto& blockedUserName = data.getUserName();
+        return blockedUserName == user->getName();
+    });
+
+    if (!fp.isActive()) {
+        return;
+    }
+
+    // Since we do not have notifications from both the user cache and the fail point itself,
+    // loop until our condition is satisfied. To avoid this loop, we would need a way to union
+    // notifications from both. This may be possible with a CancellationToken and a condition
+    // variable or with some novel Notifiable/Waitable-like type that synthesizes multiple
+    // notifications.
+    constexpr auto kCheckPeriod = Milliseconds{1};
+    auto m = MONGO_MAKE_LATCH();
+    auto cv = stdx::condition_variable{};
+    auto pred = [&] { return !fp.isStillEnabled() || !user.isValid(); };
+    auto waitOneCycle = [&] {
+        auto lk = stdx::unique_lock(m);
+        return !opCtx->waitForConditionOrInterruptFor(cv, lk, kCheckPeriod, pred);
+    };
+
+    while (waitOneCycle()) {
+        // Not yet finished.
+    }
+}
+
+
 }  // namespace
 
 int authorizationManagerCacheSize;
@@ -415,26 +446,37 @@ Status AuthorizationManagerImpl::getUserDescription(OperationContext* opCtx,
     return _externalState->getUserDescription(opCtx, UserRequest(userName, boost::none), result);
 }
 
-Status AuthorizationManagerImpl::getRoleDescription(OperationContext* opCtx,
-                                                    const RoleName& roleName,
-                                                    PrivilegeFormat privileges,
-                                                    AuthenticationRestrictionsFormat restrictions,
-                                                    BSONObj* result) {
-    return _externalState->getRoleDescription(opCtx, roleName, privileges, restrictions, result);
+Status AuthorizationManagerImpl::rolesExist(OperationContext* opCtx,
+                                            const std::vector<RoleName>& roleNames) {
+    return _externalState->rolesExist(opCtx, roleNames);
+}
+
+StatusWith<AuthorizationManager::ResolvedRoleData> AuthorizationManagerImpl::resolveRoles(
+    OperationContext* opCtx, const std::vector<RoleName>& roleNames, ResolveRoleOption option) {
+    return _externalState->resolveRoles(opCtx, roleNames, option);
 }
 
 Status AuthorizationManagerImpl::getRolesDescription(OperationContext* opCtx,
                                                      const std::vector<RoleName>& roleName,
                                                      PrivilegeFormat privileges,
                                                      AuthenticationRestrictionsFormat restrictions,
-                                                     BSONObj* result) {
+                                                     std::vector<BSONObj>* result) {
     return _externalState->getRolesDescription(opCtx, roleName, privileges, restrictions, result);
 }
 
 
+Status AuthorizationManagerImpl::getRolesAsUserFragment(
+    OperationContext* opCtx,
+    const std::vector<RoleName>& roleName,
+    AuthenticationRestrictionsFormat restrictions,
+    BSONObj* result) {
+    return _externalState->getRolesAsUserFragment(opCtx, roleName, restrictions, result);
+}
+
+
 Status AuthorizationManagerImpl::getRoleDescriptionsForDB(
     OperationContext* opCtx,
-    StringData dbname,
+    const DatabaseName& dbname,
     PrivilegeFormat privileges,
     AuthenticationRestrictionsFormat restrictions,
     bool showBuiltinRoles,
@@ -443,10 +485,16 @@ Status AuthorizationManagerImpl::getRoleDescriptionsForDB(
         opCtx, dbname, privileges, restrictions, showBuiltinRoles, result);
 }
 
+namespace {
+MONGO_FAIL_POINT_DEFINE(authUserCacheBypass);
+MONGO_FAIL_POINT_DEFINE(authUserCacheSleep);
+}  // namespace
+
 StatusWith<UserHandle> AuthorizationManagerImpl::acquireUser(OperationContext* opCtx,
                                                              const UserName& userName) try {
-    if (userName == internalSecurity.user->getName()) {
-        return internalSecurity.user;
+    auto systemUser = internalSecurity.getUser();
+    if (userName == (*systemUser)->getName()) {
+        return *systemUser;
     }
 
     UserRequest request(userName, boost::none);
@@ -466,7 +514,30 @@ StatusWith<UserHandle> AuthorizationManagerImpl::acquireUser(OperationContext* o
     }
 #endif
 
+    if (authUserCacheBypass.shouldFail()) {
+        // Bypass cache and force a fresh load of the user.
+        auto loadedUser = uassertStatusOK(_externalState->getUserObject(opCtx, request));
+        // We have to inject into the cache in order to get a UserHandle.
+        auto userHandle =
+            _userCache.insertOrAssignAndGet(request, std::move(loadedUser), Date_t::now());
+        invariant(userHandle);
+        LOGV2_DEBUG(4859401, 1, "Bypassing user cache to load user", "user"_attr = userName);
+        return userHandle;
+    }
+
+    // Track wait time and user cache access statistics for the current op for logging. An extra
+    // second of delay is added via the failpoint for testing.
+    UserAcquisitionStatsHandle userAcquisitionStatsHandle =
+        UserAcquisitionStatsHandle(CurOp::get(opCtx)->getMutableUserAcquisitionStats(),
+                                   opCtx->getServiceContext()->getTickSource(),
+                                   kCache);
+    if (authUserCacheSleep.shouldFail()) {
+        sleepsecs(1);
+    }
+
     auto cachedUser = _userCache.acquire(opCtx, request);
+
+    userAcquisitionStatsHandle.recordTimerEnd();
     invariant(cachedUser);
 
     LOGV2_DEBUG(20226, 1, "Returning user from cache", "user"_attr = userName);
@@ -475,17 +546,25 @@ StatusWith<UserHandle> AuthorizationManagerImpl::acquireUser(OperationContext* o
     return ex.toStatus();
 }
 
-StatusWith<UserHandle> AuthorizationManagerImpl::acquireUserForSessionRefresh(
-    OperationContext* opCtx, const UserName& userName, const User::UserId& uid) {
+StatusWith<UserHandle> AuthorizationManagerImpl::reacquireUser(OperationContext* opCtx,
+                                                               const UserHandle& user) {
+    const UserName& userName = user->getName();
+    handleWaitForUserCacheInvalidation(opCtx, user);
+    if (user.isValid() && !user->isInvalidated()) {
+        return user;
+    }
+
+    // Make a good faith effort to acquire an up-to-date user object, since the one
+    // we've cached is marked "out-of-date."
     auto swUserHandle = acquireUser(opCtx, userName);
     if (!swUserHandle.isOK()) {
         return swUserHandle.getStatus();
     }
 
     auto ret = std::move(swUserHandle.getValue());
-    if (uid != ret->getID()) {
+    if (user->getID() != ret->getID()) {
         return {ErrorCodes::UserNotFound,
-                str::stream() << "User id from privilege document '" << userName.toString()
+                str::stream() << "User id from privilege document '" << userName
                               << "' does not match user id in session."};
     }
 
@@ -527,7 +606,7 @@ void AuthorizationManagerImpl::_pinnedUsersThreadRoutine() noexcept try {
             _pinnedUsersCond, lk, timeout, [&] { return _usersToPin.has_value(); });
 
         if (waitRes) {
-            usersToPin = std::move(_usersToPin.get());
+            usersToPin = std::move(_usersToPin.value());
             _usersToPin = boost::none;
         }
         lk.unlock();
@@ -582,7 +661,7 @@ void AuthorizationManagerImpl::_pinnedUsersThreadRoutine() noexcept try {
                 if (status != ErrorCodes::UserNotFound) {
                     LOGV2_WARNING(20239,
                                   "Unable to fetch pinned user",
-                                  "user"_attr = userName.toString(),
+                                  "user"_attr = userName,
                                   "error"_attr = status);
                 } else {
                     LOGV2_DEBUG(20233, 2, "Pinned user not found", "user"_attr = userName);
@@ -602,15 +681,31 @@ void AuthorizationManagerImpl::invalidateUserByName(OperationContext* opCtx,
     _authSchemaVersionCache.invalidateAll();
     // Invalidate the named User, assuming no externally provided roles. When roles are defined
     // externally, there exists no user document which may become invalid.
-    _userCache.invalidate(UserRequest(userName, boost::none));
+    _userCache.invalidateKey(UserRequest(userName, boost::none));
 }
 
-void AuthorizationManagerImpl::invalidateUsersFromDB(OperationContext* opCtx, StringData dbname) {
+void AuthorizationManagerImpl::invalidateUsersFromDB(OperationContext* opCtx,
+                                                     const DatabaseName& dbname) {
     LOGV2_DEBUG(20236, 2, "Invalidating all users from database", "database"_attr = dbname);
     _updateCacheGeneration();
     _authSchemaVersionCache.invalidateAll();
-    _userCache.invalidateIf(
-        [&](const UserRequest& userRequest) { return userRequest.name.getDB() == dbname; });
+    _userCache.invalidateKeyIf([&](const UserRequest& userRequest) {
+        return userRequest.name.getDatabaseName() == dbname;
+    });
+}
+
+void AuthorizationManagerImpl::invalidateUsersByTenant(OperationContext* opCtx,
+                                                       const boost::optional<TenantId>& tenant) {
+    if (!tenant) {
+        invalidateUserCache(opCtx);
+        return;
+    }
+
+    LOGV2_DEBUG(6323600, 2, "Invalidating tenant users", "tenant"_attr = tenant);
+    _updateCacheGeneration();
+    _authSchemaVersionCache.invalidateAll();
+    _userCache.invalidateKeyIf(
+        [&](const UserRequest& userRequest) { return userRequest.name.getTenant() == tenant; });
 }
 
 void AuthorizationManagerImpl::invalidateUserCache(OperationContext* opCtx) {
@@ -620,6 +715,47 @@ void AuthorizationManagerImpl::invalidateUserCache(OperationContext* opCtx) {
     _userCache.invalidateAll();
 }
 
+Status AuthorizationManagerImpl::refreshExternalUsers(OperationContext* opCtx) {
+    LOGV2_DEBUG(5914801, 2, "Refreshing all users from the $external database");
+    // First, get a snapshot of the UserHandles in the cache.
+    auto cachedUsers =
+        _userCache.peekLatestCachedIf([&](const UserRequest& userRequest, const User&) {
+            return userRequest.name.getDB() == "$external"_sd;
+        });
+
+    // Then, retrieve the corresponding Users from the backing store for users in the $external
+    // database. Compare each of these user objects with the cached user object and call
+    // insertOrAssign if they differ.
+    bool isRefreshed{false};
+    for (const auto& cachedUser : cachedUsers) {
+        UserRequest request(cachedUser->getName(), boost::none);
+        auto storedUserStatus = _externalState->getUserObject(opCtx, request);
+        if (!storedUserStatus.isOK()) {
+            // If the user simply is not found, then just invalidate the cached user and continue.
+            if (storedUserStatus.getStatus().code() == ErrorCodes::UserNotFound) {
+                _userCache.invalidateKey(request);
+                continue;
+            } else {
+                return storedUserStatus.getStatus();
+            }
+        }
+
+        if (cachedUser->hasDifferentRoles(storedUserStatus.getValue())) {
+            _userCache.insertOrAssign(
+                request, std::move(storedUserStatus.getValue()), Date_t::now());
+            isRefreshed = true;
+        }
+    }
+
+    // If any entries were refreshed, then the cache generation must be bumped for mongos to refresh
+    // its cache.
+    if (isRefreshed) {
+        _updateCacheGeneration();
+    }
+
+    return Status::OK();
+}
+
 Status AuthorizationManagerImpl::initialize(OperationContext* opCtx) {
     Status status = _externalState->initialize(opCtx);
     if (!status.isOK())
@@ -631,7 +767,7 @@ Status AuthorizationManagerImpl::initialize(OperationContext* opCtx) {
 }
 
 void AuthorizationManagerImpl::logOp(OperationContext* opCtx,
-                                     const char* op,
+                                     StringData op,
                                      const NamespaceString& nss,
                                      const BSONObj& o,
                                      const BSONObj* o2) {
@@ -657,22 +793,25 @@ AuthorizationManagerImpl::AuthSchemaVersionCache::AuthSchemaVersionCache(
     ServiceContext* service,
     ThreadPoolInterface& threadPool,
     AuthzManagerExternalState* externalState)
-    : ReadThroughCache(
-          _mutex,
-          service,
-          threadPool,
-          [this](OperationContext* opCtx, int unusedKey) { return _lookup(opCtx, unusedKey); },
-          1 /* cacheSize */),
+    : ReadThroughCache(_mutex,
+                       service,
+                       threadPool,
+                       [this](OperationContext* opCtx, int key, const ValueHandle& cachedValue) {
+                           return _lookup(opCtx, key, cachedValue);
+                       },
+                       1 /* cacheSize */),
       _externalState(externalState) {}
 
-boost::optional<int> AuthorizationManagerImpl::AuthSchemaVersionCache::_lookup(
-    OperationContext* opCtx, int unusedKey) {
+AuthorizationManagerImpl::AuthSchemaVersionCache::LookupResult
+AuthorizationManagerImpl::AuthSchemaVersionCache::_lookup(OperationContext* opCtx,
+                                                          int unusedKey,
+                                                          const ValueHandle& unusedCachedValue) {
     invariant(unusedKey == 0);
 
     int authzVersion;
     uassertStatusOK(_externalState->getStoredAuthorizationVersion(opCtx, &authzVersion));
 
-    return authzVersion;
+    return LookupResult(authzVersion);
 }
 
 AuthorizationManagerImpl::UserCacheImpl::UserCacheImpl(
@@ -684,15 +823,17 @@ AuthorizationManagerImpl::UserCacheImpl::UserCacheImpl(
     : UserCache(_mutex,
                 service,
                 threadPool,
-                [this](OperationContext* opCtx, const UserRequest& userReq) {
-                    return _lookup(opCtx, userReq);
+                [this](OperationContext* opCtx, const UserRequest& userReq, UserHandle cachedUser) {
+                    return _lookup(opCtx, userReq, cachedUser);
                 },
                 cacheSize),
       _authSchemaVersionCache(authSchemaVersionCache),
       _externalState(externalState) {}
 
-boost::optional<User> AuthorizationManagerImpl::UserCacheImpl::_lookup(OperationContext* opCtx,
-                                                                       const UserRequest& userReq) {
+AuthorizationManagerImpl::UserCacheImpl::LookupResult
+AuthorizationManagerImpl::UserCacheImpl::_lookup(OperationContext* opCtx,
+                                                 const UserRequest& userReq,
+                                                 const UserHandle& unusedCachedUser) {
     LOGV2_DEBUG(20238, 1, "Getting user record", "user"_attr = userReq.name);
 
     // Number of times to retry a user document that fetches due to transient AuthSchemaIncompatible
@@ -709,14 +850,8 @@ boost::optional<User> AuthorizationManagerImpl::UserCacheImpl::_lookup(Operation
         switch (authzVersion) {
             case schemaVersion28SCRAM:
             case schemaVersion26Final:
-            case schemaVersion26Upgrade: {
-                BSONObj userObj;
-                uassertStatusOK(_externalState->getUserDescription(opCtx, userReq, &userObj));
-
-                User user(userReq.name);
-                uassertStatusOK(initializeUserFromPrivilegeDocument(&user, userObj));
-                return user;
-            }
+            case schemaVersion26Upgrade:
+                return LookupResult(uassertStatusOK(_externalState->getUserObject(opCtx, userReq)));
             case schemaVersion24:
                 _authSchemaVersionCache->invalidateAll();
 
