@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include "mongo/platform/basic.h"
 
@@ -38,16 +37,20 @@
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/index_consistency.h"
+#include "mongo/db/catalog/index_key_validate.h"
 #include "mongo/db/catalog/validate_adaptor.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index/index_access_method.h"
 #include "mongo/db/operation_context.h"
-#include "mongo/db/storage/durable_catalog.h"
-#include "mongo/db/views/view_catalog.h"
+#include "mongo/db/record_id_helpers.h"
+#include "mongo/db/storage/key_string.h"
 #include "mongo/logv2/log.h"
 #include "mongo/util/fail_point.h"
 #include "mongo/util/scopeguard.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
+
+
 namespace mongo {
 
 using logv2::LogComponent;
@@ -59,8 +62,6 @@ namespace CollectionValidation {
 
 namespace {
 
-using ValidateResultsMap = std::map<string, ValidateResults>;
-
 // Indicates whether the failpoint turned on by testing has been reached.
 AtomicWord<bool> _validationIsPausedForTest{false};
 
@@ -72,7 +73,6 @@ AtomicWord<bool> _validationIsPausedForTest{false};
  */
 void _validateIndexesInternalStructure(OperationContext* opCtx,
                                        ValidateState* validateState,
-                                       ValidateResultsMap* indexNsResultsMap,
                                        ValidateResults* results) {
     // Need to use the IndexCatalog here because the 'validateState->indexes' object hasn't been
     // constructed yet. It must be initialized to ensure we're validating all indexes.
@@ -92,26 +92,30 @@ void _validateIndexesInternalStructure(OperationContext* opCtx,
                       {LogComponent::kIndex},
                       "Validating internal structure",
                       "index"_attr = descriptor->indexName(),
-                      "namespace"_attr = descriptor->parentNS());
+                      "namespace"_attr = validateState->nss());
 
-        ValidateResults& curIndexResults = (*indexNsResultsMap)[descriptor->indexName()];
+        auto& curIndexResults = (results->indexResultsMap)[descriptor->indexName()];
 
-        iam->validate(opCtx, nullptr, &curIndexResults);
+        int64_t numValidated;
+        iam->validate(opCtx, &numValidated, &curIndexResults);
 
         if (!curIndexResults.valid) {
             results->valid = false;
         }
+
+        curIndexResults.keysTraversedFromFullValidate = numValidated;
     }
 }
 
 /**
  * Validates each index in the Index Catalog using the cursors in 'indexCursors'.
+ *
+ * If 'level' is kValidateFull, then we will compare new index entry counts with a previously taken
+ * count saved in 'numIndexKeysPerIndex'.
  */
 void _validateIndexes(OperationContext* opCtx,
                       ValidateState* validateState,
-                      BSONObjBuilder* keysPerIndex,
                       ValidateAdaptor* indexValidator,
-                      ValidateResultsMap* indexNsResultsMap,
                       ValidateResults* results) {
     // Validate Indexes, checking for mismatch between index entries and collection records.
     for (const auto& index : validateState->getIndexes()) {
@@ -123,14 +127,41 @@ void _validateIndexes(OperationContext* opCtx,
                       {LogComponent::kIndex},
                       "Validating index consistency",
                       "index"_attr = descriptor->indexName(),
-                      "namespace"_attr = descriptor->parentNS());
+                      "namespace"_attr = validateState->nss());
 
-        ValidateResults& curIndexResults = (*indexNsResultsMap)[descriptor->indexName()];
         int64_t numTraversedKeys;
-        indexValidator->traverseIndex(opCtx, index.get(), &numTraversedKeys, &curIndexResults);
+        indexValidator->traverseIndex(opCtx, index.get(), &numTraversedKeys, results);
+
+        auto& curIndexResults = (results->indexResultsMap)[descriptor->indexName()];
+        curIndexResults.keysTraversed = numTraversedKeys;
+
+        // If we are performing a full index validation, we have information on the number of index
+        // keys validated in _validateIndexesInternalStructure (when we validated the internal
+        // structure of the index). Check if this is consistent with 'numTraversedKeys' from
+        // traverseIndex above.
+        if (validateState->isFullIndexValidation()) {
+            invariant(opCtx->lockState()->isCollectionLockedForMode(validateState->nss(), MODE_X));
+
+            // The number of keys counted in _validateIndexesInternalStructure, when checking the
+            // internal structure of the index.
+            const int64_t numIndexKeys = curIndexResults.keysTraversedFromFullValidate;
+
+            // Check if currIndexResults is valid to ensure that this index is not corrupted or
+            // comprised (which was set in _validateIndexesInternalStructure). If the index is
+            // corrupted, there is no use in checking if the traversal yielded the same key count.
+            if (curIndexResults.valid) {
+                if (numIndexKeys != numTraversedKeys) {
+                    curIndexResults.valid = false;
+                    string msg = str::stream()
+                        << "number of traversed index entries (" << numTraversedKeys
+                        << ") does not match the number of expected index entries (" << numIndexKeys
+                        << ")";
+                    results->errors.push_back(msg);
+                    results->valid = false;
+                }
+            }
+        }
 
-        keysPerIndex->appendNumber(descriptor->indexName(),
-                                   static_cast<long long>(numTraversedKeys));
         if (!curIndexResults.valid) {
             results->valid = false;
         }
@@ -145,7 +176,6 @@ void _gatherIndexEntryErrors(OperationContext* opCtx,
                              ValidateState* validateState,
                              IndexConsistency* indexConsistency,
                              ValidateAdaptor* indexValidator,
-                             ValidateResultsMap* indexNsResultsMap,
                              ValidateResults* result) {
     indexConsistency->setSecondPhase();
     if (!indexConsistency->limitMemoryUsageForSecondPhase(result)) {
@@ -160,6 +190,7 @@ void _gatherIndexEntryErrors(OperationContext* opCtx,
     {
         ValidateResults tempValidateResults;
         BSONObjBuilder tempBuilder;
+
         indexValidator->traverseRecordStore(opCtx, &tempValidateResults, &tempBuilder);
     }
 
@@ -182,12 +213,22 @@ void _gatherIndexEntryErrors(OperationContext* opCtx,
         indexValidator->traverseIndex(opCtx,
                                       index.get(),
                                       /*numTraversedKeys=*/nullptr,
-                                      /*ValidateResults=*/nullptr);
+                                      result);
+    }
+
+    if (result->numRemovedExtraIndexEntries > 0) {
+        result->warnings.push_back(str::stream()
+                                   << "Removed " << result->numRemovedExtraIndexEntries
+                                   << " extra index entries.");
+    }
+
+    if (validateState->fixErrors()) {
+        indexConsistency->repairMissingIndexEntries(opCtx, result);
     }
 
     LOGV2_OPTIONS(20301, {LogComponent::kIndex}, "Finished traversing through all the indexes");
 
-    indexConsistency->addIndexEntryErrors(indexNsResultsMap, result);
+    indexConsistency->addIndexEntryErrors(result);
 }
 
 void _validateIndexKeyCount(OperationContext* opCtx,
@@ -196,32 +237,16 @@ void _validateIndexKeyCount(OperationContext* opCtx,
                             ValidateResultsMap* indexNsResultsMap) {
     for (const auto& index : validateState->getIndexes()) {
         const IndexDescriptor* descriptor = index->descriptor();
-        ValidateResults& curIndexResults = (*indexNsResultsMap)[descriptor->indexName()];
+        auto& curIndexResults = (*indexNsResultsMap)[descriptor->indexName()];
 
         if (curIndexResults.valid) {
-            indexValidator->validateIndexKeyCount(descriptor, curIndexResults);
+            indexValidator->validateIndexKeyCount(opCtx, index.get(), curIndexResults);
         }
     }
 }
 
-void _printIndexSpec(const ValidateState* validateState, StringData indexName) {
-    auto& indexes = validateState->getIndexes();
-    auto indexEntry =
-        std::find_if(indexes.begin(),
-                     indexes.end(),
-                     [&](const std::shared_ptr<const IndexCatalogEntry> indexEntry) -> bool {
-                         return indexEntry->descriptor()->indexName() == indexName;
-                     });
-    if (indexEntry != indexes.end()) {
-        auto indexSpec = (*indexEntry)->descriptor()->infoObj();
-        LOGV2_ERROR(7463100, "Index failed validation", "spec"_attr = indexSpec);
-    }
-}
-
 void _reportValidationResults(OperationContext* opCtx,
                               ValidateState* validateState,
-                              ValidateResultsMap* indexNsResultsMap,
-                              BSONObjBuilder* keysPerIndex,
                               ValidateResults* results,
                               BSONObjBuilder* output) {
     BSONObjBuilder indexDetails;
@@ -232,21 +257,21 @@ void _reportValidationResults(OperationContext* opCtx,
         invariant(opCtx->lockState()->isCollectionLockedForMode(validateState->nss(), MODE_X));
     }
 
+    BSONObjBuilder keysPerIndex;
+
     // Report detailed index validation results gathered when using {full: true} for validated
     // indexes.
-    int nIndexes = indexNsResultsMap->size();
-    for (const auto& [indexName, vr] : *indexNsResultsMap) {
-        if (!vr.valid) {
-            results->valid = false;
-            _printIndexSpec(validateState, indexName);
+    for (const auto& index : validateState->getIndexes()) {
+        const std::string indexName = index->descriptor()->indexName();
+        auto& indexResultsMap = results->indexResultsMap;
+        if (indexResultsMap.find(indexName) == indexResultsMap.end()) {
+            continue;
         }
 
-        if (validateState->getSkippedIndexes().contains(indexName)) {
-            // Index internal state was checked and cleared, so it was reported in indexResultsMap,
-            // but we did not verify the index contents against the collection, so we should exclude
-            // it from this report.
-            --nIndexes;
-            continue;
+        auto& vr = indexResultsMap.at(indexName);
+
+        if (!vr.valid) {
+            results->valid = false;
         }
 
         BSONObjBuilder bob(indexDetails.subobjStart(indexName));
@@ -260,29 +285,28 @@ void _reportValidationResults(OperationContext* opCtx,
             bob.append("errors", vr.errors);
         }
 
+
+        keysPerIndex.appendNumber(indexName, static_cast<long long>(vr.keysTraversed));
+
         results->warnings.insert(results->warnings.end(), vr.warnings.begin(), vr.warnings.end());
         results->errors.insert(results->errors.end(), vr.errors.begin(), vr.errors.end());
     }
 
-    output->append("nIndexes", nIndexes);
-    output->append("keysPerIndex", keysPerIndex->done());
+    output->append("nIndexes", static_cast<int>(validateState->getIndexes().size()));
+    output->append("keysPerIndex", keysPerIndex.done());
     output->append("indexDetails", indexDetails.done());
 }
 
 void _reportInvalidResults(OperationContext* opCtx,
                            ValidateState* validateState,
-                           ValidateResultsMap* indexNsResultsMap,
-                           BSONObjBuilder* keysPerIndex,
                            ValidateResults* results,
-                           BSONObjBuilder* output,
-                           const string uuidString) {
-    _reportValidationResults(
-        opCtx, validateState, indexNsResultsMap, keysPerIndex, results, output);
+                           BSONObjBuilder* output) {
+    _reportValidationResults(opCtx, validateState, results, output);
     LOGV2_OPTIONS(20302,
                   {LogComponent::kIndex},
                   "Validation complete -- Corruption found",
-                  "namespace"_attr = validateState->nss(),
-                  "uuid"_attr = uuidString);
+                  logAttrs(validateState->nss()),
+                  logAttrs(validateState->uuid()));
 }
 
 template <typename T>
@@ -295,12 +319,60 @@ void addErrorIfUnequal(T stored, T cached, StringData name, ValidateResults* res
     }
 }
 
+void addErrorIfUnequal(boost::optional<ValidationLevelEnum> stored,
+                       boost::optional<ValidationLevelEnum> cached,
+                       StringData name,
+                       ValidateResults* results) {
+    addErrorIfUnequal(ValidationLevel_serializer(validationLevelOrDefault(stored)),
+                      ValidationLevel_serializer(validationLevelOrDefault(cached)),
+                      name,
+                      results);
+}
+
+void addErrorIfUnequal(boost::optional<ValidationActionEnum> stored,
+                       boost::optional<ValidationActionEnum> cached,
+                       StringData name,
+                       ValidateResults* results) {
+    addErrorIfUnequal(ValidationAction_serializer(validationActionOrDefault(stored)),
+                      ValidationAction_serializer(validationActionOrDefault(cached)),
+                      name,
+                      results);
+}
+
+std::string multikeyPathsToString(MultikeyPaths paths) {
+    str::stream builder;
+    builder << "[";
+    auto pathIt = paths.begin();
+    while (true) {
+        builder << "{";
+
+        auto pathSet = *pathIt;
+        auto setIt = pathSet.begin();
+        while (true) {
+            builder << *setIt++;
+            if (setIt == pathSet.end()) {
+                break;
+            } else {
+                builder << ",";
+            }
+        }
+        builder << "}";
+
+        if (++pathIt == paths.end()) {
+            break;
+        } else {
+            builder << ",";
+        }
+    }
+    builder << "]";
+    return builder;
+}
+
 void _validateCatalogEntry(OperationContext* opCtx,
                            ValidateState* validateState,
                            ValidateResults* results) {
-    Collection* collection = validateState->getCollection();
-    CollectionOptions options =
-        DurableCatalog::get(opCtx)->getCollectionOptions(opCtx, collection->getCatalogId());
+    const auto& collection = validateState->getCollection();
+    const auto& options = collection->getCollectionOptions();
     if (options.uuid) {
         addErrorIfUnequal(*(options.uuid), validateState->uuid(), "UUID", results);
     } else {
@@ -319,14 +391,12 @@ void _validateCatalogEntry(OperationContext* opCtx,
     BSONObj validatorDoc = collection->getValidatorDoc();
     addErrorIfUnequal(options.validator.toString(), validatorDoc.toString(), "validator", results);
     if (!options.validator.isEmpty() && !validatorDoc.isEmpty()) {
-        addErrorIfUnequal(options.validationAction.length() ? options.validationAction : "error",
-                          collection->getValidationAction().toString(),
+        addErrorIfUnequal(options.validationAction,
+                          collection->getValidationAction(),
                           "validation action",
                           results);
-        addErrorIfUnequal(options.validationLevel.length() ? options.validationLevel : "strict",
-                          collection->getValidationLevel().toString(),
-                          "validation level",
-                          results);
+        addErrorIfUnequal(
+            options.validationLevel, collection->getValidationLevel(), "validation level", results);
     }
 
     addErrorIfUnequal(options.isView(), false, "is a view", results);
@@ -337,24 +407,49 @@ void _validateCatalogEntry(OperationContext* opCtx,
                                                 << options.toBSON());
     }
 
-    std::vector<std::string> indexes;
-    DurableCatalog::get(opCtx)->getReadyIndexes(opCtx, collection->getCatalogId(), &indexes);
-    for (auto& index : indexes) {
+    const auto& indexCatalog = collection->getIndexCatalog();
+    auto indexIt = indexCatalog->getIndexIterator(opCtx,
+                                                  IndexCatalog::InclusionPolicy::kReady |
+                                                      IndexCatalog::InclusionPolicy::kUnfinished |
+                                                      IndexCatalog::InclusionPolicy::kFrozen);
+
+    while (indexIt->more()) {
+        const IndexCatalogEntry* indexEntry = indexIt->next();
+        const std::string indexName = indexEntry->descriptor()->indexName();
+
+        Status status =
+            index_key_validate::validateIndexSpec(opCtx, indexEntry->descriptor()->infoObj())
+                .getStatus();
+        if (!status.isOK()) {
+            results->valid = false;
+            results->errors.push_back(
+                fmt::format("The index specification for index '{}' contains invalid fields. {}. "
+                            "Run the 'collMod' command on the collection without any arguments "
+                            "to fix the invalid index options",
+                            indexName,
+                            status.reason()));
+        }
+
+        if (!indexEntry->isReady(opCtx)) {
+            continue;
+        }
+
         MultikeyPaths multikeyPaths;
-        const bool isMultikey = DurableCatalog::get(opCtx)->isIndexMultikey(
-            opCtx, collection->getCatalogId(), index, &multikeyPaths);
+        const bool isMultikey = collection->isIndexMultikey(opCtx, indexName, &multikeyPaths);
         const bool hasMultiKeyPaths = std::any_of(multikeyPaths.begin(),
                                                   multikeyPaths.end(),
                                                   [](auto& pathSet) { return pathSet.size() > 0; });
-        // It is illegal for multikey paths to exist without the multikey flag set on the index, but
-        // it may be possible for multikey to be set on the index while having no multikey paths. If
-        // any of the paths are multikey, then the entire index should also be marked multikey.
+        // It is illegal for multikey paths to exist without the multikey flag set on the index,
+        // but it may be possible for multikey to be set on the index while having no multikey
+        // paths. If any of the paths are multikey, then the entire index should also be marked
+        // multikey.
         if (hasMultiKeyPaths && !isMultikey) {
             results->valid = false;
-            results->errors.push_back(fmt::format(
-                "The 'multikey' field for index {} was false with non-empty 'multikeyPaths': {}",
-                index,
-                multikeyPathsToString(multikeyPaths)));
+            results->errors.push_back(
+                fmt::format("The 'multikey' field for index {} was false with non-empty "
+                            "'multikeyPaths': {}",
+                            indexName,
+                            multikeyPathsToString(multikeyPaths)));
         }
     }
 }
@@ -362,26 +457,16 @@ void _validateCatalogEntry(OperationContext* opCtx,
 
 Status validate(OperationContext* opCtx,
                 const NamespaceString& nss,
-                ValidateOptions options,
-                bool background,
+                ValidateMode mode,
                 RepairMode repairMode,
                 ValidateResults* results,
                 BSONObjBuilder* output,
-                bool logDiagnostics) {
+                bool turnOnExtraLoggingForTest) {
     invariant(!opCtx->lockState()->isLocked() || storageGlobalParams.repair);
-    // Background validation does not support any type of full validation.
-    invariant(!(background && (options != ValidateOptions::kNoFullValidation)));
 
     // This is deliberately outside of the try-catch block, so that any errors thrown in the
     // constructor fail the cmd, as opposed to returning OK with valid:false.
-    ValidateState validateState(opCtx, nss, background, options, repairMode, logDiagnostics);
-
-    // Capped collections perform un-timestamped writes to delete expired documents. As a result,
-    // background validation will fail when reading documents that have just been deleted.
-    uassert(ErrorCodes::CommandNotSupported,
-            str::stream() << "Cannot run background validation on capped collection '" << nss
-                          << "', use foreground validation instead",
-            !background || !validateState.getCollection()->isCapped());
+    ValidateState validateState(opCtx, nss, mode, repairMode, turnOnExtraLoggingForTest);
 
     const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
     // Check whether we are allowed to read from this node after acquiring our locks. If we are
@@ -391,6 +476,8 @@ Status validate(OperationContext* opCtx,
 
     output->append("ns", validateState.nss().ns());
 
+    validateState.uuid().appendToBuilder(output, "uuid");
+
     // Foreground validation needs to ignore prepare conflicts, or else it would deadlock.
     // Repair mode cannot use ignore-prepare because it needs to be able to do writes, and there is
     // no danger of deadlock for this mode anyway since it is only used at startup (or in standalone
@@ -400,7 +487,15 @@ Status validate(OperationContext* opCtx,
         opCtx->recoveryUnit()->abandonSnapshot();
         opCtx->recoveryUnit()->setPrepareConflictBehavior(oldPrepareConflictBehavior);
     });
-    if (!validateState.isBackground() && !storageGlobalParams.repair) {
+    if (validateState.fixErrors()) {
+        // Note: cannot set PrepareConflictBehavior here, since the validate command with repair
+        // needs kIngnoreConflictsAllowWrites, but validate repair at startup cannot set that here
+        // due to an already active WriteUnitOfWork.  The prepare conflict behavior for validate
+        // command with repair is set in the command code prior to this point.
+        invariant(!validateState.isBackground());
+    } else if (!validateState.isBackground()) {
+        // Foreground validation may perform writes to fix up inconsistencies that are not
+        // correctness errors.
         opCtx->recoveryUnit()->setPrepareConflictBehavior(
             PrepareConflictBehavior::kIgnoreConflictsAllowWrites);
     } else {
@@ -408,49 +503,47 @@ Status validate(OperationContext* opCtx,
         invariant(oldPrepareConflictBehavior == PrepareConflictBehavior::kEnforce);
     }
 
-    if (validateState.isBackground()) {
-        // Background validation will hold open a snapshot at a timestamp for a long period of time
-        // while yielding and reacquiring collection locks. This has the potential to deadlock with
-        // an index build that has just failed and is trying to write using the same timestamp we
-        // are using to read. Impose a generous maximum timeout to force validate to time out and
-        // allow the index build to make progress. See SERVER-53445.
-        opCtx->lockState()->setMaxLockTimeout(duration_cast<Milliseconds>(Seconds(15)));
-    }
-
     try {
-        ValidateResultsMap indexNsResultsMap;
-        BSONObjBuilder keysPerIndex;  // not using subObjStart to be exception safe.
-
         // Full record store validation code is executed before we open cursors because it may close
         // and/or invalidate all open cursors.
-        if (options & ValidateOptions::kFullRecordStoreValidation) {
+        if (validateState.isFullValidation()) {
             invariant(opCtx->lockState()->isCollectionLockedForMode(validateState.nss(), MODE_X));
 
             // For full record store validation we use the storage engine's validation
             // functionality.
             validateState.getCollection()->getRecordStore()->validate(opCtx, results, output);
         }
-        if (options & ValidateOptions::kFullIndexValidation) {
+        if (validateState.isFullIndexValidation()) {
             invariant(opCtx->lockState()->isCollectionLockedForMode(validateState.nss(), MODE_X));
-            _validateIndexesInternalStructure(opCtx, &validateState, &indexNsResultsMap, results);
+            // For full index validation, we validate the internal structure of each index and save
+            // the number of keys in the index to compare against _validateIndexes()'s count
+            // results.
+            _validateIndexesInternalStructure(opCtx, &validateState, results);
         }
 
-        const string uuidString = str::stream() << " (UUID: " << validateState.uuid() << ")";
-
         if (!results->valid) {
-            _reportInvalidResults(opCtx,
-                                  &validateState,
-                                  &indexNsResultsMap,
-                                  &keysPerIndex,
-                                  results,
-                                  output,
-                                  uuidString);
+            _reportInvalidResults(opCtx, &validateState, results, output);
             return Status::OK();
         }
 
         // Validate in-memory catalog information with persisted info.
         _validateCatalogEntry(opCtx, &validateState, results);
 
+        if (validateState.isMetadataValidation()) {
+            if (results->valid) {
+                LOGV2(5980500,
+                      "Validation of metadata complete for collection. No problems detected",
+                      logAttrs(validateState.nss()),
+                      logAttrs(validateState.uuid()));
+            } else {
+                LOGV2(5980501,
+                      "Validation of metadata complete for collection. Problems detected",
+                      logAttrs(validateState.nss()),
+                      logAttrs(validateState.uuid()));
+            }
+            return Status::OK();
+        }
+
         // Open all cursors at once before running non-full validation code so that all steps of
         // validation during background validation use the same view of the data.
         validateState.initializeCursors(opCtx);
@@ -459,15 +552,17 @@ Status validate(OperationContext* opCtx,
         LOGV2_OPTIONS(20303,
                       {LogComponent::kIndex},
                       "validating collection",
-                      "namespace"_attr = validateState.nss(),
-                      "uuid"_attr = uuidString);
+                      logAttrs(validateState.nss()),
+                      logAttrs(validateState.uuid()));
 
         IndexConsistency indexConsistency(opCtx, &validateState);
-        ValidateAdaptor indexValidator(&indexConsistency, &validateState, &indexNsResultsMap);
+        ValidateAdaptor indexValidator(&indexConsistency, &validateState);
 
         // In traverseRecordStore(), the index validator keeps track the records in the record
         // store so that _validateIndexes() can confirm that the index entries match the records in
-        // the collection.
+        // the collection. For clustered collections, the validator also verifies that the
+        // record key (RecordId) matches the cluster key field in the record value (document's
+        // cluster key).
         indexValidator.traverseRecordStore(opCtx, results, output);
 
         // Pause collection validation while a lock is held and between collection and index data
@@ -489,19 +584,12 @@ Status validate(OperationContext* opCtx,
         }
 
         if (!results->valid) {
-            _reportInvalidResults(opCtx,
-                                  &validateState,
-                                  &indexNsResultsMap,
-                                  &keysPerIndex,
-                                  results,
-                                  output,
-                                  uuidString);
+            _reportInvalidResults(opCtx, &validateState, results, output);
             return Status::OK();
         }
 
         // Validate indexes and check for mismatches.
-        _validateIndexes(
-            opCtx, &validateState, &keysPerIndex, &indexValidator, &indexNsResultsMap, results);
+        _validateIndexes(opCtx, &validateState, &indexValidator, results);
 
         if (indexConsistency.haveEntryMismatch()) {
             LOGV2_OPTIONS(20305,
@@ -509,50 +597,33 @@ Status validate(OperationContext* opCtx,
                           "Index inconsistencies were detected. "
                           "Starting the second phase of index validation to gather concise errors",
                           "namespace"_attr = validateState.nss());
-            _gatherIndexEntryErrors(opCtx,
-                                    &validateState,
-                                    &indexConsistency,
-                                    &indexValidator,
-                                    &indexNsResultsMap,
-                                    results);
+            _gatherIndexEntryErrors(
+                opCtx, &validateState, &indexConsistency, &indexValidator, results);
         }
 
         if (!results->valid) {
-            _reportInvalidResults(opCtx,
-                                  &validateState,
-                                  &indexNsResultsMap,
-                                  &keysPerIndex,
-                                  results,
-                                  output,
-                                  uuidString);
+            _reportInvalidResults(opCtx, &validateState, results, output);
             return Status::OK();
         }
 
         // Validate index key count.
-        _validateIndexKeyCount(opCtx, &validateState, &indexValidator, &indexNsResultsMap);
+        _validateIndexKeyCount(opCtx, &validateState, &indexValidator, &results->indexResultsMap);
 
         if (!results->valid) {
-            _reportInvalidResults(opCtx,
-                                  &validateState,
-                                  &indexNsResultsMap,
-                                  &keysPerIndex,
-                                  results,
-                                  output,
-                                  uuidString);
+            _reportInvalidResults(opCtx, &validateState, results, output);
             return Status::OK();
         }
 
         // At this point, validation is complete and successful.
         // Report the validation results for the user to see.
-        _reportValidationResults(
-            opCtx, &validateState, &indexNsResultsMap, &keysPerIndex, results, output);
+        _reportValidationResults(opCtx, &validateState, results, output);
 
         LOGV2_OPTIONS(20306,
                       {LogComponent::kIndex},
                       "Validation complete for collection. No "
                       "corruption found",
-                      "namespace"_attr = validateState.nss(),
-                      "uuid"_attr = uuidString);
+                      logAttrs(validateState.nss()),
+                      logAttrs(validateState.uuid()));
     } catch (const DBException& e) {
         if (ErrorCodes::isInterruption(e.code())) {
             LOGV2_OPTIONS(5160301,
