@@ -27,28 +27,34 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/catalog/drop_indexes.h"
 
-#include "mongo/db/background.h"
+#include <boost/algorithm/string/join.hpp>
+
+#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/catalog/collection_uuid_mismatch.h"
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/client.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index_builds_coordinator.h"
-#include "mongo/db/op_observer.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl_set_member_in_standalone_mode.h"
 #include "mongo/db/s/collection_sharding_state.h"
 #include "mongo/db/s/database_sharding_state.h"
+#include "mongo/db/s/shard_key_index_util.h"
 #include "mongo/db/service_context.h"
-#include "mongo/db/views/view_catalog.h"
 #include "mongo/logv2/log.h"
+#include "mongo/util/overloaded_visitor.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
+
 
 namespace mongo {
 namespace {
@@ -60,10 +66,9 @@ constexpr auto kIndexFieldName = "index"_sd;
 
 Status checkView(OperationContext* opCtx,
                  const NamespaceString& nss,
-                 Database* db,
-                 Collection* collection) {
+                 const CollectionPtr& collection) {
     if (!collection) {
-        if (db && ViewCatalog::get(db)->lookup(opCtx, nss.ns())) {
+        if (CollectionCatalog::get(opCtx)->lookupView(opCtx, nss)) {
             return Status(ErrorCodes::CommandNotSupportedOnView,
                           str::stream() << "Cannot drop indexes on view " << nss);
         }
@@ -74,7 +79,7 @@ Status checkView(OperationContext* opCtx,
 
 Status checkReplState(OperationContext* opCtx,
                       NamespaceStringOrUUID dbAndUUID,
-                      Collection* collection) {
+                      const CollectionPtr& collection) {
     auto replCoord = repl::ReplicationCoordinator::get(opCtx);
     auto canAcceptWrites = replCoord->canAcceptWritesFor(opCtx, dbAndUUID);
     bool writesAreReplicatedAndNotPrimary = opCtx->writesAreReplicated() && !canAcceptWrites;
@@ -102,23 +107,22 @@ Status checkReplState(OperationContext* opCtx,
  * Validates the key pattern passed through the command.
  */
 StatusWith<const IndexDescriptor*> getDescriptorByKeyPattern(OperationContext* opCtx,
-                                                             IndexCatalog* indexCatalog,
-                                                             const BSONElement& keyPattern) {
+                                                             const IndexCatalog* indexCatalog,
+                                                             const BSONObj& keyPattern) {
     std::vector<const IndexDescriptor*> indexes;
     indexCatalog->findIndexesByKeyPattern(opCtx,
-                                          keyPattern.embeddedObject(),
+                                          keyPattern,
                                           IndexCatalog::InclusionPolicy::kReady |
                                               IndexCatalog::InclusionPolicy::kUnfinished |
                                               IndexCatalog::InclusionPolicy::kFrozen,
                                           &indexes);
     if (indexes.empty()) {
         return Status(ErrorCodes::IndexNotFound,
-                      str::stream()
-                          << "can't find index with key: " << keyPattern.embeddedObject());
+                      str::stream() << "can't find index with key: " << keyPattern);
     } else if (indexes.size() > 1) {
         return Status(ErrorCodes::AmbiguousIndexKeyPattern,
-                      str::stream() << indexes.size() << " indexes found for key: "
-                                    << keyPattern.embeddedObject() << ", identify by name instead."
+                      str::stream() << indexes.size() << " indexes found for key: " << keyPattern
+                                    << ", identify by name instead."
                                     << " Conflicting indexes: " << indexes[0]->infoObj() << ", "
                                     << indexes[1]->infoObj());
     }
@@ -141,34 +145,66 @@ StatusWith<const IndexDescriptor*> getDescriptorByKeyPattern(OperationContext* o
     return desc;
 }
 
+/**
+ * It is illegal to drop a collection's clusteredIndex.
+ *
+ * Returns true if 'index' is or contains the clusteredIndex.
+ */
+bool containsClusteredIndex(const CollectionPtr& collection, const IndexArgument& index) {
+    invariant(collection && collection->isClustered());
+
+    auto clusteredIndexSpec = collection->getClusteredInfo()->getIndexSpec();
+    return stdx::visit(
+        OverloadedVisitor{[&](const std::string& indexName) -> bool {
+                              // While the clusteredIndex's name is optional during user
+                              // creation, it should always be filled in by default on the
+                              // collection object.
+                              auto clusteredIndexName = clusteredIndexSpec.getName();
+                              invariant(clusteredIndexName.has_value());
+
+                              return clusteredIndexName.value() == indexName;
+                          },
+                          [&](const std::vector<std::string>& indexNames) -> bool {
+                              // While the clusteredIndex's name is optional during user
+                              // creation, it should always be filled in by default on the
+                              // collection object.
+                              auto clusteredIndexName = clusteredIndexSpec.getName();
+                              invariant(clusteredIndexName.has_value());
+
+                              return std::find(indexNames.begin(),
+                                               indexNames.end(),
+                                               clusteredIndexName.value()) != indexNames.end();
+                          },
+                          [&](const BSONObj& indexKey) -> bool {
+                              return clusteredIndexSpec.getKey().woCompare(indexKey) == 0;
+                          }},
+        index);
+}
+
 /**
  * Returns a list of index names that the caller requested to abort/drop. Requires a collection lock
  * to be held to look up the index name from the key pattern.
  */
 StatusWith<std::vector<std::string>> getIndexNames(OperationContext* opCtx,
-                                                   Collection* collection,
-                                                   const BSONElement& indexElem) {
+                                                   const CollectionPtr& collection,
+                                                   const IndexArgument& index) {
     invariant(opCtx->lockState()->isCollectionLockedForMode(collection->ns(), MODE_IX));
 
-    std::vector<std::string> indexNames;
-    if (indexElem.type() == String) {
-        std::string indexToAbort = indexElem.valuestr();
-        indexNames.push_back(indexToAbort);
-    } else if (indexElem.type() == Object) {
-        auto swDescriptor =
-            getDescriptorByKeyPattern(opCtx, collection->getIndexCatalog(), indexElem);
-        if (!swDescriptor.isOK()) {
-            return swDescriptor.getStatus();
-        }
-        indexNames.push_back(swDescriptor.getValue()->indexName());
-    } else if (indexElem.type() == Array) {
-        for (auto indexNameElem : indexElem.Array()) {
-            invariant(indexNameElem.type() == String);
-            indexNames.push_back(indexNameElem.valuestr());
-        }
-    }
-
-    return indexNames;
+    return stdx::visit(
+        OverloadedVisitor{
+            [](const std::string& arg) -> StatusWith<std::vector<std::string>> { return {{arg}}; },
+            [](const std::vector<std::string>& arg) -> StatusWith<std::vector<std::string>> {
+                return arg;
+            },
+            [&](const BSONObj& arg) -> StatusWith<std::vector<std::string>> {
+                auto swDescriptor =
+                    getDescriptorByKeyPattern(opCtx, collection->getIndexCatalog(), arg);
+                if (!swDescriptor.isOK()) {
+                    return swDescriptor.getStatus();
+                }
+                return {{swDescriptor.getValue()->indexName()}};
+            }},
+        index);
 }
 
 /**
@@ -205,7 +241,7 @@ Status dropIndexByDescriptor(OperationContext* opCtx,
         invariant(!entry->isReady(opCtx));
         invariant(getReplSetMemberInStandaloneMode(opCtx->getServiceContext()));
         // Return here. No need to fall through to op observer on standalone.
-        return indexCatalog->dropUnfinishedIndex(opCtx, desc);
+        return indexCatalog->dropUnfinishedIndex(opCtx, collection, desc);
     }
 
     // Do not allow dropping unfinished indexes that are not frozen.
@@ -221,7 +257,7 @@ Status dropIndexByDescriptor(OperationContext* opCtx,
     opCtx->getServiceContext()->getOpObserver()->onDropIndex(
         opCtx, collection->ns(), collection->uuid(), desc->indexName(), desc->infoObj());
 
-    auto s = indexCatalog->dropIndex(opCtx, desc);
+    auto s = indexCatalog->dropIndex(opCtx, collection, desc);
     if (!s.isOK()) {
         return s;
     }
@@ -235,7 +271,7 @@ Status dropIndexByDescriptor(OperationContext* opCtx,
  */
 std::vector<UUID> abortActiveIndexBuilders(OperationContext* opCtx,
                                            const NamespaceString& collectionNs,
-                                           CollectionUUID collectionUUID,
+                                           const UUID& collectionUUID,
                                            const std::vector<std::string>& indexNames) {
     if (indexNames.empty()) {
         return {};
@@ -249,47 +285,85 @@ std::vector<UUID> abortActiveIndexBuilders(OperationContext* opCtx,
     return abortIndexBuildByIndexNames(opCtx, collectionUUID, indexNames);
 }
 
-Status dropReadyIndexes(OperationContext* opCtx,
-                        Collection* collection,
-                        const std::vector<std::string>& indexNames,
-                        BSONObjBuilder* anObjBuilder) {
+void dropReadyIndexes(OperationContext* opCtx,
+                      Collection* collection,
+                      const std::vector<std::string>& indexNames,
+                      DropIndexesReply* reply,
+                      bool forceDropShardKeyIndex) {
     invariant(opCtx->lockState()->isCollectionLockedForMode(collection->ns(), MODE_X));
 
     if (indexNames.empty()) {
-        return Status::OK();
+        return;
     }
 
     IndexCatalog* indexCatalog = collection->getIndexCatalog();
-    if (indexNames.front() == "*") {
-        indexCatalog->dropAllIndexes(
-            opCtx, false, [opCtx, collection](const IndexDescriptor* desc) {
-                opCtx->getServiceContext()->getOpObserver()->onDropIndex(opCtx,
-                                                                         collection->ns(),
-                                                                         collection->uuid(),
-                                                                         desc->indexName(),
-                                                                         desc->infoObj());
-            });
+    auto collDescription =
+        CollectionShardingState::get(opCtx, collection->ns())->getCollectionDescription(opCtx);
 
-        anObjBuilder->append("msg", "non-_id indexes dropped for collection");
-        return Status::OK();
+    if (indexNames.front() == "*") {
+        if (collDescription.isSharded() && !forceDropShardKeyIndex) {
+            indexCatalog->dropIndexes(
+                opCtx,
+                collection,
+                [&](const IndexDescriptor* desc) {
+                    if (desc->isIdIndex()) {
+                        return false;
+                    }
+
+                    if (isCompatibleWithShardKey(opCtx,
+                                                 CollectionPtr(collection),
+                                                 desc->getEntry(),
+                                                 collDescription.getKeyPattern(),
+                                                 false /* requiresSingleKey */)) {
+                        return false;
+                    }
+
+                    return true;
+                },
+                [opCtx, collection](const IndexDescriptor* desc) {
+                    opCtx->getServiceContext()->getOpObserver()->onDropIndex(opCtx,
+                                                                             collection->ns(),
+                                                                             collection->uuid(),
+                                                                             desc->indexName(),
+                                                                             desc->infoObj());
+                });
+
+            reply->setMsg("non-_id indexes and non-shard key indexes dropped for collection"_sd);
+        } else {
+            indexCatalog->dropAllIndexes(
+                opCtx, collection, false, [opCtx, collection](const IndexDescriptor* desc) {
+                    opCtx->getServiceContext()->getOpObserver()->onDropIndex(opCtx,
+                                                                             collection->ns(),
+                                                                             collection->uuid(),
+                                                                             desc->indexName(),
+                                                                             desc->infoObj());
+                });
+
+            reply->setMsg("non-_id indexes dropped for collection"_sd);
+        }
+        return;
     }
 
     for (const auto& indexName : indexNames) {
+        if (collDescription.isSharded()) {
+            uassert(
+                ErrorCodes::CannotDropShardKeyIndex,
+                "Cannot drop the only compatible index for this collection's shard key",
+                !isLastNonHiddenShardKeyIndex(
+                    opCtx, collection, indexCatalog, indexName, collDescription.getKeyPattern()));
+        }
+
         auto desc = indexCatalog->findIndexByName(opCtx,
                                                   indexName,
                                                   IndexCatalog::InclusionPolicy::kReady |
                                                       IndexCatalog::InclusionPolicy::kUnfinished |
                                                       IndexCatalog::InclusionPolicy::kFrozen);
         if (!desc) {
-            return Status(ErrorCodes::IndexNotFound,
-                          str::stream() << "index not found with name [" << indexName << "]");
-        }
-        Status status = dropIndexByDescriptor(opCtx, collection, indexCatalog, desc);
-        if (!status.isOK()) {
-            return status;
+            uasserted(ErrorCodes::IndexNotFound,
+                      str::stream() << "index not found with name [" << indexName << "]");
         }
+        uassertStatusOK(dropIndexByDescriptor(opCtx, collection, indexCatalog, desc));
     }
-    return Status::OK();
 }
 
 void assertMovePrimaryInProgress(OperationContext* opCtx, const NamespaceString& ns) {
@@ -297,13 +371,13 @@ void assertMovePrimaryInProgress(OperationContext* opCtx, const NamespaceString&
     auto dssLock = DatabaseShardingState::DSSLock::lockShared(opCtx, dss);
 
     try {
-        const auto collDesc = CollectionShardingState::get(opCtx, ns)->getCollectionDescription();
+        const auto collDesc =
+            CollectionShardingState::get(opCtx, ns)->getCollectionDescription(opCtx);
         if (!collDesc.isSharded()) {
             auto mpsm = dss->getMovePrimarySourceManager(dssLock);
 
             if (mpsm) {
-                LOGV2(
-                    4976500, "assertMovePrimaryInProgress", "movePrimaryNss"_attr = ns.toString());
+                LOGV2(4976500, "assertMovePrimaryInProgress", "namespace"_attr = ns.toString());
 
                 uasserted(ErrorCodes::MovePrimaryInProgress,
                           "movePrimary is in progress for namespace " + ns.toString());
@@ -311,7 +385,7 @@ void assertMovePrimaryInProgress(OperationContext* opCtx, const NamespaceString&
         }
     } catch (const DBException& ex) {
         if (ex.toStatus() != ErrorCodes::MovePrimaryInProgress) {
-            LOGV2(4976501, "Error when getting colleciton description", "what"_attr = ex.what());
+            LOGV2(4976501, "Error when getting collection description", "what"_attr = ex.what());
             return;
         }
         throw;
@@ -320,57 +394,46 @@ void assertMovePrimaryInProgress(OperationContext* opCtx, const NamespaceString&
 
 }  // namespace
 
-Status dropIndexes(OperationContext* opCtx,
-                   const NamespaceString& nss,
-                   const BSONObj& cmdObj,
-                   BSONObjBuilder* result) {
+DropIndexesReply dropIndexes(OperationContext* opCtx,
+                             const NamespaceString& nss,
+                             const boost::optional<UUID>& expectedUUID,
+                             const IndexArgument& index) {
     // We only need to hold an intent lock to send abort signals to the active index builder(s) we
     // intend to abort.
-    boost::optional<AutoGetCollection> autoColl;
-    autoColl.emplace(opCtx, nss, MODE_IX);
+    boost::optional<AutoGetCollection> collection;
+    collection.emplace(opCtx, nss, MODE_IX);
 
-    Database* db = autoColl->getDb();
-    Collection* collection = autoColl->getCollection();
-    Status status = checkView(opCtx, nss, db, collection);
-    if (!status.isOK()) {
-        return status;
-    }
+    checkCollectionUUIDMismatch(opCtx, nss, collection->getCollection(), expectedUUID);
+    uassertStatusOK(checkView(opCtx, nss, collection->getCollection()));
 
-    const UUID collectionUUID = collection->uuid();
+    const UUID collectionUUID = (*collection)->uuid();
     const NamespaceStringOrUUID dbAndUUID = {nss.db().toString(), collectionUUID};
-
-    status = checkReplState(opCtx, dbAndUUID, collection);
-    if (!status.isOK()) {
-        return status;
-    }
-
+    uassertStatusOK(checkReplState(opCtx, dbAndUUID, collection->getCollection()));
     if (!serverGlobalParams.quiet.load()) {
         LOGV2(51806,
               "CMD: dropIndexes",
               "namespace"_attr = nss,
               "uuid"_attr = collectionUUID,
-              "indexes"_attr = cmdObj[kIndexFieldName].toString(false));
+              "indexes"_attr =
+                  stdx::visit(OverloadedVisitor{[](const std::string& arg) { return arg; },
+                                                [](const std::vector<std::string>& arg) {
+                                                    return boost::algorithm::join(arg, ",");
+                                                },
+                                                [](const BSONObj& arg) { return arg.toString(); }},
+                              index));
     }
 
-    result->appendNumber("nIndexesWas", collection->getIndexCatalog()->numIndexesTotal(opCtx));
-
-    // Validate basic user input.
-    BSONElement indexElem = cmdObj.getField(kIndexFieldName);
-    const bool isWildcard = indexElem.type() == String && indexElem.String() == "*";
-
-    // If an Array was passed in, verify that all the elements are of type String.
-    if (indexElem.type() == Array) {
-        for (auto indexNameElem : indexElem.Array()) {
-            if (indexNameElem.type() != String) {
-                return Status(ErrorCodes::TypeMismatch,
-                              str::stream()
-                                  << "dropIndexes " << collection->ns() << " (" << collectionUUID
-                                  << ") failed to drop multiple indexes "
-                                  << indexElem.toString(false) << ": index name must be a string");
-            }
-        }
+    if ((*collection)->isClustered() &&
+        containsClusteredIndex(collection->getCollection(), index)) {
+        uasserted(5979800, "It is illegal to drop the clusteredIndex");
     }
 
+    DropIndexesReply reply;
+    reply.setNIndexesWas((*collection)->getIndexCatalog()->numIndexesTotal(opCtx));
+
+    const bool isWildcard =
+        stdx::holds_alternative<std::string>(index) && stdx::get<std::string>(index) == "*";
+
     IndexBuildsCoordinator* indexBuildsCoord = IndexBuildsCoordinator::get(opCtx);
 
     // When releasing the collection lock to send the abort signal to the index builders, it's
@@ -379,22 +442,18 @@ Status dropIndexes(OperationContext* opCtx,
     std::vector<UUID> abortedIndexBuilders;
     std::vector<std::string> indexNames;
     while (true) {
-        auto swIndexNames = getIndexNames(opCtx, collection, indexElem);
-        if (!swIndexNames.isOK()) {
-            return swIndexNames.getStatus();
-        }
-
-        indexNames = swIndexNames.getValue();
+        indexNames = uassertStatusOK(getIndexNames(opCtx, collection->getCollection(), index));
 
         // Copy the namespace and UUID before dropping locks.
-        auto collUUID = collection->uuid();
-        auto collNs = collection->ns();
+        auto collUUID = (*collection)->uuid();
+        auto collNs = (*collection)->ns();
 
-        // Release locks before aborting index builds. The helper will acquire locks on our behalf.
-        autoColl = boost::none;
+        // Release locks before aborting index builds. The helper will acquire locks on our
+        // behalf.
+        collection = boost::none;
 
-        // Send the abort signal to any index builders that match the users request. Waits until all
-        // aborted builders complete.
+        // Send the abort signal to any index builders that match the users request. Waits until
+        // all aborted builders complete.
         auto justAborted = abortActiveIndexBuilders(opCtx, collNs, collUUID, indexNames);
         abortedIndexBuilders.insert(
             abortedIndexBuilders.end(), justAborted.begin(), justAborted.end());
@@ -404,28 +463,27 @@ Status dropIndexes(OperationContext* opCtx,
             hangAfterAbortingIndexes.pauseWhileSet();
         }
 
-        // Take an exclusive lock on the collection now to be able to perform index catalog writes
-        // when removing ready indexes from disk.
-        autoColl.emplace(opCtx, dbAndUUID, MODE_X);
-
-        // Abandon the snapshot as the index catalog will compare the in-memory state to the disk
-        // state, which may have changed when we released the lock temporarily.
+        // Abandon the snapshot as the index catalog will compare the in-memory state to the
+        // disk state, which may have changed when we released the lock temporarily.
         opCtx->recoveryUnit()->abandonSnapshot();
 
-        db = autoColl->getDb();
-        collection = autoColl->getCollection();
-        if (!collection) {
-            return Status(ErrorCodes::NamespaceNotFound,
-                          str::stream() << "ns not found on database " << dbAndUUID.db()
-                                        << " with collection " << dbAndUUID.uuid());
+        // Take an exclusive lock on the collection now to be able to perform index catalog
+        // writes when removing ready indexes from disk.
+        collection.emplace(opCtx, dbAndUUID, MODE_X);
+
+        if (!*collection) {
+            uasserted(ErrorCodes::NamespaceNotFound,
+                      str::stream() << "Collection '" << nss << "' with UUID " << dbAndUUID.uuid()
+                                    << " in database " << dbAndUUID.db() << " does not exist.");
         }
 
-        status = checkReplState(opCtx, dbAndUUID, collection);
-        if (!status.isOK()) {
-            return status;
-        }
+        // The collection could have been renamed when we dropped locks.
+        collNs = (*collection)->ns();
 
-        // Check to see if a new index build was started that the caller requested to be aborted.
+        uassertStatusOK(checkReplState(opCtx, dbAndUUID, collection->getCollection()));
+
+        // Check to see if a new index build was started that the caller requested to be
+        // aborted.
         bool abortAgain = false;
         if (isWildcard) {
             abortAgain = indexBuildsCoord->inProgForCollection(collectionUUID);
@@ -434,7 +492,10 @@ Status dropIndexes(OperationContext* opCtx,
         }
 
         if (!abortAgain) {
-            assertMovePrimaryInProgress(opCtx, nss);
+            assertMovePrimaryInProgress(opCtx, collNs);
+            CollectionShardingState::get(opCtx, collNs)
+                ->getCollectionDescription(opCtx)
+                .throwIfReshardingInProgress(collNs);
             break;
         }
     }
@@ -445,17 +506,30 @@ Status dropIndexes(OperationContext* opCtx,
         // The index catalog requires that no active index builders are running when dropping ready
         // indexes.
         IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(collectionUUID);
-        return writeConflictRetry(opCtx, "dropIndexes", dbAndUUID.toString(), [&] {
+        writeConflictRetry(opCtx, "dropIndexes", dbAndUUID.toString(), [&] {
             WriteUnitOfWork wuow(opCtx);
 
             // This is necessary to check shard version.
-            OldClientContext ctx(opCtx, collection->ns().ns());
+            OldClientContext ctx(opCtx, (*collection)->ns());
 
-            // Iterate through all the aborted indexes and drop any indexes that are ready in the
-            // index catalog. This would indicate that while we yielded our locks during the abort
-            // phase, a new identical index was created.
-            auto indexCatalog = collection->getIndexCatalog();
+            // Iterate through all the aborted indexes and drop any indexes that are ready in
+            // the index catalog. This would indicate that while we yielded our locks during the
+            // abort phase, a new identical index was created.
+            auto indexCatalog = collection->getWritableCollection(opCtx)->getIndexCatalog();
             for (const auto& indexName : indexNames) {
+                auto collDescription =
+                    CollectionShardingState::get(opCtx, nss)->getCollectionDescription(opCtx);
+
+                if (collDescription.isSharded()) {
+                    uassert(ErrorCodes::CannotDropShardKeyIndex,
+                            "Cannot drop the only compatible index for this collection's shard key",
+                            !isLastNonHiddenShardKeyIndex(opCtx,
+                                                          collection->getCollection(),
+                                                          indexCatalog,
+                                                          indexName,
+                                                          collDescription.getKeyPattern()));
+                }
+
                 auto desc =
                     indexCatalog->findIndexByName(opCtx,
                                                   indexName,
@@ -467,63 +541,52 @@ Status dropIndexes(OperationContext* opCtx,
                     continue;
                 }
 
-                Status status = dropIndexByDescriptor(opCtx, collection, indexCatalog, desc);
-                if (!status.isOK()) {
-                    return status;
-                }
+                uassertStatusOK(dropIndexByDescriptor(
+                    opCtx, collection->getWritableCollection(opCtx), indexCatalog, desc));
             }
 
             wuow.commit();
-            return Status::OK();
         });
+
+        return reply;
     }
 
     if (!abortedIndexBuilders.empty()) {
-        // All the index builders were sent the abort signal, remove all the remaining indexes in
-        // the index catalog.
+        // All the index builders were sent the abort signal, remove all the remaining indexes
+        // in the index catalog.
         invariant(isWildcard);
         invariant(indexNames.size() == 1);
         invariant(indexNames.front() == "*");
-        invariant(collection->getIndexCatalog()->numIndexesInProgress(opCtx) == 0);
+        invariant((*collection)->getIndexCatalog()->numIndexesInProgress(opCtx) == 0);
     }
 
-    // The index catalog requires that no active index builders are running when dropping ready
-    // indexes.
-    BackgroundOperation::assertNoBgOpInProgForNs(collection->ns());
-    IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(collectionUUID);
-    return writeConflictRetry(
-        opCtx, "dropIndexes", dbAndUUID.toString(), [opCtx, &collection, &indexNames, result] {
+    writeConflictRetry(
+        opCtx, "dropIndexes", dbAndUUID.toString(), [opCtx, &collection, &indexNames, &reply] {
             WriteUnitOfWork wunit(opCtx);
 
             // This is necessary to check shard version.
-            OldClientContext ctx(opCtx, collection->ns().ns());
-
-            // Use an empty BSONObjBuilder to avoid duplicate appends to result on retry loops.
-            BSONObjBuilder tempObjBuilder;
-            Status status = dropReadyIndexes(opCtx, collection, indexNames, &tempObjBuilder);
-            if (!status.isOK()) {
-                return status;
-            }
-
+            OldClientContext ctx(opCtx, (*collection)->ns());
+            dropReadyIndexes(
+                opCtx, collection->getWritableCollection(opCtx), indexNames, &reply, false);
             wunit.commit();
-
-            result->appendElementsUnique(
-                tempObjBuilder.done());  // This append will only happen once.
-            return Status::OK();
         });
+
+    return reply;
 }
 
 Status dropIndexesForApplyOps(OperationContext* opCtx,
                               const NamespaceString& nss,
-                              const BSONObj& cmdObj,
-                              BSONObjBuilder* result) {
-    return writeConflictRetry(opCtx, "dropIndexes", nss.db(), [opCtx, &nss, &cmdObj, result] {
-        AutoGetCollection autoColl(opCtx, nss, MODE_X);
+                              const BSONObj& cmdObj) try {
+    BSONObjBuilder bob(cmdObj);
+    bob.append("$db", nss.db());
+    auto cmdObjWithDb = bob.obj();
+    auto parsed = DropIndexes::parse(IDLParserContext{"dropIndexes"}, cmdObjWithDb);
+
+    return writeConflictRetry(opCtx, "dropIndexes", nss.db(), [opCtx, &nss, &cmdObj, &parsed] {
+        AutoGetCollection collection(opCtx, nss, MODE_X);
 
         // If db/collection does not exist, short circuit and return.
-        Database* db = autoColl.getDb();
-        Collection* collection = autoColl.getCollection();
-        Status status = checkView(opCtx, nss, db, collection);
+        Status status = checkView(opCtx, nss, collection.getCollection());
         if (!status.isOK()) {
             return status;
         }
@@ -535,12 +598,7 @@ Status dropIndexesForApplyOps(OperationContext* opCtx,
                   "indexes"_attr = cmdObj[kIndexFieldName].toString(false));
         }
 
-        BackgroundOperation::assertNoBgOpInProgForNs(nss);
-        IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(
-            collection->uuid());
-
-        BSONElement indexElem = cmdObj.getField(kIndexFieldName);
-        auto swIndexNames = getIndexNames(opCtx, collection, indexElem);
+        auto swIndexNames = getIndexNames(opCtx, collection.getCollection(), parsed.getIndex());
         if (!swIndexNames.isOK()) {
             return swIndexNames.getStatus();
         }
@@ -548,20 +606,20 @@ Status dropIndexesForApplyOps(OperationContext* opCtx,
         WriteUnitOfWork wunit(opCtx);
 
         // This is necessary to check shard version.
-        OldClientContext ctx(opCtx, nss.ns());
+        OldClientContext ctx(opCtx, nss);
 
-        // Use an empty BSONObjBuilder to avoid duplicate appends to result on retry loops.
-        BSONObjBuilder tempObjBuilder;
-        status = dropReadyIndexes(opCtx, collection, swIndexNames.getValue(), &tempObjBuilder);
-        if (!status.isOK()) {
-            return status;
-        }
+        DropIndexesReply ignoredReply;
+        dropReadyIndexes(opCtx,
+                         collection.getWritableCollection(opCtx),
+                         swIndexNames.getValue(),
+                         &ignoredReply,
+                         true);
 
         wunit.commit();
-
-        result->appendElementsUnique(tempObjBuilder.done());  // This append will only happen once.
         return Status::OK();
     });
+} catch (const DBException& exc) {
+    return exc.toStatus();
 }
 
 }  // namespace mongo
