@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include "mongo/platform/basic.h"
 
@@ -36,10 +35,9 @@
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/index_catalog.h"
-#include "mongo/db/catalog/index_timestamp_helper.h"
-#include "mongo/db/catalog/uncommitted_collections.h"
+#include "mongo/db/catalog/index_repair.h"
 #include "mongo/db/catalog_raii.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/storage/storage_repair_observer.h"
@@ -49,6 +47,9 @@
 #include "mongo/util/progress_meter.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
+
+
 namespace mongo {
 
 namespace {
@@ -81,11 +82,12 @@ IndexBuildsManager::~IndexBuildsManager() {
 }
 
 Status IndexBuildsManager::setUpIndexBuild(OperationContext* opCtx,
-                                           Collection* collection,
+                                           CollectionWriter& collection,
                                            const std::vector<BSONObj>& specs,
                                            const UUID& buildUUID,
                                            OnInitFn onInit,
-                                           SetupOptions options) {
+                                           SetupOptions options,
+                                           const boost::optional<ResumeIndexInfo>& resumeInfo) {
     _registerIndexBuild(buildUUID);
 
     const auto& nss = collection->ns();
@@ -108,10 +110,13 @@ Status IndexBuildsManager::setUpIndexBuild(OperationContext* opCtx,
         builder->ignoreUniqueConstraint();
     }
 
+    builder->setIndexBuildMethod(options.method);
+
     std::vector<BSONObj> indexes;
     try {
         indexes = writeConflictRetry(opCtx, "IndexBuildsManager::setUpIndexBuild", nss.ns(), [&]() {
-            return uassertStatusOK(builder->init(opCtx, collection, specs, onInit));
+            return uassertStatusOK(
+                builder->init(opCtx, collection, specs, onInit, options.forRecovery, resumeInfo));
         });
     } catch (const DBException& ex) {
         return ex.toStatus();
@@ -120,16 +125,24 @@ Status IndexBuildsManager::setUpIndexBuild(OperationContext* opCtx,
     return Status::OK();
 }
 
-Status IndexBuildsManager::startBuildingIndex(OperationContext* opCtx,
-                                              Collection* collection,
-                                              const UUID& buildUUID) {
+Status IndexBuildsManager::startBuildingIndex(
+    OperationContext* opCtx,
+    const CollectionPtr& collection,
+    const UUID& buildUUID,
+    const boost::optional<RecordId>& resumeAfterRecordId) {
     auto builder = invariant(_getBuilder(buildUUID));
 
-    return builder->insertAllDocumentsInCollection(opCtx, collection);
+    return builder->insertAllDocumentsInCollection(opCtx, collection, resumeAfterRecordId);
+}
+
+Status IndexBuildsManager::resumeBuildingIndexFromBulkLoadPhase(OperationContext* opCtx,
+                                                                const CollectionPtr& collection,
+                                                                const UUID& buildUUID) {
+    return invariant(_getBuilder(buildUUID))->dumpInsertsFromBulk(opCtx, collection);
 }
 
 StatusWith<std::pair<long long, long long>> IndexBuildsManager::startBuildingIndexForRecovery(
-    OperationContext* opCtx, Collection* coll, const UUID& buildUUID, RepairData repair) {
+    OperationContext* opCtx, const CollectionPtr& coll, const UUID& buildUUID, RepairData repair) {
     auto builder = invariant(_getBuilder(buildUUID));
 
     // Iterate all records in the collection. Validate the records and index them
@@ -160,11 +173,10 @@ StatusWith<std::pair<long long, long long>> IndexBuildsManager::startBuildingInd
             }
             WriteUnitOfWork wunit(opCtx);
             for (int i = 0; record && i < internalInsertMaxBatchSize.load(); i++) {
-                RecordId id = record->id;
+                auto& id = record->id;
                 RecordData& data = record->data;
-                // Use the latest BSON validation version. We retain decimal data when repairing
-                // database even if decimal is disabled.
-                auto validStatus = validateBSON(data.data(), data.size(), BSONVersion::kLatest);
+                // We retain decimal data when repairing database even if decimal is disabled.
+                auto validStatus = validateBSON(data.data(), data.size());
                 if (!validStatus.isOK()) {
                     if (repair == RepairData::kNo) {
                         LOGV2_FATAL(31396,
@@ -185,7 +197,19 @@ StatusWith<std::pair<long long, long long>> IndexBuildsManager::startBuildingInd
                 } else {
                     numRecords++;
                     dataSize += data.size();
-                    auto insertStatus = builder->insert(opCtx, data.releaseToBson(), id);
+                    auto insertStatus = builder->insertSingleDocumentForInitialSyncOrRecovery(
+                        opCtx,
+                        coll,
+                        data.releaseToBson(),
+                        id,
+                        [&cursor] { cursor->save(); },
+                        [&] {
+                            writeConflictRetry(
+                                opCtx,
+                                "insertSingleDocumentForInitialSyncOrRecovery-restoreCursor",
+                                ns.ns(),
+                                [&cursor] { cursor->restore(); });
+                        });
                     if (!insertStatus.isOK()) {
                         return insertStatus;
                     }
@@ -218,14 +242,15 @@ StatusWith<std::pair<long long, long long>> IndexBuildsManager::startBuildingInd
     long long recordsRemoved = 0;
     long long bytesRemoved = 0;
 
-    const NamespaceString lostAndFoundNss = NamespaceString(
-        NamespaceString::kLocalDb, "system.lost_and_found." + coll->uuid().toString());
+    const NamespaceString lostAndFoundNss =
+        NamespaceString(NamespaceString::kLocalDb, "lost_and_found." + coll->uuid().toString());
 
     // Delete duplicate record and insert it into local lost and found.
     Status status = [&] {
         if (repair == RepairData::kYes) {
-            return builder->dumpInsertsFromBulk(opCtx, [&](const RecordId& rid) {
-                auto moveStatus = _moveRecordToLostAndFound(opCtx, ns, lostAndFoundNss, rid);
+            return builder->dumpInsertsFromBulk(opCtx, coll, [&](const RecordId& rid) {
+                auto moveStatus =
+                    mongo::index_repair::moveRecordToLostAndFound(opCtx, ns, lostAndFoundNss, rid);
                 if (moveStatus.isOK() && (moveStatus.getValue() > 0)) {
                     recordsRemoved++;
                     bytesRemoved += moveStatus.getValue();
@@ -233,7 +258,7 @@ StatusWith<std::pair<long long, long long>> IndexBuildsManager::startBuildingInd
                 return moveStatus.getStatus();
             });
         } else {
-            return builder->dumpInsertsFromBulk(opCtx);
+            return builder->dumpInsertsFromBulk(opCtx, coll);
         }
     }();
     if (!status.isOK()) {
@@ -271,20 +296,21 @@ Status IndexBuildsManager::drainBackgroundWrites(
 
 Status IndexBuildsManager::retrySkippedRecords(OperationContext* opCtx,
                                                const UUID& buildUUID,
-                                               Collection* collection) {
+                                               const CollectionPtr& collection) {
     auto builder = invariant(_getBuilder(buildUUID));
     return builder->retrySkippedRecords(opCtx, collection);
 }
 
 Status IndexBuildsManager::checkIndexConstraintViolations(OperationContext* opCtx,
+                                                          const CollectionPtr& collection,
                                                           const UUID& buildUUID) {
     auto builder = invariant(_getBuilder(buildUUID));
 
-    return builder->checkConstraints(opCtx);
+    return builder->checkConstraints(opCtx, collection);
 }
 
 Status IndexBuildsManager::commitIndexBuild(OperationContext* opCtx,
-                                            Collection* collection,
+                                            CollectionWriter& collection,
                                             const NamespaceString& nss,
                                             const UUID& buildUUID,
                                             MultiIndexBlock::OnCreateEachFn onCreateEachFn,
@@ -295,25 +321,20 @@ Status IndexBuildsManager::commitIndexBuild(OperationContext* opCtx,
         opCtx,
         "IndexBuildsManager::commitIndexBuild",
         nss.ns(),
-        [this, builder, buildUUID, opCtx, collection, nss, &onCreateEachFn, &onCommitFn] {
+        [this, builder, buildUUID, opCtx, &collection, nss, &onCreateEachFn, &onCommitFn] {
             WriteUnitOfWork wunit(opCtx);
-            auto status = builder->commit(opCtx, collection, onCreateEachFn, onCommitFn);
+            auto status = builder->commit(
+                opCtx, collection.getWritableCollection(opCtx), onCreateEachFn, onCommitFn);
             if (!status.isOK()) {
                 return status;
             }
-
-            // Eventually, we will obtain the timestamp for completing the index build from the
-            // commitIndexBuild oplog entry.
-            // The current logic for timestamping index completion is consistent with the
-            // IndexBuilder. See SERVER-38986 and SERVER-34896.
-            IndexTimestampHelper::setGhostCommitTimestampForCatalogWrite(opCtx, nss);
             wunit.commit();
             return Status::OK();
         });
 }
 
 bool IndexBuildsManager::abortIndexBuild(OperationContext* opCtx,
-                                         Collection* collection,
+                                         CollectionWriter& collection,
                                          const UUID& buildUUID,
                                          OnCleanUpFn onCleanUpFn) {
     auto builder = _getBuilder(buildUUID);
@@ -324,28 +345,29 @@ bool IndexBuildsManager::abortIndexBuild(OperationContext* opCtx,
     // Since abortIndexBuild is special in that it can be called by threads other than the index
     // builder, ensure the caller has an exclusive lock.
     auto nss = collection->ns();
-    UncommittedCollections::get(opCtx).invariantHasExclusiveAccessToCollection(opCtx, nss);
+    CollectionCatalog::invariantHasExclusiveAccessToCollection(opCtx, nss);
 
     builder.getValue()->abortIndexBuild(opCtx, collection, onCleanUpFn);
     return true;
 }
 
 bool IndexBuildsManager::abortIndexBuildWithoutCleanup(OperationContext* opCtx,
-                                                       Collection* collection,
+                                                       const CollectionPtr& collection,
                                                        const UUID& buildUUID,
-                                                       const std::string& reason) {
+                                                       bool isResumable) {
     auto builder = _getBuilder(buildUUID);
     if (!builder.isOK()) {
         return false;
     }
 
     LOGV2(20347,
-          "Index build aborted without cleanup: {buildUUID}: {reason}",
-          "Index build aborted without cleanup",
+          "Index build: aborted without cleanup",
           "buildUUID"_attr = buildUUID,
-          "reason"_attr = reason);
+          "collectionUUID"_attr = collection->uuid(),
+          logAttrs(collection->ns()));
+
+    builder.getValue()->abortWithoutCleanup(opCtx, collection, isResumable);
 
-    builder.getValue()->abortWithoutCleanup(opCtx);
     return true;
 }
 
@@ -354,6 +376,17 @@ bool IndexBuildsManager::isBackgroundBuilding(const UUID& buildUUID) {
     return builder->isBackgroundBuilding();
 }
 
+void IndexBuildsManager::appendBuildInfo(const UUID& buildUUID, BSONObjBuilder* builder) const {
+    stdx::unique_lock<Latch> lk(_mutex);
+
+    auto builderIt = _builders.find(buildUUID);
+    if (builderIt == _builders.end()) {
+        return;
+    }
+
+    builderIt->second->appendBuildInfo(builder);
+}
+
 void IndexBuildsManager::verifyNoIndexBuilds_forTestOnly() {
     invariant(_builders.empty());
 }
@@ -383,70 +416,4 @@ StatusWith<MultiIndexBlock*> IndexBuildsManager::_getBuilder(const UUID& buildUU
     }
     return builderIt->second.get();
 }
-
-StatusWith<int> IndexBuildsManager::_moveRecordToLostAndFound(OperationContext* opCtx,
-                                                              NamespaceString nss,
-                                                              NamespaceString lostAndFoundNss,
-                                                              RecordId dupRecord) {
-    invariant(opCtx->lockState()->isCollectionLockedForMode(nss, MODE_IX));
-
-    auto originalCollection = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
-    Collection* localCollection =
-        CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, lostAndFoundNss);
-
-    // Create the collection if it doesn't exist.
-    if (!localCollection) {
-        Status status =
-            writeConflictRetry(opCtx, "createLostAndFoundCollection", lostAndFoundNss.ns(), [&]() {
-                WriteUnitOfWork wuow(opCtx);
-                AutoGetOrCreateDb autoDb(opCtx, NamespaceString::kLocalDb, MODE_X);
-                Database* db = autoDb.getDb();
-
-                // Ensure the database exists.
-                invariant(db);
-
-                // Since we are potentially deleting a document with duplicate _id values, we need
-                // to be able to insert into the lost and found collection without generating any
-                // duplicate key errors on the _id value.
-                CollectionOptions collOptions;
-                collOptions.setNoIdIndex();
-                localCollection = db->createCollection(opCtx, lostAndFoundNss, collOptions);
-
-                // Ensure the collection exists.
-                invariant(localCollection);
-
-                wuow.commit();
-                return Status::OK();
-            });
-        if (!status.isOK()) {
-            return status;
-        }
-    }
-
-    return writeConflictRetry(
-        opCtx, "writeDupDocToLostAndFoundCollection", nss.ns(), [&]() -> StatusWith<int> {
-            WriteUnitOfWork wuow(opCtx);
-            Snapshotted<BSONObj> doc;
-            int docSize = 0;
-
-            if (!originalCollection->findDoc(opCtx, dupRecord, &doc)) {
-                return docSize;
-            } else {
-                docSize = doc.value().objsize();
-            }
-
-            // Write document to lost_and_found collection and delete from original collection.
-            Status status =
-                localCollection->insertDocument(opCtx, InsertStatement(doc.value()), nullptr);
-            if (!status.isOK()) {
-                return status;
-            }
-
-            originalCollection->deleteDocument(opCtx, kUninitializedStmtId, dupRecord, nullptr);
-
-            wuow.commit();
-            return docSize;
-        });
-}
-
-}  // namespace mongo
\ No newline at end of file
+}  // namespace mongo
