@@ -33,6 +33,7 @@
 #include <vector>
 
 #include "mongo/base/clonable_ptr.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
 #include "mongo/db/catalog/index_catalog_entry.h"
 #include "mongo/db/index/multikey_paths.h"
 #include "mongo/db/jsobj.h"
@@ -45,6 +46,7 @@ namespace mongo {
 
 class Client;
 class Collection;
+class CollectionPtr;
 
 class IndexDescriptor;
 struct InsertDeleteOptions;
@@ -55,6 +57,14 @@ struct BsonRecord {
     const BSONObj* docPtr;
 };
 
+/**
+ * CheckRecordId indicates whether to confirm that the recordId matches the element we are
+ * removing when unindexing. When deleting documents, it is set to 'Off' by default to allow
+ * WiredTiger to do blind unindexing for efficacy. When set to 'On', disables blind deletes and
+ * forces recordid-matching for unindex operations.
+ */
+enum class CheckRecordId { Off, On };
+
 enum class IndexBuildMethod {
     /**
      * Use a collection scan to dump all keys into an external sorter. During this process,
@@ -65,11 +75,6 @@ enum class IndexBuildMethod {
      * This is the default for all index builds.
      */
     kHybrid,
-    /**
-     * Perform a collection scan by writing each document's generated key directly into the index.
-     * Accept writes in the background into the index as well.
-     */
-    kBackground,
     /**
      * Perform a collection scan to dump all keys into the external sorter, then into the index.
      * During this process, callers guarantee that no writes will be accepted on this collection.
@@ -96,7 +101,20 @@ enum class CreateIndexEntryFlags : int {
      * built. This is used when starting a node in standalone mode and a two-phase index build
      * is incomplete. kIsReady must not also be set.
      */
-    kFrozen = 0x4
+    kFrozen = 0x4,
+    /**
+     * kUpdateMetadata indicates that the index metadata in the storage engine should be updated
+     * based on changes to the IndexDescriptor.
+     * This is used when converting a regular index to a unique index through the collMod command.
+     */
+    kUpdateMetadata = 0x8,
+    /**
+     * kForceUpdateMetadata indicates bypassing the checks when updating the index metadata in the
+     * storage engine.
+     * This is used when forcing the conversion of a unique index to a non-unique index through the
+     * collMod command.
+     */
+    kForceUpdateMetadata = 0x10,
 };
 
 inline bool operator&(CreateIndexEntryFlags lhs, CreateIndexEntryFlags rhs) {
@@ -148,7 +166,7 @@ public:
 
     class ReadyIndexesIterator : public IndexIterator {
     public:
-        ReadyIndexesIterator(OperationContext* const opCtx,
+        ReadyIndexesIterator(OperationContext* opCtx,
                              IndexCatalogEntryContainer::const_iterator beginIterator,
                              IndexCatalogEntryContainer::const_iterator endIterator);
 
@@ -167,7 +185,7 @@ public:
          * on. If the caller will keep control of the container for the entire iterator lifetime,
          * it should pass in a null value.
          */
-        AllIndexesIterator(OperationContext* const opCtx,
+        AllIndexesIterator(OperationContext* opCtx,
                            std::unique_ptr<std::vector<IndexCatalogEntry*>> ownedContainer);
 
     private:
@@ -188,11 +206,16 @@ public:
     IndexCatalog() = default;
     virtual ~IndexCatalog() = default;
 
-    inline IndexCatalog(IndexCatalog&&) = delete;
-    inline IndexCatalog& operator=(IndexCatalog&&) = delete;
+    IndexCatalog(const IndexCatalog&) = default;
+    IndexCatalog& operator=(const IndexCatalog&) = default;
+
+    IndexCatalog(IndexCatalog&&) = delete;
+    IndexCatalog& operator=(IndexCatalog&&) = delete;
+
+    virtual std::unique_ptr<IndexCatalog> clone() const = 0;
 
     // Must be called before used.
-    virtual Status init(OperationContext* const opCtx) = 0;
+    virtual Status init(OperationContext* opCtx, Collection* collection) = 0;
 
     // ---- accessors -----
 
@@ -200,20 +223,20 @@ public:
 
     virtual bool haveAnyIndexesInProgress() const = 0;
 
-    virtual int numIndexesTotal(OperationContext* const opCtx) const = 0;
+    virtual int numIndexesTotal(OperationContext* opCtx) const = 0;
 
-    virtual int numIndexesReady(OperationContext* const opCtx) const = 0;
+    virtual int numIndexesReady(OperationContext* opCtx) const = 0;
 
-    virtual int numIndexesInProgress(OperationContext* const opCtx) const = 0;
+    virtual int numIndexesInProgress(OperationContext* opCtx) const = 0;
 
-    virtual bool haveIdIndex(OperationContext* const opCtx) const = 0;
+    virtual bool haveIdIndex(OperationContext* opCtx) const = 0;
 
     /**
      * Returns the spec for the id index to create by default for this collection.
      */
-    virtual BSONObj getDefaultIdIndexSpec() const = 0;
+    virtual BSONObj getDefaultIdIndexSpec(const CollectionPtr& collection) const = 0;
 
-    virtual const IndexDescriptor* findIdIndex(OperationContext* const opCtx) const = 0;
+    virtual const IndexDescriptor* findIdIndex(OperationContext* opCtx) const = 0;
 
     /**
      * Find index by name.  The index name uniquely identifies an index.
@@ -221,25 +244,21 @@ public:
      * @return null if cannot find
      */
     virtual const IndexDescriptor* findIndexByName(
-        OperationContext* const opCtx,
-        const StringData name,
-        const InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const = 0;
+        OperationContext* opCtx,
+        StringData name,
+        InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const = 0;
 
     /**
-     * Find index by matching key pattern and collation spec.  The key pattern and collation spec
-     * uniquely identify an index.
+     * Find index by matching key pattern and options. The key pattern, collation spec, and partial
+     * filter expression together uniquely identify an index.
      *
-     * Collation is specified as a normalized collation spec as returned by
-     * CollationInterface::getSpec.  An empty object indicates the simple collation.
-     *
-     * @return null if cannot find index, otherwise the index with a matching key pattern and
-     * collation.
+     * @return null if cannot find index, otherwise the index with a matching signature.
      */
-    virtual const IndexDescriptor* findIndexByKeyPatternAndCollationSpec(
-        OperationContext* const opCtx,
+    virtual const IndexDescriptor* findIndexByKeyPatternAndOptions(
+        OperationContext* opCtx,
         const BSONObj& key,
-        const BSONObj& collationSpec,
-        const InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const = 0;
+        const BSONObj& indexSpec,
+        InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const = 0;
 
     /**
      * Find indexes with a matching key pattern, putting them into the vector 'matches'.  The key
@@ -247,36 +266,15 @@ public:
      *
      * Consider using 'findIndexByName' if expecting to match one index.
      */
-    virtual void findIndexesByKeyPattern(
-        OperationContext* const opCtx,
-        const BSONObj& key,
-        const InclusionPolicy inclusionPolicy,
-        std::vector<const IndexDescriptor*>* const matches) const = 0;
-
-    /**
-     * Returns an index suitable for shard key range scans.
-     *
-     * This index:
-     * - must be prefixed by 'shardKey', and
-     * - must not be a partial index.
-     * - must have the simple collation.
-     *
-     * If the parameter 'requireSingleKey' is true, then this index additionally must not be
-     * multi-key.
-     *
-     * If no such index exists, returns NULL.
-     */
-    virtual const IndexDescriptor* findShardKeyPrefixedIndex(
-        OperationContext* const opCtx,
-        const BSONObj& shardKey,
-        const bool requireSingleKey,
-        std::string* errMsg = nullptr) const = 0;
-
+    virtual void findIndexesByKeyPattern(OperationContext* opCtx,
+                                         const BSONObj& key,
+                                         InclusionPolicy inclusionPolicy,
+                                         std::vector<const IndexDescriptor*>* matches) const = 0;
     virtual void findIndexByType(
-        OperationContext* const opCtx,
+        OperationContext* opCtx,
         const std::string& type,
         std::vector<const IndexDescriptor*>& matches,
-        const InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const = 0;
+        InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const = 0;
 
     /**
      * Reload the index definition for 'oldDesc' from the CollectionCatalogEntry.  'oldDesc'
@@ -290,14 +288,16 @@ public:
      * The caller must hold the collection X lock and ensure no index builds are in progress
      * on the collection.
      */
-    virtual const IndexDescriptor* refreshEntry(OperationContext* const opCtx,
-                                                const IndexDescriptor* const oldDesc) = 0;
+    virtual const IndexDescriptor* refreshEntry(OperationContext* opCtx,
+                                                Collection* collection,
+                                                const IndexDescriptor* oldDesc,
+                                                CreateIndexEntryFlags flags) = 0;
 
     /**
      * Returns a pointer to the index catalog entry associated with 'desc'. Throws if there is no
      * such index. Never returns nullptr.
      */
-    virtual const IndexCatalogEntry* getEntry(const IndexDescriptor* const desc) const = 0;
+    virtual const IndexCatalogEntry* getEntry(const IndexDescriptor* desc) const = 0;
 
     /**
      * Returns a pointer to the index catalog entry associated with 'desc', where the caller assumes
@@ -305,6 +305,7 @@ public:
      */
     virtual std::shared_ptr<const IndexCatalogEntry> getEntryShared(
         const IndexDescriptor*) const = 0;
+    virtual std::shared_ptr<IndexCatalogEntry> getEntryShared(const IndexDescriptor*) = 0;
 
     /**
      * Returns a vector of shared pointers to all index entries. Excludes unfinished indexes.
@@ -316,7 +317,7 @@ public:
      * Returns an iterator for the index descriptors in this IndexCatalog.
      */
     virtual std::unique_ptr<IndexIterator> getIndexIterator(
-        OperationContext* const opCtx, const InclusionPolicy inclusionPolicy) const = 0;
+        OperationContext* opCtx, InclusionPolicy inclusionPolicy) const = 0;
 
     // ---- index set modifiers ------
 
@@ -326,6 +327,7 @@ public:
      */
 
     virtual IndexCatalogEntry* createIndexEntry(OperationContext* opCtx,
+                                                Collection* collection,
                                                 std::unique_ptr<IndexDescriptor> descriptor,
                                                 CreateIndexEntryFlags flags) = 0;
 
@@ -334,8 +336,9 @@ public:
      * empty collection can be rolled back as part of a larger WUOW. Returns the full specification
      * of the created index, as it is stored in this index catalog.
      */
-    virtual StatusWith<BSONObj> createIndexOnEmptyCollection(OperationContext* const opCtx,
-                                                             const BSONObj spec) = 0;
+    virtual StatusWith<BSONObj> createIndexOnEmptyCollection(OperationContext* opCtx,
+                                                             Collection* collection,
+                                                             BSONObj spec) = 0;
 
     /**
      * Checks the spec 'original' to make sure nothing is incorrectly set and cleans up any legacy
@@ -345,8 +348,11 @@ public:
      * IndexAlreadyExists if the index already exists; IndexBuildAlreadyInProgress if the index is
      * already being built.
      */
-    virtual StatusWith<BSONObj> prepareSpecForCreate(OperationContext* const opCtx,
-                                                     const BSONObj& original) const = 0;
+    virtual StatusWith<BSONObj> prepareSpecForCreate(
+        OperationContext* opCtx,
+        const CollectionPtr& collection,
+        const BSONObj& original,
+        const boost::optional<ResumeIndexInfo>& resumeInfo) const = 0;
 
     /**
      * Returns a copy of 'indexSpecsToBuild' that does not contain index specifications that already
@@ -363,9 +369,10 @@ public:
      * they are not, then IndexBuildAlreadyInProgress errors can be thrown.
      */
     virtual std::vector<BSONObj> removeExistingIndexes(
-        OperationContext* const opCtx,
+        OperationContext* opCtx,
+        const CollectionPtr& collection,
         const std::vector<BSONObj>& indexSpecsToBuild,
-        const bool removeIndexBuildsToo) const = 0;
+        bool removeIndexBuildsToo) const = 0;
 
     /**
      * Filters out ready and in-progress indexes that already exist and returns the remaining
@@ -378,7 +385,19 @@ public:
      * via replica set cloning or chunk migrations.
      */
     virtual std::vector<BSONObj> removeExistingIndexesNoChecks(
-        OperationContext* const opCtx, const std::vector<BSONObj>& indexSpecsToBuild) const = 0;
+        OperationContext* opCtx,
+        const CollectionPtr& collection,
+        const std::vector<BSONObj>& indexSpecsToBuild) const = 0;
+
+    /**
+     * Drops indexes in the index catalog that returns true when it's descriptor returns true for
+     * 'matchFn'. If 'onDropFn' is provided, it will be called before each index is dropped to
+     * allow timestamping each individual drop.
+     */
+    virtual void dropIndexes(OperationContext* opCtx,
+                             Collection* collection,
+                             std::function<bool(const IndexDescriptor*)> matchFn,
+                             std::function<void(const IndexDescriptor*)> onDropFn) = 0;
 
     /**
      * Drops all indexes in the index catalog, optionally dropping the id index depending on the
@@ -386,9 +405,9 @@ public:
      * index is dropped to allow timestamping each individual drop.
      */
     virtual void dropAllIndexes(OperationContext* opCtx,
+                                Collection* collection,
                                 bool includingIdIndex,
                                 std::function<void(const IndexDescriptor*)> onDropFn) = 0;
-    virtual void dropAllIndexes(OperationContext* opCtx, bool includingIdIndex) = 0;
 
     /**
      * Drops the index given its descriptor.
@@ -396,56 +415,57 @@ public:
      * The caller must hold the collection X lock and ensure no index builds are in progress on the
      * collection.
      */
-    virtual Status dropIndex(OperationContext* const opCtx, const IndexDescriptor* const desc) = 0;
+    virtual Status dropIndex(OperationContext* opCtx,
+                             Collection* collection,
+                             const IndexDescriptor* desc) = 0;
+
+    /**
+     * Resets the index given its descriptor.
+     *
+     * This can only be called during startup recovery as it involves recreating the index table to
+     * allow bulk cursors to be used again.
+     */
+    virtual Status resetUnfinishedIndexForRecovery(OperationContext* opCtx,
+                                                   Collection* collection,
+                                                   const IndexDescriptor* desc) = 0;
 
     /**
      * Drops an unfinished index given its descriptor.
      *
      * The caller must hold the collection X lock.
      */
-    virtual Status dropUnfinishedIndex(OperationContext* const opCtx,
-                                       const IndexDescriptor* const desc) = 0;
+    virtual Status dropUnfinishedIndex(OperationContext* opCtx,
+                                       Collection* collection,
+                                       const IndexDescriptor* desc) = 0;
 
     /**
      * Drops the index given its catalog entry.
      *
      * The caller must hold the collection X lock.
      */
-    virtual Status dropIndexEntry(OperationContext* opCtx, IndexCatalogEntry* entry) = 0;
+    virtual Status dropIndexEntry(OperationContext* opCtx,
+                                  Collection* collection,
+                                  IndexCatalogEntry* entry) = 0;
 
     /**
      * Deletes the index from the durable catalog on disk.
      */
-    virtual void deleteIndexFromDisk(OperationContext* opCtx, const std::string& indexName) = 0;
+    virtual void deleteIndexFromDisk(OperationContext* opCtx,
+                                     Collection* collection,
+                                     const std::string& indexName) = 0;
 
     // ---- modify single index
 
-    /**
-     * Returns true if the index 'idx' is multikey, and returns false otherwise.
-     */
-    virtual bool isMultikey(const IndexDescriptor* const idx) = 0;
-
-    /**
-     * Returns the path components that cause the index 'idx' to be multikey if the index supports
-     * path-level multikey tracking, and returns an empty vector if path-level multikey tracking
-     * isn't supported.
-     *
-     * If the index supports path-level multikey tracking but isn't multikey, then this function
-     * returns a vector with size equal to the number of elements in the index key pattern where
-     * each element in the vector is an empty set.
-     */
-    virtual MultikeyPaths getMultikeyPaths(OperationContext* const opCtx,
-                                           const IndexDescriptor* const idx) = 0;
-
     /**
      * Sets the index 'desc' to be multikey with the provided 'multikeyPaths'.
      *
      * See IndexCatalogEntry::setMultikey().
      */
-    virtual void setMultikeyPaths(OperationContext* const opCtx,
-                                  const IndexDescriptor* const desc,
+    virtual void setMultikeyPaths(OperationContext* opCtx,
+                                  const CollectionPtr& coll,
+                                  const IndexDescriptor* desc,
                                   const KeyStringSet& multikeyMetadataKeys,
-                                  const MultikeyPaths& multikeyPaths) = 0;
+                                  const MultikeyPaths& multikeyPaths) const = 0;
 
     // ----- data modifiers ------
 
@@ -455,9 +475,10 @@ public:
      *
      * This method may throw.
      */
-    virtual Status indexRecords(OperationContext* const opCtx,
+    virtual Status indexRecords(OperationContext* opCtx,
+                                const CollectionPtr& collection,
                                 const std::vector<BsonRecord>& bsonRecords,
-                                int64_t* const keysInsertedOut) = 0;
+                                int64_t* keysInsertedOut) const = 0;
 
     /**
      * Both 'keysInsertedOut' and 'keysDeletedOut' are required and will be set to the number of
@@ -465,28 +486,31 @@ public:
      *
      * This method may throw.
      */
-    virtual Status updateRecord(OperationContext* const opCtx,
+    virtual Status updateRecord(OperationContext* opCtx,
+                                const CollectionPtr& coll,
                                 const BSONObj& oldDoc,
                                 const BSONObj& newDoc,
                                 const RecordId& recordId,
-                                int64_t* const keysInsertedOut,
-                                int64_t* const keysDeletedOut) = 0;
+                                int64_t* keysInsertedOut,
+                                int64_t* keysDeletedOut) const = 0;
 
     /**
      * When 'keysDeletedOut' is not null, it will be set to the number of index keys removed by
      * this operation.
      */
-    virtual void unindexRecord(OperationContext* const opCtx,
+    virtual void unindexRecord(OperationContext* opCtx,
+                               const CollectionPtr& collection,
                                const BSONObj& obj,
                                const RecordId& loc,
-                               const bool noWarn,
-                               int64_t* const keysDeletedOut) = 0;
+                               bool noWarn,
+                               int64_t* keysDeletedOut,
+                               CheckRecordId checkRecordId = CheckRecordId::Off) const = 0;
 
     /*
      * Attempt compaction on all ready indexes to regain disk space, if the storage engine's index
      * supports compaction in-place.
      */
-    virtual Status compactIndexes(OperationContext* opCtx) = 0;
+    virtual Status compactIndexes(OperationContext* opCtx) const = 0;
 
     virtual std::string getAccessMethodName(const BSONObj& keyPattern) = 0;
 
@@ -508,10 +532,13 @@ public:
      * index constraints, as needed by replication.
      */
     virtual void prepareInsertDeleteOptions(OperationContext* opCtx,
+                                            const NamespaceString& ns,
                                             const IndexDescriptor* desc,
                                             InsertDeleteOptions* options) const = 0;
 
-    virtual void indexBuildSuccess(OperationContext* opCtx, IndexCatalogEntry* index) = 0;
+    virtual void indexBuildSuccess(OperationContext* opCtx,
+                                   Collection* coll,
+                                   IndexCatalogEntry* index) = 0;
 };
 
 inline IndexCatalog::InclusionPolicy operator|(IndexCatalog::InclusionPolicy lhs,
