@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kIndex
 
 #include "mongo/platform/basic.h"
 
@@ -39,57 +38,59 @@
 #include "mongo/base/init.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/index/index_access_method.h"
 #include "mongo/db/index/index_descriptor.h"
-#include "mongo/db/logical_clock.h"
 #include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/multi_key_path_tracker.h"
-#include "mongo/db/op_observer.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/query/collection_query_info.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/durable_catalog.h"
-#include "mongo/db/transaction_participant.h"
+#include "mongo/db/timeseries/timeseries_index_schema_conversion_functions.h"
+#include "mongo/db/transaction/transaction_participant.h"
 #include "mongo/logv2/log.h"
 #include "mongo/util/scopeguard.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kIndex
+
+
 namespace mongo {
 MONGO_FAIL_POINT_DEFINE(skipUpdateIndexMultikey);
 
 using std::string;
 
 IndexCatalogEntryImpl::IndexCatalogEntryImpl(OperationContext* const opCtx,
+                                             const CollectionPtr& collection,
                                              const std::string& ident,
                                              std::unique_ptr<IndexDescriptor> descriptor,
-                                             CollectionQueryInfo* const queryInfo,
                                              bool isFrozen)
     : _ident(ident),
       _descriptor(std::move(descriptor)),
-      _queryInfo(queryInfo),
+      _catalogId(collection->getCatalogId()),
       _ordering(Ordering::make(_descriptor->keyPattern())),
       _isReady(false),
       _isFrozen(isFrozen),
+      _shouldValidateDocument(false),
       _isDropped(false),
-      _prefix(DurableCatalog::get(opCtx)->getIndexPrefix(
-          opCtx, _descriptor->getCollection()->getCatalogId(), _descriptor->indexName())) {
-    _descriptor->_cachedEntry = this;
+      _indexOffset(invariantStatusOK(
+          collection->checkMetaDataForIndex(_descriptor->indexName(), _descriptor->infoObj()))) {
 
-    _isReady = _catalogIsReady(opCtx);
+    _descriptor->_entry = this;
+    _isReady = collection->isIndexReady(_descriptor->indexName());
 
-    {
-        stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
-        const bool isMultikey = _catalogIsMultikey(opCtx, &_indexMultikeyPathsForWrite);
-        _isMultikeyForRead.store(isMultikey);
-        _isMultikeyForWrite.store(isMultikey);
-        _indexMultikeyPathsForRead = _indexMultikeyPathsForWrite;
-        _numUncommittedCatalogMultikeyUpdates = 0;
-        _indexTracksMultikeyPathsInCatalog = !_indexMultikeyPathsForWrite.empty();
-    }
+    // For time-series collections, we need to check that the indexed metric fields do not have
+    // expanded array values.
+    _shouldValidateDocument =
+        collection->getTimeseriesOptions() &&
+        timeseries::doesBucketsIndexIncludeMeasurement(
+            opCtx, collection->ns(), *collection->getTimeseriesOptions(), _descriptor->infoObj());
 
+    auto nss = DurableCatalog::get(opCtx)->getEntry(_catalogId).nss;
     const BSONObj& collation = _descriptor->collation();
     if (!collation.isEmpty()) {
         auto statusWithCollator =
@@ -105,36 +106,24 @@ IndexCatalogEntryImpl::IndexCatalogEntryImpl(OperationContext* const opCtx,
         const BSONObj& filter = _descriptor->partialFilterExpression();
 
         _expCtxForFilter = make_intrusive<ExpressionContext>(
-            opCtx, CollatorInterface::cloneCollator(_collator.get()), ns());
+            opCtx, CollatorInterface::cloneCollator(_collator.get()), nss);
 
         // Parsing the partial filter expression is not expected to fail here since the
         // expression would have been successfully parsed upstream during index creation.
-        StatusWithMatchExpression statusWithMatcher =
-            MatchExpressionParser::parse(filter,
-                                         _expCtxForFilter,
-                                         ExtensionsCallbackNoop(),
-                                         MatchExpressionParser::kBanAllSpecialFeatures);
-        invariant(statusWithMatcher.getStatus());
-        _filterExpression = std::move(statusWithMatcher.getValue());
+        _filterExpression =
+            MatchExpressionParser::parseAndNormalize(filter,
+                                                     _expCtxForFilter,
+                                                     ExtensionsCallbackNoop(),
+                                                     MatchExpressionParser::kBanAllSpecialFeatures);
         LOGV2_DEBUG(20350,
                     2,
-                    "have filter expression for {ns} {descriptor_indexName} {filter}",
-                    "ns"_attr = ns(),
-                    "descriptor_indexName"_attr = _descriptor->indexName(),
+                    "have filter expression for {namespace} {indexName} {filter}",
+                    "namespace"_attr = nss,
+                    "indexName"_attr = _descriptor->indexName(),
                     "filter"_attr = redact(filter));
     }
 }
 
-IndexCatalogEntryImpl::~IndexCatalogEntryImpl() {
-    _descriptor->_cachedEntry = nullptr;  // defensive
-
-    _descriptor.reset();
-}
-
-const NamespaceString& IndexCatalogEntryImpl::ns() const {
-    return _descriptor->parentNS();
-}
-
 void IndexCatalogEntryImpl::setAccessMethod(std::unique_ptr<IndexAccessMethod> accessMethod) {
     invariant(!_accessMethod);
     _accessMethod = std::move(accessMethod);
@@ -146,7 +135,7 @@ bool IndexCatalogEntryImpl::isReady(OperationContext* opCtx) const {
     // out-of-sync index catalog entries.  To fix this, we uassert if we detect that the
     // in-memory catalog is out-of-sync with the on-disk catalog.
     if (opCtx->inMultiDocumentTransaction()) {
-        if (!_catalogIsPresent(opCtx) || _catalogIsReady(opCtx) != _isReady) {
+        if (!isPresentInMySnapshot(opCtx) || isReadyInMySnapshot(opCtx) != _isReady) {
             uasserted(ErrorCodes::SnapshotUnavailable,
                       str::stream() << "Unable to read from a snapshot due to pending collection"
                                        " catalog changes; please retry the operation.");
@@ -154,7 +143,7 @@ bool IndexCatalogEntryImpl::isReady(OperationContext* opCtx) const {
     }
 
     if (kDebugBuild)
-        invariant(_isReady == _catalogIsReady(opCtx));
+        invariant(_isReady == isReadyInMySnapshot(opCtx));
     return _isReady;
 }
 
@@ -163,19 +152,27 @@ bool IndexCatalogEntryImpl::isFrozen() const {
     return _isFrozen;
 }
 
-bool IndexCatalogEntryImpl::isMultikey() const {
-    return _isMultikeyForRead.load();
+bool IndexCatalogEntryImpl::shouldValidateDocument() const {
+    return _shouldValidateDocument;
 }
 
-MultikeyPaths IndexCatalogEntryImpl::getMultikeyPaths(OperationContext* opCtx) const {
-    stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
-    return _indexMultikeyPathsForRead;
+bool IndexCatalogEntryImpl::isMultikey(OperationContext* const opCtx,
+                                       const CollectionPtr& collection) const {
+    return _catalogIsMultikey(opCtx, collection, nullptr);
+}
+
+MultikeyPaths IndexCatalogEntryImpl::getMultikeyPaths(OperationContext* opCtx,
+                                                      const CollectionPtr& collection) const {
+    MultikeyPaths indexMultikeyPathsForRead;
+    [[maybe_unused]] const bool isMultikeyInCatalog =
+        _catalogIsMultikey(opCtx, collection, &indexMultikeyPathsForRead);
+    return indexMultikeyPathsForRead;
 }
 
 // ---
 
 void IndexCatalogEntryImpl::setMinimumVisibleSnapshot(Timestamp newMinimumVisibleSnapshot) {
-    if (!_minVisibleSnapshot || (newMinimumVisibleSnapshot > _minVisibleSnapshot.get())) {
+    if (!_minVisibleSnapshot || (newMinimumVisibleSnapshot > _minVisibleSnapshot.value())) {
         _minVisibleSnapshot = newMinimumVisibleSnapshot;
     }
 }
@@ -184,27 +181,34 @@ void IndexCatalogEntryImpl::setIsReady(bool newIsReady) {
     _isReady = newIsReady;
 }
 
+void IndexCatalogEntryImpl::setIsFrozen(bool newIsFrozen) {
+    _isFrozen = newIsFrozen;
+}
+
 void IndexCatalogEntryImpl::setMultikey(OperationContext* opCtx,
+                                        const CollectionPtr& collection,
                                         const KeyStringSet& multikeyMetadataKeys,
-                                        const MultikeyPaths& multikeyPaths) {
+                                        const MultikeyPaths& multikeyPaths) const {
     // An index can either track path-level multikey information in the catalog or as metadata keys
     // in the index itself, but not both.
-    invariant(!(_indexTracksMultikeyPathsInCatalog && multikeyMetadataKeys.size() > 0));
+    MultikeyPaths indexMultikeyPathsForWrite;
+    auto isMultikeyForWrite = _catalogIsMultikey(opCtx, collection, &indexMultikeyPathsForWrite);
+    auto indexTracksMultikeyPathsInCatalog = !indexMultikeyPathsForWrite.empty();
+    invariant(!(indexTracksMultikeyPathsInCatalog && multikeyMetadataKeys.size() > 0));
     // If the index is already set as multikey and we don't have any path-level information to
     // update, then there's nothing more for us to do.
-    bool hasNoPathLevelInfo = (!_indexTracksMultikeyPathsInCatalog && multikeyMetadataKeys.empty());
-    if (hasNoPathLevelInfo && _isMultikeyForWrite.load()) {
+    bool hasNoPathLevelInfo = (!indexTracksMultikeyPathsInCatalog && multikeyMetadataKeys.empty());
+    if (hasNoPathLevelInfo && isMultikeyForWrite) {
         return;
     }
 
-    if (_indexTracksMultikeyPathsInCatalog) {
-        stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
-        invariant(multikeyPaths.size() == _indexMultikeyPathsForWrite.size());
+    if (indexTracksMultikeyPathsInCatalog) {
+        invariant(multikeyPaths.size() == indexMultikeyPathsForWrite.size());
 
         bool newPathIsMultikey = false;
         for (size_t i = 0; i < multikeyPaths.size(); ++i) {
-            if (!std::includes(_indexMultikeyPathsForWrite[i].begin(),
-                               _indexMultikeyPathsForWrite[i].end(),
+            if (!std::includes(indexMultikeyPathsForWrite[i].begin(),
+                               indexMultikeyPathsForWrite[i].end(),
                                multikeyPaths[i].begin(),
                                multikeyPaths[i].end())) {
                 // If 'multikeyPaths' contains a new path component that causes this index to be
@@ -225,7 +229,7 @@ void IndexCatalogEntryImpl::setMultikey(OperationContext* opCtx,
         return;
     }
 
-    MultikeyPaths paths = _indexTracksMultikeyPathsInCatalog ? multikeyPaths : MultikeyPaths{};
+    MultikeyPaths paths = indexTracksMultikeyPathsInCatalog ? multikeyPaths : MultikeyPaths{};
 
     // On a primary, we can simply assign this write the same timestamp as the index creation,
     // insert, or update that caused this index to become multikey. This is because if two
@@ -248,7 +252,6 @@ void IndexCatalogEntryImpl::setMultikey(OperationContext* opCtx,
     // it multikey, that write will be marked as "isTrackingMultikeyPathInfo" on the applier's
     // OperationContext and we can safely defer that write to the end of the batch.
     if (MultikeyPathTracker::get(opCtx).isTrackingMultikeyPathInfo()) {
-        auto collection = _descriptor->getCollection();
         MultikeyPathTracker::get(opCtx).addMultikeyPathInfo({collection->ns(),
                                                              collection->uuid(),
                                                              _descriptor->indexName(),
@@ -261,24 +264,44 @@ void IndexCatalogEntryImpl::setMultikey(OperationContext* opCtx,
     // not have to account for potential dupes, since all metadata keys are indexed against a single
     // RecordId. An attempt to write a duplicate key will therefore be ignored.
     if (!multikeyMetadataKeys.empty()) {
-        uassertStatusOK(
-            accessMethod()->insertKeys(opCtx, multikeyMetadataKeys, {}, {}, {}, nullptr));
+        uassertStatusOK(accessMethod()->asSortedData()->insertKeys(
+            opCtx, collection, multikeyMetadataKeys, {}, {}, nullptr));
     }
 
     // Mark the catalog as multikey, and record the multikey paths if applicable.
     if (opCtx->inMultiDocumentTransaction()) {
-        auto status = _setMultikeyInMultiDocumentTransaction(opCtx, paths);
+        auto status = _setMultikeyInMultiDocumentTransaction(opCtx, collection, paths);
         // Retry without side transaction.
         if (!status.isOK()) {
-            _catalogSetMultikey(opCtx, paths);
+            _catalogSetMultikey(opCtx, collection, paths);
         }
     } else {
-        _catalogSetMultikey(opCtx, paths);
+        _catalogSetMultikey(opCtx, collection, paths);
     }
 }
 
+void IndexCatalogEntryImpl::forceSetMultikey(OperationContext* const opCtx,
+                                             const CollectionPtr& coll,
+                                             bool isMultikey,
+                                             const MultikeyPaths& multikeyPaths) const {
+    invariant(opCtx->lockState()->isCollectionLockedForMode(coll->ns(), MODE_X));
+
+    // Don't check _indexTracksMultikeyPathsInCatalog because the caller may be intentionally trying
+    // to bypass this check. That is, pre-3.4 indexes may be 'stuck' in a state where they are not
+    // tracking multikey paths in the catalog (i.e. the multikeyPaths field is absent), but the
+    // caller wants to upgrade this index because it knows exactly which paths are multikey. We rely
+    // on the following function to make sure this upgrade only takes place on index types that
+    // currently support path-level multikey path tracking.
+    coll->forceSetIndexIsMultikey(opCtx, _descriptor.get(), isMultikey, multikeyPaths);
+
+    // Since multikey metadata has changed, invalidate the query cache.
+    CollectionQueryInfo::get(coll).clearQueryCacheForSetMultikey(coll);
+}
+
 Status IndexCatalogEntryImpl::_setMultikeyInMultiDocumentTransaction(
-    OperationContext* opCtx, const MultikeyPaths& multikeyPaths) {
+    OperationContext* opCtx,
+    const CollectionPtr& collection,
+    const MultikeyPaths& multikeyPaths) const {
     // If we are inside a multi-document transaction, we write the on-disk multikey update in a
     // separate transaction so that it will not generate prepare conflicts with other operations
     // that try to set the multikey flag. In general, it should always be safe to update the
@@ -292,12 +315,12 @@ Status IndexCatalogEntryImpl::_setMultikeyInMultiDocumentTransaction(
     // If the index is not visible within the side transaction, the index may have been created,
     // but not committed, in the parent transaction. Therefore, we abandon the side transaction
     // and set the multikey flag in the parent transaction.
-    if (!_catalogIsPresent(opCtx)) {
+    if (!isPresentInMySnapshot(opCtx)) {
         return {ErrorCodes::SnapshotUnavailable, "index not visible in side transaction"};
     }
 
     writeConflictRetry(
-        opCtx, "set index multikey", ns().ns(), [&] {
+        opCtx, "set index multikey", collection->ns().ns(), [&] {
             WriteUnitOfWork wuow(opCtx);
 
             // If we have a prepare optime for recovery, then we always use that. This is safe since
@@ -313,12 +336,24 @@ Status IndexCatalogEntryImpl::_setMultikeyInMultiDocumentTransaction(
                 // may not have a stable timestamp. Therefore, we need to round up
                 // the multi-key write timestamp to the max of the three so that we don't write
                 // behind the oldest/stable timestamp. This code path is only hit during initial
-                // sync/recovery when reconstructing prepared transactions and so we don't expect
+                // sync/recovery when reconstructing prepared transactions, and so we don't expect
                 // the oldest/stable timestamp to advance concurrently.
+                //
+                // WiredTiger disallows committing at the stable timestamp to avoid confusion during
+                // checkpoints, to overcome that we allow setting the timestamp slightly after the
+                // prepared timestamp of the original transaction. This is currently not an issue as
+                // the index metadata state is read from in-memory cache and this is modifying the
+                // state on-disk from the _mdb_catalog document. To put in other words, multikey
+                // doesn't go backwards. This would be a problem if we move to a versioned catalog
+                // world as a different transaction could choose an earlier timestamp (i.e. the
+                // original transaction timestamp) and encounter an invalid system state where the
+                // document that enables multikey hasn't enabled it yet but is present in the
+                // collection. In other words, the index is not set for multikey but there is
+                // already data present that relies on it.
                 auto status = opCtx->recoveryUnit()->setTimestamp(std::max(
                     {recoveryPrepareOpTime.getTimestamp(),
                      opCtx->getServiceContext()->getStorageEngine()->getOldestTimestamp(),
-                     opCtx->getServiceContext()->getStorageEngine()->getStableTimestamp()}));
+                     opCtx->getServiceContext()->getStorageEngine()->getStableTimestamp() + 1}));
                 fassert(31164, status);
             } else {
                 // If there is no recovery prepare OpTime, then this node must be a primary. We
@@ -327,11 +362,12 @@ Status IndexCatalogEntryImpl::_setMultikeyInMultiDocumentTransaction(
 
                 auto msg = BSON("msg"
                                 << "Setting index to multikey"
-                                << "coll" << ns().ns() << "index" << _descriptor->indexName());
+                                << "coll" << collection->ns().ns() << "index"
+                                << _descriptor->indexName());
                 opCtx->getClient()->getServiceContext()->getOpObserver()->onOpMessage(opCtx, msg);
             }
 
-            _catalogSetMultikey(opCtx, multikeyPaths);
+            _catalogSetMultikey(opCtx, collection, multikeyPaths);
 
             wuow.commit();
         });
@@ -339,92 +375,51 @@ Status IndexCatalogEntryImpl::_setMultikeyInMultiDocumentTransaction(
     return Status::OK();
 }
 
-// ----
-
-bool IndexCatalogEntryImpl::_catalogIsReady(OperationContext* opCtx) const {
-    return DurableCatalog::get(opCtx)->isIndexReady(
-        opCtx, _descriptor->getCollection()->getCatalogId(), _descriptor->indexName());
+std::shared_ptr<Ident> IndexCatalogEntryImpl::getSharedIdent() const {
+    return _accessMethod ? std::shared_ptr<Ident>{shared_from_this(), _accessMethod->getIdentPtr()}
+                         : nullptr;
 }
 
-bool IndexCatalogEntryImpl::_catalogIsPresent(OperationContext* opCtx) const {
-    return DurableCatalog::get(opCtx)->isIndexPresent(
-        opCtx, _descriptor->getCollection()->getCatalogId(), _descriptor->indexName());
+// ----
+
+NamespaceString IndexCatalogEntryImpl::getNSSFromCatalog(OperationContext* opCtx) const {
+    return DurableCatalog::get(opCtx)->getEntry(_catalogId).nss;
+}
+
+bool IndexCatalogEntryImpl::isReadyInMySnapshot(OperationContext* opCtx) const {
+    return DurableCatalog::get(opCtx)->isIndexReady(opCtx, _catalogId, _descriptor->indexName());
+}
+
+bool IndexCatalogEntryImpl::isPresentInMySnapshot(OperationContext* opCtx) const {
+    return DurableCatalog::get(opCtx)->isIndexPresent(opCtx, _catalogId, _descriptor->indexName());
 }
 
 bool IndexCatalogEntryImpl::_catalogIsMultikey(OperationContext* opCtx,
+                                               const CollectionPtr& collection,
                                                MultikeyPaths* multikeyPaths) const {
-    return DurableCatalog::get(opCtx)->isIndexMultikey(opCtx,
-                                                       _descriptor->getCollection()->getCatalogId(),
-                                                       _descriptor->indexName(),
-                                                       multikeyPaths);
+    return collection->isIndexMultikey(
+        opCtx, _descriptor->indexName(), multikeyPaths, _indexOffset);
 }
 
 void IndexCatalogEntryImpl::_catalogSetMultikey(OperationContext* opCtx,
-                                                const MultikeyPaths& multikeyPaths) {
+                                                const CollectionPtr& collection,
+                                                const MultikeyPaths& multikeyPaths) const {
     // It's possible that the index type (e.g. ascending/descending index) supports tracking
     // path-level multikey information, but this particular index doesn't.
     // CollectionCatalogEntry::setIndexIsMultikey() requires that we discard the path-level
     // multikey information in order to avoid unintentionally setting path-level multikey
     // information on an index created before 3.4.
-    auto indexMetadataHasChanged =
-        DurableCatalog::get(opCtx)->setIndexIsMultikey(opCtx,
-                                                       _descriptor->getCollection()->getCatalogId(),
-                                                       _descriptor->indexName(),
-                                                       multikeyPaths);
+    auto indexMetadataHasChanged = collection->setIndexIsMultikey(
+        opCtx, _descriptor->indexName(), multikeyPaths, _indexOffset);
 
-    // In the absence of using the storage engine to read from the catalog, we must set multikey
-    // prior to the storage engine transaction committing.
-    //
-    // Moreover, there must not be an `onRollback` handler to reset this back to false. Given a long
-    // enough pause in processing `onRollback` handlers, a later writer that successfully flipped
-    // multikey can be undone. Alternatively, one could use a counter instead of a boolean to avoid
-    // that problem.
-    _isMultikeyForRead.store(true);
-    if (_indexTracksMultikeyPathsInCatalog) {
-        stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
-        if (_numUncommittedCatalogMultikeyUpdates == 0) {
-            _indexMultikeyPathsForRead = _indexMultikeyPathsForWrite;
-        }
-        for (size_t i = 0; i < multikeyPaths.size(); ++i) {
-            _indexMultikeyPathsForRead[i].insert(multikeyPaths[i].begin(), multikeyPaths[i].end());
-        }
-        _numUncommittedCatalogMultikeyUpdates++;
-    }
-    if (indexMetadataHasChanged && _queryInfo) {
+    if (indexMetadataHasChanged) {
         LOGV2_DEBUG(4718705,
                     1,
                     "Index set to multi key, clearing query plan cache",
-                    "namespace"_attr = ns(),
+                    "namespace"_attr = collection->ns(),
                     "keyPattern"_attr = _descriptor->keyPattern());
-        _queryInfo->clearQueryCache();
+        CollectionQueryInfo::get(collection).clearQueryCacheForSetMultikey(collection);
     }
-
-    opCtx->recoveryUnit()->onCommit([this](boost::optional<Timestamp>) {
-        // Writers must attempt to flip multikey until it's confirmed a storage engine
-        // transaction successfully commits. Only after this point may a writer optimize out
-        // flipping multikey.
-        _isMultikeyForWrite.store(true);
-        if (_indexTracksMultikeyPathsInCatalog) {
-            stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
-            _indexMultikeyPathsForWrite = _indexMultikeyPathsForRead;
-            _numUncommittedCatalogMultikeyUpdates--;
-            invariant(_numUncommittedCatalogMultikeyUpdates >= 0,
-                      str::stream() << _ident << descriptor()->indexName());
-        }
-    });
-    opCtx->recoveryUnit()->onRollback([this] {
-        if (_indexTracksMultikeyPathsInCatalog) {
-            stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
-            _numUncommittedCatalogMultikeyUpdates--;
-            invariant(_numUncommittedCatalogMultikeyUpdates >= 0,
-                      str::stream() << _ident << ":" << descriptor()->indexName());
-        }
-    });
-}
-
-KVPrefix IndexCatalogEntryImpl::_catalogGetPrefix(OperationContext* opCtx) const {
-    return DurableCatalog::get(opCtx)->getIndexPrefix(
-        opCtx, _descriptor->getCollection()->getCatalogId(), _descriptor->indexName());
 }
 
 }  // namespace mongo
