@@ -46,20 +46,26 @@ namespace mongo {
 
 class Client;
 class Collection;
+class CollectionPtr;
 
 class IndexDescriptor;
 struct InsertDeleteOptions;
 
 /**
- * how many: 1 per Collection.
- * lifecycle: attached to a Collection.
+ * IndexCatalogImpl is stored as a member of CollectionImpl. When the Collection is cloned this is
+ * cloned with it by making shallow copies of the contained IndexCatalogEntry. The IndexCatalogEntry
+ * instances are shared across multiple Collection instances.
  */
 class IndexCatalogImpl : public IndexCatalog {
 public:
-    explicit IndexCatalogImpl(Collection* collection);
+    /**
+     * Creates a cloned IndexCatalogImpl. Will make shallow copies of IndexCatalogEntryContainers so
+     * the IndexCatalogEntry will be shared across IndexCatalogImpl instances'
+     */
+    std::unique_ptr<IndexCatalog> clone() const override;
 
     // must be called before used
-    Status init(OperationContext* opCtx) override;
+    Status init(OperationContext* opCtx, Collection* collection) override;
 
     // ---- accessors -----
 
@@ -81,7 +87,7 @@ public:
     /**
      * Returns the spec for the id index to create by default for this collection.
      */
-    BSONObj getDefaultIdIndexSpec() const override;
+    BSONObj getDefaultIdIndexSpec(const CollectionPtr& collection) const override;
 
     const IndexDescriptor* findIdIndex(OperationContext* opCtx) const override;
 
@@ -93,23 +99,19 @@ public:
     const IndexDescriptor* findIndexByName(
         OperationContext* opCtx,
         StringData name,
-        const InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const override;
+        InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const override;
 
     /**
-     * Find index by matching key pattern and collation spec.  The key pattern and collation spec
-     * uniquely identify an index.
+     * Find index by matching key pattern and options. The key pattern, collation spec, and partial
+     * filter expression together uniquely identify an index.
      *
-     * Collation is specified as a normalized collation spec as returned by
-     * CollationInterface::getSpec.  An empty object indicates the simple collation.
-     *
-     * @return null if cannot find index, otherwise the index with a matching key pattern and
-     * collation.
+     * @return null if cannot find index, otherwise the index with a matching signature.
      */
-    const IndexDescriptor* findIndexByKeyPatternAndCollationSpec(
+    const IndexDescriptor* findIndexByKeyPatternAndOptions(
         OperationContext* opCtx,
         const BSONObj& key,
-        const BSONObj& collationSpec,
-        const InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const override;
+        const BSONObj& indexSpec,
+        InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const override;
 
     /**
      * Find indexes with a matching key pattern, putting them into the vector 'matches'.  The key
@@ -119,33 +121,12 @@ public:
      */
     void findIndexesByKeyPattern(OperationContext* opCtx,
                                  const BSONObj& key,
-                                 const InclusionPolicy inclusionPolicy,
+                                 InclusionPolicy inclusionPolicy,
                                  std::vector<const IndexDescriptor*>* matches) const override;
-
-    /**
-     * Returns an index suitable for shard key range scans.
-     *
-     * This index:
-     * - must be prefixed by 'shardKey', and
-     * - must not be a partial index.
-     * - must have the simple collation.
-     *
-     * If the parameter 'requireSingleKey' is true, then this index additionally must not be
-     * multi-key.
-     *
-     * If no such index exists, returns NULL. If errMsg is non-null, the reasons that the existing
-     * indexes are incompatible will be appended to errMsg.
-     */
-    const IndexDescriptor* findShardKeyPrefixedIndex(OperationContext* opCtx,
-                                                     const BSONObj& shardKey,
-                                                     bool requireSingleKey,
-                                                     std::string* errMsg = nullptr) const override;
-
-    void findIndexByType(
-        OperationContext* opCtx,
-        const std::string& type,
-        std::vector<const IndexDescriptor*>& matches,
-        const InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const override;
+    void findIndexByType(OperationContext* opCtx,
+                         const std::string& type,
+                         std::vector<const IndexDescriptor*>& matches,
+                         InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const override;
 
 
     /**
@@ -161,21 +142,26 @@ public:
      * on the collection.
      */
     const IndexDescriptor* refreshEntry(OperationContext* opCtx,
-                                        const IndexDescriptor* oldDesc) override;
+                                        Collection* collection,
+                                        const IndexDescriptor* oldDesc,
+                                        CreateIndexEntryFlags flags) override;
 
     const IndexCatalogEntry* getEntry(const IndexDescriptor* desc) const override;
 
     std::shared_ptr<const IndexCatalogEntry> getEntryShared(const IndexDescriptor*) const override;
 
+    std::shared_ptr<IndexCatalogEntry> getEntryShared(const IndexDescriptor*) override;
+
     std::vector<std::shared_ptr<const IndexCatalogEntry>> getAllReadyEntriesShared() const override;
 
     using IndexIterator = IndexCatalog::IndexIterator;
-    std::unique_ptr<IndexIterator> getIndexIterator(
-        OperationContext* const opCtx, const InclusionPolicy inclusionPolicy) const override;
+    std::unique_ptr<IndexIterator> getIndexIterator(OperationContext* opCtx,
+                                                    InclusionPolicy inclusionPolicy) const override;
 
     // ---- index set modifiers ------
 
     IndexCatalogEntry* createIndexEntry(OperationContext* opCtx,
+                                        Collection* collection,
                                         std::unique_ptr<IndexDescriptor> descriptor,
                                         CreateIndexEntryFlags flags) override;
 
@@ -185,38 +171,52 @@ public:
      * of the created index, as it is stored in this index catalog.
      */
     StatusWith<BSONObj> createIndexOnEmptyCollection(OperationContext* opCtx,
+                                                     Collection* collection,
                                                      BSONObj spec) override;
 
-    StatusWith<BSONObj> prepareSpecForCreate(OperationContext* opCtx,
-                                             const BSONObj& original) const override;
+    StatusWith<BSONObj> prepareSpecForCreate(
+        OperationContext* opCtx,
+        const CollectionPtr& collection,
+        const BSONObj& original,
+        const boost::optional<ResumeIndexInfo>& resumeInfo = boost::none) const override;
 
-    std::vector<BSONObj> removeExistingIndexes(OperationContext* const opCtx,
+    std::vector<BSONObj> removeExistingIndexes(OperationContext* opCtx,
+                                               const CollectionPtr& collection,
                                                const std::vector<BSONObj>& indexSpecsToBuild,
-                                               const bool removeIndexBuildsToo) const override;
+                                               bool removeIndexBuildsToo) const override;
 
     std::vector<BSONObj> removeExistingIndexesNoChecks(
-        OperationContext* const opCtx,
+        OperationContext* opCtx,
+        const CollectionPtr& collection,
         const std::vector<BSONObj>& indexSpecsToBuild) const override;
 
-    /**
-     * Drops all indexes in the index catalog, optionally dropping the id index depending on the
-     * 'includingIdIndex' parameter value. If the 'droppedIndexes' parameter is not null,
-     * it is filled with the names and index info of the dropped indexes.
-     */
+    void dropIndexes(OperationContext* opCtx,
+                     Collection* collection,
+                     std::function<bool(const IndexDescriptor*)> matchFn,
+                     std::function<void(const IndexDescriptor*)> onDropFn) override;
     void dropAllIndexes(OperationContext* opCtx,
+                        Collection* collection,
                         bool includingIdIndex,
                         std::function<void(const IndexDescriptor*)> onDropFn) override;
-    void dropAllIndexes(OperationContext* opCtx, bool includingIdIndex) override;
+
+    Status dropIndex(OperationContext* opCtx,
+                     Collection* collection,
+                     const IndexDescriptor* desc) override;
+    Status resetUnfinishedIndexForRecovery(OperationContext* opCtx,
+                                           Collection* collection,
+                                           const IndexDescriptor* desc) override;
+    Status dropUnfinishedIndex(OperationContext* opCtx,
+                               Collection* collection,
+                               const IndexDescriptor* desc) override;
+
+    Status dropIndexEntry(OperationContext* opCtx,
+                          Collection* collection,
+                          IndexCatalogEntry* entry) override;
 
 
-    Status dropIndex(OperationContext* opCtx, const IndexDescriptor* desc) override;
-    Status dropUnfinishedIndex(OperationContext* const opCtx,
-                               const IndexDescriptor* const desc) override;
-
-    Status dropIndexEntry(OperationContext* opCtx, IndexCatalogEntry* entry) override;
-
-
-    void deleteIndexFromDisk(OperationContext* opCtx, const std::string& indexName) override;
+    void deleteIndexFromDisk(OperationContext* opCtx,
+                             Collection* collection,
+                             const std::string& indexName) override;
 
     struct IndexKillCriteria {
         std::string ns;
@@ -226,26 +226,11 @@ public:
 
     // ---- modify single index
 
-    /**
-     * Returns true if the index 'idx' is multikey, and returns false otherwise.
-     */
-    bool isMultikey(const IndexDescriptor* const idx) override;
-
-    /**
-     * Returns the path components that cause the index 'idx' to be multikey if the index supports
-     * path-level multikey tracking, and returns an empty vector if path-level multikey tracking
-     * isn't supported.
-     *
-     * If the index supports path-level multikey tracking but isn't multikey, then this function
-     * returns a vector with size equal to the number of elements in the index key pattern where
-     * each element in the vector is an empty set.
-     */
-    MultikeyPaths getMultikeyPaths(OperationContext* opCtx, const IndexDescriptor* idx) override;
-
-    void setMultikeyPaths(OperationContext* const opCtx,
+    void setMultikeyPaths(OperationContext* opCtx,
+                          const CollectionPtr& coll,
                           const IndexDescriptor* desc,
                           const KeyStringSet& multikeyMetadataKeys,
-                          const MultikeyPaths& multikeyPaths) override;
+                          const MultikeyPaths& multikeyPaths) const override;
 
     // ----- data modifiers ------
 
@@ -256,29 +241,33 @@ public:
      * This method may throw.
      */
     Status indexRecords(OperationContext* opCtx,
+                        const CollectionPtr& coll,
                         const std::vector<BsonRecord>& bsonRecords,
-                        int64_t* keysInsertedOut) override;
+                        int64_t* keysInsertedOut) const override;
 
     /**
      * See IndexCatalog::updateRecord
      */
-    Status updateRecord(OperationContext* const opCtx,
+    Status updateRecord(OperationContext* opCtx,
+                        const CollectionPtr& coll,
                         const BSONObj& oldDoc,
                         const BSONObj& newDoc,
                         const RecordId& recordId,
-                        int64_t* const keysInsertedOut,
-                        int64_t* const keysDeletedOut) override;
+                        int64_t* keysInsertedOut,
+                        int64_t* keysDeletedOut) const override;
     /**
      * When 'keysDeletedOut' is not null, it will be set to the number of index keys removed by
      * this operation.
      */
     void unindexRecord(OperationContext* opCtx,
+                       const CollectionPtr& collection,
                        const BSONObj& obj,
                        const RecordId& loc,
                        bool noWarn,
-                       int64_t* keysDeletedOut) override;
+                       int64_t* keysDeletedOut,
+                       CheckRecordId checkRecordId = CheckRecordId::Off) const override;
 
-    Status compactIndexes(OperationContext* opCtx) override;
+    Status compactIndexes(OperationContext* opCtx) const override;
 
     inline std::string getAccessMethodName(const BSONObj& keyPattern) override {
         return _getAccessMethodName(keyPattern);
@@ -295,10 +284,20 @@ public:
      * index constraints, as needed by replication.
      */
     void prepareInsertDeleteOptions(OperationContext* opCtx,
+                                    const NamespaceString&,
                                     const IndexDescriptor* desc,
                                     InsertDeleteOptions* options) const override;
 
-    void indexBuildSuccess(OperationContext* opCtx, IndexCatalogEntry* index) override;
+    void indexBuildSuccess(OperationContext* opCtx,
+                           Collection* coll,
+                           IndexCatalogEntry* index) override;
+
+    /**
+     * Returns a status indicating whether 'expression' is valid for use in a partial index
+     * partialFilterExpression.
+     */
+    static Status checkValidFilterExpressions(const MatchExpression* expression,
+                                              bool timeseriesMetricIndexesFeatureFlagEnabled);
 
 private:
     static const BSONObj _idObj;  // { _id : 1 }
@@ -311,48 +310,44 @@ private:
      */
     std::string _getAccessMethodName(const BSONObj& keyPattern) const;
 
-    Status _indexKeys(OperationContext* opCtx,
-                      IndexCatalogEntry* index,
-                      const KeyStringSet& keys,
-                      const KeyStringSet& multikeyMetadataKeys,
-                      const MultikeyPaths& multikeyPaths,
-                      const BSONObj& obj,
-                      RecordId loc,
-                      const InsertDeleteOptions& options,
-                      int64_t* keysInsertedOut);
-
     Status _indexFilteredRecords(OperationContext* opCtx,
-                                 IndexCatalogEntry* index,
+                                 const CollectionPtr& coll,
+                                 const IndexCatalogEntry* index,
                                  const std::vector<BsonRecord>& bsonRecords,
-                                 int64_t* keysInsertedOut);
+                                 int64_t* keysInsertedOut) const;
 
     Status _indexRecords(OperationContext* opCtx,
-                         IndexCatalogEntry* index,
+                         const CollectionPtr& coll,
+                         const IndexCatalogEntry* index,
                          const std::vector<BsonRecord>& bsonRecords,
-                         int64_t* keysInsertedOut);
+                         int64_t* keysInsertedOut) const;
 
-    Status _updateRecord(OperationContext* const opCtx,
-                         IndexCatalogEntry* index,
+    Status _updateRecord(OperationContext* opCtx,
+                         const CollectionPtr& coll,
+                         const IndexCatalogEntry* index,
                          const BSONObj& oldDoc,
                          const BSONObj& newDoc,
                          const RecordId& recordId,
-                         int64_t* const keysInsertedOut,
-                         int64_t* const keysDeletedOut);
-
-    void _unindexKeys(OperationContext* opCtx,
-                      IndexCatalogEntry* index,
-                      const KeyStringSet& keys,
-                      const BSONObj& obj,
-                      RecordId loc,
-                      bool logIfError,
-                      int64_t* const keysDeletedOut);
+                         int64_t* keysInsertedOut,
+                         int64_t* keysDeletedOut) const;
 
     void _unindexRecord(OperationContext* opCtx,
-                        IndexCatalogEntry* entry,
+                        const CollectionPtr& collection,
+                        const IndexCatalogEntry* entry,
                         const BSONObj& obj,
                         const RecordId& loc,
                         bool logIfError,
-                        int64_t* keysDeletedOut);
+                        int64_t* keysDeletedOut,
+                        CheckRecordId checkRecordId = CheckRecordId::Off) const;
+
+    /**
+     * Helper to remove the index from disk.
+     * The index should be removed from the in-memory catalog beforehand.
+     */
+    void _deleteIndexFromDisk(OperationContext* opCtx,
+                              Collection* collection,
+                              const std::string& indexName,
+                              std::shared_ptr<IndexCatalogEntry> entry);
 
     /**
      * Applies a set of transformations to the user-provided index object 'spec' to make it
@@ -360,10 +355,12 @@ private:
      * plugin-level transformations if appropriate, etc.
      */
     StatusWith<BSONObj> _fixIndexSpec(OperationContext* opCtx,
-                                      Collection* collection,
+                                      const CollectionPtr& collection,
                                       const BSONObj& spec) const;
 
-    Status _isSpecOk(OperationContext* opCtx, const BSONObj& spec) const;
+    Status _isSpecOk(OperationContext* opCtx,
+                     const CollectionPtr& collection,
+                     const BSONObj& spec) const;
 
     /**
      * Validates the 'original' index specification, alters any legacy fields and does plugin-level
@@ -371,20 +368,22 @@ private:
      * error.
      */
     StatusWith<BSONObj> _validateAndFixIndexSpec(OperationContext* opCtx,
+                                                 const CollectionPtr& collection,
                                                  const BSONObj& original) const;
 
     /**
      * Checks whether there are any spec conflicts with existing ready indexes or in-progress index
      * builds. Also checks whether any limits set on this server would be exceeded by building the
-     * index. 'inclusionPolicy' dictates whether in-progress index builds are checked for conflicts,
-     * along with ready indexes.
+     * index. 'includeUnfinishedIndexes' dictates whether in-progress index builds are checked for
+     * conflicts, along with ready indexes.
      *
      * Returns IndexAlreadyExists for both ready and in-progress index builds. Can also return other
      * errors.
      */
     Status _doesSpecConflictWithExisting(OperationContext* opCtx,
+                                         const CollectionPtr& collection,
                                          const BSONObj& spec,
-                                         const InclusionPolicy inclusionPolicy) const;
+                                         InclusionPolicy inclusionPolicy) const;
 
     /**
      * Returns true if the replica set member's config has {buildIndexes:false} set, which means
@@ -394,11 +393,9 @@ private:
     Status _isNonIDIndexAndNotAllowedToBuild(OperationContext* opCtx, const BSONObj& spec) const;
 
     void _logInternalState(OperationContext* opCtx,
+                           const CollectionPtr& collection,
                            long long numIndexesInCollectionCatalogEntry,
-                           const std::vector<std::string>& indexNamesToDrop,
-                           bool haveIdIndex);
-
-    Collection* const _collection;
+                           const std::vector<std::string>& indexNamesToDrop);
 
     IndexCatalogEntryContainer _readyIndexes;
     IndexCatalogEntryContainer _buildingIndexes;
