@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include <algorithm>
 
@@ -36,12 +35,20 @@
 #include "mongo/db/catalog/index_consistency.h"
 
 #include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/index_repair.h"
 #include "mongo/db/catalog/validate_gen.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/db_raii.h"
+#include "mongo/db/index/index_access_method.h"
 #include "mongo/db/index/index_descriptor.h"
-#include "mongo/db/storage/storage_debug_util.h"
+#include "mongo/db/multi_key_path_tracker.h"
+#include "mongo/db/record_id_helpers.h"
 #include "mongo/logv2/log.h"
 #include "mongo/util/string_map.h"
+#include "mongo/util/testing_proctor.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
+
 
 namespace mongo {
 
@@ -69,11 +76,24 @@ std::pair<std::string, std::string> _generateKeyForMap(const IndexInfo& indexInf
 
 }  // namespace
 
-IndexInfo::IndexInfo(const IndexDescriptor* descriptor)
+IndexInfo::IndexInfo(const IndexDescriptor* descriptor, IndexAccessMethod* indexAccessMethod)
     : indexName(descriptor->indexName()),
       keyPattern(descriptor->keyPattern()),
       indexNameHash(hash(descriptor->indexName())),
-      ord(Ordering::make(descriptor->keyPattern())) {}
+      ord(Ordering::make(descriptor->keyPattern())),
+      unique(descriptor->unique()),
+      accessMethod(indexAccessMethod) {}
+
+IndexEntryInfo::IndexEntryInfo(const IndexInfo& indexInfo,
+                               RecordId entryRecordId,
+                               BSONObj entryIdKey,
+                               KeyString::Value entryKeyString)
+    : indexName(indexInfo.indexName),
+      keyPattern(indexInfo.keyPattern),
+      ord(indexInfo.ord),
+      recordId(std::move(entryRecordId)),
+      idKey(entryIdKey.getOwned()),
+      keyString(entryKeyString) {}
 
 IndexConsistency::IndexConsistency(OperationContext* opCtx,
                                    CollectionValidation::ValidateState* validateState)
@@ -82,13 +102,14 @@ IndexConsistency::IndexConsistency(OperationContext* opCtx,
 
     for (const auto& index : _validateState->getIndexes()) {
         const IndexDescriptor* descriptor = index->descriptor();
-        _indexesInfo.emplace(descriptor->indexName(), IndexInfo(descriptor));
+        IndexAccessMethod* accessMethod = const_cast<IndexAccessMethod*>(index->accessMethod());
+        _indexesInfo.emplace(descriptor->indexName(), IndexInfo(descriptor, accessMethod));
     }
 }
 
 void IndexConsistency::addMultikeyMetadataPath(const KeyString::Value& ks, IndexInfo* indexInfo) {
     auto hash = _hashKeyString(ks, indexInfo->indexNameHash);
-    if (MONGO_unlikely(_validateState->logDiagnostics())) {
+    if (MONGO_unlikely(_validateState->extraLoggingForTest())) {
         LOGV2(6208500,
               "[validate](multikeyMetadataPath) Adding with the hash",
               "hash"_attr = hash,
@@ -100,7 +121,7 @@ void IndexConsistency::addMultikeyMetadataPath(const KeyString::Value& ks, Index
 void IndexConsistency::removeMultikeyMetadataPath(const KeyString::Value& ks,
                                                   IndexInfo* indexInfo) {
     auto hash = _hashKeyString(ks, indexInfo->indexNameHash);
-    if (MONGO_unlikely(_validateState->logDiagnostics())) {
+    if (MONGO_unlikely(_validateState->extraLoggingForTest())) {
         LOGV2(6208501,
               "[validate](multikeyMetadataPath) Removing with the hash",
               "hash"_attr = hash,
@@ -114,26 +135,9 @@ size_t IndexConsistency::getMultikeyMetadataPathCount(IndexInfo* indexInfo) {
 }
 
 bool IndexConsistency::haveEntryMismatch() const {
-    bool haveMismatch =
-        std::any_of(_indexKeyBuckets.begin(),
-                    _indexKeyBuckets.end(),
-                    [](const IndexKeyBucket& bucket) -> bool { return bucket.indexKeyCount; });
-
-    if (haveMismatch && _validateState->logDiagnostics()) {
-        for (size_t i = 0; i < _indexKeyBuckets.size(); i++) {
-            if (_indexKeyBuckets[i].indexKeyCount == 0) {
-                continue;
-            }
-
-            LOGV2(7404500,
-                  "[validate](bucket entry mismatch)",
-                  "hash"_attr = i,
-                  "indexKeyCount"_attr = _indexKeyBuckets[i].indexKeyCount,
-                  "bucketBytesSize"_attr = _indexKeyBuckets[i].bucketSizeBytes);
-        }
-    }
-
-    return haveMismatch;
+    return std::any_of(_indexKeyBuckets.begin(),
+                       _indexKeyBuckets.end(),
+                       [](const IndexKeyBucket& bucket) -> bool { return bucket.indexKeyCount; });
 }
 
 void IndexConsistency::setSecondPhase() {
@@ -141,8 +145,57 @@ void IndexConsistency::setSecondPhase() {
     _firstPhase = false;
 }
 
-void IndexConsistency::addIndexEntryErrors(ValidateResultsMap* indexNsResultsMap,
-                                           ValidateResults* results) {
+void IndexConsistency::repairMissingIndexEntries(OperationContext* opCtx,
+                                                 ValidateResults* results) {
+    invariant(_validateState->getIndexes().size() > 0);
+    std::shared_ptr<const IndexCatalogEntry> index = _validateState->getIndexes().front();
+    for (auto it = _missingIndexEntries.begin(); it != _missingIndexEntries.end();) {
+        const KeyString::Value& ks = it->second.keyString;
+        const KeyFormat keyFormat = _validateState->getCollection()->getRecordStore()->keyFormat();
+
+        const std::string& indexName = it->first.first;
+        if (indexName != index->descriptor()->indexName()) {
+            // Assuming that _missingIndexEntries is sorted by indexName, this lookup should not
+            // happen often.
+            for (auto currIndex : _validateState->getIndexes()) {
+                if (currIndex->descriptor()->indexName() == indexName) {
+                    index = currIndex;
+                    break;
+                }
+            }
+        }
+
+        int64_t numInserted = index_repair::repairMissingIndexEntry(opCtx,
+                                                                    index,
+                                                                    ks,
+                                                                    keyFormat,
+                                                                    _validateState->nss(),
+                                                                    _validateState->getCollection(),
+                                                                    results);
+        getIndexInfo(indexName).numKeys += numInserted;
+        it = _missingIndexEntries.erase(it);
+    }
+
+    if (results->numInsertedMissingIndexEntries > 0) {
+        results->warnings.push_back(str::stream()
+                                    << "Inserted " << results->numInsertedMissingIndexEntries
+                                    << " missing index entries.");
+    }
+    if (results->numDocumentsMovedToLostAndFound > 0) {
+        const NamespaceString lostAndFoundNss =
+            NamespaceString(NamespaceString::kLocalDb,
+                            "lost_and_found." + _validateState->getCollection()->uuid().toString());
+        results->warnings.push_back(str::stream()
+                                    << "Removed " << results->numDocumentsMovedToLostAndFound
+                                    << " duplicate documents to resolve "
+                                    << results->numDocumentsMovedToLostAndFound +
+                                        results->numOutdatedMissingIndexEntry
+                                    << " missing index entries. Removed documents can be found in '"
+                                    << lostAndFoundNss.ns() << "'.");
+    }
+}
+
+void IndexConsistency::addIndexEntryErrors(ValidateResults* results) {
     invariant(!_firstPhase);
 
     // We'll report up to 1MB for extra index entry errors and missing index entry errors.
@@ -156,31 +209,23 @@ void IndexConsistency::addIndexEntryErrors(ValidateResultsMap* indexNsResultsMap
         numExtraIndexEntryErrors += item.second.size();
     }
 
-    // Sort missing index entries by size so we can process in order of increasing size and return
-    // as many as possible within memory limits.
-    using MissingIt = decltype(_missingIndexEntries)::const_iterator;
-    std::vector<MissingIt> missingIndexEntriesBySize;
-    missingIndexEntriesBySize.reserve(_missingIndexEntries.size());
-    for (auto it = _missingIndexEntries.begin(); it != _missingIndexEntries.end(); ++it) {
-        missingIndexEntriesBySize.push_back(it);
-    }
-    std::sort(missingIndexEntriesBySize.begin(),
-              missingIndexEntriesBySize.end(),
-              [](const MissingIt& a, const MissingIt& b) {
-                  return a->second.objsize() < b->second.objsize();
-              });
-
     // Inform which indexes have inconsistencies and add the BSON objects of the inconsistent index
     // entries to the results vector.
     bool missingIndexEntrySizeLimitWarning = false;
-    bool first = true;
-    for (const auto& it : missingIndexEntriesBySize) {
-        const auto& entry = it->second;
+    for (const auto& missingIndexEntry : _missingIndexEntries) {
+        const IndexEntryInfo& entryInfo = missingIndexEntry.second;
+        KeyString::Value ks = entryInfo.keyString;
+        auto indexKey =
+            KeyString::toBsonSafe(ks.getBuffer(), ks.getSize(), entryInfo.ord, ks.getTypeBits());
+        const BSONObj entry = _generateInfo(entryInfo.indexName,
+                                            entryInfo.keyPattern,
+                                            entryInfo.recordId,
+                                            indexKey,
+                                            entryInfo.idKey);
 
         numMissingIndexEntriesSizeBytes += entry.objsize();
-        if (first || numMissingIndexEntriesSizeBytes <= kErrorSizeBytes) {
+        if (numMissingIndexEntriesSizeBytes <= kErrorSizeBytes) {
             results->missingIndexEntries.push_back(entry);
-            first = false;
         } else if (!missingIndexEntrySizeLimitWarning) {
             StringBuilder ss;
             ss << "Not all missing index entry inconsistencies are listed due to size limitations.";
@@ -190,7 +235,7 @@ void IndexConsistency::addIndexEntryErrors(ValidateResultsMap* indexNsResultsMap
         }
 
         std::string indexName = entry["indexName"].String();
-        if (!indexNsResultsMap->at(indexName).valid) {
+        if (!results->indexResultsMap.at(indexName).valid) {
             continue;
         }
 
@@ -198,61 +243,36 @@ void IndexConsistency::addIndexEntryErrors(ValidateResultsMap* indexNsResultsMap
         ss << "Index with name '" << indexName << "' has inconsistencies.";
         results->errors.push_back(ss.str());
 
-        indexNsResultsMap->at(indexName).valid = false;
+        results->indexResultsMap.at(indexName).valid = false;
     }
 
-    // Sort extra index entries by size so we can process in order of increasing size and return as
-    // many as possible within memory limits.
-    using ExtraIt = SimpleBSONObjSet::const_iterator;
-    std::vector<ExtraIt> extraIndexEntriesBySize;
-    // Since the extra entries are stored in a map of sets, we have to iterate the entries in the
-    // map and sum the size of the sets in order to get the total number. Given that we can have at
-    // most 64 indexes per collection, and the total number of entries could potentially be in the
-    // millions, we expect that iterating the map will be much less costly than the additional
-    // allocations and copies that could result from not calling 'reserve' on the vector.
-    size_t totalExtraIndexEntriesCount =
-        std::accumulate(_extraIndexEntries.begin(),
-                        _extraIndexEntries.end(),
-                        0,
-                        [](size_t total, const std::pair<IndexKey, SimpleBSONObjSet>& set) {
-                            return total + set.second.size();
-                        });
-    extraIndexEntriesBySize.reserve(totalExtraIndexEntriesCount);
-    for (const auto& extraIndexEntry : _extraIndexEntries) {
-        const SimpleBSONObjSet& entries = extraIndexEntry.second;
-        for (auto it = entries.begin(); it != entries.end(); ++it) {
-            extraIndexEntriesBySize.push_back(it);
-        }
-    }
-    std::sort(extraIndexEntriesBySize.begin(),
-              extraIndexEntriesBySize.end(),
-              [](const ExtraIt& a, const ExtraIt& b) { return a->objsize() < b->objsize(); });
-
     bool extraIndexEntrySizeLimitWarning = false;
-    for (const auto& entry : extraIndexEntriesBySize) {
-        numExtraIndexEntriesSizeBytes += entry->objsize();
-        if (first || numExtraIndexEntriesSizeBytes <= kErrorSizeBytes) {
-            results->extraIndexEntries.push_back(*entry);
-            first = false;
-        } else if (!extraIndexEntrySizeLimitWarning) {
+    for (const auto& extraIndexEntry : _extraIndexEntries) {
+        const SimpleBSONObjSet& entries = extraIndexEntry.second;
+        for (const auto& entry : entries) {
+            numExtraIndexEntriesSizeBytes += entry.objsize();
+            if (numExtraIndexEntriesSizeBytes <= kErrorSizeBytes) {
+                results->extraIndexEntries.push_back(entry);
+            } else if (!extraIndexEntrySizeLimitWarning) {
+                StringBuilder ss;
+                ss << "Not all extra index entry inconsistencies are listed due to size "
+                      "limitations.";
+                results->errors.push_back(ss.str());
+
+                extraIndexEntrySizeLimitWarning = true;
+            }
+
+            std::string indexName = entry["indexName"].String();
+            if (!results->indexResultsMap.at(indexName).valid) {
+                continue;
+            }
+
             StringBuilder ss;
-            ss << "Not all extra index entry inconsistencies are listed due to size "
-                  "limitations.";
+            ss << "Index with name '" << indexName << "' has inconsistencies.";
             results->errors.push_back(ss.str());
 
-            extraIndexEntrySizeLimitWarning = true;
+            results->indexResultsMap.at(indexName).valid = false;
         }
-
-        std::string indexName = (*entry)["indexName"].String();
-        if (!indexNsResultsMap->at(indexName).valid) {
-            continue;
-        }
-
-        StringBuilder ss;
-        ss << "Index with name '" << indexName << "' has inconsistencies.";
-        results->errors.push_back(ss.str());
-
-        indexNsResultsMap->at(indexName).valid = false;
     }
 
     // Inform how many inconsistencies were detected.
@@ -260,21 +280,32 @@ void IndexConsistency::addIndexEntryErrors(ValidateResultsMap* indexNsResultsMap
         StringBuilder ss;
         ss << "Detected " << numMissingIndexEntryErrors << " missing index entries.";
         results->warnings.push_back(ss.str());
+        results->valid = false;
     }
 
     if (numExtraIndexEntryErrors > 0) {
         StringBuilder ss;
         ss << "Detected " << numExtraIndexEntryErrors << " extra index entries.";
         results->warnings.push_back(ss.str());
+        results->valid = false;
     }
+}
 
-    results->valid = false;
+void IndexConsistency::addDocumentMultikeyPaths(IndexInfo* indexInfo,
+                                                const MultikeyPaths& newPaths) {
+    invariant(newPaths.size());
+    if (indexInfo->docMultikeyPaths.size()) {
+        MultikeyPathTracker::mergeMultikeyPaths(&indexInfo->docMultikeyPaths, newPaths);
+    } else {
+        // Instantiate the multikey paths. Also indicates that this index uses multikeyPaths.
+        indexInfo->docMultikeyPaths = newPaths;
+    }
 }
 
 void IndexConsistency::addDocKey(OperationContext* opCtx,
                                  const KeyString::Value& ks,
                                  IndexInfo* indexInfo,
-                                 RecordId recordId) {
+                                 const RecordId& recordId) {
     auto rawHash = ks.hash(indexInfo->indexNameHash);
     auto hashLower = rawHash % kNumHashBuckets;
     auto hashUpper = (rawHash / kNumHashBuckets) % kNumHashBuckets;
@@ -282,15 +313,15 @@ void IndexConsistency::addDocKey(OperationContext* opCtx,
     auto& upper = _indexKeyBuckets[hashUpper];
 
     if (_firstPhase) {
-        // During the first phase of validation we only keep track of the count for the
-        // document keys encountered.
+        // During the first phase of validation we only keep track of the count for the document
+        // keys encountered.
         lower.indexKeyCount++;
         lower.bucketSizeBytes += ks.getSize();
         upper.indexKeyCount++;
         upper.bucketSizeBytes += ks.getSize();
         indexInfo->numRecords++;
 
-        if (MONGO_unlikely(_validateState->logDiagnostics())) {
+        if (MONGO_unlikely(_validateState->extraLoggingForTest())) {
             LOGV2(4666602,
                   "[validate](record) Adding with hashes",
                   "hashUpper"_attr = hashUpper,
@@ -298,7 +329,7 @@ void IndexConsistency::addDocKey(OperationContext* opCtx,
             const BSONObj& keyPatternBson = indexInfo->keyPattern;
             auto keyStringBson = KeyString::toBsonSafe(
                 ks.getBuffer(), ks.getSize(), indexInfo->ord, ks.getTypeBits());
-            StorageDebugUtil::printKeyString(
+            KeyString::logKeyString(
                 recordId, ks, keyPatternBson, keyStringBson, "[validate](record)");
         }
     } else if (lower.indexKeyCount || upper.indexKeyCount) {
@@ -309,26 +340,30 @@ void IndexConsistency::addDocKey(OperationContext* opCtx,
         invariant(record);
 
         BSONObj data = record->data.toBson();
-        boost::optional<BSONElement> idKey = boost::none;
+
+        BSONObjBuilder idKeyBuilder;
         if (data.hasField("_id")) {
-            idKey = data["_id"];
+            idKeyBuilder.append(data["_id"]);
         }
 
-        auto indexKey =
-            KeyString::toBsonSafe(ks.getBuffer(), ks.getSize(), indexInfo->ord, ks.getTypeBits());
-        BSONObj info = _generateInfo(*indexInfo, recordId, indexKey, idKey);
-
-        // Cannot have duplicate KeyStrings during the document scan phase for the same
-        // index.
+        // Cannot have duplicate KeyStrings during the document scan phase for the same index.
         IndexKey key = _generateKeyForMap(*indexInfo, ks);
         invariant(_missingIndexEntries.count(key) == 0);
-        _missingIndexEntries.insert(std::make_pair(key, info));
+        _missingIndexEntries.insert(
+            std::make_pair(key, IndexEntryInfo(*indexInfo, recordId, idKeyBuilder.obj(), ks)));
+
+        // Prints the collection document's and index entry's metadata.
+        _validateState->getCollection()->getRecordStore()->printRecordMetadata(opCtx, recordId);
+        indexInfo->accessMethod->asSortedData()->getSortedDataInterface()->printIndexEntryMetadata(
+            opCtx, ks);
     }
 }
 
-void IndexConsistency::addIndexKey(const KeyString::Value& ks,
+void IndexConsistency::addIndexKey(OperationContext* opCtx,
+                                   const KeyString::Value& ks,
                                    IndexInfo* indexInfo,
-                                   RecordId recordId) {
+                                   const RecordId& recordId,
+                                   ValidateResults* results) {
     auto rawHash = ks.hash(indexInfo->indexNameHash);
     auto hashLower = rawHash % kNumHashBuckets;
     auto hashUpper = (rawHash / kNumHashBuckets) % kNumHashBuckets;
@@ -336,15 +371,15 @@ void IndexConsistency::addIndexKey(const KeyString::Value& ks,
     auto& upper = _indexKeyBuckets[hashUpper];
 
     if (_firstPhase) {
-        // During the first phase of validation we only keep track of the count for the
-        // index entry keys encountered.
+        // During the first phase of validation we only keep track of the count for the index entry
+        // keys encountered.
         lower.indexKeyCount--;
         lower.bucketSizeBytes += ks.getSize();
         upper.indexKeyCount--;
         upper.bucketSizeBytes += ks.getSize();
         indexInfo->numKeys++;
 
-        if (MONGO_unlikely(_validateState->logDiagnostics())) {
+        if (MONGO_unlikely(_validateState->extraLoggingForTest())) {
             LOGV2(4666603,
                   "[validate](index) Adding with hashes",
                   "hashUpper"_attr = hashUpper,
@@ -352,28 +387,61 @@ void IndexConsistency::addIndexKey(const KeyString::Value& ks,
             const BSONObj& keyPatternBson = indexInfo->keyPattern;
             auto keyStringBson = KeyString::toBsonSafe(
                 ks.getBuffer(), ks.getSize(), indexInfo->ord, ks.getTypeBits());
-            StorageDebugUtil::printKeyString(
+            KeyString::logKeyString(
                 recordId, ks, keyPatternBson, keyStringBson, "[validate](index)");
         }
     } else if (lower.indexKeyCount || upper.indexKeyCount) {
         // Found an index key for a bucket that has inconsistencies.
-        // If there is a corresponding document key for the index entry key, we remove the
-        // key from the '_missingIndexEntries' map. However if there was no document key for
-        // the index entry key, we add the key to the '_extraIndexEntries' map.
+        // If there is a corresponding document key for the index entry key, we remove the key from
+        // the '_missingIndexEntries' map. However if there was no document key for the index entry
+        // key, we add the key to the '_extraIndexEntries' map.
         auto indexKey =
             KeyString::toBsonSafe(ks.getBuffer(), ks.getSize(), indexInfo->ord, ks.getTypeBits());
-        BSONObj info = _generateInfo(*indexInfo, recordId, indexKey, boost::none);
+        BSONObj info = _generateInfo(
+            indexInfo->indexName, indexInfo->keyPattern, recordId, indexKey, BSONObj());
 
         IndexKey key = _generateKeyForMap(*indexInfo, ks);
         if (_missingIndexEntries.count(key) == 0) {
+            if (_validateState->fixErrors()) {
+                // Removing extra index entries.
+                InsertDeleteOptions options;
+                options.dupsAllowed = !indexInfo->unique;
+                int64_t numDeleted = 0;
+                writeConflictRetry(
+                    opCtx, "removingExtraIndexEntries", _validateState->nss().ns(), [&] {
+                        WriteUnitOfWork wunit(opCtx);
+                        Status status = indexInfo->accessMethod->asSortedData()->removeKeys(
+                            opCtx, {ks}, options, &numDeleted);
+                        wunit.commit();
+                    });
+                auto& indexResults = results->indexResultsMap[indexInfo->indexName];
+                indexResults.keysTraversed -= numDeleted;
+                results->numRemovedExtraIndexEntries += numDeleted;
+                results->repaired = true;
+                indexInfo->numKeys--;
+                return;
+            }
+
             // We may have multiple extra index entries for a given KeyString.
             auto search = _extraIndexEntries.find(key);
             if (search == _extraIndexEntries.end()) {
                 SimpleBSONObjSet infoSet = {info};
+                if (TestingProctor::instance().isEnabled() &&
+                    _validateState->nss() == NamespaceString("local", "replset.initialSyncId")) {
+                    invariant(false,
+                              "Validation failed with an extra index key on "
+                              "`local.replset.initialSyncId`.");
+                }
                 _extraIndexEntries.insert(std::make_pair(key, infoSet));
+
+                // Prints the collection document's and index entry's metadata.
+                _validateState->getCollection()->getRecordStore()->printRecordMetadata(opCtx,
+                                                                                       recordId);
+                indexInfo->accessMethod->asSortedData()
+                    ->getSortedDataInterface()
+                    ->printIndexEntryMetadata(opCtx, ks);
                 return;
             }
-
             search->second.insert(info);
         } else {
             _missingIndexEntries.erase(key);
@@ -393,78 +461,65 @@ bool IndexConsistency::limitMemoryUsageForSecondPhase(ValidateResults* result) {
                             return bucket.indexKeyCount ? bytes + bucket.bucketSizeBytes : bytes;
                         });
 
-    // Allows twice the "maxValidateMemoryUsageMB" because each KeyString has two hashes
-    // stored.
+    // Allows twice the "maxValidateMemoryUsageMB" because each KeyString has two hashes stored.
     if (totalMemoryNeededBytes <= maxMemoryUsageBytes * 2) {
         // The amount of memory we need is under the limit, so no need to do anything else.
         return true;
     }
 
-    // At this point we know we'll exceed the memory limit, and will pare back some of the
-    // buckets. First we'll see what the smallest bucket is, and if that's over the limit by
-    // itself, then we can zero out all the other buckets. Otherwise we'll keep as many
-    // buckets as we can.
+    bool hasNonZeroBucket = false;
+    uint64_t memoryUsedSoFarBytes = 0;
+    uint32_t smallestBucketBytes = std::numeric_limits<uint32_t>::max();
+    // Zero out any nonzero buckets that would put us over maxMemoryUsageBytes.
+    std::for_each(_indexKeyBuckets.begin(), _indexKeyBuckets.end(), [&](IndexKeyBucket& bucket) {
+        if (bucket.indexKeyCount == 0) {
+            return;
+        }
 
-    auto smallestBucketWithAnInconsistency = std::min_element(
-        _indexKeyBuckets.begin(),
-        _indexKeyBuckets.end(),
-        [](const IndexKeyBucket& lhs, const IndexKeyBucket& rhs) {
-            if (lhs.indexKeyCount != 0) {
-                return rhs.indexKeyCount == 0 || lhs.bucketSizeBytes < rhs.bucketSizeBytes;
-            }
-            return false;
-        });
-    invariant(smallestBucketWithAnInconsistency->indexKeyCount != 0);
+        smallestBucketBytes = std::min(smallestBucketBytes, bucket.bucketSizeBytes);
+        if (bucket.bucketSizeBytes + memoryUsedSoFarBytes > maxMemoryUsageBytes) {
+            // Including this bucket would put us over the memory limit, so zero this bucket. We
+            // don't want to keep any entry that will exceed the memory limit in the second phase so
+            // we don't double the 'maxMemoryUsageBytes' here.
+            bucket.indexKeyCount = 0;
+            return;
+        }
+        memoryUsedSoFarBytes += bucket.bucketSizeBytes;
+        hasNonZeroBucket = true;
+    });
 
-    if (smallestBucketWithAnInconsistency->bucketSizeBytes > maxMemoryUsageBytes) {
-        // We're going to just keep the smallest bucket, and zero everything else.
-        std::for_each(
-            _indexKeyBuckets.begin(), _indexKeyBuckets.end(), [&](IndexKeyBucket& bucket) {
-                if (&bucket == &(*smallestBucketWithAnInconsistency)) {
-                    // We keep the smallest bucket.
-                    return;
-                }
+    StringBuilder memoryLimitMessage;
+    memoryLimitMessage << "Memory limit for validation is currently set to "
+                       << maxValidateMemoryUsageMB.load()
+                       << "MB and can be configured via the 'maxValidateMemoryUsageMB' parameter.";
 
-                bucket.indexKeyCount = 0;
-            });
-    } else {
-        // We're going to scan through the buckets and keep as many as we can.
-        std::uint32_t memoryUsedSoFarBytes = 0;
-        std::for_each(
-            _indexKeyBuckets.begin(), _indexKeyBuckets.end(), [&](IndexKeyBucket& bucket) {
-                if (bucket.indexKeyCount == 0) {
-                    return;
-                }
+    if (!hasNonZeroBucket) {
+        const uint32_t minMemoryNeededMB = (smallestBucketBytes / (1024 * 1024)) + 1;
+        StringBuilder ss;
+        ss << "Unable to report index entry inconsistencies due to memory limitations. Need at "
+              "least "
+           << minMemoryNeededMB << "MB to report at least one index entry inconsistency. "
+           << memoryLimitMessage.str();
+        result->errors.push_back(ss.str());
+        result->valid = false;
 
-                if (bucket.bucketSizeBytes + memoryUsedSoFarBytes > maxMemoryUsageBytes) {
-                    // Including this bucket would put us over the memory limit, so zero
-                    // this bucket. We don't want to keep any entry that will exceed the
-                    // memory limit in the second phase so we don't double the
-                    // 'maxMemoryUsageBytes' here.
-                    bucket.indexKeyCount = 0;
-                    return;
-                }
-                memoryUsedSoFarBytes += bucket.bucketSizeBytes;
-            });
+        return false;
     }
 
     StringBuilder ss;
     ss << "Not all index entry inconsistencies are reported due to memory limitations. "
-          "Memory "
-          "limit for validation is currently set to "
-       << maxValidateMemoryUsageMB.load()
-       << "MB and can be configured via the 'maxValidateMemoryUsageMB' parameter.";
+       << memoryLimitMessage.str();
     result->errors.push_back(ss.str());
+    result->valid = false;
 
     return true;
 }
 
-BSONObj IndexConsistency::_generateInfo(const IndexInfo& indexInfo,
-                                        RecordId recordId,
+BSONObj IndexConsistency::_generateInfo(const std::string& indexName,
+                                        const BSONObj& keyPattern,
+                                        const RecordId& recordId,
                                         const BSONObj& indexKey,
-                                        boost::optional<BSONElement> idKey) {
-    const std::string& indexName = indexInfo.indexName;
-    const BSONObj& keyPattern = indexInfo.keyPattern;
+                                        const BSONObj& idKey) {
 
     // We need to rehydrate the indexKey for improved readability.
     // {"": ObjectId(...)} -> {"_id": ObjectId(...)}
@@ -482,13 +537,17 @@ BSONObj IndexConsistency::_generateInfo(const IndexInfo& indexInfo,
 
     BSONObj rehydratedKey = b.done();
 
-    if (idKey) {
-        return BSON("indexName" << indexName << "recordId" << recordId.repr() << "idKey" << *idKey
-                                << "indexKey" << rehydratedKey);
-    } else {
-        return BSON("indexName" << indexName << "recordId" << recordId.repr() << "indexKey"
-                                << rehydratedKey);
+    BSONObjBuilder infoBuilder;
+    infoBuilder.append("indexName", indexName);
+    recordId.serializeToken("recordId", &infoBuilder);
+
+    if (!idKey.isEmpty()) {
+        infoBuilder.append("idKey", idKey);
     }
+
+    infoBuilder.append("indexKey", rehydratedKey);
+
+    return infoBuilder.obj();
 }
 
 uint32_t IndexConsistency::_hashKeyString(const KeyString::Value& ks,
