@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kIndex
 
 #include "mongo/platform/basic.h"
 
@@ -41,6 +40,7 @@
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
 #include "mongo/db/field_ref.h"
+#include "mongo/db/index/column_key_generator.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index/wildcard_key_generator.h"
 #include "mongo/db/index_names.h"
@@ -55,13 +55,14 @@
 #include "mongo/util/represent_as.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kIndex
+
+
 namespace mongo {
 namespace index_key_validate {
 
 std::function<void(std::set<StringData>&)> filterAllowedIndexFieldNames;
 
-using std::string;
-
 using IndexVersion = IndexDescriptor::IndexVersion;
 
 namespace {
@@ -70,34 +71,9 @@ namespace {
 // specification.
 MONGO_FAIL_POINT_DEFINE(skipIndexCreateFieldNameValidation);
 
-static std::set<StringData> allowedFieldNames = {
-    IndexDescriptor::k2dIndexBitsFieldName,
-    IndexDescriptor::k2dIndexMaxFieldName,
-    IndexDescriptor::k2dIndexMinFieldName,
-    IndexDescriptor::k2dsphereCoarsestIndexedLevel,
-    IndexDescriptor::k2dsphereFinestIndexedLevel,
-    IndexDescriptor::k2dsphereVersionFieldName,
-    IndexDescriptor::kBackgroundFieldName,
-    IndexDescriptor::kCollationFieldName,
-    IndexDescriptor::kDefaultLanguageFieldName,
-    IndexDescriptor::kDropDuplicatesFieldName,
-    IndexDescriptor::kExpireAfterSecondsFieldName,
-    IndexDescriptor::kGeoHaystackBucketSize,
-    IndexDescriptor::kHiddenFieldName,
-    IndexDescriptor::kIndexNameFieldName,
-    IndexDescriptor::kIndexVersionFieldName,
-    IndexDescriptor::kKeyPatternFieldName,
-    IndexDescriptor::kLanguageOverrideFieldName,
-    IndexDescriptor::kNamespaceFieldName,
-    IndexDescriptor::kPartialFilterExprFieldName,
-    IndexDescriptor::kPathProjectionFieldName,
-    IndexDescriptor::kSparseFieldName,
-    IndexDescriptor::kStorageEngineFieldName,
-    IndexDescriptor::kTextVersionFieldName,
-    IndexDescriptor::kUniqueFieldName,
-    IndexDescriptor::kWeightsFieldName,
-    // Index creation under legacy writeMode can result in an index spec with an _id field.
-    "_id"};
+// When the skipTTLIndexNaNExpireAfterSecondsValidation failpoint is enabled, validation for
+// TTL index 'expireAfterSeconds' will be disabled.
+MONGO_FAIL_POINT_DEFINE(skipTTLIndexNaNExpireAfterSecondsValidation);
 
 static const std::set<StringData> allowedIdIndexFieldNames = {
     IndexDescriptor::kCollationFieldName,
@@ -107,6 +83,51 @@ static const std::set<StringData> allowedIdIndexFieldNames = {
     IndexDescriptor::kNamespaceFieldName,
     // Index creation under legacy writeMode can result in an index spec with an _id field.
     "_id"};
+
+static const std::set<StringData> allowedClusteredIndexFieldNames = {
+    ClusteredIndexSpec::kNameFieldName,
+    ClusteredIndexSpec::kUniqueFieldName,
+    ClusteredIndexSpec::kVFieldName,
+    ClusteredIndexSpec::kKeyFieldName,
+    // This is for indexSpec creation only.
+    IndexDescriptor::kClusteredFieldName,
+};
+
+/**
+ * Returns Status::OK() if indexes of version 'indexVersion' are allowed to be created, and
+ * returns ErrorCodes::CannotCreateIndex otherwise.
+ */
+Status isIndexVersionAllowedForCreation(IndexVersion indexVersion, const BSONObj& indexSpec) {
+    switch (indexVersion) {
+        case IndexVersion::kV1:
+        case IndexVersion::kV2:
+            return Status::OK();
+    }
+    return {ErrorCodes::CannotCreateIndex,
+            str::stream() << "Invalid index specification " << indexSpec
+                          << "; cannot create an index with v=" << static_cast<int>(indexVersion)};
+}
+
+BSONObj buildRepairedIndexSpec(
+    const NamespaceString& ns,
+    const BSONObj& indexSpec,
+    const std::set<StringData>& allowedFieldNames,
+    std::function<void(const BSONElement&, BSONObjBuilder*)> indexSpecHandleFn) {
+    BSONObjBuilder builder;
+    for (const auto& indexSpecElem : indexSpec) {
+        StringData fieldName = indexSpecElem.fieldNameStringData();
+        if (allowedFieldNames.count(fieldName)) {
+            indexSpecHandleFn(indexSpecElem, &builder);
+        } else {
+            LOGV2_WARNING(23878,
+                          "Removing unknown field from index spec",
+                          "namespace"_attr = redact(ns.toString()),
+                          "fieldName"_attr = redact(fieldName),
+                          "indexSpec"_attr = redact(indexSpec));
+        }
+    }
+    return builder.obj();
+}
 }  // namespace
 
 Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion indexVersion) {
@@ -118,8 +139,12 @@ Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion inde
     if (key.isEmpty())
         return Status(code, "Index keys cannot be empty.");
 
-    string pluginName = IndexNames::findPluginName(key);
+    auto pluginName = IndexNames::findPluginName(key);
     if (pluginName.size()) {
+        if (pluginName == IndexNames::GEO_HAYSTACK)
+            return {
+                ErrorCodes::CannotCreateIndex,
+                str::stream() << "GeoHaystack indexes cannot be created in version 5.0 and above"};
         if (!IndexNames::isKnownName(pluginName))
             return Status(code, str::stream() << "Unknown index plugin '" << pluginName << '\'');
     }
@@ -137,12 +162,6 @@ Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion inde
                                           << static_cast<int>(indexVersion)};
                 }
 
-                if (pluginName == IndexNames::WILDCARD) {
-                    return {code,
-                            str::stream() << "'" << pluginName
-                                          << "' index plugin is not allowed with index version v:"
-                                          << static_cast<int>(indexVersion)};
-                }
                 break;
             }
             case IndexVersion::kV2: {
@@ -178,10 +197,17 @@ Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion inde
                                         << "' index must be a non-zero number, not a string.");
         }
 
-        // Check if the wildcard index is compounded. If it is the key is invalid because
-        // compounded wildcard indexes are disallowed.
-        if (pluginName == IndexNames::WILDCARD && key.nFields() != 1) {
-            return Status(code, "wildcard indexes do not allow compounding");
+        if (pluginName == IndexNames::WILDCARD || pluginName == IndexNames::COLUMN) {
+            StringData fieldName(keyElement.fieldNameStringData());
+            if (key.nFields() != 1) {
+                // Some special index types do not support compound indexes.
+                return Status(code,
+                              str::stream() << pluginName << " indexes do not allow compounding");
+            } else if ((fieldName != "$**") && !fieldName.endsWith(".$**")) {
+                // Invalid key names for wildcard or columnstore are not supported.
+                return Status(code,
+                              str::stream() << "Invalid key name for " << pluginName << " indexes");
+            }
         }
 
         // Ensure that the fields on which we are building the index are valid: a field must not
@@ -196,13 +222,13 @@ Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion inde
             return Status(code, "Index keys cannot be an empty field.");
         }
 
-        // "$**" is acceptable for a text index or wildcard index.
+        // "$**" is acceptable for a text, wildcard, or columnstore index.
         if ((keyElement.fieldNameStringData() == "$**") &&
-            ((keyElement.isNumber()) || (keyElement.valuestrsafe() == IndexNames::TEXT)))
+            ((keyElement.isNumber()) || (keyElement.str() == IndexNames::TEXT) ||
+             (keyElement.str() == IndexNames::COLUMN)))
             continue;
 
-        if ((keyElement.fieldNameStringData() == "_fts") &&
-            keyElement.valuestrsafe() != IndexNames::TEXT) {
+        if ((keyElement.fieldNameStringData() == "_fts") && keyElement.str() != IndexNames::TEXT) {
             return Status(code, "Index key contains an illegal field name: '_fts'");
         }
 
@@ -223,10 +249,10 @@ Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion inde
             const bool mightBePartOfDbRef =
                 (i != 0) && (part == "$db" || part == "$id" || part == "$ref");
 
-            const bool isPartOfWildcard =
-                (i == numParts - 1) && (part == "$**") && (pluginName == IndexNames::WILDCARD);
+            const bool isWildcardOrColumn = (i == numParts - 1) && (part == "$**") &&
+                (pluginName == IndexNames::WILDCARD || pluginName == IndexNames::COLUMN);
 
-            if (!mightBePartOfDbRef && !isPartOfWildcard) {
+            if (!mightBePartOfDbRef && !isWildcardOrColumn) {
                 return Status(code,
                               "Index key contains an illegal field name: "
                               "field name starts with '$'.");
@@ -237,32 +263,62 @@ Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion inde
     return Status::OK();
 }
 
-BSONObj removeUnknownFields(const BSONObj& indexSpec) {
-    BSONObjBuilder builder;
-    for (const auto& indexSpecElem : indexSpec) {
-        StringData fieldName = indexSpecElem.fieldNameStringData();
-        if (allowedFieldNames.count(fieldName)) {
-            builder.append(indexSpecElem);
-        } else {
-            LOGV2_WARNING(23878,
-                          "Removing field '{fieldName}' from index spec: {indexSpec}",
-                          "Removing unknown field from index spec",
-                          "fieldName"_attr = redact(fieldName),
-                          "indexSpec"_attr = redact(indexSpec));
-        }
-    }
-    return builder.obj();
+BSONObj removeUnknownFields(const NamespaceString& ns, const BSONObj& indexSpec) {
+    auto appendIndexSpecFn = [](const BSONElement& indexSpecElem, BSONObjBuilder* builder) {
+        builder->append(indexSpecElem);
+    };
+    return buildRepairedIndexSpec(ns, indexSpec, allowedFieldNames, appendIndexSpecFn);
 }
 
-StatusWith<BSONObj> validateIndexSpec(
-    OperationContext* opCtx,
-    const BSONObj& indexSpec,
-    const ServerGlobalParams::FeatureCompatibility& featureCompatibility) {
+BSONObj repairIndexSpec(const NamespaceString& ns,
+                        const BSONObj& indexSpec,
+                        const std::set<StringData>& allowedFieldNames) {
+    auto fixIndexSpecFn = [&indexSpec, &ns](const BSONElement& indexSpecElem,
+                                            BSONObjBuilder* builder) {
+        StringData fieldName = indexSpecElem.fieldNameStringData();
+        if ((IndexDescriptor::kBackgroundFieldName == fieldName ||
+             IndexDescriptor::kUniqueFieldName == fieldName ||
+             IndexDescriptor::kSparseFieldName == fieldName ||
+             IndexDescriptor::kDropDuplicatesFieldName == fieldName ||
+             IndexDescriptor::kPrepareUniqueFieldName == fieldName ||
+             IndexDescriptor::kClusteredFieldName == fieldName) &&
+            !indexSpecElem.isNumber() && !indexSpecElem.isBoolean() && indexSpecElem.trueValue()) {
+            LOGV2_WARNING(6444400,
+                          "Fixing boolean field from index spec",
+                          "namespace"_attr = redact(ns.toString()),
+                          "fieldName"_attr = redact(fieldName),
+                          "indexSpec"_attr = redact(indexSpec));
+            builder->appendBool(fieldName, true);
+        } else if (IndexDescriptor::kExpireAfterSecondsFieldName == fieldName &&
+                   !(indexSpecElem.isNumber() && !indexSpecElem.isNaN())) {
+            LOGV2_WARNING(6835900,
+                          "Fixing expire field from TTL index spec",
+                          "namespace"_attr = redact(ns.toString()),
+                          "fieldName"_attr = redact(fieldName),
+                          "indexSpec"_attr = redact(indexSpec));
+            builder->appendNumber(fieldName,
+                                  durationCount<Seconds>(kExpireAfterSecondsForInactiveTTLIndex));
+        } else {
+            builder->append(indexSpecElem);
+        }
+    };
+
+    return buildRepairedIndexSpec(ns, indexSpec, allowedFieldNames, fixIndexSpecFn);
+}
+
+StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& indexSpec) {
     bool hasKeyPatternField = false;
     bool hasIndexNameField = false;
     bool hasNamespaceField = false;
+    bool isTTLIndexWithNaNExpireAfterSeconds = false;
     bool hasVersionField = false;
     bool hasCollationField = false;
+    bool hasWeightsField = false;
+    bool hasOriginalSpecField = false;
+    bool unique = false;
+    bool prepareUnique = false;
+    auto clusteredField = indexSpec[IndexDescriptor::kClusteredFieldName];
+    bool apiStrict = opCtx && APIParameters::get(opCtx).getAPIStrict().value_or(false);
 
     auto fieldNamesValidStatus = validateIndexSpecFieldNames(indexSpec);
     if (!fieldNamesValidStatus.isOK()) {
@@ -270,12 +326,7 @@ StatusWith<BSONObj> validateIndexSpec(
     }
 
     boost::optional<IndexVersion> resolvedIndexVersion;
-
-    // Allow hidden index only if FCV is 4.4.
-    const auto isFeatureDisabled =
-        (!featureCompatibility.isVersionInitialized() ||
-         featureCompatibility.getVersion() <
-             ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44);
+    std::string indexType;
 
     for (auto&& indexSpecElem : indexSpec) {
         auto indexSpecElemFieldName = indexSpecElem.fieldNameStringData();
@@ -286,28 +337,35 @@ StatusWith<BSONObj> validateIndexSpec(
                             << "The field '" << IndexDescriptor::kKeyPatternFieldName
                             << "' must be an object, but got " << typeName(indexSpecElem.type())};
             }
-
+            auto keyPattern = indexSpecElem.Obj();
             std::vector<StringData> keys;
-            for (auto&& keyElem : indexSpecElem.Obj()) {
+            for (auto&& keyElem : keyPattern) {
                 auto keyElemFieldName = keyElem.fieldNameStringData();
                 if (std::find(keys.begin(), keys.end(), keyElemFieldName) != keys.end()) {
                     return {ErrorCodes::BadValue,
                             str::stream() << "The field '" << keyElemFieldName
                                           << "' appears multiple times in the index key pattern "
-                                          << indexSpecElem.Obj()};
+                                          << keyPattern};
                 }
                 keys.push_back(keyElemFieldName);
             }
 
+            indexType = IndexNames::findPluginName(keyPattern);
+            if (apiStrict && indexType == IndexNames::TEXT) {
+                return {ErrorCodes::APIStrictError,
+                        str::stream()
+                            << indexType << " indexes cannot be created with apiStrict: true"};
+            }
+
             // Here we always validate the key pattern according to the most recent rules, in order
             // to enforce that all new indexes have well-formed key patterns.
             Status keyPatternValidateStatus =
-                validateKeyPattern(indexSpecElem.Obj(), IndexDescriptor::kLatestIndexVersion);
+                validateKeyPattern(keyPattern, IndexDescriptor::kLatestIndexVersion);
             if (!keyPatternValidateStatus.isOK()) {
                 return keyPatternValidateStatus;
             }
 
-            for (const auto& keyElement : indexSpecElem.Obj()) {
+            for (const auto& keyElement : keyPattern) {
                 if (keyElement.type() == String && keyElement.str().empty()) {
                     return {ErrorCodes::CannotCreateIndex,
                             str::stream()
@@ -315,18 +373,6 @@ StatusWith<BSONObj> validateIndexSpec(
                 }
             }
 
-            // Allow compound hashed index only if FCV is 4.4.
-            const auto isFCV44 =
-                (featureCompatibility.isVersionInitialized() &&
-                 featureCompatibility.getVersion() ==
-                     ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44);
-            if (!isFCV44 && (indexSpecElem.embeddedObject().nFields() > 1) &&
-                (IndexNames::findPluginName(indexSpecElem.embeddedObject()) ==
-                 IndexNames::HASHED)) {
-                return {ErrorCodes::Error(16763),
-                        "Compound hashed indexes can only be created with FCV 4.4"};
-            }
-
             hasKeyPatternField = true;
         } else if (IndexDescriptor::kIndexNameFieldName == indexSpecElemFieldName) {
             if (indexSpecElem.type() != BSONType::String) {
@@ -338,10 +384,6 @@ StatusWith<BSONObj> validateIndexSpec(
 
             hasIndexNameField = true;
         } else if (IndexDescriptor::kHiddenFieldName == indexSpecElemFieldName) {
-            if (isFeatureDisabled) {
-                return {ErrorCodes::Error(31449),
-                        "Hidden indexes can only be created with FCV 4.4"};
-            }
             if (indexSpecElem.type() != BSONType::Bool) {
                 return {ErrorCodes::TypeMismatch,
                         str::stream()
@@ -369,14 +411,29 @@ StatusWith<BSONObj> validateIndexSpec(
 
             const IndexVersion requestedIndexVersion =
                 static_cast<IndexVersion>(*requestedIndexVersionAsInt);
-            auto creationAllowedStatus = IndexDescriptor::isIndexVersionAllowedForCreation(
-                requestedIndexVersion, featureCompatibility, indexSpec);
+            auto creationAllowedStatus =
+                isIndexVersionAllowedForCreation(requestedIndexVersion, indexSpec);
             if (!creationAllowedStatus.isOK()) {
                 return creationAllowedStatus;
             }
 
             hasVersionField = true;
             resolvedIndexVersion = requestedIndexVersion;
+        } else if (IndexDescriptor::kOriginalSpecFieldName == indexSpecElemFieldName) {
+            if (indexSpecElem.type() != BSONType::Object) {
+                return {ErrorCodes::TypeMismatch,
+                        str::stream()
+                            << "The field '" << IndexDescriptor::kOriginalSpecFieldName
+                            << "' must be an object, but got " << typeName(indexSpecElem.type())};
+            }
+
+            if (indexSpecElem.Obj().isEmpty()) {
+                return {ErrorCodes::BadValue,
+                        str::stream() << "The field '" << IndexDescriptor::kOriginalSpecFieldName
+                                      << "' cannot be an empty object."};
+            }
+
+            hasOriginalSpecField = true;
         } else if (IndexDescriptor::kCollationFieldName == indexSpecElemFieldName) {
             if (indexSpecElem.type() != BSONType::Object) {
                 return {ErrorCodes::TypeMismatch,
@@ -421,41 +478,99 @@ StatusWith<BSONObj> validateIndexSpec(
             if (!statusWithMatcher.isOK()) {
                 return statusWithMatcher.getStatus();
             }
-        } else if (IndexDescriptor::kPathProjectionFieldName == indexSpecElemFieldName) {
+        } else if (IndexDescriptor::kWildcardProjectionFieldName == indexSpecElemFieldName ||
+                   IndexDescriptor::kColumnStoreProjectionFieldName == indexSpecElemFieldName) {
+            const bool isWildcard =
+                IndexDescriptor::kWildcardProjectionFieldName == indexSpecElemFieldName;
+            const auto indexName = isWildcard ? IndexNames::WILDCARD : IndexNames::COLUMN;
             const auto key = indexSpec.getObjectField(IndexDescriptor::kKeyPatternFieldName);
-            if (IndexNames::findPluginName(key) != IndexNames::WILDCARD) {
-                return {ErrorCodes::BadValue,
-                        str::stream()
-                            << "The field '" << IndexDescriptor::kPathProjectionFieldName
-                            << "' is only allowed in an '" << IndexNames::WILDCARD << "' index"};
+            if (IndexNames::findPluginName(key) != indexName) {
+                // For backwards compatibility, we will return BadValue for Wildcard indices.
+                auto code =
+                    isWildcard ? ErrorCodes::BadValue : ErrorCodes::InvalidIndexSpecificationOption;
+                return {code,
+                        str::stream() << "The field '" << indexSpecElemFieldName
+                                      << "' is only allowed in '" << indexName << "' indexes"};
             }
             if (indexSpecElem.type() != BSONType::Object) {
                 return {ErrorCodes::TypeMismatch,
-                        str::stream() << "The field '" << IndexDescriptor::kPathProjectionFieldName
+                        str::stream() << "The field '" << indexSpecElemFieldName
                                       << "' must be a non-empty object, but got "
                                       << typeName(indexSpecElem.type())};
             }
             if (!key.hasField("$**")) {
                 return {ErrorCodes::FailedToParse,
                         str::stream()
-                            << "The field '" << IndexDescriptor::kPathProjectionFieldName
-                            << "' is only allowed when '" << IndexDescriptor::kKeyPatternFieldName
-                            << "' is {\"$**\": ±1}"};
+                            << "The field '" << indexSpecElemFieldName << "' is only allowed when '"
+                            << IndexDescriptor::kKeyPatternFieldName
+                            << "' is {\"$**\": ±1} or {\"$**\": \"columnstore\"}"};
             }
-
             if (indexSpecElem.embeddedObject().isEmpty()) {
                 return {ErrorCodes::FailedToParse,
-                        str::stream() << "The '" << IndexDescriptor::kPathProjectionFieldName
+                        str::stream() << "The '" << indexSpecElemFieldName
                                       << "' field can't be an empty object"};
             }
             try {
-                // We use WildcardKeyGenerator::createProjectionExec to parse and validate the path
-                // projection spec.
-                WildcardKeyGenerator::createProjectionExecutor(key, indexSpecElem.embeddedObject());
+                // We use createProjectionExecutor to parse and validate the path projection spec.
+                if (isWildcard) {
+                    WildcardKeyGenerator::createProjectionExecutor(key,
+                                                                   indexSpecElem.embeddedObject());
+                } else {
+                    column_keygen::ColumnKeyGenerator::createProjectionExecutor(
+                        key, indexSpecElem.embeddedObject());
+                }
+
             } catch (const DBException& ex) {
-                return ex.toStatus(str::stream() << "Failed to parse: "
-                                                 << IndexDescriptor::kPathProjectionFieldName);
+                return ex.toStatus(str::stream()
+                                   << "Failed to parse projection: " << indexSpecElemFieldName);
             }
+        } else if (IndexDescriptor::kWeightsFieldName == indexSpecElemFieldName) {
+            if (!indexSpecElem.isABSONObj() && indexSpecElem.type() != String) {
+                return {ErrorCodes::TypeMismatch,
+                        str::stream()
+                            << "The field '" << indexSpecElemFieldName
+                            << "' must be an object, but got " << typeName(indexSpecElem.type())};
+            }
+            hasWeightsField = true;
+        } else if ((IndexDescriptor::kBackgroundFieldName == indexSpecElemFieldName ||
+                    IndexDescriptor::kUniqueFieldName == indexSpecElemFieldName ||
+                    IndexDescriptor::kSparseFieldName == indexSpecElemFieldName ||
+                    IndexDescriptor::kDropDuplicatesFieldName == indexSpecElemFieldName ||
+                    IndexDescriptor::kPrepareUniqueFieldName == indexSpecElemFieldName ||
+                    IndexDescriptor::kClusteredFieldName == indexSpecElemFieldName)) {
+            if (!indexSpecElem.isNumber() && !indexSpecElem.isBoolean()) {
+                return {ErrorCodes::TypeMismatch,
+                        str::stream()
+                            << "The field '" << indexSpecElemFieldName << " has value "
+                            << indexSpecElem.toString() << ", which is not convertible to bool"};
+            }
+            if (IndexDescriptor::kUniqueFieldName == indexSpecElemFieldName) {
+                unique = indexSpecElem.trueValue();
+            }
+            if (IndexDescriptor::kPrepareUniqueFieldName == indexSpecElemFieldName) {
+                prepareUnique = indexSpecElem.trueValue();
+            }
+        } else if ((IndexDescriptor::kDefaultLanguageFieldName == indexSpecElemFieldName ||
+                    IndexDescriptor::kLanguageOverrideFieldName == indexSpecElemFieldName) &&
+                   indexSpecElem.type() != BSONType::String) {
+            return {ErrorCodes::TypeMismatch,
+                    str::stream() << "The field '" << indexSpecElemFieldName
+                                  << "' must be a string, but got "
+                                  << typeName(indexSpecElem.type())};
+        } else if ((IndexDescriptor::k2dsphereVersionFieldName == indexSpecElemFieldName ||
+                    IndexDescriptor::kTextVersionFieldName == indexSpecElemFieldName ||
+                    IndexDescriptor::k2dIndexBitsFieldName == indexSpecElemFieldName ||
+                    IndexDescriptor::k2dIndexMinFieldName == indexSpecElemFieldName ||
+                    IndexDescriptor::k2dIndexMaxFieldName == indexSpecElemFieldName ||
+                    IndexDescriptor::k2dsphereCoarsestIndexedLevel == indexSpecElemFieldName ||
+                    IndexDescriptor::k2dsphereFinestIndexedLevel == indexSpecElemFieldName) &&
+                   !indexSpecElem.isNumber()) {
+            return {ErrorCodes::TypeMismatch,
+                    str::stream() << "The field '" << indexSpecElemFieldName
+                                  << "' must be a number, but got "
+                                  << typeName(indexSpecElem.type())};
+        } else if (IndexDescriptor::kExpireAfterSecondsFieldName == indexSpecElemFieldName) {
+            isTTLIndexWithNaNExpireAfterSeconds = indexSpecElem.isNaN();
         } else {
             // We can assume field name is valid at this point. Validation of fieldname is handled
             // prior to this in validateIndexSpecFieldNames().
@@ -479,6 +594,21 @@ StatusWith<BSONObj> validateIndexSpec(
                               << "' field is a required property of an index specification"};
     }
 
+    if (clusteredField) {
+        if (!clusteredField.trueValue()) {
+            // Disallow 'clustered' from taking value 'false'
+            return {ErrorCodes::Error(6492800), "Value 'false' for field 'clustered' is invalid"};
+        }
+
+        if (!indexSpec.hasField(IndexDescriptor::kUniqueFieldName) ||
+            indexSpec.getBoolField(IndexDescriptor::kUniqueFieldName) == false) {
+            // Only require 'unique' if clustered is specified.
+            return {ErrorCodes::CannotCreateIndex,
+                    str::stream() << "The '" << IndexDescriptor::kUniqueFieldName
+                                  << "' field is required when 'clustered' is specified"};
+        }
+    }
+
     if (hasCollationField && *resolvedIndexVersion < IndexVersion::kV2) {
         return {ErrorCodes::CannotCreateIndex,
                 str::stream() << "Invalid index specification " << indexSpec
@@ -488,6 +618,21 @@ StatusWith<BSONObj> validateIndexSpec(
                               << static_cast<int>(*resolvedIndexVersion)};
     }
 
+    if (indexType != IndexNames::TEXT && hasWeightsField) {
+        return {ErrorCodes::CannotCreateIndex,
+                str::stream() << "Invalid index specification " << indexSpec << "; the field '"
+                              << IndexDescriptor::kWeightsFieldName
+                              << "' can only be specified with text indexes"};
+    }
+
+    if (unique && prepareUnique) {
+        return {ErrorCodes::CannotCreateIndex,
+                str::stream() << "Invalid index specification " << indexSpec
+                              << "; cannot create an index with the '"
+                              << IndexDescriptor::kUniqueFieldName << "' option and the '"
+                              << IndexDescriptor::kPrepareUniqueFieldName << "' option"};
+    }
+
     BSONObj modifiedSpec = indexSpec;
 
     // Ignore any 'ns' field in the index spec because this field is dropped post-4.0. Don't remove
@@ -497,6 +642,19 @@ StatusWith<BSONObj> validateIndexSpec(
         modifiedSpec = modifiedSpec.removeField(IndexDescriptor::kNamespaceFieldName);
     }
 
+    if (isTTLIndexWithNaNExpireAfterSeconds &&
+        !skipTTLIndexNaNExpireAfterSecondsValidation.shouldFail()) {
+        // We create a new index specification with the 'expireAfterSeconds' field set as
+        // kExpireAfterSecondsForInactiveTTLIndex if the current value is NaN. A similar
+        // treatment is done in repairIndexSpec(). This rewrites the 'expireAfterSeconds'
+        // value to be compliant with the 'safeInt' IDL type for the listIndexes response.
+        BSONObjBuilder builder;
+        builder.appendNumber(IndexDescriptor::kExpireAfterSecondsFieldName,
+                             durationCount<Seconds>(kExpireAfterSecondsForInactiveTTLIndex));
+        auto obj = builder.obj();
+        modifiedSpec = modifiedSpec.addField(obj.firstElement());
+    }
+
     if (!hasVersionField) {
         // We create a new index specification with the 'v' field set as 'defaultIndexVersion' if
         // the field was omitted.
@@ -505,18 +663,37 @@ StatusWith<BSONObj> validateIndexSpec(
         modifiedSpec = modifiedSpec.addField(versionObj.firstElement());
     }
 
+    if (hasOriginalSpecField) {
+        StatusWith<BSONObj> modifiedOriginalSpec = validateIndexSpec(
+            opCtx, indexSpec.getObjectField(IndexDescriptor::kOriginalSpecFieldName));
+        if (!modifiedOriginalSpec.isOK()) {
+            return modifiedOriginalSpec.getStatus();
+        }
+
+        BSONObj specToAdd =
+            BSON(IndexDescriptor::kOriginalSpecFieldName << modifiedOriginalSpec.getValue());
+        modifiedSpec = modifiedSpec.addField(specToAdd.firstElement());
+    }
+
     return modifiedSpec;
 }
 
 Status validateIdIndexSpec(const BSONObj& indexSpec) {
-    for (auto&& indexSpecElem : indexSpec) {
-        auto indexSpecElemFieldName = indexSpecElem.fieldNameStringData();
-        if (!allowedIdIndexFieldNames.count(indexSpecElemFieldName)) {
-            return {
-                ErrorCodes::InvalidIndexSpecificationOption,
-                str::stream() << "The field '" << indexSpecElemFieldName
-                              << "' is not valid for an _id index specification. Specification: "
-                              << indexSpec};
+    bool isClusteredIndexSpec = indexSpec.hasField(IndexDescriptor::kClusteredFieldName);
+
+    if (!isClusteredIndexSpec) {
+        // Field names for a 'clustered' index spec have already been validated through
+        // allowedClusteredIndexFieldNames.
+
+        for (auto&& indexSpecElem : indexSpec) {
+            auto indexSpecElemFieldName = indexSpecElem.fieldNameStringData();
+            if (!allowedIdIndexFieldNames.count(indexSpecElemFieldName)) {
+                return {ErrorCodes::InvalidIndexSpecificationOption,
+                        str::stream()
+                            << "The field '" << indexSpecElemFieldName
+                            << "' is not valid for an _id index specification. Specification: "
+                            << indexSpec};
+            }
         }
     }
 
@@ -537,6 +714,23 @@ Status validateIdIndexSpec(const BSONObj& indexSpec) {
     return Status::OK();
 }
 
+/**
+ * Top-level index spec field names for a "clustered" index are specified here.
+ */
+Status validateClusteredSpecFieldNames(const BSONObj& indexSpec) {
+    for (auto&& indexSpecElem : indexSpec) {
+        auto indexSpecElemFieldName = indexSpecElem.fieldNameStringData();
+        if (!allowedClusteredIndexFieldNames.count(indexSpecElemFieldName)) {
+            return {ErrorCodes::InvalidIndexSpecificationOption,
+                    str::stream()
+                        << "The field '" << indexSpecElemFieldName
+                        << "' is not valid for a clustered index specification. Specification: "
+                        << indexSpec};
+        }
+    }
+    return Status::OK();
+}
+
 /**
  * Top-level index spec field names are validated here. When adding a new field with a document as
  * value, is the the sub-module's responsibility to ensure that the content is valid and that only
@@ -547,6 +741,10 @@ Status validateIndexSpecFieldNames(const BSONObj& indexSpec) {
         return Status::OK();
     }
 
+    if (indexSpec.hasField(IndexDescriptor::kClusteredFieldName)) {
+        return validateClusteredSpecFieldNames(indexSpec);
+    }
+
     for (auto&& indexSpecElem : indexSpec) {
         auto indexSpecElemFieldName = indexSpecElem.fieldNameStringData();
         if (!allowedFieldNames.count(indexSpecElemFieldName)) {
@@ -617,6 +815,91 @@ StatusWith<BSONObj> validateIndexSpecCollation(OperationContext* opCtx,
     return indexSpec;
 }
 
+Status validateExpireAfterSeconds(std::int64_t expireAfterSeconds,
+                                  ValidateExpireAfterSecondsMode mode) {
+    if (expireAfterSeconds < 0) {
+        return {ErrorCodes::InvalidOptions,
+                str::stream() << "TTL index '" << IndexDescriptor::kExpireAfterSecondsFieldName
+                              << "' option cannot be less than 0"};
+    }
+
+    const std::string tooLargeErr = str::stream()
+        << "TTL index '" << IndexDescriptor::kExpireAfterSecondsFieldName
+        << "' option must be within an acceptable range, try a lower number";
+
+    if (mode == ValidateExpireAfterSecondsMode::kSecondaryTTLIndex) {
+        // Relax epoch restriction on TTL indexes. This allows us to export and import existing
+        // TTL indexes with large values or NaN for the 'expireAfterSeconds' field.
+        // Additionally, the 'expireAfterSeconds' for TTL indexes is defined as safeInt (int32_t)
+        // in the IDL for listIndexes and collMod. See list_indexes.idl and coll_mod.idl.
+        if (expireAfterSeconds > std::numeric_limits<std::int32_t>::max()) {
+            return {ErrorCodes::InvalidOptions, tooLargeErr};
+        }
+    } else {
+        // Clustered collections with TTL.
+        // Note that 'expireAfterSeconds' is defined as safeInt64 in the IDL for the create and
+        // collMod commands. See create.idl and coll_mod.idl.
+        // There are two cases where we can encounter an issue here.
+        // The first case is when we try to cast to millseconds from seconds, which could cause an
+        // overflow. The second case is where 'expireAfterSeconds' is larger than the current epoch
+        // time. This isn't necessarily problematic for the general case, but for the specific case
+        // of time series collections, we cluster the collection by an OID value, where the
+        // timestamp portion is only a 32-bit unsigned integer offset of seconds since the epoch.
+        if (expireAfterSeconds > std::numeric_limits<std::int64_t>::max() / 1000) {
+            return {ErrorCodes::InvalidOptions, tooLargeErr};
+        }
+        auto expireAfterMillis = duration_cast<Milliseconds>(Seconds(expireAfterSeconds));
+        if (expireAfterMillis > Date_t::now().toDurationSinceEpoch()) {
+            return {ErrorCodes::InvalidOptions, tooLargeErr};
+        }
+    }
+    return Status::OK();
+}
+
+bool isIndexTTL(const BSONObj& indexSpec) {
+    return indexSpec.hasField(IndexDescriptor::kExpireAfterSecondsFieldName);
+}
+
+Status validateIndexSpecTTL(const BSONObj& indexSpec) {
+    if (!isIndexTTL(indexSpec)) {
+        return Status::OK();
+    }
+
+    const BSONElement expireAfterSecondsElt =
+        indexSpec[IndexDescriptor::kExpireAfterSecondsFieldName];
+    if (!expireAfterSecondsElt.isNumber()) {
+        return {ErrorCodes::CannotCreateIndex,
+                str::stream() << "TTL index '" << IndexDescriptor::kExpireAfterSecondsFieldName
+                              << "' option must be numeric, but received a type of '"
+                              << typeName(expireAfterSecondsElt.type())
+                              << "'. Index spec: " << indexSpec};
+    }
+
+    if (auto status =
+            validateExpireAfterSeconds(expireAfterSecondsElt.safeNumberLong(),
+                                       ValidateExpireAfterSecondsMode::kSecondaryTTLIndex);
+        !status.isOK()) {
+        return {ErrorCodes::CannotCreateIndex,
+                str::stream() << status.reason() << ". Index spec: " << indexSpec};
+    }
+
+    const BSONObj key = indexSpec["key"].Obj();
+    if (key.nFields() != 1) {
+        return {ErrorCodes::CannotCreateIndex,
+                str::stream() << "TTL indexes are single-field indexes, compound indexes do "
+                                 "not support TTL. Index spec: "
+                              << indexSpec};
+    }
+
+    return Status::OK();
+}
+
+bool isIndexAllowedInAPIVersion1(const IndexDescriptor& indexDesc) {
+    const auto indexName = IndexNames::findPluginName(indexDesc.keyPattern());
+    return indexName != IndexNames::TEXT && indexName != IndexNames::GEO_HAYSTACK &&
+        !indexDesc.isSparse();
+}
+
 GlobalInitializerRegisterer filterAllowedIndexFieldNamesInitializer(
     "FilterAllowedIndexFieldNames", [](InitializerContext* service) {
         if (filterAllowedIndexFieldNames)
