@@ -32,18 +32,19 @@
 #include <boost/optional.hpp>
 #include <functional>
 
+#include "mongo/client/read_preference.h"
+#include "mongo/db/api_parameters.h"
 #include "mongo/db/auth/privilege.h"
 #include "mongo/db/auth/user_name.h"
 #include "mongo/db/cursor_id.h"
 #include "mongo/db/jsobj.h"
-#include "mongo/db/logical_session_id.h"
 #include "mongo/db/query/plan_executor.h"
 #include "mongo/db/record_id.h"
 #include "mongo/db/repl/read_concern_level.h"
+#include "mongo/db/session/logical_session_id.h"
 
 namespace mongo {
 
-class Collection;
 class CursorManager;
 class RecoveryUnit;
 
@@ -55,85 +56,49 @@ class RecoveryUnit;
  * using a CursorManager. See cursor_manager.h for more details.
  */
 struct ClientCursorParams {
-    // Describes whether callers should acquire locks when using a ClientCursor. Not all cursors
-    // have the same locking behavior. In particular, find cursors require the caller to lock the
-    // collection in MODE_IS before calling methods on the underlying plan executor. Aggregate
-    // cursors, on the other hand, may access multiple collections and acquire their own locks on
-    // any involved collections while producing query results. Therefore, the caller need not
-    // explicitly acquire any locks when using a ClientCursor which houses execution machinery for
-    // an aggregate.
-    //
-    // The policy is consulted on getMore in order to determine locking behavior, since during
-    // getMore we otherwise could not easily know what flavor of cursor we're using.
-    enum class LockPolicy {
-        // The caller is responsible for locking the collection over which this ClientCursor
-        // executes.
-        kLockExternally,
-
-        // The caller need not hold no locks; this ClientCursor's plan executor acquires any
-        // necessary locks itself.
-        kLocksInternally,
-    };
-
     ClientCursorParams(std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> planExecutor,
                        NamespaceString nss,
-                       UserNameIterator authenticatedUsersIter,
+                       boost::optional<UserName> authenticatedUser,
+                       APIParameters apiParameters,
                        WriteConcernOptions writeConcernOptions,
                        repl::ReadConcernArgs readConcernArgs,
+                       ReadPreferenceSetting readPreferenceSetting,
                        BSONObj originatingCommandObj,
-                       LockPolicy lockPolicy,
-                       PrivilegeVector originatingPrivileges,
-                       bool needsMerge,
-                       bool isOpQueryExhaust)
+                       PrivilegeVector originatingPrivileges)
         : exec(std::move(planExecutor)),
           nss(std::move(nss)),
+          authenticatedUser(std::move(authenticatedUser)),
+          apiParameters(std::move(apiParameters)),
           writeConcernOptions(std::move(writeConcernOptions)),
           readConcernArgs(std::move(readConcernArgs)),
-          queryOptions(exec->getCanonicalQuery()
-                           ? exec->getCanonicalQuery()->getQueryRequest().getOptions()
-                           : 0),
+          readPreferenceSetting(std::move(readPreferenceSetting)),
+          isNoTimeout(
+              exec->getCanonicalQuery()
+                  ? static_cast<bool>(
+                        exec->getCanonicalQuery()->getFindCommandRequest().getNoCursorTimeout())
+                  : false),
+          tailableMode(exec->getCanonicalQuery()
+                           ? query_request_helper::getTailableMode(
+                                 exec->getCanonicalQuery()->getFindCommandRequest())
+                           : TailableModeEnum::kNormal),
           originatingCommandObj(originatingCommandObj.getOwned()),
-          lockPolicy(lockPolicy),
-          originatingPrivileges(std::move(originatingPrivileges)),
-          needsMerge(needsMerge) {
-        while (authenticatedUsersIter.more()) {
-            authenticatedUsers.emplace_back(authenticatedUsersIter.next());
-        }
+          originatingPrivileges(std::move(originatingPrivileges)) {}
 
-        // There are separate implementations for exhaust cursors using OP_QUERY and exhaust cursors
-        // using OP_MSG. In the case of OP_QUERY exhaust, the OP_GET_MORE code consults the cursor's
-        // 'queryOptions' bit vector in order to determine whether another getMore should run again
-        // immediately without waiting for a request from the client. Here we set the exhaust bit if
-        // the cursor is being registered as an OP_QUERY exhaust cursor.
-        if (isOpQueryExhaust) {
-            queryOptions |= QueryOption_Exhaust;
-        }
-    }
-
-    void setTailable(bool tailable) {
-        if (tailable)
-            queryOptions |= QueryOption_CursorTailable;
-        else
-            queryOptions &= ~QueryOption_CursorTailable;
-    }
-
-    void setAwaitData(bool awaitData) {
-        if (awaitData)
-            queryOptions |= QueryOption_AwaitData;
-        else
-            queryOptions &= ~QueryOption_AwaitData;
+    void setTailableMode(TailableModeEnum newMode) {
+        tailableMode = newMode;
     }
 
     std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> exec;
     const NamespaceString nss;
-    std::vector<UserName> authenticatedUsers;
+    boost::optional<UserName> authenticatedUser;
+    const APIParameters apiParameters;
     const WriteConcernOptions writeConcernOptions;
     const repl::ReadConcernArgs readConcernArgs;
-    int queryOptions = 0;
+    const ReadPreferenceSetting readPreferenceSetting;
+    const bool isNoTimeout;
+    TailableModeEnum tailableMode;
     BSONObj originatingCommandObj;
-    const LockPolicy lockPolicy;
     PrivilegeVector originatingPrivileges;
-    const bool needsMerge;
 };
 
 /**
@@ -165,8 +130,8 @@ public:
         return _nss;
     }
 
-    UserNameIterator getAuthenticatedUsers() const {
-        return makeUserNameIterator(_authenticatedUsers.begin(), _authenticatedUsers.end());
+    boost::optional<UserName> getAuthenticatedUser() const {
+        return _authenticatedUser;
     }
 
     boost::optional<LogicalSessionId> getSessionId() const {
@@ -177,16 +142,20 @@ public:
         return _txnNumber;
     }
 
-    repl::ReadConcernArgs getReadConcernArgs() const {
-        return _readConcernArgs;
+    APIParameters getAPIParameters() const {
+        return _apiParameters;
     }
 
     WriteConcernOptions getWriteConcernOptions() const {
         return _writeConcernOptions;
     }
 
-    bool needsMerge() const {
-        return _needsMerge;
+    repl::ReadConcernArgs getReadConcernArgs() const {
+        return _readConcernArgs;
+    }
+
+    ReadPreferenceSetting getReadPreferenceSetting() const {
+        return _readPreferenceSetting;
     }
 
     /**
@@ -197,21 +166,20 @@ public:
         return _exec.get();
     }
 
-    /**
-     * Returns the query options bitmask.  If you'd like to know if the cursor is tailable or
-     * awaitData, prefer using the specific methods isTailable() and isAwaitData() over using this
-     * method.
-     */
-    int queryOptions() const {
-        return _queryOptions;
-    }
-
     bool isTailable() const {
-        return _queryOptions & QueryOption_CursorTailable;
+        switch (_tailableMode) {
+            case TailableModeEnum::kNormal:
+                return false;
+            case TailableModeEnum::kTailable:
+            case TailableModeEnum::kTailableAndAwaitData:
+                return true;
+        }
+
+        MONGO_UNREACHABLE;
     }
 
     bool isAwaitData() const {
-        return _queryOptions & QueryOption_AwaitData;
+        return _tailableMode == TailableModeEnum::kTailableAndAwaitData;
     }
 
     /**
@@ -230,13 +198,6 @@ public:
     }
     void getOriginatingPrivileges() && = delete;
 
-    /**
-     * Returns the total number of query results returned by the cursor so far.
-     */
-    std::uint64_t nReturnedSoFar() const {
-        return _nReturnedSoFar;
-    }
-
     /**
      * Increments the cursor's tracked number of query results returned so far by 'n'.
      */
@@ -282,10 +243,6 @@ public:
         return StringData(_planSummary);
     }
 
-    ClientCursorParams::LockPolicy lockPolicy() const {
-        return _lockPolicy;
-    }
-
     /**
      * Returns a generic cursor containing diagnostics about this cursor.
      * The caller must either have this cursor pinned or hold a mutex from the cursor manager.
@@ -334,6 +291,14 @@ public:
         return _opKey;
     }
 
+    std::unique_ptr<RecoveryUnit> releaseStashedRecoveryUnit() {
+        return std::move(_stashedRecoveryUnit);
+    }
+
+    void stashRecoveryUnit(std::unique_ptr<RecoveryUnit> ru) {
+        _stashedRecoveryUnit = std::move(ru);
+    }
+
 private:
     friend class CursorManager;
     friend class ClientCursorPin;
@@ -380,7 +345,7 @@ private:
     void dispose(OperationContext* opCtx);
 
     bool isNoTimeout() const {
-        return (_queryOptions & QueryOption_NoCursorTimeout);
+        return _isNoTimeout;
     }
 
     // The ID of the ClientCursor. A value of 0 is used to mean that no cursor id has been assigned.
@@ -390,11 +355,11 @@ private:
     // have the correct partition of the CursorManager locked (just like _authenticatedUsers).
     const NamespaceString _nss;
 
-    // The set of authenticated users when this cursor was created. Threads may read from this
+    // The authenticated user when this cursor was created. Threads may read from this
     // field (using the getter) even if they don't have the cursor pinned as long as they hold the
     // correct partition's lock in the CursorManager. They must hold the lock to prevent the cursor
     // from being freed by another thread during the read.
-    const std::vector<UserName> _authenticatedUsers;
+    const boost::optional<UserName> _authenticatedUser;
 
     // A logical session id for this cursor, if it is running inside of a session.
     const boost::optional<LogicalSessionId> _lsid;
@@ -402,14 +367,17 @@ private:
     // A transaction number for this cursor, if it was provided in the originating command.
     const boost::optional<TxnNumber> _txnNumber;
 
+    const APIParameters _apiParameters;
     const WriteConcernOptions _writeConcernOptions;
     const repl::ReadConcernArgs _readConcernArgs;
+    const ReadPreferenceSetting _readPreferenceSetting;
 
     // Tracks whether dispose() has been called, to make sure it happens before destruction. It is
     // an error to use a ClientCursor once it has been disposed.
     bool _disposed = false;
 
-    // Tracks the number of results returned by this cursor so far.
+    // Tracks the number of results returned by this cursor so far. Tracked only as debugging info
+    // for display in $currentOp output.
     std::uint64_t _nReturnedSoFar = 0;
 
     // Tracks the number of batches returned by this cursor so far.
@@ -421,21 +389,19 @@ private:
     // The privileges required for the _originatingCommand.
     const PrivilegeVector _originatingPrivileges;
 
-    // See the QueryOptions enum in dbclientinterface.h.
-    const int _queryOptions = 0;
+    const TailableModeEnum _tailableMode;
 
-    const ClientCursorParams::LockPolicy _lockPolicy;
-
-    // The value of a flag specified on the originating command which indicates whether the result
-    // of this cursor will be consumed by a merging node (mongos or a mongod selected to perform a
-    // merge). Note that this flag is only set for aggregate() commands, and not for find()
-    // commands. It is therefore possible that 'needsMerge' is false when in fact there will be a
-    // merge performed.
-    const bool _needsMerge;
+    const bool _isNoTimeout;
 
     // Unused maxTime budget for this cursor.
     Microseconds _leftoverMaxTimeMicros = Microseconds::max();
 
+    // Stashed recovery unit. Maintains valid and positioned cursors across commands, so that data
+    // pointers remain valid and safe to access. May be nullptr. This field MUST come before
+    // '_exec' as we cannot destroy the recovery unit until the plan executor and its resources
+    // (cursors) have been destroyed.
+    std::unique_ptr<RecoveryUnit> _stashedRecoveryUnit;
+
     // The underlying query execution machinery. Must be non-null.
     std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> _exec;
 
@@ -463,9 +429,14 @@ private:
     std::string _planSummary;
 
     // Commit point at the time the last batch was returned. This is only used by internal exhaust
-    // oplog fetching. Also see lastKnownCommittedOpTime in GetMoreRequest.
+    // oplog fetching.
     boost::optional<repl::OpTime> _lastKnownCommittedOpTime;
 
+    // Passed along from the original query so that it can be logged if necessary in getMore
+    // requests.
+    boost::optional<uint32_t> _planCacheKey;
+    boost::optional<uint32_t> _queryHash;
+
     // The client OperationKey associated with this cursor.
     boost::optional<OperationKey> _opKey;
 };
@@ -496,6 +467,10 @@ private:
  *
  *     ClientCursor* cursor = pin.getValue().getCursor();
  *     // Use cursor. Pin automatically released on block exit.
+ *
+ *     // Use the PlanExecutor associated with the cursor.
+ *     auto next = cursor->planExecutor()->getNext();
+ *
  * }
  *
  * Callers need not hold any lock manager locks in order to obtain or release a client cursor pin.
@@ -546,6 +521,18 @@ public:
         return _cursor;
     }
 
+    /*
+     * Unstashes resources in the cursor onto the operation context using the cursor. This _must_
+     * be called before using the plan executor associated with the cursor.
+     */
+    void unstashResourcesOntoOperationContext();
+
+    /**
+     * Inverse of above: Transfers resources which need the same lifetime as the cursor from the
+     * operation context to the cursor itself.
+     */
+    void stashResourcesFromOperationContext();
+
 private:
     friend class CursorManager;
 
@@ -554,6 +541,7 @@ private:
     OperationContext* _opCtx = nullptr;
     ClientCursor* _cursor = nullptr;
     CursorManager* _cursorManager = nullptr;
+    bool _shouldSaveRecoveryUnit = false;
 };
 
 void startClientCursorMonitor();
