@@ -27,12 +27,10 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/cloner.h"
-#include "mongo/db/cloner_gen.h"
 
 #include <algorithm>
 
@@ -41,15 +39,15 @@
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/bson/util/builder.h"
 #include "mongo/client/authenticate.h"
-#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/catalog/collection_write_path.h"
 #include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/cloner_gen.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/list_collections_filter.h"
-#include "mongo/db/commands/rename_collection.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/index/index_descriptor.h"
@@ -60,19 +58,21 @@
 #include "mongo/db/repl/isself.h"
 #include "mongo/db/repl/read_concern_args.h"
 #include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/service_context.h"
-#include "mongo/db/storage/durable_catalog.h"
 #include "mongo/logv2/log.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/fail_point.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
+
+
 namespace mongo {
 
 using IndexVersion = IndexDescriptor::IndexVersion;
 
 MONGO_FAIL_POINT_DEFINE(movePrimaryFailPoint);
-MONGO_FAIL_POINT_DEFINE(movePrimaryClonerHangBeforeStartCloneDocuments);
 
 BSONElement getErrField(const BSONObj& o);
 
@@ -90,13 +90,15 @@ BSONObj Cloner::_getIdIndexSpec(const std::list<BSONObj>& indexSpecs) {
 
 Cloner::Cloner() {}
 
-struct Cloner::Fun {
-    Fun(OperationContext* opCtx, const std::string& dbName)
-        : lastLog(0), opCtx(opCtx), _dbName(dbName) {}
+struct Cloner::BatchHandler {
+    BatchHandler(OperationContext* opCtx, const std::string& dbName)
+        : lastLog(0), opCtx(opCtx), _dbName(dbName), numSeen(0), saveLast(0) {}
 
-    void operator()(DBClientCursorBatchIterator& i) {
+    void operator()(DBClientCursor& cursor) {
         boost::optional<Lock::DBLock> dbLock;
-        dbLock.emplace(opCtx, _dbName, MODE_X);
+        // TODO SERVER-63111 Once the Cloner holds a DatabaseName obj, use _dbName directly
+        DatabaseName dbName(boost::none, _dbName);
+        dbLock.emplace(opCtx, dbName, MODE_X);
         uassert(ErrorCodes::NotWritablePrimary,
                 str::stream() << "Not primary while cloning collection " << nss,
                 !opCtx->writesAreReplicated() ||
@@ -104,8 +106,9 @@ struct Cloner::Fun {
 
         // Make sure database still exists after we resume from the temp release
         auto databaseHolder = DatabaseHolder::get(opCtx);
-        auto db = databaseHolder->openDb(opCtx, _dbName);
-        auto collection = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
+        auto db = databaseHolder->openDb(opCtx, dbName);
+        auto catalog = CollectionCatalog::get(opCtx);
+        auto collection = catalog->lookupCollectionByNamespace(opCtx, nss);
         if (!collection) {
             writeConflictRetry(opCtx, "createCollection", nss.ns(), [&] {
                 opCtx->checkForInterrupt();
@@ -119,13 +122,13 @@ struct Cloner::Fun {
                           str::stream()
                               << "collection creation failed during clone [" << nss << "]");
                 wunit.commit();
-                collection = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
+                collection = catalog->lookupCollectionByNamespace(opCtx, nss);
                 invariant(collection,
                           str::stream() << "Missing collection during clone [" << nss << "]");
             });
         }
 
-        while (i.moreInCurrentBatch()) {
+        while (cursor.moreInCurrentBatch()) {
             if (numSeen % 128 == 127) {
                 time_t now = time(nullptr);
                 if (now - lastLog >= 60) {
@@ -140,7 +143,10 @@ struct Cloner::Fun {
 
                 CurOp::get(opCtx)->yielded();
 
-                dbLock.emplace(opCtx, _dbName, MODE_X);
+                // TODO SERVER-63111 Once the cloner takes in a DatabaseName obj, use _dbName
+                // directly
+                DatabaseName dbName(boost::none, _dbName);
+                dbLock.emplace(opCtx, dbName, MODE_X);
 
                 // Check if everything is still all right.
                 if (opCtx->writesAreReplicated()) {
@@ -150,23 +156,22 @@ struct Cloner::Fun {
                         repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, nss));
                 }
 
-                db = databaseHolder->getDb(opCtx, _dbName);
+                db = databaseHolder->getDb(opCtx, dbName);
                 uassert(28593,
                         str::stream() << "Database " << _dbName << " dropped while cloning",
                         db != nullptr);
 
-                collection = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
+                collection = catalog->lookupCollectionByNamespace(opCtx, nss);
                 uassert(28594,
                         str::stream() << "Collection " << nss << " dropped while cloning",
-                        collection != nullptr);
+                        collection);
             }
 
-            BSONObj tmp = i.nextSafe();
+            BSONObj tmp = cursor.nextSafe();
 
             /* assure object is valid.  note this will slow us down a little. */
-            // Use the latest BSON validation version. We allow cloning of collections containing
-            // decimal data even if decimal is disabled.
-            const Status status = validateBSON(tmp.objdata(), tmp.objsize(), BSONVersion::kLatest);
+            // We allow cloning of collections containing decimal data even if decimal is disabled.
+            const Status status = validateBSON(tmp.objdata(), tmp.objsize());
             if (!status.isOK()) {
                 if (gSkipCorruptDocumentsWhenCloning.load()) {
                     LOGV2_WARNING(20423,
@@ -189,9 +194,8 @@ struct Cloner::Fun {
                 WriteUnitOfWork wunit(opCtx);
 
                 BSONObj doc = tmp;
-                OpDebug* const nullOpDebug = nullptr;
-                Status status =
-                    collection->insertDocument(opCtx, InsertStatement(doc), nullOpDebug, true);
+                Status status = collection_internal::insertDocument(
+                    opCtx, collection, InsertStatement(doc), nullptr /* OpDebug */, true);
                 if (!status.isOK() && status.code() != ErrorCodes::DuplicateKey) {
                     LOGV2_ERROR(20424,
                                 "error: exception cloning object",
@@ -228,46 +232,38 @@ struct Cloner::Fun {
     time_t saveLast;
 };
 
-/* copy the specified collection
+/**
+ * Copy the specified collection.
  */
 void Cloner::_copy(OperationContext* opCtx,
                    const std::string& toDBName,
                    const NamespaceString& nss,
                    const BSONObj& from_opts,
                    const BSONObj& from_id_index,
-                   Query query,
                    DBClientBase* conn) {
     LOGV2_DEBUG(20414,
                 2,
-                "\t\tcloning collection with filter",
-                "ns"_attr = nss,
-                "conn_getServerAddress"_attr = conn->getServerAddress(),
-                "query"_attr = redact(query.toString()));
+                "\t\tcloning collection",
+                logAttrs(nss),
+                "conn_getServerAddress"_attr = conn->getServerAddress());
 
-    Fun f(opCtx, toDBName);
-    f.numSeen = 0;
-    f.nss = nss;
-    f.from_options = from_opts;
-    f.from_id_index = from_id_index;
-    f.saveLast = time(nullptr);
+    BatchHandler batchHandler{opCtx, toDBName};
+    batchHandler.numSeen = 0;
+    batchHandler.nss = nss;
+    batchHandler.from_options = from_opts;
+    batchHandler.from_id_index = from_id_index;
+    batchHandler.saveLast = time(nullptr);
 
-    int options = QueryOption_NoCursorTimeout | QueryOption_Exhaust;
-    {
-        Lock::TempRelease tempRelease(opCtx->lockState());
-        conn->query(std::function<void(DBClientCursorBatchIterator&)>(f),
-                    nss,
-                    query,
-                    nullptr,
-                    options,
-                    0 /* batchSize */,
-                    repl::ReadConcernArgs::kImplicitDefault);
+    FindCommandRequest findCmd{nss};
+    findCmd.setNoCursorTimeout(true);
+    findCmd.setReadConcern(repl::ReadConcernArgs::kLocal);
+    auto cursor = conn->find(
+        std::move(findCmd), ReadPreferenceSetting{ReadPreference::PrimaryOnly}, ExhaustMode::kOn);
+
+    // Process the results of the cursor in batches.
+    while (cursor->more()) {
+        batchHandler(*cursor);
     }
-
-    uassert(ErrorCodes::PrimarySteppedDown,
-            str::stream() << "Not primary while cloning collection " << nss.ns() << " with filter "
-                          << query.toString(),
-            !opCtx->writesAreReplicated() ||
-                repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, nss));
 }
 
 void Cloner::_copyIndexes(OperationContext* opCtx,
@@ -279,7 +275,7 @@ void Cloner::_copyIndexes(OperationContext* opCtx,
     LOGV2_DEBUG(20415,
                 2,
                 "\t\t copyIndexes",
-                "ns"_attr = nss,
+                logAttrs(nss),
                 "conn_getServerAddress"_attr = conn->getServerAddress());
 
     uassert(ErrorCodes::PrimarySteppedDown,
@@ -290,22 +286,21 @@ void Cloner::_copyIndexes(OperationContext* opCtx,
     if (from_indexes.empty())
         return;
 
-    auto collection = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
+    CollectionWriter collection(opCtx, nss);
     invariant(collection, str::stream() << "Missing collection " << nss << " (Cloner)");
 
     auto indexCatalog = collection->getIndexCatalog();
     auto indexesToBuild = indexCatalog->removeExistingIndexesNoChecks(
-        opCtx, {std::begin(from_indexes), std::end(from_indexes)});
+        opCtx, collection.get(), {std::begin(from_indexes), std::end(from_indexes)});
     if (indexesToBuild.empty()) {
         return;
     }
 
-    auto collUUID = collection->uuid();
     auto fromMigrate = false;
     writeConflictRetry(opCtx, "_copyIndexes", nss.ns(), [&] {
         WriteUnitOfWork wunit(opCtx);
         IndexBuildsCoordinator::get(opCtx)->createIndexesOnEmptyCollection(
-            opCtx, collUUID, indexesToBuild, fromMigrate);
+            opCtx, collection, indexesToBuild, fromMigrate);
         wunit.commit();
     });
 }
@@ -332,9 +327,8 @@ StatusWith<std::vector<BSONObj>> Cloner::_filterCollectionsForClone(
         }
 
         const NamespaceString ns(fromDBName, collectionName.c_str());
-
         if (ns.isSystem()) {
-            if (!ns.isLegalClientSystemNS()) {
+            if (!ns.isLegalClientSystemNS(serverGlobalParams.featureCompatibility)) {
                 LOGV2_DEBUG(20419, 2, "\t\t not cloning because system collection");
                 continue;
             }
@@ -350,9 +344,11 @@ Status Cloner::_createCollectionsForDb(
     const std::vector<CreateCollectionParams>& createCollectionParams,
     const std::string& dbName) {
     auto databaseHolder = DatabaseHolder::get(opCtx);
-    auto db = databaseHolder->openDb(opCtx, dbName);
-    invariant(opCtx->lockState()->isDbLockedForMode(dbName, MODE_X));
+    const DatabaseName tenantDbName(boost::none, dbName);
+    auto db = databaseHolder->openDb(opCtx, tenantDbName);
+    invariant(opCtx->lockState()->isDbLockedForMode(tenantDbName, MODE_X));
 
+    auto catalog = CollectionCatalog::get(opCtx);
     auto collCount = 0;
     for (auto&& params : createCollectionParams) {
         if (MONGO_unlikely(movePrimaryFailPoint.shouldFail()) && collCount > 0) {
@@ -365,13 +361,12 @@ Status Cloner::_createCollectionsForDb(
 
         const NamespaceString nss(dbName, params.collectionName);
 
-        uassertStatusOK(userAllowedCreateNS(dbName, params.collectionName));
+        uassertStatusOK(userAllowedCreateNS(opCtx, nss));
         Status status = writeConflictRetry(opCtx, "createCollection", nss.ns(), [&] {
             opCtx->checkForInterrupt();
             WriteUnitOfWork wunit(opCtx);
 
-            Collection* collection =
-                CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
+            CollectionPtr collection = catalog->lookupCollectionByNamespace(opCtx, nss);
             if (collection) {
                 if (!params.shardedColl) {
                     // If the collection is unsharded then we want to fail when a collection
@@ -385,8 +380,7 @@ Status Cloner::_createCollectionsForDb(
                 // exists on the target, we check if the existing collection's UUID matches
                 // that of the one we're trying to create. If it does, we treat the create
                 // as a no-op; if it doesn't match, we return an error.
-                auto existingOpts = DurableCatalog::get(opCtx)->getCollectionOptions(
-                    opCtx, collection->getCatalogId());
+                const auto& existingOpts = collection->getCollectionOptions();
                 const UUID clonedUUID =
                     uassertStatusOK(UUID::parse(params.collectionInfo["info"]["uuid"]));
 
@@ -415,10 +409,15 @@ Status Cloner::_createCollectionsForDb(
 
             CollectionOptions collectionOptions = uassertStatusOK(
                 CollectionOptions::parse(options, CollectionOptions::ParseKind::parseForStorage));
-            Status createStatus = db->userCreateNS(
-                opCtx, nss, collectionOptions, createDefaultIndexes, params.idIndexSpec);
-            if (!createStatus.isOK()) {
-                return createStatus;
+
+            {
+                OperationShardingState::ScopedAllowImplicitCollectionCreate_UNSAFE
+                    unsafeCreateCollection(opCtx);
+                Status createStatus = db->userCreateNS(
+                    opCtx, nss, collectionOptions, createDefaultIndexes, params.idIndexSpec);
+                if (!createStatus.isOK()) {
+                    return createStatus;
+                }
             }
 
             wunit.commit();
@@ -439,7 +438,10 @@ Status Cloner::copyDb(OperationContext* opCtx,
                       const std::string& masterHost,
                       const std::vector<NamespaceString>& shardedColls,
                       std::set<std::string>* clonedColls) {
-    invariant(clonedColls, str::stream() << masterHost << ":" << dBName);
+    invariant(clonedColls && clonedColls->empty(), str::stream() << masterHost << ":" << dBName);
+    // This function can potentially block for a long time on network activity, so holding of locks
+    // is disallowed.
+    invariant(!opCtx->lockState()->isLocked());
 
     auto statusWithMasterHost = ConnectionString::parse(masterHost);
     if (!statusWithMasterHost.isOK()) {
@@ -465,11 +467,11 @@ Status Cloner::copyDb(OperationContext* opCtx,
     }
 
     // Set up connection.
-    std::string errmsg;
-    std::unique_ptr<DBClientBase> conn(cs.connect(StringData(), errmsg));
-    if (!conn.get()) {
-        return Status(ErrorCodes::HostUnreachable, errmsg);
+    auto swConn = cs.connect(StringData());
+    if (!swConn.isOK()) {
+        return swConn.getStatus();
     }
+    auto& conn = swConn.getValue();
 
     if (auth::isInternalAuthSet()) {
         auto authStatus = conn->authenticateInternalUser();
@@ -479,23 +481,14 @@ Status Cloner::copyDb(OperationContext* opCtx,
     }
 
     // Gather the list of collections to clone
-    std::vector<BSONObj> toClone;
-    clonedColls->clear();
+    std::list<BSONObj> initialCollections =
+        conn->getCollectionInfos(dBName, ListCollectionsFilter::makeTypeCollectionFilter());
 
-    {
-        // getCollectionInfos may make a remote call, which may block indefinitely, so release
-        // the global lock that we are entering with.
-        Lock::TempRelease tempRelease(opCtx->lockState());
-
-        std::list<BSONObj> initialCollections =
-            conn->getCollectionInfos(dBName, ListCollectionsFilter::makeTypeCollectionFilter());
-
-        auto status = _filterCollectionsForClone(dBName, initialCollections);
-        if (!status.isOK()) {
-            return status.getStatus();
-        }
-        toClone = status.getValue();
+    auto statusWithCollections = _filterCollectionsForClone(dBName, initialCollections);
+    if (!statusWithCollections.isOK()) {
+        return statusWithCollections.getStatus();
     }
+    auto toClone = statusWithCollections.getValue();
 
     std::vector<CreateCollectionParams> createCollectionParams;
     for (auto&& collection : toClone) {
@@ -515,54 +508,54 @@ Status Cloner::copyDb(OperationContext* opCtx,
 
     // Get index specs for each collection.
     std::map<StringData, std::list<BSONObj>> collectionIndexSpecs;
-    {
-        Lock::TempRelease tempRelease(opCtx->lockState());
-        for (auto&& params : createCollectionParams) {
-            const NamespaceString nss(dBName, params.collectionName);
-            const bool includeBuildUUIDs = false;
-            const int options = 0;
-            auto indexSpecs = conn->getIndexSpecs(nss, includeBuildUUIDs, options);
+    for (auto&& params : createCollectionParams) {
+        const NamespaceString nss(dBName, params.collectionName);
+        const bool includeBuildUUIDs = false;
+        const int options = 0;
+        auto indexSpecs = conn->getIndexSpecs(nss, includeBuildUUIDs, options);
 
-            collectionIndexSpecs[params.collectionName] = indexSpecs;
+        collectionIndexSpecs[params.collectionName] = indexSpecs;
 
-            if (params.idIndexSpec.isEmpty()) {
-                params.idIndexSpec = _getIdIndexSpec(indexSpecs);
-            }
+        if (params.idIndexSpec.isEmpty()) {
+            params.idIndexSpec = _getIdIndexSpec(indexSpecs);
         }
     }
 
-    uassert(
-        ErrorCodes::NotWritablePrimary,
-        str::stream() << "Not primary while cloning database " << dBName
-                      << " (after getting list of collections to clone)",
-        !opCtx->writesAreReplicated() ||
-            repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesForDatabase(opCtx, dBName));
+    {
+        // TODO SERVER-63111 Once the cloner takes in a DatabaseName obj, use dBName directly
+        DatabaseName dbName(boost::none, dBName);
+        Lock::DBLock dbXLock(opCtx, dbName, MODE_X);
+        uassert(ErrorCodes::NotWritablePrimary,
+                str::stream() << "Not primary while cloning database " << dBName
+                              << " (after getting list of collections to clone)",
+                !opCtx->writesAreReplicated() ||
+                    repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesForDatabase(opCtx,
+                                                                                         dBName));
 
-    auto status = _createCollectionsForDb(opCtx, createCollectionParams, dBName);
-    if (!status.isOK()) {
-        return status;
+        auto status = _createCollectionsForDb(opCtx, createCollectionParams, dBName);
+        if (!status.isOK()) {
+            return status;
+        }
+
+        // now build the secondary indexes
+        for (auto&& params : createCollectionParams) {
+            LOGV2(20422,
+                  "copying indexes for: {collectionInfo}",
+                  "Copying indexes",
+                  "collectionInfo"_attr = params.collectionInfo);
+
+            const NamespaceString nss(dBName, params.collectionName);
+
+
+            _copyIndexes(opCtx,
+                         dBName,
+                         nss,
+                         params.collectionInfo["options"].Obj(),
+                         collectionIndexSpecs[params.collectionName],
+                         conn.get());
+        }
     }
 
-    // now build the secondary indexes
-    for (auto&& params : createCollectionParams) {
-        LOGV2(20422,
-              "copying indexes for: {collectionInfo}",
-              "Copying indexes",
-              "collectionInfo"_attr = params.collectionInfo);
-
-        const NamespaceString nss(dBName, params.collectionName);
-
-
-        _copyIndexes(opCtx,
-                     dBName,
-                     nss,
-                     params.collectionInfo["options"].Obj(),
-                     collectionIndexSpecs[params.collectionName],
-                     conn.get());
-    }
-
-    movePrimaryClonerHangBeforeStartCloneDocuments.pauseWhileSet();
-
     for (auto&& params : createCollectionParams) {
         if (params.shardedColl) {
             continue;
@@ -577,14 +570,13 @@ Status Cloner::copyDb(OperationContext* opCtx,
 
         clonedColls->insert(nss.ns());
 
-        LOGV2_DEBUG(20421, 1, "\t\t cloning", "ns"_attr = nss, "host"_attr = masterHost);
+        LOGV2_DEBUG(20421, 1, "\t\t cloning", logAttrs(nss), "host"_attr = masterHost);
 
         _copy(opCtx,
               dBName,
               nss,
               params.collectionInfo["options"].Obj(),
               params.idIndexSpec,
-              Query(),
               conn.get());
     }
 
