@@ -27,9 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
-
-#include "mongo/platform/basic.h"
 
 #include "mongo/db/commands.h"
 
@@ -47,39 +44,40 @@
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/auth/privilege.h"
 #include "mongo/db/client.h"
-#include "mongo/db/command_generic_argument.h"
 #include "mongo/db/commands/test_commands_enabled.h"
-#include "mongo/db/commands_in_multi_doc_txn_params_gen.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/error_labels.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/read_write_concern_defaults.h"
+#include "mongo/idl/basic_types_gen.h"
+#include "mongo/idl/command_generic_argument.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/factory.h"
 #include "mongo/rpc/metadata/client_metadata.h"
 #include "mongo/rpc/op_msg_rpc_impls.h"
-#include "mongo/rpc/protocol.h"
 #include "mongo/rpc/rewrite_state_change_errors.h"
 #include "mongo/rpc/write_concern_error_detail.h"
 #include "mongo/s/stale_exception.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/fail_point.h"
-#include "mongo/util/invariant.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
+
+
+using namespace fmt::literals;
+
 namespace mongo {
 
 using logv2::LogComponent;
+const std::set<std::string> kNoApiVersions = {};
+const std::set<std::string> kApiVersions1 = {"1"};
 
 namespace {
 
 const char kWriteConcernField[] = "writeConcern";
-const WriteConcernOptions kMajorityWriteConcern(
-    WriteConcernOptions::kMajority,
-    // Note: Even though we're setting UNSET here, kMajority implies JOURNAL if journaling is
-    // supported by the mongod.
-    WriteConcernOptions::SyncMode::UNSET,
-    WriteConcernOptions::kWriteConcernTimeoutUserCommand);
 
 // Returns true if found to be authorized, false if undecided. Throws if unauthorized.
 bool checkAuthorizationImplPreParse(OperationContext* opCtx,
@@ -107,47 +105,32 @@ bool checkAuthorizationImplPreParse(OperationContext* opCtx,
         return false;  // Still can't decide on auth because of the localhost bypass.
     uassert(ErrorCodes::Unauthorized,
             str::stream() << "command " << command->getName() << " requires authentication",
-            !command->requiresAuth() || authzSession->isAuthenticated());
+            !command->requiresAuth() || authzSession->isAuthenticated() ||
+                (request.validatedTenancyScope &&
+                 request.validatedTenancyScope->hasAuthenticatedUser()));
     return false;
 }
 
-// The command names that are allowed in a multi-document transaction.
-const StringMap<int> txnCmdWhitelistFCV44 = {{"create", 1}, {"createIndexes", 1}};
-const StringMap<int> txnCmdWhitelist = {{"abortTransaction", 1},
-                                        {"aggregate", 1},
-                                        {"commitTransaction", 1},
-                                        {"coordinateCommitTransaction", 1},
-                                        {"delete", 1},
-                                        {"distinct", 1},
-                                        {"find", 1},
-                                        {"findandmodify", 1},
-                                        {"findAndModify", 1},
-                                        {"geoSearch", 1},
-                                        {"getMore", 1},
-                                        {"insert", 1},
-                                        {"killCursors", 1},
-                                        {"prepareTransaction", 1},
-                                        {"update", 1}};
-
-// The commands that can be run on the 'admin' database in multi-document transactions.
-const StringMap<int> txnAdminCommands = {{"abortTransaction", 1},
-                                         {"commitTransaction", 1},
-                                         {"coordinateCommitTransaction", 1},
-                                         {"prepareTransaction", 1}};
-
-auto getCommandInvocationHooksHandle =
-    ServiceContext::declareDecoration<std::shared_ptr<CommandInvocationHooks>>();
+auto getCommandInvocationHooks =
+    ServiceContext::declareDecoration<std::unique_ptr<CommandInvocationHooks>>();
 
 }  // namespace
 
 void CommandInvocationHooks::set(ServiceContext* serviceContext,
-                                 std::shared_ptr<CommandInvocationHooks> hooks) {
-    getCommandInvocationHooksHandle(serviceContext) = std::move(hooks);
+                                 std::unique_ptr<CommandInvocationHooks> hooks) {
+    getCommandInvocationHooks(serviceContext) = std::move(hooks);
 }
 
 //////////////////////////////////////////////////////////////
 // CommandHelpers
 
+const WriteConcernOptions CommandHelpers::kMajorityWriteConcern(
+    WriteConcernOptions::kMajority,
+    // Note: Even though we're setting UNSET here, kMajority implies JOURNAL if journaling is
+    // supported by the mongod.
+    WriteConcernOptions::SyncMode::UNSET,
+    WriteConcernOptions::kWriteConcernTimeoutUserCommand);
+
 BSONObj CommandHelpers::runCommandDirectly(OperationContext* opCtx, const OpMsgRequest& request) {
     auto command = globalCommandRegistry()->findCommand(request.getCommandName());
     invariant(command);
@@ -172,11 +155,24 @@ BSONObj CommandHelpers::runCommandDirectly(OperationContext* opCtx, const OpMsgR
     return replyBuilder.releaseBody();
 }
 
+Future<void> CommandHelpers::runCommandInvocation(std::shared_ptr<RequestExecutionContext> rec,
+                                                  std::shared_ptr<CommandInvocation> invocation,
+                                                  bool useDedicatedThread) {
+    if (useDedicatedThread)
+        return makeReadyFutureWith([opCtx = rec->getOpCtx(),
+                                    request = rec->getRequest(),
+                                    invocation = invocation.get(),
+                                    replyBuilder = rec->getReplyBuilder()] {
+            runCommandInvocation(opCtx, request, invocation, replyBuilder);
+        });
+    return runCommandInvocationAsync(std::move(rec), std::move(invocation));
+}
+
 void CommandHelpers::runCommandInvocation(OperationContext* opCtx,
                                           const OpMsgRequest& request,
                                           CommandInvocation* invocation,
                                           rpc::ReplyBuilderInterface* response) {
-    auto hooks = getCommandInvocationHooksHandle(opCtx->getServiceContext());
+    auto&& hooks = getCommandInvocationHooks(opCtx->getServiceContext());
 
     if (hooks) {
         hooks->onBeforeRun(opCtx, request, invocation);
@@ -189,14 +185,36 @@ void CommandHelpers::runCommandInvocation(OperationContext* opCtx,
     }
 }
 
+Future<void> CommandHelpers::runCommandInvocationAsync(
+    std::shared_ptr<RequestExecutionContext> rec,
+    std::shared_ptr<CommandInvocation> invocation) try {
+    auto&& hooks = getCommandInvocationHooks(rec->getOpCtx()->getServiceContext());
+    if (hooks)
+        hooks->onBeforeAsyncRun(rec, invocation.get());
+    return invocation->runAsync(rec).then([rec, hooks = hooks.get(), invocation] {
+        if (hooks)
+            hooks->onAfterAsyncRun(rec, invocation.get());
+    });
+} catch (const DBException& e) {
+    return e.toStatus();
+}
+
 void CommandHelpers::auditLogAuthEvent(OperationContext* opCtx,
                                        const CommandInvocation* invocation,
                                        const OpMsgRequest& request,
                                        ErrorCodes::Error err) {
     class Hook final : public audit::CommandInterface {
     public:
-        explicit Hook(const CommandInvocation* invocation, const NamespaceString* nss)
-            : _invocation(invocation), _nss(nss) {}
+        Hook(const CommandInvocation* invocation, const OpMsgRequest& request)
+            : _invocation(invocation) {
+            if (_invocation) {
+                _nss = _invocation->ns();
+                _name = _invocation->definition()->getName();
+            } else {
+                _nss = NamespaceString(request.getDatabase());
+                _name = request.getCommandName().toString();
+            }
+        }
 
         void snipForLogging(mutablebson::Document* cmdObj) const override {
             if (_invocation) {
@@ -204,22 +222,19 @@ void CommandHelpers::auditLogAuthEvent(OperationContext* opCtx,
             }
         }
 
-        StringData sensitiveFieldName() const override {
+        std::set<StringData> sensitiveFieldNames() const override {
             if (_invocation) {
-                return _invocation->definition()->sensitiveFieldName();
+                return _invocation->definition()->sensitiveFieldNames();
             }
-            return StringData{};
+            return {};
         }
 
         StringData getName() const override {
-            if (!_invocation) {
-                return "Error"_sd;
-            }
-            return _invocation->definition()->getName();
+            return _name;
         }
 
         NamespaceString ns() const override {
-            return *_nss;
+            return _nss;
         }
 
         bool redactArgs() const override {
@@ -228,11 +243,10 @@ void CommandHelpers::auditLogAuthEvent(OperationContext* opCtx,
 
     private:
         const CommandInvocation* _invocation;
-        const NamespaceString* _nss;
+        NamespaceString _nss;
+        std::string _name;
     };
 
-    NamespaceString nss = invocation ? invocation->ns() : NamespaceString(request.getDatabase());
-
     // Always audit errors other than Unauthorized.
     //
     // When we get Unauthorized (usually),
@@ -240,7 +254,7 @@ void CommandHelpers::auditLogAuthEvent(OperationContext* opCtx,
     // or if we don't know our Command definition.
     if ((err != ErrorCodes::Unauthorized) || !invocation ||
         invocation->definition()->auditAuthorizationFailure()) {
-        audit::logCommandAuthzCheck(opCtx->getClient(), request, Hook(invocation, &nss), err);
+        audit::logCommandAuthzCheck(opCtx->getClient(), request, Hook(invocation, request), err);
     }
 }
 
@@ -266,6 +280,11 @@ std::string CommandHelpers::parseNsFullyQualified(const BSONObj& cmdObj) {
 
 NamespaceString CommandHelpers::parseNsCollectionRequired(StringData dbname,
                                                           const BSONObj& cmdObj) {
+    return parseNsCollectionRequired({boost::none, dbname}, cmdObj);
+}
+
+NamespaceString CommandHelpers::parseNsCollectionRequired(const DatabaseName& dbName,
+                                                          const BSONObj& cmdObj) {
     // Accepts both BSON String and Symbol for collection name per SERVER-16260
     // TODO(kangas) remove Symbol support in MongoDB 3.0 after Ruby driver audit
     BSONElement first = cmdObj.firstElement();
@@ -278,20 +297,21 @@ NamespaceString CommandHelpers::parseNsCollectionRequired(StringData dbname,
     uassert(ErrorCodes::InvalidNamespace,
             str::stream() << "collection name has invalid type " << typeName(first.type()),
             first.canonicalType() == canonicalizeBSONType(mongo::String));
-    const NamespaceString nss(dbname, first.valueStringData());
+    const NamespaceString nss(dbName, first.valueStringData());
     uassert(ErrorCodes::InvalidNamespace,
             str::stream() << "Invalid namespace specified '" << nss.ns() << "'",
             nss.isValid());
     return nss;
 }
 
-NamespaceStringOrUUID CommandHelpers::parseNsOrUUID(StringData dbname, const BSONObj& cmdObj) {
+NamespaceStringOrUUID CommandHelpers::parseNsOrUUID(const DatabaseName& dbName,
+                                                    const BSONObj& cmdObj) {
     BSONElement first = cmdObj.firstElement();
     if (first.type() == BinData && first.binDataType() == BinDataType::newUUID) {
-        return {dbname.toString(), uassertStatusOK(UUID::parse(first))};
+        return {dbName, uassertStatusOK(UUID::parse(first))};
     } else {
         // Ensure collection identifier is not a Command
-        const NamespaceString nss(parseNsCollectionRequired(dbname, cmdObj));
+        const NamespaceString nss(parseNsCollectionRequired(dbName, cmdObj));
         uassert(ErrorCodes::InvalidNamespace,
                 str::stream() << "Invalid collection name specified '" << nss.ns(),
                 !(nss.ns().find('$') != std::string::npos && nss.ns() != "local.oplog.$main"));
@@ -300,10 +320,15 @@ NamespaceStringOrUUID CommandHelpers::parseNsOrUUID(StringData dbname, const BSO
 }
 
 std::string CommandHelpers::parseNsFromCommand(StringData dbname, const BSONObj& cmdObj) {
+    return parseNsFromCommand({boost::none, dbname}, cmdObj).ns();
+}
+
+NamespaceString CommandHelpers::parseNsFromCommand(const DatabaseName& dbName,
+                                                   const BSONObj& cmdObj) {
     BSONElement first = cmdObj.firstElement();
     if (first.type() != mongo::String)
-        return dbname.toString();
-    return str::stream() << dbname << '.' << cmdObj.firstElement().valueStringData();
+        return NamespaceString(dbName);
+    return NamespaceString(dbName, cmdObj.firstElement().valueStringData());
 }
 
 ResourcePattern CommandHelpers::resourcePatternForNamespace(const std::string& ns) {
@@ -327,6 +352,21 @@ bool CommandHelpers::appendCommandStatusNoThrow(BSONObjBuilder& result, const St
     if (auto extraInfo = status.extraInfo()) {
         extraInfo->serialize(&result);
     }
+    // If the command has errored, assert that it satisfies the IDL-defined requirements on a
+    // command error reply.
+    // Only validate error reply in test mode so that we don't expose users to errors if we
+    // construct an invalid error reply.
+    if (!status.isOK() && getTestCommandsEnabled()) {
+        try {
+            ErrorReply::parse(IDLParserContext("appendCommandStatusNoThrow"), result.asTempObj());
+        } catch (const DBException&) {
+            invariant(false,
+                      "invalid error-response to a command constructed in "
+                      "CommandHelpers::appendComandStatusNoThrow. All erroring command responses "
+                      "must comply with the format specified by the IDL-defined struct ErrorReply, "
+                      "defined in idl/basic_types.idl");
+        }
+    }
     return status.isOK();
 }
 
@@ -372,11 +412,11 @@ void CommandHelpers::appendCommandWCStatus(BSONObjBuilder& result,
     }
 }
 
-BSONObj CommandHelpers::appendPassthroughFields(const BSONObj& cmdObjWithPassthroughFields,
-                                                const BSONObj& request) {
+BSONObj CommandHelpers::appendGenericCommandArgs(const BSONObj& cmdObjWithGenericArgs,
+                                                 const BSONObj& request) {
     BSONObjBuilder b;
     b.appendElements(request);
-    for (const auto& elem : filterCommandRequestForPassthrough(cmdObjWithPassthroughFields)) {
+    for (const auto& elem : filterCommandRequestForPassthrough(cmdObjWithGenericArgs)) {
         const auto name = elem.fieldNameStringData();
         if (isGenericArgument(name) && !request.hasField(name)) {
             b.append(elem);
@@ -385,29 +425,43 @@ BSONObj CommandHelpers::appendPassthroughFields(const BSONObj& cmdObjWithPassthr
     return b.obj();
 }
 
+void CommandHelpers::appendGenericReplyFields(const BSONObj& replyObjWithGenericReplyFields,
+                                              const BSONObj& reply,
+                                              BSONObjBuilder* replyBuilder) {
+    replyBuilder->appendElements(reply);
+    for (const auto& elem : filterCommandReplyForPassthrough(replyObjWithGenericReplyFields)) {
+        const auto name = elem.fieldNameStringData();
+        if (isGenericArgument(name) && !reply.hasField(name)) {
+            replyBuilder->append(elem);
+        }
+    }
+}
+
+BSONObj CommandHelpers::appendGenericReplyFields(const BSONObj& replyObjWithGenericReplyFields,
+                                                 const BSONObj& reply) {
+    BSONObjBuilder b;
+    appendGenericReplyFields(replyObjWithGenericReplyFields, reply, &b);
+    return b.obj();
+}
+
 BSONObj CommandHelpers::appendMajorityWriteConcern(const BSONObj& cmdObj,
                                                    WriteConcernOptions defaultWC) {
     WriteConcernOptions newWC = kMajorityWriteConcern;
-
     if (cmdObj.hasField(kWriteConcernField)) {
-        auto wc = cmdObj.getField(kWriteConcernField);
-        // The command has a writeConcern field and it's majority, so we can
-        // return it as-is.
-        if (wc["w"].ok() && wc["w"].str() == "majority") {
+        auto wc = uassertStatusOK(WriteConcernOptions::extractWCFromCommand(cmdObj));
+
+        // The command has a writeConcern field and it's majority, so we can return it as-is.
+        if (wc.isMajority()) {
             return cmdObj;
         }
 
-        if (wc["wtimeout"].ok()) {
-            // They set a timeout, but aren't using majority WC. We want to use their
-            // timeout along with majority WC.
-            newWC = WriteConcernOptions(WriteConcernOptions::kMajority,
-                                        WriteConcernOptions::SyncMode::UNSET,
-                                        wc["wtimeout"].Number());
-        }
-    } else if (!defaultWC.usedDefault) {
+        newWC = WriteConcernOptions{
+            WriteConcernOptions::kMajority, WriteConcernOptions::SyncMode::UNSET, wc.wTimeout};
+    } else if (!defaultWC.usedDefaultConstructedWC) {
         auto minimumAcceptableWTimeout = newWC.wTimeout;
         newWC = defaultWC;
-        newWC.wMode = "majority";
+        newWC.w = "majority";
+
         if (defaultWC.wTimeout < minimumAcceptableWTimeout) {
             newWC.wTimeout = minimumAcceptableWTimeout;
         }
@@ -443,7 +497,7 @@ void CommandHelpers::filterCommandRequestForPassthrough(BSONObjIterator* cmdIter
             BSONObjBuilder(requestBuilder->subobjStart("$queryOptions")).append(elem);
             continue;
         }
-        if (isRequestStripArgument(name))
+        if (!shouldForwardToShards(name))
             continue;
         requestBuilder->append(elem);
     }
@@ -453,7 +507,7 @@ void CommandHelpers::filterCommandReplyForPassthrough(const BSONObj& cmdObj,
                                                       BSONObjBuilder* output) {
     for (auto elem : cmdObj) {
         const auto name = elem.fieldNameStringData();
-        if (isReplyStripArgument(name))
+        if (!shouldForwardFromShards(name))
             continue;
         output->append(elem);
     }
@@ -482,6 +536,13 @@ bool CommandHelpers::uassertShouldAttemptParse(OperationContext* opCtx,
     }
 }
 
+void CommandHelpers::uassertCommandRunWithMajority(StringData commandName,
+                                                   const WriteConcernOptions& writeConcern) {
+    uassert(ErrorCodes::InvalidOptions,
+            "\"{}\" must be called with majority writeConcern, got: {} "_format(
+                commandName, writeConcern.toBSON().toString()),
+            writeConcern.isMajority());
+}
 
 void CommandHelpers::canUseTransactions(const NamespaceString& nss,
                                         StringData cmdName,
@@ -492,38 +553,22 @@ void CommandHelpers::canUseTransactions(const NamespaceString& nss,
             "http://dochub.mongodb.org/core/transaction-count for a recommended alternative.",
             cmdName != "count"_sd);
 
-    auto inTxnWhitelist = txnCmdWhitelist.find(cmdName) != txnCmdWhitelist.cend();
-    auto inTxnWhitelistFCV44 = txnCmdWhitelistFCV44.find(cmdName) != txnCmdWhitelistFCV44.cend();
-    auto isFullyUpgradedTo44 =
-        (serverGlobalParams.featureCompatibility.isVersionInitialized() &&
-         serverGlobalParams.featureCompatibility.getVersion() ==
-             ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44);
-
-    uassert(ErrorCodes::OperationNotSupportedInTransaction,
-            str::stream() << "Cannot run '" << cmdName
-                          << "' in a multi-document transaction unless the "
-                             "featureCompatibilityVersion is equal to 4.4.",
-            isFullyUpgradedTo44 || !inTxnWhitelistFCV44);
+    auto command = findCommand(cmdName);
+    uassert(ErrorCodes::CommandNotFound,
+            str::stream() << "Encountered unknown command during check if can run in transactions: "
+                          << cmdName,
+            command);
 
     uassert(ErrorCodes::OperationNotSupportedInTransaction,
             str::stream() << "Cannot run '" << cmdName << "' in a multi-document transaction.",
-            inTxnWhitelist || inTxnWhitelistFCV44);
-
-    uassert(ErrorCodes::OperationNotSupportedInTransaction,
-            str::stream() << "Cannot run '" << cmdName
-                          << "' in a multi-document transaction "
-                             "because creation of collections and "
-                             "indexes inside multi-document transactions is disabled.",
-            gShouldMultiDocTxnCreateCollectionAndIndexes.load() || !inTxnWhitelistFCV44);
+            command->allowedInTransactions());
 
     const auto dbName = nss.db();
 
     uassert(ErrorCodes::OperationNotSupportedInTransaction,
             str::stream() << "Cannot run command against the '" << dbName
                           << "' database in a transaction.",
-            dbName != NamespaceString::kLocalDb &&
-                (dbName != NamespaceString::kAdminDb ||
-                 txnAdminCommands.find(cmdName) != txnAdminCommands.cend()));
+            dbName != NamespaceString::kLocalDb);
 
     uassert(ErrorCodes::OperationNotSupportedInTransaction,
             str::stream() << "Cannot run command against the '" << nss
@@ -572,7 +617,8 @@ bool CommandHelpers::shouldActivateFailCommandFailPoint(const BSONObj& data,
     if (cmd->getName() == "configureFailPoint"_sd)  // Banned even if in failCommands.
         return false;
 
-    if (!client->session()) {
+    if (!(data.hasField("failLocalClients") && data.getBoolField("failLocalClients")) &&
+        !client->session()) {
         return false;
     }
 
@@ -581,7 +627,8 @@ bool CommandHelpers::shouldActivateFailCommandFailPoint(const BSONObj& data,
     if (auto clientMetadata = ClientMetadata::get(client)) {
         appName = clientMetadata->getApplicationName();
     }
-    auto isInternalClient = client->session()->getTags() & transport::Session::kInternalClient;
+    auto isInternalClient =
+        !client->session() || (client->session()->getTags() & transport::Session::kInternalClient);
 
     if (data.hasField("threadName") && (threadName != data.getStringField("threadName"))) {
         return false;  // only activate failpoint on thread from certain client
@@ -600,6 +647,18 @@ bool CommandHelpers::shouldActivateFailCommandFailPoint(const BSONObj& data,
         return false;
     }
 
+    if (data.hasField("failAllCommands")) {
+        LOGV2(6348500,
+              "Activating 'failCommand' failpoint for all commands",
+              "data"_attr = data,
+              "threadName"_attr = threadName,
+              "appName"_attr = appName,
+              "namespace"_attr = nss,
+              "isInternalClient"_attr = isInternalClient,
+              "command"_attr = cmd->getName());
+        return true;
+    }
+
     for (auto&& failCommand : data.getObjectField("failCommands")) {
         if (failCommand.type() == String && cmd->hasAlias(failCommand.valueStringData())) {
             LOGV2(4898500,
@@ -627,7 +686,6 @@ void CommandHelpers::evaluateFailCommandFailPoint(OperationContext* opCtx,
      * Default value is used to suppress the uassert for `errorExtraInfo` if `errorCode` is not set.
      */
     long long errorCode = ErrorCodes::OK;
-    bool hasExtraInfo;
     /**
      * errorExtraInfo only holds a reference to the BSONElement of the parent object (data).
      * The copy constructor of BSONObj handles cloning to keep references valid outside the scope.
@@ -636,12 +694,7 @@ void CommandHelpers::evaluateFailCommandFailPoint(OperationContext* opCtx,
     const Command* cmd = invocation->definition();
     failCommand.executeIf(
         [&](const BSONObj& data) {
-            // State change codes are rewritten on the way out of a `mongos`
-            // server. Errors injected via failpoint manipulation are normally
-            // exempt from this. However, we provide an override option so they
-            // can be made subject to rewriting if that's really necessary.
-            if (bool b; !bsonExtractBooleanField(data, "allowRewriteStateChange", &b).isOK() || !b)
-                rpc::RewriteStateChangeErrors::setEnabled(opCtx, false);
+            rpc::RewriteStateChangeErrors::onActiveFailCommand(opCtx, data);
 
             if (data.hasField(kErrorLabelsFieldName) &&
                 data[kErrorLabelsFieldName].type() == Array) {
@@ -652,15 +705,6 @@ void CommandHelpers::evaluateFailCommandFailPoint(OperationContext* opCtx,
                     data.getObjectField(kErrorLabelsFieldName).getOwned());
             }
 
-            if (closeConnection) {
-                opCtx->getClient()->session()->end();
-                LOGV2(20431,
-                      "Failing {command} via 'failCommand' failpoint: closing connection",
-                      "Failing command via 'failCommand' failpoint: closing connection",
-                      "command"_attr = cmd->getName());
-                uasserted(50985, "Failing command due to 'failCommand' failpoint");
-            }
-
             if (blockConnection) {
                 long long blockTimeMS = 0;
                 uassert(ErrorCodes::InvalidOptions,
@@ -683,26 +727,60 @@ void CommandHelpers::evaluateFailCommandFailPoint(OperationContext* opCtx,
                       "command"_attr = cmd->getName());
             }
 
-            if (hasExtraInfo) {
+            auto tassert = [&] {
+                bool b;
+                Status st = bsonExtractBooleanField(data, "tassert", &b);
+                if (st == ErrorCodes::NoSuchKey) {
+                    return false;
+                }
+                uassertStatusOK(st);
+                return b;
+            }();
+
+            auto raise = [&](const Status& status) {
+                if (tassert) {
+                    tassert(status);
+                } else {
+                    uassertStatusOK(status);
+                }
+            };
+
+            static constexpr auto failpointMsg = "Failing command via 'failCommand' failpoint"_sd;
+
+            if (closeConnection) {
+                opCtx->getClient()->session()->end();
+                LOGV2(20431,
+                      "Failing {command} via 'failCommand' failpoint: closing connection",
+                      "command"_attr = cmd->getName());
+                raise(Status(tassert ? ErrorCodes::Error(5704000) : ErrorCodes::Error(50985),
+                             failpointMsg));
+            }
+
+            auto errorExtraInfo = [&]() -> boost::optional<BSONObj> {
+                BSONElement e;
+                Status st = bsonExtractTypedField(data, "errorExtraInfo", BSONType::Object, &e);
+                if (st == ErrorCodes::NoSuchKey)
+                    return {};  // It's optional. Missing is allowed. Other errors aren't.
+                uassertStatusOK(st);
+                return {e.Obj()};
+            }();
+
+            if (errorExtraInfo) {
                 LOGV2(20434,
                       "Failing {command} via 'failCommand' failpoint: returning {errorCode} and "
                       "{errorExtraInfo}",
-                      "Failing command via 'failCommand' failpoint",
                       "command"_attr = cmd->getName(),
                       "errorCode"_attr = errorCode,
                       "errorExtraInfo"_attr = errorExtraInfo);
-                uassertStatusOK(Status(ErrorCodes::Error(errorCode),
-                                       "Failing command due to 'failCommand' failpoint",
-                                       errorExtraInfo.Obj()));
+                raise(Status(ErrorCodes::Error(errorCode), failpointMsg, *errorExtraInfo));
+
             } else if (hasErrorCode) {
                 LOGV2(
                     20435,
                     "Failing command {command} via 'failCommand' failpoint: returning {errorCode}",
-                    "Failing command via 'failCommand' failpoint",
                     "command"_attr = cmd->getName(),
                     "errorCode"_attr = errorCode);
-                uasserted(ErrorCodes::Error(errorCode),
-                          "Failing command due to 'failCommand' failpoint");
+                raise(Status(ErrorCodes::Error(errorCode), failpointMsg));
             }
         },
         [&](const BSONObj& data) {
@@ -711,9 +789,6 @@ void CommandHelpers::evaluateFailCommandFailPoint(OperationContext* opCtx,
                 closeConnection;
             hasErrorCode = data.hasField("errorCode") &&
                 bsonExtractIntegerField(data, "errorCode", &errorCode).isOK();
-            hasExtraInfo = data.hasField("errorExtraInfo") &&
-                bsonExtractTypedField(data, "errorExtraInfo", BSONType::Object, &errorExtraInfo)
-                    .isOK();
             blockConnection = data.hasField("blockConnection") &&
                 bsonExtractBooleanField(data, "blockConnection", &blockConnection).isOK() &&
                 blockConnection;
@@ -806,7 +881,7 @@ public:
         : CommandInvocation(command),
           _command(command),
           _request(request),
-          _dbName(_request.getDatabase().toString()) {}
+          _dbName(_request.getValidatedTenantId(), _request.getDatabase()) {}
 
 private:
     void run(OperationContext* opCtx, rpc::ReplyBuilderInterface* result) override {
@@ -818,6 +893,16 @@ private:
         }
     }
 
+    Future<void> runAsync(std::shared_ptr<RequestExecutionContext> rec) override {
+        return _command->runAsync(rec, _dbName).onError([rec](Status status) {
+            if (status.code() != ErrorCodes::FailedToRunWithReplyBuilder)
+                return status;
+            BSONObjBuilder bob = rec->getReplyBuilder()->getBodyBuilder();
+            CommandHelpers::appendSimpleCommandStatus(bob, false);
+            return Status::OK();
+        });
+    }
+
     void explain(OperationContext* opCtx,
                  ExplainOptions::Verbosity verbosity,
                  rpc::ReplyBuilderInterface* result) override {
@@ -825,15 +910,16 @@ private:
     }
 
     NamespaceString ns() const override {
-        return NamespaceString(_command->parseNs(_dbName, cmdObj()));
+        return _command->parseNs(_dbName, cmdObj());
     }
 
     bool supportsWriteConcern() const override {
         return _command->supportsWriteConcern(cmdObj());
     }
 
-    ReadConcernSupportResult supportsReadConcern(repl::ReadConcernLevel level) const override {
-        return _command->supportsReadConcern(cmdObj(), level);
+    ReadConcernSupportResult supportsReadConcern(repl::ReadConcernLevel level,
+                                                 bool isImplicitDefault) const override {
+        return _command->supportsReadConcern(cmdObj(), level, isImplicitDefault);
     }
 
     bool supportsReadMirroring() const override {
@@ -864,21 +950,21 @@ private:
 
     BasicCommandWithReplyBuilderInterface* const _command;
     const OpMsgRequest _request;
-    const std::string _dbName;
+    const DatabaseName _dbName;
 };
 
 Command::~Command() = default;
 
 void Command::snipForLogging(mutablebson::Document* cmdObj) const {
-    StringData sensitiveField = sensitiveFieldName();
-    if (!sensitiveField.empty()) {
-
-        for (mutablebson::Element pwdElement =
-                 mutablebson::findFirstChildNamed(cmdObj->root(), sensitiveField);
-             pwdElement.ok();
-             pwdElement =
-                 mutablebson::findElementNamed(pwdElement.rightSibling(), sensitiveField)) {
-            uassertStatusOK(pwdElement.setValueString("xxx"));
+    auto sensitiveFields = sensitiveFieldNames();
+    if (!sensitiveFields.empty()) {
+        for (auto& sensitiveField : sensitiveFields) {
+            for (mutablebson::Element element =
+                     mutablebson::findFirstChildNamed(cmdObj->root(), sensitiveField);
+                 element.ok();
+                 element = mutablebson::findElementNamed(element.rightSibling(), sensitiveField)) {
+                uassertStatusOK(element.setValueString("xxx"));
+            }
         }
     }
 }
@@ -893,11 +979,19 @@ std::unique_ptr<CommandInvocation> BasicCommandWithReplyBuilderInterface::parse(
 Command::Command(StringData name, std::vector<StringData> aliases)
     : _name(name.toString()),
       _aliases(std::move(aliases)),
-      _commandsExecutedMetric("commands." + _name + ".total", &_commandsExecuted),
-      _commandsFailedMetric("commands." + _name + ".failed", &_commandsFailed) {
+      _commandsExecuted("commands." + _name + ".total"),
+      _commandsFailed("commands." + _name + ".failed") {
     globalCommandRegistry()->registerCommand(this, _name, _aliases);
 }
 
+const std::set<std::string>& Command::apiVersions() const {
+    return kNoApiVersions;
+}
+
+const std::set<std::string>& Command::deprecatedApiVersions() const {
+    return kNoApiVersions;
+}
+
 bool Command::hasAlias(const StringData& alias) const {
     return globalCommandRegistry()->findCommand(alias) == this;
 }
@@ -935,11 +1029,11 @@ void Command::generateHelpResponse(OperationContext* opCtx,
 }
 
 bool ErrmsgCommandDeprecated::run(OperationContext* opCtx,
-                                  const std::string& db,
+                                  const DatabaseName& dbName,
                                   const BSONObj& cmdObj,
                                   BSONObjBuilder& result) {
     std::string errmsg;
-    auto ok = errmsgRun(opCtx, db, cmdObj, errmsg, result);
+    auto ok = errmsgRun(opCtx, dbName.toStringWithTenantId(), cmdObj, errmsg, result);
     if (!errmsg.empty()) {
         CommandHelpers::appendSimpleCommandStatus(result, ok, errmsg);
     }
