@@ -30,27 +30,32 @@
 #pragma once
 
 #include <boost/optional.hpp>
+#include <fmt/format.h>
 #include <functional>
 #include <string>
 #include <vector>
 
-#include "mongo/base/counter.h"
 #include "mongo/base/init.h"
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
+#include "mongo/db/api_parameters.h"
 #include "mongo/db/auth/privilege.h"
 #include "mongo/db/auth/resource_pattern.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands/server_status_metric.h"
 #include "mongo/db/commands/test_commands_enabled.h"
 #include "mongo/db/jsobj.h"
-#include "mongo/db/query/explain.h"
+#include "mongo/db/query/explain_options.h"
 #include "mongo/db/read_concern_support_result.h"
 #include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/request_execution_context.h"
 #include "mongo/db/write_concern.h"
+#include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/op_msg.h"
 #include "mongo/rpc/reply_builder_interface.h"
+#include "mongo/transport/service_executor.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/future.h"
 #include "mongo/util/string_map.h"
 
 namespace mongo {
@@ -58,7 +63,10 @@ namespace mongo {
 extern FailPoint failCommand;
 extern FailPoint waitInCommandMarkKillOnClientDisconnect;
 extern const OperationContext::Decoration<boost::optional<BSONArray>> errorLabelsOverride;
+extern const std::set<std::string> kNoApiVersions;
+extern const std::set<std::string> kApiVersions1;
 
+class AuthorizationContract;
 class Command;
 class CommandInvocation;
 class OperationContext;
@@ -76,9 +84,9 @@ class Document;
 class CommandInvocationHooks {
 public:
     /**
-     * Set the current hooks
+     * Set `hooks` as the `CommandInvocationHooks` decoration of `serviceContext`
      */
-    static void set(ServiceContext* serviceContext, std::shared_ptr<CommandInvocationHooks> hooks);
+    static void set(ServiceContext* serviceContext, std::unique_ptr<CommandInvocationHooks> hooks);
 
     virtual ~CommandInvocationHooks() = default;
 
@@ -89,27 +97,49 @@ public:
                              const OpMsgRequest& request,
                              CommandInvocation* invocation) = 0;
 
+    /**
+     * A behavior to perform before CommandInvocation::asyncRun(). Defaults to `onBeforeRun(...)`.
+     */
+    virtual void onBeforeAsyncRun(std::shared_ptr<RequestExecutionContext> rec,
+                                  CommandInvocation* invocation) {
+        onBeforeRun(rec->getOpCtx(), rec->getRequest(), invocation);
+    }
+
     /**
      * A behavior to perform after CommandInvocation::run()
      */
     virtual void onAfterRun(OperationContext* opCtx,
                             const OpMsgRequest& request,
                             CommandInvocation* invocation) = 0;
+
+    /**
+     * A behavior to perform after CommandInvocation::asyncRun(). Defaults to `onAfterRun(...)`.
+     */
+    virtual void onAfterAsyncRun(std::shared_ptr<RequestExecutionContext> rec,
+                                 CommandInvocation* invocation) {
+        onAfterRun(rec->getOpCtx(), rec->getRequest(), invocation);
+    }
 };
 
 // Various helpers unrelated to any single command or to the command registry.
 // Would be a namespace, but want to keep it closed rather than open.
 // Some of these may move to the BasicCommand shim if they are only for legacy implementations.
 struct CommandHelpers {
+    static const WriteConcernOptions kMajorityWriteConcern;
+
     // The type of the first field in 'cmdObj' must be mongo::String. The first field is
     // interpreted as a collection name.
     static std::string parseNsFullyQualified(const BSONObj& cmdObj);
 
     // The type of the first field in 'cmdObj' must be mongo::String or Symbol.
     // The first field is interpreted as a collection name.
+    static NamespaceString parseNsCollectionRequired(const DatabaseName& dbName,
+                                                     const BSONObj& cmdObj);
+    // TODO SERVER-68421: Remove this method once all call sites have been updated to pass
+    // DatabaseName.
     static NamespaceString parseNsCollectionRequired(StringData dbname, const BSONObj& cmdObj);
 
-    static NamespaceStringOrUUID parseNsOrUUID(StringData dbname, const BSONObj& cmdObj);
+    static NamespaceStringOrUUID parseNsOrUUID(const DatabaseName& dbName, const BSONObj& cmdObj);
 
     /**
      * Return the namespace for the command. If the first field in 'cmdObj' is of type
@@ -117,8 +147,17 @@ struct CommandHelpers {
      * appended to 'dbname' after a '.' character. If the first field is not of type
      * mongo::String, then 'dbname' is returned unmodified.
      */
+    // TODO SERVER-68423: Remove this method once all call sites have been updated to pass
+    // DatabaseName.
     static std::string parseNsFromCommand(StringData dbname, const BSONObj& cmdObj);
 
+    /**
+     * Return the namespace for the command. If the first field in 'cmdObj' is of type
+     * mongo::String, then that field is interpreted as the collection name.
+     * If the first field is not of type mongo::String, then the namespace only has database name.
+     */
+    static NamespaceString parseNsFromCommand(const DatabaseName& dbName, const BSONObj& cmdObj);
+
     /**
      * Utility that returns a ResourcePattern for the namespace returned from
      * BasicCommand::parseNs(dbname, cmdObj). This will be either an exact namespace resource
@@ -170,10 +209,19 @@ struct CommandHelpers {
                                       const WriteConcernResult& wcResult = WriteConcernResult());
 
     /**
-     * Appends passthrough fields from a cmdObj to a given request.
+     * Forward generic arguments from a client request to shards.
      */
-    static BSONObj appendPassthroughFields(const BSONObj& cmdObjWithPassthroughFields,
-                                           const BSONObj& request);
+    static BSONObj appendGenericCommandArgs(const BSONObj& cmdObjWithGenericArgs,
+                                            const BSONObj& request);
+
+    /**
+     * Forward generic reply fields from a shard's reply to the client.
+     */
+    static void appendGenericReplyFields(const BSONObj& replyObjWithGenericReplyFields,
+                                         const BSONObj& reply,
+                                         BSONObjBuilder* replyBuilder);
+    static BSONObj appendGenericReplyFields(const BSONObj& replyObjWithGenericReplyFields,
+                                            const BSONObj& reply);
 
     /**
      * Returns a copy of 'cmdObj' with a majority writeConcern appended.  If the command object does
@@ -223,6 +271,14 @@ struct CommandHelpers {
      */
     static BSONObj runCommandDirectly(OperationContext* opCtx, const OpMsgRequest& request);
 
+    /**
+     * Runs the command synchronously in presence of a dedicated thread.
+     * Otherwise, runs the command asynchronously.
+     */
+    static Future<void> runCommandInvocation(std::shared_ptr<RequestExecutionContext> rec,
+                                             std::shared_ptr<CommandInvocation> invocation,
+                                             bool useDedicatedThread);
+
     /**
      * Runs a previously parsed CommandInvocation and propagates the result to the
      * ReplyBuilderInterface. This function is agnostic to the derived type of the CommandInvocation
@@ -233,6 +289,15 @@ struct CommandHelpers {
                                      CommandInvocation* invocation,
                                      rpc::ReplyBuilderInterface* response);
 
+    /**
+     * Runs a previously parsed command and propagates the result to the ReplyBuilderInterface. For
+     * commands that do not offer an implementation tailored for asynchronous execution, the future
+     * schedules the execution of the default implementation, historically designed for synchronous
+     * execution.
+     */
+    static Future<void> runCommandInvocationAsync(std::shared_ptr<RequestExecutionContext> rec,
+                                                  std::shared_ptr<CommandInvocation> invocation);
+
     /**
      * If '!invocation', we're logging about a Command pre-parse. It has to punt on the logged
      * namespace, giving only the request's $db. Since the Command hasn't parsed the request body,
@@ -254,6 +319,12 @@ struct CommandHelpers {
                                           const Command* command,
                                           const OpMsgRequest& request);
 
+    /**
+     * Asserts that a majority write concern was used for a command.
+     */
+    static void uassertCommandRunWithMajority(StringData commandName,
+                                              const WriteConcernOptions& wc);
+
     /**
      * Verifies that command is allowed to run under a transaction in the given database or
      * namespace, and throws if that verification doesn't pass.
@@ -353,6 +424,24 @@ public:
         return false;
     }
 
+    /*
+     * Returns the list of API versions that include this command.
+     */
+    virtual const std::set<std::string>& apiVersions() const;
+
+    /*
+     * Returns the list of API versions in which this command is deprecated.
+     */
+    virtual const std::set<std::string>& deprecatedApiVersions() const;
+
+    /*
+     * Some commands permit any values for apiVersion, apiStrict, and apiDeprecationErrors.
+     * For internal (server to server) commands we should skip checking api version.
+     */
+    virtual bool skipApiVersionCheck() const {
+        return false;
+    }
+
     /**
      * Like adminOnly, but even stricter: we must either be authenticated for admin db,
      * or, if running without auth, on the local interface.  Used for things which
@@ -366,8 +455,8 @@ public:
 
     /**
      * Note that secondaryAllowed should move to CommandInvocation but cannot because there is
-     * one place (i.e. 'listCommands') that inappropriately produces the "slaveOk" and
-     * "slaveOverrideOk" fields for each Command without regard to payload. This is
+     * one place (i.e. 'listCommands') that inappropriately produces the "secondaryOk" and
+     * "secondaryOverrideOk" fields for each Command without regard to payload. This is
      * inappropriate because for some Commands (e.g. 'aggregate'), these properties depend
      * on request payload. See SERVER-34578 for fixing listCommands.
      */
@@ -381,6 +470,21 @@ public:
         return true;
     }
 
+    /**
+     * Override and return true if the readConcernCounters in serverStatus should not be incremented
+     * on behalf of this command.
+     */
+    virtual bool shouldAffectReadConcernCounter() const {
+        return false;
+    }
+
+    /**
+      Returns true if this command collects operation resource consumption metrics.
+     */
+    virtual bool collectsResourceConsumptionMetrics() const {
+        return false;
+    }
+
     /**
      * Return true if the command requires auth.
      */
@@ -412,8 +516,8 @@ public:
      * The default snipForLogging shall remove these field names. Auditing shall not
      * include these fields in audit outputs.
      */
-    virtual StringData sensitiveFieldName() const {
-        return StringData{};
+    virtual std::set<StringData> sensitiveFieldNames() const {
+        return {};
     }
 
     /**
@@ -494,6 +598,51 @@ public:
         return true;
     }
 
+    /**
+     * By default, no newly created command is permitted under multitenancy.
+     * Implementations must override this to true to permit use.
+     */
+    virtual bool allowedWithSecurityToken() const {
+        return false;
+    }
+
+    /**
+     * Get the authorization contract for this command. nullptr means no contract has been
+     * specified.
+     */
+    virtual const AuthorizationContract* getAuthorizationContract() const {
+        return nullptr;
+    }
+
+    /**
+     * Returns true if this command supports apply once semantic when retried.
+     */
+    virtual bool supportsRetryableWrite() const {
+        return false;
+    }
+
+    /**
+     * Returns true if sessions should be checked out when lsid and txnNumber is present in the
+     * request.
+     */
+    virtual bool shouldCheckoutSession() const {
+        return true;
+    }
+
+    /**
+     * Returns true if this is a command related to managing the lifecycle of a transaction.
+     */
+    virtual bool isTransactionCommand() const {
+        return false;
+    }
+
+    /**
+     * Returns true if this command can be run in a transaction.
+     */
+    virtual bool allowedInTransactions() const {
+        return false;
+    }
+
 private:
     // The full name of the command
     const std::string _name;
@@ -502,11 +651,8 @@ private:
     const std::vector<StringData> _aliases;
 
     // Counters for how many times this command has been executed and failed
-    mutable Counter64 _commandsExecuted;
-    mutable Counter64 _commandsFailed;
-    // Pointers to hold the metrics tree references
-    ServerStatusMetricField<Counter64> _commandsExecutedMetric;
-    ServerStatusMetricField<Counter64> _commandsFailedMetric;
+    CounterMetric _commandsExecuted;
+    CounterMetric _commandsFailed;
 };
 
 /**
@@ -534,6 +680,16 @@ public:
      */
     virtual void run(OperationContext* opCtx, rpc::ReplyBuilderInterface* result) = 0;
 
+    /**
+     * Returns a future that can schedule asynchronous execution of the command. By default, the
+     * future falls back to the execution of `run(...)`, thus the default semantics of
+     * `runAsync(...)` is identical to that of `run(...).
+     */
+    virtual Future<void> runAsync(std::shared_ptr<RequestExecutionContext> rec) {
+        run(rec->getOpCtx(), rec->getReplyBuilder());
+        return Status::OK();
+    }
+
     virtual void explain(OperationContext* opCtx,
                          ExplainOptions::Verbosity verbosity,
                          rpc::ReplyBuilderInterface* result) {
@@ -555,7 +711,8 @@ public:
     /**
      * Returns this invocation's support for readConcern.
      */
-    virtual ReadConcernSupportResult supportsReadConcern(repl::ReadConcernLevel level) const {
+    virtual ReadConcernSupportResult supportsReadConcern(repl::ReadConcernLevel level,
+                                                         bool isImplicitDefault) const {
         static const Status kReadConcernNotSupported{ErrorCodes::InvalidOptions,
                                                      "read concern not supported"};
         static const Status kDefaultReadConcernNotPermitted{ErrorCodes::InvalidOptions,
@@ -564,6 +721,35 @@ public:
                 {kDefaultReadConcernNotPermitted}};
     }
 
+    /**
+     * Returns if this invocation is safe to run on a borrowed threading model.
+     *
+     * In practice, this is attempting to predict if the operation will do network or storage reads
+     * and writes. It will allow auth commands for the most part, since while they do involve
+     * network or storage operations, they are not targeting the storage engine or remote
+     * mongo-server nodes.
+     */
+    virtual bool isSafeForBorrowedThreads() const {
+        if (definition()->maintenanceMode() || !definition()->maintenanceOk()) {
+            // If the command has maintenance implications, it has storage implications.
+            return false;
+        }
+
+        if (supportsWriteConcern()) {
+            // If the command supports write concern, it has storage and network implications.
+            return false;
+        }
+
+        if (auto result = supportsReadConcern(repl::ReadConcernLevel::kMajorityReadConcern,
+                                              false /* isImplicitDefault */);
+            result.readConcernSupport.isOK()) {
+            // If the command supports read concern, it has storage and newtork implications.
+            return false;
+        }
+
+        return true;
+    }
+
     /**
      * Return if this invocation can be mirrored to secondaries
      */
@@ -578,6 +764,20 @@ public:
         MONGO_UNREACHABLE;
     }
 
+    /**
+     * Returns if this invocation is a mirrored read.
+     */
+    bool isMirrored() const {
+        return _mirrored;
+    }
+
+    /**
+     * Sets that this operation is a mirrored read.
+     */
+    void markMirrored() {
+        _mirrored = true;
+    }
+
     /**
      * Returns true if command allows afterClusterTime in its readConcern. The command may not allow
      * it if it is specifically intended not to take any LockManager locks. Waiting for
@@ -636,6 +836,8 @@ private:
     virtual void doCheckAuthorization(OperationContext* opCtx) const = 0;
 
     const Command* const _definition;
+
+    bool _mirrored = false;
 };
 
 /**
@@ -650,12 +852,13 @@ private:
 public:
     using Command::Command;
 
-    virtual std::string parseNs(const std::string& dbname, const BSONObj& cmdObj) const {
-        return CommandHelpers::parseNsFromCommand(dbname, cmdObj);
+    virtual NamespaceString parseNs(const DatabaseName& dbName, const BSONObj& cmdObj) const {
+        return CommandHelpers::parseNsFromCommand(dbName, cmdObj);
     }
 
     ResourcePattern parseResourcePattern(const std::string& dbname, const BSONObj& cmdObj) const {
-        return CommandHelpers::resourcePatternForNamespace(parseNs(dbname, cmdObj));
+        return CommandHelpers::resourcePatternForNamespace(
+            parseNs({boost::none, dbname}, cmdObj).ns());
     }
 
     //
@@ -666,10 +869,23 @@ public:
      * Runs the given command. Returns true upon success.
      */
     virtual bool runWithReplyBuilder(OperationContext* opCtx,
-                                     const std::string& db,
+                                     const DatabaseName& dbName,
                                      const BSONObj& cmdObj,
                                      rpc::ReplyBuilderInterface* replyBuilder) = 0;
 
+    /**
+     * Provides a future that may run the command asynchronously. By default, it falls back to
+     * runWithReplyBuilder.
+     */
+    virtual Future<void> runAsync(std::shared_ptr<RequestExecutionContext> rec,
+                                  const DatabaseName& dbName) {
+        if (!runWithReplyBuilder(
+                rec->getOpCtx(), dbName, rec->getRequest().body, rec->getReplyBuilder()))
+            return Status(ErrorCodes::FailedToRunWithReplyBuilder,
+                          fmt::format("Failed to run command: {}", rec->getCommand()->getName()));
+        return Status::OK();
+    }
+
     /**
      * Commands which can be explained override this method. Any operation which has a query
      * part and executes as a tree of execution stages can be explained. A command should
@@ -717,11 +933,12 @@ public:
      * cases where it isn't supported.
      */
     virtual ReadConcernSupportResult supportsReadConcern(const BSONObj& cmdObj,
-                                                         repl::ReadConcernLevel level) const {
+                                                         repl::ReadConcernLevel level,
+                                                         bool isImplicitDefault) const {
         static const Status kReadConcernNotSupported{ErrorCodes::InvalidOptions,
                                                      "read concern not supported"};
-        static const Status kDefaultReadConcernNotPermitted{ErrorCodes::InvalidOptions,
-                                                            "default read concern not permitted"};
+        static const Status kDefaultReadConcernNotPermitted{
+            ErrorCodes::InvalidOptions, "cluster wide default read concern not permitted"};
         return {{level != repl::ReadConcernLevel::kLocalReadConcern, kReadConcernNotSupported},
                 {kDefaultReadConcernNotPermitted}};
     }
@@ -794,17 +1011,160 @@ public:
      * Runs the given command. Returns true upon success.
      */
     virtual bool run(OperationContext* opCtx,
-                     const std::string& db,
+                     const DatabaseName& dbName,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) = 0;
 
     bool runWithReplyBuilder(OperationContext* opCtx,
-                             const std::string& db,
+                             const DatabaseName& dbName,
+                             const BSONObj& cmdObj,
+                             rpc::ReplyBuilderInterface* replyBuilder) override {
+        auto result = replyBuilder->getBodyBuilder();
+        return run(opCtx, dbName, cmdObj, result);
+    }
+};
+
+namespace {
+// Used in BasicCommandWithRequestParser below.
+template <typename T, typename = int>
+struct CommandAlias {
+    // An empty alias is equivalent to no alias, see CommandRegistry::registerCommand.
+    static constexpr StringData kAlias = ""_sd;
+};
+
+template <typename T>
+struct CommandAlias<T, decltype((void)T::kCommandAlias, 0)> {
+    static constexpr StringData kAlias = T::kCommandAlias;
+};
+
+template <typename T>
+constexpr StringData command_alias_v = CommandAlias<T>::kAlias;
+}  // namespace
+
+/**
+ * A CRTP base class for BasicCommandWithRequestParser, which simplifies writing commands that
+ * accept requests generated by IDL to enforce API versioning and to overcome the complexity
+ * to standardize the output generated by commands using IDL.
+ *
+ * Derive from it as follows:
+ *
+ *     class MyCommand : public BasicCommandWithRequestParser<MyCommand> {...};
+ *
+ * The 'Derived' type parameter must have:
+ *
+ *   - 'Request' naming a usable request type.
+ *     A usable Request type must have:
+ *
+ *      - a static member factory function 'parse', callable as:
+ *
+ *         const IDLParserContext& idlCtx = ...;
+ *         const OpMsgRequest& opMsgRequest = ...;
+ *         Request r = Request::parse(idlCtx, opMsgRequest);
+ *
+ *      which enables it to be parsed as an IDL command.
+ *
+ *      - a 'static constexpr StringData kCommandName' member.
+ *      - (optional) a 'static constexpr StringData kCommandAlias' member.
+ *
+ *   - validateResult: that has a custom logic to validate the result BSON object
+ *     to enforce API versioning.
+ *
+ */
+template <typename Derived>
+class BasicCommandWithRequestParser : public BasicCommandWithReplyBuilderInterface {
+protected:
+    BasicCommandWithRequestParser()
+        : BasicCommandWithReplyBuilderInterface(Derived::Request::kCommandName,
+                                                command_alias_v<typename Derived::Request>) {}
+
+    BasicCommandWithRequestParser(StringData name) : BasicCommandWithReplyBuilderInterface(name) {}
+
+    bool runWithReplyBuilder(OperationContext* opCtx,
+                             const DatabaseName& dbName,
                              const BSONObj& cmdObj,
                              rpc::ReplyBuilderInterface* replyBuilder) final {
         auto result = replyBuilder->getBodyBuilder();
-        return run(opCtx, db, cmdObj, result);
+
+        // To enforce API versioning
+        auto requestParser = RequestParser(opCtx, dbName, cmdObj);
+
+        auto cmdDone = runWithRequestParser(opCtx, dbName, cmdObj, requestParser, result);
+
+        // Only validate results in test mode so that we don't expose users to errors if we
+        // construct an invalid reply.
+        if (getTestCommandsEnabled()) {
+            validateResult(result.asTempObj());
+        }
+
+        return cmdDone;
     }
+
+    class RequestParser;
+
+    /**
+     * Runs the given command. Returns true upon success.
+     */
+    virtual bool runWithRequestParser(OperationContext* opCtx,
+                                      const DatabaseName& dbName,
+                                      const BSONObj& cmdObj,
+                                      const RequestParser& requestParser,
+                                      BSONObjBuilder& result) = 0;
+
+    // Custom logic to validate results to enforce API versioning.
+    virtual void validateResult(const BSONObj& resultObj) = 0;
+
+    /*
+     * If the result is an error, assert that it satisfies the IDL-defined requirements on a
+     * command error reply.
+     * Calls to this function should be done only in test mode so that we don't expose users to
+     * errors if we construct an invalid error reply.
+     */
+    static bool checkIsErrorStatus(const BSONObj& resultObj, const IDLParserContext& ctx) {
+        auto wcStatus = getWriteConcernStatusFromCommandResult(resultObj);
+        if (!wcStatus.isOK()) {
+            if (wcStatus.code() == ErrorCodes::TypeMismatch) {
+                // Result has "writeConcernError" field but it is not valid wce object.
+                uassertStatusOK(wcStatus);
+            }
+        }
+
+        if (resultObj.hasField("ok")) {
+            auto status = getStatusFromCommandResult(resultObj);
+            if (!status.isOK()) {
+                // Will throw if the result doesn't match the ErrorReply.
+                ErrorReply::parse(IDLParserContext("ErrorType", &ctx), resultObj);
+                return true;
+            }
+        }
+
+        return false;
+    }
+};
+
+template <typename Derived>
+class BasicCommandWithRequestParser<Derived>::RequestParser {
+public:
+    using RequestType = typename Derived::Request;
+
+    RequestParser(OperationContext* opCtx, const DatabaseName& dbName, const BSONObj& cmdObj)
+        : _request{_parseRequest(opCtx, dbName, cmdObj)} {}
+
+    const RequestType& request() const {
+        return _request;
+    }
+
+private:
+    static RequestType _parseRequest(OperationContext* opCtx,
+                                     const DatabaseName& dbName,
+                                     const BSONObj& cmdObj) {
+        // TODO SERVER-67155 pass tenantId to the BSONObj parse function
+        return RequestType::parse(
+            IDLParserContext(RequestType::kCommandName,
+                             APIParameters::get(opCtx).getAPIStrict().value_or(false)),
+            cmdObj);
+    }
+
+    RequestType _request;
 };
 
 /**
@@ -813,7 +1173,7 @@ public:
 class ErrmsgCommandDeprecated : public BasicCommand {
     using BasicCommand::BasicCommand;
     bool run(OperationContext* opCtx,
-             const std::string& db,
+             const DatabaseName& dbName,
              const BSONObj& cmdObj,
              BSONObjBuilder& result) final;
 
@@ -830,14 +1190,14 @@ class ErrmsgCommandDeprecated : public BasicCommand {
  *
  *     class MyCommand : public TypedCommand<MyCommand> {...};
  *
- * The 'Derived' type paramter must have:
+ * The 'Derived' type parameter must have:
  *
  *   - 'Request' naming a usable request type.
  *     A usable Request type must have:
  *
  *      - a static member factory function 'parse', callable as:
  *
- *         const IDLParserErrorContext& idlCtx = ...;
+ *         const IDLParserContext& idlCtx = ...;
  *         const OpMsgRequest& opMsgRequest = ...;
  *         Request r = Request::parse(idlCtx, opMsgRequest);
  *
@@ -878,22 +1238,44 @@ class TypedCommand<Derived>::InvocationBaseInternal : public CommandInvocation {
 public:
     using RequestType = typename Derived::Request;
 
-    InvocationBaseInternal(OperationContext*,
+    InvocationBaseInternal(OperationContext* opCtx,
                            const Command* command,
                            const OpMsgRequest& opMsgRequest)
-        : CommandInvocation(command), _request{_parseRequest(command->getName(), opMsgRequest)} {}
+        : CommandInvocation(command),
+          _request{_parseRequest(opCtx, command, opMsgRequest)},
+          _opMsgRequest{opMsgRequest} {}
 
 protected:
     const RequestType& request() const {
         return _request;
     }
 
+    const OpMsgRequest& unparsedRequest() const {
+        return _opMsgRequest;
+    }
+
 private:
-    static RequestType _parseRequest(StringData name, const OpMsgRequest& opMsgRequest) {
-        return RequestType::parse(IDLParserErrorContext(name), opMsgRequest);
+    static RequestType _parseRequest(OperationContext* opCtx,
+                                     const Command* command,
+                                     const OpMsgRequest& opMsgRequest) {
+
+        bool apiStrict = APIParameters::get(opCtx).getAPIStrict().value_or(false);
+
+        // A command with 'apiStrict' cannot be invoked with alias.
+        if (opMsgRequest.getCommandName() != command->getName() && apiStrict) {
+            uasserted(ErrorCodes::APIStrictError,
+                      str::stream() << "Command invocation with name '"
+                                    << opMsgRequest.getCommandName().toString()
+                                    << "' is not allowed in 'apiStrict' mode, use '"
+                                    << command->getName() << "' instead");
+        }
+
+        return RequestType::parse(IDLParserContext(command->getName(), apiStrict), opMsgRequest);
     }
 
     RequestType _request;
+
+    const OpMsgRequest _opMsgRequest;
 };
 
 template <typename Derived>
@@ -969,8 +1351,7 @@ class CommandRegistry {
 public:
     using CommandMap = Command::CommandMap;
 
-    CommandRegistry() : _unknownsMetricField("commands.<UNKNOWN>", &_unknowns) {}
-
+    CommandRegistry() = default;
     CommandRegistry(const CommandRegistry&) = delete;
     CommandRegistry& operator=(const CommandRegistry&) = delete;
 
@@ -987,9 +1368,7 @@ public:
     }
 
 private:
-    Counter64 _unknowns;
-    ServerStatusMetricField<Counter64> _unknownsMetricField;
-
+    CounterMetric _unknowns{"commands.<UNKNOWN>"};
     CommandMap _commands;
 };
 
@@ -999,16 +1378,35 @@ private:
 CommandRegistry* globalCommandRegistry();
 
 /**
- * Creates a test command object of type CmdType if test commands are enabled for this process.
- * Prefer this syntax to using MONGO_INITIALIZER directly.
- * The created Command object is "leaked" intentionally, since it will register itself.
+ * Creates a test command object of type CmdType if test commands are enabled
+ * for this process. Prefer this syntax to using MONGO_INITIALIZER directly.
+ * The created Command object is "leaked" intentionally, since it will register
+ * itself.
+ *
+ * The command objects will be created after the "default" initializer, and all
+ * startup option processing happens prior to "default" (see base/init.h).
  */
 #define MONGO_REGISTER_TEST_COMMAND(CmdType)                                \
     MONGO_INITIALIZER(RegisterTestCommand_##CmdType)(InitializerContext*) { \
         if (getTestCommandsEnabled()) {                                     \
             new CmdType();                                                  \
         }                                                                   \
-        return Status::OK();                                                \
+    }
+
+/**
+ * Creates a command object of type CmdType if the featureFlag is enabled for
+ * this process, regardless of the current FCV. Prefer this syntax to using
+ * MONGO_INITIALIZER directly. The created Command object is "leaked"
+ * intentionally, since it will register itself.
+ *
+ * The command objects will be created after the "default" initializer, and all
+ * startup option processing happens prior to "default" (see base/init.h).
+ */
+#define MONGO_REGISTER_FEATURE_FLAGGED_COMMAND(CmdType, featureFlag)        \
+    MONGO_INITIALIZER(RegisterTestCommand_##CmdType)(InitializerContext*) { \
+        if (featureFlag.isEnabledAndIgnoreFCV()) {                          \
+            new CmdType();                                                  \
+        }                                                                   \
     }
 
 }  // namespace mongo
