@@ -27,19 +27,18 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
+#include "mongo/db/auth/authorization_checks.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/collection_catalog_helper.h"
 #include "mongo/db/catalog/database.h"
-#include "mongo/db/catalog/health_log_interface.h"
-#include "mongo/db/command_generic_argument.h"
+#include "mongo/db/catalog/health_log.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/test_commands_enabled.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/operation_context.h"
@@ -47,10 +46,14 @@
 #include "mongo/db/repl/oplog.h"
 #include "mongo/db/repl/optime.h"
 #include "mongo/db/write_concern_options.h"
+#include "mongo/idl/command_generic_argument.h"
 #include "mongo/util/background.h"
 
 #include "mongo/logv2/log.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
+
+
 namespace mongo {
 
 namespace {
@@ -92,7 +95,7 @@ public:
                                                               OplogEntriesEnum::Start,
                                                               boost::none /*data*/
             );
-            HealthLogInterface::get(_opCtx->getServiceContext())->log(*healthLogEntry);
+            HealthLog::get(_opCtx->getServiceContext()).log(*healthLogEntry);
 
             DbCheckOplogStartStop oplogEntry;
             const auto nss = NamespaceString("admin.$cmd");
@@ -118,7 +121,7 @@ public:
                                                               OplogEntriesEnum::Stop,
                                                               boost::none /*data*/
             );
-            HealthLogInterface::get(_opCtx->getServiceContext())->log(*healthLogEntry);
+            HealthLog::get(_opCtx->getServiceContext()).log(*healthLogEntry);
         } catch (const DBException&) {
             LOGV2(6202201, "Could not log stop event");
         }
@@ -151,7 +154,7 @@ struct DbCheckCollectionInfo {
 using DbCheckRun = std::vector<DbCheckCollectionInfo>;
 
 std::unique_ptr<DbCheckRun> singleCollectionRun(OperationContext* opCtx,
-                                                const std::string& dbName,
+                                                const DatabaseName& dbName,
                                                 const DbCheckSingleInvocation& invocation) {
     NamespaceString nss(dbName, invocation.getColl());
     AutoGetCollectionForRead agc(opCtx, nss);
@@ -189,13 +192,14 @@ std::unique_ptr<DbCheckRun> singleCollectionRun(OperationContext* opCtx,
 }
 
 std::unique_ptr<DbCheckRun> fullDatabaseRun(OperationContext* opCtx,
-                                            const std::string& dbName,
+                                            const DatabaseName& dbName,
                                             const DbCheckAllInvocation& invocation) {
-    uassert(
-        ErrorCodes::InvalidNamespace, "Cannot run dbCheck on local database", dbName != "local");
+    uassert(ErrorCodes::InvalidNamespace,
+            "Cannot run dbCheck on local database",
+            dbName.db() != "local");
 
-    AutoGetDb agd(opCtx, StringData(dbName), MODE_IS);
-    uassert(ErrorCodes::NamespaceNotFound, "Database " + dbName + " not found", agd.getDb());
+    AutoGetDb agd(opCtx, dbName, MODE_IS);
+    uassert(ErrorCodes::NamespaceNotFound, "Database " + dbName.db() + " not found", agd.getDb());
 
     const int64_t max = std::numeric_limits<int64_t>::max();
     const auto rate = invocation.getMaxCountPerSecond();
@@ -203,7 +207,7 @@ std::unique_ptr<DbCheckRun> fullDatabaseRun(OperationContext* opCtx,
     const auto maxBytesPerBatch = invocation.getMaxBytesPerBatch();
     const auto maxBatchTimeMillis = invocation.getMaxBatchTimeMillis();
     auto result = std::make_unique<DbCheckRun>();
-    auto perCollectionWork = [&](const Collection* coll) {
+    auto perCollectionWork = [&](const CollectionPtr& coll) {
         if (!coll->ns().isReplicated()) {
             return true;
         }
@@ -231,7 +235,7 @@ std::unique_ptr<DbCheckRun> fullDatabaseRun(OperationContext* opCtx,
  * Factory function for producing DbCheckRun's from command objects.
  */
 std::unique_ptr<DbCheckRun> getRun(OperationContext* opCtx,
-                                   const std::string& dbName,
+                                   const DatabaseName& dbName,
                                    const BSONObj& obj) {
     BSONObjBuilder builder;
 
@@ -247,11 +251,11 @@ std::unique_ptr<DbCheckRun> getRun(OperationContext* opCtx,
     // If the dbCheck argument is a string, this is the per-collection form.
     if (toParse["dbCheck"].type() == BSONType::String) {
         return singleCollectionRun(
-            opCtx, dbName, DbCheckSingleInvocation::parse(IDLParserErrorContext(""), toParse));
+            opCtx, dbName, DbCheckSingleInvocation::parse(IDLParserContext(""), toParse));
     } else {
         // Otherwise, it's the database-wide form.
         return fullDatabaseRun(
-            opCtx, dbName, DbCheckAllInvocation::parse(IDLParserErrorContext(""), toParse));
+            opCtx, dbName, DbCheckAllInvocation::parse(IDLParserContext(""), toParse));
     }
 }
 
@@ -261,7 +265,7 @@ std::unique_ptr<DbCheckRun> getRun(OperationContext* opCtx,
  */
 class DbCheckJob : public BackgroundJob {
 public:
-    DbCheckJob(const StringData& dbName, std::unique_ptr<DbCheckRun> run)
+    DbCheckJob(const DatabaseName& dbName, std::unique_ptr<DbCheckRun> run)
         : BackgroundJob(true), _done(false), _dbName(dbName.toString()), _run(std::move(run)) {}
 
 protected:
@@ -275,7 +279,7 @@ protected:
 
         {
             stdx::lock_guard<Client> lk(*tc.get());
-            tc.get()->setSystemOperationKillable(lk);
+            tc.get()->setSystemOperationKillableByStepdown(lk);
         }
 
         auto uniqueOpCtx = tc->makeOperationContext();
@@ -289,7 +293,7 @@ protected:
             } catch (const DBException& e) {
                 auto logEntry = dbCheckErrorHealthLogEntry(
                     coll.nss, "dbCheck failed", OplogEntriesEnum::Batch, e.toStatus());
-                HealthLogInterface::get(Client::getCurrent()->getServiceContext())->log(*logEntry);
+                HealthLog::get(Client::getCurrent()->getServiceContext()).log(*logEntry);
                 return;
             }
 
@@ -309,8 +313,7 @@ private:
         const std::string curOpMessage = "Scanning namespace " + info.nss.toString();
         ProgressMeterHolder progress;
         {
-            AutoGetCollection autoColl(opCtx, info.nss, MODE_IS);
-            auto coll = autoColl.getCollection();
+            AutoGetCollection coll(opCtx, info.nss, MODE_IS);
             if (coll) {
                 stdx::unique_lock<Client> lk(*opCtx->getClient());
                 progress.set(CurOp::get(opCtx)->setProgress_inlock(StringData(curOpMessage),
@@ -321,7 +324,7 @@ private:
                     "abandoning dbCheck batch because collection no longer exists",
                     OplogEntriesEnum::Batch,
                     Status(ErrorCodes::NamespaceNotFound, "collection not found"));
-                HealthLogInterface::get(Client::getCurrent()->getServiceContext())->log(*entry);
+                HealthLog::get(Client::getCurrent()->getServiceContext()).log(*entry);
                 return;
             }
         }
@@ -355,6 +358,7 @@ private:
                 return;
             }
 
+
             if (!result.isOK()) {
                 bool retryable = false;
                 std::unique_ptr<HealthLogEntry> entry;
@@ -398,17 +402,16 @@ private:
                                                        OplogEntriesEnum::Batch,
                                                        result.getStatus());
                 }
-                HealthLogInterface::get(opCtx)->log(*entry);
+                HealthLog::get(opCtx).log(*entry);
                 if (retryable) {
                     continue;
                 }
                 return;
             }
 
+            const auto stats = result.getValue();
 
             _batchesProcessed++;
-
-            const auto stats = result.getValue();
             auto entry = dbCheckBatchEntry(info.nss,
                                            stats.nDocs,
                                            stats.nBytes,
@@ -422,7 +425,7 @@ private:
                 (_batchesProcessed % gDbCheckHealthLogEveryNBatches.load() == 0)) {
                 // On debug builds, health-log every batch result; on release builds, health-log
                 // every N batches.
-                HealthLogInterface::get(opCtx)->log(*entry);
+                HealthLog::get(opCtx).log(*entry);
             }
 
             WriteConcernResult unused;
@@ -432,7 +435,7 @@ private:
                                                           "dbCheck failed waiting for writeConcern",
                                                           OplogEntriesEnum::Batch,
                                                           status);
-                HealthLogInterface::get(opCtx)->log(*entry);
+                HealthLog::get(opCtx).log(*entry);
             }
 
             start = stats.lastKey;
@@ -509,7 +512,7 @@ private:
                 AutoGetCollection agc(opCtx,
                                       info.nss,
                                       lockMode,
-                                      AutoGetCollection::ViewMode::kViewsForbidden,
+                                      AutoGetCollectionViewMode::kViewsForbidden,
                                       lockDeadline);
 
                 if (_stepdownHasOccurred(opCtx, info.nss)) {
@@ -519,13 +522,13 @@ private:
                 }
 
                 const auto& collection =
-                    CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, info.nss);
+                    CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, info.nss);
                 if (!collection) {
                     const auto msg = "Collection under dbCheck no longer exists";
                     return {ErrorCodes::NamespaceNotFound, msg};
                 }
 
-                auto readTimestamp = opCtx->recoveryUnit()->getPointInTimeReadTimestamp();
+                auto readTimestamp = opCtx->recoveryUnit()->getPointInTimeReadTimestamp(opCtx);
                 auto minVisible = collection->getMinimumVisibleSnapshot();
                 if (readTimestamp && minVisible &&
                     *readTimestamp < *collection->getMinimumVisibleSnapshot()) {
@@ -546,8 +549,8 @@ private:
                     return e.toStatus();
                 }
 
-                const auto deadline = Date_t::now() + Milliseconds(info.maxBatchTimeMillis);
-                Status status = hasher->hashAll(opCtx, deadline);
+                const auto batchDeadline = Date_t::now() + Milliseconds(info.maxBatchTimeMillis);
+                Status status = hasher->hashAll(opCtx, batchDeadline);
 
                 if (!status.isOK()) {
                     return status;
@@ -657,8 +660,8 @@ public:
                "Invoke with { dbCheck: <collection name/uuid>,\n"
                "              minKey: <first key, exclusive>,\n"
                "              maxKey: <last key, inclusive>,\n"
-               "              maxCount: <max number of docs>,\n"
-               "              maxSize: <max size of docs>,\n"
+               "              maxCount: <try to keep a batch within maxCount number of docs>,\n"
+               "              maxSize: <try to keep a batch withing maxSize of docs (bytes)>,\n"
                "              maxCountPerSecond: <max rate in docs/sec>\n"
                "              maxDocsPerBatch: <max number of docs/batch>\n"
                "              maxBytesPerBatch: <try to keep a batch within max bytes/batch>\n"
@@ -678,12 +681,12 @@ public:
     }
 
     virtual bool run(OperationContext* opCtx,
-                     const std::string& dbname,
+                     const DatabaseName& dbName,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
-        auto job = getRun(opCtx, dbname, cmdObj);
+        auto job = getRun(opCtx, dbName, cmdObj);
         try {
-            (new DbCheckJob(dbname, std::move(job)))->go();
+            (new DbCheckJob(dbName, std::move(job)))->go();
         } catch (const DBException& e) {
             result.append("ok", false);
             result.append("err", e.toString());
