@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
@@ -47,8 +46,7 @@
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/fsync_locked.h"
 #include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
-#include "mongo/db/query/query_request.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/backup_cursor_hooks.h"
 #include "mongo/db/storage/storage_engine.h"
@@ -58,6 +56,9 @@
 #include "mongo/util/background.h"
 #include "mongo/util/exit.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
+
+
 namespace mongo {
 namespace {
 
@@ -70,13 +71,10 @@ Lock::ResourceMutex commandMutex("fsyncCommandMutex");
  */
 class FSyncLockThread : public BackgroundJob {
 public:
-    FSyncLockThread(ServiceContext* serviceContext,
-                    bool allowFsyncFailure,
-                    const Milliseconds deadline)
+    FSyncLockThread(ServiceContext* serviceContext, bool allowFsyncFailure)
         : BackgroundJob(false),
           _serviceContext(serviceContext),
-          _allowFsyncFailure(allowFsyncFailure),
-          _deadline(deadline) {}
+          _allowFsyncFailure(allowFsyncFailure) {}
 
     std::string name() const override {
         return "FSyncLockThread";
@@ -88,16 +86,15 @@ private:
     ServiceContext* const _serviceContext;
     bool _allowFsyncFailure;
     static bool _shutdownTaskRegistered;
-    const Milliseconds _deadline;
 };
 
-class FSyncCommand : public ErrmsgCommandDeprecated {
+class FSyncCommand : public BasicCommand {
 public:
     static const char* url() {
         return "http://dochub.mongodb.org/core/fsynccommand";
     }
 
-    FSyncCommand() : ErrmsgCommandDeprecated("fsync") {}
+    FSyncCommand() : BasicCommand("fsync") {}
 
     virtual ~FSyncCommand() {
         // The FSyncLockThread is owned by the FSyncCommand and accesses FsyncCommand state. It must
@@ -130,23 +127,17 @@ public:
         actions.addAction(ActionType::fsync);
         out->push_back(Privilege(ResourcePattern::forClusterResource(), actions));
     }
-    virtual bool errmsgRun(OperationContext* opCtx,
-                           const std::string& dbname,
-                           const BSONObj& cmdObj,
-                           std::string& errmsg,
-                           BSONObjBuilder& result) {
-        if (opCtx->lockState()->isLocked()) {
-            errmsg = "fsync: Cannot execute fsync command from contexts that hold a data lock";
-            return false;
-        }
+
+    bool run(OperationContext* opCtx,
+             const DatabaseName&,
+             const BSONObj& cmdObj,
+             BSONObjBuilder& result) override {
+        uassert(ErrorCodes::IllegalOperation,
+                "fsync: Cannot execute fsync command from contexts that hold a data lock",
+                !opCtx->lockState()->isLocked());
 
         const bool lock = cmdObj["lock"].trueValue();
-        const bool forBackup = cmdObj["forBackup"].trueValue();
-        LOGV2(20461,
-              "CMD fsync: lock:{lock}",
-              "CMD fsync",
-              "lock"_attr = lock,
-              "forBackup"_attr = forBackup);
+        LOGV2(20461, "CMD fsync: lock:{lock}", "CMD fsync", "lock"_attr = lock);
 
         // fsync + lock is sometimes used to block writes out of the system and does not care if
         // the `BackupCursorService::fsyncLock` call succeeds.
@@ -178,24 +169,8 @@ public:
                 stdx::unique_lock<Latch> lk(lockStateMutex);
                 threadStatus = Status::OK();
                 threadStarted = false;
-
-                Milliseconds deadline = Milliseconds::max();
-                if (forBackup) {
-                    // Set a default deadline of 90s for the fsyncLock to be acquired.
-                    deadline = Milliseconds(90000);
-                    // Parse the cmdObj and update the deadline if
-                    // "fsyncLockAcquisitionTimeoutMillis" exists.
-                    for (const auto& elem : cmdObj) {
-                        if (elem.fieldNameStringData() == "fsyncLockAcquisitionTimeoutMillis") {
-                            deadline =
-                                Milliseconds{uassertStatusOK(QueryRequest::parseMaxTimeMS(elem))};
-                        }
-                    }
-                }
-
-                _lockThread = std::make_unique<FSyncLockThread>(
-                    opCtx->getServiceContext(), allowFsyncFailure, deadline);
-
+                _lockThread = std::make_unique<FSyncLockThread>(opCtx->getServiceContext(),
+                                                                allowFsyncFailure);
                 _lockThread->go();
 
                 while (!threadStarted && threadStatus.isOK()) {
@@ -299,9 +274,9 @@ private:
 
 } fsyncCmd;
 
-class FSyncUnlockCommand : public ErrmsgCommandDeprecated {
+class FSyncUnlockCommand : public BasicCommand {
 public:
-    FSyncUnlockCommand() : ErrmsgCommandDeprecated("fsyncUnlock") {}
+    FSyncUnlockCommand() : BasicCommand("fsyncUnlock") {}
 
     bool supportsWriteConcern(const BSONObj& cmd) const override {
         return false;
@@ -324,11 +299,10 @@ public:
         return isAuthorized ? Status::OK() : Status(ErrorCodes::Unauthorized, "Unauthorized");
     }
 
-    bool errmsgRun(OperationContext* opCtx,
-                   const std::string& db,
-                   const BSONObj& cmdObj,
-                   std::string& errmsg,
-                   BSONObjBuilder& result) override {
+    bool run(OperationContext* opCtx,
+             const DatabaseName&,
+             const BSONObj& cmdObj,
+             BSONObjBuilder& result) override {
         LOGV2(20465, "command: unlock requested");
 
         Lock::ExclusiveLock lk(opCtx->lockState(), commandMutex);
@@ -336,10 +310,8 @@ public:
         stdx::unique_lock<Latch> stateLock(fsyncCmd.lockStateMutex);
 
         auto lockCount = fsyncCmd.getLockCount_inLock();
-        if (lockCount == 0) {
-            errmsg = "fsyncUnlock called when not locked";
-            return false;
-        }
+
+        uassert(ErrorCodes::IllegalOperation, "fsyncUnlock called when not locked", lockCount != 0);
 
         fsyncCmd.releaseLock_inLock(stateLock);
 
@@ -375,18 +347,7 @@ void FSyncLockThread::run() {
     try {
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
-
-        // If the deadline exists, set it on the opCtx and GlobalRead lock.
-        Date_t lockDeadline = Date_t::max();
-        if (_deadline < Milliseconds::max()) {
-            lockDeadline = Date_t::now() + _deadline;
-        }
-
-        opCtx.setDeadlineAfterNowBy(Milliseconds(_deadline), ErrorCodes::ExceededTimeLimit);
-        Lock::GlobalRead global(
-            &opCtx,
-            lockDeadline,
-            Lock::InterruptBehavior::kThrow);  // Block any writes in order to flush the files.
+        Lock::GlobalRead global(&opCtx);  // Block any writes in order to flush the files.
 
         StorageEngine* storageEngine = _serviceContext->getStorageEngine();
 
@@ -411,13 +372,15 @@ void FSyncLockThread::run() {
         try {
             storageEngine->flushAllFiles(&opCtx, /*callerHoldsReadLock*/ true);
         } catch (const std::exception& e) {
-            LOGV2_ERROR(20472,
-                        "Error doing flushAll: {error}",
-                        "Error doing flushAll",
-                        "error"_attr = e.what());
-            fsyncCmd.threadStatus = Status(ErrorCodes::CommandFailed, e.what());
-            fsyncCmd.acquireFsyncLockSyncCV.notify_one();
-            return;
+            if (!_allowFsyncFailure) {
+                LOGV2_ERROR(20472,
+                            "Error doing flushAll: {error}",
+                            "Error doing flushAll",
+                            "error"_attr = e.what());
+                fsyncCmd.threadStatus = Status(ErrorCodes::CommandFailed, e.what());
+                fsyncCmd.acquireFsyncLockSyncCV.notify_one();
+                return;
+            }
         }
 
         bool successfulFsyncLock = false;
@@ -474,16 +437,7 @@ void FSyncLockThread::run() {
                 storageEngine->endBackup(&opCtx);
             }
         }
-    } catch (const ExceptionForCat<ErrorCategory::ExceededTimeLimitError>&) {
-        LOGV2_ERROR(204739, "Fsync timed out with ExceededTimeLimitError");
-        fsyncCmd.threadStatus = Status(ErrorCodes::Error::LockTimeout, "Fsync lock timed out");
-        fsyncCmd.acquireFsyncLockSyncCV.notify_one();
-        return;
-    } catch (const ExceptionFor<ErrorCodes::LockTimeout>&) {
-        LOGV2_ERROR(204740, "Fsync timed out with LockTimeout");
-        fsyncCmd.threadStatus = Status(ErrorCodes::Error::LockTimeout, "Fsync lock timed out");
-        fsyncCmd.acquireFsyncLockSyncCV.notify_one();
-        return;
+
     } catch (const std::exception& e) {
         LOGV2_FATAL(40350,
                     "FSyncLockThread exception: {error}",
@@ -494,7 +448,6 @@ void FSyncLockThread::run() {
 
 MONGO_INITIALIZER(fsyncLockedForWriting)(InitializerContext* context) {
     setLockedForWritingImpl([]() { return fsyncCmd.fsyncLocked(); });
-    return Status::OK();
 }
 
 }  // namespace
