@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
@@ -44,11 +43,16 @@
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/locker.h"
+#include "mongo/db/service_context.h"
 #include "mongo/logv2/log.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/str.h"
 #include "mongo/util/timer.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
+
 namespace mongo {
 namespace {
 
@@ -129,6 +133,7 @@ static const char* LockRequestStatusNames[] = {
 MONGO_STATIC_ASSERT((sizeof(LockRequestStatusNames) / sizeof(LockRequestStatusNames[0])) ==
                     LockRequest::StatusCount);
 
+auto getLockManager = ServiceContext::declareDecoration<LockManager>();
 }  // namespace
 
 /**
@@ -417,6 +422,22 @@ const unsigned LockManager::_numLockBuckets(128);
 // The exact value doesn't appear very important, but should be power of two
 const unsigned LockManager::_numPartitions = 32;
 
+// static
+LockManager* LockManager::get(ServiceContext* service) {
+    return &getLockManager(service);
+}
+
+// static
+LockManager* LockManager::get(ServiceContext& service) {
+    return &getLockManager(service);
+}
+
+// static
+LockManager* LockManager::get(OperationContext* opCtx) {
+    return get(opCtx->getServiceContext());
+}
+
+
 // static
 std::map<LockerId, BSONObj> LockManager::getLockToClientMap(ServiceContext* serviceContext) {
     std::map<LockerId, BSONObj> lockToClientMap;
@@ -462,7 +483,6 @@ LockManager::~LockManager() {
 
 LockResult LockManager::lock(ResourceId resId, LockRequest* request, LockMode mode) {
     // Sanity check that requests are not being reused without proper cleanup
-    invariant(request->status == LockRequest::STATUS_NEW);
     invariant(request->recursiveCount == 1);
 
     request->partitioned = (mode == MODE_IX || mode == MODE_IS);
@@ -472,6 +492,7 @@ LockResult LockManager::lock(ResourceId resId, LockRequest* request, LockMode mo
     if (request->partitioned) {
         Partition* partition = _getPartition(request);
         stdx::lock_guard<SimpleMutex> scopedLock(partition->mutex);
+        invariant(request->status == LockRequest::STATUS_NEW);
 
         // Fast path for intent locks
         PartitionedLockHead* partitionedLock = partition->find(resId);
@@ -488,6 +509,7 @@ LockResult LockManager::lock(ResourceId resId, LockRequest* request, LockMode mo
     // Use regular LockHead, maybe start partitioning
     LockBucket* bucket = _getBucket(resId);
     stdx::lock_guard<SimpleMutex> scopedLock(bucket->mutex);
+    invariant(request->status == LockRequest::STATUS_NEW);
 
     LockHead* lock = bucket->findOrInsert(resId);
 
@@ -514,7 +536,6 @@ LockResult LockManager::lock(ResourceId resId, LockRequest* request, LockMode mo
 LockResult LockManager::convert(ResourceId resId, LockRequest* request, LockMode newMode) {
     // If we are here, we already hold the lock in some mode. In order to keep it simple, we do
     // not allow requesting a conversion while a lock is already waiting or pending conversion.
-    invariant(request->status == LockRequest::STATUS_GRANTED);
     invariant(request->recursiveCount > 0);
 
     request->recursiveCount++;
@@ -536,6 +557,7 @@ LockResult LockManager::convert(ResourceId resId, LockRequest* request, LockMode
 
     LockBucket* bucket = _getBucket(resId);
     stdx::lock_guard<SimpleMutex> scopedLock(bucket->mutex);
+    invariant(request->status == LockRequest::STATUS_GRANTED);
 
     LockBucket::Map::iterator it = bucket->data.find(resId);
     invariant(it != bucket->data.end());
@@ -586,24 +608,21 @@ LockResult LockManager::convert(ResourceId resId, LockRequest* request, LockMode
 }
 
 bool LockManager::unlock(LockRequest* request) {
-    // Fast path for decrementing multiple references of the same lock. It is safe to do this
-    // without locking, because 1) all calls for the same lock request must be done on the same
-    // thread and 2) if there are lock requests hanging of a given LockHead, then this lock
-    // will never disappear.
     invariant(request->recursiveCount > 0);
     request->recursiveCount--;
-    if ((request->status == LockRequest::STATUS_GRANTED) && (request->recursiveCount > 0)) {
-        return false;
-    }
 
     if (request->partitioned) {
         // Unlocking a lock that was acquired as partitioned. The lock request may since have
         // moved to the lock head, but there is no safe way to find out without synchronizing
         // thorough the partition mutex. Migrations are expected to be rare.
-        invariant(request->status == LockRequest::STATUS_GRANTED ||
-                  request->status == LockRequest::STATUS_CONVERTING);
         Partition* partition = _getPartition(request);
         stdx::lock_guard<SimpleMutex> scopedLock(partition->mutex);
+        invariant(request->status == LockRequest::STATUS_GRANTED ||
+                  request->status == LockRequest::STATUS_CONVERTING);
+
+        if (request->status == LockRequest::STATUS_GRANTED && request->recursiveCount > 0)
+            return false;
+
         //  Fast path: still partitioned.
         if (request->partitionedLock) {
             request->partitionedLock->grantedList.remove(request);
@@ -619,6 +638,9 @@ bool LockManager::unlock(LockRequest* request) {
     stdx::lock_guard<SimpleMutex> scopedLock(bucket->mutex);
 
     if (request->status == LockRequest::STATUS_GRANTED) {
+        if (request->recursiveCount > 0)
+            return false;
+
         // This releases a currently held lock and is the most common path, so it should be
         // as efficient as possible. The fast path for decrementing multiple references did
         // already ensure request->recursiveCount == 0.
@@ -667,7 +689,6 @@ bool LockManager::unlock(LockRequest* request) {
 
 void LockManager::downgrade(LockRequest* request, LockMode newMode) {
     invariant(request->lock);
-    invariant(request->status == LockRequest::STATUS_GRANTED);
     invariant(request->recursiveCount > 0);
 
     // The conflict set of the newMode should be a subset of the conflict set of the old mode.
@@ -679,6 +700,7 @@ void LockManager::downgrade(LockRequest* request, LockMode newMode) {
 
     LockBucket* bucket = _getBucket(lock->resourceId);
     stdx::lock_guard<SimpleMutex> scopedLock(bucket->mutex);
+    invariant(request->status == LockRequest::STATUS_GRANTED);
 
     lock->incGrantedModeCount(newMode);
     lock->decGrantedModeCount(request->mode);
@@ -953,8 +975,8 @@ std::string ResourceId::toString() const {
     }
 
     if (getType() == RESOURCE_DATABASE || getType() == RESOURCE_COLLECTION) {
-        CollectionCatalog& catalog = CollectionCatalog::get(getGlobalServiceContext());
-        boost::optional<std::string> resourceName = catalog.lookupResourceName(*this);
+        auto catalog = CollectionCatalog::get(getGlobalServiceContext());
+        boost::optional<std::string> resourceName = catalog->lookupResourceName(*this);
         if (resourceName) {
             ss << ", " << *resourceName;
         }
