@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
@@ -51,6 +50,9 @@
 #include "mongo/util/scopeguard.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
+
 namespace mongo {
 
 MONGO_FAIL_POINT_DEFINE(failNonIntentLocksIfWaitNeeded);
@@ -58,6 +60,15 @@ MONGO_FAIL_POINT_DEFINE(enableTestOnlyFlagforRSTL);
 
 namespace {
 
+// Ignore data races in certain functions when running with TSAN. For performance reasons,
+// diagnostic commands are expected to race with concurrent lock acquisitions while gathering
+// statistics.
+#if defined(__has_feature) && __has_feature(thread_sanitizer)
+#define MONGO_TSAN_IGNORE __attribute__((no_sanitize("thread")))
+#else
+#define MONGO_TSAN_IGNORE
+#endif
+
 /**
  * Tracks global (across all clients) lock acquisition statistics, partitioned into multiple
  * buckets to minimize concurrent access conflicts.
@@ -117,10 +128,6 @@ private:
     AlignedLockStats _partitions[NumPartitions];
 };
 
-
-// Global lock manager instance.
-LockManager globalLockManager;
-
 // How often (in millis) to check for deadlock if a lock has not been granted for some time
 const Milliseconds MaxWaitTime = Milliseconds(500);
 
@@ -279,24 +286,15 @@ void CondVarLockGrantNotification::notify(ResourceId resId, LockResult result) {
     _cond.notify_all();
 }
 
-namespace {
-TicketHolder* ticketHolders[LockModesCount] = {};
-}  // namespace
-
-
 //
 // Locker
 //
 
-/* static */
-void Locker::setGlobalThrottling(class TicketHolder* reading, class TicketHolder* writing) {
-    ticketHolders[MODE_S] = reading;
-    ticketHolders[MODE_IS] = reading;
-    ticketHolders[MODE_IX] = writing;
-}
-
-LockerImpl::LockerImpl()
-    : _id(idCounter.addAndFetch(1)), _wuowNestingLevel(0), _threadId(stdx::this_thread::get_id()) {}
+LockerImpl::LockerImpl(ServiceContext* serviceCtx)
+    : _id(idCounter.addAndFetch(1)),
+      _wuowNestingLevel(0),
+      _threadId(stdx::this_thread::get_id()),
+      _ticketHolder(TicketHolder::get(serviceCtx)) {}
 
 stdx::thread::id LockerImpl::getThreadId() const {
     return _threadId;
@@ -316,6 +314,7 @@ LockerImpl::~LockerImpl() {
     // to delete with unaccounted locks anyways.
     invariant(!inAWriteUnitOfWork());
     invariant(_numResourcesToUnlockAtEndUnitOfWork == 0);
+    invariant(!_ticket || !_ticket->valid());
 
     if (!_requests.empty()) {
         _dumpLockerAndLockManagerRequests();
@@ -363,18 +362,31 @@ void LockerImpl::reacquireTicket(OperationContext* opCtx) {
 }
 
 bool LockerImpl::_acquireTicket(OperationContext* opCtx, LockMode mode, Date_t deadline) {
+    _admCtx.setLockMode(mode);
     const bool reader = isSharedLockMode(mode);
-    auto holder = shouldAcquireTicket() ? ticketHolders[mode] : nullptr;
-    if (holder) {
+    auto holder = shouldAcquireTicket() ? _ticketHolder : nullptr;
+    // MODE_X is exclusive of all other locks, thus acquiring a ticket is unnecessary.
+    if (mode != MODE_X && mode != MODE_NONE && holder) {
         _clientState.store(reader ? kQueuedReader : kQueuedWriter);
-
         // If the ticket wait is interrupted, restore the state of the client.
-        auto restoreStateOnErrorGuard = makeGuard([&] { _clientState.store(kInactive); });
+        ScopeGuard restoreStateOnErrorGuard([&] {
+            _clientState.store(kInactive);
+            _admCtx.setLockMode(MODE_NONE);
+        });
 
-        OperationContext* interruptible = _uninterruptibleLocksRequested ? nullptr : opCtx;
+        // Acquiring a ticket is a potentially blocking operation. This must not be called after a
+        // transaction timestamp has been set, indicating this transaction has created an oplog
+        // hole.
+        invariant(!opCtx->recoveryUnit()->isTimestamped());
+
+        auto waitMode = _uninterruptibleLocksRequested ? TicketHolder::WaitMode::kUninterruptible
+                                                       : TicketHolder::WaitMode::kInterruptible;
+        _admCtx.setLockMode(mode);
         if (deadline == Date_t::max()) {
-            holder->waitForTicket(interruptible);
-        } else if (!holder->waitForTicketUntil(interruptible, deadline)) {
+            _ticket = holder->waitForTicket(opCtx, &_admCtx, waitMode);
+        } else if (auto ticket = holder->waitForTicketUntil(opCtx, &_admCtx, deadline, waitMode)) {
+            _ticket = std::move(*ticket);
+        } else {
             return false;
         }
         restoreStateOnErrorGuard.dismiss();
@@ -394,7 +406,7 @@ void LockerImpl::lockGlobal(OperationContext* opCtx, LockMode mode, Date_t deadl
             deadline = std::min(deadline,
                                 _maxLockTimeout ? beforeAcquire + *_maxLockTimeout : Date_t::max());
             uassert(ErrorCodes::LockTimeout,
-                    str::stream() << "Unable to acquire ticket with mode '" << _modeForTicket
+                    str::stream() << "Unable to acquire ticket with mode '" << mode
                                   << "' within a max lock request timeout of '"
                                   << Date_t::now() - beforeAcquire << "' milliseconds.",
                     _acquireTicket(opCtx, mode, deadline));
@@ -402,21 +414,13 @@ void LockerImpl::lockGlobal(OperationContext* opCtx, LockMode mode, Date_t deadl
         _modeForTicket = mode;
     }
 
-    LockMode actualLockMode = mode;
-    if (opCtx) {
-        auto storageEngine = opCtx->getServiceContext()->getStorageEngine();
-        if (storageEngine && !storageEngine->supportsDBLocking()) {
-            actualLockMode = isSharedLockMode(mode) ? MODE_S : MODE_X;
-        }
-    }
-
-    const LockResult result = _lockBegin(opCtx, resourceIdGlobal, actualLockMode);
+    const LockResult result = _lockBegin(opCtx, resourceIdGlobal, mode);
     // Fast, uncontended path
     if (result == LOCK_OK)
         return;
 
     invariant(result == LOCK_WAITING);
-    _lockComplete(opCtx, resourceIdGlobal, actualLockMode, deadline);
+    _lockComplete(opCtx, resourceIdGlobal, mode, deadline);
 }
 
 bool LockerImpl::unlockGlobal() {
@@ -556,7 +560,7 @@ void LockerImpl::lock(OperationContext* opCtx, ResourceId resId, LockMode mode,
 
 void LockerImpl::downgrade(ResourceId resId, LockMode newMode) {
     LockRequestsMap::Iterator it = _requests.find(resId);
-    globalLockManager.downgrade(it.objAddr(), newMode);
+    getGlobalLockManager()->downgrade(it.objAddr(), newMode);
 }
 
 bool LockerImpl::unlock(ResourceId resId) {
@@ -623,9 +627,7 @@ bool LockerImpl::isLockHeldForMode(ResourceId resId, LockMode mode) const {
     return isModeCovered(mode, getLockMode(resId));
 }
 
-bool LockerImpl::isDbLockedForMode(StringData dbName, LockMode mode) const {
-    invariant(nsIsDbOnly(dbName));
-
+bool LockerImpl::isDbLockedForMode(const DatabaseName& dbName, LockMode mode) const {
     if (isW())
         return true;
     if (isR() && isSharedLockMode(mode))
@@ -643,7 +645,7 @@ bool LockerImpl::isCollectionLockedForMode(const NamespaceString& nss, LockMode
     if (isR() && isSharedLockMode(mode))
         return true;
 
-    const ResourceId resIdDb(RESOURCE_DATABASE, nss.db());
+    const ResourceId resIdDb(RESOURCE_DATABASE, nss.dbName());
 
     LockMode dbMode = getLockMode(resIdDb);
     if (!shouldConflictWithSecondaryBatchApplication())
@@ -658,7 +660,7 @@ bool LockerImpl::isCollectionLockedForMode(const NamespaceString& nss, LockMode
             return isSharedLockMode(mode);
         case MODE_IX:
         case MODE_IS: {
-            const ResourceId resIdColl(RESOURCE_COLLECTION, nss.ns());
+            const ResourceId resIdColl(RESOURCE_COLLECTION, nss);
             return isLockHeldForMode(resIdColl, mode);
         } break;
         case LockModesCount:
@@ -692,19 +694,10 @@ void LockerImpl::setGlobalLockTakenInMode(LockMode mode) {
 ResourceId LockerImpl::getWaitingResource() const {
     scoped_spinlock scopedLock(_lock);
 
-    LockRequestsMap::ConstIterator it = _requests.begin();
-    while (!it.finished()) {
-        if (it->status == LockRequest::STATUS_WAITING ||
-            it->status == LockRequest::STATUS_CONVERTING) {
-            return it.key();
-        }
-
-        it.next();
-    }
-
-    return ResourceId();
+    return _waitingResource;
 }
 
+MONGO_TSAN_IGNORE
 void LockerImpl::getLockerInfo(LockerInfo* lockerInfo,
                                const boost::optional<SingleThreadedLockStats> lockStatsBase) const {
     invariant(lockerInfo);
@@ -749,7 +742,8 @@ boost::optional<Locker::LockerInfo> LockerImpl::getLockerInfo(
 bool LockerImpl::saveLockStateAndUnlock(Locker::LockSnapshot* stateOut) {
     // We shouldn't be saving and restoring lock state from inside a WriteUnitOfWork.
     invariant(!inAWriteUnitOfWork());
-
+    invariant(!(_modeForTicket == MODE_S || _modeForTicket == MODE_X),
+              "Yielding a strong global MODE_X/MODE_S lock is forbidden");
     // Clear out whatever is in stateOut.
     stateOut->locks.clear();
     stateOut->globalMode = MODE_NONE;
@@ -774,6 +768,11 @@ bool LockerImpl::saveLockStateAndUnlock(Locker::LockSnapshot* stateOut) {
         return false;
     }
 
+    // If the RSTL is exclusive, then this operation should not yield.
+    if (rstlRequest && rstlRequest->mode != MODE_IX) {
+        return false;
+    }
+
     // The global lock must have been acquired just once
     stateOut->globalMode = globalRequest->mode;
     invariant(unlock(resourceIdGlobal));
@@ -795,7 +794,10 @@ bool LockerImpl::saveLockStateAndUnlock(Locker::LockSnapshot* stateOut) {
         OneLock info;
         info.resourceId = resId;
         info.mode = it->mode;
-
+        invariant(
+            !(info.mode == MODE_S || info.mode == MODE_X),
+            str::stream() << "Yielding a strong MODE_X/MODE_S lock is forbidden. ResourceId was "
+                          << resId.toString());
         stateOut->locks.push_back(info);
 
         invariant(unlock(resId));
@@ -819,21 +821,13 @@ void LockerImpl::restoreLockState(OperationContext* opCtx, const Locker::LockSna
     }
 
     std::vector<OneLock>::const_iterator it = state.locks.begin();
-    // If we locked the PBWM, it must be locked before the
-    // resourceIdFeatureCompatibilityVersion, resourceIdReplicationStateTransitionLock, and
-    // resourceIdGlobal resources.
+    // If we locked the PBWM, it must be locked before the resourceIdGlobal and
+    // resourceIdReplicationStateTransitionLock resources.
     if (it != state.locks.end() && it->resourceId == resourceIdParallelBatchWriterMode) {
         lock(opCtx, it->resourceId, it->mode);
         it++;
     }
 
-    // If we locked the FCV lock, it must be locked before the
-    // resourceIdReplicationStateTransitionLock and resourceIdGlobal resources.
-    if (it != state.locks.end() && it->resourceId == resourceIdFeatureCompatibilityVersion) {
-        lock(opCtx, it->resourceId, it->mode);
-        it++;
-    }
-
     // If we locked the RSTL, it must be locked before the resourceIdGlobal resource.
     if (it != state.locks.end() && it->resourceId == resourceIdReplicationStateTransitionLock) {
         lock(opCtx, it->resourceId, it->mode);
@@ -842,16 +836,28 @@ void LockerImpl::restoreLockState(OperationContext* opCtx, const Locker::LockSna
 
     lockGlobal(opCtx, state.globalMode);
     for (; it != state.locks.end(); it++) {
-        // Ensures we don't acquire locks out of order which can lead to deadlock.
-        invariant(it->resourceId.getType() != ResourceType::RESOURCE_GLOBAL);
         lock(opCtx, it->resourceId, it->mode);
     }
     invariant(_modeForTicket != MODE_NONE);
 }
 
+MONGO_TSAN_IGNORE
+FlowControlTicketholder::CurOp LockerImpl::getFlowControlStats() const {
+    return _flowControlStats;
+}
+
+MONGO_TSAN_IGNORE
 LockResult LockerImpl::_lockBegin(OperationContext* opCtx, ResourceId resId, LockMode mode) {
     dassert(!getWaitingResource().isValid());
 
+    // Operations which are holding open an oplog hole cannot block when acquiring locks.
+    if (opCtx && !shouldAllowLockAcquisitionOnTimestampedUnitOfWork()) {
+        invariant(!opCtx->recoveryUnit()->isTimestamped(),
+                  str::stream()
+                      << "Operation holding open an oplog hole tried to acquire locks. ResourceId: "
+                      << resId << ", mode: " << modeName(mode));
+    }
+
     LockRequest* request;
     bool isNew = true;
 
@@ -904,12 +910,13 @@ LockResult LockerImpl::_lockBegin(OperationContext* opCtx, ResourceId resId, Loc
     // otherwise we might reset state if the lock becomes granted very fast.
     _notify.clear();
 
-    LockResult result = isNew ? globalLockManager.lock(resId, request, mode)
-                              : globalLockManager.convert(resId, request, mode);
+    LockResult result = isNew ? getGlobalLockManager()->lock(resId, request, mode)
+                              : getGlobalLockManager()->convert(resId, request, mode);
 
     if (result == LOCK_WAITING) {
         globalStats.recordWait(_id, resId, mode);
         _stats.recordWait(resId, mode);
+        _setWaitingResource(resId);
     } else if (result == LOCK_OK && opCtx && _uninterruptibleLocksRequested == 0) {
         // Lock acquisitions are not allowed to succeed when opCtx is marked as interrupted, unless
         // the caller requested an uninterruptible lock.
@@ -929,12 +936,22 @@ void LockerImpl::_lockComplete(OperationContext* opCtx,
                                ResourceId resId,
                                LockMode mode,
                                Date_t deadline) {
+    // Operations which are holding open an oplog hole cannot block when acquiring locks. Lock
+    // requests entering this function have been queued up and will be granted the lock as soon as
+    // the lock is released, which is a blocking operation.
+    if (opCtx && !shouldAllowLockAcquisitionOnTimestampedUnitOfWork()) {
+        invariant(!opCtx->recoveryUnit()->isTimestamped(),
+                  str::stream()
+                      << "Operation holding open an oplog hole tried to acquire locks. ResourceId: "
+                      << resId << ", mode: " << modeName(mode));
+    }
 
     // Clean up the state on any failed lock attempts.
-    auto unlockOnErrorGuard = makeGuard([&] {
+    ScopeGuard unlockOnErrorGuard([&] {
         LockRequestsMap::Iterator it = _requests.find(resId);
         invariant(it);
         _unlockImpl(&it);
+        _setWaitingResource(ResourceId());
     });
 
     // This failpoint is used to time out non-intent locks if they cannot be granted immediately
@@ -1018,6 +1035,7 @@ void LockerImpl::_lockComplete(OperationContext* opCtx,
 
     invariant(result == LOCK_OK);
     unlockOnErrorGuard.dismiss();
+    _setWaitingResource(ResourceId());
 }
 
 void LockerImpl::getFlowControlTicket(OperationContext* opCtx, LockMode lockMode) {
@@ -1029,7 +1047,11 @@ void LockerImpl::getFlowControlTicket(OperationContext* opCtx, LockMode lockMode
         // method must not exit with a side-effect on the clientState. That value is also used for
         // tracking whether other resources need to be released.
         _clientState.store(kQueuedWriter);
-        auto restoreState = makeGuard([&] { _clientState.store(kInactive); });
+        ScopeGuard restoreState([&] { _clientState.store(kInactive); });
+        // Acquiring a ticket is a potentially blocking operation. This must not be called after a
+        // transaction timestamp has been set, indicating this transaction has created an oplog
+        // hole.
+        invariant(!opCtx->recoveryUnit()->isTimestamped());
         ticketholder->getTicket(opCtx, &_flowControlStats);
     }
 }
@@ -1055,15 +1077,13 @@ void LockerImpl::releaseTicket() {
 }
 
 void LockerImpl::_releaseTicket() {
-    auto holder = shouldAcquireTicket() ? ticketHolders[_modeForTicket] : nullptr;
-    if (holder) {
-        holder->release();
-    }
+    _ticket.reset();
+    _admCtx.setLockMode(MODE_NONE);
     _clientState.store(kInactive);
 }
 
 bool LockerImpl::_unlockImpl(LockRequestsMap::Iterator* it) {
-    if (globalLockManager.unlock(it->objAddr())) {
+    if (getGlobalLockManager()->unlock(it->objAddr())) {
         if (it->key() == resourceIdGlobal) {
             invariant(_modeForTicket != MODE_NONE);
 
@@ -1089,6 +1109,12 @@ bool LockerImpl::isGlobalLockedRecursively() {
     return !globalLockRequest.finished() && globalLockRequest->recursiveCount > 1;
 }
 
+void LockerImpl::_setWaitingResource(ResourceId resId) {
+    scoped_spinlock scopedLock(_lock);
+
+    _waitingResource = resId;
+}
+
 //
 // Auto classes
 //
@@ -1112,13 +1138,14 @@ public:
 } unusedLockCleaner;
 }  // namespace
 
-
 //
 // Standalone functions
 //
 
 LockManager* getGlobalLockManager() {
-    return &globalLockManager;
+    auto serviceContext = getGlobalServiceContext();
+    invariant(serviceContext);
+    return LockManager::get(serviceContext);
 }
 
 void reportGlobalLockingStats(SingleThreadedLockStats* outStats) {
@@ -1129,17 +1156,4 @@ void resetGlobalLockStats() {
     globalStats.reset();
 }
 
-// Hardcoded resource IDs.
-const ResourceId resourceIdLocalDB = ResourceId(RESOURCE_DATABASE, StringData("local"));
-const ResourceId resourceIdOplog = ResourceId(RESOURCE_COLLECTION, StringData("local.oplog.rs"));
-const ResourceId resourceIdAdminDB = ResourceId(RESOURCE_DATABASE, StringData("admin"));
-const ResourceId resourceIdGlobal =
-    ResourceId(RESOURCE_GLOBAL, static_cast<uint8_t>(ResourceGlobalId::kGlobal));
-const ResourceId resourceIdParallelBatchWriterMode =
-    ResourceId(RESOURCE_GLOBAL, static_cast<uint8_t>(ResourceGlobalId::kParallelBatchWriterMode));
-const ResourceId resourceIdFeatureCompatibilityVersion = ResourceId(
-    RESOURCE_GLOBAL, static_cast<uint8_t>(ResourceGlobalId::kFeatureCompatibilityVersion));
-const ResourceId resourceIdReplicationStateTransitionLock = ResourceId(
-    RESOURCE_GLOBAL, static_cast<uint8_t>(ResourceGlobalId::kReplicationStateTransitionLock));
-
 }  // namespace mongo
