@@ -32,10 +32,13 @@
 #include <queue>
 
 #include "mongo/db/concurrency/fast_map_noalloc.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/concurrency/locker.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/platform/atomic_word.h"
+#include "mongo/util/concurrency/admission_context.h"
 #include "mongo/util/concurrency/spin_lock.h"
+#include "mongo/util/concurrency/ticketholder.h"
 
 namespace mongo {
 
@@ -82,7 +85,6 @@ private:
     LockResult _result;
 };
 
-
 /**
  * Interface for acquiring locks. One of those objects will have to be instantiated for each
  * request (transaction).
@@ -98,7 +100,7 @@ public:
      * Instantiates new locker. Must be given a unique identifier for disambiguation. Lockers
      * having the same identifier will not conflict on lock acquisition.
      */
-    LockerImpl();
+    LockerImpl(ServiceContext* serviceContext);
 
     virtual ~LockerImpl();
 
@@ -135,9 +137,6 @@ public:
     virtual void lockGlobal(OperationContext* opCtx,
                             LockMode mode,
                             Date_t deadline = Date_t::max());
-    virtual void lockGlobal(LockMode mode, Date_t deadline = Date_t::max()) {
-        return lockGlobal(nullptr, mode, deadline);
-    }
 
     virtual bool unlockGlobal();
 
@@ -182,15 +181,15 @@ public:
 
     virtual LockMode getLockMode(ResourceId resId) const;
     virtual bool isLockHeldForMode(ResourceId resId, LockMode mode) const;
-    virtual bool isDbLockedForMode(StringData dbName, LockMode mode) const;
+    virtual bool isDbLockedForMode(const DatabaseName& dbName, LockMode mode) const;
     virtual bool isCollectionLockedForMode(const NamespaceString& nss, LockMode mode) const;
 
     virtual ResourceId getWaitingResource() const;
 
     virtual void getLockerInfo(LockerInfo* lockerInfo,
-                               const boost::optional<SingleThreadedLockStats> lockStatsBase) const;
+                               boost::optional<SingleThreadedLockStats> lockStatsBase) const;
     virtual boost::optional<LockerInfo> getLockerInfo(
-        const boost::optional<SingleThreadedLockStats> lockStatsBase) const final;
+        boost::optional<SingleThreadedLockStats> lockStatsBase) const final;
 
     virtual bool saveLockStateAndUnlock(LockSnapshot* stateOut);
 
@@ -209,11 +208,17 @@ public:
     virtual void releaseTicket();
     virtual void reacquireTicket(OperationContext* opCtx);
 
+    bool hasReadTicket() const override {
+        return _modeForTicket == MODE_IS || _modeForTicket == MODE_S;
+    }
+
+    bool hasWriteTicket() const override {
+        return _modeForTicket == MODE_IX || _modeForTicket == MODE_X;
+    }
+
     void getFlowControlTicket(OperationContext* opCtx, LockMode lockMode) override;
 
-    FlowControlTicketholder::CurOp getFlowControlStats() const override {
-        return _flowControlStats;
-    }
+    FlowControlTicketholder::CurOp getFlowControlStats() const override;
 
     //
     // Below functions are for testing only.
@@ -318,6 +323,8 @@ private:
      */
     bool _acquireTicket(OperationContext* opCtx, LockMode mode, Date_t deadline);
 
+    void _setWaitingResource(ResourceId resId);
+
     /**
      * Calls dump() on this locker instance and the lock manager.
      */
@@ -343,7 +350,7 @@ private:
 
     // Per-locker locking statistics. Reported in the slow-query log message and through
     // db.currentOp. Complementary to the per-instance locking statistics.
-    SingleThreadedLockStats _stats;
+    AtomicLockStats _stats;
 
     // Delays release of exclusive/intent-exclusive locked resources until the write unit of
     // work completes. Value of 0 means we are not inside a write unit of work.
@@ -370,6 +377,15 @@ private:
     // A structure for accumulating time spent getting flow control tickets.
     FlowControlTicketholder::CurOp _flowControlStats;
 
+    // Keeps state and statistics related to admission control.
+    AdmissionContext _admCtx;
+
+    // The global ticketholders of the service context.
+    TicketHolder* _ticketHolder;
+
+    // This will only be valid when holding a ticket.
+    boost::optional<Ticket> _ticket;
+
     // Tracks the global lock modes ever acquired in this Locker's life. This value should only ever
     // be accessed from the thread that owns the Locker.
     unsigned char _globalLockMode = (1 << MODE_NONE);
@@ -377,6 +393,10 @@ private:
     // Tracks whether this operation should be killed on step down.
     AtomicWord<bool> _wasGlobalLockTakenInModeConflictingWithWrites{false};
 
+    // If isValid(), the ResourceId of the resource currently waiting for the lock. If not valid,
+    // there is no resource currently waiting.
+    ResourceId _waitingResource;
+
     //////////////////////////////////////////////////////////////////////////////////////////
     //
     // Methods merged from LockState, which should eventually be removed or changed to methods
@@ -434,6 +454,11 @@ private:
 
 /**
  * Retrieves the global lock manager instance.
+ * Legacy global lock manager accessor for internal lock implementation * and debugger scripts
+ * such as gdb/mongo_lock.py.
+ * The lock manager is now a decoration on the service context and this accessor is retained for
+ * startup, lock internals, and debugger scripts.
+ * Using LockManager::get(ServiceContext*) where possible is preferable.
  */
 LockManager* getGlobalLockManager();
 
