@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/platform/basic.h"
 
@@ -46,15 +45,20 @@
 #include "mongo/util/fail_point.h"
 #include "mongo/util/timer.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
+
 namespace mongo {
 
 class LockerImplTest : public ServiceContextTest {};
 
 TEST_F(LockerImplTest, LockNoConflict) {
-    const ResourceId resId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    auto opCtx = makeOperationContext();
 
-    LockerImpl locker;
-    locker.lockGlobal(MODE_IX);
+    const ResourceId resId(RESOURCE_COLLECTION, NamespaceString(boost::none, "TestDB.collection"));
+
+    LockerImpl locker(opCtx->getServiceContext());
+    locker.lockGlobal(opCtx.get(), MODE_IX);
 
     locker.lock(resId, MODE_X);
 
@@ -69,10 +73,12 @@ TEST_F(LockerImplTest, LockNoConflict) {
 }
 
 TEST_F(LockerImplTest, ReLockNoConflict) {
-    const ResourceId resId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    auto opCtx = makeOperationContext();
 
-    LockerImpl locker;
-    locker.lockGlobal(MODE_IX);
+    const ResourceId resId(RESOURCE_COLLECTION, NamespaceString(boost::none, "TestDB.collection"));
+
+    LockerImpl locker(opCtx->getServiceContext());
+    locker.lockGlobal(opCtx.get(), MODE_IX);
 
     locker.lock(resId, MODE_S);
     locker.lock(resId, MODE_X);
@@ -87,16 +93,17 @@ TEST_F(LockerImplTest, ReLockNoConflict) {
 }
 
 TEST_F(LockerImplTest, ConflictWithTimeout) {
-    const ResourceId resId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    auto opCtx = makeOperationContext();
 
-    LockerImpl locker1;
-    locker1.lockGlobal(MODE_IX);
+    const ResourceId resId(RESOURCE_COLLECTION, NamespaceString(boost::none, "TestDB.collection"));
+
+    LockerImpl locker1(opCtx->getServiceContext());
+    locker1.lockGlobal(opCtx.get(), MODE_IX);
     locker1.lock(resId, MODE_X);
 
-    LockerImpl locker2;
-    locker2.lockGlobal(MODE_IX);
+    LockerImpl locker2(opCtx->getServiceContext());
+    locker2.lockGlobal(opCtx.get(), MODE_IX);
 
-    auto opCtx = makeOperationContext();
     ASSERT_THROWS_CODE(locker2.lock(opCtx.get(), resId, MODE_S, Date_t::now()),
                        AssertionException,
                        ErrorCodes::LockTimeout);
@@ -110,18 +117,19 @@ TEST_F(LockerImplTest, ConflictWithTimeout) {
 }
 
 TEST_F(LockerImplTest, ConflictUpgradeWithTimeout) {
-    const ResourceId resId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    auto opCtx = makeOperationContext();
 
-    LockerImpl locker1;
-    locker1.lockGlobal(MODE_IS);
+    const ResourceId resId(RESOURCE_COLLECTION, NamespaceString(boost::none, "TestDB.collection"));
+
+    LockerImpl locker1(opCtx->getServiceContext());
+    locker1.lockGlobal(opCtx.get(), MODE_IS);
     locker1.lock(resId, MODE_S);
 
-    LockerImpl locker2;
-    locker2.lockGlobal(MODE_IS);
+    LockerImpl locker2(opCtx->getServiceContext());
+    locker2.lockGlobal(opCtx.get(), MODE_IS);
     locker2.lock(resId, MODE_S);
 
     // Try upgrading locker 1, which should block and timeout
-    auto opCtx = makeOperationContext();
     ASSERT_THROWS_CODE(locker1.lock(opCtx.get(), resId, MODE_X, Date_t::now() + Milliseconds(1)),
                        AssertionException,
                        ErrorCodes::LockTimeout);
@@ -138,19 +146,19 @@ TEST_F(LockerImplTest, FailPointInLockFailsGlobalNonIntentLocksIfTheyCannotBeImm
     AlternativeClientRegion acr(newClient);
     auto newOpCtx = cc().makeOperationContext();
 
-    LockerImpl locker1;
-    locker1.lockGlobal(MODE_IX);
+    LockerImpl locker1(newOpCtx->getServiceContext());
+    locker1.lockGlobal(newOpCtx.get(), MODE_IX);
 
     {
         FailPointEnableBlock failWaitingNonPartitionedLocks("failNonIntentLocksIfWaitNeeded");
 
         // MODE_S attempt.
-        LockerImpl locker2;
+        LockerImpl locker2(newOpCtx->getServiceContext());
         ASSERT_THROWS_CODE(
             locker2.lockGlobal(newOpCtx.get(), MODE_S), DBException, ErrorCodes::LockTimeout);
 
         // MODE_X attempt.
-        LockerImpl locker3;
+        LockerImpl locker3(newOpCtx->getServiceContext());
         ASSERT_THROWS_CODE(
             locker3.lockGlobal(newOpCtx.get(), MODE_X), DBException, ErrorCodes::LockTimeout);
     }
@@ -167,9 +175,9 @@ TEST_F(LockerImplTest, FailPointInLockFailsNonIntentLocksIfTheyCannotBeImmediate
     auto newOpCtx = cc().makeOperationContext();
 
     // Granted MODE_X lock, fail incoming MODE_S and MODE_X.
-    const ResourceId resId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    const ResourceId resId(RESOURCE_COLLECTION, NamespaceString(boost::none, "TestDB.collection"));
 
-    LockerImpl locker1;
+    LockerImpl locker1(newOpCtx->getServiceContext());
     locker1.lockGlobal(newOpCtx.get(), MODE_IX);
     locker1.lock(newOpCtx.get(), resId, MODE_X);
 
@@ -177,7 +185,7 @@ TEST_F(LockerImplTest, FailPointInLockFailsNonIntentLocksIfTheyCannotBeImmediate
         FailPointEnableBlock failWaitingNonPartitionedLocks("failNonIntentLocksIfWaitNeeded");
 
         // MODE_S attempt.
-        LockerImpl locker2;
+        LockerImpl locker2(newOpCtx->getServiceContext());
         locker2.lockGlobal(newOpCtx.get(), MODE_IS);
         ASSERT_THROWS_CODE(locker2.lock(newOpCtx.get(), resId, MODE_S, Date_t::max()),
                            DBException,
@@ -189,7 +197,7 @@ TEST_F(LockerImplTest, FailPointInLockFailsNonIntentLocksIfTheyCannotBeImmediate
         locker2.unlockGlobal();
 
         // MODE_X attempt.
-        LockerImpl locker3;
+        LockerImpl locker3(newOpCtx->getServiceContext());
         locker3.lockGlobal(newOpCtx.get(), MODE_IX);
         ASSERT_THROWS_CODE(locker3.lock(newOpCtx.get(), resId, MODE_X, Date_t::max()),
                            DBException,
@@ -205,16 +213,18 @@ TEST_F(LockerImplTest, FailPointInLockFailsNonIntentLocksIfTheyCannotBeImmediate
 }
 
 TEST_F(LockerImplTest, ReadTransaction) {
-    LockerImpl locker;
+    auto opCtx = makeOperationContext();
 
-    locker.lockGlobal(MODE_IS);
+    LockerImpl locker(opCtx->getServiceContext());
+
+    locker.lockGlobal(opCtx.get(), MODE_IS);
     locker.unlockGlobal();
 
-    locker.lockGlobal(MODE_IX);
+    locker.lockGlobal(opCtx.get(), MODE_IX);
     locker.unlockGlobal();
 
-    locker.lockGlobal(MODE_IX);
-    locker.lockGlobal(MODE_IS);
+    locker.lockGlobal(opCtx.get(), MODE_IX);
+    locker.lockGlobal(opCtx.get(), MODE_IS);
     locker.unlockGlobal();
     locker.unlockGlobal();
 }
@@ -223,16 +233,18 @@ TEST_F(LockerImplTest, ReadTransaction) {
  * Test that saveLockerImpl works by examining the output.
  */
 TEST_F(LockerImplTest, saveAndRestoreGlobal) {
+    auto opCtx = makeOperationContext();
+
     Locker::LockSnapshot lockInfo;
 
-    LockerImpl locker;
+    LockerImpl locker(opCtx->getServiceContext());
 
     // No lock requests made, no locks held.
     locker.saveLockStateAndUnlock(&lockInfo);
     ASSERT_EQUALS(0U, lockInfo.locks.size());
 
     // Lock the global lock, but just once.
-    locker.lockGlobal(MODE_IX);
+    locker.lockGlobal(opCtx.get(), MODE_IX);
 
     // We've locked the global lock.  This should be reflected in the lockInfo.
     locker.saveLockStateAndUnlock(&lockInfo);
@@ -250,15 +262,17 @@ TEST_F(LockerImplTest, saveAndRestoreGlobal) {
  * Test that saveLockerImpl can save and restore the RSTL.
  */
 TEST_F(LockerImplTest, saveAndRestoreRSTL) {
+    auto opCtx = makeOperationContext();
+
     Locker::LockSnapshot lockInfo;
 
-    LockerImpl locker;
+    LockerImpl locker(opCtx->getServiceContext());
 
-    const ResourceId resIdDatabase(RESOURCE_DATABASE, "TestDB"_sd);
+    const ResourceId resIdDatabase(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB"));
 
     // Acquire locks.
     locker.lock(resourceIdReplicationStateTransitionLock, MODE_IX);
-    locker.lockGlobal(MODE_IX);
+    locker.lockGlobal(opCtx.get(), MODE_IX);
     locker.lock(resIdDatabase, MODE_IX);
 
     // Save the lock state.
@@ -287,17 +301,19 @@ TEST_F(LockerImplTest, saveAndRestoreRSTL) {
  * Test that we don't unlock when we have the global lock more than once.
  */
 TEST_F(LockerImplTest, saveAndRestoreGlobalAcquiredTwice) {
+    auto opCtx = makeOperationContext();
+
     Locker::LockSnapshot lockInfo;
 
-    LockerImpl locker;
+    LockerImpl locker(opCtx->getServiceContext());
 
     // No lock requests made, no locks held.
     locker.saveLockStateAndUnlock(&lockInfo);
     ASSERT_EQUALS(0U, lockInfo.locks.size());
 
     // Lock the global lock.
-    locker.lockGlobal(MODE_IX);
-    locker.lockGlobal(MODE_IX);
+    locker.lockGlobal(opCtx.get(), MODE_IX);
+    locker.lockGlobal(opCtx.get(), MODE_IX);
 
     // This shouldn't actually unlock as we're in a nested scope.
     ASSERT(!locker.saveLockStateAndUnlock(&lockInfo));
@@ -313,17 +329,20 @@ TEST_F(LockerImplTest, saveAndRestoreGlobalAcquiredTwice) {
  * Tests that restoreLockerImpl works by locking a db and collection and saving + restoring.
  */
 TEST_F(LockerImplTest, saveAndRestoreDBAndCollection) {
+    auto opCtx = makeOperationContext();
+
     Locker::LockSnapshot lockInfo;
 
-    LockerImpl locker;
+    LockerImpl locker(opCtx->getServiceContext());
 
-    const ResourceId resIdDatabase(RESOURCE_DATABASE, "TestDB"_sd);
-    const ResourceId resIdCollection(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    const ResourceId resIdDatabase(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB"));
+    const ResourceId resIdCollection(RESOURCE_COLLECTION,
+                                     NamespaceString(boost::none, "TestDB.collection"));
 
     // Lock some stuff.
-    locker.lockGlobal(MODE_IX);
+    locker.lockGlobal(opCtx.get(), MODE_IX);
     locker.lock(resIdDatabase, MODE_IX);
-    locker.lock(resIdCollection, MODE_X);
+    locker.lock(resIdCollection, MODE_IX);
     locker.saveLockStateAndUnlock(&lockInfo);
 
     // Things shouldn't be locked anymore.
@@ -335,24 +354,27 @@ TEST_F(LockerImplTest, saveAndRestoreDBAndCollection) {
 
     // Make sure things were re-locked.
     ASSERT_EQUALS(MODE_IX, locker.getLockMode(resIdDatabase));
-    ASSERT_EQUALS(MODE_X, locker.getLockMode(resIdCollection));
+    ASSERT_EQUALS(MODE_IX, locker.getLockMode(resIdCollection));
 
     ASSERT(locker.unlockGlobal());
 }
 
 TEST_F(LockerImplTest, releaseWriteUnitOfWork) {
+    auto opCtx = makeOperationContext();
+
     Locker::LockSnapshot lockInfo;
 
-    LockerImpl locker;
+    LockerImpl locker(opCtx->getServiceContext());
 
-    const ResourceId resIdDatabase(RESOURCE_DATABASE, "TestDB"_sd);
-    const ResourceId resIdCollection(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    const ResourceId resIdDatabase(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB"));
+    const ResourceId resIdCollection(RESOURCE_COLLECTION,
+                                     NamespaceString(boost::none, "TestDB.collection"));
 
     locker.beginWriteUnitOfWork();
     // Lock some stuff.
-    locker.lockGlobal(MODE_IX);
+    locker.lockGlobal(opCtx.get(), MODE_IX);
     locker.lock(resIdDatabase, MODE_IX);
-    locker.lock(resIdCollection, MODE_X);
+    locker.lock(resIdCollection, MODE_IX);
     // Unlock them so that they will be pending to unlock.
     ASSERT_FALSE(locker.unlock(resIdCollection));
     ASSERT_FALSE(locker.unlock(resIdDatabase));
@@ -369,18 +391,21 @@ TEST_F(LockerImplTest, releaseWriteUnitOfWork) {
 }
 
 TEST_F(LockerImplTest, restoreWriteUnitOfWork) {
+    auto opCtx = makeOperationContext();
+
     Locker::LockSnapshot lockInfo;
 
-    LockerImpl locker;
+    LockerImpl locker(opCtx->getServiceContext());
 
-    const ResourceId resIdDatabase(RESOURCE_DATABASE, "TestDB"_sd);
-    const ResourceId resIdCollection(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    const ResourceId resIdDatabase(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB"));
+    const ResourceId resIdCollection(RESOURCE_COLLECTION,
+                                     NamespaceString(boost::none, "TestDB.collection"));
 
     locker.beginWriteUnitOfWork();
     // Lock some stuff.
-    locker.lockGlobal(MODE_IX);
+    locker.lockGlobal(opCtx.get(), MODE_IX);
     locker.lock(resIdDatabase, MODE_IX);
-    locker.lock(resIdCollection, MODE_X);
+    locker.lock(resIdCollection, MODE_IX);
     // Unlock them so that they will be pending to unlock.
     ASSERT_FALSE(locker.unlock(resIdCollection));
     ASSERT_FALSE(locker.unlock(resIdDatabase));
@@ -398,7 +423,7 @@ TEST_F(LockerImplTest, restoreWriteUnitOfWork) {
 
     // Make sure things were re-locked.
     ASSERT_EQUALS(MODE_IX, locker.getLockMode(resIdDatabase));
-    ASSERT_EQUALS(MODE_X, locker.getLockMode(resIdCollection));
+    ASSERT_EQUALS(MODE_IX, locker.getLockMode(resIdCollection));
     ASSERT(locker.isLocked());
 
     locker.endWriteUnitOfWork();
@@ -409,22 +434,26 @@ TEST_F(LockerImplTest, restoreWriteUnitOfWork) {
 }
 
 TEST_F(LockerImplTest, releaseAndRestoreWriteUnitOfWorkWithoutUnlock) {
+    auto opCtx = makeOperationContext();
+
     Locker::WUOWLockSnapshot lockInfo;
 
-    LockerImpl locker;
+    LockerImpl locker(opCtx->getServiceContext());
 
-    const ResourceId resIdDatabase(RESOURCE_DATABASE, "TestDB"_sd);
-    const ResourceId resIdCollection(RESOURCE_COLLECTION, "TestDB.collection"_sd);
-    const ResourceId resIdCollection2(RESOURCE_COLLECTION, "TestDB.collection2"_sd);
+    const ResourceId resIdDatabase(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB"));
+    const ResourceId resIdCollection(RESOURCE_COLLECTION,
+                                     NamespaceString(boost::none, "TestDB.collection"));
+    const ResourceId resIdCollection2(RESOURCE_COLLECTION,
+                                      NamespaceString(boost::none, "TestDB.collection2"));
 
     locker.beginWriteUnitOfWork();
     // Lock some stuff.
-    locker.lockGlobal(MODE_IX);
+    locker.lockGlobal(opCtx.get(), MODE_IX);
     locker.lock(resIdDatabase, MODE_IX);
     locker.lock(resIdCollection, MODE_X);
 
     // Recursive global lock.
-    locker.lockGlobal(MODE_IX);
+    locker.lockGlobal(opCtx.get(), MODE_IX);
     ASSERT_EQ(locker.getRequestsForTest().find(resourceIdGlobal).objAddr()->recursiveCount, 2U);
 
     ASSERT_FALSE(locker.unlockGlobal());
@@ -461,7 +490,7 @@ TEST_F(LockerImplTest, releaseAndRestoreWriteUnitOfWorkWithoutUnlock) {
         locker.beginWriteUnitOfWork();
 
         // Grab new locks inside the new WUOW.
-        locker.lockGlobal(MODE_IX);
+        locker.lockGlobal(opCtx.get(), MODE_IX);
         locker.lock(resIdDatabase, MODE_IX);
         locker.lock(resIdCollection2, MODE_IX);
 
@@ -523,19 +552,22 @@ TEST_F(LockerImplTest, releaseAndRestoreWriteUnitOfWorkWithoutUnlock) {
 }
 
 TEST_F(LockerImplTest, releaseAndRestoreReadOnlyWriteUnitOfWork) {
+    auto opCtx = makeOperationContext();
+
     Locker::LockSnapshot lockInfo;
 
-    LockerImpl locker;
+    LockerImpl locker(opCtx->getServiceContext());
 
-    const ResourceId resIdDatabase(RESOURCE_DATABASE, "TestDB"_sd);
-    const ResourceId resIdCollection(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    const ResourceId resIdDatabase(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB"));
+    const ResourceId resIdCollection(RESOURCE_COLLECTION,
+                                     NamespaceString(boost::none, "TestDB.collection"));
 
     // Snapshot transactions delay shared locks as well.
     locker.setSharedLocksShouldTwoPhaseLock(true);
 
     locker.beginWriteUnitOfWork();
     // Lock some stuff in IS mode.
-    locker.lockGlobal(MODE_IS);
+    locker.lockGlobal(opCtx.get(), MODE_IS);
     locker.lock(resIdDatabase, MODE_IS);
     locker.lock(resIdCollection, MODE_IS);
     // Unlock them.
@@ -567,7 +599,8 @@ TEST_F(LockerImplTest, releaseAndRestoreReadOnlyWriteUnitOfWork) {
 
 TEST_F(LockerImplTest, releaseAndRestoreEmptyWriteUnitOfWork) {
     Locker::LockSnapshot lockInfo;
-    LockerImpl locker;
+    auto opCtx = makeOperationContext();
+    LockerImpl locker(opCtx->getServiceContext());
 
     // Snapshot transactions delay shared locks as well.
     locker.setSharedLocksShouldTwoPhaseLock(true);
@@ -587,26 +620,29 @@ TEST_F(LockerImplTest, releaseAndRestoreEmptyWriteUnitOfWork) {
 }
 
 TEST_F(LockerImplTest, releaseAndRestoreWriteUnitOfWorkWithRecursiveLocks) {
+    auto opCtx = makeOperationContext();
+
     Locker::LockSnapshot lockInfo;
 
-    LockerImpl locker;
+    LockerImpl locker(opCtx->getServiceContext());
 
-    const ResourceId resIdDatabase(RESOURCE_DATABASE, "TestDB"_sd);
-    const ResourceId resIdCollection(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    const ResourceId resIdDatabase(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB"));
+    const ResourceId resIdCollection(RESOURCE_COLLECTION,
+                                     NamespaceString(boost::none, "TestDB.collection"));
 
     locker.beginWriteUnitOfWork();
     // Lock some stuff.
-    locker.lockGlobal(MODE_IX);
+    locker.lockGlobal(opCtx.get(), MODE_IX);
     locker.lock(resIdDatabase, MODE_IX);
-    locker.lock(resIdCollection, MODE_X);
+    locker.lock(resIdCollection, MODE_IX);
     // Recursively lock them again with a weaker mode.
-    locker.lockGlobal(MODE_IS);
+    locker.lockGlobal(opCtx.get(), MODE_IS);
     locker.lock(resIdDatabase, MODE_IS);
-    locker.lock(resIdCollection, MODE_S);
+    locker.lock(resIdCollection, MODE_IS);
 
     // Make sure locks are converted.
     ASSERT_EQUALS(MODE_IX, locker.getLockMode(resIdDatabase));
-    ASSERT_EQUALS(MODE_X, locker.getLockMode(resIdCollection));
+    ASSERT_EQUALS(MODE_IX, locker.getLockMode(resIdCollection));
     ASSERT_TRUE(locker.isWriteLocked());
     ASSERT_EQ(locker.getRequestsForTest().find(resourceIdGlobal).objAddr()->recursiveCount, 2U);
     ASSERT_EQ(locker.getRequestsForTest().find(resIdDatabase).objAddr()->recursiveCount, 2U);
@@ -618,7 +654,7 @@ TEST_F(LockerImplTest, releaseAndRestoreWriteUnitOfWorkWithRecursiveLocks) {
     ASSERT_FALSE(locker.unlockGlobal());
     // Make sure locks are still acquired in the correct mode.
     ASSERT_EQUALS(MODE_IX, locker.getLockMode(resIdDatabase));
-    ASSERT_EQUALS(MODE_X, locker.getLockMode(resIdCollection));
+    ASSERT_EQUALS(MODE_IX, locker.getLockMode(resIdCollection));
     ASSERT_TRUE(locker.isWriteLocked());
     // Make sure unlocking converted locks decrements the locks' recursiveCount instead of
     // incrementing unlockPending.
@@ -652,7 +688,7 @@ TEST_F(LockerImplTest, releaseAndRestoreWriteUnitOfWorkWithRecursiveLocks) {
 
     // Make sure things were re-locked in the correct mode.
     ASSERT_EQUALS(MODE_IX, locker.getLockMode(resIdDatabase));
-    ASSERT_EQUALS(MODE_X, locker.getLockMode(resIdCollection));
+    ASSERT_EQUALS(MODE_IX, locker.getLockMode(resIdCollection));
     ASSERT_TRUE(locker.isWriteLocked());
     // Make sure locks were coalesced after restore and are pending to unlock as before.
     ASSERT_EQ(locker.getRequestsForTest().find(resourceIdGlobal).objAddr()->recursiveCount, 1U);
@@ -670,10 +706,12 @@ TEST_F(LockerImplTest, releaseAndRestoreWriteUnitOfWorkWithRecursiveLocks) {
 }
 
 TEST_F(LockerImplTest, DefaultLocker) {
-    const ResourceId resId(RESOURCE_DATABASE, "TestDB"_sd);
+    auto opCtx = makeOperationContext();
 
-    LockerImpl locker;
-    locker.lockGlobal(MODE_IX);
+    const ResourceId resId(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB"));
+
+    LockerImpl locker(opCtx->getServiceContext());
+    locker.lockGlobal(opCtx.get(), MODE_IX);
     locker.lock(resId, MODE_X);
 
     // Make sure only Global and TestDB resources are locked.
@@ -692,15 +730,19 @@ TEST_F(LockerImplTest, SharedLocksShouldTwoPhaseLockIsTrue) {
     // and S locks are postponed until endWriteUnitOfWork() is called. Mode IX and X locks always
     // participate in two-phased locking, regardless of the setting.
 
-    const ResourceId resId1(RESOURCE_DATABASE, "TestDB1"_sd);
-    const ResourceId resId2(RESOURCE_DATABASE, "TestDB2"_sd);
-    const ResourceId resId3(RESOURCE_COLLECTION, "TestDB.collection3"_sd);
-    const ResourceId resId4(RESOURCE_COLLECTION, "TestDB.collection4"_sd);
+    auto opCtx = makeOperationContext();
 
-    LockerImpl locker;
+    const ResourceId resId1(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB1"));
+    const ResourceId resId2(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB2"));
+    const ResourceId resId3(RESOURCE_COLLECTION,
+                            NamespaceString(boost::none, "TestDB.collection3"));
+    const ResourceId resId4(RESOURCE_COLLECTION,
+                            NamespaceString(boost::none, "TestDB.collection4"));
+
+    LockerImpl locker(opCtx->getServiceContext());
     locker.setSharedLocksShouldTwoPhaseLock(true);
 
-    locker.lockGlobal(MODE_IS);
+    locker.lockGlobal(opCtx.get(), MODE_IS);
     ASSERT_EQ(locker.getLockMode(resourceIdGlobal), MODE_IS);
 
     locker.lock(resourceIdReplicationStateTransitionLock, MODE_IS);
@@ -746,14 +788,18 @@ TEST_F(LockerImplTest, ModeIXAndXLockParticipatesInTwoPhaseLocking) {
     // Unlock on mode IX and X locks during a WUOW should always be postponed until
     // endWriteUnitOfWork() is called. Mode IS and S locks should unlock immediately.
 
-    const ResourceId resId1(RESOURCE_DATABASE, "TestDB1"_sd);
-    const ResourceId resId2(RESOURCE_DATABASE, "TestDB2"_sd);
-    const ResourceId resId3(RESOURCE_COLLECTION, "TestDB.collection3"_sd);
-    const ResourceId resId4(RESOURCE_COLLECTION, "TestDB.collection4"_sd);
+    auto opCtx = makeOperationContext();
 
-    LockerImpl locker;
+    const ResourceId resId1(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB1"));
+    const ResourceId resId2(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB2"));
+    const ResourceId resId3(RESOURCE_COLLECTION,
+                            NamespaceString(boost::none, "TestDB.collection3"));
+    const ResourceId resId4(RESOURCE_COLLECTION,
+                            NamespaceString(boost::none, "TestDB.collection4"));
 
-    locker.lockGlobal(MODE_IX);
+    LockerImpl locker(opCtx->getServiceContext());
+
+    locker.lockGlobal(opCtx.get(), MODE_IX);
     ASSERT_EQ(locker.getLockMode(resourceIdGlobal), MODE_IX);
 
     locker.lock(resourceIdReplicationStateTransitionLock, MODE_IX);
@@ -794,7 +840,8 @@ TEST_F(LockerImplTest, ModeIXAndXLockParticipatesInTwoPhaseLocking) {
 }
 
 TEST_F(LockerImplTest, RSTLUnlocksWithNestedLock) {
-    LockerImpl locker;
+    auto opCtx = makeOperationContext();
+    LockerImpl locker(opCtx->getServiceContext());
 
     locker.lock(resourceIdReplicationStateTransitionLock, MODE_IX);
     ASSERT_EQ(locker.getLockMode(resourceIdReplicationStateTransitionLock), MODE_IX);
@@ -819,7 +866,8 @@ TEST_F(LockerImplTest, RSTLUnlocksWithNestedLock) {
 }
 
 TEST_F(LockerImplTest, RSTLModeIXWithTwoPhaseLockingCanBeUnlockedWhenPrepared) {
-    LockerImpl locker;
+    auto opCtx = makeOperationContext();
+    LockerImpl locker(opCtx->getServiceContext());
 
     locker.lock(resourceIdReplicationStateTransitionLock, MODE_IX);
     ASSERT_EQ(locker.getLockMode(resourceIdReplicationStateTransitionLock), MODE_IX);
@@ -843,7 +891,8 @@ TEST_F(LockerImplTest, RSTLModeIXWithTwoPhaseLockingCanBeUnlockedWhenPrepared) {
 }
 
 TEST_F(LockerImplTest, RSTLModeISWithTwoPhaseLockingCanBeUnlockedWhenPrepared) {
-    LockerImpl locker;
+    auto opCtx = makeOperationContext();
+    LockerImpl locker(opCtx->getServiceContext());
 
     locker.lock(resourceIdReplicationStateTransitionLock, MODE_IS);
     ASSERT_EQ(locker.getLockMode(resourceIdReplicationStateTransitionLock), MODE_IS);
@@ -864,7 +913,8 @@ TEST_F(LockerImplTest, RSTLModeISWithTwoPhaseLockingCanBeUnlockedWhenPrepared) {
 }
 
 TEST_F(LockerImplTest, RSTLTwoPhaseLockingBehaviorModeIS) {
-    LockerImpl locker;
+    auto opCtx = makeOperationContext();
+    LockerImpl locker(opCtx->getServiceContext());
 
     locker.lock(resourceIdReplicationStateTransitionLock, MODE_IS);
     ASSERT_EQ(locker.getLockMode(resourceIdReplicationStateTransitionLock), MODE_IS);
@@ -885,17 +935,19 @@ TEST_F(LockerImplTest, RSTLTwoPhaseLockingBehaviorModeIS) {
 }
 
 TEST_F(LockerImplTest, OverrideLockRequestTimeout) {
-    const ResourceId resIdFirstDB(RESOURCE_DATABASE, "FirstDB"_sd);
-    const ResourceId resIdSecondDB(RESOURCE_DATABASE, "SecondDB"_sd);
+    auto opCtx = makeOperationContext();
 
-    LockerImpl locker1;
-    LockerImpl locker2;
+    const ResourceId resIdFirstDB(RESOURCE_DATABASE, DatabaseName(boost::none, "FirstDB"));
+    const ResourceId resIdSecondDB(RESOURCE_DATABASE, DatabaseName(boost::none, "SecondDB"));
+
+    LockerImpl locker1(opCtx->getServiceContext());
+    LockerImpl locker2(opCtx->getServiceContext());
 
     // Set up locker2 to override lock requests' provided timeout if greater than 1000 milliseconds.
     locker2.setMaxLockTimeout(Milliseconds(1000));
 
-    locker1.lockGlobal(MODE_IX);
-    locker2.lockGlobal(MODE_IX);
+    locker1.lockGlobal(opCtx.get(), MODE_IX);
+    locker2.lockGlobal(opCtx.get(), MODE_IX);
 
     // locker1 acquires FirstDB under an exclusive lock.
     locker1.lock(resIdFirstDB, MODE_X);
@@ -903,7 +955,6 @@ TEST_F(LockerImplTest, OverrideLockRequestTimeout) {
 
     // locker2's attempt to acquire FirstDB with unlimited wait time should timeout after 1000
     // milliseconds and throw because _maxLockRequestTimeout is set to 1000 milliseconds.
-    auto opCtx = makeOperationContext();
     ASSERT_THROWS_CODE(locker2.lock(opCtx.get(), resIdFirstDB, MODE_X, Date_t::max()),
                        AssertionException,
                        ErrorCodes::LockTimeout);
@@ -921,18 +972,20 @@ TEST_F(LockerImplTest, OverrideLockRequestTimeout) {
 }
 
 TEST_F(LockerImplTest, DoNotWaitForLockAcquisition) {
-    const ResourceId resIdFirstDB(RESOURCE_DATABASE, "FirstDB"_sd);
-    const ResourceId resIdSecondDB(RESOURCE_DATABASE, "SecondDB"_sd);
+    auto opCtx = makeOperationContext();
 
-    LockerImpl locker1;
-    LockerImpl locker2;
+    const ResourceId resIdFirstDB(RESOURCE_DATABASE, DatabaseName(boost::none, "FirstDB"));
+    const ResourceId resIdSecondDB(RESOURCE_DATABASE, DatabaseName(boost::none, "SecondDB"));
+
+    LockerImpl locker1(opCtx->getServiceContext());
+    LockerImpl locker2(opCtx->getServiceContext());
 
     // Set up locker2 to immediately return if a lock is unavailable, regardless of supplied
     // deadlines in the lock request.
     locker2.setMaxLockTimeout(Milliseconds(0));
 
-    locker1.lockGlobal(MODE_IX);
-    locker2.lockGlobal(MODE_IX);
+    locker1.lockGlobal(opCtx.get(), MODE_IX);
+    locker2.lockGlobal(opCtx.get(), MODE_IX);
 
     // locker1 acquires FirstDB under an exclusive lock.
     locker1.lock(resIdFirstDB, MODE_X);
@@ -940,7 +993,6 @@ TEST_F(LockerImplTest, DoNotWaitForLockAcquisition) {
 
     // locker2's attempt to acquire FirstDB with unlimited wait time should fail immediately and
     // throw because _maxLockRequestTimeout was set to 0.
-    auto opCtx = makeOperationContext();
     ASSERT_THROWS_CODE(locker2.lock(opCtx.get(), resIdFirstDB, MODE_X, Date_t::max()),
                        AssertionException,
                        ErrorCodes::LockTimeout);
@@ -975,12 +1027,15 @@ bool lockerInfoContainsLock(const Locker::LockerInfo& lockerInfo,
 }  // namespace
 
 TEST_F(LockerImplTest, GetLockerInfoShouldReportHeldLocks) {
-    const ResourceId dbId(RESOURCE_DATABASE, "TestDB"_sd);
-    const ResourceId collectionId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    auto opCtx = makeOperationContext();
+
+    const ResourceId dbId(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB"));
+    const ResourceId collectionId(RESOURCE_COLLECTION,
+                                  NamespaceString(boost::none, "TestDB.collection"));
 
     // Take an exclusive lock on the collection.
-    LockerImpl locker;
-    locker.lockGlobal(MODE_IX);
+    LockerImpl locker(opCtx->getServiceContext());
+    locker.lockGlobal(opCtx.get(), MODE_IX);
     locker.lock(dbId, MODE_IX);
     locker.lock(collectionId, MODE_X);
 
@@ -999,18 +1054,21 @@ TEST_F(LockerImplTest, GetLockerInfoShouldReportHeldLocks) {
 }
 
 TEST_F(LockerImplTest, GetLockerInfoShouldReportPendingLocks) {
-    const ResourceId dbId(RESOURCE_DATABASE, "TestDB"_sd);
-    const ResourceId collectionId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    auto opCtx = makeOperationContext();
+
+    const ResourceId dbId(RESOURCE_DATABASE, DatabaseName(boost::none, "TestDB"));
+    const ResourceId collectionId(RESOURCE_COLLECTION,
+                                  NamespaceString(boost::none, "TestDB.collection"));
 
     // Take an exclusive lock on the collection.
-    LockerImpl successfulLocker;
-    successfulLocker.lockGlobal(MODE_IX);
+    LockerImpl successfulLocker(opCtx->getServiceContext());
+    successfulLocker.lockGlobal(opCtx.get(), MODE_IX);
     successfulLocker.lock(dbId, MODE_IX);
     successfulLocker.lock(collectionId, MODE_X);
 
     // Now attempt to get conflicting locks.
-    LockerImpl conflictingLocker;
-    conflictingLocker.lockGlobal(MODE_IS);
+    LockerImpl conflictingLocker(opCtx->getServiceContext());
+    conflictingLocker.lockGlobal(opCtx.get(), MODE_IS);
     conflictingLocker.lock(dbId, MODE_IS);
     ASSERT_EQ(LOCK_WAITING,
               conflictingLocker.lockBeginForTest(nullptr /* opCtx */, collectionId, MODE_IS));
@@ -1043,10 +1101,12 @@ TEST_F(LockerImplTest, GetLockerInfoShouldReportPendingLocks) {
 }
 
 TEST_F(LockerImplTest, ReaquireLockPendingUnlock) {
-    const ResourceId resId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    auto opCtx = makeOperationContext();
 
-    LockerImpl locker;
-    locker.lockGlobal(MODE_IS);
+    const ResourceId resId(RESOURCE_COLLECTION, NamespaceString(boost::none, "TestDB.collection"));
+
+    LockerImpl locker(opCtx->getServiceContext());
+    locker.lockGlobal(opCtx.get(), MODE_IS);
 
     locker.lock(resId, MODE_X);
     ASSERT_TRUE(locker.isLockHeldForMode(resId, MODE_X));
@@ -1071,10 +1131,12 @@ TEST_F(LockerImplTest, ReaquireLockPendingUnlock) {
 }
 
 TEST_F(LockerImplTest, AcquireLockPendingUnlockWithCoveredMode) {
-    const ResourceId resId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    auto opCtx = makeOperationContext();
 
-    LockerImpl locker;
-    locker.lockGlobal(MODE_IS);
+    const ResourceId resId(RESOURCE_COLLECTION, NamespaceString(boost::none, "TestDB.collection"));
+
+    LockerImpl locker(opCtx->getServiceContext());
+    locker.lockGlobal(opCtx.get(), MODE_IS);
 
     locker.lock(resId, MODE_X);
     ASSERT_TRUE(locker.isLockHeldForMode(resId, MODE_X));
@@ -1099,10 +1161,12 @@ TEST_F(LockerImplTest, AcquireLockPendingUnlockWithCoveredMode) {
 }
 
 TEST_F(LockerImplTest, ConvertLockPendingUnlock) {
-    const ResourceId resId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    auto opCtx = makeOperationContext();
 
-    LockerImpl locker;
-    locker.lockGlobal(MODE_IS);
+    const ResourceId resId(RESOURCE_COLLECTION, NamespaceString(boost::none, "TestDB.collection"));
+
+    LockerImpl locker(opCtx->getServiceContext());
+    locker.lockGlobal(opCtx.get(), MODE_IS);
 
     locker.lock(resId, MODE_IX);
     ASSERT_TRUE(locker.isLockHeldForMode(resId, MODE_IX));
@@ -1131,10 +1195,12 @@ TEST_F(LockerImplTest, ConvertLockPendingUnlock) {
 }
 
 TEST_F(LockerImplTest, ConvertLockPendingUnlockAndUnlock) {
-    const ResourceId resId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    auto opCtx = makeOperationContext();
 
-    LockerImpl locker;
-    locker.lockGlobal(MODE_IS);
+    const ResourceId resId(RESOURCE_COLLECTION, NamespaceString(boost::none, "TestDB.collection"));
+
+    LockerImpl locker(opCtx->getServiceContext());
+    locker.lockGlobal(opCtx.get(), MODE_IS);
 
     locker.lock(resId, MODE_IX);
     ASSERT_TRUE(locker.isLockHeldForMode(resId, MODE_IX));
@@ -1171,15 +1237,40 @@ TEST_F(LockerImplTest, ConvertLockPendingUnlockAndUnlock) {
     locker.unlockGlobal();
 }
 
+TEST_F(LockerImplTest, SkipTicketAcquisitionForLockRAIIType) {
+    auto opCtx = makeOperationContext();
+
+    // By default, ticket acquisition is required.
+    ASSERT_TRUE(opCtx->lockState()->shouldAcquireTicket());
+
+    {
+        SkipTicketAcquisitionForLock skipTicketAcquisition(opCtx.get());
+        ASSERT_FALSE(opCtx->lockState()->shouldAcquireTicket());
+    }
+
+    ASSERT_TRUE(opCtx->lockState()->shouldAcquireTicket());
+
+    // If ticket acquisitions are disabled on the lock state, the RAII type has no effect.
+    opCtx->lockState()->skipAcquireTicket();
+    ASSERT_FALSE(opCtx->lockState()->shouldAcquireTicket());
+
+    {
+        SkipTicketAcquisitionForLock skipTicketAcquisition(opCtx.get());
+        ASSERT_FALSE(opCtx->lockState()->shouldAcquireTicket());
+    }
+
+    ASSERT_FALSE(opCtx->lockState()->shouldAcquireTicket());
+}
+
 // This test exercises the lock dumping code in ~LockerImpl in case locks are held on destruction.
 DEATH_TEST_F(LockerImplTest,
              LocksHeldOnDestructionCausesALocksDump,
              "Operation ending while holding locks.") {
     auto opCtx = makeOperationContext();
 
-    const ResourceId resId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
+    const ResourceId resId(RESOURCE_COLLECTION, NamespaceString(boost::none, "TestDB.collection"));
 
-    LockerImpl locker;
+    LockerImpl locker(opCtx->getServiceContext());
     locker.lockGlobal(opCtx.get(), MODE_IX);
     locker.lock(resId, MODE_X);
 
