@@ -30,8 +30,11 @@
 
 #pragma once
 
+#include <memory>
+
 #include "mongo/config.h"
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/user_acquisition_stats.h"
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/clientcursor.h"
 #include "mongo/db/commands.h"
@@ -39,11 +42,13 @@
 #include "mongo/db/operation_context.h"
 #include "mongo/db/profile_filter.h"
 #include "mongo/db/server_options.h"
+#include "mongo/db/stats/resource_consumption_metrics.h"
 #include "mongo/db/write_concern_options.h"
 #include "mongo/logv2/attribute_storage.h"
 #include "mongo/logv2/log_component.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/util/progress_meter.h"
+#include "mongo/util/tick_source.h"
 #include "mongo/util/time_support.h"
 
 #ifndef MONGO_CONFIG_USE_RAW_LATCHES
@@ -99,6 +104,11 @@ public:
          */
         void incrementWriteConflicts(long long n);
 
+        /**
+         * Increments temporarilyUnavailableErrors by n.
+         */
+        void incrementTemporarilyUnavailableErrors(long long n);
+
         /**
          * Increments keysInserted by n.
          */
@@ -114,6 +124,11 @@ public:
          */
         void incrementNinserted(long long n);
 
+        /**
+         * Increments nUpserted by n.
+         */
+        void incrementNUpserted(long long n);
+
         /**
          * Increments prepareReadConflicts by n.
          */
@@ -138,6 +153,7 @@ public:
         boost::optional<long long> nModified;
         boost::optional<long long> ninserted;
         boost::optional<long long> ndeleted;
+        boost::optional<long long> nUpserted;
 
         // Number of index keys inserted.
         boost::optional<long long> keysInserted;
@@ -152,14 +168,14 @@ public:
         // Number of read conflicts caused by a prepared transaction.
         AtomicWord<long long> prepareReadConflicts{0};
         AtomicWord<long long> writeConflicts{0};
+        AtomicWord<long long> temporarilyUnavailableErrors{0};
     };
 
     OpDebug() = default;
 
-    std::string report(OperationContext* opCtx, const SingleThreadedLockStats* lockStats) const;
-
     void report(OperationContext* opCtx,
                 const SingleThreadedLockStats* lockStats,
+                const ResourceConsumption::OperationMetrics* operationMetrics,
                 logv2::DynamicAttributes* pAttrs) const;
 
     /**
@@ -192,6 +208,18 @@ public:
      */
     BSONObj makeMongotDebugStatsObject() const;
 
+    /**
+     * Accumulate resolved views.
+     */
+    void addResolvedViews(const std::vector<NamespaceString>& namespaces,
+                          const std::vector<BSONObj>& pipeline);
+
+    /**
+     * Get or append the array with resolved views' info.
+     */
+    BSONArray getResolvedViewsInfo() const;
+    void appendResolvedViewsInfo(BSONObjBuilder& builder) const;
+
     // -------------------
 
     // basic options
@@ -205,13 +233,12 @@ public:
 
     // detailed options
     long long cursorid{-1};
-    long long ntoreturn{-1};
-    long long ntoskip{-1};
     bool exhaust{false};
 
     // For search using mongot.
     boost::optional<long long> mongotCursorId{boost::none};
     boost::optional<long long> msWaitingForMongot{boost::none};
+    long long mongotBatchNum = 0;
 
     bool hasSortStage{false};  // true if the query plan involves an in-memory sort
 
@@ -222,9 +249,8 @@ public:
     bool fromMultiPlanner{false};
 
     // True if a replan was triggered during the execution of this operation.
-    std::optional<std::string> replanReason;
+    boost::optional<std::string> replanReason;
 
-    bool upsert{false};  // true if the update actually did an insert
     bool cursorExhausted{
         false};  // true if the cursor has been closed at end a find/getMore operation
 
@@ -236,11 +262,35 @@ public:
     // The hash of the query's "stable" key. This represents the query's shape.
     boost::optional<uint32_t> queryHash;
 
+    // Has a value if this operation is a query. True if the execution tree for the find part of the
+    // query was built exclusively using the classic query engine, false if any part was built using
+    // SBE.
+    boost::optional<bool> classicEngineUsed;
+
+    // Has a value if this operation is an aggregation query. True if `DocumentSources` were
+    // involved in the execution tree for this query, false if they were not.
+    boost::optional<bool> documentSourceUsed;
+
+    // Indicates whether this operation used the common query framework (CQF).
+    bool cqfUsed{false};
+
+    // Tracks the amount of indexed loop joins in a pushed down lookup stage.
+    int indexedLoopJoin{0};
+
+    // Tracks the amount of nested loop joins in a pushed down lookup stage.
+    int nestedLoopJoin{0};
+
+    // Tracks the amount of hash lookups in a pushed down lookup stage.
+    int hashLookup{0};
+
+    // Tracks the amount of spills by hash lookup in a pushed down lookup stage.
+    int hashLookupSpillToDisk{0};
+
     // Details of any error (whether from an exception or a command returning failure).
     Status errInfo = Status::OK();
 
     // response info
-    long long executionTimeMicros{0};
+    Microseconds executionTime{0};
     long long nreturned{-1};
     int responseLength{-1};
 
@@ -250,10 +300,21 @@ public:
     // Stores the duration of time spent blocked on prepare conflicts.
     Milliseconds prepareConflictDurationMillis{0};
 
-    // Stores the total time an operation spends with an uncommitted oplog slot held open. Indicator
-    // that an operation is holding back replication by causing oplog holes to remain open for
-    // unusual amounts of time.
-    Microseconds totalOplogSlotDurationMicros{0};
+    // Total time spent looking up database entry in the local catalog cache, including eventual
+    // refreshes.
+    Milliseconds catalogCacheDatabaseLookupMillis{0};
+
+    // Total time spent looking up collection entry in the local catalog cache, including eventual
+    // refreshes.
+    Milliseconds catalogCacheCollectionLookupMillis{0};
+
+    // Stores the duration of time spent waiting for the shard to refresh the database and wait for
+    // the database critical section.
+    Milliseconds databaseVersionRefreshMillis{0};
+
+    // Stores the duration of time spent waiting for the shard to refresh the collection and wait
+    // for the collection critical section.
+    Milliseconds shardVersionRefreshMillis{0};
 
     // Stores the amount of the data processed by the throttle cursors in MB/sec.
     boost::optional<float> dataThroughputLastSecond;
@@ -276,6 +337,12 @@ public:
 
     // Whether this is an oplog getMore operation for replication oplog fetching.
     bool isReplOplogGetMore{false};
+
+    // Maps namespace of a resolved view to its dependency chain and the fully unrolled pipeline. To
+    // make log line deterministic and easier to test, use ordered map. As we don't expect many
+    // resolved views per query, a hash map would unlikely provide any benefits.
+    std::map<NamespaceString, std::pair<std::vector<NamespaceString>, std::vector<BSONObj>>>
+        resolvedViews;
 };
 
 /**
@@ -417,6 +484,21 @@ public:
         return _ns;
     }
 
+    /**
+     * Returns a const pointer to the UserAcquisitionStats for the current operation.
+     * This can only be used for reading (i.e., when logging or profiling).
+     */
+    const UserAcquisitionStats* getReadOnlyUserAcquisitionStats() const {
+        return &_userAcquisitionStats;
+    }
+
+    /**
+     * Returns a non-const raw pointers to UserAcquisitionStats member.
+     */
+    UserAcquisitionStats* getMutableUserAcquisitionStats() {
+        return &_userAcquisitionStats;
+    }
+
     /**
      * Gets the name of the namespace on which the current operation operates.
      */
@@ -439,7 +521,7 @@ public:
         if (_dbprofile <= 0)
             return false;
 
-        if (CollectionCatalog::get(opCtx).getDatabaseProfileSettings(getNSS().db()).filter)
+        if (CollectionCatalog::get(opCtx)->getDatabaseProfileSettings(getNSS().db()).filter)
             return true;
 
         return elapsedTimeExcludingPauses() >= Milliseconds{serverGlobalParams.slowMS};
@@ -453,6 +535,10 @@ public:
      */
     void raiseDbProfileLevel(int dbProfileLevel);
 
+    int dbProfileLevel() const {
+        return _dbprofile;
+    }
+
     /**
      * Gets the network operation type. No lock is required if called by the thread executing
      * the operation, but the lock must be held if called from another thread.
@@ -469,13 +555,6 @@ public:
         return _logicalOp;
     }
 
-    /**
-     * Returns true if this CurOp represents a non-command OP_QUERY request.
-     */
-    bool isLegacyQuery() const {
-        return _networkOp == NetworkOp::dbQuery && !isCommand();
-    }
-
     /**
      * Returns true if the current operation is known to be a command.
      */
@@ -488,17 +567,13 @@ public:
     // negative, if the system time has been reset during the course of this operation.
     //
 
-    void ensureStarted();
+    void ensureStarted() {
+        static_cast<void>(startTime());
+    }
     bool isStarted() const {
-        return _start > 0;
-    }
-    long long startTime() {  // micros
-        ensureStarted();
-        return _start;
-    }
-    void done() {
-        _end = curTimeMicros64();
+        return _start.load() != 0;
     }
+    void done();
     bool isDone() const {
         return _end > 0;
     }
@@ -516,7 +591,7 @@ public:
     void pauseTimer() {
         invariant(isStarted());
         invariant(_lastPauseTime == 0);
-        _lastPauseTime = curTimeMicros64();
+        _lastPauseTime = _tickSource->getTicks();
     }
 
     /**
@@ -527,7 +602,7 @@ public:
         invariant(isStarted());
         invariant(_lastPauseTime > 0);
         _totalPausedDuration +=
-            Microseconds{static_cast<long long>(curTimeMicros64()) - _lastPauseTime};
+            _tickSource->ticksTo<Microseconds>(_tickSource->getTicks() - _lastPauseTime);
         _lastPauseTime = 0;
     }
 
@@ -575,7 +650,9 @@ public:
         if (_debug.remoteOpWaitTime) {
             Microseconds end = elapsedTimeTotal();
             invariant(_remoteOpStartTime);
-            // A backward shift of the realtime system clock could lead to a negative delta.
+            // On most systems a monotonic clock source will be used to measure time. When a
+            // monotonic clock is not available we fallback to using the realtime system clock. When
+            // used, a backward shift of the realtime system clock could lead to a negative delta.
             Microseconds delta = std::max((end - *_remoteOpStartTime), Microseconds{0});
             *_debug.remoteOpWaitTime += delta;
             _remoteOpStartTime = boost::none;
@@ -592,15 +669,12 @@ public:
      * If this op has not yet been started, returns 0.
      */
     Microseconds elapsedTimeTotal() {
-        if (!isStarted()) {
+        auto start = _start.load();
+        if (start == 0) {
             return Microseconds{0};
         }
 
-        if (!_end) {
-            return Microseconds{static_cast<long long>(curTimeMicros64() - startTime())};
-        } else {
-            return Microseconds{static_cast<long long>(_end - startTime())};
-        }
+        return computeElapsedTimeTotal(start, _end.load());
     }
 
     /**
@@ -614,11 +688,13 @@ public:
      */
     Microseconds elapsedTimeExcludingPauses() {
         invariant(!_lastPauseTime);
-        if (!isStarted()) {
+
+        auto start = _start.load();
+        if (start == 0) {
             return Microseconds{0};
         }
 
-        return elapsedTimeTotal() - _totalPausedDuration;
+        return computeElapsedTimeTotal(start, _end.load()) - _totalPausedDuration;
     }
 
     /**
@@ -658,6 +734,13 @@ public:
      */
     void reportState(OperationContext* opCtx, BSONObjBuilder* builder, bool truncateOps = false);
 
+    /**
+     * Sets the message for FailPoints used.
+     */
+    void setFailPointMessage_inlock(StringData message) {
+        _failPointMessage = message.toString();
+    }
+
     /**
      * Sets the message for this CurOp.
      */
@@ -673,6 +756,14 @@ public:
     ProgressMeter& setProgress_inlock(StringData name,
                                       unsigned long long progressMeterTotal = 0,
                                       int secondsBetween = 3);
+
+    /*
+     * Gets the message for FailPoints used.
+     */
+    const std::string& getFailPointMessage() const {
+        return _failPointMessage;
+    }
+
     /**
      * Gets the message for this CurOp.
      */
@@ -690,15 +781,15 @@ public:
     }
 
     void yielded(int numYields = 1) {
-        _numYields += numYields;
-    }  // Should be _inlock()?
+        _numYields.fetchAndAdd(numYields);
+    }
 
     /**
      * Returns the number of times yielded() was called.  Callers on threads other
      * than the one executing the operation must lock the client.
      */
     int numYields() const {
-        return _numYields;
+        return _numYields.load();
     }
 
     /**
@@ -726,9 +817,32 @@ public:
         return _lockStatsBase;
     }
 
+    void setTickSource_forTest(TickSource* tickSource) {
+        _tickSource = tickSource;
+    }
+
+    /**
+     * Merge match counters from the current operation into the global map and stop counting.
+     */
+    void stopMatchExprCounter();
+
+    /**
+     * Increment the counter for the match expression with given name in the current operation.
+     */
+    void incrementMatchExprCounter(StringData name);
+
 private:
     class CurOpStack;
 
+    TickSource::Tick startTime();
+    Microseconds computeElapsedTimeTotal(TickSource::Tick startTime,
+                                         TickSource::Tick endTime) const;
+
+    /**
+     * Adds 'this' to the stack of active CurOp objects.
+     */
+    void _finishInit(OperationContext* opCtx, CurOpStack* stack);
+
     /**
      * Handles failpoints that check whether a command has completed or not.
      * Used for testing purposes instead of the getLog command.
@@ -744,14 +858,14 @@ private:
     const Command* _command{nullptr};
 
     // The time at which this CurOp instance was marked as started.
-    long long _start{0};
+    std::atomic<TickSource::Tick> _start{0};  // NOLINT
 
-    // The time at which this CurOp instance was marked as done.
-    long long _end{0};
+    // The time at which this CurOp instance was marked as done or 0 if the CurOp is not yet done.
+    std::atomic<TickSource::Tick> _end{0};  // NOLINT
 
     // The time at which this CurOp instance had its timer paused, or 0 if the timer is not
     // currently paused.
-    long long _lastPauseTime{0};
+    TickSource::Tick _lastPauseTime{0};
 
     // The cumulative duration for which the timer has been paused.
     Microseconds _totalPausedDuration{0};
@@ -773,28 +887,20 @@ private:
     BSONObj _opDescription;
     BSONObj _originatingCommand;  // Used by getMore to display original command.
     OpDebug _debug;
+    std::string _failPointMessage;  // Used to store FailPoint information.
     std::string _message;
     ProgressMeter _progressMeter;
-    int _numYields{0};
+    AtomicWord<int> _numYields{0};
     // A GenericCursor containing information about the active cursor for a getMore operation.
     boost::optional<GenericCursor> _genericCursor;
 
     std::string _planSummary;
     boost::optional<SingleThreadedLockStats>
         _lockStatsBase;  // This is the snapshot of lock stats taken when curOp is constructed.
+
+    UserAcquisitionStats _userAcquisitionStats;
+
+    TickSource* _tickSource = nullptr;
 };
 
-/**
- * Upconverts a legacy query object such that it matches the format of the find command.
- */
-BSONObj upconvertQueryEntry(const BSONObj& query,
-                            const NamespaceString& nss,
-                            int ntoreturn,
-                            int ntoskip);
-
-/**
- * Generates a getMore command object from the specified namespace, cursor ID and batchsize.
- */
-BSONObj upconvertGetMoreEntry(const NamespaceString& nss, CursorId cursorId, int ntoreturn);
-
 }  // namespace mongo
