@@ -27,9 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
-
-#include "mongo/platform/basic.h"
 
 #include "mongo/db/cursor_manager.h"
 
@@ -37,28 +34,41 @@
 
 #include "mongo/base/data_cursor.h"
 #include "mongo/base/init.h"
+#include "mongo/db/allocate_cursor_id.h"
 #include "mongo/db/audit.h"
+#include "mongo/db/auth/authorization_checks.h"
 #include "mongo/db/auth/authorization_session.h"
-#include "mongo/db/background.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/database.h"
-#include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/client.h"
+#include "mongo/db/curop.h"
 #include "mongo/db/cursor_server_params.h"
 #include "mongo/db/db_raii.h"
-#include "mongo/db/kill_sessions_common.h"
-#include "mongo/db/logical_session_cache.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/query/plan_executor.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/session/kill_sessions_common.h"
+#include "mongo/db/session/logical_session_cache.h"
 #include "mongo/logv2/log.h"
 #include "mongo/platform/random.h"
 #include "mongo/util/exit.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
+
+
 namespace mongo {
 
+static CounterMetric cursorStatsLifespanLessThan1Second{"cursor.lifespan.lessThan1Second"};
+static CounterMetric cursorStatsLifespanLessThan5Seconds{"cursor.lifespan.lessThan5Seconds"};
+static CounterMetric cursorStatsLifespanLessThan15Seconds{"cursor.lifespan.lessThan15Seconds"};
+static CounterMetric cursorStatsLifespanLessThan30Seconds{"cursor.lifespan.lessThan30Seconds"};
+static CounterMetric cursorStatsLifespanLessThan1Minute{"cursor.lifespan.lessThan1Minute"};
+static CounterMetric cursorStatsLifespanLessThan10Minutes{"cursor.lifespan.lessThan10Minutes"};
+static CounterMetric cursorStatsLifespanGreaterThanOrEqual10Minutes{
+    "cursor.lifespan.greaterThanOrEqual10Minutes"};
+
 constexpr int CursorManager::kNumPartitions;
 
 namespace {
@@ -68,9 +78,29 @@ const auto serviceCursorManager =
 
 ServiceContext::ConstructorActionRegisterer cursorManagerRegisterer{
     "CursorManagerRegisterer", [](ServiceContext* svcCtx) {
-        auto cursorManager = std::make_unique<CursorManager>();
+        auto cursorManager = std::make_unique<CursorManager>(svcCtx->getPreciseClockSource());
         CursorManager::set(svcCtx, std::move(cursorManager));
     }};
+
+void incrementCursorLifespanMetric(Date_t birth, Date_t death) {
+    auto elapsed = death - birth;
+
+    if (elapsed < Seconds(1)) {
+        cursorStatsLifespanLessThan1Second.increment();
+    } else if (elapsed < Seconds(5)) {
+        cursorStatsLifespanLessThan5Seconds.increment();
+    } else if (elapsed < Seconds(15)) {
+        cursorStatsLifespanLessThan15Seconds.increment();
+    } else if (elapsed < Seconds(30)) {
+        cursorStatsLifespanLessThan30Seconds.increment();
+    } else if (elapsed < Minutes(1)) {
+        cursorStatsLifespanLessThan1Minute.increment();
+    } else if (elapsed < Minutes(10)) {
+        cursorStatsLifespanLessThan10Minutes.increment();
+    } else {
+        cursorStatsLifespanGreaterThanOrEqual10Minutes.increment();
+    }
+}
 }  // namespace
 
 CursorManager* CursorManager::get(ServiceContext* svcCtx) {
@@ -90,7 +120,7 @@ void CursorManager::set(ServiceContext* svcCtx, std::unique_ptr<CursorManager> n
 std::pair<Status, int> CursorManager::killCursorsWithMatchingSessions(
     OperationContext* opCtx, const SessionKiller::Matcher& matcher) {
     auto eraser = [&](CursorManager& mgr, CursorId id) {
-        uassertStatusOK(mgr.killCursor(opCtx, id, true));
+        uassertStatusOK(mgr.killCursor(opCtx, id));
         LOGV2(20528,
               "killing cursor: {id} as part of killing session(s)",
               "Killing cursor as part of killing session(s)",
@@ -104,14 +134,16 @@ std::pair<Status, int> CursorManager::killCursorsWithMatchingSessions(
                           bySessionCursorKiller.getCursorsKilled());
 }
 
-CursorManager::CursorManager()
+CursorManager::CursorManager(ClockSource* preciseClockSource)
     : _random(std::make_unique<PseudoRandom>(SecureRandom().nextInt64())),
-      _cursorMap(std::make_unique<Partitioned<stdx::unordered_map<CursorId, ClientCursor*>>>()) {}
+      _cursorMap(std::make_unique<Partitioned<stdx::unordered_map<CursorId, ClientCursor*>>>(
+          kNumPartitions)),
+      _preciseClockSource(preciseClockSource) {}
 
 CursorManager::~CursorManager() {
     auto allPartitions = _cursorMap->lockAllPartitions();
     for (auto&& partition : allPartitions) {
-        for (auto&& cursor : partition) {
+        for (auto&& cursor : *partition) {
             // Callers must ensure that no cursors are in use.
             invariant(!cursor.second->_operationUsingCursor);
             cursor.second->dispose(nullptr);
@@ -159,9 +191,11 @@ std::size_t CursorManager::timeoutCursors(OperationContext* opCtx, Date_t now) {
     return toDisposeWithoutMutex.size();
 }
 
-StatusWith<ClientCursorPin> CursorManager::pinCursor(OperationContext* opCtx,
-                                                     CursorId id,
-                                                     AuthCheck checkSessionAuth) {
+StatusWith<ClientCursorPin> CursorManager::pinCursor(
+    OperationContext* opCtx,
+    CursorId id,
+    const std::function<void(const ClientCursor&)>& checkPinAllowed,
+    AuthCheck checkSessionAuth) {
     auto lockedPartition = _cursorMap->lockOnePartition(id);
     auto it = lockedPartition->find(id);
     if (it == lockedPartition->end()) {
@@ -188,25 +222,35 @@ StatusWith<ClientCursorPin> CursorManager::pinCursor(OperationContext* opCtx,
         }
     }
 
+    if (checkPinAllowed) {
+        checkPinAllowed(*cursor);
+    }
+
+    // Pass along the original queryHash and planCacheKey for slow query logging.
+    CurOp::get(opCtx)->debug().queryHash = cursor->_queryHash;
+    CurOp::get(opCtx)->debug().planCacheKey = cursor->_planCacheKey;
+
     cursor->_operationUsingCursor = opCtx;
 
     // We use pinning of a cursor as a proxy for active, user-initiated use of a cursor.  Therefore,
     // we pass down to the logical session cache and vivify the record (updating last use).
     if (cursor->getSessionId()) {
         auto vivifyCursorStatus =
-            LogicalSessionCache::get(opCtx)->vivify(opCtx, cursor->getSessionId().get());
+            LogicalSessionCache::get(opCtx)->vivify(opCtx, cursor->getSessionId().value());
         if (!vivifyCursorStatus.isOK()) {
             return vivifyCursorStatus;
         }
     }
 
-    return ClientCursorPin(opCtx, cursor, this);
+    auto pin = ClientCursorPin(opCtx, cursor, this);
+    pin.unstashResourcesOntoOperationContext();
+    return StatusWith(std::move(pin));
 }
 
 void CursorManager::unpin(OperationContext* opCtx,
                           std::unique_ptr<ClientCursor, ClientCursor::Deleter> cursor) {
     // Avoid computing the current time within the critical section.
-    auto now = opCtx->getServiceContext()->getPreciseClockSource()->now();
+    auto now = _preciseClockSource->now();
 
     auto partition = _cursorMap->lockOnePartition(cursor->cursorid());
     invariant(cursor->_operationUsingCursor);
@@ -241,7 +285,7 @@ void CursorManager::unpin(OperationContext* opCtx,
 void CursorManager::appendActiveSessions(LogicalSessionIdSet* lsids) const {
     auto allPartitions = _cursorMap->lockAllPartitions();
     for (auto&& partition : allPartitions) {
-        for (auto&& entry : partition) {
+        for (auto&& entry : *partition) {
             auto cursor = entry.second;
             if (auto id = cursor->getSessionId()) {
                 lsids->insert(id.value());
@@ -257,13 +301,13 @@ std::vector<GenericCursor> CursorManager::getIdleCursors(
 
     auto allPartitions = _cursorMap->lockAllPartitions();
     for (auto&& partition : allPartitions) {
-        for (auto&& entry : partition) {
+        for (auto&& entry : *partition) {
             auto cursor = entry.second;
 
             // Exclude cursors that this user does not own if auth is enabled.
             if (ctxAuth->getAuthorizationManager().isAuthEnabled() &&
                 userMode == MongoProcessInterface::CurrentOpUserMode::kExcludeOthers &&
-                !ctxAuth->isCoauthorizedWith(cursor->getAuthenticatedUsers())) {
+                !ctxAuth->isCoauthorizedWith(cursor->getAuthenticatedUser())) {
                 continue;
             }
             // Exclude pinned cursors.
@@ -282,7 +326,7 @@ stdx::unordered_set<CursorId> CursorManager::getCursorsForSession(LogicalSession
 
     auto allPartitions = _cursorMap->lockAllPartitions();
     for (auto&& partition : allPartitions) {
-        for (auto&& entry : partition) {
+        for (auto&& entry : *partition) {
             auto cursor = entry.second;
             if (cursor->getSessionId() == lsid) {
                 cursors.insert(cursor->cursorid());
@@ -309,42 +353,10 @@ size_t CursorManager::numCursors() const {
     return _cursorMap->size();
 }
 
-CursorId CursorManager::allocateCursorId_inlock() {
-    for (int i = 0; i < 10000; i++) {
-        CursorId id = _random->nextInt64();
-
-        // A cursor id of zero is reserved to indicate that the cursor has been closed. If the
-        // random number generator gives us zero, then try again.
-        if (id == 0) {
-            continue;
-        }
-
-        // Avoid negative cursor ids by taking the absolute value. If the cursor id is the minimum
-        // representable negative number, then just generate another random id.
-        if (id == std::numeric_limits<CursorId>::min()) {
-            continue;
-        }
-        id = std::abs(id);
-
-        auto partition = _cursorMap->lockOnePartition(id);
-        if (partition->count(id) == 0) {
-            // The cursor id is not already in use, so return it. Even though we drop the lock on
-            // the '_cursorMap' partition, another thread cannot register a cursor with the same id
-            // because we still hold '_registrationLock'.
-            return id;
-        }
-
-        // The cursor id is already in use. Generate another random id.
-    }
-
-    // We failed to generate a unique cursor id.
-    fassertFailed(17360);
-}
-
 ClientCursorPin CursorManager::registerCursor(OperationContext* opCtx,
                                               ClientCursorParams&& cursorParams) {
     // Avoid computing the current time within the critical section.
-    auto now = opCtx->getServiceContext()->getPreciseClockSource()->now();
+    auto now = _preciseClockSource->now();
 
     // Make sure the PlanExecutor isn't registered, since we will register the ClientCursor wrapping
     // it.
@@ -354,7 +366,15 @@ ClientCursorPin CursorManager::registerCursor(OperationContext* opCtx,
     // Note we must hold the registration lock from now until insertion into '_cursorMap' to ensure
     // we don't insert two cursors with the same cursor id.
     stdx::lock_guard<SimpleMutex> lock(_registrationLock);
-    CursorId cursorId = allocateCursorId_inlock();
+    CursorId cursorId = generic_cursor::allocateCursorId(
+        [&](CursorId cursorId) -> bool {
+            // Even though we drop the lock on the '_cursorMap' partition, another thread cannot
+            // register a cursor with the same id because we still hold '_registrationLock'.
+            auto partition = _cursorMap->lockOnePartition(cursorId);
+            return partition->count(cursorId) == 0;
+        },
+        *_random);
+
     std::unique_ptr<ClientCursor, ClientCursor::Deleter> clientCursor(
         new ClientCursor(std::move(cursorParams), cursorId, opCtx, now));
 
@@ -384,16 +404,19 @@ ClientCursorPin CursorManager::registerCursor(OperationContext* opCtx,
 
 void CursorManager::deregisterCursor(ClientCursor* cursor) {
     removeCursorFromMap(_cursorMap, cursor);
+    incrementCursorLifespanMetric(cursor->_createdDate, _preciseClockSource->now());
 }
 
 void CursorManager::deregisterAndDestroyCursor(
-    Partitioned<stdx::unordered_map<CursorId, ClientCursor*>, kNumPartitions>::OnePartition&& lk,
+    Partitioned<stdx::unordered_map<CursorId, ClientCursor*>>::OnePartition&& lk,
     OperationContext* opCtx,
     std::unique_ptr<ClientCursor, ClientCursor::Deleter> cursor) {
     {
         auto lockWithRestrictedScope = std::move(lk);
         removeCursorFromMap(lockWithRestrictedScope, cursor.get());
     }
+
+    incrementCursorLifespanMetric(cursor->_createdDate, _preciseClockSource->now());
     // Dispose of the cursor without holding any cursor manager mutexes. Disposal of a cursor can
     // require taking lock manager locks, which we want to avoid while holding a mutex. If we did
     // so, any caller of a CursorManager method which already held a lock manager lock could induce
@@ -401,13 +424,10 @@ void CursorManager::deregisterAndDestroyCursor(
     cursor->dispose(opCtx);
 }
 
-Status CursorManager::killCursor(OperationContext* opCtx, CursorId id, bool shouldAudit) {
+Status CursorManager::killCursor(OperationContext* opCtx, CursorId id) {
     auto lockedPartition = _cursorMap->lockOnePartition(id);
     auto it = lockedPartition->find(id);
     if (it == lockedPartition->end()) {
-        if (shouldAudit) {
-            audit::logKillCursorsAuthzCheck(opCtx->getClient(), {}, id, ErrorCodes::CursorNotFound);
-        }
         return {ErrorCodes::CursorNotFound, str::stream() << "Cursor id not found: " << id};
     }
     auto cursor = it->second;
@@ -421,18 +441,10 @@ Status CursorManager::killCursor(OperationContext* opCtx, CursorId id, bool shou
             cursor->_operationUsingCursor->getServiceContext()->killOperation(
                 lk, cursor->_operationUsingCursor, ErrorCodes::CursorKilled);
         }
-
-        if (shouldAudit) {
-            audit::logKillCursorsAuthzCheck(opCtx->getClient(), cursor->nss(), id, ErrorCodes::OK);
-        }
         return Status::OK();
     }
     std::unique_ptr<ClientCursor, ClientCursor::Deleter> ownedCursor(cursor);
 
-    if (shouldAudit) {
-        audit::logKillCursorsAuthzCheck(opCtx->getClient(), cursor->nss(), id, ErrorCodes::OK);
-    }
-
     deregisterAndDestroyCursor(std::move(lockedPartition), opCtx, std::move(ownedCursor));
     return Status::OK();
 }
@@ -450,7 +462,7 @@ Status CursorManager::checkAuthForKillCursors(OperationContext* opCtx, CursorId
     // after the cursor's creation. We're guaranteed that the cursor won't get destroyed while we're
     // reading from it because we hold the partition's lock.
     AuthorizationSession* as = AuthorizationSession::get(opCtx->getClient());
-    return as->checkAuthForKillCursors(cursor->nss(), cursor->getAuthenticatedUsers());
+    return auth::checkAuthForKillCursors(as, cursor->nss(), cursor->getAuthenticatedUser());
 }
 
 }  // namespace mongo
