@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/platform/basic.h"
 
@@ -46,19 +45,21 @@
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/time_support.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
+
 namespace mongo {
 namespace {
 
 class DBRAIITestFixture : public CatalogTestFixture {
 public:
-    DBRAIITestFixture() : CatalogTestFixture("wiredTiger") {}
     typedef std::pair<ServiceContext::UniqueClient, ServiceContext::UniqueOperationContext>
         ClientAndCtx;
 
     ClientAndCtx makeClientWithLocker(const std::string& clientName) {
         auto client = getServiceContext()->makeClient(clientName);
         auto opCtx = client->makeOperationContext();
-        client->swapLockState(std::make_unique<LockerImpl>());
+        client->swapLockState(std::make_unique<LockerImpl>(opCtx->getServiceContext()));
         return std::make_pair(std::move(client), std::move(opCtx));
     }
 
@@ -68,10 +69,11 @@ public:
     const ClientAndCtx client2 = makeClientWithLocker("client2");
 };
 
-std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> makeTailableQueryPlan(OperationContext* opCtx,
-                                                                           Collection* collection) {
-    auto qr = std::make_unique<QueryRequest>(collection->ns());
-    qr->setTailableMode(TailableModeEnum::kTailableAndAwaitData);
+std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> makeTailableQueryPlan(
+    OperationContext* opCtx, const CollectionPtr& collection) {
+    auto findCommand = std::make_unique<FindCommandRequest>(collection->ns());
+    query_request_helper::setTailableMode(TailableModeEnum::kTailableAndAwaitData,
+                                          findCommand.get());
 
     awaitDataState(opCtx).shouldWaitForInserts = true;
     awaitDataState(opCtx).waitForInsertsDeadline =
@@ -81,7 +83,8 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> makeTailableQueryPlan(Opera
     const boost::intrusive_ptr<ExpressionContext> expCtx;
 
     auto statusWithCQ = CanonicalQuery::canonicalize(opCtx,
-                                                     std::move(qr),
+                                                     std::move(findCommand),
+                                                     false,
                                                      expCtx,
                                                      ExtensionsCallbackNoop(),
                                                      MatchExpressionParser::kBanAllSpecialFeatures);
@@ -89,7 +92,11 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> makeTailableQueryPlan(Opera
     std::unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
 
     bool permitYield = true;
-    auto swExec = getExecutorFind(opCtx, collection, std::move(cq), permitYield);
+    auto swExec = getExecutorFind(opCtx,
+                                  &collection,
+                                  std::move(cq),
+                                  nullptr /* extractAndAttachPipelineStages */,
+                                  permitYield);
     ASSERT_OK(swExec.getStatus());
     return std::move(swExec.getValue());
 }
@@ -107,28 +114,28 @@ void failsWithLockTimeout(std::function<void()> func, Milliseconds timeoutMillis
 }
 
 TEST_F(DBRAIITestFixture, AutoGetCollectionForReadCollLockDeadline) {
-    Lock::DBLock dbLock1(client1.second.get(), nss.db(), MODE_IX);
-    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.db(), MODE_IX));
+    Lock::DBLock dbLock1(client1.second.get(), nss.dbName(), MODE_IX);
+    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.dbName(), MODE_IX));
     Lock::CollectionLock collLock1(client1.second.get(), nss, MODE_X);
     ASSERT(client1.second->lockState()->isCollectionLockedForMode(nss, MODE_X));
     failsWithLockTimeout(
         [&] {
             AutoGetCollectionForRead acfr(client2.second.get(),
                                           nss,
-                                          AutoGetCollection::ViewMode::kViewsForbidden,
+                                          AutoGetCollectionViewMode::kViewsForbidden,
                                           Date_t::now() + timeoutMs);
         },
         timeoutMs);
 }
 
 TEST_F(DBRAIITestFixture, AutoGetCollectionForReadDBLockDeadline) {
-    Lock::DBLock dbLock1(client1.second.get(), nss.db(), MODE_X);
-    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.db(), MODE_X));
+    Lock::DBLock dbLock1(client1.second.get(), nss.dbName(), MODE_X);
+    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.dbName(), MODE_X));
     failsWithLockTimeout(
         [&] {
             AutoGetCollectionForRead coll(client2.second.get(),
                                           nss,
-                                          AutoGetCollection::ViewMode::kViewsForbidden,
+                                          AutoGetCollectionViewMode::kViewsForbidden,
                                           Date_t::now() + timeoutMs);
         },
         timeoutMs);
@@ -141,15 +148,15 @@ TEST_F(DBRAIITestFixture, AutoGetCollectionForReadGlobalLockDeadline) {
         [&] {
             AutoGetCollectionForRead coll(client2.second.get(),
                                           nss,
-                                          AutoGetCollection::ViewMode::kViewsForbidden,
+                                          AutoGetCollectionViewMode::kViewsForbidden,
                                           Date_t::now() + timeoutMs);
         },
         timeoutMs);
 }
 
 TEST_F(DBRAIITestFixture, AutoGetCollectionForReadDeadlineNow) {
-    Lock::DBLock dbLock1(client1.second.get(), nss.db(), MODE_IX);
-    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.db(), MODE_IX));
+    Lock::DBLock dbLock1(client1.second.get(), nss.dbName(), MODE_IX);
+    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.dbName(), MODE_IX));
     Lock::CollectionLock collLock1(client1.second.get(), nss, MODE_X);
     ASSERT(client1.second->lockState()->isCollectionLockedForMode(nss, MODE_X));
 
@@ -157,29 +164,29 @@ TEST_F(DBRAIITestFixture, AutoGetCollectionForReadDeadlineNow) {
         [&] {
             AutoGetCollectionForRead coll(client2.second.get(),
                                           nss,
-                                          AutoGetCollection::ViewMode::kViewsForbidden,
+                                          AutoGetCollectionViewMode::kViewsForbidden,
                                           Date_t::now());
         },
         Milliseconds(0));
 }
 
 TEST_F(DBRAIITestFixture, AutoGetCollectionForReadDeadlineMin) {
-    Lock::DBLock dbLock1(client1.second.get(), nss.db(), MODE_IX);
-    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.db(), MODE_IX));
+    Lock::DBLock dbLock1(client1.second.get(), nss.dbName(), MODE_IX);
+    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.dbName(), MODE_IX));
     Lock::CollectionLock collLock1(client1.second.get(), nss, MODE_X);
     ASSERT(client1.second->lockState()->isCollectionLockedForMode(nss, MODE_X));
 
     failsWithLockTimeout(
         [&] {
             AutoGetCollectionForRead coll(
-                client2.second.get(), nss, AutoGetCollection::ViewMode::kViewsForbidden, Date_t());
+                client2.second.get(), nss, AutoGetCollectionViewMode::kViewsForbidden, Date_t());
         },
         Milliseconds(0));
 }
 
 TEST_F(DBRAIITestFixture, AutoGetCollectionForReadDBLockCompatibleXNoCollection) {
-    Lock::DBLock dbLock1(client1.second.get(), nss.db(), MODE_IX);
-    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.db(), MODE_IX));
+    Lock::DBLock dbLock1(client1.second.get(), nss.dbName(), MODE_IX);
+    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.dbName(), MODE_IX));
 
     AutoGetCollectionForRead coll(client2.second.get(), nss);
 }
@@ -189,8 +196,8 @@ TEST_F(DBRAIITestFixture, AutoGetCollectionForReadDBLockCompatibleXCollectionExi
     ASSERT_OK(
         storageInterface()->createCollection(client1.second.get(), nss, defaultCollectionOptions));
 
-    Lock::DBLock dbLock1(client1.second.get(), nss.db(), MODE_IX);
-    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.db(), MODE_IX));
+    Lock::DBLock dbLock1(client1.second.get(), nss.dbName(), MODE_IX);
+    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.dbName(), MODE_IX));
 
     AutoGetCollectionForRead coll(client2.second.get(), nss);
 }
@@ -200,8 +207,8 @@ TEST_F(DBRAIITestFixture, AutoGetCollectionForReadDBLockCompatibleXCollectionExi
     ASSERT_OK(
         storageInterface()->createCollection(client1.second.get(), nss, defaultCollectionOptions));
 
-    Lock::DBLock dbLock1(client1.second.get(), nss.db(), MODE_IX);
-    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.db(), MODE_IX));
+    Lock::DBLock dbLock1(client1.second.get(), nss.dbName(), MODE_IX);
+    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.dbName(), MODE_IX));
     auto opCtx = client2.second.get();
     opCtx->recoveryUnit()->setTimestampReadSource(RecoveryUnit::ReadSource::kProvided,
                                                   Timestamp(1, 2));
@@ -217,8 +224,8 @@ TEST_F(DBRAIITestFixture,
         storageInterface()->createCollection(client1.second.get(), nss, defaultCollectionOptions));
     ASSERT_OK(repl::ReplicationCoordinator::get(client1.second.get())
                   ->setFollowerMode(repl::MemberState::RS_SECONDARY));
-    Lock::DBLock dbLock1(client1.second.get(), nss.db(), MODE_IX);
-    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.db(), MODE_IX));
+    Lock::DBLock dbLock1(client1.second.get(), nss.dbName(), MODE_IX);
+    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.dbName(), MODE_IX));
 
     // Simulate using a DBDirectClient to test this behavior for user reads.
     client2.first->setInDirectClient(true);
@@ -235,12 +242,11 @@ TEST_F(DBRAIITestFixture,
 
     // Don't call into the ReplicationCoordinator to update lastApplied because it is only a mock
     // class and does not update the correct state in the SnapshotManager.
-    repl::OpTime opTime(Timestamp(200, 1), 1);
     auto snapshotManager =
         client1.second.get()->getServiceContext()->getStorageEngine()->getSnapshotManager();
-    snapshotManager->setLastApplied(opTime.getTimestamp());
-    Lock::DBLock dbLock1(client1.second.get(), nss.db(), MODE_IX);
-    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.db(), MODE_IX));
+    snapshotManager->setLastApplied(replCoord->getMyLastAppliedOpTime().getTimestamp());
+    Lock::DBLock dbLock1(client1.second.get(), nss.dbName(), MODE_IX);
+    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.dbName(), MODE_IX));
 
     // Simulate using a DBDirectClient to test this behavior for user reads.
     client2.first->setInDirectClient(true);
@@ -268,8 +274,8 @@ TEST_F(DBRAIITestFixture,
         client1.second.get()->getServiceContext()->getStorageEngine()->getSnapshotManager();
     snapshotManager->setLastApplied(opTime.getTimestamp());
 
-    Lock::DBLock dbLock1(client1.second.get(), nss.db(), MODE_IX);
-    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.db(), MODE_IX));
+    Lock::DBLock dbLock1(client1.second.get(), nss.dbName(), MODE_IX);
+    ASSERT(client1.second->lockState()->isDbLockedForMode(nss.dbName(), MODE_IX));
 
     // Simulate using a DBDirectClient to test this behavior for user reads.
     client2.first->setInDirectClient(true);
@@ -310,14 +316,24 @@ TEST_F(DBRAIITestFixture, AutoGetCollectionForReadLastAppliedConflict) {
 
     // Simulate using a DBDirectClient to test this behavior for user reads.
     client1.first->setInDirectClient(true);
-    AutoGetCollectionForRead coll(client1.second.get(), nss);
 
-    // We can't read from kLastApplied in this scenario because there is a catalog conflict. Resort
-    // to taking the PBWM lock and reading without a timestamp.
+    auto timeoutError = client1.second->getTimeoutError();
+    auto waitForLock = [&] {
+        auto deadline = Date_t::now() + Milliseconds(10);
+        client1.second->runWithDeadline(deadline, timeoutError, [&] {
+            AutoGetCollectionForRead coll(client1.second.get(), nss);
+        });
+    };
+
+    // Expect that the lock acquisition eventually times out because lastApplied is not advancing.
+    ASSERT_THROWS_CODE(waitForLock(), DBException, timeoutError);
+
+    // Advance lastApplied and ensure the lock acquisition succeeds.
+    snapshotManager->setLastApplied(replCoord->getMyLastAppliedOpTime().getTimestamp());
+
+    AutoGetCollectionForRead coll(client1.second.get(), nss);
     ASSERT_EQ(client1.second.get()->recoveryUnit()->getTimestampReadSource(),
-              RecoveryUnit::ReadSource::kNoTimestamp);
-    ASSERT_TRUE(client1.second.get()->lockState()->isLockHeldForMode(
-        resourceIdParallelBatchWriterMode, MODE_IS));
+              RecoveryUnit::ReadSource::kLastApplied);
 }
 
 TEST_F(DBRAIITestFixture, AutoGetCollectionForReadLastAppliedUnavailable) {
@@ -342,7 +358,8 @@ TEST_F(DBRAIITestFixture, AutoGetCollectionForReadLastAppliedUnavailable) {
 
     ASSERT_EQ(client1.second.get()->recoveryUnit()->getTimestampReadSource(),
               RecoveryUnit::ReadSource::kLastApplied);
-    ASSERT_FALSE(client1.second.get()->recoveryUnit()->getPointInTimeReadTimestamp());
+    ASSERT_FALSE(
+        client1.second.get()->recoveryUnit()->getPointInTimeReadTimestamp(client1.second.get()));
     ASSERT_FALSE(client1.second.get()->lockState()->isLockHeldForMode(
         resourceIdParallelBatchWriterMode, MODE_IS));
 }
