@@ -31,14 +31,64 @@
 
 #include "mongo/db/curop.h"
 #include "mongo/db/error_labels.h"
-#include "mongo/db/logical_session_id.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
+#include "mongo/db/pipeline/aggregation_request_helper.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
 #include "mongo/db/service_context_test_fixture.h"
+#include "mongo/db/session/logical_session_id.h"
 #include "mongo/unittest/unittest.h"
 
 namespace mongo {
 namespace {
 
+const repl::OpTime kOpTime{Timestamp(10, 10), 10};
+const repl::OpTime kLaterOpTime{Timestamp(10, 11), 10};
+
+TEST(IsTransientTransactionErrorTest, WriteConflictIsTransient) {
+    ASSERT_TRUE(isTransientTransactionError(
+        ErrorCodes::WriteConflict, false /* hasWriteConcernError */, false /* isCommitOrAbort */));
+}
+
+TEST(IsTransientTransactionErrorTest, LockTimeoutIsTransient) {
+    ASSERT_TRUE(isTransientTransactionError(
+        ErrorCodes::LockTimeout, false /* hasWriteConcernError */, false /* isCommitOrAbort */));
+}
+
+TEST(IsTransientTransactionErrorTest, PreparedTransactionInProgressIsTransient) {
+    ASSERT_TRUE(isTransientTransactionError(ErrorCodes::PreparedTransactionInProgress,
+                                            false /* hasWriteConcernError */,
+                                            false /* isCommitOrAbort */));
+}
+
+TEST(IsTransientTransactionErrorTest, TenantMigrationCommittedIsTransient) {
+    ASSERT_TRUE(isTransientTransactionError(ErrorCodes::TenantMigrationCommitted,
+                                            false /* hasWriteConcernError */,
+                                            false /* isCommitOrAbort */));
+}
+
+TEST(IsTransientTransactionErrorTest, TenantMigrationAbortedIsTransient) {
+    ASSERT_TRUE(isTransientTransactionError(ErrorCodes::TenantMigrationAborted,
+                                            false /* hasWriteConcernError */,
+                                            false /* isCommitOrAbort */));
+}
+
+TEST(IsTransientTransactionErrorTest, ShardCannotRefreshDueToLocksHeldIsTransient) {
+    ASSERT_TRUE(isTransientTransactionError(ErrorCodes::ShardCannotRefreshDueToLocksHeld,
+                                            false /* hasWriteConcernError */,
+                                            false /* isCommitOrAbort */));
+}
+
+TEST(IsTransientTransactionErrorTest, ShardInvalidatedForTargetingIsTransient) {
+    ASSERT_TRUE(isTransientTransactionError(ErrorCodes::ShardInvalidatedForTargeting,
+                                            false /* hasWriteConcernError */,
+                                            false /* isCommitOrAbort */));
+}
+
+TEST(IsTransientTransactionErrorTest, StaleDbVersionIsTransient) {
+    ASSERT_TRUE(isTransientTransactionError(
+        ErrorCodes::StaleDbVersion, false /* hasWriteConcernError */, false /* isCommitOrAbort */));
+}
+
 TEST(IsTransientTransactionErrorTest, NetworkErrorsAreTransientBeforeCommit) {
     ASSERT_TRUE(isTransientTransactionError(ErrorCodes::HostUnreachable,
                                             false /* hasWriteConcernError */,
@@ -101,11 +151,19 @@ private:
 TEST_F(ErrorLabelBuilderTest, NonErrorCodesHaveNoLabel) {
     OperationSessionInfoFromClient sessionInfo;
     std::string commandName = "insert";
-    ErrorLabelBuilder builder(
-        opCtx(), sessionInfo, commandName, boost::none, boost::none, false, false /* isMongos */);
+    ErrorLabelBuilder builder(opCtx(),
+                              sessionInfo,
+                              commandName,
+                              boost::none,
+                              boost::none,
+                              false,
+                              false /* isMongos */,
+                              kOpTime,
+                              kOpTime);
     ASSERT_FALSE(builder.isTransientTransactionError());
     ASSERT_FALSE(builder.isRetryableWriteError());
     ASSERT_FALSE(builder.isResumableChangeStreamError());
+    ASSERT_FALSE(builder.isErrorWithNoWritesPerformed());
 }
 
 TEST_F(ErrorLabelBuilderTest, NonTransactionsHaveNoTransientTransactionErrorLabel) {
@@ -117,7 +175,9 @@ TEST_F(ErrorLabelBuilderTest, NonTransactionsHaveNoTransientTransactionErrorLabe
                               ErrorCodes::WriteConflict,
                               boost::none,
                               false /* isInternalClient */,
-                              false /* isMongos */);
+                              false /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
     ASSERT_FALSE(builder.isTransientTransactionError());
 }
 
@@ -131,7 +191,9 @@ TEST_F(ErrorLabelBuilderTest, RetryableWritesHaveNoTransientTransactionErrorLabe
                               ErrorCodes::WriteConflict,
                               boost::none,
                               false /* isInternalClient */,
-                              false /* isMongos */);
+                              false /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
     ASSERT_FALSE(builder.isTransientTransactionError());
 }
 
@@ -146,7 +208,9 @@ TEST_F(ErrorLabelBuilderTest, NonTransientTransactionErrorsHaveNoTransientTransa
                               ErrorCodes::NotWritablePrimary,
                               boost::none,
                               false /* isInternalClient */,
-                              false /* isMongos */);
+                              false /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
     ASSERT_FALSE(builder.isTransientTransactionError());
 }
 
@@ -161,7 +225,9 @@ TEST_F(ErrorLabelBuilderTest, TransientTransactionErrorsHaveTransientTransaction
                               ErrorCodes::NoSuchTransaction,
                               boost::none,
                               false /* isInternalClient */,
-                              false /* isMongos */);
+                              false /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
     ASSERT_TRUE(builder.isTransientTransactionError());
 }
 
@@ -181,7 +247,9 @@ TEST_F(
                                             transientError,
                                             retryableError,
                                             false /* isInternalClient */,
-                                            false /* isMongos */);
+                                            false /* isMongos */,
+                                            repl::OpTime{},
+                                            repl::OpTime{});
 
     // Ensure only the TransientTransactionError label is attached so users know to retry the entire
     // transaction.
@@ -199,7 +267,9 @@ TEST_F(ErrorLabelBuilderTest, NonRetryableWritesHaveNoRetryableWriteErrorLabel)
                               ErrorCodes::NotWritablePrimary,
                               boost::none,
                               false /* isInternalClient */,
-                              false /* isMongos */);
+                              false /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
 
     // Test regular writes.
     ASSERT_FALSE(builder.isRetryableWriteError());
@@ -220,7 +290,9 @@ TEST_F(ErrorLabelBuilderTest, NonRetryableWriteErrorsHaveNoRetryableWriteErrorLa
                               ErrorCodes::WriteConflict,
                               boost::none,
                               false /* isInternalClient */,
-                              false /* isMongos */);
+                              false /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
     ASSERT_FALSE(builder.isRetryableWriteError());
 }
 
@@ -234,7 +306,9 @@ TEST_F(ErrorLabelBuilderTest, RetryableWriteErrorsHaveRetryableWriteErrorLabel)
                               ErrorCodes::NotWritablePrimary,
                               boost::none,
                               false /* isInternalClient */,
-                              false /* isMongos */);
+                              false /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
     ASSERT_TRUE(builder.isRetryableWriteError());
 }
 
@@ -248,7 +322,9 @@ TEST_F(ErrorLabelBuilderTest, NonLocalShutDownErrorsOnMongosDoNotHaveRetryableWr
                               ErrorCodes::InterruptedAtShutdown,
                               boost::none,
                               false /* isInternalClient */,
-                              true /* isMongos */);
+                              true /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
     ASSERT_FALSE(builder.isRetryableWriteError());
 }
 
@@ -264,7 +340,9 @@ TEST_F(ErrorLabelBuilderTest,
                               ErrorCodes::InterruptedAtShutdown,
                               boost::none,
                               false /* isInternalClient */,
-                              true /* isMongos */);
+                              true /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
     ASSERT_TRUE(builder.isRetryableWriteError());
 }
 
@@ -280,7 +358,9 @@ TEST_F(ErrorLabelBuilderTest,
                               ErrorCodes::CallbackCanceled,
                               boost::none,
                               false /* isInternalClient */,
-                              true /* isMongos */);
+                              true /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
     ASSERT_TRUE(builder.isRetryableWriteError());
 }
 
@@ -295,7 +375,9 @@ TEST_F(ErrorLabelBuilderTest,
                               ErrorCodes::NotWritablePrimary,
                               boost::none,
                               true /* isInternalClient */,
-                              false /* isMongos */);
+                              false /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
     ASSERT_FALSE(builder.isRetryableWriteError());
 }
 
@@ -310,7 +392,9 @@ TEST_F(ErrorLabelBuilderTest,
                               ErrorCodes::WriteConcernFailed,
                               ErrorCodes::WriteConcernFailed,
                               false /* isInternalClient */,
-                              false /* isMongos */);
+                              false /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
     ASSERT_FALSE(builder.isRetryableWriteError());
 }
 
@@ -325,7 +409,9 @@ TEST_F(ErrorLabelBuilderTest,
                               ErrorCodes::WriteConcernFailed,
                               ErrorCodes::PrimarySteppedDown,
                               false /* isInternalClient */,
-                              false /* isMongos */);
+                              false /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
     ASSERT_TRUE(builder.isRetryableWriteError());
 }
 
@@ -342,8 +428,24 @@ TEST_F(ErrorLabelBuilderTest, RetryableWriteErrorsOnCommitAbortHaveRetryableWrit
                                     ErrorCodes::NotWritablePrimary,
                                     boost::none,
                                     false /* isInternalClient */,
-                                    false /* isMongos */);
+                                    false /* isMongos */,
+                                    repl::OpTime{},
+                                    repl::OpTime{});
     ASSERT_TRUE(commitBuilder.isRetryableWriteError());
+    ASSERT_FALSE(commitBuilder.isTransientTransactionError());
+
+    commandName = "clusterCommitTransaction";
+    ErrorLabelBuilder clusterCommitBuilder(opCtx(),
+                                           sessionInfo,
+                                           commandName,
+                                           ErrorCodes::NotWritablePrimary,
+                                           boost::none,
+                                           false /* isInternalClient */,
+                                           true /* isMongos */,
+                                           repl::OpTime{},
+                                           repl::OpTime{});
+    ASSERT_TRUE(commitBuilder.isRetryableWriteError());
+    ASSERT_FALSE(commitBuilder.isTransientTransactionError());
 
     commandName = "coordinateCommitTransaction";
     ErrorLabelBuilder coordinateCommitBuilder(opCtx(),
@@ -352,8 +454,11 @@ TEST_F(ErrorLabelBuilderTest, RetryableWriteErrorsOnCommitAbortHaveRetryableWrit
                                               ErrorCodes::NotWritablePrimary,
                                               boost::none,
                                               false /* isInternalClient */,
-                                              false /* isMongos */);
+                                              false /* isMongos */,
+                                              repl::OpTime{},
+                                              repl::OpTime{});
     ASSERT_TRUE(coordinateCommitBuilder.isRetryableWriteError());
+    ASSERT_FALSE(commitBuilder.isTransientTransactionError());
 
     commandName = "abortTransaction";
     ErrorLabelBuilder abortBuilder(opCtx(),
@@ -362,8 +467,24 @@ TEST_F(ErrorLabelBuilderTest, RetryableWriteErrorsOnCommitAbortHaveRetryableWrit
                                    ErrorCodes::NotWritablePrimary,
                                    boost::none,
                                    false /* isInternalClient */,
-                                   false /* isMongos */);
+                                   false /* isMongos */,
+                                   repl::OpTime{},
+                                   repl::OpTime{});
     ASSERT_TRUE(abortBuilder.isRetryableWriteError());
+    ASSERT_FALSE(commitBuilder.isTransientTransactionError());
+
+    commandName = "clusterAbortTransaction";
+    ErrorLabelBuilder clusterAbortBuilder(opCtx(),
+                                          sessionInfo,
+                                          commandName,
+                                          ErrorCodes::NotWritablePrimary,
+                                          boost::none,
+                                          false /* isInternalClient */,
+                                          true /* isMongos */,
+                                          repl::OpTime{},
+                                          repl::OpTime{});
+    ASSERT_TRUE(commitBuilder.isRetryableWriteError());
+    ASSERT_FALSE(commitBuilder.isTransientTransactionError());
 }
 
 TEST_F(ErrorLabelBuilderTest, NonResumableChangeStreamError) {
@@ -375,7 +496,9 @@ TEST_F(ErrorLabelBuilderTest, NonResumableChangeStreamError) {
                               ErrorCodes::ChangeStreamHistoryLost,
                               boost::none,
                               false /* isInternalClient */,
-                              false /* isMongos */);
+                              false /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
     ASSERT_TRUE(builder.isNonResumableChangeStreamError());
 }
 
@@ -385,8 +508,9 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorAppliesToChangeStreamAgg
     // is the only factor that determines the success or failure of isResumableChangeStreamError().
     auto cmdObj = BSON("aggregate" << nss().coll() << "pipeline"
                                    << BSON_ARRAY(BSON("$changeStream" << BSONObj())) << "cursor"
-                                   << BSONObj());
-    auto aggRequest = uassertStatusOK(AggregationRequest::parseFromBSON(nss(), cmdObj));
+                                   << BSONObj() << "$db" << nss().db());
+    auto aggRequest =
+        uassertStatusOK(aggregation_request_helper::parseFromBSONForTests(nss(), cmdObj));
     ASSERT_TRUE(LiteParsedPipeline(aggRequest).hasChangeStream());
 
     // The label applies to a $changeStream "aggregate" command.
@@ -398,7 +522,9 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorAppliesToChangeStreamAgg
                                           ErrorCodes::NetworkTimeout,
                                           boost::none,
                                           false /* isInternalClient */,
-                                          false /* isMongos */);
+                                          false /* isMongos */,
+                                          repl::OpTime{},
+                                          repl::OpTime{});
     ASSERT_TRUE(resumableAggBuilder.isResumableChangeStreamError());
     // The label applies to a "getMore" command on a $changeStream cursor.
     commandName = "getMore";
@@ -409,7 +535,9 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorAppliesToChangeStreamAgg
                                               ErrorCodes::NetworkTimeout,
                                               boost::none,
                                               false /* isInternalClient */,
-                                              false /* isMongos */);
+                                              false /* isMongos */,
+                                              repl::OpTime{},
+                                              repl::OpTime{});
     ASSERT_TRUE(resumableGetMoreBuilder.isResumableChangeStreamError());
 }
 
@@ -419,8 +547,9 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonResumab
     // is the only factor that determines the success or failure of isResumableChangeStreamError().
     auto cmdObj = BSON("aggregate" << nss().coll() << "pipeline"
                                    << BSON_ARRAY(BSON("$changeStream" << BSONObj())) << "cursor"
-                                   << BSONObj());
-    auto aggRequest = uassertStatusOK(AggregationRequest::parseFromBSON(nss(), cmdObj));
+                                   << BSONObj() << "$db" << nss().db());
+    auto aggRequest =
+        uassertStatusOK(aggregation_request_helper::parseFromBSONForTests(nss(), cmdObj));
     ASSERT_TRUE(LiteParsedPipeline(aggRequest).hasChangeStream());
 
     // The label does not apply to a ChangeStreamFatalError error on a $changeStream aggregation.
@@ -432,7 +561,9 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonResumab
                                           ErrorCodes::ChangeStreamFatalError,
                                           boost::none,
                                           false /* isInternalClient */,
-                                          false /* isMongos */);
+                                          false /* isMongos */,
+                                          repl::OpTime{},
+                                          repl::OpTime{});
     ASSERT_FALSE(resumableAggBuilder.isResumableChangeStreamError());
     // The label does not apply to a ChangeStreamFatalError error on a $changeStream getMore.
     commandName = "getMore";
@@ -443,7 +574,9 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonResumab
                                               ErrorCodes::ChangeStreamFatalError,
                                               boost::none,
                                               false /* isInternalClient */,
-                                              false /* isMongos */);
+                                              false /* isMongos */,
+                                              repl::OpTime{},
+                                              repl::OpTime{});
     ASSERT_FALSE(resumableGetMoreBuilder.isResumableChangeStreamError());
 }
 
@@ -453,8 +586,9 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonChangeS
     // is the only factor that determines the success or failure of isResumableChangeStreamError().
     auto cmdObj =
         BSON("aggregate" << nss().coll() << "pipeline" << BSON_ARRAY(BSON("$match" << BSONObj()))
-                         << "cursor" << BSONObj());
-    auto aggRequest = uassertStatusOK(AggregationRequest::parseFromBSON(nss(), cmdObj));
+                         << "cursor" << BSONObj() << "$db" << nss().db());
+    auto aggRequest =
+        uassertStatusOK(aggregation_request_helper::parseFromBSONForTests(nss(), cmdObj));
     ASSERT_FALSE(LiteParsedPipeline(aggRequest).hasChangeStream());
 
     // The label does not apply to a non-$changeStream "aggregate" command.
@@ -466,7 +600,9 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonChangeS
                                              ErrorCodes::NetworkTimeout,
                                              boost::none,
                                              false /* isInternalClient */,
-                                             false /* isMongos */);
+                                             false /* isMongos */,
+                                             repl::OpTime{},
+                                             repl::OpTime{});
     ASSERT_FALSE(nonResumableAggBuilder.isResumableChangeStreamError());
     // The label does not apply to a "getMore" command on a non-$changeStream cursor.
     commandName = "getMore";
@@ -477,7 +613,9 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonChangeS
                                                  ErrorCodes::NetworkTimeout,
                                                  boost::none,
                                                  false /* isInternalClient */,
-                                                 false /* isMongos */);
+                                                 false /* isMongos */,
+                                                 repl::OpTime{},
+                                                 repl::OpTime{});
     ASSERT_FALSE(nonResumableGetMoreBuilder.isResumableChangeStreamError());
 }
 
@@ -493,7 +631,9 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonAggrega
                                               ErrorCodes::NetworkTimeout,
                                               boost::none,
                                               false /* isInternalClient */,
-                                              false /* isMongos */);
+                                              false /* isMongos */,
+                                              repl::OpTime{},
+                                              repl::OpTime{});
     ASSERT_FALSE(nonResumableFindBuilder.isResumableChangeStreamError());
     // The label does not apply to a "getMore" command on a "find" cursor.
     commandName = "getMore";
@@ -504,9 +644,109 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonAggrega
                                                  ErrorCodes::NetworkTimeout,
                                                  boost::none,
                                                  false /* isInternalClient */,
-                                                 false /* isMongos */);
+                                                 false /* isMongos */,
+                                                 repl::OpTime{},
+                                                 repl::OpTime{});
     ASSERT_FALSE(nonResumableGetMoreBuilder.isResumableChangeStreamError());
 }
 
+TEST_F(ErrorLabelBuilderTest, NoWritesPerformedLabelApplied) {
+    OperationSessionInfoFromClient sessionInfo;
+    std::string commandName = "find";
+    ErrorLabelBuilder builder(opCtx(),
+                              sessionInfo,
+                              commandName,
+                              ErrorCodes::WriteConcernFailed,
+                              ErrorCodes::WriteConcernFailed,
+                              false,
+                              false /* isMongos */,
+                              kOpTime,
+                              kOpTime);
+    ASSERT_TRUE(builder.isErrorWithNoWritesPerformed());
+}
+
+TEST_F(ErrorLabelBuilderTest, NoWritesPerformedLabelNotAppliedAfterWrite) {
+    OperationSessionInfoFromClient sessionInfo;
+    std::string commandName = "update";
+    ErrorLabelBuilder builder(opCtx(),
+                              sessionInfo,
+                              commandName,
+                              ErrorCodes::WriteConcernFailed,
+                              ErrorCodes::WriteConcernFailed,
+                              false,
+                              false /* isMongos */,
+                              kOpTime,
+                              kLaterOpTime);
+    ASSERT_FALSE(builder.isErrorWithNoWritesPerformed());
+}
+
+TEST_F(ErrorLabelBuilderTest, NoWritesPerformedLabelNotAppliedIfUnknown) {
+    OperationSessionInfoFromClient sessionInfo;
+    std::string commandName = "update";
+    ErrorLabelBuilder builder(opCtx(),
+                              sessionInfo,
+                              commandName,
+                              ErrorCodes::WriteConcernFailed,
+                              ErrorCodes::WriteConcernFailed,
+                              false,
+                              false /* isMongos */,
+                              repl::OpTime{},
+                              repl::OpTime{});
+    ASSERT_FALSE(builder.isErrorWithNoWritesPerformed());
+}
+
+TEST_F(ErrorLabelBuilderTest, NoWritesPerformedAndRetryableWriteAppliesBothLabels) {
+    OperationSessionInfoFromClient sessionInfo;
+    sessionInfo.setTxnNumber(1);
+    std::string commandName = "update";
+    auto actualErrorLabels = getErrorLabels(opCtx(),
+                                            sessionInfo,
+                                            commandName,
+                                            ErrorCodes::NotWritablePrimary,
+                                            boost::none,
+                                            false /* isInternalClient */,
+                                            false /* isMongos */,
+                                            kOpTime,
+                                            kOpTime);
+    BSONArrayBuilder expectedLabelArray;
+    expectedLabelArray << ErrorLabel::kRetryableWrite;
+    expectedLabelArray << ErrorLabel::kNoWritesPerformed;
+    ASSERT_BSONOBJ_EQ(actualErrorLabels, BSON(kErrorLabelsFieldName << expectedLabelArray.arr()));
+}
+
+TEST_F(ErrorLabelBuilderTest, NoWritesPerformedNotAppliedDuringOrdinaryUpdate) {
+    OperationSessionInfoFromClient sessionInfo;
+    std::string commandName = "update";
+    auto actualErrorLabels = getErrorLabels(opCtx(),
+                                            sessionInfo,
+                                            commandName,
+                                            ErrorCodes::NotWritablePrimary,
+                                            boost::none,
+                                            false /* isInternalClient */,
+                                            false /* isMongos */,
+                                            kOpTime,
+                                            kOpTime);
+    ASSERT_BSONOBJ_EQ(actualErrorLabels, BSONObj());
+}
+
+TEST_F(ErrorLabelBuilderTest, NoWritesPerformedNotAppliedDuringTransientTransactionError) {
+    OperationSessionInfoFromClient sessionInfo;
+    sessionInfo.setTxnNumber(1);
+    sessionInfo.setAutocommit(false);
+    std::string commandName = "commitTransaction";
+    auto actualErrorLabels = getErrorLabels(opCtx(),
+                                            sessionInfo,
+                                            commandName,
+                                            ErrorCodes::NoSuchTransaction,
+                                            boost::none,
+                                            false /* isInternalClient */,
+                                            false /* isMongos */,
+                                            kOpTime,
+                                            kOpTime);
+    BSONArrayBuilder expectedLabelArray;
+    expectedLabelArray << ErrorLabel::kTransientTransaction;
+    ASSERT_BSONOBJ_EQ(actualErrorLabels, BSON(kErrorLabelsFieldName << expectedLabelArray.arr()));
+}
+
 }  // namespace
 }  // namespace mongo
