@@ -27,8 +27,6 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/exec/document_value/document.h"
 
 #include <boost/functional/hash.hpp>
@@ -44,6 +42,43 @@ using boost::intrusive_ptr;
 using std::string;
 using std::vector;
 
+namespace {
+/**
+ * Assert that a given field path does not exceed the length limit.
+ */
+void assertFieldPathLengthOK(const FieldPath& path) {
+    uassert(5984700,
+            "Field path exceeds path length limit",
+            path.getPathLength() < BSONDepth::getMaxAllowableDepth());
+}
+void assertFieldPathLengthOK(const std::vector<Position>& path) {
+    uassert(5984701,
+            "Field path exceeds path length limit",
+            path.size() < BSONDepth::getMaxAllowableDepth());
+}
+
+stdx::variant<BSONElement, Value, Document::TraversesArrayTag, stdx::monostate>
+getNestedFieldHelperBSON(BSONElement elt, const FieldPath& fp, size_t level) {
+    if (level == fp.getPathLength()) {
+        if (elt.ok()) {
+            return elt;
+        } else {
+            return stdx::monostate{};
+        }
+    }
+
+    if (elt.type() == BSONType::Array) {
+        return Document::TraversesArrayTag{};
+    } else if (elt.type() == BSONType::Object) {
+        auto subFieldElt = elt.embeddedObject()[fp.getFieldName(level)];
+        return getNestedFieldHelperBSON(subFieldElt, fp, level + 1);
+    }
+
+    // The path continues "past" a scalar, and therefore does not exist.
+    return stdx::monostate{};
+}
+}  // namespace
+
 const DocumentStorage DocumentStorage::kEmptyDoc;
 
 const StringDataSet Document::allMetadataFieldNames{Document::metaFieldTextScore,
@@ -53,7 +88,6 @@ const StringDataSet Document::allMetadataFieldNames{Document::metaFieldTextScore
                                                     Document::metaFieldGeoNearPoint,
                                                     Document::metaFieldSearchScore,
                                                     Document::metaFieldSearchHighlights,
-                                                    Document::metaFieldSearchSortValues,
                                                     Document::metaFieldIndexKey,
                                                     Document::metaFieldSearchScoreDetails};
 
@@ -128,7 +162,8 @@ bool DocumentStorageIterator::shouldSkipDeleted() {
     return false;
 }
 
-Position DocumentStorage::findFieldInCache(StringData requested) const {
+template <typename T>
+Position DocumentStorage::findFieldInCache(T requested) const {
     int reqSize = requested.size();  // get size calculation out of the way if needed
 
     if (_numFields >= HASH_TAB_MIN) {  // hash lookup
@@ -156,14 +191,17 @@ Position DocumentStorage::findFieldInCache(StringData requested) const {
     // if we got here, there's no such field
     return Position();
 }
+template Position DocumentStorage::findFieldInCache<StringData>(StringData field) const;
+template Position DocumentStorage::findFieldInCache<HashedFieldName>(HashedFieldName field) const;
 
-Position DocumentStorage::findField(StringData requested, LookupPolicy policy) const {
-    if (auto pos = findFieldInCache(requested); pos.found() || policy == LookupPolicy::kCacheOnly) {
+template <typename T>
+Position DocumentStorage::findField(T field, LookupPolicy policy) const {
+    if (auto pos = findFieldInCache(field); pos.found() || policy == LookupPolicy::kCacheOnly) {
         return pos;
     }
 
     for (auto&& bsonElement : _bson) {
-        if (requested == bsonElement.fieldNameStringData()) {
+        if (field == bsonElement.fieldNameStringData()) {
             return const_cast<DocumentStorage*>(this)->constructInCache(bsonElement);
         }
     }
@@ -171,6 +209,10 @@ Position DocumentStorage::findField(StringData requested, LookupPolicy policy) c
     // if we got here, there's no such field
     return Position();
 }
+template Position DocumentStorage::findField<StringData>(StringData field,
+                                                         LookupPolicy policy) const;
+template Position DocumentStorage::findField<HashedFieldName>(HashedFieldName field,
+                                                              LookupPolicy policy) const;
 
 Position DocumentStorage::constructInCache(const BSONElement& elem) {
     auto savedModified = _modified;
@@ -186,9 +228,10 @@ Position DocumentStorage::constructInCache(const BSONElement& elem) {
     return pos;
 }
 
-Value& DocumentStorage::appendField(StringData name, ValueElement::Kind kind) {
+template <typename T>
+Value& DocumentStorage::appendField(T field, ValueElement::Kind kind) {
     Position pos = getNextPosition();
-    const int nameSize = name.size();
+    const int nameSize = field.size();
 
     // these are the same for everyone
     const Position nextCollision;
@@ -196,7 +239,7 @@ Value& DocumentStorage::appendField(StringData name, ValueElement::Kind kind) {
 
     // Make room for new field (and padding at end for alignment)
     const unsigned newUsed = ValueElement::align(_usedBytes + sizeof(ValueElement) + nameSize);
-    if (_cache + newUsed > _cacheEnd)
+    if (newUsed > _cacheEnd - _cache)
         alloc(newUsed);
     _usedBytes = newUsed;
 
@@ -209,7 +252,7 @@ Value& DocumentStorage::appendField(StringData name, ValueElement::Kind kind) {
     append(nextCollision);
     append(nameSize);
     append(kind);
-    name.copyTo(dest, true);
+    field.copyTo(dest, true);
 // Padding for alignment handled above
 #undef append
 
@@ -219,7 +262,7 @@ Value& DocumentStorage::appendField(StringData name, ValueElement::Kind kind) {
     _numFields++;
 
     if (_numFields > HASH_TAB_MIN) {
-        addFieldToHashTable(pos);
+        addFieldToHashTable(field, pos);
     } else if (_numFields == HASH_TAB_MIN) {
         // adds all fields to hash table (including the one we just added)
         rehash();
@@ -227,13 +270,16 @@ Value& DocumentStorage::appendField(StringData name, ValueElement::Kind kind) {
 
     return getField(pos).val;
 }
+template Value& DocumentStorage::appendField<StringData>(StringData, ValueElement::Kind);
+template Value& DocumentStorage::appendField<HashedFieldName>(HashedFieldName, ValueElement::Kind);
 
 // Call after adding field to _fields and increasing _numFields
-void DocumentStorage::addFieldToHashTable(Position pos) {
+template <typename T>
+void DocumentStorage::addFieldToHashTable(T field, Position pos) {
     ValueElement& elem = getField(pos);
     elem.nextCollision = Position();
 
-    const unsigned bucket = bucketForKey(elem.nameSD());
+    const unsigned bucket = bucketForKey(field);
 
     Position* posPtr = &_hashTab[bucket];
     while (posPtr->found()) {
@@ -327,6 +373,7 @@ intrusive_ptr<DocumentStorage> DocumentStorage::clone() const {
 
     out->_haveLazyLoadedMetadata = _haveLazyLoadedMetadata;
     out->_metadataFields = _metadataFields;
+    out->_snapshottedSize = _snapshottedSize;
 
     return out;
 }
@@ -348,6 +395,7 @@ void DocumentStorage::reset(const BSONObj& bson, bool stripMetadata) {
     _numBytesFromBSONInCache = 0;
     _stripMetadata = stripMetadata;
     _modified = false;
+    _snapshottedSize = 0;
 
     // Clean cache.
     for (auto it = iteratorCacheOnly(); !it.atEnd(); it.advance()) {
@@ -363,6 +411,12 @@ void DocumentStorage::reset(const BSONObj& bson, bool stripMetadata) {
     _metadataFields = DocumentMetadataFields{};
 }
 
+void DocumentStorage::fillCache() const {
+    for (DocumentStorageIterator it = iterator(); !it.atEnd(); it.advance()) {
+        it->val.fillCache();
+    }
+}
+
 void DocumentStorage::loadLazyMetadata() const {
     if (_haveLazyLoadedMetadata) {
         return;
@@ -408,8 +462,6 @@ void DocumentStorage::loadLazyMetadata() const {
                 _metadataFields.setIndexKey(elem.Obj());
             } else if (fieldName == Document::metaFieldSearchScoreDetails) {
                 _metadataFields.setSearchScoreDetails(elem.Obj());
-            } else if (fieldName == Document::metaFieldSearchSortValues) {
-                _metadataFields.setSearchSortValues(elem.Obj());
             }
         }
     }
@@ -434,6 +486,13 @@ Document::Document(std::initializer_list<std::pair<StringData, ImplicitValue>> i
     *this = mutableDoc.freeze();
 }
 
+Document::Document(std::vector<std::pair<StringData, Value>> fields) {
+    MutableDocument mutableDoc(fields.size());
+    for (auto&& pair : fields)
+        mutableDoc.addField(pair.first, pair.second);
+    *this = mutableDoc.freeze();
+}
+
 BSONObjBuilder& operator<<(BSONObjBuilderValueStream& builder, const Document& doc) {
     BSONObjBuilder subobj(builder.subobjStart());
     doc.toBson(&subobj);
@@ -482,9 +541,8 @@ constexpr StringData Document::metaFieldGeoNearPoint;
 constexpr StringData Document::metaFieldSearchScore;
 constexpr StringData Document::metaFieldSearchHighlights;
 constexpr StringData Document::metaFieldSearchScoreDetails;
-constexpr StringData Document::metaFieldSearchSortValues;
 
-BSONObj Document::toBsonWithMetaData(SortKeyFormat sortKeyFormat) const {
+BSONObj Document::toBsonWithMetaData() const {
     BSONObjBuilder bb;
     toBson(&bb);
     if (!metadata()) {
@@ -495,24 +553,10 @@ BSONObj Document::toBsonWithMetaData(SortKeyFormat sortKeyFormat) const {
         bb.append(metaFieldTextScore, metadata().getTextScore());
     if (metadata().hasRandVal())
         bb.append(metaFieldRandVal, metadata().getRandVal());
-    if (metadata().hasSortKey()) {
-        switch (sortKeyFormat) {
-            case SortKeyFormat::k42ChangeStreamSortKey:
-                invariant(metadata().isSingleElementKey());
-                metadata().getSortKey().addToBsonObj(&bb, metaFieldSortKey);
-                break;
-            case SortKeyFormat::k42SortKey:
-                bb.append(metaFieldSortKey,
-                          DocumentMetadataFields::serializeSortKeyAsObject(
-                              metadata().isSingleElementKey(), metadata().getSortKey()));
-                break;
-            case SortKeyFormat::k44SortKey:
-                bb.append(metaFieldSortKey,
-                          DocumentMetadataFields::serializeSortKeyAsArray(
-                              metadata().isSingleElementKey(), metadata().getSortKey()));
-                break;
-        }
-    }
+    if (metadata().hasSortKey())
+        bb.append(metaFieldSortKey,
+                  DocumentMetadataFields::serializeSortKey(metadata().isSingleElementKey(),
+                                                           metadata().getSortKey()));
     if (metadata().hasGeoNearDistance())
         bb.append(metaFieldGeoNearDistance, metadata().getGeoNearDistance());
     if (metadata().hasGeoNearPoint())
@@ -525,9 +569,6 @@ BSONObj Document::toBsonWithMetaData(SortKeyFormat sortKeyFormat) const {
         bb.append(metaFieldIndexKey, metadata().getIndexKey());
     if (metadata().hasSearchScoreDetails())
         bb.append(metaFieldSearchScoreDetails, metadata().getSearchScoreDetails());
-    if (metadata().hasSearchSortValues()) {
-        bb.append(metaFieldSearchSortValues, metadata().getSearchSortValues());
-    }
     return bb.obj();
 }
 
@@ -538,7 +579,7 @@ Document Document::fromBsonWithMetaData(const BSONObj& bson) {
     return md.freeze();
 }
 
-Document Document::getOwned() const {
+Document Document::getOwned() const& {
     if (isOwned()) {
         return *this;
     } else {
@@ -548,6 +589,16 @@ Document Document::getOwned() const {
     }
 }
 
+Document Document::getOwned() && {
+    if (isOwned()) {
+        return std::move(*this);
+    } else {
+        MutableDocument md(std::move(*this));
+        md.makeOwned();
+        return md.freeze();
+    }
+}
+
 MutableDocument::MutableDocument(size_t expectedFields)
     : _storageHolder(nullptr), _storage(_storageHolder) {
     if (expectedFields) {
@@ -566,6 +617,7 @@ MutableValue MutableDocument::getNestedFieldHelper(const FieldPath& dottedField,
 
 MutableValue MutableDocument::getNestedField(const FieldPath& dottedField) {
     fassert(16601, dottedField.getPathLength());
+    assertFieldPathLengthOK(dottedField);
     return getNestedFieldHelper(dottedField, 0);
 }
 
@@ -581,31 +633,10 @@ MutableValue MutableDocument::getNestedFieldHelper(const vector<Position>& posit
 
 MutableValue MutableDocument::getNestedField(const vector<Position>& positions) {
     fassert(16488, !positions.empty());
+    assertFieldPathLengthOK(positions);
     return getNestedFieldHelper(positions, 0);
 }
 
-namespace {
-stdx::variant<BSONElement, Value, Document::TraversesArrayTag, stdx::monostate>
-getNestedFieldHelperBSON(BSONElement elt, const FieldPath& fp, size_t level) {
-    if (level == fp.getPathLength()) {
-        if (elt.ok()) {
-            return elt;
-        } else {
-            return stdx::monostate{};
-        }
-    }
-
-    if (elt.type() == BSONType::Array) {
-        return Document::TraversesArrayTag{};
-    } else if (elt.type() == BSONType::Object) {
-        auto subFieldElt = elt.embeddedObject()[fp.getFieldName(level)];
-        return getNestedFieldHelperBSON(subFieldElt, fp, level + 1);
-    }
-
-    // The path continues "past" a scalar, and therefore does not exist.
-    return stdx::monostate{};
-}
-}  // namespace
 
 stdx::variant<BSONElement, Value, Document::TraversesArrayTag, stdx::monostate>
 Document::getNestedFieldNonCachingHelper(const FieldPath& dottedField, size_t level) const {
@@ -669,36 +700,23 @@ static Value getNestedFieldHelper(const Document& doc,
     return getNestedFieldHelper(val.getDocument(), fieldNames, positions, level + 1);
 }
 
-const Value Document::getNestedField(const FieldPath& path, vector<Position>* positions) const {
+Value Document::getNestedField(const FieldPath& path, vector<Position>* positions) const {
     fassert(16489, path.getPathLength());
+    assertFieldPathLengthOK(path);
     return getNestedFieldHelper(*this, path, positions, 0);
 }
 
-size_t Document::getApproximateSizeWithoutBackingBSON() const {
-    size_t size = sizeof(Document);
-    if (!_storage)
-        return size;
-
-    size += sizeof(DocumentStorage);
-    size += storage().allocatedBytes();
-
-    for (auto it = storage().iteratorCacheOnly(); !it.atEnd(); it.advance()) {
-        size += it->val.getApproximateSize();
-        size -= sizeof(Value);  // already accounted for above
-    }
-
-    // The metadata also occupies space in the document storage that's pre-allocated.
-    size += storage().getMetadataApproximateSize();
-
-    return size;
+size_t Document::getApproximateSize() const {
+    return sizeof(Document) + storage().snapshottedApproximateSize();
 }
 
-size_t Document::getApproximateSize() const {
-    return getApproximateSizeWithoutBackingBSON() + storage().bsonObjSize();
+size_t Document::getCurrentApproximateSize() const {
+    return sizeof(Document) + storage().currentApproximateSize();
 }
 
 size_t Document::memUsageForSorter() const {
-    return getApproximateSizeWithoutBackingBSON() + storage().nonCachedBsonObjSize();
+    return storage().currentApproximateSize() - storage().bsonObjSize() +
+        storage().nonCachedBsonObjSize();
 }
 
 void Document::hash_combine(size_t& seed,
