@@ -59,26 +59,6 @@ class MutableDocument;
  */
 class Position;
 
-/**
- * "Sort keys" are stored in memory as a Value with Array type (with an exception for singleton sort
- * keys). When serializing a sort key for storage in document metadata or as part of a
- * {$meta: "sortKey"} projection, there are three possible storage formats:
- */
-enum class SortKeyFormat {
-    // We expect the in-memory sort key to have one object, which has the format:
-    // {_data: ..., _typeBits:...}. This same object becomes the serialized sort key. This format
-    // exists for compatibility with 4.2 and will be deleted in 4.6 (SERVER-43361).
-    k42ChangeStreamSortKey,
-
-    // A sort key with values "a" and "b" would get translated to an object that looks like:
-    // {"": "a", "": "b"}. Also scheduled for deletion in 4.6.
-    k42SortKey,
-
-    // A sort key with values "a" and "b" would get translated to an array that looks like:
-    // ["a", "b"].
-    k44SortKey,
-};
-
 /** A Document is similar to a BSONObj but with a different in-memory representation.
  *
  *  A Document can be treated as a const std::map<std::string, const Value> that is
@@ -120,7 +100,6 @@ public:
     static constexpr StringData metaFieldSearchScore = "$searchScore"_sd;
     static constexpr StringData metaFieldSearchHighlights = "$searchHighlights"_sd;
     static constexpr StringData metaFieldSearchScoreDetails = "$searchScoreDetails"_sd;
-    static constexpr StringData metaFieldSearchSortValues = "$searchSortValues"_sd;
     static constexpr StringData metaFieldIndexKey = "$indexKey"_sd;
 
     static const StringDataSet allMetadataFieldNames;
@@ -141,6 +120,7 @@ public:
      * auto document = Document{{"hello", "world"}, {"number": 1}};
      */
     Document(std::initializer_list<std::pair<StringData, ImplicitValue>> initializerList);
+    Document(std::vector<std::pair<StringData, Value>> fields);
 
     void swap(Document& rhs) {
         _storage.swap(rhs._storage);
@@ -150,19 +130,23 @@ public:
     Document& operator=(const Document&) = default;
     Document& operator=(Document&&) = default;
 
-    /// Look up a field by key name. Returns Value() if no such field. O(1)
-    const Value operator[](StringData key) const {
+    /**
+     * Look up a field by key name. Returns Value() if no such field. O(1).
+     *
+     * Note that this method does *not* traverse nested documents and arrays, use getNestedField()
+     * instead.
+     */
+    template <typename T>
+    Value operator[](T key) const {
         return getField(key);
     }
-    const Value getField(StringData key) const {
+    template <typename T>
+    Value getField(T key) const {
         return storage().getField(key);
     }
 
     /// Look up a field by Position. See positionOf and getNestedField.
-    const Value operator[](Position pos) const {
-        return getField(pos);
-    }
-    const Value getField(Position pos) const {
+    Value getField(Position pos) const {
         return storage().getField(pos).val;
     }
 
@@ -171,8 +155,7 @@ public:
      * If 'positions' is non-null, it will be filled with a path suitable to pass to
      * MutableDocument::setNestedField().
      */
-    const Value getNestedField(const FieldPath& path,
-                               std::vector<Position>* positions = nullptr) const;
+    Value getNestedField(const FieldPath& path, std::vector<Position>* positions = nullptr) const;
 
     /**
      * Returns field at given path as either BSONElement or Value, depending on how it is
@@ -207,7 +190,8 @@ public:
 
     /**
      * Get the approximate size of the Document, plus its underlying storage and sub-values. Returns
-     * size in bytes.
+     * size in bytes. The return value of this function is snapshotted. All subsequent calls of this
+     * method will return the same value.
      *
      * Note: Some memory may be shared with other Documents or between fields within a single
      * Document so this can overestimate usage.
@@ -217,6 +201,11 @@ public:
      */
     size_t getApproximateSize() const;
 
+    /**
+     * Same as 'getApproximateSize()', but this method re-computes the size on every call.
+     */
+    size_t getCurrentApproximateSize() const;
+
     /**
      * Return the approximate amount of space used by metadata.
      */
@@ -254,6 +243,13 @@ public:
         return out << doc.toString();
     }
 
+    /**
+     * Populates the internal cache by recursively walking the underlying BSON.
+     */
+    void fillCache() const {
+        storage().fillCache();
+    }
+
     /** Calculate a hash value.
      *
      * Meant to be used to create composite hashes suitable for
@@ -279,16 +275,9 @@ public:
     boost::optional<BSONObj> toBsonIfTriviallyConvertible() const;
 
     /**
-     * Like the 'toBson()' method, but includes metadata at the top-level. When the metadata
-     * includes a sort key, the 'sortKeyFormat' parameter controls how it gets converted from its
-     * in-memory representation as a Value to its serialized representation as either a BSONObj or
-     * BSONArray. The possible formats are described in the comment above the 'SortKeyFormat' enum.
-     *
-     * Note that the 'fromBsonWithMetaData()' function does not need a corresponding 'sortKeyFormat'
-     * parameter, because sort key deserialization is able to infer the sort key format based on the
-     * layout of the object.
+     * Like the 'toBson()' method, but includes metadata as top-level fields.
      */
-    BSONObj toBsonWithMetaData(SortKeyFormat sortKeyFormat) const;
+    BSONObj toBsonWithMetaData() const;
 
     /**
      * Like Document(BSONObj) but treats top-level fields with special names as metadata.
@@ -343,7 +332,8 @@ public:
     /**
      * Returns a document that owns the underlying BSONObj.
      */
-    Document getOwned() const;
+    Document getOwned() const&;
+    Document getOwned() &&;
 
     /**
      * Returns true if the underlying BSONObj is owned.
@@ -385,12 +375,6 @@ private:
     getNestedFieldNonCachingHelper(const FieldPath& dottedField, size_t level) const;
 
     boost::intrusive_ptr<const DocumentStorage> _storage;
-
-    /**
-     * Returns the approximate size of this `Document` instance without considering the size of its
-     * backing BSON object.
-     */
-    size_t getApproximateSizeWithoutBackingBSON() const;
 };
 
 //
@@ -549,8 +533,18 @@ public:
      *        Decide what level of support is needed for duplicate fields.
      *        If duplicates are not allowed, consider removing this method.
      */
-    void addField(StringData fieldName, const Value& val) {
-        storage().appendField(fieldName, ValueElement::Kind::kInserted) = val;
+    void addField(StringData name, const Value& val) {
+        storage().appendField(name, ValueElement::Kind::kInserted) = val;
+    }
+    void addField(HashedFieldName field, const Value& val) {
+        storage().appendField(field, ValueElement::Kind::kInserted) = val;
+    }
+
+    void addField(StringData name, Value&& val) {
+        storage().appendField(name, ValueElement::Kind::kInserted) = std::move(val);
+    }
+    void addField(HashedFieldName field, Value&& val) {
+        storage().appendField(field, ValueElement::Kind::kInserted) = std::move(val);
     }
 
     /** Update field by key. If there is no field with that key, add one.
@@ -563,6 +557,9 @@ public:
     void setField(StringData key, const Value& val) {
         getField(key) = val;
     }
+    void setField(StringData key, Value&& val) {
+        getField(key) = std::move(val);
+    }
     MutableValue getField(StringData key) {
         return MutableValue(storage().getField(key, DocumentStorage::LookupPolicy::kCacheOnly));
     }
@@ -577,6 +574,9 @@ public:
     void setField(Position pos, const Value& val) {
         getField(pos) = val;
     }
+    void setField(Position pos, Value&& val) {
+        getField(pos) = std::move(val);
+    }
     MutableValue getField(Position pos) {
         return MutableValue(storage().getField(pos).val);
     }
@@ -599,12 +599,18 @@ public:
     void setNestedField(const FieldPath& dottedField, const Value& val) {
         getNestedField(dottedField) = val;
     }
+    void setNestedField(const FieldPath& dottedField, Value&& val) {
+        getNestedField(dottedField) = std::move(val);
+    }
 
     /// Takes positions vector from Document::getNestedField. All fields in path must exist.
     MutableValue getNestedField(const std::vector<Position>& positions);
     void setNestedField(const std::vector<Position>& positions, const Value& val) {
         getNestedField(positions) = val;
     }
+    void setNestedField(const std::vector<Position>& positions, Value&& val) {
+        getNestedField(positions) = std::move(val);
+    }
 
     /**
      * Copies all metadata from source if it has any.
@@ -647,6 +653,7 @@ public:
      *  TODO: there are some optimizations that may make sense at freeze time.
      */
     Document freeze() {
+        resetSnapshottedApproximateSize();
         // This essentially moves _storage into a new Document by way of temp.
         Document ret;
         boost::intrusive_ptr<const DocumentStorage> temp(storagePtr(), /*inc_ref_count=*/false);
@@ -665,8 +672,12 @@ public:
      *  Note that unlike freeze(), this indicates intention to continue
      *  modifying this document. The returned Document will not observe
      *  future changes to this MutableDocument.
+     *
+     *  Note that the computed snapshotted approximate size of the Document
+     *  is not preserved across calls.
      */
     Document peek() {
+        resetSnapshottedApproximateSize();
         return Document(storagePtr());
     }
 
@@ -732,12 +743,19 @@ private:
     MutableValue getNestedFieldHelper(const FieldPath& dottedField, size_t level);
     MutableValue getNestedFieldHelper(const std::vector<Position>& positions, size_t level);
 
-    // this should only be called by storage methods and peek/freeze
+    // this should only be called by storage methods and peek/freeze/resetsnapshottedApproximateSize
     const DocumentStorage* storagePtr() const {
         dassert(!_storage || typeid(*_storage) == typeid(const DocumentStorage));
         return static_cast<const DocumentStorage*>(_storage);
     }
 
+    void resetSnapshottedApproximateSize() {
+        auto mutableStorage = const_cast<DocumentStorage*>(storagePtr());
+        if (mutableStorage) {
+            mutableStorage->resetSnapshottedApproximateSize();
+        }
+    }
+
     // These are both const to prevent modifications bypassing storage() method.
     // They always point to NULL or an object with dynamic type DocumentStorage.
     const RefCountable* _storageHolder;  // Only used in constructors and destructor
