@@ -45,7 +45,6 @@
 #include "mongo/db/query/datetime/date_time_support.h"
 #include "mongo/platform/decimal128.h"
 #include "mongo/util/hex.h"
-#include "mongo/util/murmur3.h"
 #include "mongo/util/represent_as.h"
 #include "mongo/util/str.h"
 
@@ -60,8 +59,6 @@ using std::stringstream;
 using std::vector;
 using namespace std::string_literals;
 
-constexpr StringData Value::kISOFormatString;
-
 void ValueStorage::verifyRefCountingIfShould() const {
     switch (type) {
         case MinKey:
@@ -134,6 +131,10 @@ void ValueStorage::putDocument(const Document& d) {
     putRefCountable(d._storage);
 }
 
+void ValueStorage::putDocument(Document&& d) {
+    putRefCountable(std::move(d._storage));
+}
+
 void ValueStorage::putVector(boost::intrusive_ptr<RCVector>&& vec) {
     fassert(16485, bool(vec));
     putRefCountable(std::move(vec));
@@ -162,7 +163,14 @@ Document ValueStorage::getDocument() const {
 
 // not in header because document is fwd declared
 Value::Value(const BSONObj& obj) : _storage(Object, Document(obj.getOwned())) {}
+
+// An option of providing 'Value(Document)' was rejected in favor of 'Value(const Document&)' and
+// 'Value(Document&&)' overloads, and lvalue/rvalue reference overloads of callees, since
+// 'Value(Document)' option with a lvalue parameter would result in one extra move operation in
+// 'ValueStorage::putDocument()'.
 Value::Value(const Document& doc) : _storage(Object, doc.isOwned() ? doc : doc.getOwned()) {}
+Value::Value(Document&& doc)
+    : _storage(Object, doc.isOwned() ? std::move(doc) : std::move(doc).getOwned()) {}
 
 Value::Value(const BSONElement& elem) : _storage(elem.type()) {
     switch (elem.type()) {
@@ -637,7 +645,7 @@ string Value::coerceToString() const {
 
         case Date:
             return uassertStatusOKWithContext(
-                TimeZoneDatabase::utcZone().formatDate(Value::kISOFormatString, getDate()),
+                TimeZoneDatabase::utcZone().formatDate(kISOFormatString, getDate()),
                 "failed while coercing date to string");
 
         case EOO:
@@ -900,6 +908,7 @@ void Value::hash_combine(size_t& seed,
             // Else, fall through and convert the decimal to a double and hash.
             // At this point the decimal fits into the range of doubles, is infinity, or is NaN,
             // which doubles have a cheaper representation for.
+            [[fallthrough]];
         }
         // This converts all numbers to doubles, which ignores the low-order bits of
         // NumberLongs > 2**53 and precise decimal numbers without double representations,
@@ -927,7 +936,7 @@ void Value::hash_combine(size_t& seed,
         case Code:
         case Symbol: {
             StringData sd = getRawData();
-            seed = murmur3<sizeof(size_t)>(sd, seed);
+            MurmurHash3_x86_32(sd.rawData(), sd.size(), seed, &seed);
             break;
         }
 
@@ -936,7 +945,7 @@ void Value::hash_combine(size_t& seed,
             if (stringComparator) {
                 stringComparator->hash_combine(seed, sd);
             } else {
-                seed = murmur3<sizeof(size_t)>(sd, seed);
+                MurmurHash3_x86_32(sd.rawData(), sd.size(), seed, &seed);
             }
             break;
         }
@@ -960,14 +969,14 @@ void Value::hash_combine(size_t& seed,
 
         case BinData: {
             StringData sd = getRawData();
-            seed = murmur3<sizeof(size_t)>(sd, seed);
+            MurmurHash3_x86_32(sd.rawData(), sd.size(), seed, &seed);
             boost::hash_combine(seed, _storage.binDataType());
             break;
         }
 
         case RegEx: {
             StringData sd = getRawData();
-            seed = murmur3<sizeof(size_t)>(sd, seed);
+            MurmurHash3_x86_32(sd.rawData(), sd.size(), seed, &seed);
             break;
         }
 
@@ -1063,6 +1072,36 @@ bool Value::integral() const {
     }
 }
 
+bool Value::isNaN() const {
+    switch (getType()) {
+        case NumberInt:
+        case NumberLong:
+        case NumberDouble: {
+            const double dbl = getDouble();
+            return std::isnan(dbl);
+        }
+        case NumberDecimal: {
+            return _storage.getDecimal().isNaN();
+        }
+
+        default:
+            return false;
+    }
+}
+
+bool Value::isInfinite() const {
+    switch (getType()) {
+        case NumberDouble:
+            return (_storage.doubleValue == std::numeric_limits<double>::infinity() ||
+                    _storage.doubleValue == -std::numeric_limits<double>::infinity());
+        case NumberDecimal:
+            return _storage.getDecimal().isInfinite();
+
+        default:
+            return false;
+    }
+}
+
 bool Value::integral64Bit() const {
     switch (getType()) {
         case NumberInt:
@@ -1175,7 +1214,7 @@ ostream& operator<<(ostream& out, const Value& val) {
             return out << "undefined";
         case Date:
             return out << [&] {
-                if (auto string = TimeZoneDatabase::utcZone().formatDate(Value::kISOFormatString,
+                if (auto string = TimeZoneDatabase::utcZone().formatDate(kISOFormatString,
                                                                          val.coerceToDate());
                     string.isOK())
                     return string.getValue();
@@ -1204,8 +1243,7 @@ ostream& operator<<(ostream& out, const Value& val) {
 
         case BinData:
             return out << "BinData(" << val._storage.binDataType() << ", \""
-                       << toHex(val._storage.getString().rawData(), val._storage.getString().size())
-                       << "\")";
+                       << hexblob::encode(val._storage.getString()) << "\")";
 
         case DBRef:
             return out << "DBRef(\"" << val._storage.getDBRef()->ns << "\", "
@@ -1216,6 +1254,16 @@ ostream& operator<<(ostream& out, const Value& val) {
     verify(false);
 }
 
+void Value::fillCache() const {
+    if (isObject()) {
+        getDocument().fillCache();
+    } else if (isArray()) {
+        for (auto&& val : getArray()) {
+            val.fillCache();
+        }
+    }
+}
+
 void Value::serializeForSorter(BufBuilder& buf) const {
     buf.appendChar(getType());
     switch (getType()) {
@@ -1399,4 +1447,10 @@ Value Value::deserializeForIDL(const BSONElement& element) {
     return Value(element);
 }
 
+BSONObj Value::wrap(StringData newName) const {
+    BSONObjBuilder b(getApproximateSize() + 6 + newName.size());
+    addToBsonObj(&b, newName);
+    return b.obj();
+}
+
 }  // namespace mongo
