@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/db/exec/geo_near.h"
 
@@ -38,7 +37,6 @@
 // For s2 search
 #include "third_party/s2/s2regionintersection.h"
 
-#include "mongo/base/owned_pointer_vector.h"
 #include "mongo/db/bson/dotted_path_support.h"
 #include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/exec/fetch.h"
@@ -52,13 +50,14 @@
 
 #include <algorithm>
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
+
+
 namespace mongo {
 
 using std::abs;
 using std::unique_ptr;
 
-namespace dps = ::mongo::dotted_path_support;
-
 //
 // Shared GeoNear search functionality
 //
@@ -67,79 +66,7 @@ static const double kCircOfEarthInMeters = 2 * M_PI * kRadiusOfEarthInMeters;
 static const double kMaxEarthDistanceInMeters = kCircOfEarthInMeters / 2;
 static const double kMetersPerDegreeAtEquator = kCircOfEarthInMeters / 360;
 
-namespace {
-
-/**
- * Structure that holds BSON addresses (BSONElements) and the corresponding geometry parsed
- * at those locations.
- * Used to separate the parsing of geometries from a BSONObj (which must stay in scope) from
- * the computation over those geometries.
- * TODO: Merge with 2D/2DSphere key extraction?
- */
-struct StoredGeometry {
-    static StoredGeometry* parseFrom(const BSONElement& element) {
-        if (!element.isABSONObj())
-            return nullptr;
-
-        unique_ptr<StoredGeometry> stored(new StoredGeometry);
-
-        // GeoNear stage can only be run with an existing index
-        // Therefore, it is always safe to skip geometry validation
-        if (!stored->geometry.parseFromStorage(element, true).isOK())
-            return nullptr;
-        stored->element = element;
-        return stored.release();
-    }
-
-    BSONElement element;
-    GeometryContainer geometry;
-};
-}  // namespace
-
-/**
- * Find and parse all geometry elements on the appropriate field path from the document.
- */
-static void extractGeometries(const BSONObj& doc,
-                              const string& path,
-                              std::vector<std::unique_ptr<StoredGeometry>>* geometries) {
-    BSONElementSet geomElements;
-    // NOTE: Annoyingly, we cannot just expand arrays b/c single 2d points are arrays, we need
-    // to manually expand all results to check if they are geometries
-    dps::extractAllElementsAlongPath(doc, path, geomElements, false /* expand arrays */);
-
-    for (BSONElementSet::iterator it = geomElements.begin(); it != geomElements.end(); ++it) {
-        const BSONElement& el = *it;
-        unique_ptr<StoredGeometry> stored(StoredGeometry::parseFrom(el));
-
-        if (stored.get()) {
-            // Valid geometry element
-            geometries->push_back(std::move(stored));
-        } else if (el.type() == Array) {
-            // Many geometries may be in an array
-            BSONObjIterator arrIt(el.Obj());
-            while (arrIt.more()) {
-                const BSONElement nextEl = arrIt.next();
-                stored.reset(StoredGeometry::parseFrom(nextEl));
-
-                if (stored.get()) {
-                    // Valid geometry element
-                    geometries->push_back(std::move(stored));
-                } else {
-                    LOGV2_WARNING(23760,
-                                  "geoNear stage read non-geometry element {nextEl} in array {el}",
-                                  "nextEl"_attr = redact(nextEl),
-                                  "el"_attr = redact(el));
-                }
-            }
-        } else {
-            LOGV2_WARNING(
-                23761, "geoNear stage read non-geometry element {el}", "el"_attr = redact(el));
-        }
-    }
-}
-
-static StatusWith<double> computeGeoNearDistance(const GeoNearParams& nearParams,
-                                                 WorkingSetMember* member) {
+static double computeGeoNearDistance(const GeoNearParams& nearParams, WorkingSetMember* member) {
     //
     // Generic GeoNear distance computation
     // Distances are computed by projecting the stored geometry into the query CRS, and
@@ -155,7 +82,8 @@ static StatusWith<double> computeGeoNearDistance(const GeoNearParams& nearParams
     // stage can only be run with an existing index. Therefore, it is always safe to skip geometry
     // validation.
     std::vector<std::unique_ptr<StoredGeometry>> geometries;
-    extractGeometries(member->doc.value().toBson(), nearParams.nearQuery->field, &geometries);
+    StoredGeometry::extractGeometries(
+        member->doc.value().toBson(), nearParams.nearQuery->field, &geometries, true);
 
     // Compute the minimum distance of all the geometries in the document
     double minDistance = -1;
@@ -185,7 +113,7 @@ static StatusWith<double> computeGeoNearDistance(const GeoNearParams& nearParams
 
     if (minDistance < 0) {
         // No distance to report
-        return StatusWith<double>(-1);
+        return -1;
     }
 
     if (nearParams.addDistMeta) {
@@ -203,7 +131,7 @@ static StatusWith<double> computeGeoNearDistance(const GeoNearParams& nearParams
         member->metadata().setGeoNearPoint(minDistanceMetadata);
     }
 
-    return StatusWith<double>(minDistance);
+    return minDistance;
 }
 
 static R2Annulus geoNearDistanceBounds(const GeoNearExpression& query) {
@@ -247,13 +175,13 @@ static R2Annulus twoDDistanceBounds(const GeoNearParams& nearParams,
     const CRS queryCRS = nearParams.nearQuery->centroid->crs;
 
     if (FLAT == queryCRS) {
-        // Reset the full bounds based on our index bounds
-        GeoHashConverter::Parameters hashParams;
-        Status status = GeoHashConverter::parseParameters(twoDIndex->infoObj(), &hashParams);
-        invariantStatusOK(status);  // The index status should always be valid
+        // Reset the full bounds based on our index bounds.
+        // The index status should always be valid.
+        auto result = invariantStatusOK(GeoHashConverter::createFromDoc(twoDIndex->infoObj()));
 
         // The biggest distance possible in this indexed collection is the diagonal of the
         // square indexed region.
+        const GeoHashConverter::Parameters& hashParams = result->getParams();
         const double sqrt2Approx = 1.5;
         const double diagonalDist = sqrt2Approx * (hashParams.max - hashParams.min);
 
@@ -269,23 +197,26 @@ static R2Annulus twoDDistanceBounds(const GeoNearParams& nearParams,
     return fullBounds;
 }
 
-GeoNear2DStage::DensityEstimator::DensityEstimator(PlanStage::Children* children,
+GeoNear2DStage::DensityEstimator::DensityEstimator(const CollectionPtr& collection,
+                                                   PlanStage::Children* children,
                                                    BSONObj infoObj,
                                                    const GeoNearParams* nearParams,
                                                    const R2Annulus& fullBounds)
-    : _children(children), _nearParams(nearParams), _fullBounds(fullBounds), _currentLevel(0) {
-    GeoHashConverter::Parameters hashParams;
-    Status status = GeoHashConverter::parseParameters(std::move(infoObj), &hashParams);
+    : _collection(collection),
+      _children(children),
+      _nearParams(nearParams),
+      _fullBounds(fullBounds),
+      _currentLevel(0) {
     // The index status should always be valid.
-    invariantStatusOK(status);
+    auto result = invariantStatusOK(GeoHashConverter::createFromDoc(std::move(infoObj)));
 
-    _converter.reset(new GeoHashConverter(hashParams));
+    _converter = std::move(result);
     _centroidCell = _converter->hash(_nearParams->nearQuery->centroid->oldPoint);
 
     // Since appendVertexNeighbors(level, output) requires level < hash.getBits(),
     // we have to start to find documents at most GeoHash::kMaxBits - 1. Thus the finest
     // search area is 16 * finest cell area at GeoHash::kMaxBits.
-    _currentLevel = std::max(0, hashParams.bits - 1);
+    _currentLevel = std::max(0, _converter->getParams().bits - 1);
 }
 
 // Initialize the internal states
@@ -294,7 +225,7 @@ void GeoNear2DStage::DensityEstimator::buildIndexScan(ExpressionContext* expCtx,
                                                       const IndexDescriptor* twoDIndex) {
     // Scan bounds on 2D indexes are only over the 2D field - other bounds aren't applicable.
     // This is handled in query planning.
-    IndexScanParams scanParams(expCtx->opCtx, twoDIndex);
+    IndexScanParams scanParams(expCtx->opCtx, _collection, twoDIndex);
     scanParams.bounds = _nearParams->baseBounds;
 
     // The "2d" field is always the first in the index
@@ -325,7 +256,7 @@ void GeoNear2DStage::DensityEstimator::buildIndexScan(ExpressionContext* expCtx,
     IndexBoundsBuilder::intersectize(oil, &scanParams.bounds.fields[twoDFieldPosition]);
 
     invariant(!_indexScan);
-    _indexScan = new IndexScan(expCtx, scanParams, workingSet, nullptr);
+    _indexScan = new IndexScan(expCtx, _collection, scanParams, workingSet, nullptr);
     _children->emplace_back(_indexScan);
 }
 
@@ -411,7 +342,7 @@ PlanStage::StageState GeoNear2DStage::initialize(OperationContext* opCtx,
                                                  WorkingSetID* out) {
     if (!_densityEstimator) {
         _densityEstimator.reset(new DensityEstimator(
-            &_children, indexDescriptor()->infoObj(), &_nearParams, _fullBounds));
+            collection(), &_children, indexDescriptor()->infoObj(), &_nearParams, _fullBounds));
     }
 
     double estimatedDistance;
@@ -454,8 +385,14 @@ static const string kTwoDIndexNearStage("GEO_NEAR_2D");
 GeoNear2DStage::GeoNear2DStage(const GeoNearParams& nearParams,
                                ExpressionContext* expCtx,
                                WorkingSet* workingSet,
+                               const CollectionPtr& collection,
                                const IndexDescriptor* twoDIndex)
-    : NearStage(expCtx, kTwoDIndexNearStage.c_str(), STAGE_GEO_NEAR_2D, workingSet, twoDIndex),
+    : NearStage(expCtx,
+                kTwoDIndexNearStage.c_str(),
+                STAGE_GEO_NEAR_2D,
+                workingSet,
+                collection,
+                twoDIndex),
       _nearParams(nearParams),
       _fullBounds(twoDDistanceBounds(nearParams, twoDIndex)),
       _currBounds(_fullBounds.center(), -1, _fullBounds.getInner()),
@@ -467,62 +404,6 @@ GeoNear2DStage::GeoNear2DStage(const GeoNearParams& nearParams,
 
 
 namespace {
-
-/**
- * Expression which checks whether a legacy 2D index point is contained within our near
- * search annulus.  See nextInterval() below for more discussion.
- * TODO: Make this a standard type of GEO match expression
- */
-class TwoDPtInAnnulusExpression : public LeafMatchExpression {
-public:
-    TwoDPtInAnnulusExpression(const R2Annulus& annulus, StringData twoDPath)
-        : LeafMatchExpression(INTERNAL_2D_POINT_IN_ANNULUS, twoDPath), _annulus(annulus) {}
-
-    void serialize(BSONObjBuilder* out, bool includePath) const final {
-        out->append("TwoDPtInAnnulusExpression", true);
-    }
-
-    bool matchesSingleElement(const BSONElement& e, MatchDetails* details = nullptr) const final {
-        if (!e.isABSONObj())
-            return false;
-
-        PointWithCRS point;
-        if (!GeoParser::parseStoredPoint(e, &point).isOK())
-            return false;
-
-        return _annulus.contains(point.oldPoint);
-    }
-
-    //
-    // These won't be called.
-    //
-
-    BSONObj getSerializedRightHandSide() const final {
-        MONGO_UNREACHABLE;
-    }
-
-    void debugString(StringBuilder& debug, int level = 0) const final {
-        MONGO_UNREACHABLE;
-    }
-
-    bool equivalent(const MatchExpression* other) const final {
-        MONGO_UNREACHABLE;
-        return false;
-    }
-
-    unique_ptr<MatchExpression> shallowClone() const final {
-        MONGO_UNREACHABLE;
-        return nullptr;
-    }
-
-private:
-    ExpressionOptimizerFunc getOptimizer() const final {
-        return [](std::unique_ptr<MatchExpression> expression) { return expression; };
-    }
-
-    R2Annulus _annulus;
-};
-
 // Helper class to maintain ownership of a match expression alongside an index scan
 class FetchStageWithMatch final : public FetchStage {
 public:
@@ -530,7 +411,7 @@ public:
                         WorkingSet* ws,
                         std::unique_ptr<PlanStage> child,
                         MatchExpression* filter,
-                        const Collection* collection)
+                        const CollectionPtr& collection)
         : FetchStage(expCtx, ws, std::move(child), filter, collection), _matcher(filter) {}
 
 private:
@@ -541,15 +422,13 @@ private:
 
 static double min2DBoundsIncrement(const GeoNearExpression& query,
                                    const IndexDescriptor* twoDIndex) {
-    GeoHashConverter::Parameters hashParams;
-    Status status = GeoHashConverter::parseParameters(twoDIndex->infoObj(), &hashParams);
-    invariantStatusOK(status);  // The index status should always be valid
-    GeoHashConverter hasher(hashParams);
+    // The index status should always be valid.
+    auto result = invariantStatusOK(GeoHashConverter::createFromDoc(twoDIndex->infoObj()));
 
     // The hasher error is the diagonal of a 2D hash region - it's generally not helpful
     // to change region size such that a search radius is smaller than the 2D hash region
     // max radius.  This is slightly conservative for now (box diagonal vs circle radius).
-    double minBoundsIncrement = hasher.getError() / 2;
+    const double minBoundsIncrement = result->getError() / 2;
 
     const CRS queryCRS = query.centroid->crs;
     if (FLAT == queryCRS)
@@ -571,13 +450,11 @@ static R2Annulus projectBoundsToTwoDDegrees(R2Annulus sphereBounds) {
                      outerDegrees + maxErrorDegrees);
 }
 
-StatusWith<NearStage::CoveredInterval*>  //
-GeoNear2DStage::nextInterval(OperationContext* opCtx,
-                             WorkingSet* workingSet,
-                             const Collection* collection) {
+std::unique_ptr<NearStage::CoveredInterval> GeoNear2DStage::nextInterval(
+    OperationContext* opCtx, WorkingSet* workingSet, const CollectionPtr& collection) {
     // The search is finished if we searched at least once and all the way to the edge
     if (_currBounds.getInner() >= 0 && _currBounds.getOuter() == _fullBounds.getOuter()) {
-        return StatusWith<CoveredInterval*>(nullptr);
+        return nullptr;
     }
 
     //
@@ -666,7 +543,7 @@ GeoNear2DStage::nextInterval(OperationContext* opCtx,
 
     // Scan bounds on 2D indexes are only over the 2D field - other bounds aren't applicable.
     // This is handled in query planning.
-    IndexScanParams scanParams(opCtx, indexDescriptor());
+    IndexScanParams scanParams(opCtx, collection, indexDescriptor());
 
     // This does force us to do our own deduping of results.
     scanParams.bounds = _nearParams.baseBounds;
@@ -699,12 +576,11 @@ GeoNear2DStage::nextInterval(OperationContext* opCtx,
                                      &scanParams.bounds.fields[twoDFieldPosition]);
 
     // These parameters are stored by the index, and so must be ok
-    GeoHashConverter::Parameters hashParams;
-    GeoHashConverter::parseParameters(indexDescriptor()->infoObj(), &hashParams)
-        .transitional_ignore();
+    invariantStatusOK(GeoHashConverter::createFromDoc(indexDescriptor()->infoObj()));
 
     // 2D indexes support covered search over additional fields they contain
-    auto scan = std::make_unique<IndexScan>(expCtx(), scanParams, workingSet, _nearParams.filter);
+    auto scan = std::make_unique<IndexScan>(
+        expCtx(), collection, scanParams, workingSet, _nearParams.filter);
 
     MatchExpression* docMatcher = nullptr;
 
@@ -718,11 +594,11 @@ GeoNear2DStage::nextInterval(OperationContext* opCtx,
     _children.emplace_back(std::make_unique<FetchStageWithMatch>(
         expCtx(), workingSet, std::move(scan), docMatcher, collection));
 
-    return StatusWith<CoveredInterval*>(new CoveredInterval(
-        _children.back().get(), nextBounds.getInner(), nextBounds.getOuter(), isLastInterval));
+    return std::make_unique<CoveredInterval>(
+        _children.back().get(), nextBounds.getInner(), nextBounds.getOuter(), isLastInterval);
 }
 
-StatusWith<double> GeoNear2DStage::computeDistance(WorkingSetMember* member) {
+double GeoNear2DStage::computeDistance(WorkingSetMember* member) {
     return computeGeoNearDistance(_nearParams, member);
 }
 
@@ -752,8 +628,14 @@ static const string kS2IndexNearStage("GEO_NEAR_2DSPHERE");
 GeoNear2DSphereStage::GeoNear2DSphereStage(const GeoNearParams& nearParams,
                                            ExpressionContext* expCtx,
                                            WorkingSet* workingSet,
+                                           const CollectionPtr& collection,
                                            const IndexDescriptor* s2Index)
-    : NearStage(expCtx, kS2IndexNearStage.c_str(), STAGE_GEO_NEAR_2DSPHERE, workingSet, s2Index),
+    : NearStage(expCtx,
+                kS2IndexNearStage.c_str(),
+                STAGE_GEO_NEAR_2DSPHERE,
+                workingSet,
+                collection,
+                s2Index),
       _nearParams(nearParams),
       _fullBounds(geoNearDistanceBounds(*nearParams.nearQuery)),
       _currBounds(_fullBounds.center(), -1, _fullBounds.getInner()),
@@ -770,8 +652,6 @@ GeoNear2DSphereStage::GeoNear2DSphereStage(const GeoNearParams& nearParams,
     ExpressionParams::initialize2dsphereParams(s2Index->infoObj(), collator, &_indexParams);
 }
 
-GeoNear2DSphereStage::~GeoNear2DSphereStage() {}
-
 namespace {
 
 S2Region* buildS2Region(const R2Annulus& sphereBounds) {
@@ -828,11 +708,13 @@ S2Region* buildS2Region(const R2Annulus& sphereBounds) {
 }
 }  // namespace
 
-GeoNear2DSphereStage::DensityEstimator::DensityEstimator(PlanStage::Children* children,
+GeoNear2DSphereStage::DensityEstimator::DensityEstimator(const CollectionPtr& collection,
+                                                         PlanStage::Children* children,
                                                          const GeoNearParams* nearParams,
                                                          const S2IndexingParams& indexParams,
                                                          const R2Annulus& fullBounds)
-    : _children(children),
+    : _collection(collection),
+      _children(children),
       _nearParams(nearParams),
       _indexParams(indexParams),
       _fullBounds(fullBounds),
@@ -847,7 +729,7 @@ GeoNear2DSphereStage::DensityEstimator::DensityEstimator(PlanStage::Children* ch
 void GeoNear2DSphereStage::DensityEstimator::buildIndexScan(ExpressionContext* expCtx,
                                                             WorkingSet* workingSet,
                                                             const IndexDescriptor* s2Index) {
-    IndexScanParams scanParams(expCtx->opCtx, s2Index);
+    IndexScanParams scanParams(expCtx->opCtx, _collection, s2Index);
     scanParams.bounds = _nearParams->baseBounds;
 
     // Because the planner doesn't yet set up 2D index bounds, do it ourselves here
@@ -870,7 +752,7 @@ void GeoNear2DSphereStage::DensityEstimator::buildIndexScan(ExpressionContext* e
 
     // Index scan
     invariant(!_indexScan);
-    _indexScan = new IndexScan(expCtx, scanParams, workingSet, nullptr);
+    _indexScan = new IndexScan(expCtx, _collection, scanParams, workingSet, nullptr);
     _children->emplace_back(_indexScan);
 }
 
@@ -956,8 +838,8 @@ PlanStage::StageState GeoNear2DSphereStage::initialize(OperationContext* opCtx,
                                                        WorkingSet* workingSet,
                                                        WorkingSetID* out) {
     if (!_densityEstimator) {
-        _densityEstimator.reset(
-            new DensityEstimator(&_children, &_nearParams, _indexParams, _fullBounds));
+        _densityEstimator.reset(new DensityEstimator(
+            collection(), &_children, &_nearParams, _indexParams, _fullBounds));
     }
 
     double estimatedDistance;
@@ -982,13 +864,11 @@ PlanStage::StageState GeoNear2DSphereStage::initialize(OperationContext* opCtx,
     return state;
 }
 
-StatusWith<NearStage::CoveredInterval*>  //
-GeoNear2DSphereStage::nextInterval(OperationContext* opCtx,
-                                   WorkingSet* workingSet,
-                                   const Collection* collection) {
+std::unique_ptr<NearStage::CoveredInterval> GeoNear2DSphereStage::nextInterval(
+    OperationContext* opCtx, WorkingSet* workingSet, const CollectionPtr& collection) {
     // The search is finished if we searched at least once and all the way to the edge
     if (_currBounds.getInner() >= 0 && _currBounds.getOuter() == _fullBounds.getOuter()) {
-        return StatusWith<CoveredInterval*>(nullptr);
+        return nullptr;
     }
 
     //
@@ -1018,7 +898,7 @@ GeoNear2DSphereStage::nextInterval(OperationContext* opCtx,
     // Setup the covering region and stages for this interval
     //
 
-    IndexScanParams scanParams(opCtx, indexDescriptor());
+    IndexScanParams scanParams(opCtx, collection, indexDescriptor());
 
     // This does force us to do our own deduping of results.
     scanParams.bounds = _nearParams.baseBounds;
@@ -1050,17 +930,17 @@ GeoNear2DSphereStage::nextInterval(OperationContext* opCtx,
     OrderedIntervalList* coveredIntervals = &scanParams.bounds.fields[s2FieldPosition];
     ExpressionMapping::S2CellIdsToIntervalsWithParents(cover, _indexParams, coveredIntervals);
 
-    auto scan = std::make_unique<IndexScan>(expCtx(), scanParams, workingSet, nullptr);
+    auto scan = std::make_unique<IndexScan>(expCtx(), collection, scanParams, workingSet, nullptr);
 
     // FetchStage owns index scan
     _children.emplace_back(std::make_unique<FetchStage>(
         expCtx(), workingSet, std::move(scan), _nearParams.filter, collection));
 
-    return StatusWith<CoveredInterval*>(new CoveredInterval(
-        _children.back().get(), nextBounds.getInner(), nextBounds.getOuter(), isLastInterval));
+    return std::make_unique<CoveredInterval>(
+        _children.back().get(), nextBounds.getInner(), nextBounds.getOuter(), isLastInterval);
 }
 
-StatusWith<double> GeoNear2DSphereStage::computeDistance(WorkingSetMember* member) {
+double GeoNear2DSphereStage::computeDistance(WorkingSetMember* member) {
     return computeGeoNearDistance(_nearParams, member);
 }
 
