@@ -27,8 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
-
 #include "mongo/platform/basic.h"
 
 #include <memory>
@@ -46,7 +44,7 @@
 #include "mongo/db/exec/and_hash.h"
 #include "mongo/db/exec/and_sorted.h"
 #include "mongo/db/exec/collection_scan.h"
-#include "mongo/db/exec/delete.h"
+#include "mongo/db/exec/delete_stage.h"
 #include "mongo/db/exec/fetch.h"
 #include "mongo/db/exec/index_scan.h"
 #include "mongo/db/exec/limit.h"
@@ -54,7 +52,6 @@
 #include "mongo/db/exec/or.h"
 #include "mongo/db/exec/skip.h"
 #include "mongo/db/exec/sort.h"
-#include "mongo/db/exec/text.h"
 #include "mongo/db/exec/working_set_common.h"
 #include "mongo/db/index/fts_access_method.h"
 #include "mongo/db/jsobj.h"
@@ -62,8 +59,7 @@
 #include "mongo/db/matcher/expression_text_base.h"
 #include "mongo/db/matcher/extensions_callback_real.h"
 #include "mongo/db/namespace_string.h"
-#include "mongo/db/query/plan_executor.h"
-#include "mongo/logv2/log.h"
+#include "mongo/db/query/plan_executor_factory.h"
 
 namespace mongo {
 
@@ -128,7 +124,7 @@ public:
     }
 
     bool run(OperationContext* opCtx,
-             const string& dbname,
+             const DatabaseName& dbName,
              const BSONObj& cmdObj,
              BSONObjBuilder& result) {
         BSONElement argElt = cmdObj["stageDebug"];
@@ -143,7 +139,7 @@ public:
             return false;
         }
 
-        const NamespaceString nss(dbname, collElt.String());
+        const NamespaceString nss(dbName, collElt.String());
         uassert(ErrorCodes::InvalidNamespace,
                 str::stream() << nss.toString() << " is not a valid namespace",
                 nss.isValid());
@@ -151,14 +147,14 @@ public:
         auto expCtx = make_intrusive<ExpressionContext>(
             opCtx, std::unique_ptr<CollatorInterface>(nullptr), nss);
 
-        // Need a context to get the actual Collection*
+        // Need a context to get the actual const CollectionPtr&
         // TODO A write lock is currently taken here to accommodate stages that perform writes
         //      (e.g. DeleteStage).  This should be changed to use a read lock for read-only
         //      execution trees.
         AutoGetCollection autoColl(opCtx, nss, MODE_IX);
 
         // Make sure the collection is valid.
-        Collection* collection = autoColl.getCollection();
+        const auto& collection = autoColl.getCollection();
         uassert(ErrorCodes::NamespaceNotFound,
                 str::stream() << "Couldn't find collection " << nss.ns(),
                 collection);
@@ -182,8 +178,13 @@ public:
         unique_ptr<PlanStage> rootFetch = std::make_unique<FetchStage>(
             expCtx.get(), ws.get(), std::move(userRoot), nullptr, collection);
 
-        auto statusWithPlanExecutor = PlanExecutor::make(
-            opCtx, std::move(ws), std::move(rootFetch), collection, PlanExecutor::YIELD_AUTO);
+        auto statusWithPlanExecutor =
+            plan_executor_factory::make(expCtx,
+                                        std::move(ws),
+                                        std::move(rootFetch),
+                                        &collection,
+                                        PlanYieldPolicy::YieldPolicy::YIELD_AUTO,
+                                        false /* whether owned BSON must be returned */);
         fassert(28536, statusWithPlanExecutor.getStatus());
         auto exec = std::move(statusWithPlanExecutor.getValue());
 
@@ -196,23 +197,11 @@ public:
         }
 
         resultBuilder.done();
-
-        if (PlanExecutor::FAILURE == state) {
-            LOGV2_ERROR(23795,
-                        "Plan executor error during StageDebug command: FAILURE, stats: "
-                        "{Explain_getWinningPlanStats_exec_get}",
-                        "Explain_getWinningPlanStats_exec_get"_attr =
-                            redact(Explain::getWinningPlanStats(exec.get())));
-
-            uassertStatusOK(WorkingSetCommon::getMemberObjectStatus(obj).withContext(
-                "Executor error during StageDebug command"));
-        }
-
         return true;
     }
 
     PlanStage* parseQuery(const boost::intrusive_ptr<ExpressionContext>& expCtx,
-                          Collection* collection,
+                          const CollectionPtr& collection,
                           BSONObj obj,
                           WorkingSet* workingSet,
                           const NamespaceString& nss,
@@ -292,16 +281,16 @@ public:
                 uassert(40223, str::stream() << "Can't find index: " << name.toString(), desc);
             }
 
-            IndexScanParams params(opCtx, desc);
+            IndexScanParams params(opCtx, collection, desc);
             params.bounds.isSimpleRange = true;
             params.bounds.startKey = BSONObj::stripFieldNames(nodeArgs["startKey"].Obj());
             params.bounds.endKey = BSONObj::stripFieldNames(nodeArgs["endKey"].Obj());
             params.bounds.boundInclusion = IndexBounds::makeBoundInclusionFromBoundBools(
                 nodeArgs["startKeyInclusive"].Bool(), nodeArgs["endKeyInclusive"].Bool());
             params.direction = nodeArgs["direction"].numberInt();
-            params.shouldDedup = desc->isMultikey();
+            params.shouldDedup = desc->getEntry()->isMultikey(opCtx, collection);
 
-            return new IndexScan(expCtx.get(), params, workingSet, matcher);
+            return new IndexScan(expCtx.get(), collection, params, workingSet, matcher);
         } else if ("andHash" == nodeName) {
             uassert(
                 16921, "Nodes argument must be provided to AND", nodeArgs["nodes"].isABSONObj());
@@ -447,38 +436,6 @@ public:
                 mergeStage->addChild(std::move(subNode));
             }
             return mergeStage.release();
-        } else if ("text" == nodeName) {
-            string search = nodeArgs["search"].String();
-
-            vector<const IndexDescriptor*> idxMatches;
-            collection->getIndexCatalog()->findIndexByType(opCtx, "text", idxMatches);
-            uassert(17194, "Expected exactly one text index", idxMatches.size() == 1);
-
-            const IndexDescriptor* index = idxMatches[0];
-            const FTSAccessMethod* fam = dynamic_cast<const FTSAccessMethod*>(
-                collection->getIndexCatalog()->getEntry(index)->accessMethod());
-            invariant(fam);
-            TextStageParams params(fam->getSpec());
-            params.index = index;
-
-            // TODO: Deal with non-empty filters.  This is a hack to put in covering information
-            // that can only be checked for equality.  We ignore this now.
-            Status s = fam->getSpec().getIndexPrefix(BSONObj(), &params.indexPrefix);
-            if (!s.isOK()) {
-                return nullptr;
-            }
-
-            params.spec = fam->getSpec();
-
-            params.query.setQuery(search);
-            params.query.setLanguage(fam->getSpec().defaultLanguage().str());
-            params.query.setCaseSensitive(TextMatchExpressionBase::kCaseSensitiveDefault);
-            params.query.setDiacriticSensitive(TextMatchExpressionBase::kDiacriticSensitiveDefault);
-            if (!params.query.parse(fam->getSpec().getTextIndexVersion()).isOK()) {
-                return nullptr;
-            }
-
-            return new TextStage(expCtx.get(), params, workingSet, matcher);
         } else if ("delete" == nodeName) {
             uassert(18636,
                     "Delete stage doesn't have a filter (put it on the child)",
