@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kWrite
 
 #include "mongo/platform/basic.h"
 
@@ -39,25 +38,35 @@
 #include "mongo/base/status_with.h"
 #include "mongo/bson/bson_comparator_interface_base.h"
 #include "mongo/bson/mutable/algorithm.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
+#include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/exec/scoped_timer.h"
+#include "mongo/db/exec/shard_filterer_impl.h"
 #include "mongo/db/exec/working_set_common.h"
 #include "mongo/db/exec/write_stage_common.h"
-#include "mongo/db/op_observer.h"
+#include "mongo/db/internal_transactions_feature_flag_gen.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/query/collection_query_info.h"
 #include "mongo/db/query/explain.h"
+#include "mongo/db/query/plan_executor_impl.h"
 #include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/s/collection_sharding_state.h"
 #include "mongo/db/s/operation_sharding_state.h"
+#include "mongo/db/s/sharding_state.h"
+#include "mongo/db/s/sharding_write_router.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/duplicate_key_error_info.h"
 #include "mongo/db/update/path_support.h"
 #include "mongo/db/update/storage_validation.h"
 #include "mongo/logv2/log.h"
+#include "mongo/s/grid.h"
 #include "mongo/s/shard_key_pattern.h"
 #include "mongo/s/would_change_owning_shard_exception.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/scopeguard.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kWrite
+
+
 namespace mongo {
 
 MONGO_FAIL_POINT_DEFINE(hangBeforeThrowWouldChangeOwningShard);
@@ -105,15 +114,11 @@ CollectionUpdateArgs::StoreDocOption getStoreDocMode(const UpdateRequest& update
 }
 }  // namespace
 
-const char* UpdateStage::kStageType = "UPDATE";
-
-const UpdateStats UpdateStage::kEmptyUpdateStats;
-
 // Public constructor.
 UpdateStage::UpdateStage(ExpressionContext* expCtx,
                          const UpdateStageParams& params,
                          WorkingSet* ws,
-                         Collection* collection,
+                         const CollectionPtr& collection,
                          PlanStage* child)
     : UpdateStage(expCtx, params, ws, collection) {
     // We should never reach here if the request is an upsert.
@@ -125,35 +130,32 @@ UpdateStage::UpdateStage(ExpressionContext* expCtx,
 UpdateStage::UpdateStage(ExpressionContext* expCtx,
                          const UpdateStageParams& params,
                          WorkingSet* ws,
-                         Collection* collection)
-    : RequiresMutableCollectionStage(kStageType, expCtx, collection),
+                         const CollectionPtr& collection)
+    : RequiresMutableCollectionStage(kStageType.rawData(), expCtx, collection),
       _params(params),
       _ws(ws),
       _doc(params.driver->getDocument()),
       _idRetrying(WorkingSet::INVALID_ID),
       _idReturning(WorkingSet::INVALID_ID),
-      _updatedRecordIds(params.request->isMulti() ? new RecordIdSet() : nullptr) {
+      _updatedRecordIds(params.request->isMulti() ? new RecordIdSet() : nullptr),
+      _preWriteFilter(opCtx(), collection->ns()) {
 
-    // Should the modifiers validate their embedded docs via storage_validation::storageValid()?
+    // Should the modifiers validate their embedded docs via storage_validation::scanDocument()?
     // Only user updates should be checked. Any system or replication stuff should pass through.
-    // Config db docs also do not get checked.
     const auto request = _params.request;
-    _enforceOkForStorage =
-        !(request->isFromOplogApplication() || request->getNamespaceString().isConfigDB() ||
-          request->isFromMigration());
 
-    // We should only check for an update to the shard key if the update is coming from a user and
-    // the request is versioned.
-    _shouldCheckForShardKeyUpdate =
-        !(request->isFromOplogApplication() || request->getNamespaceString().isConfigDB() ||
-          request->isFromMigration()) &&
-        OperationShardingState::isOperationVersioned(expCtx->opCtx);
+    _isUserInitiatedWrite = opCtx()->writesAreReplicated() &&
+        !(request->isFromOplogApplication() ||
+          params.driver->type() == UpdateDriver::UpdateType::kDelta ||
+          request->source() == OperationSource::kFromMigrate);
 
     _specificStats.isModUpdate = params.driver->type() == UpdateDriver::UpdateType::kOperator;
 }
 
-BSONObj UpdateStage::transformAndUpdate(const Snapshotted<BSONObj>& oldObj, RecordId& recordId) {
-    const UpdateRequest* request = _params.request;
+BSONObj UpdateStage::transformAndUpdate(const Snapshotted<BSONObj>& oldObj,
+                                        RecordId& recordId,
+                                        bool writeToOrphan) {
+    const UpdateRequest* const request = _params.request;
     UpdateDriver* driver = _params.driver;
     CanonicalQuery* cq = _params.canonicalQuery;
 
@@ -175,23 +177,29 @@ BSONObj UpdateStage::transformAndUpdate(const Snapshotted<BSONObj>& oldObj, Reco
 
     bool docWasModified = false;
 
-    auto* const css = CollectionShardingState::get(opCtx(), collection()->ns());
-    auto collDesc = css->getCollectionDescription();
     Status status = Status::OK();
-    const bool validateForStorage = opCtx()->writesAreReplicated() && _enforceOkForStorage;
     const bool isInsert = false;
     FieldRefSet immutablePaths;
-    if (opCtx()->writesAreReplicated() && !request->isFromMigration()) {
-        if (collDesc.isSharded() && !OperationShardingState::isOperationVersioned(opCtx())) {
+
+    if (_isUserInitiatedWrite) {
+        // Documents coming directly from users should be validated for storage. It is safe to
+        // access the CollectionShardingState in this write context and to throw SSV if the sharding
+        // metadata has not been initialized.
+        const auto collDesc = CollectionShardingState::get(opCtx(), collection()->ns())
+                                  ->getCollectionDescription(opCtx());
+        if (collDesc.isSharded() && !OperationShardingState::isComingFromRouter(opCtx())) {
             immutablePaths.fillFrom(collDesc.getKeyPatternFields());
         }
+
         immutablePaths.keepShortest(&idFieldRef);
     }
+
     if (!driver->needMatchDetails()) {
         // If we don't need match details, avoid doing the rematch
-        status = driver->update(StringData(),
+        status = driver->update(opCtx(),
+                                StringData(),
                                 &_doc,
-                                validateForStorage,
+                                _isUserInitiatedWrite,
                                 immutablePaths,
                                 isInsert,
                                 &logObj,
@@ -208,9 +216,10 @@ BSONObj UpdateStage::transformAndUpdate(const Snapshotted<BSONObj>& oldObj, Reco
         if (matchDetails.hasElemMatchKey())
             matchedField = matchDetails.elemMatchKey();
 
-        status = driver->update(matchedField,
+        status = driver->update(opCtx(),
+                                matchedField,
                                 &_doc,
-                                validateForStorage,
+                                _isUserInitiatedWrite,
                                 immutablePaths,
                                 isInsert,
                                 &logObj,
@@ -249,33 +258,39 @@ BSONObj UpdateStage::transformAndUpdate(const Snapshotted<BSONObj>& oldObj, Reco
         RecordId newRecordId;
         CollectionUpdateArgs args;
 
-        if (!request->isExplain()) {
-            args.stmtId = request->getStmtId();
+        if (!request->explain()) {
+            args.stmtIds = request->getStmtIds();
             args.update = logObj;
-            args.criteria = collDesc.extractDocumentKey(newObj).getShardKeyAndId();
+            if (_isUserInitiatedWrite) {
+                args.criteria = CollectionShardingState::get(opCtx(), collection()->ns())
+                                    ->getCollectionDescription(opCtx())
+                                    .extractDocumentKey(newObj);
+            } else {
+                const auto docId = newObj[idFieldName];
+                args.criteria = docId ? docId.wrap() : newObj;
+            }
             uassert(16980,
                     "Multi-update operations require all documents to have an '_id' field",
                     !request->isMulti() || args.criteria.hasField("_id"_sd));
-            args.fromMigrate = request->isFromMigration();
             args.storeDocOption = getStoreDocMode(*request);
             if (args.storeDocOption == CollectionUpdateArgs::StoreDocOption::PreImage) {
                 args.preImageDoc = oldObj.value().getOwned();
             }
         }
 
+        // Ensure we set the type correctly
+        args.source = writeToOrphan ? OperationSource::kFromMigrate : request->source();
+
         if (inPlace) {
-            if (!request->isExplain()) {
+            if (!request->explain()) {
                 newObj = oldObj.value();
                 const RecordData oldRec(oldObj.value().objdata(), oldObj.value().objsize());
 
                 Snapshotted<RecordData> snap(oldObj.snapshotId(), oldRec);
 
-                if (collDesc.isSharded() && _shouldCheckForShardKeyUpdate) {
-                    bool changesShardKeyOnSameNode =
-                        checkUpdateChangesShardKeyFields(collDesc, oldObj);
-                    if (changesShardKeyOnSameNode && !args.preImageDoc) {
-                        args.preImageDoc = oldObj.value().getOwned();
-                    }
+                if (_isUserInitiatedWrite &&
+                    checkUpdateChangesShardKeyFields(boost::none, oldObj) && !args.preImageDoc) {
+                    args.preImageDoc = oldObj.value().getOwned();
                 }
 
                 WriteUnitOfWork wunit(opCtx());
@@ -292,18 +307,17 @@ BSONObj UpdateStage::transformAndUpdate(const Snapshotted<BSONObj>& oldObj, Reco
             // The updates were not in place. Apply them through the file manager.
 
             newObj = _doc.getObject();
-            uassert(17419,
-                    str::stream() << "Resulting document after update is larger than "
-                                  << BSONObjMaxUserSize,
-                    newObj.objsize() <= BSONObjMaxUserSize);
+            if (!DocumentValidationSettings::get(opCtx()).isInternalValidationDisabled()) {
+                uassert(17419,
+                        str::stream() << "Resulting document after update is larger than "
+                                      << BSONObjMaxUserSize,
+                        newObj.objsize() <= BSONObjMaxUserSize);
+            }
 
-            if (!request->isExplain()) {
-                if (collDesc.isSharded() && _shouldCheckForShardKeyUpdate) {
-                    bool changesShardKeyOnSameNode =
-                        checkUpdateChangesShardKeyFields(collDesc, oldObj);
-                    if (changesShardKeyOnSameNode && !args.preImageDoc) {
-                        args.preImageDoc = oldObj.value().getOwned();
-                    }
+            if (!request->explain()) {
+                if (_isUserInitiatedWrite && checkUpdateChangesShardKeyFields(newObj, oldObj) &&
+                    !args.preImageDoc) {
+                    args.preImageDoc = oldObj.value().getOwned();
                 }
 
                 WriteUnitOfWork wunit(opCtx());
@@ -334,8 +348,8 @@ BSONObj UpdateStage::transformAndUpdate(const Snapshotted<BSONObj>& oldObj, Reco
 
     // Only record doc modifications if they wrote (exclude no-ops). Explains get
     // recorded as if they wrote.
-    if (docWasModified || request->isExplain()) {
-        _specificStats.nModified++;
+    if (docWasModified || request->explain()) {
+        _specificStats.nModified += _params.numStatsForDoc ? _params.numStatsForDoc(newObj) : 1;
     }
 
     return newObj;
@@ -356,82 +370,6 @@ void UpdateStage::_assertPathsNotArray(const mb::Document& document, const Field
     }
 }
 
-bool UpdateStage::matchContainsOnlyAndedEqualityNodes(const MatchExpression& root) {
-    if (root.matchType() == MatchExpression::EQ) {
-        return true;
-    }
-
-    if (root.matchType() == MatchExpression::AND) {
-        for (size_t i = 0; i < root.numChildren(); ++i) {
-            if (root.getChild(i)->matchType() != MatchExpression::EQ) {
-                return false;
-            }
-        }
-
-        return true;
-    }
-
-    return false;
-}
-
-bool UpdateStage::shouldRetryDuplicateKeyException(const ParsedUpdate& parsedUpdate,
-                                                   const DuplicateKeyErrorInfo& errorInfo) {
-    invariant(parsedUpdate.hasParsedQuery());
-
-    const auto updateRequest = parsedUpdate.getRequest();
-
-    // In order to be retryable, the update must be an upsert with multi:false.
-    if (!updateRequest->isUpsert() || updateRequest->isMulti()) {
-        return false;
-    }
-
-    auto matchExpr = parsedUpdate.getParsedQuery()->root();
-    invariant(matchExpr);
-
-    // In order to be retryable, the update query must contain no expressions other than AND and EQ.
-    if (!matchContainsOnlyAndedEqualityNodes(*matchExpr)) {
-        return false;
-    }
-
-    // In order to be retryable, the update equality field paths must be identical to the unique
-    // index key field paths. Also, the values that triggered the DuplicateKey error must match the
-    // values used in the upsert query predicate.
-    pathsupport::EqualityMatches equalities;
-    auto status = pathsupport::extractEqualityMatches(*matchExpr, &equalities);
-    if (!status.isOK()) {
-        return false;
-    }
-
-    auto keyPattern = errorInfo.getKeyPattern();
-    if (equalities.size() != static_cast<size_t>(keyPattern.nFields())) {
-        return false;
-    }
-
-    auto keyValue = errorInfo.getDuplicatedKeyValue();
-
-    BSONObjIterator keyPatternIter(keyPattern);
-    BSONObjIterator keyValueIter(keyValue);
-    while (keyPatternIter.more() && keyValueIter.more()) {
-        auto keyPatternElem = keyPatternIter.next();
-        auto keyValueElem = keyValueIter.next();
-
-        auto keyName = keyPatternElem.fieldNameStringData();
-        if (!equalities.count(keyName)) {
-            return false;
-        }
-
-        // Comparison which obeys field ordering but ignores field name.
-        BSONElementComparator cmp{BSONElementComparator::FieldNamesMode::kIgnore, nullptr};
-        if (cmp.evaluate(equalities[keyName]->getData() != keyValueElem)) {
-            return false;
-        }
-    }
-    invariant(!keyPatternIter.more());
-    invariant(!keyValueIter.more());
-
-    return true;
-}
-
 bool UpdateStage::isEOF() {
     // We're done updating if either the child has no more results to give us, or we've
     // already gotten a result back and we're not a multi-update.
@@ -444,6 +382,14 @@ PlanStage::StageState UpdateStage::doWork(WorkingSetID* out) {
         return PlanStage::IS_EOF;
     }
 
+    boost::optional<repl::UnreplicatedWritesBlock> unReplBlock;
+    if (collection()->ns().isImplicitlyReplicated() && !_isUserInitiatedWrite) {
+        // Implictly replicated collections do not replicate updates.
+        // However, user-initiated writes and some background maintenance tasks are allowed
+        // to replicate as they cannot be derived from the oplog.
+        unReplBlock.emplace(opCtx());
+    }
+
     // It is possible that after an update was applied, a WriteConflictException
     // occurred and prevented us from returning ADVANCED with the requested version
     // of the document.
@@ -478,7 +424,7 @@ PlanStage::StageState UpdateStage::doWork(WorkingSetID* out) {
 
         // We want to free this member when we return, unless we need to retry updating or returning
         // it.
-        auto memberFreer = makeGuard([&] { _ws->free(id); });
+        ScopeGuard memberFreer([&] { _ws->free(id); });
 
         invariant(member->hasRecordId());
         recordId = member->recordId;
@@ -496,34 +442,93 @@ PlanStage::StageState UpdateStage::doWork(WorkingSetID* out) {
         }
 
         bool docStillMatches;
-        try {
-            docStillMatches = write_stage_common::ensureStillMatches(
-                collection(), opCtx(), _ws, id, _params.canonicalQuery);
-        } catch (const WriteConflictException&) {
-            // There was a problem trying to detect if the document still exists, so retry.
-            memberFreer.dismiss();
-            return prepareToRetryWSM(id, out);
+        const auto ensureStillMatchesRet =
+            handlePlanStageYield(expCtx(),
+                                 "UpdateStage ensureStillMatches",
+                                 collection()->ns().ns(),
+                                 [&] {
+                                     docStillMatches = write_stage_common::ensureStillMatches(
+                                         collection(), opCtx(), _ws, id, _params.canonicalQuery);
+                                     return PlanStage::NEED_TIME;
+                                 },
+                                 [&] {
+                                     // yieldHandler
+                                     // There was a problem trying to detect if the document still
+                                     // exists, so retry.
+                                     memberFreer.dismiss();
+                                     prepareToRetryWSM(id, out);
+                                 });
+
+        if (ensureStillMatchesRet != PlanStage::NEED_TIME) {
+            return ensureStillMatchesRet;
         }
 
         if (!docStillMatches) {
             // Either the document has been deleted, or it has been updated such that it no longer
             // matches the predicate.
             if (shouldRestartUpdateIfNoLongerMatches(_params)) {
-                throw WriteConflictException();
+                throwWriteConflictException("Document no longer matches the predicate.");
             }
             return PlanStage::NEED_TIME;
         }
 
+        bool writeToOrphan = false;
+        if (!_params.request->explain() && _isUserInitiatedWrite) {
+            try {
+                const auto action = _preWriteFilter.computeAction(member->doc.value());
+                if (action == write_stage_common::PreWriteFilter::Action::kSkip) {
+                    LOGV2_DEBUG(
+                        5983200,
+                        3,
+                        "Skipping update operation to orphan document to prevent a wrong change "
+                        "stream event",
+                        "namespace"_attr = collection()->ns(),
+                        "record"_attr = member->doc.value());
+                    return PlanStage::NEED_TIME;
+                } else if (action ==
+                           write_stage_common::PreWriteFilter::Action::kWriteAsFromMigrate) {
+                    LOGV2_DEBUG(
+                        6184701,
+                        3,
+                        "Marking update operation to orphan document with the fromMigrate flag "
+                        "to prevent a wrong change stream event",
+                        "namespace"_attr = collection()->ns(),
+                        "record"_attr = member->doc.value());
+                    writeToOrphan = true;
+                }
+            } catch (const ExceptionFor<ErrorCodes::StaleConfig>& ex) {
+                if (ex->getVersionReceived() == ChunkVersion::IGNORED() &&
+                    ex->getCriticalSectionSignal()) {
+                    // If ChunkVersion is IGNORED and we encountered a critical section, then yield,
+                    // wait for critical section to finish and then we'll resume the write from the
+                    // point we had left. We do this to prevent large multi-writes from repeatedly
+                    // failing due to StaleConfig and exhausting the mongos retry attempts.
+                    planExecutorShardingCriticalSectionFuture(opCtx()) =
+                        ex->getCriticalSectionSignal();
+                    memberFreer.dismiss();  // Keep this member around so we can retry deleting it.
+                    prepareToRetryWSM(id, out);
+                    return PlanStage::NEED_YIELD;
+                }
+                throw;
+            }
+        }
+
         // Ensure that the BSONObj underlying the WorkingSetMember is owned because saveState()
         // is allowed to free the memory.
         member->makeObjOwnedIfNeeded();
 
         // Save state before making changes.
-        try {
-            child()->saveState();
-        } catch (const WriteConflictException&) {
-            std::terminate();
-        }
+        handlePlanStageYield(expCtx(),
+                             "UpdateStage saveState",
+                             collection()->ns().ns(),
+                             [&] {
+                                 child()->saveState();
+                                 return PlanStage::NEED_TIME /* unused */;
+                             },
+                             [&] {
+                                 // yieldHandler
+                                 std::terminate();
+                             });
 
         // If we care about the pre-updated version of the doc, save it out here.
         BSONObj oldObj;
@@ -533,12 +538,40 @@ PlanStage::StageState UpdateStage::doWork(WorkingSetID* out) {
         }
 
         BSONObj newObj;
+
         try {
-            // Do the update, get us the new version of the doc.
-            newObj = transformAndUpdate({oldSnapshot, member->doc.value().toBson()}, recordId);
-        } catch (const WriteConflictException&) {
-            memberFreer.dismiss();  // Keep this member around so we can retry updating it.
-            return prepareToRetryWSM(id, out);
+            const auto updateRet = handlePlanStageYield(
+                expCtx(),
+                "UpdateStage update",
+                collection()->ns().ns(),
+                [&] {
+                    // Do the update, get us the new version of the doc.
+                    newObj = transformAndUpdate(
+                        {oldSnapshot, member->doc.value().toBson()}, recordId, writeToOrphan);
+                    return PlanStage::NEED_TIME;
+                },
+                [&] {
+                    // yieldHandler
+                    memberFreer.dismiss();  // Keep this member around so we can retry updating it.
+                    prepareToRetryWSM(id, out);
+                });
+
+            if (updateRet != PlanStage::NEED_TIME) {
+                return updateRet;
+            }
+        } catch (const ExceptionFor<ErrorCodes::StaleConfig>& ex) {
+            if (ex->getVersionReceived() == ChunkVersion::IGNORED() &&
+                ex->getCriticalSectionSignal()) {
+                // If ChunkVersion is IGNORED and we encountered a critical section, then yield,
+                // wait for critical section to finish and then we'll resume the write from the
+                // point we had left. We do this to prevent large multi-writes from repeatedly
+                // failing due to StaleConfig and exhausting the mongos retry attempts.
+                planExecutorShardingCriticalSectionFuture(opCtx()) = ex->getCriticalSectionSignal();
+                memberFreer.dismiss();  // Keep this member around so we can retry updating it.
+                prepareToRetryWSM(id, out);
+                return PlanStage::NEED_YIELD;
+            }
+            throw;
         }
 
         // Set member's obj to be the doc we want to return.
@@ -554,28 +587,37 @@ PlanStage::StageState UpdateStage::doWork(WorkingSetID* out) {
         }
 
         // This should be after transformAndUpdate to make sure we actually updated this doc.
-        ++_specificStats.nMatched;
+        _specificStats.nMatched += _params.numStatsForDoc ? _params.numStatsForDoc(newObj) : 1;
 
-        // Restore state after modification
+        // Restore state after modification. As restoreState may restore (recreate) cursors, make
+        // sure to restore the state outside of the WritUnitOfWork.
+        const auto restoreStateRet = handlePlanStageYield(
+            expCtx(),
+            "UpdateStage restoreState",
+            collection()->ns().ns(),
+            [&] {
+                child()->restoreState(&collection());
+                return PlanStage::NEED_TIME;
+            },
+            [&] {
+                // yieldHandler
+                // Note we don't need to retry updating anything in this case since the update
+                // already was committed. However, we still need to return the updated document (if
+                // it was requested).
+                if (_params.request->shouldReturnAnyDocs()) {
+                    // member->obj should refer to the document we want to return.
+                    invariant(member->getState() == WorkingSetMember::OWNED_OBJ);
 
-        // As restoreState may restore (recreate) cursors, make sure to restore the
-        // state outside of the WritUnitOfWork.
-        try {
-            child()->restoreState();
-        } catch (const WriteConflictException&) {
-            // Note we don't need to retry updating anything in this case since the update
-            // already was committed. However, we still need to return the updated document
-            // (if it was requested).
-            if (_params.request->shouldReturnAnyDocs()) {
-                // member->obj should refer to the document we want to return.
-                invariant(member->getState() == WorkingSetMember::OWNED_OBJ);
+                    _idReturning = id;
+                    // Keep this member around so that we can return it on the next
+                    // work() call.
+                    memberFreer.dismiss();
+                }
+                *out = WorkingSet::INVALID_ID;
+            });
 
-                _idReturning = id;
-                // Keep this member around so that we can return it on the next work() call.
-                memberFreer.dismiss();
-            }
-            *out = WorkingSet::INVALID_ID;
-            return NEED_YIELD;
+        if (restoreStateRet != PlanStage::NEED_TIME) {
+            return restoreStateRet;
         }
 
         if (_params.request->shouldReturnAnyDocs()) {
@@ -591,17 +633,6 @@ PlanStage::StageState UpdateStage::doWork(WorkingSetID* out) {
     } else if (PlanStage::IS_EOF == status) {
         // The child is out of results, and therefore so are we.
         return PlanStage::IS_EOF;
-    } else if (PlanStage::FAILURE == status) {
-        *out = id;
-        // If a stage fails, it may create a status WSM to indicate why it failed, in which case
-        // 'id' is valid.  If ID is invalid, we create our own error message.
-        if (WorkingSet::INVALID_ID == id) {
-            const std::string errmsg = "update stage failed to read in results from child";
-            *out = WorkingSetCommon::allocateStatusMember(
-                _ws, Status(ErrorCodes::InternalError, errmsg));
-            return PlanStage::FAILURE;
-        }
-        return status;
     } else if (PlanStage::NEED_YIELD == status) {
         *out = id;
     }
@@ -640,6 +671,8 @@ void UpdateStage::doRestoreStateRequiresCollection() {
     // date index information.
     const auto& updateIndexData = CollectionQueryInfo::get(collection()).getIndexKeys(opCtx());
     _params.driver->refreshIndexKeys(&updateIndexData);
+
+    _preWriteFilter.restoreState();
 }
 
 unique_ptr<PlanStageStats> UpdateStage::getStats() {
@@ -654,79 +687,22 @@ const SpecificStats* UpdateStage::getSpecificStats() const {
     return &_specificStats;
 }
 
-const UpdateStats* UpdateStage::getUpdateStats(const PlanExecutor* exec) {
-    invariant(exec->getRootStage()->isEOF());
-
-    // If we're updating a non-existent collection, then the delete plan may have an EOF as the root
-    // stage.
-    if (exec->getRootStage()->stageType() == STAGE_EOF) {
-        return &kEmptyUpdateStats;
-    }
-
-    // If the collection exists, then we expect the root of the plan tree to either be an update
-    // stage, or (for findAndModify) a projection stage wrapping an update stage.
-    switch (exec->getRootStage()->stageType()) {
-        case StageType::STAGE_PROJECTION_DEFAULT:
-        case StageType::STAGE_PROJECTION_COVERED:
-        case StageType::STAGE_PROJECTION_SIMPLE: {
-            invariant(exec->getRootStage()->getChildren().size() == 1U);
-            invariant(StageType::STAGE_UPDATE == exec->getRootStage()->child()->stageType());
-            const SpecificStats* stats = exec->getRootStage()->child()->getSpecificStats();
-            return static_cast<const UpdateStats*>(stats);
-        }
-        default:
-            invariant(StageType::STAGE_UPDATE == exec->getRootStage()->stageType());
-            return static_cast<const UpdateStats*>(exec->getRootStage()->getSpecificStats());
-    }
-}
-
-void UpdateStage::recordUpdateStatsInOpDebug(const UpdateStats* updateStats, OpDebug* opDebug) {
-    invariant(opDebug);
-    opDebug->additiveMetrics.nMatched = updateStats->nMatched;
-    opDebug->additiveMetrics.nModified = updateStats->nModified;
-    opDebug->upsert = updateStats->inserted;
-}
-
-UpdateResult UpdateStage::makeUpdateResult(const UpdateStats* updateStats) {
-    return UpdateResult(updateStats->nMatched > 0 /* Did we update at least one obj? */,
-                        updateStats->isModUpdate /* Is this a $mod update? */,
-                        updateStats->nModified /* number of modified docs, no no-ops */,
-                        updateStats->nMatched /* # of docs matched/updated, even no-ops */,
-                        updateStats->objInserted);
-};
-
-PlanStage::StageState UpdateStage::prepareToRetryWSM(WorkingSetID idToRetry, WorkingSetID* out) {
+void UpdateStage::prepareToRetryWSM(WorkingSetID idToRetry, WorkingSetID* out) {
     _idRetrying = idToRetry;
     *out = WorkingSet::INVALID_ID;
-    return NEED_YIELD;
 }
 
-bool UpdateStage::checkUpdateChangesShardKeyFields(ScopedCollectionDescription collDesc,
-                                                   const Snapshotted<BSONObj>& oldObj) {
-    auto newObj = _doc.getObject();
-    const ShardKeyPattern shardKeyPattern(collDesc.getKeyPattern());
-    auto oldShardKey = shardKeyPattern.extractShardKeyFromDoc(oldObj.value());
-    auto newShardKey = shardKeyPattern.extractShardKeyFromDoc(newObj);
 
-    // If the shard key fields remain unchanged by this update we can skip the rest of the checks.
-    // Using BSONObj::binaryEqual() still allows a missing shard key field to be filled in with an
-    // explicit null value.
-    if (newShardKey.binaryEqual(oldShardKey)) {
-        return false;
-    }
-
-    FieldRefSet shardKeyPaths(collDesc.getKeyPatternFields());
-
-    // Assert that the updated doc has no arrays or array descendants for the shard key fields.
-    _assertPathsNotArray(_doc, shardKeyPaths);
-
-    // We do not allow modifying shard key value without specifying the full shard key in the query.
+void UpdateStage::_checkRestrictionsOnUpdatingShardKeyAreNotViolated(
+    const ScopedCollectionDescription& collDesc, const FieldRefSet& shardKeyPaths) {
+    // We do not allow modifying either the current shard key value or new shard key value (if
+    // resharding) without specifying the full current shard key in the query.
     // If the query is a simple equality match on _id, then '_params.canonicalQuery' will be null.
     // But if we are here, we already know that the shard key is not _id, since we have an assertion
     // earlier for requests that try to modify the immutable _id field. So it is safe to uassert if
     // '_params.canonicalQuery' is null OR if the query does not include equality matches on all
     // shard key fields.
-    const auto& shardKeyPathsVector = collDesc->getKeyPatternFields();
+    const auto& shardKeyPathsVector = collDesc.getKeyPatternFields();
     pathsupport::EqualityMatches equalities;
     uassert(31025,
             "Shard key update is not allowed without specifying the full shard key in the query",
@@ -743,25 +719,127 @@ bool UpdateStage::checkUpdateChangesShardKeyFields(ScopedCollectionDescription c
 
     // If this node is a replica set primary node, an attempted update to the shard key value must
     // either be a retryable write or inside a transaction.
+    // An update without a transaction number is legal if
+    // gFeatureFlagUpdateDocumentShardKeyUsingTransactionApi is enabled because mongos
+    // will be able to start an internal transaction to handle the wouldChangeOwningShard error
+    // thrown below.
     // If this node is a replica set secondary node, we can skip validation.
-    uassert(ErrorCodes::IllegalOperation,
-            "Must run update to shard key field in a multi-statement transaction or with "
-            "retryWrites: true.",
-            opCtx()->getTxnNumber() || !opCtx()->writesAreReplicated());
+    if (!feature_flags::gFeatureFlagUpdateDocumentShardKeyUsingTransactionApi.isEnabled(
+            serverGlobalParams.featureCompatibility)) {
+        uassert(ErrorCodes::IllegalOperation,
+                "Must run update to shard key field in a multi-statement transaction or with "
+                "retryWrites: true.",
+                opCtx()->getTxnNumber());
+    }
+}
+
+
+bool UpdateStage::wasReshardingKeyUpdated(const ShardingWriteRouter& shardingWriteRouter,
+                                          const ScopedCollectionDescription& collDesc,
+                                          const BSONObj& newObj,
+                                          const Snapshotted<BSONObj>& oldObj) {
+    auto reshardingKeyPattern = collDesc.getReshardingKeyIfShouldForwardOps();
+    if (!reshardingKeyPattern)
+        return false;
+
+    auto oldShardKey = reshardingKeyPattern->extractShardKeyFromDoc(oldObj.value());
+    auto newShardKey = reshardingKeyPattern->extractShardKeyFromDoc(newObj);
+
+    if (newShardKey.binaryEqual(oldShardKey))
+        return false;
+
+    FieldRefSet shardKeyPaths(collDesc.getKeyPatternFields());
+    _checkRestrictionsOnUpdatingShardKeyAreNotViolated(collDesc, shardKeyPaths);
+
+    auto oldRecipShard = *shardingWriteRouter.getReshardingDestinedRecipient(oldObj.value());
+    auto newRecipShard = *shardingWriteRouter.getReshardingDestinedRecipient(newObj);
+
+    uassert(
+        WouldChangeOwningShardInfo(
+            oldObj.value(), newObj, false /* upsert */, collection()->ns(), collection()->uuid()),
+        "This update would cause the doc to change owning shards under the new shard key",
+        oldRecipShard == newRecipShard);
+
+    return true;
+}
+
+bool UpdateStage::checkUpdateChangesShardKeyFields(const boost::optional<BSONObj>& newObjCopy,
+                                                   const Snapshotted<BSONObj>& oldObj) {
+    ShardingWriteRouter shardingWriteRouter(
+        opCtx(), collection()->ns(), Grid::get(opCtx())->catalogCache());
+    auto css = shardingWriteRouter.getCollectionShardingState();
+
+    // css can be null when this is a config server.
+    if (css == nullptr) {
+        return false;
+    }
+
+    const auto collDesc = css->getCollectionDescription(opCtx());
+
+    // Calling mutablebson::Document::getObject() renders a full copy of the updated document. This
+    // can be expensive for larger documents, so we skip calling it when the collection isn't even
+    // sharded.
+    if (!collDesc.isSharded()) {
+        return false;
+    }
+
+    const auto& newObj = newObjCopy ? *newObjCopy : _doc.getObject();
+
+    // It is possible that both the existing and new shard keys are being updated, so we do not want
+    // to short-circuit checking whether either is being modified.
+    const auto existingShardKeyUpdated =
+        wasExistingShardKeyUpdated(shardingWriteRouter, collDesc, newObj, oldObj);
+    const auto reshardingKeyUpdated =
+        wasReshardingKeyUpdated(shardingWriteRouter, collDesc, newObj, oldObj);
+
+    return existingShardKeyUpdated || reshardingKeyUpdated;
+}
+
+bool UpdateStage::wasExistingShardKeyUpdated(const ShardingWriteRouter& shardingWriteRouter,
+                                             const ScopedCollectionDescription& collDesc,
+                                             const BSONObj& newObj,
+                                             const Snapshotted<BSONObj>& oldObj) {
+    const auto css = shardingWriteRouter.getCollectionShardingState();
+
+    const ShardKeyPattern& shardKeyPattern = collDesc.getShardKeyPattern();
+    auto oldShardKey = shardKeyPattern.extractShardKeyFromDoc(oldObj.value());
+    auto newShardKey = shardKeyPattern.extractShardKeyFromDoc(newObj);
+
+    // If the shard key fields remain unchanged by this update we can skip the rest of the checks.
+    // Using BSONObj::binaryEqual() still allows a missing shard key field to be filled in with an
+    // explicit null value.
+    if (newShardKey.binaryEqual(oldShardKey)) {
+        return false;
+    }
+
+    FieldRefSet shardKeyPaths(collDesc.getKeyPatternFields());
+
+    // Assert that the updated doc has no arrays or array descendants for the shard key fields.
+    _assertPathsNotArray(_doc, shardKeyPaths);
+
+    _checkRestrictionsOnUpdatingShardKeyAreNotViolated(collDesc, shardKeyPaths);
+
+    // At this point we already asserted that the complete shardKey have been specified in the
+    // query, this implies that mongos is not doing a broadcast update and that it attached a
+    // shardVersion to the command. Thus it is safe to call getOwnershipFilter
+    const auto collFilter = css->getOwnershipFilter(
+        opCtx(), CollectionShardingState::OrphanCleanupPolicy::kAllowOrphanCleanup);
 
     // If the shard key of an orphan document is allowed to change, and the document is allowed to
     // become owned by the shard, the global uniqueness assumption for _id values would be violated.
-    uassert(468110,
-            "Changing the shard key of an orphan document is not allowed",
-            collDesc->keyBelongsToMe(oldShardKey));
+    invariant(collFilter.keyBelongsToMe(oldShardKey));
 
-    if (!collDesc->keyBelongsToMe(newShardKey)) {
+    if (!collFilter.keyBelongsToMe(newShardKey)) {
         if (MONGO_unlikely(hangBeforeThrowWouldChangeOwningShard.shouldFail())) {
             LOGV2(20605, "Hit hangBeforeThrowWouldChangeOwningShard failpoint");
             hangBeforeThrowWouldChangeOwningShard.pauseWhileSet(opCtx());
         }
 
-        uasserted(WouldChangeOwningShardInfo(oldObj.value(), newObj, false /* upsert */),
+        uasserted(WouldChangeOwningShardInfo(oldObj.value(),
+                                             newObj,
+                                             false /* upsert */,
+                                             collection()->ns(),
+                                             collection()->uuid()),
                   "This update would cause the doc to change owning shards");
     }
 
