@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kIndex
 
 #include "mongo/platform/basic.h"
 
@@ -36,9 +35,8 @@
 #include <vector>
 
 #include "mongo/bson/bsonobj.h"
-#include "mongo/db/catalog/index_timestamp_helper.h"
 #include "mongo/db/catalog_raii.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index/index_access_method.h"
@@ -51,32 +49,62 @@
 #include "mongo/util/progress_meter.h"
 #include "mongo/util/uuid.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kIndex
+
+
 namespace mongo {
 
 MONGO_FAIL_POINT_DEFINE(hangDuringIndexBuildDrainYield);
+MONGO_FAIL_POINT_DEFINE(hangDuringIndexBuildDrainYieldSecond);
+MONGO_FAIL_POINT_DEFINE(hangIndexBuildDuringDrainWritesPhase);
+MONGO_FAIL_POINT_DEFINE(hangIndexBuildDuringDrainWritesPhaseSecond);
 
-IndexBuildInterceptor::IndexBuildInterceptor(OperationContext* opCtx, IndexCatalogEntry* entry)
+IndexBuildInterceptor::IndexBuildInterceptor(OperationContext* opCtx,
+                                             const IndexCatalogEntry* entry)
     : _indexCatalogEntry(entry),
-      _sideWritesTable(
-          opCtx->getServiceContext()->getStorageEngine()->makeTemporaryRecordStore(opCtx)),
-      _skippedRecordTracker(entry),
-      _sideWritesCounter(std::make_shared<AtomicWord<long long>>()) {
+      _sideWritesTable(opCtx->getServiceContext()->getStorageEngine()->makeTemporaryRecordStore(
+          opCtx, KeyFormat::Long)),
+      _skippedRecordTracker(opCtx, entry, boost::none) {
 
     if (entry->descriptor()->unique()) {
         _duplicateKeyTracker = std::make_unique<DuplicateKeyTracker>(opCtx, entry);
     }
 }
 
-void IndexBuildInterceptor::deleteTemporaryTables(OperationContext* opCtx) {
-    _sideWritesTable->deleteTemporaryTable(opCtx);
-    if (_duplicateKeyTracker) {
-        _duplicateKeyTracker->deleteTemporaryTable(opCtx);
+IndexBuildInterceptor::IndexBuildInterceptor(OperationContext* opCtx,
+                                             const IndexCatalogEntry* entry,
+                                             StringData sideWritesIdent,
+                                             boost::optional<StringData> duplicateKeyTrackerIdent,
+                                             boost::optional<StringData> skippedRecordTrackerIdent)
+    : _indexCatalogEntry(entry),
+      _sideWritesTable(
+          opCtx->getServiceContext()->getStorageEngine()->makeTemporaryRecordStoreFromExistingIdent(
+              opCtx, sideWritesIdent)),
+      _skippedRecordTracker(opCtx, entry, skippedRecordTrackerIdent),
+      _skipNumAppliedCheck(true) {
+
+    auto dupKeyTrackerIdentExists = duplicateKeyTrackerIdent ? true : false;
+    uassert(ErrorCodes::BadValue,
+            str::stream() << "Resume info must contain the duplicate key tracker ident ["
+                          << duplicateKeyTrackerIdent
+                          << "] if and only if the index is unique: " << entry->descriptor(),
+            entry->descriptor()->unique() == dupKeyTrackerIdentExists);
+    if (duplicateKeyTrackerIdent) {
+        _duplicateKeyTracker =
+            std::make_unique<DuplicateKeyTracker>(opCtx, entry, duplicateKeyTrackerIdent.value());
     }
-    _skippedRecordTracker.deleteTemporaryTable(opCtx);
+}
+
+void IndexBuildInterceptor::keepTemporaryTables() {
+    _sideWritesTable->keep();
+    if (_duplicateKeyTracker) {
+        _duplicateKeyTracker->keepTemporaryTable();
+    }
+    _skippedRecordTracker.keepTemporaryTable();
 }
 
 Status IndexBuildInterceptor::recordDuplicateKey(OperationContext* opCtx,
-                                                 const KeyString::Value& key) {
+                                                 const KeyString::Value& key) const {
     invariant(_indexCatalogEntry->descriptor()->unique());
     return _duplicateKeyTracker->recordKey(opCtx, key);
 }
@@ -89,13 +117,11 @@ Status IndexBuildInterceptor::checkDuplicateKeyConstraints(OperationContext* opC
 }
 
 Status IndexBuildInterceptor::drainWritesIntoIndex(OperationContext* opCtx,
+                                                   const CollectionPtr& coll,
                                                    const InsertDeleteOptions& options,
                                                    TrackDuplicates trackDuplicates,
-                                                   RecoveryUnit::ReadSource readSource,
                                                    DrainYieldPolicy drainYieldPolicy) {
     invariant(!opCtx->lockState()->inAWriteUnitOfWork());
-    // Reading at a timestamp during hybrid index builds is not supported.
-    invariant(readSource == RecoveryUnit::ReadSource::kNoTimestamp);
 
     // These are used for logging only.
     int64_t totalDeleted = 0;
@@ -137,6 +163,15 @@ Status IndexBuildInterceptor::drainWritesIntoIndex(OperationContext* opCtx,
     // Returns true if the cursor has reached the end of the table, false if there are more records,
     // and an error Status otherwise.
     auto applySingleBatch = [&]() -> StatusWith<bool> {
+        // This write is performed without a durable/commit timestamp. This transaction trips the
+        // ordered assertion for the side-table documents which are inserted with a timestamp and,
+        // in here, being deleted without a timestamp. Because the data being read is majority
+        // committed, there's no risk of needing to roll back the writes done by this "drain".
+        //
+        // Note that index builds will only "resume" once. A second resume results in the index
+        // build starting from scratch. A "resumed" index build does not use a majority read
+        // concern. And thus will observe data that can be rolled back via replication.
+        opCtx->recoveryUnit()->allowUntimestampedWrite();
         WriteUnitOfWork wuow(opCtx);
 
         int32_t batchSize = 0;
@@ -152,7 +187,7 @@ Status IndexBuildInterceptor::drainWritesIntoIndex(OperationContext* opCtx,
         while (record) {
             opCtx->checkForInterrupt();
 
-            RecordId currentRecordId = record->id;
+            auto& currentRecordId = record->id;
             BSONObj unownedDoc = record->data.toBson();
 
             // Don't apply this record if the total batch size in bytes would be too large.
@@ -161,18 +196,28 @@ Status IndexBuildInterceptor::drainWritesIntoIndex(OperationContext* opCtx,
                 break;
             }
 
+            const long long iteration = _numApplied + batchSize;
+            _checkDrainPhaseFailPoint(opCtx, &hangIndexBuildDuringDrainWritesPhase, iteration);
+            _checkDrainPhaseFailPoint(
+                opCtx, &hangIndexBuildDuringDrainWritesPhaseSecond, iteration);
+
             batchSize += 1;
             batchSizeBytes += objSize;
 
-            if (auto status = _applyWrite(
-                    opCtx, unownedDoc, options, trackDuplicates, &totalInserted, &totalDeleted);
+            if (auto status = _applyWrite(opCtx,
+                                          coll,
+                                          unownedDoc,
+                                          options,
+                                          trackDuplicates,
+                                          &totalInserted,
+                                          &totalDeleted);
                 !status.isOK()) {
                 return status;
             }
 
             // Save the record ids of the documents inserted into the index for deletion later.
             // We can't delete records while holding a positioned cursor.
-            recordsAddedToIndex.push_back(currentRecordId);
+            recordsAddedToIndex.emplace_back(std::move(currentRecordId));
 
             // Don't continue if the batch is full. Allow the transaction to commit.
             if (batchSize == kBatchMaxSize) {
@@ -201,7 +246,7 @@ Status IndexBuildInterceptor::drainWritesIntoIndex(OperationContext* opCtx,
         // Lock yielding will be directed by the yield policy provided.
         // We will typically yield locks during the draining phase if we are holding intent locks.
         if (DrainYieldPolicy::kYield == drainYieldPolicy) {
-            _yield(opCtx);
+            _yield(opCtx, &coll);
         }
 
         // Account for more writes coming in during a batch.
@@ -214,8 +259,8 @@ Status IndexBuildInterceptor::drainWritesIntoIndex(OperationContext* opCtx,
 
     // Apply batches of side writes until the last record in the table is seen.
     while (!atEof) {
-        auto swAtEof = writeConflictRetry(
-            opCtx, "index build drain", _indexCatalogEntry->ns().ns(), applySingleBatch);
+        auto swAtEof =
+            writeConflictRetry(opCtx, "index build drain", coll->ns().ns(), applySingleBatch);
         if (!swAtEof.isOK()) {
             return swAtEof.getStatus();
         }
@@ -227,17 +272,20 @@ Status IndexBuildInterceptor::drainWritesIntoIndex(OperationContext* opCtx,
     int logLevel = (_numApplied - appliedAtStart > 0) ? 0 : 1;
     LOGV2_DEBUG(20689,
                 logLevel,
-                "index build: drained side writes",
+                "Index build: drained side writes",
+                "index"_attr = _indexCatalogEntry->descriptor()->indexName(),
+                "collectionUUID"_attr = coll->uuid(),
+                logAttrs(coll->ns()),
                 "numApplied"_attr = (_numApplied - appliedAtStart),
                 "totalInserted"_attr = totalInserted,
                 "totalDeleted"_attr = totalDeleted,
-                "indexName"_attr = _indexCatalogEntry->descriptor()->indexName(),
                 "durationMillis"_attr = timer.millis());
 
     return Status::OK();
 }
 
 Status IndexBuildInterceptor::_applyWrite(OperationContext* opCtx,
+                                          const CollectionPtr& coll,
                                           const BSONObj& operation,
                                           const InsertDeleteOptions& options,
                                           TrackDuplicates trackDups,
@@ -247,26 +295,21 @@ Status IndexBuildInterceptor::_applyWrite(OperationContext* opCtx,
     int keyLen;
     const char* binKey = operation["key"].binData(keyLen);
     BufReader reader(binKey, keyLen);
+    auto accessMethod = _indexCatalogEntry->accessMethod()->asSortedData();
     const KeyString::Value keyString = KeyString::Value::deserialize(
-        reader,
-        _indexCatalogEntry->accessMethod()->getSortedDataInterface()->getKeyStringVersion());
+        reader, accessMethod->getSortedDataInterface()->getKeyStringVersion());
 
-    const Op opType =
-        (strcmp(operation.getStringField("op"), "i") == 0) ? Op::kInsert : Op::kDelete;
+    const Op opType = operation.getStringField("op") == "i"_sd ? Op::kInsert : Op::kDelete;
 
     const KeyStringSet keySet{keyString};
-    const RecordId opRecordId =
-        KeyString::decodeRecordIdAtEnd(keyString.getBuffer(), keyString.getSize());
-
-    auto accessMethod = _indexCatalogEntry->accessMethod();
     if (opType == Op::kInsert) {
         int64_t numInserted;
         auto status = accessMethod->insertKeysAndUpdateMultikeyPaths(
             opCtx,
-            keySet,
+            coll,
+            {keySet.begin(), keySet.end()},
             {},
             MultikeyPaths{},
-            opRecordId,
             options,
             [=](const KeyString::Value& duplicateKey) {
                 return trackDups == TrackDuplicates::kTrack
@@ -284,10 +327,11 @@ Status IndexBuildInterceptor::_applyWrite(OperationContext* opCtx,
     } else {
         invariant(opType == Op::kDelete);
         if (kDebugBuild)
-            invariant(strcmp(operation.getStringField("op"), "d") == 0);
+            invariant(operation.getStringField("op") == "d"_sd);
 
         int64_t numDeleted;
-        Status s = accessMethod->removeKeys(opCtx, keySet, opRecordId, options, &numDeleted);
+        Status s =
+            accessMethod->removeKeys(opCtx, {keySet.begin(), keySet.end()}, options, &numDeleted);
         if (!s.isOK()) {
             return s;
         }
@@ -299,9 +343,10 @@ Status IndexBuildInterceptor::_applyWrite(OperationContext* opCtx,
     return Status::OK();
 }
 
-void IndexBuildInterceptor::_yield(OperationContext* opCtx) {
+void IndexBuildInterceptor::_yield(OperationContext* opCtx, const Yieldable* yieldable) {
     // Releasing locks means a new snapshot should be acquired when restored.
     opCtx->recoveryUnit()->abandonSnapshot();
+    yieldable->yield();
 
     auto locker = opCtx->lockState();
     Locker::LockSnapshot snapshot;
@@ -311,42 +356,65 @@ void IndexBuildInterceptor::_yield(OperationContext* opCtx) {
     // Track the number of yields in CurOp.
     CurOp::get(opCtx)->yielded();
 
-    hangDuringIndexBuildDrainYield.executeIf(
-        [&](auto&&) {
-            LOGV2(20690, "Hanging index build during drain yield");
-            hangDuringIndexBuildDrainYield.pauseWhileSet();
-        },
-        [&](auto&& config) {
-            return config.getStringField("namespace") == _indexCatalogEntry->ns().ns();
-        });
+    auto failPointHang = [opCtx, indexCatalogEntry = _indexCatalogEntry](FailPoint* fp) {
+        fp->executeIf(
+            [fp](auto&&) {
+                LOGV2(20690, "Hanging index build during drain yield");
+                fp->pauseWhileSet();
+            },
+            [opCtx, indexCatalogEntry](auto&& config) {
+                return config.getStringField("namespace") ==
+                    indexCatalogEntry->getNSSFromCatalog(opCtx).ns();
+            });
+    };
+    failPointHang(&hangDuringIndexBuildDrainYield);
+    failPointHang(&hangDuringIndexBuildDrainYieldSecond);
 
     locker->restoreLockState(opCtx, snapshot);
+    yieldable->restore();
 }
 
 bool IndexBuildInterceptor::areAllWritesApplied(OperationContext* opCtx) const {
+    return _checkAllWritesApplied(opCtx, false);
+}
+
+void IndexBuildInterceptor::invariantAllWritesApplied(OperationContext* opCtx) const {
+    _checkAllWritesApplied(opCtx, true);
+}
+
+bool IndexBuildInterceptor::_checkAllWritesApplied(OperationContext* opCtx, bool fatal) const {
     invariant(_sideWritesTable);
-    auto cursor = _sideWritesTable->rs()->getCursor(opCtx);
-    auto record = cursor->next();
 
     // The table is empty only when all writes are applied.
-    if (!record) {
-        auto writesRecorded = _sideWritesCounter->load();
-        if (writesRecorded != _numApplied) {
-            dassert(writesRecorded == _numApplied,
-                    (str::stream()
-                     << "The number of side writes recorded does not match the number "
-                        "applied, despite the table appearing empty. Writes recorded: "
-                     << writesRecorded << ", applied: " << _numApplied));
-            LOGV2_WARNING(20692,
-                          "The number of side writes recorded does not match the number applied, "
-                          "despite the table appearing empty",
-                          "writesRecorded"_attr = writesRecorded,
-                          "applied"_attr = _numApplied);
-        }
+    auto cursor = _sideWritesTable->rs()->getCursor(opCtx);
+    auto record = cursor->next();
+    if (fatal) {
+        invariant(
+            !record,
+            str::stream() << "Expected all side writes to be drained but found record with id "
+                          << record->id << " and data " << record->data.toBson());
+    } else if (record) {
+        return false;
+    }
+
+    if (_skipNumAppliedCheck) {
         return true;
     }
 
-    return false;
+    auto writesRecorded = _sideWritesCounter->load();
+    if (writesRecorded != _numApplied) {
+        dassert(writesRecorded == _numApplied,
+                (str::stream() << "The number of side writes recorded does not match the number "
+                                  "applied, despite the table appearing empty. Writes recorded: "
+                               << writesRecorded << ", applied: " << _numApplied));
+        LOGV2_WARNING(20692,
+                      "The number of side writes recorded does not match the number applied, "
+                      "despite the table appearing empty",
+                      "writesRecorded"_attr = writesRecorded,
+                      "applied"_attr = _numApplied);
+    }
+
+    return true;
 }
 
 boost::optional<MultikeyPaths> IndexBuildInterceptor::getMultikeyPaths() const {
@@ -358,7 +426,6 @@ Status IndexBuildInterceptor::sideWrite(OperationContext* opCtx,
                                         const KeyStringSet& keys,
                                         const KeyStringSet& multikeyMetadataKeys,
                                         const MultikeyPaths& multikeyPaths,
-                                        RecordId loc,
                                         Op op,
                                         int64_t* const numKeysOut) {
     invariant(opCtx->lockState()->inAWriteUnitOfWork());
@@ -369,7 +436,7 @@ Status IndexBuildInterceptor::sideWrite(OperationContext* opCtx,
 
     // Maintain parity with IndexAccessMethod's handling of whether keys could change the multikey
     // state on the index.
-    bool isMultikey = _indexCatalogEntry->accessMethod()->shouldMarkIndexAsMultikey(
+    bool isMultikey = _indexCatalogEntry->accessMethod()->asSortedData()->shouldMarkIndexAsMultikey(
         keys.size(), multikeyMetadataKeys, multikeyPaths);
 
     // No need to take the multikeyPaths mutex if this would not change any multikey state.
@@ -379,7 +446,7 @@ Status IndexBuildInterceptor::sideWrite(OperationContext* opCtx,
         // expectations.
         stdx::unique_lock<Latch> lk(_multikeyPathMutex);
         if (_multikeyPaths) {
-            MultikeyPathTracker::mergeMultikeyPaths(&_multikeyPaths.get(), multikeyPaths);
+            MultikeyPathTracker::mergeMultikeyPaths(&_multikeyPaths.value(), multikeyPaths);
         } else {
             // `mergeMultikeyPaths` is sensitive to the two inputs having the same multikey
             // "shape". Initialize `_multikeyPaths` with the right shape from the first result.
@@ -454,9 +521,30 @@ Status IndexBuildInterceptor::sideWrite(OperationContext* opCtx,
 }
 
 Status IndexBuildInterceptor::retrySkippedRecords(OperationContext* opCtx,
-                                                  const Collection* collection) {
+                                                  const CollectionPtr& collection) {
     return _skippedRecordTracker.retrySkippedRecords(opCtx, collection);
 }
 
+void IndexBuildInterceptor::_checkDrainPhaseFailPoint(OperationContext* opCtx,
+                                                      FailPoint* fp,
+                                                      long long iteration) const {
+    fp->executeIf(
+        [=](const BSONObj& data) {
+            LOGV2(4841800,
+                  "Hanging index build during drain writes phase",
+                  "iteration"_attr = iteration,
+                  "index"_attr = _indexCatalogEntry->descriptor()->indexName());
+
+            fp->pauseWhileSet(opCtx);
+        },
+        [iteration,
+         &indexName = _indexCatalogEntry->descriptor()->indexName()](const BSONObj& data) {
+            auto indexNames = data.getObjectField("indexNames");
+            return iteration == data["iteration"].numberLong() &&
+                std::any_of(indexNames.begin(), indexNames.end(), [&indexName](const auto& elem) {
+                       return indexName == elem.String();
+                   });
+        });
+}
 
 }  // namespace mongo
