@@ -36,17 +36,21 @@
 #include <vector>
 
 #include "mongo/base/string_data.h"
+#include "mongo/db/active_index_builds.h"
 #include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/commit_quorum_options.h"
 #include "mongo/db/catalog/index_build_oplog_entry.h"
 #include "mongo/db/catalog/index_builds.h"
 #include "mongo/db/catalog/index_builds_manager.h"
+#include "mongo/db/commands/server_status.h"
 #include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/index/column_key_generator.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/rebuild_indexes.h"
 #include "mongo/db/repl/oplog_entry.h"
 #include "mongo/db/repl_index_build_state.h"
-#include "mongo/db/storage/durable_catalog.h"
+#include "mongo/db/resumable_index_builds_gen.h"
+#include "mongo/db/serverless/serverless_types_gen.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/executor/thread_pool_task_executor.h"
 #include "mongo/platform/mutex.h"
@@ -86,14 +90,10 @@ public:
      */
     struct IndexBuildOptions {
         boost::optional<CommitQuorumOptions> commitQuorum;
-        bool replSetAndNotPrimaryAtStart = false;
         ApplicationMode applicationMode = ApplicationMode::kNormal;
     };
 
-    /**
-     * Invariants that there are no index builds in-progress.
-     */
-    virtual ~IndexBuildsCoordinator();
+    virtual ~IndexBuildsCoordinator() = default;
 
     /**
      * Executes tasks that must be done prior to destruction of the instance.
@@ -120,12 +120,6 @@ public:
                                          const NamespaceString& nss,
                                          const std::vector<BSONObj>& indexSpecs);
 
-    /**
-     * Returns true if two phase index builds are supported.
-     * This is determined by the current FCV and the server parameter 'enableTwoPhaseIndexBuild'.
-     */
-    static bool supportsTwoPhaseIndexBuild();
-
     /**
      * Returns index names listed from the index specs list "specs".
      */
@@ -138,7 +132,7 @@ public:
      *
      * A Future is returned that will complete when the index build commits or aborts.
      *
-     * On a successful index build, calling Future::get(), or Future::getNoThrows(), returns index
+     * On a successful index build, calling Future::get(), or Future::getNoThrow(), returns index
      * catalog statistics.
      *
      * Returns an error status if there are any errors setting up the index build.
@@ -146,18 +140,39 @@ public:
     virtual StatusWith<SharedSemiFuture<ReplIndexBuildState::IndexCatalogStats>> startIndexBuild(
         OperationContext* opCtx,
         std::string dbName,
-        CollectionUUID collectionUUID,
+        const UUID& collectionUUID,
         const std::vector<BSONObj>& specs,
         const UUID& buildUUID,
         IndexBuildProtocol protocol,
         IndexBuildOptions indexBuildOptions) = 0;
 
     /**
-     * Given a set of two-phase index builds, start, but do not complete each one in a background
-     * thread. Each index build will wait for a replicated commit or abort, as in steady-state
-     * replication.
+     * Reconstructs the in-memory state of the index build. When successful, returns after the index
+     * build has been resumed from the phase it left off in.
+     *
+     * A Future is returned that will complete when the index build commits or aborts.
+     *
+     * On a successful index build, calling Future::get(), or Future::getNoThrow(), returns index
+     * catalog statistics.
+     *
+     * Returns an error status if there are any errors setting up the index build.
      */
-    void restartIndexBuildsForRecovery(OperationContext* opCtx, const IndexBuilds& buildsToRestart);
+    virtual StatusWith<SharedSemiFuture<ReplIndexBuildState::IndexCatalogStats>> resumeIndexBuild(
+        OperationContext* opCtx,
+        std::string dbName,
+        const UUID& collectionUUID,
+        const std::vector<BSONObj>& specs,
+        const UUID& buildUUID,
+        const ResumeIndexInfo& resumeInfo) = 0;
+
+    /**
+     * Resumes and restarts index builds for recovery. Anything that fails to resume will be started
+     * in a background thread. Each index build will wait for a replicated commit or abort, as in
+     * steady-state.
+     */
+    void restartIndexBuildsForRecovery(OperationContext* opCtx,
+                                       const IndexBuilds& buildsToRestart,
+                                       const std::vector<ResumeIndexInfo>& buildsToResume);
 
     /**
      * Runs the full index rebuild for recovery. This will only rebuild single-phase index builds.
@@ -220,10 +235,9 @@ public:
      * being aborted by another caller.
      */
     std::vector<UUID> abortCollectionIndexBuilds(OperationContext* opCx,
-                                                 const NamespaceString collectionNss,
-                                                 const UUID collectionUUID,
-                                                 const std::string& reason,
-                                                 const bool onlyAbortTwoPhaseIndexBuilds = false);
+                                                 NamespaceString collectionNss,
+                                                 UUID collectionUUID,
+                                                 const std::string& reason);
 
     /**
      * Signals all of the index builds on the specified 'db' to abort and then waits until the index
@@ -236,9 +250,21 @@ public:
      */
     void abortDatabaseIndexBuilds(OperationContext* opCtx,
                                   StringData db,
-                                  const std::string& reason,
-                                  const bool onlyAbortTwoPhaseIndexBuilds = false);
+                                  const std::string& reason);
 
+    /**
+     * Signals all of the index builds belonging to the specified tenant to abort and then waits
+     * until the index builds are no longer running. The provided 'reason' will be used in the
+     * error message that the index builders return to their callers.
+     *
+     * Does not require holding locks.
+     *
+     * Does not stop new index builds from starting. Caller must make that guarantee.
+     */
+    void abortTenantIndexBuilds(OperationContext* opCtx,
+                                MigrationProtocolEnum protocol,
+                                StringData db,
+                                const std::string& reason);
     /**
      * Signals all of the index builds to abort and then waits until the index builds are no longer
      * running. The provided 'reason' will be used in the error message that the index builders
@@ -250,11 +276,20 @@ public:
      */
     void abortAllIndexBuildsForInitialSync(OperationContext* opCtx, const std::string& reason);
 
+    /**
+     * Signals all index builds on non-internal databases to abort and waits until they are no
+     * longer running.
+     *
+     * Does not require holding locks.
+     *
+     * Does not stop new index builds from starting. Caller must make that guarantee.
+     */
+    void abortUserIndexBuildsForUserWriteBlocking(OperationContext* opCtx);
+
     /**
      * Aborts an index build by index build UUID. Returns when the index build thread exits.
      *
-     * Returns true if the index build was aborted or the index build is already in the process of
-     * being aborted.
+     * Returns true if the index build was aborted or the index build is already aborted.
      * Returns false if the index build does not exist or the index build is already in the process
      * of committing and cannot be aborted.
      */
@@ -321,11 +356,6 @@ public:
                                    const std::vector<StringData>& indexNames,
                                    const CommitQuorumOptions& newCommitQuorum) = 0;
 
-    /**
-     * TODO: This is not yet implemented.
-     */
-    void recoverIndexBuilds();
-
     /**
      * Returns true if there are no index builds in progress.
      */
@@ -362,12 +392,6 @@ public:
      */
     void assertNoBgOpInProgForDb(StringData db) const;
 
-    /**
-     * Waits for the index build with 'buildUUID' to finish before returning.
-     * Returns immediately if no such index build with 'buildUUID' is found.
-     */
-    void awaitIndexBuildFinished(OperationContext* opCtx, const UUID& buildUUID);
-
     /**
      * Waits for all index builds on a specified collection to finish.
      */
@@ -383,13 +407,17 @@ public:
     void awaitNoBgOpInProgForDb(OperationContext* opCtx, StringData db);
 
     /**
-     * Called by the replication coordinator when a replica set reconfig occurs, which could affect
-     * any index build to make their commit quorum unachievable.
-     *
-     * Checks if the commit quorum is still satisfiable for each index build, if it is no longer
-     * satisfiable, then those index builds are aborted.
+     * Waits until an index build completes. If there are no index builds in progress, returns
+     * immediately.
      */
-    void onReplicaSetReconfig();
+    void waitUntilAnIndexBuildFinishes(OperationContext* opCtx);
+
+
+    /**
+     * Appends the current state information of the index build to the builder.
+     * Does nothing if build UUID does not refer to an active index build.
+     */
+    void appendBuildInfo(const UUID& buildUUID, BSONObjBuilder* builder) const;
 
     //
     // Helper functions for creating indexes that do not have to be managed by the
@@ -397,35 +425,31 @@ public:
     //
 
     /**
-     * Creates indexes in collection.
-     * Assumes callers has necessary locks.
-     * For two phase index builds, writes both startIndexBuild and commitIndexBuild oplog entries
-     * on success. No two phase index build oplog entries, including abortIndexBuild, will be
-     * written on failure.
+     * Creates the specified index without yielding locks.
+     * Assumes the caller has collection MODE_X lock.
      * Throws exception on error.
      */
-    void createIndexes(OperationContext* opCtx,
-                       UUID collectionUUID,
-                       const std::vector<BSONObj>& specs,
-                       IndexBuildsManager::IndexConstraints indexConstraints,
-                       bool fromMigrate);
+    void createIndex(OperationContext* opCtx,
+                     UUID collectionUUID,
+                     const BSONObj& spec,
+                     IndexBuildsManager::IndexConstraints indexConstraints,
+                     bool fromMigrate);
 
     /**
-     * Creates indexes on an empty collection.
-     * Assumes we are enclosed in a WriteUnitOfWork and caller has necessary locks.
-     * For two phase index builds, writes both startIndexBuild and commitIndexBuild oplog entries
-     * on success. No two phase index build oplog entries, including abortIndexBuild, will be
-     * written on failure.
-     * Throws exception on error.
+     * Creates the specified indexes on an empty collection without yielding locks.
+     * Assumes we are enclosed in a WriteUnitOfWork and the caller has exclusive access to the
+     * collection. For two phase index builds, writes both startIndexBuild and commitIndexBuild
+     * oplog entries on success. No two phase index build oplog entries, including abortIndexBuild,
+     * will be written on failure. Throws exception on error.
      */
     static void createIndexesOnEmptyCollection(OperationContext* opCtx,
-                                               UUID collectionUUID,
+                                               CollectionWriter& collection,
                                                const std::vector<BSONObj>& specs,
                                                bool fromMigrate);
 
     void sleepIndexBuilds_forTestOnly(bool sleep);
 
-    void verifyNoIndexBuilds_forTestOnly();
+    void verifyNoIndexBuilds_forTestOnly() const;
 
     /**
      * Helper function that adds collation defaults to 'indexSpecs', as well as filtering out
@@ -437,10 +461,26 @@ public:
      * This function throws on error. Expects caller to have exclusive access to `collection`.
      */
     static std::vector<BSONObj> prepareSpecListForCreate(OperationContext* opCtx,
-                                                         Collection* collection,
+                                                         const CollectionPtr& collection,
                                                          const NamespaceString& nss,
                                                          const std::vector<BSONObj>& indexSpecs);
 
+    /**
+     * Helper function which normalizes a vector of index specs. This function will populate a
+     * complete collation spec in cases where the index spec specifies a collation, and will add
+     * the collection-default collation, if present, in cases where collation is omitted. If the
+     * index spec omits the collation and the collection does not have a default, the collation
+     * field is omitted from the spec. This function also converts the 'wildcardProjection' to
+     * canonical form in any cases where it exists.
+     *
+     * If 'collection' is null, no changes are made to the input specs.
+     *
+     * This function throws on error.
+     */
+    static std::vector<BSONObj> normalizeIndexSpecs(OperationContext* opCtx,
+                                                    const CollectionPtr& collection,
+                                                    const std::vector<BSONObj>& indexSpecs);
+
     /**
      * Returns total number of indexes in collection, including unfinished/in-progress indexes.
      *
@@ -448,34 +488,57 @@ public:
      *
      * Expects a lock to be held by the caller, so that 'collection' is safe to use.
      */
-    static int getNumIndexesTotal(OperationContext* opCtx, Collection* collection);
+    static int getNumIndexesTotal(OperationContext* opCtx, const CollectionPtr& collection);
 
+    class IndexBuildsSSS : public ServerStatusSection {
+    public:
+        IndexBuildsSSS();
 
-    /**
-     * Sets the index build action 'signal' for the index build pointed by 'replState'. Also, it
-     * cancels if there is any active remote 'voteCommitIndexBuild' command request callback handle
-     * for this index build.
-     */
-    virtual void setSignalAndCancelVoteRequestCbkIfActive(
-        WithLock ReplIndexBuildStateLk,
-        OperationContext* opCtx,
-        std::shared_ptr<ReplIndexBuildState> replState,
-        IndexBuildAction signal) = 0;
+        bool includeByDefault() const final {
+            return true;
+        }
+
+        void addRequiredPrivileges(std::vector<Privilege>* out) final {}
+
+        BSONObj generateSection(OperationContext* opCtx,
+                                const BSONElement& configElement) const final {
+            BSONObjBuilder indexBuilds;
+            BSONObjBuilder phases;
+
+            indexBuilds.append("total", registered.loadRelaxed());
+
+            phases.append("scanCollection", scanCollection.loadRelaxed());
+            phases.append("drainSideWritesTable", drainSideWritesTable.loadRelaxed());
+            phases.append("drainSideWritesTablePreCommit",
+                          drainSideWritesTablePreCommit.loadRelaxed());
+            phases.append("waitForCommitQuorum", waitForCommitQuorum.loadRelaxed());
+            phases.append("drainSideWritesTableOnCommit",
+                          drainSideWritesTableOnCommit.loadRelaxed());
+            phases.append("processConstraintsViolatonTableOnCommit",
+                          processConstraintsViolatonTableOnCommit.loadRelaxed());
+            phases.append("commit", commit.loadRelaxed());
+
+            indexBuilds.append("phases", phases.obj());
+
+            return indexBuilds.obj();
+        }
+
+        AtomicWord<int> registered;
+        AtomicWord<int> scanCollection;
+        AtomicWord<int> drainSideWritesTable;
+        AtomicWord<int> drainSideWritesTablePreCommit;
+        AtomicWord<int> waitForCommitQuorum;
+        AtomicWord<int> drainSideWritesTableOnCommit;
+        AtomicWord<int> processConstraintsViolatonTableOnCommit;
+        AtomicWord<int> commit;
+    } indexBuildsSSS;
 
 private:
-    /**
-     * Registers an index build so that the rest of the system can discover it.
-     *
-     * If stopIndexBuildsOnNsOrDb has been called on the index build's collection or database, then
-     * an error will be returned.
-     */
-    Status _registerIndexBuild(WithLock, std::shared_ptr<ReplIndexBuildState> replIndexBuildState);
-
     /**
      * Sets up the in-memory and durable state of the index build.
      *
-     * This function should only be called when in recovery mode, because we drop and replace
-     * existing indexes in a single WriteUnitOfWork.
+     * This function should only be called when in recovery mode, because the index tables are
+     * recreated.
      */
     Status _startIndexBuildForRecovery(OperationContext* opCtx,
                                        const NamespaceString& nss,
@@ -483,13 +546,17 @@ private:
                                        const UUID& buildUUID,
                                        IndexBuildProtocol protocol);
 
-protected:
     /**
-     * Unregisters the index build.
+     * Removes the in-memory and durable state of the passed in indexes in preparation of rebuilding
+     * them for repair.
+     *
+     * This function should only be called when in recovery mode.
      */
-    void _unregisterIndexBuild(WithLock lk,
-                               std::shared_ptr<ReplIndexBuildState> replIndexBuildState);
+    Status _dropIndexesForRepair(OperationContext* opCtx,
+                                 CollectionWriter& collection,
+                                 const std::vector<std::string>& indexNames);
 
+protected:
     /**
      * Sets up the in-memory state of the index build. Validates index specs and filters out
      * existing indexes from the list of specs.
@@ -502,7 +569,7 @@ protected:
     StatusWith<boost::optional<SharedSemiFuture<ReplIndexBuildState::IndexCatalogStats>>>
     _filterSpecsAndRegisterBuild(OperationContext* opCtx,
                                  StringData dbName,
-                                 CollectionUUID collectionUUID,
+                                 const UUID& collectionUUID,
                                  const std::vector<BSONObj>& specs,
                                  const UUID& buildUUID,
                                  IndexBuildProtocol protocol);
@@ -535,9 +602,20 @@ protected:
      */
     Status _setUpIndexBuildForTwoPhaseRecovery(OperationContext* opCtx,
                                                StringData dbName,
-                                               CollectionUUID collectionUUID,
+                                               const UUID& collectionUUID,
                                                const std::vector<BSONObj>& specs,
                                                const UUID& buildUUID);
+    /**
+     * Reconstructs the in-memory state of the index build so that it can be resumed from the phase
+     * it was in when the node cleanly shut down.
+     */
+    Status _setUpResumeIndexBuild(OperationContext* opCtx,
+                                  std::string dbName,
+                                  const UUID& collectionUUID,
+                                  const std::vector<BSONObj>& specs,
+                                  const UUID& buildUUID,
+                                  const ResumeIndexInfo& resumeInfo);
+
     /**
      * Runs the index build on the caller thread. Handles unregistering the index build and setting
      * the index build's Promise with the outcome of the index build.
@@ -545,7 +623,8 @@ protected:
      */
     void _runIndexBuild(OperationContext* opCtx,
                         const UUID& buildUUID,
-                        const IndexBuildOptions& indexBuildOptions) noexcept;
+                        const IndexBuildOptions& indexBuildOptions,
+                        const boost::optional<ResumeIndexInfo>& resumeInfo) noexcept;
 
     /**
      * Acquires locks and runs index build. Throws on error.
@@ -553,13 +632,24 @@ protected:
      */
     void _runIndexBuildInner(OperationContext* opCtx,
                              std::shared_ptr<ReplIndexBuildState> replState,
-                             const IndexBuildOptions& indexBuildOptions);
+                             const IndexBuildOptions& indexBuildOptions,
+                             const boost::optional<ResumeIndexInfo>& resumeInfo);
+
+    /**
+     * Resumes the index build from the phase that it was in when the node cleanly shut down. By the
+     * time this function is called, the in-memory state of the index build should already have been
+     * reconstructed.
+     */
+    void _resumeIndexBuildFromPhase(OperationContext* opCtx,
+                                    std::shared_ptr<ReplIndexBuildState> replState,
+                                    const IndexBuildOptions& indexBuildOptions,
+                                    const ResumeIndexInfo& resumeInfo);
 
     /**
      * Cleans up a single-phase index build after a failure.
      */
     void _cleanUpSinglePhaseAfterFailure(OperationContext* opCtx,
-                                         Collection* collection,
+                                         const CollectionPtr& collection,
                                          std::shared_ptr<ReplIndexBuildState> replState,
                                          const IndexBuildOptions& indexBuildOptions,
                                          const Status& status);
@@ -568,32 +658,34 @@ protected:
      * Cleans up a two-phase index build after a failure.
      */
     void _cleanUpTwoPhaseAfterFailure(OperationContext* opCtx,
-                                      Collection* collection,
+                                      const CollectionPtr& collection,
                                       std::shared_ptr<ReplIndexBuildState> replState,
                                       const IndexBuildOptions& indexBuildOptions,
                                       const Status& status);
 
-    /**
-     * Attempt to abort an index build. Returns a flag indicating how the caller should proceed.
-     */
-    enum class TryAbortResult { kRetry, kAlreadyAborted, kNotAborted, kContinueAbort };
-    TryAbortResult _tryAbort(OperationContext* opCtx,
-                             std::shared_ptr<ReplIndexBuildState> replState,
-                             IndexBuildAction signalAction,
-                             std::string reason);
     /**
      * Performs last steps of aborting an index build.
      */
     void _completeAbort(OperationContext* opCtx,
                         std::shared_ptr<ReplIndexBuildState> replState,
+                        const CollectionPtr& indexBuildEntryCollection,
                         IndexBuildAction signalAction,
                         Status reason);
     void _completeSelfAbort(OperationContext* opCtx,
                             std::shared_ptr<ReplIndexBuildState> replState,
+                            const CollectionPtr& indexBuildEntryCollection,
                             Status reason);
     void _completeAbortForShutdown(OperationContext* opCtx,
                                    std::shared_ptr<ReplIndexBuildState> replState,
-                                   Collection* collection);
+                                   const CollectionPtr& collection);
+
+    /**
+     * Waits for the last optime before the interceptors were installed on the node to be majority
+     * committed and sets that the collection scan for the index build should use a majority read
+     * cursor. If no such optime was recorded, it will do nothing.
+     */
+    void _awaitLastOpTimeBeforeInterceptorsMajorityCommitted(
+        OperationContext* opCtx, std::shared_ptr<ReplIndexBuildState> replState);
 
     /**
      * Modularizes the _indexBuildsManager calls part of _runIndexBuildInner. Throws on error.
@@ -604,12 +696,22 @@ protected:
 
     /**
      * First phase is the collection scan and insertion of the keys into the sorter.
+     * Second phase is extracting the sorted keys and writing them into the new index table.
      */
-    void _scanCollectionAndInsertKeysIntoSorter(OperationContext* opCtx,
-                                                std::shared_ptr<ReplIndexBuildState> replState);
+    void _scanCollectionAndInsertSortedKeysIntoIndex(
+        OperationContext* opCtx,
+        std::shared_ptr<ReplIndexBuildState> replState,
+        const boost::optional<RecordId>& resumeAfterRecordId = boost::none);
+    /**
+     * Performs the second phase of the index build, for use when resuming from the second phase.
+     */
+    void _insertSortedKeysIntoIndexForResume(OperationContext* opCtx,
+                                             std::shared_ptr<ReplIndexBuildState> replState);
+    CollectionPtr _setUpForScanCollectionAndInsertSortedKeysIntoIndex(
+        OperationContext* opCtx, std::shared_ptr<ReplIndexBuildState> replState);
 
     /**
-     * Second phase is extracting the sorted keys and writing them into the new index table.
+     * Third phase is catching up on all the writes that occurred during the first two phases.
      */
     void _insertKeysFromSideTablesWithoutBlockingWrites(
         OperationContext* opCtx, std::shared_ptr<ReplIndexBuildState> replState);
@@ -674,9 +776,6 @@ protected:
         std::shared_ptr<ReplIndexBuildState> replState,
         const IndexBuildOptions& indexBuildOptions) = 0;
 
-    std::string _indexBuildActionToString(IndexBuildAction action);
-
-
     /**
      * Third phase is catching up on all the writes that occurred during the first two phases.
      * Accepts a commit timestamp for the index, which could be null. See
@@ -716,7 +815,7 @@ protected:
      */
     StatusWith<std::pair<long long, long long>> _runIndexRebuildForRecovery(
         OperationContext* opCtx,
-        Collection* collection,
+        CollectionWriter& collection,
         const UUID& buildUUID,
         RepairData repair) noexcept;
 
@@ -736,25 +835,13 @@ protected:
      * Requires caller to lock '_mutex'.
      */
     using IndexBuildFilterFn = std::function<bool(const ReplIndexBuildState& replState)>;
-    std::vector<std::shared_ptr<ReplIndexBuildState>> _filterIndexBuilds_inlock(
-        WithLock lk, IndexBuildFilterFn indexBuildFilter) const;
-
-    void _awaitNoBgOpInProgForDb(stdx::unique_lock<Latch>& lk,
-                                 OperationContext* opCtx,
-                                 StringData db);
-    // Protects the below state.
-    mutable Mutex _mutex = MONGO_MAKE_LATCH("IndexBuildsCoordinator::_mutex");
-
-    // Build UUID to index build information map.
-    stdx::unordered_map<UUID, std::shared_ptr<ReplIndexBuildState>, UUID::Hash> _allIndexBuilds;
-
-    // Waiters are notified whenever one of the three maps above has something added or removed.
-    stdx::condition_variable _indexBuildsCondVar;
 
     // Handles actually building the indexes.
     IndexBuildsManager _indexBuildsManager;
 
-    bool _sleepForTest = false;
+    // Maintains data structures relating to activeIndexBuilds. Thread safe, unless a specific
+    // function specifies otherwise.
+    ActiveIndexBuilds activeIndexBuilds;
 };
 
 // These fail points are used to control index build progress. Declared here to be shared
