@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include "mongo/platform/basic.h"
 
@@ -35,6 +34,7 @@
 
 #include <algorithm>
 
+#include "mongo/db/audit.h"
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/index_build_entry_gen.h"
 #include "mongo/db/concurrency/locker.h"
@@ -43,21 +43,32 @@
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index_build_entry_helpers.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/repl/tenant_migration_access_blocker_util.h"
+#include "mongo/db/s/forwardable_operation_metadata.h"
+#include "mongo/db/s/global_user_write_block_state.h"
 #include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/stats/resource_consumption_metrics.h"
 #include "mongo/db/storage/two_phase_index_build_knobs_gen.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/scoped_counter.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
+
+
 namespace mongo {
 
 namespace {
 
+MONGO_FAIL_POINT_DEFINE(hangAfterAcquiringIndexBuildSlot);
 MONGO_FAIL_POINT_DEFINE(hangBeforeInitializingIndexBuild);
+MONGO_FAIL_POINT_DEFINE(hangIndexBuildAfterSignalPrimaryForCommitReadiness);
+MONGO_FAIL_POINT_DEFINE(hangBeforeRunningIndexBuild);
 
 const StringData kMaxNumActiveUserIndexBuildsServerParameterName = "maxNumActiveUserIndexBuilds"_sd;
 
@@ -92,11 +103,10 @@ IndexBuildsCoordinatorMongod::IndexBuildsCoordinatorMongod()
 
     // Change the 'setOnUpdate' function for the server parameter to signal the condition variable
     // when the value changes.
-    ServerParameter* serverParam =
-        ServerParameterSet::getGlobal()->get(kMaxNumActiveUserIndexBuildsServerParameterName);
-    static_cast<
-        IDLServerParameterWithStorage<ServerParameterType::kStartupAndRuntime, AtomicWord<int>>*>(
-        serverParam)
+    using ParamT =
+        IDLServerParameterWithStorage<ServerParameterType::kStartupAndRuntime, AtomicWord<int>>;
+    ServerParameterSet::getNodeParameterSet()
+        ->get<ParamT>(kMaxNumActiveUserIndexBuildsServerParameterName)
         ->setOnUpdate([this](const int) -> Status {
             _indexBuildFinished.notify_all();
             return Status::OK();
@@ -117,13 +127,51 @@ void IndexBuildsCoordinatorMongod::shutdown(OperationContext* opCtx) {
 StatusWith<SharedSemiFuture<ReplIndexBuildState::IndexCatalogStats>>
 IndexBuildsCoordinatorMongod::startIndexBuild(OperationContext* opCtx,
                                               std::string dbName,
-                                              CollectionUUID collectionUUID,
+                                              const UUID& collectionUUID,
                                               const std::vector<BSONObj>& specs,
                                               const UUID& buildUUID,
                                               IndexBuildProtocol protocol,
                                               IndexBuildOptions indexBuildOptions) {
+    return _startIndexBuild(
+        opCtx, dbName, collectionUUID, specs, buildUUID, protocol, indexBuildOptions, boost::none);
+}
+
+StatusWith<SharedSemiFuture<ReplIndexBuildState::IndexCatalogStats>>
+IndexBuildsCoordinatorMongod::resumeIndexBuild(OperationContext* opCtx,
+                                               std::string dbName,
+                                               const UUID& collectionUUID,
+                                               const std::vector<BSONObj>& specs,
+                                               const UUID& buildUUID,
+                                               const ResumeIndexInfo& resumeInfo) {
+    IndexBuildsCoordinator::IndexBuildOptions indexBuildOptions;
+    indexBuildOptions.applicationMode = ApplicationMode::kStartupRepair;
+    return _startIndexBuild(opCtx,
+                            dbName,
+                            collectionUUID,
+                            specs,
+                            buildUUID,
+                            IndexBuildProtocol::kTwoPhase,
+                            indexBuildOptions,
+                            resumeInfo);
+}
+
+StatusWith<SharedSemiFuture<ReplIndexBuildState::IndexCatalogStats>>
+IndexBuildsCoordinatorMongod::_startIndexBuild(OperationContext* opCtx,
+                                               std::string dbName,
+                                               const UUID& collectionUUID,
+                                               const std::vector<BSONObj>& specs,
+                                               const UUID& buildUUID,
+                                               IndexBuildProtocol protocol,
+                                               IndexBuildOptions indexBuildOptions,
+                                               const boost::optional<ResumeIndexInfo>& resumeInfo) {
     const NamespaceStringOrUUID nssOrUuid{dbName, collectionUUID};
 
+    auto writeBlockState = GlobalUserWriteBlockState::get(opCtx);
+
+    invariant(!opCtx->lockState()->isRSTLExclusive(), buildUUID.toString());
+
+    const auto nss = CollectionCatalog::get(opCtx)->resolveNamespaceStringOrUUID(opCtx, nssOrUuid);
+
     {
         // Only operations originating from user connections need to wait while there are more than
         // 'maxNumActiveUserIndexBuilds' index builds currently running.
@@ -146,7 +194,14 @@ IndexBuildsCoordinatorMongod::startIndexBuild(OperationContext* opCtx,
                         replCoord->canAcceptWritesFor(opCtx, nssOrUuid));
             }
 
-            stdx::unique_lock<Latch> lk(_mutex);
+            // The checks here catch empty index builds and also allow us to stop index
+            // builds before waiting for throttling. It may race with the abort at the start
+            // of migration so we do check again later.
+            uassertStatusOK(tenant_migration_access_blocker::checkIfCanBuildIndex(opCtx, dbName));
+            uassertStatusOK(writeBlockState->checkIfIndexBuildAllowedToStart(opCtx, nss));
+
+            stdx::unique_lock<Latch> lk(_throttlingMutex);
+            bool messageLogged = false;
             opCtx->waitForConditionOrInterrupt(_indexBuildFinished, lk, [&] {
                 const int maxActiveBuilds = maxNumActiveUserIndexBuilds.load();
                 if (_numActiveIndexBuilds < maxActiveBuilds) {
@@ -154,35 +209,50 @@ IndexBuildsCoordinatorMongod::startIndexBuild(OperationContext* opCtx,
                     return true;
                 }
 
-                LOGV2(4715500,
-                      "Too many index builds running simultaneously, waiting until the number of "
-                      "active index builds is below the threshold",
-                      "numActiveIndexBuilds"_attr = _numActiveIndexBuilds,
-                      "maxNumActiveUserIndexBuilds"_attr = maxActiveBuilds,
-                      "indexSpecs"_attr = specs,
-                      "buildUUID"_attr = buildUUID,
-                      "collectionUUID"_attr = collectionUUID);
+                if (!messageLogged) {
+                    LOGV2(
+                        4715500,
+                        "Too many index builds running simultaneously, waiting until the number of "
+                        "active index builds is below the threshold",
+                        "numActiveIndexBuilds"_attr = _numActiveIndexBuilds,
+                        "maxNumActiveUserIndexBuilds"_attr = maxActiveBuilds,
+                        "indexSpecs"_attr = specs,
+                        "buildUUID"_attr = buildUUID,
+                        "collectionUUID"_attr = collectionUUID);
+                    messageLogged = true;
+                }
                 return false;
             });
         } else {
             // System index builds have no limit and never wait, but do consume a slot.
-            stdx::unique_lock<Latch> lk(_mutex);
+            stdx::unique_lock<Latch> lk(_throttlingMutex);
             _numActiveIndexBuilds++;
         }
     }
 
-    auto onScopeExitGuard = makeGuard([&] {
-        stdx::unique_lock<Latch> lk(_mutex);
+    ScopeGuard onScopeExitGuard([&] {
+        stdx::unique_lock<Latch> lk(_throttlingMutex);
         _numActiveIndexBuilds--;
         _indexBuildFinished.notify_one();
     });
 
+    if (MONGO_unlikely(hangAfterAcquiringIndexBuildSlot.shouldFail())) {
+        LOGV2(4886201, "Hanging index build due to failpoint 'hangAfterAcquiringIndexBuildSlot'");
+        hangAfterAcquiringIndexBuildSlot.pauseWhileSet();
+    }
+
     if (indexBuildOptions.applicationMode == ApplicationMode::kStartupRepair) {
-        // Two phase index build recovery goes though a different set-up procedure because the
-        // original index will be dropped first.
+        // Two phase index build recovery goes though a different set-up procedure because we will
+        // either resume the index build or the original index will be dropped first.
         invariant(protocol == IndexBuildProtocol::kTwoPhase);
-        auto status =
-            _setUpIndexBuildForTwoPhaseRecovery(opCtx, dbName, collectionUUID, specs, buildUUID);
+        auto status = Status::OK();
+        if (resumeInfo) {
+            status = _setUpResumeIndexBuild(
+                opCtx, dbName, collectionUUID, specs, buildUUID, resumeInfo.value());
+        } else {
+            status = _setUpIndexBuildForTwoPhaseRecovery(
+                opCtx, dbName, collectionUUID, specs, buildUUID);
+        }
         if (!status.isOK()) {
             return status;
         }
@@ -194,22 +264,41 @@ IndexBuildsCoordinatorMongod::startIndexBuild(OperationContext* opCtx,
         }
 
         if (statusWithOptionalResult.getValue()) {
-            // TODO (SERVER-37644): when joining is implemented, the returned Future will no longer
-            // always be set.
             invariant(statusWithOptionalResult.getValue()->isReady());
             // The requested index (specs) are already built or are being built. Return success
             // early (this is v4.0 behavior compatible).
-            return statusWithOptionalResult.getValue().get();
+            return statusWithOptionalResult.getValue().value();
+        }
+
+        if (opCtx->getClient()->isFromUserConnection()) {
+            auto migrationStatus =
+                tenant_migration_access_blocker::checkIfCanBuildIndex(opCtx, dbName);
+            if (!migrationStatus.isOK()) {
+                LOGV2(4886200,
+                      "Aborted index build before start due to tenant migration",
+                      "error"_attr = migrationStatus,
+                      "buildUUID"_attr = buildUUID,
+                      "collectionUUID"_attr = collectionUUID);
+                activeIndexBuilds.unregisterIndexBuild(&_indexBuildsManager,
+                                                       invariant(_getIndexBuild(buildUUID)));
+                return migrationStatus;
+            }
+
+            auto buildBlockedStatus = writeBlockState->checkIfIndexBuildAllowedToStart(opCtx, nss);
+            if (!buildBlockedStatus.isOK()) {
+                LOGV2(6511603,
+                      "Aborted index build due to user index builds being blocked",
+                      "error"_attr = buildBlockedStatus,
+                      "buildUUID"_attr = buildUUID,
+                      "collectionUUID"_attr = collectionUUID);
+                activeIndexBuilds.unregisterIndexBuild(&_indexBuildsManager,
+                                                       invariant(_getIndexBuild(buildUUID)));
+                return buildBlockedStatus;
+            }
         }
     }
 
-    invariant(!opCtx->lockState()->isRSTLExclusive(), buildUUID.toString());
-
-    const auto nss = CollectionCatalog::get(opCtx).resolveNamespaceStringOrUUID(opCtx, nssOrUuid);
-
-    const auto& oss = OperationShardingState::get(opCtx);
-    const auto shardVersion = oss.getShardVersion(nss);
-    const auto dbVersion = oss.getDbVersion(dbName);
+    auto& oss = OperationShardingState::get(opCtx);
 
     // Task in thread pool should have similar CurOp representation to the caller so that it can be
     // identified as a createIndexes operation.
@@ -234,6 +323,11 @@ IndexBuildsCoordinatorMongod::startIndexBuild(OperationContext* opCtx,
 
     auto replState = invariant(_getIndexBuild(buildUUID));
 
+    // Since index builds occur in a separate thread, client attributes that are audited must be
+    // extracted from the client object and passed into the thread separately.
+    audit::ImpersonatedClientAttrs impersonatedClientAttrs(opCtx->getClient());
+    ForwardableOperationMetadata forwardableOpMetadata(opCtx);
+
     // The thread pool task will be responsible for signalling the condition variable when the index
     // build thread is done running.
     onScopeExitGuard.dismiss();
@@ -248,27 +342,39 @@ IndexBuildsCoordinatorMongod::startIndexBuild(OperationContext* opCtx,
         replState,
         startPromise = std::move(startPromise),
         startTimestamp,
-        shardVersion,
-        dbVersion
+        shardVersion = oss.getShardVersion(nss),
+        dbVersion = oss.getDbVersion(dbName),
+        resumeInfo,
+        impersonatedClientAttrs = std::move(impersonatedClientAttrs),
+        forwardableOpMetadata = std::move(forwardableOpMetadata)
     ](auto status) mutable noexcept {
-        auto onScopeExitGuard = makeGuard([&] {
-            stdx::unique_lock<Latch> lk(_mutex);
+        ScopeGuard onScopeExitGuard([&] {
+            stdx::unique_lock<Latch> lk(_throttlingMutex);
             _numActiveIndexBuilds--;
             _indexBuildFinished.notify_one();
         });
 
         // Clean up if we failed to schedule the task.
         if (!status.isOK()) {
-            stdx::unique_lock<Latch> lk(_mutex);
-            _unregisterIndexBuild(lk, replState);
+            activeIndexBuilds.unregisterIndexBuild(&_indexBuildsManager, replState);
             startPromise.setError(status);
             return;
         }
 
         auto opCtx = Client::getCurrent()->makeOperationContext();
 
-        auto& oss = OperationShardingState::get(opCtx.get());
-        oss.initializeClientRoutingVersions(nss, shardVersion, dbVersion);
+        // Forward the forwardable operation metadata from the external client to this thread's
+        // client.
+        forwardableOpMetadata.setOn(opCtx.get());
+
+        // Load the external client's attributes into this thread's client for auditing.
+        auto authSession = AuthorizationSession::get(opCtx->getClient());
+        if (authSession) {
+            authSession->setImpersonatedUserData(std::move(impersonatedClientAttrs.userName),
+                                                 std::move(impersonatedClientAttrs.roleNames));
+        }
+
+        ScopedSetShardRole scopedSetShardRole(opCtx.get(), nss, shardVersion, dbVersion);
 
         {
             stdx::unique_lock<Client> lk(*opCtx->getClient());
@@ -281,6 +387,14 @@ IndexBuildsCoordinatorMongod::startIndexBuild(OperationContext* opCtx,
             sleepmillis(100);
         }
 
+        // Start collecting metrics for the index build. The metrics for this operation will only be
+        // aggregated globally if the node commits or aborts while it is primary.
+        auto& metricsCollector = ResourceConsumption::MetricsCollector::get(opCtx.get());
+        if (ResourceConsumption::shouldCollectMetricsForDatabase(dbName) &&
+            ResourceConsumption::isMetricsCollectionEnabled()) {
+            metricsCollector.beginScopedCollecting(opCtx.get(), dbName);
+        }
+
         // Index builds should never take the PBWM lock, even on a primary. This allows the
         // index build to continue running after the node steps down to a secondary.
         ShouldNotConflictWithSecondaryBatchApplicationBlock shouldNotConflictBlock(
@@ -297,9 +411,11 @@ IndexBuildsCoordinatorMongod::startIndexBuild(OperationContext* opCtx,
         // Signal that the index build started successfully.
         startPromise.setWith([] {});
 
+        hangBeforeRunningIndexBuild.pauseWhileSet(opCtx.get());
+
         // Runs the remainder of the index build. Sets the promise result and cleans up the index
         // build.
-        _runIndexBuild(opCtx.get(), buildUUID, indexBuildOptions);
+        _runIndexBuild(opCtx.get(), buildUUID, indexBuildOptions, resumeInfo);
 
         // Do not exit with an incomplete future.
         invariant(replState->sharedPromise.getFuture().isReady());
@@ -341,7 +457,7 @@ Status IndexBuildsCoordinatorMongod::voteCommitIndexBuild(OperationContext* opCt
         // commit quorum is disabled, do not record their entry into the commit ready nodes.
         // If we fail to retrieve the persisted commit quorum, the index build might be in the
         // middle of tearing down.
-        Lock::SharedLock commitQuorumLk(opCtx->lockState(), replState->commitQuorumLock.get());
+        Lock::SharedLock commitQuorumLk(opCtx->lockState(), replState->commitQuorumLock.value());
         auto commitQuorum =
             uassertStatusOK(indexbuildentryhelpers::getCommitQuorum(opCtx, buildUUID));
         if (commitQuorum.numNodes == CommitQuorumOptions::kDisabled) {
@@ -373,41 +489,9 @@ Status IndexBuildsCoordinatorMongod::voteCommitIndexBuild(OperationContext* opCt
     return persistStatus;
 }
 
-void IndexBuildsCoordinatorMongod::setSignalAndCancelVoteRequestCbkIfActive(
-    WithLock ReplIndexBuildStateLk,
-    OperationContext* opCtx,
-    std::shared_ptr<ReplIndexBuildState> replState,
-    IndexBuildAction signal) {
-    // set the signal
-    replState->waitForNextAction->emplaceValue(signal);
-    // Cancel the callback.
-    if (replState->voteCmdCbkHandle.isValid()) {
-        repl::ReplicationCoordinator::get(opCtx)->cancelCbkHandle(replState->voteCmdCbkHandle);
-    }
-}
-
 void IndexBuildsCoordinatorMongod::_sendCommitQuorumSatisfiedSignal(
     OperationContext* opCtx, std::shared_ptr<ReplIndexBuildState> replState) {
-    stdx::unique_lock<Latch> ReplIndexBuildStateLk(replState->mutex);
-    if (!replState->waitForNextAction->getFuture().isReady()) {
-        setSignalAndCancelVoteRequestCbkIfActive(
-            ReplIndexBuildStateLk, opCtx, replState, IndexBuildAction::kCommitQuorumSatisfied);
-    } else {
-        // This implies we already got a commit or abort signal by other ways. This might have
-        // been signaled earlier with kPrimaryAbort or kCommitQuorumSatisfied. Or, it's also
-        // possible the node got stepped down and received kOplogCommit/koplogAbort or got
-        // kRollbackAbort. So, it's ok to skip signaling.
-        auto action = replState->waitForNextAction->getFuture().get(opCtx);
-
-        LOGV2(3856200,
-              "Not signaling \"{skippedAction}\" as it was previously signaled with "
-              "\"{previousAction}\" for index build: {buildUUID}",
-              "Skipping signaling as it was previously signaled for index build",
-              "skippedAction"_attr =
-                  _indexBuildActionToString(IndexBuildAction::kCommitQuorumSatisfied),
-              "previousAction"_attr = _indexBuildActionToString(action),
-              "buildUUID"_attr = replState->buildUUID);
-    }
+    replState->setCommitQuorumSatisfied(opCtx);
 }
 
 void IndexBuildsCoordinatorMongod::_signalIfCommitQuorumIsSatisfied(
@@ -415,7 +499,7 @@ void IndexBuildsCoordinatorMongod::_signalIfCommitQuorumIsSatisfied(
 
     // Acquire the commitQuorumLk in shared mode to make sure commit quorum value did not change
     // after reading it from config.system.indexBuilds collection.
-    Lock::SharedLock commitQuorumLk(opCtx->lockState(), replState->commitQuorumLock.get());
+    Lock::SharedLock commitQuorumLk(opCtx->lockState(), replState->commitQuorumLock.value());
 
     // Read the index builds entry from config.system.indexBuilds collection.
     auto swIndexBuildEntry =
@@ -429,7 +513,7 @@ void IndexBuildsCoordinatorMongod::_signalIfCommitQuorumIsSatisfied(
         return;
 
     bool commitQuorumSatisfied = repl::ReplicationCoordinator::get(opCtx)->isCommitQuorumSatisfied(
-        indexBuildEntry.getCommitQuorum(), voteMemberList.get());
+        indexBuildEntry.getCommitQuorum(), voteMemberList.value());
 
     if (!commitQuorumSatisfied)
         return;
@@ -444,22 +528,7 @@ bool IndexBuildsCoordinatorMongod::_signalIfCommitQuorumNotEnabled(
     auto replCoord = repl::ReplicationCoordinator::get(opCtx);
 
     if (IndexBuildProtocol::kSinglePhase == replState->protocol) {
-        // Single-phase builds don't support commit quorum, but they must go through the process of
-        // updating their state to synchronize with concurrent abort operations.
-        stdx::unique_lock<Latch> lk(replState->mutex);
-        if (replState->waitForNextAction->getFuture().isReady()) {
-            // If the signal action has been set, it should only be because a concurrent operation
-            // already aborted the index build.
-            auto action = replState->waitForNextAction->getFuture().get(opCtx);
-            invariant(action == IndexBuildAction::kPrimaryAbort,
-                      str::stream() << "action: " << _indexBuildActionToString(action)
-                                    << ", buildUUID: " << replState->buildUUID);
-            LOGV2(4639700,
-                  "Not committing single-phase build because it has already been aborted",
-                  "buildUUID"_attr = replState->buildUUID);
-            return true;
-        }
-        replState->waitForNextAction->emplaceValue(IndexBuildAction::kSinglePhaseCommit);
+        replState->setSinglePhaseCommit(opCtx);
         return true;
     }
 
@@ -477,7 +546,7 @@ bool IndexBuildsCoordinatorMongod::_signalIfCommitQuorumNotEnabled(
 
     // Acquire the commitQuorumLk in shared mode to make sure commit quorum value did not change
     // after reading it from config.system.indexBuilds collection.
-    Lock::SharedLock commitQuorumLk(opCtx->lockState(), replState->commitQuorumLock.get());
+    Lock::SharedLock commitQuorumLk(opCtx->lockState(), replState->commitQuorumLock.value());
 
     // Read the commit quorum value from config.system.indexBuilds collection.
     auto commitQuorum = uassertStatusOKWithContext(
@@ -523,27 +592,15 @@ void IndexBuildsCoordinatorMongod::_signalPrimaryForCommitReadiness(
 
     Backoff exponentialBackoff(Seconds(1), Seconds(2));
 
-    auto onRemoteCmdScheduled = [replState,
-                                 replCoord](executor::TaskExecutor::CallbackHandle handle) {
-        stdx::unique_lock<Latch> lk(replState->mutex);
-        // We have already received commit or abort signal, So skip voting.
-        if (replState->waitForNextAction->getFuture().isReady()) {
-            replCoord->cancelCbkHandle(handle);
-        } else {
-            invariant(!replState->voteCmdCbkHandle.isValid());
-            replState->voteCmdCbkHandle = handle;
-        }
+    auto onRemoteCmdScheduled = [opCtx, replState](executor::TaskExecutor::CallbackHandle handle) {
+        replState->onVoteRequestScheduled(opCtx, handle);
     };
 
     auto onRemoteCmdComplete = [replState](executor::TaskExecutor::CallbackHandle) {
-        stdx::unique_lock<Latch> lk(replState->mutex);
-        replState->voteCmdCbkHandle = executor::TaskExecutor::CallbackHandle();
+        replState->clearVoteRequestCbk();
     };
 
-    auto needToVote = [replState]() -> bool {
-        stdx::unique_lock<Latch> lk(replState->mutex);
-        return !replState->waitForNextAction->getFuture().isReady() ? true : false;
-    };
+    auto needToVote = [replState]() -> bool { return !replState->getNextActionNoWait(); };
 
     // Retry 'voteCommitIndexBuild' command on error until we have been signaled either with commit
     // or abort. This way, we can make sure majority of nodes will never stop voting and wait for
@@ -598,16 +655,17 @@ void IndexBuildsCoordinatorMongod::_signalPrimaryForCommitReadiness(
             break;
         }
     }
+
+    if (MONGO_unlikely(hangIndexBuildAfterSignalPrimaryForCommitReadiness.shouldFail())) {
+        LOGV2(4841707, "Hanging index build after signaling the primary for commit readiness");
+        hangIndexBuildAfterSignalPrimaryForCommitReadiness.pauseWhileSet(opCtx);
+    }
     return;
 }
 
 IndexBuildAction IndexBuildsCoordinatorMongod::_drainSideWritesUntilNextActionIsAvailable(
     OperationContext* opCtx, std::shared_ptr<ReplIndexBuildState> replState) {
-    auto future = [&] {
-        stdx::unique_lock<Latch> lk(replState->mutex);
-        invariant(replState->waitForNextAction);
-        return replState->waitForNextAction->getFuture();
-    }();
+    auto future = replState->getNextActionFuture();
 
     // Waits until the promise is fulfilled or the deadline expires.
     IndexBuildAction nextAction;
@@ -649,13 +707,19 @@ void IndexBuildsCoordinatorMongod::_waitForNextIndexBuildActionAndCommit(
         invariant(!opCtx->lockState()->isLocked(),
                   str::stream() << "holding locks while waiting for commit or abort: "
                                 << replState->buildUUID);
-        // Future wait can be interrupted.
-        const auto nextAction = _drainSideWritesUntilNextActionIsAvailable(opCtx, replState);
 
+        auto const nextAction = [&] {
+            indexBuildsSSS.waitForCommitQuorum.addAndFetch(1);
+            // Future wait can be interrupted.
+            return _drainSideWritesUntilNextActionIsAvailable(opCtx, replState);
+        }();
         LOGV2(3856204,
               "Index build: received signal",
               "buildUUID"_attr = replState->buildUUID,
-              "action"_attr = _indexBuildActionToString(nextAction));
+              "action"_attr = indexBuildActionToString(nextAction));
+
+        // Tenant migration abort should have been re-written as primary abort before reaching here.
+        invariant(nextAction != IndexBuildAction::kTenantMigrationAbort);
 
         // If the index build was aborted, this serves as a final interruption point. Since the
         // index builder thread is interrupted before the action is set, this must fail if the build
@@ -664,39 +728,41 @@ void IndexBuildsCoordinatorMongod::_waitForNextIndexBuildActionAndCommit(
 
         bool needsToRetryWait = false;
 
+        auto commitTimestamp = replState->getCommitTimestamp();
+
         switch (nextAction) {
             case IndexBuildAction::kOplogCommit: {
                 invariant(replState->protocol == IndexBuildProtocol::kTwoPhase);
-                invariant(replState->indexBuildState.getTimestamp(),
-                          replState->buildUUID.toString());
+                invariant(!commitTimestamp.isNull(), replState->buildUUID.toString());
                 LOGV2(3856205,
                       "Index build: committing from oplog entry",
                       "buildUUID"_attr = replState->buildUUID,
-                      "commitTimestamp"_attr = replState->indexBuildState.getTimestamp().get(),
+                      "commitTimestamp"_attr = commitTimestamp,
                       "collectionUUID"_attr = replState->collectionUUID);
                 break;
             }
             case IndexBuildAction::kCommitQuorumSatisfied: {
-                invariant(!replState->indexBuildState.getTimestamp());
+                invariant(commitTimestamp.isNull(),
+                          str::stream() << "commit ts: " << commitTimestamp.toString()
+                                        << "; index build: " << replState->buildUUID.toString());
                 break;
             }
             case IndexBuildAction::kSinglePhaseCommit:
-                invariant(replState->protocol == IndexBuildProtocol::kSinglePhase);
+                invariant(replState->protocol == IndexBuildProtocol::kSinglePhase,
+                          str::stream() << "commit ts: " << commitTimestamp.toString()
+                                        << "; index build: " << replState->buildUUID.toString());
                 break;
             case IndexBuildAction::kOplogAbort:
             case IndexBuildAction::kRollbackAbort:
             case IndexBuildAction::kInitialSyncAbort:
             case IndexBuildAction::kPrimaryAbort:
+            case IndexBuildAction::kTenantMigrationAbort:
                 // The calling thread should have interrupted us before signaling an abort action.
                 LOGV2_FATAL(4698901, "Index build abort should have interrupted this operation");
             case IndexBuildAction::kNoAction:
                 return;
         }
 
-        Timestamp commitTimestamp = replState->indexBuildState.getTimestamp()
-            ? replState->indexBuildState.getTimestamp().get()
-            : Timestamp();
-
         auto result = _insertKeysFromSideTablesAndCommit(
             opCtx, replState, nextAction, indexBuildOptions, commitTimestamp);
         switch (result) {
@@ -708,11 +774,7 @@ void IndexBuildsCoordinatorMongod::_waitForNextIndexBuildActionAndCommit(
                       "No longer primary while attempting to commit. Waiting again for next action "
                       "before completing final phase",
                       "buildUUID"_attr = replState->buildUUID);
-                {
-                    stdx::unique_lock<Latch> lk(replState->mutex);
-                    replState->waitForNextAction =
-                        std::make_unique<SharedPromise<IndexBuildAction>>();
-                }
+                replState->resetNextActionPromise();
                 needsToRetryWait = true;
                 break;
             case CommitResult::kLockTimeout:
@@ -743,14 +805,9 @@ Status IndexBuildsCoordinatorMongod::setCommitQuorum(OperationContext* opCtx,
                           << nss << "' without providing any indexes.");
     }
 
-    auto replCoord = repl::ReplicationCoordinator::get(opCtx);
-    if (!replCoord->isReplEnabled()) {
-        return Status(ErrorCodes::BadValue,
-                      str::stream() << "Standalones can't specify commitQuorum");
-    }
-
-    AutoGetCollectionForRead autoColl(opCtx, nss);
-    Collection* collection = autoColl.getCollection();
+    // Take the MODE_IX lock now, so that when we actually persist the value later, we don't need to
+    // upgrade the lock.
+    AutoGetCollection collection(opCtx, nss, MODE_IX);
     if (!collection) {
         return Status(ErrorCodes::NamespaceNotFound,
                       str::stream() << "Collection '" << nss << "' was not found.");
@@ -758,41 +815,34 @@ Status IndexBuildsCoordinatorMongod::setCommitQuorum(OperationContext* opCtx,
 
     UUID collectionUUID = collection->uuid();
     std::shared_ptr<ReplIndexBuildState> replState;
-    {
-        stdx::unique_lock<Latch> lk(_mutex);
-        auto pred = [&](const auto& replState) {
-            if (collectionUUID != replState.collectionUUID) {
-                return false;
-            }
-            if (indexNames.size() != replState.indexNames.size()) {
-                return false;
-            }
-            // Ensure the ReplIndexBuildState has the same indexes as 'indexNames'.
-            return std::equal(
-                replState.indexNames.begin(), replState.indexNames.end(), indexNames.begin());
-        };
-        auto collIndexBuilds = _filterIndexBuilds_inlock(lk, pred);
-        if (collIndexBuilds.empty()) {
-            return Status(ErrorCodes::IndexNotFound,
-                          str::stream() << "Cannot find an index build on collection '" << nss
-                                        << "' with the provided index names");
+
+    auto pred = [&](const auto& replState) {
+        if (collectionUUID != replState.collectionUUID) {
+            return false;
         }
-        invariant(
-            1U == collIndexBuilds.size(),
-            str::stream() << "Found multiple index builds with the same index names on collection "
-                          << nss << " (" << collectionUUID
-                          << "): first index name: " << indexNames.front());
-
-        replState = collIndexBuilds.front();
+        if (indexNames.size() != replState.indexNames.size()) {
+            return false;
+        }
+        // Ensure the ReplIndexBuildState has the same indexes as 'indexNames'.
+        return std::equal(
+            replState.indexNames.begin(), replState.indexNames.end(), indexNames.begin());
+    };
+    auto collIndexBuilds = activeIndexBuilds.filterIndexBuilds(pred);
+    if (collIndexBuilds.empty()) {
+        return Status(ErrorCodes::IndexNotFound,
+                      str::stream() << "Cannot find an index build on collection '" << nss
+                                    << "' with the provided index names");
     }
+    invariant(
+        1U == collIndexBuilds.size(),
+        str::stream() << "Found multiple index builds with the same index names on collection "
+                      << nss << " (" << collectionUUID
+                      << "): first index name: " << indexNames.front());
 
-    if (!(IndexBuildProtocol::kTwoPhase == replState->protocol && enableIndexBuildCommitQuorum)) {
-        return Status(ErrorCodes::BadValue,
-                      str::stream() << "commitQuorum is supported only for two phase index builds "
-                                       "with commit quorum support enabled, and requires FCV 4.4");
-    }
+    replState = collIndexBuilds.front();
 
     // See if the new commit quorum is satisfiable.
+    auto replCoord = repl::ReplicationCoordinator::get(opCtx);
     Status status = replCoord->checkIfCommitQuorumCanBeSatisfied(newCommitQuorum);
     if (!status.isOK()) {
         return status;
@@ -824,13 +874,13 @@ Status IndexBuildsCoordinatorMongod::setCommitQuorum(OperationContext* opCtx,
     // About to update the commit quorum value on-disk. So, take the lock in exclusive mode to
     // prevent readers from reading the commit quorum value and making decision on commit quorum
     // satisfied with the stale read commit quorum value.
-    Lock::ExclusiveLock commitQuorumLk(opCtx->lockState(), replState->commitQuorumLock.get());
+    Lock::ExclusiveLock commitQuorumLk(opCtx->lockState(), replState->commitQuorumLock.value());
     {
-        stdx::unique_lock<Latch> lk(replState->mutex);
-        if (replState->waitForNextAction->getFuture().isReady()) {
+        if (auto action = replState->getNextActionNoWait()) {
             return Status(ErrorCodes::CommandFailed,
                           str::stream() << "Commit quorum can't be changed as index build is "
-                                           "ready to commit or abort");
+                                           "ready to commit or abort: "
+                                        << indexBuildActionToString(*action));
         }
     }
 
@@ -841,17 +891,12 @@ Status IndexBuildsCoordinatorMongod::setCommitQuorum(OperationContext* opCtx,
         return status;
     }
 
-    {
-        // Check to see the index build hasn't received commit index build signal while updating
-        // the commit quorum value on-disk.
-        stdx::unique_lock<Latch> lk(replState->mutex);
-        if (replState->waitForNextAction->getFuture().isReady()) {
-            auto action = replState->waitForNextAction->getFuture().get();
-            uassert(
-                ErrorCodes::CommandFailed,
+    // Check to see the index build hasn't received commit index build signal while updating
+    // the commit quorum value on-disk.
+    if (auto action = replState->getNextActionNoWait()) {
+        uassert(ErrorCodes::CommandFailed,
                 str::stream() << "Commit quorum is already satisfied, this command has no effect",
-                action != IndexBuildAction::kCommitQuorumSatisfied);
-        }
+                *action != IndexBuildAction::kCommitQuorumSatisfied);
     }
 
     // If the index builder is already waiting for the commit quorum to be satisfied and the commit
