@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/db/matcher/expression_geo.h"
 
@@ -38,6 +37,9 @@
 #include "mongo/platform/basic.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
+
 namespace mongo {
 
 //
@@ -79,10 +81,7 @@ Status GeoExpression::parseQuery(const BSONObj& obj) {
         BSONElement elt = geoIt.next();
         if (elt.fieldNameStringData() == "$uniqueDocs") {
             // Deprecated "$uniqueDocs" field
-            LOGV2_WARNING(23847,
-                          "Deprecated $uniqueDocs option: {query}",
-                          "Deprecated $uniqueDocs option",
-                          "query"_attr = redact(obj));
+            LOGV2_WARNING(23847, "Deprecated $uniqueDocs option", "query"_attr = redact(obj));
         } else {
             // The element must be a geo specifier. "$box", "$center", "$geometry", etc.
             geoContainer.reset(new GeometryContainer());
@@ -236,6 +235,9 @@ Status GeoNearExpression::parseNewQuery(const BSONObj& obj) {
                       str::stream() << "invalid geo near query operator: " << e.fieldName());
     }
 
+    // Returns true if 'x' is a valid numeric value, that is, a non-negative finite number.
+    auto isValidNumericValue = [](double x) -> bool { return x >= 0.0 && std::isfinite(x); };
+
     // Iterate over the argument.
     BSONObjIterator it(e.embeddedObject());
     while (it.more()) {
@@ -251,19 +253,29 @@ Status GeoNearExpression::parseNewQuery(const BSONObj& obj) {
                                       << "invalid point in geo near query $geometry argument: "
                                       << embeddedObj << "  " << status.reason());
                 }
-                uassert(16681,
-                        "$near requires geojson point, given " + embeddedObj.toString(),
-                        (SPHERE == centroid->crs));
+                if (SPHERE != centroid->crs) {
+                    return Status(ErrorCodes::BadValue,
+                                  str::stream() << "$near requires geojson point, given "
+                                                << embeddedObj.toString());
+                }
                 hasGeometry = true;
             }
         } else if (fieldName == "$minDistance") {
-            uassert(16897, "$minDistance must be a number", e.isNumber());
+            if (!e.isNumber()) {
+                return Status(ErrorCodes::BadValue, "$minDistance must be a number");
+            }
             minDistance = e.Number();
-            uassert(16898, "$minDistance must be non-negative", minDistance >= 0.0);
+            if (!isValidNumericValue(minDistance)) {
+                return Status(ErrorCodes::BadValue, "$minDistance must be non-negative");
+            }
         } else if (fieldName == "$maxDistance") {
-            uassert(16899, "$maxDistance must be a number", e.isNumber());
+            if (!e.isNumber()) {
+                return Status(ErrorCodes::BadValue, "$maxDistance must be a number");
+            }
             maxDistance = e.Number();
-            uassert(16900, "$maxDistance must be non-negative", maxDistance >= 0.0);
+            if (!isValidNumericValue(maxDistance)) {
+                return Status(ErrorCodes::BadValue, "$maxDistance must be non-negative");
+            }
         } else {
             // Return an error if a bad argument was passed inside the query document.
             return Status(ErrorCodes::BadValue,
@@ -298,9 +310,10 @@ Status GeoNearExpression::parseFrom(const BSONObj& obj) {
     // Fixup the near query for anonoyances caused by $nearSphere
     if (isNearSphere) {
         // The user-provided point can be flat for a spherical query - needs to be projectable
-        uassert(17444,
-                "Legacy point is out of bounds for spherical query",
-                ShapeProjection::supportsProject(*centroid, SPHERE));
+        if (!ShapeProjection::supportsProject(*centroid, SPHERE)) {
+            return Status(ErrorCodes::BadValue,
+                          str::stream() << "Legacy point is out of bounds for spherical query");
+        }
 
         unitsAreRadians = SPHERE != centroid->crs;
         // GeoJSON points imply wrapping queries
@@ -330,44 +343,87 @@ Status GeoNearExpression::parseFrom(const BSONObj& obj) {
  */
 GeoMatchExpression::GeoMatchExpression(StringData path,
                                        const GeoExpression* query,
-                                       const BSONObj& rawObj)
-    : LeafMatchExpression(GEO, path), _rawObj(rawObj), _query(query), _canSkipValidation(false) {}
+                                       const BSONObj& rawObj,
+                                       clonable_ptr<ErrorAnnotation> annotation)
+    : LeafMatchExpression(GEO, path, std::move(annotation)),
+      _rawObj(rawObj),
+      _query(query),
+      _canSkipValidation(false) {}
 
 /**
  * Takes shared ownership of the passed-in GeoExpression.
  */
 GeoMatchExpression::GeoMatchExpression(StringData path,
                                        std::shared_ptr<const GeoExpression> query,
-                                       const BSONObj& rawObj)
-    : LeafMatchExpression(GEO, path), _rawObj(rawObj), _query(query), _canSkipValidation(false) {}
+                                       const BSONObj& rawObj,
+                                       clonable_ptr<ErrorAnnotation> annotation)
+    : LeafMatchExpression(GEO, path, std::move(annotation)),
+      _rawObj(rawObj),
+      _query(query),
+      _canSkipValidation(false) {}
 
 bool GeoMatchExpression::matchesSingleElement(const BSONElement& e, MatchDetails* details) const {
+    return contains(_query->getGeometry(), _query->getPred(), _canSkipValidation, e, details);
+}
+
+bool GeoMatchExpression::contains(const GeometryContainer& queryGeom,
+                                  const GeoExpression::Predicate& queryPredicate,
+                                  bool skipValidation,
+                                  const BSONElement& e,
+                                  MatchDetails*) {
     if (!e.isABSONObj())
         return false;
 
     GeometryContainer geometry;
-
-    if (!geometry.parseFromStorage(e, _canSkipValidation).isOK())
+    if (!geometry.parseFromStorage(e, skipValidation).isOK())
         return false;
 
     // Never match big polygon
     if (geometry.getNativeCRS() == STRICT_SPHERE)
         return false;
 
-    // Project this geometry into the CRS of the query
-    if (!geometry.supportsProject(_query->getGeometry().getNativeCRS()))
+    // Project this geometry into the CRS of the larger geometry.
+
+    // In the case of index validation, we are projecting the geometry of the query
+    // into the CRS of the index to confirm that the index region convers/includes
+    // the region described by the predicate.
+
+    if (!geometry.supportsProject(queryGeom.getNativeCRS()))
         return false;
 
-    geometry.projectInto(_query->getGeometry().getNativeCRS());
+    return contains(queryGeom, queryPredicate, &geometry);
+}
 
-    if (GeoExpression::WITHIN == _query->getPred()) {
-        return _query->getGeometry().contains(geometry);
+bool GeoMatchExpression::contains(const GeometryContainer& queryGeom,
+                                  const GeoExpression::Predicate& queryPredicate,
+                                  GeometryContainer* geometry) {
+    geometry->projectInto(queryGeom.getNativeCRS());
+    if (GeoExpression::WITHIN == queryPredicate) {
+        return queryGeom.contains(*geometry);
     } else {
-        verify(GeoExpression::INTERSECT == _query->getPred());
-        return _query->getGeometry().intersects(geometry);
+        verify(GeoExpression::INTERSECT == queryPredicate);
+        return queryGeom.intersects(*geometry);
     }
 }
 
+bool GeoMatchExpression::matchesGeoContainer(const GeometryContainer& input) const {
+    // Never match big polygon
+    if (input.getNativeCRS() == STRICT_SPHERE)
+        return false;
+
+    // Project this geometry into the CRS of the larger geometry.
+
+    // In the case of index validation, we are projecting the geometry of the query
+    // into the CRS of the index to confirm that the index region convers/includes
+    // the region described by the predicate.
+
+    if (!input.supportsProject(_query->getGeometry().getNativeCRS()))
+        return false;
+
+    GeometryContainer geometry{input};
+    return contains(_query->getGeometry(), _query->getPred(), &geometry);
+}
+
 void GeoMatchExpression::debugString(StringBuilder& debug, int indentationLevel) const {
     _debugAddSpace(debug, indentationLevel);
 
@@ -403,12 +459,12 @@ bool GeoMatchExpression::equivalent(const MatchExpression* other) const {
 
 std::unique_ptr<MatchExpression> GeoMatchExpression::shallowClone() const {
     std::unique_ptr<GeoMatchExpression> next =
-        std::make_unique<GeoMatchExpression>(path(), _query, _rawObj);
+        std::make_unique<GeoMatchExpression>(path(), _query, _rawObj, _errorAnnotation);
     next->_canSkipValidation = _canSkipValidation;
     if (getTag()) {
         next->setTag(getTag()->clone());
     }
-    return std::move(next);
+    return next;
 }
 
 //
@@ -465,6 +521,6 @@ std::unique_ptr<MatchExpression> GeoNearMatchExpression::shallowClone() const {
     if (getTag()) {
         next->setTag(getTag()->clone());
     }
-    return std::move(next);
+    return next;
 }
 }  // namespace mongo
