@@ -33,7 +33,6 @@
 
 #include <cmath>
 #include <memory>
-#include <pcrecpp.h>
 
 #include "mongo/bson/bsonelement_comparator.h"
 #include "mongo/bson/bsonmisc.h"
@@ -44,7 +43,9 @@
 #include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/matcher/path.h"
 #include "mongo/db/query/collation/collator_interface.h"
-#include "mongo/util/regex_util.h"
+#include "mongo/util/errno_util.h"
+#include "mongo/util/pcre.h"
+#include "mongo/util/pcre_util.h"
 #include "mongo/util/represent_as.h"
 #include "mongo/util/str.h"
 
@@ -53,11 +54,16 @@ namespace mongo {
 ComparisonMatchExpressionBase::ComparisonMatchExpressionBase(
     MatchType type,
     StringData path,
-    const BSONElement& rhs,
+    Value rhs,
     ElementPath::LeafArrayBehavior leafArrBehavior,
-    ElementPath::NonLeafArrayBehavior nonLeafArrBehavior)
-    : LeafMatchExpression(type, path, leafArrBehavior, nonLeafArrBehavior), _rhs(rhs) {
-    invariant(_rhs);
+    ElementPath::NonLeafArrayBehavior nonLeafArrBehavior,
+    clonable_ptr<ErrorAnnotation> annotation,
+    const CollatorInterface* collator)
+    : LeafMatchExpression(type, path, leafArrBehavior, nonLeafArrBehavior, std::move(annotation)),
+      _backingBSON(BSON(path << rhs)),
+      _collator(collator) {
+    setData(_backingBSON.firstElement());
+    invariant(_rhs.type() != BSONType::EOO);
 }
 
 bool ComparisonMatchExpressionBase::equivalent(const MatchExpression* other) const {
@@ -94,12 +100,16 @@ BSONObj ComparisonMatchExpressionBase::getSerializedRightHandSide() const {
 
 ComparisonMatchExpression::ComparisonMatchExpression(MatchType type,
                                                      StringData path,
-                                                     const BSONElement& rhs)
+                                                     Value rhs,
+                                                     clonable_ptr<ErrorAnnotation> annotation,
+                                                     const CollatorInterface* collator)
     : ComparisonMatchExpressionBase(type,
                                     path,
-                                    rhs,
+                                    std::move(rhs),
                                     ElementPath::LeafArrayBehavior::kTraverse,
-                                    ElementPath::NonLeafArrayBehavior::kTraverse) {
+                                    ElementPath::NonLeafArrayBehavior::kTraverse,
+                                    std::move(annotation),
+                                    collator) {
     uassert(
         ErrorCodes::BadValue, "cannot compare to undefined", _rhs.type() != BSONType::Undefined);
 
@@ -117,59 +127,76 @@ ComparisonMatchExpression::ComparisonMatchExpression(MatchType type,
 
 bool ComparisonMatchExpression::matchesSingleElement(const BSONElement& e,
                                                      MatchDetails* details) const {
-    if (e.canonicalType() != _rhs.canonicalType()) {
-        // We can't call 'compareElements' on elements of different canonical types.  Usually
-        // elements with different canonical types should never match any comparison, but there are
-        // a few exceptions, handled here.
-
-        // jstNULL and undefined are treated the same
-        if (e.canonicalType() + _rhs.canonicalType() == 5) {
-            return matchType() == EQ || matchType() == LTE || matchType() == GTE;
+    if (e.type() != _rhs.type()) {
+        const auto ect = e.canonicalType();
+        const auto rct = _rhs.canonicalType();
+        if (ect != rct) {
+            // We can't call 'compareElements' on elements of different canonical types.  Usually
+            // elements with different canonical types should never match any comparison, but there
+            // are a few exceptions, handled here.
+            // jstNULL and undefined are treated the same
+            if (ect + rct == 5) {
+                return matchType() == EQ || matchType() == LTE || matchType() == GTE;
+            }
+            if (_rhs.type() == MaxKey || _rhs.type() == MinKey) {
+                switch (matchType()) {
+                    // LT and LTE need no distinction here because the two elements that we are
+                    // comparing do not even have the same canonical type and are thus not equal
+                    // (i.e.the case where we compare MinKey against MinKey would not reach this
+                    // switch statement at all).  The same reasoning follows for the lack of
+                    // distinction between GTE and GT.
+                    case LT:
+                    case LTE:
+                        return _rhs.type() == MaxKey;
+                    case EQ:
+                        return false;
+                    case GT:
+                    case GTE:
+                        return _rhs.type() == MinKey;
+                    default:
+                        // This is a comparison match expression, so it must be either
+                        // a $lt, $lte, $gt, $gte, or equality expression.
+                        MONGO_UNREACHABLE;
+                }
+            }
+            return false;
         }
-        if (_rhs.type() == MaxKey || _rhs.type() == MinKey) {
+    }
+
+    if (matchType() == EQ) {
+        if (!_collator && e.type() == String) {
+            // We know from above that _rhs must also be a String (or Symbol which has the same
+            // representation) so if they have different value sizes, they must be different
+            // strings. We can only stop here with the default collator, since other collators may
+            // consider different length strings as equal.
+            if (e.valuesize() != _rhs.valuesize())
+                return false;
+        }
+    } else {
+        // Special case handling for NaN. NaN is equal to NaN but otherwise always compares to
+        // false. This follows the normal comparison rules (where NaN is less than all numbers) for
+        // EQ, so we only need to do this for other comparison types.
+        const bool lhsIsNan =  //
+            (((e.type() == NumberDouble) && (std::isnan(e._numberDouble())))) ||
+            ((e.type() == NumberDecimal) && (e._numberDecimal().isNaN()));
+        const bool rhsIsNan =
+            (((_rhs.type() == NumberDouble) && (std::isnan(_rhs._numberDouble()))) ||
+             ((_rhs.type() == NumberDecimal) && (_rhs._numberDecimal().isNaN())));
+        if (lhsIsNan || rhsIsNan) {
+            bool bothNaN = lhsIsNan && rhsIsNan;
             switch (matchType()) {
-                // LT and LTE need no distinction here because the two elements that we are
-                // comparing do not even have the same canonical type and are thus not equal
-                // (i.e.the case where we compare MinKey against MinKey would not reach this switch
-                // statement at all).  The same reasoning follows for the lack of distinction
-                // between GTE and GT.
                 case LT:
-                case LTE:
-                    return _rhs.type() == MaxKey;
-                case EQ:
-                    return false;
                 case GT:
+                    return false;
+                case LTE:
                 case GTE:
-                    return _rhs.type() == MinKey;
+                    return bothNaN;
                 default:
                     // This is a comparison match expression, so it must be either
                     // a $lt, $lte, $gt, $gte, or equality expression.
-                    MONGO_UNREACHABLE;
+                    fassertFailed(17448);
             }
         }
-        return false;
-    }
-
-    // Special case handling for NaN. NaN is equal to NaN but
-    // otherwise always compares to false.
-    if (std::isnan(e.numberDouble()) || std::isnan(_rhs.numberDouble())) {
-        bool bothNaN = std::isnan(e.numberDouble()) && std::isnan(_rhs.numberDouble());
-        switch (matchType()) {
-            case LT:
-                return false;
-            case LTE:
-                return bothNaN;
-            case EQ:
-                return bothNaN;
-            case GT:
-                return false;
-            case GTE:
-                return bothNaN;
-            default:
-                // This is a comparison match expression, so it must be either
-                // a $lt, $lte, $gt, $gte, or equality expression.
-                fassertFailed(17448);
-        }
     }
 
     int x = BSONElement::compareElements(
@@ -200,35 +227,27 @@ constexpr StringData GTEMatchExpression::kName;
 
 const std::set<char> RegexMatchExpression::kValidRegexFlags = {'i', 'm', 's', 'x'};
 
-RegexMatchExpression::RegexMatchExpression(StringData path, const BSONElement& e)
-    : LeafMatchExpression(REGEX, path),
-      _regex(e.regex()),
-      _flags(e.regexFlags()),
-      _re(new pcrecpp::RE(_regex.c_str(), regex_util::flagsToPcreOptions(_flags, true))) {
-    uassert(ErrorCodes::BadValue, "regex not a regex", e.type() == RegEx);
-    _init();
+std::unique_ptr<pcre::Regex> RegexMatchExpression::makeRegex(const std::string& regex,
+                                                             const std::string& flags) {
+    return std::make_unique<pcre::Regex>(regex, pcre_util::flagsToOptions(flags));
 }
 
-RegexMatchExpression::RegexMatchExpression(StringData path, StringData regex, StringData options)
-    : LeafMatchExpression(REGEX, path),
+RegexMatchExpression::RegexMatchExpression(StringData path,
+                                           StringData regex,
+                                           StringData options,
+                                           clonable_ptr<ErrorAnnotation> annotation)
+    : LeafMatchExpression(REGEX, path, std::move(annotation)),
       _regex(regex.toString()),
       _flags(options.toString()),
-      _re(new pcrecpp::RE(_regex.c_str(), regex_util::flagsToPcreOptions(_flags, true))) {
-    _init();
-}
+      _re(makeRegex(_regex, _flags)) {
 
-void RegexMatchExpression::_init() {
     uassert(ErrorCodes::BadValue,
             "Regular expression cannot contain an embedded null byte",
             _regex.find('\0') == std::string::npos);
 
-    uassert(ErrorCodes::BadValue,
-            "Regular expression options string cannot contain an embedded null byte",
-            _flags.find('\0') == std::string::npos);
-
     uassert(51091,
-            str::stream() << "Regular expression is invalid: " << _re->error(),
-            _re->error().empty());
+            str::stream() << "Regular expression is invalid: " << errorMessage(_re->error()),
+            *_re);
 }
 
 RegexMatchExpression::~RegexMatchExpression() {}
@@ -245,13 +264,8 @@ bool RegexMatchExpression::equivalent(const MatchExpression* other) const {
 bool RegexMatchExpression::matchesSingleElement(const BSONElement& e, MatchDetails* details) const {
     switch (e.type()) {
         case String:
-        case Symbol: {
-            // String values stored in documents can contain embedded NUL bytes. We construct a
-            // pcrecpp::StringPiece instance using the full length of the string to avoid truncating
-            // 'data' early.
-            pcrecpp::StringPiece data(e.valuestr(), e.valuestrsize() - 1);
-            return _re->PartialMatch(data);
-        }
+        case Symbol:
+            return !!_re->matchView(e.valueStringData());
         case RegEx:
             return _regex == e.regex() && _flags == e.regexFlags();
         default:
@@ -292,8 +306,13 @@ void RegexMatchExpression::shortDebugString(StringBuilder& debug) const {
 
 // ---------
 
-ModMatchExpression::ModMatchExpression(StringData path, long long divisor, long long remainder)
-    : LeafMatchExpression(MOD, path), _divisor(divisor), _remainder(remainder) {
+ModMatchExpression::ModMatchExpression(StringData path,
+                                       long long divisor,
+                                       long long remainder,
+                                       clonable_ptr<ErrorAnnotation> annotation)
+    : LeafMatchExpression(MOD, path, std::move(annotation)),
+      _divisor(divisor),
+      _remainder(remainder) {
     uassert(ErrorCodes::BadValue, "divisor cannot be 0", divisor != 0);
 }
 
@@ -363,7 +382,9 @@ bool ModMatchExpression::equivalent(const MatchExpression* other) const {
 
 // ------------------
 
-ExistsMatchExpression::ExistsMatchExpression(StringData path) : LeafMatchExpression(EXISTS, path) {}
+ExistsMatchExpression::ExistsMatchExpression(StringData path,
+                                             clonable_ptr<ErrorAnnotation> annotation)
+    : LeafMatchExpression(EXISTS, path, std::move(annotation)) {}
 
 bool ExistsMatchExpression::matchesSingleElement(const BSONElement& e,
                                                  MatchDetails* details) const {
@@ -396,26 +417,31 @@ bool ExistsMatchExpression::equivalent(const MatchExpression* other) const {
 
 // ----
 
-InMatchExpression::InMatchExpression(StringData path)
-    : LeafMatchExpression(MATCH_IN, path),
+InMatchExpression::InMatchExpression(StringData path, clonable_ptr<ErrorAnnotation> annotation)
+    : LeafMatchExpression(MATCH_IN, path, std::move(annotation)),
       _eltCmp(BSONElementComparator::FieldNamesMode::kIgnore, _collator) {}
 
 std::unique_ptr<MatchExpression> InMatchExpression::shallowClone() const {
-    auto next = std::make_unique<InMatchExpression>(path());
+    auto next = std::make_unique<InMatchExpression>(path(), _errorAnnotation);
     next->setCollator(_collator);
     if (getTag()) {
         next->setTag(getTag()->clone());
     }
     next->_hasNull = _hasNull;
     next->_hasEmptyArray = _hasEmptyArray;
+    next->_hasNonEmptyArrayOrObject = _hasNonEmptyArrayOrObject;
     next->_equalitySet = _equalitySet;
     next->_originalEqualityVector = _originalEqualityVector;
+    next->_equalityStorage = _equalityStorage;
     for (auto&& regex : _regexes) {
         std::unique_ptr<RegexMatchExpression> clonedRegex(
             static_cast<RegexMatchExpression*>(regex->shallowClone().release()));
         next->_regexes.push_back(std::move(clonedRegex));
     }
-    return std::move(next);
+    if (getInputParamId()) {
+        next->setInputParamId(*getInputParamId());
+    }
+    return next;
 }
 
 bool InMatchExpression::contains(const BSONElement& e) const {
@@ -423,7 +449,9 @@ bool InMatchExpression::contains(const BSONElement& e) const {
 }
 
 bool InMatchExpression::matchesSingleElement(const BSONElement& e, MatchDetails* details) const {
-    if (_hasNull && e.eoo()) {
+    // When an $in has a null, it adopts the same semantics as {$eq:null}. Namely, in addition to
+    // matching literal null values, the $in should match missing and undefined.
+    if (_hasNull && (e.eoo() || e.type() == BSONType::Undefined)) {
         return true;
     }
     if (contains(e)) {
@@ -545,6 +573,8 @@ Status InMatchExpression::setEqualities(std::vector<BSONElement> equalities) {
             _hasNull = true;
         } else if (equality.type() == BSONType::Array && equality.Obj().isEmpty()) {
             _hasEmptyArray = true;
+        } else if (equality.type() == BSONType::Array || equality.type() == BSONType::Object) {
+            _hasNonEmptyArrayOrObject = true;
         }
     }
 
@@ -567,6 +597,10 @@ Status InMatchExpression::setEqualities(std::vector<BSONElement> equalities) {
     return Status::OK();
 }
 
+void InMatchExpression::setBackingBSON(BSONObj equalityStorage) {
+    _equalityStorage = std::move(equalityStorage);
+}
+
 Status InMatchExpression::addRegex(std::unique_ptr<RegexMatchExpression> expr) {
     _regexes.push_back(std::move(expr));
     return Status::OK();
@@ -590,7 +624,7 @@ MatchExpression::ExpressionOptimizerFunc InMatchExpression::getOptimizer() const
             if (expression->getTag()) {
                 simplifiedExpression->setTag(expression->getTag()->clone());
             }
-            return std::move(simplifiedExpression);
+            return simplifiedExpression;
         } else if (equalitySet.size() == 1 && regexList.empty()) {
             // Simplify IN of exactly one equality to be an EqualityMatchExpression.
             auto simplifiedExpression = std::make_unique<EqualityMatchExpression>(
@@ -600,7 +634,7 @@ MatchExpression::ExpressionOptimizerFunc InMatchExpression::getOptimizer() const
                 simplifiedExpression->setTag(expression->getTag()->clone());
             }
 
-            return std::move(simplifiedExpression);
+            return simplifiedExpression;
         }
 
         return expression;
@@ -611,8 +645,10 @@ MatchExpression::ExpressionOptimizerFunc InMatchExpression::getOptimizer() const
 
 BitTestMatchExpression::BitTestMatchExpression(MatchType type,
                                                StringData path,
-                                               std::vector<uint32_t> bitPositions)
-    : LeafMatchExpression(type, path), _bitPositions(std::move(bitPositions)) {
+                                               std::vector<uint32_t> bitPositions,
+                                               clonable_ptr<ErrorAnnotation> annotation)
+    : LeafMatchExpression(type, path, std::move(annotation)),
+      _bitPositions(std::move(bitPositions)) {
     // Process bit positions into bitmask.
     for (auto bitPosition : _bitPositions) {
         // Checking bits > 63 is just checking the sign bit, since we sign-extend numbers. For
@@ -623,8 +659,11 @@ BitTestMatchExpression::BitTestMatchExpression(MatchType type,
     }
 }
 
-BitTestMatchExpression::BitTestMatchExpression(MatchType type, StringData path, uint64_t bitMask)
-    : LeafMatchExpression(type, path), _bitMask(bitMask) {
+BitTestMatchExpression::BitTestMatchExpression(MatchType type,
+                                               StringData path,
+                                               uint64_t bitMask,
+                                               clonable_ptr<ErrorAnnotation> annotation)
+    : LeafMatchExpression(type, path, std::move(annotation)), _bitMask(bitMask) {
     // Process bitmask into bit positions.
     for (int bit = 0; bit < 64; bit++) {
         if (_bitMask & (1ULL << bit)) {
@@ -636,8 +675,9 @@ BitTestMatchExpression::BitTestMatchExpression(MatchType type, StringData path,
 BitTestMatchExpression::BitTestMatchExpression(MatchType type,
                                                StringData path,
                                                const char* bitMaskBinary,
-                                               uint32_t bitMaskLen)
-    : LeafMatchExpression(type, path) {
+                                               uint32_t bitMaskLen,
+                                               clonable_ptr<ErrorAnnotation> annotation)
+    : LeafMatchExpression(type, path, std::move(annotation)) {
     for (uint32_t byte = 0; byte < bitMaskLen; byte++) {
         char byteAt = bitMaskBinary[byte];
         if (!byteAt) {
@@ -840,4 +880,19 @@ bool BitTestMatchExpression::equivalent(const MatchExpression* other) const {
 
     return path() == realOther->path() && myBitPositions == otherBitPositions;
 }
+
+void EncryptedBetweenMatchExpression::debugString(StringBuilder& debug,
+                                                  int indentationLevel) const {
+    _debugAddSpace(debug, indentationLevel);
+    debug << path() << " " << kName;
+    debug << " " << rhs().toString(false);
+
+    MatchExpression::TagData* td = getTag();
+    if (td) {
+        debug << " ";
+        td->debugString(&debug);
+    }
+
+    debug << "\n";
+}
 }  // namespace mongo
