@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
@@ -47,7 +46,7 @@
 #include "mongo/db/commands/server_status.h"
 #include "mongo/db/mirror_maestro_gen.h"
 #include "mongo/db/mirroring_sampler.h"
-#include "mongo/db/repl/is_master_response.h"
+#include "mongo/db/repl/hello_response.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/topology_version_observer.h"
 #include "mongo/executor/connection_pool.h"
@@ -58,6 +57,9 @@
 #include "mongo/util/concurrency/thread_pool.h"
 #include "mongo/util/synchronized_value.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
+
+
 namespace mongo {
 
 namespace {
@@ -70,6 +72,7 @@ constexpr auto kMirroredReadsParamName = "mirrorReads"_sd;
 
 constexpr auto kMirroredReadsSeenKey = "seen"_sd;
 constexpr auto kMirroredReadsSentKey = "sent"_sd;
+constexpr auto kMirroredReadsReceivedKey = "received"_sd;
 constexpr auto kMirroredReadsResolvedKey = "resolved"_sd;
 constexpr auto kMirroredReadsResolvedBreakdownKey = "resolvedBreakdown"_sd;
 constexpr auto kMirroredReadsPendingKey = "pending"_sd;
@@ -80,17 +83,17 @@ MONGO_FAIL_POINT_DEFINE(mirrorMaestroTracksPending);
 class MirrorMaestroImpl {
 public:
     /**
-     * Make the TaskExecutor and initialize other components
+     * Make the TaskExecutor and initialize other components.
      */
     void init(ServiceContext* serviceContext) noexcept;
 
     /**
-     * Shutdown the TaskExecutor and cancel any outstanding work
+     * Shutdown the TaskExecutor and cancel any outstanding work.
      */
     void shutdown() noexcept;
 
     /**
-     * Mirror only if this maestro has been initialized
+     * Mirror only if this maestro has been initialized.
      */
     void tryMirror(std::shared_ptr<CommandInvocation> invocation) noexcept;
 
@@ -182,6 +185,7 @@ public:
         BSONObjBuilder section;
         section.append(kMirroredReadsSeenKey, seen.loadRelaxed());
         section.append(kMirroredReadsSentKey, sent.loadRelaxed());
+        section.append(kMirroredReadsReceivedKey, received.loadRelaxed());
 
         if (MONGO_unlikely(mirrorMaestroExpectsResponse.shouldFail())) {
             // We only can see if the command resolved if we got a response
@@ -233,10 +237,12 @@ public:
     AtomicWord<CounterT> resolved;
     // Counts the number of operations that are scheduled to be mirrored, but haven't yet been sent.
     AtomicWord<CounterT> pending;
+    // Counts the number of mirrored operations received by this node as a secondary.
+    AtomicWord<CounterT> received;
 } gMirroredReadsSection;
 
 auto parseMirroredReadsParameters(const BSONObj& obj) {
-    IDLParserErrorContext ctx("mirrorReads");
+    IDLParserContext ctx("mirrorReads");
     return MirroredReadsParameters::parse(ctx, obj);
 }
 
@@ -294,6 +300,13 @@ void MirrorMaestro::tryMirrorRequest(OperationContext* opCtx) noexcept {
     impl.tryMirror(std::move(invocation));
 }
 
+void MirrorMaestro::onReceiveMirroredRead(OperationContext* opCtx) noexcept {
+    const auto& invocation = CommandInvocation::get(opCtx);
+    if (MONGO_unlikely(invocation->isMirrored())) {
+        gMirroredReadsSection.received.fetchAndAddRelaxed(1);
+    }
+}
+
 void MirrorMaestroImpl::tryMirror(std::shared_ptr<CommandInvocation> invocation) noexcept {
     if (!_isInitialized.load()) {
         // If we're not even available, nothing to do
@@ -327,7 +340,8 @@ void MirrorMaestroImpl::tryMirror(std::shared_ptr<CommandInvocation> invocation)
     auto clientExecutor = ClientOutOfLineExecutor::get(Client::getCurrent());
     auto clientExecutorHandle = clientExecutor->getHandle();
 
-    // TODO SERVER-46619 delegates collection to the client's baton
+    // NOTE: before using Client's out-of-line executor outside of MirrorMaestro, we must first
+    // move the consumption (i.e., `consumeAllTasks`) to the baton.
     clientExecutor->consumeAllTasks();
 
     // There is the potential to actually mirror requests, so schedule the _mirror() invocation
@@ -369,6 +383,9 @@ void MirrorMaestroImpl::_mirror(const std::vector<HostAndPort>& hosts,
         // Limit the maxTimeMS
         bob.append("maxTimeMS", params.getMaxTimeMS());
 
+        // Indicate that this is a mirrored read.
+        bob.append("mirrored", true);
+
         {
             // Set secondaryPreferred read preference
             BSONObjBuilder rpBob = bob.subobjStart("$readPreference");
@@ -415,22 +432,26 @@ void MirrorMaestroImpl::_mirror(const std::vector<HostAndPort>& hosts,
 
         auto newRequest = executor::RemoteCommandRequest(
             host, invocation->ns().db().toString(), payload, nullptr);
-        if (MONGO_likely(!mirrorMaestroExpectsResponse.shouldFail())) {
-            // If we're not expecting a response, set to fire and forget
-            newRequest.fireAndForgetMode = executor::RemoteCommandRequest::FireAndForgetMode::kOn;
-        }
+
+        newRequest.options.fireAndForget = true;
+        if (MONGO_unlikely(mirrorMaestroExpectsResponse.shouldFail()))
+            newRequest.options.fireAndForget = false;
 
         LOGV2_DEBUG(31455, 4, "About to mirror", "host"_attr = host, "request"_attr = newRequest);
 
         auto status =
             _executor->scheduleRemoteCommand(newRequest, std::move(mirrorResponseCallback))
                 .getStatus();
-        uassertStatusOK(status);
 
+        if (ErrorCodes::isShutdownError(status.code())) {
+            LOGV2_DEBUG(5723501, 1, "Aborted mirroring due to shutdown", "reason"_attr = status);
+            return;
+        }
+
+        tassert(status);
         gMirroredReadsSection.sent.fetchAndAdd(1);
     }
 } catch (const DBException& e) {
-    // TODO SERVER-44570 Invariant this only in testing
     LOGV2_DEBUG(31456, 2, "Mirroring failed", "reason"_attr = e);
 }
 
@@ -473,8 +494,8 @@ void MirrorMaestroImpl::init(ServiceContext* serviceContext) noexcept {
     _executor->startup();
     _topologyVersionObserver.init(serviceContext);
 
-    _params =
-        ServerParameterSet::getGlobal()->get<MirroredReadsServerParameter>(kMirroredReadsParamName);
+    _params = ServerParameterSet::getNodeParameterSet()->get<MirroredReadsServerParameter>(
+        kMirroredReadsParamName);
     invariant(_params);
 
     // Set _initGuard.liveness to kRunning
