@@ -35,8 +35,10 @@
 #include "mongo/base/status.h"
 #include "mongo/db/client.h"
 #include "mongo/db/concurrency/locker.h"
-#include "mongo/db/logical_session_id.h"
 #include "mongo/db/operation_id.h"
+#include "mongo/db/query/datetime/date_time_support.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/logical_session_id_helpers.h"
 #include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/storage_options.h"
 #include "mongo/db/storage/write_unit_of_work.h"
@@ -45,8 +47,10 @@
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
 #include "mongo/transport/session.h"
+#include "mongo/util/cancellation.h"
 #include "mongo/util/concurrency/with_lock.h"
 #include "mongo/util/decorable.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/interruptible.h"
 #include "mongo/util/lockable_adapter.h"
 #include "mongo/util/time_support.h"
@@ -63,6 +67,19 @@ namespace repl {
 class UnreplicatedWritesBlock;
 }  // namespace repl
 
+// Enabling the maxTimeAlwaysTimeOut fail point will cause any query or command run with a
+// valid non-zero max time to fail immediately.  Any getmore operation on a cursor already
+// created with a valid non-zero max time will also fail immediately.
+//
+// This fail point cannot be used with the maxTimeNeverTimeOut fail point.
+extern FailPoint maxTimeAlwaysTimeOut;
+
+// Enabling the maxTimeNeverTimeOut fail point will cause the server to never time out any
+// query, command, or getmore operation, regardless of whether a max time is set.
+//
+// This fail point cannot be used with the maxTimeAlwaysTimeOut fail point.
+extern FailPoint maxTimeNeverTimeOut;
+
 /**
  * This class encompasses the state required by an operation and lives from the time a network
  * operation is dispatched until its execution is finished. Note that each "getmore" on a cursor
@@ -79,6 +96,8 @@ class OperationContext : public Interruptible, public Decorable<OperationContext
     OperationContext& operator=(const OperationContext&) = delete;
 
 public:
+    static constexpr auto kDefaultOperationContextTimeoutError = ErrorCodes::ExceededTimeLimit;
+
     /**
      * Creates an op context with no unique operation ID tracking - prefer using the OperationIdSlot
      * CTOR if possible to avoid OperationId collisions.
@@ -105,17 +124,16 @@ public:
     /**
      * Returns the RecoveryUnit (same return value as recoveryUnit()) but the caller takes
      * ownership of the returned RecoveryUnit, and the OperationContext instance relinquishes
-     * ownership.  Sets the RecoveryUnit to NULL.
-     *
-     * Used to transfer ownership of storage engine state from OperationContext
-     * to ClientCursor for getMore-able queries.
-     *
-     * Note that we don't allow the top-level locks to be stored across getMore.
-     * We rely on active cursors being killed when collections or databases are dropped,
-     * or when collection metadata changes.
+     * ownership. Sets the RecoveryUnit to NULL.
      */
     std::unique_ptr<RecoveryUnit> releaseRecoveryUnit();
 
+    /*
+     * Similar to releaseRecoveryUnit(), but sets up a new, inactive RecoveryUnit after releasing
+     * the existing one.
+     */
+    std::unique_ptr<RecoveryUnit> releaseAndReplaceRecoveryUnit();
+
     /**
      * Associates the OperatingContext with a different RecoveryUnit for getMore or
      * subtransactions, see RecoveryUnitSwap. The new state is passed and the old state is
@@ -216,6 +234,22 @@ public:
         return _txnNumber;
     }
 
+    /**
+     * Returns the txnRetryCounter associated with this operation.
+     */
+    boost::optional<TxnRetryCounter> getTxnRetryCounter() const {
+        return _txnRetryCounter;
+    }
+
+    /**
+     * Returns a CancellationToken that will be canceled when the OperationContext is killed via
+     * markKilled (including for internal reasons, like the OperationContext deadline being
+     * reached).
+     */
+    CancellationToken getCancellationToken() {
+        return _cancelSource.token();
+    }
+
     /**
      * Sets a transport Baton on the operation.  This will trigger the Baton on markKilled.
      */
@@ -236,6 +270,13 @@ public:
      */
     void setTxnNumber(TxnNumber txnNumber);
 
+    /**
+     * Associates a txnRetryCounter with this operation context. May only be called once for the
+     * lifetime of the operation and the operation must have a logical session id and a transaction
+     * number assigned.
+     */
+    void setTxnRetryCounter(TxnRetryCounter txnRetryCounter);
+
     /**
      * Returns the top-level WriteUnitOfWork associated with this operation context, if any.
      */
@@ -272,6 +313,13 @@ public:
         return _writesAreReplicated;
     }
 
+    /**
+     * Returns true if the operation is running lock-free.
+     */
+    bool isLockFreeReadsOp() const {
+        return _lockFreeReadOpCount;
+    }
+
     /**
      * Returns true if operations' durations should be added to serverStatus latency metrics.
      */
@@ -403,19 +451,56 @@ public:
         return _inMultiDocumentTransaction;
     }
 
+    bool isRetryableWrite() const {
+        return _txnNumber &&
+            (!_inMultiDocumentTransaction || isInternalSessionForRetryableWrite(*_lsid));
+    }
+
     /**
      * Sets that this operation is part of a multi-document transaction. Once this is set, it cannot
      * be unset.
      */
     void setInMultiDocumentTransaction() {
         _inMultiDocumentTransaction = true;
+        if (!_txnRetryCounter.has_value()) {
+            _txnRetryCounter = 0;
+        }
+    }
+
+    /**
+     * Some operations coming into the system must be validated to ensure they meet constraints,
+     * such as collection namespace length limits or unique index key constraints. However,
+     * operations being performed from a source of truth such as during initial sync and oplog
+     * application often must ignore constraint violations.
+     *
+     * Initial sync and oplog application opt in to relaxed constraint checking by setting this
+     * value to false.
+     */
+    void setEnforceConstraints(bool enforceConstraints) {
+        _enforceConstraints = enforceConstraints;
+    }
+
+    /**
+     * This method can be used to tell if an operation requires validation of constraints. This
+     * should be preferred to alternatives such as checking if a node is primary or if a client is
+     * from a user connection as those have nuances (e.g: primary catch up and client disassociation
+     * due to task executors).
+     */
+    bool isEnforcingConstraints() {
+        return _enforceConstraints;
     }
 
     /**
      * Sets that this operation should always get killed during stepDown and stepUp, regardless of
      * whether or not it's taken a write lock.
+     *
+     * Note: This function is NOT synchronized with the ReplicationStateTransitionLock!  This means
+     * that the node's view of it's replication state can change concurrently with this function
+     * running - in which case your operation may _not_ be interrupted by that concurrent
+     * replication state change. If you need to ensure that your node does not change
+     * replication-state while calling this function, take the RSTL. See SERVER-66353 for more info.
      */
-    void setAlwaysInterruptAtStepDownOrUp() {
+    void setAlwaysInterruptAtStepDownOrUp_UNSAFE() {
         _alwaysInterruptAtStepDownOrUp.store(true);
     }
 
@@ -446,6 +531,7 @@ public:
         _isStartingMultiDocumentTransaction = false;
         _lsid = boost::none;
         _txnNumber = boost::none;
+        _txnRetryCounter = boost::none;
     }
 
     /**
@@ -455,6 +541,13 @@ public:
         return _isStartingMultiDocumentTransaction;
     }
 
+    /**
+     * Returns whether this operation is continuing (not starting) a multi-document transaction.
+     */
+    bool isContinuingMultiDocumentTransaction() const {
+        return inMultiDocumentTransaction() && !isStartingMultiDocumentTransaction();
+    }
+
     /**
      * Sets whether this operation is starting a multi-document transaction.
      */
@@ -462,10 +555,17 @@ public:
         _isStartingMultiDocumentTransaction = isStartingMultiDocumentTransaction;
     }
 
+    /**
+     * Sets '_comment'. The client lock must be acquired before calling this method.
+     */
     void setComment(const BSONObj& comment) {
         _comment = comment.getOwned();
     }
 
+    /**
+     * Gets '_comment'. The client lock must be acquired when calling from any thread that does
+     * not own the client associated with the operation.
+     */
     boost::optional<BSONElement> getComment() {
         // The '_comment' object, if present, will only ever have one field.
         return _comment ? boost::optional<BSONElement>(_comment->firstElement()) : boost::none;
@@ -494,6 +594,33 @@ public:
      */
     void restoreMaxTimeMS();
 
+    /**
+     * Returns whether this operation must run in read-only mode.
+     *
+     * If the read-only flag is set on the ServiceContext then:
+     * - Internal operations are allowed to perform writes.
+     * - User originating operations are not allowed to perform writes.
+     */
+    bool readOnly() const {
+        if (!(getClient() && getClient()->isFromUserConnection()))
+            return false;
+        return !getServiceContext()->userWritesAllowed();
+    }
+
+    /**
+     * Sets whether this operation was started by a compressed command.
+     */
+    void setOpCompressed(bool opCompressed) {
+        _opCompressed = opCompressed;
+    }
+
+    /**
+     * Returns whether this operation was started by a compressed command.
+     */
+    bool isOpCompressed() const {
+        return _opCompressed;
+    }
+
 private:
     StatusWith<stdx::cv_status> waitForConditionOrInterruptNoAssertUntil(
         stdx::condition_variable& cv, BasicLockableAdapter m, Date_t deadline) noexcept override;
@@ -540,6 +667,16 @@ private:
         }
     }
 
+    /**
+     * Returns true if ignoring interrupts other than repl state change and no repl state change
+     * has occurred.
+     */
+    bool _noReplStateChangeWhileIgnoringOtherInterrupts() const {
+        return _ignoreInterruptsExceptForReplStateChange &&
+            getKillStatus() != ErrorCodes::InterruptedDueToReplStateChange &&
+            !_killRequestedForReplStateChange.loadRelaxed();
+    }
+
     /**
      * Returns true if this operation has a deadline and it has passed according to the fast clock
      * on ServiceContext.
@@ -570,8 +707,19 @@ private:
         _writesAreReplicated = writesAreReplicated;
     }
 
+    /**
+     * Increment a count to indicate that the operation is running lock-free.
+     */
+    void incrementLockFreeReadOpCount() {
+        ++_lockFreeReadOpCount;
+    }
+    void decrementLockFreeReadOpCount() {
+        --_lockFreeReadOpCount;
+    }
+
     friend class WriteUnitOfWork;
     friend class repl::UnreplicatedWritesBlock;
+    friend class LockFreeReadsBlock;
 
     Client* const _client;
 
@@ -580,6 +728,7 @@ private:
 
     boost::optional<LogicalSessionId> _lsid;
     boost::optional<TxnNumber> _txnNumber;
+    boost::optional<TxnRetryCounter> _txnRetryCounter;
 
     std::unique_ptr<Locker> _locker;
 
@@ -596,6 +745,10 @@ private:
     // once from OK to some kill code.
     AtomicWord<ErrorCodes::Error> _killCode{ErrorCodes::OK};
 
+    // Used to cancel all tokens obtained via getCancellationToken() when this OperationContext is
+    // killed.
+    CancellationSource _cancelSource;
+
     BatonHandle _baton;
 
     WriteConcernOptions _writeConcern;
@@ -603,7 +756,7 @@ private:
     // The timepoint at which this operation exceeds its time limit.
     Date_t _deadline = Date_t::max();
 
-    ErrorCodes::Error _timeoutError = ErrorCodes::ExceededTimeLimit;
+    ErrorCodes::Error _timeoutError = kDefaultOperationContextTimeoutError;
     bool _ignoreInterrupts = false;
     bool _hasArtificialDeadline = false;
     bool _markKillOnClientDisconnect = false;
@@ -629,6 +782,15 @@ private:
     bool _inMultiDocumentTransaction = false;
     bool _isStartingMultiDocumentTransaction = false;
     bool _ignoreInterruptsExceptForReplStateChange = false;
+    // Commands from user applications must run validations and enforce constraints. Operations from
+    // a trusted source, such as initial sync or consuming an oplog entry generated by a primary
+    // typically desire to ignore constraints.
+    bool _enforceConstraints = true;
+
+    // Counts how many lock-free read operations are running nested.
+    // Necessary to use a counter rather than a boolean because there is existing code that
+    // destructs lock helpers out of order.
+    int _lockFreeReadOpCount = 0;
 
     // If true, this OpCtx will get interrupted during replica set stepUp and stepDown, regardless
     // of what locks it's taken.
@@ -642,8 +804,17 @@ private:
 
     // Whether this operation is an exhaust command.
     bool _exhaust = false;
+
+    // Whether this operation was started by a compressed command.
+    bool _opCompressed = false;
 };
 
+// Gets a TimeZoneDatabase pointer from the ServiceContext.
+inline const TimeZoneDatabase* getTimeZoneDatabase(OperationContext* opCtx) {
+    return opCtx && opCtx->getServiceContext() ? TimeZoneDatabase::get(opCtx->getServiceContext())
+                                               : nullptr;
+}
+
 namespace repl {
 /**
  * RAII-style class to turn off replicated writes. Writes do not create oplog entries while the
@@ -668,4 +839,24 @@ private:
     const bool _shouldReplicateWrites;
 };
 }  // namespace repl
+
+/**
+ * RAII-style class to indicate the operation is lock-free and code should behave accordingly.
+ */
+class LockFreeReadsBlock {
+    LockFreeReadsBlock(const LockFreeReadsBlock&) = delete;
+    LockFreeReadsBlock& operator=(const LockFreeReadsBlock&) = delete;
+
+public:
+    LockFreeReadsBlock(OperationContext* opCtx) : _opCtx(opCtx) {
+        _opCtx->incrementLockFreeReadOpCount();
+    }
+
+    ~LockFreeReadsBlock() {
+        _opCtx->decrementLockFreeReadOpCount();
+    }
+
+private:
+    OperationContext* _opCtx;
+};
 }  // namespace mongo
