@@ -36,19 +36,16 @@
 
 namespace mongo {
 
-REGISTER_ACCUMULATOR_WITH_MIN_VERSION(
-    _internalJsReduce,
-    AccumulatorInternalJsReduce::parseInternalJsReduce,
-    ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44);
+REGISTER_ACCUMULATOR(_internalJsReduce, AccumulatorInternalJsReduce::parseInternalJsReduce);
 
 AccumulationExpression AccumulatorInternalJsReduce::parseInternalJsReduce(
-    boost::intrusive_ptr<ExpressionContext> expCtx, BSONElement elem, VariablesParseState vps) {
+    ExpressionContext* const expCtx, BSONElement elem, VariablesParseState vps) {
     uassert(31326,
-            str::stream() << kAccumulatorName << " requires a document argument, but found "
-                          << elem.type(),
+            str::stream() << kName << " requires a document argument, but found " << elem.type(),
             elem.type() == BSONType::Object);
     BSONObj obj = elem.embeddedObject();
 
+    expCtx->sbeGroupCompatible = false;
     std::string funcSource;
     boost::intrusive_ptr<Expression> argument;
 
@@ -59,16 +56,16 @@ AccumulationExpression AccumulatorInternalJsReduce::parseInternalJsReduce(
             argument = Expression::parseOperand(expCtx, element, vps);
         } else {
             uasserted(31243,
-                      str::stream() << "Invalid argument specified to " << kAccumulatorName << ": "
+                      str::stream() << "Invalid argument specified to " << kName << ": "
                                     << element.toString());
         }
     }
     uassert(31245,
-            str::stream() << kAccumulatorName
+            str::stream() << kName
                           << " requires 'eval' argument, recieved input: " << obj.toString(false),
             !funcSource.empty());
     uassert(31349,
-            str::stream() << kAccumulatorName
+            str::stream() << kName
                           << " requires 'data' argument, recieved input: " << obj.toString(false),
             argument);
 
@@ -77,13 +74,16 @@ AccumulationExpression AccumulatorInternalJsReduce::parseInternalJsReduce(
     };
 
     auto initializer = ExpressionConstant::create(expCtx, Value(BSONNULL));
-    return {std::move(initializer), std::move(argument), std::move(factory)};
+    return {std::move(initializer),
+            std::move(argument),
+            std::move(factory),
+            AccumulatorInternalJsReduce::kName};
 }
 
 std::string AccumulatorInternalJsReduce::parseReduceFunction(BSONElement func) {
     uassert(
         31244,
-        str::stream() << kAccumulatorName
+        str::stream() << kName
                       << " requires the 'eval' argument to be of type string, or code but found "
                       << func.type(),
         func.type() == BSONType::String || func.type() == BSONType::Code);
@@ -95,7 +95,7 @@ void AccumulatorInternalJsReduce::processInternal(const Value& input, bool mergi
         return;
     }
     uassert(31242,
-            str::stream() << kAccumulatorName << " requires a document argument, but found "
+            str::stream() << kName << " requires a document argument, but found "
                           << input.getType(),
             input.getType() == BSONType::Object);
     Document data = input.getDocument();
@@ -106,7 +106,7 @@ void AccumulatorInternalJsReduce::processInternal(const Value& input, bool mergi
 
     uassert(
         31251,
-        str::stream() << kAccumulatorName
+        str::stream() << kName
                       << " requires the 'data' argument to have a 'k' and 'v' field. Instead found"
                       << data.toString(),
         data.computeSize() == 2ull && !kField.missing() && !vField.missing());
@@ -120,53 +120,52 @@ void AccumulatorInternalJsReduce::processInternal(const Value& input, bool mergi
 Value AccumulatorInternalJsReduce::getValue(bool toBeMerged) {
     if (_values.size() < 1) {
         return Value{};
-    }
-    Value result;
-    if (mrSingleReduceOptimizationEnabled && _values.size() == 1) {
+    } else if (mrSingleReduceOptimizationEnabled && _values.size() == 1) {
         // This optimization existed in the old Pre-4.4 MapReduce implementation. If the flag is
         // set, then we should replicate the optimization. See SERVER-68766 for more details.
-        result = std::move(_values[0]);
-    } else {
-        const auto keySize = _key.getApproximateSize();
+        return _values[0];
+    }
 
-        // Keep reducing until we have exactly one value.
-        while (true) {
-            BSONArrayBuilder bsonValues;
-            size_t numLeft = _values.size();
-            for (; numLeft > 0; numLeft--) {
-                Value val = _values[numLeft - 1];
+    const auto keySize = _key.getApproximateSize();
 
-                // Do not insert if doing so would exceed the the maximum allowed BSONObj size.
-                if (bsonValues.len() + keySize + val.getApproximateSize() > BSONObjMaxUserSize) {
-                    // If we have reached the threshold for maximum allowed BSONObj size and only
-                    // have a single value then no progress will be made on reduce. We must fail
-                    // when this scenario is encountered.
-                    size_t numNextReduce = _values.size() - numLeft;
-                    uassert(31392, "Value too large to reduce", numNextReduce > 1);
-                    break;
-                }
-                bsonValues << val;
-            }
+    Value result;
+    // Keep reducing until we have exactly one value.
+    while (true) {
+        BSONArrayBuilder bsonValues;
+        size_t numLeft = _values.size();
+        for (; numLeft > 0; numLeft--) {
+            Value val = _values[numLeft - 1];
 
-            auto expCtx = getExpressionContext();
-            auto reduceFunc = makeJsFunc(expCtx, _funcSource);
-
-            // Function signature: reduce(key, values).
-            BSONObj params = BSON_ARRAY(_key << bsonValues.arr());
-            // For reduce, the key and values are both passed as 'params' so there's no need to set
-            // 'this'.
-            BSONObj thisObj;
-            Value reduceResult =
-                expCtx->getJsExecWithScope()->callFunction(reduceFunc, params, thisObj);
-            if (numLeft == 0) {
-                result = reduceResult;
+            // Do not insert if doing so would exceed the the maximum allowed BSONObj size.
+            if (bsonValues.len() + keySize + val.getApproximateSize() > BSONObjMaxUserSize) {
+                // If we have reached the threshold for maximum allowed BSONObj size and only have a
+                // single value then no progress will be made on reduce. We must fail when this
+                // scenario is encountered.
+                size_t numNextReduce = _values.size() - numLeft;
+                uassert(31392, "Value too large to reduce", numNextReduce > 1);
                 break;
-            } else {
-                // Remove all values which have been reduced.
-                _values.resize(numLeft);
-                // Include most recent result in the set of values to be reduced.
-                _values.push_back(reduceResult);
             }
+            bsonValues << val;
+        }
+
+        auto expCtx = getExpressionContext();
+        auto reduceFunc = makeJsFunc(expCtx, _funcSource);
+
+        // Function signature: reduce(key, values).
+        BSONObj params = BSON_ARRAY(_key << bsonValues.arr());
+        // For reduce, the key and values are both passed as 'params' so there's no need to set
+        // 'this'.
+        BSONObj thisObj;
+        Value reduceResult =
+            expCtx->getJsExecWithScope()->callFunction(reduceFunc, params, thisObj);
+        if (numLeft == 0) {
+            result = reduceResult;
+            break;
+        } else {
+            // Remove all values which have been reduced.
+            _values.resize(numLeft);
+            // Include most recent result in the set of values to be reduced.
+            _values.push_back(reduceResult);
         }
     }
 
@@ -174,7 +173,7 @@ Value AccumulatorInternalJsReduce::getValue(bool toBeMerged) {
     if (toBeMerged) {
         MutableDocument output;
         output.addField("k", _key);
-        output.addField("v", result);
+        output.addField("v", std::move(result));
         return Value(output.freeze());
     } else {
         return result;
@@ -182,7 +181,7 @@ Value AccumulatorInternalJsReduce::getValue(bool toBeMerged) {
 }
 
 boost::intrusive_ptr<AccumulatorState> AccumulatorInternalJsReduce::create(
-    const boost::intrusive_ptr<ExpressionContext>& expCtx, StringData funcSource) {
+    ExpressionContext* const expCtx, StringData funcSource) {
 
     return make_intrusive<AccumulatorInternalJsReduce>(expCtx, funcSource);
 }
@@ -197,16 +196,13 @@ void AccumulatorInternalJsReduce::reset() {
 Document AccumulatorInternalJsReduce::serialize(boost::intrusive_ptr<Expression> initializer,
                                                 boost::intrusive_ptr<Expression> argument,
                                                 bool explain) const {
-    return DOC(getOpName() << DOC("data" << argument->serialize(explain) << "eval" << _funcSource));
+    return DOC(kName << DOC("data" << argument->serialize(explain) << "eval" << _funcSource));
 }
 
-REGISTER_ACCUMULATOR_WITH_MIN_VERSION(
-    accumulator,
-    AccumulatorJs::parse,
-    ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44);
+REGISTER_ACCUMULATOR(accumulator, AccumulatorJs::parse);
 
 boost::intrusive_ptr<AccumulatorState> AccumulatorJs::create(
-    const boost::intrusive_ptr<ExpressionContext>& expCtx,
+    ExpressionContext* const expCtx,
     std::string init,
     std::string accumulate,
     std::string merge,
@@ -218,7 +214,7 @@ boost::intrusive_ptr<AccumulatorState> AccumulatorJs::create(
 namespace {
 // Parses a constant expression of type String or Code.
 std::string parseFunction(StringData fieldName,
-                          boost::intrusive_ptr<ExpressionContext> expCtx,
+                          ExpressionContext* const expCtx,
                           BSONElement elem,
                           VariablesParseState vps) {
     boost::intrusive_ptr<Expression> expr = Expression::parseOperand(expCtx, elem, vps);
@@ -249,10 +245,10 @@ Document AccumulatorJs::serialize(boost::intrusive_ptr<Expression> initializer,
         args.addField("finalize", Value(*_finalize));
     }
     args.addField("lang", Value("js"_sd));
-    return DOC(getOpName() << args.freeze());
+    return DOC(kName << args.freeze());
 }
 
-AccumulationExpression AccumulatorJs::parse(boost::intrusive_ptr<ExpressionContext> expCtx,
+AccumulationExpression AccumulatorJs::parse(ExpressionContext* const expCtx,
                                             BSONElement elem,
                                             VariablesParseState vps) {
     /*
@@ -275,6 +271,7 @@ AccumulationExpression AccumulatorJs::parse(boost::intrusive_ptr<ExpressionConte
             elem.type() == BSONType::Object);
     BSONObj obj = elem.embeddedObject();
 
+    expCtx->sbeGroupCompatible = false;
     std::string init, accumulate, merge;
     boost::optional<std::string> finalize;
     boost::intrusive_ptr<Expression> initArgs, accumulateArgs;
@@ -326,7 +323,8 @@ AccumulationExpression AccumulatorJs::parse(boost::intrusive_ptr<ExpressionConte
                     finalize = std::move(finalize)]() {
         return new AccumulatorJs(expCtx, init, accumulate, merge, finalize);
     };
-    return {std::move(initArgs), std::move(accumulateArgs), std::move(factory)};
+    return {
+        std::move(initArgs), std::move(accumulateArgs), std::move(factory), AccumulatorJs::kName};
 }
 
 Value AccumulatorJs::getValue(bool toBeMerged) {
@@ -348,7 +346,7 @@ Value AccumulatorJs::getValue(bool toBeMerged) {
     // Get the final value given the current accumulator state.
 
     if (_finalize) {
-        auto& expCtx = getExpressionContext();
+        auto expCtx = getExpressionContext();
         auto jsExec = expCtx->getJsExecWithScope();
         auto func = makeJsFunc(expCtx, *_finalize);
 
@@ -373,7 +371,7 @@ void AccumulatorJs::startNewGroup(Value const& input) {
     // constructor, and we clear it at the end of each group (in .reset()).
     invariant(!_state);
 
-    auto& expCtx = getExpressionContext();
+    auto expCtx = getExpressionContext();
     auto jsExec = expCtx->getJsExecWithScope();
     auto func = makeJsFunc(expCtx, _init);
 
@@ -432,7 +430,7 @@ void AccumulatorJs::reduceMemoryConsumptionIfAble() {
         return;
     }
 
-    auto& expCtx = getExpressionContext();
+    auto expCtx = getExpressionContext();
     auto jsExec = expCtx->getJsExecWithScope();
 
     // Expose user functions.
