@@ -36,13 +36,12 @@
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/catalog/collection_mock.h"
-#include "mongo/db/catalog/database_holder_mock.h"
-#include "mongo/db/catalog/database_impl.h"
 #include "mongo/db/exec/collection_scan.h"
 #include "mongo/db/exec/document_value/document_value_test_util.h"
 #include "mongo/db/exec/plan_stats.h"
 #include "mongo/db/pipeline/aggregation_context_fixture.h"
-#include "mongo/db/pipeline/document_source_check_resume_token.h"
+#include "mongo/db/pipeline/document_source_change_stream_check_resumability.h"
+#include "mongo/db/pipeline/document_source_change_stream_ensure_resume_token_present.h"
 #include "mongo/db/pipeline/document_source_mock.h"
 #include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/process_interface/stub_mongo_process_interface.h"
@@ -79,12 +78,16 @@ public:
     boost::optional<Record> seekExact(const RecordId& id) override {
         return Record{};
     }
+    boost::optional<Record> seekNear(const RecordId& id) override {
+        return boost::none;
+    }
     void save() override {}
-    bool restore() override {
+    bool restore(bool tolerateCappedRepositioning) override {
         return true;
     }
     void detachFromOperationContext() override {}
     void reattachToOperationContext(OperationContext* opCtx) override {}
+    void setSaveStorageCursorOnDetachFromOperationContext(bool) {}
 
 private:
     std::deque<Record>* _records;
@@ -94,7 +97,7 @@ class ChangeStreamOplogCollectionMock : public CollectionMock {
 public:
     ChangeStreamOplogCollectionMock() : CollectionMock(NamespaceString::kRsOplogNamespace) {
         _recordStore =
-            _devNullEngine.getRecordStore(nullptr, NamespaceString::kRsOplogNamespace.ns(), "", {});
+            _devNullEngine.getRecordStore(nullptr, NamespaceString::kRsOplogNamespace, "", {});
     }
 
     void push_back(Document doc) {
@@ -117,7 +120,7 @@ public:
         _data.push_back(mutableDoc.freeze().toBson());
         Record record;
         record.data = {_data.back().objdata(), _data.back().objsize()};
-        record.id = RecordId{static_cast<int64_t>(_data.size())};
+        record.id = RecordId{doc["ts"].getTimestamp().asLL()};
         _records.push_back(std::move(record));
     }
 
@@ -126,10 +129,7 @@ public:
         return std::make_unique<ChangeStreamOplogCursorMock>(&_records);
     }
 
-    const RecordStore* getRecordStore() const override {
-        return _recordStore.get();
-    }
-    RecordStore* getRecordStore() override {
+    RecordStore* getRecordStore() const override {
         return _recordStore.get();
     }
 
@@ -143,23 +143,6 @@ private:
     DevNullKVEngine _devNullEngine;
 };
 
-/**
- * The RequiresCollectionStageBase class attempts to obtain the current epoch of the database
- * containing the collection to be scanned (in this case, the oplog). Here we provide a dummy
- * DatabaseHolder which will always return a valid pointer to the _database member variable.
- */
-class ChangeStreamDatabaseHolderMock : public DatabaseHolderMock {
-public:
-    ChangeStreamDatabaseHolderMock() : _database(std::make_unique<DatabaseImpl>("local"_sd, 0)){};
-
-    Database* getDb(OperationContext* opCtx, StringData ns) const override {
-        return _database.get();
-    }
-
-private:
-    std::unique_ptr<Database> _database;
-};
-
 /**
  * Acts as an initial source for the change stream pipeline, taking the place of DSOplogMatch. This
  * class maintains its own queue of documents added by each test, but also pushes each doc into an
@@ -180,21 +163,22 @@ private:
  */
 class DocumentSourceChangeStreamMock : public DocumentSourceMock {
 public:
-    DocumentSourceChangeStreamMock(const boost::intrusive_ptr<ExpressionContext>& expCtx)
-        : DocumentSourceMock({}, expCtx) {
+    DocumentSourceChangeStreamMock(const boost::intrusive_ptr<ExpressionContextForTest>& expCtx)
+        : DocumentSourceMock({}, expCtx), _collectionPtr(&_collection) {
         _filterExpr = BSON("ns" << kTestNs);
-        _filter = _parseAndNormalize(_filterExpr);
-        _params.assertMinTsHasNotFallenOffOplog = true;
+        _filter = MatchExpressionParser::parseAndNormalize(_filterExpr, pExpCtx);
+        _params.assertTsHasNotFallenOff = Timestamp(0);
         _params.shouldTrackLatestOplogTimestamp = true;
-        _params.minTs = Timestamp(0, 0);
+        _params.minRecord = RecordIdBound(RecordId(0));
         _params.tailable = true;
     }
 
     void setResumeToken(ResumeTokenData resumeToken) {
         invariant(!_collScan);
         _filterExpr = BSON("ns" << kTestNs << "ts" << BSON("$gte" << resumeToken.clusterTime));
-        _filter = _parseAndNormalize(_filterExpr);
-        _params.minTs = resumeToken.clusterTime;
+        _filter = MatchExpressionParser::parseAndNormalize(_filterExpr, pExpCtx);
+        _params.minRecord = RecordIdBound(RecordId(resumeToken.clusterTime.asLL()));
+        _params.assertTsHasNotFallenOff = resumeToken.clusterTime;
     }
 
     void push_back(GetNextResult&& result) {
@@ -221,7 +205,7 @@ protected:
         // If this is the first call to doGetNext, we must create the COLLSCAN.
         if (!_collScan) {
             _collScan = std::make_unique<CollectionScan>(
-                pExpCtx.get(), &_collection, _params, &_ws, _filter.get());
+                pExpCtx.get(), _collectionPtr, _params, &_ws, _filter.get());
             // The first call to doWork will create the cursor and return NEED_TIME. But it won't
             // actually scan any of the documents that are present in the mock cursor queue.
             ASSERT_EQ(_collScan->doWork(nullptr), PlanStage::NEED_TIME);
@@ -249,7 +233,6 @@ protected:
                 case PlanStage::NEED_TIME:
                     continue;
                 case PlanStage::NEED_YIELD:
-                case PlanStage::FAILURE:
                     MONGO_UNREACHABLE;
             }
         }
@@ -257,18 +240,12 @@ protected:
     }
 
 private:
-    std::unique_ptr<MatchExpression> _parseAndNormalize(BSONObj filterExpr) {
-        auto filter = uassertStatusOK(MatchExpressionParser::parse(filterExpr, pExpCtx));
-        filter = MatchExpression::optimize(std::move(filter));
-        CanonicalQuery::sortTree(filter.get());
-        return filter;
-    }
-
     size_t _getNumDocsTested() {
         return static_cast<const CollectionScanStats*>(_collScan->getSpecificStats())->docsTested;
     }
 
     ChangeStreamOplogCollectionMock _collection;
+    CollectionPtr _collectionPtr;
     std::unique_ptr<CollectionScan> _collScan;
     CollectionScanParams _params;
 
@@ -280,10 +257,7 @@ private:
 
 class CheckResumeTokenTest : public AggregationContextFixture {
 public:
-    CheckResumeTokenTest() : _mock(make_intrusive<DocumentSourceChangeStreamMock>(getExpCtx())) {
-        DatabaseHolder::set(getServiceContext(),
-                            std::make_unique<ChangeStreamDatabaseHolderMock>());
-    }
+    CheckResumeTokenTest() : _mock(make_intrusive<DocumentSourceChangeStreamMock>(getExpCtx())) {}
 
 protected:
     /**
@@ -339,10 +313,13 @@ protected:
     /**
      * Convenience method to create the class under test with a given ResumeTokenData.
      */
-    intrusive_ptr<DocumentSourceEnsureResumeTokenPresent> createDSEnsureResumeTokenPresent(
-        ResumeTokenData tokenData) {
+
+    intrusive_ptr<DocumentSourceChangeStreamEnsureResumeTokenPresent>
+    createDSEnsureResumeTokenPresent(ResumeTokenData tokenData) {
+        DocumentSourceChangeStreamSpec spec;
+        spec.setStartAfter(ResumeToken(tokenData));
         auto checkResumeToken =
-            DocumentSourceEnsureResumeTokenPresent::create(getExpCtx(), tokenData);
+            DocumentSourceChangeStreamEnsureResumeTokenPresent::create(getExpCtx(), spec);
         _mock->setResumeToken(std::move(tokenData));
         checkResumeToken->setSource(_mock.get());
         return checkResumeToken;
@@ -352,12 +329,12 @@ protected:
      * Convenience method to create the class under test with a given timestamp, docKey, and
      * namespace.
      */
-    intrusive_ptr<DocumentSourceEnsureResumeTokenPresent> createDSEnsureResumeTokenPresent(
-        Timestamp ts,
-        int version,
-        std::size_t txnOpIndex,
-        boost::optional<Document> docKey,
-        UUID uuid) {
+    intrusive_ptr<DocumentSourceChangeStreamEnsureResumeTokenPresent>
+    createDSEnsureResumeTokenPresent(Timestamp ts,
+                                     int version,
+                                     std::size_t txnOpIndex,
+                                     boost::optional<Document> docKey,
+                                     UUID uuid) {
         return createDSEnsureResumeTokenPresent(
             {ts, version, txnOpIndex, uuid, docKey ? Value(*docKey) : Value()});
     }
@@ -366,8 +343,10 @@ protected:
      * Convenience method to create the class under test with a given timestamp, docKey, and
      * namespace.
      */
-    intrusive_ptr<DocumentSourceEnsureResumeTokenPresent> createDSEnsureResumeTokenPresent(
-        Timestamp ts, boost::optional<Document> docKey, UUID uuid = testUuid()) {
+    intrusive_ptr<DocumentSourceChangeStreamEnsureResumeTokenPresent>
+    createDSEnsureResumeTokenPresent(Timestamp ts,
+                                     boost::optional<Document> docKey,
+                                     UUID uuid = testUuid()) {
         return createDSEnsureResumeTokenPresent(ts, 0, 0, docKey, uuid);
     }
 
@@ -375,8 +354,8 @@ protected:
      * Convenience method to create the class under test with a given timestamp, _id string, and
      * namespace.
      */
-    intrusive_ptr<DocumentSourceEnsureResumeTokenPresent> createDSEnsureResumeTokenPresent(
-        Timestamp ts, StringData id, UUID uuid = testUuid()) {
+    intrusive_ptr<DocumentSourceChangeStreamEnsureResumeTokenPresent>
+    createDSEnsureResumeTokenPresent(Timestamp ts, StringData id, UUID uuid = testUuid()) {
         return createDSEnsureResumeTokenPresent(ts, 0, 0, Document{{"_id", id}}, uuid);
     }
 
@@ -394,15 +373,21 @@ protected:
 
 class CheckResumabilityTest : public CheckResumeTokenTest {
 protected:
-    intrusive_ptr<DocumentSourceCheckResumability> createDSCheckResumability(
+    intrusive_ptr<DocumentSourceChangeStreamCheckResumability> createDSCheckResumability(
         ResumeTokenData tokenData) {
-        auto dsCheckResumability = DocumentSourceCheckResumability::create(getExpCtx(), tokenData);
+        DocumentSourceChangeStreamSpec spec;
+        spec.setStartAfter(ResumeToken(tokenData));
+        auto dsCheckResumability =
+            DocumentSourceChangeStreamCheckResumability::create(getExpCtx(), spec);
         _mock->setResumeToken(std::move(tokenData));
         dsCheckResumability->setSource(_mock.get());
         return dsCheckResumability;
     }
-    intrusive_ptr<DocumentSourceCheckResumability> createDSCheckResumability(Timestamp ts) {
-        return createDSCheckResumability(ResumeToken::makeHighWaterMarkToken(ts).getData());
+    intrusive_ptr<DocumentSourceChangeStreamCheckResumability> createDSCheckResumability(
+        Timestamp ts) {
+        return createDSCheckResumability(
+            ResumeToken::makeHighWaterMarkToken(ts, ResumeTokenData::kDefaultTokenVersion)
+                .getData());
     }
 };
 
@@ -517,32 +502,6 @@ TEST_F(CheckResumeTokenTest, ShouldSucceedWithBinaryCollation) {
     ASSERT_TRUE(checkResumeToken->getNext().isEOF());
 }
 
-TEST_F(CheckResumeTokenTest, UnshardedTokenSucceedsForShardedResumeOnMongosIfIdMatchesFirstDoc) {
-    // Verify that a resume token whose documentKey only contains _id can be used to resume a stream
-    // on a sharded collection as long as its _id matches the first document. We set 'inMongos'
-    // since this behaviour is only applicable when DocumentSourceEnsureResumeTokenPresent is
-    // running on mongoS.
-    Timestamp resumeTimestamp(100, 1);
-    getExpCtx()->inMongos = true;
-
-    auto checkResumeToken =
-        createDSEnsureResumeTokenPresent(resumeTimestamp, Document{{"_id"_sd, 1}});
-
-    Timestamp doc1Timestamp(100, 1);
-    addOplogEntryOnTestNS(doc1Timestamp, {{"x"_sd, 0}, {"_id"_sd, 1}});
-    Timestamp doc2Timestamp(100, 2);
-    Document doc2DocKey{{"x"_sd, 0}, {"_id"_sd, 2}};
-    addOplogEntryOnTestNS(doc2Timestamp, doc2DocKey);
-
-    // We should skip doc1 since it satisfies the resume token, and retrieve doc2.
-    const auto firstDocAfterResume = checkResumeToken->getNext();
-    const auto tokenFromFirstDocAfterResume =
-        ResumeToken::parse(firstDocAfterResume.getDocument()["_id"].getDocument()).getData();
-
-    ASSERT_EQ(tokenFromFirstDocAfterResume.clusterTime, doc2Timestamp);
-    ASSERT_DOCUMENT_EQ(tokenFromFirstDocAfterResume.documentKey.getDocument(), doc2DocKey);
-}
-
 TEST_F(CheckResumeTokenTest, UnshardedTokenFailsForShardedResumeOnMongosIfIdDoesNotMatchFirstDoc) {
     Timestamp resumeTimestamp(100, 1);
     getExpCtx()->inMongos = true;
@@ -560,8 +519,8 @@ TEST_F(CheckResumeTokenTest, UnshardedTokenFailsForShardedResumeOnMongosIfIdDoes
 TEST_F(CheckResumeTokenTest, ShardedResumeFailsOnMongosIfTokenHasSubsetOfDocumentKeyFields) {
     // Verify that the relaxed _id check only applies if _id is the sole field present in the
     // client's resume token, even if all the fields that are present match the first doc. We set
-    // 'inMongos' since this is only applicable when DocumentSourceEnsureResumeTokenPresent is
-    // running on mongoS.
+    // 'inMongos' since this is only applicable when DSCSEnsureResumeTokenPresent is running on
+    // mongoS.
     Timestamp resumeTimestamp(100, 1);
     getExpCtx()->inMongos = true;
 
@@ -577,7 +536,7 @@ TEST_F(CheckResumeTokenTest, ShardedResumeFailsOnMongosIfTokenHasSubsetOfDocumen
 
 TEST_F(CheckResumeTokenTest, ShardedResumeFailsOnMongosIfDocumentKeyIsNonObject) {
     // Verify that a resume token whose documentKey is not a valid object will neither succeed nor
-    // cause an invariant when we perform the relaxed documentKey._id check when running in a
+    // cause an invariant when we perform the relaxed eventIdentifier._id check when running in a
     // sharded context.
     Timestamp resumeTimestamp(100, 1);
     getExpCtx()->inMongos = true;
@@ -593,8 +552,8 @@ TEST_F(CheckResumeTokenTest, ShardedResumeFailsOnMongosIfDocumentKeyIsNonObject)
 
 TEST_F(CheckResumeTokenTest, ShardedResumeFailsOnMongosIfDocumentKeyOmitsId) {
     // Verify that a resume token whose documentKey omits the _id field will neither succeed nor
-    // cause an invariant when we perform the relaxed documentKey._id, even when compared against an
-    // artificial stream token whose _id is also missing.
+    // cause an invariant when we perform the relaxed eventIdentifier._id, even when compared
+    // against an artificial stream token whose _id is also missing.
     Timestamp resumeTimestamp(100, 1);
     getExpCtx()->inMongos = true;
 
@@ -649,7 +608,7 @@ TEST_F(CheckResumeTokenTest,
         ResumeToken::parse(firstDocAfterResume.getDocument()["_id"].getDocument()).getData();
 
     ASSERT_EQ(tokenFromFirstDocAfterResume.clusterTime, resumeTimestamp);
-    ASSERT_DOCUMENT_EQ(tokenFromFirstDocAfterResume.documentKey.getDocument(), expectedDocKey);
+    ASSERT_DOCUMENT_EQ(tokenFromFirstDocAfterResume.eventIdentifier.getDocument(), expectedDocKey);
 }
 
 TEST_F(CheckResumeTokenTest,
@@ -692,10 +651,13 @@ TEST_F(CheckResumeTokenTest, ShouldSwallowInvalidateFromEachShardForStartAfterIn
     // Create a resume token representing an 'invalidate' event, and use it to seed the stage. A
     // resume token with {fromInvalidate:true} can only be used with startAfter, to start a new
     // stream after the old stream is invalidated.
-    ResumeTokenData invalidateToken;
-    invalidateToken.clusterTime = resumeTimestamp;
-    invalidateToken.uuid = uuids[0];
-    invalidateToken.fromInvalidate = ResumeTokenData::kFromInvalidate;
+    auto eventIdentifier = Value{Document{{"operationType", "drop"_sd}}};
+    ResumeTokenData invalidateToken{resumeTimestamp,
+                                    ResumeTokenData::kDefaultTokenVersion,
+                                    /* txnOpIndex */ 0,
+                                    uuids[0],
+                                    std::move(eventIdentifier),
+                                    ResumeTokenData::kFromInvalidate};
     auto checkResumeToken = createDSEnsureResumeTokenPresent(invalidateToken);
 
     // Add three documents which each have the invalidate resume token. We expect to see this in the
@@ -716,7 +678,7 @@ TEST_F(CheckResumeTokenTest, ShouldSwallowInvalidateFromEachShardForStartAfterIn
         ResumeToken::parse(firstDocAfterResume.getDocument()["_id"].getDocument()).getData();
 
     ASSERT_EQ(tokenFromFirstDocAfterResume.clusterTime, firstEventAfter);
-    ASSERT_DOCUMENT_EQ(tokenFromFirstDocAfterResume.documentKey.getDocument(), expectedDocKey);
+    ASSERT_DOCUMENT_EQ(tokenFromFirstDocAfterResume.eventIdentifier.getDocument(), expectedDocKey);
 }
 
 TEST_F(CheckResumeTokenTest, ShouldNotSwallowUnrelatedInvalidateForStartAfterInvalidate) {
@@ -732,10 +694,13 @@ TEST_F(CheckResumeTokenTest, ShouldNotSwallowUnrelatedInvalidateForStartAfterInv
     // Create a resume token representing an 'invalidate' event, and use it to seed the stage. A
     // resume token with {fromInvalidate:true} can only be used with startAfter, to start a new
     // stream after the old stream is invalidated.
-    ResumeTokenData invalidateToken;
-    invalidateToken.clusterTime = resumeTimestamp;
-    invalidateToken.uuid = uuids[0];
-    invalidateToken.fromInvalidate = ResumeTokenData::kFromInvalidate;
+    auto eventIdentifier = Value{Document{{"operationType", "drop"_sd}}};
+    ResumeTokenData invalidateToken{resumeTimestamp,
+                                    ResumeTokenData::kDefaultTokenVersion,
+                                    /* txnOpIndex */ 0,
+                                    uuids[0],
+                                    eventIdentifier,
+                                    ResumeTokenData::kFromInvalidate};
     auto checkResumeToken = createDSEnsureResumeTokenPresent(invalidateToken);
 
     // Create a second invalidate token with the same clusterTime but a different UUID.
@@ -792,7 +757,7 @@ TEST_F(CheckResumeTokenTest, ShouldSkipResumeTokensWithEarlierTxnOpIndex) {
         ResumeToken::parse(firstDocAfterResume.getDocument()["_id"].getDocument()).getData();
 
     ASSERT_EQ(tokenFromFirstDocAfterResume.clusterTime, resumeTimestamp);
-    ASSERT_DOCUMENT_EQ(tokenFromFirstDocAfterResume.documentKey.getDocument(), expectedDocKey);
+    ASSERT_DOCUMENT_EQ(tokenFromFirstDocAfterResume.eventIdentifier.getDocument(), expectedDocKey);
 }
 
 /**
@@ -1003,7 +968,7 @@ TEST_F(CheckResumabilityTest, ShouldIgnoreOplogAfterFirstEOF) {
 TEST_F(CheckResumabilityTest, ShouldSwallowAllEventsAtSameClusterTimeUpToResumeToken) {
     Timestamp resumeTimestamp(100, 2);
 
-    // Set up the DocumentSourceCheckResumability to check for an exact event ResumeToken.
+    // Set up the DSCSCheckResumability to check for an exact event ResumeToken.
     ResumeTokenData token(resumeTimestamp, 0, 0, testUuid(), Value(Document{{"_id"_sd, "3"_sd}}));
     auto dsCheckResumability = createDSCheckResumability(token);
 
@@ -1032,7 +997,7 @@ TEST_F(CheckResumabilityTest, ShouldSwallowAllEventsAtSameClusterTimeUpToResumeT
 TEST_F(CheckResumabilityTest, ShouldSwallowAllEventsAtSameClusterTimePriorToResumeToken) {
     Timestamp resumeTimestamp(100, 2);
 
-    // Set up the DocumentSourceCheckResumability to check for an exact event ResumeToken.
+    // Set up the DSCSCheckResumability to check for an exact event ResumeToken.
     ResumeTokenData token(resumeTimestamp, 0, 0, testUuid(), Value(Document{{"_id"_sd, "3"_sd}}));
     auto dsCheckResumability = createDSCheckResumability(token);
 
