@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/platform/basic.h"
 
@@ -41,6 +40,9 @@
 #include "mongo/db/storage/key_string.h"
 #include "mongo/logv2/log.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
+
+
 namespace mongo {
 
 MONGO_FAIL_POINT_DEFINE(exchangeFailLoadNextBatch);
@@ -394,7 +396,7 @@ size_t Exchange::loadNextBatch() {
 
     // We have reached the end so send EOS to all consumers.
     for (auto& c : _consumers) {
-        c->appendDocument(input, _maxBufferSize);
+        [[maybe_unused]] auto full = c->appendDocument(input, _maxBufferSize);
     }
 
     return kInvalidThreadId;
