@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/platform/basic.h"
 
@@ -47,6 +46,8 @@
 #include "mongo/util/system_clock_source.h"
 #include "mongo/util/time_support.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
 
 namespace mongo {
 
@@ -110,7 +111,7 @@ protected:
     }
 
     auto getMockSource(int cnt) {
-        auto source = DocumentSourceMock::createForTest();
+        auto source = DocumentSourceMock::createForTest(getExpCtx());
 
         for (int i = 0; i < cnt; ++i)
             source->emplace_back(Document{{"a", i}, {"b", "aaaaaaaaaaaaaaaaaaaaaaaaaaa"_sd}});
@@ -128,7 +129,7 @@ protected:
     auto getRandomMockSource(size_t cnt, int64_t seed) {
         PseudoRandom prng(seed);
 
-        auto source = DocumentSourceMock::createForTest();
+        auto source = DocumentSourceMock::createForTest(getExpCtx());
 
         for (size_t i = 0; i < cnt; ++i)
             source->emplace_back(Document{{"a", static_cast<int>(prng.nextInt32() % cnt)},
@@ -138,7 +139,7 @@ protected:
     }
 
     auto parseSpec(const BSONObj& spec) {
-        IDLParserErrorContext ctx("internalExchange");
+        IDLParserContext ctx("internalExchange");
         return ExchangeSpec::parse(ctx, spec);
     }
 
@@ -171,12 +172,16 @@ TEST_F(DocumentSourceExchangeTest, SimpleExchange1Consumer) {
     spec.setConsumers(1);
     spec.setBufferSize(1024);
 
+    // Upon creation, the Exchange object detaches the pipeline from the operation context, and, as
+    // a result, the optCtx on the ExpressionContext is reset to nullptr. So, we need to preserve
+    // the opCtx in order to pass it to the getNext call below, which will re-attach the pipeline to
+    // the provided opCtx.
+    auto opCtx = getOpCtx();
     boost::intrusive_ptr<Exchange> ex = new Exchange(spec, Pipeline::create({source}, getExpCtx()));
-
-    auto input = ex->getNext(getExpCtx()->opCtx, 0, nullptr);
+    auto input = ex->getNext(opCtx, 0, nullptr);
 
     size_t docs = 0;
-    for (; input.isAdvanced(); input = ex->getNext(getExpCtx()->opCtx, 0, nullptr)) {
+    for (; input.isAdvanced(); input = ex->getNext(opCtx, 0, nullptr)) {
         ++docs;
     }
 
