@@ -39,7 +39,9 @@
 #include "mongo/db/pipeline/document_source_group.h"
 #include "mongo/db/pipeline/expression.h"
 #include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/expression_dependencies.h"
 #include "mongo/db/pipeline/lite_parsed_document_source.h"
+#include "mongo/db/stats/resource_consumption_metrics.h"
 #include "mongo/util/destructor_guard.h"
 
 namespace mongo {
@@ -72,7 +74,7 @@ Document GroupFromFirstDocumentTransformation::applyTransformation(const Documen
 
     for (auto&& expr : _accumulatorExprs) {
         auto value = expr.second->evaluate(input, &expr.second->getExpressionContext()->variables);
-        output.addField(expr.first, value.missing() ? Value(BSONNULL) : value);
+        output.addField(expr.first, value.missing() ? Value(BSONNULL) : std::move(value));
     }
 
     return output.freeze();
@@ -97,7 +99,7 @@ Document GroupFromFirstDocumentTransformation::serializeTransformation(
 
 DepsTracker::State GroupFromFirstDocumentTransformation::addDependencies(DepsTracker* deps) const {
     for (auto&& expr : _accumulatorExprs) {
-        expr.second->addDependencies(deps);
+        expression::addDependencies(expr.second.get(), deps);
     }
 
     // This stage will replace the entire document with a new document, so any existing fields
@@ -107,9 +109,15 @@ DepsTracker::State GroupFromFirstDocumentTransformation::addDependencies(DepsTra
     return DepsTracker::State::EXHAUSTIVE_ALL;
 }
 
+void GroupFromFirstDocumentTransformation::addVariableRefs(std::set<Variables::Id>* refs) const {
+    for (auto&& expr : _accumulatorExprs) {
+        expression::addVariableRefs(expr.second.get(), refs);
+    }
+}
+
 DocumentSource::GetModPathsReturn GroupFromFirstDocumentTransformation::getModifiedPaths() const {
     // Replaces the entire root, so all paths are modified.
-    return {DocumentSource::GetModPathsReturn::Type::kAllPaths, std::set<std::string>{}, {}};
+    return {DocumentSource::GetModPathsReturn::Type::kAllPaths, OrderedPathSet{}, {}};
 }
 
 std::unique_ptr<GroupFromFirstDocumentTransformation> GroupFromFirstDocumentTransformation::create(
@@ -124,42 +132,46 @@ constexpr StringData DocumentSourceGroup::kStageName;
 
 REGISTER_DOCUMENT_SOURCE(group,
                          LiteParsedDocumentSourceDefault::parse,
-                         DocumentSourceGroup::createFromBson);
+                         DocumentSourceGroup::createFromBson,
+                         AllowedWithApiStrict::kAlways);
 
 const char* DocumentSourceGroup::getSourceName() const {
     return kStageName.rawData();
 }
 
-bool DocumentSourceGroup::MemoryUsageTracker::shouldSpillWithAttemptToSaveMemory(
-    std::function<int()> saveMemory) {
-    if (!allowDiskUse && (memoryUsageBytes > maxMemoryUsageBytes)) {
-        memoryUsageBytes -= saveMemory();
+bool DocumentSourceGroup::shouldSpillWithAttemptToSaveMemory() {
+    if (!_memoryTracker._allowDiskUse &&
+        (_memoryTracker.currentMemoryBytes() >
+         static_cast<long long>(_memoryTracker._maxAllowedMemoryUsageBytes))) {
+        freeMemory();
     }
 
-    if (memoryUsageBytes > maxMemoryUsageBytes) {
+    if (_memoryTracker.currentMemoryBytes() >
+        static_cast<long long>(_memoryTracker._maxAllowedMemoryUsageBytes)) {
         uassert(ErrorCodes::QueryExceededMemoryLimitNoDiskUseAllowed,
                 "Exceeded memory limit for $group, but didn't allow external sort."
                 " Pass allowDiskUse:true to opt in.",
-                allowDiskUse);
-        memoryUsageBytes = 0;
+                _memoryTracker._allowDiskUse);
+        _memoryTracker.resetCurrent();
         return true;
     }
     return false;
 }
 
-int DocumentSourceGroup::freeMemory() {
+void DocumentSourceGroup::freeMemory() {
     invariant(_groups);
-    int totalMemorySaved = 0;
     for (auto&& group : *_groups) {
-        for (auto&& groupObj : group.second) {
-            auto prevMemUsage = groupObj->memUsageForSorter();
-            groupObj->reduceMemoryConsumptionIfAble();
+        for (size_t i = 0; i < group.second.size(); i++) {
+            // Subtract the current usage.
+            _memoryTracker.update(_accumulatedFields[i].fieldName,
+                                  -1 * group.second[i]->getMemUsage());
 
-            // Update the memory usage for this group.
-            totalMemorySaved += (prevMemUsage - groupObj->memUsageForSorter());
+            group.second[i]->reduceMemoryConsumptionIfAble();
+
+            // Update the memory usage for this AccumulationStatement.
+            _memoryTracker.update(_accumulatedFields[i].fieldName, group.second[i]->getMemUsage());
         }
     }
-    return totalMemorySaved;
 }
 
 DocumentSource::GetNextResult DocumentSourceGroup::doGetNext() {
@@ -206,6 +218,7 @@ DocumentSource::GetNextResult DocumentSourceGroup::getNextSpilled() {
         switch (numAccumulators) {  // mirrors switch in spill()
             case 1:                 // Single accumulators serialize as a single Value.
                 _currentAccumulators[0]->process(_firstPartOfNextGroup.second, true);
+                [[fallthrough]];
             case 0:  // No accumulators so no Values.
                 break;
             default: {  // Multiple accumulators serialize as an array of Values.
@@ -237,7 +250,7 @@ DocumentSource::GetNextResult DocumentSourceGroup::getNextStandard() {
     if (++groupsIterator == _groups->end())
         dispose();
 
-    return std::move(out);
+    return out;
 }
 
 void DocumentSourceGroup::doDispose() {
@@ -250,6 +263,18 @@ void DocumentSourceGroup::doDispose() {
 }
 
 intrusive_ptr<DocumentSource> DocumentSourceGroup::optimize() {
+    // Optimizing a 'DocumentSourceGroup' might modify its expressions to become incompatible with
+    // SBE. We temporarily highjack the context's 'sbeCompatible' flag to communicate the situation
+    // back to the 'DocumentSourceGroup'. Notice, that while a particular 'DocumentSourceGroup'
+    // might become incompatible with SBE, other groups in the pipeline and the collection access
+    // could be still eligible for lowering to SBE, thus we must reset the context's 'sbeCompatible'
+    // flag back to its original value at the end of the 'optimize()' call.
+    //
+    // TODO SERVER-XXXXX: replace this hack with a proper per-stage tracking of SBE compatibility.
+    auto expCtx = _idExpressions[0]->getExpressionContext();
+    auto orgSbeCompatible = expCtx->sbeCompatible;
+    expCtx->sbeCompatible = true;
+
     // TODO: If all _idExpressions are ExpressionConstants after optimization, then we know there
     // will be only one group. We should take advantage of that to avoid going through the hash
     // table.
@@ -262,6 +287,9 @@ intrusive_ptr<DocumentSource> DocumentSourceGroup::optimize() {
         accumulatedField.expr.argument = accumulatedField.expr.argument->optimize();
     }
 
+    _sbeCompatible = _sbeCompatible && expCtx->sbeCompatible;
+    expCtx->sbeCompatible = orgSbeCompatible;
+
     return this;
 }
 
@@ -297,24 +325,52 @@ Value DocumentSourceGroup::serialize(boost::optional<ExplainOptions::Verbosity>
         insides["$doingMerge"] = Value(true);
     }
 
-    return Value(DOC(getSourceName() << insides.freeze()));
+    MutableDocument out;
+    out[getSourceName()] = Value(insides.freeze());
+
+    if (explain && *explain >= ExplainOptions::Verbosity::kExecStats) {
+        MutableDocument md;
+
+        for (size_t i = 0; i < _accumulatedFields.size(); i++) {
+            md[_accumulatedFields[i].fieldName] = Value(static_cast<long long>(
+                _memoryTracker[_accumulatedFields[i].fieldName].maxMemoryBytes()));
+        }
+
+        out["maxAccumulatorMemoryUsageBytes"] = Value(md.freezeToValue());
+        out["totalOutputDataSizeBytes"] =
+            Value(static_cast<long long>(_stats.totalOutputDataSizeBytes));
+        out["usedDisk"] = Value(_stats.spills > 0);
+        out["spills"] = Value(static_cast<long long>(_stats.spills));
+    }
+
+    return Value(out.freezeToValue());
 }
 
 DepsTracker::State DocumentSourceGroup::getDependencies(DepsTracker* deps) const {
     // add the _id
     for (size_t i = 0; i < _idExpressions.size(); i++) {
-        _idExpressions[i]->addDependencies(deps);
+        expression::addDependencies(_idExpressions[i].get(), deps);
     }
 
     // add the rest
     for (auto&& accumulatedField : _accumulatedFields) {
-        accumulatedField.expr.argument->addDependencies(deps);
+        expression::addDependencies(accumulatedField.expr.argument.get(), deps);
         // Don't add initializer, because it doesn't refer to docs from the input stream.
     }
 
     return DepsTracker::State::EXHAUSTIVE_ALL;
 }
 
+void DocumentSourceGroup::addVariableRefs(std::set<Variables::Id>* refs) const {
+    for (const auto& idExpr : _idExpressions) {
+        expression::addVariableRefs(idExpr.get(), refs);
+    }
+
+    for (auto&& accumulatedField : _accumulatedFields) {
+        expression::addVariableRefs(accumulatedField.expr.argument.get(), refs);
+    }
+}
+
 DocumentSource::GetModPathsReturn DocumentSourceGroup::getModifiedPaths() const {
     // We preserve none of the fields, but any fields referenced as part of the group key are
     // logically just renamed.
@@ -330,7 +386,7 @@ DocumentSource::GetModPathsReturn DocumentSourceGroup::getModifiedPaths() const
     }
 
     return {DocumentSource::GetModPathsReturn::Type::kAllExcept,
-            std::set<std::string>{},  // No fields are preserved.
+            OrderedPathSet{},  // No fields are preserved.
             std::move(renames)};
 }
 
@@ -353,37 +409,33 @@ const std::vector<AccumulationStatement>& DocumentSourceGroup::getAccumulatedFie
 }
 
 intrusive_ptr<DocumentSourceGroup> DocumentSourceGroup::create(
-    const intrusive_ptr<ExpressionContext>& pExpCtx,
+    const intrusive_ptr<ExpressionContext>& expCtx,
     const boost::intrusive_ptr<Expression>& groupByExpression,
     std::vector<AccumulationStatement> accumulationStatements,
     boost::optional<size_t> maxMemoryUsageBytes) {
-    size_t memoryBytes = maxMemoryUsageBytes ? *maxMemoryUsageBytes
-                                             : internalDocumentSourceGroupMaxMemoryBytes.load();
-    intrusive_ptr<DocumentSourceGroup> groupStage(new DocumentSourceGroup(pExpCtx, memoryBytes));
+    intrusive_ptr<DocumentSourceGroup> groupStage(
+        new DocumentSourceGroup(expCtx, maxMemoryUsageBytes));
     groupStage->setIdExpression(groupByExpression);
     for (auto&& statement : accumulationStatements) {
         groupStage->addAccumulator(statement);
+        groupStage->_memoryTracker.set(statement.fieldName, 0);
     }
 
     return groupStage;
 }
 
-DocumentSourceGroup::DocumentSourceGroup(const intrusive_ptr<ExpressionContext>& pExpCtx,
+DocumentSourceGroup::DocumentSourceGroup(const intrusive_ptr<ExpressionContext>& expCtx,
                                          boost::optional<size_t> maxMemoryUsageBytes)
-    : DocumentSource(kStageName, pExpCtx),
-      _usedDisk(false),
+    : DocumentSource(kStageName, expCtx),
       _doingMerge(false),
-      _memoryTracker{pExpCtx->allowDiskUse && !pExpCtx->inMongos,
-                     maxMemoryUsageBytes ? *maxMemoryUsageBytes
-                                         : internalDocumentSourceGroupMaxMemoryBytes.load()},
-      // We spill to disk in debug mode, regardless of allowDiskUse, to stress the system.
-      _file(!pExpCtx->inMongos && (pExpCtx->allowDiskUse || kDebugBuild)
-                ? std::make_shared<Sorter<Value, Value>::File>(pExpCtx->tempDir + "/" +
-                                                               nextFileName())
-                : nullptr),
+      _memoryTracker{expCtx->allowDiskUse && !expCtx->inMongos,
+                     maxMemoryUsageBytes
+                         ? *maxMemoryUsageBytes
+                         : static_cast<size_t>(internalDocumentSourceGroupMaxMemoryBytes.load())},
       _initialized(false),
-      _groups(pExpCtx->getValueComparator().makeUnorderedValueMap<Accumulators>()),
-      _spilled(false) {}
+      _groups(expCtx->getValueComparator().makeUnorderedValueMap<Accumulators>()),
+      _spilled(false),
+      _sbeCompatible(false) {}
 
 void DocumentSourceGroup::addAccumulator(AccumulationStatement accumulationStatement) {
     _accumulatedFields.push_back(accumulationStatement);
@@ -397,30 +449,29 @@ intrusive_ptr<Expression> parseIdExpression(const intrusive_ptr<ExpressionContex
     if (groupField.type() == Object) {
         // {_id: {}} is treated as grouping on a constant, not an expression
         if (groupField.Obj().isEmpty()) {
-            return ExpressionConstant::create(expCtx, Value(groupField));
+            return ExpressionConstant::create(expCtx.get(), Value(groupField));
         }
 
         const BSONObj idKeyObj = groupField.Obj();
         if (idKeyObj.firstElementFieldName()[0] == '$') {
             // grouping on a $op expression
-            return Expression::parseObject(expCtx, idKeyObj, vps);
+            return Expression::parseObject(expCtx.get(), idKeyObj, vps);
         } else {
             for (auto&& field : idKeyObj) {
                 uassert(17390,
                         "$group does not support inclusion-style expressions",
                         !field.isNumber() && field.type() != Bool);
             }
-            return ExpressionObject::parse(expCtx, idKeyObj, vps);
+            return ExpressionObject::parse(expCtx.get(), idKeyObj, vps);
         }
     } else {
-        return Expression::parseOperand(expCtx, groupField, vps);
+        return Expression::parseOperand(expCtx.get(), groupField, vps);
     }
 }
 
 }  // namespace
 
 void DocumentSourceGroup::setIdExpression(const boost::intrusive_ptr<Expression> idExpression) {
-
     if (auto object = dynamic_cast<ExpressionObject*>(idExpression.get())) {
         auto& childExpressions = object->getChildExpressions();
         invariant(!childExpressions.empty());  // We expect to have converted an empty object into a
@@ -438,36 +489,63 @@ void DocumentSourceGroup::setIdExpression(const boost::intrusive_ptr<Expression>
     }
 }
 
+boost::intrusive_ptr<Expression> DocumentSourceGroup::getIdExpression() const {
+    // _idFieldNames is empty and _idExpressions has one element when the _id expression is not an
+    // object expression.
+    if (_idFieldNames.empty() && _idExpressions.size() == 1) {
+        return _idExpressions[0];
+    }
+
+    tassert(6586300,
+            "Field and its expression must be always paired in ExpressionObject",
+            _idFieldNames.size() > 0 && _idFieldNames.size() == _idExpressions.size());
+
+    // Each expression in '_idExpressions' may have been optimized and so, compose the object _id
+    // expression out of the optimized expressions.
+    std::vector<std::pair<std::string, boost::intrusive_ptr<Expression>>> fieldsAndExprs;
+    for (size_t i = 0; i < _idExpressions.size(); ++i) {
+        fieldsAndExprs.emplace_back(_idFieldNames[i], _idExpressions[i]);
+    }
+
+    return ExpressionObject::create(_idExpressions[0]->getExpressionContext(),
+                                    std::move(fieldsAndExprs));
+}
+
 intrusive_ptr<DocumentSource> DocumentSourceGroup::createFromBson(
-    BSONElement elem, const intrusive_ptr<ExpressionContext>& pExpCtx) {
+    BSONElement elem, const intrusive_ptr<ExpressionContext>& expCtx) {
     uassert(15947, "a group's fields must be specified in an object", elem.type() == Object);
 
-    intrusive_ptr<DocumentSourceGroup> pGroup(new DocumentSourceGroup(pExpCtx));
+    intrusive_ptr<DocumentSourceGroup> groupStage(new DocumentSourceGroup(expCtx));
 
     BSONObj groupObj(elem.Obj());
     BSONObjIterator groupIterator(groupObj);
-    VariablesParseState vps = pExpCtx->variablesParseState;
+    VariablesParseState vps = expCtx->variablesParseState;
+    expCtx->sbeGroupCompatible = true;
     while (groupIterator.more()) {
         BSONElement groupField(groupIterator.next());
         StringData pFieldName = groupField.fieldNameStringData();
         if (pFieldName == "_id") {
-            uassert(
-                15948, "a group's _id may only be specified once", pGroup->_idExpressions.empty());
-            pGroup->setIdExpression(parseIdExpression(pExpCtx, groupField, vps));
-            invariant(!pGroup->_idExpressions.empty());
+            uassert(15948,
+                    "a group's _id may only be specified once",
+                    groupStage->_idExpressions.empty());
+            groupStage->setIdExpression(parseIdExpression(expCtx, groupField, vps));
+            invariant(!groupStage->_idExpressions.empty());
         } else if (pFieldName == "$doingMerge") {
             massert(17030, "$doingMerge should be true if present", groupField.Bool());
 
-            pGroup->setDoingMerge(true);
+            groupStage->setDoingMerge(true);
         } else {
             // Any other field will be treated as an accumulator specification.
-            pGroup->addAccumulator(
-                AccumulationStatement::parseAccumulationStatement(pExpCtx, groupField, vps));
+            groupStage->addAccumulator(
+                AccumulationStatement::parseAccumulationStatement(expCtx.get(), groupField, vps));
+            groupStage->_memoryTracker.set(pFieldName, 0);
         }
     }
+    groupStage->_sbeCompatible = expCtx->sbeGroupCompatible && expCtx->sbeCompatible;
 
-    uassert(15955, "a group specification must include an _id", !pGroup->_idExpressions.empty());
-    return pGroup;
+    uassert(
+        15955, "a group specification must include an _id", !groupStage->_idExpressions.empty());
+    return groupStage;
 }
 
 namespace {
@@ -513,7 +591,7 @@ MONGO_COMPILER_NOINLINE DocumentSource::GetNextResult DocumentSourceGroup::initi
     const size_t numAccumulators = _accumulatedFields.size();
     // Barring any pausing, this loop exhausts 'pSource' and populates '_groups'.
     for (; input.isAdvanced(); input = pSource->getNext()) {
-        if (_memoryTracker.shouldSpillWithAttemptToSaveMemory([this]() { return freeMemory(); })) {
+        if (shouldSpillWithAttemptToSaveMemory()) {
             _sortedFiles.push_back(spill());
         }
 
@@ -529,8 +607,9 @@ MONGO_COMPILER_NOINLINE DocumentSource::GetNextResult DocumentSourceGroup::initi
         vector<intrusive_ptr<AccumulatorState>>& group = (*_groups)[id];
         const bool inserted = _groups->size() != oldSize;
 
+        vector<uint64_t> oldAccumMemUsage(numAccumulators, 0);
         if (inserted) {
-            _memoryTracker.memoryUsageBytes += id.getApproximateSize();
+            _memoryTracker.set(_memoryTracker.currentMemoryBytes() + id.getApproximateSize());
 
             // Initialize and add the accumulators
             Value expandedId = expandId(id);
@@ -544,30 +623,31 @@ MONGO_COMPILER_NOINLINE DocumentSource::GetNextResult DocumentSourceGroup::initi
                 accum->startNewGroup(initializerValue);
                 group.push_back(accum);
             }
-        } else {
-            for (auto&& groupObj : group) {
-                // subtract old mem usage. New usage added back after processing.
-                _memoryTracker.memoryUsageBytes -= groupObj->memUsageForSorter();
-            }
         }
 
         /* tickle all the accumulators for the group we found */
         dassert(numAccumulators == group.size());
 
         for (size_t i = 0; i < numAccumulators; i++) {
-            group[i]->process(
-                _accumulatedFields[i].expr.argument->evaluate(rootDocument, &pExpCtx->variables),
-                _doingMerge);
-
-            _memoryTracker.memoryUsageBytes += group[i]->memUsageForSorter();
+            // Only process the input and update the memory footprint if the current accumulator
+            // needs more input.
+            if (group[i]->needsInput()) {
+                const auto prevMemUsage = inserted ? 0 : group[i]->getMemUsage();
+                group[i]->process(_accumulatedFields[i].expr.argument->evaluate(
+                                      rootDocument, &pExpCtx->variables),
+                                  _doingMerge);
+                _memoryTracker.update(_accumulatedFields[i].fieldName,
+                                      group[i]->getMemUsage() - prevMemUsage);
+            }
         }
 
-        if (kDebugBuild && !storageGlobalParams.readOnly) {
+        if (kDebugBuild && !pExpCtx->opCtx->readOnly()) {
             // In debug mode, spill every time we have a duplicate id to stress merge logic.
-            if (!inserted &&                     // is a dup
-                !pExpCtx->inMongos &&            // can't spill to disk in mongos
-                !_memoryTracker.allowDiskUse &&  // don't change behavior when testing external sort
-                _sortedFiles.size() < 20) {      // don't open too many FDs
+            if (!inserted &&           // is a dup
+                !pExpCtx->inMongos &&  // can't spill to disk in mongos
+                !_memoryTracker
+                     ._allowDiskUse &&       // don't change behavior when testing external sort
+                _sortedFiles.size() < 20) {  // don't open too many FDs
 
                 _sortedFiles.push_back(spill());
             }
@@ -617,12 +697,9 @@ MONGO_COMPILER_NOINLINE DocumentSource::GetNextResult DocumentSourceGroup::initi
     MONGO_UNREACHABLE;
 }
 
-bool DocumentSourceGroup::usedDisk() {
-    return _usedDisk;
-}
-
 shared_ptr<Sorter<Value, Value>::Iterator> DocumentSourceGroup::spill() {
-    _usedDisk = true;
+    _stats.spills++;
+
     vector<const GroupsMap::value_type*> ptrs;  // using pointers to speed sorting
     ptrs.reserve(_groups->size());
     for (GroupsMap::const_iterator it = _groups->begin(), end = _groups->end(); it != end; ++it) {
@@ -631,6 +708,11 @@ shared_ptr<Sorter<Value, Value>::Iterator> DocumentSourceGroup::spill() {
 
     stable_sort(ptrs.begin(), ptrs.end(), SpillSTLComparator(pExpCtx->getValueComparator()));
 
+    // Initialize '_file' in a lazy manner only when it is needed.
+    if (!_file) {
+        _file =
+            std::make_shared<Sorter<Value, Value>::File>(pExpCtx->tempDir + "/" + nextFileName());
+    }
     SortedFileWriter<Value, Value> writer(SortOptions().TempDir(pExpCtx->tempDir), _file);
     switch (_accumulatedFields.size()) {  // same as ptrs[i]->second.size() for all i.
         case 0:                           // no values, essentially a distinct
@@ -657,7 +739,16 @@ shared_ptr<Sorter<Value, Value>::Iterator> DocumentSourceGroup::spill() {
             break;
     }
 
+    auto& metricsCollector = ResourceConsumption::MetricsCollector::get(pExpCtx->opCtx);
+    metricsCollector.incrementKeysSorted(ptrs.size());
+    metricsCollector.incrementSorterSpills(1);
+
     _groups->clear();
+    // Zero out the current per-accumulation statement memory consumption, as the memory has been
+    // freed by spilling.
+    for (auto accum : _accumulatedFields) {
+        _memoryTracker.set(accum.fieldName, 0);
+    }
 
     Sorter<Value, Value>::Iterator* iteratorPtr = writer.done();
     return shared_ptr<Sorter<Value, Value>::Iterator>(iteratorPtr);
@@ -714,10 +805,11 @@ Document DocumentSourceGroup::makeDocument(const Value& id,
             // we return null in this case so return objects are predictable
             out.addField(_accumulatedFields[i].fieldName, Value(BSONNULL));
         } else {
-            out.addField(_accumulatedFields[i].fieldName, val);
+            out.addField(_accumulatedFields[i].fieldName, std::move(val));
         }
     }
 
+    _stats.totalOutputDataSizeBytes += out.getApproximateSize();
     return out.freeze();
 }
 
@@ -727,7 +819,7 @@ boost::optional<DocumentSource::DistributedPlanLogic> DocumentSourceGroup::distr
 
     VariablesParseState vps = pExpCtx->variablesParseState;
     /* the merger will use the same grouping key */
-    mergingGroup->setIdExpression(ExpressionFieldPath::parse(pExpCtx, "$$ROOT._id", vps));
+    mergingGroup->setIdExpression(ExpressionFieldPath::parse(pExpCtx.get(), "$$ROOT._id", vps));
 
     for (auto&& accumulatedField : _accumulatedFields) {
         // The merger's output field names will be the same, as will the accumulator factories.
@@ -735,9 +827,10 @@ boost::optional<DocumentSource::DistributedPlanLogic> DocumentSourceGroup::distr
         // original accumulator may be collecting an expression based on a field expression or
         // constant.  Here, we accumulate the output of the same name from the prior group.
         auto copiedAccumulatedField = accumulatedField;
-        copiedAccumulatedField.expr.argument =
-            ExpressionFieldPath::parse(pExpCtx, "$$ROOT." + copiedAccumulatedField.fieldName, vps);
+        copiedAccumulatedField.expr.argument = ExpressionFieldPath::parse(
+            pExpCtx.get(), "$$ROOT." + copiedAccumulatedField.fieldName, vps);
         mergingGroup->addAccumulator(copiedAccumulatedField);
+        mergingGroup->_memoryTracker.set(copiedAccumulatedField.fieldName, 0);
     }
 
     // {shardsStage, mergingStage, sortPattern}
@@ -757,7 +850,7 @@ bool DocumentSourceGroup::pathIncludedInGroupKeys(const std::string& dottedPath)
 }
 
 bool DocumentSourceGroup::canRunInParallelBeforeWriteStage(
-    const std::set<std::string>& nameOfShardKeyFieldsUponEntryToStage) const {
+    const OrderedPathSet& nameOfShardKeyFieldsUponEntryToStage) const {
     if (_doingMerge) {
         return true;  // This is fine.
     }
@@ -788,7 +881,7 @@ DocumentSourceGroup::rewriteGroupAsTransformOnFirstDocument() const {
     }
 
     auto fieldPathExpr = dynamic_cast<ExpressionFieldPath*>(_idExpressions.front().get());
-    if (!fieldPathExpr || !fieldPathExpr->isRootFieldPath()) {
+    if (!fieldPathExpr || fieldPathExpr->isVariableReference()) {
         return nullptr;
     }
 
@@ -798,7 +891,9 @@ DocumentSourceGroup::rewriteGroupAsTransformOnFirstDocument() const {
         // each document has a unique _id, it will just return the entire collection). We only
         // apply the rewrite when grouping by a single field, so we cannot apply it in this case,
         // where we are grouping by the entire document.
-        invariant(fieldPath.getFieldName(0) == "CURRENT" || fieldPath.getFieldName(0) == "ROOT");
+        tassert(5943200,
+                "Optimization attempted on group by always-dissimilar system variable",
+                fieldPath.getFieldName(0) == "CURRENT" || fieldPath.getFieldName(0) == "ROOT");
         return nullptr;
     }
 
@@ -818,7 +913,7 @@ DocumentSourceGroup::rewriteGroupAsTransformOnFirstDocument() const {
     // The _id field can be specified either as a fieldpath (ex. _id: "$a") or as a singleton
     // object (ex. _id: {v: "$a"}).
     if (_idFieldNames.empty()) {
-        idField = ExpressionFieldPath::create(pExpCtx.get(), groupId);
+        idField = ExpressionFieldPath::deprecatedCreate(pExpCtx.get(), groupId);
     } else {
         invariant(_idFieldNames.size() == 1);
         idField = ExpressionObject::create(pExpCtx.get(),
@@ -836,6 +931,10 @@ DocumentSourceGroup::rewriteGroupAsTransformOnFirstDocument() const {
     return GroupFromFirstDocumentTransformation::create(pExpCtx, groupId, std::move(fields));
 }
 
+size_t DocumentSourceGroup::getMaxMemoryUsageBytes() const {
+    return _memoryTracker._maxAllowedMemoryUsageBytes;
+}
+
 }  // namespace mongo
 
 #include "mongo/db/sorter/sorter.cpp"
