@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/platform/basic.h"
 
@@ -40,21 +39,26 @@
 #include "mongo/db/curop_failpoint_helpers.h"
 #include "mongo/db/ops/write_ops.h"
 #include "mongo/db/pipeline/document_path_support.h"
+#include "mongo/db/pipeline/variable_validation.h"
+#include "mongo/db/storage/duplicate_key_error_info.h"
 #include "mongo/logv2/log.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
+
+
 namespace mongo {
 using namespace fmt::literals;
 
 MONGO_FAIL_POINT_DEFINE(hangWhileBuildingDocumentSourceMergeBatch);
 REGISTER_DOCUMENT_SOURCE(merge,
                          DocumentSourceMerge::LiteParsed::parse,
-                         DocumentSourceMerge::createFromBson);
+                         DocumentSourceMerge::createFromBson,
+                         AllowedWithApiStrict::kAlways);
 
 namespace {
 using MergeStrategyDescriptor = DocumentSourceMerge::MergeStrategyDescriptor;
 using MergeMode = MergeStrategyDescriptor::MergeMode;
 using MergeStrategy = MergeStrategyDescriptor::MergeStrategy;
-using BatchedCommandGenerator = MergeStrategyDescriptor::BatchedCommandGenerator;
 using MergeStrategyDescriptorsMap = std::map<const MergeMode, const MergeStrategyDescriptor>;
 using WhenMatched = MergeStrategyDescriptor::WhenMatched;
 using WhenNotMatched = MergeStrategyDescriptor::WhenNotMatched;
@@ -81,53 +85,6 @@ constexpr auto kPipelineDiscardMode = MergeMode{WhenMatched::kPipeline, WhenNotM
 const auto kDefaultPipelineLet = BSON("new"
                                       << "$$ROOT");
 
-BatchedCommandGenerator makeInsertCommandGenerator() {
-    return [](const auto& expCtx, const auto& ns) -> BatchedCommandRequest {
-        return DocumentSourceMerge::DocumentSourceWriter::makeInsertCommand(
-            ns, expCtx->bypassDocumentValidation);
-    };
-}
-
-BatchedCommandGenerator makeUpdateCommandGenerator() {
-    return [](const auto& expCtx, const auto& ns) -> BatchedCommandRequest {
-        write_ops::Update updateOp(ns);
-        updateOp.setWriteCommandBase([&] {
-            write_ops::WriteCommandBase wcb;
-            wcb.setOrdered(false);
-            wcb.setBypassDocumentValidation(expCtx->bypassDocumentValidation);
-            return wcb;
-        }());
-        updateOp.setRuntimeConstants(expCtx->getRuntimeConstants());
-        return BatchedCommandRequest(std::move(updateOp));
-    };
-}
-
-/**
- * Converts 'batch' into a vector of UpdateOpEntries.
- */
-std::vector<write_ops::UpdateOpEntry> constructUpdateEntries(
-    DocumentSourceMerge::DocumentSourceWriter::BatchedObjects&& batch,
-    UpsertType upsert,
-    bool multi,
-    bool useNewUpsert) {
-    std::vector<write_ops::UpdateOpEntry> updateEntries;
-    for (auto&& obj : batch) {
-        write_ops::UpdateOpEntry entry;
-        auto&& [q, u, c] = obj;
-        entry.setQ(std::move(q));
-        entry.setU(std::move(u));
-        entry.setC(std::move(c));
-        entry.setUpsert(upsert != UpsertType::kNone);
-        // Note that 'useNewUpsert' has been deleted on newer branches.
-        entry.setUpsertSupplied(
-            {{entry.getUpsert() && useNewUpsert, upsert == UpsertType::kInsertSuppliedDoc}});
-        entry.setMulti(multi);
-
-        updateEntries.push_back(std::move(entry));
-    }
-    return updateEntries;
-}
-
 /**
  * Creates a merge strategy which uses update semantics to perform a merge operation.
  */
@@ -137,15 +94,10 @@ MergeStrategy makeUpdateStrategy() {
               const auto& wc,
               auto epoch,
               auto&& batch,
-              auto&& bcr,
               UpsertType upsert) {
         constexpr auto multi = false;
-        auto updateCommand = bcr.extractUpdateRequest();
-        // Note that 'useNewUpsert' has been deleted on newer branches.
-        updateCommand->setUpdates(
-            constructUpdateEntries(std::move(batch), upsert, multi, expCtx->useNewUpsert));
         uassertStatusOK(expCtx->mongoProcessInterface->update(
-            expCtx, ns, std::move(updateCommand), wc, upsert, multi, epoch));
+            expCtx, ns, std::move(batch), wc, upsert, multi, epoch));
     };
 }
 
@@ -162,16 +114,11 @@ MergeStrategy makeStrictUpdateStrategy() {
               const auto& wc,
               auto epoch,
               auto&& batch,
-              auto&& bcr,
               UpsertType upsert) {
         const int64_t batchSize = batch.size();
         constexpr auto multi = false;
-        auto updateCommand = bcr.extractUpdateRequest();
-        // Note that 'useNewUpsert' has been deleted on newer branches.
-        updateCommand->setUpdates(
-            constructUpdateEntries(std::move(batch), upsert, multi, expCtx->useNewUpsert));
         auto updateResult = uassertStatusOK(expCtx->mongoProcessInterface->update(
-            expCtx, ns, std::move(updateCommand), wc, upsert, multi, epoch));
+            expCtx, ns, std::move(batch), wc, upsert, multi, epoch));
         uassert(ErrorCodes::MergeStageNoMatchingDocument,
                 "{} could not find a matching document in the target collection "
                 "for at least one document in the source collection"_format(kStageName),
@@ -188,18 +135,15 @@ MergeStrategy makeInsertStrategy() {
               const auto& wc,
               auto epoch,
               auto&& batch,
-              auto&& bcr,
               UpsertType upsertType) {
         std::vector<BSONObj> objectsToInsert(batch.size());
         // The batch stores replacement style updates, but for this "insert" style of $merge we'd
         // like to just insert the new document without attempting any sort of replacement.
         std::transform(batch.begin(), batch.end(), objectsToInsert.begin(), [](const auto& obj) {
-            return std::get<UpdateModification>(obj).getUpdateClassic();
+            return std::get<UpdateModification>(obj).getUpdateReplacement();
         });
-        auto insertCommand = bcr.extractInsertRequest();
-        insertCommand->setDocuments(std::move(objectsToInsert));
-        uassertStatusOK(
-            expCtx->mongoProcessInterface->insert(expCtx, ns, std::move(insertCommand), wc, epoch));
+        uassertStatusOK(expCtx->mongoProcessInterface->insert(
+            expCtx, ns, std::move(objectsToInsert), wc, epoch));
     };
 }
 
@@ -209,8 +153,8 @@ MergeStrategy makeInsertStrategy() {
  */
 BatchTransform makeUpdateTransform(const std::string& updateOp) {
     return [updateOp](auto& obj) {
-        std::get<UpdateModification>(obj) =
-            BSON(updateOp << std::get<UpdateModification>(obj).getUpdateClassic());
+        std::get<UpdateModification>(obj) = UpdateModification::parseFromClassicUpdate(
+            BSON(updateOp << std::get<UpdateModification>(obj).getUpdateReplacement()));
     };
 }
 
@@ -229,95 +173,72 @@ const MergeStrategyDescriptorsMap& getDescriptors() {
     // be initialized first. By wrapping the map into a function we can guarantee that it won't be
     // initialized until the first use, which is when the program already started and all global
     // variables had been initialized.
-    static const auto mergeStrategyDescriptors =
-        MergeStrategyDescriptorsMap{// whenMatched: replace, whenNotMatched: insert
-                                    {kReplaceInsertMode,
-                                     {kReplaceInsertMode,
-                                      {ActionType::insert, ActionType::update},
-                                      makeUpdateStrategy(),
-                                      {},
-                                      UpsertType::kGenerateNewDoc,
-                                      makeUpdateCommandGenerator()}},
-                                    // whenMatched: replace, whenNotMatched: fail
-                                    {kReplaceFailMode,
-                                     {kReplaceFailMode,
-                                      {ActionType::update},
-                                      makeStrictUpdateStrategy(),
-                                      {},
-                                      UpsertType::kNone,
-                                      makeUpdateCommandGenerator()}},
-                                    // whenMatched: replace, whenNotMatched: discard
-                                    {kReplaceDiscardMode,
-                                     {kReplaceDiscardMode,
-                                      {ActionType::update},
-                                      makeUpdateStrategy(),
-                                      {},
-                                      UpsertType::kNone,
-                                      makeUpdateCommandGenerator()}},
-                                    // whenMatched: merge, whenNotMatched: insert
-                                    {kMergeInsertMode,
-                                     {kMergeInsertMode,
-                                      {ActionType::insert, ActionType::update},
-                                      makeUpdateStrategy(),
-                                      makeUpdateTransform("$set"),
-                                      UpsertType::kGenerateNewDoc,
-                                      makeUpdateCommandGenerator()}},
-                                    // whenMatched: merge, whenNotMatched: fail
-                                    {kMergeFailMode,
-                                     {kMergeFailMode,
-                                      {ActionType::update},
-                                      makeStrictUpdateStrategy(),
-                                      makeUpdateTransform("$set"),
-                                      UpsertType::kNone,
-                                      makeUpdateCommandGenerator()}},
-                                    // whenMatched: merge, whenNotMatched: discard
-                                    {kMergeDiscardMode,
-                                     {kMergeDiscardMode,
-                                      {ActionType::update},
-                                      makeUpdateStrategy(),
-                                      makeUpdateTransform("$set"),
-                                      UpsertType::kNone,
-                                      makeUpdateCommandGenerator()}},
-                                    // whenMatched: keepExisting, whenNotMatched: insert
-                                    {kKeepExistingInsertMode,
-                                     {kKeepExistingInsertMode,
-                                      {ActionType::insert, ActionType::update},
-                                      makeUpdateStrategy(),
-                                      makeUpdateTransform("$setOnInsert"),
-                                      UpsertType::kGenerateNewDoc,
-                                      makeUpdateCommandGenerator()}},
-                                    // whenMatched: [pipeline], whenNotMatched: insert
-                                    {kPipelineInsertMode,
-                                     {kPipelineInsertMode,
-                                      {ActionType::insert, ActionType::update},
-                                      makeUpdateStrategy(),
-                                      {},
-                                      UpsertType::kInsertSuppliedDoc,
-                                      makeUpdateCommandGenerator()}},
-                                    // whenMatched: [pipeline], whenNotMatched: fail
-                                    {kPipelineFailMode,
-                                     {kPipelineFailMode,
-                                      {ActionType::update},
-                                      makeStrictUpdateStrategy(),
-                                      {},
-                                      UpsertType::kNone,
-                                      makeUpdateCommandGenerator()}},
-                                    // whenMatched: [pipeline], whenNotMatched: discard
-                                    {kPipelineDiscardMode,
-                                     {kPipelineDiscardMode,
-                                      {ActionType::update},
-                                      makeUpdateStrategy(),
-                                      {},
-                                      UpsertType::kNone,
-                                      makeUpdateCommandGenerator()}},
-                                    // whenMatched: fail, whenNotMatched: insert
-                                    {kFailInsertMode,
-                                     {kFailInsertMode,
-                                      {ActionType::insert},
-                                      makeInsertStrategy(),
-                                      {},
-                                      UpsertType::kNone,
-                                      makeInsertCommandGenerator()}}};
+    static const auto mergeStrategyDescriptors = MergeStrategyDescriptorsMap{
+        // whenMatched: replace, whenNotMatched: insert
+        {kReplaceInsertMode,
+         {kReplaceInsertMode,
+          {ActionType::insert, ActionType::update},
+          makeUpdateStrategy(),
+          {},
+          UpsertType::kGenerateNewDoc}},
+        // whenMatched: replace, whenNotMatched: fail
+        {kReplaceFailMode,
+         {kReplaceFailMode,
+          {ActionType::update},
+          makeStrictUpdateStrategy(),
+          {},
+          UpsertType::kNone}},
+        // whenMatched: replace, whenNotMatched: discard
+        {kReplaceDiscardMode,
+         {kReplaceDiscardMode, {ActionType::update}, makeUpdateStrategy(), {}, UpsertType::kNone}},
+        // whenMatched: merge, whenNotMatched: insert
+        {kMergeInsertMode,
+         {kMergeInsertMode,
+          {ActionType::insert, ActionType::update},
+          makeUpdateStrategy(),
+          makeUpdateTransform("$set"),
+          UpsertType::kGenerateNewDoc}},
+        // whenMatched: merge, whenNotMatched: fail
+        {kMergeFailMode,
+         {kMergeFailMode,
+          {ActionType::update},
+          makeStrictUpdateStrategy(),
+          makeUpdateTransform("$set"),
+          UpsertType::kNone}},
+        // whenMatched: merge, whenNotMatched: discard
+        {kMergeDiscardMode,
+         {kMergeDiscardMode,
+          {ActionType::update},
+          makeUpdateStrategy(),
+          makeUpdateTransform("$set"),
+          UpsertType::kNone}},
+        // whenMatched: keepExisting, whenNotMatched: insert
+        {kKeepExistingInsertMode,
+         {kKeepExistingInsertMode,
+          {ActionType::insert, ActionType::update},
+          makeUpdateStrategy(),
+          makeUpdateTransform("$setOnInsert"),
+          UpsertType::kGenerateNewDoc}},
+        // whenMatched: [pipeline], whenNotMatched: insert
+        {kPipelineInsertMode,
+         {kPipelineInsertMode,
+          {ActionType::insert, ActionType::update},
+          makeUpdateStrategy(),
+          {},
+          UpsertType::kInsertSuppliedDoc}},
+        // whenMatched: [pipeline], whenNotMatched: fail
+        {kPipelineFailMode,
+         {kPipelineFailMode,
+          {ActionType::update},
+          makeStrictUpdateStrategy(),
+          {},
+          UpsertType::kNone}},
+        // whenMatched: [pipeline], whenNotMatched: discard
+        {kPipelineDiscardMode,
+         {kPipelineDiscardMode, {ActionType::update}, makeUpdateStrategy(), {}, UpsertType::kNone}},
+        // whenMatched: fail, whenNotMatched: insert
+        {kFailInsertMode,
+         {kFailInsertMode, {ActionType::insert}, makeInsertStrategy(), {}, UpsertType::kNone}}};
     return mergeStrategyDescriptors;
 }
 
@@ -355,7 +276,7 @@ BSONObj extractMergeOnFieldsFromDoc(const Document& doc, const std::set<FieldPat
  * explicitly specified, it will be defaulted to 'defaultDb'.
  */
 DocumentSourceMergeSpec parseMergeSpecAndResolveTargetNamespace(const BSONElement& spec,
-                                                                StringData defaultDb) {
+                                                                const DatabaseName& defaultDb) {
     NamespaceString targetNss;
     DocumentSourceMergeSpec mergeSpec;
 
@@ -366,7 +287,8 @@ DocumentSourceMergeSpec parseMergeSpecAndResolveTargetNamespace(const BSONElemen
     if (spec.type() == BSONType::String) {
         targetNss = {defaultDb, spec.valueStringData()};
     } else {
-        mergeSpec = DocumentSourceMergeSpec::parse({kStageName}, spec.embeddedObject());
+        mergeSpec =
+            DocumentSourceMergeSpec::parse(IDLParserContext{kStageName}, spec.embeddedObject());
         targetNss = mergeSpec.getTargetNss();
         if (targetNss.coll().empty()) {
             // If the $merge spec is an object, the target namespace can be specified as a string
@@ -376,7 +298,7 @@ DocumentSourceMergeSpec parseMergeSpecAndResolveTargetNamespace(const BSONElemen
             // target namespace collection is empty, we'll use the default database name as a target
             // database, and the provided namespace value as a collection name.
             targetNss = {defaultDb, targetNss.ns()};
-        } else if (targetNss.db().empty()) {
+        } else if (targetNss.dbName().db().empty()) {
             // Use the default database name if it wasn't specified explicilty.
             targetNss = {defaultDb, targetNss.coll()};
         }
@@ -414,7 +336,7 @@ std::unique_ptr<DocumentSourceMerge::LiteParsed> DocumentSourceMerge::LiteParsed
                                                                            typeName(spec.type())),
             spec.type() == BSONType::String || spec.type() == BSONType::Object);
 
-    auto mergeSpec = parseMergeSpecAndResolveTargetNamespace(spec, nss.db());
+    auto mergeSpec = parseMergeSpecAndResolveTargetNamespace(spec, nss.dbName());
     auto targetNss = mergeSpec.getTargetNss();
 
     uassert(ErrorCodes::InvalidNamespace,
@@ -473,11 +395,11 @@ DocumentSourceMerge::DocumentSourceMerge(NamespaceString outputNs,
 
         for (auto&& varElem : *letVariables) {
             const auto varName = varElem.fieldNameStringData();
-            Variables::uassertValidNameForUserWrite(varName);
+            variableValidation::validateNameForUserWrite(varName);
 
             _letVariables->emplace(
                 varName.toString(),
-                Expression::parseOperand(expCtx, varElem, expCtx->variablesParseState));
+                Expression::parseOperand(expCtx.get(), varElem, expCtx->variablesParseState));
         }
     }
 }
@@ -549,7 +471,7 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceMerge::createFromBson(
             "{} only supports a string or object argument, not {}"_format(kStageName, spec.type()),
             spec.type() == BSONType::String || spec.type() == BSONType::Object);
 
-    auto mergeSpec = parseMergeSpecAndResolveTargetNamespace(spec, expCtx->ns.db());
+    auto mergeSpec = parseMergeSpecAndResolveTargetNamespace(spec, expCtx->ns.dbName());
     auto targetNss = mergeSpec.getTargetNss();
     auto whenMatched =
         mergeSpec.getWhenMatched() ? mergeSpec.getWhenMatched()->mode : kDefaultWhenMatched;
@@ -558,7 +480,12 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceMerge::createFromBson(
     auto fieldPaths = convertToFieldPaths(mergeSpec.getOn());
     auto [mergeOnFields, targetCollectionVersion] =
         expCtx->mongoProcessInterface->ensureFieldsUniqueOrResolveDocumentKey(
-            expCtx, std::move(fieldPaths), mergeSpec.getTargetCollectionVersion(), targetNss);
+            expCtx,
+            std::move(fieldPaths),
+            mergeSpec.getTargetCollectionVersion()
+                ? boost::make_optional((ChunkVersion)*mergeSpec.getTargetCollectionVersion())
+                : boost::none,
+            targetNss);
 
     return DocumentSourceMerge::create(std::move(targetNss),
                                        expCtx,
@@ -570,6 +497,40 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceMerge::createFromBson(
                                        targetCollectionVersion);
 }
 
+StageConstraints DocumentSourceMerge::constraints(Pipeline::SplitState pipeState) const {
+    // A $merge to an unsharded collection should merge on the primary shard to perform local
+    // writes. A $merge to a sharded collection has no requirement, since each shard can perform its
+    // own portion of the write. We use 'kAnyShard' to direct it to execute on one of the shards in
+    // case some of the writes happen to end up being local.
+    //
+    // Note that this decision is inherently racy and subject to become stale. This is okay because
+    // either choice will work correctly, we are simply applying a heuristic optimization.
+    return {StreamType::kStreaming,
+            PositionRequirement::kLast,
+            pExpCtx->inMongos &&
+                    pExpCtx->mongoProcessInterface->isSharded(pExpCtx->opCtx, _outputNs)
+                ? HostTypeRequirement::kAnyShard
+                : HostTypeRequirement::kPrimaryShard,
+            DiskUseRequirement::kWritesPersistentData,
+            FacetRequirement::kNotAllowed,
+            TransactionRequirement::kNotAllowed,
+            LookupRequirement::kNotAllowed,
+            UnionRequirement::kNotAllowed};
+}
+
+boost::optional<DocumentSource::DistributedPlanLogic> DocumentSourceMerge::distributedPlanLogic() {
+    // It should always be faster to avoid splitting the pipeline if the output collection is
+    // sharded. If we avoid splitting the pipeline then each shard can perform the writes to the
+    // target collection in parallel.
+    //
+    // Note that this decision is inherently racy and subject to become stale. This is okay because
+    // either choice will work correctly, we are simply applying a heuristic optimization.
+    if (pExpCtx->mongoProcessInterface->isSharded(pExpCtx->opCtx, _outputNs)) {
+        return boost::none;
+    }
+    return DocumentSourceWriter::distributedPlanLogic();
+}
+
 Value DocumentSourceMerge::serialize(boost::optional<ExplainOptions::Verbosity> explain) const {
     DocumentSourceMergeSpec spec;
     spec.setTargetNss(_outputNs);
@@ -593,7 +554,9 @@ Value DocumentSourceMerge::serialize(boost::optional<ExplainOptions::Verbosity>
         }
         return mergeOnFields;
     }());
-    spec.setTargetCollectionVersion(_targetCollectionVersion);
+    spec.setTargetCollectionVersion(
+        _targetCollectionVersion ? boost::make_optional(ShardVersion(*_targetCollectionVersion))
+                                 : boost::none);
     return Value(Document{{getSourceName(), spec.toBSON()}});
 }
 
@@ -614,37 +577,42 @@ std::pair<DocumentSourceMerge::BatchObject, int> DocumentSourceMerge::makeBatchO
         _descriptor.transform(batchObject);
     }
 
-    invariant(_writeSizeEstimator);
+    tassert(6628901, "_writeSizeEstimator should be initialized", _writeSizeEstimator);
     return {batchObject,
             _writeSizeEstimator->estimateUpdateSizeBytes(batchObject, _descriptor.upsertType)};
 }
 
-void DocumentSourceMerge::spill(BatchedCommandRequest&& bcr, BatchedObjects&& batch) {
+void DocumentSourceMerge::spill(BatchedObjects&& batch) try {
     DocumentSourceWriteBlock writeBlock(pExpCtx->opCtx);
+    auto targetEpoch = _targetCollectionVersion
+        ? boost::optional<OID>(_targetCollectionVersion->epoch())
+        : boost::none;
 
-    try {
-        auto targetEpoch = _targetCollectionVersion
-            ? boost::optional<OID>(_targetCollectionVersion->epoch())
-            : boost::none;
+    _descriptor.strategy(
+        pExpCtx, _outputNs, _writeConcern, targetEpoch, std::move(batch), _descriptor.upsertType);
+} catch (const ExceptionFor<ErrorCodes::ImmutableField>& ex) {
+    uassertStatusOKWithContext(ex.toStatus(),
+                               "$merge failed to update the matching document, did you "
+                               "attempt to modify the _id or the shard key?");
+} catch (const ExceptionFor<ErrorCodes::DuplicateKey>& ex) {
+    // A DuplicateKey error could be due to a collision on the 'on' fields or on any other unique
+    // index.
+    auto dupKeyPattern = ex->getKeyPattern();
+    bool dupKeyFromMatchingOnFields =
+        (static_cast<size_t>(dupKeyPattern.nFields()) == _mergeOnFields.size()) &&
+        std::all_of(_mergeOnFields.begin(), _mergeOnFields.end(), [&](auto onField) {
+            return dupKeyPattern.hasField(onField.fullPath());
+        });
 
-        _descriptor.strategy(pExpCtx,
-                             _outputNs,
-                             _writeConcern,
-                             targetEpoch,
-                             std::move(batch),
-                             std::move(bcr),
-                             _descriptor.upsertType);
-    } catch (const ExceptionFor<ErrorCodes::ImmutableField>& ex) {
+    if (_descriptor.mode == kFailInsertMode && dupKeyFromMatchingOnFields) {
         uassertStatusOKWithContext(ex.toStatus(),
-                                   "$merge failed to update the matching document, did you "
-                                   "attempt to modify the _id or the shard key?");
+                                   "$merge with whenMatched: fail found an existing document with "
+                                   "the same values for the 'on' fields");
+    } else {
+        uassertStatusOKWithContext(ex.toStatus(), "$merge failed due to a DuplicateKey error");
     }
 }
 
-BatchedCommandRequest DocumentSourceMerge::initializeBatchedWriteRequest() const {
-    return _descriptor.batchedCommandGenerator(pExpCtx, _outputNs);
-}
-
 void DocumentSourceMerge::waitWhileFailPointEnabled() {
     CurOpFailpointHelpers::waitWhileFailPointEnabled(
         &hangWhileBuildingDocumentSourceMergeBatch,
