@@ -43,9 +43,9 @@
 #include "mongo/db/pipeline/document_source_sort.h"
 #include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/query/cursor_response.h"
-#include "mongo/db/query/getmore_request.h"
+#include "mongo/db/query/getmore_command_gen.h"
 #include "mongo/db/query/query_knobs_gen.h"
-#include "mongo/db/query/query_request.h"
+#include "mongo/db/query/query_request_helper.h"
 #include "mongo/executor/network_interface_mock.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/executor/thread_pool_task_executor_test_fixture.h"
@@ -71,7 +71,7 @@ const std::vector<HostAndPort> kTestShardHosts = {HostAndPort("FakeShard1Host",
                                                   HostAndPort("FakeShard2Host", 12345),
                                                   HostAndPort("FakeShard3Host", 12345)};
 
-const NamespaceString kTestNss = NamespaceString("test.mergeCursors"_sd);
+const NamespaceString kTestNss = NamespaceString(boost::none, "test.mergeCursors"_sd);
 const HostAndPort kTestHost = HostAndPort("localhost:27017"_sd);
 
 const CursorId kExhaustedCursorID = 0;
@@ -80,14 +80,15 @@ class DocumentSourceMergeCursorsTest : public ShardingTestFixture {
 public:
     DocumentSourceMergeCursorsTest() {
         TimeZoneDatabase::set(getServiceContext(), std::make_unique<TimeZoneDatabase>());
-        _expCtx = new ExpressionContext(operationContext(), nullptr, kTestNss);
-        _expCtx->mongoProcessInterface = std::make_shared<StubMongoProcessInterface>(executor());
     }
 
     void setUp() override {
         ShardingTestFixture::setUp();
         setRemote(HostAndPort("ClientHost", 12345));
 
+        _expCtx = new ExpressionContext(operationContext(), nullptr, kTestNss);
+        _expCtx->mongoProcessInterface = std::make_shared<StubMongoProcessInterface>(executor());
+
         configTargeter()->setFindHostReturnValue(kTestConfigShardHost);
 
         std::vector<ShardType> shards;
@@ -178,7 +179,7 @@ TEST_F(DocumentSourceMergeCursorsTest, ShouldBeAbleToParseSerializedARMParams) {
     // Make sure the serialized version can be parsed into an identical AsyncResultsMergerParams.
     auto newSpec = serializationArray[0].getDocument().toBson();
     ASSERT(newSpec["$mergeCursors"].type() == BSONType::Object);
-    auto newParams = AsyncResultsMergerParams::parse(IDLParserErrorContext("$mergeCursors test"),
+    auto newParams = AsyncResultsMergerParams::parse(IDLParserContext("$mergeCursors test"),
                                                      newSpec["$mergeCursors"].Obj());
     ASSERT_TRUE(params.getSort());
     ASSERT_BSONOBJ_EQ(*params.getSort(), *newParams.getSort());
@@ -367,14 +368,14 @@ TEST_F(DocumentSourceMergeCursorsTest, ShouldEnforceSortSpecifiedViaARMParams) {
     onCommand([&](const auto& request) {
         return cursorResponseObj(expCtx->ns,
                                  kExhaustedCursorID,
-                                 {BSON("x" << 1 << "$sortKey" << BSON("" << 1)),
-                                  BSON("x" << 3 << "$sortKey" << BSON("" << 3))});
+                                 {BSON("x" << 1 << "$sortKey" << BSON_ARRAY(1)),
+                                  BSON("x" << 3 << "$sortKey" << BSON_ARRAY(3))});
     });
     onCommand([&](const auto& request) {
         return cursorResponseObj(expCtx->ns,
                                  kExhaustedCursorID,
-                                 {BSON("x" << 2 << "$sortKey" << BSON("" << 2)),
-                                  BSON("x" << 4 << "$sortKey" << BSON("" << 4))});
+                                 {BSON("x" << 2 << "$sortKey" << BSON_ARRAY(2)),
+                                  BSON("x" << 4 << "$sortKey" << BSON_ARRAY(4))});
     });
 
     future.default_timed_get();
