@@ -26,7 +26,7 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
+
 
 #include "mongo/platform/basic.h"
 
@@ -35,20 +35,22 @@
 #include "mongo/db/commands/test_commands_enabled.h"
 #include "mongo/db/pipeline/document_source_documents.h"
 #include "mongo/db/pipeline/document_source_match.h"
+#include "mongo/db/pipeline/document_source_queue.h"
 #include "mongo/db/pipeline/document_source_single_document_transformation.h"
 #include "mongo/db/pipeline/document_source_union_with.h"
 #include "mongo/db/pipeline/document_source_union_with_gen.h"
-#include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/views/resolved_view.h"
 #include "mongo/logv2/log.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
+
+
 namespace mongo {
 
-REGISTER_DOCUMENT_SOURCE_WITH_MIN_VERSION(
-    unionWith,
-    DocumentSourceUnionWith::LiteParsed::parse,
-    DocumentSourceUnionWith::createFromBson,
-    ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44);
+REGISTER_DOCUMENT_SOURCE(unionWith,
+                         DocumentSourceUnionWith::LiteParsed::parse,
+                         DocumentSourceUnionWith::createFromBson,
+                         AllowedWithApiStrict::kAlways);
 
 namespace {
 std::unique_ptr<Pipeline, PipelineDeleter> buildPipelineFromViewDefinition(
@@ -66,22 +68,14 @@ std::unique_ptr<Pipeline, PipelineDeleter> buildPipelineFromViewDefinition(
         });
     };
 
-    // Copy the ExpressionContext of the base aggregation, using the inner namespace instead.
-    auto unionExpCtx = expCtx->copyForSubPipeline(resolvedNs.ns);
-
-    if (resolvedNs.pipeline.empty()) {
-        return Pipeline::parse(currentPipeline, unionExpCtx, validatorCallback);
-    }
-    auto resolvedPipeline = std::move(resolvedNs.pipeline);
-    resolvedPipeline.reserve(currentPipeline.size() + resolvedPipeline.size());
-    resolvedPipeline.insert(resolvedPipeline.end(),
-                            std::make_move_iterator(currentPipeline.begin()),
-                            std::make_move_iterator(currentPipeline.end()));
-
     MakePipelineOptions opts;
     opts.attachCursorSource = false;
+    // Only call optimize() here if we actually have a pipeline to resolve in the view definition.
+    opts.optimize = !resolvedNs.pipeline.empty();
     opts.validator = validatorCallback;
-    return Pipeline::makePipeline(std::move(resolvedPipeline), unionExpCtx, opts);
+
+    return Pipeline::makePipelineFromViewDefinition(
+        expCtx->copyForSubPipeline(expCtx->ns, resolvedNs.uuid), resolvedNs, currentPipeline, opts);
 }
 
 }  // namespace
@@ -95,11 +89,31 @@ DocumentSourceUnionWith::~DocumentSourceUnionWith() {
 
 void validateUnionWithCollectionlessPipeline(
     const boost::optional<std::vector<mongo::BSONObj>>& pipeline) {
+    const auto errMsg =
+        "$unionWith stage without explicit collection must have a pipeline with $documents as "
+        "first stage";
+
+    uassert(ErrorCodes::FailedToParse, errMsg, pipeline && pipeline->size() > 0);
+    const auto firstStageBson = (*pipeline)[0];
+    LOGV2_DEBUG(5909700,
+                4,
+                "$unionWith validating collectionless pipeline",
+                "pipeline"_attr = pipeline,
+                "first"_attr = firstStageBson);
     uassert(ErrorCodes::FailedToParse,
-            "$unionWith stage without explicit collection must have a pipeline with $documents as "
-            "first stage",
-            pipeline && pipeline->size() > 0 &&
-                !(*pipeline)[0].getField(DocumentSourceDocuments::kStageName).eoo());
+            errMsg,
+            // TODO SERVER-59628 replace with constraints check
+            (firstStageBson.hasField(DocumentSourceDocuments::kStageName) ||
+             firstStageBson.hasField(DocumentSourceQueue::kStageName))
+
+    );
+}
+
+boost::intrusive_ptr<DocumentSource> DocumentSourceUnionWith::clone(
+    const boost::intrusive_ptr<ExpressionContext>& newExpCtx) const {
+    // At this point the ExpressionContext already has info about any resolved namespaces, so there
+    // is no need to resolve them again when creating the clone.
+    return make_intrusive<DocumentSourceUnionWith>(*this, newExpCtx);
 }
 
 std::unique_ptr<DocumentSourceUnionWith::LiteParsed> DocumentSourceUnionWith::LiteParsed::parse(
@@ -113,16 +127,16 @@ std::unique_ptr<DocumentSourceUnionWith::LiteParsed> DocumentSourceUnionWith::Li
     NamespaceString unionNss;
     boost::optional<LiteParsedPipeline> liteParsedPipeline;
     if (spec.type() == BSONType::String) {
-        unionNss = NamespaceString(nss.db(), spec.valueStringData());
+        unionNss = NamespaceString(nss.dbName(), spec.valueStringData());
     } else {
         auto unionWithSpec =
-            UnionWithSpec::parse(IDLParserErrorContext(kStageName), spec.embeddedObject());
+            UnionWithSpec::parse(IDLParserContext(kStageName), spec.embeddedObject());
         if (unionWithSpec.getColl()) {
-            unionNss = NamespaceString(nss.db(), *unionWithSpec.getColl());
+            unionNss = NamespaceString(nss.dbName(), *unionWithSpec.getColl());
         } else {
             // If no collection specified, it must have $documents as first field in pipeline.
             validateUnionWithCollectionlessPipeline(unionWithSpec.getPipeline());
-            unionNss = NamespaceString::makeCollectionlessAggregateNSS(nss.db());
+            unionNss = NamespaceString::makeCollectionlessAggregateNSS(nss.dbName());
         }
 
         // Recursively lite parse the nested pipeline, if one exists.
@@ -171,16 +185,16 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceUnionWith::createFromBson(
     NamespaceString unionNss;
     std::vector<BSONObj> pipeline;
     if (elem.type() == BSONType::String) {
-        unionNss = NamespaceString(expCtx->ns.db().toString(), elem.valueStringData());
+        unionNss = NamespaceString(expCtx->ns.dbName(), elem.valueStringData());
     } else {
         auto unionWithSpec =
-            UnionWithSpec::parse(IDLParserErrorContext(kStageName), elem.embeddedObject());
+            UnionWithSpec::parse(IDLParserContext(kStageName), elem.embeddedObject());
         if (unionWithSpec.getColl()) {
-            unionNss = NamespaceString(expCtx->ns.db().toString(), *unionWithSpec.getColl());
+            unionNss = NamespaceString(expCtx->ns.dbName(), *unionWithSpec.getColl());
         } else {
             // if no collection specified, it must have $documents as first field in pipeline
             validateUnionWithCollectionlessPipeline(unionWithSpec.getPipeline());
-            unionNss = NamespaceString::makeCollectionlessAggregateNSS(expCtx->ns.db());
+            unionNss = NamespaceString::makeCollectionlessAggregateNSS(expCtx->ns.dbName());
         }
         pipeline = unionWithSpec.getPipeline().value_or(std::vector<BSONObj>{});
     }
@@ -209,17 +223,6 @@ DocumentSource::GetNextResult DocumentSourceUnionWith::doGetNext() {
     if (_executionState == ExecutionProgress::kStartingSubPipeline) {
         auto serializedPipe = _pipeline->serializeToBson();
         logStartingSubPipeline(serializedPipe);
-        // $$SEARCH_META can be set during runtime earlier in the pipeline, and therefore must be
-        // copied to the subpipeline manually.
-        if (serverGlobalParams.featureCompatibility.getVersion() >=
-                ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44 &&
-            enableSearchMeta.load()) {
-            auto metaVal = pExpCtx->variables.getValue(Variables::kSearchMetaId, Document());
-            if (!metaVal.missing()) {
-                _pipeline->getContext()->variables.setReservedValue(
-                    Variables::kSearchMetaId, std::move(metaVal), true);
-            }
-        }
         try {
             _pipeline =
                 pExpCtx->mongoProcessInterface->attachCursorSourceToPipeline(_pipeline.release());
@@ -234,10 +237,18 @@ DocumentSource::GetNextResult DocumentSourceUnionWith::doGetNext() {
         }
     }
 
+    // The $unionWith stage takes responsibility for disposing of its Pipeline. When the outer
+    // Pipeline that contains the $unionWith is disposed of, it will propagate dispose() to its
+    // subpipeline.
+    _pipeline.get_deleter().dismissDisposal();
+
     auto res = _pipeline->getNext();
     if (res)
         return std::move(*res);
 
+    // Record the plan summary stats after $unionWith operation is done.
+    accumulatePipelinePlanSummaryStats(*_pipeline, _stats.planSummaryStats);
+
     _executionState = ExecutionProgress::kFinished;
     return GetNextResult::makeEOF();
 }
@@ -256,9 +267,9 @@ MONGO_COMPILER_NOINLINE void DocumentSourceUnionWith::logShardedViewFound(
     const ExceptionFor<ErrorCodes::CommandOnShardedViewNotSupportedOnMongod>& e) {
     LOGV2_DEBUG(4556300,
                 3,
-                "$unionWith found view definition. ns: {ns}, pipeline: {pipeline}. New "
+                "$unionWith found view definition. ns: {namespace}, pipeline: {pipeline}. New "
                 "$unionWith sub-pipeline: {new_pipe}",
-                "ns"_attr = e->getNamespace(),
+                logAttrs(e->getNamespace()),
                 "pipeline"_attr = Value(e->getPipeline()),
                 "new_pipe"_attr = _pipeline->serializeToBson());
 }
@@ -291,14 +302,19 @@ Pipeline::SourceContainer::iterator DocumentSourceUnionWith::doOptimizeAt(
 
 bool DocumentSourceUnionWith::usedDisk() {
     if (_pipeline) {
-        _usedDisk = _usedDisk || _pipeline->usedDisk();
+        _stats.planSummaryStats.usedDisk =
+            _stats.planSummaryStats.usedDisk || _pipeline->usedDisk();
     }
-    return _usedDisk;
+    return _stats.planSummaryStats.usedDisk;
 }
 
 void DocumentSourceUnionWith::doDispose() {
     if (_pipeline) {
-        _usedDisk = _usedDisk || _pipeline->usedDisk();
+        _pipeline.get_deleter().dismissDisposal();
+        _stats.planSummaryStats.usedDisk =
+            _stats.planSummaryStats.usedDisk || _pipeline->usedDisk();
+        accumulatePipelinePlanSummaryStats(*_pipeline, _stats.planSummaryStats);
+
         if (!_pipeline->getContext()->explain) {
             _pipeline->dispose(pExpCtx->opCtx);
             _pipeline.reset();
@@ -330,7 +346,7 @@ Value DocumentSourceUnionWith::serialize(boost::optional<ExplainOptions::Verbosi
             // The plan does not require reading from the sub-pipeline, so just include the
             // serialization in the explain output.
             BSONArrayBuilder bab;
-            for (auto&& stage : _pipeline->serialize())
+            for (auto&& stage : _pipeline->serialize(explain))
                 bab << stage;
             auto spec = collectionless
                 ? DOC("pipeline" << bab.arr())
@@ -360,11 +376,34 @@ Value DocumentSourceUnionWith::serialize(boost::optional<ExplainOptions::Verbosi
     }
 }
 
+// Extracting dependencies for the outer collection. Although, this method walks the inner pipeline,
+// the field dependencies are not collected - only variable dependencies are.
 DepsTracker::State DocumentSourceUnionWith::getDependencies(DepsTracker* deps) const {
-    // Since the $unionWith stage is a simple passthrough, we *could* report SEE_NEXT here in an
-    // attempt to get a covered plan for the base collection. The ideal solution would involve
-    // pushing down any dependencies to the inner pipeline as well.
-    return DepsTracker::State::NOT_SUPPORTED;
+    if (!_pipeline) {
+        return DepsTracker::State::SEE_NEXT;
+    }
+
+    // We only need to know what variable dependencies exist in the subpipeline. So without
+    // knowledge of what metadata is in fact unavailable, we "lie" and say that all metadata
+    // is available to avoid tripping any assertions.
+    DepsTracker subDeps(DepsTracker::kNoMetadata);
+    // Get the subpipeline dependencies.
+    for (auto&& source : _pipeline->getSources()) {
+        source->getDependencies(&subDeps);
+    }
+
+    return DepsTracker::State::SEE_NEXT;
+}
+
+void DocumentSourceUnionWith::addVariableRefs(std::set<Variables::Id>* refs) const {
+    // Add sub-pipeline variable dependencies. Do not add SEARCH_META as a dependency, since it is
+    // scoped to one pipeline.
+    std::set<Variables::Id> subPipeRefs;
+    _pipeline->addVariableRefs(&subPipeRefs);
+    for (auto&& varId : subPipeRefs) {
+        if (varId != Variables::kSearchMetaId)
+            refs->insert(varId);
+    }
 }
 
 void DocumentSourceUnionWith::detachFromOperationContext() {
