@@ -41,14 +41,15 @@
 #include "mongo/db/exec/document_value/value_comparator.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
-#include "mongo/db/pipeline/aggregation_request.h"
+#include "mongo/db/pipeline/document_source_change_stream_gen.h"
 #include "mongo/db/pipeline/javascript_execution.h"
+#include "mongo/db/pipeline/legacy_runtime_constants_gen.h"
 #include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
-#include "mongo/db/pipeline/runtime_constants_gen.h"
 #include "mongo/db/pipeline/variables.h"
 #include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/datetime/date_time_support.h"
 #include "mongo/db/query/explain_options.h"
+#include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/tailable_mode.h"
 #include "mongo/db/server_options.h"
 #include "mongo/util/intrusive_counter.h"
@@ -59,24 +60,17 @@ namespace mongo {
 
 class AggregateCommandRequest;
 
-/**
- * The structure ExpressionCounters encapsulates counters for match, aggregate, and other
- * expression types as seen in the end-user queries.
- */
-struct ExpressionCounters {
-    StringMap<uint64_t> aggExprCountersMap;
-    StringMap<uint64_t> matchExprCountersMap;
-};
-
 class ExpressionContext : public RefCountable {
 public:
-    static constexpr size_t kMaxSubPipelineViewDepth = 20;
     struct ResolvedNamespace {
         ResolvedNamespace() = default;
-        ResolvedNamespace(NamespaceString ns, std::vector<BSONObj> pipeline);
+        ResolvedNamespace(NamespaceString ns,
+                          std::vector<BSONObj> pipeline,
+                          boost::optional<UUID> uuid = boost::none);
 
         NamespaceString ns;
         std::vector<BSONObj> pipeline;
+        boost::optional<UUID> uuid = boost::none;
     };
 
     /**
@@ -99,8 +93,7 @@ public:
          * This constructor is private, all CollatorStashes should be created by calling
          * ExpressionContext::temporarilyChangeCollator().
          */
-        CollatorStash(const boost::intrusive_ptr<ExpressionContext>& expCtx,
-                      std::unique_ptr<CollatorInterface> newCollator);
+        CollatorStash(ExpressionContext* expCtx, std::unique_ptr<CollatorInterface> newCollator);
 
         friend class ExpressionContext;
 
@@ -116,6 +109,8 @@ public:
     struct ExpressionCounters {
         StringMap<uint64_t> aggExprCountersMap;
         StringMap<uint64_t> matchExprCountersMap;
+        StringMap<uint64_t> groupAccumulatorExprCountersMap;
+        StringMap<uint64_t> windowAccumulatorExprCountersMap;
     };
 
     /**
@@ -123,15 +118,16 @@ public:
      * 'resolvedNamespaces' maps collection names (not full namespaces) to ResolvedNamespaces.
      */
     ExpressionContext(OperationContext* opCtx,
-                      const AggregationRequest& request,
+                      const AggregateCommandRequest& request,
                       std::unique_ptr<CollatorInterface> collator,
                       std::shared_ptr<MongoProcessInterface> mongoProcessInterface,
                       StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces,
-                      boost::optional<UUID> collUUID);
+                      boost::optional<UUID> collUUID,
+                      bool mayDbProfile = true);
 
     /**
      * Constructs an ExpressionContext to be used for Pipeline parsing and evaluation. This version
-     * requires finer-grained parameters but does not require an AggregationRequest.
+     * requires finer-grained parameters but does not require an AggregateCommandRequest.
      * 'resolvedNamespaces' maps collection names (not full namespaces) to ResolvedNamespaces.
      */
     ExpressionContext(OperationContext* opCtx,
@@ -142,11 +138,13 @@ public:
                       bool bypassDocumentValidation,
                       bool isMapReduceCommand,
                       const NamespaceString& ns,
-                      const boost::optional<RuntimeConstants>& runtimeConstants,
+                      const boost::optional<LegacyRuntimeConstants>& runtimeConstants,
                       std::unique_ptr<CollatorInterface> collator,
                       const std::shared_ptr<MongoProcessInterface>& mongoProcessInterface,
                       StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces,
-                      boost::optional<UUID> collUUID);
+                      boost::optional<UUID> collUUID,
+                      const boost::optional<BSONObj>& letParameters = boost::none,
+                      bool mayDbProfile = true);
 
     /**
      * Constructs an ExpressionContext suitable for use outside of the aggregation system, including
@@ -157,13 +155,20 @@ public:
     ExpressionContext(OperationContext* opCtx,
                       std::unique_ptr<CollatorInterface> collator,
                       const NamespaceString& ns,
-                      const boost::optional<RuntimeConstants>& runtimeConstants = boost::none);
+                      const boost::optional<LegacyRuntimeConstants>& runtimeConstants = boost::none,
+                      const boost::optional<BSONObj>& letParameters = boost::none,
+                      bool mayDbProfile = true,
+                      boost::optional<ExplainOptions::Verbosity> explain = boost::none);
 
     /**
      * Used by a pipeline to check for interrupts so that killOp() works. Throws a UserAssertion if
      * this aggregation pipeline has been interrupted.
      */
-    void checkForInterrupt();
+    void checkForInterrupt() {
+        if (--_interruptCounter == 0) {
+            checkForInterruptSlow();
+        }
+    }
 
     /**
      * Returns true if this is a collectionless aggregation on the specified database.
@@ -190,7 +195,10 @@ public:
         return _collator.get();
     }
 
-    bool shouldCollectExecStats() const {
+    /**
+     * Whether to track timing information and "work" counts in the agg layer.
+     */
+    bool shouldCollectDocumentSourceExecStats() const {
         return static_cast<bool>(explain);
     }
 
@@ -245,13 +253,22 @@ public:
         boost::optional<UUID> uuid = boost::none,
         boost::optional<std::unique_ptr<CollatorInterface>> updatedCollator = boost::none) const;
 
-    boost::intrusive_ptr<ExpressionContext> copyForSubPipeline(NamespaceString nss) const {
+    /**
+     * Returns an ExpressionContext that is identical to 'this' except for the 'subPipelineDepth'
+     * and 'needsMerge' fields.
+     */
+    boost::intrusive_ptr<ExpressionContext> copyForSubPipeline(
+        NamespaceString nss, boost::optional<UUID> uuid = boost::none) const {
         uassert(ErrorCodes::MaxSubPipelineDepthExceeded,
                 str::stream() << "Maximum number of nested sub-pipelines exceeded. Limit is "
-                              << ExpressionContext::kMaxSubPipelineViewDepth,
-                subPipelineDepth < kMaxSubPipelineViewDepth);
-        auto newCopy = copyWith(std::move(nss));
+                              << internalMaxSubPipelineViewDepth.load(),
+                subPipelineDepth < internalMaxSubPipelineViewDepth.load());
+        auto newCopy = copyWith(std::move(nss), uuid);
         newCopy->subPipelineDepth += 1;
+        // The original expCtx might have been attached to an aggregation pipeline running on the
+        // shards. We must reset 'needsMerge' in order to get fully merged results for the
+        // subpipeline.
+        newCopy->needsMerge = false;
         return newCopy;
     }
 
@@ -265,6 +282,15 @@ public:
         return it->second;
     };
 
+    /**
+     * Returns true if there are no namespaces in the query other than the namespace the query was
+     * issued against. eg if there is no $out, $lookup ect. If namespaces have not yet been resolved
+     * then it will also return false.
+     */
+    bool noForeignNamespaces() const {
+        return _resolvedNamespaces.empty();
+    }
+
     /**
      * Convenience call that returns true if the tailableMode indicates a tailable and awaitData
      * query.
@@ -277,8 +303,19 @@ public:
         _resolvedNamespaces = std::move(resolvedNamespaces);
     }
 
-    const RuntimeConstants& getRuntimeConstants() const {
-        return variables.getRuntimeConstants();
+    void addResolvedNamespaces(
+        mongo::stdx::unordered_set<mongo::NamespaceString> resolvedNamespaces) {
+        for (auto&& nss : resolvedNamespaces) {
+            _resolvedNamespaces.try_emplace(nss.coll(), nss, std::vector<BSONObj>{});
+        }
+    }
+
+    void setIsCappedDelete() {
+        _isCappedDelete = true;
+    }
+
+    bool getIsCappedDelete() const {
+        return _isCappedDelete;
     }
 
     /**
@@ -293,15 +330,17 @@ public:
         uassert(31264,
                 "Cannot run server-side javascript without the javascript engine enabled",
                 getGlobalScriptEngine());
-        const auto& runtimeConstants = getRuntimeConstants();
-        const boost::optional<bool> isMapReduceCommand = runtimeConstants.getIsMapReduce();
+        const auto isMapReduce =
+            (variables.hasValue(Variables::kIsMapReduceId) &&
+             variables.getValue(Variables::kIsMapReduceId).getType() == BSONType::Bool &&
+             variables.getValue(Variables::kIsMapReduceId).coerceToBool());
         if (inMongos) {
             invariant(!forceLoadOfStoredProcedures);
-            invariant(!isMapReduceCommand);
+            invariant(!isMapReduce);
         }
 
         // Stored procedures are only loaded for the $where expression and MapReduce command.
-        const bool loadStoredProcedures = forceLoadOfStoredProcedures || isMapReduceCommand;
+        const bool loadStoredProcedures = forceLoadOfStoredProcedures || isMapReduce;
 
         if (hasWhereClause && !loadStoredProcedures) {
             uasserted(4649200,
@@ -309,9 +348,13 @@ public:
                       "$where.");
         }
 
-        const boost::optional<mongo::BSONObj>& scope = runtimeConstants.getJsScope();
-        return JsExecution::get(
-            opCtx, scope.get_value_or(BSONObj()), ns.db(), loadStoredProcedures, jsHeapLimitMB);
+        auto scopeObj = BSONObj();
+        if (variables.hasValue(Variables::kJsScopeId)) {
+            auto scopeVar = variables.getValue(Variables::kJsScopeId);
+            invariant(scopeVar.isObject());
+            scopeObj = scopeVar.getDocument().toBson();
+        }
+        return JsExecution::get(opCtx, scopeObj, ns.db(), loadStoredProcedures, jsHeapLimitMB);
     }
 
     /**
@@ -329,6 +372,16 @@ public:
      */
     void incrementAggExprCounter(StringData name);
 
+    /**
+     * Increment the counter for the $group accumulator expression with a given name.
+     */
+    void incrementGroupAccumulatorExprCounter(StringData name);
+
+    /**
+     * Increment the counter for the $setWindowFields accumulator expression with a given name.
+     */
+    void incrementWindowAccumulatorExprCounter(StringData name);
+
     /**
      * Merge expression counters from the current expression context into the global maps
      * and stop counting.
@@ -364,10 +417,6 @@ public:
     // 'jsHeapLimitMB' server parameter.
     boost::optional<int> jsHeapLimitMB;
 
-    // When set this timeout limits the allowed execution time for a JavaScript function invocation
-    // under any Scope returned by getJsExecWithScope().
-    int jsFnTimeoutMillis;
-
     // An interface for accessing information or performing operations that have different
     // implementations on mongod and mongos, or that only make sense on one of the two.
     // Additionally, putting some of this functionality behind an interface prevents aggregation
@@ -385,22 +434,13 @@ public:
     BSONObj initialPostBatchResumeToken;
 
     // Tracks the depth of nested aggregation sub-pipelines. Used to enforce depth limits.
-    size_t subPipelineDepth = 0;
+    long long subPipelineDepth = 0;
+
+    // True if this 'ExpressionContext' object is for the inner side of a $lookup or $graphLookup.
+    bool inLookup = false;
 
     // If set, this will disallow use of features introduced in versions above the provided version.
-    boost::optional<ServerGlobalParams::FeatureCompatibility::Version>
-        maxFeatureCompatibilityVersion;
-
-    // In a shard mongoD, we use this to determine the format for "sortKey" data. This format has to
-    // match the format that the mongoS expects.
-    // TODO (SERVER-43361): After branching for 4.5, there will be no need to support any format
-    // other than the "k44SortKey" format, so this will be removed.
-    SortKeyFormat sortKeyFormat = SortKeyFormat::k44SortKey;
-
-    // True if this context is associated with a pipeline which is permitted to use the new
-    // upsertSupplied mechanism for applicable $merge modes.
-    // TODO SERVER-44884: remove this when we branch for 4.5.
-    bool useNewUpsert = false;
+    boost::optional<multiversion::FeatureCompatibilityVersion> maxFeatureCompatibilityVersion;
 
     // True if this ExpressionContext is used to parse a view definition pipeline.
     bool isParsingViewDefinition = false;
@@ -415,6 +455,15 @@ public:
     // construction.
     const bool mayDbProfile = true;
 
+    // True if all expressions which use this expression context can be translated into equivalent
+    // SBE expressions.
+    bool sbeCompatible = true;
+
+    // True if all accumulators in the $group stage currently being parsed using this expression
+    // context can be translated into equivalent SBE expressions. This value is transient and gets
+    // reset for every $group stage we parse. Each $group stage has their per-stage flag.
+    bool sbeGroupCompatible = true;
+
     // These fields can be used in a context when API version validations were not enforced during
     // parse time (Example creating a view or validator), but needs to be enforce while querying
     // later.
@@ -429,16 +478,36 @@ public:
     // When non-empty, contains the unmodified user provided aggregation command.
     BSONObj originalAggregateCommand;
 
+    // If present, the spec associated with the current change stream pipeline.
+    boost::optional<DocumentSourceChangeStreamSpec> changeStreamSpec;
+
+    // The resume token version that should be generated by a change stream.
+    int changeStreamTokenVersion = ResumeTokenData::kDefaultTokenVersion;
+
     // True if the expression context is the original one for a given pipeline.
     // False if another context is created for the same pipeline. Used to disable duplicate
     // expression counting.
     bool enabledCounters = true;
 
+    // Returns true if we've received a TemporarilyUnavailableException.
+    bool getTemporarilyUnavailableException() {
+        return _gotTemporarilyUnavailableException;
+    }
+
+    // Sets or clears the flag indicating whether we've received a TemporarilyUnavailableException.
+    void setTemporarilyUnavailableException(bool v) {
+        _gotTemporarilyUnavailableException = v;
+    }
+
 protected:
     static const int kInterruptCheckPeriod = 128;
 
     friend class CollatorStash;
 
+    // Performs the heavy work of checking whether an interrupt has occurred. Should only be called
+    // when _interruptCounter has been decremented to zero.
+    void checkForInterruptSlow();
+
     // Collator used for comparisons.
     std::unique_ptr<CollatorInterface> _collator;
 
@@ -452,8 +521,11 @@ protected:
 
     int _interruptCounter = kInterruptCheckPeriod;
 
+    bool _isCappedDelete = false;
+
 private:
     boost::optional<ExpressionCounters> _expressionCounters = boost::none;
+    bool _gotTemporarilyUnavailableException = false;
 };
 
 }  // namespace mongo
