@@ -27,46 +27,47 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
 
 #include "mongo/db/pipeline/pipeline.h"
+#include "mongo/logv2/log.h"
 
 #include <algorithm>
 
 #include "mongo/base/error_codes.h"
 #include "mongo/db/bson/dotted_path_support.h"
-#include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/accumulator.h"
 #include "mongo/db/pipeline/document_source.h"
-#include "mongo/db/pipeline/document_source_geo_near.h"
 #include "mongo/db/pipeline/document_source_match.h"
 #include "mongo/db/pipeline/document_source_merge.h"
 #include "mongo/db/pipeline/document_source_out.h"
-#include "mongo/db/pipeline/document_source_project.h"
-#include "mongo/db/pipeline/document_source_sort.h"
-#include "mongo/db/pipeline/document_source_unwind.h"
-#include "mongo/db/pipeline/expression.h"
 #include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/lite_parsed_pipeline.h"
+#include "mongo/db/pipeline/search_helper.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/storage/storage_options.h"
 #include "mongo/util/fail_point.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
+
 namespace mongo {
 
 namespace {
 
 // Given a serialized document source, appends execution stats 'nReturned' and
 // 'executionTimeMillisEstimate' to it.
-Value appendExecStats(Value docSource, const CommonStats& stats) {
+Value appendCommonExecStats(Value docSource, const CommonStats& stats) {
     invariant(docSource.getType() == BSONType::Object);
     MutableDocument doc(docSource.getDocument());
     auto nReturned = static_cast<long long>(stats.advanced);
-    auto executionTimeMillisEstimate = static_cast<long long>(stats.executionTimeMillis);
     doc.addField("nReturned", Value(nReturned));
+
+    invariant(stats.executionTime);
+    auto executionTimeMillisEstimate = durationCount<Milliseconds>(*stats.executionTime);
     doc.addField("executionTimeMillisEstimate", Value(executionTimeMillisEstimate));
     return Value(doc.freeze());
 }
@@ -106,7 +107,7 @@ void validateTopLevelPipeline(const Pipeline& pipeline) {
                   firstStageConstraints.isIndependentOfAnyCollection));
 
         // If the first stage is a $changeStream stage, then all stages in the pipeline must be
-        // either $changeStream stages or whitelisted as being able to run in a change stream.
+        // either $changeStream stages or allowlisted as being able to run in a change stream.
         if (firstStageConstraints.isChangeStreamStage()) {
             for (auto&& source : sources) {
                 uassert(ErrorCodes::IllegalOperation,
@@ -116,6 +117,12 @@ void validateTopLevelPipeline(const Pipeline& pipeline) {
             }
         }
     }
+
+    // Verify that usage of $searchMeta and $search is legal.
+    if (pipeline.getContext()->opCtx->getServiceContext()) {
+        getSearchHelpers(pipeline.getContext()->opCtx->getServiceContext())
+            ->assertSearchMetaAccessValid(sources, pipeline.getContext().get());
+    }
 }
 
 }  // namespace
@@ -149,27 +156,26 @@ Pipeline::~Pipeline() {
     invariant(_disposed);
 }
 
-std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::clone() const {
-    const auto& serialized = serializeToBson();
-    try {
-        return parse(serialized, getContext());
-    } catch (DBException& ex) {
-        ex.addContext(str::stream()
-                      << "Failed to copy pipeline. Could not parse serialized version: "
-                      << Value(serialized).toString());
-        throw;
+std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::clone(
+    const boost::intrusive_ptr<ExpressionContext>& newExpCtx) const {
+    SourceContainer clonedStages;
+    for (auto&& stage : _sources) {
+        clonedStages.push_back(stage->clone(newExpCtx));
     }
+    return create(clonedStages, newExpCtx ? newExpCtx : getContext());
 }
 
-std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::parse(
-    const std::vector<BSONObj>& rawPipeline,
-    const intrusive_ptr<ExpressionContext>& expCtx,
-    PipelineValidatorCallback validator) {
+template <class T>
+std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::parseCommon(
+    const std::vector<T>& rawPipeline,
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
+    PipelineValidatorCallback validator,
+    std::function<BSONObj(T)> getElemFunc) {
 
     SourceContainer stages;
 
-    for (auto&& stageObj : rawPipeline) {
-        auto parsedSources = DocumentSource::parse(expCtx, stageObj);
+    for (auto&& stageElem : rawPipeline) {
+        auto parsedSources = DocumentSource::parse(expCtx, getElemFunc(stageElem));
         stages.insert(stages.end(), parsedSources.begin(), parsedSources.end());
     }
 
@@ -185,23 +191,49 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::parse(
     }
 
     // Next run through the common validation rules that apply to every pipeline.
-    pipeline->validateCommon();
+    constexpr bool alreadyOptimized = false;
+    pipeline->validateCommon(alreadyOptimized);
 
     pipeline->stitch();
     return pipeline;
 }
 
+std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::parseFromArray(
+    BSONElement rawPipelineElement,
+    const intrusive_ptr<ExpressionContext>& expCtx,
+    PipelineValidatorCallback validator) {
+
+    tassert(6253719,
+            "Expected array for Pipeline::parseFromArray",
+            rawPipelineElement.type() == BSONType::Array);
+    auto rawStages = rawPipelineElement.Array();
+
+    return parseCommon<BSONElement>(rawStages, expCtx, validator, [](BSONElement e) {
+        uassert(6253720, "Pipeline array element must be an object", e.type() == BSONType::Object);
+        return e.embeddedObject();
+    });
+}
+
+std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::parse(
+    const std::vector<BSONObj>& rawPipeline,
+    const intrusive_ptr<ExpressionContext>& expCtx,
+    PipelineValidatorCallback validator) {
+
+    return parseCommon<BSONObj>(rawPipeline, expCtx, validator, [](BSONObj o) { return o; });
+}
+
 std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::create(
     SourceContainer stages, const intrusive_ptr<ExpressionContext>& expCtx) {
     std::unique_ptr<Pipeline, PipelineDeleter> pipeline(new Pipeline(std::move(stages), expCtx),
                                                         PipelineDeleter(expCtx->opCtx));
 
-    pipeline->validateCommon();
+    constexpr bool alreadyOptimized = false;
+    pipeline->validateCommon(alreadyOptimized);
     pipeline->stitch();
     return pipeline;
 }
 
-void Pipeline::validateCommon() const {
+void Pipeline::validateCommon(bool alreadyOptimized) const {
     size_t i = 0;
 
     uassert(ErrorCodes::FailedToParse,
@@ -215,8 +247,14 @@ void Pipeline::validateCommon() const {
         // Verify that all stages adhere to their PositionRequirement constraints.
         uassert(40602,
                 str::stream() << stage->getSourceName()
-                              << " is only valid as the first stage in a pipeline.",
+                              << " is only valid as the first stage in a pipeline",
                 !(constraints.requiredPosition == PositionRequirement::kFirst && i != 0));
+        uassert(40603,
+                str::stream() << stage->getSourceName()
+                              << " is only valid as the first stage in an optimized pipeline",
+                !(alreadyOptimized &&
+                  constraints.requiredPosition == PositionRequirement::kFirstAfterOptimization &&
+                  i != 0));
 
         auto matchStage = dynamic_cast<DocumentSourceMatch*>(stage.get());
         uassert(17313,
@@ -249,31 +287,32 @@ void Pipeline::optimizePipeline() {
         return;
     }
 
+    optimizeContainer(&_sources);
+}
+
+void Pipeline::optimizeContainer(SourceContainer* container) {
     SourceContainer optimizedSources;
 
-    SourceContainer::iterator itr = _sources.begin();
-
-    // We could be swapping around stages during this process, so disconnect the pipeline to prevent
-    // us from entering a state with dangling pointers.
-    unstitch();
+    SourceContainer::iterator itr = container->begin();
     try {
-        while (itr != _sources.end()) {
+        while (itr != container->end()) {
             invariant((*itr).get());
-            itr = (*itr).get()->optimizeAt(itr, &_sources);
+            itr = (*itr).get()->optimizeAt(itr, container);
         }
 
         // Once we have reached our final number of stages, optimize each individually.
-        for (auto&& source : _sources) {
+        for (auto&& source : *container) {
             if (auto out = source->optimize()) {
                 optimizedSources.push_back(out);
             }
         }
-        _sources.swap(optimizedSources);
+        container->swap(optimizedSources);
     } catch (DBException& ex) {
         ex.addContext("Failed to optimize pipeline");
         throw;
     }
-    stitch();
+
+    stitch(container);
 }
 
 bool Pipeline::aggHasWriteStage(const BSONObj& cmd) {
@@ -335,21 +374,16 @@ bool Pipeline::usedDisk() {
 }
 
 BSONObj Pipeline::getInitialQuery() const {
-    if (_sources.empty())
-        return BSONObj();
-
-    /* look for an initial $match */
-    DocumentSourceMatch* match = dynamic_cast<DocumentSourceMatch*>(_sources.front().get());
-    if (match) {
-        return match->getQuery();
+    if (_sources.empty()) {
+        return BSONObj{};
     }
 
-    DocumentSourceGeoNear* geoNear = dynamic_cast<DocumentSourceGeoNear*>(_sources.front().get());
-    if (geoNear) {
-        return geoNear->getQuery();
+    const DocumentSource* doc = _sources.front().get();
+    if (doc->hasQuery()) {
+        return doc->getQuery();
     }
 
-    return BSONObj();
+    return BSONObj{};
 }
 
 bool Pipeline::needsPrimaryShardMerger() const {
@@ -415,16 +449,21 @@ stdx::unordered_set<NamespaceString> Pipeline::getInvolvedCollections() const {
     return collectionNames;
 }
 
-vector<Value> Pipeline::serialize() const {
+vector<Value> Pipeline::serializeContainer(const SourceContainer& container,
+                                           boost::optional<ExplainOptions::Verbosity> explain) {
     vector<Value> serializedSources;
-    for (auto&& source : _sources) {
-        source->serializeToArray(serializedSources);
+    for (auto&& source : container) {
+        source->serializeToArray(serializedSources, explain);
     }
     return serializedSources;
 }
+vector<Value> Pipeline::serialize(boost::optional<ExplainOptions::Verbosity> explain) const {
+    return serializeContainer(_sources, explain);
+}
 
-vector<BSONObj> Pipeline::serializeToBson() const {
-    const auto serialized = serialize();
+vector<BSONObj> Pipeline::serializeToBson(
+    boost::optional<ExplainOptions::Verbosity> explain) const {
+    const auto serialized = serialize(explain);
     std::vector<BSONObj> asBson;
     asBson.reserve(serialized.size());
     for (auto&& stage : serialized) {
@@ -434,26 +473,8 @@ vector<BSONObj> Pipeline::serializeToBson() const {
     return asBson;
 }
 
-void Pipeline::unstitch() {
-    for (auto&& stage : _sources) {
-        stage->setSource(nullptr);
-    }
-}
-
 void Pipeline::stitch() {
-    if (_sources.empty()) {
-        return;
-    }
-    // Chain together all the stages.
-    DocumentSource* prevSource = _sources.front().get();
-    prevSource->setSource(nullptr);
-    for (SourceContainer::iterator iter(++_sources.begin()), listEnd(_sources.end());
-         iter != listEnd;
-         ++iter) {
-        intrusive_ptr<DocumentSource> pTemp(*iter);
-        pTemp->setSource(prevSource);
-        prevSource = pTemp.get();
-    }
+    stitch(&_sources);
 }
 
 void Pipeline::stitch(SourceContainer* container) {
@@ -494,7 +515,7 @@ vector<Value> Pipeline::writeExplainOps(ExplainOptions::Verbosity verbosity) con
         invariant(afterSize - beforeSize == 1u);
         if (verbosity >= ExplainOptions::Verbosity::kExecStats) {
             auto serializedStage = array.back();
-            array.back() = appendExecStats(serializedStage, stage->getCommonStats());
+            array.back() = appendCommonExecStats(serializedStage, stage->getCommonStats());
         }
     }
     return array;
@@ -514,60 +535,68 @@ void Pipeline::addFinalSource(intrusive_ptr<DocumentSource> source) {
     _sources.push_back(source);
 }
 
-DepsTracker Pipeline::getDependencies(QueryMetadataBitSet unavailableMetadata) const {
-    DepsTracker deps(unavailableMetadata);
-    const bool scopeHasVariables = pCtx->variablesParseState.hasDefinedVariables();
-    bool skipFieldsAndMetadataDeps = false;
+void Pipeline::addVariableRefs(std::set<Variables::Id>* refs) const {
+    for (auto&& source : _sources) {
+        source->addVariableRefs(refs);
+    }
+}
+
+DepsTracker Pipeline::getDependencies(
+    boost::optional<QueryMetadataBitSet> unavailableMetadata) const {
+    return getDependenciesForContainer(getContext(), _sources, unavailableMetadata);
+}
+
+DepsTracker Pipeline::getDependenciesForContainer(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
+    const SourceContainer& container,
+    boost::optional<QueryMetadataBitSet> unavailableMetadata) {
+    // If 'unavailableMetadata' was not specified, we assume all metadata is available. This allows
+    // us to call 'deps.setNeedsMetadata()' without throwing.
+    DepsTracker deps(unavailableMetadata.get_value_or(DepsTracker::kNoMetadata));
+
+    bool hasUnsupportedStage = false;
     bool knowAllFields = false;
     bool knowAllMeta = false;
-    for (auto&& source : _sources) {
+    for (auto&& source : container) {
         DepsTracker localDeps(deps.getUnavailableMetadata());
         DepsTracker::State status = source->getDependencies(&localDeps);
 
-        deps.vars.insert(localDeps.vars.begin(), localDeps.vars.end());
+        deps.needRandomGenerator |= localDeps.needRandomGenerator;
 
-        if ((skipFieldsAndMetadataDeps |= (status == DepsTracker::State::NOT_SUPPORTED))) {
-            // Assume this stage needs everything. We may still know something about our
-            // dependencies if an earlier stage returned EXHAUSTIVE_FIELDS or EXHAUSTIVE_META. If
-            // this scope has variables, we need to keep enumerating the remaining stages but will
-            // skip adding any further field or metadata dependencies.
-            if (scopeHasVariables) {
-                continue;
-            } else {
-                break;
-            }
+        if (status == DepsTracker::State::NOT_SUPPORTED) {
+            // We don't know anything about this stage, so we have to assume it depends on
+            // everything. We may still know something about our dependencies if an earlier stage
+            // returned EXHAUSTIVE_FIELDS or EXHAUSTIVE_META.
+            hasUnsupportedStage = true;
         }
 
-        if (!knowAllFields) {
+        // If we ever saw an unsupported stage, don't bother continuing to track field and metadata
+        // deps: we already have to assume the pipeline depends on everything.
+        if (!hasUnsupportedStage && !knowAllFields) {
             deps.fields.insert(localDeps.fields.begin(), localDeps.fields.end());
             if (localDeps.needWholeDocument)
                 deps.needWholeDocument = true;
             knowAllFields = status & DepsTracker::State::EXHAUSTIVE_FIELDS;
         }
 
-        if (!knowAllMeta) {
+        if (!hasUnsupportedStage && !knowAllMeta) {
             deps.requestMetadata(localDeps.metadataDeps());
             knowAllMeta = status & DepsTracker::State::EXHAUSTIVE_META;
         }
-
-        // If there are variables defined at this pipeline's scope, there may be dependencies upon
-        // them in subsequent stages. Keep enumerating.
-        if (knowAllMeta && knowAllFields && !scopeHasVariables) {
-            break;
-        }
     }
 
     if (!knowAllFields)
         deps.needWholeDocument = true;  // don't know all fields we need
 
-    if (!unavailableMetadata[DocumentMetadataFields::kTextScore]) {
-        // If there is a text score, assume we need to keep it if we can't prove we don't. If we are
-        // the first half of a pipeline which has been split, future stages might need it.
-        if (!knowAllMeta) {
+    if (!deps.getUnavailableMetadata()[DocumentMetadataFields::kTextScore]) {
+        // There is a text score available. If we are the first half of a split pipeline, then we
+        // have to assume future stages might depend on the textScore (unless we've encountered a
+        // stage that doesn't preserve metadata).
+        if (expCtx->needsMerge && !knowAllMeta) {
             deps.setNeedsMetadata(DocumentMetadataFields::kTextScore, true);
         }
     } else {
-        // If there is no text score available, then we don't need to ask for it.
+        // There is no text score available, so we don't need to ask for it.
         deps.setNeedsMetadata(DocumentMetadataFields::kTextScore, false);
     }
 
@@ -585,7 +614,7 @@ Status Pipeline::_pipelineCanRunOnMongoS() const {
         const bool mustWriteToDisk =
             (constraints.diskRequirement == DiskUseRequirement::kWritesPersistentData);
         const bool mayWriteTmpDataAndDiskUseIsAllowed =
-            (pCtx->allowDiskUse && !storageGlobalParams.readOnly &&
+            (pCtx->allowDiskUse && !pCtx->opCtx->readOnly() &&
              constraints.diskRequirement == DiskUseRequirement::kWritesTmpData);
         const bool needsDisk = (mustWriteToDisk || mayWriteTmpDataAndDiskUseIsAllowed);
 
@@ -677,11 +706,72 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::makePipeline(
         pipeline->optimizePipeline();
     }
 
+    constexpr bool alreadyOptimized = true;
+    pipeline->validateCommon(alreadyOptimized);
+
     if (opts.attachCursorSource) {
         pipeline = expCtx->mongoProcessInterface->attachCursorSourceToPipeline(
-            pipeline.release(), opts.allowTargetingShards);
+            pipeline.release(), opts.shardTargetingPolicy, std::move(opts.readConcern));
     }
 
     return pipeline;
 }
+
+Pipeline::SourceContainer::iterator Pipeline::optimizeEndOfPipeline(
+    Pipeline::SourceContainer::iterator itr, Pipeline::SourceContainer* container) {
+    // We must create a new SourceContainer representing the subsection of the pipeline we wish to
+    // optimize, since otherwise calls to optimizeAt() will overrun these limits.
+    auto endOfPipeline = Pipeline::SourceContainer(std::next(itr), container->end());
+    Pipeline::optimizeContainer(&endOfPipeline);
+    container->erase(std::next(itr), container->end());
+    container->splice(std::next(itr), endOfPipeline);
+
+    return std::next(itr);
+}
+
+Pipeline::SourceContainer::iterator Pipeline::optimizeAtEndOfPipeline(
+    Pipeline::SourceContainer::iterator itr, Pipeline::SourceContainer* container) {
+    if (itr == container->end()) {
+        return itr;
+    }
+    itr = std::next(itr);
+    try {
+        while (itr != container->end()) {
+            invariant((*itr).get());
+            itr = (*itr).get()->optimizeAt(itr, container);
+        }
+    } catch (DBException& ex) {
+        ex.addContext("Failed to optimize pipeline");
+        throw;
+    }
+    return itr;
+}
+
+std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::makePipelineFromViewDefinition(
+    const boost::intrusive_ptr<ExpressionContext>& subPipelineExpCtx,
+    ExpressionContext::ResolvedNamespace resolvedNs,
+    std::vector<BSONObj> currentPipeline,
+    MakePipelineOptions opts) {
+
+    // Update subpipeline's ExpressionContext with the resolved namespace.
+    subPipelineExpCtx->ns = resolvedNs.ns;
+
+    if (resolvedNs.pipeline.empty()) {
+        return Pipeline::makePipeline(std::move(currentPipeline), subPipelineExpCtx, opts);
+    }
+    auto resolvedPipeline = std::move(resolvedNs.pipeline);
+
+    // When we get a resolved pipeline back, we may not yet have its namespaces available in the
+    // expression context, e.g. if the view's pipeline contains a $lookup on another collection.
+    LiteParsedPipeline liteParsedPipeline(resolvedNs.ns, resolvedPipeline);
+    subPipelineExpCtx->addResolvedNamespaces(liteParsedPipeline.getInvolvedNamespaces());
+
+    resolvedPipeline.reserve(currentPipeline.size() + resolvedPipeline.size());
+    resolvedPipeline.insert(resolvedPipeline.end(),
+                            std::make_move_iterator(currentPipeline.begin()),
+                            std::make_move_iterator(currentPipeline.end()));
+
+    return Pipeline::makePipeline(std::move(resolvedPipeline), subPipelineExpCtx, opts);
+}
+
 }  // namespace mongo
