@@ -40,6 +40,8 @@
 #include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/dependencies.h"
+#include "mongo/db/pipeline/sharded_agg_helpers_targeting_policy.h"
+#include "mongo/db/query/cursor_response_gen.h"
 #include "mongo/db/query/explain_options.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/executor/task_executor.h"
@@ -69,8 +71,9 @@ using PipelineValidatorCallback = std::function<void(const Pipeline&)>;
 struct MakePipelineOptions {
     bool optimize = true;
     bool attachCursorSource = true;
-    bool allowTargetingShards = true;
+    ShardTargetingPolicy shardTargetingPolicy = ShardTargetingPolicy::kAllowed;
     PipelineValidatorCallback validator = nullptr;
+    boost::optional<BSONObj> readConcern;
 };
 
 /**
@@ -122,6 +125,15 @@ public:
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
         PipelineValidatorCallback validator = nullptr);
 
+    /**
+     * Like parse, but takes a BSONElement instead of a vector of objects. 'arrElem' must be an
+     * array of objects.
+     */
+    static std::unique_ptr<Pipeline, PipelineDeleter> parseFromArray(
+        BSONElement arrayElem,
+        const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        PipelineValidatorCallback validator = nullptr);
+
     /**
      * Creates a Pipeline from an existing SourceContainer.
      *
@@ -148,27 +160,53 @@ public:
     static std::unique_ptr<Pipeline, PipelineDeleter> makePipeline(
         const std::vector<BSONObj>& rawPipeline,
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
-        const MakePipelineOptions opts = MakePipelineOptions{});
+        MakePipelineOptions opts = MakePipelineOptions{});
 
-    std::unique_ptr<Pipeline, PipelineDeleter> clone() const;
+    /**
+     * Optimize the given pipeline after the stage that 'itr' points to.
+     *
+     * Returns a valid iterator that points to the new "end of the pipeline": i.e., the stage that
+     * comes after 'itr' in the newly optimized pipeline.
+     */
+    static Pipeline::SourceContainer::iterator optimizeEndOfPipeline(
+        Pipeline::SourceContainer::iterator itr, Pipeline::SourceContainer* container);
+
+    /**
+     * Applies optimizeAt() to all stages in the given pipeline after the stage that 'itr' points
+     * to.
+     *
+     * Returns a valid iterator that points to the new "end of the pipeline": i.e., the stage that
+     * comes after 'itr' in the newly optimized pipeline.
+     */
+    static Pipeline::SourceContainer::iterator optimizeAtEndOfPipeline(
+        Pipeline::SourceContainer::iterator itr, Pipeline::SourceContainer* container);
+
+    static std::unique_ptr<Pipeline, PipelineDeleter> makePipelineFromViewDefinition(
+        const boost::intrusive_ptr<ExpressionContext>& subPipelineExpCtx,
+        ExpressionContext::ResolvedNamespace resolvedNs,
+        std::vector<BSONObj> currentPipeline,
+        MakePipelineOptions opts);
+
+    /**
+     * Callers can optionally specify 'newExpCtx' to construct the deep clone with it. This will be
+     * used to construct all the cloned DocumentSources as well.
+     */
+    std::unique_ptr<Pipeline, PipelineDeleter> clone(
+        const boost::intrusive_ptr<ExpressionContext>& = nullptr) const;
 
     const boost::intrusive_ptr<ExpressionContext>& getContext() const {
         return pCtx;
     }
 
     /**
-     * Sets the OperationContext of 'pCtx' to nullptr.
-     *
-     * The PipelineProxyStage is responsible for detaching the OperationContext and releasing any
-     * storage-engine state of the DocumentSourceCursor that may be present in '_sources'.
+     * Sets the OperationContext of 'pCtx' to nullptr and calls 'detachFromOperationContext()' on
+     * all underlying DocumentSources.
      */
     void detachFromOperationContext();
 
     /**
-     * Sets the OperationContext of 'pCtx' to 'opCtx'.
-     *
-     * The PipelineProxyStage is responsible for reattaching the OperationContext and reacquiring
-     * any storage-engine state of the DocumentSourceCursor that may be present in '_sources'.
+     * Sets the OperationContext of 'pCtx' to 'opCtx', and reattaches all underlying DocumentSources
+     * to 'opCtx'.
      */
     void reattachToOperationContext(OperationContext* opCtx);
 
@@ -186,6 +224,10 @@ public:
      */
     void dispose(OperationContext* opCtx);
 
+    bool isDisposed() const {
+        return _disposed;
+    }
+
     /**
      * Checks to see if disk is ever used within the pipeline.
      */
@@ -237,6 +279,11 @@ public:
      */
     void optimizePipeline();
 
+    /**
+     * Modifies the container, optimizing it by combining and swapping stages.
+     */
+    static void optimizeContainer(SourceContainer* container);
+
     /**
      * Returns any other collections involved in the pipeline in addition to the collection the
      * aggregation is run on. All namespaces returned are the names of collections, after views have
@@ -245,10 +292,14 @@ public:
     stdx::unordered_set<NamespaceString> getInvolvedCollections() const;
 
     /**
-     * Serializes the pipeline into a form that can be parsed into an equivalent pipeline.
+     * Helpers to serialize a pipeline.
      */
-    std::vector<Value> serialize() const;
-    std::vector<BSONObj> serializeToBson() const;
+    std::vector<Value> serialize(
+        boost::optional<ExplainOptions::Verbosity> explain = boost::none) const;
+    std::vector<BSONObj> serializeToBson(
+        boost::optional<ExplainOptions::Verbosity> explain = boost::none) const;
+    static std::vector<Value> serializeContainer(
+        const SourceContainer& container, boost::optional<ExplainOptions::Verbosity> = boost::none);
 
     // The initial source is special since it varies between mongos and mongod.
     void addInitialSource(boost::intrusive_ptr<DocumentSource> source);
@@ -268,9 +319,28 @@ public:
 
     /**
      * Returns the dependencies needed by this pipeline. 'unavailableMetadata' should reflect what
-     * metadata is not present on documents that are input to the front of the pipeline.
+     * metadata is not present on documents that are input to the front of the pipeline. If
+     * 'unavailableMetadata' is specified, this method will throw if any of the dependencies
+     * reference unavailable metadata.
      */
-    DepsTracker getDependencies(QueryMetadataBitSet unavailableMetadata) const;
+    DepsTracker getDependencies(boost::optional<QueryMetadataBitSet> unavailableMetadata) const;
+
+    /**
+     * Populate 'refs' with the variables referred to by this pipeline, including user and system
+     * variables but excluding $$ROOT. Note that field path references are not considered variables.
+     */
+    void addVariableRefs(std::set<Variables::Id>* refs) const;
+
+    /**
+     * Returns the dependencies needed by the SourceContainer. 'unavailableMetadata' should reflect
+     * what metadata is not present on documents that are input to the front of the pipeline. If
+     * 'unavailableMetadata' is specified, this method will throw if any of the dependencies
+     * reference unavailable metadata.
+     */
+    static DepsTracker getDependenciesForContainer(
+        const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        const SourceContainer& container,
+        boost::optional<QueryMetadataBitSet> unavailableMetadata);
 
     const SourceContainer& getSources() const {
         return _sources;
@@ -322,6 +392,16 @@ public:
     boost::intrusive_ptr<DocumentSource> popFrontWithNameAndCriteria(
         StringData targetStageName, std::function<bool(const DocumentSource* const)> predicate);
 
+    /**
+     * Performs common validation for top-level or facet pipelines. Throws if the pipeline is
+     * invalid.
+     *
+     * Includes checking for illegal stage positioning. For example, $out must be at the end, while
+     * a $match stage with a text query must be at the start. Note that this method accepts an
+     * initial source as the first stage, which is illegal for $facet pipelines.
+     */
+    void validateCommon(bool alreadyOptimized) const;
+
     /**
      * PipelineD is a "sister" class that has additional functionality for the Pipeline. It exists
      * because of linkage requirements. Pipeline needs to function in mongod and mongos. PipelineD
@@ -331,6 +411,21 @@ public:
      */
     friend class PipelineD;
 
+    /**
+     * For commands that return multiple pipelines, this value will contain the type of pipeline.
+     * This can be populated to the cursor so consumers do not have to depend on order or guess
+     * which pipeline is which. Default to a regular result pipeline.
+     */
+    CursorTypeEnum pipelineType = CursorTypeEnum::DocumentResult;
+
+    /**
+     * Get a string representation of the pipeline type.
+     */
+    auto getTypeString() {
+        return CursorType_serializer(pipelineType);
+    }
+
+
 private:
     friend class PipelineDeleter;
 
@@ -339,6 +434,16 @@ private:
 
     ~Pipeline();
 
+    /**
+     * Helper for public methods that parse pipelines from vectors of different types.
+     */
+    template <class T>
+    static std::unique_ptr<Pipeline, PipelineDeleter> parseCommon(
+        const std::vector<T>& rawPipeline,
+        const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        PipelineValidatorCallback validator,
+        std::function<BSONObj(T)> getElemFunc);
+
     /**
      * Stitch together the source pointers by calling setSource() for each source in '_sources'.
      * This function must be called any time the order of stages within the pipeline changes, e.g.
@@ -346,22 +451,6 @@ private:
      */
     void stitch();
 
-    /**
-     * Reset all stages' child pointers to nullptr. Used to prevent dangling pointers during the
-     * optimization process, where we might swap or destroy stages.
-     */
-    void unstitch();
-
-    /**
-     * Performs common validation for top-level or facet pipelines. Throws if the pipeline is
-     * invalid.
-     *
-     * Includes checking for illegal stage positioning. For example, $out must be at the end, while
-     * a $match stage with a text query must be at the start. Note that this method accepts an
-     * initial source as the first stage, which is illegal for $facet pipelines.
-     */
-    void validateCommon() const;
-
     /**
      * Returns Status::OK if the pipeline can run on mongoS, or an error with a message explaining
      * why it cannot.
@@ -415,4 +504,15 @@ private:
     bool _dismissed = false;
 };
 
+/**
+ * A 'ServiceContext' decorator that by default does nothing but can be set to generate a
+ * complimentary, metadata pipeline to the one passed in.
+ */
+extern ServiceContext::Decoration<std::unique_ptr<Pipeline, PipelineDeleter> (*)(
+    OperationContext* opCtx,
+    boost::intrusive_ptr<ExpressionContext> expCtx,
+    const AggregateCommandRequest& request,
+    Pipeline* origPipeline,
+    boost::optional<UUID> uuid)>
+    generateMetadataPipelineFunc;
 }  // namespace mongo
