@@ -41,7 +41,7 @@
 #include "mongo/bson/json.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/aggregation_context_fixture.h"
-#include "mongo/db/pipeline/aggregation_request.h"
+#include "mongo/db/pipeline/aggregation_request_helper.h"
 #include "mongo/db/pipeline/document_source_bucket_auto.h"
 #include "mongo/db/pipeline/document_source_facet.h"
 #include "mongo/db/pipeline/document_source_graph_lookup.h"
@@ -58,16 +58,6 @@
 #include "mongo/unittest/temp_dir.h"
 #include "mongo/unittest/unittest.h"
 
-#define ASSERT_DOES_NOT_THROW(EXPRESSION)                                          \
-    try {                                                                          \
-        EXPRESSION;                                                                \
-    } catch (const AssertionException& e) {                                        \
-        ::mongo::str::stream err;                                                  \
-        err << "Threw an exception incorrectly: " << e.toString()                  \
-            << " Exception occured in: " << #EXPRESSION;                           \
-        ::mongo::unittest::TestAssertionFailure(__FILE__, __LINE__, err).stream(); \
-    }
-
 namespace mongo {
 namespace {
 
@@ -77,17 +67,16 @@ protected:
         const auto inputBson = fromjson("{pipeline: " + jsonArray + "}");
 
         ASSERT_EQUALS(inputBson["pipeline"].type(), BSONType::Array);
-        auto rawPipeline =
-            uassertStatusOK(AggregationRequest::parsePipelineFromBSON(inputBson["pipeline"]));
+        auto rawPipeline = parsePipelineFromBSON(inputBson["pipeline"]);
         NamespaceString testNss("test", "collection");
-        AggregationRequest request(testNss, rawPipeline);
+        AggregateCommandRequest request(testNss, rawPipeline);
         getExpCtx()->ns = testNss;
 
         return Pipeline::parse(request.getPipeline(), getExpCtx());
     }
 
     template <typename T, typename... Args>
-    std::vector<T> make_vector(Args&&... args) {
+    std::vector<T> makeVector(Args&&... args) {
         std::vector<T> v;
         v.reserve(sizeof...(Args));
         (v.push_back(std::forward<Args>(args)), ...);
@@ -130,7 +119,7 @@ TEST_F(PipelineMetadataTreeTest, LinearPipelinesConstructProperTrees) {
         return makeTree<TestThing>(
             {{NamespaceString("test.collection"), initial}}, *pipePtr, ignoreDocumentSourceAddOne);
     }()
-               .first.get() == Stage(TestThing{23}, {}, {}));
+               .first.value() == Stage(TestThing{23}, {}, {}));
 
     ASSERT([&]() {
         auto pipePtr = jsonToPipeline(
@@ -139,7 +128,7 @@ TEST_F(PipelineMetadataTreeTest, LinearPipelinesConstructProperTrees) {
         return makeTree<TestThing>(
             {{NamespaceString("test.collection"), initial}}, *pipePtr, ignoreDocumentSourceAddOne);
     }()
-               .first.get() == Stage(TestThing{24}, makeUniqueStage(TestThing{23}, {}, {}), {}));
+               .first.value() == Stage(TestThing{24}, makeUniqueStage(TestThing{23}, {}, {}), {}));
 
     ASSERT([&]() {
         auto pipePtr = jsonToPipeline(
@@ -152,7 +141,7 @@ TEST_F(PipelineMetadataTreeTest, LinearPipelinesConstructProperTrees) {
         return makeTree<TestThing>(
             {{NamespaceString("test.collection"), initial}}, *pipePtr, ignoreDocumentSourceAddOne);
     }()
-               .first.get() ==
+               .first.value() ==
            Stage(TestThing{28},
                  makeUniqueStage(
                      TestThing{27},
@@ -202,7 +191,7 @@ TEST_F(PipelineMetadataTreeTest, BranchingPipelinesConstructProperTrees) {
         if (typeid(source) == typeid(DocumentSourceGroup))
             return {previousThing.string + "g"};
         if (auto lookupSource = dynamic_cast<const DocumentSourceLookUp*>(&source)) {
-            if (lookupSource->wasConstructedWithPipelineSyntax())
+            if (lookupSource->hasPipeline())
                 return {previousThing.string + "l[" + extraThings.front().string + "]"};
             else
                 return {previousThing.string + "l"};
@@ -251,7 +240,7 @@ TEST_F(PipelineMetadataTreeTest, BranchingPipelinesConstructProperTrees) {
                                    *pipePtr,
                                    buildRepresentativeString);
     }()
-               .first.get() ==
+               .first.value() ==
            Stage(TestThing{"1mpxul[2m]ulu"},
                  makeUniqueStage(
                      TestThing{"1mpxul[2m]ul"},
@@ -270,7 +259,7 @@ TEST_F(PipelineMetadataTreeTest, BranchingPipelinesConstructProperTrees) {
                                                          {}),
                                          {}),
                                      {}),
-                                 make_vector<Stage<TestThing>>(Stage(TestThing{"2"}, {}, {}))),
+                                 makeVector<Stage<TestThing>>(Stage(TestThing{"2"}, {}, {}))),
                              {}),
                          {}),
                      {}),
@@ -288,12 +277,12 @@ TEST_F(PipelineMetadataTreeTest, BranchingPipelinesConstructProperTrees) {
         return makeTree<TestThing>(
             {{NamespaceString("test.collection"), {""}}}, *pipePtr, buildRepresentativeString);
     }()
-               .first.get() ==
+               .first.value() ==
            Stage(TestThing{"f[tugs, tmgs, tb]"},
                  makeUniqueStage(
                      TestThing{""},
                      {},
-                     make_vector<Stage<TestThing>>(
+                     makeVector<Stage<TestThing>>(
                          Stage(TestThing{"tug"},
                                makeUniqueStage(
                                    TestThing{"tu"},
@@ -367,7 +356,7 @@ TEST_F(PipelineMetadataTreeTest, ZipWalksAPipelineAndTreeInTandemAndInOrder) {
                                         *pipePtr,
                                         takeTypeInfo)
                         .first;
-        zip<TestThing>(&tree.get(), &*pipePtr, tookTypeInfoOrThrow);
+        zip<TestThing>(&tree.value(), &*pipePtr, tookTypeInfoOrThrow);
         previousStack.pop();
     }());
 
@@ -385,7 +374,7 @@ TEST_F(PipelineMetadataTreeTest, ZipWalksAPipelineAndTreeInTandemAndInOrder) {
                                         *pipePtr,
                                         takeTypeInfo)
                         .first;
-        zip<TestThing>(&tree.get(), &*pipePtr, tookTypeInfoOrThrow);
+        zip<TestThing>(&tree.value(), &*pipePtr, tookTypeInfoOrThrow);
         previousStack.pop();
     }());
 }
