@@ -27,12 +27,12 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
-
-#include "mongo/platform/basic.h"
 
 #include "mongo/db/pipeline/process_interface/common_mongod_process_interface.h"
 
+#include <algorithm>
+#include <vector>
+
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
@@ -44,77 +44,42 @@
 #include "mongo/db/catalog/list_indexes.h"
 #include "mongo/db/catalog/rename_collection.h"
 #include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/cursor_manager.h"
 #include "mongo/db/db_raii.h"
+#include "mongo/db/dbhelpers.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/pipeline/document_source_cursor.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
 #include "mongo/db/pipeline/pipeline_d.h"
+#include "mongo/db/query/collection_index_usage_tracker_decoration.h"
 #include "mongo/db/query/collection_query_info.h"
-#include "mongo/db/repl/speculative_majority_read_info.h"
-#include "mongo/db/s/collection_sharding_state.h"
+#include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/query/sbe_plan_cache.h"
+#include "mongo/db/repl/primary_only_service.h"
 #include "mongo/db/s/sharding_state.h"
 #include "mongo/db/s/transaction_coordinator_curop.h"
 #include "mongo/db/s/transaction_coordinator_worker_curop_repository.h"
-#include "mongo/db/session_catalog.h"
-#include "mongo/db/session_catalog_mongod.h"
+#include "mongo/db/server_feature_flags_gen.h"
+#include "mongo/db/session/session_catalog.h"
 #include "mongo/db/stats/fill_locker_info.h"
 #include "mongo/db/stats/storage_stats.h"
 #include "mongo/db/storage/backup_cursor_hooks.h"
 #include "mongo/db/storage/durable_catalog.h"
-#include "mongo/db/transaction_history_iterator.h"
-#include "mongo/db/transaction_participant.h"
+#include "mongo/db/transaction/transaction_history_iterator.h"
+#include "mongo/db/transaction/transaction_participant.h"
+#include "mongo/db/transaction/transaction_participant_resource_yielder.h"
 #include "mongo/logv2/log.h"
 #include "mongo/s/cluster_commands_helpers.h"
 #include "mongo/s/query/document_source_merge_cursors.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
+
+
 namespace mongo {
-
 namespace {
 
-class MongoDResourceYielder : public ResourceYielder {
-public:
-    void yield(OperationContext* opCtx) override {
-        // We're about to block. Check back in the session so that it's available to other
-        // threads. Note that we may block on a request to _ourselves_, meaning that we may have to
-        // wait for another thread which will use the same session. This step is necessary
-        // to prevent deadlocks.
-
-        Session* const session = OperationContextSession::get(opCtx);
-        if (session) {
-            if (auto txnParticipant = TransactionParticipant::get(opCtx)) {
-                txnParticipant.stashTransactionResources(opCtx);
-            }
-
-            MongoDOperationContextSession::checkIn(opCtx);
-        }
-        _yielded = (session != nullptr);
-    }
-
-    void unyield(OperationContext* opCtx) override {
-        if (_yielded) {
-            // This may block on a sub-operation on this node finishing. It's possible that while
-            // blocked on the network layer, another shard could have responded, theoretically
-            // unblocking this thread of execution. However, we must wait until the child operation
-            // on this shard finishes so we can get the session back. This may limit the throughput
-            // of the operation, but it's correct.
-            MongoDOperationContextSession::checkOut(opCtx);
-
-            if (auto txnParticipant = TransactionParticipant::get(opCtx)) {
-                // Assumes this is only called from the 'aggregate' or 'getMore' commands.  The code
-                // which relies on this parameter does not distinguish/care about the difference so
-                // we simply always pass 'aggregate'.
-                txnParticipant.unstashTransactionResources(opCtx, "aggregate");
-            }
-        }
-    }
-
-private:
-    bool _yielded = false;
-};
-
 // Returns true if the field names of 'keyPattern' are exactly those in 'uniqueKeyPaths', and each
 // of the elements of 'keyPattern' is numeric, i.e. not "text", "$**", or any other special type of
 // index.
@@ -141,6 +106,59 @@ bool supportsUniqueKey(const boost::intrusive_ptr<ExpressionContext>& expCtx,
             CollatorInterface::collatorsMatch(index->getCollator(), expCtx->getCollator()));
 }
 
+// Proactively assert that this operation can safely write before hitting an assertion in the
+// storage engine. We can safely write if we are enforcing prepare conflicts by blocking or if we
+// are ignoring prepare conflicts and explicitly allowing writes. Ignoring prepare conflicts
+// without allowing writes will cause this operation to fail in the storage engine.
+void assertIgnorePrepareConflictsBehavior(const boost::intrusive_ptr<ExpressionContext>& expCtx) {
+    tassert(5996900,
+            "Expected operation to either be blocking on prepare conflicts or ignoring prepare "
+            "conflicts and allowing writes",
+            expCtx->opCtx->recoveryUnit()->getPrepareConflictBehavior() !=
+                PrepareConflictBehavior::kIgnoreConflicts);
+}
+
+/**
+ * Returns all documents from _mdb_catalog along with a sorted list of all
+ * <db>.system.views namespaces found.
+ */
+void listDurableCatalog(OperationContext* opCtx,
+                        StringData shardName,
+                        std::deque<BSONObj>* docs,
+                        std::vector<NamespaceStringOrUUID>* systemViewsNamespaces) {
+    auto durableCatalog = DurableCatalog::get(opCtx);
+    auto rs = durableCatalog->getRecordStore();
+    if (!rs) {
+        return;
+    }
+
+    auto cursor = rs->getCursor(opCtx);
+    while (auto record = cursor->next()) {
+        BSONObj obj = record->data.releaseToBson();
+
+        // For backwards compatibility where older version have a written feature document.
+        // See SERVER-57125.
+        if (DurableCatalog::isFeatureDocument(obj)) {
+            continue;
+        }
+
+        NamespaceString ns(obj.getStringField("ns"));
+        if (ns.isSystemDotViews()) {
+            systemViewsNamespaces->push_back(ns);
+        }
+
+        BSONObjBuilder builder;
+        builder.append("db", ns.db());
+        builder.append("name", ns.coll());
+        builder.append("type", "collection");
+        if (!shardName.empty()) {
+            builder.append("shard", shardName);
+        }
+        builder.appendElements(obj);
+        docs->push_back(builder.obj());
+    }
+}
+
 }  // namespace
 
 std::unique_ptr<TransactionHistoryIteratorBase>
@@ -154,9 +172,8 @@ std::vector<Document> CommonMongodProcessInterface::getIndexStats(OperationConte
                                                                   const NamespaceString& ns,
                                                                   StringData host,
                                                                   bool addShardName) {
-    AutoGetCollectionForReadCommand autoColl(opCtx, ns);
+    AutoGetCollectionForReadCommandMaybeLockFree collection(opCtx, ns);
 
-    Collection* collection = autoColl.getCollection();
     std::vector<Document> indexStats;
     if (!collection) {
         LOGV2_DEBUG(23881,
@@ -166,16 +183,18 @@ std::vector<Document> CommonMongodProcessInterface::getIndexStats(OperationConte
         return indexStats;
     }
 
-    auto indexStatsMap = CollectionQueryInfo::get(collection).getIndexUsageStats();
-    for (auto&& indexStatsMapIter : indexStatsMap) {
+    auto indexStatsMap =
+        CollectionIndexUsageTrackerDecoration::get(collection->getSharedDecorations())
+            .getUsageStats();
+    for (auto&& indexStatsMapIter : *indexStatsMap) {
         auto indexName = indexStatsMapIter.first;
         auto stats = indexStatsMapIter.second;
         MutableDocument doc;
         doc["name"] = Value(indexName);
-        doc["key"] = Value(stats.indexKey);
+        doc["key"] = Value(stats->indexKey);
         doc["host"] = Value(host);
-        doc["accesses"]["ops"] = Value(stats.accesses.loadRelaxed());
-        doc["accesses"]["since"] = Value(stats.trackerStartTime);
+        doc["accesses"]["ops"] = Value(stats->accesses.loadRelaxed());
+        doc["accesses"]["since"] = Value(stats->trackerStartTime);
 
         if (addShardName)
             doc["shard"] = Value(getShardName(opCtx));
@@ -192,7 +211,14 @@ std::vector<Document> CommonMongodProcessInterface::getIndexStats(OperationConte
         auto entry = idxCatalog->getEntry(idx);
         doc["spec"] = Value(idx->infoObj());
 
-        if (!entry->isReady(opCtx)) {
+        // Not all indexes in the CollectionIndexUsageTracker may be visible or consistent with our
+        // snapshot. For this reason, it is unsafe to check `isReady` on the entry, which
+        // asserts that the index's in-memory state is consistent with our snapshot.
+        if (!entry->isPresentInMySnapshot(opCtx)) {
+            continue;
+        }
+
+        if (!entry->isReadyInMySnapshot(opCtx)) {
             doc["building"] = Value(true);
         }
 
@@ -201,6 +227,121 @@ std::vector<Document> CommonMongodProcessInterface::getIndexStats(OperationConte
     return indexStats;
 }
 
+std::deque<BSONObj> CommonMongodProcessInterface::listCatalog(OperationContext* opCtx) const {
+    while (true) {
+        std::deque<BSONObj> docs;
+        std::vector<NamespaceStringOrUUID> systemViewsNamespaces;
+        {
+            Lock::GlobalLock globalLock(opCtx, MODE_IS);
+            listDurableCatalog(opCtx, getShardName(opCtx), &docs, &systemViewsNamespaces);
+        }
+
+        if (systemViewsNamespaces.empty()) {
+            return docs;
+        }
+
+        // Clear 'docs' because we will read _mdb_catalog again using a consistent snapshot for all
+        // the system.views collections.
+        docs.clear();
+
+        // We want to read all the system.views as well as _mdb_catalog (again) using a consistent
+        // snapshot.
+        // TODO(SERVER-63754): Replace with a less verbose constructor overload when available.
+        AutoGetCollectionForReadCommandMaybeLockFree collLock(
+            opCtx,
+            systemViewsNamespaces.front(),
+            AutoGetCollectionViewMode::kViewsForbidden,
+            Date_t::max(),
+            AutoStatsTracker::LogMode::kUpdateTopAndCurOp,
+            {++systemViewsNamespaces.cbegin(), systemViewsNamespaces.cend()});
+
+        // If the primary collection is not available, it means the information from parsing
+        // _mdb_catalog is no longer valid. Therefore, we restart this process from the top.
+        if (!collLock.getCollection()) {
+            continue;
+        }
+
+        // Read _mdb_catalog again using the same snapshot set up by our collection(s) lock helper.
+        // If _mdb_catalog contains a different set of system.views namespaces from the first time
+        // we read it, we should discard this set of results and retry from the top (with the
+        // global read lock) of this loop.
+        std::vector<NamespaceStringOrUUID> systemViewsNamespacesFromSecondCatalogRead;
+        listDurableCatalog(
+            opCtx, getShardName(opCtx), &docs, &systemViewsNamespacesFromSecondCatalogRead);
+        if (!std::equal(
+                systemViewsNamespaces.cbegin(),
+                systemViewsNamespaces.cend(),
+                systemViewsNamespacesFromSecondCatalogRead.cbegin(),
+                [](const auto& lhs, const auto& rhs) { return *lhs.nss() == *rhs.nss(); })) {
+            continue;
+        }
+
+        for (const auto& svns : systemViewsNamespaces) {
+            auto collection = CollectionCatalog::get(opCtx)->lookupCollectionByNamespaceForRead(
+                opCtx, *svns.nss());
+            if (!collection) {
+                continue;
+            }
+
+            auto cursor = collection->getCursor(opCtx);
+            while (auto record = cursor->next()) {
+                BSONObj obj = record->data.releaseToBson();
+
+                NamespaceString ns(obj.getStringField("_id"));
+                NamespaceString viewOnNs(ns.db(), obj.getStringField("viewOn"));
+
+                BSONObjBuilder builder;
+                builder.append("db", ns.db());
+                builder.append("name", ns.coll());
+                if (viewOnNs.isTimeseriesBucketsCollection()) {
+                    builder.append("type", "timeseries");
+                } else {
+                    builder.append("type", "view");
+                }
+                if (auto shardName = getShardName(opCtx); !shardName.empty()) {
+                    builder.append("shard", shardName);
+                }
+                builder.appendAs(obj["_id"], "ns");
+                builder.appendElements(obj);
+                docs.push_back(builder.obj());
+            }
+        }
+
+        return docs;
+    }
+}
+
+boost::optional<BSONObj> CommonMongodProcessInterface::getCatalogEntry(
+    OperationContext* opCtx, const NamespaceString& ns) const {
+    Lock::GlobalLock globalLock{opCtx, MODE_IS};
+
+    auto rs = DurableCatalog::get(opCtx)->getRecordStore();
+    if (!rs) {
+        return boost::none;
+    }
+
+    auto cursor = rs->getCursor(opCtx);
+    while (auto record = cursor->next()) {
+        auto obj = record->data.toBson();
+        if (NamespaceString{obj.getStringField("ns")} != ns) {
+            continue;
+        }
+
+        BSONObjBuilder builder;
+        builder.append("db", ns.db());
+        builder.append("name", ns.coll());
+        builder.append("type", "collection");
+        if (auto shardName = getShardName(opCtx); !shardName.empty()) {
+            builder.append("shard", shardName);
+        }
+        builder.appendElements(obj);
+
+        return builder.obj();
+    }
+
+    return boost::none;
+}
+
 void CommonMongodProcessInterface::appendLatencyStats(OperationContext* opCtx,
                                                       const NamespaceString& nss,
                                                       bool includeHistograms,
@@ -210,9 +351,9 @@ void CommonMongodProcessInterface::appendLatencyStats(OperationContext* opCtx,
 
 Status CommonMongodProcessInterface::appendStorageStats(OperationContext* opCtx,
                                                         const NamespaceString& nss,
-                                                        const BSONObj& param,
+                                                        const StorageStatsSpec& spec,
                                                         BSONObjBuilder* builder) const {
-    return appendCollectionStorageStats(opCtx, nss, param, builder);
+    return appendCollectionStorageStats(opCtx, nss, spec, builder);
 }
 
 Status CommonMongodProcessInterface::appendRecordCount(OperationContext* opCtx,
@@ -224,21 +365,15 @@ Status CommonMongodProcessInterface::appendRecordCount(OperationContext* opCtx,
 Status CommonMongodProcessInterface::appendQueryExecStats(OperationContext* opCtx,
                                                           const NamespaceString& nss,
                                                           BSONObjBuilder* builder) const {
-    AutoGetCollectionForReadCommand autoColl(opCtx, nss);
-
-    if (!autoColl.getDb()) {
-        return {ErrorCodes::NamespaceNotFound,
-                str::stream() << "Database [" << nss.db().toString() << "] not found."};
-    }
-
-    Collection* collection = autoColl.getCollection();
-
+    AutoGetCollectionForReadCommand collection(opCtx, nss);
     if (!collection) {
         return {ErrorCodes::NamespaceNotFound,
                 str::stream() << "Collection [" << nss.toString() << "] not found."};
     }
 
-    auto collectionScanStats = CollectionQueryInfo::get(collection).getCollectionScanStats();
+    auto collectionScanStats =
+        CollectionIndexUsageTrackerDecoration::get(collection->getSharedDecorations())
+            .getCollectionScanStats();
 
     dassert(collectionScanStats.collectionScans <=
             static_cast<unsigned long long>(std::numeric_limits<long long>::max()));
@@ -254,24 +389,23 @@ Status CommonMongodProcessInterface::appendQueryExecStats(OperationContext* opCt
     return Status::OK();
 }
 
-BSONObj CommonMongodProcessInterface::getCollectionOptions(OperationContext* opCtx,
-                                                           const NamespaceString& nss) {
-    AutoGetCollectionForReadCommand autoColl(opCtx, nss);
+BSONObj CommonMongodProcessInterface::getCollectionOptionsLocally(OperationContext* opCtx,
+                                                                  const NamespaceString& nss) {
+    AutoGetCollectionForReadCommand collection(opCtx, nss);
     BSONObj collectionOptions = {};
-    if (!autoColl.getDb()) {
-        return collectionOptions;
-    }
-    Collection* collection = autoColl.getCollection();
     if (!collection) {
         return collectionOptions;
     }
 
-    collectionOptions = DurableCatalog::get(opCtx)
-                            ->getCollectionOptions(opCtx, collection->getCatalogId())
-                            .toBSON();
+    collectionOptions = collection->getCollectionOptions().toBSON();
     return collectionOptions;
 }
 
+BSONObj CommonMongodProcessInterface::getCollectionOptions(OperationContext* opCtx,
+                                                           const NamespaceString& nss) {
+    return getCollectionOptionsLocally(opCtx, nss);
+}
+
 std::unique_ptr<Pipeline, PipelineDeleter>
 CommonMongodProcessInterface::attachCursorSourceToPipelineForLocalRead(Pipeline* ownedPipeline) {
     auto expCtx = ownedPipeline->getContext();
@@ -287,18 +421,30 @@ CommonMongodProcessInterface::attachCursorSourceToPipelineForLocalRead(Pipeline*
         return pipeline;
     }
 
-    boost::optional<AutoGetCollectionForReadCommand> autoColl;
+    boost::optional<AutoGetCollectionForReadCommandMaybeLockFree> autoColl;
     const NamespaceStringOrUUID nsOrUUID = expCtx->uuid
         ? NamespaceStringOrUUID{expCtx->ns.db().toString(), *expCtx->uuid}
         : expCtx->ns;
+
+    // Reparse 'pipeline' to discover whether there are secondary namespaces that we need to lock
+    // when constructing our query executor.
+    auto lpp = LiteParsedPipeline(expCtx->ns, pipeline->serializeToBson());
+    std::vector<NamespaceStringOrUUID> secondaryNamespaces = lpp.getForeignExecutionNamespaces();
+
     autoColl.emplace(expCtx->opCtx,
                      nsOrUUID,
-                     AutoGetCollection::ViewMode::kViewsForbidden,
+                     AutoGetCollectionViewMode::kViewsForbidden,
                      Date_t::max(),
-                     AutoStatsTracker::LogMode::kUpdateTop);
+                     AutoStatsTracker::LogMode::kUpdateTop,
+                     secondaryNamespaces);
 
+    MultipleCollectionAccessor holder{expCtx->opCtx,
+                                      &autoColl->getCollection(),
+                                      autoColl->getNss(),
+                                      autoColl->isAnySecondaryNamespaceAViewOrSharded(),
+                                      secondaryNamespaces};
     PipelineD::buildAndAttachInnerQueryExecutorToPipeline(
-        autoColl->getCollection(), expCtx->ns, nullptr, pipeline.get());
+        holder, expCtx->ns, nullptr, pipeline.get());
 
     return pipeline;
 }
@@ -320,19 +466,12 @@ std::vector<GenericCursor> CommonMongodProcessInterface::getIdleCursors(
     return CursorManager::get(expCtx->opCtx)->getIdleCursors(expCtx->opCtx, userMode);
 }
 
-boost::optional<Document> CommonMongodProcessInterface::lookupSingleDocument(
+boost::optional<Document> CommonMongodProcessInterface::doLookupSingleDocument(
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     const NamespaceString& nss,
     UUID collectionUUID,
     const Document& documentKey,
-    boost::optional<BSONObj> readConcern,
-    bool allowSpeculativeMajorityRead) {
-    invariant(!readConcern);  // We don't currently support a read concern on mongod - it's only
-                              // expected to be necessary on mongos.
-    invariant(!allowSpeculativeMajorityRead);  // We don't expect 'allowSpeculativeMajorityRead' on
-                                               // mongod - it's only expected to be necessary on
-                                               // mongos.
-
+    MakePipelineOptions opts) {
     std::unique_ptr<Pipeline, PipelineDeleter> pipeline;
     try {
         // Be sure to do the lookup using the collection default collation
@@ -340,11 +479,12 @@ boost::optional<Document> CommonMongodProcessInterface::lookupSingleDocument(
             nss,
             collectionUUID,
             _getCollectionDefaultCollator(expCtx->opCtx, nss.db(), collectionUUID));
-        // When looking up on a mongoD, we only ever want to read from the local collection. By
-        // default, makePipeline will attach a cursor source which may read from remote if the
-        // collection is sharded, so we configure it to not allow that here.
-        MakePipelineOptions opts;
-        opts.allowTargetingShards = false;
+
+        // If we are here, we are either executing the pipeline normally or running in one of the
+        // execution stat explain verbosities. In either case, we disable explain on the foreign
+        // context so that we actually retrieve the document.
+        foreignExpCtx->explain = boost::none;
+
         pipeline = Pipeline::makePipeline({BSON("$match" << documentKey)}, foreignExpCtx, opts);
     } catch (const ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
         return boost::none;
@@ -358,20 +498,6 @@ boost::optional<Document> CommonMongodProcessInterface::lookupSingleDocument(
                                 << ", " << next->toString() << "]");
     }
 
-    // Set the speculative read timestamp appropriately after we do a document lookup locally. We
-    // set the speculative read timestamp based on the timestamp used by the transaction.
-    repl::SpeculativeMajorityReadInfo& speculativeMajorityReadInfo =
-        repl::SpeculativeMajorityReadInfo::get(expCtx->opCtx);
-    if (speculativeMajorityReadInfo.isSpeculativeRead()) {
-        // Speculative majority reads are required to use the 'kNoOverlap' read source.
-        invariant(expCtx->opCtx->recoveryUnit()->getTimestampReadSource() ==
-                  RecoveryUnit::ReadSource::kNoOverlap);
-        boost::optional<Timestamp> readTs =
-            expCtx->opCtx->recoveryUnit()->getPointInTimeReadTimestamp();
-        invariant(readTs);
-        speculativeMajorityReadInfo.setSpeculativeReadTimestampForward(*readTs);
-    }
-
     return lookedUpDocument;
 }
 
@@ -407,7 +533,7 @@ BackupCursorExtendState CommonMongodProcessInterface::extendBackupCursor(
 
 std::vector<BSONObj> CommonMongodProcessInterface::getMatchingPlanCacheEntryStats(
     OperationContext* opCtx, const NamespaceString& nss, const MatchExpression* matchExp) const {
-    const auto serializer = [](const PlanCacheEntry& entry) {
+    const auto serializer = [](const auto& entry) {
         BSONObjBuilder out;
         Explain::planCacheEntryToBSON(entry, &out);
         return out.obj();
@@ -417,15 +543,35 @@ std::vector<BSONObj> CommonMongodProcessInterface::getMatchingPlanCacheEntryStat
         return !matchExp ? true : matchExp->matchesBSON(obj);
     };
 
-    AutoGetCollection autoColl(opCtx, nss, MODE_IS);
-    const auto collection = autoColl.getCollection();
+    AutoGetCollection collection(opCtx, nss, MODE_IS);
     uassert(
         50933, str::stream() << "collection '" << nss.toString() << "' does not exist", collection);
 
-    const auto planCache = CollectionQueryInfo::get(collection).getPlanCache();
+    const auto& collQueryInfo = CollectionQueryInfo::get(collection.getCollection());
+    const auto planCache = collQueryInfo.getPlanCache();
     invariant(planCache);
 
-    return planCache->getMatchingStats(serializer, predicate);
+    auto planCacheEntries =
+        planCache->getMatchingStats({} /* cacheKeyFilterFunc */, serializer, predicate);
+
+    if (feature_flags::gFeatureFlagSbeFull.isEnabledAndIgnoreFCV()) {
+        // Retrieve plan cache entries from the SBE plan cache.
+        const auto cacheKeyFilter = [uuid = collection->uuid(),
+                                     collVersion = collQueryInfo.getPlanCacheInvalidatorVersion()](
+                                        const sbe::PlanCacheKey& key) {
+            // Only fetch plan cache entries with keys matching given UUID and collectionVersion.
+            return uuid == key.getMainCollectionState().uuid &&
+                collVersion == key.getMainCollectionState().version;
+        };
+
+        auto planCacheEntriesSBE =
+            sbe::getPlanCache(opCtx).getMatchingStats(cacheKeyFilter, serializer, predicate);
+
+        planCacheEntries.insert(
+            planCacheEntries.end(), planCacheEntriesSBE.begin(), planCacheEntriesSBE.end());
+    }
+
+    return planCacheEntries;
 }
 
 bool CommonMongodProcessInterface::fieldsHaveSupportingUniqueIndex(
@@ -433,15 +579,16 @@ bool CommonMongodProcessInterface::fieldsHaveSupportingUniqueIndex(
     const NamespaceString& nss,
     const std::set<FieldPath>& fieldPaths) const {
     auto* opCtx = expCtx->opCtx;
+
     // We purposefully avoid a helper like AutoGetCollection here because we don't want to check the
     // db version or do anything else. We simply want to protect against concurrent modifications to
     // the catalog.
-    Lock::DBLock dbLock(opCtx, nss.db(), MODE_IS);
+    Lock::DBLock dbLock(opCtx, nss.dbName(), MODE_IS);
     Lock::CollectionLock collLock(opCtx, nss, MODE_IS);
     auto databaseHolder = DatabaseHolder::get(opCtx);
-    auto db = databaseHolder->getDb(opCtx, nss.db());
+    auto db = databaseHolder->getDb(opCtx, nss.dbName());
     auto collection =
-        db ? CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss) : nullptr;
+        db ? CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss) : nullptr;
     if (!collection) {
         return fieldPaths == std::set<FieldPath>{"_id"};
     }
@@ -457,18 +604,6 @@ bool CommonMongodProcessInterface::fieldsHaveSupportingUniqueIndex(
     return false;
 }
 
-bool CommonMongodProcessInterface::supportsReadPreferenceForWriteOp(
-    const boost::intrusive_ptr<ExpressionContext>& expCtx) {
-    // The global copy of the FCV is checked instead of the ExpressionContext's copy of the
-    // FCV because the FCV on ExpressionContext is only set when persisting queries in the
-    // catalog.
-    if (ReadPreferenceSetting::get(expCtx->opCtx).equals(ReadPreferenceSetting()))
-        return true;
-    return serverGlobalParams.featureCompatibility.isVersionInitialized() &&
-        serverGlobalParams.featureCompatibility.getVersion() ==
-        ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44;
-}
-
 BSONObj CommonMongodProcessInterface::_reportCurrentOpForClient(
     OperationContext* opCtx,
     Client* client,
@@ -511,6 +646,17 @@ void CommonMongodProcessInterface::_reportCurrentOpsForTransactionCoordinators(
     reportCurrentOpsForTransactionCoordinators(opCtx, includeIdle, ops);
 }
 
+void CommonMongodProcessInterface::_reportCurrentOpsForPrimaryOnlyServices(
+    OperationContext* opCtx,
+    CurrentOpConnectionsMode connMode,
+    CurrentOpSessionsMode sessionMode,
+    std::vector<BSONObj>* ops) const {
+    auto registry = repl::PrimaryOnlyServiceRegistry::get(opCtx->getServiceContext());
+    invariant(registry);
+
+    registry->reportServiceInfoForCurrentOp(connMode, sessionMode, ops);
+}
+
 void CommonMongodProcessInterface::_reportCurrentOpsForIdleSessions(
     OperationContext* opCtx, CurrentOpUserMode userMode, std::vector<BSONObj>* ops) const {
     auto sessionCatalog = SessionCatalog::get(opCtx);
@@ -558,8 +704,9 @@ std::unique_ptr<CollatorInterface> CommonMongodProcessInterface::_getCollectionD
     return collator ? collator->clone() : nullptr;
 }
 
-std::unique_ptr<ResourceYielder> CommonMongodProcessInterface::getResourceYielder() const {
-    return std::make_unique<MongoDResourceYielder>();
+std::unique_ptr<ResourceYielder> CommonMongodProcessInterface::getResourceYielder(
+    StringData cmdName) const {
+    return TransactionParticipantResourceYielder::make(cmdName);
 }
 
 
@@ -569,12 +716,9 @@ CommonMongodProcessInterface::ensureFieldsUniqueOrResolveDocumentKey(
     boost::optional<std::set<FieldPath>> fieldPaths,
     boost::optional<ChunkVersion> targetCollectionVersion,
     const NamespaceString& outputNs) const {
-    if (targetCollectionVersion) {
-        uassert(51123, "Unexpected target chunk version specified", expCtx->fromMongos);
-        // If mongos has sent us a target shard version, we need to be sure we are prepared to
-        // act as a router which is at least as recent as that mongos.
-        checkRoutingInfoEpochOrThrow(expCtx, outputNs, *targetCollectionVersion);
-    }
+    uassert(51123,
+            "Unexpected target chunk version specified",
+            !targetCollectionVersion || expCtx->fromMongos);
 
     if (!fieldPaths) {
         uassert(51124, "Expected fields to be provided from mongos", !expCtx->fromMongos);
@@ -591,16 +735,76 @@ CommonMongodProcessInterface::ensureFieldsUniqueOrResolveDocumentKey(
     return {*fieldPaths, targetCollectionVersion};
 }
 
+write_ops::InsertCommandRequest CommonMongodProcessInterface::buildInsertOp(
+    const NamespaceString& nss, std::vector<BSONObj>&& objs, bool bypassDocValidation) {
+    write_ops::InsertCommandRequest insertOp(nss);
+    insertOp.setDocuments(std::move(objs));
+    insertOp.setWriteCommandRequestBase([&] {
+        write_ops::WriteCommandRequestBase wcb;
+        wcb.setOrdered(false);
+        wcb.setBypassDocumentValidation(bypassDocValidation);
+        return wcb;
+    }());
+    return insertOp;
+}
+
+write_ops::UpdateCommandRequest CommonMongodProcessInterface::buildUpdateOp(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
+    const NamespaceString& nss,
+    BatchedObjects&& batch,
+    UpsertType upsert,
+    bool multi) {
+    write_ops::UpdateCommandRequest updateOp(nss);
+    updateOp.setUpdates([&] {
+        std::vector<write_ops::UpdateOpEntry> updateEntries;
+        for (auto&& obj : batch) {
+            updateEntries.push_back([&] {
+                write_ops::UpdateOpEntry entry;
+                auto&& [q, u, c] = obj;
+                entry.setQ(std::move(q));
+                entry.setU(std::move(u));
+                entry.setC(std::move(c));
+                entry.setUpsert(upsert != UpsertType::kNone);
+                entry.setUpsertSupplied(
+                    {{entry.getUpsert(), upsert == UpsertType::kInsertSuppliedDoc}});
+                entry.setMulti(multi);
+                return entry;
+            }());
+        }
+        return updateEntries;
+    }());
+    updateOp.setWriteCommandRequestBase([&] {
+        write_ops::WriteCommandRequestBase wcb;
+        wcb.setOrdered(false);
+        wcb.setBypassDocumentValidation(expCtx->bypassDocumentValidation);
+        return wcb;
+    }());
+    auto [constants, letParams] =
+        expCtx->variablesParseState.transitionalCompatibilitySerialize(expCtx->variables);
+    updateOp.setLegacyRuntimeConstants(std::move(constants));
+    if (!letParams.isEmpty()) {
+        updateOp.setLet(std::move(letParams));
+    }
+    return updateOp;
+}
+
 BSONObj CommonMongodProcessInterface::_convertRenameToInternalRename(
     OperationContext* opCtx,
-    const BSONObj& renameCommandObj,
+    const NamespaceString& sourceNs,
+    const NamespaceString& targetNs,
     const BSONObj& originalCollectionOptions,
     const std::list<BSONObj>& originalIndexes) {
 
     BSONObjBuilder newCmd;
     newCmd.append("internalRenameIfOptionsAndIndexesMatch", 1);
-    newCmd.append("from", renameCommandObj["renameCollection"].String());
-    newCmd.append("to", renameCommandObj["to"].String());
+    // TODO SERVER-62114 Change to check for upgraded FCV rather than feature flag
+    if (gFeatureFlagRequireTenantID.isEnabled(serverGlobalParams.featureCompatibility)) {
+        newCmd.append("from", sourceNs.ns());
+        newCmd.append("to", targetNs.ns());
+    } else {
+        newCmd.append("from", sourceNs.toStringWithTenantId());
+        newCmd.append("to", targetNs.toStringWithTenantId());
+    }
     newCmd.append("collectionOptions", originalCollectionOptions);
     BSONArrayBuilder indexArrayBuilder(newCmd.subarrayStart("indexes"));
     for (auto&& index : originalIndexes) {
@@ -610,4 +814,73 @@ BSONObj CommonMongodProcessInterface::_convertRenameToInternalRename(
     return newCmd.obj();
 }
 
+void CommonMongodProcessInterface::writeRecordsToRecordStore(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
+    RecordStore* rs,
+    std::vector<Record>* records,
+    const std::vector<Timestamp>& ts) const {
+    tassert(5643012, "Attempted to write to record store with nullptr", records);
+    assertIgnorePrepareConflictsBehavior(expCtx);
+    writeConflictRetry(expCtx->opCtx, "MPI::writeRecordsToRecordStore", expCtx->ns.ns(), [&] {
+        Lock::GlobalLock lk(expCtx->opCtx, MODE_IS);
+        WriteUnitOfWork wuow(expCtx->opCtx);
+        auto writeResult = rs->insertRecords(expCtx->opCtx, records, ts);
+        tassert(5643002,
+                str::stream() << "Failed to write to disk because " << writeResult.reason(),
+                writeResult.isOK());
+        wuow.commit();
+    });
+}
+
+std::unique_ptr<TemporaryRecordStore> CommonMongodProcessInterface::createTemporaryRecordStore(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx, KeyFormat keyFormat) const {
+    assertIgnorePrepareConflictsBehavior(expCtx);
+    return expCtx->opCtx->getServiceContext()->getStorageEngine()->makeTemporaryRecordStore(
+        expCtx->opCtx, keyFormat);
+}
+
+Document CommonMongodProcessInterface::readRecordFromRecordStore(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx, RecordStore* rs, RecordId rID) const {
+    RecordData possibleRecord;
+    Lock::GlobalLock lk(expCtx->opCtx, MODE_IS);
+    auto foundDoc = rs->findRecord(expCtx->opCtx, RecordId(rID), &possibleRecord);
+    tassert(775101, str::stream() << "Could not find document id " << rID, foundDoc);
+    return Document(possibleRecord.toBson());
+}
+
+void CommonMongodProcessInterface::deleteRecordFromRecordStore(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx, RecordStore* rs, RecordId rID) const {
+    assertIgnorePrepareConflictsBehavior(expCtx);
+    writeConflictRetry(expCtx->opCtx, "MPI::deleteFromRecordStore", expCtx->ns.ns(), [&] {
+        Lock::GlobalLock lk(expCtx->opCtx, MODE_IS);
+        WriteUnitOfWork wuow(expCtx->opCtx);
+        rs->deleteRecord(expCtx->opCtx, rID);
+        wuow.commit();
+    });
+}
+
+void CommonMongodProcessInterface::truncateRecordStore(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx, RecordStore* rs) const {
+    assertIgnorePrepareConflictsBehavior(expCtx);
+    writeConflictRetry(expCtx->opCtx, "MPI::truncateRecordStore", expCtx->ns.ns(), [&] {
+        Lock::GlobalLock lk(expCtx->opCtx, MODE_IS);
+        WriteUnitOfWork wuow(expCtx->opCtx);
+        auto status = rs->truncate(expCtx->opCtx);
+        tassert(5643000, "Unable to clear record store", status.isOK());
+        wuow.commit();
+    });
+}
+
+boost::optional<Document> CommonMongodProcessInterface::lookupSingleDocumentLocally(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
+    const NamespaceString& nss,
+    const Document& documentKey) {
+    AutoGetCollectionForRead autoColl(expCtx->opCtx, nss);
+    BSONObj document;
+    if (!Helpers::findById(expCtx->opCtx, nss.ns(), documentKey.toBson(), document)) {
+        return boost::none;
+    }
+    return Document(document).getOwned();
+}
+
 }  // namespace mongo
