@@ -31,6 +31,7 @@
 
 #include <boost/intrusive_ptr.hpp>
 #include <boost/optional.hpp>
+#include <deque>
 #include <list>
 #include <memory>
 #include <string>
@@ -44,16 +45,19 @@
 #include "mongo/db/matcher/expression.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/ops/write_ops_exec.h"
-#include "mongo/db/ops/write_ops_parsers.h"
 #include "mongo/db/pipeline/field_path.h"
 #include "mongo/db/pipeline/lite_parsed_document_source.h"
+#include "mongo/db/pipeline/sharded_agg_helpers_targeting_policy.h"
+#include "mongo/db/pipeline/storage_stats_spec_gen.h"
 #include "mongo/db/query/explain_options.h"
+#include "mongo/db/record_id.h"
 #include "mongo/db/repl/oplog_entry.h"
 #include "mongo/db/resource_yielder.h"
 #include "mongo/db/storage/backup_cursor_hooks.h"
 #include "mongo/db/storage/backup_cursor_state.h"
+#include "mongo/db/storage/temporary_record_store.h"
 #include "mongo/executor/task_executor.h"
-#include "mongo/s/chunk_version.h"
+#include "mongo/s/shard_version.h"
 
 namespace mongo {
 
@@ -108,17 +112,8 @@ public:
     public:
         virtual ~WriteSizeEstimator() = default;
 
-        /**
-         * Set of functions which estimate the entire size of a write command except for the array
-         * of write statements themselves.
-         */
-        virtual int estimateInsertHeaderSize(const write_ops::Insert& insertReq) const = 0;
-        virtual int estimateUpdateHeaderSize(const write_ops::Update& updateReq) const = 0;
-
-        /**
-         * Set of functions which estimate the size of a single write statement.
-         */
         virtual int estimateInsertSizeBytes(const BSONObj& insert) const = 0;
+
         virtual int estimateUpdateSizeBytes(const BatchObject& batchObject,
                                             UpsertType type) const = 0;
     };
@@ -165,18 +160,6 @@ public:
      */
     virtual bool isSharded(OperationContext* opCtx, const NamespaceString& ns) = 0;
 
-    /**
-     * Returns whether the specified read preference is supported assuming that the agg operation
-     * will perform writes e.g. $out/$merge.
-     */
-    virtual bool supportsReadPreferenceForWriteOp(
-        const boost::intrusive_ptr<ExpressionContext>& expCtx) = 0;
-
-    /**
-     * Returns whether this process is running as part of a sharded cluster.
-     */
-    virtual bool inShardedEnvironment(OperationContext* opCtx) const = 0;
-
     /**
      * Advances the proxied write time associated with the client in ReplClientInfo to
      * be at least as high as the one tracked by the OperationTimeTracker associated with the
@@ -185,25 +168,25 @@ public:
     virtual void updateClientOperationTime(OperationContext* opCtx) const = 0;
 
     /**
-     * Executes 'insertCommand' against 'ns' and returns an error Status if the insert fails. If
-     * 'targetEpoch' is set, throws ErrorCodes::StaleEpoch if the targeted collection does not have
-     * the same epoch or the epoch changes during the course of the insert.
+     * Inserts 'objs' into 'ns' and returns an error Status if the insert fails. If 'targetEpoch' is
+     * set, throws ErrorCodes::StaleEpoch if the targeted collection does not have the same epoch or
+     * the epoch changes during the course of the insert.
      */
     virtual Status insert(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                           const NamespaceString& ns,
-                          std::unique_ptr<write_ops::Insert> insertCommand,
+                          std::vector<BSONObj>&& objs,
                           const WriteConcernOptions& wc,
                           boost::optional<OID> targetEpoch) = 0;
 
     /**
-     * Executes the updates described by 'updateCommand'. Returns an error Status if any of the
-     * updates fail, otherwise returns an 'UpdateResult' objects with the details of the update
-     * operation.  If 'targetEpoch' is set, throws ErrorCodes::StaleEpoch if the targeted collection
-     * does not have the same epoch, or if the epoch changes during the update.
+     * Updates the documents matching 'queries' with the objects 'updates'. Returns an error Status
+     * if any of the updates fail, otherwise returns an 'UpdateResult' objects with the details of
+     * the update operation.  If 'targetEpoch' is set, throws ErrorCodes::StaleEpoch if the targeted
+     * collection does not have the same epoch, or if the epoch changes during the update.
      */
     virtual StatusWith<UpdateResult> update(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                             const NamespaceString& ns,
-                                            std::unique_ptr<write_ops::Update> updateCommand,
+                                            BatchedObjects&& batch,
                                             const WriteConcernOptions& wc,
                                             UpsertType upsert,
                                             bool multi,
@@ -226,6 +209,17 @@ public:
                                              const NamespaceString& ns,
                                              bool includeBuildUUIDs) = 0;
 
+    /**
+     * Returns all documents in `_mdb_catalog`.
+     */
+    virtual std::deque<BSONObj> listCatalog(OperationContext* opCtx) const = 0;
+
+    /**
+     * Returns the catalog entry for the given namespace, if it exists.
+     */
+    virtual boost::optional<BSONObj> getCatalogEntry(OperationContext* opCtx,
+                                                     const NamespaceString& ns) const = 0;
+
     /**
      * Appends operation latency statistics for collection "nss" to "builder"
      */
@@ -239,7 +233,7 @@ public:
      */
     virtual Status appendStorageStats(OperationContext* opCtx,
                                       const NamespaceString& nss,
-                                      const BSONObj& param,
+                                      const StorageStatsSpec& spec,
                                       BSONObjBuilder* builder) const = 0;
 
     /**
@@ -270,8 +264,10 @@ public:
      */
     virtual void renameIfOptionsAndIndexesHaveNotChanged(
         OperationContext* opCtx,
-        const BSONObj& renameCommandObj,
+        const NamespaceString& sourceNs,
         const NamespaceString& targetNs,
+        bool dropTarget,
+        bool stayTemp,
         const BSONObj& originalCollectionOptions,
         const std::list<BSONObj>& originalIndexes) = 0;
 
@@ -280,7 +276,7 @@ public:
      * the primary shard of 'dbName'.
      */
     virtual void createCollection(OperationContext* opCtx,
-                                  const std::string& dbName,
+                                  const DatabaseName& dbName,
                                   const BSONObj& cmdObj) = 0;
 
     /**
@@ -303,11 +299,13 @@ public:
      * Changing it to a unique_ptr introduces a circular dependency on certain platforms where the
      * compiler expects to find an implementation of PipelineDeleter.
      *
-     * If `allowTargetingShards` is true, the cursor will only be for local reads regardless of
-     * whether or not this function is called in a sharded environment.
+     * If `shardTargetingPolicy` is kNotAllowed, the cursor will only be for local reads regardless
+     * of whether or not this function is called in a sharded environment.
      */
     virtual std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipeline(
-        Pipeline* pipeline, bool allowTargetingShards = true) = 0;
+        Pipeline* pipeline,
+        ShardTargetingPolicy shardTargetingPolicy = ShardTargetingPolicy::kAllowed,
+        boost::optional<BSONObj> readConcern = boost::none) = 0;
 
     /**
      * Accepts a pipeline and attaches a cursor source to it. Returns a BSONObj of the form
@@ -359,25 +357,16 @@ public:
      */
     virtual std::string getShardName(OperationContext* opCtx) const = 0;
 
+    /**
+     * Returns whether or not this process is running as part of a sharded cluster.
+     */
+    virtual bool inShardedEnvironment(OperationContext* opCtx) const = 0;
+
     /**
      * Returns the "host:port" string for this node.
      */
     virtual std::string getHostAndPort(OperationContext* opCtx) const = 0;
 
-    /**
-     * Returns the fields of the document key (in order) for the collection corresponding to 'uuid',
-     * including the shard key and _id. If _id is not in the shard key, it is added last. If the
-     * collection is not sharded or no longer exists, returns only _id. Also returns a boolean that
-     * indicates whether the returned fields of the document key are final and will never change for
-     * the given collection, either because the collection was dropped or has become sharded.
-     *
-     * This method is meant to be called from a mongod which owns at least one chunk for this
-     * collection. It will inspect the CollectionShardingState, not the CatalogCache. If asked about
-     * a collection not hosted on this shard, the answer will be incorrect.
-     */
-    virtual std::pair<std::vector<FieldPath>, bool> collectDocumentKeyFieldsForHostedCollection(
-        OperationContext* opCtx, const NamespaceString&, UUID) const = 0;
-
     /**
      * Returns the fields of the document key (in order) for the collection 'nss', according to the
      * CatalogCache. The document key fields are the shard key (if sharded) and the _id (if not
@@ -393,14 +382,28 @@ public:
      * as a unique identifier of a document, and may include an _id or all fields from the shard key
      * and an _id. Throws if more than one match was found. Returns boost::none if no matching
      * documents were found, including cases where the given namespace does not exist.
+     *
+     * If this interface needs to send requests (possibly to other nodes) in order to look up the
+     * document, 'readConcern' will be attached to these requests. Otherwise 'readConcern' will be
+     * ignored.
      */
     virtual boost::optional<Document> lookupSingleDocument(
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
         const NamespaceString& nss,
         UUID,
         const Document& documentKey,
-        boost::optional<BSONObj> readConcern,
-        bool allowSpeculativeMajorityRead = false) = 0;
+        boost::optional<BSONObj> readConcern) = 0;
+
+    /**
+     * Returns zero or one document with the document _id being equal to 'documentKey'. The document
+     * is looked up only on the current node. Returns boost::none if no matching documents were
+     * found, including cases where the given namespace does not exist. It is illegal to call this
+     * method on nodes other than mongod.
+     */
+    virtual boost::optional<Document> lookupSingleDocumentLocally(
+        const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        const NamespaceString& nss,
+        const Document& documentKey) = 0;
 
     /**
      * Returns a vector of all idle (non-pinned) local cursors.
@@ -449,7 +452,7 @@ public:
      * request to be sent to the config servers. If another thread has already requested a refresh,
      * it will instead wait for that response.
      */
-    virtual boost::optional<ChunkVersion> refreshAndGetCollectionVersion(
+    virtual boost::optional<ShardVersion> refreshAndGetCollectionVersion(
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
         const NamespaceString& nss) const = 0;
 
@@ -462,7 +465,26 @@ public:
                                               const NamespaceString& nss,
                                               ChunkVersion targetCollectionVersion) const = 0;
 
-    virtual std::unique_ptr<ResourceYielder> getResourceYielder() const = 0;
+    /**
+     * Used to enforce the constraint that the foreign collection must be unsharded.
+     */
+    class ScopedExpectUnshardedCollection {
+    public:
+        virtual ~ScopedExpectUnshardedCollection() = default;
+    };
+    virtual std::unique_ptr<ScopedExpectUnshardedCollection> expectUnshardedCollectionInScope(
+        OperationContext* opCtx,
+        const NamespaceString& nss,
+        const boost::optional<DatabaseVersion>& dbVersion) = 0;
+
+    /**
+     * Checks if this process is on the primary shard for db specified by the given namespace.
+     * Throws an IllegalOperation exception otherwise. Assumes the operation context has a db
+     * version attached to it for db name specified by the namespace.
+     */
+    virtual void checkOnPrimaryShardForDb(OperationContext* opCtx, const NamespaceString& nss) = 0;
+
+    virtual std::unique_ptr<ResourceYielder> getResourceYielder(StringData cmdName) const = 0;
 
     /**
      * If the user did not provide the 'fieldPaths' set, a default unique key will be picked,
@@ -479,6 +501,43 @@ public:
                                            const NamespaceString& outputNs) const = 0;
 
     std::shared_ptr<executor::TaskExecutor> taskExecutor;
+
+    /**
+     * Create a temporary record store.
+     */
+    virtual std::unique_ptr<TemporaryRecordStore> createTemporaryRecordStore(
+        const boost::intrusive_ptr<ExpressionContext>& expCtx, KeyFormat keyFormat) const = 0;
+
+    /**
+     * Write the records in 'records' to the record store. Record store must already exist. Asserts
+     * that the writes succeeded.
+     */
+    virtual void writeRecordsToRecordStore(const boost::intrusive_ptr<ExpressionContext>& expCtx,
+                                           RecordStore* rs,
+                                           std::vector<Record>* records,
+                                           const std::vector<Timestamp>& ts) const = 0;
+
+    /**
+     * Search for the RecordId 'rID' in 'rs'. RecordStore must already exist and be populated.
+     * Asserts that a document was found.
+     */
+    virtual Document readRecordFromRecordStore(
+        const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        RecordStore* rs,
+        RecordId rID) const = 0;
+
+    /**
+     * Deletes the record with RecordId `rID` from `rs`. RecordStore must already exist.
+     */
+    virtual void deleteRecordFromRecordStore(const boost::intrusive_ptr<ExpressionContext>& expCtx,
+                                             RecordStore* rs,
+                                             RecordId rID) const = 0;
+
+    /**
+     * Deletes all Records from `rs`. RecordStore must already exist.
+     */
+    virtual void truncateRecordStore(const boost::intrusive_ptr<ExpressionContext>& expCtx,
+                                     RecordStore* rs) const = 0;
 };
 
 }  // namespace mongo
