@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/platform/basic.h"
 
@@ -43,7 +42,6 @@
 #include "mongo/db/query/collation/collation_spec.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/repl/read_concern_args.h"
-#include "mongo/db/s/scoped_collection_metadata.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/redaction.h"
 #include "mongo/s/catalog_cache.h"
@@ -53,29 +51,27 @@
 #include "mongo/s/query/document_source_merge_cursors.h"
 #include "mongo/s/query/establish_cursors.h"
 #include "mongo/s/query/router_exec_stage.h"
+#include "mongo/s/stale_shard_version_helpers.h"
 #include "mongo/s/transaction_router.h"
 #include "mongo/util/fail_point.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
+
+
 namespace mongo {
-
-using boost::intrusive_ptr;
-using std::shared_ptr;
-using std::string;
-using std::unique_ptr;
-
 namespace {
 
 /**
  * Returns the routing information for the namespace set on the passed ExpressionContext. Also
  * verifies that the ExpressionContext's UUID, if present, matches that of the routing table entry.
  */
-StatusWith<CachedCollectionRoutingInfo> getCollectionRoutingInfo(
-    const intrusive_ptr<ExpressionContext>& expCtx) {
+StatusWith<ChunkManager> getCollectionRoutingInfo(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx) {
     auto catalogCache = Grid::get(expCtx->opCtx)->catalogCache();
     auto swRoutingInfo = catalogCache->getCollectionRoutingInfo(expCtx->opCtx, expCtx->ns);
     // Additionally check that the ExpressionContext's UUID matches the collection routing info.
-    if (swRoutingInfo.isOK() && expCtx->uuid && swRoutingInfo.getValue().cm()) {
-        if (!swRoutingInfo.getValue().cm()->uuidMatches(*expCtx->uuid)) {
+    if (swRoutingInfo.isOK() && expCtx->uuid && swRoutingInfo.getValue().isSharded()) {
+        if (!swRoutingInfo.getValue().uuidMatches(*expCtx->uuid)) {
             return {ErrorCodes::NamespaceNotFound,
                     str::stream() << "The UUID of collection " << expCtx->ns.ns()
                                   << " changed; it may have been dropped and re-created."};
@@ -105,16 +101,23 @@ bool supportsUniqueKey(const boost::intrusive_ptr<ExpressionContext>& expCtx,
 
 }  // namespace
 
-std::unique_ptr<MongoProcessInterface::WriteSizeEstimator>
+std::unique_ptr<CommonProcessInterface::WriteSizeEstimator>
 MongosProcessInterface::getWriteSizeEstimator(OperationContext* opCtx,
                                               const NamespaceString& ns) const {
     return std::make_unique<TargetPrimaryWriteSizeEstimator>();
 }
 
 std::unique_ptr<Pipeline, PipelineDeleter> MongosProcessInterface::attachCursorSourceToPipeline(
-    Pipeline* ownedPipeline, bool allowTargetingShards) {
+    Pipeline* ownedPipeline,
+    ShardTargetingPolicy shardTargetingPolicy,
+    boost::optional<BSONObj> readConcern) {
     // On mongos we can't have local cursors.
-    return sharded_agg_helpers::attachCursorToPipeline(ownedPipeline, allowTargetingShards);
+    tassert(5530900,
+            "shardTargetingPolicy cannot be kNotAllowed on mongos",
+            shardTargetingPolicy != ShardTargetingPolicy::kNotAllowed);
+
+    return sharded_agg_helpers::attachCursorToPipeline(
+        ownedPipeline, shardTargetingPolicy, std::move(readConcern));
 }
 
 BSONObj MongosProcessInterface::preparePipelineAndExplain(Pipeline* ownedPipeline,
@@ -134,8 +137,7 @@ boost::optional<Document> MongosProcessInterface::lookupSingleDocument(
     const NamespaceString& nss,
     UUID collectionUUID,
     const Document& filter,
-    boost::optional<BSONObj> readConcern,
-    bool allowSpeculativeMajorityRead) {
+    boost::optional<BSONObj> readConcern) {
     auto foreignExpCtx = expCtx->copyWith(nss, collectionUUID);
 
     // Create the find command to be dispatched to the shard(s) in order to return the post-image.
@@ -148,27 +150,25 @@ boost::optional<Document> MongosProcessInterface::lookupSingleDocument(
         cmdBuilder.append("find", nss.coll());
     }
     cmdBuilder.append("filter", filterObj);
+    cmdBuilder.append("allowSpeculativeMajorityRead", true);
     if (readConcern) {
         cmdBuilder.append(repl::ReadConcernArgs::kReadConcernFieldName, *readConcern);
     }
-    if (allowSpeculativeMajorityRead) {
-        cmdBuilder.append("allowSpeculativeMajorityRead", true);
-    }
 
     try {
         auto findCmd = cmdBuilder.obj();
         auto catalogCache = Grid::get(expCtx->opCtx)->catalogCache();
-        auto shardResults = sharded_agg_helpers::shardVersionRetry(
+        auto shardResults = shardVersionRetry(
             expCtx->opCtx,
             catalogCache,
             foreignExpCtx->ns,
             str::stream() << "Looking up document matching " << redact(filter.toBson()),
             [&]() -> std::vector<RemoteCursor> {
                 // Verify that the collection exists, with the correct UUID.
-                auto routingInfo = uassertStatusOK(getCollectionRoutingInfo(foreignExpCtx));
+                auto cm = uassertStatusOK(getCollectionRoutingInfo(foreignExpCtx));
 
                 // Finalize the 'find' command object based on the routing table information.
-                if (findCmdIsByUuid && routingInfo.cm()) {
+                if (findCmdIsByUuid && cm.isSharded()) {
                     // Find by UUID and shard versioning do not work together (SERVER-31946).  In
                     // the sharded case we've already checked the UUID, so find by namespace is
                     // safe.  In the unlikely case that the collection has been deleted and a new
@@ -183,12 +183,8 @@ boost::optional<Document> MongosProcessInterface::lookupSingleDocument(
                 // single shard will be targeted here; however, in certain cases where only the _id
                 // is present, we may need to scatter-gather the query to all shards in order to
                 // find the document.
-                auto requests = getVersionedRequestsForTargetedShards(expCtx->opCtx,
-                                                                      nss,
-                                                                      routingInfo,
-                                                                      findCmd,
-                                                                      filterObj,
-                                                                      CollationSpec::kSimpleSpec);
+                auto requests = getVersionedRequestsForTargetedShards(
+                    expCtx->opCtx, nss, cm, findCmd, filterObj, CollationSpec::kSimpleSpec);
 
                 // Dispatch the requests. The 'establishCursors' method conveniently prepares the
                 // result into a vector of cursor responses for us.
@@ -230,6 +226,13 @@ boost::optional<Document> MongosProcessInterface::lookupSingleDocument(
     }
 }
 
+boost::optional<Document> MongosProcessInterface::lookupSingleDocumentLocally(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
+    const NamespaceString& nss,
+    const Document& documentKey) {
+    MONGO_UNREACHABLE_TASSERT(6148001);
+}
+
 BSONObj MongosProcessInterface::_reportCurrentOpForClient(
     OperationContext* opCtx,
     Client* client,
@@ -272,7 +275,7 @@ void MongosProcessInterface::_reportCurrentOpsForIdleSessions(OperationContext*
                               : KillAllSessionsByPatternSet{{}});
 
     sessionCatalog->scanSessions({std::move(sessionFilter)}, [&](const ObservableSession& session) {
-        if (!session.currentOperation()) {
+        if (!session.hasCurrentOperation()) {
             auto op =
                 TransactionRouter::get(session).reportState(opCtx, false /* sessionIsActive */);
             if (!op.isEmpty()) {
@@ -285,8 +288,14 @@ void MongosProcessInterface::_reportCurrentOpsForIdleSessions(OperationContext*
 void MongosProcessInterface::_reportCurrentOpsForTransactionCoordinators(
     OperationContext* opCtx, bool includeIdle, std::vector<BSONObj>* ops) const {};
 
+void MongosProcessInterface::_reportCurrentOpsForPrimaryOnlyServices(
+    OperationContext* opCtx,
+    CurrentOpConnectionsMode connMode,
+    CurrentOpSessionsMode sessionMode,
+    std::vector<BSONObj>* ops) const {};
+
 std::vector<GenericCursor> MongosProcessInterface::getIdleCursors(
-    const intrusive_ptr<ExpressionContext>& expCtx, CurrentOpUserMode userMode) const {
+    const boost::intrusive_ptr<ExpressionContext>& expCtx, CurrentOpUserMode userMode) const {
     invariant(hasGlobalServiceContext());
     auto cursorManager = Grid::get(expCtx->opCtx->getServiceContext())->getCursorManager();
     invariant(cursorManager);
@@ -294,8 +303,9 @@ std::vector<GenericCursor> MongosProcessInterface::getIdleCursors(
 }
 
 bool MongosProcessInterface::isSharded(OperationContext* opCtx, const NamespaceString& nss) {
-    auto routingInfo = Grid::get(opCtx)->catalogCache()->getCollectionRoutingInfo(opCtx, nss);
-    return routingInfo.isOK() && routingInfo.getValue().cm();
+    auto cm =
+        uassertStatusOK(Grid::get(opCtx)->catalogCache()->getCollectionRoutingInfo(opCtx, nss));
+    return cm.isSharded();
 }
 
 bool MongosProcessInterface::fieldsHaveSupportingUniqueIndex(
@@ -354,7 +364,9 @@ MongosProcessInterface::ensureFieldsUniqueOrResolveDocumentKey(
     // collection was dropped a long time ago. Because of this, we are okay with piggy-backing
     // off another thread's request to refresh the cache, simply waiting for that request to
     // return instead of forcing another refresh.
-    targetCollectionVersion = refreshAndGetCollectionVersion(expCtx, outputNs);
+    boost::optional<ShardVersion> targetVersion = refreshAndGetCollectionVersion(expCtx, outputNs);
+    targetCollectionVersion =
+        targetVersion ? boost::make_optional((ChunkVersion)*targetVersion) : boost::none;
 
     auto docKeyPaths = collectDocumentKeyFieldsActingAsRouter(expCtx->opCtx, outputNs);
     return {std::set<FieldPath>(std::make_move_iterator(docKeyPaths.begin()),
