@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kProcessHealth
 
 #include "mongo/platform/basic.h"
 
@@ -38,7 +37,6 @@
 #include "mongo/db/process_health/fault.h"
 #include "mongo/db/process_health/fault_facet_impl.h"
 #include "mongo/db/process_health/fault_manager_config.h"
-#include "mongo/db/process_health/health_monitoring_feature_flag.h"
 #include "mongo/db/process_health/health_monitoring_gen.h"
 #include "mongo/db/process_health/health_observer_registration.h"
 #include "mongo/executor/network_interface_factory.h"
@@ -48,7 +46,9 @@
 #include "mongo/logv2/log.h"
 #include "mongo/util/concurrency/thread_pool.h"
 #include "mongo/util/exit_code.h"
-#include "mongo/util/future_util.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kProcessHealth
+
 
 namespace mongo {
 
@@ -92,7 +92,7 @@ void FaultManager::set(ServiceContext* svcCtx, std::unique_ptr<FaultManager> new
 
 
 bool FaultManager::isInitialized() {
-    stdx::lock_guard lock(_stateMutex);
+    stdx::lock_guard lock(_mutex);
     return _initialized;
 }
 
@@ -106,7 +106,7 @@ void FaultManager::healthMonitoringIntensitiesUpdated(HealthObserverIntensities
 
     auto manager = FaultManager::get(getGlobalServiceContext());
     if (manager && manager->isInitialized()) {
-        auto cancellationToken = manager->_managerShuttingDownCancellation;
+        auto cancellationToken = manager->_managerShuttingDownCancellationSource.token();
         auto findByType =
             [](const auto& values,
                HealthObserverTypeEnum type) -> boost::optional<HealthObserverIntensitySetting> {
@@ -133,7 +133,7 @@ void FaultManager::healthMonitoringIntensitiesUpdated(HealthObserverIntensities
             if (!oldSetting) {
                 continue;
             }
-            if (cancellationToken->load()) {
+            if (cancellationToken.isCanceled()) {
                 break;
             }
             auto oldIntensity = oldSetting->getIntensity();
@@ -144,7 +144,8 @@ void FaultManager::healthMonitoringIntensitiesUpdated(HealthObserverIntensities
                     if (auto* observer =
                             manager->getHealthObserver(toFaultFacetType(setting.getType()));
                         observer != nullptr) {
-                        manager->healthCheck(observer, cancellationToken);
+                        manager->scheduleNextHealthCheck(
+                            observer, cancellationToken, true /* immediate */);
                     }
                 } else if (newIntensity == HealthObserverIntensityEnum::kOff) {
                     // {critical, non-critical} -> off
@@ -161,15 +162,10 @@ FaultManager::TransientFaultDeadline::TransientFaultDeadline(
     FaultManager* faultManager,
     std::shared_ptr<executor::TaskExecutor> executor,
     Milliseconds timeout)
-    : activeFaultTransition(sleepFor(executor, timeout)
+    : cancelActiveFaultTransition(CancellationSource()),
+      activeFaultTransition(executor->sleepFor(timeout, cancelActiveFaultTransition.token())
                                 .thenRunOn(executor)
-                                .then([faultManager, cancelled = cancelActiveFaultTransition]() {
-                                    // `faultManager` waits for all tasks scheduled in the executor
-                                    // to be canceled or complete, so this check is safe.
-                                    if (!cancelled->load()) {
-                                        faultManager->accept(boost::none);
-                                    }
-                                })
+                                .then([faultManager]() { faultManager->accept(boost::none); })
                                 .onError([](Status status) {
                                     LOGV2_DEBUG(
                                         5937001,
@@ -179,7 +175,9 @@ FaultManager::TransientFaultDeadline::TransientFaultDeadline(
                                 })) {}
 
 FaultManager::TransientFaultDeadline::~TransientFaultDeadline() {
-    cancelActiveFaultTransition->store(true);
+    if (!cancelActiveFaultTransition.token().isCanceled()) {
+        cancelActiveFaultTransition.cancel();
+    }
 }
 
 FaultManager::FaultManager(ServiceContext* svcCtx,
@@ -194,7 +192,7 @@ FaultManager::FaultManager(ServiceContext* svcCtx,
                       "Fault manager progress monitor is terminating the server",
                       "cause"_attr = cause);
           // This calls the exit_group syscall on Linux
-          ::_exit(ExitCode::EXIT_PROCESS_HEALTH_CHECK);
+          ::_exit(static_cast<int>(ExitCode::processHealthCheck));
       }) {
     invariant(_svcCtx);
     invariant(_svcCtx->getFastClockSource());
@@ -354,6 +352,7 @@ boost::optional<FaultState> FaultManager::handleTransientFault(const OptionalMes
 }
 
 boost::optional<FaultState> FaultManager::handleActiveFault(const OptionalMessageType& message) {
+    invariant(_fault);
     LOGV2_FATAL(5936509, "Halting Process due to ongoing fault", "fault"_attr = *_fault);
     return boost::none;
 }
@@ -369,6 +368,10 @@ void FaultManager::logMessageReceived(FaultState state, const HealthCheckStatus&
 }
 
 void FaultManager::logCurrentState(FaultState, FaultState newState, const OptionalMessageType&) {
+    {
+        stdx::lock_guard<Latch> lk(_stateMutex);
+        _lastTransitionTime = _svcCtx->getFastClockSource()->now();
+    }
     if (_fault) {
         LOGV2(5939703,
               "Fault manager changed state ",
@@ -401,7 +404,9 @@ void FaultManager::setInitialHealthCheckComplete(FaultState,
 }
 
 void FaultManager::schedulePeriodicHealthCheckThread() {
-    if (!feature_flags::gFeatureFlagHealthMonitoring || _config->periodicChecksDisabledForTests()) {
+    if (!feature_flags::gFeatureFlagHealthMonitoring.isEnabled(
+            serverGlobalParams.featureCompatibility) ||
+        _config->periodicChecksDisabledForTests()) {
         return;
     }
 
@@ -420,22 +425,32 @@ void FaultManager::schedulePeriodicHealthCheckThread() {
                     "observerType"_attr = str::stream() << observer->getType());
         listOfActiveObservers << observer->getType() << " ";
 
-        healthCheck(observer, _managerShuttingDownCancellation);
+        auto token = _managerShuttingDownCancellationSource.token();
+        if (!observer->isConfigured()) {
+            // Transition to an active fault if a health observer is not configured properly.
+            updateWithCheckStatus(HealthCheckStatus(
+                observer->getType(),
+                Severity::kFailure,
+                "Health observer failed to start because it was not configured properly."_sd));
+            setState(FaultState::kActiveFault, HealthCheckStatus(observer->getType()));
+            return;
+        }
+        scheduleNextHealthCheck(observer, token, true /* immediate */);
     }
     LOGV2(5936804, "Health observers started", "detail"_attr = listOfActiveObservers);
 }
 
 FaultManager::~FaultManager() {
-    _managerShuttingDownCancellation->store(true);
+    _managerShuttingDownCancellationSource.cancel();
     _taskExecutor->shutdown();
 
     LOGV2(5936601, "Shutting down periodic health checks");
     {
         stdx::lock_guard lock(_mutex);
         for (auto& pair : _healthCheckContexts) {
-            auto cbHandle = pair.second.resultStatus;
+            auto cbHandle = pair.second.callbackHandle;
             if (cbHandle) {
-                _taskExecutor->cancel(cbHandle.get());
+                _taskExecutor->cancel(cbHandle.value());
             }
         }
     }
@@ -458,7 +473,8 @@ FaultManager::~FaultManager() {
 }
 
 SharedSemiFuture<void> FaultManager::startPeriodicHealthChecks() {
-    if (!feature_flags::gFeatureFlagHealthMonitoring) {
+    if (!feature_flags::gFeatureFlagHealthMonitoring.isEnabled(
+            serverGlobalParams.featureCompatibility)) {
         LOGV2_DEBUG(6187201, 1, "Health checks disabled by feature flag");
         return Future<void>::makeReady();
     }
@@ -506,78 +522,111 @@ FaultPtr FaultManager::getOrCreateFault() {
     return _fault;
 }
 
-void FaultManager::healthCheck(HealthObserver* observer, std::shared_ptr<AtomicWord<bool>> token) {
-    auto schedulerCb = [this, observer, token] {
-        auto scheduledTime = _taskExecutor->now() +
-            _config->getPeriodicHealthCheckInterval(observer->getType()) +
+void FaultManager::scheduleNextHealthCheck(HealthObserver* observer,
+                                           CancellationToken token,
+                                           bool immediately) {
+    stdx::lock_guard lock(_mutex);
+
+    // Check that context callbackHandle is not set and if future exists, it is ready.
+    auto existingIt = _healthCheckContexts.find(observer->getType());
+    if (existingIt != _healthCheckContexts.end()) {
+        if (existingIt->second.callbackHandle) {
+            LOGV2_WARNING(6418201,
+                          "Cannot schedule health check while another one is in queue",
+                          "observerType"_attr = str::stream() << observer->getType());
+            return;
+        }
+        if (existingIt->second.result && !existingIt->second.result->isReady()) {
+            LOGV2_WARNING(6418202,
+                          "Cannot schedule health check while another one is currently executing",
+                          "observerType"_attr = str::stream() << observer->getType());
+            return;
+        }
+    }
+    _healthCheckContexts.insert_or_assign(observer->getType(),
+                                          HealthCheckContext(nullptr, boost::none));
+
+    auto scheduledTime = immediately
+        ? _taskExecutor->now()
+        : _taskExecutor->now() + _config->getPeriodicHealthCheckInterval(observer->getType()) +
             std::min(observer->healthCheckJitter(),
                      FaultManagerConfig::kPeriodicHealthCheckMaxJitter);
-        LOGV2_DEBUG(5939701,
-                    3,
-                    "Schedule next health check",
-                    "observerType"_attr = str::stream() << observer->getType(),
-                    "scheduledTime"_attr = scheduledTime);
+    LOGV2_DEBUG(5939701,
+                3,
+                "Schedule next health check",
+                "observerType"_attr = str::stream() << observer->getType(),
+                "scheduledTime"_attr = scheduledTime);
 
-        auto periodicThreadCbHandleStatus = _taskExecutor->scheduleWorkAt(
-            scheduledTime,
-            [this, observer, token](const mongo::executor::TaskExecutor::CallbackArgs& cbData) {
-                if (!cbData.status.isOK()) {
-                    LOGV2_DEBUG(5939702,
-                                1,
-                                "Fault manager received an error",
-                                "status"_attr = cbData.status);
-                    if (ErrorCodes::isA<ErrorCategory::CancelationError>(cbData.status.code())) {
-                        return;
-                    }
-                    // continue health checking otherwise
+    auto periodicThreadCbHandleStatus = _taskExecutor->scheduleWorkAt(
+        scheduledTime,
+        [this, observer, token](const mongo::executor::TaskExecutor::CallbackArgs& cbData) {
+            if (!cbData.status.isOK()) {
+                LOGV2_DEBUG(
+                    5939702, 1, "Fault manager received an error", "status"_attr = cbData.status);
+                if (ErrorCodes::isA<ErrorCategory::CancellationError>(cbData.status.code())) {
+                    return;
                 }
-                if (!token->load()) {
-                    healthCheck(observer, token);
-                }
-            });
-
-        if (!periodicThreadCbHandleStatus.isOK()) {
-            if (ErrorCodes::isA<ErrorCategory::ShutdownError>(
-                    periodicThreadCbHandleStatus.getStatus().code())) {
-                return;
+                // continue health checking otherwise
             }
+            healthCheck(observer, token);
+        });
 
-            uassert(5936101,
-                    str::stream() << "Failed to schedule periodic health check for "
-                                  << observer->getType() << ": "
-                                  << periodicThreadCbHandleStatus.getStatus().codeString(),
-                    periodicThreadCbHandleStatus.isOK());
+    if (!periodicThreadCbHandleStatus.isOK()) {
+        if (ErrorCodes::isA<ErrorCategory::ShutdownError>(
+                periodicThreadCbHandleStatus.getStatus().code())) {
+            LOGV2_DEBUG(6418203, 1, "Not scheduling health check because of shutdown");
+            return;
         }
 
-        stdx::lock_guard lock(_mutex);
-        _healthCheckContexts.at(observer->getType()).resultStatus =
-            std::move(periodicThreadCbHandleStatus.getValue());
-    };
+        uassert(5936101,
+                str::stream() << "Failed to schedule periodic health check for "
+                              << observer->getType() << ": "
+                              << periodicThreadCbHandleStatus.getStatus().codeString(),
+                periodicThreadCbHandleStatus.isOK());
+    }
 
+    _healthCheckContexts.at(observer->getType()).callbackHandle =
+        std::move(periodicThreadCbHandleStatus.getValue());
+}
+
+void FaultManager::healthCheck(HealthObserver* observer, CancellationToken token) {
     auto acceptNotOKStatus = [this, observer](Status s) {
-        auto healthCheckStatus = HealthCheckStatus(observer->getType(), 1.0, s.reason());
+        auto healthCheckStatus =
+            HealthCheckStatus(observer->getType(), Severity::kFailure, s.reason());
         LOGV2_ERROR(
             6007901, "Unexpected failure during health check", "status"_attr = healthCheckStatus);
         accept(healthCheckStatus);
         return healthCheckStatus;
     };
 
-    {
-        stdx::lock_guard lock(_mutex);
-        _healthCheckContexts.insert(
-            {observer->getType(), HealthCheckContext(nullptr, boost::none)});
-    }
 
     // Run asynchronous health check.  Send output to the state machine. Schedule next run.
-    auto healthCheckFuture =
-        observer->periodicCheck(_taskExecutor, token)
-            .thenRunOn(_taskExecutor)
-            .onCompletion([this, acceptNotOKStatus, schedulerCb, observer](
-                              StatusWith<HealthCheckStatus> status) {
-                ON_BLOCK_EXIT([this, schedulerCb, observer]() {
+    auto healthCheckFuture = observer->periodicCheck(_taskExecutor, token);
+
+    stdx::lock_guard lock(_mutex);
+    auto contextIt = _healthCheckContexts.find(observer->getType());
+    if (contextIt == _healthCheckContexts.end()) {
+        LOGV2_ERROR(6418204, "Unexpected failure during health check: context not found");
+        return;
+    }
+    contextIt->second.result =
+        std::make_unique<SharedSemiFuture<HealthCheckStatus>>(std::move(healthCheckFuture));
+
+    contextIt->second.result->thenRunOn(_taskExecutor)
+        .onCompletion(
+            [this, acceptNotOKStatus, observer, token](StatusWith<HealthCheckStatus> status) {
+                ON_BLOCK_EXIT([this, observer, token]() {
+                    {
+                        stdx::lock_guard lock(_mutex);
+                        // Rescheduling requires the previous handle to be cleaned.
+                        auto contextIt = _healthCheckContexts.find(observer->getType());
+                        if (contextIt != _healthCheckContexts.end()) {
+                            contextIt->second.callbackHandle = {};
+                        }
+                    }
                     if (!_config->periodicChecksDisabledForTests() &&
                         _config->isHealthObserverEnabled(observer->getType())) {
-                        schedulerCb();
+                        scheduleNextHealthCheck(observer, token, false /* immediate */);
                     }
                 });
 
@@ -587,11 +636,8 @@ void FaultManager::healthCheck(HealthObserver* observer, std::shared_ptr<AtomicW
 
                 accept(status.getValue());
                 return status.getValue();
-            });
-
-    stdx::lock_guard lock(_mutex);
-    _healthCheckContexts.at(observer->getType()).result =
-        std::make_unique<ExecutorFuture<HealthCheckStatus>>(std::move(healthCheckFuture));
+            })
+        .getAsync([](StatusOrStatusWith<mongo::process_health::HealthCheckStatus>) {});
 }
 
 void FaultManager::updateWithCheckStatus(HealthCheckStatus&& checkStatus) {
@@ -613,33 +659,38 @@ void FaultManager::updateWithCheckStatus(HealthCheckStatus&& checkStatus) {
         type, _svcCtx->getFastClockSource(), std::move(checkStatus)));
 }
 
-FaultManagerConfig FaultManager::getConfig() const {
+const FaultManagerConfig& FaultManager::getConfig() const {
     auto lk = stdx::lock_guard(_mutex);
     return *_config;
 }
 
 void FaultManager::_init() {
-    auto lk = stdx::lock_guard(_mutex);
+    std::set<FaultFacetType> allTypes;
+    std::vector<std::unique_ptr<HealthObserver>>::size_type observersSize = _observers.size();
+    {
+        auto lk = stdx::lock_guard(_mutex);
 
-    _observers = HealthObserverRegistration::instantiateAllObservers(_svcCtx);
+        _observers = HealthObserverRegistration::instantiateAllObservers(_svcCtx);
+
+        for (const auto& observer : _observers) {
+            allTypes.insert(observer->getType());
+        }
+        observersSize = _observers.size();
+    }
 
     // Verify that all observer types are unique.
-    std::set<FaultFacetType> allTypes;
-    for (const auto& observer : _observers) {
-        allTypes.insert(observer->getType());
-    }
-    invariant(allTypes.size() == _observers.size());
+    invariant(allTypes.size() == observersSize);
 
     // Start the monitor thread after all observers are initialized.
     _progressMonitor = std::make_unique<ProgressMonitor>(this, _svcCtx, _crashCb);
 
-    auto lk2 = stdx::lock_guard(_stateMutex);
     _initialized = true;
+
     LOGV2_DEBUG(5956701,
                 1,
                 "Instantiated health observers",
                 "managerState"_attr = str::stream() << state(),
-                "observersCount"_attr = _observers.size());
+                "observersCount"_attr = observersSize);
 }
 
 std::vector<HealthObserver*> FaultManager::getHealthObservers() const {
