@@ -27,79 +27,32 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/query/canonical_query.h"
 
+#include "mongo/crypto/encryption_fields_gen.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/commands/test_commands_enabled.h"
+#include "mongo/db/cst/cst_parser.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/matcher/expression_array.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/query/canonical_query_encoder.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
+#include "mongo/db/query/fle/server_rewrite.h"
 #include "mongo/db/query/indexability.h"
 #include "mongo/db/query/projection_parser.h"
 #include "mongo/db/query/query_planner_common.h"
+#include "mongo/logv2/log.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 namespace mongo {
 namespace {
 
-/**
- * Comparator for MatchExpression nodes.  Returns an integer less than, equal to, or greater
- * than zero if 'lhs' is less than, equal to, or greater than 'rhs', respectively.
- *
- * Sorts by:
- * 1) operator type (MatchExpression::MatchType)
- * 2) path name (MatchExpression::path())
- * 3) sort order of children
- * 4) number of children (MatchExpression::numChildren())
- *
- * The third item is needed to ensure that match expression trees which should have the same
- * cache key always sort the same way. If you're wondering when the tuple (operator type, path
- * name) could ever be equal, consider this query:
- *
- * {$and:[{$or:[{a:1},{a:2}]},{$or:[{a:1},{b:2}]}]}
- *
- * The two OR nodes would compare as equal in this case were it not for tuple item #3 (sort
- * order of children).
- */
-int matchExpressionComparator(const MatchExpression* lhs, const MatchExpression* rhs) {
-    MatchExpression::MatchType lhsMatchType = lhs->matchType();
-    MatchExpression::MatchType rhsMatchType = rhs->matchType();
-    if (lhsMatchType != rhsMatchType) {
-        return lhsMatchType < rhsMatchType ? -1 : 1;
-    }
-
-    StringData lhsPath = lhs->path();
-    StringData rhsPath = rhs->path();
-    int pathsCompare = lhsPath.compare(rhsPath);
-    if (pathsCompare != 0) {
-        return pathsCompare;
-    }
-
-    const size_t numChildren = std::min(lhs->numChildren(), rhs->numChildren());
-    for (size_t childIdx = 0; childIdx < numChildren; ++childIdx) {
-        int childCompare =
-            matchExpressionComparator(lhs->getChild(childIdx), rhs->getChild(childIdx));
-        if (childCompare != 0) {
-            return childCompare;
-        }
-    }
-
-    if (lhs->numChildren() != rhs->numChildren()) {
-        return lhs->numChildren() < rhs->numChildren() ? -1 : 1;
-    }
-
-    // They're equal!
-    return 0;
-}
-
-bool matchExpressionLessThan(const MatchExpression* lhs, const MatchExpression* rhs) {
-    return matchExpressionComparator(lhs, rhs) < 0;
-}
-
 bool parsingCanProduceNoopMatchNodes(const ExtensionsCallback& extensionsCallback,
                                      MatchExpressionParser::AllowedFeatureSet allowedFeatures) {
     return extensionsCallback.hasNoopExtensions() &&
@@ -112,37 +65,22 @@ bool parsingCanProduceNoopMatchNodes(const ExtensionsCallback& extensionsCallbac
 // static
 StatusWith<std::unique_ptr<CanonicalQuery>> CanonicalQuery::canonicalize(
     OperationContext* opCtx,
-    const QueryMessage& qm,
-    const boost::intrusive_ptr<ExpressionContext>& expCtx,
-    const ExtensionsCallback& extensionsCallback,
-    MatchExpressionParser::AllowedFeatureSet allowedFeatures) {
-    // Make QueryRequest.
-    auto qrStatus = QueryRequest::fromLegacyQueryMessage(qm);
-    if (!qrStatus.isOK()) {
-        return qrStatus.getStatus();
-    }
-
-    return CanonicalQuery::canonicalize(
-        opCtx, std::move(qrStatus.getValue()), expCtx, extensionsCallback, allowedFeatures);
-}
-
-// static
-StatusWith<std::unique_ptr<CanonicalQuery>> CanonicalQuery::canonicalize(
-    OperationContext* opCtx,
-    std::unique_ptr<QueryRequest> qr,
+    std::unique_ptr<FindCommandRequest> findCommand,
+    bool explain,
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     const ExtensionsCallback& extensionsCallback,
     MatchExpressionParser::AllowedFeatureSet allowedFeatures,
-    const ProjectionPolicies& projectionPolicies) {
-    auto qrStatus = qr->validate();
-    if (!qrStatus.isOK()) {
-        return qrStatus;
+    const ProjectionPolicies& projectionPolicies,
+    std::vector<std::unique_ptr<InnerPipelineStageInterface>> pipeline) {
+    auto status = query_request_helper::validateFindCommandRequest(*findCommand);
+    if (!status.isOK()) {
+        return status;
     }
 
     std::unique_ptr<CollatorInterface> collator;
-    if (!qr->getCollation().isEmpty()) {
+    if (!findCommand->getCollation().isEmpty()) {
         auto statusWithCollator = CollatorFactoryInterface::get(opCtx->getServiceContext())
-                                      ->makeFromBSON(qr->getCollation());
+                                      ->makeFromBSON(findCommand->getCollation());
         if (!statusWithCollator.isOK()) {
             return statusWithCollator.getStatus();
         }
@@ -152,15 +90,39 @@ StatusWith<std::unique_ptr<CanonicalQuery>> CanonicalQuery::canonicalize(
     // Make MatchExpression.
     boost::intrusive_ptr<ExpressionContext> newExpCtx;
     if (!expCtx.get()) {
-        newExpCtx = make_intrusive<ExpressionContext>(
-            opCtx, std::move(collator), qr->nss(), qr->getRuntimeConstants());
+        invariant(findCommand->getNamespaceOrUUID().nss());
+        newExpCtx = make_intrusive<ExpressionContext>(opCtx,
+                                                      std::move(collator),
+                                                      *findCommand->getNamespaceOrUUID().nss(),
+                                                      findCommand->getLegacyRuntimeConstants(),
+                                                      findCommand->getLet());
     } else {
         newExpCtx = expCtx;
-        invariant(CollatorInterface::collatorsMatch(collator.get(), expCtx->getCollator()));
+        // A collator can enter through both the FindCommandRequest and ExpressionContext arguments.
+        // This invariant ensures that both collators are the same because downstream we
+        // pull the collator from only one of the ExpressionContext carrier.
+        if (collator.get() && expCtx->getCollator()) {
+            invariant(CollatorInterface::collatorsMatch(collator.get(), expCtx->getCollator()));
+        }
     }
 
-    StatusWithMatchExpression statusWithMatcher = MatchExpressionParser::parse(
-        qr->getFilter(), newExpCtx, extensionsCallback, allowedFeatures);
+    // Make the CQ we'll hopefully return.
+    std::unique_ptr<CanonicalQuery> cq(new CanonicalQuery());
+    cq->setExplain(explain);
+
+    StatusWithMatchExpression statusWithMatcher = [&]() -> StatusWithMatchExpression {
+        if (getTestCommandsEnabled() && internalQueryEnableCSTParser.load()) {
+            try {
+                return cst::parseToMatchExpression(
+                    findCommand->getFilter(), newExpCtx, extensionsCallback);
+            } catch (const DBException& ex) {
+                return ex.toStatus();
+            }
+        } else {
+            return MatchExpressionParser::parse(
+                findCommand->getFilter(), newExpCtx, extensionsCallback, allowedFeatures);
+        }
+    }();
     if (!statusWithMatcher.isOK()) {
         return statusWithMatcher.getStatus();
     }
@@ -171,16 +133,14 @@ StatusWith<std::unique_ptr<CanonicalQuery>> CanonicalQuery::canonicalize(
 
     std::unique_ptr<MatchExpression> me = std::move(statusWithMatcher.getValue());
 
-    // Make the CQ we'll hopefully return.
-    std::unique_ptr<CanonicalQuery> cq(new CanonicalQuery());
-
     Status initStatus =
         cq->init(opCtx,
                  std::move(newExpCtx),
-                 std::move(qr),
+                 std::move(findCommand),
                  parsingCanProduceNoopMatchNodes(extensionsCallback, allowedFeatures),
                  std::move(me),
-                 projectionPolicies);
+                 projectionPolicies,
+                 std::move(pipeline));
 
     if (!initStatus.isOK()) {
         return initStatus;
@@ -191,27 +151,28 @@ StatusWith<std::unique_ptr<CanonicalQuery>> CanonicalQuery::canonicalize(
 // static
 StatusWith<std::unique_ptr<CanonicalQuery>> CanonicalQuery::canonicalize(
     OperationContext* opCtx, const CanonicalQuery& baseQuery, MatchExpression* root) {
-    auto qr = std::make_unique<QueryRequest>(baseQuery.nss());
+    auto findCommand = std::make_unique<FindCommandRequest>(baseQuery.nss());
     BSONObjBuilder builder;
     root->serialize(&builder, true);
-    qr->setFilter(builder.obj());
-    qr->setProj(baseQuery.getQueryRequest().getProj());
-    qr->setSort(baseQuery.getQueryRequest().getSort());
-    qr->setCollation(baseQuery.getQueryRequest().getCollation());
-    qr->setExplain(baseQuery.getQueryRequest().isExplain());
-    auto qrStatus = qr->validate();
-    if (!qrStatus.isOK()) {
-        return qrStatus;
+    findCommand->setFilter(builder.obj());
+    findCommand->setProjection(baseQuery.getFindCommandRequest().getProjection().getOwned());
+    findCommand->setSort(baseQuery.getFindCommandRequest().getSort().getOwned());
+    findCommand->setCollation(baseQuery.getFindCommandRequest().getCollation().getOwned());
+    auto status = query_request_helper::validateFindCommandRequest(*findCommand);
+    if (!status.isOK()) {
+        return status;
     }
 
     // Make the CQ we'll hopefully return.
     std::unique_ptr<CanonicalQuery> cq(new CanonicalQuery());
+    cq->setExplain(baseQuery.getExplain());
     Status initStatus = cq->init(opCtx,
                                  baseQuery.getExpCtx(),
-                                 std::move(qr),
+                                 std::move(findCommand),
                                  baseQuery.canHaveNoopMatchNodes(),
                                  root->shallowClone(),
-                                 ProjectionPolicies::findProjectionPolicies());
+                                 ProjectionPolicies::findProjectionPolicies(),
+                                 {} /* an empty pipeline */);
 
     if (!initStatus.isOK()) {
         return initStatus;
@@ -221,38 +182,58 @@ StatusWith<std::unique_ptr<CanonicalQuery>> CanonicalQuery::canonicalize(
 
 Status CanonicalQuery::init(OperationContext* opCtx,
                             boost::intrusive_ptr<ExpressionContext> expCtx,
-                            std::unique_ptr<QueryRequest> qr,
+                            std::unique_ptr<FindCommandRequest> findCommand,
                             bool canHaveNoopMatchNodes,
                             std::unique_ptr<MatchExpression> root,
-                            const ProjectionPolicies& projectionPolicies) {
+                            const ProjectionPolicies& projectionPolicies,
+                            std::vector<std::unique_ptr<InnerPipelineStageInterface>> pipeline) {
     _expCtx = expCtx;
-    _qr = std::move(qr);
+    _findCommand = std::move(findCommand);
 
     _canHaveNoopMatchNodes = canHaveNoopMatchNodes;
+    _forceClassicEngine = ServerParameterSet::getNodeParameterSet()
+                              ->get<QueryFrameworkControl>("internalQueryFrameworkControl")
+                              ->_data.get() == QueryFrameworkControlEnum::kForceClassicEngine;
 
-    // Validate, normalize, and sort tree.
-    auto validStatus = isValid(root.get(), *_qr);
+    auto validStatus = isValid(root.get(), *_findCommand);
     if (!validStatus.isOK()) {
         return validStatus.getStatus();
     }
     auto unavailableMetadata = validStatus.getValue();
-    _root = MatchExpression::optimize(std::move(root));
-    sortTree(_root.get());
+    _root = MatchExpression::normalize(std::move(root));
+
+    // If caching is disabled, do not perform any autoparameterization.
+    if (!internalQueryDisablePlanCache.load() &&
+        feature_flags::gFeatureFlagSbeFull.isEnabledAndIgnoreFCV()) {
+        const bool hasNoTextNodes =
+            !QueryPlannerCommon::hasNode(_root.get(), MatchExpression::TEXT);
+        if (hasNoTextNodes) {
+            // When the SBE plan cache is enabled, we auto-parameterize queries in the hopes of
+            // caching a parameterized plan. Here we add parameter markers to the appropriate match
+            // expression leaf nodes.
+            _inputParamIdToExpressionMap = MatchExpression::parameterize(_root.get());
+        } else {
+            LOGV2_DEBUG(6579310,
+                        5,
+                        "The query was not auto-parameterized since its match expression tree "
+                        "contains TEXT nodes");
+        }
+    }
     // The tree must always be valid after normalization.
-    dassert(isValid(_root.get(), *_qr).isOK());
+    dassert(isValid(_root.get(), *_findCommand).isOK());
     if (auto status = isValidNormalized(_root.get()); !status.isOK()) {
         return status;
     }
 
     // Validate the projection if there is one.
-    if (!_qr->getProj().isEmpty()) {
+    if (!_findCommand->getProjection().isEmpty()) {
         try {
-            _proj.emplace(projection_ast::parse(expCtx,
-                                                _qr->getProj(),
-                                                _root.get(),
-                                                _qr->getFilter(),
-                                                projectionPolicies,
-                                                true /* Should optimize? */));
+            _proj.emplace(projection_ast::parseAndAnalyze(expCtx,
+                                                          _findCommand->getProjection(),
+                                                          _root.get(),
+                                                          _findCommand->getFilter(),
+                                                          projectionPolicies,
+                                                          true /* Should optimize? */));
 
             // Fail if any of the projection's dependencies are unavailable.
             DepsTracker{unavailableMetadata}.requestMetadata(_proj->metadataDeps());
@@ -263,8 +244,10 @@ Status CanonicalQuery::init(OperationContext* opCtx,
         _metadataDeps = _proj->metadataDeps();
     }
 
+    _pipeline = std::move(pipeline);
+
     if (_proj && _proj->metadataDeps()[DocumentMetadataFields::kSortKey] &&
-        _qr->getSort().isEmpty()) {
+        _findCommand->getSort().isEmpty()) {
         return Status(ErrorCodes::BadValue, "cannot use sortKey $meta projection without a sort");
     }
 
@@ -276,7 +259,7 @@ Status CanonicalQuery::init(OperationContext* opCtx,
     }
 
     // If the 'returnKey' option is set, then the plan should produce index key metadata.
-    if (_qr->returnKey()) {
+    if (_findCommand->getReturnKey()) {
         _metadataDeps.set(DocumentMetadataFields::kIndexKey);
     }
 
@@ -284,7 +267,7 @@ Status CanonicalQuery::init(OperationContext* opCtx,
 }
 
 void CanonicalQuery::initSortPattern(QueryMetadataBitSet unavailableMetadata) {
-    if (_qr->getSort().isEmpty()) {
+    if (_findCommand->getSort().isEmpty()) {
         return;
     }
 
@@ -294,12 +277,16 @@ void CanonicalQuery::initSortPattern(QueryMetadataBitSet unavailableMetadata) {
     // We have already validated that if there is a $natural sort and a hint, that the hint
     // also specifies $natural with the same direction. Therefore, it is safe to clear the $natural
     // sort and rewrite it as a $natural hint.
-    if (_qr->getSort()[QueryRequest::kNaturalSortField]) {
-        _qr->setHint(_qr->getSort());
-        _qr->setSort(BSONObj{});
+    if (_findCommand->getSort()[query_request_helper::kNaturalSortField]) {
+        _findCommand->setHint(_findCommand->getSort().getOwned());
+        _findCommand->setSort(BSONObj{});
     }
 
-    _sortPattern = SortPattern{_qr->getSort(), _expCtx};
+    if (getTestCommandsEnabled() && internalQueryEnableCSTParser.load()) {
+        _sortPattern = cst::parseToSortPattern(_findCommand->getSort(), _expCtx);
+    } else {
+        _sortPattern = SortPattern{_findCommand->getSort(), _expCtx};
+    }
     _metadataDeps |= _sortPattern->metadataDeps(unavailableMetadata);
 
     // If the results of this query might have to be merged on a remote node, then that node might
@@ -349,17 +336,6 @@ bool CanonicalQuery::isSimpleIdQuery(const BSONObj& query) {
     return hasID;
 }
 
-// static
-void CanonicalQuery::sortTree(MatchExpression* tree) {
-    for (size_t i = 0; i < tree->numChildren(); ++i) {
-        sortTree(tree->getChild(i));
-    }
-    if (auto&& children = tree->getChildVector()) {
-        std::stable_sort(children->begin(), children->end(), matchExpressionLessThan);
-    }
-}
-
-// static
 size_t CanonicalQuery::countNodes(const MatchExpression* root, MatchExpression::MatchType type) {
     size_t sum = 0;
     if (type == root->matchType()) {
@@ -389,7 +365,7 @@ bool hasNodeInSubtree(const MatchExpression* root,
 }
 
 StatusWith<QueryMetadataBitSet> CanonicalQuery::isValid(const MatchExpression* root,
-                                                        const QueryRequest& request) {
+                                                        const FindCommandRequest& findCommand) {
     QueryMetadataBitSet unavailableMetadata{};
 
     // There can only be one TEXT.  If there is a TEXT, it cannot appear inside a NOR.
@@ -420,9 +396,9 @@ StatusWith<QueryMetadataBitSet> CanonicalQuery::isValid(const MatchExpression* r
         unavailableMetadata |= DepsTracker::kAllGeoNearData;
     }
 
-    const BSONObj& sortObj = request.getSort();
+    const BSONObj& sortObj = findCommand.getSort();
     BSONElement sortNaturalElt = sortObj["$natural"];
-    const BSONObj& hintObj = request.getHint();
+    const BSONObj& hintObj = findCommand.getHint();
     BSONElement hintNaturalElt = hintObj["$natural"];
 
     if (sortNaturalElt && sortObj.nFields() != 1) {
@@ -464,12 +440,12 @@ StatusWith<QueryMetadataBitSet> CanonicalQuery::isValid(const MatchExpression* r
     }
 
     // TEXT and tailable are incompatible.
-    if (numText > 0 && request.isTailable()) {
+    if (numText > 0 && findCommand.getTailable()) {
         return Status(ErrorCodes::BadValue, "text and tailable cursor not allowed in same query");
     }
 
     // NEAR and tailable are incompatible.
-    if (numGeoNear > 0 && request.isTailable()) {
+    if (numGeoNear > 0 && findCommand.getTailable()) {
         return Status(ErrorCodes::BadValue,
                       "Tailable cursors and geo $near cannot be used together");
     }
@@ -492,7 +468,7 @@ StatusWith<QueryMetadataBitSet> CanonicalQuery::isValid(const MatchExpression* r
 
 Status CanonicalQuery::isValidNormalized(const MatchExpression* root) {
     if (auto numGeoNear = countNodes(root, MatchExpression::GEO_NEAR); numGeoNear > 0) {
-        invariant(numGeoNear == 1);
+        tassert(5705300, "Only one geo $near expression is expected", numGeoNear == 1);
 
         auto topLevel = false;
         if (MatchExpression::GEO_NEAR == root->matchType()) {
@@ -507,7 +483,7 @@ Status CanonicalQuery::isValidNormalized(const MatchExpression* root) {
         }
 
         if (!topLevel) {
-            return Status(ErrorCodes::BadValue, "geoNear must be top-level expr");
+            return Status(ErrorCodes::BadValue, "geo $near must be top-level expr");
         }
     }
 
@@ -516,64 +492,64 @@ Status CanonicalQuery::isValidNormalized(const MatchExpression* root) {
 
 std::string CanonicalQuery::toString() const {
     str::stream ss;
-    ss << "ns=" << _qr->nss().ns();
+    ss << "ns=" << _findCommand->getNamespaceOrUUID().nss().value_or(NamespaceString()).ns();
 
-    if (_qr->getBatchSize()) {
-        ss << " batchSize=" << *_qr->getBatchSize();
+    if (_findCommand->getBatchSize()) {
+        ss << " batchSize=" << *_findCommand->getBatchSize();
     }
 
-    if (_qr->getLimit()) {
-        ss << " limit=" << *_qr->getLimit();
+    if (_findCommand->getLimit()) {
+        ss << " limit=" << *_findCommand->getLimit();
     }
 
-    if (_qr->getSkip()) {
-        ss << " skip=" << *_qr->getSkip();
-    }
-
-    if (_qr->getNToReturn()) {
-        ss << " ntoreturn=" << *_qr->getNToReturn() << '\n';
+    if (_findCommand->getSkip()) {
+        ss << " skip=" << *_findCommand->getSkip();
     }
 
     // The expression tree puts an endl on for us.
     ss << "Tree: " << _root->debugString();
-    ss << "Sort: " << _qr->getSort().toString() << '\n';
-    ss << "Proj: " << _qr->getProj().toString() << '\n';
-    if (!_qr->getCollation().isEmpty()) {
-        ss << "Collation: " << _qr->getCollation().toString() << '\n';
+    ss << "Sort: " << _findCommand->getSort().toString() << '\n';
+    ss << "Proj: " << _findCommand->getProjection().toString() << '\n';
+    if (!_findCommand->getCollation().isEmpty()) {
+        ss << "Collation: " << _findCommand->getCollation().toString() << '\n';
     }
     return ss;
 }
 
 std::string CanonicalQuery::toStringShort() const {
     str::stream ss;
-    ss << "ns: " << _qr->nss().ns() << " query: " << _qr->getFilter().toString()
-       << " sort: " << _qr->getSort().toString() << " projection: " << _qr->getProj().toString();
+    ss << "ns: " << _findCommand->getNamespaceOrUUID().nss().value_or(NamespaceString()).ns()
+       << " query: " << _findCommand->getFilter().toString()
+       << " sort: " << _findCommand->getSort().toString()
+       << " projection: " << _findCommand->getProjection().toString();
 
-    if (!_qr->getCollation().isEmpty()) {
-        ss << " collation: " << _qr->getCollation().toString();
+    if (!_findCommand->getCollation().isEmpty()) {
+        ss << " collation: " << _findCommand->getCollation().toString();
     }
 
-    if (_qr->getBatchSize()) {
-        ss << " batchSize: " << *_qr->getBatchSize();
+    if (_findCommand->getBatchSize()) {
+        ss << " batchSize: " << *_findCommand->getBatchSize();
     }
 
-    if (_qr->getLimit()) {
-        ss << " limit: " << *_qr->getLimit();
+    if (_findCommand->getLimit()) {
+        ss << " limit: " << *_findCommand->getLimit();
     }
 
-    if (_qr->getSkip()) {
-        ss << " skip: " << *_qr->getSkip();
-    }
-
-    if (_qr->getNToReturn()) {
-        ss << " ntoreturn=" << *_qr->getNToReturn();
+    if (_findCommand->getSkip()) {
+        ss << " skip: " << *_findCommand->getSkip();
     }
 
     return ss;
 }
 
 CanonicalQuery::QueryShapeString CanonicalQuery::encodeKey() const {
-    return canonical_query_encoder::encode(*this);
+    return (feature_flags::gFeatureFlagSbeFull.isEnabledAndIgnoreFCV() && !_forceClassicEngine &&
+            _sbeCompatible)
+        ? canonical_query_encoder::encodeSBE(*this)
+        : canonical_query_encoder::encode(*this);
 }
 
+CanonicalQuery::QueryShapeString CanonicalQuery::encodeKeyForPlanCacheCommand() const {
+    return canonical_query_encoder::encodeForPlanCacheCommand(*this);
+}
 }  // namespace mongo
