@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/db/query/plan_enumerator.h"
 
@@ -38,6 +37,9 @@
 #include "mongo/logv2/log.h"
 #include "mongo/util/string_map.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
+
+
 namespace {
 
 using namespace mongo;
@@ -166,7 +168,7 @@ void getPossibleFirstAssignments(const IndexEntry& thisIndex,
  * will be assigned to the index.
  */
 bool canAssignPredToIndex(const RelevantTag* rt,
-                          const std::set<size_t>& multikeyComponents,
+                          const MultikeyComponents& multikeyComponents,
                           StringMap<MatchExpression*>* used) {
     invariant(used);
     const FieldRef path(rt->path);
@@ -378,11 +380,7 @@ unique_ptr<MatchExpression> PlanEnumerator::getNext() {
     tagForSort(tree.get());
 
     _root->resetTag();
-    LOGV2_DEBUG(20943,
-                5,
-                "Enumerator: memo just before moving:\n{memo}",
-                "Enumerator: memo just before moving",
-                "memo"_attr = dumpMemo());
+    LOGV2_DEBUG(20943, 5, "Enumerator: memo just before moving", "memo"_attr = dumpMemo());
     _done = nextMemo(memoIDForNode(_root));
     return tree;
 }
@@ -414,6 +412,14 @@ bool PlanEnumerator::prepMemo(MatchExpression* node, PrepMemoContext context) {
     childContext.outsidePreds = context.outsidePreds;
 
     if (MatchExpression::OR == node->matchType()) {
+        if (_orLimit == 0) {
+            LOGV2_DEBUG(4862501,
+                        1,
+                        "plan enumerator exceeded threshold for OR enumerations",
+                        "orEnumerationLimit"_attr = _orLimit);
+            _explainInfo.hitIndexedOrLimit = true;
+            return false;
+        }
         // For an OR to be indexed, all its children must be indexed.
         for (size_t i = 0; i < node->numChildren(); ++i) {
 
@@ -1098,6 +1104,12 @@ void PlanEnumerator::enumerateAndIntersect(const IndexToPredMap& idxToFirst,
             andAssignment->choices.push_back(std::move(indexAndSubnode));
             // Limit n^2.
             if (andAssignment->choices.size() - sizeBefore > _intersectLimit) {
+
+                LOGV2_DEBUG(4862502,
+                            1,
+                            "plan enumerator exceeded threshold for AND enumerations",
+                            "intersectLimit"_attr = _intersectLimit);
+                _explainInfo.hitIndexedAndLimit = true;
                 return;
             }
         }
@@ -1112,6 +1124,11 @@ void PlanEnumerator::enumerateAndIntersect(const IndexToPredMap& idxToFirst,
 
             // Limit n^2.
             if (andAssignment->choices.size() - sizeBefore > _intersectLimit) {
+                LOGV2_DEBUG(4862503,
+                            1,
+                            "plan enumerator exceeded threshold for AND enumerations",
+                            "intersectLimit"_attr = _intersectLimit);
+                _explainInfo.hitIndexedAndLimit = true;
                 return;
             }
 
@@ -1302,12 +1319,21 @@ void PlanEnumerator::getIndexedPreds(MatchExpression* node,
 }
 
 bool PlanEnumerator::prepSubNodes(MatchExpression* node,
+
                                   PrepMemoContext context,
                                   vector<MemoID>* subnodesOut,
                                   vector<MemoID>* mandatorySubnodes) {
     for (size_t i = 0; i < node->numChildren(); ++i) {
         MatchExpression* child = node->getChild(i);
         if (MatchExpression::OR == child->matchType()) {
+            if (_orLimit == 0) {
+                LOGV2_DEBUG(4862500,
+                            1,
+                            "plan enumerator exceeded threshold for OR enumerations",
+                            "orEnumerationLimit"_attr = _orLimit);
+                _explainInfo.hitIndexedOrLimit = true;
+                return false;
+            }
             bool mandatory = expressionRequiresIndex(child);
             if (prepMemo(child, context)) {
                 size_t childID = memoIDForNode(child);
@@ -1333,7 +1359,6 @@ bool PlanEnumerator::prepSubNodes(MatchExpression* node,
             prepSubNodes(child, context, subnodesOut, mandatorySubnodes);
         }
     }
-
     return true;
 }
 
@@ -1469,7 +1494,7 @@ void PlanEnumerator::assignMultikeySafePredicates(
 
     size_t posInIdx = 0;
 
-    for (const auto keyElem : thisIndex.keyPattern) {
+    for (const auto& keyElem : thisIndex.keyPattern) {
         // Attempt to assign the predicates to 'thisIndex' according to their position in the index
         // key pattern.
         for (auto* couldAssignPred : couldAssign) {
@@ -1599,7 +1624,7 @@ void PlanEnumerator::compound(const vector<MatchExpression*>& tryCompound,
 //
 
 void PlanEnumerator::tagMemo(size_t id) {
-    LOGV2_DEBUG(20944, 5, "Tagging memoID {id}", "Tagging memoID", "id"_attr = id);
+    LOGV2_DEBUG(20944, 5, "Tagging memoID", "id"_attr = id);
     NodeAssignment* assign = _memo[id];
     verify(nullptr != assign);
 
@@ -1666,8 +1691,8 @@ bool PlanEnumerator::LockstepOrAssignment::allIdentical() const {
     return true;
 }
 
-bool PlanEnumerator::LockstepOrAssignment::shouldResetBeforeProceeding(size_t totalEnumerated,
-                                                                       size_t orLimit) const {
+bool PlanEnumerator::LockstepOrAssignment::shouldResetBeforeProceeding(
+    size_t totalEnumerated) const {
     if (totalEnumerated == 0 || !exhaustedLockstepIteration) {
         return false;
     }
@@ -1678,11 +1703,6 @@ bool PlanEnumerator::LockstepOrAssignment::shouldResetBeforeProceeding(size_t to
             return false;  // Haven't yet looped over this child entirely, not ready yet.
         }
         totalPossibleEnumerations *= subnode.maxIterCount.value();
-        // If 'totalPossibleEnumerations' reaches the limit, we can just shortcut it. Otherwise,
-        // 'totalPossibleEnumerations' could overflow if we have a large $or.
-        if (totalPossibleEnumerations >= orLimit) {
-            return false;
-        }
     }
 
     // If we're able to compute a total number expected enumerations, we must have already cycled
@@ -1719,7 +1739,7 @@ bool PlanEnumerator::_nextMemoForLockstepOrAssignment(
         }
         // Edge case: if every child has only one option available, we are already finished
         // enumerating.
-        if (assignment->shouldResetBeforeProceeding(assignment->totalEnumerated, _orLimit)) {
+        if (assignment->shouldResetBeforeProceeding(assignment->totalEnumerated)) {
             assignment->exhaustedLockstepIteration = false;
             return true;  // We're back at the beginning, no need to reset.
         }
@@ -1758,7 +1778,7 @@ bool PlanEnumerator::_nextMemoForLockstepOrAssignment(
     // This special ordering is tricky to reset. Because it iterates the sub nodes in such a
     // unique order, it can be difficult to know when it has actually finished iterating. Our
     // strategy is just to compute a total and go back to the beginning once we hit that total.
-    if (!assignment->shouldResetBeforeProceeding(assignment->totalEnumerated, _orLimit)) {
+    if (!assignment->shouldResetBeforeProceeding(assignment->totalEnumerated)) {
         return false;
     }
     // Reset!
@@ -1786,6 +1806,7 @@ bool PlanEnumerator::nextMemo(size_t id) {
                         1,
                         "plan enumerator exceeded threshold for OR enumerations",
                         "orEnumerationLimit"_attr = _orLimit);
+            _explainInfo.hitIndexedOrLimit = true;
             return true;
         }
 
@@ -1809,6 +1830,7 @@ bool PlanEnumerator::nextMemo(size_t id) {
                         1,
                         "plan enumerator exceeded threshold for OR enumerations",
                         "orEnumerationLimit"_attr = _orLimit);
+            _explainInfo.hitIndexedOrLimit = true;
             return true;
         }
         return _nextMemoForLockstepOrAssignment(assignment);
