@@ -29,8 +29,13 @@
 
 #pragma once
 
+#include <memory>
+
 #include "mongo/db/query/canonical_query.h"
 #include "mongo/db/query/index_bounds_builder.h"
+#include "mongo/db/query/index_tag.h"
+#include "mongo/db/query/interval_evaluation_tree.h"
+#include "mongo/db/query/query_planner.h"
 #include "mongo/db/query/query_planner_params.h"
 #include "mongo/db/query/query_solution.h"
 
@@ -100,7 +105,8 @@ public:
      */
     static std::unique_ptr<QuerySolutionNode> makeCollectionScan(const CanonicalQuery& query,
                                                                  bool tailable,
-                                                                 const QueryPlannerParams& params);
+                                                                 const QueryPlannerParams& params,
+                                                                 int direction = 1);
 
     /**
      * Return a plan that uses the provided index as a proxy for a collection scan.
@@ -138,8 +144,8 @@ private:
      */
     struct ScanBuildingState {
         ScanBuildingState(MatchExpression* theRoot,
-                          bool inArrayOp,
-                          const std::vector<IndexEntry>& indexList)
+                          const std::vector<IndexEntry>& indexList,
+                          bool inArrayOp)
             : root(theRoot),
               inArrayOperator(inArrayOp),
               indices(indexList),
@@ -155,15 +161,21 @@ private:
          * Reset the scan building state in preparation for building a new scan.
          *
          * This always should be called prior to allocating a new 'currentScan'.
+         *
+         * If `isQueryParameterized` is true an Interval Evaluation Tree will be built for every key
+         * element.
          */
-        void resetForNextScan(IndexTag* newTag) {
-            currentScan.reset(nullptr);
-            currentIndexNumber = newTag->index;
-            tightness = IndexBoundsBuilder::INEXACT_FETCH;
-            loosestBounds = IndexBoundsBuilder::EXACT;
+        void resetForNextScan(IndexTag* newTag, bool isQueryParameterized);
 
-            if (MatchExpression::OR == root->matchType()) {
-                curOr = std::make_unique<OrMatchExpression>();
+        interval_evaluation_tree::Builder* getCurrentIETBuilder() {
+            if (ietBuilders.empty()) {
+                return nullptr;
+            } else {
+                tassert(6334910,
+                        "IET Builder list size must be equal to the number of fields in the key "
+                        "pattern",
+                        ixtag->pos < ietBuilders.size());
+                return &ietBuilders[ixtag->pos];
             }
         }
 
@@ -214,6 +226,11 @@ private:
         // INEXACT_FETCH, then 'loosestBounds' is INEXACT_COVERED.
         IndexBoundsBuilder::BoundsTightness loosestBounds;
 
+        // The list of Interval Evaluation Tree builders used to build IETs for SBE. Every
+        // iet::Builder in the list corresponds to the corresponding element in the key pattern. The
+        // vector is empty if the query has no parameter markers.
+        std::vector<interval_evaluation_tree::Builder> ietBuilders;
+
     private:
         // Default constructor is not allowed.
         ScanBuildingState();
@@ -346,7 +363,8 @@ private:
         const IndexEntry& index,
         size_t pos,
         const MatchExpression* expr,
-        IndexBoundsBuilder::BoundsTightness* tightnessOut);
+        IndexBoundsBuilder::BoundsTightness* tightnessOut,
+        interval_evaluation_tree::Builder* ietBuilder);
 
     /**
      * Merge the predicate 'expr' with the leaf node 'node'.
@@ -369,7 +387,9 @@ private:
      * If geo, do nothing.
      * If text, punt to finishTextNode.
      */
-    static void finishLeafNode(QuerySolutionNode* node, const IndexEntry& index);
+    static void finishLeafNode(QuerySolutionNode* node,
+                               const IndexEntry& index,
+                               std::vector<interval_evaluation_tree::Builder> ietBuilders);
 
     /**
      * Fills in any missing bounds by calling finishLeafNode(...) for the scan contained in
@@ -397,7 +417,7 @@ private:
      * AND or an OR match expression.
      */
     static void addFilterToSolutionNode(QuerySolutionNode* node,
-                                        MatchExpression* match,
+                                        std::unique_ptr<MatchExpression> match,
                                         MatchExpression::MatchType type);
 
     /**
