@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/db/query/planner_ixselect.h"
 
@@ -40,7 +39,8 @@
 #include "mongo/db/index_names.h"
 #include "mongo/db/matcher/expression_algo.h"
 #include "mongo/db/matcher/expression_geo.h"
-#include "mongo/db/matcher/expression_internal_expr_eq.h"
+#include "mongo/db/matcher/expression_internal_bucket_geo_within.h"
+#include "mongo/db/matcher/expression_internal_expr_comparison.h"
 #include "mongo/db/matcher/expression_text.h"
 #include "mongo/db/query/canonical_query_encoder.h"
 #include "mongo/db/query/collation/collator_interface.h"
@@ -50,17 +50,22 @@
 #include "mongo/db/query/query_planner_common.h"
 #include "mongo/logv2/log.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
+
+
 namespace mongo {
 
 namespace {
 
 namespace wcp = ::mongo::wildcard_planning;
 
-// Can't index negations of {$eq: <Array>} or {$in: [<Array>, ...]}. Note that we could
-// use the index in principle, though we would need to generate special bounds.
-bool isEqualsArrayOrNotInArray(const MatchExpression* me) {
+// Can't index negations of {$eq: <Array>}, {$lt: <Array>}, {$gt: <Array>}, {$lte: <Array>}, {$gte:
+// <Array>}, or {$in: [<Array>, ...]}. Note that we could use the index in principle, though we
+// would need to generate special bounds.
+bool isComparisonWithArrayPred(const MatchExpression* me) {
     const auto type = me->matchType();
-    if (type == MatchExpression::EQ) {
+    if (type == MatchExpression::EQ || type == MatchExpression::LT || type == MatchExpression::GT ||
+        type == MatchExpression::LTE || type == MatchExpression::GTE) {
         return static_cast<const ComparisonMatchExpression*>(me)->getData().type() ==
             BSONType::Array;
     } else if (type == MatchExpression::MATCH_IN) {
@@ -69,7 +74,6 @@ bool isEqualsArrayOrNotInArray(const MatchExpression* me) {
             return elt.type() == BSONType::Array;
         });
     }
-
     return false;
 }
 
@@ -180,7 +184,7 @@ bool QueryPlannerIXSelect::canUseIndexForNin(const InMatchExpression* ime) {
     auto containsEmptyArray = [](const BSONElement& elt) {
         return elt.type() == Array && elt.embeddedObject().isEmpty();
     };
-    return ime->getRegexes().empty() && inList.size() == 2 &&
+    return !ime->hasRegex() && inList.size() == 2 &&
         std::any_of(inList.begin(), inList.end(), containsNull) &&
         std::any_of(inList.begin(), inList.end(), containsEmptyArray);
 }
@@ -189,14 +193,11 @@ bool QueryPlannerIXSelect::canUseIndexForNin(const InMatchExpression* ime) {
  * 2d indices don't handle wrapping so we can't use them for queries that wrap.
  */
 static bool twoDWontWrap(const Circle& circle, const IndexEntry& index) {
-    GeoHashConverter::Parameters hashParams;
-    Status paramStatus = GeoHashConverter::parseParameters(index.infoObj, &hashParams);
-    verify(paramStatus.isOK());  // we validated the params on index creation
-
-    GeoHashConverter conv(hashParams);
+    auto conv = GeoHashConverter::createFromDoc(index.infoObj);
+    uassertStatusOK(conv.getStatus());  // We validated the parameters when creating the index.
 
     // FYI: old code used flat not spherical error.
-    double yscandist = rad2deg(circle.radius) + conv.getErrorSphere();
+    double yscandist = rad2deg(circle.radius) + conv.getValue()->getErrorSphere();
     double xscandist = computeXScanDistance(circle.center.y, yscandist);
     bool ret = circle.center.x + xscandist < 180 && circle.center.x - xscandist > -180 &&
         circle.center.y + yscandist < 90 && circle.center.y - yscandist > -90;
@@ -246,7 +247,7 @@ static bool boundsGeneratingNodeContainsComparisonToType(MatchExpression* node,
 // static
 void QueryPlannerIXSelect::getFields(const MatchExpression* node,
                                      string prefix,
-                                     RelevantFieldIndexMap* out) {
+                                     stdx::unordered_set<string>* out) {
     // Do not traverse tree beyond a NOR negation node
     MatchExpression::MatchType exprtype = node->matchType();
     if (exprtype == MatchExpression::NOR) {
@@ -255,8 +256,7 @@ void QueryPlannerIXSelect::getFields(const MatchExpression* node,
 
     // Leaf nodes with a path and some array operators.
     if (Indexability::nodeCanUseIndexOnOwnField(node)) {
-        bool supportSparse = Indexability::nodeSupportedBySparseIndex(node);
-        (*out)[prefix + node->path().toString()] = {supportSparse};
+        out->insert(prefix + node->path().toString());
     } else if (Indexability::arrayUsesIndexOnChildren(node) && !node->path().empty()) {
         // If the array uses an index on its children, it's something like
         // {foo : {$elemMatch: {bar: 1}}}, in which case the predicate is really over foo.bar.
@@ -273,7 +273,8 @@ void QueryPlannerIXSelect::getFields(const MatchExpression* node,
     }
 }
 
-void QueryPlannerIXSelect::getFields(const MatchExpression* node, RelevantFieldIndexMap* out) {
+void QueryPlannerIXSelect::getFields(const MatchExpression* node,
+                                     stdx::unordered_set<string>* out) {
     getFields(node, "", out);
 }
 
@@ -289,7 +290,6 @@ std::vector<IndexEntry> QueryPlannerIXSelect::findIndexesByHint(
             if (entry.identifier.catalogName == hintName) {
                 LOGV2_DEBUG(20952,
                             5,
-                            "Hint by name specified, restricting indices to {keyPattern}",
                             "Hint by name specified, restricting indices",
                             "name"_attr = entry.identifier.catalogName,
                             "keyPattern"_attr = entry.keyPattern);
@@ -301,7 +301,6 @@ std::vector<IndexEntry> QueryPlannerIXSelect::findIndexesByHint(
             if (SimpleBSONObjComparator::kInstance.evaluate(entry.keyPattern == hintedIndex)) {
                 LOGV2_DEBUG(20953,
                             5,
-                            "Hint specified, restricting indices to {keyPattern}",
                             "Hint specified, restricting indices",
                             "name"_attr = entry.identifier.catalogName,
                             "keyPattern"_attr = entry.keyPattern);
@@ -315,40 +314,26 @@ std::vector<IndexEntry> QueryPlannerIXSelect::findIndexesByHint(
 
 // static
 std::vector<IndexEntry> QueryPlannerIXSelect::findRelevantIndices(
-    const RelevantFieldIndexMap& fields, const std::vector<IndexEntry>& allIndices) {
+    const stdx::unordered_set<std::string>& fields, const std::vector<IndexEntry>& allIndices) {
 
     std::vector<IndexEntry> out;
-    for (auto&& index : allIndices) {
-        BSONObjIterator it(index.keyPattern);
+    for (auto&& entry : allIndices) {
+        BSONObjIterator it(entry.keyPattern);
         BSONElement elt = it.next();
-        const std::string fieldName = elt.fieldNameStringData().toString();
-
-        // If the index is non-sparse we can use the field regardless its sparsity, otherwise we
-        // should find the field that can be answered by a sparse index.
-        if (fields.contains(fieldName) &&
-            (!index.sparse || fields.find(fieldName)->second.isSparse)) {
-            out.push_back(index);
+        if (fields.end() != fields.find(elt.fieldName())) {
+            out.push_back(entry);
         }
     }
 
     return out;
 }
 
-std::vector<IndexEntry> QueryPlannerIXSelect::expandIndexes(const RelevantFieldIndexMap& fields,
-                                                            std::vector<IndexEntry> relevantIndices,
-                                                            bool indexHinted) {
+std::vector<IndexEntry> QueryPlannerIXSelect::expandIndexes(
+    const stdx::unordered_set<std::string>& fields, std::vector<IndexEntry> relevantIndices) {
     std::vector<IndexEntry> out;
-    // Filter out fields that cannot be answered by any sparse index. We know wildcard indexes are
-    // sparse, so we don't want to expand the wildcard index based on such fields.
-    stdx::unordered_set<std::string> sparseIncompatibleFields;
-    for (auto&& [fieldName, idxProperty] : fields) {
-        if (idxProperty.isSparse || indexHinted) {
-            sparseIncompatibleFields.insert(fieldName);
-        }
-    }
     for (auto&& entry : relevantIndices) {
         if (entry.type == IndexType::INDEX_WILDCARD) {
-            wcp::expandWildcardIndexEntry(entry, sparseIncompatibleFields, &out);
+            wcp::expandWildcardIndexEntry(entry, fields, &out);
         } else {
             out.push_back(std::move(entry));
         }
@@ -400,9 +385,10 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
     // We know keyPatternElt.fieldname() == node->path().
     MatchExpression::MatchType exprtype = node->matchType();
 
-    if (exprtype == MatchExpression::INTERNAL_EXPR_EQ &&
+    if (ComparisonMatchExpressionBase::isInternalExprComparison(exprtype) &&
         index.pathHasMultikeyComponent(keyPatternElt.fieldNameStringData())) {
-        // Expression language equality cannot be indexed if the field path has multikey components.
+        // Expression language comparisons cannot be indexed if the field path has multikey
+        // components.
         return false;
     }
 
@@ -413,7 +399,8 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
         }
 
         // We can't use a btree-indexed field for geo expressions.
-        if (exprtype == MatchExpression::GEO || exprtype == MatchExpression::GEO_NEAR) {
+        if (exprtype == MatchExpression::GEO || exprtype == MatchExpression::GEO_NEAR ||
+            exprtype == MatchExpression::INTERNAL_BUCKET_GEO_WITHIN) {
             return false;
         }
 
@@ -448,6 +435,12 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
                 return false;
             }
 
+            // $gt and $lt to MinKey/MaxKey must build inexact bounds if the index is multikey and
+            // therefore cannot be inverted safely in a $not.
+            if (index.multikey && (child->isGTMinKey() || child->isLTMaxKey())) {
+                return false;
+            }
+
             // Most of the time we can't use a multikey index for a $ne: null query, however there
             // are a few exceptions around $elemMatch.
             const bool isNotEqualsNull = isQueryNegatingEqualToNull(node);
@@ -477,7 +470,10 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
                 }
             }
 
-            if (isEqualsArrayOrNotInArray(child)) {
+            // Comparisons with arrays have strange enough semantics that inverting the bounds
+            // within a $not has many complex special cases. We avoid indexing these queries, even
+            // though it is sometimes possible to build useful bounds.
+            if (isComparisonWithArrayPred(child)) {
                 return false;
             }
         }
@@ -499,10 +495,15 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
             }
 
             auto&& children = node->getChildVector();
-            if (!std::all_of(children->begin(), children->end(), [&](MatchExpression* child) {
+            if (!std::all_of(children->begin(), children->end(), [&](auto&& child) {
                     const auto newPath = fullPathToNode.toString() + child->path();
-                    return _compatible(
-                        keyPatternElt, index, keyPatternIdx, child, newPath, collator, newContext);
+                    return _compatible(keyPatternElt,
+                                       index,
+                                       keyPatternIdx,
+                                       child.get(),
+                                       newPath,
+                                       collator,
+                                       newContext);
                 })) {
                 return false;
             }
@@ -569,6 +570,14 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
             return gnme->getData().centroid->crs == SPHERE;
         }
         return false;
+    } else if (IndexNames::GEO_2DSPHERE_BUCKET == indexedFieldType) {
+        if (exprtype == MatchExpression::INTERNAL_BUCKET_GEO_WITHIN) {
+            const InternalBucketGeoWithinMatchExpression* ibgwme =
+                static_cast<InternalBucketGeoWithinMatchExpression*>(node);
+            auto gc = ibgwme->getGeoContainer();
+            return gc.hasS2Region();
+        }
+        return false;
     } else if (IndexNames::GEO_2D == indexedFieldType) {
         if (exprtype == MatchExpression::GEO_NEAR) {
             GeoNearMatchExpression* gnme = static_cast<GeoNearMatchExpression*>(node);
@@ -609,12 +618,12 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
         return false;
     } else {
         LOGV2_WARNING(20954,
-                      "Unknown indexing for node {node} and field {field}",
                       "Unknown indexing for given node and field",
                       "node"_attr = node->debugString(),
                       "field"_attr = keyPatternElt.toString());
         verify(0);
     }
+    MONGO_UNREACHABLE;
 }
 
 bool QueryPlannerIXSelect::nodeIsSupportedBySparseIndex(const MatchExpression* queryExpr,
@@ -664,7 +673,7 @@ bool QueryPlannerIXSelect::nodeIsSupportedBySparseIndex(const MatchExpression* q
 
 bool QueryPlannerIXSelect::logicalNodeMayBeSupportedByAnIndex(const MatchExpression* queryExpr) {
     return !(queryExpr->matchType() == MatchExpression::NOT &&
-             isEqualsArrayOrNotInArray(queryExpr->getChild(0)));
+             isComparisonWithArrayPred(queryExpr->getChild(0)));
 }
 
 bool QueryPlannerIXSelect::nodeIsSupportedByWildcardIndex(const MatchExpression* queryExpr) {
