@@ -186,12 +186,6 @@ void addNodeAtPathHelper(ProjectionPathASTNode* root,
     addNodeAtPathHelper(childPathNode, path, componentIndex + 1, std::move(newChild));
 }
 
-void addNodeAtPath(ProjectionPathASTNode* root,
-                   const FieldPath& path,
-                   std::unique_ptr<ASTNode> newChild) {
-    addNodeAtPathHelper(root, path, 0, std::move(newChild));
-}
-
 bool hasPositionalOperator(StringData path) {
     return path.endsWith(".$");
 }
@@ -292,7 +286,7 @@ bool attemptToParseGenericExpression(ParseContext* parseCtx,
     }
 
     auto expr = Expression::parseExpression(
-        parseCtx->expCtx, subObj, parseCtx->expCtx->variablesParseState);
+        parseCtx->expCtx.get(), subObj, parseCtx->expCtx->variablesParseState);
     addNodeAtPath(parent, path, std::make_unique<ExpressionASTNode>(expr));
     parseCtx->hasMeta = parseCtx->hasMeta || isMeta;
     return true;
@@ -413,7 +407,7 @@ void parseInclusion(ParseContext* ctx,
                 elemFieldName != ".$");
 
         // Get everything up to the first positional operator.
-        uassert(5392901,
+        tassert(5392901,
                 "Expected element field name size to be greater than 2",
                 elemFieldName.size() > 2);
         StringData pathWithoutPositionalOperator =
@@ -421,11 +415,17 @@ void parseInclusion(ParseContext* ctx,
 
         FieldPath path(pathWithoutPositionalOperator);
 
-        auto matcher = CopyableMatchExpression{ctx->queryObj,
-                                               ctx->expCtx,
-                                               std::make_unique<ExtensionsCallbackNoop>(),
-                                               MatchExpressionParser::kBanAllSpecialFeatures,
-                                               true /* optimize expression */};
+        // Parse the match expression in order to ensure that no special features are used with
+        // positional projection.
+        uassertStatusOK(
+            MatchExpressionParser::parse(ctx->queryObj,
+                                         ctx->expCtx,
+                                         ExtensionsCallbackNoop{},
+                                         MatchExpressionParser::kBanAllSpecialFeatures));
+
+        // Copy the original match expression, which makes sure to preserve any input parameter ids
+        // attached to the tree.
+        CopyableMatchExpression matcher{ctx->queryObj, ctx->query->shallowClone()};
 
         invariant(ctx->query);
         addNodeAtPath(parent,
@@ -471,7 +471,7 @@ void parseExclusion(ParseContext* ctx, BSONElement elem, ProjectionPathASTNode*
 void parseLiteral(ParseContext* ctx, BSONElement elem, ProjectionPathASTNode* parent) {
     verifyComputedFieldsAllowed(ctx->policies);
 
-    auto expr = Expression::parseOperand(ctx->expCtx, elem, ctx->expCtx->variablesParseState);
+    auto expr = Expression::parseOperand(ctx->expCtx.get(), elem, ctx->expCtx->variablesParseState);
 
     FieldPath pathFromParent(elem.fieldNameStringData());
     addNodeAtPath(parent, pathFromParent, std::make_unique<ExpressionASTNode>(expr));
@@ -586,13 +586,13 @@ void parseElement(ParseContext* ctx,
 }
 }  // namespace
 
-Projection parse(boost::intrusive_ptr<ExpressionContext> expCtx,
-                 const BSONObj& obj,
-                 const MatchExpression* const query,
-                 const BSONObj& queryObj,
-                 ProjectionPolicies policies,
-                 bool shouldOptimize) {
-    if (!policies.findOnlyFeaturesAllowed()) {
+Projection parseAndAnalyze(boost::intrusive_ptr<ExpressionContext> expCtx,
+                           const BSONObj& obj,
+                           const MatchExpression* const query,
+                           const BSONObj& queryObj,
+                           ProjectionPolicies policies,
+                           bool shouldOptimize) {
+    if (!policies.emptyProjectionAllowed()) {
         // In agg-style syntax it is illegal to have an empty projection specification.
         uassert(51272, "projection specification must have at least one field", !obj.isEmpty());
     }
@@ -637,18 +637,29 @@ Projection parse(boost::intrusive_ptr<ExpressionContext> expCtx,
         invariant(root.removeChild("_id"));
     }
 
-    if (shouldOptimize) {
+    // Optimize the projection expression if requested and as long as not explicitly disabled
+    // pipeline optimization.
+    auto fp = globalFailPointRegistry().find("disablePipelineOptimization");
+    if (shouldOptimize && !(fp && fp->shouldFail())) {
         optimizeProjection(&root);
     }
 
     return Projection{std::move(root), *ctx.type};
 }
 
-Projection parse(boost::intrusive_ptr<ExpressionContext> expCtx,
-                 const BSONObj& obj,
-                 ProjectionPolicies policies,
-                 bool shouldOptimize) {
-    return parse(std::move(expCtx), obj, nullptr, BSONObj(), std::move(policies), shouldOptimize);
+Projection parseAndAnalyze(boost::intrusive_ptr<ExpressionContext> expCtx,
+                           const BSONObj& obj,
+                           ProjectionPolicies policies,
+                           bool shouldOptimize) {
+    return parseAndAnalyze(
+        std::move(expCtx), obj, nullptr, BSONObj(), std::move(policies), shouldOptimize);
 }
+
+void addNodeAtPath(ProjectionPathASTNode* root,
+                   const FieldPath& path,
+                   std::unique_ptr<ASTNode> newChild) {
+    addNodeAtPathHelper(root, path, 0, std::move(newChild));
+}
+
 }  // namespace projection_ast
 }  // namespace mongo
