@@ -43,6 +43,49 @@
 #include "mongo/db/query/query_test_service_context.h"
 #include "mongo/unittest/unittest.h"
 
+namespace mongo {
+
+/**
+ * Output stream operator for FieldAvailability values is required by ASSERT_ macros in tests.
+ */
+std::ostream& operator<<(std::ostream& os, FieldAvailability value) {
+    switch (value) {
+        case FieldAvailability::kNotProvided:
+            return os << "NotProvided";
+        case FieldAvailability::kHashedValueProvided:
+            return os << "HashedValueProvided";
+        case FieldAvailability::kFullyProvided:
+            return os << "FullyProvided";
+    }
+    MONGO_UNREACHABLE;
+}
+
+/**
+ * Output stream operator for ProvidedSortSet instances is required by ASSERT_ macros in tests.
+ */
+std::ostream& operator<<(std::ostream& os, const ProvidedSortSet& value) {
+    return os << value.debugString();
+}
+
+/**
+ * Equality operator for ProvidedSortSet instances is required by ASSERT_EQ macros in tests.
+ * This operator uses 'BSONObj::woCompare()' method for comparing base sort patterns.
+ */
+bool operator==(const ProvidedSortSet& lhs, const ProvidedSortSet& rhs) {
+    return (lhs.getIgnoredFields() == rhs.getIgnoredFields() &&
+            lhs.getBaseSortPattern().woCompare(rhs.getBaseSortPattern()) == 0);
+}
+
+/**
+ * Non-equality operator for ProvidedSortSet instances is required by ASSERT_NE macros in tests.
+ * This operator uses 'BSONObj::woCompare()' method for comparing base sort patterns.
+ */
+bool operator!=(const ProvidedSortSet& lhs, const ProvidedSortSet& rhs) {
+    return !(lhs == rhs);
+}
+
+}  // namespace mongo
+
 namespace {
 
 using namespace mongo;
@@ -52,6 +95,7 @@ using namespace mongo;
 IndexEntry buildSimpleIndexEntry(const BSONObj& kp) {
     return {kp,
             IndexNames::nameToType(IndexNames::findPluginName(kp)),
+            IndexDescriptor::kLatestIndexVersion,
             false,
             {},
             {},
@@ -64,6 +108,18 @@ IndexEntry buildSimpleIndexEntry(const BSONObj& kp) {
             nullptr};
 }
 
+void assertNamespaceVectorsAreEqual(const std::vector<NamespaceStringOrUUID>& secondaryNssVector,
+                                    const std::vector<NamespaceStringOrUUID>& expectedNssVector) {
+    ASSERT_EQ(secondaryNssVector.size(), expectedNssVector.size());
+    for (size_t i = 0; i < secondaryNssVector.size(); ++i) {
+        auto secondary = secondaryNssVector[i].nss();
+        auto expected = expectedNssVector[i].nss();
+        ASSERT(secondary != boost::none);
+        ASSERT(expected != boost::none);
+        ASSERT_EQ(*secondary, *expected);
+    }
+}
+
 // Index: {a: 1, b: 1, c: 1, d: 1, e: 1}
 // Min: {a: 1, b: 1, c: 1, d: 1, e: 1}
 // Max: {a: 1, b: 1, c: 1, d: 1, e: 1}
@@ -76,16 +132,17 @@ TEST(QuerySolutionTest, SimpleRangeAllEqual) {
     node.computeProperties();
 
     // Expected sort orders
-    ASSERT_EQUALS(node.getSort().size(), 9U);
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1)));
-    ASSERT(node.getSort().count(BSON("b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("e" << 1)));
+    ASSERT_EQ(node.providedSorts(), ProvidedSortSet(BSONObj(), {"a", "b", "c", "d", "e"}));
+
+    ASSERT(node.providedSorts().contains(
+        BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << -1 << "c" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << -1 << "b" << -1)));
+    ASSERT(node.providedSorts().contains(BSON("d" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("b" << 1 << "c" << 1 << "e" << 1 << "d" << -1)));
+    ASSERT(node.providedSorts().contains(BSON("c" << -1 << "b" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("b" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("e" << 1)));
 }
 
 // Index: {a: 1, b: 1, c: 1, d: 1, e: 1}
@@ -100,12 +157,15 @@ TEST(QuerySolutionTest, SimpleRangeNoneEqual) {
     node.computeProperties();
 
     // Expected sort orders
-    ASSERT_EQUALS(node.getSort().size(), 5U);
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1)));
+    ASSERT_EQ(node.providedSorts(),
+              ProvidedSortSet(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1), {}));
+    ASSERT(node.providedSorts().contains(
+        BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1 << "c" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1)));
+    ASSERT_FALSE(node.providedSorts().contains(BSON("b" << 1)));
 }
 
 // Index: {a: 1, b: 1, c: 1, d: 1, e: 1}
@@ -120,16 +180,19 @@ TEST(QuerySolutionTest, SimpleRangeSomeEqual) {
     node.computeProperties();
 
     // Expected sort orders
-    ASSERT_EQUALS(node.getSort().size(), 9U);
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1)));
-    ASSERT(node.getSort().count(BSON("b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("c" << 1 << "d" << 1)));
-    ASSERT(node.getSort().count(BSON("c" << 1)));
+    ASSERT_EQ(node.providedSorts(),
+              ProvidedSortSet(BSON("c" << 1 << "d" << 1 << "e" << 1), {"a", "b"}));
+
+    ASSERT(node.providedSorts().contains(
+        BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1 << "c" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("c" << 1 << "d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("c" << 1 << "d" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("c" << 1)));
 }
 
 // Index: {a: 1, b: 1, c: 1, d: 1, e: 1}
@@ -165,17 +228,28 @@ TEST(QuerySolutionTest, IntervalListAllPoints) {
 
     node.computeProperties();
 
-    // Expected sort orders
-    ASSERT_EQUALS(node.getSort().size(), 9U);
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1)));
-    ASSERT(node.getSort().count(BSON("b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("e" << 1)));
+    // Expected internal state of 'ProvidedSortSet'.
+    ASSERT_EQ(node.providedSorts(), ProvidedSortSet(BSONObj(), {"a", "b", "c", "d", "e"}));
+
+    // Expected sort orders.
+    ASSERT(node.providedSorts().contains(
+        BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << -1 << "b" << 1 << "c" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("c" << 1 << "d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("d" << 1 << "e" << 1 << "b" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("b" << 1 << "d" << 1 << "a" << 1)));
+
+    // Verify that the order of equality fields is irrelvant.
+    ASSERT(node.providedSorts().contains(BSON("a" << -1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("b" << -1 << "d" << -1 << "a" << -1)));
+    ASSERT(node.providedSorts().contains(BSON("d" << 1 << "e" << -1)));
+    ASSERT(node.providedSorts().contains(BSON("b" << 1 << "c" << -1 << "d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << -1)));
 }
 
 
@@ -218,12 +292,15 @@ TEST(QuerySolutionTest, IntervalListNoPoints) {
     node.computeProperties();
 
     // Expected sort orders
-    ASSERT_EQUALS(node.getSort().size(), 5U);
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1)));
+    ASSERT_EQ(node.providedSorts(),
+              ProvidedSortSet(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1), {}));
+
+    ASSERT(node.providedSorts().contains(
+        BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1 << "c" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1)));
 }
 
 // Index: {a: 1, b: 1, c: 1, d: 1, e: 1}
@@ -263,16 +340,25 @@ TEST(QuerySolutionTest, IntervalListSomePoints) {
     node.computeProperties();
 
     // Expected sort orders
-    ASSERT_EQUALS(node.getSort().size(), 9U);
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1 << "c" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1)));
-    ASSERT(node.getSort().count(BSON("b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("c" << 1 << "d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("c" << 1 << "d" << 1)));
-    ASSERT(node.getSort().count(BSON("c" << 1)));
+    ASSERT_EQ(node.providedSorts(),
+              ProvidedSortSet(BSON("c" << 1 << "d" << 1 << "e" << 1), {"a", "b"}));
+
+    ASSERT(node.providedSorts().contains(
+        BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1 << "c" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("b" << 1 << "c" << 1 << "d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("c" << 1 << "d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("c" << 1 << "d" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("c" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("c" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "c" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "c" << 1 << "d" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << -1 << "c" << 1 << "d" << 1 << "b" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("c" << 1 << "b" << -1 << "d" << 1 << "a" << 1)));
+    ASSERT_FALSE(node.providedSorts().contains(BSON("a" << 1 << "d" << 1)));
 }
 
 TEST(QuerySolutionTest, GetFieldsWithStringBoundsIdentifiesFieldsContainingStrings) {
@@ -457,9 +543,9 @@ TEST(QuerySolutionTest, IndexScanNodeRemovesNonMatchingCollatedFieldsFromSortsOn
 
     node.computeProperties();
 
-    BSONObjSet sorts = node.getSort();
-    ASSERT_EQUALS(sorts.size(), 1U);
-    ASSERT_TRUE(sorts.count(BSON("a" << 1)));
+    auto sorts = node.providedSorts();
+    ASSERT_EQ(sorts, ProvidedSortSet(BSON("a" << 1), {}));
+    ASSERT_TRUE(sorts.contains(BSON("a" << 1)));
 }
 
 TEST(QuerySolutionTest, IndexScanNodeGetFieldsWithStringBoundsCorrectlyHandlesEndKeyInclusive) {
@@ -472,22 +558,24 @@ TEST(QuerySolutionTest, IndexScanNodeGetFieldsWithStringBoundsCorrectlyHandlesEn
     node.bounds.endKey = BSON("a" << 1 << "b"
                                   << "");
     node.bounds.boundInclusion = BoundInclusion::kIncludeStartKeyOnly;
-
     node.computeProperties();
 
-    BSONObjSet sorts = node.getSort();
-    ASSERT_EQUALS(sorts.size(), 3U);
-    ASSERT_TRUE(sorts.count(BSON("a" << 1)));
-    ASSERT_TRUE(sorts.count(BSON("a" << 1 << "b" << 1)));
-    ASSERT_TRUE(sorts.count(BSON("b" << 1)));
+    auto sorts = node.providedSorts();
+    ASSERT_EQ(sorts, ProvidedSortSet(BSON("b" << 1), {"a"}));
+
+    ASSERT_TRUE(sorts.contains(BSON("a" << 1)));
+    ASSERT_TRUE(sorts.contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT_TRUE(sorts.contains(BSON("b" << 1)));
 
     node.bounds.boundInclusion = BoundInclusion::kIncludeBothStartAndEndKeys;
-
     node.computeProperties();
 
-    sorts = node.getSort();
-    ASSERT_EQUALS(sorts.size(), 1U);
-    ASSERT_TRUE(sorts.count(BSON("a" << 1)));
+    sorts = node.providedSorts();
+    ASSERT_EQ(sorts, ProvidedSortSet(BSONObj(), {"a"}));
+
+    ASSERT_TRUE(sorts.contains(BSON("a" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b" << 1)));
 }
 
 // Index: {a: 1}
@@ -505,8 +593,9 @@ TEST(QuerySolutionTest, IndexScanNodeRemovesCollatedFieldsFromSortsIfCollationDi
 
     node.computeProperties();
 
-    BSONObjSet sorts = node.getSort();
-    ASSERT_EQUALS(sorts.size(), 0U);
+    auto sorts = node.providedSorts();
+    ASSERT_EQ(sorts, kEmptySet);
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1)));
 }
 
 TEST(QuerySolutionTest, IndexScanNodeDoesNotRemoveCollatedFieldsFromSortsIfCollationMatches) {
@@ -521,13 +610,14 @@ TEST(QuerySolutionTest, IndexScanNodeDoesNotRemoveCollatedFieldsFromSortsIfColla
 
     node.computeProperties();
 
-    BSONObjSet sorts = node.getSort();
-    ASSERT_EQUALS(sorts.size(), 1U);
-    ASSERT_TRUE(sorts.count(BSON("a" << 1)));
+    auto sorts = node.providedSorts();
+    ASSERT_EQ(sorts, ProvidedSortSet(BSON("a" << 1), {}));
+
+    ASSERT_TRUE(sorts.contains(BSON("a" << 1)));
 }
 
 // Index: {a: 1, b: 1, c: 1, d: 1, e: 1}
-// Intervals: a: [1,1], b: [1,1], c: [MinKey, MaxKey], d: [1,2], e: [1,2]
+// Intervals: a: [1,1], b: ['p','p'], c: [1,2], d: [MinKey, MaxKey], e: [1,2]
 TEST(QuerySolutionTest, CompoundIndexWithNonMatchingCollationFiltersAllSortsWithCollatedField) {
     IndexScanNode node{
         buildSimpleIndexEntry(BSON("a" << 1 << "b" << 1 << "c" << 1 << "d" << 1 << "e" << 1))};
@@ -544,19 +634,20 @@ TEST(QuerySolutionTest, CompoundIndexWithNonMatchingCollationFiltersAllSortsWith
 
     OrderedIntervalList b{};
     b.name = "b";
-    b.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON("" << 1)));
+    b.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON(""
+                                                                     << "p")));
     node.bounds.fields.push_back(b);
 
     OrderedIntervalList c{};
     c.name = "c";
     c.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
-        BSON("" << MINKEY << "" << MAXKEY), BoundInclusion::kIncludeBothStartAndEndKeys));
+        BSON("" << 1 << "" << 2), BoundInclusion::kIncludeBothStartAndEndKeys));
     node.bounds.fields.push_back(c);
 
     OrderedIntervalList d{};
     d.name = "d";
     d.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
-        BSON("" << 1 << "" << 2), BoundInclusion::kIncludeBothStartAndEndKeys));
+        BSON("" << MINKEY << "" << MAXKEY), BoundInclusion::kIncludeBothStartAndEndKeys));
     node.bounds.fields.push_back(d);
 
     OrderedIntervalList e{};
@@ -568,9 +659,13 @@ TEST(QuerySolutionTest, CompoundIndexWithNonMatchingCollationFiltersAllSortsWith
     node.computeProperties();
 
     // Expected sort orders
-    ASSERT_EQUALS(node.getSort().size(), 2U);
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1)));
+    ASSERT_EQ(node.providedSorts(), ProvidedSortSet(BSON("c" << 1), {"a"}));
+
+    ASSERT(node.providedSorts().contains(BSON("a" << 1)));
+    ASSERT_FALSE(node.providedSorts().contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "c" << 1)));
+    ASSERT_FALSE(node.providedSorts().contains(BSON("a" << 1 << "c" << 1 << "d" << 1)));
+    ASSERT_FALSE(node.providedSorts().contains(BSON("a" << 1 << "c" << 1 << "e" << 1)));
 }
 
 // Index: {a : 1}
@@ -589,8 +684,7 @@ TEST(QuerySolutionTest, IndexScanNodeWithNonMatchingCollationFiltersObjectField)
 
     node.computeProperties();
 
-    BSONObjSet sorts = node.getSort();
-    ASSERT_EQUALS(sorts.size(), 0U);
+    ASSERT_EQ(node.providedSorts(), kEmptySet);
 }
 
 // Index: {a : 1}
@@ -609,8 +703,7 @@ TEST(QuerySolutionTest, IndexScanNodeWithNonMatchingCollationFiltersArrayField)
 
     node.computeProperties();
 
-    BSONObjSet sorts = node.getSort();
-    ASSERT_EQUALS(sorts.size(), 0U);
+    ASSERT_EQ(node.providedSorts(), kEmptySet);
 }
 
 TEST(QuerySolutionTest, WithNonMatchingCollatorAndNoEqualityPrefixSortsAreNotDuplicated) {
@@ -633,12 +726,11 @@ TEST(QuerySolutionTest, WithNonMatchingCollatorAndNoEqualityPrefixSortsAreNotDup
                                                    << "b"),
                                               BoundInclusion::kIncludeBothStartAndEndKeys));
     node.bounds.fields.push_back(oilB);
-
     node.computeProperties();
 
     // Expected sort orders
-    ASSERT_EQUALS(node.getSort().size(), 1U);
-    ASSERT(node.getSort().count(BSON("a" << 1)));
+    ASSERT_EQ(node.providedSorts(), ProvidedSortSet(BSON("a" << 1), {}));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1)));
 }
 
 TEST(QuerySolutionTest, IndexScanNodeHasFieldIncludesStringFieldWhenNoCollator) {
@@ -719,10 +811,9 @@ auto createMatchExprAndProjection(const BSONObj& query, const BSONObj& projObj)
     auto opCtx = serviceCtx.makeOperationContext();
     const boost::intrusive_ptr<ExpressionContext> expCtx(new ExpressionContext(
         opCtx.get(), std::unique_ptr<CollatorInterface>(nullptr), NamespaceString("test.dummy")));
-    StatusWithMatchExpression queryMatchExpr =
-        MatchExpressionParser::parse(query, std::move(expCtx));
+    StatusWithMatchExpression queryMatchExpr = MatchExpressionParser::parse(query, expCtx);
     ASSERT(queryMatchExpr.isOK());
-    projection_ast::Projection res = projection_ast::parse(
+    projection_ast::Projection res = projection_ast::parseAndAnalyze(
         expCtx, projObj, queryMatchExpr.getValue().get(), query, ProjectionPolicies{});
     return std::make_pair(std::move(queryMatchExpr.getValue()), std::move(res));
 }
@@ -740,8 +831,8 @@ TEST(QuerySolutionTest, InclusionProjectionPreservesSort) {
         std::move(node), *matchExprAndProjection.first, matchExprAndProjection.second};
     proj.computeProperties();
 
-    ASSERT_EQ(proj.getSort().size(), 1U);
-    ASSERT(proj.getSort().count(BSON("a" << 1)));
+    ASSERT_EQ(proj.providedSorts(), ProvidedSortSet(BSON("a" << 1), {}));
+    ASSERT(proj.providedSorts().contains(BSON("a" << 1)));
 }
 
 TEST(QuerySolutionTest, ExclusionProjectionDoesNotPreserveSort) {
@@ -757,7 +848,8 @@ TEST(QuerySolutionTest, ExclusionProjectionDoesNotPreserveSort) {
         std::move(node), *matchExprAndProjection.first, matchExprAndProjection.second};
     proj.computeProperties();
 
-    ASSERT_EQ(proj.getSort().size(), 0U);
+    ASSERT_EQ(proj.providedSorts(), kEmptySet);
+    ASSERT(!proj.providedSorts().contains(BSON("a" << 1)));
 }
 
 TEST(QuerySolutionTest, InclusionProjectionTruncatesSort) {
@@ -772,8 +864,8 @@ TEST(QuerySolutionTest, InclusionProjectionTruncatesSort) {
         std::move(node), *matchExprAndProjection.first, matchExprAndProjection.second};
     proj.computeProperties();
 
-    ASSERT_EQ(proj.getSort().size(), 1U);
-    ASSERT(proj.getSort().count(BSON("a" << 1)));
+    ASSERT_EQ(proj.providedSorts(), ProvidedSortSet(BSON("a" << 1), {}));
+    ASSERT(proj.providedSorts().contains(BSON("a" << 1)));
 }
 
 TEST(QuerySolutionTest, ExclusionProjectionTruncatesSort) {
@@ -788,8 +880,9 @@ TEST(QuerySolutionTest, ExclusionProjectionTruncatesSort) {
         std::move(node), *matchExprAndProjection.first, matchExprAndProjection.second};
     proj.computeProperties();
 
-    ASSERT_EQ(proj.getSort().size(), 1U);
-    ASSERT(proj.getSort().count(BSON("a" << 1)));
+    ASSERT_EQ(proj.providedSorts(), ProvidedSortSet(BSON("a" << 1), {}));
+    ASSERT(proj.providedSorts().contains(BSON("a" << 1)));
+    ASSERT_FALSE(proj.providedSorts().contains(BSON("a" << 1 << "b" << 1)));
 }
 
 TEST(QuerySolutionTest, NonMultikeyIndexWithoutPathLevelInfoCanCoverItsFields) {
@@ -877,27 +970,36 @@ TEST(QuerySolutionTest, MultikeyIndexWithoutPathLevelInfoCannotProvideAnySorts)
     }
 
     node.computeProperties();
-    ASSERT(node.getSort().empty());
+    ASSERT_EQ(node.providedSorts(), kEmptySet);
 }
 
-TEST(QuerySolutionTest, SimpleRangeAllEqualExcludesFieldWithMultikeyComponent) {
+// Index: {a: 1, b: 1, 'c.z': 1, d: 1, e: 1}
+// Intervals: a: [1,1], b: [1,1], 'c.z': [1,1], d: [1, 1], e: [1,2]
+// Multikeys: ['b', 'c.z']
+TEST(QuerySolutionTest, SimpleRangeWithEqualIgnoresFieldWithMultikeyComponent) {
     IndexScanNode node{
         buildSimpleIndexEntry(BSON("a" << 1 << "b" << 1 << "c.z" << 1 << "d" << 1 << "e" << 1))};
     node.bounds.isSimpleRange = true;
     node.bounds.startKey = BSON("a" << 1 << "b" << 1 << "c.z" << 1 << "d" << 1 << "e" << 1);
-    node.bounds.endKey = BSON("a" << 1 << "b" << 1 << "c.z" << 1 << "d" << 1 << "e" << 1);
+    node.bounds.endKey = BSON("a" << 1 << "b" << 1 << "c.z" << 1 << "d" << 1 << "e" << 2);
 
-    // Add metadata indicating that "c.z" is multikey.
+    // Add metadata indicating that 'b', 'c.z' is multikey.
     node.index.multikey = true;
-    node.index.multikeyPaths = MultikeyPaths{{}, {}, {1U}, {}, {}};
+    node.index.multikeyPaths = MultikeyPaths{{}, {1U}, {1U}, {}, {}};
 
     node.computeProperties();
 
-    ASSERT_EQUALS(node.getSort().size(), 4U);
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1)));
-    ASSERT(node.getSort().count(BSON("d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("e" << 1)));
+    ASSERT_EQ(node.providedSorts(), ProvidedSortSet(BSON("e" << 1), {"a", "d"}));
+
+    ASSERT(node.providedSorts().contains(BSON("a" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "d" << -1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("e" << 1)));
+
+    // Cannot provide sorts that include a multikey field.
+    ASSERT_FALSE(node.providedSorts().contains(BSON("a" << 1 << "b" << -1)));
+    ASSERT_FALSE(node.providedSorts().contains(BSON("a" << 1 << "c.z" << 1)));
+    ASSERT_FALSE(node.providedSorts().contains(BSON("c.z" << 1)));
 }
 
 TEST(QuerySolutionTest, MultikeyFieldsEmptyWhenIndexIsNotMultikey) {
@@ -941,11 +1043,13 @@ TEST(QuerySolutionTest, NonSimpleRangeAllEqualExcludesFieldWithMultikeyComponent
     }
 
     node.computeProperties();
-    ASSERT_EQUALS(node.getSort().size(), 4U);
-    ASSERT(node.getSort().count(BSON("a" << 1 << "b" << 1)));
-    ASSERT(node.getSort().count(BSON("a" << 1)));
-    ASSERT(node.getSort().count(BSON("d" << 1 << "e" << 1)));
-    ASSERT(node.getSort().count(BSON("e" << 1)));
+    ASSERT_EQ(node.providedSorts(), ProvidedSortSet(BSONObj(), {"a", "b", "d", "e"}));
+
+    ASSERT(node.providedSorts().contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("a" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("d" << 1 << "e" << 1)));
+    ASSERT(node.providedSorts().contains(BSON("e" << 1)));
+    ASSERT_FALSE(node.providedSorts().contains(BSON("a" << 1 << "b" << 1 << "c.z" << 1)));
 }
 
 TEST(QuerySolutionTest, SharedPrefixMultikeyNonMinMaxBoundsDoesNotProvideAnySorts) {
@@ -968,6 +1072,282 @@ TEST(QuerySolutionTest, SharedPrefixMultikeyNonMinMaxBoundsDoesNotProvideAnySort
     }
 
     node.computeProperties();
-    ASSERT_EQUALS(node.getSort().size(), 0U);
+    ASSERT_EQ(node.providedSorts(), kEmptySet);
+}
+
+TEST(QuerySolutionTest, NodeIdsAssignedInPostOrderFashionStartingFromOne) {
+    // Construct a QuerySolution consisting of a root node with two children.
+    std::vector<std::unique_ptr<QuerySolutionNode>> children;
+    children.push_back(std::make_unique<IndexScanNode>(buildSimpleIndexEntry(BSON("a" << 1))));
+    children.push_back(std::make_unique<IndexScanNode>(buildSimpleIndexEntry(BSON("b" << 1))));
+    auto orNode = std::make_unique<OrNode>();
+    orNode->addChildren(std::move(children));
+
+    // Before being added to the QuerySolution, all the nodes should have a nodeId of zero, which
+    // means that an id has not yet been assigned.
+    ASSERT_EQ(orNode->nodeId(), 0u);
+    ASSERT_EQ(orNode->children[0]->nodeId(), 0u);
+    ASSERT_EQ(orNode->children[1]->nodeId(), 0u);
+
+    auto querySolution = std::make_unique<QuerySolution>();
+    querySolution->setRoot(std::move(orNode));
+    auto root = querySolution->root();
+
+    // Since ids are assigned according to a post-order traversal, the root node should have id 3,
+    // the left child should have id 1, and the right child should have id 2.
+    ASSERT_EQ(root->nodeId(), 3);
+    ASSERT_EQ(root->children[0]->nodeId(), 1);
+    ASSERT_EQ(root->children[1]->nodeId(), 2);
+}
+
+TEST(QuerySolutionTest, GroupNodeWithIndexScan) {
+    QueryTestServiceContext serviceCtx;
+    auto opCtx = serviceCtx.makeOperationContext();
+    const boost::intrusive_ptr<ExpressionContext> expCtx(new ExpressionContext(
+        opCtx.get(), std::unique_ptr<CollatorInterface>(nullptr), NamespaceString("test.dummy")));
+    auto scanNode =
+        std::make_unique<IndexScanNode>(buildSimpleIndexEntry(BSON("a" << 1 << "b" << 1)));
+    scanNode->bounds.isSimpleRange = true;
+    scanNode->bounds.startKey = BSON("a" << 1 << "b" << 1);
+    scanNode->bounds.endKey = BSON("a" << 1 << "b" << 1);
+    GroupNode node(
+        std::move(scanNode),
+        boost::intrusive_ptr<ExpressionConstant>(new ExpressionConstant(expCtx.get(), Value(0))),
+        {},
+        false,
+        false);
+    node.computeProperties();
+
+    ASSERT_EQ(node.fetched(), true);
+    ASSERT_EQ(node.sortedByDiskLoc(), false);
+    ASSERT_EQ(node.providedSorts(), kEmptySet);
+
+    ASSERT_EQ(node.getFieldAvailability("any_field"), FieldAvailability::kNotProvided);
+}
+
+TEST(QuerySolutionTest, EqLookupNodeWithIndexScan) {
+    // Simple EqLookupNode with IndexScan subtree.
+    auto scanNode =
+        std::make_unique<IndexScanNode>(buildSimpleIndexEntry(BSON("a" << 1 << "b" << 1)));
+    scanNode->bounds.isSimpleRange = true;
+    scanNode->bounds.startKey = BSON("a" << 1 << "b" << 1);
+    scanNode->bounds.endKey = BSON("a" << 1 << "b" << 1);
+
+    EqLookupNode node(std::move(scanNode),
+                      NamespaceString("db.col"),
+                      "local",
+                      "foreign",
+                      "as",
+                      EqLookupNode::LookupStrategy::kNestedLoopJoin,
+                      boost::none /* idxEntry */,
+                      false /* shouldProduceBson */);
+
+    node.computeProperties();
+
+    auto child = node.children[0].get();
+    ASSERT_EQ(node.fetched(), child->fetched());
+    ASSERT_EQ(node.sortedByDiskLoc(), child->sortedByDiskLoc());
+
+    // Expected empty sort order, as the EqLookupNode order inferrence is not supported yet.
+    ASSERT_EQ(node.providedSorts(), kEmptySet);
+
+    ASSERT_EQ(node.getFieldAvailability("as"), FieldAvailability::kNotProvided);
+    ASSERT_EQ(node.getFieldAvailability("a"), child->getFieldAvailability("a"));
+    ASSERT_EQ(node.getFieldAvailability("b"), child->getFieldAvailability("b"));
+}
+
+TEST(QuerySolutionTest, EqLookupNodeWithIndexScanFieldOverwrite) {
+    // A EqLookupNode with IndexScan subtree, where local field "b" is overwritten.
+    // This affects the field availability and sort order.
+    auto scanNode =
+        std::make_unique<IndexScanNode>(buildSimpleIndexEntry(BSON("a" << 1 << "b" << 1 << "c"
+                                                                       << "1")));
+    scanNode->bounds.isSimpleRange = true;
+    scanNode->bounds.startKey = BSON("a" << 1 << "b" << 1 << "c"
+                                         << "1");
+    scanNode->bounds.endKey = BSON("a" << 1 << "b" << 1 << "c"
+                                       << "1");
+
+    EqLookupNode node(std::move(scanNode),
+                      NamespaceString("db.col"),
+                      "local",
+                      "foreign",
+                      "b",
+                      EqLookupNode::LookupStrategy::kNestedLoopJoin,
+                      boost::none /* idxEntry */,
+                      false /* shouldProduceBson */);
+
+    node.computeProperties();
+
+    auto child = node.children[0].get();
+    // Expected empty sort order, as the EqLookupNode order inferrence is not supported yet.
+    ASSERT_EQ(node.providedSorts(), kEmptySet);
+
+    ASSERT_EQ(node.getFieldAvailability("a"), child->getFieldAvailability("a"));
+    ASSERT_EQ(node.getFieldAvailability("b"), FieldAvailability::kNotProvided);
+    ASSERT_EQ(node.getFieldAvailability("c"), child->getFieldAvailability("c"));
+    ASSERT_EQ(node.getFieldAvailability("other"), child->getFieldAvailability("other"));
+}
+
+TEST(QuerySolutionTest, ProvidedSortSetOutputStreamOperator) {
+    ProvidedSortSet ex1;
+    std::stringstream ex1_stream;
+    ex1_stream << ex1;
+    ASSERT_EQ(ex1_stream.str(), "baseSortPattern: {}, ignoredFields: []");
+
+    ProvidedSortSet ex2(BSONObj(), {"b", "a"});
+    std::stringstream ex2_stream;
+    ex2_stream << ex2;
+    ASSERT_EQ(ex2_stream.str(), "baseSortPattern: {}, ignoredFields: [a, b]");
+
+    ProvidedSortSet ex3(BSON("b" << 1 << "a" << 1), {});
+    std::stringstream ex3_stream;
+    ex3_stream << ex3;
+    ASSERT_EQ(ex3_stream.str(), "baseSortPattern: { b: 1, a: 1 }, ignoredFields: []");
+
+    ProvidedSortSet ex4(BSON("a" << 1 << "b" << 1), {"c", "d"});
+    std::stringstream ex4_stream;
+    ex4_stream << ex4;
+    ASSERT_EQ(ex4_stream.str(), "baseSortPattern: { a: 1, b: 1 }, ignoredFields: [c, d]");
+}
+
+TEST(QuerySolutionTest, ProvidedSortSetEqualityOperators) {
+    ProvidedSortSet ex1(BSON("a" << 1 << "b" << 1), {"c", "d"});
+    ProvidedSortSet ex2(BSON("b" << 1 << "a" << 1), {"c", "d"});
+    ProvidedSortSet ex3(BSON("a" << 1 << "b" << 1), {"d", "c"});
+    ProvidedSortSet ex4(BSON("a" << 1 << "b" << 1), {"c", "d", "e"});
+    ProvidedSortSet ex5(BSON("a" << 1 << "b" << 1 << "c" << 1), {"d", "c"});
+
+    ASSERT_EQ(ex1, ex1);
+    ASSERT_NE(ex1, ex2);
+    ASSERT_EQ(ex1, ex3);
+    ASSERT_NE(ex1, ex4);
+    ASSERT_NE(ex1, ex5);
+}
+
+TEST(QuerySolutionTest, FieldAvailabilityOutputStreamOperator) {
+    std::stringstream ex1;
+    ex1 << FieldAvailability::kNotProvided;
+    ASSERT_EQ(ex1.str(), "NotProvided");
+
+    std::stringstream ex2;
+    ex2 << FieldAvailability::kFullyProvided;
+    ASSERT_EQ(ex2.str(), "FullyProvided");
+
+    std::stringstream ex3;
+    ex3 << FieldAvailability::kHashedValueProvided;
+    ASSERT_EQ(ex3.str(), "HashedValueProvided");
+}
+
+TEST(QuerySolutionTest, GetSecondaryNamespaceVectorOverSingleEqLookupNode) {
+    auto scanNode = std::make_unique<IndexScanNode>(buildSimpleIndexEntry(BSON("a" << 1)));
+    const NamespaceString mainNss("db.main");
+    const NamespaceString foreignColl("db.col");
+    auto root = std::make_unique<EqLookupNode>(std::move(scanNode),
+                                               foreignColl,
+                                               "local",
+                                               "remote",
+                                               "b",
+                                               EqLookupNode::LookupStrategy::kNestedLoopJoin,
+                                               boost::none /* idxEntry */,
+                                               false /* shouldProduceBson */);
+
+
+    QuerySolution qs;
+    qs.setRoot(std::move(root));
+
+    // The output vector should only contain 'foreignColl'.
+    std::vector<NamespaceStringOrUUID> expectedNssVector{foreignColl};
+    assertNamespaceVectorsAreEqual(qs.getAllSecondaryNamespaces(mainNss), expectedNssVector);
+}
+
+TEST(QuerySolutionTest, GetSecondaryNamespaceVectorDeduplicatesMainNss) {
+    auto scanNode = std::make_unique<IndexScanNode>(buildSimpleIndexEntry(BSON("a" << 1)));
+    const NamespaceString mainNss("db.main");
+    auto root = std::make_unique<EqLookupNode>(std::move(scanNode),
+                                               mainNss,
+                                               "local",
+                                               "remote",
+                                               "b",
+                                               EqLookupNode::LookupStrategy::kNestedLoopJoin,
+                                               boost::none /* idxEntry */,
+                                               false /* shouldProduceBson */);
+
+
+    QuerySolution qs;
+    qs.setRoot(std::move(root));
+
+    // There should be no secondary namespaces as 'mainNss' is ignored in
+    // 'getAllSecondaryNamespaces'.
+    std::vector<NamespaceStringOrUUID> expectedNssVector{};
+    assertNamespaceVectorsAreEqual(qs.getAllSecondaryNamespaces(mainNss), expectedNssVector);
+}
+
+TEST(QuerySolutionTest, GetSecondaryNamespaceVectorOverNestedEqLookupNodes) {
+    auto scanNode = std::make_unique<IndexScanNode>(buildSimpleIndexEntry(BSON("a" << 1)));
+    const NamespaceString mainNss("db.main");
+    const NamespaceString foreignCollOne("db.col");
+    const NamespaceString foreignCollTwo("db.foo");
+    auto childEqLookupNode =
+        std::make_unique<EqLookupNode>(std::move(scanNode),
+                                       foreignCollOne,
+                                       "local",
+                                       "remote",
+                                       "b",
+                                       EqLookupNode::LookupStrategy::kNestedLoopJoin,
+                                       boost::none /* idxEntry */,
+                                       false /* shouldProduceBson */);
+
+    auto parentEqLookupNode =
+        std::make_unique<EqLookupNode>(std::move(childEqLookupNode),
+                                       foreignCollTwo,
+                                       "local",
+                                       "remote",
+                                       "b",
+                                       EqLookupNode::LookupStrategy::kNestedLoopJoin,
+                                       boost::none /* idxEntry */,
+                                       false /* shouldProduceBson */);
+
+    QuerySolution qs;
+    qs.setRoot(std::move(parentEqLookupNode));
+
+    // The foreign collections are unique, so our output vector should contain both of them. Note
+    // that because 'getAllSecondaryNamespaces' uses a set internally, these namespaces are
+    // expected to be in sorted order in the output vector.
+    std::vector<NamespaceStringOrUUID> expectedNssVector{foreignCollOne, foreignCollTwo};
+    assertNamespaceVectorsAreEqual(qs.getAllSecondaryNamespaces(mainNss), expectedNssVector);
+}
+
+TEST(QuerySolutionTest, GetSecondaryNamespaceVectorDeduplicatesNestedEqLookupNodes) {
+    auto scanNode = std::make_unique<IndexScanNode>(buildSimpleIndexEntry(BSON("a" << 1)));
+    const NamespaceString mainNss("db.main");
+    const NamespaceString foreignColl("db.col");
+    auto childEqLookupNode =
+        std::make_unique<EqLookupNode>(std::move(scanNode),
+                                       foreignColl,
+                                       "local",
+                                       "remote",
+                                       "b",
+                                       EqLookupNode::LookupStrategy::kNestedLoopJoin,
+                                       boost::none /* idxEntry */,
+                                       false /* shouldProduceBson */);
+
+    auto parentEqLookupNode =
+        std::make_unique<EqLookupNode>(std::move(childEqLookupNode),
+                                       foreignColl,
+                                       "local",
+                                       "remote",
+                                       "b",
+                                       EqLookupNode::LookupStrategy::kNestedLoopJoin,
+                                       boost::none /* idxEntry */,
+                                       false /* shouldProduceBson */);
+
+    QuerySolution qs;
+    qs.setRoot(std::move(parentEqLookupNode));
+
+    // Both nodes reference the same foreign collection. Therefore, our output vector should contain
+    // a single copy of that namespace.
+    std::vector<NamespaceStringOrUUID> expectedNssVector{foreignColl};
+    assertNamespaceVectorsAreEqual(qs.getAllSecondaryNamespaces(mainNss), expectedNssVector);
 }
 }  // namespace
