@@ -27,38 +27,39 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
-
-#include "mongo/platform/basic.h"
 
 #include "mongo/db/repl/apply_ops.h"
 
 #include "mongo/bson/util/bson_extract.h"
-#include "mongo/db/background.h"
+#include "mongo/client/client_deprecated.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/client.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/concurrency/lock_state.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/matcher/matcher.h"
-#include "mongo/db/op_observer.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/query/collation/collation_spec.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/service_context.h"
-#include "mongo/db/session_catalog_mongod.h"
-#include "mongo/db/transaction_participant.h"
+#include "mongo/db/session/session_catalog_mongod.h"
+#include "mongo/db/transaction/transaction_participant.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/util/fail_point.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
+
+
 namespace mongo {
 namespace repl {
 
@@ -70,34 +71,6 @@ namespace {
 // If enabled, causes loop in _applyOps() to hang after applying current operation.
 MONGO_FAIL_POINT_DEFINE(applyOpsPauseBetweenOperations);
 
-/**
- * Return true iff the applyOpsCmd can be executed in a single WriteUnitOfWork.
- */
-bool _parseAreOpsCrudOnly(const BSONObj& applyOpCmd) {
-    for (const auto& elem : applyOpCmd.firstElement().Obj()) {
-        const char* opType = elem.Obj().getField("op").valuestrsafe();
-
-        // All atomic ops have an opType of length 1.
-        if (opType[0] == '\0' || opType[1] != '\0')
-            return false;
-
-        // Only consider CRUD operations.
-        switch (*opType) {
-            case 'd':
-            case 'n':
-            case 'u':
-                break;
-            case 'i':
-                break;
-            // Fallthrough.
-            default:
-                return false;
-        }
-    }
-
-    return true;
-}
-
 Status _applyOps(OperationContext* opCtx,
                  const ApplyOpsCommandInfo& info,
                  repl::OplogApplication::Mode oplogApplicationMode,
@@ -116,7 +89,7 @@ Status _applyOps(OperationContext* opCtx,
     // Apply each op in the given 'applyOps' command object.
     for (const auto& opObj : ops) {
         // Ignore 'n' operations.
-        const char* opType = opObj["op"].valuestrsafe();
+        const char* opType = opObj.getStringField("op").rawData();
         if (*opType == 'n')
             continue;
 
@@ -134,16 +107,7 @@ Status _applyOps(OperationContext* opCtx,
 
             // ApplyOps does not have the global writer lock when applying transaction
             // operations, so we need to acquire the DB and Collection locks.
-            Lock::DBLock dbLock(opCtx, nss.db(), MODE_IX);
-            auto databaseHolder = DatabaseHolder::get(opCtx);
-            auto db = databaseHolder->getDb(opCtx, nss.ns());
-            if (!db) {
-                // Retry in non-atomic mode, since MMAP cannot implicitly create a new database
-                // within an active WriteUnitOfWork.
-                uasserted(ErrorCodes::AtomicityFailure,
-                          "cannot create a database in atomic applyOps mode; will retry without "
-                          "atomicity");
-            }
+            Lock::DBLock dbLock(opCtx, nss.dbName(), MODE_IX);
 
             // When processing an update on a non-existent collection, applyOperation_inlock()
             // returns UpdateOperationFailed on updates and allows the collection to be
@@ -151,7 +115,8 @@ Status _applyOps(OperationContext* opCtx,
             // NamespaceNotFound.
             // Additionally for inserts, we fail early on non-existent collections.
             Lock::CollectionLock collectionLock(opCtx, nss, MODE_IX);
-            auto collection = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
+            auto collection =
+                CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
             if (!collection && (*opType == 'i' || *opType == 'u')) {
                 uasserted(
                     ErrorCodes::AtomicityFailure,
@@ -159,10 +124,14 @@ Status _applyOps(OperationContext* opCtx,
                         << "cannot apply insert or update operation on a non-existent namespace "
                         << nss.ns() << " in atomic applyOps mode: " << redact(opObj));
             }
+            uassert(ErrorCodes::AtomicityFailure,
+                    str::stream() << "cannot run atomic applyOps on namespace " << nss.ns()
+                                  << " which has change stream pre- or post-images enabled",
+                    !collection->isChangeStreamPreAndPostImagesEnabled());
 
             // Reject malformed or over-specified operations in an atomic applyOps.
             try {
-                ReplOperation::parse(IDLParserErrorContext("applyOps"), opObj);
+                ReplOperation::parse(IDLParserContext("applyOps"), opObj);
             } catch (...) {
                 uasserted(ErrorCodes::AtomicityFailure,
                           str::stream() << "cannot apply a malformed or over-specified operation "
@@ -183,7 +152,7 @@ Status _applyOps(OperationContext* opCtx,
             // Malformed operations should have already been caught and retried in non-atomic mode.
             invariant(entry.isOK());
 
-            OldClientContext ctx(opCtx, nss.ns());
+            OldClientContext ctx(opCtx, nss);
 
             const auto& op = entry.getValue();
             const bool isDataConsistent = true;
@@ -212,7 +181,7 @@ Status _applyOps(OperationContext* opCtx,
                     opCtx,
                     "applyOps",
                     nss.ns(),
-                    [opCtx, nss, opObj, opType, alwaysUpsert, oplogApplicationMode] {
+                    [opCtx, nss, opObj, opType, alwaysUpsert, oplogApplicationMode, &info] {
                         BSONObjBuilder builder;
                         builder.appendElements(opObj);
                         if (!builder.hasField(OplogEntry::kTimestampFieldName)) {
@@ -225,7 +194,18 @@ Status _applyOps(OperationContext* opCtx,
                             builder.append(OplogEntry::kWallClockTimeFieldName, Date_t());
                         }
                         auto entry = uassertStatusOK(OplogEntry::parse(builder.done()));
+
                         if (*opType == 'c') {
+                            if (entry.getCommandType() == OplogEntry::CommandType::kDropDatabase) {
+                                invariant(info.getOperations().size() == 1,
+                                          "dropDatabase in applyOps must be the only entry");
+                                // This method is explicitly called without locks in spite of the
+                                // _inlock suffix. dropDatabase cannot hold any locks for execution
+                                // of the operation due to potential replication waits.
+                                uassertStatusOK(
+                                    applyCommand_inlock(opCtx, entry, oplogApplicationMode));
+                                return Status::OK();
+                            }
                             invariant(opCtx->lockState()->isW());
                             uassertStatusOK(
                                 applyCommand_inlock(opCtx, entry, oplogApplicationMode));
@@ -246,7 +226,7 @@ Status _applyOps(OperationContext* opCtx,
                                           << nss.ns() << ": " << mongo::redact(opObj));
                         }
 
-                        OldClientContext ctx(opCtx, nss.ns());
+                        OldClientContext ctx(opCtx, nss);
 
                         // We return the status rather than merely aborting so failure of CRUD
                         // ops doesn't stop the applyOps from trying to process the rest of the
@@ -267,7 +247,7 @@ Status _applyOps(OperationContext* opCtx,
                 result->append("codeName", ErrorCodes::errorString(ex.code()));
                 result->append("errmsg", ex.what());
                 result->append("results", ab.arr());
-                return Status(ex.code(), ex.what());
+                return ex.toStatus();
             }
         }
 
@@ -314,16 +294,22 @@ Status _checkPrecondition(OperationContext* opCtx,
         }
 
         DBDirectClient db(opCtx);
-        BSONObj realres = db.findOne(nss.ns(), preCondition["q"].Obj());
+        // The preconditions come in "q: {{query: {...}, orderby: ..., etc.}}" format. This format
+        // is no longer used either internally or over the wire in other contexts. We are using a
+        // legacy API from 'client_deprecated' in order to parse this format and convert it into the
+        // corresponding find command.
+        FindCommandRequest findCmd{nss};
+        client_deprecated::initFindFromLegacyOptions(preCondition["q"].Obj(), 0, &findCmd);
+        BSONObj realres = db.findOne(std::move(findCmd));
 
         // Get collection default collation.
         auto databaseHolder = DatabaseHolder::get(opCtx);
-        auto database = databaseHolder->getDb(opCtx, nss.db());
+        auto database = databaseHolder->getDb(opCtx, nss.dbName());
         if (!database) {
             return {ErrorCodes::NamespaceNotFound, "database in ns does not exist: " + nss.ns()};
         }
-        Collection* collection =
-            CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
+        CollectionPtr collection =
+            CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
         if (!collection) {
             return {ErrorCodes::NamespaceNotFound, "collection in ns does not exist: " + nss.ns()};
         }
@@ -345,38 +331,6 @@ Status _checkPrecondition(OperationContext* opCtx,
 }
 }  // namespace
 
-// static
-ApplyOpsCommandInfo ApplyOpsCommandInfo::parse(const BSONObj& applyOpCmd) {
-    try {
-        return ApplyOpsCommandInfo(applyOpCmd);
-    } catch (DBException& ex) {
-        ex.addContext(str::stream() << "Failed to parse applyOps command: " << redact(applyOpCmd));
-        throw;
-    }
-}
-
-bool ApplyOpsCommandInfo::areOpsCrudOnly() const {
-    return _areOpsCrudOnly;
-}
-
-bool ApplyOpsCommandInfo::isAtomic() const {
-    return getAllowAtomic() && areOpsCrudOnly();
-}
-
-ApplyOpsCommandInfo::ApplyOpsCommandInfo(const BSONObj& applyOpCmd)
-    : _areOpsCrudOnly(_parseAreOpsCrudOnly(applyOpCmd)) {
-    parseProtected(IDLParserErrorContext("applyOps"), applyOpCmd);
-
-    if (getPreCondition()) {
-        uassert(ErrorCodes::InvalidOptions,
-                "Cannot use preCondition with {allowAtomic: false}",
-                getAllowAtomic());
-        uassert(ErrorCodes::InvalidOptions,
-                "Cannot use preCondition when operations include commands.",
-                areOpsCrudOnly());
-    }
-}
-
 Status applyApplyOpsOplogEntry(OperationContext* opCtx,
                                const OplogEntry& entry,
                                repl::OplogApplication::Mode oplogApplicationMode) {
@@ -406,10 +360,12 @@ Status applyOps(OperationContext* opCtx,
     uassert(31056, "applyOps command can't have 'partialTxn' field.", !info.getPartialTxn());
     uassert(31240, "applyOps command can't have 'count' field.", !info.getCount());
 
-    // There's only one case where we are allowed to take the database lock instead of the global
+    // There's one case where we are allowed to take the database lock instead of the global
     // lock - no preconditions; only CRUD ops; and non-atomic mode.
     if (!info.getPreCondition() && info.areOpsCrudOnly() && !info.getAllowAtomic()) {
-        dbWriteLock.emplace(opCtx, dbName, MODE_IX);
+        // TODO SERVER-62880 Once the dbName is of type DatabaseName, pass it directly to the DBlock
+        DatabaseName databaseName(boost::none, dbName);
+        dbWriteLock.emplace(opCtx, databaseName, MODE_IX);
     } else {
         globalWriteLock.emplace(opCtx);
     }
@@ -430,7 +386,30 @@ Status applyOps(OperationContext* opCtx,
         }
     }
 
+    LOGV2_DEBUG(5854600,
+                2,
+                "applyOps command",
+                "dbName"_attr = redact(dbName),
+                "cmd"_attr = redact(applyOpCmd));
+
     if (!info.isAtomic()) {
+        auto hasDropDatabase = std::any_of(
+            info.getOperations().begin(), info.getOperations().end(), [](const BSONObj& op) {
+                return op.getStringField("op") == "c" &&
+                    parseCommandType(op.getObjectField("o")) ==
+                    OplogEntry::CommandType::kDropDatabase;
+            });
+        if (hasDropDatabase) {
+            // Normally the contract for applyOps is to hold a global exclusive lock during
+            // application of ops. However, dropDatabase must specially not hold locks because it
+            // may need to await replication internally during application. Additionally, since
+            // dropDatabase is abnormal in locking behavior, applyOps is only allowed to apply a
+            // dropDatabase op singly, not in combination with additional ops.
+            uassert(6275900,
+                    "dropDatabase in an applyOps must be the only entry",
+                    info.getOperations().size() == 1);
+            globalWriteLock.reset();
+        }
         return _applyOps(opCtx, info, oplogApplicationMode, result, &numApplied, nullptr);
     }
 
@@ -441,48 +420,19 @@ Status applyOps(OperationContext* opCtx,
         writeConflictRetry(opCtx, "applyOps", dbName, [&] {
             BSONObjBuilder intermediateResult;
             std::unique_ptr<BSONArrayBuilder> opsBuilder;
-            if (opCtx->writesAreReplicated()) {
-                opsBuilder = std::make_unique<BSONArrayBuilder>();
-            }
-            WriteUnitOfWork wunit(opCtx);
+
+            // If we were to replicate the original applyOps operation we received, we could
+            // replicate an applyOps that includes no-op writes. Oplog readers, like change streams,
+            // would then see entries for writes that did not happen. To work around this, we group
+            // all writes in this WUOW into a new applyOps entry so that we only replicate writes
+            // that actually happen.
+            // Note that the applyOps command doesn't update config.transactions for retryable
+            // writes, nor does it support change stream pre- and post-images.
+
+            WriteUnitOfWork wunit(opCtx, true /*groupOplogEntries*/);
             numApplied = 0;
-            {
-                // Suppress replication for atomic operations until end of applyOps.
-                repl::UnreplicatedWritesBlock uwb(opCtx);
-                uassertStatusOK(_applyOps(opCtx,
-                                          info,
-                                          oplogApplicationMode,
-                                          &intermediateResult,
-                                          &numApplied,
-                                          opsBuilder.get()));
-            }
-            // Generate oplog entry for all atomic ops collectively.
-            if (opCtx->writesAreReplicated()) {
-                // We want this applied atomically on slaves so we rewrite the oplog entry without
-                // the pre-condition for speed.
-
-                BSONObjBuilder cmdBuilder;
-
-                auto opsFieldName = applyOpCmd.firstElement().fieldNameStringData();
-                for (auto elem : applyOpCmd) {
-                    auto name = elem.fieldNameStringData();
-                    if (name == opsFieldName && opsBuilder) {
-                        cmdBuilder.append(opsFieldName, opsBuilder->arr());
-                        continue;
-                    }
-                    if (name == ApplyOps::kPreconditionFieldName)
-                        continue;
-                    if (name == bypassDocumentValidationCommandOption())
-                        continue;
-                    cmdBuilder.append(elem);
-                }
-
-                const BSONObj cmdRewritten = cmdBuilder.done();
-
-                auto opObserver = getGlobalServiceContext()->getOpObserver();
-                invariant(opObserver);
-                opObserver->onApplyOps(opCtx, dbName, cmdRewritten);
-            }
+            uassertStatusOK(_applyOps(
+                opCtx, info, oplogApplicationMode, &intermediateResult, &numApplied, nullptr));
             wunit.commit();
             result->appendElements(intermediateResult.obj());
         });
@@ -506,50 +456,5 @@ Status applyOps(OperationContext* opCtx,
     return Status::OK();
 }
 
-// static
-std::vector<OplogEntry> ApplyOps::extractOperations(const OplogEntry& applyOpsOplogEntry) {
-    std::vector<OplogEntry> result;
-    extractOperationsTo(applyOpsOplogEntry, applyOpsOplogEntry.toBSON(), &result);
-    return result;
-}
-
-// static
-void ApplyOps::extractOperationsTo(const OplogEntry& applyOpsOplogEntry,
-                                   const BSONObj& topLevelDoc,
-                                   std::vector<OplogEntry>* operations) {
-    uassert(ErrorCodes::TypeMismatch,
-            str::stream() << "ApplyOps::extractOperations(): not a command: "
-                          << redact(applyOpsOplogEntry.toBSON()),
-            applyOpsOplogEntry.isCommand());
-
-    uassert(ErrorCodes::CommandNotSupported,
-            str::stream() << "ApplyOps::extractOperations(): not applyOps command: "
-                          << redact(applyOpsOplogEntry.toBSON()),
-            OplogEntry::CommandType::kApplyOps == applyOpsOplogEntry.getCommandType());
-
-    auto cmdObj = applyOpsOplogEntry.getOperationToApply();
-    auto info = ApplyOpsCommandInfo::parse(cmdObj);
-    auto operationDocs = info.getOperations();
-    bool alwaysUpsert = info.getAlwaysUpsert() && !applyOpsOplogEntry.getTxnNumber();
-
-    for (const auto& operationDoc : operationDocs) {
-        // Make sure that the inner ops are not malformed or over-specified.
-        ReplOperation::parse(IDLParserErrorContext("extractOperations"), operationDoc);
-
-        BSONObjBuilder builder(operationDoc);
-
-        // Oplog entries can have an oddly-named "b" field for "upsert". MongoDB stopped creating
-        // such entries in 4.0, but we can use the "b" field for the extracted entry here.
-        if (alwaysUpsert && !operationDoc.hasField("b")) {
-            builder.append("b", true);
-        }
-
-        builder.appendElementsUnique(topLevelDoc);
-        auto operation = builder.obj();
-
-        operations->emplace_back(operation);
-    }
-}
-
 }  // namespace repl
 }  // namespace mongo
