@@ -34,13 +34,12 @@
 #include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/client.h"
-#include "mongo/db/op_observer_noop.h"
+#include "mongo/db/op_observer/op_observer_noop.h"
 #include "mongo/db/repl/apply_ops.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/repl/storage_interface_impl.h"
 #include "mongo/db/service_context_d_test_fixture.h"
-#include "mongo/logger/logger.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/unittest/log_test.h"
 
@@ -57,7 +56,7 @@ public:
      * Called by applyOps() when ops are applied atomically.
      */
     void onApplyOps(OperationContext* opCtx,
-                    const std::string& dbName,
+                    const DatabaseName& dbName,
                     const BSONObj& applyOpCmd) override;
 
     // If not empty, holds the command object passed to last invocation of onApplyOps().
@@ -65,7 +64,7 @@ public:
 };
 
 void OpObserverMock::onApplyOps(OperationContext* opCtx,
-                                const std::string& dbName,
+                                const DatabaseName& dbName,
                                 const BSONObj& applyOpCmd) {
     ASSERT_FALSE(applyOpCmd.isEmpty());
     // Get owned copy because 'applyOpCmd' may be a temporary BSONObj created by applyOps().
@@ -97,7 +96,6 @@ void ApplyOpsTest::setUp() {
 
     // Set up ReplicationCoordinator and create oplog.
     ReplicationCoordinator::set(service, std::make_unique<ReplicationCoordinatorMock>(service));
-    setOplogCollectionName(service);
     createOplog(opCtx.get());
 
     // Ensure that we are primary.
@@ -132,7 +130,7 @@ Status getStatusFromApplyOpsResult(const BSONObj& result) {
     BSONObjBuilder builder;
     builder.appendElements(result);
     auto code = result.getIntField("code");
-    builder.appendIntOrLL("ok", code == 0);
+    builder.appendNumber("ok", code == 0);
     auto newResult = builder.obj();
     return getStatusFromCommandResult(newResult);
 }
@@ -156,45 +154,11 @@ TEST_F(ApplyOpsTest, CommandInNestedApplyOpsReturnsSuccess) {
     ASSERT_BSONOBJ_EQ({}, _opObserver->onApplyOpsCmdObj);
 }
 
-TEST_F(ApplyOpsTest, InsertInNestedApplyOpsReturnsSuccess) {
-    auto opCtx = cc().makeOperationContext();
-    auto mode = OplogApplication::Mode::kApplyOpsCmd;
-    // Make sure the apply ops command object contains the correct UUID information.
-    CollectionOptions options;
-    options.uuid = UUID::gen();
-    BSONObjBuilder resultBuilder;
-    NamespaceString nss("test", "foo");
-    auto innerCmdObj = BSON("op"
-                            << "i"
-                            << "ns" << nss.ns() << "o"
-                            << BSON("_id"
-                                    << "a")
-                            << "ui" << options.uuid.get());
-    auto innerApplyOpsObj = BSON("op"
-                                 << "c"
-                                 << "ns" << nss.getCommandNS().ns() << "o"
-                                 << BSON("applyOps" << BSON_ARRAY(innerCmdObj)));
-    auto cmdObj = BSON("applyOps" << BSON_ARRAY(innerApplyOpsObj));
-
-    ASSERT_OK(_storage->createCollection(opCtx.get(), nss, options));
-    ASSERT_OK(applyOps(opCtx.get(), nss.db().toString(), cmdObj, mode, &resultBuilder));
-    ASSERT_BSONOBJ_EQ(BSON("applyOps" << BSON_ARRAY(innerCmdObj)), _opObserver->onApplyOpsCmdObj);
-}
-
-TEST_F(ApplyOpsTest, AtomicApplyOpsWithNoOpsReturnsSuccess) {
-    auto opCtx = cc().makeOperationContext();
-    auto mode = OplogApplication::Mode::kApplyOpsCmd;
-    BSONObjBuilder resultBuilder;
-    auto cmdObj = BSON("applyOps" << BSONArray());
-    ASSERT_OK(applyOps(opCtx.get(), "test", cmdObj, mode, &resultBuilder));
-    ASSERT_BSONOBJ_EQ(cmdObj, _opObserver->onApplyOpsCmdObj);
-}
-
 /**
  * Creates an applyOps command object with a single insert operation.
  */
 BSONObj makeApplyOpsWithInsertOperation(const NamespaceString& nss,
-                                        const OptionalCollectionUUID& uuid,
+                                        const boost::optional<UUID>& uuid,
                                         const BSONObj& documentToInsert) {
     auto insertOp = uuid ? BSON("op"
                                 << "i"
@@ -220,25 +184,6 @@ TEST_F(ApplyOpsTest,
     ASSERT_EQUALS(ErrorCodes::NamespaceNotFound, status);
 }
 
-
-TEST_F(ApplyOpsTest, AtomicApplyOpsInsertWithUuidIntoCollectionWithUuid) {
-    auto opCtx = cc().makeOperationContext();
-    auto mode = OplogApplication::Mode::kApplyOpsCmd;
-    NamespaceString nss("test.t");
-
-    auto uuid = UUID::gen();
-
-    CollectionOptions collectionOptions;
-    collectionOptions.uuid = uuid;
-    ASSERT_OK(_storage->createCollection(opCtx.get(), nss, collectionOptions));
-
-    auto documentToInsert = BSON("_id" << 0);
-    auto cmdObj = makeApplyOpsWithInsertOperation(nss, uuid, documentToInsert);
-    BSONObjBuilder resultBuilder;
-    ASSERT_OK(applyOps(opCtx.get(), "test", cmdObj, mode, &resultBuilder));
-    ASSERT_BSONOBJ_EQ(cmdObj, _opObserver->onApplyOpsCmdObj);
-}
-
 TEST_F(ApplyOpsTest, AtomicApplyOpsInsertWithUuidIntoCollectionWithOtherUuid) {
     auto opCtx = cc().makeOperationContext();
     auto mode = OplogApplication::Mode::kApplyOpsCmd;
@@ -264,28 +209,6 @@ TEST_F(ApplyOpsTest, AtomicApplyOpsInsertWithUuidIntoCollectionWithOtherUuid) {
     ASSERT_EQUALS(ErrorCodes::NamespaceNotFound, status);
 }
 
-TEST_F(ApplyOpsTest, AtomicApplyOpsInsertWithoutUuidIntoCollectionWithUuid) {
-    auto opCtx = cc().makeOperationContext();
-    auto mode = OplogApplication::Mode::kApplyOpsCmd;
-    NamespaceString nss("test.t");
-
-    auto uuid = UUID::gen();
-
-    CollectionOptions collectionOptions;
-    collectionOptions.uuid = uuid;
-    ASSERT_OK(_storage->createCollection(opCtx.get(), nss, collectionOptions));
-
-    auto documentToInsert = BSON("_id" << 0);
-    auto cmdObj = makeApplyOpsWithInsertOperation(nss, boost::none, documentToInsert);
-    BSONObjBuilder resultBuilder;
-    ASSERT_OK(applyOps(opCtx.get(), "test", cmdObj, mode, &resultBuilder));
-
-    // Insert operation provided by caller did not contain collection uuid but applyOps() should add
-    // the uuid to the oplog entry.
-    auto expectedCmdObj = makeApplyOpsWithInsertOperation(nss, uuid, documentToInsert);
-    ASSERT_BSONOBJ_EQ(expectedCmdObj, _opObserver->onApplyOpsCmdObj);
-}
-
 TEST_F(ApplyOpsTest, ApplyOpsPropagatesOplogApplicationMode) {
     auto opCtx = cc().makeOperationContext();
 
@@ -338,23 +261,28 @@ TEST_F(ApplyOpsTest, ApplyOpsPropagatesOplogApplicationMode) {
 /**
  * Generates oplog entries with the given number used for the timestamp.
  */
-OplogEntry makeOplogEntry(OpTypeEnum opType, const BSONObj& oField) {
-    return OplogEntry(OpTime(Timestamp(1, 1), 1),  // optime
-                      boost::none,                 // hash
-                      opType,                      // op type
-                      NamespaceString("a.a"),      // namespace
-                      boost::none,                 // uuid
-                      boost::none,                 // fromMigrate
-                      OplogEntry::kOplogVersion,   // version
-                      oField,                      // o
-                      boost::none,                 // o2
-                      {},                          // sessionInfo
-                      boost::none,                 // upsert
-                      Date_t(),                    // wall clock time
-                      boost::none,                 // statement id
-                      boost::none,   // optime of previous write within same transaction
-                      boost::none,   // pre-image optime
-                      boost::none);  // post-image optime
+OplogEntry makeOplogEntry(OpTypeEnum opType,
+                          const BSONObj& oField,
+                          const std::vector<StmtId>& stmtIds = {}) {
+    return {DurableOplogEntry(OpTime(Timestamp(1, 1), 1),  // optime
+                              boost::none,                 // hash
+                              opType,                      // op type
+                              NamespaceString("a.a"),      // namespace
+                              boost::none,                 // uuid
+                              boost::none,                 // fromMigrate
+                              OplogEntry::kOplogVersion,   // version
+                              oField,                      // o
+                              boost::none,                 // o2
+                              {},                          // sessionInfo
+                              boost::none,                 // upsert
+                              Date_t(),                    // wall clock time
+                              stmtIds,                     // statement ids
+                              boost::none,    // optime of previous write within same transaction
+                              boost::none,    // pre-image optime
+                              boost::none,    // post-image optime
+                              boost::none,    // ShardId of resharding recipient
+                              boost::none,    // _id
+                              boost::none)};  // needsRetryImage
 }
 
 TEST_F(ApplyOpsTest, ExtractOperationsReturnsTypeMismatchIfNotCommand) {
@@ -403,7 +331,7 @@ TEST_F(ApplyOpsTest, ExtractOperationsReturnsOperationsWithSameOpTimeAsApplyOps)
 
     auto operations = ApplyOps::extractOperations(oplogEntry);
     ASSERT_EQUALS(3U, operations.size())
-        << "Unexpected number of operations extracted: " << oplogEntry.toBSON();
+        << "Unexpected number of operations extracted: " << oplogEntry.toBSONForLogging();
 
     // Check extracted CRUD operations.
     auto it = operations.cbegin();
@@ -411,7 +339,7 @@ TEST_F(ApplyOpsTest, ExtractOperationsReturnsOperationsWithSameOpTimeAsApplyOps)
         ASSERT(operations.cend() != it);
         const auto& operation1 = *(it++);
         ASSERT(OpTypeEnum::kInsert == operation1.getOpType())
-            << "Unexpected op type: " << operation1.toBSON();
+            << "Unexpected op type: " << operation1.toBSONForLogging();
         ASSERT_EQUALS(ui1, *operation1.getUuid());
         ASSERT_EQUALS(ns1, operation1.getNss());
         ASSERT_BSONOBJ_EQ(BSON("_id" << 1), operation1.getOperationToApply());
@@ -424,7 +352,7 @@ TEST_F(ApplyOpsTest, ExtractOperationsReturnsOperationsWithSameOpTimeAsApplyOps)
         ASSERT(operations.cend() != it);
         const auto& operation2 = *(it++);
         ASSERT(OpTypeEnum::kInsert == operation2.getOpType())
-            << "Unexpected op type: " << operation2.toBSON();
+            << "Unexpected op type: " << operation2.toBSONForLogging();
         ASSERT_EQUALS(ui2, *operation2.getUuid());
         ASSERT_EQUALS(ns2, operation2.getNss());
         ASSERT_BSONOBJ_EQ(BSON("_id" << 2), operation2.getOperationToApply());
@@ -437,7 +365,7 @@ TEST_F(ApplyOpsTest, ExtractOperationsReturnsOperationsWithSameOpTimeAsApplyOps)
         ASSERT(operations.cend() != it);
         const auto& operation3 = *(it++);
         ASSERT(OpTypeEnum::kUpdate == operation3.getOpType())
-            << "Unexpected op type: " << operation3.toBSON();
+            << "Unexpected op type: " << operation3.toBSONForLogging();
         ASSERT_EQUALS(ui3, *operation3.getUuid());
         ASSERT_EQUALS(ns3, operation3.getNss());
         ASSERT_BSONOBJ_EQ(BSON("x" << 1), operation3.getOperationToApply());
@@ -453,6 +381,62 @@ TEST_F(ApplyOpsTest, ExtractOperationsReturnsOperationsWithSameOpTimeAsApplyOps)
     ASSERT(operations.cend() == it);
 }
 
+TEST_F(ApplyOpsTest, ExtractOperationsFromApplyOpsMultiStmtIds) {
+    NamespaceString ns1("test.a");
+    auto ui1 = UUID::gen();
+    auto op1 = BSON("op"
+                    << "i"
+                    << "ns" << ns1.ns() << "ui" << ui1 << "o" << BSON("_id" << 1));
+
+    NamespaceString ns2("test.b");
+    auto ui2 = UUID::gen();
+    auto op2 = BSON("op"
+                    << "u"
+                    << "ns" << ns2.ns() << "ui" << ui2 << "b" << true << "o" << BSON("x" << 1)
+                    << "o2" << BSON("_id" << 2));
+
+    auto oplogEntry =
+        makeOplogEntry(OpTypeEnum::kCommand, BSON("applyOps" << BSON_ARRAY(op1 << op2)), {0, 1});
+
+    auto operations = ApplyOps::extractOperations(oplogEntry);
+    ASSERT_EQUALS(2U, operations.size())
+        << "Unexpected number of operations extracted: " << oplogEntry.toBSONForLogging();
+
+    // Check extracted CRUD operations.
+    auto it = operations.cbegin();
+    {
+        ASSERT(operations.cend() != it);
+        const auto& operation1 = *(it++);
+        ASSERT(OpTypeEnum::kInsert == operation1.getOpType())
+            << "Unexpected op type: " << operation1.toBSONForLogging();
+        ASSERT_EQUALS(ui1, *operation1.getUuid());
+        ASSERT_EQUALS(ns1, operation1.getNss());
+        ASSERT_BSONOBJ_EQ(BSON("_id" << 1), operation1.getOperationToApply());
+
+        // OpTime of CRUD operation should match applyOps.
+        ASSERT_EQUALS(oplogEntry.getOpTime(), operation1.getOpTime());
+    }
+
+    {
+        ASSERT(operations.cend() != it);
+        const auto& operation2 = *(it++);
+        ASSERT(OpTypeEnum::kUpdate == operation2.getOpType())
+            << "Unexpected op type: " << operation2.toBSONForLogging();
+        ASSERT_EQUALS(ui2, *operation2.getUuid());
+        ASSERT_EQUALS(ns2, operation2.getNss());
+        ASSERT_BSONOBJ_EQ(BSON("x" << 1), operation2.getOperationToApply());
+
+        auto optionalUpsertBool = operation2.getUpsert();
+        ASSERT(optionalUpsertBool);
+        ASSERT(*optionalUpsertBool);
+
+        // OpTime of CRUD operation should match applyOps.
+        ASSERT_EQUALS(oplogEntry.getOpTime(), operation2.getOpTime());
+    }
+
+    ASSERT(operations.cend() == it);
+}
+
 TEST_F(ApplyOpsTest, ApplyOpsFailsToDropAdmin) {
     auto opCtx = cc().makeOperationContext();
     auto mode = OplogApplication::Mode::kApplyOpsCmd;
