@@ -27,23 +27,18 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
-
-#include "mongo/platform/basic.h"
 
 #include "mongo/db/repl/bgsync.h"
 
 #include <memory>
 
-#include "mongo/base/counter.h"
 #include "mongo/base/string_data.h"
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/client/connection_pool.h"
 #include "mongo/db/auth/authorization_session.h"
-#include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/concurrency/replication_state_transition_lock_guard.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/dbhelpers.h"
 #include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/repl/data_replicator_external_state_impl.h"
@@ -58,7 +53,7 @@
 #include "mongo/db/repl/rollback_source_impl.h"
 #include "mongo/db/repl/rs_rollback.h"
 #include "mongo/db/repl/storage_interface.h"
-#include "mongo/db/s/shard_identity_rollback_notifier.h"
+#include "mongo/db/shutdown_in_progress_quiesce_info.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/metadata/repl_set_metadata.h"
@@ -66,6 +61,9 @@
 #include "mongo/util/testing_proctor.h"
 #include "mongo/util/time_support.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
+
+
 namespace mongo {
 
 using std::string;
@@ -80,27 +78,19 @@ const Milliseconds kRollbackOplogSocketTimeout(10 * 60 * 1000);
 // The number of times a node attempted to choose a node to sync from among the available sync
 // source options. This occurs if we re-evaluate our sync source, receive an error from the source,
 // or step down.
-Counter64 numSyncSourceSelections;
-ServerStatusMetricField<Counter64> displayNumSyncSourceSelections("repl.syncSource.numSelections",
-                                                                  &numSyncSourceSelections);
+CounterMetric numSyncSourceSelections("repl.syncSource.numSelections");
 
 // The number of times a node kept it's original sync source after re-evaluating if its current sync
 // source was optimal.
-Counter64 numTimesChoseSameSyncSource;
-ServerStatusMetricField<Counter64> displayNumTimesChoseSameSyncSource(
-    "repl.syncSource.numTimesChoseSame", &numTimesChoseSameSyncSource);
+CounterMetric numTimesChoseSameSyncSource("repl.syncSource.numTimesChoseSame");
 
 // The number of times a node chose a new sync source after re-evaluating if its current sync source
 // was optimal.
-Counter64 numTimesChoseDifferentSyncSource;
-ServerStatusMetricField<Counter64> displayNumTimesChoseDifferentSyncSource(
-    "repl.syncSource.numTimesChoseDifferent", &numTimesChoseDifferentSyncSource);
+CounterMetric numTimesChoseDifferentSyncSource("repl.syncSource.numTimesChoseDifferent");
 
 // The number of times a node could not find a sync source when choosing a node to sync from among
 // the available options.
-Counter64 numTimesCouldNotFindSyncSource;
-ServerStatusMetricField<Counter64> displayNumTimesCouldNotFindSyncSource(
-    "repl.syncSource.numTimesCouldNotFind", &numTimesCouldNotFindSyncSource);
+CounterMetric numTimesCouldNotFindSyncSource("repl.syncSource.numTimesCouldNotFind");
 
 /**
  * Extends DataReplicatorExternalStateImpl to be member state aware.
@@ -111,9 +101,14 @@ public:
         ReplicationCoordinator* replicationCoordinator,
         ReplicationCoordinatorExternalState* replicationCoordinatorExternalState,
         BackgroundSync* bgsync);
-    bool shouldStopFetching(const HostAndPort& source,
-                            const rpc::ReplSetMetadata& replMetadata,
-                            const rpc::OplogQueryMetadata& oqMetadata) override;
+    ChangeSyncSourceAction shouldStopFetching(const HostAndPort& source,
+                                              const rpc::ReplSetMetadata& replMetadata,
+                                              const rpc::OplogQueryMetadata& oqMetadata,
+                                              const OpTime& previousOpTimeFetched,
+                                              const OpTime& lastOpTimeFetched) const final;
+
+    ChangeSyncSourceAction shouldStopFetchingOnError(const HostAndPort& source,
+                                                     const OpTime& lastOpTimeFetched) const final;
 
 private:
     BackgroundSync* _bgsync;
@@ -126,15 +121,36 @@ DataReplicatorExternalStateBackgroundSync::DataReplicatorExternalStateBackground
     : DataReplicatorExternalStateImpl(replicationCoordinator, replicationCoordinatorExternalState),
       _bgsync(bgsync) {}
 
-bool DataReplicatorExternalStateBackgroundSync::shouldStopFetching(
+ChangeSyncSourceAction DataReplicatorExternalStateBackgroundSync::shouldStopFetching(
     const HostAndPort& source,
     const rpc::ReplSetMetadata& replMetadata,
-    const rpc::OplogQueryMetadata& oqMetadata) {
-    if (_bgsync->shouldStopFetching()) {
-        return true;
+    const rpc::OplogQueryMetadata& oqMetadata,
+    const OpTime& previousOpTimeFetched,
+    const OpTime& lastOpTimeFetched) const {
+    if (getReplicationCoordinator()->shouldDropSyncSourceAfterShardSplit(
+            replMetadata.getReplicaSetId())) {
+        // Drop the last batch of message following a change of replica set due to a shard split.
+        LOGV2(6493902,
+              "Choosing new sync source because we have joined a new replica set following a shard "
+              "split.");
+        return ChangeSyncSourceAction::kStopSyncingAndDropLastBatchIfPresent;
     }
 
-    return DataReplicatorExternalStateImpl::shouldStopFetching(source, replMetadata, oqMetadata);
+    if (_bgsync->shouldStopFetching()) {
+        return ChangeSyncSourceAction::kStopSyncingAndEnqueueLastBatch;
+    }
+
+    return DataReplicatorExternalStateImpl::shouldStopFetching(
+        source, replMetadata, oqMetadata, previousOpTimeFetched, lastOpTimeFetched);
+}
+
+ChangeSyncSourceAction DataReplicatorExternalStateBackgroundSync::shouldStopFetchingOnError(
+    const HostAndPort& source, const OpTime& lastOpTimeFetched) const {
+    if (_bgsync->shouldStopFetching()) {
+        return ChangeSyncSourceAction::kStopSyncingAndDropLastBatchIfPresent;
+    }
+
+    return DataReplicatorExternalStateImpl::shouldStopFetchingOnError(source, lastOpTimeFetched);
 }
 
 size_t getSize(const BSONObj& o) {
@@ -171,6 +187,8 @@ void BackgroundSync::shutdown(OperationContext* opCtx) {
     stdx::lock_guard<Latch> lock(_mutex);
 
     setState(lock, ProducerState::Stopped);
+    // If we happen to be waiting for sync source data, stop.
+    _notifySyncSourceSelectionDataChanged(lock);
 
     if (_syncSourceResolver) {
         _syncSourceResolver->shutdown();
@@ -264,7 +282,7 @@ void BackgroundSync::_produce() {
         LOGV2(21079,
               "bgsync - stopReplProducer fail point "
               "enabled. Blocking until fail point is disabled.");
-        mongo::sleepsecs(1);
+        mongo::sleepmillis(_getRetrySleepMS());
         return;
     }
 
@@ -293,16 +311,10 @@ void BackgroundSync::_produce() {
 
     // find a target to sync from the last optime fetched
     {
-        OpTime minValidSaved;
-        {
-            auto opCtx = cc().makeOperationContext();
-            minValidSaved = _replicationProcess->getConsistencyMarkers()->getMinValid(opCtx.get());
-        }
         stdx::lock_guard<Latch> lock(_mutex);
         if (_state != ProducerState::Running) {
             return;
         }
-        const auto requiredOpTime = (minValidSaved > _lastOpTimeFetched) ? minValidSaved : OpTime();
         lastOpTimeFetched = _lastOpTimeFetched;
         if (!_syncSourceHost.empty()) {
             LOGV2(21080,
@@ -315,8 +327,13 @@ void BackgroundSync::_produce() {
             _replicationCoordinatorExternalState->getTaskExecutor(),
             _replCoord,
             lastOpTimeFetched,
-            requiredOpTime,
+            OpTime(),
             [&syncSourceResp](const SyncSourceResolverResponse& resp) { syncSourceResp = resp; });
+        // It is possible for _syncSourceSelectionDataChanged to become true between when we release
+        // the lock at the end of this block and when the syncSourceResolver retrieves the relevant
+        // heartbeat data, which means if we don't get a sync source we won't sleep even though we
+        // used the relevant data.  But that's OK because we'll only spin once.
+        _syncSourceSelectionDataChanged = false;
     }
     // This may deadlock if called inside the mutex because SyncSourceResolver::startup() calls
     // ReplicationCoordinator::chooseNewSyncSource(). ReplicationCoordinatorImpl's mutex has to
@@ -337,7 +354,7 @@ void BackgroundSync::_produce() {
 
     numSyncSourceSelections.increment(1);
 
-    if (syncSourceResp.syncSourceStatus == ErrorCodes::OplogStartMissing) {
+    if (syncSourceResp.syncSourceStatus == ErrorCodes::TooStaleToSyncFromSource) {
         // All (accessible) sync sources are too far ahead of us.
         if (_replCoord->getMemberState().primary()) {
             LOGV2_WARNING(21115,
@@ -403,23 +420,18 @@ void BackgroundSync::_produce() {
             source = _syncSourceHost;
         }
         // If our sync source has not changed, it is likely caused by our heartbeat data map being
-        // out of date. In that case we sleep for 1 second to reduce the amount we spin waiting
-        // for our map to update.
+        // out of date. In that case we sleep for up to 1 second to reduce the amount we spin
+        // waiting for our map to update.  If we are notified of heartbeat data change, we will
+        // interrupt the wait early.
         if (oldSource == source) {
-            long long sleepMS = 1000;
-            forceBgSyncSyncSourceRetryWaitMS.execute(
-                [&](const BSONObj& data) { sleepMS = data["sleepMS"].numberInt(); });
-
+            long long sleepMS = _getRetrySleepMS();
             LOGV2(21087,
-                  "Chose same sync source candidate as last time, {syncSource}. Sleeping for "
-                  "{sleepDurationMillis}ms to avoid immediately choosing a new sync source for the "
-                  "same reason as last time.",
                   "Chose same sync source candidate as last time. Sleeping to avoid immediately "
                   "choosing a new sync source for the same reason as last time",
                   "syncSource"_attr = source,
                   "sleepDurationMillis"_attr = sleepMS);
             numTimesChoseSameSyncSource.increment(1);
-            mongo::sleepmillis(sleepMS);
+            _waitForNewSyncSourceSelectionData(sleepMS);
         } else {
             LOGV2(21088,
                   "Changed sync source from {oldSyncSource} to {newSyncSource}",
@@ -438,19 +450,14 @@ void BackgroundSync::_produce() {
                   "error"_attr = syncSourceResp.syncSourceStatus.getStatus());
         }
 
-        long long sleepMS = 1000;
-        forceBgSyncSyncSourceRetryWaitMS.execute(
-            [&](const BSONObj& data) { sleepMS = data["sleepMS"].numberInt(); });
-
+        long long sleepMS = _getRetrySleepMS();
         // No sync source found.
         LOGV2_DEBUG(21090,
                     1,
-                    "Could not find a sync source. Sleeping for {sleepDurationMillis}ms before "
-                    "trying again.",
                     "Could not find a sync source. Sleeping before trying again",
                     "sleepDurationMillis"_attr = sleepMS);
         numTimesCouldNotFindSyncSource.increment(1);
-        mongo::sleepmillis(sleepMS);
+        _waitForNewSyncSourceSelectionData(sleepMS);
         return;
     }
 
@@ -484,39 +491,22 @@ void BackgroundSync::_produce() {
         _replCoord->signalUpstreamUpdater();
     }
 
-    // Set the applied point if unset. This is most likely the first time we've established a sync
-    // source since stepping down or otherwise clearing the applied point. We need to set this here,
-    // before the OplogWriter gets a chance to append to the oplog.
-    {
-        auto opCtx = cc().makeOperationContext();
-
-        // Check if the producer has been stopped so that we can prevent setting the applied point
-        // after step up has already cleared it. We need to acquire the collection lock before the
-        // mutex to preserve proper lock ordering.
-        AutoGetCollection autoColl(
-            opCtx.get(),
-            NamespaceString(ReplicationConsistencyMarkersImpl::kDefaultMinValidNamespace),
-            MODE_IX);
-        stdx::lock_guard<Latch> lock(_mutex);
-
-        if (_state != ProducerState::Running) {
-            return;
-        }
-
-        if (_replicationProcess->getConsistencyMarkers()->getAppliedThrough(opCtx.get()).isNull()) {
-            _replicationProcess->getConsistencyMarkers()->setAppliedThrough(
-                opCtx.get(), _replCoord->getMyLastAppliedOpTime());
-        }
-    }
-
     // "lastFetched" not used. Already set in _enqueueDocuments.
     Status fetcherReturnStatus = Status::OK();
+    int syncSourceRBID = syncSourceResp.rbid;
+
     DataReplicatorExternalStateBackgroundSync dataReplicatorExternalState(
         _replCoord, _replicationCoordinatorExternalState, this);
     OplogFetcher* oplogFetcher;
     try {
-        auto onOplogFetcherShutdownCallbackFn = [&fetcherReturnStatus](const Status& status) {
+        auto onOplogFetcherShutdownCallbackFn = [&fetcherReturnStatus,
+                                                 &syncSourceRBID](const Status& status, int rbid) {
             fetcherReturnStatus = status;
+            // If the syncSourceResp rbid is uninitialized, syncSourceRBID will be set to the
+            // rbid obtained in the oplog fetcher.
+            if (syncSourceRBID == ReplicationProcess::kUninitializedRollbackId) {
+                syncSourceRBID = rbid;
+            }
         };
         // The construction of OplogFetcher has to be outside bgsync mutex, because it calls
         // replication coordinator.
@@ -524,18 +514,17 @@ void BackgroundSync::_produce() {
             _replicationCoordinatorExternalState->getOplogFetcherSteadyStateMaxFetcherRestarts();
         auto oplogFetcherPtr = std::make_unique<OplogFetcher>(
             _replicationCoordinatorExternalState->getTaskExecutor(),
-            lastOpTimeFetched,
-            source,
-            _replCoord->getConfig(),
             std::make_unique<OplogFetcher::OplogFetcherRestartDecisionDefault>(numRestarts),
-            syncSourceResp.rbid,
-            true /* requireFresherSyncSource */,
             &dataReplicatorExternalState,
             [this](const auto& a1, const auto& a2, const auto& a3) {
                 return this->_enqueueDocuments(a1, a2, a3);
             },
             onOplogFetcherShutdownCallbackFn,
-            bgSyncOplogFetcherBatchSize);
+            OplogFetcher::Config(lastOpTimeFetched,
+                                 source,
+                                 _replCoord->getConfig(),
+                                 syncSourceResp.rbid,
+                                 bgSyncOplogFetcherBatchSize));
         stdx::lock_guard<Latch> lock(_mutex);
         if (_state != ProducerState::Running) {
             return;
@@ -581,20 +570,20 @@ void BackgroundSync::_produce() {
         return;
     }
 
+    Milliseconds denylistDuration(60000);
     if (fetcherReturnStatus.code() == ErrorCodes::OplogOutOfOrder) {
         // This is bad because it means that our source
         // has not returned oplog entries in ascending ts order, and they need to be.
 
         LOGV2_WARNING(
-            21120, "{error}", "Fetcher returned error", "error"_attr = redact(fetcherReturnStatus));
-        // Do not blacklist the server here, it will be blacklisted when we try to reuse it,
+            21120, "Oplog fetcher returned error", "error"_attr = redact(fetcherReturnStatus));
+        // Do not denylist the server here, it will be denylisted when we try to reuse it,
         // if it can't return a matching oplog start from the last fetch oplog ts field.
         return;
     } else if (fetcherReturnStatus.code() == ErrorCodes::OplogStartMissing) {
         auto opCtx = cc().makeOperationContext();
         auto storageInterface = StorageInterface::get(opCtx.get());
-        _runRollback(
-            opCtx.get(), fetcherReturnStatus, source, syncSourceResp.rbid, storageInterface);
+        _runRollback(opCtx.get(), fetcherReturnStatus, source, syncSourceRBID, storageInterface);
 
         if (bgSyncHangAfterRunRollback.shouldFail()) {
             LOGV2(21095, "bgSyncHangAfterRunRollback failpoint is set");
@@ -602,19 +591,38 @@ void BackgroundSync::_produce() {
                 mongo::sleepmillis(100);
             }
         }
+    } else if (fetcherReturnStatus.code() == ErrorCodes::TooStaleToSyncFromSource) {
+        LOGV2_WARNING(
+            5579700,
+            "Oplog fetcher discovered we are too stale to sync from sync source. Denylisting "
+            "sync source",
+            "syncSource"_attr = source,
+            "denylistDuration"_attr = denylistDuration);
+        _replCoord->denylistSyncSource(source, Date_t::now() + denylistDuration);
     } else if (fetcherReturnStatus == ErrorCodes::InvalidBSON) {
-        Seconds blacklistDuration(60);
-        LOGV2_WARNING(21121,
-                      "Fetcher got invalid BSON while querying oplog. Blacklisting sync source "
-                      "{syncSource} for {blacklistDuration}.",
-                      "Fetcher got invalid BSON while querying oplog. Blacklisting sync source",
-                      "syncSource"_attr = source,
-                      "blacklistDuration"_attr = blacklistDuration);
-        _replCoord->blacklistSyncSource(source, Date_t::now() + blacklistDuration);
+        LOGV2_WARNING(
+            5579701,
+            "Oplog fetcher got invalid BSON while querying oplog. Denylisting sync source "
+            "{syncSource} for {denylistDuration}.",
+            "Oplog fetcher got invalid BSON while querying oplog. Denylisting sync source",
+            "syncSource"_attr = source,
+            "denylistDuration"_attr = denylistDuration);
+        _replCoord->denylistSyncSource(source, Date_t::now() + denylistDuration);
+    } else if (fetcherReturnStatus.code() == ErrorCodes::ShutdownInProgress) {
+        if (auto quiesceInfo = fetcherReturnStatus.extraInfo<ShutdownInProgressQuiesceInfo>()) {
+            denylistDuration = Milliseconds(quiesceInfo->getRemainingQuiesceTimeMillis());
+            LOGV2_WARNING(
+                5579702,
+                "Sync source was in quiesce mode while we were querying its oplog. Denylisting "
+                "sync source",
+                "syncSource"_attr = source,
+                "denylistDuration"_attr = denylistDuration);
+            _replCoord->denylistSyncSource(source, Date_t::now() + denylistDuration);
+        }
     } else if (!fetcherReturnStatus.isOK()) {
         LOGV2_WARNING(21122,
-                      "Fetcher stopped querying remote oplog with error: {error}",
-                      "Fetcher stopped querying remote oplog with error",
+                      "Oplog fetcher stopped querying remote oplog with error: {error}",
+                      "Oplog fetcher stopped querying remote oplog with error",
                       "error"_attr = redact(fetcherReturnStatus));
     }
 }
@@ -838,6 +846,33 @@ void BackgroundSync::_fallBackOnRollbackViaRefetch(
     rollback(opCtx, *localOplog, rollbackSource, requiredRBID, _replCoord, _replicationProcess);
 }
 
+void BackgroundSync::notifySyncSourceSelectionDataChanged() {
+    stdx::lock_guard lock(_mutex);
+    _notifySyncSourceSelectionDataChanged(lock);
+}
+
+void BackgroundSync::_notifySyncSourceSelectionDataChanged(WithLock) {
+    if (!_syncSourceSelectionDataChanged) {
+        _syncSourceSelectionDataChanged = true;
+        _syncSourceSelectionDataCv.notify_one();
+    }
+}
+
+void BackgroundSync::_waitForNewSyncSourceSelectionData(long long waitTimeMillis) {
+    stdx::unique_lock<Latch> lock(_mutex);
+    if (_syncSourceSelectionDataCv.wait_for(
+            lock, stdx::chrono::milliseconds(waitTimeMillis), [this] {
+                return _syncSourceSelectionDataChanged || _inShutdown;
+            })) {
+        LOGV2_DEBUG(6795401,
+                    1,
+                    "Sync source wait interrupted early",
+                    "syncSourceSelectionDataChanged"_attr = _syncSourceSelectionDataChanged,
+                    "inShutdown"_attr = _inShutdown,
+                    "waitTimeMillis"_attr = waitTimeMillis);
+    }
+}
+
 HostAndPort BackgroundSync::getSyncTarget() const {
     stdx::unique_lock<Latch> lock(_mutex);
     return _syncSourceHost;
@@ -850,12 +885,16 @@ void BackgroundSync::clearSyncTarget() {
           "Resetting sync source to empty",
           "previousSyncSource"_attr = _syncSourceHost);
     _syncSourceHost = HostAndPort();
+    _notifySyncSourceSelectionDataChanged(lock);
 }
 
 void BackgroundSync::_stop(WithLock lock, bool resetLastFetchedOptime) {
     setState(lock, ProducerState::Stopped);
     LOGV2(21107, "Stopping replication producer");
 
+    // If we happen to be waiting for sync source data, stop.
+    _notifySyncSourceSelectionDataChanged(lock);
+
     _syncSourceHost = HostAndPort();
     if (resetLastFetchedOptime) {
         invariant(_oplogApplier->getBuffer()->isEmpty());
@@ -926,8 +965,7 @@ OpTime BackgroundSync::_readLastAppliedOpTime(OperationContext* opCtx) {
     try {
         bool success = writeConflictRetry(
             opCtx, "readLastAppliedOpTime", NamespaceString::kRsOplogNamespace.ns(), [&] {
-                return Helpers::getLast(
-                    opCtx, NamespaceString::kRsOplogNamespace.ns().c_str(), oplogEntry);
+                return Helpers::getLast(opCtx, NamespaceString::kRsOplogNamespace, oplogEntry);
             });
 
         if (!success) {
@@ -990,6 +1028,12 @@ void BackgroundSync::startProducerIfStopped() {
     }
 }
 
+long long BackgroundSync::_getRetrySleepMS() {
+    long long sleepMS = 1000;
+    forceBgSyncSyncSourceRetryWaitMS.execute(
+        [&](const BSONObj& data) { sleepMS = data["sleepMS"].numberInt(); });
+    return sleepMS;
+}
 
 }  // namespace repl
 }  // namespace mongo
