@@ -27,13 +27,12 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplicationInitialSync
 
-#include "mongo/db/index/index_descriptor_fwd.h"
-#include "mongo/db/service_context.h"
 #include "mongo/platform/basic.h"
 
 #include "mongo/base/string_data.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
+#include "mongo/db/catalog/clustered_collection_util.h"
 #include "mongo/db/commands/list_collections_filter.h"
 #include "mongo/db/index_build_entry_helpers.h"
 #include "mongo/db/index_builds_coordinator.h"
@@ -47,6 +46,9 @@
 
 #include "mongo/util/assert_util.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplicationInitialSync
+
+
 namespace mongo {
 namespace repl {
 
@@ -65,7 +67,8 @@ CollectionCloner::CollectionCloner(const NamespaceString& sourceNss,
                                    DBClientConnection* client,
                                    StorageInterface* storageInterface,
                                    ThreadPool* dbPool)
-    : BaseCloner("CollectionCloner"_sd, sharedData, source, client, storageInterface, dbPool),
+    : InitialSyncBaseCloner(
+          "CollectionCloner"_sd, sharedData, source, client, storageInterface, dbPool),
       _sourceNss(sourceNss),
       _collectionOptions(collectionOptions),
       _sourceDbAndUuid(NamespaceString("UNINITIALIZED")),
@@ -90,7 +93,7 @@ CollectionCloner::CollectionCloner(const NamespaceString& sourceNss,
               try {
                   work(executor::TaskExecutor::CallbackArgs(nullptr, {}, status, opCtx));
               } catch (const DBException& e) {
-                  setInitialSyncFailedStatus(e.toStatus());
+                  setSyncFailedStatus(e.toStatus());
               }
               return TaskRunner::NextAction::kDisposeOperationContext;
           };
@@ -102,12 +105,16 @@ CollectionCloner::CollectionCloner(const NamespaceString& sourceNss,
     invariant(collectionOptions.uuid);
     _sourceDbAndUuid = NamespaceStringOrUUID(sourceNss.db().toString(), *collectionOptions.uuid);
     _stats.ns = _sourceNss.ns();
-
-    // Find out whether the sync source supports resumable queries.
-    _resumeSupported = (getClient()->getMaxWireVersion() >= WireVersion::RESUMABLE_INITIAL_SYNC);
 }
 
 BaseCloner::ClonerStages CollectionCloner::getStages() {
+    if (_sourceNss.isChangeStreamPreImagesCollection() || _sourceNss.isChangeCollection()) {
+        // The change stream pre-images collection and the change collection only need to be created
+        // - their documents should not be copied.
+        return {&_listIndexesStage,
+                &_createCollectionStage,
+                &_setupIndexBuildersForUnfinishedIndexesStage};
+    }
     return {&_countStage,
             &_listIndexesStage,
             &_createCollectionStage,
@@ -134,7 +141,7 @@ void CollectionCloner::preStage() {
                     1,
                     "Skipping the recording of some initial sync metrics due to failure in the "
                     "'collStats' command",
-                    "ns"_attr = _sourceNss.coll().toString(),
+                    logAttrs(_sourceNss),
                     "status"_attr = status);
     }
 }
@@ -166,10 +173,10 @@ BaseCloner::AfterStageBehavior CollectionCloner::CollectionClonerStage::run() {
 BaseCloner::AfterStageBehavior CollectionCloner::countStage() {
     auto count = getClient()->count(_sourceDbAndUuid,
                                     {} /* Query */,
-                                    QueryOption_SlaveOk,
+                                    QueryOption_SecondaryOk,
                                     0 /* limit */,
                                     0 /* skip */,
-                                    ReadConcernArgs::kImplicitDefault);
+                                    ReadConcernArgs::kLocal);
 
     // The count command may return a negative value after an unclean shutdown,
     // so we set it to zero here to avoid aborting the collection clone.
@@ -190,28 +197,9 @@ BaseCloner::AfterStageBehavior CollectionCloner::countStage() {
 }
 
 BaseCloner::AfterStageBehavior CollectionCloner::listIndexesStage() {
-    std::list<BSONObj> indexSpecs;
-    bool includeBuildUUIDs = false;
-
-    if (IndexBuildsCoordinator::supportsTwoPhaseIndexBuild()) {
-        // In FCV 4.4, build all the ready indexes to completion and start building the unfinished
-        // indexes without commiting them until the outcome of the index build is determined.
-        invariant(serverGlobalParams.featureCompatibility.isVersionInitialized() &&
-                  serverGlobalParams.featureCompatibility.getVersion() ==
-                      ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44);
-
-        includeBuildUUIDs = true;
-        indexSpecs =
-            getClient()->getIndexSpecs(_sourceDbAndUuid, includeBuildUUIDs, QueryOption_SlaveOk);
-    } else {
-        // In FCV 4.2, build all the ready and unfinished indexes from the sync source to
-        // completion. Using 'includeBuildUUIDs = false', the listIndexes commands returns all ready
-        // and unfinished indexes without a way to distinguish between the two states.
-        includeBuildUUIDs = false;
-        indexSpecs =
-            getClient()->getIndexSpecs(_sourceDbAndUuid, includeBuildUUIDs, QueryOption_SlaveOk);
-    }
-
+    const bool includeBuildUUIDs = true;
+    auto indexSpecs =
+        getClient()->getIndexSpecs(_sourceDbAndUuid, includeBuildUUIDs, QueryOption_SecondaryOk);
     if (indexSpecs.empty()) {
         LOGV2_WARNING(21143,
                       "No indexes found for collection {namespace} while cloning from {source}",
@@ -220,22 +208,16 @@ BaseCloner::AfterStageBehavior CollectionCloner::listIndexesStage() {
                       "source"_attr = getSource());
     }
 
-    const auto storageEngine = getGlobalServiceContext()->getStorageEngine();
     // Parse the index specs into their respective state, ready or unfinished.
     for (auto&& spec : indexSpecs) {
-        // Sanitize storage engine options to remove options which might not apply to this node. See
-        // SERVER-68122.
-        if (auto storageEngineElem = spec.getField(IndexDescriptor::kStorageEngineFieldName)) {
-            auto sanitizedStorageEngineOpts =
-                storageEngine->getSanitizedStorageOptionsForSecondaryReplication(
-                    storageEngineElem.embeddedObject());
-            spec = spec.addField(
-                BSON(IndexDescriptor::kStorageEngineFieldName << sanitizedStorageEngineOpts)
-                    .firstElement());
-        }
-
-        if (spec.hasField("buildUUID")) {
-            invariant(includeBuildUUIDs);
+        if (spec.hasField("clustered")) {
+            invariant(_collectionOptions.clusteredIndex);
+            invariant(spec.getBoolField("clustered") == true);
+            invariant(clustered_util::formatClusterKeyForListIndexes(
+                          _collectionOptions.clusteredIndex.value(), _collectionOptions.collation)
+                          .woCompare(spec) == 0);
+            // Skip if the spec is for the collection's clusteredIndex.
+        } else if (spec.hasField("buildUUID")) {
             _unfinishedIndexSpecs.push_back(spec.getOwned());
         } else if (spec.hasField("name") && spec.getStringField("name") == "_id_"_sd) {
             _idIndexSpec = spec.getOwned();
@@ -261,13 +243,6 @@ BaseCloner::AfterStageBehavior CollectionCloner::listIndexesStage() {
 }
 
 BaseCloner::AfterStageBehavior CollectionCloner::createCollectionStage() {
-    if (!IndexBuildsCoordinator::supportsTwoPhaseIndexBuild()) {
-        // Single phase index builds should have an empty '_unfinishedIndexSpecs' vector because in
-        // the 'listIndexesStage', we only populate '_unfinishedIndexSpecs' if a buildUUID is
-        // present. A buildUUID is only present for two phase index builds.
-        invariant(_unfinishedIndexSpecs.empty());
-    }
-
     auto collectionBulkLoader = getStorageInterface()->createCollectionForBulkLoading(
         _sourceNss, _collectionOptions, _idIndexSpec, _readyIndexSpecs);
     uassertStatusOK(collectionBulkLoader.getStatus());
@@ -276,8 +251,6 @@ BaseCloner::AfterStageBehavior CollectionCloner::createCollectionStage() {
 }
 
 BaseCloner::AfterStageBehavior CollectionCloner::queryStage() {
-    // Attempt to clean up cursor from the last retry (if applicable).
-    killOldQueryCursor();
     runQuery();
     waitForDatabaseWorkToComplete();
     // We want to free the _collLoader regardless of whether the commit succeeds.
@@ -287,7 +260,7 @@ BaseCloner::AfterStageBehavior CollectionCloner::queryStage() {
 }
 
 BaseCloner::AfterStageBehavior CollectionCloner::setupIndexBuildersForUnfinishedIndexesStage() {
-    if (!IndexBuildsCoordinator::supportsTwoPhaseIndexBuild() || _unfinishedIndexSpecs.empty()) {
+    if (_unfinishedIndexSpecs.empty()) {
         return kContinueNormally;
     }
 
@@ -305,7 +278,8 @@ BaseCloner::AfterStageBehavior CollectionCloner::setupIndexBuildersForUnfinished
         std::vector<std::string> indexNames;
         std::vector<BSONObj> indexSpecs;
         for (const auto& indexSpec : groupedIndexSpec.second) {
-            std::string indexName = indexSpec.getStringField(IndexDescriptor::kIndexNameFieldName);
+            std::string indexName =
+                indexSpec.getStringField(IndexDescriptor::kIndexNameFieldName).toString();
             indexNames.push_back(indexName);
             indexSpecs.push_back(indexSpec.getOwned());
         }
@@ -343,86 +317,51 @@ BaseCloner::AfterStageBehavior CollectionCloner::setupIndexBuildersForUnfinished
 }
 
 void CollectionCloner::runQuery() {
-    // Non-resumable query.
-    Query query;
+    FindCommandRequest findCmd{_sourceDbAndUuid};
 
-    if (_resumeSupported) {
-        if (_resumeToken) {
-            // Resume the query from where we left off.
-            LOGV2_DEBUG(21133, 1, "Collection cloner will resume the last successful query");
-            query = QUERY("query" << BSONObj() << "$_requestResumeToken" << true << "$_resumeAfter"
-                                  << _resumeToken.get());
-        } else {
-            // New attempt at a resumable query.
-            LOGV2_DEBUG(21134, 1, "Collection cloner will run a new query");
-            query = QUERY("query" << BSONObj() << "$_requestResumeToken" << true);
-        }
-        query.hint(BSON("$natural" << 1));
+    if (_resumeToken) {
+        // Resume the query from where we left off.
+        LOGV2_DEBUG(21133, 1, "Collection cloner will resume the last successful query");
+        findCmd.setRequestResumeToken(true);
+        findCmd.setResumeAfter(_resumeToken.value());
+    } else {
+        // New attempt at a resumable query.
+        LOGV2_DEBUG(21134, 1, "Collection cloner will run a new query");
+        findCmd.setRequestResumeToken(true);
     }
 
+    findCmd.setHint(BSON("$natural" << 1));
+    findCmd.setNoCursorTimeout(true);
+    findCmd.setReadConcern(ReadConcernArgs::kLocal);
+    if (_collectionClonerBatchSize) {
+        findCmd.setBatchSize(_collectionClonerBatchSize);
+    }
+
+    ExhaustMode exhaustMode = collectionClonerUsesExhaust ? ExhaustMode::kOn : ExhaustMode::kOff;
+
     // We reset this every time we retry or resume a query.
     // We distinguish the first batch from the rest so that we only store the remote cursor id
     // the first time we get it.
     _firstBatchOfQueryRound = true;
 
-    try {
-        getClient()->query([this](DBClientCursorBatchIterator& iter) { handleNextBatch(iter); },
-                           _sourceDbAndUuid,
-                           query,
-                           nullptr /* fieldsToReturn */,
-                           QueryOption_NoCursorTimeout | QueryOption_SlaveOk |
-                               (collectionClonerUsesExhaust ? QueryOption_Exhaust : 0),
-                           _collectionClonerBatchSize,
-                           ReadConcernArgs::kImplicitDefault);
-    } catch (...) {
-        auto status = exceptionToStatus();
+    auto cursor = getClient()->find(
+        std::move(findCmd), ReadPreferenceSetting{ReadPreference::SecondaryPreferred}, exhaustMode);
 
-        // If the collection was dropped at any point, we can just move on to the next cloner.
-        // This applies to both resumable (4.4) and non-resumable (4.2) queries.
-        if (status == ErrorCodes::NamespaceNotFound) {
-            throw;  // This will re-throw the NamespaceNotFound, resulting in a clean exit.
-        }
-
-        // Wire version 4.2 only.
-        if (!_resumeSupported) {
-            // If we lost our cursor last round, the only time we can can continue is if we find out
-            // this round that the collection was dropped on the source (that scenario is covered
-            // right above). If that is not the case, then the cloner would have more work to do,
-            // but since we cannot resume the query, we must abort initial sync.
-            if (_lostNonResumableCursor) {
-                abortNonResumableClone(status);
-            }
-
-            // Collection has changed upstream. This will trigger the code block above next round,
-            // (unless we find out the collection was dropped via getting a NamespaceNotFound).
-            if (_queryStage.isCursorError(status)) {
-                LOGV2(21135,
-                      "Lost cursor during non-resumable query: {error}",
-                      "Lost cursor during non-resumable query",
-                      "error"_attr = status);
-                _lostNonResumableCursor = true;
-                throw;
-            }
-            // Any other errors (including network errors, but excluding NamespaceNotFound) result
-            // in immediate failure.
-            abortNonResumableClone(status);
-        }
-
-        // Re-throw all query errors for resumable (4.4) queries.
-        throw;
+    // Process the results of the cursor one batch at a time.
+    while (cursor->more()) {
+        handleNextBatch(*cursor);
     }
 }
 
-void CollectionCloner::handleNextBatch(DBClientCursorBatchIterator& iter) {
+void CollectionCloner::handleNextBatch(DBClientCursor& cursor) {
     {
         stdx::lock_guard<InitialSyncSharedData> lk(*getSharedData());
-        if (!getSharedData()->getInitialSyncStatus(lk).isOK()) {
+        if (!getSharedData()->getStatus(lk).isOK()) {
             static constexpr char message[] =
                 "Collection cloning cancelled due to initial sync failure";
-            LOGV2(21136, message, "error"_attr = getSharedData()->getInitialSyncStatus(lk));
+            LOGV2(21136, message, "error"_attr = getSharedData()->getStatus(lk));
             uasserted(ErrorCodes::CallbackCanceled,
-                      str::stream()
-                          << message << ": " << getSharedData()->getInitialSyncStatus(lk));
+                      str::stream() << message << ": " << getSharedData()->getStatus(lk));
         }
     }
 
@@ -434,17 +373,17 @@ void CollectionCloner::handleNextBatch(DBClientCursorBatchIterator& iter) {
         uasserted(ErrorCodes::InitialSyncFailure, "Lost remote cursor");
     }
 
-    if (_firstBatchOfQueryRound && _resumeSupported) {
+    if (_firstBatchOfQueryRound) {
         // Store the cursorId of the remote cursor.
-        _remoteCursorId = iter.getCursorId();
+        _remoteCursorId = cursor.getCursorId();
     }
     _firstBatchOfQueryRound = false;
 
     {
         stdx::lock_guard<Latch> lk(_mutex);
         _stats.receivedBatches++;
-        while (iter.moreInCurrentBatch()) {
-            _documentsToInsert.emplace_back(iter.nextSafe());
+        while (cursor.moreInCurrentBatch()) {
+            _documentsToInsert.emplace_back(cursor.nextSafe());
         }
     }
 
@@ -459,10 +398,8 @@ void CollectionCloner::handleNextBatch(DBClientCursorBatchIterator& iter) {
         uassertStatusOK(newStatus);
     }
 
-    if (_resumeSupported) {
-        // Store the resume token for this batch.
-        _resumeToken = iter.getPostBatchResumeToken();
-    }
+    // Store the resume token for this batch.
+    _resumeToken = cursor.getPostBatchResumeToken();
 
     initialSyncHangCollectionClonerAfterHandlingBatchResponse.executeIf(
         [&](const BSONObj&) {
@@ -537,44 +474,6 @@ void CollectionCloner::waitForDatabaseWorkToComplete() {
     _dbWorkTaskRunner.join();
 }
 
-void CollectionCloner::killOldQueryCursor() {
-    // No cursor stored. Do nothing.
-    if (_remoteCursorId == -1) {
-        return;
-    }
-
-    BSONObj infoObj;
-    auto nss = _sourceNss;
-    auto id = _remoteCursorId;
-
-    auto cmdObj = BSON("killCursors" << nss.coll() << "cursors" << BSON_ARRAY(id));
-    LOGV2_DEBUG(21139, 1, "Attempting to kill old remote cursor with id: {id}", "id"_attr = id);
-    try {
-        getClient()->runCommand(nss.db().toString(), cmdObj, infoObj);
-    } catch (...) {
-        LOGV2(21140, "Error while trying to kill remote cursor after transient query error");
-    }
-
-    // Clear the stored cursorId on success.
-    _remoteCursorId = -1;
-}
-
-void CollectionCloner::forgetOldQueryCursor() {
-    _remoteCursorId = -1;
-}
-
-// Throws.
-void CollectionCloner::abortNonResumableClone(const Status& status) {
-    invariant(!_resumeSupported);
-    LOGV2(21141,
-          "Error during non-resumable clone: {error}",
-          "Error during non-resumable clone",
-          "error"_attr = status);
-    std::string message = str::stream()
-        << "Collection clone failed and is not resumable. nss: " << _sourceNss;
-    uasserted(ErrorCodes::InitialSyncFailure, message);
-}
-
 CollectionCloner::Stats CollectionCloner::getStats() const {
     stdx::lock_guard<Latch> lk(_mutex);
     return _stats;
@@ -593,10 +492,10 @@ BSONObj CollectionCloner::Stats::toBSON() const {
 }
 
 void CollectionCloner::Stats::append(BSONObjBuilder* builder) const {
-    builder->appendNumber(kDocumentsToCopyFieldName, documentToCopy);
-    builder->appendNumber(kDocumentsCopiedFieldName, documentsCopied);
-    builder->appendNumber("indexes", indexes);
-    builder->appendNumber("fetchedBatches", fetchedBatches);
+    builder->appendNumber(kDocumentsToCopyFieldName, static_cast<long long>(documentToCopy));
+    builder->appendNumber(kDocumentsCopiedFieldName, static_cast<long long>(documentsCopied));
+    builder->appendNumber("indexes", static_cast<long long>(indexes));
+    builder->appendNumber("fetchedBatches", static_cast<long long>(fetchedBatches));
     builder->appendNumber("bytesToCopy", bytesToCopy);
     if (bytesToCopy) {
         builder->appendNumber("approxBytesCopied", approxBytesCopied);
@@ -610,7 +509,7 @@ void CollectionCloner::Stats::append(BSONObjBuilder* builder) const {
             builder->appendNumber("elapsedMillis", elapsedMillis);
         }
     }
-    builder->appendNumber("receivedBatches", receivedBatches);
+    builder->appendNumber("receivedBatches", static_cast<long long>(receivedBatches));
 }
 
 }  // namespace repl
