@@ -43,12 +43,10 @@
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/client.h"
 #include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/jsobj.h"
-#include "mongo/db/logical_session_id.h"
 #include "mongo/db/query/internal_plans.h"
 #include "mongo/db/repl/bgsync.h"
 #include "mongo/db/repl/drop_pending_collection_reaper.h"
@@ -60,7 +58,7 @@
 #include "mongo/db/repl/replication_consistency_markers_mock.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/repl/storage_interface.h"
-#include "mongo/db/storage/durable_catalog.h"
+#include "mongo/db/session/logical_session_id.h"
 #include "mongo/util/md5.hpp"
 
 namespace mongo {
@@ -164,8 +162,7 @@ std::ostream& operator<<(std::ostream& stream, const CollectionState& state) {
     return stream << state.toString();
 }
 
-StringBuilderImpl<SharedBufferAllocator>& operator<<(StringBuilderImpl<SharedBufferAllocator>& sb,
-                                                     const CollectionState& state) {
+StringBuilder& operator<<(StringBuilder& sb, const CollectionState& state) {
     return sb << state.toString();
 }
 
@@ -212,12 +209,12 @@ void IdempotencyTest::testOpsAreIdempotent(std::vector<OplogEntry> ops, Sequence
 
         auto state2 = validateAllCollections();
         if (state1 != state2) {
-            FAIL(getStatesString(state1, state2, fullSequence));
+            FAIL(getStatesString(state1, state2, ops, fullSequence));
         }
     }
 }
 
-OplogEntry IdempotencyTest::createCollection(CollectionUUID uuid) {
+OplogEntry IdempotencyTest::createCollection(UUID uuid) {
     return makeCreateCollectionOplogEntry(nextOpTime(), nss, BSON("uuid" << uuid));
 }
 
@@ -266,7 +263,7 @@ OplogEntry IdempotencyTest::prepare(LogicalSessionId lsid,
                           boost::none /* o2 */,
                           info /* sessionInfo */,
                           Date_t::min() /* wallClockTime -- required but not checked */,
-                          stmtId,
+                          {stmtId},
                           boost::none /* uuid */,
                           prevOpTime);
 }
@@ -279,21 +276,21 @@ OplogEntry IdempotencyTest::commitUnprepared(LogicalSessionId lsid,
     OperationSessionInfo info;
     info.setSessionId(lsid);
     info.setTxnNumber(txnNum);
-    return makeCommandOplogEntryWithSessionInfoAndStmtId(
-        nextOpTime(), nss, BSON("applyOps" << ops), lsid, txnNum, stmtId, prevOpTime);
+    return makeCommandOplogEntryWithSessionInfoAndStmtIds(
+        nextOpTime(), nss, BSON("applyOps" << ops), lsid, txnNum, {stmtId}, prevOpTime);
 }
 
 OplogEntry IdempotencyTest::commitPrepared(LogicalSessionId lsid,
                                            TxnNumber txnNum,
                                            StmtId stmtId,
                                            OpTime prepareOpTime) {
-    return makeCommandOplogEntryWithSessionInfoAndStmtId(
+    return makeCommandOplogEntryWithSessionInfoAndStmtIds(
         nextOpTime(),
         nss,
         BSON("commitTransaction" << 1 << "commitTimestamp" << prepareOpTime.getTimestamp()),
         lsid,
         txnNum,
-        stmtId,
+        {stmtId},
         prepareOpTime);
 }
 
@@ -301,8 +298,8 @@ OplogEntry IdempotencyTest::abortPrepared(LogicalSessionId lsid,
                                           TxnNumber txnNum,
                                           StmtId stmtId,
                                           OpTime prepareOpTime) {
-    return makeCommandOplogEntryWithSessionInfoAndStmtId(
-        nextOpTime(), nss, BSON("abortTransaction" << 1), lsid, txnNum, stmtId, prepareOpTime);
+    return makeCommandOplogEntryWithSessionInfoAndStmtIds(
+        nextOpTime(), nss, BSON("abortTransaction" << 1), lsid, txnNum, {stmtId}, prepareOpTime);
 }
 
 OplogEntry IdempotencyTest::partialTxn(LogicalSessionId lsid,
@@ -320,21 +317,21 @@ OplogEntry IdempotencyTest::partialTxn(LogicalSessionId lsid,
                           boost::none /* o2 */,
                           info /* sessionInfo */,
                           Date_t::min() /* wallClockTime -- required but not checked */,
-                          stmtId,
+                          {stmtId},
                           boost::none /* uuid */,
                           prevOpTime);
 }
 
-std::string IdempotencyTest::computeDataHash(Collection* collection) {
+std::string IdempotencyTest::computeDataHash(const CollectionPtr& collection) {
     auto desc = collection->getIndexCatalog()->findIdIndex(_opCtx.get());
     ASSERT_TRUE(desc);
     auto exec = InternalPlanner::indexScan(_opCtx.get(),
-                                           collection,
+                                           &collection,
                                            desc,
                                            BSONObj(),
                                            BSONObj(),
                                            BoundInclusion::kIncludeStartKeyOnly,
-                                           PlanExecutor::NO_YIELD,
+                                           PlanYieldPolicy::YieldPolicy::NO_YIELD,
                                            InternalPlanner::FORWARD,
                                            InternalPlanner::IXSCAN_FETCH);
     ASSERT(nullptr != exec.get());
@@ -355,15 +352,15 @@ std::string IdempotencyTest::computeDataHash(Collection* collection) {
 
 std::vector<CollectionState> IdempotencyTest::validateAllCollections() {
     std::vector<CollectionState> collStates;
-    auto& catalog = CollectionCatalog::get(_opCtx.get());
-    auto dbs = catalog.getAllDbNames();
-    for (auto& db : dbs) {
+    auto catalog = CollectionCatalog::get(_opCtx.get());
+    auto dbNames = catalog->getAllDbNames();
+    for (auto& dbName : dbNames) {
         // Skip local database.
-        if (db != "local") {
+        if (dbName.db() != "local") {
             std::vector<NamespaceString> collectionNames;
             {
-                Lock::DBLock lk(_opCtx.get(), db, MODE_S);
-                collectionNames = catalog.getAllCollectionNamesFromDb(_opCtx.get(), db);
+                Lock::DBLock lk(_opCtx.get(), dbName, MODE_S);
+                collectionNames = catalog->getAllCollectionNamesFromDb(_opCtx.get(), dbName);
             }
             for (const auto& nss : collectionNames) {
                 collStates.push_back(validate(nss));
@@ -374,9 +371,9 @@ std::vector<CollectionState> IdempotencyTest::validateAllCollections() {
 }
 
 CollectionState IdempotencyTest::validate(const NamespaceString& nss) {
-    auto collUUID = [&]() -> OptionalCollectionUUID {
+    auto collUUID = [&]() -> boost::optional<UUID> {
         AutoGetCollectionForReadCommand autoColl(_opCtx.get(), nss);
-        if (auto collection = autoColl.getCollection()) {
+        if (const auto& collection = autoColl.getCollection()) {
             return collection->uuid();
         }
         return boost::none;
@@ -385,12 +382,11 @@ CollectionState IdempotencyTest::validate(const NamespaceString& nss) {
     if (collUUID) {
         // Allow in-progress indexes to complete before validating collection contents.
         IndexBuildsCoordinator::get(_opCtx.get())
-            ->awaitNoIndexBuildInProgressForCollection(_opCtx.get(), collUUID.get());
+            ->awaitNoIndexBuildInProgressForCollection(_opCtx.get(), collUUID.value());
     }
 
     {
-        AutoGetCollectionForReadCommand autoColl(_opCtx.get(), nss);
-        auto collection = autoColl.getCollection();
+        AutoGetCollectionForReadCommand collection(_opCtx.get(), nss);
 
         if (!collection) {
             // Return a mostly default initialized CollectionState struct with exists set to false
@@ -406,29 +402,23 @@ CollectionState IdempotencyTest::validate(const NamespaceString& nss) {
         ASSERT_OK(
             CollectionValidation::validate(_opCtx.get(),
                                            nss,
-                                           CollectionValidation::ValidateOptions::kFullValidation,
-                                           false,
+                                           CollectionValidation::ValidateMode::kForegroundFull,
                                            CollectionValidation::RepairMode::kNone,
                                            &validateResults,
-                                           &bob,
-                                           /*logDiagnostics=*/false));
+                                           &bob));
         ASSERT_TRUE(validateResults.valid);
     }
 
-    AutoGetCollectionForReadCommand autoColl(_opCtx.get(), nss);
-    auto collection = autoColl.getCollection();
+    AutoGetCollectionForReadCommand collection(_opCtx.get(), nss);
 
-    std::string dataHash = computeDataHash(collection);
+    std::string dataHash = computeDataHash(collection.getCollection());
 
-    auto durableCatalog = DurableCatalog::get(_opCtx.get());
-    auto collectionOptions =
-        durableCatalog->getCollectionOptions(_opCtx.get(), collection->getCatalogId());
+    auto collectionOptions = collection->getCollectionOptions();
     std::vector<std::string> allIndexes;
     BSONObjSet indexSpecs = SimpleBSONObjComparator::kInstance.makeBSONObjSet();
-    durableCatalog->getAllIndexes(_opCtx.get(), collection->getCatalogId(), &allIndexes);
+    collection->getAllIndexes(&allIndexes);
     for (auto const& index : allIndexes) {
-        indexSpecs.insert(
-            durableCatalog->getIndexSpec(_opCtx.get(), collection->getCatalogId(), index));
+        indexSpecs.insert(collection->getIndexSpec(index));
     }
     ASSERT_EQUALS(indexSpecs.size(), allIndexes.size());
 
@@ -439,7 +429,8 @@ CollectionState IdempotencyTest::validate(const NamespaceString& nss) {
 
 std::string IdempotencyTest::getStatesString(const std::vector<CollectionState>& state1,
                                              const std::vector<CollectionState>& state2,
-                                             const std::vector<OplogEntry>& ops) {
+                                             const std::vector<OplogEntry>& state1Ops,
+                                             const std::vector<OplogEntry>& state2Ops) {
     StringBuilder sb;
     sb << "The states:\n";
     for (const auto& s : state1) {
@@ -451,8 +442,8 @@ std::string IdempotencyTest::getStatesString(const std::vector<CollectionState>&
     }
     sb << "found after applying the operations a second time, therefore breaking idempotency.\n";
     sb << "Applied ops:\n";
-    for (const auto& op : ops) {
-        sb << op.toString() << "\n";
+    for (const auto& op : state2Ops) {
+        sb << op.toStringForLogging() << "\n";
     }
     return sb.str();
 }
