@@ -27,16 +27,16 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/repl/oplog_applier_impl_test_fixture.h"
 
+#include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/document_validation.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
-#include "mongo/db/logical_clock.h"
-#include "mongo/db/op_observer_registry.h"
+#include "mongo/db/index_builds_coordinator.h"
+#include "mongo/db/op_observer/op_observer_registry.h"
 #include "mongo/db/query/internal_plans.h"
 #include "mongo/db/repl/drop_pending_collection_reaper.h"
 #include "mongo/db/repl/oplog_applier.h"
@@ -46,13 +46,13 @@
 #include "mongo/db/repl/replication_recovery_mock.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/storage_interface_impl.h"
+#include "mongo/db/vector_clock_mutable.h"
 
 namespace mongo {
 namespace repl {
 
 void OplogApplierImplOpObserver::onInserts(OperationContext* opCtx,
-                                           const NamespaceString& nss,
-                                           OptionalCollectionUUID uuid,
+                                           const CollectionPtr& coll,
                                            std::vector<InsertStatement>::const_iterator begin,
                                            std::vector<InsertStatement>::const_iterator end,
                                            bool fromMigrate) {
@@ -64,31 +64,89 @@ void OplogApplierImplOpObserver::onInserts(OperationContext* opCtx,
         const InsertStatement& insertStatement = *it;
         docs.push_back(insertStatement.doc.getOwned());
     }
-    onInsertsFn(opCtx, nss, docs);
+    onInsertsFn(opCtx, coll->ns(), docs);
 }
 
 void OplogApplierImplOpObserver::onDelete(OperationContext* opCtx,
                                           const NamespaceString& nss,
-                                          OptionalCollectionUUID uuid,
+                                          const UUID& uuid,
                                           StmtId stmtId,
-                                          const OpObserver::OplogDeleteEntryArgs& args) {
+                                          const OplogDeleteEntryArgs& args) {
     if (!onDeleteFn) {
         return;
     }
     onDeleteFn(opCtx, nss, uuid, stmtId, args);
 }
 
+void OplogApplierImplOpObserver::onUpdate(OperationContext* opCtx,
+                                          const OplogUpdateEntryArgs& args) {
+    if (!onUpdateFn) {
+        return;
+    }
+    onUpdateFn(opCtx, args);
+}
+
 void OplogApplierImplOpObserver::onCreateCollection(OperationContext* opCtx,
-                                                    Collection* coll,
+                                                    const CollectionPtr& coll,
                                                     const NamespaceString& collectionName,
                                                     const CollectionOptions& options,
                                                     const BSONObj& idIndex,
-                                                    const OplogSlot& createOpTime) {
+                                                    const OplogSlot& createOpTime,
+                                                    bool fromMigrate) {
     if (!onCreateCollectionFn) {
         return;
     }
     onCreateCollectionFn(opCtx, coll, collectionName, options, idIndex);
 }
+
+void OplogApplierImplOpObserver::onRenameCollection(OperationContext* opCtx,
+                                                    const NamespaceString& fromCollection,
+                                                    const NamespaceString& toCollection,
+                                                    const UUID& uuid,
+                                                    const boost::optional<UUID>& dropTargetUUID,
+                                                    std::uint64_t numRecords,
+                                                    bool stayTemp) {
+    if (!onRenameCollectionFn) {
+        return;
+    }
+    onRenameCollectionFn(
+        opCtx, fromCollection, toCollection, uuid, dropTargetUUID, numRecords, stayTemp);
+}
+
+void OplogApplierImplOpObserver::onCreateIndex(OperationContext* opCtx,
+                                               const NamespaceString& nss,
+                                               const UUID& uuid,
+                                               BSONObj indexDoc,
+                                               bool fromMigrate) {
+    if (!onCreateIndexFn) {
+        return;
+    }
+    onCreateIndexFn(opCtx, nss, uuid, indexDoc, fromMigrate);
+}
+
+void OplogApplierImplOpObserver::onDropIndex(OperationContext* opCtx,
+                                             const NamespaceString& nss,
+                                             const UUID& uuid,
+                                             const std::string& indexName,
+                                             const BSONObj& idxDescriptor) {
+    if (!onDropIndexFn) {
+        return;
+    }
+    onDropIndexFn(opCtx, nss, uuid, indexName, idxDescriptor);
+}
+
+void OplogApplierImplOpObserver::onCollMod(OperationContext* opCtx,
+                                           const NamespaceString& nss,
+                                           const UUID& uuid,
+                                           const BSONObj& collModCmd,
+                                           const CollectionOptions& oldCollOptions,
+                                           boost::optional<IndexCollModInfo> indexInfo) {
+    if (!onCollModFn) {
+        return;
+    }
+    onCollModFn(opCtx, nss, uuid, collModCmd, oldCollOptions, indexInfo);
+}
+
 void OplogApplierImplTest::setUp() {
     ServiceContextMongoDTest::setUp();
 
@@ -103,7 +161,6 @@ void OplogApplierImplTest::setUp() {
 
     DropPendingCollectionReaper::set(
         serviceContext, std::make_unique<DropPendingCollectionReaper>(getStorageInterface()));
-    repl::setOplogCollectionName(serviceContext);
     repl::createOplog(_opCtx.get());
 
     _consistencyMarkers = std::make_unique<ReplicationConsistencyMarkersMock>();
@@ -117,12 +174,12 @@ void OplogApplierImplTest::setUp() {
     // Initialize the featureCompatibilityVersion server parameter. This is necessary because this
     // test fixture does not create a featureCompatibilityVersion document from which to initialize
     // the server parameter.
-    serverGlobalParams.featureCompatibility.setVersion(
-        ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44);
+    // (Generic FCV reference): This FCV reference should exist across LTS binary versions.
+    serverGlobalParams.mutableFeatureCompatibility.setVersion(multiversion::GenericFCV::kLatest);
 
     // This is necessary to generate ghost timestamps for index builds that are not 0, since 0 is an
     // invalid timestamp.
-    ASSERT_OK(LogicalClock::get(_opCtx.get())->advanceClusterTime(LogicalTime(Timestamp(1, 0))));
+    VectorClockMutable::get(_opCtx.get())->tickClusterTimeTo(LogicalTime(Timestamp(1, 0)));
 }
 
 void OplogApplierImplTest::tearDown() {
@@ -159,12 +216,14 @@ void OplogApplierImplTest::_testApplyOplogEntryOrGroupedInsertsCrudOperation(
 
     auto checkOpCtx = [](OperationContext* opCtx) {
         ASSERT_TRUE(opCtx);
-        ASSERT_TRUE(opCtx->lockState()->isDbLockedForMode("test", MODE_IX));
-        ASSERT_FALSE(opCtx->lockState()->isDbLockedForMode("test", MODE_X));
+        ASSERT_TRUE(
+            opCtx->lockState()->isDbLockedForMode(DatabaseName(boost::none, "test"), MODE_IX));
+        ASSERT_FALSE(
+            opCtx->lockState()->isDbLockedForMode(DatabaseName(boost::none, "test"), MODE_X));
         ASSERT_TRUE(
             opCtx->lockState()->isCollectionLockedForMode(NamespaceString("test.t"), MODE_IX));
         ASSERT_FALSE(opCtx->writesAreReplicated());
-        ASSERT_TRUE(documentValidationDisabled(opCtx));
+        ASSERT_TRUE(DocumentValidationSettings::get(opCtx).isSchemaValidationDisabled());
     };
 
     _opObserver->onInsertsFn =
@@ -173,15 +232,18 @@ void OplogApplierImplTest::_testApplyOplogEntryOrGroupedInsertsCrudOperation(
             checkOpCtx(opCtx);
             ASSERT_EQUALS(NamespaceString("test.t"), nss);
             ASSERT_EQUALS(1U, docs.size());
-            ASSERT_BSONOBJ_EQ(op.getObject(), docs[0]);
+            // For upserts we don't know the intended value of the document.
+            if (op.getOpType() == repl::OpTypeEnum::kInsert) {
+                ASSERT_BSONOBJ_EQ(op.getObject(), docs[0]);
+            }
             return Status::OK();
         };
 
     _opObserver->onDeleteFn = [&](OperationContext* opCtx,
                                   const NamespaceString& nss,
-                                  OptionalCollectionUUID uuid,
+                                  const boost::optional<UUID>& uuid,
                                   StmtId stmtId,
-                                  const OpObserver::OplogDeleteEntryArgs& args) {
+                                  const OplogDeleteEntryArgs& args) {
         applyOpCalled = true;
         checkOpCtx(opCtx);
         ASSERT_EQUALS(NamespaceString("test.t"), nss);
@@ -190,6 +252,13 @@ void OplogApplierImplTest::_testApplyOplogEntryOrGroupedInsertsCrudOperation(
         return Status::OK();
     };
 
+    _opObserver->onUpdateFn = [&](OperationContext* opCtx, const OplogUpdateEntryArgs& args) {
+        applyOpCalled = true;
+        checkOpCtx(opCtx);
+        ASSERT_EQUALS(NamespaceString("test.t"), args.nss);
+        return Status::OK();
+    };
+
     ASSERT_EQ(_applyOplogEntryOrGroupedInsertsWrapper(
                   _opCtx.get(), &op, OplogApplication::Mode::kSecondary),
               expectedError);
@@ -252,11 +321,18 @@ Status OplogApplierImplTest::runOpsInitialSync(std::vector<OplogEntry> ops) {
     // its own batch and update oplog visibility after each batch to make sure all previously
     // applied entries are visible to subsequent batches.
     for (auto& op : ops) {
-        auto status = oplogApplier.applyOplogBatch(_opCtx.get(), {op});
-        if (!status.isOK()) {
-            return status.getStatus();
+        auto applyResult = oplogApplier.applyOplogBatch(_opCtx.get(), {op});
+        if (!applyResult.isOK()) {
+            std::vector<BSONObj> docsFromOps;
+            for (const auto& opForContext : ops) {
+                docsFromOps.push_back(opForContext.getEntry().toBSON());
+            }
+            auto status = applyResult.getStatus();
+            return status.withContext(str::stream()
+                                      << "failed to apply operation: " << op.toBSONForLogging()
+                                      << ". " << BSON("ops" << docsFromOps));
         }
-        auto lastApplied = status.getValue();
+        auto lastApplied = applyResult.getValue();
         const bool orderedCommit = true;
         // Update oplog visibility by notifying the storage engine of the new oplog entries.
         storageInterface->oplogDiskLocRegister(
@@ -273,12 +349,11 @@ void checkTxnTable(OperationContext* opCtx,
                    boost::optional<repl::OpTime> expectedStartOpTime,
                    boost::optional<DurableTxnStateEnum> expectedState) {
     DBDirectClient client(opCtx);
-    auto result = client.findOne(NamespaceString::kSessionTransactionsTableNamespace.ns(),
-                                 {BSON(SessionTxnRecord::kSessionIdFieldName << lsid.toBSON())});
+    auto result = client.findOne(NamespaceString::kSessionTransactionsTableNamespace,
+                                 BSON(SessionTxnRecord::kSessionIdFieldName << lsid.toBSON()));
     ASSERT_FALSE(result.isEmpty());
 
-    auto txnRecord =
-        SessionTxnRecord::parse(IDLParserErrorContext("parse txn record for test"), result);
+    auto txnRecord = SessionTxnRecord::parse(IDLParserContext("parse txn record for test"), result);
 
     ASSERT_EQ(txnNum, txnRecord.getTxnNum());
     ASSERT_EQ(expectedOpTime, txnRecord.getLastWriteOpTime());
@@ -297,9 +372,8 @@ void checkTxnTable(OperationContext* opCtx,
 CollectionReader::CollectionReader(OperationContext* opCtx, const NamespaceString& nss)
     : _collToScan(opCtx, nss),
       _exec(InternalPlanner::collectionScan(opCtx,
-                                            nss.ns(),
-                                            _collToScan.getCollection(),
-                                            PlanExecutor::NO_YIELD,
+                                            &_collToScan.getCollection(),
+                                            PlanYieldPolicy::YieldPolicy::NO_YIELD,
                                             InternalPlanner::FORWARD)) {}
 
 StatusWith<BSONObj> CollectionReader::next() {
@@ -318,8 +392,121 @@ StatusWith<BSONObj> CollectionReader::next() {
 
 bool docExists(OperationContext* opCtx, const NamespaceString& nss, const BSONObj& doc) {
     DBDirectClient client(opCtx);
-    auto result = client.findOne(nss.ns(), {doc});
+    auto result = client.findOne(nss, doc);
     return !result.isEmpty();
 }
+
+/**
+ * Creates an OplogEntry with given parameters and preset defaults for this test suite.
+ */
+OplogEntry makeOplogEntry(OpTypeEnum opType,
+                          NamespaceString nss,
+                          const boost::optional<UUID>& uuid,
+                          BSONObj o,
+                          boost::optional<BSONObj> o2,
+                          boost::optional<bool> fromMigrate) {
+    return makeOplogEntry({{1, 1}, 1}, opType, std::move(nss), uuid, o, o2, fromMigrate);
+}
+
+OplogEntry makeOplogEntry(OpTime opTime,
+                          OpTypeEnum opType,
+                          NamespaceString nss,
+                          const boost::optional<UUID>& uuid,
+                          BSONObj o,
+                          boost::optional<BSONObj> o2,
+                          boost::optional<bool> fromMigrate) {
+    return {DurableOplogEntry(opTime,                     // optime
+                              boost::none,                // hash
+                              opType,                     // opType
+                              std::move(nss),             // namespace
+                              uuid,                       // uuid
+                              fromMigrate,                // fromMigrate
+                              OplogEntry::kOplogVersion,  // version
+                              o,                          // o
+                              o2,                         // o2
+                              {},                         // sessionInfo
+                              boost::none,                // upsert
+                              Date_t(),                   // wall clock time
+                              {},                         // statement ids
+                              boost::none,    // optime of previous write within same transaction
+                              boost::none,    // pre-image optime
+                              boost::none,    // post-image optime
+                              boost::none,    // ShardId of resharding recipient
+                              boost::none,    // _id
+                              boost::none)};  // needsRetryImage
+}
+
+OplogEntry makeOplogEntry(OpTypeEnum opType, NamespaceString nss, boost::optional<UUID> uuid) {
+    return makeOplogEntry(opType, nss, uuid, BSON("_id" << 0), boost::none);
+}
+
+CollectionOptions createOplogCollectionOptions() {
+    CollectionOptions options;
+    options.capped = true;
+    options.cappedSize = 64 * 1024 * 1024LL;
+    options.autoIndexId = CollectionOptions::NO;
+    return options;
+}
+
+CollectionOptions createRecordPreImageCollectionOptions() {
+    CollectionOptions options;
+    options.recordPreImages = true;
+    return options;
+}
+
+void createCollection(OperationContext* opCtx,
+                      const NamespaceString& nss,
+                      const CollectionOptions& options) {
+    writeConflictRetry(opCtx, "createCollection", nss.ns(), [&] {
+        Lock::DBLock dbLk(opCtx, nss.dbName(), MODE_IX);
+        Lock::CollectionLock collLk(opCtx, nss, MODE_X);
+
+        OldClientContext ctx(opCtx, nss);
+        auto db = ctx.db();
+        ASSERT_TRUE(db);
+
+        mongo::WriteUnitOfWork wuow(opCtx);
+        auto coll = db->createCollection(opCtx, nss, options);
+        ASSERT_TRUE(coll);
+        wuow.commit();
+    });
+}
+
+UUID createCollectionWithUuid(OperationContext* opCtx, const NamespaceString& nss) {
+    CollectionOptions options;
+    options.uuid = UUID::gen();
+    createCollection(opCtx, nss, options);
+    return options.uuid.value();
+}
+
+void createDatabase(OperationContext* opCtx, StringData dbName) {
+    Lock::GlobalWrite globalLock(opCtx);
+    bool justCreated;
+    auto databaseHolder = DatabaseHolder::get(opCtx);
+    const DatabaseName tenantDbName(boost::none, dbName);
+    auto db = databaseHolder->openDb(opCtx, tenantDbName, &justCreated);
+    ASSERT_TRUE(db);
+    ASSERT_TRUE(justCreated);
+}
+
+bool collectionExists(OperationContext* opCtx, const NamespaceString& nss) {
+    return AutoGetCollectionForRead(opCtx, nss).getCollection() != nullptr;
+}
+
+void createIndex(OperationContext* opCtx,
+                 const NamespaceString& nss,
+                 const UUID collUUID,
+                 const BSONObj& spec) {
+    Lock::DBLock dbLk(opCtx, nss.dbName(), MODE_IX);
+    Lock::CollectionLock collLk(opCtx, nss, MODE_X);
+    auto indexBuildsCoord = IndexBuildsCoordinator::get(opCtx);
+    // This fixture sets up some replication, but notably omits installing an OpObserverImpl. This
+    // state causes collection creation to timestamp catalog writes, but secondary index creation
+    // does not. We use an UnreplicatedWritesBlock to avoid timestamping any of the catalog setup.
+    repl::UnreplicatedWritesBlock noRep(opCtx);
+    indexBuildsCoord->createIndex(
+        opCtx, collUUID, spec, IndexBuildsManager::IndexConstraints::kEnforce, false);
+}
+
 }  // namespace repl
 }  // namespace mongo
