@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
 #include "mongo/platform/basic.h"
 
@@ -35,63 +34,23 @@
 
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/server_feature_flags_gen.h"
 #include "mongo/logv2/redaction.h"
 #include "mongo/util/time_support.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
+
+
 namespace mongo {
 namespace repl {
 
 namespace {
 
-OplogEntry::CommandType parseCommandType(const BSONObj& objectField) {
-    StringData commandString(objectField.firstElementFieldName());
-    if (commandString == "create") {
-        return OplogEntry::CommandType::kCreate;
-    } else if (commandString == "renameCollection") {
-        return OplogEntry::CommandType::kRenameCollection;
-    } else if (commandString == "drop") {
-        return OplogEntry::CommandType::kDrop;
-    } else if (commandString == "collMod") {
-        return OplogEntry::CommandType::kCollMod;
-    } else if (commandString == "applyOps") {
-        return OplogEntry::CommandType::kApplyOps;
-    } else if (commandString == "dbCheck") {
-        return OplogEntry::CommandType::kDbCheck;
-    } else if (commandString == "dropDatabase") {
-        return OplogEntry::CommandType::kDropDatabase;
-    } else if (commandString == "emptycapped") {
-        return OplogEntry::CommandType::kEmptyCapped;
-    } else if (commandString == "convertToCapped") {
-        return OplogEntry::CommandType::kConvertToCapped;
-    } else if (commandString == "createIndexes") {
-        return OplogEntry::CommandType::kCreateIndexes;
-    } else if (commandString == "startIndexBuild") {
-        return OplogEntry::CommandType::kStartIndexBuild;
-    } else if (commandString == "commitIndexBuild") {
-        return OplogEntry::CommandType::kCommitIndexBuild;
-    } else if (commandString == "abortIndexBuild") {
-        return OplogEntry::CommandType::kAbortIndexBuild;
-    } else if (commandString == "dropIndexes") {
-        return OplogEntry::CommandType::kDropIndexes;
-    } else if (commandString == "deleteIndexes") {
-        return OplogEntry::CommandType::kDropIndexes;
-    } else if (commandString == "commitTransaction") {
-        return OplogEntry::CommandType::kCommitTransaction;
-    } else if (commandString == "abortTransaction") {
-        return OplogEntry::CommandType::kAbortTransaction;
-    } else {
-        uasserted(ErrorCodes::BadValue,
-                  str::stream() << "Unknown oplog entry command type: " << commandString
-                                << " Object field: " << redact(objectField));
-    }
-    MONGO_UNREACHABLE;
-}
-
 /**
  * Returns a document representing an oplog entry with the given fields.
  */
 BSONObj makeOplogEntryDoc(OpTime opTime,
-                          const boost::optional<long long> hash,
+                          const boost::optional<int64_t> hash,
                           OpTypeEnum opType,
                           const NamespaceString& nss,
                           const boost::optional<UUID>& uuid,
@@ -102,71 +61,134 @@ BSONObj makeOplogEntryDoc(OpTime opTime,
                           const OperationSessionInfo& sessionInfo,
                           const boost::optional<bool>& isUpsert,
                           const mongo::Date_t& wallClockTime,
-                          const boost::optional<StmtId>& statementId,
+                          const std::vector<StmtId>& statementIds,
                           const boost::optional<OpTime>& prevWriteOpTimeInTransaction,
                           const boost::optional<OpTime>& preImageOpTime,
                           const boost::optional<OpTime>& postImageOpTime,
+                          const boost::optional<ShardId>& destinedRecipient,
+                          const boost::optional<Value>& idField,
                           const boost::optional<repl::RetryImageEnum>& needsRetryImage) {
     BSONObjBuilder builder;
+    if (idField) {
+        idField->addToBsonObj(&builder, OplogEntryBase::k_idFieldName);
+    }
     sessionInfo.serialize(&builder);
     builder.append(OplogEntryBase::kTimestampFieldName, opTime.getTimestamp());
     builder.append(OplogEntryBase::kTermFieldName, opTime.getTerm());
     builder.append(OplogEntryBase::kVersionFieldName, version);
     builder.append(OplogEntryBase::kOpTypeFieldName, OpType_serializer(opType));
+    // TODO SERVER-62114 Change to check for upgraded FCV rather than feature flag
+    if (nss.tenantId() &&
+        gFeatureFlagRequireTenantID.isEnabled(serverGlobalParams.featureCompatibility)) {
+        nss.tenantId()->serializeToBSON(OplogEntryBase::kTidFieldName, &builder);
+    }
     builder.append(OplogEntryBase::kNssFieldName, nss.toString());
     builder.append(OplogEntryBase::kWallClockTimeFieldName, wallClockTime);
     if (hash) {
-        builder.append(OplogEntryBase::kHashFieldName, hash.get());
+        builder.append(OplogEntryBase::kHashFieldName, hash.value());
     }
     if (uuid) {
         uuid->appendToBuilder(&builder, OplogEntryBase::kUuidFieldName);
     }
     if (fromMigrate) {
-        builder.append(OplogEntryBase::kFromMigrateFieldName, fromMigrate.get());
+        builder.append(OplogEntryBase::kFromMigrateFieldName, fromMigrate.value());
     }
     builder.append(OplogEntryBase::kObjectFieldName, oField);
     if (o2Field) {
-        builder.append(OplogEntryBase::kObject2FieldName, o2Field.get());
+        builder.append(OplogEntryBase::kObject2FieldName, o2Field.value());
     }
     if (isUpsert) {
         invariant(o2Field);
-        builder.append(OplogEntryBase::kUpsertFieldName, isUpsert.get());
+        builder.append(OplogEntryBase::kUpsertFieldName, isUpsert.value());
     }
-    if (statementId) {
-        builder.append(OplogEntryBase::kStatementIdFieldName, statementId.get());
+    if (statementIds.size() == 1) {
+        builder.append(OplogEntryBase::kStatementIdsFieldName, statementIds.front());
+    } else if (!statementIds.empty()) {
+        builder.append(OplogEntryBase::kStatementIdsFieldName, statementIds);
     }
     if (prevWriteOpTimeInTransaction) {
-        const BSONObj localObject = prevWriteOpTimeInTransaction.get().toBSON();
+        const BSONObj localObject = prevWriteOpTimeInTransaction.value().toBSON();
         builder.append(OplogEntryBase::kPrevWriteOpTimeInTransactionFieldName, localObject);
     }
     if (preImageOpTime) {
-        const BSONObj localObject = preImageOpTime.get().toBSON();
+        const BSONObj localObject = preImageOpTime.value().toBSON();
         builder.append(OplogEntryBase::kPreImageOpTimeFieldName, localObject);
     }
     if (postImageOpTime) {
-        const BSONObj localObject = postImageOpTime.get().toBSON();
+        const BSONObj localObject = postImageOpTime.value().toBSON();
         builder.append(OplogEntryBase::kPostImageOpTimeFieldName, localObject);
     }
+
+    if (destinedRecipient) {
+        builder.append(OplogEntryBase::kDestinedRecipientFieldName,
+                       destinedRecipient.value().toString());
+    }
+
     if (needsRetryImage) {
         builder.append(OplogEntryBase::kNeedsRetryImageFieldName,
-                       RetryImage_serializer(needsRetryImage.get()));
+                       RetryImage_serializer(needsRetryImage.value()));
     }
     return builder.obj();
 }
-
 }  // namespace
 
-const int MutableOplogEntry::kOplogVersion = 2;
+DurableOplogEntry::CommandType parseCommandType(const BSONObj& objectField) {
+    StringData commandString(objectField.firstElementFieldName());
+    if (commandString == "create") {
+        return DurableOplogEntry::CommandType::kCreate;
+    } else if (commandString == "renameCollection") {
+        return DurableOplogEntry::CommandType::kRenameCollection;
+    } else if (commandString == "drop") {
+        return DurableOplogEntry::CommandType::kDrop;
+    } else if (commandString == "collMod") {
+        return DurableOplogEntry::CommandType::kCollMod;
+    } else if (commandString == "applyOps") {
+        return DurableOplogEntry::CommandType::kApplyOps;
+    } else if (commandString == "dbCheck") {
+        return DurableOplogEntry::CommandType::kDbCheck;
+    } else if (commandString == "dropDatabase") {
+        return DurableOplogEntry::CommandType::kDropDatabase;
+    } else if (commandString == "emptycapped") {
+        return DurableOplogEntry::CommandType::kEmptyCapped;
+    } else if (commandString == "createIndexes") {
+        return DurableOplogEntry::CommandType::kCreateIndexes;
+    } else if (commandString == "startIndexBuild") {
+        return DurableOplogEntry::CommandType::kStartIndexBuild;
+    } else if (commandString == "commitIndexBuild") {
+        return DurableOplogEntry::CommandType::kCommitIndexBuild;
+    } else if (commandString == "abortIndexBuild") {
+        return DurableOplogEntry::CommandType::kAbortIndexBuild;
+    } else if (commandString == "dropIndexes") {
+        return DurableOplogEntry::CommandType::kDropIndexes;
+    } else if (commandString == "deleteIndexes") {
+        return DurableOplogEntry::CommandType::kDropIndexes;
+    } else if (commandString == "commitTransaction") {
+        return DurableOplogEntry::CommandType::kCommitTransaction;
+    } else if (commandString == "abortTransaction") {
+        return DurableOplogEntry::CommandType::kAbortTransaction;
+    } else if (commandString == "importCollection") {
+        return DurableOplogEntry::CommandType::kImportCollection;
+    } else {
+        uasserted(ErrorCodes::BadValue,
+                  str::stream() << "Unknown oplog entry command type: " << commandString
+                                << " Object field: " << redact(objectField));
+    }
+    MONGO_UNREACHABLE;
+}
 
 // Static
 ReplOperation MutableOplogEntry::makeInsertOperation(const NamespaceString& nss,
                                                      UUID uuid,
-                                                     const BSONObj& docToInsert) {
+                                                     const BSONObj& docToInsert,
+                                                     const BSONObj& docKey) {
     ReplOperation op;
     op.setOpType(OpTypeEnum::kInsert);
+
+    op.setTid(nss.tenantId());
     op.setNss(nss);
     op.setUuid(uuid);
     op.setObject(docToInsert.getOwned());
+    op.setObject2(docKey.getOwned());
     return op;
 }
 
@@ -201,6 +223,8 @@ ReplOperation MutableOplogEntry::makeUpdateOperation(const NamespaceString nss,
                                                      const BSONObj& criteria) {
     ReplOperation op;
     op.setOpType(OpTypeEnum::kUpdate);
+
+    op.setTid(nss.tenantId());
     op.setNss(nss);
     op.setUuid(uuid);
     op.setObject(update.getOwned());
@@ -214,6 +238,8 @@ ReplOperation MutableOplogEntry::makeCreateCommand(const NamespaceString nss,
 
     ReplOperation op;
     op.setOpType(OpTypeEnum::kCommand);
+
+    op.setTid(nss.tenantId());
     op.setNss(nss.getCommandNS());
     op.setUuid(options.uuid);
     op.setObject(makeCreateCollCmdObj(nss, options, idIndex));
@@ -221,10 +247,12 @@ ReplOperation MutableOplogEntry::makeCreateCommand(const NamespaceString nss,
 }
 
 ReplOperation MutableOplogEntry::makeCreateIndexesCommand(const NamespaceString nss,
-                                                          CollectionUUID uuid,
+                                                          const UUID& uuid,
                                                           const BSONObj& indexDoc) {
     ReplOperation op;
     op.setOpType(OpTypeEnum::kCommand);
+
+    op.setTid(nss.tenantId());
     op.setNss(nss.getCommandNS());
     op.setUuid(uuid);
 
@@ -242,6 +270,8 @@ ReplOperation MutableOplogEntry::makeDeleteOperation(const NamespaceString& nss,
                                                      const BSONObj& docToDelete) {
     ReplOperation op;
     op.setOpType(OpTypeEnum::kDelete);
+
+    op.setTid(nss.tenantId());
     op.setNss(nss);
     op.setUuid(uuid);
     op.setObject(docToDelete.getOwned());
@@ -249,9 +279,14 @@ ReplOperation MutableOplogEntry::makeDeleteOperation(const NamespaceString& nss,
 }
 
 StatusWith<MutableOplogEntry> MutableOplogEntry::parse(const BSONObj& object) {
+    boost::optional<TenantId> tid;
+    if (object.hasElement("tid"))
+        tid = TenantId::parseFromBSON(object["tid"]);
+
     try {
         MutableOplogEntry oplogEntry;
-        oplogEntry.parseProtected(IDLParserErrorContext("OplogEntryBase"), object);
+        oplogEntry.parseProtected(IDLParserContext("OplogEntryBase", false /* apiStrict */, tid),
+                                  object);
         return oplogEntry;
     } catch (...) {
         return exceptionToStatus();
@@ -268,29 +303,35 @@ void MutableOplogEntry::setOpTime(const OpTime& opTime) & {
 OpTime MutableOplogEntry::getOpTime() const {
     long long term = OpTime::kUninitializedTerm;
     if (getTerm()) {
-        term = getTerm().get();
+        term = getTerm().value();
     }
     return OpTime(getTimestamp(), term);
 }
 
-size_t OplogEntry::getDurableReplOperationSize(const DurableReplOperation& op) {
-    return sizeof(op) + op.getNss().size() + op.getObject().objsize() +
-        (op.getObject2() ? op.getObject2()->objsize() : 0);
+size_t DurableOplogEntry::getDurableReplOperationSize(const DurableReplOperation& op) {
+    const auto stmtIds = variant_util::toVector<StmtId>(op.getStatementIds());
+    return sizeof(op) + (op.getTid() ? op.getTid()->toString().size() : 0) + op.getNss().size() +
+        op.getObject().objsize() + (op.getObject2() ? op.getObject2()->objsize() : 0) +
+        (sizeof(std::vector<StmtId>) + (sizeof(StmtId) * stmtIds.size()));
 }
 
-StatusWith<OplogEntry> OplogEntry::parse(const BSONObj& object) {
+StatusWith<DurableOplogEntry> DurableOplogEntry::parse(const BSONObj& object) {
     try {
-        return OplogEntry(object);
+        return DurableOplogEntry(object);
     } catch (...) {
         return exceptionToStatus();
     }
     MONGO_UNREACHABLE;
 }
 
-OplogEntry::OplogEntry(BSONObj rawInput) : _raw(std::move(rawInput)) {
+DurableOplogEntry::DurableOplogEntry(BSONObj rawInput) : _raw(std::move(rawInput)) {
     _raw = _raw.getOwned();
 
-    parseProtected(IDLParserErrorContext("OplogEntryBase"), _raw);
+    boost::optional<TenantId> tid;
+    if (_raw.hasElement("tid"))
+        tid = TenantId::parseFromBSON(_raw["tid"]);
+
+    parseProtected(IDLParserContext("OplogEntryBase", false /* apiStrict */, tid), _raw);
 
     // Parse command type from 'o' and 'o2' fields.
     if (isCommand()) {
@@ -298,47 +339,51 @@ OplogEntry::OplogEntry(BSONObj rawInput) : _raw(std::move(rawInput)) {
     }
 }
 
-OplogEntry::OplogEntry(OpTime opTime,
-                       const boost::optional<long long> hash,
-                       OpTypeEnum opType,
-                       const NamespaceString& nss,
-                       const boost::optional<UUID>& uuid,
-                       const boost::optional<bool>& fromMigrate,
-                       int version,
-                       const BSONObj& oField,
-                       const boost::optional<BSONObj>& o2Field,
-                       const OperationSessionInfo& sessionInfo,
-                       const boost::optional<bool>& isUpsert,
-                       const mongo::Date_t& wallClockTime,
-                       const boost::optional<StmtId>& statementId,
-                       const boost::optional<OpTime>& prevWriteOpTimeInTransaction,
-                       const boost::optional<OpTime>& preImageOpTime,
-                       const boost::optional<OpTime>& postImageOpTime,
-                       const boost::optional<repl::RetryImageEnum>& needsRetryImage)
-    : OplogEntry(makeOplogEntryDoc(opTime,
-                                   hash,
-                                   opType,
-                                   nss,
-                                   uuid,
-                                   fromMigrate,
-                                   version,
-                                   oField,
-                                   o2Field,
-                                   sessionInfo,
-                                   isUpsert,
-                                   wallClockTime,
-                                   statementId,
-                                   prevWriteOpTimeInTransaction,
-                                   preImageOpTime,
-                                   postImageOpTime,
-                                   needsRetryImage)) {}
+DurableOplogEntry::DurableOplogEntry(OpTime opTime,
+                                     const boost::optional<int64_t> hash,
+                                     OpTypeEnum opType,
+                                     const NamespaceString& nss,
+                                     const boost::optional<UUID>& uuid,
+                                     const boost::optional<bool>& fromMigrate,
+                                     int version,
+                                     const BSONObj& oField,
+                                     const boost::optional<BSONObj>& o2Field,
+                                     const OperationSessionInfo& sessionInfo,
+                                     const boost::optional<bool>& isUpsert,
+                                     const mongo::Date_t& wallClockTime,
+                                     const std::vector<StmtId>& statementIds,
+                                     const boost::optional<OpTime>& prevWriteOpTimeInTransaction,
+                                     const boost::optional<OpTime>& preImageOpTime,
+                                     const boost::optional<OpTime>& postImageOpTime,
+                                     const boost::optional<ShardId>& destinedRecipient,
+                                     const boost::optional<Value>& idField,
+                                     const boost::optional<repl::RetryImageEnum>& needsRetryImage)
+    : DurableOplogEntry(makeOplogEntryDoc(opTime,
+                                          hash,
+                                          opType,
+                                          nss,
+                                          uuid,
+                                          fromMigrate,
+                                          version,
+                                          oField,
+                                          o2Field,
+                                          sessionInfo,
+                                          isUpsert,
+                                          wallClockTime,
+                                          statementIds,
+                                          prevWriteOpTimeInTransaction,
+                                          preImageOpTime,
+                                          postImageOpTime,
+                                          destinedRecipient,
+                                          idField,
+                                          needsRetryImage)) {}
 
-bool OplogEntry::isCommand() const {
+bool DurableOplogEntry::isCommand() const {
     return getOpType() == OpTypeEnum::kCommand;
 }
 
 // static
-bool OplogEntry::isCrudOpType(OpTypeEnum opType) {
+bool DurableOplogEntry::isCrudOpType(OpTypeEnum opType) {
     switch (opType) {
         case OpTypeEnum::kInsert:
         case OpTypeEnum::kDelete:
@@ -351,11 +396,11 @@ bool OplogEntry::isCrudOpType(OpTypeEnum opType) {
     MONGO_UNREACHABLE;
 }
 
-bool OplogEntry::isCrudOpType() const {
+bool DurableOplogEntry::isCrudOpType() const {
     return isCrudOpType(getOpType());
 }
 
-bool OplogEntry::isUpdateOrDelete() const {
+bool DurableOplogEntry::isUpdateOrDelete() const {
     auto opType = getOpType();
     switch (opType) {
         case OpTypeEnum::kDelete:
@@ -369,12 +414,12 @@ bool OplogEntry::isUpdateOrDelete() const {
     MONGO_UNREACHABLE;
 }
 
-bool OplogEntry::shouldPrepare() const {
+bool DurableOplogEntry::shouldPrepare() const {
     return getCommandType() == CommandType::kApplyOps &&
         getObject()[ApplyOpsCommandInfoBase::kPrepareFieldName].booleanSafe();
 }
 
-bool OplogEntry::isSingleOplogEntryTransaction() const {
+bool DurableOplogEntry::isSingleOplogEntryTransaction() const {
     if (getCommandType() != CommandType::kApplyOps || !getTxnNumber() || !getSessionId() ||
         getObject()[ApplyOpsCommandInfoBase::kPartialTxnFieldName].booleanSafe()) {
         return false;
@@ -387,7 +432,7 @@ bool OplogEntry::isSingleOplogEntryTransaction() const {
     return prevOptimeOpt->isNull();
 }
 
-bool OplogEntry::isEndOfLargeTransaction() const {
+bool DurableOplogEntry::isEndOfLargeTransaction() const {
     if (getCommandType() != CommandType::kApplyOps) {
         // If the oplog entry is neither commit nor abort, then it must be an applyOps. Otherwise,
         // it cannot be a termainal oplog entry of a large transaction.
@@ -405,7 +450,7 @@ bool OplogEntry::isEndOfLargeTransaction() const {
     return !prevOptimeOpt->isNull() && !isPartialTransaction();
 }
 
-bool OplogEntry::isSingleOplogEntryTransactionWithCommand() const {
+bool DurableOplogEntry::isSingleOplogEntryTransactionWithCommand() const {
     if (!isSingleOplogEntryTransaction()) {
         return false;
     }
@@ -424,7 +469,16 @@ bool OplogEntry::isSingleOplogEntryTransactionWithCommand() const {
     return false;
 }
 
-BSONElement OplogEntry::getIdElement() const {
+bool DurableOplogEntry::isIndexCommandType() const {
+    return getOpType() == OpTypeEnum::kCommand &&
+        ((getCommandType() == CommandType::kCreateIndexes) ||
+         (getCommandType() == CommandType::kStartIndexBuild) ||
+         (getCommandType() == CommandType::kCommitIndexBuild) ||
+         (getCommandType() == CommandType::kAbortIndexBuild) ||
+         (getCommandType() == CommandType::kDropIndexes));
+}
+
+BSONElement DurableOplogEntry::getIdElement() const {
     invariant(isCrudOpType());
     if (getOpType() == OpTypeEnum::kUpdate) {
         // We cannot use getObjectContainingDocumentKey() here because the BSONObj will go out
@@ -436,11 +490,11 @@ BSONElement OplogEntry::getIdElement() const {
     }
 }
 
-BSONObj OplogEntry::getOperationToApply() const {
+BSONObj DurableOplogEntry::getOperationToApply() const {
     return getObject();
 }
 
-BSONObj OplogEntry::getObjectContainingDocumentKey() const {
+BSONObj DurableOplogEntry::getObjectContainingDocumentKey() const {
     invariant(isCrudOpType());
     if (getOpType() == OpTypeEnum::kUpdate) {
         fassert(31081, getObject2() != boost::none);
@@ -450,25 +504,275 @@ BSONObj OplogEntry::getObjectContainingDocumentKey() const {
     }
 }
 
-OplogEntry::CommandType OplogEntry::getCommandType() const {
+DurableOplogEntry::CommandType DurableOplogEntry::getCommandType() const {
     return _commandType;
 }
 
-int OplogEntry::getRawObjSizeBytes() const {
+int DurableOplogEntry::getRawObjSizeBytes() const {
     return _raw.objsize();
 }
 
-std::string OplogEntry::toString() const {
+std::string DurableOplogEntry::toString() const {
     return _raw.toString();
 }
 
-std::ostream& operator<<(std::ostream& s, const OplogEntry& o) {
+std::ostream& operator<<(std::ostream& s, const DurableOplogEntry& o) {
     return s << o.toString();
 }
 
+std::ostream& operator<<(std::ostream& s, const OplogEntry& o) {
+    return s << o.toStringForLogging();
+}
+
 std::ostream& operator<<(std::ostream& s, const ReplOperation& o) {
     return s << o.toBSON().toString();
 }
 
+OplogEntry::OplogEntry(DurableOplogEntry entry) : _entry(std::move(entry)) {}
+
+OplogEntry::OplogEntry(const BSONObj& entry)
+    : OplogEntry(uassertStatusOK(DurableOplogEntry::parse(entry))) {}
+void OplogEntry::setEntry(DurableOplogEntry entry) {
+    _entry = std::move(entry);
+}
+
+bool operator==(const OplogEntry& lhs, const OplogEntry& rhs) {
+    if (lhs.isForCappedCollection() != rhs.isForCappedCollection()) {
+        return false;
+    }
+
+    return lhs.getEntry() == rhs.getEntry();
+}
+
+StatusWith<OplogEntry> OplogEntry::parse(const BSONObj& object) {
+    auto parseStatus = DurableOplogEntry::parse(object);
+
+    if (!parseStatus.isOK()) {
+        return parseStatus;
+    }
+
+    return OplogEntry(std::move(parseStatus.getValue()));
+}
+std::string OplogEntry::toStringForLogging() const {
+    return toBSONForLogging().toString();
+}
+BSONObj OplogEntry::toBSONForLogging() const {
+    BSONObjBuilder builder;
+    auto entry = _entry.toBSON();
+
+    builder.append("oplogEntry", entry);
+
+    if (_isForCappedCollection) {
+        builder.append("isForCappedCollection", _isForCappedCollection);
+    }
+
+    return builder.obj();
+}
+
+bool OplogEntry::isForCappedCollection() const {
+    return _isForCappedCollection;
+}
+
+void OplogEntry::setIsForCappedCollection(bool isForCappedCollection) {
+    _isForCappedCollection = isForCappedCollection;
+}
+
+const boost::optional<mongo::Value>& OplogEntry::get_id() const& {
+    return _entry.get_id();
+}
+
+std::vector<StmtId> OplogEntry::getStatementIds() const& {
+    return _entry.getStatementIds();
+}
+
+const OperationSessionInfo& OplogEntry::getOperationSessionInfo() const {
+    return _entry.getOperationSessionInfo();
+}
+const boost::optional<mongo::LogicalSessionId>& OplogEntry::getSessionId() const {
+    return _entry.getSessionId();
+}
+
+boost::optional<std::int64_t> OplogEntry::getTxnNumber() const {
+    return _entry.getTxnNumber();
+}
+
+const DurableReplOperation& OplogEntry::getDurableReplOperation() const {
+    return _entry.getDurableReplOperation();
+}
+
+mongo::repl::OpTypeEnum OplogEntry::getOpType() const {
+    return _entry.getOpType();
+}
+
+const boost::optional<mongo::TenantId>& OplogEntry::getTid() const {
+    return _entry.getTid();
+}
+
+const mongo::NamespaceString& OplogEntry::getNss() const {
+    return _entry.getNss();
+}
+
+const boost::optional<mongo::UUID>& OplogEntry::getUuid() const {
+    return _entry.getUuid();
+}
+
+const mongo::BSONObj& OplogEntry::getObject() const {
+    return _entry.getObject();
+}
+
+const boost::optional<mongo::BSONObj>& OplogEntry::getObject2() const {
+    return _entry.getObject2();
+}
+
+boost::optional<bool> OplogEntry::getUpsert() const {
+    return _entry.getUpsert();
+}
+
+const boost::optional<mongo::repl::OpTime>& OplogEntry::getPreImageOpTime() const {
+    return _entry.getPreImageOpTime();
+}
+
+const boost::optional<mongo::ShardId>& OplogEntry::getDestinedRecipient() const {
+    return _entry.getDestinedRecipient();
+}
+
+const mongo::Timestamp& OplogEntry::getTimestamp() const {
+    return _entry.getTimestamp();
+}
+
+boost::optional<std::int64_t> OplogEntry::getTerm() const {
+    return _entry.getTerm();
+}
+
+const mongo::Date_t& OplogEntry::getWallClockTime() const {
+    return _entry.getWallClockTime();
+}
+
+boost::optional<std::int64_t> OplogEntry::getHash() const& {
+    return _entry.getHash();
+}
+
+std::int64_t OplogEntry::getVersion() const {
+    return _entry.getVersion();
+}
+
+boost::optional<bool> OplogEntry::getFromMigrate() const& {
+    return _entry.getFromMigrate();
+}
+
+const boost::optional<mongo::UUID>& OplogEntry::getFromTenantMigration() const& {
+    return _entry.getFromTenantMigration();
+}
+
+const boost::optional<mongo::repl::OpTime>& OplogEntry::getPrevWriteOpTimeInTransaction() const& {
+    return _entry.getPrevWriteOpTimeInTransaction();
+}
+
+const boost::optional<mongo::repl::OpTime>& OplogEntry::getPostImageOpTime() const& {
+    return _entry.getPostImageOpTime();
+}
+
+boost::optional<RetryImageEnum> OplogEntry::getNeedsRetryImage() const {
+    return _entry.getNeedsRetryImage();
+}
+
+OpTime OplogEntry::getOpTime() const {
+    return _entry.getOpTime();
+}
+
+bool OplogEntry::isCommand() const {
+    return _entry.isCommand();
+}
+
+bool OplogEntry::isPartialTransaction() const {
+    return _entry.isPartialTransaction();
+}
+
+bool OplogEntry::isEndOfLargeTransaction() const {
+    return _entry.isEndOfLargeTransaction();
+}
+
+bool OplogEntry::isPreparedCommit() const {
+    return _entry.isPreparedCommit();
+}
+
+bool OplogEntry::isTerminalApplyOps() const {
+    return _entry.isTerminalApplyOps();
+}
+
+bool OplogEntry::isSingleOplogEntryTransaction() const {
+    return _entry.isSingleOplogEntryTransaction();
+}
+
+bool OplogEntry::isSingleOplogEntryTransactionWithCommand() const {
+    return _entry.isSingleOplogEntryTransactionWithCommand();
+}
+
+uint64_t OplogEntry::getApplyOpsIndex() const {
+    return _applyOpsIndex;
+}
+
+void OplogEntry::setApplyOpsIndex(uint64_t value) {
+    _applyOpsIndex = value;
+}
+
+const boost::optional<mongo::Timestamp>& OplogEntry::getApplyOpsTimestamp() const {
+    return _applyOpsTimestamp;
+}
+
+void OplogEntry::setApplyOpsTimestamp(boost::optional<mongo::Timestamp> value) {
+    _applyOpsTimestamp = value;
+}
+
+const boost::optional<mongo::Date_t>& OplogEntry::getApplyOpsWallClockTime() const {
+    return _applyOpsWallClockTime;
+}
+void OplogEntry::setApplyOpsWallClockTime(boost::optional<mongo::Date_t> value) {
+    _applyOpsWallClockTime = value;
+}
+
+mongo::Timestamp OplogEntry::getTimestampForPreImage() const {
+    return getApplyOpsTimestamp().get_value_or(getTimestamp());
+}
+
+mongo::Date_t OplogEntry::getWallClockTimeForPreImage() const {
+    return getApplyOpsWallClockTime().get_value_or(getWallClockTime());
+}
+
+bool OplogEntry::isCrudOpType() const {
+    return _entry.isCrudOpType();
+}
+bool OplogEntry::isUpdateOrDelete() const {
+    return _entry.isUpdateOrDelete();
+}
+
+bool OplogEntry::isIndexCommandType() const {
+    return _entry.isIndexCommandType();
+}
+
+bool OplogEntry::shouldPrepare() const {
+    return _entry.shouldPrepare();
+}
+
+BSONElement OplogEntry::getIdElement() const {
+    return _entry.getIdElement();
+}
+
+BSONObj OplogEntry::getOperationToApply() const {
+    return _entry.getOperationToApply();
+}
+
+BSONObj OplogEntry::getObjectContainingDocumentKey() const {
+    return _entry.getObjectContainingDocumentKey();
+}
+
+OplogEntry::CommandType OplogEntry::getCommandType() const {
+    return _entry.getCommandType();
+}
+
+int OplogEntry::getRawObjSizeBytes() const {
+    return _entry.getRawObjSizeBytes();
+}
+
 }  // namespace repl
 }  // namespace mongo
