@@ -27,14 +27,11 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #define LOGV2_FOR_HEARTBEATS(ID, DLEVEL, MESSAGE, ...) \
     LOGV2_DEBUG_OPTIONS(                               \
         ID, DLEVEL, {logv2::LogComponent::kReplicationHeartbeats}, MESSAGE, ##__VA_ARGS__)
 
-#include "mongo/platform/basic.h"
-
 #include <boost/algorithm/string.hpp>
 
 #include "mongo/db/repl/repl_set_command.h"
@@ -47,13 +44,10 @@
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/server_status_metric.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/dbhelpers.h"
-#include "mongo/db/lasterror.h"
-#include "mongo/db/op_observer.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/repl/drop_pending_collection_reaper.h"
 #include "mongo/db/repl/oplog.h"
-#include "mongo/db/repl/repl_server_parameters_gen.h"
 #include "mongo/db/repl/repl_set_heartbeat_args_v1.h"
 #include "mongo/db/repl/repl_set_heartbeat_response.h"
 #include "mongo/db/repl/replication_coordinator.h"
@@ -72,13 +66,16 @@
 #include "mongo/util/net/socket_utils.h"
 #include "mongo/util/scopeguard.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
+
+
 namespace mongo {
 namespace repl {
 
 using std::string;
 using std::stringstream;
 
-static const std::string kReplSetReconfigNss = "local.replset.reconfig";
+constexpr StringData kInternalIncludeNewlyAddedFieldName = "$_internalIncludeNewlyAdded"_sd;
 
 class ReplExecutorSSM : public ServerStatusMetric {
 public:
@@ -86,7 +83,9 @@ public:
     virtual void appendAtLeaf(BSONObjBuilder& b) const {
         ReplicationCoordinator::get(getGlobalServiceContext())->appendDiagnosticBSON(&b);
     }
-} replExecutorSSM;
+};
+
+auto& replExecutorSSM = addMetricToTree(std::make_unique<ReplExecutorSSM>());
 
 // Testing only, enabled via command-line.
 class CmdReplSetTest : public ReplSetCommand {
@@ -102,13 +101,13 @@ public:
     }
     CmdReplSetTest() : ReplSetCommand("replSetTest") {}
     virtual bool run(OperationContext* opCtx,
-                     const string&,
+                     const DatabaseName&,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         LOGV2(21573,
               "replSetTest command received: {cmdObj}",
               "replSetTest command received",
-              "cmdObj"_attr = cmdObj.toString());
+              "cmdObj"_attr = cmdObj);
 
         auto replCoord = ReplicationCoordinator::get(getGlobalServiceContext());
 
@@ -131,18 +130,44 @@ public:
                   "timeout"_attr = timeout,
                   "expectedState"_attr = expectedState);
 
-            status = replCoord->waitForMemberState(expectedState, timeout);
+            status = replCoord->waitForMemberState(opCtx, expectedState, timeout);
 
             uassertStatusOK(status);
             return true;
         } else if (cmdObj.hasElement("getLastStableRecoveryTimestamp")) {
-            boost::optional<Timestamp> ts =
-                StorageInterface::get(getGlobalServiceContext())
-                    ->getLastStableRecoveryTimestamp(getGlobalServiceContext());
-            if (ts) {
-                result.append("lastStableRecoveryTimestamp", ts.get());
+            try {
+                // Retrieving last stable recovery timestamp should not be blocked by oplog
+                // application.
+                ShouldNotConflictWithSecondaryBatchApplicationBlock shouldNotConflictBlock(
+                    opCtx->lockState());
+                opCtx->lockState()->skipAcquireTicket();
+                // We need to hold the lock so that we don't run when storage is being shutdown.
+                Lock::GlobalLock lk(opCtx,
+                                    MODE_IS,
+                                    Date_t::now() + Milliseconds(5),
+                                    Lock::InterruptBehavior::kLeaveUnlocked,
+                                    true /* skipRSTLLock */);
+                if (lk.isLocked()) {
+                    boost::optional<Timestamp> ts =
+                        StorageInterface::get(getGlobalServiceContext())
+                            ->getLastStableRecoveryTimestamp(getGlobalServiceContext());
+                    if (ts) {
+                        result.append("lastStableRecoveryTimestamp", ts.value());
+                    }
+                } else {
+                    LOGV2_WARNING(6100700,
+                                  "Failed to get last stable recovery timestamp due to {error}",
+                                  "error"_attr = "lock acquire timeout"_sd);
+                }
+            } catch (const ExceptionForCat<ErrorCategory::CancellationError>& ex) {
+                LOGV2_WARNING(6100701,
+                              "Failed to get last stable recovery timestamp due to {error}",
+                              "error"_attr = redact(ex));
             }
             return true;
+        } else if (cmdObj.hasElement("restartHeartbeats")) {
+            replCoord->restartScheduledHeartbeats_forTest();
+            return true;
         }
 
         Status status = replCoord->checkReplEnabledForCommand(&result);
@@ -161,7 +186,7 @@ class CmdReplSetGetRBID : public ReplSetCommand {
 public:
     CmdReplSetGetRBID() : ReplSetCommand("replSetGetRBID") {}
     virtual bool run(OperationContext* opCtx,
-                     const string&,
+                     const DatabaseName&,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         Status status = ReplicationCoordinator::get(opCtx)->checkReplEnabledForCommand(&result);
@@ -181,7 +206,7 @@ public:
     }
     CmdReplSetGetConfig() : ReplSetCommand("replSetGetConfig") {}
     virtual bool run(OperationContext* opCtx,
-                     const string&,
+                     const DatabaseName&,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         Status status = ReplicationCoordinator::get(opCtx)->checkReplEnabledForCommand(&result);
@@ -190,7 +215,21 @@ public:
         bool wantCommitmentStatus;
         uassertStatusOK(bsonExtractBooleanFieldWithDefault(
             cmdObj, "commitmentStatus", false, &wantCommitmentStatus));
-        ReplicationCoordinator::get(opCtx)->processReplSetGetConfig(&result, wantCommitmentStatus);
+
+        if (cmdObj[kInternalIncludeNewlyAddedFieldName]) {
+            uassert(ErrorCodes::InvalidOptions,
+                    "The '$_internalIncludeNewlyAdded' option is only supported when testing"
+                    " commands are enabled",
+                    getTestCommandsEnabled());
+        }
+
+        bool includeNewlyAdded;
+        uassertStatusOK(bsonExtractBooleanFieldWithDefault(
+            cmdObj, kInternalIncludeNewlyAddedFieldName, false, &includeNewlyAdded));
+
+        ReplicationCoordinator::get(opCtx)->processReplSetGetConfig(
+            &result, wantCommitmentStatus, includeNewlyAdded);
+
         return true;
     }
 
@@ -299,7 +338,7 @@ public:
                "http://dochub.mongodb.org/core/replicasetcommands";
     }
     virtual bool run(OperationContext* opCtx,
-                     const string&,
+                     const DatabaseName&,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         BSONObj configObj;
@@ -307,14 +346,20 @@ public:
             configObj = cmdObj["replSetInitiate"].Obj();
         }
 
-        std::string replSetString =
-            ReplicationCoordinator::get(opCtx)->getSettings().getReplSetString();
-        if (replSetString.empty()) {
+        const auto& settings = ReplicationCoordinator::get(opCtx)->getSettings();
+        if (!settings.usingReplSets()) {
             uasserted(ErrorCodes::NoReplicationEnabled,
-                      "This node was not started with the replSet option");
+                      "This node was not started with replication enabled.");
+        }
+
+        if (settings.isServerless() && configObj.isEmpty()) {
+            uasserted(ErrorCodes::InvalidReplicaSetConfig,
+                      "A config must be provided when started in serverless mode.");
         }
 
         if (configObj.isEmpty()) {
+            std::string replSetString = settings.getReplSetString();
+
             string noConfigMessage =
                 "no configuration specified. "
                 "Using a default configuration for the set";
@@ -355,7 +400,7 @@ public:
             LOGV2(21578,
                   "created this configuration for initiation : {config}",
                   "Created configuration for initiation",
-                  "config"_attr = configObj.toString());
+                  "config"_attr = configObj);
         }
 
         if (configObj.getField("version").eoo()) {
@@ -388,7 +433,7 @@ public:
     }
 
     bool run(OperationContext* opCtx,
-             const string&,
+             const DatabaseName&,
              const BSONObj& cmdObj,
              BSONObjBuilder& result) override {
         const auto replCoord = ReplicationCoordinator::get(opCtx);
@@ -406,9 +451,9 @@ public:
         // For safe reconfig, wait for the current config to be committed before running a new one.
         // We will check again after acquiring the repl mutex in processReplSetReconfig(), in case
         // of concurrent reconfigs.
-        if (!parsedArgs.force && enableSafeReplicaSetReconfig) {
+        if (!parsedArgs.force) {
             // Skip the waiting if the current config is from a force reconfig.
-            auto oplogWait = replCoord->getConfig().getConfigTerm() != OpTime::kUninitializedTerm;
+            auto oplogWait = replCoord->getConfigTerm() != OpTime::kUninitializedTerm;
             auto status = replCoord->awaitConfigCommitment(opCtx, oplogWait);
             status.addContext("New config is rejected");
             if (status == ErrorCodes::MaxTimeMSExpired) {
@@ -425,13 +470,8 @@ public:
         uassertStatusOK(status);
 
         // Now that the new config has been persisted and installed in memory, wait for the new
-        // config to become committed. For force reconfigs we don't need to do this waiting, and in
-        // any FCV < 4.4 we also bypass this to preserve client facing behavior in mixed version
-        // sets.
-        if (!parsedArgs.force &&
-            serverGlobalParams.featureCompatibility.isVersion(
-                ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44) &&
-            enableSafeReplicaSetReconfig) {
+        // config to become replicated. For force reconfigs we don't need to do this waiting.
+        if (!parsedArgs.force) {
             auto status =
                 replCoord->awaitConfigCommitment(opCtx, false /* waitForOplogCommitment */);
             uassertStatusOK(
@@ -461,13 +501,13 @@ public:
     }
     CmdReplSetFreeze() : ReplSetCommand("replSetFreeze") {}
     virtual bool run(OperationContext* opCtx,
-                     const string&,
+                     const DatabaseName&,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         Status status = ReplicationCoordinator::get(opCtx)->checkReplEnabledForCommand(&result);
         uassertStatusOK(status);
 
-        int secs = (int)cmdObj.firstElement().numberInt();
+        auto secs = cmdObj.firstElement().safeNumberInt();
         uassertStatusOK(ReplicationCoordinator::get(opCtx)->processReplSetFreeze(secs, &result));
         return true;
     }
@@ -488,14 +528,15 @@ public:
                "primary.)\n"
                "http://dochub.mongodb.org/core/replicasetcommands";
     }
-    CmdReplSetStepDown()
-        : ReplSetCommand("replSetStepDown"),
-          _stepDownCmdsWithForceExecutedMetric("commands.replSetStepDownWithForce.total",
-                                               &_stepDownCmdsWithForceExecuted),
-          _stepDownCmdsWithForceFailedMetric("commands.replSetStepDownWithForce.failed",
-                                             &_stepDownCmdsWithForceFailed) {}
+
+    bool shouldCheckoutSession() const final {
+        return false;
+    }
+
+    CmdReplSetStepDown() : ReplSetCommand("replSetStepDown") {}
+
     virtual bool run(OperationContext* opCtx,
-                     const string&,
+                     const DatabaseName&,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         const bool force = cmdObj["force"].trueValue();
@@ -504,7 +545,7 @@ public:
             _stepDownCmdsWithForceExecuted.increment();
         }
 
-        auto onExitGuard = makeGuard([&] {
+        ScopeGuard onExitGuard([&] {
             if (force) {
                 _stepDownCmdsWithForceFailed.increment();
             }
@@ -559,10 +600,8 @@ public:
     }
 
 private:
-    mutable Counter64 _stepDownCmdsWithForceExecuted;
-    mutable Counter64 _stepDownCmdsWithForceFailed;
-    ServerStatusMetricField<Counter64> _stepDownCmdsWithForceExecutedMetric;
-    ServerStatusMetricField<Counter64> _stepDownCmdsWithForceFailedMetric;
+    CounterMetric _stepDownCmdsWithForceExecuted{"commands.replSetStepDownWithForce.total"};
+    CounterMetric _stepDownCmdsWithForceFailed{"commands.replSetStepDownWithForce.failed"};
 
     ActionSet getAuthActionSet() const override {
         return ActionSet{ActionType::replSetStateChange};
@@ -577,7 +616,7 @@ public:
     }
     CmdReplSetMaintenance() : ReplSetCommand("replSetMaintenance") {}
     virtual bool run(OperationContext* opCtx,
-                     const string&,
+                     const DatabaseName&,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         Status status = ReplicationCoordinator::get(opCtx)->checkReplEnabledForCommand(&result);
@@ -603,14 +642,14 @@ public:
     }
     CmdReplSetSyncFrom() : ReplSetCommand("replSetSyncFrom") {}
     virtual bool run(OperationContext* opCtx,
-                     const string&,
+                     const DatabaseName&,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         Status status = ReplicationCoordinator::get(opCtx)->checkReplEnabledForCommand(&result);
         uassertStatusOK(status);
 
         HostAndPort targetHostAndPort;
-        status = targetHostAndPort.initialize(cmdObj["replSetSyncFrom"].valuestrsafe());
+        status = targetHostAndPort.initialize(cmdObj.getStringField("replSetSyncFrom"));
         uassertStatusOK(status);
 
         uassertStatusOK(ReplicationCoordinator::get(opCtx)->processReplSetSyncFrom(
@@ -628,7 +667,7 @@ class CmdReplSetUpdatePosition : public ReplSetCommand {
 public:
     CmdReplSetUpdatePosition() : ReplSetCommand("replSetUpdatePosition") {}
     virtual bool run(OperationContext* opCtx,
-                     const string&,
+                     const DatabaseName&,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         auto replCoord = repl::ReplicationCoordinator::get(opCtx->getClient()->getServiceContext());
@@ -645,24 +684,23 @@ public:
         if (metadataResult.isOK()) {
             // New style update position command has metadata, which may inform the
             // upstream of a higher term.
-            auto metadata = metadataResult.getValue();
+            const auto& metadata = metadataResult.getValue();
+            if (metadata.hasReplicaSetId()) {
+                auto config = replCoord->getConfig();
+                uassert(ErrorCodes::InconsistentReplicaSetNames,
+                        "The current replicaSetId does not match the one in replSetMetaData",
+                        !config.isInitialized() ||
+                            config.getReplicaSetId() == metadata.getReplicaSetId());
+            }
+
             replCoord->processReplSetMetadata(metadata);
         }
 
-        // In the case of an update from a member with an invalid replica set config,
-        // we return our current config version.
-        long long configVersion = -1;
-
         UpdatePositionArgs args;
 
         status = args.initialize(cmdObj);
         if (status.isOK()) {
-            status = replCoord->processReplSetUpdatePosition(args, &configVersion);
-
-            if (status == ErrorCodes::InvalidReplicaSetConfig) {
-                result.append("configVersion", configVersion);
-            }
-            // TODO convert to uassertStatusOK once SERVER-34806 is done.
+            status = replCoord->processReplSetUpdatePosition(args);
             return CommandHelpers::appendCommandStatusNoThrow(result, status);
         } else {
             // Parsing error from UpdatePositionArgs.
@@ -680,17 +718,17 @@ namespace {
  */
 bool replHasDatabases(OperationContext* opCtx) {
     StorageEngine* storageEngine = getGlobalServiceContext()->getStorageEngine();
-    std::vector<string> names = storageEngine->listDatabases();
+    std::vector<DatabaseName> dbNames = storageEngine->listDatabases();
 
-    if (names.size() >= 2)
+    if (dbNames.size() >= 2)
         return true;
-    if (names.size() == 1) {
-        if (names[0] != "local")
+    if (dbNames.size() == 1) {
+        if (dbNames[0].db() != "local")
             return true;
 
         // we have a local database.  return true if oplog isn't empty
         BSONObj o;
-        if (Helpers::getSingleton(opCtx, NamespaceString::kRsOplogNamespace.ns().c_str(), o)) {
+        if (Helpers::getSingleton(opCtx, NamespaceString::kRsOplogNamespace, o)) {
             return true;
         }
     }
@@ -706,7 +744,7 @@ class CmdReplSetHeartbeat : public ReplSetCommand {
 public:
     CmdReplSetHeartbeat() : ReplSetCommand("replSetHeartbeat") {}
     virtual bool run(OperationContext* opCtx,
-                     const string&,
+                     const DatabaseName&,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         rsDelayHeartbeatResponse.execute(
@@ -722,8 +760,8 @@ public:
         Status status = Status(ErrorCodes::InternalError, "status not set in heartbeat code");
         /* we don't call ReplSetCommand::check() here because heartbeat
            checks many things that are pre-initialization. */
-        if (!ReplicationCoordinator::get(opCtx)->getSettings().usingReplSets()) {
-            status = Status(ErrorCodes::NoReplicationEnabled, "not running with --replSet");
+        if (!ReplicationCoordinator::get(opCtx)->isReplEnabled()) {
+            status = Status(ErrorCodes::NoReplicationEnabled, "not running using replication");
             uassertStatusOK(status);
         }
 
@@ -757,7 +795,7 @@ public:
     CmdReplSetStepUp() : ReplSetCommand("replSetStepUp") {}
 
     virtual bool run(OperationContext* opCtx,
-                     const string&,
+                     const DatabaseName&,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         Status status = ReplicationCoordinator::get(opCtx)->checkReplEnabledForCommand(&result);
@@ -796,7 +834,7 @@ public:
     CmdReplSetAbortPrimaryCatchUp() : ReplSetCommand("replSetAbortPrimaryCatchUp") {}
 
     virtual bool run(OperationContext* opCtx,
-                     const string&,
+                     const DatabaseName&,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) override {
         Status status = ReplicationCoordinator::get(opCtx)->checkReplEnabledForCommand(&result);
