@@ -44,6 +44,7 @@
 #include "mongo/db/storage/storage_engine_init.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/rpc/topology_version_gen.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/net/hostandport.h"
 #include "mongo/util/time_support.h"
 
@@ -73,7 +74,7 @@ class ReplSetMetadata;
 namespace repl {
 
 class BackgroundSync;
-class IsMasterResponse;
+class HelloResponse;
 class OpTime;
 class OpTimeAndWallTime;
 class ReadConcernArgs;
@@ -101,7 +102,7 @@ public:
     static void set(ServiceContext* service,
                     std::unique_ptr<ReplicationCoordinator> replCoordinator);
 
-    struct StatusAndDuration {
+    struct MONGO_WARN_UNUSED_RESULT_CLASS StatusAndDuration {
     public:
         Status status;
         Milliseconds duration;
@@ -117,7 +118,7 @@ public:
      * initialization they need.
      */
     virtual void startup(OperationContext* opCtx,
-                         LastStorageEngineShutdownState lastStorageEngineShutdownState) = 0;
+                         StorageEngine::LastShutdownState lastShutdownState) = 0;
 
     /**
      * Start terminal shutdown.  This causes the topology coordinator to refuse to vote in any
@@ -128,6 +129,23 @@ public:
      */
     virtual void enterTerminalShutdown() = 0;
 
+    /**
+     * We enter quiesce mode during the shutdown process if we are in secondary mode. While in
+     * quiesce mode, we allow reads to continue and accept new reads, but we fail hello requests
+     * with ShutdownInProgress. This function causes us to increment the topologyVersion and start
+     * failing hello requests with ShutdownInProgress. Returns true if the server entered quiesce
+     * mode.
+     *
+     * We take in quiesceTime only for reporting purposes. The waiting during quiesce mode happens
+     * external to the ReplicationCoordinator.
+     */
+    virtual bool enterQuiesceModeIfSecondary(Milliseconds quiesceTime) = 0;
+
+    /**
+     * Returns whether the server is in quiesce mode.
+     */
+    virtual bool inQuiesceMode() const = 0;
+
     /**
      * Does whatever cleanup is required to stop replication, including instructing the other
      * components of the replication system to shut down and stop any threads they are using,
@@ -135,15 +153,6 @@ public:
      */
     virtual void shutdown(OperationContext* opCtx) = 0;
 
-    /**
-     * Performs some bookkeeping to make sure that it's a clean shutdown (i.e. dataset is
-     * consistent with top of the oplog).
-     * This should be called after calling shutdown() and should make sure that that are no
-     * active readers while executing this method as this does perform timestamped storage
-     * writes at lastAppliedTimestamp.
-     */
-    virtual void markAsCleanShutdownIfPossible(OperationContext* opCtx) = 0;
-
     /**
      * Returns a reference to the parsed command line arguments that are related to replication.
      */
@@ -178,8 +187,11 @@ public:
      * Returns OK if member state is 'state'.
      * Returns ErrorCodes::ExceededTimeLimit if we timed out waiting for the state change.
      * Returns ErrorCodes::BadValue if timeout is negative.
+     * Throws if interrupted (pass Interruptible::notInterruptible() if interruption is not desired.
      */
-    virtual Status waitForMemberState(MemberState expectedState, Milliseconds timeout) = 0;
+    virtual Status waitForMemberState(Interruptible* interruptible,
+                                      MemberState expectedState,
+                                      Milliseconds timeout) = 0;
 
     /**
      * Returns true if this node is in state PRIMARY or SECONDARY.
@@ -198,10 +210,10 @@ public:
     virtual bool isInPrimaryOrSecondaryState_UNSAFE() const = 0;
 
     /**
-     * Returns how slave delayed this node is configured to be, or 0 seconds if this node is not a
-     * member of the current replica set configuration.
+     * Returns how secondary delayed this node is configured to be, or 0 seconds if this node is not
+     * a member of the current replica set configuration.
      */
-    virtual Seconds getSlaveDelaySecs() const = 0;
+    virtual Seconds getSecondaryDelaySecs() const = 0;
 
     /**
      * Blocks the calling thread for up to writeConcern.wTimeout millis, or until "opTime" has
@@ -213,7 +225,7 @@ public:
      * ErrorCodes::ExceededTimeLimit if the opCtx->getMaxTimeMicrosRemaining is reached before
      *     the data has been sufficiently replicated
      * ErrorCodes::NotWritablePrimary if the node is not a writable primary
-     * ErrorCodes::UnknownReplWriteConcern if the writeConcern.wMode contains a write concern
+     * ErrorCodes::UnknownReplWriteConcern if the writeConcern.w contains a write concern
      *     mode that is not known
      * ErrorCodes::ShutdownInProgress if we are mid-shutdown
      * ErrorCodes::Interrupted if the operation was killed with killop()
@@ -222,6 +234,14 @@ public:
                                                const OpTime& opTime,
                                                const WriteConcernOptions& writeConcern) = 0;
 
+    /**
+     * Returns a future that will be set when the given writeConcern has been satisfied or the node
+     * is not a writable primary, is interrupted, or shuts down. Notably this will ignore the
+     * wTimeout in the given write concern.
+     */
+    virtual SharedSemiFuture<void> awaitReplicationAsyncNoWTimeout(
+        const OpTime& opTime, const WriteConcernOptions& writeConcern) = 0;
+
     /**
      * Causes this node to relinquish being primary for at least 'stepdownTime'.  If 'force' is
      * false, before doing so it will wait for 'waitTime' for one other electable node to be caught
@@ -233,10 +253,10 @@ public:
                           const Milliseconds& stepdownTime) = 0;
 
     /**
-     * Returns true if the node can be considered master for the purpose of introspective
-     * commands such as isMaster() and rs.status().
+     * Returns true if the primary can be considered writable for the purpose of introspective
+     * commands such as hello() and rs.status().
      */
-    virtual bool isMasterForReportingPurposes() = 0;
+    virtual bool isWritablePrimaryForReportingPurposes() = 0;
 
     /**
      * Returns true if it is valid for this node to accept writes on the given database.  Currently
@@ -302,10 +322,13 @@ public:
     /**
      * Returns Status::OK() if it is valid for this node to serve reads on the given collection
      * and an errorcode indicating why the node cannot if it cannot.
+     *
+     * Should not be called if namespace is locked is a view. The user should instead check if we
+     * can serve reads for the backing collection.
      */
     virtual Status checkCanServeReadsFor(OperationContext* opCtx,
                                          const NamespaceString& ns,
-                                         bool slaveOk) = 0;
+                                         bool secondaryOk) = 0;
 
     /**
      * Version which does not check for the RSTL.  Do not use in new code. Without the RSTL held,
@@ -313,7 +336,7 @@ public:
      */
     virtual Status checkCanServeReadsFor_UNSAFE(OperationContext* opCtx,
                                                 const NamespaceString& ns,
-                                                bool slaveOk) = 0;
+                                                bool secondaryOk) = 0;
 
     /**
      * Returns true if this node should ignore index constraints for idempotency reasons.
@@ -348,29 +371,23 @@ public:
      */
     virtual void setMyLastDurableOpTimeAndWallTime(const OpTimeAndWallTime& opTimeAndWallTime) = 0;
 
-    /**
-     * This type is used to represent the "consistency" of a current database state. In
-     * replication, there may be times when our database data is not represented by a single optime,
-     * because we have fetched remote data from different points in time. For example, when we are
-     * in RECOVERING following a refetch based rollback. We never allow external clients to read
-     * from the database if it is not consistent.
-     */
-    enum class DataConsistency { Consistent, Inconsistent };
-
     /**
      * Updates our internal tracking of the last OpTime applied to this node, but only
      * if the supplied optime is later than the current last OpTime known to the replication
-     * coordinator. The 'consistency' argument must tell whether or not the optime argument
-     * represents a consistent database state. Also updates the wall clock time corresponding to
-     * that operation.
+     * coordinator.
      *
      * This function is used by logOp() on a primary, since the ops in the oplog do not
      * necessarily commit in sequential order. It is also used when we finish oplog batch
      * application on secondaries, to avoid any potential race conditions around setting the
      * applied optime from more than one thread.
+     *
+     * Since the last applied op time and wall time might not be visible (i.e. there may be
+     * "oplog holes" from oplog entries with earlier timestamps which commit after this one)
+     * this method does not notify oplog waiters.  Callers which know the new lastApplied is at
+     * a no-holes point should call signalOplogWaiters after calling this method.
      */
     virtual void setMyLastAppliedOpTimeAndWallTimeForward(
-        const OpTimeAndWallTime& opTimeAndWallTime, DataConsistency consistency) = 0;
+        const OpTimeAndWallTime& opTimeAndWallTime) = 0;
 
     /**
      * Updates our internal tracking of the last OpTime durable to this node, but only
@@ -402,8 +419,13 @@ public:
     /*
      * Returns the same as getMyLastAppliedOpTime() and additionally returns the wall clock time
      * corresponding to that OpTime.
+     *
+     * When rollbackSafe is true, this returns an empty OpTimeAndWallTime if the node is in ROLLBACK
+     * state. The lastAppliedOpTime during ROLLBACK might be temporarily pointing to an oplog entry
+     * in the divergent branch of history which would become invalid after the rollback finishes.
      */
-    virtual OpTimeAndWallTime getMyLastAppliedOpTimeAndWallTime() const = 0;
+    virtual OpTimeAndWallTime getMyLastAppliedOpTimeAndWallTime(
+        bool rollbackSafe = false) const = 0;
 
     /**
      * Returns the last optime recorded by setMyLastDurableOpTime.
@@ -416,6 +438,13 @@ public:
      */
     virtual OpTimeAndWallTime getMyLastDurableOpTimeAndWallTime() const = 0;
 
+    /**
+     * Waits until the majority committed snapshot is at least the 'targetOpTime'.
+     */
+    virtual Status waitUntilMajorityOpTime(OperationContext* opCtx,
+                                           OpTime targetOpTime,
+                                           boost::optional<Date_t> deadline = boost::none) = 0;
+
     /**
      * Waits until the optime of the current node is at least the opTime specified in 'settings'.
      *
@@ -514,14 +543,14 @@ public:
      *      |
      *      | applier signals drain is complete
      *      V
-     * - primary is in master mode
+     * - primary is in writable mode
      * (producer: Stopped, applier: Stopped)
      *
      *
      * Step-down
      * =========
      * The state transitions become:
-     * - primary is in master mode
+     * - primary is in writable mode
      * (producer: Stopped, applier: Stopped)
      *      |
      *      | step down
@@ -564,7 +593,8 @@ public:
      * steps down and steps up so quickly that the applier signals drain complete in the wrong
      * term.
      */
-    virtual void signalDrainComplete(OperationContext* opCtx, long long termWhenBufferIsEmpty) = 0;
+    virtual void signalDrainComplete(OperationContext* opCtx,
+                                     long long termWhenBufferIsEmpty) noexcept = 0;
 
     /**
      * Signals the sync source feedback thread to wake up and send a handshake and
@@ -584,32 +614,106 @@ public:
      * Handles an incoming replSetGetStatus command. Adds BSON to 'result'. If kInitialSync is
      * requested but initial sync is not running, kBasic will be used.
      */
-    virtual Status processReplSetGetStatus(BSONObjBuilder* result,
+    virtual Status processReplSetGetStatus(OperationContext* opCtx,
+                                           BSONObjBuilder* result,
                                            ReplSetGetStatusResponseStyle responseStyle) = 0;
 
     /**
-     * Does an initial sync of data, after dropping existing data.
-     */
-    virtual Status resyncData(OperationContext* opCtx, bool waitUntilCompleted) = 0;
-
-    /**
-     * Adds to "result" a description of the slaveInfo data structure used to map RIDs to their
+     * Adds to "result" a description of the memberData data structure used to map RIDs to their
      * last known optimes.
      */
-    virtual void appendSlaveInfoData(BSONObjBuilder* result) = 0;
+    virtual void appendSecondaryInfoData(BSONObjBuilder* result) = 0;
 
     /**
      * Returns a copy of the current ReplSetConfig.
+     *
+     * To avoid unnecessarily copying the ReplSetConfig through this method, please use the
+     * getConfig* methods below if you are only accessing a specific field of the ReplSetConfig,
+     * i.e. replCoord->getConfigNumMembers() instead of replCoord->getConfig().getNumMembers(). See
+     * SERVER-47828.
      */
     virtual ReplSetConfig getConfig() const = 0;
 
+    /**
+     * Returns the current ReplSetConfig's connection string.
+     */
+    virtual ConnectionString getConfigConnectionString() const = 0;
+
+    /**
+     * Returns the current ReplSetConfig's election timeout period.
+     */
+    virtual Milliseconds getConfigElectionTimeoutPeriod() const = 0;
+
+    /**
+     * Returns the current ReplSetConfig's voting members.
+     */
+    virtual std::vector<MemberConfig> getConfigVotingMembers() const = 0;
+
+    /**
+     * Returns the current ReplSetConfig's term.
+     */
+    virtual std::int64_t getConfigTerm() const = 0;
+
+    /**
+     * Returns the current ReplSetConfig's version.
+     */
+    virtual std::int64_t getConfigVersion() const = 0;
+
+    /**
+     * Returns the (version, term) pair of the current ReplSetConfig.
+     */
+    virtual ConfigVersionAndTerm getConfigVersionAndTerm() const = 0;
+
+    /**
+     * Returns the number of members in the current ReplSetConfig.
+     */
+    virtual int getConfigNumMembers() const = 0;
+
+    /**
+     * Returns the amount of time to wait for a response to heartbeats sent to other
+     * nodes in the current ReplSetConfig.
+     */
+    virtual Milliseconds getConfigHeartbeatTimeoutPeriodMillis() const = 0;
+
+    /**
+     * Returns the BSON of the current ReplSetConfig.
+     */
+    virtual BSONObj getConfigBSON() const = 0;
+
+    /**
+     * Validates the given WriteConcernOptions on the current ReplSetConfig.
+     */
+    virtual Status validateWriteConcern(const WriteConcernOptions& writeConcern) const = 0;
+
+    /**
+     * Returns a pointer to the MemberConfig corresponding to the member with the given
+     * HostAndPort in the current ReplSetConfig, or NULL if there is no member with that address.
+     */
+    virtual const MemberConfig* findConfigMemberByHostAndPort(const HostAndPort& hap) const = 0;
+
+    /**
+     * Returns whether all members of the current ReplSetConfig set have hostname localhost.
+     */
+    virtual bool isConfigLocalHostAllowed() const = 0;
+
+    /**
+     * Returns the interval between the time the last heartbeat from a node was received
+     * successfully, or the time when we gave up retrying, and when the next heartbeat should be
+     * sent to a target, for the current ReplSetConfig.
+     */
+    virtual Milliseconds getConfigHeartbeatInterval() const = 0;
+
     /**
      * Handles an incoming replSetGetConfig command. Adds BSON to 'result'.
      *
      * If commitmentStatus is true, adds a boolean 'commitmentStatus' field to 'result' indicating
      * whether the current config is committed.
+     *
+     * If includeNewlyAdded is true, does not omit 'newlyAdded' fields from the config.
      */
-    virtual void processReplSetGetConfig(BSONObjBuilder* result, bool commitmentStatus = false) = 0;
+    virtual void processReplSetGetConfig(BSONObjBuilder* result,
+                                         bool commitmentStatus = false,
+                                         bool includeNewlyAdded = false) = 0;
 
     /**
      * Processes the ReplSetMetadata returned from a command run against another
@@ -654,6 +758,13 @@ public:
      */
     virtual bool getMaintenanceMode() = 0;
 
+    /**
+     * Returns true if serverless mode is enabled and the replicaSetId differs from the current
+     * replicaSetId. It signals the sync source should be dropped and the new batch of oplogs should
+     * be discarded.
+     */
+    virtual bool shouldDropSyncSourceAfterShardSplit(OID replicaSetId) const = 0;
+
     /**
      * Handles an incoming replSetSyncFrom command. Adds BSON to 'result'
      * returns Status::OK if the sync target could be set and an ErrorCode indicating why it
@@ -703,6 +814,18 @@ public:
                                      GetNewConfigFn getNewConfig,
                                      bool force) = 0;
 
+    /**
+     * Performs a reconfig that skips certain safety checks, including the following:
+     * 1) Wait for the current config to be majority committed
+     * 2) Wait for oplog commitment
+     * 3) Quorum check
+     * This function is only intended to be called for internal reconfigs that do not change the
+     * consensus group (eg. only bumping the config version or term). These scenarios are expected
+     * to be able to bypass certain safety checks because the caller guarantees the reconfig to be
+     * safe.
+     */
+    virtual Status doOptimizedReconfig(OperationContext* opCtx, GetNewConfigFn) = 0;
+
     /**
      * Waits until the following two conditions are satisfied:
      *  (1) The current config has propagated to a majority of nodes.
@@ -731,8 +854,7 @@ public:
      * "configVersion" will be populated with our config version if and only if we return
      * InvalidReplicaSetConfig.
      */
-    virtual Status processReplSetUpdatePosition(const UpdatePositionArgs& updates,
-                                                long long* configVersion) = 0;
+    virtual Status processReplSetUpdatePosition(const UpdatePositionArgs& updates) = 0;
 
     /**
      * Returns a bool indicating whether or not this node builds indexes.
@@ -762,11 +884,9 @@ public:
 
     /**
      * Loads the optime from the last op in the oplog into the coordinator's lastAppliedOpTime and
-     * lastDurableOpTime values. The 'consistency' argument must tell whether or not the optime of
-     * the op in the oplog represents a consistent database state.
+     * lastDurableOpTime values.
      */
-    virtual void resetLastOpTimesFromOplog(OperationContext* opCtx,
-                                           DataConsistency consistency) = 0;
+    virtual void resetLastOpTimesFromOplog(OperationContext* opCtx) = 0;
 
     /**
      * Returns the OpTime of the latest replica set-committed op known to this server.
@@ -836,20 +956,15 @@ public:
                                             const Timestamp& untilSnapshot) = 0;
 
     /**
-     * Resets all information related to snapshotting.
+     * Clears the current committed snapshot.
      */
-    virtual void dropAllSnapshots() = 0;
+    virtual void clearCommittedSnapshot() = 0;
 
     /**
      * Gets the latest OpTime of the currentCommittedSnapshot.
      */
     virtual OpTime getCurrentCommittedSnapshotOpTime() const = 0;
 
-    /**
-     * Gets the latest OpTime of the currentCommittedSnapshot and its corresponding wall clock time.
-     */
-    virtual OpTimeAndWallTime getCurrentCommittedSnapshotOpTimeAndWallTime() const = 0;
-
     /**
      * Appends diagnostics about the replication subsystem.
      */
@@ -959,35 +1074,35 @@ public:
      * internal operations occurs (i.e. step up, step down, or rollback). Also logs the metrics.
      */
     virtual void updateAndLogStateTransitionMetrics(
-        const ReplicationCoordinator::OpsKillingStateTransitionEnum stateTransition,
-        const size_t numOpsKilled,
-        const size_t numOpsRunning) const = 0;
+        ReplicationCoordinator::OpsKillingStateTransitionEnum stateTransition,
+        size_t numOpsKilled,
+        size_t numOpsRunning) const = 0;
 
     /**
      * Increment the server TopologyVersion and fulfill the promise of any currently waiting
-     * isMaster request.
+     * hello request.
      */
     virtual void incrementTopologyVersion() = 0;
 
     /**
-     * Constructs and returns an IsMasterResponse. Will block until the given deadline waiting for a
+     * Constructs and returns a HelloResponse. Will block until the given deadline waiting for a
      * significant topology change if the 'counter' field of 'clientTopologyVersion' is equal to the
      * current TopologyVersion 'counter' from the TopologyCoordinator. Returns immediately if
      * 'clientTopologyVersion' < TopologyVersion of the TopologyCoordinator or if the processId
      * differs.
      */
-    virtual std::shared_ptr<const IsMasterResponse> awaitIsMasterResponse(
+    virtual std::shared_ptr<const HelloResponse> awaitHelloResponse(
         OperationContext* opCtx,
         const SplitHorizon::Parameters& horizonParams,
         boost::optional<TopologyVersion> clientTopologyVersion,
         boost::optional<Date_t> deadline) = 0;
 
     /**
-     * The futurized version of `awaitIsMasterResponse()`:
-     * * The future is ready for all cases that `awaitIsMasterResponse()` returns immediately.
-     * * For cases that `awaitIsMasterResponse()` blocks, calling `get()` on the future is blocking.
+     * The futurized version of `awaitHelloResponse()`:
+     * * The future is ready for all cases that `awaitHelloResponse()` returns immediately.
+     * * For cases that `awaitHelloResponse()` blocks, calling `get()` on the future is blocking.
      */
-    virtual SharedSemiFuture<std::shared_ptr<const IsMasterResponse>> getIsMasterResponseFuture(
+    virtual SharedSemiFuture<std::shared_ptr<const HelloResponse>> getHelloResponseFuture(
         const SplitHorizon::Parameters& horizonParams,
         boost::optional<TopologyVersion> clientTopologyVersion) = 0;
 
@@ -1032,6 +1147,40 @@ public:
                                                     OnRemoteCmdScheduledFn onRemoteCmdScheduled,
                                                     OnRemoteCmdCompleteFn onRemoteCmdComplete) = 0;
 
+    /**
+     * A testing only function that cancels and reschedules replication heartbeats immediately.
+     */
+    virtual void restartScheduledHeartbeats_forTest() = 0;
+
+    /**
+     * Records if the cluster-wide write concern is set during sharding initialization.
+     *
+     * This function will assert if the shard can't talk to config server.
+     */
+    virtual void recordIfCWWCIsSetOnConfigServerOnStartup(OperationContext* opCtx) = 0;
+
+    /**
+     * Interface used to synchronize changes to custom write concern tags in the config and
+     * custom default write concern settings.
+     *
+     * Use [reserve|release]DefaultWriteConcernChanges when making changes to the current
+     * default read/write concern.
+     * Use [reserve|release]ConfigWriteConcernTagChanges when executing a reconfig that
+     * could potentially change read/write concern tags.
+     */
+    class WriteConcernTagChanges {
+    public:
+        WriteConcernTagChanges() = default;
+        virtual ~WriteConcernTagChanges() = default;
+        virtual bool reserveDefaultWriteConcernChange() = 0;
+        virtual void releaseDefaultWriteConcernChange() = 0;
+
+        virtual bool reserveConfigWriteConcernTagChange() = 0;
+        virtual void releaseConfigWriteConcernTagChange() = 0;
+    };
+
+    virtual WriteConcernTagChanges* getWriteConcernTagChanges() = 0;
+
 protected:
     ReplicationCoordinator();
 };
