@@ -27,13 +27,14 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 #define LOGV2_FOR_ELECTION(ID, DLEVEL, MESSAGE, ...) \
     LOGV2_DEBUG_OPTIONS(                             \
         ID, DLEVEL, {logv2::LogComponent::kReplicationElection}, MESSAGE, ##__VA_ARGS__)
 #define LOGV2_FOR_HEARTBEATS(ID, DLEVEL, MESSAGE, ...) \
     LOGV2_DEBUG_OPTIONS(                               \
         ID, DLEVEL, {logv2::LogComponent::kReplicationHeartbeats}, MESSAGE, ##__VA_ARGS__)
+#define LOGV2_FOR_SHARD_SPLIT(ID, DLEVEL, MESSAGE, ...) \
+    LOGV2_DEBUG_OPTIONS(ID, DLEVEL, {logv2::LogComponent::kTenantMigration}, MESSAGE, ##__VA_ARGS__)
 
 #include "mongo/platform/basic.h"
 
@@ -41,22 +42,24 @@
 #include <functional>
 
 #include "mongo/base/status.h"
+#include "mongo/db/commands/test_commands_enabled.h"
+#include "mongo/db/global_settings.h"
 #include "mongo/db/index_builds_coordinator.h"
-#include "mongo/db/kill_sessions_local.h"
-#include "mongo/db/logical_clock.h"
-#include "mongo/db/logical_time_validator.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/heartbeat_response_action.h"
+#include "mongo/db/repl/repl_server_parameters_gen.h"
 #include "mongo/db/repl/repl_set_config_checks.h"
 #include "mongo/db/repl/repl_set_heartbeat_args_v1.h"
 #include "mongo/db/repl/repl_set_heartbeat_response.h"
+#include "mongo/db/repl/replica_set_aware_service.h"
 #include "mongo/db/repl/replication_coordinator_impl.h"
 #include "mongo/db/repl/replication_metrics.h"
 #include "mongo/db/repl/replication_process.h"
+#include "mongo/db/repl/tenant_migration_access_blocker_registry.h"
 #include "mongo/db/repl/topology_coordinator.h"
 #include "mongo/db/repl/vote_requester.h"
 #include "mongo/db/service_context.h"
-#include "mongo/db/session_catalog.h"
+#include "mongo/db/session/kill_sessions_local.h"
 #include "mongo/db/storage/control/journal_flusher.h"
 #include "mongo/logv2/log.h"
 #include "mongo/platform/mutex.h"
@@ -67,6 +70,9 @@
 #include "mongo/util/str.h"
 #include "mongo/util/time_support.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
+
+
 namespace mongo {
 namespace repl {
 
@@ -74,17 +80,20 @@ namespace {
 
 MONGO_FAIL_POINT_DEFINE(blockHeartbeatStepdown);
 MONGO_FAIL_POINT_DEFINE(blockHeartbeatReconfigFinish);
+MONGO_FAIL_POINT_DEFINE(hangAfterTrackingNewHandleInHandleHeartbeatResponseForTest);
 MONGO_FAIL_POINT_DEFINE(waitForPostActionCompleteInHbReconfig);
 
 }  // namespace
 
 using executor::RemoteCommandRequest;
 
-Milliseconds ReplicationCoordinatorImpl::_getRandomizedElectionOffset_inlock() {
+long long ReplicationCoordinatorImpl::_getElectionOffsetUpperBound_inlock() {
     long long electionTimeout = durationCount<Milliseconds>(_rsConfig.getElectionTimeoutPeriod());
-    long long randomOffsetUpperBound =
-        electionTimeout * _externalState->getElectionTimeoutOffsetLimitFraction();
+    return electionTimeout * _externalState->getElectionTimeoutOffsetLimitFraction();
+}
 
+Milliseconds ReplicationCoordinatorImpl::_getRandomizedElectionOffset_inlock() {
+    long long randomOffsetUpperBound = _getElectionOffsetUpperBound_inlock();
     // Avoid divide by zero error in random number generator.
     if (randomOffsetUpperBound == 0) {
         return Milliseconds(0);
@@ -94,7 +103,8 @@ Milliseconds ReplicationCoordinatorImpl::_getRandomizedElectionOffset_inlock() {
 }
 
 void ReplicationCoordinatorImpl::_doMemberHeartbeat(executor::TaskExecutor::CallbackArgs cbData,
-                                                    const HostAndPort& target) {
+                                                    const HostAndPort& target,
+                                                    const std::string& replSetName) {
     stdx::lock_guard<Latch> lk(_mutex);
 
     _untrackHeartbeatHandle_inlock(cbData.myHandle);
@@ -106,7 +116,7 @@ void ReplicationCoordinatorImpl::_doMemberHeartbeat(executor::TaskExecutor::Call
     BSONObj heartbeatObj;
     Milliseconds timeout(0);
     const std::pair<ReplSetHeartbeatArgsV1, Milliseconds> hbRequest =
-        _topCoord->prepareHeartbeatRequestV1(now, _settings.ourSetName(), target);
+        _topCoord->prepareHeartbeatRequestV1(now, replSetName, target);
     heartbeatObj = hbRequest.first.toBSON();
     timeout = hbRequest.second;
 
@@ -114,7 +124,7 @@ void ReplicationCoordinatorImpl::_doMemberHeartbeat(executor::TaskExecutor::Call
         target, "admin", heartbeatObj, BSON(rpc::kReplSetMetadataFieldName << 1), nullptr, timeout);
     const executor::TaskExecutor::RemoteCommandCallbackFn callback =
         [=](const executor::TaskExecutor::RemoteCommandCallbackArgs& cbData) {
-            return _handleHeartbeatResponse(cbData);
+            return _handleHeartbeatResponse(cbData, replSetName);
         };
 
     LOGV2_FOR_HEARTBEATS(4615670,
@@ -129,7 +139,8 @@ void ReplicationCoordinatorImpl::_doMemberHeartbeat(executor::TaskExecutor::Call
 }
 
 void ReplicationCoordinatorImpl::_scheduleHeartbeatToTarget_inlock(const HostAndPort& target,
-                                                                   Date_t when) {
+                                                                   Date_t when,
+                                                                   std::string replSetName) {
     LOGV2_FOR_HEARTBEATS(4615618,
                          2,
                          "Scheduling heartbeat to {target} at {when}",
@@ -138,8 +149,9 @@ void ReplicationCoordinatorImpl::_scheduleHeartbeatToTarget_inlock(const HostAnd
                          "when"_attr = when);
     _trackHeartbeatHandle_inlock(
         _replExecutor->scheduleWorkAt(when,
-                                      [=](const executor::TaskExecutor::CallbackArgs& cbData) {
-                                          _doMemberHeartbeat(cbData, target);
+                                      [=, replSetName = std::move(replSetName)](
+                                          const executor::TaskExecutor::CallbackArgs& cbData) {
+                                          _doMemberHeartbeat(cbData, target, replSetName);
                                       }),
         HeartbeatState::kScheduled,
         target);
@@ -148,7 +160,9 @@ void ReplicationCoordinatorImpl::_scheduleHeartbeatToTarget_inlock(const HostAnd
 void ReplicationCoordinatorImpl::handleHeartbeatResponse_forTest(BSONObj response,
                                                                  int targetIndex,
                                                                  Milliseconds ping) {
-    CallbackHandle handle;
+    // Make a handle to a valid no-op.
+    CallbackHandle handle = uassertStatusOK(
+        _replExecutor->scheduleWork([=](const executor::TaskExecutor::CallbackArgs& args) {}));
     RemoteCommandRequest request;
 
     {
@@ -168,11 +182,13 @@ void ReplicationCoordinatorImpl::handleHeartbeatResponse_forTest(BSONObj respons
     executor::TaskExecutor::RemoteCommandCallbackArgs cbData(
         _replExecutor.get(), handle, request, status);
 
-    _handleHeartbeatResponse(cbData);
+    hangAfterTrackingNewHandleInHandleHeartbeatResponseForTest.pauseWhileSet();
+
+    _handleHeartbeatResponse(cbData, _rsConfig.getReplSetName().toString());
 }
 
 void ReplicationCoordinatorImpl::_handleHeartbeatResponse(
-    const executor::TaskExecutor::RemoteCommandCallbackArgs& cbData) {
+    const executor::TaskExecutor::RemoteCommandCallbackArgs& cbData, const std::string& setName) {
     stdx::unique_lock<Latch> lk(_mutex);
 
     // remove handle from queued heartbeats
@@ -211,6 +227,17 @@ void ReplicationCoordinatorImpl::_handleHeartbeatResponse(
             "target"_attr = target,
             "response"_attr = resp);
 
+        if (responseStatus.isOK() && _rsConfig.isInitialized() &&
+            _rsConfig.getReplSetName() != hbResponse.getReplicaSetName()) {
+            responseStatus =
+                Status(ErrorCodes::InconsistentReplicaSetNames,
+                       str::stream()
+                           << "replica set names do not match, ours: " << _rsConfig.getReplSetName()
+                           << "; remote node's: " << hbResponse.getReplicaSetName());
+            // Ignore metadata.
+            replMetadata = responseStatus;
+        }
+
         // Reject heartbeat responses (and metadata) from nodes with mismatched replica set IDs.
         // It is problematic to perform this check in the heartbeat reconfiguring logic because it
         // is possible for two mismatched replica sets to have the same replica set name and
@@ -233,7 +260,8 @@ void ReplicationCoordinatorImpl::_handleHeartbeatResponse(
             // the new commit point we learned of is on the same branch of history as our own
             // oplog.
             if (_getMemberState_inlock().arbiter() ||
-                (!_getMemberState_inlock().startup() && !_getMemberState_inlock().startup2())) {
+                (!_getMemberState_inlock().startup() && !_getMemberState_inlock().startup2() &&
+                 !_getMemberState_inlock().rollback())) {
                 // The node that sent the heartbeat is not guaranteed to be our sync source.
                 const bool fromSyncSource = false;
                 _advanceCommitPoint(
@@ -313,6 +341,40 @@ void ReplicationCoordinatorImpl::_handleHeartbeatResponse(
         }
     }
 
+    // When receiving a heartbeat response indicating that the remote is in a state past
+    // STARTUP_2, the primary will initiate a reconfig to remove the 'newlyAdded' field for that
+    // node (if present). This field is normally set when we add new members with votes:1 to the
+    // set.
+    if (_getMemberState_inlock().primary() && hbStatusResponse.isOK() &&
+        hbStatusResponse.getValue().hasState()) {
+        auto remoteState = hbStatusResponse.getValue().getState();
+        if (remoteState == MemberState::RS_SECONDARY || remoteState == MemberState::RS_RECOVERING ||
+            remoteState == MemberState::RS_ROLLBACK) {
+            const auto mem = _rsConfig.findMemberByHostAndPort(target);
+            if (mem && mem->isNewlyAdded()) {
+                const auto memId = mem->getId();
+                auto status = _replExecutor->scheduleWork(
+                    [=](const executor::TaskExecutor::CallbackArgs& cbData) {
+                        _reconfigToRemoveNewlyAddedField(
+                            cbData, memId, _rsConfig.getConfigVersionAndTerm());
+                    });
+
+                if (!status.isOK()) {
+                    LOGV2_DEBUG(4634500,
+                                1,
+                                "Failed to schedule work for removing 'newlyAdded' field.",
+                                "memberId"_attr = memId.getData(),
+                                "error"_attr = status.getStatus());
+                } else {
+                    LOGV2_DEBUG(4634501,
+                                1,
+                                "Scheduled automatic reconfig to remove 'newlyAdded' field.",
+                                "memberId"_attr = memId.getData());
+                }
+            }
+        }
+    }
+
     // Abort catchup if we have caught up to the latest known optime after heartbeat refreshing.
     if (_catchupState) {
         _catchupState->signalHeartbeatUpdate_inlock();
@@ -328,7 +390,8 @@ void ReplicationCoordinatorImpl::_handleHeartbeatResponse(
         }
     }
 
-    _scheduleHeartbeatToTarget_inlock(target, std::max(now, action.getNextHeartbeatStartDate()));
+    _scheduleHeartbeatToTarget_inlock(
+        target, std::max(now, action.getNextHeartbeatStartDate()), setName);
 
     _handleHeartbeatResponseAction_inlock(action, hbStatusResponse, std::move(lk));
 }
@@ -406,6 +469,11 @@ stdx::unique_lock<Latch> ReplicationCoordinatorImpl::_handleHeartbeatResponseAct
             break;
         }
     }
+    if (action.getChangedSignificantly()) {
+        lock.unlock();
+        _externalState->notifyOtherMemberDataChanged();
+        lock.lock();
+    }
     return lock;
 }
 
@@ -477,11 +545,6 @@ void ReplicationCoordinatorImpl::_stepDownFinish(
 
     auto opCtx = cc().makeOperationContext();
 
-    // To prevent a deadlock between session checkout and RSTL lock taking, disallow new sessions
-    // from being checked out. Existing sessions currently checked out will be killed by the
-    // killOpThread.
-    ScopedBlockSessionCheckouts blockSessions(opCtx.get());
-
     // kill all write operations which are no longer safe to run on step down. Also, operations that
     // have taken global lock in S mode and operations blocked on prepare conflict will be killed to
     // avoid 3-way deadlock between read, prepared transaction and step down thread.
@@ -601,6 +664,58 @@ void ReplicationCoordinatorImpl::_scheduleHeartbeatReconfig(WithLock lk,
         .status_with_transitional_ignore();
 }
 
+std::tuple<StatusWith<ReplSetConfig>, bool> ReplicationCoordinatorImpl::_resolveConfigToApply(
+    const ReplSetConfig& config) {
+    if (!_settings.isServerless() || !config.isSplitConfig()) {
+        return {config, false};
+    }
+
+    stdx::unique_lock<Latch> lk(_mutex);
+    if (!_rsConfig.isInitialized()) {
+        // Unlock the lock because isSelf performs network I/O.
+        lk.unlock();
+
+        // If this node is listed in the members of incoming config, accept the config.
+        const auto foundSelfInMembers = std::any_of(
+            config.membersBegin(),
+            config.membersEnd(),
+            [externalState = _externalState.get()](const MemberConfig& config) {
+                return externalState->isSelf(config.getHostAndPort(), getGlobalServiceContext());
+            });
+
+        if (foundSelfInMembers) {
+            return {config, false};
+        }
+
+        return {Status(ErrorCodes::NotYetInitialized,
+                       "Cannot apply a split config if the current config is uninitialized"),
+                false};
+    }
+
+    auto recipientConfig = config.getRecipientConfig();
+    const auto& selfMember = _rsConfig.getMemberAt(_selfIndex);
+    if (recipientConfig->findMemberByHostAndPort(selfMember.getHostAndPort())) {
+        if (selfMember.getNumVotes() > 0) {
+            return {Status(ErrorCodes::BadValue, "Cannot apply recipient config to a voting node"),
+                    false};
+        }
+
+        if (_rsConfig.getReplSetName() == recipientConfig->getReplSetName()) {
+            return {Status(ErrorCodes::InvalidReplicaSetConfig,
+                           "Cannot apply recipient config since current config and recipient "
+                           "config have the same set name."),
+                    false};
+        }
+
+        auto mutableConfig = recipientConfig->getMutable();
+        mutableConfig.setConfigVersion(1);
+        mutableConfig.setConfigTerm(1);
+        return {ReplSetConfig(std::move(mutableConfig)), true};
+    }
+
+    return {config, false};
+}
+
 void ReplicationCoordinatorImpl::_heartbeatReconfigStore(
     const executor::TaskExecutor::CallbackArgs& cbd, const ReplSetConfig& newConfig) {
 
@@ -614,12 +729,48 @@ void ReplicationCoordinatorImpl::_heartbeatReconfigStore(
         return;
     }
 
-    const StatusWith<int> myIndex = validateConfigForHeartbeatReconfig(
-        _externalState.get(), newConfig, getGlobalServiceContext());
+    const auto [swConfig, isRecipientConfig] = _resolveConfigToApply(newConfig);
+    if (!swConfig.isOK()) {
+        LOGV2_WARNING(6234600,
+                      "Ignoring new configuration in heartbeat response because it is invalid",
+                      "status"_attr = swConfig.getStatus());
+
+        stdx::lock_guard<Latch> lg(_mutex);
+        invariant(_rsConfigState == kConfigHBReconfiguring);
+        _setConfigState_inlock(!_rsConfig.isInitialized() ? kConfigUninitialized : kConfigSteady);
+        return;
+    }
+
+    const auto configToApply = swConfig.getValue();
+    if (isRecipientConfig) {
+        LOGV2(6309200,
+              "Applying a recipient config for a shard split operation.",
+              "config"_attr = configToApply);
+    }
+
+    const auto myIndex = [&]() -> StatusWith<int> {
+        // We always check the config when _selfIndex is not valid, in order to be able to
+        // recover from transient DNS errors.
+        {
+            stdx::lock_guard<Latch> lk(_mutex);
+            if (_selfIndex >= 0 && sameConfigContents(_rsConfig, configToApply)) {
+                LOGV2_FOR_HEARTBEATS(6351200,
+                                     2,
+                                     "New heartbeat config is only a version/term change, skipping "
+                                     "validation checks",
+                                     "oldConfig"_attr = _rsConfig,
+                                     "newConfig"_attr = configToApply);
+                // If the configs are the same, so is our index.
+                return _selfIndex;
+            }
+        }
+        return validateConfigForHeartbeatReconfig(
+            _externalState.get(), configToApply, getGlobalServiceContext());
+    }();
 
     if (myIndex.getStatus() == ErrorCodes::NodeNotFound) {
         stdx::lock_guard<Latch> lk(_mutex);
-        // If this node absent in newConfig, and this node was not previously initialized,
+        // If this node absent in configToApply, and this node was not previously initialized,
         // return to kConfigUninitialized immediately, rather than storing the config and
         // transitioning into the RS_REMOVED state.  See SERVER-15740.
         if (!_rsConfig.isInitialized()) {
@@ -644,17 +795,35 @@ void ReplicationCoordinatorImpl::_heartbeatReconfigStore(
     } else {
         LOGV2_FOR_HEARTBEATS(4615626,
                              2,
-                             "Config with {newConfigVersionAndTerm} validated for "
+                             "Config with {configToApplyVersionAndTerm} validated for "
                              "reconfig; persisting to disk.",
                              "Config validated for reconfig; persisting to disk",
-                             "newConfigVersionAndTerm"_attr = newConfig.getConfigVersionAndTerm());
+                             "configToApplyVersionAndTerm"_attr =
+                                 configToApply.getConfigVersionAndTerm());
 
         auto opCtx = cc().makeOperationContext();
         // Don't write the no-op for config learned via heartbeats.
-        auto status = _externalState->storeLocalConfigDocument(
-            opCtx.get(), newConfig.toBSON(), false /* writeOplog */);
+        auto status = [&, isRecipientConfig = isRecipientConfig]() {
+            if (isRecipientConfig) {
+                return _externalState->replaceLocalConfigDocument(opCtx.get(),
+                                                                  configToApply.toBSON());
+            } else {
+                return _externalState->storeLocalConfigDocument(
+                    opCtx.get(), configToApply.toBSON(), false /* writeOplog */);
+            }
+        }();
+
         // Wait for durability of the new config document.
-        JournalFlusher::get(opCtx.get())->waitForJournalFlush();
+        try {
+            JournalFlusher::get(opCtx.get())->waitForJournalFlush();
+        } catch (const ExceptionFor<ErrorCodes::InterruptedDueToStorageChange>& e) {
+            // Anyone changing the storage engine is responsible for copying the on-disk
+            // configuration between the old engine and the new.
+            LOGV2_DEBUG(6121300,
+                        1,
+                        "Storage engine changed while waiting for new config to become durable.",
+                        "error"_attr = e.toStatus());
+        }
 
         bool isFirstConfig;
         {
@@ -678,43 +847,68 @@ void ReplicationCoordinatorImpl::_heartbeatReconfigStore(
         }
 
         bool isArbiter = myIndex.isOK() && myIndex.getValue() != -1 &&
-            newConfig.getMemberAt(myIndex.getValue()).isArbiter();
+            configToApply.getMemberAt(myIndex.getValue()).isArbiter();
 
         if (isArbiter) {
-            LogicalClock::get(getGlobalServiceContext())->disable();
-            if (auto validator = LogicalTimeValidator::get(getGlobalServiceContext())) {
-                validator->stopKeyManager();
-            }
+            ReplicaSetAwareServiceRegistry::get(_service).onBecomeArbiter();
         }
 
         if (!isArbiter && myIndex.isOK() && myIndex.getValue() != -1) {
             shouldStartDataReplication = true;
         }
 
+        if (isRecipientConfig) {
+            // Donor access blockers are removed from donor nodes via the shard split op observer.
+            // Donor access blockers are removed from recipient nodes when the node applies the
+            // recipient config. When the recipient primary steps up it will delete its state
+            // document, the call to remove access blockers there will be a no-op.
+
+            LOGV2_FOR_SHARD_SPLIT(8423354, 1, "Removing donor access blockers on recipient node.");
+            TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext())
+                .removeAll(TenantMigrationAccessBlocker::BlockerType::kDonor);
+        }
+
         LOGV2_FOR_HEARTBEATS(
             4615627,
             2,
-            "New configuration with {newConfigVersionAndTerm} persisted "
+            "New configuration with {configToApplyVersionAndTerm} persisted "
             "to local storage; installing new config in memory",
             "New configuration persisted to local storage; installing new config in memory",
-            "newConfigVersionAndTerm"_attr = newConfig.getConfigVersionAndTerm());
+            "configToApplyVersionAndTerm"_attr = configToApply.getConfigVersionAndTerm());
     }
 
-    _heartbeatReconfigFinish(cbd, newConfig, myIndex);
+    _heartbeatReconfigFinish(cbd, configToApply, myIndex, isRecipientConfig);
 
     // Start data replication after the config has been installed.
     if (shouldStartDataReplication) {
-        auto opCtx = cc().makeOperationContext();
-        _replicationProcess->getConsistencyMarkers()->initializeMinValidDocument(opCtx.get());
-        _externalState->startThreads(_settings);
-        _startDataReplication(opCtx.get());
+        while (true) {
+            try {
+                auto opCtx = cc().makeOperationContext();
+                // Initializing minvalid is not allowed to be interrupted.  Make sure it
+                // can't be interrupted by a storage change by taking the global lock first.
+                {
+                    Lock::GlobalLock lk(opCtx.get(), MODE_IX);
+                    _replicationProcess->getConsistencyMarkers()->initializeMinValidDocument(
+                        opCtx.get());
+                }
+                _externalState->startThreads();
+                _startDataReplication(opCtx.get());
+                break;
+            } catch (const ExceptionFor<ErrorCodes::InterruptedDueToStorageChange>& e) {
+                LOGV2_DEBUG(6137701,
+                            1,
+                            "Interrupted while trying to start data replication",
+                            "error"_attr = e.toStatus());
+            }
+        }
     }
 }
 
 void ReplicationCoordinatorImpl::_heartbeatReconfigFinish(
     const executor::TaskExecutor::CallbackArgs& cbData,
     const ReplSetConfig& newConfig,
-    StatusWith<int> myIndex) {
+    StatusWith<int> myIndex,
+    const bool isRecipientConfig) {
     if (cbData.status == ErrorCodes::CallbackCanceled) {
         return;
     }
@@ -727,7 +921,8 @@ void ReplicationCoordinatorImpl::_heartbeatReconfigFinish(
         _replExecutor
             ->scheduleWorkAt(_replExecutor->now() + Milliseconds{10},
                              [=](const executor::TaskExecutor::CallbackArgs& cbData) {
-                                 _heartbeatReconfigFinish(cbData, newConfig, myIndex);
+                                 _heartbeatReconfigFinish(
+                                     cbData, newConfig, myIndex, isRecipientConfig);
                              })
             .status_with_transitional_ignore();
         return;
@@ -739,7 +934,7 @@ void ReplicationCoordinatorImpl::_heartbeatReconfigFinish(
     // "kConfigReconfiguring" which prevents new elections from happening.
     {
         stdx::lock_guard<Latch> lk(_mutex);
-        if (auto electionFinishedEvent = _cancelElectionIfNeeded_inlock()) {
+        if (auto electionFinishedEvent = _cancelElectionIfNeeded(lk)) {
             LOGV2_FOR_HEARTBEATS(4615629,
                                  0,
                                  "Waiting for election to complete before finishing reconfig to "
@@ -751,7 +946,8 @@ void ReplicationCoordinatorImpl::_heartbeatReconfigFinish(
             _replExecutor
                 ->onEvent(electionFinishedEvent,
                           [=](const executor::TaskExecutor::CallbackArgs& cbData) {
-                              _heartbeatReconfigFinish(cbData, newConfig, myIndex);
+                              _heartbeatReconfigFinish(
+                                  cbData, newConfig, myIndex, isRecipientConfig);
                           })
                 .status_with_transitional_ignore();
             return;
@@ -766,11 +962,6 @@ void ReplicationCoordinatorImpl::_heartbeatReconfigFinish(
         _topCoord->prepareForUnconditionalStepDown();
         lk.unlock();
 
-        // To prevent a deadlock between session checkout and RSTL lock taking, disallow new
-        // sessions from being checked out. Existing sessions currently checked out will be killed
-        // by the killOpThread.
-        ScopedBlockSessionCheckouts blockSessions(opCtx.get());
-
         // Primary node will be either unelectable or removed after the configuration change.
         // So, finish the reconfig under RSTL, so that the step down occurs safely.
         arsd.emplace(
@@ -809,7 +1000,7 @@ void ReplicationCoordinatorImpl::_heartbeatReconfigFinish(
     invariant(_rsConfigState == kConfigHBReconfiguring);
     invariant(!_rsConfig.isInitialized() ||
               _rsConfig.getConfigVersionAndTerm() < newConfig.getConfigVersionAndTerm() ||
-              _selfIndex < 0);
+              _selfIndex < 0 || isRecipientConfig);
 
     if (!myIndex.isOK()) {
         switch (myIndex.getStatus().code()) {
@@ -839,33 +1030,24 @@ void ReplicationCoordinatorImpl::_heartbeatReconfigFinish(
         }
         myIndex = StatusWith<int>(-1);
     }
-    const ReplSetConfig oldConfig = _rsConfig;
+    const bool contentChanged = !sameConfigContents(_rsConfig, newConfig);
     // If we do not have an index, we should pass -1 as our index to avoid falsely adding ourself to
     // the data structures inside of the TopologyCoordinator.
     const int myIndexValue = myIndex.getStatus().isOK() ? myIndex.getValue() : -1;
 
-    // If we were initial syncing and are now REMOVED, we must cancel the initial sync before
-    // installing the new config, to avoid triggering a storage invariant that has initial
-    // sync as a special case.  We'll still fatally fail, but with a more meaningful error.
-    if (_initialSyncer && _memberState.startup2() && myIndexValue == -1) {
-        LOGV2_FATAL(4848002,
-                    "Initial sync failed due to node being removed from the configuration. "
-                    "Shutting down now. Restart the server to attempt a new initial sync");
-    }
-
     const PostMemberStateUpdateAction action =
         _setCurrentRSConfig(lk, opCtx.get(), newConfig, myIndexValue);
 
     lk.unlock();
+    if (contentChanged) {
+        _externalState->notifyOtherMemberDataChanged();
+    }
     _performPostMemberStateUpdateAction(action);
     if (MONGO_unlikely(waitForPostActionCompleteInHbReconfig.shouldFail())) {
         // Used in tests that wait for the post member state update action to complete.
         // eg. Closing connections upon being removed.
         LOGV2(5286701, "waitForPostActionCompleteInHbReconfig failpoint enabled");
     }
-
-    // Inform the index builds coordinator of the replica set reconfig.
-    IndexBuildsCoordinator::get(opCtx.get())->onReplicaSetReconfig();
 }
 
 void ReplicationCoordinatorImpl::_trackHeartbeatHandle_inlock(
@@ -902,12 +1084,17 @@ void ReplicationCoordinatorImpl::_cancelHeartbeats_inlock() {
     // Heartbeat callbacks will remove themselves from _heartbeatHandles when they execute with
     // CallbackCanceled status, so it's better to leave the handles in the list, for now.
 
-    if (_handleLivenessTimeoutCbh.isValid()) {
-        _replExecutor->cancel(_handleLivenessTimeoutCbh);
-    }
+    _handleLivenessTimeoutCallback.cancel();
 }
 
-void ReplicationCoordinatorImpl::_restartScheduledHeartbeats_inlock() {
+void ReplicationCoordinatorImpl::restartScheduledHeartbeats_forTest() {
+    stdx::unique_lock<Latch> lk(_mutex);
+    invariant(getTestCommandsEnabled());
+    _restartScheduledHeartbeats_inlock(_rsConfig.getReplSetName().toString());
+};
+
+void ReplicationCoordinatorImpl::_restartScheduledHeartbeats_inlock(
+    const std::string& replSetName) {
     LOGV2_FOR_HEARTBEATS(5031800, 2, "Restarting all scheduled heartbeats");
 
     const Date_t now = _replExecutor->now();
@@ -928,7 +1115,7 @@ void ReplicationCoordinatorImpl::_restartScheduledHeartbeats_inlock() {
     }
 
     for (auto target : restartedTargets) {
-        _scheduleHeartbeatToTarget_inlock(target, now);
+        _scheduleHeartbeatToTarget_inlock(target, now, replSetName);
         _topCoord->restartHeartbeat(now, target);
     }
 }
@@ -942,20 +1129,16 @@ void ReplicationCoordinatorImpl::_startHeartbeats_inlock() {
             continue;
         }
         auto target = _rsConfig.getMemberAt(i).getHostAndPort();
-        _scheduleHeartbeatToTarget_inlock(target, now);
+        _scheduleHeartbeatToTarget_inlock(target, now, _rsConfig.getReplSetName().toString());
         _topCoord->restartHeartbeat(now, target);
     }
 
-    _scheduleNextLivenessUpdate_inlock();
+    _scheduleNextLivenessUpdate_inlock(/* reschedule = */ false);
 }
 
 void ReplicationCoordinatorImpl::_handleLivenessTimeout(
     const executor::TaskExecutor::CallbackArgs& cbData) {
     stdx::unique_lock<Latch> lk(_mutex);
-    // Only reset the callback handle if it matches, otherwise more will be coming through
-    if (cbData.myHandle == _handleLivenessTimeoutCbh) {
-        _handleLivenessTimeoutCbh = CallbackHandle();
-    }
     if (!cbData.status.isOK()) {
         return;
     }
@@ -965,12 +1148,12 @@ void ReplicationCoordinatorImpl::_handleLivenessTimeout(
     // Don't mind potential asynchronous stepdown as this is the last step of
     // liveness check.
     lk = _handleHeartbeatResponseAction_inlock(
-        action, makeStatusWith<ReplSetHeartbeatResponse>(), std::move(lk));
+        action, StatusWith(ReplSetHeartbeatResponse()), std::move(lk));
 
-    _scheduleNextLivenessUpdate_inlock();
+    _scheduleNextLivenessUpdate_inlock(/* reschedule = */ false);
 }
 
-void ReplicationCoordinatorImpl::_scheduleNextLivenessUpdate_inlock() {
+void ReplicationCoordinatorImpl::_scheduleNextLivenessUpdate_inlock(bool reschedule) {
     // Scan liveness table for earliest date; schedule a run at (that date plus election
     // timeout).
     Date_t earliestDate;
@@ -983,7 +1166,7 @@ void ReplicationCoordinatorImpl::_scheduleNextLivenessUpdate_inlock() {
         return;
     }
 
-    if (_handleLivenessTimeoutCbh.isValid() && !_handleLivenessTimeoutCbh.isCanceled()) {
+    if (!reschedule && _handleLivenessTimeoutCallback.isActive()) {
         // don't bother to schedule; one is already scheduled and pending.
         return;
     }
@@ -996,31 +1179,21 @@ void ReplicationCoordinatorImpl::_scheduleNextLivenessUpdate_inlock() {
                 "nextTimeout"_attr = nextTimeout);
 
     // It is possible we will schedule the next timeout in the past.
-    // ThreadPoolTaskExecutor::_scheduleWorkAt() schedules its work immediately if it's given a
-    // time <= now().
+    // DelayableTimeoutCallback schedules its work immediately if it's given a time <= now().
     // If we missed the timeout, it means that on our last check the earliest live member was
     // just barely fresh and it has become stale since then. We must schedule another liveness
     // check to continue conducting liveness checks and be able to step down from primary if we
     // lose contact with a majority of nodes.
-    auto cbh =
-        _scheduleWorkAt(nextTimeout, [=](const executor::TaskExecutor::CallbackArgs& cbData) {
-            _handleLivenessTimeout(cbData);
-        });
-    if (!cbh) {
-        return;
-    }
-    _handleLivenessTimeoutCbh = cbh;
+    // We ignore shutdown errors; any other error triggers an fassert.
+    _handleLivenessTimeoutCallback.delayUntil(nextTimeout).ignore();
     _earliestMemberId = earliestMemberId.getData();
 }
 
-void ReplicationCoordinatorImpl::_cancelAndRescheduleLivenessUpdate_inlock(int updatedMemberId) {
+void ReplicationCoordinatorImpl::_rescheduleLivenessUpdate_inlock(int updatedMemberId) {
     if ((_earliestMemberId != -1) && (_earliestMemberId != updatedMemberId)) {
         return;
     }
-    if (_handleLivenessTimeoutCbh.isValid()) {
-        _replExecutor->cancel(_handleLivenessTimeoutCbh);
-    }
-    _scheduleNextLivenessUpdate_inlock();
+    _scheduleNextLivenessUpdate_inlock(/* reschedule = */ true);
 }
 
 void ReplicationCoordinatorImpl::_cancelPriorityTakeover_inlock() {
@@ -1054,7 +1227,8 @@ void ReplicationCoordinatorImpl::_cancelAndRescheduleElectionTimeout_inlock() {
     // the logs.
     int cancelAndRescheduleLogLevel = 5;
     static auto logThrottleTime = _replExecutor->now();
-    const bool wasActive = _handleElectionTimeoutCbh.isValid();
+    auto oldWhen = _handleElectionTimeoutCallback.getNextCall();
+    const bool wasActive = oldWhen != Date_t();
     auto now = _replExecutor->now();
     const bool doNotReschedule = _inShutdown || !_memberState.secondary() || _selfIndex < 0 ||
         !_rsConfig.getMemberAt(_selfIndex).isElectable();
@@ -1063,48 +1237,40 @@ void ReplicationCoordinatorImpl::_cancelAndRescheduleElectionTimeout_inlock() {
         cancelAndRescheduleLogLevel = 4;
         logThrottleTime = now;
     }
-    if (wasActive) {
+    if (wasActive && doNotReschedule) {
         LOGV2_FOR_ELECTION(4615649,
                            cancelAndRescheduleLogLevel,
                            "Canceling election timeout callback at {when}",
                            "Canceling election timeout callback",
-                           "when"_attr = _handleElectionTimeoutWhen);
-        _replExecutor->cancel(_handleElectionTimeoutCbh);
-        _handleElectionTimeoutCbh = CallbackHandle();
-        _handleElectionTimeoutWhen = Date_t();
+                           "when"_attr = oldWhen);
+        _handleElectionTimeoutCallback.cancel();
     }
 
     if (doNotReschedule)
         return;
 
-    Milliseconds randomOffset = _getRandomizedElectionOffset_inlock();
-    auto when = now + _rsConfig.getElectionTimeoutPeriod() + randomOffset;
-    invariant(when > now);
+    Milliseconds upperBound = Milliseconds(_getElectionOffsetUpperBound_inlock());
+    auto requestedWhen = now + _rsConfig.getElectionTimeoutPeriod();
+    invariant(requestedWhen > now);
+    Status delayStatus =
+        _handleElectionTimeoutCallback.delayUntilWithJitter(requestedWhen, upperBound);
+    Date_t when = _handleElectionTimeoutCallback.getNextCall();
     if (wasActive) {
         // The log level here is 4 once per second, otherwise 5.
         LOGV2_FOR_ELECTION(4615650,
                            cancelAndRescheduleLogLevel,
-                           "Rescheduling election timeout callback at {when}",
-                           "Rescheduling election timeout callback",
-                           "when"_attr = when);
+                           "Rescheduled election timeout callback",
+                           "when"_attr = when,
+                           "requestedWhen"_attr = requestedWhen,
+                           "error"_attr = delayStatus);
     } else {
         LOGV2_FOR_ELECTION(4615651,
                            4,
-                           "Scheduling election timeout callback at {when}",
-                           "Scheduling election timeout callback",
-                           "when"_attr = when);
+                           "Scheduled election timeout callback",
+                           "when"_attr = when,
+                           "requestedWhen"_attr = requestedWhen,
+                           "error"_attr = delayStatus);
     }
-    _handleElectionTimeoutWhen = when;
-    _handleElectionTimeoutCbh =
-        _scheduleWorkAt(when, [=](const mongo::executor::TaskExecutor::CallbackArgs& cbData) {
-            stdx::lock_guard<Latch> lk(_mutex);
-            if (_handleElectionTimeoutCbh == cbData.myHandle) {
-                // This lets _cancelAndRescheduleElectionTimeout_inlock know the callback
-                // has happened.
-                _handleElectionTimeoutCbh = CallbackHandle();
-            }
-            _startElectSelfIfEligibleV1(lk, StartElectionReasonEnum::kElectionTimeout);
-        });
 }
 
 void ReplicationCoordinatorImpl::_startElectSelfIfEligibleV1(StartElectionReasonEnum reason) {
@@ -1112,7 +1278,7 @@ void ReplicationCoordinatorImpl::_startElectSelfIfEligibleV1(StartElectionReason
     _startElectSelfIfEligibleV1(lock, reason);
 }
 
-void ReplicationCoordinatorImpl::_startElectSelfIfEligibleV1(WithLock,
+void ReplicationCoordinatorImpl::_startElectSelfIfEligibleV1(WithLock lk,
                                                              StartElectionReasonEnum reason) {
     // If it is not a single node replica set, no need to start an election after stepdown timeout.
     if (reason == StartElectionReasonEnum::kSingleNodePromptElection &&
@@ -1216,7 +1382,10 @@ void ReplicationCoordinatorImpl::_startElectSelfIfEligibleV1(WithLock,
             MONGO_UNREACHABLE;
     }
 
-    _startElectSelfV1_inlock(reason);
+    invariant(!_electionState);
+
+    _electionState = std::make_unique<ElectionState>(this);
+    _electionState->start(lk, reason);
 }
 
 }  // namespace repl
