@@ -1,5 +1,5 @@
 /**
- *    Copyright (C) 2019 MongoDB, Inc.
+ *    Copyright (C) 2019-present MongoDB, Inc.
  *
  *    This program is free software: you can redistribute it and/or modify
  *    it under the terms of the Server Side Public License, version 1,
@@ -37,14 +37,20 @@ namespace repl {
 ReplicationCoordinatorNoOp::ReplicationCoordinatorNoOp(ServiceContext* service)
     : _service(service) {}
 
-void ReplicationCoordinatorNoOp::startup(
-    OperationContext* opCtx, LastStorageEngineShutdownState lastStorageEngineShutdownState) {}
+void ReplicationCoordinatorNoOp::startup(OperationContext* opCtx,
+                                         StorageEngine::LastShutdownState lastShutdownState) {}
 
 void ReplicationCoordinatorNoOp::enterTerminalShutdown() {}
 
-void ReplicationCoordinatorNoOp::shutdown(OperationContext* opCtx) {}
+bool ReplicationCoordinatorNoOp::enterQuiesceModeIfSecondary(Milliseconds quiesceTime) {
+    MONGO_UNREACHABLE;
+}
 
-void ReplicationCoordinatorNoOp::markAsCleanShutdownIfPossible(OperationContext* opCtx) {}
+bool ReplicationCoordinatorNoOp::inQuiesceMode() const {
+    MONGO_UNREACHABLE;
+}
+
+void ReplicationCoordinatorNoOp::shutdown(OperationContext* opCtx) {}
 
 ReplicationCoordinator::Mode ReplicationCoordinatorNoOp::getReplicationMode() const {
     return modeReplSet;
@@ -66,7 +72,7 @@ const ReplSettings& ReplicationCoordinatorNoOp::getSettings() const {
     MONGO_UNREACHABLE;
 }
 
-bool ReplicationCoordinatorNoOp::isMasterForReportingPurposes() {
+bool ReplicationCoordinatorNoOp::isWritablePrimaryForReportingPurposes() {
     MONGO_UNREACHABLE;
 }
 
@@ -92,13 +98,13 @@ bool ReplicationCoordinatorNoOp::canAcceptWritesFor(OperationContext* opCtx,
 
 Status ReplicationCoordinatorNoOp::checkCanServeReadsFor_UNSAFE(OperationContext* opCtx,
                                                                 const NamespaceString& ns,
-                                                                bool slaveOk) {
+                                                                bool secondaryOkay) {
     MONGO_UNREACHABLE;
 }
 
 Status ReplicationCoordinatorNoOp::checkCanServeReadsFor(OperationContext* opCtx,
                                                          const NamespaceString& ns,
-                                                         bool slaveOk) {
+                                                         bool secondaryOkay) {
     MONGO_UNREACHABLE;
 }
 
@@ -127,7 +133,8 @@ bool ReplicationCoordinatorNoOp::buildsIndexes() {
     MONGO_UNREACHABLE;
 }
 
-OpTimeAndWallTime ReplicationCoordinatorNoOp::getMyLastAppliedOpTimeAndWallTime() const {
+OpTimeAndWallTime ReplicationCoordinatorNoOp::getMyLastAppliedOpTimeAndWallTime(
+    bool rollbackSafe) const {
     MONGO_UNREACHABLE;
 }
 
@@ -140,9 +147,6 @@ OpTime ReplicationCoordinatorNoOp::getCurrentCommittedSnapshotOpTime() const {
     MONGO_UNREACHABLE;
 }
 
-OpTimeAndWallTime ReplicationCoordinatorNoOp::getCurrentCommittedSnapshotOpTimeAndWallTime() const {
-    MONGO_UNREACHABLE;
-}
 void ReplicationCoordinatorNoOp::appendDiagnosticBSON(mongo::BSONObjBuilder*) {
     MONGO_UNREACHABLE;
 }
@@ -159,15 +163,15 @@ bool ReplicationCoordinatorNoOp::canAcceptNonLocalWrites() const {
     MONGO_UNREACHABLE;
 }
 
-Status ReplicationCoordinatorNoOp::waitForMemberState(MemberState, Milliseconds) {
+Status ReplicationCoordinatorNoOp::waitForMemberState(Interruptible*, MemberState, Milliseconds) {
     MONGO_UNREACHABLE;
 }
 
-Seconds ReplicationCoordinatorNoOp::getSlaveDelaySecs() const {
+Seconds ReplicationCoordinatorNoOp::getSecondaryDelaySecs() const {
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::clearSyncSourceBlacklist() {
+void ReplicationCoordinatorNoOp::clearSyncSourceDenylist() {
     MONGO_UNREACHABLE;
 }
 
@@ -183,7 +187,7 @@ ReplicationCoordinator::ApplierState ReplicationCoordinatorNoOp::getApplierState
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::signalDrainComplete(OperationContext*, long long) {
+void ReplicationCoordinatorNoOp::signalDrainComplete(OperationContext*, long long) noexcept {
     MONGO_UNREACHABLE;
 }
 
@@ -195,8 +199,8 @@ void ReplicationCoordinatorNoOp::setMyHeartbeatMessage(const std::string&) {
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::setMyLastAppliedOpTimeAndWallTimeForward(const OpTimeAndWallTime&,
-                                                                          DataConsistency) {
+void ReplicationCoordinatorNoOp::setMyLastAppliedOpTimeAndWallTimeForward(
+    const OpTimeAndWallTime&) {
     MONGO_UNREACHABLE;
 }
 
@@ -225,6 +229,12 @@ OpTime ReplicationCoordinatorNoOp::getMyLastDurableOpTime() const {
     MONGO_UNREACHABLE;
 }
 
+Status ReplicationCoordinatorNoOp::waitUntilMajorityOpTime(OperationContext* opCtx,
+                                                           OpTime targetOpTime,
+                                                           boost::optional<Date_t> deadline) {
+    MONGO_UNREACHABLE;
+}
+
 Status ReplicationCoordinatorNoOp::waitUntilOpTimeForRead(OperationContext*,
                                                           const ReadConcernArgs& readConcern) {
     MONGO_UNREACHABLE;
@@ -245,6 +255,11 @@ ReplicationCoordinator::StatusAndDuration ReplicationCoordinatorNoOp::awaitRepli
     MONGO_UNREACHABLE;
 }
 
+SharedSemiFuture<void> ReplicationCoordinatorNoOp::awaitReplicationAsyncNoWTimeout(
+    const OpTime& opTime, const WriteConcernOptions& writeConcern) {
+    MONGO_UNREACHABLE;
+}
+
 void ReplicationCoordinatorNoOp::stepDown(OperationContext*,
                                           const bool,
                                           const Milliseconds&,
@@ -264,20 +279,17 @@ HostAndPort ReplicationCoordinatorNoOp::getMyHostAndPort() const {
     MONGO_UNREACHABLE;
 }
 
-Status ReplicationCoordinatorNoOp::resyncData(OperationContext*, bool) {
-    MONGO_UNREACHABLE;
-}
-
 StatusWith<BSONObj> ReplicationCoordinatorNoOp::prepareReplSetUpdatePositionCommand() const {
     MONGO_UNREACHABLE;
 }
 
-Status ReplicationCoordinatorNoOp::processReplSetGetStatus(BSONObjBuilder*,
+Status ReplicationCoordinatorNoOp::processReplSetGetStatus(OperationContext* opCtx,
+                                                           BSONObjBuilder*,
                                                            ReplSetGetStatusResponseStyle) {
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::appendSlaveInfoData(BSONObjBuilder*) {
+void ReplicationCoordinatorNoOp::appendSecondaryInfoData(BSONObjBuilder*) {
     MONGO_UNREACHABLE;
 }
 
@@ -285,8 +297,63 @@ ReplSetConfig ReplicationCoordinatorNoOp::getConfig() const {
     MONGO_UNREACHABLE;
 }
 
+ConnectionString ReplicationCoordinatorNoOp::getConfigConnectionString() const {
+    MONGO_UNREACHABLE;
+}
+
+Milliseconds ReplicationCoordinatorNoOp::getConfigElectionTimeoutPeriod() const {
+    MONGO_UNREACHABLE;
+}
+
+std::vector<MemberConfig> ReplicationCoordinatorNoOp::getConfigVotingMembers() const {
+    MONGO_UNREACHABLE;
+}
+
+std::int64_t ReplicationCoordinatorNoOp::getConfigTerm() const {
+    MONGO_UNREACHABLE;
+}
+
+std::int64_t ReplicationCoordinatorNoOp::getConfigVersion() const {
+    MONGO_UNREACHABLE;
+}
+
+ConfigVersionAndTerm ReplicationCoordinatorNoOp::getConfigVersionAndTerm() const {
+    MONGO_UNREACHABLE;
+}
+
+int ReplicationCoordinatorNoOp::getConfigNumMembers() const {
+    MONGO_UNREACHABLE;
+}
+
+Milliseconds ReplicationCoordinatorNoOp::getConfigHeartbeatTimeoutPeriodMillis() const {
+    MONGO_UNREACHABLE;
+}
+
+BSONObj ReplicationCoordinatorNoOp::getConfigBSON() const {
+    MONGO_UNREACHABLE;
+}
+
+const MemberConfig* ReplicationCoordinatorNoOp::findConfigMemberByHostAndPort(
+    const HostAndPort& hap) const {
+    MONGO_UNREACHABLE;
+}
+
+bool ReplicationCoordinatorNoOp::isConfigLocalHostAllowed() const {
+    MONGO_UNREACHABLE;
+}
+
+Milliseconds ReplicationCoordinatorNoOp::getConfigHeartbeatInterval() const {
+    MONGO_UNREACHABLE;
+}
+
+Status ReplicationCoordinatorNoOp::validateWriteConcern(
+    const WriteConcernOptions& writeConcern) const {
+    MONGO_UNREACHABLE;
+}
+
 void ReplicationCoordinatorNoOp::processReplSetGetConfig(BSONObjBuilder* result,
-                                                         bool commitmentStatus) {
+                                                         bool commitmentStatus,
+                                                         bool includeNewlyAdded) {
     MONGO_UNREACHABLE;
 }
 
@@ -302,6 +369,11 @@ Status ReplicationCoordinatorNoOp::setMaintenanceMode(OperationContext*, bool) {
     MONGO_UNREACHABLE;
 }
 
+ChangeSyncSourceAction ReplicationCoordinatorNoOp::shouldChangeSyncSourceOnError(
+    const HostAndPort&, const OpTime&) const {
+    MONGO_UNREACHABLE;
+}
+
 Status ReplicationCoordinatorNoOp::processReplSetSyncFrom(OperationContext*,
                                                           const HostAndPort&,
                                                           BSONObjBuilder*) {
@@ -324,6 +396,11 @@ Status ReplicationCoordinatorNoOp::doReplSetReconfig(OperationContext* opCtx,
     MONGO_UNREACHABLE;
 }
 
+Status ReplicationCoordinatorNoOp::doOptimizedReconfig(OperationContext* opCtx,
+                                                       GetNewConfigFn getNewConfig) {
+    MONGO_UNREACHABLE;
+}
+
 Status ReplicationCoordinatorNoOp::awaitConfigCommitment(OperationContext* opCtx,
                                                          bool waitForOplogCommitment) {
     MONGO_UNREACHABLE;
@@ -343,8 +420,7 @@ void ReplicationCoordinatorNoOp::incrementNumCatchUpOpsIfCatchingUp(long numOps)
     MONGO_UNREACHABLE;
 }
 
-Status ReplicationCoordinatorNoOp::processReplSetUpdatePosition(const UpdatePositionArgs&,
-                                                                long long*) {
+Status ReplicationCoordinatorNoOp::processReplSetUpdatePosition(const UpdatePositionArgs&) {
     MONGO_UNREACHABLE;
 }
 
@@ -375,17 +451,24 @@ HostAndPort ReplicationCoordinatorNoOp::chooseNewSyncSource(const OpTime&) {
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::blacklistSyncSource(const HostAndPort&, Date_t) {
+void ReplicationCoordinatorNoOp::denylistSyncSource(const HostAndPort&, Date_t) {
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::resetLastOpTimesFromOplog(OperationContext*, DataConsistency) {
+void ReplicationCoordinatorNoOp::resetLastOpTimesFromOplog(OperationContext*) {
     MONGO_UNREACHABLE;
 }
 
-bool ReplicationCoordinatorNoOp::shouldChangeSyncSource(const HostAndPort&,
-                                                        const rpc::ReplSetMetadata&,
-                                                        const rpc::OplogQueryMetadata&) {
+ChangeSyncSourceAction ReplicationCoordinatorNoOp::shouldChangeSyncSource(
+    const HostAndPort&,
+    const rpc::ReplSetMetadata&,
+    const rpc::OplogQueryMetadata&,
+    const OpTime&,
+    const OpTime&) const {
+    MONGO_UNREACHABLE;
+}
+
+bool ReplicationCoordinatorNoOp::shouldDropSyncSourceAfterShardSplit(OID replicaSetId) const {
     MONGO_UNREACHABLE;
 }
 
@@ -438,7 +521,7 @@ void ReplicationCoordinatorNoOp::createWMajorityWriteAvailabilityDateWaiter(OpTi
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::dropAllSnapshots() {
+void ReplicationCoordinatorNoOp::clearCommittedSnapshot() {
     MONGO_UNREACHABLE;
 }
 
@@ -481,7 +564,7 @@ void ReplicationCoordinatorNoOp::incrementTopologyVersion() {
     MONGO_UNREACHABLE;
 }
 
-std::shared_ptr<const IsMasterResponse> ReplicationCoordinatorNoOp::awaitIsMasterResponse(
+std::shared_ptr<const HelloResponse> ReplicationCoordinatorNoOp::awaitHelloResponse(
     OperationContext* opCtx,
     const SplitHorizon::Parameters& horizonParams,
     boost::optional<TopologyVersion> clientTopologyVersion,
@@ -490,8 +573,8 @@ std::shared_ptr<const IsMasterResponse> ReplicationCoordinatorNoOp::awaitIsMaste
 }
 
 
-SharedSemiFuture<std::shared_ptr<const IsMasterResponse>>
-ReplicationCoordinatorNoOp::getIsMasterResponseFuture(
+SharedSemiFuture<std::shared_ptr<const HelloResponse>>
+ReplicationCoordinatorNoOp::getHelloResponseFuture(
     const SplitHorizon::Parameters& horizonParams,
     boost::optional<TopologyVersion> clientTopologyVersion) {
     MONGO_UNREACHABLE;
@@ -520,5 +603,19 @@ BSONObj ReplicationCoordinatorNoOp::runCmdOnPrimaryAndAwaitResponse(
     MONGO_UNREACHABLE;
 }
 
+void ReplicationCoordinatorNoOp::restartScheduledHeartbeats_forTest() {
+    MONGO_UNREACHABLE;
+}
+
+void ReplicationCoordinatorNoOp::recordIfCWWCIsSetOnConfigServerOnStartup(OperationContext* opCtx) {
+    MONGO_UNREACHABLE;
+}
+
+ReplicationCoordinatorNoOp::WriteConcernTagChanges*
+ReplicationCoordinatorNoOp::getWriteConcernTagChanges() {
+    MONGO_UNREACHABLE;
+    return nullptr;
+}
+
 }  // namespace repl
 }  // namespace mongo
