@@ -26,7 +26,6 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 #define LOGV2_FOR_RECOVERY(ID, DLEVEL, MESSAGE, ...) \
     LOGV2_DEBUG_OPTIONS(ID, DLEVEL, {logv2::LogComponent::kStorageRecovery}, MESSAGE, ##__VA_ARGS__)
 
@@ -46,21 +45,30 @@
 #include "mongo/db/repl/oplog_buffer.h"
 #include "mongo/db/repl/oplog_interface_local.h"
 #include "mongo/db/repl/repl_server_parameters_gen.h"
+#include "mongo/db/repl/replica_set_aware_service.h"
 #include "mongo/db/repl/replication_consistency_markers_impl.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/transaction_oplog_application.h"
 #include "mongo/db/server_recovery.h"
-#include "mongo/db/session.h"
+#include "mongo/db/session/session.h"
 #include "mongo/db/storage/control/journal_flusher.h"
+#include "mongo/db/storage/durable_history_pin.h"
 #include "mongo/db/storage/storage_parameters_gen.h"
-#include "mongo/db/transaction_history_iterator.h"
-#include "mongo/db/transaction_participant.h"
+#include "mongo/db/transaction/transaction_history_iterator.h"
+#include "mongo/db/transaction/transaction_participant.h"
 #include "mongo/logv2/log.h"
+#include "mongo/util/scopeguard.h"
 #include "mongo/util/timer.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
+
+
 namespace mongo {
 namespace repl {
 
+MONGO_FAIL_POINT_DEFINE(hangAfterOplogTruncationInRollback);
+MONGO_FAIL_POINT_DEFINE(skipResettingValidateFeaturesAsPrimaryAfterRecoveryOplogApplication);
+
 namespace {
 
 const auto kRecoveryBatchLogLevel = logv2::LogSeverity::Debug(2);
@@ -99,7 +107,7 @@ public:
                                    "opIndex"_attr = i,
                                    "batchSize"_attr = batch.size(),
                                    "numBatches"_attr = _numBatches,
-                                   "oplogEntry"_attr = redact(entry.getRaw()));
+                                   "oplogEntry"_attr = redact(entry.toBSONForLogging()));
             }
         }
     }
@@ -140,20 +148,16 @@ public:
         BSONObj predicate = _oplogApplicationEndPoint
             ? BSON("$gte" << _oplogApplicationStartPoint << "$lte" << *_oplogApplicationEndPoint)
             : BSON("$gte" << _oplogApplicationStartPoint);
-        _cursor = _client->query(NamespaceString::kRsOplogNamespace,
-                                 QUERY("ts" << predicate),
-                                 /*batchSize*/ 0,
-                                 /*skip*/ 0,
-                                 /*projection*/ nullptr,
-                                 QueryOption_OplogReplay);
+        FindCommandRequest findRequest{NamespaceString::kRsOplogNamespace};
+        findRequest.setFilter(BSON("ts" << predicate));
+        _cursor = _client->find(std::move(findRequest));
 
         // Check that the first document matches our appliedThrough point then skip it since it's
         // already been applied.
         if (!_cursor->more()) {
             // This should really be impossible because we check above that the top of the oplog is
             // strictly > appliedThrough. If this fails it represents a serious bug in either the
-            // storage engine or query's implementation of OplogReplay.
-
+            // storage engine or query's implementation of the oplog scan.
             logv2::DynamicAttributes attrs;
             attrs.add("oplogApplicationStartPoint", _oplogApplicationStartPoint.toBSON());
             if (_oplogApplicationEndPoint) {
@@ -212,7 +216,10 @@ public:
     void clear(OperationContext*) final {
         MONGO_UNREACHABLE;
     }
-    bool waitForData(Seconds) final {
+    bool waitForDataFor(Milliseconds, Interruptible*) final {
+        MONGO_UNREACHABLE;
+    }
+    bool waitForDataUntil(Date_t, Interruptible*) final {
         MONGO_UNREACHABLE;
     }
     boost::optional<Value> lastObjectPushed(OperationContext*) const final {
@@ -318,9 +325,15 @@ void ReplicationRecoveryImpl::_assertNoRecoveryNeededOnUnstableCheckpoint(Operat
     }
 }
 
-void ReplicationRecoveryImpl::recoverFromOplogAsStandalone(OperationContext* opCtx) {
+void ReplicationRecoveryImpl::recoverFromOplogAsStandalone(OperationContext* opCtx,
+                                                           bool duringInitialSync) {
+    _duringInitialSync = duringInitialSync;
+    ScopeGuard resetInitialSyncFlagOnExit([this] { _duringInitialSync = false; });
     auto recoveryTS = recoverFromOplogPrecursor(opCtx, _storageInterface);
 
+    // We support only recovery from stable checkpoints during initial sync.
+    invariant(!_duringInitialSync || recoveryTS);
+
     // Initialize the cached pointer to the oplog collection.
     acquireOplogCollectionForLogging(opCtx);
 
@@ -353,15 +366,9 @@ void ReplicationRecoveryImpl::recoverFromOplogAsStandalone(OperationContext* opC
         }
     }
 
-    reconstructPreparedTransactions(opCtx, OplogApplication::Mode::kRecovering);
-
-    // Two-phase index builds are built in the background, which may still be in-progress after
-    // recovering from the oplog. To prevent crashing the server, skip enabling read-only mode.
-    if (IndexBuildsCoordinator::get(opCtx)->noIndexBuildInProgress()) {
-        LOGV2_WARNING(21558,
-                      "Setting mongod to readOnly mode as a result of specifying "
-                      "'recoverFromOplogAsStandalone'");
-        storageGlobalParams.readOnly = true;
+    if (!_duringInitialSync) {
+        // Initial sync will reconstruct prepared transactions when it is completely done.
+        reconstructPreparedTransactions(opCtx, OplogApplication::Mode::kRecovering);
     }
 }
 
@@ -381,7 +388,7 @@ void ReplicationRecoveryImpl::recoverFromOplogUpTo(OperationContext* opCtx, Time
     acquireOplogCollectionForLogging(opCtx);
 
     // This may take an IS lock on the oplog collection.
-    _truncateOplogIfNeededAndThenClearOplogTruncateAfterPoint(opCtx, recoveryTS);
+    _truncateOplogIfNeededAndThenClearOplogTruncateAfterPoint(opCtx, &recoveryTS);
 
     boost::optional<Timestamp> startPoint =
         _storageInterface->getRecoveryTimestamp(opCtx->getServiceContext());
@@ -389,6 +396,8 @@ void ReplicationRecoveryImpl::recoverFromOplogUpTo(OperationContext* opCtx, Time
         fassert(31436, "No recovery timestamp, cannot recover from the oplog");
     }
 
+    startPoint = _adjustStartPointIfNecessary(opCtx, startPoint.value());
+
     invariant(!endPoint.isNull());
 
     if (*startPoint == endPoint) {
@@ -453,18 +462,16 @@ void ReplicationRecoveryImpl::recoverFromOplog(OperationContext* opCtx,
         stableTimestamp = _storageInterface->getRecoveryTimestamp(opCtx->getServiceContext());
     }
 
-    const auto appliedThrough = _consistencyMarkers->getAppliedThrough(opCtx);
-    if (assertStableTimestampEqualsAppliedThroughOnRecovery) {
-        invariant(!stableTimestamp || stableTimestamp->isNull() || appliedThrough.isNull() ||
-                      *stableTimestamp == appliedThrough.getTimestamp(),
-                  str::stream() << "Stable timestamp " << stableTimestamp->toString()
-                                << " does not equal appliedThrough timestamp "
-                                << appliedThrough.toString());
-    }
-
-
     // This may take an IS lock on the oplog collection.
-    _truncateOplogIfNeededAndThenClearOplogTruncateAfterPoint(opCtx, stableTimestamp);
+    _truncateOplogIfNeededAndThenClearOplogTruncateAfterPoint(opCtx, &stableTimestamp);
+
+    hangAfterOplogTruncationInRollback.pauseWhileSet();
+
+    // Truncation may need to adjust the initialDataTimestamp so we let it complete first.
+    if (!isRollbackRecovery) {
+        ReplicaSetAwareServiceRegistry::get(getGlobalServiceContext())
+            .onInitialDataAvailable(opCtx, true /* isMajorityDataAvailable */);
+    }
 
     auto topOfOplogSW = _getTopOfOplog(opCtx);
     if (topOfOplogSW.getStatus() == ErrorCodes::CollectionIsEmpty ||
@@ -481,10 +488,10 @@ void ReplicationRecoveryImpl::recoverFromOplog(OperationContext* opCtx,
         invariant(supportsRecoveryTimestamp);
         const auto recoveryMode = isRollbackRecovery ? RecoveryMode::kRollbackFromStableTimestamp
                                                      : RecoveryMode::kStartupFromStableTimestamp;
-        _recoverFromStableTimestamp(
-            opCtx, *stableTimestamp, appliedThrough, topOfOplog, recoveryMode);
+        _recoverFromStableTimestamp(opCtx, *stableTimestamp, topOfOplog, recoveryMode);
     } else {
-        _recoverFromUnstableCheckpoint(opCtx, appliedThrough, topOfOplog);
+        _recoverFromUnstableCheckpoint(
+            opCtx, _consistencyMarkers->getAppliedThrough(opCtx), topOfOplog);
     }
 } catch (...) {
     LOGV2_FATAL_CONTINUE(21570,
@@ -496,42 +503,48 @@ void ReplicationRecoveryImpl::recoverFromOplog(OperationContext* opCtx,
 
 void ReplicationRecoveryImpl::_recoverFromStableTimestamp(OperationContext* opCtx,
                                                           Timestamp stableTimestamp,
-                                                          OpTime appliedThrough,
                                                           OpTime topOfOplog,
                                                           RecoveryMode recoveryMode) {
     invariant(!stableTimestamp.isNull());
     invariant(!topOfOplog.isNull());
 
-    const auto truncateAfterPoint = _consistencyMarkers->getOplogTruncateAfterPoint(opCtx);
-
     LOGV2(21544,
           "Recovering from stable timestamp: {stableTimestamp} (top of oplog: {topOfOplog}, "
           "appliedThrough: {appliedThrough}, TruncateAfter: {oplogTruncateAfterPoint})",
           "Recovering from stable timestamp",
           "stableTimestamp"_attr = stableTimestamp,
           "topOfOplog"_attr = topOfOplog,
-          "appliedThrough"_attr = appliedThrough,
-          "oplogTruncateAfterPoint"_attr = truncateAfterPoint);
+          "appliedThrough"_attr = _consistencyMarkers->getAppliedThrough(opCtx));
 
     LOGV2(21545,
           "Starting recovery oplog application at the stable timestamp: {stableTimestamp}",
           "Starting recovery oplog application at the stable timestamp",
           "stableTimestamp"_attr = stableTimestamp);
 
-    if (recoveryMode == RecoveryMode::kStartupFromStableTimestamp && startupRecoveryForRestore) {
-        LOGV2_WARNING(5576600,
-                      "Replication startup parameter 'startupRecoveryForRestore' is set, "
-                      "recovering without preserving history before top of oplog.");
+    if (recoveryMode == RecoveryMode::kStartupFromStableTimestamp &&
+        (startupRecoveryForRestore || _duringInitialSync)) {
+        if (startupRecoveryForRestore) {
+            LOGV2_WARNING(5576600,
+                          "Replication startup parameter 'startupRecoveryForRestore' is set, "
+                          "recovering without preserving history before top of oplog.");
+        }
         // Take only unstable checkpoints during the recovery process.
         _storageInterface->setInitialDataTimestamp(opCtx->getServiceContext(),
                                                    Timestamp::kAllowUnstableCheckpointsSentinel);
         // Allow "oldest" timestamp to move forward freely.
         _storageInterface->setStableTimestamp(opCtx->getServiceContext(), Timestamp::min());
     }
-    _applyToEndOfOplog(opCtx, stableTimestamp, topOfOplog.getTimestamp(), recoveryMode);
-    if (recoveryMode == RecoveryMode::kStartupFromStableTimestamp && startupRecoveryForRestore) {
+    auto startPoint = _adjustStartPointIfNecessary(opCtx, stableTimestamp);
+    _applyToEndOfOplog(opCtx, startPoint, topOfOplog.getTimestamp(), recoveryMode);
+    if (recoveryMode == RecoveryMode::kStartupFromStableTimestamp &&
+        (startupRecoveryForRestore || _duringInitialSync)) {
         _storageInterface->setInitialDataTimestamp(opCtx->getServiceContext(),
                                                    topOfOplog.getTimestamp());
+        // Clear the appliedThrough so this reflects in the first stable checkpoint. See
+        // _recoverFromUnstableCheckpoint for details.
+        if (!gTakeUnstableCheckpointOnShutdown) {
+            _consistencyMarkers->clearAppliedThrough(opCtx);
+        }
     }
 }
 
@@ -564,13 +577,19 @@ void ReplicationRecoveryImpl::_recoverFromUnstableCheckpoint(OperationContext* o
         // When `recoverFromOplog` truncates the oplog, that also happens to set the "oldest
         // timestamp" to the truncation point[1]. `_applyToEndOfOplog` will then perform writes
         // before the truncation point. Doing so violates the constraint that all updates must be
-        // timestamped newer than the "oldest timestamp". This call will move the "oldest
+        // timestamped newer than the "oldest timestamp". So we will need to move the "oldest
         // timestamp" back to the `startPoint`.
         //
+        // Before doing so, we will remove any pins. Forcing the oldest timestamp backwards will
+        // error if there are pins in place, as those pin requests will no longer be satisfied.
+        // Recovering from an unstable checkpoint has no history in the first place. Thus, clearing
+        // pins has no real effect on history being held.
+        //
         // [1] This is arguably incorrect. On rollback for nodes that are not keeping history to
         // the "majority point", the "oldest timestamp" likely needs to go back in time. The
         // oplog's `cappedTruncateAfter` method was a convenient location for this logic, which,
         // unfortunately, conflicts with the usage above.
+        DurableHistoryRegistry::get(opCtx->getServiceContext())->clearPins(opCtx);
         opCtx->getServiceContext()->getStorageEngine()->setOldestTimestamp(
             appliedThrough.getTimestamp());
 
@@ -608,6 +627,19 @@ void ReplicationRecoveryImpl::_recoverFromUnstableCheckpoint(OperationContext* o
     // recovery timestamp does not exist, the applied through is used to determine where to start
     // playing oplog entries from.
     opCtx->recoveryUnit()->waitUntilUnjournaledWritesDurable(opCtx, /*stableCheckpoint*/ true);
+
+    // Now that we have set the initial data timestamp and taken an unstable checkpoint with the
+    // appliedThrough being the topOfOplog, it is safe to clear the appliedThrough. This minValid
+    // document write would never get into an unstable checkpoint because we will no longer take
+    // unstable checkpoints from now on, except when gTakeUnstableCheckpointOnShutdown is true in
+    // certain standalone restore cases. Additionally, future stable checkpoints are guaranteed to
+    // be taken with the appliedThrough cleared. Therefore, if this node crashes before the first
+    // stable checkpoint, it can safely recover from the last unstable checkpoint with a correct
+    // appliedThrough value. Otherwise, if this node crashes after the first stable checkpoint, it
+    // can safely recover from a stable checkpoint (with an empty appliedThrough).
+    if (!gTakeUnstableCheckpointOnShutdown) {
+        _consistencyMarkers->clearAppliedThrough(opCtx);
+    }
 }
 
 void ReplicationRecoveryImpl::_applyToEndOfOplog(OperationContext* opCtx,
@@ -645,9 +677,26 @@ Timestamp ReplicationRecoveryImpl::_applyOplogOperations(OperationContext* opCtx
                                                          const Timestamp& startPoint,
                                                          const Timestamp& endPoint,
                                                          RecoveryMode recoveryMode) {
+
+    // Make sure we skip validation checks that are only intended for primaries while recovering.
+    auto validateValue = serverGlobalParams.validateFeaturesAsPrimary.load();
+    ON_BLOCK_EXIT([validateValue] {
+        if (MONGO_unlikely(
+                skipResettingValidateFeaturesAsPrimaryAfterRecoveryOplogApplication.shouldFail())) {
+            LOGV2(5717600,
+                  "Hit skipResettingValidateFeaturesAsPrimaryAfterRecoveryOplogApplication "
+                  "failpoint");
+            return;
+        }
+        serverGlobalParams.validateFeaturesAsPrimary.store(validateValue);
+    });
+    serverGlobalParams.validateFeaturesAsPrimary.store(false);
+
+    // The oplog buffer will fetch all entries >= the startPoint timestamp, but it skips the first
+    // op on startup, which is why the startPoint is described as "exclusive".
     LOGV2(21550,
-          "Replaying stored operations from {startPoint} (inclusive) to {endPoint} (inclusive).",
-          "Replaying stored operations from startPoint (inclusive) to endPoint (inclusive)",
+          "Replaying stored operations from {startPoint} (exclusive) to {endPoint} (inclusive).",
+          "Replaying stored operations from startPoint (exclusive) to endPoint (inclusive)",
           "startPoint"_attr = startPoint,
           "endPoint"_attr = endPoint);
 
@@ -680,7 +729,7 @@ Timestamp ReplicationRecoveryImpl::_applyOplogOperations(OperationContext* opCtx
     // mode, which discards history before the top of oplog), we aren't doing new checkpoints during
     // recovery so there is no point in advancing the consistency marker and we cannot advance
     // "oldest" becaue it would be later than "stable".
-    const bool advanceTimestampsEachBatch = startupRecoveryForRestore &&
+    const bool advanceTimestampsEachBatch = (startupRecoveryForRestore || _duringInitialSync) &&
         (recoveryMode == RecoveryMode::kStartupFromStableTimestamp ||
          recoveryMode == RecoveryMode::kStartupFromUnstableCheckpoint);
 
@@ -721,7 +770,19 @@ Timestamp ReplicationRecoveryImpl::_applyOplogOperations(OperationContext* opCtx
     // It is safe to do startup recovery from an unstable checkpoint provided we recover to the
     // end of the oplog and discard history before it, as _recoverFromUnstableCheckpoint does.
     if (!advanceTimestampsEachBatch) {
-        _consistencyMarkers->setAppliedThrough(opCtx, applyThroughOpTime);
+        if (recoveryMode == RecoveryMode::kStartupFromStableTimestamp) {
+            // Advance all_durable timestamp to the last applied timestamp. This is needed because
+            // the last applied entry during recovery could be a no-op entry which doesn't do
+            // timestamped writes or advance the all_durable timestamp. We may set the stable
+            // timestamp to this last applied timestamp later and we require the stable timestamp to
+            // be less than or equal to the all_durable timestamp.
+            WriteUnitOfWork wunit(opCtx);
+            uassertStatusOK(opCtx->recoveryUnit()->setTimestamp(applyThroughOpTime.getTimestamp()));
+            opCtx->recoveryUnit()->setOrderedCommit(false);
+            wunit.commit();
+        } else {
+            _consistencyMarkers->setAppliedThrough(opCtx, applyThroughOpTime);
+        }
     }
     return applyThroughOpTime.getTimestamp();
 }
@@ -739,15 +800,16 @@ StatusWith<OpTime> ReplicationRecoveryImpl::_getTopOfOplog(OperationContext* opC
 }
 
 void ReplicationRecoveryImpl::_truncateOplogTo(OperationContext* opCtx,
-                                               Timestamp truncateAfterTimestamp) {
+                                               Timestamp truncateAfterTimestamp,
+                                               boost::optional<Timestamp>* stableTimestamp) {
     Timer timer;
 
     // Fetch the oplog collection.
     const NamespaceString oplogNss(NamespaceString::kRsOplogNamespace);
-    AutoGetDb autoDb(opCtx, oplogNss.db(), MODE_IX);
+    AutoGetDb autoDb(opCtx, oplogNss.dbName(), MODE_IX);
     Lock::CollectionLock oplogCollectionLoc(opCtx, oplogNss, MODE_X);
-    Collection* oplogCollection =
-        CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, oplogNss);
+    auto oplogCollection =
+        CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, oplogNss);
     if (!oplogCollection) {
         fassertFailedWithStatusNoTrace(
             34418,
@@ -770,15 +832,15 @@ void ReplicationRecoveryImpl::_truncateOplogTo(OperationContext* opCtx,
 
     // Parse the response.
     auto truncateAfterOpTime =
-        fassert(51766, repl::OpTime::parseFromOplogEntry(truncateAfterOplogEntryBSON.get()));
+        fassert(51766, repl::OpTime::parseFromOplogEntry(truncateAfterOplogEntryBSON.value()));
     auto truncateAfterOplogEntryTs = truncateAfterOpTime.getTimestamp();
     auto truncateAfterRecordId = RecordId(truncateAfterOplogEntryTs.asULL());
 
     invariant(truncateAfterRecordId <= RecordId(truncateAfterTimestamp.asULL()),
               str::stream() << "Should have found a oplog entry timestamp lte to "
                             << truncateAfterTimestamp.toString() << ", but instead found "
-                            << redact(truncateAfterOplogEntryBSON.get()) << " with timestamp "
-                            << Timestamp(truncateAfterRecordId.repr()).toString());
+                            << redact(truncateAfterOplogEntryBSON.value()) << " with timestamp "
+                            << Timestamp(truncateAfterRecordId.getLong()).toString());
 
     // Truncate the oplog AFTER the oplog entry found to be <= truncateAfterTimestamp.
     LOGV2(21553,
@@ -788,7 +850,36 @@ void ReplicationRecoveryImpl::_truncateOplogTo(OperationContext* opCtx,
           "truncateAfterOplogEntryTimestamp"_attr = truncateAfterOplogEntryTs,
           "oplogTruncateAfterPoint"_attr = truncateAfterTimestamp);
 
-    oplogCollection->cappedTruncateAfter(opCtx, truncateAfterRecordId, /*inclusive*/ false);
+    if (*stableTimestamp && (**stableTimestamp) > truncateAfterOplogEntryTs) {
+        // Truncating the oplog sets the storage engine's maximum durable timestamp to the new top
+        // of the oplog.  It is illegal for this maximum durable timestamp to be before the oldest
+        // timestamp, so if the oldest timestamp is ahead of that point, we need to move it back.
+        // Since the stable timestamp is never behind the oldest and also must not be ahead of the
+        // maximum durable timesatmp, it has to be moved back as well.  This usually happens when
+        // the truncateAfterTimestamp does not exist in the oplog because there was a hole open when
+        // we crashed; in that case the oldest timestamp and the stable timestamp will be the
+        // timestamp immediately prior to the hole.
+        LOGV2_DEBUG(5104900,
+                    0,
+                    "Resetting stable and oldest timestamp to oplog entry we truncate after",
+                    "stableTimestamp"_attr = *stableTimestamp,
+                    "truncateAfterRecordTimestamp"_attr = truncateAfterOplogEntryTs);
+        // We're moving the stable timestamp backwards, so we need to force it.
+        const bool force = true;
+        opCtx->getServiceContext()->getStorageEngine()->setStableTimestamp(
+            truncateAfterOplogEntryTs, force);
+        **stableTimestamp = truncateAfterOplogEntryTs;
+
+        // The initialDataTimestamp may also be at the hole; move it back.
+        auto initialDataTimestamp =
+            opCtx->getServiceContext()->getStorageEngine()->getInitialDataTimestamp();
+        if (initialDataTimestamp > truncateAfterOplogEntryTs) {
+            _storageInterface->setInitialDataTimestamp(opCtx->getServiceContext(),
+                                                       truncateAfterOplogEntryTs);
+        }
+    }
+    oplogCollection->getRecordStore()->cappedTruncateAfter(
+        opCtx, truncateAfterRecordId, false /*inclusive*/, nullptr /* aboutToDelete callback */);
 
     LOGV2(21554,
           "Replication recovery oplog truncation finished in: {durationMillis}ms",
@@ -797,7 +888,7 @@ void ReplicationRecoveryImpl::_truncateOplogTo(OperationContext* opCtx,
 }
 
 void ReplicationRecoveryImpl::_truncateOplogIfNeededAndThenClearOplogTruncateAfterPoint(
-    OperationContext* opCtx, boost::optional<Timestamp> stableTimestamp) {
+    OperationContext* opCtx, boost::optional<Timestamp>* stableTimestamp) {
 
     Timestamp truncatePoint = _consistencyMarkers->getOplogTruncateAfterPoint(opCtx);
 
@@ -806,23 +897,23 @@ void ReplicationRecoveryImpl::_truncateOplogIfNeededAndThenClearOplogTruncateAft
         return;
     }
 
-    if (stableTimestamp && !stableTimestamp->isNull() && truncatePoint <= stableTimestamp) {
+    if (*stableTimestamp && !(*stableTimestamp)->isNull() && truncatePoint <= *stableTimestamp) {
         LOGV2(21556,
               "The oplog truncation point ({truncatePoint}) is equal to or earlier than the stable "
               "timestamp ({stableTimestamp}), so truncating after the stable timestamp instead",
               "The oplog truncation point is equal to or earlier than the stable timestamp, so "
               "truncating after the stable timestamp instead",
               "truncatePoint"_attr = truncatePoint,
-              "stableTimestamp"_attr = stableTimestamp.get());
+              "stableTimestamp"_attr = (*stableTimestamp).value());
 
-        truncatePoint = stableTimestamp.get();
+        truncatePoint = (*stableTimestamp).value();
     }
 
     LOGV2(21557,
           "Removing unapplied oplog entries starting after: {oplogTruncateAfterPoint}",
           "Removing unapplied oplog entries after oplogTruncateAfterPoint",
           "oplogTruncateAfterPoint"_attr = truncatePoint.toBSON());
-    _truncateOplogTo(opCtx, truncatePoint);
+    _truncateOplogTo(opCtx, truncatePoint, stableTimestamp);
 
     // Clear the oplogTruncateAfterPoint now that we have removed any holes that might exist in the
     // oplog -- and so that we do not truncate future entries erroneously.
@@ -830,5 +921,48 @@ void ReplicationRecoveryImpl::_truncateOplogIfNeededAndThenClearOplogTruncateAft
     JournalFlusher::get(opCtx)->waitForJournalFlush();
 }
 
+Timestamp ReplicationRecoveryImpl::_adjustStartPointIfNecessary(OperationContext* opCtx,
+                                                                Timestamp startPoint) {
+    // Set up read on oplog collection.
+    AutoGetOplog oplogRead(opCtx, OplogAccessMode::kRead);
+    const auto& oplogCollection = oplogRead.getCollection();
+    if (!oplogCollection) {
+        LOGV2_FATAL_NOTRACE(
+            5466600,
+            "Cannot find oplog collection for recovery oplog application start point",
+            "oplogNss"_attr = NamespaceString::kRsOplogNamespace);
+    }
+
+    boost::optional<BSONObj> adjustmentOplogEntryBSON =
+        _storageInterface->findOplogEntryLessThanOrEqualToTimestamp(
+            opCtx, oplogCollection, startPoint);
+
+    if (!adjustmentOplogEntryBSON) {
+        LOGV2_FATAL_NOTRACE(
+            5466601,
+            "Could not find LTE oplog entry for oplog application start point for recovery",
+            "startPoint"_attr = startPoint);
+    }
+
+    auto adjustmentOpTime =
+        fassert(5466602, OpTime::parseFromOplogEntry(adjustmentOplogEntryBSON.value()));
+    auto adjustmentTimestamp = adjustmentOpTime.getTimestamp();
+
+    if (startPoint != adjustmentTimestamp) {
+        LOGV2(5466603,
+              "Start point for recovery oplog application not found in oplog. Adjusting start "
+              "point to earlier entry",
+              "oldStartPoint"_attr = startPoint,
+              "newStartPoint"_attr = adjustmentTimestamp);
+        invariant(adjustmentTimestamp < startPoint);
+        return adjustmentTimestamp;
+    }
+
+    LOGV2(5466604,
+          "Start point for recovery oplog application exists in oplog. No adjustment necessary",
+          "startPoint"_attr = startPoint);
+    return startPoint;
+}
+
 }  // namespace repl
 }  // namespace mongo
