@@ -27,25 +27,24 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplicationRollback
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/repl/rollback_impl.h"
-#include "mongo/db/repl/rollback_impl_gen.h"
 
 #include <fmt/format.h>
 
 #include "mongo/bson/util/bson_extract.h"
-#include "mongo/db/background.h"
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/database_holder.h"
+#include "mongo/db/catalog/drop_collection.h"
+#include "mongo/db/catalog/import_collection_oplog_entry_gen.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/concurrency/replication_state_transition_lock_guard.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/db_raii.h"
+#include "mongo/db/dbdirectclient.h"
+#include "mongo/db/dbhelpers.h"
 #include "mongo/db/index_builds_coordinator.h"
-#include "mongo/db/kill_sessions_local.h"
 #include "mongo/db/logical_time_validator.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/apply_ops.h"
@@ -53,19 +52,24 @@
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_process.h"
 #include "mongo/db/repl/roll_back_local_operations.h"
+#include "mongo/db/repl/rollback_impl_gen.h"
 #include "mongo/db/repl/storage_interface.h"
+#include "mongo/db/repl/tenant_migration_access_blocker_util.h"
 #include "mongo/db/repl/transaction_oplog_application.h"
-#include "mongo/db/s/shard_identity_rollback_notifier.h"
 #include "mongo/db/s/type_shard_identity.h"
 #include "mongo/db/server_recovery.h"
-#include "mongo/db/session_catalog_mongod.h"
-#include "mongo/db/session_txn_record_gen.h"
+#include "mongo/db/session/kill_sessions_local.h"
+#include "mongo/db/session/session_catalog_mongod.h"
+#include "mongo/db/session/session_txn_record_gen.h"
+#include "mongo/db/storage/historical_ident_tracker.h"
 #include "mongo/db/storage/remove_saver.h"
-#include "mongo/db/transaction_history_iterator.h"
+#include "mongo/db/transaction/transaction_history_iterator.h"
 #include "mongo/logv2/log.h"
 #include "mongo/s/catalog/type_config_version.h"
 #include "mongo/util/scopeguard.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplicationRollback
+
 namespace mongo {
 namespace repl {
 
@@ -103,7 +107,7 @@ boost::optional<long long> _parseDroppedCollectionCount(const OplogEntry& oplogE
         LOGV2_WARNING(21634,
                       "Unable to get collection count from oplog entry without the o2 field",
                       "type"_attr = desc,
-                      "oplogEntry"_attr = redact(oplogEntry.toBSON()));
+                      "oplogEntry"_attr = redact(oplogEntry.toBSONForLogging()));
         return boost::none;
     }
 
@@ -115,7 +119,7 @@ boost::optional<long long> _parseDroppedCollectionCount(const OplogEntry& oplogE
                       "Failed to parse oplog entry for collection count",
                       "type"_attr = desc,
                       "error"_attr = status,
-                      "oplogEntry"_attr = redact(oplogEntry.toBSON()));
+                      "oplogEntry"_attr = redact(oplogEntry.toBSONForLogging()));
         return boost::none;
     }
 
@@ -124,7 +128,7 @@ boost::optional<long long> _parseDroppedCollectionCount(const OplogEntry& oplogE
                       "Invalid collection count found in oplog entry",
                       "type"_attr = desc,
                       "count"_attr = count,
-                      "oplogEntry"_attr = redact(oplogEntry.toBSON()));
+                      "oplogEntry"_attr = redact(oplogEntry.toBSONForLogging()));
         return boost::none;
     }
 
@@ -133,7 +137,7 @@ boost::optional<long long> _parseDroppedCollectionCount(const OplogEntry& oplogE
                 "Parsed collection count of oplog entry",
                 "count"_attr = count,
                 "type"_attr = desc,
-                "oplogEntry"_attr = redact(oplogEntry.toBSON()));
+                "oplogEntry"_attr = redact(oplogEntry.toBSONForLogging()));
     return count;
 }
 
@@ -248,8 +252,7 @@ Status RollbackImpl::runRollback(OperationContext* opCtx) {
     // At this point, the last applied and durable optimes on this node still point to ops on
     // the divergent branch of history. We therefore update the last optimes to the top of the
     // oplog, which should now be at the common point.
-    _replicationCoordinator->resetLastOpTimesFromOplog(
-        opCtx, ReplicationCoordinator::DataConsistency::Consistent);
+    _replicationCoordinator->resetLastOpTimesFromOplog(opCtx);
     status = _triggerOpObserver(opCtx);
     if (!status.isOK()) {
         return status;
@@ -281,7 +284,7 @@ void RollbackImpl::_killAllUserOperations(OperationContext* opCtx) {
 
     for (ServiceContext::LockedClientsCursor cursor(serviceCtx); Client* client = cursor.next();) {
         stdx::lock_guard<Client> lk(*client);
-        if (client->isFromSystemConnection() && !client->shouldKillSystemOperation(lk)) {
+        if (client->isFromSystemConnection() && !client->canKillSystemOperationInStepdown(lk)) {
             continue;
         }
 
@@ -348,11 +351,12 @@ void RollbackImpl::_stopAndWaitForIndexBuilds(OperationContext* opCtx) {
     invariant(opCtx);
 
     // Aborts all active, two-phase index builds.
-    IndexBuildsCoordinator::get(opCtx)->stopIndexBuildsForRollback(opCtx);
+    [[maybe_unused]] auto stoppedIndexBuilds =
+        IndexBuildsCoordinator::get(opCtx)->stopIndexBuildsForRollback(opCtx);
 
     // Get a list of all databases.
     StorageEngine* storageEngine = opCtx->getServiceContext()->getStorageEngine();
-    std::vector<std::string> dbs;
+    std::vector<DatabaseName> dbs;
     {
         Lock::GlobalLock lk(opCtx, MODE_IS);
         dbs = storageEngine->listDatabases();
@@ -361,23 +365,19 @@ void RollbackImpl::_stopAndWaitForIndexBuilds(OperationContext* opCtx) {
     // Wait for all background operations to complete by waiting on each database. Single-phase
     // index builds are not stopped before rollback, so we must wait for these index builds to
     // complete.
-    std::vector<StringData> dbNames(dbs.begin(), dbs.end());
+    std::vector<DatabaseName> dbNames(dbs.begin(), dbs.end());
     LOGV2(21595, "Waiting for all background operations to complete before starting rollback");
-    for (auto db : dbNames) {
-        auto numInProg = BackgroundOperation::numInProgForDb(db);
-        auto numInProgInCoordinator = IndexBuildsCoordinator::get(opCtx)->numInProgForDb(db);
-        if (numInProg > 0 || numInProgInCoordinator > 0) {
-            LOGV2_DEBUG(
-                21596,
-                1,
-                "Waiting for {numBackgroundOperationsInProgress} "
-                "background operations to complete on database '{db}'",
-                "Waiting for background operations to complete",
-                "numBackgroundOperationsInProgress"_attr =
-                    (numInProg > numInProgInCoordinator ? numInProg : numInProgInCoordinator),
-                "db"_attr = db);
-            BackgroundOperation::awaitNoBgOpInProgForDb(db);
-            IndexBuildsCoordinator::get(opCtx)->awaitNoBgOpInProgForDb(opCtx, db);
+    for (auto dbName : dbNames) {
+        auto numInProg = IndexBuildsCoordinator::get(opCtx)->numInProgForDb(dbName.toString());
+        if (numInProg > 0) {
+            LOGV2_DEBUG(21596,
+                        1,
+                        "Waiting for {numBackgroundOperationsInProgress} "
+                        "background operations to complete on database '{db}'",
+                        "Waiting for background operations to complete",
+                        "numBackgroundOperationsInProgress"_attr = numInProg,
+                        "db"_attr = dbName);
+            IndexBuildsCoordinator::get(opCtx)->awaitNoBgOpInProgForDb(opCtx, dbName.toString());
         }
     }
 
@@ -413,7 +413,7 @@ StatusWith<std::set<NamespaceString>> RollbackImpl::_namespacesForOp(const Oplog
         switch (oplogEntry.getCommandType()) {
             case OplogEntry::CommandType::kRenameCollection: {
                 // Add both the 'from' and 'to' namespaces.
-                namespaces.insert(NamespaceString(firstElem.valuestrsafe()));
+                namespaces.insert(NamespaceString(firstElem.valueStringDataSafe()));
                 namespaces.insert(NamespaceString(obj.getStringField("to")));
                 break;
             }
@@ -422,7 +422,6 @@ StatusWith<std::set<NamespaceString>> RollbackImpl::_namespacesForOp(const Oplog
                 break;
             }
             case OplogEntry::CommandType::kDbCheck:
-            case OplogEntry::CommandType::kConvertToCapped:
             case OplogEntry::CommandType::kEmptyCapped: {
                 // These commands do not need to be supported by rollback. 'convertToCapped' should
                 // always be converted to lower level DDL operations, and 'emptycapped' is a
@@ -434,6 +433,7 @@ StatusWith<std::set<NamespaceString>> RollbackImpl::_namespacesForOp(const Oplog
             }
             case OplogEntry::CommandType::kCreate:
             case OplogEntry::CommandType::kDrop:
+            case OplogEntry::CommandType::kImportCollection:
             case OplogEntry::CommandType::kCreateIndexes:
             case OplogEntry::CommandType::kDropIndexes:
             case OplogEntry::CommandType::kStartIndexBuild:
@@ -464,6 +464,81 @@ StatusWith<std::set<NamespaceString>> RollbackImpl::_namespacesForOp(const Oplog
     return namespaces;
 }
 
+void RollbackImpl::_restoreTxnsTableEntryFromRetryableWrites(OperationContext* opCtx,
+                                                             Timestamp stableTimestamp) {
+    auto client = std::make_unique<DBDirectClient>(opCtx);
+    // Query for retryable writes oplog entries with a non-null 'prevWriteOpTime' value
+    // less than or equal to the 'stableTimestamp'. This query intends to include no-op
+    // retryable writes oplog entries that have been applied through a migration process.
+    const auto filter = BSON("op" << BSON("$in" << BSON_ARRAY("i"
+                                                              << "u"
+                                                              << "d")));
+    // We use the 'fromMigrate' field to differentiate migrated retryable writes entries from
+    // transactions entries.
+    const auto filterFromMigration = BSON("op"
+                                          << "n"
+                                          << "fromMigrate" << true);
+    FindCommandRequest findRequest{NamespaceString::kRsOplogNamespace};
+    findRequest.setFilter(BSON("ts" << BSON("$gt" << stableTimestamp) << "txnNumber"
+                                    << BSON("$exists" << true) << "stmtId"
+                                    << BSON("$exists" << true) << "prevOpTime.ts"
+                                    << BSON("$gte" << Timestamp(1, 0) << "$lte" << stableTimestamp)
+                                    << "$or" << BSON_ARRAY(filter << filterFromMigration)));
+    auto cursor = client->find(std::move(findRequest));
+    while (cursor->more()) {
+        auto doc = cursor->next();
+        auto swEntry = OplogEntry::parse(doc);
+        fassert(5530502, swEntry.isOK());
+        auto entry = swEntry.getValue();
+        auto prevWriteOpTime = *entry.getPrevWriteOpTimeInTransaction();
+        OperationSessionInfo opSessionInfo = entry.getOperationSessionInfo();
+        const auto sessionId = *opSessionInfo.getSessionId();
+        const auto txnNumber = *opSessionInfo.getTxnNumber();
+        const auto wallClockTime = entry.getWallClockTime();
+
+        invariant(!prevWriteOpTime.isNull() && prevWriteOpTime.getTimestamp() <= stableTimestamp);
+        // This is a retryable writes oplog entry with a non-null 'prevWriteOpTime' value that
+        // is less than or equal to the 'stableTimestamp'.
+        LOGV2(5530501,
+              "Restoring sessions entry to be consistent with 'stableTimestamp'",
+              "stableTimestamp"_attr = stableTimestamp,
+              "sessionId"_attr = sessionId,
+              "txnNumber"_attr = txnNumber,
+              "lastWriteOpTime"_attr = prevWriteOpTime);
+        SessionTxnRecord sessionTxnRecord;
+        sessionTxnRecord.setSessionId(sessionId);
+        sessionTxnRecord.setTxnNum(txnNumber);
+        try {
+            TransactionHistoryIterator iter(prevWriteOpTime);
+            auto nextOplogEntry = iter.next(opCtx);
+            sessionTxnRecord.setLastWriteOpTime(nextOplogEntry.getOpTime());
+            sessionTxnRecord.setLastWriteDate(nextOplogEntry.getWallClockTime());
+        } catch (ExceptionFor<ErrorCodes::IncompleteTransactionHistory>&) {
+            // It's possible that the next entry in the oplog chain has been truncated due to
+            // oplog cap maintenance.
+            sessionTxnRecord.setLastWriteOpTime(prevWriteOpTime);
+            sessionTxnRecord.setLastWriteDate(wallClockTime);
+        }
+        const auto nss = NamespaceString::kSessionTransactionsTableNamespace;
+        writeConflictRetry(opCtx, "updateSessionTransactionsTableInRollback", nss.ns(), [&] {
+            opCtx->recoveryUnit()->allowUntimestampedWrite();
+            AutoGetCollection collection(opCtx, nss, MODE_IX);
+            auto filter = BSON(SessionTxnRecord::kSessionIdFieldName << sessionId.toBSON());
+            UnreplicatedWritesBlock uwb(opCtx);
+            // Perform an untimestamped write so that it will not be rolled back on recovering
+            // to the 'stableTimestamp' if we were to crash. This is safe because this update is
+            // meant to be consistent with the 'stableTimestamp' and not the common point.
+            Helpers::upsert(opCtx, nss, filter, sessionTxnRecord.toBSON(), /*fromMigrate=*/false);
+        });
+    }
+    // Take a stable checkpoint so that writes to the 'config.transactions' table are
+    // persisted to disk before truncating the oplog. If we were to take an unstable checkpoint, we
+    // would have to update replication metadata like 'minValid.appliedThrough' to be consistent
+    // with the oplog.
+    opCtx->recoveryUnit()->waitUntilUnjournaledWritesDurable(opCtx,
+                                                             /*stableCheckpoint=*/true);
+}
+
 void RollbackImpl::_runPhaseFromAbortToReconstructPreparedTxns(
     OperationContext* opCtx, RollBackLocalOperations::RollbackCommonPoint commonPoint) noexcept {
     // Stop and wait for all background index builds to complete before starting the rollback
@@ -511,15 +586,29 @@ void RollbackImpl::_runPhaseFromAbortToReconstructPreparedTxns(
     _rollbackStats.stableTimestamp = stableTimestamp;
     _listener->onRecoverToStableTimestamp(stableTimestamp);
 
+    // Rollback historical ident entries.
+    HistoricalIdentTracker::get(opCtx).rollbackTo(stableTimestamp);
+
     // Log the total number of insert and update operations that have been rolled back as a
     // result of recovering to the stable timestamp.
+    auto getCommandCount = [&](StringData key) {
+        const auto& m = _observerInfo.rollbackCommandCounts;
+        auto it = m.find(key);
+        return (it == m.end()) ? 0 : it->second;
+    };
     LOGV2(21599,
           "Rollback reverted {insert} insert operations, {update} update operations and {delete} "
           "delete operations.",
           "Rollback reverted command counts",
-          "insert"_attr = _observerInfo.rollbackCommandCounts[kInsertCmdName],
-          "update"_attr = _observerInfo.rollbackCommandCounts[kUpdateCmdName],
-          "delete"_attr = _observerInfo.rollbackCommandCounts[kDeleteCmdName]);
+          "insert"_attr = getCommandCount(kInsertCmdName),
+          "update"_attr = getCommandCount(kUpdateCmdName),
+          "delete"_attr = getCommandCount(kDeleteCmdName));
+
+    // Retryable writes create derived updates to the transactions table which can be coalesced into
+    // one operation, so certain session operations history may be lost after restoring to the
+    // 'stableTimestamp'. We must scan the oplog and restore the transactions table entries to
+    // detail the last executed writes.
+    _restoreTxnsTableEntryFromRetryableWrites(opCtx, stableTimestamp);
 
     // During replication recovery, we truncate all oplog entries with timestamps greater than the
     // oplog truncate after point. If we entered rollback, we are guaranteed to have at least one
@@ -558,6 +647,8 @@ void RollbackImpl::_runPhaseFromAbortToReconstructPreparedTxns(
     // rollback.
     _correctRecordStoreCounts(opCtx);
 
+    tenant_migration_access_blocker::recoverTenantMigrationAccessBlockers(opCtx);
+
     // Reconstruct prepared transactions after counts have been adjusted. Since prepared
     // transactions were aborted (i.e. the in-memory counts were rolled-back) before computing
     // collection counts, reconstruct the prepared transactions now, adding on any additional counts
@@ -568,10 +659,10 @@ void RollbackImpl::_runPhaseFromAbortToReconstructPreparedTxns(
 void RollbackImpl::_correctRecordStoreCounts(OperationContext* opCtx) {
     // This function explicitly does not check for shutdown since a clean shutdown post oplog
     // truncation is not allowed to occur until the record store counts are corrected.
-    const auto& catalog = CollectionCatalog::get(opCtx);
+    auto catalog = CollectionCatalog::get(opCtx);
     for (const auto& uiCount : _newCounts) {
         const auto uuid = uiCount.first;
-        const auto coll = catalog.lookupCollectionByUUID(opCtx, uuid);
+        const auto coll = catalog->lookupCollectionByUUID(opCtx, uuid);
         invariant(coll,
                   str::stream() << "The collection with UUID " << uuid
                                 << " is unexpectedly missing in the CollectionCatalog");
@@ -611,13 +702,14 @@ void RollbackImpl::_correctRecordStoreCounts(OperationContext* opCtx) {
                   "Scanning collection to fix collection count",
                   "namespace"_attr = nss.ns(),
                   "uuid"_attr = uuid.toString());
-            AutoGetCollectionForRead autoCollToScan(opCtx, nss);
-            auto collToScan = autoCollToScan.getCollection();
-            invariant(coll == collToScan,
+            AutoGetCollectionForRead collToScan(opCtx, nss);
+            invariant(coll == collToScan.getCollection(),
                       str::stream() << "Catalog returned invalid collection: " << nss.ns() << " ("
                                     << uuid.toString() << ")");
-            auto exec = collToScan->makePlanExecutor(
-                opCtx, PlanExecutor::INTERRUPT_ONLY, Collection::ScanDirection::kForward);
+            auto exec = collToScan->makePlanExecutor(opCtx,
+                                                     collToScan.getCollection(),
+                                                     PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
+                                                     Collection::ScanDirection::kForward);
             long long countFromScan = 0;
             PlanExecutor::ExecState state;
             while (PlanExecutor::ADVANCED ==
@@ -634,7 +726,7 @@ void RollbackImpl::_correctRecordStoreCounts(OperationContext* opCtx) {
                               "namespace"_attr = nss.ns(),
                               "uuid"_attr = uuid.toString(),
                               "ident"_attr = ident,
-                              "error"_attr = exec->statestr(state));
+                              "error"_attr = exec->stateToStr(state));
                 continue;
             }
             newCount = countFromScan;
@@ -668,7 +760,7 @@ void RollbackImpl::_correctRecordStoreCounts(OperationContext* opCtx) {
 }
 
 Status RollbackImpl::_findRecordStoreCounts(OperationContext* opCtx) {
-    const auto& catalog = CollectionCatalog::get(opCtx);
+    auto catalog = CollectionCatalog::get(opCtx);
     auto storageEngine = opCtx->getServiceContext()->getStorageEngine();
 
     LOGV2(21604, "Finding record store counts");
@@ -679,7 +771,7 @@ Status RollbackImpl::_findRecordStoreCounts(OperationContext* opCtx) {
             continue;
         }
 
-        auto nss = catalog.lookupNSSByUUID(opCtx, uuid);
+        auto nss = catalog->lookupNSSByUUID(opCtx, uuid);
         StorageInterface::CollectionCount oldCount = 0;
 
         // Drop-pending collections are not visible to rollback via the catalog when they are
@@ -825,13 +917,13 @@ Status RollbackImpl::_processRollbackOp(OperationContext* opCtx, const OplogEntr
         const auto uuid = oplogEntry.getUuid();
         invariant(uuid,
                   str::stream() << "Oplog entry to roll back is unexpectedly missing a UUID: "
-                                << redact(oplogEntry.toBSON()));
+                                << redact(oplogEntry.toBSONForLogging()));
         const auto idElem = oplogEntry.getIdElement();
         if (!idElem.eoo()) {
             // We call BSONElement::wrap() on each _id element to create a new BSONObj with an owned
             // buffer, as the underlying storage may be gone when we access this map to write
             // rollback files.
-            _observerInfo.rollbackDeletedIdsMap[uuid.get()].insert(idElem.wrap());
+            _observerInfo.rollbackDeletedIdsMap[uuid.value()].insert(idElem.wrap());
             const auto cmdName = opType == OpTypeEnum::kInsert ? kInsertCmdName : kUpdateCmdName;
             ++_observerInfo.rollbackCommandCounts[cmdName];
         }
@@ -847,7 +939,7 @@ Status RollbackImpl::_processRollbackOp(OperationContext* opCtx, const OplogEntr
             LOGV2_WARNING(21641,
                           "Shard identity document rollback detected. oplog op: {oplogEntry}",
                           "Shard identity document rollback detected",
-                          "oplogEntry"_attr = redact(oplogEntry.toBSON()));
+                          "oplogEntry"_attr = redact(oplogEntry.toBSONForLogging()));
         } else if (serverGlobalParams.clusterRole == ClusterRole::ConfigServer &&
                    opNss == VersionType::ConfigNS) {
             // Check if the creation of the config server config version document is being rolled
@@ -856,20 +948,36 @@ Status RollbackImpl::_processRollbackOp(OperationContext* opCtx, const OplogEntr
             LOGV2_WARNING(21642,
                           "Config version document rollback detected. oplog op: {oplogEntry}",
                           "Config version document rollback detected",
-                          "oplogEntry"_attr = redact(oplogEntry.toBSON()));
+                          "oplogEntry"_attr = redact(oplogEntry.toBSONForLogging()));
         }
 
         // Rolling back an insert must decrement the count by 1.
-        _countDiffs[oplogEntry.getUuid().get()] -= 1;
+        _countDiffs[oplogEntry.getUuid().value()] -= 1;
     } else if (opType == OpTypeEnum::kDelete) {
         // Rolling back a delete must increment the count by 1.
-        _countDiffs[oplogEntry.getUuid().get()] += 1;
+        _countDiffs[oplogEntry.getUuid().value()] += 1;
     } else if (opType == OpTypeEnum::kCommand) {
         if (oplogEntry.getCommandType() == OplogEntry::CommandType::kCreate) {
             // If we roll back a create, then we do not need to change the size of that uuid.
-            _countDiffs.erase(oplogEntry.getUuid().get());
-            _pendingDrops.erase(oplogEntry.getUuid().get());
-            _newCounts.erase(oplogEntry.getUuid().get());
+            _countDiffs.erase(oplogEntry.getUuid().value());
+            _pendingDrops.erase(oplogEntry.getUuid().value());
+            _newCounts.erase(oplogEntry.getUuid().value());
+        } else if (oplogEntry.getCommandType() == OplogEntry::CommandType::kImportCollection) {
+            auto importEntry = mongo::ImportCollectionOplogEntry::parse(
+                IDLParserContext("importCollectionOplogEntry"), oplogEntry.getObject());
+            // Nothing to roll back if this is a dryRun.
+            if (!importEntry.getDryRun()) {
+                const auto& catalogEntry = importEntry.getCatalogEntry();
+                auto importTargetUUID =
+                    invariant(UUID::parse(catalogEntry["md"]["options"]["uuid"]),
+                              str::stream() << "Oplog entry to roll back is unexpectedly missing "
+                                               "import collection UUID: "
+                                            << redact(oplogEntry.toBSONForLogging()));
+                // If we roll back an import, then we do not need to change the size of that uuid.
+                _countDiffs.erase(importTargetUUID);
+                _pendingDrops.erase(importTargetUUID);
+                _newCounts.erase(importTargetUUID);
+            }
         } else if (oplogEntry.getCommandType() == OplogEntry::CommandType::kDrop) {
             // If we roll back a collection drop, parse the o2 field for the collection count for
             // use later by _findRecordStoreCounts().
@@ -877,10 +985,10 @@ Status RollbackImpl::_processRollbackOp(OperationContext* opCtx, const OplogEntr
             // collection is managed by the storage engine and is not accessible through the UUID
             // catalog.
             // Adding a _newCounts entry ensures that the count will be set after the rollback.
-            const auto uuid = oplogEntry.getUuid().get();
+            const auto uuid = oplogEntry.getUuid().value();
             invariant(_countDiffs.find(uuid) == _countDiffs.end(),
                       str::stream() << "Unexpected existing count diff for " << uuid.toString()
-                                    << " op: " << redact(oplogEntry.toBSON()));
+                                    << " op: " << redact(oplogEntry.toBSONForLogging()));
             if (auto countResult = _parseDroppedCollectionCount(oplogEntry)) {
                 PendingDropInfo info;
                 info.count = *countResult;
@@ -904,11 +1012,11 @@ Status RollbackImpl::_processRollbackOp(OperationContext* opCtx, const OplogEntr
                 UUID::parse(oplogEntry.getObject()[kDropTargetFieldName]),
                 str::stream()
                     << "Oplog entry to roll back is unexpectedly missing dropTarget UUID: "
-                    << redact(oplogEntry.toBSON()));
+                    << redact(oplogEntry.toBSONForLogging()));
             invariant(_countDiffs.find(dropTargetUUID) == _countDiffs.end(),
                       str::stream()
                           << "Unexpected existing count diff for " << dropTargetUUID.toString()
-                          << " op: " << redact(oplogEntry.toBSON()));
+                          << " op: " << redact(oplogEntry.toBSONForLogging()));
             if (auto countResult = _parseDroppedCollectionCount(oplogEntry)) {
                 PendingDropInfo info;
                 info.count = *countResult;
@@ -964,7 +1072,6 @@ Status RollbackImpl::_processRollbackOpForApplyOps(OperationContext* opCtx,
                 return subStatus;
             }
         }
-        return Status::OK();
     } catch (DBException& e) {
         return e.toStatus();
     }
@@ -1126,11 +1233,11 @@ boost::optional<BSONObj> RollbackImpl::_findDocumentById(OperationContext* opCtx
 }
 
 Status RollbackImpl::_writeRollbackFiles(OperationContext* opCtx) {
-    const auto& catalog = CollectionCatalog::get(opCtx);
+    auto catalog = CollectionCatalog::get(opCtx);
     auto storageEngine = opCtx->getServiceContext()->getStorageEngine();
     for (auto&& entry : _observerInfo.rollbackDeletedIdsMap) {
         const auto& uuid = entry.first;
-        const auto nss = catalog.lookupNSSByUUID(opCtx, uuid);
+        const auto nss = catalog->lookupNSSByUUID(opCtx, uuid);
 
         // Drop-pending collections are not visible to rollback via the catalog when they are
         // managed by the storage engine. See StorageEngine::supportsPendingDrops().
@@ -1257,12 +1364,10 @@ void RollbackImpl::_resetDropPendingState(OperationContext* opCtx) {
     auto storageEngine = opCtx->getServiceContext()->getStorageEngine();
     storageEngine->clearDropPendingState();
 
-    std::vector<std::string> dbNames = storageEngine->listDatabases();
-    auto databaseHolder = DatabaseHolder::get(opCtx);
+    std::vector<DatabaseName> dbNames = storageEngine->listDatabases();
     for (const auto& dbName : dbNames) {
         Lock::DBLock dbLock(opCtx, dbName, MODE_X);
-        auto db = databaseHolder->openDb(opCtx, dbName);
-        db->checkForIdIndexesAndDropPendingCollections(opCtx);
+        checkForIdIndexesAndDropPendingCollections(opCtx, dbName);
     }
 }
 
