@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
 #include "mongo/platform/basic.h"
 
@@ -37,7 +36,6 @@
 
 #include "mongo/db/jsobj.h"
 #include "mongo/db/repl/oplog_entry.h"
-#include "mongo/db/repl/replication_process.h"
 #include "mongo/db/repl/sync_source_selector.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
@@ -45,17 +43,20 @@
 #include "mongo/util/destructor_guard.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
+
+
 namespace mongo {
 namespace repl {
 
 const NamespaceString SyncSourceResolver::kLocalOplogNss("local.oplog.rs");
 const Seconds SyncSourceResolver::kFetcherTimeout(30);
-const Seconds SyncSourceResolver::kFetcherErrorBlacklistDuration(10);
-const Seconds SyncSourceResolver::kOplogEmptyBlacklistDuration(10);
-const Seconds SyncSourceResolver::kFirstOplogEntryEmptyBlacklistDuration(10);
-const Seconds SyncSourceResolver::kFirstOplogEntryNullTimestampBlacklistDuration(10);
-const Minutes SyncSourceResolver::kTooStaleBlacklistDuration(1);
-const Seconds SyncSourceResolver::kNoRequiredOpTimeBlacklistDuration(60);
+const Seconds SyncSourceResolver::kFetcherErrorDenylistDuration(10);
+const Seconds SyncSourceResolver::kOplogEmptyDenylistDuration(10);
+const Seconds SyncSourceResolver::kFirstOplogEntryEmptyDenylistDuration(10);
+const Seconds SyncSourceResolver::kFirstOplogEntryNullTimestampDenylistDuration(10);
+const Minutes SyncSourceResolver::kTooStaleDenylistDuration(1);
+const Seconds SyncSourceResolver::kNoRequiredOpTimeDenylistDuration(60);
 
 SyncSourceResolver::SyncSourceResolver(executor::TaskExecutor* taskExecutor,
                                        SyncSourceSelector* syncSourceSelector,
@@ -165,9 +166,6 @@ StatusWith<HostAndPort> SyncSourceResolver::_chooseNewSyncSource() {
 
 std::unique_ptr<Fetcher> SyncSourceResolver::_makeFirstOplogEntryFetcher(
     HostAndPort candidate, OpTime earliestOpTimeSeen) {
-    // This needs to be explicitly declared as a "long", otherwise the
-    // find command will not be able to correctly parse the term field.
-    long long dummyTerm = -1;
     return std::make_unique<Fetcher>(
         _taskExecutor,
         candidate,
@@ -176,8 +174,7 @@ std::unique_ptr<Fetcher> SyncSourceResolver::_makeFirstOplogEntryFetcher(
                     << "projection"
                     << BSON(OplogEntryBase::kTimestampFieldName
                             << 1 << OplogEntryBase::kTermFieldName << 1)
-                    << ReadConcernArgs::kReadConcernFieldName << ReadConcernArgs::kImplicitDefault
-                    << "term" << dummyTerm),
+                    << ReadConcernArgs::kReadConcernFieldName << ReadConcernArgs::kLocal),
         [=](const StatusWith<Fetcher::QueryResponse>& response,
             Fetcher::NextAction*,
             BSONObjBuilder*) {
@@ -191,16 +188,17 @@ std::unique_ptr<Fetcher> SyncSourceResolver::_makeFirstOplogEntryFetcher(
 std::unique_ptr<Fetcher> SyncSourceResolver::_makeRequiredOpTimeFetcher(HostAndPort candidate,
                                                                         OpTime earliestOpTimeSeen,
                                                                         int rbid) {
-    // This query is structured so that it is executed on the sync source using the oplog
-    // start hack (oplogReplay=true and $gt/$gte predicate over "ts").
+    // This query is structured so that it is executed on the sync source using the "oplog start
+    // hack". The sync source should recognize that it can optimize an oplog query with a $gt/$gte
+    // predicate over "ts".
     return std::make_unique<Fetcher>(
         _taskExecutor,
         candidate,
         kLocalOplogNss.db().toString(),
-        BSON("find" << kLocalOplogNss.coll() << "oplogReplay" << true << "filter"
+        BSON("find" << kLocalOplogNss.coll() << "filter"
                     << BSON("ts" << BSON("$gte" << _requiredOpTime.getTimestamp() << "$lte"
                                                 << _requiredOpTime.getTimestamp()))
-                    << ReadConcernArgs::kReadConcernFieldName << ReadConcernArgs::kImplicitDefault),
+                    << ReadConcernArgs::kReadConcernFieldName << ReadConcernArgs::kLocal),
         [=](const StatusWith<Fetcher::QueryResponse>& response,
             Fetcher::NextAction*,
             BSONObjBuilder*) {
@@ -238,61 +236,61 @@ OpTime SyncSourceResolver::_parseRemoteEarliestOpTime(const HostAndPort& candida
                                                       const Fetcher::QueryResponse& queryResponse) {
     if (queryResponse.documents.empty()) {
         // Remote oplog is empty.
-        const auto until = _taskExecutor->now() + kOplogEmptyBlacklistDuration;
-        LOGV2(21766,
-              "Blacklisting {candidate} due to empty oplog for {blacklistDuration} "
-              "until: {blacklistUntil}",
-              "Blacklisting candidate due to empty oplog",
+        const auto until = _taskExecutor->now() + kOplogEmptyDenylistDuration;
+        LOGV2(5579703,
+              "Denylisting {candidate} due to empty oplog for {denylistDuration} "
+              "until: {denylistUntil}",
+              "Denylisting candidate due to empty oplog",
               "candidate"_attr = candidate,
-              "blacklistDuration"_attr = kOplogEmptyBlacklistDuration,
-              "blacklistUntil"_attr = until);
-        _syncSourceSelector->blacklistSyncSource(candidate, until);
+              "denylistDuration"_attr = kOplogEmptyDenylistDuration,
+              "denylistUntil"_attr = until);
+        _syncSourceSelector->denylistSyncSource(candidate, until);
         return OpTime();
     }
 
     const auto& firstObjFound = queryResponse.documents.front();
     if (firstObjFound.isEmpty()) {
         // First document in remote oplog is empty.
-        const auto until = _taskExecutor->now() + kFirstOplogEntryEmptyBlacklistDuration;
-        LOGV2(21767,
-              "Blacklisting {candidate} due to empty first document for "
-              "{blacklistDuration} until: {blacklistUntil}",
-              "Blacklisting candidate due to empty first document",
+        const auto until = _taskExecutor->now() + kFirstOplogEntryEmptyDenylistDuration;
+        LOGV2(5579704,
+              "Denylisting {candidate} due to empty first document for "
+              "{denylistDuration} until: {denylistUntil}",
+              "Denylisting candidate due to empty first document",
               "candidate"_attr = candidate,
-              "blacklistDuration"_attr = kFirstOplogEntryEmptyBlacklistDuration,
-              "blacklistUntil"_attr = until);
-        _syncSourceSelector->blacklistSyncSource(candidate, until);
+              "denylistDuration"_attr = kFirstOplogEntryEmptyDenylistDuration,
+              "denylistUntil"_attr = until);
+        _syncSourceSelector->denylistSyncSource(candidate, until);
         return OpTime();
     }
 
     const auto remoteEarliestOpTime = OpTime::parseFromOplogEntry(firstObjFound);
     if (!remoteEarliestOpTime.isOK()) {
-        const auto until = _taskExecutor->now() + kFirstOplogEntryNullTimestampBlacklistDuration;
-        LOGV2(21768,
-              "Blacklisting {candidate} due to error parsing OpTime from the oldest oplog entry "
-              "for {blacklistDuration} until: {blacklistUntil}. Error: "
+        const auto until = _taskExecutor->now() + kFirstOplogEntryNullTimestampDenylistDuration;
+        LOGV2(5579705,
+              "Denylisting {candidate} due to error parsing OpTime from the oldest oplog entry "
+              "for {denylistDuration} until: {denylistUntil}. Error: "
               "{error}, Entry: {oldestOplogEntry}",
-              "Blacklisting candidate due to error parsing OpTime from the oldest oplog entry",
+              "Denylisting candidate due to error parsing OpTime from the oldest oplog entry",
               "candidate"_attr = candidate,
-              "blacklistDuration"_attr = kFirstOplogEntryNullTimestampBlacklistDuration,
-              "blacklistUntil"_attr = until,
+              "denylistDuration"_attr = kFirstOplogEntryNullTimestampDenylistDuration,
+              "denylistUntil"_attr = until,
               "error"_attr = remoteEarliestOpTime.getStatus(),
               "oldestOplogEntry"_attr = redact(firstObjFound));
-        _syncSourceSelector->blacklistSyncSource(candidate, until);
+        _syncSourceSelector->denylistSyncSource(candidate, until);
         return OpTime();
     }
 
     if (remoteEarliestOpTime.getValue().isNull()) {
         // First document in remote oplog is empty.
-        const auto until = _taskExecutor->now() + kFirstOplogEntryNullTimestampBlacklistDuration;
-        LOGV2(21769,
-              "Blacklisting {candidate} due to null timestamp in first document for "
-              "{blacklistDuration} until: {blacklistUntil}",
-              "Blacklisting candidate due to null timestamp in first document",
+        const auto until = _taskExecutor->now() + kFirstOplogEntryNullTimestampDenylistDuration;
+        LOGV2(5579706,
+              "Denylisting {candidate} due to null timestamp in first document for "
+              "{denylistDuration} until: {denylistUntil}",
+              "Denylisting candidate due to null timestamp in first document",
               "candidate"_attr = candidate,
-              "blacklistDuration"_attr = kFirstOplogEntryNullTimestampBlacklistDuration,
-              "blacklistUntil"_attr = until);
-        _syncSourceSelector->blacklistSyncSource(candidate, until);
+              "denylistDuration"_attr = kFirstOplogEntryNullTimestampDenylistDuration,
+              "denylistUntil"_attr = until);
+        _syncSourceSelector->denylistSyncSource(candidate, until);
         return OpTime();
     }
 
@@ -319,16 +317,16 @@ void SyncSourceResolver::_firstOplogEntryFetcherCallback(
 
     if (!queryResult.isOK()) {
         // We got an error.
-        const auto until = _taskExecutor->now() + kFetcherErrorBlacklistDuration;
-        LOGV2(21770,
-              "Blacklisting {candidate} due to error: '{error}' for "
-              "{blacklistDuration} until: {blacklistUntil}",
-              "Blacklisting candidate due to error",
+        const auto until = _taskExecutor->now() + kFetcherErrorDenylistDuration;
+        LOGV2(5579707,
+              "Denylisting {candidate} due to error: '{error}' for "
+              "{denylistDuration} until: {denylistUntil}",
+              "Denylisting candidate due to error",
               "candidate"_attr = candidate,
               "error"_attr = queryResult.getStatus(),
-              "blacklistDuration"_attr = kFetcherErrorBlacklistDuration,
-              "blacklistUntil"_attr = until);
-        _syncSourceSelector->blacklistSyncSource(candidate, until);
+              "denylistDuration"_attr = kFetcherErrorDenylistDuration,
+              "denylistUntil"_attr = until);
+        _syncSourceSelector->denylistSyncSource(candidate, until);
 
         _chooseAndProbeNextSyncSource(earliestOpTimeSeen).transitional_ignore();
         return;
@@ -344,23 +342,23 @@ void SyncSourceResolver::_firstOplogEntryFetcherCallback(
     // remoteEarliestOpTime may come from a very old config, so we cannot compare their terms.
     if (_lastOpTimeFetched.getTimestamp() < remoteEarliestOpTime.getTimestamp()) {
         // We're too stale to use this sync source.
-        const auto blacklistDuration = kTooStaleBlacklistDuration;
-        const auto until = _taskExecutor->now() + Minutes(1);
+        const auto denylistDuration = kTooStaleDenylistDuration;
+        const auto until = _taskExecutor->now() + denylistDuration;
 
-        LOGV2(21771,
-              "We are too stale to use {candidate} as a sync source. Blacklisting this sync source "
+        LOGV2(5579708,
+              "We are too stale to use {candidate} as a sync source. Denylisting this sync source "
               "because our last fetched timestamp: {lastOpTimeFetchedTimestamp} is before "
               "their earliest timestamp: {remoteEarliestOpTimeTimestamp} for "
-              "{blacklistDuration} until: {blacklistUntil}",
-              "We are too stale to use candidate as a sync source. Blacklisting this sync source "
+              "{denylistDuration} until: {denylistUntil}",
+              "We are too stale to use candidate as a sync source. Denylisting this sync source "
               "because our last fetched timestamp is before their earliest timestamp",
               "candidate"_attr = candidate,
               "lastOpTimeFetchedTimestamp"_attr = _lastOpTimeFetched.getTimestamp(),
               "remoteEarliestOpTimeTimestamp"_attr = remoteEarliestOpTime.getTimestamp(),
-              "blacklistDuration"_attr = blacklistDuration,
-              "blacklistUntil"_attr = until);
+              "denylistDuration"_attr = denylistDuration,
+              "denylistUntil"_attr = until);
 
-        _syncSourceSelector->blacklistSyncSource(candidate, until);
+        _syncSourceSelector->denylistSyncSource(candidate, until);
 
         // If all the viable sync sources are too far ahead of us (i.e. we are "too stale" relative
         // each sync source), we will want to return the starting timestamp of the sync source
@@ -375,6 +373,13 @@ void SyncSourceResolver::_firstOplogEntryFetcherCallback(
         return;
     }
 
+    // If we should not proceed with the rollback-via-refetch checks, we can safely return the
+    // candidate with an uninitialized rbid.
+    if (_requiredOpTime.isNull()) {
+        _finishCallback(candidate, ReplicationProcess::kUninitializedRollbackId).ignore();
+        return;
+    }
+
     auto status = _scheduleRBIDRequest(candidate, earliestOpTimeSeen);
     if (!status.isOK()) {
         _finishCallback(status).ignore();
@@ -423,32 +428,26 @@ void SyncSourceResolver::_rbidRequestCallback(
         uassertStatusOK(getStatusFromCommandResult(rbidReply.response.data));
         rbid = rbidReply.response.data["rbid"].Int();
     } catch (const DBException& ex) {
-        const auto until = _taskExecutor->now() + kFetcherErrorBlacklistDuration;
-        LOGV2(21772,
-              "Blacklisting {candidate} due to error: '{error}' for {blacklistDuration} "
-              "until: {blacklistUntil}",
-              "Blacklisting candidate due to error",
+        const auto until = _taskExecutor->now() + kFetcherErrorDenylistDuration;
+        LOGV2(5579709,
+              "Denylisting {candidate} due to error: '{error}' for {denylistDuration} "
+              "until: {denylistUntil}",
+              "Denylisting candidate due to error",
               "candidate"_attr = candidate,
               "error"_attr = ex,
-              "blacklistDuration"_attr = kFetcherErrorBlacklistDuration,
-              "blacklistUntil"_attr = until);
-        _syncSourceSelector->blacklistSyncSource(candidate, until);
+              "denylistDuration"_attr = kFetcherErrorDenylistDuration,
+              "denylistUntil"_attr = until);
+        _syncSourceSelector->denylistSyncSource(candidate, until);
         _chooseAndProbeNextSyncSource(earliestOpTimeSeen).transitional_ignore();
         return;
     }
 
-    if (!_requiredOpTime.isNull()) {
-        // Schedule fetcher to look for '_requiredOpTime' in the remote oplog.
-        // Unittest requires that this kind of failure be handled specially.
-        auto status =
-            _scheduleFetcher(_makeRequiredOpTimeFetcher(candidate, earliestOpTimeSeen, rbid));
-        if (!status.isOK()) {
-            _finishCallback(status).transitional_ignore();
-        }
-        return;
+    // Schedule fetcher to look for '_requiredOpTime' in the remote oplog.
+    // Unittest requires that this kind of failure be handled specially.
+    auto status = _scheduleFetcher(_makeRequiredOpTimeFetcher(candidate, earliestOpTimeSeen, rbid));
+    if (!status.isOK()) {
+        _finishCallback(status).ignore();
     }
-
-    _finishCallback(candidate, rbid).ignore();
 }
 
 Status SyncSourceResolver::_compareRequiredOpTimeWithQueryResponse(
@@ -499,18 +498,18 @@ void SyncSourceResolver::_requiredOpTimeFetcherCallback(
 
     if (!queryResult.isOK()) {
         // We got an error.
-        const auto until = _taskExecutor->now() + kFetcherErrorBlacklistDuration;
-        LOGV2(21773,
-              "Blacklisting {candidate} due to required optime fetcher error: "
-              "'{error}' for {blacklistDuration} until: {blacklistUntil}. "
+        const auto until = _taskExecutor->now() + kFetcherErrorDenylistDuration;
+        LOGV2(5579710,
+              "Denylisting {candidate} due to required optime fetcher error: "
+              "'{error}' for {denylistDuration} until: {denylistUntil}. "
               "required optime: {requiredOpTime}",
-              "Blacklisting candidate due to required optime fetcher error",
+              "Denylisting candidate due to required optime fetcher error",
               "candidate"_attr = candidate,
               "error"_attr = queryResult.getStatus(),
-              "blacklistDuration"_attr = kFetcherErrorBlacklistDuration,
-              "blacklistUntil"_attr = until,
+              "denylistDuration"_attr = kFetcherErrorDenylistDuration,
+              "denylistUntil"_attr = until,
               "requiredOpTime"_attr = _requiredOpTime);
-        _syncSourceSelector->blacklistSyncSource(candidate, until);
+        _syncSourceSelector->denylistSyncSource(candidate, until);
 
         _chooseAndProbeNextSyncSource(earliestOpTimeSeen).transitional_ignore();
         return;
@@ -519,22 +518,22 @@ void SyncSourceResolver::_requiredOpTimeFetcherCallback(
     const auto& queryResponse = queryResult.getValue();
     auto status = _compareRequiredOpTimeWithQueryResponse(queryResponse);
     if (!status.isOK()) {
-        const auto until = _taskExecutor->now() + kNoRequiredOpTimeBlacklistDuration;
+        const auto until = _taskExecutor->now() + kNoRequiredOpTimeDenylistDuration;
         LOGV2_WARNING(
-            21774,
+            5579711,
             "We cannot use {candidate} as a sync source because it does not contain the necessary "
             "operations for us to reach a consistent state: {error} last fetched optime: "
-            "{lastOpTimeFetched}. required optime: {requiredOpTime}. Blacklisting this sync source "
-            "for {blacklistDuration} until: {blacklistUntil}",
+            "{lastOpTimeFetched}. required optime: {requiredOpTime}. Denylisting this sync source "
+            "for {denylistDuration} until: {denylistUntil}",
             "We cannot use candidate as a sync source because it does not contain the necessary "
-            "operations for us to reach a consistent state. Blacklisting this sync source",
+            "operations for us to reach a consistent state. Denylisting this sync source",
             "candidate"_attr = candidate.toString(),
             "error"_attr = status,
             "lastOpTimeFetched"_attr = _lastOpTimeFetched,
             "requiredOpTime"_attr = _requiredOpTime,
-            "blacklistDuration"_attr = kNoRequiredOpTimeBlacklistDuration,
-            "blacklistUntil"_attr = until);
-        _syncSourceSelector->blacklistSyncSource(candidate, until);
+            "denylistDuration"_attr = kNoRequiredOpTimeDenylistDuration,
+            "denylistUntil"_attr = until);
+        _syncSourceSelector->denylistSyncSource(candidate, until);
 
         _chooseAndProbeNextSyncSource(earliestOpTimeSeen).transitional_ignore();
         return;
@@ -556,7 +555,7 @@ Status SyncSourceResolver::_chooseAndProbeNextSyncSource(OpTime earliestOpTimeSe
         }
 
         SyncSourceResolverResponse response;
-        response.syncSourceStatus = {ErrorCodes::OplogStartMissing, "too stale to catch up"};
+        response.syncSourceStatus = {ErrorCodes::TooStaleToSyncFromSource, "too stale to catch up"};
         response.earliestOpTimeSeen = earliestOpTimeSeen;
         return _finishCallback(response);
     }
@@ -573,9 +572,7 @@ Status SyncSourceResolver::_chooseAndProbeNextSyncSource(OpTime earliestOpTimeSe
 Status SyncSourceResolver::_finishCallback(HostAndPort hostAndPort, int rbid) {
     SyncSourceResolverResponse response;
     response.syncSourceStatus = std::move(hostAndPort);
-    if (rbid != ReplicationProcess::kUninitializedRollbackId) {
-        response.rbid = rbid;
-    }
+    response.rbid = rbid;
     return _finishCallback(response);
 }
 
