@@ -31,10 +31,11 @@
 
 #include <functional>
 #include <iosfwd>
+#include <queue>
 #include <string>
 
 #include "mongo/client/read_preference.h"
-#include "mongo/db/repl/is_master_response.h"
+#include "mongo/db/repl/hello_response.h"
 #include "mongo/db/repl/last_vote.h"
 #include "mongo/db/repl/repl_set_heartbeat_args_v1.h"
 #include "mongo/db/repl/repl_set_heartbeat_response.h"
@@ -75,6 +76,37 @@ class TopologyCoordinator {
     TopologyCoordinator& operator=(const TopologyCoordinator&) = delete;
 
 public:
+    /**
+     * RecentSyncSourceChanges stores the times that recent sync source changes happened. It will
+     * maintain a max size of maxSyncSourceChangesPerHour. If any additional entries are added,
+     * older entries will be removed. It is used to restrict the number of sync source changes that
+     * happen per hour when the node already has a valid sync source.
+     */
+    class RecentSyncSourceChanges {
+    public:
+        /**
+         * Checks if all the entries occurred within the last hour or not. It will remove additional
+         * entries if it sees that there are more than maxSyncSourceChangesPerHour entries. If there
+         * are fewer than maxSyncSourceChangesPerHour entries, it returns false.
+         */
+        bool changedTooOftenRecently(Date_t now);
+
+        /**
+         * Adds a new entry. It will remove additional entries if it sees that there are more than
+         * maxSyncSourceChangesPerHour entries. This should only be called if the sync source was
+         * changed to another node, not if the sync source was cleared.
+         */
+        void addNewEntry(Date_t now);
+
+        /**
+         * Return the underlying queue. Used for testing purposes only.
+         */
+        std::queue<Date_t> getChanges_forTest();
+
+    private:
+        std::queue<Date_t> _recentChanges;
+    };
+
     /**
      * Type that denotes the role of a node in the replication protocol.
      *
@@ -199,35 +231,29 @@ public:
      */
     void setForceSyncSourceIndex(int index);
 
-    enum class ChainingPreference { kAllowChaining, kUseConfiguration };
-
     /**
      * Chooses and sets a new sync source, based on our current knowledge of the world.
-     * If chaining is disabled in the configuration and chainingPreference is kUseConfiguration,
-     * only the primary will be selected (regardless of read preference).  Otherwise,
-     * the readPreference is respected.  Chaining disabled with SecondaryOnly read preference is
-     * not allowed.
+     * If readPreference is PrimaryOnly, only the primary will be selected.
      */
     HostAndPort chooseNewSyncSource(Date_t now,
                                     const OpTime& lastOpTimeFetched,
-                                    ChainingPreference chainingPreference,
                                     ReadPreference readPreference);
 
     /**
      * Suppresses selecting "host" as sync source until "until".
      */
-    void blacklistSyncSource(const HostAndPort& host, Date_t until);
+    void denylistSyncSource(const HostAndPort& host, Date_t until);
 
     /**
      * Removes a single entry "host" from the list of potential sync sources which we
-     * have blacklisted, if it is supposed to be unblacklisted by "now".
+     * have denylisted, if it is supposed to be undenylisted by "now".
      */
-    void unblacklistSyncSource(const HostAndPort& host, Date_t now);
+    void undenylistSyncSource(const HostAndPort& host, Date_t now);
 
     /**
-     * Clears the list of potential sync sources we have blacklisted.
+     * Clears the list of potential sync sources we have denylisted.
      */
-    void clearSyncSourceBlacklist();
+    void clearSyncSourceDenylist();
 
     /**
      * Determines if a new sync source should be chosen, if a better candidate sync source is
@@ -237,13 +263,32 @@ public:
      * running in ProtocolVersion 1, our current sync source is not primary, has no sync source
      * ("syncSourceHasSyncSource" is false), and only has data up to "myLastOpTime", returns true.
      *
-     * "now" is used to skip over currently blacklisted sync sources.
+     * "now" is used to skip over currently denylisted sync sources.
      */
     bool shouldChangeSyncSource(const HostAndPort& currentSource,
                                 const rpc::ReplSetMetadata& replMetadata,
                                 const rpc::OplogQueryMetadata& oqMetadata,
+                                const OpTime& lastOpTimeFetched,
                                 Date_t now) const;
 
+    /**
+     * Determines if a new sync source should be chosen when an error occurs. In this case
+     * we do not have current metadata from the sync source and so can only do a subset of
+     * the checks we do when we get a response.
+     */
+    bool shouldChangeSyncSourceOnError(const HostAndPort& currentSource,
+                                       const OpTime& lastOpTimeFetched,
+                                       Date_t now) const;
+    /**
+     * Returns true if we find an eligible sync source that is significantly closer than our current
+     * sync source.
+     */
+    bool shouldChangeSyncSourceDueToPingTime(const HostAndPort& currentSource,
+                                             const MemberState& memberState,
+                                             const OpTime& previousOpTimeFetched,
+                                             Date_t now,
+                                             ReadPreference readPreference);
+
     /**
      * Sets the reported mode of this node to one of RS_SECONDARY, RS_STARTUP2, RS_ROLLBACK or
      * RS_RECOVERING, when getRole() == Role::follower.  This is the interface by which the
@@ -269,9 +314,12 @@ public:
      * 'committedOpTime' if it has a different term than our lastApplied, unless
      * 'fromSyncSource'=true, which guarantees we are on the same branch of history as
      * 'committedOpTime', so we update our commit point to min(committedOpTime, lastApplied).
+     * The 'forInitiate' flag is to force-advance our committedOpTime during the execution of
+     * the replSetInitiate command.
      */
     bool advanceLastCommittedOpTimeAndWallTime(OpTimeAndWallTime committedOpTimeAndWallTime,
-                                               bool fromSyncSource);
+                                               bool fromSyncSource,
+                                               bool forInitiate = false);
 
     /**
      * Returns the OpTime of the latest majority-committed op known to this server.
@@ -281,7 +329,7 @@ public:
     OpTimeAndWallTime getLastCommittedOpTimeAndWallTime() const;
 
     /**
-     * Returns true if it's safe to transition to LeaderMode::kMaster.
+     * Returns true if it's safe to transition to LeaderMode::kWritablePrimary.
      */
     bool canCompleteTransitionToPrimary(long long termWhenDrainCompleted) const;
 
@@ -318,21 +366,22 @@ public:
                                  BSONObjBuilder* response,
                                  Status* result);
 
-    // produce a reply to a V1 heartbeat
-    Status prepareHeartbeatResponseV1(Date_t now,
-                                      const ReplSetHeartbeatArgsV1& args,
-                                      const std::string& ourSetName,
-                                      ReplSetHeartbeatResponse* response);
+    // produce a reply to a V1 heartbeat, and return whether the remote node's config has changed.
+    StatusWith<bool> prepareHeartbeatResponseV1(Date_t now,
+                                                const ReplSetHeartbeatArgsV1& args,
+                                                StringData ourSetName,
+                                                ReplSetHeartbeatResponse* response);
 
     struct ReplSetStatusArgs {
         const Date_t now;
         const unsigned selfUptime;
-        const OpTimeAndWallTime readConcernMajorityOpTime;
+        const OpTime readConcernMajorityOpTime;
         const BSONObj initialSyncStatus;
         const BSONObj electionCandidateMetrics;
         const BSONObj electionParticipantMetrics;
 
-        // boost::none if the storage engine does not support recovery to a timestamp.
+        // boost::none if the storage engine does not support recovery to a timestamp, or if the
+        // storage engine is not available.
         // Timestamp::min() if a stable recovery timestamp is yet to be taken.
         //
         // On the replication layer, a non-min() timestamp ensures recoverable rollback is possible,
@@ -351,11 +400,11 @@ public:
     StatusWith<BSONObj> prepareReplSetUpdatePositionCommand(
         OpTime currentCommittedSnapshotOpTime) const;
 
-    // Produce a reply to an ismaster request.  It is only valid to call this if we are a
-    // replset.  Drivers interpret the isMaster fields according to the Server Discovery and
+    // Produce a reply to a hello request.  It is only valid to call this if we are a
+    // replset.  Drivers interpret the hello fields according to the Server Discovery and
     // Monitoring Spec, see the "Parsing an isMaster response" section.
-    void fillIsMasterForReplSet(std::shared_ptr<IsMasterResponse> response,
-                                const StringData& horizonString) const;
+    void fillHelloForReplSet(std::shared_ptr<HelloResponse> response,
+                             const StringData& horizonString) const;
 
     // Produce member data for the serverStatus command and diagnostic logging.
     void fillMemberData(BSONObjBuilder* result);
@@ -402,7 +451,7 @@ public:
      * processHeartbeatResponse for the same "target".
      */
     std::pair<ReplSetHeartbeatArgsV1, Milliseconds> prepareHeartbeatRequestV1(
-        Date_t now, const std::string& ourSetName, const HostAndPort& target);
+        Date_t now, StringData ourSetName, const HostAndPort& target);
 
     /**
      * Processes a heartbeat response from "target" that arrived around "now", having spent
@@ -475,7 +524,7 @@ public:
      * Marks a member as down from our perspective and returns a bool which indicates if we can no
      * longer see a majority of the nodes and thus should step down.
      */
-    bool setMemberAsDown(Date_t now, const int memberIndex);
+    bool setMemberAsDown(Date_t now, int memberIndex);
 
     /**
      * Goes through the memberData and determines which member that is currently live
@@ -536,9 +585,7 @@ public:
      * Returns a Status if the position could not be set, false if the last optimes for the node
      * did not change, or true if either the last applied or last durable optime did change.
      */
-    StatusWith<bool> setLastOptime(const UpdatePositionArgs::UpdateInfo& args,
-                                   Date_t now,
-                                   long long* configVersion);
+    StatusWith<bool> setLastOptimeForMember(const UpdatePositionArgs::UpdateInfo& args, Date_t now);
 
     /**
      * Sets the latest optime committed in the previous config to the current lastCommitted optime.
@@ -668,7 +715,7 @@ public:
     /**
      * Set the outgoing heartbeat message from self
      */
-    void setMyHeartbeatMessage(const Date_t now, const std::string& s);
+    void setMyHeartbeatMessage(Date_t now, const std::string& s);
 
     /**
      * Prepares a ReplSetMetadata object describing the current term, primary, and lastOp
@@ -708,7 +755,7 @@ public:
     /**
      * Transitions to the candidate role if the node is electable.
      */
-    Status becomeCandidateIfElectable(const Date_t now, StartElectionReasonEnum reason);
+    Status becomeCandidateIfElectable(Date_t now, StartElectionReasonEnum reason);
 
     /**
      * Updates the storage engine read committed support in the TopologyCoordinator options after
@@ -719,7 +766,7 @@ public:
     /**
      * Reset the booleans to record the last heartbeat restart for the target node.
      */
-    void restartHeartbeat(const Date_t now, const HostAndPort& target);
+    void restartHeartbeat(Date_t now, const HostAndPort& target);
 
     /**
      * Increments the counter field of the current TopologyVersion.
@@ -777,6 +824,13 @@ public:
     void setCurrentPrimary_forTest(int primaryIndex,
                                    const Timestamp& electionTime = Timestamp(0, 0));
 
+    /**
+     * Get a raw pointer to the list of recent sync source changes. It is the caller's
+     * responsibility to not use this pointer beyond the lifetime of the object. Used for testing
+     * only.
+     */
+    RecentSyncSourceChanges* getRecentSyncSourceChanges_forTest();
+
     /**
      * Change config (version, term) of each member in the initial test config so that
      * it will be majority replicated without having to mock heartbeats.
@@ -815,7 +869,7 @@ private:
      *          |    ^  |                |                |
      *          |    |  |                |                |
      *          v    |  |                |                |
-     *       kMaster --------------------------           |
+     *       kWritablePrimary -----------------           |
      *        |  ^   |  |                |    |           |
      *        |  |   |  |                |    |           |
      *        |  |   |  |                |    |           |
@@ -823,11 +877,12 @@ private:
      *  kAttemptingStepDown----------->kSteppingDown------|
      */
     enum class LeaderMode {
-        kNotLeader,           // This node is not currently a leader.
-        kLeaderElect,         // This node has been elected leader, but can't yet accept writes.
-        kMaster,              // This node reports ismaster:true and can accept writes.
-        kSteppingDown,        // This node is in the middle of a hb, force reconfig or stepdown
-                              // command that must complete.
+        kNotLeader,        // This node is not currently a leader.
+        kLeaderElect,      // This node has been elected leader, but can't yet accept writes.
+        kWritablePrimary,  // This node can accept writes. Depending on whether the client sent
+                           // hello or isMaster, will report isWritablePrimary:true or ismaster:true
+        kSteppingDown,     // This node is in the middle of a hb, force reconfig or stepdown
+                           // command that must complete.
         kAttemptingStepDown,  // This node is in the middle of a cmd initiated step down that might
                               // fail.
     };
@@ -848,19 +903,16 @@ private:
     // Set what type of PRIMARY this node currently is.
     void _setLeaderMode(LeaderMode mode);
 
-    // Returns the number of heartbeat pings which have occurred.
-    int _getTotalPings();
+    // Returns a HostAndPort if one is forced via the 'replSetSyncFrom' command.
+    boost::optional<HostAndPort> _chooseSyncSourceReplSetSyncFrom(Date_t now);
 
     // Does preliminary checks involved in choosing sync source
     // * Do we have a valid configuration?
-    // * Do we have a forced sync source?
+    // * Is the 'forceSyncSourceCandidate' failpoint enabled?
     // * Have we gotten enough pings?
     // Returns a HostAndPort if one is decided (may be empty), boost:none if we need to move to the
     // next step.
-    boost::optional<HostAndPort> _chooseSyncSourceInitialStep(Date_t now);
-
-    // Returns a HostAndPort if one is forced via the 'unsupportedSyncSource' startup parameter.
-    boost::optional<HostAndPort> _chooseSyncSourceUnsupportedSyncSourceParameter(Date_t now);
+    boost::optional<HostAndPort> _chooseSyncSourceInitialChecks(Date_t now);
 
     // Returns the primary node if it is a valid sync source, otherwise returns an empty
     // HostAndPort.
@@ -873,7 +925,78 @@ private:
                                         const OpTime& lastOpTimeFetched,
                                         ReadPreference readPreference);
 
-    // Returns the current "ping" value for the given member by their address
+    // Does preliminary checkes to see if a new sync source should be chosen
+    // * Do we have a valid configuration -- if so, we do not change sync source.
+    // * Are we in initial sync -- if so, we do not change sync source.
+    // * Do we have a new forced sync source -- if so, we do change sync source.
+    // Returns decision and current sync source candidate if decision is kMaybe.
+    // (kMaybe indicates to continue with further checks).
+    enum class ChangeSyncSourceDecision { kNo, kYes, kMaybe };
+    std::pair<ChangeSyncSourceDecision, int> _shouldChangeSyncSourceInitialChecks(
+        const HostAndPort& currentSource) const;
+
+    // Returns true if we should choose a new sync source because chaining is disabled
+    // and there is a new primary.
+    bool _shouldChangeSyncSourceDueToNewPrimary(const HostAndPort& currentSource,
+                                                int currentSourceIndex) const;
+
+    // Change sync source if they are not ahead of us, and don't have a sync source.
+    // Note 'syncSourceIndex' is the index of our sync source's sync source. The 'currentSource'
+    // is our sync source.
+    bool _shouldChangeSyncSourceDueToSourceNotAhead(const HostAndPort& currentSource,
+                                                    int syncSourceIndex,
+                                                    bool syncSourceIsPrimary,
+                                                    const OpTime& currentSourceOpTime,
+                                                    const OpTime& lastOpTimeFetched) const;
+
+    // Change sync source if our sync source is also syncing from us when we are in primary
+    // catchup mode, forming a sync source selection cycle, and the sync source is not ahead
+    // of us.
+    // Note 'syncSourceHost' and 'syncSourceIndex' are the host and index of ourb sync source's
+    // sync source. The 'currentSource' is our sync source.
+    bool _shouldChangeSyncSourceToBreakCycle(const HostAndPort& currentSource,
+                                             const std::string& syncSourceHost,
+                                             int syncSourceIndex,
+                                             const OpTime& currentSourceOpTime,
+                                             const OpTime& lastOpTimeFetched) const;
+
+    // Returns true if we should choose a new sync source due to our current sync source being
+    // greater than maxSyncSourceLagSeconds and a better source being available.
+    bool _shouldChangeSyncSourceDueToLag(const HostAndPort& currentSource,
+                                         const OpTime& currentSourceOpTime,
+                                         const OpTime& lastOpTimeFetched,
+                                         Date_t now) const;
+
+    // Returns true if we should choose a new sync source because our current sync source does
+    // not match our strict criteria for sync source candidates, but another member does.
+    bool _shouldChangeSyncSourceDueToBetterEligibleSource(const HostAndPort& currentSource,
+                                                          int currentSourceIndex,
+                                                          const OpTime& lastOpTimeFetched,
+                                                          Date_t now) const;
+    /*
+     * Clear this node's sync source.
+     */
+    void _clearSyncSource();
+
+    /**
+     * Sets this node's sync source. It will also update whether the sync source was forced and add
+     * a new entry to recent sync source changes.
+     */
+    void _setSyncSource(HostAndPort newSyncSource, Date_t now, bool forced = false);
+
+    // Returns the oldest acceptable OpTime that a node must have for us to choose it as our sync
+    // source.
+    OpTime _getOldestSyncOpTime() const;
+
+    // Returns true if the candidate node is viable as our sync source.
+    bool _isEligibleSyncSource(int candidateIndex,
+                               Date_t now,
+                               const OpTime& lastOpTimeFetched,
+                               ReadPreference readPreference,
+                               bool firstAttempt,
+                               bool shouldCheckStaleness) const;
+
+    // Returns the current "ping" value for the given member by their address.
     Milliseconds _getPing(const HostAndPort& host);
 
     // Returns the index of the member with the matching id, or -1 if none match.
@@ -884,7 +1007,7 @@ private:
 
     // Checks if the node can see a healthy primary of equal or greater priority to the
     // candidate. If so, returns the index of that node. Otherwise returns -1.
-    int _findHealthyPrimaryOfEqualOrGreaterPriority(const int candidateIndex) const;
+    int _findHealthyPrimaryOfEqualOrGreaterPriority(int candidateIndex) const;
 
     // Is our optime close enough to the latest known optime to call for a priority takeover.
     bool _amIFreshEnoughForPriorityTakeover() const;
@@ -894,8 +1017,7 @@ private:
     bool _amIFreshEnoughForCatchupTakeover() const;
 
     // Returns reason why "self" member is unelectable
-    UnelectableReasonMask _getMyUnelectableReason(const Date_t now,
-                                                  StartElectionReasonEnum reason) const;
+    UnelectableReasonMask _getMyUnelectableReason(Date_t now, StartElectionReasonEnum reason) const;
 
     // Returns reason why memberIndex is unelectable
     UnelectableReasonMask _getUnelectableReason(int memberIndex) const;
@@ -916,13 +1038,13 @@ private:
     MemberData& _selfMemberData();
 
     // Index of self member in member data.
-    const int _selfMemberDataIndex() const;
+    int _selfMemberDataIndex() const;
 
     /*
      * Returns information we have on the state of the node identified by memberId.  Returns
      * nullptr if memberId is not found in the configuration.
      */
-    MemberData* _findMemberDataByMemberId(const int memberId);
+    MemberData* _findMemberDataByMemberId(int memberId);
 
     /**
      * Performs updating "_currentPrimaryIndex" for processHeartbeatResponse().
@@ -956,12 +1078,12 @@ private:
     void _stepDownSelfAndReplaceWith(int newPrimary);
 
     /**
-     * Looks up the provided member in the blacklist and returns true if the member's blacklist
+     * Looks up the provided member in the denylist and returns true if the member's denylist
      * expire time is after 'now'.  If the member is found but the expire time is before 'now',
-     * the function returns false.  If the member is not found in the blacklist, the function
+     * the function returns false.  If the member is not found in the denylist, the function
      * returns false.
      **/
-    bool _memberIsBlacklisted(const MemberConfig& memberConfig, Date_t now) const;
+    bool _memberIsDenylisted(const MemberConfig& memberConfig, Date_t now) const;
 
     // Returns a string representation of the current replica set status for logging purposes.
     std::string _getReplSetStatusString();
@@ -990,9 +1112,11 @@ private:
     HostAndPort _syncSource;
     // These members are not chosen as sync sources for a period of time, due to connection
     // issues with them
-    std::map<HostAndPort, Date_t> _syncSourceBlacklist;
+    std::map<HostAndPort, Date_t> _syncSourceDenylist;
     // The next sync source to be chosen, requested via a replSetSyncFrom command
     int _forceSyncSourceIndex;
+    // Whether the current sync source has been set via a replSetSyncFrom command
+    bool _replSetSyncFromSet;
 
     // Options for this TopologyCoordinator
     Options _options;
@@ -1066,6 +1190,8 @@ private:
 
     // Whether or not the storage engine supports read committed.
     ReadCommittedSupport _storageEngineSupportsReadCommitted{ReadCommittedSupport::kUnknown};
+
+    RecentSyncSourceChanges _recentSyncSourceChanges;
 };
 
 /**
