@@ -27,27 +27,27 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/repl/transaction_oplog_application.h"
 
-#include "mongo/db/background.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/commands/txn_cmds_gen.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/index_builds_coordinator.h"
-#include "mongo/db/op_observer.h"
 #include "mongo/db/repl/apply_ops.h"
 #include "mongo/db/repl/storage_interface_impl.h"
 #include "mongo/db/repl/timestamp_block.h"
-#include "mongo/db/session_catalog_mongod.h"
-#include "mongo/db/transaction_history_iterator.h"
-#include "mongo/db/transaction_participant.h"
+#include "mongo/db/session/session_catalog_mongod.h"
+#include "mongo/db/transaction/transaction_history_iterator.h"
+#include "mongo/db/transaction/transaction_participant.h"
 #include "mongo/logv2/log.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
+
+
 namespace mongo {
 using repl::OplogEntry;
 namespace {
@@ -70,6 +70,10 @@ Status _applyOperationsForTransaction(OperationContext* opCtx,
     // Apply each the operations via repl::applyOperation.
     for (const auto& op : ops) {
         try {
+            if (op.getOpType() == repl::OpTypeEnum::kNoop) {
+                continue;
+            }
+
             // Presently, it is not allowed to run a prepared transaction with a command
             // inside. TODO(SERVER-46105)
             invariant(!op.isCommand());
@@ -97,7 +101,7 @@ Status _applyOperationsForTransaction(OperationContext* opCtx,
                     "Error applying operation in transaction. {error}- oplog entry: {oplogEntry}",
                     "Error applying operation in transaction",
                     "error"_attr = redact(ex),
-                    "oplogEntry"_attr = redact(op.toBSON()));
+                    "oplogEntry"_attr = redact(op.toBSONForLogging()));
                 return exceptionToStatus();
             }
             LOGV2_DEBUG(21846,
@@ -108,7 +112,7 @@ Status _applyOperationsForTransaction(OperationContext* opCtx,
                         "Encountered but ignoring error while applying operations for transaction "
                         "because we are either in initial sync or recovering mode",
                         "error"_attr = redact(ex),
-                        "oplogEntry"_attr = redact(op.toBSON()),
+                        "oplogEntry"_attr = redact(op.toBSONForLogging()),
                         "oplogApplicationMode"_attr =
                             repl::OplogApplication::modeToString(oplogApplicationMode));
         }
@@ -145,6 +149,8 @@ Status _applyTransactionFromOplogChain(OperationContext* opCtx,
 
         status = _applyOperationsForTransaction(opCtx, ops, mode);
         if (status.isOK()) {
+            // If the transaction was empty then we have no locks, ensure at least Global IX.
+            Lock::GlobalLock lk(opCtx, MODE_IX);
             opCtx->recoveryUnit()->setPrepareTimestamp(commitTimestamp);
             wunit.prepare();
 
@@ -164,11 +170,10 @@ Status _applyTransactionFromOplogChain(OperationContext* opCtx,
 /**
  * Helper used to get previous oplog entry from the same transaction.
  */
-const repl::OplogEntry getPreviousOplogEntry(OperationContext* opCtx,
-                                             const repl::OplogEntry& entry) {
+repl::OplogEntry getPreviousOplogEntry(OperationContext* opCtx, const repl::OplogEntry& entry) {
     const auto prevOpTime = entry.getPrevWriteOpTimeInTransaction();
     invariant(prevOpTime);
-    TransactionHistoryIterator iter(prevOpTime.get());
+    TransactionHistoryIterator iter(prevOpTime.value());
     invariant(iter.hasNext());
     const auto prevOplogEntry = iter.next(opCtx);
 
@@ -179,7 +184,7 @@ const repl::OplogEntry getPreviousOplogEntry(OperationContext* opCtx,
 Status applyCommitTransaction(OperationContext* opCtx,
                               const OplogEntry& entry,
                               repl::OplogApplication::Mode mode) {
-    IDLParserErrorContext ctx("commitTransaction");
+    IDLParserContext ctx("commitTransaction");
     auto commitOplogEntryOpTime = entry.getOpTime();
     auto commitCommand = CommitTransactionOplogObject::parse(ctx, entry.getObject());
     invariant(commitCommand.getCommitTimestamp());
@@ -207,6 +212,9 @@ Status applyCommitTransaction(OperationContext* opCtx,
             invariant(entry.getTxnNumber());
             opCtx->setLogicalSessionId(*entry.getSessionId());
             opCtx->setTxnNumber(*entry.getTxnNumber());
+            if (auto txnRetryCounter = entry.getOperationSessionInfo().getTxnRetryCounter()) {
+                opCtx->setTxnRetryCounter(*txnRetryCounter);
+            }
             opCtx->setInMultiDocumentTransaction();
 
             // The write on transaction table may be applied concurrently, so refreshing state
@@ -249,6 +257,9 @@ Status applyAbortTransaction(OperationContext* opCtx,
             invariant(entry.getTxnNumber());
             opCtx->setLogicalSessionId(*entry.getSessionId());
             opCtx->setTxnNumber(*entry.getTxnNumber());
+            if (auto txnRetryCounter = entry.getOperationSessionInfo().getTxnRetryCounter()) {
+                opCtx->setTxnRetryCounter(*txnRetryCounter);
+            }
             opCtx->setInMultiDocumentTransaction();
 
             // The write on transaction table may be applied concurrently, so refreshing state
@@ -288,7 +299,7 @@ std::pair<std::vector<OplogEntry>, bool> _readTransactionOperationsFromOplogChai
     const auto lastEntryWrittenToOplogOpTime = oldestEntryInBatch.getPrevWriteOpTimeInTransaction();
     invariant(lastEntryWrittenToOplogOpTime < lastEntryInTxn.getOpTime());
 
-    TransactionHistoryIterator iter(lastEntryWrittenToOplogOpTime.get());
+    TransactionHistoryIterator iter(lastEntryWrittenToOplogOpTime.value());
 
     // If we started with a prepared commit, we want to forget about that operation and move onto
     // the prepare.
@@ -304,7 +315,7 @@ std::pair<std::vector<OplogEntry>, bool> _readTransactionOperationsFromOplogChai
     // The non-DurableReplOperation fields of the extracted transaction operations will match those
     // of the lastEntryInTxn. For a prepared commit, this will include the commit oplog entry's
     // 'ts' field, which is what we want.
-    auto lastEntryInTxnObj = lastEntryInTxn.toBSON();
+    auto lastEntryInTxnObj = lastEntryInTxn.getEntry().toBSON();
 
     // First retrieve and transform the ops from the oplog, which will be retrieved in reverse
     // order.
@@ -401,6 +412,10 @@ Status _applyPrepareTransaction(OperationContext* opCtx,
     // a secondary if a prepared transaction blocks on an index build, but the index build can't
     // re-acquire its X lock because of the transaction.
     for (const auto& op : ops) {
+        if (op.getOpType() == repl::OpTypeEnum::kNoop) {
+            continue;
+        }
+
         auto indexBuildsCoord = IndexBuildsCoordinator::get(opCtx);
         auto ns = op.getNss();
         auto uuid = *op.getUuid();
@@ -420,6 +435,9 @@ Status _applyPrepareTransaction(OperationContext* opCtx,
     invariant(entry.getTxnNumber());
     opCtx->setLogicalSessionId(*entry.getSessionId());
     opCtx->setTxnNumber(*entry.getTxnNumber());
+    if (auto txnRetryCounter = entry.getOperationSessionInfo().getTxnRetryCounter()) {
+        opCtx->setTxnRetryCounter(*txnRetryCounter);
+    }
     opCtx->setInMultiDocumentTransaction();
 
     return writeConflictRetry(opCtx, "applying prepare transaction", entry.getNss().ns(), [&] {
@@ -437,7 +455,7 @@ Status _applyPrepareTransaction(OperationContext* opCtx,
 
         // Release the WUOW, transaction lock resources and abort storage transaction so that the
         // writeConflictRetry loop will be able to retry applying transactional ops on WCE error.
-        auto abortOnError = makeGuard([&txnParticipant, opCtx] {
+        ScopeGuard abortOnError([&txnParticipant, opCtx] {
             // Abort the transaction and invalidate the session it is associated with.
             txnParticipant.abortTransaction(opCtx);
             txnParticipant.invalidate(opCtx);
@@ -455,6 +473,15 @@ Status _applyPrepareTransaction(OperationContext* opCtx,
 
         auto status = _applyOperationsForTransaction(opCtx, ops, mode);
 
+        if (opCtx->isRetryableWrite()) {
+            for (const auto& op : ops) {
+                if (!op.getStatementIds().empty()) {
+                    txnParticipant.addCommittedStmtIds(
+                        opCtx, op.getStatementIds(), entry.getOpTime());
+                }
+            }
+        }
+
         if (MONGO_unlikely(applyPrepareTxnOpsFailsWithWriteConflict.shouldFail())) {
             LOGV2(4947101, "Hit applyPrepareTxnOpsFailsWithWriteConflict failpoint");
             status = Status(ErrorCodes::WriteConflict,
@@ -462,7 +489,8 @@ Status _applyPrepareTransaction(OperationContext* opCtx,
         }
 
         if (status == ErrorCodes::WriteConflict) {
-            throw WriteConflictException();
+            throwWriteConflictException(
+                "Conflict encountered when applying a prepare transaction.");
         }
         fassert(31137, status);
 
@@ -472,11 +500,6 @@ Status _applyPrepareTransaction(OperationContext* opCtx,
         }
 
         txnParticipant.prepareTransaction(opCtx, entry.getOpTime());
-
-        auto opObserver = opCtx->getServiceContext()->getOpObserver();
-        invariant(opObserver);
-        opObserver->onTransactionPrepareNonPrimary(opCtx, ops, entry.getOpTime());
-
         // Prepare transaction success.
         abortOnError.dismiss();
 
@@ -564,15 +587,16 @@ void reconstructPreparedTransactions(OperationContext* opCtx, repl::OplogApplica
               opCtx->recoveryUnit()->getTimestampReadSource());
 
     DBDirectClient client(opCtx);
-    const auto cursor = client.query(NamespaceString::kSessionTransactionsTableNamespace,
-                                     {BSON("state"
-                                           << "prepared")});
+    FindCommandRequest findRequest{NamespaceString::kSessionTransactionsTableNamespace};
+    findRequest.setFilter(BSON("state"
+                               << "prepared"));
+    const auto cursor = client.find(std::move(findRequest));
 
     // Iterate over each entry in the transactions table that has a prepared transaction.
     while (cursor->more()) {
         const auto txnRecordObj = cursor->next();
         const auto txnRecord = SessionTxnRecord::parse(
-            IDLParserErrorContext("recovering prepared transaction"), txnRecordObj);
+            IDLParserContext("recovering prepared transaction"), txnRecordObj);
 
         invariant(txnRecord.getState() == DurableTxnStateEnum::kPrepared);
 
@@ -591,10 +615,7 @@ void reconstructPreparedTransactions(OperationContext* opCtx, repl::OplogApplica
             AlternativeClientRegion acr(newClient);
             const auto newOpCtx = cc().makeOperationContext();
 
-            // Ignore interruptions while reconstructing prepared transactions, so that we do not
-            // fassert and crash due to interruptions inside this call.
-            newOpCtx->runWithoutInterruptionExceptAtGlobalShutdown(
-                [&] { _reconstructPreparedTransaction(newOpCtx.get(), prepareOplogEntry, mode); });
+            _reconstructPreparedTransaction(newOpCtx.get(), prepareOplogEntry, mode);
         }
     }
 }
