@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
@@ -35,7 +34,6 @@
 
 #include "mongo/base/status_with.h"
 #include "mongo/db/bson/dotted_path_support.h"
-#include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/dbhelpers.h"
 #include "mongo/db/exec/working_set_common.h"
@@ -44,8 +42,13 @@
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/query/internal_plans.h"
 #include "mongo/db/query/plan_executor.h"
+#include "mongo/db/s/shard_key_index_util.h"
+
 #include "mongo/logv2/log.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
+
+
 namespace mongo {
 namespace {
 
@@ -71,10 +74,10 @@ BSONObj prettyKey(const BSONObj& keyPattern, const BSONObj& key) {
  * object extended to cover the entire shardKey. See KeyPattern::extendRangeBound documentation for
  * some examples.
  */
-const std::tuple<BSONObj, BSONObj> getMinMaxExtendedBounds(const IndexDescriptor* shardKeyIdx,
-                                                           const BSONObj& min,
-                                                           const BSONObj& max) {
-    KeyPattern kp(shardKeyIdx->keyPattern());
+std::tuple<BSONObj, BSONObj> getMinMaxExtendedBounds(const ShardKeyIndex& shardKeyIdx,
+                                                     const BSONObj& min,
+                                                     const BSONObj& max) {
+    KeyPattern kp(shardKeyIdx.keyPattern());
 
     // Extend min to get (min, MinKey, MinKey, ....)
     BSONObj minKey = Helpers::toKeyFormat(kp.extendRangeBound(min, false /* upperInclusive */));
@@ -90,48 +93,6 @@ const std::tuple<BSONObj, BSONObj> getMinMaxExtendedBounds(const IndexDescriptor
     return {minKey, maxKey};
 }
 
-/*
- * Returns true if the final key in the range is the same as the first key, false otherwise.
- */
-bool maxKeyEqualToMinKey(OperationContext* opCtx,
-                         const Collection* collection,
-                         const IndexDescriptor* shardKeyIdx,
-                         const BSONObj& minBound,
-                         const BSONObj& maxBound,
-                         const BSONObj& minKeyInChunk) {
-    BSONObj maxKeyInChunk;
-    {
-        auto backwardIdxScanner = InternalPlanner::indexScan(opCtx,
-                                                             collection,
-                                                             shardKeyIdx,
-                                                             maxBound,
-                                                             minBound,
-                                                             BoundInclusion::kIncludeEndKeyOnly,
-                                                             PlanExecutor::YIELD_AUTO,
-                                                             InternalPlanner::BACKWARD);
-
-        PlanExecutor::ExecState state = backwardIdxScanner->getNext(&maxKeyInChunk, nullptr);
-        uassert(ErrorCodes::OperationFailed,
-                "can't open a cursor to find final key in range (desired range is possibly empty)",
-                state == PlanExecutor::ADVANCED);
-    }
-
-    if (minKeyInChunk.woCompare(maxKeyInChunk) == 0) {
-        // Range contains only documents with a single key value.  So we cannot possibly find a
-        // split point, and there is no need to scan any further.
-        LOGV2_WARNING(
-            5865001,
-            "Possible low cardinality key detected in range. Range contains only a single key.",
-            "namespace"_attr = collection->ns(),
-            "minKey"_attr = redact(prettyKey(shardKeyIdx->keyPattern(), minBound)),
-            "maxKey"_attr = redact(prettyKey(shardKeyIdx->keyPattern(), maxBound)),
-            "key"_attr = redact(prettyKey(shardKeyIdx->keyPattern(), minKeyInChunk)));
-        return true;
-    }
-
-    return false;
-}
-
 /*
  * Reshuffle fields according to the shard key pattern.
  */
@@ -144,13 +105,19 @@ auto orderShardKeyFields(const BSONObj& keyPattern, const BSONObj& key) {
 
 }  // namespace
 
-std::vector<BSONObj> autoSplitVector(OperationContext* opCtx,
-                                     const NamespaceString& nss,
-                                     const BSONObj& keyPattern,
-                                     const BSONObj& min,
-                                     const BSONObj& max,
-                                     long long maxChunkSizeBytes) {
+std::pair<std::vector<BSONObj>, bool> autoSplitVector(OperationContext* opCtx,
+                                                      const NamespaceString& nss,
+                                                      const BSONObj& keyPattern,
+                                                      const BSONObj& min,
+                                                      const BSONObj& max,
+                                                      long long maxChunkSizeBytes,
+                                                      boost::optional<int> limit) {
+    if (limit) {
+        uassert(ErrorCodes::InvalidOptions, "autoSplitVector expects a positive limit", *limit > 0);
+    }
+
     std::vector<BSONObj> splitKeys;
+    bool reachedMaxBSONSize = false;  // True if the split points vector becomes too big
 
     int elapsedMillisToFindSplitPoints;
 
@@ -158,10 +125,11 @@ std::vector<BSONObj> autoSplitVector(OperationContext* opCtx,
     auto tooFrequentKeys = SimpleBSONObjComparator::kInstance.makeBSONObjSet();
 
     {
-        AutoGetCollection autoGetColl(opCtx, nss, MODE_IS);
-        auto* collection = autoGetColl.getCollection();
+        AutoGetCollection collection(opCtx, nss, MODE_IS);
 
-        uassert(ErrorCodes::NamespaceNotFound, "ns not found", collection);
+        uassert(ErrorCodes::NamespaceNotFound,
+                str::stream() << "namespace " << nss << " does not exists",
+                collection);
 
         // Get the size estimate for this namespace
         const long long totalLocalCollDocuments = collection->numRecords(opCtx);
@@ -174,44 +142,74 @@ std::vector<BSONObj> autoSplitVector(OperationContext* opCtx,
 
         // Allow multiKey based on the invariant that shard keys must be single-valued. Therefore,
         // any multi-key index prefixed by shard key cannot be multikey over the shard key fields.
-        auto catalog = collection->getIndexCatalog();
-        auto shardKeyIdx =
-            catalog->findShardKeyPrefixedIndex(opCtx, keyPattern, /*requireSingleKey=*/false);
+        auto shardKeyIdx = findShardKeyPrefixedIndex(opCtx,
+                                                     *collection,
+                                                     collection->getIndexCatalog(),
+                                                     keyPattern,
+                                                     /*requireSingleKey=*/false);
         uassert(ErrorCodes::IndexNotFound,
                 str::stream() << "couldn't find index over splitting key "
                               << keyPattern.clientReadable().toString(),
                 shardKeyIdx);
 
-        const auto [minKey, maxKey] = getMinMaxExtendedBounds(shardKeyIdx, min, max);
+        const auto [minKey, maxKey] = getMinMaxExtendedBounds(*shardKeyIdx, min, max);
 
         // Setup the index scanner that will be used to find the split points
-        auto forwardIdxScanner = InternalPlanner::indexScan(opCtx,
-                                                            collection,
-                                                            shardKeyIdx,
-                                                            minKey,
-                                                            maxKey,
-                                                            BoundInclusion::kIncludeStartKeyOnly,
-                                                            PlanExecutor::YIELD_AUTO,
-                                                            InternalPlanner::FORWARD);
+        auto forwardIdxScanner =
+            InternalPlanner::shardKeyIndexScan(opCtx,
+                                               &(*collection),
+                                               *shardKeyIdx,
+                                               minKey,
+                                               maxKey,
+                                               BoundInclusion::kIncludeStartKeyOnly,
+                                               PlanYieldPolicy::YieldPolicy::YIELD_AUTO,
+                                               InternalPlanner::FORWARD);
 
         // Get minimum key belonging to the chunk
         BSONObj minKeyInOriginalChunk;
         {
             PlanExecutor::ExecState state =
                 forwardIdxScanner->getNext(&minKeyInOriginalChunk, nullptr);
-            uassert(ErrorCodes::OperationFailed,
-                    "can't open a cursor to scan the range (desired range is possibly empty)",
-                    state == PlanExecutor::ADVANCED);
+            if (state == PlanExecutor::IS_EOF) {
+                // Range is empty
+                return {};
+            }
         }
 
-        // Return empty vector if chunk's min and max keys are the same.
-        if (maxKeyEqualToMinKey(
-                opCtx, collection, shardKeyIdx, minKey, maxKey, minKeyInOriginalChunk)) {
+        BSONObj maxKeyInChunk;
+        {
+            auto backwardIdxScanner =
+                InternalPlanner::shardKeyIndexScan(opCtx,
+                                                   &(*collection),
+                                                   *shardKeyIdx,
+                                                   maxKey,
+                                                   minKey,
+                                                   BoundInclusion::kIncludeEndKeyOnly,
+                                                   PlanYieldPolicy::YieldPolicy::YIELD_AUTO,
+                                                   InternalPlanner::BACKWARD);
+
+            PlanExecutor::ExecState state = backwardIdxScanner->getNext(&maxKeyInChunk, nullptr);
+            if (state == PlanExecutor::IS_EOF) {
+                // Range is empty
+                return {};
+            }
+        }
+
+        if (minKeyInOriginalChunk.woCompare(maxKeyInChunk) == 0) {
+            // Range contains only documents with a single key value.  So we cannot possibly find a
+            // split point, and there is no need to scan any further.
+            LOGV2_WARNING(
+                5865001,
+                "Possible low cardinality key detected in range. Range contains only a single key.",
+                "namespace"_attr = collection.getNss(),
+                "minKey"_attr = redact(prettyKey(keyPattern, minKey)),
+                "maxKey"_attr = redact(prettyKey(keyPattern, maxKey)),
+                "key"_attr = redact(prettyKey(shardKeyIdx->keyPattern(), minKeyInOriginalChunk)));
             return {};
         }
 
         LOGV2(5865000,
-              "Requested split points lookup for chunk",
+              "Requested split points lookup for range",
               "namespace"_attr = nss,
               "minKey"_attr = redact(prettyKey(keyPattern, minKey)),
               "maxKey"_attr = redact(prettyKey(keyPattern, maxKey)));
@@ -226,7 +224,6 @@ std::vector<BSONObj> autoSplitVector(OperationContext* opCtx,
         BSONObj currentKey;               // Last key seen during the index scan
         long long numScannedKeys = 1;     // minKeyInOriginalChunk has already been scanned
         std::size_t resultArraySize = 0;  // Approximate size in bytes of the split points array
-        bool reachedMaxBSONSize = false;  // True if the split points vector becomes too big
 
         // Lambda to check whether the split points vector would exceed BSONObjMaxUserSize in case
         // of additional split key of the specified size.
@@ -246,7 +243,11 @@ std::vector<BSONObj> autoSplitVector(OperationContext* opCtx,
             if (++numScannedKeys >= maxDocsPerChunk) {
                 currentKey = orderShardKeyFields(keyPattern, currentKey);
 
-                if (currentKey.woCompare(lastSplitPoint) == 0) {
+                const auto compareWithPreviousSplitPoint = currentKey.woCompare(lastSplitPoint);
+                dassert(compareWithPreviousSplitPoint >= 0,
+                        str::stream() << "Found split key smaller then the last one: " << currentKey
+                                      << " < " << lastSplitPoint);
+                if (compareWithPreviousSplitPoint == 0) {
                     // Do not add again the same split point in case of frequent shard key.
                     tooFrequentKeys.insert(currentKey.getOwned());
                     continue;
@@ -270,6 +271,12 @@ std::vector<BSONObj> autoSplitVector(OperationContext* opCtx,
                 lastSplitPoint = splitKeys.back();
                 numScannedKeys = 0;
 
+                if (limit && splitKeys.size() == static_cast<size_t>(*limit + 1)) {
+                    // If the user has specified a limit, calculate the first `limit + 1` split
+                    // points (avoid creating small chunks)
+                    break;
+                }
+
                 LOGV2_DEBUG(5865003, 4, "Picked a split key", "key"_attr = redact(currentKey));
             }
         }
@@ -297,15 +304,15 @@ std::vector<BSONObj> autoSplitVector(OperationContext* opCtx,
                 // Fairly recalculate the last `nSplitPointsToReposition` split points.
                 splitKeys.erase(splitKeys.end() - nSplitPointsToReposition, splitKeys.end());
 
-                auto forwardIdxScanner =
-                    InternalPlanner::indexScan(opCtx,
-                                               collection,
-                                               shardKeyIdx,
-                                               splitKeys.empty() ? minKeyElement : splitKeys.back(),
-                                               maxKey,
-                                               BoundInclusion::kIncludeStartKeyOnly,
-                                               PlanExecutor::YIELD_AUTO,
-                                               InternalPlanner::FORWARD);
+                auto forwardIdxScanner = InternalPlanner::shardKeyIndexScan(
+                    opCtx,
+                    &collection.getCollection(),
+                    *shardKeyIdx,
+                    splitKeys.empty() ? minKeyElement : splitKeys.back(),
+                    maxKey,
+                    BoundInclusion::kIncludeStartKeyOnly,
+                    PlanYieldPolicy::YieldPolicy::YIELD_AUTO,
+                    InternalPlanner::FORWARD);
 
                 numScannedKeys = 0;
 
@@ -316,6 +323,10 @@ std::vector<BSONObj> autoSplitVector(OperationContext* opCtx,
 
                         const auto compareWithPreviousSplitPoint =
                             currentKey.woCompare(previousSplitPoint);
+
+                        dassert(compareWithPreviousSplitPoint >= 0,
+                                str::stream() << "Found split key smaller then the previous one: "
+                                              << currentKey << " < " << previousSplitPoint);
                         if (compareWithPreviousSplitPoint > 0) {
                             const auto additionalKeySize =
                                 currentKey.objsize() + estimatedAdditionalBytesPerItemInBSONArray;
@@ -368,12 +379,11 @@ std::vector<BSONObj> autoSplitVector(OperationContext* opCtx,
                       "duration"_attr = Milliseconds(elapsedMillisToFindSplitPoints));
     }
 
-    // TODO SERVER-58750: investigate if it is really needed to sort the vector
-    // Make sure splitKeys is in ascending order
-    std::sort(
-        splitKeys.begin(), splitKeys.end(), SimpleBSONObjComparator::kInstance.makeLessThan());
+    if (limit && splitKeys.size() > static_cast<size_t>(*limit)) {
+        splitKeys.resize(*limit);
+    }
 
-    return splitKeys;
+    return std::make_pair(std::move(splitKeys), reachedMaxBSONSize);
 }
 
 }  // namespace mongo
