@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
@@ -39,65 +38,146 @@
 
 #include "mongo/base/status_with.h"
 #include "mongo/bson/bsonobj_comparator_interface.h"
-#include "mongo/db/s/sharding_runtime_d_params_gen.h"
+#include "mongo/db/s/sharding_config_server_parameters_gen.h"
+#include "mongo/db/s/sharding_util.h"
 #include "mongo/logv2/log.h"
 #include "mongo/platform/bits.h"
 #include "mongo/s/balancer_configuration.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog/type_collection.h"
-#include "mongo/s/catalog/type_tags.h"
 #include "mongo/s/catalog_cache.h"
-#include "mongo/s/chunk_manager.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/request_types/get_stats_for_balancing_gen.h"
+#include "mongo/s/sharding_feature_flags_gen.h"
 #include "mongo/util/str.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
+
+
 namespace mongo {
 
-using MigrateInfoVector = BalancerChunkSelectionPolicy::MigrateInfoVector;
-using SplitInfoVector = BalancerChunkSelectionPolicy::SplitInfoVector;
-using std::shared_ptr;
-using std::unique_ptr;
-using std::vector;
-
 namespace {
 
-StatusWith<ZoneInfo> createCollectionZoneInfo(OperationContext* opCtx,
-                                              const NamespaceString& nss,
-                                              const KeyPattern& keyPattern) {
-    ZoneInfo zoneInfo;
-    const auto swCollectionTags =
-        Grid::get(opCtx)->catalogClient()->getTagsForCollection(opCtx, nss);
-    if (!swCollectionTags.isOK()) {
-        return swCollectionTags.getStatus().withContext(
-            str::stream() << "Unable to load tags for collection " << nss);
-    }
-    const auto& collectionTags = swCollectionTags.getValue();
-
-    for (const auto& tag : collectionTags) {
-        auto status =
-            zoneInfo.addRangeToZone(ZoneRange{keyPattern.extendRangeBound(tag.getMinKey(), false),
-                                              keyPattern.extendRangeBound(tag.getMaxKey(), false),
-                                              tag.getTag()});
-        if (!status.isOK()) {
-            return status;
-        }
-    }
-    return {std::move(zoneInfo)};
-}
-
 /**
  * Does a linear pass over the information cached in the specified chunk manager and extracts chunk
  * distribution and chunk placement information which is needed by the balancer policy.
  */
 StatusWith<DistributionStatus> createCollectionDistributionStatus(
-    OperationContext* opCtx, const ShardStatisticsVector& allShards, ChunkManager* chunkMgr) {
+    OperationContext* opCtx,
+    const NamespaceString& nss,
+    const ShardStatisticsVector& allShards,
+    const ChunkManager& chunkMgr) {
+    ShardToChunksMap shardToChunksMap;
 
-    auto swZoneInfo = createCollectionZoneInfo(
-        opCtx, chunkMgr->getns(), chunkMgr->getShardKeyPattern().getKeyPattern());
+    // Makes sure there is an entry in shardToChunksMap for every shard, so empty shards will also
+    // be accounted for
+    for (const auto& stat : allShards) {
+        shardToChunksMap[stat.shardId];
+    }
+
+    chunkMgr.forEachChunk([&](const auto& chunkEntry) {
+        ChunkType chunk;
+        chunk.setCollectionUUID(chunkMgr.getUUID());
+        chunk.setMin(chunkEntry.getMin());
+        chunk.setMax(chunkEntry.getMax());
+        chunk.setJumbo(chunkEntry.isJumbo());
+        chunk.setShard(chunkEntry.getShardId());
+        chunk.setVersion(chunkEntry.getLastmod());
+
+        shardToChunksMap[chunkEntry.getShardId()].push_back(chunk);
+
+        return true;
+    });
+
+    const auto& keyPattern = chunkMgr.getShardKeyPattern().getKeyPattern();
+
+    // Cache the collection zones
+    auto swZoneInfo = ZoneInfo::getZonesForCollection(opCtx, nss, keyPattern);
     if (!swZoneInfo.isOK()) {
         return swZoneInfo.getStatus();
     }
 
-    return {DistributionStatus{chunkMgr->getns(), std::move(swZoneInfo.getValue()), chunkMgr}};
+    DistributionStatus distribution(nss, std::move(shardToChunksMap), swZoneInfo.getValue());
+
+    return {std::move(distribution)};
+}
+
+stdx::unordered_map<NamespaceString, CollectionDataSizeInfoForBalancing>
+getDataSizeInfoForCollections(OperationContext* opCtx,
+                              const std::vector<CollectionType>& collections) {
+    const auto balancerConfig = Grid::get(opCtx)->getBalancerConfiguration();
+    uassertStatusOK(balancerConfig->refreshAndCheck(opCtx));
+
+    const auto shardRegistry = Grid::get(opCtx)->shardRegistry();
+    const auto shardIds = shardRegistry->getAllShardIds(opCtx);
+
+    // Map to be returned, incrementally populated with the collected statistics
+    stdx::unordered_map<NamespaceString, CollectionDataSizeInfoForBalancing> dataSizeInfoMap;
+
+    std::vector<NamespaceWithOptionalUUID> namespacesWithUUIDsForStatsRequest;
+    for (const auto& coll : collections) {
+        const auto& nss = coll.getNss();
+        const auto maxChunkSizeBytes =
+            coll.getMaxChunkSizeBytes().value_or(balancerConfig->getMaxChunkSizeBytes());
+
+        dataSizeInfoMap.emplace(
+            nss,
+            CollectionDataSizeInfoForBalancing(std::map<ShardId, int64_t>(), maxChunkSizeBytes));
+
+        NamespaceWithOptionalUUID nssWithUUID(nss);
+        nssWithUUID.setUUID(coll.getUuid());
+        namespacesWithUUIDsForStatsRequest.push_back(nssWithUUID);
+    }
+
+    ShardsvrGetStatsForBalancing req{namespacesWithUUIDsForStatsRequest};
+    req.setScaleFactor(1);
+    const auto reqObj = req.toBSON({});
+
+    const auto executor = Grid::get(opCtx)->getExecutorPool()->getFixedExecutor();
+    const auto responsesFromShards =
+        sharding_util::sendCommandToShards(opCtx,
+                                           NamespaceString::kAdminDb.toString(),
+                                           reqObj,
+                                           shardIds,
+                                           executor,
+                                           false /* throwOnError */);
+
+    for (auto&& response : responsesFromShards) {
+        try {
+            const auto& shardId = response.shardId;
+            const auto errorContext =
+                "Failed to get stats for balancing from shard '{}'"_format(shardId.toString());
+            const auto responseValue =
+                uassertStatusOKWithContext(std::move(response.swResponse), errorContext);
+
+            const ShardsvrGetStatsForBalancingReply reply =
+                ShardsvrGetStatsForBalancingReply::parse(
+                    IDLParserContext("ShardsvrGetStatsForBalancingReply"),
+                    std::move(responseValue.data));
+            const auto collStatsFromShard = reply.getStats();
+
+            invariant(collStatsFromShard.size() == collections.size());
+            for (const auto& stats : collStatsFromShard) {
+                invariant(dataSizeInfoMap.contains(stats.getNs()));
+                dataSizeInfoMap.at(stats.getNs()).shardToDataSizeMap[shardId] = stats.getCollSize();
+            }
+        } catch (const ExceptionFor<ErrorCodes::ShardNotFound>& ex) {
+            // Handle `removeShard`: skip shards removed during a balancing round
+            LOGV2_DEBUG(6581603,
+                        1,
+                        "Skipping shard for the current balancing round",
+                        "error"_attr = redact(ex));
+        }
+    }
+
+    return dataSizeInfoMap;
+}
+
+CollectionDataSizeInfoForBalancing getDataSizeInfoForCollection(OperationContext* opCtx,
+                                                                const NamespaceString& nss) {
+    const auto coll = Grid::get(opCtx)->catalogClient()->getCollection(opCtx, nss);
+    std::vector<CollectionType> vec{coll};
+    return std::move(getDataSizeInfoForCollections(opCtx, vec).at(nss));
 }
 
 /**
@@ -114,8 +194,7 @@ public:
     SplitCandidatesBuffer(NamespaceString nss, ChunkVersion collectionVersion)
         : _nss(std::move(nss)),
           _collectionVersion(collectionVersion),
-          _chunkSplitPoints(SimpleBSONObjComparator::kInstance
-                                .makeBSONObjIndexedMap<BalancerChunkSelectionPolicy::SplitInfo>()) {
+          _chunkSplitPoints(SimpleBSONObjComparator::kInstance.makeBSONObjIndexedMap<SplitInfo>()) {
     }
 
     /**
@@ -126,13 +205,13 @@ public:
         auto it = _chunkSplitPoints.find(chunk.getMin());
         if (it == _chunkSplitPoints.end()) {
             _chunkSplitPoints.emplace(chunk.getMin(),
-                                      BalancerChunkSelectionPolicy::SplitInfo(chunk.getShardId(),
-                                                                              _nss,
-                                                                              _collectionVersion,
-                                                                              chunk.getLastmod(),
-                                                                              chunk.getMin(),
-                                                                              chunk.getMax(),
-                                                                              {splitPoint}));
+                                      SplitInfo(chunk.getShardId(),
+                                                _nss,
+                                                _collectionVersion,
+                                                chunk.getLastmod(),
+                                                chunk.getMin(),
+                                                chunk.getMax(),
+                                                {splitPoint}));
         } else if (splitPoint.woCompare(it->second.splitKeys.back()) > 0) {
             it->second.splitKeys.push_back(splitPoint);
         } else {
@@ -146,7 +225,7 @@ public:
      * a vector of split infos to be passed to the split call.
      */
     SplitInfoVector done() {
-        BalancerChunkSelectionPolicy::SplitInfoVector splitPoints;
+        SplitInfoVector splitPoints;
         for (const auto& entry : _chunkSplitPoints) {
             splitPoints.push_back(std::move(entry.second));
         }
@@ -160,40 +239,40 @@ private:
     const ChunkVersion _collectionVersion;
 
     // Chunk min key and split vector associated with that chunk
-    BSONObjIndexedMap<BalancerChunkSelectionPolicy::SplitInfo> _chunkSplitPoints;
+    BSONObjIndexedMap<SplitInfo> _chunkSplitPoints;
 };
 
 /**
- * Populates splitCandidates with chunk and splitPoint pairs for chunks that violate tag
+ * Populates splitCandidates with chunk and splitPoint pairs for chunks that violate zone
  * range boundaries.
  */
-void getSplitCandidatesToEnforceTagRanges(const ChunkManager* cm,
-                                          const ZoneInfo& zoneInfo,
-                                          SplitCandidatesBuffer* splitCandidates) {
-    const auto& globalMax = cm->getShardKeyPattern().getKeyPattern().globalMax();
+void getSplitCandidatesToEnforceZoneRanges(const ChunkManager& cm,
+                                           const DistributionStatus& distribution,
+                                           SplitCandidatesBuffer* splitCandidates) {
+    const auto& globalMax = cm.getShardKeyPattern().getKeyPattern().globalMax();
 
-    // For each tag range, find chunks that need to be split.
-    for (const auto& tagRangeEntry : zoneInfo.zoneRanges()) {
-        const auto& tagRange = tagRangeEntry.second;
+    // For each zone range, find chunks that need to be split.
+    for (const auto& zoneRangeEntry : distribution.zoneRanges()) {
+        const auto& zoneRange = zoneRangeEntry.second;
 
-        const auto chunkAtZoneMin = cm->findIntersectingChunkWithSimpleCollation(tagRange.min);
-        invariant(chunkAtZoneMin.getMax().woCompare(tagRange.min) > 0);
+        const auto chunkAtZoneMin = cm.findIntersectingChunkWithSimpleCollation(zoneRange.min);
+        invariant(chunkAtZoneMin.getMax().woCompare(zoneRange.min) > 0);
 
-        if (chunkAtZoneMin.getMin().woCompare(tagRange.min)) {
-            splitCandidates->addSplitPoint(chunkAtZoneMin, tagRange.min);
+        if (chunkAtZoneMin.getMin().woCompare(zoneRange.min)) {
+            splitCandidates->addSplitPoint(chunkAtZoneMin, zoneRange.min);
         }
 
         // The global max key can never fall in the middle of a chunk.
-        if (!tagRange.max.woCompare(globalMax))
+        if (!zoneRange.max.woCompare(globalMax))
             continue;
 
-        const auto chunkAtZoneMax = cm->findIntersectingChunkWithSimpleCollation(tagRange.max);
+        const auto chunkAtZoneMax = cm.findIntersectingChunkWithSimpleCollation(zoneRange.max);
 
         // We need to check that both the chunk's minKey does not match the zone's max and also that
         // the max is not equal, which would only happen in the case of the zone ending in MaxKey.
-        if (chunkAtZoneMax.getMin().woCompare(tagRange.max) &&
-            chunkAtZoneMax.getMax().woCompare(tagRange.max)) {
-            splitCandidates->addSplitPoint(chunkAtZoneMax, tagRange.max);
+        if (chunkAtZoneMax.getMin().woCompare(zoneRange.max) &&
+            chunkAtZoneMax.getMax().woCompare(zoneRange.max)) {
+            splitCandidates->addSplitPoint(chunkAtZoneMax, zoneRange.max);
         }
     }
 }
@@ -206,18 +285,11 @@ void getSplitCandidatesToEnforceTagRanges(const ChunkManager* cm,
  * splitCandidates with chunk and splitPoint pairs for chunks that need to split.
  */
 void getSplitCandidatesForSessionsCollection(OperationContext* opCtx,
-                                             const ChunkManager* cm,
+                                             const ChunkManager& cm,
                                              SplitCandidatesBuffer* splitCandidates) {
-    // Do not compute split points if shard key is not _id,
-    // since the split points depend on _id.id.
-    const auto& keyPattern = cm->getShardKeyPattern().getKeyPattern();
-    if (!KeyPattern::isIdKeyPattern(keyPattern.toBSON())) {
-        return;
-    }
-
     const auto minNumChunks = minNumChunksForSessionsCollection.load();
 
-    if (cm->numChunks() >= minNumChunks) {
+    if (cm.numChunks() >= minNumChunks) {
         return;
     }
 
@@ -248,7 +320,7 @@ void getSplitCandidatesForSessionsCollection(OperationContext* opCtx,
 
     // For each split point, find a chunk that needs to be split.
     for (auto& splitPoint : splitPoints) {
-        const auto chunkAtSplitPoint = cm->findIntersectingChunkWithSimpleCollation(splitPoint);
+        const auto chunkAtSplitPoint = cm.findIntersectingChunkWithSimpleCollation(splitPoint);
         invariant(chunkAtSplitPoint.getMax().woCompare(splitPoint) > 0);
 
         if (chunkAtSplitPoint.getMin().woCompare(splitPoint)) {
@@ -276,13 +348,7 @@ StatusWith<SplitInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToSpli
 
     const auto& shardStats = shardStatsStatus.getValue();
 
-    auto swCollections = Grid::get(opCtx)->catalogClient()->getCollections(opCtx, nullptr, nullptr);
-    if (!swCollections.isOK()) {
-        return swCollections.getStatus();
-    }
-
-    auto& collections = swCollections.getValue();
-
+    auto collections = Grid::get(opCtx)->catalogClient()->getCollections(opCtx, {});
     if (collections.empty()) {
         return SplitInfoVector{};
     }
@@ -292,11 +358,7 @@ StatusWith<SplitInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToSpli
     std::shuffle(collections.begin(), collections.end(), _random);
 
     for (const auto& coll : collections) {
-        if (coll.getDropped()) {
-            continue;
-        }
-
-        const NamespaceString nss(coll.getNs());
+        const NamespaceString& nss(coll.getNss());
 
         auto candidatesStatus = _getSplitCandidatesForCollection(opCtx, nss, shardStats);
         if (candidatesStatus == ErrorCodes::NamespaceNotFound) {
@@ -311,8 +373,8 @@ StatusWith<SplitInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToSpli
             } else {
                 LOGV2_WARNING(
                     21852,
-                    "Unable to enforce tag range policy for collection {namespace}: {error}",
-                    "Unable to enforce tag range policy for collection",
+                    "Unable to enforce zone range policy for collection {namespace}: {error}",
+                    "Unable to enforce zone range policy for collection",
                     "namespace"_attr = nss.ns(),
                     "error"_attr = candidatesStatus.getStatus());
             }
@@ -342,7 +404,7 @@ StatusWith<SplitInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToSpli
 }
 
 StatusWith<MigrateInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToMove(
-    OperationContext* opCtx) {
+    OperationContext* opCtx, stdx::unordered_set<ShardId>* usedShards) {
     auto shardStatsStatus = _clusterStats->getStats(opCtx);
     if (!shardStatsStatus.isOK()) {
         return shardStatsStatus.getStatus();
@@ -354,72 +416,80 @@ StatusWith<MigrateInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToMo
         return MigrateInfoVector{};
     }
 
-    auto swCollections = Grid::get(opCtx)->catalogClient()->getCollections(opCtx, nullptr, nullptr);
-    if (!swCollections.isOK()) {
-        return swCollections.getStatus();
-    }
-
-    auto& collections = swCollections.getValue();
-
+    auto collections = Grid::get(opCtx)->catalogClient()->getCollections(opCtx, {});
     if (collections.empty()) {
         return MigrateInfoVector{};
     }
 
-    stdx::unordered_set<ShardId> availableShards;
-    std::transform(shardStats.begin(),
-                   shardStats.end(),
-                   std::inserter(availableShards, availableShards.end()),
-                   [](const ClusterStatistics::ShardStatistics& shardStatistics) -> ShardId {
-                       return shardStatistics.shardId;
-                   });
-
     MigrateInfoVector candidateChunks;
 
     std::shuffle(collections.begin(), collections.end(), _random);
 
-    for (const auto& coll : collections) {
-        if (coll.getDropped()) {
-            continue;
-        }
+    static constexpr auto kStatsForBalancingBatchSize = 20;
 
-        const NamespaceString nss(coll.getNs());
-
-        if (!coll.getAllowBalance() || !coll.getPermitMigrations()) {
-            LOGV2_DEBUG(21851,
+    std::vector<CollectionType> collBatch;
+    for (auto collIt = collections.begin(); collIt != collections.end();) {
+        const auto& coll = *(collIt++);
+        if (!coll.getAllowBalance() || !coll.getAllowMigrations() || !coll.getPermitMigrations() ||
+            coll.getDefragmentCollection()) {
+            LOGV2_DEBUG(5966401,
                         1,
-                        "Not balancing collection {namespace}; explicitly disabled.",
                         "Not balancing explicitly disabled collection",
-                        "namespace"_attr = nss);
+                        "namespace"_attr = coll.getNss(),
+                        "allowBalance"_attr = coll.getAllowBalance(),
+                        "allowMigrations"_attr = coll.getAllowMigrations(),
+                        "permitMigrations"_attr = coll.getPermitMigrations(),
+                        "defragmentCollection"_attr = coll.getDefragmentCollection());
+        } else {
+            collBatch.push_back(coll);
+        }
+
+        if (collBatch.size() < kStatsForBalancingBatchSize && collIt != collections.end()) {
+            // keep Accumulating in the batch
             continue;
         }
 
-        auto candidatesStatus =
-            _getMigrateCandidatesForCollection(opCtx, nss, shardStats, &availableShards);
-        if (candidatesStatus == ErrorCodes::NamespaceNotFound) {
-            // Namespace got dropped before we managed to get to it, so just skip it
-            continue;
-        } else if (!candidatesStatus.isOK()) {
-            LOGV2_WARNING(21853,
-                          "Unable to balance collection {namespace}: {error}",
-                          "Unable to balance collection",
-                          "namespace"_attr = nss.ns(),
-                          "error"_attr = candidatesStatus.getStatus());
-            continue;
+        boost::optional<stdx::unordered_map<NamespaceString, CollectionDataSizeInfoForBalancing>>
+            collsDataSizeInfo;
+        if (feature_flags::gBalanceAccordingToDataSize.isEnabled(
+                serverGlobalParams.featureCompatibility)) {
+            collsDataSizeInfo.emplace(getDataSizeInfoForCollections(opCtx, collBatch));
         }
 
-        candidateChunks.insert(candidateChunks.end(),
-                               std::make_move_iterator(candidatesStatus.getValue().begin()),
-                               std::make_move_iterator(candidatesStatus.getValue().end()));
+        for (const auto& collFromBatch : collBatch) {
+            const auto& nss = collFromBatch.getNss();
 
-        if (availableShards.size() < 2) {
-            return candidateChunks;
+            boost::optional<CollectionDataSizeInfoForBalancing> optDataSizeInfo;
+            if (collsDataSizeInfo.has_value()) {
+                optDataSizeInfo.emplace(std::move(collsDataSizeInfo->at(nss)));
+            }
+
+            auto candidatesStatus = _getMigrateCandidatesForCollection(
+                opCtx, nss, shardStats, optDataSizeInfo, usedShards);
+            if (candidatesStatus == ErrorCodes::NamespaceNotFound) {
+                // Namespace got dropped before we managed to get to it, so just skip it
+                continue;
+            } else if (!candidatesStatus.isOK()) {
+                LOGV2_WARNING(21853,
+                              "Unable to balance collection",
+                              "namespace"_attr = nss.ns(),
+                              "error"_attr = candidatesStatus.getStatus());
+                continue;
+            }
+
+            candidateChunks.insert(
+                candidateChunks.end(),
+                std::make_move_iterator(candidatesStatus.getValue().first.begin()),
+                std::make_move_iterator(candidatesStatus.getValue().first.end()));
         }
+
+        collBatch.clear();
     }
 
     return candidateChunks;
 }
 
-StatusWith<MigrateInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToMove(
+StatusWith<MigrateInfosWithReason> BalancerChunkSelectionPolicyImpl::selectChunksToMove(
     OperationContext* opCtx, const NamespaceString& nss) {
     auto shardStatsStatus = _clusterStats->getStats(opCtx);
     if (!shardStatsStatus.isOK()) {
@@ -428,29 +498,20 @@ StatusWith<MigrateInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToMo
 
     const auto& shardStats = shardStatsStatus.getValue();
 
-    // Validate collection information
-    auto swCollection = Grid::get(opCtx)->catalogClient()->getCollection(opCtx, nss);
-    if (!swCollection.isOK()) {
-        return swCollection.getStatus();
+    // Used to check locally if the collection exists, it should trow NamespaceNotFound if it
+    // doesn't.
+    Grid::get(opCtx)->catalogClient()->getCollection(opCtx, nss);
+
+    stdx::unordered_set<ShardId> usedShards;
+
+    boost::optional<CollectionDataSizeInfoForBalancing> optCollDataSizeInfo;
+    if (feature_flags::gBalanceAccordingToDataSize.isEnabled(
+            serverGlobalParams.featureCompatibility)) {
+        optCollDataSizeInfo.emplace(getDataSizeInfoForCollection(opCtx, nss));
     }
 
-    const auto& collection = swCollection.getValue().value;
-
-    if (collection.getDropped()) {
-        return Status(ErrorCodes::NamespaceNotFound,
-                      str::stream() << "collection " << nss.ns() << " not found");
-    }
-
-    stdx::unordered_set<ShardId> availableShards;
-    std::transform(shardStats.begin(),
-                   shardStats.end(),
-                   std::inserter(availableShards, availableShards.end()),
-                   [](const ClusterStatistics::ShardStatistics& shardStatistics) -> ShardId {
-                       return shardStatistics.shardId;
-                   });
-
-    auto candidatesStatus =
-        _getMigrateCandidatesForCollection(opCtx, nss, shardStats, &availableShards);
+    auto candidatesStatus = _getMigrateCandidatesForCollection(
+        opCtx, nss, shardStats, optCollDataSizeInfo, &usedShards);
     if (!candidatesStatus.isOK()) {
         return candidatesStatus.getStatus();
     }
@@ -460,6 +521,7 @@ StatusWith<MigrateInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToMo
 
 StatusWith<boost::optional<MigrateInfo>>
 BalancerChunkSelectionPolicyImpl::selectSpecificChunkToMove(OperationContext* opCtx,
+                                                            const NamespaceString& nss,
                                                             const ChunkType& chunk) {
     auto shardStatsStatus = _clusterStats->getStats(opCtx);
     if (!shardStatsStatus.isOK()) {
@@ -469,15 +531,14 @@ BalancerChunkSelectionPolicyImpl::selectSpecificChunkToMove(OperationContext* op
     const auto& shardStats = shardStatsStatus.getValue();
 
     auto routingInfoStatus =
-        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithRefresh(opCtx,
-                                                                                     chunk.getNS());
+        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithRefresh(opCtx, nss);
     if (!routingInfoStatus.isOK()) {
         return routingInfoStatus.getStatus();
     }
 
-    const auto cm = routingInfoStatus.getValue().cm().get();
+    const auto& cm = routingInfoStatus.getValue();
 
-    const auto collInfoStatus = createCollectionDistributionStatus(opCtx, shardStats, cm);
+    const auto collInfoStatus = createCollectionDistributionStatus(opCtx, nss, shardStats, cm);
     if (!collInfoStatus.isOK()) {
         return collInfoStatus.getStatus();
     }
@@ -495,18 +556,22 @@ Status BalancerChunkSelectionPolicyImpl::checkMoveAllowed(OperationContext* opCt
         return shardStatsStatus.getStatus();
     }
 
+    const CollectionType collection = Grid::get(opCtx)->catalogClient()->getCollection(
+        opCtx, chunk.getCollectionUUID(), repl::ReadConcernLevel::kLocalReadConcern);
+    const auto& nss = collection.getNss();
+
+
     auto shardStats = std::move(shardStatsStatus.getValue());
 
     auto routingInfoStatus =
-        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithRefresh(opCtx,
-                                                                                     chunk.getNS());
+        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithRefresh(opCtx, nss);
     if (!routingInfoStatus.isOK()) {
         return routingInfoStatus.getStatus();
     }
 
-    const auto cm = routingInfoStatus.getValue().cm().get();
+    const auto& cm = routingInfoStatus.getValue();
 
-    const auto collInfoStatus = createCollectionDistributionStatus(opCtx, shardStats, cm);
+    const auto collInfoStatus = createCollectionDistributionStatus(opCtx, nss, shardStats, cm);
     if (!collInfoStatus.isOK()) {
         return collInfoStatus.getStatus();
     }
@@ -526,7 +591,7 @@ Status BalancerChunkSelectionPolicyImpl::checkMoveAllowed(OperationContext* opCt
     }
 
     return BalancerPolicy::isShardSuitableReceiver(*newShardIterator,
-                                                   distribution.getTagForRange(chunk.getRange()));
+                                                   distribution.getZoneForChunk(chunk));
 }
 
 StatusWith<SplitInfoVector> BalancerChunkSelectionPolicyImpl::_getSplitCandidatesForCollection(
@@ -537,64 +602,66 @@ StatusWith<SplitInfoVector> BalancerChunkSelectionPolicyImpl::_getSplitCandidate
         return routingInfoStatus.getStatus();
     }
 
-    const auto cm = routingInfoStatus.getValue().cm().get();
+    const auto& cm = routingInfoStatus.getValue();
 
-    const auto swCollZoneInfo =
-        createCollectionZoneInfo(opCtx, nss, cm->getShardKeyPattern().getKeyPattern());
-    if (!swCollZoneInfo.isOK()) {
-        return swCollZoneInfo.getStatus();
-    }
-    const auto& collZoneInfo = swCollZoneInfo.getValue();
-
-    // Accumulate split points for the same chunk together
-    SplitCandidatesBuffer splitCandidates(nss, cm->getVersion());
-
-    if (nss == NamespaceString::kLogicalSessionsNamespace) {
-        if (!collZoneInfo.allZones().empty()) {
-            LOGV2_WARNING(4562401,
-                          "Ignoring zones for the sessions collection",
-                          "tags"_attr = collZoneInfo.allZones());
-        }
-
-        getSplitCandidatesForSessionsCollection(opCtx, cm, &splitCandidates);
-    } else {
-        getSplitCandidatesToEnforceTagRanges(cm, collZoneInfo, &splitCandidates);
-    }
-
-    return splitCandidates.done();
-}
-
-StatusWith<MigrateInfoVector> BalancerChunkSelectionPolicyImpl::_getMigrateCandidatesForCollection(
-    OperationContext* opCtx,
-    const NamespaceString& nss,
-    const ShardStatisticsVector& shardStats,
-    stdx::unordered_set<ShardId>* availableShards) {
-    auto routingInfoStatus =
-        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithRefresh(opCtx, nss);
-    if (!routingInfoStatus.isOK()) {
-        return routingInfoStatus.getStatus();
-    }
-
-    const auto cm = routingInfoStatus.getValue().cm().get();
-
-    const auto& shardKeyPattern = cm->getShardKeyPattern().getKeyPattern();
-
-    const auto collInfoStatus = createCollectionDistributionStatus(opCtx, shardStats, cm);
+    const auto collInfoStatus = createCollectionDistributionStatus(opCtx, nss, shardStats, cm);
     if (!collInfoStatus.isOK()) {
         return collInfoStatus.getStatus();
     }
 
     const DistributionStatus& distribution = collInfoStatus.getValue();
 
-    for (const auto& tagRangeEntry : distribution.getZoneInfo().zoneRanges()) {
-        const auto& tagRange = tagRangeEntry.second;
+    // Accumulate split points for the same chunk together
+    SplitCandidatesBuffer splitCandidates(nss, cm.getVersion());
 
-        const auto chunkAtZoneMin = cm->findIntersectingChunkWithSimpleCollation(tagRange.min);
+    if (nss == NamespaceString::kLogicalSessionsNamespace) {
+        if (!distribution.zones().empty()) {
+            LOGV2_WARNING(4562401,
+                          "Ignoring zones for the sessions collection",
+                          "zones"_attr = distribution.zones());
+        }
 
-        if (chunkAtZoneMin.getMin().woCompare(tagRange.min)) {
+        getSplitCandidatesForSessionsCollection(opCtx, cm, &splitCandidates);
+    } else {
+        getSplitCandidatesToEnforceZoneRanges(cm, distribution, &splitCandidates);
+    }
+
+    return splitCandidates.done();
+}
+
+StatusWith<MigrateInfosWithReason>
+BalancerChunkSelectionPolicyImpl::_getMigrateCandidatesForCollection(
+    OperationContext* opCtx,
+    const NamespaceString& nss,
+    const ShardStatisticsVector& shardStats,
+    const boost::optional<CollectionDataSizeInfoForBalancing>& collDataSizeInfo,
+    stdx::unordered_set<ShardId>* usedShards) {
+    auto routingInfoStatus =
+        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithRefresh(opCtx, nss);
+    if (!routingInfoStatus.isOK()) {
+        return routingInfoStatus.getStatus();
+    }
+
+    const auto& cm = routingInfoStatus.getValue();
+
+    const auto& shardKeyPattern = cm.getShardKeyPattern().getKeyPattern();
+
+    const auto collInfoStatus = createCollectionDistributionStatus(opCtx, nss, shardStats, cm);
+    if (!collInfoStatus.isOK()) {
+        return collInfoStatus.getStatus();
+    }
+
+    const DistributionStatus& distribution = collInfoStatus.getValue();
+
+    for (const auto& zoneRangeEntry : distribution.zoneRanges()) {
+        const auto& zoneRange = zoneRangeEntry.second;
+
+        const auto chunkAtZoneMin = cm.findIntersectingChunkWithSimpleCollation(zoneRange.min);
+
+        if (chunkAtZoneMin.getMin().woCompare(zoneRange.min)) {
             return {ErrorCodes::IllegalOperation,
                     str::stream()
-                        << "Tag boundaries " << tagRange.toString()
+                        << "Zone boundaries " << zoneRange.toString()
                         << " fall in the middle of an existing chunk "
                         << ChunkRange(chunkAtZoneMin.getMin(), chunkAtZoneMin.getMax()).toString()
                         << ". Balancing for collection " << nss.ns()
@@ -602,18 +669,18 @@ StatusWith<MigrateInfoVector> BalancerChunkSelectionPolicyImpl::_getMigrateCandi
         }
 
         // The global max key can never fall in the middle of a chunk
-        if (!tagRange.max.woCompare(shardKeyPattern.globalMax()))
+        if (!zoneRange.max.woCompare(shardKeyPattern.globalMax()))
             continue;
 
-        const auto chunkAtZoneMax = cm->findIntersectingChunkWithSimpleCollation(tagRange.max);
+        const auto chunkAtZoneMax = cm.findIntersectingChunkWithSimpleCollation(zoneRange.max);
 
         // We need to check that both the chunk's minKey does not match the zone's max and also that
         // the max is not equal, which would only happen in the case of the zone ending in MaxKey.
-        if (chunkAtZoneMax.getMin().woCompare(tagRange.max) &&
-            chunkAtZoneMax.getMax().woCompare(tagRange.max)) {
+        if (chunkAtZoneMax.getMin().woCompare(zoneRange.max) &&
+            chunkAtZoneMax.getMax().woCompare(zoneRange.max)) {
             return {ErrorCodes::IllegalOperation,
                     str::stream()
-                        << "Tag boundaries " << tagRange.toString()
+                        << "Zone boundaries " << zoneRange.toString()
                         << " fall in the middle of an existing chunk "
                         << ChunkRange(chunkAtZoneMax.getMin(), chunkAtZoneMax.getMax()).toString()
                         << ". Balancing for collection " << nss.ns()
@@ -624,7 +691,8 @@ StatusWith<MigrateInfoVector> BalancerChunkSelectionPolicyImpl::_getMigrateCandi
     return BalancerPolicy::balance(
         shardStats,
         distribution,
-        availableShards,
+        collDataSizeInfo,
+        usedShards,
         Grid::get(opCtx)->getBalancerConfiguration()->attemptToBalanceJumboChunks());
 }
 
