@@ -27,40 +27,93 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/s/config/initial_split_policy.h"
 
-#include "mongo/bson/util/bson_extract.h"
 #include "mongo/client/read_preference.h"
-#include "mongo/db/logical_clock.h"
+#include "mongo/db/bson/dotted_path_support.h"
+#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/curop.h"
+#include "mongo/db/pipeline/document_source.h"
+#include "mongo/db/pipeline/lite_parsed_pipeline.h"
+#include "mongo/db/pipeline/process_interface/shardsvr_process_interface.h"
+#include "mongo/db/pipeline/sharded_agg_helpers.h"
+#include "mongo/db/s/balancer/balancer_policy.h"
+#include "mongo/db/s/sharding_state.h"
+#include "mongo/db/vector_clock.h"
+#include "mongo/logv2/log.h"
 #include "mongo/s/balancer_configuration.h"
 #include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/shard_util.h"
+#include "mongo/stdx/unordered_map.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 namespace mongo {
 namespace {
 
-/*
- * Creates a chunk based on the given arguments, appends it to 'chunks', and
- * increments the given chunk version
+using ChunkDistributionMap = stdx::unordered_map<ShardId, size_t>;
+using ZoneShardMap = StringMap<std::vector<ShardId>>;
+using boost::intrusive_ptr;
+
+std::vector<ShardId> getAllShardIdsSorted(OperationContext* opCtx) {
+    // Many tests assume that chunks will be placed on shards
+    // according to their IDs in ascending lexical order.
+    auto shardIds = Grid::get(opCtx)->shardRegistry()->getAllShardIds(opCtx);
+    std::sort(shardIds.begin(), shardIds.end());
+    return shardIds;
+}
+
+/**
+ * Creates a chunk based on the given arguments, appends it to 'chunks' and increments the given
+ * chunk version
  */
-void appendChunk(const NamespaceString& nss,
+void appendChunk(const SplitPolicyParams& params,
                  const BSONObj& min,
                  const BSONObj& max,
                  ChunkVersion* version,
-                 const Timestamp& validAfter,
                  const ShardId& shardId,
                  std::vector<ChunkType>* chunks) {
-    chunks->emplace_back(nss, ChunkRange(min, max), *version, shardId);
+    chunks->emplace_back(params.collectionUUID, ChunkRange(min, max), *version, shardId);
     auto& chunk = chunks->back();
-    chunk.setHistory({ChunkHistory(validAfter, shardId)});
+    chunk.setHistory({ChunkHistory(version->getTimestamp(), shardId)});
     version->incMinor();
 }
 
+/**
+ * Return the shard with least amount of chunks while respecting the zone settings.
+ */
+ShardId selectBestShard(const ChunkDistributionMap& chunkMap,
+                        const ZoneInfo& zoneInfo,
+                        const ZoneShardMap& zoneToShards,
+                        const ChunkRange& chunkRange) {
+    auto zone = zoneInfo.getZoneForChunk(chunkRange);
+    auto iter = zoneToShards.find(zone);
+
+    uassert(4952605,
+            str::stream() << "no shards found for zone: " << zone
+                          << ", while creating initial chunks for new resharded collection",
+            iter != zoneToShards.end());
+    const auto& shards = iter->second;
+
+    uassert(4952607,
+            str::stream() << "no shards found for zone: " << zone
+                          << ", while creating initial chunks for new resharded collection",
+            !shards.empty());
+
+    auto bestShardIter = chunkMap.end();
+
+    for (const auto& shard : shards) {
+        auto candidateIter = chunkMap.find(shard);
+        if (bestShardIter == chunkMap.end() || candidateIter->second < bestShardIter->second) {
+            bestShardIter = candidateIter;
+        }
+    }
+
+    invariant(bestShardIter != chunkMap.end());
+    return bestShardIter->first;
+}
+
 /*
  * Returns a map mapping each tag name to a vector of shard ids with that tag name
  */
@@ -78,10 +131,10 @@ StringMap<std::vector<ShardId>> buildTagsToShardIdsMap(OperationContext* opCtx,
         configServer->exhaustiveFindOnConfig(opCtx,
                                              ReadPreferenceSetting(ReadPreference::Nearest),
                                              repl::ReadConcernLevel::kMajorityReadConcern,
-                                             ShardType::ConfigNS,
+                                             NamespaceString::kConfigsvrShardsNamespace,
                                              BSONObj(),
                                              BSONObj(),
-                                             0));
+                                             boost::none));
     uassert(50986, str::stream() << "Could not find any shard documents", !shardDocs.docs.empty());
 
     for (const auto& tag : tags) {
@@ -157,9 +210,8 @@ std::vector<BSONObj> InitialSplitPolicy::calculateHashedSplitPoints(
 }
 
 InitialSplitPolicy::ShardCollectionConfig InitialSplitPolicy::generateShardCollectionInitialChunks(
-    const NamespaceString& nss,
+    const SplitPolicyParams& params,
     const ShardKeyPattern& shardKeyPattern,
-    const ShardId& databasePrimaryShardId,
     const Timestamp& validAfter,
     const std::vector<BSONObj>& splitPoints,
     const std::vector<ShardId>& allShardIds,
@@ -179,7 +231,7 @@ InitialSplitPolicy::ShardCollectionConfig InitialSplitPolicy::generateShardColle
         finalSplitPoints.push_back(splitPoint);
     }
 
-    ChunkVersion version(1, 0, OID::gen());
+    ChunkVersion version({OID::gen(), validAfter}, {1, 0});
     const auto& keyPattern(shardKeyPattern.getKeyPattern());
 
     std::vector<ChunkType> chunks;
@@ -192,10 +244,10 @@ InitialSplitPolicy::ShardCollectionConfig InitialSplitPolicy::generateShardColle
         // It's possible there are no split points or fewer split points than total number of
         // shards, and we need to be sure that at least one chunk is placed on the primary shard
         const ShardId shardId = (i == 0 && finalSplitPoints.size() + 1 < allShardIds.size())
-            ? databasePrimaryShardId
+            ? params.primaryShardId
             : allShardIds[(i / numContiguousChunksPerShard) % allShardIds.size()];
 
-        appendChunk(nss, min, max, &version, validAfter, shardId, &chunks);
+        appendChunk(params, min, max, &version, shardId, &chunks);
     }
 
     return {std::move(chunks)};
@@ -204,41 +256,43 @@ InitialSplitPolicy::ShardCollectionConfig InitialSplitPolicy::generateShardColle
 std::unique_ptr<InitialSplitPolicy> InitialSplitPolicy::calculateOptimizationStrategy(
     OperationContext* opCtx,
     const ShardKeyPattern& shardKeyPattern,
-    const ShardsvrShardCollection& request,
+    const std::int64_t numInitialChunks,
+    const bool presplitHashedZones,
+    const boost::optional<std::vector<BSONObj>>& initialSplitPoints,
     const std::vector<TagsType>& tags,
     size_t numShards,
-    bool collectionIsEmpty) {
+    bool collectionIsEmpty,
+    bool useAutoSplitter) {
     uassert(ErrorCodes::InvalidOptions,
             str::stream() << "numInitialChunks is only supported when the collection is empty "
                              "and has a hashed field in the shard key pattern",
-            !request.getNumInitialChunks() ||
-                (shardKeyPattern.isHashedPattern() && collectionIsEmpty));
+            !numInitialChunks || (shardKeyPattern.isHashedPattern() && collectionIsEmpty));
     uassert(ErrorCodes::InvalidOptions,
             str::stream()
                 << "When the prefix of the hashed shard key is a range field, "
                    "'numInitialChunks' can only be used when the 'presplitHashedZones' is true",
-            !request.getNumInitialChunks() || shardKeyPattern.hasHashedPrefix() ||
-                request.getPresplitHashedZones());
+            !numInitialChunks || shardKeyPattern.hasHashedPrefix() || presplitHashedZones);
     uassert(ErrorCodes::InvalidOptions,
             str::stream() << "Cannot have both initial split points and tags set",
-            !request.getInitialSplitPoints() || tags.empty());
+            !initialSplitPoints || tags.empty());
 
     // If 'presplitHashedZones' flag is set, we always use 'PresplitHashedZonesSplitPolicy', to make
     // sure we throw the correct assertion if further validation fails.
-    if (request.getPresplitHashedZones()) {
+    if (presplitHashedZones) {
         return std::make_unique<PresplitHashedZonesSplitPolicy>(
-            opCtx, shardKeyPattern, tags, request.getNumInitialChunks(), collectionIsEmpty);
+            opCtx, shardKeyPattern, tags, numInitialChunks, collectionIsEmpty);
     }
 
     // The next preference is to use split points based strategy. This is only possible if
     // 'initialSplitPoints' is set, or if the collection is empty with shard key having a hashed
     // prefix.
-    if (request.getInitialSplitPoints()) {
-        return std::make_unique<SplitPointsBasedSplitPolicy>(*request.getInitialSplitPoints());
+    if (initialSplitPoints) {
+        return std::make_unique<SplitPointsBasedSplitPolicy>(*initialSplitPoints);
     }
+
     if (tags.empty() && shardKeyPattern.hasHashedPrefix() && collectionIsEmpty) {
         return std::make_unique<SplitPointsBasedSplitPolicy>(
-            shardKeyPattern, numShards, request.getNumInitialChunks());
+            shardKeyPattern, numShards, numInitialChunks);
     }
 
     if (!tags.empty()) {
@@ -251,109 +305,78 @@ std::unique_ptr<InitialSplitPolicy> InitialSplitPolicy::calculateOptimizationStr
     if (collectionIsEmpty) {
         return std::make_unique<SingleChunkOnPrimarySplitPolicy>();
     }
-    return std::make_unique<UnoptimizedSplitPolicy>();
-}
 
-boost::optional<CollectionType> InitialSplitPolicy::checkIfCollectionAlreadyShardedWithSameOptions(
-    OperationContext* opCtx,
-    const NamespaceString& nss,
-    const ShardsvrShardCollection& request,
-    repl::ReadConcernLevel readConcernLevel) {
-    auto const catalogClient = Grid::get(opCtx)->catalogClient();
-
-    auto collStatus = catalogClient->getCollection(opCtx, nss, readConcernLevel);
-    if (collStatus == ErrorCodes::NamespaceNotFound) {
-        // Not currently sharded.
-        return boost::none;
+    if (useAutoSplitter) {
+        return std::make_unique<AutoSplitInChunksOnPrimaryPolicy>();
     }
 
-    uassertStatusOK(collStatus);
-    auto existingOptions = collStatus.getValue().value;
-
-    CollectionType requestedOptions;
-    requestedOptions.setNs(nss);
-    requestedOptions.setKeyPattern(KeyPattern(request.getKey()));
-    requestedOptions.setDefaultCollation(*request.getCollation());
-    requestedOptions.setUnique(request.getUnique());
-
-    // Set the distributionMode to "sharded" because this CollectionType represents the requested
-    // target state for the collection after shardCollection. The requested CollectionType will be
-    // compared with the existing CollectionType below, and if the existing CollectionType either
-    // does not have a distributionMode (FCV 4.2) or has distributionMode "sharded" (FCV 4.4), the
-    // collection will be considered to already be in its target state.
-    requestedOptions.setDistributionMode(CollectionType::DistributionMode::kSharded);
-
-    // If the collection is already sharded, fail if the deduced options in this request do not
-    // match the options the collection was originally sharded with.
-    uassert(ErrorCodes::AlreadyInitialized,
-            str::stream() << "sharding already enabled for collection " << nss.ns()
-                          << " with options " << existingOptions.toString(),
-            requestedOptions.hasSameOptions(existingOptions));
-
-    return existingOptions;
+    return std::make_unique<SingleChunkOnPrimarySplitPolicy>();
 }
 
 InitialSplitPolicy::ShardCollectionConfig SingleChunkOnPrimarySplitPolicy::createFirstChunks(
-    OperationContext* opCtx, const ShardKeyPattern& shardKeyPattern, SplitPolicyParams params) {
-    ShardCollectionConfig initialChunks;
-    ChunkVersion version(1, 0, OID::gen());
+    OperationContext* opCtx,
+    const ShardKeyPattern& shardKeyPattern,
+    const SplitPolicyParams& params) {
+    const auto currentTime = VectorClock::get(opCtx)->getTime();
+    const auto validAfter = currentTime.clusterTime().asTimestamp();
+
+    ChunkVersion version({OID::gen(), validAfter}, {1, 0});
     const auto& keyPattern = shardKeyPattern.getKeyPattern();
-    appendChunk(params.nss,
+    std::vector<ChunkType> chunks;
+    appendChunk(params,
                 keyPattern.globalMin(),
                 keyPattern.globalMax(),
                 &version,
-                LogicalClock::get(opCtx)->getClusterTime().asTimestamp(),
                 params.primaryShardId,
-                &initialChunks.chunks);
-    return initialChunks;
+                &chunks);
+
+    return {std::move(chunks)};
 }
 
-InitialSplitPolicy::ShardCollectionConfig UnoptimizedSplitPolicy::createFirstChunks(
-    OperationContext* opCtx, const ShardKeyPattern& shardKeyPattern, SplitPolicyParams params) {
+InitialSplitPolicy::ShardCollectionConfig AutoSplitInChunksOnPrimaryPolicy::createFirstChunks(
+    OperationContext* opCtx,
+    const ShardKeyPattern& shardKeyPattern,
+    const SplitPolicyParams& params) {
     // Under this policy, chunks are only placed on the primary shard.
     std::vector<ShardId> shardIds{params.primaryShardId};
 
     // Refresh the balancer settings to ensure the chunk size setting, which is sent as part of
-    // the autoSplitVector command and affects the number of chunks returned, has been loaded.
+    // the splitVector command and affects the number of chunks returned, has been loaded.
     const auto balancerConfig = Grid::get(opCtx)->getBalancerConfiguration();
     uassertStatusOK(balancerConfig->refreshAndCheck(opCtx));
+    auto optNss = CollectionCatalog::get(opCtx)->lookupNSSByUUID(opCtx, params.collectionUUID);
+    invariant(optNss);
     const auto shardSelectedSplitPoints = uassertStatusOK(
         shardutil::selectChunkSplitPoints(opCtx,
                                           params.primaryShardId,
-                                          params.nss,
+                                          *optNss,
                                           shardKeyPattern,
                                           ChunkRange(shardKeyPattern.getKeyPattern().globalMin(),
                                                      shardKeyPattern.getKeyPattern().globalMax()),
-                                          balancerConfig->getMaxChunkSizeBytes(),
-                                          0));
-    return generateShardCollectionInitialChunks(
-        params.nss,
-        shardKeyPattern,
-        params.primaryShardId,
-        LogicalClock::get(opCtx)->getClusterTime().asTimestamp(),
-        shardSelectedSplitPoints,
-        shardIds,
-        1  // numContiguousChunksPerShard
+                                          balancerConfig->getMaxChunkSizeBytes()));
+
+    const auto currentTime = VectorClock::get(opCtx)->getTime();
+    return generateShardCollectionInitialChunks(params,
+                                                shardKeyPattern,
+                                                currentTime.clusterTime().asTimestamp(),
+                                                shardSelectedSplitPoints,
+                                                shardIds,
+                                                1  // numContiguousChunksPerShard
     );
 }
 
 InitialSplitPolicy::ShardCollectionConfig SplitPointsBasedSplitPolicy::createFirstChunks(
-    OperationContext* opCtx, const ShardKeyPattern& shardKeyPattern, SplitPolicyParams params) {
-
-    const auto shardRegistry = Grid::get(opCtx)->shardRegistry();
+    OperationContext* opCtx,
+    const ShardKeyPattern& shardKeyPattern,
+    const SplitPolicyParams& params) {
 
     // On which shards are the generated chunks allowed to be placed.
-    std::vector<ShardId> shardIds;
-    shardRegistry->getAllShardIdsNoReload(&shardIds);
+    const auto shardIds = getAllShardIdsSorted(opCtx);
 
+    const auto currentTime = VectorClock::get(opCtx)->getTime();
+    const auto validAfter = currentTime.clusterTime().asTimestamp();
     return generateShardCollectionInitialChunks(
-        params.nss,
-        shardKeyPattern,
-        params.primaryShardId,
-        LogicalClock::get(opCtx)->getClusterTime().asTimestamp(),
-        _splitPoints,
-        shardIds,
-        _numContiguousChunksPerShard);
+        params, shardKeyPattern, validAfter, _splitPoints, shardIds, _numContiguousChunksPerShard);
 }
 
 AbstractTagsBasedSplitPolicy::AbstractTagsBasedSplitPolicy(OperationContext* opCtx,
@@ -374,14 +397,14 @@ AbstractTagsBasedSplitPolicy::SplitInfo SingleChunkPerTagSplitPolicy::buildSplit
 }
 
 InitialSplitPolicy::ShardCollectionConfig AbstractTagsBasedSplitPolicy::createFirstChunks(
-    OperationContext* opCtx, const ShardKeyPattern& shardKeyPattern, SplitPolicyParams params) {
+    OperationContext* opCtx,
+    const ShardKeyPattern& shardKeyPattern,
+    const SplitPolicyParams& params) {
     invariant(!_tags.empty());
 
-    const auto shardRegistry = Grid::get(opCtx)->shardRegistry();
-
-    std::vector<ShardId> shardIds;
-    shardRegistry->getAllShardIdsNoReload(&shardIds);
-    const auto validAfter = LogicalClock::get(opCtx)->getClusterTime().asTimestamp();
+    const auto shardIds = getAllShardIdsSorted(opCtx);
+    const auto currentTime = VectorClock::get(opCtx)->getTime();
+    const auto validAfter = currentTime.clusterTime().asTimestamp();
     const auto& keyPattern = shardKeyPattern.getKeyPattern();
 
     auto tagToShards = getTagsToShardIds();
@@ -390,27 +413,21 @@ InitialSplitPolicy::ShardCollectionConfig AbstractTagsBasedSplitPolicy::createFi
         return shardIds[indx++ % shardIds.size()];
     };
 
-    ChunkVersion version(1, 0, OID::gen());
+    ChunkVersion version({OID::gen(), validAfter}, {1, 0});
     auto lastChunkMax = keyPattern.globalMin();
     std::vector<ChunkType> chunks;
     for (const auto& tag : _tags) {
         // Create a chunk for the hole [lastChunkMax, tag.getMinKey)
         if (tag.getMinKey().woCompare(lastChunkMax) > 0) {
-            appendChunk(params.nss,
-                        lastChunkMax,
-                        tag.getMinKey(),
-                        &version,
-                        validAfter,
-                        nextShardIdForHole(),
-                        &chunks);
+            appendChunk(
+                params, lastChunkMax, tag.getMinKey(), &version, nextShardIdForHole(), &chunks);
         }
         // Create chunk for the actual tag - [tag.getMinKey, tag.getMaxKey)
         const auto it = tagToShards.find(tag.getTag());
         invariant(it != tagToShards.end());
         uassert(50973,
                 str::stream()
-                    << "Cannot shard collection " << params.nss.ns() << " due to zone "
-                    << tag.getTag()
+                    << "Cannot shard collection " << tag.getNS() << " due to zone " << tag.getTag()
                     << " which is not assigned to a shard. Please assign this zone to a shard.",
                 !it->second.empty());
 
@@ -440,7 +457,7 @@ InitialSplitPolicy::ShardCollectionConfig AbstractTagsBasedSplitPolicy::createFi
                 const BSONObj max = (splitPointIdx == splitInfo.splitPoints.size())
                     ? tag.getMaxKey()
                     : splitInfo.splitPoints[splitPointIdx];
-                appendChunk(params.nss, min, max, &version, validAfter, targetShard, &chunks);
+                appendChunk(params, min, max, &version, targetShard, &chunks);
             }
         }
         lastChunkMax = tag.getMaxKey();
@@ -448,13 +465,8 @@ InitialSplitPolicy::ShardCollectionConfig AbstractTagsBasedSplitPolicy::createFi
 
     // Create a chunk for the hole [lastChunkMax, MaxKey]
     if (lastChunkMax.woCompare(keyPattern.globalMax()) < 0) {
-        appendChunk(params.nss,
-                    lastChunkMax,
-                    keyPattern.globalMax(),
-                    &version,
-                    validAfter,
-                    nextShardIdForHole(),
-                    &chunks);
+        appendChunk(
+            params, lastChunkMax, keyPattern.globalMax(), &version, nextShardIdForHole(), &chunks);
     }
 
     return {std::move(chunks)};
@@ -621,4 +633,236 @@ void PresplitHashedZonesSplitPolicy::_validate(const ShardKeyPattern& shardKeyPa
     }
 }
 
+std::vector<BSONObj> ReshardingSplitPolicy::createRawPipeline(const ShardKeyPattern& shardKey,
+                                                              int numSplitPoints,
+                                                              int samplesPerChunk) {
+
+    std::vector<BSONObj> res;
+    const auto& shardKeyFields = shardKey.getKeyPatternFields();
+    BSONObjBuilder sortValBuilder;
+    using Doc = Document;
+    using Arr = std::vector<Value>;
+    using V = Value;
+    Arr arrayToObjectBuilder;
+    for (auto&& fieldRef : shardKeyFields) {
+        // If the shard key includes a hashed field and current fieldRef is the hashed field.
+        if (shardKey.isHashedPattern() &&
+            fieldRef->dottedField().compare(shardKey.getHashedField().fieldNameStringData()) == 0) {
+            arrayToObjectBuilder.emplace_back(
+                Doc{{"k", V{fieldRef->dottedField()}},
+                    {"v", Doc{{"$toHashedIndexKey", V{"$" + fieldRef->dottedField()}}}}});
+        } else {
+            arrayToObjectBuilder.emplace_back(Doc{
+                {"k", V{fieldRef->dottedField()}},
+                {"v", Doc{{"$ifNull", V{Arr{V{"$" + fieldRef->dottedField()}, V{BSONNULL}}}}}}});
+        }
+        sortValBuilder.append(fieldRef->dottedField().toString(), 1);
+    }
+    res.push_back(BSON("$sample" << BSON("size" << numSplitPoints * samplesPerChunk)));
+    res.push_back(BSON("$sort" << sortValBuilder.obj()));
+    res.push_back(
+        Doc{{"$replaceWith", Doc{{"$arrayToObject", Arr{V{arrayToObjectBuilder}}}}}}.toBson());
+    return res;
+}
+
+ReshardingSplitPolicy ReshardingSplitPolicy::make(OperationContext* opCtx,
+                                                  const NamespaceString& origNs,
+                                                  const NamespaceString& reshardingTempNs,
+                                                  const ShardKeyPattern& shardKey,
+                                                  int numInitialChunks,
+                                                  boost::optional<std::vector<TagsType>> zones,
+                                                  int samplesPerChunk) {
+    uassert(4952603, "samplesPerChunk should be > 0", samplesPerChunk > 0);
+    return ReshardingSplitPolicy(
+        numInitialChunks,
+        zones,
+        _makePipelineDocumentSource(opCtx, origNs, shardKey, numInitialChunks, samplesPerChunk));
+}
+
+ReshardingSplitPolicy::ReshardingSplitPolicy(int numInitialChunks,
+                                             boost::optional<std::vector<TagsType>> zones,
+                                             std::unique_ptr<SampleDocumentSource> samples)
+    : _numInitialChunks(numInitialChunks), _zones(std::move(zones)), _samples(std::move(samples)) {
+    uassert(4952602, "numInitialChunks should be > 0", numInitialChunks > 0);
+    uassert(4952604, "provided zones should not be empty", !_zones || _zones->size());
+}
+
+InitialSplitPolicy::ShardCollectionConfig ReshardingSplitPolicy::createFirstChunks(
+    OperationContext* opCtx, const ShardKeyPattern& shardKey, const SplitPolicyParams& params) {
+
+    if (_zones) {
+        for (auto& zone : *_zones) {
+            zone.setMinKey(shardKey.getKeyPattern().extendRangeBound(zone.getMinKey(), false));
+            zone.setMaxKey(shardKey.getKeyPattern().extendRangeBound(zone.getMaxKey(), false));
+        }
+    }
+
+    auto splitPoints = _extractSplitPointsFromZones(shardKey);
+    if (splitPoints.size() < static_cast<size_t>(_numInitialChunks - 1)) {
+        // The BlockingResultsMerger underlying the $mergeCursors stage records how long was
+        // spent waiting for samples from the donor shards. It doing so requires the CurOp
+        // to be marked as having started.
+        CurOp::get(opCtx)->ensureStarted();
+
+        _appendSplitPointsFromSample(
+            &splitPoints, shardKey, _numInitialChunks - splitPoints.size() - 1);
+    }
+
+    uassert(4952606,
+            "The shard key provided does not have enough cardinality to make the required amount "
+            "of chunks",
+            splitPoints.size() >= static_cast<size_t>(_numInitialChunks - 1));
+
+    ZoneShardMap zoneToShardMap;
+    ChunkDistributionMap chunkDistribution;
+
+    ZoneInfo zoneInfo;
+    if (_zones) {
+        zoneToShardMap = buildTagsToShardIdsMap(opCtx, *_zones);
+
+        for (const auto& zone : *_zones) {
+            uassertStatusOK(
+                zoneInfo.addRangeToZone({zone.getMinKey(), zone.getMaxKey(), zone.getTag()}));
+        }
+    }
+
+    {
+        auto allShardIds = getAllShardIdsSorted(opCtx);
+        for (const auto& shard : allShardIds) {
+            chunkDistribution.emplace(shard, 0);
+        }
+
+        zoneToShardMap.emplace("", std::move(allShardIds));
+    }
+
+    std::vector<ChunkType> chunks;
+
+    const auto& keyPattern = shardKey.getKeyPattern();
+    auto lastChunkMax = keyPattern.globalMin();
+    const auto currentTime = VectorClock::get(opCtx)->getTime();
+    const auto validAfter = currentTime.clusterTime().asTimestamp();
+
+    ChunkVersion version({OID::gen(), validAfter}, {1, 0});
+
+    splitPoints.insert(keyPattern.globalMax());
+    for (const auto& splitPoint : splitPoints) {
+        auto bestShard = selectBestShard(
+            chunkDistribution, zoneInfo, zoneToShardMap, {lastChunkMax, splitPoint});
+        appendChunk(params, lastChunkMax, splitPoint, &version, bestShard, &chunks);
+
+        lastChunkMax = splitPoint;
+        chunkDistribution[bestShard]++;
+    }
+
+    return {std::move(chunks)};
+}
+
+BSONObjSet ReshardingSplitPolicy::_extractSplitPointsFromZones(const ShardKeyPattern& shardKey) {
+    auto splitPoints = SimpleBSONObjComparator::kInstance.makeBSONObjSet();
+
+    if (!_zones) {
+        return splitPoints;
+    }
+
+    for (auto zone : *_zones) {
+        splitPoints.insert(zone.getMinKey());
+        splitPoints.insert(zone.getMaxKey());
+    }
+
+    const auto keyPattern = shardKey.getKeyPattern();
+    splitPoints.erase(keyPattern.globalMin());
+    splitPoints.erase(keyPattern.globalMax());
+
+    return splitPoints;
+}
+
+void ReshardingSplitPolicy::_appendSplitPointsFromSample(BSONObjSet* splitPoints,
+                                                         const ShardKeyPattern& shardKey,
+                                                         int nToAppend) {
+    int nRemaining = nToAppend;
+    auto nextKey = _samples->getNext();
+
+    while (nextKey && nRemaining > 0) {
+        // if key is hashed, nextKey values are already hashed
+        auto result = splitPoints->insert(nextKey->getOwned());
+        if (result.second) {
+            nRemaining--;
+        }
+        nextKey = _samples->getNext();
+    }
+}
+
+std::unique_ptr<ReshardingSplitPolicy::SampleDocumentSource>
+ReshardingSplitPolicy::makePipelineDocumentSource_forTest(OperationContext* opCtx,
+                                                          const NamespaceString& ns,
+                                                          const ShardKeyPattern& shardKey,
+                                                          int numInitialChunks,
+                                                          int samplesPerChunk) {
+    MakePipelineOptions opts;
+    opts.attachCursorSource = false;
+    return _makePipelineDocumentSource(
+        opCtx, ns, shardKey, numInitialChunks, samplesPerChunk, std::move(opts));
+}
+
+std::unique_ptr<ReshardingSplitPolicy::SampleDocumentSource>
+ReshardingSplitPolicy::_makePipelineDocumentSource(OperationContext* opCtx,
+                                                   const NamespaceString& ns,
+                                                   const ShardKeyPattern& shardKey,
+                                                   int numInitialChunks,
+                                                   int samplesPerChunk,
+                                                   MakePipelineOptions opts) {
+    auto rawPipeline = createRawPipeline(shardKey, numInitialChunks - 1, samplesPerChunk);
+    StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces;
+    resolvedNamespaces[ns.coll()] = {ns, std::vector<BSONObj>{}};
+
+    // Config servers don't have ShardingState enabled, so we have to manually create
+    // ShardServerProcessInterface instead of getting it from the generic factory so the pipeline
+    // can talk to the shards.
+    auto pi = std::make_shared<ShardServerProcessInterface>(
+        Grid::get(opCtx)->getExecutorPool()->getArbitraryExecutor());
+
+    auto expCtx = make_intrusive<ExpressionContext>(opCtx,
+                                                    boost::none, /* explain */
+                                                    false,       /* fromMongos */
+                                                    false,       /* needsMerge */
+                                                    true,        /* allowDiskUse */
+                                                    true,        /* bypassDocumentValidation */
+                                                    false,       /* isMapReduceCommand */
+                                                    ns,
+                                                    boost::none, /* runtimeConstants */
+                                                    nullptr,     /* collator */
+                                                    std::move(pi),
+                                                    std::move(resolvedNamespaces),
+                                                    boost::none); /* collUUID */
+
+    expCtx->tempDir = storageGlobalParams.dbpath + "/tmp";
+
+    return std::make_unique<PipelineDocumentSource>(
+        Pipeline::makePipeline(rawPipeline, expCtx, opts), samplesPerChunk - 1);
+}
+
+ReshardingSplitPolicy::PipelineDocumentSource::PipelineDocumentSource(
+    SampleDocumentPipeline pipeline, int skip)
+    : _pipeline(std::move(pipeline)), _skip(skip) {}
+
+boost::optional<BSONObj> ReshardingSplitPolicy::PipelineDocumentSource::getNext() {
+    auto val = _pipeline->getNext();
+
+    if (!val) {
+        return boost::none;
+    }
+
+    for (int skippedSamples = 0; skippedSamples < _skip; skippedSamples++) {
+        auto newVal = _pipeline->getNext();
+
+        if (!newVal) {
+            break;
+        }
+
+        val = newVal;
+    }
+
+    return val->toBson();
+}
+
 }  // namespace mongo
