@@ -27,18 +27,17 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/bson/json.h"
-#include "mongo/db/logical_clock.h"
+#include "mongo/db/s/config/config_server_test_fixture.h"
 #include "mongo/db/s/config/initial_split_policy.h"
+#include "mongo/db/vector_clock.h"
+#include "mongo/logv2/log.h"
 #include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/catalog/type_tags.h"
-#include "mongo/s/config_server_test_fixture.h"
 #include "mongo/unittest/unittest.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
 namespace mongo {
 namespace {
 
@@ -199,15 +198,15 @@ public:
      * shardIds[i] is the id of shard for the chunk for chunkRanges[i].
      * Checks that chunkRanges and shardIds have the same length.
      */
-    const std::vector<ChunkType> makeChunks(const std::vector<ChunkRange> chunkRanges,
-                                            const std::vector<ShardId> shardIds,
-                                            Timestamp timeStamp) {
+    std::vector<ChunkType> makeChunks(const std::vector<ChunkRange> chunkRanges,
+                                      const std::vector<ShardId> shardIds,
+                                      Timestamp timeStamp) {
         ASSERT_EQ(chunkRanges.size(), shardIds.size());
         std::vector<ChunkType> chunks;
 
         for (unsigned long i = 0; i < chunkRanges.size(); ++i) {
-            ChunkVersion version(1, 0, OID::gen());
-            ChunkType chunk(_nss, chunkRanges[i], version, shardIds[i]);
+            ChunkVersion version({OID::gen(), Timestamp(1, 1)}, {1, 0});
+            ChunkType chunk(_uuid, chunkRanges[i], version, shardIds[i]);
             chunk.setHistory({ChunkHistory(timeStamp, shardIds[i])});
             chunks.push_back(chunk);
         }
@@ -218,7 +217,7 @@ public:
      * Returns a vector of numShards shard ids with shard names
      * prefixed by _shardName
      */
-    const std::vector<ShardId> makeShardIds(const int numShards) {
+    std::vector<ShardId> makeShardIds(const int numShards) {
         std::vector<ShardId> shardIds;
         for (int i = 0; i < numShards; i++) {
             shardIds.push_back(shardId(std::to_string(i)));
@@ -226,10 +225,14 @@ public:
         return shardIds;
     }
 
-    const NamespaceString nss() {
+    NamespaceString nss() {
         return _nss;
     }
 
+    const UUID& uuid() {
+        return _uuid;
+    }
+
     const ShardKeyPattern& shardKeyPattern() {
         return _shardKeyPattern;
     }
@@ -238,16 +241,17 @@ public:
         return _shardKeyPattern.getKeyPattern();
     }
 
-    const ShardId shardId(std::string shardNum) {
+    ShardId shardId(std::string shardNum) {
         return ShardId(_shardName + shardNum);
     }
 
-    const Timestamp timeStamp() {
+    Timestamp timeStamp() {
         return _timeStamp;
     }
 
 private:
     const NamespaceString _nss{"test.foo"};
+    const UUID _uuid{UUID::gen()};
     const ShardKeyPattern _shardKeyPattern = ShardKeyPattern(BSON("x"
                                                                   << "hashed"));
     const std::string _shardName = "testShard";
@@ -279,7 +283,7 @@ TEST_F(GenerateInitialHashedSplitChunksTest, NoSplitPoints) {
     const std::vector<BSONObj> splitPoints;
     const std::vector<ShardId> shardIds = makeShardIds(2);
     const auto shardCollectionConfig = InitialSplitPolicy::generateShardCollectionInitialChunks(
-        nss(), shardKeyPattern(), shardIds[0], timeStamp(), splitPoints, shardIds);
+        {UUID::gen(), shardIds[0]}, shardKeyPattern(), timeStamp(), splitPoints, shardIds, 1);
 
     // there should only be one chunk
     const auto expectedChunks =
@@ -291,10 +295,15 @@ TEST_F(GenerateInitialHashedSplitChunksTest, NoSplitPoints) {
 
 TEST_F(GenerateInitialHashedSplitChunksTest, SplitPointsMoreThanAvailableShards) {
     const std::vector<ShardId> shardIds = makeShardIds(2);
-    const auto shardCollectionConfig = InitialSplitPolicy::generateShardCollectionInitialChunks(
-        nss(), shardKeyPattern(), shardIds[0], timeStamp(), hashedSplitPoints(), shardIds);
+    const auto shardCollectionConfig =
+        InitialSplitPolicy::generateShardCollectionInitialChunks({UUID::gen(), shardIds[0]},
+                                                                 shardKeyPattern(),
+                                                                 timeStamp(),
+                                                                 hashedSplitPoints(),
+                                                                 shardIds,
+                                                                 1);
 
-    // // chunks should be distributed in a round-robin manner
+    // chunks should be distributed in a round-robin manner
     const std::vector<ChunkType> expectedChunks = makeChunks(
         hashedChunkRanges(), {shardId("0"), shardId("1"), shardId("0"), shardId("1")}, timeStamp());
     assertChunkVectorsAreEqual(expectedChunks, shardCollectionConfig.chunks);
@@ -303,8 +312,13 @@ TEST_F(GenerateInitialHashedSplitChunksTest, SplitPointsMoreThanAvailableShards)
 TEST_F(GenerateInitialHashedSplitChunksTest,
        SplitPointsNumContiguousChunksPerShardsGreaterThanOne) {
     const std::vector<ShardId> shardIds = makeShardIds(2);
-    const auto shardCollectionConfig = InitialSplitPolicy::generateShardCollectionInitialChunks(
-        nss(), shardKeyPattern(), shardIds[0], timeStamp(), hashedSplitPoints(), shardIds, 2);
+    const auto shardCollectionConfig =
+        InitialSplitPolicy::generateShardCollectionInitialChunks({UUID::gen(), shardIds[0]},
+                                                                 shardKeyPattern(),
+                                                                 timeStamp(),
+                                                                 hashedSplitPoints(),
+                                                                 shardIds,
+                                                                 2);
 
     // chunks should be distributed in a round-robin manner two chunks at a time
     const std::vector<ChunkType> expectedChunks = makeChunks(
@@ -328,21 +342,20 @@ public:
         setupShards(shards);
         shardRegistry()->reload(opCtx);
         SingleChunkPerTagSplitPolicy splitPolicy(opCtx, tags);
-        const auto shardCollectionConfig =
-            splitPolicy.createFirstChunks(opCtx, shardKeyPattern, {});
+        const auto shardCollectionConfig = splitPolicy.createFirstChunks(
+            opCtx, shardKeyPattern, {UUID::gen(), expectedShardIds.front()});
 
-        const std::vector<ChunkType> expectedChunks =
-            makeChunks(expectedChunkRanges,
-                       expectedShardIds,
-                       LogicalClock::get(opCtx)->getClusterTime().asTimestamp());
+        const auto currentTime = VectorClock::get(opCtx)->getTime();
+        const std::vector<ChunkType> expectedChunks = makeChunks(
+            expectedChunkRanges, expectedShardIds, currentTime.clusterTime().asTimestamp());
         assertChunkVectorsAreEqual(expectedChunks, shardCollectionConfig.chunks);
     }
 
-    const std::string shardKey() {
+    std::string shardKey() {
         return _shardKey;
     }
 
-    const std::string zoneName(std::string zoneNum) {
+    std::string zoneName(std::string zoneNum) {
         return _zoneName + zoneNum;
     }
 
@@ -518,7 +531,7 @@ TEST_F(SingleChunkPerTagSplitPolicyTest, MultipleChunksToOneZoneWithMultipleShar
         shardId("0"), shardId("0"), shardId("1"), shardId("0")};
     checkGeneratedInitialZoneChunks(
         kShards, tags, expectedChunkRanges, expectedShardIds, ShardKeyPattern(BSON("x" << 1)));
-};
+}
 
 TEST_F(SingleChunkPerTagSplitPolicyTest, MultipleChunksToInterleavedZonesWithMultipleShards) {
     const auto zone0 = zoneName("Z0");
@@ -542,7 +555,7 @@ TEST_F(SingleChunkPerTagSplitPolicyTest, MultipleChunksToInterleavedZonesWithMul
         shardId("0"), shardId("0"), shardId("0"), shardId("1")};
     checkGeneratedInitialZoneChunks(
         kShards, tags, expectedChunkRanges, expectedShardIds, ShardKeyPattern(BSON("x" << 1)));
-};
+}
 
 TEST_F(SingleChunkPerTagSplitPolicyTest, ZoneNotAssociatedWithAnyShardShouldFail) {
     const std::vector<ShardType> kShards{
@@ -559,7 +572,9 @@ TEST_F(SingleChunkPerTagSplitPolicyTest, ZoneNotAssociatedWithAnyShardShouldFail
 
     SingleChunkPerTagSplitPolicy splitPolicy(operationContext(), tags);
 
-    ASSERT_THROWS_CODE(splitPolicy.createFirstChunks(operationContext(), shardKeyPattern(), {}),
+    ASSERT_THROWS_CODE(splitPolicy.createFirstChunks(operationContext(),
+                                                     shardKeyPattern(),
+                                                     {UUID::gen(), ShardId("shardId")}),
                        AssertionException,
                        50973);
 }
@@ -579,13 +594,12 @@ public:
                                          bool isCollEmpty = true) {
         PresplitHashedZonesSplitPolicy splitPolicy(
             operationContext(), shardKeyPattern, tags, numInitialChunk, isCollEmpty);
-        const auto shardCollectionConfig =
-            splitPolicy.createFirstChunks(operationContext(), shardKeyPattern, {});
+        const auto shardCollectionConfig = splitPolicy.createFirstChunks(
+            operationContext(), shardKeyPattern, {UUID::gen(), expectedShardIds.front()});
 
-        const std::vector<ChunkType> expectedChunks =
-            makeChunks(expectedChunkRanges,
-                       expectedShardIds,
-                       LogicalClock::get(operationContext())->getClusterTime().asTimestamp());
+        const auto currentTime = VectorClock::get(operationContext())->getTime();
+        const std::vector<ChunkType> expectedChunks = makeChunks(
+            expectedChunkRanges, expectedShardIds, currentTime.clusterTime().asTimestamp());
         assertChunkVectorsAreEqual(expectedChunks, shardCollectionConfig.chunks);
     }
 };
@@ -604,7 +618,7 @@ BSONObj buildObj(const ShardKeyPattern& shardKeyPattern, TagsType tag, long long
         }
     }
     return bob.obj();
-};
+}
 
 /**
  * Generates chunk ranges for each tag using the split points.
@@ -1698,5 +1712,432 @@ TEST_F(PresplitHashedZonesChunksTest, RestrictionsDoNotApplyToUpperBound) {
         operationContext(), shardKeyPattern, {makeTag(zoneRange, zoneName("1"))}, 0, true);
 }
 
+class MockPipelineSource : public ReshardingSplitPolicy::SampleDocumentSource {
+public:
+    MockPipelineSource(std::list<BSONObj> toReturn) : _toReturn(std::move(toReturn)) {}
+
+    boost::optional<BSONObj> getNext() override {
+        if (_toReturn.empty()) {
+            return {};
+        }
+
+        auto next = _toReturn.front();
+        _toReturn.pop_front();
+        return next;
+    }
+
+    Pipeline* getPipeline_forTest() override {
+        return nullptr;
+    }
+
+private:
+    std::list<BSONObj> _toReturn;
+};
+
+class ReshardingInitSplitTest : public SingleChunkPerTagSplitPolicyTest {
+public:
+    /**
+     * Calls createFirstChunks() according to the given arguments and asserts that returned chunks
+     * match with the chunks created using expectedChunkRanges and expectedShardIds.
+     */
+    void checkGeneratedInitialZoneChunks(ReshardingSplitPolicy* splitPolicy,
+                                         const ShardKeyPattern& shardKeyPattern,
+                                         const std::vector<ChunkRange>& expectedChunkRanges,
+                                         const std::vector<ShardId>& expectedShardIds) {
+        const auto shardCollectionConfig = splitPolicy->createFirstChunks(
+            operationContext(), shardKeyPattern, {UUID::gen(), expectedShardIds.front()});
+
+        const auto currentTime = VectorClock::get(operationContext())->getTime();
+        const std::vector<ChunkType> expectedChunks = makeChunks(
+            expectedChunkRanges, expectedShardIds, currentTime.clusterTime().asTimestamp());
+        assertChunkVectorsAreEqual(expectedChunks, shardCollectionConfig.chunks);
+    }
+};
+
+TEST_F(ReshardingInitSplitTest, NoZones) {
+    const NamespaceString ns("reshard", "foo");
+    const ShardKeyPattern shardKey(BSON("y" << 1));
+
+    std::vector<ShardType> shardList;
+    shardList.emplace_back(
+        ShardType(shardId("0").toString(), "rs0/fakeShard0:123", {std::string("zoneA")}));
+    shardList.emplace_back(
+        ShardType(shardId("1").toString(), "rs1/fakeShard1:123", {std::string("zoneB")}));
+
+    setupShards(shardList);
+    shardRegistry()->reload(operationContext());
+
+    std::list<BSONObj> mockSamples;
+    mockSamples.push_back(BSON("y" << 10));
+    mockSamples.push_back(BSON("y" << 20));
+    mockSamples.push_back(BSON("y" << 30));
+
+    auto mockSampleSource = std::make_unique<MockPipelineSource>(std::move(mockSamples));
+
+    ReshardingSplitPolicy initSplitPolicy(
+        4 /* numInitialChunks */, boost::none /* zones */, std::move(mockSampleSource));
+
+    std::vector<ChunkRange> expectedChunkRanges = {
+        ChunkRange(BSON("y" << MINKEY), BSON("y" << 10)),
+        ChunkRange(BSON("y" << 10), BSON("y" << 20)),
+        ChunkRange(BSON("y" << 20), BSON("y" << 30)),
+        ChunkRange(BSON("y" << 30), BSON("y" << MAXKEY))};
+
+    std::vector<ShardId> expectedShardForEachChunk = {
+        shardId("0"), shardId("1"), shardId("0"), shardId("1")};
+
+    checkGeneratedInitialZoneChunks(
+        &initSplitPolicy, shardKey, expectedChunkRanges, expectedShardForEachChunk);
+}
+
+TEST_F(ReshardingInitSplitTest, HashedShardKey) {
+    const NamespaceString ns("reshard", "foo");
+    const ShardKeyPattern shardKey(BSON("y"
+                                        << "hashed"));
+
+    std::vector<ShardType> shardList;
+    shardList.emplace_back(
+        ShardType(shardId("0").toString(), "rs0/fakeShard0:123", {std::string("zoneA")}));
+    shardList.emplace_back(
+        ShardType(shardId("1").toString(), "rs1/fakeShard1:123", {std::string("zoneB")}));
+
+    setupShards(shardList);
+    shardRegistry()->reload(operationContext());
+
+    std::list<BSONObj> mockSamples;
+    mockSamples.push_back(BSON("y" << 7766103514953448109LL));
+    mockSamples.push_back(BSON("y" << -9117533237618642180LL));
+    mockSamples.push_back(BSON("y" << -1196399207910989725LL));
+
+    auto mockSampleSource = std::make_unique<MockPipelineSource>(std::move(mockSamples));
+
+    ReshardingSplitPolicy initSplitPolicy(
+        4 /* numInitialChunks */, boost::none /* zones */, std::move(mockSampleSource));
+
+    std::vector<ChunkRange> expectedChunkRanges = {
+        ChunkRange(BSON("y" << MINKEY), BSON("y" << -9117533237618642180LL)),
+        ChunkRange(BSON("y" << -9117533237618642180LL), BSON("y" << -1196399207910989725LL)),
+        ChunkRange(BSON("y" << -1196399207910989725LL), BSON("y" << 7766103514953448109LL)),
+        ChunkRange(BSON("y" << 7766103514953448109LL), BSON("y" << MAXKEY))};
+
+    std::vector<ShardId> expectedShardForEachChunk = {
+        shardId("0"), shardId("1"), shardId("0"), shardId("1")};
+
+    checkGeneratedInitialZoneChunks(
+        &initSplitPolicy, shardKey, expectedChunkRanges, expectedShardForEachChunk);
+}
+
+TEST_F(ReshardingInitSplitTest, SingleInitialChunk) {
+    const NamespaceString ns("reshard", "foo");
+    const ShardKeyPattern shardKey(BSON("y" << 1));
+
+    std::vector<ShardType> shardList;
+    shardList.emplace_back(
+        ShardType(shardId("0").toString(), "rs0/fakeShard0:123", {std::string("zoneA")}));
+    shardList.emplace_back(
+        ShardType(shardId("1").toString(), "rs1/fakeShard1:123", {std::string("zoneB")}));
+
+    setupShards(shardList);
+    shardRegistry()->reload(operationContext());
+
+    std::list<BSONObj> mockSamples;
+
+    auto mockSampleSource = std::make_unique<MockPipelineSource>(std::move(mockSamples));
+
+    ReshardingSplitPolicy initSplitPolicy(
+        1 /* numInitialChunks */, boost::none /* zones */, std::move(mockSampleSource));
+
+    std::vector<ChunkRange> expectedChunkRanges = {
+        ChunkRange(BSON("y" << MINKEY), BSON("y" << MAXKEY))};
+
+    std::vector<ShardId> expectedShardForEachChunk = {shardId("0")};
+
+    checkGeneratedInitialZoneChunks(
+        &initSplitPolicy, shardKey, expectedChunkRanges, expectedShardForEachChunk);
+}
+
+TEST_F(ReshardingInitSplitTest, ZonesCoversEntireDomainButInsufficient) {
+    const NamespaceString ns("reshard", "foo");
+    const ShardKeyPattern shardKey(BSON("y" << 1));
+
+    std::vector<ShardType> shardList;
+    shardList.emplace_back(
+        ShardType(shardId("0").toString(), "rs0/fakeShard0:123", {std::string("zoneA")}));
+    shardList.emplace_back(
+        ShardType(shardId("1").toString(), "rs1/fakeShard1:123", {std::string("zoneB")}));
+
+    setupShards(shardList);
+    shardRegistry()->reload(operationContext());
+
+    std::list<BSONObj> mockSamples;
+    mockSamples.push_back(BSON("y" << 10));
+    mockSamples.push_back(BSON("y" << 20));
+    mockSamples.push_back(BSON("y" << 30));
+
+    auto mockSampleSource = std::make_unique<MockPipelineSource>(std::move(mockSamples));
+
+    std::vector<TagsType> zones;
+    zones.emplace_back(nss(), "zoneB", ChunkRange(BSON("y" << MINKEY), BSON("y" << 0)));
+    zones.emplace_back(nss(), "zoneA", ChunkRange(BSON("y" << 0), BSON("y" << MAXKEY)));
+
+    ReshardingSplitPolicy initSplitPolicy(
+        4 /* numInitialChunks */, zones, std::move(mockSampleSource));
+
+    std::vector<ChunkRange> expectedChunkRanges = {
+        ChunkRange(BSON("y" << MINKEY), BSON("y" << 0)),
+        ChunkRange(BSON("y" << 0), BSON("y" << 10)),
+        ChunkRange(BSON("y" << 10), BSON("y" << 20)),
+        ChunkRange(BSON("y" << 20), BSON("y" << MAXKEY))};
+
+    std::vector<ShardId> expectedShardForEachChunk = {
+        shardId("1"), shardId("0"), shardId("0"), shardId("0")};
+
+    checkGeneratedInitialZoneChunks(
+        &initSplitPolicy, shardKey, expectedChunkRanges, expectedShardForEachChunk);
+}
+
+TEST_F(ReshardingInitSplitTest, SamplesCoincidingWithZones) {
+    const NamespaceString ns("reshard", "foo");
+    const ShardKeyPattern shardKey(BSON("y" << 1));
+
+    std::vector<ShardType> shardList;
+    shardList.emplace_back(
+        ShardType(shardId("0").toString(), "rs0/fakeShard0:123", {std::string("zoneA")}));
+    shardList.emplace_back(
+        ShardType(shardId("1").toString(), "rs1/fakeShard1:123", {std::string("zoneB")}));
+
+    setupShards(shardList);
+    shardRegistry()->reload(operationContext());
+
+    std::list<BSONObj> mockSamples;
+    mockSamples.push_back(BSON("y" << 10));
+    mockSamples.push_back(BSON("y" << 20));
+    mockSamples.push_back(BSON("y" << 30));
+
+    auto mockSampleSource = std::make_unique<MockPipelineSource>(std::move(mockSamples));
+
+    std::vector<TagsType> zones;
+    zones.emplace_back(nss(), "zoneA", ChunkRange(BSON("y" << 10), BSON("y" << 20)));
+
+    ReshardingSplitPolicy initSplitPolicy(
+        4 /* numInitialChunks */, zones, std::move(mockSampleSource));
+
+    std::vector<ChunkRange> expectedChunkRanges = {
+        ChunkRange(BSON("y" << MINKEY), BSON("y" << 10)),
+        ChunkRange(BSON("y" << 10), BSON("y" << 20)),
+        ChunkRange(BSON("y" << 20), BSON("y" << 30)),
+        ChunkRange(BSON("y" << 30), BSON("y" << MAXKEY))};
+
+    std::vector<ShardId> expectedShardForEachChunk = {
+        shardId("0"),  // hole
+        shardId("0"),  // zoneA
+        shardId("1"),  // hole
+        shardId("1"),  // hole
+    };
+
+    checkGeneratedInitialZoneChunks(
+        &initSplitPolicy, shardKey, expectedChunkRanges, expectedShardForEachChunk);
+}
+
+TEST_F(ReshardingInitSplitTest, ZoneWithHoles) {
+    const NamespaceString ns("reshard", "foo");
+    const ShardKeyPattern shardKey(BSON("y" << 1));
+
+    std::vector<ShardType> shardList;
+    shardList.emplace_back(
+        ShardType(shardId("0").toString(), "rs0/fakeShard0:123", {std::string("zoneA")}));
+    shardList.emplace_back(
+        ShardType(shardId("1").toString(), "rs1/fakeShard1:123", {std::string("zoneB")}));
+
+    setupShards(shardList);
+    shardRegistry()->reload(operationContext());
+
+    std::list<BSONObj> mockSamples;
+
+    auto mockSampleSource = std::make_unique<MockPipelineSource>(std::move(mockSamples));
+
+    std::vector<TagsType> zones;
+    zones.emplace_back(nss(), "zoneB", ChunkRange(BSON("y" << 0), BSON("y" << 20)));
+    zones.emplace_back(nss(), "zoneA", ChunkRange(BSON("y" << 30), BSON("y" << 40)));
+
+    ReshardingSplitPolicy initSplitPolicy(
+        4 /* numInitialChunks */, zones, std::move(mockSampleSource));
+
+    std::vector<ChunkRange> expectedChunkRanges = {
+        ChunkRange(BSON("y" << MINKEY), BSON("y" << 0)),
+        ChunkRange(BSON("y" << 0), BSON("y" << 20)),
+        ChunkRange(BSON("y" << 20), BSON("y" << 30)),
+        ChunkRange(BSON("y" << 30), BSON("y" << 40)),
+        ChunkRange(BSON("y" << 40), BSON("y" << MAXKEY))};
+
+    std::vector<ShardId> expectedShardForEachChunk = {
+        shardId("0"),  // hole
+        shardId("1"),  // zoneB
+        shardId("0"),  // hole
+        shardId("0"),  // zoneA
+        shardId("1"),  // hole
+    };
+
+    checkGeneratedInitialZoneChunks(
+        &initSplitPolicy, shardKey, expectedChunkRanges, expectedShardForEachChunk);
+}
+
+TEST_F(ReshardingInitSplitTest, UnsortedZoneWithHoles) {
+    const NamespaceString ns("reshard", "foo");
+    const ShardKeyPattern shardKey(BSON("y" << 1));
+
+    std::vector<ShardType> shardList;
+    shardList.emplace_back(
+        ShardType(shardId("0").toString(), "rs0/fakeShard0:123", {std::string("zoneA")}));
+    shardList.emplace_back(
+        ShardType(shardId("1").toString(), "rs1/fakeShard1:123", {std::string("zoneB")}));
+
+    setupShards(shardList);
+    shardRegistry()->reload(operationContext());
+
+    std::list<BSONObj> mockSamples;
+
+    auto mockSampleSource = std::make_unique<MockPipelineSource>(std::move(mockSamples));
+
+    std::vector<TagsType> zones;
+    zones.emplace_back(nss(), "zoneA", ChunkRange(BSON("y" << 30), BSON("y" << 40)));
+    zones.emplace_back(nss(), "zoneB", ChunkRange(BSON("y" << 0), BSON("y" << 20)));
+
+    ReshardingSplitPolicy initSplitPolicy(
+        4 /* numInitialChunks */, zones, std::move(mockSampleSource));
+
+    std::vector<ChunkRange> expectedChunkRanges = {
+        ChunkRange(BSON("y" << MINKEY), BSON("y" << 0)),
+        ChunkRange(BSON("y" << 0), BSON("y" << 20)),
+        ChunkRange(BSON("y" << 20), BSON("y" << 30)),
+        ChunkRange(BSON("y" << 30), BSON("y" << 40)),
+        ChunkRange(BSON("y" << 40), BSON("y" << MAXKEY))};
+
+    std::vector<ShardId> expectedShardForEachChunk = {
+        shardId("0"),  // hole
+        shardId("1"),  // zoneB
+        shardId("0"),  // hole
+        shardId("0"),  // zoneA
+        shardId("1"),  // hole
+    };
+
+    checkGeneratedInitialZoneChunks(
+        &initSplitPolicy, shardKey, expectedChunkRanges, expectedShardForEachChunk);
+}
+
+TEST_F(ReshardingInitSplitTest, ZonesIsPrefixOfReshardKey) {
+    const NamespaceString ns("reshard", "foo");
+    const ShardKeyPattern shardKey(BSON("y" << 1 << "z" << 1));
+
+    std::vector<ShardType> shardList;
+    shardList.emplace_back(
+        ShardType(shardId("0").toString(), "rs0/fakeShard0:123", {std::string("zoneA")}));
+    shardList.emplace_back(
+        ShardType(shardId("1").toString(), "rs1/fakeShard1:123", {std::string("zoneB")}));
+
+    setupShards(shardList);
+    shardRegistry()->reload(operationContext());
+
+    std::list<BSONObj> mockSamples;
+    auto mockSampleSource = std::make_unique<MockPipelineSource>(std::move(mockSamples));
+
+    std::vector<TagsType> zones;
+    zones.emplace_back(nss(), "zoneB", ChunkRange(BSON("y" << MINKEY), BSON("y" << 0)));
+    zones.emplace_back(nss(), "zoneA", ChunkRange(BSON("y" << 0), BSON("y" << MAXKEY)));
+
+    ReshardingSplitPolicy initSplitPolicy(
+        2 /* numInitialChunks */, zones, std::move(mockSampleSource));
+
+    std::vector<ChunkRange> expectedChunkRanges = {
+        ChunkRange(BSON("y" << MINKEY << "z" << MINKEY), BSON("y" << 0 << "z" << MINKEY)),
+        ChunkRange(BSON("y" << 0 << "z" << MINKEY), BSON("y" << MAXKEY << "z" << MINKEY)),
+        ChunkRange(BSON("y" << MAXKEY << "z" << MINKEY), BSON("y" << MAXKEY << "z" << MAXKEY)),
+    };
+
+    std::vector<ShardId> expectedShardForEachChunk = {
+        shardId("1"),
+        shardId("0"),
+        shardId("0"),
+    };
+
+    checkGeneratedInitialZoneChunks(
+        &initSplitPolicy, shardKey, expectedChunkRanges, expectedShardForEachChunk);
+}
+
+TEST_F(ReshardingInitSplitTest, ZonesHasIncompatibleReshardKey) {
+    const NamespaceString ns("reshard", "foo");
+    const ShardKeyPattern shardKey(BSON("y" << 1 << "z" << 1));
+
+    std::vector<ShardType> shardList;
+    shardList.emplace_back(
+        ShardType(shardId("0").toString(), "rs0/fakeShard0:123", {std::string("zoneA")}));
+    shardList.emplace_back(
+        ShardType(shardId("1").toString(), "rs1/fakeShard1:123", {std::string("zoneB")}));
+
+    setupShards(shardList);
+    shardRegistry()->reload(operationContext());
+
+    std::list<BSONObj> mockSamples;
+    auto mockSampleSource = std::make_unique<MockPipelineSource>(std::move(mockSamples));
+
+    std::vector<TagsType> zones;
+    zones.emplace_back(nss(), "zoneB", ChunkRange(BSON("x" << MINKEY), BSON("x" << 0)));
+    zones.emplace_back(nss(), "zoneA", ChunkRange(BSON("x" << 0), BSON("x" << MAXKEY)));
+
+    ReshardingSplitPolicy initSplitPolicy(
+        2 /* numInitialChunks */, zones, std::move(mockSampleSource));
+
+    SplitPolicyParams params{UUID::gen(), shardId("0")};
+    ASSERT_THROWS(initSplitPolicy.createFirstChunks(operationContext(), shardKey, params),
+                  DBException);
+}
+
+TEST_F(ReshardingInitSplitTest, InsufficientSamples) {
+    const NamespaceString ns("reshard", "foo");
+    const ShardKeyPattern shardKey(BSON("y" << 1));
+
+    std::vector<ShardType> shardList;
+    shardList.emplace_back(
+        ShardType(shardId("0").toString(), "rs0/fakeShard0:123", {std::string("zoneA")}));
+    shardList.emplace_back(
+        ShardType(shardId("1").toString(), "rs1/fakeShard1:123", {std::string("zoneB")}));
+
+    setupShards(shardList);
+    shardRegistry()->reload(operationContext());
+
+    std::list<BSONObj> mockSamples;
+    mockSamples.push_back(BSON("x" << 10 << "y" << 10));
+    auto mockSampleSource = std::make_unique<MockPipelineSource>(std::move(mockSamples));
+
+    ReshardingSplitPolicy initSplitPolicy(
+        10 /* numInitialChunks */, boost::none /* zones */, std::move(mockSampleSource));
+
+    SplitPolicyParams params{UUID::gen(), shardId("0")};
+    ASSERT_THROWS(initSplitPolicy.createFirstChunks(operationContext(), shardKey, params),
+                  DBException);
+}
+
+TEST_F(ReshardingInitSplitTest, ZeroInitialChunks) {
+    const NamespaceString ns("reshard", "foo");
+    const ShardKeyPattern shardKey(BSON("y" << 1));
+
+    std::vector<ShardType> shardList;
+    shardList.emplace_back(
+        ShardType(shardId("0").toString(), "rs0/fakeShard0:123", {std::string("zoneA")}));
+    shardList.emplace_back(
+        ShardType(shardId("1").toString(), "rs1/fakeShard1:123", {std::string("zoneB")}));
+
+    setupShards(shardList);
+    shardRegistry()->reload(operationContext());
+
+    std::list<BSONObj> mockSamples;
+    auto mockSampleSource = std::make_unique<MockPipelineSource>(std::move(mockSamples));
+
+    ASSERT_THROWS(ReshardingSplitPolicy(0 /* numInitialChunks */,
+                                        boost::none /* zones */,
+                                        std::move(mockSampleSource)),
+                  DBException);
+}
+
 }  // namespace
 }  // namespace mongo
