@@ -27,39 +27,29 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
-#include "mongo/platform/basic.h"
-
-#include <set>
-#include <string>
 #include <vector>
 
 #include "mongo/client/read_preference.h"
 #include "mongo/client/remote_command_targeter_factory_mock.h"
 #include "mongo/client/remote_command_targeter_mock.h"
+#include "mongo/db/catalog/collection_mock.h"
+#include "mongo/db/s/config/config_server_test_fixture.h"
 #include "mongo/db/s/config/initial_split_policy.h"
-#include "mongo/db/s/config/sharding_catalog_manager.h"
 #include "mongo/rpc/metadata/tracking_metadata.h"
+#include "mongo/s/balancer_configuration.h"
 #include "mongo/s/catalog/type_chunk.h"
-#include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/client/shard_registry.h"
-#include "mongo/s/config_server_test_fixture.h"
-#include "mongo/s/grid.h"
+#include "mongo/s/request_types/sharded_ddl_commands_gen.h"
 #include "mongo/s/shard_key_pattern.h"
-#include "mongo/util/scopeguard.h"
-#include "mongo/util/time_support.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
+
 
 namespace mongo {
 namespace {
 
-using executor::NetworkInterfaceMock;
 using executor::RemoteCommandRequest;
-using executor::RemoteCommandResponse;
-using executor::TaskExecutor;
-using std::set;
-using std::string;
-using std::vector;
 using unittest::assertGet;
 
 class ShardCollectionTestBase : public ConfigServerTestFixture {
@@ -69,7 +59,7 @@ protected:
                            const BSONArray& splitPoints) {
         onCommand([&](const RemoteCommandRequest& request) {
             ASSERT_EQUALS(shardHost, request.target);
-            string cmdName = request.cmdObj.firstElement().fieldName();
+            std::string cmdName = request.cmdObj.firstElement().fieldName();
             ASSERT_EQUALS("autoSplitVector", cmdName);
             // autoSplitVector concatenates the collection name to the command's db
             const auto receivedNs =
@@ -79,16 +69,14 @@ protected:
             ASSERT_BSONOBJ_EQ(keyPattern.toBSON(), request.cmdObj["keyPattern"].Obj());
             ASSERT_BSONOBJ_EQ(keyPattern.getKeyPattern().globalMin(), request.cmdObj["min"].Obj());
             ASSERT_BSONOBJ_EQ(keyPattern.getKeyPattern().globalMax(), request.cmdObj["max"].Obj());
-            ASSERT_EQUALS(64 * 1024 * 1024ULL,
+            ASSERT_EQUALS(ChunkSizeSettingsType::kDefaultMaxChunkSizeBytes,
                           static_cast<uint64_t>(request.cmdObj["maxChunkSizeBytes"].numberLong()));
-            ASSERT_EQUALS(0, request.cmdObj["maxSplitPoints"].numberLong());
-            ASSERT_EQUALS(0, request.cmdObj["maxChunkObjects"].numberLong());
 
             ASSERT_BSONOBJ_EQ(
                 ReadPreferenceSetting(ReadPreference::PrimaryPreferred).toContainingBSON(),
                 rpc::TrackingMetadata::removeTrackingData(request.metadata));
 
-            return BSON("ok" << 1 << "splitKeys" << splitPoints);
+            return BSON("ok" << 1 << "splitKeys" << splitPoints << "continuation" << false);
         });
     }
 
@@ -109,8 +97,10 @@ protected:
 };
 
 TEST_F(CreateFirstChunksTest, Split_Disallowed_With_Both_SplitPoints_And_Zones) {
-    ShardsvrShardCollection request;
+    CreateCollectionRequest request;
     std::vector<BSONObj> splitPoints = {BSONObj()};
+    request.setNumInitialChunks(0);
+    request.setPresplitHashedZones(false);
     request.setInitialSplitPoints(splitPoints);
     std::vector<TagsType> tags = {
         TagsType(kNamespace,
@@ -120,7 +110,9 @@ TEST_F(CreateFirstChunksTest, Split_Disallowed_With_Both_SplitPoints_And_Zones)
     ASSERT_THROWS_CODE(
         InitialSplitPolicy::calculateOptimizationStrategy(operationContext(),
                                                           kShardKeyPattern,
-                                                          request,
+                                                          request.getNumInitialChunks().value(),
+                                                          request.getPresplitHashedZones().value(),
+                                                          request.getInitialSplitPoints(),
                                                           tags,
                                                           2 /* numShards */,
                                                           true /* collectionIsEmpty */),
@@ -130,7 +122,9 @@ TEST_F(CreateFirstChunksTest, Split_Disallowed_With_Both_SplitPoints_And_Zones)
     ASSERT_THROWS_CODE(
         InitialSplitPolicy::calculateOptimizationStrategy(operationContext(),
                                                           kShardKeyPattern,
-                                                          request,
+                                                          request.getNumInitialChunks().value(),
+                                                          request.getPresplitHashedZones().value(),
+                                                          request.getInitialSplitPoints(),
                                                           tags,
                                                           2 /* numShards */,
                                                           false /* collectionIsEmpty */),
@@ -154,21 +148,31 @@ TEST_F(CreateFirstChunksTest, NonEmptyCollection_SplitPoints_FromSplitVector_Man
     setupShards(kShards);
     shardRegistry()->reload(operationContext());
 
+    auto uuid = UUID::gen();
+    CollectionCatalog::write(getServiceContext(), [&](CollectionCatalog& catalog) {
+        catalog.registerCollection(
+            operationContext(), uuid, std::make_shared<CollectionMock>(kNamespace));
+    });
+
     auto future = launchAsync([&] {
         ThreadClient tc("Test", getServiceContext());
         auto opCtx = cc().makeOperationContext();
 
-        ShardsvrShardCollection request;
-        auto optimization =
-            InitialSplitPolicy::calculateOptimizationStrategy(operationContext(),
-                                                              kShardKeyPattern,
-                                                              request,
-                                                              {}, /* tags */
-                                                              3 /* numShards */,
-                                                              false /* collectionIsEmpty */);
+        CreateCollectionRequest request;
+        request.setNumInitialChunks(0);
+        request.setPresplitHashedZones(false);
+        auto optimization = InitialSplitPolicy::calculateOptimizationStrategy(
+            operationContext(),
+            kShardKeyPattern,
+            request.getNumInitialChunks().value(),
+            request.getPresplitHashedZones().value(),
+            request.getInitialSplitPoints(),
+            {}, /* tags */
+            3 /* numShards */,
+            false /* collectionIsEmpty */);
         ASSERT(!optimization->isOptimized());
         return optimization->createFirstChunks(
-            opCtx.get(), kShardKeyPattern, {kNamespace, ShardId("shard1")});
+            opCtx.get(), kShardKeyPattern, {uuid, ShardId("shard1")});
     });
 
     expectSplitVector(connStr.getServers()[0], kShardKeyPattern, BSON_ARRAY(BSON("x" << 0)));
@@ -203,19 +207,22 @@ TEST_F(CreateFirstChunksTest, NonEmptyCollection_SplitPoints_FromClient_ManyChun
         std::vector<TagsType> zones{};
         bool collectionIsEmpty = false;
 
-        ShardsvrShardCollection request;
+        CreateCollectionRequest request;
+        request.setNumInitialChunks(0);
+        request.setPresplitHashedZones(false);
         request.setInitialSplitPoints(splitPoints);
-        auto optimization = InitialSplitPolicy::calculateOptimizationStrategy(operationContext(),
-                                                                              kShardKeyPattern,
-                                                                              request,
-                                                                              zones,
-                                                                              3 /* numShards */,
-                                                                              collectionIsEmpty);
-
-
+        auto optimization = InitialSplitPolicy::calculateOptimizationStrategy(
+            operationContext(),
+            kShardKeyPattern,
+            request.getNumInitialChunks().value(),
+            request.getPresplitHashedZones().value(),
+            request.getInitialSplitPoints(),
+            zones,
+            3 /* numShards */,
+            collectionIsEmpty);
         ASSERT(optimization->isOptimized());
         return optimization->createFirstChunks(
-            opCtx.get(), kShardKeyPattern, {kNamespace, ShardId("shard1")});
+            opCtx.get(), kShardKeyPattern, {UUID::gen(), ShardId("shard1")});
     });
 
     const auto& firstChunks = future.default_timed_get();
@@ -237,15 +244,22 @@ TEST_F(CreateFirstChunksTest, NonEmptyCollection_WithZones_OneChunkToPrimary) {
                  ChunkRange(kShardKeyPattern.getKeyPattern().globalMin(), BSON("x" << 0)))};
     bool collectionIsEmpty = false;
 
-    ShardsvrShardCollection request;
-    auto optimization = InitialSplitPolicy::calculateOptimizationStrategy(
-        operationContext(), kShardKeyPattern, request, zones, 3 /* numShards */, collectionIsEmpty);
-
-
+    CreateCollectionRequest request;
+    request.setNumInitialChunks(0);
+    request.setPresplitHashedZones(false);
+    auto optimization =
+        InitialSplitPolicy::calculateOptimizationStrategy(operationContext(),
+                                                          kShardKeyPattern,
+                                                          request.getNumInitialChunks().value(),
+                                                          request.getPresplitHashedZones().value(),
+                                                          request.getInitialSplitPoints(),
+                                                          zones,
+                                                          3 /* numShards */,
+                                                          collectionIsEmpty);
     ASSERT(optimization->isOptimized());
 
     const auto firstChunks = optimization->createFirstChunks(
-        operationContext(), kShardKeyPattern, {kNamespace, ShardId("shard1")});
+        operationContext(), kShardKeyPattern, {UUID::gen(), ShardId("shard1")});
 
     ASSERT_EQ(1U, firstChunks.chunks.size());
     ASSERT_EQ(kShards[1].getName(), firstChunks.chunks[0].getShard());
@@ -275,20 +289,23 @@ TEST_F(CreateFirstChunksTest, EmptyCollection_SplitPoints_FromClient_ManyChunksD
         std::vector<TagsType> zones{};
         bool collectionIsEmpty = true;
 
-        ShardsvrShardCollection request;
+        CreateCollectionRequest request;
+        request.setNumInitialChunks(0);
+        request.setPresplitHashedZones(false);
         request.setInitialSplitPoints(splitPoints);
-        auto optimization = InitialSplitPolicy::calculateOptimizationStrategy(operationContext(),
-                                                                              kShardKeyPattern,
-                                                                              request,
-                                                                              zones,
-                                                                              3 /* numShards */,
-                                                                              collectionIsEmpty);
-
-
+        auto optimization = InitialSplitPolicy::calculateOptimizationStrategy(
+            operationContext(),
+            kShardKeyPattern,
+            request.getNumInitialChunks().value(),
+            request.getPresplitHashedZones().value(),
+            request.getInitialSplitPoints(),
+            zones,
+            3 /* numShards */,
+            collectionIsEmpty);
         ASSERT(optimization->isOptimized());
 
         return optimization->createFirstChunks(
-            operationContext(), kShardKeyPattern, {kNamespace, ShardId("shard1")});
+            operationContext(), kShardKeyPattern, {UUID::gen(), ShardId("shard1")});
     });
 
     const auto& firstChunks = future.default_timed_get();
@@ -322,20 +339,23 @@ TEST_F(CreateFirstChunksTest, EmptyCollection_NoSplitPoints_OneChunkToPrimary) {
         std::vector<TagsType> zones{};
         bool collectionIsEmpty = true;
 
-        ShardsvrShardCollection request;
+        CreateCollectionRequest request;
+        request.setNumInitialChunks(0);
+        request.setPresplitHashedZones(false);
         request.setInitialSplitPoints(splitPoints);
-        auto optimization = InitialSplitPolicy::calculateOptimizationStrategy(operationContext(),
-                                                                              kShardKeyPattern,
-                                                                              request,
-                                                                              zones,
-                                                                              3 /* numShards */,
-                                                                              collectionIsEmpty);
-
-
+        auto optimization = InitialSplitPolicy::calculateOptimizationStrategy(
+            operationContext(),
+            kShardKeyPattern,
+            request.getNumInitialChunks().value(),
+            request.getPresplitHashedZones().value(),
+            request.getInitialSplitPoints(),
+            zones,
+            3 /* numShards */,
+            collectionIsEmpty);
         ASSERT(optimization->isOptimized());
 
         return optimization->createFirstChunks(
-            operationContext(), kShardKeyPattern, {kNamespace, ShardId("shard1")});
+            operationContext(), kShardKeyPattern, {UUID::gen(), ShardId("shard1")});
     });
 
     const auto& firstChunks = future.default_timed_get();
@@ -356,15 +376,22 @@ TEST_F(CreateFirstChunksTest, EmptyCollection_WithZones_ManyChunksOnFirstZoneSha
                  "TestZone",
                  ChunkRange(kShardKeyPattern.getKeyPattern().globalMin(), BSON("x" << 0)))};
     bool collectionIsEmpty = true;
-    ShardsvrShardCollection request;
-    auto optimization = InitialSplitPolicy::calculateOptimizationStrategy(
-        operationContext(), kShardKeyPattern, request, zones, 3 /* numShards */, collectionIsEmpty);
-
-
+    CreateCollectionRequest request;
+    request.setNumInitialChunks(0);
+    request.setPresplitHashedZones(false);
+    auto optimization =
+        InitialSplitPolicy::calculateOptimizationStrategy(operationContext(),
+                                                          kShardKeyPattern,
+                                                          request.getNumInitialChunks().value(),
+                                                          request.getPresplitHashedZones().value(),
+                                                          request.getInitialSplitPoints(),
+                                                          zones,
+                                                          3 /* numShards */,
+                                                          collectionIsEmpty);
     ASSERT(optimization->isOptimized());
 
     const auto firstChunks = optimization->createFirstChunks(
-        operationContext(), kShardKeyPattern, {kNamespace, ShardId("shard1")});
+        operationContext(), kShardKeyPattern, {UUID::gen(), ShardId("shard1")});
 
     ASSERT_EQ(2U, firstChunks.chunks.size());
     ASSERT_EQ(kShards[0].getName(), firstChunks.chunks[0].getShard());
