@@ -27,26 +27,25 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
 #include <boost/optional.hpp>
-#include <memory>
 
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/namespace_string.h"
-#include "mongo/db/repl/storage_interface_impl.h"
+#include "mongo/db/persistent_task_store.h"
 #include "mongo/db/s/metadata_manager.h"
+#include "mongo/db/s/range_deletion_task_gen.h"
+#include "mongo/db/s/shard_server_test_fixture.h"
 #include "mongo/db/s/sharding_runtime_d_params_gen.h"
 #include "mongo/db/s/sharding_state.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/vector_clock.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/client/shard_registry.h"
-#include "mongo/s/shard_server_test_fixture.h"
 #include "mongo/stdx/condition_variable.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/assert_util.h"
@@ -68,7 +67,6 @@ protected:
         ShardServerTestFixture::setUp();
         _manager = std::make_shared<MetadataManager>(
             getServiceContext(), kNss, executor(), makeEmptyMetadata());
-        _storage = std::make_unique<repl::StorageInterfaceImpl>();
         orphanCleanupDelaySecs.store(1);
     }
 
@@ -88,11 +86,18 @@ protected:
             nullptr,
             false,
             epoch,
-            {ChunkType{kNss, range, ChunkVersion(1, 0, epoch), kOtherShard}});
+            Timestamp(1, 1),
+            boost::none /* timeseriesFields */,
+            boost::none,
+            boost::none /* chunkSizeBytes */,
+            true,
+            {ChunkType{uuid, range, ChunkVersion({epoch, Timestamp(1, 1)}, {1, 0}), kOtherShard}});
 
-        std::shared_ptr<ChunkManager> cm = std::make_shared<ChunkManager>(rt, boost::none);
-
-        return CollectionMetadata(cm, kThisShard);
+        return CollectionMetadata(ChunkManager(kThisShard,
+                                               DatabaseVersion(UUID::gen(), Timestamp(1, 1)),
+                                               makeStandaloneRoutingTableHistory(std::move(rt)),
+                                               boost::none),
+                                  kThisShard);
     }
 
     /**
@@ -103,16 +108,11 @@ protected:
      * It will fassert if the chunk bounds are incorrect or overlap an existing chunk or if the
      * chunk version is lower than the maximum one.
      */
-    static CollectionMetadata cloneMetadataPlusChunk(const ScopedCollectionDescription& collDesc,
-                                                     const ChunkRange& range) {
-        return cloneMetadataPlusChunk(collDesc.get(), range);
-    }
-
     static CollectionMetadata cloneMetadataPlusChunk(const CollectionMetadata& collMetadata,
                                                      const ChunkRange& range) {
         const BSONObj& minKey = range.getMin();
         const BSONObj& maxKey = range.getMax();
-        ASSERT(!rangeMapOverlaps(collMetadata.getOwnedChunks(), minKey, maxKey));
+        ASSERT(!rangeMapOverlaps(collMetadata.getChunks(), minKey, maxKey));
 
         auto cm = collMetadata.getChunkManager();
 
@@ -126,29 +126,39 @@ protected:
 
         if (SimpleBSONObjComparator::kInstance.evaluate(chunkToSplit.getMin() < minKey)) {
             chunkVersion.incMajor();
-            splitChunks.emplace_back(
-                kNss, ChunkRange(chunkToSplit.getMin(), minKey), chunkVersion, kOtherShard);
+            splitChunks.emplace_back(collMetadata.getUUID(),
+                                     ChunkRange(chunkToSplit.getMin(), minKey),
+                                     chunkVersion,
+                                     kOtherShard);
         }
 
         chunkVersion.incMajor();
-        splitChunks.emplace_back(kNss, ChunkRange(minKey, maxKey), chunkVersion, kThisShard);
+        splitChunks.emplace_back(
+            collMetadata.getUUID(), ChunkRange(minKey, maxKey), chunkVersion, kThisShard);
 
         chunkVersion.incMajor();
-        splitChunks.emplace_back(
-            kNss, ChunkRange(maxKey, chunkToSplit.getMax()), chunkVersion, kOtherShard);
+        splitChunks.emplace_back(collMetadata.getUUID(),
+                                 ChunkRange(maxKey, chunkToSplit.getMax()),
+                                 chunkVersion,
+                                 kOtherShard);
 
-        auto rt = cm->getRoutingHistory()->makeUpdated(splitChunks);
+        auto rt = cm->getRoutingTableHistory_ForTest().makeUpdated(
+            boost::none /* timeseriesFields */, boost::none, boost::none, true, splitChunks);
 
-        return CollectionMetadata(std::make_shared<ChunkManager>(rt, boost::none), kThisShard);
+        return CollectionMetadata(ChunkManager(cm->dbPrimary(),
+                                               cm->dbVersion(),
+                                               makeStandaloneRoutingTableHistory(std::move(rt)),
+                                               boost::none),
+                                  kThisShard);
     }
 
-    static CollectionMetadata cloneMetadataMinusChunk(const ScopedCollectionDescription& collDesc,
+    static CollectionMetadata cloneMetadataMinusChunk(const CollectionMetadata& metadata,
                                                       const ChunkRange& range) {
         const BSONObj& minKey = range.getMin();
         const BSONObj& maxKey = range.getMax();
-        ASSERT(rangeMapOverlaps(collDesc->getOwnedChunks(), minKey, maxKey));
+        ASSERT(rangeMapOverlaps(metadata.getChunks(), minKey, maxKey));
 
-        auto cm = collDesc->getChunkManager();
+        auto cm = metadata.getChunkManager();
 
         const auto chunkToMoveOut = cm->findIntersectingChunkWithSimpleCollation(minKey);
         ASSERT_BSONOBJ_EQ(minKey, chunkToMoveOut.getMin());
@@ -157,124 +167,61 @@ protected:
         auto chunkVersion = cm->getVersion();
         chunkVersion.incMajor();
 
-        auto rt = cm->getRoutingHistory()->makeUpdated(
-            {ChunkType(kNss, ChunkRange(minKey, maxKey), chunkVersion, kOtherShard)});
+        auto rt = cm->getRoutingTableHistory_ForTest().makeUpdated(
+            boost::none /* timeseriesFields */,
+            boost::none,
+            boost::none,
+            true,
+            {ChunkType(metadata.getUUID(), ChunkRange(minKey, maxKey), chunkVersion, kOtherShard)});
 
-        return CollectionMetadata(std::make_shared<ChunkManager>(rt, boost::none), kThisShard);
+        return CollectionMetadata(ChunkManager(cm->dbPrimary(),
+                                               cm->dbVersion(),
+                                               makeStandaloneRoutingTableHistory(std::move(rt)),
+                                               boost::none),
+                                  kThisShard);
     }
 
     std::shared_ptr<MetadataManager> _manager;
-    std::unique_ptr<repl::StorageInterface> _storage;
 };
 
-TEST_F(MetadataManagerTest, CleanUpForMigrateIn) {
-    _manager->setFilteringMetadata(makeEmptyMetadata());
+// The 'pending' field must not be set in order for a range deletion task to succeed, but the
+// ShardServerOpObserver will submit the task for deletion upon seeing an insert without the
+// 'pending' field. The tests call removeDocumentsFromRange directly, so we want to avoid having
+// the op observer also submit the task. The ShardServerOpObserver will ignore replacement
+//  updates on the range deletions namespace though, so we can get around the issue by inserting
+// the task with the 'pending' field set, and then remove the field using a replacement update
+// after.
+RangeDeletionTask insertRangeDeletionTask(OperationContext* opCtx,
+                                          const NamespaceString& nss,
+                                          const UUID& uuid,
+                                          const ChunkRange& range,
+                                          int64_t numOrphans) {
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
+    auto migrationId = UUID::gen();
+    RangeDeletionTask t(migrationId, nss, uuid, ShardId("donor"), range, CleanWhenEnum::kDelayed);
+    t.setPending(true);
+    t.setNumOrphanDocs(numOrphans);
+    const auto currentTime = VectorClock::get(opCtx)->getTime();
+    t.setTimestamp(currentTime.clusterTime().asTimestamp());
+    store.add(opCtx, t);
 
-    CollectionOptions emptyCollOptions;
-    ASSERT_OK(_storage->createCollection(operationContext(), kNss, emptyCollOptions));
+    auto query = BSON(RangeDeletionTask::kIdFieldName << migrationId);
+    t.setPending(boost::none);
+    auto update = t.toBSON();
+    store.update(opCtx, query, update);
 
-    // Sanity checks
-    ASSERT(_manager->getActiveMetadata(boost::none)->isSharded());
-    ASSERT_EQ(0UL, _manager->getActiveMetadata(boost::none)->getOwnedChunks().size());
-
-    ChunkRange range1(BSON("key" << 0), BSON("key" << 10));
-    ChunkRange range2(BSON("key" << 10), BSON("key" << 20));
-
-    auto notif1 = _manager->beginReceive(range1);
-    ASSERT(!notif1.isReady());
-
-    auto notif2 = _manager->beginReceive(range2);
-    ASSERT(!notif2.isReady());
-
-    ASSERT_EQ(2UL, _manager->numberOfRangesToClean());
-    ASSERT_EQ(0UL, _manager->numberOfRangesToCleanStillInUse());
-}
-
-TEST_F(MetadataManagerTest, CleanUpForMigrateWithNonExistingCollection) {
-    _manager->setFilteringMetadata(makeEmptyMetadata());
-
-    // Sanity checks
-    ASSERT(_manager->getActiveMetadata(boost::none)->isSharded());
-    ASSERT_EQ(0UL, _manager->getActiveMetadata(boost::none)->getOwnedChunks().size());
-
-    ChunkRange range1(BSON("key" << 0), BSON("key" << 10));
-
-    {
-        FailPointEnableBlock fpb("suspendRangeDeletion");
-        auto notif1 = _manager->beginReceive(range1);
-
-        // This will hang if the range deletion task on the non-existing collection is scheduled
-        notif1.wait();
-    }
-}
-
-TEST_F(MetadataManagerTest,
-       ChunkInReceivingChunksListIsRemovedAfterShardKeyRefineIfMigrationSucceeded) {
-    _manager->setFilteringMetadata(makeEmptyMetadata());
-
-    // Simulate receiving a range. This will add an item to _receivingChunks.
-    ChunkRange range(BSON("key" << 0), BSON("key" << 10));
-    auto notif1 = _manager->beginReceive(range);
-
-    ASSERT_EQ(_manager->numberOfReceivingChunks(), 1);
-
-    // Simulate a situation in which the migration completes, and then the shard key is refined,
-    // before this shard discovers the updated metadata.
-    auto uuid = _manager->getActiveMetadata(boost::none)->getChunkManager()->getUUID().get();
-    ChunkRange refinedRange(BSON("key" << 0 << "other" << MINKEY),
-                            BSON("key" << 10 << "other" << MINKEY));
-    auto refinedMetadata = makeEmptyMetadata(BSON(kPattern << 1 << "other" << 1),
-                                             ChunkRange(BSON("key" << MINKEY << "other" << MINKEY),
-                                                        BSON("key" << MAXKEY << "other" << MAXKEY)),
-                                             uuid);
-
-    // Set the updated chunk map on the MetadataManager.
-    _manager->setFilteringMetadata(cloneMetadataPlusChunk(refinedMetadata, refinedRange));
-    // Because the refined range overlaps with the received range (pre-refine), this should remove
-    // the item in _receivingChunks.
-    ASSERT_EQ(_manager->numberOfReceivingChunks(), 0);
-}
-
-TEST_F(MetadataManagerTest,
-       ChunkInReceivingChunksListIsNotRemovedAfterShardKeyRefineIfNonOverlappingRangeIsReceived) {
-    _manager->setFilteringMetadata(makeEmptyMetadata());
-
-    // Simulate receiving a range. This will add an item to _receivingChunks.
-    ChunkRange range(BSON("key" << 0), BSON("key" << 10));
-    auto notif1 = _manager->beginReceive(range);
-    ASSERT_EQ(_manager->numberOfReceivingChunks(), 1);
-
-    // Simulate a situation in which the shard key is refined and this shard discovers
-    // updated metadata where it owns some range that does not overlap with the range being migrated
-    // in.
-    auto uuid = _manager->getActiveMetadata(boost::none)->getChunkManager()->getUUID().get();
-    ChunkRange refinedNonOverlappingRange(BSON("key" << -10 << "other" << MINKEY),
-                                          BSON("key" << 0 << "other" << MINKEY));
-
-    auto refinedMetadata = makeEmptyMetadata(BSON(kPattern << 1 << "other" << 1),
-                                             ChunkRange(BSON("key" << MINKEY << "other" << MINKEY),
-                                                        BSON("key" << MAXKEY << "other" << MAXKEY)),
-                                             uuid);
-
-    // Set the updated chunk map on the MetadataManager.
-    _manager->setFilteringMetadata(
-        cloneMetadataPlusChunk(refinedMetadata, refinedNonOverlappingRange));
-
-    // Because the refined range does not overlap with the received range (pre-refine), this should
-    // NOT remove the item in _receivingChunks.
-    ASSERT_EQ(_manager->numberOfReceivingChunks(), 1);
+    return t;
 }
 
 TEST_F(MetadataManagerTest, TrackOrphanedDataCleanupBlocksOnScheduledRangeDeletions) {
     ChunkRange cr1(BSON("key" << 0), BSON("key" << 10));
-
-    CollectionOptions emptyCollOptions;
-    ASSERT_OK(_storage->createCollection(operationContext(), kNss, emptyCollOptions));
+    const auto task =
+        insertRangeDeletionTask(operationContext(), kNss, _manager->getCollectionUuid(), cr1, 0);
 
     // Enable fail point to suspendRangeDeletion.
     globalFailPointRegistry().find("suspendRangeDeletion")->setMode(FailPoint::alwaysOn);
 
-    auto notifn1 = _manager->cleanUpRange(cr1, boost::none, false /*delayBeforeDeleting*/);
+    auto notifn1 = _manager->cleanUpRange(cr1, task.getId(), false /*delayBeforeDeleting*/);
     ASSERT_FALSE(notifn1.isReady());
     ASSERT_EQ(_manager->numberOfRangesToClean(), 1UL);
 
@@ -287,21 +234,24 @@ TEST_F(MetadataManagerTest, TrackOrphanedDataCleanupBlocksOnScheduledRangeDeleti
 
 TEST_F(MetadataManagerTest, CleanupNotificationsAreSignaledWhenMetadataManagerIsDestroyed) {
     const ChunkRange rangeToClean(BSON("key" << 20), BSON("key" << 30));
+    const auto task = insertRangeDeletionTask(
+        operationContext(), kNss, _manager->getCollectionUuid(), rangeToClean, 0);
 
-    _manager->setFilteringMetadata(cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none),
-                                                          {BSON("key" << 0), BSON("key" << 20)}));
+
+    _manager->setFilteringMetadata(cloneMetadataPlusChunk(
+        _manager->getActiveMetadata(boost::none)->get(), {BSON("key" << 0), BSON("key" << 20)}));
 
     _manager->setFilteringMetadata(
-        cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none), rangeToClean));
+        cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none)->get(), rangeToClean));
 
     // Optional so that it can be reset.
     boost::optional<ScopedCollectionDescription> cursorOnMovedMetadata{
         _manager->getActiveMetadata(boost::none)};
 
     _manager->setFilteringMetadata(
-        cloneMetadataMinusChunk(_manager->getActiveMetadata(boost::none), rangeToClean));
+        cloneMetadataMinusChunk(_manager->getActiveMetadata(boost::none)->get(), rangeToClean));
 
-    auto notif = _manager->cleanUpRange(rangeToClean, boost::none, false /*delayBeforeDeleting*/);
+    auto notif = _manager->cleanUpRange(rangeToClean, task.getId(), false /*delayBeforeDeleting*/);
     ASSERT(!notif.isReady());
 
     auto optNotif = _manager->trackOrphanedDataCleanup(rangeToClean);
@@ -332,8 +282,8 @@ TEST_F(MetadataManagerTest, RefreshAfterSuccessfulMigrationSinglePending) {
     ChunkRange cr1(BSON("key" << 0), BSON("key" << 10));
 
     _manager->setFilteringMetadata(
-        cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none), cr1));
-    ASSERT_EQ(_manager->getActiveMetadata(boost::none)->getOwnedChunks().size(), 1UL);
+        cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none)->get(), cr1));
+    ASSERT_EQ(_manager->getActiveMetadata(boost::none)->get().getChunks().size(), 1UL);
 }
 
 TEST_F(MetadataManagerTest, RefreshAfterSuccessfulMigrationMultiplePending) {
@@ -342,15 +292,15 @@ TEST_F(MetadataManagerTest, RefreshAfterSuccessfulMigrationMultiplePending) {
 
     {
         _manager->setFilteringMetadata(
-            cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none), cr1));
+            cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none)->get(), cr1));
         ASSERT_EQ(_manager->numberOfRangesToClean(), 0UL);
-        ASSERT_EQ(_manager->getActiveMetadata(boost::none)->getOwnedChunks().size(), 1UL);
+        ASSERT_EQ(_manager->getActiveMetadata(boost::none)->get().getChunks().size(), 1UL);
     }
 
     {
         _manager->setFilteringMetadata(
-            cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none), cr2));
-        ASSERT_EQ(_manager->getActiveMetadata(boost::none)->getOwnedChunks().size(), 2UL);
+            cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none)->get(), cr2));
+        ASSERT_EQ(_manager->getActiveMetadata(boost::none)->get().getChunks().size(), 2UL);
     }
 }
 
@@ -358,9 +308,9 @@ TEST_F(MetadataManagerTest, RefreshAfterNotYetCompletedMigrationMultiplePending)
     ChunkRange cr1(BSON("key" << 0), BSON("key" << 10));
     ChunkRange cr2(BSON("key" << 30), BSON("key" << 40));
 
-    _manager->setFilteringMetadata(cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none),
-                                                          {BSON("key" << 50), BSON("key" << 60)}));
-    ASSERT_EQ(_manager->getActiveMetadata(boost::none)->getOwnedChunks().size(), 1UL);
+    _manager->setFilteringMetadata(cloneMetadataPlusChunk(
+        _manager->getActiveMetadata(boost::none)->get(), {BSON("key" << 50), BSON("key" << 60)}));
+    ASSERT_EQ(_manager->getActiveMetadata(boost::none)->get().getChunks().size(), 1UL);
 }
 
 TEST_F(MetadataManagerTest, BeginReceiveWithOverlappingRange) {
@@ -368,9 +318,9 @@ TEST_F(MetadataManagerTest, BeginReceiveWithOverlappingRange) {
     ChunkRange cr2(BSON("key" << 30), BSON("key" << 40));
 
     _manager->setFilteringMetadata(
-        cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none), cr1));
+        cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none)->get(), cr1));
     _manager->setFilteringMetadata(
-        cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none), cr2));
+        cloneMetadataPlusChunk(_manager->getActiveMetadata(boost::none)->get(), cr2));
 
     ChunkRange crOverlap(BSON("key" << 5), BSON("key" << 35));
 }
@@ -378,16 +328,15 @@ TEST_F(MetadataManagerTest, BeginReceiveWithOverlappingRange) {
 // Tests membership functions for _rangesToClean
 TEST_F(MetadataManagerTest, RangesToCleanMembership) {
     ChunkRange cr(BSON("key" << 0), BSON("key" << 10));
+    const auto task =
+        insertRangeDeletionTask(operationContext(), kNss, _manager->getCollectionUuid(), cr, 0);
 
     ASSERT_EQ(0UL, _manager->numberOfRangesToClean());
 
-    CollectionOptions emptyCollOptions;
-    ASSERT_OK(_storage->createCollection(operationContext(), kNss, emptyCollOptions));
-
     // Enable fail point to suspendRangeDeletion.
     globalFailPointRegistry().find("suspendRangeDeletion")->setMode(FailPoint::alwaysOn);
 
-    auto notifn = _manager->cleanUpRange(cr, boost::none, false /*delayBeforeDeleting*/);
+    auto notifn = _manager->cleanUpRange(cr, task.getId(), false /*delayBeforeDeleting*/);
     ASSERT(!notifn.isReady());
     ASSERT_EQ(1UL, _manager->numberOfRangesToClean());
 
@@ -400,13 +349,13 @@ TEST_F(MetadataManagerTest, ClearUnneededChunkManagerObjectsLastSnapshotInList)
 
     auto scm1 = _manager->getActiveMetadata(boost::none);
     {
-        _manager->setFilteringMetadata(cloneMetadataPlusChunk(scm1, cr1));
+        _manager->setFilteringMetadata(cloneMetadataPlusChunk(scm1->get(), cr1));
         ASSERT_EQ(_manager->numberOfMetadataSnapshots(), 1UL);
         ASSERT_EQ(_manager->numberOfRangesToClean(), 0UL);
 
         auto scm2 = _manager->getActiveMetadata(boost::none);
-        ASSERT_EQ(scm2->getOwnedChunks().size(), 1UL);
-        _manager->setFilteringMetadata(cloneMetadataPlusChunk(scm2, cr2));
+        ASSERT_EQ(scm2->get().getChunks().size(), 1UL);
+        _manager->setFilteringMetadata(cloneMetadataPlusChunk(scm2->get(), cr2));
         ASSERT_EQ(_manager->numberOfMetadataSnapshots(), 2UL);
         ASSERT_EQ(_manager->numberOfEmptyMetadataSnapshots(), 0);
     }
@@ -415,7 +364,9 @@ TEST_F(MetadataManagerTest, ClearUnneededChunkManagerObjectsLastSnapshotInList)
     // is now out of scope, but that in scm1 should remain
     ASSERT_EQ(_manager->numberOfEmptyMetadataSnapshots(), 1);
     ASSERT_EQ(_manager->numberOfMetadataSnapshots(), 2UL);
-    ASSERT_EQ(_manager->getActiveMetadata(boost::none)->getOwnedChunks().size(), 2UL);
+
+    auto scm = _manager->getActiveMetadata(boost::none);
+    ASSERT_EQ(scm->get().getChunks().size(), 2UL);
 }
 
 TEST_F(MetadataManagerTest, ClearUnneededChunkManagerObjectSnapshotInMiddleOfList) {
@@ -425,18 +376,18 @@ TEST_F(MetadataManagerTest, ClearUnneededChunkManagerObjectSnapshotInMiddleOfLis
     ChunkRange cr4(BSON("key" << 90), BSON("key" << 100));
 
     auto scm = _manager->getActiveMetadata(boost::none);
-    _manager->setFilteringMetadata(cloneMetadataPlusChunk(scm, cr1));
+    _manager->setFilteringMetadata(cloneMetadataPlusChunk(scm->get(), cr1));
     ASSERT_EQ(_manager->numberOfMetadataSnapshots(), 1UL);
     ASSERT_EQ(_manager->numberOfRangesToClean(), 0UL);
 
     auto scm2 = _manager->getActiveMetadata(boost::none);
-    ASSERT_EQ(scm2->getOwnedChunks().size(), 1UL);
-    _manager->setFilteringMetadata(cloneMetadataPlusChunk(scm2, cr2));
+    ASSERT_EQ(scm2->get().getChunks().size(), 1UL);
+    _manager->setFilteringMetadata(cloneMetadataPlusChunk(scm2->get(), cr2));
 
     {
         auto scm3 = _manager->getActiveMetadata(boost::none);
-        ASSERT_EQ(scm3->getOwnedChunks().size(), 2UL);
-        _manager->setFilteringMetadata(cloneMetadataPlusChunk(scm3, cr3));
+        ASSERT_EQ(scm3->get().getChunks().size(), 2UL);
+        _manager->setFilteringMetadata(cloneMetadataPlusChunk(scm3->get(), cr3));
         ASSERT_EQ(_manager->numberOfMetadataSnapshots(), 3UL);
         ASSERT_EQ(_manager->numberOfEmptyMetadataSnapshots(), 0);
 
@@ -451,8 +402,8 @@ TEST_F(MetadataManagerTest, ClearUnneededChunkManagerObjectSnapshotInMiddleOfLis
          * ]
          */
         scm2 = _manager->getActiveMetadata(boost::none);
-        ASSERT_EQ(scm2->getOwnedChunks().size(), 3UL);
-        _manager->setFilteringMetadata(cloneMetadataPlusChunk(scm2, cr4));
+        ASSERT_EQ(scm2->get().getChunks().size(), 3UL);
+        _manager->setFilteringMetadata(cloneMetadataPlusChunk(scm2->get(), cr4));
         ASSERT_EQ(_manager->numberOfMetadataSnapshots(), 4UL);
         ASSERT_EQ(_manager->numberOfEmptyMetadataSnapshots(), 1);
     }
