@@ -42,7 +42,7 @@
 #include "mongo/db/s/migration_session_id.h"
 #include "mongo/db/s/session_catalog_migration_source.h"
 #include "mongo/platform/mutex.h"
-#include "mongo/s/request_types/move_chunk_request.h"
+#include "mongo/s/request_types/move_range_request_gen.h"
 #include "mongo/s/shard_key_pattern.h"
 #include "mongo/stdx/condition_variable.h"
 #include "mongo/util/net/hostandport.h"
@@ -52,31 +52,33 @@ namespace mongo {
 class BSONArrayBuilder;
 class BSONObjBuilder;
 class Collection;
+class CollectionPtr;
 class Database;
 class RecordId;
 
+// Overhead to prevent mods buffers from being too large
+const long long kFixedCommandOverhead = 32 * 1024;
+
 /**
  * Used to commit work for LogOpForSharding. Used to keep track of changes in documents that are
  * part of a chunk being migrated.
  */
 class LogTransactionOperationsForShardingHandler final : public RecoveryUnit::Change {
 public:
-    LogTransactionOperationsForShardingHandler(ServiceContext* svcCtx,
-                                               const std::vector<repl::OplogEntry>& stmts,
-                                               repl::OpTime prepareOrCommitOpTime);
-
-    LogTransactionOperationsForShardingHandler(ServiceContext* svcCtx,
+    /**
+     * Invariant: idObj should belong to a document that is part of the active chunk being migrated
+     */
+    LogTransactionOperationsForShardingHandler(const LogicalSessionId lsid,
                                                const std::vector<repl::ReplOperation>& stmts,
-                                               repl::OpTime prepareOrCommitOpTime);
+                                               const repl::OpTime& prepareOrCommitOpTime)
+        : _lsid(lsid), _stmts(stmts), _prepareOrCommitOpTime(prepareOrCommitOpTime) {}
 
     void commit(boost::optional<Timestamp>) override;
 
     void rollback() override{};
 
 private:
-    ServiceContext* _svcCtx;
-    // Use to keep BSON obj alive for the lifetime of this object.
-    std::vector<BSONObj> _ownedReplBSONObj;
+    const LogicalSessionId _lsid;
     std::vector<repl::ReplOperation> _stmts;
     const repl::OpTime _prepareOrCommitOpTime;
 };
@@ -86,7 +88,8 @@ class MigrationChunkClonerSourceLegacy final : public MigrationChunkClonerSource
     MigrationChunkClonerSourceLegacy& operator=(const MigrationChunkClonerSourceLegacy&) = delete;
 
 public:
-    MigrationChunkClonerSourceLegacy(MoveChunkRequest request,
+    MigrationChunkClonerSourceLegacy(const ShardsvrMoveRange& request,
+                                     const WriteConcernOptions& writeConcern,
                                      const BSONObj& shardKeyPattern,
                                      ConnectionString donorConnStr,
                                      HostAndPort recipientHost);
@@ -97,15 +100,14 @@ public:
                       const LogicalSessionId& lsid,
                       TxnNumber txnNumber) override;
 
-    // TODO (SERVER-44787): Remove this function after 4.4 is released.
-    Status startClone(OperationContext* opCtx) override;
-
     Status awaitUntilCriticalSectionIsAppropriate(OperationContext* opCtx,
                                                   Milliseconds maxTimeToWait) override;
 
     StatusWith<BSONObj> commitClone(OperationContext* opCtx) override;
 
-    void cancelClone(OperationContext* opCtx) override;
+    void cancelClone(OperationContext* opCtx) noexcept override;
+
+    bool isDocumentInMigratingChunk(const BSONObj& doc) override;
 
     void onInsertOp(OperationContext* opCtx,
                     const BSONObj& insertedDoc,
@@ -118,7 +120,7 @@ public:
                     const repl::OpTime& prePostImageOpTime) override;
 
     void onDeleteOp(OperationContext* opCtx,
-                    const DocumentKey& documentKey,
+                    const BSONObj& deletedDocId,
                     const repl::OpTime& opTime,
                     const repl::OpTime& preImageOpTime) override;
 
@@ -162,7 +164,7 @@ public:
      * NOTE: Must be called with the collection lock held in at least IS mode.
      */
     Status nextCloneBatch(OperationContext* opCtx,
-                          Collection* collection,
+                          const CollectionPtr& collection,
                           BSONArrayBuilder* arrBuilder);
 
     /**
@@ -172,7 +174,7 @@ public:
      *
      * NOTE: Must be called with the collection lock held in at least IS mode.
      */
-    Status nextModsBatch(OperationContext* opCtx, Database* db, BSONObjBuilder* builder);
+    Status nextModsBatch(OperationContext* opCtx, BSONObjBuilder* builder);
 
     /**
      * Appends to 'arrBuilder' oplog entries which wrote to the currently migrated chunk and contain
@@ -204,6 +206,20 @@ public:
      */
     std::shared_ptr<Notification<bool>> getNotificationForNextSessionMigrationBatch();
 
+    const NamespaceString& nss() {
+        return _args.getCommandParameter();
+    }
+
+    const BSONObj& getMin() {
+        invariant(_args.getMin());
+        return *_args.getMin();
+    }
+
+    const BSONObj& getMax() {
+        invariant(_args.getMax());
+        return *_args.getMax();
+    }
+
 private:
     friend class LogOpForShardingHandler;
     friend class LogTransactionOperationsForShardingHandler;
@@ -215,7 +231,7 @@ private:
      * Idempotent method, which cleans up any previously initialized state. It is safe to be called
      * at any time, but no methods should be called after it.
      */
-    void _cleanup(OperationContext* opCtx);
+    void _cleanup();
 
     /**
      * Synchronously invokes the recipient shard with the specified command and either returns the
@@ -225,24 +241,24 @@ private:
 
     StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> _getIndexScanExecutor(
         OperationContext* opCtx,
-        Collection* const collection,
+        const CollectionPtr& collection,
         InternalPlanner::IndexScanOptions scanOption);
 
     void _nextCloneBatchFromIndexScan(OperationContext* opCtx,
-                                      Collection* collection,
+                                      const CollectionPtr& collection,
                                       BSONArrayBuilder* arrBuilder);
 
-    void _nextCloneBatchFromCloneLocs(OperationContext* opCtx,
-                                      Collection* collection,
-                                      BSONArrayBuilder* arrBuilder);
+    void _nextCloneBatchFromCloneRecordIds(OperationContext* opCtx,
+                                           const CollectionPtr& collection,
+                                           BSONArrayBuilder* arrBuilder);
 
     /**
-     * Get the disklocs that belong to the chunk migrated and sort them in _cloneLocs (to avoid
-     * seeking disk later).
+     * Get the recordIds that belong to the chunk migrated and sort them in _cloneRecordIds (to
+     * avoid seeking disk later).
      *
      * Returns OK or any error status otherwise.
      */
-    Status _storeCurrentLocs(OperationContext* opCtx);
+    Status _storeCurrentRecordId(OperationContext* opCtx);
 
     /**
      * Adds the OpTime to the list of OpTimes for oplog entries that we should consider migrating as
@@ -262,10 +278,7 @@ private:
      * part of a part of a chunk being migrated. In doing so, this the method also removes the
      * corresponding operation track request from the operation track requests queue.
      */
-    void _addToTransferModsQueue(const BSONObj& idObj,
-                                 const char op,
-                                 const repl::OpTime& opTime,
-                                 const repl::OpTime& prePostImageOpTime);
+    void _addToTransferModsQueue(const BSONObj& idObj, char op, const repl::OpTime& opTime);
 
     /**
      * Adds an operation to the outstanding operation track requests. Returns false if the cloner
@@ -303,51 +316,17 @@ private:
      */
     void _drainAllOutstandingOperationTrackRequests(stdx::unique_lock<Latch>& lk);
 
-    /**
-     * Appends to the builder the list of _id of documents that were deleted during migration.
-     * Entries appended to the builder are removed from the list.
-     * Returns the total size of the documents that were appended + initialSize.
-     */
-    long long _xferDeletes(BSONObjBuilder* builder,
-                           std::list<BSONObj>* removeList,
-                           long long initialSize);
-
-    /**
-     * Appends to the builder the list of full documents that were modified/inserted during the
-     * migration. Entries appended to the builder are removed from the list.
-     * Returns the total size of the documents that were appended + initialSize.
-     */
-    long long _xferUpdates(OperationContext* opCtx,
-                           Database* db,
-                           BSONObjBuilder* builder,
-                           std::list<BSONObj>* updateList,
-                           long long initialSize);
-
     /**
      * Sends _recvChunkStatus to the recipient shard until it receives 'steady' from the recipient,
      * an error has occurred, or a timeout is hit.
      */
     Status _checkRecipientCloningStatus(OperationContext* opCtx, Milliseconds maxTimeToWait);
 
-    /**
-     * Inspects the pre and post image document keys and determines which xferMods bucket to
-     * add a new entry. Returns false if neither pre or post image document keys fall into
-     * the chunk boundaries being migrated.
-     */
-    bool _processUpdateForXferMod(const BSONObj& preImageDocKey, const BSONObj& postImageDocKey);
-
-    /**
-     * Defer processing of update ops into xferMods entries to when nextModsBatch is called.
-     */
-    void _deferProcessingForXferMod(const BSONObj& preImageDocKey);
-
-    /**
-     * Converts all deferred update ops captured by the op observer into xferMods entries.
-     */
-    void _processDeferredXferMods(OperationContext* opCtx, Database* database);
-
     // The original move range request
-    const MoveChunkRequest _args;
+    const ShardsvrMoveRange _args;
+
+    // The write concern associated with the move range
+    const WriteConcernOptions _writeConcern;
 
     // The shard key associated with the namespace
     const ShardKeyPattern _shardKeyPattern;
@@ -370,11 +349,11 @@ private:
     State _state{kNew};
 
     // List of record ids that needs to be transferred (initial clone)
-    std::set<RecordId> _cloneLocs;
+    std::set<RecordId> _cloneRecordIds;
 
     // The estimated average object size during the clone phase. Used for buffer size
     // pre-allocation (initial clone).
-    uint64_t _averageObjectSizeForCloneLocs{0};
+    uint64_t _averageObjectSizeForCloneRecordIds{0};
 
     // The estimated average object _id size during the clone phase.
     uint64_t _averageObjectIdSize{0};
@@ -402,13 +381,6 @@ private:
     // Amount of delete xfer mods that have not yet reached the recipient.
     size_t _untransferredDeletesCounter{0};
 
-    // Amount of ops that are yet to be converted to update/delete xferMods.
-    size_t _deferredUntransferredOpsCounter{0};
-
-    // Stores document keys of document that needs to be examined if we need to put in to xferMods
-    // list later.
-    std::vector<BSONObj> _deferredReloadOrDeletePreImageDocKeys;
-
     // Total bytes in _reload + _deleted (xfer mods)
     uint64_t _memoryUsed{0};
 
@@ -430,4 +402,14 @@ private:
     boost::optional<JumboChunkCloneState> _jumboChunkCloneState;
 };
 
+/**
+ * Appends to the builder the list of documents either deleted or modified during migration.
+ * Entries appended to the builder are removed from the list.
+ * Returns the total size of the documents that were appended + initialSize.
+ */
+long long xferMods(BSONArrayBuilder* arr,
+                   std::list<BSONObj>* modsList,
+                   long long initialSize,
+                   std::function<bool(BSONObj, BSONObj*)> extractDocToAppendFn);
+
 }  // namespace mongo
