@@ -30,16 +30,19 @@
 #include "mongo/platform/basic.h"
 
 #include "mongo/client/remote_command_targeter_mock.h"
+#include "mongo/db/catalog/create_collection.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/dbdirectclient.h"
-#include "mongo/db/logical_session_id_helpers.h"
 #include "mongo/db/namespace_string.h"
-#include "mongo/db/s/collection_metadata.h"
+#include "mongo/db/s/collection_sharding_runtime.h"
 #include "mongo/db/s/migration_chunk_cloner_source_legacy.h"
+#include "mongo/db/s/operation_sharding_state.h"
+#include "mongo/db/s/shard_server_test_fixture.h"
+#include "mongo/db/session/logical_session_id_helpers.h"
+#include "mongo/db/session/session_catalog_mongod.h"
 #include "mongo/s/catalog/sharding_catalog_client_mock.h"
 #include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/client/shard_registry.h"
-#include "mongo/s/shard_server_test_fixture.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/clock_source_mock.h"
 
@@ -50,12 +53,8 @@ using executor::RemoteCommandRequest;
 using unittest::assertGet;
 
 const NamespaceString kNss("TestDB", "TestColl");
-const BSONObj kShardKeyPattern{BSON("X" << 1)};
-const ConnectionString kConfigConnStr =
-    ConnectionString::forReplicaSet("Donor",
-                                    {HostAndPort("DonorHost1:1234"),
-                                     HostAndPort{"DonorHost2:1234"},
-                                     HostAndPort{"DonorHost3:1234"}});
+const std::string kShardKey = "X";
+const BSONObj kShardKeyPattern{BSON(kShardKey << 1)};
 const ConnectionString kDonorConnStr =
     ConnectionString::forReplicaSet("Donor",
                                     {HostAndPort("DonorHost1:1234"),
@@ -69,12 +68,16 @@ const ConnectionString kRecipientConnStr =
 
 class MigrationChunkClonerSourceLegacyTest : public ShardServerTestFixture {
 protected:
+    MigrationChunkClonerSourceLegacyTest() : ShardServerTestFixture(Options{}.useMockClock(true)) {}
+
     void setUp() override {
         ShardServerTestFixture::setUp();
 
         auto opCtx = operationContext();
         DBDirectClient client(opCtx);
         client.createCollection(NamespaceString::kSessionTransactionsTableNamespace.ns());
+        client.createIndexes(NamespaceString::kSessionTransactionsTableNamespace.ns(),
+                             {MongoDSessionCatalog::getConfigTxnPartialIndexSpec()});
 
         // TODO: SERVER-26919 set the flag on the mock repl coordinator just for the window where it
         // actually needs to bypass the op observer.
@@ -100,14 +103,6 @@ protected:
                 ->setFindHostReturnValue(kRecipientConnStr.getServers()[0]);
         }
 
-        auto clockSource = std::make_unique<ClockSourceMock>();
-
-        // Timestamps of "0 seconds" are not allowed, so we must advance our clock mock to the first
-        // real second.
-        clockSource->advance(Seconds(1));
-
-        operationContext()->getServiceContext()->setFastClockSource(std::move(clockSource));
-
         _lsid = makeLogicalSessionId(operationContext());
     }
 
@@ -132,25 +127,75 @@ protected:
         if (docs.empty())
             return;
 
-        client()->insert(kNss.ns(), docs);
-        ASSERT_EQ("", client()->getLastError());
+        auto response = client()->insertAcknowledged(kNss.ns(), docs);
+        ASSERT_OK(getStatusFromWriteCommandReply(response));
+        ASSERT_GT(response["n"].Int(), 0);
     }
 
     void deleteDocsInShardedCollection(BSONObj query) {
-        client()->remove(kNss.ns(), query);
+        auto response = client()->removeAcknowledged(kNss.ns(), query);
+        ASSERT_OK(getStatusFromWriteCommandReply(response));
+        ASSERT_GT(response["n"].Int(), 0);
     }
 
     void updateDocsInShardedCollection(BSONObj filter, BSONObj updated) {
-        client()->update(kNss.ns(), filter, updated);
+        auto response = client()->updateAcknowledged(kNss.ns(), filter, updated);
+        ASSERT_OK(getStatusFromWriteCommandReply(response));
+        ASSERT_GT(response["n"].Int(), 0);
     }
 
     /**
-     * Creates a collection, which contains an index corresponding to kShardKeyPattern and insers
+     * Creates a collection, which contains an index corresponding to kShardKeyPattern and inserts
      * the specified initial documents.
      */
     void createShardedCollection(const std::vector<BSONObj>& initialDocs) {
-        ASSERT(_client->createCollection(kNss.ns()));
-        _client->createIndex(kNss.ns(), kShardKeyPattern);
+        {
+            OperationShardingState::ScopedAllowImplicitCollectionCreate_UNSAFE
+                unsafeCreateCollection(operationContext());
+            uassertStatusOK(
+                createCollection(operationContext(), kNss.dbName(), BSON("create" << kNss.coll())));
+        }
+
+        const auto uuid = [&] {
+            AutoGetCollection autoColl(operationContext(), kNss, MODE_IX);
+            return autoColl.getCollection()->uuid();
+        }();
+
+        [&] {
+            const OID epoch = OID::gen();
+            const Timestamp timestamp(1);
+
+            auto rt = RoutingTableHistory::makeNew(
+                kNss,
+                uuid,
+                kShardKeyPattern,
+                nullptr,
+                false,
+                epoch,
+                timestamp,
+                boost::none /* timeseriesFields */,
+                boost::none /* resharding Fields */,
+                boost::none /* chunkSizeBytes */,
+                true,
+                {ChunkType{uuid,
+                           ChunkRange{BSON(kShardKey << MINKEY), BSON(kShardKey << MAXKEY)},
+                           ChunkVersion({epoch, timestamp}, {1, 0}),
+                           ShardId("dummyShardId")}});
+
+            AutoGetDb autoDb(operationContext(), kNss.dbName(), MODE_IX);
+            Lock::CollectionLock collLock(operationContext(), kNss, MODE_IX);
+            CollectionShardingRuntime::get(operationContext(), kNss)
+                ->setFilteringMetadata(
+                    operationContext(),
+                    CollectionMetadata(
+                        ChunkManager(ShardId("dummyShardId"),
+                                     DatabaseVersion(UUID::gen(), Timestamp(1, 1)),
+                                     makeStandaloneRoutingTableHistory(std::move(rt)),
+                                     boost::none),
+                        ShardId("dummyShardId")));
+        }();
+
+        client()->createIndex(kNss.ns(), kShardKeyPattern);
         insertDocsInShardedCollection(initialDocs);
     }
 
@@ -158,22 +203,15 @@ protected:
      * Shortcut to create BSON represenation of a moveChunk request for the specified range with
      * fixed kDonorConnStr and kRecipientConnStr, respectively.
      */
-    static MoveChunkRequest createMoveChunkRequest(const ChunkRange& chunkRange) {
-        BSONObjBuilder cmdBuilder;
-        MoveChunkRequest::appendAsCommand(
-            &cmdBuilder,
-            kNss,
-            ChunkVersion(1, 0, OID::gen()),
-            kConfigConnStr,
-            kDonorConnStr.getSetName(),
-            kRecipientConnStr.getSetName(),
-            chunkRange,
-            1024 * 1024,
-            MigrationSecondaryThrottleOptions::create(MigrationSecondaryThrottleOptions::kDefault),
-            false,
-            MoveChunkRequest::ForceJumbo::kDoNotForce);
-
-        return assertGet(MoveChunkRequest::createFromCommand(kNss, cmdBuilder.obj()));
+    static ShardsvrMoveRange createMoveRangeRequest(const ChunkRange& chunkRange) {
+        ShardsvrMoveRange req(kNss);
+        req.setEpoch(OID::gen());
+        req.setFromShard(ShardId(kDonorConnStr.getSetName()));
+        req.setMaxChunkSizeBytes(1024);
+        req.getMoveRangeRequestBase().setToShard(ShardId(kRecipientConnStr.getSetName()));
+        req.getMoveRangeRequestBase().setMin(chunkRange.getMin());
+        req.getMoveRangeRequestBase().setMax(chunkRange.getMax());
+        return req;
     }
 
     /**
@@ -203,11 +241,10 @@ protected:
     TxnNumber _txnNumber{0};
 
 private:
-    std::unique_ptr<ShardingCatalogClient> makeShardingCatalogClient(
-        std::unique_ptr<DistLockManager> distLockManager) override {
+    std::unique_ptr<ShardingCatalogClient> makeShardingCatalogClient() override {
         class StaticCatalogClient final : public ShardingCatalogClientMock {
         public:
-            StaticCatalogClient() : ShardingCatalogClientMock(nullptr) {}
+            StaticCatalogClient() = default;
 
             StatusWith<repl::OpTimeWith<std::vector<ShardType>>> getAllShards(
                 OperationContext* opCtx, repl::ReadConcernLevel readConcern) override {
@@ -235,15 +272,16 @@ TEST_F(MigrationChunkClonerSourceLegacyTest, CorrectDocumentsFetched) {
                                            createCollectionDocument(100),
                                            createCollectionDocument(199),
                                            createCollectionDocument(200)};
-    const ShardKeyPattern shardKeyPattern(kShardKeyPattern);
 
     createShardedCollection(contents);
 
-    MigrationChunkClonerSourceLegacy cloner(
-        createMoveChunkRequest(ChunkRange(BSON("X" << 100), BSON("X" << 200))),
-        kShardKeyPattern,
-        kDonorConnStr,
-        kRecipientConnStr.getServers()[0]);
+    const ShardsvrMoveRange req =
+        createMoveRangeRequest(ChunkRange(BSON("X" << 100), BSON("X" << 200)));
+    MigrationChunkClonerSourceLegacy cloner(req,
+                                            WriteConcernOptions(),
+                                            kShardKeyPattern,
+                                            kDonorConnStr,
+                                            kRecipientConnStr.getServers()[0]);
 
     {
         auto futureStartClone = launchAsync([&]() {
@@ -298,21 +336,9 @@ TEST_F(MigrationChunkClonerSourceLegacyTest, CorrectDocumentsFetched) {
         cloner.onInsertOp(operationContext(), createCollectionDocument(151), {});
         cloner.onInsertOp(operationContext(), createCollectionDocument(210), {});
 
-        cloner.onDeleteOp(
-            operationContext(),
-            CollectionMetadata::extractDocumentKey(&shardKeyPattern, createCollectionDocument(80)),
-            {},
-            {});
-        cloner.onDeleteOp(
-            operationContext(),
-            CollectionMetadata::extractDocumentKey(&shardKeyPattern, createCollectionDocument(199)),
-            {},
-            {});
-        cloner.onDeleteOp(
-            operationContext(),
-            CollectionMetadata::extractDocumentKey(&shardKeyPattern, createCollectionDocument(220)),
-            {},
-            {});
+        cloner.onDeleteOp(operationContext(), createCollectionDocument(80), {}, {});
+        cloner.onDeleteOp(operationContext(), createCollectionDocument(199), {}, {});
+        cloner.onDeleteOp(operationContext(), createCollectionDocument(220), {}, {});
 
         wuow.commit();
     }
@@ -329,15 +355,254 @@ TEST_F(MigrationChunkClonerSourceLegacyTest, CorrectDocumentsFetched) {
 
         {
             BSONObjBuilder modsBuilder;
-            ASSERT_OK(cloner.nextModsBatch(operationContext(), autoColl.getDb(), &modsBuilder));
+            ASSERT_OK(cloner.nextModsBatch(operationContext(), &modsBuilder));
 
             const auto modsObj = modsBuilder.obj();
             ASSERT_EQ(2U, modsObj["reload"].Array().size());
             ASSERT_BSONOBJ_EQ(createCollectionDocument(150), modsObj["reload"].Array()[0].Obj());
             ASSERT_BSONOBJ_EQ(createCollectionDocument(151), modsObj["reload"].Array()[1].Obj());
 
+            // The legacy chunk cloner cannot filter out deletes because we don't preserve the shard
+            // key on delete
+            ASSERT_EQ(3U, modsObj["deleted"].Array().size());
+            ASSERT_BSONOBJ_EQ(BSON("_id" << 80), modsObj["deleted"].Array()[0].Obj());
+            ASSERT_BSONOBJ_EQ(BSON("_id" << 199), modsObj["deleted"].Array()[1].Obj());
+            ASSERT_BSONOBJ_EQ(BSON("_id" << 220), modsObj["deleted"].Array()[2].Obj());
+        }
+    }
+
+    auto futureCommit = launchAsync([&]() {
+        onCommand([&](const RemoteCommandRequest& request) { return BSON("ok" << true); });
+    });
+
+    ASSERT_OK(cloner.commitClone(operationContext()));
+    futureCommit.default_timed_get();
+}
+
+
+TEST_F(MigrationChunkClonerSourceLegacyTest, RemoveDuplicateDocuments) {
+    const std::vector<BSONObj> contents = {createCollectionDocument(100),
+                                           createCollectionDocument(199)};
+
+    createShardedCollection(contents);
+
+    const ShardsvrMoveRange req =
+        createMoveRangeRequest(ChunkRange(BSON("X" << 100), BSON("X" << 200)));
+    MigrationChunkClonerSourceLegacy cloner(req,
+                                            WriteConcernOptions(),
+                                            kShardKeyPattern,
+                                            kDonorConnStr,
+                                            kRecipientConnStr.getServers()[0]);
+
+    {
+        auto futureStartClone = launchAsync([&]() {
+            onCommand([&](const RemoteCommandRequest& request) { return BSON("ok" << true); });
+        });
+
+        ASSERT_OK(cloner.startClone(operationContext(), UUID::gen(), _lsid, _txnNumber));
+        futureStartClone.default_timed_get();
+    }
+
+    // Ensure the initial clone documents are available
+    {
+        AutoGetCollection autoColl(operationContext(), kNss, MODE_IS);
+
+        {
+            BSONArrayBuilder arrBuilder;
+            ASSERT_OK(
+                cloner.nextCloneBatch(operationContext(), autoColl.getCollection(), &arrBuilder));
+            ASSERT_EQ(2, arrBuilder.arrSize());
+
+            const auto arr = arrBuilder.arr();
+            ASSERT_BSONOBJ_EQ(contents[0], arr[0].Obj());
+            ASSERT_BSONOBJ_EQ(contents[1], arr[1].Obj());
+        }
+    }
+
+    {
+        AutoGetCollection autoColl(operationContext(), kNss, MODE_IX);
+
+        deleteDocsInShardedCollection(createCollectionDocument(100));
+        insertDocsInShardedCollection({createCollectionDocument(100)});
+        deleteDocsInShardedCollection(createCollectionDocument(100));
+
+        updateDocsInShardedCollection(createCollectionDocument(199),
+                                      createCollectionDocumentForUpdate(199, 198));
+        updateDocsInShardedCollection(createCollectionDocumentForUpdate(199, 198),
+                                      createCollectionDocumentForUpdate(199, 197));
+
+        WriteUnitOfWork wuow(operationContext());
+
+        cloner.onDeleteOp(operationContext(), createCollectionDocument(100), {}, {});
+        cloner.onInsertOp(operationContext(), createCollectionDocument(100), {});
+        cloner.onDeleteOp(operationContext(), createCollectionDocument(100), {}, {});
+
+        cloner.onUpdateOp(operationContext(),
+                          createCollectionDocument(199),
+                          createCollectionDocumentForUpdate(199, 198),
+                          {},
+                          {});
+        cloner.onUpdateOp(operationContext(),
+                          createCollectionDocument(199),
+                          createCollectionDocumentForUpdate(199, 197),
+                          {},
+                          {});
+
+        wuow.commit();
+    }
+
+    {
+        AutoGetCollection autoColl(operationContext(), kNss, MODE_IS);
+        {
+            BSONObjBuilder modsBuilder;
+            ASSERT_OK(cloner.nextModsBatch(operationContext(), &modsBuilder));
+
+            const auto modsObj = modsBuilder.obj();
+            ASSERT_EQ(1U, modsObj["reload"].Array().size());
+            ASSERT_BSONOBJ_EQ(createCollectionDocumentForUpdate(199, 197),
+                              modsObj["reload"].Array()[0].Obj());
             ASSERT_EQ(1U, modsObj["deleted"].Array().size());
-            ASSERT_BSONOBJ_EQ(BSON("_id" << 199), modsObj["deleted"].Array()[0].Obj());
+            ASSERT_BSONOBJ_EQ(BSON("_id" << 100), modsObj["deleted"].Array()[0].Obj());
+        }
+    }
+
+    auto futureCommit = launchAsync([&]() {
+        onCommand([&](const RemoteCommandRequest& request) { return BSON("ok" << true); });
+    });
+
+    ASSERT_OK(cloner.commitClone(operationContext()));
+    futureCommit.default_timed_get();
+}
+
+
+TEST_F(MigrationChunkClonerSourceLegacyTest, OneLargeDocumentTransferMods) {
+    const std::vector<BSONObj> contents = {createCollectionDocument(1)};
+
+    createShardedCollection(contents);
+
+    const ShardsvrMoveRange req =
+        createMoveRangeRequest(ChunkRange(BSON("X" << 1), BSON("X" << 100)));
+    MigrationChunkClonerSourceLegacy cloner(req,
+                                            WriteConcernOptions(),
+                                            kShardKeyPattern,
+                                            kDonorConnStr,
+                                            kRecipientConnStr.getServers()[0]);
+
+    {
+        auto futureStartClone = launchAsync([&]() {
+            onCommand([&](const RemoteCommandRequest& request) { return BSON("ok" << true); });
+        });
+
+        ASSERT_OK(cloner.startClone(operationContext(), UUID::gen(), _lsid, _txnNumber));
+        futureStartClone.default_timed_get();
+    }
+
+    {
+        AutoGetCollection autoColl(operationContext(), kNss, MODE_IS);
+
+        {
+            BSONArrayBuilder arrBuilder;
+            ASSERT_OK(
+                cloner.nextCloneBatch(operationContext(), autoColl.getCollection(), &arrBuilder));
+            ASSERT_EQ(1, arrBuilder.arrSize());
+        }
+    }
+
+    {
+        AutoGetCollection autoColl(operationContext(), kNss, MODE_IX);
+        BSONObj insertDoc =
+            createSizedCollectionDocument(2, BSONObjMaxUserSize - kFixedCommandOverhead + 2 * 1024);
+        insertDocsInShardedCollection({insertDoc});
+        WriteUnitOfWork wuow(operationContext());
+        cloner.onInsertOp(operationContext(), insertDoc, {});
+        wuow.commit();
+    }
+
+    {
+        AutoGetCollection autoColl(operationContext(), kNss, MODE_IS);
+        {
+            BSONObjBuilder modsBuilder;
+            ASSERT_OK(cloner.nextModsBatch(operationContext(), &modsBuilder));
+
+            const auto modsObj = modsBuilder.obj();
+            ASSERT_EQ(1, modsObj["reload"].Array().size());
+        }
+    }
+
+    auto futureCommit = launchAsync([&]() {
+        onCommand([&](const RemoteCommandRequest& request) { return BSON("ok" << true); });
+    });
+
+    ASSERT_OK(cloner.commitClone(operationContext()));
+    futureCommit.default_timed_get();
+}
+
+TEST_F(MigrationChunkClonerSourceLegacyTest, ManySmallDocumentsTransferMods) {
+    const std::vector<BSONObj> contents = {createCollectionDocument(1)};
+
+    createShardedCollection(contents);
+
+    const ShardsvrMoveRange req =
+        createMoveRangeRequest(ChunkRange(BSON("X" << 1), BSON("X" << 1000000)));
+    MigrationChunkClonerSourceLegacy cloner(req,
+                                            WriteConcernOptions(),
+                                            kShardKeyPattern,
+                                            kDonorConnStr,
+                                            kRecipientConnStr.getServers()[0]);
+
+    {
+        auto futureStartClone = launchAsync([&]() {
+            onCommand([&](const RemoteCommandRequest& request) { return BSON("ok" << true); });
+        });
+
+        ASSERT_OK(cloner.startClone(operationContext(), UUID::gen(), _lsid, _txnNumber));
+        futureStartClone.default_timed_get();
+    }
+
+    {
+        AutoGetCollection autoColl(operationContext(), kNss, MODE_IS);
+
+        {
+            BSONArrayBuilder arrBuilder;
+            ASSERT_OK(
+                cloner.nextCloneBatch(operationContext(), autoColl.getCollection(), &arrBuilder));
+            ASSERT_EQ(1, arrBuilder.arrSize());
+        }
+    }
+
+    long long numDocuments = 0;
+    {
+        AutoGetCollection autoColl(operationContext(), kNss, MODE_IX);
+
+        std::vector<BSONObj> insertDocs;
+        long long totalSize = 0;
+        long long id = 2;
+        while (true) {
+            BSONObj add = createSizedCollectionDocument(id++, 4 * 1024);
+            // The overhead for a BSONObjBuilder with 4KB documents is ~ 22 * 1024, so this is the
+            // max documents to fit in one batch
+            if (totalSize + add.objsize() > BSONObjMaxUserSize - kFixedCommandOverhead - 22 * 1024)
+                break;
+            insertDocs.push_back(add);
+            totalSize += add.objsize();
+            numDocuments++;
+            insertDocsInShardedCollection({add});
+        }
+
+        WriteUnitOfWork wuow(operationContext());
+        for (BSONObj add : insertDocs) {
+            cloner.onInsertOp(operationContext(), add, {});
+        }
+        wuow.commit();
+    }
+
+    {
+        AutoGetCollection autoColl(operationContext(), kNss, MODE_IS);
+        {
+            BSONObjBuilder modsBuilder;
+            ASSERT_OK(cloner.nextModsBatch(operationContext(), &modsBuilder));
+            const auto modsObj = modsBuilder.obj();
+            ASSERT_EQ(modsObj["reload"].Array().size(), numDocuments);
         }
     }
 
@@ -350,24 +615,33 @@ TEST_F(MigrationChunkClonerSourceLegacyTest, CorrectDocumentsFetched) {
 }
 
 TEST_F(MigrationChunkClonerSourceLegacyTest, CollectionNotFound) {
-    MigrationChunkClonerSourceLegacy cloner(
-        createMoveChunkRequest(ChunkRange(BSON("X" << 100), BSON("X" << 200))),
-        kShardKeyPattern,
-        kDonorConnStr,
-        kRecipientConnStr.getServers()[0]);
+    const ShardsvrMoveRange req =
+        createMoveRangeRequest(ChunkRange(BSON("X" << 100), BSON("X" << 200)));
+    MigrationChunkClonerSourceLegacy cloner(req,
+                                            WriteConcernOptions(),
+                                            kShardKeyPattern,
+                                            kDonorConnStr,
+                                            kRecipientConnStr.getServers()[0]);
 
     ASSERT_NOT_OK(cloner.startClone(operationContext(), UUID::gen(), _lsid, _txnNumber));
     cloner.cancelClone(operationContext());
 }
 
 TEST_F(MigrationChunkClonerSourceLegacyTest, ShardKeyIndexNotFound) {
-    ASSERT(client()->createCollection(kNss.ns()));
+    {
+        OperationShardingState::ScopedAllowImplicitCollectionCreate_UNSAFE unsafeCreateCollection(
+            operationContext());
+        uassertStatusOK(
+            createCollection(operationContext(), kNss.dbName(), BSON("create" << kNss.coll())));
+    }
 
-    MigrationChunkClonerSourceLegacy cloner(
-        createMoveChunkRequest(ChunkRange(BSON("X" << 100), BSON("X" << 200))),
-        kShardKeyPattern,
-        kDonorConnStr,
-        kRecipientConnStr.getServers()[0]);
+    const ShardsvrMoveRange req =
+        createMoveRangeRequest(ChunkRange(BSON("X" << 100), BSON("X" << 200)));
+    MigrationChunkClonerSourceLegacy cloner(req,
+                                            WriteConcernOptions(),
+                                            kShardKeyPattern,
+                                            kDonorConnStr,
+                                            kRecipientConnStr.getServers()[0]);
 
     ASSERT_NOT_OK(cloner.startClone(operationContext(), UUID::gen(), _lsid, _txnNumber));
     cloner.cancelClone(operationContext());
@@ -381,11 +655,13 @@ TEST_F(MigrationChunkClonerSourceLegacyTest, FailedToEngageRecipientShard) {
 
     createShardedCollection(contents);
 
-    MigrationChunkClonerSourceLegacy cloner(
-        createMoveChunkRequest(ChunkRange(BSON("X" << 100), BSON("X" << 200))),
-        kShardKeyPattern,
-        kDonorConnStr,
-        kRecipientConnStr.getServers()[0]);
+    const ShardsvrMoveRange req =
+        createMoveRangeRequest(ChunkRange(BSON("X" << 100), BSON("X" << 200)));
+    MigrationChunkClonerSourceLegacy cloner(req,
+                                            WriteConcernOptions(),
+                                            kShardKeyPattern,
+                                            kDonorConnStr,
+                                            kRecipientConnStr.getServers()[0]);
 
     {
         auto futureStartClone = launchAsync([&]() {
@@ -425,341 +701,5 @@ TEST_F(MigrationChunkClonerSourceLegacyTest, FailedToEngageRecipientShard) {
     cloner.cancelClone(operationContext());
 }
 
-TEST_F(MigrationChunkClonerSourceLegacyTest, CorrectDocumentsFetchedWithDottedShardKeyPattern) {
-    const ShardKeyPattern dottedShardKeyPattern(BSON("x.a" << 1 << "x.b" << 1));
-
-    auto createDoc =
-        ([](int val) { return BSON("_id" << val << "x" << BSON("a" << val << "b" << val)); });
-
-    const auto req = createMoveChunkRequest(
-        ChunkRange(BSON("x.a" << 100 << "x.b" << 100), BSON("x.a" << 200 << "x.b" << 200)));
-    MigrationChunkClonerSourceLegacy cloner(
-        req, dottedShardKeyPattern.toBSON(), kDonorConnStr, kRecipientConnStr.getServers()[0]);
-
-    // Insert some documents in the chunk range to be included for migration
-    insertDocsInShardedCollection({createDoc(150)});
-    insertDocsInShardedCollection({createDoc(151)});
-
-    // Insert some documents which are outside of the chunk range and should not be included for
-    // migration
-    insertDocsInShardedCollection({createDoc(90)});
-    insertDocsInShardedCollection({createDoc(210)});
-
-    {
-        AutoGetCollection autoColl(operationContext(), kNss, MODE_IX);
-
-        WriteUnitOfWork wuow(operationContext());
-
-        cloner.onInsertOp(operationContext(), createDoc(90), {});
-        cloner.onInsertOp(operationContext(), createDoc(150), {});
-        cloner.onInsertOp(operationContext(), createDoc(151), {});
-        cloner.onInsertOp(operationContext(), createDoc(210), {});
-
-        cloner.onDeleteOp(
-            operationContext(),
-            CollectionMetadata::extractDocumentKey(&dottedShardKeyPattern, createDoc(80)),
-            {},
-            {});
-        cloner.onDeleteOp(
-            operationContext(),
-            CollectionMetadata::extractDocumentKey(&dottedShardKeyPattern, createDoc(199)),
-            {},
-            {});
-        cloner.onDeleteOp(
-            operationContext(),
-            CollectionMetadata::extractDocumentKey(&dottedShardKeyPattern, createDoc(220)),
-            {},
-            {});
-
-        wuow.commit();
-    }
-
-    {
-        AutoGetCollection autoColl(operationContext(), kNss, MODE_IS);
-
-        {
-            BSONArrayBuilder arrBuilder;
-            ASSERT_OK(
-                cloner.nextCloneBatch(operationContext(), autoColl.getCollection(), &arrBuilder));
-            ASSERT_EQ(0, arrBuilder.arrSize());
-        }
-
-        {
-            BSONObjBuilder modsBuilder;
-            ASSERT_OK(cloner.nextModsBatch(operationContext(), autoColl.getDb(), &modsBuilder));
-
-            const auto modsObj = modsBuilder.obj();
-            ASSERT_EQ(2U, modsObj["reload"].Array().size());
-            ASSERT_BSONOBJ_EQ(createDoc(150), modsObj["reload"].Array()[0].Obj());
-            ASSERT_BSONOBJ_EQ(createDoc(151), modsObj["reload"].Array()[1].Obj());
-
-            ASSERT_EQ(1U, modsObj["deleted"].Array().size());
-            ASSERT_BSONOBJ_EQ(BSON("_id" << 199), modsObj["deleted"].Array()[0].Obj());
-        }
-    }
-
-    cloner.cancelClone(operationContext());
-}
-
-TEST_F(MigrationChunkClonerSourceLegacyTest, CorrectDocumentsFetchedWithHasheddShardKeyPattern) {
-    const ShardKeyPattern hashedShardKeyPattern(BSON("X"
-                                                     << "hashed"));
-
-    const auto req = createMoveChunkRequest(
-        ChunkRange(BSON("X" << 6000000000000000000ll), BSON("X" << 9003000000000000000ll)));
-    MigrationChunkClonerSourceLegacy cloner(
-        req, hashedShardKeyPattern.toBSON(), kDonorConnStr, kRecipientConnStr.getServers()[0]);
-
-    // Insert some documents in the chunk range to be included for migration
-    insertDocsInShardedCollection({createCollectionDocument(150)});
-    insertDocsInShardedCollection({createCollectionDocument(151)});
-
-    // Insert some documents which are outside of the chunk range and should not be included for
-    // migration
-    insertDocsInShardedCollection({createCollectionDocument(90)});
-    insertDocsInShardedCollection({createCollectionDocument(210)});
-
-    {
-        AutoGetCollection autoColl(operationContext(), kNss, MODE_IX);
-
-        WriteUnitOfWork wuow(operationContext());
-
-        cloner.onInsertOp(
-            operationContext(), createCollectionDocument(90), {});  // hashed = 1348713528393582036
-        cloner.onInsertOp(
-            operationContext(), createCollectionDocument(150), {});  // hashed = 9002984030040611364
-        cloner.onInsertOp(
-            operationContext(), createCollectionDocument(151), {});  // hashed = 6186237390842619770
-        cloner.onInsertOp(
-            operationContext(), createCollectionDocument(210), {});  // hashed = 4420792252088815836
-
-        cloner.onDeleteOp(operationContext(),
-                          CollectionMetadata::extractDocumentKey(&hashedShardKeyPattern,
-                                                                 createCollectionDocument(80)),
-                          {},
-                          {});  // hashed = 6910253216116676730
-        cloner.onDeleteOp(operationContext(),
-                          CollectionMetadata::extractDocumentKey(&hashedShardKeyPattern,
-                                                                 createCollectionDocument(199)),
-                          {},
-                          {});  // hashed = 3000073935277689405
-        cloner.onDeleteOp(operationContext(),
-                          CollectionMetadata::extractDocumentKey(&hashedShardKeyPattern,
-                                                                 createCollectionDocument(220)),
-                          {},
-                          {});  // hashed = -6432749148213749320
-
-        wuow.commit();
-    }
-
-    {
-        AutoGetCollection autoColl(operationContext(), kNss, MODE_IS);
-
-        {
-            BSONArrayBuilder arrBuilder;
-            ASSERT_OK(
-                cloner.nextCloneBatch(operationContext(), autoColl.getCollection(), &arrBuilder));
-            ASSERT_EQ(0, arrBuilder.arrSize());
-        }
-
-        {
-            BSONObjBuilder modsBuilder;
-            ASSERT_OK(cloner.nextModsBatch(operationContext(), autoColl.getDb(), &modsBuilder));
-
-            const auto modsObj = modsBuilder.obj();
-            ASSERT_EQ(2U, modsObj["reload"].Array().size());
-            ASSERT_BSONOBJ_EQ(createCollectionDocument(150), modsObj["reload"].Array()[0].Obj());
-            ASSERT_BSONOBJ_EQ(createCollectionDocument(151), modsObj["reload"].Array()[1].Obj());
-
-            ASSERT_EQ(1U, modsObj["deleted"].Array().size());
-            ASSERT_BSONOBJ_EQ(BSON("_id" << 80), modsObj["deleted"].Array()[0].Obj());
-        }
-    }
-
-    cloner.cancelClone(operationContext());
-}
-
-TEST_F(MigrationChunkClonerSourceLegacyTest, UpdatedDocumentsFetched) {
-    const ShardKeyPattern shardKeyPattern(kShardKeyPattern);
-
-    const auto req = createMoveChunkRequest(ChunkRange(BSON("X" << 100), BSON("X" << 200)));
-    MigrationChunkClonerSourceLegacy cloner(
-        req, kShardKeyPattern, kDonorConnStr, kRecipientConnStr.getServers()[0]);
-
-    // Insert some documents in the chunk range to be included for migration
-    insertDocsInShardedCollection({createCollectionDocument(150)});
-    insertDocsInShardedCollection({createCollectionDocument(151)});
-
-    {
-        AutoGetCollection autoColl(operationContext(), kNss, MODE_IX);
-
-        WriteUnitOfWork wuow(operationContext());
-
-        cloner.onUpdateOp(operationContext(), boost::none, createCollectionDocument(150), {}, {});
-        cloner.onUpdateOp(operationContext(),
-                          createCollectionDocument(80),
-                          createCollectionDocument(151),
-                          {},
-                          {});
-
-        // From in doc in chunk range to outside of range will be converted to a delete xferMods.
-        cloner.onUpdateOp(operationContext(),
-                          createCollectionDocument(199),
-                          createCollectionDocument(90),
-                          {},
-                          {});
-
-        wuow.commit();
-    }
-
-    {
-        AutoGetCollection autoColl(operationContext(), kNss, MODE_IS);
-
-        {
-            BSONArrayBuilder arrBuilder;
-            ASSERT_OK(
-                cloner.nextCloneBatch(operationContext(), autoColl.getCollection(), &arrBuilder));
-            ASSERT_EQ(0, arrBuilder.arrSize());
-        }
-
-        {
-            BSONObjBuilder modsBuilder;
-            ASSERT_OK(cloner.nextModsBatch(operationContext(), autoColl.getDb(), &modsBuilder));
-
-            const auto modsObj = modsBuilder.obj();
-            ASSERT_EQ(2U, modsObj["reload"].Array().size());
-            ASSERT_BSONOBJ_EQ(createCollectionDocument(150), modsObj["reload"].Array()[0].Obj());
-            ASSERT_BSONOBJ_EQ(createCollectionDocument(151), modsObj["reload"].Array()[1].Obj());
-
-            ASSERT_EQ(1U, modsObj["deleted"].Array().size());
-            ASSERT_BSONOBJ_EQ(BSON("_id" << 199), modsObj["deleted"].Array()[0].Obj());
-        }
-    }
-
-    cloner.cancelClone(operationContext());
-}
-
-TEST_F(MigrationChunkClonerSourceLegacyTest, UpdatedDocumentsFetchedWithHashedShardKey) {
-    const ShardKeyPattern shardKeyPattern(BSON("X"
-                                               << "hashed"));
-
-    const auto req = createMoveChunkRequest(
-        ChunkRange(BSON("X" << 6000000000000000000ll), BSON("X" << 9003000000000000000ll)));
-    MigrationChunkClonerSourceLegacy cloner(
-        req, shardKeyPattern.toBSON(), kDonorConnStr, kRecipientConnStr.getServers()[0]);
-
-    // Insert some documents in the chunk range to be included for migration
-    insertDocsInShardedCollection({createCollectionDocument(150)});
-    insertDocsInShardedCollection({createCollectionDocument(151)});
-
-    {
-        AutoGetCollection autoColl(operationContext(), kNss, MODE_IX);
-
-        WriteUnitOfWork wuow(operationContext());
-
-        cloner.onUpdateOp(operationContext(),
-                          boost::none,
-                          createCollectionDocument(150),
-                          {},
-                          {});  // hashed = 9002984030040611364
-        cloner.onUpdateOp(operationContext(),
-                          createCollectionDocument(90),
-                          createCollectionDocument(151),
-                          {},
-                          {});  // hashed = 1348713528393582036 -> 6186237390842619770
-
-        // From in doc in chunk range to outside of range will be converted to a delete xferMods.
-        cloner.onUpdateOp(operationContext(),
-                          createCollectionDocument(80),
-                          createCollectionDocument(199),
-                          {},
-                          {});  // hashed = 6910253216116676730 -> 3000073935277689405
-
-        wuow.commit();
-    }
-
-    {
-        AutoGetCollection autoColl(operationContext(), kNss, MODE_IS);
-
-        {
-            BSONArrayBuilder arrBuilder;
-            ASSERT_OK(
-                cloner.nextCloneBatch(operationContext(), autoColl.getCollection(), &arrBuilder));
-            ASSERT_EQ(0, arrBuilder.arrSize());
-        }
-
-        {
-            BSONObjBuilder modsBuilder;
-            ASSERT_OK(cloner.nextModsBatch(operationContext(), autoColl.getDb(), &modsBuilder));
-
-            const auto modsObj = modsBuilder.obj();
-            ASSERT_EQ(2U, modsObj["reload"].Array().size());
-            ASSERT_BSONOBJ_EQ(createCollectionDocument(150), modsObj["reload"].Array()[0].Obj());
-            ASSERT_BSONOBJ_EQ(createCollectionDocument(151), modsObj["reload"].Array()[1].Obj());
-
-            ASSERT_EQ(1U, modsObj["deleted"].Array().size());
-            ASSERT_BSONOBJ_EQ(BSON("_id" << 80), modsObj["deleted"].Array()[0].Obj());
-        }
-    }
-
-    cloner.cancelClone(operationContext());
-}
-
-TEST_F(MigrationChunkClonerSourceLegacyTest, UpdatedDocumentsFetchedWithDottedShardKeyPattern) {
-    const ShardKeyPattern dottedShardKeyPattern(BSON("x.a" << 1 << "x.b" << 1));
-
-    auto createDoc =
-        ([](int val) { return BSON("_id" << val << "x" << BSON("a" << val << "b" << val)); });
-
-    const auto req = createMoveChunkRequest(
-        ChunkRange(BSON("x.a" << 100 << "x.b" << 100), BSON("x.a" << 200 << "x.b" << 200)));
-    MigrationChunkClonerSourceLegacy cloner(
-        req, dottedShardKeyPattern.toBSON(), kDonorConnStr, kRecipientConnStr.getServers()[0]);
-
-    // Insert some documents in the chunk range to be included for migration
-    insertDocsInShardedCollection({createDoc(150)});
-    insertDocsInShardedCollection({createDoc(151)});
-
-    {
-        AutoGetCollection autoColl(operationContext(), kNss, MODE_IX);
-
-        WriteUnitOfWork wuow(operationContext());
-
-        cloner.onUpdateOp(operationContext(), boost::none, createDoc(150), {}, {});
-        cloner.onUpdateOp(operationContext(), createDoc(80), createDoc(151), {}, {});
-
-        // From in doc in chunk range to outside of range will be converted to a delete xferMods.
-        cloner.onUpdateOp(operationContext(), createDoc(199), createDoc(90), {}, {});
-
-        wuow.commit();
-    }
-
-    {
-        AutoGetCollection autoColl(operationContext(), kNss, MODE_IS);
-
-        {
-            BSONArrayBuilder arrBuilder;
-            ASSERT_OK(
-                cloner.nextCloneBatch(operationContext(), autoColl.getCollection(), &arrBuilder));
-            ASSERT_EQ(0, arrBuilder.arrSize());
-        }
-
-        {
-            BSONObjBuilder modsBuilder;
-            ASSERT_OK(cloner.nextModsBatch(operationContext(), autoColl.getDb(), &modsBuilder));
-
-            const auto modsObj = modsBuilder.obj();
-            ASSERT_EQ(2U, modsObj["reload"].Array().size());
-            ASSERT_BSONOBJ_EQ(createDoc(150), modsObj["reload"].Array()[0].Obj());
-            ASSERT_BSONOBJ_EQ(createDoc(151), modsObj["reload"].Array()[1].Obj());
-
-            ASSERT_EQ(1U, modsObj["deleted"].Array().size());
-            ASSERT_BSONOBJ_EQ(BSON("_id" << 199), modsObj["deleted"].Array()[0].Obj());
-        }
-    }
-
-    cloner.cancelClone(operationContext());
-}
-
 }  // namespace
 }  // namespace mongo
