@@ -30,22 +30,21 @@
 #include "mongo/client/remote_command_targeter_factory_mock.h"
 #include "mongo/client/remote_command_targeter_mock.h"
 #include "mongo/db/catalog_raii.h"
-#include "mongo/db/logical_clock.h"
-#include "mongo/db/s/catalog_cache_loader_mock.h"
+#include "mongo/db/persistent_task_store.h"
+#include "mongo/db/repl/wait_for_majority_service.h"
 #include "mongo/db/s/collection_sharding_runtime.h"
-#include "mongo/db/s/collection_sharding_state.h"
+#include "mongo/db/s/collection_sharding_runtime_test.cpp"
 #include "mongo/db/s/migration_util.h"
-#include "mongo/db/s/persistent_task_store.h"
+#include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/s/shard_filtering_metadata_refresh.h"
 #include "mongo/db/s/shard_server_catalog_cache_loader.h"
+#include "mongo/db/s/shard_server_test_fixture.h"
 #include "mongo/db/s/sharding_state.h"
-#include "mongo/db/s/wait_for_majority_service.h"
+#include "mongo/db/vector_clock.h"
 #include "mongo/s/catalog/sharding_catalog_client_mock.h"
 #include "mongo/s/catalog/type_shard.h"
-#include "mongo/s/database_version_helpers.h"
-#include "mongo/s/shard_server_test_fixture.h"
-#include "mongo/unittest/barrier.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/s/catalog_cache_loader_mock.h"
+#include "mongo/s/database_version.h"
 #include "mongo/util/future.h"
 
 namespace mongo {
@@ -61,14 +60,6 @@ UUID getCollectionUuid(OperationContext* opCtx, const NamespaceString& nss) {
     return autoColl.getCollection()->uuid();
 }
 
-void addRangeToReceivingChunks(OperationContext* opCtx,
-                               const NamespaceString& nss,
-                               const ChunkRange& range) {
-    AutoGetCollection autoColl(opCtx, nss, MODE_IS);
-
-    std::ignore = CollectionShardingRuntime::get(opCtx, nss)->beginReceive(range);
-}
-
 template <typename ShardKey>
 RangeDeletionTask createDeletionTask(OperationContext* opCtx,
                                      const NamespaceString& nss,
@@ -83,8 +74,8 @@ RangeDeletionTask createDeletionTask(OperationContext* opCtx,
                                   donorShard,
                                   ChunkRange{BSON("_id" << min), BSON("_id" << max)},
                                   CleanWhenEnum::kNow);
-    const auto clusterTime = LogicalClock::get(opCtx)->getClusterTime();
-    task.setTimestamp(clusterTime.asTimestamp());
+    const auto currentTime = VectorClock::get(opCtx)->getTime();
+    task.setTimestamp(currentTime.clusterTime().asTimestamp());
 
     if (pending)
         task.setPending(true);
@@ -110,8 +101,7 @@ RangeDeletionTask createDeletionTask(OperationContext* opCtx,
 TEST_F(MigrationUtilsTest, TestOverlappingRangeQueryWithIntegerShardKey) {
     auto opCtx = operationContext();
     const auto uuid = UUID::gen();
-
-    PersistentTaskStore<RangeDeletionTask> store(opCtx, NamespaceString::kRangeDeletionNamespace);
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
 
     store.add(opCtx, createDeletionTask(opCtx, NamespaceString{"one"}, uuid, 0, 10));
     store.add(opCtx, createDeletionTask(opCtx, NamespaceString{"two"}, uuid, 10, 20));
@@ -167,8 +157,7 @@ TEST_F(MigrationUtilsTest, TestOverlappingRangeQueryWithIntegerShardKey) {
 TEST_F(MigrationUtilsTest, TestOverlappingRangeQueryWithCompoundShardKeyWhereFirstValueIsConstant) {
     auto opCtx = operationContext();
     const auto uuid = UUID::gen();
-
-    PersistentTaskStore<RangeDeletionTask> store(opCtx, NamespaceString::kRangeDeletionNamespace);
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
 
     auto deletionTasks = {
         createDeletionTask(opCtx,
@@ -251,8 +240,7 @@ TEST_F(MigrationUtilsTest,
        TestOverlappingRangeQueryWithCompoundShardKeyWhereSecondValueIsConstant) {
     auto opCtx = operationContext();
     const auto uuid = UUID::gen();
-
-    PersistentTaskStore<RangeDeletionTask> store(opCtx, NamespaceString::kRangeDeletionNamespace);
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
 
     auto deletionTasks = {
         createDeletionTask(opCtx,
@@ -334,8 +322,7 @@ TEST_F(MigrationUtilsTest,
 TEST_F(MigrationUtilsTest, TestInvalidUUID) {
     auto opCtx = operationContext();
     const auto uuid = UUID::gen();
-
-    PersistentTaskStore<RangeDeletionTask> store(opCtx, NamespaceString::kRangeDeletionNamespace);
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
 
     store.add(opCtx, createDeletionTask(opCtx, NamespaceString{"one"}, uuid, 0, 10));
     store.add(opCtx, createDeletionTask(opCtx, NamespaceString{"two"}, uuid, 10, 20));
@@ -350,17 +337,35 @@ TEST_F(MigrationUtilsTest, TestInvalidUUID) {
     ASSERT_FALSE(migrationutil::checkForConflictingDeletions(opCtx, range, wrongUuid));
 }
 
-// Fixture that uses a mocked CatalogCacheLoader and CatalogClient to allow metadata refreshes
-// without using the mock network.
-class SubmitRangeDeletionTaskTest : public ShardServerTestFixture {
+TEST_F(MigrationUtilsTest, TestUpdateNumberOfOrphans) {
+    auto opCtx = operationContext();
+    const auto collectionUuid = UUID::gen();
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
+    auto rangeDeletionDoc = createDeletionTask(opCtx, kTestNss, collectionUuid, 0, 10);
+    store.add(opCtx, rangeDeletionDoc);
+
+    migrationutil::persistUpdatedNumOrphans(opCtx, rangeDeletionDoc.getId(), collectionUuid, 5);
+    rangeDeletionDoc.setNumOrphanDocs(5);
+    ASSERT_EQ(store.count(opCtx, rangeDeletionDoc.toBSON().removeField("timestamp")), 1);
+
+    migrationutil::persistUpdatedNumOrphans(opCtx, rangeDeletionDoc.getId(), collectionUuid, -5);
+    rangeDeletionDoc.setNumOrphanDocs(0);
+    ASSERT_EQ(store.count(opCtx, rangeDeletionDoc.toBSON().removeField("timestamp")), 1);
+}
+
+/**
+ * Fixture that uses a mocked CatalogCacheLoader and CatalogClient to allow metadata refreshes
+ * without using the mock network.
+ */
+class SubmitRangeDeletionTaskTest : public CollectionShardingRuntimeWithRangeDeleterTest {
 public:
     const HostAndPort kConfigHostAndPort{"dummy", 123};
-    const NamespaceString kNss{"test.foo"};
     const ShardKeyPattern kShardKeyPattern = ShardKeyPattern(BSON("_id" << 1));
     const UUID kDefaultUUID = UUID::gen();
     const OID kEpoch = OID::gen();
-    const DatabaseType kDefaultDatabaseType =
-        DatabaseType(kNss.db().toString(), ShardId("0"), true, DatabaseVersion(kDefaultUUID, 1));
+    const Timestamp kDefaultTimestamp = Timestamp(2, 0);
+    const DatabaseType kDefaultDatabaseType = DatabaseType(
+        kTestNss.db().toString(), ShardId("0"), DatabaseVersion(kDefaultUUID, kDefaultTimestamp));
     const std::vector<ShardType> kShardList = {ShardType("0", "Host0:12345"),
                                                ShardType("1", "Host1:12345")};
 
@@ -374,8 +379,7 @@ public:
         _clusterId = OID::gen();
         ShardingState::get(getServiceContext())->setInitialized(_myShardName, _clusterId);
 
-        std::unique_ptr<CatalogCacheLoaderMock> mockLoader =
-            std::make_unique<CatalogCacheLoaderMock>();
+        auto mockLoader = std::make_unique<CatalogCacheLoaderMock>();
         _mockCatalogCacheLoader = mockLoader.get();
         CatalogCacheLoader::set(getServiceContext(), std::move(mockLoader));
 
@@ -384,7 +388,7 @@ public:
 
         configTargeterMock()->setFindHostReturnValue(kConfigHostAndPort);
 
-        WaitForMajorityService::get(getServiceContext()).setUp(getServiceContext());
+        WaitForMajorityService::get(getServiceContext()).startup(getServiceContext());
 
         // Set up 2 default shards.
         for (const auto& shard : kShardList) {
@@ -399,27 +403,24 @@ public:
 
     void tearDown() override {
         WaitForMajorityService::get(getServiceContext()).shutDown();
-        CatalogCacheLoader::clearForTests(getServiceContext());
-        ShardingMongodTestFixture::tearDown();
-        CollectionShardingStateFactory::clear(getServiceContext());
+
+        ShardServerTestFixture::tearDown();
     }
 
     // Mock for the ShardingCatalogClient used to satisfy loading all shards for the ShardRegistry
     // and loading all collections when a database is loaded for the first time by the CatalogCache.
     class StaticCatalogClient final : public ShardingCatalogClientMock {
     public:
-        StaticCatalogClient(std::vector<ShardType> shards)
-            : ShardingCatalogClientMock(nullptr), _shards(std::move(shards)) {}
+        StaticCatalogClient(std::vector<ShardType> shards) : _shards(std::move(shards)) {}
 
         StatusWith<repl::OpTimeWith<std::vector<ShardType>>> getAllShards(
             OperationContext* opCtx, repl::ReadConcernLevel readConcern) override {
             return repl::OpTimeWith<std::vector<ShardType>>(_shards);
         }
 
-        StatusWith<std::vector<CollectionType>> getCollections(
+        std::vector<CollectionType> getCollections(
             OperationContext* opCtx,
-            const std::string* dbName,
-            repl::OpTime* optime,
+            StringData dbName,
             repl::ReadConcernLevel readConcernLevel) override {
             return _colls;
         }
@@ -434,48 +435,49 @@ public:
     };
 
     UUID createCollectionAndGetUUID(const NamespaceString& nss) {
-        DBDirectClient client(operationContext());
-        client.createCollection(nss.ns());
+        {
+            OperationShardingState::ScopedAllowImplicitCollectionCreate_UNSAFE
+                unsafeCreateCollection(operationContext());
+            uassertStatusOK(
+                createCollection(operationContext(), nss.dbName(), BSON("create" << nss.coll())));
+        }
 
-        AutoGetCollection autoColl(operationContext(), kNss, MODE_IX);
+        AutoGetCollection autoColl(operationContext(), nss, MODE_IX);
         return autoColl.getCollection()->uuid();
     }
 
-    std::unique_ptr<ShardingCatalogClient> makeShardingCatalogClient(
-        std::unique_ptr<DistLockManager> distLockManager) override {
+    std::unique_ptr<ShardingCatalogClient> makeShardingCatalogClient() override {
         auto mockCatalogClient = std::make_unique<StaticCatalogClient>(kShardList);
         // Stash a pointer to the mock so its return values can be set.
         _mockCatalogClient = mockCatalogClient.get();
         return mockCatalogClient;
     }
 
-    CollectionType makeCollectionType(UUID uuid, OID epoch) {
-        CollectionType coll;
-        coll.setNs(kNss);
-        coll.setEpoch(epoch);
-        coll.setKeyPattern(kShardKeyPattern.getKeyPattern());
+    CollectionType makeCollectionType(UUID uuid, OID epoch, Timestamp timestamp) {
+        CollectionType coll(
+            kTestNss, epoch, timestamp, Date_t::now(), uuid, kShardKeyPattern.getKeyPattern());
         coll.setUnique(true);
-        coll.setUUID(uuid);
         return coll;
     }
 
     std::vector<ChunkType> makeChangedChunks(ChunkVersion startingVersion) {
-        ChunkType chunk1(kNss,
+        const auto uuid = UUID::gen();
+        ChunkType chunk1(uuid,
                          {kShardKeyPattern.getKeyPattern().globalMin(), BSON("_id" << -100)},
                          startingVersion,
                          {"0"});
         chunk1.setName(OID::gen());
         startingVersion.incMinor();
 
-        ChunkType chunk2(kNss, {BSON("_id" << -100), BSON("_id" << 0)}, startingVersion, {"1"});
+        ChunkType chunk2(uuid, {BSON("_id" << -100), BSON("_id" << 0)}, startingVersion, {"1"});
         chunk2.setName(OID::gen());
         startingVersion.incMinor();
 
-        ChunkType chunk3(kNss, {BSON("_id" << 0), BSON("_id" << 100)}, startingVersion, {"0"});
+        ChunkType chunk3(uuid, {BSON("_id" << 0), BSON("_id" << 100)}, startingVersion, {"0"});
         chunk3.setName(OID::gen());
         startingVersion.incMinor();
 
-        ChunkType chunk4(kNss,
+        ChunkType chunk4(uuid,
                          {BSON("_id" << 100), kShardKeyPattern.getKeyPattern().globalMax()},
                          startingVersion,
                          {"1"});
@@ -492,10 +494,9 @@ public:
 TEST_F(SubmitRangeDeletionTaskTest,
        FailsAndDeletesTaskIfFilteringMetadataIsUnknownEvenAfterRefresh) {
     auto opCtx = operationContext();
+    auto deletionTask = createDeletionTask(opCtx, kTestNss, kDefaultUUID, 0, 10, _myShardName);
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
 
-    auto deletionTask = createDeletionTask(opCtx, kNss, kDefaultUUID, 0, 10, _myShardName);
-
-    PersistentTaskStore<RangeDeletionTask> store(opCtx, NamespaceString::kRangeDeletionNamespace);
     store.add(opCtx, deletionTask);
     ASSERT_EQ(store.count(opCtx), 1);
     migrationutil::markAsReadyRangeDeletionTaskLocally(opCtx, deletionTask.getId());
@@ -518,9 +519,10 @@ TEST_F(SubmitRangeDeletionTaskTest,
 TEST_F(SubmitRangeDeletionTaskTest, FailsAndDeletesTaskIfNamespaceIsUnshardedEvenAfterRefresh) {
     auto opCtx = operationContext();
 
-    auto deletionTask = createDeletionTask(opCtx, kNss, kDefaultUUID, 0, 10, _myShardName);
+    auto deletionTask = createDeletionTask(opCtx, kTestNss, kDefaultUUID, 0, 10, _myShardName);
+
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
 
-    PersistentTaskStore<RangeDeletionTask> store(opCtx, NamespaceString::kRangeDeletionNamespace);
     store.add(opCtx, deletionTask);
     ASSERT_EQ(store.count(opCtx), 1);
     migrationutil::markAsReadyRangeDeletionTaskLocally(opCtx, deletionTask.getId());
@@ -545,9 +547,10 @@ TEST_F(SubmitRangeDeletionTaskTest,
        FailsAndDeletesTaskIfNamespaceIsUnshardedBeforeAndAfterRefresh) {
     auto opCtx = operationContext();
 
-    auto deletionTask = createDeletionTask(opCtx, kNss, kDefaultUUID, 0, 10, _myShardName);
+    auto deletionTask = createDeletionTask(opCtx, kTestNss, kDefaultUUID, 0, 10, _myShardName);
+
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
 
-    PersistentTaskStore<RangeDeletionTask> store(opCtx, NamespaceString::kRangeDeletionNamespace);
     store.add(opCtx, deletionTask);
     ASSERT_EQ(store.count(opCtx), 1);
     migrationutil::markAsReadyRangeDeletionTaskLocally(opCtx, deletionTask.getId());
@@ -557,7 +560,7 @@ TEST_F(SubmitRangeDeletionTaskTest,
     _mockCatalogCacheLoader->setDatabaseRefreshReturnValue(kDefaultDatabaseType);
     _mockCatalogCacheLoader->setCollectionRefreshReturnValue(
         Status(ErrorCodes::NamespaceNotFound, "dummy errmsg"));
-    forceShardFilteringMetadataRefresh(opCtx, kNss, true);
+    forceShardFilteringMetadataRefresh(opCtx, kTestNss);
 
     auto cleanupCompleteFuture = migrationutil::submitRangeDeletionTask(opCtx, deletionTask);
 
@@ -572,22 +575,23 @@ TEST_F(SubmitRangeDeletionTaskTest,
 TEST_F(SubmitRangeDeletionTaskTest, SucceedsIfFilteringMetadataUUIDMatchesTaskUUID) {
     auto opCtx = operationContext();
 
-    auto collectionUUID = createCollectionAndGetUUID(kNss);
-    auto deletionTask = createDeletionTask(opCtx, kNss, collectionUUID, 0, 10, _myShardName);
+    auto collectionUUID = createCollectionAndGetUUID(kTestNss);
+    auto deletionTask = createDeletionTask(opCtx, kTestNss, collectionUUID, 0, 10, _myShardName);
+
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
 
-    PersistentTaskStore<RangeDeletionTask> store(opCtx, NamespaceString::kRangeDeletionNamespace);
     store.add(opCtx, deletionTask);
     ASSERT_EQ(store.count(opCtx), 1);
     migrationutil::markAsReadyRangeDeletionTaskLocally(opCtx, deletionTask.getId());
 
     // Force a metadata refresh with the task's UUID before the task is submitted.
-    auto coll = makeCollectionType(collectionUUID, kEpoch);
+    auto coll = makeCollectionType(collectionUUID, kEpoch, kDefaultTimestamp);
     _mockCatalogCacheLoader->setDatabaseRefreshReturnValue(kDefaultDatabaseType);
     _mockCatalogCacheLoader->setCollectionRefreshReturnValue(coll);
     _mockCatalogCacheLoader->setChunkRefreshReturnValue(
-        makeChangedChunks(ChunkVersion(1, 0, kEpoch)));
+        makeChangedChunks(ChunkVersion({kEpoch, kDefaultTimestamp}, {1, 0})));
     _mockCatalogClient->setCollections({coll});
-    forceShardFilteringMetadataRefresh(opCtx, kNss, true);
+    forceShardFilteringMetadataRefresh(opCtx, kTestNss);
 
     // The task should have been submitted successfully.
     auto cleanupCompleteFuture = migrationutil::submitRangeDeletionTask(opCtx, deletionTask);
@@ -599,22 +603,26 @@ TEST_F(
     SucceedsIfFilteringMetadataInitiallyUnknownButFilteringMetadataUUIDMatchesTaskUUIDAfterRefresh) {
     auto opCtx = operationContext();
 
-    auto collectionUUID = createCollectionAndGetUUID(kNss);
-    auto deletionTask = createDeletionTask(opCtx, kNss, collectionUUID, 0, 10, _myShardName);
+    auto collectionUUID = createCollectionAndGetUUID(kTestNss);
+    auto deletionTask = createDeletionTask(opCtx, kTestNss, collectionUUID, 0, 10, _myShardName);
+
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
 
-    PersistentTaskStore<RangeDeletionTask> store(opCtx, NamespaceString::kRangeDeletionNamespace);
     store.add(opCtx, deletionTask);
     ASSERT_EQ(store.count(opCtx), 1);
     migrationutil::markAsReadyRangeDeletionTaskLocally(opCtx, deletionTask.getId());
 
     // Make the refresh triggered by submitting the task return a UUID that matches the task's UUID.
-    auto coll = makeCollectionType(collectionUUID, kEpoch);
+    auto coll = makeCollectionType(collectionUUID, kEpoch, kDefaultTimestamp);
     _mockCatalogCacheLoader->setDatabaseRefreshReturnValue(kDefaultDatabaseType);
     _mockCatalogCacheLoader->setCollectionRefreshReturnValue(coll);
     _mockCatalogCacheLoader->setChunkRefreshReturnValue(
-        makeChangedChunks(ChunkVersion(1, 0, kEpoch)));
+        makeChangedChunks(ChunkVersion({kEpoch, kDefaultTimestamp}, {1, 0})));
     _mockCatalogClient->setCollections({coll});
 
+    auto metadata = makeShardedMetadata(opCtx, collectionUUID);
+    csr().setFilteringMetadata(opCtx, metadata);
+
     // The task should have been submitted successfully.
     auto cleanupCompleteFuture = migrationutil::submitRangeDeletionTask(opCtx, deletionTask);
     cleanupCompleteFuture.get(opCtx);
@@ -629,58 +637,26 @@ TEST_F(SubmitRangeDeletionTaskTest,
     _mockCatalogCacheLoader->setDatabaseRefreshReturnValue(kDefaultDatabaseType);
     _mockCatalogCacheLoader->setCollectionRefreshReturnValue(
         Status(ErrorCodes::NamespaceNotFound, "dummy errmsg"));
-    forceShardFilteringMetadataRefresh(opCtx, kNss, true);
+    forceShardFilteringMetadataRefresh(opCtx, kTestNss);
 
-    auto collectionUUID = createCollectionAndGetUUID(kNss);
-    auto deletionTask = createDeletionTask(opCtx, kNss, collectionUUID, 0, 10, _myShardName);
+    auto collectionUUID = createCollectionAndGetUUID(kTestNss);
+    auto deletionTask = createDeletionTask(opCtx, kTestNss, collectionUUID, 0, 10, _myShardName);
+
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
 
-    PersistentTaskStore<RangeDeletionTask> store(opCtx, NamespaceString::kRangeDeletionNamespace);
     store.add(opCtx, deletionTask);
     ASSERT_EQ(store.count(opCtx), 1);
     migrationutil::markAsReadyRangeDeletionTaskLocally(opCtx, deletionTask.getId());
 
     // Make the refresh triggered by submitting the task return a UUID that matches the task's UUID.
-    auto matchingColl = makeCollectionType(collectionUUID, kEpoch);
+    auto matchingColl = makeCollectionType(collectionUUID, kEpoch, kDefaultTimestamp);
     _mockCatalogCacheLoader->setCollectionRefreshReturnValue(matchingColl);
     _mockCatalogCacheLoader->setChunkRefreshReturnValue(
-        makeChangedChunks(ChunkVersion(10, 0, kEpoch)));
+        makeChangedChunks(ChunkVersion({kEpoch, kDefaultTimestamp}, {10, 0})));
     _mockCatalogClient->setCollections({matchingColl});
 
-    // The task should have been submitted successfully.
-    auto cleanupCompleteFuture = migrationutil::submitRangeDeletionTask(opCtx, deletionTask);
-    cleanupCompleteFuture.get(opCtx);
-}
-
-TEST_F(SubmitRangeDeletionTaskTest,
-       SucceedsIfFilteringMetadataUUIDInitiallyDifferentFromTaskUUIDButMatchesAfterRefresh) {
-    auto opCtx = operationContext();
-
-    // Force a metadata refresh with an arbitrary UUID so that the node's filtering metadata is
-    // stale when the task is submitted.
-    const auto staleUUID = UUID::gen();
-    const auto staleEpoch = OID::gen();
-    auto staleColl = makeCollectionType(staleUUID, staleEpoch);
-    _mockCatalogCacheLoader->setDatabaseRefreshReturnValue(kDefaultDatabaseType);
-    _mockCatalogCacheLoader->setCollectionRefreshReturnValue(staleColl);
-    _mockCatalogCacheLoader->setChunkRefreshReturnValue(
-        makeChangedChunks(ChunkVersion(1, 0, staleEpoch)));
-    _mockCatalogClient->setCollections({staleColl});
-    forceShardFilteringMetadataRefresh(opCtx, kNss, true);
-
-    auto collectionUUID = createCollectionAndGetUUID(kNss);
-    auto deletionTask = createDeletionTask(opCtx, kNss, collectionUUID, 0, 10, _myShardName);
-
-    PersistentTaskStore<RangeDeletionTask> store(opCtx, NamespaceString::kRangeDeletionNamespace);
-    store.add(opCtx, deletionTask);
-    ASSERT_EQ(store.count(opCtx), 1);
-    migrationutil::markAsReadyRangeDeletionTaskLocally(opCtx, deletionTask.getId());
-
-    // Make the refresh triggered by submitting the task return a UUID that matches the task's UUID.
-    auto matchingColl = makeCollectionType(collectionUUID, kEpoch);
-    _mockCatalogCacheLoader->setCollectionRefreshReturnValue(matchingColl);
-    _mockCatalogCacheLoader->setChunkRefreshReturnValue(
-        makeChangedChunks(ChunkVersion(10, 0, kEpoch)));
-    _mockCatalogClient->setCollections({matchingColl});
+    auto metadata = makeShardedMetadata(opCtx, collectionUUID);
+    csr().setFilteringMetadata(opCtx, metadata);
 
     // The task should have been submitted successfully.
     auto cleanupCompleteFuture = migrationutil::submitRangeDeletionTask(opCtx, deletionTask);
@@ -691,20 +667,22 @@ TEST_F(SubmitRangeDeletionTaskTest,
        FailsAndDeletesTaskIfFilteringMetadataUUIDDifferentFromTaskUUIDEvenAfterRefresh) {
     auto opCtx = operationContext();
 
-    auto deletionTask = createDeletionTask(opCtx, kNss, kDefaultUUID, 0, 10, _myShardName);
+    auto deletionTask = createDeletionTask(opCtx, kTestNss, kDefaultUUID, 0, 10, _myShardName);
+
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
 
-    PersistentTaskStore<RangeDeletionTask> store(opCtx, NamespaceString::kRangeDeletionNamespace);
     store.add(opCtx, deletionTask);
     ASSERT_EQ(store.count(opCtx), 1);
     migrationutil::markAsReadyRangeDeletionTaskLocally(opCtx, deletionTask.getId());
 
     // Make the refresh triggered by submitting the task return an arbitrary UUID.
     const auto otherEpoch = OID::gen();
-    auto otherColl = makeCollectionType(UUID::gen(), otherEpoch);
+    const auto otherTimestamp = Timestamp(3, 0);
+    auto otherColl = makeCollectionType(UUID::gen(), otherEpoch, otherTimestamp);
     _mockCatalogCacheLoader->setDatabaseRefreshReturnValue(kDefaultDatabaseType);
     _mockCatalogCacheLoader->setCollectionRefreshReturnValue(otherColl);
     _mockCatalogCacheLoader->setChunkRefreshReturnValue(
-        makeChangedChunks(ChunkVersion(1, 0, otherEpoch)));
+        makeChangedChunks(ChunkVersion({otherEpoch, otherTimestamp}, {1, 0})));
     _mockCatalogClient->setCollections({otherColl});
 
     // The task should not have been submitted, and the task's entry should have been removed from
@@ -716,90 +694,5 @@ TEST_F(SubmitRangeDeletionTaskTest,
     ASSERT_EQ(store.count(opCtx), 0);
 }
 
-using ForEachOrphanRangeTest = ShardServerTestFixture;
-
-ChunkRange getRangeForChunk(int i, int nChunks) {
-    invariant(i >= 0);
-    invariant(nChunks > 0);
-    invariant(i < nChunks);
-    if (i == 0) {
-        return {BSON("_id" << MINKEY), BSON("_id" << 0)};
-    }
-    if (i + 1 == nChunks) {
-        return {BSON("_id" << (i - 1) * 100), BSON("_id" << MAXKEY)};
-    }
-    return {BSON("_id" << (i - 1) * 100), BSON("_id" << i * 100)};
-}
-
-
-TEST_F(ForEachOrphanRangeTest, ForEachOrphanRangeIsInterruptible) {
-    const auto kNss = NamespaceString("test.foo");
-
-    // Create a chunk distribution with 1000 chunks alternating between shard1 and shard2.
-    const auto cm = [&]() {
-        const auto nChunks = 1000;
-        const auto collEpoch = OID::gen();
-
-        std::vector<ChunkType> chunks;
-        chunks.reserve(nChunks);
-        for (uint32_t i = 0; i < nChunks; ++i) {
-            chunks.emplace_back(kNss,
-                                getRangeForChunk(i, nChunks),
-                                ChunkVersion{i + 1, 0, collEpoch},
-                                (i % 2 == 0 ? ShardId("shard1") : ShardId("shard2")));
-        }
-
-        auto routingTableHistory = RoutingTableHistory::makeNew(
-            kNss, UUID::gen(), KeyPattern(BSON("_id" << 1)), nullptr, true, collEpoch, chunks);
-        return std::make_shared<ChunkManager>(routingTableHistory, boost::none);
-    }();
-
-    // Install the chunk distribution in the filtering metadata for this shard, "shard1".
-    {
-        AutoGetDb autoDb(operationContext(), kNss.db(), MODE_IX);
-        Lock::CollectionLock collLock(operationContext(), kNss, MODE_IX);
-        CollectionShardingRuntime::get(operationContext(), kNss)
-            ->setFilteringMetadata(operationContext(), CollectionMetadata(cm, ShardId("shard1")));
-    }
-
-    unittest::Barrier barrier1(2);
-    unittest::Barrier barrier2(2);
-
-    auto numOrphanRangesEnumerated = 0;
-    auto mainThreadOpCtx = operationContext();
-
-    auto interruptThread = stdx::async(stdx::launch::async, [&] {
-        // Wait for getNextOrphanRange to have enumerated one range.
-        barrier1.countDownAndWait();
-        {
-            stdx::lock_guard<Client> lk(*mainThreadOpCtx->getClient());
-            mainThreadOpCtx->markKilled();
-        }
-
-        // Signal that we have interrupted getNextOrphanRange's opCtx.
-        barrier2.countDownAndWait();
-    });
-
-    // getNextOrphanRange should throw because its OperationContext gets interrupted.
-    ASSERT_THROWS_CODE(migrationutil::forEachOrphanRange(operationContext(),
-                                                         kNss,
-                                                         [&](const auto& range) {
-                                                             numOrphanRangesEnumerated++;
-                                                             // Signal that we have enumerated one
-                                                             // range.
-                                                             barrier1.countDownAndWait();
-
-                                                             // Wait for our opCtx to be killed.
-                                                             barrier2.countDownAndWait();
-                                                         }),
-                       AssertionException,
-                       ErrorCodes::Interrupted);
-
-    interruptThread.get();
-
-    // Only one orphan range should have been enumerated.
-    ASSERT_EQUALS(1, numOrphanRangesEnumerated);
-}
-
 }  // namespace
 }  // namespace mongo
