@@ -27,13 +27,10 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/s/move_primary_source_manager.h"
 
 #include "mongo/client/connpool.h"
+#include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/dbdirectclient.h"
@@ -41,21 +38,22 @@
 #include "mongo/db/s/sharding_logging.h"
 #include "mongo/db/s/sharding_state_recovery.h"
 #include "mongo/db/s/sharding_statistics.h"
+#include "mongo/db/s/type_shard_database.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
-#include "mongo/s/catalog/type_shard_database.h"
 #include "mongo/s/catalog_cache.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/request_types/move_primary_gen.h"
 #include "mongo/util/exit.h"
 #include "mongo/util/scopeguard.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
+
 namespace mongo {
 
 MONGO_FAIL_POINT_DEFINE(hangInCloneStage);
 MONGO_FAIL_POINT_DEFINE(hangInCleanStaleDataStage);
 
-using namespace shardmetadatautil;
-
 MovePrimarySourceManager::MovePrimarySourceManager(OperationContext* opCtx,
                                                    ShardMovePrimary requestArgs,
                                                    StringData dbname,
@@ -64,18 +62,22 @@ MovePrimarySourceManager::MovePrimarySourceManager(OperationContext* opCtx,
     : _requestArgs(std::move(requestArgs)),
       _dbname(dbname),
       _fromShard(std::move(fromShard)),
-      _toShard(std::move(toShard)) {}
+      _toShard(std::move(toShard)),
+      _critSecReason(BSON("command"
+                          << "movePrimary"
+                          << "dbName" << _dbname << "fromShard" << fromShard << "toShard"
+                          << toShard)) {}
 
 MovePrimarySourceManager::~MovePrimarySourceManager() {}
 
 NamespaceString MovePrimarySourceManager::getNss() const {
-    return _requestArgs.get_shardsvrMovePrimary().get();
+    return _requestArgs.get_shardsvrMovePrimary();
 }
 
 Status MovePrimarySourceManager::clone(OperationContext* opCtx) {
     invariant(!opCtx->lockState()->isLocked());
     invariant(_state == kCreated);
-    auto scopedGuard = makeGuard([&] { cleanupOnError(opCtx); });
+    ScopeGuard scopedGuard([&] { cleanupOnError(opCtx); });
 
     LOGV2(22042,
           "Moving {db} primary from: {fromShard} to: {toShard}",
@@ -85,17 +87,22 @@ Status MovePrimarySourceManager::clone(OperationContext* opCtx) {
           "toShard"_attr = _toShard);
 
     // Record start in changelog
-    uassertStatusOK(ShardingLogging::get(opCtx)->logChangeChecked(
+    auto logChangeCheckedStatus = ShardingLogging::get(opCtx)->logChangeChecked(
         opCtx,
         "movePrimary.start",
         _dbname.toString(),
         _buildMoveLogEntry(_dbname.toString(), _fromShard.toString(), _toShard.toString()),
-        ShardingCatalogClient::kMajorityWriteConcern));
+        ShardingCatalogClient::kMajorityWriteConcern);
+
+    if (!logChangeCheckedStatus.isOK()) {
+        return logChangeCheckedStatus;
+    }
 
     {
-        // We use AutoGetOrCreateDb the first time just in case movePrimary was called before any
-        // data was inserted into the database.
-        AutoGetOrCreateDb autoDb(opCtx, getNss().toString(), MODE_X);
+        // We use AutoGetDb::ensureDbExists() the first time just in case movePrimary was called
+        // before any data was inserted into the database.
+        AutoGetDb autoDb(opCtx, getNss().dbName(), MODE_X);
+        invariant(autoDb.ensureDbExists(opCtx), getNss().toString());
 
         auto dss = DatabaseShardingState::get(opCtx, getNss().toString());
         auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
@@ -124,30 +131,13 @@ Status MovePrimarySourceManager::clone(OperationContext* opCtx) {
         ReadPreferenceSetting(ReadPreference::PrimaryOnly),
         "admin",
         CommandHelpers::appendMajorityWriteConcern(cloneCatalogDataCommandBuilder.obj()),
-        Shard::RetryPolicy::kIdempotent);
+        Shard::RetryPolicy::kNotIdempotent);
 
     auto cloneCommandStatus = Shard::CommandResponse::getEffectiveStatus(cloneCommandResponse);
-
-    // If the `toShard` is on v4.2 or earlier, it will not recognize the command name
-    // _shardsvrCloneCatalogData. We will retry the command with the old name _cloneCatalogData.
-    if (cloneCommandStatus == ErrorCodes::CommandNotFound) {
-        BSONObjBuilder legacyCloneCatalogDataCommandBuilder;
-        legacyCloneCatalogDataCommandBuilder << "_cloneCatalogData" << _dbname << "from"
-                                             << fromShardObj->getConnString().toString();
-
-
-        cloneCommandResponse = toShardObj->runCommandWithFixedRetryAttempts(
-            opCtx,
-            ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-            "admin",
-            CommandHelpers::appendMajorityWriteConcern(legacyCloneCatalogDataCommandBuilder.obj()),
-            Shard::RetryPolicy::kIdempotent);
-
-        cloneCommandStatus = Shard::CommandResponse::getEffectiveStatus(cloneCommandResponse);
+    if (!cloneCommandStatus.isOK()) {
+        return cloneCommandStatus;
     }
 
-    uassertStatusOK(cloneCommandStatus);
-
     auto clonedCollsArray = cloneCommandResponse.getValue().response["clonedColls"];
     for (const auto& elem : clonedCollsArray.Obj()) {
         if (elem.type() == String) {
@@ -163,16 +153,19 @@ Status MovePrimarySourceManager::clone(OperationContext* opCtx) {
 Status MovePrimarySourceManager::enterCriticalSection(OperationContext* opCtx) {
     invariant(!opCtx->lockState()->isLocked());
     invariant(_state == kCloneCaughtUp);
-    auto scopedGuard = makeGuard([&] { cleanupOnError(opCtx); });
+    ScopeGuard scopedGuard([&] { cleanupOnError(opCtx); });
 
     // Mark the shard as running a critical operation that requires recovery on crash.
-    uassertStatusOK(ShardingStateRecovery::startMetadataOp(opCtx));
+    auto startMetadataOpStatus = ShardingStateRecovery::startMetadataOp(opCtx);
+    if (!startMetadataOpStatus.isOK()) {
+        return startMetadataOpStatus;
+    }
 
     {
         // The critical section must be entered with the database X lock in order to ensure there
         // are no writes which could have entered and passed the database version check just before
         // we entered the critical section, but will potentially complete after we left it.
-        AutoGetDb autoDb(opCtx, getNss().toString(), MODE_X);
+        AutoGetDb autoDb(opCtx, getNss().dbName(), MODE_X);
 
         if (!autoDb.getDb()) {
             uasserted(ErrorCodes::ConflictingOperationInProgress,
@@ -184,7 +177,7 @@ Status MovePrimarySourceManager::enterCriticalSection(OperationContext* opCtx) {
         auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
 
         // IMPORTANT: After this line, the critical section is in place and needs to be signaled
-        dss->enterCriticalSectionCatchUpPhase(opCtx, dssLock);
+        dss->enterCriticalSectionCatchUpPhase(opCtx, dssLock, _critSecReason);
     }
 
     _state = kCriticalSection;
@@ -195,12 +188,12 @@ Status MovePrimarySourceManager::enterCriticalSection(OperationContext* opCtx) {
     // time inclusive of the move primary config commit update from accessing secondary data.
     // Note: this write must occur after the critSec flag is set, to ensure the secondary refresh
     // will stall behind the flag.
-    Status signalStatus =
-        updateShardDatabasesEntry(opCtx,
-                                  BSON(ShardDatabaseType::name() << getNss().toString()),
-                                  BSONObj(),
-                                  BSON(ShardDatabaseType::enterCriticalSectionCounter() << 1),
-                                  false /*upsert*/);
+    Status signalStatus = shardmetadatautil::updateShardDatabasesEntry(
+        opCtx,
+        BSON(ShardDatabaseType::kNameFieldName << getNss().toString()),
+        BSONObj(),
+        BSON(ShardDatabaseType::kEnterCriticalSectionCounterFieldName << 1),
+        false /*upsert*/);
     if (!signalStatus.isOK()) {
         return {
             ErrorCodes::OperationFailed,
@@ -218,14 +211,12 @@ Status MovePrimarySourceManager::enterCriticalSection(OperationContext* opCtx) {
 Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
     invariant(!opCtx->lockState()->isLocked());
     invariant(_state == kCriticalSection);
-    auto scopedGuard = makeGuard([&] { cleanupOnError(opCtx); });
+    ScopeGuard scopedGuard([&] { cleanupOnError(opCtx); });
 
-    ConfigsvrCommitMovePrimary commitMovePrimaryRequest;
-    commitMovePrimaryRequest.set_configsvrCommitMovePrimary(getNss().ns());
-    commitMovePrimaryRequest.setTo(_toShard.toString());
+    boost::optional<DatabaseVersion> expectedDbVersion;
 
     {
-        AutoGetDb autoDb(opCtx, getNss().toString(), MODE_X);
+        AutoGetDb autoDb(opCtx, getNss().dbName(), MODE_X);
 
         if (!autoDb.getDb()) {
             uasserted(ErrorCodes::ConflictingOperationInProgress,
@@ -238,26 +229,22 @@ Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
 
         // Read operations must begin to wait on the critical section just before we send the
         // commit operation to the config server
-        dss->enterCriticalSectionCommitPhase(opCtx, dssLock);
+        dss->enterCriticalSectionCommitPhase(opCtx, dssLock, _critSecReason);
+
+        expectedDbVersion = DatabaseHolder::get(opCtx)->getDbVersion(opCtx, _dbname);
     }
 
-    auto configShard = Grid::get(opCtx)->shardRegistry()->getConfigShard();
-
-    BSONObj finalCommandObj;
-    auto commitMovePrimaryResponse = configShard->runCommandWithFixedRetryAttempts(
-        opCtx,
-        ReadPreferenceSetting{ReadPreference::PrimaryOnly},
-        "admin",
-        CommandHelpers::appendMajorityWriteConcern(CommandHelpers::appendPassthroughFields(
-            finalCommandObj, commitMovePrimaryRequest.toBSON())),
-        Shard::RetryPolicy::kIdempotent);
-
-    auto commitStatus = Shard::CommandResponse::getEffectiveStatus(commitMovePrimaryResponse);
+    auto commitStatus = [&]() {
+        try {
+            return _commitOnConfig(opCtx, *expectedDbVersion);
+        } catch (const DBException& ex) {
+            return ex.toStatus();
+        }
+    }();
 
     if (!commitStatus.isOK()) {
         // Need to get the latest optime in case the refresh request goes to a secondary --
-        // otherwise the read won't wait for the write that _configsvrCommitMovePrimary may have
-        // done
+        // otherwise the read won't wait for the write that commit on config server may have done.
         LOGV2(22044,
               "Error occurred while committing the movePrimary. Performing a majority write "
               "against the config server to obtain its latest optime: {error}",
@@ -288,7 +275,7 @@ Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
         // this node can accept writes for this collection as a proxy for it being primary.
         if (!validateStatus.isOK()) {
             UninterruptibleLockGuard noInterrupt(opCtx->lockState());
-            AutoGetDb autoDb(opCtx, getNss().toString(), MODE_IX);
+            AutoGetDb autoDb(opCtx, getNss().dbName(), MODE_IX);
 
             if (!autoDb.getDb()) {
                 uasserted(ErrorCodes::ConflictingOperationInProgress,
@@ -297,10 +284,8 @@ Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
             }
 
             if (!repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, getNss())) {
-                auto dss = DatabaseShardingState::get(opCtx, getNss().toString());
-                auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
-
-                dss->setDbVersion(opCtx, boost::none, dssLock);
+                DatabaseHolder::get(opCtx)->clearDbInfo(
+                    opCtx, DatabaseName(boost::none, getNss().toString()));
                 uassertStatusOK(validateStatus.withContext(
                     str::stream() << "Unable to verify movePrimary commit for database: "
                                   << getNss().ns()
@@ -343,6 +328,121 @@ Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
     return Status::OK();
 }
 
+Status MovePrimarySourceManager::_commitOnConfig(OperationContext* opCtx,
+                                                 const DatabaseVersion& expectedDbVersion) {
+    LOGV2_DEBUG(6854100,
+                3,
+                "Committing movePrimary",
+                "db"_attr = _dbname,
+                "fromShard"_attr = _fromShard,
+                "toShard"_attr = _toShard,
+                "expectedDbVersion"_attr = expectedDbVersion);
+
+    const auto commitStatus = [&] {
+        ConfigsvrCommitMovePrimary commitRequest(_dbname, expectedDbVersion, _toShard);
+        commitRequest.setDbName(NamespaceString::kAdminDb);
+
+        const auto commitResponse =
+            Grid::get(opCtx)->shardRegistry()->getConfigShard()->runCommandWithFixedRetryAttempts(
+                opCtx,
+                ReadPreferenceSetting(ReadPreference::PrimaryOnly),
+                NamespaceString::kAdminDb.toString(),
+                CommandHelpers::appendMajorityWriteConcern(commitRequest.toBSON({})),
+                Shard::RetryPolicy::kIdempotent);
+
+        const auto status = Shard::CommandResponse::getEffectiveStatus(commitResponse);
+        if (status != ErrorCodes::CommandNotFound) {
+            return status;
+        }
+
+        LOGV2(6854101,
+              "_configsvrCommitMovePrimary command not found on config server, so try to update "
+              "the metadata document directly",
+              "db"_attr = _dbname);
+
+        // The fallback logic is not synchronized with the removeShard command and simultaneous
+        // invocations of movePrimary and removeShard can lead to data loss.
+        return _fallbackCommitOnConfig(opCtx, expectedDbVersion);
+    }();
+
+    if (!commitStatus.isOK()) {
+        LOGV2(6854102,
+              "Error committing movePrimary",
+              "db"_attr = _dbname,
+              "error"_attr = redact(commitStatus));
+        return commitStatus;
+    }
+
+    const auto updatedDbType = [&]() {
+        auto findResponse = uassertStatusOK(
+            Grid::get(opCtx)->shardRegistry()->getConfigShard()->exhaustiveFindOnConfig(
+                opCtx,
+                ReadPreferenceSetting{ReadPreference::PrimaryOnly},
+                repl::ReadConcernLevel::kMajorityReadConcern,
+                NamespaceString::kConfigDatabasesNamespace,
+                BSON(DatabaseType::kNameFieldName << _dbname),
+                BSON(DatabaseType::kNameFieldName << -1),
+                1));
+
+        const auto databases = std::move(findResponse.docs);
+        uassert(ErrorCodes::IncompatibleShardingMetadata,
+                "Tried to find version for database {}, but found no databases"_format(_dbname),
+                !databases.empty());
+
+        return DatabaseType::parse(IDLParserContext("DatabaseType"), databases.front());
+    }();
+    tassert(6851100,
+            "Error committing movePrimary: database version went backwards",
+            updatedDbType.getVersion() > expectedDbVersion);
+    uassert(6851101,
+            "Error committing movePrimary: update of config.databases failed",
+            updatedDbType.getPrimary() != _fromShard);
+
+    LOGV2_DEBUG(6854103,
+                3,
+                "Commited movePrimary",
+                "db"_attr = _dbname,
+                "fromShard"_attr = _fromShard,
+                "toShard"_attr = _toShard,
+                "updatedDbVersion"_attr = updatedDbType.getVersion());
+
+    return Status::OK();
+}
+
+Status MovePrimarySourceManager::_fallbackCommitOnConfig(OperationContext* opCtx,
+                                                         const DatabaseVersion& expectedDbVersion) {
+    const auto query = [&] {
+        BSONObjBuilder bsonBuilder;
+        bsonBuilder.append(DatabaseType::kNameFieldName, _dbname);
+        // Include the version in the update filter to be resilient to potential network retries and
+        // delayed messages.
+        for (const auto [fieldName, fieldValue] : expectedDbVersion.toBSON()) {
+            const auto dottedFieldName = DatabaseType::kVersionFieldName + "." + fieldName;
+            bsonBuilder.appendAs(fieldValue, dottedFieldName);
+        }
+        return bsonBuilder.obj();
+    }();
+
+    const auto update = [&] {
+        const auto newDbVersion = expectedDbVersion.makeUpdated();
+
+        BSONObjBuilder bsonBuilder;
+        bsonBuilder.append(DatabaseType::kPrimaryFieldName, _toShard);
+        bsonBuilder.append(DatabaseType::kVersionFieldName, newDbVersion.toBSON());
+        return BSON("$set" << bsonBuilder.obj());
+    }();
+
+    return Grid::get(opCtx)
+        ->catalogClient()
+        ->updateConfigDocument(opCtx,
+                               NamespaceString::kConfigDatabasesNamespace,
+                               query,
+                               update,
+                               false,
+                               ShardingCatalogClient::kMajorityWriteConcern)
+        .getStatus();
+}
+
 Status MovePrimarySourceManager::cleanStaleData(OperationContext* opCtx) {
     invariant(!opCtx->lockState()->isLocked());
     invariant(_state == kNeedCleanStaleData);
@@ -371,7 +471,6 @@ Status MovePrimarySourceManager::cleanStaleData(OperationContext* opCtx) {
     return Status::OK();
 }
 
-
 void MovePrimarySourceManager::cleanupOnError(OperationContext* opCtx) {
     if (_state == kDone) {
         return;
@@ -404,15 +503,14 @@ void MovePrimarySourceManager::_cleanup(OperationContext* opCtx) {
     {
         // Unregister from the database's sharding state if we're still registered.
         UninterruptibleLockGuard noInterrupt(opCtx->lockState());
-        AutoGetDb autoDb(opCtx, getNss().toString(), MODE_IX);
-
-        auto dss = DatabaseShardingState::get(opCtx, getNss().toString());
-        auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
-
-        dss->clearMovePrimarySourceManager(opCtx, dssLock);
+        AutoGetDb autoDb(opCtx, getNss().dbName(), MODE_IX);
 
+        auto dss = DatabaseShardingState::get(opCtx, getNss().db());
+        dss->clearMovePrimarySourceManager(opCtx);
+        DatabaseHolder::get(opCtx)->clearDbInfo(opCtx,
+                                                DatabaseName(boost::none, getNss().toString()));
         // Leave the critical section if we're still registered.
-        dss->exitCriticalSection(opCtx, boost::none, dssLock);
+        dss->exitCriticalSection(opCtx, _critSecReason);
     }
 
     if (_state == kCriticalSection || _state == kCloneCompleted) {
