@@ -27,92 +27,52 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kShardingRangeDeleter
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/s/range_deletion_util.h"
 
 #include <algorithm>
+#include <boost/optional.hpp>
 #include <utility>
 
-#include <boost/optional.hpp>
-
-#include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/dbhelpers.h"
-#include "mongo/db/exec/delete.h"
+#include "mongo/db/exec/delete_stage.h"
 #include "mongo/db/exec/working_set_common.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/keypattern.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/persistent_task_store.h"
 #include "mongo/db/query/internal_plans.h"
 #include "mongo/db/query/plan_yield_policy.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/query_planner.h"
 #include "mongo/db/repl/repl_client_info.h"
+#include "mongo/db/repl/wait_for_majority_service.h"
 #include "mongo/db/s/migration_util.h"
-#include "mongo/db/s/persistent_task_store.h"
-#include "mongo/db/s/range_deletion_task_gen.h"
+#include "mongo/db/s/shard_key_index_util.h"
+#include "mongo/db/s/sharding_runtime_d_params_gen.h"
 #include "mongo/db/s/sharding_statistics.h"
-#include "mongo/db/s/wait_for_majority_service.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/remove_saver.h"
 #include "mongo/db/write_concern.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/logv2/log.h"
+#include "mongo/s/catalog/sharding_catalog_client.h"
+#include "mongo/util/cancellation.h"
 #include "mongo/util/future_util.h"
 
-namespace mongo {
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kShardingRangeDeleter
 
+namespace mongo {
 namespace {
-const WriteConcernOptions kMajorityWriteConcern(WriteConcernOptions::kMajority,
-                                                WriteConcernOptions::SyncMode::UNSET,
-                                                WriteConcernOptions::kWriteConcernTimeoutSharding);
 
 MONGO_FAIL_POINT_DEFINE(hangBeforeDoingDeletion);
+MONGO_FAIL_POINT_DEFINE(hangAfterDoingDeletion);
 MONGO_FAIL_POINT_DEFINE(suspendRangeDeletion);
 MONGO_FAIL_POINT_DEFINE(throwWriteConflictExceptionInDeleteRange);
 MONGO_FAIL_POINT_DEFINE(throwInternalErrorInDeleteRange);
 
-/**
- * Returns whether the currentCollection has the same UUID as the expectedCollectionUuid. Used to
- * ensure that the collection has not been dropped or dropped and recreated since the range was
- * enqueued for deletion.
- */
-bool collectionUuidHasChanged(const NamespaceString& nss,
-                              Collection* currentCollection,
-                              UUID expectedCollectionUuid) {
-
-    if (!currentCollection) {
-        LOGV2_DEBUG(23763,
-                    1,
-                    "Abandoning range deletion task for {namespace} with UUID "
-                    "{expectedCollectionUuid} because the collection has been dropped",
-                    "Abandoning range deletion task for because the collection has been dropped",
-                    "namespace"_attr = nss.ns(),
-                    "expectedCollectionUuid"_attr = expectedCollectionUuid);
-        return true;
-    }
-
-    if (currentCollection->uuid() != expectedCollectionUuid) {
-        LOGV2_DEBUG(
-            23764,
-            1,
-            "Abandoning range deletion task for {namespace} with UUID {expectedCollectionUUID} "
-            "because UUID of {namespace} has changed (current is {currentCollectionUUID})",
-            "Abandoning range deletion task because UUID has changed",
-            "namespace"_attr = nss.ns(),
-            "expectedCollectionUUID"_attr = expectedCollectionUuid,
-            "currentCollectionUUID"_attr = currentCollection->uuid());
-        return true;
-    }
-
-    return false;
-}
-
 /**
  * Performs the deletion of up to numDocsToRemovePerBatch entries within the range in progress. Must
  * be called under the collection lock.
@@ -121,37 +81,29 @@ bool collectionUuidHasChanged(const NamespaceString& nss,
  * the range failed.
  */
 StatusWith<int> deleteNextBatch(OperationContext* opCtx,
-                                Collection* collection,
+                                const CollectionPtr& collection,
                                 BSONObj const& keyPattern,
                                 ChunkRange const& range,
                                 int numDocsToRemovePerBatch) {
-    invariant(collection != nullptr);
+    invariant(collection);
 
     auto const nss = collection->ns();
 
     // The IndexChunk has a keyPattern that may apply to more than one index - we need to
     // select the index and get the full index keyPattern here.
-    auto catalog = collection->getIndexCatalog();
-    const IndexDescriptor* idx = catalog->findShardKeyPrefixedIndex(opCtx, keyPattern, false);
-    if (!idx) {
-        LOGV2_ERROR(23765,
-                    "Unable to find shard key index",
-                    "keyPattern"_attr = keyPattern,
-                    "namespace"_attr = nss.ns());
-
-        // When a shard key index is not found, the range deleter gets stuck and indefinitely logs
-        // an error message. This sleep is aimed at avoiding logging too aggressively in order to
-        // prevent log files to increase too much in size.
-        opCtx->sleepFor(Seconds(5));
-
-        uasserted(ErrorCodes::IndexNotFound,
-                  str::stream() << "Unable to find shard key index"
-                                << " for " << nss.ns() << " and key pattern `"
-                                << keyPattern.toString() << "'");
+    auto shardKeyIdx = findShardKeyPrefixedIndex(
+        opCtx, collection, collection->getIndexCatalog(), keyPattern, /*requireSingleKey=*/false);
+    if (!shardKeyIdx) {
+        LOGV2_ERROR_OPTIONS(23765,
+                            {logv2::UserAssertAfterLog(ErrorCodes::InternalError)},
+                            "Unable to find shard key index for {keyPattern} in {namespace}",
+                            "Unable to find shard key index",
+                            "keyPattern"_attr = keyPattern,
+                            "namespace"_attr = nss.ns());
     }
 
     // Extend bounds to match the index we found
-    const KeyPattern indexKeyPattern(idx->keyPattern());
+    const KeyPattern indexKeyPattern(shardKeyIdx->keyPattern());
     const auto extend = [&](const auto& key) {
         return Helpers::toKeyFormat(indexKeyPattern.extendRangeBound(key, false));
     };
@@ -167,18 +119,6 @@ StatusWith<int> deleteNextBatch(OperationContext* opCtx,
                 "max"_attr = max,
                 "namespace"_attr = nss.ns());
 
-    const auto indexName = idx->indexName();
-    const IndexDescriptor* descriptor =
-        collection->getIndexCatalog()->findIndexByName(opCtx, indexName);
-    if (!descriptor) {
-        LOGV2_ERROR_OPTIONS(23767,
-                            {logv2::UserAssertAfterLog(ErrorCodes::InternalError)},
-                            "Shard key index with name {indexName} on {namespace} was dropped",
-                            "Shard key index was dropped",
-                            "indexName"_attr = indexName,
-                            "namespace"_attr = nss.ns());
-    }
-
     auto deleteStageParams = std::make_unique<DeleteStageParams>();
     deleteStageParams->fromMigrate = true;
     deleteStageParams->isMulti = true;
@@ -189,14 +129,15 @@ StatusWith<int> deleteNextBatch(OperationContext* opCtx,
             std::make_unique<RemoveSaver>("moveChunk", nss.ns(), "cleaning");
     }
 
-    auto exec = InternalPlanner::deleteWithIndexScan(opCtx,
-                                                     collection,
+    auto exec =
+        InternalPlanner::deleteWithShardKeyIndexScan(opCtx,
+                                                     &collection,
                                                      std::move(deleteStageParams),
-                                                     descriptor,
+                                                     *shardKeyIdx,
                                                      min,
                                                      max,
                                                      BoundInclusion::kIncludeStartKeyOnly,
-                                                     PlanExecutor::YIELD_AUTO,
+                                                     PlanYieldPolicy::YieldPolicy::YIELD_AUTO,
                                                      InternalPlanner::FORWARD);
 
     if (MONGO_unlikely(hangBeforeDoingDeletion.shouldFail())) {
@@ -204,71 +145,64 @@ StatusWith<int> deleteNextBatch(OperationContext* opCtx,
         hangBeforeDoingDeletion.pauseWhileSet(opCtx);
     }
 
-    long long bytesDeleted = 0;
-    int numDocsDeleted = 0;
+    int numDeleted = 0;
     do {
         BSONObj deletedObj;
 
         if (throwWriteConflictExceptionInDeleteRange.shouldFail()) {
-            throw WriteConflictException();
+            throwWriteConflictException(
+                str::stream() << "Hit failpoint '"
+                              << throwWriteConflictExceptionInDeleteRange.getName() << "'.");
         }
 
         if (throwInternalErrorInDeleteRange.shouldFail()) {
             uasserted(ErrorCodes::InternalError, "Failing for test");
         }
 
-        PlanExecutor::ExecState state = exec->getNext(&deletedObj, nullptr);
+        PlanExecutor::ExecState state;
+        try {
+            state = exec->getNext(&deletedObj, nullptr);
+        } catch (const DBException& ex) {
+            auto&& explainer = exec->getPlanExplainer();
+            auto&& [stats, _] =
+                explainer.getWinningPlanStats(ExplainOptions::Verbosity::kExecStats);
+            LOGV2_WARNING(23776,
+                          "Cursor error while trying to delete {min} to {max} in {namespace}, "
+                          "stats: {stats}, error: {error}",
+                          "Cursor error while trying to delete range",
+                          "min"_attr = redact(min),
+                          "max"_attr = redact(max),
+                          "namespace"_attr = nss,
+                          "stats"_attr = redact(stats),
+                          "error"_attr = redact(ex.toStatus()));
+            throw;
+        }
 
         if (state == PlanExecutor::IS_EOF) {
             break;
         }
 
-        if (state == PlanExecutor::FAILURE) {
-            LOGV2_WARNING(23776,
-                          "{planExecutorState} - cursor error while trying to delete {min} to "
-                          "{max} in {namespace}: FAILURE, stats: {explainGetWinningPlanStats}",
-                          "Cursor error while trying to delete range",
-                          "planExecutorState"_attr = PlanExecutor::statestr(state),
-                          "min"_attr = redact(min),
-                          "max"_attr = redact(max),
-                          "namespace"_attr = nss,
-                          "explainGetWinningPlanStats"_attr =
-                              Explain::getWinningPlanStats(exec.get()));
-            uasserted(ErrorCodes::OperationFailed,
-                      str::stream() << "Cursor error while trying to delete " << redact(min)
-                                    << " to " << redact(max) << " in " << nss.ns()
-                                    << ", stats: " << Explain::getWinningPlanStats(exec.get()));
-        }
-
-        bytesDeleted += deletedObj.objsize();
         invariant(PlanExecutor::ADVANCED == state);
-    } while (++numDocsDeleted < numDocsToRemovePerBatch);
+        ShardingStatistics::get(opCtx).countDocsDeletedOnDonor.addAndFetch(1);
 
-    ShardingStatistics::get(opCtx).countDocsDeletedByRangeDeleter.addAndFetch(numDocsDeleted);
-    ShardingStatistics::get(opCtx).countBytesDeletedByRangeDeleter.addAndFetch(bytesDeleted);
+    } while (++numDeleted < numDocsToRemovePerBatch);
 
-    return numDocsDeleted;
+    return numDeleted;
 }
 
-
 template <typename Callable>
 auto withTemporaryOperationContext(Callable&& callable, const NamespaceString& nss) {
     ThreadClient tc(migrationutil::kRangeDeletionThreadName, getGlobalServiceContext());
     {
         stdx::lock_guard<Client> lk(*tc.get());
-        tc->setSystemOperationKillable(lk);
+        tc->setSystemOperationKillableByStepdown(lk);
     }
     auto uniqueOpCtx = Client::getCurrent()->makeOperationContext();
     auto opCtx = uniqueOpCtx.get();
 
-    {
-        // We acquire the global IX lock and then immediately release it to ensure this operation
-        // would be killed by the RstlKillOpThread during step-up or stepdown. Note that the
-        // RstlKillOpThread kills any operations on step-up or stepdown for which
-        // Locker::wasGlobalLockTakenInModeConflictingWithWrites() returns true.
-        Lock::GlobalLock lk(opCtx, MODE_IX);
-    }
-    invariant(opCtx->lockState()->wasGlobalLockTakenInModeConflictingWithWrites());
+    // Ensure that this operation will be killed by the RstlKillOpThread during step-up or stepdown.
+    opCtx->setAlwaysInterruptAtStepDownOrUp_UNSAFE();
+    invariant(opCtx->shouldAlwaysInterruptAtStepDownOrUp());
 
     {
         auto replCoord = repl::ReplicationCoordinator::get(opCtx);
@@ -276,7 +210,7 @@ auto withTemporaryOperationContext(Callable&& callable, const NamespaceString& n
         uassert(ErrorCodes::PrimarySteppedDown,
                 str::stream() << "Not primary while running range deletion task for collection"
                               << nss,
-                replCoord->getReplicationMode() != repl::ReplicationCoordinator::modeReplSet ||
+                replCoord->getReplicationMode() == repl::ReplicationCoordinator::modeReplSet &&
                     replCoord->canAcceptWritesFor(opCtx, nss));
     }
 
@@ -293,11 +227,11 @@ void ensureRangeDeletionTaskStillExists(OperationContext* opCtx, const UUID& mig
     // corruption. The scheme for checking whether the range deletion task document still exists
     // relies on the executor only having a single thread and that thread being solely responsible
     // for deleting the range deletion task document.
-    PersistentTaskStore<RangeDeletionTask> store(opCtx, NamespaceString::kRangeDeletionNamespace);
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
     auto count = store.count(opCtx,
-                             QUERY(RangeDeletionTask::kIdFieldName
-                                   << migrationId << RangeDeletionTask::kPendingFieldName
-                                   << BSON("$exists" << false)));
+                             BSON(RangeDeletionTask::kIdFieldName
+                                  << migrationId << RangeDeletionTask::kPendingFieldName
+                                  << BSON("$exists" << false)));
     invariant(count == 0 || count == 1, "found duplicate range deletion tasks");
     uassert(ErrorCodes::RangeDeletionAbandonedBecauseTaskDocumentDoesNotExist,
             "Range deletion task no longer exists",
@@ -309,124 +243,48 @@ void ensureRangeDeletionTaskStillExists(OperationContext* opCtx, const UUID& mig
     // holding any locks.
 }
 
-/**
- * Delete the range in a sequence of batches until there are no more documents to
- * delete or deletion returns an error.
- */
-ExecutorFuture<void> deleteRangeInBatches(const std::shared_ptr<executor::TaskExecutor>& executor,
-                                          const NamespaceString& nss,
-                                          const UUID& collectionUuid,
-                                          const BSONObj& keyPattern,
-                                          const ChunkRange& range,
-                                          const boost::optional<UUID>& migrationId,
-                                          int numDocsToRemovePerBatch,
-                                          Milliseconds delayBetweenBatches) {
-    return ExecutorFuture<void>(executor)
-        .then([=] {
-            bool allDocsRemoved = false;
-            // Delete all batches in this range unless a stepdown error occurs. Do not yield the
-            // executor to ensure that this range is fully deleted before another range is
-            // processed.
-            while (!allDocsRemoved) {
-                try {
-                    allDocsRemoved = withTemporaryOperationContext(
-                        [=](OperationContext* opCtx) {
-                            LOGV2_DEBUG(5346200,
-                                        1,
-                                        "Starting batch deletion",
-                                        "namespace"_attr = nss,
-                                        "range"_attr = redact(range.toString()),
-                                        "numDocsToRemovePerBatch"_attr = numDocsToRemovePerBatch,
-                                        "delayBetweenBatches"_attr = delayBetweenBatches);
+void markRangeDeletionTaskAsProcessing(OperationContext* opCtx, const UUID& migrationId) {
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
+    auto query = BSON(RangeDeletionTask::kIdFieldName << migrationId);
+    static const auto update =
+        BSON("$set" << BSON(RangeDeletionTask::kProcessingFieldName
+                            << true << RangeDeletionTask::kWhenToCleanFieldName
+                            << CleanWhen_serializer(CleanWhenEnum::kNow)));
 
-                            if (migrationId) {
-                                ensureRangeDeletionTaskStillExists(opCtx, *migrationId);
-                            }
-
-                            AutoGetCollection autoColl(opCtx, nss, MODE_IX);
-                            auto* const collection = autoColl.getCollection();
-
-                            // Ensure the collection exists and has not been dropped or dropped and
-                            // recreated.
-                            uassert(ErrorCodes::
-                                        RangeDeletionAbandonedBecauseCollectionWithUUIDDoesNotExist,
-                                    "Collection has been dropped since enqueuing this range "
-                                    "deletion task. No need to delete documents.",
-                                    !collectionUuidHasChanged(nss, collection, collectionUuid));
-
-                            auto numDeleted = uassertStatusOK(deleteNextBatch(
-                                opCtx, collection, keyPattern, range, numDocsToRemovePerBatch));
-
-                            LOGV2_DEBUG(23769,
-                                        1,
-                                        "Deleted {numDeleted} documents in pass in namespace "
-                                        "{namespace} with "
-                                        "UUID  {collectionUUID} for range {range}",
-                                        "Deleted documents in pass",
-                                        "numDeleted"_attr = numDeleted,
-                                        "namespace"_attr = nss.ns(),
-                                        "collectionUUID"_attr = collectionUuid,
-                                        "range"_attr = range.toString());
-
-                            if (numDeleted > 0) {
-                                // (SERVER-62368) The range-deleter executor is mono-threaded, so
-                                // sleeping synchronously for `delayBetweenBatches` ensures that no
-                                // other batch is going to be cleared up before the expected delay.
-                                opCtx->sleepFor(delayBetweenBatches);
-                            }
-
-                            return numDeleted < numDocsToRemovePerBatch;
-                        },
-                        nss);
-                } catch (const DBException& ex) {
-                    // Errors other than those indicating stepdown and those that indicate that the
-                    // range deletion can no longer occur should be retried.
-                    auto errorCode = ex.code();
-                    if (errorCode ==
-                            ErrorCodes::
-                                RangeDeletionAbandonedBecauseCollectionWithUUIDDoesNotExist ||
-                        errorCode ==
-                            ErrorCodes::RangeDeletionAbandonedBecauseTaskDocumentDoesNotExist ||
-                        ErrorCodes::isShutdownError(errorCode) ||
-                        ErrorCodes::isNotPrimaryError(errorCode)) {
-                        return ex.toStatus();
-                    };
-                }
-            }
-            return Status::OK();
-        })
-        .ignoreValue();
+    try {
+        store.update(opCtx, query, update, WriteConcerns::kLocalWriteConcern);
+    } catch (const ExceptionFor<ErrorCodes::NoMatchingDocument>&) {
+        // The collection may have been dropped or the document could have been manually deleted
+    }
 }
 
 /**
- * Notify the secondaries that this range is being deleted. Secondaries will watch for this update,
- * and kill any queries that may depend on documents in the range -- excepting any queries with a
- * read-concern option 'ignoreChunkMigration'.
+ * Delete the range in a sequence of batches until there are no more documents to delete or deletion
+ * returns an error.
  */
-void notifySecondariesThatDeletionIsOccurring(const NamespaceString& nss,
-                                              const UUID& collectionUuid,
-                                              const ChunkRange& range) {
-    withTemporaryOperationContext(
-        [&](OperationContext* opCtx) {
-            AutoGetCollection autoAdmin(
-                opCtx, NamespaceString::kServerConfigurationNamespace, MODE_IX);
-            Helpers::upsert(opCtx,
-                            NamespaceString::kServerConfigurationNamespace.ns(),
-                            BSON("_id"
-                                 << "startRangeDeletion"
-                                 << "ns" << nss.ns() << "uuid" << collectionUuid << "min"
-                                 << range.getMin() << "max" << range.getMax()));
-        },
-        nss);
+ExecutorFuture<void> deleteRangeInBatchesWithExecutor(
+    const std::shared_ptr<executor::TaskExecutor>& executor,
+    const NamespaceString& nss,
+    const UUID& collectionUuid,
+    const BSONObj& keyPattern,
+    const ChunkRange& range,
+    const UUID& migrationId) {
+    return ExecutorFuture<void>(executor).then([=] {
+        return withTemporaryOperationContext(
+            [=](OperationContext* opCtx) {
+                return deleteRangeInBatches(
+                    opCtx, nss.db(), collectionUuid, keyPattern, range, migrationId);
+            },
+            nss);
+    });
 }
 
 void removePersistentRangeDeletionTask(const NamespaceString& nss, UUID migrationId) {
     withTemporaryOperationContext(
         [&](OperationContext* opCtx) {
-            PersistentTaskStore<RangeDeletionTask> store(opCtx,
-                                                         NamespaceString::kRangeDeletionNamespace);
+            PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
 
-            store.remove(opCtx, QUERY(RangeDeletionTask::kIdFieldName << migrationId));
+            store.remove(opCtx, BSON(RangeDeletionTask::kIdFieldName << migrationId));
         },
         nss);
 }
@@ -451,14 +309,174 @@ ExecutorFuture<void> waitForDeletionsToMajorityReplicate(
 
             // Asynchronously wait for majority write concern.
             return WaitForMajorityService::get(opCtx->getServiceContext())
-                .waitUntilMajority(clientOpTime)
+                .waitUntilMajority(clientOpTime, CancellationToken::uncancelable())
                 .thenRunOn(executor);
         },
         nss);
 }
 
+std::vector<RangeDeletionTask> getPersistentRangeDeletionTasks(OperationContext* opCtx,
+                                                               const NamespaceString& nss) {
+    std::vector<RangeDeletionTask> tasks;
+
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
+    auto query = BSON(RangeDeletionTask::kNssFieldName << nss.ns());
+
+    store.forEach(opCtx, query, [&](const RangeDeletionTask& deletionTask) {
+        tasks.push_back(std::move(deletionTask));
+        return true;
+    });
+
+    return tasks;
+}
+
 }  // namespace
 
+Status deleteRangeInBatches(OperationContext* opCtx,
+                            const DatabaseName& dbName,
+                            const UUID& collectionUuid,
+                            const BSONObj& keyPattern,
+                            const ChunkRange& range,
+                            const UUID& migrationId) {
+    bool allDocsRemoved = false;
+    // Delete all batches in this range unless a stepdown error occurs. Do not yield the
+    // executor to ensure that this range is fully deleted before another range is
+    // processed.
+    while (!allDocsRemoved) {
+        try {
+            int numDocsToRemovePerBatch = rangeDeleterBatchSize.load();
+            if (numDocsToRemovePerBatch <= 0) {
+                numDocsToRemovePerBatch = kRangeDeleterBatchSizeDefault;
+            }
+
+            Milliseconds delayBetweenBatches(rangeDeleterBatchDelayMS.load());
+
+            ensureRangeDeletionTaskStillExists(opCtx, migrationId);
+
+            markRangeDeletionTaskAsProcessing(opCtx, migrationId);
+
+            int numDeleted;
+            const auto nss = [&]() {
+                try {
+                    AutoGetCollection collection(
+                        opCtx, NamespaceStringOrUUID{dbName.toString(), collectionUuid}, MODE_IX);
+
+                    LOGV2_DEBUG(6777800,
+                                1,
+                                "Starting batch deletion",
+                                "namespace"_attr = collection.getNss(),
+                                "collectionUUID"_attr = collectionUuid,
+                                "range"_attr = redact(range.toString()),
+                                "numDocsToRemovePerBatch"_attr = numDocsToRemovePerBatch,
+                                "delayBetweenBatches"_attr = delayBetweenBatches);
+
+                    numDeleted = uassertStatusOK(deleteNextBatch(opCtx,
+                                                                 collection.getCollection(),
+                                                                 keyPattern,
+                                                                 range,
+                                                                 numDocsToRemovePerBatch));
+
+                    return collection.getNss();
+                } catch (const ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
+                    // Throw specific error code that stops range deletions in case of errors
+                    uasserted(
+                        ErrorCodes::RangeDeletionAbandonedBecauseCollectionWithUUIDDoesNotExist,
+                        "Collection has been dropped since enqueuing this range "
+                        "deletion task. No need to delete documents.");
+                }
+            }();
+
+            migrationutil::persistUpdatedNumOrphans(
+                opCtx, migrationId, collectionUuid, -numDeleted);
+
+            if (MONGO_unlikely(hangAfterDoingDeletion.shouldFail())) {
+                hangAfterDoingDeletion.pauseWhileSet(opCtx);
+            }
+
+            LOGV2_DEBUG(23769,
+                        1,
+                        "Deleted documents in pass",
+                        "numDeleted"_attr = numDeleted,
+                        "namespace"_attr = nss.ns(),
+                        "collectionUUID"_attr = collectionUuid,
+                        "range"_attr = range.toString());
+
+            if (numDeleted > 0) {
+                // (SERVER-62368) The range-deleter executor is mono-threaded, so
+                // sleeping synchronously for `delayBetweenBatches` ensures that no
+                // other batch is going to be cleared up before the expected delay.
+                opCtx->sleepFor(delayBetweenBatches);
+            }
+
+            allDocsRemoved = numDeleted < numDocsToRemovePerBatch;
+        } catch (const DBException& ex) {
+            // Errors other than those indicating stepdown and those that indicate that the
+            // range deletion can no longer occur should be retried.
+            auto errorCode = ex.code();
+            if (errorCode ==
+                    ErrorCodes::RangeDeletionAbandonedBecauseCollectionWithUUIDDoesNotExist ||
+                errorCode == ErrorCodes::RangeDeletionAbandonedBecauseTaskDocumentDoesNotExist ||
+                errorCode == ErrorCodes::KeyPatternShorterThanBound ||
+                ErrorCodes::isShutdownError(errorCode) ||
+                ErrorCodes::isNotPrimaryError(errorCode)) {
+                return ex.toStatus();
+            };
+        }
+    }
+    return Status::OK();
+}
+
+void snapshotRangeDeletionsForRename(OperationContext* opCtx,
+                                     const NamespaceString& fromNss,
+                                     const NamespaceString& toNss) {
+    // Clear out eventual snapshots associated with the target collection: always restart from a
+    // clean state in case of stepdown or primary killed.
+    PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionForRenameNamespace);
+    store.remove(opCtx, BSON(RangeDeletionTask::kNssFieldName << toNss.ns()));
+
+    auto rangeDeletionTasks = getPersistentRangeDeletionTasks(opCtx, fromNss);
+    for (auto& task : rangeDeletionTasks) {
+        // Associate task to the new namespace
+        task.setNss(toNss);
+        // Assign a new id to prevent duplicate key conflicts with the source range deletion task
+        task.setId(UUID::gen());
+        store.add(opCtx, task);
+    }
+}
+
+void restoreRangeDeletionTasksForRename(OperationContext* opCtx, const NamespaceString& nss) {
+    PersistentTaskStore<RangeDeletionTask> rangeDeletionsForRenameStore(
+        NamespaceString::kRangeDeletionForRenameNamespace);
+    PersistentTaskStore<RangeDeletionTask> rangeDeletionsStore(
+        NamespaceString::kRangeDeletionNamespace);
+
+    const auto query = BSON(RangeDeletionTask::kNssFieldName << nss.ns());
+
+    rangeDeletionsForRenameStore.forEach(opCtx, query, [&](const RangeDeletionTask& deletionTask) {
+        try {
+            rangeDeletionsStore.add(opCtx, deletionTask);
+        } catch (const ExceptionFor<ErrorCodes::DuplicateKey>&) {
+            // Task already scheduled in a previous call of this method
+        }
+        return true;
+    });
+}
+
+void deleteRangeDeletionTasksForRename(OperationContext* opCtx,
+                                       const NamespaceString& fromNss,
+                                       const NamespaceString& toNss) {
+    // Delete range deletion tasks associated to the source collection
+    PersistentTaskStore<RangeDeletionTask> rangeDeletionsStore(
+        NamespaceString::kRangeDeletionNamespace);
+    rangeDeletionsStore.remove(opCtx, BSON(RangeDeletionTask::kNssFieldName << fromNss.ns()));
+
+    // Delete already restored snapshots associated to the target collection
+    PersistentTaskStore<RangeDeletionTask> rangeDeletionsForRenameStore(
+        NamespaceString::kRangeDeletionForRenameNamespace);
+    rangeDeletionsForRenameStore.remove(opCtx,
+                                        BSON(RangeDeletionTask::kNssFieldName << toNss.ns()));
+}
+
 SharedSemiFuture<void> removeDocumentsInRange(
     const std::shared_ptr<executor::TaskExecutor>& executor,
     SemiFuture<void> waitForActiveQueriesToComplete,
@@ -466,8 +484,7 @@ SharedSemiFuture<void> removeDocumentsInRange(
     const UUID& collectionUuid,
     const BSONObj& keyPattern,
     const ChunkRange& range,
-    boost::optional<UUID> migrationId,
-    int numDocsToRemovePerBatch,
+    const UUID& migrationId,
     Seconds delayForActiveQueriesOnSecondariesToComplete) {
     return std::move(waitForActiveQueriesToComplete)
         .thenRunOn(executor)
@@ -477,31 +494,7 @@ SharedSemiFuture<void> removeDocumentsInRange(
             invariant(s.isOK());
         })
         .then([=]() mutable {
-            auto shouldSkipDeletion = false;
-            withTemporaryOperationContext(
-                [&](OperationContext* opCtx) {
-                    AutoGetCollection autoColl(opCtx, nss, MODE_IS);
-                    auto* const collection = autoColl.getCollection();
-
-                    if (!collection) {
-                        LOGV2(5774300,
-                              "Skip scheduling deletion of any documents in {namespace} "
-                              "range {range} since the collection does not exist",
-                              "Skip scheduling deletion of any documents in range "
-                              "since the collection does not exist",
-                              "namespace"_attr = nss.ns(),
-                              "range"_attr = redact(range.toString()));
-                        shouldSkipDeletion = true;
-                    }
-                },
-                nss);
-
-            if (shouldSkipDeletion) {
-                return ExecutorFuture(executor);
-            }
-
             suspendRangeDeletion.pauseWhileSet();
-
             // Wait for possibly ongoing queries on secondaries to complete.
             return sleepUntil(executor,
                               executor->now() + delayForActiveQueriesOnSecondariesToComplete);
@@ -509,23 +502,12 @@ SharedSemiFuture<void> removeDocumentsInRange(
         .then([=]() mutable {
             LOGV2_DEBUG(23772,
                         1,
-                        "Beginning deletion of any documents in {namespace} range {range} with  "
-                        "numDocsToRemovePerBatch {numDocsToRemovePerBatch}",
                         "Beginning deletion of documents",
                         "namespace"_attr = nss.ns(),
-                        "range"_attr = redact(range.toString()),
-                        "numDocsToRemovePerBatch"_attr = numDocsToRemovePerBatch);
+                        "range"_attr = redact(range.toString()));
 
-            notifySecondariesThatDeletionIsOccurring(nss, collectionUuid, range);
-
-            return deleteRangeInBatches(executor,
-                                        nss,
-                                        collectionUuid,
-                                        keyPattern,
-                                        range,
-                                        migrationId,
-                                        numDocsToRemovePerBatch,
-                                        Milliseconds(rangeDeleterBatchDelayMS.load()))
+            return deleteRangeInBatchesWithExecutor(
+                       executor, nss, collectionUuid, keyPattern, range, migrationId)
                 .onCompletion([=](Status s) {
                     if (!s.isOK() &&
                         s.code() !=
@@ -570,16 +552,15 @@ SharedSemiFuture<void> removeDocumentsInRange(
                 return Status::OK();
             }
 
-            if (!migrationId ||
-                (!s.isOK() &&
-                 s.code() !=
-                     ErrorCodes::RangeDeletionAbandonedBecauseCollectionWithUUIDDoesNotExist)) {
+            if (!s.isOK() &&
+                s.code() !=
+                    ErrorCodes::RangeDeletionAbandonedBecauseCollectionWithUUIDDoesNotExist) {
                 // Propagate any errors to callers waiting on the result.
                 return s;
             }
 
             try {
-                removePersistentRangeDeletionTask(nss, std::move(*migrationId));
+                removePersistentRangeDeletionTask(nss, migrationId);
             } catch (const DBException& e) {
                 LOGV2_ERROR(23770,
                             "Failed to delete range deletion task for range {range} in collection "
