@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
@@ -37,19 +36,25 @@
 
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/client/connection_string.h"
+#include "mongo/db/cancelable_operation_context.h"
 #include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
-#include "mongo/db/logical_session_id.h"
+#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/ops/write_ops_retryability.h"
 #include "mongo/db/repl/oplog.h"
 #include "mongo/db/repl/oplog_entry.h"
 #include "mongo/db/s/migration_session_id.h"
-#include "mongo/db/session_catalog_mongod.h"
-#include "mongo/db/transaction_participant.h"
+#include "mongo/db/s/session_catalog_migration.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/session_catalog_mongod.h"
+#include "mongo/db/shard_id.h"
+#include "mongo/db/transaction/transaction_participant.h"
 #include "mongo/db/write_concern.h"
+#include "mongo/logv2/log.h"
 #include "mongo/logv2/redaction.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
-#include "mongo/s/shard_id.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 namespace mongo {
 namespace {
@@ -57,14 +62,13 @@ namespace {
 MONGO_FAIL_POINT_DEFINE(interruptBeforeProcessingPrePostImageOriginatingOp);
 
 const auto kOplogField = "oplog";
-const auto kWaitsForNewOplogField = "waitsForNewOplog";
 const WriteConcernOptions kMajorityWC(WriteConcernOptions::kMajority,
                                       WriteConcernOptions::SyncMode::UNSET,
                                       Milliseconds(0));
 
 struct ProcessOplogResult {
     LogicalSessionId sessionId;
-    TxnNumber txnNum;
+    TxnNumber txnNum{kUninitializedTxnNumber};
 
     repl::OpTime oplogTime;
     bool isPrePostImage = false;
@@ -115,7 +119,7 @@ void setPrePostImageTs(const ProcessOplogResult& lastResult, repl::MutableOplogE
     // the appropriate no-op. This code on the destination patches up the CRUD operation oplog entry
     // to look like the classic format.
     if (entry->getNeedsRetryImage()) {
-        switch (entry->getNeedsRetryImage().get()) {
+        switch (entry->getNeedsRetryImage().value()) {
             case repl::RetryImageEnum::kPreImage:
                 entry->setPreImageOpTime({repl::OpTime()});
                 break;
@@ -160,7 +164,7 @@ repl::MutableOplogEntry parseOplog(const BSONObj& oplogBSON) {
     uassert(ErrorCodes::UnsupportedFormat,
             str::stream() << "oplog with opTime " << oplogEntry.getTimestamp().toString()
                           << " does not have stmtId: " << redact(oplogBSON),
-            oplogEntry.getStatementId());
+            !oplogEntry.getStatementIds().empty());
 
     return oplogEntry;
 }
@@ -199,7 +203,9 @@ BSONObj getNextSessionOplogBatch(OperationContext* opCtx,
  * information. The new oplogEntry will also link to prePostImageTs if not null.
  */
 ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
-                                       const ProcessOplogResult& lastResult) {
+                                       const ProcessOplogResult& lastResult,
+                                       ServiceContext* serviceContext,
+                                       CancellationToken cancellationToken) {
     auto oplogEntry = parseOplog(oplogBSON);
 
     ProcessOplogResult result;
@@ -208,13 +214,19 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
 
     if (oplogEntry.getOpType() == repl::OpTypeEnum::kNoop) {
         // Note: Oplog is already no-op type, no need to nest.
-        // There are two types of type 'n' oplog format expected here:
+        // There are three types of type 'n' oplog format expected here:
         // (1) Oplog entries that has been transformed by a previous migration into a
         //     nested oplog. In this case, o field contains {$sessionMigrateInfo: 1}
         //     and o2 field contains the details of the original oplog.
         // (2) Oplog entries that contains the pre/post-image information of a
         //     findAndModify operation. In this case, o field contains the relevant info
         //     and o2 will be empty.
+        // (3) Oplog entries that are a dead sentinel, which the donor sent over as the replacement
+        //     for a prepare oplog entry or unprepared transaction commit oplog entry.
+        // (4) Oplog entries that are a WouldChangeOwningShard sentinel entry, used for making
+        //     retries of a WouldChangeOwningShard update or findAndModify fail with
+        //     IncompleteTransactionHistory. In this case, the o field is non-empty and the o2
+        //     field is an empty BSONObj.
 
         BSONObj object2;
         if (oplogEntry.getObject2()) {
@@ -223,7 +235,7 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
             oplogEntry.setObject2(object2);
         }
 
-        if (object2.isEmpty()) {
+        if (object2.isEmpty() && !isWouldChangeOwningShardSentinelOplogEntry(oplogEntry)) {
             result.isPrePostImage = true;
 
             uassert(40632,
@@ -237,9 +249,11 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
         oplogEntry.setObject2(oplogBSON);  // TODO: strip redundant info?
     }
 
-    const auto stmtId = *oplogEntry.getStatementId();
+    const auto stmtIds = oplogEntry.getStatementIds();
 
-    auto uniqueOpCtx = cc().makeOperationContext();
+    auto executor = Grid::get(serviceContext)->getExecutorPool()->getFixedExecutor();
+    auto uniqueOpCtx =
+        CancelableOperationContext(cc().makeOperationContext(), cancellationToken, executor);
     auto opCtx = uniqueOpCtx.get();
     opCtx->setLogicalSessionId(result.sessionId);
     opCtx->setTxnNumber(result.txnNum);
@@ -247,8 +261,11 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
     auto txnParticipant = TransactionParticipant::get(opCtx);
 
     try {
-        txnParticipant.beginOrContinue(opCtx, result.txnNum, boost::none, boost::none);
-        if (txnParticipant.checkStatementExecuted(opCtx, stmtId)) {
+        txnParticipant.beginOrContinue(opCtx,
+                                       {result.txnNum},
+                                       boost::none /* autocommit */,
+                                       boost::none /* startTransaction */);
+        if (txnParticipant.checkStatementExecuted(opCtx, stmtIds.front())) {
             // Skip the incoming statement because it has already been logged locally
             return lastResult;
         }
@@ -259,17 +276,22 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
             return lastResult;
         }
 
-        if (stmtId == kIncompleteHistoryStmtId) {
+        if (stmtIds.front() == kIncompleteHistoryStmtId) {
             // No need to log entries for transactions whose history has been truncated
+            invariant(stmtIds.size() == 1);
             return lastResult;
         }
 
         throw;
     }
 
-    if (!result.isPrePostImage)
-        oplogEntry.setObject(
-            BSON(SessionCatalogMigrationDestination::kSessionMigrateOplogTag << 1));
+    if (!result.isPrePostImage && !isWouldChangeOwningShardSentinelOplogEntry(oplogEntry)) {
+        // Do not overwrite the "o" field if this is a pre/post image oplog entry. Also do not
+        // overwrite it if this is a WouldChangeOwningShard sentinel oplog entry since it contains
+        // a special BSONObj used for making retries fail with an IncompleteTransactionHistory
+        // error.
+        oplogEntry.setObject(SessionCatalogMigration::kSessionOplogTag);
+    }
     setPrePostImageTs(lastResult, &oplogEntry);
     oplogEntry.setPrevWriteOpTimeInTransaction(txnParticipant.getLastWriteOpTime());
 
@@ -284,12 +306,11 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
         NamespaceString::kSessionTransactionsTableNamespace.ns(),
         [&] {
             // Need to take global lock here so repl::logOp will not unlock it and trigger the
-            // invariant that disallows unlocking global lock while inside a WUOW. Grab a DBLock
-            // here instead of plain GlobalLock to make sure the MMAPV1 flush lock will be
-            // lock/unlocked correctly. Take the transaction table db lock to ensure the same lock
-            // ordering with normal replicated updates to the table.
+            // invariant that disallows unlocking global lock while inside a WUOW. Take the
+            // transaction table db lock to ensure the same lock ordering with normal replicated
+            // updates to the table.
             Lock::DBLock lk(
-                opCtx, NamespaceString::kSessionTransactionsTableNamespace.db(), MODE_IX);
+                opCtx, NamespaceString::kSessionTransactionsTableNamespace.dbName(), MODE_IX);
             WriteUnitOfWork wunit(opCtx);
 
             result.oplogTime = repl::logOp(opCtx, &oplogEntry);
@@ -307,9 +328,17 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
                 sessionTxnRecord.setSessionId(result.sessionId);
                 sessionTxnRecord.setTxnNum(result.txnNum);
                 sessionTxnRecord.setLastWriteOpTime(oplogOpTime);
+
+                // Use the same wallTime as oplog since SessionUpdateTracker looks at the oplog
+                // entry wallTime when replicating.
                 sessionTxnRecord.setLastWriteDate(oplogEntry.getWallClockTime());
+
+                if (isInternalSessionForRetryableWrite(result.sessionId)) {
+                    sessionTxnRecord.setParentSessionId(*getParentSessionId(result.sessionId));
+                }
+
                 // We do not migrate transaction oplog entries so don't set the txn state.
-                txnParticipant.onMigrateCompletedOnPrimary(opCtx, {stmtId}, sessionTxnRecord);
+                txnParticipant.onRetryableWriteCloningCompleted(opCtx, stmtIds, sessionTxnRecord);
             }
 
             wunit.commit();
@@ -320,11 +349,15 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
 
 }  // namespace
 
-const char SessionCatalogMigrationDestination::kSessionMigrateOplogTag[] = "$sessionMigrateInfo";
-
 SessionCatalogMigrationDestination::SessionCatalogMigrationDestination(
-    ShardId fromShard, MigrationSessionId migrationSessionId)
-    : _fromShard(std::move(fromShard)), _migrationSessionId(std::move(migrationSessionId)) {}
+    NamespaceString nss,
+    ShardId fromShard,
+    MigrationSessionId migrationSessionId,
+    CancellationToken cancellationToken)
+    : _nss(std::move(nss)),
+      _fromShard(std::move(fromShard)),
+      _migrationSessionId(std::move(migrationSessionId)),
+      _cancellationToken(std::move(cancellationToken)) {}
 
 SessionCatalogMigrationDestination::~SessionCatalogMigrationDestination() {
     if (_thread.joinable()) {
@@ -338,7 +371,6 @@ void SessionCatalogMigrationDestination::start(ServiceContext* service) {
         stdx::lock_guard<Latch> lk(_mutex);
         invariant(_state == State::NotStarted);
         _state = State::Migrating;
-        _isStateChanged.notify_all();
     }
 
     _thread = stdx::thread([=] {
@@ -362,7 +394,6 @@ void SessionCatalogMigrationDestination::finish() {
     stdx::lock_guard<Latch> lk(_mutex);
     if (_state != State::ErrorOccurred) {
         _state = State::Committing;
-        _isStateChanged.notify_all();
     }
 }
 
@@ -385,8 +416,13 @@ void SessionCatalogMigrationDestination::join() {
  * 6. Wait for writes to be committed to majority of the replica set.
  */
 void SessionCatalogMigrationDestination::_retrieveSessionStateFromSource(ServiceContext* service) {
-    Client::initKillableThread("sessionCatalogMigrationProducer-" + _migrationSessionId.toString(),
-                               service);
+    Client::initThread(
+        "sessionCatalogMigrationProducer-" + _migrationSessionId.toString(), service, nullptr);
+    auto client = Client::getCurrent();
+    {
+        stdx::lock_guard lk(*client);
+        client->setSystemOperationKillableByStepdown(lk);
+    }
 
     bool oplogDrainedAfterCommiting = false;
     ProcessOplogResult lastResult;
@@ -403,12 +439,13 @@ void SessionCatalogMigrationDestination::_retrieveSessionStateFromSource(Service
         BSONObj nextBatch;
         BSONArray oplogArray;
         {
-            auto uniqueCtx = cc().makeOperationContext();
+            auto executor = Grid::get(service)->getExecutorPool()->getFixedExecutor();
+            auto uniqueCtx = CancelableOperationContext(
+                cc().makeOperationContext(), _cancellationToken, executor);
             auto opCtx = uniqueCtx.get();
 
             nextBatch = getNextSessionOplogBatch(opCtx, _fromShard, _migrationSessionId);
             oplogArray = BSONArray{nextBatch[kOplogField].Obj()};
-            const auto donorWaitsForNewOplog = nextBatch[kWaitsForNewOplogField].trueValue();
 
             if (oplogArray.isEmpty()) {
                 {
@@ -420,6 +457,13 @@ void SessionCatalogMigrationDestination::_retrieveSessionStateFromSource(Service
                         // this depleted the buffer from the source shard and receiving the commit
                         // command.
                         if (oplogDrainedAfterCommiting) {
+                            LOGV2(5087100,
+                                  "Recipient finished draining oplog entries for retryable writes "
+                                  "and transactions from donor again after receiving "
+                                  "_recvChunkCommit",
+                                  "namespace"_attr = _nss,
+                                  "migrationSessionId"_attr = _migrationSessionId,
+                                  "fromShard"_attr = _fromShard);
                             break;
                         }
 
@@ -431,25 +475,22 @@ void SessionCatalogMigrationDestination::_retrieveSessionStateFromSource(Service
                 uassertStatusOK(
                     waitForWriteConcern(opCtx, lastResult.oplogTime, kMajorityWC, &unusedWCResult));
 
-                // We depleted the buffer at least once, transition to ready for commit.
                 {
                     stdx::lock_guard<Latch> lk(_mutex);
                     // Note: only transition to "ready to commit" if state is not error/force stop.
                     if (_state == State::Migrating) {
+                        // We depleted the buffer at least once, transition to ready for commit.
+                        LOGV2(5087101,
+                              "Recipient finished draining oplog entries for retryable writes and "
+                              "transactions from donor for the first time, before receiving "
+                              "_recvChunkCommit",
+                              "namespace"_attr = _nss,
+                              "migrationSessionId"_attr = _migrationSessionId,
+                              "fromShard"_attr = _fromShard);
                         _state = State::ReadyToCommit;
-                        _isStateChanged.notify_all();
                     }
                 }
 
-                // TODO: SERVER-40187 Completely remove after v4.2. donorWaitsForNewOplog is a
-                // 'feature flag' indicating that the donor does block until there are new oplog
-                // to return so we don't need to sleep here.
-                if (!donorWaitsForNewOplog && lastOpTimeWaited == lastResult.oplogTime) {
-                    // We got an empty result at least twice in a row from the source shard so space
-                    // it out a little bit so we don't hammer the shard
-                    opCtx->sleepFor(Milliseconds(200));
-                }
-
                 lastOpTimeWaited = lastResult.oplogTime;
             }
         }
@@ -467,11 +508,8 @@ void SessionCatalogMigrationDestination::_retrieveSessionStateFromSource(Service
                         !oplogEntry["preImageOpTime"].eoo() || !oplogEntry["postImageOpTime"].eoo();
                 });
             try {
-                lastResult = processSessionOplog(oplogEntry, lastResult);
-            } catch (const ExceptionFor<ErrorCodes::ConfigurationInProgress>&) {
-                // This means that the server has a newer txnNumber than the oplog being
-                // migrated, so just skip it
-                continue;
+                lastResult =
+                    processSessionOplog(oplogEntry, lastResult, service, _cancellationToken);
             } catch (const ExceptionFor<ErrorCodes::TransactionTooOld>&) {
                 // This means that the server has a newer txnNumber than the oplog being
                 // migrated, so just skip it
@@ -481,14 +519,17 @@ void SessionCatalogMigrationDestination::_retrieveSessionStateFromSource(Service
     }
 
     WriteConcernResult unusedWCResult;
-    auto uniqueOpCtx = cc().makeOperationContext();
+
+    auto executor = Grid::get(service)->getExecutorPool()->getFixedExecutor();
+    auto uniqueOpCtx =
+        CancelableOperationContext(cc().makeOperationContext(), _cancellationToken, executor);
+
     uassertStatusOK(
         waitForWriteConcern(uniqueOpCtx.get(), lastResult.oplogTime, kMajorityWC, &unusedWCResult));
 
     {
         stdx::lock_guard<Latch> lk(_mutex);
         _state = State::Done;
-        _isStateChanged.notify_all();
     }
 }
 
@@ -498,11 +539,16 @@ std::string SessionCatalogMigrationDestination::getErrMsg() {
 }
 
 void SessionCatalogMigrationDestination::_errorOccurred(StringData errMsg) {
+    LOGV2(5087102,
+          "Recipient failed to copy oplog entries for retryable writes and transactions from donor",
+          "namespace"_attr = _nss,
+          "migrationSessionId"_attr = _migrationSessionId,
+          "fromShard"_attr = _fromShard,
+          "error"_attr = errMsg);
+
     stdx::lock_guard<Latch> lk(_mutex);
     _state = State::ErrorOccurred;
     _errMsg = errMsg.toString();
-
-    _isStateChanged.notify_all();
 }
 
 SessionCatalogMigrationDestination::State SessionCatalogMigrationDestination::getState() {
