@@ -27,29 +27,24 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/s/shard_metadata_util.h"
 
-#include <memory>
-
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/ops/write_ops.h"
+#include "mongo/db/s/type_shard_collection.h"
+#include "mongo/db/s/type_shard_database.h"
 #include "mongo/db/write_concern_options.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/unique_message.h"
 #include "mongo/s/catalog/type_chunk.h"
-#include "mongo/s/catalog/type_shard_collection.h"
-#include "mongo/s/catalog/type_shard_database.h"
-#include "mongo/s/chunk_version.h"
+#include "mongo/s/catalog/type_collection.h"
 #include "mongo/s/write_ops/batched_command_response.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
+
 namespace mongo {
 namespace shardmetadatautil {
-
 namespace {
 
 const WriteConcernOptions kLocalWriteConcern(1,
@@ -70,6 +65,14 @@ Status getStatusFromWriteCommandResponse(const BSONObj& commandResult) {
     return batchResponse.toStatus();
 }
 
+Status setPersistedRefreshFlags(OperationContext* opCtx, const NamespaceString& nss) {
+    return updateShardCollectionsEntry(
+        opCtx,
+        BSON(ShardCollectionType::kNssFieldName << nss.ns()),
+        BSON("$set" << BSON(ShardCollectionType::kRefreshingFieldName << true)),
+        false /*upsert*/);
+}
+
 }  // namespace
 
 QueryAndSort createShardChunkDiffQuery(const ChunkVersion& collectionVersion) {
@@ -78,12 +81,12 @@ QueryAndSort createShardChunkDiffQuery(const ChunkVersion& collectionVersion) {
 }
 
 bool RefreshState::operator==(const RefreshState& other) const {
-    return (other.epoch == epoch) && (other.refreshing == refreshing) &&
-        (other.lastRefreshedCollectionVersion == lastRefreshedCollectionVersion);
+    return generation.isSameCollection(other.generation) && (refreshing == other.refreshing) &&
+        (lastRefreshedCollectionVersion == other.lastRefreshedCollectionVersion);
 }
 
 std::string RefreshState::toString() const {
-    return str::stream() << "epoch: " << epoch
+    return str::stream() << "generation: " << generation.toString()
                          << ", refreshing: " << (refreshing ? "true" : "false")
                          << ", lastRefreshedCollectionVersion: "
                          << lastRefreshedCollectionVersion.toString();
@@ -95,13 +98,13 @@ Status unsetPersistedRefreshFlags(OperationContext* opCtx,
     // Set 'refreshing' to false and update the last refreshed collection version.
     BSONObjBuilder updateBuilder;
     updateBuilder.append(ShardCollectionType::kRefreshingFieldName, false);
-    updateBuilder.appendTimestamp(ShardCollectionType::kLastRefreshedCollectionVersionFieldName,
-                                  refreshedVersion.toLong());
+    updateBuilder.appendTimestamp(
+        ShardCollectionType::kLastRefreshedCollectionMajorMinorVersionFieldName,
+        refreshedVersion.toLong());
 
     return updateShardCollectionsEntry(opCtx,
                                        BSON(ShardCollectionType::kNssFieldName << nss.ns()),
-                                       updateBuilder.obj(),
-                                       BSONObj(),
+                                       BSON("$set" << updateBuilder.obj()),
                                        false /*upsert*/);
 }
 
@@ -124,7 +127,7 @@ StatusWith<RefreshState> getPersistedRefreshFlags(OperationContext* opCtx,
         invariant(!entry.getLastRefreshedCollectionVersion());
     }
 
-    return RefreshState{entry.getEpoch(),
+    return RefreshState{CollectionGeneration(entry.getEpoch(), entry.getTimestamp()),
                         // If the refreshing field has not yet been added, this means that the first
                         // refresh has started, but no chunks have ever yet been applied, around
                         // which these flags are set. So default to refreshing true because the
@@ -132,18 +135,17 @@ StatusWith<RefreshState> getPersistedRefreshFlags(OperationContext* opCtx,
                         entry.getRefreshing() ? *entry.getRefreshing() : true,
                         entry.getLastRefreshedCollectionVersion()
                             ? *entry.getLastRefreshedCollectionVersion()
-                            : ChunkVersion(0, 0, entry.getEpoch())};
+                            : ChunkVersion({entry.getEpoch(), entry.getTimestamp()}, {0, 0})};
 }
 
 StatusWith<ShardCollectionType> readShardCollectionsEntry(OperationContext* opCtx,
                                                           const NamespaceString& nss) {
-
-    Query fullQuery(BSON(ShardCollectionType::kNssFieldName << nss.ns()));
-
     try {
         DBDirectClient client(opCtx);
-        std::unique_ptr<DBClientCursor> cursor =
-            client.query(NamespaceString::kShardConfigCollectionsNamespace, fullQuery, 1);
+        FindCommandRequest findRequest{NamespaceString::kShardConfigCollectionsNamespace};
+        findRequest.setFilter(BSON(ShardCollectionType::kNssFieldName << nss.ns()));
+        findRequest.setLimit(1);
+        std::unique_ptr<DBClientCursor> cursor = client.find(std::move(findRequest));
         if (!cursor) {
             return Status(ErrorCodes::OperationFailed,
                           str::stream() << "Failed to establish a cursor for reading "
@@ -158,7 +160,7 @@ StatusWith<ShardCollectionType> readShardCollectionsEntry(OperationContext* opCt
         }
 
         BSONObj document = cursor->nextSafe();
-        return ShardCollectionType::fromBSON(document);
+        return ShardCollectionType(document);
     } catch (const DBException& ex) {
         return ex.toStatus(str::stream() << "Failed to read the '" << nss.ns()
                                          << "' entry locally from config.collections");
@@ -166,12 +168,12 @@ StatusWith<ShardCollectionType> readShardCollectionsEntry(OperationContext* opCt
 }
 
 StatusWith<ShardDatabaseType> readShardDatabasesEntry(OperationContext* opCtx, StringData dbName) {
-    Query fullQuery(BSON(ShardDatabaseType::name() << dbName.toString()));
-
     try {
         DBDirectClient client(opCtx);
-        std::unique_ptr<DBClientCursor> cursor =
-            client.query(NamespaceString::kShardConfigDatabasesNamespace, fullQuery, 1);
+        FindCommandRequest findRequest{NamespaceString::kShardConfigDatabasesNamespace};
+        findRequest.setFilter(BSON(ShardDatabaseType::kNameFieldName << dbName.toString()));
+        findRequest.setLimit(1);
+        std::unique_ptr<DBClientCursor> cursor = client.find(std::move(findRequest));
         if (!cursor) {
             return Status(ErrorCodes::OperationFailed,
                           str::stream() << "Failed to establish a cursor for reading "
@@ -186,49 +188,35 @@ StatusWith<ShardDatabaseType> readShardDatabasesEntry(OperationContext* opCtx, S
         }
 
         BSONObj document = cursor->nextSafe();
-        auto statusWithDatabaseEntry = ShardDatabaseType::fromBSON(document);
-        if (!statusWithDatabaseEntry.isOK()) {
-            return statusWithDatabaseEntry.getStatus();
-        }
-
-        return statusWithDatabaseEntry.getValue();
+        return ShardDatabaseType(document);
     } catch (const DBException& ex) {
-        return ex.toStatus(str::stream() << "Failed to read the '" << dbName.toString()
-                                         << "' entry locally from config.databases");
+        return ex.toStatus(str::stream()
+                           << "Failed to read the '" << dbName.toString() << "' entry locally from "
+                           << NamespaceString::kShardConfigDatabasesNamespace);
     }
 }
 
 Status updateShardCollectionsEntry(OperationContext* opCtx,
                                    const BSONObj& query,
                                    const BSONObj& update,
-                                   const BSONObj& inc,
                                    const bool upsert) {
     invariant(query.hasField("_id"));
     if (upsert) {
         // If upserting, this should be an update from the config server that does not have shard
         // refresh / migration inc signal information.
-        invariant(!update.hasField(ShardCollectionType::kLastRefreshedCollectionVersionFieldName));
-        invariant(inc.isEmpty());
+        invariant(!update.hasField(
+            ShardCollectionType::kLastRefreshedCollectionMajorMinorVersionFieldName));
     }
 
     try {
         DBDirectClient client(opCtx);
-
-        BSONObjBuilder builder;
-        if (!update.isEmpty()) {
-            // Want to modify the document if it already exists, not replace it.
-            builder.append("$set", update);
-        }
-        if (!inc.isEmpty()) {
-            builder.append("$inc", inc);
-        }
-
         auto commandResponse = client.runCommand([&] {
-            write_ops::Update updateOp(NamespaceString::kShardConfigCollectionsNamespace);
+            write_ops::UpdateCommandRequest updateOp(
+                NamespaceString::kShardConfigCollectionsNamespace);
             updateOp.setUpdates({[&] {
                 write_ops::UpdateOpEntry entry;
                 entry.setQ(query);
-                entry.setU(builder.obj());
+                entry.setU(write_ops::UpdateModification::parseFromClassicUpdate(update));
                 entry.setUpsert(upsert);
                 return entry;
             }()});
@@ -267,11 +255,12 @@ Status updateShardDatabasesEntry(OperationContext* opCtx,
         }
 
         auto commandResponse = client.runCommand([&] {
-            write_ops::Update updateOp(NamespaceString::kShardConfigDatabasesNamespace);
+            write_ops::UpdateCommandRequest updateOp(
+                NamespaceString::kShardConfigDatabasesNamespace);
             updateOp.setUpdates({[&] {
                 write_ops::UpdateOpEntry entry;
                 entry.setQ(query);
-                entry.setU(builder.obj());
+                entry.setU(write_ops::UpdateModification::parseFromClassicUpdate(builder.obj()));
                 entry.setUpsert(upsert);
                 return entry;
             }()});
@@ -290,26 +279,29 @@ StatusWith<std::vector<ChunkType>> readShardChunks(OperationContext* opCtx,
                                                    const BSONObj& query,
                                                    const BSONObj& sort,
                                                    boost::optional<long long> limit,
-                                                   const OID& epoch) {
-    try {
-        Query fullQuery(query);
-        fullQuery.sort(sort);
+                                                   const OID& epoch,
+                                                   const Timestamp& timestamp) {
+    const auto chunksNss = NamespaceString{ChunkType::ShardNSPrefix + nss.ns()};
 
+    try {
         DBDirectClient client(opCtx);
 
-        const std::string chunkMetadataNs = ChunkType::ShardNSPrefix + nss.ns();
-
-        std::unique_ptr<DBClientCursor> cursor =
-            client.query(NamespaceString(chunkMetadataNs), fullQuery, limit.get_value_or(0));
+        FindCommandRequest findRequest{chunksNss};
+        findRequest.setFilter(query);
+        findRequest.setSort(sort);
+        if (limit) {
+            findRequest.setLimit(*limit);
+        }
+        std::unique_ptr<DBClientCursor> cursor = client.find(std::move(findRequest));
         uassert(ErrorCodes::OperationFailed,
-                str::stream() << "Failed to establish a cursor for reading " << chunkMetadataNs
+                str::stream() << "Failed to establish a cursor for reading " << chunksNss.ns()
                               << " from local storage",
                 cursor);
 
         std::vector<ChunkType> chunks;
         while (cursor->more()) {
             BSONObj document = cursor->nextSafe().getOwned();
-            auto statusWithChunk = ChunkType::fromShardBSON(document, epoch);
+            auto statusWithChunk = ChunkType::parseFromShardBSON(document, epoch, timestamp);
             if (!statusWithChunk.isOK()) {
                 return statusWithChunk.getStatus().withContext(
                     str::stream() << "Failed to parse chunk '" << document.toString() << "'");
@@ -330,19 +322,19 @@ Status updateShardChunks(OperationContext* opCtx,
                          const OID& currEpoch) {
     invariant(!chunks.empty());
 
-    NamespaceString chunkMetadataNss(ChunkType::ShardNSPrefix + nss.ns());
+    const auto chunksNss = NamespaceString{ChunkType::ShardNSPrefix + nss.ns()};
 
     try {
         DBDirectClient client(opCtx);
 
         // This may be the first update, so the first opportunity to create an index.
         // If the index already exists, this is a no-op.
-        client.createIndex(chunkMetadataNss.ns(), BSON(ChunkType::lastmod() << 1));
+        client.createIndex(chunksNss.ns(), BSON(ChunkType::lastmod() << 1));
 
         /**
          * Here are examples of the operations that can happen on the config server to update
-         * the config.chunks collection. 'chunks' only includes the chunks that result from the
-         * operations, which can be read from the config server, not any that were removed, so
+         * the config.cache.chunks collection. 'chunks' only includes the chunks that result from
+         * the operations, which can be read from the config server, not any that were removed, so
          * we must delete any chunks that overlap with the new 'chunks'.
          *
          * CollectionVersion = 10.3
@@ -369,7 +361,7 @@ Status updateShardChunks(OperationContext* opCtx,
             //
             // query: { "_id" : {"$gte": chunk.min, "$lt": chunk.max}}
             auto deleteCommandResponse = client.runCommand([&] {
-                write_ops::Delete deleteOp(chunkMetadataNss);
+                write_ops::DeleteCommandRequest deleteOp(chunksNss);
                 deleteOp.setDeletes({[&] {
                     write_ops::DeleteOpEntry entry;
                     entry.setQ(BSON(ChunkType::minShardID
@@ -384,7 +376,7 @@ Status updateShardChunks(OperationContext* opCtx,
 
             // Now the document can be expected to cleanly insert without overlap
             auto insertCommandResponse = client.runCommand([&] {
-                write_ops::Insert insertOp(chunkMetadataNss);
+                write_ops::InsertCommandRequest insertOp(chunksNss);
                 insertOp.setDocuments({chunk.toShardBSON()});
                 return insertOp.serialize({});
             }());
@@ -400,10 +392,12 @@ Status updateShardChunks(OperationContext* opCtx,
 
 Status dropChunksAndDeleteCollectionsEntry(OperationContext* opCtx, const NamespaceString& nss) {
     try {
-        DBDirectClient client(opCtx);
 
+        // Delete the collection entry from 'config.cache.collections'.
+        DBDirectClient client(opCtx);
         auto deleteCommandResponse = client.runCommand([&] {
-            write_ops::Delete deleteOp(NamespaceString::kShardConfigCollectionsNamespace);
+            write_ops::DeleteCommandRequest deleteOp(
+                NamespaceString::kShardConfigCollectionsNamespace);
             deleteOp.setDeletes({[&] {
                 write_ops::DeleteOpEntry entry;
                 entry.setQ(BSON(ShardCollectionType::kNssFieldName << nss.ns()));
@@ -415,33 +409,26 @@ Status dropChunksAndDeleteCollectionsEntry(OperationContext* opCtx, const Namesp
         uassertStatusOK(
             getStatusFromWriteCommandResponse(deleteCommandResponse->getCommandReply()));
 
-        // Drop the corresponding config.chunks.ns collection
-        BSONObj result;
-        if (!client.dropCollection(
-                ChunkType::ShardNSPrefix + nss.ns(), kLocalWriteConcern, &result)) {
-            Status status = getStatusFromCommandResult(result);
-            if (status != ErrorCodes::NamespaceNotFound) {
-                uassertStatusOK(status);
-            }
-        }
-
-        LOGV2_DEBUG(
-            22090,
-            1,
-            "Successfully cleared persisted chunk metadata and collection entry for collection "
-            "{namespace}",
-            "Successfully cleared persisted chunk metadata and collection entry",
-            "namespace"_attr = nss);
-        return Status::OK();
+        // Drop the 'config.cache.chunks.<ns>' collection.
+        dropChunks(opCtx, nss);
     } catch (const DBException& ex) {
+        LOGV2_ERROR(5966301,
+                    "Failed to drop persisted chunk metadata and collection entry",
+                    "namespace"_attr = nss,
+                    "error"_attr = redact(ex.toStatus()));
+
         return ex.toStatus();
     }
+
+    LOGV2(5966302, "Dropped persisted chunk metadata and collection entry", "namespace"_attr = nss);
+
+    return Status::OK();
 }
 
 void dropChunks(OperationContext* opCtx, const NamespaceString& nss) {
     DBDirectClient client(opCtx);
 
-    // Drop the config.chunks collection associated with namespace 'nss'.
+    // Drop the 'config.cache.chunks.<ns>' collection.
     BSONObj result;
     if (!client.dropCollection(ChunkType::ShardNSPrefix + nss.ns(), kLocalWriteConcern, &result)) {
         auto status = getStatusFromCommandResult(result);
@@ -450,21 +437,18 @@ void dropChunks(OperationContext* opCtx, const NamespaceString& nss) {
         }
     }
 
-    LOGV2_DEBUG(22091,
-                1,
-                "Successfully cleared persisted chunk metadata for collection",
-                "namespace"_attr = nss);
+    LOGV2_DEBUG(22091, 1, "Dropped persisted chunk metadata", "namespace"_attr = nss);
 }
 
 Status deleteDatabasesEntry(OperationContext* opCtx, StringData dbName) {
     try {
         DBDirectClient client(opCtx);
-
         auto deleteCommandResponse = client.runCommand([&] {
-            write_ops::Delete deleteOp(NamespaceString::kShardConfigDatabasesNamespace);
+            write_ops::DeleteCommandRequest deleteOp(
+                NamespaceString::kShardConfigDatabasesNamespace);
             deleteOp.setDeletes({[&] {
                 write_ops::DeleteOpEntry entry;
-                entry.setQ(BSON(ShardDatabaseType::name << dbName.toString()));
+                entry.setQ(BSON(ShardDatabaseType::kNameFieldName << dbName.toString()));
                 entry.setMulti(false);
                 return entry;
             }()});
