@@ -27,15 +27,12 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/s/shard_server_catalog_cache_loader.h"
-
-#include "mongo/db/s/catalog_cache_loader_mock.h"
+#include "mongo/db/s/shard_server_test_fixture.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog/type_collection.h"
-#include "mongo/s/shard_server_test_fixture.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/s/catalog_cache_loader_mock.h"
+#include "mongo/s/type_collection_common_types_gen.h"
 
 namespace mongo {
 namespace {
@@ -77,15 +74,11 @@ public:
     /**
      * Sets up the _shardLoader with the results of makeFiveChunks().
      */
-    vector<ChunkType> setUpChunkLoaderWithFiveChunks();
+    std::pair<CollectionType, vector<ChunkType>> setUpChunkLoaderWithFiveChunks();
 
     void refreshCollectionEpochOnRemoteLoader();
 
     const KeyPattern kKeyPattern = KeyPattern(BSON(kPattern << 1));
-    const std::function<void(OperationContext*, StatusWith<CollectionAndChangedChunks>)>
-        kDoNothingCallbackFn = [](
-            OperationContext * opCtx,
-            StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> swCollAndChunks) noexcept {};
 
     CatalogCacheLoaderMock* _remoteLoaderMock;
     std::unique_ptr<ShardServerCatalogCacheLoader> _shardLoader;
@@ -100,7 +93,7 @@ void ShardServerCatalogCacheLoaderTest::setUp() {
 
     // Create mock remote and real shard loader, retaining a pointer to the mock remote loader so
     // that unit tests can manipulate it to return certain responses.
-    std::unique_ptr<CatalogCacheLoaderMock> mockLoader = std::make_unique<CatalogCacheLoaderMock>();
+    auto mockLoader = std::make_unique<CatalogCacheLoaderMock>();
     _remoteLoaderMock = mockLoader.get();
     _shardLoader = std::make_unique<ShardServerCatalogCacheLoader>(std::move(mockLoader));
 
@@ -118,6 +111,7 @@ vector<ChunkType> ShardServerCatalogCacheLoaderTest::makeFiveChunks(
     const ChunkVersion& collectionVersion) {
     ChunkVersion collVersion(collectionVersion);
     vector<ChunkType> chunks;
+    const UUID uuid = UUID::gen();
 
     BSONObj mins[] = {
         BSON("a" << MINKEY), BSON("a" << 10), BSON("a" << 50), BSON("a" << 100), BSON("a" << 200)};
@@ -128,7 +122,7 @@ vector<ChunkType> ShardServerCatalogCacheLoaderTest::makeFiveChunks(
         collVersion.incMajor();
 
         ChunkType chunk;
-        chunk.setNS(kNss);
+        chunk.setCollectionUUID(uuid);
         chunk.setMin(mins[i]);
         chunk.setMax(maxs[i]);
         chunk.setShard(kShardId);
@@ -144,6 +138,7 @@ vector<ChunkType> ShardServerCatalogCacheLoaderTest::makeThreeUpdatedChunksDiff(
     const ChunkVersion& collectionVersion) {
     ChunkVersion collVersion(collectionVersion);
     vector<ChunkType> chunks;
+    const UUID uuid = UUID::gen();
 
     // The diff query is for GTE a known version, so prepend the previous newest chunk, which is
     // unmodified by this change and so should be found. Note: it is important for testing that the
@@ -151,7 +146,7 @@ vector<ChunkType> ShardServerCatalogCacheLoaderTest::makeThreeUpdatedChunksDiff(
     // dependent on a race between persistence and retrieving data because it combines enqueued and
     // persisted results without applying modifications.
     ChunkType oldChunk;
-    oldChunk.setNS(kNss);
+    oldChunk.setCollectionUUID(uuid);
     oldChunk.setMin(BSON("a" << 200));
     oldChunk.setMax(BSON("a" << MAXKEY));
     oldChunk.setShard(kShardId);
@@ -167,7 +162,7 @@ vector<ChunkType> ShardServerCatalogCacheLoaderTest::makeThreeUpdatedChunksDiff(
         collVersion.incMinor();
 
         ChunkType chunk;
-        chunk.setNS(kNss);
+        chunk.setCollectionUUID(uuid);
         chunk.setMin(mins[i]);
         chunk.setMax(maxs[i]);
         chunk.setShard(kShardId);
@@ -194,45 +189,33 @@ ShardServerCatalogCacheLoaderTest::makeCombinedOriginalFiveChunksAndThreeNewChun
 
 CollectionType ShardServerCatalogCacheLoaderTest::makeCollectionType(
     const ChunkVersion& collVersion) {
-    CollectionType coll;
-    coll.setNs(kNss);
-    coll.setEpoch(collVersion.epoch());
-    coll.setUpdatedAt(Date_t::fromMillisSinceEpoch(collVersion.toLong()));
-    coll.setKeyPattern(kKeyPattern);
-    coll.setUnique(false);
-    return coll;
+    return {kNss,
+            collVersion.epoch(),
+            collVersion.getTimestamp(),
+            Date_t::now(),
+            UUID::gen(),
+            kKeyPattern};
 }
 
-vector<ChunkType> ShardServerCatalogCacheLoaderTest::setUpChunkLoaderWithFiveChunks() {
-    ChunkVersion collectionVersion(1, 0, OID::gen());
+std::pair<CollectionType, vector<ChunkType>>
+ShardServerCatalogCacheLoaderTest::setUpChunkLoaderWithFiveChunks() {
+    ChunkVersion collectionVersion({OID::gen(), Timestamp(1, 1)}, {1, 0});
 
     CollectionType collectionType = makeCollectionType(collectionVersion);
     vector<ChunkType> chunks = makeFiveChunks(collectionVersion);
     _remoteLoaderMock->setCollectionRefreshReturnValue(collectionType);
     _remoteLoaderMock->setChunkRefreshReturnValue(chunks);
 
-    StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> results{
-        Status(ErrorCodes::InternalError, "")};
-    const auto refreshCallbackFn = [&results](
-        OperationContext * opCtx,
-        StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> swCollAndChunks) noexcept {
-        results = std::move(swCollAndChunks);
-    };
+    auto collAndChunksRes = _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED()).get();
 
-    auto notification =
-        _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED(), refreshCallbackFn);
-    notification->get();
-
-    // Check refreshCallbackFn thread results where we can safely throw.
-    ASSERT_OK(results.getStatus());
-    auto collAndChunkRes = results.getValue();
-    ASSERT_EQUALS(collAndChunkRes.epoch, collectionType.getEpoch());
-    ASSERT_EQUALS(collAndChunkRes.changedChunks.size(), 5UL);
-    for (unsigned int i = 0; i < collAndChunkRes.changedChunks.size(); ++i) {
-        ASSERT_BSONOBJ_EQ(collAndChunkRes.changedChunks[i].toShardBSON(), chunks[i].toShardBSON());
+    ASSERT_EQUALS(collAndChunksRes.epoch, collectionType.getEpoch());
+    ASSERT_EQUALS(collAndChunksRes.changedChunks.size(), 5UL);
+    ASSERT(!collAndChunksRes.timeseriesFields.has_value());
+    for (unsigned int i = 0; i < collAndChunksRes.changedChunks.size(); ++i) {
+        ASSERT_BSONOBJ_EQ(collAndChunksRes.changedChunks[i].toShardBSON(), chunks[i].toShardBSON());
     }
 
-    return chunks;
+    return std::pair{collectionType, std::move(chunks)};
 }
 
 TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromUnshardedToUnsharded) {
@@ -241,25 +224,16 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromUnshardedToUnsharded) {
     Status errorStatus = Status(ErrorCodes::NamespaceNotFound, "collection not found");
     _remoteLoaderMock->setCollectionRefreshReturnValue(errorStatus);
 
-    StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> results{
-        Status(ErrorCodes::InternalError, "")};
-    const auto refreshCallbackFn = [&results](
-        OperationContext * opCtx,
-        StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> swCollAndChunks) noexcept {
-        results = std::move(swCollAndChunks);
-    };
-
-    auto notification =
-        _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED(), refreshCallbackFn);
-    notification->get();
-
-    ASSERT_EQUALS(results.getStatus(), errorStatus);
+    ASSERT_THROWS_CODE_AND_WHAT(_shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED()).get(),
+                                DBException,
+                                errorStatus.code(),
+                                errorStatus.reason());
 }
 
 TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedToUnsharded) {
     // First set up the shard chunk loader as sharded.
-
-    auto chunks = setUpChunkLoaderWithFiveChunks();
+    auto collAndChunks = setUpChunkLoaderWithFiveChunks();
+    auto& chunks = collAndChunks.second;
 
     // Then return a NamespaceNotFound error, which means the collection must have been dropped,
     // clearing the chunk metadata.
@@ -267,25 +241,17 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedToUnsharded) {
     Status errorStatus = Status(ErrorCodes::NamespaceNotFound, "collection not found");
     _remoteLoaderMock->setCollectionRefreshReturnValue(errorStatus);
 
-    StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> results{
-        Status(ErrorCodes::InternalError, "")};
-    const auto nextRefreshCallbackFn = [&results](
-        OperationContext * opCtx,
-        StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> swCollAndChunks) noexcept {
-        results = std::move(swCollAndChunks);
-    };
-
-    auto notification =
-        _shardLoader->getChunksSince(kNss, chunks.back().getVersion(), nextRefreshCallbackFn);
-    notification->get();
-
-    ASSERT_EQUALS(results.getStatus(), errorStatus);
+    ASSERT_THROWS_CODE_AND_WHAT(
+        _shardLoader->getChunksSince(kNss, chunks.back().getVersion()).get(),
+        DBException,
+        errorStatus.code(),
+        errorStatus.reason());
 }
 
 TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindNoDiff) {
     // First set up the shard chunk loader as sharded.
-
-    vector<ChunkType> chunks = setUpChunkLoaderWithFiveChunks();
+    auto collAndChunks = setUpChunkLoaderWithFiveChunks();
+    auto& chunks = collAndChunks.second;
 
     // Then set up the remote loader to return a single document we've already seen -- indicates
     // there's nothing new.
@@ -294,22 +260,10 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindNoDiff) {
     lastChunk.push_back(chunks.back());
     _remoteLoaderMock->setChunkRefreshReturnValue(lastChunk);
 
-    StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> results{
-        Status(ErrorCodes::InternalError, "")};
-    const auto refreshCallbackFn = [&results](
-        OperationContext * opCtx,
-        StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> swCollAndChunks) noexcept {
-        results = std::move(swCollAndChunks);
-    };
-
-    auto notification =
-        _shardLoader->getChunksSince(kNss, chunks.back().getVersion(), refreshCallbackFn);
-    notification->get();
+    auto collAndChunksRes = _shardLoader->getChunksSince(kNss, chunks.back().getVersion()).get();
 
     // Check that refreshing from the latest version returned a single document matching that
     // version.
-    ASSERT_OK(results.getStatus());
-    auto collAndChunksRes = results.getValue();
     ASSERT_EQUALS(collAndChunksRes.epoch, chunks.back().getVersion().epoch());
     ASSERT_EQUALS(collAndChunksRes.changedChunks.size(), 1UL);
     ASSERT_BSONOBJ_EQ(collAndChunksRes.changedChunks.back().toShardBSON(),
@@ -320,8 +274,8 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindNoDiff) {
 // routing table, rather than diff from a known version.
 TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindNoDiffRequestAll) {
     // First set up the shard chunk loader as sharded.
-
-    vector<ChunkType> chunks = setUpChunkLoaderWithFiveChunks();
+    auto collAndChunks = setUpChunkLoaderWithFiveChunks();
+    auto& chunks = collAndChunks.second;
 
     // Then set up the remote loader to return a single document we've already seen -- indicates
     // there's nothing new.
@@ -330,21 +284,7 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindNoDiffReq
     lastChunk.push_back(chunks.back());
     _remoteLoaderMock->setChunkRefreshReturnValue(lastChunk);
 
-    StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> results{
-        Status(ErrorCodes::InternalError, "")};
-    const auto completeRefreshCallbackFn = [&results](
-        OperationContext * opCtx,
-        StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> swCollAndChunks) noexcept {
-        results = std::move(swCollAndChunks);
-    };
-
-    auto notification =
-        _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED(), completeRefreshCallbackFn);
-    notification->get();
-
-    // Check that the complete routing table was returned successfully.
-    ASSERT_OK(results.getStatus());
-    auto collAndChunksRes = results.getValue();
+    auto collAndChunksRes = _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED()).get();
     ASSERT_EQUALS(collAndChunksRes.epoch, chunks.back().getVersion().epoch());
     ASSERT_EQUALS(collAndChunksRes.changedChunks.size(), 5UL);
     for (unsigned int i = 0; i < collAndChunksRes.changedChunks.size(); ++i) {
@@ -356,8 +296,8 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindNoDiffReq
 
 TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindDiff) {
     // First set up the shard chunk loader as sharded.
-
-    vector<ChunkType> chunks = setUpChunkLoaderWithFiveChunks();
+    auto collAndChunks = setUpChunkLoaderWithFiveChunks();
+    auto& chunks = collAndChunks.second;
 
     // Then refresh again and find updated chunks.
 
@@ -365,21 +305,9 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindDiff) {
     vector<ChunkType> updatedChunksDiff = makeThreeUpdatedChunksDiff(collVersion);
     _remoteLoaderMock->setChunkRefreshReturnValue(updatedChunksDiff);
 
-    StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> results{
-        Status(ErrorCodes::InternalError, "")};
-    const auto refreshCallbackFn = [&results](
-        OperationContext * opCtx,
-        StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> swCollAndChunks) noexcept {
-        results = std::move(swCollAndChunks);
-    };
-
-    auto notification =
-        _shardLoader->getChunksSince(kNss, chunks.back().getVersion(), refreshCallbackFn);
-    notification->get();
+    auto collAndChunksRes = _shardLoader->getChunksSince(kNss, chunks.back().getVersion()).get();
 
     // Check that the diff was returned successfull.
-    ASSERT_OK(results.getStatus());
-    auto collAndChunksRes = results.getValue();
     ASSERT_EQUALS(collAndChunksRes.epoch, updatedChunksDiff.front().getVersion().epoch());
     ASSERT_EQUALS(collAndChunksRes.changedChunks.size(), 4UL);
     for (unsigned int i = 0; i < collAndChunksRes.changedChunks.size(); ++i) {
@@ -394,8 +322,8 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindDiff) {
 
 TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindDiffRequestAll) {
     // First set up the shard chunk loader as sharded.
-
-    vector<ChunkType> chunks = setUpChunkLoaderWithFiveChunks();
+    auto collAndChunks = setUpChunkLoaderWithFiveChunks();
+    auto& chunks = collAndChunks.second;
 
     // First cause a remote refresh to find the updated chunks. Then wait for persistence, so that
     // we ensure that nothing is enqueued and the next getChunksSince call will return a predictable
@@ -404,9 +332,7 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindDiffReque
     vector<ChunkType> updatedChunksDiff = makeThreeUpdatedChunksDiff(chunks.back().getVersion());
     _remoteLoaderMock->setChunkRefreshReturnValue(updatedChunksDiff);
 
-    auto notification =
-        _shardLoader->getChunksSince(kNss, chunks.back().getVersion(), kDoNothingCallbackFn);
-    notification->get();
+    _shardLoader->getChunksSince(kNss, chunks.back().getVersion()).get();
 
     // Wait for persistence of update
     _shardLoader->waitForCollectionFlush(operationContext(), kNss);
@@ -420,21 +346,7 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindDiffReque
     vector<ChunkType> completeRoutingTableWithDiffApplied =
         makeCombinedOriginalFiveChunksAndThreeNewChunksDiff(chunks, updatedChunksDiff);
 
-    StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> results{
-        Status(ErrorCodes::InternalError, "")};
-    const auto refreshCallbackFn = [&results](
-        OperationContext * opCtx,
-        StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> swCollAndChunks) noexcept {
-        results = std::move(swCollAndChunks);
-    };
-
-    auto nextNotification =
-        _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED(), refreshCallbackFn);
-    nextNotification->get();
-
-    // Check that the complete routing table, with diff applied, was returned.
-    ASSERT_OK(results.getStatus());
-    auto collAndChunksRes = results.getValue();
+    auto collAndChunksRes = _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED()).get();
     ASSERT_EQUALS(collAndChunksRes.epoch,
                   completeRoutingTableWithDiffApplied.front().getVersion().epoch());
     ASSERT_EQUALS(collAndChunksRes.changedChunks.size(), 5UL);
@@ -450,32 +362,18 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindDiffReque
 
 TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindNewEpoch) {
     // First set up the shard chunk loader as sharded.
-
-    vector<ChunkType> chunks = setUpChunkLoaderWithFiveChunks();
+    auto collAndChunks = setUpChunkLoaderWithFiveChunks();
+    auto& chunks = collAndChunks.second;
 
     // Then refresh again and find that the collection has been dropped and recreated.
 
-    ChunkVersion collVersionWithNewEpoch(1, 0, OID::gen());
+    ChunkVersion collVersionWithNewEpoch({OID::gen(), Timestamp(2, 0)}, {1, 0});
     CollectionType collectionTypeWithNewEpoch = makeCollectionType(collVersionWithNewEpoch);
     vector<ChunkType> chunksWithNewEpoch = makeFiveChunks(collVersionWithNewEpoch);
     _remoteLoaderMock->setCollectionRefreshReturnValue(collectionTypeWithNewEpoch);
     _remoteLoaderMock->setChunkRefreshReturnValue(chunksWithNewEpoch);
 
-    StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> results{
-        Status(ErrorCodes::InternalError, "")};
-    const auto refreshCallbackFn = [&results](
-        OperationContext * opCtx,
-        StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> swCollAndChunks) noexcept {
-        results = std::move(swCollAndChunks);
-    };
-
-    auto notification =
-        _shardLoader->getChunksSince(kNss, chunks.back().getVersion(), refreshCallbackFn);
-    notification->get();
-
-    // Check that the complete routing table for the new epoch was returned.
-    ASSERT_OK(results.getStatus());
-    auto collAndChunksRes = results.getValue();
+    auto collAndChunksRes = _shardLoader->getChunksSince(kNss, chunks.back().getVersion()).get();
     ASSERT_EQUALS(collAndChunksRes.epoch, collectionTypeWithNewEpoch.getEpoch());
     ASSERT_EQUALS(collAndChunksRes.changedChunks.size(), 5UL);
     for (unsigned int i = 0; i < collAndChunksRes.changedChunks.size(); ++i) {
@@ -490,15 +388,13 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindNewEpoch)
 
 TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindMixedChunkVersions) {
     // First set up the shard chunk loader as sharded.
-
-    vector<ChunkType> chunks = setUpChunkLoaderWithFiveChunks();
+    auto collAndChunks = setUpChunkLoaderWithFiveChunks();
+    auto& chunks = collAndChunks.second;
 
     // Then refresh again and retrieve chunks from the config server that have mixed epoches, like
     // as if the chunks read yielded around a drop and recreate of the collection.
 
-    CollectionType originalCollectionType = makeCollectionType(chunks.back().getVersion());
-
-    ChunkVersion collVersionWithNewEpoch(1, 0, OID::gen());
+    ChunkVersion collVersionWithNewEpoch({OID::gen(), Timestamp(2, 0)}, {1, 0});
     CollectionType collectionTypeWithNewEpoch = makeCollectionType(collVersionWithNewEpoch);
     vector<ChunkType> chunksWithNewEpoch = makeFiveChunks(collVersionWithNewEpoch);
     vector<ChunkType> mixedChunks;
@@ -506,19 +402,9 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindMixedChun
     mixedChunks.insert(mixedChunks.end(), chunksWithNewEpoch.begin(), chunksWithNewEpoch.end());
     _remoteLoaderMock->setChunkRefreshReturnValue(mixedChunks);
 
-    StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> mixedResults{
-        Status(ErrorCodes::InternalError, "")};
-    const auto mixedRefreshCallbackFn = [&mixedResults](
-        OperationContext * opCtx,
-        StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> swCollAndChunks) noexcept {
-        mixedResults = std::move(swCollAndChunks);
-    };
-
-    auto mixedNotification =
-        _shardLoader->getChunksSince(kNss, chunks.back().getVersion(), mixedRefreshCallbackFn);
-    mixedNotification->get();
-
-    ASSERT_EQUALS(mixedResults.getStatus().code(), ErrorCodes::ConflictingOperationInProgress);
+    ASSERT_THROWS_CODE(_shardLoader->getChunksSince(kNss, chunks.back().getVersion()).get(),
+                       DBException,
+                       ErrorCodes::ConflictingOperationInProgress);
 
     // Now make sure the newly recreated collection is cleanly loaded. We cannot ensure a
     // non-variable response until the loader has remotely retrieved the new metadata and applied
@@ -528,9 +414,7 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindMixedChun
     _remoteLoaderMock->setCollectionRefreshReturnValue(collectionTypeWithNewEpoch);
     _remoteLoaderMock->setChunkRefreshReturnValue(chunksWithNewEpoch);
 
-    auto cleanNotification =
-        _shardLoader->getChunksSince(kNss, chunks.back().getVersion(), kDoNothingCallbackFn);
-    cleanNotification->get();
+    _shardLoader->getChunksSince(kNss, chunks.back().getVersion()).get();
 
     // Wait for persistence of update.
     _shardLoader->waitForCollectionFlush(operationContext(), kNss);
@@ -539,20 +423,7 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindMixedChun
     lastChunkWithNewEpoch.push_back(chunksWithNewEpoch.back());
     _remoteLoaderMock->setChunkRefreshReturnValue(lastChunkWithNewEpoch);
 
-    StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> results{
-        Status(ErrorCodes::InternalError, "")};
-    const auto refreshCallbackFn = [&results](
-        OperationContext * opCtx,
-        StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> swCollAndChunks) noexcept {
-        results = std::move(swCollAndChunks);
-    };
-
-    auto notification =
-        _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED(), refreshCallbackFn);
-    notification->get();
-
-    ASSERT_OK(results.getStatus());
-    auto collAndChunksRes = results.getValue();
+    auto collAndChunksRes = _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED()).get();
     ASSERT_EQUALS(collAndChunksRes.epoch, collectionTypeWithNewEpoch.getEpoch());
     ASSERT_EQUALS(collAndChunksRes.changedChunks.size(), 5UL);
     for (unsigned int i = 0; i < collAndChunksRes.changedChunks.size(); ++i) {
@@ -565,8 +436,50 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindMixedChun
     }
 }
 
+TEST_F(ShardServerCatalogCacheLoaderTest, TimeseriesFieldsAreProperlyPropagatedOnSSCCL) {
+    ChunkVersion collectionVersion({OID::gen(), Timestamp(1, 1)}, {1, 0});
+
+    CollectionType collectionType = makeCollectionType(collectionVersion);
+    vector<ChunkType> chunks = makeFiveChunks(collectionVersion);
+    auto timeseriesOptions = TimeseriesOptions("fieldName");
+
+    {
+        TypeCollectionTimeseriesFields tsFields;
+        tsFields.setTimeseriesOptions(timeseriesOptions);
+        collectionType.setTimeseriesFields(tsFields);
+
+        _remoteLoaderMock->setCollectionRefreshReturnValue(collectionType);
+        _remoteLoaderMock->setChunkRefreshReturnValue(chunks);
+
+        auto collAndChunksRes = _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED()).get();
+        ASSERT(collAndChunksRes.timeseriesFields.has_value());
+        ASSERT(collAndChunksRes.timeseriesFields->getGranularity() ==
+               BucketGranularityEnum::Seconds);
+    }
+
+    {
+        auto& lastChunk = chunks.back();
+        const auto maxLoaderVersion = lastChunk.getVersion();
+        ChunkVersion newCollectionVersion = maxLoaderVersion;
+        newCollectionVersion.incMinor();
+        lastChunk.setVersion(newCollectionVersion);
+
+        TypeCollectionTimeseriesFields tsFields;
+        timeseriesOptions.setGranularity(BucketGranularityEnum::Hours);
+        tsFields.setTimeseriesOptions(timeseriesOptions);
+        collectionType.setTimeseriesFields(tsFields);
+
+        _remoteLoaderMock->setCollectionRefreshReturnValue(collectionType);
+        _remoteLoaderMock->setChunkRefreshReturnValue(std::vector{lastChunk});
+
+        auto collAndChunksRes = _shardLoader->getChunksSince(kNss, maxLoaderVersion).get();
+        ASSERT(collAndChunksRes.timeseriesFields.has_value());
+        ASSERT(collAndChunksRes.timeseriesFields->getGranularity() == BucketGranularityEnum::Hours);
+    }
+}
+
 void ShardServerCatalogCacheLoaderTest::refreshCollectionEpochOnRemoteLoader() {
-    ChunkVersion collectionVersion(1, 2, OID::gen());
+    ChunkVersion collectionVersion({OID::gen(), Timestamp(1, 1)}, {1, 2});
     CollectionType collectionType = makeCollectionType(collectionVersion);
     vector<ChunkType> chunks = makeFiveChunks(collectionVersion);
     _remoteLoaderMock->setCollectionRefreshReturnValue(collectionType);
@@ -576,7 +489,7 @@ void ShardServerCatalogCacheLoaderTest::refreshCollectionEpochOnRemoteLoader() {
 TEST_F(ShardServerCatalogCacheLoaderTest, CollAndChunkTasksConsistency) {
     // Put some metadata in the persisted cache (config.cache.chunks.*)
     refreshCollectionEpochOnRemoteLoader();
-    _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED(), kDoNothingCallbackFn)->get();
+    _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED()).get();
     _shardLoader->waitForCollectionFlush(operationContext(), kNss);
 
     // Pause the thread processing the pending updates on metadata
@@ -584,7 +497,7 @@ TEST_F(ShardServerCatalogCacheLoaderTest, CollAndChunkTasksConsistency) {
 
     // Put a first task in the list of pending updates on metadata (in-memory)
     refreshCollectionEpochOnRemoteLoader();
-    _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED(), kDoNothingCallbackFn)->get();
+    _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED()).get();
 
     // Bump the shard's term
     _shardLoader->onStepUp();
@@ -592,7 +505,7 @@ TEST_F(ShardServerCatalogCacheLoaderTest, CollAndChunkTasksConsistency) {
     // Putting a second task causes a verification of the contiguous versions in the list pending
     // updates on metadata
     refreshCollectionEpochOnRemoteLoader();
-    _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED(), kDoNothingCallbackFn)->get();
+    _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED()).get();
 }
 
 }  // namespace
