@@ -33,22 +33,21 @@
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/dbdirectclient.h"
-#include "mongo/db/op_observer_registry.h"
+#include "mongo/db/op_observer/op_observer_registry.h"
+#include "mongo/db/op_observer/oplog_writer_mock.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/s/collection_sharding_state_factory_shard.h"
 #include "mongo/db/s/collection_sharding_state_factory_standalone.h"
-#include "mongo/db/s/config_server_op_observer.h"
 #include "mongo/db/s/op_observer_sharding_impl.h"
 #include "mongo/db/s/shard_server_catalog_cache_loader.h"
 #include "mongo/db/s/shard_server_op_observer.h"
+#include "mongo/db/s/shard_server_test_fixture.h"
 #include "mongo/db/s/sharding_initialization_mongod.h"
 #include "mongo/db/s/type_shard_identity.h"
 #include "mongo/db/server_options.h"
-#include "mongo/s/catalog/dist_lock_manager_mock.h"
 #include "mongo/s/catalog/sharding_catalog_client_impl.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/config_server_catalog_cache_loader.h"
-#include "mongo/s/shard_server_test_fixture.h"
 
 namespace mongo {
 namespace {
@@ -61,9 +60,6 @@ const std::string kShardName("TestShard");
  */
 class ShardingInitializationMongoDTest : public ShardingMongodTestFixture {
 protected:
-    // Used to write to set up local collections before exercising server logic.
-    std::unique_ptr<DBDirectClient> _dbDirectClient;
-
     void setUp() override {
         serverGlobalParams.clusterRole = ClusterRole::None;
         ShardingMongodTestFixture::setUp();
@@ -76,23 +72,21 @@ protected:
                                     std::make_unique<ConfigServerCatalogCacheLoader>()));
 
         ShardingInitializationMongoD::get(getServiceContext())
-            ->setGlobalInitMethodForTest([&](OperationContext* opCtx,
-                                             const ShardIdentity& shardIdentity,
-                                             StringData distLockProcessId) {
-                const auto& configConnStr = shardIdentity.getConfigsvrConnectionString();
+            ->setGlobalInitMethodForTest(
+                [&](OperationContext* opCtx, const ShardIdentity& shardIdentity) {
+                    const auto& configConnStr = shardIdentity.getConfigsvrConnectionString();
 
-                uassertStatusOK(initializeGlobalShardingStateForMongodForTest(configConnStr));
+                    uassertStatusOK(initializeGlobalShardingStateForMongodForTest(configConnStr));
 
-                // Set the ConnectionString return value on the mock targeter so that later calls to
-                // the
-                // targeter's getConnString() return the appropriate value
-                auto configTargeter = RemoteCommandTargeterMock::get(
-                    shardRegistry()->getConfigShard()->getTargeter());
-                configTargeter->setConnectionStringReturnValue(configConnStr);
-                configTargeter->setFindHostReturnValue(configConnStr.getServers()[0]);
+                    // Set the ConnectionString return value on the mock targeter so that later
+                    // calls to the targeter's getConnString() return the appropriate value
+                    auto configTargeter = RemoteCommandTargeterMock::get(
+                        shardRegistry()->getConfigShard()->getTargeter());
+                    configTargeter->setConnectionStringReturnValue(configConnStr);
+                    configTargeter->setFindHostReturnValue(configConnStr.getServers()[0]);
 
-                return Status::OK();
-            });
+                    return Status::OK();
+                });
 
         _dbDirectClient = std::make_unique<DBDirectClient>(operationContext());
     }
@@ -101,7 +95,7 @@ protected:
         _dbDirectClient.reset();
 
         // Restore the defaults before calling tearDown
-        storageGlobalParams.readOnly = false;
+        storageGlobalParams.queryableBackupMode = false;
         serverGlobalParams.overrideShardIdentity = BSONObj();
 
         CatalogCacheLoader::clearForTests(getServiceContext());
@@ -110,15 +104,8 @@ protected:
         ShardingMongodTestFixture::tearDown();
     }
 
-    std::unique_ptr<DistLockManager> makeDistLockManager(
-        std::unique_ptr<DistLockCatalog> distLockCatalog) override {
-        return std::make_unique<DistLockManagerMock>(nullptr);
-    }
-
-    std::unique_ptr<ShardingCatalogClient> makeShardingCatalogClient(
-        std::unique_ptr<DistLockManager> distLockManager) override {
-        invariant(distLockManager);
-        return std::make_unique<ShardingCatalogClientImpl>(std::move(distLockManager));
+    std::unique_ptr<ShardingCatalogClient> makeShardingCatalogClient() override {
+        return std::make_unique<ShardingCatalogClientImpl>();
     }
 
     auto* shardingInitialization() {
@@ -128,6 +115,9 @@ protected:
     auto* shardingState() {
         return ShardingState::get(getServiceContext());
     }
+
+    // Used to write to set up local collections before exercising server logic.
+    std::unique_ptr<DBDirectClient> _dbDirectClient;
 };
 
 /**
@@ -155,15 +145,13 @@ public:
             std::make_unique<CollectionShardingStateFactoryShard>(_serviceContext));
 
         serverGlobalParams.clusterRole = ClusterRole::ShardServer;
-        auto makeOpObserver = [&] {
+        _serviceContext->setOpObserver([&] {
             auto opObserver = std::make_unique<OpObserverRegistry>();
-            opObserver->addObserver(std::make_unique<OpObserverImpl>());
-            opObserver->addObserver(std::make_unique<ConfigServerOpObserver>());
+            opObserver->addObserver(
+                std::make_unique<OpObserverShardingImpl>(std::make_unique<OplogWriterMock>()));
             opObserver->addObserver(std::make_unique<ShardServerOpObserver>());
             return opObserver;
-        };
-
-        _serviceContext->setOpObserver(makeOpObserver());
+        }());
     }
 
 private:
@@ -176,7 +164,7 @@ TEST_F(ShardingInitializationMongoDTest, ValidShardIdentitySucceeds) {
 
     ShardIdentityType shardIdentity;
     shardIdentity.setConfigsvrConnectionString(
-        ConnectionString(ConnectionString::SET, "a:1,b:2", "config"));
+        ConnectionString(ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config"));
     shardIdentity.setShardName(kShardName);
     shardIdentity.setClusterId(OID::gen());
 
@@ -193,23 +181,22 @@ TEST_F(ShardingInitializationMongoDTest, InitWhilePreviouslyInErrorStateWillStay
 
     ShardIdentityType shardIdentity;
     shardIdentity.setConfigsvrConnectionString(
-        ConnectionString(ConnectionString::SET, "a:1,b:2", "config"));
+        ConnectionString(ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config"));
     shardIdentity.setShardName(kShardName);
     shardIdentity.setClusterId(OID::gen());
 
-    shardingInitialization()->setGlobalInitMethodForTest([](OperationContext* opCtx,
-                                                            const ShardIdentity& shardIdentity,
-                                                            StringData distLockProcessId) {
-        uasserted(ErrorCodes::ShutdownInProgress, "Not an actual shutdown");
-    });
+    shardingInitialization()->setGlobalInitMethodForTest(
+        [](OperationContext* opCtx, const ShardIdentity& shardIdentity) {
+            uasserted(ErrorCodes::ShutdownInProgress, "Not an actual shutdown");
+        });
 
     shardingInitialization()->initializeFromShardIdentity(operationContext(), shardIdentity);
 
     // ShardingState is now in error state, attempting to call it again will still result in error.
     shardingInitialization()->setGlobalInitMethodForTest(
-        [](OperationContext* opCtx,
-           const ShardIdentity& shardIdentity,
-           StringData distLockProcessId) { FAIL("Should not be invoked!"); });
+        [](OperationContext* opCtx, const ShardIdentity& shardIdentity) {
+            FAIL("Should not be invoked!");
+        });
 
     ASSERT_THROWS_CODE(
         shardingInitialization()->initializeFromShardIdentity(operationContext(), shardIdentity),
@@ -226,7 +213,7 @@ TEST_F(ShardingInitializationMongoDTest, InitializeAgainWithMatchingShardIdentit
     auto clusterID = OID::gen();
     ShardIdentityType shardIdentity;
     shardIdentity.setConfigsvrConnectionString(
-        ConnectionString(ConnectionString::SET, "a:1,b:2", "config"));
+        ConnectionString(ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config"));
     shardIdentity.setShardName(kShardName);
     shardIdentity.setClusterId(clusterID);
 
@@ -234,14 +221,14 @@ TEST_F(ShardingInitializationMongoDTest, InitializeAgainWithMatchingShardIdentit
 
     ShardIdentityType shardIdentity2;
     shardIdentity2.setConfigsvrConnectionString(
-        ConnectionString(ConnectionString::SET, "a:1,b:2", "config"));
+        ConnectionString(ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config"));
     shardIdentity2.setShardName(kShardName);
     shardIdentity2.setClusterId(clusterID);
 
     shardingInitialization()->setGlobalInitMethodForTest(
-        [](OperationContext* opCtx,
-           const ShardIdentity& shardIdentity,
-           StringData distLockProcessId) { FAIL("Should not be invoked!"); });
+        [](OperationContext* opCtx, const ShardIdentity& shardIdentity) {
+            FAIL("Should not be invoked!");
+        });
 
     shardingInitialization()->initializeFromShardIdentity(operationContext(), shardIdentity2);
 
@@ -252,6 +239,56 @@ TEST_F(ShardingInitializationMongoDTest, InitializeAgainWithMatchingShardIdentit
     ASSERT_EQ("config/a:1,b:2", shardRegistry()->getConfigServerConnectionString().toString());
 }
 
+TEST_F(ShardingInitializationMongoDTest, InitializeAgainWithNonMatchingShardIdentityFails) {
+    // Must hold a lock to call initializeFromShardIdentity.
+    Lock::GlobalWrite lk(operationContext());
+
+    auto clusterID = OID::gen();
+    ShardIdentityType shardIdentity;
+    shardIdentity.setConfigsvrConnectionString(
+        ConnectionString(ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config"));
+    shardIdentity.setShardName(kShardName);
+    shardIdentity.setClusterId(clusterID);
+
+    shardingInitialization()->initializeFromShardIdentity(operationContext(), shardIdentity);
+
+    shardingInitialization()->setGlobalInitMethodForTest(
+        [](OperationContext* opCtx, const ShardIdentity& shardIdentity) {
+            FAIL("Should not be invoked!");
+        });
+
+    // Running again the initialization with a different shardName must result in an error
+    {
+        ShardIdentityType shardIdentity2 = shardIdentity;
+        shardIdentity2.setShardName("AnotherShardName");
+        ASSERT_THROWS_CODE(shardingInitialization()->initializeFromShardIdentity(operationContext(),
+                                                                                 shardIdentity2),
+                           AssertionException,
+                           40371);
+    }
+
+    // Running again the initialization with a different clusterId must result in an error
+    {
+        ShardIdentityType shardIdentity2 = shardIdentity;
+        shardIdentity2.setClusterId(OID::gen());
+        ASSERT_THROWS_CODE(shardingInitialization()->initializeFromShardIdentity(operationContext(),
+                                                                                 shardIdentity2),
+                           AssertionException,
+                           40372);
+    }
+
+    // Running again the initialization with a different ReplicaSetName must result in an error
+    {
+        ShardIdentityType shardIdentity2 = shardIdentity;
+        shardIdentity2.setConfigsvrConnectionString(
+            ConnectionString(ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config2"));
+        ASSERT_THROWS_CODE(shardingInitialization()->initializeFromShardIdentity(operationContext(),
+                                                                                 shardIdentity2),
+                           AssertionException,
+                           40374);
+    }
+}
+
 TEST_F(ShardingInitializationMongoDTest, InitializeAgainWithMatchingReplSetNameSucceeds) {
     // Must hold a lock to call initializeFromShardIdentity.
     Lock::GlobalWrite lk(operationContext());
@@ -259,7 +296,7 @@ TEST_F(ShardingInitializationMongoDTest, InitializeAgainWithMatchingReplSetNameS
     auto clusterID = OID::gen();
     ShardIdentityType shardIdentity;
     shardIdentity.setConfigsvrConnectionString(
-        ConnectionString(ConnectionString::SET, "a:1,b:2", "config"));
+        ConnectionString(ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config"));
     shardIdentity.setShardName(kShardName);
     shardIdentity.setClusterId(clusterID);
 
@@ -267,14 +304,14 @@ TEST_F(ShardingInitializationMongoDTest, InitializeAgainWithMatchingReplSetNameS
 
     ShardIdentityType shardIdentity2;
     shardIdentity2.setConfigsvrConnectionString(
-        ConnectionString(ConnectionString::SET, "b:2,c:3", "config"));
+        ConnectionString(ConnectionString::ConnectionType::kReplicaSet, "b:2,c:3", "config"));
     shardIdentity2.setShardName(kShardName);
     shardIdentity2.setClusterId(clusterID);
 
     shardingInitialization()->setGlobalInitMethodForTest(
-        [](OperationContext* opCtx,
-           const ShardIdentity& shardIdentity,
-           StringData distLockProcessId) { FAIL("Should not be invoked!"); });
+        [](OperationContext* opCtx, const ShardIdentity& shardIdentity) {
+            FAIL("Should not be invoked!");
+        });
 
     shardingInitialization()->initializeFromShardIdentity(operationContext(), shardIdentity2);
 
@@ -286,14 +323,15 @@ TEST_F(ShardingInitializationMongoDTest, InitializeAgainWithMatchingReplSetNameS
 }
 
 // The tests below check for different combinations of the compatible startup parameters for
-// --shardsvr, --overrideShardIdentity, and queryableBackup (readOnly) mode
+// --shardsvr, --overrideShardIdentity, and queryableBackup mode
 
 /**
- * readOnly and --shardsvr
+ * queryableBackupMode and --shardsvr
  */
-TEST_F(ShardingInitializationMongoDTest,
-       InitializeShardingAwarenessIfNeededReadOnlyAndShardServerAndNoOverrideShardIdentity) {
-    storageGlobalParams.readOnly = true;
+TEST_F(
+    ShardingInitializationMongoDTest,
+    InitializeShardingAwarenessIfNeededQueryableBackupModeAndShardServerAndNoOverrideShardIdentity) {
+    storageGlobalParams.queryableBackupMode = true;
 
     ASSERT_THROWS_CODE(
         shardingInitialization()->initializeShardingAwarenessIfNeeded(operationContext()),
@@ -301,9 +339,10 @@ TEST_F(ShardingInitializationMongoDTest,
         ErrorCodes::InvalidOptions);
 }
 
-TEST_F(ShardingInitializationMongoDTest,
-       InitializeShardingAwarenessIfNeededReadOnlyAndShardServerAndInvalidOverrideShardIdentity) {
-    storageGlobalParams.readOnly = true;
+TEST_F(
+    ShardingInitializationMongoDTest,
+    InitializeShardingAwarenessIfNeededQueryableBackupModeAndShardServerAndInvalidOverrideShardIdentity) {
+    storageGlobalParams.queryableBackupMode = true;
     serverGlobalParams.overrideShardIdentity =
         BSON("_id"
              << "shardIdentity" << ShardIdentity::kShardNameFieldName << kShardName
@@ -316,14 +355,15 @@ TEST_F(ShardingInitializationMongoDTest,
         ErrorCodes::UnsupportedFormat);
 }
 
-TEST_F(ShardingInitializationMongoDTest,
-       InitializeShardingAwarenessIfNeededReadOnlyAndShardServerAndValidOverrideShardIdentity) {
-    storageGlobalParams.readOnly = true;
+TEST_F(
+    ShardingInitializationMongoDTest,
+    InitializeShardingAwarenessIfNeededQueryableBackupModeAndShardServerAndValidOverrideShardIdentity) {
+    storageGlobalParams.queryableBackupMode = true;
     serverGlobalParams.clusterRole = ClusterRole::ShardServer;
     serverGlobalParams.overrideShardIdentity = [] {
         ShardIdentityType shardIdentity;
         shardIdentity.setConfigsvrConnectionString(
-            ConnectionString(ConnectionString::SET, "a:1,b:2", "config"));
+            ConnectionString(ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config"));
         shardIdentity.setShardName(kShardName);
         shardIdentity.setClusterId(OID::gen());
         ASSERT_OK(shardIdentity.validate());
@@ -331,14 +371,17 @@ TEST_F(ShardingInitializationMongoDTest,
     }();
 
     ASSERT(shardingInitialization()->initializeShardingAwarenessIfNeeded(operationContext()));
+    // A second call with the same parameters shouldn't trigger any error.
+    ASSERT(shardingInitialization()->initializeShardingAwarenessIfNeeded(operationContext()));
 }
 
 /**
- * readOnly and not --shardsvr
+ * queryableBackupMode and not --shardsvr
  */
-TEST_F(ShardingInitializationMongoDTest,
-       InitializeShardingAwarenessIfNeededReadOnlyAndNotShardServerAndNoOverrideShardIdentity) {
-    storageGlobalParams.readOnly = true;
+TEST_F(
+    ShardingInitializationMongoDTest,
+    InitializeShardingAwarenessIfNeededQueryableBackupModeAndNotShardServerAndNoOverrideShardIdentity) {
+    storageGlobalParams.queryableBackupMode = true;
     serverGlobalParams.clusterRole = ClusterRole::None;
 
     ASSERT(!shardingInitialization()->initializeShardingAwarenessIfNeeded(operationContext()));
@@ -346,8 +389,8 @@ TEST_F(ShardingInitializationMongoDTest,
 
 TEST_F(
     ShardingInitializationMongoDTest,
-    InitializeShardingAwarenessIfNeededReadOnlyAndNotShardServerAndInvalidOverrideShardIdentity) {
-    storageGlobalParams.readOnly = true;
+    InitializeShardingAwarenessIfNeededQueryableBackupModeAndNotShardServerAndInvalidOverrideShardIdentity) {
+    storageGlobalParams.queryableBackupMode = true;
     serverGlobalParams.clusterRole = ClusterRole::None;
     serverGlobalParams.overrideShardIdentity = BSON("_id"
                                                     << "shardIdentity"
@@ -360,14 +403,15 @@ TEST_F(
         ErrorCodes::InvalidOptions);
 }
 
-TEST_F(ShardingInitializationMongoDTest,
-       InitializeShardingAwarenessIfNeededReadOnlyAndNotShardServerAndValidOverrideShardIdentity) {
-    storageGlobalParams.readOnly = true;
+TEST_F(
+    ShardingInitializationMongoDTest,
+    InitializeShardingAwarenessIfNeededQueryableBackupModeAndNotShardServerAndValidOverrideShardIdentity) {
+    storageGlobalParams.queryableBackupMode = true;
     serverGlobalParams.clusterRole = ClusterRole::None;
     serverGlobalParams.overrideShardIdentity = [] {
         ShardIdentityType shardIdentity;
         shardIdentity.setConfigsvrConnectionString(
-            ConnectionString(ConnectionString::SET, "a:1,b:2", "config"));
+            ConnectionString(ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config"));
         shardIdentity.setShardName(kShardName);
         shardIdentity.setClusterId(OID::gen());
         ASSERT_OK(shardIdentity.validate());
@@ -381,10 +425,10 @@ TEST_F(ShardingInitializationMongoDTest,
 }
 
 /**
- * not readOnly and --overrideShardIdentity
+ * not queryableBackupMode and --overrideShardIdentity
  */
 TEST_F(ShardingInitializationMongoDTest,
-       InitializeShardingAwarenessIfNeededNotReadOnlyAndInvalidOverrideShardIdentity) {
+       InitializeShardingAwarenessIfNeededNotQueryableBackupModeAndInvalidOverrideShardIdentity) {
     serverGlobalParams.clusterRole = ClusterRole::ShardServer;
     serverGlobalParams.overrideShardIdentity = BSON("_id"
                                                     << "shardIdentity"
@@ -405,12 +449,12 @@ TEST_F(ShardingInitializationMongoDTest,
 }
 
 TEST_F(ShardingInitializationMongoDTest,
-       InitializeShardingAwarenessIfNeededNotReadOnlyAndValidOverrideShardIdentity) {
+       InitializeShardingAwarenessIfNeededNotQueryableBackupModeAndValidOverrideShardIdentity) {
     serverGlobalParams.clusterRole = ClusterRole::ShardServer;
     serverGlobalParams.overrideShardIdentity = [] {
         ShardIdentityType shardIdentity;
         shardIdentity.setConfigsvrConnectionString(
-            ConnectionString(ConnectionString::SET, "a:1,b:2", "config"));
+            ConnectionString(ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config"));
         shardIdentity.setShardName(kShardName);
         shardIdentity.setClusterId(OID::gen());
         ASSERT_OK(shardIdentity.validate());
@@ -431,15 +475,16 @@ TEST_F(ShardingInitializationMongoDTest,
 }
 
 /**
- * not readOnly and --shardsvr
+ * not queryableBackupMode and --shardsvr
  */
 TEST_F(ShardingInitializationMongoDTest,
-       InitializeShardingAwarenessIfNeededNotReadOnlyAndShardServerAndNoShardIdentity) {
+       InitializeShardingAwarenessIfNeededNotQueryableBackupModeAndShardServerAndNoShardIdentity) {
     ASSERT(!shardingInitialization()->initializeShardingAwarenessIfNeeded(operationContext()));
 }
 
-TEST_F(ShardingInitializationMongoDTest,
-       InitializeShardingAwarenessIfNeededNotReadOnlyAndShardServerAndInvalidShardIdentity) {
+TEST_F(
+    ShardingInitializationMongoDTest,
+    InitializeShardingAwarenessIfNeededNotQueryableBackupModeAndShardServerAndInvalidShardIdentity) {
     // Insert the shardIdentity doc to disk while pretending that we are in "standalone" mode,
     // otherwise OpObserver for inserts will prevent the insert from occurring because the
     // shardIdentity doc is invalid
@@ -463,8 +508,9 @@ TEST_F(ShardingInitializationMongoDTest,
         ErrorCodes::UnsupportedFormat);
 }
 
-TEST_F(ShardingInitializationMongoDTest,
-       InitializeShardingAwarenessIfNeededNotReadOnlyAndShardServerAndValidShardIdentity) {
+TEST_F(
+    ShardingInitializationMongoDTest,
+    InitializeShardingAwarenessIfNeededNotQueryableBackupModeAndShardServerAndValidShardIdentity) {
     // Insert the shardIdentity doc to disk while pretending that we are in "standalone" mode,
     // otherwise OpObserver for inserts will prevent the insert from occurring because the
     // shardIdentity doc is invalid
@@ -473,8 +519,8 @@ TEST_F(ShardingInitializationMongoDTest,
 
         BSONObj validShardIdentity = [&] {
             ShardIdentityType shardIdentity;
-            shardIdentity.setConfigsvrConnectionString(
-                ConnectionString(ConnectionString::SET, "a:1,b:2", "config"));
+            shardIdentity.setConfigsvrConnectionString(ConnectionString(
+                ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config"));
             shardIdentity.setShardName(kShardName);
             shardIdentity.setClusterId(OID::gen());
             ASSERT_OK(shardIdentity.validate());
@@ -486,20 +532,80 @@ TEST_F(ShardingInitializationMongoDTest,
     }
 
     ASSERT(shardingInitialization()->initializeShardingAwarenessIfNeeded(operationContext()));
+    // A second call with the same parameters shouldn't trigger any error.
+    ASSERT(shardingInitialization()->initializeShardingAwarenessIfNeeded(operationContext()));
+}
+
+TEST_F(
+    ShardingInitializationMongoDTest,
+    InitializeShardingAwarenessIfNeededNotQueryableBackupModeAndShardServerAsIfLogicalInitialSync) {
+    // No valid ShardIdentity yet, since we will get it through initial sync.
+    ASSERT(!shardingInitialization()->initializeShardingAwarenessIfNeeded(operationContext()));
+
+    BSONObj validShardIdentity = [&] {
+        ShardIdentityType shardIdentity;
+        shardIdentity.setConfigsvrConnectionString(
+            ConnectionString(ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config"));
+        shardIdentity.setShardName(kShardName);
+        shardIdentity.setClusterId(OID::gen());
+        ASSERT_OK(shardIdentity.validate());
+        return shardIdentity.toShardIdentityDocument();
+    }();
+
+    // An OpObserver will react to this insertion and initialize the ShardingState.
+    _dbDirectClient->insert(NamespaceString::kServerConfigurationNamespace.toString(),
+                            validShardIdentity);
+    ASSERT(shardingState()->enabled());
+
+    // This call represents the one done by the onInitialDataAvailable. It should be a no-op.
+    ASSERT(shardingInitialization()->initializeShardingAwarenessIfNeeded(operationContext()));
+}
+
+TEST_F(ShardingInitializationMongoDTest,
+       InitializeShardingAwarenessIfNeededNotQueryableBackupModeAndShardServerAsIfFCBIS) {
+    // No valid ShardIdentity yet, since we will get it through initial sync.
+    ASSERT(!shardingInitialization()->initializeShardingAwarenessIfNeeded(operationContext()));
+
+    // Insert the shardIdentity doc to disk while pretending that we are in "standalone" mode,
+    // otherwise OpObserver for inserts will prevent the insert from occurring because the
+    // shardIdentity doc is invalid
+    {
+        ScopedSetStandaloneMode standalone(getServiceContext());
+
+        BSONObj validShardIdentity = [&] {
+            ShardIdentityType shardIdentity;
+            shardIdentity.setConfigsvrConnectionString(ConnectionString(
+                ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config"));
+            shardIdentity.setShardName(kShardName);
+            shardIdentity.setClusterId(OID::gen());
+            ASSERT_OK(shardIdentity.validate());
+            return shardIdentity.toShardIdentityDocument();
+        }();
+
+        _dbDirectClient->insert(NamespaceString::kServerConfigurationNamespace.toString(),
+                                validShardIdentity);
+    }
+
+    ASSERT(!shardingState()->enabled());
+
+    // This call represents the one done by the onInitialDataAvailable. It should be a no-op.
+    ASSERT(shardingInitialization()->initializeShardingAwarenessIfNeeded(operationContext()));
 }
 
 /**
- * not readOnly and not --shardsvr
+ * not queryableBackupMode and not --shardsvr
  */
-TEST_F(ShardingInitializationMongoDTest,
-       InitializeShardingAwarenessIfNeededNotReadOnlyAndNotShardServerAndNoShardIdentity) {
+TEST_F(
+    ShardingInitializationMongoDTest,
+    InitializeShardingAwarenessIfNeededNotQueryableBackupModeAndNotShardServerAndNoShardIdentity) {
     ScopedSetStandaloneMode standalone(getServiceContext());
 
     ASSERT(!shardingInitialization()->initializeShardingAwarenessIfNeeded(operationContext()));
 }
 
-TEST_F(ShardingInitializationMongoDTest,
-       InitializeShardingAwarenessIfNeededNotReadOnlyAndNotShardServerAndInvalidShardIdentity) {
+TEST_F(
+    ShardingInitializationMongoDTest,
+    InitializeShardingAwarenessIfNeededNotQueryableBackupModeAndNotShardServerAndInvalidShardIdentity) {
     ScopedSetStandaloneMode standalone(getServiceContext());
 
     _dbDirectClient->insert(NamespaceString::kServerConfigurationNamespace.toString(),
@@ -513,14 +619,15 @@ TEST_F(ShardingInitializationMongoDTest,
     ASSERT(!shardingInitialization()->initializeShardingAwarenessIfNeeded(operationContext()));
 }
 
-TEST_F(ShardingInitializationMongoDTest,
-       InitializeShardingAwarenessIfNeededNotReadOnlyAndNotShardServerAndValidShardIdentity) {
+TEST_F(
+    ShardingInitializationMongoDTest,
+    InitializeShardingAwarenessIfNeededNotQueryableBackupModeAndNotShardServerAndValidShardIdentity) {
     ScopedSetStandaloneMode standalone(getServiceContext());
 
     BSONObj validShardIdentity = [&] {
         ShardIdentityType shardIdentity;
         shardIdentity.setConfigsvrConnectionString(
-            ConnectionString(ConnectionString::SET, "a:1,b:2", "config"));
+            ConnectionString(ConnectionString::ConnectionType::kReplicaSet, "a:1,b:2", "config"));
         shardIdentity.setShardName(kShardName);
         shardIdentity.setClusterId(OID::gen());
         ASSERT_OK(shardIdentity.validate());
