@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
@@ -35,7 +34,6 @@
 
 #include "mongo/base/status_with.h"
 #include "mongo/db/bson/dotted_path_support.h"
-#include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/dbhelpers.h"
 #include "mongo/db/exec/working_set_common.h"
@@ -44,8 +42,13 @@
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/query/internal_plans.h"
 #include "mongo/db/query/plan_executor.h"
+#include "mongo/db/s/collection_sharding_runtime.h"
+#include "mongo/db/s/shard_key_index_util.h"
 #include "mongo/logv2/log.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
+
+
 namespace mongo {
 namespace {
 
@@ -68,15 +71,15 @@ auto orderShardKeyFields(const BSONObj& keyPattern, const BSONObj& key) {
 
 }  // namespace
 
-StatusWith<std::vector<BSONObj>> splitVector(OperationContext* opCtx,
-                                             const NamespaceString& nss,
-                                             const BSONObj& keyPattern,
-                                             const BSONObj& min,
-                                             const BSONObj& max,
-                                             bool force,
-                                             boost::optional<long long> maxSplitPoints,
-                                             boost::optional<long long> maxChunkObjects,
-                                             boost::optional<long long> maxChunkSizeBytes) {
+std::vector<BSONObj> splitVector(OperationContext* opCtx,
+                                 const NamespaceString& nss,
+                                 const BSONObj& keyPattern,
+                                 const BSONObj& min,
+                                 const BSONObj& max,
+                                 bool force,
+                                 boost::optional<long long> maxSplitPoints,
+                                 boost::optional<long long> maxChunkObjects,
+                                 boost::optional<long long> maxChunkSizeBytes) {
     std::vector<BSONObj> splitKeys;
     std::size_t splitVectorResponseSize = 0;
 
@@ -86,25 +89,26 @@ StatusWith<std::vector<BSONObj>> splitVector(OperationContext* opCtx,
     }
 
     {
-        AutoGetCollection autoColl(opCtx, nss, MODE_IS);
+        AutoGetCollection collection(opCtx, nss, MODE_IS);
 
-        Collection* const collection = autoColl.getCollection();
-        if (!collection) {
-            return {ErrorCodes::NamespaceNotFound, "ns not found"};
-        }
+        CollectionShardingState::get(opCtx, nss)->checkShardVersionOrThrow(opCtx);
+
+        uassert(ErrorCodes::NamespaceNotFound, "ns not found", collection);
 
         // Allow multiKey based on the invariant that shard keys must be single-valued. Therefore,
         // any multi-key index prefixed by shard key cannot be multikey over the shard key fields.
-        const IndexDescriptor* idx =
-            collection->getIndexCatalog()->findShardKeyPrefixedIndex(opCtx, keyPattern, false);
-        if (idx == nullptr) {
-            return {ErrorCodes::IndexNotFound,
-                    "couldn't find index over splitting key " +
-                        keyPattern.clientReadable().toString()};
-        }
+        auto shardKeyIdx = findShardKeyPrefixedIndex(opCtx,
+                                                     *collection,
+                                                     collection->getIndexCatalog(),
+                                                     keyPattern,
+                                                     /*requireSingleKey=*/false);
+        uassert(ErrorCodes::IndexNotFound,
+                str::stream() << "couldn't find index over splitting key "
+                              << keyPattern.clientReadable().toString(),
+                shardKeyIdx);
 
         // extend min to get (min, MinKey, MinKey, ....)
-        KeyPattern kp(idx->keyPattern());
+        KeyPattern kp(shardKeyIdx->keyPattern());
         BSONObj minKey = Helpers::toKeyFormat(kp.extendRangeBound(min, false));
         BSONObj maxKey;
         if (max.isEmpty()) {
@@ -130,12 +134,12 @@ StatusWith<std::vector<BSONObj>> splitVector(OperationContext* opCtx,
         }
 
         // We need a maximum size for the chunk.
-        if (!maxChunkSizeBytes || maxChunkSizeBytes.get() <= 0) {
-            return {ErrorCodes::InvalidOptions, "need to specify the desired max chunk size"};
+        if (!maxChunkSizeBytes || maxChunkSizeBytes.value() <= 0) {
+            uasserted(ErrorCodes::InvalidOptions, "need to specify the desired max chunk size");
         }
 
         // If there's not enough data for more than one chunk, no point continuing.
-        if (dataSize < maxChunkSizeBytes.get() || recCount == 0) {
+        if (dataSize < maxChunkSizeBytes.value() || recCount == 0) {
             std::vector<BSONObj> emptyVector;
             return emptyVector;
         }
@@ -152,18 +156,18 @@ StatusWith<std::vector<BSONObj>> splitVector(OperationContext* opCtx,
         // maxChunkObjects, if provided.
         const long long avgRecSize = dataSize / recCount;
 
-        long long keyCount = maxChunkSizeBytes.get() / (2 * avgRecSize);
+        long long keyCount = maxChunkSizeBytes.value() / (2 * avgRecSize);
 
-        if (maxChunkObjects.get() && (maxChunkObjects.get() < keyCount)) {
+        if (maxChunkObjects.value() && (maxChunkObjects.value() < keyCount)) {
             LOGV2(22108,
                   "Limiting the number of documents per chunk to {maxChunkObjects} based "
                   "on the maxChunkObjects parameter for split vector command (compared to maximum "
                   "possible: {maxPossibleDocumentsPerChunk})",
                   "Limiting the number of documents per chunk for split vector command based on "
                   "the maxChunksObject parameter",
-                  "maxChunkObjects"_attr = maxChunkObjects.get(),
+                  "maxChunkObjects"_attr = maxChunkObjects.value(),
                   "maxPossibleDocumentsPerChunk"_attr = keyCount);
-            keyCount = maxChunkObjects.get();
+            keyCount = maxChunkObjects.value();
         }
 
         //
@@ -176,40 +180,38 @@ StatusWith<std::vector<BSONObj>> splitVector(OperationContext* opCtx,
         long long currCount = 0;
         long long numChunks = 0;
 
-        auto exec = InternalPlanner::indexScan(opCtx,
-                                               collection,
-                                               idx,
-                                               minKey,
-                                               maxKey,
-                                               BoundInclusion::kIncludeStartKeyOnly,
-                                               PlanExecutor::YIELD_AUTO,
-                                               InternalPlanner::FORWARD);
+        auto exec = InternalPlanner::shardKeyIndexScan(opCtx,
+                                                       &collection.getCollection(),
+                                                       *shardKeyIdx,
+                                                       minKey,
+                                                       maxKey,
+                                                       BoundInclusion::kIncludeStartKeyOnly,
+                                                       PlanYieldPolicy::YieldPolicy::YIELD_AUTO,
+                                                       InternalPlanner::FORWARD);
 
         BSONObj currKey;
         PlanExecutor::ExecState state = exec->getNext(&currKey, nullptr);
-        if (PlanExecutor::ADVANCED != state) {
-            return {ErrorCodes::OperationFailed,
-                    "can't open a cursor to scan the range (desired range is possibly empty)"};
-        }
+        uassert(ErrorCodes::OperationFailed,
+                "can't open a cursor to scan the range (desired range is possibly empty)",
+                state == PlanExecutor::ADVANCED);
 
         // Get the final key in the range, and see if it's the same as the first key.
         BSONObj maxKeyInChunk;
         {
-            auto exec = InternalPlanner::indexScan(opCtx,
-                                                   collection,
-                                                   idx,
-                                                   maxKey,
-                                                   minKey,
-                                                   BoundInclusion::kIncludeEndKeyOnly,
-                                                   PlanExecutor::YIELD_AUTO,
-                                                   InternalPlanner::BACKWARD);
+            auto exec = InternalPlanner::shardKeyIndexScan(opCtx,
+                                                           &collection.getCollection(),
+                                                           *shardKeyIdx,
+                                                           maxKey,
+                                                           minKey,
+                                                           BoundInclusion::kIncludeEndKeyOnly,
+                                                           PlanYieldPolicy::YieldPolicy::YIELD_AUTO,
+                                                           InternalPlanner::BACKWARD);
 
             PlanExecutor::ExecState state = exec->getNext(&maxKeyInChunk, nullptr);
-            if (PlanExecutor::ADVANCED != state) {
-                return {ErrorCodes::OperationFailed,
-                        "can't open a cursor to find final key in range (desired range is possibly "
-                        "empty)"};
-            }
+            uassert(
+                ErrorCodes::OperationFailed,
+                "can't open a cursor to find final key in range (desired range is possibly empty)",
+                state == PlanExecutor::ADVANCED);
         }
 
         if (currKey.woCompare(maxKeyInChunk) == 0) {
@@ -221,9 +223,9 @@ StatusWith<std::vector<BSONObj>> splitVector(OperationContext* opCtx,
                 "{maxKey} contains only the key {key}",
                 "Possible low cardinality key detected in range. Range contains only a single key.",
                 "namespace"_attr = nss.toString(),
-                "minKey"_attr = redact(prettyKey(idx->keyPattern(), minKey)),
-                "maxKey"_attr = redact(prettyKey(idx->keyPattern(), maxKey)),
-                "key"_attr = redact(prettyKey(idx->keyPattern(), currKey)));
+                "minKey"_attr = redact(prettyKey(shardKeyIdx->keyPattern(), minKey)),
+                "maxKey"_attr = redact(prettyKey(shardKeyIdx->keyPattern(), maxKey)),
+                "key"_attr = redact(prettyKey(shardKeyIdx->keyPattern(), currKey)));
             std::vector<BSONObj> emptyVector;
             return emptyVector;
         }
@@ -241,9 +243,12 @@ StatusWith<std::vector<BSONObj>> splitVector(OperationContext* opCtx,
                 if (currCount > keyCount && !force) {
                     currKey = orderShardKeyFields(keyPattern, currKey.getOwned());
 
-                    // Do not use this split key if it is the same used in the previous split
-                    // point.
+                    const auto compareWithPreviousSplitPoint = currKey.woCompare(splitKeys.back());
+                    dassert(compareWithPreviousSplitPoint >= 0,
+                            str::stream() << "Found split key smaller then the previous one: "
+                                          << currKey << " < " << splitKeys.back());
                     if (currKey.woCompare(splitKeys.back()) == 0) {
+                        // Do not use this split key if it is the same of the previous split point.
                         tooFrequentKeys.insert(currKey.getOwned());
                     } else {
                         auto additionalKeySize =
@@ -262,8 +267,10 @@ StatusWith<std::vector<BSONObj>> splitVector(OperationContext* opCtx,
                                   "Max BSON response size reached for split vector before the end "
                                   "of chunk",
                                   "namespace"_attr = nss.toString(),
-                                  "minKey"_attr = redact(prettyKey(idx->keyPattern(), minKey)),
-                                  "maxKey"_attr = redact(prettyKey(idx->keyPattern(), maxKey)));
+                                  "minKey"_attr =
+                                      redact(prettyKey(shardKeyIdx->keyPattern(), minKey)),
+                                  "maxKey"_attr =
+                                      redact(prettyKey(shardKeyIdx->keyPattern(), maxKey)));
                             break;
                         }
 
@@ -280,26 +287,22 @@ StatusWith<std::vector<BSONObj>> splitVector(OperationContext* opCtx,
                 }
 
                 // Stop if we have enough split points.
-                if (maxSplitPoints && maxSplitPoints.get() && (numChunks >= maxSplitPoints.get())) {
+                if (maxSplitPoints && maxSplitPoints.value() &&
+                    (numChunks >= maxSplitPoints.value())) {
                     LOGV2(22111,
                           "Max number of requested split points reached ({numSplitPoints}) before "
                           "the end of chunk {namespace} {minKey} -->> {maxKey}",
                           "Max number of requested split points reached before the end of chunk",
                           "numSplitPoints"_attr = numChunks,
                           "namespace"_attr = nss.toString(),
-                          "minKey"_attr = redact(prettyKey(idx->keyPattern(), minKey)),
-                          "maxKey"_attr = redact(prettyKey(idx->keyPattern(), maxKey)));
+                          "minKey"_attr = redact(prettyKey(shardKeyIdx->keyPattern(), minKey)),
+                          "maxKey"_attr = redact(prettyKey(shardKeyIdx->keyPattern(), maxKey)));
                     break;
                 }
 
                 state = exec->getNext(&currKey, nullptr);
             }
 
-            if (PlanExecutor::FAILURE == state) {
-                return WorkingSetCommon::getMemberObjectStatus(currKey).withContext(
-                    "Executor error during splitVector command");
-            }
-
             if (!force)
                 break;
 
@@ -316,14 +319,14 @@ StatusWith<std::vector<BSONObj>> splitVector(OperationContext* opCtx,
                   "splitVector doing another cycle because of force",
                   "keyCount"_attr = keyCount);
 
-            exec = InternalPlanner::indexScan(opCtx,
-                                              collection,
-                                              idx,
-                                              minKey,
-                                              maxKey,
-                                              BoundInclusion::kIncludeStartKeyOnly,
-                                              PlanExecutor::YIELD_AUTO,
-                                              InternalPlanner::FORWARD);
+            exec = InternalPlanner::shardKeyIndexScan(opCtx,
+                                                      &collection.getCollection(),
+                                                      *shardKeyIdx,
+                                                      minKey,
+                                                      maxKey,
+                                                      BoundInclusion::kIncludeStartKeyOnly,
+                                                      PlanYieldPolicy::YieldPolicy::YIELD_AUTO,
+                                                      InternalPlanner::FORWARD);
 
             state = exec->getNext(&currKey, nullptr);
         }
@@ -340,7 +343,7 @@ StatusWith<std::vector<BSONObj>> splitVector(OperationContext* opCtx,
                           "{key}",
                           "Possible low cardinality key detected",
                           "namespace"_attr = nss.toString(),
-                          "key"_attr = redact(prettyKey(idx->keyPattern(), *it)));
+                          "key"_attr = redact(prettyKey(shardKeyIdx->keyPattern(), *it)));
         }
 
         // Remove the sentinel at the beginning before returning
@@ -361,10 +364,6 @@ StatusWith<std::vector<BSONObj>> splitVector(OperationContext* opCtx,
         }
     }
 
-    // Make sure splitKeys is in ascending order
-    std::sort(
-        splitKeys.begin(), splitKeys.end(), SimpleBSONObjComparator::kInstance.makeLessThan());
-
     return splitKeys;
 }
 
