@@ -27,14 +27,19 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/platform/basic.h"
 
+#include "mongo/db/catalog/create_collection.h"
+#include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/s/collection_sharding_runtime.h"
+#include "mongo/db/s/operation_sharding_state.h"
+#include "mongo/db/s/shard_server_test_fixture.h"
 #include "mongo/db/s/split_vector.h"
-#include "mongo/s/shard_server_test_fixture.h"
-#include "mongo/unittest/unittest.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
 
 namespace mongo {
 namespace {
@@ -42,14 +47,28 @@ namespace {
 const NamespaceString kNss = NamespaceString("foo", "bar");
 const std::string kPattern = "_id";
 
+void setUnshardedFilteringMetadata(OperationContext* opCtx, const NamespaceString& nss) {
+    AutoGetDb autoDb(opCtx, nss.dbName(), MODE_IX);
+    Lock::CollectionLock collLock(opCtx, nss, MODE_IX);
+    CollectionShardingRuntime::get(opCtx, nss)->setFilteringMetadata(opCtx, CollectionMetadata());
+}
+
 class SplitVectorTest : public ShardServerTestFixture {
 public:
     void setUp() {
         ShardServerTestFixture::setUp();
 
-        DBDirectClient dbclient(operationContext());
-        ASSERT_TRUE(dbclient.createCollection(kNss.ns()));
-        dbclient.createIndex(kNss.ns(), BSON(kPattern << 1));
+        auto opCtx = operationContext();
+
+        {
+            OperationShardingState::ScopedAllowImplicitCollectionCreate_UNSAFE
+                unsafeCreateCollection(opCtx);
+            uassertStatusOK(
+                createCollection(operationContext(), kNss.dbName(), BSON("create" << kNss.coll())));
+        }
+        setUnshardedFilteringMetadata(opCtx, kNss);
+        DBDirectClient client(opCtx);
+        client.createIndex(kNss.ns(), BSON(kPattern << 1));
 
         // Insert 100 documents into the collection so the tests can test splitting with different
         // constraints.
@@ -57,9 +76,9 @@ public:
             BSONObjBuilder builder;
             builder.append(kPattern, i);
             BSONObj obj = builder.obj();
-            dbclient.insert(kNss.toString(), obj);
+            client.insert(kNss.toString(), obj);
         }
-        ASSERT_EQUALS(100ULL, dbclient.count(kNss));
+        ASSERT_EQUALS(100ULL, client.count(kNss));
     }
 
     const long long& getDocSizeBytes() {
@@ -72,15 +91,15 @@ private:
 };
 
 TEST_F(SplitVectorTest, SplitVectorInHalf) {
-    std::vector<BSONObj> splitKeys = unittest::assertGet(splitVector(operationContext(),
-                                                                     kNss,
-                                                                     BSON(kPattern << 1),
-                                                                     BSON(kPattern << 0),
-                                                                     BSON(kPattern << 100),
-                                                                     false,
-                                                                     boost::none,
-                                                                     boost::none,
-                                                                     getDocSizeBytes() * 100LL));
+    std::vector<BSONObj> splitKeys = splitVector(operationContext(),
+                                                 kNss,
+                                                 BSON(kPattern << 1),
+                                                 BSON(kPattern << 0),
+                                                 BSON(kPattern << 100),
+                                                 false,
+                                                 boost::none,
+                                                 boost::none,
+                                                 getDocSizeBytes() * 100LL);
     std::vector<BSONObj> expected = {BSON(kPattern << 50)};
     ASSERT_EQ(splitKeys.size(), expected.size());
 
@@ -92,15 +111,15 @@ TEST_F(SplitVectorTest, SplitVectorInHalf) {
 }
 
 TEST_F(SplitVectorTest, ForceSplit) {
-    std::vector<BSONObj> splitKeys = unittest::assertGet(splitVector(operationContext(),
-                                                                     kNss,
-                                                                     BSON(kPattern << 1),
-                                                                     BSON(kPattern << 0),
-                                                                     BSON(kPattern << 100),
-                                                                     true,
-                                                                     boost::none,
-                                                                     boost::none,
-                                                                     getDocSizeBytes() * 6LL));
+    std::vector<BSONObj> splitKeys = splitVector(operationContext(),
+                                                 kNss,
+                                                 BSON(kPattern << 1),
+                                                 BSON(kPattern << 0),
+                                                 BSON(kPattern << 100),
+                                                 true,
+                                                 boost::none,
+                                                 boost::none,
+                                                 getDocSizeBytes() * 6LL);
     std::vector<BSONObj> expected = {BSON(kPattern << 50)};
     ASSERT_EQ(splitKeys.size(), expected.size());
 
@@ -112,15 +131,15 @@ TEST_F(SplitVectorTest, ForceSplit) {
 }
 
 TEST_F(SplitVectorTest, MaxChunkObjectsSet) {
-    std::vector<BSONObj> splitKeys = unittest::assertGet(splitVector(operationContext(),
-                                                                     kNss,
-                                                                     BSON(kPattern << 1),
-                                                                     BSON(kPattern << 0),
-                                                                     BSON(kPattern << 100),
-                                                                     false,
-                                                                     boost::none,
-                                                                     10,
-                                                                     getDocSizeBytes() * 100LL));
+    std::vector<BSONObj> splitKeys = splitVector(operationContext(),
+                                                 kNss,
+                                                 BSON(kPattern << 1),
+                                                 BSON(kPattern << 0),
+                                                 BSON(kPattern << 100),
+                                                 false,
+                                                 boost::none,
+                                                 10,
+                                                 getDocSizeBytes() * 100LL);
     // Unlike the SplitVectorInHalf test, should split at every 10th key.
     std::vector<BSONObj> expected = {BSON(kPattern << 10),
                                      BSON(kPattern << 21),
@@ -141,15 +160,15 @@ TEST_F(SplitVectorTest, MaxChunkObjectsSet) {
 }
 
 TEST_F(SplitVectorTest, SplitEveryThird) {
-    std::vector<BSONObj> splitKeys = unittest::assertGet(splitVector(operationContext(),
-                                                                     kNss,
-                                                                     BSON(kPattern << 1),
-                                                                     BSON(kPattern << 0),
-                                                                     BSON(kPattern << 100),
-                                                                     false,
-                                                                     boost::none,
-                                                                     boost::none,
-                                                                     getDocSizeBytes() * 6LL));
+    std::vector<BSONObj> splitKeys = splitVector(operationContext(),
+                                                 kNss,
+                                                 BSON(kPattern << 1),
+                                                 BSON(kPattern << 0),
+                                                 BSON(kPattern << 100),
+                                                 false,
+                                                 boost::none,
+                                                 boost::none,
+                                                 getDocSizeBytes() * 6LL);
     std::vector<BSONObj> expected = {
         BSON(kPattern << 3),  BSON(kPattern << 7),  BSON(kPattern << 11), BSON(kPattern << 15),
         BSON(kPattern << 19), BSON(kPattern << 23), BSON(kPattern << 27), BSON(kPattern << 31),
@@ -168,15 +187,15 @@ TEST_F(SplitVectorTest, SplitEveryThird) {
 }
 
 TEST_F(SplitVectorTest, MaxSplitPointsSet) {
-    std::vector<BSONObj> splitKeys = unittest::assertGet(splitVector(operationContext(),
-                                                                     kNss,
-                                                                     BSON(kPattern << 1),
-                                                                     BSON(kPattern << 0),
-                                                                     BSON(kPattern << 100),
-                                                                     false,
-                                                                     3,
-                                                                     boost::none,
-                                                                     getDocSizeBytes() * 6LL));
+    std::vector<BSONObj> splitKeys = splitVector(operationContext(),
+                                                 kNss,
+                                                 BSON(kPattern << 1),
+                                                 BSON(kPattern << 0),
+                                                 BSON(kPattern << 100),
+                                                 false,
+                                                 3,
+                                                 boost::none,
+                                                 getDocSizeBytes() * 6LL);
     // Unlike the SplitEveryThird test, should only return the first 3 split points since
     // maxSplitPoints is 3.
     std::vector<BSONObj> expected = {
@@ -191,15 +210,15 @@ TEST_F(SplitVectorTest, MaxSplitPointsSet) {
 }
 
 TEST_F(SplitVectorTest, IgnoreMaxChunkObjects) {
-    std::vector<BSONObj> splitKeys = unittest::assertGet(splitVector(operationContext(),
-                                                                     kNss,
-                                                                     BSON(kPattern << 1),
-                                                                     BSON(kPattern << 0),
-                                                                     BSON(kPattern << 100),
-                                                                     false,
-                                                                     boost::none,
-                                                                     10,
-                                                                     getDocSizeBytes() * 6LL));
+    std::vector<BSONObj> splitKeys = splitVector(operationContext(),
+                                                 kNss,
+                                                 BSON(kPattern << 1),
+                                                 BSON(kPattern << 0),
+                                                 BSON(kPattern << 100),
+                                                 false,
+                                                 boost::none,
+                                                 10,
+                                                 getDocSizeBytes() * 6LL);
     // The "maxChunkObjects"th key (10) is larger than the key count at half the maxChunkSize (3),
     // so it should be ignored.
     std::vector<BSONObj> expected = {
@@ -220,59 +239,59 @@ TEST_F(SplitVectorTest, IgnoreMaxChunkObjects) {
 }
 
 TEST_F(SplitVectorTest, NoSplit) {
-    std::vector<BSONObj> splitKeys = unittest::assertGet(splitVector(operationContext(),
-                                                                     kNss,
-                                                                     BSON(kPattern << 1),
-                                                                     BSON(kPattern << 0),
-                                                                     BSON(kPattern << 100),
-                                                                     false,
-                                                                     boost::none,
-                                                                     boost::none,
-                                                                     getDocSizeBytes() * 1000LL));
+    std::vector<BSONObj> splitKeys = splitVector(operationContext(),
+                                                 kNss,
+                                                 BSON(kPattern << 1),
+                                                 BSON(kPattern << 0),
+                                                 BSON(kPattern << 100),
+                                                 false,
+                                                 boost::none,
+                                                 boost::none,
+                                                 getDocSizeBytes() * 1000LL);
 
     ASSERT_EQUALS(splitKeys.size(), 0UL);
 }
 
 TEST_F(SplitVectorTest, NoCollection) {
-    auto status = splitVector(operationContext(),
-                              NamespaceString("dummy", "collection"),
-                              BSON(kPattern << 1),
-                              BSON(kPattern << 0),
-                              BSON(kPattern << 100),
-                              false,
-                              boost::none,
-                              boost::none,
-                              boost::none)
-                      .getStatus();
-    ASSERT_EQUALS(status.code(), ErrorCodes::NamespaceNotFound);
+    ASSERT_THROWS_CODE(splitVector(operationContext(),
+                                   NamespaceString("dummy", "collection"),
+                                   BSON(kPattern << 1),
+                                   BSON(kPattern << 0),
+                                   BSON(kPattern << 100),
+                                   false,
+                                   boost::none,
+                                   boost::none,
+                                   boost::none),
+                       DBException,
+                       ErrorCodes::NamespaceNotFound);
 }
 
 TEST_F(SplitVectorTest, NoIndex) {
-    auto status = splitVector(operationContext(),
-                              kNss,
-                              BSON("foo" << 1),
-                              BSON(kPattern << 0),
-                              BSON(kPattern << 100),
-                              false,
-                              boost::none,
-                              boost::none,
-                              boost::none)
-                      .getStatus();
-    ASSERT_EQUALS(status.code(), ErrorCodes::IndexNotFound);
+    ASSERT_THROWS_CODE(splitVector(operationContext(),
+                                   kNss,
+                                   BSON("foo" << 1),
+                                   BSON(kPattern << 0),
+                                   BSON(kPattern << 100),
+                                   false,
+                                   boost::none,
+                                   boost::none,
+                                   boost::none),
+                       DBException,
+                       ErrorCodes::IndexNotFound);
 }
 
 TEST_F(SplitVectorTest, NoMaxChunkSize) {
-    auto status = splitVector(operationContext(),
-                              kNss,
-                              BSON(kPattern << 1),
-                              BSON(kPattern << 0),
-                              BSON(kPattern << 100),
-                              false,
-                              boost::none,
-                              boost::none,
-                              boost::none)
-                      .getStatus();
-    ASSERT_EQUALS(status.code(), ErrorCodes::InvalidOptions);
+    ASSERT_THROWS_CODE(splitVector(operationContext(),
+                                   kNss,
+                                   BSON(kPattern << 1),
+                                   BSON(kPattern << 0),
+                                   BSON(kPattern << 100),
+                                   false,
+                                   boost::none,
+                                   boost::none,
+                                   boost::none),
+                       DBException,
+                       ErrorCodes::InvalidOptions);
 }
 
 const NamespaceString kJumboNss = NamespaceString("foo", "bar2");
@@ -283,18 +302,26 @@ public:
     void setUp() {
         ShardServerTestFixture::setUp();
 
-        DBDirectClient dbclient(operationContext());
-        ASSERT_TRUE(dbclient.createCollection(kJumboNss.ns()));
-        dbclient.createIndex(kJumboNss.ns(), BSON(kJumboPattern << 1));
+        auto opCtx = operationContext();
+
+        {
+            OperationShardingState::ScopedAllowImplicitCollectionCreate_UNSAFE
+                unsafeCreateCollection(opCtx);
+            uassertStatusOK(createCollection(
+                operationContext(), kJumboNss.dbName(), BSON("create" << kJumboNss.coll())));
+        }
+        setUnshardedFilteringMetadata(opCtx, kJumboNss);
+        DBDirectClient client(opCtx);
+        client.createIndex(kJumboNss.ns(), BSON(kJumboPattern << 1));
 
         // Insert 10000 documents into the collection with the same shard key value.
         BSONObjBuilder builder;
         builder.append(kJumboPattern, 1);
         BSONObj obj = builder.obj();
         for (int i = 0; i < 1000; i++) {
-            dbclient.insert(kJumboNss.toString(), obj);
+            client.insert(kJumboNss.toString(), obj);
         }
-        ASSERT_EQUALS(1000ULL, dbclient.count(kJumboNss));
+        ASSERT_EQUALS(1000ULL, client.count(kJumboNss));
     }
 
     const long long& getDocSizeBytes() {
@@ -307,15 +334,15 @@ private:
 };
 
 TEST_F(SplitVectorJumboTest, JumboChunk) {
-    std::vector<BSONObj> splitKeys = unittest::assertGet(splitVector(operationContext(),
-                                                                     kJumboNss,
-                                                                     BSON(kJumboPattern << 1),
-                                                                     BSON(kJumboPattern << 1),
-                                                                     BSON(kJumboPattern << 2),
-                                                                     false,
-                                                                     boost::none,
-                                                                     boost::none,
-                                                                     getDocSizeBytes() * 1LL));
+    std::vector<BSONObj> splitKeys = splitVector(operationContext(),
+                                                 kJumboNss,
+                                                 BSON(kJumboPattern << 1),
+                                                 BSON(kJumboPattern << 1),
+                                                 BSON(kJumboPattern << 2),
+                                                 false,
+                                                 boost::none,
+                                                 boost::none,
+                                                 getDocSizeBytes() * 1LL);
 
     ASSERT_EQUALS(splitKeys.size(), 0UL);
 }
@@ -338,9 +365,18 @@ public:
     void setUp() {
         ShardServerTestFixture::setUp();
 
-        DBDirectClient dbclient(operationContext());
-        ASSERT_TRUE(dbclient.createCollection(kMaxResponseNss.ns()));
-        dbclient.createIndex(kMaxResponseNss.ns(), BSON("a" << 1));
+        auto opCtx = operationContext();
+
+        {
+            OperationShardingState::ScopedAllowImplicitCollectionCreate_UNSAFE
+                unsafeCreateCollection(opCtx);
+            uassertStatusOK(createCollection(operationContext(),
+                                             kMaxResponseNss.dbName(),
+                                             BSON("create" << kMaxResponseNss.coll())));
+        }
+        setUnshardedFilteringMetadata(opCtx, kMaxResponseNss);
+        DBDirectClient client(opCtx);
+        client.createIndex(kMaxResponseNss.ns(), BSON("a" << 1));
 
         for (int i = 0; i < numDocs; ++i) {
             BSONObjBuilder builder;
@@ -348,9 +384,9 @@ public:
             // ensure that our documents are unique.
             builder.append("a", createUniqueHalfMegabyteString(i));
             BSONObj obj = builder.obj();
-            dbclient.insert(kMaxResponseNss.toString(), obj);
+            client.insert(kMaxResponseNss.toString(), obj);
         }
-        ASSERT_EQUALS(numDocs, (int)dbclient.count(kMaxResponseNss));
+        ASSERT_EQUALS(numDocs, (int)client.count(kMaxResponseNss));
     }
 
     std::string createUniqueHalfMegabyteString(int uniqueInt) {
@@ -364,15 +400,15 @@ public:
 };
 
 TEST_F(SplitVectorMaxResponseSizeTest, MaxResponseSize) {
-    std::vector<BSONObj> splitKeys = unittest::assertGet(splitVector(operationContext(),
-                                                                     kMaxResponseNss,
-                                                                     BSON("a" << 1),
-                                                                     {},
-                                                                     {},
-                                                                     false,
-                                                                     boost::none,
-                                                                     boost::none,
-                                                                     1LL));
+    std::vector<BSONObj> splitKeys = splitVector(operationContext(),
+                                                 kMaxResponseNss,
+                                                 BSON("a" << 1),
+                                                 {},
+                                                 {},
+                                                 false,
+                                                 boost::none,
+                                                 boost::none,
+                                                 1LL);
 
     ASSERT_EQUALS((int)splitKeys.size(), numDocs - 2);
 
