@@ -33,10 +33,10 @@
 #include <vector>
 
 #include "mongo/client/read_preference.h"
+#include "mongo/db/shard_id.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/s/client/shard.h"
-#include "mongo/s/shard_id.h"
 #include "mongo/util/concurrency/mutex.h"
 #include "mongo/util/future.h"
 #include "mongo/util/time_support.h"
@@ -55,6 +55,13 @@ public:
     AsyncWorkScheduler(ServiceContext* serviceContext);
     ~AsyncWorkScheduler();
 
+    /**
+     * @brief Returns a shared pointer to the executor
+     */
+    ExecutorPtr getExecutor() {
+        return _executor;
+    }
+
     /**
      * Schedules the specified callable to execute asynchronously and returns a future which will be
      * set with its result.
