@@ -27,22 +27,25 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTransaction
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/catalog/uncommitted_catalog_updates.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/test_commands_enabled.h"
 #include "mongo/db/commands/txn_two_phase_commit_cmds_gen.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/s/sharding_state.h"
 #include "mongo/db/s/transaction_coordinator_service.h"
-#include "mongo/db/session_catalog_mongod.h"
-#include "mongo/db/transaction_participant.h"
+#include "mongo/db/session/session_catalog_mongod.h"
+#include "mongo/db/transaction/transaction_participant.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTransaction
+
+
 namespace mongo {
 namespace {
 
@@ -51,6 +54,19 @@ MONGO_FAIL_POINT_DEFINE(participantReturnNetworkErrorForPrepareAfterExecutingPre
 
 class PrepareTransactionCmd : public TypedCommand<PrepareTransactionCmd> {
 public:
+    bool skipApiVersionCheck() const override {
+        // Internal command (server to server).
+        return true;
+    }
+
+    bool isTransactionCommand() const final {
+        return true;
+    }
+
+    bool allowedInTransactions() const final {
+        return true;
+    }
+
     class PrepareTimestamp {
     public:
         PrepareTimestamp(Timestamp timestamp) : _timestamp(std::move(timestamp)) {}
@@ -100,17 +116,21 @@ public:
                     "prepareTransaction must be run within a transaction",
                     txnParticipant);
 
+            TxnNumberAndRetryCounter txnNumberAndRetryCounter{*opCtx->getTxnNumber(),
+                                                              *opCtx->getTxnRetryCounter()};
+
             LOGV2_DEBUG(22483,
                         3,
-                        "{sessionId}:{txnNumber} Participant shard received prepareTransaction",
+                        "{sessionId}:{txnNumberAndRetryCounter} Participant shard received "
+                        "prepareTransaction",
                         "Participant shard received prepareTransaction",
                         "sessionId"_attr = opCtx->getLogicalSessionId()->toBSON(),
-                        "txnNumber"_attr = opCtx->getTxnNumber());
+                        "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter);
 
             // TODO(SERVER-46105) remove
             uassert(ErrorCodes::OperationNotSupportedInTransaction,
                     "Cannot create new collections inside distributed transactions",
-                    UncommittedCollections::get(opCtx).isEmpty());
+                    UncommittedCatalogUpdates::get(opCtx).isEmpty());
 
             uassert(ErrorCodes::NoSuchTransaction,
                     "Transaction isn't in progress",
@@ -215,6 +235,7 @@ std::set<ShardId> validateParticipants(OperationContext* opCtx,
         "Coordinator shard received request to coordinate commit",
         "sessionId"_attr = opCtx->getLogicalSessionId()->getId(),
         "txnNumber"_attr = opCtx->getTxnNumber(),
+        "txnRetryCounter"_attr = opCtx->getTxnRetryCounter(),
         "participantList"_attr = ss.str());
 
     return participantsSet;
@@ -224,6 +245,11 @@ class CoordinateCommitTransactionCmd : public TypedCommand<CoordinateCommitTrans
 public:
     using Request = CoordinateCommitTransaction;
 
+    bool skipApiVersionCheck() const override {
+        // Internal command (server to server).
+        return true;
+    }
+
     class Invocation final : public InvocationBase {
     public:
         using InvocationBase::InvocationBase;
@@ -237,13 +263,16 @@ public:
             const auto& cmd = request();
             const auto tcs = TransactionCoordinatorService::get(opCtx);
 
+            const TxnNumberAndRetryCounter txnNumberAndRetryCounter{*opCtx->getTxnNumber(),
+                                                                    *opCtx->getTxnRetryCounter()};
+
             // Coordinate the commit, or recover the commit decision from disk if this command was
             // sent without a participant list.
             auto coordinatorDecisionFuture = cmd.getParticipants().empty()
-                ? tcs->recoverCommit(opCtx, *opCtx->getLogicalSessionId(), *opCtx->getTxnNumber())
+                ? tcs->recoverCommit(opCtx, *opCtx->getLogicalSessionId(), txnNumberAndRetryCounter)
                 : tcs->coordinateCommit(opCtx,
                                         *opCtx->getLogicalSessionId(),
-                                        *opCtx->getTxnNumber(),
+                                        txnNumberAndRetryCounter,
                                         validateParticipants(opCtx, cmd.getParticipants()));
 
             if (MONGO_unlikely(hangAfterStartingCoordinateCommit.shouldFail())) {
@@ -256,7 +285,7 @@ public:
                 // (in all cases except the one where this command aborts the local participant), so
                 // ensure waiting for the client's writeConcern of the decision.
                 repl::ReplClientInfo::forClient(opCtx->getClient())
-                    .setLastOpToSystemLastOpTimeIgnoringInterrupt(opCtx);
+                    .setLastOpToSystemLastOpTimeIgnoringShutdownCtxCancelled(opCtx);
             });
 
             if (coordinatorDecisionFuture) {
@@ -292,17 +321,18 @@ public:
 
             LOGV2_DEBUG(22486,
                         3,
-                        "{sessionId}:{txnNumber} Going to recover decision from local participant",
+                        "{sessionId}:{txnNumberAndRetryCounter} Going to recover decision from "
+                        "local participant",
                         "Going to recover decision from local participant",
                         "sessionId"_attr = opCtx->getLogicalSessionId()->getId(),
-                        "txnNumber"_attr = opCtx->getTxnNumber());
+                        "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter);
 
             boost::optional<SharedSemiFuture<void>> participantExitPrepareFuture;
             {
                 MongoDOperationContextSession sessionTxnState(opCtx);
                 auto txnParticipant = TransactionParticipant::get(opCtx);
                 txnParticipant.beginOrContinue(opCtx,
-                                               *opCtx->getTxnNumber(),
+                                               txnNumberAndRetryCounter,
                                                false /* autocommit */,
                                                boost::none /* startTransaction */);
 
@@ -324,7 +354,7 @@ public:
 
                 // Call beginOrContinue again in case the transaction number has changed.
                 txnParticipant.beginOrContinue(opCtx,
-                                               *opCtx->getTxnNumber(),
+                                               txnNumberAndRetryCounter,
                                                false /* autocommit */,
                                                boost::none /* startTransaction */);
 
@@ -361,6 +391,18 @@ public:
         return AllowedOnSecondary::kNever;
     }
 
+    bool isTransactionCommand() const final {
+        return true;
+    }
+
+    bool shouldCheckoutSession() const final {
+        return false;
+    }
+
+    bool allowedInTransactions() const final {
+        return true;
+    }
+
 } coordinateCommitTransactionCmd;
 
 }  // namespace
