@@ -27,8 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kControl
-
 #include "mongo/db/server_options_helpers.h"
 
 #ifdef _WIN32
@@ -39,6 +37,7 @@
 #endif
 #include <boost/filesystem.hpp>
 #include <boost/filesystem/operations.hpp>
+#include <fmt/format.h>
 #include <ios>
 #include <iostream>
 
@@ -48,22 +47,20 @@
 #include "mongo/config.h"
 #include "mongo/db/server_options.h"
 #include "mongo/idl/server_parameter.h"
-#include "mongo/logger/message_event_utf8_encoder.h"
+#include "mongo/logv2/log.h"
 #include "mongo/logv2/log_component.h"
 #include "mongo/logv2/log_component_settings.h"
 #include "mongo/logv2/log_manager.h"
 #include "mongo/transport/message_compressor_registry.h"
 #include "mongo/util/cmdline_utils/censor_cmdline.h"
 #include "mongo/util/fail_point.h"
-#include "mongo/util/map_util.h"
 #include "mongo/util/net/sock.h"
 #include "mongo/util/net/socket_utils.h"
 #include "mongo/util/net/ssl_options.h"
 #include "mongo/util/options_parser/startup_options.h"
 #include "mongo/util/str.h"
 
-using std::endl;
-using std::string;
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kControl
 
 namespace mongo {
 
@@ -151,19 +148,21 @@ Status validateBaseOptions(const moe::Environment& params) {
             globalFailPointRegistry().registerAllFailPointsAsServerParameters();
         } else {
             // Deregister test-only parameters.
-            ServerParameterSet::getGlobal()->disableTestParameters();
+            ServerParameterSet::getNodeParameterSet()->disableTestParameters();
+            ServerParameterSet::getClusterParameterSet()->disableTestParameters();
         }
 
         // Must come after registerAllFailPointsAsServerParameters() above.
-        const auto& spMap = ServerParameterSet::getGlobal()->getMap();
-        for (const auto setParam : parameters) {
-            const auto it = spMap.find(setParam.first);
+        auto* paramSet = ServerParameterSet::getNodeParameterSet();
+        for (const auto& setParam : parameters) {
+            auto* param = paramSet->getIfExists(setParam.first);
 
-            if (it == spMap.end()) {
+            if (!param) {
                 return {ErrorCodes::BadValue,
                         str::stream() << "Unknown --setParameter '" << setParam.first << "'"};
             }
-            if (!enableTestCommandsValue && it->second->isTestOnly()) {
+
+            if (!param->isEnabled()) {
                 return {ErrorCodes::BadValue,
                         str::stream() << "--setParameter '" << setParam.first
                                       << "' only available when used with 'enableTestCommands'"};
@@ -259,6 +258,31 @@ Status setupBaseOptions(const std::vector<std::string>& args) {
     return Status::OK();
 }
 
+namespace server_options_detail {
+StatusWith<BSONObj> applySetParameterOptions(const std::map<std::string, std::string>& toApply,
+                                             ServerParameterSet& parameterSet) {
+    using namespace fmt::literals;
+    BSONObjBuilder summaryBuilder;
+    for (const auto& [name, value] : toApply) {
+        auto sp = parameterSet.getIfExists(name);
+        if (!sp)
+            return Status(ErrorCodes::BadValue,
+                          "Illegal --setParameter parameter: \"{}\""_format(name));
+        if (!sp->allowedToChangeAtStartup())
+            return Status(ErrorCodes::BadValue,
+                          "Cannot use --setParameter to set \"{}\" at startup"_format(name));
+        BSONObjBuilder sub(summaryBuilder.subobjStart(name));
+        sp->append(nullptr, sub, "default");
+        Status status = sp->setFromString(value);
+        if (!status.isOK())
+            return Status(ErrorCodes::BadValue,
+                          "Bad value for parameter \"{}\": {}"_format(name, status.reason()));
+        sp->append(nullptr, sub, "value");
+    }
+    return summaryBuilder.obj();
+}
+}  // namespace server_options_detail
+
 Status storeBaseOptions(const moe::Environment& params) {
     Status ret = setParsedOpts(params);
     if (!ret.isOK()) {
@@ -282,7 +306,8 @@ Status storeBaseOptions(const moe::Environment& params) {
         if (component == logv2::LogComponent::kDefault) {
             continue;
         }
-        const string dottedName = "systemLog.component." + component.getDottedName() + ".verbosity";
+        const std::string dottedName =
+            "systemLog.component." + component.getDottedName() + ".verbosity";
         if (params.count(dottedName)) {
             int verbosity = params[dottedName].as<int>();
             // Clear existing log level if log level is negative.
@@ -310,14 +335,11 @@ Status storeBaseOptions(const moe::Environment& params) {
     }
 
     if (params.count("systemLog.timeStampFormat")) {
-        using logger::MessageEventDetailsEncoder;
-        std::string formatterName = params["systemLog.timeStampFormat"].as<string>();
+        std::string formatterName = params["systemLog.timeStampFormat"].as<std::string>();
         if (formatterName == "iso8601-utc") {
-            MessageEventDetailsEncoder::setDateFormatter(outputDateAsISOStringUTC);
             serverGlobalParams.logTimestampFormat = logv2::LogTimestampFormat::kISO8601UTC;
             setDateFormatIsLocalTimezone(false);
         } else if (formatterName == "iso8601-local") {
-            MessageEventDetailsEncoder::setDateFormatter(outputDateAsISOStringLocal);
             serverGlobalParams.logTimestampFormat = logv2::LogTimestampFormat::kISO8601Local;
             setDateFormatIsLocalTimezone(true);
         } else {
@@ -360,7 +382,7 @@ Status storeBaseOptions(const moe::Environment& params) {
 
 #ifndef _WIN32
     if (params.count("systemLog.syslogFacility")) {
-        std::string facility = params["systemLog.syslogFacility"].as<string>();
+        std::string facility = params["systemLog.syslogFacility"].as<std::string>();
         bool set = false;
         // match facility string to facility value
         size_t facilitynamesLength = sizeof(facilitynames) / sizeof(facilitynames[0]);
@@ -387,7 +409,7 @@ Status storeBaseOptions(const moe::Environment& params) {
     }
 
     if (params.count("systemLog.logRotate")) {
-        std::string logRotateParam = params["systemLog.logRotate"].as<string>();
+        std::string logRotateParam = params["systemLog.logRotate"].as<std::string>();
         if (logRotateParam == "reopen") {
             serverGlobalParams.logRenameOnRotate = false;
 
@@ -408,42 +430,22 @@ Status storeBaseOptions(const moe::Environment& params) {
     }
 
     if (params.count("processManagement.pidFilePath")) {
-        serverGlobalParams.pidFile = params["processManagement.pidFilePath"].as<string>();
+        serverGlobalParams.pidFile = params["processManagement.pidFilePath"].as<std::string>();
     }
 
     if (params.count("processManagement.timeZoneInfo")) {
-        serverGlobalParams.timeZoneInfoPath = params["processManagement.timeZoneInfo"].as<string>();
+        serverGlobalParams.timeZoneInfoPath =
+            params["processManagement.timeZoneInfo"].as<std::string>();
     }
 
     if (params.count("setParameter")) {
-        std::map<std::string, std::string> parameters =
-            params["setParameter"].as<std::map<std::string, std::string>>();
-        for (std::map<std::string, std::string>::iterator parametersIt = parameters.begin();
-             parametersIt != parameters.end();
-             parametersIt++) {
-            ServerParameter* parameter =
-                mapFindWithDefault(ServerParameterSet::getGlobal()->getMap(),
-                                   parametersIt->first,
-                                   static_cast<ServerParameter*>(nullptr));
-            if (nullptr == parameter) {
-                StringBuilder sb;
-                sb << "Illegal --setParameter parameter: \"" << parametersIt->first << "\"";
-                return Status(ErrorCodes::BadValue, sb.str());
-            }
-            if (!parameter->allowedToChangeAtStartup()) {
-                StringBuilder sb;
-                sb << "Cannot use --setParameter to set \"" << parametersIt->first
-                   << "\" at startup";
-                return Status(ErrorCodes::BadValue, sb.str());
-            }
-            Status status = parameter->setFromString(parametersIt->second);
-            if (!status.isOK()) {
-                StringBuilder sb;
-                sb << "Bad value for parameter \"" << parametersIt->first
-                   << "\": " << status.reason();
-                return Status(ErrorCodes::BadValue, sb.str());
-            }
-        }
+        auto swObj = server_options_detail::applySetParameterOptions(
+            params["setParameter"].as<std::map<std::string, std::string>>(),
+            *ServerParameterSet::getNodeParameterSet());
+        if (!swObj.isOK())
+            return swObj.getStatus();
+        if (const BSONObj& obj = swObj.getValue(); !obj.isEmpty())
+            LOGV2(5760901, "Applied --setParameter options", "serverParameters"_attr = obj);
     }
 
     if (params.count("operationProfiling.slowOpThresholdMs")) {
