@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kControl
 
 #include "mongo/db/server_options_server_helpers.h"
 
@@ -37,7 +36,6 @@
 #include <boost/algorithm/string/trim.hpp>
 #include <boost/filesystem.hpp>
 #include <boost/filesystem/operations.hpp>
-#include <cstdlib>
 #include <fmt/format.h>
 #include <ios>
 #include <iostream>
@@ -49,13 +47,11 @@
 #include "mongo/db/server_options.h"
 #include "mongo/db/server_options_base.h"
 #include "mongo/db/server_options_helpers.h"
-#include "mongo/logger/message_event_utf8_encoder.h"
 #include "mongo/logv2/log.h"
 #include "mongo/logv2/log_component.h"
 #include "mongo/transport/message_compressor_registry.h"
 #include "mongo/util/cmdline_utils/censor_cmdline.h"
 #include "mongo/util/fail_point.h"
-#include "mongo/util/map_util.h"
 #include "mongo/util/net/sock.h"
 #include "mongo/util/net/socket_utils.h"
 #include "mongo/util/net/ssl_options.h"
@@ -63,6 +59,9 @@
 #include "mongo/util/options_parser/startup_options.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kControl
+
+
 using std::endl;
 using std::string;
 
@@ -114,34 +113,6 @@ Status setParsedOpts(const moe::Environment& params) {
     cmdline_utils::censorBSONObj(&serverGlobalParams.parsedOpts);
     return Status::OK();
 }
-
-bool shouldFork(const moe::Environment& params) {
-    auto paramYes = [](const moe::Environment& params, const std::string& key) {
-        return params.count(key) && params[key].as<bool>();
-    };
-
-    auto envVarYes = [](const std::string& envKey) {
-        auto envVal = getenv(envKey.c_str());
-        return envVal && std::string{envVal} == "1";
-    };
-
-    if (paramYes(params, "shutdown")) {
-        return false;
-    }
-
-    if (envVarYes("MONGODB_CONFIG_OVERRIDE_NOFORK")) {
-        LOGV2(7484500,
-              "Environment variable MONGODB_CONFIG_OVERRIDE_NOFORK == 1, "
-              "overriding \"processManagement.fork\" to false");
-        return false;
-    }
-
-    if (paramYes(params, "processManagement.fork")) {
-        return true;
-    }
-
-    return false;
-}
 }  // namespace
 
 void printCommandLineOpts(std::ostream* os) {
@@ -205,14 +176,22 @@ Status validateServerOptions(const moe::Environment& params) {
             haveAuthenticationMechanisms = false;
         }
 
-        if (parameters.find("internalValidateFeaturesAsMaster") != parameters.end()) {
-            // Command line options that are disallowed when internalValidateFeaturesAsMaster is
-            // specified.
+        bool internalValidateFeaturesAsPrimaryUsed =
+            parameters.find("internalValidateFeaturesAsPrimary") != parameters.end();
+        bool internalValidateFeaturesAsMasterUsed =
+            parameters.find("internalValidateFeaturesAsMaster") != parameters.end();
+
+        if (internalValidateFeaturesAsPrimaryUsed || internalValidateFeaturesAsMasterUsed) {
+            // Command line options that are disallowed when internalValidateFeaturesAsPrimary or
+            // internalValidateFeaturesAsMaster, the deprecated alias, is specified.
+            std::string parameterName = internalValidateFeaturesAsPrimaryUsed
+                ? "internalValidateFeaturesAsPrimary"
+                : "internalValidateFeaturesAsMaster";
             if (params.count("replication.replSet")) {
                 return Status(ErrorCodes::BadValue,
                               str::stream() <<  //
-                                  "Cannot specify both internalValidateFeaturesAsMaster and "
-                                  "replication.replSet");
+                                  "Cannot specify both " + parameterName +
+                                      " and replication.replSet");
             }
         }
     }
@@ -334,40 +313,15 @@ Status storeServerOptions(const moe::Environment& params) {
         }
     }
 
-    if (params.count("net.serviceExecutor")) {
-        auto value = params["net.serviceExecutor"].as<std::string>();
-        const auto valid = {"synchronous"_sd, "adaptive"_sd};
-        if (std::find(valid.begin(), valid.end(), value) == valid.end()) {
-            return {ErrorCodes::BadValue, "Unsupported value for serviceExecutor"};
-        }
-        serverGlobalParams.serviceExecutor = value;
-    } else {
-        serverGlobalParams.serviceExecutor = "synchronous";
-    }
-
     if (params.count("security.transitionToAuth")) {
         serverGlobalParams.transitionToAuth = params["security.transitionToAuth"].as<bool>();
     }
 
     if (params.count("security.clusterAuthMode")) {
-        std::string clusterAuthMode = params["security.clusterAuthMode"].as<std::string>();
-
-        if (clusterAuthMode == "keyFile") {
-            serverGlobalParams.clusterAuthMode.store(ServerGlobalParams::ClusterAuthMode_keyFile);
-        } else if (clusterAuthMode == "sendKeyFile") {
-            serverGlobalParams.clusterAuthMode.store(
-                ServerGlobalParams::ClusterAuthMode_sendKeyFile);
-        } else if (clusterAuthMode == "sendX509") {
-            serverGlobalParams.clusterAuthMode.store(ServerGlobalParams::ClusterAuthMode_sendX509);
-        } else if (clusterAuthMode == "x509") {
-            serverGlobalParams.clusterAuthMode.store(ServerGlobalParams::ClusterAuthMode_x509);
-        } else {
-            return Status(ErrorCodes::BadValue,
-                          "unsupported value for clusterAuthMode " + clusterAuthMode);
-        }
+        const auto modeStr = params["security.clusterAuthMode"].as<std::string>();
+        serverGlobalParams.startupClusterAuthMode =
+            uassertStatusOK(ClusterAuthMode::parse(modeStr));
         serverGlobalParams.authState = ServerGlobalParams::AuthState::kEnabled;
-    } else {
-        serverGlobalParams.clusterAuthMode.store(ServerGlobalParams::ClusterAuthMode_undefined);
     }
 
     if (params.count("net.maxIncomingConnections")) {
@@ -429,7 +383,10 @@ Status storeServerOptions(const moe::Environment& params) {
         serverGlobalParams.unixSocketPermissions =
             params["net.unixDomainSocket.filePermissions"].as<int>();
     }
-    if (shouldFork(params)) {
+
+    if ((params.count("processManagement.fork") &&
+         params["processManagement.fork"].as<bool>() == true) &&
+        (!params.count("shutdown") || params["shutdown"].as<bool>() == false)) {
         serverGlobalParams.doFork = true;
     }
 #endif  // _WIN32
@@ -463,12 +420,14 @@ Status storeServerOptions(const moe::Environment& params) {
     }
 
     if (!params.count("security.clusterAuthMode") && params.count("security.keyFile")) {
-        serverGlobalParams.clusterAuthMode.store(ServerGlobalParams::ClusterAuthMode_keyFile);
+        serverGlobalParams.startupClusterAuthMode = ClusterAuthMode::keyFile();
     }
-    int clusterAuthMode = serverGlobalParams.clusterAuthMode.load();
+
+    const bool hasClusterAuthMode = serverGlobalParams.startupClusterAuthMode.isDefined();
     if (serverGlobalParams.transitionToAuth &&
-        (clusterAuthMode != ServerGlobalParams::ClusterAuthMode_keyFile &&
-         clusterAuthMode != ServerGlobalParams::ClusterAuthMode_x509)) {
+        !(hasClusterAuthMode &&
+          (serverGlobalParams.startupClusterAuthMode.x509Only() ||
+           serverGlobalParams.startupClusterAuthMode.keyFileOnly()))) {
         return Status(ErrorCodes::BadValue,
                       "--transitionToAuth must be used with keyFile or x509 authentication");
     }
