@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
@@ -39,9 +38,8 @@
 #include "mongo/base/init.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/db/client.h"
-#include "mongo/db/concurrency/locker_noop.h"
 #include "mongo/db/default_baton.h"
-#include "mongo/db/op_observer.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/storage/recovery_unit_noop.h"
 #include "mongo/logv2/log.h"
@@ -49,11 +47,15 @@
 #include "mongo/transport/session.h"
 #include "mongo/transport/transport_layer.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/processinfo.h"
 #include "mongo/util/str.h"
 #include "mongo/util/system_clock_source.h"
 #include "mongo/util/system_tick_source.h"
 #include <iostream>
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
+
 namespace mongo {
 namespace {
 
@@ -96,16 +98,11 @@ void setGlobalServiceContext(ServiceContext::UniqueServiceContext&& serviceConte
     globalServiceContext = serviceContext.release();
 }
 
-bool _supportsDocLocking = false;
-
-bool supportsDocLocking() {
-    return _supportsDocLocking;
-}
-
 ServiceContext::ServiceContext()
     : _opIdRegistry(UniqueOperationIdRegistry::create()),
       _tickSource(std::make_unique<SystemTickSource>()),
       _fastClockSource(std::make_unique<SystemClockSource>()),
+      _clockTickSource(_fastClockSource.get()),
       _preciseClockSource(std::make_unique<SystemClockSource>()) {}
 
 
@@ -201,10 +198,6 @@ ServiceEntryPoint* ServiceContext::getServiceEntryPoint() const {
     return _serviceEntryPoint.get();
 }
 
-transport::ServiceExecutor* ServiceContext::getServiceExecutor() const {
-    return _serviceExecutor.get();
-}
-
 void ServiceContext::setStorageEngine(std::unique_ptr<StorageEngine> engine) {
     invariant(engine);
     invariant(!_storageEngine);
@@ -220,6 +213,7 @@ void ServiceContext::setTickSource(std::unique_ptr<TickSource> newSource) {
 }
 
 void ServiceContext::setFastClockSource(std::unique_ptr<ClockSource> newSource) {
+    _clockTickSource.setClockSource(newSource.get());
     _fastClockSource = std::move(newSource);
 }
 
@@ -235,10 +229,6 @@ void ServiceContext::setTransportLayer(std::unique_ptr<transport::TransportLayer
     _transportLayer = std::move(tl);
 }
 
-void ServiceContext::setServiceExecutor(std::unique_ptr<transport::ServiceExecutor> exec) {
-    _serviceExecutor = std::move(exec);
-}
-
 void ServiceContext::ClientDeleter::operator()(Client* client) const {
     ServiceContext* const service = client->getServiceContext();
     {
@@ -256,18 +246,20 @@ ServiceContext::UniqueOperationContext ServiceContext::makeOperationContext(Clie
         _numCurrentOps.addAndFetch(1);
     }
 
-    auto numOpsGuard = makeGuard([&] {
+    ScopeGuard numOpsGuard([&] {
         if (client->session()) {
             _numCurrentOps.subtractAndFetch(1);
         }
     });
 
+    // We must prevent changing the storage engine while setting a new opCtx on the client.
+    auto sharedStorageChangeToken = _storageChangeLk.acquireSharedStorageChangeToken();
+
     onCreate(opCtx.get(), _clientObservers);
-    auto onCreateGuard = makeGuard([&] { onDestroy(opCtx.get(), _clientObservers); });
+    ScopeGuard onCreateGuard([&] { onDestroy(opCtx.get(), _clientObservers); });
+
+    invariant(opCtx->lockState(), ProcessInfo().getProcessName());
 
-    if (!opCtx->lockState()) {
-        opCtx->setLockState(std::make_unique<LockerNoop>());
-    }
     if (!opCtx->recoveryUnit()) {
         opCtx->setRecoveryUnit(std::make_unique<RecoveryUnitNoop>(),
                                WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
@@ -279,11 +271,23 @@ ServiceContext::UniqueOperationContext ServiceContext::makeOperationContext(Clie
         makeBaton(opCtx.get());
     }
 
-    auto batonGuard = makeGuard([&] { opCtx->getBaton()->detach(); });
+    ScopeGuard batonGuard([&] { opCtx->getBaton()->detach(); });
 
     {
         stdx::lock_guard<Client> lk(*client);
-        client->setOperationContext(opCtx.get());
+
+        // If we have a previous operation context, it's not worth crashing the process in
+        // production. However, we do want to prevent it from doing more work and complain
+        // loudly.
+        auto lastOpCtx = client->getOperationContext();
+        if (lastOpCtx) {
+            killOperation(lk, lastOpCtx, ErrorCodes::Error(4946800));
+            tasserted(4946801,
+                      "Client has attempted to create a new OperationContext, but it already "
+                      "has one");
+        }
+
+        client->_setOperationContext(opCtx.get());
     }
 
     numOpsGuard.dismiss();
@@ -307,10 +311,10 @@ void ServiceContext::OperationContextDeleter::operator()(OperationContext* opCtx
     auto service = client->getServiceContext();
     invariant(service);
 
+    onDestroy(opCtx, service->_clientObservers);
     service->_delistOperation(opCtx);
     opCtx->getBaton()->detach();
 
-    onDestroy(opCtx, service->_clientObservers);
     delete opCtx;
 }
 
@@ -406,7 +410,7 @@ void ServiceContext::_delistOperation(OperationContext* opCtx) noexcept {
     // Assigning a new opCtx to the client must never precede the destruction of any existing opCtx
     // that references the client.
     invariant(client->getOperationContext() == opCtx);
-    client->resetOperationContext();
+    client->_setOperationContext({});
 
     if (client->session()) {
         _numCurrentOps.subtractAndFetch(1);
@@ -489,20 +493,16 @@ ServiceContext::ConstructorActionRegisterer::ConstructorActionRegisterer(
     DestructorAction destructor) {
     if (!destructor)
         destructor = [](ServiceContext*) {};
-    _registerer.emplace(std::move(name),
-                        [this, constructor, destructor](InitializerContext* context) {
-                            _iter = registeredConstructorActions().emplace(
-                                registeredConstructorActions().end(),
-                                std::move(constructor),
-                                std::move(destructor));
-                            return Status::OK();
-                        },
-                        [this](DeinitializerContext* context) {
-                            registeredConstructorActions().erase(_iter);
-                            return Status::OK();
-                        },
-                        std::move(prereqs),
-                        std::move(dependents));
+    _registerer.emplace(
+        std::move(name),
+        [this, constructor, destructor](InitializerContext*) {
+            _iter = registeredConstructorActions().emplace(registeredConstructorActions().end(),
+                                                           std::move(constructor),
+                                                           std::move(destructor));
+        },
+        [this](DeinitializerContext*) { registeredConstructorActions().erase(_iter); },
+        std::move(prereqs),
+        std::move(dependents));
 }
 
 ServiceContext::UniqueServiceContext ServiceContext::make() {
