@@ -35,21 +35,22 @@
 #include <memory>
 #include <vector>
 
-#include "mongo/base/global_initializer_registerer.h"
-#include "mongo/db/logical_session_id.h"
 #include "mongo/db/operation_id.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/storage/storage_change_lock.h"
 #include "mongo/db/storage/storage_engine.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
 #include "mongo/stdx/unordered_set.h"
-#include "mongo/transport/service_executor.h"
 #include "mongo/transport/session.h"
 #include "mongo/util/clock_source.h"
+#include "mongo/util/clock_tick_source.h"
 #include "mongo/util/concurrency/with_lock.h"
 #include "mongo/util/decorable.h"
 #include "mongo/util/hierarchical_acquisition.h"
 #include "mongo/util/periodic_runner.h"
+#include "mongo/util/synchronized_value.h"
 #include "mongo/util/tick_source.h"
 #include "mongo/util/uuid.h"
 
@@ -181,6 +182,10 @@ public:
 
         /**
          * Hook called on a "opCtx" created by a service before deleting "opCtx".
+         * Note that this hook is called before any other work is done in the
+         * OperationContext destructor, meaning the OperationContext is still
+         * valid and registered with the ServiceContext when this hook is
+         * executed.
          *
          * Like a destructor, must not throw exceptions.
          */
@@ -381,7 +386,9 @@ public:
     //
 
     /**
-     * Sets the storage engine for this instance. May be called up to once per instance.
+     * Sets the storage engine for this instance. May be called up to once per instance, unless
+     * clearStorageEngine() is called in which it may be called once after each call to
+     * clearStorageEngine().
      */
     void setStorageEngine(std::unique_ptr<StorageEngine> engine);
 
@@ -392,6 +399,24 @@ public:
         return _storageEngine.get();
     }
 
+    /**
+     * Clear the current storage engine so we can set a new one.  This is safe to call only if
+     * the caller has arranged for no opCtxs to be accessing the existing storage engine,
+     * and that no new opCtxs can be created which will access storage until this call returns.
+     *
+     * See StorageEngineChangeContext for one way this may be done.
+     */
+    void clearStorageEngine() {
+        _storageEngine = nullptr;
+    }
+
+    /**
+     * Return the storage change lock.
+     */
+    StorageChangeLock& getStorageChangeLock() {
+        return _storageChangeLk;
+    }
+
     //
     // Global operation management.  This may not belong here and there may be too many methods
     // here.
@@ -482,14 +507,6 @@ public:
      */
     ServiceEntryPoint* getServiceEntryPoint() const;
 
-    /**
-     * Get the service executor for the service context.
-     *
-     * See ServiceStateMachine for how this is used. Some configurations may not have a service
-     * executor registered and this will return a nullptr.
-     */
-    transport::ServiceExecutor* getServiceExecutor() const;
-
     /**
      * Waits for the ServiceContext to be fully initialized and for all TransportLayers to have been
      * added/started.
@@ -528,6 +545,13 @@ public:
         return _tickSource.get();
     }
 
+    /**
+     * Returns the tick/clock source that wraps '_fastClockSource' in this context.
+     */
+    TickSource* getFastTickSource() {
+        return &_clockTickSource;
+    }
+
     /**
      * Get a ClockSource implementation that may be less precise than the _preciseClockSource but
      * may be cheaper to call.
@@ -549,6 +573,12 @@ public:
      */
     void setTickSource(std::unique_ptr<TickSource> newSource);
 
+    /**
+     * Replaces the current tick/clock source with a new one. In other words, the old source will be
+     * destroyed. So make sure that no one is using the old source when calling this.
+     */
+    void setFastTickSource(std::unique_ptr<TickSource> newSource);
+
     /**
      * Call this method with a ClockSource implementation that may be less precise than
      * the _preciseClockSource but may be cheaper to call.
@@ -574,11 +604,6 @@ public:
      */
     void setTransportLayer(std::unique_ptr<transport::TransportLayer> tl);
 
-    /**
-     * Binds the service executor to the service context
-     */
-    void setServiceExecutor(std::unique_ptr<transport::ServiceExecutor> exec);
-
     /**
      * Creates a delayed execution baton with basic functionality
      */
@@ -592,9 +617,75 @@ public:
         _catalogGeneration.fetchAndAdd(1);
     }
 
+    void disallowUserWrites() {
+        _userWritesAllowed.store(false);
+    }
+
+    /**
+     * Returns true if user writes are allowed.
+     *
+     * User writes are disallowed when starting with queryableBackupMode or
+     * recoverFromOplogAsStandalone to prevent users from writing to replicated collections in
+     * standalone mode.
+     *
+     * To determine whether an operation must run in read-only mode, use
+     * OperationContext::readOnly().
+     */
+    bool userWritesAllowed() const {
+        return _userWritesAllowed.load();
+    }
+
     LockedClient getLockedClient(OperationId id);
 
 private:
+    /**
+     * A synchronized owning pointer to avoid setters racing with getters.
+     * This only guarantees that getters receive a coherent value, and
+     * not that the pointer is still valid.
+     *
+     * The kernel of operations is `set` and and `get`, others ops are sugar.
+     */
+    template <typename T>
+    class SyncUnique {
+    public:
+        SyncUnique() = default;
+        explicit SyncUnique(std::unique_ptr<T> p) {
+            set(std::move(p));
+        }
+
+        ~SyncUnique() {
+            set(nullptr);
+        }
+
+        SyncUnique& operator=(std::unique_ptr<T> p) {
+            set(std::move(p));
+            return *this;
+        }
+
+        void set(std::unique_ptr<T> p) {
+            delete _ptr.swap(p.release());
+        }
+
+        T* get() const {
+            return _ptr.load();
+        }
+
+        T* operator->() const {
+            return get();
+        }
+
+        T& operator*() const {
+            return *get();
+        }
+
+        explicit operator bool() const {
+            return static_cast<bool>(get());
+        }
+
+    private:
+        AtomicWord<T*> _ptr{nullptr};
+    };
+
     class ClientObserverHolder {
     public:
         explicit ClientObserverHolder(std::unique_ptr<ClientObserver> observer)
@@ -629,27 +720,27 @@ private:
     /**
      * The periodic runner.
      */
-    std::unique_ptr<PeriodicRunner> _runner;
+    SyncUnique<PeriodicRunner> _runner;
 
     /**
      * The TransportLayer.
      */
-    std::unique_ptr<transport::TransportLayer> _transportLayer;
+    SyncUnique<transport::TransportLayer> _transportLayer;
 
     /**
      * The service entry point
      */
-    std::unique_ptr<ServiceEntryPoint> _serviceEntryPoint;
-
-    /**
-     * The ServiceExecutor
-     */
-    std::unique_ptr<transport::ServiceExecutor> _serviceExecutor;
+    SyncUnique<ServiceEntryPoint> _serviceEntryPoint;
 
     /**
      * The storage engine, if any.
      */
-    std::unique_ptr<StorageEngine> _storageEngine;
+    SyncUnique<StorageEngine> _storageEngine;
+
+    /**
+     * The lock that protects changing out the storage engine.
+     */
+    StorageChangeLock _storageChangeLk;
 
     /**
      * Vector of registered observers.
@@ -666,20 +757,26 @@ private:
     /**
      * The registered OpObserver.
      */
-    std::unique_ptr<OpObserver> _opObserver;
+    SyncUnique<OpObserver> _opObserver;
 
-    std::unique_ptr<TickSource> _tickSource;
+    SyncUnique<TickSource> _tickSource;
 
     /**
      * A ClockSource implementation that may be less precise than the _preciseClockSource but
      * may be cheaper to call.
      */
-    std::unique_ptr<ClockSource> _fastClockSource;
+    SyncUnique<ClockSource> _fastClockSource;
+
+    /**
+     * A type of TickSource that wraps '_fastClockSource'. Note that '_clockTickSource' does not own
+     * '_fastClockSource'.
+     */
+    ClockTickSource _clockTickSource;
 
     /**
      * A ClockSource implementation that is very precise but may be expensive to call.
      */
-    std::unique_ptr<ClockSource> _preciseClockSource;
+    SyncUnique<ClockSource> _preciseClockSource;
 
     // Flag set to indicate that all operations are to be interrupted ASAP.
     AtomicWord<bool> _globalKill{false};
@@ -690,6 +787,9 @@ private:
     // When the catalog is restarted, the generation goes up by one each time.
     AtomicWord<uint64_t> _catalogGeneration{0};
 
+    // Server-wide flag indicating whether users' writes are allowed.
+    AtomicWord<bool> _userWritesAllowed{true};
+
     bool _startupComplete = false;
     stdx::condition_variable _startupCompleteCondVar;
 };
@@ -725,11 +825,4 @@ ServiceContext* getCurrentServiceContext();
  */
 void setGlobalServiceContext(ServiceContext::UniqueServiceContext&& serviceContext);
 
-/**
- * Shortcut for querying the storage engine about whether it supports document-level locking.
- * If this call becomes too expensive, we could cache the value somewhere so we don't have to
- * fetch the storage engine every time.
- */
-bool supportsDocLocking();
-
 }  // namespace mongo
