@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/platform/basic.h"
 
@@ -40,7 +39,6 @@
 #include <typeinfo>
 #include <vector>
 
-#include "mongo/base/owned_pointer_vector.h"
 #include "mongo/base/simple_string_data_comparator.h"
 #include "mongo/bson/bson_depth.h"
 #include "mongo/bson/bson_validate.h"
@@ -56,6 +54,9 @@
 #include "mongo/util/hex.h"
 #include "mongo/util/timer.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
+
 using std::string;
 using namespace mongo;
 
@@ -139,7 +140,7 @@ void checkSizeWhileAppendingTypeBits(int numOfBitsUsedForType, T&& appendBitsFun
 // patterns, so including it here specifically.
 TEST(InvalidKeyStringTest, FuzzedCodeWithScopeNesting) {
     BufBuilder keyData;
-    fromHexString(
+    hexblob::decode(
         "aa00aa4200aafa00aa0200aa0a01aa02aa00aa4200aafa00aa0200aa0a01aa0200aa00aa4200aafa00aa0200aa"
         "0a01aa0200aa4200aafa00aa0200aa00aaaa00aa00aafa00aa0200aa3900aafa00aa0200aa00aa004200aafa00"
         "aaaafa00aa0200aa0a01aa0200aa4200aafa00aa0200aa00aaaa00aa00aafa00aa0200aa00aafa00aa0200aa00"
@@ -234,8 +235,8 @@ TEST(TypeBitsTest, AllZerosTypeBits) {
     }
 
     {
-        std::string allZerosBuffer = "0000000000000000";
-        BufReader reader(allZerosBuffer.c_str(), 0);
+        char allZerosBuffer[16] = {0};
+        BufReader reader(allZerosBuffer, sizeof(allZerosBuffer));
         KeyString::TypeBits typeBits =
             KeyString::TypeBits::fromBuffer(KeyString::Version::V1, &reader);
         ASSERT_EQ(typeBits.getSize(), 1u);
@@ -301,13 +302,12 @@ TEST_F(KeyStringBuilderTest, ExceededBSONDepth) {
         nestedObj = BSON("" << nestedObj);
     }
     // This BSON object should not be valid.
-    auto validateStatus =
-        validateBSON(nestedObj.objdata(), nestedObj.objsize(), BSONVersion::kV1_1);
+    auto validateStatus = validateBSON(nestedObj.objdata(), nestedObj.objsize());
     ASSERT_EQ(ErrorCodes::Overflow, validateStatus.code());
 
     // Construct a KeyString from the invalid BSON, and confirm that it fails to convert back to
     // BSON.
-    ks.resetToKey(nestedObj, ALL_ASCENDING, RecordId());
+    ks.resetToKey(nestedObj, ALL_ASCENDING, RecordId(1));
     ASSERT_THROWS_CODE(
         KeyString::toBsonSafe(ks.getBuffer(), ks.getSize(), ALL_ASCENDING, ks.getTypeBits()),
         AssertionException,
@@ -320,8 +320,8 @@ TEST_F(KeyStringBuilderTest, Simple1) {
 
     ASSERT_BSONOBJ_LT(a, b);
 
-    ASSERT_LESS_THAN(KeyString::Builder(version, a, ALL_ASCENDING, RecordId()),
-                     KeyString::Builder(version, b, ALL_ASCENDING, RecordId()));
+    ASSERT_LESS_THAN(KeyString::Builder(version, a, ALL_ASCENDING, RecordId(1)),
+                     KeyString::Builder(version, b, ALL_ASCENDING, RecordId(1)));
 }
 
 #define ROUNDTRIP_ORDER(version, x, order)                            \
@@ -376,7 +376,7 @@ TEST_F(KeyStringBuilderTest, ActualBytesDouble) {
           "[{toHex_ks_getBuffer_ks_getSize}]",
           "keyStringVersionToString_version"_attr = keyStringVersionToString(version),
           "ks_getSize"_attr = ks.getSize(),
-          "toHex_ks_getBuffer_ks_getSize"_attr = toHex(ks.getBuffer(), ks.getSize()));
+          "toHex_ks_getBuffer_ks_getSize"_attr = hexblob::encode(ks.getBuffer(), ks.getSize()));
 
     ASSERT_EQUALS(10U, ks.getSize());
 
@@ -389,7 +389,7 @@ TEST_F(KeyStringBuilderTest, ActualBytesDouble) {
                                                      "80000000000000"  // fractional bytes
                                                      "04";             // kEnd
 
-    ASSERT_EQUALS(hex, toHex(ks.getBuffer(), ks.getSize()));
+    ASSERT_EQUALS(hex, hexblob::encode(ks.getBuffer(), ks.getSize()));
 
     ks.resetToKey(a, Ordering::make(BSON("a" << -1)));
 
@@ -399,13 +399,13 @@ TEST_F(KeyStringBuilderTest, ActualBytesDouble) {
     // last byte (kEnd) doesn't get flipped
     string hexFlipped;
     for (size_t i = 0; i < hex.size() - 2; i += 2) {
-        char c = uassertStatusOK(fromHex(hex.c_str() + i));
+        char c = hexblob::decodePair(StringData(hex).substr(i, 2));
         c = ~c;
-        hexFlipped += toHex(&c, 1);
+        hexFlipped += hexblob::encode(StringData(&c, 1));
     }
     hexFlipped += hex.substr(hex.size() - 2);
 
-    ASSERT_EQUALS(hexFlipped, toHex(ks.getBuffer(), ks.getSize()));
+    ASSERT_EQUALS(hexFlipped, hexblob::encode(ks.getBuffer(), ks.getSize()));
 }
 
 TEST_F(KeyStringBuilderTest, AllTypesSimple) {
@@ -445,7 +445,7 @@ TEST_F(KeyStringBuilderTest, Array1) {
     ROUNDTRIP(version, BSON("" << BSON_ARRAY(1 << 2 << 3)));
 
     {
-        KeyString::Builder a(version, emptyArray, ALL_ASCENDING, RecordId::min());
+        KeyString::Builder a(version, emptyArray, ALL_ASCENDING, RecordId::minLong());
         KeyString::Builder b(version, emptyArray, ALL_ASCENDING, RecordId(5));
         ASSERT_LESS_THAN(a, b);
     }
@@ -751,24 +751,52 @@ TEST_F(KeyStringBuilderTest, ReasonableSize) {
     // generation. These upper bounds were the calculate sizes of each type at the time this
     // test was written.
     KeyString::Builder stackBuilder(KeyString::Version::kLatestVersion, BSONObj(), ALL_ASCENDING);
-    ASSERT_LTE(sizeof(stackBuilder), 608);
+    static_assert(sizeof(stackBuilder) <= 624);
 
     KeyString::HeapBuilder heapBuilder(
         KeyString::Version::kLatestVersion, BSONObj(), ALL_ASCENDING);
-    ASSERT_LTE(sizeof(heapBuilder), 96);
+    static_assert(sizeof(heapBuilder) <= 104);
+
+    // Use large 1KB blocks and verify that we use way less
+    SharedBufferFragmentBuilder fragmentBuilder(1024);
+    KeyString::PooledBuilder pooledBuilder(
+        fragmentBuilder, KeyString::Version::kLatestVersion, BSONObj(), ALL_ASCENDING);
+    static_assert(sizeof(pooledBuilder) <= 104);
 
     // Test the dynamic memory usage reported to the sorter.
     KeyString::Value value1 = stackBuilder.getValueCopy();
-    ASSERT_LTE(sizeof(value1), 24);
-    ASSERT_LTE(value1.memUsageForSorter(), 26);
+    ASSERT_LTE(sizeof(value1), 32);
+    ASSERT_LTE(value1.memUsageForSorter(), 34);
 
     KeyString::Value value2 = heapBuilder.getValueCopy();
-    ASSERT_LTE(sizeof(value2), 24);
-    ASSERT_LTE(value2.memUsageForSorter(), 26);
+    ASSERT_LTE(sizeof(value2), 32);
+    ASSERT_LTE(value2.memUsageForSorter(), 34);
 
     KeyString::Value value3 = heapBuilder.release();
-    ASSERT_LTE(sizeof(value3), 24);
-    ASSERT_LTE(value3.memUsageForSorter(), 56);
+    ASSERT_LTE(sizeof(value3), 32);
+    ASSERT_LTE(value3.memUsageForSorter(), 64);
+
+    KeyString::Value value4 = pooledBuilder.getValueCopy();
+    ASSERT_LTE(sizeof(value4), 32);
+    ASSERT_LTE(value4.memUsageForSorter(), 34);
+
+    KeyString::Value value5 = pooledBuilder.release();
+    ASSERT_LTE(sizeof(value5), 32);
+    ASSERT_LTE(value5.memUsageForSorter(), 34);
+}
+
+TEST_F(KeyStringBuilderTest, DiscardIfNotReleased) {
+    SharedBufferFragmentBuilder fragmentBuilder(1024);
+    {
+        // Intentially not released, but the data should be discarded correctly.
+        KeyString::PooledBuilder pooledBuilder(
+            fragmentBuilder, KeyString::Version::kLatestVersion, BSONObj(), ALL_ASCENDING);
+    }
+    {
+        KeyString::PooledBuilder pooledBuilder(
+            fragmentBuilder, KeyString::Version::kLatestVersion, BSONObj(), ALL_ASCENDING);
+        pooledBuilder.release();
+    }
 }
 
 TEST_F(KeyStringBuilderTest, LotsOfNumbers1) {
@@ -812,69 +840,25 @@ TEST_F(KeyStringBuilderTest, LotsOfNumbers2) {
     }
 }
 
-TEST_F(KeyStringBuilderTest, LotsOfNumbers3) {
-    std::vector<stdx::future<void>> futures;
-
-    for (double k = 0; k < 8; k++) {
-        futures.push_back(stdx::async(stdx::launch::async, [k, this] {
-            for (double i = -1100; i < 1100; i++) {
-                for (double j = 0; j < 52; j++) {
-                    const auto V1 = KeyString::Version::V1;
-                    Decimal128::RoundingPrecision roundingPrecisions[]{
-                        Decimal128::kRoundTo15Digits, Decimal128::kRoundTo34Digits};
-                    Decimal128::RoundingMode roundingModes[]{Decimal128::kRoundTowardNegative,
-                                                             Decimal128::kRoundTowardPositive};
-                    double x = pow(2, i);
-                    double y = pow(2, i - j);
-                    double z = pow(2, i - 53 + k);
-                    double bin = x + y - z;
-
-                    // In general NaNs don't roundtrip as we only store a single NaN, see the NaNs
-                    // test.
-                    if (std::isnan(bin))
-                        continue;
-
-                    ROUNDTRIP(version, BSON("" << bin));
-                    ROUNDTRIP(version, BSON("" << -bin));
-
-                    if (version < V1)
-                        continue;
-
-                    for (auto precision : roundingPrecisions) {
-                        for (auto mode : roundingModes) {
-                            Decimal128 rounded = Decimal128(bin, precision, mode);
-                            ROUNDTRIP(V1, BSON("" << rounded));
-                            ROUNDTRIP(V1, BSON("" << rounded.negate()));
-                        }
-                    }
-                }
-            }
-        }));
-    }
-    for (auto&& future : futures) {
-        future.get();
-    }
-}
-
 TEST_F(KeyStringBuilderTest, RecordIdOrder1) {
     Ordering ordering = Ordering::make(BSON("a" << 1));
 
-    KeyString::Builder a(version, BSON("" << 5), ordering, RecordId::min());
+    KeyString::Builder a(version, BSON("" << 5), ordering, RecordId::minLong());
     KeyString::Builder b(version, BSON("" << 5), ordering, RecordId(2));
     KeyString::Builder c(version, BSON("" << 5), ordering, RecordId(3));
-    KeyString::Builder d(version, BSON("" << 6), ordering, RecordId());
+    KeyString::Builder d(version, BSON("" << 6), ordering, RecordId(4));
     KeyString::Builder e(version, BSON("" << 6), ordering, RecordId(1));
 
     ASSERT_LESS_THAN(a, b);
     ASSERT_LESS_THAN(b, c);
     ASSERT_LESS_THAN(c, d);
-    ASSERT_LESS_THAN(d, e);
+    ASSERT_LESS_THAN(e, d);
 }
 
 TEST_F(KeyStringBuilderTest, RecordIdOrder2) {
     Ordering ordering = Ordering::make(BSON("a" << -1 << "b" << -1));
 
-    KeyString::Builder a(version, BSON("" << 5 << "" << 6), ordering, RecordId::min());
+    KeyString::Builder a(version, BSON("" << 5 << "" << 6), ordering, RecordId::minLong());
     KeyString::Builder b(version, BSON("" << 5 << "" << 6), ordering, RecordId(5));
     KeyString::Builder c(version, BSON("" << 5 << "" << 5), ordering, RecordId(4));
     KeyString::Builder d(version, BSON("" << 3 << "" << 4), ordering, RecordId(3));
@@ -890,7 +874,7 @@ TEST_F(KeyStringBuilderTest, RecordIdOrder2) {
 TEST_F(KeyStringBuilderTest, RecordIdOrder2Double) {
     Ordering ordering = Ordering::make(BSON("a" << -1 << "b" << -1));
 
-    KeyString::Builder a(version, BSON("" << 5.0 << "" << 6.0), ordering, RecordId::min());
+    KeyString::Builder a(version, BSON("" << 5.0 << "" << 6.0), ordering, RecordId::minLong());
     KeyString::Builder b(version, BSON("" << 5.0 << "" << 6.0), ordering, RecordId(5));
     KeyString::Builder c(version, BSON("" << 3.0 << "" << 4.0), ordering, RecordId(3));
 
@@ -1296,6 +1280,212 @@ std::vector<BSONObj> thinElements(std::vector<BSONObj> elements,
 }
 }  // namespace
 
+namespace {
+RecordId ridFromOid(const OID& oid) {
+    KeyString::Builder builder(KeyString::Version::kLatestVersion);
+    builder.appendOID(oid);
+    return RecordId(builder.getBuffer(), builder.getSize());
+}
+}  // namespace
+
+TEST_F(KeyStringBuilderTest, RecordIdStr) {
+    const int kSize = 12;
+    for (int i = 0; i < kSize; i++) {
+        unsigned char buf[kSize];
+        memset(buf, 0x80, kSize);
+        buf[i] = 0xFE;
+
+        const RecordId rid = ridFromOid(OID::from(buf));
+
+        {  // Test encoding / decoding of single RecordIds
+            const KeyString::Builder ks(version, rid);
+            invariant(ks.getSize() == 14);
+
+            ASSERT_EQ(KeyString::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()), rid);
+
+            if (rid.isValid()) {
+                ASSERT_GT(ks, KeyString::Builder(version, RecordId(1)));
+                ASSERT_GT(ks, KeyString::Builder(version, ridFromOid(OID())));
+                ASSERT_LT(ks, KeyString::Builder(version, ridFromOid(OID::max())));
+
+                char bufLt[kSize];
+                memcpy(bufLt, buf, kSize);
+                bufLt[kSize - 1] -= 1;
+                auto ltRid = ridFromOid(OID::from(bufLt));
+                ASSERT(ltRid < rid);
+                ASSERT_GT(ks, KeyString::Builder(version, ltRid));
+
+                char bufGt[kSize];
+                memcpy(bufGt, buf, kSize);
+                bufGt[kSize - 1] += 1;
+                auto gtRid = ridFromOid(OID::from(bufGt));
+                ASSERT(gtRid > rid);
+                ASSERT_LT(ks, KeyString::Builder(version, gtRid));
+            }
+        }
+
+        for (int j = 0; j < kSize; j++) {
+            unsigned char otherBuf[kSize] = {0};
+            otherBuf[j] = 0xFE;
+            RecordId other = ridFromOid(OID::from(otherBuf));
+
+            if (rid == other) {
+                ASSERT_EQ(KeyString::Builder(version, rid), KeyString::Builder(version, other));
+            }
+            if (rid < other) {
+                ASSERT_LT(KeyString::Builder(version, rid), KeyString::Builder(version, other));
+            }
+            if (rid > other) {
+                ASSERT_GT(KeyString::Builder(version, rid), KeyString::Builder(version, other));
+            }
+        }
+    }
+}
+
+namespace {
+
+RecordId ridFromStr(const char* str, size_t len) {
+    KeyString::Builder builder(KeyString::Version::kLatestVersion);
+    builder.appendString(mongo::StringData(str, len));
+    return RecordId(builder.getBuffer(), builder.getSize());
+}
+}  // namespace
+
+
+TEST_F(KeyStringBuilderTest, RecordIdStrBig1SizeSegment) {
+    const int pad = 3;  // kStringLike CType + StringData terminator + RecordId len
+    {
+        const int size = 90;
+        const auto ridStr = std::string(size, 'a');
+        auto rid = ridFromStr(ridStr.c_str(), size);
+        const KeyString::Builder ks(version, rid);
+        ASSERT_EQ(ks.getSize(), size + pad);
+        ASSERT_EQ(KeyString::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()), rid);
+        ASSERT_EQ(0, KeyString::sizeWithoutRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()));
+    }
+    {
+        // Max 1-byte encoded string size is 127B: 1B CType + ridStr + string terminator
+        const int size = 125;
+        const auto ridStr = std::string(size, 'a');
+        auto rid = ridFromStr(ridStr.c_str(), size);
+        const KeyString::Builder ks(version, rid);
+        ASSERT_EQ(ks.getSize(), size + pad);
+        ASSERT_EQ(KeyString::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()), rid);
+        ASSERT_EQ(0, KeyString::sizeWithoutRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()));
+    }
+}
+
+TEST_F(KeyStringBuilderTest, RecordIdStrBig2SizeSegments) {
+    const int pad = 3;  // kStringLike CType + StringData terminator + RecordId len
+    {
+        // Min 2-byte encoded string size is 128B: 1B CType + ridStr + string terminator
+        const int size = 126;
+        const auto ridStr = std::string(size, 'a');
+        auto rid = ridFromStr(ridStr.c_str(), size);
+        const KeyString::Builder ks(version, rid);
+        ASSERT_EQ(ks.getSize(), size + pad + 1);  // 1 byte with continuation bit
+        ASSERT_EQ(KeyString::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()), rid);
+        ASSERT_EQ(0, KeyString::sizeWithoutRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()));
+    }
+    {
+        const int size = 128;
+        const auto ridStr = std::string(size, 'a');
+        auto rid = ridFromStr(ridStr.c_str(), size);
+        const KeyString::Builder ks(version, rid);
+        ASSERT_EQ(ks.getSize(), size + pad + 1);  // 1 byte with continuation bit
+        ASSERT_EQ(KeyString::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()), rid);
+        ASSERT_EQ(0, KeyString::sizeWithoutRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()));
+    }
+    {
+        // Max 2-byte encoded string size is 16383B: 1B CType + ridStr + string terminator
+        const int size = 16381;
+        const auto ridStr = std::string(size, 'a');
+        auto rid = ridFromStr(ridStr.c_str(), size);
+        const KeyString::Builder ks(version, rid);
+        ASSERT_EQ(ks.getSize(), size + pad + 1);  // 1 byte with continuation bit
+        ASSERT_EQ(KeyString::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()), rid);
+        ASSERT_EQ(0, KeyString::sizeWithoutRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()));
+    }
+}
+
+TEST_F(KeyStringBuilderTest, RecordIdStrBig3SizeSegments) {
+    const int pad = 3;  // kStringLike CType + StringData terminator + RecordId len
+    {
+        // Min 3-byte encoded string size is 16384B: 1B CType + ridStr + string terminator
+        const int size = 16382;
+        const auto ridStr = std::string(size, 'a');
+        auto rid = ridFromStr(ridStr.c_str(), size);
+        const KeyString::Builder ks(version, rid);
+        ASSERT_EQ(ks.getSize(), size + pad + 2);  // 2 bytes with continuation bit
+        ASSERT_EQ(KeyString::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()), rid);
+        ASSERT_EQ(0, KeyString::sizeWithoutRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()));
+    }
+    {
+        // Max 3-byte encoded string size is 2097151B: 1B CType + ridStr + string terminator
+        const int size = 2097149;
+        const auto ridStr = std::string(size, 'a');
+        auto rid = ridFromStr(ridStr.c_str(), size);
+        const KeyString::Builder ks(version, rid);
+        ASSERT_EQ(ks.getSize(), size + pad + 2);  // 2 bytes with continuation bit
+        ASSERT_EQ(KeyString::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()), rid);
+        ASSERT_EQ(0, KeyString::sizeWithoutRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()));
+    }
+}
+
+TEST_F(KeyStringBuilderTest, RecordIdStrBig4SizeSegments) {
+    const int pad = 3;  // kStringLike CType + StringData terminator + RecordId len
+    {
+        // Min 4-byte encoded string size is 2097152B: 1B CType + ridStr + string terminator
+        const int size = 2097150;
+        const auto ridStr = std::string(size, 'a');
+        auto rid = ridFromStr(ridStr.c_str(), size);
+        const KeyString::Builder ks(version, rid);
+        ASSERT_EQ(ks.getSize(), size + pad + 3);  // 3 bytes with continuation bit
+        ASSERT_EQ(KeyString::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()), rid);
+        ASSERT_EQ(0, KeyString::sizeWithoutRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()));
+    }
+    {
+        // Support up to RecordId::kBigStrMaxSize
+        const int size = RecordId::kBigStrMaxSize - 2 /* CType + string terminator */;
+        const auto ridStr = std::string(size, 'a');
+        auto rid = ridFromStr(ridStr.c_str(), size);
+        const KeyString::Builder ks(version, rid);
+        ASSERT_EQ(ks.getSize(), size + pad + 3);  // 3 bytes with continuation bit
+        ASSERT_EQ(KeyString::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()), rid);
+        ASSERT_EQ(0, KeyString::sizeWithoutRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()));
+    }
+}
+
+TEST_F(KeyStringBuilderTest, RecordIdStrBigSizeWithoutRecordIdStr) {
+    const int pad = 3;  // kStringLike CType + StringData terminator + RecordId len
+    const char str[] = "keyval";
+    const int padStr = 3;  // kStringLike CType + string terminator + discriminator
+    {
+        const int ridStrlen = 90;
+        const auto ridStr = std::string(ridStrlen, 'a');
+        auto rid = ridFromStr(ridStr.c_str(), ridStrlen);
+        KeyString::Builder ks(version);
+        ks.appendString(mongo::StringData(str, strlen(str)));
+        ks.appendRecordId(rid);
+        ASSERT_EQ(ks.getSize(), strlen(str) + padStr + ridStrlen + pad);
+        ASSERT_EQ(KeyString::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()), rid);
+        ASSERT_EQ(strlen(str) + padStr,
+                  KeyString::sizeWithoutRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()));
+    }
+    {
+        const int ridStrlen = 260;
+        const auto ridStr = std::string(ridStrlen, 'a');
+        auto rid = ridFromStr(ridStr.c_str(), ridStrlen);
+        KeyString::Builder ks(version);
+        ks.appendString(mongo::StringData(str, strlen(str)));
+        ks.appendRecordId(rid);
+        ASSERT_EQ(ks.getSize(),
+                  strlen(str) + padStr + ridStrlen + pad + 1);  // 1 0x80 cont byte
+        ASSERT_EQ(KeyString::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()), rid);
+        ASSERT_EQ(strlen(str) + padStr,
+                  KeyString::sizeWithoutRecordIdStrAtEnd(ks.getBuffer(), ks.getSize()));
+    }
+}
 
 TEST_F(KeyStringBuilderTest, AllPermCompare) {
     std::vector<BSONObj> elements = getInterestingElements(version);
@@ -1357,7 +1547,7 @@ TEST_F(KeyStringBuilderTest, AllPerm2Compare) {
 #define COMPARE_HELPER(LHS, RHS) (((LHS) < (RHS)) ? -1 : (((LHS) == (RHS)) ? 0 : 1))
 
 int compareLongToDouble(long long lhs, double rhs) {
-    if (rhs >= std::numeric_limits<long long>::max())
+    if (rhs >= static_cast<double>(std::numeric_limits<long long>::max()))
         return -1;
     if (rhs < std::numeric_limits<long long>::min())
         return 1;
@@ -1429,85 +1619,6 @@ TEST_F(KeyStringBuilderTest, NaNs) {
     ASSERT(toBson(ks3d, ONE_DESCENDING)[""].Decimal().isNaN());
     ASSERT(toBson(ks4d, ONE_DESCENDING)[""].Decimal().isNaN());
 }
-TEST_F(KeyStringBuilderTest, NumberOrderLots) {
-    std::vector<BSONObj> numbers;
-    {
-        numbers.push_back(BSON("" << 0));
-        numbers.push_back(BSON("" << 0.0));
-        numbers.push_back(BSON("" << -0.0));
-
-        numbers.push_back(BSON("" << std::numeric_limits<long long>::min()));
-        numbers.push_back(BSON("" << std::numeric_limits<long long>::max()));
-        numbers.push_back(BSON("" << static_cast<double>(std::numeric_limits<long long>::min())));
-        numbers.push_back(BSON("" << static_cast<double>(std::numeric_limits<long long>::max())));
-        numbers.push_back(BSON("" << std::numeric_limits<double>::min()));
-        numbers.push_back(BSON("" << std::numeric_limits<double>::max()));
-        numbers.push_back(BSON("" << std::numeric_limits<int>::min()));
-        numbers.push_back(BSON("" << std::numeric_limits<int>::max()));
-        numbers.push_back(BSON("" << std::numeric_limits<short>::min()));
-        numbers.push_back(BSON("" << std::numeric_limits<short>::max()));
-
-        for (int i = 0; i < 64; i++) {
-            int64_t x = 1LL << i;
-            numbers.push_back(BSON("" << static_cast<long long>(x)));
-            numbers.push_back(BSON("" << static_cast<int>(x)));
-            numbers.push_back(BSON("" << static_cast<double>(x)));
-            numbers.push_back(BSON("" << (static_cast<double>(x) + .1)));
-
-            numbers.push_back(BSON("" << (static_cast<long long>(x) + 1)));
-            numbers.push_back(BSON("" << (static_cast<int>(x) + 1)));
-            numbers.push_back(BSON("" << (static_cast<double>(x) + 1)));
-            numbers.push_back(BSON("" << (static_cast<double>(x) + 1.1)));
-
-            // Avoid negating signed integral minima
-            if (i < 63)
-                numbers.push_back(BSON("" << -static_cast<long long>(x)));
-
-            if (i < 31)
-                numbers.push_back(BSON("" << -static_cast<int>(x)));
-
-            numbers.push_back(BSON("" << -static_cast<double>(x)));
-            numbers.push_back(BSON("" << -(static_cast<double>(x) + .1)));
-
-            numbers.push_back(BSON("" << -(static_cast<long long>(x) + 1)));
-            numbers.push_back(BSON("" << -(static_cast<int>(x) + 1)));
-            numbers.push_back(BSON("" << -(static_cast<double>(x) + 1)));
-            numbers.push_back(BSON("" << -(static_cast<double>(x) + 1.1)));
-        }
-
-        for (double i = 0; i < 1000; i++) {
-            double x = pow(2.1, i);
-            numbers.push_back(BSON("" << x));
-        }
-    }
-
-    Ordering ordering = Ordering::make(BSON("a" << 1));
-
-    std::vector<std::unique_ptr<KeyString::Builder>> KeyStringBuilders;
-    for (size_t i = 0; i < numbers.size(); i++) {
-        KeyStringBuilders.push_back(
-            std::make_unique<KeyString::Builder>(version, numbers[i], ordering));
-    }
-
-    for (size_t i = 0; i < numbers.size(); i++) {
-        for (size_t j = 0; j < numbers.size(); j++) {
-            const KeyString::Builder& a = *KeyStringBuilders[i];
-            const KeyString::Builder& b = *KeyStringBuilders[j];
-            ASSERT_EQUALS(a.compare(b), -b.compare(a));
-
-            if (a.compare(b) !=
-                compareNumbers(numbers[i].firstElement(), numbers[j].firstElement())) {
-                LOGV2(22235,
-                      "{numbers_i} {numbers_j}",
-                      "numbers_i"_attr = numbers[i],
-                      "numbers_j"_attr = numbers[j]);
-            }
-
-            ASSERT_EQUALS(a.compare(b),
-                          compareNumbers(numbers[i].firstElement(), numbers[j].firstElement()));
-        }
-    }
-}
 
 TEST_F(KeyStringBuilderTest, RecordIds) {
     for (int i = 0; i < 63; i++) {
@@ -1518,21 +1629,21 @@ TEST_F(KeyStringBuilderTest, RecordIds) {
             ASSERT_GTE(ks.getSize(), 2u);
             ASSERT_LTE(ks.getSize(), 10u);
 
-            ASSERT_EQ(KeyString::decodeRecordIdAtEnd(ks.getBuffer(), ks.getSize()), rid);
+            ASSERT_EQ(KeyString::decodeRecordIdLongAtEnd(ks.getBuffer(), ks.getSize()), rid);
 
             {
                 BufReader reader(ks.getBuffer(), ks.getSize());
-                ASSERT_EQ(KeyString::decodeRecordId(&reader), rid);
+                ASSERT_EQ(KeyString::decodeRecordIdLong(&reader), rid);
                 ASSERT(reader.atEof());
             }
 
             if (rid.isValid()) {
-                ASSERT_GT(ks, KeyString::Builder(version, RecordId()));
-                ASSERT_GT(ks, KeyString::Builder(version, RecordId::min()));
-                ASSERT_LT(ks, KeyString::Builder(version, RecordId::max()));
+                ASSERT_GTE(ks, KeyString::Builder(version, RecordId(1)));
+                ASSERT_GT(ks, KeyString::Builder(version, RecordId::minLong()));
+                ASSERT_LT(ks, KeyString::Builder(version, RecordId::maxLong()));
 
-                ASSERT_GT(ks, KeyString::Builder(version, RecordId(rid.repr() - 1)));
-                ASSERT_LT(ks, KeyString::Builder(version, RecordId(rid.repr() + 1)));
+                ASSERT_GT(ks, KeyString::Builder(version, RecordId(rid.getLong() - 1)));
+                ASSERT_LT(ks, KeyString::Builder(version, RecordId(rid.getLong() + 1)));
             }
         }
 
@@ -1552,7 +1663,7 @@ TEST_F(KeyStringBuilderTest, RecordIds) {
             {
                 // Test concatenating RecordIds like in a unique index.
                 KeyString::Builder ks(version);
-                ks.appendRecordId(RecordId::max());  // uses all bytes
+                ks.appendRecordId(RecordId::maxLong());  // uses all bytes
                 ks.appendRecordId(rid);
                 ks.appendRecordId(RecordId(0xDEADBEEF));  // uses some extra bytes
                 ks.appendRecordId(rid);
@@ -1560,17 +1671,17 @@ TEST_F(KeyStringBuilderTest, RecordIds) {
                 ks.appendRecordId(rid);
                 ks.appendRecordId(other);
 
-                ASSERT_EQ(KeyString::decodeRecordIdAtEnd(ks.getBuffer(), ks.getSize()), other);
+                ASSERT_EQ(KeyString::decodeRecordIdLongAtEnd(ks.getBuffer(), ks.getSize()), other);
 
                 // forward scan
                 BufReader reader(ks.getBuffer(), ks.getSize());
-                ASSERT_EQ(KeyString::decodeRecordId(&reader), RecordId::max());
-                ASSERT_EQ(KeyString::decodeRecordId(&reader), rid);
-                ASSERT_EQ(KeyString::decodeRecordId(&reader), RecordId(0xDEADBEEF));
-                ASSERT_EQ(KeyString::decodeRecordId(&reader), rid);
-                ASSERT_EQ(KeyString::decodeRecordId(&reader), RecordId(1));
-                ASSERT_EQ(KeyString::decodeRecordId(&reader), rid);
-                ASSERT_EQ(KeyString::decodeRecordId(&reader), other);
+                ASSERT_EQ(KeyString::decodeRecordIdLong(&reader), RecordId::maxLong());
+                ASSERT_EQ(KeyString::decodeRecordIdLong(&reader), rid);
+                ASSERT_EQ(KeyString::decodeRecordIdLong(&reader), RecordId(0xDEADBEEF));
+                ASSERT_EQ(KeyString::decodeRecordIdLong(&reader), rid);
+                ASSERT_EQ(KeyString::decodeRecordIdLong(&reader), RecordId(1));
+                ASSERT_EQ(KeyString::decodeRecordIdLong(&reader), rid);
+                ASSERT_EQ(KeyString::decodeRecordIdLong(&reader), other);
                 ASSERT(reader.atEof());
             }
         }
@@ -1632,8 +1743,8 @@ void checkKeyWithNByteOfTypeBits(KeyString::Version version, size_t n, bool allZ
     // Also test TypeBits::fromBuffer()
     BufReader bufReader(ks.getTypeBits().getBuffer(), typeBitsSize);
     KeyString::TypeBits newTypeBits = KeyString::TypeBits::fromBuffer(version, &bufReader);
-    ASSERT_EQ(toHex(newTypeBits.getBuffer(), newTypeBits.getSize()),
-              toHex(ks.getTypeBits().getBuffer(), ks.getTypeBits().getSize()));
+    ASSERT_EQ(hexblob::encode(newTypeBits.getBuffer(), newTypeBits.getSize()),
+              hexblob::encode(ks.getTypeBits().getBuffer(), ks.getTypeBits().getSize()));
 }
 
 TEST_F(KeyStringBuilderTest, KeysWithNBytesTypeBits) {
@@ -1948,3 +2059,129 @@ DEATH_TEST(KeyStringBuilderTest, ToBsonPromotesAssertionsToTerminate, "terminate
     KeyString::TypeBits typeBits(KeyString::Version::V1);
     KeyString::toBson(invalidString, sizeof(invalidString), ALL_ASCENDING, typeBits);
 }
+
+// The following tests run last because they take a very long time.
+
+TEST_F(KeyStringBuilderTest, LotsOfNumbers3) {
+    std::vector<stdx::future<void>> futures;
+
+    for (double k = 0; k < 8; k++) {
+        futures.push_back(stdx::async(stdx::launch::async, [k, this] {
+            for (double i = -1100; i < 1100; i++) {
+                for (double j = 0; j < 52; j++) {
+                    const auto V1 = KeyString::Version::V1;
+                    Decimal128::RoundingPrecision roundingPrecisions[]{
+                        Decimal128::kRoundTo15Digits, Decimal128::kRoundTo34Digits};
+                    Decimal128::RoundingMode roundingModes[]{Decimal128::kRoundTowardNegative,
+                                                             Decimal128::kRoundTowardPositive};
+                    double x = pow(2, i);
+                    double y = pow(2, i - j);
+                    double z = pow(2, i - 53 + k);
+                    double bin = x + y - z;
+
+                    // In general NaNs don't roundtrip as we only store a single NaN, see the NaNs
+                    // test.
+                    if (std::isnan(bin))
+                        continue;
+
+                    ROUNDTRIP(version, BSON("" << bin));
+                    ROUNDTRIP(version, BSON("" << -bin));
+
+                    if (version < V1)
+                        continue;
+
+                    for (auto precision : roundingPrecisions) {
+                        for (auto mode : roundingModes) {
+                            Decimal128 rounded = Decimal128(bin, precision, mode);
+                            ROUNDTRIP(V1, BSON("" << rounded));
+                            ROUNDTRIP(V1, BSON("" << rounded.negate()));
+                        }
+                    }
+                }
+            }
+        }));
+    }
+    for (auto&& future : futures) {
+        future.get();
+    }
+}
+
+TEST_F(KeyStringBuilderTest, NumberOrderLots) {
+    std::vector<BSONObj> numbers;
+    {
+        numbers.push_back(BSON("" << 0));
+        numbers.push_back(BSON("" << 0.0));
+        numbers.push_back(BSON("" << -0.0));
+
+        numbers.push_back(BSON("" << std::numeric_limits<long long>::min()));
+        numbers.push_back(BSON("" << std::numeric_limits<long long>::max()));
+        numbers.push_back(BSON("" << static_cast<double>(std::numeric_limits<long long>::min())));
+        numbers.push_back(BSON("" << static_cast<double>(std::numeric_limits<long long>::max())));
+        numbers.push_back(BSON("" << std::numeric_limits<double>::min()));
+        numbers.push_back(BSON("" << std::numeric_limits<double>::max()));
+        numbers.push_back(BSON("" << std::numeric_limits<int>::min()));
+        numbers.push_back(BSON("" << std::numeric_limits<int>::max()));
+        numbers.push_back(BSON("" << std::numeric_limits<short>::min()));
+        numbers.push_back(BSON("" << std::numeric_limits<short>::max()));
+
+        for (int i = 0; i < 64; i++) {
+            int64_t x = 1LL << i;
+            numbers.push_back(BSON("" << static_cast<long long>(x)));
+            numbers.push_back(BSON("" << static_cast<int>(x)));
+            numbers.push_back(BSON("" << static_cast<double>(x)));
+            numbers.push_back(BSON("" << (static_cast<double>(x) + .1)));
+
+            numbers.push_back(BSON("" << (static_cast<long long>(x) + 1)));
+            numbers.push_back(BSON("" << (static_cast<int>(x) + 1)));
+            numbers.push_back(BSON("" << (static_cast<double>(x) + 1)));
+            numbers.push_back(BSON("" << (static_cast<double>(x) + 1.1)));
+
+            // Avoid negating signed integral minima
+            if (i < 63)
+                numbers.push_back(BSON("" << -static_cast<long long>(x)));
+
+            if (i < 31)
+                numbers.push_back(BSON("" << -static_cast<int>(x)));
+
+            numbers.push_back(BSON("" << -static_cast<double>(x)));
+            numbers.push_back(BSON("" << -(static_cast<double>(x) + .1)));
+
+            numbers.push_back(BSON("" << -(static_cast<long long>(x) + 1)));
+            numbers.push_back(BSON("" << -(static_cast<int>(x) + 1)));
+            numbers.push_back(BSON("" << -(static_cast<double>(x) + 1)));
+            numbers.push_back(BSON("" << -(static_cast<double>(x) + 1.1)));
+        }
+
+        for (double i = 0; i < 1000; i++) {
+            double x = pow(2.1, i);
+            numbers.push_back(BSON("" << x));
+        }
+    }
+
+    Ordering ordering = Ordering::make(BSON("a" << 1));
+
+    std::vector<std::unique_ptr<KeyString::Builder>> KeyStringBuilders;
+    for (size_t i = 0; i < numbers.size(); i++) {
+        KeyStringBuilders.push_back(
+            std::make_unique<KeyString::Builder>(version, numbers[i], ordering));
+    }
+
+    for (size_t i = 0; i < numbers.size(); i++) {
+        for (size_t j = 0; j < numbers.size(); j++) {
+            const KeyString::Builder& a = *KeyStringBuilders[i];
+            const KeyString::Builder& b = *KeyStringBuilders[j];
+            ASSERT_EQUALS(a.compare(b), -b.compare(a));
+
+            if (a.compare(b) !=
+                compareNumbers(numbers[i].firstElement(), numbers[j].firstElement())) {
+                LOGV2(22235,
+                      "{numbers_i} {numbers_j}",
+                      "numbers_i"_attr = numbers[i],
+                      "numbers_j"_attr = numbers[j]);
+            }
+
+            ASSERT_EQUALS(a.compare(b),
+                          compareNumbers(numbers[i].firstElement(), numbers[j].firstElement()));
+        }
+    }
+}
