@@ -42,18 +42,22 @@
 #include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/service_context_d_test_fixture.h"
+#include "mongo/db/startup_recovery.h"
+#include "mongo/db/storage/control/storage_control.h"
 #include "mongo/db/storage/devnull/devnull_kv_engine.h"
-#include "mongo/db/storage/durable_catalog.h"
-#include "mongo/db/storage/ephemeral_for_test/ephemeral_for_test_engine.h"
 #include "mongo/db/storage/kv/kv_engine.h"
 #include "mongo/db/storage/storage_engine_impl.h"
 #include "mongo/db/storage/storage_engine_test_fixture.h"
 #include "mongo/db/storage/storage_repair_observer.h"
-#include "mongo/db/unclean_shutdown.h"
+#include "mongo/logv2/log.h"
 #include "mongo/unittest/barrier.h"
 #include "mongo/unittest/unittest.h"
+#include "mongo/util/future.h"
 #include "mongo/util/periodic_runner_factory.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
+
 namespace mongo {
 namespace {
 
@@ -64,6 +68,7 @@ TEST_F(StorageEngineTest, ReconcileIdentsTest) {
     // the `ident` name given to the collection.
     auto swCollInfo = createCollection(opCtx.get(), NamespaceString("db.coll1"));
     ASSERT_OK(swCollInfo.getStatus());
+
     // Create a table in the KVEngine not reflected in the DurableCatalog. This should be dropped
     // when reconciling.
     ASSERT_OK(createCollTable(opCtx.get(), NamespaceString("db.coll2")));
@@ -74,26 +79,14 @@ TEST_F(StorageEngineTest, ReconcileIdentsTest) {
 
     auto identsVec = getAllKVEngineIdents(opCtx.get());
     auto idents = std::set<std::string>(identsVec.begin(), identsVec.end());
+
     // There are two idents. `_mdb_catalog` and the ident for `db.coll1`.
     ASSERT_EQUALS(static_cast<const unsigned long>(2), idents.size());
     ASSERT_TRUE(idents.find(swCollInfo.getValue().ident) != idents.end());
     ASSERT_TRUE(idents.find("_mdb_catalog") != idents.end());
 
-    // Create a catalog entry for the `_id` index. Drop the created the table.
-    ASSERT_OK(createIndex(
-        opCtx.get(), NamespaceString("db.coll1"), "_id", false /* isBackgroundSecondaryBuild */));
-    ASSERT_OK(dropIndexTable(opCtx.get(), NamespaceString("db.coll1"), "_id"));
-    // The reconcile response should include this index as needing to be rebuilt.
-    reconcileResult = unittest::assertGet(reconcile(opCtx.get()));
-    ASSERT_EQUALS(1UL, reconcileResult.indexesToRebuild.size());
-    ASSERT_EQUALS(0UL, reconcileResult.indexBuildsToRestart.size());
-
-    StorageEngine::IndexIdentifier& toRebuild = reconcileResult.indexesToRebuild[0];
-    ASSERT_EQUALS("db.coll1", toRebuild.nss.ns());
-    ASSERT_EQUALS("_id", toRebuild.indexName);
-
-    // Now drop the `db.coll1` table, while leaving the DurableCatalog entry.
-    ASSERT_OK(dropIdent(opCtx.get(), opCtx.get()->recoveryUnit(), swCollInfo.getValue().ident));
+    // Drop the `db.coll1` table, while leaving the DurableCatalog entry.
+    ASSERT_OK(dropIdent(opCtx.get()->recoveryUnit(), swCollInfo.getValue().ident));
     ASSERT_EQUALS(static_cast<const unsigned long>(1), getAllKVEngineIdents(opCtx.get()).size());
 
     // Reconciling this should result in an error.
@@ -109,7 +102,7 @@ TEST_F(StorageEngineTest, LoadCatalogDropsOrphansAfterUncleanShutdown) {
     auto swCollInfo = createCollection(opCtx.get(), collNs);
     ASSERT_OK(swCollInfo.getStatus());
 
-    ASSERT_OK(dropIdent(opCtx.get(), opCtx.get()->recoveryUnit(), swCollInfo.getValue().ident));
+    ASSERT_OK(dropIdent(opCtx.get()->recoveryUnit(), swCollInfo.getValue().ident));
     ASSERT(collectionExists(opCtx.get(), collNs));
 
     // After the catalog is reloaded, we expect that the collection has been dropped because the
@@ -117,14 +110,38 @@ TEST_F(StorageEngineTest, LoadCatalogDropsOrphansAfterUncleanShutdown) {
     {
         Lock::GlobalWrite writeLock(opCtx.get(), Date_t::max(), Lock::InterruptBehavior::kThrow);
         _storageEngine->closeCatalog(opCtx.get());
-        startingAfterUncleanShutdown(getGlobalServiceContext()) = true;
-        _storageEngine->loadCatalog(opCtx.get());
+        _storageEngine->loadCatalog(opCtx.get(), StorageEngine::LastShutdownState::kUnclean);
     }
 
     ASSERT(!identExists(opCtx.get(), swCollInfo.getValue().ident));
     ASSERT(!collectionExists(opCtx.get(), collNs));
 }
 
+TEST_F(StorageEngineTest, TemporaryRecordStoreClustered) {
+    auto opCtx = cc().makeOperationContext();
+
+    Lock::GlobalLock lk(&*opCtx, MODE_IS);
+
+    const auto trs = makeTemporaryClustered(opCtx.get());
+    ASSERT(trs.get());
+    const auto rs = trs->rs();
+    ASSERT(identExists(opCtx.get(), rs->getIdent()));
+
+    // Insert record with RecordId of KeyFormat::String.
+    const auto id = StringData{"1"};
+    const auto rid = RecordId(id.rawData(), id.size());
+    const auto data = "data";
+    WriteUnitOfWork wuow(opCtx.get());
+    StatusWith<RecordId> s = rs->insertRecord(opCtx.get(), rid, data, strlen(data), Timestamp());
+    ASSERT_TRUE(s.isOK());
+    wuow.commit();
+
+    // Read the record back.
+    RecordData rd;
+    ASSERT_TRUE(rs->findRecord(opCtx.get(), rid, &rd));
+    ASSERT_EQ(0, memcmp(data, rd.data(), strlen(data)));
+}
+
 TEST_F(StorageEngineTest, ReconcileDropsTemporary) {
     auto opCtx = cc().makeOperationContext();
 
@@ -136,40 +153,97 @@ TEST_F(StorageEngineTest, ReconcileDropsTemporary) {
 
     ASSERT(identExists(opCtx.get(), ident));
 
+    // Reconcile will only drop temporary idents when starting up after an unclean shutdown.
+    auto reconcileResult = unittest::assertGet(reconcileAfterUncleanShutdown(opCtx.get()));
+    ASSERT_EQUALS(0UL, reconcileResult.indexesToRebuild.size());
+    ASSERT_EQUALS(0UL, reconcileResult.indexBuildsToRestart.size());
+    ASSERT_EQUALS(0UL, reconcileResult.indexBuildsToResume.size());
+
+    // The storage engine is responsible for dropping its temporary idents.
+    ASSERT(!identExists(opCtx.get(), ident));
+}
+
+TEST_F(StorageEngineTest, ReconcileKeepsTemporary) {
+    auto opCtx = cc().makeOperationContext();
+
+    Lock::GlobalLock lk(&*opCtx, MODE_IS);
+
+    auto rs = makeTemporary(opCtx.get());
+    ASSERT(rs.get());
+    const std::string ident = rs->rs()->getIdent();
+
+    ASSERT(identExists(opCtx.get(), ident));
+
     auto reconcileResult = unittest::assertGet(reconcile(opCtx.get()));
     ASSERT_EQUALS(0UL, reconcileResult.indexesToRebuild.size());
     ASSERT_EQUALS(0UL, reconcileResult.indexBuildsToRestart.size());
 
-    // The storage engine is responsible for dropping its temporary idents.
-    ASSERT(!identExists(opCtx.get(), ident));
-
-    rs->deleteTemporaryTable(opCtx.get());
+    ASSERT_FALSE(identExists(opCtx.get(), ident));
 }
 
-TEST_F(StorageEngineTest, TemporaryDropsItself) {
-    auto opCtx = cc().makeOperationContext();
+class StorageEngineTimestampMonitorTest : public StorageEngineTest {
+public:
+    void setUp() {
+        StorageEngineTest::setUp();
+        _storageEngine->startTimestampMonitor();
+    }
 
-    Lock::GlobalLock lk(&*opCtx, MODE_IS);
+    void waitForTimestampMonitorPass() {
+        auto timestampMonitor =
+            dynamic_cast<StorageEngineImpl*>(_storageEngine)->getTimestampMonitor();
+        using TimestampType = StorageEngineImpl::TimestampMonitor::TimestampType;
+        using TimestampListener = StorageEngineImpl::TimestampMonitor::TimestampListener;
+        auto pf = makePromiseFuture<void>();
+        auto listener =
+            TimestampListener(TimestampType::kOldest, [promise = &pf.promise](Timestamp t) mutable {
+                promise->emplaceValue();
+            });
+        timestampMonitor->addListener(&listener);
+        pf.future.wait();
+        timestampMonitor->removeListener_forTestOnly(&listener);
+    }
+};
+
+TEST_F(StorageEngineTimestampMonitorTest, TemporaryRecordStoreEventuallyDropped) {
+    auto opCtx = cc().makeOperationContext();
 
     std::string ident;
     {
-        auto rs = makeTemporary(opCtx.get());
-        ASSERT(rs.get());
-        ident = rs->rs()->getIdent();
+        auto tempRs = _storageEngine->makeTemporaryRecordStore(opCtx.get(), KeyFormat::Long);
+        ASSERT(tempRs.get());
+        ident = tempRs->rs()->getIdent();
 
         ASSERT(identExists(opCtx.get(), ident));
-
-        rs->deleteTemporaryTable(opCtx.get());
     }
 
-    // The temporary record store RAII class should drop itself.
+    // The temporary record store RAII object should queue itself to be dropped by the storage
+    // engine eventually.
+    waitForTimestampMonitorPass();
     ASSERT(!identExists(opCtx.get(), ident));
 }
 
+TEST_F(StorageEngineTimestampMonitorTest, TemporaryRecordStoreKeep) {
+    auto opCtx = cc().makeOperationContext();
+
+    std::string ident;
+    {
+        auto tempRs = _storageEngine->makeTemporaryRecordStore(opCtx.get(), KeyFormat::Long);
+        ASSERT(tempRs.get());
+        ident = tempRs->rs()->getIdent();
+
+        ASSERT(identExists(opCtx.get(), ident));
+        tempRs->keep();
+    }
+
+    // The ident for the record store should still exist even after a pass of the timestamp monitor.
+    waitForTimestampMonitorPass();
+    ASSERT(identExists(opCtx.get(), ident));
+}
+
 TEST_F(StorageEngineTest, ReconcileUnfinishedIndex) {
     auto opCtx = cc().makeOperationContext();
 
-    Lock::GlobalLock lk(&*opCtx, MODE_IS);
+    Lock::GlobalLock lk(&*opCtx, MODE_X);
 
     const NamespaceString ns("db.coll1");
     const std::string indexName("a_1");
@@ -181,7 +255,12 @@ TEST_F(StorageEngineTest, ReconcileUnfinishedIndex) {
     // Start an non-backgroundSecondary single-phase (i.e. no build UUID) index.
     const bool isBackgroundSecondaryBuild = false;
     const boost::optional<UUID> buildUUID = boost::none;
-    ASSERT_OK(startIndexBuild(opCtx.get(), ns, indexName, isBackgroundSecondaryBuild, buildUUID));
+    {
+        WriteUnitOfWork wuow(opCtx.get());
+        ASSERT_OK(
+            startIndexBuild(opCtx.get(), ns, indexName, isBackgroundSecondaryBuild, buildUUID));
+        wuow.commit();
+    }
 
     const auto indexIdent = _storageEngine->getCatalog()->getIndexIdent(
         opCtx.get(), swCollInfo.getValue().catalogId, indexName);
@@ -195,14 +274,15 @@ TEST_F(StorageEngineTest, ReconcileUnfinishedIndex) {
     // not require it to be rebuilt.
     ASSERT_EQUALS(0UL, reconcileResult.indexesToRebuild.size());
 
-    // There are no two-phase builds to restart.
+    // There are no two-phase builds to resume or restart.
     ASSERT_EQUALS(0UL, reconcileResult.indexBuildsToRestart.size());
+    ASSERT_EQUALS(0UL, reconcileResult.indexBuildsToResume.size());
 }
 
 TEST_F(StorageEngineTest, ReconcileUnfinishedBackgroundSecondaryIndex) {
     auto opCtx = cc().makeOperationContext();
 
-    Lock::GlobalLock lk(&*opCtx, MODE_IS);
+    Lock::GlobalLock lk(&*opCtx, MODE_IX);
 
     const NamespaceString ns("db.coll1");
     const std::string indexName("a_1");
@@ -213,7 +293,15 @@ TEST_F(StorageEngineTest, ReconcileUnfinishedBackgroundSecondaryIndex) {
     // Start a backgroundSecondary single-phase (i.e. no build UUID) index.
     const bool isBackgroundSecondaryBuild = true;
     const boost::optional<UUID> buildUUID = boost::none;
-    ASSERT_OK(startIndexBuild(opCtx.get(), ns, indexName, isBackgroundSecondaryBuild, buildUUID));
+    {
+        Lock::DBLock dbLk(opCtx.get(), ns.dbName(), MODE_IX);
+        Lock::CollectionLock collLk(opCtx.get(), ns, MODE_X);
+
+        WriteUnitOfWork wuow(opCtx.get());
+        ASSERT_OK(
+            startIndexBuild(opCtx.get(), ns, indexName, isBackgroundSecondaryBuild, buildUUID));
+        wuow.commit();
+    }
 
     const auto indexIdent = _storageEngine->getCatalog()->getIndexIdent(
         opCtx.get(), swCollInfo.getValue().catalogId, indexName);
@@ -231,14 +319,15 @@ TEST_F(StorageEngineTest, ReconcileUnfinishedBackgroundSecondaryIndex) {
     ASSERT_EQUALS(ns.ns(), toRebuild.nss.ns());
     ASSERT_EQUALS(indexName, toRebuild.indexName);
 
-    // There are no two-phase builds to restart.
+    // There are no two-phase builds to restart or resume.
     ASSERT_EQUALS(0UL, reconcileResult.indexBuildsToRestart.size());
+    ASSERT_EQUALS(0UL, reconcileResult.indexBuildsToResume.size());
 }
 
 TEST_F(StorageEngineTest, ReconcileTwoPhaseIndexBuilds) {
     auto opCtx = cc().makeOperationContext();
 
-    Lock::GlobalLock lk(&*opCtx, MODE_IS);
+    Lock::GlobalLock lk(&*opCtx, MODE_IX);
 
     const NamespaceString ns("db.coll1");
     const std::string indexA("a_1");
@@ -254,8 +343,22 @@ TEST_F(StorageEngineTest, ReconcileTwoPhaseIndexBuilds) {
 
     // Start two indexes with the same buildUUID to simulate building multiple indexes within the
     // same build.
-    ASSERT_OK(startIndexBuild(opCtx.get(), ns, indexA, isBackgroundSecondaryBuild, buildUUID));
-    ASSERT_OK(startIndexBuild(opCtx.get(), ns, indexB, isBackgroundSecondaryBuild, buildUUID));
+    {
+        Lock::DBLock dbLk(opCtx.get(), ns.dbName(), MODE_IX);
+        Lock::CollectionLock collLk(opCtx.get(), ns, MODE_X);
+        {
+            WriteUnitOfWork wuow(opCtx.get());
+            ASSERT_OK(
+                startIndexBuild(opCtx.get(), ns, indexA, isBackgroundSecondaryBuild, buildUUID));
+            wuow.commit();
+        }
+        {
+            WriteUnitOfWork wuow(opCtx.get());
+            ASSERT_OK(
+                startIndexBuild(opCtx.get(), ns, indexB, isBackgroundSecondaryBuild, buildUUID));
+            wuow.commit();
+        }
+    }
 
     const auto indexIdentA = _storageEngine->getCatalog()->getIndexIdent(
         opCtx.get(), swCollInfo.getValue().catalogId, indexA);
@@ -283,8 +386,12 @@ TEST_F(StorageEngineTest, ReconcileTwoPhaseIndexBuilds) {
     ASSERT_EQ(2UL, specs.size());
     ASSERT_EQ(indexA, specs[0]["name"].str());
     ASSERT_EQ(indexB, specs[1]["name"].str());
+
+    // There should be no index builds to resume.
+    ASSERT_EQUALS(0UL, reconcileResult.indexBuildsToResume.size());
 }
 
+#ifndef _WIN32  // WiredTiger does not support orphan file recovery on Windows.
 TEST_F(StorageEngineRepairTest, LoadCatalogRecoversOrphans) {
     auto opCtx = cc().makeOperationContext();
 
@@ -292,7 +399,8 @@ TEST_F(StorageEngineRepairTest, LoadCatalogRecoversOrphans) {
     auto swCollInfo = createCollection(opCtx.get(), collNs);
     ASSERT_OK(swCollInfo.getStatus());
 
-    ASSERT_OK(dropIdent(opCtx.get(), opCtx.get()->recoveryUnit(), swCollInfo.getValue().ident));
+    // Drop the ident from the storage engine but keep the underlying files.
+    _storageEngine->getEngine()->dropIdentForImport(opCtx.get(), swCollInfo.getValue().ident);
     ASSERT(collectionExists(opCtx.get(), collNs));
 
     // After the catalog is reloaded, we expect that the ident has been recovered because the
@@ -300,7 +408,7 @@ TEST_F(StorageEngineRepairTest, LoadCatalogRecoversOrphans) {
     {
         Lock::GlobalWrite writeLock(opCtx.get(), Date_t::max(), Lock::InterruptBehavior::kThrow);
         _storageEngine->closeCatalog(opCtx.get());
-        _storageEngine->loadCatalog(opCtx.get());
+        _storageEngine->loadCatalog(opCtx.get(), StorageEngine::LastShutdownState::kClean);
     }
 
     ASSERT(identExists(opCtx.get(), swCollInfo.getValue().ident));
@@ -308,6 +416,7 @@ TEST_F(StorageEngineRepairTest, LoadCatalogRecoversOrphans) {
     StorageRepairObserver::get(getGlobalServiceContext())->onRepairDone(opCtx.get());
     ASSERT_EQ(1U, StorageRepairObserver::get(getGlobalServiceContext())->getModifications().size());
 }
+#endif
 
 TEST_F(StorageEngineRepairTest, ReconcileSucceeds) {
     auto opCtx = cc().makeOperationContext();
@@ -316,7 +425,7 @@ TEST_F(StorageEngineRepairTest, ReconcileSucceeds) {
     auto swCollInfo = createCollection(opCtx.get(), collNs);
     ASSERT_OK(swCollInfo.getStatus());
 
-    ASSERT_OK(dropIdent(opCtx.get(), opCtx.get()->recoveryUnit(), swCollInfo.getValue().ident));
+    ASSERT_OK(dropIdent(opCtx.get()->recoveryUnit(), swCollInfo.getValue().ident));
     ASSERT(collectionExists(opCtx.get(), collNs));
 
     // Reconcile would normally return an error if a collection existed with a missing ident in the
@@ -324,6 +433,7 @@ TEST_F(StorageEngineRepairTest, ReconcileSucceeds) {
     auto reconcileResult = unittest::assertGet(reconcile(opCtx.get()));
     ASSERT_EQUALS(0UL, reconcileResult.indexesToRebuild.size());
     ASSERT_EQUALS(0UL, reconcileResult.indexBuildsToRestart.size());
+    ASSERT_EQUALS(0UL, reconcileResult.indexBuildsToResume.size());
 
     ASSERT(!identExists(opCtx.get(), swCollInfo.getValue().ident));
     ASSERT(collectionExists(opCtx.get(), collNs));
@@ -339,15 +449,23 @@ TEST_F(StorageEngineRepairTest, LoadCatalogRecoversOrphansInCatalog) {
     ASSERT_OK(swCollInfo.getStatus());
     ASSERT(collectionExists(opCtx.get(), collNs));
 
-    AutoGetDb db(opCtx.get(), collNs.db(), LockMode::MODE_X);
+    AutoGetDb db(opCtx.get(), collNs.dbName(), LockMode::MODE_X);
     // Only drop the catalog entry; storage engine still knows about this ident.
     // This simulates an unclean shutdown happening between dropping the catalog entry and
     // the actual drop in storage engine.
-    ASSERT_OK(removeEntry(opCtx.get(), collNs.ns(), _storageEngine->getCatalog()));
+    {
+        WriteUnitOfWork wuow(opCtx.get());
+        ASSERT_OK(removeEntry(opCtx.get(), collNs.ns(), _storageEngine->getCatalog()));
+        wuow.commit();
+    }
+
     ASSERT(!collectionExists(opCtx.get(), collNs));
 
     // When in a repair context, loadCatalog() recreates catalog entries for orphaned idents.
-    _storageEngine->loadCatalog(opCtx.get());
+    {
+        Lock::GlobalWrite writeLock(opCtx.get(), Date_t::max(), Lock::InterruptBehavior::kThrow);
+        _storageEngine->loadCatalog(opCtx.get(), StorageEngine::LastShutdownState::kClean);
+    }
     auto identNs = swCollInfo.getValue().ident;
     std::replace(identNs.begin(), identNs.end(), '-', '_');
     NamespaceString orphanNs = NamespaceString("local.orphan." + identNs);
@@ -367,16 +485,23 @@ TEST_F(StorageEngineTest, LoadCatalogDropsOrphans) {
     ASSERT_OK(swCollInfo.getStatus());
     ASSERT(collectionExists(opCtx.get(), collNs));
 
-    AutoGetDb db(opCtx.get(), collNs.db(), LockMode::MODE_X);
+    AutoGetDb db(opCtx.get(), collNs.dbName(), LockMode::MODE_X);
     // Only drop the catalog entry; storage engine still knows about this ident.
     // This simulates an unclean shutdown happening between dropping the catalog entry and
     // the actual drop in storage engine.
-    ASSERT_OK(removeEntry(opCtx.get(), collNs.ns(), _storageEngine->getCatalog()));
+    {
+        WriteUnitOfWork wuow(opCtx.get());
+        ASSERT_OK(removeEntry(opCtx.get(), collNs.ns(), _storageEngine->getCatalog()));
+        wuow.commit();
+    }
     ASSERT(!collectionExists(opCtx.get(), collNs));
 
     // When in a normal startup context, loadCatalog() does not recreate catalog entries for
     // orphaned idents.
-    _storageEngine->loadCatalog(opCtx.get());
+    {
+        Lock::GlobalWrite writeLock(opCtx.get(), Date_t::max(), Lock::InterruptBehavior::kThrow);
+        _storageEngine->loadCatalog(opCtx.get(), StorageEngine::LastShutdownState::kClean);
+    }
     // reconcileCatalogAndIdents() drops orphaned idents.
     auto reconcileResult = unittest::assertGet(reconcile(opCtx.get()));
     ASSERT_EQUALS(0UL, reconcileResult.indexesToRebuild.size());
@@ -418,7 +543,7 @@ public:
     mutable std::unique_ptr<Timestamp> stableTimestamp = std::make_unique<Timestamp>();
 };
 
-class TimestampKVEngineTest : public ServiceContextMongoDTest {
+class TimestampKVEngineTest : public ServiceContextTest {
 public:
     using TimestampType = StorageEngineImpl::TimestampMonitor::TimestampType;
     using TimestampListener = StorageEngineImpl::TimestampMonitor::TimestampListener;
@@ -426,16 +551,28 @@ public:
     /**
      * Create an instance of the KV Storage Engine so that we have a timestamp monitor operating.
      */
-    TimestampKVEngineTest() {
-        StorageEngineOptions options{
-            /*directoryPerDB=*/false, /*directoryForIndexes=*/false, /*forRepair=*/false};
-        _storageEngine = std::make_unique<StorageEngineImpl>(new TimestampMockKVEngine, options);
-        _storageEngine->finishInit();
+    void setUp() {
+        ServiceContextTest::setUp();
+
+        auto opCtx = makeOperationContext();
+
+        auto runner = makePeriodicRunner(getServiceContext());
+        getServiceContext()->setPeriodicRunner(std::move(runner));
+
+        StorageEngineOptions options{/*directoryPerDB=*/false,
+                                     /*directoryForIndexes=*/false,
+                                     /*forRepair=*/false,
+                                     /*lockFileCreatedByUncleanShutdown=*/false};
+        _storageEngine = std::make_unique<StorageEngineImpl>(
+            opCtx.get(), std::make_unique<TimestampMockKVEngine>(), options);
+        _storageEngine->startTimestampMonitor();
     }
 
-    ~TimestampKVEngineTest() {
+    void tearDown() {
         _storageEngine->cleanShutdown();
         _storageEngine.reset();
+
+        ServiceContextTest::tearDown();
     }
 
     std::unique_ptr<StorageEngineImpl> _storageEngine;
@@ -461,16 +598,14 @@ TEST_F(TimestampKVEngineTest, TimestampListeners) {
     // Can only register the listener once.
     _storageEngine->getTimestampMonitor()->addListener(&first);
 
-    _storageEngine->getTimestampMonitor()->removeListener(&first);
+    _storageEngine->getTimestampMonitor()->clearListeners();
     _storageEngine->getTimestampMonitor()->addListener(&first);
 
     // Can register all three types of listeners.
     _storageEngine->getTimestampMonitor()->addListener(&second);
     _storageEngine->getTimestampMonitor()->addListener(&third);
 
-    _storageEngine->getTimestampMonitor()->removeListener(&first);
-    _storageEngine->getTimestampMonitor()->removeListener(&second);
-    _storageEngine->getTimestampMonitor()->removeListener(&third);
+    _storageEngine->getTimestampMonitor()->clearListeners();
 }
 
 TEST_F(TimestampKVEngineTest, TimestampMonitorNotifiesListeners) {
@@ -527,31 +662,70 @@ TEST_F(TimestampKVEngineTest, TimestampMonitorNotifiesListeners) {
         return true;
     });
 
-    _storageEngine->getTimestampMonitor()->removeListener(&first);
-    _storageEngine->getTimestampMonitor()->removeListener(&second);
-    _storageEngine->getTimestampMonitor()->removeListener(&third);
-    _storageEngine->getTimestampMonitor()->removeListener(&fourth);
+    _storageEngine->getTimestampMonitor()->clearListeners();
 }
 
 TEST_F(TimestampKVEngineTest, TimestampAdvancesOnNotification) {
     Timestamp previous = Timestamp();
-    int timesNotified = 0;
+    AtomicWord<int> timesNotified{0};
 
     TimestampListener listener(stable, [&](Timestamp timestamp) {
         ASSERT_TRUE(previous < timestamp);
         previous = timestamp;
-        timesNotified++;
+        timesNotified.fetchAndAdd(1);
     });
     _storageEngine->getTimestampMonitor()->addListener(&listener);
 
     // Let three rounds of notifications happen while ensuring that each new notification produces
     // an increasing timestamp.
-    while (timesNotified < 3) {
+    while (timesNotified.load() < 3) {
         sleepmillis(100);
     }
 
-    _storageEngine->getTimestampMonitor()->removeListener(&listener);
+    _storageEngine->getTimestampMonitor()->clearListeners();
 }
 
+TEST_F(StorageEngineTestNotEphemeral, UseAlternateStorageLocation) {
+    auto opCtx = cc().makeOperationContext();
+
+    const NamespaceString coll1Ns("db.coll1");
+    const NamespaceString coll2Ns("db.coll2");
+    auto swCollInfo = createCollection(opCtx.get(), coll1Ns);
+    ASSERT_OK(swCollInfo.getStatus());
+    ASSERT(collectionExists(opCtx.get(), coll1Ns));
+    ASSERT_FALSE(collectionExists(opCtx.get(), coll2Ns));
+
+    LOGV2(5781102, "Starting up storage engine in alternate location");
+    const auto oldPath = storageGlobalParams.dbpath;
+    const auto newPath = boost::filesystem::path(oldPath).append(".alternate").string();
+    boost::filesystem::create_directory(newPath);
+    auto lastShutdownState =
+        reinitializeStorageEngine(opCtx.get(), StorageEngineInitFlags{}, [&newPath] {
+            storageGlobalParams.dbpath = newPath;
+        });
+    getGlobalServiceContext()->getStorageEngine()->notifyStartupComplete();
+    LOGV2(5781103, "Started up storage engine in alternate location");
+    ASSERT(StorageEngine::LastShutdownState::kClean == lastShutdownState);
+    StorageEngineTest::_storageEngine = getServiceContext()->getStorageEngine();
+    // Alternate storage location should have no collections.
+    ASSERT_FALSE(collectionExists(opCtx.get(), coll1Ns));
+    ASSERT_FALSE(collectionExists(opCtx.get(), coll2Ns));
+
+    swCollInfo = createCollection(opCtx.get(), coll2Ns);
+    ASSERT_OK(swCollInfo.getStatus());
+    ASSERT_FALSE(collectionExists(opCtx.get(), coll1Ns));
+    ASSERT_TRUE(collectionExists(opCtx.get(), coll2Ns));
+
+    LOGV2(5781104, "Starting up storage engine in original location");
+    lastShutdownState =
+        reinitializeStorageEngine(opCtx.get(), StorageEngineInitFlags{}, [&oldPath] {
+            storageGlobalParams.dbpath = oldPath;
+        });
+    getGlobalServiceContext()->getStorageEngine()->notifyStartupComplete();
+    ASSERT(StorageEngine::LastShutdownState::kClean == lastShutdownState);
+    StorageEngineTest::_storageEngine = getServiceContext()->getStorageEngine();
+    ASSERT_TRUE(collectionExists(opCtx.get(), coll1Ns));
+    ASSERT_FALSE(collectionExists(opCtx.get(), coll2Ns));
+}
 }  // namespace
 }  // namespace mongo
