@@ -27,11 +27,8 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
+#include "mongo/db/catalog/capped_collection_maintenance.h"
 #include "mongo/db/storage/record_store_test_harness.h"
-
-#include "mongo/db/storage/record_store.h"
 #include "mongo/unittest/unittest.h"
 
 namespace mongo {
@@ -63,16 +60,9 @@ RecordId _oplogOrderInsertOplog(OperationContext* opCtx,
     return res.getValue();
 }
 
-TEST(RecordStoreTestHarness, OplogHack) {
+TEST(RecordStoreTestHarness, SeekNearOplog) {
     std::unique_ptr<RecordStoreHarnessHelper> harnessHelper = newRecordStoreHarnessHelper();
-    if (!harnessHelper->supportsDocLocking())
-        return;
-
-    // Use a large enough cappedMaxSize so that the limit is not reached by doing the inserts within
-    // the test itself.
-    const int64_t cappedMaxSize = 10 * 1024;  // 10KB
-    std::unique_ptr<RecordStore> rs(
-        harnessHelper->newCappedRecordStore("local.oplog.rs", cappedMaxSize, -1));
+    std::unique_ptr<RecordStore> rs(harnessHelper->newOplogRecordStore());
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
 
@@ -106,44 +96,126 @@ TEST(RecordStoreTestHarness, OplogHack) {
     // Make sure all are visible.
     rs->waitForAllEarlierOplogWritesToBeVisible(harnessHelper->newOperationContext().get());
 
+    // Forward cursor seeks
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
         WriteUnitOfWork wuow(opCtx.get());
-        // find start
-        ASSERT_EQ(rs->oplogStartHack(opCtx.get(), RecordId(0, 1)), RecordId());      // nothing <=
-        ASSERT_EQ(rs->oplogStartHack(opCtx.get(), RecordId(2, 1)), RecordId(1, 2));  // between
-        ASSERT_EQ(rs->oplogStartHack(opCtx.get(), RecordId(2, 2)), RecordId(2, 2));  // ==
-        ASSERT_EQ(rs->oplogStartHack(opCtx.get(), RecordId(2, 3)), RecordId(2, 2));  // > highest
+        auto cur = rs->getCursor(opCtx.get());
+        auto rec = cur->seekNear(RecordId(0, 1));
+        ASSERT(rec);
+        ASSERT_EQ(rec->id, RecordId(1, 1));
     }
 
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
-        rs->cappedTruncateAfter(opCtx.get(), RecordId(2, 2), false);  // no-op
+        WriteUnitOfWork wuow(opCtx.get());
+        auto cur = rs->getCursor(opCtx.get());
+        auto rec = cur->seekNear(RecordId(2, 1));
+        ASSERT(rec);
+        ASSERT_EQ(rec->id, RecordId(1, 2));
     }
 
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
-        ASSERT_EQ(rs->oplogStartHack(opCtx.get(), RecordId(2, 3)), RecordId(2, 2));
+        WriteUnitOfWork wuow(opCtx.get());
+        auto cur = rs->getCursor(opCtx.get());
+        auto rec = cur->seekNear(RecordId(2, 2));
+        ASSERT(rec);
+        ASSERT_EQ(rec->id, RecordId(2, 2));
     }
 
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
-        rs->cappedTruncateAfter(opCtx.get(), RecordId(1, 2), false);  // deletes 2,2
+        WriteUnitOfWork wuow(opCtx.get());
+        auto cur = rs->getCursor(opCtx.get());
+        auto rec = cur->seekNear(RecordId(2, 3));
+        ASSERT(rec);
+        ASSERT_EQ(rec->id, RecordId(2, 2));
+    }
+
+    // Reverse cursor seeks
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        WriteUnitOfWork wuow(opCtx.get());
+        auto cur = rs->getCursor(opCtx.get(), false /* forward */);
+        auto rec = cur->seekNear(RecordId(0, 1));
+        ASSERT(rec);
+        ASSERT_EQ(rec->id, RecordId(1, 1));
     }
 
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
-        ASSERT_EQ(rs->oplogStartHack(opCtx.get(), RecordId(2, 3)), RecordId(1, 2));
+        WriteUnitOfWork wuow(opCtx.get());
+        auto cur = rs->getCursor(opCtx.get(), false /* forward */);
+        auto rec = cur->seekNear(RecordId(2, 1));
+        ASSERT(rec);
+        ASSERT_EQ(rec->id, RecordId(2, 2));
     }
 
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
-        rs->cappedTruncateAfter(opCtx.get(), RecordId(1, 2), true);  // deletes 1,2
+        WriteUnitOfWork wuow(opCtx.get());
+        auto cur = rs->getCursor(opCtx.get(), false /* forward */);
+        auto rec = cur->seekNear(RecordId(2, 2));
+        ASSERT(rec);
+        ASSERT_EQ(rec->id, RecordId(2, 2));
     }
 
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
-        ASSERT_EQ(rs->oplogStartHack(opCtx.get(), RecordId(2, 3)), RecordId(1, 1));
+        WriteUnitOfWork wuow(opCtx.get());
+        auto cur = rs->getCursor(opCtx.get(), false /* forward */);
+        auto rec = cur->seekNear(RecordId(2, 3));
+        ASSERT(rec);
+        ASSERT_EQ(rec->id, RecordId(2, 2));
+    }
+
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        rs->cappedTruncateAfter(opCtx.get(),
+                                RecordId(2, 2),
+                                false /* inclusive */,
+                                nullptr /* aboutToDelete callback */);  // no-op
+    }
+
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        auto cur = rs->getCursor(opCtx.get());
+        auto rec = cur->seekNear(RecordId(2, 3));
+        ASSERT(rec);
+        ASSERT_EQ(rec->id, RecordId(2, 2));
+    }
+
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        rs->cappedTruncateAfter(opCtx.get(),
+                                RecordId(1, 2),
+                                false /* inclusive */,
+                                nullptr /* aboutToDelete callback */);  // deletes 2,2
+    }
+
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        auto cur = rs->getCursor(opCtx.get());
+        auto rec = cur->seekNear(RecordId(2, 3));
+        ASSERT(rec);
+        ASSERT_EQ(rec->id, RecordId(1, 2));
+    }
+
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        rs->cappedTruncateAfter(opCtx.get(),
+                                RecordId(1, 2),
+                                true /* inclusive */,
+                                nullptr /* aboutToDelete callback */);  // deletes 1,2
+    }
+
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        auto cur = rs->getCursor(opCtx.get());
+        auto rec = cur->seekNear(RecordId(2, 3));
+        ASSERT(rec);
+        ASSERT_EQ(rec->id, RecordId(1, 1));
     }
 
     {
@@ -155,18 +227,16 @@ TEST(RecordStoreTestHarness, OplogHack) {
 
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
-        ASSERT_EQ(rs->oplogStartHack(opCtx.get(), RecordId(2, 3)), RecordId());
+        auto cur = rs->getCursor(opCtx.get());
+        auto rec = cur->seekNear(RecordId(2, 3));
+        ASSERT_FALSE(rec);
     }
 }
 
 TEST(RecordStoreTestHarness, OplogInsertOutOfOrder) {
     std::unique_ptr<RecordStoreHarnessHelper> harnessHelper = newRecordStoreHarnessHelper();
-    if (!harnessHelper->supportsDocLocking())
-        return;
+    std::unique_ptr<RecordStore> rs(harnessHelper->newOplogRecordStore());
 
-    const int64_t cappedMaxSize = 10 * 1024;  // Large enough to not exceed.
-    std::unique_ptr<RecordStore> rs(
-        harnessHelper->newCappedRecordStore("local.oplog.rs", cappedMaxSize, -1));
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
 
@@ -186,12 +256,9 @@ TEST(RecordStoreTestHarness, OplogInsertOutOfOrder) {
     }
 }
 
-TEST(RecordStoreTestHarness, OplogHackOnNonOplog) {
+TEST(RecordStoreTestHarness, SeekNearOnNonOplog) {
     std::unique_ptr<RecordStoreHarnessHelper> harnessHelper = newRecordStoreHarnessHelper();
-    if (!harnessHelper->supportsDocLocking())
-        return;
-
-    std::unique_ptr<RecordStore> rs(harnessHelper->newNonCappedRecordStore("local.NOT_oplog.foo"));
+    std::unique_ptr<RecordStore> rs(harnessHelper->newRecordStore("local.NOT_oplog.foo"));
 
     ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
 
@@ -202,17 +269,46 @@ TEST(RecordStoreTestHarness, OplogHackOnNonOplog) {
                       .getStatus());
         wuow.commit();
     }
-    ASSERT_EQ(rs->oplogStartHack(opCtx.get(), RecordId(0, 1)), boost::none);
+    auto cur = rs->getCursor(opCtx.get());
+    auto rec = cur->seekNear(RecordId(0, 1));
+    ASSERT(rec);
+    // Regular record stores don't use timestamps for their RecordId, so expect the first
+    // auto-incrementing RecordId to be 1.
+    ASSERT_EQ(rec->id, RecordId(1));
 }
 
+/**
+ * Stringifies the current 'record', as well as any more records in the 'cursor'. Additionally adds
+ * the latest oplog visibitility timestamp (this is the current oplog read timestamp, but may not
+ * have been the timestamp used by the cursor).
+ */
+std::string stringifyForDebug(OperationContext* opCtx,
+                              boost::optional<Record> record,
+                              SeekableRecordCursor* cursor) {
+    str::stream output;
+
+    auto optOplogReadTimestampInt = opCtx->recoveryUnit()->getOplogVisibilityTs();
+    if (optOplogReadTimestampInt) {
+        output << "Latest oplog visibility timestamp: "
+               << Timestamp(optOplogReadTimestampInt.value());
+    }
+
+    if (record) {
+        output << ". Current record: " << record->id << ", " << record->data.toBson();
+        while (auto nextRecord = cursor->next()) {
+            if (nextRecord) {
+                output << ". Cursor Record: " << nextRecord->id << ", "
+                       << nextRecord->data.toBson();
+            }
+        }
+    }
+
+    return output;
+}
 
 TEST(RecordStoreTestHarness, OplogOrder) {
     std::unique_ptr<RecordStoreHarnessHelper> harnessHelper(newRecordStoreHarnessHelper());
-    if (!harnessHelper->supportsDocLocking())
-        return;
-
-    std::unique_ptr<RecordStore> rs(
-        harnessHelper->newCappedRecordStore("local.oplog.rs", 100000, -1));
+    std::unique_ptr<RecordStore> rs(harnessHelper->newOplogRecordStore());
 
     RecordId id1;
 
@@ -237,6 +333,15 @@ TEST(RecordStoreTestHarness, OplogOrder) {
         ASSERT(!cursor->next());
     }
 
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        auto cursor = rs->getCursor(opCtx.get());
+        auto record = cursor->seekNear(RecordId(id1.getLong() + 1));
+        ASSERT(record);
+        ASSERT_EQ(id1, record->id);
+        ASSERT(!cursor->next());
+    }
+
     {
         // now we insert 2 docs, but commit the 2nd one first.
         // we make sure we can't find the 2nd until the first is committed.
@@ -250,15 +355,16 @@ TEST(RecordStoreTestHarness, OplogOrder) {
         auto client1 = harnessHelper->serviceContext()->makeClient("c1");
         auto t1 = harnessHelper->newOperationContext(client1.get());
         WriteUnitOfWork w1(t1.get());
-        _oplogOrderInsertOplog(t1.get(), rs, 20);
+        RecordId id2 = _oplogOrderInsertOplog(t1.get(), rs, 20);
         // do not commit yet
 
+        RecordId id3;
         {  // create 2nd doc
             auto client2 = harnessHelper->serviceContext()->makeClient("c2");
             auto t2 = harnessHelper->newOperationContext(client2.get());
             {
                 WriteUnitOfWork w2(t2.get());
-                _oplogOrderInsertOplog(t2.get(), rs, 30);
+                id3 = _oplogOrderInsertOplog(t2.get(), rs, 30);
                 w2.commit();
             }
         }
@@ -271,8 +377,32 @@ TEST(RecordStoreTestHarness, OplogOrder) {
             auto opCtx = harnessHelper->newOperationContext(client2.get());
             auto cursor = rs->getCursor(opCtx.get());
             auto record = cursor->seekExact(id1);
-            ASSERT_EQ(id1, record->id);
-            ASSERT(!cursor->next());
+            ASSERT(record) << stringifyForDebug(opCtx.get(), record, cursor.get());
+            ASSERT_EQ(id1, record->id) << stringifyForDebug(opCtx.get(), record, cursor.get());
+            auto nextRecord = cursor->next();
+            ASSERT(!nextRecord) << stringifyForDebug(opCtx.get(), nextRecord, cursor.get());
+        }
+
+        {
+            auto client2 = harnessHelper->serviceContext()->makeClient("c2");
+            auto opCtx = harnessHelper->newOperationContext(client2.get());
+            auto cursor = rs->getCursor(opCtx.get());
+            auto record = cursor->seekNear(id2);
+            ASSERT(record) << stringifyForDebug(opCtx.get(), record, cursor.get());
+            ASSERT_EQ(id1, record->id) << stringifyForDebug(opCtx.get(), record, cursor.get());
+            auto nextRecord = cursor->next();
+            ASSERT(!nextRecord) << stringifyForDebug(opCtx.get(), nextRecord, cursor.get());
+        }
+
+        {
+            auto client2 = harnessHelper->serviceContext()->makeClient("c2");
+            auto opCtx = harnessHelper->newOperationContext(client2.get());
+            auto cursor = rs->getCursor(opCtx.get());
+            auto record = cursor->seekNear(id3);
+            ASSERT(record) << stringifyForDebug(opCtx.get(), record, cursor.get());
+            ASSERT_EQ(id1, record->id) << stringifyForDebug(opCtx.get(), record, cursor.get());
+            auto nextRecord = cursor->next();
+            ASSERT(!nextRecord) << stringifyForDebug(opCtx.get(), nextRecord, cursor.get());
         }
 
         w1.commit();
@@ -285,10 +415,13 @@ TEST(RecordStoreTestHarness, OplogOrder) {
         auto opCtx = harnessHelper->newOperationContext(client2.get());
         auto cursor = rs->getCursor(opCtx.get());
         auto record = cursor->seekExact(id1);
-        ASSERT_EQ(id1, record->id);
-        ASSERT(cursor->next());
-        ASSERT(cursor->next());
-        ASSERT(!cursor->next());
+        ASSERT_EQ(id1, record->id) << stringifyForDebug(opCtx.get(), record, cursor.get());
+        auto nextRecord = cursor->next();
+        ASSERT(nextRecord) << stringifyForDebug(opCtx.get(), nextRecord, cursor.get());
+        nextRecord = cursor->next();
+        ASSERT(nextRecord) << stringifyForDebug(opCtx.get(), nextRecord, cursor.get());
+        nextRecord = cursor->next();
+        ASSERT(!nextRecord) << stringifyForDebug(opCtx.get(), nextRecord, cursor.get());
     }
 
     // Rollback the last two oplog entries, then insert entries with older optimes and ensure that
@@ -296,7 +429,9 @@ TEST(RecordStoreTestHarness, OplogOrder) {
     {
         auto client2 = harnessHelper->serviceContext()->makeClient("c2");
         auto opCtx = harnessHelper->newOperationContext(client2.get());
-        rs->cappedTruncateAfter(opCtx.get(), id1, /*inclusive*/ false);
+        rs->cappedTruncateAfter(
+            opCtx.get(), id1, false /* inclusive */, nullptr /* aboutToDelete callback */);
+        harnessHelper->getEngine()->setStableTimestamp(Timestamp(id1.getLong()), true /* force */);
     }
 
     rs->waitForAllEarlierOplogWritesToBeVisible(harnessHelper->newOperationContext().get());
@@ -313,15 +448,17 @@ TEST(RecordStoreTestHarness, OplogOrder) {
         auto client1 = harnessHelper->serviceContext()->makeClient("c1");
         auto t1 = harnessHelper->newOperationContext(client1.get());
         WriteUnitOfWork w1(t1.get());
-        _oplogOrderInsertOplog(t1.get(), rs, 2);
+        RecordId id2 = _oplogOrderInsertOplog(t1.get(), rs, 2);
+
         // do not commit yet
 
+        RecordId id3;
         {  // create 2nd doc
             auto client2 = harnessHelper->serviceContext()->makeClient("c2");
             auto t2 = harnessHelper->newOperationContext(client2.get());
             {
                 WriteUnitOfWork w2(t2.get());
-                _oplogOrderInsertOplog(t2.get(), rs, 3);
+                id3 = _oplogOrderInsertOplog(t2.get(), rs, 3);
                 w2.commit();
             }
         }
@@ -334,8 +471,32 @@ TEST(RecordStoreTestHarness, OplogOrder) {
             auto opCtx = harnessHelper->newOperationContext(client2.get());
             auto cursor = rs->getCursor(opCtx.get());
             auto record = cursor->seekExact(id1);
-            ASSERT_EQ(id1, record->id);
-            ASSERT(!cursor->next());
+            ASSERT(record);
+            ASSERT_EQ(id1, record->id) << stringifyForDebug(opCtx.get(), record, cursor.get());
+            auto nextRecord = cursor->next();
+            ASSERT(!nextRecord) << stringifyForDebug(opCtx.get(), nextRecord, cursor.get());
+        }
+
+        {
+            auto client2 = harnessHelper->serviceContext()->makeClient("c2");
+            auto opCtx = harnessHelper->newOperationContext(client2.get());
+            auto cursor = rs->getCursor(opCtx.get());
+            auto record = cursor->seekNear(id2);
+            ASSERT(record);
+            ASSERT_EQ(id1, record->id) << stringifyForDebug(opCtx.get(), record, cursor.get());
+            auto nextRecord = cursor->next();
+            ASSERT(!nextRecord) << stringifyForDebug(opCtx.get(), nextRecord, cursor.get());
+        }
+
+        {
+            auto client2 = harnessHelper->serviceContext()->makeClient("c2");
+            auto opCtx = harnessHelper->newOperationContext(client2.get());
+            auto cursor = rs->getCursor(opCtx.get());
+            auto record = cursor->seekNear(id3);
+            ASSERT(record);
+            ASSERT_EQ(id1, record->id) << stringifyForDebug(opCtx.get(), record, cursor.get());
+            auto nextRecord = cursor->next();
+            ASSERT(!nextRecord) << stringifyForDebug(opCtx.get(), nextRecord, cursor.get());
         }
 
         w1.commit();
@@ -348,8 +509,60 @@ TEST(RecordStoreTestHarness, OplogOrder) {
         auto cursor = rs->getCursor(opCtx.get());
         auto record = cursor->seekExact(id1);
         ASSERT_EQ(id1, record->id);
-        ASSERT(cursor->next());
-        ASSERT(cursor->next());
+        auto nextRecord = cursor->next();
+        ASSERT(nextRecord) << stringifyForDebug(opCtx.get(), nextRecord, cursor.get());
+        nextRecord = cursor->next();
+        ASSERT(nextRecord) << stringifyForDebug(opCtx.get(), nextRecord, cursor.get());
+        nextRecord = cursor->next();
+        ASSERT(!nextRecord) << stringifyForDebug(opCtx.get(), nextRecord, cursor.get());
+    }
+}
+
+TEST(RecordStoreTestHarness, OplogVisibilityStandalone) {
+    std::unique_ptr<RecordStoreHarnessHelper> harnessHelper(
+        newRecordStoreHarnessHelper(RecordStoreHarnessHelper::Options::Standalone));
+    std::unique_ptr<RecordStore> rs(harnessHelper->newOplogRecordStore());
+
+    RecordId id1;
+
+    // insert a document
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        {
+            WriteUnitOfWork uow(opCtx.get());
+            // We must have a "ts" field with a timestamp.
+            Timestamp ts(5, 1);
+            BSONObj obj = BSON("ts" << ts);
+            // However, the insert is not timestamped in standalone mode.
+            StatusWith<RecordId> res =
+                rs->insertRecord(opCtx.get(), obj.objdata(), obj.objsize(), Timestamp());
+            ASSERT_OK(res.getStatus());
+            id1 = res.getValue();
+            StatusWith<RecordId> expectedId = record_id_helpers::keyForOptime(ts, KeyFormat::Long);
+            ASSERT_OK(expectedId.getStatus());
+            // RecordId should be extracted from 'ts' field when inserting into oplog namespace
+            ASSERT(expectedId.getValue().compare(id1) == 0);
+
+            uow.commit();
+        }
+    }
+
+    // verify that we can read it
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        auto cursor = rs->getCursor(opCtx.get());
+        auto record = cursor->seekExact(id1);
+        ASSERT(record);
+        ASSERT_EQ(id1, record->id);
+        ASSERT(!cursor->next());
+    }
+
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        auto cursor = rs->getCursor(opCtx.get());
+        auto record = cursor->seekNear(RecordId(id1.getLong() + 1));
+        ASSERT(record);
+        ASSERT_EQ(id1, record->id);
         ASSERT(!cursor->next());
     }
 }
