@@ -37,6 +37,7 @@
 #include "mongo/bson/timestamp.h"
 #include "mongo/db/repl/read_concern_level.h"
 #include "mongo/db/storage/snapshot.h"
+#include "mongo/util/decorable.h"
 
 namespace mongo {
 
@@ -109,11 +110,17 @@ public:
  * A RecoveryUnit is responsible for ensuring that data is persisted.
  * All on-disk information must be mutated through this interface.
  */
-class RecoveryUnit {
+class RecoveryUnit : public Decorable<RecoveryUnit> {
     RecoveryUnit(const RecoveryUnit&) = delete;
     RecoveryUnit& operator=(const RecoveryUnit&) = delete;
 
 public:
+    // Behavior for abandonSnapshot().
+    enum class AbandonSnapshotMode {
+        kAbort,  // default
+        kCommit
+    };
+
     void commitRegisteredChanges(boost::optional<Timestamp> commitTimestamp);
     void abortRegisteredChanges();
     virtual ~RecoveryUnit() {}
@@ -122,9 +129,12 @@ public:
      * Marks the beginning of a unit of work. Each call must be matched with exactly one call to
      * either commitUnitOfWork or abortUnitOfWork.
      *
+     * When called with readOnly=true, no unit of work is started. Calling commitUnitOfWork or
+     * abortUnitOfWork will invariant.
+     *
      * Should be called through WriteUnitOfWork rather than directly.
      */
-    virtual void beginUnitOfWork(OperationContext* opCtx) = 0;
+    void beginUnitOfWork(bool readOnly);
 
     /**
      * Marks the end of a unit of work and commits all changes registered by calls to onCommit or
@@ -132,10 +142,7 @@ public:
      *
      * Should be called through WriteUnitOfWork rather than directly.
      */
-    void commitUnitOfWork() {
-        assignNextSnapshotId();
-        doCommitUnitOfWork();
-    }
+    void commitUnitOfWork();
 
     /**
      * Marks the end of a unit of work and rolls back all changes registered by calls to onRollback
@@ -144,10 +151,14 @@ public:
      *
      * Should be called through WriteUnitOfWork rather than directly.
      */
-    void abortUnitOfWork() {
-        assignNextSnapshotId();
-        doAbortUnitOfWork();
-    }
+    void abortUnitOfWork();
+
+    /**
+     * Cleans up any state set for this unit of work.
+     *
+     * Should be called through WriteUnitOfWork rather than directly.
+     */
+    void endReadOnlyUnitOfWork();
 
     /**
      * Transitions the active unit of work to the "prepared" state. Must be called after
@@ -192,7 +203,7 @@ public:
     /**
      * Waits until all commits that happened before this call are durable in the journal. Returns
      * true, unless the storage engine cannot guarantee durability, which should never happen when
-     * isDurable() returned true. This cannot be called from inside a unit of work, and should
+     * the engine is non-ephemeral. This cannot be called from inside a unit of work, and should
      * fail if it is. This method invariants if the caller holds any locks, except for repair.
      *
      * Can throw write interruption errors from the JournalListener.
@@ -213,12 +224,24 @@ public:
     }
 
     /**
-     * If there is an open transaction, it is closed. On return no transaction is active. This
-     * cannot be called inside of a WriteUnitOfWork, and should fail if it is.
+     * If there is an open transaction, it is closed. If the current AbandonSnapshotMode is
+     * 'kAbort', the transaction is aborted. If the mode is 'kCommit' the transaction is committed,
+     * and all data currently pointed to by cursors remains pinned until the cursors are
+     * repositioned.
+
+     * On return no transaction is active. It is a programming error to call this inside of a
+     * WriteUnitOfWork, even if the AbandonSnapshotMode is 'kCommit'.
      */
     void abandonSnapshot() {
-        assignNextSnapshotId();
         doAbandonSnapshot();
+        assignNextSnapshotId();
+    }
+
+    void setAbandonSnapshotMode(AbandonSnapshotMode mode) {
+        _abandonSnapshotMode = mode;
+    }
+    AbandonSnapshotMode abandonSnapshotMode() const {
+        return _abandonSnapshotMode;
     }
 
     /**
@@ -230,20 +253,27 @@ public:
     virtual void preallocateSnapshot() {}
 
     /**
-     * Obtains a majority committed snapshot. Snapshots should still be separately acquired and
-     * newer committed snapshots should be used if available whenever implementations would normally
-     * change snapshots.
+     * Like preallocateSnapshot() above but also indicates that the snapshot will be used for
+     * reading the oplog.
      *
-     * If no snapshot has yet been marked as Majority Committed, returns a status with error code
+     * StorageEngines may not implement this in which case it works like preallocateSnapshot.
+     */
+    virtual void preallocateSnapshotForOplogRead() {
+        preallocateSnapshot();
+    }
+
+    /**
+     * Returns whether or not a majority commmitted snapshot is available. If no snapshot has yet
+     * been marked as Majority Committed, returns a status with error code
      * ReadConcernMajorityNotAvailableYet. After this returns successfully, at any point where
      * implementations attempt to acquire committed snapshot, if there are none available due to a
-     * call to SnapshotManager::dropAllSnapshots(), a AssertionException with the same code should
-     * be thrown.
+     * call to SnapshotManager::clearCommittedSnapshot(), a AssertionException with the same code
+     * should be thrown.
      *
      * StorageEngines that don't support a SnapshotManager should use the default
      * implementation.
      */
-    virtual Status obtainMajorityCommittedSnapshot() {
+    virtual Status majorityCommittedSnapshotAvailable() const {
         return {ErrorCodes::CommandNotSupported,
                 "Current storage engine does not support majority readConcerns"};
     }
@@ -255,14 +285,14 @@ public:
      *  - when using ReadSource::kNoOverlap, the timestamp chosen by the storage engine.
      *  - when using ReadSource::kAllDurableSnapshot, the timestamp chosen using the storage
      * engine's all_durable timestamp.
-     * applied timestamp. Can return boost::none if no timestamp has been established.
+     *  - when using ReadSource::kLastAppplied, the last applied timestamp. Can return boost::none
+     * if no timestamp has been established.
      *  - when using ReadSource::kMajorityCommitted, the majority committed timestamp chosen by the
-     * storage engine after a transaction has been opened or after a call to
-     * obtainMajorityCommittedSnapshot().
+     * storage engine after a transaction has been opened.
      *
      * This may passively start a storage engine transaction to establish a read timestamp.
      */
-    virtual boost::optional<Timestamp> getPointInTimeReadTimestamp() {
+    virtual boost::optional<Timestamp> getPointInTimeReadTimestamp(OperationContext* opCtx) {
         return boost::none;
     }
 
@@ -379,6 +409,14 @@ public:
         return {};
     }
 
+    /**
+     * MongoDB must update documents with non-decreasing timestamp values. A storage engine is
+     * allowed to assert when this contract is violated. An untimestamped write is a subset of these
+     * violations, which may be necessary in limited circumstances. This API can be called before a
+     * transaction begins to suppress this subset of errors.
+     */
+    virtual void allowUntimestampedWrite() {}
+
     /**
      * Fetches the storage level statistics.
      */
@@ -387,8 +425,7 @@ public:
     }
 
     /**
-     * Refreshes a read transaction by starting a new one at the same read timestamp and then ending
-     * the current one.
+     * Refreshes a read transaction by resetting the snapshot in use
      */
     virtual void refreshSnapshot() {}
 
@@ -402,7 +439,7 @@ public:
          */
         kNoTimestamp,
         /**
-         * Read from the majority all-commmitted timestamp.
+         * Read from the majority all-committed timestamp.
          */
         kMajorityCommitted,
         /**
@@ -458,6 +495,10 @@ public:
         return ReadSource::kNoTimestamp;
     };
 
+    virtual boost::optional<int64_t> getOplogVisibilityTs() {
+        return boost::none;
+    }
+
     /**
      * Pinning informs callers not to change the ReadSource on this RecoveryUnit. Callers are
      * expected to first check isReadSourcePinned before attempting to change the ReadSource. An
@@ -483,10 +524,21 @@ public:
     };
 
     /**
-     * Indicates whether a unit of work is active. Will be true after beginUnitOfWork
-     * is called and before either commitUnitOfWork or abortUnitOfWork gets called.
+     * Indicates whether the RecoveryUnit has an open snapshot. A snapshot can be opened inside or
+     * outside of a WriteUnitOfWork.
      */
-    virtual bool inActiveTxn() const = 0;
+    virtual bool isActive() const {
+        return _isActive();
+    };
+
+    /**
+     * When called, the WriteUnitOfWork ignores the multi timestamp constraint for the remainder of
+     * the WriteUnitOfWork, where if within a WriteUnitOfWork multiple timestamps are set, the first
+     * timestamp must be set prior to any writes.
+     *
+     * Must be reset when the WriteUnitOfWork is either committed or rolled back.
+     */
+    virtual void ignoreAllMultiTimestampConstraints(){};
 
     /**
      * Registers a callback to be called prior to a WriteUnitOfWork committing the storage
@@ -529,7 +581,51 @@ public:
      * The registerChange() method may only be called when a WriteUnitOfWork is active, and
      * may not be called during commit or rollback.
      */
-    virtual void registerChange(std::unique_ptr<Change> change);
+    void registerChange(std::unique_ptr<Change> change);
+
+    /**
+     * Registers a change with the given rollback and commit functions.
+     *
+     * Be careful about the lifetimes of all variables captured by the callback!
+     */
+    template <typename RollbackCallback, typename CommitCallback>
+    void registerChange(CommitCallback commit, RollbackCallback rollback) {
+        class CallbackChange final : public Change {
+        public:
+            CallbackChange(CommitCallback&& commit, RollbackCallback&& rollback)
+                : _rollback(std::move(rollback)), _commit(std::move(commit)) {}
+            void rollback() final {
+                _rollback();
+            }
+            void commit(boost::optional<Timestamp> ts) final {
+                _commit(ts);
+            }
+
+        private:
+            RollbackCallback _rollback;
+            CommitCallback _commit;
+        };
+
+        registerChange(std::make_unique<CallbackChange>(std::move(commit), std::move(rollback)));
+    }
+
+    /**
+     * Like registerChange() above but should only be used to make new state visible in the
+     * in-memory catalog. Only one change of this kind may be registered at a given time to ensure
+     * catalog updates are atomic. Change registered with this function will commit after the commit
+     * changes registered with registerChange and rollback will run before the rollback changes
+     * registered with registerChange.
+     *
+     * This separation ensures that regular Changes that can modify state are run before the Change
+     * to install the new state in the in-memory catalog, after which there should be no further
+     * changes.
+     */
+    void registerChangeForCatalogVisibility(std::unique_ptr<Change> change);
+
+    /**
+     * Returns true if a change has been registered with registerChangeForCatalogVisibility() above.
+     */
+    bool hasRegisteredChangeForCatalogVisibility();
 
     /**
      * Registers a callback to be called if the current WriteUnitOfWork rolls back.
@@ -623,7 +719,7 @@ public:
         kCommitting,
     };
 
-    std::string toString(State state) const {
+    static std::string toString(State state) {
         switch (state) {
             case State::kInactive:
                 return "Inactive";
@@ -641,8 +737,11 @@ public:
         MONGO_UNREACHABLE;
     }
 
-    void setMustBeTimestamped() {
-        _mustBeTimestamped = true;
+    /**
+     * Exposed for debugging purposes.
+     */
+    State getState() {
+        return _getState();
     }
 
     void setNoEvictionAfterRollback() {
@@ -653,6 +752,13 @@ public:
         return _noEvictionAfterRollback;
     }
 
+    /**
+     * Returns true if this is an instance of RecoveryUnitNoop.
+     */
+    virtual bool isNoop() const {
+        return false;
+    }
+
 protected:
     RecoveryUnit();
 
@@ -691,24 +797,39 @@ protected:
         return State::kCommitting == _state || State::kAborting == _state;
     }
 
-    bool _mustBeTimestamped = false;
+    /**
+     * Executes all registered commit handlers and clears all registered changes
+     */
+    void _executeCommitHandlers(boost::optional<Timestamp> commitTimestamp);
+
+    /**
+     * Executes all registered rollback handlers and clears all registered changes
+     */
+    void _executeRollbackHandlers();
 
     bool _noEvictionAfterRollback = false;
 
+    AbandonSnapshotMode _abandonSnapshotMode = AbandonSnapshotMode::kAbort;
+
 private:
     // Sets the snapshot associated with this RecoveryUnit to a new globally unique id number.
     void assignNextSnapshotId();
 
+    virtual void doBeginUnitOfWork() = 0;
     virtual void doAbandonSnapshot() = 0;
     virtual void doCommitUnitOfWork() = 0;
     virtual void doAbortUnitOfWork() = 0;
 
+    virtual void validateInUnitOfWork() const;
+
     std::vector<std::function<void(OperationContext*)>> _preCommitHooks;
 
     typedef std::vector<std::unique_ptr<Change>> Changes;
     Changes _changes;
+    std::unique_ptr<Change> _changeForCatalogVisibility;
     State _state = State::kInactive;
     uint64_t _mySnapshotId;
+    bool _readOnly = false;
 };
 
 /**
