@@ -29,6 +29,7 @@
 
 #pragma once
 
+#include <memory>
 #include <string>
 #include <vector>
 
@@ -36,12 +37,15 @@
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/timestamp.h"
 #include "mongo/db/catalog/index_builds.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/resumable_index_builds_gen.h"
 #include "mongo/db/storage/temporary_record_store.h"
 #include "mongo/util/functional.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
 
+class BackupBlock;
 class JournalListener;
 class DurableCatalog;
 class KVEngine;
@@ -63,9 +67,9 @@ public:
     /**
      * This is the minimum valid timestamp; it can be used for reads that need to see all
      * untimestamped data but no timestamped data. We cannot use 0 here because 0 means see all
-     * timestamped data.
+     * timestamped data. The high-order 4 bytes are for the seconds field in a Timestamp object.
      */
-    static const uint64_t kMinimumTimestamp = 1;
+    static const unsigned long long kMinimumTimestamp = 1ULL << 32;
 
     /**
      * When the storage engine needs to know how much oplog to preserve for the sake of active
@@ -76,34 +80,13 @@ public:
     using OldestActiveTransactionTimestampCallback =
         std::function<OldestActiveTransactionTimestampResult(Timestamp stableTimestamp)>;
 
-    struct BackupOptions {
-        bool disableIncrementalBackup = false;
-        bool incrementalBackup = false;
-        int blockSizeMB = 16;
-        boost::optional<std::string> thisBackupName;
-        boost::optional<std::string> srcBackupName;
-    };
-
-    struct BackupBlock {
-        std::uint64_t offset = 0;
-        std::uint64_t length = 0;
-    };
+    using DropIdentCallback = std::function<void()>;
 
     /**
-     * Contains the size of the file to be backed up. This allows the backup application to safely
-     * truncate the file for incremental backups. Files that have had changes since the last
-     * incremental backup will have their changed file blocks listed.
+     * Information on last storage engine shutdown state that is relevant to the recovery process.
+     * Determined by initializeStorageEngine() during mongod.lock initialization.
      */
-    struct BackupFile {
-        BackupFile() = delete;
-        explicit BackupFile(std::uint64_t fileSize) : fileSize(fileSize){};
-
-        std::uint64_t fileSize;
-        std::vector<BackupBlock> blocksToCopy;
-    };
-
-    // Map of filenames to backup file information.
-    using BackupInformation = stdx::unordered_map<std::string, BackupFile>;
+    enum class LastShutdownState { kClean, kUnclean };
 
     /**
      * The interface for creating new instances of storage engines.
@@ -117,11 +100,12 @@ public:
         virtual ~Factory() {}
 
         /**
-         * Return a new instance of the StorageEngine. The lockFile parameter may be null if
-         * params.readOnly is set. Caller owns the returned pointer.
+         * Return a new instance of the StorageEngine. Caller owns the returned pointer.
          */
-        virtual StorageEngine* create(const StorageGlobalParams& params,
-                                      const StorageEngineLockFile* lockFile) const = 0;
+        virtual std::unique_ptr<StorageEngine> create(
+            OperationContext* opCtx,
+            const StorageGlobalParams& params,
+            const StorageEngineLockFile* lockFile) const = 0;
 
         /**
          * Returns the name of the storage engine.
@@ -176,13 +160,11 @@ public:
         virtual BSONObj createMetadataOptions(const StorageGlobalParams& params) const = 0;
 
         /**
-         * Returns whether the engine supports read-only mode. If read-only mode is enabled, the
-         * engine may be started on a read-only filesystem (either mounted read-only or with
-         * read-only permissions). If readOnly mode is enabled, it is undefined behavior to call
-         * methods that write data (e.g. insertRecord). This method is provided on the Factory
-         * because it must be called before the storageEngine is instantiated.
+         * Returns whether the engine supports queryable backup mode. If queryable backup mode is
+         * enabled, user writes are not permitted but internally generated writes are still
+         * permitted.
          */
-        virtual bool supportsReadOnly() const {
+        virtual bool supportsQueryableBackupMode() const {
             return false;
         }
     };
@@ -192,13 +174,6 @@ public:
      */
     virtual ~StorageEngine() {}
 
-    /**
-     * Called after the globalStorageEngine pointer has been set up, before any other methods
-     * are called. Any initialization work that requires the ability to create OperationContexts
-     * should be done here rather than in the constructor.
-     */
-    virtual void finishInit() = 0;
-
     /**
      * During the startup process, the storage engine is one of the first components to be started
      * up and fully initialized. But that fully initialized storage engine may not be recognized as
@@ -224,37 +199,13 @@ public:
     /**
      * List the databases stored in this storage engine.
      */
-    virtual std::vector<std::string> listDatabases() const = 0;
-
-    /**
-     * Returns whether the storage engine supports its own locking locking below the collection
-     * level. If the engine returns true, MongoDB will acquire intent locks down to the
-     * collection level and will assume that the engine will ensure consistency at the level of
-     * documents. If false, MongoDB will lock the entire collection in Shared/Exclusive mode
-     * for read/write operations respectively.
-     */
-    virtual bool supportsDocLocking() const = 0;
-
-    /**
-     * Returns whether the storage engine supports locking at a database level.
-     */
-    virtual bool supportsDBLocking() const = 0;
+    virtual std::vector<DatabaseName> listDatabases() const = 0;
 
     /**
      * Returns whether the storage engine supports capped collections.
      */
     virtual bool supportsCappedCollections() const = 0;
 
-    /**
-     * Returns whether the storage engine supports checkpoints.
-     */
-    virtual bool supportsCheckpoints() const = 0;
-
-    /**
-     * Returns whether the engine supports a journalling concept or not.
-     */
-    virtual bool isDurable() const = 0;
-
     /**
      * Returns true if the engine does not persist data to disk; false otherwise.
      */
@@ -265,19 +216,23 @@ public:
      * engines that support recoverToStableTimestamp().
      *
      * Must be called with the global lock acquired in exclusive mode.
+     *
+     * Unrecognized idents require special handling based on the context known only to the
+     * caller. For example, on starting from a previous unclean shutdown, we may try to recover
+     * orphaned idents, which are known to the storage engine but not referenced in the catalog.
      */
-    virtual void loadCatalog(OperationContext* opCtx) = 0;
+    virtual void loadCatalog(OperationContext* opCtx, LastShutdownState lastShutdownState) = 0;
     virtual void closeCatalog(OperationContext* opCtx) = 0;
 
     /**
      * Closes all file handles associated with a database.
      */
-    virtual Status closeDatabase(OperationContext* opCtx, StringData db) = 0;
+    virtual Status closeDatabase(OperationContext* opCtx, const DatabaseName& dbName) = 0;
 
     /**
      * Deletes all data and metadata for a database.
      */
-    virtual Status dropDatabase(OperationContext* opCtx, StringData db) = 0;
+    virtual Status dropDatabase(OperationContext* opCtx, const DatabaseName& dbName) = 0;
 
     /**
      * Checkpoints the data to disk.
@@ -327,26 +282,56 @@ public:
     virtual Status disableIncrementalBackup(OperationContext* opCtx) = 0;
 
     /**
-     * When performing an incremental backup, we first need a basis for future incremental backups.
-     * The basis will be a full backup called 'thisBackupName'. For future incremental backups, the
-     * storage engine will take a backup called 'thisBackupName' which will contain the changes made
-     * to data files since the backup named 'srcBackupName'.
+     * Represents the options that the storage engine can use during full and incremental backups.
      *
-     * The storage engine must use an upper bound limit of 'blockSizeMB' when returning changed
-     * file blocks.
+     * When performing a full backup where incrementalBackup=false, the values of 'blockSizeMB',
+     * 'thisBackupName', and 'srcBackupName' should not be modified.
      *
-     * The first full backup meant for incremental and future incremental backups must pass
-     * 'incrementalBackup' as true.
-     * 'thisBackupName' must exist only if 'incrementalBackup' is true.
-     * 'srcBackupName' must not exist when 'incrementalBackup' is false but may or may not exist
-     * when 'incrementalBackup' is true.
+     * When performing an incremental backup where incrementalBackup=true, we first need a basis for
+     * future incremental backups. This first basis (named 'thisBackupName'), which is a full
+     * backup, must pass incrementalBackup=true and should not set 'srcBackupName'. An incremental
+     * backup will include changed blocks since 'srcBackupName' was taken. This backup (also named
+     * 'thisBackupName') will then become the basis for future incremental backups.
+     *
+     * Note that 'thisBackupName' must exist if and only if incrementalBackup=true while
+     * 'srcBackupName' must not exist if incrementalBackup=false but may or may not exist if
+     * incrementalBackup=true.
      */
-    virtual StatusWith<StorageEngine::BackupInformation> beginNonBlockingBackup(
-        OperationContext* opCtx, const BackupOptions& options) = 0;
+    struct BackupOptions {
+        bool disableIncrementalBackup = false;
+        bool incrementalBackup = false;
+        int blockSizeMB = 16;
+        boost::optional<std::string> thisBackupName;
+        boost::optional<std::string> srcBackupName;
+    };
+
+    /**
+     * Abstract class required for streaming both full and incremental backups. The function
+     * getNextBatch() returns a vector containing 'batchSize' or less BackupBlocks. The
+     * StreamingCursor has been exhausted if getNextBatch() returns an empty vector.
+     */
+    class StreamingCursor {
+    public:
+        StreamingCursor() = delete;
+        explicit StreamingCursor(BackupOptions options) : options(options){};
+
+        virtual ~StreamingCursor() = default;
+
+        virtual StatusWith<std::deque<BackupBlock>> getNextBatch(OperationContext* opCtx,
+                                                                 std::size_t batchSize) = 0;
+
+    protected:
+        BackupOptions options;
+    };
+
+    virtual StatusWith<std::unique_ptr<StreamingCursor>> beginNonBlockingBackup(
+        OperationContext* opCtx,
+        boost::optional<Timestamp> checkpointTimestamp,
+        const BackupOptions& options) = 0;
 
     virtual void endNonBlockingBackup(OperationContext* opCtx) = 0;
 
-    virtual StatusWith<std::vector<std::string>> extendBackupCursor(OperationContext* opCtx) = 0;
+    virtual StatusWith<std::deque<std::string>> extendBackupCursor(OperationContext* opCtx) = 0;
 
     /**
      * Recover as much data as possible from a potentially corrupt RecordStore.
@@ -360,13 +345,27 @@ public:
                                      const NamespaceString& nss) = 0;
 
     /**
-     * Creates a temporary RecordStore on the storage engine. This record store will drop itself
-     * automatically when it goes out of scope. This means the TemporaryRecordStore should not exist
-     * any longer than the OperationContext used to create it. On startup, the storage engine will
-     * drop any un-dropped temporary record stores.
+     * Creates a temporary RecordStore on the storage engine. On startup after an unclean shutdown,
+     * the storage engine will drop any un-dropped temporary record stores.
      */
-    virtual std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStore(
-        OperationContext* opCtx) = 0;
+    virtual std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStore(OperationContext* opCtx,
+                                                                           KeyFormat keyFormat) = 0;
+
+    /**
+     * Creates a temporary RecordStore on the storage engine for a resumable index build. On
+     * startup after an unclean shutdown, the storage engine will drop any un-dropped temporary
+     * record stores.
+     */
+    virtual std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStoreForResumableIndexBuild(
+        OperationContext* opCtx, KeyFormat keyFormat) = 0;
+
+    /**
+     * Creates a temporary RecordStore on the storage engine from an existing ident on disk. On
+     * startup after an unclean shutdown, the storage engine will drop any un-dropped temporary
+     * record stores.
+     */
+    virtual std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStoreFromExistingIdent(
+        OperationContext* opCtx, StringData ident) = 0;
 
     /**
      * This method will be called before there is a clean shutdown.  Storage engines should
@@ -420,6 +419,8 @@ public:
      */
     virtual bool supportsOplogStones() const = 0;
 
+    virtual bool supportsResumableIndexBuilds() const = 0;
+
     /**
      * Returns true if the storage engine supports deferring collection drops until the the storage
      * engine determines that the storage layer artifacts for the pending drops are no longer needed
@@ -439,9 +440,25 @@ public:
     virtual void clearDropPendingState() = 0;
 
     /**
-     * Returns true if the storage engine supports two phase index builds.
+     * Adds 'ident' to a list of indexes/collections whose data will be dropped when:
+     * - the dropTimestamp' is sufficiently old to ensure no future data accesses
+     * - and no holders of 'ident' remain (the index/collection is no longer in active use)
      */
-    virtual bool supportsTwoPhaseIndexBuild() const = 0;
+    virtual void addDropPendingIdent(const Timestamp& dropTimestamp,
+                                     std::shared_ptr<Ident> ident,
+                                     DropIdentCallback&& onDrop = nullptr) = 0;
+
+    /**
+     * Starts the timestamp monitor. This periodically drops idents queued by addDropPendingIdent,
+     * and removes historical ident entries no longer necessary.
+     */
+    virtual void startTimestampMonitor() = 0;
+
+    /**
+     * Called when the checkpoint thread instructs the storage engine to take a checkpoint. The
+     * underlying storage engine must take a checkpoint at this point.
+     */
+    virtual void checkpoint() = 0;
 
     /**
      * Recovers the storage engine state to the last stable timestamp. "Stable" in this case
@@ -501,14 +518,14 @@ public:
     /**
      * Returns the initial data timestamp.
      */
-    virtual Timestamp getInitialDataTimestamp() = 0;
+    virtual Timestamp getInitialDataTimestamp() const = 0;
 
     /**
      * Uses the current stable timestamp to set the oldest timestamp for which the storage engine
      * must maintain snapshot history through.
      *
      * oldest_timestamp will be set to stable_timestamp adjusted by
-     * 'targetSnapshotHistoryWindowInSeconds' to create a window of available snapshots on the
+     * 'minSnapshotHistoryWindowInSeconds' to create a window of available snapshots on the
      * storage engine from oldest to stable. Furthermore, oldest_timestamp will never be set ahead
      * of the oplog read timestamp, ensuring the oplog reader's 'read_timestamp' can always be
      * serviced.
@@ -535,21 +552,6 @@ public:
     virtual void setOldestActiveTransactionTimestampCallback(
         OldestActiveTransactionTimestampCallback callback) = 0;
 
-    /**
-     * Indicates whether the storage engine cache is under pressure.
-     *
-     * Retrieves a cache pressure value in the range [0, 100] from the storage engine, and compares
-     * it against storageGlobalParams.cachePressureThreshold, a dynamic server parameter, to
-     * determine whether cache pressure is too high.
-     */
-    virtual bool isCacheUnderPressure(OperationContext* opCtx) const = 0;
-
-    /**
-     * For unit tests only. Sets the cache pressure value with which isCacheUnderPressure()
-     * evalutates to 'pressure'.
-     */
-    virtual void setCachePressureForTest(int pressure) = 0;
-
     struct IndexIdentifier {
         const RecordId catalogId;
         const NamespaceString nss;
@@ -568,18 +570,27 @@ public:
         // not to completion; they will wait for replicated commit or abort operations. This is a
         // mapping from index build UUID to index build.
         IndexBuilds indexBuildsToRestart;
+
+        // List of index builds to be resumed. Each ResumeIndexInfo may contain multiple indexes to
+        // resume as part of the same build.
+        std::vector<ResumeIndexInfo> indexBuildsToResume;
     };
 
     /**
      * Drop abandoned idents. If successful, returns a ReconcileResult with indexes that need to be
      * rebuilt or builds that need to be restarted.
-     * */
-    virtual StatusWith<ReconcileResult> reconcileCatalogAndIdents(OperationContext* opCtx) = 0;
+     *
+     * Abandoned internal idents require special handling based on the context known only to the
+     * caller. For example, on starting from a previous unclean shutdown, we would always drop all
+     * unknown internal idents. If we started from a clean shutdown, the internal idents may contain
+     * information for resuming index builds.
+     */
+    virtual StatusWith<ReconcileResult> reconcileCatalogAndIdents(
+        OperationContext* opCtx, LastShutdownState lastShutdownState) = 0;
 
     /**
      * Returns the all_durable timestamp. All transactions with timestamps earlier than the
-     * all_durable timestamp are committed. Only storage engines that support document level locking
-     * must provide an implementation. Other storage engines may provide a no-op implementation.
+     * all_durable timestamp are committed.
      *
      * The all_durable timestamp is the in-memory no holes point. That does not mean that there are
      * no holes behind it on disk. The all_durable timestamp also might not correspond with any
@@ -594,13 +605,6 @@ public:
      */
     virtual Timestamp getAllDurableTimestamp() const = 0;
 
-    /**
-     * Returns the oldest read timestamp in use by an open transaction. Storage engines that support
-     * the 'snapshot' ReadConcern must provide an implementation. Other storage engines may provide
-     * a no-op implementation.
-     */
-    virtual Timestamp getOldestOpenReadTimestamp() const = 0;
-
     /**
      * Returns the minimum possible Timestamp value in the oplog that replication may need for
      * recovery in the event of a crash.
@@ -612,29 +616,43 @@ public:
     /**
      * Returns the path to the directory which has the data files of database with `dbName`.
      */
-    virtual std::string getFilesystemPathForDb(const std::string& dbName) const = 0;
+    virtual std::string getFilesystemPathForDb(const DatabaseName& dbName) const = 0;
 
-    /**
-     * Returns whethers the data files are compatible with the current code:
-     *
-     *   - Status::OK() if the data files are compatible with the current code.
-     *
-     *   - ErrorCodes::CanRepairToDowngrade if the data files are incompatible with the current
-     *     code, but a --repair would make them compatible. For example, when rebuilding all indexes
-     *     in the data files would resolve the incompatibility.
-     *
-     *   - ErrorCodes::MustUpgrade if the data files are incompatible with the current code and a
-     *     newer version is required to start up.
-     */
-    virtual Status currentFilesCompatible(OperationContext* opCtx) const = 0;
+    virtual int64_t sizeOnDiskForDb(OperationContext* opCtx, const DatabaseName& dbName) = 0;
 
-    virtual int64_t sizeOnDiskForDb(OperationContext* opCtx, StringData dbName) = 0;
+    virtual bool isUsingDirectoryPerDb() const = 0;
+
+    virtual bool isUsingDirectoryForIndexes() const = 0;
 
     virtual KVEngine* getEngine() = 0;
     virtual const KVEngine* getEngine() const = 0;
     virtual DurableCatalog* getCatalog() = 0;
     virtual const DurableCatalog* getCatalog() const = 0;
 
+    /**
+     * A service that would like to pin the oldest timestamp registers its request here. If the
+     * request can be satisfied, OK is returned with the oldest timestamp the caller can
+     * use. Otherwise, SnapshotTooOld is returned. See the following enumerations:
+     *
+     * | Timestamp Relation  | roundUpIfTooOld | Result                    |
+     * |---------------------+-----------------+---------------------------|
+     * | requested >= oldest | false/true      | <OK, requested timestamp> |
+     * | requested < oldest  | false           | <SnapshotTooOld>          |
+     * | requested < oldest  | true            | <OK, oldest timestamp>    |
+     *
+     * If the input OperationContext is in a WriteUnitOfWork, an `onRollback` handler will be
+     * registered to return the pin for the `requestingServiceName` to the previous value.
+     */
+    virtual StatusWith<Timestamp> pinOldestTimestamp(OperationContext* opCtx,
+                                                     const std::string& requestingServiceName,
+                                                     Timestamp requestedTimestamp,
+                                                     bool roundUpIfTooOld) = 0;
+
+    /**
+     * Unpins the request registered under `requestingServiceName`.
+     */
+    virtual void unpinOldestTimestamp(const std::string& requestingServiceName) = 0;
+
     /**
      * Prevents oplog history at 'pinnedTimestamp' and later from being truncated. Setting
      * Timestamp::max() effectively nullifies the pin because no oplog truncation will be stopped by
@@ -643,12 +661,9 @@ public:
     virtual void setPinnedOplogTimestamp(const Timestamp& pinnedTimestamp) = 0;
 
     /**
-     * Returns the input storage engine options, sanitized to remove options that may not apply to
-     * this node, such as encryption. Might be called for both collection and index options. See
-     * SERVER-68122.
+     * Instructs the storage engine to dump its internal state.
      */
-    virtual BSONObj getSanitizedStorageOptionsForSecondaryReplication(
-        const BSONObj& options) const = 0;
+    virtual void dump() const = 0;
 };
 
 }  // namespace mongo
