@@ -40,12 +40,15 @@
 #include "mongo/bson/ordering.h"
 #include "mongo/bson/timestamp.h"
 #include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/storage/backup_block.h"
+#include "mongo/db/storage/durable_catalog.h"
 #include "mongo/db/storage/kv/kv_engine.h"
 #include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_oplog_manager.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
 #include "mongo/platform/mutex.h"
+#include "mongo/util/concurrency/with_lock.h"
 #include "mongo/util/elapsed_tracker.h"
 
 namespace mongo {
@@ -58,16 +61,42 @@ class WiredTigerSizeStorer;
 class WiredTigerEngineRuntimeConfigParameter;
 
 struct WiredTigerFileVersion {
-    // MongoDB 4.4+ will not open on datafiles left behind by 4.0 and earlier. MongoDB 4.4
+    // MongoDB 4.4+ will not open on datafiles left behind by 4.2.5 and earlier. MongoDB 4.4
     // shutting down in FCV 4.2 will leave data files that 4.2.6+ will understand
     // (IS_44_FCV_42). MongoDB 4.2.x always writes out IS_42.
     enum class StartupVersion { IS_42, IS_44_FCV_42, IS_44_FCV_44 };
 
+    inline static const std::string kLastLTSWTRelease = "compatibility=(release=10.0)";
+    inline static const std::string kLastContinuousWTRelease = "compatibility=(release=10.0)";
+    inline static const std::string kLatestWTRelease = "compatibility=(release=10.0)";
+
     StartupVersion _startupVersion;
-    bool shouldDowngrade(bool readOnly, bool repairMode, bool hasRecoveryTimestamp);
+    bool shouldDowngrade(bool hasRecoveryTimestamp);
     std::string getDowngradeString();
 };
 
+struct WiredTigerBackup {
+    WT_CURSOR* cursor = nullptr;
+    WT_CURSOR* dupCursor = nullptr;
+    std::set<std::string> logFilePathsSeenByExtendBackupCursor;
+    std::set<std::string> logFilePathsSeenByGetNextBatch;
+    BackupBlock::IdentToNamespaceAndUUIDMap identToNamespaceAndUUIDMap;
+
+    // 'wtBackupCursorMutex' provides concurrency control between beginNonBlockingBackup(),
+    // endNonBlockingBackup(), and getNextBatch() because we stream the output of the backup cursor.
+    Mutex wtBackupCursorMutex = MONGO_MAKE_LATCH("WiredTigerKVEngine::wtBackupCursorMutex");
+
+    // 'wtBackupDupCursorMutex' provides concurrency control between getNextBatch() and
+    // extendBackupCursor() because WiredTiger only allows one duplicate cursor to be open at a
+    // time. extendBackupCursor() blocks on condition variable 'wtBackupDupCursorCV' if a duplicate
+    // cursor is already open.
+    Mutex wtBackupDupCursorMutex = MONGO_MAKE_LATCH("WiredTigerKVEngine::wtBackupDupCursorMutex");
+    stdx::condition_variable wtBackupDupCursorCV;
+
+    // This file flags there was an ongoing backup when an unclean shutdown happened.
+    inline static const std::string kOngoingBackupFile = "ongoingBackup.lock";
+};
+
 class WiredTigerKVEngine final : public KVEngine {
 public:
     static StringData kTableUriPrefix;
@@ -78,34 +107,19 @@ public:
                        const std::string& extraOpenOptions,
                        size_t cacheSizeMB,
                        size_t maxHistoryFileSizeMB,
-                       bool durable,
                        bool ephemeral,
-                       bool repair,
-                       bool readOnly);
+                       bool repair);
 
     ~WiredTigerKVEngine();
 
-    void startAsyncThreads() override;
-
     void notifyStartupComplete() override;
 
     void setRecordStoreExtraOptions(const std::string& options);
     void setSortedDataInterfaceExtraOptions(const std::string& options);
 
-    bool supportsDocLocking() const override;
-
     bool supportsDirectoryPerDB() const override;
 
-    /**
-     * WiredTiger supports checkpoints when it isn't running in memory.
-     */
-    bool supportsCheckpoints() const override {
-        return !isEphemeral();
-    }
-
-    bool isDurable() const override {
-        return _durable;
-    }
+    void checkpoint() override;
 
     bool isEphemeral() const override {
         return _ephemeral;
@@ -117,63 +131,76 @@ public:
     RecoveryUnit* newRecoveryUnit() override;
 
     Status createRecordStore(OperationContext* opCtx,
-                             StringData ns,
+                             const NamespaceString& ns,
                              StringData ident,
-                             const CollectionOptions& options) override {
-        return createGroupedRecordStore(opCtx, ns, ident, options, KVPrefix::kNotPrefixed);
-    }
+                             const CollectionOptions& options,
+                             KeyFormat keyFormat = KeyFormat::Long) override;
 
     std::unique_ptr<RecordStore> getRecordStore(OperationContext* opCtx,
-                                                StringData ns,
+                                                const NamespaceString& nss,
                                                 StringData ident,
-                                                const CollectionOptions& options) override {
-        return getGroupedRecordStore(opCtx, ns, ident, options, KVPrefix::kNotPrefixed);
-    }
+                                                const CollectionOptions& options) override;
 
     std::unique_ptr<RecordStore> makeTemporaryRecordStore(OperationContext* opCtx,
-                                                          StringData ident) override;
+                                                          StringData ident,
+                                                          KeyFormat keyFormat) override;
 
     Status createSortedDataInterface(OperationContext* opCtx,
+                                     const NamespaceString& ns,
                                      const CollectionOptions& collOptions,
                                      StringData ident,
-                                     const IndexDescriptor* desc) override {
-        return createGroupedSortedDataInterface(
-            opCtx, collOptions, ident, desc, KVPrefix::kNotPrefixed);
-    }
-
+                                     const IndexDescriptor* desc) override;
     std::unique_ptr<SortedDataInterface> getSortedDataInterface(
-        OperationContext* opCtx, StringData ident, const IndexDescriptor* desc) override {
-        return getGroupedSortedDataInterface(opCtx, ident, desc, KVPrefix::kNotPrefixed);
-    }
+        OperationContext* opCtx,
+        const NamespaceString& nss,
+        const CollectionOptions& collOptions,
+        StringData ident,
+        const IndexDescriptor* desc) override;
 
-    Status createGroupedRecordStore(OperationContext* opCtx,
-                                    StringData ns,
-                                    StringData ident,
-                                    const CollectionOptions& options,
-                                    KVPrefix prefix) override;
+    /**
+     * Creates a new column store for the provided ident.
+     */
+    Status createColumnStore(OperationContext* opCtx,
+                             const NamespaceString& ns,
+                             const CollectionOptions& collOptions,
+                             StringData ident,
+                             const IndexDescriptor* desc) override;
+    /**
+     * Creates a ColumnStore object representing an existing column store for the provided ident.
+     */
+    std::unique_ptr<ColumnStore> getColumnStore(OperationContext* opCtx,
+                                                const NamespaceString& nss,
+                                                const CollectionOptions& collOptions,
+                                                StringData ident,
+                                                const IndexDescriptor*) override;
 
-    std::unique_ptr<RecordStore> getGroupedRecordStore(OperationContext* opCtx,
-                                                       StringData ns,
-                                                       StringData ident,
-                                                       const CollectionOptions& options,
-                                                       KVPrefix prefix) override;
+    Status importRecordStore(OperationContext* opCtx,
+                             StringData ident,
+                             const BSONObj& storageMetadata,
+                             const ImportOptions& importOptions) override;
 
-    Status createGroupedSortedDataInterface(OperationContext* opCtx,
-                                            const CollectionOptions& collOptions,
-                                            StringData ident,
-                                            const IndexDescriptor* desc,
-                                            KVPrefix prefix) override;
+    Status importSortedDataInterface(OperationContext* opCtx,
+                                     StringData ident,
+                                     const BSONObj& storageMetadata,
+                                     const ImportOptions& importOptions) override;
 
-    std::unique_ptr<SortedDataInterface> getGroupedSortedDataInterface(OperationContext* opCtx,
-                                                                       StringData ident,
-                                                                       const IndexDescriptor* desc,
-                                                                       KVPrefix prefix) override;
+    /**
+     * Drops the specified ident for resumable index builds.
+     */
+    Status dropSortedDataInterface(OperationContext* opCtx, StringData ident) override;
 
-    Status dropIdent(OperationContext* opCtx, RecoveryUnit* ru, StringData ident) override;
+    Status dropIdent(RecoveryUnit* ru,
+                     StringData ident,
+                     StorageEngine::DropIdentCallback&& onDrop = nullptr) override;
+
+    void dropIdentForImport(OperationContext* opCtx, StringData ident) override;
 
     void alterIdentMetadata(OperationContext* opCtx,
                             StringData ident,
-                            const IndexDescriptor* desc) override;
+                            const IndexDescriptor* desc,
+                            bool isForceUpdateMetadata) override;
+
+    Status alterMetadata(StringData uri, StringData config);
 
     void flushAllFiles(OperationContext* opCtx, bool callerHoldsReadLock) override;
 
@@ -183,12 +210,14 @@ public:
 
     Status disableIncrementalBackup(OperationContext* opCtx) override;
 
-    StatusWith<StorageEngine::BackupInformation> beginNonBlockingBackup(
-        OperationContext* opCtx, const StorageEngine::BackupOptions& options) override;
+    StatusWith<std::unique_ptr<StorageEngine::StreamingCursor>> beginNonBlockingBackup(
+        OperationContext* opCtx,
+        boost::optional<Timestamp> checkpointTimestamp,
+        const StorageEngine::BackupOptions& options) override;
 
     void endNonBlockingBackup(OperationContext* opCtx) override;
 
-    virtual StatusWith<std::vector<std::string>> extendBackupCursor(
+    virtual StatusWith<std::deque<std::string>> extendBackupCursor(
         OperationContext* opCtx) override;
 
     int64_t getIdentSize(OperationContext* opCtx, StringData ident) override;
@@ -216,7 +245,7 @@ public:
 
     void setInitialDataTimestamp(Timestamp initialDataTimestamp) override;
 
-    Timestamp getInitialDataTimestamp() override;
+    Timestamp getInitialDataTimestamp() const override;
 
     void setOldestTimestampFromStable() override;
 
@@ -251,14 +280,10 @@ public:
 
     Timestamp getAllDurableTimestamp() const override;
 
-    Timestamp getOldestOpenReadTimestamp() const override;
-
     bool supportsReadConcernSnapshot() const final override;
 
     bool supportsOplogStones() const final override;
 
-    bool isCacheUnderPressure(OperationContext* opCtx) const override;
-
     bool supportsReadConcernMajority() const final;
 
     // wiredtiger specific
@@ -285,7 +310,7 @@ public:
      * be started and stopped multiple times as tests create and destroy the oplog record store.
      */
     void startOplogManager(OperationContext* opCtx, WiredTigerRecordStore* oplogRecordStore);
-    void haltOplogManager();
+    void haltOplogManager(WiredTigerRecordStore* oplogRecordStore, bool shuttingDown);
 
     /*
      * Always returns a non-nil pointer. However, the WiredTigerOplogManager may not have been
@@ -301,7 +326,7 @@ public:
         return _oplogManager.get();
     }
 
-    static void appendGlobalStats(BSONObjBuilder& b);
+    static void appendGlobalStats(OperationContext* opCtx, BSONObjBuilder& b);
 
     Timestamp getStableTimestamp() const override;
     Timestamp getOldestTimestamp() const override;
@@ -346,14 +371,39 @@ public:
         return _clockSource;
     }
 
+    StatusWith<Timestamp> pinOldestTimestamp(OperationContext* opCtx,
+                                             const std::string& requestingServiceName,
+                                             Timestamp requestedTimestamp,
+                                             bool roundUpIfTooOld) override;
+
+private:
+    StatusWith<Timestamp> _pinOldestTimestamp(WithLock,
+                                              const std::string& requestingServiceName,
+                                              Timestamp requestedTimestamp,
+                                              bool roundUpIfTooOld);
+
+public:
+    void unpinOldestTimestamp(const std::string& requestingServiceName) override;
+
+    std::map<std::string, Timestamp> getPinnedTimestampRequests();
+
     void setPinnedOplogTimestamp(const Timestamp& pinnedTimestamp) override;
 
-    BSONObj getSanitizedStorageOptionsForSecondaryReplication(
-        const BSONObj& options) const override;
+    void dump() const override;
+
+    Status reconfigureLogging() override;
+
+    StatusWith<BSONObj> getStorageMetadata(StringData ident) const override;
 
 private:
     class WiredTigerSessionSweeper;
-    class WiredTigerCheckpointThread;
+
+    struct IdentToDrop {
+        std::string uri;
+        StorageEngine::DropIdentCallback callback;
+    };
+
+    void _checkpoint(WT_SESSION* session);
 
     /**
      * Opens a connection on the WiredTiger database 'path' with the configuration 'wtOpenConfig'.
@@ -381,7 +431,7 @@ private:
     std::string _uri(StringData ident) const;
 
     /**
-     * Uses the 'stableTimestamp', the 'targetSnapshotHistoryWindowInSeconds' setting and the
+     * Uses the 'stableTimestamp', the 'minSnapshotHistoryWindowInSeconds' setting and the
      * current _oldestTimestamp to calculate what the new oldest_timestamp should be, in order to
      * maintain a window of available snapshots on the storage engine from oldest to stable
      * timestamp.
@@ -416,8 +466,9 @@ private:
     std::unique_ptr<WiredTigerSessionCache> _sessionCache;
     ClockSource* const _clockSource;
 
-    // Mutex to protect use of _oplogManager by this instance of KV engine.
+    // Mutex to protect use of _oplogRecordStore by this instance of KV engine.
     mutable Mutex _oplogManagerMutex = MONGO_MAKE_LATCH("::_oplogManagerMutex");
+    const WiredTigerRecordStore* _oplogRecordStore = nullptr;
     std::unique_ptr<WiredTigerOplogManager> _oplogManager;
 
     std::string _canonicalName;
@@ -427,11 +478,8 @@ private:
     std::unique_ptr<WiredTigerSizeStorer> _sizeStorer;
     std::string _sizeStorerUri;
     mutable ElapsedTracker _sizeStorerSyncTracker;
-
-    bool _durable;
     bool _ephemeral;  // whether we are using the in-memory mode of the WT engine
     const bool _inRepairMode;
-    bool _readOnly;
 
     // If _keepDataHistory is true, then the storage engine keeps all history after the stable
     // timestamp, and WiredTigerKVEngine is responsible for advancing the oldest timestamp. If
@@ -441,19 +489,18 @@ private:
     const bool _keepDataHistory = true;
 
     std::unique_ptr<WiredTigerSessionSweeper> _sessionSweeper;
-    std::unique_ptr<WiredTigerCheckpointThread> _checkpointThread;
 
     std::string _rsOptions;
     std::string _indexOptions;
 
-    mutable Mutex _dropAllQueuesMutex = MONGO_MAKE_LATCH("WiredTigerKVEngine::_dropAllQueuesMutex");
     mutable Mutex _identToDropMutex = MONGO_MAKE_LATCH("WiredTigerKVEngine::_identToDropMutex");
-    std::list<std::string> _identToDrop;
+    std::list<IdentToDrop> _identToDrop;
 
-    mutable Date_t _previousCheckedDropsQueued;
+    mutable AtomicWord<long long> _previousCheckedDropsQueued;
 
     std::unique_ptr<WiredTigerSession> _backupSession;
-    WT_CURSOR* _backupCursor;
+    WiredTigerBackup _wtBackup;
+
     mutable Mutex _oplogPinnedByBackupMutex =
         MONGO_MAKE_LATCH("WiredTigerKVEngine::_oplogPinnedByBackupMutex");
     boost::optional<Timestamp> _oplogPinnedByBackup;
@@ -467,14 +514,25 @@ private:
     // timestamp. Provided by replication layer because WT does not persist timestamps.
     AtomicWord<std::uint64_t> _initialDataTimestamp;
 
+    AtomicWord<std::uint64_t> _oplogNeededForCrashRecovery;
+
     std::unique_ptr<WiredTigerEngineRuntimeConfigParameter> _runTimeConfigParam;
 
     mutable Mutex _highestDurableTimestampMutex =
         MONGO_MAKE_LATCH("WiredTigerKVEngine::_highestDurableTimestampMutex");
     mutable unsigned long long _highestSeenDurableTimestamp = StorageEngine::kMinimumTimestamp;
 
+    mutable Mutex _oldestTimestampPinRequestsMutex =
+        MONGO_MAKE_LATCH("WiredTigerKVEngine::_oldestTimestampPinRequestsMutex");
+    std::map<std::string, Timestamp> _oldestTimestampPinRequests;
+
     // Pins the oplog so that OplogStones will not truncate oplog history equal or newer to this
     // timestamp.
     AtomicWord<std::uint64_t> _pinnedOplogTimestamp;
+
+    // Limits the actions of concurrent checkpoint callers as we update some internal data during a
+    // checkpoint. WT has a mutex of its own to only have one checkpoint active at all times so this
+    // is only to protect our internal updates.
+    Mutex _checkpointMutex = MONGO_MAKE_LATCH("WiredTigerKVEngine::_checkpointMutex");
 };
 }  // namespace mongo
