@@ -34,14 +34,12 @@
 #include <string>
 #include <wiredtiger.h>
 
-#include "mongo/db/catalog/collection_options.h"
-#include "mongo/db/storage/capped_callback.h"
-#include "mongo/db/storage/kv/kv_prefix.h"
 #include "mongo/db/storage/record_store.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_cursor.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_size_storer.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
@@ -74,7 +72,6 @@ class WiredTigerRecordStore : public RecordStore {
     friend class WiredTigerRecordStoreCursorBase;
 
     friend class StandardWiredTigerRecordStore;
-    friend class PrefixedWiredTigerRecordStore;
 
 public:
     /**
@@ -82,7 +79,7 @@ public:
      * The document 'options' is typically obtained from the 'wiredTiger' field of
      * CollectionOptions::storageEngine.
      */
-    static StatusWith<std::string> parseOptionsField(const BSONObj options);
+    static StatusWith<std::string> parseOptionsField(BSONObj options);
 
     /**
      * Creates a configuration string suitable for 'config' parameter in WT_SESSION::create().
@@ -97,23 +94,26 @@ public:
      * fail with the constructed configuration string.
      */
     static StatusWith<std::string> generateCreateString(const std::string& engineName,
-                                                        StringData ns,
+                                                        const NamespaceString& nss,
+                                                        StringData ident,
                                                         const CollectionOptions& options,
                                                         StringData extraStrings,
-                                                        bool prefixed);
+                                                        KeyFormat keyFormat,
+                                                        bool loggingEnabled);
 
     struct Params {
-        StringData ns;
+        NamespaceString nss;
         std::string ident;
         std::string engineName;
         bool isCapped;
+        KeyFormat keyFormat;
+        bool overwrite;
         bool isEphemeral;
-        int64_t cappedMaxSize;
-        int64_t cappedMaxDocs;
-        CappedCallback* cappedCallback;
+        bool isLogged;
+        boost::optional<int64_t> oplogMaxSize;
         WiredTigerSizeStorer* sizeStorer;
-        bool isReadOnly;
         bool tracksSizeAdjustments;
+        bool forceUpdateWithFullDocument;
     };
 
     WiredTigerRecordStore(WiredTigerKVEngine* kvEngine, OperationContext* opCtx, Params params);
@@ -127,11 +127,15 @@ public:
     // name of the RecordStore implementation
     virtual const char* name() const;
 
+    virtual KeyFormat keyFormat() const;
+
     virtual long long dataSize(OperationContext* opCtx) const;
 
     virtual long long numRecords(OperationContext* opCtx) const;
 
-    virtual bool isCapped() const;
+    virtual bool selfManagedOplogTruncation() const {
+        return true;
+    }
 
     virtual int64_t storageSize(OperationContext* opCtx,
                                 BSONObjBuilder* extraInfo = nullptr,
@@ -143,34 +147,33 @@ public:
 
     virtual bool findRecord(OperationContext* opCtx, const RecordId& id, RecordData* out) const;
 
-    virtual void deleteRecord(OperationContext* opCtx, const RecordId& id);
+    void doDeleteRecord(OperationContext* opCtx, const RecordId& id) final;
 
-    virtual Status insertRecords(OperationContext* opCtx,
-                                 std::vector<Record>* records,
-                                 const std::vector<Timestamp>& timestamps);
+    Status doInsertRecords(OperationContext* opCtx,
+                           std::vector<Record>* records,
+                           const std::vector<Timestamp>& timestamps) final;
 
-    virtual Status updateRecord(OperationContext* opCtx,
-                                const RecordId& recordId,
-                                const char* data,
-                                int len);
+    Status doUpdateRecord(OperationContext* opCtx,
+                          const RecordId& recordId,
+                          const char* data,
+                          int len) final;
 
     virtual bool updateWithDamagesSupported() const;
 
-    virtual StatusWith<RecordData> updateWithDamages(OperationContext* opCtx,
-                                                     const RecordId& id,
-                                                     const RecordData& oldRec,
-                                                     const char* damageSource,
-                                                     const mutablebson::DamageVector& damages);
+    StatusWith<RecordData> doUpdateWithDamages(OperationContext* opCtx,
+                                               const RecordId& id,
+                                               const RecordData& oldRec,
+                                               const char* damageSource,
+                                               const mutablebson::DamageVector& damages) final;
+
+    virtual void printRecordMetadata(OperationContext* opCtx, const RecordId& recordId) const;
 
     virtual std::unique_ptr<SeekableRecordCursor> getCursor(OperationContext* opCtx,
                                                             bool forward) const = 0;
 
     std::unique_ptr<RecordCursor> getRandomCursor(OperationContext* opCtx) const final;
 
-    virtual std::unique_ptr<RecordCursor> getRandomCursorWithOptions(
-        OperationContext* opCtx, StringData extraConfig) const = 0;
-
-    virtual Status truncate(OperationContext* opCtx);
+    Status doTruncate(OperationContext* opCtx) final;
 
     virtual bool compactSupported() const {
         return !_isEphemeral;
@@ -181,54 +184,36 @@ public:
 
     virtual Timestamp getPinnedOplog() const final;
 
-    virtual Status compact(OperationContext* opCtx) final;
-
-    virtual bool isInRecordIdOrder() const override {
-        return true;
-    }
+    Status doCompact(OperationContext* opCtx) final;
 
     virtual void validate(OperationContext* opCtx,
                           ValidateResults* results,
                           BSONObjBuilder* output);
 
-    virtual void appendCustomStats(OperationContext* opCtx,
-                                   BSONObjBuilder* result,
-                                   double scale) const;
+    virtual void appendNumericCustomStats(OperationContext* opCtx,
+                                          BSONObjBuilder* result,
+                                          double scale) const;
 
-    virtual void cappedTruncateAfter(OperationContext* opCtx, RecordId end, bool inclusive);
+    virtual void appendAllCustomStats(OperationContext* opCtx,
+                                      BSONObjBuilder* result,
+                                      double scale) const;
 
-    virtual boost::optional<RecordId> oplogStartHack(OperationContext* opCtx,
-                                                     const RecordId& startingPosition) const;
-
-    virtual Status oplogDiskLocRegister(OperationContext* opCtx,
-                                        const Timestamp& opTime,
-                                        bool orderedCommit);
+    void doCappedTruncateAfter(OperationContext* opCtx,
+                               const RecordId& end,
+                               bool inclusive,
+                               const AboutToDeleteRecordCallback& aboutToDelete) final;
 
     virtual void updateStatsAfterRepair(OperationContext* opCtx,
                                         long long numRecords,
                                         long long dataSize);
 
 
-    void waitForAllEarlierOplogWritesToBeVisible(OperationContext* opCtx) const override;
-
-    Status updateCappedSize(OperationContext* opCtx, long long cappedSize) final;
-
-    void setCappedCallback(CappedCallback* cb) {
-        stdx::lock_guard<Latch> lk(_cappedCallbackMutex);
-        _cappedCallback = cb;
-    }
-
-    int64_t cappedMaxDocs() const;
-    int64_t cappedMaxSize() const;
+    Status updateOplogSize(long long newOplogSize) final;
 
     const std::string& getURI() const {
         return _uri;
     }
 
-    const std::string& getIdent() const override {
-        return _ident;
-    }
-
     uint64_t tableId() const {
         return _tableId;
     }
@@ -244,9 +229,17 @@ public:
         _sizeStorer = ss;
     }
 
-    bool isOpHidden_forTest(const RecordId& id) const;
+    /**
+     * Sets the new number of records and flushes the size storer.
+     */
+    void setNumRecords(long long numRecords);
 
-    bool inShutdown() const;
+    /**
+     * Sets the new data size and flushes the size storer.
+     */
+    void setDataSize(long long dataSize);
+
+    bool isOpHidden_forTest(const RecordId& id) const;
 
     bool yieldAndAwaitOplogDeletionRequest(OperationContext* opCtx) override;
 
@@ -262,10 +255,6 @@ public:
      */
     void reclaimOplog(OperationContext* opCtx, Timestamp recoveryTimestamp);
 
-    bool haveCappedWaiters();
-
-    void notifyCappedWaitersIfNeeded();
-
     class OplogStones;
 
     // Exposed only for testing.
@@ -273,26 +262,27 @@ public:
         return _oplogStones.get();
     };
 
+    typedef stdx::variant<int64_t, WiredTigerItem> CursorKey;
+
 protected:
     virtual RecordId getKey(WT_CURSOR* cursor) const = 0;
 
-    virtual void setKey(WT_CURSOR* cursor, RecordId id) const = 0;
+    virtual void setKey(WT_CURSOR* cursor, const CursorKey* key) const = 0;
+
+    Status oplogDiskLocRegisterImpl(OperationContext* opCtx,
+                                    const Timestamp& opTime,
+                                    bool orderedCommit) override;
+
+    void waitForAllEarlierOplogWritesToBeVisibleImpl(OperationContext* opCtx) const override;
 
 private:
     class RandomCursor;
 
-    class NumRecordsChange;
-    class DataSizeChange;
-
-    static WiredTigerRecoveryUnit* _getRecoveryUnit(OperationContext* opCtx);
-
     Status _insertRecords(OperationContext* opCtx,
                           Record* records,
                           const Timestamp* timestamps,
                           size_t nRecords);
-
-    RecordId _nextId(OperationContext* opCtx);
-    bool cappedAndNeedDelete() const;
+    long long _reserveIdBlock(OperationContext* opCtx, size_t nRecords);
     RecordData _getData(const WiredTigerCursor& cursor) const;
 
 
@@ -303,16 +293,6 @@ private:
      */
     void _initNextIdIfNeeded(OperationContext* opCtx);
 
-    /**
-     * Position the cursor at the first key. The previously known first key is
-     * provided, as well as an indicator that this is being positioned for
-     * use by a truncate call.
-     */
-    void _positionAtFirstRecordId(OperationContext* opCtx,
-                                  WT_CURSOR* cursor,
-                                  const RecordId& firstKey,
-                                  bool forTruncate) const;
-
     /**
      * Adjusts the record count and data size metadata for this record store, respectively. These
      * functions consult the SizeRecoveryState to determine whether or not to actually change the
@@ -330,49 +310,35 @@ private:
      *      are pending writes to this ident as part of the recovery process, and so we must
      *      always adjust size metadata for these idents.
      */
-    void _changeNumRecordsAndDataSize(OperationContext* opCtx,
-                                      int64_t numRecordDiff,
-                                      int64_t dataSizeDiff);
-
-    /**
-     * Delete records from this record store as needed while _cappedMaxSize or _cappedMaxDocs is
-     * exceeded.
-     *
-     * _inlock version to be called once a lock has been acquired.
-     */
-    int64_t _cappedDeleteAsNeeded(OperationContext* opCtx, const RecordId& justInserted);
-    int64_t _cappedDeleteAsNeeded_inlock(OperationContext* opCtx, const RecordId& justInserted);
+    void _changeNumRecords(OperationContext* opCtx, int64_t diff);
+    void _increaseDataSize(OperationContext* opCtx, int64_t amount);
 
     const std::string _uri;
-    const std::string _ident;
     const uint64_t _tableId;  // not persisted
 
     // Canonical engine name to use for retrieving options
     const std::string _engineName;
     // The capped settings should not be updated once operations have started
     const bool _isCapped;
+    // The format of this RecordStore's RecordId keys.
+    const KeyFormat _keyFormat;
+    // Whether or not to allow writes to overwrite existing records with the same RecordId.
+    const bool _overwrite;
     // True if the storage engine is an in-memory storage engine
     const bool _isEphemeral;
     // True if WiredTiger is logging updates to this table
     const bool _isLogged;
     // True if the namespace of this record store starts with "local.oplog.", and false otherwise.
     const bool _isOplog;
-    int64_t _cappedMaxSize;
-    const int64_t _cappedMaxSizeSlack;  // when to start applying backpressure
-    const int64_t _cappedMaxDocs;
-    RecordId _cappedFirstRecord;
-    AtomicWord<long long> _cappedSleep;
-    AtomicWord<long long> _cappedSleepMS;
+    // True if the namespace of this record store starts with "config.system.change_collection", and
+    // false otherwise.
+    const bool _isChangeCollection;
 
-    // guards _cappedCallback and _shuttingDown
-    mutable Mutex _cappedCallbackMutex =
-        MONGO_MAKE_LATCH("WiredTigerRecordStore::_cappedCallbackMutex");
-    CappedCallback* _cappedCallback;
-    bool _shuttingDown;
-
-    // See comment in ::cappedDeleteAsNeeded
-    int _cappedDeleteCheckCount;
-    mutable stdx::timed_mutex _cappedDeleterMutex;
+    // TODO (SERVER-57482): Remove special handling of skipping "wiredtiger_calc_modify()".
+    // True if force to update with the full document, and false otherwise.
+    const bool _forceUpdateWithFullDocument;
+    boost::optional<int64_t> _oplogMaxSize;
+    RecordId _oplogFirstRecord;
 
     // Protects initialization of the _nextIdNum.
     mutable Mutex _initNextIdMutex = MONGO_MAKE_LATCH("WiredTigerRecordStore::_initNextIdMutex");
@@ -401,39 +367,10 @@ public:
     virtual std::unique_ptr<SeekableRecordCursor> getCursor(OperationContext* opCtx,
                                                             bool forward) const override;
 
-    virtual std::unique_ptr<RecordCursor> getRandomCursorWithOptions(
-        OperationContext* opCtx, StringData extraConfig) const override;
-
 protected:
-    virtual RecordId getKey(WT_CURSOR* cursor) const;
+    virtual RecordId getKey(WT_CURSOR* cursor) const override;
 
-    virtual void setKey(WT_CURSOR* cursor, RecordId id) const;
-};
-
-class PrefixedWiredTigerRecordStore final : public WiredTigerRecordStore {
-public:
-    PrefixedWiredTigerRecordStore(WiredTigerKVEngine* kvEngine,
-                                  OperationContext* opCtx,
-                                  Params params,
-                                  KVPrefix prefix);
-
-    virtual std::unique_ptr<SeekableRecordCursor> getCursor(OperationContext* opCtx,
-                                                            bool forward) const override;
-
-    virtual std::unique_ptr<RecordCursor> getRandomCursorWithOptions(
-        OperationContext* opCtx, StringData extraConfig) const override;
-
-    virtual KVPrefix getPrefix() const {
-        return _prefix;
-    }
-
-protected:
-    virtual RecordId getKey(WT_CURSOR* cursor) const;
-
-    virtual void setKey(WT_CURSOR* cursor, RecordId id) const;
-
-private:
-    KVPrefix _prefix;
+    virtual void setKey(WT_CURSOR* cursor, const CursorKey* key) const override;
 };
 
 class WiredTigerRecordStoreCursorBase : public SeekableRecordCursor {
@@ -446,29 +383,26 @@ public:
 
     boost::optional<Record> seekExact(const RecordId& id);
 
+    boost::optional<Record> seekNear(const RecordId& start);
+
     void save();
 
     void saveUnpositioned();
 
-    bool restore();
+    bool restore(bool tolerateCappedRepositioning = true);
 
     void detachFromOperationContext();
 
     void reattachToOperationContext(OperationContext* opCtx);
 
+    void setSaveStorageCursorOnDetachFromOperationContext(bool saveCursor) override {
+        _saveStorageCursorOnDetachFromOperationContext = saveCursor;
+    }
+
 protected:
     virtual RecordId getKey(WT_CURSOR* cursor) const = 0;
 
-    virtual void setKey(WT_CURSOR* cursor, RecordId id) const = 0;
-
-    /**
-     * Callers must have already checked the return value of a positioning method against
-     * 'WT_NOTFOUND'. This method allows for additional predicates to be considered on a validly
-     * positioned cursor. 'id' is an out parameter. Implementations are not required to fill it
-     * in. It's simply a possible optimization to avoid a future 'getKey' call if 'hasWrongPrefix'
-     * already did one.
-     */
-    virtual bool hasWrongPrefix(WT_CURSOR* cursor, RecordId* id) const = 0;
+    virtual void setKey(WT_CURSOR* cursor, const WiredTigerRecordStore::CursorKey* key) const = 0;
 
     /**
      * Called when restoring a cursor that has not been advanced.
@@ -486,6 +420,7 @@ protected:
 
 private:
     bool isVisible(const RecordId& id);
+    void initOplogVisibility(OperationContext* opCtx);
 
     /**
      * This value is used for visibility calculations on what oplog entries can be returned to a
@@ -493,6 +428,16 @@ private:
      * established.
      */
     boost::optional<std::int64_t> _oplogVisibleTs = boost::none;
+
+    /**
+     * With WT-8601, WiredTiger no longer maintains commit_timestamp information on writes to logged
+     * tables, such as the oplog. There are occasions where the server applies a TimestampReadsource
+     * (e.g: majority) and expects the storage engine to obey the prior semantics. When a cursor is
+     * opened against the oplog, we will populate this variable with the recovery unit's read
+     * timestamp to apply a visibility check.
+     */
+    boost::optional<std::int64_t> _readTimestampForOplog = boost::none;
+    bool _saveStorageCursorOnDetachFromOperationContext = false;
 };
 
 class WiredTigerRecordStoreStandardCursor final : public WiredTigerRecordStoreCursorBase {
@@ -504,45 +449,10 @@ public:
 protected:
     virtual RecordId getKey(WT_CURSOR* cursor) const override;
 
-    virtual void setKey(WT_CURSOR* cursor, RecordId id) const override;
+    virtual void setKey(WT_CURSOR* cursor,
+                        const WiredTigerRecordStore::CursorKey* key) const override;
 
-    /**
-     * Callers must have already checked the return value of a positioning method against
-     * 'WT_NOTFOUND'. This method allows for additional predicates to be considered on a validly
-     * positioned cursor. 'id' is an out parameter. Implementations are not required to fill it
-     * in. It's simply a possible optimization to avoid a future 'getKey' call if 'hasWrongPrefix'
-     * already did one.
-     */
-    virtual bool hasWrongPrefix(WT_CURSOR* cursor, RecordId* id) const override;
-
-    virtual void initCursorToBeginning(){};
-};
-
-class WiredTigerRecordStorePrefixedCursor final : public WiredTigerRecordStoreCursorBase {
-public:
-    WiredTigerRecordStorePrefixedCursor(OperationContext* opCtx,
-                                        const WiredTigerRecordStore& rs,
-                                        KVPrefix prefix,
-                                        bool forward = true);
-
-protected:
-    virtual RecordId getKey(WT_CURSOR* cursor) const override;
-
-    virtual void setKey(WT_CURSOR* cursor, RecordId id) const override;
-
-    /**
-     * Callers must have already checked the return value of a positioning method against
-     * 'WT_NOTFOUND'. This method allows for additional predicates to be considered on a validly
-     * positioned cursor. 'id' is an out parameter. Implementations are not required to fill it
-     * in. It's simply a possible optimization to avoid a future 'getKey' call if 'hasWrongPrefix'
-     * already did one.
-     */
-    virtual bool hasWrongPrefix(WT_CURSOR* cursor, RecordId* id) const override;
-
-    virtual void initCursorToBeginning() override;
-
-private:
-    KVPrefix _prefix;
+    virtual void initCursorToBeginning() override{};
 };
 
 // WT failpoint to throw write conflict exceptions randomly
