@@ -27,9 +27,6 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include <memory>
 #include <sstream>
 #include <string>
 #include <time.h>
@@ -38,19 +35,19 @@
 #include "mongo/base/init.h"
 #include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobjbuilder.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
+#include "mongo/db/catalog/clustered_collection_util.h"
 #include "mongo/db/json.h"
 #include "mongo/db/operation_context_noop.h"
-#include "mongo/db/storage/kv/kv_prefix.h"
-#include "mongo/db/storage/record_store_test_harness.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_record_store.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_record_store_oplog_stones.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_record_store_test_harness.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_size_storer.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
 #include "mongo/unittest/barrier.h"
 #include "mongo/unittest/temp_dir.h"
 #include "mongo/unittest/unittest.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/fail_point.h"
 #include "mongo/util/scopeguard.h"
 
@@ -104,7 +101,7 @@ TEST(WiredTigerRecordStoreTest, GenerateCreateStringValidConfigStringOption) {
 
 TEST(WiredTigerRecordStoreTest, Isolation1) {
     const auto harnessHelper(newRecordStoreHarnessHelper());
-    unique_ptr<RecordStore> rs(harnessHelper->newNonCappedRecordStore());
+    unique_ptr<RecordStore> rs(harnessHelper->newRecordStore());
 
     RecordId id1;
     RecordId id2;
@@ -155,7 +152,7 @@ TEST(WiredTigerRecordStoreTest, Isolation1) {
 
 TEST(WiredTigerRecordStoreTest, Isolation2) {
     const auto harnessHelper(newRecordStoreHarnessHelper());
-    unique_ptr<RecordStore> rs(harnessHelper->newNonCappedRecordStore());
+    unique_ptr<RecordStore> rs(harnessHelper->newRecordStore());
 
     RecordId id1;
     RecordId id2;
@@ -205,7 +202,6 @@ TEST(WiredTigerRecordStoreTest, Isolation2) {
     }
 }
 
-
 StatusWith<RecordId> insertBSON(ServiceContext::UniqueOperationContext& opCtx,
                                 unique_ptr<RecordStore>& rs,
                                 const Timestamp& opTime) {
@@ -222,45 +218,6 @@ StatusWith<RecordId> insertBSON(ServiceContext::UniqueOperationContext& opCtx,
     return res;
 }
 
-TEST(WiredTigerRecordStoreTest, CappedCursorRollover) {
-    unique_ptr<RecordStoreHarnessHelper> harnessHelper(newRecordStoreHarnessHelper());
-    unique_ptr<RecordStore> rs(harnessHelper->newCappedRecordStore("a.b", 10000, 5));
-
-    {  // first insert 3 documents
-        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
-        for (int i = 0; i < 3; ++i) {
-            WriteUnitOfWork uow(opCtx.get());
-            StatusWith<RecordId> res = rs->insertRecord(opCtx.get(), "a", 2, Timestamp());
-            ASSERT_OK(res.getStatus());
-            uow.commit();
-        }
-    }
-
-    // set up our cursor that should rollover
-
-    auto client2 = harnessHelper->serviceContext()->makeClient("c2");
-    auto cursorCtx = harnessHelper->newOperationContext(client2.get());
-    auto cursor = rs->getCursor(cursorCtx.get());
-    ASSERT(cursor->next());
-    cursor->save();
-    cursorCtx->recoveryUnit()->abandonSnapshot();
-
-    {  // insert 100 documents which causes rollover
-        auto client3 = harnessHelper->serviceContext()->makeClient("c3");
-        auto opCtx = harnessHelper->newOperationContext(client3.get());
-        for (int i = 0; i < 100; i++) {
-            WriteUnitOfWork uow(opCtx.get());
-            StatusWith<RecordId> res = rs->insertRecord(opCtx.get(), "a", 2, Timestamp());
-            ASSERT_OK(res.getStatus());
-            uow.commit();
-        }
-    }
-
-    // cursor should now be dead
-    ASSERT_FALSE(cursor->restore());
-    ASSERT(!cursor->next());
-}
-
 RecordId _oplogOrderInsertOplog(OperationContext* opCtx,
                                 const unique_ptr<RecordStore>& rs,
                                 int inc) {
@@ -280,7 +237,7 @@ TEST(WiredTigerRecordStoreTest, OplogDurableVisibilityInOrder) {
     WTPauseOplogVisibilityUpdateLoop.setMode(FailPoint::alwaysOn);
 
     unique_ptr<RecordStoreHarnessHelper> harnessHelper(newRecordStoreHarnessHelper());
-    unique_ptr<RecordStore> rs(harnessHelper->newCappedRecordStore("local.oplog.rs", 100000, -1));
+    unique_ptr<RecordStore> rs(harnessHelper->newOplogRecordStore());
     auto wtrs = checked_cast<WiredTigerRecordStore*>(rs.get());
 
     {
@@ -309,7 +266,7 @@ TEST(WiredTigerRecordStoreTest, OplogDurableVisibilityOutOfOrder) {
     WTPauseOplogVisibilityUpdateLoop.setMode(FailPoint::alwaysOn);
 
     unique_ptr<RecordStoreHarnessHelper> harnessHelper(newRecordStoreHarnessHelper());
-    unique_ptr<RecordStore> rs(harnessHelper->newCappedRecordStore("local.oplog.rs", 100000, -1));
+    unique_ptr<RecordStore> rs(harnessHelper->newOplogRecordStore());
 
     auto wtrs = checked_cast<WiredTigerRecordStore*>(rs.get());
 
@@ -353,11 +310,11 @@ TEST(WiredTigerRecordStoreTest, OplogDurableVisibilityOutOfOrder) {
 
 TEST(WiredTigerRecordStoreTest, AppendCustomStatsMetadata) {
     std::unique_ptr<RecordStoreHarnessHelper> harnessHelper = newRecordStoreHarnessHelper();
-    unique_ptr<RecordStore> rs(harnessHelper->newNonCappedRecordStore("a.b"));
+    unique_ptr<RecordStore> rs(harnessHelper->newRecordStore("a.b"));
 
     ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
     BSONObjBuilder builder;
-    rs->appendCustomStats(opCtx.get(), &builder, 1.0);
+    rs->appendAllCustomStats(opCtx.get(), &builder, 1.0);
     BSONObj customStats = builder.obj();
 
     BSONElement wiredTigerElement = customStats.getField(kWiredTigerEngineName);
@@ -375,32 +332,28 @@ TEST(WiredTigerRecordStoreTest, AppendCustomStatsMetadata) {
     ASSERT_EQUALS(creationStringElement.type(), String);
 }
 
-TEST(WiredTigerRecordStoreTest, CappedCursorYieldFirst) {
-    unique_ptr<RecordStoreHarnessHelper> harnessHelper(newRecordStoreHarnessHelper());
-    unique_ptr<RecordStore> rs(harnessHelper->newCappedRecordStore("a.b", 10000, 50));
+TEST(WiredTigerRecordStoreTest, AppendCustomNumericStats) {
+    std::unique_ptr<RecordStoreHarnessHelper> harnessHelper = newRecordStoreHarnessHelper();
+    unique_ptr<RecordStore> rs(harnessHelper->newRecordStore("a.c"));
 
-    RecordId id1;
+    ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+    BSONObjBuilder builder;
+    rs->appendNumericCustomStats(opCtx.get(), &builder, 1.0);
+    BSONObj customStats = builder.obj();
 
-    {  // first insert a document
-        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
-        WriteUnitOfWork uow(opCtx.get());
-        StatusWith<RecordId> res = rs->insertRecord(opCtx.get(), "a", 2, Timestamp());
-        ASSERT_OK(res.getStatus());
-        id1 = res.getValue();
-        uow.commit();
-    }
+    BSONElement wiredTigerElement = customStats.getField(kWiredTigerEngineName);
+    ASSERT_TRUE(wiredTigerElement.isABSONObj());
+    BSONObj wiredTiger = wiredTigerElement.Obj();
 
-    ServiceContext::UniqueOperationContext cursorCtx(harnessHelper->newOperationContext());
-    auto cursor = rs->getCursor(cursorCtx.get());
+    ASSERT_FALSE(wiredTiger.hasField("metadata"));
+    ASSERT_FALSE(wiredTiger.hasField("creationString"));
 
-    // See that things work if you yield before you first call next().
-    cursor->save();
-    cursorCtx->recoveryUnit()->abandonSnapshot();
-    ASSERT_TRUE(cursor->restore());
-    auto record = cursor->next();
-    ASSERT(record);
-    ASSERT_EQ(id1, record->id);
-    ASSERT(!cursor->next());
+    BSONElement cacheElement = wiredTiger.getField("cache");
+    ASSERT_TRUE(cacheElement.isABSONObj());
+    BSONObj cache = cacheElement.Obj();
+
+    BSONElement bytesElement = cache.getField("bytes currently in the cache");
+    ASSERT_TRUE(bytesElement.isNumber());
 }
 
 BSONObj makeBSONObjWithSize(const Timestamp& opTime, int size, char fill = 'x') {
@@ -438,10 +391,7 @@ StatusWith<RecordId> insertBSONWithSize(OperationContext* opCtx,
 // Insert records into an oplog and verify the number of stones that are created.
 TEST(WiredTigerRecordStoreTest, OplogStones_CreateNewStone) {
     std::unique_ptr<RecordStoreHarnessHelper> harnessHelper = newRecordStoreHarnessHelper();
-
-    const int64_t cappedMaxSize = 10 * 1024;  // 10KB
-    unique_ptr<RecordStore> rs(
-        harnessHelper->newCappedRecordStore("local.oplog.rs", cappedMaxSize, -1));
+    std::unique_ptr<RecordStore> rs(harnessHelper->newOplogRecordStore());
 
     WiredTigerRecordStore* wtrs = static_cast<WiredTigerRecordStore*>(rs.get());
     WiredTigerRecordStore::OplogStones* oplogStones = wtrs->oplogStones();
@@ -494,10 +444,7 @@ TEST(WiredTigerRecordStoreTest, OplogStones_CreateNewStone) {
 // record is changed.
 TEST(WiredTigerRecordStoreTest, OplogStones_UpdateRecord) {
     std::unique_ptr<RecordStoreHarnessHelper> harnessHelper = newRecordStoreHarnessHelper();
-
-    const int64_t cappedMaxSize = 10 * 1024;  // 10KB
-    unique_ptr<RecordStore> rs(
-        harnessHelper->newCappedRecordStore("local.oplog.rs", cappedMaxSize, -1));
+    std::unique_ptr<RecordStore> rs(harnessHelper->newOplogRecordStore());
 
     WiredTigerRecordStore* wtrs = static_cast<WiredTigerRecordStore*>(rs.get());
     WiredTigerRecordStore::OplogStones* oplogStones = wtrs->oplogStones();
@@ -553,6 +500,8 @@ TEST(WiredTigerRecordStoreTest, OplogStones_UpdateRecord) {
         BSONObj changed2 = makeBSONObjWithSize(Timestamp(1, 2), 50, 'z');
 
         WriteUnitOfWork wuow(opCtx.get());
+        // Explicitly sets the timestamp to ensure ordered writes.
+        ASSERT_OK(opCtx->recoveryUnit()->setTimestamp(Timestamp(1, 3)));
         ASSERT_OK(
             rs->updateRecord(opCtx.get(), RecordId(1, 1), changed1.objdata(), changed1.objsize()));
         ASSERT_OK(
@@ -569,10 +518,7 @@ TEST(WiredTigerRecordStoreTest, OplogStones_UpdateRecord) {
 // should leave no stones, including the partially filled one.
 TEST(WiredTigerRecordStoreTest, OplogStones_Truncate) {
     std::unique_ptr<RecordStoreHarnessHelper> harnessHelper = newRecordStoreHarnessHelper();
-
-    const int64_t cappedMaxSize = 10 * 1024;  // 10KB
-    unique_ptr<RecordStore> rs(
-        harnessHelper->newCappedRecordStore("local.oplog.rs", cappedMaxSize, -1));
+    std::unique_ptr<RecordStore> rs(harnessHelper->newOplogRecordStore());
 
     WiredTigerRecordStore* wtrs = static_cast<WiredTigerRecordStore*>(rs.get());
     WiredTigerRecordStore::OplogStones* oplogStones = wtrs->oplogStones();
@@ -614,10 +560,7 @@ TEST(WiredTigerRecordStoreTest, OplogStones_Truncate) {
 // it should become the stone currently being filled.
 TEST(WiredTigerRecordStoreTest, OplogStones_CappedTruncateAfter) {
     std::unique_ptr<RecordStoreHarnessHelper> harnessHelper = newRecordStoreHarnessHelper();
-
-    const int64_t cappedMaxSize = 10 * 1024;  // 10KB
-    unique_ptr<RecordStore> rs(
-        harnessHelper->newCappedRecordStore("local.oplog.rs", cappedMaxSize, -1));
+    std::unique_ptr<RecordStore> rs(harnessHelper->newOplogRecordStore());
 
     WiredTigerRecordStore* wtrs = static_cast<WiredTigerRecordStore*>(rs.get());
     WiredTigerRecordStore::OplogStones* oplogStones = wtrs->oplogStones();
@@ -654,7 +597,10 @@ TEST(WiredTigerRecordStoreTest, OplogStones_CappedTruncateAfter) {
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
 
-        rs->cappedTruncateAfter(opCtx.get(), RecordId(1, 8), true);
+        rs->cappedTruncateAfter(opCtx.get(),
+                                RecordId(1, 8),
+                                true /* inclusive */,
+                                nullptr /* aboutToDelete callback */);
 
         ASSERT_EQ(7, rs->numRecords(opCtx.get()));
         ASSERT_EQ(2350, rs->dataSize(opCtx.get()));
@@ -668,7 +614,10 @@ TEST(WiredTigerRecordStoreTest, OplogStones_CappedTruncateAfter) {
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
 
-        rs->cappedTruncateAfter(opCtx.get(), RecordId(1, 6), true);
+        rs->cappedTruncateAfter(opCtx.get(),
+                                RecordId(1, 6),
+                                true /* inclusive */,
+                                nullptr /* aboutToDelete callback */);
 
         ASSERT_EQ(5, rs->numRecords(opCtx.get()));
         ASSERT_EQ(1950, rs->dataSize(opCtx.get()));
@@ -682,7 +631,10 @@ TEST(WiredTigerRecordStoreTest, OplogStones_CappedTruncateAfter) {
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
 
-        rs->cappedTruncateAfter(opCtx.get(), RecordId(1, 3), false);
+        rs->cappedTruncateAfter(opCtx.get(),
+                                RecordId(1, 3),
+                                false /* inclusive */,
+                                nullptr /* aboutToDelete callback */);
 
         ASSERT_EQ(3, rs->numRecords(opCtx.get()));
         ASSERT_EQ(1400, rs->dataSize(opCtx.get()));
@@ -696,7 +648,10 @@ TEST(WiredTigerRecordStoreTest, OplogStones_CappedTruncateAfter) {
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
 
-        rs->cappedTruncateAfter(opCtx.get(), RecordId(1, 2), false);
+        rs->cappedTruncateAfter(opCtx.get(),
+                                RecordId(1, 2),
+                                false /* inclusive */,
+                                nullptr /* aboutToDelete callback */);
 
         ASSERT_EQ(2, rs->numRecords(opCtx.get()));
         ASSERT_EQ(1200, rs->dataSize(opCtx.get()));
@@ -710,7 +665,10 @@ TEST(WiredTigerRecordStoreTest, OplogStones_CappedTruncateAfter) {
     {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
 
-        rs->cappedTruncateAfter(opCtx.get(), RecordId(1, 1), false);
+        rs->cappedTruncateAfter(opCtx.get(),
+                                RecordId(1, 1),
+                                false /* inclusive */,
+                                nullptr /* aboutToDelete callback */);
 
         ASSERT_EQ(1, rs->numRecords(opCtx.get()));
         ASSERT_EQ(400, rs->dataSize(opCtx.get()));
@@ -723,18 +681,12 @@ TEST(WiredTigerRecordStoreTest, OplogStones_CappedTruncateAfter) {
 // Verify that oplog stones are reclaimed when cappedMaxSize is exceeded.
 TEST(WiredTigerRecordStoreTest, OplogStones_ReclaimStones) {
     std::unique_ptr<RecordStoreHarnessHelper> harnessHelper = newRecordStoreHarnessHelper();
-
-    const int64_t cappedMaxSize = 10 * 1024;  // 10KB
-    unique_ptr<RecordStore> rs(
-        harnessHelper->newCappedRecordStore("local.oplog.rs", cappedMaxSize, -1));
+    std::unique_ptr<RecordStore> rs(harnessHelper->newOplogRecordStore());
 
     WiredTigerRecordStore* wtrs = static_cast<WiredTigerRecordStore*>(rs.get());
     WiredTigerRecordStore::OplogStones* oplogStones = wtrs->oplogStones();
 
-    {
-        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
-        ASSERT_OK(wtrs->updateCappedSize(opCtx.get(), 230U));
-    }
+    ASSERT_OK(wtrs->updateOplogSize(230));
 
     oplogStones->setMinBytesPerStone(100);
 
@@ -818,16 +770,89 @@ TEST(WiredTigerRecordStoreTest, OplogStones_ReclaimStones) {
         ASSERT_EQ(1, oplogStones->currentRecords());
         ASSERT_EQ(50, oplogStones->currentBytes());
     }
+
+    // Don't truncate the last stone before the truncate point, even if the truncate point
+    // is ahead of it.
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+
+        ASSERT_EQ(insertBSONWithSize(opCtx.get(), rs.get(), Timestamp(1, 7), 190), RecordId(1, 7));
+        ASSERT_EQ(insertBSONWithSize(opCtx.get(), rs.get(), Timestamp(1, 9), 120), RecordId(1, 9));
+
+        ASSERT_EQ(4, rs->numRecords(opCtx.get()));
+        ASSERT_EQ(500, rs->dataSize(opCtx.get()));
+        ASSERT_EQ(3U, oplogStones->numStones());
+        ASSERT_EQ(0, oplogStones->currentRecords());
+        ASSERT_EQ(0, oplogStones->currentBytes());
+    }
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+
+        wtrs->reclaimOplog(opCtx.get(), Timestamp(1, 8));
+
+        ASSERT_EQ(3, rs->numRecords(opCtx.get()));
+        ASSERT_EQ(360, rs->dataSize(opCtx.get()));
+        ASSERT_EQ(2U, oplogStones->numStones());
+        ASSERT_EQ(0, oplogStones->currentRecords());
+        ASSERT_EQ(0, oplogStones->currentBytes());
+    }
+
+    // Don't truncate entire oplog.
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+
+        ASSERT_EQ(insertBSONWithSize(opCtx.get(), rs.get(), Timestamp(1, 10), 90), RecordId(1, 10));
+        ASSERT_EQ(insertBSONWithSize(opCtx.get(), rs.get(), Timestamp(1, 11), 210),
+                  RecordId(1, 11));
+
+        ASSERT_EQ(5, rs->numRecords(opCtx.get()));
+        ASSERT_EQ(660, rs->dataSize(opCtx.get()));
+        ASSERT_EQ(3U, oplogStones->numStones());
+        ASSERT_EQ(0, oplogStones->currentRecords());
+        ASSERT_EQ(0, oplogStones->currentBytes());
+    }
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+
+        wtrs->reclaimOplog(opCtx.get(), Timestamp(1, 12));
+
+        ASSERT_EQ(2, rs->numRecords(opCtx.get()));
+        ASSERT_EQ(300, rs->dataSize(opCtx.get()));
+        ASSERT_EQ(1U, oplogStones->numStones());
+        ASSERT_EQ(0, oplogStones->currentRecords());
+        ASSERT_EQ(0, oplogStones->currentBytes());
+    }
+
+    // OK to truncate all stones if there are records in the oplog that are before or at the
+    // truncate-up-to point, that have not yet created a stone.
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        ASSERT_EQ(insertBSONWithSize(opCtx.get(), rs.get(), Timestamp(1, 12), 90), RecordId(1, 12));
+
+        ASSERT_EQ(3, rs->numRecords(opCtx.get()));
+        ASSERT_EQ(390, rs->dataSize(opCtx.get()));
+        ASSERT_EQ(1U, oplogStones->numStones());
+        ASSERT_EQ(1, oplogStones->currentRecords());
+        ASSERT_EQ(90, oplogStones->currentBytes());
+    }
+    {
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+
+        wtrs->reclaimOplog(opCtx.get(), Timestamp(1, 12));
+
+        ASSERT_EQ(1, rs->numRecords(opCtx.get()));
+        ASSERT_EQ(90, rs->dataSize(opCtx.get()));
+        ASSERT_EQ(0U, oplogStones->numStones());
+        ASSERT_EQ(1, oplogStones->currentRecords());
+        ASSERT_EQ(90, oplogStones->currentBytes());
+    }
 }
 
 // Verify that an oplog stone isn't created if it would cause the logical representation of the
 // records to not be in increasing order.
 TEST(WiredTigerRecordStoreTest, OplogStones_AscendingOrder) {
     std::unique_ptr<RecordStoreHarnessHelper> harnessHelper = newRecordStoreHarnessHelper();
-
-    const int64_t cappedMaxSize = 10 * 1024;  // 10KB
-    unique_ptr<RecordStore> rs(
-        harnessHelper->newCappedRecordStore("local.oplog.rs", cappedMaxSize, -1));
+    std::unique_ptr<RecordStore> rs(harnessHelper->newOplogRecordStore());
 
     WiredTigerRecordStore* wtrs = static_cast<WiredTigerRecordStore*>(rs.get());
     WiredTigerRecordStore::OplogStones* oplogStones = wtrs->oplogStones();
@@ -867,58 +892,119 @@ TEST(WiredTigerRecordStoreTest, OplogStones_AscendingOrder) {
     }
 }
 
+// Ensure that if we sample and create duplicate oplog stones, perform truncation correctly, and
+// with no crashing behavior. This scenario may be possible if the same record is sampled multiple
+// times during startup, which can be very likely if the size storer is very inaccurate.
+TEST(WiredTigerRecordStoreTest, OplogStones_Duplicates) {
+    std::unique_ptr<RecordStoreHarnessHelper> harnessHelper = newRecordStoreHarnessHelper();
+    auto wtHarnessHelper = dynamic_cast<WiredTigerHarnessHelper*>(harnessHelper.get());
+    std::unique_ptr<RecordStore> rs(wtHarnessHelper->newOplogRecordStoreNoInit());
+
+    WiredTigerRecordStore* wtrs = static_cast<WiredTigerRecordStore*>(rs.get());
+
+    {
+        // Before initializing the RecordStore, which also starts the oplog sampling process,
+        // populate with a few records.
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        ASSERT_EQ(insertBSONWithSize(opCtx.get(), rs.get(), Timestamp(1, 0), 100), RecordId(1, 0));
+        ASSERT_EQ(insertBSONWithSize(opCtx.get(), rs.get(), Timestamp(2, 0), 100), RecordId(2, 0));
+        ASSERT_EQ(insertBSONWithSize(opCtx.get(), rs.get(), Timestamp(3, 0), 100), RecordId(3, 0));
+        ASSERT_EQ(insertBSONWithSize(opCtx.get(), rs.get(), Timestamp(4, 0), 100), RecordId(4, 0));
+    }
+
+    // Force the oplog visibility timestamp to be up-to-date to the last record.
+    auto wtKvEngine = dynamic_cast<WiredTigerKVEngine*>(harnessHelper->getEngine());
+    wtKvEngine->getOplogManager()->setOplogReadTimestamp(Timestamp(4, 0));
+
+    {
+        // Force initialize the oplog stones to use sampling by providing very large, inaccurate
+        // sizes. This should cause us to oversample the records in the oplog.
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        wtrs->setNumRecords(1024 * 1024);
+        wtrs->setDataSize(1024 * 1024 * 1024);
+        wtrs->postConstructorInit(opCtx.get());
+    }
+
+    WiredTigerRecordStore::OplogStones* oplogStones = wtrs->oplogStones();
+
+    // Confirm that sampling occurred and that some stones were generated.
+    ASSERT(oplogStones->processedBySampling());
+    auto stonesBefore = oplogStones->numStones();
+    ASSERT_GT(stonesBefore, 0U);
+
+    {
+        // Reclaiming should do nothing because the data size is still under the maximum.
+        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        wtrs->reclaimOplog(opCtx.get(), Timestamp(4, 0));
+        ASSERT_EQ(stonesBefore, oplogStones->numStones());
+
+        // Reduce the oplog size to ensure we create a stone and truncate on the next insert.
+        ASSERT_OK(wtrs->updateOplogSize(400));
+
+        // Inserting these records should meet the requirements for truncation. That is: there is a
+        // record, 5, after the last stone, 4, and before the truncation point, 6.
+        ASSERT_EQ(insertBSONWithSize(opCtx.get(), rs.get(), Timestamp(5, 0), 100), RecordId(5, 0));
+        ASSERT_EQ(insertBSONWithSize(opCtx.get(), rs.get(), Timestamp(6, 0), 100), RecordId(6, 0));
+
+        // Ensure every stone has been cleaned up except for the last one ending in 6.
+        wtrs->reclaimOplog(opCtx.get(), Timestamp(6, 0));
+        ASSERT_EQ(1, oplogStones->numStones());
+
+        // The original oplog should have rolled over and the size and count should be accurate.
+        ASSERT_EQ(1, wtrs->numRecords(opCtx.get()));
+        ASSERT_EQ(100, wtrs->dataSize(opCtx.get()));
+    }
+}
+
 TEST(WiredTigerRecordStoreTest, GetLatestOplogTest) {
     unique_ptr<RecordStoreHarnessHelper> harnessHelper(newRecordStoreHarnessHelper());
-    unique_ptr<RecordStore> rs(harnessHelper->newCappedRecordStore("local.oplog.rs", 100000, -1));
+    unique_ptr<RecordStore> rs(harnessHelper->newOplogRecordStore());
 
     auto wtrs = checked_cast<WiredTigerRecordStore*>(rs.get());
 
     // 1) Initialize the top of oplog to "1".
     ServiceContext::UniqueOperationContext op1(harnessHelper->newOperationContext());
-    op1->recoveryUnit()->beginUnitOfWork(op1.get());
-    Timestamp tsOne =
-        Timestamp(static_cast<unsigned long long>(_oplogOrderInsertOplog(op1.get(), rs, 1).repr()));
+    op1->recoveryUnit()->beginUnitOfWork(op1->readOnly());
+    Timestamp tsOne = Timestamp(
+        static_cast<unsigned long long>(_oplogOrderInsertOplog(op1.get(), rs, 1).getLong()));
     op1->recoveryUnit()->commitUnitOfWork();
     // Asserting on a recovery unit without a snapshot.
     ASSERT_EQ(tsOne, wtrs->getLatestOplogTimestamp(op1.get()));
 
     // 2) Open a hole at time "2".
-    op1->recoveryUnit()->beginUnitOfWork(op1.get());
+    op1->recoveryUnit()->beginUnitOfWork(op1->readOnly());
     // Don't save the return value because the compiler complains about unused variables.
     _oplogOrderInsertOplog(op1.get(), rs, 2);
-    // Querying with the recovery unit with a snapshot will not return the uncommitted value.
-    ASSERT_EQ(tsOne, wtrs->getLatestOplogTimestamp(op1.get()));
 
     // Store the client with an uncommitted transaction. Create a new, concurrent client.
     auto client1 = Client::releaseCurrent();
     Client::initThread("client2");
 
     ServiceContext::UniqueOperationContext op2(harnessHelper->newOperationContext());
-    op2->recoveryUnit()->beginUnitOfWork(op2.get());
-    Timestamp tsThree =
-        Timestamp(static_cast<unsigned long long>(_oplogOrderInsertOplog(op2.get(), rs, 3).repr()));
-    // Before committing, the query still only sees timestamp "1".
+    // Should not see uncommited write from op1.
     ASSERT_EQ(tsOne, wtrs->getLatestOplogTimestamp(op2.get()));
+
+    op2->recoveryUnit()->beginUnitOfWork(op2->readOnly());
+    Timestamp tsThree = Timestamp(
+        static_cast<unsigned long long>(_oplogOrderInsertOplog(op2.get(), rs, 3).getLong()));
     op2->recoveryUnit()->commitUnitOfWork();
     // After committing, three is the top of oplog.
     ASSERT_EQ(tsThree, wtrs->getLatestOplogTimestamp(op2.get()));
 
-    // Destroy client2.
+    // Switch to client 1.
     op2.reset();
-    Client::releaseCurrent();
-    // Reinstall client 1.
+    auto client2 = Client::releaseCurrent();
     Client::setCurrent(std::move(client1));
 
-    // A new query with client 1 will see timestamp "3".
-    ASSERT_EQ(tsThree, wtrs->getLatestOplogTimestamp(op1.get()));
     op1->recoveryUnit()->commitUnitOfWork();
-    // Committing the write at timestamp "2" does not change the top of oplog result.
+    // Committing the write at timestamp "2" does not change the top of oplog result. A new query
+    // with client 1 will see timestamp "3".
     ASSERT_EQ(tsThree, wtrs->getLatestOplogTimestamp(op1.get()));
 }
 
 TEST(WiredTigerRecordStoreTest, CursorInActiveTxnAfterNext) {
     unique_ptr<RecordStoreHarnessHelper> harnessHelper(newRecordStoreHarnessHelper());
-    unique_ptr<RecordStore> rs(harnessHelper->newNonCappedRecordStore());
+    unique_ptr<RecordStore> rs(harnessHelper->newRecordStore());
 
     RecordId rid1;
     {
@@ -942,23 +1028,23 @@ TEST(WiredTigerRecordStoreTest, CursorInActiveTxnAfterNext) {
 
         auto cursor = rs->getCursor(opCtx.get());
         ASSERT(cursor->next());
-        ASSERT_TRUE(ru->inActiveTxn());
+        ASSERT_TRUE(ru->isActive());
 
         // Committing a WriteUnitOfWork will end the current transaction.
         WriteUnitOfWork wuow(opCtx.get());
-        ASSERT_TRUE(ru->inActiveTxn());
+        ASSERT_TRUE(ru->isActive());
         wuow.commit();
-        ASSERT_FALSE(ru->inActiveTxn());
+        ASSERT_FALSE(ru->isActive());
 
         // If a cursor is used after a WUOW commits, it should implicitly start a new transaction.
         ASSERT(cursor->next());
-        ASSERT_TRUE(ru->inActiveTxn());
+        ASSERT_TRUE(ru->isActive());
     }
 }
 
 TEST(WiredTigerRecordStoreTest, CursorInActiveTxnAfterSeek) {
     unique_ptr<RecordStoreHarnessHelper> harnessHelper(newRecordStoreHarnessHelper());
-    unique_ptr<RecordStore> rs(harnessHelper->newNonCappedRecordStore());
+    unique_ptr<RecordStore> rs(harnessHelper->newRecordStore());
 
     RecordId rid1;
     {
@@ -982,25 +1068,100 @@ TEST(WiredTigerRecordStoreTest, CursorInActiveTxnAfterSeek) {
 
         auto cursor = rs->getCursor(opCtx.get());
         ASSERT(cursor->seekExact(rid1));
-        ASSERT_TRUE(ru->inActiveTxn());
+        ASSERT_TRUE(ru->isActive());
 
         // Committing a WriteUnitOfWork will end the current transaction.
         WriteUnitOfWork wuow(opCtx.get());
-        ASSERT_TRUE(ru->inActiveTxn());
+        ASSERT_TRUE(ru->isActive());
         wuow.commit();
-        ASSERT_FALSE(ru->inActiveTxn());
+        ASSERT_FALSE(ru->isActive());
 
         // If a cursor is used after a WUOW commits, it should implicitly start a new transaction.
         ASSERT(cursor->seekExact(rid1));
-        ASSERT_TRUE(ru->inActiveTxn());
+        ASSERT_TRUE(ru->isActive());
     }
 }
 
-// Make sure numRecords and dataSize are accurate after a delete rolls back and some other
-// transaction deletes the same rows before we have a chance of patching up the metadata.
-TEST(WiredTigerRecordStoreTest, SizeInfoAccurateAfterRollbackWithDelete) {
+// Verify clustered record stores.
+// This test case complements StorageEngineTest:TemporaryRecordStoreClustered which verifies
+// clustered temporary record stores.
+TEST(WiredTigerRecordStoreTest, ClusteredRecordStore) {
+    const unique_ptr<RecordStoreHarnessHelper> harnessHelper(newRecordStoreHarnessHelper());
+    const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+
+    ASSERT(opCtx.get());
+    const std::string ns = "testRecordStore";
+    const NamespaceString nss(ns);
+    const std::string uri = WiredTigerKVEngine::kTableUriPrefix + ns;
+    const StatusWith<std::string> result =
+        WiredTigerRecordStore::generateCreateString(kWiredTigerEngineName,
+                                                    nss,
+                                                    "",
+                                                    CollectionOptions(),
+                                                    "",
+                                                    KeyFormat::String,
+                                                    WiredTigerUtil::useTableLogging(nss));
+    ASSERT_TRUE(result.isOK());
+    const std::string config = result.getValue();
+
+    {
+        WriteUnitOfWork uow(opCtx.get());
+        WiredTigerRecoveryUnit* ru =
+            checked_cast<WiredTigerRecoveryUnit*>(opCtx.get()->recoveryUnit());
+        WT_SESSION* s = ru->getSession()->getSession();
+        invariantWTOK(s->create(s, uri.c_str(), config.c_str()), s);
+        uow.commit();
+    }
+
+    WiredTigerRecordStore::Params params;
+    params.nss = nss;
+    params.ident = ns;
+    params.engineName = kWiredTigerEngineName;
+    params.isCapped = false;
+    params.keyFormat = KeyFormat::String;
+    params.overwrite = false;
+    params.isEphemeral = false;
+    params.isLogged = WiredTigerUtil::useTableLogging(nss);
+    params.sizeStorer = nullptr;
+    params.tracksSizeAdjustments = true;
+    params.forceUpdateWithFullDocument = false;
+
+    const auto wtKvEngine = dynamic_cast<WiredTigerKVEngine*>(harnessHelper->getEngine());
+    auto rs = std::make_unique<StandardWiredTigerRecordStore>(wtKvEngine, opCtx.get(), params);
+    rs->postConstructorInit(opCtx.get());
+
+    const auto id = StringData{"1"};
+    const auto rid = RecordId(id.rawData(), id.size());
+    const auto data = "data";
+    {
+        WriteUnitOfWork wuow(opCtx.get());
+        StatusWith<RecordId> s =
+            rs->insertRecord(opCtx.get(), rid, data, strlen(data), Timestamp());
+        ASSERT_TRUE(s.isOK());
+        ASSERT_EQUALS(1, rs->numRecords(opCtx.get()));
+        wuow.commit();
+    }
+    // Read the record back.
+    RecordData rd;
+    ASSERT_TRUE(rs->findRecord(opCtx.get(), rid, &rd));
+    ASSERT_EQ(0, memcmp(data, rd.data(), strlen(data)));
+    // Update the record.
+    const auto dataUpdated = "updated";
+    {
+        WriteUnitOfWork wuow(opCtx.get());
+        ASSERT_OK(rs->updateRecord(opCtx.get(), rid, dataUpdated, strlen(dataUpdated)));
+        ASSERT_EQUALS(1, rs->numRecords(opCtx.get()));
+        wuow.commit();
+    }
+    ASSERT_TRUE(rs->findRecord(opCtx.get(), rid, &rd));
+    ASSERT_EQ(0, memcmp(dataUpdated, rd.data(), strlen(dataUpdated)));
+}
+
+// Make sure numRecords is accurate after a delete rolls back and some other transaction deletes the
+// same rows before we have a chance of patching up the metadata.
+TEST(WiredTigerRecordStoreTest, NumRecordsAccurateAfterRollbackWithDelete) {
     const auto harnessHelper(newRecordStoreHarnessHelper());
-    unique_ptr<RecordStore> rs(harnessHelper->newNonCappedRecordStore());
+    unique_ptr<RecordStore> rs(harnessHelper->newRecordStore());
 
     RecordId rid;  // This record will be deleted by two transactions.
 
@@ -1012,7 +1173,6 @@ TEST(WiredTigerRecordStoreTest, SizeInfoAccurateAfterRollbackWithDelete) {
     }
 
     ASSERT_EQ(1, rs->numRecords(ctx.get()));
-    ASSERT_EQ(2, rs->dataSize(ctx.get()));
 
     WriteUnitOfWork uow(ctx.get());
 
@@ -1044,7 +1204,6 @@ TEST(WiredTigerRecordStoreTest, SizeInfoAccurateAfterRollbackWithDelete) {
 
     abortedThread.join();
     ASSERT_EQ(0, rs->numRecords(ctx.get()));
-    ASSERT_EQ(0, rs->dataSize(ctx.get()));
 }
 
 }  // namespace
