@@ -27,13 +27,13 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
 
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_begin_transaction_block.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
@@ -42,6 +42,14 @@
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
 #include "mongo/logv2/log.h"
 #include "mongo/util/hex.h"
+#include "mongo/util/stacktrace.h"
+#include "mongo/util/testing_proctor.h"
+
+#include <fmt/compile.h>
+#include <fmt/format.h>
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
+
 
 namespace mongo {
 namespace {
@@ -56,8 +64,24 @@ logv2::LogSeverity kSlowTransactionSeverity = logv2::LogSeverity::Debug(1);
 
 MONGO_FAIL_POINT_DEFINE(doUntimestampedWritesForIdempotencyTests);
 
+void handleWriteContextForDebugging(WiredTigerRecoveryUnit& ru, Timestamp& ts) {
+    if (ru.gatherWriteContextForDebugging()) {
+        BSONObjBuilder builder;
+
+        std::string s;
+        StringStackTraceSink sink{s};
+        printStackTrace(sink);
+        builder.append("stacktrace", s);
+
+        builder.append("timestamp", ts);
+
+        ru.storeWriteContextForDebugging(builder.obj());
+    }
+}
 }  // namespace
 
+AtomicWord<std::int64_t> snapshotTooOldErrorCount{0};
+
 using Section = WiredTigerOperationStats::Section;
 
 std::map<int, std::pair<StringData, Section>> WiredTigerOperationStats::_statNameMap = {
@@ -97,7 +121,7 @@ void WiredTigerOperationStats::fetchStats(WT_SESSION* session,
     }
 
     // Reset the statistics so that the next fetch gives the recent values.
-    invariantWTOK(c->reset(c));
+    invariantWTOK(c->reset(c), c->session);
 }
 
 BSONObj WiredTigerOperationStats::toBSON() {
@@ -108,7 +132,10 @@ BSONObj WiredTigerOperationStats::toBSON() {
     for (auto const& stat : _stats) {
         // Find the user consumable name for this statistic.
         auto statIt = _statNameMap.find(stat.first);
-        invariant(statIt != _statNameMap.end());
+
+        // Ignore the session statistic that is not reported for the slow operations.
+        if (statIt == _statNameMap.end())
+            continue;
 
         auto statName = statIt->second.first;
         Section subs = statIt->second.second;
@@ -207,7 +234,7 @@ void WiredTigerRecoveryUnit::_abort() {
     _setState(State::kInactive);
 }
 
-void WiredTigerRecoveryUnit::beginUnitOfWork(OperationContext* opCtx) {
+void WiredTigerRecoveryUnit::doBeginUnitOfWork() {
     invariant(!_inUnitOfWork(), toString(_getState()));
     invariant(!_isCommittingOrAborting(),
               str::stream() << "cannot begin unit of work while commit or rollback handlers are "
@@ -228,9 +255,9 @@ void WiredTigerRecoveryUnit::prepareUnitOfWork() {
                 "preparing transaction at time: {prepareTimestamp}",
                 "prepareTimestamp"_attr = _prepareTimestamp);
 
-    const std::string conf = "prepare_timestamp=" + integerToHex(_prepareTimestamp.asULL());
+    const std::string conf = "prepare_timestamp=" + unsignedHex(_prepareTimestamp.asULL());
     // Prepare the transaction.
-    invariantWTOK(s->prepare_transaction(s, conf.c_str()));
+    invariantWTOK(s->prepare_transaction(s, conf.c_str()), s);
 }
 
 void WiredTigerRecoveryUnit::doCommitUnitOfWork() {
@@ -291,6 +318,17 @@ void WiredTigerRecoveryUnit::assertInActiveTxn() const {
                 "currentState"_attr = _getState());
 }
 
+void WiredTigerRecoveryUnit::setTxnModified() {
+    if (_multiTimestampConstraintTracker.isTxnModified) {
+        return;
+    }
+
+    _multiTimestampConstraintTracker.isTxnModified = true;
+    if (!_lastTimestampSet) {
+        _multiTimestampConstraintTracker.txnHasNonTimestampedWrite = true;
+    }
+}
+
 boost::optional<int64_t> WiredTigerRecoveryUnit::getOplogVisibilityTs() {
     if (!_isOplogReader) {
         return boost::none;
@@ -321,8 +359,9 @@ WiredTigerSession* WiredTigerRecoveryUnit::getSessionNoTxn() {
 void WiredTigerRecoveryUnit::doAbandonSnapshot() {
     invariant(!_inUnitOfWork(), toString(_getState()));
     if (_isActive()) {
-        // Can't be in a WriteUnitOfWork, so safe to rollback
-        _txnClose(false);
+        // Can't be in a WriteUnitOfWork, so safe to rollback if the AbandonSnapshotMode is
+        // kAbort. If kCommit, however, then any active cursors will remain positioned and valid.
+        _txnClose(_abandonSnapshotMode == AbandonSnapshotMode::kCommit /* commit */);
     }
     _setState(State::kInactive);
 }
@@ -332,43 +371,74 @@ void WiredTigerRecoveryUnit::preallocateSnapshot() {
     getSession();
 }
 
-void WiredTigerRecoveryUnit::refreshSnapshot() {
-    // First, start a new transaction at the same timestamp as the current one.  Then end the
-    // current transaction.  This overlap will prevent WT from cleaning up history required to serve
-    // the read timestamp.
+void WiredTigerRecoveryUnit::preallocateSnapshotForOplogRead() {
+    // Indicate that we are an oplog reader before opening the snapshot
+    setIsOplogReader();
+    preallocateSnapshot();
+}
 
+void WiredTigerRecoveryUnit::refreshSnapshot() {
     // Currently, this code only works for kNoOverlap or kNoTimestamp.
     invariant(_timestampReadSource == ReadSource::kNoOverlap ||
               _timestampReadSource == ReadSource::kNoTimestamp);
     invariant(_isActive());
     invariant(!_inUnitOfWork());
     invariant(!_noEvictionAfterRollback);
+    invariant(_abandonSnapshotMode == AbandonSnapshotMode::kAbort);
 
-    auto newSession = _sessionCache->getSession();
-    WiredTigerBeginTxnBlock txnOpen(newSession->getSession(),
-                                    _prepareConflictBehavior,
-                                    _roundUpPreparedTimestamps,
-                                    RoundUpReadTimestamp::kNoRoundForce);
-    if (_timestampReadSource != ReadSource::kNoTimestamp) {
-        auto status = txnOpen.setReadSnapshot(_readAtTimestamp);
-        fassert(5035300, status);
-    }
-    txnOpen.done();
-
-    // Now end the previous transaction.
-    auto wtSession = _session->getSession();
-    auto wtRet = wtSession->rollback_transaction(wtSession, nullptr);
-    invariantWTOK(wtRet);
-    LOGV2_DEBUG(5035301,
-                3,
-                "WT begin_transaction & rollback_transaction",
-                "snapshotId"_attr = getSnapshotId().toNumber());
-
-    _session = std::move(newSession);
+    auto session = _session->getSession();
+    invariantWTOK(session->reset_snapshot(session), session);
+    LOGV2_DEBUG(
+        6235000, 3, "WT refreshed snapshot", "snapshotId"_attr = getSnapshotId().toNumber());
 }
 
 void WiredTigerRecoveryUnit::_txnClose(bool commit) {
     invariant(_isActive(), toString(_getState()));
+
+    if (TestingProctor::instance().isEnabled() && _gatherWriteContextForDebugging && commit) {
+        LOGV2(5703402,
+              "Closing transaction with write context for debugging",
+              "count"_attr = _writeContextForDebugging.size());
+        for (auto const& ctx : _writeContextForDebugging) {
+            LOGV2_OPTIONS(5703403,
+                          {logv2::LogTruncation::Disabled},
+                          "Write context for debugging",
+                          "context"_attr = ctx);
+        }
+
+        _writeContextForDebugging.clear();
+        // We clear the context here, but we don't unset the flag. We need it still set to prevent a
+        // WCE loop in the multi-timestamp constraint code below. We are also expecting to hit the
+        // LOGV2_FATAL below, and don't really need to worry about re-using this recovery unit. If
+        // this changes in the future, we might need to unset _gatherWriteContextForDebugging under
+        // some conditions.
+    }
+
+    if (!_multiTimestampConstraintTracker.ignoreAllMultiTimestampConstraints &&
+        _multiTimestampConstraintTracker.txnHasNonTimestampedWrite &&
+        _multiTimestampConstraintTracker.timestampOrder.size() >= 2) {
+        // The first write in this transaction was not timestamped. Other writes have used at least
+        // two different timestamps. This violates the multi timestamp constraint where if a
+        // transaction sets multiple timestamps, the first timestamp must be set prior to any
+        // writes. Vice-versa, if a transaction writes a document before setting a timestamp, it
+        // must not set multiple timestamps.
+        if (TestingProctor::instance().isEnabled() && !_gatherWriteContextForDebugging) {
+            _gatherWriteContextForDebugging = true;
+            LOGV2_ERROR(5703401,
+                        "Found a violation of multi-timestamp constraint. Retrying operation to "
+                        "collect extra debugging context for the involved writes.");
+            throwWriteConflictException("Violation of multi-timestamp constraint.");
+        }
+        if (commit) {
+            LOGV2_FATAL(
+                4877100,
+                "Multi timestamp constraint violated. Transactions setting multiple timestamps "
+                "must set the first timestamp prior to any writes.",
+                "numTimestampsUsed"_attr = _multiTimestampConstraintTracker.timestampOrder.size(),
+                "lastSetTimestamp"_attr = _multiTimestampConstraintTracker.timestampOrder.top());
+        }
+    }
+
     WT_SESSION* s = _session->getSession();
     if (_timer) {
         const int transactionTime = _timer->millis();
@@ -377,9 +447,9 @@ void WiredTigerRecoveryUnit::_txnClose(bool commit) {
         if (transactionTime >= std::max(1, serverGlobalParams.slowMS)) {
             LOGV2_DEBUG(22411,
                         kSlowTransactionSeverity.toInt(),
-                        "Slow WT transaction. Lifetime of SnapshotId {getSnapshotId_toNumber} was "
+                        "Slow WT transaction. Lifetime of SnapshotId {snapshotId} was "
                         "{transactionTime}ms",
-                        "getSnapshotId_toNumber"_attr = getSnapshotId().toNumber(),
+                        "snapshotId"_attr = getSnapshotId().toNumber(),
                         "transactionTime"_attr = transactionTime);
         }
     }
@@ -403,12 +473,10 @@ void WiredTigerRecoveryUnit::_txnClose(bool commit) {
 
         wtRet = s->commit_transaction(s, nullptr);
 
-        LOGV2_DEBUG(22412,
-                    3,
-                    "WT commit_transaction for snapshot id {snapshotId}",
-                    "WT commit_transaction",
-                    "snapshotId"_attr = getSnapshotId().toNumber());
+        LOGV2_DEBUG(
+            22412, 3, "WT commit_transaction", "snapshotId"_attr = getSnapshotId().toNumber());
     } else {
+        invariant(_abandonSnapshotMode == AbandonSnapshotMode::kAbort);
         StringBuilder config;
         if (_noEvictionAfterRollback) {
             // The only point at which rollback_transaction() can time out is in the bonus-eviction
@@ -419,11 +487,8 @@ void WiredTigerRecoveryUnit::_txnClose(bool commit) {
 
         wtRet = s->rollback_transaction(s, config.str().c_str());
 
-        LOGV2_DEBUG(22413,
-                    3,
-                    "WT rollback_transaction for snapshot id {snapshotId}",
-                    "WT rollback_transaction",
-                    "snapshotId"_attr = getSnapshotId().toNumber());
+        LOGV2_DEBUG(
+            22413, 3, "WT rollback_transaction", "snapshotId"_attr = getSnapshotId().toNumber());
     }
 
     if (_isTimestamped) {
@@ -438,7 +503,7 @@ void WiredTigerRecoveryUnit::_txnClose(bool commit) {
         }
         _isTimestamped = false;
     }
-    invariantWTOK(wtRet);
+    invariantWTOK(wtRet, s);
 
     invariant(!_lastTimestampSet || _commitTimestamp.isNull(),
               str::stream() << "Cannot have both a _lastTimestampSet and a "
@@ -450,7 +515,7 @@ void WiredTigerRecoveryUnit::_txnClose(bool commit) {
     // transaction on a RecoveryUnit to call setTimestamp() and another to call
     // setCommitTimestamp().
     _lastTimestampSet = boost::none;
-
+    _multiTimestampConstraintTracker = MultiTimestampConstraintTracker();
     _prepareTimestamp = Timestamp();
     _durableTimestamp = Timestamp();
     _catalogConflictTimestamp = Timestamp();
@@ -458,48 +523,50 @@ void WiredTigerRecoveryUnit::_txnClose(bool commit) {
     _isOplogReader = false;
     _oplogVisibleTs = boost::none;
     _orderedCommit = true;  // Default value is true; we assume all writes are ordered.
-    _mustBeTimestamped = false;
+    _untimestampedWriteAssertion = WiredTigerBeginTxnBlock::UntimestampedWriteAssertion::kEnforce;
 }
 
-Status WiredTigerRecoveryUnit::obtainMajorityCommittedSnapshot() {
+Status WiredTigerRecoveryUnit::majorityCommittedSnapshotAvailable() const {
     invariant(_timestampReadSource == ReadSource::kMajorityCommitted);
     auto snapshotName = _sessionCache->snapshotManager().getMinSnapshotForNextCommittedRead();
     if (!snapshotName) {
         return {ErrorCodes::ReadConcernMajorityNotAvailableYet,
                 "Read concern majority reads are currently not possible."};
     }
-    _majorityCommittedSnapshot = *snapshotName;
     return Status::OK();
 }
 
-boost::optional<Timestamp> WiredTigerRecoveryUnit::getPointInTimeReadTimestamp() {
+boost::optional<Timestamp> WiredTigerRecoveryUnit::getPointInTimeReadTimestamp(
+    OperationContext* opCtx) {
     // After a ReadSource has been set on this RecoveryUnit, callers expect that this method returns
     // the read timestamp that will be used for current or future transactions. Because callers use
-    // this timestamp to inform visiblity of operations, it is therefore necessary to open a
+    // this timestamp to inform visibility of operations, it is therefore necessary to open a
     // transaction to establish a read timestamp, but only for ReadSources that are expected to have
     // read timestamps.
     switch (_timestampReadSource) {
         case ReadSource::kNoTimestamp:
             return boost::none;
-        case ReadSource::kMajorityCommitted:
-            // This ReadSource depends on a previous call to obtainMajorityCommittedSnapshot() and
-            // does not require an open transaction to return a valid timestamp.
-            invariant(!_majorityCommittedSnapshot.isNull());
-            return _majorityCommittedSnapshot;
         case ReadSource::kProvided:
             // The read timestamp is set by the user and does not require a transaction to be open.
             invariant(!_readAtTimestamp.isNull());
             return _readAtTimestamp;
-
+        case ReadSource::kLastApplied:
+            // The lastApplied timestamp is not always available if the system has not accepted
+            // writes, so it is not possible to invariant that it exists.
+            if (_readAtTimestamp.isNull()) {
+                return boost::none;
+            }
+            return _readAtTimestamp;
         // The following ReadSources can only establish a read timestamp when a transaction is
         // opened.
         case ReadSource::kNoOverlap:
-        case ReadSource::kLastApplied:
         case ReadSource::kAllDurableSnapshot:
+        case ReadSource::kMajorityCommitted:
             break;
     }
 
-    // Ensure a transaction is opened.
+    // Ensure a transaction is opened. Storage engine operations require the global lock.
+    invariant(opCtx->lockState()->isNoop() || opCtx->lockState()->isLocked());
     getSession();
 
     switch (_timestampReadSource) {
@@ -513,12 +580,12 @@ boost::optional<Timestamp> WiredTigerRecoveryUnit::getPointInTimeReadTimestamp()
             }
             return boost::none;
         case ReadSource::kAllDurableSnapshot:
+        case ReadSource::kMajorityCommitted:
             invariant(!_readAtTimestamp.isNull());
             return _readAtTimestamp;
 
         // The follow ReadSources returned values in the first switch block.
         case ReadSource::kNoTimestamp:
-        case ReadSource::kMajorityCommitted:
         case ReadSource::kProvided:
             MONGO_UNREACHABLE;
     }
@@ -533,7 +600,7 @@ void WiredTigerRecoveryUnit::_txnOpen() {
     _ensureSession();
 
     // Only start a timer for transaction's lifetime if we're going to log it.
-    if (shouldLog(kSlowTransactionSeverity)) {
+    if (shouldLog(MONGO_LOGV2_DEFAULT_COMPONENT, kSlowTransactionSeverity)) {
         _timer.reset(new Timer());
     }
     WT_SESSION* session = _session->getSession();
@@ -543,20 +610,24 @@ void WiredTigerRecoveryUnit::_txnOpen() {
             if (_isOplogReader) {
                 _oplogVisibleTs = static_cast<std::int64_t>(_oplogManager->getOplogReadTimestamp());
             }
-            WiredTigerBeginTxnBlock(session, _prepareConflictBehavior, _roundUpPreparedTimestamps)
+            WiredTigerBeginTxnBlock(session,
+                                    _prepareConflictBehavior,
+                                    _roundUpPreparedTimestamps,
+                                    RoundUpReadTimestamp::kNoRoundError,
+                                    _untimestampedWriteAssertion)
                 .done();
             break;
         }
         case ReadSource::kMajorityCommitted: {
-            // We reset _majorityCommittedSnapshot to the actual read timestamp used when the
-            // transaction was started.
-            _majorityCommittedSnapshot =
-                _sessionCache->snapshotManager().beginTransactionOnCommittedSnapshot(
-                    session, _prepareConflictBehavior, _roundUpPreparedTimestamps);
+            _readAtTimestamp = _sessionCache->snapshotManager().beginTransactionOnCommittedSnapshot(
+                session,
+                _prepareConflictBehavior,
+                _roundUpPreparedTimestamps,
+                _untimestampedWriteAssertion);
             break;
         }
         case ReadSource::kLastApplied: {
-            _readAtTimestamp = _beginTransactionAtLastAppliedTimestamp(session);
+            _beginTransactionAtLastAppliedTimestamp(session);
             break;
         }
         case ReadSource::kNoOverlap: {
@@ -568,14 +639,20 @@ void WiredTigerRecoveryUnit::_txnOpen() {
                 _readAtTimestamp = _beginTransactionAtAllDurableTimestamp(session);
                 break;
             }
-            // Intentionally continue to the next case to read at the _readAtTimestamp.
+            [[fallthrough]];  // Continue to the next case to read at the _readAtTimestamp.
         }
         case ReadSource::kProvided: {
-            WiredTigerBeginTxnBlock txnOpen(
-                session, _prepareConflictBehavior, _roundUpPreparedTimestamps);
+            WiredTigerBeginTxnBlock txnOpen(session,
+                                            _prepareConflictBehavior,
+                                            _roundUpPreparedTimestamps,
+                                            RoundUpReadTimestamp::kNoRoundError,
+                                            _untimestampedWriteAssertion);
             auto status = txnOpen.setReadSnapshot(_readAtTimestamp);
 
             if (!status.isOK() && status.code() == ErrorCodes::BadValue) {
+                // SnapshotTooOld errors indicate that PIT ops are failing to find an available
+                // snapshot at their specified atClusterTime.
+                snapshotTooOldErrorCount.addAndFetch(1);
                 uasserted(ErrorCodes::SnapshotTooOld,
                           str::stream() << "Read timestamp " << _readAtTimestamp.toString()
                                         << " is older than the oldest available timestamp.");
@@ -597,7 +674,8 @@ Timestamp WiredTigerRecoveryUnit::_beginTransactionAtAllDurableTimestamp(WT_SESS
     WiredTigerBeginTxnBlock txnOpen(session,
                                     _prepareConflictBehavior,
                                     _roundUpPreparedTimestamps,
-                                    RoundUpReadTimestamp::kRound);
+                                    RoundUpReadTimestamp::kRound,
+                                    _untimestampedWriteAssertion);
     Timestamp txnTimestamp = _sessionCache->getKVEngine()->getAllDurableTimestamp();
     auto status = txnOpen.setReadSnapshot(txnTimestamp);
     fassert(50948, status);
@@ -610,20 +688,8 @@ Timestamp WiredTigerRecoveryUnit::_beginTransactionAtAllDurableTimestamp(WT_SESS
     return readTimestamp;
 }
 
-namespace {
-/**
- * This mutex serializes starting read transactions at lastApplied. This throttles new transactions
- * so they do not overwhelm the WiredTiger spinlock that manages the global read timestamp queue.
- * Because this queue can grow larger than the number of active transactions, the MongoDB ticketing
- * system alone cannot restrict its growth and thus bound the amount of time the queue is locked.
- * TODO: WT-6709
- */
-Mutex _lastAppliedTxnMutex = MONGO_MAKE_LATCH("lastAppliedTxnMutex");
-}  // namespace
-
-Timestamp WiredTigerRecoveryUnit::_beginTransactionAtLastAppliedTimestamp(WT_SESSION* session) {
-    auto lastApplied = _sessionCache->snapshotManager().getLastApplied();
-    if (!lastApplied) {
+void WiredTigerRecoveryUnit::_beginTransactionAtLastAppliedTimestamp(WT_SESSION* session) {
+    if (_readAtTimestamp.isNull()) {
         // When there is not a lastApplied timestamp available, read without a timestamp. Do not
         // round up the read timestamp to the oldest timestamp.
 
@@ -633,27 +699,29 @@ Timestamp WiredTigerRecoveryUnit::_beginTransactionAtLastAppliedTimestamp(WT_SES
         // is only possible for readers that start immediately after an initial sync that did not
         // replicate any oplog entries. Future transactions will start reading at a timestamp once
         // timestamped writes have been made.
-        WiredTigerBeginTxnBlock txnOpen(
-            session, _prepareConflictBehavior, _roundUpPreparedTimestamps);
-        LOGV2_DEBUG(4847500, 2, "no read timestamp available for kLastApplied");
-        txnOpen.done();
-        return Timestamp();
-    }
-
-    {
-        stdx::lock_guard<Mutex> lock(_lastAppliedTxnMutex);
         WiredTigerBeginTxnBlock txnOpen(session,
                                         _prepareConflictBehavior,
                                         _roundUpPreparedTimestamps,
-                                        RoundUpReadTimestamp::kRound);
-        auto status = txnOpen.setReadSnapshot(*lastApplied);
-        fassert(4847501, status);
+                                        RoundUpReadTimestamp::kNoRoundError,
+                                        _untimestampedWriteAssertion);
+        LOGV2_DEBUG(4847500, 2, "no read timestamp available for kLastApplied");
         txnOpen.done();
+        return;
     }
 
-    // We might have rounded to oldest between calling getLastApplied and setReadSnapshot. We
-    // need to get the actual read timestamp we used.
-    return _getTransactionReadTimestamp(session);
+    WiredTigerBeginTxnBlock txnOpen(session,
+                                    _prepareConflictBehavior,
+                                    _roundUpPreparedTimestamps,
+                                    RoundUpReadTimestamp::kRound,
+                                    _untimestampedWriteAssertion);
+    auto status = txnOpen.setReadSnapshot(_readAtTimestamp);
+    fassert(4847501, status);
+
+    // We might have rounded to oldest between calling setTimestampReadSource and setReadSnapshot.
+    // We need to get the actual read timestamp we used.
+    auto actualTimestamp = _getTransactionReadTimestamp(session);
+    txnOpen.done();
+    _readAtTimestamp = actualTimestamp;
 }
 
 Timestamp WiredTigerRecoveryUnit::_beginTransactionAtNoOverlapTimestamp(WT_SESSION* session) {
@@ -697,8 +765,11 @@ Timestamp WiredTigerRecoveryUnit::_beginTransactionAtNoOverlapTimestamp(WT_SESSI
         // is only possible for readers that start immediately after an initial sync that did not
         // replicate any oplog entries. Future transactions will start reading at a timestamp once
         // timestamped writes have been made.
-        WiredTigerBeginTxnBlock txnOpen(
-            session, _prepareConflictBehavior, _roundUpPreparedTimestamps);
+        WiredTigerBeginTxnBlock txnOpen(session,
+                                        _prepareConflictBehavior,
+                                        _roundUpPreparedTimestamps,
+                                        RoundUpReadTimestamp::kNoRoundError,
+                                        _untimestampedWriteAssertion);
         LOGV2_DEBUG(4452900, 1, "no read timestamp available for kNoOverlap");
         txnOpen.done();
         return readTimestamp;
@@ -707,26 +778,37 @@ Timestamp WiredTigerRecoveryUnit::_beginTransactionAtNoOverlapTimestamp(WT_SESSI
     WiredTigerBeginTxnBlock txnOpen(session,
                                     _prepareConflictBehavior,
                                     _roundUpPreparedTimestamps,
-                                    RoundUpReadTimestamp::kRound);
+                                    RoundUpReadTimestamp::kRound,
+                                    _untimestampedWriteAssertion);
     auto status = txnOpen.setReadSnapshot(readTimestamp);
     fassert(51066, status);
-    txnOpen.done();
 
     // We might have rounded to oldest between calling getAllDurable and setReadSnapshot. We
     // need to get the actual read timestamp we used.
     readTimestamp = _getTransactionReadTimestamp(session);
+    txnOpen.done();
     return readTimestamp;
 }
 
 Timestamp WiredTigerRecoveryUnit::_getTransactionReadTimestamp(WT_SESSION* session) {
     char buf[(2 * 8 /*bytes in hex*/) + 1 /*nul terminator*/];
     auto wtstatus = session->query_timestamp(session, buf, "get=read");
-    invariantWTOK(wtstatus);
+    invariantWTOK(wtstatus, session);
     uint64_t read_timestamp;
     fassert(50949, NumberParser().base(16)(buf, &read_timestamp));
     return Timestamp(read_timestamp);
 }
 
+void WiredTigerRecoveryUnit::_updateMultiTimestampConstraint(Timestamp timestamp) {
+    std::stack<Timestamp>& timestampOrder = _multiTimestampConstraintTracker.timestampOrder;
+    if (!timestampOrder.empty() && timestampOrder.top() == timestamp) {
+        // We're still on the same timestamp.
+        return;
+    }
+
+    timestampOrder.push(timestamp);
+}
+
 Status WiredTigerRecoveryUnit::setTimestamp(Timestamp timestamp) {
     _ensureSession();
     LOGV2_DEBUG(22415,
@@ -744,8 +826,13 @@ Status WiredTigerRecoveryUnit::setTimestamp(Timestamp timestamp) {
                             << " cannot be older than read timestamp "
                             << _readAtTimestamp.toString());
 
+    _updateMultiTimestampConstraint(timestamp);
     _lastTimestampSet = timestamp;
 
+    if (TestingProctor::instance().isEnabled()) {
+        handleWriteContextForDebugging(*this, timestamp);
+    }
+
     // Starts the WT transaction associated with this session.
     getSession();
 
@@ -759,7 +846,7 @@ Status WiredTigerRecoveryUnit::setTimestamp(Timestamp timestamp) {
     if (rc == 0) {
         _isTimestamped = true;
     }
-    return wtRCToStatus(rc, "timestamp_transaction");
+    return wtRCToStatus(rc, session, "timestamp_transaction");
 }
 
 void WiredTigerRecoveryUnit::setCommitTimestamp(Timestamp timestamp) {
@@ -864,7 +951,7 @@ void WiredTigerRecoveryUnit::setRoundUpPreparedTimestamps(bool value) {
 
 void WiredTigerRecoveryUnit::setTimestampReadSource(ReadSource readSource,
                                                     boost::optional<Timestamp> provided) {
-    uassert(5863604, "Cannot change ReadSource as it is pinned.", !isReadSourcePinned());
+    tassert(5863604, "Cannot change ReadSource as it is pinned.", !isReadSourcePinned());
 
     LOGV2_DEBUG(22416,
                 3,
@@ -880,7 +967,16 @@ void WiredTigerRecoveryUnit::setTimestampReadSource(ReadSource readSource,
     invariant(!(provided && provided->isNull()));
 
     _timestampReadSource = readSource;
-    _readAtTimestamp = (provided) ? *provided : Timestamp();
+    if (readSource == kLastApplied) {
+        // The lastApplied timestamp is not always available if the system has not accepted writes.
+        if (auto lastApplied = _sessionCache->snapshotManager().getLastApplied()) {
+            _readAtTimestamp = *lastApplied;
+        } else {
+            _readAtTimestamp = Timestamp();
+        }
+    } else {
+        _readAtTimestamp = (provided) ? *provided : Timestamp();
+    }
 }
 
 RecoveryUnit::ReadSource WiredTigerRecoveryUnit::getTimestampReadSource() const {
@@ -942,4 +1038,12 @@ Timestamp WiredTigerRecoveryUnit::getCatalogConflictingTimestamp() const {
     return _catalogConflictTimestamp;
 }
 
+bool WiredTigerRecoveryUnit::gatherWriteContextForDebugging() const {
+    return _gatherWriteContextForDebugging;
+}
+
+void WiredTigerRecoveryUnit::storeWriteContextForDebugging(const BSONObj& info) {
+    _writeContextForDebugging.push_back(info);
+}
+
 }  // namespace mongo
