@@ -30,11 +30,11 @@
 #include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
 
 #include "mongo/base/checked_cast.h"
-#include "mongo/db/concurrency/lock_state.h"
 #include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/recovery_unit_test_harness.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_cursor_helpers.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_record_store.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
@@ -42,6 +42,7 @@
 #include "mongo/unittest/death_test.h"
 #include "mongo/unittest/temp_dir.h"
 #include "mongo/unittest/unittest.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/clock_source_mock.h"
 
 namespace mongo {
@@ -57,22 +58,14 @@ public:
                   "",                     // .extraOpenOptions
                   1,                      // .cacheSizeMB
                   0,                      // .maxCacheOverflowFileSizeMB
-                  false,                  // .durable
                   false,                  // .ephemeral
-                  false,                  // .repair
-                  false                   // .readOnly
+                  false                   // .repair
           ) {
-        // Deliberately not calling _engine->startAsyncThreads() because it starts an asynchronous
-        // checkpointing thread that can interfere with unit tests manipulating checkpoints
-        // manually.
-        //
-        // Alternatively, we would have to start using wiredTigerGlobalOptions.checkpointDelaySecs
-        // to set a high enough value such that the async thread never runs during testing.
-
         repl::ReplicationCoordinator::set(
             getGlobalServiceContext(),
             std::unique_ptr<repl::ReplicationCoordinator>(new repl::ReplicationCoordinatorMock(
                 getGlobalServiceContext(), repl::ReplSettings())));
+        _engine.notifyStartupComplete();
     }
 
     ~WiredTigerRecoveryUnitHarnessHelper() {}
@@ -84,10 +77,16 @@ public:
     virtual std::unique_ptr<RecordStore> createRecordStore(OperationContext* opCtx,
                                                            const std::string& ns) final {
         std::string ident = ns;
+        NamespaceString nss(ns);
         std::string uri = WiredTigerKVEngine::kTableUriPrefix + ns;
-        const bool prefixed = false;
-        StatusWith<std::string> result = WiredTigerRecordStore::generateCreateString(
-            kWiredTigerEngineName, ns, CollectionOptions(), "", prefixed);
+        StatusWith<std::string> result =
+            WiredTigerRecordStore::generateCreateString(kWiredTigerEngineName,
+                                                        nss,
+                                                        ident,
+                                                        CollectionOptions(),
+                                                        "",
+                                                        KeyFormat::Long,
+                                                        WiredTigerUtil::useTableLogging(nss));
         ASSERT_TRUE(result.isOK());
         std::string config = result.getValue();
 
@@ -96,22 +95,22 @@ public:
             WiredTigerRecoveryUnit* ru =
                 checked_cast<WiredTigerRecoveryUnit*>(opCtx->recoveryUnit());
             WT_SESSION* s = ru->getSession()->getSession();
-            invariantWTOK(s->create(s, uri.c_str(), config.c_str()));
+            invariantWTOK(s->create(s, uri.c_str(), config.c_str()), s);
             uow.commit();
         }
 
         WiredTigerRecordStore::Params params;
-        params.ns = ns;
+        params.nss = nss;
         params.ident = ident;
         params.engineName = kWiredTigerEngineName;
         params.isCapped = false;
+        params.keyFormat = KeyFormat::Long;
+        params.overwrite = true;
         params.isEphemeral = false;
-        params.cappedMaxSize = -1;
-        params.cappedMaxDocs = -1;
-        params.cappedCallback = nullptr;
+        params.isLogged = WiredTigerUtil::useTableLogging(nss);
         params.sizeStorer = nullptr;
         params.tracksSizeAdjustments = true;
-        params.isReadOnly = false;
+        params.forceUpdateWithFullDocument = false;
 
         auto ret = std::make_unique<StandardWiredTigerRecordStore>(&_engine, opCtx, params);
         ret->postConstructorInit(opCtx);
@@ -134,7 +133,6 @@ std::unique_ptr<RecoveryUnitHarnessHelper> makeWTRUHarnessHelper() {
 
 MONGO_INITIALIZER(RegisterHarnessFactory)(InitializerContext* const) {
     mongo::registerRecoveryUnitHarnessHelperFactory(makeWTRUHarnessHelper);
-    return Status::OK();
 }
 
 class WiredTigerRecoveryUnitTestFixture : public unittest::Test {
@@ -154,8 +152,9 @@ public:
 
     void getCursor(WiredTigerRecoveryUnit* ru, WT_CURSOR** cursor) {
         WT_SESSION* wt_session = ru->getSession()->getSession();
-        invariantWTOK(wt_session->create(wt_session, wt_uri, wt_config));
-        invariantWTOK(wt_session->open_cursor(wt_session, wt_uri, nullptr, nullptr, cursor));
+        invariantWTOK(wt_session->create(wt_session, wt_uri, wt_config), wt_session);
+        invariantWTOK(wt_session->open_cursor(wt_session, wt_uri, nullptr, nullptr, cursor),
+                      wt_session);
     }
 
     void setUp() override {
@@ -175,17 +174,26 @@ public:
 
 private:
     const char* wt_uri = "table:prepare_transaction";
-    const char* wt_config = "key_format=S,value_format=S";
+    const char* wt_config = "key_format=S,value_format=S,log=(enabled=false)";
 };
 
 TEST_F(WiredTigerRecoveryUnitTestFixture, SetReadSource) {
+    // Storage engine operations require at least Global IS.
+    Lock::GlobalLock lk(clientAndCtx1.second.get(), MODE_IS);
     ru1->setTimestampReadSource(RecoveryUnit::ReadSource::kProvided, Timestamp(1, 1));
     ASSERT_EQ(RecoveryUnit::ReadSource::kProvided, ru1->getTimestampReadSource());
-    ASSERT_EQ(Timestamp(1, 1), ru1->getPointInTimeReadTimestamp());
+    ASSERT_EQ(Timestamp(1, 1), ru1->getPointInTimeReadTimestamp(clientAndCtx1.second.get()));
 }
 
 TEST_F(WiredTigerRecoveryUnitTestFixture, NoOverlapReadSource) {
     OperationContext* opCtx1 = clientAndCtx1.second.get();
+    OperationContext* opCtx2 = clientAndCtx2.second.get();
+
+    // Hold the global locks throughout the test to avoid having the global lock destructor
+    // prematurely abandon snapshots.
+    Lock::GlobalLock globalLock1(opCtx1, MODE_IX);
+    Lock::GlobalLock globalLock2(opCtx2, MODE_IX);
+
     std::unique_ptr<RecordStore> rs(harnessHelper->createRecordStore(opCtx1, "a.b"));
 
     const std::string str = str::stream() << "test";
@@ -219,12 +227,12 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, NoOverlapReadSource) {
         // Start, but do not commit a transaction with opCtx2. This sets a timestamp at ts2, which
         // creates a hole. kNoOverlap, which is a function of all_durable, will only be able to read
         // at the time immediately before.
-        OperationContext* opCtx2 = clientAndCtx2.second.get();
         WriteUnitOfWork wuow(opCtx2);
         StatusWith<RecordId> res =
             rs->insertRecord(opCtx2, str.c_str(), str.size() + 1, Timestamp());
         ASSERT_OK(opCtx2->recoveryUnit()->setTimestamp(ts2));
         ASSERT_OK(res);
+        rid2 = res.getValue();
 
         // While holding open a transaction with opCtx2, perform an insert at ts3 with opCtx1. This
         // creates a "hole".
@@ -254,7 +262,6 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, NoOverlapReadSource) {
         ASSERT_FALSE(rs->findRecord(opCtx1, rid3, &unused));
 
         wuow.commit();
-        rid2 = res.getValue();
     }
 
     // Now that the hole has been closed, kNoOverlap should see all 3 records.
@@ -265,62 +272,20 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, NoOverlapReadSource) {
     ASSERT_TRUE(rs->findRecord(opCtx1, rid3, &unused));
 }
 
-TEST_F(WiredTigerRecoveryUnitTestFixture, CreateAndCheckForCachePressure) {
-    int time = 1;
-
-    // Reconfigure the size of the cache to be very small so that building cache pressure is fast.
-    WiredTigerKVEngine* engine = harnessHelper->getEngine();
-    std::string cacheSizeReconfig = "cache_size=1MB";
-    ASSERT_EQ(engine->reconfigure(cacheSizeReconfig.c_str()), 0);
-
-    OperationContext* opCtx = clientAndCtx1.second.get();
-    std::unique_ptr<RecordStore> rs(harnessHelper->createRecordStore(opCtx, "a.b"));
-
-    // Insert one document so that we can then update it in a loop to create cache pressure.
-    // Note: inserts will not create cache pressure.
-    WriteUnitOfWork wu(opCtx);
-    ASSERT_OK(ru1->setTimestamp(Timestamp(time++)));
-    std::string str = str::stream() << "foobarbaz";
-    StatusWith<RecordId> ress = rs->insertRecord(opCtx, str.c_str(), str.size() + 1, Timestamp());
-    ASSERT_OK(ress.getStatus());
-    auto recordId = ress.getValue();
-    wu.commit();
-
-    for (int j = 0; j < 1000; ++j) {
-        // Once we hit the cache pressure threshold, i.e. have successfully created cache pressure
-        // that is detectable, we are done.
-        if (engine->isCacheUnderPressure(opCtx)) {
-            invariant(j != 0);
-            break;
-        }
-
-        try {
-            WriteUnitOfWork wuow(opCtx);
-            ASSERT_OK(ru1->setTimestamp(Timestamp(time++)));
-            std::string s = str::stream()
-                << "abcbcdcdedefefgfghghihijijkjklklmlmnmnomopopqpqrqrsrststutuv" << j;
-            ASSERT_OK(rs->updateRecord(opCtx, recordId, s.c_str(), s.size() + 1));
-            wuow.commit();
-        } catch (const DBException& ex) {
-            invariant(ex.toStatus().code() == ErrorCodes::WriteConflict);
-        }
-    }
-}
-
 TEST_F(WiredTigerRecoveryUnitTestFixture,
        LocalReadOnADocumentBeingPreparedWithoutIgnoringPreparedTriggersPrepareConflict) {
     // Prepare but don't commit a transaction
-    ru1->beginUnitOfWork(clientAndCtx1.second.get());
+    ru1->beginUnitOfWork(clientAndCtx1.second->readOnly());
     WT_CURSOR* cursor;
     getCursor(ru1, &cursor);
     cursor->set_key(cursor, "key");
     cursor->set_value(cursor, "value");
-    invariantWTOK(cursor->insert(cursor));
+    invariantWTOK(wiredTigerCursorInsert(clientAndCtx1.second.get(), cursor), cursor->session);
     ru1->setPrepareTimestamp({1, 1});
     ru1->prepareUnitOfWork();
 
     // The transaction read default enforces prepare conflicts and triggers a WT_PREPARE_CONFLICT.
-    ru2->beginUnitOfWork(clientAndCtx2.second.get());
+    ru2->beginUnitOfWork(clientAndCtx2.second->readOnly());
     getCursor(ru2, &cursor);
     cursor->set_key(cursor, "key");
     int ret = cursor->search(cursor);
@@ -333,18 +298,18 @@ TEST_F(WiredTigerRecoveryUnitTestFixture,
 TEST_F(WiredTigerRecoveryUnitTestFixture,
        LocalReadOnADocumentBeingPreparedDoesntTriggerPrepareConflict) {
     // Prepare but don't commit a transaction
-    ru1->beginUnitOfWork(clientAndCtx1.second.get());
+    ru1->beginUnitOfWork(clientAndCtx1.second->readOnly());
     WT_CURSOR* cursor;
     getCursor(ru1, &cursor);
     cursor->set_key(cursor, "key");
     cursor->set_value(cursor, "value");
-    invariantWTOK(cursor->insert(cursor));
+    invariantWTOK(wiredTigerCursorInsert(clientAndCtx1.second.get(), cursor), cursor->session);
     ru1->setPrepareTimestamp({1, 1});
     ru1->prepareUnitOfWork();
 
     // A transaction that chooses to ignore prepare conflicts does not see the record instead of
     // returning a prepare conflict.
-    ru2->beginUnitOfWork(clientAndCtx2.second.get());
+    ru2->beginUnitOfWork(clientAndCtx2.second->readOnly());
     ru2->setPrepareConflictBehavior(PrepareConflictBehavior::kIgnoreConflicts);
     getCursor(ru2, &cursor);
     cursor->set_key(cursor, "key");
@@ -357,18 +322,18 @@ TEST_F(WiredTigerRecoveryUnitTestFixture,
 
 TEST_F(WiredTigerRecoveryUnitTestFixture, WriteAllowedWhileIgnorePrepareFalse) {
     // Prepare but don't commit a transaction
-    ru1->beginUnitOfWork(clientAndCtx1.second.get());
+    ru1->beginUnitOfWork(clientAndCtx1.second->readOnly());
     WT_CURSOR* cursor;
     getCursor(ru1, &cursor);
     cursor->set_key(cursor, "key1");
     cursor->set_value(cursor, "value1");
-    invariantWTOK(cursor->insert(cursor));
+    invariantWTOK(wiredTigerCursorInsert(clientAndCtx1.second.get(), cursor), cursor->session);
     ru1->setPrepareTimestamp({1, 1});
     ru1->prepareUnitOfWork();
 
     // A transaction that chooses to ignore prepare conflicts with kIgnoreConflictsAllowWrites does
     // not see the record
-    ru2->beginUnitOfWork(clientAndCtx2.second.get());
+    ru2->beginUnitOfWork(clientAndCtx2.second->readOnly());
     ru2->setPrepareConflictBehavior(PrepareConflictBehavior::kIgnoreConflictsAllowWrites);
 
     // The prepared write is not visible.
@@ -381,7 +346,7 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, WriteAllowedWhileIgnorePrepareFalse) {
     cursor->set_value(cursor, "value2");
 
     // The write is allowed.
-    invariantWTOK(cursor->insert(cursor));
+    invariantWTOK(wiredTigerCursorInsert(clientAndCtx2.second.get(), cursor), cursor->session);
 
     ru1->abortUnitOfWork();
     ru2->abortUnitOfWork();
@@ -389,21 +354,21 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, WriteAllowedWhileIgnorePrepareFalse) {
 
 TEST_F(WiredTigerRecoveryUnitTestFixture, WriteOnADocumentBeingPreparedTriggersWTRollback) {
     // Prepare but don't commit a transaction
-    ru1->beginUnitOfWork(clientAndCtx1.second.get());
+    ru1->beginUnitOfWork(clientAndCtx1.second->readOnly());
     WT_CURSOR* cursor;
     getCursor(ru1, &cursor);
     cursor->set_key(cursor, "key");
     cursor->set_value(cursor, "value");
-    invariantWTOK(cursor->insert(cursor));
+    invariantWTOK(wiredTigerCursorInsert(clientAndCtx1.second.get(), cursor), cursor->session);
     ru1->setPrepareTimestamp({1, 1});
     ru1->prepareUnitOfWork();
 
     // Another transaction with write triggers WT_ROLLBACK
-    ru2->beginUnitOfWork(clientAndCtx2.second.get());
+    ru2->beginUnitOfWork(clientAndCtx2.second->readOnly());
     getCursor(ru2, &cursor);
     cursor->set_key(cursor, "key");
     cursor->set_value(cursor, "value2");
-    int ret = cursor->insert(cursor);
+    int ret = wiredTigerCursorInsert(clientAndCtx2.second.get(), cursor);
     ASSERT_EQ(WT_ROLLBACK, ret);
 
     ru1->abortUnitOfWork();
@@ -658,19 +623,23 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, CommitTimestampAfterSetTimestampOnAbor
     ASSERT(!commitTs);
 }
 
-TEST_F(WiredTigerRecoveryUnitTestFixture, ReadOnceCursorsAreNotCached) {
+TEST_F(WiredTigerRecoveryUnitTestFixture, ReadOnceCursorsCached) {
     auto opCtx = clientAndCtx1.second.get();
+
+    // Hold the global lock throughout the test to avoid having the global lock destructor
+    // prematurely abandon snapshots.
+    Lock::GlobalLock globalLock(opCtx, MODE_IX);
     auto ru = WiredTigerRecoveryUnit::get(opCtx);
 
     std::unique_ptr<RecordStore> rs(harnessHelper->createRecordStore(opCtx, "test.read_once"));
     auto uri = dynamic_cast<WiredTigerRecordStore*>(rs.get())->getURI();
 
     // Insert a record.
-    ru->beginUnitOfWork(opCtx);
+    WriteUnitOfWork wuow(opCtx);
     StatusWith<RecordId> s = rs->insertRecord(opCtx, "data", 4, Timestamp());
     ASSERT_TRUE(s.isOK());
     ASSERT_EQUALS(1, rs->numRecords(opCtx));
-    ru->commitUnitOfWork();
+    wuow.commit();
 
     // Test 1: A normal read should create a new cursor and release it into the session cache.
 
@@ -688,7 +657,8 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, ReadOnceCursorsAreNotCached) {
 
     ru->abandonSnapshot();
 
-    // Test 2: A read-once operation should create a new cursor and immediately close it when done.
+    // Test 2: A read-once operation should create a new cursor because it has a different
+    // configuration. This will be released into the cache.
 
     ru->setReadOnce(true);
 
@@ -696,18 +666,96 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, ReadOnceCursorsAreNotCached) {
     ru->getSession()->closeAllCursors(uri);
     cachedCursorsBefore = ru->getSession()->cachedCursors();
 
-    // The subsequent read operation will use a read_once cursor, which will not be from the cache,
-    // and will not be released into the cache.
+    // The subsequent read operation will create a new read_once cursor and release into the cache.
     ASSERT_TRUE(rs->findRecord(opCtx, s.getValue(), &rd));
 
-    // No new cursors should have been released into the cache.
-    ASSERT_EQ(ru->getSession()->cachedCursors(), cachedCursorsBefore);
+    // A new cursor should have been released into the cache.
+    ASSERT_GT(ru->getSession()->cachedCursors(), cachedCursorsBefore);
     // All opened cursors are closed.
     ASSERT_EQ(0, ru->getSession()->cursorsOut());
 
     ASSERT(ru->getReadOnce());
 }
 
+TEST_F(WiredTigerRecoveryUnitTestFixture, CacheMixedOverwrite) {
+    auto opCtx = clientAndCtx1.second.get();
+    std::unique_ptr<RecordStore> rs(harnessHelper->createRecordStore(opCtx, "test.A"));
+    auto uri = dynamic_cast<WiredTigerRecordStore*>(rs.get())->getURI();
+
+    // Hold the global lock throughout the test to avoid having the global lock destructor
+    // prematurely abandon snapshots.
+    Lock::GlobalLock globalLock(opCtx, MODE_IX);
+    auto ru = WiredTigerRecoveryUnit::get(opCtx);
+
+    // Close all cached cursors to establish a 'before' state.
+    auto session = ru->getSession();
+    ru->getSession()->closeAllCursors(uri);
+    int cachedCursorsBefore = ru->getSession()->cachedCursors();
+
+    // Use a large, unused table ID for this test to ensure we don't collide with any other table
+    // ids.
+    int tableId = 999999999;
+    WT_CURSOR* cursor;
+
+    // Expect no cached cursors.
+    {
+        auto config = "";
+        cursor = session->getCachedCursor(tableId, config);
+        ASSERT_FALSE(cursor);
+
+        cursor = session->getNewCursor(uri, config);
+        ASSERT(cursor);
+        session->releaseCursor(tableId, cursor, config);
+        ASSERT_GT(session->cachedCursors(), cachedCursorsBefore);
+    }
+
+    cachedCursorsBefore = session->cachedCursors();
+
+    // Use a different overwrite setting, expect no cached cursors.
+    {
+        auto config = "overwrite=false";
+        cursor = session->getCachedCursor(tableId, config);
+        ASSERT_FALSE(cursor);
+
+        cursor = session->getNewCursor(uri, config);
+        ASSERT(cursor);
+        session->releaseCursor(tableId, cursor, config);
+        ASSERT_GT(session->cachedCursors(), cachedCursorsBefore);
+    }
+
+    cachedCursorsBefore = session->cachedCursors();
+
+    // Expect cursors to be cached.
+    {
+        auto config = "";
+        cursor = session->getCachedCursor(tableId, config);
+        ASSERT(cursor);
+        session->releaseCursor(tableId, cursor, config);
+        ASSERT_EQ(session->cachedCursors(), cachedCursorsBefore);
+    }
+
+    // Expect cursors to be cached.
+    {
+        auto config = "overwrite=false";
+        cursor = session->getCachedCursor(tableId, config);
+        ASSERT(cursor);
+        session->releaseCursor(tableId, cursor, config);
+        ASSERT_EQ(session->cachedCursors(), cachedCursorsBefore);
+    }
+
+    // Use yet another cursor config, and expect no cursors to be cached.
+    {
+        auto config = "overwrite=true";
+        cursor = session->getCachedCursor(tableId, config);
+        ASSERT_FALSE(cursor);
+
+        cursor = session->getNewCursor(uri, config);
+        ASSERT(cursor);
+        session->releaseCursor(tableId, cursor, config);
+        ASSERT_GT(session->cachedCursors(), cachedCursorsBefore);
+    }
+}
+
 TEST_F(WiredTigerRecoveryUnitTestFixture, CommitWithDurableTimestamp) {
     auto opCtx = clientAndCtx1.second.get();
     Timestamp ts1(3, 3);
@@ -735,6 +783,119 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, CommitWithoutDurableTimestamp) {
     }
 }
 
+TEST_F(WiredTigerRecoveryUnitTestFixture, MultiTimestampConstraintsInternalState) {
+    Timestamp ts1(1, 1);
+    Timestamp ts2(2, 2);
+
+    OperationContext* opCtx = clientAndCtx1.second.get();
+    ru1->beginUnitOfWork(opCtx->readOnly());
+
+    // Perform an non timestamped write.
+    WT_CURSOR* cursor;
+    getCursor(ru1, &cursor);
+    cursor->set_key(cursor, "key");
+    cursor->set_value(cursor, "value");
+    invariantWTOK(wiredTigerCursorInsert(opCtx, cursor), cursor->session);
+
+    // Perform a write at ts1.
+    cursor->set_key(cursor, "key2");
+    cursor->set_value(cursor, "value");
+    ASSERT_OK(ru1->setTimestamp(ts1));
+    invariantWTOK(wiredTigerCursorInsert(opCtx, cursor), cursor->session);
+
+    // Setting the timestamp again to the same value should not fail.
+    ASSERT_OK(ru1->setTimestamp(ts1));
+
+    // Committing the unit of work should reset the internal state for the multi timestamp
+    // constraint checks.
+    ru1->commitUnitOfWork();
+    ru1->beginUnitOfWork(opCtx->readOnly());
+
+    // Perform a write at ts2.
+    cursor->set_key(cursor, "key3");
+    cursor->set_value(cursor, "value");
+    ASSERT_OK(ru1->setTimestamp(ts2));
+    invariantWTOK(wiredTigerCursorInsert(opCtx, cursor), cursor->session);
+
+    ru1->commitUnitOfWork();
+}
+
+TEST_F(WiredTigerRecoveryUnitTestFixture, AbandonSnapshotAbortMode) {
+    ru1->setAbandonSnapshotMode(RecoveryUnit::AbandonSnapshotMode::kAbort);
+
+    OperationContext* opCtx = clientAndCtx1.second.get();
+    const char* const key = "key";
+
+    {
+        ru1->beginUnitOfWork(opCtx->readOnly());
+
+        WT_CURSOR* cursor;
+        getCursor(ru1, &cursor);
+        cursor->set_key(cursor, key);
+        cursor->set_value(cursor, "value");
+        invariantWTOK(wiredTigerCursorInsert(opCtx, cursor), cursor->session);
+
+        ru1->commitUnitOfWork();
+    }
+
+    // Create a cursor. We will check that once positioned, the cursor is reset by a call to
+    // abandonSnapshot() on the associated RecoveryUnit.
+    WT_CURSOR* cursor;
+    getCursor(ru1, &cursor);
+    cursor->set_key(cursor, key);
+    ASSERT_EQ(0, cursor->search(cursor));
+
+    ru1->abandonSnapshot();
+
+    // The WT transaction should have been aborted and the cursor reset.
+
+    // Advancing to the "next" record now that the cursor has been reset should give us the first
+    // record again.
+    ASSERT_EQ(0, cursor->next(cursor));
+
+    const char* returnedKey = nullptr;
+    ASSERT_EQ(0, cursor->get_key(cursor, &returnedKey));
+    ASSERT_EQ(0, strncmp(key, returnedKey, strlen(key)));
+}
+
+DEATH_TEST_REGEX_F(WiredTigerRecoveryUnitTestFixture,
+                   MultiTimestampConstraints,
+                   "Fatal assertion.*4877100") {
+    Timestamp ts1(1, 1);
+    Timestamp ts2(2, 2);
+
+    OperationContext* opCtx = clientAndCtx1.second.get();
+    ru1->beginUnitOfWork(opCtx->readOnly());
+
+    auto writeTest = [&]() {
+        // Perform an non timestamped write.
+        WT_CURSOR* cursor;
+        getCursor(ru1, &cursor);
+        cursor->set_key(cursor, "key");
+        cursor->set_value(cursor, "value");
+        invariantWTOK(wiredTigerCursorInsert(opCtx, cursor), cursor->session);
+
+        // Perform a write at ts1.
+        cursor->set_key(cursor, "key2");
+        cursor->set_value(cursor, "value");
+        ASSERT_OK(ru1->setTimestamp(ts1));
+        invariantWTOK(wiredTigerCursorInsert(opCtx, cursor), cursor->session);
+
+        // Setting the timestamp again to a different value should detect that we're trying to set
+        // multiple timestamps with the first write being non timestamped.
+        ASSERT_OK(ru1->setTimestamp(ts2));
+        ru1->commitUnitOfWork();
+    };
+
+    try {
+        writeTest();
+    } catch (WriteConflictException const&) {
+        // It's expected to get a WCE the first time we try this, due to the multi-timestamp
+        // constraint. We'll try again and it will fassert and print out extra debug info.
+    }
+    writeTest();
+}
+
 DEATH_TEST_F(WiredTigerRecoveryUnitTestFixture,
              SetDurableTimestampTwice,
              "Trying to reset durable timestamp when it was already set.") {
@@ -758,15 +919,14 @@ DEATH_TEST_F(WiredTigerRecoveryUnitTestFixture,
     }
 }
 
-TEST_F(WiredTigerRecoveryUnitTestFixture, MayNotChangeReadSourceWhilePinned) {
-    // Storage engine operations require at least Global IS.
-    auto opCtx = cc().makeOperationContext();
-    opCtx->swapLockState(std::make_unique<LockerImpl>(), WithLock::withoutLock());
-    Lock::GlobalLock lk(opCtx.get(), MODE_IS);
-    ru1->pinReadSource();
+DEATH_TEST_F(WiredTigerRecoveryUnitTestFixture,
+             MayNotChangeReadSourceWhilePinned,
+             "Cannot change ReadSource as it is pinned.") {
 
-    ASSERT_THROWS_CODE(
-        ru1->setTimestampReadSource(RecoveryUnit::ReadSource::kNoOverlap), DBException, 5863604);
+    // Storage engine operations require at least Global IS.
+    Lock::GlobalLock lk(clientAndCtx1.second.get(), MODE_IS);
+    ru1->pinReadSource();
+    ru1->setTimestampReadSource(RecoveryUnit::ReadSource::kNoOverlap);
 }
 
 }  // namespace
