@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include "mongo/platform/basic.h"
 
@@ -36,7 +35,6 @@
 #include <memory>
 
 #include "mongo/base/error_codes.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/global_settings.h"
 #include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/storage/journal_listener.h"
@@ -47,6 +45,9 @@
 #include "mongo/stdx/thread.h"
 #include "mongo/util/scopeguard.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
+
+
 namespace mongo {
 
 WiredTigerSession::WiredTigerSession(WT_CONNECTION* conn, uint64_t epoch, uint64_t cursorEpoch)
@@ -56,7 +57,7 @@ WiredTigerSession::WiredTigerSession(WT_CONNECTION* conn, uint64_t epoch, uint64
       _cursorGen(0),
       _cursorsOut(0),
       _idleExpireTime(Date_t::min()) {
-    invariantWTOK(conn->open_session(conn, nullptr, "isolation=snapshot", &_session));
+    invariantWTOK(conn->open_session(conn, nullptr, "isolation=snapshot", &_session), nullptr);
 }
 
 WiredTigerSession::WiredTigerSession(WT_CONNECTION* conn,
@@ -70,12 +71,12 @@ WiredTigerSession::WiredTigerSession(WT_CONNECTION* conn,
       _cursorGen(0),
       _cursorsOut(0),
       _idleExpireTime(Date_t::min()) {
-    invariantWTOK(conn->open_session(conn, nullptr, "isolation=snapshot", &_session));
+    invariantWTOK(conn->open_session(conn, nullptr, "isolation=snapshot", &_session), nullptr);
 }
 
 WiredTigerSession::~WiredTigerSession() {
     if (_session) {
-        invariantWTOK(_session->close(_session, nullptr));
+        invariantWTOK(_session->close(_session, nullptr), nullptr);
     }
 }
 
@@ -85,49 +86,48 @@ void _openCursor(WT_SESSION* session,
                  const char* config,
                  WT_CURSOR** cursorOut) {
     int ret = session->open_cursor(session, uri.c_str(), nullptr, config, cursorOut);
+    if (ret == 0) {
+        return;
+    }
+
+    auto status = wtRCToStatus(ret, session);
+
     if (ret == EBUSY) {
-        // This can only happen when trying to open a cursor on the oplog and it is currently locked
-        // by a verify or salvage, because we don't employ database locks to protect the oplog.
-        throw WriteConflictException();
+        // This may happen when there is an ongoing full validation, with a call to WT::verify.
+        // Other operations which may trigger this include salvage, rollback_to_stable, upgrade,
+        // alter, or if there is a bulk cursor open. Mongo (currently) does not run any of
+        // these operations concurrently with this code path, except for validation.
+
+        uassertStatusOK(status);
+    } else if (ret == ENOENT) {
+        uasserted(ErrorCodes::CursorNotFound,
+                  str::stream() << "Failed to open a WiredTiger cursor. Reason: " << status
+                                << ", uri: " << uri << ", config: " << config);
     }
 
-    if (ret != 0) {
-        auto status = wtRCToStatus(ret);
-        std::string cursorErrMsg = str::stream()
-            << "Failed to open a WiredTiger cursor. Reason: " << status << ", uri: " << uri
-            << ", config: " << config;
-
-        if (ret == ENOENT) {
-            uasserted(ErrorCodes::CursorNotFound, cursorErrMsg);
-        }
-
-        LOGV2_FATAL_NOTRACE(50882,
-                            "Failed to open WiredTiger cursor. This may be due to data corruption",
-                            "uri"_attr = uri,
-                            "config"_attr = config,
-                            "error"_attr = status,
-                            "message"_attr = kWTRepairMsg);
-    }
+    LOGV2_FATAL_NOTRACE(50882,
+                        "Failed to open WiredTiger cursor. This may be due to data corruption",
+                        "uri"_attr = uri,
+                        "config"_attr = config,
+                        "error"_attr = status,
+                        "message"_attr = kWTRepairMsg);
 }
 }  // namespace
 
-WT_CURSOR* WiredTigerSession::getCachedCursor(const std::string& uri,
-                                              uint64_t id,
-                                              const char* config) {
+WT_CURSOR* WiredTigerSession::getCachedCursor(uint64_t id, const std::string& config) {
     // Find the most recently used cursor
     for (CursorCache::iterator i = _cursors.begin(); i != _cursors.end(); ++i) {
-        if (i->_id == id) {
+        // Ensure that all properties of this cursor are identical to avoid mixing cursor
+        // configurations. Note that this uses an exact string match, so cursor configurations with
+        // parameters in different orders will not be considered equivalent.
+        if (i->_id == id && i->_config == config) {
             WT_CURSOR* c = i->_cursor;
             _cursors.erase(i);
             _cursorsOut++;
             return c;
         }
     }
-
-    WT_CURSOR* cursor = nullptr;
-    _openCursor(_session, uri, config, &cursor);
-    _cursorsOut++;
-    return cursor;
+    return nullptr;
 }
 
 WT_CURSOR* WiredTigerSession::getNewCursor(const std::string& uri, const char* config) {
@@ -137,7 +137,7 @@ WT_CURSOR* WiredTigerSession::getNewCursor(const std::string& uri, const char* c
     return cursor;
 }
 
-void WiredTigerSession::releaseCursor(uint64_t id, WT_CURSOR* cursor) {
+void WiredTigerSession::releaseCursor(uint64_t id, WT_CURSOR* cursor, const std::string& config) {
     // When releasing the cursor, we would want to check if the session cache is already in shutdown
     // and prevent the race condition that the shutdown starts after the check.
     WiredTigerSessionCache::BlockShutdown blockShutdown(_cache);
@@ -151,10 +151,10 @@ void WiredTigerSession::releaseCursor(uint64_t id, WT_CURSOR* cursor) {
     invariant(cursor);
     _cursorsOut--;
 
-    invariantWTOK(cursor->reset(cursor));
+    invariantWTOK(cursor->reset(cursor), _session);
 
     // Cursors are pushed to the front of the list and removed from the back
-    _cursors.push_front(WiredTigerCachedCursor(id, _cursorGen++, cursor));
+    _cursors.push_front(WiredTigerCachedCursor(id, _cursorGen++, cursor, config));
 
     // A negative value for wiredTigercursorCacheSize means to use hybrid caching.
     std::uint32_t cacheSize = abs(gWiredTigerCursorCacheSize.load());
@@ -162,7 +162,7 @@ void WiredTigerSession::releaseCursor(uint64_t id, WT_CURSOR* cursor) {
     while (!_cursors.empty() && _cursorGen - _cursors.back()._gen > cacheSize) {
         cursor = _cursors.back()._cursor;
         _cursors.pop_back();
-        invariantWTOK(cursor->close(cursor));
+        invariantWTOK(cursor->close(cursor), _session);
     }
 }
 
@@ -171,7 +171,7 @@ void WiredTigerSession::closeCursor(WT_CURSOR* cursor) {
     invariant(cursor);
     _cursorsOut--;
 
-    invariantWTOK(cursor->close(cursor));
+    invariantWTOK(cursor->close(cursor), _session);
 }
 
 void WiredTigerSession::closeAllCursors(const std::string& uri) {
@@ -181,7 +181,7 @@ void WiredTigerSession::closeAllCursors(const std::string& uri) {
     for (auto i = _cursors.begin(); i != _cursors.end();) {
         WT_CURSOR* cursor = i->_cursor;
         if (cursor && (all || uri == cursor->uri)) {
-            invariantWTOK(cursor->close(cursor));
+            invariantWTOK(cursor->close(cursor), _session);
             i = _cursors.erase(i);
         } else
             ++i;
@@ -197,7 +197,7 @@ void WiredTigerSession::closeCursorsForQueuedDrops(WiredTigerKVEngine* engine) {
     for (auto i = toDrop.begin(); i != toDrop.end(); i++) {
         WT_CURSOR* cursor = i->_cursor;
         if (cursor) {
-            invariantWTOK(cursor->close(cursor));
+            invariantWTOK(cursor->close(cursor), _session);
         }
     }
 }
@@ -282,13 +282,13 @@ void WiredTigerSessionCache::waitUntilDurable(OperationContext* opCtx,
     // WT_CONNECTION, i.e: replication is on) requires `forceCheckpoint` to be true and journaling
     // to be enabled.
     if (syncType == Fsync::kCheckpointStableTimestamp && getGlobalReplSettings().usingReplSets()) {
-        invariant(_engine->isDurable());
+        invariant(!isEphemeral());
     }
 
     // When forcing a checkpoint with journaling enabled, don't synchronize with other
     // waiters, as a log flush is much cheaper than a full checkpoint.
     if ((syncType == Fsync::kCheckpointStableTimestamp || syncType == Fsync::kCheckpointAll) &&
-        _engine->isDurable()) {
+        !isEphemeral()) {
         UniqueWiredTigerSession session = getSession();
         WT_SESSION* s = session->getSession();
         {
@@ -310,10 +310,10 @@ void WiredTigerSessionCache::waitUntilDurable(OperationContext* opCtx,
 
             auto config = syncType == Fsync::kCheckpointStableTimestamp ? "use_timestamp=true"
                                                                         : "use_timestamp=false";
-            invariantWTOK(s->checkpoint(s, config));
+            invariantWTOK(s->checkpoint(s, config), s);
 
             if (token) {
-                journalListener->onDurable(token.get());
+                journalListener->onDurable(token.value());
             }
         }
         LOGV2_DEBUG(22418, 4, "created checkpoint (forced)");
@@ -352,20 +352,23 @@ void WiredTigerSessionCache::waitUntilDurable(OperationContext* opCtx,
     // Initialize on first use.
     if (!_waitUntilDurableSession) {
         invariantWTOK(
-            _conn->open_session(_conn, nullptr, "isolation=snapshot", &_waitUntilDurableSession));
+            _conn->open_session(_conn, nullptr, "isolation=snapshot", &_waitUntilDurableSession),
+            nullptr);
     }
 
     // Use the journal when available, or a checkpoint otherwise.
-    if (_engine && _engine->isDurable()) {
-        invariantWTOK(_waitUntilDurableSession->log_flush(_waitUntilDurableSession, "sync=on"));
+    if (!isEphemeral()) {
+        invariantWTOK(_waitUntilDurableSession->log_flush(_waitUntilDurableSession, "sync=on"),
+                      _waitUntilDurableSession);
         LOGV2_DEBUG(22419, 4, "flushed journal");
     } else {
-        invariantWTOK(_waitUntilDurableSession->checkpoint(_waitUntilDurableSession, nullptr));
+        invariantWTOK(_waitUntilDurableSession->checkpoint(_waitUntilDurableSession, nullptr),
+                      _waitUntilDurableSession);
         LOGV2_DEBUG(22420, 4, "created checkpoint");
     }
 
     if (token) {
-        journalListener->onDurable(token.get());
+        journalListener->onDurable(token.value());
     }
 }
 
@@ -505,7 +508,7 @@ void WiredTigerSessionCache::releaseSession(WiredTigerSession* session) {
         uint64_t range;
         // This checks that we are only caching idle sessions and not something which might hold
         // locks or otherwise prevent truncation.
-        invariantWTOK(ss->transaction_pinned_range(ss, &range));
+        invariantWTOK(ss->transaction_pinned_range(ss, &range), ss);
         invariant(range == 0);
 
         // Release resources in the session we're about to cache.
@@ -514,7 +517,7 @@ void WiredTigerSessionCache::releaseSession(WiredTigerSession* session) {
         if (gWiredTigerCursorCacheSize.load() < 0) {
             session->closeAllCursors("");
         }
-        invariantWTOK(ss->reset(ss));
+        invariantWTOK(ss->reset(ss), ss);
     }
 
     // If the cursor epoch has moved on, close all cursors in the session.
