@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include "mongo/platform/basic.h"
 
@@ -36,14 +35,19 @@
 #include <limits>
 
 #include <boost/filesystem.hpp>
+#include <boost/filesystem/fstream.hpp>
 #include <boost/filesystem/path.hpp>
-#include <pcrecpp.h>
 
 #include "mongo/base/simple_string_data_comparator.h"
 #include "mongo/bson/bsonobjbuilder.h"
-#include "mongo/db/concurrency/write_conflict_exception.h"
-#include "mongo/db/snapshot_window_options.h"
+#include "mongo/bson/json.h"
+#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/exception_util_gen.h"
+#include "mongo/db/global_settings.h"
+#include "mongo/db/server_options_general_gen.h"
+#include "mongo/db/snapshot_window_options_gen.h"
 #include "mongo/db/storage/storage_file_util.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_global_options.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_parameters_gen.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
@@ -54,12 +58,21 @@
 #include "mongo/util/scopeguard.h"
 #include "mongo/util/static_immortal.h"
 #include "mongo/util/str.h"
+#include "mongo/util/testing_proctor.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kWiredTiger
+
+
+// From src/third_party/wiredtiger/src/include/txn.h
+#define WT_TXN_ROLLBACK_REASON_OLDEST_FOR_EVICTION \
+    "oldest pinned transaction ID rolled back for eviction"
 namespace mongo {
 
 namespace {
 
 const std::string kTableChecksFileName = "_wt_table_checks";
+const std::string kTableExtension = ".wt";
+const std::string kWiredTigerBackupFile = "WiredTiger.backup";
 
 /**
  * Removes the 'kTableChecksFileName' file in the dbpath, if it exists.
@@ -83,23 +96,68 @@ void removeTableChecksFile() {
     }
 }
 
+void setTableWriteTimestampAssertion(WiredTigerSessionCache* sessionCache,
+                                     const std::string& uri,
+                                     bool on) {
+    const std::string setting = on ? "assert=(write_timestamp=on)" : "assert=(write_timestamp=off)";
+    LOGV2_DEBUG(6003700,
+                1,
+                "Changing table write timestamp assertion settings",
+                "uri"_attr = uri,
+                "writeTimestampAssertionOn"_attr = on);
+    auto status = sessionCache->getKVEngine()->alterMetadata(uri, setting);
+    if (!status.isOK()) {
+        auto sessionPtr = sessionCache->getSession();
+        LOGV2_FATAL(
+            6003701,
+            "Failed to update write timestamp assertion setting",
+            "uri"_attr = uri,
+            "writeTimestampAssertionOn"_attr = on,
+            "error"_attr = status.code(),
+            "metadata"_attr =
+                redact(WiredTigerUtil::getMetadataCreate(sessionPtr->getSession(), uri).getValue()),
+            "message"_attr = status.reason());
+    }
+}
+
 }  // namespace
 
 using std::string;
 
-Status wtRCToStatus_slow(int retCode, const char* prefix) {
+bool wasRollbackReasonCachePressure(WT_SESSION* session) {
+    if (session) {
+        const auto reason = session->get_rollback_reason(session);
+        if (reason) {
+            return strncmp(WT_TXN_ROLLBACK_REASON_OLDEST_FOR_EVICTION,
+                           reason,
+                           sizeof(WT_TXN_ROLLBACK_REASON_OLDEST_FOR_EVICTION)) == 0;
+        }
+    }
+    return false;
+}
+
+Status wtRCToStatus_slow(int retCode, WT_SESSION* session, StringData prefix) {
     if (retCode == 0)
         return Status::OK();
 
     if (retCode == WT_ROLLBACK) {
-        throw WriteConflictException();
+        if (gEnableTemporarilyUnavailableExceptions.load() &&
+            wasRollbackReasonCachePressure(session)) {
+            str::stream s;
+            if (!prefix.empty())
+                s << prefix << " ";
+            s << retCode << ": " << WT_TXN_ROLLBACK_REASON_OLDEST_FOR_EVICTION;
+            throwTemporarilyUnavailableException(s);
+        }
+
+        throwWriteConflictException(prefix);
     }
 
     // Don't abort on WT_PANIC when repairing, as the error will be handled at a higher layer.
     fassert(28559, retCode != WT_PANIC || storageGlobalParams.repair);
 
     str::stream s;
-    if (prefix)
+    if (!prefix.empty())
         s << prefix << " ";
     s << retCode << ": " << wiredtiger_strerror(retCode);
 
@@ -109,6 +167,9 @@ Status wtRCToStatus_slow(int retCode, const char* prefix) {
     if (retCode == EMFILE) {
         return Status(ErrorCodes::TooManyFilesOpen, s);
     }
+    if (retCode == EBUSY) {
+        return Status(ErrorCodes::ObjectIsBusy, s);
+    }
 
     uassert(ErrorCodes::ExceededMemoryLimit, s, retCode != WT_CACHE_FULL);
 
@@ -148,12 +209,12 @@ StatusWith<std::string> _getMetadata(WT_CURSOR* cursor, StringData uri) {
         return StatusWith<std::string>(ErrorCodes::NoSuchKey,
                                        str::stream() << "Unable to find metadata for " << uri);
     } else if (ret != 0) {
-        return StatusWith<std::string>(wtRCToStatus(ret));
+        return StatusWith<std::string>(wtRCToStatus(ret, cursor->session));
     }
     const char* metadata = nullptr;
     ret = cursor->get_value(cursor, &metadata);
     if (ret != 0) {
-        return StatusWith<std::string>(wtRCToStatus(ret));
+        return StatusWith<std::string>(wtRCToStatus(ret, cursor->session));
     }
     invariant(metadata);
     return StatusWith<std::string>(metadata);
@@ -162,9 +223,9 @@ StatusWith<std::string> _getMetadata(WT_CURSOR* cursor, StringData uri) {
 
 StatusWith<std::string> WiredTigerUtil::getMetadataCreate(WT_SESSION* session, StringData uri) {
     WT_CURSOR* cursor;
-    invariantWTOK(session->open_cursor(session, "metadata:create", nullptr, "", &cursor));
+    invariantWTOK(session->open_cursor(session, "metadata:create", nullptr, "", &cursor), session);
     invariant(cursor);
-    ON_BLOCK_EXIT([cursor] { invariantWTOK(cursor->close(cursor)); });
+    ON_BLOCK_EXIT([cursor, session] { invariantWTOK(cursor->close(cursor), session); });
 
     return _getMetadata(cursor, uri);
 }
@@ -176,23 +237,27 @@ StatusWith<std::string> WiredTigerUtil::getMetadataCreate(OperationContext* opCt
 
     WT_CURSOR* cursor = nullptr;
     try {
-        cursor = session->getCachedCursor(
-            "metadata:create", WiredTigerSession::kMetadataCreateTableId, NULL);
+        const std::string metadataURI = "metadata:create";
+        cursor = session->getCachedCursor(WiredTigerSession::kMetadataCreateTableId, "");
+        if (!cursor) {
+            cursor = session->getNewCursor(metadataURI);
+        }
     } catch (const ExceptionFor<ErrorCodes::CursorNotFound>& ex) {
         LOGV2_FATAL_NOTRACE(51257, "Cursor not found", "error"_attr = ex);
     }
     invariant(cursor);
-    auto releaser = makeGuard(
-        [&] { session->releaseCursor(WiredTigerSession::kMetadataCreateTableId, cursor); });
+    ScopeGuard releaser = [&] {
+        session->releaseCursor(WiredTigerSession::kMetadataCreateTableId, cursor, "");
+    };
 
     return _getMetadata(cursor, uri);
 }
 
 StatusWith<std::string> WiredTigerUtil::getMetadata(WT_SESSION* session, StringData uri) {
     WT_CURSOR* cursor;
-    invariantWTOK(session->open_cursor(session, "metadata:", nullptr, "", &cursor));
+    invariantWTOK(session->open_cursor(session, "metadata:", nullptr, "", &cursor), session);
     invariant(cursor);
-    ON_BLOCK_EXIT([cursor] { invariantWTOK(cursor->close(cursor)); });
+    ON_BLOCK_EXIT([cursor, session] { invariantWTOK(cursor->close(cursor), session); });
 
     return _getMetadata(cursor, uri);
 }
@@ -203,13 +268,18 @@ StatusWith<std::string> WiredTigerUtil::getMetadata(OperationContext* opCtx, Str
     auto session = WiredTigerRecoveryUnit::get(opCtx)->getSessionNoTxn();
     WT_CURSOR* cursor = nullptr;
     try {
-        cursor = session->getCachedCursor("metadata:", WiredTigerSession::kMetadataTableId, NULL);
+        const std::string metadataURI = "metadata:";
+        cursor = session->getCachedCursor(WiredTigerSession::kMetadataTableId, "");
+        if (!cursor) {
+            cursor = session->getNewCursor(metadataURI);
+        }
     } catch (const ExceptionFor<ErrorCodes::CursorNotFound>& ex) {
         LOGV2_FATAL_NOTRACE(31293, "Cursor not found", "error"_attr = ex);
     }
     invariant(cursor);
-    auto releaser =
-        makeGuard([&] { session->releaseCursor(WiredTigerSession::kMetadataTableId, cursor); });
+    ScopeGuard releaser = [&] {
+        session->releaseCursor(WiredTigerSession::kMetadataTableId, cursor, "");
+    };
 
     return _getMetadata(cursor, uri);
 }
@@ -254,7 +324,7 @@ Status WiredTigerUtil::getApplicationMetadata(OperationContext* opCtx,
                 bob->appendBool(key, valueItem.val);
                 break;
             case WT_CONFIG_ITEM::WT_CONFIG_ITEM_NUM:
-                bob->appendIntOrLL(key, valueItem.val);
+                bob->appendNumber(key, static_cast<long long>(valueItem.val));
                 break;
             default:
                 bob->append(key, StringData(valueItem.str, valueItem.len));
@@ -262,7 +332,7 @@ Status WiredTigerUtil::getApplicationMetadata(OperationContext* opCtx,
         }
     }
     if (ret != WT_NOTFOUND) {
-        return wtRCToStatus(ret);
+        return wtRCToStatus(ret, nullptr);
     }
 
     return Status::OK();
@@ -338,7 +408,7 @@ StatusWith<int64_t> WiredTigerUtil::checkApplicationMetadataFormatVersion(Operat
 
 // static
 Status WiredTigerUtil::checkTableCreationOptions(const BSONElement& configElem) {
-    invariant(configElem.fieldNameStringData() == WiredTigerUtil::kConfigStringField);
+    invariant(configElem.fieldNameStringData() == "configString");
 
     if (configElem.type() != String) {
         return {ErrorCodes::TypeMismatch, "'configString' must be a string."};
@@ -353,8 +423,13 @@ Status WiredTigerUtil::checkTableCreationOptions(const BSONElement& configElem)
         return {ErrorCodes::FailedToParse, "malformed 'configString' value."};
     }
 
+    if (config.find("type=lsm") != std::string::npos) {
+        return {ErrorCodes::Error(6627201), "Configuration 'type=lsm' is not supported."};
+    }
+
     Status status = wtRCToStatus(
-        wiredtiger_config_validate(nullptr, &eventHandler, "WT_SESSION.create", config.rawData()));
+        wiredtiger_config_validate(nullptr, &eventHandler, "WT_SESSION.create", config.rawData()),
+        nullptr);
     if (!status.isOK()) {
         StringBuilder errorMsg;
         errorMsg << status.reason();
@@ -377,8 +452,11 @@ StatusWith<int64_t> WiredTigerUtil::getStatisticsValue(WT_SESSION* session,
     const char* cursorConfig = config.empty() ? nullptr : config.c_str();
     int ret = session->open_cursor(session, uri.c_str(), nullptr, cursorConfig, &cursor);
     if (ret != 0) {
+        // The numerical 'statisticsKey' can be located in the WT_STATS_* preprocessor macros in
+        // wiredtiger.h.
         return StatusWith<int64_t>(ErrorCodes::CursorNotFound,
                                    str::stream() << "unable to open cursor at URI " << uri
+                                                 << " for statistic: " << statisticsKey
                                                  << ". reason: " << wiredtiger_strerror(ret));
     }
     invariant(cursor);
@@ -419,6 +497,13 @@ int64_t WiredTigerUtil::getIdentSize(WT_SESSION* s, const std::string& uri) {
     return result.getValue();
 }
 
+int64_t WiredTigerUtil::getIdentReuseSize(WT_SESSION* s, const std::string& uri) {
+    auto result = WiredTigerUtil::getStatisticsValue(
+        s, "statistics:" + uri, "statistics=(fast)", WT_STAT_DSRC_BLOCK_REUSE_BYTES);
+    uassertStatusOK(result.getStatus());
+    return result.getValue();
+}
+
 size_t WiredTigerUtil::getCacheSizeMB(double requestedCacheSizeGB) {
     double cacheSizeMB;
     const double kMaxSizeCacheMB = 10 * 1000 * 1000;
@@ -442,7 +527,103 @@ size_t WiredTigerUtil::getCacheSizeMB(double requestedCacheSizeGB) {
     return static_cast<size_t>(cacheSizeMB);
 }
 
+logv2::LogSeverity getWTLOGV2SeverityLevel(const BSONObj& obj) {
+    const std::string field = "verbose_level_id";
+
+    if (!obj.hasField(field)) {
+        throw std::logic_error("The following field is missing: " + field);
+    }
+
+    BSONElement verbose_level_id_ele = obj[field];
+    if (!verbose_level_id_ele.isNumber()) {
+        throw std::logic_error("The value associated to " + field + " must be a number");
+    }
+
+    // Matching each WiredTiger verbosity level to the equivalent LOGV2 severity level.
+    switch (verbose_level_id_ele.Int()) {
+        case WT_VERBOSE_ERROR:
+            return logv2::LogSeverity::Error();
+        case WT_VERBOSE_WARNING:
+            return logv2::LogSeverity::Warning();
+        case WT_VERBOSE_NOTICE:
+            return logv2::LogSeverity::Info();
+        case WT_VERBOSE_INFO:
+            return logv2::LogSeverity::Log();
+        case WT_VERBOSE_DEBUG:
+            return logv2::LogSeverity::Debug(1);
+        default:
+            return logv2::LogSeverity::Log();
+    }
+}
+
+logv2::LogComponent getWTLOGV2Component(const BSONObj& obj) {
+    const std::string field = "category_id";
+
+    if (!obj.hasField(field)) {
+        throw std::logic_error("The following field is missing: " + field);
+    }
+
+    BSONElement category_id_ele = obj[field];
+    if (!category_id_ele.isNumber()) {
+        throw std::logic_error("The value associated to " + field + " must be a number");
+    }
+
+    switch (category_id_ele.Int()) {
+        case WT_VERB_BACKUP:
+            return logv2::LogComponent::kWiredTigerBackup;
+        case WT_VERB_CHECKPOINT:
+        case WT_VERB_CHECKPOINT_CLEANUP:
+        case WT_VERB_CHECKPOINT_PROGRESS:
+            return logv2::LogComponent::kWiredTigerCheckpoint;
+        case WT_VERB_COMPACT:
+        case WT_VERB_COMPACT_PROGRESS:
+            return logv2::LogComponent::kWiredTigerCompact;
+        case WT_VERB_EVICT:
+            return logv2::LogComponent::kWiredTigerEviction;
+        case WT_VERB_HS:
+        case WT_VERB_HS_ACTIVITY:
+            return logv2::LogComponent::kWiredTigerHS;
+        case WT_VERB_RECOVERY:
+        case WT_VERB_RECOVERY_PROGRESS:
+            return logv2::LogComponent::kWiredTigerRecovery;
+        case WT_VERB_RTS:
+            return logv2::LogComponent::kWiredTigerRTS;
+        case WT_VERB_SALVAGE:
+            return logv2::LogComponent::kWiredTigerSalvage;
+        case WT_VERB_TIERED:
+            return logv2::LogComponent::kWiredTigerTiered;
+        case WT_VERB_TIMESTAMP:
+            return logv2::LogComponent::kWiredTigerTimestamp;
+        case WT_VERB_TRANSACTION:
+            return logv2::LogComponent::kWiredTigerTransaction;
+        case WT_VERB_VERIFY:
+            return logv2::LogComponent::kWiredTigerVerify;
+        case WT_VERB_LOG:
+            return logv2::LogComponent::kWiredTigerWriteLog;
+        default:
+            return logv2::LogComponent::kWiredTiger;
+    }
+}
+
 namespace {
+
+void logWTErrorMessage(int id, int errorCode, const std::string& message) {
+    logv2::LogComponent component = logv2::LogComponent::kWiredTiger;
+    logv2::DynamicAttributes attr;
+    attr.add("error", errorCode);
+
+    try {
+        // Parse the WT JSON message string.
+        BSONObj obj = fromjson(message);
+        attr.add("message", obj);
+        component = getWTLOGV2Component(obj);
+    } catch (...) {
+        // Fall back to default behaviour.
+        attr.add("message", message);
+    }
+    LOGV2_ERROR_OPTIONS(id, logv2::LogOptions{component}, "WiredTiger error message", attr);
+}
+
 int mdb_handle_error_with_startup_suppression(WT_EVENT_HANDLER* handler,
                                               WT_SESSION* session,
                                               int errorCode,
@@ -468,11 +649,8 @@ int mdb_handle_error_with_startup_suppression(WT_EVENT_HANDLER* handler,
                 return 0;
             }
         }
-        LOGV2_ERROR(22435,
-                    "WiredTiger error ({error}) {message}",
-                    "WiredTiger error",
-                    "error"_attr = errorCode,
-                    "message"_attr = message);
+
+        logWTErrorMessage(22435, errorCode, message);
 
         // Don't abort on WT_PANIC when repairing, as the error will be handled at a higher layer.
         if (storageGlobalParams.repair) {
@@ -490,11 +668,7 @@ int mdb_handle_error(WT_EVENT_HANDLER* handler,
                      int errorCode,
                      const char* message) {
     try {
-        LOGV2_ERROR(22436,
-                    "WiredTiger error ({errorCode}) {message}",
-                    "WiredTiger error",
-                    "error"_attr = errorCode,
-                    "message"_attr = redact(message));
+        logWTErrorMessage(22436, errorCode, std::string(redact(message)));
 
         // Don't abort on WT_PANIC when repairing, as the error will be handled at a higher layer.
         if (storageGlobalParams.repair) {
@@ -508,8 +682,23 @@ int mdb_handle_error(WT_EVENT_HANDLER* handler,
 }
 
 int mdb_handle_message(WT_EVENT_HANDLER* handler, WT_SESSION* session, const char* message) {
+    logv2::DynamicAttributes attr;
+    logv2::LogSeverity severity = ::mongo::logv2::LogSeverity::Log();
+    logv2::LogOptions options = ::mongo::logv2::LogOptions{MONGO_LOGV2_DEFAULT_COMPONENT};
+
     try {
-        LOGV2(22430, "WiredTiger message", "message"_attr = redact(message));
+        try {
+            // Parse the WT JSON message string.
+            const BSONObj obj = fromjson(message);
+            severity = getWTLOGV2SeverityLevel(obj);
+            options = logv2::LogOptions{getWTLOGV2Component(obj)};
+            attr.add("message", redact(obj));
+        } catch (...) {
+            // Fall back to default behaviour.
+            attr.add("message", redact(message));
+        }
+
+        LOGV2_IMPL(22430, severity, options, "WiredTiger message", attr);
     } catch (...) {
         std::terminate();
     }
@@ -594,7 +783,7 @@ int WiredTigerUtil::verifyTable(OperationContext* opCtx,
     // Open a new session with custom error handlers.
     WT_CONNECTION* conn = WiredTigerRecoveryUnit::get(opCtx)->getSessionCache()->conn();
     WT_SESSION* session;
-    invariantWTOK(conn->open_session(conn, &eventHandler, nullptr, &session));
+    invariantWTOK(conn->open_session(conn, &eventHandler, nullptr, &session), nullptr);
     ON_BLOCK_EXIT([&] { session->close(session, ""); });
 
     // Do the verify. Weird parens prevent treating "verify" as a macro.
@@ -605,47 +794,47 @@ void WiredTigerUtil::notifyStartupComplete() {
     removeTableChecksFile();
 }
 
-bool WiredTigerUtil::useTableLogging(NamespaceString ns, bool replEnabled) {
-    if (!replEnabled) {
-        // All tables on standalones are logged.
+bool WiredTigerUtil::useTableLogging(const NamespaceString& nss) {
+    // We only turn off logging in the case of:
+    // 1) Replication is enabled (the typical deployment), or
+    // 2) We're running as a standalone with recoverFromOplogAsStandalone=true
+    const bool journalWritesBecauseStandalone = !getGlobalReplSettings().usingReplSets() &&
+        !repl::ReplSettings::shouldRecoverFromOplogAsStandalone();
+    if (journalWritesBecauseStandalone) {
         return true;
     }
 
+    // Don't make assumptions if there is no namespace string.
+    invariant(nss.size() > 0);
+
     // Of the replica set configurations:
-    if (ns.db() != "local") {
+    if (!nss.isLocal()) {
         // All replicated collections are not logged.
         return false;
     }
 
-    if (ns.coll() == "replset.minvalid") {
+    if (nss.coll() == "replset.minvalid") {
         // Of local collections, this is derived from the state of the data and therefore
         // not logged.
         return false;
     }
 
-    // The remainder of local gets logged. In particular, the oplog and user created collections.
+    // The remainder of local gets logged. In particular, the oplog and user created
+    // collections.
     return true;
 }
 
 Status WiredTigerUtil::setTableLogging(OperationContext* opCtx, const std::string& uri, bool on) {
-    // Try to close as much as possible to avoid EBUSY errors.
-    WiredTigerRecoveryUnit::get(opCtx)->getSession()->closeAllCursors(uri);
-    WiredTigerSessionCache* sessionCache = WiredTigerRecoveryUnit::get(opCtx)->getSessionCache();
-    sessionCache->closeAllCursors(uri);
-
-    // Use a dedicated session for alter operations to avoid transaction issues.
-    WiredTigerSession session(sessionCache->conn());
-    return setTableLogging(session.getSession(), uri, on);
-}
-
-Status WiredTigerUtil::setTableLogging(WT_SESSION* session, const std::string& uri, bool on) {
-    invariant(!storageGlobalParams.readOnly);
-
     if (gWiredTigerSkipTableLoggingChecksOnStartup) {
         LOGV2_DEBUG(5548302, 1, "Skipping table logging check", "uri"_attr = uri);
         return Status::OK();
     }
 
+    // Try to close as much as possible to avoid EBUSY errors.
+    WiredTigerRecoveryUnit::get(opCtx)->getSession()->closeAllCursors(uri);
+    WiredTigerSessionCache* sessionCache = WiredTigerRecoveryUnit::get(opCtx)->getSessionCache();
+    sessionCache->closeAllCursors(uri);
+
     const std::string setting = on ? "log=(enabled=true)" : "log=(enabled=false)";
 
     // This method does some "weak" parsing to see if the table is in the expected logging
@@ -655,7 +844,11 @@ Status WiredTigerUtil::setTableLogging(WT_SESSION* session, const std::string& u
     //
     // If the settings need to be changed (only expected at startup), the alter table call must
     // succeed.
-    std::string existingMetadata = getMetadataCreate(session, uri).getValue();
+    std::string existingMetadata;
+    {
+        auto session = sessionCache->getSession();
+        existingMetadata = getMetadataCreate(session->getSession(), uri).getValue();
+    }
     if (existingMetadata.find("log=(enabled=true)") != std::string::npos &&
         existingMetadata.find("log=(enabled=false)") != std::string::npos) {
         // Sanity check against a table having multiple logging specifications.
@@ -671,15 +864,26 @@ Status WiredTigerUtil::setTableLogging(WT_SESSION* session, const std::string& u
 
     LOGV2_DEBUG(
         22432, 1, "Changing table logging settings", "uri"_attr = uri, "loggingEnabled"_attr = on);
-    int ret = session->alter(session, uri.c_str(), setting.c_str());
-    if (ret) {
+    // Only alter the metadata once we're sure that we need to change the table settings, since
+    // WT_SESSION::alter may return EBUSY and require taking a checkpoint to make progress.
+    auto status = sessionCache->getKVEngine()->alterMetadata(uri, setting);
+    if (!status.isOK()) {
         LOGV2_FATAL(50756,
                     "Failed to update log setting",
                     "uri"_attr = uri,
                     "loggingEnabled"_attr = on,
-                    "error"_attr = ret,
+                    "error"_attr = status.code(),
                     "metadata"_attr = redact(existingMetadata),
-                    "message"_attr = session->strerror(session, ret));
+                    "message"_attr = status.reason());
+    }
+
+    // The write timestamp assertion setting only needs to be changed at startup. It will be turned
+    // on when logging is disabled, and off when logging is enabled.
+    if (TestingProctor::instance().isEnabled()) {
+        setTableWriteTimestampAssertion(sessionCache, uri, !on);
+    } else {
+        // Disables the assertion when the testing proctor is off.
+        setTableWriteTimestampAssertion(sessionCache, uri, false /* on */);
     }
 
     return Status::OK();
@@ -762,6 +966,95 @@ Status WiredTigerUtil::exportTableToBSON(WT_SESSION* session,
     return Status::OK();
 }
 
+StatusWith<std::string> WiredTigerUtil::generateImportString(const StringData& ident,
+                                                             const BSONObj& storageMetadata,
+                                                             const ImportOptions& importOptions) {
+    if (!storageMetadata.hasField(ident)) {
+        return Status(ErrorCodes::FailedToParse,
+                      str::stream() << "Missing the storage metadata for ident " << ident << " in "
+                                    << redact(storageMetadata));
+    }
+
+    if (storageMetadata.getField(ident).type() != BSONType::Object) {
+        return Status(ErrorCodes::FailedToParse,
+                      str::stream() << "The storage metadata for ident " << ident
+                                    << " is not of type object but is of type "
+                                    << storageMetadata.getField(ident).type() << " in "
+                                    << redact(storageMetadata));
+    }
+
+    const BSONObj& identMd = storageMetadata.getField(ident).Obj();
+    if (!identMd.hasField("tableMetadata") || !identMd.hasField("fileMetadata")) {
+        return Status(ErrorCodes::FailedToParse,
+                      str::stream()
+                          << "The storage metadata for ident " << ident
+                          << " is missing either the 'tableMetadata' or 'fileMetadata' field in "
+                          << redact(storageMetadata));
+    }
+
+    const BSONElement tableMetadata = identMd.getField("tableMetadata");
+    const BSONElement fileMetadata = identMd.getField("fileMetadata");
+
+    if (tableMetadata.type() != BSONType::String || fileMetadata.type() != BSONType::String) {
+        return Status(ErrorCodes::FailedToParse,
+                      str::stream() << "The storage metadata for ident " << ident
+                                    << " is not of type string for either the 'tableMetadata' or "
+                                       "'fileMetadata' field in "
+                                    << redact(storageMetadata));
+    }
+
+    std::stringstream ss;
+    ss << tableMetadata.String();
+    ss << ",import=(enabled=true,repair=false,";
+    if (importOptions.importTimestampRule == ImportOptions::ImportTimestampRule::kStable) {
+        ss << "compare_timestamp=stable,";
+    }
+    ss << "file_metadata=(" << fileMetadata.String() << "))";
+
+    return StatusWith<std::string>(ss.str());
+}
+
+std::string WiredTigerUtil::generateRestoreConfig() {
+    std::stringstream ss;
+    ss << "backup_restore_target=[";
+
+    const auto dbpath = boost::filesystem::path(storageGlobalParams.dbpath);
+    for (const auto& entry : boost::filesystem::recursive_directory_iterator(dbpath)) {
+        if (boost::filesystem::is_directory(entry)) {
+            continue;
+        }
+
+        if (entry.path().extension() != kTableExtension) {
+            continue;
+        }
+
+        // Skip WiredTiger metadata files with the ".wt" extension.
+        const std::string filename = entry.path().filename().string();
+        if (filename == "WiredTiger.wt" || filename == "WiredTigerHS.wt") {
+            continue;
+        }
+
+        boost::filesystem::path relativePath =
+            boost::filesystem::relative(entry.path(), dbpath).parent_path();
+        relativePath /= entry.path().stem();
+
+        ss << "\"table:" << relativePath.string() << "\",";
+    }
+
+    ss << "]";
+
+    return ss.str();
+}
+
+bool WiredTigerUtil::willRestoreFromBackup() {
+    if (!storageGlobalParams.restore) {
+        return false;
+    }
+
+    const auto dbpath = boost::filesystem::path(storageGlobalParams.dbpath);
+    return boost::filesystem::exists(dbpath / kWiredTigerBackupFile);
+}
+
 void WiredTigerUtil::appendSnapshotWindowSettings(WiredTigerKVEngine* engine,
                                                   WiredTigerSession* session,
                                                   BSONObjBuilder* bob) {
@@ -775,51 +1068,75 @@ void WiredTigerUtil::appendSnapshotWindowSettings(WiredTigerKVEngine* engine,
     const unsigned currentAvailableSnapshotWindow =
         stableTimestamp.getSecs() - oldestTimestamp.getSecs();
 
-    int64_t score = uassertStatusOK(WiredTigerUtil::getStatisticsValue(
-        session->getSession(), "statistics:", "", WT_STAT_CONN_CACHE_LOOKASIDE_SCORE));
-
-    auto totalNumberOfSnapshotTooOldErrors = snapshotWindowParams.snapshotTooOldErrorCount.load();
+    auto totalNumberOfSnapshotTooOldErrors = snapshotTooOldErrorCount.load();
 
     BSONObjBuilder settings(bob->subobjStart("snapshot-window-settings"));
-    settings.append("cache pressure percentage threshold",
-                    snapshotWindowParams.cachePressureThreshold.load());
-    settings.append("current cache pressure percentage", score);
     settings.append("total number of SnapshotTooOld errors", totalNumberOfSnapshotTooOldErrors);
-    settings.append("max target available snapshots window size in seconds",
-                    snapshotWindowParams.maxTargetSnapshotHistoryWindowInSeconds.load());
-    settings.append("target available snapshots window size in seconds",
-                    snapshotWindowParams.targetSnapshotHistoryWindowInSeconds.load());
-    settings.append("current available snapshots window size in seconds",
+    settings.append("minimum target snapshot window size in seconds",
+                    minSnapshotHistoryWindowInSeconds.load());
+    settings.append("current available snapshot window size in seconds",
                     static_cast<int>(currentAvailableSnapshotWindow));
     settings.append("latest majority snapshot timestamp available",
                     stableTimestamp.toStringPretty());
     settings.append("oldest majority snapshot timestamp available",
                     oldestTimestamp.toStringPretty());
+
+    std::map<std::string, Timestamp> pinnedTimestamps = engine->getPinnedTimestampRequests();
+    settings.append("pinned timestamp requests", static_cast<int>(pinnedTimestamps.size()));
+
+    Timestamp minPinned = Timestamp::max();
+    for (auto it : pinnedTimestamps) {
+        minPinned = std::min(minPinned, it.second);
+    }
+    settings.append("min pinned timestamp", minPinned);
 }
 
-void WiredTigerUtil::removeEncryptionFromConfigString(std::string* configString) {
-    static const StaticImmortal<pcrecpp::RE> encryptionOptsRegex(R"re(encryption=\([^\)]*\),?)re");
-    encryptionOptsRegex->GlobalReplace("", configString);
-}
+std::string WiredTigerUtil::generateWTVerboseConfiguration() {
+    // Mapping between LOGV2 WiredTiger components and their WiredTiger verbose setting counterpart.
+    static const StaticImmortal wtVerboseComponents = std::map<logv2::LogComponent, std::string>{
+        {logv2::LogComponent::kWiredTigerBackup, "backup"},
+        {logv2::LogComponent::kWiredTigerCheckpoint, "checkpoint"},
+        {logv2::LogComponent::kWiredTigerCompact, "compact"},
+        {logv2::LogComponent::kWiredTigerEviction, "evict"},
+        {logv2::LogComponent::kWiredTigerHS, "history_store"},
+        {logv2::LogComponent::kWiredTigerRecovery, "recovery"},
+        {logv2::LogComponent::kWiredTigerRTS, "rts"},
+        {logv2::LogComponent::kWiredTigerSalvage, "salvage"},
+        {logv2::LogComponent::kWiredTigerTiered, "tiered"},
+        {logv2::LogComponent::kWiredTigerTimestamp, "timestamp"},
+        {logv2::LogComponent::kWiredTigerTransaction, "transaction"},
+        {logv2::LogComponent::kWiredTigerVerify, "verify"},
+        {logv2::LogComponent::kWiredTigerWriteLog, "log"},
+    };
 
-// static
-BSONObj WiredTigerUtil::getSanitizedStorageOptionsForSecondaryReplication(const BSONObj& options) {
-    // Storage options may contain settings for non-WiredTiger storage engines (e.g. inMemory).
-    // We should leave these settings intact.
-    if (auto wtElem = options[kWiredTigerEngineName]) {
-        BSONObj wtObj = wtElem.Obj();
-        if (auto configStringElem = wtObj.getField(kConfigStringField)) {
-            auto configString = configStringElem.String();
-            removeEncryptionFromConfigString(&configString);
-            // Return a new BSONObj with the configString field sanitized.
-            return options.addField(
-                BSON(kWiredTigerEngineName << wtObj.addField(
-                         BSON(WiredTigerUtil::kConfigStringField << configString).firstElement()))
-                    .firstElement());
-        }
+    str::stream cfg;
+
+    // Define the verbose level for each component.
+    cfg << "verbose=[";
+
+    // Enable WiredTiger progress messages.
+    cfg << "recovery_progress:1,checkpoint_progress:1,compact_progress:1";
+
+    // Process each LOGV2 WiredTiger component and set the desired verbosity level.
+    for (const auto& [component, componentStr] : *wtVerboseComponents) {
+        auto severity =
+            logv2::LogManager::global().getGlobalSettings().getMinimumLogSeverity(component);
+
+        cfg << ",";
+
+        int level;
+        if (severity.toInt() >= logv2::LogSeverity::Debug(2).toInt())
+            level = WT_VERBOSE_DEBUG;
+        else
+            level = WT_VERBOSE_INFO;
+
+        cfg << componentStr << ":" << level;
     }
 
-    return options;
+    cfg << "]";
+
+    return cfg;
 }
 
+
 }  // namespace mongo
