@@ -27,6 +27,7 @@
  *    it in the license file.
  */
 
+
 #include "mongo/platform/basic.h"
 
 #include <sstream>
@@ -34,15 +35,18 @@
 
 #include "mongo/base/string_data.h"
 #include "mongo/db/operation_context_noop.h"
-#include "mongo/db/storage/kv/kv_prefix.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_oplog_manager.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
+#include "mongo/unittest/log_test.h"
 #include "mongo/unittest/temp_dir.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/system_clock_source.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
+
+
 namespace mongo {
 
 using std::string;
@@ -50,14 +54,17 @@ using std::stringstream;
 
 class WiredTigerConnection {
 public:
-    WiredTigerConnection(StringData dbpath, StringData extraStrings) : _conn(nullptr) {
+    WiredTigerConnection(StringData dbpath,
+                         StringData extraStrings,
+                         WT_EVENT_HANDLER* eventHandler = nullptr)
+        : _conn(nullptr) {
         std::stringstream ss;
         ss << "create,";
         ss << extraStrings;
         string config = ss.str();
         _fastClockSource = std::make_unique<SystemClockSource>();
-        int ret = wiredtiger_open(dbpath.toString().c_str(), nullptr, config.c_str(), &_conn);
-        ASSERT_OK(wtRCToStatus(ret));
+        int ret = wiredtiger_open(dbpath.toString().c_str(), eventHandler, config.c_str(), &_conn);
+        ASSERT_OK(wtRCToStatus(ret, nullptr));
         ASSERT(_conn);
     }
     ~WiredTigerConnection() {
@@ -77,9 +84,12 @@ private:
 
 class WiredTigerUtilHarnessHelper {
 public:
-    WiredTigerUtilHarnessHelper(StringData extraStrings)
+    explicit WiredTigerUtilHarnessHelper(StringData extraStrings,
+                                         WiredTigerEventHandler* eventHandler = nullptr)
         : _dbpath("wt_test"),
-          _connection(_dbpath.path(), extraStrings),
+          _connection(_dbpath.path(),
+                      extraStrings,
+                      eventHandler == nullptr ? nullptr : eventHandler->getWtEventHandler()),
           _sessionCache(_connection.getConnection(), _connection.getClockSource()) {}
 
 
@@ -128,7 +138,7 @@ protected:
     void createSession(const char* config) {
         WT_SESSION* wtSession =
             WiredTigerRecoveryUnit::get(_opCtx.get())->getSession()->getSession();
-        ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, getURI(), config)));
+        ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, getURI(), config), wtSession));
     }
 
 private:
@@ -307,7 +317,7 @@ TEST(WiredTigerUtilTest, GetStatisticsValueStatisticsDisabled) {
                                         harnessHelper.getOplogManager());
     WiredTigerSession* session = recoveryUnit.getSession();
     WT_SESSION* wtSession = session->getSession();
-    ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, "table:mytable", nullptr)));
+    ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, "table:mytable", nullptr), wtSession));
     auto result = WiredTigerUtil::getStatisticsValue(session->getSession(),
                                                      "statistics:table:mytable",
                                                      "statistics=(fast)",
@@ -322,7 +332,7 @@ TEST(WiredTigerUtilTest, GetStatisticsValueInvalidKey) {
                                         harnessHelper.getOplogManager());
     WiredTigerSession* session = recoveryUnit.getSession();
     WT_SESSION* wtSession = session->getSession();
-    ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, "table:mytable", nullptr)));
+    ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, "table:mytable", nullptr), wtSession));
     // Use connection statistics key which does not apply to a table.
     auto result = WiredTigerUtil::getStatisticsValue(session->getSession(),
                                                      "statistics:table:mytable",
@@ -338,7 +348,7 @@ TEST(WiredTigerUtilTest, GetStatisticsValueValidKey) {
                                         harnessHelper.getOplogManager());
     WiredTigerSession* session = recoveryUnit.getSession();
     WT_SESSION* wtSession = session->getSession();
-    ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, "table:mytable", nullptr)));
+    ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, "table:mytable", nullptr), wtSession));
     // Use connection statistics key which does not apply to a table.
     auto result = WiredTigerUtil::getStatisticsValue(session->getSession(),
                                                      "statistics:table:mytable",
@@ -349,118 +359,98 @@ TEST(WiredTigerUtilTest, GetStatisticsValueValidKey) {
     ASSERT_EQUALS(0U, result.getValue());
 }
 
-TEST(WiredTigerUtilTest, RemoveEncryptionFromConfigString) {
-    {  // Found at the middle.
-        std::string input{
-            "debug_mode=(table_logging=true,checkpoint_retention=4),encryption=(name=AES256-CBC,"
-            "keyid="
-            "\".system\"),extensions=[local={entry=mongo_addWiredTigerEncryptors,early_load=true},,"
-            "],"};
-        const std::string expectedOutput{
-            "debug_mode=(table_logging=true,checkpoint_retention=4),extensions=[local={entry=mongo_"
-            "addWiredTigerEncryptors,early_load=true},,],"};
-        WiredTigerUtil::removeEncryptionFromConfigString(&input);
-        ASSERT_EQUALS(input, expectedOutput);
-    }
-    {  // Found at start.
-        std::string input{
-            "encryption=(name=AES256-CBC,keyid=\".system\"),extensions=[local={entry=mongo_"
-            "addWiredTigerEncryptors,early_load=true},,],"};
-        const std::string expectedOutput{
-            "extensions=[local={entry=mongo_addWiredTigerEncryptors,early_load=true},,],"};
-        WiredTigerUtil::removeEncryptionFromConfigString(&input);
-        ASSERT_EQUALS(input, expectedOutput);
-    }
-    {  // Found at the end.
-        std::string input{
-            "debug_mode=(table_logging=true,checkpoint_retention=4),encryption=(name=AES256-CBC,"
-            "keyid=\".system\")"};
-        const std::string expectedOutput{"debug_mode=(table_logging=true,checkpoint_retention=4),"};
-        WiredTigerUtil::removeEncryptionFromConfigString(&input);
-        ASSERT_EQUALS(input, expectedOutput);
-    }
-    {  // Matches full configString.
-        std::string input{"encryption=(name=AES256-CBC,keyid=\".system\")"};
-        const std::string expectedOutput{""};
-        WiredTigerUtil::removeEncryptionFromConfigString(&input);
-        ASSERT_EQUALS(input, expectedOutput);
-    }
-    {  // Matches full configString, trailing comma.
-        std::string input{"encryption=(name=AES256-CBC,keyid=\".system\"),"};
-        const std::string expectedOutput{""};
-        WiredTigerUtil::removeEncryptionFromConfigString(&input);
-        ASSERT_EQUALS(input, expectedOutput);
-    }
-    {  // No match.
-        std::string input{"debug_mode=(table_logging=true,checkpoint_retention=4)"};
-        const std::string expectedOutput{"debug_mode=(table_logging=true,checkpoint_retention=4)"};
-        WiredTigerUtil::removeEncryptionFromConfigString(&input);
-        ASSERT_EQUALS(input, expectedOutput);
-    }
-    {  // No match, empty.
-        std::string input{""};
-        const std::string expectedOutput{""};
-        WiredTigerUtil::removeEncryptionFromConfigString(&input);
-        ASSERT_EQUALS(input, expectedOutput);
-    }
-    {  // Removes multiple instances.
-        std::string input{
-            "encryption=(name=AES256-CBC,keyid=\".system\"),debug_mode=(table_logging=true,"
-            "checkpoint_retention=4),encryption=(name=AES256-CBC,keyid=\".system\")"};
-        const std::string expectedOutput{"debug_mode=(table_logging=true,checkpoint_retention=4),"};
-        WiredTigerUtil::removeEncryptionFromConfigString(&input);
-        ASSERT_EQUALS(input, expectedOutput);
+TEST(WiredTigerUtilTest, ParseAPIMessages) {
+    // Custom event handler.
+    WiredTigerEventHandler eventHandler;
+
+    // Define a WiredTiger connection configuration that enables JSON encoding for all messages
+    // related to the WT_VERB_API category.
+    const std::string connection_cfg = "json_output=[error,message],verbose=[api]";
+
+    // Initialize WiredTiger.
+    WiredTigerUtilHarnessHelper harnessHelper(connection_cfg.c_str(), &eventHandler);
+
+    // Create a session.
+    WiredTigerRecoveryUnit recoveryUnit(harnessHelper.getSessionCache(),
+                                        harnessHelper.getOplogManager());
+    WT_SESSION* wtSession = recoveryUnit.getSession()->getSession();
+
+    // Perform simple WiredTiger operations while capturing the generated logs.
+    startCapturingLogMessages();
+    ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, "table:ev_api", nullptr), wtSession));
+    stopCapturingLogMessages();
+
+    // Verify there is at least one message from WiredTiger and their content.
+    bool foundWTMessage = false;
+    for (auto&& bson : getCapturedBSONFormatLogMessages()) {
+        if (bson["c"].String() == "WT") {
+            foundWTMessage = true;
+            ASSERT_EQUALS(bson["attr"]["message"]["category"].String(), "WT_VERB_API");
+            ASSERT_EQUALS(bson["attr"]["message"]["category_id"].Int(), WT_VERB_API);
+        }
     }
+    ASSERT_TRUE(foundWTMessage);
 }
 
-TEST(WiredTigerUtilTest, GetSanitizedStorageOptionsForSecondaryReplication) {
-    {  // Empty storage options.
-        auto input = BSONObj();
-        auto expectedOutput = input;
-        auto output = WiredTigerUtil::getSanitizedStorageOptionsForSecondaryReplication(input);
-        ASSERT_BSONOBJ_EQ(output, expectedOutput);
+TEST(WiredTigerUtilTest, ParseCompactMessages) {
+    // Custom event handler.
+    WiredTigerEventHandler eventHandler;
+
+    // Define a WiredTiger connection configuration that enables JSON encoding for all messages
+    // related to the WT_VERB_COMPACT category.
+    const std::string connection_cfg = "json_output=[error,message],verbose=[compact]";
+
+    // Initialize WiredTiger.
+    WiredTigerUtilHarnessHelper harnessHelper(connection_cfg.c_str(), &eventHandler);
+
+    // Create a session.
+    WT_SESSION* wtSession;
+    ASSERT_OK(
+        wtRCToStatus(harnessHelper.getSessionCache()->conn()->open_session(
+                         harnessHelper.getSessionCache()->conn(), nullptr, nullptr, &wtSession),
+                     nullptr));
+
+    // Perform simple WiredTiger operations while capturing the generated logs.
+    const std::string uri = "table:ev_compact";
+    startCapturingLogMessages();
+    ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, uri.c_str(), nullptr), wtSession));
+    ASSERT_OK(wtRCToStatus(wtSession->compact(wtSession, uri.c_str(), nullptr), wtSession));
+    stopCapturingLogMessages();
+
+    // Verify there is at least one message from WiredTiger and their content.
+    bool foundWTMessage = false;
+    for (auto&& bson : getCapturedBSONFormatLogMessages()) {
+        if (bson["c"].String() == "WTCMPCT") {
+            foundWTMessage = true;
+            ASSERT_EQUALS(bson["attr"]["message"]["category"].String(), "WT_VERB_COMPACT");
+            ASSERT_EQUALS(bson["attr"]["message"]["category_id"].Int(), WT_VERB_COMPACT);
+        }
+    }
+    ASSERT_TRUE(foundWTMessage);
+}
+
+TEST(WiredTigerUtilTest, GenerateVerboseConfiguration) {
+    // Perform each test in their own limited scope in order to establish different
+    // severity levels.
+    {
+        // Set the WiredTiger Checkpoint LOGV2 component severity to the Log level.
+        auto severityGuard = unittest::MinimumLoggedSeverityGuard{
+            logv2::LogComponent::kWiredTigerCheckpoint, logv2::LogSeverity::Log()};
+        // Generate the configuration string and verify the default severity of the
+        // checkpoint component is at the Log level.
+        std::string config = WiredTigerUtil::generateWTVerboseConfiguration();
+        ASSERT_TRUE(config.find("checkpoint:0") != std::string::npos);
+        ASSERT_TRUE(config.find("checkpoint:1") == std::string::npos);
     }
     {
-        // Preserve WT config string without encryption options.
-        auto input = BSON("wiredTiger" << BSON("configString"
-                                               << "split_pct=88"));
-        auto expectedOutput = input;
-        auto output = WiredTigerUtil::getSanitizedStorageOptionsForSecondaryReplication(input);
-        ASSERT_BSONOBJ_EQ(output, expectedOutput);
-    }
-    {
-        // Remove encryption options from WT config string in results.
-        auto input = BSON(
-            "wiredTiger" << BSON("configString"
-                                 << "encryption=(name=AES256-CBC,keyid=\".system\"),split_pct=88"));
-        auto expectedOutput = BSON("wiredTiger" << BSON("configString"
-                                                        << "split_pct=88"));
-        auto output = WiredTigerUtil::getSanitizedStorageOptionsForSecondaryReplication(input);
-        ASSERT_BSONOBJ_EQ(output, expectedOutput);
-    }
-    {
-        // Leave non-WT settings intact.
-        auto input = BSON("inMemory" << BSON("configString"
-                                             << "split_pct=66"));
-        auto expectedOutput = input;
-        auto output = WiredTigerUtil::getSanitizedStorageOptionsForSecondaryReplication(input);
-        ASSERT_BSONOBJ_EQ(output, expectedOutput);
-    }
-    {
-        // Change only WT settings in storage options containing a mix of WT and non-WT settings.
-        auto input = BSON(
-            "inMemory" << BSON("configString"
-                               << "split_pct=66")
-                       << "wiredTiger"
-                       << BSON("configString"
-                               << "encryption=(name=AES256-CBC,keyid=\".system\"),split_pct=88"));
-        auto expectedOutput = BSON("inMemory" << BSON("configString"
-                                                      << "split_pct=66")
-                                              << "wiredTiger"
-                                              << BSON("configString"
-                                                      << "split_pct=88"));
-        auto output = WiredTigerUtil::getSanitizedStorageOptionsForSecondaryReplication(input);
-        ASSERT_BSONOBJ_EQ(output, expectedOutput);
+        // Set the WiredTiger Checkpoint LOGV2 component severity to the Debug(2) level.
+        // We want to ensure this setting is subsequently reflected in a new WiredTiger
+        // verbose configuration string.
+        auto severityGuard = unittest::MinimumLoggedSeverityGuard{
+            logv2::LogComponent::kWiredTigerCheckpoint, logv2::LogSeverity::Debug(2)};
+        std::string config = WiredTigerUtil::generateWTVerboseConfiguration();
+        ASSERT_TRUE(config.find("checkpoint:1") != std::string::npos);
+        ASSERT_TRUE(config.find("checkpoint:0") == std::string::npos);
     }
 }
 
