@@ -53,8 +53,9 @@
 #include "mongo/db/service_context.h"
 #include "mongo/db/views/durable_view_catalog.h"
 #include "mongo/db/views/view.h"
-#include "mongo/db/views/view_catalog.h"
+#include "mongo/db/views/view_catalog_helpers.h"
 #include "mongo/db/views/view_graph.h"
+#include "mongo/idl/server_parameter_test_util.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/str.h"
 
@@ -78,36 +79,41 @@ const auto kTinyMatchStage = BSON("$match" << BSONObj());
 
 class ViewCatalogFixture : public CatalogTestFixture {
 public:
+    ViewCatalogFixture() : ViewCatalogFixture(DatabaseName(boost::none, "db")) {}
+
+    ViewCatalogFixture(DatabaseName dbName) : _dbName(std::move(dbName)) {}
+
     void setUp() override {
         CatalogTestFixture::setUp();
 
         WriteUnitOfWork wuow(operationContext());
-        AutoGetOrCreateDb autoDb(operationContext(), "db", MODE_X);
-        _db = autoDb.getDb();
+        AutoGetDb autoDb(operationContext(), _dbName, MODE_X);
+        _db = autoDb.ensureDbExists(operationContext());
         invariant(_db);
 
+        // Create any additional databases used throughout the test.
+        ASSERT(AutoGetDb(operationContext(), DatabaseName(_dbName.tenantId(), "db1"), MODE_X)
+                   .ensureDbExists(operationContext()));
+        ASSERT(AutoGetDb(operationContext(), DatabaseName(_dbName.tenantId(), "db2"), MODE_X)
+                   .ensureDbExists(operationContext()));
+
         auto durableViewCatalogUnique = std::make_unique<DurableViewCatalogImpl>(_db);
         durableViewCatalog = durableViewCatalogUnique.get();
 
-        _viewCatalog = ViewCatalog::get(_db);
-
         // Create the system views collection for the database.
         ASSERT(_db->createCollection(
             operationContext(),
-            NamespaceString("db", NamespaceString::kSystemDotViewsCollectionName)));
+            NamespaceString(_dbName, NamespaceString::kSystemDotViewsCollectionName)));
 
-        // Create any additional databases used throughout the test.
-        ASSERT(AutoGetOrCreateDb(operationContext(), "db1", MODE_X).getDb());
-        ASSERT(AutoGetOrCreateDb(operationContext(), "db2", MODE_X).getDb());
         wuow.commit();
     }
 
-    void tearDown() {
+    void tearDown() override {
         CatalogTestFixture::tearDown();
     }
 
-    ViewCatalog* getViewCatalog() const {
-        return _viewCatalog;
+    auto getCatalog() {
+        return CollectionCatalog::get(operationContext());
     }
 
     Status createView(OperationContext* opCtx,
@@ -115,15 +121,16 @@ public:
                       const NamespaceString& viewOn,
                       const BSONArray& pipeline,
                       const BSONObj& collation) {
-        Lock::DBLock dbLock(operationContext(), viewName.db(), MODE_IX);
+        Lock::DBLock dbLock(operationContext(), viewName.dbName(), MODE_IX);
         Lock::CollectionLock collLock(operationContext(), viewName, MODE_IX);
         Lock::CollectionLock sysCollLock(
             operationContext(),
-            NamespaceString(viewName.db(), NamespaceString::kSystemDotViewsCollectionName),
+            NamespaceString(viewName.dbName(), NamespaceString::kSystemDotViewsCollectionName),
             MODE_X);
 
         WriteUnitOfWork wuow(opCtx);
-        Status s = _viewCatalog->createView(opCtx, viewName, viewOn, pipeline, collation);
+        Status s = getCatalog()->createView(
+            opCtx, viewName, viewOn, pipeline, collation, view_catalog_helpers::validatePipeline);
         wuow.commit();
 
         return s;
@@ -133,43 +140,49 @@ public:
                       const NamespaceString& viewName,
                       const NamespaceString& viewOn,
                       const BSONArray& pipeline) {
-        Lock::DBLock dbLock(operationContext(), viewName.db(), MODE_IX);
+        Lock::DBLock dbLock(operationContext(), viewName.dbName(), MODE_IX);
         Lock::CollectionLock collLock(operationContext(), viewName, MODE_X);
         Lock::CollectionLock sysCollLock(
             operationContext(),
-            NamespaceString(viewName.db(), NamespaceString::kSystemDotViewsCollectionName),
+            NamespaceString(viewName.dbName(), NamespaceString::kSystemDotViewsCollectionName),
             MODE_X);
 
         WriteUnitOfWork wuow(opCtx);
-        Status s = _viewCatalog->modifyView(opCtx, viewName, viewOn, pipeline);
+        Status s = getCatalog()->modifyView(
+            opCtx, viewName, viewOn, pipeline, view_catalog_helpers::validatePipeline);
         wuow.commit();
 
         return s;
     }
 
     Status dropView(OperationContext* opCtx, const NamespaceString& viewName) {
-        Lock::DBLock dbLock(operationContext(), viewName.db(), MODE_IX);
+        Lock::DBLock dbLock(operationContext(), viewName.dbName(), MODE_IX);
         Lock::CollectionLock collLock(operationContext(), viewName, MODE_IX);
         Lock::CollectionLock sysCollLock(
             operationContext(),
-            NamespaceString(viewName.db(), NamespaceString::kSystemDotViewsCollectionName),
+            NamespaceString(viewName.dbName(), NamespaceString::kSystemDotViewsCollectionName),
             MODE_X);
 
         WriteUnitOfWork wuow(opCtx);
-        Status s = _viewCatalog->dropView(opCtx, viewName);
+        Status s = getCatalog()->dropView(opCtx, viewName);
         wuow.commit();
 
         return s;
     }
 
-    std::shared_ptr<ViewDefinition> lookup(OperationContext* opCtx, StringData ns) {
-        Lock::DBLock dbLock(operationContext(), NamespaceString(ns).db(), MODE_IS);
-        return _viewCatalog->lookup(operationContext(), ns);
+    Database* db() {
+        return _db;
+    }
+
+    std::shared_ptr<const ViewDefinition> lookup(OperationContext* opCtx,
+                                                 const NamespaceString& ns) {
+        Lock::DBLock dbLock(operationContext(), ns.dbName(), MODE_IS);
+        return getCatalog()->lookupView(operationContext(), ns);
     }
 
 private:
+    DatabaseName _dbName;
     Database* _db;
-    ViewCatalog* _viewCatalog;
 
 protected:
     DurableViewCatalogImpl* durableViewCatalog;
@@ -386,6 +399,9 @@ TEST_F(ViewCatalogFixture, CreateViewCycles) {
 }
 
 TEST_F(ViewCatalogFixture, CanSuccessfullyCreateViewWhosePipelineIsExactlyAtMaxSizeInBytes) {
+    internalPipelineLengthLimit = 100000;
+    ON_BLOCK_EXIT([] { internalPipelineLengthLimit = 1000; });
+
     ASSERT_EQ(ViewGraph::kMaxViewPipelineSizeBytes % kOneKiBMatchStage.objsize(), 0);
 
     BSONArrayBuilder builder(ViewGraph::kMaxViewPipelineSizeBytes);
@@ -405,6 +421,9 @@ TEST_F(ViewCatalogFixture, CanSuccessfullyCreateViewWhosePipelineIsExactlyAtMaxS
 }
 
 TEST_F(ViewCatalogFixture, CannotCreateViewWhosePipelineExceedsMaxSizeInBytes) {
+    internalPipelineLengthLimit = 100000;
+    ON_BLOCK_EXIT([] { internalPipelineLengthLimit = 1000; });
+
     // Fill the builder to exactly the maximum size, then push it just over the limit by adding an
     // additional tiny match stage.
     BSONArrayBuilder builder(ViewGraph::kMaxViewPipelineSizeBytes);
@@ -422,6 +441,9 @@ TEST_F(ViewCatalogFixture, CannotCreateViewWhosePipelineExceedsMaxSizeInBytes) {
 }
 
 TEST_F(ViewCatalogFixture, CannotCreateViewIfItsFullyResolvedPipelineWouldExceedMaxSizeInBytes) {
+    internalPipelineLengthLimit = 100000;
+    ON_BLOCK_EXIT([] { internalPipelineLengthLimit = 1000; });
+
     BSONArrayBuilder builder1;
     BSONArrayBuilder builder2;
 
@@ -485,7 +507,7 @@ TEST_F(ReplViewCatalogFixture, ModifyViewWithPipelineFailsOnIneligibleStage) {
 }
 
 TEST_F(ViewCatalogFixture, LookupMissingView) {
-    ASSERT(!lookup(operationContext(), "db.view"_sd));
+    ASSERT(!lookup(operationContext(), NamespaceString("db.view")));
 }
 
 TEST_F(ViewCatalogFixture, LookupExistingView) {
@@ -494,7 +516,7 @@ TEST_F(ViewCatalogFixture, LookupExistingView) {
 
     ASSERT_OK(createView(operationContext(), viewName, viewOn, emptyPipeline, emptyCollation));
 
-    ASSERT(lookup(operationContext(), "db.view"_sd));
+    ASSERT(lookup(operationContext(), viewName));
 }
 
 TEST_F(ViewCatalogFixture, LookupRIDExistingView) {
@@ -503,16 +525,15 @@ TEST_F(ViewCatalogFixture, LookupRIDExistingView) {
 
     ASSERT_OK(createView(operationContext(), viewName, viewOn, emptyPipeline, emptyCollation));
 
-    auto resourceID = ResourceId(RESOURCE_COLLECTION, "db.view"_sd);
-    auto& collectionCatalog = CollectionCatalog::get(operationContext());
-    ASSERT(collectionCatalog.lookupResourceName(resourceID).get() == "db.view");
+    auto resourceID = ResourceId(RESOURCE_COLLECTION, NamespaceString(boost::none, "db.view"));
+    ASSERT(getCatalog()->lookupResourceName(resourceID).value() == "db.view");
 }
 
 TEST_F(ViewCatalogFixture, LookupRIDExistingViewRollback) {
     const NamespaceString viewName("db.view");
     const NamespaceString viewOn("db.coll");
     {
-        Lock::DBLock dbLock(operationContext(), viewName.db(), MODE_X);
+        Lock::DBLock dbLock(operationContext(), viewName.dbName(), MODE_X);
         Lock::CollectionLock collLock(operationContext(), viewName, MODE_IX);
         Lock::CollectionLock sysCollLock(
             operationContext(),
@@ -520,12 +541,15 @@ TEST_F(ViewCatalogFixture, LookupRIDExistingViewRollback) {
             MODE_X);
 
         WriteUnitOfWork wunit(operationContext());
-        ASSERT_OK(getViewCatalog()->createView(
-            operationContext(), viewName, viewOn, emptyPipeline, emptyCollation));
+        ASSERT_OK(getCatalog()->createView(operationContext(),
+                                           viewName,
+                                           viewOn,
+                                           emptyPipeline,
+                                           emptyCollation,
+                                           view_catalog_helpers::validatePipeline));
     }
-    auto resourceID = ResourceId(RESOURCE_COLLECTION, "db.view"_sd);
-    auto& collectionCatalog = CollectionCatalog::get(operationContext());
-    ASSERT(!collectionCatalog.lookupResourceName(resourceID));
+    auto resourceID = ResourceId(RESOURCE_COLLECTION, NamespaceString(boost::none, "db.view"));
+    ASSERT(!getCatalog()->lookupResourceName(resourceID));
 }
 
 TEST_F(ViewCatalogFixture, LookupRIDAfterDrop) {
@@ -535,26 +559,24 @@ TEST_F(ViewCatalogFixture, LookupRIDAfterDrop) {
     ASSERT_OK(createView(operationContext(), viewName, viewOn, emptyPipeline, emptyCollation));
     ASSERT_OK(dropView(operationContext(), viewName));
 
-    auto resourceID = ResourceId(RESOURCE_COLLECTION, "db.view"_sd);
-    auto& collectionCatalog = CollectionCatalog::get(operationContext());
-    ASSERT(!collectionCatalog.lookupResourceName(resourceID));
+    auto resourceID = ResourceId(RESOURCE_COLLECTION, NamespaceString(boost::none, "db.view"));
+    ASSERT(!getCatalog()->lookupResourceName(resourceID));
 }
 
 TEST_F(ViewCatalogFixture, LookupRIDAfterDropRollback) {
     const NamespaceString viewName("db.view");
     const NamespaceString viewOn("db.coll");
 
-    auto resourceID = ResourceId(RESOURCE_COLLECTION, "db.view"_sd);
-    auto& collectionCatalog = CollectionCatalog::get(operationContext());
+    auto resourceID = ResourceId(RESOURCE_COLLECTION, NamespaceString(boost::none, "db.view"));
     {
         WriteUnitOfWork wunit(operationContext());
         ASSERT_OK(createView(operationContext(), viewName, viewOn, emptyPipeline, emptyCollation));
-        ASSERT(collectionCatalog.lookupResourceName(resourceID).get() == viewName.ns());
         wunit.commit();
+        ASSERT(getCatalog()->lookupResourceName(resourceID).value() == viewName.ns());
     }
 
     {
-        Lock::DBLock dbLock(operationContext(), viewName.db(), MODE_X);
+        Lock::DBLock dbLock(operationContext(), viewName.dbName(), MODE_X);
         Lock::CollectionLock collLock(operationContext(), viewName, MODE_IX);
         Lock::CollectionLock sysCollLock(
             operationContext(),
@@ -562,37 +584,37 @@ TEST_F(ViewCatalogFixture, LookupRIDAfterDropRollback) {
             MODE_X);
 
         WriteUnitOfWork wunit(operationContext());
-        ASSERT_OK(getViewCatalog()->dropView(operationContext(), viewName));
+        ASSERT_OK(getCatalog()->dropView(operationContext(), viewName));
+        // Do not commit, rollback.
     }
-
-    ASSERT(collectionCatalog.lookupResourceName(resourceID).get() == viewName.ns());
+    // Make sure drop was rolled back and view is still in catalog.
+    ASSERT(getCatalog()->lookupResourceName(resourceID).value() == viewName.ns());
 }
 
 TEST_F(ViewCatalogFixture, LookupRIDAfterModify) {
     const NamespaceString viewName("db.view");
     const NamespaceString viewOn("db.coll");
 
-    auto resourceID = ResourceId(RESOURCE_COLLECTION, "db.view"_sd);
-    auto& collectionCatalog = CollectionCatalog::get(operationContext());
+    auto resourceID = ResourceId(RESOURCE_COLLECTION, NamespaceString(boost::none, "db.view"));
     ASSERT_OK(createView(operationContext(), viewName, viewOn, emptyPipeline, emptyCollation));
     ASSERT_OK(modifyView(operationContext(), viewName, viewOn, emptyPipeline));
-    ASSERT(collectionCatalog.lookupResourceName(resourceID).get() == viewName.ns());
+    ASSERT(getCatalog()->lookupResourceName(resourceID).value() == viewName.ns());
 }
 
 TEST_F(ViewCatalogFixture, LookupRIDAfterModifyRollback) {
     const NamespaceString viewName("db.view");
     const NamespaceString viewOn("db.coll");
 
-    auto resourceID = ResourceId(RESOURCE_COLLECTION, "db.view"_sd);
-    auto& collectionCatalog = CollectionCatalog::get(operationContext());
+    auto resourceID = ResourceId(RESOURCE_COLLECTION, NamespaceString(boost::none, "db.view"));
     {
         WriteUnitOfWork wunit(operationContext());
         ASSERT_OK(createView(operationContext(), viewName, viewOn, emptyPipeline, emptyCollation));
-        ASSERT(collectionCatalog.lookupResourceName(resourceID).get() == viewName.ns());
         wunit.commit();
+        ASSERT(getCatalog()->lookupResourceName(resourceID).value() == viewName.ns());
     }
+
     {
-        Lock::DBLock dbLock(operationContext(), viewName.db(), MODE_IX);
+        Lock::DBLock dbLock(operationContext(), viewName.dbName(), MODE_IX);
         Lock::CollectionLock collLock(operationContext(), viewName, MODE_X);
         Lock::CollectionLock sysCollLock(
             operationContext(),
@@ -600,21 +622,26 @@ TEST_F(ViewCatalogFixture, LookupRIDAfterModifyRollback) {
             MODE_X);
 
         WriteUnitOfWork wunit(operationContext());
-        ASSERT_OK(
-            getViewCatalog()->modifyView(operationContext(), viewName, viewOn, emptyPipeline));
-        ASSERT(collectionCatalog.lookupResourceName(resourceID).get() == viewName.ns());
+        ASSERT_OK(getCatalog()->modifyView(operationContext(),
+                                           viewName,
+                                           viewOn,
+                                           emptyPipeline,
+                                           view_catalog_helpers::validatePipeline));
+        ASSERT(getCatalog()->lookupResourceName(resourceID).value() == viewName.ns());
+        // Do not commit, rollback.
     }
-    ASSERT(collectionCatalog.lookupResourceName(resourceID).get() == viewName.ns());
+    // Make sure view resource is still available after rollback.
+    ASSERT(getCatalog()->lookupResourceName(resourceID).value() == viewName.ns());
 }
 
 TEST_F(ViewCatalogFixture, CreateViewThenDropAndLookup) {
-    const NamespaceString viewName("db.view");
+    NamespaceString viewName("db.view");
     const NamespaceString viewOn("db.coll");
 
     ASSERT_OK(createView(operationContext(), viewName, viewOn, emptyPipeline, emptyCollation));
     ASSERT_OK(dropView(operationContext(), viewName));
 
-    ASSERT(!lookup(operationContext(), "db.view"_sd));
+    ASSERT(!lookup(operationContext(), viewName));
 }
 
 TEST_F(ViewCatalogFixture, Iterate) {
@@ -629,12 +656,14 @@ TEST_F(ViewCatalogFixture, Iterate) {
 
     std::set<std::string> viewNames = {"db.view1", "db.view2", "db.view3"};
 
-    Lock::DBLock dbLock(operationContext(), "db", MODE_IX);
-    getViewCatalog()->iterate(operationContext(), [&viewNames](const ViewDefinition& view) {
-        std::string name = view.name().toString();
-        ASSERT(viewNames.end() != viewNames.find(name));
-        viewNames.erase(name);
-    });
+    Lock::DBLock dbLock(operationContext(), view1.dbName(), MODE_IX);
+    getCatalog()->iterateViews(
+        operationContext(), view1.dbName(), [&viewNames](const ViewDefinition& view) {
+            std::string name = view.name().toString();
+            ASSERT(viewNames.end() != viewNames.find(name));
+            viewNames.erase(name);
+            return true;
+        });
 
     ASSERT(viewNames.empty());
 }
@@ -656,8 +685,9 @@ TEST_F(ViewCatalogFixture, ResolveViewCorrectPipeline) {
     ASSERT_OK(createView(operationContext(), view2, view1, pipeline2.arr(), emptyCollation));
     ASSERT_OK(createView(operationContext(), view3, view2, pipeline3.arr(), emptyCollation));
 
-    Lock::DBLock dbLock(operationContext(), "db", MODE_IX);
-    auto resolvedView = getViewCatalog()->resolveView(operationContext(), view3);
+    Lock::DBLock dbLock(operationContext(), view1.dbName(), MODE_IX);
+    auto resolvedView =
+        view_catalog_helpers::resolveView(operationContext(), getCatalog(), view3, boost::none);
     ASSERT(resolvedView.isOK());
 
     std::vector<BSONObj> expected = {BSON("$match" << BSON("foo" << 1)),
@@ -676,9 +706,9 @@ TEST_F(ViewCatalogFixture, ResolveViewCorrectPipeline) {
 TEST_F(ViewCatalogFixture, ResolveViewOnCollectionNamespace) {
     const NamespaceString collectionNamespace("db.coll");
 
-    Lock::DBLock dbLock(operationContext(), "db", MODE_IS);
-    auto resolvedView =
-        uassertStatusOK(getViewCatalog()->resolveView(operationContext(), collectionNamespace));
+    Lock::DBLock dbLock(operationContext(), collectionNamespace.dbName(), MODE_IS);
+    auto resolvedView = uassertStatusOK(view_catalog_helpers::resolveView(
+        operationContext(), getCatalog(), collectionNamespace, boost::none));
 
     ASSERT_EQ(resolvedView.getNamespace(), collectionNamespace);
     ASSERT_EQ(resolvedView.getPipeline().size(), 0U);
@@ -700,8 +730,9 @@ TEST_F(ViewCatalogFixture, ResolveViewCorrectlyExtractsDefaultCollation) {
     ASSERT_OK(createView(operationContext(), view1, viewOn, pipeline1.arr(), collation));
     ASSERT_OK(createView(operationContext(), view2, view1, pipeline2.arr(), collation));
 
-    Lock::DBLock dbLock(operationContext(), "db", MODE_IS);
-    auto resolvedView = getViewCatalog()->resolveView(operationContext(), view2);
+    Lock::DBLock dbLock(operationContext(), view1.dbName(), MODE_IS);
+    auto resolvedView =
+        view_catalog_helpers::resolveView(operationContext(), getCatalog(), view2, boost::none);
     ASSERT(resolvedView.isOK());
 
     ASSERT_EQ(resolvedView.getValue().getNamespace(), viewOn);
@@ -721,5 +752,66 @@ TEST_F(ViewCatalogFixture, ResolveViewCorrectlyExtractsDefaultCollation) {
                       expectedCollation.getValue()->getSpec().toBSON());
 }
 
+class ServerlessViewCatalogFixture : public ViewCatalogFixture {
+public:
+    ServerlessViewCatalogFixture() : ViewCatalogFixture(DatabaseName(TenantId(OID::gen()), "db")) {}
+};
+
+TEST_F(ServerlessViewCatalogFixture, LookupExistingViewBeforeAndAfterDropFeatureFlagOff) {
+    RAIIServerParameterControllerForTest multitenanyController("multitenancySupport", true);
+    const NamespaceString viewName(db()->name(), "view");
+    const NamespaceString viewOn(db()->name(), "coll");
+
+    ASSERT_OK(createView(operationContext(), viewName, viewOn, emptyPipeline, emptyCollation));
+    ASSERT(lookup(operationContext(), viewName));
+
+    ASSERT_OK(dropView(operationContext(), viewName));
+    ASSERT(!lookup(operationContext(), viewName));
+}
+
+TEST_F(ServerlessViewCatalogFixture, LookupExistingViewBeforeAndAfterDropFeatureFlagOn) {
+    RAIIServerParameterControllerForTest multitenanyController("multitenancySupport", true);
+    RAIIServerParameterControllerForTest featureFlagController("featureFlagRequireTenantID", true);
+    const NamespaceString viewName(db()->name(), "view");
+    const NamespaceString viewOn(db()->name(), "coll");
+
+    ASSERT_OK(createView(operationContext(), viewName, viewOn, emptyPipeline, emptyCollation));
+    ASSERT(lookup(operationContext(), viewName));
+
+    ASSERT_OK(dropView(operationContext(), viewName));
+    ASSERT(!lookup(operationContext(), viewName));
+}
+
+TEST_F(ServerlessViewCatalogFixture, ModifyViewBelongingToTenantFeatureFlagOff) {
+    RAIIServerParameterControllerForTest multitenanyController("multitenancySupport", true);
+    const NamespaceString viewName(db()->name(), "db1.view");
+    const NamespaceString viewOn(db()->name(), "db2.coll");
+
+    ASSERT_OK(createView(operationContext(), viewName, viewOn, emptyPipeline, emptyCollation));
+    ASSERT_EQ(lookup(operationContext(), viewName)->pipeline().size(), 0);
+
+    // Modify the view and pass a non-empty pipeline.
+    BSONArrayBuilder builder;
+    builder << kTinyMatchStage;
+    ASSERT_OK(modifyView(operationContext(), viewName, viewOn, builder.arr()));
+    ASSERT_EQ(lookup(operationContext(), viewName)->pipeline().size(), 1);
+}
+
+TEST_F(ServerlessViewCatalogFixture, ModifyViewBelongingToTenantFeatureFlagOn) {
+    RAIIServerParameterControllerForTest multitenanyController("multitenancySupport", true);
+    RAIIServerParameterControllerForTest featureFlagController("featureFlagRequireTenantID", true);
+    const NamespaceString viewName(db()->name(), "db1.view");
+    const NamespaceString viewOn(db()->name(), "db2.coll");
+
+    ASSERT_OK(createView(operationContext(), viewName, viewOn, emptyPipeline, emptyCollation));
+    ASSERT_EQ(lookup(operationContext(), viewName)->pipeline().size(), 0);
+
+    // Modify the view and pass a non-empty pipeline.
+    BSONArrayBuilder builder;
+    builder << kTinyMatchStage;
+    ASSERT_OK(modifyView(operationContext(), viewName, viewOn, builder.arr()));
+    ASSERT_EQ(lookup(operationContext(), viewName)->pipeline().size(), 1);
+}
+
 }  // namespace
 }  // namespace mongo
