@@ -27,14 +27,9 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include <cstdint>
-
-#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_write_path.h"
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog/multi_index_block.h"
-#include "mongo/db/catalog/uncommitted_collections.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
 #include "mongo/db/db_raii.h"
@@ -44,8 +39,8 @@
 #include "mongo/db/storage/storage_engine_init.h"
 #include "mongo/dbtests/dbtests.h"
 
+namespace mongo {
 namespace IndexUpdateTests {
-
 namespace {
 const auto kIndexVersion = IndexDescriptor::IndexVersion::kV2;
 }  // namespace
@@ -56,32 +51,33 @@ static const NamespaceString _nss = NamespaceString(_ns);
 /**
  * Test fixture for a write locked test using collection _ns.  Includes functionality to
  * partially construct a new IndexDetails in a manner that supports proper cleanup in
- * dropCollection().
+ * dropcollection().get().
  */
 class IndexBuildBase {
 public:
     IndexBuildBase() {
         regenOpCtx();
 
-        AutoGetOrCreateDb db(_opCtx, _nss.db(), LockMode::MODE_IX);
+        AutoGetCollection autoColl(_opCtx, _nss, LockMode::MODE_IX);
         WriteUnitOfWork wuow(_opCtx);
-        Lock::CollectionLock lk(_opCtx, _nss, LockMode::MODE_IX);
-        db.getDb()->createCollection(_opCtx, _nss);
+        auto db = autoColl.ensureDbExists(_opCtx);
+        ASSERT(db->createCollection(_opCtx, _nss)) << _nss;
         wuow.commit();
     }
 
     ~IndexBuildBase() {
         getGlobalServiceContext()->unsetKillAllOperations();
 
-        AutoGetOrCreateDb db(_opCtx, _nss.db(), LockMode::MODE_IX);
+        AutoGetCollection autoColl(_opCtx, _nss, LockMode::MODE_X);
         WriteUnitOfWork wuow(_opCtx);
-        Lock::CollectionLock lk(_opCtx, _nss, LockMode::MODE_X);
-        ASSERT_OK(db.getDb()->dropCollection(_opCtx, _nss, {}));
+        auto db = autoColl.ensureDbExists(_opCtx);
+        ASSERT_OK(db->dropCollection(_opCtx, _nss, {})) << _nss;
         wuow.commit();
     }
 
-    Collection* collection() {
-        return CollectionCatalog::get(_opCtx).lookupCollectionByNamespace(_opCtx, _nss);
+    CollectionWriter& collection() {
+        _collection.emplace(_opCtx, _nss);
+        return *_collection;
     }
 
 protected:
@@ -91,16 +87,16 @@ protected:
         try {
             MultiIndexBlock indexer;
 
-            auto abortOnExit = makeGuard([&] {
+            ScopeGuard abortOnExit([&] {
                 indexer.abortIndexBuild(_opCtx, collection(), MultiIndexBlock::kNoopOnCleanUpFn);
             });
 
             uassertStatusOK(
                 indexer.init(_opCtx, collection(), key, MultiIndexBlock::kNoopOnInitFn));
-            uassertStatusOK(indexer.insertAllDocumentsInCollection(_opCtx, collection()));
+            uassertStatusOK(indexer.insertAllDocumentsInCollection(_opCtx, collection().get()));
             WriteUnitOfWork wunit(_opCtx);
             ASSERT_OK(indexer.commit(_opCtx,
-                                     collection(),
+                                     collection().getWritableCollection(_opCtx),
                                      MultiIndexBlock::kNoopOnCreateEachFn,
                                      MultiIndexBlock::kNoopOnCommitFn));
             wunit.commit();
@@ -122,6 +118,7 @@ protected:
 
     ServiceContext::UniqueOperationContext _txnPtr;  // = cc().makeOperationContext();
     OperationContext* _opCtx;                        // = _txnPtr.get();
+    boost::optional<CollectionWriter> _collection;
 };
 
 /** Index creation ignores unique constraints when told to. */
@@ -129,22 +126,25 @@ template <bool background>
 class InsertBuildIgnoreUnique : public IndexBuildBase {
 public:
     void run() {
-        AutoGetOrCreateDb dbRaii(_opCtx, _nss.db(), LockMode::MODE_IX);
-        Lock::CollectionLock collLk(_opCtx, _nss, LockMode::MODE_X);
-        Collection* coll = collection();
+        AutoGetCollection autoColl(_opCtx, _nss, LockMode::MODE_X);
+        auto db = autoColl.ensureDbExists(_opCtx);
+        ASSERT(db) << _nss;
+        auto& coll = collection();
         {
             WriteUnitOfWork wunit(_opCtx);
             OpDebug* const nullOpDebug = nullptr;
-            ASSERT_OK(coll->insertDocument(_opCtx,
-                                           InsertStatement(BSON("_id" << 1 << "a"
-                                                                      << "dup")),
-                                           nullOpDebug,
-                                           true));
-            ASSERT_OK(coll->insertDocument(_opCtx,
-                                           InsertStatement(BSON("_id" << 2 << "a"
-                                                                      << "dup")),
-                                           nullOpDebug,
-                                           true));
+            ASSERT_OK(collection_internal::insertDocument(_opCtx,
+                                                          coll.get(),
+                                                          InsertStatement(BSON("_id" << 1 << "a"
+                                                                                     << "dup")),
+                                                          nullOpDebug,
+                                                          true));
+            ASSERT_OK(collection_internal::insertDocument(_opCtx,
+                                                          coll.get(),
+                                                          InsertStatement(BSON("_id" << 2 << "a"
+                                                                                     << "dup")),
+                                                          nullOpDebug,
+                                                          true));
             wunit.commit();
         }
 
@@ -157,17 +157,19 @@ public:
                                   << static_cast<int>(kIndexVersion) << "unique" << true
                                   << "background" << background);
 
-        auto abortOnExit = makeGuard([&] {
+        ScopeGuard abortOnExit([&] {
             indexer.abortIndexBuild(_opCtx, collection(), MultiIndexBlock::kNoopOnCleanUpFn);
         });
 
         ASSERT_OK(indexer.init(_opCtx, coll, spec, MultiIndexBlock::kNoopOnInitFn).getStatus());
-        ASSERT_OK(indexer.insertAllDocumentsInCollection(_opCtx, coll));
-        ASSERT_OK(indexer.checkConstraints(_opCtx));
+        ASSERT_OK(indexer.insertAllDocumentsInCollection(_opCtx, coll.get()));
+        ASSERT_OK(indexer.checkConstraints(_opCtx, coll.get()));
 
         WriteUnitOfWork wunit(_opCtx);
-        ASSERT_OK(indexer.commit(
-            _opCtx, coll, MultiIndexBlock::kNoopOnCreateEachFn, MultiIndexBlock::kNoopOnCommitFn));
+        ASSERT_OK(indexer.commit(_opCtx,
+                                 coll.getWritableCollection(_opCtx),
+                                 MultiIndexBlock::kNoopOnCreateEachFn,
+                                 MultiIndexBlock::kNoopOnCommitFn));
         wunit.commit();
         abortOnExit.dismiss();
     }
@@ -179,52 +181,59 @@ class InsertBuildEnforceUnique : public IndexBuildBase {
 public:
     void run() {
         // Create a new collection.
-        AutoGetOrCreateDb dbRaii(_opCtx, _nss.db(), LockMode::MODE_IX);
-        boost::optional<Lock::CollectionLock> collLk;
-        collLk.emplace(_opCtx, _nss, LockMode::MODE_IX);
-        Collection* coll = collection();
         {
-            WriteUnitOfWork wunit(_opCtx);
-            OpDebug* const nullOpDebug = nullptr;
-            ASSERT_OK(coll->insertDocument(_opCtx,
-                                           InsertStatement(BSON("_id" << 1 << "a"
-                                                                      << "dup")),
-                                           nullOpDebug,
-                                           true));
-            ASSERT_OK(coll->insertDocument(_opCtx,
-                                           InsertStatement(BSON("_id" << 2 << "a"
-                                                                      << "dup")),
-                                           nullOpDebug,
-                                           true));
-            wunit.commit();
+            AutoGetCollection autoColl(_opCtx, _nss, LockMode::MODE_IX);
+            auto db = autoColl.ensureDbExists(_opCtx);
+            ASSERT(db) << _nss;
+
+            auto& coll = collection();
+            {
+                WriteUnitOfWork wunit(_opCtx);
+                OpDebug* const nullOpDebug = nullptr;
+                ASSERT_OK(collection_internal::insertDocument(_opCtx,
+                                                              coll.get(),
+                                                              InsertStatement(BSON("_id" << 1 << "a"
+                                                                                         << "dup")),
+                                                              nullOpDebug,
+                                                              true));
+                ASSERT_OK(collection_internal::insertDocument(_opCtx,
+                                                              coll.get(),
+                                                              InsertStatement(BSON("_id" << 2 << "a"
+                                                                                         << "dup")),
+                                                              nullOpDebug,
+                                                              true));
+                wunit.commit();
+            }
         }
+        {
+            AutoGetCollection autoColl(_opCtx, _nss, LockMode::MODE_X);
+            MultiIndexBlock indexer;
 
-        MultiIndexBlock indexer;
+            const BSONObj spec = BSON("name"
+                                      << "a"
+                                      << "key" << BSON("a" << 1) << "v"
+                                      << static_cast<int>(kIndexVersion) << "unique" << true
+                                      << "background" << background);
+            ScopeGuard abortOnExit([&] {
+                indexer.abortIndexBuild(_opCtx, collection(), MultiIndexBlock::kNoopOnCleanUpFn);
+            });
 
-        const BSONObj spec = BSON("name"
-                                  << "a"
-                                  << "key" << BSON("a" << 1) << "v"
-                                  << static_cast<int>(kIndexVersion) << "unique" << true
-                                  << "background" << background);
+            ASSERT_OK(indexer.init(_opCtx, collection(), spec, MultiIndexBlock::kNoopOnInitFn)
+                          .getStatus());
 
-        collLk.emplace(_opCtx, _nss, LockMode::MODE_X);
-        auto abortOnExit = makeGuard([&] {
-            indexer.abortIndexBuild(_opCtx, collection(), MultiIndexBlock::kNoopOnCleanUpFn);
-        });
+            auto& coll = collection();
+            auto desc = coll->getIndexCatalog()->findIndexByName(
+                _opCtx,
+                "a",
+                IndexCatalog::InclusionPolicy::kReady | IndexCatalog::InclusionPolicy::kUnfinished);
+            ASSERT(desc);
 
-        ASSERT_OK(indexer.init(_opCtx, coll, spec, MultiIndexBlock::kNoopOnInitFn).getStatus());
+            // Hybrid index builds check duplicates explicitly.
+            ASSERT_OK(indexer.insertAllDocumentsInCollection(_opCtx, coll.get()));
 
-        auto desc = coll->getIndexCatalog()->findIndexByName(
-            _opCtx,
-            "a",
-            IndexCatalog::InclusionPolicy::kReady | IndexCatalog::InclusionPolicy::kUnfinished);
-        ASSERT(desc);
-
-        // Hybrid index builds check duplicates explicitly.
-        ASSERT_OK(indexer.insertAllDocumentsInCollection(_opCtx, coll));
-
-        auto status = indexer.checkConstraints(_opCtx);
-        ASSERT_EQUALS(status.code(), ErrorCodes::DuplicateKey);
+            auto status = indexer.checkConstraints(_opCtx, coll.get());
+            ASSERT_EQUALS(status.code(), ErrorCodes::DuplicateKey);
+        }
     }
 };
 
@@ -233,21 +242,22 @@ class InsertBuildIndexInterrupt : public IndexBuildBase {
 public:
     void run() {
         {
-            AutoGetOrCreateDb dbRaii(_opCtx, _nss.db(), LockMode::MODE_IX);
-            boost::optional<Lock::CollectionLock> collLk;
-            collLk.emplace(_opCtx, _nss, LockMode::MODE_X);
+            AutoGetCollection autoColl(_opCtx, _nss, LockMode::MODE_X);
+            auto db = autoColl.ensureDbExists(_opCtx);
+            ASSERT(db) << _nss;
 
-            Collection* coll = collection();
+            auto& coll = collection();
             {
                 WriteUnitOfWork wunit(_opCtx);
                 // Drop all indexes including id index.
-                coll->getIndexCatalog()->dropAllIndexes(_opCtx, true);
+                coll.getWritableCollection(_opCtx)->getIndexCatalog()->dropAllIndexes(
+                    _opCtx, coll.getWritableCollection(_opCtx), true, {});
                 // Insert some documents.
                 int32_t nDocs = 1000;
                 OpDebug* const nullOpDebug = nullptr;
                 for (int32_t i = 0; i < nDocs; ++i) {
-                    ASSERT_OK(
-                        coll->insertDocument(_opCtx, InsertStatement(BSON("a" << i)), nullOpDebug));
+                    ASSERT_OK(collection_internal::insertDocument(
+                        _opCtx, coll.get(), InsertStatement(BSON("a" << i)), nullOpDebug));
                 }
                 wunit.commit();
             }
@@ -263,11 +273,11 @@ public:
         }
 
         regenOpCtx();
-        AutoGetDb dbRaii(_opCtx, _nss.db(), LockMode::MODE_IX);
+        AutoGetDb dbRaii(_opCtx, _nss.dbName(), LockMode::MODE_IX);
         boost::optional<Lock::CollectionLock> collLk;
         collLk.emplace(_opCtx, _nss, LockMode::MODE_IX);
         // The new index is not listed in the index catalog because the index build failed.
-        ASSERT(!collection()->getIndexCatalog()->findIndexByName(_opCtx, "a_1"));
+        ASSERT(!collection().get()->getIndexCatalog()->findIndexByName(_opCtx, "a_1"));
     }
 };
 
@@ -282,10 +292,8 @@ public:
 
         {
             // Recreate the collection as capped, without an _id index.
-            AutoGetOrCreateDb dbRaii(_opCtx, _nss.db(), LockMode::MODE_IX);
-            Database* db = dbRaii.getDb();
-            boost::optional<Lock::CollectionLock> collLk;
-            collLk.emplace(_opCtx, _nss, LockMode::MODE_X);
+            AutoGetCollection autoColl(_opCtx, _nss, LockMode::MODE_X);
+            auto db = autoColl.ensureDbExists(_opCtx);
 
             WriteUnitOfWork wunit(_opCtx);
             ASSERT_OK(db->dropCollection(_opCtx, _nss));
@@ -293,13 +301,17 @@ public:
             options.capped = true;
             options.cappedSize = 10 * 1024;
             Collection* coll = db->createCollection(_opCtx, _nss, options);
-            coll->getIndexCatalog()->dropAllIndexes(_opCtx, true);
+            coll->getIndexCatalog()->dropAllIndexes(_opCtx, coll, true, {});
             // Insert some documents.
             int32_t nDocs = 1000;
             OpDebug* const nullOpDebug = nullptr;
             for (int32_t i = 0; i < nDocs; ++i) {
-                ASSERT_OK(coll->insertDocument(
-                    _opCtx, InsertStatement(BSON("_id" << i)), nullOpDebug, true));
+                ASSERT_OK(collection_internal::insertDocument(
+                    _opCtx,
+                    CollectionPtr(coll, CollectionPtr::NoYieldTag{}),
+                    InsertStatement(BSON("_id" << i)),
+                    nullOpDebug,
+                    true));
             }
             wunit.commit();
             // Request an interrupt.
@@ -312,21 +324,19 @@ public:
             getGlobalServiceContext()->unsetKillAllOperations();
         }
         regenOpCtx();
-        AutoGetOrCreateDb dbRaii(_opCtx, _nss.db(), LockMode::MODE_IX);
-        boost::optional<Lock::CollectionLock> collLk;
-        collLk.emplace(_opCtx, _nss, LockMode::MODE_IX);
+        AutoGetCollection autoColl(_opCtx, _nss, LockMode::MODE_IX);
 
         // The new index is not listed in the index catalog because the index build failed.
-        ASSERT(!collection()->getIndexCatalog()->findIndexByName(_opCtx, "_id_"));
+        ASSERT(!collection().get()->getIndexCatalog()->findIndexByName(_opCtx, "_id_"));
     }
 };  // namespace IndexUpdateTests
 
 Status IndexBuildBase::createIndex(const BSONObj& indexSpec) {
-    Lock::DBLock dbLk(_opCtx, _nss.db(), MODE_IX);
+    Lock::DBLock dbLk(_opCtx, _nss.dbName(), MODE_IX);
     Lock::CollectionLock collLk(_opCtx, _nss, MODE_X);
 
     MultiIndexBlock indexer;
-    auto abortOnExit = makeGuard(
+    ScopeGuard abortOnExit(
         [&] { indexer.abortIndexBuild(_opCtx, collection(), MultiIndexBlock::kNoopOnCleanUpFn); });
     Status status =
         indexer.init(_opCtx, collection(), indexSpec, MultiIndexBlock::kNoopOnInitFn).getStatus();
@@ -336,17 +346,17 @@ Status IndexBuildBase::createIndex(const BSONObj& indexSpec) {
     if (!status.isOK()) {
         return status;
     }
-    status = indexer.insertAllDocumentsInCollection(_opCtx, collection());
+    status = indexer.insertAllDocumentsInCollection(_opCtx, collection().get());
     if (!status.isOK()) {
         return status;
     }
-    status = indexer.checkConstraints(_opCtx);
+    status = indexer.checkConstraints(_opCtx, collection().get());
     if (!status.isOK()) {
         return status;
     }
     WriteUnitOfWork wunit(_opCtx);
     ASSERT_OK(indexer.commit(_opCtx,
-                             collection(),
+                             collection().getWritableCollection(_opCtx),
                              MultiIndexBlock::kNoopOnCreateEachFn,
                              MultiIndexBlock::kNoopOnCommitFn));
     wunit.commit();
@@ -374,8 +384,9 @@ public:
         ASSERT_EQUALS(ErrorCodes::IndexOptionsConflict,
                       createIndex(BSON("name"
                                        << "x"
-                                       << "unique" << true << "key" << BSON("x" << 1 << "y" << 1)
-                                       << "v" << static_cast<int>(kIndexVersion))));
+                                       << "key" << BSON("x" << 1 << "y" << 1) << "storageEngine"
+                                       << BSON("wiredTiger" << BSONObj()) << "v"
+                                       << static_cast<int>(kIndexVersion))));
     }
 };
 
@@ -455,28 +466,26 @@ public:
 class SameSpecDifferentUnique : public ComplexIndex {
 public:
     void run() {
-        ASSERT_EQUALS(ErrorCodes::IndexOptionsConflict,
-                      createIndex(BSON("name"
-                                       << "super2"
-                                       << "unique" << false << "sparse" << true
-                                       << "expireAfterSeconds" << 3600 << "key"
-                                       << BSON("superIdx"
-                                               << "2d")
-                                       << "v" << static_cast<int>(kIndexVersion))));
+        ASSERT_OK(createIndex(BSON("name"
+                                   << "super2"
+                                   << "unique" << false << "sparse" << true << "expireAfterSeconds"
+                                   << 3600 << "key"
+                                   << BSON("superIdx"
+                                           << "2d")
+                                   << "v" << static_cast<int>(kIndexVersion))));
     }
 };
 
 class SameSpecDifferentSparse : public ComplexIndex {
 public:
     void run() {
-        ASSERT_EQUALS(ErrorCodes::IndexOptionsConflict,
-                      createIndex(BSON("name"
-                                       << "super2"
-                                       << "unique" << 1 << "sparse" << false << "background" << true
-                                       << "expireAfterSeconds" << 3600 << "key"
-                                       << BSON("superIdx"
-                                               << "2d")
-                                       << "v" << static_cast<int>(kIndexVersion))));
+        ASSERT_OK(createIndex(BSON("name"
+                                   << "super3"
+                                   << "unique" << 1 << "sparse" << false << "background" << true
+                                   << "expireAfterSeconds" << 3600 << "key"
+                                   << BSON("superIdx"
+                                           << "2d")
+                                   << "v" << static_cast<int>(kIndexVersion))));
     }
 };
 
@@ -485,7 +494,7 @@ public:
     void run() {
         ASSERT_EQUALS(ErrorCodes::IndexOptionsConflict,
                       createIndex(BSON("name"
-                                       << "super2"
+                                       << "super4"
                                        << "unique" << 1 << "sparse" << true << "expireAfterSeconds"
                                        << 2400 << "key"
                                        << BSON("superIdx"
@@ -544,7 +553,7 @@ protected:
 class IndexCatatalogFixIndexKey : public IndexBuildBase {
 public:
     void run() {
-        auto indexCatalog = collection()->getIndexCatalog();
+        auto indexCatalog = collection().get()->getIndexCatalog();
 
         ASSERT_BSONOBJ_EQ(BSON("x" << 1), indexCatalog->fixIndexKey(BSON("x" << 1)));
 
@@ -564,9 +573,9 @@ public:
         indexSpec.addKey("a").addOptions(BSON("collation" << BSON("locale"
                                                                   << "fr")));
         client.createIndex(_ns, indexSpec);
-        client.insert(_ns, BSON("a" << BSONSymbol("mySymbol")));
-        ASSERT_EQUALS(client.getLastErrorDetailed()["code"].numberInt(),
-                      ErrorCodes::CannotBuildIndexKeys);
+
+        auto response = client.insertAcknowledged(_ns, {BSON("a" << BSONSymbol("mySymbol"))});
+        ASSERT_EQUALS(getStatusFromWriteCommandReply(response), ErrorCodes::CannotBuildIndexKeys);
         ASSERT_EQUALS(client.count(_nss), 0U);
     }
 };
@@ -580,8 +589,10 @@ public:
         IndexSpec indexSpec;
         indexSpec.addKey("a");
         client.createIndex(_ns, indexSpec);
-        client.insert(_ns, BSON("a" << BSONSymbol("mySymbol")));
-        ASSERT(client.getLastError().empty());
+
+        auto response = client.insertAcknowledged(_ns, {BSON("a" << BSONSymbol("mySymbol"))});
+        ASSERT_OK(getStatusFromWriteCommandReply(response));
+        ASSERT_EQUALS(response["n"].Int(), 1);
         ASSERT_EQUALS(client.count(_nss), 1U);
     }
 };
@@ -596,9 +607,10 @@ public:
         indexSpec.addKey("a").addOptions(BSON("collation" << BSON("locale"
                                                                   << "fr")));
         client.createIndex(_ns, indexSpec);
-        client.insert(_ns, BSON("a" << BSON("b" << 99 << "c" << BSONSymbol("mySymbol"))));
-        ASSERT_EQUALS(client.getLastErrorDetailed()["code"].numberInt(),
-                      ErrorCodes::CannotBuildIndexKeys);
+
+        auto response = client.insertAcknowledged(
+            _ns, {BSON("a" << BSON("b" << 99 << "c" << BSONSymbol("mySymbol")))});
+        ASSERT_EQUALS(getStatusFromWriteCommandReply(response), ErrorCodes::CannotBuildIndexKeys);
         ASSERT_EQUALS(client.count(_nss), 0U);
     }
 };
@@ -613,9 +625,10 @@ public:
         indexSpec.addKey("a").addOptions(BSON("collation" << BSON("locale"
                                                                   << "fr")));
         client.createIndex(_ns, indexSpec);
-        client.insert(_ns, BSON("a" << BSON_ARRAY(99 << BSONSymbol("mySymbol"))));
-        ASSERT_EQUALS(client.getLastErrorDetailed()["code"].numberInt(),
-                      ErrorCodes::CannotBuildIndexKeys);
+
+        auto response =
+            client.insertAcknowledged(_ns, {BSON("a" << BSON_ARRAY(99 << BSONSymbol("mySymbol")))});
+        ASSERT_EQUALS(getStatusFromWriteCommandReply(response), ErrorCodes::CannotBuildIndexKeys);
         ASSERT_EQUALS(client.count(_nss), 0U);
     }
 };
@@ -654,9 +667,10 @@ public:
         IndexSpec indexSpec;
         indexSpec.addKey("a");
         client.createIndex(_ns, indexSpec);
-        client.insert(_ns, BSON("a" << BSON_ARRAY(99 << BSONSymbol("mySymbol"))));
-        ASSERT_EQUALS(client.getLastErrorDetailed()["code"].numberInt(),
-                      ErrorCodes::CannotBuildIndexKeys);
+
+        auto response =
+            client.insertAcknowledged(_ns, {BSON("a" << BSON_ARRAY(99 << BSONSymbol("mySymbol")))});
+        ASSERT_EQUALS(getStatusFromWriteCommandReply(response), ErrorCodes::CannotBuildIndexKeys);
     }
 };
 
@@ -704,3 +718,4 @@ public:
 OldStyleSuiteInitializer<IndexUpdateTests> indexUpdateTests;
 
 }  // namespace IndexUpdateTests
+}  // namespace mongo
