@@ -31,10 +31,6 @@
  * Tests for json.{h,cpp} code and BSONObj::jsonString()
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
-
-#include "mongo/platform/basic.h"
-
 #include <boost/property_tree/json_parser.hpp>
 #include <boost/property_tree/ptree.hpp>
 #include <fmt/format.h>
@@ -47,10 +43,12 @@
 #include "mongo/dbtests/dbtests.h"
 #include "mongo/logv2/log.h"
 #include "mongo/platform/decimal128.h"
-#include "mongo/unittest/unittest.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
+namespace mongo {
 namespace {
+
 std::string makeJsonEquvalent(const std::string& json) {
     boost::property_tree::ptree tree;
 
@@ -402,7 +400,8 @@ public:
         char* _oldTimezonePtr = getenv("TZ");
         _oldTimezone = std::string(_oldTimezonePtr ? _oldTimezonePtr : "");
         if (-1 == putenv(tzEnvString)) {
-            FAIL(errnoWithDescription());
+            auto ec = lastSystemError();
+            FAIL(errorMessage(ec));
         }
         tzset();
     }
@@ -418,7 +417,8 @@ public:
             }
 #else
             if (-1 == setenv("TZ", _oldTimezone.c_str(), 1)) {
-                FAIL(errnoWithDescription());
+                auto ec = lastSystemError();
+                FAIL(errorMessage(ec));
             }
 #endif
         } else {
@@ -431,7 +431,8 @@ public:
             }
 #else
             if (-1 == unsetenv("TZ")) {
-                FAIL(errnoWithDescription());
+                auto ec = lastSystemError();
+                FAIL(errorMessage(ec));
             }
 #endif
         }
@@ -624,7 +625,7 @@ void assertEquals(const std::string& json,
 }
 
 void checkEquivalence(const std::string& json, const BSONObj& bson) {
-    ASSERT(fromjson(json).valid(BSONVersion::kLatest));
+    ASSERT(validateBSON(fromjson(json)).isOK());
     assertEquals(json, bson, fromjson(json), "mode: json-to-bson");
     assertEquals(json, bson, fromjson(tojson(bson)), "mode: <default>");
     assertEquals(json, bson, fromjson(tojson(bson, LegacyStrict)), "mode: strict");
@@ -728,7 +729,7 @@ TEST(FromJsonTest, Utf8Test) {
     using namespace std::literals::string_literals;
     const std::string u = "\xea\x80\x80\xea\x80\x80"s;
     BSONObj built = B().append("a", u).obj();
-    ASSERT_EQUALS(built.firstElement().valuestr(), u);
+    ASSERT_EQUALS(built.firstElement().str(), u);
 
     checkEquivalenceEach({
         // EscapedUnicodeToUtf8
@@ -834,6 +835,7 @@ TEST(FromJsonTest, BinDataTypes) {
         {0x04, newUUID},
         {0x05, MD5Type},
         {0x06, Encrypt},
+        {0x07, Column},
         {0x80, bdtCustom},
     };
     for (const auto& ts : specs) {
@@ -1194,3 +1196,4 @@ TEST(FromJsonTest, MinMaxKey) {
 
 }  // namespace FromJsonTests
 }  // namespace
+}  // namespace mongo
