@@ -43,24 +43,19 @@
 #include <string>
 #include <vector>
 
-using mongo::BSONObj;
-using mongo::ConnectionString;
-using mongo::MockDBClientConnection;
-using mongo::MockRemoteDBServer;
-using mongo::NamespaceString;
-using mongo::Query;
-
 using std::string;
 using std::vector;
 
-namespace mongo_test {
+namespace mongo {
 
 TEST(MockDBClientConnTest, ServerAddress) {
     MockRemoteDBServer server("test");
     MockDBClientConnection conn(&server);
+    uassertStatusOK(conn.connect(server.getServerHostAndPort(), mongo::StringData(), boost::none));
 
-    ASSERT_EQUALS("test", conn.getServerAddress());
-    ASSERT_EQUALS("test", conn.toString());
+    ASSERT_EQUALS("test:27017", conn.getServerAddress());
+    ASSERT_EQUALS("test:27017", conn.toString());
+    ASSERT_EQUALS(mongo::HostAndPort("test"), conn.getServerHostAndPort());
 }
 
 TEST(MockDBClientConnTest, QueryCount) {
@@ -70,25 +65,72 @@ TEST(MockDBClientConnTest, QueryCount) {
         MockDBClientConnection conn(&server);
 
         ASSERT_EQUALS(0U, server.getQueryCount());
-        conn.query(NamespaceString("foo.bar"));
+        conn.find(FindCommandRequest(NamespaceString("foo.bar")));
     }
 
     ASSERT_EQUALS(1U, server.getQueryCount());
 
     {
         MockDBClientConnection conn(&server);
-        conn.query(NamespaceString("foo.bar"));
+        conn.find(FindCommandRequest(NamespaceString("foo.bar")));
         ASSERT_EQUALS(2U, server.getQueryCount());
     }
 }
 
+TEST(MockDBClientConnTest, SkipBasedOnResumeAfter) {
+    MockRemoteDBServer server{"test"};
+    const std::string ns{"test.user"};
+
+    {
+        MockDBClientConnection conn{&server};
+        server.insert(ns, BSON("x" << 1));
+        server.insert(ns, BSON("y" << 2));
+        server.insert(ns, BSON("z" << 3));
+    }
+
+    {
+        MockDBClientConnection conn{&server};
+        FindCommandRequest findRequest{FindCommandRequest{NamespaceString{ns}}};
+        findRequest.setResumeAfter(BSON("n" << 2));
+
+        auto cursor = conn.find(std::move(findRequest));
+        ASSERT_EQ(1, cursor->itcount());
+    }
+}
+
+TEST(MockDBClientConnTest, RequestResumeToken) {
+    MockRemoteDBServer server{"test"};
+    const std::string ns{"test.user"};
+
+    {
+        MockDBClientConnection conn{&server};
+        server.insert(ns, BSON("_id" << 1));
+        server.insert(ns, BSON("_id" << 2));
+        server.insert(ns, BSON("_id" << 3));
+    }
+
+    {
+        MockDBClientConnection conn{&server};
+        FindCommandRequest findRequest{FindCommandRequest{NamespaceString{ns}}};
+        findRequest.setRequestResumeToken(true);
+        findRequest.setBatchSize(2);
+
+        auto cursor = conn.find(std::move(findRequest));
+        ASSERT(cursor->more());
+        auto pbrt = cursor->getPostBatchResumeToken();
+        ASSERT(pbrt);
+        ASSERT_BSONOBJ_EQ(*pbrt, BSON("n" << 2));
+    }
+}
+
 TEST(MockDBClientConnTest, InsertAndQuery) {
     MockRemoteDBServer server("test");
     const string ns("test.user");
 
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns)));
         ASSERT(!cursor->more());
 
         server.insert(ns, BSON("x" << 1));
@@ -97,7 +139,8 @@ TEST(MockDBClientConnTest, InsertAndQuery) {
 
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns)));
 
         ASSERT(cursor->more());
         BSONObj firstDoc = cursor->next();
@@ -113,7 +156,8 @@ TEST(MockDBClientConnTest, InsertAndQuery) {
     // Make sure that repeated calls will still give you the same result
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns)));
 
         ASSERT(cursor->more());
         BSONObj firstDoc = cursor->next();
@@ -135,7 +179,8 @@ TEST(MockDBClientConnTest, InsertAndQueryTwice) {
 
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns)));
 
         ASSERT(cursor->more());
         BSONObj firstDoc = cursor->next();
@@ -146,7 +191,8 @@ TEST(MockDBClientConnTest, InsertAndQueryTwice) {
 
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns)));
 
         ASSERT(cursor->more());
         BSONObj firstDoc = cursor->next();
@@ -166,7 +212,8 @@ TEST(MockDBClientConnTest, QueryWithNoResults) {
 
     server.insert(ns, BSON("x" << 1));
     MockDBClientConnection conn(&server);
-    std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString("other.ns"));
+    std::unique_ptr<mongo::DBClientCursor> cursor =
+        conn.find(FindCommandRequest(NamespaceString("other.ns")));
 
     ASSERT(!cursor->more());
 }
@@ -197,7 +244,8 @@ TEST(MockDBClientConnTest, MultiNSInsertAndQuery) {
 
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns1));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns1)));
 
         ASSERT(cursor->more());
         BSONObj firstDoc = cursor->next();
@@ -212,7 +260,8 @@ TEST(MockDBClientConnTest, MultiNSInsertAndQuery) {
 
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns2));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns2)));
 
         ASSERT(cursor->more());
         BSONObj firstDoc = cursor->next();
@@ -231,7 +280,8 @@ TEST(MockDBClientConnTest, MultiNSInsertAndQuery) {
 
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns3));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns3)));
 
         ASSERT(cursor->more());
         BSONObj firstDoc = cursor->next();
@@ -247,7 +297,8 @@ TEST(MockDBClientConnTest, SimpleRemove) {
 
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns)));
         ASSERT(!cursor->more());
 
         conn.insert(ns, BSON("x" << 1));
@@ -256,12 +307,13 @@ TEST(MockDBClientConnTest, SimpleRemove) {
 
     {
         MockDBClientConnection conn(&server);
-        conn.remove(ns, Query(), false);
+        conn.remove(ns, BSONObj{} /*filter*/);
     }
 
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns)));
 
         ASSERT(!cursor->more());
     }
@@ -269,7 +321,8 @@ TEST(MockDBClientConnTest, SimpleRemove) {
     // Make sure that repeated calls will still give you the same result
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns)));
 
         ASSERT(!cursor->more());
     }
@@ -301,15 +354,17 @@ TEST(MockDBClientConnTest, MultiNSRemove) {
 
     {
         MockDBClientConnection conn(&server);
-        conn.remove(ns2, Query(), false);
+        conn.remove(ns2, BSONObj{} /*filter*/);
 
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns2));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns2)));
         ASSERT(!cursor->more());
     }
 
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns1));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns1)));
 
         ASSERT(cursor->more());
         BSONObj firstDoc = cursor->next();
@@ -324,7 +379,8 @@ TEST(MockDBClientConnTest, MultiNSRemove) {
 
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns3));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns3)));
 
         ASSERT(cursor->more());
         BSONObj firstDoc = cursor->next();
@@ -349,7 +405,7 @@ TEST(MockDBClientConnTest, InsertAfterRemove) {
 
     {
         MockDBClientConnection conn(&server);
-        conn.remove(ns, Query(), false);
+        conn.remove(ns, BSONObj{} /*filter*/);
     }
 
     {
@@ -359,7 +415,8 @@ TEST(MockDBClientConnTest, InsertAfterRemove) {
 
     {
         MockDBClientConnection conn(&server);
-        std::unique_ptr<mongo::DBClientCursor> cursor = conn.query(NamespaceString(ns));
+        std::unique_ptr<mongo::DBClientCursor> cursor =
+            conn.find(FindCommandRequest(NamespaceString(ns)));
 
         ASSERT(cursor->more());
         BSONObj firstDoc = cursor->next();
@@ -455,30 +512,9 @@ TEST(MockDBClientConnTest, CyclingCmd) {
     }
 }
 
-TEST(MockDBClientConnTest, CmdWithMultiFields) {
-    MockRemoteDBServer server("test");
-    server.setCommandReply("getLastError", BSON("ok" << 1 << "n" << 10));
-
-    MockDBClientConnection conn(&server);
-    BSONObj response;
-    ASSERT(conn.runCommand(
-        "foo.baz", BSON("getLastError" << 1 << "w" << 2 << "journal" << true), response));
-
-    ASSERT_EQUALS(10, response["n"].numberInt());
-}
-
-TEST(MockDBClientConnTest, BadCmd) {
-    MockRemoteDBServer server("test");
-    server.setCommandReply("getLastError", BSON("ok" << 0));
-
-    MockDBClientConnection conn(&server);
-    BSONObj response;
-    ASSERT(!conn.runCommand("foo.baz", BSON("getLastError" << 1), response));
-}
-
 TEST(MockDBClientConnTest, MultipleStoredResponse) {
     MockRemoteDBServer server("test");
-    server.setCommandReply("getLastError", BSON("ok" << 1 << "n" << 10));
+    server.setCommandReply("serverStatus", BSON("ok" << 0));
     server.setCommandReply("isMaster", BSON("ok" << 1 << "secondary" << false));
 
     MockDBClientConnection conn(&server);
@@ -493,8 +529,7 @@ TEST(MockDBClientConnTest, MultipleStoredResponse) {
 
     {
         BSONObj response;
-        ASSERT(conn.runCommand("a.b", BSON("getLastError" << 1), response));
-        ASSERT_EQUALS(10, response["n"].numberInt());
+        ASSERT(!conn.runCommand("a.b", BSON("serverStatus" << 1), response));
     }
 }
 
@@ -530,7 +565,8 @@ TEST(MockDBClientConnTest, Shutdown) {
         server.shutdown();
         ASSERT(!server.isRunning());
 
-        ASSERT_THROWS(conn.query(NamespaceString("test.user")), mongo::NetworkException);
+        ASSERT_THROWS(conn.find(FindCommandRequest(NamespaceString("test.user"))),
+                      mongo::NetworkException);
     }
 
     {
@@ -552,16 +588,18 @@ TEST(MockDBClientConnTest, Restart) {
 
     // Do some queries and commands then check the counters later that
     // new instance still has it
-    conn1.query(NamespaceString("test.user"));
+    conn1.find(FindCommandRequest(NamespaceString("test.user")));
     BSONObj response;
     conn1.runCommand("test.user", BSON("serverStatus" << 1), response);
 
     server.shutdown();
-    ASSERT_THROWS(conn1.query(NamespaceString("test.user")), mongo::NetworkException);
+    ASSERT_THROWS(conn1.find(FindCommandRequest(NamespaceString("test.user"))),
+                  mongo::NetworkException);
 
     // New connections shouldn't work either
     MockDBClientConnection conn2(&server);
-    ASSERT_THROWS(conn2.query(NamespaceString("test.user")), mongo::NetworkException);
+    ASSERT_THROWS(conn2.find(FindCommandRequest(NamespaceString("test.user"))),
+                  mongo::NetworkException);
 
     ASSERT_EQUALS(1U, server.getQueryCount());
     ASSERT_EQUALS(1U, server.getCmdCount());
@@ -571,12 +609,14 @@ TEST(MockDBClientConnTest, Restart) {
 
     {
         MockDBClientConnection conn(&server);
-        conn.query(NamespaceString("test.user"));
+        conn.find(FindCommandRequest(NamespaceString("test.user")));
     }
 
     // Old connections still shouldn't work
-    ASSERT_THROWS(conn1.query(NamespaceString("test.user")), mongo::NetworkException);
-    ASSERT_THROWS(conn2.query(NamespaceString("test.user")), mongo::NetworkException);
+    ASSERT_THROWS(conn1.find(FindCommandRequest(NamespaceString("test.user"))),
+                  mongo::NetworkException);
+    ASSERT_THROWS(conn2.find(FindCommandRequest(NamespaceString("test.user"))),
+                  mongo::NetworkException);
 
     ASSERT_EQUALS(2U, server.getQueryCount());
     ASSERT_EQUALS(1U, server.getCmdCount());
@@ -587,7 +627,7 @@ TEST(MockDBClientConnTest, ClearCounter) {
     server.setCommandReply("serverStatus", BSON("ok" << 1));
 
     MockDBClientConnection conn(&server);
-    conn.query(NamespaceString("test.user"));
+    conn.find(FindCommandRequest(FindCommandRequest(NamespaceString("test.user"))));
     BSONObj response;
     conn.runCommand("test.user", BSON("serverStatus" << 1), response);
 
@@ -605,7 +645,7 @@ TEST(MockDBClientConnTest, Delay) {
 
     {
         mongo::Timer timer;
-        conn.query(NamespaceString("x.x"));
+        conn.find(FindCommandRequest(NamespaceString("x.x")));
         const int nowInMilliSec = timer.millis();
         // Use a more lenient lower bound since some platforms like Windows
         // don't guarantee that sleeps will not wake up earlier (unlike
@@ -635,14 +675,8 @@ TEST(MockDBClientConnTest, SimulateCallAndRecvResponses) {
     MockRemoteDBServer server("test");
     MockDBClientConnection conn(&server);
 
-    mongo::DBClientCursor cursor(&conn,
-                                 mongo::NamespaceStringOrUUID(nss),
-                                 Query().obj,
-                                 0,
-                                 0,
-                                 nullptr,
-                                 mongo::QueryOption_Exhaust,
-                                 0);
+    FindCommandRequest findCmd{nss};
+    mongo::DBClientCursor cursor(&conn, findCmd, ReadPreferenceSetting{}, true /*isExhaust*/);
     cursor.setBatchSize(2);
 
     // Two batches from the initial find and getMore command.
@@ -707,8 +741,7 @@ TEST(MockDBClientConnTest, SimulateCallErrors) {
     MockRemoteDBServer server("test");
     MockDBClientConnection conn(&server);
 
-    mongo::DBClientCursor cursor(
-        &conn, mongo::NamespaceStringOrUUID(nss), Query().obj, 0, 0, nullptr, 0, 0);
+    mongo::DBClientCursor cursor(&conn, FindCommandRequest{nss}, ReadPreferenceSetting{}, false);
 
     // Test network exception and error response for the initial find.
     MockDBClientConnection::Responses callResponses = {
@@ -755,14 +788,8 @@ TEST(MockDBClientConnTest, SimulateRecvErrors) {
     MockRemoteDBServer server("test");
     MockDBClientConnection conn(&server);
 
-    mongo::DBClientCursor cursor(&conn,
-                                 mongo::NamespaceStringOrUUID(nss),
-                                 Query().obj,
-                                 0,
-                                 0,
-                                 nullptr,
-                                 mongo::QueryOption_Exhaust,
-                                 0);
+    mongo::DBClientCursor cursor(
+        &conn, FindCommandRequest{nss}, ReadPreferenceSetting{}, true /*isExhaust*/);
 
     runUntilExhaustRecv(&conn, &cursor);
 
@@ -803,14 +830,8 @@ TEST(MockDBClientConnTest, BlockingNetwork) {
     MockRemoteDBServer server("test");
     MockDBClientConnection conn(&server);
 
-    mongo::DBClientCursor cursor(&conn,
-                                 mongo::NamespaceStringOrUUID(nss),
-                                 Query().obj,
-                                 0,
-                                 0,
-                                 nullptr,
-                                 mongo::QueryOption_Exhaust,
-                                 0);
+    mongo::DBClientCursor cursor(
+        &conn, FindCommandRequest{nss}, ReadPreferenceSetting{}, true /*isExhaust*/);
     cursor.setBatchSize(1);
 
     mongo::stdx::thread cursorThread([&] {
@@ -862,15 +883,10 @@ TEST(MockDBClientConnTest, ShutdownServerBeforeCall) {
     MockRemoteDBServer server("test");
     MockDBClientConnection conn(&server);
 
-    ASSERT_OK(conn.connect(mongo::HostAndPort("localhost", 12345), mongo::StringData()));
-    mongo::DBClientCursor cursor(&conn,
-                                 mongo::NamespaceStringOrUUID(nss),
-                                 Query().obj,
-                                 0,
-                                 0,
-                                 nullptr,
-                                 mongo::QueryOption_Exhaust,
-                                 0);
+    ASSERT_OK(
+        conn.connect(mongo::HostAndPort("localhost", 12345), mongo::StringData(), boost::none));
+    mongo::DBClientCursor cursor(
+        &conn, FindCommandRequest{nss}, ReadPreferenceSetting{}, true /*isExhaust*/);
 
     // Shut down server before call.
     server.shutdown();
@@ -888,14 +904,8 @@ TEST(MockDBClientConnTest, ShutdownServerAfterCall) {
     MockRemoteDBServer server("test");
     MockDBClientConnection conn(&server);
 
-    mongo::DBClientCursor cursor(&conn,
-                                 mongo::NamespaceStringOrUUID(nss),
-                                 Query().obj,
-                                 0,
-                                 0,
-                                 nullptr,
-                                 mongo::QueryOption_Exhaust,
-                                 0);
+    mongo::DBClientCursor cursor(
+        &conn, FindCommandRequest{nss}, ReadPreferenceSetting{}, true /*isExhaust*/);
 
     mongo::stdx::thread cursorThread([&] {
         ASSERT_THROWS_CODE(cursor.init(), mongo::DBException, mongo::ErrorCodes::HostUnreachable);
@@ -917,15 +927,10 @@ TEST(MockDBClientConnTest, ConnectionAutoReconnect) {
     MockRemoteDBServer server("test");
     MockDBClientConnection conn(&server, autoReconnect);
 
-    ASSERT_OK(conn.connect(mongo::HostAndPort("localhost", 12345), mongo::StringData()));
-    mongo::DBClientCursor cursor(&conn,
-                                 mongo::NamespaceStringOrUUID(nss),
-                                 Query().obj,
-                                 0,
-                                 0,
-                                 nullptr,
-                                 mongo::QueryOption_Exhaust,
-                                 0);
+    ASSERT_OK(
+        conn.connect(mongo::HostAndPort("localhost", 12345), mongo::StringData(), boost::none));
+    mongo::DBClientCursor cursor(
+        &conn, FindCommandRequest{nss}, ReadPreferenceSetting{}, true /*isExhaust*/);
 
     server.shutdown();
 
@@ -950,14 +955,8 @@ TEST(MockDBClientConnTest, ShutdownServerBeforeRecv) {
     MockRemoteDBServer server("test");
     MockDBClientConnection conn(&server, autoReconnect);
 
-    mongo::DBClientCursor cursor(&conn,
-                                 mongo::NamespaceStringOrUUID(nss),
-                                 Query().obj,
-                                 0,
-                                 0,
-                                 nullptr,
-                                 mongo::QueryOption_Exhaust,
-                                 0);
+    mongo::DBClientCursor cursor(
+        &conn, FindCommandRequest{nss}, ReadPreferenceSetting{}, true /*isExhaust*/);
 
     runUntilExhaustRecv(&conn, &cursor);
 
@@ -975,14 +974,8 @@ TEST(MockDBClientConnTest, ShutdownServerAfterRecv) {
     MockRemoteDBServer server("test");
     MockDBClientConnection conn(&server);
 
-    mongo::DBClientCursor cursor(&conn,
-                                 mongo::NamespaceStringOrUUID(nss),
-                                 Query().obj,
-                                 0,
-                                 0,
-                                 nullptr,
-                                 mongo::QueryOption_Exhaust,
-                                 0);
+    mongo::DBClientCursor cursor(
+        &conn, FindCommandRequest{nss}, ReadPreferenceSetting{}, true /*isExhaust*/);
 
     runUntilExhaustRecv(&conn, &cursor);
 
@@ -1000,4 +993,4 @@ TEST(MockDBClientConnTest, ShutdownServerAfterRecv) {
 
     cursorThread.join();
 }
-}  // namespace mongo_test
+}  // namespace mongo
