@@ -54,20 +54,20 @@ public:
     CountBase() : _client(&_opCtx) {}
 
     virtual ~CountBase() {
-        dbtests::WriteContextForTests ctx(&_opCtx, ns());
-        _client.dropCollection(ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns().ns());
+        _client.dropCollection(ns().ns());
     }
 
     void addIndex(const BSONObj& obj) {
-        ASSERT_OK(dbtests::createIndex(&_opCtx, ns(), obj));
+        ASSERT_OK(dbtests::createIndex(&_opCtx, ns().ns(), obj));
     }
 
     void insert(const BSONObj& obj) {
-        _client.insert(ns(), obj);
+        _client.insert(ns().ns(), obj);
     }
 
     void remove(const BSONObj& obj) {
-        _client.remove(ns(), obj);
+        _client.remove(ns().ns(), obj);
     }
 
     /*
@@ -89,22 +89,25 @@ public:
         return countWorks;
     }
 
+    CollectionPtr getCollection() {
+        return CollectionCatalog::get(&_opCtx)->lookupCollectionByNamespace(&_opCtx, ns());
+    }
+
     const IndexDescriptor* getIndex(Database* db, const BSONObj& obj) {
-        Collection* collection = CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(
-            &_opCtx, NamespaceString(ns()));
         std::vector<const IndexDescriptor*> indexes;
-        collection->getIndexCatalog()->findIndexesByKeyPattern(
+        getCollection()->getIndexCatalog()->findIndexesByKeyPattern(
             &_opCtx, obj, IndexCatalog::InclusionPolicy::kReady, &indexes);
         return indexes.empty() ? nullptr : indexes[0];
     }
 
     CountScanParams makeCountScanParams(OperationContext* opCtx,
+                                        const CollectionPtr& collection,
                                         const IndexDescriptor* descriptor) {
-        return {opCtx, descriptor};
+        return {opCtx, collection, descriptor};
     }
 
-    static const char* ns() {
-        return "unittests.QueryStageCountScanScan";
+    static NamespaceString ns() {
+        return {"unittests", "QueryStageCountScanScan"};
     }
 
 protected:
@@ -112,7 +115,7 @@ protected:
     OperationContext& _opCtx = *_txnPtr;
 
     boost::intrusive_ptr<ExpressionContext> _expCtx =
-        make_intrusive<ExpressionContext>(&_opCtx, nullptr, NamespaceString(ns()));
+        make_intrusive<ExpressionContext>(&_opCtx, nullptr, ns());
 
 private:
     DBDirectClient _client;
@@ -125,7 +128,7 @@ private:
 class QueryStageCountScanDups : public CountBase {
 public:
     void run() {
-        dbtests::WriteContextForTests ctx(&_opCtx, ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns().ns());
 
         // Insert some docs
         insert(BSON("a" << BSON_ARRAY(5 << 7)));
@@ -134,15 +137,17 @@ public:
         // Add an index on a:1
         addIndex(BSON("a" << 1));
 
+        const CollectionPtr coll = ctx.getCollection();
+
         // Set up the count stage
-        auto params = makeCountScanParams(&_opCtx, getIndex(ctx.db(), BSON("a" << 1)));
+        auto params = makeCountScanParams(&_opCtx, coll, getIndex(ctx.db(), BSON("a" << 1)));
         params.startKey = BSON("a" << 1);
         params.startKeyInclusive = true;
         params.endKey = BSON("a" << 10);
         params.endKeyInclusive = true;
 
         WorkingSet ws;
-        CountScan count(_expCtx.get(), params, &ws);
+        CountScan count(_expCtx.get(), coll, params, &ws);
 
         int numCounted = runCount(&count);
         ASSERT_EQUALS(2, numCounted);
@@ -155,7 +160,7 @@ public:
 class QueryStageCountScanInclusiveBounds : public CountBase {
 public:
     void run() {
-        dbtests::WriteContextForTests ctx(&_opCtx, ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns().ns());
 
         // Insert some docs
         for (int i = 0; i < 10; ++i) {
@@ -165,15 +170,17 @@ public:
         // Add an index
         addIndex(BSON("a" << 1));
 
+        const CollectionPtr coll = ctx.getCollection();
+
         // Set up the count stage
-        auto params = makeCountScanParams(&_opCtx, getIndex(ctx.db(), BSON("a" << 1)));
+        auto params = makeCountScanParams(&_opCtx, coll, getIndex(ctx.db(), BSON("a" << 1)));
         params.startKey = BSON("" << 3);
         params.startKeyInclusive = true;
         params.endKey = BSON("" << 7);
         params.endKeyInclusive = true;
 
         WorkingSet ws;
-        CountScan count(_expCtx.get(), params, &ws);
+        CountScan count(_expCtx.get(), coll, params, &ws);
 
         int numCounted = runCount(&count);
         ASSERT_EQUALS(5, numCounted);
@@ -186,7 +193,7 @@ public:
 class QueryStageCountScanExclusiveBounds : public CountBase {
 public:
     void run() {
-        dbtests::WriteContextForTests ctx(&_opCtx, ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns().ns());
 
         // Insert some docs
         for (int i = 0; i < 10; ++i) {
@@ -196,15 +203,17 @@ public:
         // Add an index
         addIndex(BSON("a" << 1));
 
+        const CollectionPtr coll = ctx.getCollection();
+
         // Set up the count stage
-        auto params = makeCountScanParams(&_opCtx, getIndex(ctx.db(), BSON("a" << 1)));
+        auto params = makeCountScanParams(&_opCtx, coll, getIndex(ctx.db(), BSON("a" << 1)));
         params.startKey = BSON("" << 3);
         params.startKeyInclusive = false;
         params.endKey = BSON("" << 7);
         params.endKeyInclusive = false;
 
         WorkingSet ws;
-        CountScan count(_expCtx.get(), params, &ws);
+        CountScan count(_expCtx.get(), coll, params, &ws);
 
         int numCounted = runCount(&count);
         ASSERT_EQUALS(3, numCounted);
@@ -217,21 +226,23 @@ public:
 class QueryStageCountScanLowerBound : public CountBase {
 public:
     void run() {
-        dbtests::WriteContextForTests ctx(&_opCtx, ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns().ns());
 
         // Insert doc, add index
         insert(BSON("a" << 2));
         addIndex(BSON("a" << 1));
 
+        const CollectionPtr coll = ctx.getCollection();
+
         // Set up count, and run
-        auto params = makeCountScanParams(&_opCtx, getIndex(ctx.db(), BSON("a" << 1)));
+        auto params = makeCountScanParams(&_opCtx, coll, getIndex(ctx.db(), BSON("a" << 1)));
         params.startKey = BSON("" << 2);
         params.startKeyInclusive = false;
         params.endKey = BSON("" << 3);
         params.endKeyInclusive = false;
 
         WorkingSet ws;
-        CountScan count(_expCtx.get(), params, &ws);
+        CountScan count(_expCtx.get(), coll, params, &ws);
 
         int numCounted = runCount(&count);
         ASSERT_EQUALS(0, numCounted);
@@ -244,22 +255,24 @@ public:
 class QueryStageCountScanNothingInInterval : public CountBase {
 public:
     void run() {
-        dbtests::WriteContextForTests ctx(&_opCtx, ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns().ns());
 
         // Insert documents, add index
         insert(BSON("a" << 2));
         insert(BSON("a" << 3));
         addIndex(BSON("a" << 1));
 
+        const CollectionPtr coll = ctx.getCollection();
+
         // Set up count, and run
-        auto params = makeCountScanParams(&_opCtx, getIndex(ctx.db(), BSON("a" << 1)));
+        auto params = makeCountScanParams(&_opCtx, coll, getIndex(ctx.db(), BSON("a" << 1)));
         params.startKey = BSON("" << 2);
         params.startKeyInclusive = false;
         params.endKey = BSON("" << 3);
         params.endKeyInclusive = false;
 
         WorkingSet ws;
-        CountScan count(_expCtx.get(), params, &ws);
+        CountScan count(_expCtx.get(), coll, params, &ws);
 
         int numCounted = runCount(&count);
         ASSERT_EQUALS(0, numCounted);
@@ -273,22 +286,24 @@ public:
 class QueryStageCountScanNothingInIntervalFirstMatchTooHigh : public CountBase {
 public:
     void run() {
-        dbtests::WriteContextForTests ctx(&_opCtx, ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns().ns());
 
         // Insert some documents, add index
         insert(BSON("a" << 2));
         insert(BSON("a" << 4));
         addIndex(BSON("a" << 1));
 
+        const CollectionPtr coll = ctx.getCollection();
+
         // Set up count, and run
-        auto params = makeCountScanParams(&_opCtx, getIndex(ctx.db(), BSON("a" << 1)));
+        auto params = makeCountScanParams(&_opCtx, coll, getIndex(ctx.db(), BSON("a" << 1)));
         params.startKey = BSON("" << 2);
         params.startKeyInclusive = false;
         params.endKey = BSON("" << 3);
         params.endKeyInclusive = true;
 
         WorkingSet ws;
-        CountScan count(_expCtx.get(), params, &ws);
+        CountScan count(_expCtx.get(), coll, params, &ws);
 
         int numCounted = runCount(&count);
         ASSERT_EQUALS(0, numCounted);
@@ -302,7 +317,7 @@ public:
 class QueryStageCountScanNoChangeDuringYield : public CountBase {
 public:
     void run() {
-        dbtests::WriteContextForTests ctx(&_opCtx, ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns().ns());
 
         // Insert documents, add index
         for (int i = 0; i < 10; ++i) {
@@ -310,15 +325,17 @@ public:
         }
         addIndex(BSON("a" << 1));
 
+        const CollectionPtr coll = ctx.getCollection();
+
         // Set up count stage
-        auto params = makeCountScanParams(&_opCtx, getIndex(ctx.db(), BSON("a" << 1)));
+        auto params = makeCountScanParams(&_opCtx, coll, getIndex(ctx.db(), BSON("a" << 1)));
         params.startKey = BSON("" << 2);
         params.startKeyInclusive = false;
         params.endKey = BSON("" << 6);
         params.endKeyInclusive = true;
 
         WorkingSet ws;
-        CountScan count(_expCtx.get(), params, &ws);
+        CountScan count(_expCtx.get(), coll, params, &ws);
         WorkingSetID wsid;
 
         int numCounted = 0;
@@ -335,7 +352,9 @@ public:
         static_cast<PlanStage*>(&count)->saveState();
 
         // Recover from yield
-        static_cast<PlanStage*>(&count)->restoreState();
+        auto c = getCollection();
+
+        static_cast<PlanStage*>(&count)->restoreState(&coll);
 
         // finish counting
         while (PlanStage::IS_EOF != countState) {
@@ -354,7 +373,7 @@ public:
 class QueryStageCountScanDeleteDuringYield : public CountBase {
 public:
     void run() {
-        dbtests::WriteContextForTests ctx(&_opCtx, ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns().ns());
 
         // Insert documents, add index
         for (int i = 0; i < 10; ++i) {
@@ -362,15 +381,17 @@ public:
         }
         addIndex(BSON("a" << 1));
 
+        const CollectionPtr coll = ctx.getCollection();
+
         // Set up count stage
-        auto params = makeCountScanParams(&_opCtx, getIndex(ctx.db(), BSON("a" << 1)));
+        auto params = makeCountScanParams(&_opCtx, coll, getIndex(ctx.db(), BSON("a" << 1)));
         params.startKey = BSON("" << 2);
         params.startKeyInclusive = false;
         params.endKey = BSON("" << 6);
         params.endKeyInclusive = true;
 
         WorkingSet ws;
-        CountScan count(_expCtx.get(), params, &ws);
+        CountScan count(_expCtx.get(), coll, params, &ws);
         WorkingSetID wsid;
 
         int numCounted = 0;
@@ -386,11 +407,13 @@ public:
         // Prepare the cursor to yield
         static_cast<PlanStage*>(&count)->saveState();
 
+        // Recover from yield
+        auto c = getCollection();
+
         // Remove remaining objects
         remove(BSON("a" << GTE << 5));
 
-        // Recover from yield
-        static_cast<PlanStage*>(&count)->restoreState();
+        static_cast<PlanStage*>(&count)->restoreState(&coll);
 
         // finish counting
         while (PlanStage::IS_EOF != countState) {
@@ -409,7 +432,7 @@ public:
 class QueryStageCountScanInsertNewDocsDuringYield : public CountBase {
 public:
     void run() {
-        dbtests::WriteContextForTests ctx(&_opCtx, ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns().ns());
 
         // Insert documents, add index
         for (int i = 0; i < 10; ++i) {
@@ -417,15 +440,17 @@ public:
         }
         addIndex(BSON("a" << 1));
 
+        const CollectionPtr coll = ctx.getCollection();
+
         // Set up count stage
-        auto params = makeCountScanParams(&_opCtx, getIndex(ctx.db(), BSON("a" << 1)));
+        auto params = makeCountScanParams(&_opCtx, coll, getIndex(ctx.db(), BSON("a" << 1)));
         params.startKey = BSON("" << 2);
         params.startKeyInclusive = false;
         params.endKey = BSON("" << 6);
         params.endKeyInclusive = true;
 
         WorkingSet ws;
-        CountScan count(_expCtx.get(), params, &ws);
+        CountScan count(_expCtx.get(), coll, params, &ws);
         WorkingSetID wsid;
 
         int numCounted = 0;
@@ -448,7 +473,9 @@ public:
         insert(BSON("a" << 6.5));
 
         // Recover from yield
-        static_cast<PlanStage*>(&count)->restoreState();
+        auto c = getCollection();
+
+        static_cast<PlanStage*>(&count)->restoreState(&coll);
 
         // finish counting
         while (PlanStage::IS_EOF != countState) {
@@ -466,7 +493,7 @@ public:
 class QueryStageCountScanUnusedKeys : public CountBase {
 public:
     void run() {
-        dbtests::WriteContextForTests ctx(&_opCtx, ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns().ns());
 
         // Insert docs, add index
         for (int i = 0; i < 10; ++i) {
@@ -479,15 +506,17 @@ public:
         remove(BSON("a" << 1 << "b" << 3));
         remove(BSON("a" << 1 << "b" << 4));
 
+        const CollectionPtr coll = ctx.getCollection();
+
         // Ensure that count does not include unused keys
-        auto params = makeCountScanParams(&_opCtx, getIndex(ctx.db(), BSON("a" << 1)));
+        auto params = makeCountScanParams(&_opCtx, coll, getIndex(ctx.db(), BSON("a" << 1)));
         params.startKey = BSON("" << 1);
         params.startKeyInclusive = true;
         params.endKey = BSON("" << 1);
         params.endKeyInclusive = true;
 
         WorkingSet ws;
-        CountScan count(_expCtx.get(), params, &ws);
+        CountScan count(_expCtx.get(), coll, params, &ws);
 
         int numCounted = runCount(&count);
         ASSERT_EQUALS(7, numCounted);
@@ -500,7 +529,7 @@ public:
 class QueryStageCountScanUnusedEndKey : public CountBase {
 public:
     void run() {
-        dbtests::WriteContextForTests ctx(&_opCtx, ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns().ns());
 
         // Insert docs, add index
         for (int i = 0; i < 10; ++i) {
@@ -511,15 +540,17 @@ public:
         // Mark key at end position as 'unused' by deleting
         remove(BSON("a" << 1 << "b" << 9));
 
+        const CollectionPtr coll = ctx.getCollection();
+
         // Run count and check
-        auto params = makeCountScanParams(&_opCtx, getIndex(ctx.db(), BSON("a" << 1)));
+        auto params = makeCountScanParams(&_opCtx, coll, getIndex(ctx.db(), BSON("a" << 1)));
         params.startKey = BSON("" << 0);
         params.startKeyInclusive = true;
         params.endKey = BSON("" << 2);
         params.endKeyInclusive = true;  // yes?
 
         WorkingSet ws;
-        CountScan count(_expCtx.get(), params, &ws);
+        CountScan count(_expCtx.get(), coll, params, &ws);
 
         int numCounted = runCount(&count);
         ASSERT_EQUALS(9, numCounted);
@@ -532,7 +563,7 @@ public:
 class QueryStageCountScanKeyBecomesUnusedDuringYield : public CountBase {
 public:
     void run() {
-        dbtests::WriteContextForTests ctx(&_opCtx, ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns().ns());
 
         // Insert documents, add index
         for (int i = 0; i < 10; ++i) {
@@ -540,15 +571,17 @@ public:
         }
         addIndex(BSON("a" << 1));
 
+        const CollectionPtr coll = ctx.getCollection();
+
         // Set up count stage
-        auto params = makeCountScanParams(&_opCtx, getIndex(ctx.db(), BSON("a" << 1)));
+        auto params = makeCountScanParams(&_opCtx, coll, getIndex(ctx.db(), BSON("a" << 1)));
         params.startKey = BSON("" << 1);
         params.startKeyInclusive = true;
         params.endKey = BSON("" << 1);
         params.endKeyInclusive = true;
 
         WorkingSet ws;
-        CountScan count(_expCtx.get(), params, &ws);
+        CountScan count(_expCtx.get(), coll, params, &ws);
         WorkingSetID wsid;
 
         int numCounted = 0;
@@ -568,7 +601,8 @@ public:
         remove(BSON("a" << 1 << "b" << 5));
 
         // Recover from yield
-        static_cast<PlanStage*>(&count)->restoreState();
+        auto c = getCollection();
+        static_cast<PlanStage*>(&count)->restoreState(&coll);
 
         // finish counting
         while (PlanStage::IS_EOF != countState) {
