@@ -45,7 +45,8 @@
 #include "mongo/db/exec/working_set_common.h"
 #include "mongo/db/json.h"
 #include "mongo/db/query/collation/collator_interface_mock.h"
-#include "mongo/db/query/plan_executor.h"
+#include "mongo/db/query/plan_executor_factory.h"
+#include "mongo/db/query/query_planner_params.h"
 #include "mongo/dbtests/dbtests.h"
 
 /**
@@ -72,7 +73,7 @@ public:
         ASSERT_OK(dbtests::createIndex(&_opCtx, ns(), obj));
     }
 
-    const IndexDescriptor* getIndex(const BSONObj& obj, Collection* coll) {
+    const IndexDescriptor* getIndex(const BSONObj& obj, const CollectionPtr& coll) {
         std::vector<const IndexDescriptor*> indexes;
         coll->getIndexCatalog()->findIndexesByKeyPattern(
             &_opCtx, obj, IndexCatalog::InclusionPolicy::kReady, &indexes);
@@ -80,8 +81,9 @@ public:
     }
 
     IndexScanParams makeIndexScanParams(OperationContext* opCtx,
+                                        const CollectionPtr& collection,
                                         const IndexDescriptor* descriptor) {
-        IndexScanParams params(opCtx, descriptor);
+        IndexScanParams params(opCtx, collection, descriptor);
         params.bounds.isSimpleRange = true;
         params.bounds.startKey = objWithMinKey(1);
         params.bounds.endKey = objWithMaxKey(1);
@@ -102,7 +104,7 @@ public:
         _client.update(ns(), predicate, update);
     }
 
-    void getRecordIds(set<RecordId>* out, Collection* coll) {
+    void getRecordIds(set<RecordId>* out, const CollectionPtr& coll) {
         auto cursor = coll->getCursor(&_opCtx);
         while (auto record = cursor->next()) {
             out->insert(record->id);
@@ -148,12 +150,10 @@ class QueryStageMergeSortPrefixIndex : public QueryStageMergeSortTestBase {
 public:
     void run() {
         dbtests::WriteContextForTests ctx(&_opCtx, ns());
-        Database* db = ctx.db();
-        Collection* coll =
-            CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss());
-        if (!coll) {
+
+        if (!ctx.getCollection()) {
             WriteUnitOfWork wuow(&_opCtx);
-            coll = db->createCollection(&_opCtx, nss());
+            ctx.db()->createCollection(&_opCtx, nss());
             wuow.commit();
         }
 
@@ -169,6 +169,7 @@ public:
 
         addIndex(firstIndex);
         addIndex(secondIndex);
+        CollectionPtr coll = ctx.getCollection();
 
         unique_ptr<WorkingSet> ws = make_unique<WorkingSet>();
         // Sort by c:1
@@ -177,18 +178,23 @@ public:
         auto ms = std::make_unique<MergeSortStage>(_expCtx.get(), msparams, ws.get());
 
         // a:1
-        auto params = makeIndexScanParams(&_opCtx, getIndex(firstIndex, coll));
-        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr));
+        auto params = makeIndexScanParams(&_opCtx, coll, getIndex(firstIndex, coll));
+        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr));
 
         // b:1
-        params = makeIndexScanParams(&_opCtx, getIndex(secondIndex, coll));
-        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr));
+        params = makeIndexScanParams(&_opCtx, coll, getIndex(secondIndex, coll));
+        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr));
 
         unique_ptr<FetchStage> fetchStage =
             make_unique<FetchStage>(_expCtx.get(), ws.get(), std::move(ms), nullptr, coll);
         // Must fetch if we want to easily pull out an obj.
-        auto statusWithPlanExecutor = PlanExecutor::make(
-            &_opCtx, std::move(ws), std::move(fetchStage), coll, PlanExecutor::NO_YIELD);
+        auto statusWithPlanExecutor =
+            plan_executor_factory::make(_expCtx,
+                                        std::move(ws),
+                                        std::move(fetchStage),
+                                        &coll,
+                                        PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                        QueryPlannerParams::DEFAULT);
         ASSERT_OK(statusWithPlanExecutor.getStatus());
         auto exec = std::move(statusWithPlanExecutor.getValue());
 
@@ -214,12 +220,9 @@ class QueryStageMergeSortDups : public QueryStageMergeSortTestBase {
 public:
     void run() {
         dbtests::WriteContextForTests ctx(&_opCtx, ns());
-        Database* db = ctx.db();
-        Collection* coll =
-            CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss());
-        if (!coll) {
+        if (!ctx.getCollection()) {
             WriteUnitOfWork wuow(&_opCtx);
-            coll = db->createCollection(&_opCtx, nss());
+            ctx.db()->createCollection(&_opCtx, nss());
             wuow.commit();
         }
 
@@ -235,6 +238,7 @@ public:
 
         addIndex(firstIndex);
         addIndex(secondIndex);
+        auto coll = ctx.getCollection();
 
         unique_ptr<WorkingSet> ws = make_unique<WorkingSet>();
         // Sort by c:1
@@ -243,17 +247,22 @@ public:
         auto ms = std::make_unique<MergeSortStage>(_expCtx.get(), msparams, ws.get());
 
         // a:1
-        auto params = makeIndexScanParams(&_opCtx, getIndex(firstIndex, coll));
-        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr));
+        auto params = makeIndexScanParams(&_opCtx, coll, getIndex(firstIndex, coll));
+        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr));
 
         // b:1
-        params = makeIndexScanParams(&_opCtx, getIndex(secondIndex, coll));
-        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr));
+        params = makeIndexScanParams(&_opCtx, coll, getIndex(secondIndex, coll));
+        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr));
         unique_ptr<FetchStage> fetchStage =
             make_unique<FetchStage>(_expCtx.get(), ws.get(), std::move(ms), nullptr, coll);
 
-        auto statusWithPlanExecutor = PlanExecutor::make(
-            &_opCtx, std::move(ws), std::move(fetchStage), coll, PlanExecutor::NO_YIELD);
+        auto statusWithPlanExecutor =
+            plan_executor_factory::make(_expCtx,
+                                        std::move(ws),
+                                        std::move(fetchStage),
+                                        &coll,
+                                        PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                        QueryPlannerParams::DEFAULT);
         ASSERT_OK(statusWithPlanExecutor.getStatus());
         auto exec = std::move(statusWithPlanExecutor.getValue());
 
@@ -279,12 +288,9 @@ class QueryStageMergeSortDupsNoDedup : public QueryStageMergeSortTestBase {
 public:
     void run() {
         dbtests::WriteContextForTests ctx(&_opCtx, ns());
-        Database* db = ctx.db();
-        Collection* coll =
-            CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss());
-        if (!coll) {
+        if (!ctx.getCollection()) {
             WriteUnitOfWork wuow(&_opCtx);
-            coll = db->createCollection(&_opCtx, nss());
+            ctx.db()->createCollection(&_opCtx, nss());
             wuow.commit();
         }
 
@@ -299,6 +305,7 @@ public:
 
         addIndex(firstIndex);
         addIndex(secondIndex);
+        auto coll = ctx.getCollection();
 
         unique_ptr<WorkingSet> ws = make_unique<WorkingSet>();
         // Sort by c:1
@@ -308,17 +315,22 @@ public:
         auto ms = std::make_unique<MergeSortStage>(_expCtx.get(), msparams, ws.get());
 
         // a:1
-        auto params = makeIndexScanParams(&_opCtx, getIndex(firstIndex, coll));
-        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr));
+        auto params = makeIndexScanParams(&_opCtx, coll, getIndex(firstIndex, coll));
+        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr));
 
         // b:1
-        params = makeIndexScanParams(&_opCtx, getIndex(secondIndex, coll));
-        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr));
+        params = makeIndexScanParams(&_opCtx, coll, getIndex(secondIndex, coll));
+        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr));
         unique_ptr<FetchStage> fetchStage =
             make_unique<FetchStage>(_expCtx.get(), ws.get(), std::move(ms), nullptr, coll);
 
-        auto statusWithPlanExecutor = PlanExecutor::make(
-            &_opCtx, std::move(ws), std::move(fetchStage), coll, PlanExecutor::NO_YIELD);
+        auto statusWithPlanExecutor =
+            plan_executor_factory::make(_expCtx,
+                                        std::move(ws),
+                                        std::move(fetchStage),
+                                        &coll,
+                                        PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                        QueryPlannerParams::DEFAULT);
         ASSERT_OK(statusWithPlanExecutor.getStatus());
         auto exec = std::move(statusWithPlanExecutor.getValue());
 
@@ -345,12 +357,9 @@ class QueryStageMergeSortPrefixIndexReverse : public QueryStageMergeSortTestBase
 public:
     void run() {
         dbtests::WriteContextForTests ctx(&_opCtx, ns());
-        Database* db = ctx.db();
-        Collection* coll =
-            CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss());
-        if (!coll) {
+        if (!ctx.getCollection()) {
             WriteUnitOfWork wuow(&_opCtx);
-            coll = db->createCollection(&_opCtx, nss());
+            ctx.db()->createCollection(&_opCtx, nss());
             wuow.commit();
         }
 
@@ -367,6 +376,7 @@ public:
 
         addIndex(firstIndex);
         addIndex(secondIndex);
+        auto coll = ctx.getCollection();
 
         unique_ptr<WorkingSet> ws = make_unique<WorkingSet>();
         // Sort by c:-1
@@ -375,21 +385,26 @@ public:
         auto ms = std::make_unique<MergeSortStage>(_expCtx.get(), msparams, ws.get());
 
         // a:1
-        auto params = makeIndexScanParams(&_opCtx, getIndex(firstIndex, coll));
+        auto params = makeIndexScanParams(&_opCtx, coll, getIndex(firstIndex, coll));
         params.bounds.startKey = objWithMaxKey(1);
         params.bounds.endKey = objWithMinKey(1);
-        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr));
+        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr));
 
         // b:1
-        params = makeIndexScanParams(&_opCtx, getIndex(secondIndex, coll));
+        params = makeIndexScanParams(&_opCtx, coll, getIndex(secondIndex, coll));
         params.bounds.startKey = objWithMaxKey(1);
         params.bounds.endKey = objWithMinKey(1);
-        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr));
+        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr));
         unique_ptr<FetchStage> fetchStage =
             make_unique<FetchStage>(_expCtx.get(), ws.get(), std::move(ms), nullptr, coll);
 
-        auto statusWithPlanExecutor = PlanExecutor::make(
-            &_opCtx, std::move(ws), std::move(fetchStage), coll, PlanExecutor::NO_YIELD);
+        auto statusWithPlanExecutor =
+            plan_executor_factory::make(_expCtx,
+                                        std::move(ws),
+                                        std::move(fetchStage),
+                                        &coll,
+                                        PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                        QueryPlannerParams::DEFAULT);
         ASSERT_OK(statusWithPlanExecutor.getStatus());
         auto exec = std::move(statusWithPlanExecutor.getValue());
 
@@ -415,12 +430,9 @@ class QueryStageMergeSortOneStageEOF : public QueryStageMergeSortTestBase {
 public:
     void run() {
         dbtests::WriteContextForTests ctx(&_opCtx, ns());
-        Database* db = ctx.db();
-        Collection* coll =
-            CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss());
-        if (!coll) {
+        if (!ctx.getCollection()) {
             WriteUnitOfWork wuow(&_opCtx);
-            coll = db->createCollection(&_opCtx, nss());
+            ctx.db()->createCollection(&_opCtx, nss());
             wuow.commit();
         }
 
@@ -436,6 +448,7 @@ public:
 
         addIndex(firstIndex);
         addIndex(secondIndex);
+        auto coll = ctx.getCollection();
 
         unique_ptr<WorkingSet> ws = make_unique<WorkingSet>();
         // Sort by c:1
@@ -444,19 +457,24 @@ public:
         auto ms = std::make_unique<MergeSortStage>(_expCtx.get(), msparams, ws.get());
 
         // a:1
-        auto params = makeIndexScanParams(&_opCtx, getIndex(firstIndex, coll));
-        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr));
+        auto params = makeIndexScanParams(&_opCtx, coll, getIndex(firstIndex, coll));
+        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr));
 
         // b:51 (EOF)
-        params = makeIndexScanParams(&_opCtx, getIndex(secondIndex, coll));
+        params = makeIndexScanParams(&_opCtx, coll, getIndex(secondIndex, coll));
         params.bounds.startKey = BSON("" << 51 << "" << MinKey);
         params.bounds.endKey = BSON("" << 51 << "" << MaxKey);
-        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr));
+        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr));
         unique_ptr<FetchStage> fetchStage =
             make_unique<FetchStage>(_expCtx.get(), ws.get(), std::move(ms), nullptr, coll);
 
-        auto statusWithPlanExecutor = PlanExecutor::make(
-            &_opCtx, std::move(ws), std::move(fetchStage), coll, PlanExecutor::NO_YIELD);
+        auto statusWithPlanExecutor =
+            plan_executor_factory::make(_expCtx,
+                                        std::move(ws),
+                                        std::move(fetchStage),
+                                        &coll,
+                                        PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                        QueryPlannerParams::DEFAULT);
         ASSERT_OK(statusWithPlanExecutor.getStatus());
         auto exec = std::move(statusWithPlanExecutor.getValue());
 
@@ -479,12 +497,9 @@ class QueryStageMergeSortManyShort : public QueryStageMergeSortTestBase {
 public:
     void run() {
         dbtests::WriteContextForTests ctx(&_opCtx, ns());
-        Database* db = ctx.db();
-        Collection* coll =
-            CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss());
-        if (!coll) {
+        if (!ctx.getCollection()) {
             WriteUnitOfWork wuow(&_opCtx);
-            coll = db->createCollection(&_opCtx, nss());
+            ctx.db()->createCollection(&_opCtx, nss());
             wuow.commit();
         }
 
@@ -494,22 +509,33 @@ public:
         msparams.pattern = BSON("foo" << 1);
         auto ms = std::make_unique<MergeSortStage>(_expCtx.get(), msparams, ws.get());
 
-        int numIndices = 20;
+        const int numIndices = 20;
+        BSONObj indexSpec[numIndices];
+
         for (int i = 0; i < numIndices; ++i) {
             // 'a', 'b', ...
             string index(1, 'a' + i);
             insert(BSON(index << 1 << "foo" << i));
 
-            BSONObj indexSpec = BSON(index << 1 << "foo" << 1);
-            addIndex(indexSpec);
-            auto params = makeIndexScanParams(&_opCtx, getIndex(indexSpec, coll));
-            ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr));
+            indexSpec[i] = BSON(index << 1 << "foo" << 1);
+            addIndex(indexSpec[i]);
+        }
+        const auto& coll = ctx.getCollection();
+        for (int i = 0; i < numIndices; ++i) {
+            auto params = makeIndexScanParams(&_opCtx, coll, getIndex(indexSpec[i], coll));
+            ms->addChild(
+                std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr));
         }
         unique_ptr<FetchStage> fetchStage =
             make_unique<FetchStage>(_expCtx.get(), ws.get(), std::move(ms), nullptr, coll);
 
-        auto statusWithPlanExecutor = PlanExecutor::make(
-            &_opCtx, std::move(ws), std::move(fetchStage), coll, PlanExecutor::NO_YIELD);
+        auto statusWithPlanExecutor =
+            plan_executor_factory::make(_expCtx,
+                                        std::move(ws),
+                                        std::move(fetchStage),
+                                        &coll,
+                                        PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                        QueryPlannerParams::DEFAULT);
         ASSERT_OK(statusWithPlanExecutor.getStatus());
         auto exec = std::move(statusWithPlanExecutor.getValue());
 
@@ -532,12 +558,9 @@ class QueryStageMergeSortDeletedDocument : public QueryStageMergeSortTestBase {
 public:
     void run() {
         dbtests::WriteContextForTests ctx(&_opCtx, ns());
-        Database* db = ctx.db();
-        Collection* coll =
-            CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss());
-        if (!coll) {
+        if (!ctx.getCollection()) {
             WriteUnitOfWork wuow(&_opCtx);
-            coll = db->createCollection(&_opCtx, nss());
+            ctx.db()->createCollection(&_opCtx, nss());
             wuow.commit();
         }
 
@@ -549,16 +572,21 @@ public:
 
         // Index 'a'+i has foo equal to 'i'.
 
-        int numIndices = 20;
+        const int numIndices = 20;
+        BSONObj indexSpec[numIndices];
         for (int i = 0; i < numIndices; ++i) {
             // 'a', 'b', ...
             string index(1, 'a' + i);
             insert(BSON(index << 1 << "foo" << i));
 
-            BSONObj indexSpec = BSON(index << 1 << "foo" << 1);
-            addIndex(indexSpec);
-            auto params = makeIndexScanParams(&_opCtx, getIndex(indexSpec, coll));
-            ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, &ws, nullptr));
+            indexSpec[i] = BSON(index << 1 << "foo" << 1);
+            addIndex(indexSpec[i]);
+        }
+        const auto& coll = ctx.getCollection();
+        for (int i = 0; i < numIndices; ++i) {
+            auto params =
+                makeIndexScanParams(&_opCtx, coll, getIndex(indexSpec[i], ctx.getCollection()));
+            ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), coll, params, &ws, nullptr));
         }
 
         set<RecordId> recordIds;
@@ -591,7 +619,7 @@ public:
         // stage, and therefore should still be returned.
         ms->saveState();
         remove(BSON(std::string(1u, 'a' + count) << 1));
-        ms->restoreState();
+        ms->restoreState(&coll);
 
         // Make sure recordIds[11] is returned as expected. We expect the corresponding working set
         // member to remain in RID_AND_IDX state. It should have been marked as "suspicious", since
@@ -617,8 +645,8 @@ public:
             // An attempt to fetch the WSM should show that the key is no longer present in the
             // index.
             NamespaceString fakeNS("test", "coll");
-            ASSERT_FALSE(
-                WorkingSetCommon::fetch(&_opCtx, &ws, id, coll->getCursor(&_opCtx), fakeNS));
+            ASSERT_FALSE(WorkingSetCommon::fetch(
+                &_opCtx, &ws, id, coll->getCursor(&_opCtx).get(), coll, fakeNS));
 
             ++it;
             ++count;
@@ -652,12 +680,9 @@ class QueryStageMergeSortConcurrentUpdateDedup : public QueryStageMergeSortTestB
 public:
     void run() {
         dbtests::WriteContextForTests ctx(&_opCtx, ns());
-        Database* db = ctx.db();
-        Collection* coll =
-            CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss());
-        if (!coll) {
+        if (!ctx.getCollection()) {
             WriteUnitOfWork wuow(&_opCtx);
-            coll = db->createCollection(&_opCtx, nss());
+            ctx.db()->createCollection(&_opCtx, nss());
             wuow.commit();
         }
 
@@ -667,6 +692,7 @@ public:
         insert(BSON("_id" << 6 << "a" << 6));
 
         addIndex(BSON("a" << 1));
+        auto coll = ctx.getCollection();
 
         std::set<RecordId> rids;
         getRecordIds(&rids, coll);
@@ -680,13 +706,13 @@ public:
 
         // First child scans [5, 10].
         {
-            auto params = makeIndexScanParams(&_opCtx, getIndex(BSON("a" << 1), coll));
+            auto params = makeIndexScanParams(&_opCtx, coll, getIndex(BSON("a" << 1), coll));
             params.bounds.startKey = BSON("" << 5);
             params.bounds.endKey = BSON("" << 10);
             auto fetchStage = std::make_unique<FetchStage>(
                 _expCtx.get(),
                 &ws,
-                std::make_unique<IndexScan>(_expCtx.get(), params, &ws, nullptr),
+                std::make_unique<IndexScan>(_expCtx.get(), coll, params, &ws, nullptr),
                 nullptr,
                 coll);
             ms->addChild(std::move(fetchStage));
@@ -694,13 +720,13 @@ public:
 
         // Second child scans [4, 10].
         {
-            auto params = makeIndexScanParams(&_opCtx, getIndex(BSON("a" << 1), coll));
+            auto params = makeIndexScanParams(&_opCtx, coll, getIndex(BSON("a" << 1), coll));
             params.bounds.startKey = BSON("" << 4);
             params.bounds.endKey = BSON("" << 10);
             auto fetchStage = std::make_unique<FetchStage>(
                 _expCtx.get(),
                 &ws,
-                std::make_unique<IndexScan>(_expCtx.get(), params, &ws, nullptr),
+                std::make_unique<IndexScan>(_expCtx.get(), coll, params, &ws, nullptr),
                 nullptr,
                 coll);
             ms->addChild(std::move(fetchStage));
@@ -716,7 +742,7 @@ public:
         // Update doc {a: 5} such that the postimage will no longer match the query.
         ms->saveState();
         update(BSON("a" << 5), BSON("$set" << BSON("a" << 15)));
-        ms->restoreState();
+        ms->restoreState(&coll);
 
         // Invalidated doc {a: 5} should still get returned. We expect an RID_AND_OBJ working set
         // member with an owned BSONObj.
@@ -755,12 +781,9 @@ class QueryStageMergeSortStringsWithNullCollation : public QueryStageMergeSortTe
 public:
     void run() {
         dbtests::WriteContextForTests ctx(&_opCtx, ns());
-        Database* db = ctx.db();
-        Collection* coll =
-            CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss());
-        if (!coll) {
+        if (!ctx.getCollection()) {
             WriteUnitOfWork wuow(&_opCtx);
-            coll = db->createCollection(&_opCtx, nss());
+            ctx.db()->createCollection(&_opCtx, nss());
             wuow.commit();
         }
 
@@ -778,6 +801,7 @@ public:
 
         addIndex(firstIndex);
         addIndex(secondIndex);
+        auto coll = ctx.getCollection();
 
         unique_ptr<WorkingSet> ws = make_unique<WorkingSet>();
         // Sort by c:1, d:1
@@ -787,18 +811,23 @@ public:
         auto ms = std::make_unique<MergeSortStage>(_expCtx.get(), msparams, ws.get());
 
         // a:1
-        auto params = makeIndexScanParams(&_opCtx, getIndex(firstIndex, coll));
-        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr));
+        auto params = makeIndexScanParams(&_opCtx, coll, getIndex(firstIndex, coll));
+        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr));
 
         // b:1
-        params = makeIndexScanParams(&_opCtx, getIndex(secondIndex, coll));
-        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr));
+        params = makeIndexScanParams(&_opCtx, coll, getIndex(secondIndex, coll));
+        ms->addChild(std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr));
 
         auto fetchStage =
             make_unique<FetchStage>(_expCtx.get(), ws.get(), std::move(ms), nullptr, coll);
         // Must fetch if we want to easily pull out an obj.
-        auto statusWithPlanExecutor = PlanExecutor::make(
-            &_opCtx, std::move(ws), std::move(fetchStage), coll, PlanExecutor::NO_YIELD);
+        auto statusWithPlanExecutor =
+            plan_executor_factory::make(_expCtx,
+                                        std::move(ws),
+                                        std::move(fetchStage),
+                                        &coll,
+                                        PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                        QueryPlannerParams::DEFAULT);
         ASSERT_OK(statusWithPlanExecutor.getStatus());
         auto exec = std::move(statusWithPlanExecutor.getValue());
 
@@ -823,12 +852,9 @@ class QueryStageMergeSortStringsRespectsCollation : public QueryStageMergeSortTe
 public:
     void run() {
         dbtests::WriteContextForTests ctx(&_opCtx, ns());
-        Database* db = ctx.db();
-        Collection* coll =
-            CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss());
-        if (!coll) {
+        if (!ctx.getCollection()) {
             WriteUnitOfWork wuow(&_opCtx);
-            coll = db->createCollection(&_opCtx, nss());
+            ctx.db()->createCollection(&_opCtx, nss());
             wuow.commit();
         }
 
@@ -846,6 +872,7 @@ public:
 
         addIndex(firstIndex);
         addIndex(secondIndex);
+        auto coll = ctx.getCollection();
 
         unique_ptr<WorkingSet> ws = make_unique<WorkingSet>();
         // Sort by c:1, d:1
@@ -856,8 +883,8 @@ public:
         auto ms = std::make_unique<MergeSortStage>(_expCtx.get(), msparams, ws.get());
 
         // a:1
-        auto params = makeIndexScanParams(&_opCtx, getIndex(firstIndex, coll));
-        auto idxScan = std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr);
+        auto params = makeIndexScanParams(&_opCtx, coll, getIndex(firstIndex, coll));
+        auto idxScan = std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr);
 
         // Wrap 'idxScan' with a FETCH stage so a document is fetched and MERGE_SORT is forced to
         // use the provided collator 'collator'. Also, this permits easier retrieval of result
@@ -866,13 +893,18 @@ public:
             make_unique<FetchStage>(_expCtx.get(), ws.get(), std::move(idxScan), nullptr, coll));
 
         // b:1
-        params = makeIndexScanParams(&_opCtx, getIndex(secondIndex, coll));
-        idxScan = std::make_unique<IndexScan>(_expCtx.get(), params, ws.get(), nullptr);
+        params = makeIndexScanParams(&_opCtx, coll, getIndex(secondIndex, coll));
+        idxScan = std::make_unique<IndexScan>(_expCtx.get(), coll, params, ws.get(), nullptr);
         ms->addChild(
             make_unique<FetchStage>(_expCtx.get(), ws.get(), std::move(idxScan), nullptr, coll));
 
         auto statusWithPlanExecutor =
-            PlanExecutor::make(_expCtx, std::move(ws), std::move(ms), coll, PlanExecutor::NO_YIELD);
+            plan_executor_factory::make(_expCtx,
+                                        std::move(ws),
+                                        std::move(ms),
+                                        &coll,
+                                        PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                        QueryPlannerParams::DEFAULT);
         ASSERT_OK(statusWithPlanExecutor.getStatus());
         auto exec = std::move(statusWithPlanExecutor.getValue());
 
