@@ -27,10 +27,8 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
 #include "mongo/bson/bsonobj.h"
-#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_write_path.h"
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/drop_collection.h"
 #include "mongo/db/catalog/rename_collection.h"
@@ -40,34 +38,31 @@
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/record_id.h"
 #include "mongo/dbtests/dbtests.h"
-#include "mongo/unittest/unittest.h"
-
-using mongo::unittest::assertGet;
-using std::list;
-using std::string;
-using std::unique_ptr;
 
+namespace mongo {
 namespace RollbackTests {
-
 namespace {
+
 const auto kIndexVersion = IndexDescriptor::IndexVersion::kV2;
 
 void dropDatabase(OperationContext* opCtx, const NamespaceString& nss) {
     Lock::GlobalWrite globalWriteLock(opCtx);
     auto databaseHolder = DatabaseHolder::get(opCtx);
-    auto db = databaseHolder->getDb(opCtx, nss.db());
+    auto db = databaseHolder->getDb(opCtx, nss.dbName());
 
     if (db) {
         databaseHolder->dropDb(opCtx, db);
     }
 }
-bool collectionExists(OperationContext* opCtx, OldClientContext* ctx, const string& ns) {
-    return CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, NamespaceString(ns));
+
+bool collectionExists(OperationContext* opCtx, OldClientContext* ctx, const std::string& ns) {
+    return (bool)CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx,
+                                                                            NamespaceString(ns));
 }
 
 void createCollection(OperationContext* opCtx, const NamespaceString& nss) {
-    Lock::DBLock dbXLock(opCtx, nss.db(), MODE_X);
-    OldClientContext ctx(opCtx, nss.ns());
+    Lock::DBLock dbXLock(opCtx, nss.dbName(), MODE_X);
+    OldClientContext ctx(opCtx, nss);
     {
         WriteUnitOfWork uow(opCtx);
         ASSERT(!collectionExists(opCtx, &ctx, nss.ns()));
@@ -77,24 +72,28 @@ void createCollection(OperationContext* opCtx, const NamespaceString& nss) {
         uow.commit();
     }
 }
+
 Status renameCollection(OperationContext* opCtx,
                         const NamespaceString& source,
                         const NamespaceString& target) {
     ASSERT_EQ(source.db(), target.db());
     return renameCollection(opCtx, source, target, {});
 }
+
 Status truncateCollection(OperationContext* opCtx, const NamespaceString& nss) {
-    auto coll = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
-    return coll->truncate(opCtx);
+    CollectionWriter coll(opCtx, nss);
+    return coll.getWritableCollection(opCtx)->truncate(opCtx);
 }
 
 void insertRecord(OperationContext* opCtx, const NamespaceString& nss, const BSONObj& data) {
-    auto coll = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
+    auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
     OpDebug* const nullOpDebug = nullptr;
-    ASSERT_OK(coll->insertDocument(opCtx, InsertStatement(data), nullOpDebug, false));
+    ASSERT_OK(collection_internal::insertDocument(
+        opCtx, coll, InsertStatement(data), nullOpDebug, false));
 }
+
 void assertOnlyRecord(OperationContext* opCtx, const NamespaceString& nss, const BSONObj& data) {
-    auto coll = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
+    auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
     auto cursor = coll->getCursor(opCtx);
 
     auto record = cursor->next();
@@ -103,41 +102,42 @@ void assertOnlyRecord(OperationContext* opCtx, const NamespaceString& nss, const
 
     ASSERT(!cursor->next());
 }
+
 void assertEmpty(OperationContext* opCtx, const NamespaceString& nss) {
-    auto coll = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
+    auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
     ASSERT(!coll->getCursor(opCtx)->next());
 }
-bool indexExists(OperationContext* opCtx, const NamespaceString& nss, const string& idxName) {
-    auto coll = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
+
+bool indexExists(OperationContext* opCtx, const NamespaceString& nss, const std::string& idxName) {
+    auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
     return coll->getIndexCatalog()->findIndexByName(
                opCtx,
                idxName,
                IndexCatalog::InclusionPolicy::kReady |
                    IndexCatalog::InclusionPolicy::kUnfinished) != nullptr;
 }
-bool indexReady(OperationContext* opCtx, const NamespaceString& nss, const string& idxName) {
-    auto coll = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
+
+bool indexReady(OperationContext* opCtx, const NamespaceString& nss, const std::string& idxName) {
+    auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
     return coll->getIndexCatalog()->findIndexByName(
                opCtx, idxName, IndexCatalog::InclusionPolicy::kReady) != nullptr;
 }
+
 size_t getNumIndexEntries(OperationContext* opCtx,
                           const NamespaceString& nss,
-                          const string& idxName) {
+                          const std::string& idxName) {
     size_t numEntries = 0;
 
-    auto coll = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
-    IndexCatalog* catalog = coll->getIndexCatalog();
+    auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
+    const IndexCatalog* catalog = coll->getIndexCatalog();
     auto desc = catalog->findIndexByName(opCtx, idxName, IndexCatalog::InclusionPolicy::kReady);
 
     if (desc) {
-        auto cursor = catalog->getEntry(desc)->accessMethod()->newCursor(opCtx);
-        KeyString::Builder keyString(
-            catalog->getEntry(desc)
-                ->accessMethod()
-                ->getSortedDataInterface()
-                ->getKeyStringVersion(),
-            BSONObj(),
-            catalog->getEntry(desc)->accessMethod()->getSortedDataInterface()->getOrdering());
+        auto iam = catalog->getEntry(desc)->accessMethod()->asSortedData();
+        auto cursor = iam->newCursor(opCtx);
+        KeyString::Builder keyString(iam->getSortedDataInterface()->getKeyStringVersion(),
+                                     BSONObj(),
+                                     iam->getSortedDataInterface()->getOrdering());
         for (auto kv = cursor->seek(keyString.getValueCopy()); kv; kv = cursor->next()) {
             numEntries++;
         }
@@ -146,12 +146,15 @@ size_t getNumIndexEntries(OperationContext* opCtx,
     return numEntries;
 }
 
-void dropIndex(OperationContext* opCtx, const NamespaceString& nss, const string& idxName) {
-    auto coll = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
-    auto desc = coll->getIndexCatalog()->findIndexByName(opCtx, idxName);
+void dropIndex(OperationContext* opCtx, const NamespaceString& nss, const std::string& idxName) {
+    CollectionWriter coll(opCtx, nss);
+    auto desc =
+        coll.getWritableCollection(opCtx)->getIndexCatalog()->findIndexByName(opCtx, idxName);
     ASSERT(desc);
-    ASSERT_OK(coll->getIndexCatalog()->dropIndex(opCtx, desc));
+    ASSERT_OK(coll.getWritableCollection(opCtx)->getIndexCatalog()->dropIndex(
+        opCtx, coll.getWritableCollection(opCtx), desc));
 }
+
 }  // namespace
 
 template <bool rollback, bool defaultIndexes, bool capped>
@@ -163,14 +166,14 @@ public:
             return;
         }
 
-        string ns = "unittests.rollback_create_collection";
+        std::string ns = "unittests.rollback_create_collection";
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
         NamespaceString nss(ns);
         dropDatabase(&opCtx, nss);
 
-        Lock::DBLock dbXLock(&opCtx, nss.db(), MODE_X);
-        OldClientContext ctx(&opCtx, ns);
+        Lock::DBLock dbXLock(&opCtx, nss.dbName(), MODE_X);
+        OldClientContext ctx(&opCtx, nss);
         {
             WriteUnitOfWork uow(&opCtx);
             ASSERT(!collectionExists(&opCtx, &ctx, ns));
@@ -200,14 +203,14 @@ public:
             return;
         }
 
-        string ns = "unittests.rollback_drop_collection";
+        std::string ns = "unittests.rollback_drop_collection";
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
         NamespaceString nss(ns);
         dropDatabase(&opCtx, nss);
 
-        Lock::DBLock dbXLock(&opCtx, nss.db(), MODE_X);
-        OldClientContext ctx(&opCtx, ns);
+        Lock::DBLock dbXLock(&opCtx, nss.dbName(), MODE_X);
+        OldClientContext ctx(&opCtx, nss);
         {
             WriteUnitOfWork uow(&opCtx);
             ASSERT(!collectionExists(&opCtx, &ctx, ns));
@@ -256,7 +259,7 @@ public:
         dropDatabase(&opCtx, target);
 
         Lock::GlobalWrite globalWriteLock(&opCtx);
-        OldClientContext ctx(&opCtx, source.ns());
+        OldClientContext ctx(&opCtx, source);
 
         {
             WriteUnitOfWork uow(&opCtx);
@@ -310,7 +313,7 @@ public:
         dropDatabase(&opCtx, target);
 
         Lock::GlobalWrite globalWriteLock(&opCtx);
-        OldClientContext ctx(&opCtx, source.ns());
+        OldClientContext ctx(&opCtx, source);
 
         BSONObj sourceDoc = BSON("_id"
                                  << "source");
@@ -377,8 +380,8 @@ public:
         OperationContext& opCtx = *opCtxPtr;
         dropDatabase(&opCtx, nss);
 
-        Lock::DBLock dbXLock(&opCtx, nss.db(), MODE_X);
-        OldClientContext ctx(&opCtx, nss.ns());
+        Lock::DBLock dbXLock(&opCtx, nss.dbName(), MODE_X);
+        OldClientContext ctx(&opCtx, nss);
 
         BSONObj oldDoc = BSON("_id"
                               << "old");
@@ -435,8 +438,8 @@ public:
         OperationContext& opCtx = *opCtxPtr;
         dropDatabase(&opCtx, nss);
 
-        Lock::DBLock dbXLock(&opCtx, nss.db(), MODE_X);
-        OldClientContext ctx(&opCtx, nss.ns());
+        Lock::DBLock dbXLock(&opCtx, nss.dbName(), MODE_X);
+        OldClientContext ctx(&opCtx, nss);
 
         BSONObj doc = BSON("_id"
                            << "foo");
@@ -481,19 +484,18 @@ template <bool rollback>
 class CreateIndex {
 public:
     void run() {
-        string ns = "unittests.rollback_create_index";
+        std::string ns = "unittests.rollback_create_index";
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
         NamespaceString nss(ns);
         dropDatabase(&opCtx, nss);
         createCollection(&opCtx, nss);
 
-        AutoGetDb autoDb(&opCtx, nss.db(), MODE_X);
+        AutoGetDb autoDb(&opCtx, nss.dbName(), MODE_X);
 
-        Collection* coll = CollectionCatalog::get(&opCtx).lookupCollectionByNamespace(&opCtx, nss);
-        IndexCatalog* catalog = coll->getIndexCatalog();
+        CollectionWriter coll(&opCtx, nss);
 
-        string idxName = "a";
+        std::string idxName = "a";
         BSONObj spec = BSON("key" << BSON("a" << 1) << "name" << idxName << "v"
                                   << static_cast<int>(kIndexVersion));
 
@@ -501,7 +503,9 @@ public:
 
         {
             WriteUnitOfWork uow(&opCtx);
-            ASSERT_OK(catalog->createIndexOnEmptyCollection(&opCtx, spec));
+            IndexCatalog* catalog = coll.getWritableCollection(&opCtx)->getIndexCatalog();
+            ASSERT_OK(catalog->createIndexOnEmptyCollection(
+                &opCtx, coll.getWritableCollection(&opCtx), spec));
             insertRecord(&opCtx, nss, BSON("a" << 1));
             insertRecord(&opCtx, nss, BSON("a" << 2));
             insertRecord(&opCtx, nss, BSON("a" << 3));
@@ -522,25 +526,26 @@ template <bool rollback>
 class DropIndex {
 public:
     void run() {
-        string ns = "unittests.rollback_drop_index";
+        std::string ns = "unittests.rollback_drop_index";
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
         NamespaceString nss(ns);
         dropDatabase(&opCtx, nss);
         createCollection(&opCtx, nss);
 
-        AutoGetDb autoDb(&opCtx, nss.db(), MODE_X);
+        AutoGetDb autoDb(&opCtx, nss.dbName(), MODE_X);
 
-        Collection* coll = CollectionCatalog::get(&opCtx).lookupCollectionByNamespace(&opCtx, nss);
-        IndexCatalog* catalog = coll->getIndexCatalog();
+        CollectionWriter coll(&opCtx, nss);
 
-        string idxName = "a";
+        std::string idxName = "a";
         BSONObj spec = BSON("key" << BSON("a" << 1) << "name" << idxName << "v"
                                   << static_cast<int>(kIndexVersion));
 
         {
             WriteUnitOfWork uow(&opCtx);
-            ASSERT_OK(catalog->createIndexOnEmptyCollection(&opCtx, spec));
+            IndexCatalog* catalog = coll.getWritableCollection(&opCtx)->getIndexCatalog();
+            ASSERT_OK(catalog->createIndexOnEmptyCollection(
+                &opCtx, coll.getWritableCollection(&opCtx), spec));
             insertRecord(&opCtx, nss, BSON("a" << 1));
             insertRecord(&opCtx, nss, BSON("a" << 2));
             insertRecord(&opCtx, nss, BSON("a" << 3));
@@ -575,19 +580,17 @@ template <bool rollback>
 class CreateDropIndex {
 public:
     void run() {
-        string ns = "unittests.rollback_create_drop_index";
+        std::string ns = "unittests.rollback_create_drop_index";
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
         NamespaceString nss(ns);
         dropDatabase(&opCtx, nss);
         createCollection(&opCtx, nss);
 
-        AutoGetDb autoDb(&opCtx, nss.db(), MODE_X);
+        AutoGetDb autoDb(&opCtx, nss.dbName(), MODE_X);
+        CollectionWriter coll(&opCtx, nss);
 
-        Collection* coll = CollectionCatalog::get(&opCtx).lookupCollectionByNamespace(&opCtx, nss);
-        IndexCatalog* catalog = coll->getIndexCatalog();
-
-        string idxName = "a";
+        std::string idxName = "a";
         BSONObj spec = BSON("key" << BSON("a" << 1) << "name" << idxName << "v"
                                   << static_cast<int>(kIndexVersion));
 
@@ -595,8 +598,10 @@ public:
 
         {
             WriteUnitOfWork uow(&opCtx);
+            IndexCatalog* catalog = coll.getWritableCollection(&opCtx)->getIndexCatalog();
 
-            ASSERT_OK(catalog->createIndexOnEmptyCollection(&opCtx, spec));
+            ASSERT_OK(catalog->createIndexOnEmptyCollection(
+                &opCtx, coll.getWritableCollection(&opCtx), spec));
             insertRecord(&opCtx, nss, BSON("a" << 1));
             insertRecord(&opCtx, nss, BSON("a" << 2));
             insertRecord(&opCtx, nss, BSON("a" << 3));
@@ -619,18 +624,18 @@ template <bool rollback>
 class CreateCollectionAndIndexes {
 public:
     void run() {
-        string ns = "unittests.rollback_create_collection_and_indexes";
+        std::string ns = "unittests.rollback_create_collection_and_indexes";
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
         NamespaceString nss(ns);
         dropDatabase(&opCtx, nss);
 
-        Lock::DBLock dbXLock(&opCtx, nss.db(), MODE_X);
-        OldClientContext ctx(&opCtx, nss.ns());
+        Lock::DBLock dbXLock(&opCtx, nss.dbName(), MODE_X);
+        OldClientContext ctx(&opCtx, nss);
 
-        string idxNameA = "indexA";
-        string idxNameB = "indexB";
-        string idxNameC = "indexC";
+        std::string idxNameA = "indexA";
+        std::string idxNameB = "indexB";
+        std::string idxNameC = "indexC";
         BSONObj specA = BSON("key" << BSON("a" << 1) << "name" << idxNameA << "v"
                                    << static_cast<int>(kIndexVersion));
         BSONObj specB = BSON("key" << BSON("b" << 1) << "name" << idxNameB << "v"
@@ -647,13 +652,13 @@ public:
                 assertGet(CollectionOptions::parse(BSONObj(), CollectionOptions::parseForCommand));
             ASSERT_OK(ctx.db()->userCreateNS(&opCtx, nss, collectionOptions, false));
             ASSERT(collectionExists(&opCtx, &ctx, nss.ns()));
-            Collection* coll =
-                CollectionCatalog::get(&opCtx).lookupCollectionByNamespace(&opCtx, nss);
-            IndexCatalog* catalog = coll->getIndexCatalog();
+            CollectionWriter coll(&opCtx, nss);
+            auto writableColl = coll.getWritableCollection(&opCtx);
+            IndexCatalog* catalog = writableColl->getIndexCatalog();
 
-            ASSERT_OK(catalog->createIndexOnEmptyCollection(&opCtx, specA));
-            ASSERT_OK(catalog->createIndexOnEmptyCollection(&opCtx, specB));
-            ASSERT_OK(catalog->createIndexOnEmptyCollection(&opCtx, specC));
+            ASSERT_OK(catalog->createIndexOnEmptyCollection(&opCtx, writableColl, specA));
+            ASSERT_OK(catalog->createIndexOnEmptyCollection(&opCtx, writableColl, specB));
+            ASSERT_OK(catalog->createIndexOnEmptyCollection(&opCtx, writableColl, specC));
 
             if (!rollback) {
                 uow.commit();
@@ -717,3 +722,4 @@ public:
 OldStyleSuiteInitializer<All> all;
 
 }  // namespace RollbackTests
+}  // namespace mongo
