@@ -27,24 +27,29 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include <memory>
 
 #include "mongo/db/catalog/multi_index_block.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index/wildcard_access_method.h"
+#include "mongo/db/query/wildcard_multikey_paths.h"
+#include "mongo/db/record_id_helpers.h"
 #include "mongo/db/repl/storage_interface_impl.h"
 #include "mongo/db/storage/sorted_data_interface.h"
 #include "mongo/logv2/log.h"
 #include "mongo/unittest/unittest.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
+
 namespace mongo {
 namespace {
 
 using namespace unittest;
 
-static const RecordId kMetadataId = RecordId::minReserved();
+static const RecordId kMetadataId = record_id_helpers::reservedIdFor(
+    record_id_helpers::ReservationId::kWildcardMultikeyMetadataId, KeyFormat::Long);
 
 static const int kIndexVersion = static_cast<int>(IndexDescriptor::kLatestIndexVersion);
 static const NamespaceString kDefaultNSS{"wildcard_multikey_persistence.test"};
@@ -93,14 +98,16 @@ protected:
                                    const NamespaceString& nss = kDefaultNSS,
                                    const std::string& indexName = kDefaultIndexName) {
         // Subsequent operations must take place under a collection lock.
-        AutoGetCollectionForRead autoColl(opCtx(), nss);
-        auto collection = autoColl.getCollection();
+        AutoGetCollectionForRead collection(opCtx(), nss);
 
         // Verify whether or not the index has been marked as multikey.
-        ASSERT_EQ(expectIndexIsMultikey, getIndexDesc(collection, indexName)->isMultikey());
+        ASSERT_EQ(expectIndexIsMultikey,
+                  getIndexDesc(collection.getCollection(), indexName)
+                      ->getEntry()
+                      ->isMultikey(opCtx(), *collection));
 
         // Obtain a cursor over the index, and confirm that the keys are present in order.
-        auto indexCursor = getIndexCursor(collection, indexName);
+        auto indexCursor = getIndexCursor(collection.getCollection(), indexName);
 
         KeyString::Value keyStringForSeek = IndexEntryComparison::makeKeyStringFromBSONKeyForSeek(
             BSONObj(), KeyString::Version::V1, Ordering::make(BSONObj()), true, true);
@@ -132,7 +139,7 @@ protected:
      * Verifes that the index access method associated with 'indexName' in the collection identified
      * by 'nss' reports 'expectedPaths' as the set of multikey paths.
      */
-    void assertMultikeyPathSetEquals(const std::set<std::string>& expectedPaths,
+    void assertMultikeyPathSetEquals(const OrderedPathSet& expectedPaths,
                                      const NamespaceString& nss = kDefaultNSS,
                                      const std::string& indexName = kDefaultIndexName) {
         // Convert the set of std::string to a set of FieldRef.
@@ -142,11 +149,12 @@ protected:
         }
         ASSERT_EQ(expectedPaths.size(), expectedFieldRefs.size());
 
-        AutoGetCollectionForRead autoColl(opCtx(), nss);
-        auto collection = autoColl.getCollection();
-        auto indexAccessMethod = getIndex(collection, indexName);
+        AutoGetCollectionForRead collection(opCtx(), nss);
+        auto indexAccessMethod = getIndex(collection.getCollection(), indexName);
         MultikeyMetadataAccessStats stats;
-        auto multikeyPathSet = indexAccessMethod->getMultikeyPathSet(opCtx(), &stats);
+        auto wam = dynamic_cast<const WildcardAccessMethod*>(indexAccessMethod);
+        ASSERT(wam != nullptr);
+        auto multikeyPathSet = getWildcardMultikeyPathSet(wam, opCtx(), &stats);
 
         ASSERT(expectedFieldRefs == multikeyPathSet);
     }
@@ -191,27 +199,29 @@ protected:
         BSONObjBuilder bob = std::move(BSONObjBuilder() << "name" << name << "key" << key);
 
         if (!pathProjection.isEmpty())
-            bob << IndexDescriptor::kPathProjectionFieldName << pathProjection;
+            bob << IndexDescriptor::kWildcardProjectionFieldName << pathProjection;
 
         auto indexSpec = (bob << "v" << kIndexVersion << "background" << background).obj();
 
-        Lock::DBLock dbLock(opCtx(), nss.db(), MODE_X);
+        Lock::DBLock dbLock(opCtx(), nss.dbName(), MODE_X);
         AutoGetCollection autoColl(opCtx(), nss, MODE_X);
-        auto coll = autoColl.getCollection();
+        CollectionWriter coll(opCtx(), autoColl);
 
         MultiIndexBlock indexer;
-        auto abortOnExit = makeGuard(
+        ScopeGuard abortOnExit(
             [&] { indexer.abortIndexBuild(opCtx(), coll, MultiIndexBlock::kNoopOnCleanUpFn); });
 
         // Initialize the index builder and add all documents currently in the collection.
         ASSERT_OK(
             indexer.init(opCtx(), coll, indexSpec, MultiIndexBlock::kNoopOnInitFn).getStatus());
-        ASSERT_OK(indexer.insertAllDocumentsInCollection(opCtx(), coll));
-        ASSERT_OK(indexer.checkConstraints(opCtx()));
+        ASSERT_OK(indexer.insertAllDocumentsInCollection(opCtx(), coll.get()));
+        ASSERT_OK(indexer.checkConstraints(opCtx(), coll.get()));
 
         WriteUnitOfWork wunit(opCtx());
-        ASSERT_OK(indexer.commit(
-            opCtx(), coll, MultiIndexBlock::kNoopOnCreateEachFn, MultiIndexBlock::kNoopOnCommitFn));
+        ASSERT_OK(indexer.commit(opCtx(),
+                                 coll.getWritableCollection(opCtx()),
+                                 MultiIndexBlock::kNoopOnCreateEachFn,
+                                 MultiIndexBlock::kNoopOnCommitFn));
         abortOnExit.dismiss();
         wunit.commit();
     }
@@ -225,17 +235,20 @@ protected:
         return docs;
     }
 
-    const IndexDescriptor* getIndexDesc(const Collection* collection, const StringData indexName) {
+    const IndexDescriptor* getIndexDesc(const CollectionPtr& collection,
+                                        const StringData indexName) {
         return collection->getIndexCatalog()->findIndexByName(opCtx(), indexName);
     }
 
-    const IndexAccessMethod* getIndex(Collection* collection, const StringData indexName) {
+    const SortedDataIndexAccessMethod* getIndex(const CollectionPtr& collection,
+                                                const StringData indexName) {
         return collection->getIndexCatalog()
             ->getEntry(getIndexDesc(collection, indexName))
-            ->accessMethod();
+            ->accessMethod()
+            ->asSortedData();
     }
 
-    std::unique_ptr<SortedDataInterface::Cursor> getIndexCursor(Collection* collection,
+    std::unique_ptr<SortedDataInterface::Cursor> getIndexCursor(const CollectionPtr& collection,
                                                                 const StringData indexName) {
         return getIndex(collection, indexName)->newCursor(opCtx());
     }
