@@ -35,6 +35,7 @@
 #include <yaml-cpp/yaml.h>
 
 #include "mongo/base/initializer.h"
+#include "mongo/bson/bson_validate.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/commands/test_commands_enabled.h"
 #include "mongo/db/json.h"
@@ -44,11 +45,14 @@
 #include "mongo/rpc/op_msg.h"
 #include "mongo/stdx/thread.h"
 #include "mongo/unittest/temp_dir.h"
+#include "mongo/unittest/thread_assertion_monitor.h"
 #include "mongo/unittest/unittest.h"
+#include "mongo/util/exit_code.h"
 #include "mongo/util/options_parser/environment.h"
 #include "mongo/util/options_parser/option_section.h"
 #include "mongo/util/options_parser/options_parser.h"
 #include "mongo/util/quick_exit.h"
+#include "mongo/util/scopeguard.h"
 #include "mongo/util/shared_buffer.h"
 #include "mongo/util/signal_handlers_synchronous.h"
 #include "mongo/util/text.h"
@@ -59,7 +63,14 @@ mongo_embedded_v1_lib* global_lib_handle;
 
 namespace {
 
-std::unique_ptr<mongo::unittest::TempDir> globalTempDir;
+
+auto& getGlobalTempDir() {
+    static std::unique_ptr<mongo::unittest::TempDir> globalTempDir;
+    if (!globalTempDir) {
+        globalTempDir = std::make_unique<mongo::unittest::TempDir>("embedded_mongo");
+    }
+    return globalTempDir;
+}
 
 struct StatusDestructor {
     void operator()(mongo_embedded_v1_status* const p) const noexcept {
@@ -97,13 +108,6 @@ using MongoDBCAPIClientPtr = std::unique_ptr<mongo_embedded_v1_client, ClientDes
 class MongodbCAPITest : public mongo::unittest::Test {
 protected:
     void setUp() {
-        status = mongo_embedded_v1_status_create();
-        ASSERT(status != nullptr);
-
-        if (!globalTempDir) {
-            globalTempDir = std::make_unique<mongo::unittest::TempDir>("embedded_mongo");
-        }
-
         mongo_embedded_v1_init_params params;
         params.log_flags = MONGO_EMBEDDED_V1_LOG_STDOUT;
         params.log_callback = nullptr;
@@ -115,25 +119,34 @@ protected:
         yaml << YAML::Key << "storage";
         yaml << YAML::Value << YAML::BeginMap;
         yaml << YAML::Key << "dbPath";
-        yaml << YAML::Value << globalTempDir->path();
+        yaml << YAML::Value << getGlobalTempDir()->path();
         yaml << YAML::EndMap;  // storage
 
         yaml << YAML::EndMap;
 
         params.yaml_config = yaml.c_str();
 
+        auto* status = mongo_embedded_v1_status_create();
+        ASSERT(status);
+
         lib = mongo_embedded_v1_lib_init(&params, status);
         ASSERT(lib != nullptr) << mongo_embedded_v1_status_get_explanation(status);
 
         db = mongo_embedded_v1_instance_create(lib, yaml.c_str(), status);
         ASSERT(db != nullptr) << mongo_embedded_v1_status_get_explanation(status);
+
+        mongo_embedded_v1_status_destroy(status);
     }
 
     void tearDown() {
+        auto* status = mongo_embedded_v1_status_create();
+        ASSERT(status);
+
         ASSERT_EQUALS(mongo_embedded_v1_instance_destroy(db, status), MONGO_EMBEDDED_V1_SUCCESS)
             << mongo_embedded_v1_status_get_explanation(status);
         ASSERT_EQUALS(mongo_embedded_v1_lib_fini(lib, status), MONGO_EMBEDDED_V1_SUCCESS)
             << mongo_embedded_v1_status_get_explanation(status);
+
         mongo_embedded_v1_status_destroy(status);
     }
 
@@ -142,8 +155,13 @@ protected:
     }
 
     MongoDBCAPIClientPtr createClient() const {
+        auto* status = mongo_embedded_v1_status_create();
+        ASSERT(status);
+
         MongoDBCAPIClientPtr client(mongo_embedded_v1_client_create(db, status));
         ASSERT(client.get() != nullptr) << mongo_embedded_v1_status_get_explanation(status);
+
+        mongo_embedded_v1_status_destroy(status);
         return client;
     }
 
@@ -161,11 +179,16 @@ protected:
         void* output;
         size_t outputSize;
 
+        auto* status = mongo_embedded_v1_status_create();
+        ASSERT(status);
+
         // call the wire protocol
         int err = mongo_embedded_v1_client_invoke(
             client.get(), inputMessage.buf(), inputMessage.size(), &output, &outputSize, status);
         ASSERT_EQUALS(err, MONGO_EMBEDDED_V1_SUCCESS);
 
+        mongo_embedded_v1_status_destroy(status);
+
         // convert the shared buffer to a mongo::message and ensure that it is valid
         auto outputMessage = messageFromBuffer(output, outputSize);
         ASSERT(outputMessage.size() > 0);
@@ -173,7 +196,7 @@ protected:
 
         // convert the message into an OpMessage to examine its BSON
         auto outputOpMsg = mongo::OpMsg::parseOwned(outputMessage);
-        ASSERT(outputOpMsg.body.valid(mongo::BSONVersion::kLatest));
+        ASSERT_OK(validateBSON(outputOpMsg.body));
         return outputOpMsg.body;
     }
 
@@ -181,7 +204,6 @@ protected:
 protected:
     mongo_embedded_v1_lib* lib;
     mongo_embedded_v1_instance* db;
-    mongo_embedded_v1_status* status;
 };
 
 TEST_F(MongodbCAPITest, CreateAndDestroyDB) {
@@ -329,52 +351,54 @@ TEST_F(MongodbCAPITest, InsertMultipleDocuments) {
 }
 
 TEST_F(MongodbCAPITest, KillOp) {
-    auto client = createClient();
-
-    mongo::stdx::thread killOpThread([this]() {
+    mongo::unittest::threadAssertionMonitoredTest([&](auto& assertionMonitor) {
         auto client = createClient();
 
-        mongo::BSONObj currentOpObj = mongo::fromjson("{currentOp: 1}");
-        auto currentOpMsg = mongo::OpMsgRequest::fromDBAndBody("admin", currentOpObj);
-        mongo::BSONObj outputBSON;
+        auto killOpThread = assertionMonitor.spawn([&]() {
+            auto client = createClient();
 
-        // Wait for the sleep command to start in the main test thread.
-        int opid = -1;
-        do {
-            outputBSON = performRpc(client, currentOpMsg);
-            auto inprog = outputBSON.getObjectField("inprog");
+            mongo::BSONObj currentOpObj = mongo::fromjson("{currentOp: 1}");
+            auto currentOpMsg = mongo::OpMsgRequest::fromDBAndBody("admin", currentOpObj);
+            mongo::BSONObj outputBSON;
 
-            // See if we find the sleep command among the running commands
-            for (const auto& elt : inprog) {
-                auto inprogObj = inprog.getObjectField(elt.fieldNameStringData());
-                std::string ns = inprogObj.getStringField("ns");
-                if (ns == "admin.$cmd") {
-                    opid = inprogObj.getIntField("opid");
-                    break;
+            // Wait for the sleep command to start in the main test thread.
+            int opid = -1;
+            do {
+                outputBSON = performRpc(client, currentOpMsg);
+                auto inprog = outputBSON.getObjectField("inprog");
+
+                // See if we find the sleep command among the running commands
+                for (const auto& elt : inprog) {
+                    auto inprogObj = inprog.getObjectField(elt.fieldNameStringData());
+                    std::string ns = inprogObj.getStringField("ns").toString();
+                    if (ns == "admin.$cmd") {
+                        opid = inprogObj.getIntField("opid");
+                        break;
+                    }
                 }
-            }
-        } while (opid == -1);
+            } while (opid == -1);
 
-        // Sleep command found, kill it.
-        std::stringstream ss;
-        ss << "{'killOp': 1, 'op': " << opid << "}";
-        mongo::BSONObj killOpObj = mongo::fromjson(ss.str());
-        auto killOpMsg = mongo::OpMsgRequest::fromDBAndBody("admin", killOpObj);
-        outputBSON = performRpc(client, killOpMsg);
+            // Sleep command found, kill it.
+            std::stringstream ss;
+            ss << "{'killOp': 1, 'op': " << opid << "}";
+            mongo::BSONObj killOpObj = mongo::fromjson(ss.str());
+            auto killOpMsg = mongo::OpMsgRequest::fromDBAndBody("admin", killOpObj);
+            outputBSON = performRpc(client, killOpMsg);
+
+            ASSERT(outputBSON.hasField("ok"));
+            ASSERT(outputBSON.getField("ok").numberDouble() == 1.0);
+        });
+
+        mongo::ScopeGuard guard = [&] { killOpThread.join(); };
+
+        mongo::BSONObj sleepObj = mongo::fromjson("{'sleep': {'secs': 1000}}");
+        auto sleepOpMsg = mongo::OpMsgRequest::fromDBAndBody("admin", sleepObj);
+        auto outputBSON = performRpc(client, sleepOpMsg);
 
         ASSERT(outputBSON.hasField("ok"));
-        ASSERT(outputBSON.getField("ok").numberDouble() == 1.0);
+        ASSERT(outputBSON.getField("ok").numberDouble() != 1.0);
+        ASSERT(outputBSON.getIntField("code") == mongo::ErrorCodes::Interrupted);
     });
-
-    mongo::BSONObj sleepObj = mongo::fromjson("{'sleep': {'secs': 1000}}");
-    auto sleepOpMsg = mongo::OpMsgRequest::fromDBAndBody("admin", sleepObj);
-    auto outputBSON = performRpc(client, sleepOpMsg);
-
-    ASSERT(outputBSON.hasField("ok"));
-    ASSERT(outputBSON.getField("ok").numberDouble() != 1.0);
-    ASSERT(outputBSON.getIntField("code") == mongo::ErrorCodes::Interrupted);
-
-    killOpThread.join();
 }
 
 TEST_F(MongodbCAPITest, ReadDB) {
@@ -385,7 +409,7 @@ TEST_F(MongodbCAPITest, ReadDB) {
     auto outputBSON = performRpc(client, findMsg);
 
 
-    ASSERT(outputBSON.valid(mongo::BSONVersion::kLatest));
+    ASSERT_OK(validateBSON(outputBSON));
     ASSERT(outputBSON.hasField("cursor"));
     ASSERT(outputBSON.getField("cursor").embeddedObject().hasField("firstBatch"));
     mongo::BSONObj arrObj =
@@ -410,7 +434,7 @@ TEST_F(MongodbCAPITest, InsertAndRead) {
         "{insert: 'collection_name', documents: [{firstName: 'Mongo', lastName: 'DB', age: 10}]}");
     auto insertOpMsg = mongo::OpMsgRequest::fromDBAndBody("db_name", insertObj);
     auto outputBSON1 = performRpc(client, insertOpMsg);
-    ASSERT(outputBSON1.valid(mongo::BSONVersion::kLatest));
+    ASSERT_OK(validateBSON(outputBSON1));
     ASSERT(outputBSON1.hasField("n"));
     ASSERT(outputBSON1.getIntField("n") == 1);
     ASSERT(outputBSON1.hasField("ok"));
@@ -419,7 +443,7 @@ TEST_F(MongodbCAPITest, InsertAndRead) {
     mongo::BSONObj findObj = mongo::fromjson("{find: 'collection_name', limit: 1}");
     auto findMsg = mongo::OpMsgRequest::fromDBAndBody("db_name", findObj);
     auto outputBSON2 = performRpc(client, findMsg);
-    ASSERT(outputBSON2.valid(mongo::BSONVersion::kLatest));
+    ASSERT_OK(validateBSON(outputBSON2));
     ASSERT(outputBSON2.hasField("cursor"));
     ASSERT(outputBSON2.getField("cursor").embeddedObject().hasField("firstBatch"));
     mongo::BSONObj arrObj =
@@ -445,7 +469,7 @@ TEST_F(MongodbCAPITest, InsertAndReadDifferentClients) {
         "{insert: 'collection_name', documents: [{firstName: 'Mongo', lastName: 'DB', age: 10}]}");
     auto insertOpMsg = mongo::OpMsgRequest::fromDBAndBody("db_name", insertObj);
     auto outputBSON1 = performRpc(client1, insertOpMsg);
-    ASSERT(outputBSON1.valid(mongo::BSONVersion::kLatest));
+    ASSERT_OK(validateBSON(outputBSON1));
     ASSERT(outputBSON1.hasField("n"));
     ASSERT(outputBSON1.getIntField("n") == 1);
     ASSERT(outputBSON1.hasField("ok"));
@@ -454,7 +478,7 @@ TEST_F(MongodbCAPITest, InsertAndReadDifferentClients) {
     mongo::BSONObj findObj = mongo::fromjson("{find: 'collection_name', limit: 1}");
     auto findMsg = mongo::OpMsgRequest::fromDBAndBody("db_name", findObj);
     auto outputBSON2 = performRpc(client2, findMsg);
-    ASSERT(outputBSON2.valid(mongo::BSONVersion::kLatest));
+    ASSERT_OK(validateBSON(outputBSON2));
     ASSERT(outputBSON2.hasField("cursor"));
     ASSERT(outputBSON2.getField("cursor").embeddedObject().hasField("firstBatch"));
     mongo::BSONObj arrObj =
@@ -479,7 +503,7 @@ TEST_F(MongodbCAPITest, InsertAndDelete) {
         "age: 10}]}");
     auto insertOpMsg = mongo::OpMsgRequest::fromDBAndBody("db_name", insertObj);
     auto outputBSON1 = performRpc(client, insertOpMsg);
-    ASSERT(outputBSON1.valid(mongo::BSONVersion::kLatest));
+    ASSERT_OK(validateBSON(outputBSON1));
     ASSERT(outputBSON1.hasField("n"));
     ASSERT(outputBSON1.getIntField("n") == 1);
     ASSERT(outputBSON1.hasField("ok"));
@@ -492,7 +516,7 @@ TEST_F(MongodbCAPITest, InsertAndDelete) {
         "1}]}");
     auto deleteOpMsg = mongo::OpMsgRequest::fromDBAndBody("db_name", deleteObj);
     auto outputBSON2 = performRpc(client, deleteOpMsg);
-    ASSERT(outputBSON2.valid(mongo::BSONVersion::kLatest));
+    ASSERT_OK(validateBSON(outputBSON2));
     ASSERT(outputBSON2.hasField("n"));
     ASSERT(outputBSON2.getIntField("n") == 1);
     ASSERT(outputBSON2.hasField("ok"));
@@ -508,7 +532,7 @@ TEST_F(MongodbCAPITest, InsertAndUpdate) {
         "age: 10}]}");
     auto insertOpMsg = mongo::OpMsgRequest::fromDBAndBody("db_name", insertObj);
     auto outputBSON1 = performRpc(client, insertOpMsg);
-    ASSERT(outputBSON1.valid(mongo::BSONVersion::kLatest));
+    ASSERT_OK(validateBSON(outputBSON1));
     ASSERT(outputBSON1.hasField("n"));
     ASSERT(outputBSON1.getIntField("n") == 1);
     ASSERT(outputBSON1.hasField("ok"));
@@ -521,7 +545,7 @@ TEST_F(MongodbCAPITest, InsertAndUpdate) {
         "{age: 5}}}]}");
     auto updateOpMsg = mongo::OpMsgRequest::fromDBAndBody("db_name", updateObj);
     auto outputBSON2 = performRpc(client, updateOpMsg);
-    ASSERT(outputBSON2.valid(mongo::BSONVersion::kLatest));
+    ASSERT_OK(validateBSON(outputBSON2));
     ASSERT(outputBSON2.hasField("ok"));
     ASSERT(outputBSON2.getField("ok").numberDouble() == 1.0);
     ASSERT(outputBSON2.hasField("nModified"));
@@ -531,9 +555,10 @@ TEST_F(MongodbCAPITest, InsertAndUpdate) {
 TEST_F(MongodbCAPITest, RunListCommands) {
     auto client = createClient();
 
-    std::vector<std::string> whitelist = {"_hashBSONElement",
+    std::vector<std::string> allowlist = {"_hashBSONElement",
                                           "_killOperations",
                                           "aggregate",
+                                          "analyze",
                                           "buildInfo",
                                           "collMod",
                                           "collStats",
@@ -568,6 +593,7 @@ TEST_F(MongodbCAPITest, RunListCommands) {
                                           "listCollections",
                                           "listCommands",
                                           "listDatabases",
+                                          "listDatabasesForAllTenants",
                                           "listIndexes",
                                           "lockInfo",
                                           "logMessage",
@@ -580,17 +606,17 @@ TEST_F(MongodbCAPITest, RunListCommands) {
                                           "refreshLogicalSessionCacheNow",
                                           "refreshSessions",
                                           "renameCollection",
-                                          "resetError",
                                           "serverStatus",
                                           "setParameter",
                                           "sleep",
                                           "startSession",
                                           "update",
                                           "validate",
+                                          "validateDBMetadata",
                                           "waitForFailPoint",
                                           "whatsmysni"};
 
-    std::sort(whitelist.begin(), whitelist.end());
+    std::sort(allowlist.begin(), allowlist.end());
 
     mongo::BSONObj listCommandsObj = mongo::fromjson("{ listCommands: 1 }");
     auto listCommandsOpMsg = mongo::OpMsgRequest::fromDBAndBody("db_name", listCommandsObj);
@@ -604,15 +630,15 @@ TEST_F(MongodbCAPITest, RunListCommands) {
 
     std::vector<std::string> missing;
     std::vector<std::string> unsupported;
-    std::set_difference(whitelist.begin(),
-                        whitelist.end(),
+    std::set_difference(allowlist.begin(),
+                        allowlist.end(),
                         commands.begin(),
                         commands.end(),
                         std::back_inserter(missing));
     std::set_difference(commands.begin(),
                         commands.end(),
-                        whitelist.begin(),
-                        whitelist.end(),
+                        allowlist.begin(),
+                        allowlist.end(),
                         std::back_inserter(unsupported));
 
     if (!missing.empty()) {
@@ -655,14 +681,14 @@ int main(const int argc, const char* const* const argv) {
     auto ret = mongo::embedded::addMongoEmbeddedTestOptions(&options);
     if (!ret.isOK()) {
         std::cerr << ret << std::endl;
-        return EXIT_FAILURE;
+        return static_cast<int>(mongo::ExitCode::fail);
     }
 
     ret = moe::OptionsParser().run(
         options, std::vector<std::string>(argv, argv + argc), &environment);
     if (!ret.isOK()) {
         std::cerr << options.helpString();
-        return EXIT_FAILURE;
+        return static_cast<int>(mongo::ExitCode::fail);
     }
     if (environment.count("tempPath")) {
         ::mongo::unittest::TempDir::setTempPath(environment["tempPath"].as<std::string>());
@@ -682,16 +708,16 @@ int main(const int argc, const char* const* const argv) {
     // The reason this works is that the unittest system relies on other systems being initialized
     // through global init and deinitialize just deinitializes systems that explicitly supports
     // deinit leaving the systems unittest needs initialized.
-    ret = mongo::runGlobalInitializers(std::vector<std::string>{});
+    ret = mongo::runGlobalInitializers(std::vector<std::string>{argv, argv + argc});
     if (!ret.isOK()) {
         std::cerr << "Global initilization failed";
-        return EXIT_FAILURE;
+        return static_cast<int>(mongo::ExitCode::fail);
     }
 
     ret = mongo::runGlobalDeinitializers();
     if (!ret.isOK()) {
         std::cerr << "Global deinitilization failed";
-        return EXIT_FAILURE;
+        return static_cast<int>(mongo::ExitCode::fail);
     }
 
     // Check so we can initialize the library without providing init params
@@ -700,14 +726,14 @@ int main(const int argc, const char* const* const argv) {
         std::cerr << "mongo_embedded_v1_init() failed with "
                   << mongo_embedded_v1_status_get_error(status.get()) << ": "
                   << mongo_embedded_v1_status_get_explanation(status.get()) << std::endl;
-        return EXIT_FAILURE;
+        return static_cast<int>(mongo::ExitCode::fail);
     }
 
     if (mongo_embedded_v1_lib_fini(lib, status.get()) != MONGO_EMBEDDED_V1_SUCCESS) {
         std::cerr << "mongo_embedded_v1_fini() failed with "
                   << mongo_embedded_v1_status_get_error(status.get()) << ": "
                   << mongo_embedded_v1_status_get_explanation(status.get()) << std::endl;
-        return EXIT_FAILURE;
+        return static_cast<int>(mongo::ExitCode::fail);
     }
 
     // Initialize the library with a log callback and test so we receive at least one callback
@@ -727,6 +753,16 @@ int main(const int argc, const char* const* const argv) {
     };
     params.log_user_data = &receivedCallback;
 
+    YAML::Emitter yaml;
+    yaml << YAML::BeginMap;
+    yaml << YAML::Key << "storage";
+    yaml << YAML::Value << YAML::BeginMap;
+    yaml << YAML::Key << "dbPath";
+    yaml << YAML::Value << getGlobalTempDir()->path();
+    yaml << YAML::EndMap;  // storage
+    yaml << YAML::EndMap;
+    params.yaml_config = yaml.c_str();
+
     lib = mongo_embedded_v1_lib_init(&params, nullptr);
     if (lib == nullptr) {
         std::cerr << "mongo_embedded_v1_init() failed with "
@@ -736,7 +772,8 @@ int main(const int argc, const char* const* const argv) {
 
     // Attempt to create an embedded instance to make sure something gets logged. This will probably
     // fail but that's fine.
-    mongo_embedded_v1_instance* instance = mongo_embedded_v1_instance_create(lib, nullptr, nullptr);
+    mongo_embedded_v1_instance* instance =
+        mongo_embedded_v1_instance_create(lib, yaml.c_str(), nullptr);
     if (instance) {
         mongo_embedded_v1_instance_destroy(instance, nullptr);
     }
@@ -749,11 +786,11 @@ int main(const int argc, const char* const* const argv) {
 
     if (!receivedCallback) {
         std::cerr << "Did not get a log callback." << std::endl;
-        return EXIT_FAILURE;
+        return static_cast<int>(mongo::ExitCode::fail);
     }
 
     const auto result = ::mongo::unittest::Suite::run(std::vector<std::string>(), "", "", 1);
 
-    globalTempDir.reset();
+    getGlobalTempDir().reset();
     mongo::quickExit(result);
 }
