@@ -36,12 +36,12 @@
 #include "mongo/base/initializer.h"
 #include "mongo/bson/json.h"
 #include "mongo/unittest/unittest.h"
+#include "mongo/util/exit_code.h"
 #include "mongo/util/quick_exit.h"
 #include "mongo/util/scopeguard.h"
 
 namespace {
 
-using mongo::makeGuard;
 using mongo::ScopeGuard;
 
 class StitchSupportTest : public mongo::unittest::Test {
@@ -285,7 +285,7 @@ protected:
         return std::string(stitch_support_v1_status_get_explanation(updateStatus));
     }
 
-    const std::string getModifiedPaths() {
+    std::string getModifiedPaths() {
         ASSERT(updateDetails);
 
         std::stringstream ss;
@@ -635,16 +635,16 @@ TEST_F(StitchSupportTest, TestUpsertProducesProperStatus) {
 // the Stitch Support Library intializer function that gets tested here.
 int main(const int argc, const char* const* const argv) {
     // See comment by the same code block in mongo_embedded_test.cpp
-    auto ret = mongo::runGlobalInitializers(std::vector<std::string>{});
+    auto ret = mongo::runGlobalInitializers(std::vector<std::string>{argv, argv + argc});
     if (!ret.isOK()) {
         std::cerr << "Global initilization failed";
-        return EXIT_FAILURE;
+        return static_cast<int>(mongo::ExitCode::fail);
     }
 
     ret = mongo::runGlobalDeinitializers();
     if (!ret.isOK()) {
         std::cerr << "Global deinitilization failed";
-        return EXIT_FAILURE;
+        return static_cast<int>(mongo::ExitCode::fail);
     }
 
     const auto result = ::mongo::unittest::Suite::run(std::vector<std::string>(), "", "", 1);
