@@ -27,16 +27,15 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kConnectionPool
-
-#include "mongo/platform/basic.h"
 
 #include "mongo/executor/connection_pool.h"
 
 #include <fmt/format.h>
 #include <fmt/ostream.h>
+#include <memory>
 
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/server_feature_flags_gen.h"
 #include "mongo/executor/connection_pool_stats.h"
 #include "mongo/executor/remote_command_request.h"
 #include "mongo/logv2/log.h"
@@ -48,6 +47,9 @@
 #include "mongo/util/lru_cache.h"
 #include "mongo/util/scopeguard.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kConnectionPool
+
+
 using namespace fmt::literals;
 
 // One interesting implementation note herein concerns how setup() and
@@ -61,6 +63,9 @@ namespace mongo {
 
 namespace {
 
+MONGO_FAIL_POINT_DEFINE(refreshConnectionAfterEveryCommand);
+MONGO_FAIL_POINT_DEFINE(forceExecutorConnectionPoolTimeout);
+
 auto makeSeveritySuppressor() {
     return std::make_unique<logv2::KeyedSeveritySuppressor<HostAndPort>>(
         Seconds{1}, logv2::LogSeverity::Log(), logv2::LogSeverity::Debug(2));
@@ -273,7 +278,7 @@ public:
      * Gets a connection from the specific pool. Sinks a unique_lock from the
      * parent to preserve the lock on _mutex
      */
-    Future<ConnectionHandle> getConnection(Milliseconds timeout);
+    Future<ConnectionHandle> getConnection(Milliseconds timeout, ErrorCodes::Error timeoutCode);
 
     /**
      * Triggers the shutdown procedure. This function sets isShutdown to true
@@ -305,11 +310,21 @@ public:
      */
     size_t refreshingConnections() const;
 
+    /**
+     * Returns the number of all refreshed connections in the pool.
+     */
+    size_t refreshedConnections() const;
+
     /**
      * Returns the total number of connections ever created in this pool.
      */
     size_t createdConnections() const;
 
+    /**
+     * Returns the number of connections that expire and are destroyed before they are ever used.
+     */
+    size_t neverUsedConnections() const;
+
     /**
      * Returns the total number of connections currently open that belong to
      * this pool. This is the sum of refreshingConnections, availableConnections,
@@ -322,6 +337,22 @@ public:
      */
     size_t requestsPending() const;
 
+    /**
+     * Records the time it took to return the connection since it was requested, so that it can be
+     * reported in the connection pool stats.
+     */
+    void recordConnectionWaitTime(Date_t requestedAt) {
+        _connAcquisitionWaitTimeStats.increment(_parent->_factory->now() - requestedAt);
+    }
+
+    /**
+     * Returns connection acquisition wait time statistics to be included in the connection pool
+     * stats.
+     */
+    const ConnectionWaitTimeHistogram& connectionWaitTimeStats() {
+        return _connAcquisitionWaitTimeStats;
+    };
+
     /**
      * Returns the HostAndPort for this pool.
      */
@@ -354,15 +385,41 @@ private:
     using OwnedConnection = std::shared_ptr<ConnectionInterface>;
     using OwnershipPool = stdx::unordered_map<ConnectionInterface*, OwnedConnection>;
     using LRUOwnershipPool = LRUCache<OwnershipPool::key_type, OwnershipPool::mapped_type>;
-    using Request = std::pair<Date_t, Promise<ConnectionHandle>>;
+    struct Request {
+        Date_t expiration;
+        Promise<ConnectionHandle> promise;
+        ErrorCodes::Error timeoutCode;
+    };
+
     struct RequestComparator {
-        bool operator()(const Request& a, const Request& b) {
-            return a.first > b.first;
+        bool operator()(const Request& a, const Request& b) const {
+            return a.expiration > b.expiration;
         }
     };
 
     ConnectionHandle makeHandle(ConnectionInterface* connection);
 
+    /**
+     * Given a uniquely-owned OwnedConnection, returns an OwnedConnection
+     * pointing to the same object, but which gathers stats just before destruction.
+     */
+    OwnedConnection makeDeathNotificationWrapper(OwnedConnection h) {
+        invariant(h.use_count() == 1);
+        struct ConnWrap {
+            ConnWrap(OwnedConnection conn, std::weak_ptr<SpecificPool> owner)
+                : conn{std::move(conn)}, owner{std::move(owner)} {}
+            ~ConnWrap() {
+                if (conn->getLastUsed() == Date_t{})
+                    if (auto ownerSp = owner.lock())
+                        ++ownerSp->_neverUsed;
+            }
+            const OwnedConnection conn;
+            const std::weak_ptr<SpecificPool> owner;
+        };
+        ConnectionInterface* ptr = h.get();
+        return {std::make_shared<ConnWrap>(std::move(h), shared_from_this()), ptr};
+    }
+
     /**
      * Establishes connections until the ControllerInterface's target is met.
      */
@@ -428,6 +485,12 @@ private:
 
     size_t _created = 0;
 
+    size_t _refreshed = 0;
+
+    size_t _neverUsed = 0;
+
+    ConnectionWaitTimeHistogram _connAcquisitionWaitTimeStats{};
+
     transport::Session::TagMask _tags = transport::Session::kPending;
 
     HostHealth _health;
@@ -510,8 +573,12 @@ void ConnectionPool::dropConnections(const HostAndPort& hostAndPort) {
 void ConnectionPool::dropConnections(transport::Session::TagMask tags) {
     stdx::lock_guard lk(_mutex);
 
-    for (const auto& pair : _pools) {
-        auto& pool = pair.second;
+    // SpecificPool::triggerShutdown can cause iterator invalidation (e.g. `pool` removing itself
+    // from `_pools`), so we increment `it` early after grabbing a reference to the `pool` it points
+    // to.
+    for (auto it = _pools.begin(); it != _pools.end();) {
+        auto& pool = it->second;
+        ++it;
 
         if (pool->matchesTags(tags))
             continue;
@@ -537,19 +604,24 @@ void ConnectionPool::mutateTags(
 
 void ConnectionPool::get_forTest(const HostAndPort& hostAndPort,
                                  Milliseconds timeout,
+                                 ErrorCodes::Error timeoutCode,
                                  GetConnectionCallback cb) {
     // We kick ourselves onto the executor queue to prevent us from deadlocking with our own thread
-    auto getConnectionFunc = [this, hostAndPort, timeout, cb = std::move(cb)](Status&&) mutable {
-        get(hostAndPort, transport::kGlobalSSLMode, timeout)
-            .thenRunOn(_factory->getExecutor())
-            .getAsync(std::move(cb));
-    };
+    auto getConnectionFunc =
+        [this, hostAndPort, timeout, timeoutCode, cb = std::move(cb)](Status&&) mutable {
+            get(hostAndPort, transport::kGlobalSSLMode, timeout, timeoutCode)
+                .thenRunOn(_factory->getExecutor())
+                .getAsync(std::move(cb));
+        };
     _factory->getExecutor()->schedule(std::move(getConnectionFunc));
 }
 
 SemiFuture<ConnectionPool::ConnectionHandle> ConnectionPool::get(const HostAndPort& hostAndPort,
                                                                  transport::ConnectSSLMode sslMode,
-                                                                 Milliseconds timeout) {
+                                                                 Milliseconds timeout,
+                                                                 ErrorCodes::Error timeoutCode) {
+    auto connRequestedAt = _factory->now();
+
     stdx::lock_guard lk(_mutex);
 
     auto& pool = _pools[hostAndPort];
@@ -561,9 +633,15 @@ SemiFuture<ConnectionPool::ConnectionHandle> ConnectionPool::get(const HostAndPo
 
     invariant(pool);
 
-    auto connFuture = pool->getConnection(timeout);
+    auto connFuture = pool->getConnection(timeout, timeoutCode);
     pool->updateState();
 
+    if (gFeatureFlagConnHealthMetrics.isEnabledAndIgnoreFCV()) {
+        connFuture = std::move(connFuture).tap([connRequestedAt, pool = pool](const auto& conn) {
+            pool->recordConnectionWaitTime(connRequestedAt);
+        });
+    }
+
     return std::move(connFuture).semi();
 }
 
@@ -578,7 +656,13 @@ void ConnectionPool::appendConnectionStats(ConnectionPoolStats* stats) const {
         ConnectionStatsPer hostStats{pool->inUseConnections(),
                                      pool->availableConnections(),
                                      pool->createdConnections(),
-                                     pool->refreshingConnections()};
+                                     pool->refreshingConnections(),
+                                     pool->refreshedConnections(),
+                                     pool->neverUsedConnections()};
+
+        if (gFeatureFlagConnHealthMetrics.isEnabledAndIgnoreFCV()) {
+            hostStats.acquisitionWaitTimes = pool->connectionWaitTimeStats();
+        }
         stats->updateStatsForHost(_name, host, hostStats);
     }
 }
@@ -626,10 +710,18 @@ size_t ConnectionPool::SpecificPool::refreshingConnections() const {
     return _processingPool.size();
 }
 
+size_t ConnectionPool::SpecificPool::refreshedConnections() const {
+    return _refreshed;
+}
+
 size_t ConnectionPool::SpecificPool::createdConnections() const {
     return _created;
 }
 
+size_t ConnectionPool::SpecificPool::neverUsedConnections() const {
+    return _neverUsed;
+}
+
 size_t ConnectionPool::SpecificPool::openConnections() const {
     return _checkedOutPool.size() + _readyPool.size() + _processingPool.size();
 }
@@ -639,12 +731,38 @@ size_t ConnectionPool::SpecificPool::requestsPending() const {
 }
 
 Future<ConnectionPool::ConnectionHandle> ConnectionPool::SpecificPool::getConnection(
-    Milliseconds timeout) {
+    Milliseconds timeout, ErrorCodes::Error timeoutCode) {
 
     // Reset our activity timestamp
     auto now = _parent->_factory->now();
     _lastActiveTime = now;
 
+    auto pendingTimeout = _parent->_controller->pendingTimeout();
+    if (timeout < Milliseconds(0) || timeout > pendingTimeout) {
+        timeout = pendingTimeout;
+        // If controller's pending timeout is closest, timeoutCode is rewritten to the internal time
+        // limit error
+        timeoutCode = ErrorCodes::NetworkInterfaceExceededTimeLimit;
+    }
+
+    if (auto sfp = forceExecutorConnectionPoolTimeout.scoped(); MONGO_unlikely(sfp.isActive())) {
+        if (const Milliseconds failpointTimeout{sfp.getData()["timeout"].numberInt()};
+            failpointTimeout > Milliseconds{0}) {
+            auto pf = makePromiseFuture<ConnectionHandle>();
+            auto request = std::make_shared<Request>();
+            request->expiration = now + failpointTimeout;
+            request->promise = std::move(pf.promise);
+            request->timeoutCode = timeoutCode;
+            auto timeoutTimer = _parent->_factory->makeTimer();
+            timeoutTimer->setTimeout(failpointTimeout, [request, timeoutTimer]() mutable {
+                request->promise.setError(Status(
+                    request->timeoutCode,
+                    "Connection timed out due to forceExecutorConnectionPoolTimeout failpoint"));
+            });
+            return std::move(pf.future);
+        }
+    }
+
     // If we do not have requests, then we can fulfill immediately
     if (_requests.size() == 0) {
         auto conn = tryGetConnection();
@@ -659,10 +777,6 @@ Future<ConnectionPool::ConnectionHandle> ConnectionPool::SpecificPool::getConnec
         }
     }
 
-    auto pendingTimeout = _parent->_controller->pendingTimeout();
-    if (timeout < Milliseconds(0) || timeout > pendingTimeout) {
-        timeout = pendingTimeout;
-    }
     LOGV2_DEBUG(22560,
                 kDiagnosticLogLevel,
                 "Requesting new connection to {hostAndPort} with timeout {timeout}",
@@ -673,7 +787,7 @@ Future<ConnectionPool::ConnectionHandle> ConnectionPool::SpecificPool::getConnec
     const auto expiration = now + timeout;
     auto pf = makePromiseFuture<ConnectionHandle>();
 
-    _requests.push_back(make_pair(expiration, std::move(pf.promise)));
+    _requests.push_back({expiration, std::move(pf.promise), timeoutCode});
     std::push_heap(begin(_requests), end(_requests), RequestComparator{});
 
     return std::move(pf.future);
@@ -699,7 +813,7 @@ ConnectionPool::ConnectionHandle ConnectionPool::SpecificPool::tryGetConnection(
         _readyPool.erase(iter);
         conn->cancelTimeout();
 
-        if (!conn->isHealthy()) {
+        if (!conn->maybeHealthy()) {
             LOGV2(22561,
                   "Dropping unhealthy pooled connection to {hostAndPort}",
                   "Dropping unhealthy pooled connection",
@@ -726,6 +840,10 @@ ConnectionPool::ConnectionHandle ConnectionPool::SpecificPool::tryGetConnection(
 void ConnectionPool::SpecificPool::finishRefresh(ConnectionInterface* connPtr, Status status) {
     auto conn = takeFromProcessingPool(connPtr);
 
+    // We increment the total number of refreshed connections right upfront to track all completed
+    // refreshes.
+    _refreshed++;
+
     // If we're in shutdown, we don't need refreshed connections
     if (_health.isShutdown) {
         return;
@@ -810,10 +928,15 @@ void ConnectionPool::SpecificPool::returnConnection(ConnectionInterface* connPtr
         return;
     }
 
-    auto now = _parent->_factory->now();
-    if (needsRefreshTP <= now) {
-        // If we need to refresh this connection
+    // If we need to refresh this connection
+    bool shouldRefreshConnection = needsRefreshTP <= _parent->_factory->now();
 
+    if (MONGO_unlikely(refreshConnectionAfterEveryCommand.shouldFail())) {
+        LOGV2(5505501, "refresh connection after every command is on");
+        shouldRefreshConnection = true;
+    }
+
+    if (shouldRefreshConnection) {
         auto controls = _parent->_controller->getControls(_id);
         if (_readyPool.size() + _processingPool.size() + _checkedOutPool.size() >=
             controls.targetConnections) {
@@ -950,7 +1073,7 @@ void ConnectionPool::SpecificPool::processFailure(const Status& status) {
     }
 
     for (auto& request : _requests) {
-        request.second.setError(status);
+        request.promise.setError(status);
     }
 
     LOGV2_DEBUG(22573,
@@ -979,7 +1102,7 @@ void ConnectionPool::SpecificPool::fulfillRequests() {
         }
 
         // Grab the request and callback
-        auto promise = std::move(_requests.front().second);
+        auto promise = std::move(_requests.front().promise);
         std::pop_heap(begin(_requests), end(_requests), RequestComparator{});
         _requests.pop_back();
 
@@ -1037,6 +1160,7 @@ void ConnectionPool::SpecificPool::spawnConnections() {
                 "Spawning connections",
                 "connAllowance"_attr = allowance,
                 "hostAndPort"_attr = _hostAndPort);
+
     for (decltype(allowance) i = 0; i < allowance; ++i) {
         OwnedConnection handle;
         try {
@@ -1048,6 +1172,7 @@ void ConnectionPool::SpecificPool::spawnConnections() {
                         "reason"_attr = e.what());
         }
 
+        handle = makeDeathNotificationWrapper(std::move(handle));
         _processingPool[handle.get()] = handle;
         ++_created;
 
@@ -1055,7 +1180,8 @@ void ConnectionPool::SpecificPool::spawnConnections() {
         handle->setup(_parent->_controller->pendingTimeout(),
                       guardCallback([this](auto conn, auto status) {
                           finishRefresh(std::move(conn), std::move(status));
-                      }));
+                      }),
+                      _parent->getName());
     }
 }
 
@@ -1111,8 +1237,16 @@ void ConnectionPool::SpecificPool::updateEventTimer() {
     }
 
     // If a request would timeout before the next event, then it is the next event
-    if (_requests.size() && (_requests.front().first < nextEventTime)) {
-        nextEventTime = _requests.front().first;
+    if (_requests.size() && (_requests.front().expiration < nextEventTime)) {
+        nextEventTime = _requests.front().expiration;
+    }
+
+    // Clamp next event time to be either now or in the future. Next event time
+    // can be in the past anytime we wait a long time between invocations of
+    // updateState; in these cases, we want to set our event timer to expire
+    // immediately.
+    if (nextEventTime < now) {
+        nextEventTime = now;
     }
 
     // If our timer is already set to the next event, then we're done
@@ -1127,18 +1261,17 @@ void ConnectionPool::SpecificPool::updateEventTimer() {
     _eventTimer->cancelTimeout();
 
     // Set our event timer to timeout requests, refresh the state, and potentially expire this pool
-    auto deferredStateUpdateFunc = guardCallback([this, timeout]() {
+    auto deferredStateUpdateFunc = guardCallback([this]() {
         auto now = _parent->_factory->now();
 
         _health.isFailed = false;
 
-        while (_requests.size() && (_requests.front().first <= now)) {
+        while (_requests.size() && (_requests.front().expiration <= now)) {
             std::pop_heap(begin(_requests), end(_requests), RequestComparator{});
 
             auto& request = _requests.back();
-            request.second.setError(Status(
-                ErrorCodes::NetworkInterfaceExceededTimeLimit,
-                fmt::format("Couldn't get a connection within the time limit of {}", timeout)));
+            request.promise.setError(
+                Status(request.timeoutCode, "Couldn't get a connection within the time limit"));
             _requests.pop_back();
 
             // Since we've failed a request, we've interacted with external users
@@ -1202,7 +1335,7 @@ void ConnectionPool::SpecificPool::updateController() {
                 invariant(pool->_requests.empty());
             }
 
-            pool->triggerShutdown(Status(ErrorCodes::ShutdownInProgress,
+            pool->triggerShutdown(Status(ErrorCodes::ConnectionPoolExpired,
                                          str::stream() << "Pool for " << host << " has expired."));
         }
         return;
