@@ -27,9 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
-
-#include "mongo/platform/basic.h"
 
 #include <algorithm>
 #include <exception>
@@ -37,7 +34,7 @@
 
 #include "mongo/base/status_with.h"
 #include "mongo/client/connection_string.h"
-#include "mongo/db/wire_version.h"
+#include "mongo/db/concurrency/locker_noop_client_observer.h"
 #include "mongo/executor/connection_pool_stats.h"
 #include "mongo/executor/network_connection_hook.h"
 #include "mongo/executor/network_interface_integration_fixture.h"
@@ -52,6 +49,9 @@
 #include "mongo/util/assert_util.h"
 #include "mongo/util/scopeguard.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
+
 namespace mongo {
 namespace executor {
 namespace {
@@ -165,15 +165,15 @@ public:
 
     // NetworkInterfaceIntegrationFixture::tearDown() shuts down the NetworkInterface. We always
     // need to do it even if we have additional tearDown tasks.
-    using NetworkInterfaceIntegrationFixture::tearDown;
+    void tearDown() override {
+        NetworkInterfaceIntegrationFixture::tearDown();
+        ASSERT_EQ(getInProgress(), 0);
+    }
 
-    RemoteCommandRequest makeTestCommand(
-        Milliseconds timeout,
-        BSONObj cmd,
-        OperationContext* opCtx = nullptr,
-        boost::optional<RemoteCommandRequest::HedgeOptions> hedgeOptions = boost::none,
-        RemoteCommandRequest::FireAndForgetMode fireAndForgetMode =
-            RemoteCommandRequest::FireAndForgetMode::kOff) {
+    RemoteCommandRequest makeTestCommand(Milliseconds timeout,
+                                         BSONObj cmd,
+                                         OperationContext* opCtx = nullptr,
+                                         RemoteCommandRequest::Options options = {}) {
         auto cs = fixture();
         return RemoteCommandRequest(cs.getServers().front(),
                                     "admin",
@@ -181,8 +181,7 @@ public:
                                     BSONObj(),
                                     opCtx,
                                     timeout,
-                                    hedgeOptions,
-                                    fireAndForgetMode);
+                                    std::move(options));
     }
 
     BSONObj makeEchoCmdObj() {
@@ -300,13 +299,13 @@ private:
 class NetworkInterfaceInternalClientTest : public NetworkInterfaceTest {
 public:
     void setUp() override {
-        WireSpec::instance().isInternalClient = true;
+        resetIsInternalClient(true);
         NetworkInterfaceTest::setUp();
     }
 
     void tearDown() override {
         NetworkInterfaceTest::tearDown();
-        WireSpec::instance().isInternalClient = false;
+        resetIsInternalClient(false);
     }
 };
 
@@ -368,13 +367,12 @@ TEST_F(NetworkInterfaceTest, CancelRemotely) {
 
     auto cbh = makeCallbackHandle();
     auto deferred = [&] {
+        RemoteCommandRequest::Options options;
+        options.isHedgeEnabled = true;
         // Kick off an "echo" operation, which should block until cancelCommand causes
         // the operation to be killed.
-        auto deferred = runCommand(cbh,
-                                   makeTestCommand(kNoTimeout,
-                                                   makeEchoCmdObj(),
-                                                   nullptr /* opCtx */,
-                                                   RemoteCommandRequest::HedgeOptions()));
+        auto deferred = runCommand(
+            cbh, makeTestCommand(kNoTimeout, makeEchoCmdObj(), nullptr /* opCtx */, options));
 
         // Wait for the "echo" operation to start.
         numCurrentOpRan += waitForCommandToStart("echo", kMaxWait);
@@ -426,12 +424,11 @@ TEST_F(NetworkInterfaceTest, CancelRemotelyTimedOut) {
 
     auto cbh = makeCallbackHandle();
     auto deferred = [&] {
+        RemoteCommandRequest::Options options;
+        options.isHedgeEnabled = true;
         // Kick off a blocking "echo" operation.
-        auto deferred = runCommand(cbh,
-                                   makeTestCommand(kNoTimeout,
-                                                   makeEchoCmdObj(),
-                                                   nullptr /* opCtx */,
-                                                   RemoteCommandRequest::HedgeOptions()));
+        auto deferred = runCommand(
+            cbh, makeTestCommand(kNoTimeout, makeEchoCmdObj(), nullptr /* opCtx */, options));
 
         // Wait for the "echo" operation to start.
         numCurrentOpRan += waitForCommandToStart("echo", kMaxWait);
@@ -527,19 +524,14 @@ TEST_F(NetworkInterfaceTest, AsyncOpTimeoutWithOpCtxDeadlineSooner) {
     constexpr auto requestTimeout = Milliseconds{1000};
 
     auto serviceContext = ServiceContext::make();
+    serviceContext->registerClientObserver(std::make_unique<LockerNoopClientObserver>());
     auto client = serviceContext->makeClient("NetworkClient");
     auto opCtx = client->makeOperationContext();
-
-    auto stopWatch = serviceContext->getPreciseClockSource()->makeStopWatch();
-    opCtx->setDeadlineByDate(stopWatch.start() + opCtxDeadline, ErrorCodes::ExceededTimeLimit);
+    opCtx->setDeadlineAfterNowBy(opCtxDeadline, ErrorCodes::ExceededTimeLimit);
 
     auto request = makeTestCommand(requestTimeout, makeSleepCmdObj(), opCtx.get());
 
     auto deferred = runCommand(cb, request);
-    // The time returned in result.elapsed is measured from when the command started, which happens
-    // in runCommand. The delay between setting the deadline on opCtx and starting the command can
-    // be long enough that the assertion about opCtxDeadline fails.
-    auto networkStartCommandDelay = stopWatch.elapsed();
 
     waitForIsMaster();
 
@@ -551,12 +543,11 @@ TEST_F(NetworkInterfaceTest, AsyncOpTimeoutWithOpCtxDeadlineSooner) {
         return;
     }
 
-    ASSERT_EQ(ErrorCodes::NetworkInterfaceExceededTimeLimit, result.status);
+    ASSERT_EQ(ErrorCodes::ExceededTimeLimit, result.status);
     ASSERT(result.elapsed);
-
     // check that the request timeout uses the smaller of the operation context deadline and
     // the timeout specified in the request constructor.
-    ASSERT_GTE(result.elapsed.value() + networkStartCommandDelay, opCtxDeadline);
+    ASSERT_GTE(result.elapsed.value(), opCtxDeadline);
     ASSERT_LT(result.elapsed.value(), requestTimeout);
     assertNumOps(0u, 1u, 0u, 0u);
 }
@@ -569,19 +560,13 @@ TEST_F(NetworkInterfaceTest, AsyncOpTimeoutWithOpCtxDeadlineLater) {
     constexpr auto requestTimeout = Milliseconds{600};
 
     auto serviceContext = ServiceContext::make();
+    serviceContext->registerClientObserver(std::make_unique<LockerNoopClientObserver>());
     auto client = serviceContext->makeClient("NetworkClient");
     auto opCtx = client->makeOperationContext();
-
-    auto stopWatch = serviceContext->getPreciseClockSource()->makeStopWatch();
-    opCtx->setDeadlineByDate(stopWatch.start() + opCtxDeadline, ErrorCodes::ExceededTimeLimit);
-
+    opCtx->setDeadlineAfterNowBy(opCtxDeadline, ErrorCodes::ExceededTimeLimit);
     auto request = makeTestCommand(requestTimeout, makeSleepCmdObj(), opCtx.get());
 
     auto deferred = runCommand(cb, request);
-    // The time returned in result.elapsed is measured from when the command started, which happens
-    // in runCommand. The delay between setting the deadline on opCtx and starting the command can
-    // be long enough that the assertion about opCtxDeadline fails.
-    auto networkStartCommandDelay = stopWatch.elapsed();
 
     waitForIsMaster();
 
@@ -595,19 +580,18 @@ TEST_F(NetworkInterfaceTest, AsyncOpTimeoutWithOpCtxDeadlineLater) {
 
     ASSERT_EQ(ErrorCodes::NetworkInterfaceExceededTimeLimit, result.status);
     ASSERT(result.elapsed);
-
     // check that the request timeout uses the smaller of the operation context deadline and
     // the timeout specified in the request constructor.
     ASSERT_GTE(duration_cast<Milliseconds>(result.elapsed.value()), requestTimeout);
-    ASSERT_LT(duration_cast<Milliseconds>(result.elapsed.value() + networkStartCommandDelay),
-              opCtxDeadline);
+    ASSERT_LT(duration_cast<Milliseconds>(result.elapsed.value()), opCtxDeadline);
 
     assertNumOps(0u, 1u, 0u, 0u);
 }
 
 TEST_F(NetworkInterfaceTest, StartCommand) {
-    auto request = makeTestCommand(
-        kNoTimeout, makeEchoCmdObj(), nullptr /* opCtx */, RemoteCommandRequest::HedgeOptions());
+    RemoteCommandRequest::Options options;
+    options.isHedgeEnabled = true;
+    auto request = makeTestCommand(kNoTimeout, makeEchoCmdObj(), nullptr /* opCtx */, options);
 
     auto deferred = runCommand(makeCallbackHandle(), std::move(request));
 
@@ -649,13 +633,12 @@ TEST_F(NetworkInterfaceTest, FireAndForget) {
     const int numFireAndForgetRequests = 3;
     std::vector<Future<RemoteCommandResponse>> futures;
 
+    RemoteCommandRequest::Options options;
+    options.fireAndForget = true;
     for (int i = 0; i < numFireAndForgetRequests; i++) {
         auto cbh = makeCallbackHandle();
-        auto fireAndForgetRequest = makeTestCommand(kNoTimeout,
-                                                    makeEchoCmdObj(),
-                                                    nullptr /* opCtx */,
-                                                    boost::none /* hedgeOptions */,
-                                                    RemoteCommandRequest::FireAndForgetMode::kOn);
+        auto fireAndForgetRequest =
+            makeTestCommand(kNoTimeout, makeEchoCmdObj(), nullptr /* opCtx */, options);
         futures.push_back(runCommand(cbh, fireAndForgetRequest));
     }
 
@@ -686,8 +669,9 @@ TEST_F(NetworkInterfaceInternalClientTest, StartCommandOnAny) {
     auto commandRequest = makeEchoCmdObj();
     auto request = [&] {
         auto cs = fixture();
-        RemoteCommandRequestBase::HedgeOptions ho;
-        ho.count = 1;
+        RemoteCommandRequestBase::Options options;
+        options.isHedgeEnabled = true;
+        options.hedgeCount = 1;
 
         return RemoteCommandRequestOnAny({cs.getServers()},
                                          "admin",
@@ -695,7 +679,7 @@ TEST_F(NetworkInterfaceInternalClientTest, StartCommandOnAny) {
                                          BSONObj(),
                                          nullptr,
                                          RemoteCommandRequest::kNoTimeout,
-                                         ho);
+                                         options);
     }();
 
     auto deferred = runCommandOnAny(makeCallbackHandle(), std::move(request));
@@ -749,14 +733,15 @@ TEST_F(NetworkInterfaceInternalClientTest,
     auto isMasterHandshake = waitForIsMaster();
 
     // Verify that the isMaster reply has the expected internalClient data.
+    auto wireSpec = WireSpec::instance().get();
     auto internalClientElem = isMasterHandshake.request["internalClient"];
     ASSERT_EQ(internalClientElem.type(), BSONType::Object);
     auto minWireVersionElem = internalClientElem.Obj()["minWireVersion"];
     auto maxWireVersionElem = internalClientElem.Obj()["maxWireVersion"];
     ASSERT_EQ(minWireVersionElem.type(), BSONType::NumberInt);
     ASSERT_EQ(maxWireVersionElem.type(), BSONType::NumberInt);
-    ASSERT_EQ(minWireVersionElem.numberInt(), WireSpec::instance().outgoing.minWireVersion);
-    ASSERT_EQ(maxWireVersionElem.numberInt(), WireSpec::instance().outgoing.maxWireVersion);
+    ASSERT_EQ(minWireVersionElem.numberInt(), wireSpec->outgoing.minWireVersion);
+    ASSERT_EQ(maxWireVersionElem.numberInt(), wireSpec->outgoing.maxWireVersion);
 
     // Verify that the ping op is counted as a success.
     auto res = deferred.get();
@@ -765,7 +750,7 @@ TEST_F(NetworkInterfaceInternalClientTest,
 }
 
 TEST_F(NetworkInterfaceTest, IsMasterRequestMissingInternalClientInfoWhenNotInternalClient) {
-    WireSpec::instance().isInternalClient = false;
+    resetIsInternalClient(false);
 
     auto deferred = runCommand(makeCallbackHandle(), makeTestCommand(kNoTimeout, makeEchoCmdObj()));
     auto isMasterHandshake = waitForIsMaster();
@@ -911,6 +896,44 @@ TEST_F(NetworkInterfaceTest, StartExhaustCommandShouldStopOnFailure) {
     }
 }
 
+TEST_F(NetworkInterfaceTest, TearDownWaitsForInProgress) {
+    boost::optional<stdx::thread> tearDownThread;
+    auto tearDownPF = makePromiseFuture<void>();
+
+    auto deferred = [&] {
+        // Enable failpoint to make sure tearDown is blocked
+        FailPointEnableBlock fpb("networkInterfaceFixtureHangOnCompletion");
+
+        auto future = runCommand(makeCallbackHandle(), makeTestCommand(kMaxWait, makeEchoCmdObj()));
+
+        // Wait for the completion of the command
+        fpb->waitForTimesEntered(fpb.initialTimesEntered() + 1);
+
+        tearDownThread.emplace([this, promise = std::move(tearDownPF.promise)]() mutable {
+            tearDown();
+            promise.setWith([] {});
+        });
+
+        // Arbitrary delay between spawning the tearDown thread and checking futures
+        // to increase the chance of failures if tearDown doesn't wait for
+        // in-progress commands.
+        sleepFor(Milliseconds(50));
+
+        ASSERT_EQ(getInProgress(), 1);
+        ASSERT_FALSE(future.isReady()) << "Expected the command to be blocked";
+        ASSERT_FALSE(tearDownPF.future.isReady())
+            << "Expected tearDown to wait for blocked command";
+
+        return future;
+    }();
+
+    tearDownThread->join();
+
+    ASSERT(deferred.isReady());
+    ASSERT(tearDownPF.future.isReady());
+    ASSERT_EQ(getInProgress(), 0);
+}
+
 }  // namespace
 }  // namespace executor
 }  // namespace mongo
