@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
 #include "mongo/platform/basic.h"
 
@@ -43,6 +42,9 @@
 #include "mongo/util/str.h"
 #include "mongo/util/time_support.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
+
+
 namespace mongo {
 namespace executor {
 
@@ -80,10 +82,15 @@ std::string NetworkInterfaceMock::getHostName() {
     return "thisisourhostname";
 }
 
-Status NetworkInterfaceMock::startCommand(const CallbackHandle& cbHandle,
-                                          RemoteCommandRequestOnAny& request,
-                                          RemoteCommandCompletionFn&& onResponse,
-                                          const BatonHandle& baton) {
+/**
+ * Starts a remote command with an implementation common to both the exhaust and non-exhaust
+ * variants.
+ */
+template <typename CallbackFn>
+Status NetworkInterfaceMock::_startCommand(const TaskExecutor::CallbackHandle& cbHandle,
+                                           RemoteCommandRequestOnAny& request,
+                                           CallbackFn&& onResponse,
+                                           const BatonHandle& baton) {
     if (inShutdown()) {
         return {ErrorCodes::ShutdownInProgress, "NetworkInterfaceMock shutdown in progress"};
     }
@@ -109,11 +116,18 @@ Status NetworkInterfaceMock::startCommand(const CallbackHandle& cbHandle,
     return Status::OK();
 }
 
+Status NetworkInterfaceMock::startCommand(const CallbackHandle& cbHandle,
+                                          RemoteCommandRequestOnAny& request,
+                                          RemoteCommandCompletionFn&& onFinish,
+                                          const BatonHandle& baton) {
+    return _startCommand(cbHandle, request, onFinish, baton);
+}
+
 Status NetworkInterfaceMock::startExhaustCommand(const CallbackHandle& cbHandle,
                                                  RemoteCommandRequestOnAny& request,
                                                  RemoteCommandOnReplyFn&& onReply,
                                                  const BatonHandle& baton) {
-    MONGO_UNREACHABLE;
+    return _startCommand(cbHandle, request, onReply, baton);
 }
 
 void NetworkInterfaceMock::setHandshakeReplyForHost(
@@ -143,9 +157,12 @@ void NetworkInterfaceMock::_interruptWithResponse_inlock(const CallbackHandle& c
     auto matchFn = [&cbHandle](const auto& ops) { return ops.isForCallback(cbHandle); };
     auto noi = std::find_if(_operations.begin(), _operations.end(), matchFn);
 
-    // We've effectively observed the NetworkOperation.
-    noi->markAsProcessing();
-    _scheduleResponse_inlock(noi, _now_inlock(), response);
+    if (!noi->isFinished()) {
+        // We've effectively observed the NetworkOperation.
+        noi->markAsProcessing();
+        _counters.canceled++;
+        _scheduleResponse_inlock(noi, _now_inlock(), response);
+    }
 }
 
 Status NetworkInterfaceMock::setAlarm(const TaskExecutor::CallbackHandle& cbHandle,
@@ -587,6 +604,13 @@ void NetworkInterfaceMock::_runReadyNetworkOperations_inlock(stdx::unique_lock<s
               "request"_attr = noi->getRequest(),
               "response"_attr = response.response);
 
+        _counters.sent++;
+        if (response.response.status.isOK()) {
+            _counters.succeeded++;
+        } else {
+            _counters.failed++;
+        }
+
         if (_metadataHook) {
             _metadataHook
                 ->readReplyMetadata(noi->getRequest().opCtx,
@@ -612,6 +636,19 @@ void NetworkInterfaceMock::_runReadyNetworkOperations_inlock(stdx::unique_lock<s
     _waitingToRunMask &= ~kNetworkThread;
 }
 
+bool NetworkInterfaceMock::hasReadyNetworkOperations() {
+    stdx::lock_guard<stdx::mutex> lk(_mutex);
+    invariant(_currentlyRunning == kNetworkThread);
+    if (!_alarms.empty() && _now_inlock() >= _alarms.top().when) {
+        return true;
+    }
+
+    if (!_responses.empty() && _responses.front().when <= _now_inlock()) {
+        return true;
+    }
+    return false;
+}
+
 void NetworkInterfaceMock::_waitForWork_inlock(stdx::unique_lock<stdx::mutex>* lk) {
     if (_waitingToRunMask & kExecutorThread) {
         _waitingToRunMask &= ~kExecutorThread;
@@ -644,9 +681,6 @@ bool NetworkInterfaceMock::_isExecutorThreadRunnable_inlock() {
     return _waitingToRunMask & kExecutorThread;
 }
 
-static const ResponseStatus kUnsetResponse(ErrorCodes::InternalError,
-                                           "NetworkOperation::_response never set");
-
 NetworkInterfaceMock::NetworkOperation::NetworkOperation()
     : _requestDate(), _request(), _onResponse() {}
 
@@ -654,7 +688,7 @@ NetworkInterfaceMock::NetworkOperation::NetworkOperation(
     const CallbackHandle& cbHandle,
     const RemoteCommandRequestOnAny& theRequest,
     Date_t theRequestDate,
-    ResponseCallback onResponse)
+    RemoteCommandCompletionFn onResponse)
     : _requestDate(theRequestDate),
       _cbHandle(cbHandle),
       _requestOnAny(theRequest),
