@@ -105,7 +105,7 @@ public:
     std::string getDiagnosticString() override;
 
     Counters getCounters() const override {
-        return Counters();
+        return _counters;
     }
 
     void startup() override;
@@ -146,6 +146,8 @@ public:
 
     void dropConnections(const HostAndPort&) override {}
 
+    void testEgress(const HostAndPort&, transport::ConnectSSLMode, Milliseconds, Status) override {}
+
     ////////////////////////////////////////////////////////////////////////////////
     //
     // Methods for simulating network operations and the passage of time.
@@ -237,7 +239,7 @@ public:
      * "when" defaults to now().
      */
     RemoteCommandRequest scheduleErrorResponse(const Status& response);
-    RemoteCommandRequest scheduleErrorResponse(const TaskExecutor::ResponseStatus response);
+    RemoteCommandRequest scheduleErrorResponse(TaskExecutor::ResponseStatus response);
     RemoteCommandRequest scheduleErrorResponse(NetworkOperationIterator noi,
                                                const Status& response);
     RemoteCommandRequest scheduleErrorResponse(NetworkOperationIterator noi,
@@ -291,6 +293,12 @@ public:
     void _interruptWithResponse_inlock(const TaskExecutor::CallbackHandle& cbHandle,
                                        const TaskExecutor::ResponseStatus& response);
 
+    /**
+     * Returns true if there is no scheduled work (i.e. alarms and scheduled responses) for the
+     * network thread to process.
+     */
+    bool hasReadyNetworkOperations();
+
 private:
     /**
      * Information describing a scheduled alarm.
@@ -373,6 +381,12 @@ private:
      */
     void _runReadyNetworkOperations_inlock(stdx::unique_lock<stdx::mutex>* lk);
 
+    template <typename CallbackFn>
+    Status _startCommand(const TaskExecutor::CallbackHandle& cbHandle,
+                         RemoteCommandRequestOnAny& request,
+                         CallbackFn&& onReply,
+                         const BatonHandle& baton = nullptr);
+
     // Mutex that synchronizes access to mutable data in this class and its subclasses.
     // Fields guarded by the mutex are labled (M), below, and those that are read-only
     // in multi-threaded execution, and so unsynchronized, are labeled (R).
@@ -407,7 +421,7 @@ private:
     // NetworkInterfaceMock.
     NetworkOperationList _operations;  // (M)
 
-    // The list of responses that have been enqueued from scheduleResponse(), cancelation, or
+    // The list of responses that have been enqueued from scheduleResponse(), cancellation, or
     // timeout. This list is ordered by NetworkResponse::when and is drained front to back by
     // runReadyNetworkOperations().
     NetworkResponseList _responses;  // (M)
@@ -432,6 +446,12 @@ private:
 
     // The handshake replies set for each host.
     stdx::unordered_map<HostAndPort, RemoteCommandResponse> _handshakeReplies;  // (M)
+
+    // Track statistics about processed responses. Right now, the mock tracks the total responses
+    // processed with sent, the number of OK responses with succeeded, non-OK responses with failed,
+    // and cancellation errors with canceled. It does not track the timedOut or failedRemotely
+    // statistics.
+    Counters _counters;
 };
 
 /**
@@ -504,6 +524,10 @@ public:
         return !_isProcessing && !_isFinished;
     }
 
+    bool isFinished() const {
+        return _isFinished;
+    }
+
     /**
      * Assert that this operation has not been blackholed.
      */
