@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT mongo::logv2::LogComponent::kExecutor
 
 #include "mongo/platform/basic.h"
 
@@ -49,6 +48,9 @@
 #include "mongo/util/fail_point.h"
 #include "mongo/util/time_support.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT mongo::logv2::LogComponent::kExecutor
+
+
 namespace mongo {
 namespace executor {
 
@@ -241,22 +243,29 @@ stdx::unique_lock<Latch> ThreadPoolTaskExecutor::_join(stdx::unique_lock<Latch>
     return lk;
 }
 
+bool ThreadPoolTaskExecutor::isShuttingDown() const {
+    stdx::lock_guard lk(_mutex);
+    return _inShutdown_inlock();
+}
+
 void ThreadPoolTaskExecutor::appendDiagnosticBSON(BSONObjBuilder* b) const {
     stdx::lock_guard<Latch> lk(_mutex);
 
     // ThreadPool details
     // TODO: fill in
     BSONObjBuilder poolCounters(b->subobjStart("pool"));
-    poolCounters.appendIntOrLL("inProgressCount", _poolInProgressQueue.size());
+    poolCounters.appendNumber("inProgressCount",
+                              static_cast<long long>(_poolInProgressQueue.size()));
     poolCounters.done();
 
     // Queues
     BSONObjBuilder queues(b->subobjStart("queues"));
-    queues.appendIntOrLL("networkInProgress", _networkInProgressQueue.size());
-    queues.appendIntOrLL("sleepers", _sleepersQueue.size());
+    queues.appendNumber("networkInProgress",
+                        static_cast<long long>(_networkInProgressQueue.size()));
+    queues.appendNumber("sleepers", static_cast<long long>(_sleepersQueue.size()));
     queues.done();
 
-    b->appendIntOrLL("unsignaledEvents", _unsignaledEvents.size());
+    b->appendNumber("unsignaledEvents", static_cast<long long>(_unsignaledEvents.size()));
     b->append("shuttingDown", _inShutdown_inlock());
     b->append("networkInterface", _net->getDiagnosticString());
 }
@@ -421,11 +430,7 @@ StatusWith<TaskExecutor::CallbackHandle> ThreadPoolTaskExecutor::scheduleRemoteC
     const BatonHandle& baton) {
 
     RemoteCommandRequestOnAny scheduledRequest = request;
-    if (request.timeout == RemoteCommandRequest::kNoTimeout) {
-        scheduledRequest.expirationDate = RemoteCommandRequest::kNoExpirationDate;
-    } else {
-        scheduledRequest.expirationDate = _net->now() + scheduledRequest.timeout;
-    }
+    scheduledRequest.dateScheduled = _net->now();
 
     // In case the request fails to even get a connection from the pool,
     // we wrap the callback in a method that prepares its input parameters.
@@ -564,8 +569,10 @@ void ThreadPoolTaskExecutor::signalEvent_inlock(const EventHandle& event,
                                                 stdx::unique_lock<Latch> lk) {
     invariant(event.isValid());
     auto eventState = checked_cast<EventState*>(getEventFromHandle(event));
-    invariant(!eventState->isSignaledFlag);
-    eventState->isSignaledFlag = true;
+    const auto wasSignaled = std::exchange(eventState->isSignaledFlag, true);
+    if (MONGO_unlikely(wasSignaled && _inShutdown_inlock()))
+        return;
+    invariant(!wasSignaled);
     eventState->isSignaledCondition.notify_all();
     _unsignaledEvents.erase(eventState->iter);
     scheduleIntoPool_inlock(&eventState->waiters, std::move(lk));
@@ -614,14 +621,14 @@ void ThreadPoolTaskExecutor::scheduleIntoPool_inlock(WorkQueue* fromQueue,
                 }
 
                 _pool->schedule([this, cbState](auto status) {
-                    invariant(status.isOK() || ErrorCodes::isCancelationError(status.code()));
+                    invariant(status.isOK() || ErrorCodes::isCancellationError(status.code()));
 
                     runCallback(std::move(cbState));
                 });
             });
         } else {
             _pool->schedule([this, cbState](auto status) {
-                if (ErrorCodes::isCancelationError(status.code())) {
+                if (ErrorCodes::isCancellationError(status.code())) {
                     stdx::lock_guard<Latch> lk(_mutex);
 
                     cbState->canceled.store(1);
@@ -641,9 +648,7 @@ void ThreadPoolTaskExecutor::runCallback(std::shared_ptr<CallbackState> cbStateA
     setCallbackForHandle(&cbHandle, cbStateArg);
     CallbackArgs args(this,
                       std::move(cbHandle),
-                      cbStateArg->canceled.load()
-                          ? Status({ErrorCodes::CallbackCanceled, "Callback canceled"})
-                          : Status::OK());
+                      cbStateArg->canceled.load() ? kCallbackCanceledErrorStatus : Status::OK());
     invariant(!cbStateArg->isFinished.load());
     {
         // After running callback function, clear 'cbStateArg->callback' to release any resources
@@ -673,11 +678,7 @@ StatusWith<TaskExecutor::CallbackHandle> ThreadPoolTaskExecutor::scheduleExhaust
     const RemoteCommandOnAnyCallbackFn& cb,
     const BatonHandle& baton) {
     RemoteCommandRequestOnAny scheduledRequest = request;
-    if (request.timeout == RemoteCommandRequest::kNoTimeout) {
-        scheduledRequest.expirationDate = RemoteCommandRequest::kNoExpirationDate;
-    } else {
-        scheduledRequest.expirationDate = _net->now() + scheduledRequest.timeout;
-    }
+    scheduledRequest.dateScheduled = _net->now();
 
     // In case the request fails to even get a connection from the pool,
     // we wrap the callback in a method that prepares its input parameters.
@@ -724,7 +725,7 @@ StatusWith<TaskExecutor::CallbackHandle> ThreadPoolTaskExecutor::scheduleExhaust
             stdx::unique_lock<Latch> lk(_mutex);
             if (_inShutdown_inlock() || cbState->exhaustErased.load()) {
                 if (cbState->exhaustIter) {
-                    _poolInProgressQueue.erase(cbState->exhaustIter.get());
+                    _poolInProgressQueue.erase(cbState->exhaustIter.value());
                     cbState->exhaustIter = boost::none;
                 }
                 return;
@@ -739,7 +740,7 @@ StatusWith<TaskExecutor::CallbackHandle> ThreadPoolTaskExecutor::scheduleExhaust
                 cbState->exhaustErased.store(1);
 
                 if (cbState->exhaustIter) {
-                    _poolInProgressQueue.erase(cbState->exhaustIter.get());
+                    _poolInProgressQueue.erase(cbState->exhaustIter.value());
                     cbState->exhaustIter = boost::none;
                 }
 
@@ -780,7 +781,7 @@ void ThreadPoolTaskExecutor::scheduleExhaustIntoPool_inlock(std::shared_ptr<Call
                                                             stdx::unique_lock<Latch> lk) {
     _poolInProgressQueue.push_back(cbState);
     cbState->exhaustIter = --_poolInProgressQueue.end();
-    auto expectedExhaustIter = cbState->exhaustIter.get();
+    auto expectedExhaustIter = cbState->exhaustIter.value();
     lk.unlock();
 
     if (cbState->baton) {
@@ -796,14 +797,14 @@ void ThreadPoolTaskExecutor::scheduleExhaustIntoPool_inlock(std::shared_ptr<Call
             }
 
             _pool->schedule([this, cbState, expectedExhaustIter](auto status) {
-                invariant(status.isOK() || ErrorCodes::isCancelationError(status.code()));
+                invariant(status.isOK() || ErrorCodes::isCancellationError(status.code()));
 
                 runCallbackExhaust(cbState, expectedExhaustIter);
             });
         });
     } else {
         _pool->schedule([this, cbState, expectedExhaustIter](auto status) {
-            if (ErrorCodes::isCancelationError(status.code())) {
+            if (ErrorCodes::isCancellationError(status.code())) {
                 stdx::lock_guard<Latch> lk(_mutex);
 
                 cbState->canceled.store(1);
@@ -824,9 +825,7 @@ void ThreadPoolTaskExecutor::runCallbackExhaust(std::shared_ptr<CallbackState> c
     setCallbackForHandle(&cbHandle, cbState);
     CallbackArgs args(this,
                       std::move(cbHandle),
-                      cbState->canceled.load()
-                          ? Status({ErrorCodes::CallbackCanceled, "Callback canceled"})
-                          : Status::OK());
+                      cbState->canceled.load() ? kCallbackCanceledErrorStatus : Status::OK());
 
     if (!cbState->isFinished.load()) {
         TaskExecutor::CallbackFn callback = [](const CallbackArgs&) {};
@@ -854,7 +853,7 @@ void ThreadPoolTaskExecutor::runCallbackExhaust(std::shared_ptr<CallbackState> c
     // 'expectedExhaustIter' so that we can still remove this task from the 'poolInProgressQueue' if
     // this happens, but we do not want to reset the 'exhaustIter' value in this case.
     if (cbState->exhaustIter) {
-        if (cbState->exhaustIter.get() == expectedExhaustIter) {
+        if (cbState->exhaustIter.value() == expectedExhaustIter) {
             cbState->exhaustIter = boost::none;
         }
         _poolInProgressQueue.erase(expectedExhaustIter);
