@@ -32,15 +32,16 @@
 #include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/logv2/constants.h"
+#include "mongo/stdx/type_traits.h"
 #include "mongo/stdx/variant.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/duration.h"
 
 #include <boost/container/small_vector.hpp>
 #include <functional>
 #include <string_view>
 
-namespace mongo {
-namespace logv2 {
+namespace mongo::logv2 {
 
 class TypeErasedAttributeStorage;
 
@@ -70,29 +71,28 @@ auto mapLog(It begin, It end);
 
 namespace detail {
 
+template <template <class...> class Template, typename... Args>
+struct IsInstantiationOf : std::false_type {};
+template <template <class...> class Template, typename... Args>
+struct IsInstantiationOf<Template, Template<Args...>> : std::true_type {};
+template <template <class...> class Template, typename... Args>
+constexpr bool isInstantiationOf = IsInstantiationOf<Template, Args...>::value;
+
 // Helper traits to figure out capabilities on custom types
 template <class T>
-struct IsOptional : std::false_type {};
+constexpr bool isOptional = isInstantiationOf<boost::optional, T>;
 
 template <class T>
-struct IsOptional<boost::optional<T>> : std::true_type {};
-
-template <class T>
-struct IsDuration : std::false_type {};
-
-template <class T>
-struct IsDuration<Duration<T>> : std::true_type {};
-
-template <class T, typename = void>
-struct IsContainer : std::false_type {};
-
-template <class T, typename = void>
-struct HasMappedType : std::false_type {};
+constexpr bool isDuration = isInstantiationOf<Duration, T>;
 
 template <typename T>
-struct HasMappedType<T, std::void_t<typename T::mapped_type>> : std::true_type {};
+using HasMappedTypeOp = typename T::mapped_type;
+template <typename T>
+constexpr bool hasMappedType = stdx::is_detected_v<HasMappedTypeOp, T>;
 
 // Trait to detect container, common interface for both std::array and std::forward_list
+template <class T, typename = void>
+struct IsContainer : std::false_type {};
 template <typename T>
 struct IsContainer<T,
                    std::void_t<typename T::value_type,
@@ -102,80 +102,57 @@ struct IsContainer<T,
                                decltype(std::declval<T>().empty()),
                                decltype(std::declval<T>().begin()),
                                decltype(std::declval<T>().end())>> : std::true_type {};
+template <typename T>
+constexpr bool isContainer = IsContainer<T>::value;
 
-template <class T, class = void>
-struct HasToBSON : std::false_type {};
+template <typename T>
+using HasToBSONOp = decltype(std::declval<T>().toBSON());
+template <typename T>
+constexpr bool hasToBSON = stdx::is_detected_v<HasToBSONOp, T>;
 
-template <class T>
-struct HasToBSON<T, std::void_t<decltype(std::declval<T>().toBSON())>> : std::true_type {};
+template <typename T>
+using HasToBSONArrayOp = decltype(std::declval<T>().toBSONArray());
+template <typename T>
+constexpr bool hasToBSONArray = stdx::is_detected_v<HasToBSONArrayOp, T>;
 
-template <class T, class = void>
-struct HasToBSONArray : std::false_type {};
+template <typename T>
+using HasBSONSerializeOp = decltype(std::declval<T>().serialize(std::declval<BSONObjBuilder*>()));
+template <typename T>
+constexpr bool hasBSONSerialize = stdx::is_detected_v<HasBSONSerializeOp, T>;
 
-template <class T>
-struct HasToBSONArray<T, std::void_t<decltype(std::declval<T>().toBSONArray())>> : std::true_type {
-};
+template <typename T>
+using HasBSONBuilderAppendOp =
+    decltype(std::declval<BSONObjBuilder>().append(std::declval<StringData>(), std::declval<T>()));
+template <typename T>
+constexpr bool hasBSONBuilderAppend = stdx::is_detected_v<HasBSONBuilderAppendOp, T>;
 
-template <class T, class = void>
-struct HasBSONSerialize : std::false_type {};
+template <typename T>
+using HasStringSerializeOp =
+    decltype(std::declval<T>().serialize(std::declval<fmt::memory_buffer&>()));
+template <typename T>
+constexpr bool hasStringSerialize = stdx::is_detected_v<HasStringSerializeOp, T>;
 
-template <class T>
-struct HasBSONSerialize<
-    T,
-    std::void_t<decltype(std::declval<T>().serialize(std::declval<BSONObjBuilder*>()))>>
-    : std::true_type {};
+template <typename T>
+using HasToStringOp = std::remove_cv_t<decltype(std::declval<T>().toString())>;
+template <typename T>
+constexpr bool hasToString = stdx::is_detected_exact_v<std::string, HasToStringOp, T>;
+template <typename T>
+constexpr bool hasToStringReturnStringData =
+    stdx::is_detected_convertible_v<StringData, HasToStringOp, T>;
 
-template <class T, class = void>
-struct HasBSONBuilderAppend : std::false_type {};
+template <typename T>
+using HasNonMemberToStringOp = decltype(toString(std::declval<T>()));
+template <typename T>
+constexpr bool hasNonMemberToString =
+    stdx::is_detected_exact_v<std::string, HasNonMemberToStringOp, T>;
+template <typename T>
+constexpr bool hasNonMemberToStringReturnStringData =
+    stdx::is_detected_convertible_v<StringData, HasNonMemberToStringOp, T>;
 
-template <class T>
-struct HasBSONBuilderAppend<T,
-                            std::void_t<decltype(std::declval<BSONObjBuilder>().append(
-                                std::declval<StringData>(), std::declval<T>()))>> : std::true_type {
-};
-
-template <class T, class = void>
-struct HasStringSerialize : std::false_type {};
-
-template <class T>
-struct HasStringSerialize<
-    T,
-    std::void_t<decltype(std::declval<T>().serialize(std::declval<fmt::memory_buffer&>()))>>
-    : std::true_type {};
-
-template <class T, class = void>
-struct HasToString : std::false_type {};
-
-template <class T>
-struct HasToString<T, std::void_t<decltype(std::declval<T>().toString())>>
-    : std::is_same<std::remove_cv_t<decltype(std::declval<T>().toString())>, std::string> {};
-
-template <class T, class = void>
-struct HasToStringReturnStringData : std::false_type {};
-
-template <class T>
-struct HasToStringReturnStringData<T, std::void_t<decltype(std::declval<T>().toString())>>
-    : std::is_convertible<decltype(std::declval<T>().toString()), StringData> {};
-
-template <class T, class = void>
-struct HasNonMemberToString : std::false_type {};
-
-template <class T>
-struct HasNonMemberToString<T, std::void_t<decltype(toString(std::declval<T>()))>>
-    : std::is_same<decltype(toString(std::declval<T>())), std::string> {};
-
-template <class T, class = void>
-struct HasNonMemberToStringReturnStringData : std::false_type {};
-
-template <class T>
-struct HasNonMemberToStringReturnStringData<T, std::void_t<decltype(toString(std::declval<T>()))>>
-    : std::is_convertible<decltype(toString(std::declval<T>())), StringData> {};
-
-template <class T, class = void>
-struct HasNonMemberToBSON : std::false_type {};
-
-template <class T>
-struct HasNonMemberToBSON<T, std::void_t<decltype(toBSON(std::declval<T>()))>> : std::true_type {};
+template <typename T>
+using HasNonMemberToBSONOp = decltype(toBSON(std::declval<T>()));
+template <typename T>
+constexpr bool hasNonMemberToBSON = stdx::is_detected_v<HasNonMemberToBSONOp, T>;
 
 // Mapping functions on how to map a logged value to how it is stored in variant (reused by
 // container support)
@@ -223,10 +200,10 @@ inline StringData mapValue(const char* value) {
     return value;
 }
 
-inline const BSONObj mapValue(BSONObj const& value) {
+inline BSONObj mapValue(BSONObj const& value) {
     return value;
 }
-inline const BSONArray mapValue(BSONArray const& value) {
+inline BSONArray mapValue(BSONArray const& value) {
     return value;
 }
 inline CustomAttributeValue mapValue(BSONElement const& val) {
@@ -246,18 +223,18 @@ inline CustomAttributeValue mapValue(boost::none_t val) {
     return custom;
 }
 
-template <typename T, std::enable_if_t<IsDuration<T>::value, int> = 0>
+template <typename T, std::enable_if_t<isDuration<T>, int> = 0>
 auto mapValue(T val) {
     return val;
 }
 
 template <typename T, std::enable_if_t<std::is_enum_v<T>, int> = 0>
 auto mapValue(T val) {
-    if constexpr (HasNonMemberToString<T>::value) {
+    if constexpr (hasNonMemberToString<T>) {
         CustomAttributeValue custom;
         custom.toString = [val]() { return toString(val); };
         return custom;
-    } else if constexpr (HasNonMemberToStringReturnStringData<T>::value) {
+    } else if constexpr (hasNonMemberToStringReturnStringData<T>) {
         CustomAttributeValue custom;
         custom.stringSerialize = [val](fmt::memory_buffer& buffer) {
             StringData sd = toString(val);
@@ -269,7 +246,7 @@ auto mapValue(T val) {
     }
 }
 
-template <typename T, std::enable_if_t<IsContainer<T>::value && !HasMappedType<T>::value, int> = 0>
+template <typename T, std::enable_if_t<isContainer<T> && !hasMappedType<T>, int> = 0>
 CustomAttributeValue mapValue(const T& val) {
     CustomAttributeValue custom;
     custom.toBSONArray = [&val]() { return seqLog(val).toBSONArray(); };
@@ -277,7 +254,7 @@ CustomAttributeValue mapValue(const T& val) {
     return custom;
 }
 
-template <typename T, std::enable_if_t<IsContainer<T>::value && HasMappedType<T>::value, int> = 0>
+template <typename T, std::enable_if_t<isContainer<T> && hasMappedType<T>, int> = 0>
 CustomAttributeValue mapValue(const T& val) {
     CustomAttributeValue custom;
     custom.BSONSerialize = [&val](BSONObjBuilder& builder) { mapLog(val).serialize(&builder); };
@@ -285,54 +262,51 @@ CustomAttributeValue mapValue(const T& val) {
     return custom;
 }
 
-template <
-    typename T,
-    std::enable_if_t<!std::is_integral_v<T> && !std::is_floating_point_v<T> && !std::is_enum_v<T> &&
-                         !IsDuration<T>::value && !IsContainer<T>::value,
-                     int> = 0>
+template <typename T,
+          std::enable_if_t<!std::is_integral_v<T> && !std::is_floating_point_v<T> &&
+                               !std::is_enum_v<T> && !isDuration<T> && !isContainer<T>,
+                           int> = 0>
 CustomAttributeValue mapValue(const T& val) {
-    static_assert(HasToString<T>::value || HasToStringReturnStringData<T>::value ||
-                      HasStringSerialize<T>::value || HasNonMemberToString<T>::value ||
-                      HasNonMemberToStringReturnStringData<T>::value ||
-                      HasBSONBuilderAppend<T>::value || HasBSONSerialize<T>::value ||
-                      HasToBSON<T>::value || HasToBSONArray<T>::value ||
-                      HasNonMemberToBSON<T>::value,
+    static_assert(hasToString<T> || hasToStringReturnStringData<T> || hasStringSerialize<T> ||
+                      hasNonMemberToString<T> || hasNonMemberToStringReturnStringData<T> ||
+                      hasBSONBuilderAppend<T> || hasBSONSerialize<T> || hasToBSON<T> ||
+                      hasToBSONArray<T> || hasNonMemberToBSON<T>,
                   "custom type needs toBSON(), toBSONArray(), serialize(BSONObjBuilder*), "
                   "toString() or serialize(fmt::memory_buffer&) implementation");
 
     CustomAttributeValue custom;
-    if constexpr (HasBSONBuilderAppend<T>::value) {
+    if constexpr (hasBSONBuilderAppend<T>) {
         custom.BSONAppend = [&val](BSONObjBuilder& builder, StringData fieldName) {
             builder.append(fieldName, val);
         };
     }
 
-    if constexpr (HasBSONSerialize<T>::value) {
+    if constexpr (hasBSONSerialize<T>) {
         custom.BSONSerialize = [&val](BSONObjBuilder& builder) { val.serialize(&builder); };
-    } else if constexpr (HasToBSON<T>::value) {
+    } else if constexpr (hasToBSON<T>) {
         custom.BSONSerialize = [&val](BSONObjBuilder& builder) {
             builder.appendElements(val.toBSON());
         };
-    } else if constexpr (HasToBSONArray<T>::value) {
+    } else if constexpr (hasToBSONArray<T>) {
         custom.toBSONArray = [&val]() { return val.toBSONArray(); };
-    } else if constexpr (HasNonMemberToBSON<T>::value) {
+    } else if constexpr (hasNonMemberToBSON<T>) {
         custom.BSONSerialize = [&val](BSONObjBuilder& builder) {
             builder.appendElements(toBSON(val));
         };
     }
 
-    if constexpr (HasStringSerialize<T>::value) {
+    if constexpr (hasStringSerialize<T>) {
         custom.stringSerialize = [&val](fmt::memory_buffer& buffer) { val.serialize(buffer); };
-    } else if constexpr (HasToString<T>::value) {
+    } else if constexpr (hasToString<T>) {
         custom.toString = [&val]() { return val.toString(); };
-    } else if constexpr (HasToStringReturnStringData<T>::value) {
+    } else if constexpr (hasToStringReturnStringData<T>) {
         custom.stringSerialize = [&val](fmt::memory_buffer& buffer) {
             StringData sd = val.toString();
             buffer.append(sd.begin(), sd.end());
         };
-    } else if constexpr (HasNonMemberToString<T>::value) {
+    } else if constexpr (hasNonMemberToString<T>) {
         custom.toString = [&val]() { return toString(val); };
-    } else if constexpr (HasNonMemberToStringReturnStringData<T>::value) {
+    } else if constexpr (hasNonMemberToStringReturnStringData<T>) {
         custom.stringSerialize = [&val](fmt::memory_buffer& buffer) {
             StringData sd = toString(val);
             buffer.append(sd.begin(), sd.end());
@@ -372,7 +346,7 @@ public:
                     } else {
                         builder.append(val.toString());
                     }
-                } else if constexpr (IsDuration<std::decay_t<V>>::value) {
+                } else if constexpr (isDuration<std::decay_t<V>>) {
                     builder.append(val.toBSON());
                 } else if constexpr (std::is_same_v<std::decay_t<V>, unsigned int>) {
                     builder.append(static_cast<long long>(val));
@@ -382,7 +356,7 @@ public:
             };
 
             using item_t = std::decay_t<decltype(item)>;
-            if constexpr (IsOptional<item_t>::value) {
+            if constexpr (isOptional<item_t>) {
                 if (item) {
                     append(mapValue(*item));
                 } else {
@@ -407,11 +381,24 @@ public:
                 if constexpr (std::is_same_v<decltype(val), CustomAttributeValue&&>) {
                     if (val.stringSerialize) {
                         val.stringSerialize(buffer);
-                    } else {
+                    } else if (val.toString) {
                         fmt::format_to(buffer, "{}", val.toString());
+                    } else if (val.BSONSerialize) {
+                        BSONObjBuilder objBuilder;
+                        val.BSONSerialize(objBuilder);
+                        objBuilder.done().jsonStringBuffer(
+                            JsonStringFormat::ExtendedRelaxedV2_0_0, 0, false, buffer);
+                    } else if (val.BSONAppend) {
+                        BSONObjBuilder objBuilder;
+                        val.BSONAppend(objBuilder, ""_sd);
+                        objBuilder.done().getField(""_sd).jsonStringBuffer(
+                            JsonStringFormat::ExtendedRelaxedV2_0_0, false, false, 0, buffer);
+                    } else {
+                        val.toBSONArray().jsonStringBuffer(
+                            JsonStringFormat::ExtendedRelaxedV2_0_0, 0, true, buffer);
                     }
 
-                } else if constexpr (IsDuration<std::decay_t<decltype(val)>>::value) {
+                } else if constexpr (isDuration<std::decay_t<decltype(val)>>) {
                     fmt::format_to(buffer, "{}", val.toString());
                 } else if constexpr (std::is_same_v<std::decay_t<decltype(val)>, BSONObj>) {
                     val.jsonStringBuffer(JsonStringFormat::ExtendedRelaxedV2_0_0, 0, false, buffer);
@@ -423,7 +410,7 @@ public:
             };
 
             using item_t = std::decay_t<decltype(item)>;
-            if constexpr (IsOptional<item_t>::value) {
+            if constexpr (isOptional<item_t>) {
                 if (item) {
                     append(mapValue(*item));
                 } else {
@@ -473,7 +460,7 @@ public:
                     } else {
                         builder->append(key, val.toString());
                     }
-                } else if constexpr (IsDuration<std::decay_t<V>>::value) {
+                } else if constexpr (isDuration<std::decay_t<V>>) {
                     builder->append(key, val.toBSON());
                 } else if constexpr (std::is_same_v<std::decay_t<V>, unsigned int>) {
                     builder->append(key, static_cast<long long>(val));
@@ -483,7 +470,7 @@ public:
             };
             auto key = mapValue(item.first);
             using value_t = std::decay_t<decltype(item.second)>;
-            if constexpr (IsOptional<value_t>::value) {
+            if constexpr (isOptional<value_t>) {
                 if (item.second) {
                     append(key, mapValue(*item.second));
                 } else {
@@ -508,10 +495,26 @@ public:
                     if (val.stringSerialize) {
                         fmt::format_to(buffer, "{}: ", key);
                         val.stringSerialize(buffer);
-                    } else {
+                    } else if (val.toString) {
                         fmt::format_to(buffer, "{}: {}", key, val.toString());
+                    } else if (val.BSONSerialize) {
+                        BSONObjBuilder objBuilder;
+                        val.BSONSerialize(objBuilder);
+                        fmt::format_to(buffer, "{}: ", key);
+                        objBuilder.done().jsonStringBuffer(
+                            JsonStringFormat::ExtendedRelaxedV2_0_0, 0, false, buffer);
+                    } else if (val.BSONAppend) {
+                        BSONObjBuilder objBuilder;
+                        val.BSONAppend(objBuilder, ""_sd);
+                        fmt::format_to(buffer, "{}: ", key);
+                        objBuilder.done().getField(""_sd).jsonStringBuffer(
+                            JsonStringFormat::ExtendedRelaxedV2_0_0, false, false, 0, buffer);
+                    } else {
+                        fmt::format_to(buffer, "{}: ", key);
+                        val.toBSONArray().jsonStringBuffer(
+                            JsonStringFormat::ExtendedRelaxedV2_0_0, 0, true, buffer);
                     }
-                } else if constexpr (IsDuration<std::decay_t<decltype(val)>>::value) {
+                } else if constexpr (isDuration<std::decay_t<decltype(val)>>) {
                     fmt::format_to(buffer, "{}: {}", key, val.toString());
                 } else if constexpr (std::is_same_v<std::decay_t<decltype(val)>, BSONObj>) {
                     fmt::format_to(buffer, "{}: ", key);
@@ -526,7 +529,7 @@ public:
 
             auto key = mapValue(item.first);
             using value_t = std::decay_t<decltype(item.second)>;
-            if constexpr (IsOptional<value_t>::value) {
+            if constexpr (isOptional<value_t>) {
                 if (item.second) {
                     append(key, mapValue(*item.second));
                 } else {
@@ -550,10 +553,10 @@ private:
 class NamedAttribute {
 public:
     NamedAttribute() = default;
-    NamedAttribute(StringData n, long double val) = delete;
+    NamedAttribute(const char* n, long double val) = delete;
 
     template <typename T>
-    NamedAttribute(StringData n, const boost::optional<T>& val)
+    NamedAttribute(const char* n, const boost::optional<T>& val)
         : NamedAttribute(val ? NamedAttribute(n, *val) : NamedAttribute()) {
         if (!val) {
             name = n;
@@ -562,9 +565,9 @@ public:
     }
 
     template <typename T>
-    NamedAttribute(StringData n, const T& val) : name(n), value(mapValue(val)) {}
+    NamedAttribute(const char* n, const T& val) : name(n), value(mapValue(val)) {}
 
-    StringData name;
+    const char* name = nullptr;
     stdx::variant<int,
                   unsigned int,
                   long long,
@@ -590,8 +593,7 @@ template <typename... Args>
 class AttributeStorage {
 public:
     AttributeStorage(const Args&... args)
-        : _data{detail::NamedAttribute(StringData(args.name.data(), args.name.size()),
-                                       args.value)...} {}
+        : _data{detail::NamedAttribute(args.name, args.value)...} {}
 
 private:
     static const size_t kNumArgs = sizeof...(Args);
@@ -617,39 +619,39 @@ public:
     template <size_t N,
               typename T,
               std::enable_if_t<std::is_arithmetic_v<T> || std::is_pointer_v<T> ||
-                                   std::is_enum_v<T> || detail::IsDuration<T>::value,
+                                   std::is_enum_v<T> || detail::isDuration<T>,
                                int> = 0>
     void add(const char (&name)[N], T value) {
-        _attributes.emplace_back(StringData(name, N - 1), value);
+        _attributes.emplace_back(name, value);
     }
 
     template <size_t N>
     void add(const char (&name)[N], BSONObj value) {
         BSONObj owned = value.getOwned();
-        _attributes.emplace_back(StringData(name, N - 1), owned);
+        _attributes.emplace_back(name, owned);
     }
 
     template <size_t N>
     void add(const char (&name)[N], BSONArray value) {
         BSONArray owned = static_cast<BSONArray>(value.getOwned());
-        _attributes.emplace_back(StringData(name, N - 1), owned);
+        _attributes.emplace_back(name, owned);
     }
 
     template <size_t N,
               typename T,
-              std::enable_if_t<std::is_class_v<T> && !detail::IsDuration<T>::value, int> = 0>
+              std::enable_if_t<std::is_class_v<T> && !detail::isDuration<T>, int> = 0>
     void add(const char (&name)[N], const T& value) {
-        _attributes.emplace_back(StringData(name, N - 1), value);
+        _attributes.emplace_back(name, value);
     }
 
     template <size_t N,
               typename T,
-              std::enable_if_t<std::is_class_v<T> && !detail::IsDuration<T>::value, int> = 0>
+              std::enable_if_t<std::is_class_v<T> && !detail::isDuration<T>, int> = 0>
     void add(const char (&name)[N], T&& value) = delete;
 
     template <size_t N>
     void add(const char (&name)[N], StringData value) {
-        _attributes.emplace_back(StringData(name, N - 1), value);
+        _attributes.emplace_back(name, value);
     }
 
     // Deep copies the string instead of taking it by reference
@@ -662,7 +664,7 @@ public:
     // Does not have the protections of add() above. Be careful about lifetime of value!
     template <size_t N, typename T>
     void addUnsafe(const char (&name)[N], const T& value) {
-        _attributes.emplace_back(StringData(name, N - 1), value);
+        _attributes.emplace_back(name, value);
     }
 
 private:
@@ -708,9 +710,8 @@ public:
     // Applies a function to every stored named attribute in order they are captured
     template <typename Func>
     void apply(Func&& f) const {
-        std::for_each(_data, _data + _size, [&f](const detail::NamedAttribute& attr) {
-            StringData name = attr.name;
-            stdx::visit([name, &f](auto&& val) { f(name, val); }, attr.value);
+        std::for_each(_data, _data + _size, [&](const auto& attr) {
+            stdx::visit([&](auto&& val) { f(attr.name, val); }, attr.value);
         });
     }
 
@@ -744,5 +745,4 @@ auto mapLog(It begin, It end) {
     return detail::AssociativeContainerLogger(begin, end);
 }
 
-}  // namespace logv2
-}  // namespace mongo
+}  // namespace mongo::logv2
