@@ -27,11 +27,11 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
 #include <fstream>
+#include <signal.h>
 #include <sstream>
 #include <string>
 #include <vector>
@@ -40,6 +40,8 @@
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
 #include "mongo/bson/oid.h"
+#include "mongo/db/auth/validated_tenancy_scope.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/logv2/bson_formatter.h"
 #include "mongo/logv2/component_settings_filter.h"
 #include "mongo/logv2/composite_backend.h"
@@ -58,8 +60,10 @@
 #include "mongo/logv2/uassert_sink.h"
 #include "mongo/platform/decimal128.h"
 #include "mongo/stdx/thread.h"
+#include "mongo/unittest/death_test.h"
 #include "mongo/unittest/temp_dir.h"
 #include "mongo/unittest/unittest.h"
+#include "mongo/util/exit_code.h"
 #include "mongo/util/str_escape.h"
 #include "mongo/util/string_map.h"
 #include "mongo/util/uuid.h"
@@ -69,8 +73,10 @@
 #include <boost/property_tree/json_parser.hpp>
 #include <boost/property_tree/ptree.hpp>
 
-namespace mongo {
-namespace logv2 {
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
+
+namespace mongo::logv2 {
 namespace {
 
 using constants::kAttributesFieldName;
@@ -80,6 +86,7 @@ using constants::kIdFieldName;
 using constants::kMessageFieldName;
 using constants::kSeverityFieldName;
 using constants::kTagsFieldName;
+using constants::kTenantFieldName;
 using constants::kTimestampFieldName;
 
 struct TypeWithoutBSON {
@@ -129,11 +136,6 @@ struct TypeWithBSON : TypeWithoutBSON {
     }
 };
 
-struct TypeWithOnlyBSON : private TypeWithBSON {
-    using TypeWithBSON::toBSON;
-    using TypeWithBSON::TypeWithBSON;
-};
-
 struct TypeWithBSONSerialize : TypeWithoutBSON {
     using TypeWithoutBSON::TypeWithoutBSON;
 
@@ -196,7 +198,7 @@ void applyDefaultFilterToSink(SinkPtr&& sink) {
 class LogDuringInitShutdownTester {
 public:
     LogDuringInitShutdownTester() {
-        auto sink = LogCaptureBackend::create(lines);
+        auto sink = LogCaptureBackend::create(lines, true);
         applyDefaultFilterToSink(sink);
         // We have to leave this sink installed as it is not allowed to install sinks during
         // shutdown. Add a filter so it is only used during this test.
@@ -204,7 +206,7 @@ public:
         sink->set_formatter(PlainFormatter());
         boost::log::core::get()->add_sink(sink);
 
-        auto enabledGuard = makeGuard([this] { enabled = false; });
+        ScopeGuard enabledGuard([this] { enabled = false; });
         LOGV2(20001, "log during init");
         ASSERT_EQUALS(lines.back(), "log during init");
     }
@@ -224,9 +226,10 @@ class LogV2Test : public unittest::Test {
 public:
     class LineCapture {
     public:
-        LineCapture()
+        LineCapture() = delete;
+        LineCapture(bool stripEol)
             : _lines{std::make_unique<std::vector<std::string>>()},
-              _sink{LogCaptureBackend::create(*_lines)} {}
+              _sink{LogCaptureBackend::create(*_lines, stripEol)} {}
         auto& lines() {
             return *_lines;
         }
@@ -286,9 +289,15 @@ public:
         _attachedSinks.push_back(sink);
     }
 
+    void popSink() {
+        auto sink = _attachedSinks.back();
+        boost::log::core::get()->remove_sink(sink);
+        _attachedSinks.pop_back();
+    }
+
     template <typename Fmt>
-    LineCapture makeLineCapture(Fmt&& formatter) {
-        LineCapture ret;
+    LineCapture makeLineCapture(Fmt&& formatter, bool stripEol = true) {
+        LineCapture ret(stripEol);
         auto& s = ret.sink();
         applyDefaultFilterToSink(s);
         s->set_formatter(std::forward<Fmt>(formatter));
@@ -354,17 +363,92 @@ TEST_F(LogV2Test, Basic) {
     // Message string is selected when using API that also take a format string
     LOGV2(20084, "fmtstr {name}", "msgstr", "name"_attr = 1);
     ASSERT_EQUALS(lines.back(), "msgstr");
+}
 
-    // Test that logging exceptions does not propagate out to user code in release builds
+TEST_F(LogV2Test, MismatchAttrInLogging) {
+    auto lines = makeLineCapture(PlainFormatter());
     if (!kDebugBuild) {
         LOGV2(4638203, "mismatch {name}", "not_name"_attr = 1);
         ASSERT(StringData(lines.back()).startsWith("Exception during log"_sd));
     }
 }
 
+TEST_F(LogV2Test, MissingAttrInLogging) {
+    auto lines = makeLineCapture(PlainFormatter());
+    if (!kDebugBuild) {
+        LOGV2(6636803, "Log missing {attr}");
+        ASSERT(StringData(lines.back()).startsWith("Exception during log"_sd));
+    }
+}
+
+namespace bl_sinks = boost::log::sinks;
+// Sink backend which will grab a mutex, then immediately segfault.
+class ConsumeSegfaultsBackend
+    : public bl_sinks::basic_formatted_sink_backend<char, bl_sinks::synchronized_feeding> {
+public:
+    static auto create() {
+        return boost::make_shared<bl_sinks::synchronous_sink<ConsumeSegfaultsBackend>>(
+            boost::make_shared<ConsumeSegfaultsBackend>());
+    }
+
+    void consume(boost::log::record_view const& rec, string_type const& formattedString) {
+        if (firstRun) {
+            firstRun = false;
+            raise(SIGSEGV);
+        } else {
+            // Reentrance of consume(), which could cause deadlock. Exit normally, causing the death
+            // test to fail.
+            exit(static_cast<int>(ExitCode::clean));
+        }
+    }
+
+private:
+    bool firstRun = true;
+};
+
+// Test that signals thrown during logging will not hang process death. Uses the
+// ConsumeSegfaultsBackend so that upon the initial log call, ConsumeSegfaultsBackend::consume will
+// be called, sending SIGSEGV. If the signal handler incorrectly invokes the logging subsystem, the
+// ConsumeSegfaultsBackend::consume function will be again invoked, failing the test since this
+// could result in deadlock.
+DEATH_TEST_F(LogV2Test, SIGSEGVDoesNotHang, "Got signal: ") {
+    auto sink = ConsumeSegfaultsBackend::create();
+    attachSink(sink);
+    LOGV2(6384304, "will SIGSEGV {str}", "str"_attr = "sigsegv");
+    // If we get here, we didn't segfault, and the test will fail.
+}
+
+class ConsumeThrowsBackend
+    : public bl_sinks::basic_formatted_sink_backend<char, bl_sinks::synchronized_feeding> {
+public:
+    struct LocalException : std::exception {};
+    static auto create() {
+        return boost::make_shared<bl_sinks::synchronous_sink<ConsumeThrowsBackend>>(
+            boost::make_shared<ConsumeThrowsBackend>());
+    }
+
+    void consume(boost::log::record_view const& rec, string_type const& formattedString) {
+        throw LocalException();
+    }
+};
+
+TEST_F(LogV2Test, ExceptInLogging) {
+    auto sink = ConsumeThrowsBackend::create();
+    attachSink(sink);
+    ASSERT_THROWS(LOGV2(6636801, "will throw exception"), ConsumeThrowsBackend::LocalException);
+    popSink();
+}
+
 class LogV2TypesTest : public LogV2Test {
 public:
     using LogV2Test::LogV2Test;
+    LogV2TypesTest() : LogV2Test() {
+        detail::setGetTenantIDCallback(
+            [this]() -> boost::optional<TenantId> { return this->tenant; });
+    }
+    ~LogV2TypesTest() {
+        detail::setGetTenantIDCallback(nullptr);
+    }
 
     // The JSON formatter should make the types round-trippable without data loss
     template <typename T>
@@ -373,13 +457,17 @@ public:
         std::istringstream json_stream(json.back());
         pt::ptree ptree;
         pt::json_parser::read_json(json_stream, ptree);
+        ASSERT_EQUALS(ptree.get<std::string>(std::string(kTenantFieldName)), tenant.toString());
         ASSERT_EQUALS(ptree.get<T>(std::string(kAttributesFieldName) + ".name"), expected);
     }
 
     auto lastBSONElement() {
+        ASSERT_EQUALS(BSONObj(bson.back().data()).getField(kTenantFieldName).str(),
+                      tenant.toString());
         return BSONObj(bson.back().data()).getField(kAttributesFieldName).Obj().getField("name"_sd);
     }
 
+    TenantId tenant = TenantId(OID::gen());
     LineCapture text = makeLineCapture(PlainFormatter());
     LineCapture json = makeLineCapture(JSONFormatter());
     LineCapture bson = makeLineCapture(BSONFormatter());
@@ -1573,13 +1661,18 @@ TEST_F(LogV2Test, Ramlog) {
     sink->set_formatter(PlainFormatter());
     attachSink(sink);
 
-    auto lines = makeLineCapture(PlainFormatter());
+    auto lines = makeLineCapture(PlainFormatter(), false);
 
     auto verifyRamLog = [&] {
         RamLog::LineIterator iter(ramlog);
-        for (const auto& s : lines.lines())
-            if (s != iter.next())
+        for (const auto& s : lines.lines()) {
+            const auto next = iter.next();
+            if (s != next) {
+                std::cout << "\n\n\n********************** s='" << s << "', next='" << next
+                          << "'\n";
                 return false;
+            }
+        }
         return true;
     };
 
@@ -1696,7 +1789,7 @@ TEST_F(LogV2Test, MultipleDomains) {
     };
     LogDomain other_domain(std::make_unique<OtherDomain>());
     std::vector<std::string> other_lines;
-    auto other_sink = LogCaptureBackend::create(other_lines);
+    auto other_sink = LogCaptureBackend::create(other_lines, true);
     other_sink->set_filter(ComponentSettingsFilter(other_domain, mgr().getGlobalSettings()));
     other_sink->set_formatter(PlainFormatter());
     attachSink(other_sink);
@@ -1763,7 +1856,7 @@ TEST_F(LogV2Test, FileLogging) {
 TEST_F(LogV2Test, UserAssert) {
     std::vector<std::string> lines;
     auto sink = wrapInSynchronousSink(wrapInCompositeBackend(
-        boost::make_shared<LogCaptureBackend>(lines), boost::make_shared<UserAssertSink>()));
+        boost::make_shared<LogCaptureBackend>(lines, true), boost::make_shared<UserAssertSink>()));
     applyDefaultFilterToSink(sink);
     sink->set_formatter(PlainFormatter());
     attachSink(sink);
@@ -1819,6 +1912,18 @@ TEST_F(UnstructuredLoggingTest, Args) {
     });
 }
 
+TEST_F(UnstructuredLoggingTest, ArgsLikeFormatSpecifier) {
+    // Ensure the plain formatter does not process the formatted string
+    startCapturingLogMessages();
+
+    std::string format_str = "format {} str {} fields";
+    logd(format_str, 1, "{ x : 1}");  // NOLINT
+    validate([&format_str](const BSONObj& obj) {
+        ASSERT_EQUALS(obj.getField(kMessageFieldName).String(),
+                      fmt::format(format_str, 1, "{ x : 1}"));
+    });
+}
+
 TEST_F(UnstructuredLoggingTest, ManyArgs) {
     std::string format_str = "{}{}{}{}{}{}{}{}{}{}{}";
     logd(format_str, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11);  // NOLINT
@@ -1837,7 +1942,12 @@ TEST_F(UnstructuredLoggingTest, UserToString) {
 }
 
 TEST_F(UnstructuredLoggingTest, UserToBSON) {
-    TypeWithOnlyBSON arg(1.0, 2.0);
+    struct TypeWithOnlyBSON {
+        BSONObj toBSON() const {
+            return BSONObjBuilder{}.append("x", 1).append("y", 2).obj();
+        }
+    };
+    TypeWithOnlyBSON arg;
     logd("{}", arg);  // NOLINT
     validate([&arg](const BSONObj& obj) {
         ASSERT_EQUALS(obj.getField(kMessageFieldName).String(), arg.toBSON().toString());
@@ -1852,6 +1962,31 @@ TEST_F(UnstructuredLoggingTest, UserBothStringAndBSON) {
     });
 }
 
+TEST_F(UnstructuredLoggingTest, VectorBSON) {
+    std::vector<BSONObj> vectorBSON = {BSON("str1"
+                                            << "str2"),
+                                       BSON("str3"
+                                            << "str4")};
+    logd("{}", vectorBSON);  // NOLINT
+    validate([&vectorBSON](const BSONObj& obj) {
+        ASSERT_EQUALS(obj.getField(kMessageFieldName).String(),
+                      "({\"str1\":\"str2\"}, {\"str3\":\"str4\"})");
+    });
+}
+
+TEST_F(UnstructuredLoggingTest, MapBSON) {
+    std::map<std::string, BSONObj> mapBSON = {{"key1",
+                                               BSON("str1"
+                                                    << "str2")},
+                                              {"key2",
+                                               BSON("str3"
+                                                    << "str4")}};
+    logd("{}", mapBSON);  // NOLINT
+    validate([&mapBSON](const BSONObj& obj) {
+        ASSERT_EQUALS(obj.getField(kMessageFieldName).String(),
+                      "(key1: {\"str1\":\"str2\"}, key2: {\"str3\":\"str4\"})");
+    });
+}
+
 }  // namespace
-}  // namespace logv2
-}  // namespace mongo
+}  // namespace mongo::logv2
