@@ -26,11 +26,6 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
-
-#include "mongo/platform/basic.h"
-
-#include "mongo/unittest/unittest.h"
 
 #include "mongo/client/remote_command_targeter_mock.h"
 #include "mongo/db/commands.h"
@@ -39,17 +34,21 @@
 #include "mongo/s/catalog/sharding_catalog_client_mock.h"
 #include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/cluster_commands_helpers.h"
-#include "mongo/s/shard_server_test_fixture.h"
+#include "mongo/s/sharding_router_test_fixture.h"
+#include "mongo/unittest/unittest.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
 
 namespace mongo {
 namespace {
 
+const HostAndPort kTestConfigShardHost("FakeConfigHost", 12345);
+
 const Status kShardNotFoundStatus{ErrorCodes::ShardNotFound, "dummy"};
 const Status kError1Status{ErrorCodes::HostUnreachable, "dummy"};
 const Status kError2Status{ErrorCodes::HostUnreachable, "dummy"};
 
-const Status kStaleConfigErrorStatus{ErrorCodes::StaleShardVersion, "dummy"};
-
 const Status kWriteConcernError1Status{ErrorCodes::WriteConcernFailed, "dummy"};
 const Status kWriteConcernError2Status{ErrorCodes::UnsatisfiableWriteConcern, "dummy"};
 
@@ -77,10 +76,21 @@ HostAndPort makeHostAndPort(const ShardId& shardId) {
     return HostAndPort(str::stream() << shardId << ":123");
 }
 
-}  // namespace
-
-class AppendRawResponsesTest : public ShardServerTestFixture {
+class AppendRawResponsesTest : public ShardingTestFixture {
 protected:
+    void setUp() {
+        ShardingTestFixture::setUp();
+
+        configTargeter()->setFindHostReturnValue(kTestConfigShardHost);
+
+        std::vector<std::tuple<ShardId, HostAndPort>> remoteShards;
+        for (const auto& shardId : kShardIdList) {
+            remoteShards.emplace_back(shardId, makeHostAndPort(shardId));
+        }
+
+        addRemoteShards(remoteShards);
+    }
+
     /**
      * Runs 'appendRawResponses' and asserts that the top-level status matches 'expectedStatus', the
      * 'raw' sub-object contains the shards in 'expectedShardsInRawSubObj', and the writeConcern
@@ -120,10 +130,7 @@ protected:
         const auto rawSubObj = resultObj.getField("raw").Obj();
         ASSERT_EQ(rawSubObj.nFields(), int(expectedShardsInRawSubObj.size()));
         for (const auto& shardId : expectedShardsInRawSubObj) {
-            const auto shardConnStr =
-                ConnectionString::forReplicaSet(shardId.toString(), {makeHostAndPort(shardId)})
-                    .toString();
-            ASSERT(rawSubObj.hasField(shardConnStr));
+            ASSERT(rawSubObj.hasField(makeHostAndPort(shardId).toString()));
         }
 
         // Check the shards with successes object.
@@ -150,25 +157,11 @@ protected:
         }
     }
 
-    void setUp() override {
-        ShardServerTestFixture::setUp();
-
-        for (const auto& shardId : kShardIdList) {
-            auto shardTargeter = RemoteCommandTargeterMock::get(
-                uassertStatusOK(shardRegistry()->getShard(operationContext(), shardId))
-                    ->getTargeter());
-            shardTargeter->setConnectionStringReturnValue(
-                ConnectionString::forReplicaSet(shardId.toString(), {makeHostAndPort(shardId)}));
-        }
-    }
-
-    std::unique_ptr<ShardingCatalogClient> makeShardingCatalogClient(
-        std::unique_ptr<DistLockManager> distLockManager) override {
+    std::unique_ptr<ShardingCatalogClient> makeShardingCatalogClient() override {
 
         class StaticCatalogClient final : public ShardingCatalogClientMock {
         public:
-            StaticCatalogClient(std::vector<ShardId> shardIds)
-                : ShardingCatalogClientMock(nullptr), _shardIds(std::move(shardIds)) {}
+            StaticCatalogClient(std::vector<ShardId> shardIds) : _shardIds(std::move(shardIds)) {}
 
             StatusWith<repl::OpTimeWith<std::vector<ShardType>>> getAllShards(
                 OperationContext* opCtx, repl::ReadConcernLevel readConcern) override {
@@ -196,8 +189,20 @@ protected:
     const ShardId kShard3{"s3"};
     const ShardId kShard4{"s4"};
     const ShardId kShard5{"s5"};
+
     const std::vector<ShardId> kShardIdList{kShard1, kShard2, kShard3, kShard4, kShard5};
 
+    const Status kStaleConfigErrorStatus{[] {
+                                             OID epoch{OID::gen()};
+                                             Timestamp timestamp{1, 0};
+                                             return StaleConfigInfo(
+                                                 NamespaceString("Foo.Bar"),
+                                                 ChunkVersion({epoch, timestamp}, {1, 0}),
+                                                 boost::none,
+                                                 ShardId{"dummy"});
+                                         }(),
+                                         "dummy"};
+
 private:
     static void _assertShardIdsMatch(const std::set<ShardId>& expectedShardIds,
                                      const std::set<ShardId>& actualShardIds) {
@@ -569,4 +574,5 @@ TEST_F(AppendRawResponsesTest,
         shardResponses, kError1Status, {kShard3, kShard4, kShard5}, {kShard5}, Status::OK(), true);
 }
 
+}  // namespace
 }  // namespace mongo
