@@ -27,20 +27,20 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
 #include "mongo/s/catalog/type_chunk.h"
 
-#include <cstring>
-
 #include "mongo/base/status_with.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/bson/util/bson_extract.h"
+#include "mongo/db/server_options.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
+
 namespace mongo {
 
 const NamespaceString ChunkType::ConfigNS("config.chunks");
@@ -51,16 +51,15 @@ const NamespaceString ChunkType::ConfigNS("config.chunks");
 const std::string ChunkType::ShardNSPrefix = "config.cache.chunks.";
 
 const BSONField<OID> ChunkType::name("_id");
-const BSONField<std::string> ChunkType::legacyName("_id");
 const BSONField<BSONObj> ChunkType::minShardID("_id");
-const BSONField<std::string> ChunkType::ns("ns");
+const BSONField<UUID> ChunkType::collectionUUID("uuid");
 const BSONField<BSONObj> ChunkType::min("min");
 const BSONField<BSONObj> ChunkType::max("max");
 const BSONField<std::string> ChunkType::shard("shard");
 const BSONField<bool> ChunkType::jumbo("jumbo");
 const BSONField<Date_t> ChunkType::lastmod("lastmod");
-const BSONField<OID> ChunkType::epoch("lastmodEpoch");
 const BSONField<BSONObj> ChunkType::history("history");
+const BSONField<int64_t> ChunkType::estimatedSizeBytes("estimatedDataSizeBytes");
 const BSONField<bool> ChunkType::historyIsAt40("historyIsAt40");
 
 namespace {
@@ -133,7 +132,7 @@ BSONObj ChunkRange::toBSON() const {
     return builder.obj();
 }
 
-const Status ChunkRange::extractKeyPattern(KeyPattern* shardKeyPatternOut) const {
+Status ChunkRange::extractKeyPattern(KeyPattern* shardKeyPatternOut) const {
     BSONObjIterator min(getMin());
     BSONObjIterator max(getMax());
     BSONObjBuilder b;
@@ -194,7 +193,7 @@ StatusWith<std::vector<ChunkHistory>> ChunkHistory::fromBSON(const BSONArray& so
 
     for (const auto& arrayElement : source) {
         if (arrayElement.type() == Object) {
-            IDLParserErrorContext tempContext("chunk history array");
+            IDLParserContext tempContext("chunk history array");
             values.emplace_back(ChunkHistoryBase::parse(tempContext, arrayElement.Obj()));
         } else {
             return {ErrorCodes::BadValue,
@@ -210,52 +209,16 @@ StatusWith<std::vector<ChunkHistory>> ChunkHistory::fromBSON(const BSONArray& so
 
 ChunkType::ChunkType() = default;
 
-ChunkType::ChunkType(NamespaceString nss, ChunkRange range, ChunkVersion version, ShardId shardId)
-    : _nss(nss),
+ChunkType::ChunkType(UUID collectionUUID, ChunkRange range, ChunkVersion version, ShardId shardId)
+    : _collectionUUID(collectionUUID),
       _min(range.getMin()),
       _max(range.getMax()),
-      _version(version),
+      _version(std::move(version)),
       _shard(std::move(shardId)) {}
 
-StatusWith<ChunkType> ChunkType::parseFromConfigBSONCommand(const BSONObj& source) {
+StatusWith<ChunkType> ChunkType::_parseChunkBase(const BSONObj& source) {
     ChunkType chunk;
 
-    {
-        OID chunkID;
-        Status status = bsonExtractOIDField(source, name.name(), &chunkID);
-        if (status.isOK()) {
-            chunk._id = chunkID;
-        } else if (status == ErrorCodes::NoSuchKey || status == ErrorCodes::TypeMismatch) {
-            // Ignore NoSuchKey because when chunks are sent in commands they are not required to
-            // include it.
-            //
-            // Ignore TypeMismatch for compatibility with binaries 4.2 and earlier, since the _id
-            // type was changed from string to OID.
-            //
-            // TODO SERVER-44034: Stop ignoring TypeMismatch.
-        } else {
-            return status;
-        }
-    }
-
-    {
-        std::string chunkNS;
-        Status status = bsonExtractStringField(source, ns.name(), &chunkNS);
-        if (!status.isOK())
-            return status;
-        chunk._nss = NamespaceString(chunkNS);
-    }
-
-    {
-        auto chunkRangeStatus = ChunkRange::fromBSON(source);
-        if (!chunkRangeStatus.isOK())
-            return chunkRangeStatus.getStatus();
-
-        const auto chunkRange = std::move(chunkRangeStatus.getValue());
-        chunk._min = chunkRange.getMin().getOwned();
-        chunk._max = chunkRange.getMax().getOwned();
-    }
-
     {
         std::string chunkShard;
         Status status = bsonExtractStringField(source, shard.name(), &chunkShard);
@@ -264,26 +227,6 @@ StatusWith<ChunkType> ChunkType::parseFromConfigBSONCommand(const BSONObj& sourc
         chunk._shard = chunkShard;
     }
 
-    {
-        bool chunkJumbo;
-        Status status = bsonExtractBooleanField(source, jumbo.name(), &chunkJumbo);
-        if (status.isOK()) {
-            chunk._jumbo = chunkJumbo;
-        } else if (status == ErrorCodes::NoSuchKey) {
-            // Jumbo status is missing, so it will be presumed false
-        } else {
-            return status;
-        }
-    }
-
-    {
-        auto versionStatus = ChunkVersion::parseLegacyWithField(source, ChunkType::lastmod());
-        if (!versionStatus.isOK()) {
-            return versionStatus.getStatus();
-        }
-        chunk._version = std::move(versionStatus.getValue());
-    }
-
     {
         BSONElement historyObj;
         Status status = bsonExtractTypedField(source, history.name(), Array, &historyObj);
@@ -303,77 +246,104 @@ StatusWith<ChunkType> ChunkType::parseFromConfigBSONCommand(const BSONObj& sourc
     return chunk;
 }
 
-StatusWith<ChunkType> ChunkType::fromConfigBSON(const BSONObj& source) {
-    StatusWith<ChunkType> chunkStatus = parseFromConfigBSONCommand(source);
+StatusWith<ChunkType> ChunkType::parseFromConfigBSON(const BSONObj& source,
+                                                     const OID& epoch,
+                                                     const Timestamp& timestamp) {
+    // Parse shard, and history
+    StatusWith<ChunkType> chunkStatus = _parseChunkBase(source);
     if (!chunkStatus.isOK()) {
         return chunkStatus.getStatus();
     }
-
     ChunkType chunk = chunkStatus.getValue();
 
-    if (!chunk._id) {
-        {
-            OID chunkID;
-            Status status = bsonExtractOIDField(source, name.name(), &chunkID);
-            if (status.isOK()) {
-                chunk._id = chunkID;
-            } else if (status == ErrorCodes::TypeMismatch) {
-                // The format of _id changed between 4.2 and 4.4 so for compatibility with chunks
-                // created in earlier versions we ignore TypeMismatch.
-                //
-                // TODO SERVER-44034: Stop ignoring TypeMismatch.
-            } else {
-                return status;
+    // Parse collectionUUID.
+    {
+        BSONElement collectionUUIDElem;
+        Status status = bsonExtractField(source, collectionUUID.name(), &collectionUUIDElem);
+        if (status.isOK()) {
+            auto swUUID = UUID::parse(collectionUUIDElem);
+            if (!swUUID.isOK()) {
+                return swUUID.getStatus();
             }
+            chunk._collectionUUID = uassertStatusOK(UUID::parse(collectionUUIDElem));
+        } else {
+            return status;
+        }
+    }
+
+    // Parse id.
+    {
+        OID chunkID;
+        Status status = bsonExtractOIDField(source, name.name(), &chunkID);
+        if (status.isOK()) {
+            chunk._id = chunkID;
+        } else {
+            return status;
+        }
+    }
+
+    // Parse lastmod.
+    {
+        auto versionElem = source[ChunkType::lastmod()];
+        if (versionElem.eoo())
+            return Status(ErrorCodes::NoSuchKey, "No version found");
+        if (versionElem.type() == bsonTimestamp || versionElem.type() == Date) {
+            auto chunkLastmod = Timestamp(versionElem._numberLong());
+            chunk._version =
+                ChunkVersion({epoch, timestamp}, {chunkLastmod.getSecs(), chunkLastmod.getInc()});
+        } else {
+            return {ErrorCodes::BadValue,
+                    str::stream() << "The field " << ChunkType::lastmod() << " cannot be parsed."};
+        }
+    }
+
+    // Parse min and max.
+    {
+        auto chunkRangeStatus = ChunkRange::fromBSON(source);
+        if (!chunkRangeStatus.isOK())
+            return chunkRangeStatus.getStatus();
+
+        const auto chunkRange = std::move(chunkRangeStatus.getValue());
+        chunk._min = chunkRange.getMin().getOwned();
+        chunk._max = chunkRange.getMax().getOwned();
+    }
+
+    // Parse estimatedSizeBytes if present.
+    {
+        auto elem = source.getField(estimatedSizeBytes.name());
+        if (!elem.eoo()) {
+            chunk._estimatedSizeBytes = elem.safeNumberLong();
+        }
+    }
+
+    // Parse jumbo flag.
+    {
+        bool chunkJumbo;
+        Status status = bsonExtractBooleanField(source, jumbo.name(), &chunkJumbo);
+        if (status.isOK()) {
+            chunk._jumbo = chunkJumbo;
+        } else if (status == ErrorCodes::NoSuchKey) {
+            // Jumbo status is missing, so it will be presumed false
+        } else {
+            return status;
         }
     }
 
     return chunk;
 }
 
-BSONObj ChunkType::toConfigBSON() const {
-    BSONObjBuilder builder;
-    if (_id)
-        builder.append(name.name(), getName());
-    if (_nss)
-        builder.append(ns.name(), getNS().ns());
-    if (_min)
-        builder.append(min.name(), getMin());
-    if (_max)
-        builder.append(max.name(), getMax());
-    if (_shard)
-        builder.append(shard.name(), getShard().toString());
-    if (_version)
-        _version->appendLegacyWithField(&builder, ChunkType::lastmod());
-    if (_jumbo)
-        builder.append(jumbo.name(), getJumbo());
-    addHistoryToBSON(builder);
-    return builder.obj();
-}
+StatusWith<ChunkType> ChunkType::parseFromShardBSON(const BSONObj& source,
+                                                    const OID& epoch,
+                                                    const Timestamp& timestamp) {
+    // Parse history and shard.
+    StatusWith<ChunkType> chunkStatus = _parseChunkBase(source);
+    if (!chunkStatus.isOK()) {
+        return chunkStatus.getStatus();
+    }
 
-BSONObj ChunkType::toConfigBSONLegacyID() const {
-    BSONObjBuilder builder;
-    if (_nss && _min)
-        builder.append(name.name(), genLegacyID(*_nss, *_min));
-    if (_nss)
-        builder.append(ns.name(), getNS().ns());
-    if (_min)
-        builder.append(min.name(), getMin());
-    if (_max)
-        builder.append(max.name(), getMax());
-    if (_shard)
-        builder.append(shard.name(), getShard().toString());
-    if (_version)
-        _version->appendLegacyWithField(&builder, ChunkType::lastmod());
-    if (_jumbo)
-        builder.append(jumbo.name(), getJumbo());
-    addHistoryToBSON(builder);
-    return builder.obj();
-}
-
-StatusWith<ChunkType> ChunkType::fromShardBSON(const BSONObj& source, const OID& epoch) {
-    ChunkType chunk;
+    ChunkType chunk = chunkStatus.getValue();
 
+    // Parse min and max.
     {
         BSONElement minKey;
         Status minKeyStatus = extractObject(source, minShardID.name(), &minKey);
@@ -397,43 +367,103 @@ StatusWith<ChunkType> ChunkType::fromShardBSON(const BSONObj& source, const OID&
         chunk._max = maxKey.Obj().getOwned();
     }
 
+    // Parse version.
     {
-        std::string chunkShard;
-        Status status = bsonExtractStringField(source, shard.name(), &chunkShard);
-        if (!status.isOK())
-            return status;
-        chunk._shard = chunkShard;
-    }
-
-    {
-        auto statusWithChunkVersion =
-            ChunkVersion::parseLegacyWithField(source, ChunkType::lastmod());
-        if (!statusWithChunkVersion.isOK()) {
-            return statusWithChunkVersion.getStatus();
+        auto lastmodElem = source[ChunkType::lastmod()];
+        if (lastmodElem.eoo())
+            return Status(ErrorCodes::NoSuchKey, "No version found");
+        if (lastmodElem.type() == bsonTimestamp || lastmodElem.type() == Date) {
+            auto chunkLastmod = Timestamp(lastmodElem._numberLong());
+            chunk._version =
+                ChunkVersion({epoch, timestamp}, {chunkLastmod.getSecs(), chunkLastmod.getInc()});
+        } else {
+            return {ErrorCodes::NoSuchKey,
+                    str::stream() << "Expected field " << ChunkType::lastmod() << " not found."};
         }
-        auto version = std::move(statusWithChunkVersion.getValue());
-        chunk._version = ChunkVersion(version.majorVersion(), version.minorVersion(), epoch);
     }
 
-    {
-        BSONElement historyObj;
-        Status status = bsonExtractTypedField(source, history.name(), Array, &historyObj);
-        if (status.isOK()) {
-            auto history = ChunkHistory::fromBSON(BSONArray(historyObj.Obj()));
-            if (!history.isOK())
-                return history.getStatus();
+    return chunk;
+}
 
-            chunk._history = std::move(history.getValue());
+StatusWith<ChunkType> ChunkType::parseFromNetworkRequest(const BSONObj& source) {
+    // Parse history and shard.
+    StatusWith<ChunkType> chunkStatus = _parseChunkBase(source);
+    if (!chunkStatus.isOK()) {
+        return chunkStatus.getStatus();
+    }
+
+    ChunkType chunk = chunkStatus.getValue();
+
+    // Parse UUID.
+    {
+        BSONElement collectionUUIDElem;
+        Status status = bsonExtractField(source, collectionUUID.name(), &collectionUUIDElem);
+        if (status.isOK()) {
+            auto swUUID = UUID::parse(collectionUUIDElem);
+            if (!swUUID.isOK()) {
+                return swUUID.getStatus();
+            }
+            chunk._collectionUUID = swUUID.getValue();
         } else if (status == ErrorCodes::NoSuchKey) {
-            // History is missing, so it will be presumed empty
+            return {ErrorCodes::FailedToParse, str::stream() << "There must be a UUID present"};
         } else {
             return status;
         }
     }
 
+    // Parse min and max.
+    {
+        auto chunkRangeStatus = ChunkRange::fromBSON(source);
+        if (!chunkRangeStatus.isOK())
+            return chunkRangeStatus.getStatus();
+
+        const auto chunkRange = std::move(chunkRangeStatus.getValue());
+        chunk._min = chunkRange.getMin().getOwned();
+        chunk._max = chunkRange.getMax().getOwned();
+    }
+
+    // Parse jumbo.
+    {
+        bool chunkJumbo;
+        Status status = bsonExtractBooleanField(source, jumbo.name(), &chunkJumbo);
+        if (status.isOK()) {
+            chunk._jumbo = chunkJumbo;
+        } else if (status == ErrorCodes::NoSuchKey) {
+            // Jumbo status is missing, so it will be presumed false
+        } else {
+            return status;
+        }
+    }
+
+    // Parse version.
+    chunk._version = ChunkVersion::parse(source[ChunkType::lastmod()]);
+
     return chunk;
 }
 
+BSONObj ChunkType::toConfigBSON() const {
+    BSONObjBuilder builder;
+    if (_id)
+        builder.append(name.name(), getName());
+    if (_collectionUUID)
+        _collectionUUID->appendToBuilder(&builder, collectionUUID.name());
+    if (_min)
+        builder.append(min.name(), getMin());
+    if (_max)
+        builder.append(max.name(), getMax());
+    if (_shard)
+        builder.append(shard.name(), getShard().toString());
+    if (_version)
+        builder.appendTimestamp(lastmod.name(), _version->toLong());
+    if (_estimatedSizeBytes)
+        builder.appendNumber(estimatedSizeBytes.name(),
+                             static_cast<long long>(*_estimatedSizeBytes));
+    if (_jumbo)
+        builder.append(jumbo.name(), getJumbo());
+    addHistoryToBSON(builder);
+    return builder.obj();
+}
+
 BSONObj ChunkType::toShardBSON() const {
     BSONObjBuilder builder;
     invariant(_min);
@@ -457,9 +487,8 @@ void ChunkType::setName(const OID& id) {
     _id = id;
 }
 
-void ChunkType::setNS(const NamespaceString& nss) {
-    invariant(nss.isValid());
-    _nss = nss;
+void ChunkType::setCollectionUUID(const UUID& uuid) {
+    _collectionUUID = uuid;
 }
 
 void ChunkType::setMin(const BSONObj& min) {
@@ -482,6 +511,13 @@ void ChunkType::setShard(const ShardId& shard) {
     _shard = shard;
 }
 
+void ChunkType::setEstimatedSizeBytes(const boost::optional<int64_t>& estimatedSize) {
+    uassert(ErrorCodes::BadValue,
+            "estimatedSizeBytes cannot be negative",
+            !estimatedSize.has_value() || estimatedSize.value() >= 0);
+    _estimatedSizeBytes = estimatedSize;
+}
+
 void ChunkType::setJumbo(bool jumbo) {
     _jumbo = jumbo;
 }
@@ -497,19 +533,19 @@ void ChunkType::addHistoryToBSON(BSONObjBuilder& builder) const {
 }
 
 Status ChunkType::validate() const {
-    if (!_min.is_initialized() || _min->isEmpty()) {
+    if (!_min.has_value() || _min->isEmpty()) {
         return Status(ErrorCodes::NoSuchKey, str::stream() << "missing " << min.name() << " field");
     }
 
-    if (!_max.is_initialized() || _max->isEmpty()) {
+    if (!_max.has_value() || _max->isEmpty()) {
         return Status(ErrorCodes::NoSuchKey, str::stream() << "missing " << max.name() << " field");
     }
 
-    if (!_version.is_initialized() || !_version->isSet()) {
+    if (!_version.has_value() || !_version->isSet()) {
         return Status(ErrorCodes::NoSuchKey, str::stream() << "missing version field");
     }
 
-    if (!_shard.is_initialized() || !_shard->isValid()) {
+    if (!_shard.has_value() || !_shard->isValid()) {
         return Status(ErrorCodes::NoSuchKey,
                       str::stream() << "missing " << shard.name() << " field");
     }
@@ -541,9 +577,8 @@ Status ChunkType::validate() const {
     if (!_history.empty()) {
         if (_history.front().getShard() != *_shard) {
             return {ErrorCodes::BadValue,
-                    str::stream() << "Latest entry of chunk history refer to shard "
-                                  << _history.front().getShard()
-                                  << " that does not match the current shard " << *_shard};
+                    str::stream() << "History contains an invalid shard "
+                                  << _history.front().getShard()};
         }
     }
 
@@ -556,22 +591,4 @@ std::string ChunkType::toString() const {
     return toConfigBSON().toString();
 }
 
-std::string ChunkType::genLegacyID(const NamespaceString& nss, const BSONObj& o) {
-    StringBuilder buf;
-    buf << nss.ns() << "-";
-
-    BSONObjIterator i(o);
-    while (i.more()) {
-        BSONElement e = i.next();
-        buf << e.fieldName() << "_" << e.toString(false, true);
-    }
-
-    return buf.str();
-}
-
-std::string ChunkType::getLegacyName() const {
-    invariant(_nss && _min);
-    return genLegacyID(*_nss, *_min);
-}
-
 }  // namespace mongo
