@@ -33,224 +33,114 @@
 #include <string>
 #include <vector>
 
+#include "mongo/db/jsobj.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/s/chunk.h"
-#include "mongo/s/chunk_version.h"
-#include "mongo/s/client/shard.h"
+#include "mongo/s/database_version.h"
+#include "mongo/s/resharding/type_collection_fields_gen.h"
 #include "mongo/s/shard_key_pattern.h"
+#include "mongo/s/type_collection_common_types_gen.h"
+#include "mongo/stdx/unordered_map.h"
 #include "mongo/util/concurrency/ticketholder.h"
+#include "mongo/util/read_through_cache.h"
 
 namespace mongo {
 
 class CanonicalQuery;
 struct QuerySolutionNode;
-class OperationContext;
 class ChunkManager;
 
 struct ShardVersionTargetingInfo {
-    ShardVersionTargetingInfo(const ShardVersionTargetingInfo& other)
-        : shardVersion(other.shardVersion),
-          validAfter(other.validAfter),
-          isStale(other.isStale.load()) {}
-
-    ShardVersionTargetingInfo(const OID& epoch);
-    ShardVersionTargetingInfo(ChunkVersion shardVersion, Timestamp validAfter)
-        : shardVersion(std::move(shardVersion)), validAfter(std::move(validAfter)) {}
-
-    // Max chunk version for the shard.
-    ChunkVersion shardVersion;
-
-    // Max validAfter for the shard, effectively this is the timestamp of the latest placement
-    // change that occurred on a particular shard.
-    Timestamp validAfter;
-
     // Indicates whether the shard is stale and thus needs a catalog cache refresh
     AtomicWord<bool> isStale{false};
+
+    // Max chunk version for the shard
+    ChunkVersion shardVersion;
+
+    ShardVersionTargetingInfo(const OID& epoch, const Timestamp& timestamp);
 };
 
 // Map from a shard to a struct indicating both the max chunk version on that shard and whether the
 // shard is currently marked as needing a catalog cache refresh (stale).
 using ShardVersionMap = stdx::unordered_map<ShardId, ShardVersionTargetingInfo, ShardId::Hasher>;
 
-// This class serves as a Facade around how the mapping of ranges to chunks is represented. It also
-// provides a simpler, high-level interface for domain specific operations without exposing the
-// underlying implementation.
+/**
+ * This class serves as a Facade around how the mapping of ranges to chunks is represented. It also
+ * provides a simpler, high-level interface for domain specific operations without exposing the
+ * underlying implementation.
+ */
 class ChunkMap {
-public:
-    // Vector of chunks ordered by max key in ascending order.
+    // Vector of chunks ordered by max key.
     using ChunkVector = std::vector<std::shared_ptr<ChunkInfo>>;
-    using ChunkVectorMap = std::map<std::string, std::shared_ptr<ChunkVector>>;
 
-    explicit ChunkMap(OID epoch, size_t chunkVectorSize)
-        : _collectionVersion(0, 0, epoch), _maxChunkVectorSize(chunkVectorSize) {}
+public:
+    ChunkMap(OID epoch, const Timestamp& timestamp, size_t initialCapacity = 0);
 
-    size_t size() const;
+    size_t size() const {
+        return _chunkMap.size();
+    }
 
-    // Max version across all chunks
     ChunkVersion getVersion() const {
         return _collectionVersion;
     }
 
-    size_t getMaxChunkVectorSize() const {
-        return _maxChunkVectorSize;
-    }
-
-    const ShardVersionMap& getShardVersionsMap() const {
-        return _shardVersions;
-    }
-
-    const ChunkVectorMap& getChunkVectorMap() const {
-        return _chunkVectorMap;
-    }
-
-
-    /*
-     * Invoke the given handler for each std::shared_ptr<ChunkInfo> contained in this chunk map
-     * until either all matching chunks have been processed or @handler returns false.
-     *
-     * Chunks are yielded in ascending order of shardkey (e.g. minKey to maxKey);
-     *
-     * When shardKey is provided the function will start yileding from the chunk that contains the
-     * given shard key.
-     */
     template <typename Callable>
     void forEach(Callable&& handler, const BSONObj& shardKey = BSONObj()) const {
-        if (shardKey.isEmpty()) {
-            for (const auto& mapIt : _chunkVectorMap) {
-                for (const auto& chunkInfoPtr : *(mapIt.second)) {
-                    if (!handler(chunkInfoPtr))
-                        return;
-                }
-            }
+        auto it = shardKey.isEmpty() ? _chunkMap.begin() : _findIntersectingChunk(shardKey);
 
-            return;
-        }
-
-        auto shardKeyString = ShardKeyPattern::toKeyString(shardKey);
-
-        const auto mapItBegin = _chunkVectorMap.upper_bound(shardKeyString);
-        for (auto mapIt = mapItBegin; mapIt != _chunkVectorMap.end(); mapIt++) {
-            const auto& chunkVector = *(mapIt->second);
-            auto it = mapIt == mapItBegin ? _findIntersectingChunkIterator(shardKeyString,
-                                                                           chunkVector.begin(),
-                                                                           chunkVector.end(),
-                                                                           true /*isMaxInclusive*/)
-                                          : chunkVector.begin();
-            for (; it != chunkVector.end(); ++it) {
-                if (!handler(*it))
-                    return;
-            }
+        for (; it != _chunkMap.end(); ++it) {
+            if (!handler(*it))
+                break;
         }
     }
 
-
-    /*
-     * Invoke the given @handler for each std::shared_ptr<ChunkInfo> that overlaps with range [@min,
-     * @max] until either all matching chunks have been processed or @handler returns false.
-     *
-     * Chunks are yielded in ascending order of shardkey (e.g. minKey to maxKey);
-     *
-     * When @isMaxInclusive is true also the chunk whose minKey is equal to @max will be yielded.
-     */
     template <typename Callable>
     void forEachOverlappingChunk(const BSONObj& min,
                                  const BSONObj& max,
                                  bool isMaxInclusive,
                                  Callable&& handler) const {
-        const auto minShardKeyStr = ShardKeyPattern::toKeyString(min);
-        const auto maxShardKeyStr = ShardKeyPattern::toKeyString(max);
-        const auto bounds =
-            _overlappingVectorSlotBounds(minShardKeyStr, maxShardKeyStr, isMaxInclusive);
-        for (auto mapIt = bounds.first; mapIt != bounds.second; ++mapIt) {
+        const auto bounds = _overlappingBounds(min, max, isMaxInclusive);
 
-            const auto& chunkVector = *(mapIt->second);
-
-            const auto chunkItBegin = [&] {
-                if (mapIt == bounds.first) {
-                    // On first vector we need to start from chunk that contain the given minKey
-                    return _findIntersectingChunkIterator(minShardKeyStr,
-                                                          chunkVector.begin(),
-                                                          chunkVector.end(),
-                                                          true /* isMaxInclusive */);
-                }
-                return chunkVector.begin();
-            }();
-
-            const auto chunkItEnd = [&] {
-                if (mapIt == std::prev(bounds.second)) {
-                    // On last vector we need to skip all chunks that are greater than the give
-                    // maxKey
-                    auto it = _findIntersectingChunkIterator(
-                        maxShardKeyStr, chunkItBegin, chunkVector.end(), isMaxInclusive);
-                    return it == chunkVector.end() ? it : ++it;
-                }
-                return chunkVector.end();
-            }();
-
-            for (auto chunkIt = chunkItBegin; chunkIt != chunkItEnd; ++chunkIt) {
-                if (!handler(*chunkIt))
-                    return;
-            }
+        for (auto it = bounds.first; it != bounds.second; ++it) {
+            if (!handler(*it))
+                break;
         }
     }
 
+    ShardVersionMap constructShardVersionMap() const;
     std::shared_ptr<ChunkInfo> findIntersectingChunk(const BSONObj& shardKey) const;
 
-    ChunkMap createMerged(ChunkVector changedChunks) const;
+    void appendChunk(const std::shared_ptr<ChunkInfo>& chunk);
+
+    ChunkMap createMerged(const std::vector<std::shared_ptr<ChunkInfo>>& changedChunks) const;
 
     BSONObj toBSON() const;
 
-    std::string toString() const;
-
 private:
-    ChunkVector::const_iterator _findIntersectingChunkIterator(const std::string& shardKeyString,
-                                                               ChunkVector::const_iterator first,
-                                                               ChunkVector::const_iterator last,
-                                                               bool isMaxInclusive) const;
+    ChunkVector::const_iterator _findIntersectingChunk(const BSONObj& shardKey,
+                                                       bool isMaxInclusive = true) const;
+    std::pair<ChunkVector::const_iterator, ChunkVector::const_iterator> _overlappingBounds(
+        const BSONObj& min, const BSONObj& max, bool isMaxInclusive) const;
 
-    std::pair<ChunkVectorMap::const_iterator, ChunkVectorMap::const_iterator>
-    _overlappingVectorSlotBounds(const std::string& minShardKeyStr,
-                                 const std::string& maxShardKeyStr,
-                                 bool isMaxInclusive) const;
-    ChunkMap _makeUpdated(ChunkVector&& changedChunks) const;
-
-    void _updateShardVersionFromDiscardedChunk(const ChunkInfo& chunk);
-    void _updateShardVersionFromUpdateChunk(const ChunkInfo& chunk,
-                                            const ShardVersionMap& oldShardVersions);
-    void _commitUpdatedChunkVector(std::shared_ptr<ChunkVector>&& chunkVectorPtr,
-                                   bool checkMaxKeyConsistency);
-    void _mergeAndCommitUpdatedChunkVector(ChunkVectorMap::const_iterator pos,
-                                           std::shared_ptr<ChunkVector>&& chunkVectorPtr);
-    void _splitAndCommitUpdatedChunkVector(ChunkVectorMap::const_iterator pos,
-                                           std::shared_ptr<ChunkVector>&& chunkVectorPtr);
-
-    ChunkVectorMap _chunkVectorMap;
+    ChunkVector _chunkMap;
 
     // Max version across all chunks
     ChunkVersion _collectionVersion;
-
-    // The representation of shard versions and staleness indicators for this namespace. If a
-    // shard does not exist, it will not have an entry in the map.
-    // Note: this declaration must not be moved before _chunkMap since it is initialized by using
-    // the _chunkVectorMap instance.
-    ShardVersionMap _shardVersions;
-
-    // Maximum size of chunk vectors stored in the chunk vector map.
-    // Bigger vectors will imply slower incremental refreshes (more chunks to copy) but
-    // faster map copy (less chunk vector pointers to copy).
-    size_t _maxChunkVectorSize;
 };
 
 /**
  * In-memory representation of the routing table for a single sharded collection at various points
  * in time.
  */
-class RoutingTableHistory : public std::enable_shared_from_this<RoutingTableHistory> {
+class RoutingTableHistory {
     RoutingTableHistory(const RoutingTableHistory&) = delete;
     RoutingTableHistory& operator=(const RoutingTableHistory&) = delete;
 
 public:
+    RoutingTableHistory(RoutingTableHistory&&) = default;
+    RoutingTableHistory& operator=(RoutingTableHistory&&) = default;
+
     /**
      * Makes an instance with a routing table for collection "nss", sharded on
      * "shardKeyPattern".
@@ -261,14 +151,22 @@ public:
      *
      * The "chunks" vector must contain the chunk routing information sorted in ascending order by
      * chunk version, and adhere to the requirements of the routing table update algorithm.
+     *
+     * The existence of "reshardingFields" inside the optional implies that this field was present
+     * inside the config.collections entry when refreshing.
      */
-    static std::shared_ptr<RoutingTableHistory> makeNew(
+    static RoutingTableHistory makeNew(
         NamespaceString nss,
-        boost::optional<UUID>,
+        UUID uuid,
         KeyPattern shardKeyPattern,
         std::unique_ptr<CollatorInterface> defaultCollator,
         bool unique,
         OID epoch,
+        const Timestamp& timestamp,
+        boost::optional<TypeCollectionTimeseriesFields> timeseriesFields,
+        boost::optional<TypeCollectionReshardingFields> reshardingFields,
+        boost::optional<uint64_t> maxChunkSizeBytes,
+        bool allowMigrations,
         const std::vector<ChunkType>& chunks);
 
     /**
@@ -277,17 +175,21 @@ public:
      *
      * The changes in "changedChunks" must be sorted in ascending order by chunk version, and adhere
      * to the requirements of the routing table update algorithm.
+     *
+     * The existence of timeseriesFields/reshardingFields inside the optional implies that this
+     * field was present inside the config.collections entry when refreshing. An uninitialized
+     * timeseriesFields/reshardingFields parameter implies that the field was not present, and will
+     * clear any currently held timeseries/resharding fields inside the resulting
+     * RoutingTableHistory.
      */
-    std::shared_ptr<RoutingTableHistory> makeUpdated(const std::vector<ChunkType>& changedChunks);
+    RoutingTableHistory makeUpdated(
+        boost::optional<TypeCollectionTimeseriesFields> timeseriesFields,
+        boost::optional<TypeCollectionReshardingFields> reshardingFields,
+        boost::optional<uint64_t> maxChunkSizeBytes,
+        bool allowMigrations,
+        const std::vector<ChunkType>& changedChunks) const;
 
-    /**
-     * Returns an increasing number of the reload sequence number of this chunk manager.
-     */
-    unsigned long long getSequenceNumber() const {
-        return _sequenceNumber;
-    }
-
-    const NamespaceString& getns() const {
+    const NamespaceString& nss() const {
         return _nss;
     }
 
@@ -315,7 +217,6 @@ public:
      */
     void setAllShardsRefreshed();
 
-    // Max version across all chunks
     ChunkVersion getVersion() const {
         return _chunkMap.getVersion();
     }
@@ -324,26 +225,14 @@ public:
      * Retrieves the shard version for the given shard. Will throw a ShardInvalidatedForTargeting
      * exception if the shard is marked as stale.
      */
-    ChunkVersion getVersion(const ShardId& shardId) const {
-        return _getVersion(shardId, true).shardVersion;
-    }
+    ChunkVersion getVersion(const ShardId& shardId) const;
 
     /**
      * Retrieves the shard version for the given shard. Will not throw if the shard is marked as
      * stale. Only use when logging the given chunk version -- if the caller must execute logic
      * based on the returned version, use getVersion() instead.
      */
-    ChunkVersion getVersionForLogging(const ShardId& shardId) const {
-        return _getVersion(shardId, false).shardVersion;
-    }
-
-    /**
-     * Retrieves the maximum validAfter timestamp for the given shard. Will throw a
-     * ShardInvalidatedForTargeting exception if the shard is marked as stale.
-     */
-    Timestamp getMaxValidAfter(const ShardId& shardId) const {
-        return _getVersion(shardId, true).validAfter;
-    }
+    ChunkVersion getVersionForLogging(const ShardId& shardId) const;
 
     size_t numChunks() const {
         return _chunkMap.size();
@@ -384,42 +273,74 @@ public:
 
     std::string toString() const;
 
-    bool uuidMatches(UUID uuid) const {
-        return _uuid && *_uuid == uuid;
+    bool uuidMatches(const UUID& uuid) const {
+        return _uuid == uuid;
     }
 
-    boost::optional<UUID> getUUID() const {
+    const UUID& getUUID() const {
         return _uuid;
     }
 
+    const boost::optional<TypeCollectionTimeseriesFields>& getTimeseriesFields() const {
+        return _timeseriesFields;
+    }
+
+    const boost::optional<TypeCollectionReshardingFields>& getReshardingFields() const {
+        return _reshardingFields;
+    }
+
+    bool allowMigrations() const {
+        return _allowMigrations;
+    }
+
+    // collection default chunk size or +inf, iff no splits should happen
+    boost::optional<uint64_t> maxChunkSizeBytes() const {
+        return _maxChunkSizeBytes;
+    }
+
 private:
+    friend class ChunkManager;
+
     RoutingTableHistory(NamespaceString nss,
-                        boost::optional<UUID> uuid,
+                        UUID uuid,
                         KeyPattern shardKeyPattern,
                         std::unique_ptr<CollatorInterface> defaultCollator,
                         bool unique,
+                        boost::optional<TypeCollectionTimeseriesFields> timeseriesFields,
+                        boost::optional<TypeCollectionReshardingFields> reshardingFields,
+                        boost::optional<uint64_t> maxChunkSizeBytes,
+                        bool allowMigrations,
                         ChunkMap chunkMap);
 
-    ShardVersionTargetingInfo _getVersion(const ShardId& shardId, bool throwOnStaleShard) const;
-
-    // The shard versioning mechanism hinges on keeping track of the number of times we reload
-    // ChunkManagers.
-    const unsigned long long _sequenceNumber;
+    ChunkVersion _getVersion(const ShardId& shardName, bool throwOnStaleShard) const;
 
     // Namespace to which this routing information corresponds
-    const NamespaceString _nss;
+    NamespaceString _nss;
 
-    // The invariant UUID of the collection.  This is optional in 3.6, except in change streams.
-    const boost::optional<UUID> _uuid;
+    // The UUID of the collection
+    UUID _uuid;
 
     // The key pattern used to shard the collection
-    const ShardKeyPattern _shardKeyPattern;
+    ShardKeyPattern _shardKeyPattern;
 
     // Default collation to use for routing data queries for this collection
-    const std::unique_ptr<CollatorInterface> _defaultCollator;
+    std::unique_ptr<CollatorInterface> _defaultCollator;
 
     // Whether the sharding key is unique
-    const bool _unique;
+    bool _unique;
+
+    // This information will be valid if the collection is a time-series buckets collection.
+    boost::optional<TypeCollectionTimeseriesFields> _timeseriesFields;
+
+    // The set of fields related to an ongoing resharding operation involving this collection. The
+    // presence of the type inside the optional indicates that the collection is involved in a
+    // resharding operation, and that these fields were present in the config.collections entry
+    // for this collection.
+    boost::optional<TypeCollectionReshardingFields> _reshardingFields;
+
+    boost::optional<uint64_t> _maxChunkSizeBytes;
+
+    bool _allowMigrations;
 
     // Map from the max for each chunk to an entry describing the chunk. The union of all chunks'
     // ranges must cover the complete space from [MinKey, MaxKey).
@@ -430,75 +351,211 @@ private:
     // Note: this declaration must not be moved before _chunkMap since it is initialized by using
     // the _chunkMap instance.
     ShardVersionMap _shardVersions;
-
-    friend class ChunkManager;
 };
 
-// This will be renamed to RoutingTableHistory and the original RoutingTableHistory will be
-// ChunkHistoryMap
-class ChunkManager : public std::enable_shared_from_this<ChunkManager> {
-    ChunkManager(const ChunkManager&) = delete;
-    ChunkManager& operator=(const ChunkManager&) = delete;
-
+/**
+ * Constructed to be used exclusively by the CatalogCache as a vector clock (Time) to drive
+ * CollectionCache's lookups.
+ *
+ * The ChunkVersion class contains a non comparable epoch, which makes impossible to compare two
+ * ChunkVersions when their epochs's differ.
+ *
+ * This class wraps a ChunkVersion object with a node-local sequence number
+ * (_epochDisambiguatingSequenceNum) that allows the comparision.
+ *
+ * This class should go away once a cluster-wide comparable ChunkVersion is implemented.
+ */
+class ComparableChunkVersion {
 public:
-    ChunkManager(std::shared_ptr<RoutingTableHistory> rt, boost::optional<Timestamp> clusterTime)
-        : _rt(std::move(rt)), _clusterTime(std::move(clusterTime)) {}
+    /**
+     * Creates a ComparableChunkVersion that wraps the given ChunkVersion.
+     * Each object created through this method will have a local sequence number greater than the
+     * previously created ones.
+     */
+    static ComparableChunkVersion makeComparableChunkVersion(const ChunkVersion& version);
 
     /**
-     * Returns an increasing number of the reload sequence number of this chunk manager.
+     * Creates a new instance which will artificially be greater than any
+     * previously created ComparableChunkVersion and smaller than any instance
+     * created afterwards. Used as means to cause the collections cache to
+     * attempt a refresh in situations where causal consistency cannot be
+     * inferred.
      */
-    unsigned long long getSequenceNumber() const {
-        return _rt->getSequenceNumber();
+    static ComparableChunkVersion makeComparableChunkVersionForForcedRefresh();
+
+    /**
+     * Empty constructor needed by the ReadThroughCache.
+     *
+     * Instances created through this constructor will be always less then the ones created through
+     * the two static constructors, but they do not carry any meaningful value and can only be used
+     * for comparison purposes.
+     */
+    ComparableChunkVersion() = default;
+
+    std::string toString() const;
+
+    bool sameEpoch(const ComparableChunkVersion& other) const {
+        return _chunkVersion->epoch() == other._chunkVersion->epoch();
     }
 
-    const NamespaceString& getns() const {
-        return _rt->getns();
+    bool operator==(const ComparableChunkVersion& other) const;
+
+    bool operator!=(const ComparableChunkVersion& other) const {
+        return !(*this == other);
     }
 
+    /**
+     * In case the two compared instances have different epochs, the most recently created one will
+     * be greater, otherwise the comparision will be driven by the major/minor versions of the
+     * underlying ChunkVersion.
+     */
+    bool operator<(const ComparableChunkVersion& other) const;
+
+    bool operator>(const ComparableChunkVersion& other) const {
+        return other < *this;
+    }
+
+    bool operator<=(const ComparableChunkVersion& other) const {
+        return !(*this > other);
+    }
+
+    bool operator>=(const ComparableChunkVersion& other) const {
+        return !(*this < other);
+    }
+
+private:
+    friend class CatalogCache;
+
+    static AtomicWord<uint64_t> _epochDisambiguatingSequenceNumSource;
+    static AtomicWord<uint64_t> _forcedRefreshSequenceNumSource;
+
+    ComparableChunkVersion(uint64_t forcedRefreshSequenceNum,
+                           boost::optional<ChunkVersion> version,
+                           uint64_t epochDisambiguatingSequenceNum)
+        : _forcedRefreshSequenceNum(forcedRefreshSequenceNum),
+          _chunkVersion(std::move(version)),
+          _epochDisambiguatingSequenceNum(epochDisambiguatingSequenceNum) {}
+
+    void setChunkVersion(const ChunkVersion& version);
+
+    uint64_t _forcedRefreshSequenceNum{0};
+
+    boost::optional<ChunkVersion> _chunkVersion;
+
+    // Locally incremented sequence number that allows to compare two colection versions with
+    // different epochs. Each new comparableChunkVersion will have a greater sequence number than
+    // the ones created before.
+    uint64_t _epochDisambiguatingSequenceNum{0};
+};
+
+/**
+ * This intermediate structure is necessary to be able to store UNSHARDED collections in the routing
+ * table history cache below. The reason is that currently the RoutingTableHistory class only
+ * supports sharded collections (i.e., collections which have entries in config.collections and
+ * config.chunks).
+ */
+struct OptionalRoutingTableHistory {
+    // UNSHARDED collection constructor
+    OptionalRoutingTableHistory() = default;
+
+    // SHARDED collection constructor
+    OptionalRoutingTableHistory(std::shared_ptr<RoutingTableHistory> rt) : optRt(std::move(rt)) {}
+
+    // If nullptr, the collection is UNSHARDED, otherwise it is SHARDED
+    std::shared_ptr<RoutingTableHistory> optRt;
+};
+
+using RoutingTableHistoryCache =
+    ReadThroughCache<NamespaceString, OptionalRoutingTableHistory, ComparableChunkVersion>;
+using RoutingTableHistoryValueHandle = RoutingTableHistoryCache::ValueHandle;
+
+/**
+ * Combines a shard, the shard version, and database version that the shard should be using
+ */
+struct ShardEndpoint {
+    ShardEndpoint(const ShardId& shardName,
+                  boost::optional<ChunkVersion> shardVersion,
+                  boost::optional<DatabaseVersion> dbVersion);
+
+    ShardId shardName;
+
+    boost::optional<ChunkVersion> shardVersion;
+    boost::optional<DatabaseVersion> databaseVersion;
+};
+
+/**
+ * Wrapper around a RoutingTableHistory, which pins it to a particular point in time.
+ */
+class ChunkManager {
+public:
+    ChunkManager(ShardId dbPrimary,
+                 DatabaseVersion dbVersion,
+                 RoutingTableHistoryValueHandle rt,
+                 boost::optional<Timestamp> clusterTime)
+        : _dbPrimary(std::move(dbPrimary)),
+          _dbVersion(std::move(dbVersion)),
+          _rt(std::move(rt)),
+          _clusterTime(std::move(clusterTime)) {}
+
+    // Methods supported on both sharded and unsharded collections
+
+    bool isSharded() const {
+        return bool(_rt->optRt);
+    }
+
+    /**
+     * Indicates that this collection must not honour any moveChunk requests, because it is required
+     * to provide a stable view of its constituent shards.
+     */
+    bool allowMigrations() const;
+
+    bool allowAutoSplit() const;
+
+    boost::optional<uint64_t> maxChunkSizeBytes() const;
+
+    const ShardId& dbPrimary() const {
+        return _dbPrimary;
+    }
+
+    const DatabaseVersion& dbVersion() const {
+        return _dbVersion;
+    }
+
+    int numChunks() const {
+        return _rt->optRt ? _rt->optRt->numChunks() : 1;
+    }
+
+    std::string toString() const;
+
+    // Methods only supported on sharded collections (caller must check isSharded())
+
     const ShardKeyPattern& getShardKeyPattern() const {
-        return _rt->getShardKeyPattern();
+        return _rt->optRt->getShardKeyPattern();
     }
 
     const CollatorInterface* getDefaultCollator() const {
-        return _rt->getDefaultCollator();
+        return _rt->optRt->getDefaultCollator();
     }
 
     bool isUnique() const {
-        return _rt->isUnique();
+        return _rt->optRt->isUnique();
     }
 
     ChunkVersion getVersion() const {
-        return _rt->getVersion();
+        return _rt->optRt->getVersion();
     }
 
-    /**
-     * Retrieves the placement version for the given shard. Will throw a
-     * ShardInvalidatedForTargeting exception if the shard is marked as stale.
-     */
     ChunkVersion getVersion(const ShardId& shardId) const {
-        return _rt->getVersion(shardId);
+        return _rt->optRt->getVersion(shardId);
     }
 
-    /**
-     * Retrieves the maximum validAfter timestamp for the given shard. Will throw a
-     * ShardInvalidatedForTargeting exception if the shard is marked as stale.
-     */
-    Timestamp getMaxValidAfter(const ShardId& shardId) const {
-        return _rt->getMaxValidAfter(shardId);
-    }
-
-    /**
-     * Retrieves the placement version for the given shard. Will not throw if the shard is marked as
-     * stale. Only use when logging the given chunk version -- if the caller must execute logic
-     * based on the returned version, use getVersion() instead.
-     */
     ChunkVersion getVersionForLogging(const ShardId& shardId) const {
-        return _rt->getVersionForLogging(shardId);
+        return _rt->optRt->getVersionForLogging(shardId);
     }
 
     template <typename Callable>
     void forEachChunk(Callable&& handler) const {
-        _rt->forEachChunk(
+        _rt->optRt->forEachChunk(
             [this, handler = std::forward<Callable>(handler)](const auto& chunkInfo) mutable {
                 if (!handler(Chunk{*chunkInfo, _clusterTime}))
                     return false;
@@ -507,27 +564,6 @@ public:
             });
     }
 
-    int numChunks() const {
-        return _rt->numChunks();
-    }
-
-    template <typename Callable>
-    void forEachOverlappingChunk(const BSONObj& min,
-                                 const BSONObj& max,
-                                 bool isMaxInclusive,
-                                 Callable&& handler) const {
-        _rt->forEachOverlappingChunk(
-            min,
-            max,
-            isMaxInclusive,
-            [this, handler = std::forward<Callable>(handler)](const auto& chunkInfo) mutable {
-                if (!handler(Chunk{*chunkInfo, _clusterTime})) {
-                    return false;
-                }
-                return true;
-            });
-    }
-
     /**
      * Returns true if a document with the given "shardKey" is owned by the shard with the given
      * "shardId" in this routing table. If "shardKey" is empty returns false. If "shardKey" is not a
@@ -580,7 +616,7 @@ public:
      * Finds the shard IDs for a given filter and collation. If collation is empty, we use the
      * collection default collation for targeting.
      */
-    void getShardIdsForQuery(OperationContext* opCtx,
+    void getShardIdsForQuery(boost::intrusive_ptr<ExpressionContext> expCtx,
                              const BSONObj& query,
                              const BSONObj& collation,
                              std::set<ShardId>* shardIds) const;
@@ -597,14 +633,14 @@ public:
      * Returns the ids of all shards on which the collection has any chunks.
      */
     void getAllShardIds(std::set<ShardId>* all) const {
-        _rt->getAllShardIds(all);
+        _rt->optRt->getAllShardIds(all);
     }
 
     /**
      * Returns the number of shards on which the collection has any chunks
      */
-    int getNShardsOwningChunks() {
-        return _rt->getNShardsOwningChunks();
+    int getNShardsOwningChunks() const {
+        return _rt->optRt->getNShardsOwningChunks();
     }
 
     // Transforms query into bounds for each field in the shard key
@@ -626,31 +662,45 @@ public:
     //   =>  { a: (0, 1), (2, 3), b: (0, 1), (2, 3) }
     static IndexBounds collapseQuerySolution(const QuerySolutionNode* node);
 
+    /**
+     * Constructs a new ChunkManager, which is a view of the underlying routing table at a different
+     * `clusterTime`.
+     */
+    static ChunkManager makeAtTime(const ChunkManager& cm, Timestamp clusterTime);
+
     /**
      * Returns true if, for this shard, the chunks are identical in both chunk managers
      */
     bool compatibleWith(const ChunkManager& other, const ShardId& shard) const {
-        return _rt->compatibleWith(*other._rt, shard);
+        return _rt->optRt->compatibleWith(*other._rt->optRt, shard);
     }
 
-    std::string toString() const {
-        return _rt->toString();
+    bool uuidMatches(const UUID& uuid) const {
+        return _rt->optRt->uuidMatches(uuid);
     }
 
-    bool uuidMatches(UUID uuid) const {
-        return _rt->uuidMatches(uuid);
+    const UUID& getUUID() const {
+        return _rt->optRt->getUUID();
     }
 
-    auto getRoutingHistory() const {
-        return _rt;
+    const boost::optional<TypeCollectionTimeseriesFields>& getTimeseriesFields() const {
+        return _rt->optRt->getTimeseriesFields();
     }
 
-    boost::optional<UUID> getUUID() const {
-        return _rt->getUUID();
+    const boost::optional<TypeCollectionReshardingFields>& getReshardingFields() const {
+        return _rt->optRt->getReshardingFields();
+    }
+
+    const RoutingTableHistory& getRoutingTableHistory_ForTest() const {
+        return *_rt->optRt;
     }
 
 private:
-    std::shared_ptr<RoutingTableHistory> _rt;
+    ShardId _dbPrimary;
+    DatabaseVersion _dbVersion;
+
+    RoutingTableHistoryValueHandle _rt;
+
     boost::optional<Timestamp> _clusterTime;
 };
 
