@@ -27,8 +27,6 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
 #include <benchmark/benchmark.h>
 
 #include "mongo/base/init.h"
@@ -43,48 +41,71 @@ namespace {
 
 const NamespaceString kNss("test", "foo");
 
-ShardId getShardId(int i) {
-    return {std::string(str::stream() << "shard_" << i)};
+RoutingTableHistoryValueHandle makeStandaloneRoutingTableHistory(RoutingTableHistory rt) {
+    const auto version = rt.getVersion();
+    return RoutingTableHistoryValueHandle(
+        std::make_shared<RoutingTableHistory>(std::move(rt)),
+        ComparableChunkVersion::makeComparableChunkVersion(version));
 }
 
 ChunkRange getRangeForChunk(int i, int nChunks) {
     invariant(i >= 0);
     invariant(nChunks > 0);
     invariant(i < nChunks);
-    auto min = (i == 0) ? BSON("_id" << MINKEY) : BSON("_id" << (i - 1) * 100);
-    auto max = (i == nChunks - 1) ? BSON("_id" << MAXKEY) : BSON("_id" << i * 100);
-    return {std::move(min), std::move(max)};
+    if (i == 0) {
+        return {BSON("_id" << MINKEY), BSON("_id" << 0)};
+    }
+    if (i + 1 == nChunks) {
+        return {BSON("_id" << (i - 1) * 100), BSON("_id" << MAXKEY)};
+    }
+    return {BSON("_id" << (i - 1) * 100), BSON("_id" << i * 100)};
 }
 
 template <typename ShardSelectorFn>
-auto makeChunkManagerWithShardSelector(int nShards, uint32_t nChunks, ShardSelectorFn selectShard) {
+CollectionMetadata makeChunkManagerWithShardSelector(int nShards,
+                                                     uint32_t nChunks,
+                                                     ShardSelectorFn selectShard) {
+    const auto collUuid = UUID::gen();
     const auto collEpoch = OID::gen();
-    const auto collName = NamespaceString("test.foo");
     const auto shardKeyPattern = KeyPattern(BSON("_id" << 1));
 
     std::vector<ChunkType> chunks;
     chunks.reserve(nChunks);
+
     for (uint32_t i = 0; i < nChunks; ++i) {
-        chunks.emplace_back(collName,
+        chunks.emplace_back(collUuid,
                             getRangeForChunk(i, nChunks),
-                            ChunkVersion{i + 1, 0, collEpoch},
+                            ChunkVersion({collEpoch, Timestamp(1, 0)}, {i + 1, 0}),
                             selectShard(i, nShards, nChunks));
     }
 
-    auto routingTableHistory = RoutingTableHistory::makeNew(
-        collName, UUID::gen(), shardKeyPattern, nullptr, true, collEpoch, chunks);
-    auto chunkManager = std::make_shared<ChunkManager>(routingTableHistory, boost::none);
-    return std::make_unique<CollectionMetadata>(std::move(chunkManager), getShardId(0));
+    auto rt = RoutingTableHistory::makeNew(kNss,
+                                           collUuid,
+                                           shardKeyPattern,
+                                           nullptr,
+                                           true,
+                                           collEpoch,
+                                           Timestamp(1, 0),
+                                           boost::none /* timeseriesFields */,
+                                           boost::none,
+                                           boost::none /* chunkSizeBytes */,
+                                           true,
+                                           chunks);
+    return CollectionMetadata(ChunkManager(ShardId("Shard0"),
+                                           DatabaseVersion(UUID::gen(), Timestamp(1, 0)),
+                                           makeStandaloneRoutingTableHistory(std::move(rt)),
+                                           boost::none),
+                              ShardId("shard0"));
 }
 
 ShardId pessimalShardSelector(int i, int nShards, int nChunks) {
-    return getShardId(i % nShards);
+    return ShardId(str::stream() << "shard" << (i % nShards));
 }
 
 ShardId optimalShardSelector(int i, int nShards, int nChunks) {
     invariant(nShards <= nChunks);
     const auto shardNum = (int64_t(i) * nShards / nChunks) % nShards;
-    return getShardId(shardNum);
+    return ShardId(str::stream() << "shard" << shardNum);
 }
 
 MONGO_COMPILER_NOINLINE auto makeChunkManagerWithPessimalBalancedDistribution(int nShards,
@@ -99,235 +120,76 @@ MONGO_COMPILER_NOINLINE auto makeChunkManagerWithOptimalBalancedDistribution(int
 
 MONGO_COMPILER_NOINLINE auto runIncrementalUpdate(const CollectionMetadata& cm,
                                                   const std::vector<ChunkType>& newChunks) {
-    auto rt = cm.getChunkManager()->getRoutingHistory()->makeUpdated(newChunks);
-    return std::make_unique<CollectionMetadata>(std::make_shared<ChunkManager>(rt, boost::none),
-                                                getShardId(0));
+    auto rt = cm.getChunkManager()->getRoutingTableHistory_ForTest().makeUpdated(
+        boost::none /* timeseriesFields */, boost::none, boost::none, true, newChunks);
+    return CollectionMetadata(ChunkManager(ShardId("shard0"),
+                                           DatabaseVersion(UUID::gen(), Timestamp(1, 0)),
+                                           makeStandaloneRoutingTableHistory(std::move(rt)),
+                                           boost::none),
+                              ShardId("shard0"));
 }
 
-void BM_IncrementalRefreshWithNoChange(benchmark::State& state) {
-    const int nShards = state.range(0);
-    const int nChunks = state.range(1);
-    auto cm = makeChunkManagerWithOptimalBalancedDistribution(nShards, nChunks);
-
-    auto postMoveVersion = cm->getChunkManager()->getVersion();
-    const auto collName = NamespaceString(cm->getChunkManager()->getns());
-    std::vector<ChunkType> newChunks;
-    newChunks.emplace_back(collName, getRangeForChunk(0, nChunks), postMoveVersion, getShardId(0));
-
-    for (auto keepRunning : state) {
-        benchmark::DoNotOptimize(runIncrementalUpdate(*cm, newChunks));
-    }
-}
-
-BENCHMARK(BM_IncrementalRefreshWithNoChange)
-    ->Args({1, 1})
-    ->Args({4, 10})
-    ->Args({4, 100})
-    ->Args({4, 1000})
-    ->Args({4, 10000})
-    ->Args({4, 100000});
-
 void BM_IncrementalRefreshOfPessimalBalancedDistribution(benchmark::State& state) {
     const int nShards = state.range(0);
     const int nChunks = state.range(1);
-    auto cm = makeChunkManagerWithPessimalBalancedDistribution(nShards, nChunks);
+    auto metadata = makeChunkManagerWithPessimalBalancedDistribution(nShards, nChunks);
 
-    auto postMoveVersion = cm->getChunkManager()->getVersion();
-    const auto collName = NamespaceString(cm->getChunkManager()->getns());
+    auto postMoveVersion = metadata.getChunkManager()->getVersion();
+    const UUID uuid = metadata.getUUID();
     std::vector<ChunkType> newChunks;
     postMoveVersion.incMajor();
-    newChunks.emplace_back(collName, getRangeForChunk(1, nChunks), postMoveVersion, getShardId(0));
+    newChunks.emplace_back(uuid, getRangeForChunk(1, nChunks), postMoveVersion, ShardId("shard0"));
     postMoveVersion.incMajor();
-    newChunks.emplace_back(collName, getRangeForChunk(3, nChunks), postMoveVersion, getShardId(1));
+    newChunks.emplace_back(uuid, getRangeForChunk(3, nChunks), postMoveVersion, ShardId("shard1"));
 
     for (auto keepRunning : state) {
-        benchmark::DoNotOptimize(runIncrementalUpdate(*cm, newChunks));
+        benchmark::DoNotOptimize(runIncrementalUpdate(metadata, newChunks));
     }
 }
 
-/*
- * Simulate a refresh of the ChunkManager where a number of chunks is migrated from one shard to
- * another.
- *
- * The chunks modified in the routing table are equally spaced.
- */
-void BM_IncrementalSpacedRefreshMoveChunks(benchmark::State& state) {
-    const int nShards = state.range(0);
-    const int nChunks = state.range(1);
-    const int nUpdates = state.range(2);
-    auto cm = makeChunkManagerWithPessimalBalancedDistribution(nShards, nChunks);
-
-    auto lastVersion = cm->getChunkManager()->getVersion();
-
-    std::vector<ChunkType> newChunks;
-    newChunks.reserve(nUpdates);
-    const auto updateSpacing = nChunks / nUpdates;
-    for (int i = 0; i < nUpdates; i++) {
-        const auto idx = i * updateSpacing;
-        lastVersion.incMajor();
-        newChunks.emplace_back(kNss,
-                               getRangeForChunk(idx, nChunks),
-                               lastVersion,
-                               pessimalShardSelector(idx, nShards, nChunks));
-    }
-
-    std::mt19937 g;
-    g.seed(456);
-    std::shuffle(newChunks.begin(), newChunks.end(), g);
-
-    for (auto _ : state) {
-        benchmark::DoNotOptimize(runIncrementalUpdate(*cm, newChunks));
-    }
-}
-
-BENCHMARK(BM_IncrementalSpacedRefreshMoveChunks)
-    ->Args({4, 1, 1})
-    ->Args({4, 10, 1})
-    ->Args({4, 100, 1})
-    ->Args({4, 1000, 1})
-    ->Args({4, 10000, 1})
-    ->Args({4, 100000, 1})
-    ->Args({4, 10000, 10})
-    ->Args({4, 10000, 100})
-    ->Args({4, 10000, 1000})
-    ->Args({4, 10000, 10000});
-
-/*
- * Simulate a refresh of the ChunkManager where a number of chunks is merged together.
- */
-void BM_IncrementalSpacedRefreshMergeChunks(benchmark::State& state) {
-    const int nShards = state.range(0);
-    const int nChunks = state.range(1);
-    const int nUpdates = state.range(2);
-    auto cm = makeChunkManagerWithOptimalBalancedDistribution(nShards, nChunks);
-
-    auto lastVersion = cm->getChunkManager()->getVersion();
-
-    std::vector<ChunkType> newChunks;
-    newChunks.reserve(nUpdates);
-    invariant(nUpdates <= nShards);
-    const auto shardSpacing = nShards / (nUpdates + 1);
-    std::set<ShardId> shardsToMerge;
-    for (int i = 0; i < nUpdates; i++) {
-        invariant(i * shardSpacing <= nShards);
-        shardsToMerge.emplace(getShardId(i * shardSpacing));
-    }
-
-    ShardId shardId;
-    std::vector<ChunkRange> rangesToMerge;
-
-    const auto flushRanges = [&] {
-        if (rangesToMerge.empty()) {
-            return;
-        }
-
-        lastVersion.incMajor();
-        newChunks.emplace_back(
-            kNss,
-            ChunkRange(rangesToMerge.front().getMin(), rangesToMerge.back().getMax()),
-            lastVersion,
-            shardId);
-        rangesToMerge.clear();
-    };
-
-    for (int i = 0; i < nChunks; i++) {
-        auto nextShardId = pessimalShardSelector(i, nShards, nChunks);
-        if (nextShardId != shardId) {
-            flushRanges();
-            shardId = nextShardId;
-        }
-        if (shardsToMerge.count(shardId) == 1) {
-            rangesToMerge.emplace_back(getRangeForChunk(i, nChunks));
-        }
-    }
-    flushRanges();
-
-    std::mt19937 g;
-    g.seed(456);
-    std::shuffle(newChunks.begin(), newChunks.end(), g);
-
-    for (auto _ : state) {
-        benchmark::DoNotOptimize(runIncrementalUpdate(*cm, newChunks));
-    }
-}
-
-/*
- * Simulate chunks merge on a routing table of 10000 chunks partitioned among 4 shards.
- *
- * [   0, 2500)  -> shard0
- * [2500, 5000)  -> shard1
- * [5000, 7500)  -> shard2
- * [7500, 10000) -> shard3
- */
-
-BENCHMARK(BM_IncrementalSpacedRefreshMergeChunks)
-    ->Args({4, 10000, 1})   // merge all chunks on shard2
-    ->Args({4, 10000, 2})   // merge all chunks on shard2 and shard3
-    ->Args({4, 10000, 3})   // merge all chunks on shard1, shard2 and shard3
-    ->Args({4, 10000, 4});  // merge all chunks on shard1, shard2, shard3 and shard4
+BENCHMARK(BM_IncrementalRefreshOfPessimalBalancedDistribution)
+    ->Args({2, 50000})
+    ->Args({2, 250000})
+    ->Args({2, 500000});
 
 template <typename ShardSelectorFn>
 auto BM_FullBuildOfChunkManager(benchmark::State& state, ShardSelectorFn selectShard) {
     const int nShards = state.range(0);
     const uint32_t nChunks = state.range(1);
 
+    const auto collUuid = UUID::gen();
     const auto collEpoch = OID::gen();
-    const auto collName = NamespaceString("test.foo");
     const auto shardKeyPattern = KeyPattern(BSON("_id" << 1));
 
     std::vector<ChunkType> chunks;
     chunks.reserve(nChunks);
+
     for (uint32_t i = 0; i < nChunks; ++i) {
-        chunks.emplace_back(collName,
+        chunks.emplace_back(collUuid,
                             getRangeForChunk(i, nChunks),
-                            ChunkVersion{i + 1, 0, collEpoch},
+                            ChunkVersion({collEpoch, Timestamp(1, 0)}, {i + 1, 0}),
                             selectShard(i, nShards, nChunks));
     }
 
     for (auto keepRunning : state) {
-        auto routingTableHistory = RoutingTableHistory::makeNew(
-            collName, UUID::gen(), shardKeyPattern, nullptr, true, collEpoch, chunks);
-        auto chunkManager = std::make_shared<ChunkManager>(routingTableHistory, boost::none);
-        benchmark::DoNotOptimize(CollectionMetadata(std::move(chunkManager), getShardId(0)));
-    }
-}
-
-template <typename ShardSelectorFn>
-auto BM_GetNextOrphanedRange(benchmark::State& state, ShardSelectorFn selectShard) {
-    const int nShards = state.range(0);
-    const uint32_t nChunks = state.range(1);
-
-    const auto collEpoch = OID::gen();
-    const auto collName = NamespaceString("test.foo");
-    const auto shardKeyPattern = KeyPattern(BSON("_id" << 1));
-
-    std::vector<ChunkType> chunks;
-    chunks.reserve(nChunks);
-    for (uint32_t i = 0; i < nChunks; ++i) {
-        chunks.emplace_back(collName,
-                            getRangeForChunk(i, nChunks),
-                            ChunkVersion{i + 1, 0, collEpoch},
-                            selectShard(i, nShards, nChunks));
-    }
-    auto routingTableHistory = RoutingTableHistory::makeNew(
-        collName, UUID::gen(), shardKeyPattern, nullptr, true, collEpoch, chunks);
-    auto chunkManager = std::make_shared<ChunkManager>(routingTableHistory, boost::none);
-    CollectionMetadata metadata(std::move(chunkManager), getShardId(0));
-    const auto emptyReceivingChunks =
-        RangeMap{SimpleBSONObjComparator::kInstance.makeBSONObjIndexedMap<BSONObj>()};
-
-    for (auto keepRunning : state) {
-        auto startingKey = metadata.getMinKey();
-        auto chunkMap = metadata.getOwnedChunks();
-        while (true) {
-            auto range = metadata.getNextOrphanRange(chunkMap, emptyReceivingChunks, startingKey);
-            if (!range) {
-                break;
-            }
-
-            startingKey = range->getMax();
-            benchmark::DoNotOptimize(range);
-        }
+        auto rt = RoutingTableHistory::makeNew(kNss,
+                                               collUuid,
+                                               shardKeyPattern,
+                                               nullptr,
+                                               true,
+                                               collEpoch,
+                                               Timestamp(1, 0),
+                                               boost::none /* timeseriesFields */,
+                                               boost::none,
+                                               boost::none /* chunkSizeBytes */,
+                                               true,
+                                               chunks);
+        benchmark::DoNotOptimize(
+            CollectionMetadata(ChunkManager(ShardId("shard0"),
+                                            DatabaseVersion(UUID::gen(), Timestamp(1, 0)),
+                                            makeStandaloneRoutingTableHistory(std::move(rt)),
+                                            boost::none),
+                               ShardId("shard0")));
     }
 }
 
@@ -415,13 +277,13 @@ void BM_FindIntersectingChunk(benchmark::State& state,
     const int nShards = state.range(0);
     const int nChunks = state.range(1);
 
-    auto cm = makeCollectionMetadata(nShards, nChunks);
+    auto metadata = makeCollectionMetadata(nShards, nChunks);
     auto keys = makeKeys(nChunks);
     auto keysIter = makeCircularIterator(keys);
 
     for (auto keepRunning : state) {
         benchmark::DoNotOptimize(
-            cm->getChunkManager()->findIntersectingChunkWithSimpleCollation(*keysIter));
+            metadata.getChunkManager()->findIntersectingChunkWithSimpleCollation(*keysIter));
         ++keysIter;
     }
 
@@ -434,14 +296,14 @@ void BM_GetShardIdsForRange(benchmark::State& state,
     const int nShards = state.range(0);
     const int nChunks = state.range(1);
 
-    auto cm = makeCollectionMetadata(nShards, nChunks);
+    auto metadata = makeCollectionMetadata(nShards, nChunks);
     auto keys = makeKeys(nChunks);
     auto ranges = makeRanges(keys);
     auto rangesIter = makeCircularIterator(ranges);
 
     for (auto keepRunning : state) {
         std::set<ShardId> shardIds;
-        cm->getChunkManager()->getShardIdsForRange(
+        metadata.getChunkManager()->getShardIdsForRange(
             rangesIter->first, rangesIter->second, &shardIds);
         ++rangesIter;
     }
@@ -455,13 +317,13 @@ void BM_GetShardIdsForRangeMinKeyToMaxKey(benchmark::State& state,
     const int nShards = state.range(0);
     const int nChunks = state.range(1);
 
-    auto cm = makeCollectionMetadata(nShards, nChunks);
+    auto metadata = makeCollectionMetadata(nShards, nChunks);
     auto min = BSON("_id" << MINKEY);
     auto max = BSON("_id" << MAXKEY);
 
     for (auto keepRunning : state) {
         std::set<ShardId> shardIds;
-        cm->getChunkManager()->getShardIdsForRange(min, max, &shardIds);
+        metadata.getChunkManager()->getShardIdsForRange(min, max, &shardIds);
     }
 
     state.SetItemsProcessed(state.iterations());
@@ -473,14 +335,14 @@ void BM_KeyBelongsToMe(benchmark::State& state,
     const int nShards = state.range(0);
     const int nChunks = state.range(1);
 
-    auto cm = makeCollectionMetadata(nShards, nChunks);
+    auto metadata = makeCollectionMetadata(nShards, nChunks);
     auto keys = makeKeys(nChunks);
     auto keysIter = makeCircularIterator(keys);
 
     size_t nOwned = 0;
 
     for (auto keepRunning : state) {
-        if (cm->keyBelongsToMe(*keysIter)) {
+        if (metadata.keyBelongsToMe(*keysIter)) {
             ++nOwned;
         }
         ++keysIter;
@@ -496,7 +358,7 @@ void BM_RangeOverlapsChunk(benchmark::State& state,
     const int nShards = state.range(0);
     const int nChunks = state.range(1);
 
-    auto cm = makeCollectionMetadata(nShards, nChunks);
+    auto metadata = makeCollectionMetadata(nShards, nChunks);
     auto keys = makeKeys(nChunks);
     auto ranges = makeRanges(keys);
     auto rangesIter = makeCircularIterator(ranges);
@@ -504,7 +366,7 @@ void BM_RangeOverlapsChunk(benchmark::State& state,
     size_t nOverlapped = 0;
 
     for (auto keepRunning : state) {
-        if (cm->rangeOverlapsChunk(ChunkRange(rangesIter->first, rangesIter->second))) {
+        if (metadata.rangeOverlapsChunk(ChunkRange(rangesIter->first, rangesIter->second))) {
             ++nOverlapped;
         }
         ++rangesIter;
@@ -524,8 +386,6 @@ MONGO_INITIALIZER(RegisterBenchmarks)(InitializerContext* context) {
     std::initializer_list<benchmark::internal::Benchmark*> bmCases{
         REGISTER_BENCHMARK_CAPTURE(BM_FullBuildOfChunkManager, Pessimal, pessimalShardSelector),
         REGISTER_BENCHMARK_CAPTURE(BM_FullBuildOfChunkManager, Optimal, optimalShardSelector),
-        REGISTER_BENCHMARK_CAPTURE(BM_GetNextOrphanedRange, Pessimal, pessimalShardSelector),
-        REGISTER_BENCHMARK_CAPTURE(BM_GetNextOrphanedRange, Optimal, optimalShardSelector),
         REGISTER_BENCHMARK_CAPTURE(
             BM_FindIntersectingChunk, Pessimal, makeChunkManagerWithPessimalBalancedDistribution),
         REGISTER_BENCHMARK_CAPTURE(
@@ -551,17 +411,12 @@ MONGO_INITIALIZER(RegisterBenchmarks)(InitializerContext* context) {
     };
 
     for (auto bmCase : bmCases) {
-        bmCase->Args({2, 2})
-            ->Args({1, 10000})
-            ->Args({10, 10000})
-            ->Args({100, 10000})
-            ->Args({1000, 10000})
-            ->Args({10, 10})
-            ->Args({10, 100})
-            ->Args({10, 1000});
+        bmCase->Args({2, 50000})
+            ->Args({10, 50000})
+            ->Args({100, 50000})
+            ->Args({1000, 50000})
+            ->Args({2, 2});
     }
-
-    return Status::OK();
 }
 
 }  // namespace
