@@ -27,9 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
-
-#include "mongo/platform/basic.h"
 
 #include "mongo/s/client/shard_remote.h"
 
@@ -42,10 +39,10 @@
 #include "mongo/client/remote_command_targeter.h"
 #include "mongo/client/replica_set_monitor.h"
 #include "mongo/db/jsobj.h"
-#include "mongo/db/logical_time.h"
 #include "mongo/db/operation_context.h"
-#include "mongo/db/query/query_request.h"
+#include "mongo/db/query/query_request_helper.h"
 #include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/vector_clock.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
@@ -56,6 +53,9 @@
 #include "mongo/util/str.h"
 #include "mongo/util/time_support.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
+
+
 namespace mongo {
 
 using executor::RemoteCommandRequest;
@@ -79,13 +79,13 @@ BSONObj appendMaxTimeToCmdObj(Milliseconds maxTimeMSOverride, const BSONObj& cmd
 
     // Remove the user provided maxTimeMS so we can attach the one from the override
     for (const auto& elem : cmdObj) {
-        if (elem.fieldNameStringData() != QueryRequest::cmdOptionMaxTimeMS) {
+        if (elem.fieldNameStringData() != query_request_helper::cmdOptionMaxTimeMS) {
             updatedCmdBuilder.append(elem);
         }
     }
 
     if (maxTimeMSOverride < Milliseconds::max()) {
-        updatedCmdBuilder.append(QueryRequest::cmdOptionMaxTimeMS,
+        updatedCmdBuilder.append(query_request_helper::cmdOptionMaxTimeMS,
                                  durationCount<Milliseconds>(maxTimeMSOverride));
     }
 
@@ -95,9 +95,9 @@ BSONObj appendMaxTimeToCmdObj(Milliseconds maxTimeMSOverride, const BSONObj& cmd
 }  // unnamed namespace
 
 ShardRemote::ShardRemote(const ShardId& id,
-                         const ConnectionString& originalConnString,
+                         const ConnectionString& connString,
                          std::unique_ptr<RemoteCommandTargeter> targeter)
-    : Shard(id), _originalConnString(originalConnString), _targeter(targeter.release()) {}
+    : Shard(id), _connString(connString), _targeter(std::move(targeter)) {}
 
 ShardRemote::~ShardRemote() = default;
 
@@ -115,6 +115,11 @@ bool ShardRemote::isRetriableError(ErrorCodes::Error code, RetryPolicy options)
             return isMongosRetriableError(code);
         } break;
 
+        case RetryPolicy::kIdempotentOrCursorInvalidated: {
+            return isRetriableError(code, Shard::RetryPolicy::kIdempotent) ||
+                ErrorCodes::isCursorInvalidatedError(code);
+        } break;
+
         case RetryPolicy::kNotIdempotent: {
             return ErrorCodes::isNotPrimaryError(code);
         } break;
@@ -123,43 +128,14 @@ bool ShardRemote::isRetriableError(ErrorCodes::Error code, RetryPolicy options)
     MONGO_UNREACHABLE;
 }
 
-const ConnectionString ShardRemote::getConnString() const {
-    return _targeter->connectionString();
-}
-
 // Any error code changes should possibly also be made to Shard::shouldErrorBePropagated!
 void ShardRemote::updateReplSetMonitor(const HostAndPort& remoteHost,
                                        const Status& remoteCommandStatus) {
-    if (remoteCommandStatus.isOK())
-        return;
-
-    if (ErrorCodes::isNotPrimaryError(remoteCommandStatus.code())) {
-        _targeter->markHostNotMaster(remoteHost, remoteCommandStatus);
-    } else if (ErrorCodes::isNetworkError(remoteCommandStatus.code())) {
-        _targeter->markHostUnreachable(remoteHost, remoteCommandStatus);
-    } else if (remoteCommandStatus == ErrorCodes::NetworkInterfaceExceededTimeLimit) {
-        _targeter->markHostUnreachable(remoteHost, remoteCommandStatus);
-    } else if (ErrorCodes::isShutdownError(remoteCommandStatus.code())) {
-        _targeter->markHostShuttingDown(remoteHost, remoteCommandStatus);
-    }
-}
-
-void ShardRemote::updateLastCommittedOpTime(LogicalTime lastCommittedOpTime) {
-    stdx::lock_guard<Latch> lk(_lastCommittedOpTimeMutex);
-
-    // A secondary may return a lastCommittedOpTime less than the latest seen so far.
-    if (lastCommittedOpTime > _lastCommittedOpTime) {
-        _lastCommittedOpTime = lastCommittedOpTime;
-    }
-}
-
-LogicalTime ShardRemote::getLastCommittedOpTime() const {
-    stdx::lock_guard<Latch> lk(_lastCommittedOpTimeMutex);
-    return _lastCommittedOpTime;
+    _targeter->updateHostWithStatus(remoteHost, remoteCommandStatus);
 }
 
 std::string ShardRemote::toString() const {
-    return getId().toString() + ":" + _originalConnString.toString();
+    return getId().toString() + ":" + _connString.toString();
 }
 
 BSONObj ShardRemote::_appendMetadataForCommand(OperationContext* opCtx,
@@ -285,7 +261,7 @@ StatusWith<Shard::QueryResponse> ShardRemote::_runExhaustiveCursorCommand(
         const auto& data = dataStatus.getValue();
 
         if (data.otherFields.metadata.hasField(rpc::kReplSetMetadataFieldName)) {
-            // Sharding users of ReplSetMetadata do not require the wall clock time field to be set
+            // Sharding users of ReplSetMetadata do not require the wall clock time field to be set.
             auto replParseStatus =
                 rpc::ReplSetMetadata::readFromMetadata(data.otherFields.metadata);
             if (!replParseStatus.isOK()) {
@@ -329,7 +305,17 @@ StatusWith<Shard::QueryResponse> ShardRemote::_runExhaustiveCursorCommand(
         return scheduleStatus;
     }
 
-    fetcher.join();
+    Status joinStatus = fetcher.join(opCtx);
+    if (!joinStatus.isOK()) {
+        if (ErrorCodes::isExceededTimeLimitError(joinStatus.code())) {
+            LOGV2(6195000,
+                  "Operation timed out {error}",
+                  "Operation timed out",
+                  "error"_attr = joinStatus);
+        }
+
+        return joinStatus;
+    }
 
     updateReplSetMonitor(host.getValue(), status);
 
@@ -352,22 +338,31 @@ StatusWith<Shard::QueryResponse> ShardRemote::_exhaustiveFindOnConfig(
     const NamespaceString& nss,
     const BSONObj& query,
     const BSONObj& sort,
-    boost::optional<long long> limit) {
+    boost::optional<long long> limit,
+    const boost::optional<BSONObj>& hint) {
+
     invariant(isConfig());
-    auto const grid = Grid::get(opCtx);
 
-    ReadPreferenceSetting readPrefWithMinOpTime(readPref);
-    readPrefWithMinOpTime.minOpTime = grid->configOpTime();
+    const auto configTime = [&] {
+        const auto currentTime = VectorClock::get(opCtx)->getTime();
+        return currentTime.configTime();
+    }();
 
-    BSONObj readConcernObj;
-    {
+    const auto readPrefWithConfigTime = [&] {
+        ReadPreferenceSetting readPrefToReturn{readPref};
+        readPrefToReturn.minClusterTime = configTime.asTimestamp();
+        return readPrefToReturn;
+    }();
+
+    BSONObj readConcernObj = [&] {
         invariant(readConcernLevel == repl::ReadConcernLevel::kMajorityReadConcern);
-        const repl::ReadConcernArgs readConcern(grid->configOpTime(), readConcernLevel);
+        repl::OpTime configOpTime{configTime.asTimestamp(),
+                                  mongo::repl::OpTime::kUninitializedTerm};
+        repl::ReadConcernArgs readConcern{configOpTime, readConcernLevel};
         BSONObjBuilder bob;
         readConcern.appendInfo(&bob);
-        readConcernObj =
-            bob.done().getObjectField(repl::ReadConcernArgs::kReadConcernFieldName).getOwned();
-    }
+        return bob.done().getObjectField(repl::ReadConcernArgs::kReadConcernFieldName).getOwned();
+    }();
 
     const Milliseconds maxTimeMS =
         std::min(opCtx->getRemainingMaxTimeMillis(),
@@ -377,28 +372,25 @@ StatusWith<Shard::QueryResponse> ShardRemote::_exhaustiveFindOnConfig(
     BSONObjBuilder findCmdBuilder;
 
     {
-        QueryRequest qr(nss);
-        qr.setFilter(query);
-        qr.setSort(sort);
-        qr.setReadConcern(readConcernObj);
-        qr.setLimit(limit);
-
-        if (maxTimeMS < Milliseconds::max()) {
-            qr.setMaxTimeMS(durationCount<Milliseconds>(maxTimeMS));
+        FindCommandRequest findCommand(nss);
+        findCommand.setFilter(query.getOwned());
+        findCommand.setSort(sort.getOwned());
+        findCommand.setReadConcern(readConcernObj.getOwned());
+        findCommand.setLimit(limit ? static_cast<boost::optional<std::int64_t>>(*limit)
+                                   : boost::none);
+        if (hint) {
+            findCommand.setHint(*hint);
         }
 
-        qr.asFindCommand(&findCmdBuilder);
+        if (maxTimeMS < Milliseconds::max()) {
+            findCommand.setMaxTimeMS(durationCount<Milliseconds>(maxTimeMS));
+        }
+
+        findCommand.serialize(BSONObj(), &findCmdBuilder);
     }
 
     return _runExhaustiveCursorCommand(
-        opCtx, readPrefWithMinOpTime, nss.db().toString(), maxTimeMS, findCmdBuilder.done());
-}
-
-Status ShardRemote::createIndexOnConfig(OperationContext* opCtx,
-                                        const NamespaceString& ns,
-                                        const BSONObj& keys,
-                                        bool unique) {
-    MONGO_UNREACHABLE;
+        opCtx, readPrefWithConfigTime, nss.db().toString(), maxTimeMS, findCmdBuilder.done());
 }
 
 void ShardRemote::runFireAndForgetCommand(OperationContext* opCtx,
@@ -415,6 +407,103 @@ void ShardRemote::runFireAndForgetCommand(OperationContext* opCtx,
         .ignore();
 }
 
+Status ShardRemote::runAggregation(
+    OperationContext* opCtx,
+    const AggregateCommandRequest& aggRequest,
+    std::function<bool(const std::vector<BSONObj>& batch,
+                       const boost::optional<BSONObj>& postBatchResumeToken)> callback) {
+
+    BSONObj readPrefMetadata;
+
+    ReadPreferenceSetting readPreference =
+        uassertStatusOK(ReadPreferenceSetting::fromContainingBSON(
+            aggRequest.getUnwrappedReadPref().value_or(BSONObj()),
+            ReadPreference::SecondaryPreferred));
+
+    auto swHost = _targeter->findHost(opCtx, readPreference);
+    if (!swHost.isOK()) {
+        return swHost.getStatus();
+    }
+    HostAndPort host = swHost.getValue();
+
+    BSONObjBuilder builder;
+    readPreference.toContainingBSON(&builder);
+    readPrefMetadata = builder.obj();
+
+    Status status =
+        Status(ErrorCodes::InternalError, "Internal error running cursor callback in command");
+    auto fetcherCallback = [&status, callback](const Fetcher::QueryResponseStatus& dataStatus,
+                                               Fetcher::NextAction* nextAction,
+                                               BSONObjBuilder* getMoreBob) {
+        // Throw out any accumulated results on error
+        if (!dataStatus.isOK()) {
+            status = dataStatus.getStatus();
+            return;
+        }
+
+        const auto& data = dataStatus.getValue();
+
+        if (data.otherFields.metadata.hasField(rpc::kReplSetMetadataFieldName)) {
+            // Sharding users of ReplSetMetadata do not require the wall clock time field to be set.
+            auto replParseStatus =
+                rpc::ReplSetMetadata::readFromMetadata(data.otherFields.metadata);
+            if (!replParseStatus.isOK()) {
+                status = replParseStatus.getStatus();
+                return;
+            }
+        }
+
+        try {
+            boost::optional<BSONObj> postBatchResumeToken =
+                data.documents.empty() ? data.otherFields.postBatchResumeToken : boost::none;
+            if (!callback(data.documents, postBatchResumeToken)) {
+                *nextAction = Fetcher::NextAction::kNoAction;
+            }
+        } catch (...) {
+            status = exceptionToStatus();
+            return;
+        }
+
+        status = Status::OK();
+
+        if (!getMoreBob) {
+            return;
+        }
+        getMoreBob->append("getMore", data.cursorId);
+        getMoreBob->append("collection", data.nss.coll());
+    };
+
+    Milliseconds requestTimeout(-1);
+    if (aggRequest.getMaxTimeMS()) {
+        requestTimeout = Milliseconds(aggRequest.getMaxTimeMS().value_or(0));
+    }
+
+    auto executor = Grid::get(opCtx)->getExecutorPool()->getFixedExecutor();
+    Fetcher fetcher(executor.get(),
+                    host,
+                    aggRequest.getNamespace().db().toString(),
+                    aggregation_request_helper::serializeToCommandObj(aggRequest),
+                    fetcherCallback,
+                    readPrefMetadata,
+                    requestTimeout, /* command network timeout */
+                    requestTimeout /* getMore network timeout */);
+
+    Status scheduleStatus = fetcher.schedule();
+    if (!scheduleStatus.isOK()) {
+        return scheduleStatus;
+    }
+
+    Status joinStatus = fetcher.join(opCtx);
+    if (!joinStatus.isOK()) {
+        return joinStatus;
+    }
+
+    updateReplSetMonitor(host, status);
+
+    return status;
+}
+
+
 StatusWith<ShardRemote::AsyncCmdHandle> ShardRemote::_scheduleCommand(
     OperationContext* opCtx,
     const ReadPreferenceSetting& readPref,
@@ -422,13 +511,19 @@ StatusWith<ShardRemote::AsyncCmdHandle> ShardRemote::_scheduleCommand(
     Milliseconds maxTimeMSOverride,
     const BSONObj& cmdObj,
     const TaskExecutor::RemoteCommandCallbackFn& cb) {
-    ReadPreferenceSetting readPrefWithMinOpTime(readPref);
 
-    if (isConfig()) {
-        readPrefWithMinOpTime.minOpTime = Grid::get(opCtx)->configOpTime();
-    }
+    const auto readPrefWithConfigTime = [&]() -> ReadPreferenceSetting {
+        if (isConfig()) {
+            const auto vcTime = VectorClock::get(opCtx)->getTime();
+            ReadPreferenceSetting readPrefToReturn{readPref};
+            readPrefToReturn.minClusterTime = vcTime.configTime().asTimestamp();
+            return readPrefToReturn;
+        } else {
+            return {readPref};
+        }
+    }();
 
-    const auto swHost = _targeter->findHost(opCtx, readPrefWithMinOpTime);
+    const auto swHost = _targeter->findHost(opCtx, readPrefWithConfigTime);
     if (!swHost.isOK()) {
         return swHost.getStatus();
     }
@@ -443,7 +538,7 @@ StatusWith<ShardRemote::AsyncCmdHandle> ShardRemote::_scheduleCommand(
         asyncHandle.hostTargetted,
         dbName.toString(),
         appendMaxTimeToCmdObj(requestTimeout, cmdObj),
-        _appendMetadataForCommand(opCtx, readPrefWithMinOpTime),
+        _appendMetadataForCommand(opCtx, readPrefWithConfigTime),
         opCtx,
         requestTimeout < Milliseconds::max() ? requestTimeout : RemoteCommandRequest::kNoTimeout);
 
