@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
@@ -36,49 +35,39 @@
 #include "mongo/s/cluster_commands_helpers.h"
 
 #include "mongo/bson/util/bson_extract.h"
+#include "mongo/db/catalog/collection_uuid_mismatch_info.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/error_labels.h"
-#include "mongo/db/logical_clock.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
+#include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/query/cursor_response.h"
 #include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/shard_id.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/write_concern_error_detail.h"
 #include "mongo/s/catalog/type_collection.h"
 #include "mongo/s/catalog_cache.h"
 #include "mongo/s/client/shard_registry.h"
-#include "mongo/s/database_version_helpers.h"
+#include "mongo/s/database_version.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/multi_statement_transaction_requests_sender.h"
-#include "mongo/s/request_types/create_database_gen.h"
-#include "mongo/s/shard_id.h"
+#include "mongo/s/request_types/sharded_ddl_commands_gen.h"
 #include "mongo/s/stale_exception.h"
 #include "mongo/s/transaction_router.h"
 #include "mongo/util/scopeguard.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
+
+
 namespace mongo {
 
-WriteConcernErrorDetail getWriteConcernErrorDetail(const BSONElement& wcErrorElem) {
-    WriteConcernErrorDetail wcError;
-    std::string errMsg;
-    auto wcErrorObj = wcErrorElem.Obj();
-    if (!wcError.parseBSON(wcErrorObj, &errMsg)) {
-        wcError.clear();
-        wcError.setStatus({ErrorCodes::FailedToParse,
-                           "Failed to parse writeConcernError: " + wcErrorObj.toString() +
-                               ", Received error: " + errMsg});
-    }
-
-    return wcError;
-}
-
-void appendWriteConcernErrorToCmdResponse(const ShardId& shardId,
-                                          const BSONElement& wcErrorElem,
-                                          BSONObjBuilder& responseBuilder) {
-    WriteConcernErrorDetail wcError = getWriteConcernErrorDetail(wcErrorElem);
-
+void appendWriteConcernErrorDetailToCmdResponse(const ShardId& shardId,
+                                                WriteConcernErrorDetail wcError,
+                                                BSONObjBuilder& responseBuilder) {
     auto status = wcError.toStatus();
     wcError.setStatus(
         status.withReason(str::stream() << status.reason() << " at " << shardId.toString()));
@@ -86,46 +75,115 @@ void appendWriteConcernErrorToCmdResponse(const ShardId& shardId,
     responseBuilder.append("writeConcernError", wcError.toBSON());
 }
 
-std::unique_ptr<WriteConcernErrorDetail> getWriteConcernErrorDetailFromBSONObj(const BSONObj& obj) {
-    BSONElement wcErrorElem;
-    Status status = bsonExtractTypedField(obj, "writeConcernError", Object, &wcErrorElem);
-    if (!status.isOK()) {
-        if (status == ErrorCodes::NoSuchKey) {
-            return nullptr;
-        } else {
-            uassertStatusOK(status);
-        }
-    }
+void appendWriteConcernErrorToCmdResponse(const ShardId& shardId,
+                                          const BSONElement& wcErrorElem,
+                                          BSONObjBuilder& responseBuilder) {
+    WriteConcernErrorDetail wcError = getWriteConcernErrorDetail(wcErrorElem);
+    appendWriteConcernErrorDetailToCmdResponse(shardId, wcError, responseBuilder);
+}
 
-    return std::make_unique<WriteConcernErrorDetail>(getWriteConcernErrorDetail(wcErrorElem));
+boost::intrusive_ptr<ExpressionContext> makeExpressionContextWithDefaultsForTargeter(
+    OperationContext* opCtx,
+    const NamespaceString& nss,
+    const BSONObj& collation,
+    const boost::optional<ExplainOptions::Verbosity>& verbosity,
+    const boost::optional<BSONObj>& letParameters,
+    const boost::optional<LegacyRuntimeConstants>& runtimeConstants) {
+
+    auto&& cif = [&]() {
+        if (collation.isEmpty()) {
+            return std::unique_ptr<CollatorInterface>{};
+        } else {
+            return uassertStatusOK(
+                CollatorFactoryInterface::get(opCtx->getServiceContext())->makeFromBSON(collation));
+        }
+    }();
+
+    StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces;
+    resolvedNamespaces.emplace(nss.coll(),
+                               ExpressionContext::ResolvedNamespace(nss, std::vector<BSONObj>{}));
+
+    return make_intrusive<ExpressionContext>(
+        opCtx,
+        verbosity,
+        true,   // fromMongos
+        false,  // needs merge
+        false,  // disk use is banned on mongos
+        true,   // bypass document validation, mongos isn't a storage node
+        false,  // not mapReduce
+        nss,
+        runtimeConstants,
+        std::move(cif),
+        MongoProcessInterface::create(opCtx),
+        std::move(resolvedNamespaces),
+        boost::none,  // collection uuid
+        letParameters,
+        false  // mongos has no profile collection
+    );
 }
 
 namespace {
 
-const auto kAllowImplicitCollectionCreation = "allowImplicitCollectionCreation"_sd;
-
 /**
- * Constructs a requests vector targeting each of the specified shard ids. Each request contains the
- * same cmdObj combined with the default sharding parameters.
+ * Consults the routing info to build requests for:
+ * 1. If sharded, shards that own chunks for the namespace, or
+ * 2. If unsharded, the primary shard for the database.
+ *
+ * If a shard is included in shardsToSkip, it will be excluded from the list returned to the
+ * caller.
  */
-std::vector<AsyncRequestsSender::Request> buildUnversionedRequestsForShards(
-    OperationContext* opCtx, std::vector<ShardId> shardIds, const BSONObj& cmdObj) {
+std::vector<AsyncRequestsSender::Request> buildVersionedRequestsForTargetedShards(
+    OperationContext* opCtx,
+    const NamespaceString& nss,
+    const ChunkManager& cm,
+    const std::set<ShardId>& shardsToSkip,
+    const BSONObj& cmdObj,
+    const BSONObj& query,
+    const BSONObj& collation) {
+
     auto cmdToSend = cmdObj;
 
+    if (!cm.isSharded()) {
+        // The collection is unsharded. Target only the primary shard for the database.
+
+        auto primaryShardId = cm.dbPrimary();
+
+        if (shardsToSkip.find(primaryShardId) != shardsToSkip.end()) {
+            return {};
+        }
+
+        // Attach shardVersion "UNSHARDED", unless targeting the config server.
+        const auto cmdObjWithShardVersion = (primaryShardId != ShardId::kConfigServerId)
+            ? appendShardVersion(cmdToSend, ChunkVersion::UNSHARDED())
+            : cmdToSend;
+
+        return std::vector<AsyncRequestsSender::Request>{AsyncRequestsSender::Request(
+            std::move(primaryShardId),
+            appendDbVersionIfPresent(cmdObjWithShardVersion, cm.dbVersion()))};
+    }
+
     std::vector<AsyncRequestsSender::Request> requests;
-    for (auto&& shardId : shardIds)
-        requests.emplace_back(std::move(shardId), cmdToSend);
+
+    // The collection is sharded. Target all shards that own chunks that match the query.
+    std::set<ShardId> shardIds;
+    std::unique_ptr<CollatorInterface> collator;
+    if (!collation.isEmpty()) {
+        collator = uassertStatusOK(
+            CollatorFactoryInterface::get(opCtx->getServiceContext())->makeFromBSON(collation));
+    }
+
+    auto expCtx = make_intrusive<ExpressionContext>(opCtx, std::move(collator), nss);
+    cm.getShardIdsForQuery(expCtx, query, collation, &shardIds);
+
+    for (const ShardId& shardId : shardIds) {
+        if (shardsToSkip.find(shardId) == shardsToSkip.end()) {
+            requests.emplace_back(shardId, appendShardVersion(cmdToSend, cm.getVersion(shardId)));
+        }
+    }
 
     return requests;
 }
 
-std::vector<AsyncRequestsSender::Request> buildUnversionedRequestsForAllShards(
-    OperationContext* opCtx, const BSONObj& cmdObj) {
-    std::vector<ShardId> shardIds;
-    Grid::get(opCtx)->shardRegistry()->getAllShardIdsNoReload(&shardIds);
-    return buildUnversionedRequestsForShards(opCtx, std::move(shardIds), cmdObj);
-}
-
 std::vector<AsyncRequestsSender::Response> gatherResponsesImpl(
     OperationContext* opCtx,
     StringData dbName,
@@ -183,12 +241,20 @@ std::vector<AsyncRequestsSender::Response> gatherResponsesImpl(
             if (ErrorCodes::CommandOnShardedViewNotSupportedOnMongod == status) {
                 uassertStatusOK(status);
             }
+
+            if (ErrorCodes::TenantMigrationAborted == status) {
+                uassertStatusOK(status.withContext(
+                    str::stream() << "got TenantMigrationAborted response from shard "
+                                  << response.shardId << " at host "
+                                  << response.shardHostAndPort->toString()));
+            }
         }
         responses.push_back(std::move(response));
     }
 
     return responses;
 }
+
 }  // namespace
 
 std::vector<AsyncRequestsSender::Response> gatherResponses(
@@ -212,11 +278,11 @@ std::vector<AsyncRequestsSender::Response> gatherResponsesNoThrowOnStaleShardVer
 }
 
 BSONObj appendDbVersionIfPresent(BSONObj cmdObj, const CachedDatabaseInfo& dbInfo) {
-    return appendDbVersionIfPresent(std::move(cmdObj), dbInfo.databaseVersion());
+    return appendDbVersionIfPresent(std::move(cmdObj), dbInfo->getVersion());
 }
 
 BSONObj appendDbVersionIfPresent(BSONObj cmdObj, DatabaseVersion dbVersion) {
-    if (databaseVersion::isFixed(dbVersion)) {
+    if (dbVersion.isFixed()) {
         return cmdObj;
     }
     BSONObjBuilder cmdWithDbVersion(std::move(cmdObj));
@@ -224,18 +290,12 @@ BSONObj appendDbVersionIfPresent(BSONObj cmdObj, DatabaseVersion dbVersion) {
     return cmdWithDbVersion.obj();
 }
 
-BSONObj appendShardVersion(BSONObj cmdObj, ChunkVersion version) {
+BSONObj appendShardVersion(BSONObj cmdObj, ShardVersion version) {
     BSONObjBuilder cmdWithVersionBob(std::move(cmdObj));
-    version.appendToCommand(&cmdWithVersionBob);
+    version.serialize(ShardVersion::kShardVersionField, &cmdWithVersionBob);
     return cmdWithVersionBob.obj();
 }
 
-BSONObj appendAllowImplicitCreate(BSONObj cmdObj, bool allow) {
-    BSONObjBuilder newCmdBuilder(std::move(cmdObj));
-    newCmdBuilder.append(kAllowImplicitCollectionCreation, allow);
-    return newCmdBuilder.obj();
-}
-
 BSONObj applyReadWriteConcern(OperationContext* opCtx,
                               bool appendRC,
                               bool appendWC,
@@ -266,6 +326,7 @@ BSONObj applyReadWriteConcern(OperationContext* opCtx,
     BSONObjBuilder output;
     bool seenReadConcern = false;
     bool seenWriteConcern = false;
+    const auto& readConcernArgs = repl::ReadConcernArgs::get(opCtx);
     for (const auto& elem : cmdObj) {
         const auto name = elem.fieldNameStringData();
         if (appendRC && name == repl::ReadConcernArgs::kReadConcernFieldName) {
@@ -275,13 +336,18 @@ BSONObj applyReadWriteConcern(OperationContext* opCtx,
             seenWriteConcern = true;
         }
         if (!output.hasField(name)) {
-            output.append(elem);
+            // If mongos selected atClusterTime, forward it to the shard.
+            if (name == repl::ReadConcernArgs::kReadConcernFieldName &&
+                readConcernArgs.wasAtClusterTimeSelected()) {
+                output.appendElements(readConcernArgs.toBSON());
+            } else {
+                output.append(elem);
+            }
         }
     }
 
     // Finally, add the new read/write concern.
     if (appendRC && !seenReadConcern) {
-        const auto& readConcernArgs = repl::ReadConcernArgs::get(opCtx);
         output.appendElements(readConcernArgs.toBSON());
     }
     if (appendWC && !seenWriteConcern) {
@@ -295,107 +361,36 @@ BSONObj applyReadWriteConcern(OperationContext* opCtx,
                               CommandInvocation* invocation,
                               const BSONObj& cmdObj) {
     const auto& readConcernArgs = repl::ReadConcernArgs::get(opCtx);
-    const auto readConcernSupport = invocation->supportsReadConcern(readConcernArgs.getLevel());
+    const auto readConcernSupport = invocation->supportsReadConcern(
+        readConcernArgs.getLevel(), readConcernArgs.isImplicitDefault());
     return applyReadWriteConcern(opCtx,
                                  readConcernSupport.readConcernSupport.isOK(),
                                  invocation->supportsWriteConcern(),
                                  cmdObj);
 }
 
-BSONObj applyReadWriteConcern(OperationContext* opCtx, BasicCommand* cmd, const BSONObj& cmdObj) {
+BSONObj applyReadWriteConcern(OperationContext* opCtx,
+                              BasicCommandWithReplyBuilderInterface* cmd,
+                              const BSONObj& cmdObj) {
     const auto& readConcernArgs = repl::ReadConcernArgs::get(opCtx);
-    const auto readConcernSupport = cmd->supportsReadConcern(cmdObj, readConcernArgs.getLevel());
+    const auto readConcernSupport = cmd->supportsReadConcern(
+        cmdObj, readConcernArgs.getLevel(), readConcernArgs.isImplicitDefault());
     return applyReadWriteConcern(opCtx,
                                  readConcernSupport.readConcernSupport.isOK(),
                                  cmd->supportsWriteConcern(cmdObj),
                                  cmdObj);
 }
 
-BSONObj stripWriteConcern(const BSONObj& cmdObj) {
-    BSONObjBuilder output;
-    for (const auto& elem : cmdObj) {
-        const auto name = elem.fieldNameStringData();
-        if (name == WriteConcernOptions::kWriteConcernField) {
-            continue;
-        }
-        output.append(elem);
-    }
-    return output.obj();
-}
-
-std::vector<AsyncRequestsSender::Request> buildVersionedRequestsForTargetedShards(
-    OperationContext* opCtx,
-    const NamespaceString& nss,
-    const CachedCollectionRoutingInfo& routingInfo,
-    const std::set<ShardId>& shardsToSkip,
-    const BSONObj& cmdObj,
-    const BSONObj& query,
-    const BSONObj& collation) {
-
-    auto cmdToSend = cmdObj;
-
-    if (!routingInfo.cm()) {
-        // The collection is unsharded. Target only the primary shard for the database.
-
-        const auto primaryShardId = routingInfo.db().primaryId();
-
-        if (shardsToSkip.find(primaryShardId) != shardsToSkip.end()) {
-            return {};
-        }
-
-        // Attach shardVersion "UNSHARDED", unless targeting the config server.
-        const auto cmdObjWithShardVersion = (primaryShardId != "config")
-            ? appendShardVersion(cmdToSend, ChunkVersion::UNSHARDED())
-            : cmdToSend;
-
-        return buildUnversionedRequestsForShards(
-            opCtx,
-            {primaryShardId},
-            appendDbVersionIfPresent(cmdObjWithShardVersion, routingInfo.db()));
-    }
-
-    std::vector<AsyncRequestsSender::Request> requests;
-
-    // The collection is sharded. Target all shards that own chunks that match the query.
-    std::set<ShardId> shardIds;
-    routingInfo.cm()->getShardIdsForQuery(opCtx, query, collation, &shardIds);
-
-    for (const ShardId& shardId : shardIds) {
-        if (shardsToSkip.find(shardId) == shardsToSkip.end()) {
-            requests.emplace_back(
-                shardId, appendShardVersion(cmdToSend, routingInfo.cm()->getVersion(shardId)));
-        }
-    }
-
-    return requests;
-}
-
 std::vector<AsyncRequestsSender::Response> scatterGatherUnversionedTargetAllShards(
     OperationContext* opCtx,
     StringData dbName,
     const BSONObj& cmdObj,
     const ReadPreferenceSetting& readPref,
     Shard::RetryPolicy retryPolicy) {
-    return gatherResponses(
-        opCtx, dbName, readPref, retryPolicy, buildUnversionedRequestsForAllShards(opCtx, cmdObj));
-}
-
-std::vector<AsyncRequestsSender::Response> scatterGatherUnversionedTargetConfigServerAndShards(
-    OperationContext* opCtx,
-    StringData dbName,
-    const BSONObj& cmdObj,
-    const ReadPreferenceSetting& readPref,
-    Shard::RetryPolicy retryPolicy) {
-    auto shardRegistry = Grid::get(opCtx)->shardRegistry();
     std::vector<AsyncRequestsSender::Request> requests;
-    std::vector<ShardId> shardIds;
-    shardRegistry->getAllShardIds(opCtx, &shardIds);
-    for (auto shardId : shardIds)
+    for (auto&& shardId : Grid::get(opCtx)->shardRegistry()->getAllShardIds(opCtx))
         requests.emplace_back(std::move(shardId), cmdObj);
 
-    auto configShardId = shardRegistry->getConfigShard()->getId();
-    requests.emplace_back(std::move(configShardId), cmdObj);
-
     return gatherResponses(opCtx, dbName, readPref, retryPolicy, requests);
 }
 
@@ -403,14 +398,14 @@ std::vector<AsyncRequestsSender::Response> scatterGatherVersionedTargetByRouting
     OperationContext* opCtx,
     StringData dbName,
     const NamespaceString& nss,
-    const CachedCollectionRoutingInfo& routingInfo,
+    const ChunkManager& cm,
     const BSONObj& cmdObj,
     const ReadPreferenceSetting& readPref,
     Shard::RetryPolicy retryPolicy,
     const BSONObj& query,
     const BSONObj& collation) {
     const auto requests = buildVersionedRequestsForTargetedShards(
-        opCtx, nss, routingInfo, {} /* shardsToSkip */, cmdObj, query, collation);
+        opCtx, nss, cm, {} /* shardsToSkip */, cmdObj, query, collation);
 
     return gatherResponses(opCtx, dbName, readPref, retryPolicy, requests);
 }
@@ -420,7 +415,7 @@ scatterGatherVersionedTargetByRoutingTableNoThrowOnStaleShardVersionErrors(
     OperationContext* opCtx,
     StringData dbName,
     const NamespaceString& nss,
-    const CachedCollectionRoutingInfo& routingInfo,
+    const ChunkManager& cm,
     const std::set<ShardId>& shardsToSkip,
     const BSONObj& cmdObj,
     const ReadPreferenceSetting& readPref,
@@ -428,36 +423,12 @@ scatterGatherVersionedTargetByRoutingTableNoThrowOnStaleShardVersionErrors(
     const BSONObj& query,
     const BSONObj& collation) {
     const auto requests = buildVersionedRequestsForTargetedShards(
-        opCtx, nss, routingInfo, shardsToSkip, cmdObj, query, collation);
+        opCtx, nss, cm, shardsToSkip, cmdObj, query, collation);
 
     return gatherResponsesNoThrowOnStaleShardVersionErrors(
         opCtx, dbName, readPref, retryPolicy, requests);
 }
 
-std::vector<AsyncRequestsSender::Response> scatterGatherOnlyVersionIfUnsharded(
-    OperationContext* opCtx,
-    const NamespaceString& nss,
-    const BSONObj& cmdObj,
-    const ReadPreferenceSetting& readPref,
-    Shard::RetryPolicy retryPolicy,
-    const std::set<ErrorCodes::Error>& ignorableErrors) {
-    auto routingInfo =
-        uassertStatusOK(Grid::get(opCtx)->catalogCache()->getCollectionRoutingInfo(opCtx, nss));
-
-    std::vector<AsyncRequestsSender::Request> requests;
-    if (routingInfo.cm()) {
-        // An unversioned request on a sharded collection can cause a shard that has not owned data
-        // for the collection yet to implicitly create the collection without all the collection
-        // options. So, we signal to shards that they should not implicitly create the collection.
-        requests = buildUnversionedRequestsForAllShards(opCtx, cmdObj);
-    } else {
-        requests = buildVersionedRequestsForTargetedShards(
-            opCtx, nss, routingInfo, {} /* shardsToSkip */, cmdObj, BSONObj(), BSONObj());
-    }
-
-    return gatherResponses(opCtx, nss.db(), readPref, retryPolicy, requests);
-}
-
 AsyncRequestsSender::Response executeCommandAgainstDatabasePrimary(
     OperationContext* opCtx,
     StringData dbName,
@@ -465,40 +436,39 @@ AsyncRequestsSender::Response executeCommandAgainstDatabasePrimary(
     const BSONObj& cmdObj,
     const ReadPreferenceSetting& readPref,
     Shard::RetryPolicy retryPolicy) {
+    // Attach shardVersion "UNSHARDED", unless targeting the config server.
+    const auto cmdObjWithShardVersion = (dbInfo->getPrimary() != ShardId::kConfigServerId)
+        ? appendShardVersion(cmdObj, ChunkVersion::UNSHARDED())
+        : cmdObj;
+
     auto responses =
         gatherResponses(opCtx,
                         dbName,
                         readPref,
                         retryPolicy,
-                        buildUnversionedRequestsForShards(
-                            opCtx, {dbInfo.primaryId()}, appendDbVersionIfPresent(cmdObj, dbInfo)));
+                        std::vector<AsyncRequestsSender::Request>{AsyncRequestsSender::Request(
+                            dbInfo->getPrimary(), appendDbVersionIfPresent(cmdObj, dbInfo))});
     return std::move(responses.front());
 }
 
 AsyncRequestsSender::Response executeCommandAgainstShardWithMinKeyChunk(
     OperationContext* opCtx,
     const NamespaceString& nss,
-    const CachedCollectionRoutingInfo& routingInfo,
+    const ChunkManager& cm,
     const BSONObj& cmdObj,
     const ReadPreferenceSetting& readPref,
     Shard::RetryPolicy retryPolicy) {
 
-    const auto query = routingInfo.cm()
-        ? routingInfo.cm()->getShardKeyPattern().getKeyPattern().globalMin()
-        : BSONObj();
+    const auto query =
+        cm.isSharded() ? cm.getShardKeyPattern().getKeyPattern().globalMin() : BSONObj();
 
-    auto responses =
-        gatherResponses(opCtx,
-                        nss.db(),
-                        readPref,
-                        retryPolicy,
-                        buildVersionedRequestsForTargetedShards(opCtx,
-                                                                nss,
-                                                                routingInfo,
-                                                                {} /* shardsToSkip */,
-                                                                cmdObj,
-                                                                query,
-                                                                BSONObj() /* collation */));
+    auto responses = gatherResponses(
+        opCtx,
+        nss.db(),
+        readPref,
+        retryPolicy,
+        buildVersionedRequestsForTargetedShards(
+            opCtx, nss, cm, {} /* shardsToSkip */, cmdObj, query, BSONObj() /* collation */));
     return std::move(responses.front());
 }
 
@@ -507,7 +477,6 @@ RawResponsesResult appendRawResponses(
     std::string* errmsg,
     BSONObjBuilder* output,
     const std::vector<AsyncRequestsSender::Response>& shardResponses) {
-
     std::vector<AsyncRequestsSender::Response> successARSResponses;
     std::vector<std::pair<ShardId, BSONObj>> successResponsesReceived;
     std::vector<std::pair<ShardId, Status>> shardNotFoundErrorsReceived;
@@ -613,63 +582,31 @@ RawResponsesResult appendRawResponses(
     }
 
     // There was an error. Choose the first error as the top-level error.
-    const auto& firstError = genericErrorsReceived.front().second;
-    output->append("code", firstError.code());
-    output->append("codeName", ErrorCodes::errorString(firstError.code()));
-    *errmsg = firstError.reason();
-    return {false, shardsWithSuccessResponses, successARSResponses, firstStaleConfigErrorReceived};
-}
+    auto& firstError = genericErrorsReceived.front().second;
 
-BSONObj extractQuery(const BSONObj& cmdObj) {
-    auto queryElem = cmdObj["query"];
-    if (!queryElem)
-        queryElem = cmdObj["q"];
-
-    if (!queryElem || queryElem.isNull())
-        return BSONObj();
-
-    uassert(ErrorCodes::TypeMismatch,
-            str::stream() << "Illegal type specified for query " << queryElem,
-            queryElem.type() == Object);
-    return queryElem.embeddedObject();
-}
-
-BSONObj extractCollation(const BSONObj& cmdObj) {
-    const auto collationElem = cmdObj["collation"];
-    if (!collationElem)
-        return BSONObj();
-
-    uassert(ErrorCodes::TypeMismatch,
-            str::stream() << "Illegal type specified for collation " << collationElem,
-            collationElem.type() == Object);
-    return collationElem.embeddedObject();
-}
-
-int getUniqueCodeFromCommandResults(const std::vector<Strategy::CommandResult>& results) {
-    int commonErrCode = -1;
-    for (std::vector<Strategy::CommandResult>::const_iterator it = results.begin();
-         it != results.end();
-         ++it) {
-        // Only look at shards with errors.
-        if (!it->result["ok"].trueValue()) {
-            int errCode = it->result["code"].numberInt();
-
-            if (commonErrCode == -1) {
-                commonErrCode = errCode;
-            } else if (commonErrCode != errCode) {
-                // At least two shards with errors disagree on the error code
-                commonErrCode = 0;
+    if (firstError.code() == ErrorCodes::CollectionUUIDMismatch &&
+        !firstError.extraInfo<CollectionUUIDMismatchInfo>()->actualCollection()) {
+        // The first error is a CollectionUUIDMismatchInfo but it doesn't contain an actual
+        // namespace. It's possible that the actual namespace is unsharded, in which case only the
+        // error from the primary shard will contain this information. Iterate through the errors to
+        // see if this is the case.
+        for (const auto& error : genericErrorsReceived) {
+            if (error.second.code() == ErrorCodes::CollectionUUIDMismatch &&
+                error.second.extraInfo<CollectionUUIDMismatchInfo>()->actualCollection()) {
+                firstError = error.second;
+                break;
             }
         }
     }
 
-    // If no error encountered or shards with errors disagree on the error code, return 0
-    if (commonErrCode == -1 || commonErrCode == 0) {
-        return 0;
+    output->append("code", firstError.code());
+    output->append("codeName", ErrorCodes::errorString(firstError.code()));
+    if (auto extra = firstError.extraInfo()) {
+        extra->serialize(output);
     }
+    *errmsg = firstError.reason();
 
-    // Otherwise, shards with errors agree on the error code; return that code
-    return commonErrCode;
+    return {false, shardsWithSuccessResponses, successARSResponses, firstStaleConfigErrorReceived};
 }
 
 bool appendEmptyResultSet(OperationContext* opCtx,
@@ -682,11 +619,8 @@ bool appendEmptyResultSet(OperationContext* opCtx,
     CurOp::get(opCtx)->debug().nShards = 0;
 
     if (status == ErrorCodes::NamespaceNotFound) {
-        // Old style reply
-        result << "result" << BSONArray();
-
         // New (command) style reply
-        appendCursorResponseObject(0LL, ns, BSONArray(), &result);
+        appendCursorResponseObject(0LL, ns, BSONArray(), boost::none, &result);
 
         return true;
     }
@@ -695,66 +629,32 @@ bool appendEmptyResultSet(OperationContext* opCtx,
     return true;
 }
 
-void createShardDatabase(OperationContext* opCtx, StringData dbName) {
-    auto dbStatus = Grid::get(opCtx)->catalogCache()->getDatabase(opCtx, dbName);
-
-    if (dbStatus == ErrorCodes::NamespaceNotFound) {
-        ConfigsvrCreateDatabase configCreateDatabaseRequest(dbName.toString());
-        configCreateDatabaseRequest.setDbName(NamespaceString::kAdminDb);
-
-        auto configShard = Grid::get(opCtx)->shardRegistry()->getConfigShard();
-
-        auto createDbResponse = uassertStatusOK(configShard->runCommandWithFixedRetryAttempts(
-            opCtx,
-            ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-            "admin",
-            CommandHelpers::appendMajorityWriteConcern(configCreateDatabaseRequest.toBSON({})),
-            Shard::RetryPolicy::kIdempotent));
-
-        uassertStatusOK(createDbResponse.writeConcernStatus);
-
-        if (createDbResponse.commandStatus != ErrorCodes::NamespaceExists) {
-            uassertStatusOKWithContext(createDbResponse.commandStatus,
-                                       str::stream()
-                                           << "Database " << dbName << " could not be created");
-        }
-
-        dbStatus = Grid::get(opCtx)->catalogCache()->getDatabase(opCtx, dbName);
-
-        if (auto txnRouter = TransactionRouter::get(opCtx)) {
-            txnRouter.annotateCreatedDatabase(dbName);
-        }
-    }
-
-    uassertStatusOKWithContext(dbStatus, str::stream() << "Database " << dbName << " not found");
-}
-
-std::set<ShardId> getTargetedShardsForQuery(OperationContext* opCtx,
-                                            const CachedCollectionRoutingInfo& routingInfo,
+std::set<ShardId> getTargetedShardsForQuery(boost::intrusive_ptr<ExpressionContext> expCtx,
+                                            const ChunkManager& cm,
                                             const BSONObj& query,
                                             const BSONObj& collation) {
-    if (routingInfo.cm()) {
-        // The collection is sharded. Use the routing table to decide which shards to target
-        // based on the query and collation.
+    if (cm.isSharded()) {
+        // The collection is sharded. Use the routing table to decide which shards to target based
+        // on the query and collation.
         std::set<ShardId> shardIds;
-        routingInfo.cm()->getShardIdsForQuery(opCtx, query, collation, &shardIds);
+        cm.getShardIdsForQuery(expCtx, query, collation, &shardIds);
         return shardIds;
     }
 
     // The collection is unsharded. Target only the primary shard for the database.
-    return {routingInfo.db().primaryId()};
+    return {cm.dbPrimary()};
 }
 
 std::vector<std::pair<ShardId, BSONObj>> getVersionedRequestsForTargetedShards(
     OperationContext* opCtx,
     const NamespaceString& nss,
-    const CachedCollectionRoutingInfo& routingInfo,
+    const ChunkManager& cm,
     const BSONObj& cmdObj,
     const BSONObj& query,
     const BSONObj& collation) {
     std::vector<std::pair<ShardId, BSONObj>> requests;
     auto ars_requests = buildVersionedRequestsForTargetedShards(
-        opCtx, nss, routingInfo, {} /* shardsToSkip */, cmdObj, query, collation);
+        opCtx, nss, cm, {} /* shardsToSkip */, cmdObj, query, collation);
     std::transform(std::make_move_iterator(ars_requests.begin()),
                    std::make_move_iterator(ars_requests.end()),
                    std::back_inserter(requests),
@@ -765,11 +665,17 @@ std::vector<std::pair<ShardId, BSONObj>> getVersionedRequestsForTargetedShards(
     return requests;
 }
 
-StatusWith<CachedCollectionRoutingInfo> getCollectionRoutingInfoForTxnCmd(
-    OperationContext* opCtx, const NamespaceString& nss) {
+StatusWith<ChunkManager> getCollectionRoutingInfoForTxnCmd(OperationContext* opCtx,
+                                                           const NamespaceString& nss) {
     auto catalogCache = Grid::get(opCtx)->catalogCache();
     invariant(catalogCache);
 
+    auto argsAtClusterTime = repl::ReadConcernArgs::get(opCtx).getArgsAtClusterTime();
+    if (argsAtClusterTime) {
+        return catalogCache->getCollectionRoutingInfoAt(
+            opCtx, nss, argsAtClusterTime->asTimestamp());
+    }
+
     // Return the latest routing table if not running in a transaction with snapshot level read
     // concern.
     auto txnRouter = TransactionRouter::get(opCtx);
@@ -783,29 +689,28 @@ StatusWith<CachedCollectionRoutingInfo> getCollectionRoutingInfoForTxnCmd(
 
 StatusWith<Shard::QueryResponse> loadIndexesFromAuthoritativeShard(OperationContext* opCtx,
                                                                    const NamespaceString& nss) {
-    const auto routingInfo =
+    const auto cm =
         uassertStatusOK(Grid::get(opCtx)->catalogCache()->getCollectionRoutingInfo(opCtx, nss));
 
     auto [indexShard, listIndexesCmd] = [&]() -> std::pair<std::shared_ptr<Shard>, BSONObj> {
         auto cmdNoVersion = applyReadWriteConcern(
             opCtx, true /* appendRC */, false /* appendWC */, BSON("listIndexes" << nss.coll()));
-        if (routingInfo.cm()) {
+        if (cm.isSharded()) {
             // For a sharded collection we must load indexes from a shard with chunks. For
             // consistency with cluster listIndexes, load from the shard that owns the minKey chunk.
-            const auto minKeyShardId = routingInfo.cm()->getMinKeyShardIdWithSimpleCollation();
-            auto minKeyShard =
-                uassertStatusOK(Grid::get(opCtx)->shardRegistry()->getShard(opCtx, minKeyShardId));
-            return {minKeyShard,
-                    appendShardVersion(cmdNoVersion, routingInfo.cm()->getVersion(minKeyShardId))};
+            const auto minKeyShardId = cm.getMinKeyShardIdWithSimpleCollation();
+            return {
+                uassertStatusOK(Grid::get(opCtx)->shardRegistry()->getShard(opCtx, minKeyShardId)),
+                appendShardVersion(cmdNoVersion, cm.getVersion(minKeyShardId))};
         } else {
             // For an unsharded collection, the primary shard will have correct indexes. We attach
             // unsharded shard version to detect if the collection has become sharded.
-            const auto cmdObjWithShardVersion =
-                (routingInfo.db().primaryId() != ShardRegistry::kConfigServerShardId)
+            const auto cmdObjWithShardVersion = (cm.dbPrimary() != ShardId::kConfigServerId)
                 ? appendShardVersion(cmdNoVersion, ChunkVersion::UNSHARDED())
                 : cmdNoVersion;
-            return {routingInfo.db().primary(),
-                    appendDbVersionIfPresent(cmdObjWithShardVersion, routingInfo.db())};
+            return {
+                uassertStatusOK(Grid::get(opCtx)->shardRegistry()->getShard(opCtx, cm.dbPrimary())),
+                appendDbVersionIfPresent(cmdObjWithShardVersion, cm.dbVersion())};
         }
     }();
 
