@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
@@ -35,13 +34,16 @@
 
 #include <boost/intrusive_ptr.hpp>
 
+#include "mongo/db/api_parameters.h"
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/catalog/collection_uuid_mismatch_info.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/curop.h"
-#include "mongo/db/logical_clock.h"
+#include "mongo/db/fle_crud.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/document_source_change_stream.h"
+#include "mongo/db/pipeline/document_source_internal_unpack_bucket.h"
 #include "mongo/db/pipeline/document_source_out.h"
 #include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
@@ -52,10 +54,13 @@
 #include "mongo/db/query/cursor_response.h"
 #include "mongo/db/query/explain_common.h"
 #include "mongo/db/query/find_common.h"
+#include "mongo/db/query/fle/server_rewrite.h"
+#include "mongo/db/timeseries/timeseries_options.h"
 #include "mongo/db/views/resolved_view.h"
 #include "mongo/db/views/view.h"
 #include "mongo/db/write_concern_options.h"
 #include "mongo/executor/task_executor_pool.h"
+#include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/op_msg_rpc_impls.h"
 #include "mongo/s/catalog_cache.h"
@@ -78,6 +83,9 @@
 #include "mongo/s/transaction_router.h"
 #include "mongo/util/net/socket_utils.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
+
+
 namespace mongo {
 
 constexpr unsigned ClusterAggregate::kMaxViewRetries;
@@ -102,10 +110,11 @@ auto resolveInvolvedNamespaces(stdx::unordered_set<NamespaceString> involvedName
 // collection UUID if provided.
 boost::intrusive_ptr<ExpressionContext> makeExpressionContext(
     OperationContext* opCtx,
-    const AggregationRequest& request,
+    const AggregateCommandRequest& request,
     BSONObj collationObj,
     boost::optional<UUID> uuid,
-    StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces) {
+    StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces,
+    bool hasChangeStream) {
 
     std::unique_ptr<CollatorInterface> collation;
     if (!collationObj.isEmpty()) {
@@ -116,16 +125,35 @@ boost::intrusive_ptr<ExpressionContext> makeExpressionContext(
 
     // Create the expression context, and set 'inMongos' to true. We explicitly do *not* set
     // mergeCtx->tempDir.
-    auto mergeCtx =
-        new ExpressionContext(opCtx,
-                              request,
-                              std::move(collation),
-                              std::make_shared<MongosProcessInterface>(
-                                  Grid::get(opCtx)->getExecutorPool()->getArbitraryExecutor()),
-                              std::move(resolvedNamespaces),
-                              uuid);
+    auto mergeCtx = make_intrusive<ExpressionContext>(
+        opCtx,
+        request,
+        std::move(collation),
+        std::make_shared<MongosProcessInterface>(
+            Grid::get(opCtx)->getExecutorPool()->getArbitraryExecutor()),
+        std::move(resolvedNamespaces),
+        uuid);
 
     mergeCtx->inMongos = true;
+
+    // If the request explicity specified NOT to use v2 resume tokens for change streams, set this
+    // on the expCtx. We only ever expect to see an explicit value during testing.
+    if (request.getGenerateV2ResumeTokens().has_value()) {
+        // If $_generateV2ResumeTokens was specified, we must be testing and it must be false.
+        uassert(6528201,
+                "Invalid request for v2 resume tokens",
+                getTestCommandsEnabled() && !request.getGenerateV2ResumeTokens());
+        mergeCtx->changeStreamTokenVersion = 1;
+    }
+
+    // Serialize the 'AggregateCommandRequest' and save it so that the original command can be
+    // reconstructed for dispatch to a new shard, which is sometimes necessary for change streams
+    // pipelines.
+    if (hasChangeStream) {
+        mergeCtx->originalAggregateCommand =
+            aggregation_request_helper::serializeToCommandObj(request);
+    }
+
     return mergeCtx;
 }
 
@@ -142,9 +170,9 @@ void appendEmptyResultSetWithStatus(OperationContext* opCtx,
 
 void updateHostsTargetedMetrics(OperationContext* opCtx,
                                 const NamespaceString& executionNss,
-                                boost::optional<CachedCollectionRoutingInfo> executionNsRoutingInfo,
+                                const boost::optional<ChunkManager>& cm,
                                 stdx::unordered_set<NamespaceString> involvedNamespaces) {
-    if (!executionNsRoutingInfo)
+    if (!cm)
         return;
 
     // Create a set of ShardIds that own a chunk belonging to any of the collections involved in
@@ -153,9 +181,9 @@ void updateHostsTargetedMetrics(OperationContext* opCtx,
     std::set<ShardId> shardsOwningChunks = [&]() {
         std::set<ShardId> shardsIds;
 
-        if (executionNsRoutingInfo->cm()) {
+        if (cm->isSharded()) {
             std::set<ShardId> shardIdsForNs;
-            executionNsRoutingInfo->cm()->getAllShardIds(&shardIdsForNs);
+            cm->getAllShardIds(&shardIdsForNs);
             for (const auto& shardId : shardIdsForNs) {
                 shardsIds.insert(shardId);
             }
@@ -165,11 +193,11 @@ void updateHostsTargetedMetrics(OperationContext* opCtx,
             if (nss == executionNss)
                 continue;
 
-            const auto resolvedNsRoutingInfo =
+            const auto resolvedNsCM =
                 uassertStatusOK(getCollectionRoutingInfoForTxnCmd(opCtx, nss));
-            if (resolvedNsRoutingInfo.cm()) {
+            if (resolvedNsCM.isSharded()) {
                 std::set<ShardId> shardIdsForNs;
-                resolvedNsRoutingInfo.cm()->getAllShardIds(&shardIdsForNs);
+                resolvedNsCM.getAllShardIds(&shardIdsForNs);
                 for (const auto& shardId : shardIdsForNs) {
                     shardsIds.insert(shardId);
                 }
@@ -188,11 +216,57 @@ void updateHostsTargetedMetrics(OperationContext* opCtx,
     }
 }
 
+/**
+ * Performs validations related to API versioning and time-series stages.
+ * Throws UserAssertion if any of the validations fails
+ *     - validation of API versioning on each stage on the pipeline
+ *     - validation of API versioning on 'AggregateCommandRequest' request
+ *     - validation of time-series related stages
+ */
+void performValidationChecks(const OperationContext* opCtx,
+                             const AggregateCommandRequest& request,
+                             const LiteParsedPipeline& liteParsedPipeline) {
+    liteParsedPipeline.validate(opCtx);
+    aggregation_request_helper::validateRequestForAPIVersion(opCtx, request);
+}
+
+/**
+ * Rebuilds the pipeline and uses a different granularity value for the 'bucketMaxSpanSeconds' field
+ * in the $_internalUnpackBucket stage.
+ */
+std::vector<BSONObj> rebuildPipelineWithTimeSeriesGranularity(const std::vector<BSONObj>& pipeline,
+                                                              BucketGranularityEnum granularity) {
+    const auto bucketSpan = timeseries::getMaxSpanSecondsFromGranularity(granularity);
+    std::vector<BSONObj> newPipeline;
+    for (auto& stage : pipeline) {
+        if (stage.firstElementFieldNameStringData() ==
+                DocumentSourceInternalUnpackBucket::kStageNameInternal ||
+            stage.firstElementFieldNameStringData() ==
+                DocumentSourceInternalUnpackBucket::kStageNameExternal) {
+            BSONObjBuilder newOptions;
+            for (auto& elem : stage.firstElement().Obj()) {
+                if (elem.fieldNameStringData() ==
+                    DocumentSourceInternalUnpackBucket::kBucketMaxSpanSeconds) {
+                    newOptions.append(DocumentSourceInternalUnpackBucket::kBucketMaxSpanSeconds,
+                                      bucketSpan);
+                } else {
+                    newOptions.append(elem);
+                }
+            }
+            newPipeline.push_back(
+                BSON(stage.firstElementFieldNameStringData() << newOptions.obj()));
+            continue;
+        }
+        newPipeline.push_back(stage);
+    }
+    return newPipeline;
+}
+
 }  // namespace
 
 Status ClusterAggregate::runAggregate(OperationContext* opCtx,
                                       const Namespaces& namespaces,
-                                      const AggregationRequest& request,
+                                      AggregateCommandRequest& request,
                                       const PrivilegeVector& privileges,
                                       BSONObjBuilder* result) {
     return runAggregate(opCtx, namespaces, request, {request}, privileges, result);
@@ -200,47 +274,83 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
 
 Status ClusterAggregate::runAggregate(OperationContext* opCtx,
                                       const Namespaces& namespaces,
-                                      const AggregationRequest& request,
+                                      AggregateCommandRequest& request,
                                       const LiteParsedPipeline& liteParsedPipeline,
                                       const PrivilegeVector& privileges,
                                       BSONObjBuilder* result) {
-    uassert(51028, "Cannot specify exchange option to a mongos", !request.getExchangeSpec());
+    return runAggregate(
+        opCtx, namespaces, request, liteParsedPipeline, privileges, boost::none, result);
+}
+
+Status ClusterAggregate::runAggregate(OperationContext* opCtx,
+                                      const Namespaces& namespaces,
+                                      AggregateCommandRequest& request,
+                                      const LiteParsedPipeline& liteParsedPipeline,
+                                      const PrivilegeVector& privileges,
+                                      boost::optional<ChunkManager> cm,
+                                      BSONObjBuilder* result) {
+    // Perform some validations on the LiteParsedPipeline and request before continuing with the
+    // aggregation command.
+    performValidationChecks(opCtx, request, liteParsedPipeline);
+
+    uassert(51028, "Cannot specify exchange option to a mongos", !request.getExchange());
     uassert(51143,
             "Cannot specify runtime constants option to a mongos",
-            !request.getRuntimeConstants());
+            !request.getLegacyRuntimeConstants());
     uassert(51089,
-            str::stream() << "Internal parameter(s) [" << AggregationRequest::kNeedsMergeName
-                          << ", " << AggregationRequest::kFromMongosName
+            str::stream() << "Internal parameter(s) ["
+                          << AggregateCommandRequest::kNeedsMergeFieldName << ", "
+                          << AggregateCommandRequest::kFromMongosFieldName
                           << "] cannot be set to 'true' when sent to mongos",
-            !request.needsMerge() && !request.isFromMongos());
+            !request.getNeedsMerge() && !request.getFromMongos());
 
-    const auto isSharded = [](OperationContext* opCtx, const NamespaceString& nss) -> bool {
-        const auto resolvedNsRoutingInfo =
-            uassertStatusOK(getCollectionRoutingInfoForTxnCmd(opCtx, nss));
-        return resolvedNsRoutingInfo.cm().get();
+    const auto isSharded = [](OperationContext* opCtx, const NamespaceString& nss) {
+        const auto resolvedNsCM = uassertStatusOK(getCollectionRoutingInfoForTxnCmd(opCtx, nss));
+        return resolvedNsCM.isSharded();
     };
 
     liteParsedPipeline.verifyIsSupported(
         opCtx, isSharded, request.getExplain(), serverGlobalParams.enableMajorityReadConcern);
     auto hasChangeStream = liteParsedPipeline.hasChangeStream();
     auto involvedNamespaces = liteParsedPipeline.getInvolvedNamespaces();
+    auto shouldDoFLERewrite = ::mongo::shouldDoFLERewrite(request);
+    auto startsWithDocuments = liteParsedPipeline.startsWithDocuments();
 
-    // If the routing table is valid, we obtain a reference to it. If the table is not valid, then
-    // either the database does not exist, or there are no shards in the cluster. In the latter
-    // case, we always return an empty cursor. In the former case, if the requested aggregation is a
-    // $changeStream, we allow the operation to continue so that stream cursors can be established
-    // on the given namespace before the database or collection is actually created. If the database
-    // does not exist and this is not a $changeStream, then we return an empty cursor.
-    boost::optional<CachedCollectionRoutingInfo> routingInfo;
-    auto executionNsRoutingInfoStatus =
-        sharded_agg_helpers::getExecutionNsRoutingInfo(opCtx, namespaces.executionNss);
-    if (executionNsRoutingInfoStatus.isOK()) {
-        routingInfo = std::move(executionNsRoutingInfoStatus.getValue());
-    } else if (!(hasChangeStream &&
-                 executionNsRoutingInfoStatus == ErrorCodes::NamespaceNotFound)) {
-        appendEmptyResultSetWithStatus(
-            opCtx, namespaces.requestedNss, executionNsRoutingInfoStatus.getStatus(), result);
-        return Status::OK();
+    // If the routing table is not already taken by the higher level, fill it now.
+    if (!cm) {
+        // If the routing table is valid, we obtain a reference to it. If the table is not valid,
+        // then either the database does not exist, or there are no shards in the cluster. In the
+        // latter case, we always return an empty cursor. In the former case, if the requested
+        // aggregation is a $changeStream, we allow the operation to continue so that stream cursors
+        // can be established on the given namespace before the database or collection is actually
+        // created. If the database does not exist and this is not a $changeStream, then we return
+        // an empty cursor.
+        auto executionNsRoutingInfoStatus =
+            sharded_agg_helpers::getExecutionNsRoutingInfo(opCtx, namespaces.executionNss);
+
+        if (!executionNsRoutingInfoStatus.isOK()) {
+            uassert(CollectionUUIDMismatchInfo(request.getDbName().toString(),
+                                               *request.getCollectionUUID(),
+                                               request.getNamespace().coll().toString(),
+                                               boost::none),
+                    "Database does not exist",
+                    executionNsRoutingInfoStatus != ErrorCodes::NamespaceNotFound ||
+                        !request.getCollectionUUID());
+
+            if (liteParsedPipeline.startsWithCollStats()) {
+                uassertStatusOKWithContext(executionNsRoutingInfoStatus,
+                                           "Unable to retrieve information for $collStats stage");
+            }
+        }
+
+        if (executionNsRoutingInfoStatus.isOK()) {
+            cm = std::move(executionNsRoutingInfoStatus.getValue());
+        } else if (!((hasChangeStream || startsWithDocuments) &&
+                     executionNsRoutingInfoStatus == ErrorCodes::NamespaceNotFound)) {
+            appendEmptyResultSetWithStatus(
+                opCtx, namespaces.requestedNss, executionNsRoutingInfoStatus.getStatus(), result);
+            return Status::OK();
+        }
     }
 
     boost::intrusive_ptr<ExpressionContext> expCtx;
@@ -253,35 +363,64 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
             // database, the standard logic would attempt to resolve its non-existent UUID and
             // collation by sending a specious 'listCollections' command to the config servers.
             if (hasChangeStream) {
-                return {request.getCollation(), boost::none};
+                return {request.getCollation().value_or(BSONObj()), boost::none};
             }
 
             return cluster_aggregation_planner::getCollationAndUUID(
-                routingInfo, namespaces.executionNss, request.getCollation());
+                opCtx, cm, namespaces.executionNss, request.getCollation().value_or(BSONObj()));
         }();
 
         // Build an ExpressionContext for the pipeline. This instantiates an appropriate collator,
         // resolves all involved namespaces, and creates a shared MongoProcessInterface for use by
         // the pipeline's stages.
-        expCtx = makeExpressionContext(
-            opCtx, request, collationObj, uuid, resolveInvolvedNamespaces(involvedNamespaces));
+        expCtx = makeExpressionContext(opCtx,
+                                       request,
+                                       collationObj,
+                                       uuid,
+                                       resolveInvolvedNamespaces(involvedNamespaces),
+                                       hasChangeStream);
 
         // Parse and optimize the full pipeline.
         auto pipeline = Pipeline::parse(request.getPipeline(), expCtx);
+
+        // If the aggregate command supports encrypted collections, do rewrites of the pipeline to
+        // support querying against encrypted fields.
+        if (shouldDoFLERewrite) {
+            // After this rewriting, the encryption info does not need to be kept around.
+            pipeline = processFLEPipelineS(opCtx,
+                                           namespaces.executionNss,
+                                           request.getEncryptionInformation().value(),
+                                           std::move(pipeline));
+            request.setEncryptionInformation(boost::none);
+        }
+
         pipeline->optimizePipeline();
         return pipeline;
     };
 
+    // The pipeline is not allowed to passthrough if any stage is not allowed to passthrough or if
+    // the pipeline needs to undergo FLE rewriting first.
+    auto allowedToPassthrough =
+        liteParsedPipeline.allowedToPassthroughFromMongos() && !shouldDoFLERewrite;
     auto targeter = cluster_aggregation_planner::AggregationTargeter::make(
         opCtx,
         namespaces.executionNss,
         pipelineBuilder,
-        routingInfo,
+        cm,
         involvedNamespaces,
         hasChangeStream,
-        liteParsedPipeline.allowedToPassthroughFromMongos(),
+        startsWithDocuments,
+        allowedToPassthrough,
         request.getPassthroughToShard().has_value());
 
+    uassert(
+        6487500,
+        fmt::format("Cannot use {} with an aggregation that executes entirely on mongos",
+                    AggregateCommandRequest::kCollectionUUIDFieldName),
+        !request.getCollectionUUID() ||
+            targeter.policy !=
+                cluster_aggregation_planner::AggregationTargeter::TargetingPolicy::kMongosRequired);
+
     if (!expCtx) {
         // When the AggregationTargeter chooses a "passthrough" policy, it does not call the
         // 'pipelineBuilder' function, so we never get an expression context. Because this is a
@@ -290,11 +429,13 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
                       cluster_aggregation_planner::AggregationTargeter::kPassthrough ||
                   targeter.policy ==
                       cluster_aggregation_planner::AggregationTargeter::kSpecificShardOnly);
-        expCtx = make_intrusive<ExpressionContext>(opCtx, nullptr, namespaces.executionNss);
+        expCtx = make_intrusive<ExpressionContext>(
+            opCtx, nullptr, namespaces.executionNss, boost::none, request.getLet());
     }
 
     if (request.getExplain()) {
         explain_common::generateServerInfo(result);
+        explain_common::generateServerParameters(result);
     }
 
     auto status = [&]() {
@@ -305,9 +446,9 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
                 return cluster_aggregation_planner::runPipelineOnPrimaryShard(
                     expCtx,
                     namespaces,
-                    targeter.routingInfo->db(),
+                    *targeter.cm,
                     request.getExplain(),
-                    request.serializeToCommandObj(),
+                    aggregation_request_helper::serializeToCommandDoc(request),
                     privileges,
                     result);
             }
@@ -326,7 +467,8 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
 
                 return cluster_aggregation_planner::runPipelineOnMongoS(
                     namespaces,
-                    request.getBatchSize(),
+                    request.getCursor().getBatchSize().value_or(
+                        aggregation_request_helper::kDefaultBatchSize),
                     std::move(targeter.pipeline),
                     result,
                     privileges);
@@ -336,18 +478,19 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
                 return cluster_aggregation_planner::dispatchPipelineAndMerge(
                     opCtx,
                     std::move(targeter),
-                    request.serializeToCommandObj(),
-                    request.getBatchSize(),
+                    aggregation_request_helper::serializeToCommandDoc(request),
+                    request.getCursor().getBatchSize().value_or(
+                        aggregation_request_helper::kDefaultBatchSize),
                     namespaces,
                     privileges,
                     result,
-                    hasChangeStream);
+                    hasChangeStream,
+                    startsWithDocuments);
             }
             case cluster_aggregation_planner::AggregationTargeter::TargetingPolicy::
                 kSpecificShardOnly: {
                 // Mark expCtx as tailable and await data so CCC behaves accordingly.
                 expCtx->tailableMode = TailableModeEnum::kTailableAndAwaitData;
-                expCtx->forPerShardCursor = true;
 
                 uassert(6273801,
                         "per shard cursor pipeline must contain $changeStream",
@@ -356,23 +499,22 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
                 // Make sure the rest of the pipeline can be pushed down.
                 auto pipeline = request.getPipeline();
                 std::vector<BSONObj> nonChangeStreamPart(pipeline.begin() + 1, pipeline.end());
-                LiteParsedPipeline nonChangeStreamLite(request.getNamespaceString(),
-                                                       nonChangeStreamPart);
+                LiteParsedPipeline nonChangeStreamLite(request.getNamespace(), nonChangeStreamPart);
                 uassert(6273802,
                         "$_passthroughToShard specified with a stage that is not allowed to "
                         "passthrough from mongos",
                         nonChangeStreamLite.allowedToPassthroughFromMongos());
-                ShardId shardId = *request.getPassthroughToShard();
+                ShardId shardId(std::string(request.getPassthroughToShard()->getShard()));
                 uassert(6273803,
                         "$_passthroughToShard not supported for queries against config replica set",
-                        shardId != ShardRegistry::kConfigServerShardId);
+                        shardId != ShardId::kConfigServerId);
 
                 return cluster_aggregation_planner::runPipelineOnSpecificShardOnly(
                     expCtx,
                     namespaces,
                     boost::none,
                     request.getExplain(),
-                    request.serializeToCommandObj(),
+                    aggregation_request_helper::serializeToCommandDoc(request),
                     privileges,
                     shardId,
                     true,
@@ -385,15 +527,21 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
     }();
 
     if (status.isOK()) {
-        updateHostsTargetedMetrics(opCtx, namespaces.executionNss, routingInfo, involvedNamespaces);
+        updateHostsTargetedMetrics(opCtx, namespaces.executionNss, cm, involvedNamespaces);
         // Report usage statistics for each stage in the pipeline.
         liteParsedPipeline.tickGlobalStageCounters();
+
+        // Add 'command' object to explain output.
+        if (expCtx->explain) {
+            explain_common::appendIfRoom(
+                aggregation_request_helper::serializeToCommandObj(request), "command", result);
+        }
     }
     return status;
 }
 
 Status ClusterAggregate::retryOnViewError(OperationContext* opCtx,
-                                          const AggregationRequest& request,
+                                          const AggregateCommandRequest& request,
                                           const ResolvedView& resolvedView,
                                           const NamespaceString& requestedNss,
                                           const PrivilegeVector& privileges,
@@ -405,6 +553,7 @@ Status ClusterAggregate::retryOnViewError(OperationContext* opCtx,
     }
 
     auto resolvedAggRequest = resolvedView.asExpandedViewAggregation(request);
+
     result->resetToEmpty();
 
     if (auto txnRouter = TransactionRouter::get(opCtx)) {
@@ -419,8 +568,27 @@ Status ClusterAggregate::retryOnViewError(OperationContext* opCtx,
     nsStruct.requestedNss = requestedNss;
     nsStruct.executionNss = resolvedView.getNamespace();
 
+    // For a sharded time-series collection, the routing is based on both routing table and the
+    // granularity value. We need to make sure we use the granularity value of the same version as
+    // the routing table, instead of the one attached in the view error. This way the shard
+    // versioning check can correctly catch stale routing information.
+    boost::optional<ChunkManager> snapshotCm;
+    if (nsStruct.executionNss.isTimeseriesBucketsCollection()) {
+        auto executionNsRoutingInfoStatus =
+            sharded_agg_helpers::getExecutionNsRoutingInfo(opCtx, nsStruct.executionNss);
+        if (executionNsRoutingInfoStatus.isOK()) {
+            const auto& cm = executionNsRoutingInfoStatus.getValue();
+            if (cm.isSharded() && cm.getTimeseriesFields()) {
+                const auto patchedPipeline = rebuildPipelineWithTimeSeriesGranularity(
+                    resolvedAggRequest.getPipeline(), cm.getTimeseriesFields()->getGranularity());
+                resolvedAggRequest.setPipeline(patchedPipeline);
+                snapshotCm = cm;
+            }
+        }
+    }
+
     auto status = ClusterAggregate::runAggregate(
-        opCtx, nsStruct, resolvedAggRequest, {resolvedAggRequest}, privileges, result);
+        opCtx, nsStruct, resolvedAggRequest, {resolvedAggRequest}, privileges, snapshotCm, result);
 
     // If the underlying namespace was changed to a view during retry, then re-run the aggregation
     // on the new resolved namespace.
