@@ -26,7 +26,6 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/platform/basic.h"
 
@@ -36,10 +35,12 @@
 #include "mongo/client/connpool.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/pipeline/change_stream_constants.h"
+#include "mongo/db/pipeline/change_stream_invalidation_info.h"
 #include "mongo/db/pipeline/document_source_limit.h"
 #include "mongo/db/pipeline/document_source_skip.h"
 #include "mongo/db/pipeline/sharded_agg_helpers.h"
 #include "mongo/db/query/find_common.h"
+#include "mongo/db/shard_id.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/op_msg_rpc_impls.h"
@@ -55,16 +56,17 @@
 #include "mongo/s/query/router_stage_remove_metadata_fields.h"
 #include "mongo/s/query/router_stage_skip.h"
 #include "mongo/s/query/store_possible_cursor.h"
-#include "mongo/s/shard_id.h"
 #include "mongo/s/shard_key_pattern.h"
 #include "mongo/s/transaction_router.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
+
+
 namespace mongo {
 namespace cluster_aggregation_planner {
 
 MONGO_FAIL_POINT_DEFINE(shardedAggregateFailToDispatchExchangeConsumerPipeline);
 MONGO_FAIL_POINT_DEFINE(shardedAggregateFailToEstablishMergingShardCursor);
-MONGO_FAIL_POINT_DEFINE(shardedAggregateHangBeforeDispatchMergingPipeline);
 
 using sharded_agg_helpers::DispatchShardPipelineResults;
 using sharded_agg_helpers::SplitPipeline;
@@ -102,7 +104,7 @@ AsyncRequestsSender::Response establishMergingShardCursor(OperationContext* opCt
         ReadPreferenceSetting::get(opCtx),
         sharded_agg_helpers::getDesiredRetryPolicy(opCtx));
     const auto response = ars.next();
-    uassert(6273807,
+    tassert(6273807,
             "requested and received data from just one shard, but results are still pending",
             ars.done());
     return response;
@@ -128,10 +130,10 @@ BSONObj createCommandForMergingShard(Document serializedCommand,
     MutableDocument mergeCmd(serializedCommand);
 
     mergeCmd["pipeline"] = Value(pipelineForMerging->serialize());
-    mergeCmd[AggregationRequest::kFromMongosName] = Value(true);
+    mergeCmd[AggregateCommandRequest::kFromMongosFieldName] = Value(true);
 
-    mergeCmd[AggregationRequest::kRuntimeConstants] =
-        Value(mergeCtx->getRuntimeConstants().toBSON());
+    mergeCmd[AggregateCommandRequest::kLetFieldName] =
+        Value(mergeCtx->variablesParseState.serialize(mergeCtx->variables));
 
     // If the user didn't specify a collation already, make sure there's a collation attached to
     // the merge command, since the merging shard may not have the collection metadata.
@@ -141,13 +143,6 @@ BSONObj createCommandForMergingShard(Document serializedCommand,
             : Value(Document{CollationSpec::kSimpleSpec});
     }
 
-    if (mergeCtx->inMongos) {
-        // TODO SERVER-44884: We set this flag to indicate that the shards should always use the new
-        // upsert mechanism when executing relevant $merge modes. After branching for 4.5, supported
-        // upgrade versions will all use the new mechanism, and we can remove this flag.
-        mergeCmd[AggregationRequest::kUseNewUpsert] = Value(true);
-    }
-
     const auto txnRouter = TransactionRouter::get(mergeCtx->opCtx);
     if (txnRouter && mergingShardContributesData) {
         // Don't include a readConcern since we can only include read concerns on the _first_
@@ -172,19 +167,15 @@ Status dispatchMergingPipeline(const boost::intrusive_ptr<ExpressionContext>& ex
                                const ClusterAggregate::Namespaces& namespaces,
                                Document serializedCommand,
                                long long batchSize,
-                               const boost::optional<CachedCollectionRoutingInfo>& routingInfo,
+                               const boost::optional<ChunkManager>& cm,
                                DispatchShardPipelineResults&& shardDispatchResults,
                                BSONObjBuilder* result,
                                const PrivilegeVector& privileges,
                                bool hasChangeStream) {
     // We should never be in a situation where we call this function on a non-merge pipeline.
-    uassert(6525900,
-            "tried to dispatch merge pipeline but the pipeline was not split",
-            shardDispatchResults.splitPipeline);
+    invariant(shardDispatchResults.splitPipeline);
     auto* mergePipeline = shardDispatchResults.splitPipeline->mergePipeline.get();
-    uassert(6525901,
-            "tried to dispatch merge pipeline but there was no merge portion of the split pipeline",
-            mergePipeline);
+    invariant(mergePipeline);
     auto* opCtx = expCtx->opCtx;
 
     std::vector<ShardId> targetedShards;
@@ -193,13 +184,10 @@ Status dispatchMergingPipeline(const boost::intrusive_ptr<ExpressionContext>& ex
         targetedShards.emplace_back(remoteCursor->getShardId().toString());
     }
 
-    sharded_agg_helpers::addMergeCursorsSource(
+    sharded_agg_helpers::partitionAndAddMergeCursorsSource(
         mergePipeline,
-        shardDispatchResults.commandForTargetedShards,
         std::move(shardDispatchResults.remoteCursors),
-        targetedShards,
-        shardDispatchResults.splitPipeline->shardCursorsSortSpec,
-        hasChangeStream);
+        shardDispatchResults.splitPipeline->shardCursorsSortSpec);
 
     // First, check whether we can merge on the mongoS. If the merge pipeline MUST run on mongoS,
     // then ignore the internalQueryProhibitMergingOnMongoS parameter.
@@ -214,12 +202,10 @@ Status dispatchMergingPipeline(const boost::intrusive_ptr<ExpressionContext>& ex
 
     // If we are not merging on mongoS, then this is not a $changeStream aggregation, and we
     // therefore must have a valid routing table.
-    invariant(routingInfo);
+    invariant(cm);
 
-    const ShardId mergingShardId = pickMergingShard(opCtx,
-                                                    shardDispatchResults.needsPrimaryShardMerge,
-                                                    targetedShards,
-                                                    routingInfo->db().primaryId());
+    const ShardId mergingShardId = pickMergingShard(
+        opCtx, shardDispatchResults.needsPrimaryShardMerge, targetedShards, cm->dbPrimary());
     const bool mergingShardContributesData =
         std::find(targetedShards.begin(), targetedShards.end(), mergingShardId) !=
         targetedShards.end();
@@ -229,7 +215,6 @@ Status dispatchMergingPipeline(const boost::intrusive_ptr<ExpressionContext>& ex
 
     LOGV2_DEBUG(22835,
                 1,
-                "Dispatching merge pipeline {command} to designated shard",
                 "Dispatching merge pipeline to designated shard",
                 "command"_attr = redact(mergeCmdObj));
 
@@ -249,13 +234,9 @@ Status dispatchMergingPipeline(const boost::intrusive_ptr<ExpressionContext>& ex
                             privileges,
                             expCtx->tailableMode));
 
-    // If the mergingShard returned an error and did not accept ownership it is our responsibility
-    // to kill the cursors.
-    uassertStatusOK(getStatusFromCommandResult(mergeResponse.swResponse.getValue().data));
-
-    // If we didn't get an error from the merging shard, ownership for the shard cursors has been
-    // transferred to the merging shard. Dismiss the ownership in the current merging pipeline such
-    // that when it goes out of scope it does not attempt to kill the cursors.
+    // Ownership for the shard cursors has been transferred to the merging shard. Dismiss the
+    // ownership in the current merging pipeline such that when it goes out of scope it does not
+    // attempt to kill the cursors.
     auto mergeCursors = static_cast<DocumentSourceMergeCursors*>(mergePipeline->peekFront());
     mergeCursors->dismissCursorOwnership();
 
@@ -268,7 +249,10 @@ BSONObj establishMergingMongosCursor(OperationContext* opCtx,
                                      std::unique_ptr<Pipeline, PipelineDeleter> pipelineForMerging,
                                      const PrivilegeVector& privileges) {
 
-    ClusterClientCursorParams params(requestedNss, ReadPreferenceSetting::get(opCtx));
+    ClusterClientCursorParams params(requestedNss,
+                                     APIParameters::get(opCtx),
+                                     ReadPreferenceSetting::get(opCtx),
+                                     repl::ReadConcernArgs::get(opCtx));
 
     params.originatingCommandObj = CurOp::get(opCtx)->opDescription().getOwned();
     params.tailableMode = pipelineForMerging->getContext()->tailableMode;
@@ -292,20 +276,33 @@ BSONObj establishMergingMongosCursor(OperationContext* opCtx,
     rpc::OpMsgReplyBuilder replyBuilder;
     CursorResponseBuilder::Options options;
     options.isInitialResponse = true;
-
+    if (!opCtx->inMultiDocumentTransaction()) {
+        options.atClusterTime = repl::ReadConcernArgs::get(opCtx).getArgsAtClusterTime();
+    }
     CursorResponseBuilder responseBuilder(&replyBuilder, options);
     bool stashedResult = false;
 
     for (long long objCount = 0; objCount < batchSize; ++objCount) {
         ClusterQueryResult next;
         try {
-            next = uassertStatusOK(ccc->next(RouterExecStage::ExecContext::kInitialFind));
+            next = uassertStatusOK(ccc->next());
         } catch (const ExceptionFor<ErrorCodes::CloseChangeStream>&) {
             // This exception is thrown when a $changeStream stage encounters an event
             // that invalidates the cursor. We should close the cursor and return without
             // error.
             cursorState = ClusterCursorManager::CursorState::Exhausted;
             break;
+        } catch (const ExceptionFor<ErrorCodes::ChangeStreamInvalidated>& ex) {
+            // This exception is thrown when a change-stream cursor is invalidated. Set the PBRT
+            // to the resume token of the invalidating event, and mark the cursor response as
+            // invalidated. We always expect to have ExtraInfo for this error code.
+            const auto extraInfo = ex.extraInfo<ChangeStreamInvalidationInfo>();
+            tassert(5493706, "Missing ChangeStreamInvalidationInfo on exception", extraInfo);
+
+            responseBuilder.setPostBatchResumeToken(extraInfo->getInvalidateResumeToken());
+            responseBuilder.setInvalidated();
+            cursorState = ClusterCursorManager::CursorState::Exhausted;
+            break;
         }
 
         // Check whether we have exhausted the pipeline's results.
@@ -347,14 +344,14 @@ BSONObj establishMergingMongosCursor(OperationContext* opCtx,
     CursorId clusterCursorId = 0;
 
     if (cursorState == ClusterCursorManager::CursorState::NotExhausted) {
-        auto authUsers = AuthorizationSession::get(opCtx->getClient())->getAuthenticatedUserNames();
+        auto authUser = AuthorizationSession::get(opCtx->getClient())->getAuthenticatedUserName();
         clusterCursorId = uassertStatusOK(Grid::get(opCtx)->getCursorManager()->registerCursor(
             opCtx,
             ccc.releaseCursor(),
             requestedNss,
             ClusterCursorManager::CursorType::MultiTarget,
             ClusterCursorManager::CursorLifetime::Mortal,
-            authUsers));
+            authUser));
     }
 
     // Fill out the aggregation metrics in CurOp.
@@ -403,24 +400,22 @@ DispatchShardPipelineResults dispatchExchangeConsumerPipeline(
         auto consumerPipeline = Pipeline::create(
             shardDispatchResults->splitPipeline->mergePipeline->getSources(), expCtx);
 
-        sharded_agg_helpers::addMergeCursorsSource(
+        sharded_agg_helpers::partitionAndAddMergeCursorsSource(
             consumerPipeline.get(),
-            BSONObj(),
             std::move(producers),
-            {},
-            shardDispatchResults->splitPipeline->shardCursorsSortSpec,
-            false);
+            shardDispatchResults->splitPipeline->shardCursorsSortSpec);
 
         consumerPipelines.emplace_back(std::move(consumerPipeline), nullptr, boost::none);
 
-        auto consumerCmdObj = sharded_agg_helpers::createCommandForTargetedShards(
-            expCtx, serializedCommand, consumerPipelines.back(), boost::none, false);
+        auto consumerCmdObj =
+            sharded_agg_helpers::createCommandForTargetedShards(expCtx,
+                                                                serializedCommand,
+                                                                consumerPipelines.back(),
+                                                                boost::none, /* exchangeSpec */
+                                                                false /* needsMerge */);
 
         requests.emplace_back(shardDispatchResults->exchangeSpec->consumerShards[idx],
-                              applyReadWriteConcern(opCtx,
-                                                    true,             /* appendRC */
-                                                    !expCtx->explain, /* appendWC */
-                                                    consumerCmdObj));
+                              consumerCmdObj);
     }
     auto cursors = establishCursors(opCtx,
                                     Grid::get(opCtx)->getExecutorPool()->getArbitraryExecutor(),
@@ -442,11 +437,8 @@ DispatchShardPipelineResults dispatchExchangeConsumerPipeline(
 
     SplitPipeline splitPipeline{nullptr, std::move(mergePipeline), boost::none};
 
-    // Relinquish ownership of the consumer pipelines' cursors. These cursors are now set up to be
-    // merged by a set of $mergeCursors pipelines that we just dispatched to the shards above. Now
-    // that we've established those pipelines on the shards, we are no longer responsible for
-    // ensuring they are cleaned up. If there was a problem establishing the cursors then
-    // establishCursors() would have thrown and mongos would kill all the consumer cursors itself.
+    // Relinquish ownership of the local consumer pipelines' cursors as each shard is now
+    // responsible for its own producer cursors.
     for (const auto& pipeline : consumerPipelines) {
         const auto& mergeCursors =
             static_cast<DocumentSourceMergeCursors*>(pipeline.shardsPipeline->peekFront());
@@ -498,11 +490,13 @@ ClusterClientCursorGuard convertPipelineToRouterStages(
 /**
  * Returns the output of the listCollections command filtered to the namespace 'nss'.
  */
-BSONObj getUnshardedCollInfo(const Shard* primaryShard, const NamespaceString& nss) {
-    ScopedDbConnection conn(primaryShard->getConnString());
+BSONObj getUnshardedCollInfo(OperationContext* opCtx,
+                             const ShardId& shardId,
+                             const NamespaceString& nss) {
+    auto shard = uassertStatusOK(Grid::get(opCtx)->shardRegistry()->getShard(opCtx, shardId));
+    ScopedDbConnection conn(shard->getConnString());
     std::list<BSONObj> all =
         conn->getCollectionInfos(nss.db().toString(), BSON("name" << nss.coll()));
-    conn.done();
     if (all.empty()) {
         // Collection does not exist, return an empty object.
         return BSONObj();
@@ -510,7 +504,6 @@ BSONObj getUnshardedCollInfo(const Shard* primaryShard, const NamespaceString& n
     return all.front();
 }
 
-
 /**
  * Returns the collection default collation or the simple collator if there is no default. If the
  * collection does not exist, then returns an empty BSON Object.
@@ -569,21 +562,22 @@ AggregationTargeter AggregationTargeter::make(
     OperationContext* opCtx,
     const NamespaceString& executionNss,
     const std::function<std::unique_ptr<Pipeline, PipelineDeleter>()> buildPipelineFn,
-    boost::optional<CachedCollectionRoutingInfo> routingInfo,
+    boost::optional<ChunkManager> cm,
     stdx::unordered_set<NamespaceString> involvedNamespaces,
     bool hasChangeStream,
+    bool startsWithDocuments,
     bool allowedToPassthrough,
     bool perShardCursor) {
     if (perShardCursor) {
-        return {TargetingPolicy::kSpecificShardOnly, nullptr, routingInfo};
+        return {TargetingPolicy::kSpecificShardOnly, nullptr, cm};
     }
 
     // Check if any of the involved collections are sharded.
     bool involvesShardedCollections = [&]() {
         for (const auto& nss : involvedNamespaces) {
-            const auto resolvedNsRoutingInfo =
+            const auto resolvedNsCM =
                 uassertStatusOK(getCollectionRoutingInfoForTxnCmd(opCtx, nss));
-            if (resolvedNsRoutingInfo.cm()) {
+            if (resolvedNsCM.isSharded()) {
                 return true;
             }
         }
@@ -592,10 +586,11 @@ AggregationTargeter AggregationTargeter::make(
 
     // Determine whether this aggregation must be dispatched to all shards in the cluster.
     const bool mustRunOnAll =
-        sharded_agg_helpers::mustRunOnAllShards(executionNss, hasChangeStream);
+        sharded_agg_helpers::mustRunOnAllShards(executionNss, hasChangeStream, startsWithDocuments);
 
-    // If we don't have a routing table, then this is a $changeStream which must run on all shards.
-    invariant(routingInfo || (mustRunOnAll && hasChangeStream));
+    // If we don't have a routing table, then this is either a $changeStream which must run on all
+    // shards or a $documents stage which must not.
+    invariant(cm || (mustRunOnAll && hasChangeStream) || (startsWithDocuments && !mustRunOnAll));
 
     // A pipeline is allowed to passthrough to the primary shard iff the following conditions are
     // met:
@@ -607,34 +602,33 @@ AggregationTargeter AggregationTargeter::make(
     //    $currentOp.
     // 4. Doesn't need transformation via DocumentSource::serialize(). For example, list sessions
     //    needs to include information about users that can only be deduced on mongos.
-    if (routingInfo && !routingInfo->cm() && !mustRunOnAll && allowedToPassthrough &&
+    if (cm && !cm->isSharded() && !mustRunOnAll && allowedToPassthrough &&
         !involvesShardedCollections) {
-        return AggregationTargeter{TargetingPolicy::kPassthrough, nullptr, routingInfo};
+        return AggregationTargeter{TargetingPolicy::kPassthrough, nullptr, cm};
     } else {
         auto pipeline = buildPipelineFn();
         auto policy = pipeline->requiredToRunOnMongos() ? TargetingPolicy::kMongosRequired
                                                         : TargetingPolicy::kAnyShard;
-        return AggregationTargeter{policy, std::move(pipeline), routingInfo};
+        return AggregationTargeter{policy, std::move(pipeline), cm};
     }
 }
 
 Status runPipelineOnPrimaryShard(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                  const ClusterAggregate::Namespaces& namespaces,
-                                 const CachedDatabaseInfo& dbInfo,
+                                 const ChunkManager& cm,
                                  boost::optional<ExplainOptions::Verbosity> explain,
                                  Document serializedCommand,
                                  const PrivilegeVector& privileges,
                                  BSONObjBuilder* out) {
-    return runPipelineOnSpecificShardOnly(
-        expCtx,
-        namespaces,
-        boost::optional<DatabaseVersion>(dbInfo.databaseVersion()),
-        explain,
-        serializedCommand,
-        privileges,
-        dbInfo.primary()->getId(),
-        false,
-        out);
+    return runPipelineOnSpecificShardOnly(expCtx,
+                                          namespaces,
+                                          boost::optional<DatabaseVersion>(cm.dbVersion()),
+                                          explain,
+                                          serializedCommand,
+                                          privileges,
+                                          cm.dbPrimary(),
+                                          false,
+                                          out);
 }
 
 Status runPipelineOnMongoS(const ClusterAggregate::Namespaces& namespaces,
@@ -673,11 +667,12 @@ Status dispatchPipelineAndMerge(OperationContext* opCtx,
                                 const ClusterAggregate::Namespaces& namespaces,
                                 const PrivilegeVector& privileges,
                                 BSONObjBuilder* result,
-                                bool hasChangeStream) {
+                                bool hasChangeStream,
+                                bool startsWithDocuments) {
     auto expCtx = targeter.pipeline->getContext();
     // If not, split the pipeline as necessary and dispatch to the relevant shards.
     auto shardDispatchResults = sharded_agg_helpers::dispatchShardPipeline(
-        serializedCommand, hasChangeStream, std::move(targeter.pipeline));
+        serializedCommand, hasChangeStream, startsWithDocuments, std::move(targeter.pipeline));
 
     // If the operation is an explain, then we verify that it succeeded on all targeted
     // shards, write the results to the output builder, and return immediately.
@@ -692,7 +687,9 @@ Status dispatchPipelineAndMerge(OperationContext* opCtx,
 
     // If we sent the entire pipeline to a single shard, store the remote cursor and return.
     if (!shardDispatchResults.splitPipeline) {
-        invariant(shardDispatchResults.remoteCursors.size() == 1);
+        tassert(4457012,
+                "pipeline was split, but more than one remote cursor is present",
+                shardDispatchResults.remoteCursors.size() == 1);
         auto&& remoteCursor = std::move(shardDispatchResults.remoteCursors.front());
         const auto shardId = remoteCursor->getShardId().toString();
         const auto reply = uassertStatusOK(storePossibleCursor(opCtx,
@@ -709,14 +706,12 @@ Status dispatchPipelineAndMerge(OperationContext* opCtx,
             expCtx, namespaces.executionNss, serializedCommand, &shardDispatchResults);
     }
 
-    shardedAggregateHangBeforeDispatchMergingPipeline.pauseWhileSet();
-
     // If we reach here, we have a merge pipeline to dispatch.
     return dispatchMergingPipeline(expCtx,
                                    namespaces,
                                    serializedCommand,
                                    batchSize,
-                                   targeter.routingInfo,
+                                   targeter.cm,
                                    std::move(shardDispatchResults),
                                    result,
                                    privileges,
@@ -724,11 +719,12 @@ Status dispatchPipelineAndMerge(OperationContext* opCtx,
 }
 
 std::pair<BSONObj, boost::optional<UUID>> getCollationAndUUID(
-    const boost::optional<CachedCollectionRoutingInfo>& routingInfo,
+    OperationContext* opCtx,
+    const boost::optional<ChunkManager>& cm,
     const NamespaceString& nss,
     const BSONObj& collation) {
-    const bool collectionIsSharded = (routingInfo && routingInfo->cm());
-    const bool collectionIsNotSharded = (routingInfo && !routingInfo->cm());
+    const bool collectionIsSharded = (cm && cm->isSharded());
+    const bool collectionIsNotSharded = (cm && !cm->isSharded());
 
     // If this is a collectionless aggregation, we immediately return the user-
     // defined collation if one exists, or an empty BSONObj otherwise. Collectionless aggregations
@@ -740,14 +736,13 @@ std::pair<BSONObj, boost::optional<UUID>> getCollationAndUUID(
     }
 
     // If the collection is unsharded, obtain collInfo from the primary shard.
-    const auto unshardedCollInfo = collectionIsNotSharded
-        ? getUnshardedCollInfo(routingInfo->db().primary().get(), nss)
-        : BSONObj();
+    const auto unshardedCollInfo =
+        collectionIsNotSharded ? getUnshardedCollInfo(opCtx, cm->dbPrimary(), nss) : BSONObj();
 
     // Return the collection UUID if available, or boost::none otherwise.
-    const auto getUUID = [&]() -> auto {
+    const auto getUUID = [&]() -> boost::optional<UUID> {
         if (collectionIsSharded) {
-            return routingInfo->cm()->getUUID();
+            return cm->getUUID();
         } else {
             return unshardedCollInfo["info"] && unshardedCollInfo["info"]["uuid"]
                 ? boost::optional<UUID>{uassertStatusOK(
@@ -766,9 +761,8 @@ std::pair<BSONObj, boost::optional<UUID>> getCollationAndUUID(
         if (collectionIsNotSharded) {
             return getDefaultCollationForUnshardedCollection(unshardedCollInfo);
         } else {
-            return routingInfo->cm()->getDefaultCollator()
-                ? routingInfo->cm()->getDefaultCollator()->getSpec().toBSON()
-                : CollationSpec::kSimpleSpec;
+            return cm->getDefaultCollator() ? cm->getDefaultCollator()->getSpec().toBSON()
+                                            : CollationSpec::kSimpleSpec;
         }
     };
 
@@ -791,7 +785,7 @@ Status runPipelineOnSpecificShardOnly(const boost::intrusive_ptr<ExpressionConte
 
     boost::optional<int> overrideBatchSize;
     if (forPerShardCursor) {
-        uassert(6273804,
+        tassert(6273804,
                 "Per shard cursors are supposed to pass fromMongos: false to shards",
                 !expCtx->inMongos);
         // By using an initial batchSize of zero all of the events will get returned through
@@ -801,25 +795,20 @@ Status runPipelineOnSpecificShardOnly(const boost::intrusive_ptr<ExpressionConte
 
     // Format the command for the shard. This wraps the command as an explain if necessary, and
     // rewrites the result into a format safe to forward to shards.
-    BSONObj cmdObj = applyReadWriteConcern(
-        opCtx,
-        true,     /* appendRC */
-        !explain, /* appendWC */
-        CommandHelpers::filterCommandRequestForPassthrough(
-            sharded_agg_helpers::createPassthroughCommandForShard(expCtx,
-                                                                  serializedCommand,
-                                                                  explain,
-                                                                  boost::none,
-                                                                  nullptr, /* pipeline */
-                                                                  BSONObj(),
-                                                                  overrideBatchSize)));
+    BSONObj cmdObj = sharded_agg_helpers::createPassthroughCommandForShard(expCtx,
+                                                                           serializedCommand,
+                                                                           explain,
+                                                                           nullptr, /* pipeline */
+                                                                           BSONObj(),
+                                                                           boost::none,
+                                                                           overrideBatchSize);
 
-    if (!forPerShardCursor && shardId != ShardRegistry::kConfigServerShardId) {
+    if (!forPerShardCursor && shardId != ShardId::kConfigServerId) {
         cmdObj = appendShardVersion(std::move(cmdObj), ChunkVersion::UNSHARDED());
     }
     if (!forPerShardCursor) {
         // Unless this is a per shard cursor, we need to send shard version info.
-        uassert(6377400, "Missing shard versioning information", dbVersion.has_value());
+        tassert(6377400, "Missing shard versioning information", dbVersion.has_value());
         cmdObj = appendDbVersionIfPresent(std::move(cmdObj), *dbVersion);
     }
 
@@ -831,7 +820,7 @@ Status runPipelineOnSpecificShardOnly(const boost::intrusive_ptr<ExpressionConte
         ReadPreferenceSetting::get(opCtx),
         Shard::RetryPolicy::kIdempotent);
     auto response = ars.next();
-    uassert(6273806,
+    tassert(6273806,
             "requested and received data from just one shard, but results are still pending",
             ars.done());
 
