@@ -33,7 +33,6 @@
 
 #include <vector>
 
-#include "mongo/bson/simple_bsonelement_comparator.h"
 #include "mongo/db/field_ref.h"
 #include "mongo/db/field_ref_set.h"
 #include "mongo/db/hasher.h"
@@ -86,20 +85,10 @@ std::vector<std::unique_ptr<FieldRef>> parseShardKeyPattern(const BSONObj& keyPa
 
         // Empty parts of the path, ".."?
         for (size_t i = 0; i < newFieldRef->numParts(); ++i) {
-            const StringData part = newFieldRef->getPart(i);
-
             uassert(ErrorCodes::BadValue,
                     str::stream() << "Field " << patternEl.fieldNameStringData()
                                   << " contains empty parts",
-                    !part.empty());
-
-            // Reject a shard key that has a field name that starts with '$' or contains parts that
-            // start with '$' unless the part is a DBRef (i.e. is equal to '$id', '$db' or '$ref').
-            uassert(ErrorCodes::BadValue,
-                    str::stream() << "Field " << patternEl.fieldNameStringData()
-                                  << " contains parts that start with '$'",
-                    !part.startsWith("$") ||
-                        (i != 0 && (part == "$db" || part == "$id" || part == "$ref")));
+                    !newFieldRef->getPart(i).empty());
         }
 
         // Numeric and ascending (1.0), or "hashed" with exactly hashed field.
@@ -110,7 +99,7 @@ std::vector<std::unique_ptr<FieldRef>> parseShardKeyPattern(const BSONObj& keyPa
                               << " can contain at most one 'hashed' field, and/or multiple "
                                  "numerical fields set to a value of 1. Failed to parse field "
                               << patternEl.fieldNameStringData(),
-                (patternEl.isNumber() && patternEl.numberInt() == 1) ||
+                (patternEl.isNumber() && patternEl.safeNumberInt() == 1) ||
                     (isHashedPattern && numHashedFields == 1));
         parsedPaths.emplace_back(std::move(newFieldRef));
     }
@@ -299,7 +288,7 @@ bool ShardKeyPattern::isShardKey(const BSONObj& shardKey) const {
 }
 
 bool ShardKeyPattern::isExtendedBy(const ShardKeyPattern& newShardKeyPattern) const {
-    return toBSON().isPrefixOf(newShardKeyPattern.toBSON(), SimpleBSONElementComparator::kInstance);
+    return toBSON().isFieldNamePrefixOf(newShardKeyPattern.toBSON());
 }
 
 BSONObj ShardKeyPattern::normalizeShardKey(const BSONObj& shardKey) const {
@@ -392,6 +381,16 @@ BSONObj ShardKeyPattern::extractShardKeyFromDocumentKey(const BSONObj& documentK
     return keyBuilder.obj();
 }
 
+BSONObj ShardKeyPattern::extractShardKeyFromDocumentKeyThrows(const BSONObj& documentKey) const {
+    auto shardKey = extractShardKeyFromDocumentKey(documentKey);
+
+    uassert(ErrorCodes::ShardKeyNotFound,
+            "Shard key cannot contain array values or array descendants.",
+            !shardKey.isEmpty());
+
+    return shardKey;
+}
+
 BSONObj ShardKeyPattern::extractShardKeyFromDoc(const BSONObj& doc) const {
     BSONObjBuilder keyBuilder;
     for (auto&& patternEl : _keyPattern.toBSON()) {
@@ -420,6 +419,16 @@ BSONObj ShardKeyPattern::extractShardKeyFromDoc(const BSONObj& doc) const {
     return keyBuilder.obj();
 }
 
+BSONObj ShardKeyPattern::extractShardKeyFromDocThrows(const BSONObj& doc) const {
+    auto shardKey = extractShardKeyFromDoc(doc);
+
+    uassert(ErrorCodes::ShardKeyNotFound,
+            "Shard key cannot contain array values or array descendants.",
+            !shardKey.isEmpty());
+
+    return shardKey;
+}
+
 BSONObj ShardKeyPattern::extractShardKeyFromOplogEntry(const repl::OplogEntry& entry) const {
     if (!entry.isCrudOpType()) {
         return BSONObj();
@@ -453,13 +462,36 @@ BSONObj ShardKeyPattern::emplaceMissingShardKeyValuesForDocument(const BSONObj d
 StatusWith<BSONObj> ShardKeyPattern::extractShardKeyFromQuery(OperationContext* opCtx,
                                                               const NamespaceString& nss,
                                                               const BSONObj& basicQuery) const {
-    auto qr = std::make_unique<QueryRequest>(nss);
-    qr->setFilter(basicQuery);
+    auto findCommand = std::make_unique<FindCommandRequest>(nss);
+    findCommand->setFilter(basicQuery.getOwned());
 
     const boost::intrusive_ptr<ExpressionContext> expCtx;
     auto statusWithCQ =
         CanonicalQuery::canonicalize(opCtx,
-                                     std::move(qr),
+                                     std::move(findCommand),
+                                     false, /* isExplain */
+                                     expCtx,
+                                     ExtensionsCallbackNoop(),
+                                     MatchExpressionParser::kAllowAllSpecialFeatures);
+    if (!statusWithCQ.isOK()) {
+        return statusWithCQ.getStatus();
+    }
+
+    return extractShardKeyFromQuery(*statusWithCQ.getValue());
+}
+
+StatusWith<BSONObj> ShardKeyPattern::extractShardKeyFromQuery(
+    boost::intrusive_ptr<ExpressionContext> expCtx, const BSONObj& basicQuery) const {
+    auto findCommand = std::make_unique<FindCommandRequest>(expCtx->ns);
+    findCommand->setFilter(basicQuery.getOwned());
+    if (!expCtx->getCollatorBSON().isEmpty()) {
+        findCommand->setCollation(expCtx->getCollatorBSON().getOwned());
+    }
+
+    auto statusWithCQ =
+        CanonicalQuery::canonicalize(expCtx->opCtx,
+                                     std::move(findCommand),
+                                     false, /* isExplain */
                                      expCtx,
                                      ExtensionsCallbackNoop(),
                                      MatchExpressionParser::kAllowAllSpecialFeatures);
@@ -512,12 +544,12 @@ BSONObj ShardKeyPattern::extractShardKeyFromQuery(const CanonicalQuery& query) c
     return keyBuilder.obj();
 }
 
-bool ShardKeyPattern::isUniqueIndexCompatible(const BSONObj& uniqueIndexPattern) const {
-    if (!uniqueIndexPattern.isEmpty() && uniqueIndexPattern.firstElementFieldName() == kIdField) {
+bool ShardKeyPattern::isIndexUniquenessCompatible(const BSONObj& indexPattern) const {
+    if (!indexPattern.isEmpty() && indexPattern.firstElementFieldName() == kIdField) {
         return true;
     }
 
-    return _keyPattern.toBSON().isFieldNamePrefixOf(uniqueIndexPattern);
+    return _keyPattern.toBSON().isFieldNamePrefixOf(indexPattern);
 }
 
 BoundList ShardKeyPattern::flattenBounds(const IndexBounds& indexBounds) const {
@@ -611,4 +643,19 @@ BoundList ShardKeyPattern::flattenBounds(const IndexBounds& indexBounds) const {
     return ret;
 }
 
+size_t ShardKeyPattern::getApproximateSize() const {
+    auto computeVectorSize = [](const std::vector<std::unique_ptr<FieldRef>>& v) {
+        size_t size = 0;
+        for (const auto& ptr : v) {
+            size += sizeof(ptr) + (ptr ? ptr->estimateObjectSizeInBytes() : 0);
+        }
+        return size;
+    };
+
+    auto size = sizeof(ShardKeyPattern);
+    size += _keyPattern.getApproximateSize() - sizeof(KeyPattern);
+    size += computeVectorSize(_keyPatternPaths);
+    return size;
+}
+
 }  // namespace mongo
