@@ -29,18 +29,38 @@
 
 #include "mongo/platform/basic.h"
 
-#include "mongo/s/shard_key_pattern.h"
-
+#include "mongo/db/bson/dotted_path_support.h"
 #include "mongo/db/hasher.h"
 #include "mongo/db/json.h"
-#include "mongo/db/query/query_test_service_context.h"
+#include "mongo/db/service_context_test_fixture.h"
+#include "mongo/s/concurrency/locker_mongos_client_observer.h"
+#include "mongo/s/shard_key_pattern.h"
 #include "mongo/unittest/death_test.h"
-#include "mongo/unittest/unittest.h"
 
 namespace mongo {
 namespace {
 
-using std::string;
+class ShardKeyPatternTest : public ServiceContextTest {
+protected:
+    ShardKeyPatternTest() {
+        auto service = getServiceContext();
+        service->registerClientObserver(std::make_unique<LockerMongosClientObserver>());
+        _opCtxHolder = makeOperationContext();
+        _opCtx = _opCtxHolder.get();
+    }
+
+    BSONObj queryKey(const ShardKeyPattern& pattern, const BSONObj& query) {
+        const NamespaceString nss("foo");
+
+        StatusWith<BSONObj> status = pattern.extractShardKeyFromQuery(_opCtx, nss, query);
+        if (!status.isOK())
+            return BSONObj();
+        return status.getValue();
+    }
+
+    ServiceContext::UniqueOperationContext _opCtxHolder;
+    OperationContext* _opCtx;
+};
 
 /**
  * Creates OplogEntry with given field values.
@@ -50,31 +70,34 @@ repl::OplogEntry makeOplogEntry(repl::OpTime opTime,
                                 NamespaceString nss,
                                 BSONObj oField,
                                 boost::optional<BSONObj> o2Field = boost::none) {
-    return {repl::OplogEntry(opTime,                                  // optime
-                             boost::none,                             // hash
-                             opType,                                  // opType
-                             nss,                                     // namespace
-                             boost::none,                             // uuid
-                             boost::none,                             // fromMigrate
-                             repl::MutableOplogEntry::kOplogVersion,  // version
-                             oField,                                  // o
-                             o2Field,                                 // o2
-                             {},                                      // sessionInfo
-                             boost::none,                             // upsert
-                             Date_t(),                                // wall clock time
-                             {},                                      // statement ids
-                             boost::none,    // optime of previous write within same transaction
-                             boost::none,    // pre-image optime
-                             boost::none)};  // post-image optime
+    return {
+        repl::DurableOplogEntry(opTime,                           // optime
+                                boost::none,                      // hash
+                                opType,                           // opType
+                                nss,                              // namespace
+                                boost::none,                      // uuid
+                                boost::none,                      // fromMigrate
+                                repl::OplogEntry::kOplogVersion,  // version
+                                oField,                           // o
+                                o2Field,                          // o2
+                                {},                               // sessionInfo
+                                boost::none,                      // upsert
+                                Date_t(),                         // wall clock time
+                                {},                               // statement ids
+                                boost::none,    // optime of previous write within same transaction
+                                boost::none,    // pre-image optime
+                                boost::none,    // post-image optime
+                                boost::none,    // ShardId of resharding recipient
+                                boost::none,    // _id
+                                boost::none)};  // needsRetryImage
 }
 
-TEST(ShardKeyPattern, SingleFieldShardKeyPatternsValidityCheck) {
+TEST_F(ShardKeyPatternTest, SingleFieldShardKeyPatternsValidityCheck) {
     ShardKeyPattern s1(BSON("a" << 1));
     ShardKeyPattern s2(BSON("a" << 1.0f));
     ShardKeyPattern s3(BSON("a" << (long long)1L));
     ShardKeyPattern s4(BSON("a"
                             << "hashed"));
-    ShardKeyPattern s5(BSON("a$" << 1));
 
     ASSERT_THROWS(ShardKeyPattern(BSONObj()), DBException);
     ASSERT_THROWS(ShardKeyPattern(BSON("a" << -1)), DBException);
@@ -87,21 +110,12 @@ TEST(ShardKeyPattern, SingleFieldShardKeyPatternsValidityCheck) {
                   DBException);
     ASSERT_THROWS(ShardKeyPattern(BSON("" << 1)), DBException);
     ASSERT_THROWS(ShardKeyPattern(BSON("." << 1)), DBException);
-    ASSERT_THROWS(ShardKeyPattern(BSON("$" << 1)), DBException);
-    ASSERT_THROWS(ShardKeyPattern(BSON("$a" << 1)), DBException);
-    ASSERT_THROWS(ShardKeyPattern(BSON("$**" << 1)), DBException);
-    ASSERT_THROWS(ShardKeyPattern(BSON("$id" << 1)), DBException);
-    ASSERT_THROWS(ShardKeyPattern(BSON("$db" << 1)), DBException);
-    ASSERT_THROWS(ShardKeyPattern(BSON("$ref" << 1)), DBException);
 }
 
-TEST(ShardKeyPattern, CompositeShardKeyPatternsValidityCheck) {
+TEST_F(ShardKeyPatternTest, CompositeShardKeyPatternsValidityCheck) {
     ShardKeyPattern s1(BSON("a" << 1 << "b" << 1));
     ShardKeyPattern s2(BSON("a" << 1.0f << "b" << 1.0));
     ShardKeyPattern s3(BSON("a" << 1 << "b" << 1.0 << "c" << 1.0f));
-    ShardKeyPattern s4(BSON("a.$id" << 1));
-    ShardKeyPattern s5(BSON("a.$db" << 1));
-    ShardKeyPattern s6(BSON("a.$ref" << 1));
 
     ASSERT_THROWS(ShardKeyPattern(BSON("a" << 1 << "b" << -1)), DBException);
     ASSERT_THROWS(ShardKeyPattern(BSON("a" << 1 << "b"
@@ -109,17 +123,13 @@ TEST(ShardKeyPattern, CompositeShardKeyPatternsValidityCheck) {
                   DBException);
     ASSERT_THROWS(ShardKeyPattern(BSON("a" << 1 << "b." << 1.0)), DBException);
     ASSERT_THROWS(ShardKeyPattern(BSON("a" << 1 << "" << 1.0)), DBException);
-    ASSERT_THROWS(ShardKeyPattern(BSON("a" << 1 << "$" << 1.0)), DBException);
-    ASSERT_THROWS(ShardKeyPattern(BSON("a" << 1 << "$b" << 1.0)), DBException);
-    ASSERT_THROWS(ShardKeyPattern(BSON("a" << 1 << "$**" << 1.0)), DBException);
 }
 
-TEST(ShardKeyPattern, NestedShardKeyPatternsValidityCheck) {
+TEST_F(ShardKeyPatternTest, NestedShardKeyPatternsValidtyCheck) {
     ShardKeyPattern s1(BSON("a.b" << 1));
     ShardKeyPattern s2(BSON("a.b.c.d" << 1.0));
     ShardKeyPattern s3(BSON("a" << 1 << "c.d" << 1.0 << "e.f.g" << 1.0f));
     ShardKeyPattern s4(BSON("a" << 1 << "a.b" << 1.0 << "a.b.c" << 1.0f));
-    ShardKeyPattern s6(BSON("a.b$" << 1));
 
     ASSERT_THROWS(ShardKeyPattern(BSON("a.b" << -1)), DBException);
     ASSERT_THROWS(ShardKeyPattern(BSON("a" << BSON("b" << 1))), DBException);
@@ -128,12 +138,9 @@ TEST(ShardKeyPattern, NestedShardKeyPatternsValidityCheck) {
     ASSERT_THROWS(ShardKeyPattern(BSON("a..b" << 1)), DBException);
     ASSERT_THROWS(ShardKeyPattern(BSON("a" << 1 << "a.b." << 1.0)), DBException);
     ASSERT_THROWS(ShardKeyPattern(BSON("a" << BSON("b" << 1) << "c.d" << 1.0)), DBException);
-    ASSERT_THROWS(ShardKeyPattern(BSON("a.$" << 1)), DBException);
-    ASSERT_THROWS(ShardKeyPattern(BSON("a.$b" << 1)), DBException);
-    ASSERT_THROWS(ShardKeyPattern(BSON("a.$**" << 1)), DBException);
 }
 
-TEST(ShardKeyPattern, IsShardKey) {
+TEST_F(ShardKeyPatternTest, IsShardKey) {
     ShardKeyPattern pattern(BSON("a.b" << 1 << "c" << 1.0f));
 
     ASSERT(pattern.isShardKey(BSON("a.b" << 10 << "c" << 30)));
@@ -148,7 +155,7 @@ static BSONObj normKey(const ShardKeyPattern& pattern, const BSONObj& doc) {
     return pattern.normalizeShardKey(doc);
 }
 
-TEST(ShardKeyPattern, NormalizeShardKey) {
+TEST_F(ShardKeyPatternTest, NormalizeShardKey) {
     ShardKeyPattern pattern(BSON("a.b" << 1 << "c" << 1.0f));
 
     ASSERT_BSONOBJ_EQ(normKey(pattern, BSON("a.b" << 10 << "c" << 30)),
@@ -172,7 +179,7 @@ static BSONObj docKeyFromOplog(const ShardKeyPattern& pattern, const repl::Oplog
     return pattern.extractShardKeyFromOplogEntry(entry);
 }
 
-TEST(ShardKeyPattern, ExtractDocShardKeySingle) {
+TEST_F(ShardKeyPatternTest, ExtractDocShardKeySingle) {
     //
     // Single field ShardKeyPatterns
     //
@@ -207,7 +214,7 @@ TEST(ShardKeyPattern, ExtractDocShardKeySingle) {
     // ASSERT_EQUALS(docKey(pattern, BSON("a" << 10 << "a" << 20)), BSONObj());
 }
 
-TEST(ShardKeyPattern, ExtractDocShardKeyCompound) {
+TEST_F(ShardKeyPatternTest, ExtractDocShardKeyCompound) {
     //
     // Compound ShardKeyPatterns
     //
@@ -239,7 +246,7 @@ TEST(ShardKeyPattern, ExtractDocShardKeyCompound) {
     ASSERT_EQUALS(docKey(pattern, BSON("b" << 20 << "a" << 10)).firstElement().numberInt(), 10);
 }
 
-TEST(ShardKeyPattern, ExtractDocShardKeyNested) {
+TEST_F(ShardKeyPatternTest, ExtractDocShardKeyNested) {
     //
     // Nested ShardKeyPatterns
     //
@@ -263,7 +270,7 @@ TEST(ShardKeyPattern, ExtractDocShardKeyNested) {
     ASSERT_BSONOBJ_EQ(docKey(pattern, fromjson("{a:{b:[10, 20]}, c:30}")), BSONObj());
 }
 
-TEST(ShardKeyPattern, ExtractShardKeyFromOplogUnnested) {
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromOplogUnnested) {
     //
     // Unnested ShardKeyPatterns from oplog entries with CRUD operation
     //
@@ -288,7 +295,7 @@ TEST(ShardKeyPattern, ExtractShardKeyFromOplogUnnested) {
     ASSERT_BSONOBJ_EQ(docKeyFromOplog(pattern, updateOplog), fromjson("{a: 7}"));
 }
 
-TEST(ShardKeyPattern, ExtractShardKeyFromOplogNested) {
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromOplogNested) {
     //
     // Nested ShardKeyPatterns from oplog entries with CRUD operation
     //
@@ -313,7 +320,7 @@ TEST(ShardKeyPattern, ExtractShardKeyFromOplogNested) {
     ASSERT_BSONOBJ_EQ(docKeyFromOplog(pattern, updateOplog), fromjson("{'a.b': 7}"));
 }
 
-TEST(ShardKeyPattern, ExtractShardKeyFromOplogNonCRUD) {
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromOplogNonCRUD) {
     //
     // Oplogs with non-CRUD op types
     //
@@ -333,7 +340,7 @@ TEST(ShardKeyPattern, ExtractShardKeyFromOplogNonCRUD) {
     ASSERT_BSONOBJ_EQ(docKeyFromOplog(pattern, commandOplog), BSONObj());
 }
 
-TEST(ShardKeyPattern, ExtractDocShardKeyDeepNested) {
+TEST_F(ShardKeyPatternTest, ExtractDocShardKeyDeepNested) {
     //
     // Deeply nested ShardKeyPatterns
     //
@@ -355,7 +362,7 @@ TEST(ShardKeyPattern, ExtractDocShardKeyDeepNested) {
                       BSONObj());
 }
 
-TEST(ShardKeyPattern, ExtractDocShardKeyWithNumericFieldName) {
+TEST_F(ShardKeyPatternTest, ExtractDocShardKeyWithNumericFieldName) {
     ShardKeyPattern pattern(BSON("a.0.1" << 1));
     ASSERT_BSONOBJ_EQ(docKey(pattern, fromjson("{a:{'0':{'1':{shardKey:1}}}}")),
                       fromjson("{'a.0.1':{shardKey:1}}"));
@@ -377,12 +384,12 @@ TEST(ShardKeyPattern, ExtractDocShardKeyWithNumericFieldName) {
                       fromjson("{'a.0.1':null}"));
 }
 
-TEST(ShardKeyPattern, ExtractDocShardKeyHashed) {
+TEST_F(ShardKeyPatternTest, ExtractDocShardKeyHashed) {
     //
     // Hashed ShardKeyPattern
     //
 
-    const string value = "12345";
+    const std::string value = "12345";
     const BSONObj bsonValue = BSON("" << value);
     const long long hashValue =
         BSONElementHasher::hash64(bsonValue.firstElement(), BSONElementHasher::DEFAULT_HASH_SEED);
@@ -407,18 +414,7 @@ TEST(ShardKeyPattern, ExtractDocShardKeyHashed) {
     ASSERT_BSONOBJ_EQ(docKey(pattern, BSON("a" << BSON_ARRAY(BSON("b" << value)))), BSONObj());
 }
 
-static BSONObj queryKey(const ShardKeyPattern& pattern, const BSONObj& query) {
-    QueryTestServiceContext serviceContext;
-    auto opCtx = serviceContext.makeOperationContext();
-    const NamespaceString& nss{NamespaceString("foo")};
-
-    StatusWith<BSONObj> status = pattern.extractShardKeyFromQuery(opCtx.get(), nss, query);
-    if (!status.isOK())
-        return BSONObj();
-    return status.getValue();
-}
-
-TEST(ShardKeyPattern, ExtractQueryShardKeySingle) {
+TEST_F(ShardKeyPatternTest, ExtractQueryShardKeySingle) {
     //
     // Single field ShardKeyPatterns
     //
@@ -454,7 +450,7 @@ TEST(ShardKeyPattern, ExtractQueryShardKeySingle) {
                       BSONObj());
 }
 
-TEST(ShardKeyPattern, ExtractQueryShardKeyCompound) {
+TEST_F(ShardKeyPatternTest, ExtractQueryShardKeyCompound) {
     //
     // Compound ShardKeyPatterns
     //
@@ -491,7 +487,7 @@ TEST(ShardKeyPattern, ExtractQueryShardKeyCompound) {
     ASSERT_EQUALS(queryKey(pattern, BSON("b" << 20 << "a" << 10)).firstElement().numberInt(), 10);
 }
 
-TEST(ShardKeyPattern, ExtractQueryShardKeyNested) {
+TEST_F(ShardKeyPatternTest, ExtractQueryShardKeyNested) {
     //
     // Nested ShardKeyPatterns
     //
@@ -522,7 +518,7 @@ TEST(ShardKeyPattern, ExtractQueryShardKeyNested) {
     ASSERT_BSONOBJ_EQ(queryKey(pattern, fromjson("{a:{b:{$eq:[10, 20]}},c:30}")), BSONObj());
 }
 
-TEST(ShardKeyPattern, ExtractQueryShardKeyDeepNested) {
+TEST_F(ShardKeyPatternTest, ExtractQueryShardKeyDeepNested) {
     //
     // Deeply nested ShardKeyPatterns
     //
@@ -544,7 +540,7 @@ TEST(ShardKeyPattern, ExtractQueryShardKeyDeepNested) {
                       BSONObj());
 }
 
-TEST(ShardKeyPattern, ExtractQueryShardKeyWithNumericFieldNames) {
+TEST_F(ShardKeyPatternTest, ExtractQueryShardKeyWithNumericFieldNames) {
     ShardKeyPattern pattern(BSON("a.0.0" << 1));
     ASSERT_BSONOBJ_EQ(queryKey(pattern, fromjson("{a:{'0':{'0':10}}}")), fromjson("{'a.0.0':10}"));
     ASSERT_BSONOBJ_EQ(queryKey(pattern, fromjson("{'a.0.0':10}")), fromjson("{'a.0.0':10}"));
@@ -565,12 +561,12 @@ TEST(ShardKeyPattern, ExtractQueryShardKeyWithNumericFieldNames) {
     ASSERT_BSONOBJ_EQ(queryKey(pattern, fromjson("{}")), BSONObj());
 }
 
-TEST(ShardKeyPattern, ExtractQueryShardKeyHashed) {
+TEST_F(ShardKeyPatternTest, ExtractQueryShardKeyHashed) {
     //
     // Hashed ShardKeyPattern
     //
 
-    const string value = "12345";
+    const std::string value = "12345";
     const BSONObj bsonValue = BSON("" << value);
     const long long hashValue =
         BSONElementHasher::hash64(bsonValue.firstElement(), BSONElementHasher::DEFAULT_HASH_SEED);
@@ -611,10 +607,10 @@ TEST(ShardKeyPattern, ExtractQueryShardKeyHashed) {
 }
 
 static bool indexComp(const ShardKeyPattern& pattern, const BSONObj& indexPattern) {
-    return pattern.isUniqueIndexCompatible(indexPattern);
+    return pattern.isIndexUniquenessCompatible(indexPattern);
 }
 
-TEST(ShardKeyPattern, UniqueIndexCompatibleSingle) {
+TEST_F(ShardKeyPatternTest, UniqueIndexCompatibleSingle) {
     //
     // Single field ShardKeyPatterns
     //
@@ -632,7 +628,7 @@ TEST(ShardKeyPattern, UniqueIndexCompatibleSingle) {
     ASSERT(!indexComp(pattern, BSON("b" << -1 << "a" << 1)));
 }
 
-TEST(ShardKeyPattern, UniqueIndexCompatibleCompound) {
+TEST_F(ShardKeyPatternTest, UniqueIndexCompatibleCompound) {
     //
     // Compound ShardKeyPatterns
     //
@@ -651,7 +647,7 @@ TEST(ShardKeyPattern, UniqueIndexCompatibleCompound) {
     ASSERT(!indexComp(pattern, BSON("b" << -1 << "a" << 1 << "c" << 1)));
 }
 
-TEST(ShardKeyPattern, UniqueIndexCompatibleNested) {
+TEST_F(ShardKeyPatternTest, UniqueIndexCompatibleNested) {
     //
     // Nested ShardKeyPatterns
     //
@@ -664,7 +660,7 @@ TEST(ShardKeyPattern, UniqueIndexCompatibleNested) {
     ASSERT(!indexComp(pattern, BSON("c" << -1 << "a.b" << 1)));
 }
 
-TEST(ShardKeyPattern, UniqueIndexCompatibleHashed) {
+TEST_F(ShardKeyPatternTest, UniqueIndexCompatibleHashed) {
     //
     // Hashed ShardKeyPatterns
     //
@@ -688,7 +684,7 @@ TEST(ShardKeyPattern, UniqueIndexCompatibleHashed) {
     ASSERT(!indexComp(pattern, BSON("c" << -1 << "a.b" << 1)));
 }
 
-TEST(ShardKeyPattern, IsHashedPattern) {
+TEST_F(ShardKeyPatternTest, IsHashedPattern) {
     ASSERT(ShardKeyPattern(BSON("a.b"
                                 << "hashed"))
                .isHashedPattern());
@@ -699,7 +695,7 @@ TEST(ShardKeyPattern, IsHashedPattern) {
     ASSERT(!ShardKeyPattern(BSON("a.b" << 1 << "d" << 1)).isHashedPattern());
 }
 
-TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_WithNumericFieldNames) {
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromIndexKeyData_WithNumericFieldNames) {
     ShardKeyPattern pattern(BSON("a.0.1" << 1 << "c.d.0"
                                          << "hashed"));
     auto indexPattern = BSON("a.0.1" << 1.0 << "p" << 1 << "c.d.0"
@@ -725,7 +721,7 @@ TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_WithNumericFieldNames) {
                       BSONObj());
 }
 
-TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_NonHashedIndexAndShardKey) {
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromIndexKeyData_NonHashedIndexAndShardKey) {
     ShardKeyPattern pattern(BSON("a.b" << 1 << "c.d" << 1.0));
     auto indexPattern = BSON("c.d" << 1.0 << "p"
                                    << "hashed"
@@ -742,7 +738,7 @@ TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_NonHashedIndexAndShardKey)
                                    << "subObject")));
 }
 
-TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_HashedIndexAndShardKey) {
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromIndexKeyData_HashedIndexAndShardKey) {
     ShardKeyPattern pattern(BSON("a.b" << 1 << "c.d"
                                        << "hashed"
                                        << "p.q" << 1));
@@ -774,7 +770,7 @@ DEATH_TEST_REGEX(ShardKeyPattern,
     pattern.extractShardKeyFromIndexKeyData({{indexData, indexPattern}});
 }
 
-TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_WithArrayInIndex) {
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromIndexKeyData_WithArrayInIndex) {
     ShardKeyPattern pattern(BSON("a.b" << 1 << "c.d"
                                        << "hashed"
                                        << "p.q" << 1));
@@ -789,7 +785,7 @@ TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_WithArrayInIndex) {
                       BSONObj());
 }
 
-TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_WithArrayForNonShardKeyFields) {
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromIndexKeyData_WithArrayForNonShardKeyFields) {
     ShardKeyPattern pattern(BSON("a.b" << 1 << "c.d"
                                        << "hashed"
                                        << "p.q" << 1));
@@ -807,7 +803,7 @@ TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_WithArrayForNonShardKeyFie
                            << "c.d" << 123 << "p.q" << BSONNULL));
 }
 
-TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_NonHashedIndexAndHashedShardKey) {
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromIndexKeyData_NonHashedIndexAndHashedShardKey) {
     ShardKeyPattern pattern(BSON("a.b"
                                  << "hashed"
                                  << "c.d" << 1.0 << "e.null" << 1));
@@ -845,7 +841,7 @@ DEATH_TEST_REGEX(ShardKeyPattern,
     pattern.extractShardKeyFromIndexKeyData({{indexData, indexPattern}});
 }
 
-TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_FromMultipleIndexes) {
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromIndexKeyData_FromMultipleIndexes) {
     ShardKeyPattern pattern(BSON("a.firstIndex" << 1 << "a.secondIndex" << 1.0 << "c.secondIndex"
                                                 << "hashed"
                                                 << "null" << 1));
@@ -870,7 +866,7 @@ TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_FromMultipleIndexes) {
                            << "c.secondIndex" << hashedValue << "null" << BSONNULL));
 }
 
-TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_FromMultipleIndexesProvidingSameField) {
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromIndexKeyData_FromMultipleIndexesProvidingSameField) {
     ShardKeyPattern pattern(BSON("a.firstIndex" << 1 << "a.secondIndex" << 1.0 << "a.thirdIndex"
                                                 << "hashed"
                                                 << "null" << 1));
@@ -901,82 +897,122 @@ TEST(ShardKeyPattern, ExtractShardKeyFromIndexKeyData_FromMultipleIndexesProvidi
                            << "a.thirdIndex" << hashedValue << "null" << BSONNULL));
 }
 
-TEST(ShardKeyPattern, IsExtendedBy) {
+BSONObj getDocumentKey(const BSONObj& obj, const ShardKeyPattern& shardKeyPattern) {
+    return dotted_path_support::extractElementsBasedOnTemplate(obj, shardKeyPattern.toBSON());
+}
 
-    // NumberOfFields
-    ShardKeyPattern shardKeyPattern1(BSON("a" << 1));
-    ShardKeyPattern shardKeyPattern2(BSON("a" << 1 << "b" << 1));
-    ShardKeyPattern shardKeyPattern3(BSON("a" << 1 << "b" << 1 << "c" << 1));
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromDocumentKey_Single) {
+    ShardKeyPattern shardKeyPattern(BSON("_id" << 1));
 
-    // NumberOfFields_PositionOfHash
-    ShardKeyPattern shardKeyPatternHashed1_0(BSON("a"
-                                                  << "hashed"));
-    ShardKeyPattern shardKeyPatternHashed2_0(BSON("a"
-                                                  << "hashed"
-                                                  << "b" << 1));
-    ShardKeyPattern shardKeyPatternHashed2_1(BSON("a" << 1 << "b"
-                                                      << "hashed"));
-    ShardKeyPattern shardKeyPatternHashed3_0(BSON("a"
-                                                  << "hashed"
-                                                  << "b" << 1 << "c" << 1));
-    ShardKeyPattern shardKeyPatternHashed3_1(BSON("a" << 1 << "b"
-                                                      << "hashed"
-                                                      << "c" << 1));
-    ShardKeyPattern shardKeyPatternHashed3_2(BSON("a" << 1 << "b" << 1 << "c"
-                                                      << "hashed"));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(BSON("_id" << 10), shardKeyPattern)),
+                      BSON("_id" << 10));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(BSON("_id" << 10 << "a" << 20), shardKeyPattern)),
+                      BSON("_id" << 10));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(BSON("b" << 30), shardKeyPattern)),
+                      BSON("_id" << BSONNULL));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(BSON("_id" << BSON_ARRAY(10)), shardKeyPattern)),
+                      BSONObj());
+}
 
-    // same pattern, always true
-    ASSERT_TRUE(shardKeyPattern1.isExtendedBy(shardKeyPattern1));
-    ASSERT_TRUE(shardKeyPattern2.isExtendedBy(shardKeyPattern2));
-    ASSERT_TRUE(shardKeyPattern3.isExtendedBy(shardKeyPattern3));
-    ASSERT_TRUE(shardKeyPatternHashed1_0.isExtendedBy(shardKeyPatternHashed1_0));
-    ASSERT_TRUE(shardKeyPatternHashed2_0.isExtendedBy(shardKeyPatternHashed2_0));
-    ASSERT_TRUE(shardKeyPatternHashed2_1.isExtendedBy(shardKeyPatternHashed2_1));
-    ASSERT_TRUE(shardKeyPatternHashed3_0.isExtendedBy(shardKeyPatternHashed3_0));
-    ASSERT_TRUE(shardKeyPatternHashed3_1.isExtendedBy(shardKeyPatternHashed3_1));
-    ASSERT_TRUE(shardKeyPatternHashed3_2.isExtendedBy(shardKeyPatternHashed3_2));
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromDocumentKey_Compound) {
+    ShardKeyPattern shardKeyPattern(BSON("a" << 1 << "b" << 1));
 
-    // different number of fields, same values
-    ASSERT_TRUE(shardKeyPattern1.isExtendedBy(shardKeyPattern2));
-    ASSERT_TRUE(shardKeyPattern2.isExtendedBy(shardKeyPattern3));
-    ASSERT_TRUE(shardKeyPattern1.isExtendedBy(shardKeyPattern3));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(BSON("a" << 10 << "b" << 20), shardKeyPattern)),
+                      BSON("a" << 10 << "b" << 20));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(getDocumentKey(
+                          BSON("a" << 10 << "b" << 20 << "c" << 30), shardKeyPattern)),
+                      BSON("a" << 10 << "b" << 20));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(BSON("b" << 20), shardKeyPattern)),
+                      BSON("a" << BSONNULL << "b" << 20));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(BSON("c" << 30), shardKeyPattern)),
+                      BSON("a" << BSONNULL << "b" << BSONNULL));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(getDocumentKey(
+                          BSON("a" << BSON_ARRAY(10) << "b" << 20), shardKeyPattern)),
+                      BSONObj());
+}
 
-    ASSERT_FALSE(shardKeyPattern2.isExtendedBy(shardKeyPattern1));
-    ASSERT_FALSE(shardKeyPattern3.isExtendedBy(shardKeyPattern2));
-    ASSERT_FALSE(shardKeyPattern3.isExtendedBy(shardKeyPattern1));
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromDocumentKey_Nested) {
+    ShardKeyPattern shardKeyPattern(BSON("a.b" << 1 << "c" << 1));
 
-    // different number of fields, different values
-    // { a : 1 } is not extended by { a : "hashed" } and viceversa
-    ASSERT_FALSE(shardKeyPattern1.isExtendedBy(shardKeyPatternHashed1_0));
-    ASSERT_FALSE(shardKeyPatternHashed1_0.isExtendedBy(shardKeyPattern1));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(fromjson("{a:{b:10}, c:20}"), shardKeyPattern)),
+                      fromjson("{'a.b':10, c:20}"));
 
-    // { a : 1, b : 1 } is not extended by { a : 1, b : "hashed" } and viceversa
-    ASSERT_FALSE(shardKeyPattern2.isExtendedBy(shardKeyPatternHashed2_1));
-    ASSERT_FALSE(shardKeyPatternHashed2_1.isExtendedBy(shardKeyPattern2));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(fromjson("{d:30}"), shardKeyPattern)),
+                      fromjson("{'a.b':null, c:null}"));
 
-    // { a : 1 } is extended by { a : 1, b : "hashed" } but not viceversa
-    ASSERT_TRUE(shardKeyPattern1.isExtendedBy(shardKeyPatternHashed2_1));
-    ASSERT_FALSE(shardKeyPatternHashed2_1.isExtendedBy(shardKeyPattern1));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(fromjson("{a:{b:10}}"), shardKeyPattern)),
+                      fromjson("{'a.b':10, c:null}"));
 
-    // { a : 1, b : 1 } is extended by { a : 1, b : 1, c : "hashed" } but not viceversa
-    ASSERT_TRUE(shardKeyPattern2.isExtendedBy(shardKeyPatternHashed3_2));
-    ASSERT_FALSE(shardKeyPatternHashed3_2.isExtendedBy(shardKeyPattern2));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(fromjson("{c:20}"), shardKeyPattern)),
+                      fromjson("{'a.b':null, c:20}"));
 
-    // { a : 1, b : 1, c : 1 } is not extended by { a : 1, b : 1, c : "hashed" } and viceversa
-    ASSERT_FALSE(shardKeyPattern3.isExtendedBy(shardKeyPatternHashed3_2));
-    ASSERT_FALSE(shardKeyPatternHashed3_2.isExtendedBy(shardKeyPattern3));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(fromjson("{a:10, c:20}"), shardKeyPattern)),
+                      fromjson("{'a.b':null, c:20}"));
 
-    // { a: "hashed", b : 1 } is not extended by { a : 1, b : 1, c : "hashed" } and viceversa
-    ASSERT_FALSE(shardKeyPatternHashed2_1.isExtendedBy(shardKeyPatternHashed3_2));
-    ASSERT_FALSE(shardKeyPatternHashed3_2.isExtendedBy(shardKeyPatternHashed2_1));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(fromjson("{a:{d:10}, c:20}"), shardKeyPattern)),
+                      fromjson("{'a.b':null, c:20}"));
 
-    // { a : "hashed", b : 1 } is extended by { a : "hashed", b : 1, c : "1" } but not viceversa
-    ASSERT_TRUE(shardKeyPatternHashed2_0.isExtendedBy(shardKeyPatternHashed3_0));
-    ASSERT_FALSE(shardKeyPatternHashed3_0.isExtendedBy(shardKeyPatternHashed2_0));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(fromjson("{a:{b:10, d:30}, c:20}"), shardKeyPattern)),
+                      fromjson("{'a.b':10, c:20}"));
 
-    // { a : 1, b : "hashed " } is extended by { a : 1, b : "hashed", c : "1" } but not viceversa
-    ASSERT_TRUE(shardKeyPatternHashed2_1.isExtendedBy(shardKeyPatternHashed3_1));
-    ASSERT_FALSE(shardKeyPatternHashed3_1.isExtendedBy(shardKeyPatternHashed2_1));
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(getDocumentKey(
+                          fromjson("{a:{b:{y:10,z:11}}, c:20, d:30}"), shardKeyPattern)),
+                      fromjson("{'a.b':{y:10,z:11}, c:20}"));
+
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(fromjson("{a:{b:[10,11]}, c:20, d:30}"), shardKeyPattern)),
+                      BSONObj());
+}
+
+TEST_F(ShardKeyPatternTest, ExtractShardKeyFromDocumentKey_Hashed) {
+    const std::string value = "12345";
+    const BSONObj bsonValue = BSON("" << value);
+    const long long hashValue =
+        BSONElementHasher::hash64(bsonValue.firstElement(), BSONElementHasher::DEFAULT_HASH_SEED);
+
+    const BSONObj nullBsonValue = BSON("" << BSONNULL);
+    const long long nullHashValue = BSONElementHasher::hash64(nullBsonValue.firstElement(),
+                                                              BSONElementHasher::DEFAULT_HASH_SEED);
+
+    ShardKeyPattern shardKeyPattern(BSON("a.b"
+                                         << "hashed"
+                                         << "c" << 1));
+
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(getDocumentKey(
+                          BSON("a" << BSON("b" << value) << "c" << 20), shardKeyPattern)),
+                      BSON("a.b" << hashValue << "c" << 20));
+
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(getDocumentKey(
+                          BSON("a" << BSON("b" << value << "z" << 11) << "c" << 20 << "d" << 30),
+                          shardKeyPattern)),
+                      BSON("a.b" << hashValue << "c" << 20));
+
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(
+                          getDocumentKey(BSON("c" << 20), shardKeyPattern)),
+                      BSON("a.b" << nullHashValue << "c" << 20));
+
+    ASSERT_BSONOBJ_EQ(shardKeyPattern.extractShardKeyFromDocumentKey(getDocumentKey(
+                          BSON("a" << BSON("z" << value) << "c" << 20), shardKeyPattern)),
+                      BSON("a.b" << nullHashValue << "c" << 20));
+
+    ASSERT_BSONOBJ_EQ(
+        shardKeyPattern.extractShardKeyFromDocumentKey(getDocumentKey(
+            BSON("a" << BSON("b" << BSON_ARRAY(value)) << "c" << 20), shardKeyPattern)),
+        BSONObj());
 }
 
 }  // namespace
