@@ -41,22 +41,20 @@
 #include "mongo/db/client.h"
 #include "mongo/db/client_metadata_propagation_egress_hook.h"
 #include "mongo/db/commands.h"
-#include "mongo/db/logical_time_metadata_hook.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/ops/write_ops.h"
 #include "mongo/db/query/collation/collator_factory_mock.h"
-#include "mongo/db/query/query_request.h"
+#include "mongo/db/query/query_request_helper.h"
 #include "mongo/db/repl/read_concern_args.h"
-#include "mongo/executor/network_interface_mock.h"
+#include "mongo/db/vector_clock.h"
+#include "mongo/db/vector_clock_metadata_hook.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/executor/thread_pool_task_executor_test_fixture.h"
 #include "mongo/rpc/metadata/egress_metadata_hook_list.h"
 #include "mongo/rpc/metadata/repl_set_metadata.h"
 #include "mongo/rpc/metadata/tracking_metadata.h"
 #include "mongo/s/balancer_configuration.h"
-#include "mongo/s/catalog/dist_lock_manager_mock.h"
 #include "mongo/s/catalog/sharding_catalog_client_impl.h"
-#include "mongo/s/catalog/type_changelog.h"
 #include "mongo/s/catalog/type_collection.h"
 #include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/catalog_cache.h"
@@ -64,12 +62,9 @@
 #include "mongo/s/client/shard_factory.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/client/shard_remote.h"
-#include "mongo/s/committed_optime_metadata_hook.h"
 #include "mongo/s/config_server_catalog_cache_loader.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/query/cluster_cursor_manager.h"
-#include "mongo/s/request_types/set_shard_version_request.h"
-#include "mongo/s/sharding_egress_metadata_hook_for_mongos.h"
 #include "mongo/s/sharding_task_executor.h"
 #include "mongo/s/write_ops/batched_command_response.h"
 #include "mongo/transport/mock_session.h"
@@ -96,8 +91,9 @@ std::unique_ptr<ShardingTaskExecutor> makeShardingTestExecutor(
 
 }  // namespace
 
-ShardingTestFixture::ShardingTestFixture() {
-    auto const service = getServiceContext();
+ShardingTestFixture::ShardingTestFixture()
+    : _transportSession(transport::MockSession::create(nullptr)) {
+    const auto service = getServiceContext();
 
     // Configure the service context
     service->setFastClockSource(std::make_unique<ClockSourceMock>());
@@ -105,16 +101,12 @@ ShardingTestFixture::ShardingTestFixture() {
     service->setTickSource(std::make_unique<TickSourceMock<>>());
 
     CollatorFactoryInterface::set(service, std::make_unique<CollatorFactoryMock>());
-    _transportSession = transport::MockSession::create(nullptr);
-    _opCtx = makeOperationContext();
 
     // Set up executor pool used for most operations.
     auto makeMetadataHookList = [&] {
         auto hookList = std::make_unique<rpc::EgressMetadataHookList>();
-        hookList->addHook(std::make_unique<rpc::LogicalTimeMetadataHook>(service));
-        hookList->addHook(std::make_unique<rpc::CommittedOpTimeMetadataHook>(service));
+        hookList->addHook(std::make_unique<rpc::VectorClockMetadataHook>(service));
         hookList->addHook(std::make_unique<rpc::ClientMetadataPropagationEgressHook>());
-        hookList->addHook(std::make_unique<rpc::ShardingEgressMetadataHookForMongos>(service));
         return hookList;
     };
 
@@ -136,13 +128,6 @@ ShardingTestFixture::ShardingTestFixture() {
     auto executorPool = std::make_unique<executor::TaskExecutorPool>();
     executorPool->addExecutors(std::move(executorsForPool), _fixedExecutor);
 
-    auto uniqueDistLockManager = std::make_unique<DistLockManagerMock>(nullptr);
-    _distLockManager = uniqueDistLockManager.get();
-
-    std::unique_ptr<ShardingCatalogClientImpl> catalogClient(
-        std::make_unique<ShardingCatalogClientImpl>(std::move(uniqueDistLockManager)));
-    catalogClient->startup();
-
     NumHostsTargetedMetrics::get(service).startup();
 
     ConnectionString configCS = ConnectionString::forReplicaSet(
@@ -152,10 +137,6 @@ ShardingTestFixture::ShardingTestFixture() {
     auto targeterFactoryPtr = targeterFactory.get();
     _targeterFactory = targeterFactoryPtr;
 
-    auto configTargeter(std::make_unique<RemoteCommandTargeterMock>());
-    _configTargeter = configTargeter.get();
-    _targeterFactory->addTargeterToReturn(configCS, std::move(configTargeter));
-
     ShardFactory::BuilderCallable setBuilder = [targeterFactoryPtr](
                                                    const ShardId& shardId,
                                                    const ConnectionString& connStr) {
@@ -169,23 +150,25 @@ ShardingTestFixture::ShardingTestFixture() {
     };
 
     ShardFactory::BuildersMap buildersMap{
-        {ConnectionString::SET, std::move(setBuilder)},
-        {ConnectionString::MASTER, std::move(masterBuilder)},
+        {ConnectionString::ConnectionType::kReplicaSet, std::move(setBuilder)},
+        {ConnectionString::ConnectionType::kStandalone, std::move(masterBuilder)},
     };
 
     auto shardFactory =
         std::make_unique<ShardFactory>(std::move(buildersMap), std::move(targeterFactory));
 
-    auto shardRegistry(std::make_unique<ShardRegistry>(std::move(shardFactory), configCS));
+    auto shardRegistry(std::make_unique<ShardRegistry>(service, std::move(shardFactory), configCS));
     executorPool->startup();
 
     CatalogCacheLoader::set(service, std::make_unique<ConfigServerCatalogCacheLoader>());
 
+    auto catalogCache = std::make_unique<CatalogCache>(service, CatalogCacheLoader::get(service));
     // For now initialize the global grid object. All sharding objects will be accessible from there
     // until we get rid of it.
-    Grid::get(operationContext())
-        ->init(std::move(catalogClient),
-               std::make_unique<CatalogCache>(CatalogCacheLoader::get(service)),
+    auto uniqueOpCtx = makeOperationContext();
+    auto const grid = Grid::get(uniqueOpCtx.get());
+    grid->init(makeShardingCatalogClient(),
+               std::move(catalogCache),
                std::move(shardRegistry),
                std::make_unique<ClusterCursorManager>(service->getPreciseClockSource()),
                std::make_unique<BalancerConfiguration>(),
@@ -193,9 +176,22 @@ ShardingTestFixture::ShardingTestFixture() {
                _mockNetwork);
 }
 
-
 ShardingTestFixture::~ShardingTestFixture() {
     CatalogCacheLoader::clearForTests(getServiceContext());
+
+    if (auto grid = Grid::get(getServiceContext())) {
+        if (grid->getExecutorPool()) {
+            grid->getExecutorPool()->shutdownAndJoin();
+        }
+        if (grid->shardRegistry()) {
+            grid->shardRegistry()->shutdown();
+        }
+        grid->clearForUnitTests();
+    }
+}
+
+std::shared_ptr<RemoteCommandTargeterMock> ShardingTestFixture::configTargeter() {
+    return RemoteCommandTargeterMock::get(shardRegistry()->getConfigShard()->getTargeter());
 }
 
 void ShardingTestFixture::shutdownExecutor() {
@@ -211,36 +207,12 @@ ShardRegistry* ShardingTestFixture::shardRegistry() const {
     return Grid::get(operationContext())->shardRegistry();
 }
 
-RemoteCommandTargeterFactoryMock* ShardingTestFixture::targeterFactory() const {
-    invariant(_targeterFactory);
-
-    return _targeterFactory;
-}
-
-RemoteCommandTargeterMock* ShardingTestFixture::configTargeter() const {
-    invariant(_configTargeter);
-
-    return _configTargeter;
-}
-
 std::shared_ptr<executor::TaskExecutor> ShardingTestFixture::executor() const {
     invariant(_fixedExecutor);
 
     return _fixedExecutor;
 }
 
-DistLockManagerMock* ShardingTestFixture::distLock() const {
-    invariant(_distLockManager);
-
-    return _distLockManager;
-}
-
-OperationContext* ShardingTestFixture::operationContext() const {
-    invariant(_opCtx);
-
-    return _opCtx.get();
-}
-
 void ShardingTestFixture::onCommandForPoolExecutor(NetworkTestEnv::OnCommandFunction func) {
     _networkTestEnvForPool->onCommand(func);
 }
@@ -278,19 +250,20 @@ void ShardingTestFixture::setupShards(const std::vector<ShardType>& shards) {
 void ShardingTestFixture::expectGetShards(const std::vector<ShardType>& shards) {
     onFindCommand([this, &shards](const RemoteCommandRequest& request) {
         const NamespaceString nss(request.dbname, request.cmdObj.firstElement().String());
-        ASSERT_EQ(nss, ShardType::ConfigNS);
+        ASSERT_EQ(nss, NamespaceString::kConfigsvrShardsNamespace);
 
-        auto queryResult = QueryRequest::makeFromFindCommand(nss, request.cmdObj, false);
-        ASSERT_OK(queryResult.getStatus());
-
-        const auto& query = queryResult.getValue();
-        ASSERT_EQ(query->nss(), ShardType::ConfigNS);
+        // If there is no '$db', append it.
+        auto cmd = OpMsgRequest::fromDBAndBody(nss.db(), request.cmdObj).body;
+        auto query = query_request_helper::makeFromFindCommandForTests(cmd, nss);
+        ASSERT_EQ(*query->getNamespaceOrUUID().nss(), NamespaceString::kConfigsvrShardsNamespace);
 
         ASSERT_BSONOBJ_EQ(query->getFilter(), BSONObj());
         ASSERT_BSONOBJ_EQ(query->getSort(), BSONObj());
-        ASSERT_FALSE(query->getLimit().is_initialized());
+        ASSERT_FALSE(query->getLimit().has_value());
 
-        checkReadConcern(request.cmdObj, Timestamp(0, 0), repl::OpTime::kUninitializedTerm);
+        checkReadConcern(request.cmdObj,
+                         VectorClock::kInitialComponentTime.asTimestamp(),
+                         repl::OpTime::kUninitializedTerm);
 
         std::vector<BSONObj> shardsToReturn;
 
@@ -329,94 +302,6 @@ void ShardingTestFixture::expectInserts(const NamespaceString& nss,
     });
 }
 
-void ShardingTestFixture::expectConfigCollectionCreate(const HostAndPort& configHost,
-                                                       StringData collName,
-                                                       int cappedSize,
-                                                       const BSONObj& response) {
-    onCommand([&](const RemoteCommandRequest& request) {
-        ASSERT_EQUALS(configHost, request.target);
-        ASSERT_EQUALS("config", request.dbname);
-
-        BSONObj expectedCreateCmd =
-            BSON("create" << collName << "capped" << true << "size" << cappedSize << "writeConcern"
-                          << BSON("w"
-                                  << "majority"
-                                  << "wtimeout" << 60000)
-                          << "maxTimeMS" << 30000);
-        ASSERT_BSONOBJ_EQ(expectedCreateCmd, request.cmdObj);
-
-        return response;
-    });
-}
-
-void ShardingTestFixture::expectConfigCollectionInsert(const HostAndPort& configHost,
-                                                       StringData collName,
-                                                       Date_t timestamp,
-                                                       const std::string& what,
-                                                       const std::string& ns,
-                                                       const BSONObj& detail) {
-    onCommand([&](const RemoteCommandRequest& request) {
-        ASSERT_EQUALS(configHost, request.target);
-        ASSERT_EQUALS(NamespaceString::kConfigDb, request.dbname);
-
-        const auto opMsgRequest = OpMsgRequest::fromDBAndBody(request.dbname, request.cmdObj);
-        const auto insertOp = InsertOp::parse(opMsgRequest);
-
-        ASSERT_EQ(NamespaceString::kConfigDb, insertOp.getNamespace().db());
-        ASSERT_EQ(collName, insertOp.getNamespace().coll());
-
-        const auto& inserts = insertOp.getDocuments();
-        ASSERT_EQUALS(1U, inserts.size());
-
-        const ChangeLogType& actualChangeLog = assertGet(ChangeLogType::fromBSON(inserts.front()));
-
-        ASSERT_EQUALS(operationContext()->getClient()->clientAddress(true),
-                      actualChangeLog.getClientAddr());
-        ASSERT_BSONOBJ_EQ(detail, actualChangeLog.getDetails());
-        ASSERT_EQUALS(ns, actualChangeLog.getNS());
-        const std::string expectedServer = str::stream() << network()->getHostName() << ":27017";
-        ASSERT_EQUALS(expectedServer, actualChangeLog.getServer());
-        ASSERT_EQUALS(timestamp, actualChangeLog.getTime());
-        ASSERT_EQUALS(what, actualChangeLog.getWhat());
-
-        // Handle changeId specially because there's no way to know what OID was generated
-        std::string changeId = actualChangeLog.getChangeId();
-        size_t firstDash = changeId.find("-");
-        size_t lastDash = changeId.rfind("-");
-
-        const std::string serverPiece = changeId.substr(0, firstDash);
-        const std::string timePiece = changeId.substr(firstDash + 1, lastDash - firstDash - 1);
-        const std::string oidPiece = changeId.substr(lastDash + 1);
-
-        const std::string expectedServerPiece = str::stream()
-            << Grid::get(operationContext())->getNetwork()->getHostName() << ":27017";
-        ASSERT_EQUALS(expectedServerPiece, serverPiece);
-        ASSERT_EQUALS(timestamp.toString(), timePiece);
-
-        OID generatedOID;
-        // Just make sure this doesn't throws and assume the OID is valid
-        generatedOID.init(oidPiece);
-
-        BatchedCommandResponse response;
-        response.setStatus(Status::OK());
-
-        return response.toBSON();
-    });
-}
-
-void ShardingTestFixture::expectChangeLogCreate(const HostAndPort& configHost,
-                                                const BSONObj& response) {
-    expectConfigCollectionCreate(configHost, "changelog", 10 * 1024 * 1024, response);
-}
-
-void ShardingTestFixture::expectChangeLogInsert(const HostAndPort& configHost,
-                                                Date_t timestamp,
-                                                const std::string& what,
-                                                const std::string& ns,
-                                                const BSONObj& detail) {
-    expectConfigCollectionInsert(configHost, "changelog", timestamp, what, ns, detail);
-}
-
 void ShardingTestFixture::expectUpdateCollection(const HostAndPort& expectedHost,
                                                  const CollectionType& coll,
                                                  bool expectUpsert) {
@@ -436,8 +321,13 @@ void ShardingTestFixture::expectUpdateCollection(const HostAndPort& expectedHost
         const auto& update = updates.front();
         ASSERT_EQ(expectUpsert, update.getUpsert());
         ASSERT(!update.getMulti());
-        ASSERT_BSONOBJ_EQ(BSON(CollectionType::fullNs(coll.getNs().toString())), update.getQ());
-        ASSERT_BSONOBJ_EQ(coll.toBSON(), update.getU().getUpdateClassic());
+        ASSERT_BSONOBJ_EQ(BSON(CollectionType::kNssFieldName << coll.getNss().toString()),
+                          update.getQ());
+        const auto& updateBSON =
+            update.getU().type() == write_ops::UpdateModification::Type::kReplacement
+            ? update.getU().getUpdateReplacement()
+            : update.getU().getUpdateModifier();
+        ASSERT_BSONOBJ_EQ(coll.toBSON(), updateBSON);
 
         BatchedCommandResponse response;
         response.setStatus(Status::OK());
@@ -447,28 +337,6 @@ void ShardingTestFixture::expectUpdateCollection(const HostAndPort& expectedHost
     });
 }
 
-void ShardingTestFixture::expectSetShardVersion(const HostAndPort& expectedHost,
-                                                const ShardType& expectedShard,
-                                                const NamespaceString& expectedNs,
-                                                const ChunkVersion& expectedChunkVersion) {
-    onCommand([&](const RemoteCommandRequest& request) {
-        ASSERT_EQ(expectedHost, request.target);
-        ASSERT_BSONOBJ_EQ(rpc::makeEmptyMetadata(),
-                          rpc::TrackingMetadata::removeTrackingData(request.metadata));
-
-        SetShardVersionRequest ssv =
-            assertGet(SetShardVersionRequest::parseFromBSON(request.cmdObj));
-
-        ASSERT(!ssv.isInit());
-        ASSERT(ssv.isAuthoritative());
-        ASSERT_EQ(expectedShard.getHost(), ssv.getShardConnectionString().toString());
-        ASSERT_EQ(expectedNs, ssv.getNS());
-        ASSERT_EQ(expectedChunkVersion.toString(), ssv.getNSVersion().toString());
-
-        return BSON("ok" << true);
-    });
-}
-
 void ShardingTestFixture::expectCount(const HostAndPort& configHost,
                                       const NamespaceString& expectedNs,
                                       const BSONObj& expectedQuery,
@@ -491,7 +359,9 @@ void ShardingTestFixture::expectCount(const HostAndPort& configHost,
             return BSON("ok" << 1 << "n" << response.getValue());
         }
 
-        checkReadConcern(request.cmdObj, Timestamp(0, 0), repl::OpTime::kUninitializedTerm);
+        checkReadConcern(request.cmdObj,
+                         VectorClock::kInitialComponentTime.asTimestamp(),
+                         repl::OpTime::kUninitializedTerm);
 
         BSONObjBuilder responseBuilder;
         CommandHelpers::appendCommandStatusNoThrow(responseBuilder, response.getStatus());
@@ -522,15 +392,28 @@ void ShardingTestFixture::checkReadConcern(const BSONObj& cmdObj,
     auto readConcernObj = readConcernElem.Obj();
     ASSERT_EQ("majority", readConcernObj[repl::ReadConcernArgs::kLevelFieldName].str());
 
-    auto afterElem = readConcernObj[repl::ReadConcernArgs::kAfterOpTimeFieldName];
-    ASSERT_EQ(Object, afterElem.type());
+    auto afterOpTimeElem = readConcernObj[repl::ReadConcernArgs::kAfterOpTimeFieldName];
+    auto afterClusterTimeElem = readConcernObj[repl::ReadConcernArgs::kAfterClusterTimeFieldName];
+    if (afterOpTimeElem.type() != EOO) {
+        ASSERT_EQ(EOO, afterClusterTimeElem.type());
+        ASSERT_EQ(Object, afterOpTimeElem.type());
 
-    auto afterObj = afterElem.Obj();
+        auto afterOpTimeObj = afterOpTimeElem.Obj();
 
-    ASSERT_TRUE(afterObj.hasField(repl::OpTime::kTimestampFieldName));
-    ASSERT_EQ(expectedTS, afterObj[repl::OpTime::kTimestampFieldName].timestamp());
-    ASSERT_TRUE(afterObj.hasField(repl::OpTime::kTermFieldName));
-    ASSERT_EQ(expectedTerm, afterObj[repl::OpTime::kTermFieldName].numberLong());
+        ASSERT_TRUE(afterOpTimeObj.hasField(repl::OpTime::kTimestampFieldName));
+        ASSERT_EQ(expectedTS, afterOpTimeObj[repl::OpTime::kTimestampFieldName].timestamp());
+        ASSERT_TRUE(afterOpTimeObj.hasField(repl::OpTime::kTermFieldName));
+        ASSERT_EQ(expectedTerm, afterOpTimeObj[repl::OpTime::kTermFieldName].numberLong());
+    } else {
+        ASSERT_EQ(EOO, afterOpTimeElem.type());
+        ASSERT_EQ(bsonTimestamp, afterClusterTimeElem.type());
+
+        ASSERT_EQ(expectedTS, afterClusterTimeElem.timestamp());
+    }
+}
+
+std::unique_ptr<ShardingCatalogClient> ShardingTestFixture::makeShardingCatalogClient() {
+    return std::make_unique<ShardingCatalogClientImpl>();
 }
 
 }  // namespace mongo
