@@ -31,15 +31,16 @@
 
 #include <boost/optional.hpp>
 
+#include "mongo/db/api_parameters.h"
 #include "mongo/db/commands/txn_cmds_gen.h"
-#include "mongo/db/logical_session_id.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/read_concern_args.h"
-#include "mongo/db/session_catalog.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/session_catalog.h"
+#include "mongo/db/shard_id.h"
 #include "mongo/db/stats/single_transaction_stats.h"
 #include "mongo/s/async_requests_sender.h"
 #include "mongo/s/client/shard.h"
-#include "mongo/s/shard_id.h"
 #include "mongo/util/string_map.h"
 
 namespace mongo {
@@ -70,6 +71,9 @@ public:
         kRecoverWithToken,
     };
 
+    // The reason why TransactionRouter::Router::stash() is called.
+    enum class StashReason { kDone, kYield };
+
     // The default value to use as the statement id of the first command in the transaction if none
     // was sent.
     static const StmtId kDefaultFirstStmtId = 0;
@@ -81,13 +85,14 @@ public:
      */
     struct SharedTransactionOptions {
         // Set for all distributed transactions.
-        TxnNumber txnNumber;
+        TxnNumberAndRetryCounter txnNumberAndRetryCounter;
+        APIParameters apiParameters;
         repl::ReadConcernArgs readConcernArgs;
 
         // Only set for transactions with snapshot level read concern.
-        boost::optional<LogicalTime> atClusterTimeForSnapshotReadConcern;
+        boost::optional<LogicalTime> atClusterTime;
 
-        boost::optional<LogicalTime> placementConflictTimeForNonSnapshotReadConcern;
+        bool isInternalTransactionForRetryableWrite;
     };
 
     /**
@@ -267,8 +272,8 @@ public:
      * The first command in a transaction to target at least one shard must select a cluster time
      * timestamp before targeting, but may change the timestamp before contacting any shards to
      * allow optimizing the timestamp based on the targeted shards. If the first command encounters
-     * a retryable error, e.g. StaleShardVersion or SnapshotTooOld, the retry may also select a new
-     * timestamp. Once the first command has successfully completed, the timestamp cannot be
+     * a retryable error, e.g. "retargeting needed" or SnapshotTooOld, the retry may also select a
+     * new timestamp. Once the first command has successfully completed, the timestamp cannot be
      * changed.
      */
     class AtClusterTime {
@@ -320,8 +325,17 @@ public:
         /**
          * Returns if the router has received at least one request for a transaction.
          */
-        auto isInitialized() {
-            return o().txnNumber != kUninitializedTxnNumber;
+        auto isInitialized() const {
+            return o().txnNumberAndRetryCounter.getTxnNumber() != kUninitializedTxnNumber;
+        }
+
+        /**
+         * Returns if this TransactionRouter instance can be reaped. Always true unless an operation
+         * has yielded the router and has not unyielded yet. We cannot reap the instance in that
+         * case or the unyield would check out a different TransactionRouter than it yielded.
+         */
+        auto canBeReaped() const {
+            return o().activeYields == 0;
         }
 
     protected:
@@ -370,9 +384,15 @@ public:
                                 TransactionActions action);
 
         /**
-         * Updates transaction diagnostics when the transaction's session is checked in.
+         * Updates transaction diagnostics and, if necessary, the number of active yielders when the
+         * transaction's session is checked in.
          */
-        void stash(OperationContext* opCtx);
+        void stash(OperationContext* opCtx, StashReason reason);
+
+        /**
+         * Validates transaction state is still compatible after a yield.
+         */
+        void unstash(OperationContext* opCtx);
 
         /**
          * Attaches the required transaction related fields for a request to be sent to the given
@@ -386,12 +406,11 @@ public:
          */
         BSONObj attachTxnFieldsIfNeeded(OperationContext* opCtx,
                                         const ShardId& shardId,
-                                        const BSONObj& cmdObj,
-                                        const StringData& dbName);
+                                        const BSONObj& cmdObj);
 
         /**
-         * Processes the transaction metadata in the response from the participant if the
-         * response indicates the operation succeeded.
+         * Processes the transaction metadata in the response from the participant if the response
+         * indicates the operation succeeded.
          */
         void processParticipantResponse(OperationContext* opCtx,
                                         const ShardId& shardId,
@@ -453,11 +472,6 @@ public:
          */
         LogicalTime getSelectedAtClusterTime() const;
 
-        /**
-         * Returns the placement conflict timestamp chosen for this transaction.
-         */
-        LogicalTime getPlacementConflictTime() const;
-
         /**
          * Sets the atClusterTime for the current transaction to the latest time in the router's
          * logical clock. Does nothing if the transaction does not have snapshot read concern or an
@@ -535,10 +549,12 @@ public:
         }
 
         /**
-         * Annotate that this transaction has attempted to create database 'dbName'.
+         * Returns if the router is not currently tracking an active transaction.
          */
-        void annotateCreatedDatabase(const StringData dbName) {
-            p().createdDatabases.insert(dbName.toString());
+        bool isTrackingOver() {
+            if (o().metricsTracker)
+                return o().metricsTracker->isTrackingOver();
+            return true;
         }
 
     private:
@@ -547,7 +563,31 @@ public:
          * time. This is required because we don't create a new router object for each transaction,
          * but instead reuse the same object across different transactions.
          */
-        void _resetRouterState(OperationContext* opCtx, const TxnNumber& txnNumber);
+        void _resetRouterState(OperationContext* opCtx,
+                               const TxnNumberAndRetryCounter& txnNumberAndRetryCounter);
+
+        /**
+         * Calls _resetRouterState and then resets the read concern and the cluster time of the
+         * timestamp that all participant shards in the current transaction with snapshot level read
+         * concern must read from.
+         */
+        void _resetRouterStateForStartTransaction(
+            OperationContext* opCtx, const TxnNumberAndRetryCounter& txnNumberAndRetryCounter);
+
+        /**
+         * Continues or restarts the currently active transaction.
+         */
+        void _continueTxn(OperationContext* opCtx,
+                          TxnNumberAndRetryCounter txnNumberAndRetryCounter,
+                          TransactionActions action);
+
+        /**
+         * Starts a new transaction or continues a transaction started by a different router to
+         * recover the commit decision.
+         */
+        void _beginTxn(OperationContext* opCtx,
+                       TxnNumberAndRetryCounter txnNumberAndRetryCounter,
+                       TransactionActions action);
 
         /**
          * Internal method for committing a transaction. Should only throw on failure to send
@@ -568,6 +608,14 @@ public:
          */
         BSONObj _handOffCommitToCoordinator(OperationContext* opCtx);
 
+        /**
+         * Sets the given logical time as the atClusterTime for the transaction to be the greater of
+         * the given time and the user's afterClusterTime, if one was provided.
+         */
+        void _setAtClusterTime(OperationContext* opCtx,
+                               const boost::optional<LogicalTime>& afterClusterTime,
+                               LogicalTime candidateTime);
+
         /**
          * Throws NoSuchTransaction if the response from abortTransaction failed with a code other
          * than NoSuchTransaction. Does not check for write concern errors.
@@ -599,7 +647,7 @@ public:
          */
         void _setReadOnlyForParticipant(OperationContext* opCtx,
                                         const ShardId& shard,
-                                        const Participant::ReadOnly readOnly);
+                                        Participant::ReadOnly readOnly);
 
         /**
          * Updates relevant metrics when the router receives an explicit abort from the client.
@@ -652,12 +700,6 @@ public:
          */
         void _logSlowTransaction(OperationContext* opCtx, TerminationCause terminationCause) const;
 
-        /**
-         * Returns a string to be logged for slow transactions.
-         */
-        std::string _transactionInfoForLog(OperationContext* opCtx,
-                                           TerminationCause terminationCause) const;
-
         /**
          * Returns the LastClientInfo object.
          */
@@ -674,14 +716,6 @@ public:
          */
         bool _errorAllowsRetryOnStaleShardOrDb(const Status& status) const;
 
-        /**
-         * Check if the routing table has a higher timestamp than this transaction's
-         * placement conflict timestamp.
-         */
-        void _checkForPlacementConflict(OperationContext* opCtx,
-                                        const ShardId& shardId,
-                                        const NamespaceString& nss);
-
         TransactionRouter::PrivateState& p() {
             return _tr->_p;
         }
@@ -711,9 +745,13 @@ private:
      * o(WithLock) method for writing.
      */
     struct ObservableState {
-        // The currently active transaction number on this router, if beginOrContinueTxn has been
-        // called. Otherwise set to kUninitializedTxnNumber.
-        TxnNumber txnNumber{kUninitializedTxnNumber};
+
+        // Struct with fields txnNumber and txnRetryCounter.
+        // If beginOrContinueTxn has been called, txnNumber and txnRetryCounter reflect
+        // the router's currently active transaction. Otherwise, they are set to
+        // kUninitializedTxnNumber and kUninitializedTxnRetryCounter by default.
+        TxnNumberAndRetryCounter txnNumberAndRetryCounter{kUninitializedTxnNumber,
+                                                          kUninitializedTxnRetryCounter};
 
         // Is updated at commit time to reflect which commit path was taken.
         CommitType commitType{CommitType::kNotInitiated};
@@ -727,15 +765,16 @@ private:
         // targeted. Is reset if the first participant targeted returns a "needs retargeting" error.
         boost::optional<ShardId> coordinatorId;
 
+        // The API parameters the current transaction was started with.
+        APIParameters apiParameters;
+
         // The read concern the current transaction was started with.
         repl::ReadConcernArgs readConcernArgs;
 
         // The cluster time of the timestamp all participant shards in the current transaction with
         // snapshot level read concern must read from. Only set for transactions running with
         // snapshot level read concern.
-        boost::optional<AtClusterTime> atClusterTimeForSnapshotReadConcern;
-
-        boost::optional<AtClusterTime> placementConflictTimeForNonSnapshotReadConcern;
+        boost::optional<AtClusterTime> atClusterTime;
 
         // String representing the reason a transaction aborted. Either the string name of the error
         // code that led to an implicit abort or "abort" if the client sent abortTransaction.
@@ -749,6 +788,11 @@ private:
         // certain transaction events. Unset until the transaction router has processed at least one
         // transaction command.
         boost::optional<MetricsTracker> metricsTracker;
+
+        // How many operations that checked out the router's session have currently yielded it. The
+        // transaction number cannot be changed until this returns to 0, otherwise we cannot
+        // guarantee that unyielding the session cannot fail.
+        int32_t activeYields{0};
     } _o;
 
     /**
@@ -779,9 +823,6 @@ private:
 
         // Track whether commit or abort have been initiated.
         bool terminationInitiated{false};
-
-        // Tracks databases that this transaction has attempted to create.
-        std::set<std::string> createdDatabases;
     } _p;
 };
 
