@@ -27,20 +27,18 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/s/write_ops/batch_write_exec.h"
 
 #include "mongo/base/error_codes.h"
-#include "mongo/base/owned_pointer_map.h"
 #include "mongo/base/status.h"
 #include "mongo/bson/util/builder.h"
 #include "mongo/client/connection_string.h"
 #include "mongo/client/remote_command_targeter.h"
 #include "mongo/db/error_labels.h"
-#include "mongo/db/logical_session_id_helpers.h"
+#include "mongo/db/session/logical_session_id_helpers.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/log.h"
 #include "mongo/s/client/shard_registry.h"
@@ -48,51 +46,47 @@
 #include "mongo/s/multi_statement_transaction_requests_sender.h"
 #include "mongo/s/transaction_router.h"
 #include "mongo/s/write_ops/batch_write_op.h"
-#include "mongo/s/write_ops/write_error_detail.h"
 #include "mongo/util/exit.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
+
+
 namespace mongo {
 namespace {
 
 const ReadPreferenceSetting kPrimaryOnlyReadPreference(ReadPreference::PrimaryOnly);
 
-//
-// Map which allows associating ConnectionString hosts with TargetedWriteBatches
-// This is needed since the dispatcher only returns hosts with responses.
-//
-
-WriteErrorDetail errorFromStatus(const Status& status) {
-    WriteErrorDetail error;
-    error.setStatus(status);
-    return error;
-}
-
 // Helper to note several stale shard errors from a response
-void noteStaleShardResponses(const std::vector<ShardError>& staleErrors, NSTargeter* targeter) {
+void noteStaleShardResponses(OperationContext* opCtx,
+                             const std::vector<ShardError>& staleErrors,
+                             NSTargeter* targeter) {
     for (const auto& error : staleErrors) {
         LOGV2_DEBUG(22902,
                     4,
                     "Noting stale config response from {shardId}: {errorInfo}",
                     "Noting stale config response",
                     "shardId"_attr = error.endpoint.shardName,
-                    "errorInfo"_attr = error.error.getErrInfo());
-        targeter->noteStaleShardResponse(
-            error.endpoint,
-            StaleConfigInfo::parseFromCommandError(
-                error.error.isErrInfoSet() ? error.error.getErrInfo() : BSONObj()));
+                    "status"_attr = error.error.getStatus());
+
+        auto extraInfo = error.error.getStatus().extraInfo<StaleConfigInfo>();
+        invariant(extraInfo);
+        targeter->noteStaleShardResponse(opCtx, error.endpoint, *extraInfo);
     }
 }
 
 // Helper to note several stale db errors from a response
-void noteStaleDbResponses(const std::vector<ShardError>& staleErrors, NSTargeter* targeter) {
+void noteStaleDbResponses(OperationContext* opCtx,
+                          const std::vector<ShardError>& staleErrors,
+                          NSTargeter* targeter) {
     for (const auto& error : staleErrors) {
         LOGV2_DEBUG(22903,
                     4,
-                    "Noting stale database response from {shardId}: errorInfo",
+                    "Noting stale database response",
                     "shardId"_attr = error.endpoint.shardName,
-                    "errorInfo"_attr = error.error.toBSON());
-        targeter->noteStaleDbResponse(
-            error.endpoint, StaleDbRoutingVersion::parseFromCommandError(error.error.toBSON()));
+                    "status"_attr = error.error.getStatus());
+        auto extraInfo = error.error.getStatus().extraInfo<StaleDbRoutingVersion>();
+        invariant(extraInfo);
+        targeter->noteStaleDbResponse(opCtx, error.endpoint, *extraInfo);
     }
 }
 
@@ -119,7 +113,7 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
                                   const BatchedCommandRequest& clientRequest,
                                   BatchedCommandResponse* clientResponse,
                                   BatchWriteExecStats* stats) {
-    const auto& nss(clientRequest.getNS());
+    const auto& nss(targeter.getNS());
 
     LOGV2_DEBUG(22904,
                 4,
@@ -162,8 +156,7 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
         //    exactly when the metadata changed.
         //
 
-        OwnedPointerMap<ShardId, TargetedWriteBatch> childBatchesOwned;
-        std::map<ShardId, TargetedWriteBatch*>& childBatches = childBatchesOwned.mutableMap();
+        std::map<ShardId, std::unique_ptr<TargetedWriteBatch>> childBatches;
 
         // If we've already had a targeting error, we've refreshed the metadata once and can
         // record target errors definitively.
@@ -173,7 +166,6 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
             // Don't do anything until a targeter refresh
             targeter.noteCouldNotTarget();
             refreshedTargeter = true;
-            ++stats->numTargetErrors;
             dassert(childBatches.size() == 0u);
 
             if (TransactionRouter::get(opCtx)) {
@@ -198,9 +190,7 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
 
         while (numSent != numToSend) {
             // Collect batches out on the network, mapped by endpoint
-            OwnedPointerMap<ShardId, TargetedWriteBatch> ownedPendingBatches;
-            OwnedPointerMap<ShardId, TargetedWriteBatch>::MapType& pendingBatches =
-                ownedPendingBatches.mutableMap();
+            std::map<ShardId, std::unique_ptr<TargetedWriteBatch>> pendingBatches;
 
             //
             // Construct the requests.
@@ -209,8 +199,8 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
             std::vector<AsyncRequestsSender::Request> requests;
 
             // Get as many batches as we can at once
-            for (auto& childBatch : childBatches) {
-                TargetedWriteBatch* const nextBatch = childBatch.second;
+            for (auto&& childBatch : childBatches) {
+                auto nextBatch = std::move(childBatch.second);
 
                 // If the batch is nullptr, we sent it previously, so skip
                 if (!nextBatch)
@@ -218,14 +208,13 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
 
                 // If we already have a batch for this shard, wait until the next time
                 const auto& targetShardId = nextBatch->getEndpoint().shardName;
-
                 if (pendingBatches.count(targetShardId))
                     continue;
 
                 stats->noteTargetedShard(targetShardId);
 
                 const auto request = [&] {
-                    const auto shardBatchRequest(batchOp.buildBatchRequest(*nextBatch));
+                    const auto shardBatchRequest(batchOp.buildBatchRequest(*nextBatch, targeter));
 
                     BSONObjBuilder requestBuilder;
                     shardBatchRequest.serialize(&requestBuilder);
@@ -249,7 +238,7 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
                 childBatch.second = nullptr;
 
                 // Recv-side is responsible for cleaning up the nextBatch when used
-                pendingBatches.emplace(targetShardId, nextBatch);
+                pendingBatches.emplace(targetShardId, std::move(nextBatch));
             }
 
             bool isRetryableWrite = opCtx->getTxnNumber() && !TransactionRouter::get(opCtx);
@@ -273,7 +262,7 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
 
                 // Get the TargetedWriteBatch to find where to put the response
                 dassert(pendingBatches.find(response.shardId) != pendingBatches.end());
-                TargetedWriteBatch* batch = pendingBatches.find(response.shardId)->second;
+                TargetedWriteBatch* batch = pendingBatches.find(response.shardId)->second.get();
 
                 const auto shardInfo = response.shardHostAndPort
                     ? response.shardHostAndPort->toString()
@@ -285,16 +274,16 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
                 if (responseStatus.isOK()) {
                     std::string errMsg;
                     if (!batchedCommandResponse.parseBSON(response.swResponse.getValue().data,
-                                                          &errMsg) ||
-                        !batchedCommandResponse.isValid(&errMsg)) {
+                                                          &errMsg)) {
                         responseStatus = {ErrorCodes::FailedToParse, errMsg};
                     }
                 }
 
                 if (responseStatus.isOK()) {
                     TrackedErrors trackedErrors;
-                    trackedErrors.startTracking(ErrorCodes::StaleShardVersion);
+                    trackedErrors.startTracking(ErrorCodes::StaleConfig);
                     trackedErrors.startTracking(ErrorCodes::StaleDbVersion);
+                    trackedErrors.startTracking(ErrorCodes::TenantMigrationAborted);
 
                     LOGV2_DEBUG(22907,
                                 4,
@@ -330,22 +319,27 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
                     }
 
                     // Note if anything was stale
-                    const auto& staleShardErrors =
-                        trackedErrors.getErrors(ErrorCodes::StaleShardVersion);
+                    auto staleConfigErrors = trackedErrors.getErrors(ErrorCodes::StaleConfig);
                     const auto& staleDbErrors = trackedErrors.getErrors(ErrorCodes::StaleDbVersion);
+                    const auto& tenantMigrationAbortedErrors =
+                        trackedErrors.getErrors(ErrorCodes::TenantMigrationAborted);
 
-                    if (!staleShardErrors.empty()) {
+                    if (!staleConfigErrors.empty()) {
                         invariant(staleDbErrors.empty());
-                        noteStaleShardResponses(staleShardErrors, &targeter);
+                        noteStaleShardResponses(opCtx, staleConfigErrors, &targeter);
                         ++stats->numStaleShardBatches;
                     }
 
                     if (!staleDbErrors.empty()) {
-                        invariant(staleShardErrors.empty());
-                        noteStaleDbResponses(staleDbErrors, &targeter);
+                        invariant(staleConfigErrors.empty());
+                        noteStaleDbResponses(opCtx, staleDbErrors, &targeter);
                         ++stats->numStaleDbBatches;
                     }
 
+                    if (!tenantMigrationAbortedErrors.empty()) {
+                        ++stats->numTenantMigrationAbortedErrors;
+                    }
+
                     if (response.shardHostAndPort) {
                         // Remember that we successfully wrote to this shard
                         // NOTE: This will record lastOps for shards where we actually didn't update
@@ -375,7 +369,7 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
                                               : "from failing to target a host in the shard ")
                                       << shardInfo);
 
-                    batchOp.noteBatchError(*batch, errorFromStatus(status));
+                    batchOp.noteBatchError(*batch, write_ops::WriteError(0, status));
 
                     LOGV2_DEBUG(22908,
                                 4,
@@ -389,8 +383,7 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
                         batchOp.forgetTargetedBatchesOnTransactionAbortingError();
 
                         // Throw when there is a transient transaction error since this should be a
-                        // top
-                        // level error and not just a write error.
+                        // top level error and not just a write error.
                         if (isTransientTransactionError(status.code(), false, false)) {
                             uassertStatusOK(status);
                         }
@@ -416,20 +409,37 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
         //
 
         bool targeterChanged = false;
-        Status refreshStatus = targeter.refreshIfNeeded(opCtx, &targeterChanged);
+        try {
+            LOGV2_DEBUG_OPTIONS(4817406,
+                                2,
+                                {logv2::LogComponent::kShardMigrationPerf},
+                                "Starting post-migration commit refresh on the router");
+            targeterChanged = targeter.refreshIfNeeded(opCtx);
+            LOGV2_DEBUG_OPTIONS(4817407,
+                                2,
+                                {logv2::LogComponent::kShardMigrationPerf},
+                                "Finished post-migration commit refresh on the router");
+        } catch (const ExceptionFor<ErrorCodes::StaleEpoch>& ex) {
+            LOGV2_DEBUG_OPTIONS(4817408,
+                                2,
+                                {logv2::LogComponent::kShardMigrationPerf},
+                                "Finished post-migration commit refresh on the router with error",
+                                "error"_attr = redact(ex));
+            batchOp.abortBatch(write_ops::WriteError(
+                0, ex.toStatus("collection was dropped in the middle of the operation")));
+            break;
+        } catch (const DBException& ex) {
+            LOGV2_DEBUG_OPTIONS(4817409,
+                                2,
+                                {logv2::LogComponent::kShardMigrationPerf},
+                                "Finished post-migration commit refresh on the router with error",
+                                "error"_attr = redact(ex));
 
-        LOGV2_DEBUG(22909,
-                    4,
-                    "executeBatch targeter changed: {targeterChanged}",
-                    "targeterChanged"_attr = targeterChanged);
-
-        if (!refreshStatus.isOK()) {
-            // It's okay if we can't refresh, we'll just record errors for the ops if
-            // needed.
+            // It's okay if we can't refresh, we'll just record errors for the ops if needed
             LOGV2_WARNING(22911,
                           "Could not refresh targeter due to {error}",
                           "Could not refresh targeter",
-                          "error"_attr = redact(refreshStatus.reason()));
+                          "error"_attr = redact(ex));
         }
 
         //
@@ -445,7 +455,8 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
         numCompletedOps = currCompletedOps;
 
         if (numRoundsWithoutProgress > kMaxRoundsWithoutProgress) {
-            batchOp.abortBatch(errorFromStatus(
+            batchOp.abortBatch(write_ops::WriteError(
+                0,
                 {ErrorCodes::NoProgressMade,
                  str::stream() << "no progress was made executing batch write op in "
                                << clientRequest.getNS().ns() << " after "
@@ -456,8 +467,8 @@ void BatchWriteExec::executeBatch(OperationContext* opCtx,
     }
 
     auto nShardsOwningChunks = batchOp.getNShardsOwningChunks();
-    if (nShardsOwningChunks.is_initialized())
-        stats->noteNumShardsOwningChunks(nShardsOwningChunks.get());
+    if (nShardsOwningChunks)
+        stats->noteNumShardsOwningChunks(*nShardsOwningChunks);
 
     batchOp.buildClientResponse(clientResponse);
 
@@ -495,7 +506,7 @@ const HostOpTimeMap& BatchWriteExecStats::getWriteOpTimes() const {
     return _writeOpTimes;
 }
 
-const boost::optional<int> BatchWriteExecStats::getNumShardsOwningChunks() const {
+boost::optional<int> BatchWriteExecStats::getNumShardsOwningChunks() const {
     return _numShardsOwningChunks;
 }
 
