@@ -31,31 +31,25 @@
 
 #include "mongo/s/write_ops/batch_write_op.h"
 
-#include <memory>
 #include <numeric>
 
 #include "mongo/base/error_codes.h"
+#include "mongo/db/catalog/collection_uuid_mismatch_info.h"
 #include "mongo/db/operation_context.h"
-#include "mongo/db/ops/write_ops_parsers.h"
-#include "mongo/db/s/database_sharding_state.h"
+#include "mongo/db/ops/write_ops.h"
 #include "mongo/s/client/num_hosts_targeted_metrics.h"
 #include "mongo/s/cluster_commands_helpers.h"
-#include "mongo/s/grid.h"
+#include "mongo/s/collection_uuid_mismatch.h"
 #include "mongo/s/transaction_router.h"
 #include "mongo/util/transitional_tools_do_not_use/vector_spooling.h"
 
 namespace mongo {
-
-using std::set;
-using std::stringstream;
-using std::unique_ptr;
-using std::vector;
-
 namespace {
 
-struct WriteErrorDetailComp {
-    bool operator()(const WriteErrorDetail* errorA, const WriteErrorDetail* errorB) const {
-        return errorA->getIndex() < errorB->getIndex();
+struct WriteErrorComp {
+    bool operator()(const write_ops::WriteError& errorA,
+                    const write_ops::WriteError& errorB) const {
+        return errorA.getIndex() < errorB.getIndex();
     }
 };
 
@@ -89,16 +83,12 @@ BSONObj upgradeWriteConcern(const BSONObj& origWriteConcern) {
     return newWriteConcern.obj();
 }
 
-void buildTargetError(const Status& errStatus, WriteErrorDetail* details) {
-    details->setStatus(errStatus);
-}
-
 /**
  * Helper to determine whether a number of targeted writes require a new targeted batch.
  */
-bool isNewBatchRequiredOrdered(const std::vector<TargetedWrite*>& writes,
+bool isNewBatchRequiredOrdered(const std::vector<std::unique_ptr<TargetedWrite>>& writes,
                                const TargetedBatchMap& batchMap) {
-    for (const auto write : writes) {
+    for (auto&& write : writes) {
         if (batchMap.find(&write->endpoint) == batchMap.end()) {
             return true;
         }
@@ -112,10 +102,10 @@ bool isNewBatchRequiredOrdered(const std::vector<TargetedWrite*>& writes,
  * necessitates a new batch. This happens when a batch write incldues a multi target write and
  * a single target write.
  */
-bool isNewBatchRequiredUnordered(const std::vector<TargetedWrite*>& writes,
+bool isNewBatchRequiredUnordered(const std::vector<std::unique_ptr<TargetedWrite>>& writes,
                                  const TargetedBatchMap& batchMap,
                                  const std::set<ShardId>& targetedShards) {
-    for (const auto write : writes) {
+    for (auto&& write : writes) {
         if (batchMap.find(&write->endpoint) == batchMap.end()) {
             if (targetedShards.find((&write->endpoint)->shardName) != targetedShards.end()) {
                 return true;
@@ -129,24 +119,22 @@ bool isNewBatchRequiredUnordered(const std::vector<TargetedWrite*>& writes,
 /**
  * Helper to determine whether a number of targeted writes require a new targeted batch.
  */
-bool wouldMakeBatchesTooBig(const std::vector<TargetedWrite*>& writes,
+bool wouldMakeBatchesTooBig(const std::vector<std::unique_ptr<TargetedWrite>>& writes,
                             int writeSizeBytes,
                             const TargetedBatchMap& batchMap) {
-    for (const auto write : writes) {
+    for (auto&& write : writes) {
         TargetedBatchMap::const_iterator it = batchMap.find(&write->endpoint);
         if (it == batchMap.end()) {
             // If this is the first item in the batch, it can't be too big
             continue;
         }
 
-        const auto& batch = it->second;
-
-        if (batch->getNumOps() >= write_ops::kMaxWriteBatchSize) {
+        if (it->second->getNumOps() >= write_ops::kMaxWriteBatchSize) {
             // Too many items in batch
             return true;
         }
 
-        if (batch->getEstimatedSizeBytes() + writeSizeBytes > BSONObjMaxUserSize) {
+        if (it->second->getEstimatedSizeBytes() + writeSizeBytes > BSONObjMaxUserSize) {
             // Batch would be too big
             return true;
         }
@@ -195,6 +183,11 @@ int getWriteSizeBytes(const WriteOp& writeOp) {
         // Add the size of the 'limit' field.
         estSize += DeleteOpEntry::kMultiFieldName.size() + intSize;
 
+        // Add the size of 'hint' field if present.
+        if (auto hint = item.getDelete().getHint(); !hint.isEmpty()) {
+            estSize += DeleteOpEntry::kHintFieldName.size() + hint.objsize();
+        }
+
         // Add the size of the 'q' field, plus the constant deleteOp overhead size.
         estSize += kEstDeleteOverheadBytes +
             (DeleteOpEntry::kQFieldName.size() + item.getDelete().getQ().objsize());
@@ -212,22 +205,70 @@ int getWriteSizeBytes(const WriteOp& writeOp) {
  * into a TrackedErrorMap
  */
 void trackErrors(const ShardEndpoint& endpoint,
-                 const vector<WriteErrorDetail*> itemErrors,
+                 const std::vector<write_ops::WriteError>& itemErrors,
                  TrackedErrors* trackedErrors) {
-    for (const auto error : itemErrors) {
-        if (trackedErrors->isTracking(error->toStatus().code())) {
-            trackedErrors->addError(ShardError(endpoint, *error));
+    for (auto&& error : itemErrors) {
+        if (trackedErrors->isTracking(error.getStatus().code())) {
+            trackedErrors->addError(ShardError(endpoint, error));
         }
     }
 }
 
+/**
+ * Attempts to populate the actualCollection field of a CollectionUUIDMismatch error if it is not
+ * populated already, contacting the primary shard if necessary.
+ */
+void populateCollectionUUIDMismatch(OperationContext* opCtx,
+                                    write_ops::WriteError* error,
+                                    boost::optional<std::string>* actualCollection,
+                                    bool* hasContactedPrimaryShard) {
+    if (error->getStatus() != ErrorCodes::CollectionUUIDMismatch) {
+        return;
+    }
+
+    auto info = error->getStatus().extraInfo<CollectionUUIDMismatchInfo>();
+    if (info->actualCollection()) {
+        return;
+    }
+
+    if (*actualCollection) {
+        error->setStatus({CollectionUUIDMismatchInfo{info->db(),
+                                                     info->collectionUUID(),
+                                                     info->expectedCollection(),
+                                                     **actualCollection},
+                          error->getStatus().reason()});
+        return;
+    }
+
+    if (*hasContactedPrimaryShard) {
+        return;
+    }
+
+    error->setStatus(populateCollectionUUIDMismatch(opCtx, error->getStatus()));
+    if (error->getStatus() == ErrorCodes::CollectionUUIDMismatch) {
+        *hasContactedPrimaryShard = true;
+        if (auto& populatedActualCollection =
+                error->getStatus().extraInfo<CollectionUUIDMismatchInfo>()->actualCollection()) {
+            *actualCollection = populatedActualCollection;
+        }
+    }
+}
+
+int getEncryptionInformationSize(const BatchedCommandRequest& req) {
+    if (!req.getWriteCommandRequestBase().getEncryptionInformation()) {
+        return 0;
+    }
+    return req.getWriteCommandRequestBase().getEncryptionInformation().value().toBSON().objsize();
+}
+
 }  // namespace
 
 BatchWriteOp::BatchWriteOp(OperationContext* opCtx, const BatchedCommandRequest& clientRequest)
     : _opCtx(opCtx),
       _clientRequest(clientRequest),
       _batchTxnNum(_opCtx->getTxnNumber()),
-      _inTransaction(bool(TransactionRouter::get(opCtx))) {
+      _inTransaction(bool(TransactionRouter::get(opCtx))),
+      _isRetryableWrite(opCtx->isRetryableWrite()) {
     _writeOps.reserve(_clientRequest.sizeWriteOps());
 
     for (size_t i = 0; i < _clientRequest.sizeWriteOps(); ++i) {
@@ -235,9 +276,10 @@ BatchWriteOp::BatchWriteOp(OperationContext* opCtx, const BatchedCommandRequest&
     }
 }
 
-Status BatchWriteOp::targetBatch(const NSTargeter& targeter,
-                                 bool recordTargetErrors,
-                                 std::map<ShardId, TargetedWriteBatch*>* targetedBatches) {
+Status BatchWriteOp::targetBatch(
+    const NSTargeter& targeter,
+    bool recordTargetErrors,
+    std::map<ShardId, std::unique_ptr<TargetedWriteBatch>>* targetedBatches) {
     //
     // Targeting of unordered batches is fairly simple - each remaining write op is targeted,
     // and each of those targeted writes are grouped into a batch for a particular shard
@@ -271,13 +313,11 @@ Status BatchWriteOp::targetBatch(const NSTargeter& targeter,
     //  [{ skey : y }, { skey : z }]
     //
 
-    const bool ordered = _clientRequest.getWriteCommandBase().getOrdered();
+    const bool ordered = _clientRequest.getWriteCommandRequestBase().getOrdered();
 
     TargetedBatchMap batchMap;
     std::set<ShardId> targetedShards;
 
-    int numTargetErrors = 0;
-
     const size_t numWriteOps = _clientRequest.sizeWriteOps();
 
     for (size_t i = 0; i < numWriteOps; ++i) {
@@ -291,19 +331,20 @@ Status BatchWriteOp::targetBatch(const NSTargeter& targeter,
         // Get TargetedWrites from the targeter for the write operation
         //
         // TargetedWrites need to be owned once returned
+        std::vector<std::unique_ptr<TargetedWrite>> writes;
 
-        OwnedPointerVector<TargetedWrite> writesOwned;
-        vector<TargetedWrite*>& writes = writesOwned.mutableVector();
-
-        Status targetStatus = writeOp.targetWrites(_opCtx, targeter, &writes);
+        Status targetStatus = Status::OK();
+        try {
+            writeOp.targetWrites(_opCtx, targeter, &writes);
+        } catch (const DBException& ex) {
+            targetStatus = ex.toStatus();
+        }
 
         if (!targetStatus.isOK()) {
-            WriteErrorDetail targetError;
-            buildTargetError(targetStatus, &targetError);
+            write_ops::WriteError targetError(0, targetStatus);
 
             if (TransactionRouter::get(_opCtx)) {
                 writeOp.setOpError(targetError);
-                ++numTargetErrors;
 
                 // Cleanup all the writes we have targetted in this call so far since we are going
                 // to abort the entire transaction.
@@ -318,7 +359,6 @@ Status BatchWriteOp::targetBatch(const NSTargeter& targeter,
                 // Record an error for this batch
 
                 writeOp.setOpError(targetError);
-                ++numTargetErrors;
 
                 if (ordered)
                     return Status::OK();
@@ -347,13 +387,28 @@ Status BatchWriteOp::targetBatch(const NSTargeter& targeter,
             }
         }
 
-        // Account the array overhead once for the actual updates array and once for the statement
-        // ids array, if retryable writes are used
+        // If retryable writes are used, MongoS needs to send an additional array of stmtId(s)
+        // corresponding to the statements that got routed to each individual shard, so they need to
+        // be accounted in the potential request size so it does not exceed the max BSON size.
+        //
+        // The constant 4 is chosen as the size of the BSON representation of the stmtId.
         const int writeSizeBytes = getWriteSizeBytes(writeOp) +
+            getEncryptionInformationSize(_clientRequest) +
             write_ops::kWriteCommandBSONArrayPerElementOverheadBytes +
             (_batchTxnNum ? write_ops::kWriteCommandBSONArrayPerElementOverheadBytes + 4 : 0);
 
-        if (wouldMakeBatchesTooBig(writes, writeSizeBytes, batchMap)) {
+        // For unordered writes, the router must return an entry for each failed write. This
+        // constant is a pessimistic attempt to ensure that if a request to a shard hits
+        // "retargeting needed" error and has to return number of errors equivalent to the number of
+        // writes in the batch, the response size will not exceed the max BSON size.
+        //
+        // The constant of 272 is chosen as an approximation of the size of the BSON representation
+        // of the StaleConfigInfo (which contains the shard id) and the adjacent error message.
+        const int errorResponsePotentialSizeBytes =
+            ordered ? 0 : write_ops::kWriteCommandBSONArrayPerElementOverheadBytes + 272;
+
+        if (wouldMakeBatchesTooBig(
+                writes, std::max(writeSizeBytes, errorResponsePotentialSizeBytes), batchMap)) {
             invariant(!batchMap.empty());
             writeOp.cancelWrites(nullptr);
             break;
@@ -369,20 +424,21 @@ Status BatchWriteOp::targetBatch(const NSTargeter& targeter,
         // Targeting went ok, add to appropriate TargetedBatch
         //
 
-        for (const auto write : writes) {
+        for (auto&& write : writes) {
             TargetedBatchMap::iterator batchIt = batchMap.find(&write->endpoint);
             if (batchIt == batchMap.end()) {
-                TargetedWriteBatch* newBatch = new TargetedWriteBatch(write->endpoint);
-                batchIt = batchMap.emplace(&newBatch->getEndpoint(), newBatch).first;
-                targetedShards.insert((&newBatch->getEndpoint())->shardName);
+                auto newBatch = std::make_unique<TargetedWriteBatch>(write->endpoint);
+                auto endpoint = &newBatch->getEndpoint();
+                batchIt = batchMap.emplace(endpoint, std::move(newBatch)).first;
+                targetedShards.insert(endpoint->shardName);
             }
 
-            TargetedWriteBatch* batch = batchIt->second;
-            batch->addWrite(write, writeSizeBytes);
+            batchIt->second->addWrite(std::move(write),
+                                      std::max(writeSizeBytes, errorResponsePotentialSizeBytes));
         }
 
         // Relinquish ownership of TargetedWrites, now the TargetedBatches own them
-        writesOwned.mutableVector().clear();
+        writes.clear();
 
         //
         // Break if we're ordered and we have more than one endpoint - later writes cannot be
@@ -398,17 +454,16 @@ Status BatchWriteOp::targetBatch(const NSTargeter& targeter,
     //
 
     for (TargetedBatchMap::iterator it = batchMap.begin(); it != batchMap.end(); ++it) {
-        TargetedWriteBatch* batch = it->second;
-
+        auto batch = std::move(it->second);
         if (batch->getWrites().empty())
             continue;
 
         // Remember targeted batch for reporting
-        _targeted.insert(batch);
+        _targeted.insert(batch.get());
 
         // Send the handle back to caller
         invariant(targetedBatches->find(batch->getEndpoint().shardName) == targetedBatches->end());
-        targetedBatches->emplace(batch->getEndpoint().shardName, batch);
+        targetedBatches->emplace(batch->getEndpoint().shardName, std::move(batch));
     }
 
     _nShardsOwningChunks = targeter.getNShardsOwningChunks();
@@ -416,12 +471,12 @@ Status BatchWriteOp::targetBatch(const NSTargeter& targeter,
     return Status::OK();
 }
 
-BatchedCommandRequest BatchWriteOp::buildBatchRequest(
-    const TargetedWriteBatch& targetedBatch) const {
+BatchedCommandRequest BatchWriteOp::buildBatchRequest(const TargetedWriteBatch& targetedBatch,
+                                                      const NSTargeter& targeter) const {
     const auto batchType = _clientRequest.getBatchType();
 
     boost::optional<std::vector<int32_t>> stmtIdsForOp;
-    if (_batchTxnNum) {
+    if (_isRetryableWrite) {
         stmtIdsForOp.emplace();
     }
 
@@ -429,7 +484,7 @@ BatchedCommandRequest BatchWriteOp::buildBatchRequest(
     boost::optional<std::vector<write_ops::UpdateOpEntry>> updates;
     boost::optional<std::vector<write_ops::DeleteOpEntry>> deletes;
 
-    for (const auto& targetedWrite : targetedBatch.getWrites()) {
+    for (auto&& targetedWrite : targetedBatch.getWrites()) {
         const WriteOpRef& writeOpRef = targetedWrite->writeOpRef;
 
         switch (batchType) {
@@ -457,7 +512,7 @@ BatchedCommandRequest BatchWriteOp::buildBatchRequest(
 
         if (stmtIdsForOp) {
             stmtIdsForOp->push_back(write_ops::getStmtIdForWriteAt(
-                _clientRequest.getWriteCommandBase(), writeOpRef.first));
+                _clientRequest.getWriteCommandRequestBase(), writeOpRef.first));
         }
     }
 
@@ -465,37 +520,51 @@ BatchedCommandRequest BatchWriteOp::buildBatchRequest(
         switch (batchType) {
             case BatchedCommandRequest::BatchType_Insert:
                 return BatchedCommandRequest([&] {
-                    write_ops::Insert insertOp(_clientRequest.getNS());
+                    write_ops::InsertCommandRequest insertOp(targeter.getNS());
                     insertOp.setDocuments(std::move(*insertDocs));
                     return insertOp;
                 }());
             case BatchedCommandRequest::BatchType_Update: {
                 return BatchedCommandRequest([&] {
-                    write_ops::Update updateOp(_clientRequest.getNS());
+                    write_ops::UpdateCommandRequest updateOp(targeter.getNS());
                     updateOp.setUpdates(std::move(*updates));
-                    // Each child batch inherits its runtime constants from the parent batch.
-                    updateOp.setRuntimeConstants(_clientRequest.getRuntimeConstants());
+                    // Each child batch inherits its let params/runtime constants from the parent
+                    // batch.
+                    updateOp.setLegacyRuntimeConstants(_clientRequest.getLegacyRuntimeConstants());
+                    updateOp.setLet(_clientRequest.getLet());
                     return updateOp;
                 }());
             }
             case BatchedCommandRequest::BatchType_Delete:
                 return BatchedCommandRequest([&] {
-                    write_ops::Delete deleteOp(_clientRequest.getNS());
+                    write_ops::DeleteCommandRequest deleteOp(targeter.getNS());
                     deleteOp.setDeletes(std::move(*deletes));
+                    // Each child batch inherits its let params from the parent batch.
+                    deleteOp.setLet(_clientRequest.getLet());
+                    deleteOp.setLegacyRuntimeConstants(_clientRequest.getLegacyRuntimeConstants());
                     return deleteOp;
                 }());
         }
         MONGO_UNREACHABLE;
     }());
 
-    request.setWriteCommandBase([&] {
-        write_ops::WriteCommandBase wcb;
+    request.setWriteCommandRequestBase([&] {
+        write_ops::WriteCommandRequestBase wcb;
 
         wcb.setBypassDocumentValidation(
-            _clientRequest.getWriteCommandBase().getBypassDocumentValidation());
-        wcb.setOrdered(_clientRequest.getWriteCommandBase().getOrdered());
+            _clientRequest.getWriteCommandRequestBase().getBypassDocumentValidation());
+        wcb.setOrdered(_clientRequest.getWriteCommandRequestBase().getOrdered());
+        wcb.setCollectionUUID(_clientRequest.getWriteCommandRequestBase().getCollectionUUID());
 
-        if (_batchTxnNum) {
+        wcb.setEncryptionInformation(
+            _clientRequest.getWriteCommandRequestBase().getEncryptionInformation());
+
+        if (targeter.isShardedTimeSeriesBucketsNamespace() &&
+            !_clientRequest.getNS().isTimeseriesBucketsCollection()) {
+            wcb.setIsTimeseriesNamespace(true);
+        }
+
+        if (_isRetryableWrite) {
             wcb.setStmtIds(std::move(stmtIdsForOp));
         }
 
@@ -504,18 +573,12 @@ BatchedCommandRequest BatchWriteOp::buildBatchRequest(
 
 
     auto shardVersion = targetedBatch.getEndpoint().shardVersion;
-    request.setShardVersion(shardVersion);
+    if (shardVersion)
+        request.setShardVersion(*shardVersion);
 
     auto dbVersion = targetedBatch.getEndpoint().databaseVersion;
-    invariant((shardVersion == ChunkVersion::UNSHARDED() && dbVersion) ||
-              (shardVersion != ChunkVersion::UNSHARDED() && !dbVersion));
-
-    if (serverGlobalParams.featureCompatibility.getVersion() ==
-        ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44) {
-        if (dbVersion) {
-            request.setDbVersion(dbVersion.get());
-        }
-    }
+    if (dbVersion)
+        request.setDbVersion(*dbVersion);
 
     if (_clientRequest.hasWriteConcern()) {
         if (_clientRequest.isVerboseWC()) {
@@ -530,8 +593,6 @@ BatchedCommandRequest BatchWriteOp::buildBatchRequest(
         request.setWriteConcern(_opCtx->getWriteConcern().toBSON());
     }
 
-    request.setAllowImplicitCreate(_clientRequest.isImplicitCreateAllowed());
-
     return request;
 }
 
@@ -539,8 +600,7 @@ void BatchWriteOp::noteBatchResponse(const TargetedWriteBatch& targetedBatch,
                                      const BatchedCommandResponse& response,
                                      TrackedErrors* trackedErrors) {
     if (!response.getOk()) {
-        WriteErrorDetail error;
-        error.setStatus(response.getTopLevelStatus());
+        write_ops::WriteError error(0, response.getTopLevelStatus());
 
         // Treat command errors exactly like other failures of the batch.
         //
@@ -565,7 +625,7 @@ void BatchWriteOp::noteBatchResponse(const TargetedWriteBatch& targetedBatch,
         _wcErrors.emplace_back(targetedBatch.getEndpoint(), *response.getWriteConcernError());
     }
 
-    vector<WriteErrorDetail*> itemErrors;
+    std::vector<write_ops::WriteError> itemErrors;
 
     // Handle batch and per-item errors
     if (response.isErrDetailsSet()) {
@@ -574,40 +634,36 @@ void BatchWriteOp::noteBatchResponse(const TargetedWriteBatch& targetedBatch,
             itemErrors.begin(), response.getErrDetails().begin(), response.getErrDetails().end());
 
         // Sort per-item errors by index
-        std::sort(itemErrors.begin(), itemErrors.end(), WriteErrorDetailComp());
+        std::sort(itemErrors.begin(), itemErrors.end(), WriteErrorComp());
     }
 
     //
-    // Go through all pending responses of the op and sorted remote reponses, populate errors
+    // Go through all pending responses of the op and sorted remote responses, populate errors
     // This will either set all errors to the batch error or apply per-item errors as-needed
     //
     // If the batch is ordered, cancel all writes after the first error for retargeting.
     //
 
-    const bool ordered = _clientRequest.getWriteCommandBase().getOrdered();
+    const bool ordered = _clientRequest.getWriteCommandRequestBase().getOrdered();
 
-    vector<WriteErrorDetail*>::iterator itemErrorIt = itemErrors.begin();
+    auto itemErrorIt = itemErrors.begin();
     int index = 0;
-    WriteErrorDetail* lastError = nullptr;
-    for (vector<TargetedWrite*>::const_iterator it = targetedBatch.getWrites().begin();
-         it != targetedBatch.getWrites().end();
-         ++it, ++index) {
-        const TargetedWrite* write = *it;
+    write_ops::WriteError* lastError = nullptr;
+    for (auto&& write : targetedBatch.getWrites()) {
         WriteOp& writeOp = _writeOps[write->writeOpRef.first];
-
-        dassert(writeOp.getWriteState() == WriteOpState_Pending);
+        invariant(writeOp.getWriteState() == WriteOpState_Pending);
 
         // See if we have an error for the write
-        WriteErrorDetail* writeError = nullptr;
+        write_ops::WriteError* writeError = nullptr;
 
-        if (itemErrorIt != itemErrors.end() && (*itemErrorIt)->getIndex() == index) {
+        if (itemErrorIt != itemErrors.end() && itemErrorIt->getIndex() == index) {
             // We have an per-item error for this write op's index
-            writeError = *itemErrorIt;
+            writeError = &(*itemErrorIt);
             ++itemErrorIt;
         }
 
         // Finish the response (with error, if needed)
-        if (nullptr == writeError) {
+        if (!writeError) {
             if (!ordered || !lastError) {
                 writeOp.noteWriteComplete(*write);
             } else {
@@ -619,6 +675,7 @@ void BatchWriteOp::noteBatchResponse(const TargetedWriteBatch& targetedBatch,
             writeOp.noteWriteError(*write, *writeError);
             lastError = writeError;
         }
+        ++index;
     }
 
     // Track errors we care about, whether batch or individual errors
@@ -628,8 +685,8 @@ void BatchWriteOp::noteBatchResponse(const TargetedWriteBatch& targetedBatch,
 
     // Track upserted ids if we need to
     if (response.isUpsertDetailsSet()) {
-        const vector<BatchedUpsertDetail*>& upsertedIds = response.getUpsertDetails();
-        for (vector<BatchedUpsertDetail*>::const_iterator it = upsertedIds.begin();
+        const std::vector<BatchedUpsertDetail*>& upsertedIds = response.getUpsertDetails();
+        for (std::vector<BatchedUpsertDetail*>::const_iterator it = upsertedIds.begin();
              it != upsertedIds.end();
              ++it) {
             // The child upserted details don't have the correct index for the full batch
@@ -649,32 +706,31 @@ void BatchWriteOp::noteBatchResponse(const TargetedWriteBatch& targetedBatch,
 }
 
 void BatchWriteOp::noteBatchError(const TargetedWriteBatch& targetedBatch,
-                                  const WriteErrorDetail& error) {
+                                  const write_ops::WriteError& error) {
     // Treat errors to get a batch response as failures of the contained writes
     BatchedCommandResponse emulatedResponse;
     emulatedResponse.setStatus(Status::OK());
     emulatedResponse.setN(0);
 
-    const int numErrors =
-        _clientRequest.getWriteCommandBase().getOrdered() ? 1 : targetedBatch.getWrites().size();
+    const int numErrors = _clientRequest.getWriteCommandRequestBase().getOrdered()
+        ? 1
+        : targetedBatch.getWrites().size();
 
     for (int i = 0; i < numErrors; i++) {
-        auto errorClone(std::make_unique<WriteErrorDetail>());
-        error.cloneTo(errorClone.get());
-        errorClone->setIndex(i);
-        emulatedResponse.addToErrDetails(errorClone.release());
+        write_ops::WriteError errorClone = error;
+        errorClone.setIndex(i);
+        emulatedResponse.addToErrDetails(std::move(errorClone));
     }
 
-    dassert(emulatedResponse.isValid(nullptr));
     noteBatchResponse(targetedBatch, emulatedResponse, nullptr);
 }
 
-void BatchWriteOp::abortBatch(const WriteErrorDetail& error) {
+void BatchWriteOp::abortBatch(const write_ops::WriteError& error) {
     dassert(!isFinished());
     dassert(numWriteOpsIn(WriteOpState_Pending) == 0);
 
     const size_t numWriteOps = _clientRequest.sizeWriteOps();
-    const bool orderedOps = _clientRequest.getWriteCommandBase().getOrdered();
+    const bool orderedOps = _clientRequest.getWriteCommandRequestBase().getOrdered();
     for (size_t i = 0; i < numWriteOps; ++i) {
         WriteOp& writeOp = _writeOps[i];
         // Can only be called with no outstanding batches
@@ -698,7 +754,7 @@ void BatchWriteOp::forgetTargetedBatchesOnTransactionAbortingError() {
 
 bool BatchWriteOp::isFinished() {
     const size_t numWriteOps = _clientRequest.sizeWriteOps();
-    const bool orderedOps = _clientRequest.getWriteCommandBase().getOrdered();
+    const bool orderedOps = _clientRequest.getWriteCommandRequestBase().getOrdered();
     for (size_t i = 0; i < numWriteOps; ++i) {
         WriteOp& writeOp = _writeOps[i];
         if (writeOp.getWriteState() < WriteOpState_Completed)
@@ -711,7 +767,7 @@ bool BatchWriteOp::isFinished() {
 }
 
 void BatchWriteOp::buildClientResponse(BatchedCommandResponse* batchResp) {
-    // Note: we aggresively abandon the batch when encountering errors during transactions, so
+    // Note: we aggressively abandon the batch when encountering errors during transactions, so
     // it can be in a state that is not "finished" even for unordered batches.
     dassert(_inTransaction || isFinished());
 
@@ -720,7 +776,6 @@ void BatchWriteOp::buildClientResponse(BatchedCommandResponse* batchResp) {
 
     // For non-verbose, it's all we need.
     if (!_clientRequest.isVerboseWC()) {
-        dassert(batchResp->isValid(nullptr));
         return;
     }
 
@@ -728,7 +783,7 @@ void BatchWriteOp::buildClientResponse(BatchedCommandResponse* batchResp) {
     // Find all the errors in the batch
     //
 
-    vector<WriteOp*> errOps;
+    std::vector<WriteOp*> errOps;
 
     const size_t numWriteOps = _clientRequest.sizeWriteOps();
     for (size_t i = 0; i < numWriteOps; ++i) {
@@ -744,17 +799,36 @@ void BatchWriteOp::buildClientResponse(BatchedCommandResponse* batchResp) {
     //
 
     if (!errOps.empty()) {
-        for (vector<WriteOp*>::iterator it = errOps.begin(); it != errOps.end(); ++it) {
+        boost::optional<std::string> collectionUUIDMismatchActualCollection;
+
+        for (std::vector<WriteOp*>::iterator it = errOps.begin(); it != errOps.end(); ++it) {
             WriteOp& writeOp = **it;
-            WriteErrorDetail* error = new WriteErrorDetail();
-            writeOp.getOpError().cloneTo(error);
-            batchResp->addToErrDetails(error);
+            write_ops::WriteError error = writeOp.getOpError();
+            auto status = error.getStatus();
+
+            // For CollectionUUIDMismatch error, check if there is a response from a shard that
+            // aleady has the actualCollection information. If there is none, make an additional
+            // call to the primary shard to fetch this info in case the collection is unsharded or
+            // the targeted shard does not own any chunk of the collection with the requested uuid.
+            if (!collectionUUIDMismatchActualCollection &&
+                status.code() == ErrorCodes::CollectionUUIDMismatch) {
+                collectionUUIDMismatchActualCollection =
+                    status.extraInfo<CollectionUUIDMismatchInfo>()->actualCollection();
+            }
+
+            batchResp->addToErrDetails(std::move(error));
+        }
+
+        bool hasContactedPrimaryShard = false;
+        for (auto& error : batchResp->getErrDetails()) {
+            populateCollectionUUIDMismatch(
+                _opCtx, &error, &collectionUUIDMismatchActualCollection, &hasContactedPrimaryShard);
         }
     }
 
     // Only return a write concern error if everything succeeded (unordered or ordered)
     // OR if something succeeded and we're unordered
-    const bool orderedOps = _clientRequest.getWriteCommandBase().getOrdered();
+    const bool orderedOps = _clientRequest.getWriteCommandRequestBase().getOrdered();
     const bool reportWCError =
         errOps.empty() || (!orderedOps && errOps.size() < _clientRequest.sizeWriteOps());
     if (!_wcErrors.empty() && reportWCError) {
@@ -798,8 +872,9 @@ void BatchWriteOp::buildClientResponse(BatchedCommandResponse* batchResp) {
         _numModified >= 0) {
         batchResp->setNModified(_numModified);
     }
-
-    dassert(batchResp->isValid(nullptr));
+    if (!_retriedStmtIds.empty()) {
+        batchResp->setRetriedStmtIds(_retriedStmtIds);
+    }
 }
 
 int BatchWriteOp::numWriteOpsIn(WriteOpState opState) const {
@@ -837,63 +912,65 @@ void BatchWriteOp::_incBatchStats(const BatchedCommandResponse& response) {
         dassert(batchType == BatchedCommandRequest::BatchType_Delete);
         _numDeleted += response.getN();
     }
+
+    if (auto retriedStmtIds = response.getRetriedStmtIds(); !retriedStmtIds.empty()) {
+        _retriedStmtIds.insert(_retriedStmtIds.end(), retriedStmtIds.begin(), retriedStmtIds.end());
+    }
 }
 
-void BatchWriteOp::_cancelBatches(const WriteErrorDetail& why,
+void BatchWriteOp::_cancelBatches(const write_ops::WriteError& why,
                                   TargetedBatchMap&& batchMapToCancel) {
-    TargetedBatchMap batchMap(batchMapToCancel);
-
     // Collect all the writeOps that are currently targeted
-    for (TargetedBatchMap::iterator it = batchMap.begin(); it != batchMap.end();) {
-        TargetedWriteBatch* batch = it->second;
-        const vector<TargetedWrite*>& writes = batch->getWrites();
-
-        for (vector<TargetedWrite*>::const_iterator writeIt = writes.begin();
-             writeIt != writes.end();
-             ++writeIt) {
-            TargetedWrite* write = *writeIt;
-
+    for (TargetedBatchMap::iterator it = batchMapToCancel.begin(); it != batchMapToCancel.end();) {
+        for (auto&& write : it->second->getWrites()) {
             // NOTE: We may repeatedly cancel a write op here, but that's fast and we want to cancel
             // before erasing the TargetedWrite* (which owns the cancelled targeting info) for
             // reporting reasons.
             _writeOps[write->writeOpRef.first].cancelWrites(&why);
         }
 
-        // Note that we need to *erase* first, *then* delete, since the map keys are ptrs from
-        // the values
-        batchMap.erase(it++);
-        delete batch;
+        it = batchMapToCancel.erase(it);
     }
 }
 
 bool EndpointComp::operator()(const ShardEndpoint* endpointA,
                               const ShardEndpoint* endpointB) const {
     const int shardNameDiff = endpointA->shardName.compare(endpointB->shardName);
-    if (shardNameDiff) {
+    if (shardNameDiff)
         return shardNameDiff < 0;
-    }
 
-    const long shardVersionDiff =
-        endpointA->shardVersion.toLong() - endpointB->shardVersion.toLong();
-    if (shardVersionDiff) {
-        return shardVersionDiff < 0;
-    }
+    if (endpointA->shardVersion && endpointB->shardVersion) {
+        const int epochDiff =
+            endpointA->shardVersion->epoch().compare(endpointB->shardVersion->epoch());
+        if (epochDiff)
+            return epochDiff < 0;
 
-    const long epochDiff = endpointA->shardVersion.epoch().compare(endpointB->shardVersion.epoch());
-    if (epochDiff) {
-        return epochDiff < 0;
+        const int shardVersionDiff =
+            endpointA->shardVersion->toLong() - endpointB->shardVersion->toLong();
+        if (shardVersionDiff)
+            return shardVersionDiff < 0;
+    } else if (!endpointA->shardVersion && !endpointB->shardVersion) {
+        // TODO (SERVER-51070): Can only happen if the destination is the config server
+        return false;
+    } else {
+        // TODO (SERVER-51070): Can only happen if the destination is the config server
+        return !endpointA->shardVersion && endpointB->shardVersion;
     }
 
     if (endpointA->databaseVersion && endpointB->databaseVersion) {
-        if (endpointA->databaseVersion->getUuid() < endpointB->databaseVersion->getUuid())
-            return true;
+        const int uuidDiff =
+            endpointA->databaseVersion->getUuid().compare(endpointB->databaseVersion->getUuid());
+        if (uuidDiff)
+            return uuidDiff < 0;
 
-        return (endpointA->databaseVersion->getLastMod() -
-                    endpointB->databaseVersion->getLastMod() <
-                0);
+        return endpointA->databaseVersion->getLastMod() < endpointB->databaseVersion->getLastMod();
+    } else if (!endpointA->databaseVersion && !endpointB->databaseVersion) {
+        return false;
+    } else {
+        return !endpointA->databaseVersion && endpointB->databaseVersion;
     }
 
-    return false;
+    MONGO_UNREACHABLE;
 }
 
 void TrackedErrors::startTracking(int errCode) {
@@ -906,7 +983,7 @@ bool TrackedErrors::isTracking(int errCode) const {
 }
 
 void TrackedErrors::addError(ShardError error) {
-    TrackedErrorMap::iterator seenIt = _errorMap.find(error.error.toStatus().code());
+    TrackedErrorMap::iterator seenIt = _errorMap.find(error.error.getStatus().code());
     if (seenIt == _errorMap.end())
         return;
     seenIt->second.emplace_back(std::move(error));
@@ -917,8 +994,8 @@ const std::vector<ShardError>& TrackedErrors::getErrors(int errCode) const {
     return _errorMap.find(errCode)->second;
 }
 
-void TargetedWriteBatch::addWrite(TargetedWrite* targetedWrite, int estWriteSize) {
-    _writes.mutableVector().push_back(targetedWrite);
+void TargetedWriteBatch::addWrite(std::unique_ptr<TargetedWrite> targetedWrite, int estWriteSize) {
+    _writes.push_back(std::move(targetedWrite));
     _estimatedSizeBytes += estWriteSize;
 }
 
