@@ -27,25 +27,29 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/scripting/engine.h"
 
+#include <algorithm>
 #include <boost/filesystem/operations.hpp>
-#include <cctype>
 
+#include "mongo/base/string_data.h"
 #include "mongo/client/dbclient_base.h"
 #include "mongo/client/dbclient_cursor.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/service_context.h"
 #include "mongo/logv2/log.h"
 #include "mongo/scripting/dbdirectclient_factory.h"
+#include "mongo/util/ctype.h"
 #include "mongo/util/fail_point.h"
 #include "mongo/util/file.h"
 #include "mongo/util/text.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
+
 namespace mongo {
 
 using std::set;
@@ -215,22 +219,17 @@ bool Scope::execFile(const string& filename, bool printResult, bool reportError,
     return exec(code, filename, printResult, reportError, false, timeoutMs);
 }
 
-class Scope::StoredFuncModLogOpHandler : public RecoveryUnit::Change {
-public:
-    void commit(boost::optional<Timestamp>) {
-        _lastVersion.fetchAndAdd(1);
-    }
-    void rollback() {}
-};
-
 void Scope::storedFuncMod(OperationContext* opCtx) {
-    opCtx->recoveryUnit()->registerChange(std::make_unique<StoredFuncModLogOpHandler>());
+    opCtx->recoveryUnit()->onCommit(
+        [](boost::optional<Timestamp>) { _lastVersion.fetchAndAdd(1); });
 }
 
 void Scope::validateObjectIdString(const string& str) {
     uassert(10448, "invalid object id: length", str.size() == 24);
-    for (size_t i = 0; i < str.size(); i++)
-        uassert(10430, "invalid object id: not hex", std::isxdigit(str.at(i)));
+    auto isAllHex = [](StringData s) {
+        return std::all_of(s.begin(), s.end(), [](char c) { return ctype::isXdigit(c); });
+    };
+    uassert(10430, "invalid object id: not hex", isAllHex(str));
 }
 
 void Scope::loadStored(OperationContext* opCtx, bool ignoreNotConnected) {
@@ -250,8 +249,8 @@ void Scope::loadStored(OperationContext* opCtx, bool ignoreNotConnected) {
 
     auto directDBClient = DBDirectClientFactory::get(opCtx).create(opCtx);
 
-    unique_ptr<DBClientCursor> c =
-        directDBClient->query(coll, Query(), 0, 0, nullptr, QueryOption_SlaveOk, 0);
+    std::unique_ptr<DBClientCursor> c = directDBClient->find(
+        FindCommandRequest{coll}, ReadPreferenceSetting{ReadPreference::SecondaryPreferred});
     massert(16669, "unable to get db client cursor from query", c.get());
 
     set<string> thisTime;
@@ -271,6 +270,7 @@ void Scope::loadStored(OperationContext* opCtx, bool ignoreNotConnected) {
                 v.type() != BSONType::CodeWScope);
 
         if (MONGO_unlikely(mr_killop_test_fp.shouldFail())) {
+            LOGV2(5062200, "Pausing mr_killop_test_fp for system.js entry", "entryName"_attr = n);
 
             /* This thread sleep makes the interrupts in the test come in at a time
              *  where the js misses the interrupt and throw an exception instead of
@@ -280,9 +280,9 @@ void Scope::loadStored(OperationContext* opCtx, bool ignoreNotConnected) {
         }
 
         try {
-            setElement(n.valuestr(), v, o);
-            thisTime.insert(n.valuestr());
-            _storedNames.insert(n.valuestr());
+            setElement(n.valueStringDataSafe().rawData(), v, o);
+            thisTime.insert(n.str());
+            _storedNames.insert(n.str());
         } catch (const DBException& setElemEx) {
             if (setElemEx.code() == ErrorCodes::Interrupted) {
                 throw;
@@ -290,7 +290,7 @@ void Scope::loadStored(OperationContext* opCtx, bool ignoreNotConnected) {
 
             LOGV2_ERROR(22781,
                         "unable to load stored JavaScript function {n_valuestr}(): {setElemEx}",
-                        "n_valuestr"_attr = n.valuestr(),
+                        "n_valuestr"_attr = n.valueStringDataSafe(),
                         "setElemEx"_attr = redact(setElemEx));
         }
     }
@@ -380,11 +380,13 @@ public:
             return;
         }
 
-        if (Date_t::now() - scope->getCreateTime() > kMaxScopeReuseTime)
+        if (Date_t::now() - scope->getCreateTime() > kMaxScopeReuseTime) {
             return;  // too old to save
+        }
 
-        if (!scope->getError().empty())
+        if (!scope->getError().empty()) {
             return;  // not saving errored scopes
+        }
 
         if (_pools.size() >= kMaxPoolSize) {
             // prefer to keep recently-used scopes
@@ -655,12 +657,13 @@ bool hasJSReturn(const string& code) {
 
     // return is at start OR preceded by space
     // AND return is not followed by digit or letter
-    return (x == 0 || isspace(code[x - 1])) && !(isalpha(code[x + 6]) || isdigit(code[x + 6]));
+    return (x == 0 || ctype::isSpace(code[x - 1])) &&
+        !(ctype::isAlpha(code[x + 6]) || ctype::isDigit(code[x + 6]));
 }
 
 const char* jsSkipWhiteSpace(const char* raw) {
     while (raw[0]) {
-        while (isspace(*raw)) {
+        while (ctype::isSpace(*raw)) {
             ++raw;
         }
         if (raw[0] != '/' || raw[1] != '/')
