@@ -8,9 +8,9 @@ doassert = function(msg, obj) {
 
     if (jsTest.options().traceExceptions) {
         if (typeof (msg) == "string" && msg.indexOf("assert") == 0)
-            print(msg);
+            print(new Date().toISOString() + " " + msg);
         else
-            print("assert: " + msg);
+            print(new Date().toISOString() + " assert: " + msg);
     }
 
     var ex;
@@ -110,7 +110,8 @@ assert = (function() {
                     doassert("msg function cannot expect any parameters.");
                 }
             } else if (typeof msg !== "string" && typeof msg !== "object") {
-                doassert("msg parameter must be a string, function or object.");
+                doassert("msg parameter must be a string, function or object. Found type: " +
+                         typeof (msg));
             }
 
             if (msg && assert._debug) {
@@ -440,7 +441,7 @@ assert = (function() {
         // Used up all attempts
         msg = _buildAssertionMessage(msg);
         if (runHangAnalyzer) {
-            msg = msg + "The hang analyzer is automatically called in assert.retry functions. " +
+            msg = msg + " The hang analyzer is automatically called in assert.retry functions. " +
                 "If you are *expecting* assert.soon to possibly fail, call assert.retry " +
                 "with {runHangAnalyzer: false} as the fifth argument " +
                 "(you can fill unused arguments with `undefined`).";
@@ -502,7 +503,8 @@ assert = (function() {
                 "assert.time failed timeout " + timeout + "ms took " + diff + "ms : " + f + ", msg";
             msg = _buildAssertionMessage(msg, msgPrefix);
             if (runHangAnalyzer) {
-                msg = msg + "The hang analyzer is automatically called in assert.time functions. " +
+                msg = msg +
+                    " The hang analyzer is automatically called in assert.time functions. " +
                     "If you are *expecting* assert.soon to possibly fail, call assert.time " +
                     "with {runHangAnalyzer: false} as the fourth argument " +
                     "(you can fill unused arguments with `undefined`).";
@@ -677,13 +679,65 @@ assert = (function() {
         }
     }
 
+    function _runHangAnalyzerIfWriteConcernTimedOut(res) {
+        const timeoutMsg = "waiting for replication timed out";
+        let isWriteConcernTimeout = false;
+        if (_isWriteResultType(res)) {
+            if (res.hasWriteConcernError() && res.getWriteConcernError().errmsg === timeoutMsg) {
+                isWriteConcernTimeout = true;
+            }
+        } else if ((res.hasOwnProperty("errmsg") && res.errmsg === timeoutMsg) ||
+                   (res.hasOwnProperty("writeConcernError") &&
+                    res.writeConcernError.errmsg === timeoutMsg)) {
+            isWriteConcernTimeout = true;
+        }
+        if (isWriteConcernTimeout) {
+            print("Running hang analyzer for writeConcern timeout " + tojson(res));
+            MongoRunner.runHangAnalyzer();
+            return true;
+        }
+        return false;
+    }
+
+    function _runHangAnalyzerIfNonTransientLockTimeoutError(res) {
+        // Concurrency suites see a lot of LockTimeouts when running concurrent transactions.
+        // However, they will also abort transactions and continue running rather than fail the
+        // test, so we don't want to run the hang analyzer when the error has a
+        // TransientTransactionError error label.
+        const isTransientTxnError = res.hasOwnProperty("errorLabels") &&
+            res.errorLabels.includes("TransientTransactionError");
+        const isLockTimeout = res.hasOwnProperty("code") && ErrorCodes.LockTimeout === res.code;
+        if (isLockTimeout && !isTransientTxnError) {
+            print("Running hang analyzer for lock timeout " + tojson(res));
+            MongoRunner.runHangAnalyzer();
+            return true;
+        }
+        return false;
+    }
+
+    function _runHangAnalyzerForSpecificFailureTypes(res) {
+        // If the hang analyzer is run, then we shouldn't try to run it again.
+        if (_runHangAnalyzerIfWriteConcernTimedOut(res)) {
+            return;
+        }
+
+        _runHangAnalyzerIfNonTransientLockTimeoutError(res);
+    }
+
     function _assertCommandWorked(res, msg, {ignoreWriteErrors, ignoreWriteConcernErrors}) {
         _validateAssertionMessage(msg);
         _validateCommandResponse(res, "commandWorked");
 
         // Keep this as a function so we don't call tojson if not necessary.
         const makeFailMsg = () => {
-            return _buildAssertionMessage(msg, "command failed: " + tojson(res));
+            let prefix = "command failed: " + tojson(res);
+            if (typeof res._commandObj === "object" && res._commandObj !== null) {
+                prefix += " with original command request: " + tojson(res._commandObj);
+            }
+            if (typeof res._mongo === "object" && res._mongo !== null) {
+                prefix += " on connection: " + res._mongo;
+            }
+            return _buildAssertionMessage(msg, prefix);
         };
 
         if (_isWriteResultType(res)) {
@@ -703,6 +757,7 @@ assert = (function() {
                     ignoreWriteErrors: ignoreWriteErrors,
                     ignoreWriteConcernErrors: ignoreWriteConcernErrors
                 })) {
+                _runHangAnalyzerForSpecificFailureTypes(res);
                 doassert(makeFailMsg(), res);
             }
         } else if (res.hasOwnProperty("acknowledged")) {
@@ -716,12 +771,13 @@ assert = (function() {
         return res;
     }
 
-    const kAnyErrorCode = Object.create(null);
+    assert._kAnyErrorCode = Object.create(null);
+
     function _assertCommandFailed(res, expectedCode, msg) {
         _validateAssertionMessage(msg);
         _validateCommandResponse(res, "commandFailed");
 
-        if (expectedCode !== kAnyErrorCode && !Array.isArray(expectedCode)) {
+        if (expectedCode !== assert._kAnyErrorCode && !Array.isArray(expectedCode)) {
             expectedCode = [expectedCode];
         }
 
@@ -732,7 +788,7 @@ assert = (function() {
         };
 
         const makeFailCodeMsg = () => {
-            return (expectedCode !== kAnyErrorCode)
+            return (expectedCode !== assert._kAnyErrorCode)
                 ? _buildAssertionMessage(msg,
                                          "command did not fail with any of the following codes " +
                                              tojson(expectedCode) + " " + tojson(res))
@@ -746,7 +802,7 @@ assert = (function() {
             // A WriteCommandError implies ok:0.
             // Error objects may have a `code` property added (e.g.
             // DBCollection.prototype.mapReduce) without a `ok` property.
-            if (expectedCode !== kAnyErrorCode) {
+            if (expectedCode !== assert._kAnyErrorCode) {
                 if (!res.hasOwnProperty("code") || !expectedCode.includes(res.code)) {
                     doassert(makeFailCodeMsg(), res);
                 }
@@ -758,7 +814,7 @@ assert = (function() {
                 doassert(makeFailMsg(), res);
             }
 
-            if (expectedCode !== kAnyErrorCode) {
+            if (expectedCode !== assert._kAnyErrorCode) {
                 let foundCode = false;
                 if (res.hasOwnProperty("code") && expectedCode.includes(res.code)) {
                     foundCode = true;
@@ -769,6 +825,7 @@ assert = (function() {
                 }
 
                 if (!foundCode) {
+                    _runHangAnalyzerForSpecificFailureTypes(res);
                     doassert(makeFailCodeMsg(), res);
                 }
             }
@@ -810,7 +867,7 @@ assert = (function() {
     };
 
     assert.commandFailed = function(res, msg) {
-        return _assertCommandFailed(res, kAnyErrorCode, msg);
+        return _assertCommandFailed(res, assert._kAnyErrorCode, msg);
     };
 
     // expectedCode can be an array of possible codes.
@@ -844,6 +901,7 @@ assert = (function() {
         }
 
         if (errMsg) {
+            _runHangAnalyzerForSpecificFailureTypes(res);
             doassert(_buildAssertionMessage(msg, errMsg), res);
         }
 
@@ -851,7 +909,7 @@ assert = (function() {
     };
 
     assert.writeError = function(res, msg) {
-        return assert.writeErrorWithCode(res, kAnyErrorCode, msg);
+        return assert.writeErrorWithCode(res, assert._kAnyErrorCode, msg);
     };
 
     // If expectedCode is an array then this asserts that the found code is one of the codes in
@@ -893,7 +951,7 @@ assert = (function() {
             }
         }
 
-        if (!errMsg && expectedCode !== kAnyErrorCode) {
+        if (!errMsg && expectedCode !== assert._kAnyErrorCode) {
             if (!Array.isArray(expectedCode)) {
                 expectedCode = [expectedCode];
             }
@@ -905,6 +963,7 @@ assert = (function() {
         }
 
         if (errMsg) {
+            _runHangAnalyzerForSpecificFailureTypes(res);
             doassert(_buildAssertionMessage(msg, errMsg));
         }
 
@@ -1050,65 +1109,6 @@ assert = (function() {
         doassert(_buildAssertionMessage(msg, msgPrefix));
     };
 
-    assert.gleOK = function(res, msg) {
-        var errMsg = null;
-
-        if (!res) {
-            errMsg = "missing first argument, no response to check";
-        } else if (!res.ok) {
-            errMsg = "getLastError failed: " + tojson(res);
-        } else if ('code' in res || 'errmsg' in res || ('err' in res && res['err'] != null)) {
-            errMsg = "write or write concern failed: " + tojson(res);
-        }
-
-        if (errMsg) {
-            doassert(_buildAssertionMessage(msg, errMsg), res);
-        }
-
-        return res;
-    };
-
-    assert.gleSuccess = function(dbOrGLEDoc, msg) {
-        var gle = dbOrGLEDoc instanceof DB ? dbOrGLEDoc.getLastErrorObj() : dbOrGLEDoc;
-        if (gle.err) {
-            if (typeof (msg) == "function")
-                msg = msg(gle);
-            doassert(_buildAssertionMessage(msg, "getLastError not null: " + tojson(gle)), gle);
-        }
-        return gle;
-    };
-
-    assert.gleError = function(dbOrGLEDoc, msg) {
-        var gle = dbOrGLEDoc instanceof DB ? dbOrGLEDoc.getLastErrorObj() : dbOrGLEDoc;
-        if (!gle.err) {
-            if (typeof (msg) == "function")
-                msg = msg(gle);
-            doassert(_buildAssertionMessage(msg, "getLastError is null: " + tojson(gle)));
-        }
-    };
-
-    assert.gleErrorCode = function(dbOrGLEDoc, code, msg) {
-        var gle = dbOrGLEDoc instanceof DB ? dbOrGLEDoc.getLastErrorObj() : dbOrGLEDoc;
-        if (!gle.err || gle.code != code) {
-            if (typeof (msg) == "function")
-                msg = msg(gle);
-            doassert(_buildAssertionMessage(
-                msg,
-                "getLastError is null or has code other than \"" + code + "\": " + tojson(gle)));
-        }
-    };
-
-    assert.gleErrorRegex = function(dbOrGLEDoc, regex, msg) {
-        var gle = dbOrGLEDoc instanceof DB ? dbOrGLEDoc.getLastErrorObj() : dbOrGLEDoc;
-        if (!gle.err || !regex.test(gle.err)) {
-            if (typeof (msg) == "function")
-                msg = msg(gle);
-            doassert(_buildAssertionMessage(
-                msg,
-                "getLastError is null or doesn't match regex (" + regex + "): " + tojson(gle)));
-        }
-    };
-
     assert.includes = function(haystack, needle, msg) {
         if (!haystack.includes(needle)) {
             var assertMsg = "string [" + haystack + "] does not include [" + needle + "]";
@@ -1120,5 +1120,15 @@ assert = (function() {
         }
     };
 
+    assert.noAPIParams = function(cmdOptions) {
+        if (!(cmdOptions instanceof Object)) {
+            return;
+        }
+        assert(!cmdOptions.hasOwnProperty("apiVersion") &&
+                   !cmdOptions.hasOwnProperty("apiStrict") &&
+                   !cmdOptions.hasOwnProperty("apiDeprecationErrors"),
+               "API parameters are not allowed in this context");
+    };
+
     return assert;
 })();
