@@ -34,18 +34,15 @@
 
 #include "mongo/client/dbclient_base.h"
 #include "mongo/db/jsobj.h"
-#include "mongo/db/logical_session_id.h"
-#include "mongo/db/ops/write_ops_parsers.h"
+#include "mongo/db/ops/write_ops.h"
+#include "mongo/db/session/logical_session_id.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
 #include "mongo/stdx/thread.h"
+#include "mongo/util/pcre.h"
 #include "mongo/util/timer.h"
 
-namespace pcrecpp {
-class RE;
-}  // namespace pcrecpp
-
 namespace mongo {
 
 enum class OpType {
@@ -63,6 +60,14 @@ enum class OpType {
     CPULOAD
 };
 
+inline bool isReadOp(OpType opType) {
+    return opType == OpType::FINDONE || opType == OpType::FIND;
+}
+
+inline bool isWriteOp(OpType opType) {
+    return opType == OpType::UPDATE || opType == OpType::INSERT || opType == OpType::REMOVE;
+}
+
 class BenchRunConfig;
 struct BenchRunStats;
 class BsonTemplateEvaluator;
@@ -110,11 +115,9 @@ struct BenchRunOp {
     int options = 0;
     BSONObj projection;
     BSONObj query;
-    bool safe = false;
     int skip = 0;
     BSONObj sort;
     bool showError = false;
-    bool showResult = false;
     std::string target;
     bool throwGLE = false;
     write_ops::UpdateModification update;
@@ -139,7 +142,7 @@ struct BenchRunOp {
     // resources that a snapshot transaction would hold for a time.
     int maxRandomMillisecondDelayBeforeGetMore{0};
 
-    // Format: {mode: modeStr}.  Only mode field is allowed.
+    // Format: {$readPreference: {mode: modeStr}}.  Only mode field is allowed.
     BSONObj readPrefObj;
 
     // This is an owned copy of the raw operation. All unowned members point into this.
@@ -229,14 +232,13 @@ public:
     /// Base random seed for threads
     int64_t randomSeed;
 
-    bool hideResults;
     bool handleErrors;
     bool hideErrors;
 
-    std::shared_ptr<pcrecpp::RE> trapPattern;
-    std::shared_ptr<pcrecpp::RE> noTrapPattern;
-    std::shared_ptr<pcrecpp::RE> watchPattern;
-    std::shared_ptr<pcrecpp::RE> noWatchPattern;
+    std::shared_ptr<pcre::Regex> trapPattern;
+    std::shared_ptr<pcre::Regex> noTrapPattern;
+    std::shared_ptr<pcre::Regex> watchPattern;
+    std::shared_ptr<pcre::Regex> noWatchPattern;
 
     /**
      * Operation description. A list of BenchRunOps, each describing a single
