@@ -187,6 +187,10 @@ var _bulk_api_module = (function() {
                 result.writeError = {};
                 result.writeError.code = this.getWriteError().code;
                 result.writeError.errmsg = this.getWriteError().errmsg;
+                let errInfo = this.getWriteError().errInfo;
+                if (errInfo) {
+                    result.writeError.errInfo = errInfo;
+                }
             }
 
             if (this.getWriteConcernError() != null) {
@@ -438,6 +442,9 @@ var _bulk_api_module = (function() {
         defineReadOnlyProperty(this, "code", err.code);
         defineReadOnlyProperty(this, "index", err.index);
         defineReadOnlyProperty(this, "errmsg", err.errmsg);
+        // errInfo field is optional.
+        if (err.hasOwnProperty("errInfo"))
+            defineReadOnlyProperty(this, "errInfo", err.errInfo);
 
         //
         // Define access methods
@@ -510,15 +517,6 @@ var _bulk_api_module = (function() {
         this.operations = [];
     };
 
-    /**
-     * Wraps a legacy operation so we can correctly rewrite its error
-     */
-    var LegacyOp = function(batchType, operation, index) {
-        this.batchType = batchType;
-        this.index = index;
-        this.operation = operation;
-    };
-
     /***********************************************************
      * Wraps the operations done for the batch
      ***********************************************************/
@@ -530,7 +528,9 @@ var _bulk_api_module = (function() {
         // Set max byte size
         var maxBatchSizeBytes = 1024 * 1024 * 16;
         var maxNumberOfDocsInBatch = 1000;
+        var idFieldOverhead = Object.bsonsize({_id: ObjectId()}) - Object.bsonsize({});
         var writeConcern = null;
+        var letParams = null;
         var currentOp;
 
         // Final results
@@ -606,6 +606,11 @@ var _bulk_api_module = (function() {
             // Get the bsonSize
             var bsonSize = Object.bsonsize(document);
 
+            // If an _id will be added to the insert, adjust the bsonSize
+            if (docType === INSERT && documentNeedsId(document)) {
+                bsonSize += idFieldOverhead;
+            }
+
             // Create a new batch object if we don't have a current one
             if (currentBatch == null)
                 currentBatch = new Batch(docType, currentIndex);
@@ -625,12 +630,21 @@ var _bulk_api_module = (function() {
             currentBatchSizeBytes = currentBatchSizeBytes + bsonSize;
         };
 
+        /**
+         *
+         * @param obj {Object} the document to check if an _id is present
+         * @returns true if the document needs an _id and false otherwise
+         */
+        var documentNeedsId = function(obj) {
+            return typeof (obj._id) == "undefined" && !Array.isArray(obj);
+        };
+
         /**
          * @return {Object} a new document with an _id: ObjectId if _id is not present.
          *     Otherwise, returns the same object passed.
          */
         var addIdIfNeeded = function(obj) {
-            if (typeof (obj._id) == "undefined" && !Array.isArray(obj)) {
+            if (documentNeedsId(obj)) {
                 var tmp = obj;  // don't want to modify input
                 obj = {_id: new ObjectId()};
                 for (var key in tmp) {
@@ -759,31 +773,11 @@ var _bulk_api_module = (function() {
             },
 
             collation: function(collationSpec) {
-                if (!collection.getMongo().hasWriteCommands()) {
-                    throw new Error(
-                        "cannot use collation if server does not support write commands");
-                }
-
-                if (collection.getMongo().writeMode() !== "commands") {
-                    throw new Error("write mode must be 'commands' in order to use collation, " +
-                                    "but found write mode: " + collection.getMongo().writeMode());
-                }
-
                 currentOp.collation = collationSpec;
                 return findOperations;
             },
 
             arrayFilters: function(filters) {
-                if (!collection.getMongo().hasWriteCommands()) {
-                    throw new Error(
-                        "cannot use arrayFilters if server does not support write commands");
-                }
-
-                if (collection.getMongo().writeMode() !== "commands") {
-                    throw new Error("write mode must be 'commands' in order to use arrayFilters, " +
-                                    "but found write mode: " + collection.getMongo().writeMode());
-                }
-
                 currentOp.arrayFilters = filters;
                 return findOperations;
             },
@@ -801,6 +795,10 @@ var _bulk_api_module = (function() {
             return findOperations;
         };
 
+        this.setLetParams = function(userLet) {
+            letParams = userLet;
+        };
+
         //
         // Merge write command result into aggregated results object
         var mergeBatchResults = function(batch, bulkResult, result) {
@@ -851,6 +849,10 @@ var _bulk_api_module = (function() {
                         errmsg: result.writeErrors[i].errmsg,
                         op: batch.operations[result.writeErrors[i].index]
                     };
+                    var errInfo = result.writeErrors[i].errInfo;
+                    if (errInfo) {
+                        writeError['errInfo'] = errInfo;
+                    }
 
                     bulkResult.writeErrors.push(new WriteError(writeError));
                 }
@@ -886,6 +888,11 @@ var _bulk_api_module = (function() {
                 cmd.writeConcern = writeConcern;
             }
 
+            // If we have let parameters, add them to the command.
+            if (letParams) {
+                cmd.let = letParams;
+            }
+
             {
                 const kWireVersionSupportingRetryableWrites = 6;
                 const serverSupportsRetryableWrites =
@@ -919,222 +926,6 @@ var _bulk_api_module = (function() {
             mergeBatchResults(batch, bulkResult, result);
         };
 
-        // Execute a single legacy op
-        var executeLegacyOp = function(_legacyOp) {
-            // Handle the different types of operation types
-            if (_legacyOp.batchType == INSERT) {
-                if (Array.isArray(_legacyOp.operation)) {
-                    var transformedInserts = [];
-                    _legacyOp.operation.forEach(function(insertDoc) {
-                        transformedInserts.push(addIdIfNeeded(insertDoc));
-                    });
-                    _legacyOp.operation = transformedInserts;
-                } else {
-                    _legacyOp.operation = addIdIfNeeded(_legacyOp.operation);
-                }
-
-                collection.getMongo().insert(
-                    collection.getFullName(), _legacyOp.operation, ordered);
-            } else if (_legacyOp.batchType == UPDATE) {
-                collection.getMongo().update(collection.getFullName(),
-                                             _legacyOp.operation.q,
-                                             _legacyOp.operation.u,
-                                             _legacyOp.operation.upsert,
-                                             _legacyOp.operation.multi);
-            } else if (_legacyOp.batchType == REMOVE) {
-                var single = Boolean(_legacyOp.operation.limit);
-
-                collection.getMongo().remove(
-                    collection.getFullName(), _legacyOp.operation.q, single);
-            }
-        };
-
-        /**
-         * Parses the getLastError response and properly sets the write errors and
-         * write concern errors.
-         * Should kept be up to date with BatchSafeWriter::extractGLEErrors.
-         *
-         * @return {object} an object with the format:
-         *
-         * {
-         *   writeError: {object|null} raw write error object without the index.
-         *   wcError: {object|null} raw write concern error object.
-         * }
-         */
-        var extractGLEErrors = function(gleResponse) {
-            var isOK = gleResponse.ok ? true : false;
-            var err = (gleResponse.err) ? gleResponse.err : '';
-            var errMsg = (gleResponse.errmsg) ? gleResponse.errmsg : '';
-            var wNote = (gleResponse.wnote) ? gleResponse.wnote : '';
-            var jNote = (gleResponse.jnote) ? gleResponse.jnote : '';
-            var code = gleResponse.code;
-            var timeout = gleResponse.wtimeout ? true : false;
-
-            var extractedErr = {writeError: null, wcError: null, unknownError: null};
-
-            if (err == 'norepl' || err == 'noreplset') {
-                // Know this is legacy gle and the repl not enforced - write concern error in 2.4.
-                var errObj = {code: WRITE_CONCERN_FAILED};
-
-                if (errMsg != '') {
-                    errObj.errmsg = errMsg;
-                } else if (wNote != '') {
-                    errObj.errmsg = wNote;
-                } else {
-                    errObj.errmsg = err;
-                }
-
-                extractedErr.wcError = errObj;
-            } else if (timeout) {
-                // Know there was not write error.
-                var errObj = {code: WRITE_CONCERN_FAILED};
-
-                if (errMsg != '') {
-                    errObj.errmsg = errMsg;
-                } else {
-                    errObj.errmsg = err;
-                }
-
-                errObj.errInfo = {wtimeout: true};
-                extractedErr.wcError = errObj;
-            } else if (code == 19900 ||  // No longer primary
-                       code == 16805 ||  // replicatedToNum no longer primary
-                       code == 14330 ||  // gle wmode changed; invalid
-                       code == NOT_MASTER || code == UNKNOWN_REPL_WRITE_CONCERN ||
-                       code == WRITE_CONCERN_FAILED) {
-                extractedErr.wcError = {code: code, errmsg: errMsg};
-            } else if (!isOK) {
-                // This is a GLE failure we don't understand
-                extractedErr.unknownError = {code: code, errmsg: errMsg};
-            } else if (err != '') {
-                extractedErr.writeError = {code: (code == 0) ? UNKNOWN_ERROR : code, errmsg: err};
-            } else if (jNote != '') {
-                extractedErr.writeError = {code: WRITE_CONCERN_FAILED, errmsg: jNote};
-            }
-
-            // Handling of writeback not needed for mongo shell.
-            return extractedErr;
-        };
-
-        /**
-         * getLastErrorMethod that supports all write concerns
-         */
-        var executeGetLastError = function(db, options) {
-            var cmd = {getlasterror: 1};
-            cmd = Object.extend(cmd, options);
-            // Execute the getLastErrorCommand
-            return db.runCommand(cmd);
-        };
-
-        // Execute the operations, serially
-        var executeBatchWithLegacyOps = function(batch) {
-            var batchResult = {n: 0, writeErrors: [], upserted: []};
-
-            var extractedErr = null;
-
-            var totalToExecute = batch.operations.length;
-            // Run over all the operations
-            for (var i = 0; i < batch.operations.length; i++) {
-                if (batchResult.writeErrors.length > 0 && ordered)
-                    break;
-
-                var _legacyOp = new LegacyOp(batch.batchType, batch.operations[i], i);
-                executeLegacyOp(_legacyOp);
-
-                var result = executeGetLastError(collection.getDB(), {w: 1});
-                extractedErr = extractGLEErrors(result);
-
-                if (extractedErr.unknownError) {
-                    throw new WriteCommandError({
-                        ok: 0.0,
-                        code: extractedErr.unknownError.code,
-                        errmsg: extractedErr.unknownError.errmsg
-                    });
-                }
-
-                if (extractedErr.writeError != null) {
-                    // Create the emulated result set
-                    var errResult = {
-                        index: _legacyOp.index,
-                        code: extractedErr.writeError.code,
-                        errmsg: extractedErr.writeError.errmsg,
-                        op: batch.operations[_legacyOp.index]
-                    };
-
-                    batchResult.writeErrors.push(errResult);
-                } else if (_legacyOp.batchType == INSERT) {
-                    // Inserts don't give us "n" back, so we can only infer
-                    batchResult.n = batchResult.n + 1;
-                }
-
-                if (_legacyOp.batchType == UPDATE) {
-                    // Unfortunately v2.4 GLE does not include the upserted field when
-                    // the upserted _id is non-OID type.  We can detect this by the
-                    // updatedExisting field + an n of 1
-                    var upserted = result.upserted !== undefined ||
-                        (result.updatedExisting === false && result.n == 1);
-
-                    if (upserted) {
-                        batchResult.n = batchResult.n + 1;
-
-                        // If we don't have an upserted value, see if we can pull it from the update
-                        // or the
-                        // query
-                        if (result.upserted === undefined) {
-                            result.upserted = _legacyOp.operation.u._id;
-                            if (result.upserted === undefined) {
-                                result.upserted = _legacyOp.operation.q._id;
-                            }
-                        }
-
-                        batchResult.upserted.push({index: _legacyOp.index, _id: result.upserted});
-                    } else if (result.n) {
-                        batchResult.n = batchResult.n + result.n;
-                    }
-                }
-
-                if (_legacyOp.batchType == REMOVE && result.n) {
-                    batchResult.n = batchResult.n + result.n;
-                }
-            }
-
-            var needToEnforceWC = writeConcern != null &&
-                bsonWoCompare(writeConcern, {w: 1}) != 0 &&
-                bsonWoCompare(writeConcern, {w: 0}) != 0;
-
-            extractedErr = null;
-            if (needToEnforceWC &&
-                (batchResult.writeErrors.length == 0 ||
-                 (!ordered &&
-                  // not all errored.
-                  batchResult.writeErrors.length < batch.operations.length))) {
-                // if last write errored
-                if (batchResult.writeErrors.length > 0 &&
-                    batchResult.writeErrors[batchResult.writeErrors.length - 1].index ==
-                        (batch.operations.length - 1)) {
-                    // Reset previous errors so we can apply the write concern no matter what
-                    // as long as it is valid.
-                    collection.getDB().runCommand({resetError: 1});
-                }
-
-                result = executeGetLastError(collection.getDB(), writeConcern);
-                extractedErr = extractGLEErrors(result);
-
-                if (extractedErr.unknownError) {
-                    // Report as a wc failure
-                    extractedErr.wcError = extractedErr.unknownError;
-                }
-            }
-
-            if (extractedErr != null && extractedErr.wcError != null) {
-                bulkResult.writeConcernErrors.push(extractedErr.wcError);
-            }
-
-            // Merge the results
-            mergeBatchResults(batch, bulkResult, batchResult);
-        };
-
-        //
         // Execute the batch
         this.execute = function(_writeConcern) {
             if (executed)
@@ -1153,17 +944,10 @@ var _bulk_api_module = (function() {
             // Total number of batches to execute
             var totalNumberToExecute = batches.length;
 
-            var useWriteCommands = collection.getMongo().useWriteCommands();
-
             // Execute all the batches
             for (var i = 0; i < batches.length; i++) {
                 // Execute the batch
-                if (collection.getMongo().hasWriteCommands() &&
-                    collection.getMongo().writeMode() == "commands") {
-                    executeBatch(batches[i]);
-                } else {
-                    executeBatchWithLegacyOps(batches[i]);
-                }
+                executeBatch(batches[i]);
 
                 // If we are ordered and have errors and they are
                 // not all replication errors terminate the operation
