@@ -13,6 +13,10 @@ if ((typeof DBCollection) == "undefined") {
     };
 }
 
+DBCollection.prototype.compact = function() {
+    return this._db.getMongo().compact(this._fullName);
+};
+
 DBCollection.prototype.verify = function() {
     assert(this._fullName, "no fullName");
     assert(this._shortName, "no shortName");
@@ -66,8 +70,6 @@ DBCollection.prototype.help = function() {
           ".unhideIndex( \"indexName\" ) or db." + shortName +
           ".unhideIndex( { \"indexKey\" : 1 } )");
     print("\tdb." + shortName + ".dropIndexes()");
-    print("\tdb." + shortName +
-          ".ensureIndex(keypattern[,options]) - DEPRECATED, use createIndex() instead");
     print("\tdb." + shortName + ".explain().help() - show explain help");
     print("\tdb." + shortName + ".reIndex()");
     print(
@@ -125,13 +127,13 @@ DBCollection.prototype.help = function() {
     print("\tdb." + shortName + ".totalSize() - storage allocated for all data and indexes");
     print(
         "\tdb." + shortName +
-        ".update( query, <update object or pipeline>[, upsert_bool, multi_bool] ) - instead of two flags, you can pass an object with fields: upsert, multi, hint");
+        ".update( query, <update object or pipeline>[, upsert_bool, multi_bool] ) - instead of two flags, you can pass an object with fields: upsert, multi, hint, let");
     print(
         "\tdb." + shortName +
-        ".updateOne( filter, <update object or pipeline>, <optional params> ) - update the first matching document, optional parameters are: upsert, w, wtimeout, j, hint");
+        ".updateOne( filter, <update object or pipeline>, <optional params> ) - update the first matching document, optional parameters are: upsert, w, wtimeout, j, hint, let");
     print(
         "\tdb." + shortName +
-        ".updateMany( filter, <update object or pipeline>, <optional params> ) - update all matching documents, optional parameters are: upsert, w, wtimeout, j, hint");
+        ".updateMany( filter, <update object or pipeline>, <optional params> ) - update all matching documents, optional parameters are: upsert, w, wtimeout, j, hint, let");
     print("\tdb." + shortName + ".validate( <full> ) - SLOW");
     print("\tdb." + shortName + ".getShardVersion() - only for use with sharding");
     print("\tdb." + shortName +
@@ -217,13 +219,16 @@ DBCollection.prototype._massageObject = function(q) {
     throw Error("don't know how to massage : " + type);
 };
 
-DBCollection.prototype.find = function(query, fields, limit, skip, batchSize, options) {
+DBCollection.prototype.find = function(filter, projection, limit, skip, batchSize, options) {
+    // Verify that API version parameters are not supplied via the shell helper.
+    assert.noAPIParams(options);
+
     var cursor = new DBQuery(this._mongo,
                              this._db,
                              this,
                              this._fullName,
-                             this._massageObject(query),
-                             fields,
+                             this._massageObject(filter),
+                             projection,
                              limit,
                              skip,
                              batchSize,
@@ -237,8 +242,10 @@ DBCollection.prototype.find = function(query, fields, limit, skip, batchSize, op
             cursor.readPref(readPreference.mode, readPreference.tags);
         }
 
-        const readConcern = session._getSessionAwareClient().getReadConcern(session);
-        if (readConcern !== null) {
+        const client = session._getSessionAwareClient();
+        const readConcern = client.getReadConcern(session);
+        if (readConcern !== null &&
+            client.canUseReadConcern(session, cursor._convertToCommand(true))) {
             cursor.readConcern(readConcern.level);
         }
     }
@@ -246,8 +253,9 @@ DBCollection.prototype.find = function(query, fields, limit, skip, batchSize, op
     return cursor;
 };
 
-DBCollection.prototype.findOne = function(query, fields, options, readConcern, collation) {
-    var cursor = this.find(query, fields, -1 /* limit */, 0 /* skip*/, 0 /* batchSize */, options);
+DBCollection.prototype.findOne = function(filter, projection, options, readConcern, collation) {
+    var cursor =
+        this.find(filter, projection, -1 /* limit */, 0 /* skip*/, 0 /* batchSize */, options);
 
     if (readConcern) {
         cursor = cursor.readConcern(readConcern);
@@ -274,10 +282,11 @@ DBCollection.prototype.insert = function(obj, options) {
     if (!obj)
         throw Error("no object passed to insert!");
 
+    options = typeof (options) === 'undefined' ? {} : options;
+
     var flags = 0;
 
     var wc = undefined;
-    var allowDottedFields = false;
     if (options === undefined) {
         // do nothing
     } else if (typeof (options) == 'object') {
@@ -289,8 +298,6 @@ DBCollection.prototype.insert = function(obj, options) {
 
         if (options.writeConcern)
             wc = options.writeConcern;
-        if (options.allowdotted)
-            allowDottedFields = true;
     } else {
         flags = options;
     }
@@ -299,57 +306,36 @@ DBCollection.prototype.insert = function(obj, options) {
     var ordered = ((flags & 1) == 0);
 
     if (!wc)
-        wc = this.getWriteConcern();
+        wc = this._createWriteConcern(options);
 
     var result = undefined;
-    var startTime =
-        (typeof (_verboseShell) === 'undefined' || !_verboseShell) ? 0 : new Date().getTime();
 
-    if (this.getMongo().writeMode() != "legacy") {
-        // Bit 1 of option flag is continueOnError. Bit 0 (stop on error) is the default.
-        var bulk = ordered ? this.initializeOrderedBulkOp() : this.initializeUnorderedBulkOp();
-        var isMultiInsert = Array.isArray(obj);
+    // Bit 1 of option flag is continueOnError. Bit 0 (stop on error) is the default.
+    var bulk = ordered ? this.initializeOrderedBulkOp() : this.initializeUnorderedBulkOp();
+    var isMultiInsert = Array.isArray(obj);
 
-        if (isMultiInsert) {
-            obj.forEach(function(doc) {
-                bulk.insert(doc);
-            });
-        } else {
-            bulk.insert(obj);
-        }
-
-        try {
-            result = bulk.execute(wc);
-            if (!isMultiInsert)
-                result = result.toSingleResult();
-        } catch (ex) {
-            if (ex instanceof BulkWriteError) {
-                result = isMultiInsert ? ex.toResult() : ex.toSingleResult();
-            } else if (ex instanceof WriteCommandError) {
-                result = ex;
-            } else {
-                // Other exceptions rethrown as-is.
-                throw ex;
-            }
-        }
+    if (isMultiInsert) {
+        obj.forEach(function(doc) {
+            bulk.insert(doc);
+        });
     } else {
-        if (typeof (obj._id) == "undefined" && !Array.isArray(obj)) {
-            var tmp = obj;  // don't want to modify input
-            obj = {_id: new ObjectId()};
-            for (var key in tmp) {
-                obj[key] = tmp[key];
-            }
-        }
-
-        this.getMongo().insert(this._fullName, obj, flags);
-
-        // enforce write concern, if required
-        if (wc)
-            result = this.runCommand("getLastError", wc instanceof WriteConcern ? wc.toJSON() : wc);
+        bulk.insert(obj);
     }
 
-    this._lastID = obj._id;
-    this._printExtraInfo("Inserted", startTime);
+    try {
+        result = bulk.execute(wc);
+        if (!isMultiInsert)
+            result = result.toSingleResult();
+    } catch (ex) {
+        if (ex instanceof BulkWriteError) {
+            result = isMultiInsert ? ex.toResult() : ex.toSingleResult();
+        } else if (ex instanceof WriteCommandError) {
+            result = ex;
+        } else {
+            // Other exceptions rethrown as-is.
+            throw ex;
+        }
+    }
     return result;
 };
 
@@ -365,11 +351,14 @@ DBCollection.prototype._parseRemove = function(t, justOne) {
 
     var wc = undefined;
     var collation = undefined;
+    let letParams = undefined;
+
     if (typeof (justOne) === "object") {
         var opts = justOne;
         wc = opts.writeConcern;
         justOne = opts.justOne;
         collation = opts.collation;
+        letParams = opts.let;
     }
 
     // Normalize "justOne" to a bool.
@@ -380,7 +369,7 @@ DBCollection.prototype._parseRemove = function(t, justOne) {
         wc = this.getWriteConcern();
     }
 
-    return {"query": query, "justOne": justOne, "wc": wc, "collation": collation};
+    return {"query": query, "justOne": justOne, "wc": wc, "collation": collation, "let": letParams};
 };
 
 // Returns a WriteResult if write command succeeded, but may contain write errors.
@@ -391,50 +380,38 @@ DBCollection.prototype.remove = function(t, justOne) {
     var justOne = parsed.justOne;
     var wc = parsed.wc;
     var collation = parsed.collation;
+    var letParams = parsed.let;
 
     var result = undefined;
-    var startTime =
-        (typeof (_verboseShell) === 'undefined' || !_verboseShell) ? 0 : new Date().getTime();
+    var bulk = this.initializeOrderedBulkOp();
 
-    if (this.getMongo().writeMode() != "legacy") {
-        var bulk = this.initializeOrderedBulkOp();
-        var removeOp = bulk.find(query);
+    if (letParams) {
+        bulk.setLetParams(letParams);
+    }
+    var removeOp = bulk.find(query);
 
-        if (collation) {
-            removeOp.collation(collation);
-        }
-
-        if (justOne) {
-            removeOp.removeOne();
-        } else {
-            removeOp.remove();
-        }
-
-        try {
-            result = bulk.execute(wc).toSingleResult();
-        } catch (ex) {
-            if (ex instanceof BulkWriteError) {
-                result = ex.toSingleResult();
-            } else if (ex instanceof WriteCommandError) {
-                result = ex;
-            } else {
-                // Other exceptions thrown
-                throw ex;
-            }
-        }
-    } else {
-        if (collation) {
-            throw new Error("collation requires use of write commands");
-        }
-
-        this.getMongo().remove(this._fullName, query, justOne);
-
-        // enforce write concern, if required
-        if (wc)
-            result = this.runCommand("getLastError", wc instanceof WriteConcern ? wc.toJSON() : wc);
+    if (collation) {
+        removeOp.collation(collation);
     }
 
-    this._printExtraInfo("Removed", startTime);
+    if (justOne) {
+        removeOp.removeOne();
+    } else {
+        removeOp.remove();
+    }
+
+    try {
+        result = bulk.execute(wc).toSingleResult();
+    } catch (ex) {
+        if (ex instanceof BulkWriteError) {
+            result = ex.toSingleResult();
+        } else if (ex instanceof WriteCommandError) {
+            result = ex;
+        } else {
+            // Other exceptions thrown
+            throw ex;
+        }
+    }
     return result;
 };
 
@@ -454,6 +431,7 @@ DBCollection.prototype._parseUpdate = function(query, updateSpec, upsert, multi)
     var collation = undefined;
     var arrayFilters = undefined;
     let hint = undefined;
+    let letParams = undefined;
 
     // can pass options via object for improved readability
     if (typeof (upsert) === "object") {
@@ -469,6 +447,7 @@ DBCollection.prototype._parseUpdate = function(query, updateSpec, upsert, multi)
         collation = opts.collation;
         arrayFilters = opts.arrayFilters;
         hint = opts.hint;
+        letParams = opts.let;
     }
 
     // Normalize 'upsert' and 'multi' to booleans.
@@ -487,7 +466,8 @@ DBCollection.prototype._parseUpdate = function(query, updateSpec, upsert, multi)
         "multi": multi,
         "wc": wc,
         "collation": collation,
-        "arrayFilters": arrayFilters
+        "arrayFilters": arrayFilters,
+        "let": letParams
     };
 };
 
@@ -503,67 +483,50 @@ DBCollection.prototype.update = function(query, updateSpec, upsert, multi) {
     var wc = parsed.wc;
     var collation = parsed.collation;
     var arrayFilters = parsed.arrayFilters;
+    let letParams = parsed.let;
 
     var result = undefined;
-    var startTime =
-        (typeof (_verboseShell) === 'undefined' || !_verboseShell) ? 0 : new Date().getTime();
+    var bulk = this.initializeOrderedBulkOp();
 
-    if (this.getMongo().writeMode() != "legacy") {
-        var bulk = this.initializeOrderedBulkOp();
-        var updateOp = bulk.find(query);
+    if (letParams) {
+        bulk.setLetParams(letParams);
+    }
+    var updateOp = bulk.find(query);
 
-        if (hint) {
-            updateOp.hint(hint);
-        }
-
-        if (upsert) {
-            updateOp = updateOp.upsert();
-        }
-
-        if (collation) {
-            updateOp.collation(collation);
-        }
-
-        if (arrayFilters) {
-            updateOp.arrayFilters(arrayFilters);
-        }
-
-        if (multi) {
-            updateOp.update(updateSpec);
-        } else {
-            updateOp.updateOne(updateSpec);
-        }
-
-        try {
-            result = bulk.execute(wc).toSingleResult();
-        } catch (ex) {
-            if (ex instanceof BulkWriteError) {
-                result = ex.toSingleResult();
-            } else if (ex instanceof WriteCommandError) {
-                result = ex;
-            } else {
-                // Other exceptions thrown
-                throw ex;
-            }
-        }
-    } else {
-        if (collation) {
-            throw new Error("collation requires use of write commands");
-        }
-
-        if (arrayFilters) {
-            throw new Error("arrayFilters requires use of write commands");
-        }
-
-        this.getMongo().update(this._fullName, query, updateSpec, upsert, multi);
-
-        // Enforce write concern, if required
-        if (wc) {
-            result = this.runCommand("getLastError", wc instanceof WriteConcern ? wc.toJSON() : wc);
-        }
+    if (hint) {
+        updateOp.hint(hint);
     }
 
-    this._printExtraInfo("Updated", startTime);
+    if (upsert) {
+        updateOp = updateOp.upsert();
+    }
+
+    if (collation) {
+        updateOp.collation(collation);
+    }
+
+    if (arrayFilters) {
+        updateOp.arrayFilters(arrayFilters);
+    }
+
+    if (multi) {
+        updateOp.update(updateSpec);
+    } else {
+        updateOp.updateOne(updateSpec);
+    }
+
+    try {
+        result = bulk.execute(wc).toSingleResult();
+    } catch (ex) {
+        if (ex instanceof BulkWriteError) {
+            result = ex.toSingleResult();
+        } else if (ex instanceof WriteCommandError) {
+            result = ex;
+        } else {
+            // Other exceptions thrown
+            throw ex;
+        }
+    }
     return result;
 };
 
@@ -669,24 +632,6 @@ DBCollection.prototype.createIndexes = function(keys, options, commitQuorum) {
         {createIndexes: this.getName(), indexes: indexSpecs, commitQuorum: commitQuorum});
 };
 
-DBCollection.prototype.ensureIndex = function(keys, options) {
-    if (arguments.length > 2) {
-        throw new Error("ensureIndex accepts up to 2 arguments");
-    }
-
-    var result = this.createIndex(keys, options);
-
-    if (this.getMongo().writeMode() != "legacy") {
-        return result;
-    }
-
-    err = this.getDB().getLastErrorObj();
-    if (err.err) {
-        return err;
-    }
-    // nothing returned on success
-};
-
 DBCollection.prototype.reIndex = function() {
     return this._db.runCommand({reIndex: this.getName()});
 };
@@ -765,34 +710,6 @@ DBCollection.prototype.renameCollection = function(newName, dropTarget) {
     });
 };
 
-// Display verbose information about the operation
-DBCollection.prototype._printExtraInfo = function(action, startTime) {
-    if (typeof _verboseShell === 'undefined' || !_verboseShell) {
-        __callLastError = true;
-        return;
-    }
-
-    // explicit w:1 so that replset getLastErrorDefaults aren't used here which would be bad.
-    var res = this._db.getLastErrorCmd(1);
-    if (res) {
-        if (res.err != undefined && res.err != null) {
-            // error occurred, display it
-            print(res.err);
-            return;
-        }
-
-        var info = action + " ";
-        // hack for inserted because res.n is 0
-        info += action != "Inserted" ? res.n : 1;
-        if (res.n > 0 && res.updatedExisting != undefined)
-            info += " " + (res.updatedExisting ? "existing" : "new");
-        info += " record(s)";
-        var time = new Date().getTime() - startTime;
-        info += " in " + time + "ms";
-        print(info);
-    }
-};
-
 DBCollection.prototype.validate = function(options) {
     if (typeof (options) != 'object' && typeof (options) != 'undefined') {
         return "expected optional options to be of the following format: {full: <bool>, background: <bool>}.";
@@ -829,8 +746,8 @@ DBCollection.prototype.getShardVersion = function() {
     return this._db._adminCommand({getShardVersion: this._fullName});
 };
 
-DBCollection.prototype.getIndexes = function(filter) {
-    var res = this.runCommand("listIndexes", filter);
+DBCollection.prototype.getIndexes = function() {
+    var res = this.runCommand("listIndexes");
 
     if (!res.ok) {
         if (res.code == ErrorCodes.NamespaceNotFound) {
@@ -1125,8 +1042,10 @@ DBCollection.autocomplete = function(obj) {
  * @return {boolean}
  */
 DBCollection.prototype._isSharded = function() {
+    // Checking for 'dropped: {$ne: true}' to ensure mongo shell compatibility with earlier versions
+    // of the server
     return !!this._db.getSiblingDB("config").collections.countDocuments(
-        {_id: this._fullName, dropped: false});
+        {_id: this._fullName, dropped: {$ne: true}});
 };
 
 /**
@@ -1147,13 +1066,18 @@ DBCollection.prototype.getShardDistribution = function() {
 
     collStats.forEach(function(extShardStats) {
         // Extract and store only the relevant subset of the stats for this shard
+        var newVersion = config.collections.countDocuments(
+            {_id: extShardStats.ns, timestamp: {$exists: true}}, {limit: 1});
+        var collUuid = config.collections.findOne({_id: extShardStats.ns}).uuid;
         const shardStats = {
             shardId: extShardStats.shard,
             host: config.shards.findOne({_id: extShardStats.shard}).host,
             size: extShardStats.storageStats.size,
             count: extShardStats.storageStats.count,
-            numChunks:
-                config.chunks.countDocuments({ns: extShardStats.ns, shard: extShardStats.shard}),
+            numChunks: (newVersion ? config.chunks.countDocuments(
+                                         {uuid: collUuid, shard: extShardStats.shard})
+                                   : config.chunks.countDocuments(
+                                         {ns: extShardStats.ns, shard: extShardStats.shard})),
             avgObjSize: extShardStats.storageStats.avgObjSize
         };
 
@@ -1320,9 +1244,7 @@ DBCollection.prototype.getSlaveOk = function() {
     return this.getSecondaryOk();
 };
 
-DBCollection.prototype.setSecondaryOk = function(value) {
-    if (value === undefined)
-        value = true;
+DBCollection.prototype.setSecondaryOk = function(value = true) {
     this._secondaryOk = value;
 };
 
