@@ -5,6 +5,7 @@ var DB;
 (function() {
 
 var _defaultWriteConcern = {w: 'majority', wtimeout: 10 * 60 * 1000};
+const kWireVersionSupportingScramSha256Fallback = 15;
 
 if (DB === undefined) {
     DB = function(mongo, name) {
@@ -13,6 +14,14 @@ if (DB === undefined) {
     };
 }
 
+/**
+ * Rotate certificates, CRLs, and CA files.
+ * @param {String} message optional message for server to log at rotation time
+ */
+DB.prototype.rotateCertificates = function(message) {
+    return this._adminCommand({rotateCertificates: 1, message: message});
+};
+
 DB.prototype.getMongo = function() {
     assert(this._mongo, "why no mongo!");
     return this._mongo;
@@ -28,8 +37,19 @@ DB.prototype.getName = function() {
     return this._name;
 };
 
-DB.prototype.stats = function(scale) {
-    return this.runCommand({dbstats: 1, scale: scale});
+/**
+ * Gets DB level statistics. opt can be a number representing the scale for backwards compatibility
+ * or a document with options passed along to the dbstats command.
+ */
+DB.prototype.stats = function(opt) {
+    var cmd = {dbstats: 1};
+
+    if (opt === undefined)
+        return this.runCommand(cmd);
+    if (typeof (opt) !== "object")
+        return this.runCommand(Object.extend(cmd, {scale: opt}));
+
+    return this.runCommand(Object.extend(cmd, opt));
 };
 
 DB.prototype.getCollection = function(name) {
@@ -165,19 +185,13 @@ DB.prototype.runCommand = function(obj, extra, queryOptions) {
         // "error doing query: failed". Even though this message is arguably incorrect
         // for a command failing due to a connection failure, we preserve it for backwards
         // compatibility. See SERVER-18334 for details.
-        if (ex.message.indexOf("network error") >= 0) {
+        if (ex.hasOwnProperty("message") && ex.message.indexOf("network error") >= 0) {
             throw new Error("error doing query: failed: " + ex.message);
         }
         throw ex;
     }
 };
 
-DB.prototype.runCommandWithMetadata = function(commandArgs, metadata) {
-    const session = this.getSession();
-    return session._getSessionAwareClient().runCommandWithMetadata(
-        session, this._name, metadata, commandArgs);
-};
-
 DB.prototype._dbCommand = DB.prototype.runCommand;
 DB.prototype._dbReadCommand = DB.prototype.runReadCommand;
 
@@ -189,12 +203,34 @@ DB.prototype.adminCommand = function(obj, extra) {
 
 DB.prototype._adminCommand = DB.prototype.adminCommand;  // alias old name
 
+DB.prototype._helloOrLegacyHello = function(args) {
+    let cmd = this.getMongo().getApiParameters().version ? {hello: 1} : {isMaster: 1};
+    if (args) {
+        Object.assign(cmd, args);
+    }
+    return this.runCommand(cmd);
+};
+
+DB.prototype._runCommandWithoutApiStrict = function(command) {
+    let commandWithoutApiStrict = Object.assign({}, command);
+    if (this.getMongo().getApiParameters().strict) {
+        // Permit this command invocation, even if it's not in the requested API version.
+        commandWithoutApiStrict["apiStrict"] = false;
+    }
+
+    return this.runCommand(commandWithoutApiStrict);
+};
+
 DB.prototype._runAggregate = function(cmdObj, aggregateOptions) {
     assert(cmdObj.pipeline instanceof Array, "cmdObj must contain a 'pipeline' array");
     assert(cmdObj.aggregate !== undefined, "cmdObj must contain 'aggregate' field");
     assert(aggregateOptions === undefined || aggregateOptions instanceof Object,
            "'aggregateOptions' argument must be an object");
 
+    // Disallow explicit API parameters on the aggregate shell helper; callers should use runCommand
+    // directly if they want to test this.
+    assert.noAPIParams(aggregateOptions);
+
     // Make a copy of the initial command object, i.e. {aggregate: x, pipeline: [...]}.
     cmdObj = Object.extend({}, cmdObj);
 
@@ -229,23 +265,7 @@ DB.prototype._runAggregate = function(cmdObj, aggregateOptions) {
         cmdObj.cursor = {};
     }
 
-    const pipeline = cmdObj.pipeline;
-
-    // Check whether the pipeline has a stage which performs writes like $out. If not, we may
-    // run on a Secondary and should attach a readPreference.
-    const hasWritingStage = (function() {
-        if (pipeline.length == 0) {
-            return false;
-        }
-        const lastStage = pipeline[pipeline.length - 1];
-        return lastStage.hasOwnProperty("$out") || lastStage.hasOwnProperty("$merge");
-    }());
-
-    const doAgg = function(cmdObj) {
-        return hasWritingStage ? this.runCommand(cmdObj) : this.runReadCommand(cmdObj);
-    }.bind(this);
-
-    const res = doAgg(cmdObj);
+    const res = this.runReadCommand(cmdObj);
 
     if (!res.ok && (res.code == 17020 || res.errmsg == "unrecognized field \"cursor") &&
         !("cursor" in aggregateOptions)) {
@@ -405,7 +425,7 @@ DB.prototype.dropDatabase = function(writeConcern) {
  * Shuts down the database.  Must be run while using the admin database.
  * @param opts Options for shutdown. Possible options are:
  *   - force: (boolean) if the server should shut down, even if there is no
- *     up-to-date slave
+ *     up-to-date secondary
  *   - timeoutSecs: (number) the server will continue checking over timeoutSecs
  *     if any other servers have caught up enough for it to shut down.
  */
@@ -437,99 +457,6 @@ DB.prototype.shutdownServer = function(opts) {
     }
 };
 
-/**
-  Clone database on another server to here. This functionality was removed as of MongoDB 4.2.
-  The shell helper is kept to maintain compatibility with previous versions of MongoDB.
-  <p>
-  Generally, you should dropDatabase() first as otherwise the cloned information will MERGE
-  into whatever data is already present in this database.  (That is however a valid way to use
-  clone if you are trying to do something intentionally, such as union three non-overlapping
-  databases into one.)
-  <p>
-  This is a low level administrative function will is not typically used.
-
- * @param {String} from Where to clone from (dbhostname[:port]).  May not be this database
-                   (self) as you cannot clone to yourself.
- * @return Object returned has member ok set to true if operation succeeds, false otherwise.
- * See also: db.copyDatabase()
- */
-DB.prototype.cloneDatabase = function(from) {
-    print(
-        "WARNING: db.cloneDatabase will only function with MongoDB 4.0 and below. See http://dochub.mongodb.org/core/4.2-copydb-clone");
-    assert(isString(from) && from.length);
-    return this._dbCommand({clone: from});
-};
-
-/**
-  Copy database from one server or name to another server or name. This functionality was
-  removed as of MongoDB 4.2. The shell helper is kept to maintain compatibility with previous
-  versions of MongoDB.
-
-  Generally, you should dropDatabase() first as otherwise the copied information will MERGE
-  into whatever data is already present in this database (and you will get duplicate objects
-  in collections potentially.)
-
-  For security reasons this function only works when executed on the "admin" db.  However,
-  if you have access to said db, you can copy any database from one place to another.
-
-  This method provides a way to "rename" a database by copying it to a new db name and
-  location.  Additionally, it effectively provides a repair facility.
-
-  * @param {String} fromdb database name from which to copy.
-  * @param {String} todb database name to copy to.
-  * @param {String} fromhost hostname of the database (and optionally, ":port") from which to
-                    copy the data.  default if unspecified is to copy from self.
-  * @return Object returned has member ok set to true if operation succeeds, false otherwise.
-  * See also: db.clone()
-*/
-DB.prototype.copyDatabase = function(
-    fromdb, todb, fromhost, username, password, mechanism, slaveOk) {
-    print(
-        "WARNING: db.copyDatabase will only function with MongoDB 4.0 and below. See http://dochub.mongodb.org/core/4.2-copydb-clone");
-    assert(isString(fromdb) && fromdb.length);
-    assert(isString(todb) && todb.length);
-    fromhost = fromhost || "";
-    if ((typeof username === "boolean") && (typeof password === "undefined") &&
-        (typeof mechanism === "undefined") && (typeof slaveOk === "undefined")) {
-        slaveOk = username;
-        username = undefined;
-    }
-    if (typeof slaveOk !== "boolean") {
-        slaveOk = false;
-    }
-
-    if (!mechanism) {
-        mechanism = this._getDefaultAuthenticationMechanism(username, fromdb);
-    }
-    assert(mechanism == "SCRAM-SHA-1" || mechanism == "SCRAM-SHA-256" || mechanism == "MONGODB-CR");
-
-    // Check for no auth or copying from localhost
-    if (!username || !password || fromhost == "") {
-        return this._adminCommand(
-            {copydb: 1, fromhost: fromhost, fromdb: fromdb, todb: todb, slaveOk: slaveOk});
-    }
-
-    // Use the copyDatabase native helper for SCRAM-SHA-1/256
-    if (mechanism != "MONGODB-CR") {
-        // TODO SERVER-30886: Add session support for Mongo.prototype.copyDatabaseWithSCRAM().
-        return this.getMongo().copyDatabaseWithSCRAM(
-            fromdb, todb, fromhost, username, password, slaveOk);
-    }
-
-    // Fall back to MONGODB-CR
-    var n = assert.commandWorked(this._adminCommand({copydbgetnonce: 1, fromhost: fromhost}));
-    return this._adminCommand({
-        copydb: 1,
-        fromhost: fromhost,
-        fromdb: fromdb,
-        todb: todb,
-        username: username,
-        nonce: n.nonce,
-        key: this.__pwHash(n.nonce, username, password),
-        slaveOk: slaveOk,
-    });
-};
-
 DB.prototype.help = function() {
     print("DB methods:");
     print(
@@ -537,11 +464,7 @@ DB.prototype.help = function() {
     print(
         "\tdb.aggregate([pipeline], {options}) - performs a collectionless aggregation on this database; returns a cursor");
     print("\tdb.auth(username, password)");
-    print("\tdb.cloneDatabase(fromhost) - will only function with MongoDB 4.0 and below");
     print("\tdb.commandHelp(name) returns the help for the command");
-    print(
-        "\tdb.copyDatabase(fromdb, todb, fromhost) - will only function with MongoDB 4.0 and below");
-    print("\tdb.createCollection(name, {size: ..., capped: ..., max: ...})");
     print("\tdb.createUser(userDocument)");
     print("\tdb.createView(name, viewOn, [{$operator: {...}}, ...], {viewOptions})");
     print("\tdb.currentOp() displays currently executing operations in the db");
@@ -554,8 +477,6 @@ DB.prototype.help = function() {
     print("\tdb.getCollectionInfos([filter]) - returns a list that contains the names and options" +
           " of the db's collections");
     print("\tdb.getCollectionNames()");
-    print("\tdb.getLastError() - just returns the err msg string");
-    print("\tdb.getLastErrorObj() - return full status object");
     print("\tdb.getLogComponents()");
     print("\tdb.getMongo() get the server connection object");
     print("\tdb.getMongo().setSecondaryOk() allow queries on a replication secondary server");
@@ -577,7 +498,8 @@ DB.prototype.help = function() {
     print("\tdb.printReplicationInfo()");
     print("\tdb.printShardingStatus()");
     print("\tdb.printSecondaryReplicationInfo()");
-    print("\tdb.resetError()");
+    print(
+        "\tdb.rotateCertificates(message) - rotates certificates, CRLs, and CA files and logs an optional message");
     print(
         "\tdb.runCommand(cmdObj) run a database command.  if cmdObj is a string, turns it into {cmdObj: 1}");
     print("\tdb.serverStatus()");
@@ -762,37 +684,10 @@ DB.prototype._groupFixParms = function(parmsObj) {
     return parms;
 };
 
-DB.prototype.resetError = function() {
-    return this.runCommand({reseterror: 1});
-};
-
 DB.prototype.forceError = function() {
     return this.runCommand({forceerror: 1});
 };
 
-DB.prototype.getLastError = function(w, wtimeout) {
-    var res = this.getLastErrorObj(w, wtimeout);
-    if (!res.ok)
-        throw _getErrorWithCode(ret, "getlasterror failed: " + tojson(res));
-    return res.err;
-};
-DB.prototype.getLastErrorObj = function(w, wtimeout, j) {
-    var cmd = {getlasterror: 1};
-    if (w) {
-        cmd.w = w;
-        if (wtimeout)
-            cmd.wtimeout = wtimeout;
-        if (j != null)
-            cmd.j = j;
-    }
-    var res = this.runCommand(cmd);
-
-    if (!res.ok)
-        throw _getErrorWithCode(res, "getlasterror failed: " + tojson(res));
-    return res;
-};
-DB.prototype.getLastErrorCmd = DB.prototype.getLastErrorObj;
-
 DB.prototype._getCollectionInfosCommand = function(
     filter, nameOnly = false, authorizedCollections = false, options = {}) {
     filter = filter || {};
@@ -905,14 +800,24 @@ DB.prototype.hello = function() {
     return this.runCommand("hello");
 };
 
-var commandUnsupported = function(res) {
-    return (!res.ok &&
-            (res.errmsg.startsWith("no such cmd") || res.errmsg.startsWith("no such command") ||
-             res.code === 59 /* CommandNotFound */));
+DB.prototype.currentOp = function(arg) {
+    try {
+        const results = this.currentOpCursor(arg).toArray();
+        let res = {"inprog": results.length > 0 ? results : [], "ok": 1};
+        Object.defineProperty(res, "fsyncLock", {
+            get: function() {
+                throw Error(
+                    "fsyncLock is no longer included in the currentOp shell helper, run db.runCommand({currentOp: 1}) instead.");
+            }
+        });
+        return res;
+    } catch (e) {
+        return {"ok": 0, "code": e.code, "errmsg": "Error executing $currentOp: " + e.message};
+    }
 };
 
-DB.prototype.currentOp = function(arg) {
-    var q = {};
+DB.prototype.currentOpCursor = function(arg) {
+    let q = {};
     if (arg) {
         if (typeof (arg) == "object")
             Object.extend(q, arg);
@@ -920,40 +825,40 @@ DB.prototype.currentOp = function(arg) {
             q["$all"] = true;
     }
 
-    var commandObj = {"currentOp": 1};
-    Object.extend(commandObj, q);
-    var res = this.adminCommand(commandObj);
-    if (commandUnsupported(res)) {
-        // always send legacy currentOp with default (null) read preference (SERVER-17951)
-        const session = this.getSession();
-        const readPreference = session.getOptions().getReadPreference();
-        try {
-            session.getOptions().setReadPreference(null);
-            res = this.getSiblingDB("admin").$cmd.sys.inprog.findOne(q);
-        } finally {
-            session.getOptions().setReadPreference(readPreference);
+    // Convert the incoming currentOp command into an equivalent aggregate command
+    // of the form {aggregate:1, pipeline: [{$currentOp: {idleConnections: $all, allUsers:
+    // !$ownOps, truncateOps: false}}, {$match: {<user-defined filter>}}], cursor:{}}.
+    let pipeline = [];
+
+    let currOpArgs = {};
+    let currOpStage = {"$currentOp": currOpArgs};
+    currOpArgs["allUsers"] = !q["$ownOps"];
+    currOpArgs["idleConnections"] = !!q["$all"];
+    currOpArgs["truncateOps"] = false;
+
+    pipeline.push(currOpStage);
+
+    let matchArgs = {};
+    let matchStage = {"$match": matchArgs};
+    for (const fieldname of Object.keys(q)) {
+        if (fieldname !== "$all" && fieldname !== "$ownOps" && fieldname !== "$truncateOps") {
+            matchArgs[fieldname] = q[fieldname];
         }
     }
-    return res;
+
+    pipeline.push(matchStage);
+
+    // The legacy db.currentOp() shell helper ignored any explicitly set read preference and used
+    // the default, with the ability to also run on secondaries. To preserve this behavior we will
+    // run the aggregate with read preference "primaryPreferred".
+    return this.getSiblingDB("admin").aggregate(pipeline,
+                                                {"$readPreference": {"mode": "primaryPreferred"}});
 };
-DB.prototype.currentOP = DB.prototype.currentOp;
 
 DB.prototype.killOp = function(op) {
     if (!op)
         throw Error("no opNum to kill specified");
-    var res = this.adminCommand({'killOp': 1, 'op': op});
-    if (commandUnsupported(res)) {
-        // fall back for old servers
-        const session = this.getSession();
-        const readPreference = session.getOptions().getReadPreference();
-        try {
-            session.getOptions().setReadPreference(null);
-            res = this.getSiblingDB("admin").$cmd.sys.killop.findOne({'op': op});
-        } finally {
-            session.getOptions().setReadPreference(readPreference);
-        }
-    }
-    return res;
+    return this.adminCommand({'killOp': 1, 'op': op});
 };
 DB.prototype.killOP = DB.prototype.killOp;
 
@@ -972,7 +877,7 @@ DB.tsToSeconds = function(x) {
        use local
        db.getReplicationInfo();
   </pre>
-  * @return Object timeSpan: time span of the oplog from start to end  if slave is more out
+  * @return Object timeSpan: time span of the oplog from start to end  if secondary is more out
   *                          of date than that, it can't recover without a complete resync
 */
 DB.prototype.getReplicationInfo = function() {
@@ -1033,11 +938,19 @@ DB.prototype.getReplicationInfo = function() {
 DB.prototype.printReplicationInfo = function() {
     var result = this.getReplicationInfo();
     if (result.errmsg) {
-        var isMaster = this.isMaster();
-        if (isMaster.arbiterOnly) {
+        let reply, isPrimary;
+        if (this.getMongo().getApiParameters().apiVersion) {
+            reply = this.hello();
+            isPrimary = reply.isWritablePrimary;
+        } else {
+            reply = this.isMaster();
+            isPrimary = reply.ismaster;
+        }
+
+        if (reply.arbiterOnly) {
             print("cannot provide replication status from an arbiter.");
             return;
-        } else if (!isMaster.ismaster) {
+        } else if (!isPrimary) {
             print("this is a secondary, printing secondary replication info.");
             this.printSecondaryReplicationInfo();
             return;
@@ -1087,28 +1000,17 @@ DB.prototype.printSecondaryReplicationInfo = function() {
         return null;
     }
 
-    function g(x) {
-        assert(x, "how could this be null (printSecondaryReplicationInfo gx)");
-        print("source: " + x.host);
-        if (x.syncedTo) {
-            var st = new Date(DB.tsToSeconds(x.syncedTo) * 1000);
-            getReplLag(st);
-        } else {
-            print("\tdoing initial sync");
-        }
-    }
-
-    function r(x) {
-        assert(x, "how could this be null (printSecondaryReplicationInfo rx)");
-        if (x.state == 1 || x.state == 7) {  // ignore primaries (1) and arbiters (7)
+    function printNodeReplicationInfo(node) {
+        assert(node);
+        if (node.state === 1 || node.state === 7) {  // ignore primaries (1) and arbiters (7)
             return;
         }
 
-        print("source: " + x.name);
-        if (x.optime) {
-            getReplLag(x.optimeDate);
+        print("source: " + node.name);
+        if (node.optime && node.health != 0) {
+            getReplLag(node.optimeDate);
         } else {
-            print("\tno replication info, yet.  State: " + x.stateStr);
+            print("\tno replication info, yet.  State: " + node.stateStr);
         }
     }
 
@@ -1123,7 +1025,8 @@ DB.prototype.printSecondaryReplicationInfo = function() {
     var L = this.getSiblingDB("local");
 
     if (L.system.replset.count() != 0) {
-        var status = this.adminCommand({'replSetGetStatus': 1});
+        const status =
+            this.getSiblingDB('admin')._runCommandWithoutApiStrict({'replSetGetStatus': 1});
         primary = getPrimary(status.members);
         if (primary) {
             startOptimeDate = primary.optimeDate;
@@ -1150,14 +1053,14 @@ DB.prototype.printSecondaryReplicationInfo = function() {
                     status.members[i].syncSourceHost,
                     status.initialSyncStatus.remainingInitialSyncEstimatedMillis);
             } else {
-                r(status.members[i]);
+                printNodeReplicationInfo(status.members[i]);
             }
         }
     }
 };
 
 DB.prototype.serverBuildInfo = function() {
-    return this._adminCommand("buildinfo");
+    return this.getSiblingDB("admin")._runCommandWithoutApiStrict({buildinfo: 1});
 };
 
 // Used to trim entries from the metrics.commands that have never been executed
@@ -1225,8 +1128,8 @@ DB.prototype.listCommands = function() {
 
         if (c.adminOnly)
             s += " adminOnly ";
-        if (c.slaveOk)
-            s += " slaveOk ";
+        if (c.secondaryOk)
+            s += " secondaryOk ";
 
         s += "\n  ";
         s += c.help.replace(/\n/g, '\n  ');
@@ -1245,23 +1148,14 @@ DB.prototype.fsyncLock = function() {
 };
 
 DB.prototype.fsyncUnlock = function() {
-    var res = this.adminCommand({fsyncUnlock: 1});
-    if (commandUnsupported(res)) {
-        const session = this.getSession();
-        const readPreference = session.getOptions().getReadPreference();
-        try {
-            session.getOptions().setReadPreference(null);
-            res = this.getSiblingDB("admin").$cmd.sys.unlock.findOne();
-        } finally {
-            session.getOptions().setReadPreference(readPreference);
-        }
-    }
-    return res;
+    return this.adminCommand({fsyncUnlock: 1});
 };
 
 DB.autocomplete = function(obj) {
-    // Time out if a transaction or other op holds locks we need. Caller suppresses exceptions.
-    var colls = obj._getCollectionNamesInternal({maxTimeMS: 1000});
+    // In interactive mode, time out if a transaction or other op holds locks we need. Caller
+    // suppresses exceptions. In non-interactive mode, don't specify a timeout, because in an
+    // automated test we prefer consistent results over quick feedback.
+    var colls = obj._getCollectionNamesInternal(isInteractive() ? {maxTimeMS: 1000} : {});
     var ret = [];
     for (var i = 0; i < colls.length; i++) {
         if (colls[i].match(/^[a-zA-Z0-9_.\$]+$/))
@@ -1270,7 +1164,7 @@ DB.autocomplete = function(obj) {
     return ret;
 };
 
-DB.prototype.setSlaveOk = function(value) {
+DB.prototype.setSlaveOk = function(value = true) {
     print(
         "WARNING: setSlaveOk() is deprecated and may be removed in the next major release. Please use setSecondaryOk() instead.");
     this.setSecondaryOk(value);
@@ -1282,9 +1176,7 @@ DB.prototype.getSlaveOk = function() {
     return this.getSecondaryOk();
 };
 
-DB.prototype.setSecondaryOk = function(value) {
-    if (value == undefined)
-        value = true;
+DB.prototype.setSecondaryOk = function(value = true) {
     this._secondaryOk = value;
 };
 
@@ -1394,29 +1286,6 @@ function _hashPassword(username, password) {
     return hex_md5(username + ":mongo:" + password);
 }
 
-/**
- * Used for updating users in systems with V1 style user information
- * (ie MongoDB v2.4 and prior)
- */
-DB.prototype._updateUserV1 = function(name, updateObject, writeConcern) {
-    var setObj = {};
-    if (updateObject.pwd) {
-        setObj["pwd"] = _hashPassword(name, updateObject.pwd);
-    }
-    if (updateObject.extraData) {
-        setObj["extraData"] = updateObject.extraData;
-    }
-    if (updateObject.roles) {
-        setObj["roles"] = updateObject.roles;
-    }
-
-    this.system.users.update({user: name, userSource: null}, {$set: setObj});
-    var errObj = this.getLastErrorObj(writeConcern['w'], writeConcern['wtimeout']);
-    if (errObj.err) {
-        throw _getErrorWithCode(errObj, "Updating user failed: " + errObj.err);
-    }
-};
-
 DB.prototype.updateUser = function(name, updateObject, writeConcern) {
     var cmdObj = {updateUser: name};
     cmdObj = Object.extend(cmdObj, updateObject);
@@ -1428,11 +1297,6 @@ DB.prototype.updateUser = function(name, updateObject, writeConcern) {
         return;
     }
 
-    if (res.errmsg == "no such cmd: updateUser") {
-        this._updateUserV1(name, updateObject, cmdObj['writeConcern']);
-        return;
-    }
-
     throw _getErrorWithCode(res, "Updating user failed: " + res.errmsg);
 };
 
@@ -1466,33 +1330,9 @@ DB.prototype.dropUser = function(username, writeConcern) {
         return false;
     }
 
-    if (res.errmsg == "no such cmd: dropUsers") {
-        return this._removeUserV1(username, cmdObj['writeConcern']);
-    }
-
     throw _getErrorWithCode(res, res.errmsg);
 };
 
-/**
- * Used for removing users in systems with V1 style user information
- * (ie MongoDB v2.4 and prior)
- */
-DB.prototype._removeUserV1 = function(username, writeConcern) {
-    this.getCollection("system.users").remove({user: username});
-
-    var le = this.getLastErrorObj(writeConcern['w'], writeConcern['wtimeout']);
-
-    if (le.err) {
-        throw _getErrorWithCode(le, "Couldn't remove user: " + le.err);
-    }
-
-    if (le.n == 1) {
-        return true;
-    } else {
-        return false;
-    }
-};
-
 DB.prototype.dropAllUsers = function(writeConcern) {
     var res = this.runCommand({
         dropAllUsersFromDatabase: 1,
@@ -1512,10 +1352,17 @@ DB.prototype.__pwHash = function(nonce, username, pass) {
 
 DB.prototype._defaultAuthenticationMechanism = null;
 
+function _fallbackToScramSha256(helloResult) {
+    return helloResult && isNumber(helloResult.maxWireVersion) &&
+        helloResult.maxWireVersion >= kWireVersionSupportingScramSha256Fallback;
+}
+
 DB.prototype._getDefaultAuthenticationMechanism = function(username, database) {
+    let result = null;
     if (username !== undefined) {
         const userid = database + "." + username;
-        const result = this.runCommand({isMaster: 1, saslSupportedMechs: userid});
+        result = this._helloOrLegacyHello({saslSupportedMechs: userid});
+
         if (result.ok && (result.saslSupportedMechs !== undefined)) {
             const mechs = result.saslSupportedMechs;
             if (!Array.isArray(mechs)) {
@@ -1537,14 +1384,18 @@ DB.prototype._getDefaultAuthenticationMechanism = function(username, database) {
         }
         // If isMaster doesn't support saslSupportedMechs,
         // or if we couldn't agree on a mechanism,
-        // then fallthrough to configured default or SCRAM-SHA-1.
+        // then fall through to a default mech, either
+        // configured or implicit based on the wire version
     }
 
     // Use the default auth mechanism if set on the command line.
-    if (this._defaultAuthenticationMechanism != null)
+    if (this._defaultAuthenticationMechanism != null) {
         return this._defaultAuthenticationMechanism;
+    }
 
-    return "SCRAM-SHA-1";
+    // for later wire versions, we prefer (or require) SCRAM-SHA-256
+    // if a fallback is required
+    return _fallbackToScramSha256(result) ? "SCRAM-SHA-256" : "SCRAM-SHA-1";
 };
 
 DB.prototype._defaultGssapiServiceName = null;
@@ -1584,8 +1435,8 @@ DB.prototype._authOrThrow = function() {
     params.db = this.getName();
     var good = this.getMongo().auth(params);
     if (good) {
-        // auth enabled, and should try to use isMaster and replSetGetStatus to build prompt
-        this.getMongo().authStatus = {authRequired: true, isMaster: true, replSetGetStatus: true};
+        // auth enabled, and should try to use hello and replSetGetStatus to build prompt
+        this.getMongo().authStatus = {authRequired: true, hello: true, replSetGetStatus: true};
     }
 
     return good;
@@ -1831,6 +1682,57 @@ DB.prototype.watch = function(pipeline, options) {
     return this._runAggregate({aggregate: 1, pipeline: pipeline}, aggOptions);
 };
 
+DB.prototype.getFreeMonitoringStatus = function() {
+    'use strict';
+    return assert.commandWorked(this.adminCommand({getFreeMonitoringStatus: 1}));
+};
+
+DB.prototype.enableFreeMonitoring = function() {
+    'use strict';
+    let reply, isPrimary;
+    if (this.getMongo().getApiParameters().apiVersion) {
+        reply = this.hello();
+        isPrimary = reply.isWritablePrimary;
+    } else {
+        reply = this.isMaster();
+        isPrimary = reply.ismaster;
+    }
+
+    if (!isPrimary) {
+        print("ERROR: db.enableFreeMonitoring() may only be run on a primary");
+        return;
+    }
+
+    assert.commandWorked(this.adminCommand({setFreeMonitoring: 1, action: 'enable'}));
+
+    const cmd = this.adminCommand({getFreeMonitoringStatus: 1});
+    if (!cmd.ok && (cmd.code == ErrorCode.Unauthorized)) {
+        // Edge case: It's technically possible that a user can change free-mon state,
+        // but is not allowed to inspect it.
+        print("Successfully initiated free monitoring, but unable to determine status " +
+              "as you lack the 'checkFreeMonitoringStatus' privilege.");
+        return;
+    }
+    assert.commandWorked(cmd);
+
+    if (cmd.state !== 'enabled') {
+        const url = this.adminCommand({'getParameter': 1, 'cloudFreeMonitoringEndpointURL': 1})
+                        .cloudFreeMonitoringEndpointURL;
+
+        print("Unable to get immediate response from the Cloud Monitoring service. We will" +
+              "continue to retry in the background. Please check your firewall " +
+              "settings to ensure that mongod can communicate with \"" + url + "\"");
+        return;
+    }
+
+    print(tojson(cmd));
+};
+
+DB.prototype.disableFreeMonitoring = function() {
+    'use strict';
+    assert.commandWorked(this.adminCommand({setFreeMonitoring: 1, action: 'disable'}));
+};
+
 // Writing `this.hasOwnProperty` would cause DB.prototype.getCollection() to be called since the
 // DB's getProperty() handler in C++ takes precedence when a property isn't defined on the DB
 // instance directly. The "hasOwnProperty" property is defined on Object.prototype, so we must
@@ -1843,4 +1745,51 @@ DB.prototype.getSession = function() {
     return this._session;
 };
 })(Object.prototype.hasOwnProperty);
+
+DB.prototype.createEncryptedCollection = function(name, opts) {
+    assert.neq(
+        opts, undefined, `createEncryptedCollection expected an opts object, it is undefined`);
+    assert(opts.hasOwnProperty("encryptedFields") && typeof opts.encryptedFields == "object",
+           `opts must contain an encryptedFields document'`);
+
+    const res = assert.commandWorked(this.createCollection(name, opts));
+
+    const cis = this.getCollectionInfos({"name": name});
+    assert.eq(cis.length, 1, `Expected to find one collection named '${name}'`);
+
+    const ci = cis[0];
+    assert(ci.hasOwnProperty("options"), `Expected collection '${name}' to have 'options'`);
+    const options = ci.options;
+    assert(options.hasOwnProperty("encryptedFields"),
+           `Expected collection '${name}' to have 'encryptedFields'`);
+    const ef = options.encryptedFields;
+
+    assert.commandWorked(this.getCollection(name).createIndex({__safeContent__: 1}));
+
+    assert.commandWorked(
+        this.createCollection(ef.escCollection, {clusteredIndex: {key: {_id: 1}, unique: true}}));
+    assert.commandWorked(
+        this.createCollection(ef.eccCollection, {clusteredIndex: {key: {_id: 1}, unique: true}}));
+    assert.commandWorked(
+        this.createCollection(ef.ecocCollection, {clusteredIndex: {key: {_id: 1}, unique: true}}));
+
+    return res;
+};
+
+DB.prototype.dropEncryptedCollection = function(name) {
+    const ci = db.getCollectionInfos({name: name})[0];
+    if (ci == undefined) {
+        throw `Encrypted Collection '${name}' not found`;
+    }
+
+    const ef = ci.options.encryptedFields;
+    if (ef == undefined) {
+        throw `Encrypted Collection '${name}' not found`;
+    }
+
+    this.getCollection(ef.escCollection).drop();
+    this.getCollection(ef.eccCollection).drop();
+    this.getCollection(ef.ecocCollection).drop();
+    return this.getCollection(name).drop();
+};
 }());
