@@ -31,12 +31,18 @@
 
 #include "mongo/shell/encrypted_dbclient_base.h"
 
+#include <js/Object.h>
+#include <js/ValueArray.h>
+
 #include "mongo/base/data_cursor.h"
 #include "mongo/base/data_type_validated.h"
 #include "mongo/bson/bson_depth.h"
 #include "mongo/client/dbclient_base.h"
+#include "mongo/config.h"
 #include "mongo/crypto/aead_encryption.h"
+#include "mongo/crypto/fle_crypto.h"
 #include "mongo/crypto/fle_data_frames.h"
+#include "mongo/crypto/fle_field_schema_gen.h"
 #include "mongo/crypto/symmetric_crypto.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
@@ -56,6 +62,7 @@
 #include "mongo/shell/kms.h"
 #include "mongo/shell/kms_gen.h"
 #include "mongo/shell/shell_options.h"
+#include "mongo/util/assert_util.h"
 
 namespace mongo {
 
@@ -93,28 +100,18 @@ EncryptedDBClientBase::EncryptedDBClientBase(std::unique_ptr<DBClientBase> conn,
     : _conn(std::move(conn)), _encryptionOptions(std::move(encryptionOptions)), _cx(cx) {
     validateCollection(cx, collection);
     _collection = JS::Heap<JS::Value>(collection);
-    uassert(31078,
-            "Cannot use WriteMode Legacy with Field Level Encryption",
-            shellGlobalParams.writeMode != "legacy");
 };
 
 std::string EncryptedDBClientBase::getServerAddress() const {
     return _conn->getServerAddress();
 }
 
-bool EncryptedDBClientBase::call(Message& toSend,
-                                 Message& response,
-                                 bool assertOk,
-                                 std::string* actualServer) {
-    return _conn->call(toSend, response, assertOk, actualServer);
+void EncryptedDBClientBase::_call(Message& toSend, Message& response, std::string* actualServer) {
+    _conn->call(toSend, response, actualServer);
 }
 
 void EncryptedDBClientBase::say(Message& toSend, bool isRetry, std::string* actualServer) {
-    MONGO_UNREACHABLE;
-}
-
-bool EncryptedDBClientBase::lazySupported() const {
-    return _conn->lazySupported();
+    return _conn->say(toSend, isRetry, actualServer);
 }
 
 BSONObj EncryptedDBClientBase::encryptDecryptCommand(const BSONObj& object,
@@ -126,7 +123,7 @@ BSONObj EncryptedDBClientBase::encryptDecryptCommand(const BSONObj& object,
     // decrypt payload throw an exception, the stack's destructor will fire. Because a stack's
     // variables are not guaranteed to be destroyed in any order, we need to add a guard
     // to ensure the stack is destroyed in order.
-    const auto frameStackGuard = makeGuard([&] {
+    const ScopeGuard frameStackGuard([&] {
         while (!frameStack.empty()) {
             frameStack.pop();
         }
@@ -199,11 +196,24 @@ void EncryptedDBClientBase::decryptPayload(ConstDataRange data,
     }
 }
 
-std::pair<rpc::UniqueReply, DBClientBase*> EncryptedDBClientBase::processResponse(
-    rpc::UniqueReply result, const StringData databaseName) {
-    auto rawReply = result->getCommandReply();
-    BSONObj decryptedDoc = encryptDecryptCommand(rawReply, false, databaseName);
+EncryptedDBClientBase::RunCommandReturn EncryptedDBClientBase::processResponseFLE1(
+    EncryptedDBClientBase::RunCommandReturn result, const StringData databaseName) {
+    auto rawReply = result.returnReply->getCommandReply();
+    return prepareReply(
+        std::move(result), databaseName, encryptDecryptCommand(rawReply, false, databaseName));
+}
 
+EncryptedDBClientBase::RunCommandReturn EncryptedDBClientBase::processResponseFLE2(
+    EncryptedDBClientBase::RunCommandReturn result, const StringData databaseName) {
+    auto rawReply = result.returnReply->getCommandReply();
+    return prepareReply(
+        std::move(result), databaseName, FLEClientCrypto::decryptDocument(rawReply, this));
+}
+
+EncryptedDBClientBase::RunCommandReturn EncryptedDBClientBase::prepareReply(
+    EncryptedDBClientBase::RunCommandReturn result,
+    const StringData databaseName,
+    BSONObj decryptedDoc) {
     rpc::OpMsgReplyBuilder replyBuilder;
     replyBuilder.setCommandReply(StatusWith<BSONObj>(decryptedDoc));
     auto msg = replyBuilder.done();
@@ -211,21 +221,49 @@ std::pair<rpc::UniqueReply, DBClientBase*> EncryptedDBClientBase::processRespons
     auto host = _conn->getServerAddress();
     auto reply = _conn->parseCommandReplyMessage(host, msg);
 
-    return {std::move(reply), this};
+    return EncryptedDBClientBase::RunCommandReturn({std::move(reply), result});
+}
+
+EncryptedDBClientBase::RunCommandReturn EncryptedDBClientBase::doRunCommand(
+    EncryptedDBClientBase::RunCommandParams params) {
+    if (params.type == EncryptedDBClientBase::RunCommandConnectionType::rawPtr) {
+        return EncryptedDBClientBase::RunCommandReturn(
+            _conn->runCommandWithTarget(std::move(params.request)));
+    }
+    invariant(params.conn);
+    return EncryptedDBClientBase::RunCommandReturn(
+        _conn->runCommandWithTarget(std::move(params.request), params.conn));
+}
+
+EncryptedDBClientBase::RunCommandReturn EncryptedDBClientBase::handleEncryptionRequest(
+    EncryptedDBClientBase::RunCommandParams params) {
+    auto commandName = params.request.getCommandName().toString();
+    auto databaseName = params.request.getDatabase().toString();
+
+    if (std::find(kEncryptedCommands.begin(), kEncryptedCommands.end(), StringData(commandName)) ==
+        std::end(kEncryptedCommands)) {
+        return doRunCommand(std::move(params));
+    }
+
+    EncryptedDBClientBase::RunCommandReturn result(doRunCommand(std::move(params)));
+    return processResponseFLE1(processResponseFLE2(std::move(result), databaseName), databaseName);
 }
 
 std::pair<rpc::UniqueReply, DBClientBase*> EncryptedDBClientBase::runCommandWithTarget(
     OpMsgRequest request) {
-    std::string commandName = request.getCommandName().toString();
-    std::string databaseName = request.getDatabase().toString();
+    EncryptedDBClientBase::RunCommandParams params(request);
+    auto result = handleEncryptionRequest(std::move(params));
+    auto returnConn = stdx::get<DBClientBase*>(result.returnConn);
+    return {std::move(result.returnReply), returnConn};
+}
 
-    if (std::find(kEncryptedCommands.begin(), kEncryptedCommands.end(), StringData(commandName)) ==
-        std::end(kEncryptedCommands)) {
-        return _conn->runCommandWithTarget(std::move(request));
-    }
-
-    auto result = _conn->runCommandWithTarget(std::move(request)).first;
-    return processResponse(std::move(result), databaseName);
+std::pair<rpc::UniqueReply, std::shared_ptr<DBClientBase>>
+EncryptedDBClientBase::runCommandWithTarget(OpMsgRequest request,
+                                            std::shared_ptr<DBClientBase> conn) {
+    EncryptedDBClientBase::RunCommandParams params(request, conn);
+    auto result = handleEncryptionRequest(std::move(params));
+    auto returnConn = stdx::get<std::shared_ptr<DBClientBase>>(result.returnConn);
+    return {std::move(result.returnReply), returnConn};
 }
 
 /**
@@ -303,7 +341,7 @@ void EncryptedDBClientBase::generateDataKey(JSContext* cx, JS::CallArgs args) {
         kmsProvider, _encryptionOptions.getKmsProviders().toBSON());
 
     SecureVector<uint8_t> dataKey(crypto::kFieldLevelEncryptionKeySize);
-    auto res = crypto::engineRandBytes(dataKey->data(), dataKey->size());
+    auto res = crypto::engineRandBytes({dataKey->data(), dataKey->size()});
     uassert(31042, "Error generating data key: " + res.codeString(), res.isOK());
 
 
@@ -364,7 +402,7 @@ void EncryptedDBClientBase::encrypt(mozjs::MozJSImplScope* scope,
     BufBuilder plaintextBuilder;
     if (args.get(1).isObject()) {
         JS::RootedObject rootedObj(cx, &args.get(1).toObject());
-        auto jsclass = JS_GetClass(rootedObj);
+        auto jsclass = JS::GetClass(rootedObj);
 
         if (strcmp(jsclass->name, "Object") == 0 || strcmp(jsclass->name, "Array") == 0) {
             uassert(ErrorCodes::BadValue,
@@ -466,7 +504,7 @@ void EncryptedDBClientBase::encrypt(mozjs::MozJSImplScope* scope,
     ConstDataRange ciphertextBlob(encryptionFrame.get());
     std::string blobStr =
         base64::encode(StringData(ciphertextBlob.data(), ciphertextBlob.length()));
-    JS::AutoValueArray<2> arr(cx);
+    JS::RootedValueArray<2> arr(cx);
 
     arr[0].setInt32(BinDataType::Encrypt);
     mozjs::ValueReader(cx, arr[1]).fromStringData(blobStr);
@@ -503,6 +541,48 @@ void EncryptedDBClientBase::decrypt(mozjs::MozJSImplScope* scope,
     }
 }
 
+boost::optional<EncryptedFieldConfig> EncryptedDBClientBase::getEncryptedFieldConfig(
+    const NamespaceString& nss) {
+    auto collsList = _conn->getCollectionInfos(nss.db().toString(), BSON("name" << nss.coll()));
+    uassert(ErrorCodes::BadValue,
+            str::stream() << "Namespace not found: " << nss.toString(),
+            !collsList.empty());
+    auto info = collsList.front();
+    auto opts = info.getField("options");
+    if (opts.eoo() || !opts.isABSONObj()) {
+        return boost::none;
+    }
+    auto efc = opts.Obj().getField("encryptedFields");
+    if (efc.eoo() || !efc.isABSONObj()) {
+        return boost::none;
+    }
+    return EncryptedFieldConfig::parse(IDLParserContext("encryptedFields"), efc.Obj());
+}
+
+void EncryptedDBClientBase::compact(JSContext* cx, JS::CallArgs args) {
+    if (args.length() != 1) {
+        uasserted(ErrorCodes::BadValue, "compact requires 1 arg");
+    }
+    if (!args.get(0).isString()) {
+        uasserted(ErrorCodes::BadValue, "1st param to compact has to be a string");
+    }
+    std::string fullName = mozjs::ValueWriter(cx, args.get(0)).toString();
+    NamespaceString nss(fullName);
+    uassert(
+        ErrorCodes::BadValue, str::stream() << "Invalid namespace: " << fullName, nss.isValid());
+
+    auto efc = getEncryptedFieldConfig(nss);
+    BSONObjBuilder builder;
+    builder.append("compactStructuredEncryptionData", nss.coll());
+    builder.append("compactionTokens",
+                   efc ? FLEClientCrypto::generateCompactionTokens(*efc, this) : BSONObj());
+
+    BSONObj reply;
+    runCommand(nss.db().toString(), builder.obj(), reply, 0);
+    reply = reply.getOwned();
+    mozjs::ValueReader(cx, args.rval()).fromBSON(reply, nullptr, false);
+}
+
 void EncryptedDBClientBase::trace(JSTracer* trc) {
     JS::TraceEdge(trc, &_collection, "collection object");
 }
@@ -511,24 +591,10 @@ JS::Value EncryptedDBClientBase::getCollection() const {
     return _collection.get();
 }
 
-
-std::unique_ptr<DBClientCursor> EncryptedDBClientBase::query(
-    const NamespaceStringOrUUID& nsOrUuid,
-    Query query,
-    int nToReturn,
-    int nToSkip,
-    const BSONObj* fieldsToReturn,
-    int queryOptions,
-    int batchSize,
-    boost::optional<BSONObj> readConcernObj) {
-    return _conn->query(nsOrUuid,
-                        query,
-                        nToReturn,
-                        nToSkip,
-                        fieldsToReturn,
-                        queryOptions,
-                        batchSize,
-                        readConcernObj);
+std::unique_ptr<DBClientCursor> EncryptedDBClientBase::find(FindCommandRequest findRequest,
+                                                            const ReadPreferenceSetting& readPref,
+                                                            ExhaustMode exhaustMode) {
+    return _conn->find(std::move(findRequest), readPref, exhaustMode);
 }
 
 bool EncryptedDBClientBase::isFailed() const {
@@ -604,7 +670,7 @@ std::vector<uint8_t> EncryptedDBClientBase::getBinDataArg(
             str::stream() << "Incorrect bindata type, expected" << typeName(type) << " but got "
                           << typeName(binType),
             binType == type);
-    auto str = static_cast<std::string*>(JS_GetPrivate(args.get(index).toObjectOrNull()));
+    auto str = static_cast<std::string*>(JS::GetPrivate(args.get(index).toObjectOrNull()));
     uassert(ErrorCodes::BadValue, "Cannot call getter on BinData prototype", str);
     std::string string = base64::decode(*str);
     return std::vector<uint8_t>(string.data(), string.data() + string.length());
@@ -626,14 +692,22 @@ std::shared_ptr<SymmetricKey> EncryptedDBClientBase::getDataKey(const UUID& uuid
     return key;
 }
 
-std::shared_ptr<SymmetricKey> EncryptedDBClientBase::getDataKeyFromDisk(const UUID& uuid) {
+DBClientBase* EncryptedDBClientBase::getRawConnection() {
+    return _conn.get();
+}
+
+SecureVector<uint8_t> EncryptedDBClientBase::getKeyMaterialFromDisk(const UUID& uuid) {
     NamespaceString fullNameNS = getCollectionNS();
-    BSONObj dataKeyObj = _conn->findOne(fullNameNS.ns(), QUERY("_id" << uuid));
+    FindCommandRequest findCmd{fullNameNS};
+    findCmd.setFilter(BSON("_id" << uuid));
+    findCmd.setReadConcern(
+        repl::ReadConcernArgs(repl::ReadConcernLevel::kMajorityReadConcern).toBSONInner());
+    BSONObj dataKeyObj = _conn->findOne(std::move(findCmd));
     if (dataKeyObj.isEmpty()) {
         uasserted(ErrorCodes::BadValue, "Invalid keyID.");
     }
 
-    auto keyStoreRecord = KeyStoreRecord::parse(IDLParserErrorContext("root"), dataKeyObj);
+    auto keyStoreRecord = KeyStoreRecord::parse(IDLParserContext("root"), dataKeyObj);
     if (dataKeyObj.hasField("version"_sd)) {
         uassert(ErrorCodes::BadValue,
                 "Invalid version, must be either 0 or undefined",
@@ -653,10 +727,34 @@ std::shared_ptr<SymmetricKey> EncryptedDBClientBase::getDataKeyFromDisk(const UU
         _encryptionOptions.getKmsProviders().toBSON(), keyStoreRecord.getMasterKey());
     SecureVector<uint8_t> decryptedKey =
         kmsService->decrypt(dataKey, keyStoreRecord.getMasterKey());
+    return decryptedKey;
+}
+
+std::shared_ptr<SymmetricKey> EncryptedDBClientBase::getDataKeyFromDisk(const UUID& uuid) {
+    auto decryptedKey = getKeyMaterialFromDisk(uuid);
     return std::make_shared<SymmetricKey>(
         std::move(decryptedKey), crypto::aesAlgorithm, "kms_encryption");
 }
 
+KeyMaterial EncryptedDBClientBase::getKey(const UUID& uuid) {
+    auto decryptedKey = getKeyMaterialFromDisk(uuid);
+
+    KeyMaterial km;
+    km->resize(decryptedKey->size());
+    std::copy(decryptedKey->data(), decryptedKey->data() + decryptedKey->size(), km->data());
+    return km;
+}
+
+#ifdef MONGO_CONFIG_SSL
+const SSLConfiguration* EncryptedDBClientBase::getSSLConfiguration() {
+    return _conn->getSSLConfiguration();
+}
+
+bool EncryptedDBClientBase::isTLS() {
+    return _conn->isTLS();
+}
+#endif
+
 namespace {
 
 /**
@@ -679,7 +777,7 @@ void createCollectionObject(JSContext* cx,
 
     // The collection object requires a database object to be constructed as well.
     JS::RootedValue databaseRV(cx);
-    JS::AutoValueArray<2> databaseArgs(cx);
+    JS::RootedValueArray<2> databaseArgs(cx);
 
     databaseArgs[0].setObject(client.toObject());
     mozjs::ValueReader(cx, databaseArgs[1]).fromStringData(ns.db());
@@ -688,7 +786,7 @@ void createCollectionObject(JSContext* cx,
     invariant(databaseRV.isObject());
     auto databaseObj = databaseRV.toObjectOrNull();
 
-    JS::AutoValueArray<4> collectionArgs(cx);
+    JS::RootedValueArray<4> collectionArgs(cx);
     collectionArgs[0].setObject(client.toObject());
     collectionArgs[1].setObject(*databaseObj);
     mozjs::ValueReader(cx, collectionArgs[2]).fromStringData(ns.coll());
@@ -748,12 +846,12 @@ std::unique_ptr<DBClientBase> createEncryptedDBClientBase(std::unique_ptr<DBClie
                 arg.isObject());
 
         const BSONObj obj = mozjs::ValueWriter(cx, arg).toBSON();
-        encryptionOptions = encryptionOptions.parse(IDLParserErrorContext("root"), obj);
+        encryptionOptions = encryptionOptions.parse(IDLParserContext("root"), obj);
 
         // IDL does not perform a deep copy of BSONObjs when parsing, so we must get an
         // owned copy of the schemaMap.
         if (encryptionOptions.getSchemaMap()) {
-            encryptionOptions.setSchemaMap(encryptionOptions.getSchemaMap().get().getOwned());
+            encryptionOptions.setSchemaMap(encryptionOptions.getSchemaMap().value().getOwned());
         }
 
         // This logic tries to extract the client from the args. If the connection object is defined
@@ -778,9 +876,16 @@ std::unique_ptr<DBClientBase> createEncryptedDBClientBase(std::unique_ptr<DBClie
     return std::move(base);
 }
 
+DBClientBase* getNestedConnection(DBClientBase* conn) {
+    auto* encryptedConn = dynamic_cast<EncryptedDBClientBase*>(conn);
+    if (!encryptedConn) {
+        return nullptr;
+    }
+    return encryptedConn->getRawConnection();
+}
+
 MONGO_INITIALIZER(setCallbacksForEncryptedDBClientBase)(InitializerContext*) {
-    mongo::mozjs::setEncryptedDBClientCallback(createEncryptedDBClientBase);
-    return Status::OK();
+    mongo::mozjs::setEncryptedDBClientCallbacks(createEncryptedDBClientBase, getNestedConnection);
 }
 
 }  // namespace
