@@ -4,6 +4,8 @@ var MongoRunner, _startMongod, startMongoProgram, runMongoProgram, startMongoPro
 (function() {
 "use strict";
 
+const SIGTERM = 15;
+
 var shellVersion = version;
 
 // Record the exit codes of mongod and mongos processes that crashed during startup keyed by
@@ -70,14 +72,42 @@ var createMongoArgs = function(binaryName, args) {
     return fullArgs;
 };
 
+// A path.join-like thing for paths that must work
+// on Windows (\-separated) and *nix (/-separated).
+function pathJoin(...parts) {
+    const separator = _isWindows() ? '\\' : '/';
+    return parts.join(separator);
+}
+
 MongoRunner = function() {};
 
 MongoRunner.dataDir = "/data/db";
 MongoRunner.dataPath = "/data/db/";
 
-MongoRunner.mongodPath = "mongod";
-MongoRunner.mongosPath = "mongos";
-MongoRunner.mongoShellPath = "mongo";
+function getMongoSuffixPath(binary_name) {
+    let installDir = _getEnv("INSTALL_DIR");
+    if (installDir && !jsTestOptions().inEvergreen) {
+        return pathJoin(installDir, binary_name);
+    }
+    return binary_name;
+}
+
+MongoRunner.getMongodPath = function() {
+    return getMongoSuffixPath("mongod");
+};
+
+MongoRunner.getMongosPath = function() {
+    return getMongoSuffixPath("mongos");
+};
+
+MongoRunner.getMongoqPath = function() {
+    return getMongoSuffixPath("mongoqd");
+};
+
+MongoRunner.getMongoShellPath = function() {
+    let shellPath = getMongoSuffixPath("mongo");
+    return shellPath;
+};
 
 MongoRunner.VersionSub = function(pattern, version) {
     this.pattern = pattern;
@@ -96,13 +126,6 @@ function getPids() {
     return pids;
 }
 
-// A path.join-like thing for paths that must work
-// on Windows (\-separated) and *nix (/-separated).
-function pathJoin(...parts) {
-    const separator = _isWindows() ? '\\' : '/';
-    return parts.join(separator);
-}
-
 // Internal state to determine if the hang analyzer should be enabled or not.
 // Accessible via global setter/getter defined below.
 let _hangAnalyzerEnabled = true;
@@ -146,7 +169,12 @@ function runHangAnalyzer(pids) {
     print(`Running hang analyzer for pids [${pids}]`);
 
     const scriptPath = pathJoin('.', 'buildscripts', 'resmoke.py');
-    const args = ['python', scriptPath, 'hang-analyzer', '-d', pids];
+    const args =
+        ['python', scriptPath, 'hang-analyzer', '-k', '-o', 'file', '-o', 'stdout', '-d', pids];
+
+    if (jsTest.options().evergreenTaskId) {
+        args.push('-t', jsTest.options().evergreenTaskId);
+    }
 
     // Enable core dumps if not an ASAN build.
     if (!_isAddressSanitizerActive()) {
@@ -185,6 +213,14 @@ var convertVersionStringToArray = function(versionString) {
     return versionArray;
 };
 
+/**
+ * Returns an integer
+ */
+var convertVersionStringToInteger = function(versionString) {
+    const [major, minor, point] = _convertVersionToIntegerArray(versionString);
+    return major * 100 + minor + 10;
+};
+
 /**
  * Returns the major version string from a version string.
  *
@@ -197,17 +233,13 @@ var extractMajorVersionFromVersionString = function(versionString) {
 };
 
 // These patterns allow substituting the binary versions used for each version string to support
-// the
-// dev/stable MongoDB release cycle.
-//
-// If you add a new version substitution to this list, you should add it to the lists of
-// versions being checked in 'verify_versions_test.js' to verify it is susbstituted correctly.
+// the dev/stable MongoDB release cycle.
+var fcvConstants = getFCVConstants();
+
 MongoRunner.binVersionSubs = [
     new MongoRunner.VersionSub("latest", shellVersion()),
-    new MongoRunner.VersionSub(extractMajorVersionFromVersionString(shellVersion()),
-                               shellVersion()),
-    // To-be-updated when we branch for the next release.
-    new MongoRunner.VersionSub("last-stable", "4.2")
+    new MongoRunner.VersionSub("last-continuous", fcvConstants.lastContinuous),
+    new MongoRunner.VersionSub("last-lts", fcvConstants.lastLTS)
 ];
 
 MongoRunner.getBinVersionFor = function(version) {
@@ -243,7 +275,6 @@ MongoRunner.areBinVersionsTheSame = function(versionA, versionB) {
     // Check for invalid version strings first.
     convertVersionStringToArray(MongoRunner.getBinVersionFor(versionA));
     convertVersionStringToArray(MongoRunner.getBinVersionFor(versionB));
-
     try {
         return (0 === MongoRunner.compareBinVersions(versionA, versionB));
     } catch (err) {
@@ -286,7 +317,8 @@ MongoRunner.compareBinVersions = function(versionA, versionB) {
         var numA = parseInt(elementA);
         var numB = parseInt(elementB);
 
-        assert(!isNaN(numA) && !isNaN(numB), "Cannot compare non-equal non-numeric versions.");
+        assert(!isNaN(numA) && !isNaN(numB),
+               `Cannot compare non-equal non-numeric versions. ${elementA}, ${elementB}`);
 
         if (numA > numB) {
             return 1;
@@ -319,11 +351,12 @@ MongoRunner.logicalOptions = {
     noReplSet: true,
     forgetPort: true,
     arbiter: true,
-    noJournal: true,
     binVersion: true,
     waitForConnect: true,
     bridgeOptions: true,
     skipValidation: true,
+    backupOnRestartDir: true,
+    allowedExitCode: true,
 };
 
 MongoRunner.toRealPath = function(path, pathOpts) {
@@ -629,14 +662,17 @@ var _isMongodVersionEqualOrAfter = function(version1, version2) {
     return false;
 };
 
-// Removes a setParameter parameter from mongods running a version that won't recognize them.
-var _removeSetParameterIfBeforeVersion = function(opts, parameterName, requiredVersion) {
+// Removes a setParameter parameter from mongods or mongoses running a version that won't recognize
+// them.
+var _removeSetParameterIfBeforeVersion = function(
+    opts, parameterName, requiredVersion, isMongos = false) {
+    var processString = isMongos ? "mongos" : "mongod";
     var versionCompatible = (opts.binVersion === "" || opts.binVersion === undefined ||
                              _isMongodVersionEqualOrAfter(requiredVersion, opts.binVersion));
     if (!versionCompatible && opts.setParameter && opts.setParameter[parameterName] != undefined) {
         print("Removing '" + parameterName + "' setParameter with value " +
-              opts.setParameter[parameterName] +
-              " because it isn't compatibile with mongod running version " + opts.binVersion);
+              opts.setParameter[parameterName] + " because it isn't compatible with " +
+              processString + " running version " + opts.binVersion);
         delete opts.setParameter[parameterName];
     }
 };
@@ -649,7 +685,6 @@ var _removeSetParameterIfBeforeVersion = function(opts, parameterName, requiredV
  *     useLogFiles {boolean}: use with logFile option.
  *     logFile {string}: path to the log file. If not specified and useLogFiles
  *       is true, automatically creates a log file inside dbpath.
- *     noJournal {boolean}
  *     keyFile
  *     replSet
  *     oplogSize
@@ -679,7 +714,14 @@ MongoRunner.mongodOptions = function(opts = {}) {
     _removeSetParameterIfBeforeVersion(opts, "numInitialSyncAttempts", "3.3.12");
     _removeSetParameterIfBeforeVersion(opts, "numInitialSyncConnectAttempts", "3.3.12");
     _removeSetParameterIfBeforeVersion(opts, "migrationLockAcquisitionMaxWaitMS", "4.1.7");
-    _removeSetParameterIfBeforeVersion(opts, "enableReconfigRollbackCommittedWritesCheck", "4.4.7");
+    _removeSetParameterIfBeforeVersion(opts, "shutdownTimeoutMillisForSignaledShutdown", "4.5.0");
+    _removeSetParameterIfBeforeVersion(
+        opts, "failpoint.PrimaryOnlyServiceSkipRebuildingInstances", "4.8.0");
+    _removeSetParameterIfBeforeVersion(
+        opts, "enableDefaultWriteConcernUpdatesForInitiate", "5.0.0");
+    _removeSetParameterIfBeforeVersion(opts, "enableReconfigRollbackCommittedWritesCheck", "5.0.0");
+    _removeSetParameterIfBeforeVersion(opts, "featureFlagRetryableFindAndModify", "5.0.0");
+    _removeSetParameterIfBeforeVersion(opts, "allowMultipleArbiters", "5.3.0");
 
     if (!opts.logFile && opts.useLogFiles) {
         opts.logFile = opts.dbpath + "/mongod.log";
@@ -691,11 +733,6 @@ MongoRunner.mongodOptions = function(opts = {}) {
         opts.logpath = opts.logFile;
     }
 
-    if ((jsTestOptions().noJournal || opts.noJournal) && !('journal' in opts) &&
-        !('configsvr' in opts)) {
-        opts.nojournal = "";
-    }
-
     if (jsTestOptions().keyFile && !opts.keyFile) {
         opts.keyFile = jsTestOptions().keyFile;
     }
@@ -714,6 +751,18 @@ MongoRunner.mongodOptions = function(opts = {}) {
         opts.enableEncryption = "";
     }
 
+    if (opts.hasOwnProperty("encryptionCipherMode")) {
+        if (typeof opts.encryptionCipherMode !== "string") {
+            // opts.encryptionCipherMode, if set, must be a string
+            throw new Error("The encryptionCipherMode option must be a string if it is specified");
+        }
+    } else if (jsTestOptions().encryptionCipherMode !== undefined) {
+        if (typeof jsTestOptions().encryptionCipherMode !== "string") {
+            throw new Error("The encryptionCipherMode option must be a string if it is specified");
+        }
+        opts.encryptionCipherMode = jsTestOptions().encryptionCipherMode;
+    }
+
     if (opts.hasOwnProperty("encryptionKeyFile")) {
         // opts.encryptionKeyFile, if set, must be a string
         if (typeof opts.encryptionKeyFile !== "string") {
@@ -738,6 +787,11 @@ MongoRunner.mongodOptions = function(opts = {}) {
         opts.auditDestination = jsTestOptions().auditDestination;
     }
 
+    if (opts.hasOwnProperty("auditPath")) {
+        // We need to reformat the auditPath to include the proper port
+        opts.auditPath = MongoRunner.toRealPath(opts.auditPath, opts);
+    }
+
     if (opts.noReplSet)
         opts.replSet = null;
     if (opts.arbiter)
@@ -796,6 +850,14 @@ MongoRunner.mongosOptions = function(opts) {
         opts.binVersion = MongoRunner.getBinVersionFor(testOptions.mongosBinVersion);
     }
 
+    if (opts.hasOwnProperty("auditPath")) {
+        // We need to reformat the auditPath to include the proper port
+        opts.auditPath = MongoRunner.toRealPath(opts.auditPath, opts);
+    }
+
+    _removeSetParameterIfBeforeVersion(
+        opts, "mongosShutdownTimeoutMillisForSignaledShutdown", "4.5.0", true);
+
     // If the mongos is being restarted with a newer version, make sure we remove any options
     // that no longer exist in the newer version.
     if (opts.restart && MongoRunner.areBinVersionsTheSame('latest', opts.binVersion)) {
@@ -805,6 +867,63 @@ MongoRunner.mongosOptions = function(opts) {
     return opts;
 };
 
+MongoRunner.mongoqOptions = function(opts) {
+    opts = MongoRunner.mongoOptions(opts);
+
+    // Normalize configdb option to be host string if currently a host
+    if (opts.configdb && opts.configdb.getDB) {
+        opts.configdb = opts.configdb.host;
+    }
+
+    if (jsTestOptions().alwaysUseLogFiles) {
+        if (opts.useLogFiles === false) {
+            throw new Error("Always using log files, but received conflicting option.");
+        }
+
+        opts.useLogFiles = true;
+        opts.logappend = "";
+    }
+
+    opts.pathOpts = Object.merge(opts.pathOpts, {configdb: opts.configdb.replace(/:|\/|,/g, "-")});
+
+    if (!opts.logFile && opts.useLogFiles) {
+        opts.logFile = MongoRunner.toRealFile("$dataDir/mongoq-$configdb-$port.log", opts.pathOpts);
+    } else if (opts.logFile) {
+        opts.logFile = MongoRunner.toRealFile(opts.logFile, opts.pathOpts);
+    }
+
+    if (opts.logFile !== undefined) {
+        opts.logpath = opts.logFile;
+    }
+
+    var testOptions = jsTestOptions();
+    if (testOptions.keyFile && !opts.keyFile) {
+        opts.keyFile = testOptions.keyFile;
+    }
+
+    if (opts.hasOwnProperty("auditDestination")) {
+        // opts.auditDestination, if set, must be a string
+        if (typeof opts.auditDestination !== "string") {
+            throw new Error("The auditDestination option must be a string if it is specified");
+        }
+    } else if (testOptions.auditDestination !== undefined) {
+        if (typeof (testOptions.auditDestination) !== "string") {
+            throw new Error("The auditDestination option must be a string if it is specified");
+        }
+        opts.auditDestination = testOptions.auditDestination;
+    }
+
+    if (!opts.hasOwnProperty('binVersion') && testOptions.mongoqBinVersion) {
+        opts.binVersion = MongoRunner.getBinVersionFor(testOptions.mongoqBinVersion);
+    }
+
+    if (opts.hasOwnProperty("auditPath")) {
+        // We need to reformat the auditPath to include the proper port
+        opts.auditPath = MongoRunner.toRealPath(opts.auditPath, opts);
+    }
+    return opts;
+};
+
 /**
  * @return {NumberLong[]} Running pids e.g. those started by `MongoRunner.runMongod`.
  */
@@ -856,14 +975,29 @@ MongoRunner.runMongod = function(opts) {
         runId = opts.runId;
         waitForConnect = opts.waitForConnect;
 
+        let backupOnRestartDir = jsTest.options()["backupOnRestartDir"] || false;
+
         if (opts.forceLock)
             removeFile(opts.dbpath + "/mongod.lock");
-        if ((opts.cleanData || opts.startClean) || (!opts.restart && !opts.noCleanData)) {
+        if ((opts.cleanData || opts.startClean) || (!opts.restart && !opts.noCleanData) ||
+            !pathExists(opts.dbpath)) {
             print("Resetting db path '" + opts.dbpath + "'");
             resetDbpath(opts.dbpath);
+        } else {
+            if (backupOnRestartDir) {
+                let pathOpts = {"backupDir": backupOnRestartDir, "dbpath": opts.dbpath};
+                let backupDir = MongoRunner.toRealDir("$backupDir/$dbpath", pathOpts);
+                // `toRealDir` assumes the patterned directory should be under
+                // `MongoRunner.dataPath`. In this case, preserve the user input as is.
+                backupDir = backupDir.substring(MongoRunner.dataPath.length);
+
+                print("Backing up data files. DBPath: " + opts.dbpath +
+                      " Backing up under: " + backupDir);
+                copyDbpath(opts.dbpath, backupDir);
+            }
         }
 
-        var mongodProgram = MongoRunner.mongodPath;
+        var mongodProgram = MongoRunner.getMongodPath();
         opts = MongoRunner.arrOptions(mongodProgram, opts);
     }
 
@@ -901,7 +1035,7 @@ MongoRunner.runMongos = function(opts) {
         runId = opts.runId;
         waitForConnect = opts.waitForConnect;
         env = opts.env;
-        var mongosProgram = MongoRunner.mongosPath;
+        var mongosProgram = MongoRunner.getMongosPath();
         opts = MongoRunner.arrOptions(mongosProgram, opts);
     }
 
@@ -921,6 +1055,43 @@ MongoRunner.runMongos = function(opts) {
     return mongos;
 };
 
+MongoRunner.runMongoq = function(opts) {
+    opts = opts || {};
+
+    var env = undefined;
+    var useHostName = false;
+    var runId = null;
+    var waitForConnect = true;
+    var fullOptions = opts;
+
+    if (isObject(opts)) {
+        opts = MongoRunner.mongoqOptions(opts);
+        fullOptions = opts;
+
+        useHostName = opts.useHostName || opts.useHostname;
+        runId = opts.runId;
+        waitForConnect = opts.waitForConnect;
+        env = opts.env;
+        var mongoqProgram = MongoRunner.getMongoqPath();
+        opts = MongoRunner.arrOptions(mongoqProgram, opts);
+    }
+
+    var mongoq = MongoRunner._startWithArgs(opts, env, waitForConnect);
+    if (!mongoq) {
+        return null;
+    }
+
+    mongoq.commandLine = MongoRunner.arrToOpts(opts);
+    mongoq.name = (useHostName ? getHostName() : "localhost") + ":" + mongoq.commandLine.port;
+    mongoq.host = mongoq.name;
+    mongoq.port = parseInt(mongoq.commandLine.port);
+    mongoq.runId = runId || ObjectId();
+    mongoq.savedOptions = MongoRunner.savedOptions[mongoq.runId];
+    mongoq.fullOptions = fullOptions;
+
+    return mongoq;
+};
+
 MongoRunner.StopError = function(returnCode) {
     this.name = "StopError";
     this.returnCode = returnCode;
@@ -933,7 +1104,7 @@ MongoRunner.StopError.prototype.constructor = MongoRunner.StopError;
 
 // Constants for exit codes of MongoDB processes
 // On Windows, std::abort causes the process to exit with return code 14.
-MongoRunner.EXIT_ABORT = _isWindows() ? 14 : -6;
+MongoRunner.EXIT_ABORT = _isWindows() ? 14 : 6;
 MongoRunner.EXIT_CLEAN = 0;
 MongoRunner.EXIT_BADOPTIONS = 2;
 MongoRunner.EXIT_REPLICATION_ERROR = 3;
@@ -941,7 +1112,7 @@ MongoRunner.EXIT_NEED_UPGRADE = 4;
 MongoRunner.EXIT_SHARDING_ERROR = 5;
 // SIGKILL is translated to TerminateProcess() on Windows, which causes the program to
 // terminate with exit code 1.
-MongoRunner.EXIT_SIGKILL = _isWindows() ? 1 : -9;
+MongoRunner.EXIT_SIGKILL = _isWindows() ? 1 : 9;
 MongoRunner.EXIT_KILL = 12;
 MongoRunner.EXIT_ABRUPT = 14;
 MongoRunner.EXIT_NTSERVICE_ERROR = 20;
@@ -956,6 +1127,7 @@ MongoRunner.EXIT_POSSIBLE_CORRUPTION = 60;
 MongoRunner.EXIT_NEED_DOWNGRADE = 62;
 MongoRunner.EXIT_UNCAUGHT = 100;  // top level exception that wasn't caught
 MongoRunner.EXIT_TEST = 101;
+MongoRunner.EXIT_AUDIT_ROTATE_ERROR = 102;
 
 MongoRunner.validateCollectionsCallback = function(port) {};
 
@@ -971,6 +1143,7 @@ MongoRunner.validateCollectionsCallback = function(port) {};
  *        pwd {string}: admin password
  *      },
  *      skipValidation: <bool>,
+ *      skipValidatingExitCode: <bool>,
  *      allowedExitCode: <int>
  *    }
  * @param {boolean} waitpid should we wait for the process to terminate after stopping it.
@@ -978,7 +1151,7 @@ MongoRunner.validateCollectionsCallback = function(port) {};
  * Note: The auth option is required in a authenticated mongod running in Windows since
  *  it uses the shutdown command, which requires admin credentials.
  */
-MongoRunner.stopMongod = function(conn, signal, opts, waitpid) {
+var stopMongoProgram = function(conn, signal, opts, waitpid) {
     if (!conn.pid) {
         throw new Error("first arg must have a `pid` property; " +
                         "it is usually the object returned from MongoRunner.runMongod/s");
@@ -989,14 +1162,19 @@ MongoRunner.stopMongod = function(conn, signal, opts, waitpid) {
                         "it is usually the object returned from MongoRunner.runMongod/s");
     }
 
-    signal = parseInt(signal) || 15;
+    signal = parseInt(signal) || SIGTERM;
     opts = opts || {};
     waitpid = (waitpid === undefined) ? true : waitpid;
 
-    var allowedExitCode = MongoRunner.EXIT_CLEAN;
+    // If we are executing an unclean shutdown, we want to avoid checking collection counts during
+    // validation, since the counts may be inaccurate.
+    if (signal !== SIGTERM && typeof TestData !== 'undefined') {
+        TestData.skipEnforceFastCountOnValidate = true;
+    }
 
-    if (opts.allowedExitCode) {
-        allowedExitCode = opts.allowedExitCode;
+    const allowedExitCode = opts.allowedExitCode ? opts.allowedExitCode : MongoRunner.EXIT_CLEAN;
+    if (!waitpid && allowedExitCode !== MongoRunner.EXIT_CLEAN) {
+        throw new Error('Must wait for process to exit if it is expected to exit uncleanly');
     }
 
     var port = parseInt(conn.port);
@@ -1027,7 +1205,7 @@ MongoRunner.stopMongod = function(conn, signal, opts, waitpid) {
 
     // If we are not waiting for shutdown, then there is no exit code to check.
     if (!waitpid) {
-        return 0;
+        returnCode = 0;
     }
     if (allowedExitCode !== returnCode && !opts.skipValidatingExitCode) {
         throw new MongoRunner.StopError(returnCode);
@@ -1036,10 +1214,16 @@ MongoRunner.stopMongod = function(conn, signal, opts, waitpid) {
               returnCode);
     }
 
+    if (conn.undoLiveRecordPid) {
+        _stopUndoLiveRecord(conn.undoLiveRecordPid);
+    }
+
     return returnCode;
 };
 
-MongoRunner.stopMongos = MongoRunner.stopMongod;
+MongoRunner.stopMongod = stopMongoProgram;
+MongoRunner.stopMongos = stopMongoProgram;
+MongoRunner.stopMongoq = stopMongoProgram;
 
 // Given a test name figures out a directory for that test to use for dump files and makes sure
 // that directory exists and is empty.
@@ -1096,10 +1280,17 @@ function appendSetParameterArgs(argArray) {
     }
 
     let [baseProgramName, programVersion] = programName.split("-");
-    let programMajorMinorVersion = 0;
+
+    // Setting programMajorMinorVersion to the maximum value for the latest binary version
+    // simplifies version checks below.
+    const lastestMajorMinorVersion = Number.MAX_SAFE_INTEGER;
+    const lastContinuousVersion =
+        convertVersionStringToInteger(MongoRunner.getBinVersionFor("last-continuous"));
+    const lastLTSVersion = convertVersionStringToInteger(MongoRunner.getBinVersionFor("last-lts"));
+    let programMajorMinorVersion = lastestMajorMinorVersion;
+
     if (programVersion) {
-        let [major, minor, point] = programVersion.split(".");
-        programMajorMinorVersion = parseInt(major) * 100 + parseInt(minor);
+        programMajorMinorVersion = convertVersionStringToInteger(programVersion);
     }
 
     if (baseProgramName === 'mongod' || baseProgramName === 'mongos') {
@@ -1107,13 +1298,13 @@ function appendSetParameterArgs(argArray) {
             argArray.push(...['--setParameter', "enableTestCommands=1"]);
         }
 
-        if (!programMajorMinorVersion || programMajorMinorVersion >= 404) {
+        if (programMajorMinorVersion >= 440) {
             if (jsTest.options().testingDiagnosticsEnabled) {
                 argArray.push(...['--setParameter', "testingDiagnosticsEnabled=1"]);
             }
         }
 
-        if (jsTest.options().authMechanism && jsTest.options().authMechanism != "SCRAM-SHA-1") {
+        if (jsTest.options().authMechanism && jsTest.options().authMechanism != "SCRAM-SHA-256") {
             if (!argArrayContainsSetParameterValue('authenticationMechanisms=')) {
                 argArray.push(...['--setParameter',
                                   "authenticationMechanisms=" + jsTest.options().authMechanism]);
@@ -1124,13 +1315,7 @@ function appendSetParameterArgs(argArray) {
         }
 
         // New options in 3.5.x
-        if (!programMajorMinorVersion || programMajorMinorVersion >= 305) {
-            if (jsTest.options().serviceExecutor) {
-                if (!argArrayContains("--serviceExecutor")) {
-                    argArray.push(...["--serviceExecutor", jsTest.options().serviceExecutor]);
-                }
-            }
-
+        if (programMajorMinorVersion >= 350) {
             if (jsTest.options().transportLayer) {
                 if (!argArrayContains("--transportLayer")) {
                     argArray.push(...["--transportLayer", jsTest.options().transportLayer]);
@@ -1145,7 +1330,7 @@ function appendSetParameterArgs(argArray) {
 
         // Since options may not be backward compatible, mongos options are not
         // set on older versions, e.g., mongos-3.0.
-        if (programName.endsWith('mongos')) {
+        if (baseProgramName === 'mongos' && programMajorMinorVersion == lastestMajorMinorVersion) {
             // apply setParameters for mongos
             if (jsTest.options().setParametersMongos) {
                 let params = jsTest.options().setParametersMongos;
@@ -1176,16 +1361,114 @@ function appendSetParameterArgs(argArray) {
             }
 
             // Set storageEngine for mongod. There was no storageEngine parameter before 3.0.
-            if (jsTest.options().storageEngine &&
-                (!programVersion || programMajorMinorVersion >= 300)) {
+            if (jsTest.options().storageEngine && (programMajorMinorVersion >= 300)) {
                 if (!argArrayContains("--storageEngine")) {
                     argArray.push(...['--storageEngine', jsTest.options().storageEngine]);
                 }
             }
 
-            // New mongod-specific option in 4.3.x.
-            if (!programMajorMinorVersion || programMajorMinorVersion >= 430) {
+            function isSetParameterMentioned(setParameters, key) {
+                if (setParameters !== undefined && setParameters[key] !== undefined) {
+                    return true;
+                }
+
+                if (argArrayContainsSetParameterValue(key + '=')) {
+                    return true;
+                }
+
+                return false;
+            }
+
+            // The 'logComponentVerbosity' parameter must be passed in on the last continuous
+            // version and last LTS version as well.
+            if ((programMajorMinorVersion === lastContinuousVersion ||
+                 programMajorMinorVersion === lastLTSVersion) &&
+                isSetParameterMentioned(jsTest.options().setParameters, "logComponentVerbosity") &&
+                !argArrayContainsSetParameterValue("logComponentVerbosity=")) {
+                let logVerbosityParam = jsTest.options().setParameters["logComponentVerbosity"];
+                if (typeof logVerbosityParam === "object") {
+                    logVerbosityParam = JSON.stringify(logVerbosityParam);
+                }
+                argArray.push(...['--setParameter', "logComponentVerbosity=" + logVerbosityParam]);
+            }
+
+            if (programMajorMinorVersion >= 530 &&
+                !argArrayContainsSetParameterValue("backtraceLogFile=")) {
+                let randomName = "";
+                let randomStrLen = 20;
+                const chars = "qwertyuiopasdfghjklzxcvbnm1234567890";
+                for (let i = 0; i <= randomStrLen; i++) {
+                    randomName += chars[((Math.random() * 1000) % chars.length) ^ 0];
+                }
+                const backtraceLogFilePath =
+                    MongoRunner.dataDir + "/" + randomName + Date.now() + ".stacktrace";
+                argArray.push(...["--setParameter", "backtraceLogFile=" + backtraceLogFilePath]);
+            }
+
+            // When launching a 5.0 mongod, if we're mentioning the
+            // `storeFindAndModifyImagesInSideCollection` setParameter and the corresponding feature
+            // flag is not set, add it for good measure.
+            if (programMajorMinorVersion === 500 &&
+                isSetParameterMentioned(jsTest.options().setParameters,
+                                        "storeFindAndModifyImagesInSideCollection") &&
+                !argArrayContainsSetParameterValue("featureFlagRetryableFindAndModify=")) {
+                argArray.push(...['--setParameter', "featureFlagRetryableFindAndModify=true"]);
+            }
+
+            // New mongod-specific option in 4.9.x.
+            if (programMajorMinorVersion >= 490) {
+                const parameters = jsTest.options().setParameters;
+                if ((parameters === undefined ||
+                     parameters['reshardingMinimumOperationDurationMillis'] === undefined) &&
+                    !argArrayContainsSetParameterValue(
+                        'reshardingMinimumOperationDurationMillis=')) {
+                    argArray.push(
+                        ...['--setParameter', "reshardingMinimumOperationDurationMillis=5000"]);
+                }
+            }
+
+            // New mongod-specific option in 4.5.x.
+            if (programMajorMinorVersion >= 450) {
                 // Allow the parameter to be overridden if set explicitly via TestData.
+                const parameters = jsTest.options().setParameters;
+
+                if ((parameters === undefined ||
+                     parameters['coordinateCommitReturnImmediatelyAfterPersistingDecision'] ===
+                         undefined) &&
+                    !argArrayContainsSetParameterValue(
+                        'coordinateCommitReturnImmediatelyAfterPersistingDecision=')) {
+                    argArray.push(
+                        ...['--setParameter',
+                            "coordinateCommitReturnImmediatelyAfterPersistingDecision=false"]);
+                }
+            }
+
+            // New mongod-specific option in 4.4.
+            if (programMajorMinorVersion >= 440) {
+                if (jsTest.options().setParameters &&
+                    jsTest.options().setParameters['enableIndexBuildCommitQuorum'] !== undefined) {
+                    if (!argArrayContainsSetParameterValue('enableIndexBuildCommitQuorum=')) {
+                        argArray.push(...['--setParameter',
+                                          "enableIndexBuildCommitQuorum=" +
+                                              jsTest.options()
+                                                  .setParameters['enableIndexBuildCommitQuorum']]);
+                    }
+                }
+            }
+
+            // New mongod-specific option in 4.5.
+            if (programMajorMinorVersion >= 450) {
+                // Allow the parameter to be overridden if set explicitly via TestData.
+                if ((jsTest.options().setParameters === undefined ||
+                     jsTest.options()
+                             .setParameters['oplogApplicationEnforcesSteadyStateConstraints'] ===
+                         undefined) &&
+                    !argArrayContainsSetParameterValue(
+                        'oplogApplicationEnforcesSteadyStateConstraints=')) {
+                    argArray.push(...['--setParameter',
+                                      'oplogApplicationEnforcesSteadyStateConstraints=true']);
+                }
+
                 if ((jsTest.options().setParameters === undefined ||
                      jsTest.options().setParameters['minNumChunksForSessionsCollection'] ===
                          undefined) &&
@@ -1195,7 +1478,7 @@ function appendSetParameterArgs(argArray) {
             }
 
             // New mongod-specific options in 4.0.x
-            if (!programMajorMinorVersion || programMajorMinorVersion >= 400) {
+            if (programMajorMinorVersion >= 400) {
                 if (jsTest.options().transactionLifetimeLimitSeconds !== undefined) {
                     if (!argArrayContainsSetParameterValue("transactionLifetimeLimitSeconds=")) {
                         argArray.push(...["--setParameter",
@@ -1206,26 +1489,24 @@ function appendSetParameterArgs(argArray) {
             }
 
             // TODO: Make this unconditional in 3.8.
-            if (!programMajorMinorVersion || programMajorMinorVersion > 304) {
+            if (programMajorMinorVersion > 340) {
                 if (!argArrayContainsSetParameterValue('orphanCleanupDelaySecs=')) {
                     argArray.push(...['--setParameter', 'orphanCleanupDelaySecs=1']);
                 }
             }
 
-            if (!programMajorMinorVersion || programMajorMinorVersion >= 306) {
-                if (jsTest.options().storageEngine === "wiredTiger" ||
-                    !jsTest.options().storageEngine) {
-                    if (jsTest.options().enableMajorityReadConcern !== undefined &&
-                        !argArrayContains("--enableMajorityReadConcern")) {
-                        argArray.push(...['--enableMajorityReadConcern',
-                                          jsTest.options().enableMajorityReadConcern.toString()]);
-                    }
+            if (programMajorMinorVersion >= 360) {
+                if (jsTest.options().enableMajorityReadConcern !== undefined &&
+                    !argArrayContains("--enableMajorityReadConcern")) {
+                    argArray.push(...['--enableMajorityReadConcern',
+                                      jsTest.options().enableMajorityReadConcern.toString()]);
                 }
             }
 
             // Since options may not be backward compatible, mongod options are not
             // set on older versions, e.g., mongod-3.0.
-            if (programName.endsWith('mongod')) {
+            if (baseProgramName === 'mongod' &&
+                programMajorMinorVersion == lastestMajorMinorVersion) {
                 if (jsTest.options().storageEngine === "wiredTiger" ||
                     !jsTest.options().storageEngine) {
                     if (jsTest.options().storageEngineCacheSizeGB &&
@@ -1276,24 +1557,13 @@ function appendSetParameterArgs(argArray) {
                             continue;
                         }
 
-                        // Only set the 'enableTwoPhaseIndexBuild' parameter if it has not already
-                        // been set, so that tests will not try to set this parameter more than
-                        // once.
-                        if (paramName === "enableTwoPhaseIndexBuild" &&
-                            argArrayContains("enableTwoPhaseIndexBuild")) {
+                        if (paramName === 'enableIndexBuildCommitQuorum' &&
+                            argArrayContains("enableIndexBuildCommitQuorum")) {
                             continue;
                         }
 
-                        // Do not set this if test commands are not enabled or we are not on a new
-                        // enough version.
-                        if (paramName === "assertStableTimestampEqualsAppliedThroughOnRecovery" &&
-                            (!jsTest.options().enableTestCommands ||
-                             programMajorMinorVersion < 404)) {
-                            continue;
-                        }
-
-                        if (paramName === "storeFindAndModifyImagesInSideCollection" &&
-                            argArrayContains("storeFindAndModifyImagesInSideCollection")) {
+                        if (paramName === "reshardingMinimumOperationDurationMillis" &&
+                            argArrayContains("reshardingMinimumOperationDurationMillis")) {
                             continue;
                         }
 
@@ -1318,20 +1588,23 @@ function appendSetParameterArgs(argArray) {
 /**
  * Continuously tries to establish a connection to the server on the specified port.
  *
- * If a connection cannot be established within a time limit, an exception will be thrown. If the
- * process for the given 'pid' is found to no longer be running, this function will terminate and
- * return null.
+ * If a connection cannot be established within a time limit, or if the process terminated
+ * with a non-zero exit code, an exception will be thrown. If the process for the given
+ * 'pid' is found to have gracefully terminated, this function will terminate and return
+ * null.
  *
  * @param {int} [pid] the process id of the node to connect to.
  * @param {int} [port] the port of the node to connect to.
- * @returns a new Mongo connection object, or null if the process is not running.
+ * @param {int} [undoLiveRecordPid=null] the process id of the `live-record` process.
+ * @returns a new Mongo connection object, or null if the process gracefully terminated.
  */
-MongoRunner.awaitConnection = function(pid, port) {
+MongoRunner.awaitConnection = function({pid, port, undoLiveRecordPid = null} = {}) {
     var conn = null;
     assert.soon(function() {
         try {
             conn = new Mongo("127.0.0.1:" + port);
             conn.pid = pid;
+            conn.undoLiveRecordPid = undoLiveRecordPid;
             return true;
         } catch (e) {
             var res = checkProgram(pid);
@@ -1339,6 +1612,12 @@ MongoRunner.awaitConnection = function(pid, port) {
                 print("mongo program was not running at " + port +
                       ", process ended with exit code: " + res.exitCode);
                 serverExitCodeMap[port] = res.exitCode;
+                if (undoLiveRecordPid) {
+                    _stopUndoLiveRecord(undoLiveRecordPid);
+                }
+                if (res.exitCode !== MongoRunner.EXIT_CLEAN) {
+                    throw new MongoRunner.StopError(res.exitCode);
+                }
                 return true;
             }
         }
@@ -1347,6 +1626,21 @@ MongoRunner.awaitConnection = function(pid, port) {
     return conn;
 };
 
+var _runUndoLiveRecord = function(pid) {
+    var argArray = [jsTestOptions().undoRecorderPath, "-p", pid];
+    return _startMongoProgram.apply(null, argArray);
+};
+
+var _stopUndoLiveRecord = function(undoLiveRecordPid) {
+    print("Saving the UndoDB recording; it may take a few minutes...");
+    var undoReturnCode = waitProgram(undoLiveRecordPid);
+    if (undoReturnCode !== 0) {
+        throw new Error(
+            "Undo live-record failed to terminate correctly. This is likely a bug in Undo. " +
+            "Please record any logs and send them to the #server-testing Slack channel");
+    }
+};
+
 /**
  * Start a mongo process with a particular argument array.
  * If we aren't waiting for connect, return {pid: <pid>}.
@@ -1366,16 +1660,22 @@ MongoRunner._startWithArgs = function(argArray, env, waitForConnect) {
         pid = _startMongoProgram({args: argArray, env: env});
     }
 
+    let undoLiveRecordPid = null;
+    if (jsTestOptions().undoRecorderPath) {
+        undoLiveRecordPid = _runUndoLiveRecord(pid);
+    }
+
     delete serverExitCodeMap[port];
     if (!waitForConnect) {
         print("Skip waiting to connect to node with pid=" + pid + ", port=" + port);
         return {
             pid: pid,
             port: port,
+            undoLiveRecordPid: undoLiveRecordPid,
         };
     }
 
-    return MongoRunner.awaitConnection(pid, port);
+    return MongoRunner.awaitConnection({pid, port, undoLiveRecordPid});
 };
 
 /**
@@ -1439,12 +1739,6 @@ runMongoProgram = function() {
                      '--authenticationDatabase=admin');
     }
 
-    if (progName == 'mongo' && !_useWriteCommandsDefault()) {
-        progName = args[0];
-        args = args.slice(1);
-        args.unshift(progName, '--useLegacyWriteOps');
-    }
-
     return _runMongoProgram.apply(null, args);
 };
 
@@ -1466,11 +1760,6 @@ startMongoProgramNoConnect = function() {
                      '--authenticationDatabase=admin');
     }
 
-    if (progName == 'mongo' && !_useWriteCommandsDefault()) {
-        args = args.slice(1);
-        args.unshift(progName, '--useLegacyWriteOps');
-    }
-
     return _startMongoProgram.apply(null, args);
 };
 
