@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
@@ -39,7 +38,6 @@
 #include <boost/iostreams/stream.hpp>
 #include <boost/iostreams/stream_buffer.hpp>
 #include <boost/iostreams/tee.hpp>
-#include <cctype>
 #include <fcntl.h>
 #include <fmt/format.h>
 #include <iostream>
@@ -71,14 +69,20 @@
 #include "mongo/shell/shell_options.h"
 #include "mongo/shell/shell_utils.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/ctype.h"
 #include "mongo/util/destructor_guard.h"
 #include "mongo/util/exit.h"
+#include "mongo/util/exit_code.h"
 #include "mongo/util/net/hostandport.h"
 #include "mongo/util/quick_exit.h"
 #include "mongo/util/scopeguard.h"
 #include "mongo/util/signal_win32.h"
 #include "mongo/util/str.h"
 #include "mongo/util/text.h"
+#include "mongo/util/version/releases.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
 
 namespace mongo {
 
@@ -135,12 +139,39 @@ void safeClose(int fd) {
     const ScopedSignalBlocker block;
 #endif
     if (close(fd) != 0) {
-        const auto ewd = errnoWithDescription();
-        LOGV2_ERROR(22829, "Failed to close fd", "fd"_attr = fd, "error"_attr = ewd);
+        auto ec = lastPosixError();
+        LOGV2_ERROR(22829, "Failed to close fd", "fd"_attr = fd, "error"_attr = errorMessage(ec));
         fassertFailed(40318);
     }
 }
 
+#ifdef _WIN32
+
+constexpr auto kFilesystemErrorRetry = 10;
+constexpr auto kFilesystemErrorSleepIntervalMillis = 100;
+
+void retryWithBackOff(std::function<void(void)> func) {
+    for (int i = 0; i < kFilesystemErrorRetry - 1; i++) {
+        try {
+            func();
+            return;
+        } catch (const boost::filesystem::filesystem_error& fe) {
+            LOGV2_WARNING(6088701,
+                          "retryWithBackOff: Filesystem error",
+                          "desc"_attr = fe.what(),
+                          "waitMillis"_attr = (100 * i));
+        }
+
+        // A small sleep allows Windows an opportunity to close locked file
+        // handlers, and reduce false errors on remove_all
+        sleepmillis(100 * i);
+    }
+
+    // Try one last time. If this still fails, we propagate the error.
+    func();
+}
+#endif
+
 Mutex _createProcessMtx;
 }  // namespace
 
@@ -177,14 +208,14 @@ void ProgramRegistry::registerProgram(ProcessId pid, int port) {
 
 void ProgramRegistry::unregisterProgram(ProcessId pid) {
     stdx::lock_guard<stdx::recursive_mutex> lk(_mutex);
-    invariant(isPidRegistered(pid));
+    if (isPidRegistered(pid)) {
+        _outputReaderThreads[pid].join();
 
-    _outputReaderThreads[pid].join();
-
-    // Remove the PID from the registry.
-    _outputReaderThreads.erase(pid);
-    _portToPidMap.erase(portForPid(pid));
-    _registeredPids.erase(pid);
+        // Remove the PID from the registry.
+        _outputReaderThreads.erase(pid);
+        _portToPidMap.erase(portForPid(pid));
+        _registeredPids.erase(pid);
+    }
 }
 
 void ProgramRegistry::registerReaderThread(ProcessId pid, stdx::thread reader) {
@@ -194,6 +225,108 @@ void ProgramRegistry::registerReaderThread(ProcessId pid, stdx::thread reader) {
     _outputReaderThreads.emplace(pid, std::move(reader));
 }
 
+void ProgramRegistry::updatePidExitCode(ProcessId pid, int exitCode) {
+    stdx::lock_guard<stdx::recursive_mutex> lk(_mutex);
+    _pidToExitCode[pid] = exitCode;
+}
+
+bool ProgramRegistry::waitForPid(const ProcessId pid, const bool block, int* const exit_code) {
+    {
+        // Be careful not to hold the lock while waiting for the pid to finish
+        stdx::lock_guard<stdx::recursive_mutex> lk(_mutex);
+
+        // unregistered pids are dead
+        if (!this->isPidRegistered(pid)) {
+            if (exit_code) {
+                const auto code = _pidToExitCode.find(pid);
+                if (code != _pidToExitCode.end()) {
+                    *exit_code = code->second;
+                } else {
+                    // If you hit this invariant, you're waiting on a PID that was
+                    // never a child of this process.
+                    MONGO_UNREACHABLE;
+                }
+            }
+            return true;
+        }
+    }
+#ifdef _WIN32
+    HANDLE h = getHandleForPid(pid);
+
+    // wait until the process object is signaled before getting its
+    // exit code. do this even when block is false to ensure that all
+    // file handles open in the process have been closed.
+
+    DWORD ret = WaitForSingleObject(h, (block ? INFINITE : 0));
+    if (ret == WAIT_TIMEOUT) {
+        return false;
+    } else if (ret != WAIT_OBJECT_0) {
+        auto ec = lastSystemError();
+        LOGV2_INFO(22811,
+                   "ProgramRegistry::waitForPid: WaitForSingleObject failed",
+                   "error"_attr = errorMessage(ec));
+    }
+
+    DWORD tmp;
+    if (GetExitCodeProcess(h, &tmp)) {
+        if (tmp == STILL_ACTIVE) {
+            uassert(
+                ErrorCodes::UnknownError, "Process is STILL_ACTIVE even after blocking", !block);
+            return false;
+        }
+        CloseHandle(h);
+        eraseHandleForPid(pid);
+        if (exit_code)
+            *exit_code = tmp;
+        updatePidExitCode(pid, tmp);
+
+        unregisterProgram(pid);
+        return true;
+    } else {
+        auto ec = lastSystemError();
+        LOGV2_INFO(22812, "GetExitCodeProcess failed", "error"_attr = errorMessage(ec));
+        return false;
+    }
+#else
+    int status;
+    int ret;
+    do {
+        errno = 0;
+        ret = waitpid(pid.toNative(), &status, (block ? 0 : WNOHANG));
+    } while (ret == -1 && errno == EINTR);
+    if (ret) {
+        // It's possible for waitpid to return -1 if the waidpid was already
+        // run on the pid. We're not sure if this issue can actually happen
+        // due to the locking/single threaded nature of JS, so we're adding
+        // this invariant to trigger a failure if this ever happens.
+        // See SERVER-63022.
+        invariant(ret > 0);
+        int code;
+        if (WIFEXITED(status)) {
+            code = WEXITSTATUS(status);
+        } else if (WIFSIGNALED(status)) {
+            code = WTERMSIG(status);
+        } else {
+            MONGO_UNREACHABLE;
+        }
+        updatePidExitCode(pid, code);
+        if (exit_code) {
+            *exit_code = code;
+        }
+    }
+    if (ret) {
+        unregisterProgram(pid);
+    } else if (block) {
+        uasserted(ErrorCodes::UnknownError, "Process did not exit after blocking");
+    }
+    return ret == pid.toNative();
+#endif
+}
+
+bool ProgramRegistry::isPidDead(const ProcessId pid, int* const exit_code) {
+    return this->waitForPid(pid, false, exit_code);
+}
+
 void ProgramRegistry::getRegisteredPorts(vector<int>& ports) {
     stdx::lock_guard<stdx::recursive_mutex> lk(_mutex);
     for (const auto& portPid : _portToPidMap) {
@@ -278,7 +411,7 @@ ProgramRunner::ProgramRunner(const BSONObj& args, const BSONObj& env, bool isMon
             "cannot pass an empty argument to ProgramRunner",
             !args.isEmpty());
 
-    string program(args.firstElement().valuestrsafe());
+    string program(args.firstElement().str());
     uassert(ErrorCodes::FailedToParse,
             "invalid program name passed to ProgramRunner",
             !program.empty());
@@ -296,6 +429,10 @@ ProgramRunner::ProgramRunner(const BSONObj& args, const BSONObj& env, bool isMon
     bool isMongosProgram = isMongo &&
         (string("mongos") == programName ||
          programName.string().compare(0, prefix.size(), prefix) == 0);
+    prefix = "mongoqd-";
+    bool isMongoqProgram = isMongo &&
+        (string("mongoqd") == programName ||
+         programName.string().compare(0, prefix.size(), prefix) == 0);
 
     if (!isMongo) {
         _name = "sh";
@@ -303,6 +440,8 @@ ProgramRunner::ProgramRunner(const BSONObj& args, const BSONObj& env, bool isMon
         _name = "d";
     } else if (isMongosProgram) {
         _name = "s";
+    } else if (isMongoqProgram) {
+        _name = "q";
     } else if (programName == "mongobridge") {
         _name = "b";
     } else {
@@ -326,7 +465,7 @@ ProgramRunner::ProgramRunner(const BSONObj& args, const BSONObj& env, bool isMon
             uassert(ErrorCodes::FailedToParse,
                     "Program arguments must be strings",
                     e.type() == mongo::String);
-            str = e.valuestr();
+            str = e.str();
         }
         if (isMongo) {
             if (str == "--port") {
@@ -347,7 +486,7 @@ ProgramRunner::ProgramRunner(const BSONObj& args, const BSONObj& env, bool isMon
                 "Environment variable values must be strings",
                 e.type() == mongo::String);
 
-        _envp.emplace(std::string(e.fieldName()), std::string(e.valuestr()));
+        _envp.emplace(std::string(e.fieldName()), e.str());
     }
 
 // Import this process' environment into _envp, for all keys that have not already been set.
@@ -385,8 +524,8 @@ ProgramRunner::ProgramRunner(const BSONObj& args, const BSONObj& env, bool isMon
         ++environEntry;
     }
 #endif
-    bool needsPort =
-        isMongo && (isMongodProgram || isMongosProgram || (programName == "mongobridge"));
+    bool needsPort = isMongo &&
+        (isMongodProgram || isMongosProgram || isMongoqProgram || (programName == "mongobridge"));
     if (!needsPort) {
         _port = -1;
     }
@@ -423,10 +562,9 @@ void ProgramRunner::start() {
         // Holding the lock for the duration of those events prevents the leaks and thus the
         // associated deadlocks.
         stdx::lock_guard<Latch> lk(_createProcessMtx);
-        int status = pipe(pipeEnds);
-        if (status != 0) {
-            const auto ewd = errnoWithDescription();
-            LOGV2_ERROR(22830, "Failed to create pipe", "error"_attr = ewd);
+        if (pipe(pipeEnds)) {
+            auto ec = lastPosixError();
+            LOGV2_ERROR(22830, "Failed to create pipe", "error"_attr = errorMessage(ec));
             fassertFailed(16701);
         }
 #ifndef _WIN32
@@ -434,16 +572,16 @@ void ProgramRunner::start() {
         // about to fork do *not* inherit the file descriptors for the pipe. If grandchild processes
         // could inherit the FD for the pipe, than the pipe wouldn't close on child process exit. On
         // windows, instead the handle inherit flag is turned off after the call to CreateProcess.
-        status = fcntl(pipeEnds[0], F_SETFD, FD_CLOEXEC);
-        if (status != 0) {
-            const auto ewd = errnoWithDescription();
-            LOGV2_ERROR(22831, "Failed to set FD_CLOEXEC on pipe end 0", "error"_attr = ewd);
+        if (fcntl(pipeEnds[0], F_SETFD, FD_CLOEXEC)) {
+            auto ec = lastPosixError();
+            LOGV2_ERROR(
+                22831, "Failed to set FD_CLOEXEC on pipe end 0", "error"_attr = errorMessage(ec));
             fassertFailed(40308);
         }
-        status = fcntl(pipeEnds[1], F_SETFD, FD_CLOEXEC);
-        if (status != 0) {
-            const auto ewd = errnoWithDescription();
-            LOGV2_ERROR(22832, "Failed to set FD_CLOEXEC on pipe end 1", "error"_attr = ewd);
+        if (fcntl(pipeEnds[1], F_SETFD, FD_CLOEXEC)) {
+            auto ec = lastPosixError();
+            LOGV2_ERROR(
+                22832, "Failed to set FD_CLOEXEC on pipe end 1", "error"_attr = errorMessage(ec));
             fassertFailed(40317);
         }
 #endif
@@ -464,7 +602,11 @@ void ProgramRunner::start() {
 
     _pipe = pipeEnds[0];
 
-    LOGV2_INFO(22810, "shell: Started program", "pid"_attr = _pid, "argv"_attr = _argv);
+    LOGV2_INFO(22810,
+               "shell: Started program",
+               "pid"_attr = _pid,
+               "port"_attr = _port,
+               "argv"_attr = _argv);
 }
 
 void ProgramRunner::operator()() {
@@ -496,14 +638,8 @@ boost::filesystem::path ProgramRunner::findProgram(const string& prog) {
     // needs to be appended.
     //
 
-    auto isExtensionValid = [](std::string extension) {
-        for (auto c : extension) {
-            if (std::isdigit(c)) {
-                return false;
-            }
-        }
-
-        return true;
+    auto isExtensionValid = [](std::string e) {
+        return std::all_of(e.begin(), e.end(), [](char c) { return !ctype::isDigit(c); });
     };
 
     if (!p.has_extension() || !isExtensionValid(p.extension().string())) {
@@ -566,9 +702,9 @@ void ProgramRunner::launchProcess(int child_stdout) {
             ss << _argv[i];
         else {
             ss << '"';
-            // escape all embedded quotes
+            // Escape all embedded quotes and backslashes.
             for (size_t j = 0; j < _argv[i].size(); ++j) {
-                if (_argv[i][j] == '"')
+                if (_argv[i][j] == '"' || _argv[i][j] == '\\')
                     ss << '\\';
                 ss << _argv[i][j];
             }
@@ -639,8 +775,8 @@ void ProgramRunner::launchProcess(int child_stdout) {
                                   &si,
                                   &pi) != 0;
     if (!success) {
-        const auto ewd = errnoWithDescription();
-        ss << "couldn't start process " << _argv[0] << "; " << ewd;
+        const auto ec = lastSystemError();
+        ss << "couldn't start process " << _argv[0] << "; " << errorMessage(ec);
         uasserted(14042, ss.str());
     }
 
@@ -670,8 +806,8 @@ void ProgramRunner::launchProcess(int child_stdout) {
 
     if (nativePid == -1) {
         // Fork failed so it is time for the process to exit
-        const auto ewd = errnoWithDescription();
-        cout << "ProgramRunner is unable to fork child process: " << ewd << endl;
+        const auto ec = lastPosixError();
+        cout << "ProgramRunner is unable to fork child process: " << errorMessage(ec) << endl;
         fassertFailed(34363);
     }
 
@@ -686,7 +822,7 @@ void ProgramRunner::launchProcess(int child_stdout) {
         if (dup2(child_stdout, STDOUT_FILENO) == -1 || dup2(child_stdout, STDERR_FILENO) == -1) {
             // Async signal unsafe code reporting a terminal error condition.
             perror("Unable to dup2 child output: ");
-            _exit(-1);  // do not pass go, do not call atexit handlers
+            _exit(static_cast<int>(ExitCode::fail));  // do not pass go, do not call atexit handlers
         }
 
         execve(argvStorage[0],
@@ -696,74 +832,12 @@ void ProgramRunner::launchProcess(int child_stdout) {
         // Async signal unsafe code reporting a terminal error condition.
         perror(execErrMsg.c_str());
 
-        _exit(-1);
+        _exit(static_cast<int>(ExitCode::fail));
     }
 
 #endif
 }
 
-// returns true if process exited
-// If this function returns true, it will always call `registry.unregisterProgram(pid);`
-// If block is true, this will throw if it cannot wait for the processes to exit.
-bool wait_for_pid(ProcessId pid, bool block = true, int* exit_code = nullptr) {
-#ifdef _WIN32
-    HANDLE h = registry.getHandleForPid(pid);
-
-    // wait until the process object is signaled before getting its
-    // exit code. do this even when block is false to ensure that all
-    // file handles open in the process have been closed.
-
-    DWORD ret = WaitForSingleObject(h, (block ? INFINITE : 0));
-    if (ret == WAIT_TIMEOUT) {
-        return false;
-    } else if (ret != WAIT_OBJECT_0) {
-        const auto ewd = errnoWithDescription();
-        LOGV2_INFO(22811, "wait_for_pid: WaitForSingleObject failed", "error"_attr = ewd);
-    }
-
-    DWORD tmp;
-    if (GetExitCodeProcess(h, &tmp)) {
-        if (tmp == STILL_ACTIVE) {
-            uassert(
-                ErrorCodes::UnknownError, "Process is STILL_ACTIVE even after blocking", !block);
-            return false;
-        }
-        CloseHandle(h);
-        registry.eraseHandleForPid(pid);
-        if (exit_code)
-            *exit_code = tmp;
-
-        registry.unregisterProgram(pid);
-        return true;
-    } else {
-        const auto ewd = errnoWithDescription();
-        LOGV2_INFO(22812, "GetExitCodeProcess failed", "error"_attr = ewd);
-        return false;
-    }
-#else
-    int tmp;
-    int ret;
-    do {
-        ret = waitpid(pid.toNative(), &tmp, (block ? 0 : WNOHANG));
-    } while (ret == -1 && errno == EINTR);
-    if (ret && exit_code) {
-        if (WIFEXITED(tmp)) {
-            *exit_code = WEXITSTATUS(tmp);
-        } else if (WIFSIGNALED(tmp)) {
-            *exit_code = -WTERMSIG(tmp);
-        } else {
-            MONGO_UNREACHABLE;
-        }
-    }
-    if (ret) {
-        registry.unregisterProgram(pid);
-    } else if (block) {
-        uasserted(ErrorCodes::UnknownError, "Process did not exit after blocking");
-    }
-    return ret == pid.toNative();
-#endif
-}
-
 // Output up to BSONObjMaxUserSize characters of the most recent log output in order to
 // avoid hitting the 16MB size limit of a BSONObject.
 BSONObj RawMongoProgramOutput(const BSONObj& args, void* data) {
@@ -781,9 +855,11 @@ BSONObj ClearRawMongoProgramOutput(const BSONObj& args, void* data) {
 }
 
 BSONObj CheckProgram(const BSONObj& args, void* data) {
-    ProcessId pid = ProcessId::fromNative(singleArg(args).numberInt());
+    uassert(
+        ErrorCodes::BadValue, "Cannot check the program with PID = NaN", !singleArg(args).isNaN());
+    ProcessId pid = ProcessId::fromNative(singleArg(args).safeNumberInt());
     int exit_code = -123456;  // sentinel value
-    bool isDead = wait_for_pid(pid, false, &exit_code);
+    bool isDead = registry.isPidDead(pid, &exit_code);
     if (!isDead) {
         return BSON("" << BSON("alive" << true));
     }
@@ -791,9 +867,11 @@ BSONObj CheckProgram(const BSONObj& args, void* data) {
 }
 
 BSONObj WaitProgram(const BSONObj& a, void* data) {
-    ProcessId pid = ProcessId::fromNative(singleArg(a).numberInt());
+    uassert(
+        ErrorCodes::BadValue, "Cannot wait for the program with PID = NaN", !singleArg(a).isNaN());
+    ProcessId pid = ProcessId::fromNative(singleArg(a).safeNumberInt());
     int exit_code = -123456;  // sentinel value
-    wait_for_pid(pid, true, &exit_code);
+    registry.waitForPid(pid, true, &exit_code);
     return BSON(string("") << exit_code);
 }
 
@@ -810,7 +888,7 @@ BSONObj WaitMongoProgram(const BSONObj& a, void* data) {
         return BSON(string("") << 0);
     }
     pid = registry.pidForPort(port);
-    wait_for_pid(pid, true, &exit_code);
+    registry.waitForPid(pid, true, &exit_code);
     return BSON(string("") << exit_code);
 }
 
@@ -856,7 +934,7 @@ BSONObj RunProgram(const BSONObj& a, void* data, bool isMongo) {
     stdx::thread t(r);
     registry.registerReaderThread(r.pid(), std::move(t));
     int exit_code = -123456;  // sentinel value
-    wait_for_pid(r.pid(), true, &exit_code);
+    registry.waitForPid(r.pid(), true, &exit_code);
     return BSON(string("") << exit_code);
 }
 
@@ -870,30 +948,50 @@ BSONObj RunNonMongoProgram(const BSONObj& a, void* data) {
 
 BSONObj ResetDbpath(const BSONObj& a, void* data) {
     uassert(ErrorCodes::FailedToParse, "Expected 1 field", a.nFields() == 1);
-    string path = a.firstElement().valuestrsafe();
+    string path = a.firstElement().str();
     if (path.empty()) {
         LOGV2_WARNING(22824, "ResetDbpath(): nothing to do, path was empty");
         return undefinedReturn;
     }
+
     if (boost::filesystem::exists(path)) {
-        try {
-            boost::filesystem::remove_all(path);
-        } catch (const boost::filesystem::filesystem_error&) {
+        auto removeAllIfNeeded = [path]() {
+            if (boost::filesystem::exists(path)) {
+                boost::filesystem::remove_all(path);
+            }
+        };
+
 #ifdef _WIN32
-            sleepmillis(100);
-            boost::filesystem::remove_all(path);
+        retryWithBackOff(removeAllIfNeeded);
 #else
-            throw;
+        removeAllIfNeeded();
 #endif
-        }
     }
+
+#ifdef _WIN32
+    // Removing the directory may take non-zero time since it is executed asynchronously on Windows.
+    // If the directory is in process of getting removed, then the CreateDirectory fails with access
+    // denied so retry create directory on any error. We only expect to see access denied but we
+    // will retry on all errors.
+    auto wpath = toNativeString(path.c_str());
+    retryWithBackOff([wpath]() {
+        if (!CreateDirectoryW(wpath.c_str(), nullptr)) {
+            auto ec = lastSystemError();
+            uasserted(6088702,
+                      str::stream()
+                          << "CreateDirectory failed with unexpected error: " << errorMessage(ec));
+        }
+    });
+#else
     boost::filesystem::create_directory(path);
+#endif
+
     return undefinedReturn;
 }
 
 BSONObj PathExists(const BSONObj& a, void* data) {
     uassert(ErrorCodes::FailedToParse, "Expected 1 field", a.nFields() == 1);
-    string path = a.firstElement().valuestrsafe();
+    string path = a.firstElement().str();
     if (path.empty()) {
         LOGV2_WARNING(22825, "PathExists(): path was empty");
         return BSON(string("") << false);
@@ -933,7 +1031,11 @@ void copyDir(const boost::filesystem::path& from, const boost::filesystem::path&
     }
 }
 
-// NOTE target dbpath will be cleared first
+/**
+ * Called from JS as  `copyDbpath(fromDir, toDir);`
+ *
+ * The destination dbpath will be cleared first.
+ */
 BSONObj CopyDbpath(const BSONObj& a, void* data) {
     uassert(ErrorCodes::FailedToParse, "Expected 2 fields", a.nFields() == 2);
     BSONObjIterator i(a);
@@ -946,7 +1048,7 @@ BSONObj CopyDbpath(const BSONObj& a, void* data) {
     }
     if (boost::filesystem::exists(to))
         boost::filesystem::remove_all(to);
-    boost::filesystem::create_directory(to);
+    boost::filesystem::create_directories(to);
     copyDir(from, to);
     return undefinedReturn;
 }
@@ -963,10 +1065,9 @@ inline void kill_wrapper(ProcessId pid, int sig, int port, const BSONObj& opt) {
 
     HANDLE event = OpenEventA(EVENT_MODIFY_STATE, FALSE, eventName.c_str());
     if (event == nullptr) {
-        int gle = GetLastError();
-        if (gle != ERROR_FILE_NOT_FOUND) {
-            const auto ewd = errnoWithDescription();
-            LOGV2_WARNING(22827, "kill_wrapper OpenEvent failed", "error"_attr = ewd);
+        auto ec = lastSystemError();
+        if (ec != systemError(ERROR_FILE_NOT_FOUND)) {
+            LOGV2_WARNING(22827, "kill_wrapper OpenEvent failed", "error"_attr = errorMessage(ec));
         } else {
             LOGV2_INFO(
                 22815,
@@ -980,7 +1081,8 @@ inline void kill_wrapper(ProcessId pid, int sig, int port, const BSONObj& opt) {
             //
             try {
                 DBClientConnection conn;
-                conn.connect(HostAndPort{"127.0.0.1:" + std::to_string(port)}, "MongoDB Shell");
+                conn.connect(
+                    HostAndPort{"127.0.0.1:" + std::to_string(port)}, "MongoDB Shell", boost::none);
 
                 BSONElement authObj = opt["auth"];
 
@@ -1011,19 +1113,18 @@ inline void kill_wrapper(ProcessId pid, int sig, int port, const BSONObj& opt) {
 
     bool result = SetEvent(event);
     if (!result) {
-        const auto ewd = errnoWithDescription();
-        LOGV2_ERROR(22833, "kill_wrapper SetEvent failed", "error"_attr = ewd);
+        auto ec = lastSystemError();
+        LOGV2_ERROR(22833, "kill_wrapper SetEvent failed", "error"_attr = errorMessage(ec));
         return;
     }
 #else
-    int x = kill(pid.toNative(), sig);
-    if (x) {
-        if (errno == ESRCH) {
+    if (kill(pid.toNative(), sig)) {
+        auto ec = lastPosixError();
+        if (ec == posixError(ESRCH)) {
         } else {
-            const auto ewd = errnoWithDescription();
-            LOGV2_INFO(22816, "Kill failed", "error"_attr = ewd);
+            LOGV2_INFO(22816, "Kill failed", "error"_attr = errorMessage(ec));
             uasserted(ErrorCodes::UnknownError,
-                      "kill({}, {}) failed: {}"_format(pid.toNative(), sig, ewd));
+                      "kill({}, {}) failed: {}"_format(pid.toNative(), sig, errorMessage(ec)));
         }
     }
 
@@ -1050,13 +1151,13 @@ int killDb(int port, ProcessId _pid, int signal, const BSONObj& opt, bool waitPi
         return 0;
     }
 
-    int exitCode = EXIT_FAILURE;
+    int exitCode = static_cast<int>(ExitCode::fail);
     try {
         LOGV2_INFO(22819, "Waiting for process to terminate.", "pid"_attr = pid);
-        wait_for_pid(pid, true, &exitCode);
+        registry.waitForPid(pid, true, &exitCode);
     } catch (...) {
         LOGV2_WARNING(22828, "Process failed to terminate.", "pid"_attr = pid);
-        return EXIT_FAILURE;
+        return static_cast<int>(ExitCode::fail);
     }
 
     if (signal == SIGKILL) {
@@ -1148,11 +1249,11 @@ BSONObj ConvertTrafficRecordingToBSON(const BSONObj& a, void* data) {
 int KillMongoProgramInstances() {
     vector<ProcessId> pids;
     registry.getRegisteredPids(pids);
-    int returnCode = EXIT_SUCCESS;
+    int returnCode = static_cast<int>(ExitCode::clean);
     for (auto&& pid : pids) {
         int port = registry.portForPid(pid);
         int code = killDb(port != -1 ? port : 0, pid, SIGTERM);
-        if (code != EXIT_SUCCESS) {
+        if (code != static_cast<int>(ExitCode::clean)) {
             LOGV2_INFO(
                 22823, "Process exited with error code", "pid"_attr = pid, "code"_attr = code);
             returnCode = code;
@@ -1171,8 +1272,7 @@ std::vector<ProcessId> getRunningMongoChildProcessIds() {
                  registeredPids.end(),
                  std::back_inserter(outPids),
                  [](const ProcessId& pid) {
-                     const bool block = false;
-                     bool isDead = wait_for_pid(pid, block);
+                     bool isDead = registry.isPidDead(pid);
                      return !isDead;
                  });
     return outPids;
@@ -1189,8 +1289,23 @@ BSONObj RunningMongoChildProcessIds(const BSONObj&, void*) {
     return bob.obj();
 }
 
+// (Generic FCV reference): Propagate generic FCV constants to the shell.
+BSONObj GetFCVConstants(const BSONObj&, void*) {
+    BSONObjBuilder bob;
+    BSONObjBuilder subObj(bob.subobjStart(""));
+
+    subObj.append("latest", multiversion::toString(multiversion::GenericFCV::kLatest));
+    subObj.append("lastContinuous",
+                  multiversion::toString(multiversion::GenericFCV::kLastContinuous));
+    subObj.append("lastLTS", multiversion::toString(multiversion::GenericFCV::kLastLTS));
+    subObj.append("numSinceLastLTS", static_cast<int>(multiversion::kSinceLastLTS));
+    subObj.done();
+
+    return bob.obj();
+}
+
 MongoProgramScope::~MongoProgramScope() {
-    DESTRUCTOR_GUARD(KillMongoProgramInstances(); ClearRawMongoProgramOutput(BSONObj(), nullptr);)
+    DESTRUCTOR_GUARD(KillMongoProgramInstances(); ClearRawMongoProgramOutput(BSONObj(), nullptr))
 }
 
 void installShellUtilsLauncher(Scope& scope) {
@@ -1211,6 +1326,7 @@ void installShellUtilsLauncher(Scope& scope) {
     scope.injectNative("pathExists", PathExists);
     scope.injectNative("copyDbpath", CopyDbpath);
     scope.injectNative("convertTrafficRecordingToBSON", ConvertTrafficRecordingToBSON);
+    scope.injectNative("getFCVConstants", GetFCVConstants);
 }
 }  // namespace shell_utils
 }  // namespace mongo
