@@ -2,7 +2,6 @@ __quiet = false;
 __magicNoPrint = {
     __magicNoPrint: 1111
 };
-__callLastError = false;
 _verboseShell = false;
 
 chatty = function(s) {
@@ -68,45 +67,83 @@ function retryOnNetworkError(func, numRetries, sleepMs) {
     }
 }
 
-// Checks if a Javascript exception is a network error.
-function isNetworkError(error) {
-    let networkErrs = [
-        "network error",
-        "error doing query",
-        "socket exception",
-        "SocketException",
-        "HostNotFound"
-    ];
-    // See if any of the known network error strings appear in the given message.
-    return networkErrs.some(err => error.message.includes(err));
+const shellGeneratedNetworkErrs = ["network error", "error doing query", "socket exception"];
+
+const networkErrs = ["SocketException", "HostNotFound"];
+const networkErrsPlusShellGeneratedNetworkErrs = [...networkErrs, ...shellGeneratedNetworkErrs];
+/**
+ * Determine if a provided object represents a network error
+ * @param {object|Error|string|number} errorOrResponse A command response, error or scalar
+ *     representing an error
+ */
+function isNetworkError(errorOrResponse) {
+    // First check if this is a command response, if so check by error code
+    if (errorOrResponse.code) {
+        if (ErrorCodes.isNetworkError(errorOrResponse.code)) {
+            return true;
+        }
+    }
+
+    // Then check if it's an Error, if so see if any of the known network error strings appear
+    // in the given message.
+    if (errorOrResponse.message) {
+        if (networkErrsPlusShellGeneratedNetworkErrs.some(
+                err => errorOrResponse.message.includes(err))) {
+            return true;
+        }
+    }
+
+    // Otherwise fall back to checking by scalar value
+    return ErrorCodes.isNetworkError(errorOrResponse);
 }
 
-function isRetryableError(error) {
-    const retryableErrors = [
-        "Interrupted",
-        "InterruptedAtShutdown",
-        "InterruptedDueToReplStateChange",
-        "ExceededTimeLimit",
-        "MaxTimeMSExpired",
-        "CursorKilled",
-        "LockTimeout",
-        "ShutdownInProgress",
-        "HostUnreachable",
-        "HostNotFound",
-        "NetworkTimeout",
-        "SocketException",
-        "NotWritablePrimary",
-        "NotPrimaryNoSecondaryOk",
-        "NotPrimaryOrSecondary",
-        "PrimarySteppedDown",
-        "WriteConcernFailed",
-        "WriteConcernLegacyOK",
-        "UnknownReplWriteConcern",
-        "UnsatisfiableWriteConcern"
-    ];
+const retryableErrs = [
+    "Interrupted",
+    "InterruptedAtShutdown",
+    "InterruptedDueToReplStateChange",
+    "ExceededTimeLimit",
+    "MaxTimeMSExpired",
+    "CursorKilled",
+    "LockTimeout",
+    "ShutdownInProgress",
+    "HostUnreachable",
+    "HostNotFound",
+    "NetworkTimeout",
+    "SocketException",
+    "NotWritablePrimary",
+    "NotPrimaryNoSecondaryOk",
+    "NotPrimaryOrSecondary",
+    "PrimarySteppedDown",
+    "WriteConcernFailed",
+    "WriteConcernLegacyOK",
+    "UnknownReplWriteConcern",
+    "UnsatisfiableWriteConcern"
+];
+const retryableErrsPlusShellGeneratedNetworkErrs = [...retryableErrs, ...shellGeneratedNetworkErrs];
+/**
+ * Determine if a provided object represents a retryable error
+ * @param {object|Error|string|number} errorOrResponse A command response, error or scalar
+ *     representing an error
+ */
+function isRetryableError(errorOrResponse) {
+    // First check if this is a command response, if so determine retryability by error code
+    if (errorOrResponse.code) {
+        if (ErrorCodes.isRetriableError(errorOrResponse.code)) {
+            return true;
+        }
+    }
 
-    // See if any of the known network error strings appear in the given message.
-    return retryableErrors.some(err => error.message.includes(err));
+    // Then check if it's an Error, if so determine retryability by checking the error message
+    if (errorOrResponse.message) {
+        // See if any of the known network error strings appear in the given message.
+        if (retryableErrsPlusShellGeneratedNetworkErrs.some(
+                err => errorOrResponse.message.includes(err))) {
+            return true;
+        }
+    }
+
+    // Otherwise fall back to checking by scalar value
+    return ErrorCodes.isRetriableError(errorOrResponse);
 }
 
 // Please consider using bsonWoCompare instead of this as much as possible.
@@ -195,6 +232,15 @@ shellPrint = function(x) {
         shellPrintHelper(x);
 };
 
+_originalPrint = print;
+disablePrint = function() {
+    print = Function.prototype;
+};
+
+enablePrint = function() {
+    print = _originalPrint;
+};
+
 print.captureAllOutput = function(fn, args) {
     var res = {};
     res.output = [];
@@ -273,16 +319,15 @@ jsTestOptions = function() {
             testingDiagnosticsEnabled: TestData.hasOwnProperty('testingDiagnosticsEnabled')
                 ? TestData.testingDiagnosticsEnabled
                 : true,
-            serviceExecutor: TestData.serviceExecutor,
             setParameters: TestData.setParameters,
             setParametersMongos: TestData.setParametersMongos,
+            setParametersMongocryptd: TestData.setParametersMongocryptd,
             storageEngine: TestData.storageEngine,
             storageEngineCacheSizeGB: TestData.storageEngineCacheSizeGB,
             transportLayer: TestData.transportLayer,
             wiredTigerEngineConfigString: TestData.wiredTigerEngineConfigString,
             wiredTigerCollectionConfigString: TestData.wiredTigerCollectionConfigString,
             wiredTigerIndexConfigString: TestData.wiredTigerIndexConfigString,
-            noJournal: TestData.noJournal,
             auth: TestData.auth,
             // Note: keyFile is also used as a flag to indicate cluster auth is turned on, set it
             // to a truthy value if you'd like to do cluster auth, even if it's not keyFile auth.
@@ -299,12 +344,14 @@ jsTestOptions = function() {
             enableMajorityReadConcern: TestData.enableMajorityReadConcern,
             writeConcernMajorityShouldJournal: TestData.writeConcernMajorityShouldJournal,
             enableEncryption: TestData.enableEncryption,
+            encryptionCipherMode: TestData.encryptionCipherMode,
             encryptionKeyFile: TestData.encryptionKeyFile,
             auditDestination: TestData.auditDestination,
             minPort: TestData.minPort,
             maxPort: TestData.maxPort,
             // Note: does not support the array version
             mongosBinVersion: TestData.mongosBinVersion || "",
+            mongoqBinVersion: TestData.mongoqBinVersion || "",
             shardMixedBinVersions: TestData.shardMixedBinVersions || false,
             mixedBinVersions: TestData.mixedBinVersions || false,
             networkMessageCompressors: TestData.networkMessageCompressors,
@@ -350,8 +397,8 @@ jsTestOptions = function() {
             roleGraphInvalidationIsFatal: TestData.roleGraphInvalidationIsFatal || false,
             networkErrorAndTxnOverrideConfig: TestData.networkErrorAndTxnOverrideConfig || {},
             // When useRandomBinVersionsWithinReplicaSet is true, randomly assign the binary
-            // versions of each node in the replica set to 'latest' or 'last-stable'.
-            // This flag is currently a placeholder and only sets the replica set to last-stable
+            // versions of each node in the replica set to 'latest' or 'last-lts'.
+            // This flag is currently a placeholder and only sets the replica set to last-lts
             // FCV.
             useRandomBinVersionsWithinReplicaSet:
                 TestData.useRandomBinVersionsWithinReplicaSet || false,
@@ -363,12 +410,13 @@ jsTestOptions = function() {
             // is shut down.
             alwaysUseLogFiles: TestData.alwaysUseLogFiles || false,
             skipCheckOrphans: TestData.skipCheckOrphans || false,
+            skipCheckRoutingTableConsistency: TestData.skipCheckRoutingTableConsistency || false,
             inEvergreen: TestData.inEvergreen || false,
-            // The ReplSetTest sets the slaveOk bit to true on its connections to the nodes in
-            // the replset, which will result in running all commands with a readPreference of
-            // 'secondaryPreferred'. However, some tests might not want to implicitly allow
-            // secondary reads. Such tests can set this option to turn off this implicit behavior.
-            shouldSkipSettingSlaveOk: TestData.shouldSkipSettingSlaveOk || false,
+
+            undoRecorderPath: TestData.undoRecorderPath,
+            backupOnRestartDir: TestData.backupOnRestartDir || false,
+
+            evergreenTaskId: TestData.evergreenTaskId || null,
         });
     }
     return _jsTestOptions;
@@ -393,7 +441,11 @@ jsTest.basicUserRoles = ["dbOwner"];
 jsTest.adminUserRoles = ["root"];
 
 jsTest.authenticate = function(conn) {
-    if (!jsTest.options().auth && !jsTest.options().keyFile) {
+    const connOptions = conn.fullOptions || {};
+    const authMode =
+        connOptions.clusterAuthMode || conn.clusterAuthMode || jsTest.options().clusterAuthMode;
+
+    if (!jsTest.options().auth && !jsTest.options().keyFile && authMode !== "x509") {
         conn.authenticated = true;
         return true;
     }
@@ -403,12 +455,25 @@ jsTest.authenticate = function(conn) {
             // Set authenticated to stop an infinite recursion from getDB calling
             // back into authenticate.
             conn.authenticated = true;
-            print("Authenticating as user " + jsTestOptions().authUser + " with mechanism " +
-                  DB.prototype._getDefaultAuthenticationMechanism() + " on connection: " + conn);
-            conn.authenticated = conn.getDB(jsTestOptions().authenticationDatabase).auth({
-                user: jsTestOptions().authUser,
-                pwd: jsTestOptions().authPassword,
-            });
+            let mech = DB.prototype._getDefaultAuthenticationMechanism();
+            if (authMode === 'x509') {
+                mech = 'MONGODB-X509';
+            }
+
+            print("Authenticating as user " + jsTestOptions().authUser + " with mechanism " + mech +
+                  " on connection: " + conn);
+
+            if (authMode !== 'x509') {
+                conn.authenticated = conn.getDB(jsTestOptions().authenticationDatabase).auth({
+                    user: jsTestOptions().authUser,
+                    pwd: jsTestOptions().authPassword,
+                });
+            } else {
+                authutil.assertAuthenticate(conn, '$external', {
+                    mechanism: 'MONGODB-X509',
+                });
+            }
+
             return conn.authenticated;
             // Dont' run the hang analyzer because we expect that this might fail in the normal
             // course of events.
@@ -425,7 +490,7 @@ jsTest.authenticateNodes = function(nodes) {
         for (var i = 0; i < nodes.length; i++) {
             // Don't try to authenticate to arbiters
             try {
-                res = nodes[i].getDB("admin").runCommand({replSetGetStatus: 1});
+                res = nodes[i].getDB("admin")._runCommandWithoutApiStrict({replSetGetStatus: 1});
             } catch (e) {
                 // ReplicaSet tests which don't use auth are allowed to have nodes crash during
                 // startup. To allow tests which use to behavior to work with auth,
@@ -445,7 +510,7 @@ jsTest.authenticateNodes = function(nodes) {
 };
 
 jsTest.isMongos = function(conn) {
-    return conn.getDB('admin').isMaster().msg == 'isdbgrid';
+    return conn.getDB('admin')._helloOrLegacyHello().msg == 'isdbgrid';
 };
 
 defaultPrompt = function() {
@@ -454,7 +519,7 @@ defaultPrompt = function() {
 
     if (typeof prefix == 'undefined') {
         prefix = "";
-        var buildInfo = db.runCommand({buildInfo: 1});
+        var buildInfo = db._runCommandWithoutApiStrict({buildInfo: 1});
         try {
             if (buildInfo.modules.indexOf("enterprise") > -1) {
                 prefix += "MongoDB Enterprise ";
@@ -462,9 +527,9 @@ defaultPrompt = function() {
         } catch (e) {
             // Don't do anything here. Just throw the error away.
         }
-        var isMasterRes = db.runCommand({isMaster: 1, forShell: 1});
+        var hello = db._helloOrLegacyHello({forShell: 1});
         try {
-            if (isMasterRes.hasOwnProperty("automationServiceDescriptor")) {
+            if (hello.hasOwnProperty("automationServiceDescriptor")) {
                 prefix += "[automated] ";
             }
         } catch (e) {
@@ -479,12 +544,12 @@ defaultPrompt = function() {
             try {
                 var prompt = replSetMemberStatePrompt();
                 // set our status that it was good
-                db.getMongo().authStatus = {replSetGetStatus: true, isMaster: true};
+                db.getMongo().authStatus = {replSetGetStatus: true, hello: true};
                 return prefix + prompt;
             } catch (e) {
                 // don't have permission to run that, or requires auth
                 // print(e);
-                status = {authRequired: true, replSetGetStatus: false, isMaster: true};
+                status = {authRequired: true, replSetGetStatus: false, hello: true};
             }
         }
         // auth detected
@@ -505,22 +570,22 @@ defaultPrompt = function() {
             }
         }
 
-        // try to use isMaster?
-        if (status.isMaster) {
+        // try to use hello?
+        if (status.hello) {
             try {
-                var prompt = isMasterStatePrompt(isMasterRes);
-                status.isMaster = true;
+                var prompt = helloStatePrompt(hello);
+                status.hello = true;
                 db.getMongo().authStatus = status;
                 return prefix + prompt;
             } catch (e) {
                 status.authRequired = true;
-                status.isMaster = false;
+                status.hello = false;
             }
         }
     } catch (ex) {
         printjson(ex);
         // reset status and let it figure it out next time.
-        status = {isMaster: true};
+        status = {hello: true};
     }
 
     db.getMongo().authStatus = status;
@@ -529,7 +594,8 @@ defaultPrompt = function() {
 
 replSetMemberStatePrompt = function() {
     var state = '';
-    var stateInfo = db.getSiblingDB('admin').runCommand({replSetGetStatus: 1, forShell: 1});
+    var stateInfo =
+        db.getSiblingDB('admin')._runCommandWithoutApiStrict({replSetGetStatus: 1, forShell: 1});
     if (stateInfo.ok) {
         // Report the self member's stateStr if it's present.
         stateInfo.members.forEach(function(member) {
@@ -553,59 +619,35 @@ replSetMemberStatePrompt = function() {
     return state + '> ';
 };
 
-isMasterStatePrompt = function(isMasterResponse) {
+helloStatePrompt = function(helloReply) {
     var state = '';
-    var isMaster = isMasterResponse || db.runCommand({isMaster: 1, forShell: 1});
-    if (isMaster.ok) {
+    var hello = helloReply || db._helloOrLegacyHello({forShell: 1});
+    if (hello.ok) {
         var role = "";
 
-        if (isMaster.msg == "isdbgrid") {
+        if (hello.msg == "isdbgrid") {
             role = "mongos";
         }
 
-        if (isMaster.setName) {
-            if (isMaster.ismaster)
+        if (hello.setName) {
+            if (hello.isWritablePrimary || hello.ismaster)
                 role = "PRIMARY";
-            else if (isMaster.secondary)
+            else if (hello.secondary)
                 role = "SECONDARY";
-            else if (isMaster.arbiterOnly)
+            else if (hello.arbiterOnly)
                 role = "ARBITER";
             else {
                 role = "OTHER";
             }
-            state = isMaster.setName + ':';
+            state = hello.setName + ':';
         }
         state = state + role;
     } else {
-        throw _getErrorWithCode(isMaster, "Failed: " + tojson(isMaster));
+        throw _getErrorWithCode(hello, "Failed: " + tojson(hello));
     }
     return state + '> ';
 };
 
-if (typeof _useWriteCommandsDefault === "undefined") {
-    // We ensure the _useWriteCommandsDefault() function is always defined, in case the JavaScript
-    // engine is being used from someplace other than the mongo shell (e.g. map-reduce).
-    _useWriteCommandsDefault = function _useWriteCommandsDefault() {
-        return false;
-    };
-}
-
-if (typeof _writeMode === "undefined") {
-    // We ensure the _writeMode() function is always defined, in case the JavaScript engine is being
-    // used from someplace other than the mongo shell (e.g. map-reduce).
-    _writeMode = function _writeMode() {
-        return "commands";
-    };
-}
-
-if (typeof _readMode === "undefined") {
-    // We ensure the _readMode() function is always defined, in case the JavaScript engine is being
-    // used from someplace other than the mongo shell (e.g. map-reduce).
-    _readMode = function _readMode() {
-        return "legacy";
-    };
-}
-
 if (typeof _shouldRetryWrites === 'undefined') {
     // We ensure the _shouldRetryWrites() function is always defined, in case the JavaScript engine
     // is being used from someplace other than the mongo shell (e.g. map-reduce).
@@ -626,17 +668,6 @@ if (typeof _shouldUseImplicitSessions === 'undefined') {
 
 shellPrintHelper = function(x) {
     if (typeof (x) == "undefined") {
-        // Make sure that we have a db var before we use it
-        // TODO: This implicit calling of GLE can cause subtle, hard to track issues - remove?
-        if (__callLastError && typeof (db) != "undefined" && db.getMongo &&
-            db.getMongo().writeMode() == "legacy") {
-            __callLastError = false;
-            // explicit w:1 so that replset getLastErrorDefaults aren't used here which would be bad
-            var err = db.getLastError(1);
-            if (err != null) {
-                print(err);
-            }
-        }
         return;
     }
 
@@ -1019,7 +1050,8 @@ shellHelper.show = function(what) {
             dbDeclared = false;
         }
         if (dbDeclared) {
-            var res = db.adminCommand({getLog: "startupWarnings"});
+            var res =
+                db.getSiblingDB("admin")._runCommandWithoutApiStrict({getLog: "startupWarnings"});
             if (res.ok) {
                 if (res.log.length == 0) {
                     return "";
@@ -1074,7 +1106,7 @@ shellHelper.show = function(what) {
         }
 
         if (dbDeclared) {
-            var res = db.runCommand({isMaster: 1, forShell: 1});
+            var res = db._helloOrLegacyHello({forShell: 1});
             if (!res.ok) {
                 print("Note: Cannot determine if automation is active");
                 return "";
@@ -1096,13 +1128,59 @@ shellHelper.show = function(what) {
         }
     }
 
+    if (what == "freeMonitoring") {
+        var dbDeclared, ex;
+        try {
+            // !!db essentially casts db to a boolean
+            // Will throw a reference exception if db hasn't been declared.
+            dbDeclared = !!db;
+        } catch (ex) {
+            dbDeclared = false;
+        }
+
+        if (dbDeclared) {
+            const freemonStatus = db.adminCommand({getFreeMonitoringStatus: 1});
+
+            if (freemonStatus.ok) {
+                if (freemonStatus.state == 'enabled' &&
+                    freemonStatus.hasOwnProperty('userReminder')) {
+                    print("---");
+                    print(freemonStatus.userReminder);
+                    print("---");
+                } else if (freemonStatus.state === 'undecided') {
+                    print(
+                        "---\n" + messageIndent +
+                        "Enable MongoDB's free cloud-based monitoring service, which will then receive and display\n" +
+                        messageIndent +
+                        "metrics about your deployment (disk utilization, CPU, operation statistics, etc).\n" +
+                        "\n" + messageIndent +
+                        "The monitoring data will be available on a MongoDB website with a unique URL accessible to you\n" +
+                        messageIndent +
+                        "and anyone you share the URL with. MongoDB may use this information to make product\n" +
+                        messageIndent +
+                        "improvements and to suggest MongoDB products and deployment options to you.\n" +
+                        "\n" + messageIndent +
+                        "To enable free monitoring, run the following command: db.enableFreeMonitoring()\n" +
+                        messageIndent +
+                        "To permanently disable this reminder, run the following command: db.disableFreeMonitoring()\n" +
+                        "---\n");
+                }
+            }
+
+            return "";
+        } else {
+            print("Cannot show freeMonitoring, \"db\" is not set");
+            return "";
+        }
+    }
+
     if (what == "nonGenuineMongoDBCheck") {
         let matchesKnownImposterSignature = false;
 
         // A MongoDB emulation service offered by a company
         // responsible for a certain disk operating system.
         try {
-            const buildInfo = db.runCommand({buildInfo: 1});
+            const buildInfo = db._runCommandWithoutApiStrict({buildInfo: 1});
             if (buildInfo.hasOwnProperty('_t')) {
                 matchesKnownImposterSignature = true;
             }
@@ -1158,7 +1236,7 @@ __promptWrapper__ = function(promptFunction) {
     try {
         db = originalDB.getMongo().getDB(originalDB.getName());
         // Setting db._session to be a _DummyDriverSession instance makes it so that
-        // a logical session id isn't included in the isMaster and replSetGetStatus
+        // a logical session id isn't included in the hello and replSetGetStatus
         // commands and therefore won't interfere with the session associated with the
         // global "db" object.
         db._session = new _DummyDriverSession(db.getMongo());
@@ -1181,6 +1259,10 @@ var Random = (function() {
     var errorMsg = "The random number generator hasn't been seeded yet; " +
         "call Random.setRandomSeed()";
 
+    function isInitialized() {
+        return initialized;
+    }
+
     // Set the random generator seed.
     function srand(s) {
         initialized = true;
@@ -1248,6 +1330,7 @@ var Random = (function() {
     return {
         genExp: genExp,
         genNormal: genNormal,
+        isInitialized: isInitialized,
         rand: rand,
         randInt: randInt,
         setRandomSeed: setRandomSeed,
@@ -1480,13 +1563,10 @@ rs._runCmd = function(c) {
         res = db.adminCommand(c);
     } catch (e) {
         if (isNetworkError(e)) {
-            // closed connection.  reconnect.
-            db.getLastErrorObj();
-            var o = db.getLastErrorObj();
-            if (o.ok) {
+            if (reconnect(db)) {
                 print("reconnected to server after rs command (which is normal)");
             } else {
-                printjson(o);
+                print("failed to reconnect to server after:" + e);
             }
         } else {
             print("shell got exception during repl set operation: " + e);
@@ -1555,34 +1635,66 @@ rs.reconfigForPSASet = function(memberIndex, cfg, options) {
     return res;
 };
 rs.add = function(hostport, arb) {
-    var cfg = hostport;
+    let res;
+    let self = this;
 
-    var local = db.getSisterDB("local");
-    assert(local.system.replset.count() <= 1,
-           "error: local.system.replset has unexpected contents");
-    var c = local.system.replset.findOne();
-    assert(c, "no config object retrievable from local.system.replset");
+    assert.soon(function() {
+        var cfg = hostport;
 
-    c.version++;
+        var local = db.getSiblingDB("local");
+        assert(local.system.replset.count() <= 1,
+               "error: local.system.replset has unexpected contents");
+        var c = local.system.replset.findOne();
+        assert(c, "no config object retrievable from local.system.replset");
 
-    var max = 0;
-    for (var i in c.members)
-        if (c.members[i]._id > max)
-            max = c.members[i]._id;
-    if (isString(hostport)) {
-        cfg = {_id: max + 1, host: hostport};
-        if (arb)
-            cfg.arbiterOnly = true;
-    } else if (arb == true) {
-        throw Error("Expected first parameter to be a host-and-port string of arbiter, but got " +
-                    tojson(hostport));
-    }
+        const attemptedVersion = c.version++;
 
-    if (cfg._id == null) {
-        cfg._id = max + 1;
-    }
-    c.members.push(cfg);
-    return this._runCmd({replSetReconfig: c});
+        var max = 0;
+        for (var i in c.members) {
+            // Omit 'newlyAdded' field if it exists in the config.
+            delete c.members[i].newlyAdded;
+            if (c.members[i]._id > max)
+                max = c.members[i]._id;
+        }
+        if (isString(hostport)) {
+            cfg = {_id: max + 1, host: hostport};
+            if (arb)
+                cfg.arbiterOnly = true;
+        } else if (arb == true) {
+            throw Error(
+                "Expected first parameter to be a host-and-port string of arbiter, but got " +
+                tojson(hostport));
+        }
+
+        if (cfg._id == null) {
+            cfg._id = max + 1;
+        }
+        c.members.push(cfg);
+
+        res = self._runCmd({replSetReconfig: c});
+        if (res === "") {
+            // _runCmd caught an exception.
+            return true;
+        }
+        if (res.ok) {
+            return true;
+        }
+        if (res.code === ErrorCodes.ConfigurationInProgress ||
+            res.code === ErrorCodes.CurrentConfigNotCommittedYet) {
+            return false;  // keep retrying
+        }
+        if (res.code === ErrorCodes.NewReplicaSetConfigurationIncompatible) {
+            // We will retry only if this error was due to our config version being too low.
+            const cfgState = local.system.replset.findOne();
+            if (cfgState.version >= attemptedVersion) {
+                return false;  // keep retrying
+            }
+        }
+        // Take no action on other errors.
+        return true;
+    }, () => tojson(res), 10 * 60 * 1000 /* timeout */, 200 /* interval */);
+
+    return res;
 };
 rs.syncFrom = function(host) {
     return db._adminCommand({replSetSyncFrom: host});
@@ -1606,13 +1718,13 @@ rs.conf = function() {
     if (resp.ok && !(resp.errmsg) && resp.config)
         return resp.config;
     else if (resp.errmsg && resp.errmsg.startsWith("no such cmd"))
-        return db.getSisterDB("local").system.replset.findOne();
+        return db.getSiblingDB("local").system.replset.findOne();
     throw new Error("Could not retrieve replica set config: " + tojson(resp));
 };
 rs.config = rs.conf;
 
 rs.remove = function(hn) {
-    var local = db.getSisterDB("local");
+    var local = db.getSiblingDB("local");
     assert(local.system.replset.count() <= 1,
            "error: local.system.replset has unexpected contents");
     var c = local.system.replset.findOne();
@@ -1655,7 +1767,7 @@ rs.debug.nullLastOpWritten = function(primary, secondary) {
 };
 
 rs.debug.getLastOpWritten = function(server) {
-    var s = db.getSisterDB("local");
+    var s = db.getSiblingDB("local");
     if (server) {
         s = connect(server + "/local");
     }
@@ -1718,8 +1830,6 @@ help = shellHelper.help = function(x) {
         print("    var mydb = x.getDB('mydb');");
         print("  or");
         print("    var mydb = connect('host[:port]/mydb');");
-        print(
-            "\nNote: the REPL prompt only auto-reports getLastError() for the shell command line connection.\n");
         return;
     } else if (x == "keys") {
         print("Tab completion and command history is available at the command prompt.\n");
