@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kBridge
 
 #include "mongo/platform/basic.h"
 
@@ -37,6 +36,7 @@
 
 #include "mongo/base/init.h"
 #include "mongo/base/initializer.h"
+#include "mongo/db/concurrency/locker_noop_client_observer.h"
 #include "mongo/db/dbmessage.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/service_context.h"
@@ -56,6 +56,7 @@
 #include "mongo/transport/transport_layer_asio.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/exit.h"
+#include "mongo/util/future.h"
 #include "mongo/util/quick_exit.h"
 #include "mongo/util/signal_handlers.h"
 #include "mongo/util/str.h"
@@ -63,6 +64,9 @@
 #include "mongo/util/time_support.h"
 #include "mongo/util/timer.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kBridge
+
+
 namespace mongo {
 
 namespace {
@@ -93,10 +97,7 @@ public:
             return status.getStatus();
         }
 
-        LOGV2(22916,
-              "Processing bridge command: {cmdName}",
-              "Processing bridge command",
-              "cmdName"_attr = cmdName);
+        LOGV2(22916, "Processing bridge command", "cmdName"_attr = cmdName, "cmdObj"_attr = cmdObj);
 
         BridgeCommand* command = status.getValue();
         return command->run(cmdObj, &_settingsMutex, &_settings);
@@ -251,10 +252,12 @@ class ServiceEntryPointBridge final : public ServiceEntryPointImpl {
 public:
     explicit ServiceEntryPointBridge(ServiceContext* svcCtx) : ServiceEntryPointImpl(svcCtx) {}
 
-    DbResponse handleRequest(OperationContext* opCtx, const Message& request) final;
+    Future<DbResponse> handleRequest(OperationContext* opCtx,
+                                     const Message& request) noexcept final;
 };
 
-DbResponse ServiceEntryPointBridge::handleRequest(OperationContext* opCtx, const Message& request) {
+Future<DbResponse> ServiceEntryPointBridge::handleRequest(OperationContext* opCtx,
+                                                          const Message& request) noexcept try {
     if (request.operation() == dbQuery) {
         DbMessage d(request);
         QueryMessage q(d);
@@ -269,7 +272,7 @@ DbResponse ServiceEntryPointBridge::handleRequest(OperationContext* opCtx, const
 
     // If the bridge decides to return something else other than a response from an active exhaust
     // stream, make sure we close the exhaust stream properly.
-    auto earlyExhaustExitGuard = makeGuard([&] {
+    ScopeGuard earlyExhaustExitGuard([&] {
         if (dest.inExhaust()) {
             LOGV2(4622301, "mongobridge shutting down exhaust stream", "remote"_attr = dest);
             dest.setInExhaust(false);
@@ -321,7 +324,7 @@ DbResponse ServiceEntryPointBridge::handleRequest(OperationContext* opCtx, const
     if ((request.operation() == dbQuery &&
          NamespaceString(DbMessage(request).getns()).isCommand()) ||
         request.operation() == dbMsg) {
-        cmdRequest = rpc::opMsgRequestFromAnyProtocol(request);
+        cmdRequest = rpc::opMsgRequestFromAnyProtocol(request, opCtx->getClient());
 
         dest.extractHostInfo(*cmdRequest);
 
@@ -347,7 +350,8 @@ DbResponse ServiceEntryPointBridge::handleRequest(OperationContext* opCtx, const
         if (!status->isOK()) {
             commandReply = StatusWith<BSONObj>(*status);
         }
-        return {replyBuilder->setCommandReply(std::move(commandReply)).done()};
+        return Future<DbResponse>::makeReady(
+            {replyBuilder->setCommandReply(std::move(commandReply)).done()});
     }
 
 
@@ -364,7 +368,7 @@ DbResponse ServiceEntryPointBridge::handleRequest(OperationContext* opCtx, const
                   "remote"_attr = dest,
                   "source"_attr = source->remote().toString());
             source->end();
-            return {Message()};
+            return Future<DbResponse>::makeReady({Message()});
         // Forward the message to 'dest' with probability '1 - hostSettings.loss'.
         case HostSettings::State::kDiscard:
             if (dest.nextCanonicalDouble() < hostSettings.loss) {
@@ -384,10 +388,11 @@ DbResponse ServiceEntryPointBridge::handleRequest(OperationContext* opCtx, const
                           "operation"_attr = networkOpToString(request.operation()),
                           "hostName"_attr = hostName);
                 }
-                return {Message()};
+                return Future<DbResponse>::makeReady({Message()});
             }
-        // Forward the message to 'dest' after waiting for 'hostSettings.delay'
-        // milliseconds.
+            // Forward the message to 'dest' after waiting for 'hostSettings.delay'
+            // milliseconds.
+            [[fallthrough]];
         case HostSettings::State::kForward:
             sleepmillis(durationCount<Milliseconds>(hostSettings.delay));
             break;
@@ -432,7 +437,7 @@ DbResponse ServiceEntryPointBridge::handleRequest(OperationContext* opCtx, const
                   "remote"_attr = dest,
                   "source"_attr = source->remote());
             source->end();
-            return {Message()};
+            return Future<DbResponse>::makeReady({Message()});
         }
 
         // Only support OP_MSG exhaust cursors.
@@ -450,10 +455,13 @@ DbResponse ServiceEntryPointBridge::handleRequest(OperationContext* opCtx, const
         // whether this should be run again to receive more responses from the exhaust stream.
         // We do not need to set 'nextInvocation' in the DbResponse because mongobridge
         // only receives responses but ignores the next request if it is in exhaust mode.
-        return {std::move(response), isExhaust};
+        return Future<DbResponse>::makeReady({std::move(response), isExhaust});
     } else {
-        return {Message()};
+        return Future<DbResponse>::makeReady({Message()});
     }
+} catch (const DBException& e) {
+    LOGV2_ERROR(4879804, "Failed to handle request", "error"_attr = redact(e));
+    return e.toStatus();
 }
 
 int bridgeMain(int argc, char** argv) {
@@ -478,33 +486,39 @@ int bridgeMain(int argc, char** argv) {
     runGlobalInitializersOrDie(std::vector<std::string>(argv, argv + argc));
     startSignalProcessingThread(LogFileStatus::kNoLogFileToRotate);
 
-    setGlobalServiceContext(ServiceContext::make());
+    auto serviceContextHolder = ServiceContext::make();
+    serviceContextHolder->registerClientObserver(std::make_unique<LockerNoopClientObserver>());
+    setGlobalServiceContext(std::move(serviceContextHolder));
     auto serviceContext = getGlobalServiceContext();
+
     serviceContext->setServiceEntryPoint(std::make_unique<ServiceEntryPointBridge>(serviceContext));
-    serviceContext->setServiceExecutor(
-        std::make_unique<transport::ServiceExecutorSynchronous>(serviceContext));
 
-    fassert(50766, serviceContext->getServiceExecutor()->start());
+    {
+        transport::TransportLayerASIO::Options opts;
+        opts.ipList.emplace_back("0.0.0.0");
+        opts.port = mongoBridgeGlobalParams.port;
 
-    transport::TransportLayerASIO::Options opts;
-    opts.ipList.emplace_back("0.0.0.0");
-    opts.port = mongoBridgeGlobalParams.port;
-
-    serviceContext->setTransportLayer(std::make_unique<mongo::transport::TransportLayerASIO>(
-        opts, serviceContext->getServiceEntryPoint()));
-    auto tl = serviceContext->getTransportLayer();
-    if (!tl->setup().isOK()) {
-        LOGV2(22922, "Error setting up transport layer");
-        return EXIT_NET_ERROR;
+        auto tl = std::make_unique<mongo::transport::TransportLayerASIO>(
+            opts, serviceContext->getServiceEntryPoint());
+        serviceContext->setTransportLayer(std::move(tl));
     }
 
-    if (!tl->start().isOK()) {
-        LOGV2(22923, "Error starting transport layer");
-        return EXIT_NET_ERROR;
+    if (auto status = serviceContext->getServiceEntryPoint()->start(); !status.isOK()) {
+        LOGV2(4907203, "Error starting service entry point", "error"_attr = status);
+    }
+
+    if (auto status = serviceContext->getTransportLayer()->setup(); !status.isOK()) {
+        LOGV2(22922, "Error setting up transport layer", "error"_attr = status);
+        return static_cast<int>(ExitCode::netError);
+    }
+
+    if (auto status = serviceContext->getTransportLayer()->start(); !status.isOK()) {
+        LOGV2(22923, "Error starting transport layer", "error"_attr = status);
+        return static_cast<int>(ExitCode::netError);
     }
 
     serviceContext->notifyStartupComplete();
-    return waitForShutdown();
+    return static_cast<int>(waitForShutdown());
 }
 
 }  // namespace mongo
