@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
 #include "mongo/platform/basic.h"
 
@@ -44,6 +43,7 @@
 #include "mongo/config.h"
 
 #include "mongo/base/system_error.h"
+#include "mongo/db/server_feature_flags_gen.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/stats/counters.h"
@@ -52,6 +52,7 @@
 #include "mongo/transport/service_entry_point.h"
 #include "mongo/transport/transport_options_gen.h"
 #include "mongo/util/clock_source.h"
+#include "mongo/util/errno_util.h"
 #include "mongo/util/executor_stats.h"
 #include "mongo/util/hierarchical_acquisition.h"
 #include "mongo/util/net/hostandport.h"
@@ -73,6 +74,9 @@
 
 #include "mongo/transport/session_asio.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
+
+
 namespace mongo {
 namespace transport {
 
@@ -102,6 +106,11 @@ boost::optional<Status> maybeTcpFastOpenStatus;
 }  // namespace
 
 MONGO_FAIL_POINT_DEFINE(transportLayerASIOasyncConnectTimesOut);
+MONGO_FAIL_POINT_DEFINE(transportLayerASIOhangBeforeAccept);
+
+#ifdef MONGO_CONFIG_SSL
+SSLConnectionContext::~SSLConnectionContext() = default;
+#endif
 
 class ASIOReactorTimer final : public ReactorTimer {
 public:
@@ -278,8 +287,10 @@ private:
 thread_local TransportLayerASIO::ASIOReactor* TransportLayerASIO::ASIOReactor::_reactorForThread =
     nullptr;
 
-TransportLayerASIO::Options::Options(const ServerGlobalParams* params)
+TransportLayerASIO::Options::Options(const ServerGlobalParams* params,
+                                     boost::optional<int> loadBalancerPort)
     : port(params->port),
+      loadBalancerPort(loadBalancerPort),
       ipList(params->bind_ips),
 #ifndef _WIN32
       useUnixSockets(!params->noUnixSocket),
@@ -311,15 +322,9 @@ void TransportLayerASIO::TimerService::start() {
     auto precondition = State::kInitialized;
     if (_state.compareAndSwap(&precondition, State::kStarted)) {
         _thread = _spawn([reactor = _reactor] {
-            if (!serverGlobalParams.quiet.load()) {
-                LOGV2_INFO(5490002, "Started a new thread for the timer service");
-            }
-
+            LOGV2_INFO(5490002, "Started a new thread for the timer service");
             reactor->run();
-
-            if (!serverGlobalParams.quiet.load()) {
-                LOGV2_INFO(5490003, "Returning from the timer service thread");
-            }
+            LOGV2_INFO(5490003, "Returning from the timer service thread");
         });
     }
 }
@@ -352,18 +357,15 @@ Reactor* TransportLayerASIO::TimerService::_getReactor() {
 }
 
 TransportLayerASIO::TransportLayerASIO(const TransportLayerASIO::Options& opts,
-                                       ServiceEntryPoint* sep)
-    : _ingressReactor(std::make_shared<ASIOReactor>()),
+                                       ServiceEntryPoint* sep,
+                                       const WireSpec& wireSpec)
+    : TransportLayer(wireSpec),
+      _ingressReactor(std::make_shared<ASIOReactor>()),
       _egressReactor(std::make_shared<ASIOReactor>()),
       _acceptorReactor(std::make_shared<ASIOReactor>()),
-#ifdef MONGO_CONFIG_SSL
-      _ingressSSLContext(nullptr),
-      _egressSSLContext(nullptr),
-#endif
       _sep(sep),
       _listenerOptions(opts),
-      _timerService(std::make_unique<TimerService>()) {
-}
+      _timerService(std::make_unique<TimerService>()) {}
 
 TransportLayerASIO::~TransportLayerASIO() = default;
 
@@ -547,11 +549,21 @@ Status makeConnectError(Status status, const HostAndPort& peer, const WrappedEnd
 }
 
 
-StatusWith<SessionHandle> TransportLayerASIO::connect(HostAndPort peer,
-                                                      ConnectSSLMode sslMode,
-                                                      Milliseconds timeout) {
+StatusWith<SessionHandle> TransportLayerASIO::connect(
+    HostAndPort peer,
+    ConnectSSLMode sslMode,
+    Milliseconds timeout,
+    boost::optional<TransientSSLParams> transientSSLParams) {
+    if (transientSSLParams) {
+        uassert(ErrorCodes::InvalidSSLConfiguration,
+                "Specified transient SSL params but connection SSL mode is not set",
+                sslMode == kEnableSSL);
+        LOGV2_DEBUG(
+            5270701, 2, "Connecting to peer using transient SSL connection", "peer"_attr = peer);
+    }
+
     std::error_code ec;
-    GenericSocket sock(*_egressReactor);
+    ASIOSession::GenericSocket sock(*_egressReactor);
     WrappedResolver resolver(*_egressReactor);
 
     Date_t timeBefore = Date_t::now();
@@ -566,7 +578,7 @@ StatusWith<SessionHandle> TransportLayerASIO::connect(HostAndPort peer,
     }
 
     auto endpoints = std::move(swEndpoints.getValue());
-    auto sws = _doSyncConnect(endpoints.front(), peer, timeout);
+    auto sws = _doSyncConnect(endpoints.front(), peer, timeout, transientSSLParams);
     if (!sws.isOK()) {
         return sws.getStatus();
     }
@@ -590,6 +602,11 @@ StatusWith<SessionHandle> TransportLayerASIO::connect(HostAndPort peer,
         (sslMode == kGlobalSSLMode &&
          ((globalSSLMode == SSLParams::SSLMode_preferSSL) ||
           (globalSSLMode == SSLParams::SSLMode_requireSSL)))) {
+
+        if (auto sslStatus = session->buildSSLSocket(peer); !sslStatus.isOK()) {
+            return sslStatus;
+        }
+
         // The handshake is complete once either of the following passes the finish line:
         // - The thread running the handshake returns from `handshakeSSLForEgress`.
         // - The thread running `TimerService` cancels the handshake due to a timeout.
@@ -598,19 +615,21 @@ StatusWith<SessionHandle> TransportLayerASIO::connect(HostAndPort peer,
         // Schedules a task to cancel the synchronous handshake if it does not complete before the
         // specified timeout.
         auto timer = _timerService->makeTimer();
+
 #ifndef _WIN32
         // TODO SERVER-62035: enable the following on Windows.
         if (timeout > Milliseconds(0)) {
             timer->waitUntil(_timerService->now() + timeout)
                 .getAsync([finishLine, session](Status status) {
-                    if (status.isOK() && finishLine->arriveStrongly())
+                    if (status.isOK() && finishLine->arriveStrongly()) {
                         session->end();
+                    }
                 });
         }
 #endif
 
         Date_t timeBefore = Date_t::now();
-        auto sslStatus = session->handshakeSSLForEgress(peer).getNoThrow();
+        auto sslStatus = session->handshakeSSLForEgress(peer, nullptr).getNoThrow();
         Date_t timeAfter = Date_t::now();
 
         if (timeAfter - timeBefore > kSlowOperationThreshold) {
@@ -642,8 +661,11 @@ StatusWith<SessionHandle> TransportLayerASIO::connect(HostAndPort peer,
 
 template <typename Endpoint>
 StatusWith<TransportLayerASIO::ASIOSessionHandle> TransportLayerASIO::_doSyncConnect(
-    Endpoint endpoint, const HostAndPort& peer, const Milliseconds& timeout) {
-    GenericSocket sock(*_egressReactor);
+    Endpoint endpoint,
+    const HostAndPort& peer,
+    const Milliseconds& timeout,
+    boost::optional<TransientSSLParams> transientSSLParams) {
+    ASIOSession::GenericSocket sock(*_egressReactor);
     std::error_code ec;
 
     const auto protocol = endpoint->protocol();
@@ -652,7 +674,11 @@ StatusWith<TransportLayerASIO::ASIOSessionHandle> TransportLayerASIO::_doSyncCon
 #ifdef TCP_FASTOPEN_CONNECT
     const auto family = protocol.family();
     if ((family == AF_INET) || (family == AF_INET6)) {
-        sock.set_option(TCPFastOpenConnect(gTCPFastOpenClient), ec);
+        setSocketOption(sock,
+                        TCPFastOpenConnect(gTCPFastOpenClient),
+                        "connect (sync) TCP fast open",
+                        logv2::LogSeverity::Info(),
+                        ec);
         if (tcpFastOpenIsConfigured) {
             return errorCodeToStatus(ec);
         }
@@ -690,16 +716,40 @@ StatusWith<TransportLayerASIO::ASIOSessionHandle> TransportLayerASIO::_doSyncCon
 
     sock.non_blocking(false);
     try {
-        return std::make_shared<ASIOSession>(this, std::move(sock), false, *endpoint);
+        std::shared_ptr<const transport::SSLConnectionContext> transientSSLContext;
+#ifdef MONGO_CONFIG_SSL
+        if (transientSSLParams) {
+            auto statusOrContext = createTransientSSLContext(transientSSLParams.value());
+            uassertStatusOK(statusOrContext.getStatus());
+            transientSSLContext = std::move(statusOrContext.getValue());
+        }
+#endif
+        return std::make_shared<ASIOSession>(
+            this, std::move(sock), false, *endpoint, transientSSLContext);
+    } catch (const asio::system_error& e) {
+        return errorCodeToStatus(e.code());
     } catch (const DBException& e) {
         return e.toStatus();
     }
 }
 
-Future<SessionHandle> TransportLayerASIO::asyncConnect(HostAndPort peer,
-                                                       ConnectSSLMode sslMode,
-                                                       const ReactorHandle& reactor,
-                                                       Milliseconds timeout) {
+Future<SessionHandle> TransportLayerASIO::asyncConnect(
+    HostAndPort peer,
+    ConnectSSLMode sslMode,
+    const ReactorHandle& reactor,
+    Milliseconds timeout,
+    std::shared_ptr<ConnectionMetrics> connectionMetrics,
+    std::shared_ptr<const SSLConnectionContext> transientSSLContext) {
+    invariant(connectionMetrics);
+    connectionMetrics->onConnectionStarted();
+
+    if (transientSSLContext) {
+        uassert(ErrorCodes::InvalidSSLConfiguration,
+                "Specified transient SSL context but connection SSL mode is not set",
+                sslMode == kEnableSSL);
+        LOGV2_DEBUG(
+            5270601, 2, "Connecting to peer using transient SSL connection", "peer"_attr = peer);
+    }
 
     struct AsyncConnectState {
         AsyncConnectState(HostAndPort peer,
@@ -717,7 +767,7 @@ Future<SessionHandle> TransportLayerASIO::asyncConnect(HostAndPort peer,
         Promise<SessionHandle> promise;
 
         Mutex mutex = MONGO_MAKE_LATCH(HierarchicalAcquisitionLevel(0), "AsyncConnectState::mutex");
-        GenericSocket socket;
+        ASIOSession::GenericSocket socket;
         ASIOReactorTimer timeoutTimer;
         WrappedResolver resolver;
         WrappedEndpoint resolvedEndpoint;
@@ -762,8 +812,10 @@ Future<SessionHandle> TransportLayerASIO::asyncConnect(HostAndPort peer,
     Date_t timeBefore = Date_t::now();
 
     connector->resolver.asyncResolve(connector->peer, _listenerOptions.enableIPv6)
-        .then([connector, timeBefore](WrappedResolver::EndpointVector results) {
+        .then([connector, timeBefore, connectionMetrics](WrappedResolver::EndpointVector results) {
             try {
+                connectionMetrics->onDNSResolved();
+
                 Date_t timeAfter = Date_t::now();
                 if (timeAfter - timeBefore > kSlowOperationThreshold) {
                     LOGV2_WARNING(23019,
@@ -785,17 +837,32 @@ Future<SessionHandle> TransportLayerASIO::asyncConnect(HostAndPort peer,
 
 #ifdef TCP_FASTOPEN_CONNECT
             std::error_code ec;
-            connector->socket.set_option(TCPFastOpenConnect(gTCPFastOpenClient), ec);
+            setSocketOption(connector->socket,
+                            TCPFastOpenConnect(gTCPFastOpenClient),
+                            "connect (async) TCP fast open",
+                            logv2::LogSeverity::Info(),
+                            ec);
             if (tcpFastOpenIsConfigured) {
                 return futurize(ec);
             }
 #endif
             return connector->socket.async_connect(*connector->resolvedEndpoint, UseFuture{});
         })
-        .then([this, connector, sslMode]() -> Future<void> {
+        .then([this, connector, sslMode, transientSSLContext, connectionMetrics]() -> Future<void> {
+            connectionMetrics->onTCPConnectionEstablished();
+
             stdx::unique_lock<Latch> lk(connector->mutex);
-            connector->session = std::make_shared<ASIOSession>(
-                this, std::move(connector->socket), false, *connector->resolvedEndpoint);
+            connector->session = [&] {
+                try {
+                    return std::make_shared<ASIOSession>(this,
+                                                         std::move(connector->socket),
+                                                         false,
+                                                         *connector->resolvedEndpoint,
+                                                         transientSSLContext);
+                } catch (const asio::system_error& e) {
+                    iasserted(errorCodeToStatus(e.code()));
+                }
+            }();
             connector->session->ensureAsync();
 
 #ifndef MONGO_CONFIG_SSL
@@ -808,11 +875,16 @@ Future<SessionHandle> TransportLayerASIO::asyncConnect(HostAndPort peer,
                 (sslMode == kGlobalSSLMode &&
                  ((globalSSLMode == SSLParams::SSLMode_preferSSL) ||
                   (globalSSLMode == SSLParams::SSLMode_requireSSL)))) {
+                if (const auto sslStatus = connector->session->buildSSLSocket(connector->peer);
+                    !sslStatus.isOK()) {
+                    return sslStatus;
+                }
                 Date_t timeBefore = Date_t::now();
                 return connector->session
-                    ->handshakeSSLForEgressWithLock(
-                        std::move(lk), connector->peer, connector->reactor)
-                    .then([connector, timeBefore] {
+                    ->handshakeSSLForEgress(connector->peer, connector->reactor)
+                    .then([connector, timeBefore, connectionMetrics] {
+                        connectionMetrics->onTLSHandshakeFinished();
+
                         Date_t timeAfter = Date_t::now();
                         if (timeAfter - timeBefore > kSlowOperationThreshold) {
                             networkCounter.incrementNumSlowSSLOperations();
@@ -855,6 +927,11 @@ namespace {
  */
 bool trySetSockOpt(int level, int opt, int val) {
     auto sock = ::socket(AF_INET, SOCK_STREAM, 0);
+    if (sock == -1) {
+        auto ec = lastSocketError();
+        LOGV2_WARNING(5128700, "socket() failed", "error"_attr = errorMessage(ec));
+        return false;
+    }
 
 #ifdef _WIN32
     char* pval = reinterpret_cast<char*>(&val);
@@ -999,7 +1076,11 @@ Status TransportLayerASIO::setup() {
 
 #ifndef _WIN32
     if (_listenerOptions.useUnixSockets && _listenerOptions.isIngress()) {
-        listenAddrs.emplace_back(makeUnixSockPath(_listenerOptions.port));
+        listenAddrs.push_back(makeUnixSockPath(_listenerOptions.port));
+
+        if (_listenerOptions.loadBalancerPort) {
+            listenAddrs.push_back(makeUnixSockPath(*_listenerOptions.loadBalancerPort));
+        }
     }
 #endif
 
@@ -1015,37 +1096,47 @@ Status TransportLayerASIO::setup() {
     _listenerPort = _listenerOptions.port;
     WrappedResolver resolver(*_acceptorReactor);
 
-    // Self-deduplicating list of unique endpoint addresses.
-    std::set<WrappedEndpoint> endpoints;
-    for (auto& ip : listenAddrs) {
-        if (ip.empty()) {
-            LOGV2_WARNING(23020, "Skipping empty bind address");
-            continue;
-        }
-
-        auto swAddrs =
-            resolver.resolve(HostAndPort(ip, _listenerPort), _listenerOptions.enableIPv6);
-        if (!swAddrs.isOK()) {
-            LOGV2_WARNING(23021,
-                          "Found no addresses for {peer}",
-                          "Found no addresses for peer",
-                          "peer"_attr = swAddrs.getStatus());
-            continue;
-        }
-        auto& addrs = swAddrs.getValue();
-        endpoints.insert(addrs.begin(), addrs.end());
+    std::vector<int> ports = {_listenerPort};
+    if (_listenerOptions.loadBalancerPort) {
+        ports.push_back(*_listenerOptions.loadBalancerPort);
     }
 
-    for (auto& addr : endpoints) {
+    // Self-deduplicating list of unique endpoint addresses.
+    std::set<WrappedEndpoint> endpoints;
+    for (const auto& port : ports) {
+        for (const auto& listenAddr : listenAddrs) {
+            if (listenAddr.empty()) {
+                LOGV2_WARNING(23020, "Skipping empty bind address");
+                continue;
+            }
+
+            const auto& swAddrs =
+                resolver.resolve(HostAndPort(listenAddr, port), _listenerOptions.enableIPv6);
+            if (!swAddrs.isOK()) {
+                LOGV2_WARNING(23021,
+                              "Found no addresses for {peer}",
+                              "Found no addresses for peer",
+                              "peer"_attr = swAddrs.getStatus());
+                continue;
+            }
+            const auto& addrs = swAddrs.getValue();
+            endpoints.insert(addrs.begin(), addrs.end());
+        }
+    }
+
+    for (const auto& addr : endpoints) {
 #ifndef _WIN32
         if (addr.family() == AF_UNIX) {
-            if (::unlink(addr.toString().c_str()) == -1 && errno != ENOENT) {
-                LOGV2_ERROR(23024,
-                            "Failed to unlink socket file {path} {error}",
-                            "Failed to unlink socket file",
-                            "path"_attr = addr.toString().c_str(),
-                            "error"_attr = errnoWithDescription(errno));
-                fassertFailedNoTrace(40486);
+            if (::unlink(addr.toString().c_str()) == -1) {
+                auto ec = lastPosixError();
+                if (ec != posixError(ENOENT)) {
+                    LOGV2_ERROR(23024,
+                                "Failed to unlink socket file {path} {error}",
+                                "Failed to unlink socket file",
+                                "path"_attr = addr.toString().c_str(),
+                                "error"_attr = errorMessage(ec));
+                    fassertFailedNoTrace(40486);
+                }
             }
         }
 #endif
@@ -1055,13 +1146,42 @@ Status TransportLayerASIO::setup() {
         }
 
         GenericAcceptor acceptor(*_acceptorReactor);
-        acceptor.open(addr->protocol());
-        acceptor.set_option(GenericAcceptor::reuse_address(true));
+        try {
+            acceptor.open(addr->protocol());
+        } catch (std::exception&) {
+            // Allow the server to start when "ipv6: true" and "bindIpAll: true", but the platform
+            // does not support ipv6 (e.g., ipv6 kernel module is not loaded in Linux).
+            auto bindAllFmt = [](auto p) { return fmt::format(":::{}", p); };
+            bool addrIsBindAll = addr.toString() == bindAllFmt(_listenerPort);
+
+            if (!addrIsBindAll && _listenerOptions.loadBalancerPort) {
+                addrIsBindAll = (addr.toString() == bindAllFmt(*_listenerOptions.loadBalancerPort));
+            }
+
+            if (errno == EAFNOSUPPORT && _listenerOptions.enableIPv6 && addr.family() == AF_INET6 &&
+                addrIsBindAll) {
+                LOGV2_WARNING(4206501,
+                              "Failed to bind to address as the platform does not support ipv6",
+                              "Failed to bind to {address} as the platform does not support ipv6",
+                              "address"_attr = addr.toString());
+                continue;
+            }
+
+            throw;
+        }
+        setSocketOption(acceptor,
+                        GenericAcceptor::reuse_address(true),
+                        "acceptor reuse address",
+                        logv2::LogSeverity::Info());
 
         std::error_code ec;
 #ifdef TCP_FASTOPEN
         if (gTCPFastOpenServer && ((addr.family() == AF_INET) || (addr.family() == AF_INET6))) {
-            acceptor.set_option(TCPFastOpen(gTCPFastOpenQueueSize), ec);
+            setSocketOption(acceptor,
+                            TCPFastOpen(gTCPFastOpenQueueSize),
+                            "acceptor TCP fast open",
+                            logv2::LogSeverity::Info(),
+                            ec);
             if (tcpFastOpenIsConfigured) {
                 return errorCodeToStatus(ec);
             }
@@ -1069,7 +1189,8 @@ Status TransportLayerASIO::setup() {
         }
 #endif
         if (addr.family() == AF_INET6) {
-            acceptor.set_option(asio::ip::v6_only(true));
+            setSocketOption(
+                acceptor, asio::ip::v6_only(true), "acceptor v6 only", logv2::LogSeverity::Info());
         }
 
         acceptor.non_blocking(true, ec);
@@ -1085,11 +1206,12 @@ Status TransportLayerASIO::setup() {
 #ifndef _WIN32
         if (addr.family() == AF_UNIX) {
             if (::chmod(addr.toString().c_str(), serverGlobalParams.unixSocketPermissions) == -1) {
+                auto ec = lastPosixError();
                 LOGV2_ERROR(23026,
                             "Failed to chmod socket file {path} {error}",
                             "Failed to chmod socket file",
                             "path"_attr = addr.toString().c_str(),
-                            "error"_attr = errnoWithDescription(errno));
+                            "error"_attr = errorMessage(ec));
                 fassertFailedNoTrace(40487);
             }
         }
@@ -1116,42 +1238,21 @@ Status TransportLayerASIO::setup() {
     }
 
 #ifdef MONGO_CONFIG_SSL
-    const auto& sslParams = getSSLGlobalParams();
-
-    if (_sslMode() != SSLParams::SSLMode_disabled && _listenerOptions.isIngress()) {
-        _ingressSSLContext = std::make_unique<asio::ssl::context>(asio::ssl::context::sslv23);
-
-        Status status =
-            getSSLManager()->initSSLContext(_ingressSSLContext->native_handle(),
-                                            sslParams,
-                                            SSLManagerInterface::ConnectionDirection::kIncoming);
-        if (!status.isOK()) {
-            return status;
-        }
-
-        auto resp = getSSLManager()->stapleOCSPResponse(_ingressSSLContext->native_handle());
-        if (!resp.isOK()) {
-            return Status(ErrorCodes::InvalidSSLConfiguration,
-                          str::stream()
-                              << "Can not staple OCSP Response. Reason: " << resp.reason());
-        }
-    }
-
-    if (_listenerOptions.isEgress() && getSSLManager()) {
-        _egressSSLContext = std::make_unique<asio::ssl::context>(asio::ssl::context::sslv23);
-        Status status =
-            getSSLManager()->initSSLContext(_egressSSLContext->native_handle(),
-                                            sslParams,
-                                            SSLManagerInterface::ConnectionDirection::kOutgoing);
-        if (!status.isOK()) {
-            return status;
-        }
+    std::shared_ptr<SSLManagerInterface> manager = nullptr;
+    if (SSLManagerCoordinator::get()) {
+        manager = SSLManagerCoordinator::get()->getSSLManager();
     }
+    return rotateCertificates(manager, true);
 #endif
 
     return Status::OK();
 }
 
+void TransportLayerASIO::appendStats(BSONObjBuilder* bob) const {
+    if (gFeatureFlagConnHealthMetrics.isEnabledAndIgnoreFCV())
+        bob->append("listenerProcessingTime", _listenerProcessingTime.load().toBSON());
+}
+
 void TransportLayerASIO::_runListener() noexcept {
     setThreadName("listener");
 
@@ -1206,12 +1307,12 @@ void TransportLayerASIO::_runListener() noexcept {
             LOGV2(
                 23017, "removing socket file: {path}", "removing socket file", "path"_attr = path);
             if (::unlink(path.c_str()) != 0) {
-                const auto ewd = errnoWithDescription();
+                auto ec = lastPosixError();
                 LOGV2_WARNING(23022,
                               "Unable to remove UNIX socket {path}: {error}",
                               "Unable to remove UNIX socket",
                               "path"_attr = path,
-                              "error"_attr = ewd);
+                              "error"_attr = errorMessage(ec));
             }
         }
     }
@@ -1282,7 +1383,11 @@ ReactorHandle TransportLayerASIO::getReactor(WhichReactor which) {
 }
 
 void TransportLayerASIO::_acceptConnection(GenericAcceptor& acceptor) {
-    auto acceptCb = [this, &acceptor](const std::error_code& ec, GenericSocket peerSocket) mutable {
+    auto acceptCb = [this, &acceptor](const std::error_code& ec,
+                                      ASIOSession::GenericSocket peerSocket) mutable {
+        Timer timer;
+        transportLayerASIOhangBeforeAccept.pauseWhileSet();
+
         if (auto lk = stdx::lock_guard(_mutex); _isShutdown) {
             return;
         }
@@ -1309,7 +1414,26 @@ void TransportLayerASIO::_acceptConnection(GenericAcceptor& acceptor) {
         try {
             std::shared_ptr<ASIOSession> session(
                 new ASIOSession(this, std::move(peerSocket), true));
-            _sep->startSession(std::move(session));
+            if (session->isFromLoadBalancer()) {
+                session->parseProxyProtocolHeader(_acceptorReactor)
+                    .getAsync([this, session = std::move(session)](Status s) {
+                        if (s.isOK()) {
+                            _sep->startSession(std::move(session));
+                        }
+                    });
+            } else {
+                _sep->startSession(std::move(session));
+            }
+        } catch (const asio::system_error& e) {
+            // Swallow connection reset errors. Connection reset errors classically present as
+            // asio::error::eof, but can bubble up as asio::error::invalid_argument when calling
+            // into socket.set_option().
+            if (e.code() != asio::error::eof && e.code() != asio::error::invalid_argument) {
+                LOGV2_WARNING(5746600,
+                              "Error accepting new connection: {error}",
+                              "Error accepting new connection",
+                              "error"_attr = e.code().message());
+            }
         } catch (const DBException& e) {
             LOGV2_WARNING(23023,
                           "Error accepting new connection: {error}",
@@ -1317,6 +1441,9 @@ void TransportLayerASIO::_acceptConnection(GenericAcceptor& acceptor) {
                           "error"_attr = e);
         }
 
+        // _acceptConnection() is accessed by only one thread (i.e. the listener thread), so an
+        // atomic increment is not required here
+        _listenerProcessingTime.store(_listenerProcessingTime.load() + timer.elapsed());
         _acceptConnection(acceptor);
     };
 
@@ -1327,6 +1454,83 @@ void TransportLayerASIO::_acceptConnection(GenericAcceptor& acceptor) {
 SSLParams::SSLModes TransportLayerASIO::_sslMode() const {
     return static_cast<SSLParams::SSLModes>(getSSLGlobalParams().sslMode.load());
 }
+
+Status TransportLayerASIO::rotateCertificates(std::shared_ptr<SSLManagerInterface> manager,
+                                              bool asyncOCSPStaple) {
+    if (manager && manager->isTransient()) {
+        return Status(ErrorCodes::InternalError,
+                      "Should not rotate transient SSL manager's certificates");
+    }
+    auto contextOrStatus = _createSSLContext(manager, _sslMode(), asyncOCSPStaple);
+    if (!contextOrStatus.isOK()) {
+        return contextOrStatus.getStatus();
+    }
+    _sslContext = std::move(contextOrStatus.getValue());
+    return Status::OK();
+}
+
+StatusWith<std::shared_ptr<const transport::SSLConnectionContext>>
+TransportLayerASIO::_createSSLContext(std::shared_ptr<SSLManagerInterface>& manager,
+                                      SSLParams::SSLModes sslMode,
+                                      bool asyncOCSPStaple) const {
+
+    std::shared_ptr<SSLConnectionContext> newSSLContext = std::make_shared<SSLConnectionContext>();
+    newSSLContext->manager = manager;
+    const auto& sslParams = getSSLGlobalParams();
+
+    if (sslMode != SSLParams::SSLMode_disabled && _listenerOptions.isIngress()) {
+        newSSLContext->ingress = std::make_unique<asio::ssl::context>(asio::ssl::context::sslv23);
+
+        Status status = newSSLContext->manager->initSSLContext(
+            newSSLContext->ingress->native_handle(),
+            sslParams,
+            SSLManagerInterface::ConnectionDirection::kIncoming);
+        if (!status.isOK()) {
+            return status;
+        }
+
+        std::weak_ptr<const SSLConnectionContext> weakContextPtr = newSSLContext;
+        manager->registerOwnedBySSLContext(weakContextPtr);
+        auto resp = newSSLContext->manager->stapleOCSPResponse(
+            newSSLContext->ingress->native_handle(), asyncOCSPStaple);
+
+        if (!resp.isOK()) {
+            return Status(ErrorCodes::InvalidSSLConfiguration,
+                          str::stream()
+                              << "Can not staple OCSP Response. Reason: " << resp.reason());
+        }
+    }
+
+    if (_listenerOptions.isEgress() && newSSLContext->manager) {
+        newSSLContext->egress = std::make_unique<asio::ssl::context>(asio::ssl::context::sslv23);
+        Status status = newSSLContext->manager->initSSLContext(
+            newSSLContext->egress->native_handle(),
+            sslParams,
+            SSLManagerInterface::ConnectionDirection::kOutgoing);
+        if (!status.isOK()) {
+            return status;
+        }
+        if (newSSLContext->manager->isTransient()) {
+            newSSLContext->targetClusterURI =
+                newSSLContext->manager->getTargetedClusterConnectionString();
+        }
+    }
+    return newSSLContext;
+}
+
+StatusWith<std::shared_ptr<const transport::SSLConnectionContext>>
+TransportLayerASIO::createTransientSSLContext(const TransientSSLParams& transientSSLParams) {
+    auto coordinator = SSLManagerCoordinator::get();
+    if (!coordinator) {
+        return Status(ErrorCodes::InvalidSSLConfiguration,
+                      "SSLManagerCoordinator is not initialized");
+    }
+    auto manager = coordinator->createTransientSSLManager(transientSSLParams);
+    invariant(manager);
+
+    return _createSSLContext(manager, _sslMode(), true /* asyncOCSPStaple */);
+}
+
 #endif
 
 #ifdef __linux__
