@@ -33,16 +33,14 @@
 #include <string>
 #include <typeinfo>
 
-#include "mongo/base/status.h"  // NOTE: This is safe as utils depend on base
+#include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
 #include "mongo/platform/compiler.h"
 #include "mongo/platform/source_location.h"
+#include "mongo/util/assert_util_core.h"
 #include "mongo/util/concurrency/thread_name.h"
 #include "mongo/util/debug_util.h"
-
-#define MONGO_INCLUDE_INVARIANT_H_WHITELISTED
-#include "mongo/util/invariant.h"
-#undef MONGO_INCLUDE_INVARIANT_H_WHITELISTED
+#include "mongo/util/exit_code.h"
 
 namespace mongo {
 
@@ -56,12 +54,11 @@ public:
     AtomicWord<int> warning;
     AtomicWord<int> msg;
     AtomicWord<int> user;
+    AtomicWord<int> tripwire;
     AtomicWord<int> rollovers;
 };
 
 extern AssertionCount assertionCount;
-
-
 class DBException;
 std::string causedBy(const DBException& e);
 std::string causedBy(const std::string& e);
@@ -115,7 +112,7 @@ public:
     /**
      * Returns the generic ErrorExtraInfo if present.
      */
-    const ErrorExtraInfo* extraInfo() const {
+    std::shared_ptr<const ErrorExtraInfo> extraInfo() const {
         return _status.extraInfo();
     }
 
@@ -123,11 +120,18 @@ public:
      * Returns a specific subclass of ErrorExtraInfo if the error code matches that type.
      */
     template <typename ErrorDetail>
-    const ErrorDetail* extraInfo() const {
+    std::shared_ptr<const ErrorDetail> extraInfo() const {
         return _status.extraInfo<ErrorDetail>();
     }
 
-    static AtomicWord<bool> traceExceptions;
+    static inline AtomicWord<bool> traceExceptions{false};
+
+    /**
+     * Allows handling `ErrorCodes::WriteConflict` as a special case and if true, will call
+     * `printStackTrace` on every `WriteConflict` error. Can be set via the
+     * `traceWriteConflictExceptions` server parameter.
+     */
+    static inline AtomicWord<bool> traceWriteConflictExceptions{false};
 
 protected:
     DBException(const Status& status) : _status(status) {
@@ -152,6 +156,29 @@ public:
     AssertionException(const Status& status) : DBException(status) {}
 };
 
+/**
+ * Use `throwWriteConflictException()` instead of throwing `WriteConflictException` directly.
+ */
+class WriteConflictException final : public DBException {
+public:
+    WriteConflictException(const Status& status) : DBException(status) {}
+
+private:
+    void defineOnlyInFinalSubclassToPreventSlicing() final {}
+};
+
+/**
+ * Use `throwTemporarilyUnavailableException()` instead of throwing
+ * `TemporarilyUnavailableException` directly.
+ */
+class TemporarilyUnavailableException final : public DBException {
+public:
+    TemporarilyUnavailableException(const Status& status) : DBException(status) {}
+
+private:
+    void defineOnlyInFinalSubclassToPreventSlicing() final {}
+};
+
 /**
  * The base class of all DBExceptions for codes of the given ErrorCategory to allow catching by
  * category.
@@ -168,7 +195,6 @@ protected:
     }
 };
 
-
 /**
  * This namespace contains implementation details for our error handling code and should not be used
  * directly in general code.
@@ -187,7 +213,7 @@ public:
     // This is only a template to enable SFINAE. It will only be instantiated with the default
     // value.
     template <ErrorCodes::Error code_copy = kCode>
-    const ErrorExtraInfoFor<code_copy>* operator->() const {
+    std::shared_ptr<const ErrorExtraInfoFor<code_copy>> operator->() const {
         MONGO_STATIC_ASSERT(code_copy == kCode);
         return this->template extraInfo<ErrorExtraInfoFor<kCode>>();
     }
@@ -206,6 +232,16 @@ struct ExceptionForDispatcher<code, CategoryList<categories...>> {
                                     ExceptionForImpl<code, ExceptionForCat<categories>...>>;
 };
 
+template <>
+struct ExceptionForDispatcher<ErrorCodes::WriteConflict> {
+    using type = WriteConflictException;
+};
+
+template <>
+struct ExceptionForDispatcher<ErrorCodes::TemporarilyUnavailable> {
+    using type = TemporarilyUnavailableException;
+};
+
 }  // namespace error_details
 
 
@@ -323,6 +359,15 @@ inline void fassertNoTraceWithLocation(int msgid,
 
 namespace error_details {
 
+inline const Status& makeStatus(const Status& s) {
+    return s;
+}
+
+template <typename T>
+const Status& makeStatus(const StatusWith<T>& sw) {
+    return sw.getStatus();
+}
+
 // This function exists so that uassert/massert can take plain int literals rather than requiring
 // ErrorCodes::Error wrapping.
 template <typename StringLike>
@@ -436,40 +481,60 @@ inline void massertStatusOKWithLocation(const Status& status, const char* file,
     }
 }
 
+#define MONGO_BASE_ASSERT_VA_FAILED(fail_func, ...)                                          \
+    do {                                                                                     \
+        auto mongoSourceLocation = MONGO_SOURCE_LOCATION();                                  \
+        [&]() MONGO_COMPILER_COLD_FUNCTION {                                                 \
+            fail_func(::mongo::error_details::makeStatus(__VA_ARGS__), mongoSourceLocation); \
+        }();                                                                                 \
+        MONGO_COMPILER_UNREACHABLE;                                                          \
+    } while (false)
+
+#define MONGO_BASE_ASSERT_VA_4(fail_func, code, msg, cond)         \
+    do {                                                           \
+        if (MONGO_unlikely(!(cond)))                               \
+            MONGO_BASE_ASSERT_VA_FAILED(fail_func, (code), (msg)); \
+    } while (false)
+
+#define MONGO_BASE_ASSERT_VA_2(fail_func, statusExpr)                              \
+    do {                                                                           \
+        if (const auto& stLocal_ = (statusExpr); MONGO_unlikely(!stLocal_.isOK())) \
+            MONGO_BASE_ASSERT_VA_FAILED(fail_func, stLocal_);                      \
+    } while (false)
+
+#define MONGO_BASE_ASSERT_VA_DISPATCH(...) \
+    MONGO_expand(MONGO_expand(BOOST_PP_OVERLOAD(MONGO_BASE_ASSERT_VA_, __VA_ARGS__))(__VA_ARGS__))
+
 /**
- * `internalAssert` is provided as an alternative for `uassert` variants (e.g., `uassertStatusOK`)
+ * `iassert` is provided as an alternative for `uassert` variants (e.g., `uassertStatusOK`)
  * to support cases where we expect a failure, the failure is recoverable, or accounting for the
- * failure, updating assertion counters, isn't desired. `internalAssert` logs at D3 instead of D1,
+ * failure, updating assertion counters, isn't desired. `iassert` logs at D3 instead of D1,
  * which helps with reducing the noise of assertions in production. The goal is to keep one
- * interface (i.e., `internalAssert(...)`) for all possible assertion variants, and use function
+ * interface (i.e., `iassert(...)`) for all possible assertion variants, and use function
  * overloading to expand type support as needed.
  */
-#define internalAssert(...) \
-    ::mongo::internalAssertWithLocation(MONGO_SOURCE_LOCATION(), __VA_ARGS__)
+#define iassert(...) MONGO_BASE_ASSERT_VA_DISPATCH(::mongo::iassertFailed, __VA_ARGS__)
+#define iasserted(...) MONGO_BASE_ASSERT_VA_FAILED(::mongo::iassertFailed, __VA_ARGS__)
+MONGO_COMPILER_NORETURN void iassertFailed(const Status& status, SourceLocation loc);
 
-void internalAssertWithLocation(SourceLocationHolder loc, const Status& status);
+/**
+ * "tripwire/test assert". Like uassert, but with a deferred-fatality tripwire that gets
+ * checked prior to normal shutdown. Used to ensure that this assertion will both fail the
+ * operation and also cause a test suite failure.
+ */
+#define tassert(...) MONGO_BASE_ASSERT_VA_DISPATCH(::mongo::tassertFailed, __VA_ARGS__)
+#define tasserted(...) MONGO_BASE_ASSERT_VA_FAILED(::mongo::tassertFailed, __VA_ARGS__)
+MONGO_COMPILER_NORETURN void tassertFailed(const Status& status, SourceLocation loc);
 
-inline void internalAssertWithLocation(SourceLocationHolder loc, Status&& status) {
-    internalAssertWithLocation(std::move(loc), status);
-}
+/**
+ * Return true if tripwire conditions have occurred.
+ */
+bool haveTripwireAssertionsOccurred();
 
-inline void internalAssertWithLocation(SourceLocationHolder loc,
-                                       int msgid,
-                                       const std::string& msg,
-                                       bool expr) {
-    if (MONGO_unlikely(!expr))
-        internalAssertWithLocation(std::move(loc), Status(ErrorCodes::Error(msgid), msg));
-}
-
-template <typename T>
-inline void internalAssertWithLocation(SourceLocationHolder loc, const StatusWith<T>& sw) {
-    internalAssertWithLocation(std::move(loc), sw.getStatus());
-}
-
-template <typename T>
-inline void internalAssertWithLocation(SourceLocationHolder loc, StatusWith<T>&& sw) {
-    internalAssertWithLocation(std::move(loc), sw);
-}
+/**
+ * If tripwire conditions have occurred, warn via the log.
+ */
+void warnIfTripwireAssertionsOccurred();
 
 /**
  * verify is deprecated. It is like invariant() in debug builds and massert() in release builds.
@@ -630,9 +695,8 @@ Status exceptionToStatus() noexcept;
     }
 
 /**
- * The purpose of this macro is to instruct the compiler that a line of code will never be reached.
- *
- * Example:
+ * Produces an invariant failure if executed. Use when reaching this statement indicates a
+ * programming error. Example:
  *     // code above checks that expr can only be FOO or BAR
  *     switch (expr) {
  *     case FOO: { ... }
@@ -640,5 +704,114 @@ Status exceptionToStatus() noexcept;
  *     default:
  *         MONGO_UNREACHABLE;
  */
-
 #define MONGO_UNREACHABLE ::mongo::invariantFailed("Hit a MONGO_UNREACHABLE!", __FILE__, __LINE__);
+
+/**
+ * Like `MONGO_UNREACHABLE`, but triggers a `tassert` instead of an `invariant`
+ */
+#define MONGO_UNREACHABLE_TASSERT(msgid) tasserted(msgid, "Hit a MONGO_UNREACHABLE_TASSERT!")
+
+/**
+ * Produces an invariant failure if executed. Subset of MONGO_UNREACHABLE, but specifically
+ * to indicate that the program has reached a function that is unimplemented and should be
+ * unreachable from production.
+ * Example:
+ *
+ *   void myFuncToDo() {
+ *       MONGO_UNIMPLEMENTED;
+ *   }
+ */
+#define MONGO_UNIMPLEMENTED \
+    ::mongo::invariantFailed("Hit a MONGO_UNIMPLEMENTED!", __FILE__, __LINE__);
+
+/**
+ * Like `MONGO_UNIMPLEMENTED`, but triggers a `tassert` instead of an `invariant`
+ */
+#define MONGO_UNIMPLEMENTED_TASSERT(msgid) tasserted(msgid, "Hit a MONGO_UNIMPLEMENTED_TASSERT!")
+
+namespace mongo {
+
+/**
+ * A stack of auxilliary information to be dumped on invariant failure.
+ * These are intended to carry only very lightweight objects like short strings
+ * and numbers, to give some basic clue as to what was going on when a thread
+ * suffered an invariant failure.
+ */
+class ScopedDebugInfoStack {
+public:
+    struct Rec {
+        virtual ~Rec() = default;
+        virtual std::string toString() const = 0;
+    };
+
+    void push(const Rec* rec) {
+        _stack.push_back(rec);
+    }
+    void pop() {
+        _stack.pop_back();
+    }
+
+    std::vector<std::string> getAll() const {
+        std::vector<std::string> r;
+        r.reserve(_stack.size());
+        std::transform(_stack.begin(), _stack.end(), std::back_inserter(r), [](auto&& e) {
+            return e->toString();
+        });
+        return r;
+    }
+
+private:
+    std::vector<const Rec*> _stack;
+};
+
+/** Each thread has its own stack of scoped debug info. */
+inline ScopedDebugInfoStack& scopedDebugInfoStack() {
+    thread_local ScopedDebugInfoStack tls;
+    return tls;
+}
+
+/**
+ * An RAII type that attaches a datum to a ScopedDebugInfoStack, intended as a
+ * broad hint as to what the thread is doing. Pops that datum at scope
+ * exit. By default, attaches to the thread_local ScopedDebugInfoStack.
+ * If the thread encounters a fatal error, the thread's ScopedDebugInfoStack
+ * is logged.
+ *
+ * Example:
+ *
+ *     void doSomethingAsUser(const User& currentUser) {
+ *         ScopedDebugInfo userNameDbg("userName", currentUser.nameStringData());
+ *         ScopedDebugInfo userIdDbg("userId", currentUser.id());
+ *         somethingThatMightCrash(currentUser);
+ *     }
+ */
+template <typename T>
+class ScopedDebugInfo {
+public:
+    ScopedDebugInfo(StringData label, T v, ScopedDebugInfoStack* stack = &scopedDebugInfoStack())
+        : label(label), v(std::move(v)), stack(stack) {
+        stack->push(&rec);
+    }
+    ~ScopedDebugInfo() {
+        stack->pop();
+    }
+    ScopedDebugInfo(const ScopedDebugInfo&) noexcept = delete;
+    ScopedDebugInfo& operator=(const ScopedDebugInfo&) noexcept = delete;
+
+private:
+    struct ThisRec : ScopedDebugInfoStack::Rec {
+        explicit ThisRec(const ScopedDebugInfo* owner) : owner(owner) {}
+        std::string toString() const override {
+            using namespace fmt::literals;
+            return "{}: {}"_format(owner->label, owner->v);
+        }
+        const ScopedDebugInfo* owner;
+    };
+
+    StringData label;
+    T v;
+    ScopedDebugInfoStack* stack;
+    ThisRec rec{this};
+};
+
+}  // namespace mongo
