@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/config.h"
 #include "mongo/platform/basic.h"
@@ -35,11 +34,17 @@
 #include <type_traits>
 
 #include "mongo/base/static_assert.h"
+#include "mongo/unittest/assert_that.h"
 #include "mongo/unittest/death_test.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/exit_code.h"
+#include "mongo/util/quick_exit.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
+
 namespace mongo {
 namespace {
 
@@ -74,8 +79,6 @@ MONGO_STATIC_ASSERT(!std::is_base_of<ExceptionForCat<ErrorCategory::NetworkError
                                      ExceptionFor<ErrorCodes::BadValue>>());
 MONGO_STATIC_ASSERT(!std::is_base_of<ExceptionForCat<ErrorCategory::NotPrimaryError>,
                                      ExceptionFor<ErrorCodes::BadValue>>());
-MONGO_STATIC_ASSERT(!std::is_base_of<ExceptionForCat<ErrorCategory::Interruption>,
-                                     ExceptionFor<ErrorCodes::BadValue>>());
 
 TEST(AssertUtils, UassertNamedCodeWithoutCategories) {
     ASSERT_CATCHES(ErrorCodes::BadValue, DBException);
@@ -84,7 +87,6 @@ TEST(AssertUtils, UassertNamedCodeWithoutCategories) {
     ASSERT_NOT_CATCHES(ErrorCodes::BadValue, ExceptionFor<ErrorCodes::DuplicateKey>);
     ASSERT_NOT_CATCHES(ErrorCodes::BadValue, ExceptionForCat<ErrorCategory::NetworkError>);
     ASSERT_NOT_CATCHES(ErrorCodes::BadValue, ExceptionForCat<ErrorCategory::NotPrimaryError>);
-    ASSERT_NOT_CATCHES(ErrorCodes::BadValue, ExceptionForCat<ErrorCategory::Interruption>);
 }
 
 // NotWritablePrimary - NotPrimaryError, RetriableError
@@ -97,8 +99,6 @@ MONGO_STATIC_ASSERT(!std::is_base_of<ExceptionForCat<ErrorCategory::NetworkError
                                      ExceptionFor<ErrorCodes::NotWritablePrimary>>());
 MONGO_STATIC_ASSERT(std::is_base_of<ExceptionForCat<ErrorCategory::NotPrimaryError>,
                                     ExceptionFor<ErrorCodes::NotWritablePrimary>>());
-MONGO_STATIC_ASSERT(!std::is_base_of<ExceptionForCat<ErrorCategory::Interruption>,
-                                     ExceptionFor<ErrorCodes::NotWritablePrimary>>());
 
 TEST(AssertUtils, UassertNamedCodeWithOneCategory) {
     ASSERT_CATCHES(ErrorCodes::NotWritablePrimary, DBException);
@@ -108,11 +108,10 @@ TEST(AssertUtils, UassertNamedCodeWithOneCategory) {
     ASSERT_NOT_CATCHES(ErrorCodes::NotWritablePrimary,
                        ExceptionForCat<ErrorCategory::NetworkError>);
     ASSERT_CATCHES(ErrorCodes::NotWritablePrimary, ExceptionForCat<ErrorCategory::NotPrimaryError>);
-    ASSERT_NOT_CATCHES(ErrorCodes::NotWritablePrimary,
-                       ExceptionForCat<ErrorCategory::Interruption>);
 }
 
 // InterruptedDueToReplStateChange - NotPrimaryError, Interruption, RetriableError
+// TODO(SERVER-56251): revise list when removing the Interruption category.
 MONGO_STATIC_ASSERT(
     std::is_same<error_details::ErrorCategoriesFor<ErrorCodes::InterruptedDueToReplStateChange>,
                  error_details::CategoryList<ErrorCategory::Interruption,
@@ -124,8 +123,6 @@ MONGO_STATIC_ASSERT(!std::is_base_of<ExceptionForCat<ErrorCategory::NetworkError
                                      ExceptionFor<ErrorCodes::InterruptedDueToReplStateChange>>());
 MONGO_STATIC_ASSERT(std::is_base_of<ExceptionForCat<ErrorCategory::NotPrimaryError>,
                                     ExceptionFor<ErrorCodes::InterruptedDueToReplStateChange>>());
-MONGO_STATIC_ASSERT(std::is_base_of<ExceptionForCat<ErrorCategory::Interruption>,
-                                    ExceptionFor<ErrorCodes::InterruptedDueToReplStateChange>>());
 
 TEST(AssertUtils, UassertNamedCodeWithTwoCategories) {
     ASSERT_CATCHES(ErrorCodes::InterruptedDueToReplStateChange, DBException);
@@ -138,8 +135,6 @@ TEST(AssertUtils, UassertNamedCodeWithTwoCategories) {
                        ExceptionForCat<ErrorCategory::NetworkError>);
     ASSERT_CATCHES(ErrorCodes::InterruptedDueToReplStateChange,
                    ExceptionForCat<ErrorCategory::NotPrimaryError>);
-    ASSERT_CATCHES(ErrorCodes::InterruptedDueToReplStateChange,
-                   ExceptionForCat<ErrorCategory::Interruption>);
 }
 
 MONGO_STATIC_ASSERT(!error_details::isNamedCode<19999>);
@@ -151,7 +146,6 @@ TEST(AssertUtils, UassertNumericCode) {
     ASSERT_NOT_CATCHES(19999, ExceptionFor<ErrorCodes::DuplicateKey>);
     ASSERT_NOT_CATCHES(19999, ExceptionForCat<ErrorCategory::NetworkError>);
     ASSERT_NOT_CATCHES(19999, ExceptionForCat<ErrorCategory::NotPrimaryError>);
-    ASSERT_NOT_CATCHES(19999, ExceptionForCat<ErrorCategory::Interruption>);
 }
 
 TEST(AssertUtils, UassertStatusOKPreservesExtraInfo) {
@@ -232,24 +226,27 @@ TEST(AssertUtils, InternalAssertWithStatus) {
     auto userAssertions = assertionCount.user.load();
     try {
         Status status = {ErrorCodes::BadValue, "Test"};
-        internalAssert(status);
+        iassert(status);
     } catch (const DBException& ex) {
         ASSERT_EQ(ex.code(), ErrorCodes::BadValue);
         ASSERT_EQ(ex.reason(), "Test");
     }
+
+    iassert(Status::OK());
+
     ASSERT_EQ(userAssertions, assertionCount.user.load());
 }
 
 TEST(AssertUtils, InternalAssertWithExpression) {
     auto userAssertions = assertionCount.user.load();
     try {
-        internalAssert(48922, "Test", false);
+        iassert(48922, "Test", false);
     } catch (const DBException& ex) {
         ASSERT_EQ(ex.code(), 48922);
         ASSERT_EQ(ex.reason(), "Test");
     }
 
-    internalAssert(48923, "Another test", true);
+    iassert(48923, "Another test", true);
 
     ASSERT_EQ(userAssertions, assertionCount.user.load());
 }
@@ -273,6 +270,73 @@ TEST(AssertUtils, MassertTypedExtraInfoWorks) {
     }
 }
 
+// tassert
+void doTassert() {
+    auto tripwireAssertions = assertionCount.tripwire.load();
+
+    try {
+        Status status = {ErrorCodes::BadValue, "Test with Status"};
+        tassert(status);
+    } catch (const DBException& ex) {
+        ASSERT_EQ(ex.code(), ErrorCodes::BadValue);
+        ASSERT_EQ(ex.reason(), "Test with Status");
+    }
+    ASSERT_EQ(tripwireAssertions + 1, assertionCount.tripwire.load());
+
+    tassert(Status::OK());
+    ASSERT_EQ(tripwireAssertions + 1, assertionCount.tripwire.load());
+
+    try {
+        tassert(4457090, "Test with expression", false);
+    } catch (const DBException& ex) {
+        ASSERT_EQ(ex.code(), 4457090);
+        ASSERT_EQ(ex.reason(), "Test with expression");
+    }
+    ASSERT_EQ(tripwireAssertions + 2, assertionCount.tripwire.load());
+
+    tassert(4457091, "Another test with expression", true);
+    ASSERT_EQ(tripwireAssertions + 2, assertionCount.tripwire.load());
+
+    try {
+        tasserted(4457092, "Test with tasserted");
+    } catch (const DBException& ex) {
+        ASSERT_EQ(ex.code(), 4457092);
+        ASSERT_EQ(ex.reason(), "Test with tasserted");
+    }
+    ASSERT_EQ(tripwireAssertions + 3, assertionCount.tripwire.load());
+}
+
+DEATH_TEST_REGEX(TassertTerminationTest,
+                 tassertCleanLogMsg,
+                 "4457001.*Aborting process during exit due to prior failed tripwire assertions") {
+    doTassert();
+}
+
+DEATH_TEST_REGEX(TassertTerminationTest,
+                 tassertUncleanLogMsg,
+                 "4457002.*Detected prior failed tripwire assertions") {
+    doTassert();
+    quickExit(ExitCode::abrupt);
+}
+
+DEATH_TEST(TassertTerminationTest, mongoUnreachableNonFatal, "Hit a MONGO_UNREACHABLE_TASSERT!") {
+    try {
+        MONGO_UNREACHABLE_TASSERT(4457093);
+    } catch (const DBException&) {
+        // Catch the DBException, to ensure that we eventually abort during clean exit.
+    }
+}
+
+DEATH_TEST_REGEX(TassertTerminationTest,
+                 mongoUnimplementedNonFatal,
+                 "6634500.*Hit a MONGO_UNIMPLEMENTED_TASSERT!") {
+    try {
+        MONGO_UNIMPLEMENTED_TASSERT(6634500);
+    } catch (const DBException&) {
+        // Catch the DBException, to ensure that we eventually abort during clean exit.
+    }
+}
+
 // fassert and its friends
 DEATH_TEST(FassertionTerminationTest, fassert, "40206") {
     fassert(40206, false);
@@ -329,6 +393,10 @@ DEATH_TEST(InvariantTerminationTest, invariantOverload, "Terminating with invari
     invariant(Status(ErrorCodes::InternalError, "Terminating with invariant"));
 }
 
+DEATH_TEST(InvariantTerminationTest, mongoUnimplementedFatal, "Hit a MONGO_UNIMPLEMENTED!") {
+    MONGO_UNIMPLEMENTED;
+}
+
 DEATH_TEST(InvariantTerminationTest, invariantStatusWithOverload, "Terminating with invariant") {
     invariant(StatusWith<std::string>(ErrorCodes::InternalError,
                                       "Terminating with invariantStatusWithOverload"));
@@ -434,5 +502,41 @@ DEATH_TEST(DassertTerminationTest,
 }
 #endif  // defined(MONGO_CONFIG_DEBUG_BUILD)
 
+TEST(ScopedDebugInfo, Stack) {
+    using namespace unittest::match;
+    ScopedDebugInfoStack infoStack{};  // Avoiding the tls instance for now.
+    std::vector<std::string> expected;
+    ASSERT_THAT(infoStack.getAll(), Eq(expected));
+    {
+        ScopedDebugInfo greetingGuard("greeting", "hello", &infoStack);
+        expected.push_back("greeting: hello");
+        ASSERT_THAT(infoStack.getAll(), Eq(expected));
+
+        ScopedDebugInfo numberGuard("age", 123, &infoStack);
+        expected.push_back("age: 123");
+        ASSERT_THAT(infoStack.getAll(), Eq(expected));
+
+        {
+            ScopedDebugInfo innerGuard("inner", 222, &infoStack);
+            expected.push_back("inner: 222");
+            ASSERT_THAT(infoStack.getAll(), Eq(expected));
+            expected.pop_back();  // innerGuard
+        }
+        ASSERT_THAT(infoStack.getAll(), Eq(expected));
+        expected.pop_back();  // numberGuard
+        expected.pop_back();  // greetingGuard
+    }
+    ASSERT_THAT(infoStack.getAll(), Eq(expected));
+}
+
+void someRiskyBusiness() {
+    invariant(false, "ouch");
+}
+
+DEATH_TEST(ScopedDebugInfo, PrintedOnInvariant, "mission: ATestInjectedString") {
+    ScopedDebugInfo g("mission", "ATestInjectedString");
+    someRiskyBusiness();
+}
+
 }  // namespace
 }  // namespace mongo
