@@ -180,6 +180,16 @@ public:
         return _impl.isReady();
     }
 
+    /**
+     * Returns whether this SemiFuture can or will be able to access a deferred status or value.
+     *
+     * NOTE: valid() will still return true if the value inside of the future is moved from. This
+     * should not be used as a way to determine usage validity until SERVER-66036.
+     */
+    bool valid() const {
+        return _impl.valid();
+    }
+
     /**
      * Returns when the Semifuture isReady().
      *
@@ -249,6 +259,14 @@ public:
      */
     ExecutorFuture<T> thenRunOn(ExecutorPtr exec) && noexcept;
 
+    /**
+     * Returns an inline Future type from this SemiFuture.
+     *
+     * WARNING: Do not use this unless you're extremely sure of what you're doing, as callbacks
+     * chained to the resulting Future may run in unexpected places.
+     */
+    Future<T> unsafeToInlineFuture() && noexcept;
+
 private:
     friend class Promise<T>;
     friend class SharedPromise<T>;
@@ -260,8 +278,9 @@ private:
     friend class future_details::FutureImpl;
     template <typename>
     friend class SharedSemiFuture;
+    template <typename>
+    friend class SemiFuture;
 
-    Future<T> unsafeToInlineFuture() && noexcept;
 
     explicit SemiFuture(future_details::SharedStateHolder<T_unless_void>&& impl)
         : _impl(std::move(impl)) {}
@@ -307,6 +326,7 @@ public:
     using SemiFuture<T>::SemiFuture;  // Constructors.
     using SemiFuture<T>::share;
     using SemiFuture<T>::isReady;
+    using SemiFuture<T>::valid;
     using SemiFuture<T>::wait;
     using SemiFuture<T>::waitNoThrow;
     using SemiFuture<T>::get;
@@ -336,16 +356,24 @@ public:
     }
 
     /**
-     * This ends the Future continuation chain by calling a callback on completion. Use this to
-     * escape back into a callback-based API.
+     * Attaches a `func` callback that will consume the result of this `Future` when it completes. A
+     * `getAsync` call can be the tail end of a continuation chain, and that is only position at
+     * which it can appear.
+     *
+     * The result argument passed to `func` is `Status` if `T` is `void`, otherwise `StatusWith<T>`.
+     * The `func(result)` return type must be `void`, and not a discarded return value.
+     *
+     * `func` must not throw an exception. It is invoked as if by a `noexcept` function, and it will
+     * `std::terminate` the process. This is because there is no appropriate context in which to
+     * handle such an asynchronous exception.
      *
-     * For now, the callback must not fail, since there is nowhere to propagate the error to.
      * TODO decide how to handle func throwing.
      */
-    TEMPLATE(typename Func)
-    REQUIRES(future_details::isCallableExactR<void, Func, StatusOrStatusWith<T>>)
-    void getAsync(Func&& func) && noexcept {
-        std::move(this->_impl).getAsync(std::forward<Func>(func));
+    TEMPLATE(typename Policy, typename Func)
+    REQUIRES(future_details::isCallableExactR<void, Func, StatusOrStatusWith<T>>&&
+                 isFuturePolicy<Policy>)
+    void getAsync(Policy policy, Func&& func) && noexcept {
+        std::move(this->_impl).getAsync(policy, std::forward<Func>(func));
     }
 
     //
@@ -383,10 +411,10 @@ public:
      * The callback takes a T and can return anything (see above for how Statusy and Futurey returns
      * are handled.)
      */
-    TEMPLATE(typename Func)
-    REQUIRES(future_details::isCallable<Func, T>)
-    /*see above*/ auto then(Func&& func) && noexcept {
-        return wrap<Func, T>(std::move(this->_impl).then(std::forward<Func>(func)));
+    TEMPLATE(typename Policy, typename Func)
+    REQUIRES(future_details::isCallable<Func, T>&& isFuturePolicy<Policy>)
+    /*see above*/ auto then(Policy policy, Func&& func) && noexcept {
+        return wrap<Func, T>(std::move(this->_impl).then(policy, std::forward<Func>(func)));
     }
 
     /**
@@ -396,10 +424,11 @@ public:
      * The callback takes a StatusOrStatusWith<T> and can return anything (see above for how Statusy
      * and Futurey returns are handled.)
      */
-    TEMPLATE(typename Func)
-    REQUIRES(future_details::isCallable<Func, StatusOrStatusWith<T>>)
-    /*see above*/ auto onCompletion(Func&& func) && noexcept {
-        return wrap<Func, Status>(std::move(this->_impl).onCompletion(std::forward<Func>(func)));
+    TEMPLATE(typename Policy, typename Func)
+    REQUIRES(future_details::isCallable<Func, StatusOrStatusWith<T>>&& isFuturePolicy<Policy>)
+    /*see above*/ auto onCompletion(Policy policy, Func&& func) && noexcept {
+        return wrap<Func, Status>(
+            std::move(this->_impl).onCompletion(policy, std::forward<Func>(func)));
     }
 
     /**
@@ -415,10 +444,10 @@ public:
      * The callback takes a non-OK Status and returns a possibly-wrapped T (see above for how
      * Statusy and Futurey returns are handled.)
      */
-    TEMPLATE(typename Func)
-    REQUIRES(future_details::isCallableR<T, Func, Status>)
-    /*see above*/ auto onError(Func&& func) && noexcept {
-        return wrap<Func, Status>(std::move(this->_impl).onError(std::forward<Func>(func)));
+    TEMPLATE(typename Policy, typename Func)
+    REQUIRES(future_details::isCallableR<T, Func, Status>&& isFuturePolicy<Policy>)
+    /*see above*/ auto onError(Policy policy, Func&& func) && noexcept {
+        return wrap<Func, Status>(std::move(this->_impl).onError(policy, std::forward<Func>(func)));
     }
 
     /**
@@ -428,11 +457,11 @@ public:
      * The callback takes a non-OK Status and returns a possibly-wrapped T (see above for how
      * Statusy and Futurey returns are handled.)
      */
-    TEMPLATE(ErrorCodes::Error code, typename Func)
-    REQUIRES(future_details::isCallableR<T, Func, Status>)
-    /*see above*/ auto onError(Func&& func) && noexcept {
+    TEMPLATE(ErrorCodes::Error code, typename Policy, typename Func)
+    REQUIRES(future_details::isCallableR<T, Func, Status>&& isFuturePolicy<Policy>)
+    /*see above*/ auto onError(Policy policy, Func&& func) && noexcept {
         return wrap<Func, Status>(
-            std::move(this->_impl).template onError<code>(std::forward<Func>(func)));
+            std::move(this->_impl).template onError<code>(policy, std::forward<Func>(func)));
     }
 
     /**
@@ -442,11 +471,12 @@ public:
      * The callback takes a non-OK Status and returns a possibly-wrapped T (see above for how
      * Statusy and Futurey returns are handled.)
      */
-    TEMPLATE(ErrorCategory category, typename Func)
-    REQUIRES(future_details::isCallableR<T, Func, Status>)
-    /*see above*/ auto onErrorCategory(Func&& func) && noexcept {
+    TEMPLATE(ErrorCategory category, typename Policy, typename Func)
+    REQUIRES(future_details::isCallableR<T, Func, Status>&& isFuturePolicy<Policy>)
+    /*see above*/ auto onErrorCategory(Policy policy, Func&& func) && noexcept {
         return wrap<Func, Status>(
-            std::move(this->_impl).template onErrorCategory<category>(std::forward<Func>(func)));
+            std::move(this->_impl)
+                .template onErrorCategory<category>(policy, std::forward<Func>(func)));
     }
 
     //
@@ -469,10 +499,10 @@ public:
      *
      * The callback takes a const T& and must return void.
      */
-    TEMPLATE(typename Func)
-    REQUIRES(future_details::isCallableExactR<void, Func, const T>)
-    Future<T> tap(Func&& func) && noexcept {
-        return Future<T>(std::move(this->_impl).tap(std::forward<Func>(func)));
+    TEMPLATE(typename Policy, typename Func)
+    REQUIRES(future_details::isCallableExactR<void, Func, const T>&& isFuturePolicy<Policy>)
+    Future<T> tap(Policy policy, Func&& func) && noexcept {
+        return Future<T>(std::move(this->_impl).tap(policy, std::forward<Func>(func)));
     }
 
     /**
@@ -482,10 +512,10 @@ public:
      *
      * The callback takes a non-OK Status and must return void.
      */
-    TEMPLATE(typename Func)
-    REQUIRES(future_details::isCallableExactR<void, Func, const Status>)
-    Future<T> tapError(Func&& func) && noexcept {
-        return Future<T>(std::move(this->_impl).tapError(std::forward<Func>(func)));
+    TEMPLATE(typename Policy, typename Func)
+    REQUIRES(future_details::isCallableExactR<void, Func, const Status>&& isFuturePolicy<Policy>)
+    Future<T> tapError(Policy policy, Func&& func) && noexcept {
+        return Future<T>(std::move(this->_impl).tapError(policy, std::forward<Func>(func)));
     }
 
     /**
@@ -496,10 +526,69 @@ public:
      *
      * The callback takes a StatusOrStatusWith<T> and must return void.
      */
+    TEMPLATE(typename Policy, typename Func)
+    REQUIRES(future_details::isCallableExactR<void, Func, const StatusOrStatusWith<T>>&&
+                 isFuturePolicy<Policy>)
+    Future<T> tapAll(Policy policy, Func&& func) && noexcept {
+        return Future<T>(std::move(this->_impl).tapAll(policy, std::forward<Func>(func)));
+    }
+
+    // Calling this on a Future is never strictly necessary, since this is already an inline
+    // Future, but making this public helps for writing generic utilities which need to use
+    // unsafeToInlineFuture for some future types but not others.
+    using SemiFuture<T>::unsafeToInlineFuture;
+
+    /**
+     * Providing a Policy to continuation functions is optional. All callers should eventually
+     * use the default policy by removing the tag altogether.
+     *
+     * TODO(SERVER-66126): Remove all tag-taking methods and associated default implementations.
+     */
+    TEMPLATE(typename Func)
+    REQUIRES(future_details::isCallableExactR<void, Func, StatusOrStatusWith<T>>)
+    void getAsync(Func&& func) && noexcept {
+        std::move(*this).getAsync(destroyDefault, std::forward<Func>(func));
+    }
+    TEMPLATE(typename Func)
+    REQUIRES(future_details::isCallable<Func, T>)
+    auto then(Func&& func) && noexcept {
+        return std::move(*this).then(destroyDefault, std::forward<Func>(func));
+    }
+    TEMPLATE(typename Func)
+    REQUIRES(future_details::isCallable<Func, StatusOrStatusWith<T>>)
+    auto onCompletion(Func&& func) && noexcept {
+        return std::move(*this).onCompletion(destroyDefault, std::forward<Func>(func));
+    }
+    TEMPLATE(typename Func)
+    REQUIRES(future_details::isCallableR<T, Func, Status>)
+    auto onError(Func&& func) && noexcept {
+        return std::move(*this).onError(destroyDefault, std::forward<Func>(func));
+    }
+    TEMPLATE(ErrorCodes::Error code, typename Func)
+    REQUIRES(future_details::isCallableR<T, Func, Status>)
+    auto onError(Func&& func) && noexcept {
+        return std::move(*this).template onError<code>(destroyDefault, std::forward<Func>(func));
+    }
+    TEMPLATE(ErrorCategory category, typename Func)
+    REQUIRES(future_details::isCallableR<T, Func, Status>)
+    auto onErrorCategory(Func&& func) && noexcept {
+        return std::move(*this).template onErrorCategory<category>(destroyDefault,
+                                                                   std::forward<Func>(func));
+    }
+    TEMPLATE(typename Func)
+    REQUIRES(future_details::isCallableExactR<void, Func, const T>)
+    Future<T> tap(Func&& func) && noexcept {
+        return std::move(*this).tap(destroyDefault, std::forward<Func>(func));
+    }
+    TEMPLATE(typename Func)
+    REQUIRES(future_details::isCallableExactR<void, Func, const Status>)
+    Future<T> tapError(Func&& func) && noexcept {
+        return std::move(*this).tapError(destroyDefault, std::forward<Func>(func));
+    }
     TEMPLATE(typename Func)
     REQUIRES(future_details::isCallableExactR<void, Func, const StatusOrStatusWith<T>>)
     Future<T> tapAll(Func&& func) && noexcept {
-        return Future<T>(std::move(this->_impl).tapAll(std::forward<Func>(func)));
+        return std::move(*this).tapAll(destroyDefault, std::forward<Func>(func));
     }
 
 private:
@@ -512,7 +601,6 @@ private:
     friend class Promise<T>;
     friend class SharedPromise<T>;
 
-    using SemiFuture<T>::unsafeToInlineFuture;
 
     template <typename Func, typename Arg, typename U>
     static auto wrap(future_details::FutureImpl<U>&& impl) {
@@ -574,6 +662,7 @@ public:
     using value_type = T;
     using SemiFuture<T>::share;
     using SemiFuture<T>::isReady;
+    using SemiFuture<T>::valid;
     using SemiFuture<T>::wait;
     using SemiFuture<T>::waitNoThrow;
     using SemiFuture<T>::get;
@@ -600,15 +689,20 @@ public:
     // it should be doable, but will be fairly complicated.
     //
 
-    TEMPLATE(typename Func)
-    REQUIRES(future_details::isCallableExactR<void, Func, StatusOrStatusWith<T>>)
-    void getAsync(Func&& func) && noexcept {
+    /**
+     * Attach a completion callback to asynchronously consume this `ExecutorFuture`'s result.
+     * \see `Future<T>::getAsync()`.
+     */
+    TEMPLATE(typename Policy, typename Func)
+    REQUIRES(future_details::isCallableExactR<void, Func, StatusOrStatusWith<T>>&&
+                 isFuturePolicy<Policy>)
+    void getAsync(Policy policy, Func&& func) && noexcept {
         static_assert(std::is_void_v<decltype(func(std::declval<StatusOrStatusWith<T>>()))>,
                       "func passed to getAsync must return void");
 
         // Can't use wrapCB since we don't want to return a future, just schedule a non-chainable
         // callback.
-        std::move(this->_impl).getAsync([
+        std::move(this->_impl).getAsync(policy, [
             exec = std::move(_exec),  // Unlike wrapCB this can move because we won't need it later.
             func = std::forward<Func>(func)
         ](StatusOrStatusWith<T> arg) mutable noexcept {
@@ -620,46 +714,96 @@ public:
         });
     }
 
+    TEMPLATE(typename Policy, typename Func)
+    REQUIRES(future_details::isCallable<Func, T>&& isFuturePolicy<Policy>)
+    auto then(Policy policy, Func&& func) && noexcept {
+        return mongo::ExecutorFuture(
+            std::move(_exec),
+            std::move(this->_impl).then(policy, wrapCB<T>(policy, std::forward<Func>(func))));
+    }
+
+    TEMPLATE(typename Policy, typename Func)
+    REQUIRES(future_details::isCallable<Func, StatusOrStatusWith<T>>&& isFuturePolicy<Policy>)
+    auto onCompletion(Policy policy, Func&& func) && noexcept {
+        return mongo::ExecutorFuture(
+            std::move(_exec),
+            std::move(this->_impl)
+                .onCompletion(policy,
+                              wrapCB<StatusOrStatusWith<T>>(policy, std::forward<Func>(func))));
+    }
+
+    TEMPLATE(typename Policy, typename Func)
+    REQUIRES(future_details::isCallableR<T, Func, Status>&& isFuturePolicy<Policy>)
+    ExecutorFuture<T> onError(Policy policy, Func&& func) && noexcept {
+        return mongo::ExecutorFuture(
+            std::move(_exec),
+            std::move(this->_impl)
+                .onError(policy, wrapCB<Status>(policy, std::forward<Func>(func))));
+    }
+
+    TEMPLATE(ErrorCodes::Error code, typename Policy, typename Func)
+    REQUIRES(future_details::isCallableR<T, Func, Status>&& isFuturePolicy<Policy>)
+    ExecutorFuture<T> onError(Policy policy, Func&& func) && noexcept {
+        return mongo::ExecutorFuture(
+            std::move(_exec),
+            std::move(this->_impl)
+                .template onError<code>(policy, wrapCB<Status>(policy, std::forward<Func>(func))));
+    }
+
+    TEMPLATE(ErrorCategory category, typename Policy, typename Func)
+    REQUIRES(future_details::isCallableR<T, Func, Status>&& isFuturePolicy<Policy>)
+    ExecutorFuture<T> onErrorCategory(Policy policy, Func&& func) && noexcept {
+        return mongo::ExecutorFuture(
+            std::move(_exec),
+            std::move(this->_impl)
+                .template onErrorCategory<category>(
+                    policy, wrapCB<Status>(policy, std::forward<Func>(func))));
+    }
+
+    /**
+     * Returns an inline Future type from this ExecutorFuture.
+     *
+     * WARNING: Do not use this unless you're extremely sure of what you're doing, as callbacks
+     * chained to the resulting Future may run in unexpected places.
+     */
+    using SemiFuture<T>::unsafeToInlineFuture;
+
+    /**
+     * Providing a Policy to continuation functions is optional. All callers should eventually
+     * use the default policy by removing the tag altogether.
+     *
+     * TODO(SERVER-66126): Remove all tag-taking methods and associated default implementations.
+     */
+    TEMPLATE(typename Func)
+    REQUIRES(future_details::isCallableExactR<void, Func, StatusOrStatusWith<T>>)
+    void getAsync(Func&& func) && noexcept {
+        std::move(*this).getAsync(destroyDefault, std::forward<Func>(func));
+    }
     TEMPLATE(typename Func)
     REQUIRES(future_details::isCallable<Func, T>)
     auto then(Func&& func) && noexcept {
-        return mongo::ExecutorFuture(
-            std::move(_exec), std::move(this->_impl).then(wrapCB<T>(std::forward<Func>(func))));
+        return std::move(*this).then(destroyDefault, std::forward<Func>(func));
     }
-
     TEMPLATE(typename Func)
     REQUIRES(future_details::isCallable<Func, StatusOrStatusWith<T>>)
     auto onCompletion(Func&& func) && noexcept {
-        return mongo::ExecutorFuture(
-            std::move(_exec),
-            std::move(this->_impl)
-                .onCompletion(wrapCB<StatusOrStatusWith<T>>(std::forward<Func>(func))));
+        return std::move(*this).onCompletion(destroyDefault, std::forward<Func>(func));
     }
-
     TEMPLATE(typename Func)
     REQUIRES(future_details::isCallableR<T, Func, Status>)
-    ExecutorFuture<T> onError(Func&& func) && noexcept {
-        return mongo::ExecutorFuture(
-            std::move(_exec),
-            std::move(this->_impl).onError(wrapCB<Status>(std::forward<Func>(func))));
+    auto onError(Func&& func) && noexcept {
+        return std::move(*this).onError(destroyDefault, std::forward<Func>(func));
     }
-
     TEMPLATE(ErrorCodes::Error code, typename Func)
     REQUIRES(future_details::isCallableR<T, Func, Status>)
-    ExecutorFuture<T> onError(Func&& func) && noexcept {
-        return mongo::ExecutorFuture(
-            std::move(_exec),
-            std::move(this->_impl)
-                .template onError<code>(wrapCB<Status>(std::forward<Func>(func))));
+    auto onError(Func&& func) && noexcept {
+        return std::move(*this).template onError<code>(destroyDefault, std::forward<Func>(func));
     }
-
     TEMPLATE(ErrorCategory category, typename Func)
     REQUIRES(future_details::isCallableR<T, Func, Status>)
-    ExecutorFuture<T> onErrorCategory(Func&& func) && noexcept {
-        return mongo::ExecutorFuture(
-            std::move(_exec),
-            std::move(this->_impl)
-                .template onErrorCategory<category>(wrapCB<Status>(std::forward<Func>(func))));
+    auto onErrorCategory(Func&& func) && noexcept {
+        return std::move(*this).template onErrorCategory<category>(destroyDefault,
+                                                                   std::forward<Func>(func));
     }
 
 private:
@@ -673,8 +817,9 @@ private:
      * Future<U>, then schedules a task on _exec to complete the associated promise with the result
      * of calling func with that argument.
      */
-    template <typename RawArg, typename Func>
-    auto wrapCB(Func&& func) {
+    TEMPLATE(typename RawArg, typename Policy, typename Func)
+    REQUIRES(isFuturePolicy<Policy>)
+    auto wrapCB(Policy policy, Func&& func) {
         // Have to take care to never put void in argument position, since that is a hard error.
         using Result = typename std::conditional_t<std::is_void_v<RawArg>,
                                                    std::invoke_result<Func>,
@@ -685,13 +830,11 @@ private:
         using Sig = std::conditional_t<std::is_void_v<RawArg>,  //
                                        Result(),
                                        Result(DummyArg)>;
-        return wrapCBHelper(unique_function<Sig>(std::forward<Func>(func)));
+        return _wrapCBHelper(_exec, unique_function<Sig>(std::forward<Func>(func)));
     }
 
-    template <typename Sig>
-    MONGO_COMPILER_NOINLINE auto wrapCBHelper(unique_function<Sig>&& func);
-
-    using SemiFuture<T>::unsafeToInlineFuture;
+    template <typename UniqueFunc>
+    MONGO_COMPILER_NOINLINE static auto _wrapCBHelper(ExecutorPtr exec, UniqueFunc&& func);
 
     template <typename>
     friend class ExecutorFuture;
@@ -715,6 +858,8 @@ template <typename T>
 ExecutorFuture(ExecutorPtr, StatusWith<T>)->ExecutorFuture<T>;
 ExecutorFuture(ExecutorPtr)->ExecutorFuture<void>;
 
+/** Constructor tag (see the corresponding `Promise` constructor). */
+struct NonNullPromiseTag {};
 
 /**
  * This class represents the producer side of a Future.
@@ -737,6 +882,7 @@ ExecutorFuture(ExecutorPtr)->ExecutorFuture<void>;
 template <typename T>
 class Promise {
     using SharedStateT = future_details::SharedState<T>;
+    using T_unless_void = typename SemiFuture<T>::T_unless_void;
 
 public:
     using value_type = T;
@@ -746,6 +892,9 @@ public:
      */
     Promise() = default;
 
+    /** Makes a `Promise` that has a `Future`. */
+    explicit Promise(NonNullPromiseTag) : Promise{make_intrusive<SharedStateT>()} {}
+
     ~Promise() {
         breakPromiseIfNeeded();
     }
@@ -780,10 +929,15 @@ public:
      * because this method will correctly propagate errors thrown from makeResult(), rather than
      * ErrorCodes::BrokenPromise.
      */
+    TEMPLATE(typename Policy, typename Func)
+    REQUIRES(future_details::isCallableR<T, Func, void>&& isFuturePolicy<Policy>)
+    void setWith(Policy policy, Func&& func) noexcept {
+        setFrom(Future<void>::makeReady().then(policy, std::forward<Func>(func)));
+    }
     TEMPLATE(typename Func)
     REQUIRES(future_details::isCallableR<T, Func, void>)
     void setWith(Func&& func) noexcept {
-        setFrom(Future<void>::makeReady().then(std::forward<Func>(func)));
+        setWith(destroyDefault, std::forward<Func>(func));
     }
 
     /**
@@ -792,11 +946,30 @@ public:
      * involved.
      */
     void setFrom(Future<T>&& future) noexcept {
-        setImpl([&](boost::intrusive_ptr<future_details::SharedState<T>>&& sharedState) {
+        setImpl([&](boost::intrusive_ptr<SharedStateT>&& sharedState) {
             std::move(future).propagateResultTo(sharedState.get());
         });
     }
 
+    /**
+     * Sets the value into this Promise immediately.
+     *
+     * This accepts a Status for Promises<void> or a StatusWith<T> for Promise<T>.
+     */
+    void setFrom(StatusOrStatusWith<T> sosw) noexcept {
+        setImpl([&](boost::intrusive_ptr<SharedStateT>&& sharedState) {
+            sharedState->setFrom(std::move(sosw));
+        });
+    }
+
+    // Use emplaceValue(Args&&...) instead.
+    REQUIRES_FOR_NON_TEMPLATE(!std::is_void_v<T>)
+    void setFrom(T_unless_void val) noexcept = delete;
+
+    // Use setError(Status) instead.
+    REQUIRES_FOR_NON_TEMPLATE(!std::is_void_v<T>)
+    void setFrom(Status) noexcept = delete;
+
     TEMPLATE(typename... Args)
     REQUIRES(std::is_constructible_v<T, Args...> || (std::is_void_v<T> && sizeof...(Args) == 0))
     void emplaceValue(Args&&... args) noexcept {
@@ -812,34 +985,29 @@ public:
         });
     }
 
-    // TODO rename to not XXXWith and handle void
-    void setFromStatusWith(StatusWith<T> sw) noexcept {
-        setImpl([&](boost::intrusive_ptr<SharedStateT>&& sharedState) {
-            sharedState->setFromStatusWith(std::move(sw));
-        });
-    }
-
-    static auto makePromiseFutureImpl() {
-        struct PromiseAndFuture {
-            Promise<T> promise = Promise(make_intrusive<SharedStateT>());
-            Future<T> future = promise.getFuture();
-        };
-        return PromiseAndFuture();
+    /**
+     * Create and return the Future corresponding to this Promise. Cannot be
+     * called more than once (but see `SharedPromise` which supports multiple
+     * `getFuture` calls).
+     *
+     * Caution: calling `getFuture` is only valid before the promise is
+     * completed, because a Promise is nullified upon completion. This Promise
+     * behavior is different from the semantics of other future libraries. The
+     * PromiseAndFuture helper avoids a potential data race by eagerly calling
+     * `getFuture`, returning the Promise together with its Future.
+     */
+    Future<T> getFuture() noexcept {
+        // Copy `_sharedState` into a SharedStateHolder to make a Future,
+        // exploiting the knowledge that this Promise is its sole owner.
+        _sharedState->threadUnsafeIncRefCountTo(2);
+        return Future<T>(future_details::SharedStateHolder<future_details::VoidToFakeVoid<T>>(
+            boost::intrusive_ptr<SharedStateT>(_sharedState.get(), false)));
     }
 
 private:
     explicit Promise(boost::intrusive_ptr<SharedStateT>&& sharedState)
         : _sharedState(std::move(sharedState)) {}
 
-    // This is not public because we found it frequently was involved in races.  The
-    // `makePromiseFuture<T>` API avoids those races entirely.
-    Future<T> getFuture() noexcept {
-        using namespace future_details;
-        _sharedState->threadUnsafeIncRefCountTo(2);
-        return Future<T>(SharedStateHolder<VoidToFakeVoid<T>>(
-            boost::intrusive_ptr<SharedState<T>>(_sharedState.get(), /*add ref*/ false)));
-    }
-
     friend class Future<void>;
 
     template <typename Func>
@@ -920,6 +1088,10 @@ public:
         return _shared.isReady();
     }
 
+    bool valid() const {
+        return _shared.valid();
+    }
+
     void wait(Interruptible* interruptible = Interruptible::notInterruptible()) const {
         _shared.wait(interruptible);
     }
@@ -943,10 +1115,27 @@ public:
         return ExecutorFuture<T>(std::move(exec), toFutureImpl());
     }
 
+    /**
+     * Makes a copy of this SharedSemiFuture that resolves at the same time as the original.
+     */
+    SharedSemiFuture split() const noexcept {
+        return toFutureImpl().share();
+    }
+
     SemiFuture<T> semi() && noexcept {
         return SemiFuture<T>(toFutureImpl());
     }
 
+    /**
+     * Returns an inline Future type from this SharedSemiFuture.
+     *
+     * WARNING: Do not use this unless you're extremely sure of what you're doing, as callbacks
+     * chained to the resulting Future may run in unexpected places.
+     */
+    Future<T> unsafeToInlineFuture() const noexcept {
+        return Future<T>(toFutureImpl());
+    }
+
 private:
     template <typename>
     friend class SharedPromise;
@@ -970,10 +1159,6 @@ private:
     void propagateResultTo(U&& arg) const noexcept {
         toFutureImpl().propagateResultTo(std::forward<U>(arg));
     }
-    Future<T> unsafeToInlineFuture() const noexcept {
-        return Future<T>(toFutureImpl());
-    }
-
     explicit SharedSemiFuture(boost::intrusive_ptr<future_details::SharedState<T>> ptr)
         : _shared(std::move(ptr)) {}
     explicit SharedSemiFuture(future_details::SharedStateHolder<T>&& holder)
@@ -1007,6 +1192,9 @@ SharedSemiFuture(StatusWith<T>)->SharedSemiFuture<T>;
  */
 template <typename T>
 class SharedPromise {
+    using SharedStateT = future_details::SharedState<T>;
+    using T_unless_void = std::conditional_t<std::is_void_v<T>, future_details::FakeVoid, T>;
+
 public:
     using value_type = T;
 
@@ -1034,17 +1222,44 @@ public:
         return SharedSemiFuture<T>(_sharedState);
     }
 
+    TEMPLATE(typename Policy, typename Func)
+    REQUIRES(future_details::isCallableR<T, Func, void>&& isFuturePolicy<Policy>)
+    void setWith(Policy policy, Func&& func) noexcept {
+        setFrom(Future<void>::makeReady().then(policy, std::forward<Func>(func)));
+    }
     TEMPLATE(typename Func)
     REQUIRES(future_details::isCallableR<T, Func, void>)
     void setWith(Func&& func) noexcept {
-        setFrom(Future<void>::makeReady().then(std::forward<Func>(func)));
+        setWith(destroyDefault, std::forward<Func>(func));
     }
 
+    /**
+     * Sets the value into this SharedPromise when the passed-in Future completes, which may have
+     * already happened.
+     */
     void setFrom(Future<T>&& future) noexcept {
         invariant(!std::exchange(_haveCompleted, true));
         std::move(future).propagateResultTo(_sharedState.get());
     }
 
+    /**
+     * Sets the value into this SharedPromise immediately.
+     *
+     * This accepts a Status for SharedPromises<void> or a StatusWith<T> for SharedPromise<T>.
+     */
+    void setFrom(StatusOrStatusWith<T> sosw) noexcept {
+        invariant(!std::exchange(_haveCompleted, true));
+        _sharedState->setFrom(std::move(sosw));
+    }
+
+    // Use emplaceValue(Args&&...) instead.
+    REQUIRES_FOR_NON_TEMPLATE(!std::is_void_v<T>)
+    void setFrom(T_unless_void val) noexcept = delete;
+
+    // Use setError(Status) instead.
+    REQUIRES_FOR_NON_TEMPLATE(!std::is_void_v<T>)
+    void setFrom(Status) noexcept = delete;
+
     TEMPLATE(typename... Args)
     REQUIRES(std::is_constructible_v<T, Args...> || (std::is_void_v<T> && sizeof...(Args) == 0))
     void emplaceValue(Args&&... args) noexcept {
@@ -1058,40 +1273,29 @@ public:
         _sharedState->setError(std::move(status));
     }
 
-    // TODO rename to not XXXWith and handle void
-    void setFromStatusWith(StatusWith<T> sw) noexcept {
-        invariant(!std::exchange(_haveCompleted, true));
-        _sharedState->setFromStatusWith(std::move(sw));
-    }
-
 private:
     friend class Future<void>;
 
     // This is slightly different from whether the SharedState is in kFinished, because this
     // SharedPromise may have been completed with a Future that isn't ready yet.
     bool _haveCompleted = false;
-    const boost::intrusive_ptr<future_details::SharedState<T>> _sharedState =
-        make_intrusive<future_details::SharedState<T>>();
+    const boost::intrusive_ptr<SharedStateT> _sharedState = make_intrusive<SharedStateT>();
 };
 
 /**
- * Makes a ready Future with the return value of a nullary function. This has the same semantics as
- * Promise::setWith, and has the same reasons to prefer it over Future<T>::makeReady(). Also, it
- * deduces the T, so it is easier to use.
- */
-TEMPLATE(typename Func)
-REQUIRES(future_details::isCallable<Func, void>)
-auto makeReadyFutureWith(Func&& func) {
-    return Future<void>::makeReady().then(std::forward<Func>(func));
-}
-
-/**
- * Returns a bound Promise and Future in a struct with friendly names (promise and future) that also
- * works well with C++17 structured bindings.
+ * Holds a Promise and its corresponding Future.
+ * Upon construction, contains a new Promise and its Future.
  */
 template <typename T>
-inline auto makePromiseFuture() {
-    return Promise<T>::makePromiseFutureImpl();
+struct PromiseAndFuture {
+    Promise<T> promise{NonNullPromiseTag{}};
+    Future<T> future{promise.getFuture()};
+};
+
+/** Another way to write `PromiseAndFuture<T>{}`. */
+template <typename T>
+PromiseAndFuture<T> makePromiseFuture() {
+    return {};
 }
 
 /**
@@ -1120,49 +1324,72 @@ template <typename Func, typename... Args>
 using FutureContinuationResult =
     future_details::UnwrappedType<std::invoke_result_t<Func, Args&&...>>;
 
+/**
+ * This type transform is useful for coercing (T,StatusWith<T>)->Future<T> and Status->Future<void>.
+ */
+template <typename T>
+using FutureFor = Future<future_details::UnwrappedType<T>>;
+
+template <typename T>
+auto coerceToFuture(T&& value) {
+    return FutureFor<T>(std::forward<T>(value));
+}
+
+/**
+ * Makes a Future with the return value of a nullary function. This has the same semantics as
+ * Promise::setWith, and has the same reasons to prefer it over Future<T>::makeReady(). Also, it
+ * deduces the T, so it is easier to use.
+ *
+ * Note that if func returns an unready Future, this function will not block until it is ready.
+ */
+TEMPLATE(typename Func)
+REQUIRES(future_details::isCallable<Func, void>)
+auto makeReadyFutureWith(Func&& func) -> Future<FutureContinuationResult<Func&&>> try {
+    if constexpr (std::is_void_v<std::invoke_result_t<Func>>) {
+        std::forward<Func>(func)();
+        return Future<void>::makeReady();
+    } else {
+        return std::forward<Func>(func)();
+    }
+} catch (const DBException& ex) {
+    return ex.toStatus();
+}
+
 //
 // Implementations of methods that couldn't be defined in the class due to ordering requirements.
 //
 
 template <typename T>
-template <typename Sig>
-MONGO_COMPILER_NOINLINE auto ExecutorFuture<T>::wrapCBHelper(unique_function<Sig>&& func) {
-    using namespace future_details;
-    return [
-        func = std::move(func),
-        exec = _exec  // can't move this!
-    ](auto&&... args) mutable noexcept
-        ->Future<UnwrappedType<decltype(func(std::forward<decltype(args)>(args)...))>> {
-        auto [promise, future] = makePromiseFuture<
-            UnwrappedType<decltype(func(std::forward<decltype(args)>(args)...))>>();
+template <typename UniqueFunc>
+auto ExecutorFuture<T>::_wrapCBHelper(ExecutorPtr exec, UniqueFunc&& func) {
+    return [ exec = std::move(exec), func = std::move(func) ](auto&&... args) mutable noexcept {
+        using FuncR = typename UniqueFunc::result_type;
+        using BoundArgs = std::tuple<std::decay_t<decltype(args)>...>;
+        Promise<future_details::UnwrappedType<FuncR>> promise{NonNullPromiseTag{}};
+        auto future = promise.getFuture();
 
         exec->schedule([
             promise = std::move(promise),
             func = std::move(func),
-            argsT =
-                std::tuple<std::decay_t<decltype(args)>...>(std::forward<decltype(args)>(args)...)
+            boundArgs = BoundArgs{std::forward<decltype(args)>(args)...}
         ](Status execStatus) mutable noexcept {
-            if (execStatus.isOK()) {
-                promise.setWith([&] {
-                    return [&](auto nullary) {
-                        // Using a lambda taking a nullary lambda here to work around an MSVC2017
-                        // bug that caused it to not ignore the other side of the constexpr-if.
-                        // TODO Make this less silly once we upgrade to 2019.
-                        if constexpr (!isFutureLike<decltype(nullary())>) {
-                            return nullary();
-                        } else {
-                            // Cheat and convert to an inline Future since we know we will schedule
-                            // further user callbacks onto an executor.
-                            return nullary().unsafeToInlineFuture();
-                        }
-                    }([&] { return std::apply(func, std::move(argsT)); });
-                });
-            } else {
+            if (!execStatus.isOK()) {
                 promise.setError(std::move(execStatus));
+                return;
             }
+            promise.setWith([&] {
+                auto closure = [&] { return std::apply(func, std::move(boundArgs)); };
+                if constexpr (future_details::isFutureLike<FuncR>) {
+                    // Cheat and convert to an inline Future since we know we will schedule
+                    // further user callbacks onto an executor.
+                    return closure().unsafeToInlineFuture();
+                } else {
+                    return closure();
+                }
+            });
         });
 
-        return std::move(future);
+        return future;
     };
 }
 
