@@ -41,6 +41,16 @@
 
 namespace mongo {
 
+namespace interruptible_detail {
+// Helper to release a lock, call a callable, and then reacquire the lock.
+template <typename Callable>
+auto doWithoutLock(BasicLockableAdapter m, Callable&& callable) {
+    m.unlock();
+    ON_BLOCK_EXIT([&] { m.lock(); });
+    return callable();
+}
+}  // namespace interruptible_detail
+
 /**
  * A type which can be used to wait on condition variables with a level triggered one-way interrupt.
  * I.e. after the interrupt is triggered (via some non-public api call) subsequent calls to
@@ -317,7 +327,7 @@ public:
      * Raises a AssertionException if this operation is in a killed state.
      */
     void checkForInterrupt() {
-        internalAssert(checkForInterruptNoAssert());
+        iassert(checkForInterruptNoAssert());
     }
 
     /**
@@ -353,16 +363,34 @@ public:
         WaitContext waitContext(this);
         auto latchName = getLatchName(m);
 
+        auto handleInterruptAndAssert = [&](Status status, WakeSpeed speed) {
+            _onWake(latchName, WakeReason::kInterrupt, speed);
+            iassert(std::move(status));
+        };
+
+        auto checkForInterruptWithoutLockAndAssert = [&](WakeSpeed speed) {
+            // We drop the lock before checking for interrupt since checkForInterruptNoAssert can
+            // sometimes try to reacquire the same lock.
+            if (auto status = interruptible_detail::doWithoutLock(
+                    m, [&] { return checkForInterruptNoAssert(); });
+                !status.isOK()) {
+                handleInterruptAndAssert(status, speed);
+            }
+        };
+
         auto waitUntil = [&](Date_t deadline, WakeSpeed speed) -> boost::optional<WakeReason> {
             // If the result of waitForConditionOrInterruptNoAssertUntil() is non-spurious, return
             // a WakeReason. Otherwise, return boost::none
 
             auto swResult = waitForConditionOrInterruptNoAssertUntil(cv, m, deadline);
             if (!swResult.isOK()) {
-                _onWake(latchName, WakeReason::kInterrupt, speed);
-                internalAssert(std::move(swResult));
+                handleInterruptAndAssert(swResult.getStatus(), speed);
             }
 
+            // Check if an interrupt occurred while waiting.
+            checkForInterruptWithoutLockAndAssert(speed);
+
+            // Check the predicate after re-acquiring the lock.
             if (pred()) {
                 _onWake(latchName, WakeReason::kPredicate, speed);
                 return WakeReason::kPredicate;
@@ -377,14 +405,16 @@ public:
         };
 
         auto waitUntilNonSpurious = [&](Date_t deadline, WakeSpeed speed) -> WakeReason {
-            // Check waitUntil() in a loop until it says it has a genuine WakeReason
+            // Check for interrupt before waiting.
+            checkForInterruptWithoutLockAndAssert(speed);
 
+            // Check the predicate after re-acquiring the lock and before waiting.
             if (pred()) {
-                // Check for the predicate first, just in case
                 _onWake(latchName, WakeReason::kPredicate, speed);
                 return WakeReason::kPredicate;
             }
 
+            // Check waitUntil() in a loop until it says it has a genuine WakeReason
             auto maybeWakeReason = waitUntil(deadline, speed);
             while (!maybeWakeReason) {
                 maybeWakeReason = waitUntil(deadline, speed);
@@ -542,7 +572,15 @@ class Interruptible::NotInterruptible final : public Interruptible {
             return stdx::cv_status::no_timeout;
         }
 
-        return cv.wait_until(m, deadline.toSystemTimePoint());
+        try {
+            // If the system clock's time_point's compiler-dependent resolution is higher than
+            // Date_t's milliseconds, it's possible for the conversion from Date_t to time_point
+            // to overflow and trigger an exception. We catch that here to maintain the noexcept
+            // contract.
+            return cv.wait_until(m, deadline.toSystemTimePoint());
+        } catch (const ExceptionFor<ErrorCodes::DurationOverflow>& ex) {
+            return ex.toStatus();
+        }
     }
 
     Date_t getDeadline() const override {
