@@ -27,13 +27,13 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/util/net/sock.h"
 
 #include <algorithm>
+#include <fmt/format.h>
 
 #if !defined(_WIN32)
 #include <arpa/inet.h>
@@ -69,6 +69,9 @@
 #include "mongo/util/str.h"
 #include "mongo/util/winutil.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
+
+
 namespace mongo {
 
 using std::pair;
@@ -95,13 +98,9 @@ bool setBlock(int fd, bool block) {
 #endif
 }
 
-void networkWarnWithDescription(const Socket& socket, StringData call, int errorCode = -1) {
-#ifdef _WIN32
-    if (errorCode == -1) {
-        errorCode = WSAGetLastError();
-    }
-#endif
-    auto ewd = errnoWithDescription(errorCode);
+void networkWarnWithDescription(const Socket& socket,
+                                StringData call,
+                                std::error_code ec = lastSocketError()) {
     LOGV2_WARNING(23190,
                   "failed to connect to {remoteSocketAddress}:{remoteSocketAddressPort}, "
                   "in({call}), reason: {error}",
@@ -109,29 +108,34 @@ void networkWarnWithDescription(const Socket& socket, StringData call, int error
                   "remoteSocketAddress"_attr = socket.remoteAddr().getAddr(),
                   "remoteSocketAddressPort"_attr = socket.remoteAddr().getPort(),
                   "call"_attr = call,
-                  "error"_attr = ewd);
+                  "error"_attr = errorMessage(ec));
 }
 
 const double kMaxConnectTimeoutMS = 5000;
 
 void setSockTimeouts(int sock, double secs) {
-    bool report = shouldLog(logv2::LogSeverity::Debug(4)) || kDebugBuild;
+    bool report =
+        shouldLog(MONGO_LOGV2_DEFAULT_COMPONENT, logv2::LogSeverity::Debug(4)) || kDebugBuild;
 #if defined(_WIN32)
     DWORD timeout = secs * 1000;  // Windows timeout is a DWORD, in milliseconds.
     int status =
         setsockopt(sock, SOL_SOCKET, SO_RCVTIMEO, reinterpret_cast<char*>(&timeout), sizeof(DWORD));
-    if (report && (status == SOCKET_ERROR))
+    if (report && (status == SOCKET_ERROR)) {
+        auto ec = lastSocketError();
         LOGV2(23177,
               "unable to set SO_RCVTIMEO: {reason}",
               "Unable to set SO_RCVTIMEO",
-              "reason"_attr = errnoWithDescription(WSAGetLastError()));
+              "reason"_attr = errorMessage(ec));
+    }
     status =
         setsockopt(sock, SOL_SOCKET, SO_SNDTIMEO, reinterpret_cast<char*>(&timeout), sizeof(DWORD));
-    if (kDebugBuild && report && (status == SOCKET_ERROR))
+    if (kDebugBuild && report && (status == SOCKET_ERROR)) {
+        auto ec = lastSocketError();
         LOGV2(23178,
               "unable to set SO_SNDTIMEO: {reason}",
               "Unable to set SO_SNDTIME0",
-              "reason"_attr = errnoWithDescription(WSAGetLastError()));
+              "reason"_attr = errorMessage(ec));
+    }
 #else
     struct timeval tv;
     tv.tv_sec = (int)secs;
@@ -155,41 +159,37 @@ void disableNagle(int sock) {
     const int level = SOL_SOCKET;
 #endif
 
-    if (setsockopt(sock, level, TCP_NODELAY, (char*)&x, sizeof(x)))
+    if (setsockopt(sock, level, TCP_NODELAY, (char*)&x, sizeof(x))) {
+        auto ec = lastSocketError();
         LOGV2_ERROR(23195,
                     "disableNagle failed: {error}",
                     "DisableNagle failed",
-                    "error"_attr = errnoWithDescription());
+                    "error"_attr = errorMessage(ec));
+    }
 
 #ifdef SO_KEEPALIVE
-    if (setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, (char*)&x, sizeof(x)))
+    if (setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, (char*)&x, sizeof(x))) {
+        auto ec = lastSocketError();
         LOGV2_ERROR(23196,
                     "SO_KEEPALIVE failed: {error}",
                     "SO_KEEPALIVE failed",
-                    "error"_attr = errnoWithDescription());
+                    "error"_attr = errorMessage(ec));
+    }
 #endif
 
-    setSocketKeepAliveParams(sock);
-}
-
-int socketGetLastError() {
-#ifdef _WIN32
-    return WSAGetLastError();
-#else
-    return errno;
-#endif
+    setSocketKeepAliveParams(sock, logv2::LogSeverity::Error());
 }
 
 SockAddr getLocalAddrForBoundSocketFd(int fd) {
     SockAddr result;
-    int rc = getsockname(fd, result.raw(), &result.addressSize);
-    if (rc != 0) {
+    if (getsockname(fd, result.raw(), &result.addressSize)) {
+        auto ec = lastSocketError();
         LOGV2_WARNING(23191,
                       "Could not resolve local address for socket with fd {fd}: "
                       "{error}",
                       "Could not resolve local address for socket with fd",
                       "fd"_attr = fd,
-                      "error"_attr = getAddrInfoStrError(socketGetLastError()));
+                      "error"_attr = errorMessage(ec));
         result = SockAddr();
     }
     return result;
@@ -321,14 +321,15 @@ bool Socket::connect(const SockAddr& remote, Milliseconds connectTimeoutMillis)
     bool connectSucceeded = ::connect(_fd, _remote.raw(), _remote.addressSize) == 0;
 
     if (!connectSucceeded) {
+        auto ec = lastSocketError();
 #ifdef _WIN32
-        if (WSAGetLastError() != WSAEWOULDBLOCK) {
-            networkWarnWithDescription(*this, "connect");
+        if (ec != systemError(WSAEWOULDBLOCK)) {
+            networkWarnWithDescription(*this, "connect", ec);
             return false;
         }
 #else
-        if (errno != EINTR && errno != EINPROGRESS) {
-            networkWarnWithDescription(*this, "connect");
+        if (ec != posixError(EINTR) && ec != posixError(EINPROGRESS)) {
+            networkWarnWithDescription(*this, "connect", ec);
             return false;
         }
 #endif
@@ -378,7 +379,8 @@ bool Socket::connect(const SockAddr& remote, Milliseconds connectTimeoutMillis)
                 return false;
             }
             if (optVal != 0) {
-                networkWarnWithDescription(*this, "checking socket for error after poll", optVal);
+                networkWarnWithDescription(
+                    *this, "checking socket for error after poll", systemError(optVal));
                 return false;
             }
 
@@ -568,13 +570,15 @@ int Socket::_recv(char* buf, int max) {
 }
 
 void Socket::handleSendError(int ret, const char* context) {
+    const auto ec = lastSocketError();
+    auto isTimeoutCode = [](std::error_code e) {
 #if defined(_WIN32)
-    const int mongo_errno = WSAGetLastError();
-    if (mongo_errno == WSAETIMEDOUT && _timeout != 0) {
+        return e == systemError(WSAETIMEDOUT);
 #else
-    const int mongo_errno = errno;
-    if ((mongo_errno == EAGAIN || mongo_errno == EWOULDBLOCK) && _timeout != 0) {
+        return e == posixError(EAGAIN) || e == posixError(EWOULDBLOCK);
 #endif
+    };
+    if (isTimeoutCode(ec) && _timeout != 0) {
         LOGV2_DEBUG(23181,
                     _logLevel.toInt(),
                     "Socket {context} send() timed out {remoteHost}",
@@ -582,13 +586,13 @@ void Socket::handleSendError(int ret, const char* context) {
                     "context"_attr = context,
                     "remoteHost"_attr = remoteString());
         throwSocketError(SocketErrorKind::SEND_TIMEOUT, remoteString());
-    } else if (mongo_errno != EINTR) {
+    } else if (ec != posixError(EINTR)) {
         LOGV2_DEBUG(23182,
                     _logLevel.toInt(),
                     "Socket {context} send() {error} {remoteHost}",
                     "Socket send() to remote host failed",
                     "context"_attr = context,
-                    "error"_attr = errnoWithDescription(mongo_errno),
+                    "error"_attr = errorMessage(ec),
                     "remoteHost"_attr = remoteString());
         throwSocketError(SocketErrorKind::SEND_ERROR, remoteString());
     }
@@ -604,24 +608,20 @@ void Socket::handleRecvError(int ret, int len) {
         throwSocketError(SocketErrorKind::CLOSED, remoteString());
     }
 
-// ret < 0
-#if defined(_WIN32)
-    int e = WSAGetLastError();
-#else
-    int e = errno;
+    // ret < 0
+    auto ec = lastSocketError();
 #if defined(EINTR)
-    if (e == EINTR) {
+    if (ec == posixError(EINTR)) {
         return;
     }
 #endif
-#endif
 
+    if ((ec == posixError(EAGAIN)
 #if defined(_WIN32)
-    // Windows
-    if ((e == EAGAIN || e == WSAETIMEDOUT) && _timeout > 0) {
-#else
-    if (e == EAGAIN && _timeout > 0) {
+         || ec == systemError(WSAETIMEDOUT)
 #endif
+             ) &&
+        _timeout > 0) {
         // this is a timeout
         LOGV2_DEBUG(23184,
                     _logLevel.toInt(),
@@ -635,7 +635,7 @@ void Socket::handleRecvError(int ret, int len) {
                 _logLevel.toInt(),
                 "Socket recv() {error} {remoteHost}",
                 "Socket recv() error",
-                "error"_attr = errnoWithDescription(e),
+                "error"_attr = errorMessage(ec),
                 "remoteHost"_attr = remoteString());
     throwSocketError(SocketErrorKind::RECV_ERROR, remoteString());
 }
@@ -687,6 +687,7 @@ bool Socket::isStillConnected() {
     // Poll( info[], size, timeout ) - timeout == 0 => nonblocking
     int nEvents = socketPoll(&pollInfo, 1, 0);
 
+    auto ec = lastSocketError();
     LOGV2_DEBUG(
         23186,
         2,
@@ -707,7 +708,7 @@ bool Socket::isStillConnected() {
                       "Socket poll() to remote host failed during connectivity check",
                       "idleTimeSecs"_attr = idleTimeSecs,
                       "remoteHost"_attr = remoteString(),
-                      "error"_attr = causedBy(errnoWithDescription()));
+                      "error"_attr = causedBy(errorMessage(ec)));
 
         // Return true since it's not clear that we're disconnected.
         return true;
@@ -729,6 +730,7 @@ bool Socket::isStillConnected() {
         int recvd = ::recv(_fd, testBuf, testBufLength, portRecvFlags);
 
         if (recvd < 0) {
+            auto ec = lastSocketError();
             // An error occurred during recv, warn and log errno
             LOGV2_WARNING(23194,
                           "Socket recv() failed during connectivity check (idle {idleTimeSecs} "
@@ -736,7 +738,7 @@ bool Socket::isStillConnected() {
                           "Socket recv() failed during connectivity check",
                           "idleTimeSecs"_attr = idleTimeSecs,
                           "remoteHost"_attr = remoteString(),
-                          "error"_attr = causedBy(errnoWithDescription()));
+                          "error"_attr = causedBy(errorMessage(ec)));
         } else if (recvd > 0) {
             // We got nonzero data from this socket, very weird?
             // Log and warn at runtime, log and abort at devtime
@@ -750,11 +752,10 @@ bool Socket::isStillConnected() {
                         "idleTimeSecs"_attr = idleTimeSecs,
                         "remoteHost"_attr = remoteString());
             if (kDebugBuild) {
-                std::string hex = hexdump(testBuf, recvd);
                 LOGV2_ERROR(23198,
                             "Hex dump of stale log data: {hex}",
                             "Hex dump of stale log data",
-                            "hex"_attr = hex);
+                            "hex"_attr = hexdump(testBuf, recvd));
             }
             dassert(false);
         } else {
