@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
 #include "mongo/platform/basic.h"
 
@@ -38,13 +37,13 @@
 
 #include "mongo/base/checked_cast.h"
 #include "mongo/base/init.h"
-#include "mongo/base/initializer_context.h"
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
 #include "mongo/crypto/sha1_block.h"
 #include "mongo/crypto/sha256_block.h"
 #include "mongo/logv2/log.h"
 #include "mongo/platform/random.h"
+#include "mongo/transport/ssl_connection_context.h"
 #include "mongo/util/base64.h"
 #include "mongo/util/concurrency/mutex.h"
 #include "mongo/util/fail_point.h"
@@ -55,6 +54,10 @@
 #include "mongo/util/net/ssl_manager.h"
 #include "mongo/util/net/ssl_options.h"
 #include "mongo/util/net/ssl_parameters_gen.h"
+#include "mongo/util/net/ssl_peer_info.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
+
 
 using asio::ssl::apple::CFUniquePtr;
 
@@ -69,6 +72,8 @@ extern "C" SecIdentityRef SecIdentityCreate(CFAllocatorRef, SecCertificateRef, S
 
 namespace mongo {
 
+using transport::SSLConnectionContext;
+
 namespace {
 
 // This failpoint is a no-op on OSX.
@@ -332,15 +337,14 @@ StatusWith<SSLX509Name::Entry> extractSingleOIDEntry(::CFDictionaryRef entry) {
         std::move(swLabelStr.getValue()), 19, std::move(swValueStr.getValue()));
 }
 
-// Translate a raw DER subject sequence into a structured subject name.
-StatusWith<SSLX509Name> extractSubjectName(::CFDictionaryRef dict) {
-    auto swSubject = extractDictionaryValue<::CFDictionaryRef>(dict, ::kSecOIDX509V1SubjectName);
-    if (!swSubject.isOK()) {
-        return swSubject.getStatus();
+// Extract a name value from the dictionary for the given property key.
+StatusWith<SSLX509Name> extractPropertyName(::CFDictionaryRef dict, CFStringRef property) {
+    auto swProp = extractDictionaryValue<::CFDictionaryRef>(dict, property);
+    if (!swProp.isOK()) {
+        return swProp.getStatus();
     }
 
-    auto swElems =
-        extractDictionaryValue<::CFArrayRef>(swSubject.getValue(), ::kSecPropertyKeyValue);
+    auto swElems = extractDictionaryValue<::CFArrayRef>(swProp.getValue(), ::kSecPropertyKeyValue);
     if (!swElems.isOK()) {
         return swElems.getStatus();
     }
@@ -353,7 +357,7 @@ StatusWith<SSLX509Name> extractSubjectName(::CFDictionaryRef dict) {
         invariant(elem);
         if (::CFGetTypeID(elem) != ::CFDictionaryGetTypeID()) {
             return {ErrorCodes::InvalidSSLConfiguration,
-                    "Subject name element is not a dictionary"};
+                    str::stream() << toString(property) << " element is not a dictionary"};
         }
 
         auto swType = extractDictionaryValue<::CFStringRef>(elem, ::kSecPropertyKeyType);
@@ -375,7 +379,8 @@ StatusWith<SSLX509Name> extractSubjectName(::CFDictionaryRef dict) {
                 invariant(rdnElem);
                 if (::CFGetTypeID(rdnElem) != ::CFDictionaryGetTypeID()) {
                     return {ErrorCodes::InvalidSSLConfiguration,
-                            "Subject name sub-element is not a dictionary"};
+                            str::stream()
+                                << toString(property) << " sub-element is not a dictionary"};
                 }
                 auto swEntry = extractSingleOIDEntry(rdnElem);
                 if (!swEntry.isOK()) {
@@ -395,12 +400,17 @@ StatusWith<SSLX509Name> extractSubjectName(::CFDictionaryRef dict) {
         }
     }
 
-    SSLX509Name subjectName = SSLX509Name(std::move(ret));
-    Status normalize = subjectName.normalizeStrings();
+    SSLX509Name propertyName = SSLX509Name(std::move(ret));
+    Status normalize = propertyName.normalizeStrings();
     if (!normalize.isOK()) {
         return normalize;
     }
-    return subjectName;
+    return propertyName;
+}
+
+// Translate a raw DER subject sequence into a structured subject name.
+StatusWith<SSLX509Name> extractSubjectName(::CFDictionaryRef dict) {
+    return extractPropertyName(dict, ::kSecOIDX509V1SubjectName);
 }
 
 StatusWith<mongo::Date_t> extractValidityDate(::CFDictionaryRef dict,
@@ -841,6 +851,55 @@ StatusWith<CFUniquePtr<::CFArrayRef>> loadPEM(const std::string& keyfilepath,
     return std::move(cfcerts);
 }
 
+StatusWith<SSLX509Name> certificateGetPropertyName(::SecCertificateRef cert,
+                                                   CFStringRef property,
+                                                   Date_t* expire = nullptr) {
+    // Fetch expiry range and property name.
+    CFUniquePtr<::CFMutableArrayRef> oids(
+        ::CFArrayCreateMutable(nullptr, expire ? 3 : 1, &::kCFTypeArrayCallBacks));
+    ::CFArrayAppendValue(oids.get(), property);
+    if (expire) {
+        ::CFArrayAppendValue(oids.get(), ::kSecOIDX509V1ValidityNotBefore);
+        ::CFArrayAppendValue(oids.get(), ::kSecOIDX509V1ValidityNotAfter);
+    }
+
+    ::CFErrorRef cferror = nullptr;
+    CFUniquePtr<::CFDictionaryRef> cfdict(::SecCertificateCopyValues(cert, oids.get(), &cferror));
+    if (cferror) {
+        CFUniquePtr<::CFErrorRef> deleter(cferror);
+        return {ErrorCodes::InvalidSSLConfiguration,
+                str::stream() << "Unable to determine certificate validity: " << cferror};
+    }
+
+    auto swPropertyName = extractPropertyName(cfdict.get(), property);
+    if (!swPropertyName.isOK()) {
+        return swPropertyName.getStatus();
+    }
+    auto propertyName = swPropertyName.getValue();
+
+    if (!expire) {
+        return propertyName;
+    }
+
+    // Marshal expiration.
+    auto swValidFrom =
+        extractValidityDate(cfdict.get(), ::kSecOIDX509V1ValidityNotBefore, "valid-from");
+    auto swValidUntil =
+        extractValidityDate(cfdict.get(), ::kSecOIDX509V1ValidityNotAfter, "valid-until");
+    if (!swValidFrom.isOK() || !swValidUntil.isOK()) {
+        return swValidUntil.getStatus();
+    }
+
+    *expire = swValidUntil.getValue();
+    const auto now = Date_t::now();
+    if ((now < swValidFrom.getValue()) || (*expire < now)) {
+        return {ErrorCodes::InvalidSSLConfiguration,
+                "The provided SSL certificate is expired or not yet valid"};
+    }
+
+    return propertyName;
+}
+
 // Get the root certificate from an array of certs.
 StatusWith<::SecCertificateRef> getCertificate(::CFArrayRef certs) {
     if (::CFArrayGetCount(certs) <= 0) {
@@ -862,72 +921,23 @@ StatusWith<::SecCertificateRef> getCertificate(::CFArrayRef certs) {
     return idcert;
 }
 
+StatusWith<SSLX509Name> certificateGetPropertyName(::CFArrayRef certs,
+                                                   CFStringRef property,
+                                                   Date_t* expire = nullptr) {
+    auto swCert = getCertificate(certs);
+    if (!swCert.isOK()) {
+        return swCert.getStatus();
+    }
+    CFUniquePtr<::SecCertificateRef> cert(swCert.getValue());
+    return certificateGetPropertyName(cert.get(), property, expire);
+}
+
 StatusWith<SSLX509Name> certificateGetSubject(::SecCertificateRef cert, Date_t* expire = nullptr) {
-    // Fetch expiry range and full subject name.
-    CFUniquePtr<::CFMutableArrayRef> oids(
-        ::CFArrayCreateMutable(nullptr, expire ? 3 : 1, &::kCFTypeArrayCallBacks));
-    ::CFArrayAppendValue(oids.get(), ::kSecOIDX509V1SubjectName);
-    if (expire) {
-        ::CFArrayAppendValue(oids.get(), ::kSecOIDX509V1ValidityNotBefore);
-        ::CFArrayAppendValue(oids.get(), ::kSecOIDX509V1ValidityNotAfter);
-    }
-
-    ::CFErrorRef cferror = nullptr;
-    CFUniquePtr<::CFDictionaryRef> cfdict(::SecCertificateCopyValues(cert, oids.get(), &cferror));
-    if (cferror) {
-        CFUniquePtr<::CFErrorRef> deleter(cferror);
-        return {ErrorCodes::InvalidSSLConfiguration,
-                str::stream() << "Unable to determine certificate validity: " << cferror};
-    }
-
-    auto swSubjectName = extractSubjectName(cfdict.get());
-    if (!swSubjectName.isOK()) {
-        return swSubjectName.getStatus();
-    }
-    auto subject = swSubjectName.getValue();
-
-    if (!expire) {
-        return subject;
-    }
-
-    // Marshal expiration.
-    auto swValidFrom =
-        extractValidityDate(cfdict.get(), ::kSecOIDX509V1ValidityNotBefore, "valid-from");
-    auto swValidUntil =
-        extractValidityDate(cfdict.get(), ::kSecOIDX509V1ValidityNotAfter, "valid-until");
-    if (!swValidFrom.isOK() || !swValidUntil.isOK()) {
-        return swValidUntil.getStatus();
-    }
-
-    *expire = swValidUntil.getValue();
-    const auto now = Date_t::now();
-    if ((now < swValidFrom.getValue()) || (*expire < now)) {
-        return {ErrorCodes::InvalidSSLConfiguration,
-                "The provided SSL certificate is expired or not yet valid"};
-    }
-
-    return subject;
+    return certificateGetPropertyName(cert, ::kSecOIDX509V1SubjectName, expire);
 }
 
 StatusWith<SSLX509Name> certificateGetSubject(::CFArrayRef certs, Date_t* expire = nullptr) {
-    if (::CFArrayGetCount(certs) <= 0) {
-        return {ErrorCodes::InvalidSSLConfiguration, "No certificates in certificate list"};
-    }
-
-    auto root = ::CFArrayGetValueAtIndex(certs, 0);
-    if (!root || (::CFGetTypeID(root) != ::SecIdentityGetTypeID())) {
-        return {ErrorCodes::InvalidSSLConfiguration, "Root certificate not an identity pair"};
-    }
-
-    ::SecCertificateRef idcert = nullptr;
-    auto status = ::SecIdentityCopyCertificate(cf_cast<::SecIdentityRef>(root), &idcert);
-    if (status != ::errSecSuccess) {
-        return {ErrorCodes::InvalidSSLConfiguration,
-                str::stream() << "Unable to get certificate from identity: "
-                              << stringFromOSStatus(status)};
-    }
-    CFUniquePtr<::SecCertificateRef> cert(idcert);
-    return certificateGetSubject(cert.get(), expire);
+    return certificateGetPropertyName(certs, ::kSecOIDX509V1SubjectName, expire);
 }
 
 StatusWith<CFUniquePtr<::CFArrayRef>> copyMatchingCertificate(
@@ -1270,6 +1280,9 @@ public:
                           const SSLParams& params,
                           ConnectionDirection direction) final;
 
+    void registerOwnedBySSLContext(
+        std::weak_ptr<const transport::SSLConnectionContext> ownedByContext) final;
+
     SSLConnectionInterface* connect(Socket* socket) final;
     SSLConnectionInterface* accept(Socket* socket, const char* initialBytes, int len) final;
 
@@ -1283,7 +1296,7 @@ public:
                                                         const HostAndPort& hostForLogging,
                                                         const ExecutorPtr& reactor) final;
 
-    Status stapleOCSPResponse(asio::ssl::apple::Context* context) final;
+    Status stapleOCSPResponse(asio::ssl::apple::Context* context, bool asyncOCSPStaple) final;
 
     const SSLConfiguration& getSSLConfiguration() const final {
         return _sslConfiguration;
@@ -1293,6 +1306,8 @@ public:
     int SSL_write(SSLConnectionInterface* conn, const void* buf, int num) final;
     int SSL_shutdown(SSLConnectionInterface* conn) final;
 
+    SSLInformationToLog getSSLInformationToLog() const final;
+
     void stopJobs() final;
 
 private:
@@ -1317,6 +1332,10 @@ private:
     CFUniquePtr<::CFArrayRef> _serverCA;
 
     SSLConfiguration _sslConfiguration;
+
+    // Weak pointer to verify that this manager is still owned by this context.
+    // Will be used if stapling is implemented.
+    synchronized_value<std::weak_ptr<const SSLConnectionContext>> _ownedByContext;
 };
 
 SSLManagerApple::SSLManagerApple(const SSLParams& params, bool isServer)
@@ -1337,8 +1356,8 @@ SSLManagerApple::SSLManagerApple(const SSLParams& params, bool isServer)
             uassertStatusOK(
                 _sslConfiguration.setServerSubjectName(uassertStatusOK(certificateGetSubject(
                     _serverCtx.certs.get(), &_sslConfiguration.serverCertificateExpirationDate))));
-            static auto task =
-                CertificateExpirationMonitor(_sslConfiguration.serverCertificateExpirationDate);
+            CertificateExpirationMonitor::get()->updateExpirationDeadline(
+                _sslConfiguration.serverCertificateExpirationDate);
 
             auto swSans = extractSubjectAlternateNames(_serverCtx.certs.get());
             const bool hasSan = swSans.isOK() && (0 != swSans.getValue().size());
@@ -1352,15 +1371,20 @@ SSLManagerApple::SSLManagerApple(const SSLParams& params, bool isServer)
         }
     }
 
-    // If the user has specified --setParameter tlsUseSystemCA=true, then no params.sslCAFile nor
-    // params.sslClusterCAFile will be defined, and the SSL Manager will fall back to the System CA.
     if (!params.sslCAFile.empty()) {
         auto ca = uassertStatusOK(loadPEM(params.sslCAFile, "", kLoadPEMStripKeys));
         _clientCA = std::move(ca);
+        _sslConfiguration.hasCA = _clientCA && ::CFArrayGetCount(_clientCA.get());
+    }
+
+    if (!params.sslCertificateSelector.empty() || !params.sslClusterCertificateSelector.empty()) {
+        // By using the system keychain, we acknowledge it exists.
+        _sslConfiguration.hasCA = true;
     }
 
     if (!_clientCA) {
-        // No explicit CA was specified, use the Keychain CA explicitly
+        // No explicit CA was specified, use the Keychain CA explicitly on client connects,
+        // even though we're going to pretend it doesn't exist on server.
         ::CFArrayRef certs = nullptr;
         uassertOSStatusOK(SecTrustCopyAnchorCertificates(&certs));
         _clientCA.reset(certs);
@@ -1462,6 +1486,11 @@ Status SSLManagerApple::initSSLContext(asio::ssl::apple::Context* context,
         params.sslCertificateSelector, params.sslPEMKeyFile, params.sslPEMKeyPassword);
 }
 
+void SSLManagerApple::registerOwnedBySSLContext(
+    std::weak_ptr<const transport::SSLConnectionContext> ownedByContext) {
+    _ownedByContext = ownedByContext;
+}
+
 SSLConnectionInterface* SSLManagerApple::connect(Socket* socket) {
     return new SSLConnectionApple(
         &_clientCtx, socket, ::kSSLClientSide, socket->remoteAddr().hostOrIp());
@@ -1507,7 +1536,8 @@ StatusWith<TLSVersion> mapTLSVersion(SSLContextRef ssl) {
     }
 }
 
-Status SSLManagerApple::stapleOCSPResponse(asio::ssl::apple::Context* context) {
+Status SSLManagerApple::stapleOCSPResponse(asio::ssl::apple::Context* context,
+                                           bool asyncOCSPStaple) {
     return Status::OK();
 }
 
@@ -1529,6 +1559,17 @@ Future<SSLPeerInfo> SSLManagerApple::parseAndValidatePeerCertificate(
 
     recordTLSVersion(tlsVersionStatus.getValue(), hostForLogging);
 
+    /* While we always have a system CA via the Keychain,
+     * we'll pretend not to in terms of validation if the server
+     * was started using a PEM file (legacy mode).
+     *
+     * When a certificate selector is used, we'll override hasCA to true
+     * so that the validation path runs anyway.
+     */
+    if (!_sslConfiguration.hasCA && isSSLServer) {
+        return Future<SSLPeerInfo>::makeReady(SSLPeerInfo(sniName));
+    }
+
     const auto badCert = [&](StringData msg, bool warn = false) -> Future<SSLPeerInfo> {
         if (warn) {
             LOGV2_WARNING(23209,
@@ -1553,7 +1594,7 @@ Future<SSLPeerInfo> SSLManagerApple::parseAndValidatePeerCertificate(
             return SSLPeerInfo(sniName);
         } else {
             if (status == ::errSecSuccess) {
-                return badCert(str::stream() << "No SSL certificate provided by peer: "
+                return badCert(str::stream() << "no SSL certificate provided by peer: "
                                              << stringFromOSStatus(status),
                                _weakValidation);
             } else {
@@ -1631,11 +1672,14 @@ Future<SSLPeerInfo> SSLManagerApple::parseAndValidatePeerCertificate(
         return swPeerSubjectName.getStatus();
     }
     const auto peerSubjectName = std::move(swPeerSubjectName.getValue());
-    LOGV2_DEBUG(23207,
-                2,
-                "Accepted TLS connection from peer: {peerSubjectName}",
-                "Accepted TLS connection from peer",
-                "peerSubjectName"_attr = peerSubjectName);
+    // The cipher will be presented as a number.
+    ::SSLCipherSuite cipher;
+    uassertOSStatusOK(::SSLGetNegotiatedCipher(ssl, &cipher));
+
+    LOGV2_INFO(6723803,
+               "Accepted TLS connection from peer",
+               "peerSubjectName"_attr = peerSubjectName,
+               "cipher"_attr = cipher);
 
     // Server side.
     if (remoteHost.empty()) {
@@ -1770,6 +1814,57 @@ int SSLManagerApple::SSL_shutdown(SSLConnectionInterface* conn) {
     return 1;
 }
 
+void getCertInfo(CertInformationToLog* info, const ::CFArrayRef cert) {
+    info->subject = uassertStatusOK(certificateGetSubject(cert));
+    info->issuer = uassertStatusOK(certificateGetPropertyName(cert, ::kSecOIDX509V1IssuerName));
+
+    // Get validity dates via SecCertificateCopyValues
+    CFUniquePtr<::CFMutableArrayRef> oids(
+        ::CFArrayCreateMutable(nullptr, 2, &::kCFTypeArrayCallBacks));
+    ::CFArrayAppendValue(oids.get(), ::kSecOIDX509V1ValidityNotBefore);
+    ::CFArrayAppendValue(oids.get(), ::kSecOIDX509V1ValidityNotAfter);
+
+    CFUniquePtr<::SecCertificateRef> rootCert(uassertStatusOK(getCertificate(cert)));
+    ::CFErrorRef cferror = nullptr;
+    CFUniquePtr<::CFDictionaryRef> cfdict(
+        ::SecCertificateCopyValues(rootCert.get(), oids.get(), &cferror));
+    if (cferror) {
+        CFUniquePtr<::CFErrorRef> deleter(cferror);
+        uasserted(ErrorCodes::InvalidSSLConfiguration,
+                  str::stream() << "Failed to get thumbprint: " << cferror);
+    }
+
+    info->validityNotBefore = uassertStatusOK(
+        extractValidityDate(cfdict.get(), ::kSecOIDX509V1ValidityNotBefore, "valid-from"));
+    info->validityNotAfter = uassertStatusOK(
+        extractValidityDate(cfdict.get(), ::kSecOIDX509V1ValidityNotAfter, "valid-until"));
+
+    // Compute certificate thumbprint
+    CFUniquePtr<::CFDataRef> cfCertData(::SecCertificateCopyData(rootCert.get()));
+    ConstDataRange certData(reinterpret_cast<const char*>(::CFDataGetBytePtr(cfCertData.get())),
+                            ::CFDataGetLength(cfCertData.get()));
+
+    // Comupte hash from bytes of certificate
+    const auto certSha1 = SHA1Block::computeHash({certData});
+    info->thumbprint =
+        std::vector<char>((char*)certSha1.data(), (char*)certSha1.data() + certSha1.kHashLength);
+    info->hexEncodedThumbprint = hexblob::encode(info->thumbprint.data(), info->thumbprint.size());
+}
+
+SSLInformationToLog SSLManagerApple::getSSLInformationToLog() const {
+    SSLInformationToLog info;
+    // server CA should definitely exist, client CA is optional
+    if (_serverCtx.certs.get()) {
+        getCertInfo(&info.server, _serverCtx.certs.get());
+    }
+    if (_clientCtx.certs.get()) {
+        CertInformationToLog clientInfo;
+        getCertInfo(&clientInfo, _clientCtx.certs.get());
+        info.cluster = clientInfo;
+    }
+    return info;
+}
+
 }  // namespace
 
 /////////////////////////////////////////////////////////////////////////////
@@ -1778,6 +1873,14 @@ int SSLManagerApple::SSL_shutdown(SSLConnectionInterface* conn) {
 bool isSSLServer = false;
 
 extern SSLManagerInterface* theSSLManager;
+extern SSLManagerCoordinator* theSSLManagerCoordinator;
+
+std::shared_ptr<SSLManagerInterface> SSLManagerInterface::create(
+    const SSLParams& params,
+    const boost::optional<TransientSSLParams>& transientSSLParams,
+    bool isServer) {
+    return std::make_shared<SSLManagerApple>(params, isServer);
+}
 
 std::shared_ptr<SSLManagerInterface> SSLManagerInterface::create(const SSLParams& params,
                                                                  bool isServer) {
@@ -1789,10 +1892,25 @@ MONGO_INITIALIZER_WITH_PREREQUISITES(SSLManager, ("EndStartupOptionHandling"))
     kMongoDBRolesOID = ::CFStringCreateWithCString(
         nullptr, mongodbRolesOID.identifier.c_str(), ::kCFStringEncodingUTF8);
 
+    constexpr int kMaxRetries = 10;
     if (!isSSLServer || (sslGlobalParams.sslMode.load() != SSLParams::SSLMode_disabled)) {
-        theSSLManager = new SSLManagerApple(sslGlobalParams, isSSLServer);
+        for (int i = 0; i < kMaxRetries; i++) {
+            try {
+                theSSLManagerCoordinator = new SSLManagerCoordinator();
+                return;
+            } catch (const ExceptionFor<ErrorCodes::InvalidSSLConfiguration>& e) {
+                bool isRetriableError = nullptr != strstr(e.what(), "No keychain is available.");
+                if (!isRetriableError || i == kMaxRetries - 1) {
+                    // Rethrow if a different error or we fail on final iteration
+                    throw;
+                }
+                LOGV2_INFO(6741800,
+                           "Caught exception during apple SSLManagerCoordinator creation, retrying",
+                           "try"_attr = i,
+                           "error"_attr = e.what());
+            }
+        }
     }
-    return Status::OK();
 }
 
 }  // namespace mongo
