@@ -27,25 +27,95 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/util/net/ssl_manager.h"
+#include <fstream>
 
 #include "mongo/config.h"
+#include "mongo/platform/basic.h"
+
+#include "mongo/transport/service_entry_point.h"
+#include "mongo/transport/transport_layer_asio.h"
+#include "mongo/transport/transport_layer_manager.h"
+#include "mongo/util/net/ssl/context.hpp"
+#include "mongo/util/net/ssl_manager.h"
+#include "mongo/util/net/ssl_options.h"
+
 #include "mongo/logv2/log.h"
 #include "mongo/unittest/unittest.h"
-#include "mongo/util/net/ssl/context.hpp"
-#include "mongo/util/net/ssl_options.h"
 
 #if MONGO_CONFIG_SSL_PROVIDER == MONGO_CONFIG_SSL_PROVIDER_OPENSSL
 #include "mongo/util/net/dh_openssl.h"
+#include "mongo/util/net/ssl/context_openssl.hpp"
 #endif
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
 
 namespace mongo {
 namespace {
+
+// Test implementation needed by ASIO transport.
+class ServiceEntryPointUtil : public ServiceEntryPoint {
+public:
+    void startSession(transport::SessionHandle session) override {
+        stdx::unique_lock<Latch> lk(_mutex);
+        _sessions.push_back(std::move(session));
+        LOGV2(2303202, "started session");
+        _cv.notify_one();
+    }
+
+    void endAllSessions(transport::Session::TagMask tags) override {
+        LOGV2(2303302, "end all sessions");
+        std::vector<transport::SessionHandle> old_sessions;
+        {
+            stdx::unique_lock<Latch> lock(_mutex);
+            old_sessions.swap(_sessions);
+        }
+        old_sessions.clear();
+    }
+
+    Status start() override {
+        return Status::OK();
+    }
+
+    bool shutdown(Milliseconds timeout) override {
+        return true;
+    }
+
+    void appendStats(BSONObjBuilder*) const override {}
+
+    size_t numOpenSessions() const override {
+        stdx::unique_lock<Latch> lock(_mutex);
+        return _sessions.size();
+    }
+
+    Future<DbResponse> handleRequest(OperationContext* opCtx,
+                                     const Message& request) noexcept override {
+        MONGO_UNREACHABLE;
+    }
+
+    void setTransportLayer(transport::TransportLayer* tl) {
+        _transport = tl;
+    }
+
+    void waitForConnect() {
+        stdx::unique_lock<Latch> lock(_mutex);
+        _cv.wait(lock, [&] { return !_sessions.empty(); });
+    }
+
+private:
+    mutable Mutex _mutex = MONGO_MAKE_LATCH("::_mutex");
+    stdx::condition_variable _cv;
+    std::vector<transport::SessionHandle> _sessions;
+    transport::TransportLayer* _transport = nullptr;
+};
+
+std::string loadFile(const std::string& name) {
+    std::ifstream input(name);
+    std::string str((std::istreambuf_iterator<char>(input)), std::istreambuf_iterator<char>());
+    return str;
+}
+
 TEST(SSLManager, matchHostname) {
     enum Expected : bool { match = true, mismatch = false };
     const struct {
@@ -417,6 +487,254 @@ TEST(SSLManager, BadDNParsing) {
     }
 }
 
+TEST(SSLManager, RotateCertificatesFromFile) {
+    SSLParams params;
+    params.sslMode.store(::mongo::sslGlobalParams.SSLMode_requireSSL);
+    // Server is required to have the sslPEMKeyFile.
+    params.sslPEMKeyFile = "jstests/libs/server.pem";
+    params.sslCAFile = "jstests/libs/ca.pem";
+    params.sslClusterFile = "jstests/libs/client.pem";
+
+    std::shared_ptr<SSLManagerInterface> manager =
+        SSLManagerInterface::create(params, true /* isSSLServer */);
+
+    ServiceEntryPointUtil sepu;
+
+    auto options = [] {
+        ServerGlobalParams params;
+        params.noUnixSocket = true;
+        transport::TransportLayerASIO::Options opts(&params);
+        return opts;
+    }();
+    transport::TransportLayerASIO tla(options, &sepu);
+    uassertStatusOK(tla.rotateCertificates(manager, false /* asyncOCSPStaple */));
+}
+
+TEST(SSLManager, InitContextFromFileShouldFail) {
+    SSLParams params;
+    params.sslMode.store(::mongo::sslGlobalParams.SSLMode_requireSSL);
+    // Server is required to have the sslPEMKeyFile.
+    // We force the initialization to fail by omitting this param.
+    params.sslCAFile = "jstests/libs/ca.pem";
+    params.sslClusterFile = "jstests/libs/client.pem";
+#if MONGO_CONFIG_SSL_PROVIDER == MONGO_CONFIG_SSL_PROVIDER_OPENSSL
+    ASSERT_THROWS_CODE([&params] { SSLManagerInterface::create(params, true /* isSSLServer */); }(),
+                       DBException,
+                       ErrorCodes::InvalidSSLConfiguration);
+#endif
+}
+
+TEST(SSLManager, RotateClusterCertificatesFromFile) {
+    SSLParams params;
+    params.sslMode.store(::mongo::sslGlobalParams.SSLMode_requireSSL);
+    // Client doesn't need params.sslPEMKeyFile.
+    params.sslCAFile = "jstests/libs/ca.pem";
+    params.sslClusterFile = "jstests/libs/client.pem";
+
+    std::shared_ptr<SSLManagerInterface> manager =
+        SSLManagerInterface::create(params, false /* isSSLServer */);
+
+    ServiceEntryPointUtil sepu;
+
+    auto options = [] {
+        ServerGlobalParams params;
+        params.noUnixSocket = true;
+        transport::TransportLayerASIO::Options opts(&params);
+        return opts;
+    }();
+    transport::TransportLayerASIO tla(options, &sepu);
+    uassertStatusOK(tla.rotateCertificates(manager, false /* asyncOCSPStaple */));
+}
+
+#if MONGO_CONFIG_SSL_PROVIDER == MONGO_CONFIG_SSL_PROVIDER_OPENSSL
+
+TEST(SSLManager, InitContextFromFile) {
+    SSLParams params;
+    params.sslMode.store(::mongo::sslGlobalParams.SSLMode_requireSSL);
+    // Client doesn't need params.sslPEMKeyFile.
+    params.sslClusterFile = "jstests/libs/client.pem";
+
+    std::shared_ptr<SSLManagerInterface> manager =
+        SSLManagerInterface::create(params, false /* isSSLServer */);
+
+    auto egress = std::make_unique<asio::ssl::context>(asio::ssl::context::sslv23);
+    uassertStatusOK(manager->initSSLContext(
+        egress->native_handle(), params, SSLManagerInterface::ConnectionDirection::kOutgoing));
+}
+
+TEST(SSLManager, InitContextFromMemory) {
+    SSLParams params;
+    params.sslMode.store(::mongo::sslGlobalParams.SSLMode_requireSSL);
+    params.sslCAFile = "jstests/libs/ca.pem";
+
+    TransientSSLParams transientParams;
+    transientParams.sslClusterPEMPayload = loadFile("jstests/libs/client.pem");
+
+    std::shared_ptr<SSLManagerInterface> manager =
+        SSLManagerInterface::create(params, transientParams, false /* isSSLServer */);
+
+    auto egress = std::make_unique<asio::ssl::context>(asio::ssl::context::sslv23);
+    uassertStatusOK(manager->initSSLContext(
+        egress->native_handle(), params, SSLManagerInterface::ConnectionDirection::kOutgoing));
+}
+
+// Tests when 'is server' param to managed interface creation is set, it is ignored.
+TEST(SSLManager, IgnoreInitServerSideContextFromMemory) {
+    SSLParams params;
+    params.sslMode.store(::mongo::sslGlobalParams.SSLMode_requireSSL);
+    params.sslPEMKeyFile = "jstests/libs/server.pem";
+    params.sslCAFile = "jstests/libs/ca.pem";
+
+    TransientSSLParams transientParams;
+    transientParams.sslClusterPEMPayload = loadFile("jstests/libs/client.pem");
+
+    std::shared_ptr<SSLManagerInterface> manager =
+        SSLManagerInterface::create(params, transientParams, true /* isSSLServer */);
+
+    auto egress = std::make_unique<asio::ssl::context>(asio::ssl::context::sslv23);
+    uassertStatusOK(manager->initSSLContext(
+        egress->native_handle(), params, SSLManagerInterface::ConnectionDirection::kOutgoing));
+}
+
+TEST(SSLManager, TransientSSLParams) {
+    SSLParams params;
+    params.sslMode.store(::mongo::sslGlobalParams.SSLMode_requireSSL);
+    params.sslCAFile = "jstests/libs/ca.pem";
+    params.sslClusterFile = "jstests/libs/client.pem";
+
+    ServiceEntryPointUtil sepu;
+
+    auto options = [] {
+        ServerGlobalParams params;
+        params.noUnixSocket = true;
+        transport::TransportLayerASIO::Options opts(&params);
+        return opts;
+    }();
+    transport::TransportLayerASIO tla(options, &sepu);
+
+    TransientSSLParams transientSSLParams;
+    transientSSLParams.sslClusterPEMPayload = loadFile("jstests/libs/client.pem");
+    transientSSLParams.targetedClusterConnectionString = ConnectionString::forLocal();
+
+    auto swContext = tla.createTransientSSLContext(transientSSLParams);
+    uassertStatusOK(swContext.getStatus());
+
+    // Check that the manager owned by the transient context is also transient.
+    ASSERT_TRUE(swContext.getValue()->manager->isTransient());
+    ASSERT_EQ(transientSSLParams.targetedClusterConnectionString.toString(),
+              swContext.getValue()->manager->getTargetedClusterConnectionString());
+
+    // Cannot rotate certs on transient manager.
+    ASSERT_NOT_OK(tla.rotateCertificates(swContext.getValue()->manager, true));
+}
+
+TEST(SSLManager, TransientSSLParamsStressTestWithTransport) {
+    static constexpr int kMaxContexts = 100;
+    static constexpr int kThreads = 10;
+    SSLParams params;
+    params.sslMode.store(::mongo::sslGlobalParams.SSLMode_requireSSL);
+    params.sslCAFile = "jstests/libs/ca.pem";
+
+    ServiceEntryPointUtil sepu;
+
+    auto options = [] {
+        ServerGlobalParams params;
+        params.noUnixSocket = true;
+        transport::TransportLayerASIO::Options opts(&params);
+        return opts;
+    }();
+    transport::TransportLayerASIO tla(options, &sepu);
+
+    TransientSSLParams transientSSLParams;
+    transientSSLParams.sslClusterPEMPayload = loadFile("jstests/libs/client.pem");
+    transientSSLParams.targetedClusterConnectionString = ConnectionString::forLocal();
+
+    Mutex mutex = MONGO_MAKE_LATCH("::test_mutex");
+    std::deque<std::shared_ptr<const transport::SSLConnectionContext>> contexts;
+    std::vector<stdx::thread> threads;
+    Counter64 iterations;
+
+    for (int t = 0; t < kThreads; ++t) {
+        stdx::thread thread([&]() {
+            Timer timer;
+            while (timer.elapsed() < Seconds(2)) {
+                auto swContext = tla.createTransientSSLContext(transientSSLParams);
+                invariant(swContext.getStatus().isOK());
+                std::shared_ptr<const transport::SSLConnectionContext> ctxToDelete;
+                {
+                    auto lk = stdx::lock_guard(mutex);
+                    contexts.push_back(std::move(swContext.getValue()));
+                    if (contexts.size() > kMaxContexts) {
+                        ctxToDelete = contexts.front();
+                        contexts.pop_front();
+                    }
+                }
+                iterations.increment();
+            }
+        });
+        threads.push_back(std::move(thread));
+    }
+    for (auto& t : threads) {
+        t.join();
+    }
+
+    contexts.clear();
+    LOGV2(5906701, "Stress test completed", "iterations"_attr = iterations);
+}
+
+TEST(SSLManager, TransientSSLParamsStressTestWithManager) {
+    static constexpr int kMaxManagers = 100;
+    static constexpr int kThreads = 10;
+    SSLParams params;
+    params.sslMode.store(::mongo::sslGlobalParams.SSLMode_requireSSL);
+    params.sslPEMKeyFile = "jstests/libs/server.pem";
+    params.sslCAFile = "jstests/libs/ca.pem";
+
+    TransientSSLParams transientParams;
+    transientParams.sslClusterPEMPayload = loadFile("jstests/libs/client.pem");
+
+    Mutex mutex = MONGO_MAKE_LATCH("::test_mutex");
+    std::deque<std::shared_ptr<SSLManagerInterface>> managers;
+    std::vector<stdx::thread> threads;
+    Counter64 iterations;
+
+    for (int t = 0; t < kThreads; ++t) {
+        stdx::thread thread([&]() {
+            Timer timer;
+            while (timer.elapsed() < Seconds(3)) {
+                std::shared_ptr<SSLManagerInterface> manager =
+                    SSLManagerInterface::create(params, transientParams, true /* isSSLServer */);
+
+                auto egress = std::make_unique<asio::ssl::context>(asio::ssl::context::sslv23);
+                invariant(manager
+                              ->initSSLContext(egress->native_handle(),
+                                               params,
+                                               SSLManagerInterface::ConnectionDirection::kOutgoing)
+                              .isOK());
+                std::shared_ptr<SSLManagerInterface> managerToDelete;
+                {
+                    auto lk = stdx::lock_guard(mutex);
+                    managers.push_back(std::move(manager));
+                    if (managers.size() > kMaxManagers) {
+                        managerToDelete = managers.front();
+                        managers.pop_front();
+                    }
+                }
+                iterations.increment();
+            }
+        });
+        threads.push_back(std::move(thread));
+    }
+    for (auto& t : threads) {
+        t.join();
+    }
+
+    managers.clear();
+    LOGV2(5906702, "Stress test completed", "iterations"_attr = iterations);
+}
+
+#endif  // MONGO_CONFIG_SSL_PROVIDER == MONGO_CONFIG_SSL_PROVIDER_OPENSSL
+
 static bool isSanWarningWritten(const std::vector<std::string>& logLines) {
     for (const auto& line : logLines) {
         if (std::string::npos !=
@@ -463,5 +781,6 @@ TEST(SSLManager, InitContextNoSanWarning) {
     ASSERT_FALSE(isSanWarningWritten(getCapturedTextFormatLogMessages()));
 }
 
+
 }  // namespace
 }  // namespace mongo
